"no use strict";
;(function(window) {
if (typeof window.window != "undefined" && window.document) {
    return;
}

window.console = function() {
    var msgs = Array.prototype.slice.call(arguments, 0);
    postMessage({type: "log", data: msgs});
};
window.console.error =
window.console.warn = 
window.console.log =
window.console.trace = window.console;

window.window = window;
window.ace = window;

window.onerror = function(message, file, line, col, err) {
    postMessage({type: "error", data: {
        message: message,
        file: file,
        line: line, 
        col: col,
        stack: err.stack
    }});
};

window.normalizeModule = function(parentId, moduleName) {
    // normalize plugin requires
    if (moduleName.indexOf("!") !== -1) {
        var chunks = moduleName.split("!");
        return window.normalizeModule(parentId, chunks[0]) + "!" + window.normalizeModule(parentId, chunks[1]);
    }
    // normalize relative requires
    if (moduleName.charAt(0) == ".") {
        var base = parentId.split("/").slice(0, -1).join("/");
        moduleName = (base ? base + "/" : "") + moduleName;
        
        while(moduleName.indexOf(".") !== -1 && previous != moduleName) {
            var previous = moduleName;
            moduleName = moduleName.replace(/^\.\//, "").replace(/\/\.\//, "/").replace(/[^\/]+\/\.\.\//, "");
        }
    }
    
    return moduleName;
};

window.require = function(parentId, id) {
    if (!id) {
        id = parentId;
        parentId = null;
    }
    if (!id.charAt)
        throw new Error("worker.js require() accepts only (parentId, id) as arguments");

    id = window.normalizeModule(parentId, id);

    var module = window.require.modules[id];
    if (module) {
        if (!module.initialized) {
            module.initialized = true;
            module.exports = module.factory().exports;
        }
        return module.exports;
    }
    
    var chunks = id.split("/");
    if (!window.require.tlns)
        return console.log("unable to load " + id);
    chunks[0] = window.require.tlns[chunks[0]] || chunks[0];
    var path = chunks.join("/") + ".js";
    
    window.require.id = id;
    importScripts(path);
    return window.require(parentId, id);
};
window.require.modules = {};
window.require.tlns = {};

window.define = function(id, deps, factory) {
    if (arguments.length == 2) {
        factory = deps;
        if (typeof id != "string") {
            deps = id;
            id = window.require.id;
        }
    } else if (arguments.length == 1) {
        factory = id;
        deps = [];
        id = window.require.id;
    }
    
    if (typeof factory != "function") {
        window.require.modules[id] = {
            exports: factory,
            initialized: true
        };
        return;
    }

    if (!deps.length)
        // If there is no dependencies, we inject 'require', 'exports' and
        // 'module' as dependencies, to provide CommonJS compatibility.
        deps = ['require', 'exports', 'module'];

    var req = function(childId) {
        return window.require(id, childId);
    };

    window.require.modules[id] = {
        exports: {},
        factory: function() {
            var module = this;
            var returnExports = factory.apply(this, deps.map(function(dep) {
              switch(dep) {
                  // Because 'require', 'exports' and 'module' aren't actual
                  // dependencies, we must handle them seperately.
                  case 'require': return req;
                  case 'exports': return module.exports;
                  case 'module':  return module;
                  // But for all other dependencies, we can just go ahead and
                  // require them.
                  default:        return req(dep);
              }
            }));
            if (returnExports)
                module.exports = returnExports;
            return module;
        }
    };
};
window.define.amd = {};

window.initBaseUrls  = function initBaseUrls(topLevelNamespaces) {
    require.tlns = topLevelNamespaces;
};

window.initSender = function initSender() {

    var EventEmitter = window.require("ace/lib/event_emitter").EventEmitter;
    var oop = window.require("ace/lib/oop");
    
    var Sender = function() {};
    
    (function() {
        
        oop.implement(this, EventEmitter);
                
        this.callback = function(data, callbackId) {
            postMessage({
                type: "call",
                id: callbackId,
                data: data
            });
        };
    
        this.emit = function(name, data) {
            postMessage({
                type: "event",
                name: name,
                data: data
            });
        };
        
    }).call(Sender.prototype);
    
    return new Sender();
};

var main = window.main = null;
var sender = window.sender = null;

window.onmessage = function(e) {
    var msg = e.data;
    if (msg.command) {
        if (main[msg.command])
            main[msg.command].apply(main, msg.args);
        else
            throw new Error("Unknown command:" + msg.command);
    }
    else if (msg.init) {        
        initBaseUrls(msg.tlns);
        require("ace/lib/es5-shim");
        sender = window.sender = initSender();
        var clazz = require(msg.module)[msg.classname];
        main = window.main = new clazz(sender);
    } 
    else if (msg.event && sender) {
        sender._signal(msg.event, msg.data);
    }
};
})(this);

define("ace/lib/oop",["require","exports","module"], function(require, exports, module) {
"use strict";

exports.inherits = function(ctor, superCtor) {
    ctor.super_ = superCtor;
    ctor.prototype = Object.create(superCtor.prototype, {
        constructor: {
            value: ctor,
            enumerable: false,
            writable: true,
            configurable: true
        }
    });
};

exports.mixin = function(obj, mixin) {
    for (var key in mixin) {
        obj[key] = mixin[key];
    }
    return obj;
};

exports.implement = function(proto, mixin) {
    exports.mixin(proto, mixin);
};

});

define("ace/lib/event_emitter",["require","exports","module"], function(require, exports, module) {
"use strict";

var EventEmitter = {};
var stopPropagation = function() { this.propagationStopped = true; };
var preventDefault = function() { this.defaultPrevented = true; };

EventEmitter._emit =
EventEmitter._dispatchEvent = function(eventName, e) {
    this._eventRegistry || (this._eventRegistry = {});
    this._defaultHandlers || (this._defaultHandlers = {});

    var listeners = this._eventRegistry[eventName] || [];
    var defaultHandler = this._defaultHandlers[eventName];
    if (!listeners.length && !defaultHandler)
        return;

    if (typeof e != "object" || !e)
        e = {};

    if (!e.type)
        e.type = eventName;
    if (!e.stopPropagation)
        e.stopPropagation = stopPropagation;
    if (!e.preventDefault)
        e.preventDefault = preventDefault;

    listeners = listeners.slice();
    for (var i=0; i<listeners.length; i++) {
        listeners[i](e, this);
        if (e.propagationStopped)
            break;
    }
    
    if (defaultHandler && !e.defaultPrevented)
        return defaultHandler(e, this);
};


EventEmitter._signal = function(eventName, e) {
    var listeners = (this._eventRegistry || {})[eventName];
    if (!listeners)
        return;
    listeners = listeners.slice();
    for (var i=0; i<listeners.length; i++)
        listeners[i](e, this);
};

EventEmitter.once = function(eventName, callback) {
    var _self = this;
    callback && this.addEventListener(eventName, function newCallback() {
        _self.removeEventListener(eventName, newCallback);
        callback.apply(null, arguments);
    });
};


EventEmitter.setDefaultHandler = function(eventName, callback) {
    var handlers = this._defaultHandlers
    if (!handlers)
        handlers = this._defaultHandlers = {_disabled_: {}};
    
    if (handlers[eventName]) {
        var old = handlers[eventName];
        var disabled = handlers._disabled_[eventName];
        if (!disabled)
            handlers._disabled_[eventName] = disabled = [];
        disabled.push(old);
        var i = disabled.indexOf(callback);
        if (i != -1) 
            disabled.splice(i, 1);
    }
    handlers[eventName] = callback;
};
EventEmitter.removeDefaultHandler = function(eventName, callback) {
    var handlers = this._defaultHandlers
    if (!handlers)
        return;
    var disabled = handlers._disabled_[eventName];
    
    if (handlers[eventName] == callback) {
        var old = handlers[eventName];
        if (disabled)
            this.setDefaultHandler(eventName, disabled.pop());
    } else if (disabled) {
        var i = disabled.indexOf(callback);
        if (i != -1)
            disabled.splice(i, 1);
    }
};

EventEmitter.on =
EventEmitter.addEventListener = function(eventName, callback, capturing) {
    this._eventRegistry = this._eventRegistry || {};

    var listeners = this._eventRegistry[eventName];
    if (!listeners)
        listeners = this._eventRegistry[eventName] = [];

    if (listeners.indexOf(callback) == -1)
        listeners[capturing ? "unshift" : "push"](callback);
    return callback;
};

EventEmitter.off =
EventEmitter.removeListener =
EventEmitter.removeEventListener = function(eventName, callback) {
    this._eventRegistry = this._eventRegistry || {};

    var listeners = this._eventRegistry[eventName];
    if (!listeners)
        return;

    var index = listeners.indexOf(callback);
    if (index !== -1)
        listeners.splice(index, 1);
};

EventEmitter.removeAllListeners = function(eventName) {
    if (this._eventRegistry) this._eventRegistry[eventName] = [];
};

exports.EventEmitter = EventEmitter;

});

define("ace/range",["require","exports","module"], function(require, exports, module) {
"use strict";
var comparePoints = function(p1, p2) {
    return p1.row - p2.row || p1.column - p2.column;
};
var Range = function(startRow, startColumn, endRow, endColumn) {
    this.start = {
        row: startRow,
        column: startColumn
    };

    this.end = {
        row: endRow,
        column: endColumn
    };
};

(function() {
    this.isEqual = function(range) {
        return this.start.row === range.start.row &&
            this.end.row === range.end.row &&
            this.start.column === range.start.column &&
            this.end.column === range.end.column;
    };
    this.toString = function() {
        return ("Range: [" + this.start.row + "/" + this.start.column +
            "] -> [" + this.end.row + "/" + this.end.column + "]");
    };

    this.contains = function(row, column) {
        return this.compare(row, column) == 0;
    };
    this.compareRange = function(range) {
        var cmp,
            end = range.end,
            start = range.start;

        cmp = this.compare(end.row, end.column);
        if (cmp == 1) {
            cmp = this.compare(start.row, start.column);
            if (cmp == 1) {
                return 2;
            } else if (cmp == 0) {
                return 1;
            } else {
                return 0;
            }
        } else if (cmp == -1) {
            return -2;
        } else {
            cmp = this.compare(start.row, start.column);
            if (cmp == -1) {
                return -1;
            } else if (cmp == 1) {
                return 42;
            } else {
                return 0;
            }
        }
    };
    this.comparePoint = function(p) {
        return this.compare(p.row, p.column);
    };
    this.containsRange = function(range) {
        return this.comparePoint(range.start) == 0 && this.comparePoint(range.end) == 0;
    };
    this.intersects = function(range) {
        var cmp = this.compareRange(range);
        return (cmp == -1 || cmp == 0 || cmp == 1);
    };
    this.isEnd = function(row, column) {
        return this.end.row == row && this.end.column == column;
    };
    this.isStart = function(row, column) {
        return this.start.row == row && this.start.column == column;
    };
    this.setStart = function(row, column) {
        if (typeof row == "object") {
            this.start.column = row.column;
            this.start.row = row.row;
        } else {
            this.start.row = row;
            this.start.column = column;
        }
    };
    this.setEnd = function(row, column) {
        if (typeof row == "object") {
            this.end.column = row.column;
            this.end.row = row.row;
        } else {
            this.end.row = row;
            this.end.column = column;
        }
    };
    this.inside = function(row, column) {
        if (this.compare(row, column) == 0) {
            if (this.isEnd(row, column) || this.isStart(row, column)) {
                return false;
            } else {
                return true;
            }
        }
        return false;
    };
    this.insideStart = function(row, column) {
        if (this.compare(row, column) == 0) {
            if (this.isEnd(row, column)) {
                return false;
            } else {
                return true;
            }
        }
        return false;
    };
    this.insideEnd = function(row, column) {
        if (this.compare(row, column) == 0) {
            if (this.isStart(row, column)) {
                return false;
            } else {
                return true;
            }
        }
        return false;
    };
    this.compare = function(row, column) {
        if (!this.isMultiLine()) {
            if (row === this.start.row) {
                return column < this.start.column ? -1 : (column > this.end.column ? 1 : 0);
            };
        }

        if (row < this.start.row)
            return -1;

        if (row > this.end.row)
            return 1;

        if (this.start.row === row)
            return column >= this.start.column ? 0 : -1;

        if (this.end.row === row)
            return column <= this.end.column ? 0 : 1;

        return 0;
    };
    this.compareStart = function(row, column) {
        if (this.start.row == row && this.start.column == column) {
            return -1;
        } else {
            return this.compare(row, column);
        }
    };
    this.compareEnd = function(row, column) {
        if (this.end.row == row && this.end.column == column) {
            return 1;
        } else {
            return this.compare(row, column);
        }
    };
    this.compareInside = function(row, column) {
        if (this.end.row == row && this.end.column == column) {
            return 1;
        } else if (this.start.row == row && this.start.column == column) {
            return -1;
        } else {
            return this.compare(row, column);
        }
    };
    this.clipRows = function(firstRow, lastRow) {
        if (this.end.row > lastRow)
            var end = {row: lastRow + 1, column: 0};
        else if (this.end.row < firstRow)
            var end = {row: firstRow, column: 0};

        if (this.start.row > lastRow)
            var start = {row: lastRow + 1, column: 0};
        else if (this.start.row < firstRow)
            var start = {row: firstRow, column: 0};

        return Range.fromPoints(start || this.start, end || this.end);
    };
    this.extend = function(row, column) {
        var cmp = this.compare(row, column);

        if (cmp == 0)
            return this;
        else if (cmp == -1)
            var start = {row: row, column: column};
        else
            var end = {row: row, column: column};

        return Range.fromPoints(start || this.start, end || this.end);
    };

    this.isEmpty = function() {
        return (this.start.row === this.end.row && this.start.column === this.end.column);
    };
    this.isMultiLine = function() {
        return (this.start.row !== this.end.row);
    };
    this.clone = function() {
        return Range.fromPoints(this.start, this.end);
    };
    this.collapseRows = function() {
        if (this.end.column == 0)
            return new Range(this.start.row, 0, Math.max(this.start.row, this.end.row-1), 0)
        else
            return new Range(this.start.row, 0, this.end.row, 0)
    };
    this.toScreenRange = function(session) {
        var screenPosStart = session.documentToScreenPosition(this.start);
        var screenPosEnd = session.documentToScreenPosition(this.end);

        return new Range(
            screenPosStart.row, screenPosStart.column,
            screenPosEnd.row, screenPosEnd.column
        );
    };
    this.moveBy = function(row, column) {
        this.start.row += row;
        this.start.column += column;
        this.end.row += row;
        this.end.column += column;
    };

}).call(Range.prototype);
Range.fromPoints = function(start, end) {
    return new Range(start.row, start.column, end.row, end.column);
};
Range.comparePoints = comparePoints;

Range.comparePoints = function(p1, p2) {
    return p1.row - p2.row || p1.column - p2.column;
};


exports.Range = Range;
});

define("ace/anchor",["require","exports","module","ace/lib/oop","ace/lib/event_emitter"], function(require, exports, module) {
"use strict";

var oop = require("./lib/oop");
var EventEmitter = require("./lib/event_emitter").EventEmitter;

var Anchor = exports.Anchor = function(doc, row, column) {
    this.$onChange = this.onChange.bind(this);
    this.attach(doc);
    
    if (typeof column == "undefined")
        this.setPosition(row.row, row.column);
    else
        this.setPosition(row, column);
};

(function() {

    oop.implement(this, EventEmitter);
    this.getPosition = function() {
        return this.$clipPositionToDocument(this.row, this.column);
    };
    this.getDocument = function() {
        return this.document;
    };
    this.$insertRight = false;
    this.onChange = function(e) {
        var delta = e.data;
        var range = delta.range;

        if (range.start.row == range.end.row && range.start.row != this.row)
            return;

        if (range.start.row > this.row)
            return;

        if (range.start.row == this.row && range.start.column > this.column)
            return;

        var row = this.row;
        var column = this.column;
        var start = range.start;
        var end = range.end;

        if (delta.action === "insertText") {
            if (start.row === row && start.column <= column) {
                if (start.column === column && this.$insertRight) {
                } else if (start.row === end.row) {
                    column += end.column - start.column;
                } else {
                    column -= start.column;
                    row += end.row - start.row;
                }
            } else if (start.row !== end.row && start.row < row) {
                row += end.row - start.row;
            }
        } else if (delta.action === "insertLines") {
            if (start.row === row && column === 0 && this.$insertRight) {
            }
            else if (start.row <= row) {
                row += end.row - start.row;
            }
        } else if (delta.action === "removeText") {
            if (start.row === row && start.column < column) {
                if (end.column >= column)
                    column = start.column;
                else
                    column = Math.max(0, column - (end.column - start.column));

            } else if (start.row !== end.row && start.row < row) {
                if (end.row === row)
                    column = Math.max(0, column - end.column) + start.column;
                row -= (end.row - start.row);
            } else if (end.row === row) {
                row -= end.row - start.row;
                column = Math.max(0, column - end.column) + start.column;
            }
        } else if (delta.action == "removeLines") {
            if (start.row <= row) {
                if (end.row <= row)
                    row -= end.row - start.row;
                else {
                    row = start.row;
                    column = 0;
                }
            }
        }

        this.setPosition(row, column, true);
    };
    this.setPosition = function(row, column, noClip) {
        var pos;
        if (noClip) {
            pos = {
                row: row,
                column: column
            };
        } else {
            pos = this.$clipPositionToDocument(row, column);
        }

        if (this.row == pos.row && this.column == pos.column)
            return;

        var old = {
            row: this.row,
            column: this.column
        };

        this.row = pos.row;
        this.column = pos.column;
        this._signal("change", {
            old: old,
            value: pos
        });
    };
    this.detach = function() {
        this.document.removeEventListener("change", this.$onChange);
    };
    this.attach = function(doc) {
        this.document = doc || this.document;
        this.document.on("change", this.$onChange);
    };
    this.$clipPositionToDocument = function(row, column) {
        var pos = {};

        if (row >= this.document.getLength()) {
            pos.row = Math.max(0, this.document.getLength() - 1);
            pos.column = this.document.getLine(pos.row).length;
        }
        else if (row < 0) {
            pos.row = 0;
            pos.column = 0;
        }
        else {
            pos.row = row;
            pos.column = Math.min(this.document.getLine(pos.row).length, Math.max(0, column));
        }

        if (column < 0)
            pos.column = 0;

        return pos;
    };

}).call(Anchor.prototype);

});

define("ace/document",["require","exports","module","ace/lib/oop","ace/lib/event_emitter","ace/range","ace/anchor"], function(require, exports, module) {
"use strict";

var oop = require("./lib/oop");
var EventEmitter = require("./lib/event_emitter").EventEmitter;
var Range = require("./range").Range;
var Anchor = require("./anchor").Anchor;

var Document = function(text) {
    this.$lines = [];
    if (text.length === 0) {
        this.$lines = [""];
    } else if (Array.isArray(text)) {
        this._insertLines(0, text);
    } else {
        this.insert({row: 0, column:0}, text);
    }
};

(function() {

    oop.implement(this, EventEmitter);
    this.setValue = function(text) {
        var len = this.getLength();
        this.remove(new Range(0, 0, len, this.getLine(len-1).length));
        this.insert({row: 0, column:0}, text);
    };
    this.getValue = function() {
        return this.getAllLines().join(this.getNewLineCharacter());
    };
    this.createAnchor = function(row, column) {
        return new Anchor(this, row, column);
    };
    if ("aaa".split(/a/).length === 0)
        this.$split = function(text) {
            return text.replace(/\r\n|\r/g, "\n").split("\n");
        };
    else
        this.$split = function(text) {
            return text.split(/\r\n|\r|\n/);
        };


    this.$detectNewLine = function(text) {
        var match = text.match(/^.*?(\r\n|\r|\n)/m);
        this.$autoNewLine = match ? match[1] : "\n";
        this._signal("changeNewLineMode");
    };
    this.getNewLineCharacter = function() {
        switch (this.$newLineMode) {
          case "windows":
            return "\r\n";
          case "unix":
            return "\n";
          default:
            return this.$autoNewLine || "\n";
        }
    };

    this.$autoNewLine = "";
    this.$newLineMode = "auto";
    this.setNewLineMode = function(newLineMode) {
        if (this.$newLineMode === newLineMode)
            return;

        this.$newLineMode = newLineMode;
        this._signal("changeNewLineMode");
    };
    this.getNewLineMode = function() {
        return this.$newLineMode;
    };
    this.isNewLine = function(text) {
        return (text == "\r\n" || text == "\r" || text == "\n");
    };
    this.getLine = function(row) {
        return this.$lines[row] || "";
    };
    this.getLines = function(firstRow, lastRow) {
        return this.$lines.slice(firstRow, lastRow + 1);
    };
    this.getAllLines = function() {
        return this.getLines(0, this.getLength());
    };
    this.getLength = function() {
        return this.$lines.length;
    };
    this.getTextRange = function(range) {
        if (range.start.row == range.end.row) {
            return this.getLine(range.start.row)
                .substring(range.start.column, range.end.column);
        }
        var lines = this.getLines(range.start.row, range.end.row);
        lines[0] = (lines[0] || "").substring(range.start.column);
        var l = lines.length - 1;
        if (range.end.row - range.start.row == l)
            lines[l] = lines[l].substring(0, range.end.column);
        return lines.join(this.getNewLineCharacter());
    };

    this.$clipPosition = function(position) {
        var length = this.getLength();
        if (position.row >= length) {
            position.row = Math.max(0, length - 1);
            position.column = this.getLine(length-1).length;
        } else if (position.row < 0)
            position.row = 0;
        return position;
    };
    this.insert = function(position, text) {
        if (!text || text.length === 0)
            return position;

        position = this.$clipPosition(position);
        if (this.getLength() <= 1)
            this.$detectNewLine(text);

        var lines = this.$split(text);
        var firstLine = lines.splice(0, 1)[0];
        var lastLine = lines.length == 0 ? null : lines.splice(lines.length - 1, 1)[0];

        position = this.insertInLine(position, firstLine);
        if (lastLine !== null) {
            position = this.insertNewLine(position); // terminate first line
            position = this._insertLines(position.row, lines);
            position = this.insertInLine(position, lastLine || "");
        }
        return position;
    };
    this.insertLines = function(row, lines) {
        if (row >= this.getLength())
            return this.insert({row: row, column: 0}, "\n" + lines.join("\n"));
        return this._insertLines(Math.max(row, 0), lines);
    };
    this._insertLines = function(row, lines) {
        if (lines.length == 0)
            return {row: row, column: 0};
        while (lines.length > 0xF000) {
            var end = this._insertLines(row, lines.slice(0, 0xF000));
            lines = lines.slice(0xF000);
            row = end.row;
        }

        var args = [row, 0];
        args.push.apply(args, lines);
        this.$lines.splice.apply(this.$lines, args);

        var range = new Range(row, 0, row + lines.length, 0);
        var delta = {
            action: "insertLines",
            range: range,
            lines: lines
        };
        this._signal("change", { data: delta });
        return range.end;
    };
    this.insertNewLine = function(position) {
        position = this.$clipPosition(position);
        var line = this.$lines[position.row] || "";

        this.$lines[position.row] = line.substring(0, position.column);
        this.$lines.splice(position.row + 1, 0, line.substring(position.column, line.length));

        var end = {
            row : position.row + 1,
            column : 0
        };

        var delta = {
            action: "insertText",
            range: Range.fromPoints(position, end),
            text: this.getNewLineCharacter()
        };
        this._signal("change", { data: delta });

        return end;
    };
    this.insertInLine = function(position, text) {
        if (text.length == 0)
            return position;

        var line = this.$lines[position.row] || "";

        this.$lines[position.row] = line.substring(0, position.column) + text
                + line.substring(position.column);

        var end = {
            row : position.row,
            column : position.column + text.length
        };

        var delta = {
            action: "insertText",
            range: Range.fromPoints(position, end),
            text: text
        };
        this._signal("change", { data: delta });

        return end;
    };
    this.remove = function(range) {
        if (!(range instanceof Range))
            range = Range.fromPoints(range.start, range.end);
        range.start = this.$clipPosition(range.start);
        range.end = this.$clipPosition(range.end);

        if (range.isEmpty())
            return range.start;

        var firstRow = range.start.row;
        var lastRow = range.end.row;

        if (range.isMultiLine()) {
            var firstFullRow = range.start.column == 0 ? firstRow : firstRow + 1;
            var lastFullRow = lastRow - 1;

            if (range.end.column > 0)
                this.removeInLine(lastRow, 0, range.end.column);

            if (lastFullRow >= firstFullRow)
                this._removeLines(firstFullRow, lastFullRow);

            if (firstFullRow != firstRow) {
                this.removeInLine(firstRow, range.start.column, this.getLine(firstRow).length);
                this.removeNewLine(range.start.row);
            }
        }
        else {
            this.removeInLine(firstRow, range.start.column, range.end.column);
        }
        return range.start;
    };
    this.removeInLine = function(row, startColumn, endColumn) {
        if (startColumn == endColumn)
            return;

        var range = new Range(row, startColumn, row, endColumn);
        var line = this.getLine(row);
        var removed = line.substring(startColumn, endColumn);
        var newLine = line.substring(0, startColumn) + line.substring(endColumn, line.length);
        this.$lines.splice(row, 1, newLine);

        var delta = {
            action: "removeText",
            range: range,
            text: removed
        };
        this._signal("change", { data: delta });
        return range.start;
    };
    this.removeLines = function(firstRow, lastRow) {
        if (firstRow < 0 || lastRow >= this.getLength())
            return this.remove(new Range(firstRow, 0, lastRow + 1, 0));
        return this._removeLines(firstRow, lastRow);
    };

    this._removeLines = function(firstRow, lastRow) {
        var range = new Range(firstRow, 0, lastRow + 1, 0);
        var removed = this.$lines.splice(firstRow, lastRow - firstRow + 1);

        var delta = {
            action: "removeLines",
            range: range,
            nl: this.getNewLineCharacter(),
            lines: removed
        };
        this._signal("change", { data: delta });
        return removed;
    };
    this.removeNewLine = function(row) {
        var firstLine = this.getLine(row);
        var secondLine = this.getLine(row+1);

        var range = new Range(row, firstLine.length, row+1, 0);
        var line = firstLine + secondLine;

        this.$lines.splice(row, 2, line);

        var delta = {
            action: "removeText",
            range: range,
            text: this.getNewLineCharacter()
        };
        this._signal("change", { data: delta });
    };
    this.replace = function(range, text) {
        if (!(range instanceof Range))
            range = Range.fromPoints(range.start, range.end);
        if (text.length == 0 && range.isEmpty())
            return range.start;
        if (text == this.getTextRange(range))
            return range.end;

        this.remove(range);
        if (text) {
            var end = this.insert(range.start, text);
        }
        else {
            end = range.start;
        }

        return end;
    };
    this.applyDeltas = function(deltas) {
        for (var i=0; i<deltas.length; i++) {
            var delta = deltas[i];
            var range = Range.fromPoints(delta.range.start, delta.range.end);

            if (delta.action == "insertLines")
                this.insertLines(range.start.row, delta.lines);
            else if (delta.action == "insertText")
                this.insert(range.start, delta.text);
            else if (delta.action == "removeLines")
                this._removeLines(range.start.row, range.end.row - 1);
            else if (delta.action == "removeText")
                this.remove(range);
        }
    };
    this.revertDeltas = function(deltas) {
        for (var i=deltas.length-1; i>=0; i--) {
            var delta = deltas[i];

            var range = Range.fromPoints(delta.range.start, delta.range.end);

            if (delta.action == "insertLines")
                this._removeLines(range.start.row, range.end.row - 1);
            else if (delta.action == "insertText")
                this.remove(range);
            else if (delta.action == "removeLines")
                this._insertLines(range.start.row, delta.lines);
            else if (delta.action == "removeText")
                this.insert(range.start, delta.text);
        }
    };
    this.indexToPosition = function(index, startRow) {
        var lines = this.$lines || this.getAllLines();
        var newlineLength = this.getNewLineCharacter().length;
        for (var i = startRow || 0, l = lines.length; i < l; i++) {
            index -= lines[i].length + newlineLength;
            if (index < 0)
                return {row: i, column: index + lines[i].length + newlineLength};
        }
        return {row: l-1, column: lines[l-1].length};
    };
    this.positionToIndex = function(pos, startRow) {
        var lines = this.$lines || this.getAllLines();
        var newlineLength = this.getNewLineCharacter().length;
        var index = 0;
        var row = Math.min(pos.row, lines.length);
        for (var i = startRow || 0; i < row; ++i)
            index += lines[i].length + newlineLength;

        return index + pos.column;
    };

}).call(Document.prototype);

exports.Document = Document;
});

define("ace/lib/lang",["require","exports","module"], function(require, exports, module) {
"use strict";

exports.last = function(a) {
    return a[a.length - 1];
};

exports.stringReverse = function(string) {
    return string.split("").reverse().join("");
};

exports.stringRepeat = function (string, count) {
    var result = '';
    while (count > 0) {
        if (count & 1)
            result += string;

        if (count >>= 1)
            string += string;
    }
    return result;
};

var trimBeginRegexp = /^\s\s*/;
var trimEndRegexp = /\s\s*$/;

exports.stringTrimLeft = function (string) {
    return string.replace(trimBeginRegexp, '');
};

exports.stringTrimRight = function (string) {
    return string.replace(trimEndRegexp, '');
};

exports.copyObject = function(obj) {
    var copy = {};
    for (var key in obj) {
        copy[key] = obj[key];
    }
    return copy;
};

exports.copyArray = function(array){
    var copy = [];
    for (var i=0, l=array.length; i<l; i++) {
        if (array[i] && typeof array[i] == "object")
            copy[i] = this.copyObject( array[i] );
        else 
            copy[i] = array[i];
    }
    return copy;
};

exports.deepCopy = function (obj) {
    if (typeof obj !== "object" || !obj)
        return obj;
    var cons = obj.constructor;
    if (cons === RegExp)
        return obj;
    
    var copy = cons();
    for (var key in obj) {
        if (typeof obj[key] === "object") {
            copy[key] = exports.deepCopy(obj[key]);
        } else {
            copy[key] = obj[key];
        }
    }
    return copy;
};

exports.arrayToMap = function(arr) {
    var map = {};
    for (var i=0; i<arr.length; i++) {
        map[arr[i]] = 1;
    }
    return map;

};

exports.createMap = function(props) {
    var map = Object.create(null);
    for (var i in props) {
        map[i] = props[i];
    }
    return map;
};
exports.arrayRemove = function(array, value) {
  for (var i = 0; i <= array.length; i++) {
    if (value === array[i]) {
      array.splice(i, 1);
    }
  }
};

exports.escapeRegExp = function(str) {
    return str.replace(/([.*+?^${}()|[\]\/\\])/g, '\\$1');
};

exports.escapeHTML = function(str) {
    return str.replace(/&/g, "&#38;").replace(/"/g, "&#34;").replace(/'/g, "&#39;").replace(/</g, "&#60;");
};

exports.getMatchOffsets = function(string, regExp) {
    var matches = [];

    string.replace(regExp, function(str) {
        matches.push({
            offset: arguments[arguments.length-2],
            length: str.length
        });
    });

    return matches;
};
exports.deferredCall = function(fcn) {
    var timer = null;
    var callback = function() {
        timer = null;
        fcn();
    };

    var deferred = function(timeout) {
        deferred.cancel();
        timer = setTimeout(callback, timeout || 0);
        return deferred;
    };

    deferred.schedule = deferred;

    deferred.call = function() {
        this.cancel();
        fcn();
        return deferred;
    };

    deferred.cancel = function() {
        clearTimeout(timer);
        timer = null;
        return deferred;
    };
    
    deferred.isPending = function() {
        return timer;
    };

    return deferred;
};


exports.delayedCall = function(fcn, defaultTimeout) {
    var timer = null;
    var callback = function() {
        timer = null;
        fcn();
    };

    var _self = function(timeout) {
        if (timer == null)
            timer = setTimeout(callback, timeout || defaultTimeout);
    };

    _self.delay = function(timeout) {
        timer && clearTimeout(timer);
        timer = setTimeout(callback, timeout || defaultTimeout);
    };
    _self.schedule = _self;

    _self.call = function() {
        this.cancel();
        fcn();
    };

    _self.cancel = function() {
        timer && clearTimeout(timer);
        timer = null;
    };

    _self.isPending = function() {
        return timer;
    };

    return _self;
};
});

define("ace/worker/mirror",["require","exports","module","ace/document","ace/lib/lang"], function(require, exports, module) {
"use strict";

var Document = require("../document").Document;
var lang = require("../lib/lang");
    
var Mirror = exports.Mirror = function(sender) {
    this.sender = sender;
    var doc = this.doc = new Document("");
    
    var deferredUpdate = this.deferredUpdate = lang.delayedCall(this.onUpdate.bind(this));
    
    var _self = this;
    sender.on("change", function(e) {
        doc.applyDeltas(e.data);
        if (_self.$timeout)
            return deferredUpdate.schedule(_self.$timeout);
        _self.onUpdate();
    });
};

(function() {
    
    this.$timeout = 500;
    
    this.setTimeout = function(timeout) {
        this.$timeout = timeout;
    };
    
    this.setValue = function(value) {
        this.doc.setValue(value);
        this.deferredUpdate.schedule(this.$timeout);
    };
    
    this.getValue = function(callbackId) {
        this.sender.callback(this.doc.getValue(), callbackId);
    };
    
    this.onUpdate = function() {
    };
    
    this.isPending = function() {
        return this.deferredUpdate.isPending();
    };
    
}).call(Mirror.prototype);

});

define("ace/mode/xquery/xqlint",["require","exports","module"], function(require, exports, module) {
module.exports = (function e(t,n,r){function s(o,u){if(!n[o]){if(!t[o]){var a=typeof require=="function"&&require;if(!u&&a)return a(o,!0);if(i)return i(o,!0);throw new Error("Cannot find module '"+o+"'")}var f=n[o]={exports:{}};t[o][0].call(f.exports,function(e){var n=t[o][1][e];return s(n?n:e)},f,f.exports,e,t,n,r)}return n[o].exports}var i=typeof require=="function"&&require;for(var o=0;o<r.length;o++)s(r[o]);return s})({
1:[function(_dereq_,module,exports){
'use strict';

var init = function(that, code, message, pos, type){
    if(!code) {
        throw new Error(type + ' code is missing.');
    }
    
    if(!message) {
        throw new Error(type + ' message is missing.');
    }
    
    if(!pos) {
        throw new Error(type + ' position is missing.');
    }

    that.getCode = function(){
        return code;
    };
    
    that.getMessage = function(){
        return message;
    };

    that.getPos = function(){
        return pos;
    };
};

var StaticError = {};
var StaticWarning = {};
StaticError.prototype = new Error();
StaticWarning.prototype = new Error();

exports.StaticError = StaticError.prototype.constructor = function(code, message, pos) {
    init(this, code, message, pos, 'Error');
};

exports.StaticWarning = StaticWarning.prototype.constructor = function(code, message, pos) {
    init(this, code, message, pos, 'Warning');
};
},
{}],
2:[function(_dereq_,module,exports){
'use strict';

var TreeOps = _dereq_('../tree_ops').TreeOps;
var Errors = _dereq_('./errors');
var StaticWarning = Errors.StaticWarning;
exports.ModuleDecl = function(translator, rootSctx, node){
    var prefix = '';
    return {
        NCName: function(ncname){
            prefix = TreeOps.flatten(ncname);
        },

        URILiteral: function(uri) {
            uri = TreeOps.flatten(uri);
            uri = uri.substring(1, uri.length - 1);
            translator.apply(function(){
                rootSctx.moduleNamespace = uri;
                rootSctx.addNamespace(uri, prefix, node.pos, 'moduleDecl');
            });
        }
    };
};

exports.ModuleImport = function(translator, rootSctx, node) {
    var prefix = '';
    var moduleURI;

    return {
        NCName: function(ncname){
            prefix = TreeOps.flatten(ncname);
        },

        URILiteral: function(uri) {
            if(moduleURI !== undefined) {
                return;
            }
            uri = TreeOps.flatten(uri);
            uri = uri.substring(1, uri.length - 1);
            moduleURI = uri;
            translator.apply(function(){
                rootSctx.importModule(uri, prefix, node.pos);
            });
        }
    };
};

exports.SchemaImport = function(translator, rootSctx, node) {
    var prefix = '';
    var schemaURI;
    
    return {
        SchemaPrefix: function(schemaPrefix) {
            var SchemaPrefixHandler = function () {
                this.NCName = function (ncname) {
                    prefix = TreeOps.flatten(ncname);
                };
            };
            translator.visitChildren(schemaPrefix, new SchemaPrefixHandler());
        },

        URILiteral: function(uri) {
            if(schemaURI !== undefined) {
                return;
            }
            uri = TreeOps.flatten(uri);
            uri = uri.substring(1, uri.length - 1);
            schemaURI = uri;
            translator.apply(function(){
                rootSctx.addNamespace(uri, prefix, node.pos, 'schema');
            });
        }
    };
};

exports.DefaultNamespaceDecl = function(translator, rootSctx, node) {
    var fn = false;
    var ns = '';

    return {
        TOKEN: function(token){
            fn = fn ? true : (token.value === 'function');
        },
        URILiteral: function(uri){
            ns = TreeOps.flatten(uri);
            ns = ns.substring(1, ns.length - 1);
            if(!fn) {
                translator.apply(function(){
                    throw new StaticWarning('W06', 'Avoid default element namespace declarations.', node.pos);
                });
                rootSctx.defaultElementNamespace = ns;
            } else {
                rootSctx.defaultFunctionNamespace = ns;
            }
        }
    };
};

exports.NamespaceDecl = function(translator, rootSctx, node) {
    var prefix = '';
    return {
        NCName: function(ncname) {
            prefix = TreeOps.flatten(ncname);
        },
        URILiteral: function(uri) {
            uri = TreeOps.flatten(uri);
            uri = uri.substring(1, uri.length - 1);
            translator.apply(function(){
                rootSctx.addNamespace(uri, prefix, node.pos, 'declare');
            });
        }
    };
};
exports.VarHandler = function(translator, sctx, node){
    var EQNameHandler = function(eqname){
        var value = TreeOps.flatten(eqname);
        translator.apply(function(){
            var qname = sctx.resolveQName(value, eqname.pos);
            sctx.addVariable(qname, node.name, eqname.pos);
        });
    };
    return {
        ExprSingle: function(){ return true; },
        VarValue: function(){ return true; },
        VarDefaultValue: function(){ return true; },
        VarName: EQNameHandler,
        EQName: EQNameHandler
    };
};

exports.VarRefHandler = function(translator, sctx, node){
    return {
        VarName: function(eqname){
            var value = TreeOps.flatten(eqname);
            translator.apply(function(){
                var qname = sctx.resolveQName(value, node.pos);
                if(qname.uri !== '') {
                    sctx.root.namespaces[qname.uri].used = true;
                }
                sctx.addVarRef(qname, eqname.pos);
            });
        }
    };
};
},
{"../tree_ops":11,"./errors":1}],
3:[function(_dereq_,module,exports){
'use strict';
exports.getSchemaBuiltinTypes = function(){
    var ns = 'http://www.w3.org/2001/XMLSchema';
    var SchemaBuiltinTypes = {};
    SchemaBuiltinTypes[ns] = {
        variables: {},
        functions: {}
    };
    SchemaBuiltinTypes[ns].functions[ns + '#string#1'] = { params: ['$arg as xs:anyAtomicType?'], annotations: [], name: 'string', arity: 1, eqname: { uri: ns, name: 'string' } };
    SchemaBuiltinTypes[ns].functions[ns + '#boolean#1'] = { params: ['$arg as xs:anyAtomicType?'], annotations: [], name: 'boolean', arity: 1, eqname: { uri: ns, name: 'boolean' } };
    SchemaBuiltinTypes[ns].functions[ns + '#decimal#1'] = { params: ['$arg as xs:anyAtomicType?'], annotations: [], name: 'decimal', arity: 1, eqname: { uri: ns, name: 'decimal' } };
    SchemaBuiltinTypes[ns].functions[ns + '#float#1'] = { params: ['$arg as xs:anyAtomicType?'], annotations: [], name: 'float', arity: 1, eqname: { uri: ns, name: 'float' } };
    SchemaBuiltinTypes[ns].functions[ns + '#double#1'] = { params: ['$arg as xs:anyAtomicType?'], annotations: [], name: 'double', arity: 1, eqname: { uri: ns, name: 'double' } };
    SchemaBuiltinTypes[ns].functions[ns + '#duration#1'] = { params: ['$arg as xs:anyAtomicType?'], annotations: [], name: 'duration', arity: 1, eqname: { uri: ns, name: 'duration' } };
    SchemaBuiltinTypes[ns].functions[ns + '#dateTime#1'] = { params: ['$arg as xs:anyAtomicType?'], annotations: [], name: 'dateTime', arity: 1, eqname: { uri: ns, name: 'dateTime' } };
    SchemaBuiltinTypes[ns].functions[ns + '#time#1'] = { params: ['$arg as xs:anyAtomicType?'], annotations: [], name: 'time', arity: 1, eqname: { uri: ns, name: 'time' } };
    SchemaBuiltinTypes[ns].functions[ns + '#date#1'] = { params: ['$arg as xs:anyAtomicType?'], annotations: [], name: 'date', arity: 1, eqname: { uri: ns, name: 'date' } };
    SchemaBuiltinTypes[ns].functions[ns + '#gYearMonth#1'] = { params: ['$arg as xs:anyAtomicType?'], annotations: [], name: 'gYearMonth', arity: 1, eqname: { uri: ns, name: 'gYearMonth' } };
    SchemaBuiltinTypes[ns].functions[ns + '#gYear#1'] = { params: ['$arg as xs:anyAtomicType?'], annotations: [], name: 'gYear', arity: 1, eqname: { uri: ns, name: 'gYear' } };
    SchemaBuiltinTypes[ns].functions[ns + '#gMonthDay#1'] = { params: ['$arg as xs:anyAtomicType?'], annotations: [], name: 'gMonthDay', arity: 1, eqname: { uri: ns, name: 'gMonthDay' } };
    SchemaBuiltinTypes[ns].functions[ns + '#gDay#1'] = { params: ['$arg as xs:anyAtomicType?'], annotations: [], name: 'gDay', arity: 1, eqname: { uri: ns, name: 'gDay' } };
    SchemaBuiltinTypes[ns].functions[ns + '#gMonth#1'] = { params: ['$arg as xs:anyAtomicType?'], annotations: [], name: 'gMonth', arity: 1, eqname: { uri: ns, name: 'gMonth' } };
    SchemaBuiltinTypes[ns].functions[ns + '#hexBinary#1'] = { params: ['$arg as xs:anyAtomicType?'], annotations: [], name: 'hexBinary', arity: 1, eqname: { uri: ns, name: 'hexBinary' } };
    SchemaBuiltinTypes[ns].functions[ns + '#base64Binary#1'] = { params: ['$arg as xs:anyAtomicType?'], annotations: [], name: 'base64Binary', arity: 1, eqname: { uri: ns, name: 'base64Binary' } };
    SchemaBuiltinTypes[ns].functions[ns + '#anyURI#1'] = { params: ['$arg as xs:anyAtomicType?'], annotations: [], name: 'anyURI', arity: 1, eqname: { uri: ns, name: 'anyURI' } };
    SchemaBuiltinTypes[ns].functions[ns + '#QName#1'] = { params: ['$arg as xs:anyAtomicType?'], annotations: [], name: 'QName', arity: 1, eqname: { uri: ns, name: 'QName' } };
    SchemaBuiltinTypes[ns].functions[ns + '#normalizedString#1'] = { params: ['$arg as xs:anyAtomicType?'], annotations: [], name: 'normalizedString', arity: 1, eqname: { uri: ns, name: 'normalizedString' } };
    SchemaBuiltinTypes[ns].functions[ns + '#token#1'] = { params: ['$arg as xs:anyAtomicType?'], annotations: [], name: 'token', arity: 1, eqname: { uri: ns, name: 'token' } };
    SchemaBuiltinTypes[ns].functions[ns + '#language#1'] = { params: ['$arg as xs:anyAtomicType?'], annotations: [], name: 'language', arity: 1, eqname: { uri: ns, name: 'language' } };
    SchemaBuiltinTypes[ns].functions[ns + '#NMTOKEN#1'] = { params: ['$arg as xs:anyAtomicType?'], annotations: [], name: 'NMTOKEN', arity: 1, eqname: { uri: ns, name: 'NMTOKEN' } };
    SchemaBuiltinTypes[ns].functions[ns + '#Name#1'] = { params: ['$arg as xs:anyAtomicType?'], annotations: [], name: 'Name', arity: 1, eqname: { uri: ns, name: 'Name' } };
    SchemaBuiltinTypes[ns].functions[ns + '#NCName#1'] = { params: ['$arg as xs:anyAtomicType?'], annotations: [], name: 'NCName', arity: 1, eqname: { uri: ns, name: 'NCName' } };
    SchemaBuiltinTypes[ns].functions[ns + '#ID#1'] = { params: ['$arg as xs:anyAtomicType?'], annotations: [], name: 'ID', arity: 1, eqname: { uri: ns, name: 'ID' } };
    SchemaBuiltinTypes[ns].functions[ns + '#IDREF#1'] = { params: ['$arg as xs:anyAtomicType?'], annotations: [], name: 'IDREF', arity: 1, eqname: { uri: ns, name: 'IDREF' } };
    SchemaBuiltinTypes[ns].functions[ns + '#ENTITY#1'] = { params: ['$arg as xs:anyAtomicType?'], annotations: [], name: 'ENTITY', arity: 1, eqname: { uri: ns, name: 'ENTITY' } };
    SchemaBuiltinTypes[ns].functions[ns + '#integer#1'] = { params: ['$arg as xs:anyAtomicType?'], annotations: [], name: 'integer', arity: 1, eqname: { uri: ns, name: 'integer' } };
    SchemaBuiltinTypes[ns].functions[ns + '#nonPositiveInteger#1'] = { params: ['$arg as xs:anyAtomicType?'], annotations: [], name: 'nonPositiveInteger', arity: 1, eqname: { uri: ns, name: 'nonPositiveInteger' } };
    SchemaBuiltinTypes[ns].functions[ns + '#negativeInteger#1'] = { params: ['$arg as xs:anyAtomicType?'], annotations: [], name: 'negativeInteger', arity: 1, eqname: { uri: ns, name: 'negativeInteger' } };
    SchemaBuiltinTypes[ns].functions[ns + '#long#1'] = { params: ['$arg as xs:anyAtomicType?'], annotations: [], name: 'long', arity: 1, eqname: { uri: ns, name: 'long' } };
    SchemaBuiltinTypes[ns].functions[ns + '#int#1'] = { params: ['$arg as xs:anyAtomicType?'], annotations: [], name: 'int', arity: 1, eqname: { uri: ns, name: 'int' } };
    SchemaBuiltinTypes[ns].functions[ns + '#short#1'] = { params: ['$arg as xs:anyAtomicType?'], annotations: [], name: 'short', arity: 1, eqname: { uri: ns, name: 'short' } };
    SchemaBuiltinTypes[ns].functions[ns + '#byte#1'] = { params: ['$arg as xs:anyAtomicType?'], annotations: [], name: 'byte', arity: 1, eqname: { uri: ns, name: 'byte' } };
    SchemaBuiltinTypes[ns].functions[ns + '#nonNegativeInteger#1'] = { params: ['$arg as xs:anyAtomicType?'], annotations: [], name: 'nonNegativeInteger', arity: 1, eqname: { uri: ns, name: 'nonNegativeInteger' } };
    SchemaBuiltinTypes[ns].functions[ns + '#unsignedLong#1'] = { params: ['$arg as xs:anyAtomicType?'], annotations: [], name: 'unsignedLong', arity: 1, eqname: { uri: ns, name: 'unsignedLong' } };
    SchemaBuiltinTypes[ns].functions[ns + '#unsignedInt#1'] = { params: ['$arg as xs:anyAtomicType?'], annotations: [], name: 'unsignedInt', arity: 1, eqname: { uri: ns, name: 'unsignedInt' } };
    SchemaBuiltinTypes[ns].functions[ns + '#unsignedShort#1'] = { params: ['$arg as xs:anyAtomicType?'], annotations: [], name: 'unsignedShort', arity: 1, eqname: { uri: ns, name: 'unsignedShort' } };
    SchemaBuiltinTypes[ns].functions[ns + '#unsignedByte#1'] = { params: ['$arg as xs:anyAtomicType?'], annotations: [], name: 'unsignedByte', arity: 1, eqname: { uri: ns, name: 'unsignedByte' } };
    SchemaBuiltinTypes[ns].functions[ns + '#positiveInteger#1'] = { params: ['$arg as xs:anyAtomicType?'], annotations: [], name: 'positiveInteger', arity: 1, eqname: { uri: ns, name: 'positiveInteger' } };
    SchemaBuiltinTypes[ns].functions[ns + '#yearMonthDuration#1'] = { params: ['$arg as xs:anyAtomicType?'], annotations: [], name: 'yearMonthDuration', arity: 1, eqname: { uri: ns, name: 'yearMonthDuration' } };
    SchemaBuiltinTypes[ns].functions[ns + '#dayTimeDuration#1'] = { params: ['$arg as xs:anyAtomicType?'], annotations: [], name: 'dayTimeDuration', arity: 1, eqname: { uri: ns, name: 'dayTimeDuration' } };
    SchemaBuiltinTypes[ns].functions[ns + '#untypedAtomic#1'] = { params: ['$arg as xs:anyAtomicType?'], annotations: [], name: 'untypedAtomic', arity: 1, eqname: { uri: ns, name: 'untypedAtomic' } };
    return SchemaBuiltinTypes;
};
},
{}],
4:[function(_dereq_,module,exports){
exports.StaticContext = function (parent, pos) {
    'use strict';
    
    var TreeOps = _dereq_('../tree_ops').TreeOps;
    
    var Errors = _dereq_('./errors');
    var StaticError = Errors.StaticError;
    var StaticWarning = Errors.StaticWarning;
    
    var getSchemaBuiltinTypes = _dereq_('./schema_built-in_types').getSchemaBuiltinTypes;
    
    var emptyPos = { sl:0, sc: 0, el: 0, ec: 0 };
    var namespaces = {};
    
    var getVarKey = function(qname) {
        return qname.uri + '#' + qname.name;
    };

    var getFnKey = function(qname, arity) {
        return getVarKey(qname) + '#' + arity;
    };

    if(!parent) {
        namespaces['http://jsoniq.org/functions'] = {
            prefix: 'jn',
            pos: emptyPos,
            type: 'module',
            override: true
        };
        namespaces['http://www.28msec.com/modules/collections'] = {
            prefix: 'db',
            pos: emptyPos,
            type: 'module',
            override: true
        };
        namespaces['http://www.28msec.com/modules/store'] = {
            prefix: 'store',
            pos: emptyPos,
            type: 'module',
            override: true
        };
        namespaces['http://jsoniq.org/function-library'] = {
            prefix: 'libjn',
            pos: emptyPos,
            type: 'module',
            override: true
        };
        namespaces['http://www.w3.org/2005/xpath-functions'] = {
            prefix: 'fn',
            pos: emptyPos,
            type: 'module',
            override: true
        };
        namespaces['http://www.w3.org/2005/xquery-local-functions'] = {
            prefix: 'local',
            pos: emptyPos,
            type: 'declare',
            override: true
        };
        namespaces['http://www.w3.org/2001/XMLSchema-instance'] = {
            prefix: 'xsi',
            pos: emptyPos,
            type: 'declare'
        };
        namespaces['http://www.w3.org/2001/XMLSchema'] = {
            prefix: 'xs',
            pos: emptyPos,
            type: 'declare'
        };
        namespaces['http://www.w3.org/XML/1998/namespace'] = {
            prefix: 'xml',
            pos: emptyPos,
            type: 'declare'
        };
        namespaces['http://zorba.io/annotations'] = {
            prefix: 'an',
            pos: emptyPos,
            type: 'declare',
            override: true
        };
        namespaces['http://www.28msec.com/annotations/rest'] = {
            prefix: 'rest',
            pos: emptyPos,
            type: 'declare',
            override: true
        };
        namespaces['http://www.w3.org/2005/xqt-errors'] = {
            prefix: 'err',
            pos: emptyPos,
            type: 'declare',
            override: true
        };
        namespaces['http://zorba.io/errors'] = {
            prefix: 'zerr',
            pos: emptyPos,
            type: 'declare',
            override: true
        };
    }

    var s = {
        parent: parent,
        children: [],
        pos: pos,
        setModuleResolver: function(resolver){
            this.root.moduleResolver = resolver;
            return this;
        },
        setModules: function(index){
            if(this !== this.root){
                throw new Error('setModules() not invoked from the root static context.');
            }
            this.moduleResolver = function(uri){
                return index[uri];
            };
            var that = this;
            Object.keys(this.namespaces).forEach(function(uri){
                var ns = that.namespaces[uri];
                if(ns.type === 'module') {
                    var mod = that.moduleResolver(uri);
                    if(mod.variables) {
                        TreeOps.concat(that.variables, mod.variables);
                    }
                    if(mod.functions) {
                        TreeOps.concat(that.functions, mod.functions);
                    }
                }
            });
            return this;
        },
        setModulesFromXQDoc: function(xqdoc){
            if(this !== this.root){
                throw new Error('setModulesFromXQDoc() not invoked from the root static context.');
            }
            var index = {};
            Object.keys(xqdoc).forEach(function(uri) {
                var mod = xqdoc[uri];
                var variables = {};
                var functions = {};
                mod.functions.forEach(function(fn){
                    functions[uri + '#' + fn.name + '#' + fn.arity] = {
                        params: [],
                        annotations: [],
                        name: fn.name,
                        arity: fn.arity,
                        eqname: { uri: uri, name: fn.name }
                    };
                    fn.parameters.forEach(function(param){
                        functions[uri + '#' + fn.name + '#' + fn.arity].params.push('$' + param.name);
                    });
                });
                mod.variables.forEach(function(variable){
                    var name = variable.name.substring(variable.name.indexOf(':') + 1);
                    variables[uri + '#' + name] = { type: 'VarDecl', annotations: [], eqname: { uri: uri, name: name } };
                });
                index[uri] = {
                    variables: variables,
                    functions: functions
                };
            });
            this.root.moduleResolver = function(uri){
                return index[uri];
            };
            var that = this;
            Object.keys(this.namespaces).forEach(function(uri){
                var ns = that.namespaces[uri];
                if(ns.type === 'module') {
                    var mod = that.moduleResolver(uri);
                    if(mod.variables) {
                        TreeOps.concat(that.variables, mod.variables);
                    }
                    if(mod.functions) {
                        TreeOps.concat(that.functions, mod.functions);
                    }
                }
            });
            return this;
        },
        moduleNamespace: '',
        defaultFunctionNamespace: 'http://www.w3.org/2005/xpath-functions',
        defaultFunctionNamespaces: [
            'http://www.28msec.com/modules/collections',
            'http://www.28msec.com/modules/store',
            'http://jsoniq.org/functions',
            'http://jsoniq.org/function-library',
            'http://www.w3.org/2001/XMLSchema' //Built-in type constructors
        ],
        defaultElementNamespace: '',
        namespaces: namespaces,
        availableModuleNamespaces: [],
        importModule: function(uri, prefix, pos) {
            if(this !== this.root){
                throw new Error('Function not invoked from the root static context.');
            }
            this.addNamespace(uri, prefix, pos, 'module');
            if(this.moduleResolver) {
                try {
                    var mod = this.moduleResolver(uri, []);
                    if(mod.variables) {
                        TreeOps.concat(this.variables, mod.variables);
                    }
                    if(mod.functions) {
                        TreeOps.concat(this.functions, mod.functions);
                    }
                } catch(e) {
                    throw new StaticError('XQST0059', 'module "' + uri + '" not found', pos);
                }
            }
            return this;
        },
        getAvailableModuleNamespaces: function(){
            return this.root.availableModuleNamespaces;
        },
        getPrefixByNamespace: function(uri){
            return this.root.namespaces[uri].prefix;
        },
        addNamespace: function (uri, prefix, pos, type) {
            if(prefix === '' && type === 'module') {
                throw new StaticWarning('W01', 'Avoid this type of import. Use import module namespace instead', pos);
            }
            if (uri === '') {
                throw new StaticError('XQST0088', 'empty target namespace in module import or module declaration', pos);
            }
            var namespace = this.getNamespace(uri);
            if (namespace && namespace.type === type && type !== 'declare' && !namespace.override) {
                throw new StaticError('XQST0047', '"' + uri + '": duplicate target namespace', pos);
            }
            namespace = this.getNamespaceByPrefix(prefix);
            if (namespace && !namespace.override) {
                throw new StaticError('XQST0033', '"' + prefix + '": namespace prefix already bound to "' + namespace.uri + '"', pos);
            }

            namespace = this.namespaces[uri];
            this.namespaces[uri] = {
                prefix: prefix,
                pos: pos,
                type: type
            };

            if (namespace) {
                throw new StaticWarning('W02', '"' + uri + '" already bound to the "' + namespace.prefix + '" prefix', pos);
            }

        },

        getNamespaces: function(){
            return this.root.namespaces;
        },
        
        getNamespace: function (uri) {
            var that = this;
            while (that) {
                var namespace = that.namespaces[uri];
                if (namespace) {
                    return namespace;
                }
                that = that.parent;
            }

        },

        getNamespaceByPrefix: function (prefix) {
            var handler = function (uri) {
                var namespace = that.namespaces[uri];
                if (namespace.prefix === prefix) {
                    namespace.uri = uri;
                    throw namespace;
                }
            };
            var that = this;
            while (that) {
                try {
                    Object.keys(that.namespaces).forEach(handler);
                } catch (e) {
                    return e;
                }
                that = that.parent;
            }

        },
        
        resolveQName: function(value, pos){
            var qname = {
                uri: '',
                prefix: '',
                name: ''
            };
            var idx;
            if (value.substring(0, 2) === 'Q{') {
                idx = value.indexOf('}');
                qname.uri = value.substring(2, idx);
                qname.name = value.substring(idx + 1);
            } else {
                idx = value.indexOf(':');
                qname.prefix = value.substring(0, idx);
                var namespace = this.getNamespaceByPrefix(qname.prefix);
                if(!namespace && qname.prefix !== '' && ['fn', 'jn'].indexOf(qname.prefix) === -1) {
                    throw new StaticError('XPST0081', '"' + qname.prefix + '": can not expand prefix of lexical QName to namespace URI', pos);
                }
                if(namespace) {
                    qname.uri = namespace.uri;
                }
                qname.name = value.substring(idx + 1);
            }
            return qname;
        },
        
        variables: {},
        varRefs: {},
        functionCalls: {},
    
        addVariable: function(qname, type, pos){
            if(
                type === 'VarDecl' && this.moduleNamespace !== '' &&
                !(this.moduleNamespace === qname.uri || (qname.uri === '' && this.defaultFunctionNamespace === this.moduleNamespace))
            ) {
                throw new StaticError('XQST0048', '"' + qname.prefix + ':' + qname.name + '": Qname not library namespace', pos);
            }
            var key = getVarKey(qname);
            if(type === 'VarDecl' && this.variables[key]) {
                throw new StaticError('XQST0049', '"' + qname.name + '": duplicate variable declaration', pos);
            }
            this.variables[key] = {
                type: type,
                pos: pos,
                qname: qname,
                annotations: {}
            };
            return this;
        },
        
        getVariables: function(){
            var variables = {};
            var that = this;
            var handler = function(key){
                if(!variables[key]){
                    variables[key] = that.variables[key];
                }
            };
            while(that){
                Object.keys(that.variables).forEach(handler);
                that = that.parent;
            }
            return variables;
        },
        
        getVariable: function(qname) {
            var key = getVarKey(qname);
            var that = this;
            while(that) {
                if(that.variables[key]) {
                    return that.variables[key];
                }
                that = that.parent;
            }
        },
        
        addVarRef: function(qname, pos){
            var varDecl = this.getVariable(qname);
            if(!varDecl && (qname.uri === '' || this.root.moduleResolver)) {
                throw new StaticError('XPST0008', '"' + qname.name + '": undeclared variable', pos);
            }
            var key = getVarKey(qname);
            this.varRefs[key] = true;
        },
        
        addFunctionCall: function(qname, arity, pos){
            var fn = this.getFunction(qname, arity);
            if(!fn && (qname.uri === 'http://www.w3.org/2005/xquery-local-functions' || this.root.moduleResolver)){
                if((qname.uri === 'http://www.w3.org/2005/xpath-functions' ||
                    (qname.uri === '' && this.root.defaultFunctionNamespaces.concat(this.root.defaultFunctionNamespace).indexOf('http://www.w3.org/2005/xpath-functions') !== -1)) && qname.name === 'concat') {
                } else if(!fn){
                    throw new StaticError('XPST0008', '"' + qname.name + '#' + arity + '": undeclared function', pos);
                }
            }
            var key = getFnKey(qname, arity);
            this.functionCalls[key] = true;
        },
        
        functions: getSchemaBuiltinTypes()['http://www.w3.org/2001/XMLSchema'].functions,

        getFunctions: function(){
            return this.root.functions;
        },
        
        getFunction: function(qname, arity){
            var key = getFnKey(qname, arity);
            var fn;
            if(qname.uri === '') {
                var that = this;
                this.root.defaultFunctionNamespaces.concat([this.root.defaultFunctionNamespace]).forEach(function(defaultFunctionNamespace){
                    if(!fn){
                        fn = that.getFunction({ uri: defaultFunctionNamespace, prefix: qname.prefix, name: qname.name }, arity);
                    } else {
                        return false;
                    }
                });
                return fn;
            } else {
                return this.root.functions[key];
            }
        },
        
        addFunction: function(qname, pos, params) {
            if(this !== this.root){
                throw new Error('addFunction() not invoked from the root static context.');
            }
            var arity = params.length;
            if(
                this.moduleNamespace !== '' &&
                !(this.moduleNamespace === qname.uri || (qname.uri === '' && this.defaultFunctionNamespace === this.moduleNamespace))
            ) {
                throw new StaticError('XQST0048', '"' + qname.prefix + ':' + qname.name + '": Qname not library namespace', pos);
            }
            var key = getFnKey(qname, arity);
            if(this.functions[key]) {
                throw new StaticError('XQST0034', '"' + qname.name + '": duplicate function declaration', pos);
            }
            this.functions[key] = {
                pos: pos,
                params: params
            };
            return this;
        }
        
    };
    s.root = parent ? parent.root : s;
    return s;
};

},
{"../tree_ops":11,"./errors":1,"./schema_built-in_types":3}],
5:[function(_dereq_,module,exports){
exports.Translator = function(rootStcx, ast){
    'use strict';

    var Errors = _dereq_('./errors');
    var StaticError = Errors.StaticError;
    var StaticWarning = Errors.StaticWarning;
    
    var TreeOps = _dereq_('../tree_ops').TreeOps;
    var StaticContext = _dereq_('./static_context').StaticContext;
    var Handlers = _dereq_('./handlers');
    
    var get = function(node, path){
        var result = [];
        if(path.length === 0){
            return node;
        }
        node.children.forEach(function(child){
            if(child.name === path[0] && path.length > 1) {
                result = get(child, path.slice(1));
            } else if(child.name === path[0]) {
                result.push(child);
            }
        });
        return result;
    };
    
    var markers = [];
    this.apply = function(fn) {
        try {
            fn();
        } catch(e) {
            if(e instanceof StaticError) {
                addStaticError(e);
            } else if(e instanceof StaticWarning) {
                addWarning(e.getCode(), e.getMessage(), e.getPos());
            } else {
                throw e;
            }
        }
    };

    var addStaticError = function(e){
        markers.push({
            pos: e.getPos(),
            type: 'error',
            level: 'error',
            message: '[' + e.getCode() + '] ' + e.getMessage()
        });
    };
    
    var addWarning = function(code, message, pos) {
        markers.push({
            pos: pos,
            type: 'warning',
            level: 'warning',
            message: '[' + code + '] ' + message
        });
    };
    
    this.getMarkers = function(){
        return markers;
    };

    var translator = this;

    rootStcx.pos = ast.pos;
    var sctx = rootStcx;
    var pushSctx = function(pos){
        sctx = new StaticContext(sctx, pos);
        sctx.parent.children.push(sctx);
    };
    
    var popSctx = function(pos){
        if (pos !== undefined) {
            sctx.pos.el = pos.el;
            sctx.pos.ec = pos.ec;
        }

        Object.keys(sctx.varRefs).forEach(function(key){
            if(!sctx.variables[key]) {
                sctx.parent.varRefs[key] = true;
            }
        });
        Object.keys(sctx.variables).forEach(function(key){
            if(!sctx.varRefs[key] && sctx.variables[key].type !== 'GroupingVariable' && sctx.variables[key].type !== 'CatchVar') {
                addWarning('W03', 'Unused variable "$' + sctx.variables[key].qname.name + '"', sctx.variables[key].pos);
            }
        });
        
        sctx = sctx.parent;
    };
    
    this.visitOnly = function(node, names) {
        node.children.forEach(function(child){
            if (names.indexOf(child.name) !== -1){
                translator.visit(child);
            }
        });
    };
    
    this.getFirstChild = function(node, name) {
        var result;
        node.children.forEach(function(child){
            if(child.name === name && result === undefined){
                result = child;
            }
        });
        return result;
    };
    
    this.ModuleDecl = function(node){
        this.visitChildren(node, Handlers.ModuleDecl(translator, rootStcx, node));
        return true;
    };
    
    this.Prolog = function(node){
        this.visitOnly(node, ['DefaultNamespaceDecl', 'Setter', 'NamespaceDecl', 'Import']);
        ast.index.forEach(function(node){
            if(node.name === 'VarDecl') {
                node.children.forEach(function(child){
                    if(child.name === 'VarName') {
                        translator.apply(function(){
                            var value = TreeOps.flatten(child);
                            var qname = rootStcx.resolveQName(value, child.pos);
                            rootStcx.addVariable(qname, node.name, child.pos);
                        });
                    }
                });
            } else if(node.name === 'FunctionDecl') {
                var qname, pos, params = [];
                node.children.forEach(function(child){
                    if(child.name === 'EQName') {
                        qname = child;
                        pos = child.pos;
                    } else if(child.name === 'ParamList'){
                        child.children.forEach(function(c){
                            if(c.name === 'Param') {
                                params.push(TreeOps.flatten(c));
                            }
                        });
                    }
                });
                translator.apply(function(){
                    qname = TreeOps.flatten(qname);
                    qname = rootStcx.resolveQName(qname, pos);
                    rootStcx.addFunction(qname, pos, params);
                });
            }
        });
        this.visitOnly(node, ['ContextItemDecl', 'AnnotatedDecl', 'OptionDecl']);
        return true;
    };
    
    this.ModuleImport = function (node) {
        this.visitChildren(node, Handlers.ModuleImport(translator, rootStcx, node));
        return true;
    };
    
    this.SchemaImport = function (node) {
        this.visitChildren(node, Handlers.SchemaImport(translator, rootStcx, node));
        return true;
    };
    
    this.DefaultNamespaceDecl = function(node){
        this.visitChildren(node, Handlers.DefaultNamespaceDecl(translator, rootStcx, node));
        return true;
    };
    
    this.NamespaceDecl = function (node) {
        this.visitChildren(node, Handlers.NamespaceDecl(translator, rootStcx, node));
        return true;
    };
    
    var annotations = {};
    this.AnnotatedDecl = function(node) {
        annotations = {};
        this.visitChildren(node, Handlers.NamespaceDecl(translator, rootStcx, node));
        return true;
    };
    
    this.CompatibilityAnnotation = function(){
        annotations['http://www.w3.org/2012/xquery#updating'] = [];
        return true;
    };
    
    this.Annotation = function(node){
        this.visitChildren(node, {
            EQName: function(eqname){
                var value = TreeOps.flatten(eqname);
                translator.apply(function(){
                    var qname = sctx.resolveQName(value, eqname.pos);
                    annotations[qname.uri + '#' + qname.name] = [];
                });
            }
        });
        return true;
    };
    
    this.VarDecl = function(node){
        try {
            var varname = translator.getFirstChild(node, 'VarName');
            var value = TreeOps.flatten(varname);
            var qname = sctx.resolveQName(value, varname.pos);
            var variable = rootStcx.getVariable(qname);
            if(variable) {
                variable.annotations = annotations;
            }
        } catch(e) {
        }
        this.visitOnly(node, ['ExprSingle', 'VarValue', 'VarDefaultValue']);
        return true;
    };
    
    this.FunctionDecl = function(node) {
        var isUpdating = annotations['http://www.w3.org/2012/xquery#updating'] !== undefined;
        var typeDecl = get(node, ['ReturnType'])[0];
        var name = get(node, ['EQName'])[0];
        if(!typeDecl && !isUpdating){
            addWarning('W05', 'Untyped return value', name.pos);
        }
        var isExternal = false;
        node.children.forEach(function(child){
            if(child.name === 'TOKEN' && child.value === 'external') {
                isExternal = true;
                return false;
            }
        });
        if(!isExternal) {
            pushSctx(node.pos);
            this.visitChildren(node);
            popSctx();
        }
        return true;
    };
    
    this.VarRef = function(node) {
        this.visitChildren(node, Handlers.VarRefHandler(translator, sctx, node));
        return true;
    };
    
    this.Param = function(node){
        var typeDecl = get(node, ['TypeDeclaration'])[0];
        if(!typeDecl){
            addWarning('W05', 'Untyped function parameter', node.pos);
        }
        this.visitChildren(node, Handlers.VarHandler(translator, sctx, node));
        return true;
    };
    
    this.InlineFunctionExpr	= function(node) {
        pushSctx(node.pos);
        this.visitChildren(node);
        popSctx();
        return true;
    };
    var statementCount = [];
    var handleStatements = function(node) {
        pushSctx(node.pos);
        statementCount.push(0);
        translator.visitChildren(node);
        for (var i = 1; i <= statementCount[statementCount.length - 1]; i++) {
            popSctx(node.pos);
        }
        statementCount.pop();
        popSctx();
    };

    this.StatementsAndOptionalExpr = function (node) {
        handleStatements(node);
        return true;
    };

    this.StatementsAndExpr = function (node) {
        handleStatements(node);
        return true;
    };

    this.BlockStatement = function (node) {
        handleStatements(node);
        return true;
    };
    
    this.VarDeclStatement = function(node){
        pushSctx(node.pos);
        statementCount[statementCount.length - 1]++;
        this.visitChildren(node, Handlers.VarHandler(translator, sctx, node));
    };
    var clauses = [];
    this.FLWORExpr = this.FLWORStatement = function (node) {
        pushSctx(node.pos);
        clauses.push(0);
        this.visitChildren(node);
        for(var i=1; i <= clauses[clauses.length - 1]; i++) {
            popSctx(node.pos);
        }
        clauses.pop();
        popSctx();
        return true;
    };
    
    this.ForBinding = function (node) {
        this.visitOnly(node, ['ExprSingle', 'VarValue', 'VarDefaultValue']);
        pushSctx(node.pos);
        clauses[clauses.length - 1]++;
        this.visitChildren(node, Handlers.VarHandler(translator, sctx, node));
        return true;
    };
    
    this.LetBinding = function(node){
        this.visitOnly(node, ['ExprSingle', 'VarValue', 'VarDefaultValue']);
        pushSctx(node.pos);
        clauses[clauses.length - 1]++;
        this.visitChildren(node, Handlers.VarHandler(translator, sctx, node));
        return true;
    };

    this.GroupingSpec = function(node){
        var isVarDecl = false;
        node.children.forEach(function(child){
            if(child.value === ':=') {
                isVarDecl = true;
                return false;
            }
        });
        if(isVarDecl) {
            var groupingVariable = node.children[0];
            this.visitOnly(node, ['ExprSingle', 'VarValue', 'VarDefaultValue']);
            pushSctx(node.pos);
            clauses[clauses.length - 1]++;
            this.visitChildren(groupingVariable, Handlers.VarHandler(translator, sctx, groupingVariable));
            return true;
        } else {
            
        }
    };
    
    this.TumblingWindowClause = function (node) {
        this.visitOnly(node, ['ExprSingle']);
        pushSctx(node.pos);
        clauses[clauses.length - 1]++;
        this.visitChildren(node, Handlers.VarHandler(translator, sctx, node));
        this.visitOnly(node, ['WindowStartCondition', 'WindowEndCondition']);
        return true;
    };

    this.WindowVars = function (node) {
        pushSctx(node.pos);
        clauses[clauses.length - 1]++;
        this.visitChildren(node, Handlers.VarHandler(translator, sctx, node));
        return true;
    };

    this.SlidingWindowClause = function (node) {
        this.visitOnly(node, ['ExprSingle', 'VarValue', 'VarDefaultValue']);
        pushSctx(node.pos);
        clauses[clauses.length - 1]++;
        this.visitChildren(node, Handlers.VarHandler(translator, sctx, node));
        this.visitOnly(node, ['WindowStartCondition', 'WindowEndCondition']);
        return true;
    };

    this.PositionalVar = function (node) {
        this.visitChildren(node, Handlers.VarHandler(translator, sctx, node));
        return true;
    };

    this.PositionalVar = function (node) {
        this.visitChildren(node, Handlers.VarHandler(translator, sctx, node));
        return true;
    };

    this.CurrentItem = function (node) {
        this.visitChildren(node, Handlers.VarHandler(translator, sctx, node));
        return true;
    };

    this.PreviousItem = function (node) {
        this.visitChildren(node, Handlers.VarHandler(translator, sctx, node));
        return true;
    };

    this.NextItem = function (node) {
        this.visitChildren(node, Handlers.VarHandler(translator, sctx, node));
        return true;
    };

    this.CountClause = function (node) {
        pushSctx(node.pos);
        clauses[clauses.length - 1]++;
        this.visitChildren(node, Handlers.VarHandler(translator, sctx, node));
        return true;
    };
    
    this.CaseClause = function(node) {
        pushSctx(node.pos);
        this.visitChildren(node, Handlers.VarHandler(translator, sctx, node));
        this.visitOnly(node, ['ExprSingle']);
        popSctx();
        return true;
    };
    
    this.TransformExpr = function (node) {
        pushSctx(node.pos);
        this.visitChildren(node);
        popSctx();
        return true;
    };
    
    this.TransformSpec = function(node) {
        this.visitOnly(node, ['ExprSingle', 'VarValue', 'VarDefaultValue']);
        this.visitChildren(node, Handlers.VarHandler(translator, sctx, node));
        return true;
    };
    var quantifiedDecls = [];
    this.QuantifiedExpr = function (node) {
        pushSctx(node.pos);
        quantifiedDecls.push(0);
        this.visitChildren(node);
        for(var i=1; i <= quantifiedDecls[clauses.length - 1]; i++) {
            popSctx(node.pos);
        }
        quantifiedDecls.pop();
        popSctx();
        return true;
    };
    
    this.QuantifiedVarDecl = function(node) {
        this.visitOnly(node, ['ExprSingle']);
        pushSctx(node.pos);
        quantifiedDecls[quantifiedDecls.length - 1]++;
        this.visitChildren(node, Handlers.VarHandler(translator, sctx, node));
        return true;
    };
    
    this.FunctionCall = function(node){
        this.visitOnly(node, ['ArgumentList']);
        var name = translator.getFirstChild(node, 'EQName');
        var eqname = TreeOps.flatten(name);
        var arity = get(node, ['ArgumentList', 'Argument']).length;
        translator.apply(function(){
            var qname = sctx.resolveQName(eqname, node.pos);
            try {
                if(qname.uri !== '') {
                    sctx.root.namespaces[qname.uri].used = true;
                }
            } catch(e){
            }
            sctx.addFunctionCall(qname, arity, name.pos);
        });
        return true;
    };
    
    this.TryClause = function(node){
        pushSctx(node.pos);
        this.visitChildren(node);
        popSctx();
        return true;
    };
    
    this.CatchClause = function(node){
        pushSctx(node.pos);
        var prefix = 'err';
        var uri = 'http://www.w3.org/2005/xqt-errors';
        var emptyPos = { sl: 0, sc: 0, el: 0, ec: 0 };
        sctx.addVariable({ prefix: prefix, uri: uri, name: 'code' }, 'CatchVar', emptyPos);
        sctx.addVariable({ prefix: prefix, uri: uri, name: 'description' }, 'CatchVar', emptyPos);
        sctx.addVariable({ prefix: prefix, uri: uri, name: 'value' }, 'CatchVar', emptyPos);
        sctx.addVariable({ prefix: prefix, uri: uri, name: 'module' }, 'CatchVar', emptyPos);
        sctx.addVariable({ prefix: prefix, uri: uri, name: 'line-number' }, 'CatchVar', emptyPos);
        sctx.addVariable({ prefix: prefix, uri: uri, name: 'column-number' }, 'CatchVar', emptyPos);
        sctx.addVariable({ prefix: prefix, uri: uri, name: 'additional' }, 'CatchVar', emptyPos);
        this.visitChildren(node);
        popSctx();
        return true;
    };

    this.Pragma = function(node){
        var qname = TreeOps.flatten(get(node, ['EQName'])[0]);
        qname = rootStcx.resolveQName(qname, node);
        var value = TreeOps.flatten(get(node, ['PragmaContents'])[0]);
        if (qname.name === 'xqlint' && qname.uri === 'http://xqlint.io') {
            pushSctx(node.pos);
            var commands = value.match(/[a-zA-Z]+\(([^)]+)\)/g);
            commands.forEach(function (command) {
                var name = command.substring(0, command.indexOf('('));
                var args = command.substring(0, command.length - 1).substring(command.indexOf('(') + 1).split(',').map(function (val) {
                    return val.trim();
                });
                if (name === 'varrefs') {
                    args.forEach(function (arg) {
                        var qname = sctx.resolveQName(arg.substring(1), node.pos);
                        if (qname.uri !== '') {
                            sctx.root.namespaces[qname.uri].used = true;
                        }
                        sctx.addVarRef(qname, node.pos);
                    });
                }
            });
            this.visitChildren(node);
            popSctx();
            return true;
        }
    };

    this.visit = function (node) {
        var name = node.name;
        var skip = false;

        if (typeof this[name] === 'function') {
            skip = this[name](node) === true;
        }

        if (!skip) {
            this.visitChildren(node);
        }
    };

    this.visitChildren = function (node, handler) {
        for (var i = 0; i < node.children.length; i++) {
            var child = node.children[i];
            if (handler !== undefined && typeof handler[child.name] === 'function') {
                handler[child.name](child);
            } else {
                this.visit(child);
            }
        }
    };

    this.visit(ast);
    Object.keys(rootStcx.variables).forEach(function(key){
        if(!rootStcx.varRefs[key] && (rootStcx.variables[key].annotations['http://www.w3.org/2005/xpath-functions#private'] || rootStcx.moduleNamespace === '') && rootStcx.variables[key].pos) {
            addWarning('W03', 'Unused variable "' + rootStcx.variables[key].qname.name + '"', rootStcx.variables[key].pos);
        }
    });
    Object.keys(rootStcx.namespaces).forEach(function(uri){
        var namespace = rootStcx.namespaces[uri];
        if(namespace.used === undefined && !namespace.override && namespace.type === 'module') {
            addWarning('W04', 'Unused module "' + uri + '"', namespace.pos);
        }
    });
};

},
{"../tree_ops":11,"./errors":1,"./handlers":2,"./static_context":4}],
6:[function(_dereq_,module,exports){
'use strict';

var TreeOps = _dereq_('../tree_ops').TreeOps;

var ID_REGEX = /[a-zA-Z_0-9\$]/;

function retrievePrecedingIdentifier(text, pos, regex) {
    regex = regex || ID_REGEX;
    var buf = [];
    for (var i = pos-1; i >= 0; i--) {
        if (regex.test(text[i])) {
            buf.push(text[i]);
        } else {
            break;
        }
    }
    return buf.reverse().join('');
}

function prefixBinarySearch(items, prefix) {
    var startIndex = 0;
    var stopIndex = items.length - 1;
    var middle = Math.floor((stopIndex + startIndex) / 2);
    
    while (stopIndex > startIndex && middle >= 0 && items[middle].indexOf(prefix) !== 0) {
        if (prefix < items[middle]) {
            stopIndex = middle - 1;
        } else if (prefix > items[middle]) {
            startIndex = middle + 1;
        }
        middle = Math.floor((stopIndex + startIndex) / 2);
    }
    while (middle > 0 && items[middle-1].indexOf(prefix) === 0) {
        middle--;
    }
    return middle >= 0 ? middle : 0; // ensure we're not returning a negative index
}

var uriRegex = /[a-zA-Z_0-9\/\.:\-#]/;
var char = '-._A-Za-z0-9:\u00B7\u00C0-\u00D6\u00D8-\u00F6\u00F8-\u02ff\u0300-\u037D\u037F-\u1FFF\u200C\u200D\u203f\u2040\u2070-\u218F\u2C00-\u2FEF\u3001-\uD7FF\uF900-\uFDCF\uFDF0-\uFFFD';
var nameChar = '[' + char + ']';
var varChar = '[' + char + '\\$]';
var nameCharRegExp = new RegExp(nameChar);
var varCharRegExp = new RegExp(varChar);

var varDeclLabels = {
    'LetBinding': 'Let binding',
    'Param': 'Function parameter',
    'QuantifiedExpr': 'Quantified expression binding',
    'VarDeclStatement': 'Local variable',
    'ForBinding': 'For binding',
    'TumblingWindowClause': 'Tumbling window binding',
    'WindowVars': 'Window variable',
    'SlidingWindowClause': 'Sliding window binding',
    'PositionalVar': 'Positional variable',
    'CurrentItem': 'Current item',
    'PreviousItem': 'Previous item',
    'NextItem': 'Next item',
    'CountClause': 'Count binding',
    'GroupingVariable': 'Grouping variable',
    'VarDecl': 'Module variable'
};

var findCompletions = function(prefix, allIdentifiers) {
    allIdentifiers.sort();
    var startIdx = prefixBinarySearch(allIdentifiers, prefix);
    var matches = [];
    for (var i = startIdx; i < allIdentifiers.length && allIdentifiers[i].indexOf(prefix) === 0; i++) {
        matches.push(allIdentifiers[i]);
    }
    return matches;
};


var completePrefix = function(identifier, pos, sctx){
    var idx = identifier.indexOf(':');
    if(idx === -1) {
        var prefixes = [];
        var namespaces = sctx.getNamespaces();
        Object.keys(namespaces).forEach(function(key){
            if(namespaces[key].type === 'module' || key === 'http://www.w3.org/2005/xquery-local-functions') {
                prefixes.push(namespaces[key].prefix);
            }
        });
        var matches = findCompletions(identifier, prefixes);
        var match = function(name) {
            return {
                name: name + ':',
                value: name + ':',
                meta: 'prefix'
            };
        };
        return matches.map(match);
    } else {
        return [];
    }
};

var completeFunction = function(identifier, pos, sctx){
    var names = [];
    var snippets = {};
    var functions = sctx.getFunctions();
    var uri = '';
    var prefix = '';
    var name = identifier;
    var idx = identifier.indexOf(':');
    var defaultNamespace = false;
    if(idx !== -1){
        prefix = identifier.substring(0, idx);
        name = identifier.substring(idx + 1);
        var ns = sctx.getNamespaceByPrefix(prefix);
        if(ns){
            uri = sctx.getNamespaceByPrefix(prefix).uri;
        }
    } else {
        defaultNamespace = true;
        uri = sctx.root.defaultFunctionNamespace;
    }
    Object.keys(functions).forEach(function(key){
        var fn = functions[key];
        var ns = key.substring(0, key.indexOf('#'));
        var name = key.substring(key.indexOf('#') + 1);
        name = name.substring(0, name.indexOf('#'));
        if(ns !== uri) {
            return;
        }
        if(!defaultNamespace){
            name = sctx.getNamespaces()[ns].prefix + ':' + name;
        }
        name += '(';
        var snippet = name;
        snippet += fn.params.map(function(param, index){
            return '${' + (index + 1) + ':\\' + param.split(' ')[0] + '}';
        }).join(', ');
        name += fn.params.join(', ');
        name += ')';
        snippet += ')';
        names.push(name);
        snippets[name] = snippet;
    });
    var matches = findCompletions(identifier, names);
    var match = function(name) {
        return {
            name: name,
            value: name,
            meta: 'function',
            priority: 4,
            identifierRegex: nameCharRegExp,
            snippet: snippets[name]
        };
    };
    return matches.map(match);
};

var completeVariable = function(identifier, pos, sctx){
    var uri = '';
    var prefix = '';
    var idx = identifier.indexOf(':');
    if(idx !== -1){
        prefix = identifier.substring(0, idx);
        uri = sctx.getNamespaceByPrefix(prefix).uri;
    }
    var decls = sctx.getVariables();
    var names = [];
    var types = {};
    Object.keys(decls).forEach(function(key){
        var i = key.indexOf('#');
        var ns = key.substring(0, i);
        var name = key.substring(i+1);
        if(ns !== ''){
            names.push(sctx.getPrefixByNamespace(ns) + ':' + name);
            types[sctx.getPrefixByNamespace(ns) + ':' + name] = decls[key].type;
        } else {
            names.push(name);
            types[name] = decls[key].type;
        }
    });
    
    var matches = findCompletions(identifier, names);
    var match = function(name) {
        return {
            name: '$' + name,
            value: '$' + name,
            meta: varDeclLabels[types[name]],
            priority: 4,
            identifierRegex: varCharRegExp
        };
    };
    return matches.map(match);
};

var completeExpr = function(line, pos, sctx){
    var identifier = retrievePrecedingIdentifier(line, pos.col, nameCharRegExp);
    var before = line.substring(0, pos.col - (identifier.length === 0 ? 0 : identifier.length));
    var isVar = before[before.length - 1] === '$';
    if(isVar) {
        return completeVariable(identifier, pos, sctx);
    } else if(identifier !== '') {
        return completeFunction(identifier, pos, sctx).concat(completePrefix(identifier, pos, sctx));
    } else {
        return completeVariable(identifier, pos, sctx).concat(completeFunction(identifier, pos, sctx)).concat(completePrefix(identifier, pos, sctx));
    }
};

var completeModuleUri = function(line, pos, sctx){
    var identifier = retrievePrecedingIdentifier(line, pos.col, uriRegex);
    var matches = findCompletions(identifier, sctx.getAvailableModuleNamespaces());
    var match = function(uri) {
        return {
            name: uri,
            value: uri,
            meta: 'module',
            priority: 4,
            identifierRegex: uriRegex
        };
    };
    return matches.map(match);
};

exports.complete = function(source, ast, rootSctx, pos){
    var line = source.split('\n')[pos.line];
    var node = TreeOps.findNode(ast, pos);
    var sctx = TreeOps.findNode(rootSctx, pos);
    sctx = sctx ? sctx : rootSctx;
    if(node && node.name === 'URILiteral' && node.getParent && node.getParent.name === 'ModuleImport'){
        return completeModuleUri(line, pos, sctx);
    } else {
        return completeExpr(line, pos, sctx);
    }
};

},
{"../tree_ops":11}],
7:[function(_dereq_,module,exports){
exports.StyleChecker = function (ast, source) {
    'use strict';

    var tab = '    ';
    var markers = [];
    
    this.getMarkers = function(){
        return markers;
    };

    this.WS = function(node) {
        var lines = node.value.split('\n');
        lines.forEach(function(line, index){
            var isFirst = index === 0;
            var isLast  = index === (lines.length - 1);

            if(/\r$/.test(line)) {
                markers.push({
                    pos: {
                        sl: node.pos.sl + index,
                        el: node.pos.sl + index,
                        sc: line.length - 1,
                        ec: line.length
                    },
                    type: 'warning',
                    level: 'warning',
                    message: '[SW01] Detected CRLF'
                });
            }
            
            var match = line.match(/\t+/);
            if(match !== null){
                markers.push({
                    pos: {
                        sl: node.pos.sl + index,
                        el: node.pos.sl + index,
                        sc: match.index,
                        ec: match.index + match[0].length
                    },
                    type: 'warning',
                    level: 'warning',
                    message: '[SW02] Tabs detected'
                });
            }

            if((!isFirst) && isLast){
                match = line.match(/^\ +/);
                if(match !== null) {
                    var mod = match[0].length % tab.length;
                    if(mod !== 0) {
                        markers.push({
                            pos: {
                                sl: node.pos.sl + index,
                                el: node.pos.sl + index,
                                sc: match.index,
                                ec: match.index + match[0].length
                            },
                            type: 'warning',
                            level: 'warning',
                            message: '[SW03] Unexcepted indentation of ' + match[0].length
                        });
                    }
                }
            }
        });
        return true;
    };
    
    this.visit = function (node, index) {
        var name = node.name;
        var skip = false;

        if (typeof this[name] === 'function') {
            skip = this[name](node, index) === true;
        }

        if (!skip) {
            this.visitChildren(node);
        }
    };

    this.visitChildren = function (node, handler) {
        for (var i = 0; i < node.children.length; i++) {
            var child = node.children[i];
            if (handler !== undefined && typeof handler[child.name] === 'function') {
                handler[child.name](child);
            } else {
                this.visit(child);
            }
        }
    };

    source.split('\n').forEach(function(line, index){
        var match = line.match(/\ +$/);
        if(match){
            markers.push({
                pos: {
                    sl: index,
                    el: index,
                    sc: match.index,
                    ec: match.index + match[0].length
                },
                type: 'warning',
                level: 'warning',
                message: '[SW04] Trailing whitespace'
            });
        }
    });
    this.visit(ast);
};
},
{}],
8:[function(_dereq_,module,exports){
exports.JSONParseTreeHandler = function (code) {
	'use strict';
    var toBeIndex = ['VarDecl', 'FunctionDecl'];
    var list = [
        'OrExpr', 'AndExpr', 'ComparisonExpr', 'StringConcatExpr', 'RangeExpr',
        'UnionExpr', 'IntersectExceptExpr', 'InstanceofExpr', 'TreatExpr', 'CastableExpr', 'CastExpr', 'UnaryExpr', 'ValueExpr',
        'FTContainsExpr', 'SimpleMapExpr', 'PathExpr', 'RelativePathExpr', 'PostfixExpr', 'StepExpr'
    ];

    var ast = null;
    var ptr = null;
    var remains = code;
    var cursor = 0;
    var lineCursor = 0;
    var line = 0;

    function createNode(name) {
        return {
            name: name,
            children: [],
            getParent: null,
            pos: {
                sl: 0,
                sc: 0,
                el: 0,
                ec: 0
            }
        };
    }

    function pushNode(name) { //begin
        var node = createNode(name);
        if (ast === null) {
            ast = node;
            ast.index = [];
            ptr = node;
        } else {
            node.getParent = ptr;
            ptr.children.push(node);
            ptr = ptr.children[ptr.children.length - 1];
        }
    }

    function popNode() {

        if (ptr.children.length > 0) {
            var s = ptr.children[0];
            var e = null;
            for (var i = ptr.children.length - 1; i >= 0; i--) {
                e = ptr.children[i];
                if (e.pos.el !== 0 || e.pos.ec !== 0) {
                    break;
                }
            }
            ptr.pos.sl = s.pos.sl;
            ptr.pos.sc = s.pos.sc;
            ptr.pos.el = e.pos.el;
            ptr.pos.ec = e.pos.ec;
        }
        if (ptr.name === 'FunctionName') {
            ptr.name = 'EQName';
        }
        if (ptr.name === 'EQName' && ptr.value === undefined) {
            ptr.value = ptr.children[0].value;
            ptr.children.pop();
        }
    
        if(toBeIndex.indexOf(ptr.name) !== -1) {
            ast.index.push(ptr);
        }
    
        if (ptr.getParent !== null) {
            ptr = ptr.getParent;
        } else {
        }
        if (ptr.children.length > 0) {
            var lastChild = ptr.children[ptr.children.length - 1];
            if (lastChild.children.length === 1 && list.indexOf(lastChild.name) !== -1) {
                ptr.children[ptr.children.length - 1] = lastChild.children[0];
            }
        }
    }

    this.closeParseTree = function () {
        while (ptr.getParent !== null) {
            popNode();
        }
        popNode();
    };

    this.peek = function () {
        return ptr;
    };

    this.getParseTree = function () {
        return ast;
    };

    this.reset = function () {}; //input

    this.startNonterminal = function (name, begin) {
        pushNode(name, begin);
    };

    this.endNonterminal = function () {//name, end
        popNode();
    };

    this.terminal = function (name, begin, end) {
        name = (name.substring(0, 1) === '\'' && name.substring(name.length - 1) === '\'') ? 'TOKEN' : name;
        pushNode(name, begin);
        setValue(ptr, begin, end);
        popNode();
    };

    this.whitespace = function (begin, end) {
        var name = 'WS';
        pushNode(name, begin);
        setValue(ptr, begin, end);
        popNode();
    };

    function setValue(node, begin, end) {

        var e = end - cursor;
        ptr.value = remains.substring(0, e);
        remains = remains.substring(e);
        cursor = end;

        var sl = line;
        var sc = lineCursor;
        var el = sl + ptr.value.split('\n').length - 1;
        var lastIdx = ptr.value.lastIndexOf('\n');
        var ec = lastIdx === -1 ? sc + ptr.value.length : ptr.value.substring(lastIdx + 1).length;

        line = el;
        lineCursor = ec;

        ptr.pos.sl = sl;
        ptr.pos.sc = sc;
        ptr.pos.el = el;
        ptr.pos.ec = ec;
    }
};
},
{}],
9:[function(_dereq_,module,exports){
                                                            var JSONiqParser = exports.JSONiqParser = function JSONiqParser(string, parsingEventHandler)
                                                            {
                                                              init(string, parsingEventHandler);
  var self = this;

  this.ParseException = function(b, e, s, o, x)
  {
    var
      begin = b,
      end = e,
      state = s,
      offending = o,
      expected = x;

    this.getBegin = function() {return begin;};
    this.getEnd = function() {return end;};
    this.getState = function() {return state;};
    this.getExpected = function() {return expected;};
    this.getOffending = function() {return offending;};

    this.getMessage = function()
    {
      return offending < 0 ? "lexical analysis failed" : "syntax error";
    };
  };

  function init(string, parsingEventHandler)
  {
    eventHandler = parsingEventHandler;
    input = string;
    size = string.length;
    reset(0, 0, 0);
  }

  this.getInput = function()
  {
    return input;
  };

  function reset(l, b, e)
  {
            b0 = b; e0 = b;
    l1 = l; b1 = b; e1 = e;
    l2 = 0;
    end = e;
    ex = -1;
    memo = {};
    eventHandler.reset(input);
  }

  this.getOffendingToken = function(e)
  {
    var o = e.getOffending();
    return o >= 0 ? JSONiqParser.TOKEN[o] : null;
  };

  this.getExpectedTokenSet = function(e)
  {
    var expected;
    if (e.getExpected() < 0)
    {
      expected = JSONiqParser.getTokenSet(- e.getState());
    }
    else
    {
      expected = [JSONiqParser.TOKEN[e.getExpected()]];
    }
    return expected;
  };

  this.getErrorMessage = function(e)
  {
    var tokenSet = this.getExpectedTokenSet(e);
    var found = this.getOffendingToken(e);
    var prefix = input.substring(0, e.getBegin());
    var lines = prefix.split("\n");
    var line = lines.length;
    var column = lines[line - 1].length + 1;
    var size = e.getEnd() - e.getBegin();
    return e.getMessage()
         + (found == null ? "" : ", found " + found)
         + "\nwhile expecting "
         + (tokenSet.length == 1 ? tokenSet[0] : ("[" + tokenSet.join(", ") + "]"))
         + "\n"
         + (size == 0 || found != null ? "" : "after successfully scanning " + size + " characters beginning ")
         + "at line " + line + ", column " + column + ":\n..."
         + input.substring(e.getBegin(), Math.min(input.length, e.getBegin() + 64))
         + "...";
  };

  this.parse_XQuery = function()
  {
    eventHandler.startNonterminal("XQuery", e0);
    lookahead1W(278);               // EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral | StringLiteral |
    whitespace();
    parse_Module();
    shift(25);                      // EOF
    eventHandler.endNonterminal("XQuery", e0);
  };

  function parse_Module()
  {
    eventHandler.startNonterminal("Module", e0);
    switch (l1)
    {
    case 170:                       // 'jsoniq'
      lookahead2W(168);             // S^WS | '#' | '(' | '(:' | 'encoding' | 'version'
      break;
    default:
      lk = l1;
    }
    if (lk == 64682                 // 'jsoniq' 'encoding'
     || lk == 137898)               // 'jsoniq' 'version'
    {
      parse_VersionDecl();
    }
    lookahead1W(278);               // EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral | StringLiteral |
    switch (l1)
    {
    case 185:                       // 'module'
      lookahead2W(146);             // S^WS | '#' | '(' | '(:' | 'namespace'
      break;
    default:
      lk = l1;
    }
    switch (lk)
    {
    case 95929:                     // 'module' 'namespace'
      whitespace();
      parse_LibraryModule();
      break;
    default:
      whitespace();
      parse_MainModule();
    }
    eventHandler.endNonterminal("Module", e0);
  }

  function parse_VersionDecl()
  {
    eventHandler.startNonterminal("VersionDecl", e0);
    shift(170);                     // 'jsoniq'
    lookahead1W(120);               // S^WS | '(:' | 'encoding' | 'version'
    switch (l1)
    {
    case 126:                       // 'encoding'
      shift(126);                   // 'encoding'
      lookahead1W(17);              // StringLiteral | S^WS | '(:'
      shift(11);                    // StringLiteral
      break;
    default:
      shift(269);                   // 'version'
      lookahead1W(17);              // StringLiteral | S^WS | '(:'
      shift(11);                    // StringLiteral
      lookahead1W(113);             // S^WS | '(:' | ';' | 'encoding'
      if (l1 == 126)                // 'encoding'
      {
        shift(126);                 // 'encoding'
        lookahead1W(17);            // StringLiteral | S^WS | '(:'
        shift(11);                  // StringLiteral
      }
    }
    lookahead1W(29);                // S^WS | '(:' | ';'
    whitespace();
    parse_Separator();
    eventHandler.endNonterminal("VersionDecl", e0);
  }

  function parse_LibraryModule()
  {
    eventHandler.startNonterminal("LibraryModule", e0);
    parse_ModuleDecl();
    lookahead1W(142);               // S^WS | EOF | '(:' | 'declare' | 'import'
    whitespace();
    parse_Prolog();
    eventHandler.endNonterminal("LibraryModule", e0);
  }

  function parse_ModuleDecl()
  {
    eventHandler.startNonterminal("ModuleDecl", e0);
    shift(185);                     // 'module'
    lookahead1W(64);                // S^WS | '(:' | 'namespace'
    shift(187);                     // 'namespace'
    lookahead1W(240);               // NCName^Token | S^WS | '(:' | 'after' | 'allowing' | 'ancestor' |
    whitespace();
    parse_NCName();
    lookahead1W(30);                // S^WS | '(:' | '='
    shift(61);                      // '='
    lookahead1W(15);                // URILiteral | S^WS | '(:'
    shift(7);                       // URILiteral
    lookahead1W(29);                // S^WS | '(:' | ';'
    whitespace();
    parse_Separator();
    eventHandler.endNonterminal("ModuleDecl", e0);
  }

  function parse_Prolog()
  {
    eventHandler.startNonterminal("Prolog", e0);
    for (;;)
    {
      lookahead1W(278);             // EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral | StringLiteral |
      switch (l1)
      {
      case 109:                     // 'declare'
        lookahead2W(207);           // S^WS | '#' | '%' | '(' | '(:' | 'base-uri' | 'boundary-space' | 'collection' |
        break;
      case 155:                     // 'import'
        lookahead2W(169);           // S^WS | '#' | '(' | '(:' | 'module' | 'schema'
        break;
      default:
        lk = l1;
      }
      if (lk != 43117               // 'declare' 'base-uri'
       && lk != 44141               // 'declare' 'boundary-space'
       && lk != 50797               // 'declare' 'construction'
       && lk != 53869               // 'declare' 'copy-namespaces'
       && lk != 54893               // 'declare' 'decimal-format'
       && lk != 56429               // 'declare' 'default'
       && lk != 73325               // 'declare' 'ft-option'
       && lk != 94875               // 'import' 'module'
       && lk != 95853               // 'declare' 'namespace'
       && lk != 106093              // 'declare' 'ordering'
       && lk != 115821              // 'declare' 'revalidation'
       && lk != 117403)             // 'import' 'schema'
      {
        break;
      }
      switch (l1)
      {
      case 109:                     // 'declare'
        lookahead2W(201);           // S^WS | '(:' | 'base-uri' | 'boundary-space' | 'construction' |
        break;
      default:
        lk = l1;
      }
      if (lk == 56429)              // 'declare' 'default'
      {
        lk = memoized(0, e0);
        if (lk == 0)
        {
          var b0A = b0; var e0A = e0; var l1A = l1;
          var b1A = b1; var e1A = e1; var l2A = l2;
          var b2A = b2; var e2A = e2;
          try
          {
            try_DefaultNamespaceDecl();
            lk = -1;
          }
          catch (p1A)
          {
            lk = -2;
          }
          b0 = b0A; e0 = e0A; l1 = l1A; if (l1 == 0) {end = e0A;} else {
          b1 = b1A; e1 = e1A; l2 = l2A; if (l2 == 0) {end = e1A;} else {
          b2 = b2A; e2 = e2A; end = e2A; }}
          memoize(0, e0, lk);
        }
      }
      switch (lk)
      {
      case -1:
        whitespace();
        parse_DefaultNamespaceDecl();
        break;
      case 95853:                   // 'declare' 'namespace'
        whitespace();
        parse_NamespaceDecl();
        break;
      case 155:                     // 'import'
        whitespace();
        parse_Import();
        break;
      case 73325:                   // 'declare' 'ft-option'
        whitespace();
        parse_FTOptionDecl();
        break;
      default:
        whitespace();
        parse_Setter();
      }
      lookahead1W(29);              // S^WS | '(:' | ';'
      whitespace();
      parse_Separator();
    }
    for (;;)
    {
      lookahead1W(278);             // EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral | StringLiteral |
      switch (l1)
      {
      case 109:                     // 'declare'
        lookahead2W(202);           // S^WS | '#' | '%' | '(' | '(:' | 'collection' | 'context' | 'function' | 'index' |
        break;
      default:
        lk = l1;
      }
      if (lk != 17005               // 'declare' '%'
       && lk != 49261               // 'declare' 'collection'
       && lk != 52333               // 'declare' 'context'
       && lk != 75373               // 'declare' 'function'
       && lk != 80493               // 'declare' 'index'
       && lk != 83565               // 'declare' 'integrity'
       && lk != 104045              // 'declare' 'option'
       && lk != 134765              // 'declare' 'updating'
       && lk != 137325)             // 'declare' 'variable'
      {
        break;
      }
      switch (l1)
      {
      case 109:                     // 'declare'
        lookahead2W(197);           // S^WS | '%' | '(:' | 'collection' | 'context' | 'function' | 'index' |
        break;
      default:
        lk = l1;
      }
      switch (lk)
      {
      case 52333:                   // 'declare' 'context'
        whitespace();
        parse_ContextItemDecl();
        break;
      case 104045:                  // 'declare' 'option'
        whitespace();
        parse_OptionDecl();
        break;
      default:
        whitespace();
        parse_AnnotatedDecl();
      }
      lookahead1W(29);              // S^WS | '(:' | ';'
      whitespace();
      parse_Separator();
    }
    eventHandler.endNonterminal("Prolog", e0);
  }

  function parse_Separator()
  {
    eventHandler.startNonterminal("Separator", e0);
    shift(54);                      // ';'
    eventHandler.endNonterminal("Separator", e0);
  }

  function parse_Setter()
  {
    eventHandler.startNonterminal("Setter", e0);
    switch (l1)
    {
    case 109:                       // 'declare'
      lookahead2W(194);             // S^WS | '(:' | 'base-uri' | 'boundary-space' | 'construction' |
      break;
    default:
      lk = l1;
    }
    if (lk == 56429)                // 'declare' 'default'
    {
      lk = memoized(1, e0);
      if (lk == 0)
      {
        var b0A = b0; var e0A = e0; var l1A = l1;
        var b1A = b1; var e1A = e1; var l2A = l2;
        var b2A = b2; var e2A = e2;
        try
        {
          try_DefaultCollationDecl();
          lk = -2;
        }
        catch (p2A)
        {
          try
          {
            b0 = b0A; e0 = e0A; l1 = l1A; if (l1 == 0) {end = e0A;} else {
            b1 = b1A; e1 = e1A; l2 = l2A; if (l2 == 0) {end = e1A;} else {
            b2 = b2A; e2 = e2A; end = e2A; }}
            try_EmptyOrderDecl();
            lk = -6;
          }
          catch (p6A)
          {
            lk = -9;
          }
        }
        b0 = b0A; e0 = e0A; l1 = l1A; if (l1 == 0) {end = e0A;} else {
        b1 = b1A; e1 = e1A; l2 = l2A; if (l2 == 0) {end = e1A;} else {
        b2 = b2A; e2 = e2A; end = e2A; }}
        memoize(1, e0, lk);
      }
    }
    switch (lk)
    {
    case 44141:                     // 'declare' 'boundary-space'
      parse_BoundarySpaceDecl();
      break;
    case -2:
      parse_DefaultCollationDecl();
      break;
    case 43117:                     // 'declare' 'base-uri'
      parse_BaseURIDecl();
      break;
    case 50797:                     // 'declare' 'construction'
      parse_ConstructionDecl();
      break;
    case 106093:                    // 'declare' 'ordering'
      parse_OrderingModeDecl();
      break;
    case -6:
      parse_EmptyOrderDecl();
      break;
    case 115821:                    // 'declare' 'revalidation'
      parse_RevalidationDecl();
      break;
    case 53869:                     // 'declare' 'copy-namespaces'
      parse_CopyNamespacesDecl();
      break;
    default:
      parse_DecimalFormatDecl();
    }
    eventHandler.endNonterminal("Setter", e0);
  }

  function parse_BoundarySpaceDecl()
  {
    eventHandler.startNonterminal("BoundarySpaceDecl", e0);
    shift(109);                     // 'declare'
    lookahead1W(36);                // S^WS | '(:' | 'boundary-space'
    shift(86);                      // 'boundary-space'
    lookahead1W(137);               // S^WS | '(:' | 'preserve' | 'strip'
    switch (l1)
    {
    case 218:                       // 'preserve'
      shift(218);                   // 'preserve'
      break;
    default:
      shift(246);                   // 'strip'
    }
    eventHandler.endNonterminal("BoundarySpaceDecl", e0);
  }

  function parse_DefaultCollationDecl()
  {
    eventHandler.startNonterminal("DefaultCollationDecl", e0);
    shift(109);                     // 'declare'
    lookahead1W(49);                // S^WS | '(:' | 'default'
    shift(110);                     // 'default'
    lookahead1W(41);                // S^WS | '(:' | 'collation'
    shift(95);                      // 'collation'
    lookahead1W(15);                // URILiteral | S^WS | '(:'
    shift(7);                       // URILiteral
    eventHandler.endNonterminal("DefaultCollationDecl", e0);
  }

  function try_DefaultCollationDecl()
  {
    shiftT(109);                    // 'declare'
    lookahead1W(49);                // S^WS | '(:' | 'default'
    shiftT(110);                    // 'default'
    lookahead1W(41);                // S^WS | '(:' | 'collation'
    shiftT(95);                     // 'collation'
    lookahead1W(15);                // URILiteral | S^WS | '(:'
    shiftT(7);                      // URILiteral
  }

  function parse_BaseURIDecl()
  {
    eventHandler.startNonterminal("BaseURIDecl", e0);
    shift(109);                     // 'declare'
    lookahead1W(35);                // S^WS | '(:' | 'base-uri'
    shift(84);                      // 'base-uri'
    lookahead1W(15);                // URILiteral | S^WS | '(:'
    shift(7);                       // URILiteral
    eventHandler.endNonterminal("BaseURIDecl", e0);
  }

  function parse_ConstructionDecl()
  {
    eventHandler.startNonterminal("ConstructionDecl", e0);
    shift(109);                     // 'declare'
    lookahead1W(44);                // S^WS | '(:' | 'construction'
    shift(99);                      // 'construction'
    lookahead1W(137);               // S^WS | '(:' | 'preserve' | 'strip'
    switch (l1)
    {
    case 246:                       // 'strip'
      shift(246);                   // 'strip'
      break;
    default:
      shift(218);                   // 'preserve'
    }
    eventHandler.endNonterminal("ConstructionDecl", e0);
  }

  function parse_OrderingModeDecl()
  {
    eventHandler.startNonterminal("OrderingModeDecl", e0);
    shift(109);                     // 'declare'
    lookahead1W(71);                // S^WS | '(:' | 'ordering'
    shift(207);                     // 'ordering'
    lookahead1W(135);               // S^WS | '(:' | 'ordered' | 'unordered'
    switch (l1)
    {
    case 206:                       // 'ordered'
      shift(206);                   // 'ordered'
      break;
    default:
      shift(262);                   // 'unordered'
    }
    eventHandler.endNonterminal("OrderingModeDecl", e0);
  }

  function parse_EmptyOrderDecl()
  {
    eventHandler.startNonterminal("EmptyOrderDecl", e0);
    shift(109);                     // 'declare'
    lookahead1W(49);                // S^WS | '(:' | 'default'
    shift(110);                     // 'default'
    lookahead1W(70);                // S^WS | '(:' | 'order'
    shift(205);                     // 'order'
    lookahead1W(52);                // S^WS | '(:' | 'empty'
    shift(124);                     // 'empty'
    lookahead1W(125);               // S^WS | '(:' | 'greatest' | 'least'
    switch (l1)
    {
    case 149:                       // 'greatest'
      shift(149);                   // 'greatest'
      break;
    default:
      shift(176);                   // 'least'
    }
    eventHandler.endNonterminal("EmptyOrderDecl", e0);
  }

  function try_EmptyOrderDecl()
  {
    shiftT(109);                    // 'declare'
    lookahead1W(49);                // S^WS | '(:' | 'default'
    shiftT(110);                    // 'default'
    lookahead1W(70);                // S^WS | '(:' | 'order'
    shiftT(205);                    // 'order'
    lookahead1W(52);                // S^WS | '(:' | 'empty'
    shiftT(124);                    // 'empty'
    lookahead1W(125);               // S^WS | '(:' | 'greatest' | 'least'
    switch (l1)
    {
    case 149:                       // 'greatest'
      shiftT(149);                  // 'greatest'
      break;
    default:
      shiftT(176);                  // 'least'
    }
  }

  function parse_CopyNamespacesDecl()
  {
    eventHandler.startNonterminal("CopyNamespacesDecl", e0);
    shift(109);                     // 'declare'
    lookahead1W(47);                // S^WS | '(:' | 'copy-namespaces'
    shift(105);                     // 'copy-namespaces'
    lookahead1W(132);               // S^WS | '(:' | 'no-preserve' | 'preserve'
    whitespace();
    parse_PreserveMode();
    lookahead1W(25);                // S^WS | '(:' | ','
    shift(42);                      // ','
    lookahead1W(127);               // S^WS | '(:' | 'inherit' | 'no-inherit'
    whitespace();
    parse_InheritMode();
    eventHandler.endNonterminal("CopyNamespacesDecl", e0);
  }

  function parse_PreserveMode()
  {
    eventHandler.startNonterminal("PreserveMode", e0);
    switch (l1)
    {
    case 218:                       // 'preserve'
      shift(218);                   // 'preserve'
      break;
    default:
      shift(193);                   // 'no-preserve'
    }
    eventHandler.endNonterminal("PreserveMode", e0);
  }

  function parse_InheritMode()
  {
    eventHandler.startNonterminal("InheritMode", e0);
    switch (l1)
    {
    case 159:                       // 'inherit'
      shift(159);                   // 'inherit'
      break;
    default:
      shift(192);                   // 'no-inherit'
    }
    eventHandler.endNonterminal("InheritMode", e0);
  }

  function parse_DecimalFormatDecl()
  {
    eventHandler.startNonterminal("DecimalFormatDecl", e0);
    shift(109);                     // 'declare'
    lookahead1W(118);               // S^WS | '(:' | 'decimal-format' | 'default'
    switch (l1)
    {
    case 107:                       // 'decimal-format'
      shift(107);                   // 'decimal-format'
      lookahead1W(246);             // EQName^Token | S^WS | '(:' | 'after' | 'allowing' | 'ancestor' |
      whitespace();
      parse_EQName();
      break;
    default:
      shift(110);                   // 'default'
      lookahead1W(48);              // S^WS | '(:' | 'decimal-format'
      shift(107);                   // 'decimal-format'
    }
    for (;;)
    {
      lookahead1W(204);             // S^WS | '(:' | ';' | 'NaN' | 'decimal-separator' | 'digit' |
      if (l1 == 54)                 // ';'
      {
        break;
      }
      whitespace();
      parse_DFPropertyName();
      lookahead1W(30);              // S^WS | '(:' | '='
      shift(61);                    // '='
      lookahead1W(17);              // StringLiteral | S^WS | '(:'
      shift(11);                    // StringLiteral
    }
    eventHandler.endNonterminal("DecimalFormatDecl", e0);
  }

  function parse_DFPropertyName()
  {
    eventHandler.startNonterminal("DFPropertyName", e0);
    switch (l1)
    {
    case 108:                       // 'decimal-separator'
      shift(108);                   // 'decimal-separator'
      break;
    case 151:                       // 'grouping-separator'
      shift(151);                   // 'grouping-separator'
      break;
    case 158:                       // 'infinity'
      shift(158);                   // 'infinity'
      break;
    case 182:                       // 'minus-sign'
      shift(182);                   // 'minus-sign'
      break;
    case 68:                        // 'NaN'
      shift(68);                    // 'NaN'
      break;
    case 213:                       // 'percent'
      shift(213);                   // 'percent'
      break;
    case 212:                       // 'per-mille'
      shift(212);                   // 'per-mille'
      break;
    case 280:                       // 'zero-digit'
      shift(280);                   // 'zero-digit'
      break;
    case 117:                       // 'digit'
      shift(117);                   // 'digit'
      break;
    default:
      shift(211);                   // 'pattern-separator'
    }
    eventHandler.endNonterminal("DFPropertyName", e0);
  }

  function parse_Import()
  {
    eventHandler.startNonterminal("Import", e0);
    switch (l1)
    {
    case 155:                       // 'import'
      lookahead2W(130);             // S^WS | '(:' | 'module' | 'schema'
      break;
    default:
      lk = l1;
    }
    switch (lk)
    {
    case 117403:                    // 'import' 'schema'
      parse_SchemaImport();
      break;
    default:
      parse_ModuleImport();
    }
    eventHandler.endNonterminal("Import", e0);
  }

  function parse_SchemaImport()
  {
    eventHandler.startNonterminal("SchemaImport", e0);
    shift(155);                     // 'import'
    lookahead1W(76);                // S^WS | '(:' | 'schema'
    shift(229);                     // 'schema'
    lookahead1W(141);               // URILiteral | S^WS | '(:' | 'default' | 'namespace'
    if (l1 != 7)                    // URILiteral
    {
      whitespace();
      parse_SchemaPrefix();
    }
    lookahead1W(15);                // URILiteral | S^WS | '(:'
    shift(7);                       // URILiteral
    lookahead1W(112);               // S^WS | '(:' | ';' | 'at'
    if (l1 == 82)                   // 'at'
    {
      shift(82);                    // 'at'
      lookahead1W(15);              // URILiteral | S^WS | '(:'
      shift(7);                     // URILiteral
      for (;;)
      {
        lookahead1W(107);           // S^WS | '(:' | ',' | ';'
        if (l1 != 42)               // ','
        {
          break;
        }
        shift(42);                  // ','
        lookahead1W(15);            // URILiteral | S^WS | '(:'
        shift(7);                   // URILiteral
      }
    }
    eventHandler.endNonterminal("SchemaImport", e0);
  }

  function parse_SchemaPrefix()
  {
    eventHandler.startNonterminal("SchemaPrefix", e0);
    switch (l1)
    {
    case 187:                       // 'namespace'
      shift(187);                   // 'namespace'
      lookahead1W(240);             // NCName^Token | S^WS | '(:' | 'after' | 'allowing' | 'ancestor' |
      whitespace();
      parse_NCName();
      lookahead1W(30);              // S^WS | '(:' | '='
      shift(61);                    // '='
      break;
    default:
      shift(110);                   // 'default'
      lookahead1W(50);              // S^WS | '(:' | 'element'
      shift(122);                   // 'element'
      lookahead1W(64);              // S^WS | '(:' | 'namespace'
      shift(187);                   // 'namespace'
    }
    eventHandler.endNonterminal("SchemaPrefix", e0);
  }

  function parse_ModuleImport()
  {
    eventHandler.startNonterminal("ModuleImport", e0);
    shift(155);                     // 'import'
    lookahead1W(63);                // S^WS | '(:' | 'module'
    shift(185);                     // 'module'
    lookahead1W(93);                // URILiteral | S^WS | '(:' | 'namespace'
    if (l1 == 187)                  // 'namespace'
    {
      shift(187);                   // 'namespace'
      lookahead1W(240);             // NCName^Token | S^WS | '(:' | 'after' | 'allowing' | 'ancestor' |
      whitespace();
      parse_NCName();
      lookahead1W(30);              // S^WS | '(:' | '='
      shift(61);                    // '='
    }
    lookahead1W(15);                // URILiteral | S^WS | '(:'
    shift(7);                       // URILiteral
    lookahead1W(112);               // S^WS | '(:' | ';' | 'at'
    if (l1 == 82)                   // 'at'
    {
      shift(82);                    // 'at'
      lookahead1W(15);              // URILiteral | S^WS | '(:'
      shift(7);                     // URILiteral
      for (;;)
      {
        lookahead1W(107);           // S^WS | '(:' | ',' | ';'
        if (l1 != 42)               // ','
        {
          break;
        }
        shift(42);                  // ','
        lookahead1W(15);            // URILiteral | S^WS | '(:'
        shift(7);                   // URILiteral
      }
    }
    eventHandler.endNonterminal("ModuleImport", e0);
  }

  function parse_NamespaceDecl()
  {
    eventHandler.startNonterminal("NamespaceDecl", e0);
    shift(109);                     // 'declare'
    lookahead1W(64);                // S^WS | '(:' | 'namespace'
    shift(187);                     // 'namespace'
    lookahead1W(240);               // NCName^Token | S^WS | '(:' | 'after' | 'allowing' | 'ancestor' |
    whitespace();
    parse_NCName();
    lookahead1W(30);                // S^WS | '(:' | '='
    shift(61);                      // '='
    lookahead1W(15);                // URILiteral | S^WS | '(:'
    shift(7);                       // URILiteral
    eventHandler.endNonterminal("NamespaceDecl", e0);
  }

  function parse_DefaultNamespaceDecl()
  {
    eventHandler.startNonterminal("DefaultNamespaceDecl", e0);
    shift(109);                     // 'declare'
    lookahead1W(49);                // S^WS | '(:' | 'default'
    shift(110);                     // 'default'
    lookahead1W(119);               // S^WS | '(:' | 'element' | 'function'
    switch (l1)
    {
    case 122:                       // 'element'
      shift(122);                   // 'element'
      break;
    default:
      shift(147);                   // 'function'
    }
    lookahead1W(64);                // S^WS | '(:' | 'namespace'
    shift(187);                     // 'namespace'
    lookahead1W(15);                // URILiteral | S^WS | '(:'
    shift(7);                       // URILiteral
    eventHandler.endNonterminal("DefaultNamespaceDecl", e0);
  }

  function try_DefaultNamespaceDecl()
  {
    shiftT(109);                    // 'declare'
    lookahead1W(49);                // S^WS | '(:' | 'default'
    shiftT(110);                    // 'default'
    lookahead1W(119);               // S^WS | '(:' | 'element' | 'function'
    switch (l1)
    {
    case 122:                       // 'element'
      shiftT(122);                  // 'element'
      break;
    default:
      shiftT(147);                  // 'function'
    }
    lookahead1W(64);                // S^WS | '(:' | 'namespace'
    shiftT(187);                    // 'namespace'
    lookahead1W(15);                // URILiteral | S^WS | '(:'
    shiftT(7);                      // URILiteral
  }

  function parse_FTOptionDecl()
  {
    eventHandler.startNonterminal("FTOptionDecl", e0);
    shift(109);                     // 'declare'
    lookahead1W(55);                // S^WS | '(:' | 'ft-option'
    shift(143);                     // 'ft-option'
    lookahead1W(84);                // S^WS | '(:' | 'using'
    whitespace();
    parse_FTMatchOptions();
    eventHandler.endNonterminal("FTOptionDecl", e0);
  }

  function parse_AnnotatedDecl()
  {
    eventHandler.startNonterminal("AnnotatedDecl", e0);
    shift(109);                     // 'declare'
    for (;;)
    {
      lookahead1W(192);             // S^WS | '%' | '(:' | 'collection' | 'function' | 'index' | 'integrity' |
      if (l1 != 33                  // '%'
       && l1 != 263)                // 'updating'
      {
        break;
      }
      switch (l1)
      {
      case 263:                     // 'updating'
        whitespace();
        parse_CompatibilityAnnotation();
        break;
      default:
        whitespace();
        parse_Annotation();
      }
    }
    switch (l1)
    {
    case 268:                       // 'variable'
      whitespace();
      parse_VarDecl();
      break;
    case 147:                       // 'function'
      whitespace();
      parse_FunctionDecl();
      break;
    case 96:                        // 'collection'
      whitespace();
      parse_CollectionDecl();
      break;
    case 157:                       // 'index'
      whitespace();
      parse_IndexDecl();
      break;
    default:
      whitespace();
      parse_ICDecl();
    }
    eventHandler.endNonterminal("AnnotatedDecl", e0);
  }

  function parse_CompatibilityAnnotation()
  {
    eventHandler.startNonterminal("CompatibilityAnnotation", e0);
    shift(263);                     // 'updating'
    eventHandler.endNonterminal("CompatibilityAnnotation", e0);
  }

  function parse_Annotation()
  {
    eventHandler.startNonterminal("Annotation", e0);
    shift(33);                      // '%'
    lookahead1W(246);               // EQName^Token | S^WS | '(:' | 'after' | 'allowing' | 'ancestor' |
    whitespace();
    parse_EQName();
    lookahead1W(193);               // S^WS | '%' | '(' | '(:' | 'collection' | 'function' | 'index' | 'integrity' |
    if (l1 == 35)                   // '('
    {
      shift(35);                    // '('
      lookahead1W(190);             // IntegerLiteral | DecimalLiteral | DoubleLiteral | StringLiteral | S^WS | '(:' |
      whitespace();
      parse_Literal();
      for (;;)
      {
        lookahead1W(105);           // S^WS | '(:' | ')' | ','
        if (l1 != 42)               // ','
        {
          break;
        }
        shift(42);                  // ','
        lookahead1W(190);           // IntegerLiteral | DecimalLiteral | DoubleLiteral | StringLiteral | S^WS | '(:' |
        whitespace();
        parse_Literal();
      }
      shift(38);                    // ')'
    }
    eventHandler.endNonterminal("Annotation", e0);
  }

  function try_Annotation()
  {
    shiftT(33);                     // '%'
    lookahead1W(246);               // EQName^Token | S^WS | '(:' | 'after' | 'allowing' | 'ancestor' |
    try_EQName();
    lookahead1W(193);               // S^WS | '%' | '(' | '(:' | 'collection' | 'function' | 'index' | 'integrity' |
    if (l1 == 35)                   // '('
    {
      shiftT(35);                   // '('
      lookahead1W(190);             // IntegerLiteral | DecimalLiteral | DoubleLiteral | StringLiteral | S^WS | '(:' |
      try_Literal();
      for (;;)
      {
        lookahead1W(105);           // S^WS | '(:' | ')' | ','
        if (l1 != 42)               // ','
        {
          break;
        }
        shiftT(42);                 // ','
        lookahead1W(190);           // IntegerLiteral | DecimalLiteral | DoubleLiteral | StringLiteral | S^WS | '(:' |
        try_Literal();
      }
      shiftT(38);                   // ')'
    }
  }

  function parse_VarDecl()
  {
    eventHandler.startNonterminal("VarDecl", e0);
    shift(268);                     // 'variable'
    lookahead1W(21);                // S^WS | '$' | '(:'
    shift(31);                      // '$'
    lookahead1W(246);               // EQName^Token | S^WS | '(:' | 'after' | 'allowing' | 'ancestor' |
    whitespace();
    parse_VarName();
    lookahead1W(157);               // S^WS | '(:' | ':=' | 'as' | 'external'
    if (l1 == 80)                   // 'as'
    {
      whitespace();
      parse_TypeDeclaration();
    }
    lookahead1W(110);               // S^WS | '(:' | ':=' | 'external'
    switch (l1)
    {
    case 53:                        // ':='
      shift(53);                    // ':='
      lookahead1W(267);             // EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral | StringLiteral |
      whitespace();
      parse_VarValue();
      break;
    default:
      shift(134);                   // 'external'
      lookahead1W(108);             // S^WS | '(:' | ':=' | ';'
      if (l1 == 53)                 // ':='
      {
        shift(53);                  // ':='
        lookahead1W(267);           // EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral | StringLiteral |
        whitespace();
        parse_VarDefaultValue();
      }
    }
    eventHandler.endNonterminal("VarDecl", e0);
  }

  function parse_VarValue()
  {
    eventHandler.startNonterminal("VarValue", e0);
    parse_ExprSingle();
    eventHandler.endNonterminal("VarValue", e0);
  }

  function parse_VarDefaultValue()
  {
    eventHandler.startNonterminal("VarDefaultValue", e0);
    parse_ExprSingle();
    eventHandler.endNonterminal("VarDefaultValue", e0);
  }

  function parse_ContextItemDecl()
  {
    eventHandler.startNonterminal("ContextItemDecl", e0);
    shift(109);                     // 'declare'
    lookahead1W(46);                // S^WS | '(:' | 'context'
    shift(102);                     // 'context'
    lookahead1W(58);                // S^WS | '(:' | 'item'
    shift(167);                     // 'item'
    lookahead1W(157);               // S^WS | '(:' | ':=' | 'as' | 'external'
    if (l1 == 80)                   // 'as'
    {
      shift(80);                    // 'as'
      lookahead1W(254);             // EQName^Token | S^WS | '%' | '(' | '(:' | 'after' | 'allowing' | 'ancestor' |
      whitespace();
      parse_ItemType();
    }
    lookahead1W(110);               // S^WS | '(:' | ':=' | 'external'
    switch (l1)
    {
    case 53:                        // ':='
      shift(53);                    // ':='
      lookahead1W(267);             // EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral | StringLiteral |
      whitespace();
      parse_VarValue();
      break;
    default:
      shift(134);                   // 'external'
      lookahead1W(108);             // S^WS | '(:' | ':=' | ';'
      if (l1 == 53)                 // ':='
      {
        shift(53);                  // ':='
        lookahead1W(267);           // EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral | StringLiteral |
        whitespace();
        parse_VarDefaultValue();
      }
    }
    eventHandler.endNonterminal("ContextItemDecl", e0);
  }

  function parse_ParamList()
  {
    eventHandler.startNonterminal("ParamList", e0);
    parse_Param();
    for (;;)
    {
      lookahead1W(105);             // S^WS | '(:' | ')' | ','
      if (l1 != 42)                 // ','
      {
        break;
      }
      shift(42);                    // ','
      lookahead1W(21);              // S^WS | '$' | '(:'
      whitespace();
      parse_Param();
    }
    eventHandler.endNonterminal("ParamList", e0);
  }

  function try_ParamList()
  {
    try_Param();
    for (;;)
    {
      lookahead1W(105);             // S^WS | '(:' | ')' | ','
      if (l1 != 42)                 // ','
      {
        break;
      }
      shiftT(42);                   // ','
      lookahead1W(21);              // S^WS | '$' | '(:'
      try_Param();
    }
  }

  function parse_Param()
  {
    eventHandler.startNonterminal("Param", e0);
    shift(31);                      // '$'
    lookahead1W(246);               // EQName^Token | S^WS | '(:' | 'after' | 'allowing' | 'ancestor' |
    whitespace();
    parse_EQName();
    lookahead1W(153);               // S^WS | '(:' | ')' | ',' | 'as'
    if (l1 == 80)                   // 'as'
    {
      whitespace();
      parse_TypeDeclaration();
    }
    eventHandler.endNonterminal("Param", e0);
  }

  function try_Param()
  {
    shiftT(31);                     // '$'
    lookahead1W(246);               // EQName^Token | S^WS | '(:' | 'after' | 'allowing' | 'ancestor' |
    try_EQName();
    lookahead1W(153);               // S^WS | '(:' | ')' | ',' | 'as'
    if (l1 == 80)                   // 'as'
    {
      try_TypeDeclaration();
    }
  }

  function parse_FunctionBody()
  {
    eventHandler.startNonterminal("FunctionBody", e0);
    parse_EnclosedExpr();
    eventHandler.endNonterminal("FunctionBody", e0);
  }

  function try_FunctionBody()
  {
    try_EnclosedExpr();
  }

  function parse_EnclosedExpr()
  {
    eventHandler.startNonterminal("EnclosedExpr", e0);
    shift(281);                     // '{'
    lookahead1W(267);               // EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral | StringLiteral |
    whitespace();
    parse_Expr();
    shift(287);                     // '}'
    eventHandler.endNonterminal("EnclosedExpr", e0);
  }

  function try_EnclosedExpr()
  {
    shiftT(281);                    // '{'
    lookahead1W(267);               // EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral | StringLiteral |
    try_Expr();
    shiftT(287);                    // '}'
  }

  function parse_OptionDecl()
  {
    eventHandler.startNonterminal("OptionDecl", e0);
    shift(109);                     // 'declare'
    lookahead1W(69);                // S^WS | '(:' | 'option'
    shift(203);                     // 'option'
    lookahead1W(246);               // EQName^Token | S^WS | '(:' | 'after' | 'allowing' | 'ancestor' |
    whitespace();
    parse_EQName();
    lookahead1W(17);                // StringLiteral | S^WS | '(:'
    shift(11);                      // StringLiteral
    eventHandler.endNonterminal("OptionDecl", e0);
  }

  function parse_Expr()
  {
    eventHandler.startNonterminal("Expr", e0);
    parse_ExprSingle();
    for (;;)
    {
      if (l1 != 42)                 // ','
      {
        break;
      }
      shift(42);                    // ','
      lookahead1W(267);             // EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral | StringLiteral |
      whitespace();
      parse_ExprSingle();
    }
    eventHandler.endNonterminal("Expr", e0);
  }

  function try_Expr()
  {
    try_ExprSingle();
    for (;;)
    {
      if (l1 != 42)                 // ','
      {
        break;
      }
      shiftT(42);                   // ','
      lookahead1W(267);             // EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral | StringLiteral |
      try_ExprSingle();
    }
  }

  function parse_FLWORExpr()
  {
    eventHandler.startNonterminal("FLWORExpr", e0);
    parse_InitialClause();
    for (;;)
    {
      lookahead1W(195);             // S^WS | '(:' | 'count' | 'for' | 'group' | 'let' | 'order' | 'return' | 'stable' |
      if (l1 == 224)                // 'return'
      {
        break;
      }
      whitespace();
      parse_IntermediateClause();
    }
    whitespace();
    parse_ReturnClause();
    eventHandler.endNonterminal("FLWORExpr", e0);
  }

  function try_FLWORExpr()
  {
    try_InitialClause();
    for (;;)
    {
      lookahead1W(195);             // S^WS | '(:' | 'count' | 'for' | 'group' | 'let' | 'order' | 'return' | 'stable' |
      if (l1 == 224)                // 'return'
      {
        break;
      }
      try_IntermediateClause();
    }
    try_ReturnClause();
  }

  function parse_InitialClause()
  {
    eventHandler.startNonterminal("InitialClause", e0);
    switch (l1)
    {
    case 139:                       // 'for'
      lookahead2W(151);             // S^WS | '$' | '(:' | 'sliding' | 'tumbling'
      break;
    default:
      lk = l1;
    }
    switch (lk)
    {
    case 16011:                     // 'for' '$'
      parse_ForClause();
      break;
    case 177:                       // 'let'
      parse_LetClause();
      break;
    default:
      parse_WindowClause();
    }
    eventHandler.endNonterminal("InitialClause", e0);
  }

  function try_InitialClause()
  {
    switch (l1)
    {
    case 139:                       // 'for'
      lookahead2W(151);             // S^WS | '$' | '(:' | 'sliding' | 'tumbling'
      break;
    default:
      lk = l1;
    }
    switch (lk)
    {
    case 16011:                     // 'for' '$'
      try_ForClause();
      break;
    case 177:                       // 'let'
      try_LetClause();
      break;
    default:
      try_WindowClause();
    }
  }

  function parse_IntermediateClause()
  {
    eventHandler.startNonterminal("IntermediateClause", e0);
    switch (l1)
    {
    case 139:                       // 'for'
    case 177:                       // 'let'
      parse_InitialClause();
      break;
    case 272:                       // 'where'
      parse_WhereClause();
      break;
    case 150:                       // 'group'
      parse_GroupByClause();
      break;
    case 106:                       // 'count'
      parse_CountClause();
      break;
    default:
      parse_OrderByClause();
    }
    eventHandler.endNonterminal("IntermediateClause", e0);
  }

  function try_IntermediateClause()
  {
    switch (l1)
    {
    case 139:                       // 'for'
    case 177:                       // 'let'
      try_InitialClause();
      break;
    case 272:                       // 'where'
      try_WhereClause();
      break;
    case 150:                       // 'group'
      try_GroupByClause();
      break;
    case 106:                       // 'count'
      try_CountClause();
      break;
    default:
      try_OrderByClause();
    }
  }

  function parse_ForClause()
  {
    eventHandler.startNonterminal("ForClause", e0);
    shift(139);                     // 'for'
    lookahead1W(21);                // S^WS | '$' | '(:'
    whitespace();
    parse_ForBinding();
    for (;;)
    {
      if (l1 != 42)                 // ','
      {
        break;
      }
      shift(42);                    // ','
      lookahead1W(21);              // S^WS | '$' | '(:'
      whitespace();
      parse_ForBinding();
    }
    eventHandler.endNonterminal("ForClause", e0);
  }

  function try_ForClause()
  {
    shiftT(139);                    // 'for'
    lookahead1W(21);                // S^WS | '$' | '(:'
    try_ForBinding();
    for (;;)
    {
      if (l1 != 42)                 // ','
      {
        break;
      }
      shiftT(42);                   // ','
      lookahead1W(21);              // S^WS | '$' | '(:'
      try_ForBinding();
    }
  }

  function parse_ForBinding()
  {
    eventHandler.startNonterminal("ForBinding", e0);
    shift(31);                      // '$'
    lookahead1W(246);               // EQName^Token | S^WS | '(:' | 'after' | 'allowing' | 'ancestor' |
    whitespace();
    parse_VarName();
    lookahead1W(182);               // S^WS | '(:' | 'allowing' | 'as' | 'at' | 'in' | 'score'
    if (l1 == 80)                   // 'as'
    {
      whitespace();
      parse_TypeDeclaration();
    }
    lookahead1W(173);               // S^WS | '(:' | 'allowing' | 'at' | 'in' | 'score'
    if (l1 == 73)                   // 'allowing'
    {
      whitespace();
      parse_AllowingEmpty();
    }
    lookahead1W(160);               // S^WS | '(:' | 'at' | 'in' | 'score'
    if (l1 == 82)                   // 'at'
    {
      whitespace();
      parse_PositionalVar();
    }
    lookahead1W(126);               // S^WS | '(:' | 'in' | 'score'
    if (l1 == 232)                  // 'score'
    {
      whitespace();
      parse_FTScoreVar();
    }
    lookahead1W(56);                // S^WS | '(:' | 'in'
    shift(156);                     // 'in'
    lookahead1W(267);               // EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral | StringLiteral |
    whitespace();
    parse_ExprSingle();
    eventHandler.endNonterminal("ForBinding", e0);
  }

  function try_ForBinding()
  {
    shiftT(31);                     // '$'
    lookahead1W(246);               // EQName^Token | S^WS | '(:' | 'after' | 'allowing' | 'ancestor' |
    try_VarName();
    lookahead1W(182);               // S^WS | '(:' | 'allowing' | 'as' | 'at' | 'in' | 'score'
    if (l1 == 80)                   // 'as'
    {
      try_TypeDeclaration();
    }
    lookahead1W(173);               // S^WS | '(:' | 'allowing' | 'at' | 'in' | 'score'
    if (l1 == 73)                   // 'allowing'
    {
      try_AllowingEmpty();
    }
    lookahead1W(160);               // S^WS | '(:' | 'at' | 'in' | 'score'
    if (l1 == 82)                   // 'at'
    {
      try_PositionalVar();
    }
    lookahead1W(126);               // S^WS | '(:' | 'in' | 'score'
    if (l1 == 232)                  // 'score'
    {
      try_FTScoreVar();
    }
    lookahead1W(56);                // S^WS | '(:' | 'in'
    shiftT(156);                    // 'in'
    lookahead1W(267);               // EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral | StringLiteral |
    try_ExprSingle();
  }

  function parse_AllowingEmpty()
  {
    eventHandler.startNonterminal("AllowingEmpty", e0);
    shift(73);                      // 'allowing'
    lookahead1W(52);                // S^WS | '(:' | 'empty'
    shift(124);                     // 'empty'
    eventHandler.endNonterminal("AllowingEmpty", e0);
  }

  function try_AllowingEmpty()
  {
    shiftT(73);                     // 'allowing'
    lookahead1W(52);                // S^WS | '(:' | 'empty'
    shiftT(124);                    // 'empty'
  }

  function parse_PositionalVar()
  {
    eventHandler.startNonterminal("PositionalVar", e0);
    shift(82);                      // 'at'
    lookahead1W(21);                // S^WS | '$' | '(:'
    shift(31);                      // '$'
    lookahead1W(246);               // EQName^Token | S^WS | '(:' | 'after' | 'allowing' | 'ancestor' |
    whitespace();
    parse_VarName();
    eventHandler.endNonterminal("PositionalVar", e0);
  }

  function try_PositionalVar()
  {
    shiftT(82);                     // 'at'
    lookahead1W(21);                // S^WS | '$' | '(:'
    shiftT(31);                     // '$'
    lookahead1W(246);               // EQName^Token | S^WS | '(:' | 'after' | 'allowing' | 'ancestor' |
    try_VarName();
  }

  function parse_FTScoreVar()
  {
    eventHandler.startNonterminal("FTScoreVar", e0);
    shift(232);                     // 'score'
    lookahead1W(21);                // S^WS | '$' | '(:'
    shift(31);                      // '$'
    lookahead1W(246);               // EQName^Token | S^WS | '(:' | 'after' | 'allowing' | 'ancestor' |
    whitespace();
    parse_VarName();
    eventHandler.endNonterminal("FTScoreVar", e0);
  }

  function try_FTScoreVar()
  {
    shiftT(232);                    // 'score'
    lookahead1W(21);                // S^WS | '$' | '(:'
    shiftT(31);                     // '$'
    lookahead1W(246);               // EQName^Token | S^WS | '(:' | 'after' | 'allowing' | 'ancestor' |
    try_VarName();
  }

  function parse_LetClause()
  {
    eventHandler.startNonterminal("LetClause", e0);
    shift(177);                     // 'let'
    lookahead1W(100);               // S^WS | '$' | '(:' | 'score'
    whitespace();
    parse_LetBinding();
    for (;;)
    {
      if (l1 != 42)                 // ','
      {
        break;
      }
      shift(42);                    // ','
      lookahead1W(100);             // S^WS | '$' | '(:' | 'score'
      whitespace();
      parse_LetBinding();
    }
    eventHandler.endNonterminal("LetClause", e0);
  }

  function try_LetClause()
  {
    shiftT(177);                    // 'let'
    lookahead1W(100);               // S^WS | '$' | '(:' | 'score'
    try_LetBinding();
    for (;;)
    {
      if (l1 != 42)                 // ','
      {
        break;
      }
      shiftT(42);                   // ','
      lookahead1W(100);             // S^WS | '$' | '(:' | 'score'
      try_LetBinding();
    }
  }

  function parse_LetBinding()
  {
    eventHandler.startNonterminal("LetBinding", e0);
    switch (l1)
    {
    case 31:                        // '$'
      shift(31);                    // '$'
      lookahead1W(246);             // EQName^Token | S^WS | '(:' | 'after' | 'allowing' | 'ancestor' |
      whitespace();
      parse_VarName();
      lookahead1W(109);             // S^WS | '(:' | ':=' | 'as'
      if (l1 == 80)                 // 'as'
      {
        whitespace();
        parse_TypeDeclaration();
      }
      break;
    default:
      parse_FTScoreVar();
    }
    lookahead1W(28);                // S^WS | '(:' | ':='
    shift(53);                      // ':='
    lookahead1W(267);               // EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral | StringLiteral |
    whitespace();
    parse_ExprSingle();
    eventHandler.endNonterminal("LetBinding", e0);
  }

  function try_LetBinding()
  {
    switch (l1)
    {
    case 31:                        // '$'
      shiftT(31);                   // '$'
      lookahead1W(246);             // EQName^Token | S^WS | '(:' | 'after' | 'allowing' | 'ancestor' |
      try_VarName();
      lookahead1W(109);             // S^WS | '(:' | ':=' | 'as'
      if (l1 == 80)                 // 'as'
      {
        try_TypeDeclaration();
      }
      break;
    default:
      try_FTScoreVar();
    }
    lookahead1W(28);                // S^WS | '(:' | ':='
    shiftT(53);                     // ':='
    lookahead1W(267);               // EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral | StringLiteral |
    try_ExprSingle();
  }

  function parse_WindowClause()
  {
    eventHandler.startNonterminal("WindowClause", e0);
    shift(139);                     // 'for'
    lookahead1W(139);               // S^WS | '(:' | 'sliding' | 'tumbling'
    switch (l1)
    {
    case 257:                       // 'tumbling'
      whitespace();
      parse_TumblingWindowClause();
      break;
    default:
      whitespace();
      parse_SlidingWindowClause();
    }
    eventHandler.endNonterminal("WindowClause", e0);
  }

  function try_WindowClause()
  {
    shiftT(139);                    // 'for'
    lookahead1W(139);               // S^WS | '(:' | 'sliding' | 'tumbling'
    switch (l1)
    {
    case 257:                       // 'tumbling'
      try_TumblingWindowClause();
      break;
    default:
      try_SlidingWindowClause();
    }
  }

  function parse_TumblingWindowClause()
  {
    eventHandler.startNonterminal("TumblingWindowClause", e0);
    shift(257);                     // 'tumbling'
    lookahead1W(88);                // S^WS | '(:' | 'window'
    shift(275);                     // 'window'
    lookahead1W(21);                // S^WS | '$' | '(:'
    shift(31);                      // '$'
    lookahead1W(246);               // EQName^Token | S^WS | '(:' | 'after' | 'allowing' | 'ancestor' |
    whitespace();
    parse_VarName();
    lookahead1W(114);               // S^WS | '(:' | 'as' | 'in'
    if (l1 == 80)                   // 'as'
    {
      whitespace();
      parse_TypeDeclaration();
    }
    lookahead1W(56);                // S^WS | '(:' | 'in'
    shift(156);                     // 'in'
    lookahead1W(267);               // EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral | StringLiteral |
    whitespace();
    parse_ExprSingle();
    whitespace();
    parse_WindowStartCondition();
    if (l1 == 127                   // 'end'
     || l1 == 202)                  // 'only'
    {
      whitespace();
      parse_WindowEndCondition();
    }
    eventHandler.endNonterminal("TumblingWindowClause", e0);
  }

  function try_TumblingWindowClause()
  {
    shiftT(257);                    // 'tumbling'
    lookahead1W(88);                // S^WS | '(:' | 'window'
    shiftT(275);                    // 'window'
    lookahead1W(21);                // S^WS | '$' | '(:'
    shiftT(31);                     // '$'
    lookahead1W(246);               // EQName^Token | S^WS | '(:' | 'after' | 'allowing' | 'ancestor' |
    try_VarName();
    lookahead1W(114);               // S^WS | '(:' | 'as' | 'in'
    if (l1 == 80)                   // 'as'
    {
      try_TypeDeclaration();
    }
    lookahead1W(56);                // S^WS | '(:' | 'in'
    shiftT(156);                    // 'in'
    lookahead1W(267);               // EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral | StringLiteral |
    try_ExprSingle();
    try_WindowStartCondition();
    if (l1 == 127                   // 'end'
     || l1 == 202)                  // 'only'
    {
      try_WindowEndCondition();
    }
  }

  function parse_SlidingWindowClause()
  {
    eventHandler.startNonterminal("SlidingWindowClause", e0);
    shift(239);                     // 'sliding'
    lookahead1W(88);                // S^WS | '(:' | 'window'
    shift(275);                     // 'window'
    lookahead1W(21);                // S^WS | '$' | '(:'
    shift(31);                      // '$'
    lookahead1W(246);               // EQName^Token | S^WS | '(:' | 'after' | 'allowing' | 'ancestor' |
    whitespace();
    parse_VarName();
    lookahead1W(114);               // S^WS | '(:' | 'as' | 'in'
    if (l1 == 80)                   // 'as'
    {
      whitespace();
      parse_TypeDeclaration();
    }
    lookahead1W(56);                // S^WS | '(:' | 'in'
    shift(156);                     // 'in'
    lookahead1W(267);               // EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral | StringLiteral |
    whitespace();
    parse_ExprSingle();
    whitespace();
    parse_WindowStartCondition();
    whitespace();
    parse_WindowEndCondition();
    eventHandler.endNonterminal("SlidingWindowClause", e0);
  }

  function try_SlidingWindowClause()
  {
    shiftT(239);                    // 'sliding'
    lookahead1W(88);                // S^WS | '(:' | 'window'
    shiftT(275);                    // 'window'
    lookahead1W(21);                // S^WS | '$' | '(:'
    shiftT(31);                     // '$'
    lookahead1W(246);               // EQName^Token | S^WS | '(:' | 'after' | 'allowing' | 'ancestor' |
    try_VarName();
    lookahead1W(114);               // S^WS | '(:' | 'as' | 'in'
    if (l1 == 80)                   // 'as'
    {
      try_TypeDeclaration();
    }
    lookahead1W(56);                // S^WS | '(:' | 'in'
    shiftT(156);                    // 'in'
    lookahead1W(267);               // EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral | StringLiteral |
    try_ExprSingle();
    try_WindowStartCondition();
    try_WindowEndCondition();
  }

  function parse_WindowStartCondition()
  {
    eventHandler.startNonterminal("WindowStartCondition", e0);
    shift(242);                     // 'start'
    lookahead1W(181);               // S^WS | '$' | '(:' | 'at' | 'next' | 'previous' | 'when'
    whitespace();
    parse_WindowVars();
    lookahead1W(86);                // S^WS | '(:' | 'when'
    shift(271);                     // 'when'
    lookahead1W(267);               // EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral | StringLiteral |
    whitespace();
    parse_ExprSingle();
    eventHandler.endNonterminal("WindowStartCondition", e0);
  }

  function try_WindowStartCondition()
  {
    shiftT(242);                    // 'start'
    lookahead1W(181);               // S^WS | '$' | '(:' | 'at' | 'next' | 'previous' | 'when'
    try_WindowVars();
    lookahead1W(86);                // S^WS | '(:' | 'when'
    shiftT(271);                    // 'when'
    lookahead1W(267);               // EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral | StringLiteral |
    try_ExprSingle();
  }

  function parse_WindowEndCondition()
  {
    eventHandler.startNonterminal("WindowEndCondition", e0);
    if (l1 == 202)                  // 'only'
    {
      shift(202);                   // 'only'
    }
    lookahead1W(53);                // S^WS | '(:' | 'end'
    shift(127);                     // 'end'
    lookahead1W(181);               // S^WS | '$' | '(:' | 'at' | 'next' | 'previous' | 'when'
    whitespace();
    parse_WindowVars();
    lookahead1W(86);                // S^WS | '(:' | 'when'
    shift(271);                     // 'when'
    lookahead1W(267);               // EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral | StringLiteral |
    whitespace();
    parse_ExprSingle();
    eventHandler.endNonterminal("WindowEndCondition", e0);
  }

  function try_WindowEndCondition()
  {
    if (l1 == 202)                  // 'only'
    {
      shiftT(202);                  // 'only'
    }
    lookahead1W(53);                // S^WS | '(:' | 'end'
    shiftT(127);                    // 'end'
    lookahead1W(181);               // S^WS | '$' | '(:' | 'at' | 'next' | 'previous' | 'when'
    try_WindowVars();
    lookahead1W(86);                // S^WS | '(:' | 'when'
    shiftT(271);                    // 'when'
    lookahead1W(267);               // EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral | StringLiteral |
    try_ExprSingle();
  }

  function parse_WindowVars()
  {
    eventHandler.startNonterminal("WindowVars", e0);
    if (l1 == 31)                   // '$'
    {
      shift(31);                    // '$'
      lookahead1W(246);             // EQName^Token | S^WS | '(:' | 'after' | 'allowing' | 'ancestor' |
      whitespace();
      parse_CurrentItem();
    }
    lookahead1W(174);               // S^WS | '(:' | 'at' | 'next' | 'previous' | 'when'
    if (l1 == 82)                   // 'at'
    {
      whitespace();
      parse_PositionalVar();
    }
    lookahead1W(163);               // S^WS | '(:' | 'next' | 'previous' | 'when'
    if (l1 == 219)                  // 'previous'
    {
      shift(219);                   // 'previous'
      lookahead1W(21);              // S^WS | '$' | '(:'
      shift(31);                    // '$'
      lookahead1W(246);             // EQName^Token | S^WS | '(:' | 'after' | 'allowing' | 'ancestor' |
      whitespace();
      parse_PreviousItem();
    }
    lookahead1W(131);               // S^WS | '(:' | 'next' | 'when'
    if (l1 == 190)                  // 'next'
    {
      shift(190);                   // 'next'
      lookahead1W(21);              // S^WS | '$' | '(:'
      shift(31);                    // '$'
      lookahead1W(246);             // EQName^Token | S^WS | '(:' | 'after' | 'allowing' | 'ancestor' |
      whitespace();
      parse_NextItem();
    }
    eventHandler.endNonterminal("WindowVars", e0);
  }

  function try_WindowVars()
  {
    if (l1 == 31)                   // '$'
    {
      shiftT(31);                   // '$'
      lookahead1W(246);             // EQName^Token | S^WS | '(:' | 'after' | 'allowing' | 'ancestor' |
      try_CurrentItem();
    }
    lookahead1W(174);               // S^WS | '(:' | 'at' | 'next' | 'previous' | 'when'
    if (l1 == 82)                   // 'at'
    {
      try_PositionalVar();
    }
    lookahead1W(163);               // S^WS | '(:' | 'next' | 'previous' | 'when'
    if (l1 == 219)                  // 'previous'
    {
      shiftT(219);                  // 'previous'
      lookahead1W(21);              // S^WS | '$' | '(:'
      shiftT(31);                   // '$'
      lookahead1W(246);             // EQName^Token | S^WS | '(:' | 'after' | 'allowing' | 'ancestor' |
      try_PreviousItem();
    }
    lookahead1W(131);               // S^WS | '(:' | 'next' | 'when'
    if (l1 == 190)                  // 'next'
    {
      shiftT(190);                  // 'next'
      lookahead1W(21);              // S^WS | '$' | '(:'
      shiftT(31);                   // '$'
      lookahead1W(246);             // EQName^Token | S^WS | '(:' | 'after' | 'allowing' | 'ancestor' |
      try_NextItem();
    }
  }

  function parse_CurrentItem()
  {
    eventHandler.startNonterminal("CurrentItem", e0);
    parse_EQName();
    eventHandler.endNonterminal("CurrentItem", e0);
  }

  function try_CurrentItem()
  {
    try_EQName();
  }

  function parse_PreviousItem()
  {
    eventHandler.startNonterminal("PreviousItem", e0);
    parse_EQName();
    eventHandler.endNonterminal("PreviousItem", e0);
  }

  function try_PreviousItem()
  {
    try_EQName();
  }

  function parse_NextItem()
  {
    eventHandler.startNonterminal("NextItem", e0);
    parse_EQName();
    eventHandler.endNonterminal("NextItem", e0);
  }

  function try_NextItem()
  {
    try_EQName();
  }

  function parse_CountClause()
  {
    eventHandler.startNonterminal("CountClause", e0);
    shift(106);                     // 'count'
    lookahead1W(21);                // S^WS | '$' | '(:'
    shift(31);                      // '$'
    lookahead1W(246);               // EQName^Token | S^WS | '(:' | 'after' | 'allowing' | 'ancestor' |
    whitespace();
    parse_VarName();
    eventHandler.endNonterminal("CountClause", e0);
  }

  function try_CountClause()
  {
    shiftT(106);                    // 'count'
    lookahead1W(21);                // S^WS | '$' | '(:'
    shiftT(31);                     // '$'
    lookahead1W(246);               // EQName^Token | S^WS | '(:' | 'after' | 'allowing' | 'ancestor' |
    try_VarName();
  }

  function parse_WhereClause()
  {
    eventHandler.startNonterminal("WhereClause", e0);
    shift(272);                     // 'where'
    lookahead1W(267);               // EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral | StringLiteral |
    whitespace();
    parse_ExprSingle();
    eventHandler.endNonterminal("WhereClause", e0);
  }

  function try_WhereClause()
  {
    shiftT(272);                    // 'where'
    lookahead1W(267);               // EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral | StringLiteral |
    try_ExprSingle();
  }

  function parse_GroupByClause()
  {
    eventHandler.startNonterminal("GroupByClause", e0);
    shift(150);                     // 'group'
    lookahead1W(37);                // S^WS | '(:' | 'by'
    shift(88);                      // 'by'
    lookahead1W(267);               // EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral | StringLiteral |
    whitespace();
    parse_GroupingSpecList();
    eventHandler.endNonterminal("GroupByClause", e0);
  }

  function try_GroupByClause()
  {
    shiftT(150);                    // 'group'
    lookahead1W(37);                // S^WS | '(:' | 'by'
    shiftT(88);                     // 'by'
    lookahead1W(267);               // EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral | StringLiteral |
    try_GroupingSpecList();
  }

  function parse_GroupingSpecList()
  {
    eventHandler.startNonterminal("GroupingSpecList", e0);
    parse_GroupingSpec();
    for (;;)
    {
      lookahead1W(198);             // S^WS | '(:' | ',' | 'count' | 'for' | 'group' | 'let' | 'order' | 'return' |
      if (l1 != 42)                 // ','
      {
        break;
      }
      shift(42);                    // ','
      lookahead1W(267);             // EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral | StringLiteral |
      whitespace();
      parse_GroupingSpec();
    }
    eventHandler.endNonterminal("GroupingSpecList", e0);
  }

  function try_GroupingSpecList()
  {
    try_GroupingSpec();
    for (;;)
    {
      lookahead1W(198);             // S^WS | '(:' | ',' | 'count' | 'for' | 'group' | 'let' | 'order' | 'return' |
      if (l1 != 42)                 // ','
      {
        break;
      }
      shiftT(42);                   // ','
      lookahead1W(267);             // EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral | StringLiteral |
      try_GroupingSpec();
    }
  }

  function parse_GroupingSpec()
  {
    eventHandler.startNonterminal("GroupingSpec", e0);
    switch (l1)
    {
    case 31:                        // '$'
      lookahead2W(246);             // EQName^Token | S^WS | '(:' | 'after' | 'allowing' | 'ancestor' |
      break;
    default:
      lk = l1;
    }
    if (lk == 3103                  // '$' EQName^Token
     || lk == 36383                 // '$' 'after'
     || lk == 37407                 // '$' 'allowing'
     || lk == 37919                 // '$' 'ancestor'
     || lk == 38431                 // '$' 'ancestor-or-self'
     || lk == 38943                 // '$' 'and'
     || lk == 39967                 // '$' 'append'
     || lk == 40479                 // '$' 'array'
     || lk == 40991                 // '$' 'as'
     || lk == 41503                 // '$' 'ascending'
     || lk == 42015                 // '$' 'at'
     || lk == 42527                 // '$' 'attribute'
     || lk == 43039                 // '$' 'base-uri'
     || lk == 43551                 // '$' 'before'
     || lk == 44063                 // '$' 'boundary-space'
     || lk == 44575                 // '$' 'break'
     || lk == 45599                 // '$' 'case'
     || lk == 46111                 // '$' 'cast'
     || lk == 46623                 // '$' 'castable'
     || lk == 47135                 // '$' 'catch'
     || lk == 48159                 // '$' 'child'
     || lk == 48671                 // '$' 'collation'
     || lk == 49695                 // '$' 'comment'
     || lk == 50207                 // '$' 'constraint'
     || lk == 50719                 // '$' 'construction'
     || lk == 52255                 // '$' 'context'
     || lk == 52767                 // '$' 'continue'
     || lk == 53279                 // '$' 'copy'
     || lk == 53791                 // '$' 'copy-namespaces'
     || lk == 54303                 // '$' 'count'
     || lk == 54815                 // '$' 'decimal-format'
     || lk == 55839                 // '$' 'declare'
     || lk == 56351                 // '$' 'default'
     || lk == 56863                 // '$' 'delete'
     || lk == 57375                 // '$' 'descendant'
     || lk == 57887                 // '$' 'descendant-or-self'
     || lk == 58399                 // '$' 'descending'
     || lk == 60959                 // '$' 'div'
     || lk == 61471                 // '$' 'document'
     || lk == 61983                 // '$' 'document-node'
     || lk == 62495                 // '$' 'element'
     || lk == 63007                 // '$' 'else'
     || lk == 63519                 // '$' 'empty'
     || lk == 64031                 // '$' 'empty-sequence'
     || lk == 64543                 // '$' 'encoding'
     || lk == 65055                 // '$' 'end'
     || lk == 66079                 // '$' 'eq'
     || lk == 66591                 // '$' 'every'
     || lk == 67615                 // '$' 'except'
     || lk == 68127                 // '$' 'exit'
     || lk == 68639                 // '$' 'external'
     || lk == 69151                 // '$' 'false'
     || lk == 69663                 // '$' 'first'
     || lk == 70175                 // '$' 'following'
     || lk == 70687                 // '$' 'following-sibling'
     || lk == 71199                 // '$' 'for'
     || lk == 72735                 // '$' 'from'
     || lk == 73247                 // '$' 'ft-option'
     || lk == 75295                 // '$' 'function'
     || lk == 75807                 // '$' 'ge'
     || lk == 76831                 // '$' 'group'
     || lk == 77855                 // '$' 'gt'
     || lk == 78367                 // '$' 'idiv'
     || lk == 78879                 // '$' 'if'
     || lk == 79391                 // '$' 'import'
     || lk == 79903                 // '$' 'in'
     || lk == 80415                 // '$' 'index'
     || lk == 82463                 // '$' 'insert'
     || lk == 82975                 // '$' 'instance'
     || lk == 83487                 // '$' 'integrity'
     || lk == 83999                 // '$' 'intersect'
     || lk == 84511                 // '$' 'into'
     || lk == 85023                 // '$' 'is'
     || lk == 85535                 // '$' 'item'
     || lk == 86047                 // '$' 'json'
     || lk == 86559                 // '$' 'json-item'
     || lk == 87071                 // '$' 'jsoniq'
     || lk == 88607                 // '$' 'last'
     || lk == 89119                 // '$' 'lax'
     || lk == 89631                 // '$' 'le'
     || lk == 90655                 // '$' 'let'
     || lk == 91679                 // '$' 'loop'
     || lk == 92703                 // '$' 'lt'
     || lk == 93727                 // '$' 'mod'
     || lk == 94239                 // '$' 'modify'
     || lk == 94751                 // '$' 'module'
     || lk == 95775                 // '$' 'namespace'
     || lk == 96287                 // '$' 'namespace-node'
     || lk == 96799                 // '$' 'ne'
     || lk == 99359                 // '$' 'node'
     || lk == 99871                 // '$' 'nodes'
     || lk == 100895                // '$' 'null'
     || lk == 101407                // '$' 'object'
     || lk == 103455                // '$' 'only'
     || lk == 103967                // '$' 'option'
     || lk == 104479                // '$' 'or'
     || lk == 104991                // '$' 'order'
     || lk == 105503                // '$' 'ordered'
     || lk == 106015                // '$' 'ordering'
     || lk == 107551                // '$' 'parent'
     || lk == 110623                // '$' 'preceding'
     || lk == 111135                // '$' 'preceding-sibling'
     || lk == 112671                // '$' 'processing-instruction'
     || lk == 113695                // '$' 'rename'
     || lk == 114207                // '$' 'replace'
     || lk == 114719                // '$' 'return'
     || lk == 115231                // '$' 'returning'
     || lk == 115743                // '$' 'revalidation'
     || lk == 116767                // '$' 'satisfies'
     || lk == 117279                // '$' 'schema'
     || lk == 117791                // '$' 'schema-attribute'
     || lk == 118303                // '$' 'schema-element'
     || lk == 118815                // '$' 'score'
     || lk == 119327                // '$' 'select'
     || lk == 119839                // '$' 'self'
     || lk == 122399                // '$' 'sliding'
     || lk == 122911                // '$' 'some'
     || lk == 123423                // '$' 'stable'
     || lk == 123935                // '$' 'start'
     || lk == 125471                // '$' 'strict'
     || lk == 126495                // '$' 'structured-item'
     || lk == 127007                // '$' 'switch'
     || lk == 127519                // '$' 'text'
     || lk == 129567                // '$' 'to'
     || lk == 130079                // '$' 'treat'
     || lk == 130591                // '$' 'true'
     || lk == 131103                // '$' 'try'
     || lk == 131615                // '$' 'tumbling'
     || lk == 132127                // '$' 'type'
     || lk == 132639                // '$' 'typeswitch'
     || lk == 133151                // '$' 'union'
     || lk == 134175                // '$' 'unordered'
     || lk == 134687                // '$' 'updating'
     || lk == 136223                // '$' 'validate'
     || lk == 136735                // '$' 'value'
     || lk == 137247                // '$' 'variable'
     || lk == 137759                // '$' 'version'
     || lk == 139295                // '$' 'where'
     || lk == 139807                // '$' 'while'
     || lk == 141343)               // '$' 'with'
    {
      lk = memoized(2, e0);
      if (lk == 0)
      {
        var b0A = b0; var e0A = e0; var l1A = l1;
        var b1A = b1; var e1A = e1; var l2A = l2;
        var b2A = b2; var e2A = e2;
        try
        {
          try_GroupingVariable();
          lookahead1W(206);         // S^WS | '(:' | ',' | ':=' | 'as' | 'collation' | 'count' | 'for' | 'group' |
          if (l1 == 53              // ':='
           || l1 == 80)             // 'as'
          {
            if (l1 == 80)           // 'as'
            {
              try_TypeDeclaration();
            }
            lookahead1W(28);        // S^WS | '(:' | ':='
            shiftT(53);             // ':='
            lookahead1W(267);       // EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral | StringLiteral |
            try_ExprSingle();
          }
          if (l1 == 95)             // 'collation'
          {
            shiftT(95);             // 'collation'
            lookahead1W(15);        // URILiteral | S^WS | '(:'
            shiftT(7);              // URILiteral
          }
          lk = -1;
        }
        catch (p1A)
        {
          lk = -2;
        }
        b0 = b0A; e0 = e0A; l1 = l1A; if (l1 == 0) {end = e0A;} else {
        b1 = b1A; e1 = e1A; l2 = l2A; if (l2 == 0) {end = e1A;} else {
        b2 = b2A; e2 = e2A; end = e2A; }}
        memoize(2, e0, lk);
      }
    }
    switch (lk)
    {
    case -1:
      parse_GroupingVariable();
      lookahead1W(206);             // S^WS | '(:' | ',' | ':=' | 'as' | 'collation' | 'count' | 'for' | 'group' |
      if (l1 == 53                  // ':='
       || l1 == 80)                 // 'as'
      {
        if (l1 == 80)               // 'as'
        {
          whitespace();
          parse_TypeDeclaration();
        }
        lookahead1W(28);            // S^WS | '(:' | ':='
        shift(53);                  // ':='
        lookahead1W(267);           // EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral | StringLiteral |
        whitespace();
        parse_ExprSingle();
      }
      if (l1 == 95)                 // 'collation'
      {
        shift(95);                  // 'collation'
        lookahead1W(15);            // URILiteral | S^WS | '(:'
        shift(7);                   // URILiteral
      }
      break;
    default:
      parse_ExprSingle();
    }
    eventHandler.endNonterminal("GroupingSpec", e0);
  }

  function try_GroupingSpec()
  {
    switch (l1)
    {
    case 31:                        // '$'
      lookahead2W(246);             // EQName^Token | S^WS | '(:' | 'after' | 'allowing' | 'ancestor' |
      break;
    default:
      lk = l1;
    }
    if (lk == 3103                  // '$' EQName^Token
     || lk == 36383                 // '$' 'after'
     || lk == 37407                 // '$' 'allowing'
     || lk == 37919                 // '$' 'ancestor'
     || lk == 38431                 // '$' 'ancestor-or-self'
     || lk == 38943                 // '$' 'and'
     || lk == 39967                 // '$' 'append'
     || lk == 40479                 // '$' 'array'
     || lk == 40991                 // '$' 'as'
     || lk == 41503                 // '$' 'ascending'
     || lk == 42015                 // '$' 'at'
     || lk == 42527                 // '$' 'attribute'
     || lk == 43039                 // '$' 'base-uri'
     || lk == 43551                 // '$' 'before'
     || lk == 44063                 // '$' 'boundary-space'
     || lk == 44575                 // '$' 'break'
     || lk == 45599                 // '$' 'case'
     || lk == 46111                 // '$' 'cast'
     || lk == 46623                 // '$' 'castable'
     || lk == 47135                 // '$' 'catch'
     || lk == 48159                 // '$' 'child'
     || lk == 48671                 // '$' 'collation'
     || lk == 49695                 // '$' 'comment'
     || lk == 50207                 // '$' 'constraint'
     || lk == 50719                 // '$' 'construction'
     || lk == 52255                 // '$' 'context'
     || lk == 52767                 // '$' 'continue'
     || lk == 53279                 // '$' 'copy'
     || lk == 53791                 // '$' 'copy-namespaces'
     || lk == 54303                 // '$' 'count'
     || lk == 54815                 // '$' 'decimal-format'
     || lk == 55839                 // '$' 'declare'
     || lk == 56351                 // '$' 'default'
     || lk == 56863                 // '$' 'delete'
     || lk == 57375                 // '$' 'descendant'
     || lk == 57887                 // '$' 'descendant-or-self'
     || lk == 58399                 // '$' 'descending'
     || lk == 60959                 // '$' 'div'
     || lk == 61471                 // '$' 'document'
     || lk == 61983                 // '$' 'document-node'
     || lk == 62495                 // '$' 'element'
     || lk == 63007                 // '$' 'else'
     || lk == 63519                 // '$' 'empty'
     || lk == 64031                 // '$' 'empty-sequence'
     || lk == 64543                 // '$' 'encoding'
     || lk == 65055                 // '$' 'end'
     || lk == 66079                 // '$' 'eq'
     || lk == 66591                 // '$' 'every'
     || lk == 67615                 // '$' 'except'
     || lk == 68127                 // '$' 'exit'
     || lk == 68639                 // '$' 'external'
     || lk == 69151                 // '$' 'false'
     || lk == 69663                 // '$' 'first'
     || lk == 70175                 // '$' 'following'
     || lk == 70687                 // '$' 'following-sibling'
     || lk == 71199                 // '$' 'for'
     || lk == 72735                 // '$' 'from'
     || lk == 73247                 // '$' 'ft-option'
     || lk == 75295                 // '$' 'function'
     || lk == 75807                 // '$' 'ge'
     || lk == 76831                 // '$' 'group'
     || lk == 77855                 // '$' 'gt'
     || lk == 78367                 // '$' 'idiv'
     || lk == 78879                 // '$' 'if'
     || lk == 79391                 // '$' 'import'
     || lk == 79903                 // '$' 'in'
     || lk == 80415                 // '$' 'index'
     || lk == 82463                 // '$' 'insert'
     || lk == 82975                 // '$' 'instance'
     || lk == 83487                 // '$' 'integrity'
     || lk == 83999                 // '$' 'intersect'
     || lk == 84511                 // '$' 'into'
     || lk == 85023                 // '$' 'is'
     || lk == 85535                 // '$' 'item'
     || lk == 86047                 // '$' 'json'
     || lk == 86559                 // '$' 'json-item'
     || lk == 87071                 // '$' 'jsoniq'
     || lk == 88607                 // '$' 'last'
     || lk == 89119                 // '$' 'lax'
     || lk == 89631                 // '$' 'le'
     || lk == 90655                 // '$' 'let'
     || lk == 91679                 // '$' 'loop'
     || lk == 92703                 // '$' 'lt'
     || lk == 93727                 // '$' 'mod'
     || lk == 94239                 // '$' 'modify'
     || lk == 94751                 // '$' 'module'
     || lk == 95775                 // '$' 'namespace'
     || lk == 96287                 // '$' 'namespace-node'
     || lk == 96799                 // '$' 'ne'
     || lk == 99359                 // '$' 'node'
     || lk == 99871                 // '$' 'nodes'
     || lk == 100895                // '$' 'null'
     || lk == 101407                // '$' 'object'
     || lk == 103455                // '$' 'only'
     || lk == 103967                // '$' 'option'
     || lk == 104479                // '$' 'or'
     || lk == 104991                // '$' 'order'
     || lk == 105503                // '$' 'ordered'
     || lk == 106015                // '$' 'ordering'
     || lk == 107551                // '$' 'parent'
     || lk == 110623                // '$' 'preceding'
     || lk == 111135                // '$' 'preceding-sibling'
     || lk == 112671                // '$' 'processing-instruction'
     || lk == 113695                // '$' 'rename'
     || lk == 114207                // '$' 'replace'
     || lk == 114719                // '$' 'return'
     || lk == 115231                // '$' 'returning'
     || lk == 115743                // '$' 'revalidation'
     || lk == 116767                // '$' 'satisfies'
     || lk == 117279                // '$' 'schema'
     || lk == 117791                // '$' 'schema-attribute'
     || lk == 118303                // '$' 'schema-element'
     || lk == 118815                // '$' 'score'
     || lk == 119327                // '$' 'select'
     || lk == 119839                // '$' 'self'
     || lk == 122399                // '$' 'sliding'
     || lk == 122911                // '$' 'some'
     || lk == 123423                // '$' 'stable'
     || lk == 123935                // '$' 'start'
     || lk == 125471                // '$' 'strict'
     || lk == 126495                // '$' 'structured-item'
     || lk == 127007                // '$' 'switch'
     || lk == 127519                // '$' 'text'
     || lk == 129567                // '$' 'to'
     || lk == 130079                // '$' 'treat'
     || lk == 130591                // '$' 'true'
     || lk == 131103                // '$' 'try'
     || lk == 131615                // '$' 'tumbling'
     || lk == 132127                // '$' 'type'
     || lk == 132639                // '$' 'typeswitch'
     || lk == 133151                // '$' 'union'
     || lk == 134175                // '$' 'unordered'
     || lk == 134687                // '$' 'updating'
     || lk == 136223                // '$' 'validate'
     || lk == 136735                // '$' 'value'
     || lk == 137247                // '$' 'variable'
     || lk == 137759                // '$' 'version'
     || lk == 139295                // '$' 'where'
     || lk == 139807                // '$' 'while'
     || lk == 141343)               // '$' 'with'
    {
      lk = memoized(2, e0);
      if (lk == 0)
      {
        var b0A = b0; var e0A = e0; var l1A = l1;
        var b1A = b1; var e1A = e1; var l2A = l2;
        var b2A = b2; var e2A = e2;
        try
        {
          try_GroupingVariable();
          lookahead1W(206);         // S^WS | '(:' | ',' | ':=' | 'as' | 'collation' | 'count' | 'for' | 'group' |
          if (l1 == 53              // ':='
           || l1 == 80)             // 'as'
          {
            if (l1 == 80)           // 'as'
            {
              try_TypeDeclaration();
            }
            lookahead1W(28);        // S^WS | '(:' | ':='
            shiftT(53);             // ':='
            lookahead1W(267);       // EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral | StringLiteral |
            try_ExprSingle();
          }
          if (l1 == 95)             // 'collation'
          {
            shiftT(95);             // 'collation'
            lookahead1W(15);        // URILiteral | S^WS | '(:'
            shiftT(7);              // URILiteral
          }
          memoize(2, e0A, -1);
          lk = -3;
        }
        catch (p1A)
        {
          lk = -2;
          b0 = b0A; e0 = e0A; l1 = l1A; if (l1 == 0) {end = e0A;} else {
          b1 = b1A; e1 = e1A; l2 = l2A; if (l2 == 0) {end = e1A;} else {
          b2 = b2A; e2 = e2A; end = e2A; }}
          memoize(2, e0A, -2);
        }
      }
    }
    switch (lk)
    {
    case -1:
      try_GroupingVariable();
      lookahead1W(206);             // S^WS | '(:' | ',' | ':=' | 'as' | 'collation' | 'count' | 'for' | 'group' |
      if (l1 == 53                  // ':='
       || l1 == 80)                 // 'as'
      {
        if (l1 == 80)               // 'as'
        {
          try_TypeDeclaration();
        }
        lookahead1W(28);            // S^WS | '(:' | ':='
        shiftT(53);                 // ':='
        lookahead1W(267);           // EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral | StringLiteral |
        try_ExprSingle();
      }
      if (l1 == 95)                 // 'collation'
      {
        shiftT(95);                 // 'collation'
        lookahead1W(15);            // URILiteral | S^WS | '(:'
        shiftT(7);                  // URILiteral
      }
      break;
    case -3:
      break;
    default:
      try_ExprSingle();
    }
  }

  function parse_GroupingVariable()
  {
    eventHandler.startNonterminal("GroupingVariable", e0);
    shift(31);                      // '$'
    lookahead1W(246);               // EQName^Token | S^WS | '(:' | 'after' | 'allowing' | 'ancestor' |
    whitespace();
    parse_VarName();
    eventHandler.endNonterminal("GroupingVariable", e0);
  }

  function try_GroupingVariable()
  {
    shiftT(31);                     // '$'
    lookahead1W(246);               // EQName^Token | S^WS | '(:' | 'after' | 'allowing' | 'ancestor' |
    try_VarName();
  }

  function parse_OrderByClause()
  {
    eventHandler.startNonterminal("OrderByClause", e0);
    switch (l1)
    {
    case 205:                       // 'order'
      shift(205);                   // 'order'
      lookahead1W(37);              // S^WS | '(:' | 'by'
      shift(88);                    // 'by'
      break;
    default:
      shift(241);                   // 'stable'
      lookahead1W(70);              // S^WS | '(:' | 'order'
      shift(205);                   // 'order'
      lookahead1W(37);              // S^WS | '(:' | 'by'
      shift(88);                    // 'by'
    }
    lookahead1W(267);               // EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral | StringLiteral |
    whitespace();
    parse_OrderSpecList();
    eventHandler.endNonterminal("OrderByClause", e0);
  }

  function try_OrderByClause()
  {
    switch (l1)
    {
    case 205:                       // 'order'
      shiftT(205);                  // 'order'
      lookahead1W(37);              // S^WS | '(:' | 'by'
      shiftT(88);                   // 'by'
      break;
    default:
      shiftT(241);                  // 'stable'
      lookahead1W(70);              // S^WS | '(:' | 'order'
      shiftT(205);                  // 'order'
      lookahead1W(37);              // S^WS | '(:' | 'by'
      shiftT(88);                   // 'by'
    }
    lookahead1W(267);               // EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral | StringLiteral |
    try_OrderSpecList();
  }

  function parse_OrderSpecList()
  {
    eventHandler.startNonterminal("OrderSpecList", e0);
    parse_OrderSpec();
    for (;;)
    {
      lookahead1W(198);             // S^WS | '(:' | ',' | 'count' | 'for' | 'group' | 'let' | 'order' | 'return' |
      if (l1 != 42)                 // ','
      {
        break;
      }
      shift(42);                    // ','
      lookahead1W(267);             // EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral | StringLiteral |
      whitespace();
      parse_OrderSpec();
    }
    eventHandler.endNonterminal("OrderSpecList", e0);
  }

  function try_OrderSpecList()
  {
    try_OrderSpec();
    for (;;)
    {
      lookahead1W(198);             // S^WS | '(:' | ',' | 'count' | 'for' | 'group' | 'let' | 'order' | 'return' |
      if (l1 != 42)                 // ','
      {
        break;
      }
      shiftT(42);                   // ','
      lookahead1W(267);             // EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral | StringLiteral |
      try_OrderSpec();
    }
  }

  function parse_OrderSpec()
  {
    eventHandler.startNonterminal("OrderSpec", e0);
    parse_ExprSingle();
    whitespace();
    parse_OrderModifier();
    eventHandler.endNonterminal("OrderSpec", e0);
  }

  function try_OrderSpec()
  {
    try_ExprSingle();
    try_OrderModifier();
  }

  function parse_OrderModifier()
  {
    eventHandler.startNonterminal("OrderModifier", e0);
    if (l1 == 81                    // 'ascending'
     || l1 == 114)                  // 'descending'
    {
      switch (l1)
      {
      case 81:                      // 'ascending'
        shift(81);                  // 'ascending'
        break;
      default:
        shift(114);                 // 'descending'
      }
    }
    lookahead1W(203);               // S^WS | '(:' | ',' | 'collation' | 'count' | 'empty' | 'for' | 'group' | 'let' |
    if (l1 == 124)                  // 'empty'
    {
      shift(124);                   // 'empty'
      lookahead1W(125);             // S^WS | '(:' | 'greatest' | 'least'
      switch (l1)
      {
      case 149:                     // 'greatest'
        shift(149);                 // 'greatest'
        break;
      default:
        shift(176);                 // 'least'
      }
    }
    lookahead1W(199);               // S^WS | '(:' | ',' | 'collation' | 'count' | 'for' | 'group' | 'let' | 'order' |
    if (l1 == 95)                   // 'collation'
    {
      shift(95);                    // 'collation'
      lookahead1W(15);              // URILiteral | S^WS | '(:'
      shift(7);                     // URILiteral
    }
    eventHandler.endNonterminal("OrderModifier", e0);
  }

  function try_OrderModifier()
  {
    if (l1 == 81                    // 'ascending'
     || l1 == 114)                  // 'descending'
    {
      switch (l1)
      {
      case 81:                      // 'ascending'
        shiftT(81);                 // 'ascending'
        break;
      default:
        shiftT(114);                // 'descending'
      }
    }
    lookahead1W(203);               // S^WS | '(:' | ',' | 'collation' | 'count' | 'empty' | 'for' | 'group' | 'let' |
    if (l1 == 124)                  // 'empty'
    {
      shiftT(124);                  // 'empty'
      lookahead1W(125);             // S^WS | '(:' | 'greatest' | 'least'
      switch (l1)
      {
      case 149:                     // 'greatest'
        shiftT(149);                // 'greatest'
        break;
      default:
        shiftT(176);                // 'least'
      }
    }
    lookahead1W(199);               // S^WS | '(:' | ',' | 'collation' | 'count' | 'for' | 'group' | 'let' | 'order' |
    if (l1 == 95)                   // 'collation'
    {
      shiftT(95);                   // 'collation'
      lookahead1W(15);              // URILiteral | S^WS | '(:'
      shiftT(7);                    // URILiteral
    }
  }

  function parse_ReturnClause()
  {
    eventHandler.startNonterminal("ReturnClause", e0);
    shift(224);                     // 'return'
    lookahead1W(267);               // EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral | StringLiteral |
    whitespace();
    parse_ExprSingle();
    eventHandler.endNonterminal("ReturnClause", e0);
  }

  function try_ReturnClause()
  {
    shiftT(224);                    // 'return'
    lookahead1W(267);               // EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral | StringLiteral |
    try_ExprSingle();
  }

  function parse_QuantifiedExpr()
  {
    eventHandler.startNonterminal("QuantifiedExpr", e0);
    switch (l1)
    {
    case 240:                       // 'some'
      shift(240);                   // 'some'
      break;
    default:
      shift(130);                   // 'every'
    }
    lookahead1W(21);                // S^WS | '$' | '(:'
    whitespace();
    parse_QuantifiedVarDecl();
    for (;;)
    {
      if (l1 != 42)                 // ','
      {
        break;
      }
      shift(42);                    // ','
      lookahead1W(21);              // S^WS | '$' | '(:'
      whitespace();
      parse_QuantifiedVarDecl();
    }
    shift(228);                     // 'satisfies'
    lookahead1W(267);               // EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral | StringLiteral |
    whitespace();
    parse_ExprSingle();
    eventHandler.endNonterminal("QuantifiedExpr", e0);
  }

  function try_QuantifiedExpr()
  {
    switch (l1)
    {
    case 240:                       // 'some'
      shiftT(240);                  // 'some'
      break;
    default:
      shiftT(130);                  // 'every'
    }
    lookahead1W(21);                // S^WS | '$' | '(:'
    try_QuantifiedVarDecl();
    for (;;)
    {
      if (l1 != 42)                 // ','
      {
        break;
      }
      shiftT(42);                   // ','
      lookahead1W(21);              // S^WS | '$' | '(:'
      try_QuantifiedVarDecl();
    }
    shiftT(228);                    // 'satisfies'
    lookahead1W(267);               // EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral | StringLiteral |
    try_ExprSingle();
  }

  function parse_QuantifiedVarDecl()
  {
    eventHandler.startNonterminal("QuantifiedVarDecl", e0);
    shift(31);                      // '$'
    lookahead1W(246);               // EQName^Token | S^WS | '(:' | 'after' | 'allowing' | 'ancestor' |
    whitespace();
    parse_VarName();
    lookahead1W(114);               // S^WS | '(:' | 'as' | 'in'
    if (l1 == 80)                   // 'as'
    {
      whitespace();
      parse_TypeDeclaration();
    }
    lookahead1W(56);                // S^WS | '(:' | 'in'
    shift(156);                     // 'in'
    lookahead1W(267);               // EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral | StringLiteral |
    whitespace();
    parse_ExprSingle();
    eventHandler.endNonterminal("QuantifiedVarDecl", e0);
  }

  function try_QuantifiedVarDecl()
  {
    shiftT(31);                     // '$'
    lookahead1W(246);               // EQName^Token | S^WS | '(:' | 'after' | 'allowing' | 'ancestor' |
    try_VarName();
    lookahead1W(114);               // S^WS | '(:' | 'as' | 'in'
    if (l1 == 80)                   // 'as'
    {
      try_TypeDeclaration();
    }
    lookahead1W(56);                // S^WS | '(:' | 'in'
    shiftT(156);                    // 'in'
    lookahead1W(267);               // EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral | StringLiteral |
    try_ExprSingle();
  }

  function parse_SwitchExpr()
  {
    eventHandler.startNonterminal("SwitchExpr", e0);
    shift(248);                     // 'switch'
    lookahead1W(22);                // S^WS | '(' | '(:'
    shift(35);                      // '('
    lookahead1W(267);               // EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral | StringLiteral |
    whitespace();
    parse_Expr();
    shift(38);                      // ')'
    for (;;)
    {
      lookahead1W(38);              // S^WS | '(:' | 'case'
      whitespace();
      parse_SwitchCaseClause();
      if (l1 != 89)                 // 'case'
      {
        break;
      }
    }
    shift(110);                     // 'default'
    lookahead1W(73);                // S^WS | '(:' | 'return'
    shift(224);                     // 'return'
    lookahead1W(267);               // EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral | StringLiteral |
    whitespace();
    parse_ExprSingle();
    eventHandler.endNonterminal("SwitchExpr", e0);
  }

  function try_SwitchExpr()
  {
    shiftT(248);                    // 'switch'
    lookahead1W(22);                // S^WS | '(' | '(:'
    shiftT(35);                     // '('
    lookahead1W(267);               // EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral | StringLiteral |
    try_Expr();
    shiftT(38);                     // ')'
    for (;;)
    {
      lookahead1W(38);              // S^WS | '(:' | 'case'
      try_SwitchCaseClause();
      if (l1 != 89)                 // 'case'
      {
        break;
      }
    }
    shiftT(110);                    // 'default'
    lookahead1W(73);                // S^WS | '(:' | 'return'
    shiftT(224);                    // 'return'
    lookahead1W(267);               // EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral | StringLiteral |
    try_ExprSingle();
  }

  function parse_SwitchCaseClause()
  {
    eventHandler.startNonterminal("SwitchCaseClause", e0);
    for (;;)
    {
      shift(89);                    // 'case'
      lookahead1W(267);             // EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral | StringLiteral |
      whitespace();
      parse_SwitchCaseOperand();
      if (l1 != 89)                 // 'case'
      {
        break;
      }
    }
    shift(224);                     // 'return'
    lookahead1W(267);               // EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral | StringLiteral |
    whitespace();
    parse_ExprSingle();
    eventHandler.endNonterminal("SwitchCaseClause", e0);
  }

  function try_SwitchCaseClause()
  {
    for (;;)
    {
      shiftT(89);                   // 'case'
      lookahead1W(267);             // EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral | StringLiteral |
      try_SwitchCaseOperand();
      if (l1 != 89)                 // 'case'
      {
        break;
      }
    }
    shiftT(224);                    // 'return'
    lookahead1W(267);               // EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral | StringLiteral |
    try_ExprSingle();
  }

  function parse_SwitchCaseOperand()
  {
    eventHandler.startNonterminal("SwitchCaseOperand", e0);
    parse_ExprSingle();
    eventHandler.endNonterminal("SwitchCaseOperand", e0);
  }

  function try_SwitchCaseOperand()
  {
    try_ExprSingle();
  }

  function parse_TypeswitchExpr()
  {
    eventHandler.startNonterminal("TypeswitchExpr", e0);
    shift(259);                     // 'typeswitch'
    lookahead1W(22);                // S^WS | '(' | '(:'
    shift(35);                      // '('
    lookahead1W(267);               // EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral | StringLiteral |
    whitespace();
    parse_Expr();
    shift(38);                      // ')'
    for (;;)
    {
      lookahead1W(38);              // S^WS | '(:' | 'case'
      whitespace();
      parse_CaseClause();
      if (l1 != 89)                 // 'case'
      {
        break;
      }
    }
    shift(110);                     // 'default'
    lookahead1W(99);                // S^WS | '$' | '(:' | 'return'
    if (l1 == 31)                   // '$'
    {
      shift(31);                    // '$'
      lookahead1W(246);             // EQName^Token | S^WS | '(:' | 'after' | 'allowing' | 'ancestor' |
      whitespace();
      parse_VarName();
    }
    lookahead1W(73);                // S^WS | '(:' | 'return'
    shift(224);                     // 'return'
    lookahead1W(267);               // EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral | StringLiteral |
    whitespace();
    parse_ExprSingle();
    eventHandler.endNonterminal("TypeswitchExpr", e0);
  }

  function try_TypeswitchExpr()
  {
    shiftT(259);                    // 'typeswitch'
    lookahead1W(22);                // S^WS | '(' | '(:'
    shiftT(35);                     // '('
    lookahead1W(267);               // EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral | StringLiteral |
    try_Expr();
    shiftT(38);                     // ')'
    for (;;)
    {
      lookahead1W(38);              // S^WS | '(:' | 'case'
      try_CaseClause();
      if (l1 != 89)                 // 'case'
      {
        break;
      }
    }
    shiftT(110);                    // 'default'
    lookahead1W(99);                // S^WS | '$' | '(:' | 'return'
    if (l1 == 31)                   // '$'
    {
      shiftT(31);                   // '$'
      lookahead1W(246);             // EQName^Token | S^WS | '(:' | 'after' | 'allowing' | 'ancestor' |
      try_VarName();
    }
    lookahead1W(73);                // S^WS | '(:' | 'return'
    shiftT(224);                    // 'return'
    lookahead1W(267);               // EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral | StringLiteral |
    try_ExprSingle();
  }

  function parse_CaseClause()
  {
    eventHandler.startNonterminal("CaseClause", e0);
    shift(89);                      // 'case'
    lookahead1W(258);               // EQName^Token | S^WS | '$' | '%' | '(' | '(:' | 'after' | 'allowing' |
    if (l1 == 31)                   // '$'
    {
      shift(31);                    // '$'
      lookahead1W(246);             // EQName^Token | S^WS | '(:' | 'after' | 'allowing' | 'ancestor' |
      whitespace();
      parse_VarName();
      lookahead1W(33);              // S^WS | '(:' | 'as'
      shift(80);                    // 'as'
    }
    lookahead1W(254);               // EQName^Token | S^WS | '%' | '(' | '(:' | 'after' | 'allowing' | 'ancestor' |
    whitespace();
    parse_SequenceTypeUnion();
    shift(224);                     // 'return'
    lookahead1W(267);               // EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral | StringLiteral |
    whitespace();
    parse_ExprSingle();
    eventHandler.endNonterminal("CaseClause", e0);
  }

  function try_CaseClause()
  {
    shiftT(89);                     // 'case'
    lookahead1W(258);               // EQName^Token | S^WS | '$' | '%' | '(' | '(:' | 'after' | 'allowing' |
    if (l1 == 31)                   // '$'
    {
      shiftT(31);                   // '$'
      lookahead1W(246);             // EQName^Token | S^WS | '(:' | 'after' | 'allowing' | 'ancestor' |
      try_VarName();
      lookahead1W(33);              // S^WS | '(:' | 'as'
      shiftT(80);                   // 'as'
    }
    lookahead1W(254);               // EQName^Token | S^WS | '%' | '(' | '(:' | 'after' | 'allowing' | 'ancestor' |
    try_SequenceTypeUnion();
    shiftT(224);                    // 'return'
    lookahead1W(267);               // EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral | StringLiteral |
    try_ExprSingle();
  }

  function parse_SequenceTypeUnion()
  {
    eventHandler.startNonterminal("SequenceTypeUnion", e0);
    parse_SequenceType();
    for (;;)
    {
      lookahead1W(138);             // S^WS | '(:' | 'return' | '|'
      if (l1 != 284)                // '|'
      {
        break;
      }
      shift(284);                   // '|'
      lookahead1W(254);             // EQName^Token | S^WS | '%' | '(' | '(:' | 'after' | 'allowing' | 'ancestor' |
      whitespace();
      parse_SequenceType();
    }
    eventHandler.endNonterminal("SequenceTypeUnion", e0);
  }

  function try_SequenceTypeUnion()
  {
    try_SequenceType();
    for (;;)
    {
      lookahead1W(138);             // S^WS | '(:' | 'return' | '|'
      if (l1 != 284)                // '|'
      {
        break;
      }
      shiftT(284);                  // '|'
      lookahead1W(254);             // EQName^Token | S^WS | '%' | '(' | '(:' | 'after' | 'allowing' | 'ancestor' |
      try_SequenceType();
    }
  }

  function parse_IfExpr()
  {
    eventHandler.startNonterminal("IfExpr", e0);
    shift(154);                     // 'if'
    lookahead1W(22);                // S^WS | '(' | '(:'
    shift(35);                      // '('
    lookahead1W(267);               // EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral | StringLiteral |
    whitespace();
    parse_Expr();
    shift(38);                      // ')'
    lookahead1W(80);                // S^WS | '(:' | 'then'
    shift(250);                     // 'then'
    lookahead1W(267);               // EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral | StringLiteral |
    whitespace();
    parse_ExprSingle();
    shift(123);                     // 'else'
    lookahead1W(267);               // EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral | StringLiteral |
    whitespace();
    parse_ExprSingle();
    eventHandler.endNonterminal("IfExpr", e0);
  }

  function try_IfExpr()
  {
    shiftT(154);                    // 'if'
    lookahead1W(22);                // S^WS | '(' | '(:'
    shiftT(35);                     // '('
    lookahead1W(267);               // EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral | StringLiteral |
    try_Expr();
    shiftT(38);                     // ')'
    lookahead1W(80);                // S^WS | '(:' | 'then'
    shiftT(250);                    // 'then'
    lookahead1W(267);               // EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral | StringLiteral |
    try_ExprSingle();
    shiftT(123);                    // 'else'
    lookahead1W(267);               // EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral | StringLiteral |
    try_ExprSingle();
  }

  function parse_TryCatchExpr()
  {
    eventHandler.startNonterminal("TryCatchExpr", e0);
    parse_TryClause();
    for (;;)
    {
      lookahead1W(39);              // S^WS | '(:' | 'catch'
      whitespace();
      parse_CatchClause();
      lookahead1W(208);             // S^WS | EOF | '(:' | ')' | ',' | ':' | ';' | ']' | 'after' | 'as' | 'ascending' |
      if (l1 != 92)                 // 'catch'
      {
        break;
      }
    }
    eventHandler.endNonterminal("TryCatchExpr", e0);
  }

  function try_TryCatchExpr()
  {
    try_TryClause();
    for (;;)
    {
      lookahead1W(39);              // S^WS | '(:' | 'catch'
      try_CatchClause();
      lookahead1W(208);             // S^WS | EOF | '(:' | ')' | ',' | ':' | ';' | ']' | 'after' | 'as' | 'ascending' |
      if (l1 != 92)                 // 'catch'
      {
        break;
      }
    }
  }

  function parse_TryClause()
  {
    eventHandler.startNonterminal("TryClause", e0);
    shift(256);                     // 'try'
    lookahead1W(90);                // S^WS | '(:' | '{'
    shift(281);                     // '{'
    lookahead1W(267);               // EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral | StringLiteral |
    whitespace();
    parse_TryTargetExpr();
    shift(287);                     // '}'
    eventHandler.endNonterminal("TryClause", e0);
  }

  function try_TryClause()
  {
    shiftT(256);                    // 'try'
    lookahead1W(90);                // S^WS | '(:' | '{'
    shiftT(281);                    // '{'
    lookahead1W(267);               // EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral | StringLiteral |
    try_TryTargetExpr();
    shiftT(287);                    // '}'
  }

  function parse_TryTargetExpr()
  {
    eventHandler.startNonterminal("TryTargetExpr", e0);
    parse_Expr();
    eventHandler.endNonterminal("TryTargetExpr", e0);
  }

  function try_TryTargetExpr()
  {
    try_Expr();
  }

  function parse_CatchClause()
  {
    eventHandler.startNonterminal("CatchClause", e0);
    shift(92);                      // 'catch'
    lookahead1W(249);               // Wildcard | EQName^Token | S^WS | '(:' | 'after' | 'allowing' | 'ancestor' |
    whitespace();
    parse_CatchErrorList();
    shift(281);                     // '{'
    lookahead1W(267);               // EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral | StringLiteral |
    whitespace();
    parse_Expr();
    shift(287);                     // '}'
    eventHandler.endNonterminal("CatchClause", e0);
  }

  function try_CatchClause()
  {
    shiftT(92);                     // 'catch'
    lookahead1W(249);               // Wildcard | EQName^Token | S^WS | '(:' | 'after' | 'allowing' | 'ancestor' |
    try_CatchErrorList();
    shiftT(281);                    // '{'
    lookahead1W(267);               // EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral | StringLiteral |
    try_Expr();
    shiftT(287);                    // '}'
  }

  function parse_CatchErrorList()
  {
    eventHandler.startNonterminal("CatchErrorList", e0);
    parse_NameTest();
    for (;;)
    {
      lookahead1W(140);             // S^WS | '(:' | '{' | '|'
      if (l1 != 284)                // '|'
      {
        break;
      }
      shift(284);                   // '|'
      lookahead1W(249);             // Wildcard | EQName^Token | S^WS | '(:' | 'after' | 'allowing' | 'ancestor' |
      whitespace();
      parse_NameTest();
    }
    eventHandler.endNonterminal("CatchErrorList", e0);
  }

  function try_CatchErrorList()
  {
    try_NameTest();
    for (;;)
    {
      lookahead1W(140);             // S^WS | '(:' | '{' | '|'
      if (l1 != 284)                // '|'
      {
        break;
      }
      shiftT(284);                  // '|'
      lookahead1W(249);             // Wildcard | EQName^Token | S^WS | '(:' | 'after' | 'allowing' | 'ancestor' |
      try_NameTest();
    }
  }

  function parse_OrExpr()
  {
    eventHandler.startNonterminal("OrExpr", e0);
    parse_AndExpr();
    for (;;)
    {
      if (l1 != 204)                // 'or'
      {
        break;
      }
      shift(204);                   // 'or'
      lookahead1W(267);             // EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral | StringLiteral |
      whitespace();
      parse_AndExpr();
    }
    eventHandler.endNonterminal("OrExpr", e0);
  }

  function try_OrExpr()
  {
    try_AndExpr();
    for (;;)
    {
      if (l1 != 204)                // 'or'
      {
        break;
      }
      shiftT(204);                  // 'or'
      lookahead1W(267);             // EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral | StringLiteral |
      try_AndExpr();
    }
  }

  function parse_AndExpr()
  {
    eventHandler.startNonterminal("AndExpr", e0);
    parse_NotExpr();
    for (;;)
    {
      if (l1 != 76)                 // 'and'
      {
        break;
      }
      shift(76);                    // 'and'
      lookahead1W(267);             // EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral | StringLiteral |
      whitespace();
      parse_NotExpr();
    }
    eventHandler.endNonterminal("AndExpr", e0);
  }

  function try_AndExpr()
  {
    try_NotExpr();
    for (;;)
    {
      if (l1 != 76)                 // 'and'
      {
        break;
      }
      shiftT(76);                   // 'and'
      lookahead1W(267);             // EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral | StringLiteral |
      try_NotExpr();
    }
  }

  function parse_NotExpr()
  {
    eventHandler.startNonterminal("NotExpr", e0);
    if (l1 == 196)                  // 'not'
    {
      shift(196);                   // 'not'
    }
    lookahead1W(266);               // EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral | StringLiteral |
    whitespace();
    parse_ComparisonExpr();
    eventHandler.endNonterminal("NotExpr", e0);
  }

  function try_NotExpr()
  {
    if (l1 == 196)                  // 'not'
    {
      shiftT(196);                  // 'not'
    }
    lookahead1W(266);               // EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral | StringLiteral |
    try_ComparisonExpr();
  }

  function parse_ComparisonExpr()
  {
    eventHandler.startNonterminal("ComparisonExpr", e0);
    parse_FTContainsExpr();
    if (l1 == 27                    // '!='
     || l1 == 55                    // '<'
     || l1 == 58                    // '<<'
     || l1 == 59                    // '<='
     || l1 == 61                    // '='
     || l1 == 62                    // '>'
     || l1 == 63                    // '>='
     || l1 == 64                    // '>>'
     || l1 == 129                   // 'eq'
     || l1 == 148                   // 'ge'
     || l1 == 152                   // 'gt'
     || l1 == 166                   // 'is'
     || l1 == 175                   // 'le'
     || l1 == 181                   // 'lt'
     || l1 == 189)                  // 'ne'
    {
      switch (l1)
      {
      case 129:                     // 'eq'
      case 148:                     // 'ge'
      case 152:                     // 'gt'
      case 175:                     // 'le'
      case 181:                     // 'lt'
      case 189:                     // 'ne'
        whitespace();
        parse_ValueComp();
        break;
      case 58:                      // '<<'
      case 64:                      // '>>'
      case 166:                     // 'is'
        whitespace();
        parse_NodeComp();
        break;
      default:
        whitespace();
        parse_GeneralComp();
      }
      lookahead1W(266);             // EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral | StringLiteral |
      whitespace();
      parse_FTContainsExpr();
    }
    eventHandler.endNonterminal("ComparisonExpr", e0);
  }

  function try_ComparisonExpr()
  {
    try_FTContainsExpr();
    if (l1 == 27                    // '!='
     || l1 == 55                    // '<'
     || l1 == 58                    // '<<'
     || l1 == 59                    // '<='
     || l1 == 61                    // '='
     || l1 == 62                    // '>'
     || l1 == 63                    // '>='
     || l1 == 64                    // '>>'
     || l1 == 129                   // 'eq'
     || l1 == 148                   // 'ge'
     || l1 == 152                   // 'gt'
     || l1 == 166                   // 'is'
     || l1 == 175                   // 'le'
     || l1 == 181                   // 'lt'
     || l1 == 189)                  // 'ne'
    {
      switch (l1)
      {
      case 129:                     // 'eq'
      case 148:                     // 'ge'
      case 152:                     // 'gt'
      case 175:                     // 'le'
      case 181:                     // 'lt'
      case 189:                     // 'ne'
        try_ValueComp();
        break;
      case 58:                      // '<<'
      case 64:                      // '>>'
      case 166:                     // 'is'
        try_NodeComp();
        break;
      default:
        try_GeneralComp();
      }
      lookahead1W(266);             // EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral | StringLiteral |
      try_FTContainsExpr();
    }
  }

  function parse_FTContainsExpr()
  {
    eventHandler.startNonterminal("FTContainsExpr", e0);
    parse_StringConcatExpr();
    if (l1 == 100)                  // 'contains'
    {
      shift(100);                   // 'contains'
      lookahead1W(79);              // S^WS | '(:' | 'text'
      shift(249);                   // 'text'
      lookahead1W(177);             // StringLiteral | S^WS | '(' | '(#' | '(:' | 'ftnot' | '{'
      whitespace();
      parse_FTSelection();
      if (l1 == 277)                // 'without'
      {
        whitespace();
        parse_FTIgnoreOption();
      }
    }
    eventHandler.endNonterminal("FTContainsExpr", e0);
  }

  function try_FTContainsExpr()
  {
    try_StringConcatExpr();
    if (l1 == 100)                  // 'contains'
    {
      shiftT(100);                  // 'contains'
      lookahead1W(79);              // S^WS | '(:' | 'text'
      shiftT(249);                  // 'text'
      lookahead1W(177);             // StringLiteral | S^WS | '(' | '(#' | '(:' | 'ftnot' | '{'
      try_FTSelection();
      if (l1 == 277)                // 'without'
      {
        try_FTIgnoreOption();
      }
    }
  }

  function parse_StringConcatExpr()
  {
    eventHandler.startNonterminal("StringConcatExpr", e0);
    parse_RangeExpr();
    for (;;)
    {
      if (l1 != 285)                // '||'
      {
        break;
      }
      shift(285);                   // '||'
      lookahead1W(266);             // EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral | StringLiteral |
      whitespace();
      parse_RangeExpr();
    }
    eventHandler.endNonterminal("StringConcatExpr", e0);
  }

  function try_StringConcatExpr()
  {
    try_RangeExpr();
    for (;;)
    {
      if (l1 != 285)                // '||'
      {
        break;
      }
      shiftT(285);                  // '||'
      lookahead1W(266);             // EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral | StringLiteral |
      try_RangeExpr();
    }
  }

  function parse_RangeExpr()
  {
    eventHandler.startNonterminal("RangeExpr", e0);
    parse_AdditiveExpr();
    if (l1 == 253)                  // 'to'
    {
      shift(253);                   // 'to'
      lookahead1W(266);             // EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral | StringLiteral |
      whitespace();
      parse_AdditiveExpr();
    }
    eventHandler.endNonterminal("RangeExpr", e0);
  }

  function try_RangeExpr()
  {
    try_AdditiveExpr();
    if (l1 == 253)                  // 'to'
    {
      shiftT(253);                  // 'to'
      lookahead1W(266);             // EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral | StringLiteral |
      try_AdditiveExpr();
    }
  }

  function parse_AdditiveExpr()
  {
    eventHandler.startNonterminal("AdditiveExpr", e0);
    parse_MultiplicativeExpr();
    for (;;)
    {
      if (l1 != 41                  // '+'
       && l1 != 43)                 // '-'
      {
        break;
      }
      switch (l1)
      {
      case 41:                      // '+'
        shift(41);                  // '+'
        break;
      default:
        shift(43);                  // '-'
      }
      lookahead1W(266);             // EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral | StringLiteral |
      whitespace();
      parse_MultiplicativeExpr();
    }
    eventHandler.endNonterminal("AdditiveExpr", e0);
  }

  function try_AdditiveExpr()
  {
    try_MultiplicativeExpr();
    for (;;)
    {
      if (l1 != 41                  // '+'
       && l1 != 43)                 // '-'
      {
        break;
      }
      switch (l1)
      {
      case 41:                      // '+'
        shiftT(41);                 // '+'
        break;
      default:
        shiftT(43);                 // '-'
      }
      lookahead1W(266);             // EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral | StringLiteral |
      try_MultiplicativeExpr();
    }
  }

  function parse_MultiplicativeExpr()
  {
    eventHandler.startNonterminal("MultiplicativeExpr", e0);
    parse_UnionExpr();
    for (;;)
    {
      if (l1 != 39                  // '*'
       && l1 != 119                 // 'div'
       && l1 != 153                 // 'idiv'
       && l1 != 183)                // 'mod'
      {
        break;
      }
      switch (l1)
      {
      case 39:                      // '*'
        shift(39);                  // '*'
        break;
      case 119:                     // 'div'
        shift(119);                 // 'div'
        break;
      case 153:                     // 'idiv'
        shift(153);                 // 'idiv'
        break;
      default:
        shift(183);                 // 'mod'
      }
      lookahead1W(266);             // EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral | StringLiteral |
      whitespace();
      parse_UnionExpr();
    }
    eventHandler.endNonterminal("MultiplicativeExpr", e0);
  }

  function try_MultiplicativeExpr()
  {
    try_UnionExpr();
    for (;;)
    {
      if (l1 != 39                  // '*'
       && l1 != 119                 // 'div'
       && l1 != 153                 // 'idiv'
       && l1 != 183)                // 'mod'
      {
        break;
      }
      switch (l1)
      {
      case 39:                      // '*'
        shiftT(39);                 // '*'
        break;
      case 119:                     // 'div'
        shiftT(119);                // 'div'
        break;
      case 153:                     // 'idiv'
        shiftT(153);                // 'idiv'
        break;
      default:
        shiftT(183);                // 'mod'
      }
      lookahead1W(266);             // EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral | StringLiteral |
      try_UnionExpr();
    }
  }

  function parse_UnionExpr()
  {
    eventHandler.startNonterminal("UnionExpr", e0);
    parse_IntersectExceptExpr();
    for (;;)
    {
      if (l1 != 260                 // 'union'
       && l1 != 284)                // '|'
      {
        break;
      }
      switch (l1)
      {
      case 260:                     // 'union'
        shift(260);                 // 'union'
        break;
      default:
        shift(284);                 // '|'
      }
      lookahead1W(266);             // EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral | StringLiteral |
      whitespace();
      parse_IntersectExceptExpr();
    }
    eventHandler.endNonterminal("UnionExpr", e0);
  }

  function try_UnionExpr()
  {
    try_IntersectExceptExpr();
    for (;;)
    {
      if (l1 != 260                 // 'union'
       && l1 != 284)                // '|'
      {
        break;
      }
      switch (l1)
      {
      case 260:                     // 'union'
        shiftT(260);                // 'union'
        break;
      default:
        shiftT(284);                // '|'
      }
      lookahead1W(266);             // EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral | StringLiteral |
      try_IntersectExceptExpr();
    }
  }

  function parse_IntersectExceptExpr()
  {
    eventHandler.startNonterminal("IntersectExceptExpr", e0);
    parse_InstanceofExpr();
    for (;;)
    {
      lookahead1W(222);             // S^WS | EOF | '!=' | '(:' | ')' | '*' | '+' | ',' | '-' | ':' | ';' | '<' | '<<' |
      if (l1 != 132                 // 'except'
       && l1 != 164)                // 'intersect'
      {
        break;
      }
      switch (l1)
      {
      case 164:                     // 'intersect'
        shift(164);                 // 'intersect'
        break;
      default:
        shift(132);                 // 'except'
      }
      lookahead1W(266);             // EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral | StringLiteral |
      whitespace();
      parse_InstanceofExpr();
    }
    eventHandler.endNonterminal("IntersectExceptExpr", e0);
  }

  function try_IntersectExceptExpr()
  {
    try_InstanceofExpr();
    for (;;)
    {
      lookahead1W(222);             // S^WS | EOF | '!=' | '(:' | ')' | '*' | '+' | ',' | '-' | ':' | ';' | '<' | '<<' |
      if (l1 != 132                 // 'except'
       && l1 != 164)                // 'intersect'
      {
        break;
      }
      switch (l1)
      {
      case 164:                     // 'intersect'
        shiftT(164);                // 'intersect'
        break;
      default:
        shiftT(132);                // 'except'
      }
      lookahead1W(266);             // EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral | StringLiteral |
      try_InstanceofExpr();
    }
  }

  function parse_InstanceofExpr()
  {
    eventHandler.startNonterminal("InstanceofExpr", e0);
    parse_TreatExpr();
    lookahead1W(223);               // S^WS | EOF | '!=' | '(:' | ')' | '*' | '+' | ',' | '-' | ':' | ';' | '<' | '<<' |
    if (l1 == 162)                  // 'instance'
    {
      shift(162);                   // 'instance'
      lookahead1W(67);              // S^WS | '(:' | 'of'
      shift(200);                   // 'of'
      lookahead1W(254);             // EQName^Token | S^WS | '%' | '(' | '(:' | 'after' | 'allowing' | 'ancestor' |
      whitespace();
      parse_SequenceType();
    }
    eventHandler.endNonterminal("InstanceofExpr", e0);
  }

  function try_InstanceofExpr()
  {
    try_TreatExpr();
    lookahead1W(223);               // S^WS | EOF | '!=' | '(:' | ')' | '*' | '+' | ',' | '-' | ':' | ';' | '<' | '<<' |
    if (l1 == 162)                  // 'instance'
    {
      shiftT(162);                  // 'instance'
      lookahead1W(67);              // S^WS | '(:' | 'of'
      shiftT(200);                  // 'of'
      lookahead1W(254);             // EQName^Token | S^WS | '%' | '(' | '(:' | 'after' | 'allowing' | 'ancestor' |
      try_SequenceType();
    }
  }

  function parse_TreatExpr()
  {
    eventHandler.startNonterminal("TreatExpr", e0);
    parse_CastableExpr();
    lookahead1W(224);               // S^WS | EOF | '!=' | '(:' | ')' | '*' | '+' | ',' | '-' | ':' | ';' | '<' | '<<' |
    if (l1 == 254)                  // 'treat'
    {
      shift(254);                   // 'treat'
      lookahead1W(33);              // S^WS | '(:' | 'as'
      shift(80);                    // 'as'
      lookahead1W(254);             // EQName^Token | S^WS | '%' | '(' | '(:' | 'after' | 'allowing' | 'ancestor' |
      whitespace();
      parse_SequenceType();
    }
    eventHandler.endNonterminal("TreatExpr", e0);
  }

  function try_TreatExpr()
  {
    try_CastableExpr();
    lookahead1W(224);               // S^WS | EOF | '!=' | '(:' | ')' | '*' | '+' | ',' | '-' | ':' | ';' | '<' | '<<' |
    if (l1 == 254)                  // 'treat'
    {
      shiftT(254);                  // 'treat'
      lookahead1W(33);              // S^WS | '(:' | 'as'
      shiftT(80);                   // 'as'
      lookahead1W(254);             // EQName^Token | S^WS | '%' | '(' | '(:' | 'after' | 'allowing' | 'ancestor' |
      try_SequenceType();
    }
  }

  function parse_CastableExpr()
  {
    eventHandler.startNonterminal("CastableExpr", e0);
    parse_CastExpr();
    lookahead1W(225);               // S^WS | EOF | '!=' | '(:' | ')' | '*' | '+' | ',' | '-' | ':' | ';' | '<' | '<<' |
    if (l1 == 91)                   // 'castable'
    {
      shift(91);                    // 'castable'
      lookahead1W(33);              // S^WS | '(:' | 'as'
      shift(80);                    // 'as'
      lookahead1W(246);             // EQName^Token | S^WS | '(:' | 'after' | 'allowing' | 'ancestor' |
      whitespace();
      parse_SingleType();
    }
    eventHandler.endNonterminal("CastableExpr", e0);
  }

  function try_CastableExpr()
  {
    try_CastExpr();
    lookahead1W(225);               // S^WS | EOF | '!=' | '(:' | ')' | '*' | '+' | ',' | '-' | ':' | ';' | '<' | '<<' |
    if (l1 == 91)                   // 'castable'
    {
      shiftT(91);                   // 'castable'
      lookahead1W(33);              // S^WS | '(:' | 'as'
      shiftT(80);                   // 'as'
      lookahead1W(246);             // EQName^Token | S^WS | '(:' | 'after' | 'allowing' | 'ancestor' |
      try_SingleType();
    }
  }

  function parse_CastExpr()
  {
    eventHandler.startNonterminal("CastExpr", e0);
    parse_UnaryExpr();
    lookahead1W(227);               // S^WS | EOF | '!=' | '(:' | ')' | '*' | '+' | ',' | '-' | ':' | ';' | '<' | '<<' |
    if (l1 == 90)                   // 'cast'
    {
      shift(90);                    // 'cast'
      lookahead1W(33);              // S^WS | '(:' | 'as'
      shift(80);                    // 'as'
      lookahead1W(246);             // EQName^Token | S^WS | '(:' | 'after' | 'allowing' | 'ancestor' |
      whitespace();
      parse_SingleType();
    }
    eventHandler.endNonterminal("CastExpr", e0);
  }

  function try_CastExpr()
  {
    try_UnaryExpr();
    lookahead1W(227);               // S^WS | EOF | '!=' | '(:' | ')' | '*' | '+' | ',' | '-' | ':' | ';' | '<' | '<<' |
    if (l1 == 90)                   // 'cast'
    {
      shiftT(90);                   // 'cast'
      lookahead1W(33);              // S^WS | '(:' | 'as'
      shiftT(80);                   // 'as'
      lookahead1W(246);             // EQName^Token | S^WS | '(:' | 'after' | 'allowing' | 'ancestor' |
      try_SingleType();
    }
  }

  function parse_UnaryExpr()
  {
    eventHandler.startNonterminal("UnaryExpr", e0);
    for (;;)
    {
      lookahead1W(266);             // EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral | StringLiteral |
      if (l1 != 41                  // '+'
       && l1 != 43)                 // '-'
      {
        break;
      }
      switch (l1)
      {
      case 43:                      // '-'
        shift(43);                  // '-'
        break;
      default:
        shift(41);                  // '+'
      }
    }
    whitespace();
    parse_ValueExpr();
    eventHandler.endNonterminal("UnaryExpr", e0);
  }

  function try_UnaryExpr()
  {
    for (;;)
    {
      lookahead1W(266);             // EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral | StringLiteral |
      if (l1 != 41                  // '+'
       && l1 != 43)                 // '-'
      {
        break;
      }
      switch (l1)
      {
      case 43:                      // '-'
        shiftT(43);                 // '-'
        break;
      default:
        shiftT(41);                 // '+'
      }
    }
    try_ValueExpr();
  }

  function parse_ValueExpr()
  {
    eventHandler.startNonterminal("ValueExpr", e0);
    switch (l1)
    {
    case 266:                       // 'validate'
      lookahead2W(188);             // S^WS | '#' | '(' | '(:' | 'lax' | 'strict' | 'type' | '{'
      break;
    default:
      lk = l1;
    }
    switch (lk)
    {
    case 89354:                     // 'validate' 'lax'
    case 125706:                    // 'validate' 'strict'
    case 132362:                    // 'validate' 'type'
    case 144138:                    // 'validate' '{'
      parse_ValidateExpr();
      break;
    case 36:                        // '(#'
      parse_ExtensionExpr();
      break;
    default:
      parse_SimpleMapExpr();
    }
    eventHandler.endNonterminal("ValueExpr", e0);
  }

  function try_ValueExpr()
  {
    switch (l1)
    {
    case 266:                       // 'validate'
      lookahead2W(188);             // S^WS | '#' | '(' | '(:' | 'lax' | 'strict' | 'type' | '{'
      break;
    default:
      lk = l1;
    }
    switch (lk)
    {
    case 89354:                     // 'validate' 'lax'
    case 125706:                    // 'validate' 'strict'
    case 132362:                    // 'validate' 'type'
    case 144138:                    // 'validate' '{'
      try_ValidateExpr();
      break;
    case 36:                        // '(#'
      try_ExtensionExpr();
      break;
    default:
      try_SimpleMapExpr();
    }
  }

  function parse_SimpleMapExpr()
  {
    eventHandler.startNonterminal("SimpleMapExpr", e0);
    parse_PathExpr();
    for (;;)
    {
      if (l1 != 26)                 // '!'
      {
        break;
      }
      shift(26);                    // '!'
      lookahead1W(263);             // EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral | StringLiteral |
      whitespace();
      parse_PathExpr();
    }
    eventHandler.endNonterminal("SimpleMapExpr", e0);
  }

  function try_SimpleMapExpr()
  {
    try_PathExpr();
    for (;;)
    {
      if (l1 != 26)                 // '!'
      {
        break;
      }
      shiftT(26);                   // '!'
      lookahead1W(263);             // EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral | StringLiteral |
      try_PathExpr();
    }
  }

  function parse_GeneralComp()
  {
    eventHandler.startNonterminal("GeneralComp", e0);
    switch (l1)
    {
    case 61:                        // '='
      shift(61);                    // '='
      break;
    case 27:                        // '!='
      shift(27);                    // '!='
      break;
    case 55:                        // '<'
      shift(55);                    // '<'
      break;
    case 59:                        // '<='
      shift(59);                    // '<='
      break;
    case 62:                        // '>'
      shift(62);                    // '>'
      break;
    default:
      shift(63);                    // '>='
    }
    eventHandler.endNonterminal("GeneralComp", e0);
  }

  function try_GeneralComp()
  {
    switch (l1)
    {
    case 61:                        // '='
      shiftT(61);                   // '='
      break;
    case 27:                        // '!='
      shiftT(27);                   // '!='
      break;
    case 55:                        // '<'
      shiftT(55);                   // '<'
      break;
    case 59:                        // '<='
      shiftT(59);                   // '<='
      break;
    case 62:                        // '>'
      shiftT(62);                   // '>'
      break;
    default:
      shiftT(63);                   // '>='
    }
  }

  function parse_ValueComp()
  {
    eventHandler.startNonterminal("ValueComp", e0);
    switch (l1)
    {
    case 129:                       // 'eq'
      shift(129);                   // 'eq'
      break;
    case 189:                       // 'ne'
      shift(189);                   // 'ne'
      break;
    case 181:                       // 'lt'
      shift(181);                   // 'lt'
      break;
    case 175:                       // 'le'
      shift(175);                   // 'le'
      break;
    case 152:                       // 'gt'
      shift(152);                   // 'gt'
      break;
    default:
      shift(148);                   // 'ge'
    }
    eventHandler.endNonterminal("ValueComp", e0);
  }

  function try_ValueComp()
  {
    switch (l1)
    {
    case 129:                       // 'eq'
      shiftT(129);                  // 'eq'
      break;
    case 189:                       // 'ne'
      shiftT(189);                  // 'ne'
      break;
    case 181:                       // 'lt'
      shiftT(181);                  // 'lt'
      break;
    case 175:                       // 'le'
      shiftT(175);                  // 'le'
      break;
    case 152:                       // 'gt'
      shiftT(152);                  // 'gt'
      break;
    default:
      shiftT(148);                  // 'ge'
    }
  }

  function parse_NodeComp()
  {
    eventHandler.startNonterminal("NodeComp", e0);
    switch (l1)
    {
    case 166:                       // 'is'
      shift(166);                   // 'is'
      break;
    case 58:                        // '<<'
      shift(58);                    // '<<'
      break;
    default:
      shift(64);                    // '>>'
    }
    eventHandler.endNonterminal("NodeComp", e0);
  }

  function try_NodeComp()
  {
    switch (l1)
    {
    case 166:                       // 'is'
      shiftT(166);                  // 'is'
      break;
    case 58:                        // '<<'
      shiftT(58);                   // '<<'
      break;
    default:
      shiftT(64);                   // '>>'
    }
  }

  function parse_ValidateExpr()
  {
    eventHandler.startNonterminal("ValidateExpr", e0);
    shift(266);                     // 'validate'
    lookahead1W(175);               // S^WS | '(:' | 'lax' | 'strict' | 'type' | '{'
    if (l1 != 281)                  // '{'
    {
      switch (l1)
      {
      case 258:                     // 'type'
        shift(258);                 // 'type'
        lookahead1W(246);           // EQName^Token | S^WS | '(:' | 'after' | 'allowing' | 'ancestor' |
        whitespace();
        parse_TypeName();
        break;
      default:
        whitespace();
        parse_ValidationMode();
      }
    }
    lookahead1W(90);                // S^WS | '(:' | '{'
    shift(281);                     // '{'
    lookahead1W(267);               // EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral | StringLiteral |
    whitespace();
    parse_Expr();
    shift(287);                     // '}'
    eventHandler.endNonterminal("ValidateExpr", e0);
  }

  function try_ValidateExpr()
  {
    shiftT(266);                    // 'validate'
    lookahead1W(175);               // S^WS | '(:' | 'lax' | 'strict' | 'type' | '{'
    if (l1 != 281)                  // '{'
    {
      switch (l1)
      {
      case 258:                     // 'type'
        shiftT(258);                // 'type'
        lookahead1W(246);           // EQName^Token | S^WS | '(:' | 'after' | 'allowing' | 'ancestor' |
        try_TypeName();
        break;
      default:
        try_ValidationMode();
      }
    }
    lookahead1W(90);                // S^WS | '(:' | '{'
    shiftT(281);                    // '{'
    lookahead1W(267);               // EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral | StringLiteral |
    try_Expr();
    shiftT(287);                    // '}'
  }

  function parse_ValidationMode()
  {
    eventHandler.startNonterminal("ValidationMode", e0);
    switch (l1)
    {
    case 174:                       // 'lax'
      shift(174);                   // 'lax'
      break;
    default:
      shift(245);                   // 'strict'
    }
    eventHandler.endNonterminal("ValidationMode", e0);
  }

  function try_ValidationMode()
  {
    switch (l1)
    {
    case 174:                       // 'lax'
      shiftT(174);                  // 'lax'
      break;
    default:
      shiftT(245);                  // 'strict'
    }
  }

  function parse_ExtensionExpr()
  {
    eventHandler.startNonterminal("ExtensionExpr", e0);
    for (;;)
    {
      whitespace();
      parse_Pragma();
      lookahead1W(104);             // S^WS | '(#' | '(:' | '{'
      if (l1 != 36)                 // '(#'
      {
        break;
      }
    }
    shift(281);                     // '{'
    lookahead1W(275);               // EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral | StringLiteral |
    if (l1 != 287)                  // '}'
    {
      whitespace();
      parse_Expr();
    }
    shift(287);                     // '}'
    eventHandler.endNonterminal("ExtensionExpr", e0);
  }

  function try_ExtensionExpr()
  {
    for (;;)
    {
      try_Pragma();
      lookahead1W(104);             // S^WS | '(#' | '(:' | '{'
      if (l1 != 36)                 // '(#'
      {
        break;
      }
    }
    shiftT(281);                    // '{'
    lookahead1W(275);               // EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral | StringLiteral |
    if (l1 != 287)                  // '}'
    {
      try_Expr();
    }
    shiftT(287);                    // '}'
  }

  function parse_Pragma()
  {
    eventHandler.startNonterminal("Pragma", e0);
    shift(36);                      // '(#'
    lookahead1(243);                // EQName^Token | S | 'after' | 'allowing' | 'ancestor' | 'ancestor-or-self' |
    if (l1 == 21)                   // S
    {
      shift(21);                    // S
    }
    parse_EQName();
    lookahead1(10);                 // S | '#)'
    if (l1 == 21)                   // S
    {
      shift(21);                    // S
      lookahead1(0);                // PragmaContents
      shift(1);                     // PragmaContents
    }
    lookahead1(5);                  // '#)'
    shift(30);                      // '#)'
    eventHandler.endNonterminal("Pragma", e0);
  }

  function try_Pragma()
  {
    shiftT(36);                     // '(#'
    lookahead1(243);                // EQName^Token | S | 'after' | 'allowing' | 'ancestor' | 'ancestor-or-self' |
    if (l1 == 21)                   // S
    {
      shiftT(21);                   // S
    }
    try_EQName();
    lookahead1(10);                 // S | '#)'
    if (l1 == 21)                   // S
    {
      shiftT(21);                   // S
      lookahead1(0);                // PragmaContents
      shiftT(1);                    // PragmaContents
    }
    lookahead1(5);                  // '#)'
    shiftT(30);                     // '#)'
  }

  function parse_PathExpr()
  {
    eventHandler.startNonterminal("PathExpr", e0);
    switch (l1)
    {
    case 47:                        // '/'
      shift(47);                    // '/'
      lookahead1W(289);             // EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral | StringLiteral |
      switch (l1)
      {
      case 25:                      // EOF
      case 26:                      // '!'
      case 27:                      // '!='
      case 38:                      // ')'
      case 39:                      // '*'
      case 41:                      // '+'
      case 42:                      // ','
      case 43:                      // '-'
      case 50:                      // ':'
      case 54:                      // ';'
      case 58:                      // '<<'
      case 59:                      // '<='
      case 61:                      // '='
      case 62:                      // '>'
      case 63:                      // '>='
      case 64:                      // '>>'
      case 70:                      // ']'
      case 88:                      // 'by'
      case 100:                     // 'contains'
      case 209:                     // 'paragraphs'
      case 237:                     // 'sentences'
      case 252:                     // 'times'
      case 279:                     // 'words'
      case 284:                     // '|'
      case 285:                     // '||'
      case 286:                     // '|}'
      case 287:                     // '}'
        break;
      default:
        whitespace();
        parse_RelativePathExpr();
      }
      break;
    case 48:                        // '//'
      shift(48);                    // '//'
      lookahead1W(260);             // EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral | StringLiteral |
      whitespace();
      parse_RelativePathExpr();
      break;
    default:
      parse_RelativePathExpr();
    }
    eventHandler.endNonterminal("PathExpr", e0);
  }

  function try_PathExpr()
  {
    switch (l1)
    {
    case 47:                        // '/'
      shiftT(47);                   // '/'
      lookahead1W(289);             // EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral | StringLiteral |
      switch (l1)
      {
      case 25:                      // EOF
      case 26:                      // '!'
      case 27:                      // '!='
      case 38:                      // ')'
      case 39:                      // '*'
      case 41:                      // '+'
      case 42:                      // ','
      case 43:                      // '-'
      case 50:                      // ':'
      case 54:                      // ';'
      case 58:                      // '<<'
      case 59:                      // '<='
      case 61:                      // '='
      case 62:                      // '>'
      case 63:                      // '>='
      case 64:                      // '>>'
      case 70:                      // ']'
      case 88:                      // 'by'
      case 100:                     // 'contains'
      case 209:                     // 'paragraphs'
      case 237:                     // 'sentences'
      case 252:                     // 'times'
      case 279:                     // 'words'
      case 284:                     // '|'
      case 285:                     // '||'
      case 286:                     // '|}'
      case 287:                     // '}'
        break;
      default:
        try_RelativePathExpr();
      }
      break;
    case 48:                        // '//'
      shiftT(48);                   // '//'
      lookahead1W(260);             // EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral | StringLiteral |
      try_RelativePathExpr();
      break;
    default:
      try_RelativePathExpr();
    }
  }

  function parse_RelativePathExpr()
  {
    eventHandler.startNonterminal("RelativePathExpr", e0);
    parse_PostfixExpr();
    for (;;)
    {
      switch (l1)
      {
      case 26:                      // '!'
        lookahead2W(265);           // Wildcard | EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral |
        break;
      default:
        lk = l1;
      }
      if (lk != 25                  // EOF
       && lk != 27                  // '!='
       && lk != 38                  // ')'
       && lk != 39                  // '*'
       && lk != 41                  // '+'
       && lk != 42                  // ','
       && lk != 43                  // '-'
       && lk != 47                  // '/'
       && lk != 48                  // '//'
       && lk != 50                  // ':'
       && lk != 54                  // ';'
       && lk != 55                  // '<'
       && lk != 58                  // '<<'
       && lk != 59                  // '<='
       && lk != 61                  // '='
       && lk != 62                  // '>'
       && lk != 63                  // '>='
       && lk != 64                  // '>>'
       && lk != 70                  // ']'
       && lk != 71                  // 'after'
       && lk != 76                  // 'and'
       && lk != 80                  // 'as'
       && lk != 81                  // 'ascending'
       && lk != 82                  // 'at'
       && lk != 85                  // 'before'
       && lk != 88                  // 'by'
       && lk != 89                  // 'case'
       && lk != 90                  // 'cast'
       && lk != 91                  // 'castable'
       && lk != 95                  // 'collation'
       && lk != 100                 // 'contains'
       && lk != 106                 // 'count'
       && lk != 110                 // 'default'
       && lk != 114                 // 'descending'
       && lk != 119                 // 'div'
       && lk != 123                 // 'else'
       && lk != 124                 // 'empty'
       && lk != 127                 // 'end'
       && lk != 129                 // 'eq'
       && lk != 132                 // 'except'
       && lk != 139                 // 'for'
       && lk != 148                 // 'ge'
       && lk != 150                 // 'group'
       && lk != 152                 // 'gt'
       && lk != 153                 // 'idiv'
       && lk != 162                 // 'instance'
       && lk != 164                 // 'intersect'
       && lk != 165                 // 'into'
       && lk != 166                 // 'is'
       && lk != 175                 // 'le'
       && lk != 177                 // 'let'
       && lk != 181                 // 'lt'
       && lk != 183                 // 'mod'
       && lk != 184                 // 'modify'
       && lk != 189                 // 'ne'
       && lk != 202                 // 'only'
       && lk != 204                 // 'or'
       && lk != 205                 // 'order'
       && lk != 209                 // 'paragraphs'
       && lk != 224                 // 'return'
       && lk != 228                 // 'satisfies'
       && lk != 237                 // 'sentences'
       && lk != 241                 // 'stable'
       && lk != 242                 // 'start'
       && lk != 252                 // 'times'
       && lk != 253                 // 'to'
       && lk != 254                 // 'treat'
       && lk != 260                 // 'union'
       && lk != 272                 // 'where'
       && lk != 276                 // 'with'
       && lk != 279                 // 'words'
       && lk != 284                 // '|'
       && lk != 285                 // '||'
       && lk != 286                 // '|}'
       && lk != 287                 // '}'
       && lk != 2586                // '!' Wildcard
       && lk != 23578               // '!' '..'
       && lk != 24090               // '!' '/'
       && lk != 24602               // '!' '//'
       && lk != 34330)              // '!' '@'
      {
        lk = memoized(3, e0);
        if (lk == 0)
        {
          var b0A = b0; var e0A = e0; var l1A = l1;
          var b1A = b1; var e1A = e1; var l2A = l2;
          var b2A = b2; var e2A = e2;
          try
          {
            switch (l1)
            {
            case 47:                // '/'
              shiftT(47);           // '/'
              break;
            case 48:                // '//'
              shiftT(48);           // '//'
              break;
            default:
              shiftT(26);           // '!'
            }
            lookahead1W(264);       // Wildcard | EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral |
            try_StepExpr();
            lk = -1;
          }
          catch (p1A)
          {
            lk = -2;
          }
          b0 = b0A; e0 = e0A; l1 = l1A; if (l1 == 0) {end = e0A;} else {
          b1 = b1A; e1 = e1A; l2 = l2A; if (l2 == 0) {end = e1A;} else {
          b2 = b2A; e2 = e2A; end = e2A; }}
          memoize(3, e0, lk);
        }
      }
      if (lk != -1
       && lk != 47                  // '/'
       && lk != 48                  // '//'
       && lk != 2586                // '!' Wildcard
       && lk != 23578               // '!' '..'
       && lk != 34330)              // '!' '@'
      {
        break;
      }
      switch (l1)
      {
      case 47:                      // '/'
        shift(47);                  // '/'
        break;
      case 48:                      // '//'
        shift(48);                  // '//'
        break;
      default:
        shift(26);                  // '!'
      }
      lookahead1W(264);             // Wildcard | EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral |
      whitespace();
      parse_StepExpr();
    }
    eventHandler.endNonterminal("RelativePathExpr", e0);
  }

  function try_RelativePathExpr()
  {
    try_PostfixExpr();
    for (;;)
    {
      switch (l1)
      {
      case 26:                      // '!'
        lookahead2W(265);           // Wildcard | EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral |
        break;
      default:
        lk = l1;
      }
      if (lk != 25                  // EOF
       && lk != 27                  // '!='
       && lk != 38                  // ')'
       && lk != 39                  // '*'
       && lk != 41                  // '+'
       && lk != 42                  // ','
       && lk != 43                  // '-'
       && lk != 47                  // '/'
       && lk != 48                  // '//'
       && lk != 50                  // ':'
       && lk != 54                  // ';'
       && lk != 55                  // '<'
       && lk != 58                  // '<<'
       && lk != 59                  // '<='
       && lk != 61                  // '='
       && lk != 62                  // '>'
       && lk != 63                  // '>='
       && lk != 64                  // '>>'
       && lk != 70                  // ']'
       && lk != 71                  // 'after'
       && lk != 76                  // 'and'
       && lk != 80                  // 'as'
       && lk != 81                  // 'ascending'
       && lk != 82                  // 'at'
       && lk != 85                  // 'before'
       && lk != 88                  // 'by'
       && lk != 89                  // 'case'
       && lk != 90                  // 'cast'
       && lk != 91                  // 'castable'
       && lk != 95                  // 'collation'
       && lk != 100                 // 'contains'
       && lk != 106                 // 'count'
       && lk != 110                 // 'default'
       && lk != 114                 // 'descending'
       && lk != 119                 // 'div'
       && lk != 123                 // 'else'
       && lk != 124                 // 'empty'
       && lk != 127                 // 'end'
       && lk != 129                 // 'eq'
       && lk != 132                 // 'except'
       && lk != 139                 // 'for'
       && lk != 148                 // 'ge'
       && lk != 150                 // 'group'
       && lk != 152                 // 'gt'
       && lk != 153                 // 'idiv'
       && lk != 162                 // 'instance'
       && lk != 164                 // 'intersect'
       && lk != 165                 // 'into'
       && lk != 166                 // 'is'
       && lk != 175                 // 'le'
       && lk != 177                 // 'let'
       && lk != 181                 // 'lt'
       && lk != 183                 // 'mod'
       && lk != 184                 // 'modify'
       && lk != 189                 // 'ne'
       && lk != 202                 // 'only'
       && lk != 204                 // 'or'
       && lk != 205                 // 'order'
       && lk != 209                 // 'paragraphs'
       && lk != 224                 // 'return'
       && lk != 228                 // 'satisfies'
       && lk != 237                 // 'sentences'
       && lk != 241                 // 'stable'
       && lk != 242                 // 'start'
       && lk != 252                 // 'times'
       && lk != 253                 // 'to'
       && lk != 254                 // 'treat'
       && lk != 260                 // 'union'
       && lk != 272                 // 'where'
       && lk != 276                 // 'with'
       && lk != 279                 // 'words'
       && lk != 284                 // '|'
       && lk != 285                 // '||'
       && lk != 286                 // '|}'
       && lk != 287                 // '}'
       && lk != 2586                // '!' Wildcard
       && lk != 23578               // '!' '..'
       && lk != 24090               // '!' '/'
       && lk != 24602               // '!' '//'
       && lk != 34330)              // '!' '@'
      {
        lk = memoized(3, e0);
        if (lk == 0)
        {
          var b0A = b0; var e0A = e0; var l1A = l1;
          var b1A = b1; var e1A = e1; var l2A = l2;
          var b2A = b2; var e2A = e2;
          try
          {
            switch (l1)
            {
            case 47:                // '/'
              shiftT(47);           // '/'
              break;
            case 48:                // '//'
              shiftT(48);           // '//'
              break;
            default:
              shiftT(26);           // '!'
            }
            lookahead1W(264);       // Wildcard | EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral |
            try_StepExpr();
            memoize(3, e0A, -1);
            continue;
          }
          catch (p1A)
          {
            b0 = b0A; e0 = e0A; l1 = l1A; if (l1 == 0) {end = e0A;} else {
            b1 = b1A; e1 = e1A; l2 = l2A; if (l2 == 0) {end = e1A;} else {
            b2 = b2A; e2 = e2A; end = e2A; }}
            memoize(3, e0A, -2);
            break;
          }
        }
      }
      if (lk != -1
       && lk != 47                  // '/'
       && lk != 48                  // '//'
       && lk != 2586                // '!' Wildcard
       && lk != 23578               // '!' '..'
       && lk != 34330)              // '!' '@'
      {
        break;
      }
      switch (l1)
      {
      case 47:                      // '/'
        shiftT(47);                 // '/'
        break;
      case 48:                      // '//'
        shiftT(48);                 // '//'
        break;
      default:
        shiftT(26);                 // '!'
      }
      lookahead1W(264);             // Wildcard | EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral |
      try_StepExpr();
    }
  }

  function parse_StepExpr()
  {
    eventHandler.startNonterminal("StepExpr", e0);
    switch (l1)
    {
    case 83:                        // 'attribute'
      lookahead2W(288);             // EQName^Token | S^WS | EOF | '!' | '!=' | '#' | '(' | '(:' | ')' | '*' | '+' |
      break;
    case 122:                       // 'element'
      lookahead2W(287);             // EQName^Token | S^WS | EOF | '!' | '!=' | '#' | '(' | '(:' | ')' | '*' | '+' |
      break;
    case 187:                       // 'namespace'
    case 220:                       // 'processing-instruction'
      lookahead2W(285);             // NCName^Token | S^WS | EOF | '!' | '!=' | '#' | '(' | '(:' | ')' | '*' | '+' |
      break;
    case 135:                       // 'false'
    case 197:                       // 'null'
    case 255:                       // 'true'
      lookahead2W(237);             // S^WS | EOF | '!' | '!=' | '#' | '(' | '(:' | ')' | '*' | '+' | ',' | '-' | '.' |
      break;
    case 97:                        // 'comment'
    case 120:                       // 'document'
    case 206:                       // 'ordered'
    case 249:                       // 'text'
    case 262:                       // 'unordered'
      lookahead2W(239);             // S^WS | EOF | '!' | '!=' | '#' | '(' | '(:' | ')' | '*' | '+' | ',' | '-' | '/' |
      break;
    case 79:                        // 'array'
    case 125:                       // 'empty-sequence'
    case 154:                       // 'if'
    case 167:                       // 'item'
    case 169:                       // 'json-item'
    case 247:                       // 'structured-item'
    case 248:                       // 'switch'
    case 259:                       // 'typeswitch'
      lookahead2W(230);             // S^WS | EOF | '!' | '!=' | '#' | '(:' | ')' | '*' | '+' | ',' | '-' | '/' | '//' |
      break;
    case 74:                        // 'ancestor'
    case 75:                        // 'ancestor-or-self'
    case 94:                        // 'child'
    case 112:                       // 'descendant'
    case 113:                       // 'descendant-or-self'
    case 137:                       // 'following'
    case 138:                       // 'following-sibling'
    case 210:                       // 'parent'
    case 216:                       // 'preceding'
    case 217:                       // 'preceding-sibling'
    case 234:                       // 'self'
      lookahead2W(238);             // S^WS | EOF | '!' | '!=' | '#' | '(' | '(:' | ')' | '*' | '+' | ',' | '-' | '/' |
      break;
    case 6:                         // EQName^Token
    case 71:                        // 'after'
    case 73:                        // 'allowing'
    case 76:                        // 'and'
    case 78:                        // 'append'
    case 80:                        // 'as'
    case 81:                        // 'ascending'
    case 82:                        // 'at'
    case 84:                        // 'base-uri'
    case 85:                        // 'before'
    case 86:                        // 'boundary-space'
    case 87:                        // 'break'
    case 89:                        // 'case'
    case 90:                        // 'cast'
    case 91:                        // 'castable'
    case 92:                        // 'catch'
    case 95:                        // 'collation'
    case 98:                        // 'constraint'
    case 99:                        // 'construction'
    case 102:                       // 'context'
    case 103:                       // 'continue'
    case 104:                       // 'copy'
    case 105:                       // 'copy-namespaces'
    case 106:                       // 'count'
    case 107:                       // 'decimal-format'
    case 109:                       // 'declare'
    case 110:                       // 'default'
    case 111:                       // 'delete'
    case 114:                       // 'descending'
    case 119:                       // 'div'
    case 121:                       // 'document-node'
    case 123:                       // 'else'
    case 124:                       // 'empty'
    case 126:                       // 'encoding'
    case 127:                       // 'end'
    case 129:                       // 'eq'
    case 130:                       // 'every'
    case 132:                       // 'except'
    case 133:                       // 'exit'
    case 134:                       // 'external'
    case 136:                       // 'first'
    case 139:                       // 'for'
    case 142:                       // 'from'
    case 143:                       // 'ft-option'
    case 147:                       // 'function'
    case 148:                       // 'ge'
    case 150:                       // 'group'
    case 152:                       // 'gt'
    case 153:                       // 'idiv'
    case 155:                       // 'import'
    case 156:                       // 'in'
    case 157:                       // 'index'
    case 161:                       // 'insert'
    case 162:                       // 'instance'
    case 163:                       // 'integrity'
    case 164:                       // 'intersect'
    case 165:                       // 'into'
    case 166:                       // 'is'
    case 168:                       // 'json'
    case 170:                       // 'jsoniq'
    case 173:                       // 'last'
    case 174:                       // 'lax'
    case 175:                       // 'le'
    case 177:                       // 'let'
    case 179:                       // 'loop'
    case 181:                       // 'lt'
    case 183:                       // 'mod'
    case 184:                       // 'modify'
    case 185:                       // 'module'
    case 188:                       // 'namespace-node'
    case 189:                       // 'ne'
    case 194:                       // 'node'
    case 195:                       // 'nodes'
    case 198:                       // 'object'
    case 202:                       // 'only'
    case 203:                       // 'option'
    case 204:                       // 'or'
    case 205:                       // 'order'
    case 207:                       // 'ordering'
    case 222:                       // 'rename'
    case 223:                       // 'replace'
    case 224:                       // 'return'
    case 225:                       // 'returning'
    case 226:                       // 'revalidation'
    case 228:                       // 'satisfies'
    case 229:                       // 'schema'
    case 230:                       // 'schema-attribute'
    case 231:                       // 'schema-element'
    case 232:                       // 'score'
    case 233:                       // 'select'
    case 239:                       // 'sliding'
    case 240:                       // 'some'
    case 241:                       // 'stable'
    case 242:                       // 'start'
    case 245:                       // 'strict'
    case 253:                       // 'to'
    case 254:                       // 'treat'
    case 256:                       // 'try'
    case 257:                       // 'tumbling'
    case 258:                       // 'type'
    case 260:                       // 'union'
    case 263:                       // 'updating'
    case 266:                       // 'validate'
    case 267:                       // 'value'
    case 268:                       // 'variable'
    case 269:                       // 'version'
    case 272:                       // 'where'
    case 273:                       // 'while'
    case 276:                       // 'with'
      lookahead2W(234);             // S^WS | EOF | '!' | '!=' | '#' | '(' | '(:' | ')' | '*' | '+' | ',' | '-' | '/' |
      break;
    default:
      lk = l1;
    }
    if (lk == 12935                 // 'false' EOF
     || lk == 12997                 // 'null' EOF
     || lk == 13055                 // 'true' EOF
     || lk == 13447                 // 'false' '!'
     || lk == 13509                 // 'null' '!'
     || lk == 13567                 // 'true' '!'
     || lk == 13959                 // 'false' '!='
     || lk == 14021                 // 'null' '!='
     || lk == 14079                 // 'true' '!='
     || lk == 19591                 // 'false' ')'
     || lk == 19653                 // 'null' ')'
     || lk == 19711                 // 'true' ')'
     || lk == 20103                 // 'false' '*'
     || lk == 20165                 // 'null' '*'
     || lk == 20223                 // 'true' '*'
     || lk == 21127                 // 'false' '+'
     || lk == 21189                 // 'null' '+'
     || lk == 21247                 // 'true' '+'
     || lk == 21639                 // 'false' ','
     || lk == 21701                 // 'null' ','
     || lk == 21759                 // 'true' ','
     || lk == 22151                 // 'false' '-'
     || lk == 22213                 // 'null' '-'
     || lk == 22271                 // 'true' '-'
     || lk == 24199                 // 'false' '/'
     || lk == 24261                 // 'null' '/'
     || lk == 24319                 // 'true' '/'
     || lk == 24711                 // 'false' '//'
     || lk == 24773                 // 'null' '//'
     || lk == 24831                 // 'true' '//'
     || lk == 25735                 // 'false' ':'
     || lk == 25797                 // 'null' ':'
     || lk == 25855                 // 'true' ':'
     || lk == 27783                 // 'false' ';'
     || lk == 27845                 // 'null' ';'
     || lk == 27903                 // 'true' ';'
     || lk == 28295                 // 'false' '<'
     || lk == 28357                 // 'null' '<'
     || lk == 28415                 // 'true' '<'
     || lk == 29831                 // 'false' '<<'
     || lk == 29893                 // 'null' '<<'
     || lk == 29951                 // 'true' '<<'
     || lk == 30343                 // 'false' '<='
     || lk == 30405                 // 'null' '<='
     || lk == 30463                 // 'true' '<='
     || lk == 31367                 // 'false' '='
     || lk == 31429                 // 'null' '='
     || lk == 31487                 // 'true' '='
     || lk == 31879                 // 'false' '>'
     || lk == 31941                 // 'null' '>'
     || lk == 31999                 // 'true' '>'
     || lk == 32391                 // 'false' '>='
     || lk == 32453                 // 'null' '>='
     || lk == 32511                 // 'true' '>='
     || lk == 32903                 // 'false' '>>'
     || lk == 32965                 // 'null' '>>'
     || lk == 33023                 // 'true' '>>'
     || lk == 35463                 // 'false' '['
     || lk == 35525                 // 'null' '['
     || lk == 35583                 // 'true' '['
     || lk == 35975                 // 'false' ']'
     || lk == 36037                 // 'null' ']'
     || lk == 36095                 // 'true' ']'
     || lk == 36435                 // 'attribute' 'after'
     || lk == 36474                 // 'element' 'after'
     || lk == 36487                 // 'false' 'after'
     || lk == 36539                 // 'namespace' 'after'
     || lk == 36549                 // 'null' 'after'
     || lk == 36572                 // 'processing-instruction' 'after'
     || lk == 36607                 // 'true' 'after'
     || lk == 38995                 // 'attribute' 'and'
     || lk == 39034                 // 'element' 'and'
     || lk == 39047                 // 'false' 'and'
     || lk == 39099                 // 'namespace' 'and'
     || lk == 39109                 // 'null' 'and'
     || lk == 39132                 // 'processing-instruction' 'and'
     || lk == 39167                 // 'true' 'and'
     || lk == 41043                 // 'attribute' 'as'
     || lk == 41082                 // 'element' 'as'
     || lk == 41095                 // 'false' 'as'
     || lk == 41147                 // 'namespace' 'as'
     || lk == 41157                 // 'null' 'as'
     || lk == 41180                 // 'processing-instruction' 'as'
     || lk == 41215                 // 'true' 'as'
     || lk == 41555                 // 'attribute' 'ascending'
     || lk == 41594                 // 'element' 'ascending'
     || lk == 41607                 // 'false' 'ascending'
     || lk == 41659                 // 'namespace' 'ascending'
     || lk == 41669                 // 'null' 'ascending'
     || lk == 41692                 // 'processing-instruction' 'ascending'
     || lk == 41727                 // 'true' 'ascending'
     || lk == 42067                 // 'attribute' 'at'
     || lk == 42106                 // 'element' 'at'
     || lk == 42119                 // 'false' 'at'
     || lk == 42171                 // 'namespace' 'at'
     || lk == 42181                 // 'null' 'at'
     || lk == 42204                 // 'processing-instruction' 'at'
     || lk == 42239                 // 'true' 'at'
     || lk == 43603                 // 'attribute' 'before'
     || lk == 43642                 // 'element' 'before'
     || lk == 43655                 // 'false' 'before'
     || lk == 43707                 // 'namespace' 'before'
     || lk == 43717                 // 'null' 'before'
     || lk == 43740                 // 'processing-instruction' 'before'
     || lk == 43775                 // 'true' 'before'
     || lk == 45191                 // 'false' 'by'
     || lk == 45253                 // 'null' 'by'
     || lk == 45311                 // 'true' 'by'
     || lk == 45651                 // 'attribute' 'case'
     || lk == 45690                 // 'element' 'case'
     || lk == 45703                 // 'false' 'case'
     || lk == 45755                 // 'namespace' 'case'
     || lk == 45765                 // 'null' 'case'
     || lk == 45788                 // 'processing-instruction' 'case'
     || lk == 45823                 // 'true' 'case'
     || lk == 46163                 // 'attribute' 'cast'
     || lk == 46202                 // 'element' 'cast'
     || lk == 46215                 // 'false' 'cast'
     || lk == 46267                 // 'namespace' 'cast'
     || lk == 46277                 // 'null' 'cast'
     || lk == 46300                 // 'processing-instruction' 'cast'
     || lk == 46335                 // 'true' 'cast'
     || lk == 46675                 // 'attribute' 'castable'
     || lk == 46714                 // 'element' 'castable'
     || lk == 46727                 // 'false' 'castable'
     || lk == 46779                 // 'namespace' 'castable'
     || lk == 46789                 // 'null' 'castable'
     || lk == 46812                 // 'processing-instruction' 'castable'
     || lk == 46847                 // 'true' 'castable'
     || lk == 48723                 // 'attribute' 'collation'
     || lk == 48762                 // 'element' 'collation'
     || lk == 48775                 // 'false' 'collation'
     || lk == 48827                 // 'namespace' 'collation'
     || lk == 48837                 // 'null' 'collation'
     || lk == 48860                 // 'processing-instruction' 'collation'
     || lk == 48895                 // 'true' 'collation'
     || lk == 51335                 // 'false' 'contains'
     || lk == 51397                 // 'null' 'contains'
     || lk == 51455                 // 'true' 'contains'
     || lk == 54355                 // 'attribute' 'count'
     || lk == 54394                 // 'element' 'count'
     || lk == 54407                 // 'false' 'count'
     || lk == 54459                 // 'namespace' 'count'
     || lk == 54469                 // 'null' 'count'
     || lk == 54492                 // 'processing-instruction' 'count'
     || lk == 54527                 // 'true' 'count'
     || lk == 56403                 // 'attribute' 'default'
     || lk == 56442                 // 'element' 'default'
     || lk == 56455                 // 'false' 'default'
     || lk == 56507                 // 'namespace' 'default'
     || lk == 56517                 // 'null' 'default'
     || lk == 56540                 // 'processing-instruction' 'default'
     || lk == 56575                 // 'true' 'default'
     || lk == 58451                 // 'attribute' 'descending'
     || lk == 58490                 // 'element' 'descending'
     || lk == 58503                 // 'false' 'descending'
     || lk == 58555                 // 'namespace' 'descending'
     || lk == 58565                 // 'null' 'descending'
     || lk == 58588                 // 'processing-instruction' 'descending'
     || lk == 58623                 // 'true' 'descending'
     || lk == 61011                 // 'attribute' 'div'
     || lk == 61050                 // 'element' 'div'
     || lk == 61063                 // 'false' 'div'
     || lk == 61115                 // 'namespace' 'div'
     || lk == 61125                 // 'null' 'div'
     || lk == 61148                 // 'processing-instruction' 'div'
     || lk == 61183                 // 'true' 'div'
     || lk == 63059                 // 'attribute' 'else'
     || lk == 63098                 // 'element' 'else'
     || lk == 63111                 // 'false' 'else'
     || lk == 63163                 // 'namespace' 'else'
     || lk == 63173                 // 'null' 'else'
     || lk == 63196                 // 'processing-instruction' 'else'
     || lk == 63231                 // 'true' 'else'
     || lk == 63571                 // 'attribute' 'empty'
     || lk == 63610                 // 'element' 'empty'
     || lk == 63623                 // 'false' 'empty'
     || lk == 63675                 // 'namespace' 'empty'
     || lk == 63685                 // 'null' 'empty'
     || lk == 63708                 // 'processing-instruction' 'empty'
     || lk == 63743                 // 'true' 'empty'
     || lk == 65107                 // 'attribute' 'end'
     || lk == 65146                 // 'element' 'end'
     || lk == 65159                 // 'false' 'end'
     || lk == 65211                 // 'namespace' 'end'
     || lk == 65221                 // 'null' 'end'
     || lk == 65244                 // 'processing-instruction' 'end'
     || lk == 65279                 // 'true' 'end'
     || lk == 66131                 // 'attribute' 'eq'
     || lk == 66170                 // 'element' 'eq'
     || lk == 66183                 // 'false' 'eq'
     || lk == 66235                 // 'namespace' 'eq'
     || lk == 66245                 // 'null' 'eq'
     || lk == 66268                 // 'processing-instruction' 'eq'
     || lk == 66303                 // 'true' 'eq'
     || lk == 67667                 // 'attribute' 'except'
     || lk == 67706                 // 'element' 'except'
     || lk == 67719                 // 'false' 'except'
     || lk == 67771                 // 'namespace' 'except'
     || lk == 67781                 // 'null' 'except'
     || lk == 67804                 // 'processing-instruction' 'except'
     || lk == 67839                 // 'true' 'except'
     || lk == 71251                 // 'attribute' 'for'
     || lk == 71290                 // 'element' 'for'
     || lk == 71303                 // 'false' 'for'
     || lk == 71355                 // 'namespace' 'for'
     || lk == 71365                 // 'null' 'for'
     || lk == 71388                 // 'processing-instruction' 'for'
     || lk == 71423                 // 'true' 'for'
     || lk == 75859                 // 'attribute' 'ge'
     || lk == 75898                 // 'element' 'ge'
     || lk == 75911                 // 'false' 'ge'
     || lk == 75963                 // 'namespace' 'ge'
     || lk == 75973                 // 'null' 'ge'
     || lk == 75996                 // 'processing-instruction' 'ge'
     || lk == 76031                 // 'true' 'ge'
     || lk == 76883                 // 'attribute' 'group'
     || lk == 76922                 // 'element' 'group'
     || lk == 76935                 // 'false' 'group'
     || lk == 76987                 // 'namespace' 'group'
     || lk == 76997                 // 'null' 'group'
     || lk == 77020                 // 'processing-instruction' 'group'
     || lk == 77055                 // 'true' 'group'
     || lk == 77907                 // 'attribute' 'gt'
     || lk == 77946                 // 'element' 'gt'
     || lk == 77959                 // 'false' 'gt'
     || lk == 78011                 // 'namespace' 'gt'
     || lk == 78021                 // 'null' 'gt'
     || lk == 78044                 // 'processing-instruction' 'gt'
     || lk == 78079                 // 'true' 'gt'
     || lk == 78419                 // 'attribute' 'idiv'
     || lk == 78458                 // 'element' 'idiv'
     || lk == 78471                 // 'false' 'idiv'
     || lk == 78523                 // 'namespace' 'idiv'
     || lk == 78533                 // 'null' 'idiv'
     || lk == 78556                 // 'processing-instruction' 'idiv'
     || lk == 78591                 // 'true' 'idiv'
     || lk == 83027                 // 'attribute' 'instance'
     || lk == 83066                 // 'element' 'instance'
     || lk == 83079                 // 'false' 'instance'
     || lk == 83131                 // 'namespace' 'instance'
     || lk == 83141                 // 'null' 'instance'
     || lk == 83164                 // 'processing-instruction' 'instance'
     || lk == 83199                 // 'true' 'instance'
     || lk == 84051                 // 'attribute' 'intersect'
     || lk == 84090                 // 'element' 'intersect'
     || lk == 84103                 // 'false' 'intersect'
     || lk == 84155                 // 'namespace' 'intersect'
     || lk == 84165                 // 'null' 'intersect'
     || lk == 84188                 // 'processing-instruction' 'intersect'
     || lk == 84223                 // 'true' 'intersect'
     || lk == 84563                 // 'attribute' 'into'
     || lk == 84602                 // 'element' 'into'
     || lk == 84615                 // 'false' 'into'
     || lk == 84667                 // 'namespace' 'into'
     || lk == 84677                 // 'null' 'into'
     || lk == 84700                 // 'processing-instruction' 'into'
     || lk == 84735                 // 'true' 'into'
     || lk == 85075                 // 'attribute' 'is'
     || lk == 85114                 // 'element' 'is'
     || lk == 85127                 // 'false' 'is'
     || lk == 85179                 // 'namespace' 'is'
     || lk == 85189                 // 'null' 'is'
     || lk == 85212                 // 'processing-instruction' 'is'
     || lk == 85247                 // 'true' 'is'
     || lk == 89683                 // 'attribute' 'le'
     || lk == 89722                 // 'element' 'le'
     || lk == 89735                 // 'false' 'le'
     || lk == 89787                 // 'namespace' 'le'
     || lk == 89797                 // 'null' 'le'
     || lk == 89820                 // 'processing-instruction' 'le'
     || lk == 89855                 // 'true' 'le'
     || lk == 90707                 // 'attribute' 'let'
     || lk == 90746                 // 'element' 'let'
     || lk == 90759                 // 'false' 'let'
     || lk == 90811                 // 'namespace' 'let'
     || lk == 90821                 // 'null' 'let'
     || lk == 90844                 // 'processing-instruction' 'let'
     || lk == 90879                 // 'true' 'let'
     || lk == 92755                 // 'attribute' 'lt'
     || lk == 92794                 // 'element' 'lt'
     || lk == 92807                 // 'false' 'lt'
     || lk == 92859                 // 'namespace' 'lt'
     || lk == 92869                 // 'null' 'lt'
     || lk == 92892                 // 'processing-instruction' 'lt'
     || lk == 92927                 // 'true' 'lt'
     || lk == 93779                 // 'attribute' 'mod'
     || lk == 93818                 // 'element' 'mod'
     || lk == 93831                 // 'false' 'mod'
     || lk == 93883                 // 'namespace' 'mod'
     || lk == 93893                 // 'null' 'mod'
     || lk == 93916                 // 'processing-instruction' 'mod'
     || lk == 93951                 // 'true' 'mod'
     || lk == 94291                 // 'attribute' 'modify'
     || lk == 94330                 // 'element' 'modify'
     || lk == 94343                 // 'false' 'modify'
     || lk == 94395                 // 'namespace' 'modify'
     || lk == 94405                 // 'null' 'modify'
     || lk == 94428                 // 'processing-instruction' 'modify'
     || lk == 94463                 // 'true' 'modify'
     || lk == 96851                 // 'attribute' 'ne'
     || lk == 96890                 // 'element' 'ne'
     || lk == 96903                 // 'false' 'ne'
     || lk == 96955                 // 'namespace' 'ne'
     || lk == 96965                 // 'null' 'ne'
     || lk == 96988                 // 'processing-instruction' 'ne'
     || lk == 97023                 // 'true' 'ne'
     || lk == 103507                // 'attribute' 'only'
     || lk == 103546                // 'element' 'only'
     || lk == 103559                // 'false' 'only'
     || lk == 103611                // 'namespace' 'only'
     || lk == 103621                // 'null' 'only'
     || lk == 103644                // 'processing-instruction' 'only'
     || lk == 103679                // 'true' 'only'
     || lk == 104531                // 'attribute' 'or'
     || lk == 104570                // 'element' 'or'
     || lk == 104583                // 'false' 'or'
     || lk == 104635                // 'namespace' 'or'
     || lk == 104645                // 'null' 'or'
     || lk == 104668                // 'processing-instruction' 'or'
     || lk == 104703                // 'true' 'or'
     || lk == 105043                // 'attribute' 'order'
     || lk == 105082                // 'element' 'order'
     || lk == 105095                // 'false' 'order'
     || lk == 105147                // 'namespace' 'order'
     || lk == 105157                // 'null' 'order'
     || lk == 105180                // 'processing-instruction' 'order'
     || lk == 105215                // 'true' 'order'
     || lk == 107143                // 'false' 'paragraphs'
     || lk == 107205                // 'null' 'paragraphs'
     || lk == 107263                // 'true' 'paragraphs'
     || lk == 114771                // 'attribute' 'return'
     || lk == 114810                // 'element' 'return'
     || lk == 114823                // 'false' 'return'
     || lk == 114875                // 'namespace' 'return'
     || lk == 114885                // 'null' 'return'
     || lk == 114908                // 'processing-instruction' 'return'
     || lk == 114943                // 'true' 'return'
     || lk == 116819                // 'attribute' 'satisfies'
     || lk == 116858                // 'element' 'satisfies'
     || lk == 116871                // 'false' 'satisfies'
     || lk == 116923                // 'namespace' 'satisfies'
     || lk == 116933                // 'null' 'satisfies'
     || lk == 116956                // 'processing-instruction' 'satisfies'
     || lk == 116991                // 'true' 'satisfies'
     || lk == 121479                // 'false' 'sentences'
     || lk == 121541                // 'null' 'sentences'
     || lk == 121599                // 'true' 'sentences'
     || lk == 123475                // 'attribute' 'stable'
     || lk == 123514                // 'element' 'stable'
     || lk == 123527                // 'false' 'stable'
     || lk == 123579                // 'namespace' 'stable'
     || lk == 123589                // 'null' 'stable'
     || lk == 123612                // 'processing-instruction' 'stable'
     || lk == 123647                // 'true' 'stable'
     || lk == 123987                // 'attribute' 'start'
     || lk == 124026                // 'element' 'start'
     || lk == 124039                // 'false' 'start'
     || lk == 124091                // 'namespace' 'start'
     || lk == 124101                // 'null' 'start'
     || lk == 124124                // 'processing-instruction' 'start'
     || lk == 124159                // 'true' 'start'
     || lk == 129159                // 'false' 'times'
     || lk == 129221                // 'null' 'times'
     || lk == 129279                // 'true' 'times'
     || lk == 129619                // 'attribute' 'to'
     || lk == 129658                // 'element' 'to'
     || lk == 129671                // 'false' 'to'
     || lk == 129723                // 'namespace' 'to'
     || lk == 129733                // 'null' 'to'
     || lk == 129756                // 'processing-instruction' 'to'
     || lk == 129791                // 'true' 'to'
     || lk == 130131                // 'attribute' 'treat'
     || lk == 130170                // 'element' 'treat'
     || lk == 130183                // 'false' 'treat'
     || lk == 130235                // 'namespace' 'treat'
     || lk == 130245                // 'null' 'treat'
     || lk == 130268                // 'processing-instruction' 'treat'
     || lk == 130303                // 'true' 'treat'
     || lk == 133203                // 'attribute' 'union'
     || lk == 133242                // 'element' 'union'
     || lk == 133255                // 'false' 'union'
     || lk == 133307                // 'namespace' 'union'
     || lk == 133317                // 'null' 'union'
     || lk == 133340                // 'processing-instruction' 'union'
     || lk == 133375                // 'true' 'union'
     || lk == 139347                // 'attribute' 'where'
     || lk == 139386                // 'element' 'where'
     || lk == 139399                // 'false' 'where'
     || lk == 139451                // 'namespace' 'where'
     || lk == 139461                // 'null' 'where'
     || lk == 139484                // 'processing-instruction' 'where'
     || lk == 139519                // 'true' 'where'
     || lk == 141395                // 'attribute' 'with'
     || lk == 141434                // 'element' 'with'
     || lk == 141447                // 'false' 'with'
     || lk == 141499                // 'namespace' 'with'
     || lk == 141509                // 'null' 'with'
     || lk == 141532                // 'processing-instruction' 'with'
     || lk == 141567                // 'true' 'with'
     || lk == 142983                // 'false' 'words'
     || lk == 143045                // 'null' 'words'
     || lk == 143103                // 'true' 'words'
     || lk == 145543                // 'false' '|'
     || lk == 145605                // 'null' '|'
     || lk == 145663                // 'true' '|'
     || lk == 146055                // 'false' '||'
     || lk == 146117                // 'null' '||'
     || lk == 146175                // 'true' '||'
     || lk == 146567                // 'false' '|}'
     || lk == 146629                // 'null' '|}'
     || lk == 146687                // 'true' '|}'
     || lk == 147079                // 'false' '}'
     || lk == 147141                // 'null' '}'
     || lk == 147199)               // 'true' '}'
    {
      lk = memoized(4, e0);
      if (lk == 0)
      {
        var b0A = b0; var e0A = e0; var l1A = l1;
        var b1A = b1; var e1A = e1; var l2A = l2;
        var b2A = b2; var e2A = e2;
        try
        {
          try_PostfixExpr();
          lk = -1;
        }
        catch (p1A)
        {
          lk = -2;
        }
        b0 = b0A; e0 = e0A; l1 = l1A; if (l1 == 0) {end = e0A;} else {
        b1 = b1A; e1 = e1A; l2 = l2A; if (l2 == 0) {end = e1A;} else {
        b2 = b2A; e2 = e2A; end = e2A; }}
        memoize(4, e0, lk);
      }
    }
    switch (lk)
    {
    case -1:
    case 8:                         // IntegerLiteral
    case 9:                         // DecimalLiteral
    case 10:                        // DoubleLiteral
    case 11:                        // StringLiteral
    case 31:                        // '$'
    case 32:                        // '$$'
    case 33:                        // '%'
    case 35:                        // '('
    case 55:                        // '<'
    case 56:                        // '<!--'
    case 60:                        // '<?'
    case 69:                        // '['
    case 281:                       // '{'
    case 283:                       // '{|'
    case 3155:                      // 'attribute' EQName^Token
    case 3194:                      // 'element' EQName^Token
    case 9915:                      // 'namespace' NCName^Token
    case 9948:                      // 'processing-instruction' NCName^Token
    case 14854:                     // EQName^Token '#'
    case 14919:                     // 'after' '#'
    case 14921:                     // 'allowing' '#'
    case 14922:                     // 'ancestor' '#'
    case 14923:                     // 'ancestor-or-self' '#'
    case 14924:                     // 'and' '#'
    case 14926:                     // 'append' '#'
    case 14927:                     // 'array' '#'
    case 14928:                     // 'as' '#'
    case 14929:                     // 'ascending' '#'
    case 14930:                     // 'at' '#'
    case 14931:                     // 'attribute' '#'
    case 14932:                     // 'base-uri' '#'
    case 14933:                     // 'before' '#'
    case 14934:                     // 'boundary-space' '#'
    case 14935:                     // 'break' '#'
    case 14937:                     // 'case' '#'
    case 14938:                     // 'cast' '#'
    case 14939:                     // 'castable' '#'
    case 14940:                     // 'catch' '#'
    case 14942:                     // 'child' '#'
    case 14943:                     // 'collation' '#'
    case 14945:                     // 'comment' '#'
    case 14946:                     // 'constraint' '#'
    case 14947:                     // 'construction' '#'
    case 14950:                     // 'context' '#'
    case 14951:                     // 'continue' '#'
    case 14952:                     // 'copy' '#'
    case 14953:                     // 'copy-namespaces' '#'
    case 14954:                     // 'count' '#'
    case 14955:                     // 'decimal-format' '#'
    case 14957:                     // 'declare' '#'
    case 14958:                     // 'default' '#'
    case 14959:                     // 'delete' '#'
    case 14960:                     // 'descendant' '#'
    case 14961:                     // 'descendant-or-self' '#'
    case 14962:                     // 'descending' '#'
    case 14967:                     // 'div' '#'
    case 14968:                     // 'document' '#'
    case 14969:                     // 'document-node' '#'
    case 14970:                     // 'element' '#'
    case 14971:                     // 'else' '#'
    case 14972:                     // 'empty' '#'
    case 14973:                     // 'empty-sequence' '#'
    case 14974:                     // 'encoding' '#'
    case 14975:                     // 'end' '#'
    case 14977:                     // 'eq' '#'
    case 14978:                     // 'every' '#'
    case 14980:                     // 'except' '#'
    case 14981:                     // 'exit' '#'
    case 14982:                     // 'external' '#'
    case 14983:                     // 'false' '#'
    case 14984:                     // 'first' '#'
    case 14985:                     // 'following' '#'
    case 14986:                     // 'following-sibling' '#'
    case 14987:                     // 'for' '#'
    case 14990:                     // 'from' '#'
    case 14991:                     // 'ft-option' '#'
    case 14995:                     // 'function' '#'
    case 14996:                     // 'ge' '#'
    case 14998:                     // 'group' '#'
    case 15000:                     // 'gt' '#'
    case 15001:                     // 'idiv' '#'
    case 15002:                     // 'if' '#'
    case 15003:                     // 'import' '#'
    case 15004:                     // 'in' '#'
    case 15005:                     // 'index' '#'
    case 15009:                     // 'insert' '#'
    case 15010:                     // 'instance' '#'
    case 15011:                     // 'integrity' '#'
    case 15012:                     // 'intersect' '#'
    case 15013:                     // 'into' '#'
    case 15014:                     // 'is' '#'
    case 15015:                     // 'item' '#'
    case 15016:                     // 'json' '#'
    case 15017:                     // 'json-item' '#'
    case 15018:                     // 'jsoniq' '#'
    case 15021:                     // 'last' '#'
    case 15022:                     // 'lax' '#'
    case 15023:                     // 'le' '#'
    case 15025:                     // 'let' '#'
    case 15027:                     // 'loop' '#'
    case 15029:                     // 'lt' '#'
    case 15031:                     // 'mod' '#'
    case 15032:                     // 'modify' '#'
    case 15033:                     // 'module' '#'
    case 15035:                     // 'namespace' '#'
    case 15036:                     // 'namespace-node' '#'
    case 15037:                     // 'ne' '#'
    case 15042:                     // 'node' '#'
    case 15043:                     // 'nodes' '#'
    case 15045:                     // 'null' '#'
    case 15046:                     // 'object' '#'
    case 15050:                     // 'only' '#'
    case 15051:                     // 'option' '#'
    case 15052:                     // 'or' '#'
    case 15053:                     // 'order' '#'
    case 15054:                     // 'ordered' '#'
    case 15055:                     // 'ordering' '#'
    case 15058:                     // 'parent' '#'
    case 15064:                     // 'preceding' '#'
    case 15065:                     // 'preceding-sibling' '#'
    case 15068:                     // 'processing-instruction' '#'
    case 15070:                     // 'rename' '#'
    case 15071:                     // 'replace' '#'
    case 15072:                     // 'return' '#'
    case 15073:                     // 'returning' '#'
    case 15074:                     // 'revalidation' '#'
    case 15076:                     // 'satisfies' '#'
    case 15077:                     // 'schema' '#'
    case 15078:                     // 'schema-attribute' '#'
    case 15079:                     // 'schema-element' '#'
    case 15080:                     // 'score' '#'
    case 15081:                     // 'select' '#'
    case 15082:                     // 'self' '#'
    case 15087:                     // 'sliding' '#'
    case 15088:                     // 'some' '#'
    case 15089:                     // 'stable' '#'
    case 15090:                     // 'start' '#'
    case 15093:                     // 'strict' '#'
    case 15095:                     // 'structured-item' '#'
    case 15096:                     // 'switch' '#'
    case 15097:                     // 'text' '#'
    case 15101:                     // 'to' '#'
    case 15102:                     // 'treat' '#'
    case 15103:                     // 'true' '#'
    case 15104:                     // 'try' '#'
    case 15105:                     // 'tumbling' '#'
    case 15106:                     // 'type' '#'
    case 15107:                     // 'typeswitch' '#'
    case 15108:                     // 'union' '#'
    case 15110:                     // 'unordered' '#'
    case 15111:                     // 'updating' '#'
    case 15114:                     // 'validate' '#'
    case 15115:                     // 'value' '#'
    case 15116:                     // 'variable' '#'
    case 15117:                     // 'version' '#'
    case 15120:                     // 'where' '#'
    case 15121:                     // 'while' '#'
    case 15124:                     // 'with' '#'
    case 17926:                     // EQName^Token '('
    case 17991:                     // 'after' '('
    case 17993:                     // 'allowing' '('
    case 17994:                     // 'ancestor' '('
    case 17995:                     // 'ancestor-or-self' '('
    case 17996:                     // 'and' '('
    case 17998:                     // 'append' '('
    case 18000:                     // 'as' '('
    case 18001:                     // 'ascending' '('
    case 18002:                     // 'at' '('
    case 18004:                     // 'base-uri' '('
    case 18005:                     // 'before' '('
    case 18006:                     // 'boundary-space' '('
    case 18007:                     // 'break' '('
    case 18009:                     // 'case' '('
    case 18010:                     // 'cast' '('
    case 18011:                     // 'castable' '('
    case 18012:                     // 'catch' '('
    case 18014:                     // 'child' '('
    case 18015:                     // 'collation' '('
    case 18018:                     // 'constraint' '('
    case 18019:                     // 'construction' '('
    case 18022:                     // 'context' '('
    case 18023:                     // 'continue' '('
    case 18024:                     // 'copy' '('
    case 18025:                     // 'copy-namespaces' '('
    case 18026:                     // 'count' '('
    case 18027:                     // 'decimal-format' '('
    case 18029:                     // 'declare' '('
    case 18030:                     // 'default' '('
    case 18031:                     // 'delete' '('
    case 18032:                     // 'descendant' '('
    case 18033:                     // 'descendant-or-self' '('
    case 18034:                     // 'descending' '('
    case 18039:                     // 'div' '('
    case 18040:                     // 'document' '('
    case 18043:                     // 'else' '('
    case 18044:                     // 'empty' '('
    case 18046:                     // 'encoding' '('
    case 18047:                     // 'end' '('
    case 18049:                     // 'eq' '('
    case 18050:                     // 'every' '('
    case 18052:                     // 'except' '('
    case 18053:                     // 'exit' '('
    case 18054:                     // 'external' '('
    case 18055:                     // 'false' '('
    case 18056:                     // 'first' '('
    case 18057:                     // 'following' '('
    case 18058:                     // 'following-sibling' '('
    case 18059:                     // 'for' '('
    case 18062:                     // 'from' '('
    case 18063:                     // 'ft-option' '('
    case 18067:                     // 'function' '('
    case 18068:                     // 'ge' '('
    case 18070:                     // 'group' '('
    case 18072:                     // 'gt' '('
    case 18073:                     // 'idiv' '('
    case 18075:                     // 'import' '('
    case 18076:                     // 'in' '('
    case 18077:                     // 'index' '('
    case 18081:                     // 'insert' '('
    case 18082:                     // 'instance' '('
    case 18083:                     // 'integrity' '('
    case 18084:                     // 'intersect' '('
    case 18085:                     // 'into' '('
    case 18086:                     // 'is' '('
    case 18088:                     // 'json' '('
    case 18090:                     // 'jsoniq' '('
    case 18093:                     // 'last' '('
    case 18094:                     // 'lax' '('
    case 18095:                     // 'le' '('
    case 18097:                     // 'let' '('
    case 18099:                     // 'loop' '('
    case 18101:                     // 'lt' '('
    case 18103:                     // 'mod' '('
    case 18104:                     // 'modify' '('
    case 18105:                     // 'module' '('
    case 18107:                     // 'namespace' '('
    case 18109:                     // 'ne' '('
    case 18115:                     // 'nodes' '('
    case 18117:                     // 'null' '('
    case 18118:                     // 'object' '('
    case 18122:                     // 'only' '('
    case 18123:                     // 'option' '('
    case 18124:                     // 'or' '('
    case 18125:                     // 'order' '('
    case 18126:                     // 'ordered' '('
    case 18127:                     // 'ordering' '('
    case 18130:                     // 'parent' '('
    case 18136:                     // 'preceding' '('
    case 18137:                     // 'preceding-sibling' '('
    case 18142:                     // 'rename' '('
    case 18143:                     // 'replace' '('
    case 18144:                     // 'return' '('
    case 18145:                     // 'returning' '('
    case 18146:                     // 'revalidation' '('
    case 18148:                     // 'satisfies' '('
    case 18149:                     // 'schema' '('
    case 18152:                     // 'score' '('
    case 18153:                     // 'select' '('
    case 18154:                     // 'self' '('
    case 18159:                     // 'sliding' '('
    case 18160:                     // 'some' '('
    case 18161:                     // 'stable' '('
    case 18162:                     // 'start' '('
    case 18165:                     // 'strict' '('
    case 18173:                     // 'to' '('
    case 18174:                     // 'treat' '('
    case 18175:                     // 'true' '('
    case 18176:                     // 'try' '('
    case 18177:                     // 'tumbling' '('
    case 18178:                     // 'type' '('
    case 18180:                     // 'union' '('
    case 18182:                     // 'unordered' '('
    case 18183:                     // 'updating' '('
    case 18186:                     // 'validate' '('
    case 18187:                     // 'value' '('
    case 18188:                     // 'variable' '('
    case 18189:                     // 'version' '('
    case 18192:                     // 'where' '('
    case 18193:                     // 'while' '('
    case 18196:                     // 'with' '('
    case 23175:                     // 'false' '.'
    case 23237:                     // 'null' '.'
    case 23295:                     // 'true' '.'
    case 37459:                     // 'attribute' 'allowing'
    case 37498:                     // 'element' 'allowing'
    case 37563:                     // 'namespace' 'allowing'
    case 37596:                     // 'processing-instruction' 'allowing'
    case 37971:                     // 'attribute' 'ancestor'
    case 38010:                     // 'element' 'ancestor'
    case 38075:                     // 'namespace' 'ancestor'
    case 38108:                     // 'processing-instruction' 'ancestor'
    case 38483:                     // 'attribute' 'ancestor-or-self'
    case 38522:                     // 'element' 'ancestor-or-self'
    case 38587:                     // 'namespace' 'ancestor-or-self'
    case 38620:                     // 'processing-instruction' 'ancestor-or-self'
    case 40019:                     // 'attribute' 'append'
    case 40058:                     // 'element' 'append'
    case 40123:                     // 'namespace' 'append'
    case 40156:                     // 'processing-instruction' 'append'
    case 40531:                     // 'attribute' 'array'
    case 40570:                     // 'element' 'array'
    case 42579:                     // 'attribute' 'attribute'
    case 42618:                     // 'element' 'attribute'
    case 42683:                     // 'namespace' 'attribute'
    case 42716:                     // 'processing-instruction' 'attribute'
    case 43091:                     // 'attribute' 'base-uri'
    case 43130:                     // 'element' 'base-uri'
    case 43195:                     // 'namespace' 'base-uri'
    case 43228:                     // 'processing-instruction' 'base-uri'
    case 44115:                     // 'attribute' 'boundary-space'
    case 44154:                     // 'element' 'boundary-space'
    case 44219:                     // 'namespace' 'boundary-space'
    case 44252:                     // 'processing-instruction' 'boundary-space'
    case 44627:                     // 'attribute' 'break'
    case 44666:                     // 'element' 'break'
    case 44731:                     // 'namespace' 'break'
    case 44764:                     // 'processing-instruction' 'break'
    case 47187:                     // 'attribute' 'catch'
    case 47226:                     // 'element' 'catch'
    case 47291:                     // 'namespace' 'catch'
    case 47324:                     // 'processing-instruction' 'catch'
    case 48211:                     // 'attribute' 'child'
    case 48250:                     // 'element' 'child'
    case 48315:                     // 'namespace' 'child'
    case 48348:                     // 'processing-instruction' 'child'
    case 49747:                     // 'attribute' 'comment'
    case 49786:                     // 'element' 'comment'
    case 49851:                     // 'namespace' 'comment'
    case 49884:                     // 'processing-instruction' 'comment'
    case 50259:                     // 'attribute' 'constraint'
    case 50298:                     // 'element' 'constraint'
    case 50363:                     // 'namespace' 'constraint'
    case 50396:                     // 'processing-instruction' 'constraint'
    case 50771:                     // 'attribute' 'construction'
    case 50810:                     // 'element' 'construction'
    case 50875:                     // 'namespace' 'construction'
    case 50908:                     // 'processing-instruction' 'construction'
    case 52307:                     // 'attribute' 'context'
    case 52346:                     // 'element' 'context'
    case 52411:                     // 'namespace' 'context'
    case 52444:                     // 'processing-instruction' 'context'
    case 52819:                     // 'attribute' 'continue'
    case 52858:                     // 'element' 'continue'
    case 52923:                     // 'namespace' 'continue'
    case 52956:                     // 'processing-instruction' 'continue'
    case 53331:                     // 'attribute' 'copy'
    case 53370:                     // 'element' 'copy'
    case 53435:                     // 'namespace' 'copy'
    case 53468:                     // 'processing-instruction' 'copy'
    case 53843:                     // 'attribute' 'copy-namespaces'
    case 53882:                     // 'element' 'copy-namespaces'
    case 53947:                     // 'namespace' 'copy-namespaces'
    case 53980:                     // 'processing-instruction' 'copy-namespaces'
    case 54867:                     // 'attribute' 'decimal-format'
    case 54906:                     // 'element' 'decimal-format'
    case 54971:                     // 'namespace' 'decimal-format'
    case 55004:                     // 'processing-instruction' 'decimal-format'
    case 55891:                     // 'attribute' 'declare'
    case 55930:                     // 'element' 'declare'
    case 55995:                     // 'namespace' 'declare'
    case 56028:                     // 'processing-instruction' 'declare'
    case 56915:                     // 'attribute' 'delete'
    case 56954:                     // 'element' 'delete'
    case 57019:                     // 'namespace' 'delete'
    case 57052:                     // 'processing-instruction' 'delete'
    case 57427:                     // 'attribute' 'descendant'
    case 57466:                     // 'element' 'descendant'
    case 57531:                     // 'namespace' 'descendant'
    case 57564:                     // 'processing-instruction' 'descendant'
    case 57939:                     // 'attribute' 'descendant-or-self'
    case 57978:                     // 'element' 'descendant-or-self'
    case 58043:                     // 'namespace' 'descendant-or-self'
    case 58076:                     // 'processing-instruction' 'descendant-or-self'
    case 61523:                     // 'attribute' 'document'
    case 61562:                     // 'element' 'document'
    case 61627:                     // 'namespace' 'document'
    case 61660:                     // 'processing-instruction' 'document'
    case 62035:                     // 'attribute' 'document-node'
    case 62074:                     // 'element' 'document-node'
    case 62139:                     // 'namespace' 'document-node'
    case 62172:                     // 'processing-instruction' 'document-node'
    case 62547:                     // 'attribute' 'element'
    case 62586:                     // 'element' 'element'
    case 62651:                     // 'namespace' 'element'
    case 62684:                     // 'processing-instruction' 'element'
    case 64083:                     // 'attribute' 'empty-sequence'
    case 64122:                     // 'element' 'empty-sequence'
    case 64187:                     // 'namespace' 'empty-sequence'
    case 64220:                     // 'processing-instruction' 'empty-sequence'
    case 64595:                     // 'attribute' 'encoding'
    case 64634:                     // 'element' 'encoding'
    case 64699:                     // 'namespace' 'encoding'
    case 64732:                     // 'processing-instruction' 'encoding'
    case 66643:                     // 'attribute' 'every'
    case 66682:                     // 'element' 'every'
    case 66747:                     // 'namespace' 'every'
    case 66780:                     // 'processing-instruction' 'every'
    case 68179:                     // 'attribute' 'exit'
    case 68218:                     // 'element' 'exit'
    case 68283:                     // 'namespace' 'exit'
    case 68316:                     // 'processing-instruction' 'exit'
    case 68691:                     // 'attribute' 'external'
    case 68730:                     // 'element' 'external'
    case 68795:                     // 'namespace' 'external'
    case 68828:                     // 'processing-instruction' 'external'
    case 69203:                     // 'attribute' 'false'
    case 69242:                     // 'element' 'false'
    case 69307:                     // 'namespace' 'false'
    case 69340:                     // 'processing-instruction' 'false'
    case 69715:                     // 'attribute' 'first'
    case 69754:                     // 'element' 'first'
    case 69819:                     // 'namespace' 'first'
    case 69852:                     // 'processing-instruction' 'first'
    case 70227:                     // 'attribute' 'following'
    case 70266:                     // 'element' 'following'
    case 70331:                     // 'namespace' 'following'
    case 70364:                     // 'processing-instruction' 'following'
    case 70739:                     // 'attribute' 'following-sibling'
    case 70778:                     // 'element' 'following-sibling'
    case 70843:                     // 'namespace' 'following-sibling'
    case 70876:                     // 'processing-instruction' 'following-sibling'
    case 72787:                     // 'attribute' 'from'
    case 72826:                     // 'element' 'from'
    case 72891:                     // 'namespace' 'from'
    case 72924:                     // 'processing-instruction' 'from'
    case 73299:                     // 'attribute' 'ft-option'
    case 73338:                     // 'element' 'ft-option'
    case 73403:                     // 'namespace' 'ft-option'
    case 73436:                     // 'processing-instruction' 'ft-option'
    case 75347:                     // 'attribute' 'function'
    case 75386:                     // 'element' 'function'
    case 75451:                     // 'namespace' 'function'
    case 75484:                     // 'processing-instruction' 'function'
    case 78931:                     // 'attribute' 'if'
    case 78970:                     // 'element' 'if'
    case 79035:                     // 'namespace' 'if'
    case 79068:                     // 'processing-instruction' 'if'
    case 79443:                     // 'attribute' 'import'
    case 79482:                     // 'element' 'import'
    case 79547:                     // 'namespace' 'import'
    case 79580:                     // 'processing-instruction' 'import'
    case 79955:                     // 'attribute' 'in'
    case 79994:                     // 'element' 'in'
    case 80059:                     // 'namespace' 'in'
    case 80092:                     // 'processing-instruction' 'in'
    case 80467:                     // 'attribute' 'index'
    case 80506:                     // 'element' 'index'
    case 80571:                     // 'namespace' 'index'
    case 80604:                     // 'processing-instruction' 'index'
    case 82515:                     // 'attribute' 'insert'
    case 82554:                     // 'element' 'insert'
    case 82619:                     // 'namespace' 'insert'
    case 82652:                     // 'processing-instruction' 'insert'
    case 83539:                     // 'attribute' 'integrity'
    case 83578:                     // 'element' 'integrity'
    case 83643:                     // 'namespace' 'integrity'
    case 83676:                     // 'processing-instruction' 'integrity'
    case 85587:                     // 'attribute' 'item'
    case 85626:                     // 'element' 'item'
    case 85691:                     // 'namespace' 'item'
    case 85724:                     // 'processing-instruction' 'item'
    case 86099:                     // 'attribute' 'json'
    case 86138:                     // 'element' 'json'
    case 86203:                     // 'namespace' 'json'
    case 86236:                     // 'processing-instruction' 'json'
    case 86611:                     // 'attribute' 'json-item'
    case 86650:                     // 'element' 'json-item'
    case 87123:                     // 'attribute' 'jsoniq'
    case 87162:                     // 'element' 'jsoniq'
    case 87227:                     // 'namespace' 'jsoniq'
    case 87260:                     // 'processing-instruction' 'jsoniq'
    case 88659:                     // 'attribute' 'last'
    case 88698:                     // 'element' 'last'
    case 88763:                     // 'namespace' 'last'
    case 88796:                     // 'processing-instruction' 'last'
    case 89171:                     // 'attribute' 'lax'
    case 89210:                     // 'element' 'lax'
    case 89275:                     // 'namespace' 'lax'
    case 89308:                     // 'processing-instruction' 'lax'
    case 91731:                     // 'attribute' 'loop'
    case 91770:                     // 'element' 'loop'
    case 91835:                     // 'namespace' 'loop'
    case 91868:                     // 'processing-instruction' 'loop'
    case 94803:                     // 'attribute' 'module'
    case 94842:                     // 'element' 'module'
    case 94907:                     // 'namespace' 'module'
    case 94940:                     // 'processing-instruction' 'module'
    case 95827:                     // 'attribute' 'namespace'
    case 95866:                     // 'element' 'namespace'
    case 95931:                     // 'namespace' 'namespace'
    case 95964:                     // 'processing-instruction' 'namespace'
    case 96339:                     // 'attribute' 'namespace-node'
    case 96378:                     // 'element' 'namespace-node'
    case 96443:                     // 'namespace' 'namespace-node'
    case 96476:                     // 'processing-instruction' 'namespace-node'
    case 99411:                     // 'attribute' 'node'
    case 99450:                     // 'element' 'node'
    case 99515:                     // 'namespace' 'node'
    case 99548:                     // 'processing-instruction' 'node'
    case 99923:                     // 'attribute' 'nodes'
    case 99962:                     // 'element' 'nodes'
    case 100027:                    // 'namespace' 'nodes'
    case 100060:                    // 'processing-instruction' 'nodes'
    case 100947:                    // 'attribute' 'null'
    case 100986:                    // 'element' 'null'
    case 101051:                    // 'namespace' 'null'
    case 101084:                    // 'processing-instruction' 'null'
    case 101459:                    // 'attribute' 'object'
    case 101498:                    // 'element' 'object'
    case 101563:                    // 'namespace' 'object'
    case 101596:                    // 'processing-instruction' 'object'
    case 104019:                    // 'attribute' 'option'
    case 104058:                    // 'element' 'option'
    case 104123:                    // 'namespace' 'option'
    case 104156:                    // 'processing-instruction' 'option'
    case 105555:                    // 'attribute' 'ordered'
    case 105594:                    // 'element' 'ordered'
    case 105659:                    // 'namespace' 'ordered'
    case 105692:                    // 'processing-instruction' 'ordered'
    case 106067:                    // 'attribute' 'ordering'
    case 106106:                    // 'element' 'ordering'
    case 106171:                    // 'namespace' 'ordering'
    case 106204:                    // 'processing-instruction' 'ordering'
    case 107603:                    // 'attribute' 'parent'
    case 107642:                    // 'element' 'parent'
    case 107707:                    // 'namespace' 'parent'
    case 107740:                    // 'processing-instruction' 'parent'
    case 110675:                    // 'attribute' 'preceding'
    case 110714:                    // 'element' 'preceding'
    case 110779:                    // 'namespace' 'preceding'
    case 110812:                    // 'processing-instruction' 'preceding'
    case 111187:                    // 'attribute' 'preceding-sibling'
    case 111226:                    // 'element' 'preceding-sibling'
    case 111291:                    // 'namespace' 'preceding-sibling'
    case 111324:                    // 'processing-instruction' 'preceding-sibling'
    case 112723:                    // 'attribute' 'processing-instruction'
    case 112762:                    // 'element' 'processing-instruction'
    case 112827:                    // 'namespace' 'processing-instruction'
    case 112860:                    // 'processing-instruction' 'processing-instruction'
    case 113747:                    // 'attribute' 'rename'
    case 113786:                    // 'element' 'rename'
    case 113851:                    // 'namespace' 'rename'
    case 113884:                    // 'processing-instruction' 'rename'
    case 114259:                    // 'attribute' 'replace'
    case 114298:                    // 'element' 'replace'
    case 114363:                    // 'namespace' 'replace'
    case 114396:                    // 'processing-instruction' 'replace'
    case 115283:                    // 'attribute' 'returning'
    case 115322:                    // 'element' 'returning'
    case 115387:                    // 'namespace' 'returning'
    case 115420:                    // 'processing-instruction' 'returning'
    case 115795:                    // 'attribute' 'revalidation'
    case 115834:                    // 'element' 'revalidation'
    case 115899:                    // 'namespace' 'revalidation'
    case 115932:                    // 'processing-instruction' 'revalidation'
    case 117331:                    // 'attribute' 'schema'
    case 117370:                    // 'element' 'schema'
    case 117435:                    // 'namespace' 'schema'
    case 117468:                    // 'processing-instruction' 'schema'
    case 117843:                    // 'attribute' 'schema-attribute'
    case 117882:                    // 'element' 'schema-attribute'
    case 117947:                    // 'namespace' 'schema-attribute'
    case 117980:                    // 'processing-instruction' 'schema-attribute'
    case 118355:                    // 'attribute' 'schema-element'
    case 118394:                    // 'element' 'schema-element'
    case 118459:                    // 'namespace' 'schema-element'
    case 118492:                    // 'processing-instruction' 'schema-element'
    case 118867:                    // 'attribute' 'score'
    case 118906:                    // 'element' 'score'
    case 118971:                    // 'namespace' 'score'
    case 119004:                    // 'processing-instruction' 'score'
    case 119379:                    // 'attribute' 'select'
    case 119418:                    // 'element' 'select'
    case 119483:                    // 'namespace' 'select'
    case 119516:                    // 'processing-instruction' 'select'
    case 119891:                    // 'attribute' 'self'
    case 119930:                    // 'element' 'self'
    case 119995:                    // 'namespace' 'self'
    case 120028:                    // 'processing-instruction' 'self'
    case 122451:                    // 'attribute' 'sliding'
    case 122490:                    // 'element' 'sliding'
    case 122555:                    // 'namespace' 'sliding'
    case 122588:                    // 'processing-instruction' 'sliding'
    case 122963:                    // 'attribute' 'some'
    case 123002:                    // 'element' 'some'
    case 123067:                    // 'namespace' 'some'
    case 123100:                    // 'processing-instruction' 'some'
    case 125523:                    // 'attribute' 'strict'
    case 125562:                    // 'element' 'strict'
    case 125627:                    // 'namespace' 'strict'
    case 125660:                    // 'processing-instruction' 'strict'
    case 126547:                    // 'attribute' 'structured-item'
    case 126586:                    // 'element' 'structured-item'
    case 127059:                    // 'attribute' 'switch'
    case 127098:                    // 'element' 'switch'
    case 127163:                    // 'namespace' 'switch'
    case 127196:                    // 'processing-instruction' 'switch'
    case 127571:                    // 'attribute' 'text'
    case 127610:                    // 'element' 'text'
    case 127675:                    // 'namespace' 'text'
    case 127708:                    // 'processing-instruction' 'text'
    case 130643:                    // 'attribute' 'true'
    case 130682:                    // 'element' 'true'
    case 130747:                    // 'namespace' 'true'
    case 130780:                    // 'processing-instruction' 'true'
    case 131155:                    // 'attribute' 'try'
    case 131194:                    // 'element' 'try'
    case 131259:                    // 'namespace' 'try'
    case 131292:                    // 'processing-instruction' 'try'
    case 131667:                    // 'attribute' 'tumbling'
    case 131706:                    // 'element' 'tumbling'
    case 131771:                    // 'namespace' 'tumbling'
    case 131804:                    // 'processing-instruction' 'tumbling'
    case 132179:                    // 'attribute' 'type'
    case 132218:                    // 'element' 'type'
    case 132283:                    // 'namespace' 'type'
    case 132316:                    // 'processing-instruction' 'type'
    case 132691:                    // 'attribute' 'typeswitch'
    case 132730:                    // 'element' 'typeswitch'
    case 132795:                    // 'namespace' 'typeswitch'
    case 132828:                    // 'processing-instruction' 'typeswitch'
    case 134227:                    // 'attribute' 'unordered'
    case 134266:                    // 'element' 'unordered'
    case 134331:                    // 'namespace' 'unordered'
    case 134364:                    // 'processing-instruction' 'unordered'
    case 134739:                    // 'attribute' 'updating'
    case 134778:                    // 'element' 'updating'
    case 134843:                    // 'namespace' 'updating'
    case 134876:                    // 'processing-instruction' 'updating'
    case 136275:                    // 'attribute' 'validate'
    case 136314:                    // 'element' 'validate'
    case 136379:                    // 'namespace' 'validate'
    case 136412:                    // 'processing-instruction' 'validate'
    case 136787:                    // 'attribute' 'value'
    case 136826:                    // 'element' 'value'
    case 136891:                    // 'namespace' 'value'
    case 136924:                    // 'processing-instruction' 'value'
    case 137299:                    // 'attribute' 'variable'
    case 137338:                    // 'element' 'variable'
    case 137403:                    // 'namespace' 'variable'
    case 137436:                    // 'processing-instruction' 'variable'
    case 137811:                    // 'attribute' 'version'
    case 137850:                    // 'element' 'version'
    case 137915:                    // 'namespace' 'version'
    case 137948:                    // 'processing-instruction' 'version'
    case 139859:                    // 'attribute' 'while'
    case 139898:                    // 'element' 'while'
    case 139963:                    // 'namespace' 'while'
    case 139996:                    // 'processing-instruction' 'while'
    case 143955:                    // 'attribute' '{'
    case 143969:                    // 'comment' '{'
    case 143992:                    // 'document' '{'
    case 143994:                    // 'element' '{'
    case 144059:                    // 'namespace' '{'
    case 144078:                    // 'ordered' '{'
    case 144092:                    // 'processing-instruction' '{'
    case 144121:                    // 'text' '{'
    case 144134:                    // 'unordered' '{'
      parse_PostfixExpr();
      break;
    default:
      parse_AxisStep();
    }
    eventHandler.endNonterminal("StepExpr", e0);
  }

  function try_StepExpr()
  {
    switch (l1)
    {
    case 83:                        // 'attribute'
      lookahead2W(288);             // EQName^Token | S^WS | EOF | '!' | '!=' | '#' | '(' | '(:' | ')' | '*' | '+' |
      break;
    case 122:                       // 'element'
      lookahead2W(287);             // EQName^Token | S^WS | EOF | '!' | '!=' | '#' | '(' | '(:' | ')' | '*' | '+' |
      break;
    case 187:                       // 'namespace'
    case 220:                       // 'processing-instruction'
      lookahead2W(285);             // NCName^Token | S^WS | EOF | '!' | '!=' | '#' | '(' | '(:' | ')' | '*' | '+' |
      break;
    case 135:                       // 'false'
    case 197:                       // 'null'
    case 255:                       // 'true'
      lookahead2W(237);             // S^WS | EOF | '!' | '!=' | '#' | '(' | '(:' | ')' | '*' | '+' | ',' | '-' | '.' |
      break;
    case 97:                        // 'comment'
    case 120:                       // 'document'
    case 206:                       // 'ordered'
    case 249:                       // 'text'
    case 262:                       // 'unordered'
      lookahead2W(239);             // S^WS | EOF | '!' | '!=' | '#' | '(' | '(:' | ')' | '*' | '+' | ',' | '-' | '/' |
      break;
    case 79:                        // 'array'
    case 125:                       // 'empty-sequence'
    case 154:                       // 'if'
    case 167:                       // 'item'
    case 169:                       // 'json-item'
    case 247:                       // 'structured-item'
    case 248:                       // 'switch'
    case 259:                       // 'typeswitch'
      lookahead2W(230);             // S^WS | EOF | '!' | '!=' | '#' | '(:' | ')' | '*' | '+' | ',' | '-' | '/' | '//' |
      break;
    case 74:                        // 'ancestor'
    case 75:                        // 'ancestor-or-self'
    case 94:                        // 'child'
    case 112:                       // 'descendant'
    case 113:                       // 'descendant-or-self'
    case 137:                       // 'following'
    case 138:                       // 'following-sibling'
    case 210:                       // 'parent'
    case 216:                       // 'preceding'
    case 217:                       // 'preceding-sibling'
    case 234:                       // 'self'
      lookahead2W(238);             // S^WS | EOF | '!' | '!=' | '#' | '(' | '(:' | ')' | '*' | '+' | ',' | '-' | '/' |
      break;
    case 6:                         // EQName^Token
    case 71:                        // 'after'
    case 73:                        // 'allowing'
    case 76:                        // 'and'
    case 78:                        // 'append'
    case 80:                        // 'as'
    case 81:                        // 'ascending'
    case 82:                        // 'at'
    case 84:                        // 'base-uri'
    case 85:                        // 'before'
    case 86:                        // 'boundary-space'
    case 87:                        // 'break'
    case 89:                        // 'case'
    case 90:                        // 'cast'
    case 91:                        // 'castable'
    case 92:                        // 'catch'
    case 95:                        // 'collation'
    case 98:                        // 'constraint'
    case 99:                        // 'construction'
    case 102:                       // 'context'
    case 103:                       // 'continue'
    case 104:                       // 'copy'
    case 105:                       // 'copy-namespaces'
    case 106:                       // 'count'
    case 107:                       // 'decimal-format'
    case 109:                       // 'declare'
    case 110:                       // 'default'
    case 111:                       // 'delete'
    case 114:                       // 'descending'
    case 119:                       // 'div'
    case 121:                       // 'document-node'
    case 123:                       // 'else'
    case 124:                       // 'empty'
    case 126:                       // 'encoding'
    case 127:                       // 'end'
    case 129:                       // 'eq'
    case 130:                       // 'every'
    case 132:                       // 'except'
    case 133:                       // 'exit'
    case 134:                       // 'external'
    case 136:                       // 'first'
    case 139:                       // 'for'
    case 142:                       // 'from'
    case 143:                       // 'ft-option'
    case 147:                       // 'function'
    case 148:                       // 'ge'
    case 150:                       // 'group'
    case 152:                       // 'gt'
    case 153:                       // 'idiv'
    case 155:                       // 'import'
    case 156:                       // 'in'
    case 157:                       // 'index'
    case 161:                       // 'insert'
    case 162:                       // 'instance'
    case 163:                       // 'integrity'
    case 164:                       // 'intersect'
    case 165:                       // 'into'
    case 166:                       // 'is'
    case 168:                       // 'json'
    case 170:                       // 'jsoniq'
    case 173:                       // 'last'
    case 174:                       // 'lax'
    case 175:                       // 'le'
    case 177:                       // 'let'
    case 179:                       // 'loop'
    case 181:                       // 'lt'
    case 183:                       // 'mod'
    case 184:                       // 'modify'
    case 185:                       // 'module'
    case 188:                       // 'namespace-node'
    case 189:                       // 'ne'
    case 194:                       // 'node'
    case 195:                       // 'nodes'
    case 198:                       // 'object'
    case 202:                       // 'only'
    case 203:                       // 'option'
    case 204:                       // 'or'
    case 205:                       // 'order'
    case 207:                       // 'ordering'
    case 222:                       // 'rename'
    case 223:                       // 'replace'
    case 224:                       // 'return'
    case 225:                       // 'returning'
    case 226:                       // 'revalidation'
    case 228:                       // 'satisfies'
    case 229:                       // 'schema'
    case 230:                       // 'schema-attribute'
    case 231:                       // 'schema-element'
    case 232:                       // 'score'
    case 233:                       // 'select'
    case 239:                       // 'sliding'
    case 240:                       // 'some'
    case 241:                       // 'stable'
    case 242:                       // 'start'
    case 245:                       // 'strict'
    case 253:                       // 'to'
    case 254:                       // 'treat'
    case 256:                       // 'try'
    case 257:                       // 'tumbling'
    case 258:                       // 'type'
    case 260:                       // 'union'
    case 263:                       // 'updating'
    case 266:                       // 'validate'
    case 267:                       // 'value'
    case 268:                       // 'variable'
    case 269:                       // 'version'
    case 272:                       // 'where'
    case 273:                       // 'while'
    case 276:                       // 'with'
      lookahead2W(234);             // S^WS | EOF | '!' | '!=' | '#' | '(' | '(:' | ')' | '*' | '+' | ',' | '-' | '/' |
      break;
    default:
      lk = l1;
    }
    if (lk == 12935                 // 'false' EOF
     || lk == 12997                 // 'null' EOF
     || lk == 13055                 // 'true' EOF
     || lk == 13447                 // 'false' '!'
     || lk == 13509                 // 'null' '!'
     || lk == 13567                 // 'true' '!'
     || lk == 13959                 // 'false' '!='
     || lk == 14021                 // 'null' '!='
     || lk == 14079                 // 'true' '!='
     || lk == 19591                 // 'false' ')'
     || lk == 19653                 // 'null' ')'
     || lk == 19711                 // 'true' ')'
     || lk == 20103                 // 'false' '*'
     || lk == 20165                 // 'null' '*'
     || lk == 20223                 // 'true' '*'
     || lk == 21127                 // 'false' '+'
     || lk == 21189                 // 'null' '+'
     || lk == 21247                 // 'true' '+'
     || lk == 21639                 // 'false' ','
     || lk == 21701                 // 'null' ','
     || lk == 21759                 // 'true' ','
     || lk == 22151                 // 'false' '-'
     || lk == 22213                 // 'null' '-'
     || lk == 22271                 // 'true' '-'
     || lk == 24199                 // 'false' '/'
     || lk == 24261                 // 'null' '/'
     || lk == 24319                 // 'true' '/'
     || lk == 24711                 // 'false' '//'
     || lk == 24773                 // 'null' '//'
     || lk == 24831                 // 'true' '//'
     || lk == 25735                 // 'false' ':'
     || lk == 25797                 // 'null' ':'
     || lk == 25855                 // 'true' ':'
     || lk == 27783                 // 'false' ';'
     || lk == 27845                 // 'null' ';'
     || lk == 27903                 // 'true' ';'
     || lk == 28295                 // 'false' '<'
     || lk == 28357                 // 'null' '<'
     || lk == 28415                 // 'true' '<'
     || lk == 29831                 // 'false' '<<'
     || lk == 29893                 // 'null' '<<'
     || lk == 29951                 // 'true' '<<'
     || lk == 30343                 // 'false' '<='
     || lk == 30405                 // 'null' '<='
     || lk == 30463                 // 'true' '<='
     || lk == 31367                 // 'false' '='
     || lk == 31429                 // 'null' '='
     || lk == 31487                 // 'true' '='
     || lk == 31879                 // 'false' '>'
     || lk == 31941                 // 'null' '>'
     || lk == 31999                 // 'true' '>'
     || lk == 32391                 // 'false' '>='
     || lk == 32453                 // 'null' '>='
     || lk == 32511                 // 'true' '>='
     || lk == 32903                 // 'false' '>>'
     || lk == 32965                 // 'null' '>>'
     || lk == 33023                 // 'true' '>>'
     || lk == 35463                 // 'false' '['
     || lk == 35525                 // 'null' '['
     || lk == 35583                 // 'true' '['
     || lk == 35975                 // 'false' ']'
     || lk == 36037                 // 'null' ']'
     || lk == 36095                 // 'true' ']'
     || lk == 36435                 // 'attribute' 'after'
     || lk == 36474                 // 'element' 'after'
     || lk == 36487                 // 'false' 'after'
     || lk == 36539                 // 'namespace' 'after'
     || lk == 36549                 // 'null' 'after'
     || lk == 36572                 // 'processing-instruction' 'after'
     || lk == 36607                 // 'true' 'after'
     || lk == 38995                 // 'attribute' 'and'
     || lk == 39034                 // 'element' 'and'
     || lk == 39047                 // 'false' 'and'
     || lk == 39099                 // 'namespace' 'and'
     || lk == 39109                 // 'null' 'and'
     || lk == 39132                 // 'processing-instruction' 'and'
     || lk == 39167                 // 'true' 'and'
     || lk == 41043                 // 'attribute' 'as'
     || lk == 41082                 // 'element' 'as'
     || lk == 41095                 // 'false' 'as'
     || lk == 41147                 // 'namespace' 'as'
     || lk == 41157                 // 'null' 'as'
     || lk == 41180                 // 'processing-instruction' 'as'
     || lk == 41215                 // 'true' 'as'
     || lk == 41555                 // 'attribute' 'ascending'
     || lk == 41594                 // 'element' 'ascending'
     || lk == 41607                 // 'false' 'ascending'
     || lk == 41659                 // 'namespace' 'ascending'
     || lk == 41669                 // 'null' 'ascending'
     || lk == 41692                 // 'processing-instruction' 'ascending'
     || lk == 41727                 // 'true' 'ascending'
     || lk == 42067                 // 'attribute' 'at'
     || lk == 42106                 // 'element' 'at'
     || lk == 42119                 // 'false' 'at'
     || lk == 42171                 // 'namespace' 'at'
     || lk == 42181                 // 'null' 'at'
     || lk == 42204                 // 'processing-instruction' 'at'
     || lk == 42239                 // 'true' 'at'
     || lk == 43603                 // 'attribute' 'before'
     || lk == 43642                 // 'element' 'before'
     || lk == 43655                 // 'false' 'before'
     || lk == 43707                 // 'namespace' 'before'
     || lk == 43717                 // 'null' 'before'
     || lk == 43740                 // 'processing-instruction' 'before'
     || lk == 43775                 // 'true' 'before'
     || lk == 45191                 // 'false' 'by'
     || lk == 45253                 // 'null' 'by'
     || lk == 45311                 // 'true' 'by'
     || lk == 45651                 // 'attribute' 'case'
     || lk == 45690                 // 'element' 'case'
     || lk == 45703                 // 'false' 'case'
     || lk == 45755                 // 'namespace' 'case'
     || lk == 45765                 // 'null' 'case'
     || lk == 45788                 // 'processing-instruction' 'case'
     || lk == 45823                 // 'true' 'case'
     || lk == 46163                 // 'attribute' 'cast'
     || lk == 46202                 // 'element' 'cast'
     || lk == 46215                 // 'false' 'cast'
     || lk == 46267                 // 'namespace' 'cast'
     || lk == 46277                 // 'null' 'cast'
     || lk == 46300                 // 'processing-instruction' 'cast'
     || lk == 46335                 // 'true' 'cast'
     || lk == 46675                 // 'attribute' 'castable'
     || lk == 46714                 // 'element' 'castable'
     || lk == 46727                 // 'false' 'castable'
     || lk == 46779                 // 'namespace' 'castable'
     || lk == 46789                 // 'null' 'castable'
     || lk == 46812                 // 'processing-instruction' 'castable'
     || lk == 46847                 // 'true' 'castable'
     || lk == 48723                 // 'attribute' 'collation'
     || lk == 48762                 // 'element' 'collation'
     || lk == 48775                 // 'false' 'collation'
     || lk == 48827                 // 'namespace' 'collation'
     || lk == 48837                 // 'null' 'collation'
     || lk == 48860                 // 'processing-instruction' 'collation'
     || lk == 48895                 // 'true' 'collation'
     || lk == 51335                 // 'false' 'contains'
     || lk == 51397                 // 'null' 'contains'
     || lk == 51455                 // 'true' 'contains'
     || lk == 54355                 // 'attribute' 'count'
     || lk == 54394                 // 'element' 'count'
     || lk == 54407                 // 'false' 'count'
     || lk == 54459                 // 'namespace' 'count'
     || lk == 54469                 // 'null' 'count'
     || lk == 54492                 // 'processing-instruction' 'count'
     || lk == 54527                 // 'true' 'count'
     || lk == 56403                 // 'attribute' 'default'
     || lk == 56442                 // 'element' 'default'
     || lk == 56455                 // 'false' 'default'
     || lk == 56507                 // 'namespace' 'default'
     || lk == 56517                 // 'null' 'default'
     || lk == 56540                 // 'processing-instruction' 'default'
     || lk == 56575                 // 'true' 'default'
     || lk == 58451                 // 'attribute' 'descending'
     || lk == 58490                 // 'element' 'descending'
     || lk == 58503                 // 'false' 'descending'
     || lk == 58555                 // 'namespace' 'descending'
     || lk == 58565                 // 'null' 'descending'
     || lk == 58588                 // 'processing-instruction' 'descending'
     || lk == 58623                 // 'true' 'descending'
     || lk == 61011                 // 'attribute' 'div'
     || lk == 61050                 // 'element' 'div'
     || lk == 61063                 // 'false' 'div'
     || lk == 61115                 // 'namespace' 'div'
     || lk == 61125                 // 'null' 'div'
     || lk == 61148                 // 'processing-instruction' 'div'
     || lk == 61183                 // 'true' 'div'
     || lk == 63059                 // 'attribute' 'else'
     || lk == 63098                 // 'element' 'else'
     || lk == 63111                 // 'false' 'else'
     || lk == 63163                 // 'namespace' 'else'
     || lk == 63173                 // 'null' 'else'
     || lk == 63196                 // 'processing-instruction' 'else'
     || lk == 63231                 // 'true' 'else'
     || lk == 63571                 // 'attribute' 'empty'
     || lk == 63610                 // 'element' 'empty'
     || lk == 63623                 // 'false' 'empty'
     || lk == 63675                 // 'namespace' 'empty'
     || lk == 63685                 // 'null' 'empty'
     || lk == 63708                 // 'processing-instruction' 'empty'
     || lk == 63743                 // 'true' 'empty'
     || lk == 65107                 // 'attribute' 'end'
     || lk == 65146                 // 'element' 'end'
     || lk == 65159                 // 'false' 'end'
     || lk == 65211                 // 'namespace' 'end'
     || lk == 65221                 // 'null' 'end'
     || lk == 65244                 // 'processing-instruction' 'end'
     || lk == 65279                 // 'true' 'end'
     || lk == 66131                 // 'attribute' 'eq'
     || lk == 66170                 // 'element' 'eq'
     || lk == 66183                 // 'false' 'eq'
     || lk == 66235                 // 'namespace' 'eq'
     || lk == 66245                 // 'null' 'eq'
     || lk == 66268                 // 'processing-instruction' 'eq'
     || lk == 66303                 // 'true' 'eq'
     || lk == 67667                 // 'attribute' 'except'
     || lk == 67706                 // 'element' 'except'
     || lk == 67719                 // 'false' 'except'
     || lk == 67771                 // 'namespace' 'except'
     || lk == 67781                 // 'null' 'except'
     || lk == 67804                 // 'processing-instruction' 'except'
     || lk == 67839                 // 'true' 'except'
     || lk == 71251                 // 'attribute' 'for'
     || lk == 71290                 // 'element' 'for'
     || lk == 71303                 // 'false' 'for'
     || lk == 71355                 // 'namespace' 'for'
     || lk == 71365                 // 'null' 'for'
     || lk == 71388                 // 'processing-instruction' 'for'
     || lk == 71423                 // 'true' 'for'
     || lk == 75859                 // 'attribute' 'ge'
     || lk == 75898                 // 'element' 'ge'
     || lk == 75911                 // 'false' 'ge'
     || lk == 75963                 // 'namespace' 'ge'
     || lk == 75973                 // 'null' 'ge'
     || lk == 75996                 // 'processing-instruction' 'ge'
     || lk == 76031                 // 'true' 'ge'
     || lk == 76883                 // 'attribute' 'group'
     || lk == 76922                 // 'element' 'group'
     || lk == 76935                 // 'false' 'group'
     || lk == 76987                 // 'namespace' 'group'
     || lk == 76997                 // 'null' 'group'
     || lk == 77020                 // 'processing-instruction' 'group'
     || lk == 77055                 // 'true' 'group'
     || lk == 77907                 // 'attribute' 'gt'
     || lk == 77946                 // 'element' 'gt'
     || lk == 77959                 // 'false' 'gt'
     || lk == 78011                 // 'namespace' 'gt'
     || lk == 78021                 // 'null' 'gt'
     || lk == 78044                 // 'processing-instruction' 'gt'
     || lk == 78079                 // 'true' 'gt'
     || lk == 78419                 // 'attribute' 'idiv'
     || lk == 78458                 // 'element' 'idiv'
     || lk == 78471                 // 'false' 'idiv'
     || lk == 78523                 // 'namespace' 'idiv'
     || lk == 78533                 // 'null' 'idiv'
     || lk == 78556                 // 'processing-instruction' 'idiv'
     || lk == 78591                 // 'true' 'idiv'
     || lk == 83027                 // 'attribute' 'instance'
     || lk == 83066                 // 'element' 'instance'
     || lk == 83079                 // 'false' 'instance'
     || lk == 83131                 // 'namespace' 'instance'
     || lk == 83141                 // 'null' 'instance'
     || lk == 83164                 // 'processing-instruction' 'instance'
     || lk == 83199                 // 'true' 'instance'
     || lk == 84051                 // 'attribute' 'intersect'
     || lk == 84090                 // 'element' 'intersect'
     || lk == 84103                 // 'false' 'intersect'
     || lk == 84155                 // 'namespace' 'intersect'
     || lk == 84165                 // 'null' 'intersect'
     || lk == 84188                 // 'processing-instruction' 'intersect'
     || lk == 84223                 // 'true' 'intersect'
     || lk == 84563                 // 'attribute' 'into'
     || lk == 84602                 // 'element' 'into'
     || lk == 84615                 // 'false' 'into'
     || lk == 84667                 // 'namespace' 'into'
     || lk == 84677                 // 'null' 'into'
     || lk == 84700                 // 'processing-instruction' 'into'
     || lk == 84735                 // 'true' 'into'
     || lk == 85075                 // 'attribute' 'is'
     || lk == 85114                 // 'element' 'is'
     || lk == 85127                 // 'false' 'is'
     || lk == 85179                 // 'namespace' 'is'
     || lk == 85189                 // 'null' 'is'
     || lk == 85212                 // 'processing-instruction' 'is'
     || lk == 85247                 // 'true' 'is'
     || lk == 89683                 // 'attribute' 'le'
     || lk == 89722                 // 'element' 'le'
     || lk == 89735                 // 'false' 'le'
     || lk == 89787                 // 'namespace' 'le'
     || lk == 89797                 // 'null' 'le'
     || lk == 89820                 // 'processing-instruction' 'le'
     || lk == 89855                 // 'true' 'le'
     || lk == 90707                 // 'attribute' 'let'
     || lk == 90746                 // 'element' 'let'
     || lk == 90759                 // 'false' 'let'
     || lk == 90811                 // 'namespace' 'let'
     || lk == 90821                 // 'null' 'let'
     || lk == 90844                 // 'processing-instruction' 'let'
     || lk == 90879                 // 'true' 'let'
     || lk == 92755                 // 'attribute' 'lt'
     || lk == 92794                 // 'element' 'lt'
     || lk == 92807                 // 'false' 'lt'
     || lk == 92859                 // 'namespace' 'lt'
     || lk == 92869                 // 'null' 'lt'
     || lk == 92892                 // 'processing-instruction' 'lt'
     || lk == 92927                 // 'true' 'lt'
     || lk == 93779                 // 'attribute' 'mod'
     || lk == 93818                 // 'element' 'mod'
     || lk == 93831                 // 'false' 'mod'
     || lk == 93883                 // 'namespace' 'mod'
     || lk == 93893                 // 'null' 'mod'
     || lk == 93916                 // 'processing-instruction' 'mod'
     || lk == 93951                 // 'true' 'mod'
     || lk == 94291                 // 'attribute' 'modify'
     || lk == 94330                 // 'element' 'modify'
     || lk == 94343                 // 'false' 'modify'
     || lk == 94395                 // 'namespace' 'modify'
     || lk == 94405                 // 'null' 'modify'
     || lk == 94428                 // 'processing-instruction' 'modify'
     || lk == 94463                 // 'true' 'modify'
     || lk == 96851                 // 'attribute' 'ne'
     || lk == 96890                 // 'element' 'ne'
     || lk == 96903                 // 'false' 'ne'
     || lk == 96955                 // 'namespace' 'ne'
     || lk == 96965                 // 'null' 'ne'
     || lk == 96988                 // 'processing-instruction' 'ne'
     || lk == 97023                 // 'true' 'ne'
     || lk == 103507                // 'attribute' 'only'
     || lk == 103546                // 'element' 'only'
     || lk == 103559                // 'false' 'only'
     || lk == 103611                // 'namespace' 'only'
     || lk == 103621                // 'null' 'only'
     || lk == 103644                // 'processing-instruction' 'only'
     || lk == 103679                // 'true' 'only'
     || lk == 104531                // 'attribute' 'or'
     || lk == 104570                // 'element' 'or'
     || lk == 104583                // 'false' 'or'
     || lk == 104635                // 'namespace' 'or'
     || lk == 104645                // 'null' 'or'
     || lk == 104668                // 'processing-instruction' 'or'
     || lk == 104703                // 'true' 'or'
     || lk == 105043                // 'attribute' 'order'
     || lk == 105082                // 'element' 'order'
     || lk == 105095                // 'false' 'order'
     || lk == 105147                // 'namespace' 'order'
     || lk == 105157                // 'null' 'order'
     || lk == 105180                // 'processing-instruction' 'order'
     || lk == 105215                // 'true' 'order'
     || lk == 107143                // 'false' 'paragraphs'
     || lk == 107205                // 'null' 'paragraphs'
     || lk == 107263                // 'true' 'paragraphs'
     || lk == 114771                // 'attribute' 'return'
     || lk == 114810                // 'element' 'return'
     || lk == 114823                // 'false' 'return'
     || lk == 114875                // 'namespace' 'return'
     || lk == 114885                // 'null' 'return'
     || lk == 114908                // 'processing-instruction' 'return'
     || lk == 114943                // 'true' 'return'
     || lk == 116819                // 'attribute' 'satisfies'
     || lk == 116858                // 'element' 'satisfies'
     || lk == 116871                // 'false' 'satisfies'
     || lk == 116923                // 'namespace' 'satisfies'
     || lk == 116933                // 'null' 'satisfies'
     || lk == 116956                // 'processing-instruction' 'satisfies'
     || lk == 116991                // 'true' 'satisfies'
     || lk == 121479                // 'false' 'sentences'
     || lk == 121541                // 'null' 'sentences'
     || lk == 121599                // 'true' 'sentences'
     || lk == 123475                // 'attribute' 'stable'
     || lk == 123514                // 'element' 'stable'
     || lk == 123527                // 'false' 'stable'
     || lk == 123579                // 'namespace' 'stable'
     || lk == 123589                // 'null' 'stable'
     || lk == 123612                // 'processing-instruction' 'stable'
     || lk == 123647                // 'true' 'stable'
     || lk == 123987                // 'attribute' 'start'
     || lk == 124026                // 'element' 'start'
     || lk == 124039                // 'false' 'start'
     || lk == 124091                // 'namespace' 'start'
     || lk == 124101                // 'null' 'start'
     || lk == 124124                // 'processing-instruction' 'start'
     || lk == 124159                // 'true' 'start'
     || lk == 129159                // 'false' 'times'
     || lk == 129221                // 'null' 'times'
     || lk == 129279                // 'true' 'times'
     || lk == 129619                // 'attribute' 'to'
     || lk == 129658                // 'element' 'to'
     || lk == 129671                // 'false' 'to'
     || lk == 129723                // 'namespace' 'to'
     || lk == 129733                // 'null' 'to'
     || lk == 129756                // 'processing-instruction' 'to'
     || lk == 129791                // 'true' 'to'
     || lk == 130131                // 'attribute' 'treat'
     || lk == 130170                // 'element' 'treat'
     || lk == 130183                // 'false' 'treat'
     || lk == 130235                // 'namespace' 'treat'
     || lk == 130245                // 'null' 'treat'
     || lk == 130268                // 'processing-instruction' 'treat'
     || lk == 130303                // 'true' 'treat'
     || lk == 133203                // 'attribute' 'union'
     || lk == 133242                // 'element' 'union'
     || lk == 133255                // 'false' 'union'
     || lk == 133307                // 'namespace' 'union'
     || lk == 133317                // 'null' 'union'
     || lk == 133340                // 'processing-instruction' 'union'
     || lk == 133375                // 'true' 'union'
     || lk == 139347                // 'attribute' 'where'
     || lk == 139386                // 'element' 'where'
     || lk == 139399                // 'false' 'where'
     || lk == 139451                // 'namespace' 'where'
     || lk == 139461                // 'null' 'where'
     || lk == 139484                // 'processing-instruction' 'where'
     || lk == 139519                // 'true' 'where'
     || lk == 141395                // 'attribute' 'with'
     || lk == 141434                // 'element' 'with'
     || lk == 141447                // 'false' 'with'
     || lk == 141499                // 'namespace' 'with'
     || lk == 141509                // 'null' 'with'
     || lk == 141532                // 'processing-instruction' 'with'
     || lk == 141567                // 'true' 'with'
     || lk == 142983                // 'false' 'words'
     || lk == 143045                // 'null' 'words'
     || lk == 143103                // 'true' 'words'
     || lk == 145543                // 'false' '|'
     || lk == 145605                // 'null' '|'
     || lk == 145663                // 'true' '|'
     || lk == 146055                // 'false' '||'
     || lk == 146117                // 'null' '||'
     || lk == 146175                // 'true' '||'
     || lk == 146567                // 'false' '|}'
     || lk == 146629                // 'null' '|}'
     || lk == 146687                // 'true' '|}'
     || lk == 147079                // 'false' '}'
     || lk == 147141                // 'null' '}'
     || lk == 147199)               // 'true' '}'
    {
      lk = memoized(4, e0);
      if (lk == 0)
      {
        var b0A = b0; var e0A = e0; var l1A = l1;
        var b1A = b1; var e1A = e1; var l2A = l2;
        var b2A = b2; var e2A = e2;
        try
        {
          try_PostfixExpr();
          memoize(4, e0A, -1);
          lk = -3;
        }
        catch (p1A)
        {
          lk = -2;
          b0 = b0A; e0 = e0A; l1 = l1A; if (l1 == 0) {end = e0A;} else {
          b1 = b1A; e1 = e1A; l2 = l2A; if (l2 == 0) {end = e1A;} else {
          b2 = b2A; e2 = e2A; end = e2A; }}
          memoize(4, e0A, -2);
        }
      }
    }
    switch (lk)
    {
    case -1:
    case 8:                         // IntegerLiteral
    case 9:                         // DecimalLiteral
    case 10:                        // DoubleLiteral
    case 11:                        // StringLiteral
    case 31:                        // '$'
    case 32:                        // '$$'
    case 33:                        // '%'
    case 35:                        // '('
    case 55:                        // '<'
    case 56:                        // '<!--'
    case 60:                        // '<?'
    case 69:                        // '['
    case 281:                       // '{'
    case 283:                       // '{|'
    case 3155:                      // 'attribute' EQName^Token
    case 3194:                      // 'element' EQName^Token
    case 9915:                      // 'namespace' NCName^Token
    case 9948:                      // 'processing-instruction' NCName^Token
    case 14854:                     // EQName^Token '#'
    case 14919:                     // 'after' '#'
    case 14921:                     // 'allowing' '#'
    case 14922:                     // 'ancestor' '#'
    case 14923:                     // 'ancestor-or-self' '#'
    case 14924:                     // 'and' '#'
    case 14926:                     // 'append' '#'
    case 14927:                     // 'array' '#'
    case 14928:                     // 'as' '#'
    case 14929:                     // 'ascending' '#'
    case 14930:                     // 'at' '#'
    case 14931:                     // 'attribute' '#'
    case 14932:                     // 'base-uri' '#'
    case 14933:                     // 'before' '#'
    case 14934:                     // 'boundary-space' '#'
    case 14935:                     // 'break' '#'
    case 14937:                     // 'case' '#'
    case 14938:                     // 'cast' '#'
    case 14939:                     // 'castable' '#'
    case 14940:                     // 'catch' '#'
    case 14942:                     // 'child' '#'
    case 14943:                     // 'collation' '#'
    case 14945:                     // 'comment' '#'
    case 14946:                     // 'constraint' '#'
    case 14947:                     // 'construction' '#'
    case 14950:                     // 'context' '#'
    case 14951:                     // 'continue' '#'
    case 14952:                     // 'copy' '#'
    case 14953:                     // 'copy-namespaces' '#'
    case 14954:                     // 'count' '#'
    case 14955:                     // 'decimal-format' '#'
    case 14957:                     // 'declare' '#'
    case 14958:                     // 'default' '#'
    case 14959:                     // 'delete' '#'
    case 14960:                     // 'descendant' '#'
    case 14961:                     // 'descendant-or-self' '#'
    case 14962:                     // 'descending' '#'
    case 14967:                     // 'div' '#'
    case 14968:                     // 'document' '#'
    case 14969:                     // 'document-node' '#'
    case 14970:                     // 'element' '#'
    case 14971:                     // 'else' '#'
    case 14972:                     // 'empty' '#'
    case 14973:                     // 'empty-sequence' '#'
    case 14974:                     // 'encoding' '#'
    case 14975:                     // 'end' '#'
    case 14977:                     // 'eq' '#'
    case 14978:                     // 'every' '#'
    case 14980:                     // 'except' '#'
    case 14981:                     // 'exit' '#'
    case 14982:                     // 'external' '#'
    case 14983:                     // 'false' '#'
    case 14984:                     // 'first' '#'
    case 14985:                     // 'following' '#'
    case 14986:                     // 'following-sibling' '#'
    case 14987:                     // 'for' '#'
    case 14990:                     // 'from' '#'
    case 14991:                     // 'ft-option' '#'
    case 14995:                     // 'function' '#'
    case 14996:                     // 'ge' '#'
    case 14998:                     // 'group' '#'
    case 15000:                     // 'gt' '#'
    case 15001:                     // 'idiv' '#'
    case 15002:                     // 'if' '#'
    case 15003:                     // 'import' '#'
    case 15004:                     // 'in' '#'
    case 15005:                     // 'index' '#'
    case 15009:                     // 'insert' '#'
    case 15010:                     // 'instance' '#'
    case 15011:                     // 'integrity' '#'
    case 15012:                     // 'intersect' '#'
    case 15013:                     // 'into' '#'
    case 15014:                     // 'is' '#'
    case 15015:                     // 'item' '#'
    case 15016:                     // 'json' '#'
    case 15017:                     // 'json-item' '#'
    case 15018:                     // 'jsoniq' '#'
    case 15021:                     // 'last' '#'
    case 15022:                     // 'lax' '#'
    case 15023:                     // 'le' '#'
    case 15025:                     // 'let' '#'
    case 15027:                     // 'loop' '#'
    case 15029:                     // 'lt' '#'
    case 15031:                     // 'mod' '#'
    case 15032:                     // 'modify' '#'
    case 15033:                     // 'module' '#'
    case 15035:                     // 'namespace' '#'
    case 15036:                     // 'namespace-node' '#'
    case 15037:                     // 'ne' '#'
    case 15042:                     // 'node' '#'
    case 15043:                     // 'nodes' '#'
    case 15045:                     // 'null' '#'
    case 15046:                     // 'object' '#'
    case 15050:                     // 'only' '#'
    case 15051:                     // 'option' '#'
    case 15052:                     // 'or' '#'
    case 15053:                     // 'order' '#'
    case 15054:                     // 'ordered' '#'
    case 15055:                     // 'ordering' '#'
    case 15058:                     // 'parent' '#'
    case 15064:                     // 'preceding' '#'
    case 15065:                     // 'preceding-sibling' '#'
    case 15068:                     // 'processing-instruction' '#'
    case 15070:                     // 'rename' '#'
    case 15071:                     // 'replace' '#'
    case 15072:                     // 'return' '#'
    case 15073:                     // 'returning' '#'
    case 15074:                     // 'revalidation' '#'
    case 15076:                     // 'satisfies' '#'
    case 15077:                     // 'schema' '#'
    case 15078:                     // 'schema-attribute' '#'
    case 15079:                     // 'schema-element' '#'
    case 15080:                     // 'score' '#'
    case 15081:                     // 'select' '#'
    case 15082:                     // 'self' '#'
    case 15087:                     // 'sliding' '#'
    case 15088:                     // 'some' '#'
    case 15089:                     // 'stable' '#'
    case 15090:                     // 'start' '#'
    case 15093:                     // 'strict' '#'
    case 15095:                     // 'structured-item' '#'
    case 15096:                     // 'switch' '#'
    case 15097:                     // 'text' '#'
    case 15101:                     // 'to' '#'
    case 15102:                     // 'treat' '#'
    case 15103:                     // 'true' '#'
    case 15104:                     // 'try' '#'
    case 15105:                     // 'tumbling' '#'
    case 15106:                     // 'type' '#'
    case 15107:                     // 'typeswitch' '#'
    case 15108:                     // 'union' '#'
    case 15110:                     // 'unordered' '#'
    case 15111:                     // 'updating' '#'
    case 15114:                     // 'validate' '#'
    case 15115:                     // 'value' '#'
    case 15116:                     // 'variable' '#'
    case 15117:                     // 'version' '#'
    case 15120:                     // 'where' '#'
    case 15121:                     // 'while' '#'
    case 15124:                     // 'with' '#'
    case 17926:                     // EQName^Token '('
    case 17991:                     // 'after' '('
    case 17993:                     // 'allowing' '('
    case 17994:                     // 'ancestor' '('
    case 17995:                     // 'ancestor-or-self' '('
    case 17996:                     // 'and' '('
    case 17998:                     // 'append' '('
    case 18000:                     // 'as' '('
    case 18001:                     // 'ascending' '('
    case 18002:                     // 'at' '('
    case 18004:                     // 'base-uri' '('
    case 18005:                     // 'before' '('
    case 18006:                     // 'boundary-space' '('
    case 18007:                     // 'break' '('
    case 18009:                     // 'case' '('
    case 18010:                     // 'cast' '('
    case 18011:                     // 'castable' '('
    case 18012:                     // 'catch' '('
    case 18014:                     // 'child' '('
    case 18015:                     // 'collation' '('
    case 18018:                     // 'constraint' '('
    case 18019:                     // 'construction' '('
    case 18022:                     // 'context' '('
    case 18023:                     // 'continue' '('
    case 18024:                     // 'copy' '('
    case 18025:                     // 'copy-namespaces' '('
    case 18026:                     // 'count' '('
    case 18027:                     // 'decimal-format' '('
    case 18029:                     // 'declare' '('
    case 18030:                     // 'default' '('
    case 18031:                     // 'delete' '('
    case 18032:                     // 'descendant' '('
    case 18033:                     // 'descendant-or-self' '('
    case 18034:                     // 'descending' '('
    case 18039:                     // 'div' '('
    case 18040:                     // 'document' '('
    case 18043:                     // 'else' '('
    case 18044:                     // 'empty' '('
    case 18046:                     // 'encoding' '('
    case 18047:                     // 'end' '('
    case 18049:                     // 'eq' '('
    case 18050:                     // 'every' '('
    case 18052:                     // 'except' '('
    case 18053:                     // 'exit' '('
    case 18054:                     // 'external' '('
    case 18055:                     // 'false' '('
    case 18056:                     // 'first' '('
    case 18057:                     // 'following' '('
    case 18058:                     // 'following-sibling' '('
    case 18059:                     // 'for' '('
    case 18062:                     // 'from' '('
    case 18063:                     // 'ft-option' '('
    case 18067:                     // 'function' '('
    case 18068:                     // 'ge' '('
    case 18070:                     // 'group' '('
    case 18072:                     // 'gt' '('
    case 18073:                     // 'idiv' '('
    case 18075:                     // 'import' '('
    case 18076:                     // 'in' '('
    case 18077:                     // 'index' '('
    case 18081:                     // 'insert' '('
    case 18082:                     // 'instance' '('
    case 18083:                     // 'integrity' '('
    case 18084:                     // 'intersect' '('
    case 18085:                     // 'into' '('
    case 18086:                     // 'is' '('
    case 18088:                     // 'json' '('
    case 18090:                     // 'jsoniq' '('
    case 18093:                     // 'last' '('
    case 18094:                     // 'lax' '('
    case 18095:                     // 'le' '('
    case 18097:                     // 'let' '('
    case 18099:                     // 'loop' '('
    case 18101:                     // 'lt' '('
    case 18103:                     // 'mod' '('
    case 18104:                     // 'modify' '('
    case 18105:                     // 'module' '('
    case 18107:                     // 'namespace' '('
    case 18109:                     // 'ne' '('
    case 18115:                     // 'nodes' '('
    case 18117:                     // 'null' '('
    case 18118:                     // 'object' '('
    case 18122:                     // 'only' '('
    case 18123:                     // 'option' '('
    case 18124:                     // 'or' '('
    case 18125:                     // 'order' '('
    case 18126:                     // 'ordered' '('
    case 18127:                     // 'ordering' '('
    case 18130:                     // 'parent' '('
    case 18136:                     // 'preceding' '('
    case 18137:                     // 'preceding-sibling' '('
    case 18142:                     // 'rename' '('
    case 18143:                     // 'replace' '('
    case 18144:                     // 'return' '('
    case 18145:                     // 'returning' '('
    case 18146:                     // 'revalidation' '('
    case 18148:                     // 'satisfies' '('
    case 18149:                     // 'schema' '('
    case 18152:                     // 'score' '('
    case 18153:                     // 'select' '('
    case 18154:                     // 'self' '('
    case 18159:                     // 'sliding' '('
    case 18160:                     // 'some' '('
    case 18161:                     // 'stable' '('
    case 18162:                     // 'start' '('
    case 18165:                     // 'strict' '('
    case 18173:                     // 'to' '('
    case 18174:                     // 'treat' '('
    case 18175:                     // 'true' '('
    case 18176:                     // 'try' '('
    case 18177:                     // 'tumbling' '('
    case 18178:                     // 'type' '('
    case 18180:                     // 'union' '('
    case 18182:                     // 'unordered' '('
    case 18183:                     // 'updating' '('
    case 18186:                     // 'validate' '('
    case 18187:                     // 'value' '('
    case 18188:                     // 'variable' '('
    case 18189:                     // 'version' '('
    case 18192:                     // 'where' '('
    case 18193:                     // 'while' '('
    case 18196:                     // 'with' '('
    case 23175:                     // 'false' '.'
    case 23237:                     // 'null' '.'
    case 23295:                     // 'true' '.'
    case 37459:                     // 'attribute' 'allowing'
    case 37498:                     // 'element' 'allowing'
    case 37563:                     // 'namespace' 'allowing'
    case 37596:                     // 'processing-instruction' 'allowing'
    case 37971:                     // 'attribute' 'ancestor'
    case 38010:                     // 'element' 'ancestor'
    case 38075:                     // 'namespace' 'ancestor'
    case 38108:                     // 'processing-instruction' 'ancestor'
    case 38483:                     // 'attribute' 'ancestor-or-self'
    case 38522:                     // 'element' 'ancestor-or-self'
    case 38587:                     // 'namespace' 'ancestor-or-self'
    case 38620:                     // 'processing-instruction' 'ancestor-or-self'
    case 40019:                     // 'attribute' 'append'
    case 40058:                     // 'element' 'append'
    case 40123:                     // 'namespace' 'append'
    case 40156:                     // 'processing-instruction' 'append'
    case 40531:                     // 'attribute' 'array'
    case 40570:                     // 'element' 'array'
    case 42579:                     // 'attribute' 'attribute'
    case 42618:                     // 'element' 'attribute'
    case 42683:                     // 'namespace' 'attribute'
    case 42716:                     // 'processing-instruction' 'attribute'
    case 43091:                     // 'attribute' 'base-uri'
    case 43130:                     // 'element' 'base-uri'
    case 43195:                     // 'namespace' 'base-uri'
    case 43228:                     // 'processing-instruction' 'base-uri'
    case 44115:                     // 'attribute' 'boundary-space'
    case 44154:                     // 'element' 'boundary-space'
    case 44219:                     // 'namespace' 'boundary-space'
    case 44252:                     // 'processing-instruction' 'boundary-space'
    case 44627:                     // 'attribute' 'break'
    case 44666:                     // 'element' 'break'
    case 44731:                     // 'namespace' 'break'
    case 44764:                     // 'processing-instruction' 'break'
    case 47187:                     // 'attribute' 'catch'
    case 47226:                     // 'element' 'catch'
    case 47291:                     // 'namespace' 'catch'
    case 47324:                     // 'processing-instruction' 'catch'
    case 48211:                     // 'attribute' 'child'
    case 48250:                     // 'element' 'child'
    case 48315:                     // 'namespace' 'child'
    case 48348:                     // 'processing-instruction' 'child'
    case 49747:                     // 'attribute' 'comment'
    case 49786:                     // 'element' 'comment'
    case 49851:                     // 'namespace' 'comment'
    case 49884:                     // 'processing-instruction' 'comment'
    case 50259:                     // 'attribute' 'constraint'
    case 50298:                     // 'element' 'constraint'
    case 50363:                     // 'namespace' 'constraint'
    case 50396:                     // 'processing-instruction' 'constraint'
    case 50771:                     // 'attribute' 'construction'
    case 50810:                     // 'element' 'construction'
    case 50875:                     // 'namespace' 'construction'
    case 50908:                     // 'processing-instruction' 'construction'
    case 52307:                     // 'attribute' 'context'
    case 52346:                     // 'element' 'context'
    case 52411:                     // 'namespace' 'context'
    case 52444:                     // 'processing-instruction' 'context'
    case 52819:                     // 'attribute' 'continue'
    case 52858:                     // 'element' 'continue'
    case 52923:                     // 'namespace' 'continue'
    case 52956:                     // 'processing-instruction' 'continue'
    case 53331:                     // 'attribute' 'copy'
    case 53370:                     // 'element' 'copy'
    case 53435:                     // 'namespace' 'copy'
    case 53468:                     // 'processing-instruction' 'copy'
    case 53843:                     // 'attribute' 'copy-namespaces'
    case 53882:                     // 'element' 'copy-namespaces'
    case 53947:                     // 'namespace' 'copy-namespaces'
    case 53980:                     // 'processing-instruction' 'copy-namespaces'
    case 54867:                     // 'attribute' 'decimal-format'
    case 54906:                     // 'element' 'decimal-format'
    case 54971:                     // 'namespace' 'decimal-format'
    case 55004:                     // 'processing-instruction' 'decimal-format'
    case 55891:                     // 'attribute' 'declare'
    case 55930:                     // 'element' 'declare'
    case 55995:                     // 'namespace' 'declare'
    case 56028:                     // 'processing-instruction' 'declare'
    case 56915:                     // 'attribute' 'delete'
    case 56954:                     // 'element' 'delete'
    case 57019:                     // 'namespace' 'delete'
    case 57052:                     // 'processing-instruction' 'delete'
    case 57427:                     // 'attribute' 'descendant'
    case 57466:                     // 'element' 'descendant'
    case 57531:                     // 'namespace' 'descendant'
    case 57564:                     // 'processing-instruction' 'descendant'
    case 57939:                     // 'attribute' 'descendant-or-self'
    case 57978:                     // 'element' 'descendant-or-self'
    case 58043:                     // 'namespace' 'descendant-or-self'
    case 58076:                     // 'processing-instruction' 'descendant-or-self'
    case 61523:                     // 'attribute' 'document'
    case 61562:                     // 'element' 'document'
    case 61627:                     // 'namespace' 'document'
    case 61660:                     // 'processing-instruction' 'document'
    case 62035:                     // 'attribute' 'document-node'
    case 62074:                     // 'element' 'document-node'
    case 62139:                     // 'namespace' 'document-node'
    case 62172:                     // 'processing-instruction' 'document-node'
    case 62547:                     // 'attribute' 'element'
    case 62586:                     // 'element' 'element'
    case 62651:                     // 'namespace' 'element'
    case 62684:                     // 'processing-instruction' 'element'
    case 64083:                     // 'attribute' 'empty-sequence'
    case 64122:                     // 'element' 'empty-sequence'
    case 64187:                     // 'namespace' 'empty-sequence'
    case 64220:                     // 'processing-instruction' 'empty-sequence'
    case 64595:                     // 'attribute' 'encoding'
    case 64634:                     // 'element' 'encoding'
    case 64699:                     // 'namespace' 'encoding'
    case 64732:                     // 'processing-instruction' 'encoding'
    case 66643:                     // 'attribute' 'every'
    case 66682:                     // 'element' 'every'
    case 66747:                     // 'namespace' 'every'
    case 66780:                     // 'processing-instruction' 'every'
    case 68179:                     // 'attribute' 'exit'
    case 68218:                     // 'element' 'exit'
    case 68283:                     // 'namespace' 'exit'
    case 68316:                     // 'processing-instruction' 'exit'
    case 68691:                     // 'attribute' 'external'
    case 68730:                     // 'element' 'external'
    case 68795:                     // 'namespace' 'external'
    case 68828:                     // 'processing-instruction' 'external'
    case 69203:                     // 'attribute' 'false'
    case 69242:                     // 'element' 'false'
    case 69307:                     // 'namespace' 'false'
    case 69340:                     // 'processing-instruction' 'false'
    case 69715:                     // 'attribute' 'first'
    case 69754:                     // 'element' 'first'
    case 69819:                     // 'namespace' 'first'
    case 69852:                     // 'processing-instruction' 'first'
    case 70227:                     // 'attribute' 'following'
    case 70266:                     // 'element' 'following'
    case 70331:                     // 'namespace' 'following'
    case 70364:                     // 'processing-instruction' 'following'
    case 70739:                     // 'attribute' 'following-sibling'
    case 70778:                     // 'element' 'following-sibling'
    case 70843:                     // 'namespace' 'following-sibling'
    case 70876:                     // 'processing-instruction' 'following-sibling'
    case 72787:                     // 'attribute' 'from'
    case 72826:                     // 'element' 'from'
    case 72891:                     // 'namespace' 'from'
    case 72924:                     // 'processing-instruction' 'from'
    case 73299:                     // 'attribute' 'ft-option'
    case 73338:                     // 'element' 'ft-option'
    case 73403:                     // 'namespace' 'ft-option'
    case 73436:                     // 'processing-instruction' 'ft-option'
    case 75347:                     // 'attribute' 'function'
    case 75386:                     // 'element' 'function'
    case 75451:                     // 'namespace' 'function'
    case 75484:                     // 'processing-instruction' 'function'
    case 78931:                     // 'attribute' 'if'
    case 78970:                     // 'element' 'if'
    case 79035:                     // 'namespace' 'if'
    case 79068:                     // 'processing-instruction' 'if'
    case 79443:                     // 'attribute' 'import'
    case 79482:                     // 'element' 'import'
    case 79547:                     // 'namespace' 'import'
    case 79580:                     // 'processing-instruction' 'import'
    case 79955:                     // 'attribute' 'in'
    case 79994:                     // 'element' 'in'
    case 80059:                     // 'namespace' 'in'
    case 80092:                     // 'processing-instruction' 'in'
    case 80467:                     // 'attribute' 'index'
    case 80506:                     // 'element' 'index'
    case 80571:                     // 'namespace' 'index'
    case 80604:                     // 'processing-instruction' 'index'
    case 82515:                     // 'attribute' 'insert'
    case 82554:                     // 'element' 'insert'
    case 82619:                     // 'namespace' 'insert'
    case 82652:                     // 'processing-instruction' 'insert'
    case 83539:                     // 'attribute' 'integrity'
    case 83578:                     // 'element' 'integrity'
    case 83643:                     // 'namespace' 'integrity'
    case 83676:                     // 'processing-instruction' 'integrity'
    case 85587:                     // 'attribute' 'item'
    case 85626:                     // 'element' 'item'
    case 85691:                     // 'namespace' 'item'
    case 85724:                     // 'processing-instruction' 'item'
    case 86099:                     // 'attribute' 'json'
    case 86138:                     // 'element' 'json'
    case 86203:                     // 'namespace' 'json'
    case 86236:                     // 'processing-instruction' 'json'
    case 86611:                     // 'attribute' 'json-item'
    case 86650:                     // 'element' 'json-item'
    case 87123:                     // 'attribute' 'jsoniq'
    case 87162:                     // 'element' 'jsoniq'
    case 87227:                     // 'namespace' 'jsoniq'
    case 87260:                     // 'processing-instruction' 'jsoniq'
    case 88659:                     // 'attribute' 'last'
    case 88698:                     // 'element' 'last'
    case 88763:                     // 'namespace' 'last'
    case 88796:                     // 'processing-instruction' 'last'
    case 89171:                     // 'attribute' 'lax'
    case 89210:                     // 'element' 'lax'
    case 89275:                     // 'namespace' 'lax'
    case 89308:                     // 'processing-instruction' 'lax'
    case 91731:                     // 'attribute' 'loop'
    case 91770:                     // 'element' 'loop'
    case 91835:                     // 'namespace' 'loop'
    case 91868:                     // 'processing-instruction' 'loop'
    case 94803:                     // 'attribute' 'module'
    case 94842:                     // 'element' 'module'
    case 94907:                     // 'namespace' 'module'
    case 94940:                     // 'processing-instruction' 'module'
    case 95827:                     // 'attribute' 'namespace'
    case 95866:                     // 'element' 'namespace'
    case 95931:                     // 'namespace' 'namespace'
    case 95964:                     // 'processing-instruction' 'namespace'
    case 96339:                     // 'attribute' 'namespace-node'
    case 96378:                     // 'element' 'namespace-node'
    case 96443:                     // 'namespace' 'namespace-node'
    case 96476:                     // 'processing-instruction' 'namespace-node'
    case 99411:                     // 'attribute' 'node'
    case 99450:                     // 'element' 'node'
    case 99515:                     // 'namespace' 'node'
    case 99548:                     // 'processing-instruction' 'node'
    case 99923:                     // 'attribute' 'nodes'
    case 99962:                     // 'element' 'nodes'
    case 100027:                    // 'namespace' 'nodes'
    case 100060:                    // 'processing-instruction' 'nodes'
    case 100947:                    // 'attribute' 'null'
    case 100986:                    // 'element' 'null'
    case 101051:                    // 'namespace' 'null'
    case 101084:                    // 'processing-instruction' 'null'
    case 101459:                    // 'attribute' 'object'
    case 101498:                    // 'element' 'object'
    case 101563:                    // 'namespace' 'object'
    case 101596:                    // 'processing-instruction' 'object'
    case 104019:                    // 'attribute' 'option'
    case 104058:                    // 'element' 'option'
    case 104123:                    // 'namespace' 'option'
    case 104156:                    // 'processing-instruction' 'option'
    case 105555:                    // 'attribute' 'ordered'
    case 105594:                    // 'element' 'ordered'
    case 105659:                    // 'namespace' 'ordered'
    case 105692:                    // 'processing-instruction' 'ordered'
    case 106067:                    // 'attribute' 'ordering'
    case 106106:                    // 'element' 'ordering'
    case 106171:                    // 'namespace' 'ordering'
    case 106204:                    // 'processing-instruction' 'ordering'
    case 107603:                    // 'attribute' 'parent'
    case 107642:                    // 'element' 'parent'
    case 107707:                    // 'namespace' 'parent'
    case 107740:                    // 'processing-instruction' 'parent'
    case 110675:                    // 'attribute' 'preceding'
    case 110714:                    // 'element' 'preceding'
    case 110779:                    // 'namespace' 'preceding'
    case 110812:                    // 'processing-instruction' 'preceding'
    case 111187:                    // 'attribute' 'preceding-sibling'
    case 111226:                    // 'element' 'preceding-sibling'
    case 111291:                    // 'namespace' 'preceding-sibling'
    case 111324:                    // 'processing-instruction' 'preceding-sibling'
    case 112723:                    // 'attribute' 'processing-instruction'
    case 112762:                    // 'element' 'processing-instruction'
    case 112827:                    // 'namespace' 'processing-instruction'
    case 112860:                    // 'processing-instruction' 'processing-instruction'
    case 113747:                    // 'attribute' 'rename'
    case 113786:                    // 'element' 'rename'
    case 113851:                    // 'namespace' 'rename'
    case 113884:                    // 'processing-instruction' 'rename'
    case 114259:                    // 'attribute' 'replace'
    case 114298:                    // 'element' 'replace'
    case 114363:                    // 'namespace' 'replace'
    case 114396:                    // 'processing-instruction' 'replace'
    case 115283:                    // 'attribute' 'returning'
    case 115322:                    // 'element' 'returning'
    case 115387:                    // 'namespace' 'returning'
    case 115420:                    // 'processing-instruction' 'returning'
    case 115795:                    // 'attribute' 'revalidation'
    case 115834:                    // 'element' 'revalidation'
    case 115899:                    // 'namespace' 'revalidation'
    case 115932:                    // 'processing-instruction' 'revalidation'
    case 117331:                    // 'attribute' 'schema'
    case 117370:                    // 'element' 'schema'
    case 117435:                    // 'namespace' 'schema'
    case 117468:                    // 'processing-instruction' 'schema'
    case 117843:                    // 'attribute' 'schema-attribute'
    case 117882:                    // 'element' 'schema-attribute'
    case 117947:                    // 'namespace' 'schema-attribute'
    case 117980:                    // 'processing-instruction' 'schema-attribute'
    case 118355:                    // 'attribute' 'schema-element'
    case 118394:                    // 'element' 'schema-element'
    case 118459:                    // 'namespace' 'schema-element'
    case 118492:                    // 'processing-instruction' 'schema-element'
    case 118867:                    // 'attribute' 'score'
    case 118906:                    // 'element' 'score'
    case 118971:                    // 'namespace' 'score'
    case 119004:                    // 'processing-instruction' 'score'
    case 119379:                    // 'attribute' 'select'
    case 119418:                    // 'element' 'select'
    case 119483:                    // 'namespace' 'select'
    case 119516:                    // 'processing-instruction' 'select'
    case 119891:                    // 'attribute' 'self'
    case 119930:                    // 'element' 'self'
    case 119995:                    // 'namespace' 'self'
    case 120028:                    // 'processing-instruction' 'self'
    case 122451:                    // 'attribute' 'sliding'
    case 122490:                    // 'element' 'sliding'
    case 122555:                    // 'namespace' 'sliding'
    case 122588:                    // 'processing-instruction' 'sliding'
    case 122963:                    // 'attribute' 'some'
    case 123002:                    // 'element' 'some'
    case 123067:                    // 'namespace' 'some'
    case 123100:                    // 'processing-instruction' 'some'
    case 125523:                    // 'attribute' 'strict'
    case 125562:                    // 'element' 'strict'
    case 125627:                    // 'namespace' 'strict'
    case 125660:                    // 'processing-instruction' 'strict'
    case 126547:                    // 'attribute' 'structured-item'
    case 126586:                    // 'element' 'structured-item'
    case 127059:                    // 'attribute' 'switch'
    case 127098:                    // 'element' 'switch'
    case 127163:                    // 'namespace' 'switch'
    case 127196:                    // 'processing-instruction' 'switch'
    case 127571:                    // 'attribute' 'text'
    case 127610:                    // 'element' 'text'
    case 127675:                    // 'namespace' 'text'
    case 127708:                    // 'processing-instruction' 'text'
    case 130643:                    // 'attribute' 'true'
    case 130682:                    // 'element' 'true'
    case 130747:                    // 'namespace' 'true'
    case 130780:                    // 'processing-instruction' 'true'
    case 131155:                    // 'attribute' 'try'
    case 131194:                    // 'element' 'try'
    case 131259:                    // 'namespace' 'try'
    case 131292:                    // 'processing-instruction' 'try'
    case 131667:                    // 'attribute' 'tumbling'
    case 131706:                    // 'element' 'tumbling'
    case 131771:                    // 'namespace' 'tumbling'
    case 131804:                    // 'processing-instruction' 'tumbling'
    case 132179:                    // 'attribute' 'type'
    case 132218:                    // 'element' 'type'
    case 132283:                    // 'namespace' 'type'
    case 132316:                    // 'processing-instruction' 'type'
    case 132691:                    // 'attribute' 'typeswitch'
    case 132730:                    // 'element' 'typeswitch'
    case 132795:                    // 'namespace' 'typeswitch'
    case 132828:                    // 'processing-instruction' 'typeswitch'
    case 134227:                    // 'attribute' 'unordered'
    case 134266:                    // 'element' 'unordered'
    case 134331:                    // 'namespace' 'unordered'
    case 134364:                    // 'processing-instruction' 'unordered'
    case 134739:                    // 'attribute' 'updating'
    case 134778:                    // 'element' 'updating'
    case 134843:                    // 'namespace' 'updating'
    case 134876:                    // 'processing-instruction' 'updating'
    case 136275:                    // 'attribute' 'validate'
    case 136314:                    // 'element' 'validate'
    case 136379:                    // 'namespace' 'validate'
    case 136412:                    // 'processing-instruction' 'validate'
    case 136787:                    // 'attribute' 'value'
    case 136826:                    // 'element' 'value'
    case 136891:                    // 'namespace' 'value'
    case 136924:                    // 'processing-instruction' 'value'
    case 137299:                    // 'attribute' 'variable'
    case 137338:                    // 'element' 'variable'
    case 137403:                    // 'namespace' 'variable'
    case 137436:                    // 'processing-instruction' 'variable'
    case 137811:                    // 'attribute' 'version'
    case 137850:                    // 'element' 'version'
    case 137915:                    // 'namespace' 'version'
    case 137948:                    // 'processing-instruction' 'version'
    case 139859:                    // 'attribute' 'while'
    case 139898:                    // 'element' 'while'
    case 139963:                    // 'namespace' 'while'
    case 139996:                    // 'processing-instruction' 'while'
    case 143955:                    // 'attribute' '{'
    case 143969:                    // 'comment' '{'
    case 143992:                    // 'document' '{'
    case 143994:                    // 'element' '{'
    case 144059:                    // 'namespace' '{'
    case 144078:                    // 'ordered' '{'
    case 144092:                    // 'processing-instruction' '{'
    case 144121:                    // 'text' '{'
    case 144134:                    // 'unordered' '{'
      try_PostfixExpr();
      break;
    case -3:
      break;
    default:
      try_AxisStep();
    }
  }

  function parse_AxisStep()
  {
    eventHandler.startNonterminal("AxisStep", e0);
    switch (l1)
    {
    case 74:                        // 'ancestor'
    case 75:                        // 'ancestor-or-self'
    case 210:                       // 'parent'
    case 216:                       // 'preceding'
    case 217:                       // 'preceding-sibling'
      lookahead2W(232);             // S^WS | EOF | '!' | '!=' | '(:' | ')' | '*' | '+' | ',' | '-' | '/' | '//' | ':' |
      break;
    default:
      lk = l1;
    }
    switch (lk)
    {
    case 46:                        // '..'
    case 26698:                     // 'ancestor' '::'
    case 26699:                     // 'ancestor-or-self' '::'
    case 26834:                     // 'parent' '::'
    case 26840:                     // 'preceding' '::'
    case 26841:                     // 'preceding-sibling' '::'
      parse_ReverseStep();
      break;
    default:
      parse_ForwardStep();
    }
    lookahead1W(228);               // S^WS | EOF | '!' | '!=' | '(:' | ')' | '*' | '+' | ',' | '-' | '/' | '//' | ':' |
    whitespace();
    parse_PredicateList();
    eventHandler.endNonterminal("AxisStep", e0);
  }

  function try_AxisStep()
  {
    switch (l1)
    {
    case 74:                        // 'ancestor'
    case 75:                        // 'ancestor-or-self'
    case 210:                       // 'parent'
    case 216:                       // 'preceding'
    case 217:                       // 'preceding-sibling'
      lookahead2W(232);             // S^WS | EOF | '!' | '!=' | '(:' | ')' | '*' | '+' | ',' | '-' | '/' | '//' | ':' |
      break;
    default:
      lk = l1;
    }
    switch (lk)
    {
    case 46:                        // '..'
    case 26698:                     // 'ancestor' '::'
    case 26699:                     // 'ancestor-or-self' '::'
    case 26834:                     // 'parent' '::'
    case 26840:                     // 'preceding' '::'
    case 26841:                     // 'preceding-sibling' '::'
      try_ReverseStep();
      break;
    default:
      try_ForwardStep();
    }
    lookahead1W(228);               // S^WS | EOF | '!' | '!=' | '(:' | ')' | '*' | '+' | ',' | '-' | '/' | '//' | ':' |
    try_PredicateList();
  }

  function parse_ForwardStep()
  {
    eventHandler.startNonterminal("ForwardStep", e0);
    switch (l1)
    {
    case 83:                        // 'attribute'
      lookahead2W(236);             // S^WS | EOF | '!' | '!=' | '(' | '(:' | ')' | '*' | '+' | ',' | '-' | '/' | '//' |
      break;
    case 94:                        // 'child'
    case 112:                       // 'descendant'
    case 113:                       // 'descendant-or-self'
    case 137:                       // 'following'
    case 138:                       // 'following-sibling'
    case 234:                       // 'self'
      lookahead2W(232);             // S^WS | EOF | '!' | '!=' | '(:' | ')' | '*' | '+' | ',' | '-' | '/' | '//' | ':' |
      break;
    default:
      lk = l1;
    }
    switch (lk)
    {
    case 26707:                     // 'attribute' '::'
    case 26718:                     // 'child' '::'
    case 26736:                     // 'descendant' '::'
    case 26737:                     // 'descendant-or-self' '::'
    case 26761:                     // 'following' '::'
    case 26762:                     // 'following-sibling' '::'
    case 26858:                     // 'self' '::'
      parse_ForwardAxis();
      lookahead1W(249);             // Wildcard | EQName^Token | S^WS | '(:' | 'after' | 'allowing' | 'ancestor' |
      whitespace();
      parse_NodeTest();
      break;
    default:
      parse_AbbrevForwardStep();
    }
    eventHandler.endNonterminal("ForwardStep", e0);
  }

  function try_ForwardStep()
  {
    switch (l1)
    {
    case 83:                        // 'attribute'
      lookahead2W(236);             // S^WS | EOF | '!' | '!=' | '(' | '(:' | ')' | '*' | '+' | ',' | '-' | '/' | '//' |
      break;
    case 94:                        // 'child'
    case 112:                       // 'descendant'
    case 113:                       // 'descendant-or-self'
    case 137:                       // 'following'
    case 138:                       // 'following-sibling'
    case 234:                       // 'self'
      lookahead2W(232);             // S^WS | EOF | '!' | '!=' | '(:' | ')' | '*' | '+' | ',' | '-' | '/' | '//' | ':' |
      break;
    default:
      lk = l1;
    }
    switch (lk)
    {
    case 26707:                     // 'attribute' '::'
    case 26718:                     // 'child' '::'
    case 26736:                     // 'descendant' '::'
    case 26737:                     // 'descendant-or-self' '::'
    case 26761:                     // 'following' '::'
    case 26762:                     // 'following-sibling' '::'
    case 26858:                     // 'self' '::'
      try_ForwardAxis();
      lookahead1W(249);             // Wildcard | EQName^Token | S^WS | '(:' | 'after' | 'allowing' | 'ancestor' |
      try_NodeTest();
      break;
    default:
      try_AbbrevForwardStep();
    }
  }

  function parse_ForwardAxis()
  {
    eventHandler.startNonterminal("ForwardAxis", e0);
    switch (l1)
    {
    case 94:                        // 'child'
      shift(94);                    // 'child'
      lookahead1W(27);              // S^WS | '(:' | '::'
      shift(52);                    // '::'
      break;
    case 112:                       // 'descendant'
      shift(112);                   // 'descendant'
      lookahead1W(27);              // S^WS | '(:' | '::'
      shift(52);                    // '::'
      break;
    case 83:                        // 'attribute'
      shift(83);                    // 'attribute'
      lookahead1W(27);              // S^WS | '(:' | '::'
      shift(52);                    // '::'
      break;
    case 234:                       // 'self'
      shift(234);                   // 'self'
      lookahead1W(27);              // S^WS | '(:' | '::'
      shift(52);                    // '::'
      break;
    case 113:                       // 'descendant-or-self'
      shift(113);                   // 'descendant-or-self'
      lookahead1W(27);              // S^WS | '(:' | '::'
      shift(52);                    // '::'
      break;
    case 138:                       // 'following-sibling'
      shift(138);                   // 'following-sibling'
      lookahead1W(27);              // S^WS | '(:' | '::'
      shift(52);                    // '::'
      break;
    default:
      shift(137);                   // 'following'
      lookahead1W(27);              // S^WS | '(:' | '::'
      shift(52);                    // '::'
    }
    eventHandler.endNonterminal("ForwardAxis", e0);
  }

  function try_ForwardAxis()
  {
    switch (l1)
    {
    case 94:                        // 'child'
      shiftT(94);                   // 'child'
      lookahead1W(27);              // S^WS | '(:' | '::'
      shiftT(52);                   // '::'
      break;
    case 112:                       // 'descendant'
      shiftT(112);                  // 'descendant'
      lookahead1W(27);              // S^WS | '(:' | '::'
      shiftT(52);                   // '::'
      break;
    case 83:                        // 'attribute'
      shiftT(83);                   // 'attribute'
      lookahead1W(27);              // S^WS | '(:' | '::'
      shiftT(52);                   // '::'
      break;
    case 234:                       // 'self'
      shiftT(234);                  // 'self'
      lookahead1W(27);              // S^WS | '(:' | '::'
      shiftT(52);                   // '::'
      break;
    case 113:                       // 'descendant-or-self'
      shiftT(113);                  // 'descendant-or-self'
      lookahead1W(27);              // S^WS | '(:' | '::'
      shiftT(52);                   // '::'
      break;
    case 138:                       // 'following-sibling'
      shiftT(138);                  // 'following-sibling'
      lookahead1W(27);              // S^WS | '(:' | '::'
      shiftT(52);                   // '::'
      break;
    default:
      shiftT(137);                  // 'following'
      lookahead1W(27);              // S^WS | '(:' | '::'
      shiftT(52);                   // '::'
    }
  }

  function parse_AbbrevForwardStep()
  {
    eventHandler.startNonterminal("AbbrevForwardStep", e0);
    if (l1 == 67)                   // '@'
    {
      shift(67);                    // '@'
    }
    lookahead1W(249);               // Wildcard | EQName^Token | S^WS | '(:' | 'after' | 'allowing' | 'ancestor' |
    whitespace();
    parse_NodeTest();
    eventHandler.endNonterminal("AbbrevForwardStep", e0);
  }

  function try_AbbrevForwardStep()
  {
    if (l1 == 67)                   // '@'
    {
      shiftT(67);                   // '@'
    }
    lookahead1W(249);               // Wildcard | EQName^Token | S^WS | '(:' | 'after' | 'allowing' | 'ancestor' |
    try_NodeTest();
  }

  function parse_ReverseStep()
  {
    eventHandler.startNonterminal("ReverseStep", e0);
    switch (l1)
    {
    case 46:                        // '..'
      parse_AbbrevReverseStep();
      break;
    default:
      parse_ReverseAxis();
      lookahead1W(249);             // Wildcard | EQName^Token | S^WS | '(:' | 'after' | 'allowing' | 'ancestor' |
      whitespace();
      parse_NodeTest();
    }
    eventHandler.endNonterminal("ReverseStep", e0);
  }

  function try_ReverseStep()
  {
    switch (l1)
    {
    case 46:                        // '..'
      try_AbbrevReverseStep();
      break;
    default:
      try_ReverseAxis();
      lookahead1W(249);             // Wildcard | EQName^Token | S^WS | '(:' | 'after' | 'allowing' | 'ancestor' |
      try_NodeTest();
    }
  }

  function parse_ReverseAxis()
  {
    eventHandler.startNonterminal("ReverseAxis", e0);
    switch (l1)
    {
    case 210:                       // 'parent'
      shift(210);                   // 'parent'
      lookahead1W(27);              // S^WS | '(:' | '::'
      shift(52);                    // '::'
      break;
    case 74:                        // 'ancestor'
      shift(74);                    // 'ancestor'
      lookahead1W(27);              // S^WS | '(:' | '::'
      shift(52);                    // '::'
      break;
    case 217:                       // 'preceding-sibling'
      shift(217);                   // 'preceding-sibling'
      lookahead1W(27);              // S^WS | '(:' | '::'
      shift(52);                    // '::'
      break;
    case 216:                       // 'preceding'
      shift(216);                   // 'preceding'
      lookahead1W(27);              // S^WS | '(:' | '::'
      shift(52);                    // '::'
      break;
    default:
      shift(75);                    // 'ancestor-or-self'
      lookahead1W(27);              // S^WS | '(:' | '::'
      shift(52);                    // '::'
    }
    eventHandler.endNonterminal("ReverseAxis", e0);
  }

  function try_ReverseAxis()
  {
    switch (l1)
    {
    case 210:                       // 'parent'
      shiftT(210);                  // 'parent'
      lookahead1W(27);              // S^WS | '(:' | '::'
      shiftT(52);                   // '::'
      break;
    case 74:                        // 'ancestor'
      shiftT(74);                   // 'ancestor'
      lookahead1W(27);              // S^WS | '(:' | '::'
      shiftT(52);                   // '::'
      break;
    case 217:                       // 'preceding-sibling'
      shiftT(217);                  // 'preceding-sibling'
      lookahead1W(27);              // S^WS | '(:' | '::'
      shiftT(52);                   // '::'
      break;
    case 216:                       // 'preceding'
      shiftT(216);                  // 'preceding'
      lookahead1W(27);              // S^WS | '(:' | '::'
      shiftT(52);                   // '::'
      break;
    default:
      shiftT(75);                   // 'ancestor-or-self'
      lookahead1W(27);              // S^WS | '(:' | '::'
      shiftT(52);                   // '::'
    }
  }

  function parse_AbbrevReverseStep()
  {
    eventHandler.startNonterminal("AbbrevReverseStep", e0);
    shift(46);                      // '..'
    eventHandler.endNonterminal("AbbrevReverseStep", e0);
  }

  function try_AbbrevReverseStep()
  {
    shiftT(46);                     // '..'
  }

  function parse_NodeTest()
  {
    eventHandler.startNonterminal("NodeTest", e0);
    switch (l1)
    {
    case 83:                        // 'attribute'
    case 97:                        // 'comment'
    case 121:                       // 'document-node'
    case 122:                       // 'element'
    case 188:                       // 'namespace-node'
    case 194:                       // 'node'
    case 220:                       // 'processing-instruction'
    case 230:                       // 'schema-attribute'
    case 231:                       // 'schema-element'
    case 249:                       // 'text'
      lookahead2W(231);             // S^WS | EOF | '!' | '!=' | '(' | '(:' | ')' | '*' | '+' | ',' | '-' | '/' | '//' |
      break;
    default:
      lk = l1;
    }
    switch (lk)
    {
    case 18003:                     // 'attribute' '('
    case 18017:                     // 'comment' '('
    case 18041:                     // 'document-node' '('
    case 18042:                     // 'element' '('
    case 18108:                     // 'namespace-node' '('
    case 18114:                     // 'node' '('
    case 18140:                     // 'processing-instruction' '('
    case 18150:                     // 'schema-attribute' '('
    case 18151:                     // 'schema-element' '('
    case 18169:                     // 'text' '('
      parse_KindTest();
      break;
    default:
      parse_NameTest();
    }
    eventHandler.endNonterminal("NodeTest", e0);
  }

  function try_NodeTest()
  {
    switch (l1)
    {
    case 83:                        // 'attribute'
    case 97:                        // 'comment'
    case 121:                       // 'document-node'
    case 122:                       // 'element'
    case 188:                       // 'namespace-node'
    case 194:                       // 'node'
    case 220:                       // 'processing-instruction'
    case 230:                       // 'schema-attribute'
    case 231:                       // 'schema-element'
    case 249:                       // 'text'
      lookahead2W(231);             // S^WS | EOF | '!' | '!=' | '(' | '(:' | ')' | '*' | '+' | ',' | '-' | '/' | '//' |
      break;
    default:
      lk = l1;
    }
    switch (lk)
    {
    case 18003:                     // 'attribute' '('
    case 18017:                     // 'comment' '('
    case 18041:                     // 'document-node' '('
    case 18042:                     // 'element' '('
    case 18108:                     // 'namespace-node' '('
    case 18114:                     // 'node' '('
    case 18140:                     // 'processing-instruction' '('
    case 18150:                     // 'schema-attribute' '('
    case 18151:                     // 'schema-element' '('
    case 18169:                     // 'text' '('
      try_KindTest();
      break;
    default:
      try_NameTest();
    }
  }

  function parse_NameTest()
  {
    eventHandler.startNonterminal("NameTest", e0);
    switch (l1)
    {
    case 5:                         // Wildcard
      shift(5);                     // Wildcard
      break;
    default:
      parse_EQName();
    }
    eventHandler.endNonterminal("NameTest", e0);
  }

  function try_NameTest()
  {
    switch (l1)
    {
    case 5:                         // Wildcard
      shiftT(5);                    // Wildcard
      break;
    default:
      try_EQName();
    }
  }

  function parse_PostfixExpr()
  {
    eventHandler.startNonterminal("PostfixExpr", e0);
    parse_PrimaryExpr();
    for (;;)
    {
      lookahead1W(235);             // S^WS | EOF | '!' | '!=' | '(' | '(:' | ')' | '*' | '+' | ',' | '-' | '.' | '/' |
      if (l1 != 35                  // '('
       && l1 != 45                  // '.'
       && l1 != 69)                 // '['
      {
        break;
      }
      switch (l1)
      {
      case 69:                      // '['
        lookahead2W(273);           // EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral | StringLiteral |
        break;
      default:
        lk = l1;
      }
      if (lk == 35397)              // '[' '['
      {
        lk = memoized(5, e0);
        if (lk == 0)
        {
          var b0A = b0; var e0A = e0; var l1A = l1;
          var b1A = b1; var e1A = e1; var l2A = l2;
          var b2A = b2; var e2A = e2;
          try
          {
            try_Predicate();
            lk = -1;
          }
          catch (p1A)
          {
            lk = -4;
          }
          b0 = b0A; e0 = e0A; l1 = l1A; if (l1 == 0) {end = e0A;} else {
          b1 = b1A; e1 = e1A; l2 = l2A; if (l2 == 0) {end = e1A;} else {
          b2 = b2A; e2 = e2A; end = e2A; }}
          memoize(5, e0, lk);
        }
      }
      switch (lk)
      {
      case 35:                      // '('
        whitespace();
        parse_ArgumentList();
        break;
      case 45:                      // '.'
        whitespace();
        parse_ObjectLookup();
        break;
      case -4:
        whitespace();
        parse_ArrayLookup();
        break;
      case 35909:                   // '[' ']'
        whitespace();
        parse_ArrayUnboxing();
        break;
      default:
        whitespace();
        parse_Predicate();
      }
    }
    eventHandler.endNonterminal("PostfixExpr", e0);
  }

  function try_PostfixExpr()
  {
    try_PrimaryExpr();
    for (;;)
    {
      lookahead1W(235);             // S^WS | EOF | '!' | '!=' | '(' | '(:' | ')' | '*' | '+' | ',' | '-' | '.' | '/' |
      if (l1 != 35                  // '('
       && l1 != 45                  // '.'
       && l1 != 69)                 // '['
      {
        break;
      }
      switch (l1)
      {
      case 69:                      // '['
        lookahead2W(273);           // EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral | StringLiteral |
        break;
      default:
        lk = l1;
      }
      if (lk == 35397)              // '[' '['
      {
        lk = memoized(5, e0);
        if (lk == 0)
        {
          var b0A = b0; var e0A = e0; var l1A = l1;
          var b1A = b1; var e1A = e1; var l2A = l2;
          var b2A = b2; var e2A = e2;
          try
          {
            try_Predicate();
            memoize(5, e0A, -1);
            lk = -6;
          }
          catch (p1A)
          {
            lk = -4;
            b0 = b0A; e0 = e0A; l1 = l1A; if (l1 == 0) {end = e0A;} else {
            b1 = b1A; e1 = e1A; l2 = l2A; if (l2 == 0) {end = e1A;} else {
            b2 = b2A; e2 = e2A; end = e2A; }}
            memoize(5, e0A, -4);
          }
        }
      }
      switch (lk)
      {
      case 35:                      // '('
        try_ArgumentList();
        break;
      case 45:                      // '.'
        try_ObjectLookup();
        break;
      case -4:
        try_ArrayLookup();
        break;
      case 35909:                   // '[' ']'
        try_ArrayUnboxing();
        break;
      case -6:
        break;
      default:
        try_Predicate();
      }
    }
  }

  function parse_ObjectLookup()
  {
    eventHandler.startNonterminal("ObjectLookup", e0);
    shift(45);                      // '.'
    lookahead1W(251);               // StringLiteral | NCName^Token | S^WS | '$' | '$$' | '(' | '(:' | 'after' |
    switch (l1)
    {
    case 11:                        // StringLiteral
      shift(11);                    // StringLiteral
      break;
    case 35:                        // '('
      whitespace();
      parse_ParenthesizedExpr();
      break;
    case 31:                        // '$'
      whitespace();
      parse_VarRef();
      break;
    case 32:                        // '$$'
      whitespace();
      parse_ContextItemExpr();
      break;
    default:
      whitespace();
      parse_NCName();
    }
    eventHandler.endNonterminal("ObjectLookup", e0);
  }

  function try_ObjectLookup()
  {
    shiftT(45);                     // '.'
    lookahead1W(251);               // StringLiteral | NCName^Token | S^WS | '$' | '$$' | '(' | '(:' | 'after' |
    switch (l1)
    {
    case 11:                        // StringLiteral
      shiftT(11);                   // StringLiteral
      break;
    case 35:                        // '('
      try_ParenthesizedExpr();
      break;
    case 31:                        // '$'
      try_VarRef();
      break;
    case 32:                        // '$$'
      try_ContextItemExpr();
      break;
    default:
      try_NCName();
    }
  }

  function parse_ArrayLookup()
  {
    eventHandler.startNonterminal("ArrayLookup", e0);
    shift(69);                      // '['
    lookahead1W(31);                // S^WS | '(:' | '['
    shift(69);                      // '['
    lookahead1W(267);               // EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral | StringLiteral |
    whitespace();
    parse_Expr();
    shift(70);                      // ']'
    lookahead1W(32);                // S^WS | '(:' | ']'
    shift(70);                      // ']'
    eventHandler.endNonterminal("ArrayLookup", e0);
  }

  function try_ArrayLookup()
  {
    shiftT(69);                     // '['
    lookahead1W(31);                // S^WS | '(:' | '['
    shiftT(69);                     // '['
    lookahead1W(267);               // EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral | StringLiteral |
    try_Expr();
    shiftT(70);                     // ']'
    lookahead1W(32);                // S^WS | '(:' | ']'
    shiftT(70);                     // ']'
  }

  function parse_ArrayUnboxing()
  {
    eventHandler.startNonterminal("ArrayUnboxing", e0);
    shift(69);                      // '['
    lookahead1W(32);                // S^WS | '(:' | ']'
    shift(70);                      // ']'
    eventHandler.endNonterminal("ArrayUnboxing", e0);
  }

  function try_ArrayUnboxing()
  {
    shiftT(69);                     // '['
    lookahead1W(32);                // S^WS | '(:' | ']'
    shiftT(70);                     // ']'
  }

  function parse_ArgumentList()
  {
    eventHandler.startNonterminal("ArgumentList", e0);
    shift(35);                      // '('
    lookahead1W(280);               // EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral | StringLiteral |
    if (l1 != 38)                   // ')'
    {
      whitespace();
      parse_Argument();
      for (;;)
      {
        lookahead1W(105);           // S^WS | '(:' | ')' | ','
        if (l1 != 42)               // ','
        {
          break;
        }
        shift(42);                  // ','
        lookahead1W(272);           // EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral | StringLiteral |
        whitespace();
        parse_Argument();
      }
    }
    shift(38);                      // ')'
    eventHandler.endNonterminal("ArgumentList", e0);
  }

  function try_ArgumentList()
  {
    shiftT(35);                     // '('
    lookahead1W(280);               // EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral | StringLiteral |
    if (l1 != 38)                   // ')'
    {
      try_Argument();
      for (;;)
      {
        lookahead1W(105);           // S^WS | '(:' | ')' | ','
        if (l1 != 42)               // ','
        {
          break;
        }
        shiftT(42);                 // ','
        lookahead1W(272);           // EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral | StringLiteral |
        try_Argument();
      }
    }
    shiftT(38);                     // ')'
  }

  function parse_PredicateList()
  {
    eventHandler.startNonterminal("PredicateList", e0);
    for (;;)
    {
      lookahead1W(228);             // S^WS | EOF | '!' | '!=' | '(:' | ')' | '*' | '+' | ',' | '-' | '/' | '//' | ':' |
      if (l1 != 69)                 // '['
      {
        break;
      }
      whitespace();
      parse_Predicate();
    }
    eventHandler.endNonterminal("PredicateList", e0);
  }

  function try_PredicateList()
  {
    for (;;)
    {
      lookahead1W(228);             // S^WS | EOF | '!' | '!=' | '(:' | ')' | '*' | '+' | ',' | '-' | '/' | '//' | ':' |
      if (l1 != 69)                 // '['
      {
        break;
      }
      try_Predicate();
    }
  }

  function parse_Predicate()
  {
    eventHandler.startNonterminal("Predicate", e0);
    shift(69);                      // '['
    lookahead1W(267);               // EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral | StringLiteral |
    whitespace();
    parse_Expr();
    shift(70);                      // ']'
    eventHandler.endNonterminal("Predicate", e0);
  }

  function try_Predicate()
  {
    shiftT(69);                     // '['
    lookahead1W(267);               // EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral | StringLiteral |
    try_Expr();
    shiftT(70);                     // ']'
  }

  function parse_Literal()
  {
    eventHandler.startNonterminal("Literal", e0);
    switch (l1)
    {
    case 11:                        // StringLiteral
      shift(11);                    // StringLiteral
      break;
    case 135:                       // 'false'
    case 255:                       // 'true'
      parse_BooleanLiteral();
      break;
    case 197:                       // 'null'
      parse_NullLiteral();
      break;
    default:
      parse_NumericLiteral();
    }
    eventHandler.endNonterminal("Literal", e0);
  }

  function try_Literal()
  {
    switch (l1)
    {
    case 11:                        // StringLiteral
      shiftT(11);                   // StringLiteral
      break;
    case 135:                       // 'false'
    case 255:                       // 'true'
      try_BooleanLiteral();
      break;
    case 197:                       // 'null'
      try_NullLiteral();
      break;
    default:
      try_NumericLiteral();
    }
  }

  function parse_BooleanLiteral()
  {
    eventHandler.startNonterminal("BooleanLiteral", e0);
    switch (l1)
    {
    case 255:                       // 'true'
      shift(255);                   // 'true'
      break;
    default:
      shift(135);                   // 'false'
    }
    eventHandler.endNonterminal("BooleanLiteral", e0);
  }

  function try_BooleanLiteral()
  {
    switch (l1)
    {
    case 255:                       // 'true'
      shiftT(255);                  // 'true'
      break;
    default:
      shiftT(135);                  // 'false'
    }
  }

  function parse_NullLiteral()
  {
    eventHandler.startNonterminal("NullLiteral", e0);
    shift(197);                     // 'null'
    eventHandler.endNonterminal("NullLiteral", e0);
  }

  function try_NullLiteral()
  {
    shiftT(197);                    // 'null'
  }

  function parse_NumericLiteral()
  {
    eventHandler.startNonterminal("NumericLiteral", e0);
    switch (l1)
    {
    case 8:                         // IntegerLiteral
      shift(8);                     // IntegerLiteral
      break;
    case 9:                         // DecimalLiteral
      shift(9);                     // DecimalLiteral
      break;
    default:
      shift(10);                    // DoubleLiteral
    }
    eventHandler.endNonterminal("NumericLiteral", e0);
  }

  function try_NumericLiteral()
  {
    switch (l1)
    {
    case 8:                         // IntegerLiteral
      shiftT(8);                    // IntegerLiteral
      break;
    case 9:                         // DecimalLiteral
      shiftT(9);                    // DecimalLiteral
      break;
    default:
      shiftT(10);                   // DoubleLiteral
    }
  }

  function parse_VarRef()
  {
    eventHandler.startNonterminal("VarRef", e0);
    shift(31);                      // '$'
    lookahead1W(246);               // EQName^Token | S^WS | '(:' | 'after' | 'allowing' | 'ancestor' |
    whitespace();
    parse_VarName();
    eventHandler.endNonterminal("VarRef", e0);
  }

  function try_VarRef()
  {
    shiftT(31);                     // '$'
    lookahead1W(246);               // EQName^Token | S^WS | '(:' | 'after' | 'allowing' | 'ancestor' |
    try_VarName();
  }

  function parse_VarName()
  {
    eventHandler.startNonterminal("VarName", e0);
    parse_EQName();
    eventHandler.endNonterminal("VarName", e0);
  }

  function try_VarName()
  {
    try_EQName();
  }

  function parse_ParenthesizedExpr()
  {
    eventHandler.startNonterminal("ParenthesizedExpr", e0);
    shift(35);                      // '('
    lookahead1W(270);               // EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral | StringLiteral |
    if (l1 != 38)                   // ')'
    {
      whitespace();
      parse_Expr();
    }
    shift(38);                      // ')'
    eventHandler.endNonterminal("ParenthesizedExpr", e0);
  }

  function try_ParenthesizedExpr()
  {
    shiftT(35);                     // '('
    lookahead1W(270);               // EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral | StringLiteral |
    if (l1 != 38)                   // ')'
    {
      try_Expr();
    }
    shiftT(38);                     // ')'
  }

  function parse_ContextItemExpr()
  {
    eventHandler.startNonterminal("ContextItemExpr", e0);
    shift(32);                      // '$$'
    eventHandler.endNonterminal("ContextItemExpr", e0);
  }

  function try_ContextItemExpr()
  {
    shiftT(32);                     // '$$'
  }

  function parse_OrderedExpr()
  {
    eventHandler.startNonterminal("OrderedExpr", e0);
    shift(206);                     // 'ordered'
    lookahead1W(90);                // S^WS | '(:' | '{'
    shift(281);                     // '{'
    lookahead1W(267);               // EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral | StringLiteral |
    whitespace();
    parse_Expr();
    shift(287);                     // '}'
    eventHandler.endNonterminal("OrderedExpr", e0);
  }

  function try_OrderedExpr()
  {
    shiftT(206);                    // 'ordered'
    lookahead1W(90);                // S^WS | '(:' | '{'
    shiftT(281);                    // '{'
    lookahead1W(267);               // EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral | StringLiteral |
    try_Expr();
    shiftT(287);                    // '}'
  }

  function parse_UnorderedExpr()
  {
    eventHandler.startNonterminal("UnorderedExpr", e0);
    shift(262);                     // 'unordered'
    lookahead1W(90);                // S^WS | '(:' | '{'
    shift(281);                     // '{'
    lookahead1W(267);               // EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral | StringLiteral |
    whitespace();
    parse_Expr();
    shift(287);                     // '}'
    eventHandler.endNonterminal("UnorderedExpr", e0);
  }

  function try_UnorderedExpr()
  {
    shiftT(262);                    // 'unordered'
    lookahead1W(90);                // S^WS | '(:' | '{'
    shiftT(281);                    // '{'
    lookahead1W(267);               // EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral | StringLiteral |
    try_Expr();
    shiftT(287);                    // '}'
  }

  function parse_FunctionCall()
  {
    eventHandler.startNonterminal("FunctionCall", e0);
    parse_FunctionName();
    lookahead1W(22);                // S^WS | '(' | '(:'
    whitespace();
    parse_ArgumentList();
    eventHandler.endNonterminal("FunctionCall", e0);
  }

  function try_FunctionCall()
  {
    try_FunctionName();
    lookahead1W(22);                // S^WS | '(' | '(:'
    try_ArgumentList();
  }

  function parse_Argument()
  {
    eventHandler.startNonterminal("Argument", e0);
    switch (l1)
    {
    case 65:                        // '?'
      parse_ArgumentPlaceholder();
      break;
    default:
      parse_ExprSingle();
    }
    eventHandler.endNonterminal("Argument", e0);
  }

  function try_Argument()
  {
    switch (l1)
    {
    case 65:                        // '?'
      try_ArgumentPlaceholder();
      break;
    default:
      try_ExprSingle();
    }
  }

  function parse_ArgumentPlaceholder()
  {
    eventHandler.startNonterminal("ArgumentPlaceholder", e0);
    shift(65);                      // '?'
    eventHandler.endNonterminal("ArgumentPlaceholder", e0);
  }

  function try_ArgumentPlaceholder()
  {
    shiftT(65);                     // '?'
  }

  function parse_Constructor()
  {
    eventHandler.startNonterminal("Constructor", e0);
    switch (l1)
    {
    case 55:                        // '<'
    case 56:                        // '<!--'
    case 60:                        // '<?'
      parse_DirectConstructor();
      break;
    default:
      parse_ComputedConstructor();
    }
    eventHandler.endNonterminal("Constructor", e0);
  }

  function try_Constructor()
  {
    switch (l1)
    {
    case 55:                        // '<'
    case 56:                        // '<!--'
    case 60:                        // '<?'
      try_DirectConstructor();
      break;
    default:
      try_ComputedConstructor();
    }
  }

  function parse_DirectConstructor()
  {
    eventHandler.startNonterminal("DirectConstructor", e0);
    switch (l1)
    {
    case 55:                        // '<'
      parse_DirElemConstructor();
      break;
    case 56:                        // '<!--'
      parse_DirCommentConstructor();
      break;
    default:
      parse_DirPIConstructor();
    }
    eventHandler.endNonterminal("DirectConstructor", e0);
  }

  function try_DirectConstructor()
  {
    switch (l1)
    {
    case 55:                        // '<'
      try_DirElemConstructor();
      break;
    case 56:                        // '<!--'
      try_DirCommentConstructor();
      break;
    default:
      try_DirPIConstructor();
    }
  }

  function parse_DirElemConstructor()
  {
    eventHandler.startNonterminal("DirElemConstructor", e0);
    shift(55);                      // '<'
    lookahead1(4);                  // QName
    shift(20);                      // QName
    parse_DirAttributeList();
    switch (l1)
    {
    case 49:                        // '/>'
      shift(49);                    // '/>'
      break;
    default:
      shift(62);                    // '>'
      for (;;)
      {
        lookahead1(196);            // CDataSection | PredefinedEntityRef | ElementContentChar | CharRef | '<' |
        if (l1 == 57)               // '</'
        {
          break;
        }
        parse_DirElemContent();
      }
      shift(57);                    // '</'
      lookahead1(4);                // QName
      shift(20);                    // QName
      lookahead1(12);               // S | '>'
      if (l1 == 21)                 // S
      {
        shift(21);                  // S
      }
      lookahead1(8);                // '>'
      shift(62);                    // '>'
    }
    eventHandler.endNonterminal("DirElemConstructor", e0);
  }

  function try_DirElemConstructor()
  {
    shiftT(55);                     // '<'
    lookahead1(4);                  // QName
    shiftT(20);                     // QName
    try_DirAttributeList();
    switch (l1)
    {
    case 49:                        // '/>'
      shiftT(49);                   // '/>'
      break;
    default:
      shiftT(62);                   // '>'
      for (;;)
      {
        lookahead1(196);            // CDataSection | PredefinedEntityRef | ElementContentChar | CharRef | '<' |
        if (l1 == 57)               // '</'
        {
          break;
        }
        try_DirElemContent();
      }
      shiftT(57);                   // '</'
      lookahead1(4);                // QName
      shiftT(20);                   // QName
      lookahead1(12);               // S | '>'
      if (l1 == 21)                 // S
      {
        shiftT(21);                 // S
      }
      lookahead1(8);                // '>'
      shiftT(62);                   // '>'
    }
  }

  function parse_DirAttributeList()
  {
    eventHandler.startNonterminal("DirAttributeList", e0);
    for (;;)
    {
      lookahead1(19);               // S | '/>' | '>'
      if (l1 != 21)                 // S
      {
        break;
      }
      shift(21);                    // S
      lookahead1(94);               // QName | S | '/>' | '>'
      if (l1 == 20)                 // QName
      {
        shift(20);                  // QName
        lookahead1(11);             // S | '='
        if (l1 == 21)               // S
        {
          shift(21);                // S
        }
        lookahead1(7);              // '='
        shift(61);                  // '='
        lookahead1(18);             // S | '"' | "'"
        if (l1 == 21)               // S
        {
          shift(21);                // S
        }
        parse_DirAttributeValue();
      }
    }
    eventHandler.endNonterminal("DirAttributeList", e0);
  }

  function try_DirAttributeList()
  {
    for (;;)
    {
      lookahead1(19);               // S | '/>' | '>'
      if (l1 != 21)                 // S
      {
        break;
      }
      shiftT(21);                   // S
      lookahead1(94);               // QName | S | '/>' | '>'
      if (l1 == 20)                 // QName
      {
        shiftT(20);                 // QName
        lookahead1(11);             // S | '='
        if (l1 == 21)               // S
        {
          shiftT(21);               // S
        }
        lookahead1(7);              // '='
        shiftT(61);                 // '='
        lookahead1(18);             // S | '"' | "'"
        if (l1 == 21)               // S
        {
          shiftT(21);               // S
        }
        try_DirAttributeValue();
      }
    }
  }

  function parse_DirAttributeValue()
  {
    eventHandler.startNonterminal("DirAttributeValue", e0);
    lookahead1(14);                 // '"' | "'"
    switch (l1)
    {
    case 28:                        // '"'
      shift(28);                    // '"'
      for (;;)
      {
        lookahead1(185);            // PredefinedEntityRef | EscapeQuot | QuotAttrContentChar | CharRef | '"' | '{' |
        if (l1 == 28)               // '"'
        {
          break;
        }
        switch (l1)
        {
        case 13:                    // EscapeQuot
          shift(13);                // EscapeQuot
          break;
        default:
          parse_QuotAttrValueContent();
        }
      }
      shift(28);                    // '"'
      break;
    default:
      shift(34);                    // "'"
      for (;;)
      {
        lookahead1(186);            // PredefinedEntityRef | EscapeApos | AposAttrContentChar | CharRef | "'" | '{' |
        if (l1 == 34)               // "'"
        {
          break;
        }
        switch (l1)
        {
        case 14:                    // EscapeApos
          shift(14);                // EscapeApos
          break;
        default:
          parse_AposAttrValueContent();
        }
      }
      shift(34);                    // "'"
    }
    eventHandler.endNonterminal("DirAttributeValue", e0);
  }

  function try_DirAttributeValue()
  {
    lookahead1(14);                 // '"' | "'"
    switch (l1)
    {
    case 28:                        // '"'
      shiftT(28);                   // '"'
      for (;;)
      {
        lookahead1(185);            // PredefinedEntityRef | EscapeQuot | QuotAttrContentChar | CharRef | '"' | '{' |
        if (l1 == 28)               // '"'
        {
          break;
        }
        switch (l1)
        {
        case 13:                    // EscapeQuot
          shiftT(13);               // EscapeQuot
          break;
        default:
          try_QuotAttrValueContent();
        }
      }
      shiftT(28);                   // '"'
      break;
    default:
      shiftT(34);                   // "'"
      for (;;)
      {
        lookahead1(186);            // PredefinedEntityRef | EscapeApos | AposAttrContentChar | CharRef | "'" | '{' |
        if (l1 == 34)               // "'"
        {
          break;
        }
        switch (l1)
        {
        case 14:                    // EscapeApos
          shiftT(14);               // EscapeApos
          break;
        default:
          try_AposAttrValueContent();
        }
      }
      shiftT(34);                   // "'"
    }
  }

  function parse_QuotAttrValueContent()
  {
    eventHandler.startNonterminal("QuotAttrValueContent", e0);
    switch (l1)
    {
    case 16:                        // QuotAttrContentChar
      shift(16);                    // QuotAttrContentChar
      break;
    default:
      parse_CommonContent();
    }
    eventHandler.endNonterminal("QuotAttrValueContent", e0);
  }

  function try_QuotAttrValueContent()
  {
    switch (l1)
    {
    case 16:                        // QuotAttrContentChar
      shiftT(16);                   // QuotAttrContentChar
      break;
    default:
      try_CommonContent();
    }
  }

  function parse_AposAttrValueContent()
  {
    eventHandler.startNonterminal("AposAttrValueContent", e0);
    switch (l1)
    {
    case 17:                        // AposAttrContentChar
      shift(17);                    // AposAttrContentChar
      break;
    default:
      parse_CommonContent();
    }
    eventHandler.endNonterminal("AposAttrValueContent", e0);
  }

  function try_AposAttrValueContent()
  {
    switch (l1)
    {
    case 17:                        // AposAttrContentChar
      shiftT(17);                   // AposAttrContentChar
      break;
    default:
      try_CommonContent();
    }
  }

  function parse_DirElemContent()
  {
    eventHandler.startNonterminal("DirElemContent", e0);
    switch (l1)
    {
    case 55:                        // '<'
    case 56:                        // '<!--'
    case 60:                        // '<?'
      parse_DirectConstructor();
      break;
    case 4:                         // CDataSection
      shift(4);                     // CDataSection
      break;
    case 15:                        // ElementContentChar
      shift(15);                    // ElementContentChar
      break;
    default:
      parse_CommonContent();
    }
    eventHandler.endNonterminal("DirElemContent", e0);
  }

  function try_DirElemContent()
  {
    switch (l1)
    {
    case 55:                        // '<'
    case 56:                        // '<!--'
    case 60:                        // '<?'
      try_DirectConstructor();
      break;
    case 4:                         // CDataSection
      shiftT(4);                    // CDataSection
      break;
    case 15:                        // ElementContentChar
      shiftT(15);                   // ElementContentChar
      break;
    default:
      try_CommonContent();
    }
  }

  function parse_DirCommentConstructor()
  {
    eventHandler.startNonterminal("DirCommentConstructor", e0);
    shift(56);                      // '<!--'
    lookahead1(1);                  // DirCommentContents
    shift(2);                       // DirCommentContents
    lookahead1(6);                  // '-->'
    shift(44);                      // '-->'
    eventHandler.endNonterminal("DirCommentConstructor", e0);
  }

  function try_DirCommentConstructor()
  {
    shiftT(56);                     // '<!--'
    lookahead1(1);                  // DirCommentContents
    shiftT(2);                      // DirCommentContents
    lookahead1(6);                  // '-->'
    shiftT(44);                     // '-->'
  }

  function parse_DirPIConstructor()
  {
    eventHandler.startNonterminal("DirPIConstructor", e0);
    shift(60);                      // '<?'
    lookahead1(3);                  // PITarget
    shift(18);                      // PITarget
    lookahead1(13);                 // S | '?>'
    if (l1 == 21)                   // S
    {
      shift(21);                    // S
      lookahead1(2);                // DirPIContents
      shift(3);                     // DirPIContents
    }
    lookahead1(9);                  // '?>'
    shift(66);                      // '?>'
    eventHandler.endNonterminal("DirPIConstructor", e0);
  }

  function try_DirPIConstructor()
  {
    shiftT(60);                     // '<?'
    lookahead1(3);                  // PITarget
    shiftT(18);                     // PITarget
    lookahead1(13);                 // S | '?>'
    if (l1 == 21)                   // S
    {
      shiftT(21);                   // S
      lookahead1(2);                // DirPIContents
      shiftT(3);                    // DirPIContents
    }
    lookahead1(9);                  // '?>'
    shiftT(66);                     // '?>'
  }

  function parse_ComputedConstructor()
  {
    eventHandler.startNonterminal("ComputedConstructor", e0);
    switch (l1)
    {
    case 120:                       // 'document'
      parse_CompDocConstructor();
      break;
    case 122:                       // 'element'
      parse_CompElemConstructor();
      break;
    case 83:                        // 'attribute'
      parse_CompAttrConstructor();
      break;
    case 187:                       // 'namespace'
      parse_CompNamespaceConstructor();
      break;
    case 249:                       // 'text'
      parse_CompTextConstructor();
      break;
    case 97:                        // 'comment'
      parse_CompCommentConstructor();
      break;
    default:
      parse_CompPIConstructor();
    }
    eventHandler.endNonterminal("ComputedConstructor", e0);
  }

  function try_ComputedConstructor()
  {
    switch (l1)
    {
    case 120:                       // 'document'
      try_CompDocConstructor();
      break;
    case 122:                       // 'element'
      try_CompElemConstructor();
      break;
    case 83:                        // 'attribute'
      try_CompAttrConstructor();
      break;
    case 187:                       // 'namespace'
      try_CompNamespaceConstructor();
      break;
    case 249:                       // 'text'
      try_CompTextConstructor();
      break;
    case 97:                        // 'comment'
      try_CompCommentConstructor();
      break;
    default:
      try_CompPIConstructor();
    }
  }

  function parse_CompElemConstructor()
  {
    eventHandler.startNonterminal("CompElemConstructor", e0);
    shift(122);                     // 'element'
    lookahead1W(250);               // EQName^Token | S^WS | '(:' | 'after' | 'allowing' | 'ancestor' |
    switch (l1)
    {
    case 281:                       // '{'
      shift(281);                   // '{'
      lookahead1W(267);             // EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral | StringLiteral |
      whitespace();
      parse_Expr();
      shift(287);                   // '}'
      break;
    default:
      whitespace();
      parse_EQName();
    }
    lookahead1W(90);                // S^WS | '(:' | '{'
    shift(281);                     // '{'
    lookahead1W(281);               // EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral | StringLiteral |
    if (l1 != 287)                  // '}'
    {
      whitespace();
      parse_ContentExpr();
    }
    shift(287);                     // '}'
    eventHandler.endNonterminal("CompElemConstructor", e0);
  }

  function try_CompElemConstructor()
  {
    shiftT(122);                    // 'element'
    lookahead1W(250);               // EQName^Token | S^WS | '(:' | 'after' | 'allowing' | 'ancestor' |
    switch (l1)
    {
    case 281:                       // '{'
      shiftT(281);                  // '{'
      lookahead1W(267);             // EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral | StringLiteral |
      try_Expr();
      shiftT(287);                  // '}'
      break;
    default:
      try_EQName();
    }
    lookahead1W(90);                // S^WS | '(:' | '{'
    shiftT(281);                    // '{'
    lookahead1W(281);               // EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral | StringLiteral |
    if (l1 != 287)                  // '}'
    {
      try_ContentExpr();
    }
    shiftT(287);                    // '}'
  }

  function parse_CompNamespaceConstructor()
  {
    eventHandler.startNonterminal("CompNamespaceConstructor", e0);
    shift(187);                     // 'namespace'
    lookahead1W(242);               // NCName^Token | S^WS | '(:' | 'after' | 'allowing' | 'ancestor' |
    switch (l1)
    {
    case 281:                       // '{'
      shift(281);                   // '{'
      lookahead1W(267);             // EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral | StringLiteral |
      whitespace();
      parse_PrefixExpr();
      shift(287);                   // '}'
      break;
    default:
      whitespace();
      parse_Prefix();
    }
    lookahead1W(90);                // S^WS | '(:' | '{'
    shift(281);                     // '{'
    lookahead1W(267);               // EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral | StringLiteral |
    whitespace();
    parse_URIExpr();
    shift(287);                     // '}'
    eventHandler.endNonterminal("CompNamespaceConstructor", e0);
  }

  function try_CompNamespaceConstructor()
  {
    shiftT(187);                    // 'namespace'
    lookahead1W(242);               // NCName^Token | S^WS | '(:' | 'after' | 'allowing' | 'ancestor' |
    switch (l1)
    {
    case 281:                       // '{'
      shiftT(281);                  // '{'
      lookahead1W(267);             // EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral | StringLiteral |
      try_PrefixExpr();
      shiftT(287);                  // '}'
      break;
    default:
      try_Prefix();
    }
    lookahead1W(90);                // S^WS | '(:' | '{'
    shiftT(281);                    // '{'
    lookahead1W(267);               // EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral | StringLiteral |
    try_URIExpr();
    shiftT(287);                    // '}'
  }

  function parse_Prefix()
  {
    eventHandler.startNonterminal("Prefix", e0);
    parse_NCName();
    eventHandler.endNonterminal("Prefix", e0);
  }

  function try_Prefix()
  {
    try_NCName();
  }

  function parse_PrefixExpr()
  {
    eventHandler.startNonterminal("PrefixExpr", e0);
    parse_Expr();
    eventHandler.endNonterminal("PrefixExpr", e0);
  }

  function try_PrefixExpr()
  {
    try_Expr();
  }

  function parse_URIExpr()
  {
    eventHandler.startNonterminal("URIExpr", e0);
    parse_Expr();
    eventHandler.endNonterminal("URIExpr", e0);
  }

  function try_URIExpr()
  {
    try_Expr();
  }

  function parse_FunctionItemExpr()
  {
    eventHandler.startNonterminal("FunctionItemExpr", e0);
    switch (l1)
    {
    case 147:                       // 'function'
      lookahead2W(95);              // S^WS | '#' | '(' | '(:'
      break;
    default:
      lk = l1;
    }
    switch (lk)
    {
    case 33:                        // '%'
    case 18067:                     // 'function' '('
      parse_InlineFunctionExpr();
      break;
    default:
      parse_NamedFunctionRef();
    }
    eventHandler.endNonterminal("FunctionItemExpr", e0);
  }

  function try_FunctionItemExpr()
  {
    switch (l1)
    {
    case 147:                       // 'function'
      lookahead2W(95);              // S^WS | '#' | '(' | '(:'
      break;
    default:
      lk = l1;
    }
    switch (lk)
    {
    case 33:                        // '%'
    case 18067:                     // 'function' '('
      try_InlineFunctionExpr();
      break;
    default:
      try_NamedFunctionRef();
    }
  }

  function parse_NamedFunctionRef()
  {
    eventHandler.startNonterminal("NamedFunctionRef", e0);
    parse_EQName();
    lookahead1W(20);                // S^WS | '#' | '(:'
    shift(29);                      // '#'
    lookahead1W(16);                // IntegerLiteral | S^WS | '(:'
    shift(8);                       // IntegerLiteral
    eventHandler.endNonterminal("NamedFunctionRef", e0);
  }

  function try_NamedFunctionRef()
  {
    try_EQName();
    lookahead1W(20);                // S^WS | '#' | '(:'
    shiftT(29);                     // '#'
    lookahead1W(16);                // IntegerLiteral | S^WS | '(:'
    shiftT(8);                      // IntegerLiteral
  }

  function parse_InlineFunctionExpr()
  {
    eventHandler.startNonterminal("InlineFunctionExpr", e0);
    for (;;)
    {
      lookahead1W(101);             // S^WS | '%' | '(:' | 'function'
      if (l1 != 33)                 // '%'
      {
        break;
      }
      whitespace();
      parse_Annotation();
    }
    shift(147);                     // 'function'
    lookahead1W(22);                // S^WS | '(' | '(:'
    shift(35);                      // '('
    lookahead1W(98);                // S^WS | '$' | '(:' | ')'
    if (l1 == 31)                   // '$'
    {
      whitespace();
      parse_ParamList();
    }
    shift(38);                      // ')'
    lookahead1W(115);               // S^WS | '(:' | 'as' | '{'
    if (l1 == 80)                   // 'as'
    {
      shift(80);                    // 'as'
      lookahead1W(254);             // EQName^Token | S^WS | '%' | '(' | '(:' | 'after' | 'allowing' | 'ancestor' |
      whitespace();
      parse_SequenceType();
    }
    lookahead1W(90);                // S^WS | '(:' | '{'
    whitespace();
    parse_FunctionBody();
    eventHandler.endNonterminal("InlineFunctionExpr", e0);
  }

  function try_InlineFunctionExpr()
  {
    for (;;)
    {
      lookahead1W(101);             // S^WS | '%' | '(:' | 'function'
      if (l1 != 33)                 // '%'
      {
        break;
      }
      try_Annotation();
    }
    shiftT(147);                    // 'function'
    lookahead1W(22);                // S^WS | '(' | '(:'
    shiftT(35);                     // '('
    lookahead1W(98);                // S^WS | '$' | '(:' | ')'
    if (l1 == 31)                   // '$'
    {
      try_ParamList();
    }
    shiftT(38);                     // ')'
    lookahead1W(115);               // S^WS | '(:' | 'as' | '{'
    if (l1 == 80)                   // 'as'
    {
      shiftT(80);                   // 'as'
      lookahead1W(254);             // EQName^Token | S^WS | '%' | '(' | '(:' | 'after' | 'allowing' | 'ancestor' |
      try_SequenceType();
    }
    lookahead1W(90);                // S^WS | '(:' | '{'
    try_FunctionBody();
  }

  function parse_SingleType()
  {
    eventHandler.startNonterminal("SingleType", e0);
    parse_SimpleTypeName();
    lookahead1W(226);               // S^WS | EOF | '!=' | '(:' | ')' | '*' | '+' | ',' | '-' | ':' | ';' | '<' | '<<' |
    if (l1 == 65)                   // '?'
    {
      shift(65);                    // '?'
    }
    eventHandler.endNonterminal("SingleType", e0);
  }

  function try_SingleType()
  {
    try_SimpleTypeName();
    lookahead1W(226);               // S^WS | EOF | '!=' | '(:' | ')' | '*' | '+' | ',' | '-' | ':' | ';' | '<' | '<<' |
    if (l1 == 65)                   // '?'
    {
      shiftT(65);                   // '?'
    }
  }

  function parse_TypeDeclaration()
  {
    eventHandler.startNonterminal("TypeDeclaration", e0);
    shift(80);                      // 'as'
    lookahead1W(254);               // EQName^Token | S^WS | '%' | '(' | '(:' | 'after' | 'allowing' | 'ancestor' |
    whitespace();
    parse_SequenceType();
    eventHandler.endNonterminal("TypeDeclaration", e0);
  }

  function try_TypeDeclaration()
  {
    shiftT(80);                     // 'as'
    lookahead1W(254);               // EQName^Token | S^WS | '%' | '(' | '(:' | 'after' | 'allowing' | 'ancestor' |
    try_SequenceType();
  }

  function parse_SequenceType()
  {
    eventHandler.startNonterminal("SequenceType", e0);
    switch (l1)
    {
    case 35:                        // '('
      lookahead2W(259);             // EQName^Token | S^WS | '%' | '(' | '(:' | ')' | 'after' | 'allowing' |
      break;
    case 125:                       // 'empty-sequence'
      lookahead2W(233);             // S^WS | EOF | '!=' | '(' | '(:' | ')' | '*' | '*' | '+' | ',' | '-' | ':' | ':=' |
      break;
    default:
      lk = l1;
    }
    switch (lk)
    {
    case 18045:                     // 'empty-sequence' '('
    case 19491:                     // '(' ')'
      if (l1 == 125)                // 'empty-sequence'
      {
        shift(125);                 // 'empty-sequence'
      }
      lookahead1W(22);              // S^WS | '(' | '(:'
      shift(35);                    // '('
      lookahead1W(23);              // S^WS | '(:' | ')'
      shift(38);                    // ')'
      break;
    default:
      parse_ItemType();
      lookahead1W(229);             // S^WS | EOF | '!=' | '(:' | ')' | '*' | '*' | '+' | ',' | '-' | ':' | ':=' | ';' |
      switch (l1)
      {
      case 40:                      // '*'
      case 41:                      // '+'
      case 65:                      // '?'
        whitespace();
        parse_OccurrenceIndicator();
        break;
      default:
        break;
      }
    }
    eventHandler.endNonterminal("SequenceType", e0);
  }

  function try_SequenceType()
  {
    switch (l1)
    {
    case 35:                        // '('
      lookahead2W(259);             // EQName^Token | S^WS | '%' | '(' | '(:' | ')' | 'after' | 'allowing' |
      break;
    case 125:                       // 'empty-sequence'
      lookahead2W(233);             // S^WS | EOF | '!=' | '(' | '(:' | ')' | '*' | '*' | '+' | ',' | '-' | ':' | ':=' |
      break;
    default:
      lk = l1;
    }
    switch (lk)
    {
    case 18045:                     // 'empty-sequence' '('
    case 19491:                     // '(' ')'
      if (l1 == 125)                // 'empty-sequence'
      {
        shiftT(125);                // 'empty-sequence'
      }
      lookahead1W(22);              // S^WS | '(' | '(:'
      shiftT(35);                   // '('
      lookahead1W(23);              // S^WS | '(:' | ')'
      shiftT(38);                   // ')'
      break;
    default:
      try_ItemType();
      lookahead1W(229);             // S^WS | EOF | '!=' | '(:' | ')' | '*' | '*' | '+' | ',' | '-' | ':' | ':=' | ';' |
      switch (l1)
      {
      case 40:                      // '*'
      case 41:                      // '+'
      case 65:                      // '?'
        try_OccurrenceIndicator();
        break;
      default:
        break;
      }
    }
  }

  function parse_OccurrenceIndicator()
  {
    eventHandler.startNonterminal("OccurrenceIndicator", e0);
    switch (l1)
    {
    case 65:                        // '?'
      shift(65);                    // '?'
      break;
    case 40:                        // '*'
      shift(40);                    // '*'
      break;
    default:
      shift(41);                    // '+'
    }
    eventHandler.endNonterminal("OccurrenceIndicator", e0);
  }

  function try_OccurrenceIndicator()
  {
    switch (l1)
    {
    case 65:                        // '?'
      shiftT(65);                   // '?'
      break;
    case 40:                        // '*'
      shiftT(40);                   // '*'
      break;
    default:
      shiftT(41);                   // '+'
    }
  }

  function parse_ItemType()
  {
    eventHandler.startNonterminal("ItemType", e0);
    switch (l1)
    {
    case 79:                        // 'array'
    case 83:                        // 'attribute'
    case 97:                        // 'comment'
    case 121:                       // 'document-node'
    case 122:                       // 'element'
    case 147:                       // 'function'
    case 167:                       // 'item'
    case 169:                       // 'json-item'
    case 188:                       // 'namespace-node'
    case 194:                       // 'node'
    case 198:                       // 'object'
    case 220:                       // 'processing-instruction'
    case 230:                       // 'schema-attribute'
    case 231:                       // 'schema-element'
    case 247:                       // 'structured-item'
    case 249:                       // 'text'
      lookahead2W(233);             // S^WS | EOF | '!=' | '(' | '(:' | ')' | '*' | '*' | '+' | ',' | '-' | ':' | ':=' |
      break;
    default:
      lk = l1;
    }
    if (lk == 12879                 // 'array' EOF
     || lk == 12969                 // 'json-item' EOF
     || lk == 12998                 // 'object' EOF
     || lk == 13047                 // 'structured-item' EOF
     || lk == 13903                 // 'array' '!='
     || lk == 13993                 // 'json-item' '!='
     || lk == 14022                 // 'object' '!='
     || lk == 14071                 // 'structured-item' '!='
     || lk == 19535                 // 'array' ')'
     || lk == 19625                 // 'json-item' ')'
     || lk == 19654                 // 'object' ')'
     || lk == 19703                 // 'structured-item' ')'
     || lk == 20047                 // 'array' '*'
     || lk == 20137                 // 'json-item' '*'
     || lk == 20166                 // 'object' '*'
     || lk == 20215                 // 'structured-item' '*'
     || lk == 20559                 // 'array' '*'
     || lk == 20649                 // 'json-item' '*'
     || lk == 20678                 // 'object' '*'
     || lk == 20727                 // 'structured-item' '*'
     || lk == 21071                 // 'array' '+'
     || lk == 21161                 // 'json-item' '+'
     || lk == 21190                 // 'object' '+'
     || lk == 21239                 // 'structured-item' '+'
     || lk == 21583                 // 'array' ','
     || lk == 21673                 // 'json-item' ','
     || lk == 21702                 // 'object' ','
     || lk == 21751                 // 'structured-item' ','
     || lk == 22095                 // 'array' '-'
     || lk == 22185                 // 'json-item' '-'
     || lk == 22214                 // 'object' '-'
     || lk == 22263                 // 'structured-item' '-'
     || lk == 25679                 // 'array' ':'
     || lk == 25769                 // 'json-item' ':'
     || lk == 25798                 // 'object' ':'
     || lk == 25847                 // 'structured-item' ':'
     || lk == 27215                 // 'array' ':='
     || lk == 27305                 // 'json-item' ':='
     || lk == 27334                 // 'object' ':='
     || lk == 27383                 // 'structured-item' ':='
     || lk == 27727                 // 'array' ';'
     || lk == 27817                 // 'json-item' ';'
     || lk == 27846                 // 'object' ';'
     || lk == 27895                 // 'structured-item' ';'
     || lk == 28239                 // 'array' '<'
     || lk == 28329                 // 'json-item' '<'
     || lk == 28358                 // 'object' '<'
     || lk == 28407                 // 'structured-item' '<'
     || lk == 29775                 // 'array' '<<'
     || lk == 29865                 // 'json-item' '<<'
     || lk == 29894                 // 'object' '<<'
     || lk == 29943                 // 'structured-item' '<<'
     || lk == 30287                 // 'array' '<='
     || lk == 30377                 // 'json-item' '<='
     || lk == 30406                 // 'object' '<='
     || lk == 30455                 // 'structured-item' '<='
     || lk == 31311                 // 'array' '='
     || lk == 31401                 // 'json-item' '='
     || lk == 31430                 // 'object' '='
     || lk == 31479                 // 'structured-item' '='
     || lk == 31823                 // 'array' '>'
     || lk == 31913                 // 'json-item' '>'
     || lk == 31942                 // 'object' '>'
     || lk == 31991                 // 'structured-item' '>'
     || lk == 32335                 // 'array' '>='
     || lk == 32425                 // 'json-item' '>='
     || lk == 32454                 // 'object' '>='
     || lk == 32503                 // 'structured-item' '>='
     || lk == 32847                 // 'array' '>>'
     || lk == 32937                 // 'json-item' '>>'
     || lk == 32966                 // 'object' '>>'
     || lk == 33015                 // 'structured-item' '>>'
     || lk == 33359                 // 'array' '?'
     || lk == 33449                 // 'json-item' '?'
     || lk == 33478                 // 'object' '?'
     || lk == 33527                 // 'structured-item' '?'
     || lk == 35919                 // 'array' ']'
     || lk == 36009                 // 'json-item' ']'
     || lk == 36038                 // 'object' ']'
     || lk == 36087                 // 'structured-item' ']'
     || lk == 36431                 // 'array' 'after'
     || lk == 36521                 // 'json-item' 'after'
     || lk == 36550                 // 'object' 'after'
     || lk == 36599                 // 'structured-item' 'after'
     || lk == 37455                 // 'array' 'allowing'
     || lk == 37545                 // 'json-item' 'allowing'
     || lk == 37574                 // 'object' 'allowing'
     || lk == 37623                 // 'structured-item' 'allowing'
     || lk == 38991                 // 'array' 'and'
     || lk == 39081                 // 'json-item' 'and'
     || lk == 39110                 // 'object' 'and'
     || lk == 39159                 // 'structured-item' 'and'
     || lk == 41039                 // 'array' 'as'
     || lk == 41129                 // 'json-item' 'as'
     || lk == 41158                 // 'object' 'as'
     || lk == 41207                 // 'structured-item' 'as'
     || lk == 41551                 // 'array' 'ascending'
     || lk == 41641                 // 'json-item' 'ascending'
     || lk == 41670                 // 'object' 'ascending'
     || lk == 41719                 // 'structured-item' 'ascending'
     || lk == 42063                 // 'array' 'at'
     || lk == 42153                 // 'json-item' 'at'
     || lk == 42182                 // 'object' 'at'
     || lk == 42231                 // 'structured-item' 'at'
     || lk == 43599                 // 'array' 'before'
     || lk == 43689                 // 'json-item' 'before'
     || lk == 43718                 // 'object' 'before'
     || lk == 43767                 // 'structured-item' 'before'
     || lk == 45647                 // 'array' 'case'
     || lk == 45737                 // 'json-item' 'case'
     || lk == 45766                 // 'object' 'case'
     || lk == 45815                 // 'structured-item' 'case'
     || lk == 48719                 // 'array' 'collation'
     || lk == 48809                 // 'json-item' 'collation'
     || lk == 48838                 // 'object' 'collation'
     || lk == 48887                 // 'structured-item' 'collation'
     || lk == 51279                 // 'array' 'contains'
     || lk == 51369                 // 'json-item' 'contains'
     || lk == 51398                 // 'object' 'contains'
     || lk == 51447                 // 'structured-item' 'contains'
     || lk == 54351                 // 'array' 'count'
     || lk == 54441                 // 'json-item' 'count'
     || lk == 54470                 // 'object' 'count'
     || lk == 54519                 // 'structured-item' 'count'
     || lk == 56399                 // 'array' 'default'
     || lk == 56489                 // 'json-item' 'default'
     || lk == 56518                 // 'object' 'default'
     || lk == 56567                 // 'structured-item' 'default'
     || lk == 58447                 // 'array' 'descending'
     || lk == 58537                 // 'json-item' 'descending'
     || lk == 58566                 // 'object' 'descending'
     || lk == 58615                 // 'structured-item' 'descending'
     || lk == 61007                 // 'array' 'div'
     || lk == 61097                 // 'json-item' 'div'
     || lk == 61126                 // 'object' 'div'
     || lk == 61175                 // 'structured-item' 'div'
     || lk == 63055                 // 'array' 'else'
     || lk == 63145                 // 'json-item' 'else'
     || lk == 63174                 // 'object' 'else'
     || lk == 63223                 // 'structured-item' 'else'
     || lk == 63567                 // 'array' 'empty'
     || lk == 63657                 // 'json-item' 'empty'
     || lk == 63686                 // 'object' 'empty'
     || lk == 63735                 // 'structured-item' 'empty'
     || lk == 65103                 // 'array' 'end'
     || lk == 65193                 // 'json-item' 'end'
     || lk == 65222                 // 'object' 'end'
     || lk == 65271                 // 'structured-item' 'end'
     || lk == 66127                 // 'array' 'eq'
     || lk == 66217                 // 'json-item' 'eq'
     || lk == 66246                 // 'object' 'eq'
     || lk == 66295                 // 'structured-item' 'eq'
     || lk == 67663                 // 'array' 'except'
     || lk == 67753                 // 'json-item' 'except'
     || lk == 67782                 // 'object' 'except'
     || lk == 67831                 // 'structured-item' 'except'
     || lk == 68687                 // 'array' 'external'
     || lk == 68777                 // 'json-item' 'external'
     || lk == 68806                 // 'object' 'external'
     || lk == 68855                 // 'structured-item' 'external'
     || lk == 71247                 // 'array' 'for'
     || lk == 71337                 // 'json-item' 'for'
     || lk == 71366                 // 'object' 'for'
     || lk == 71415                 // 'structured-item' 'for'
     || lk == 75855                 // 'array' 'ge'
     || lk == 75945                 // 'json-item' 'ge'
     || lk == 75974                 // 'object' 'ge'
     || lk == 76023                 // 'structured-item' 'ge'
     || lk == 76879                 // 'array' 'group'
     || lk == 76969                 // 'json-item' 'group'
     || lk == 76998                 // 'object' 'group'
     || lk == 77047                 // 'structured-item' 'group'
     || lk == 77903                 // 'array' 'gt'
     || lk == 77993                 // 'json-item' 'gt'
     || lk == 78022                 // 'object' 'gt'
     || lk == 78071                 // 'structured-item' 'gt'
     || lk == 78415                 // 'array' 'idiv'
     || lk == 78505                 // 'json-item' 'idiv'
     || lk == 78534                 // 'object' 'idiv'
     || lk == 78583                 // 'structured-item' 'idiv'
     || lk == 79951                 // 'array' 'in'
     || lk == 80041                 // 'json-item' 'in'
     || lk == 80070                 // 'object' 'in'
     || lk == 80119                 // 'structured-item' 'in'
     || lk == 83023                 // 'array' 'instance'
     || lk == 83113                 // 'json-item' 'instance'
     || lk == 83142                 // 'object' 'instance'
     || lk == 83191                 // 'structured-item' 'instance'
     || lk == 84047                 // 'array' 'intersect'
     || lk == 84137                 // 'json-item' 'intersect'
     || lk == 84166                 // 'object' 'intersect'
     || lk == 84215                 // 'structured-item' 'intersect'
     || lk == 84559                 // 'array' 'into'
     || lk == 84649                 // 'json-item' 'into'
     || lk == 84678                 // 'object' 'into'
     || lk == 84727                 // 'structured-item' 'into'
     || lk == 85071                 // 'array' 'is'
     || lk == 85161                 // 'json-item' 'is'
     || lk == 85190                 // 'object' 'is'
     || lk == 85239                 // 'structured-item' 'is'
     || lk == 89679                 // 'array' 'le'
     || lk == 89769                 // 'json-item' 'le'
     || lk == 89798                 // 'object' 'le'
     || lk == 89847                 // 'structured-item' 'le'
     || lk == 90703                 // 'array' 'let'
     || lk == 90793                 // 'json-item' 'let'
     || lk == 90822                 // 'object' 'let'
     || lk == 90871                 // 'structured-item' 'let'
     || lk == 92751                 // 'array' 'lt'
     || lk == 92841                 // 'json-item' 'lt'
     || lk == 92870                 // 'object' 'lt'
     || lk == 92919                 // 'structured-item' 'lt'
     || lk == 93775                 // 'array' 'mod'
     || lk == 93865                 // 'json-item' 'mod'
     || lk == 93894                 // 'object' 'mod'
     || lk == 93943                 // 'structured-item' 'mod'
     || lk == 94287                 // 'array' 'modify'
     || lk == 94377                 // 'json-item' 'modify'
     || lk == 94406                 // 'object' 'modify'
     || lk == 94455                 // 'structured-item' 'modify'
     || lk == 96847                 // 'array' 'ne'
     || lk == 96937                 // 'json-item' 'ne'
     || lk == 96966                 // 'object' 'ne'
     || lk == 97015                 // 'structured-item' 'ne'
     || lk == 103503                // 'array' 'only'
     || lk == 103593                // 'json-item' 'only'
     || lk == 103622                // 'object' 'only'
     || lk == 103671                // 'structured-item' 'only'
     || lk == 104527                // 'array' 'or'
     || lk == 104617                // 'json-item' 'or'
     || lk == 104646                // 'object' 'or'
     || lk == 104695                // 'structured-item' 'or'
     || lk == 105039                // 'array' 'order'
     || lk == 105129                // 'json-item' 'order'
     || lk == 105158                // 'object' 'order'
     || lk == 105207                // 'structured-item' 'order'
     || lk == 107087                // 'array' 'paragraphs'
     || lk == 107177                // 'json-item' 'paragraphs'
     || lk == 107206                // 'object' 'paragraphs'
     || lk == 107255                // 'structured-item' 'paragraphs'
     || lk == 114767                // 'array' 'return'
     || lk == 114857                // 'json-item' 'return'
     || lk == 114886                // 'object' 'return'
     || lk == 114935                // 'structured-item' 'return'
     || lk == 116815                // 'array' 'satisfies'
     || lk == 116905                // 'json-item' 'satisfies'
     || lk == 116934                // 'object' 'satisfies'
     || lk == 116983                // 'structured-item' 'satisfies'
     || lk == 118863                // 'array' 'score'
     || lk == 118953                // 'json-item' 'score'
     || lk == 118982                // 'object' 'score'
     || lk == 119031                // 'structured-item' 'score'
     || lk == 121423                // 'array' 'sentences'
     || lk == 121513                // 'json-item' 'sentences'
     || lk == 121542                // 'object' 'sentences'
     || lk == 121591                // 'structured-item' 'sentences'
     || lk == 123471                // 'array' 'stable'
     || lk == 123561                // 'json-item' 'stable'
     || lk == 123590                // 'object' 'stable'
     || lk == 123639                // 'structured-item' 'stable'
     || lk == 123983                // 'array' 'start'
     || lk == 124073                // 'json-item' 'start'
     || lk == 124102                // 'object' 'start'
     || lk == 124151                // 'structured-item' 'start'
     || lk == 129103                // 'array' 'times'
     || lk == 129193                // 'json-item' 'times'
     || lk == 129222                // 'object' 'times'
     || lk == 129271                // 'structured-item' 'times'
     || lk == 129615                // 'array' 'to'
     || lk == 129705                // 'json-item' 'to'
     || lk == 129734                // 'object' 'to'
     || lk == 129783                // 'structured-item' 'to'
     || lk == 133199                // 'array' 'union'
     || lk == 133289                // 'json-item' 'union'
     || lk == 133318                // 'object' 'union'
     || lk == 133367                // 'structured-item' 'union'
     || lk == 139343                // 'array' 'where'
     || lk == 139433                // 'json-item' 'where'
     || lk == 139462                // 'object' 'where'
     || lk == 139511                // 'structured-item' 'where'
     || lk == 141391                // 'array' 'with'
     || lk == 141481                // 'json-item' 'with'
     || lk == 141510                // 'object' 'with'
     || lk == 141559                // 'structured-item' 'with'
     || lk == 142927                // 'array' 'words'
     || lk == 143017                // 'json-item' 'words'
     || lk == 143046                // 'object' 'words'
     || lk == 143095                // 'structured-item' 'words'
     || lk == 143951                // 'array' '{'
     || lk == 144041                // 'json-item' '{'
     || lk == 144070                // 'object' '{'
     || lk == 144119                // 'structured-item' '{'
     || lk == 145487                // 'array' '|'
     || lk == 145577                // 'json-item' '|'
     || lk == 145606                // 'object' '|'
     || lk == 145655                // 'structured-item' '|'
     || lk == 145999                // 'array' '||'
     || lk == 146089                // 'json-item' '||'
     || lk == 146118                // 'object' '||'
     || lk == 146167                // 'structured-item' '||'
     || lk == 146511                // 'array' '|}'
     || lk == 146601                // 'json-item' '|}'
     || lk == 146630                // 'object' '|}'
     || lk == 146679                // 'structured-item' '|}'
     || lk == 147023                // 'array' '}'
     || lk == 147113                // 'json-item' '}'
     || lk == 147142                // 'object' '}'
     || lk == 147191)               // 'structured-item' '}'
    {
      lk = memoized(6, e0);
      if (lk == 0)
      {
        var b0A = b0; var e0A = e0; var l1A = l1;
        var b1A = b1; var e1A = e1; var l2A = l2;
        var b2A = b2; var e2A = e2;
        try
        {
          try_AtomicOrUnionType();
          lk = -4;
        }
        catch (p4A)
        {
          try
          {
            b0 = b0A; e0 = e0A; l1 = l1A; if (l1 == 0) {end = e0A;} else {
            b1 = b1A; e1 = e1A; l2 = l2A; if (l2 == 0) {end = e1A;} else {
            b2 = b2A; e2 = e2A; end = e2A; }}
            try_JSONTest();
            lk = -6;
          }
          catch (p6A)
          {
            lk = -7;
          }
        }
        b0 = b0A; e0 = e0A; l1 = l1A; if (l1 == 0) {end = e0A;} else {
        b1 = b1A; e1 = e1A; l2 = l2A; if (l2 == 0) {end = e1A;} else {
        b2 = b2A; e2 = e2A; end = e2A; }}
        memoize(6, e0, lk);
      }
    }
    switch (lk)
    {
    case 18003:                     // 'attribute' '('
    case 18017:                     // 'comment' '('
    case 18041:                     // 'document-node' '('
    case 18042:                     // 'element' '('
    case 18108:                     // 'namespace-node' '('
    case 18114:                     // 'node' '('
    case 18140:                     // 'processing-instruction' '('
    case 18150:                     // 'schema-attribute' '('
    case 18151:                     // 'schema-element' '('
    case 18169:                     // 'text' '('
      parse_KindTest();
      break;
    case 18087:                     // 'item' '('
      shift(167);                   // 'item'
      lookahead1W(22);              // S^WS | '(' | '(:'
      shift(35);                    // '('
      lookahead1W(23);              // S^WS | '(:' | ')'
      shift(38);                    // ')'
      break;
    case 33:                        // '%'
    case 18067:                     // 'function' '('
      parse_FunctionTest();
      break;
    case 35:                        // '('
      parse_ParenthesizedItemType();
      break;
    case -6:
    case 17999:                     // 'array' '('
    case 18089:                     // 'json-item' '('
    case 18118:                     // 'object' '('
      parse_JSONTest();
      break;
    case -7:
    case 18167:                     // 'structured-item' '('
      parse_StructuredItemTest();
      break;
    default:
      parse_AtomicOrUnionType();
    }
    eventHandler.endNonterminal("ItemType", e0);
  }

  function try_ItemType()
  {
    switch (l1)
    {
    case 79:                        // 'array'
    case 83:                        // 'attribute'
    case 97:                        // 'comment'
    case 121:                       // 'document-node'
    case 122:                       // 'element'
    case 147:                       // 'function'
    case 167:                       // 'item'
    case 169:                       // 'json-item'
    case 188:                       // 'namespace-node'
    case 194:                       // 'node'
    case 198:                       // 'object'
    case 220:                       // 'processing-instruction'
    case 230:                       // 'schema-attribute'
    case 231:                       // 'schema-element'
    case 247:                       // 'structured-item'
    case 249:                       // 'text'
      lookahead2W(233);             // S^WS | EOF | '!=' | '(' | '(:' | ')' | '*' | '*' | '+' | ',' | '-' | ':' | ':=' |
      break;
    default:
      lk = l1;
    }
    if (lk == 12879                 // 'array' EOF
     || lk == 12969                 // 'json-item' EOF
     || lk == 12998                 // 'object' EOF
     || lk == 13047                 // 'structured-item' EOF
     || lk == 13903                 // 'array' '!='
     || lk == 13993                 // 'json-item' '!='
     || lk == 14022                 // 'object' '!='
     || lk == 14071                 // 'structured-item' '!='
     || lk == 19535                 // 'array' ')'
     || lk == 19625                 // 'json-item' ')'
     || lk == 19654                 // 'object' ')'
     || lk == 19703                 // 'structured-item' ')'
     || lk == 20047                 // 'array' '*'
     || lk == 20137                 // 'json-item' '*'
     || lk == 20166                 // 'object' '*'
     || lk == 20215                 // 'structured-item' '*'
     || lk == 20559                 // 'array' '*'
     || lk == 20649                 // 'json-item' '*'
     || lk == 20678                 // 'object' '*'
     || lk == 20727                 // 'structured-item' '*'
     || lk == 21071                 // 'array' '+'
     || lk == 21161                 // 'json-item' '+'
     || lk == 21190                 // 'object' '+'
     || lk == 21239                 // 'structured-item' '+'
     || lk == 21583                 // 'array' ','
     || lk == 21673                 // 'json-item' ','
     || lk == 21702                 // 'object' ','
     || lk == 21751                 // 'structured-item' ','
     || lk == 22095                 // 'array' '-'
     || lk == 22185                 // 'json-item' '-'
     || lk == 22214                 // 'object' '-'
     || lk == 22263                 // 'structured-item' '-'
     || lk == 25679                 // 'array' ':'
     || lk == 25769                 // 'json-item' ':'
     || lk == 25798                 // 'object' ':'
     || lk == 25847                 // 'structured-item' ':'
     || lk == 27215                 // 'array' ':='
     || lk == 27305                 // 'json-item' ':='
     || lk == 27334                 // 'object' ':='
     || lk == 27383                 // 'structured-item' ':='
     || lk == 27727                 // 'array' ';'
     || lk == 27817                 // 'json-item' ';'
     || lk == 27846                 // 'object' ';'
     || lk == 27895                 // 'structured-item' ';'
     || lk == 28239                 // 'array' '<'
     || lk == 28329                 // 'json-item' '<'
     || lk == 28358                 // 'object' '<'
     || lk == 28407                 // 'structured-item' '<'
     || lk == 29775                 // 'array' '<<'
     || lk == 29865                 // 'json-item' '<<'
     || lk == 29894                 // 'object' '<<'
     || lk == 29943                 // 'structured-item' '<<'
     || lk == 30287                 // 'array' '<='
     || lk == 30377                 // 'json-item' '<='
     || lk == 30406                 // 'object' '<='
     || lk == 30455                 // 'structured-item' '<='
     || lk == 31311                 // 'array' '='
     || lk == 31401                 // 'json-item' '='
     || lk == 31430                 // 'object' '='
     || lk == 31479                 // 'structured-item' '='
     || lk == 31823                 // 'array' '>'
     || lk == 31913                 // 'json-item' '>'
     || lk == 31942                 // 'object' '>'
     || lk == 31991                 // 'structured-item' '>'
     || lk == 32335                 // 'array' '>='
     || lk == 32425                 // 'json-item' '>='
     || lk == 32454                 // 'object' '>='
     || lk == 32503                 // 'structured-item' '>='
     || lk == 32847                 // 'array' '>>'
     || lk == 32937                 // 'json-item' '>>'
     || lk == 32966                 // 'object' '>>'
     || lk == 33015                 // 'structured-item' '>>'
     || lk == 33359                 // 'array' '?'
     || lk == 33449                 // 'json-item' '?'
     || lk == 33478                 // 'object' '?'
     || lk == 33527                 // 'structured-item' '?'
     || lk == 35919                 // 'array' ']'
     || lk == 36009                 // 'json-item' ']'
     || lk == 36038                 // 'object' ']'
     || lk == 36087                 // 'structured-item' ']'
     || lk == 36431                 // 'array' 'after'
     || lk == 36521                 // 'json-item' 'after'
     || lk == 36550                 // 'object' 'after'
     || lk == 36599                 // 'structured-item' 'after'
     || lk == 37455                 // 'array' 'allowing'
     || lk == 37545                 // 'json-item' 'allowing'
     || lk == 37574                 // 'object' 'allowing'
     || lk == 37623                 // 'structured-item' 'allowing'
     || lk == 38991                 // 'array' 'and'
     || lk == 39081                 // 'json-item' 'and'
     || lk == 39110                 // 'object' 'and'
     || lk == 39159                 // 'structured-item' 'and'
     || lk == 41039                 // 'array' 'as'
     || lk == 41129                 // 'json-item' 'as'
     || lk == 41158                 // 'object' 'as'
     || lk == 41207                 // 'structured-item' 'as'
     || lk == 41551                 // 'array' 'ascending'
     || lk == 41641                 // 'json-item' 'ascending'
     || lk == 41670                 // 'object' 'ascending'
     || lk == 41719                 // 'structured-item' 'ascending'
     || lk == 42063                 // 'array' 'at'
     || lk == 42153                 // 'json-item' 'at'
     || lk == 42182                 // 'object' 'at'
     || lk == 42231                 // 'structured-item' 'at'
     || lk == 43599                 // 'array' 'before'
     || lk == 43689                 // 'json-item' 'before'
     || lk == 43718                 // 'object' 'before'
     || lk == 43767                 // 'structured-item' 'before'
     || lk == 45647                 // 'array' 'case'
     || lk == 45737                 // 'json-item' 'case'
     || lk == 45766                 // 'object' 'case'
     || lk == 45815                 // 'structured-item' 'case'
     || lk == 48719                 // 'array' 'collation'
     || lk == 48809                 // 'json-item' 'collation'
     || lk == 48838                 // 'object' 'collation'
     || lk == 48887                 // 'structured-item' 'collation'
     || lk == 51279                 // 'array' 'contains'
     || lk == 51369                 // 'json-item' 'contains'
     || lk == 51398                 // 'object' 'contains'
     || lk == 51447                 // 'structured-item' 'contains'
     || lk == 54351                 // 'array' 'count'
     || lk == 54441                 // 'json-item' 'count'
     || lk == 54470                 // 'object' 'count'
     || lk == 54519                 // 'structured-item' 'count'
     || lk == 56399                 // 'array' 'default'
     || lk == 56489                 // 'json-item' 'default'
     || lk == 56518                 // 'object' 'default'
     || lk == 56567                 // 'structured-item' 'default'
     || lk == 58447                 // 'array' 'descending'
     || lk == 58537                 // 'json-item' 'descending'
     || lk == 58566                 // 'object' 'descending'
     || lk == 58615                 // 'structured-item' 'descending'
     || lk == 61007                 // 'array' 'div'
     || lk == 61097                 // 'json-item' 'div'
     || lk == 61126                 // 'object' 'div'
     || lk == 61175                 // 'structured-item' 'div'
     || lk == 63055                 // 'array' 'else'
     || lk == 63145                 // 'json-item' 'else'
     || lk == 63174                 // 'object' 'else'
     || lk == 63223                 // 'structured-item' 'else'
     || lk == 63567                 // 'array' 'empty'
     || lk == 63657                 // 'json-item' 'empty'
     || lk == 63686                 // 'object' 'empty'
     || lk == 63735                 // 'structured-item' 'empty'
     || lk == 65103                 // 'array' 'end'
     || lk == 65193                 // 'json-item' 'end'
     || lk == 65222                 // 'object' 'end'
     || lk == 65271                 // 'structured-item' 'end'
     || lk == 66127                 // 'array' 'eq'
     || lk == 66217                 // 'json-item' 'eq'
     || lk == 66246                 // 'object' 'eq'
     || lk == 66295                 // 'structured-item' 'eq'
     || lk == 67663                 // 'array' 'except'
     || lk == 67753                 // 'json-item' 'except'
     || lk == 67782                 // 'object' 'except'
     || lk == 67831                 // 'structured-item' 'except'
     || lk == 68687                 // 'array' 'external'
     || lk == 68777                 // 'json-item' 'external'
     || lk == 68806                 // 'object' 'external'
     || lk == 68855                 // 'structured-item' 'external'
     || lk == 71247                 // 'array' 'for'
     || lk == 71337                 // 'json-item' 'for'
     || lk == 71366                 // 'object' 'for'
     || lk == 71415                 // 'structured-item' 'for'
     || lk == 75855                 // 'array' 'ge'
     || lk == 75945                 // 'json-item' 'ge'
     || lk == 75974                 // 'object' 'ge'
     || lk == 76023                 // 'structured-item' 'ge'
     || lk == 76879                 // 'array' 'group'
     || lk == 76969                 // 'json-item' 'group'
     || lk == 76998                 // 'object' 'group'
     || lk == 77047                 // 'structured-item' 'group'
     || lk == 77903                 // 'array' 'gt'
     || lk == 77993                 // 'json-item' 'gt'
     || lk == 78022                 // 'object' 'gt'
     || lk == 78071                 // 'structured-item' 'gt'
     || lk == 78415                 // 'array' 'idiv'
     || lk == 78505                 // 'json-item' 'idiv'
     || lk == 78534                 // 'object' 'idiv'
     || lk == 78583                 // 'structured-item' 'idiv'
     || lk == 79951                 // 'array' 'in'
     || lk == 80041                 // 'json-item' 'in'
     || lk == 80070                 // 'object' 'in'
     || lk == 80119                 // 'structured-item' 'in'
     || lk == 83023                 // 'array' 'instance'
     || lk == 83113                 // 'json-item' 'instance'
     || lk == 83142                 // 'object' 'instance'
     || lk == 83191                 // 'structured-item' 'instance'
     || lk == 84047                 // 'array' 'intersect'
     || lk == 84137                 // 'json-item' 'intersect'
     || lk == 84166                 // 'object' 'intersect'
     || lk == 84215                 // 'structured-item' 'intersect'
     || lk == 84559                 // 'array' 'into'
     || lk == 84649                 // 'json-item' 'into'
     || lk == 84678                 // 'object' 'into'
     || lk == 84727                 // 'structured-item' 'into'
     || lk == 85071                 // 'array' 'is'
     || lk == 85161                 // 'json-item' 'is'
     || lk == 85190                 // 'object' 'is'
     || lk == 85239                 // 'structured-item' 'is'
     || lk == 89679                 // 'array' 'le'
     || lk == 89769                 // 'json-item' 'le'
     || lk == 89798                 // 'object' 'le'
     || lk == 89847                 // 'structured-item' 'le'
     || lk == 90703                 // 'array' 'let'
     || lk == 90793                 // 'json-item' 'let'
     || lk == 90822                 // 'object' 'let'
     || lk == 90871                 // 'structured-item' 'let'
     || lk == 92751                 // 'array' 'lt'
     || lk == 92841                 // 'json-item' 'lt'
     || lk == 92870                 // 'object' 'lt'
     || lk == 92919                 // 'structured-item' 'lt'
     || lk == 93775                 // 'array' 'mod'
     || lk == 93865                 // 'json-item' 'mod'
     || lk == 93894                 // 'object' 'mod'
     || lk == 93943                 // 'structured-item' 'mod'
     || lk == 94287                 // 'array' 'modify'
     || lk == 94377                 // 'json-item' 'modify'
     || lk == 94406                 // 'object' 'modify'
     || lk == 94455                 // 'structured-item' 'modify'
     || lk == 96847                 // 'array' 'ne'
     || lk == 96937                 // 'json-item' 'ne'
     || lk == 96966                 // 'object' 'ne'
     || lk == 97015                 // 'structured-item' 'ne'
     || lk == 103503                // 'array' 'only'
     || lk == 103593                // 'json-item' 'only'
     || lk == 103622                // 'object' 'only'
     || lk == 103671                // 'structured-item' 'only'
     || lk == 104527                // 'array' 'or'
     || lk == 104617                // 'json-item' 'or'
     || lk == 104646                // 'object' 'or'
     || lk == 104695                // 'structured-item' 'or'
     || lk == 105039                // 'array' 'order'
     || lk == 105129                // 'json-item' 'order'
     || lk == 105158                // 'object' 'order'
     || lk == 105207                // 'structured-item' 'order'
     || lk == 107087                // 'array' 'paragraphs'
     || lk == 107177                // 'json-item' 'paragraphs'
     || lk == 107206                // 'object' 'paragraphs'
     || lk == 107255                // 'structured-item' 'paragraphs'
     || lk == 114767                // 'array' 'return'
     || lk == 114857                // 'json-item' 'return'
     || lk == 114886                // 'object' 'return'
     || lk == 114935                // 'structured-item' 'return'
     || lk == 116815                // 'array' 'satisfies'
     || lk == 116905                // 'json-item' 'satisfies'
     || lk == 116934                // 'object' 'satisfies'
     || lk == 116983                // 'structured-item' 'satisfies'
     || lk == 118863                // 'array' 'score'
     || lk == 118953                // 'json-item' 'score'
     || lk == 118982                // 'object' 'score'
     || lk == 119031                // 'structured-item' 'score'
     || lk == 121423                // 'array' 'sentences'
     || lk == 121513                // 'json-item' 'sentences'
     || lk == 121542                // 'object' 'sentences'
     || lk == 121591                // 'structured-item' 'sentences'
     || lk == 123471                // 'array' 'stable'
     || lk == 123561                // 'json-item' 'stable'
     || lk == 123590                // 'object' 'stable'
     || lk == 123639                // 'structured-item' 'stable'
     || lk == 123983                // 'array' 'start'
     || lk == 124073                // 'json-item' 'start'
     || lk == 124102                // 'object' 'start'
     || lk == 124151                // 'structured-item' 'start'
     || lk == 129103                // 'array' 'times'
     || lk == 129193                // 'json-item' 'times'
     || lk == 129222                // 'object' 'times'
     || lk == 129271                // 'structured-item' 'times'
     || lk == 129615                // 'array' 'to'
     || lk == 129705                // 'json-item' 'to'
     || lk == 129734                // 'object' 'to'
     || lk == 129783                // 'structured-item' 'to'
     || lk == 133199                // 'array' 'union'
     || lk == 133289                // 'json-item' 'union'
     || lk == 133318                // 'object' 'union'
     || lk == 133367                // 'structured-item' 'union'
     || lk == 139343                // 'array' 'where'
     || lk == 139433                // 'json-item' 'where'
     || lk == 139462                // 'object' 'where'
     || lk == 139511                // 'structured-item' 'where'
     || lk == 141391                // 'array' 'with'
     || lk == 141481                // 'json-item' 'with'
     || lk == 141510                // 'object' 'with'
     || lk == 141559                // 'structured-item' 'with'
     || lk == 142927                // 'array' 'words'
     || lk == 143017                // 'json-item' 'words'
     || lk == 143046                // 'object' 'words'
     || lk == 143095                // 'structured-item' 'words'
     || lk == 143951                // 'array' '{'
     || lk == 144041                // 'json-item' '{'
     || lk == 144070                // 'object' '{'
     || lk == 144119                // 'structured-item' '{'
     || lk == 145487                // 'array' '|'
     || lk == 145577                // 'json-item' '|'
     || lk == 145606                // 'object' '|'
     || lk == 145655                // 'structured-item' '|'
     || lk == 145999                // 'array' '||'
     || lk == 146089                // 'json-item' '||'
     || lk == 146118                // 'object' '||'
     || lk == 146167                // 'structured-item' '||'
     || lk == 146511                // 'array' '|}'
     || lk == 146601                // 'json-item' '|}'
     || lk == 146630                // 'object' '|}'
     || lk == 146679                // 'structured-item' '|}'
     || lk == 147023                // 'array' '}'
     || lk == 147113                // 'json-item' '}'
     || lk == 147142                // 'object' '}'
     || lk == 147191)               // 'structured-item' '}'
    {
      lk = memoized(6, e0);
      if (lk == 0)
      {
        var b0A = b0; var e0A = e0; var l1A = l1;
        var b1A = b1; var e1A = e1; var l2A = l2;
        var b2A = b2; var e2A = e2;
        try
        {
          try_AtomicOrUnionType();
          memoize(6, e0A, -4);
          lk = -8;
        }
        catch (p4A)
        {
          try
          {
            b0 = b0A; e0 = e0A; l1 = l1A; if (l1 == 0) {end = e0A;} else {
            b1 = b1A; e1 = e1A; l2 = l2A; if (l2 == 0) {end = e1A;} else {
            b2 = b2A; e2 = e2A; end = e2A; }}
            try_JSONTest();
            memoize(6, e0A, -6);
            lk = -8;
          }
          catch (p6A)
          {
            lk = -7;
            b0 = b0A; e0 = e0A; l1 = l1A; if (l1 == 0) {end = e0A;} else {
            b1 = b1A; e1 = e1A; l2 = l2A; if (l2 == 0) {end = e1A;} else {
            b2 = b2A; e2 = e2A; end = e2A; }}
            memoize(6, e0A, -7);
          }
        }
      }
    }
    switch (lk)
    {
    case 18003:                     // 'attribute' '('
    case 18017:                     // 'comment' '('
    case 18041:                     // 'document-node' '('
    case 18042:                     // 'element' '('
    case 18108:                     // 'namespace-node' '('
    case 18114:                     // 'node' '('
    case 18140:                     // 'processing-instruction' '('
    case 18150:                     // 'schema-attribute' '('
    case 18151:                     // 'schema-element' '('
    case 18169:                     // 'text' '('
      try_KindTest();
      break;
    case 18087:                     // 'item' '('
      shiftT(167);                  // 'item'
      lookahead1W(22);              // S^WS | '(' | '(:'
      shiftT(35);                   // '('
      lookahead1W(23);              // S^WS | '(:' | ')'
      shiftT(38);                   // ')'
      break;
    case 33:                        // '%'
    case 18067:                     // 'function' '('
      try_FunctionTest();
      break;
    case 35:                        // '('
      try_ParenthesizedItemType();
      break;
    case -6:
    case 17999:                     // 'array' '('
    case 18089:                     // 'json-item' '('
    case 18118:                     // 'object' '('
      try_JSONTest();
      break;
    case -7:
    case 18167:                     // 'structured-item' '('
      try_StructuredItemTest();
      break;
    case -8:
      break;
    default:
      try_AtomicOrUnionType();
    }
  }

  function parse_JSONTest()
  {
    eventHandler.startNonterminal("JSONTest", e0);
    switch (l1)
    {
    case 169:                       // 'json-item'
      parse_JSONItemTest();
      break;
    case 198:                       // 'object'
      parse_JSONObjectTest();
      break;
    default:
      parse_JSONArrayTest();
    }
    eventHandler.endNonterminal("JSONTest", e0);
  }

  function try_JSONTest()
  {
    switch (l1)
    {
    case 169:                       // 'json-item'
      try_JSONItemTest();
      break;
    case 198:                       // 'object'
      try_JSONObjectTest();
      break;
    default:
      try_JSONArrayTest();
    }
  }

  function parse_StructuredItemTest()
  {
    eventHandler.startNonterminal("StructuredItemTest", e0);
    shift(247);                     // 'structured-item'
    lookahead1W(233);               // S^WS | EOF | '!=' | '(' | '(:' | ')' | '*' | '*' | '+' | ',' | '-' | ':' | ':=' |
    if (l1 == 35)                   // '('
    {
      shift(35);                    // '('
      lookahead1W(23);              // S^WS | '(:' | ')'
      shift(38);                    // ')'
    }
    eventHandler.endNonterminal("StructuredItemTest", e0);
  }

  function try_StructuredItemTest()
  {
    shiftT(247);                    // 'structured-item'
    lookahead1W(233);               // S^WS | EOF | '!=' | '(' | '(:' | ')' | '*' | '*' | '+' | ',' | '-' | ':' | ':=' |
    if (l1 == 35)                   // '('
    {
      shiftT(35);                   // '('
      lookahead1W(23);              // S^WS | '(:' | ')'
      shiftT(38);                   // ')'
    }
  }

  function parse_JSONItemTest()
  {
    eventHandler.startNonterminal("JSONItemTest", e0);
    shift(169);                     // 'json-item'
    lookahead1W(233);               // S^WS | EOF | '!=' | '(' | '(:' | ')' | '*' | '*' | '+' | ',' | '-' | ':' | ':=' |
    if (l1 == 35)                   // '('
    {
      shift(35);                    // '('
      lookahead1W(23);              // S^WS | '(:' | ')'
      shift(38);                    // ')'
    }
    eventHandler.endNonterminal("JSONItemTest", e0);
  }

  function try_JSONItemTest()
  {
    shiftT(169);                    // 'json-item'
    lookahead1W(233);               // S^WS | EOF | '!=' | '(' | '(:' | ')' | '*' | '*' | '+' | ',' | '-' | ':' | ':=' |
    if (l1 == 35)                   // '('
    {
      shiftT(35);                   // '('
      lookahead1W(23);              // S^WS | '(:' | ')'
      shiftT(38);                   // ')'
    }
  }

  function parse_JSONObjectTest()
  {
    eventHandler.startNonterminal("JSONObjectTest", e0);
    shift(198);                     // 'object'
    lookahead1W(233);               // S^WS | EOF | '!=' | '(' | '(:' | ')' | '*' | '*' | '+' | ',' | '-' | ':' | ':=' |
    if (l1 == 35)                   // '('
    {
      shift(35);                    // '('
      lookahead1W(23);              // S^WS | '(:' | ')'
      shift(38);                    // ')'
    }
    eventHandler.endNonterminal("JSONObjectTest", e0);
  }

  function try_JSONObjectTest()
  {
    shiftT(198);                    // 'object'
    lookahead1W(233);               // S^WS | EOF | '!=' | '(' | '(:' | ')' | '*' | '*' | '+' | ',' | '-' | ':' | ':=' |
    if (l1 == 35)                   // '('
    {
      shiftT(35);                   // '('
      lookahead1W(23);              // S^WS | '(:' | ')'
      shiftT(38);                   // ')'
    }
  }

  function parse_JSONArrayTest()
  {
    eventHandler.startNonterminal("JSONArrayTest", e0);
    shift(79);                      // 'array'
    lookahead1W(233);               // S^WS | EOF | '!=' | '(' | '(:' | ')' | '*' | '*' | '+' | ',' | '-' | ':' | ':=' |
    if (l1 == 35)                   // '('
    {
      shift(35);                    // '('
      lookahead1W(23);              // S^WS | '(:' | ')'
      shift(38);                    // ')'
    }
    eventHandler.endNonterminal("JSONArrayTest", e0);
  }

  function try_JSONArrayTest()
  {
    shiftT(79);                     // 'array'
    lookahead1W(233);               // S^WS | EOF | '!=' | '(' | '(:' | ')' | '*' | '*' | '+' | ',' | '-' | ':' | ':=' |
    if (l1 == 35)                   // '('
    {
      shiftT(35);                   // '('
      lookahead1W(23);              // S^WS | '(:' | ')'
      shiftT(38);                   // ')'
    }
  }

  function parse_AtomicOrUnionType()
  {
    eventHandler.startNonterminal("AtomicOrUnionType", e0);
    parse_EQName();
    eventHandler.endNonterminal("AtomicOrUnionType", e0);
  }

  function try_AtomicOrUnionType()
  {
    try_EQName();
  }

  function parse_KindTest()
  {
    eventHandler.startNonterminal("KindTest", e0);
    switch (l1)
    {
    case 121:                       // 'document-node'
      parse_DocumentTest();
      break;
    case 122:                       // 'element'
      parse_ElementTest();
      break;
    case 83:                        // 'attribute'
      parse_AttributeTest();
      break;
    case 231:                       // 'schema-element'
      parse_SchemaElementTest();
      break;
    case 230:                       // 'schema-attribute'
      parse_SchemaAttributeTest();
      break;
    case 220:                       // 'processing-instruction'
      parse_PITest();
      break;
    case 97:                        // 'comment'
      parse_CommentTest();
      break;
    case 249:                       // 'text'
      parse_TextTest();
      break;
    case 188:                       // 'namespace-node'
      parse_NamespaceNodeTest();
      break;
    default:
      parse_AnyKindTest();
    }
    eventHandler.endNonterminal("KindTest", e0);
  }

  function try_KindTest()
  {
    switch (l1)
    {
    case 121:                       // 'document-node'
      try_DocumentTest();
      break;
    case 122:                       // 'element'
      try_ElementTest();
      break;
    case 83:                        // 'attribute'
      try_AttributeTest();
      break;
    case 231:                       // 'schema-element'
      try_SchemaElementTest();
      break;
    case 230:                       // 'schema-attribute'
      try_SchemaAttributeTest();
      break;
    case 220:                       // 'processing-instruction'
      try_PITest();
      break;
    case 97:                        // 'comment'
      try_CommentTest();
      break;
    case 249:                       // 'text'
      try_TextTest();
      break;
    case 188:                       // 'namespace-node'
      try_NamespaceNodeTest();
      break;
    default:
      try_AnyKindTest();
    }
  }

  function parse_AnyKindTest()
  {
    eventHandler.startNonterminal("AnyKindTest", e0);
    shift(194);                     // 'node'
    lookahead1W(22);                // S^WS | '(' | '(:'
    shift(35);                      // '('
    lookahead1W(23);                // S^WS | '(:' | ')'
    shift(38);                      // ')'
    eventHandler.endNonterminal("AnyKindTest", e0);
  }

  function try_AnyKindTest()
  {
    shiftT(194);                    // 'node'
    lookahead1W(22);                // S^WS | '(' | '(:'
    shiftT(35);                     // '('
    lookahead1W(23);                // S^WS | '(:' | ')'
    shiftT(38);                     // ')'
  }

  function parse_DocumentTest()
  {
    eventHandler.startNonterminal("DocumentTest", e0);
    shift(121);                     // 'document-node'
    lookahead1W(22);                // S^WS | '(' | '(:'
    shift(35);                      // '('
    lookahead1W(154);               // S^WS | '(:' | ')' | 'element' | 'schema-element'
    if (l1 != 38)                   // ')'
    {
      switch (l1)
      {
      case 122:                     // 'element'
        whitespace();
        parse_ElementTest();
        break;
      default:
        whitespace();
        parse_SchemaElementTest();
      }
    }
    lookahead1W(23);                // S^WS | '(:' | ')'
    shift(38);                      // ')'
    eventHandler.endNonterminal("DocumentTest", e0);
  }

  function try_DocumentTest()
  {
    shiftT(121);                    // 'document-node'
    lookahead1W(22);                // S^WS | '(' | '(:'
    shiftT(35);                     // '('
    lookahead1W(154);               // S^WS | '(:' | ')' | 'element' | 'schema-element'
    if (l1 != 38)                   // ')'
    {
      switch (l1)
      {
      case 122:                     // 'element'
        try_ElementTest();
        break;
      default:
        try_SchemaElementTest();
      }
    }
    lookahead1W(23);                // S^WS | '(:' | ')'
    shiftT(38);                     // ')'
  }

  function parse_TextTest()
  {
    eventHandler.startNonterminal("TextTest", e0);
    shift(249);                     // 'text'
    lookahead1W(22);                // S^WS | '(' | '(:'
    shift(35);                      // '('
    lookahead1W(23);                // S^WS | '(:' | ')'
    shift(38);                      // ')'
    eventHandler.endNonterminal("TextTest", e0);
  }

  function try_TextTest()
  {
    shiftT(249);                    // 'text'
    lookahead1W(22);                // S^WS | '(' | '(:'
    shiftT(35);                     // '('
    lookahead1W(23);                // S^WS | '(:' | ')'
    shiftT(38);                     // ')'
  }

  function parse_CommentTest()
  {
    eventHandler.startNonterminal("CommentTest", e0);
    shift(97);                      // 'comment'
    lookahead1W(22);                // S^WS | '(' | '(:'
    shift(35);                      // '('
    lookahead1W(23);                // S^WS | '(:' | ')'
    shift(38);                      // ')'
    eventHandler.endNonterminal("CommentTest", e0);
  }

  function try_CommentTest()
  {
    shiftT(97);                     // 'comment'
    lookahead1W(22);                // S^WS | '(' | '(:'
    shiftT(35);                     // '('
    lookahead1W(23);                // S^WS | '(:' | ')'
    shiftT(38);                     // ')'
  }

  function parse_NamespaceNodeTest()
  {
    eventHandler.startNonterminal("NamespaceNodeTest", e0);
    shift(188);                     // 'namespace-node'
    lookahead1W(22);                // S^WS | '(' | '(:'
    shift(35);                      // '('
    lookahead1W(23);                // S^WS | '(:' | ')'
    shift(38);                      // ')'
    eventHandler.endNonterminal("NamespaceNodeTest", e0);
  }

  function try_NamespaceNodeTest()
  {
    shiftT(188);                    // 'namespace-node'
    lookahead1W(22);                // S^WS | '(' | '(:'
    shiftT(35);                     // '('
    lookahead1W(23);                // S^WS | '(:' | ')'
    shiftT(38);                     // ')'
  }

  function parse_PITest()
  {
    eventHandler.startNonterminal("PITest", e0);
    shift(220);                     // 'processing-instruction'
    lookahead1W(22);                // S^WS | '(' | '(:'
    shift(35);                      // '('
    lookahead1W(244);               // StringLiteral | NCName^Token | S^WS | '(:' | ')' | 'after' | 'allowing' |
    if (l1 != 38)                   // ')'
    {
      switch (l1)
      {
      case 11:                      // StringLiteral
        shift(11);                  // StringLiteral
        break;
      default:
        whitespace();
        parse_NCName();
      }
    }
    lookahead1W(23);                // S^WS | '(:' | ')'
    shift(38);                      // ')'
    eventHandler.endNonterminal("PITest", e0);
  }

  function try_PITest()
  {
    shiftT(220);                    // 'processing-instruction'
    lookahead1W(22);                // S^WS | '(' | '(:'
    shiftT(35);                     // '('
    lookahead1W(244);               // StringLiteral | NCName^Token | S^WS | '(:' | ')' | 'after' | 'allowing' |
    if (l1 != 38)                   // ')'
    {
      switch (l1)
      {
      case 11:                      // StringLiteral
        shiftT(11);                 // StringLiteral
        break;
      default:
        try_NCName();
      }
    }
    lookahead1W(23);                // S^WS | '(:' | ')'
    shiftT(38);                     // ')'
  }

  function parse_AttributeTest()
  {
    eventHandler.startNonterminal("AttributeTest", e0);
    shift(83);                      // 'attribute'
    lookahead1W(22);                // S^WS | '(' | '(:'
    shift(35);                      // '('
    lookahead1W(255);               // EQName^Token | S^WS | '(:' | ')' | '*' | 'after' | 'allowing' | 'ancestor' |
    if (l1 != 38)                   // ')'
    {
      whitespace();
      parse_AttribNameOrWildcard();
      lookahead1W(105);             // S^WS | '(:' | ')' | ','
      if (l1 == 42)                 // ','
      {
        shift(42);                  // ','
        lookahead1W(246);           // EQName^Token | S^WS | '(:' | 'after' | 'allowing' | 'ancestor' |
        whitespace();
        parse_TypeName();
      }
    }
    lookahead1W(23);                // S^WS | '(:' | ')'
    shift(38);                      // ')'
    eventHandler.endNonterminal("AttributeTest", e0);
  }

  function try_AttributeTest()
  {
    shiftT(83);                     // 'attribute'
    lookahead1W(22);                // S^WS | '(' | '(:'
    shiftT(35);                     // '('
    lookahead1W(255);               // EQName^Token | S^WS | '(:' | ')' | '*' | 'after' | 'allowing' | 'ancestor' |
    if (l1 != 38)                   // ')'
    {
      try_AttribNameOrWildcard();
      lookahead1W(105);             // S^WS | '(:' | ')' | ','
      if (l1 == 42)                 // ','
      {
        shiftT(42);                 // ','
        lookahead1W(246);           // EQName^Token | S^WS | '(:' | 'after' | 'allowing' | 'ancestor' |
        try_TypeName();
      }
    }
    lookahead1W(23);                // S^WS | '(:' | ')'
    shiftT(38);                     // ')'
  }

  function parse_AttribNameOrWildcard()
  {
    eventHandler.startNonterminal("AttribNameOrWildcard", e0);
    switch (l1)
    {
    case 39:                        // '*'
      shift(39);                    // '*'
      break;
    default:
      parse_AttributeName();
    }
    eventHandler.endNonterminal("AttribNameOrWildcard", e0);
  }

  function try_AttribNameOrWildcard()
  {
    switch (l1)
    {
    case 39:                        // '*'
      shiftT(39);                   // '*'
      break;
    default:
      try_AttributeName();
    }
  }

  function parse_SchemaAttributeTest()
  {
    eventHandler.startNonterminal("SchemaAttributeTest", e0);
    shift(230);                     // 'schema-attribute'
    lookahead1W(22);                // S^WS | '(' | '(:'
    shift(35);                      // '('
    lookahead1W(246);               // EQName^Token | S^WS | '(:' | 'after' | 'allowing' | 'ancestor' |
    whitespace();
    parse_AttributeDeclaration();
    lookahead1W(23);                // S^WS | '(:' | ')'
    shift(38);                      // ')'
    eventHandler.endNonterminal("SchemaAttributeTest", e0);
  }

  function try_SchemaAttributeTest()
  {
    shiftT(230);                    // 'schema-attribute'
    lookahead1W(22);                // S^WS | '(' | '(:'
    shiftT(35);                     // '('
    lookahead1W(246);               // EQName^Token | S^WS | '(:' | 'after' | 'allowing' | 'ancestor' |
    try_AttributeDeclaration();
    lookahead1W(23);                // S^WS | '(:' | ')'
    shiftT(38);                     // ')'
  }

  function parse_AttributeDeclaration()
  {
    eventHandler.startNonterminal("AttributeDeclaration", e0);
    parse_AttributeName();
    eventHandler.endNonterminal("AttributeDeclaration", e0);
  }

  function try_AttributeDeclaration()
  {
    try_AttributeName();
  }

  function parse_ElementTest()
  {
    eventHandler.startNonterminal("ElementTest", e0);
    shift(122);                     // 'element'
    lookahead1W(22);                // S^WS | '(' | '(:'
    shift(35);                      // '('
    lookahead1W(255);               // EQName^Token | S^WS | '(:' | ')' | '*' | 'after' | 'allowing' | 'ancestor' |
    if (l1 != 38)                   // ')'
    {
      whitespace();
      parse_ElementNameOrWildcard();
      lookahead1W(105);             // S^WS | '(:' | ')' | ','
      if (l1 == 42)                 // ','
      {
        shift(42);                  // ','
        lookahead1W(246);           // EQName^Token | S^WS | '(:' | 'after' | 'allowing' | 'ancestor' |
        whitespace();
        parse_TypeName();
        lookahead1W(106);           // S^WS | '(:' | ')' | '?'
        if (l1 == 65)               // '?'
        {
          shift(65);                // '?'
        }
      }
    }
    lookahead1W(23);                // S^WS | '(:' | ')'
    shift(38);                      // ')'
    eventHandler.endNonterminal("ElementTest", e0);
  }

  function try_ElementTest()
  {
    shiftT(122);                    // 'element'
    lookahead1W(22);                // S^WS | '(' | '(:'
    shiftT(35);                     // '('
    lookahead1W(255);               // EQName^Token | S^WS | '(:' | ')' | '*' | 'after' | 'allowing' | 'ancestor' |
    if (l1 != 38)                   // ')'
    {
      try_ElementNameOrWildcard();
      lookahead1W(105);             // S^WS | '(:' | ')' | ','
      if (l1 == 42)                 // ','
      {
        shiftT(42);                 // ','
        lookahead1W(246);           // EQName^Token | S^WS | '(:' | 'after' | 'allowing' | 'ancestor' |
        try_TypeName();
        lookahead1W(106);           // S^WS | '(:' | ')' | '?'
        if (l1 == 65)               // '?'
        {
          shiftT(65);               // '?'
        }
      }
    }
    lookahead1W(23);                // S^WS | '(:' | ')'
    shiftT(38);                     // ')'
  }

  function parse_ElementNameOrWildcard()
  {
    eventHandler.startNonterminal("ElementNameOrWildcard", e0);
    switch (l1)
    {
    case 39:                        // '*'
      shift(39);                    // '*'
      break;
    default:
      parse_ElementName();
    }
    eventHandler.endNonterminal("ElementNameOrWildcard", e0);
  }

  function try_ElementNameOrWildcard()
  {
    switch (l1)
    {
    case 39:                        // '*'
      shiftT(39);                   // '*'
      break;
    default:
      try_ElementName();
    }
  }

  function parse_SchemaElementTest()
  {
    eventHandler.startNonterminal("SchemaElementTest", e0);
    shift(231);                     // 'schema-element'
    lookahead1W(22);                // S^WS | '(' | '(:'
    shift(35);                      // '('
    lookahead1W(246);               // EQName^Token | S^WS | '(:' | 'after' | 'allowing' | 'ancestor' |
    whitespace();
    parse_ElementDeclaration();
    lookahead1W(23);                // S^WS | '(:' | ')'
    shift(38);                      // ')'
    eventHandler.endNonterminal("SchemaElementTest", e0);
  }

  function try_SchemaElementTest()
  {
    shiftT(231);                    // 'schema-element'
    lookahead1W(22);                // S^WS | '(' | '(:'
    shiftT(35);                     // '('
    lookahead1W(246);               // EQName^Token | S^WS | '(:' | 'after' | 'allowing' | 'ancestor' |
    try_ElementDeclaration();
    lookahead1W(23);                // S^WS | '(:' | ')'
    shiftT(38);                     // ')'
  }

  function parse_ElementDeclaration()
  {
    eventHandler.startNonterminal("ElementDeclaration", e0);
    parse_ElementName();
    eventHandler.endNonterminal("ElementDeclaration", e0);
  }

  function try_ElementDeclaration()
  {
    try_ElementName();
  }

  function parse_AttributeName()
  {
    eventHandler.startNonterminal("AttributeName", e0);
    parse_EQName();
    eventHandler.endNonterminal("AttributeName", e0);
  }

  function try_AttributeName()
  {
    try_EQName();
  }

  function parse_ElementName()
  {
    eventHandler.startNonterminal("ElementName", e0);
    parse_EQName();
    eventHandler.endNonterminal("ElementName", e0);
  }

  function try_ElementName()
  {
    try_EQName();
  }

  function parse_SimpleTypeName()
  {
    eventHandler.startNonterminal("SimpleTypeName", e0);
    parse_TypeName();
    eventHandler.endNonterminal("SimpleTypeName", e0);
  }

  function try_SimpleTypeName()
  {
    try_TypeName();
  }

  function parse_TypeName()
  {
    eventHandler.startNonterminal("TypeName", e0);
    parse_EQName();
    eventHandler.endNonterminal("TypeName", e0);
  }

  function try_TypeName()
  {
    try_EQName();
  }

  function parse_FunctionTest()
  {
    eventHandler.startNonterminal("FunctionTest", e0);
    for (;;)
    {
      lookahead1W(101);             // S^WS | '%' | '(:' | 'function'
      if (l1 != 33)                 // '%'
      {
        break;
      }
      whitespace();
      parse_Annotation();
    }
    switch (l1)
    {
    case 147:                       // 'function'
      lookahead2W(22);              // S^WS | '(' | '(:'
      break;
    default:
      lk = l1;
    }
    lk = memoized(7, e0);
    if (lk == 0)
    {
      var b0A = b0; var e0A = e0; var l1A = l1;
      var b1A = b1; var e1A = e1; var l2A = l2;
      var b2A = b2; var e2A = e2;
      try
      {
        try_AnyFunctionTest();
        lk = -1;
      }
      catch (p1A)
      {
        lk = -2;
      }
      b0 = b0A; e0 = e0A; l1 = l1A; if (l1 == 0) {end = e0A;} else {
      b1 = b1A; e1 = e1A; l2 = l2A; if (l2 == 0) {end = e1A;} else {
      b2 = b2A; e2 = e2A; end = e2A; }}
      memoize(7, e0, lk);
    }
    switch (lk)
    {
    case -1:
      whitespace();
      parse_AnyFunctionTest();
      break;
    default:
      whitespace();
      parse_TypedFunctionTest();
    }
    eventHandler.endNonterminal("FunctionTest", e0);
  }

  function try_FunctionTest()
  {
    for (;;)
    {
      lookahead1W(101);             // S^WS | '%' | '(:' | 'function'
      if (l1 != 33)                 // '%'
      {
        break;
      }
      try_Annotation();
    }
    switch (l1)
    {
    case 147:                       // 'function'
      lookahead2W(22);              // S^WS | '(' | '(:'
      break;
    default:
      lk = l1;
    }
    lk = memoized(7, e0);
    if (lk == 0)
    {
      var b0A = b0; var e0A = e0; var l1A = l1;
      var b1A = b1; var e1A = e1; var l2A = l2;
      var b2A = b2; var e2A = e2;
      try
      {
        try_AnyFunctionTest();
        memoize(7, e0A, -1);
        lk = -3;
      }
      catch (p1A)
      {
        lk = -2;
        b0 = b0A; e0 = e0A; l1 = l1A; if (l1 == 0) {end = e0A;} else {
        b1 = b1A; e1 = e1A; l2 = l2A; if (l2 == 0) {end = e1A;} else {
        b2 = b2A; e2 = e2A; end = e2A; }}
        memoize(7, e0A, -2);
      }
    }
    switch (lk)
    {
    case -1:
      try_AnyFunctionTest();
      break;
    case -3:
      break;
    default:
      try_TypedFunctionTest();
    }
  }

  function parse_AnyFunctionTest()
  {
    eventHandler.startNonterminal("AnyFunctionTest", e0);
    shift(147);                     // 'function'
    lookahead1W(22);                // S^WS | '(' | '(:'
    shift(35);                      // '('
    lookahead1W(24);                // S^WS | '(:' | '*'
    shift(39);                      // '*'
    lookahead1W(23);                // S^WS | '(:' | ')'
    shift(38);                      // ')'
    eventHandler.endNonterminal("AnyFunctionTest", e0);
  }

  function try_AnyFunctionTest()
  {
    shiftT(147);                    // 'function'
    lookahead1W(22);                // S^WS | '(' | '(:'
    shiftT(35);                     // '('
    lookahead1W(24);                // S^WS | '(:' | '*'
    shiftT(39);                     // '*'
    lookahead1W(23);                // S^WS | '(:' | ')'
    shiftT(38);                     // ')'
  }

  function parse_TypedFunctionTest()
  {
    eventHandler.startNonterminal("TypedFunctionTest", e0);
    shift(147);                     // 'function'
    lookahead1W(22);                // S^WS | '(' | '(:'
    shift(35);                      // '('
    lookahead1W(259);               // EQName^Token | S^WS | '%' | '(' | '(:' | ')' | 'after' | 'allowing' |
    if (l1 != 38)                   // ')'
    {
      whitespace();
      parse_SequenceType();
      for (;;)
      {
        lookahead1W(105);           // S^WS | '(:' | ')' | ','
        if (l1 != 42)               // ','
        {
          break;
        }
        shift(42);                  // ','
        lookahead1W(254);           // EQName^Token | S^WS | '%' | '(' | '(:' | 'after' | 'allowing' | 'ancestor' |
        whitespace();
        parse_SequenceType();
      }
    }
    shift(38);                      // ')'
    lookahead1W(33);                // S^WS | '(:' | 'as'
    shift(80);                      // 'as'
    lookahead1W(254);               // EQName^Token | S^WS | '%' | '(' | '(:' | 'after' | 'allowing' | 'ancestor' |
    whitespace();
    parse_SequenceType();
    eventHandler.endNonterminal("TypedFunctionTest", e0);
  }

  function try_TypedFunctionTest()
  {
    shiftT(147);                    // 'function'
    lookahead1W(22);                // S^WS | '(' | '(:'
    shiftT(35);                     // '('
    lookahead1W(259);               // EQName^Token | S^WS | '%' | '(' | '(:' | ')' | 'after' | 'allowing' |
    if (l1 != 38)                   // ')'
    {
      try_SequenceType();
      for (;;)
      {
        lookahead1W(105);           // S^WS | '(:' | ')' | ','
        if (l1 != 42)               // ','
        {
          break;
        }
        shiftT(42);                 // ','
        lookahead1W(254);           // EQName^Token | S^WS | '%' | '(' | '(:' | 'after' | 'allowing' | 'ancestor' |
        try_SequenceType();
      }
    }
    shiftT(38);                     // ')'
    lookahead1W(33);                // S^WS | '(:' | 'as'
    shiftT(80);                     // 'as'
    lookahead1W(254);               // EQName^Token | S^WS | '%' | '(' | '(:' | 'after' | 'allowing' | 'ancestor' |
    try_SequenceType();
  }

  function parse_ParenthesizedItemType()
  {
    eventHandler.startNonterminal("ParenthesizedItemType", e0);
    shift(35);                      // '('
    lookahead1W(254);               // EQName^Token | S^WS | '%' | '(' | '(:' | 'after' | 'allowing' | 'ancestor' |
    whitespace();
    parse_ItemType();
    lookahead1W(23);                // S^WS | '(:' | ')'
    shift(38);                      // ')'
    eventHandler.endNonterminal("ParenthesizedItemType", e0);
  }

  function try_ParenthesizedItemType()
  {
    shiftT(35);                     // '('
    lookahead1W(254);               // EQName^Token | S^WS | '%' | '(' | '(:' | 'after' | 'allowing' | 'ancestor' |
    try_ItemType();
    lookahead1W(23);                // S^WS | '(:' | ')'
    shiftT(38);                     // ')'
  }

  function parse_RevalidationDecl()
  {
    eventHandler.startNonterminal("RevalidationDecl", e0);
    shift(109);                     // 'declare'
    lookahead1W(75);                // S^WS | '(:' | 'revalidation'
    shift(226);                     // 'revalidation'
    lookahead1W(162);               // S^WS | '(:' | 'lax' | 'skip' | 'strict'
    switch (l1)
    {
    case 245:                       // 'strict'
      shift(245);                   // 'strict'
      break;
    case 174:                       // 'lax'
      shift(174);                   // 'lax'
      break;
    default:
      shift(238);                   // 'skip'
    }
    eventHandler.endNonterminal("RevalidationDecl", e0);
  }

  function parse_InsertExprTargetChoice()
  {
    eventHandler.startNonterminal("InsertExprTargetChoice", e0);
    switch (l1)
    {
    case 71:                        // 'after'
      shift(71);                    // 'after'
      break;
    case 85:                        // 'before'
      shift(85);                    // 'before'
      break;
    default:
      if (l1 == 80)                 // 'as'
      {
        shift(80);                  // 'as'
        lookahead1W(123);           // S^WS | '(:' | 'first' | 'last'
        switch (l1)
        {
        case 136:                   // 'first'
          shift(136);               // 'first'
          break;
        default:
          shift(173);               // 'last'
        }
      }
      lookahead1W(57);              // S^WS | '(:' | 'into'
      shift(165);                   // 'into'
    }
    eventHandler.endNonterminal("InsertExprTargetChoice", e0);
  }

  function try_InsertExprTargetChoice()
  {
    switch (l1)
    {
    case 71:                        // 'after'
      shiftT(71);                   // 'after'
      break;
    case 85:                        // 'before'
      shiftT(85);                   // 'before'
      break;
    default:
      if (l1 == 80)                 // 'as'
      {
        shiftT(80);                 // 'as'
        lookahead1W(123);           // S^WS | '(:' | 'first' | 'last'
        switch (l1)
        {
        case 136:                   // 'first'
          shiftT(136);              // 'first'
          break;
        default:
          shiftT(173);              // 'last'
        }
      }
      lookahead1W(57);              // S^WS | '(:' | 'into'
      shiftT(165);                  // 'into'
    }
  }

  function parse_InsertExpr()
  {
    eventHandler.startNonterminal("InsertExpr", e0);
    shift(161);                     // 'insert'
    lookahead1W(133);               // S^WS | '(:' | 'node' | 'nodes'
    switch (l1)
    {
    case 194:                       // 'node'
      shift(194);                   // 'node'
      break;
    default:
      shift(195);                   // 'nodes'
    }
    lookahead1W(267);               // EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral | StringLiteral |
    whitespace();
    parse_SourceExpr();
    whitespace();
    parse_InsertExprTargetChoice();
    lookahead1W(267);               // EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral | StringLiteral |
    whitespace();
    parse_TargetExpr();
    eventHandler.endNonterminal("InsertExpr", e0);
  }

  function try_InsertExpr()
  {
    shiftT(161);                    // 'insert'
    lookahead1W(133);               // S^WS | '(:' | 'node' | 'nodes'
    switch (l1)
    {
    case 194:                       // 'node'
      shiftT(194);                  // 'node'
      break;
    default:
      shiftT(195);                  // 'nodes'
    }
    lookahead1W(267);               // EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral | StringLiteral |
    try_SourceExpr();
    try_InsertExprTargetChoice();
    lookahead1W(267);               // EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral | StringLiteral |
    try_TargetExpr();
  }

  function parse_DeleteExpr()
  {
    eventHandler.startNonterminal("DeleteExpr", e0);
    shift(111);                     // 'delete'
    lookahead1W(133);               // S^WS | '(:' | 'node' | 'nodes'
    switch (l1)
    {
    case 194:                       // 'node'
      shift(194);                   // 'node'
      break;
    default:
      shift(195);                   // 'nodes'
    }
    lookahead1W(267);               // EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral | StringLiteral |
    whitespace();
    parse_TargetExpr();
    eventHandler.endNonterminal("DeleteExpr", e0);
  }

  function try_DeleteExpr()
  {
    shiftT(111);                    // 'delete'
    lookahead1W(133);               // S^WS | '(:' | 'node' | 'nodes'
    switch (l1)
    {
    case 194:                       // 'node'
      shiftT(194);                  // 'node'
      break;
    default:
      shiftT(195);                  // 'nodes'
    }
    lookahead1W(267);               // EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral | StringLiteral |
    try_TargetExpr();
  }

  function parse_ReplaceExpr()
  {
    eventHandler.startNonterminal("ReplaceExpr", e0);
    shift(223);                     // 'replace'
    lookahead1W(134);               // S^WS | '(:' | 'node' | 'value'
    if (l1 == 267)                  // 'value'
    {
      shift(267);                   // 'value'
      lookahead1W(67);              // S^WS | '(:' | 'of'
      shift(200);                   // 'of'
    }
    lookahead1W(65);                // S^WS | '(:' | 'node'
    shift(194);                     // 'node'
    lookahead1W(267);               // EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral | StringLiteral |
    whitespace();
    parse_TargetExpr();
    shift(276);                     // 'with'
    lookahead1W(267);               // EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral | StringLiteral |
    whitespace();
    parse_ExprSingle();
    eventHandler.endNonterminal("ReplaceExpr", e0);
  }

  function try_ReplaceExpr()
  {
    shiftT(223);                    // 'replace'
    lookahead1W(134);               // S^WS | '(:' | 'node' | 'value'
    if (l1 == 267)                  // 'value'
    {
      shiftT(267);                  // 'value'
      lookahead1W(67);              // S^WS | '(:' | 'of'
      shiftT(200);                  // 'of'
    }
    lookahead1W(65);                // S^WS | '(:' | 'node'
    shiftT(194);                    // 'node'
    lookahead1W(267);               // EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral | StringLiteral |
    try_TargetExpr();
    shiftT(276);                    // 'with'
    lookahead1W(267);               // EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral | StringLiteral |
    try_ExprSingle();
  }

  function parse_RenameExpr()
  {
    eventHandler.startNonterminal("RenameExpr", e0);
    shift(222);                     // 'rename'
    lookahead1W(65);                // S^WS | '(:' | 'node'
    shift(194);                     // 'node'
    lookahead1W(267);               // EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral | StringLiteral |
    whitespace();
    parse_TargetExpr();
    shift(80);                      // 'as'
    lookahead1W(267);               // EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral | StringLiteral |
    whitespace();
    parse_NewNameExpr();
    eventHandler.endNonterminal("RenameExpr", e0);
  }

  function try_RenameExpr()
  {
    shiftT(222);                    // 'rename'
    lookahead1W(65);                // S^WS | '(:' | 'node'
    shiftT(194);                    // 'node'
    lookahead1W(267);               // EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral | StringLiteral |
    try_TargetExpr();
    shiftT(80);                     // 'as'
    lookahead1W(267);               // EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral | StringLiteral |
    try_NewNameExpr();
  }

  function parse_SourceExpr()
  {
    eventHandler.startNonterminal("SourceExpr", e0);
    parse_ExprSingle();
    eventHandler.endNonterminal("SourceExpr", e0);
  }

  function try_SourceExpr()
  {
    try_ExprSingle();
  }

  function parse_TargetExpr()
  {
    eventHandler.startNonterminal("TargetExpr", e0);
    parse_ExprSingle();
    eventHandler.endNonterminal("TargetExpr", e0);
  }

  function try_TargetExpr()
  {
    try_ExprSingle();
  }

  function parse_NewNameExpr()
  {
    eventHandler.startNonterminal("NewNameExpr", e0);
    parse_ExprSingle();
    eventHandler.endNonterminal("NewNameExpr", e0);
  }

  function try_NewNameExpr()
  {
    try_ExprSingle();
  }

  function parse_TransformExpr()
  {
    eventHandler.startNonterminal("TransformExpr", e0);
    shift(104);                     // 'copy'
    lookahead1W(21);                // S^WS | '$' | '(:'
    whitespace();
    parse_TransformSpec();
    for (;;)
    {
      if (l1 != 42)                 // ','
      {
        break;
      }
      shift(42);                    // ','
      lookahead1W(21);              // S^WS | '$' | '(:'
      whitespace();
      parse_TransformSpec();
    }
    shift(184);                     // 'modify'
    lookahead1W(267);               // EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral | StringLiteral |
    whitespace();
    parse_ExprSingle();
    shift(224);                     // 'return'
    lookahead1W(267);               // EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral | StringLiteral |
    whitespace();
    parse_ExprSingle();
    eventHandler.endNonterminal("TransformExpr", e0);
  }

  function try_TransformExpr()
  {
    shiftT(104);                    // 'copy'
    lookahead1W(21);                // S^WS | '$' | '(:'
    try_TransformSpec();
    for (;;)
    {
      if (l1 != 42)                 // ','
      {
        break;
      }
      shiftT(42);                   // ','
      lookahead1W(21);              // S^WS | '$' | '(:'
      try_TransformSpec();
    }
    shiftT(184);                    // 'modify'
    lookahead1W(267);               // EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral | StringLiteral |
    try_ExprSingle();
    shiftT(224);                    // 'return'
    lookahead1W(267);               // EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral | StringLiteral |
    try_ExprSingle();
  }

  function parse_TransformSpec()
  {
    eventHandler.startNonterminal("TransformSpec", e0);
    shift(31);                      // '$'
    lookahead1W(246);               // EQName^Token | S^WS | '(:' | 'after' | 'allowing' | 'ancestor' |
    whitespace();
    parse_VarName();
    lookahead1W(28);                // S^WS | '(:' | ':='
    shift(53);                      // ':='
    lookahead1W(267);               // EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral | StringLiteral |
    whitespace();
    parse_ExprSingle();
    eventHandler.endNonterminal("TransformSpec", e0);
  }

  function try_TransformSpec()
  {
    shiftT(31);                     // '$'
    lookahead1W(246);               // EQName^Token | S^WS | '(:' | 'after' | 'allowing' | 'ancestor' |
    try_VarName();
    lookahead1W(28);                // S^WS | '(:' | ':='
    shiftT(53);                     // ':='
    lookahead1W(267);               // EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral | StringLiteral |
    try_ExprSingle();
  }

  function parse_FTSelection()
  {
    eventHandler.startNonterminal("FTSelection", e0);
    parse_FTOr();
    for (;;)
    {
      lookahead1W(212);             // S^WS | EOF | '!=' | '(:' | ')' | ',' | ':' | ';' | '<' | '<<' | '<=' | '=' |
      switch (l1)
      {
      case 82:                      // 'at'
        lookahead2W(161);           // S^WS | '(:' | 'end' | 'position' | 'start'
        break;
      default:
        lk = l1;
      }
      if (lk != 116                 // 'different'
       && lk != 118                 // 'distance'
       && lk != 128                 // 'entire'
       && lk != 206                 // 'ordered'
       && lk != 227                 // 'same'
       && lk != 275                 // 'window'
       && lk != 65106               // 'at' 'end'
       && lk != 123986)             // 'at' 'start'
      {
        break;
      }
      whitespace();
      parse_FTPosFilter();
    }
    eventHandler.endNonterminal("FTSelection", e0);
  }

  function try_FTSelection()
  {
    try_FTOr();
    for (;;)
    {
      lookahead1W(212);             // S^WS | EOF | '!=' | '(:' | ')' | ',' | ':' | ';' | '<' | '<<' | '<=' | '=' |
      switch (l1)
      {
      case 82:                      // 'at'
        lookahead2W(161);           // S^WS | '(:' | 'end' | 'position' | 'start'
        break;
      default:
        lk = l1;
      }
      if (lk != 116                 // 'different'
       && lk != 118                 // 'distance'
       && lk != 128                 // 'entire'
       && lk != 206                 // 'ordered'
       && lk != 227                 // 'same'
       && lk != 275                 // 'window'
       && lk != 65106               // 'at' 'end'
       && lk != 123986)             // 'at' 'start'
      {
        break;
      }
      try_FTPosFilter();
    }
  }

  function parse_FTWeight()
  {
    eventHandler.startNonterminal("FTWeight", e0);
    shift(270);                     // 'weight'
    lookahead1W(90);                // S^WS | '(:' | '{'
    shift(281);                     // '{'
    lookahead1W(267);               // EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral | StringLiteral |
    whitespace();
    parse_Expr();
    shift(287);                     // '}'
    eventHandler.endNonterminal("FTWeight", e0);
  }

  function try_FTWeight()
  {
    shiftT(270);                    // 'weight'
    lookahead1W(90);                // S^WS | '(:' | '{'
    shiftT(281);                    // '{'
    lookahead1W(267);               // EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral | StringLiteral |
    try_Expr();
    shiftT(287);                    // '}'
  }

  function parse_FTOr()
  {
    eventHandler.startNonterminal("FTOr", e0);
    parse_FTAnd();
    for (;;)
    {
      if (l1 != 146)                // 'ftor'
      {
        break;
      }
      shift(146);                   // 'ftor'
      lookahead1W(177);             // StringLiteral | S^WS | '(' | '(#' | '(:' | 'ftnot' | '{'
      whitespace();
      parse_FTAnd();
    }
    eventHandler.endNonterminal("FTOr", e0);
  }

  function try_FTOr()
  {
    try_FTAnd();
    for (;;)
    {
      if (l1 != 146)                // 'ftor'
      {
        break;
      }
      shiftT(146);                  // 'ftor'
      lookahead1W(177);             // StringLiteral | S^WS | '(' | '(#' | '(:' | 'ftnot' | '{'
      try_FTAnd();
    }
  }

  function parse_FTAnd()
  {
    eventHandler.startNonterminal("FTAnd", e0);
    parse_FTMildNot();
    for (;;)
    {
      if (l1 != 144)                // 'ftand'
      {
        break;
      }
      shift(144);                   // 'ftand'
      lookahead1W(177);             // StringLiteral | S^WS | '(' | '(#' | '(:' | 'ftnot' | '{'
      whitespace();
      parse_FTMildNot();
    }
    eventHandler.endNonterminal("FTAnd", e0);
  }

  function try_FTAnd()
  {
    try_FTMildNot();
    for (;;)
    {
      if (l1 != 144)                // 'ftand'
      {
        break;
      }
      shiftT(144);                  // 'ftand'
      lookahead1W(177);             // StringLiteral | S^WS | '(' | '(#' | '(:' | 'ftnot' | '{'
      try_FTMildNot();
    }
  }

  function parse_FTMildNot()
  {
    eventHandler.startNonterminal("FTMildNot", e0);
    parse_FTUnaryNot();
    for (;;)
    {
      lookahead1W(213);             // S^WS | EOF | '!=' | '(:' | ')' | ',' | ':' | ';' | '<' | '<<' | '<=' | '=' |
      if (l1 != 196)                // 'not'
      {
        break;
      }
      shift(196);                   // 'not'
      lookahead1W(56);              // S^WS | '(:' | 'in'
      shift(156);                   // 'in'
      lookahead1W(177);             // StringLiteral | S^WS | '(' | '(#' | '(:' | 'ftnot' | '{'
      whitespace();
      parse_FTUnaryNot();
    }
    eventHandler.endNonterminal("FTMildNot", e0);
  }

  function try_FTMildNot()
  {
    try_FTUnaryNot();
    for (;;)
    {
      lookahead1W(213);             // S^WS | EOF | '!=' | '(:' | ')' | ',' | ':' | ';' | '<' | '<<' | '<=' | '=' |
      if (l1 != 196)                // 'not'
      {
        break;
      }
      shiftT(196);                  // 'not'
      lookahead1W(56);              // S^WS | '(:' | 'in'
      shiftT(156);                  // 'in'
      lookahead1W(177);             // StringLiteral | S^WS | '(' | '(#' | '(:' | 'ftnot' | '{'
      try_FTUnaryNot();
    }
  }

  function parse_FTUnaryNot()
  {
    eventHandler.startNonterminal("FTUnaryNot", e0);
    if (l1 == 145)                  // 'ftnot'
    {
      shift(145);                   // 'ftnot'
    }
    lookahead1W(164);               // StringLiteral | S^WS | '(' | '(#' | '(:' | '{'
    whitespace();
    parse_FTPrimaryWithOptions();
    eventHandler.endNonterminal("FTUnaryNot", e0);
  }

  function try_FTUnaryNot()
  {
    if (l1 == 145)                  // 'ftnot'
    {
      shiftT(145);                  // 'ftnot'
    }
    lookahead1W(164);               // StringLiteral | S^WS | '(' | '(#' | '(:' | '{'
    try_FTPrimaryWithOptions();
  }

  function parse_FTPrimaryWithOptions()
  {
    eventHandler.startNonterminal("FTPrimaryWithOptions", e0);
    parse_FTPrimary();
    lookahead1W(214);               // S^WS | EOF | '!=' | '(:' | ')' | ',' | ':' | ';' | '<' | '<<' | '<=' | '=' |
    if (l1 == 265)                  // 'using'
    {
      whitespace();
      parse_FTMatchOptions();
    }
    if (l1 == 270)                  // 'weight'
    {
      whitespace();
      parse_FTWeight();
    }
    eventHandler.endNonterminal("FTPrimaryWithOptions", e0);
  }

  function try_FTPrimaryWithOptions()
  {
    try_FTPrimary();
    lookahead1W(214);               // S^WS | EOF | '!=' | '(:' | ')' | ',' | ':' | ';' | '<' | '<<' | '<=' | '=' |
    if (l1 == 265)                  // 'using'
    {
      try_FTMatchOptions();
    }
    if (l1 == 270)                  // 'weight'
    {
      try_FTWeight();
    }
  }

  function parse_FTPrimary()
  {
    eventHandler.startNonterminal("FTPrimary", e0);
    switch (l1)
    {
    case 35:                        // '('
      shift(35);                    // '('
      lookahead1W(177);             // StringLiteral | S^WS | '(' | '(#' | '(:' | 'ftnot' | '{'
      whitespace();
      parse_FTSelection();
      shift(38);                    // ')'
      break;
    case 36:                        // '(#'
      parse_FTExtensionSelection();
      break;
    default:
      parse_FTWords();
      lookahead1W(215);             // S^WS | EOF | '!=' | '(:' | ')' | ',' | ':' | ';' | '<' | '<<' | '<=' | '=' |
      if (l1 == 199)                // 'occurs'
      {
        whitespace();
        parse_FTTimes();
      }
    }
    eventHandler.endNonterminal("FTPrimary", e0);
  }

  function try_FTPrimary()
  {
    switch (l1)
    {
    case 35:                        // '('
      shiftT(35);                   // '('
      lookahead1W(177);             // StringLiteral | S^WS | '(' | '(#' | '(:' | 'ftnot' | '{'
      try_FTSelection();
      shiftT(38);                   // ')'
      break;
    case 36:                        // '(#'
      try_FTExtensionSelection();
      break;
    default:
      try_FTWords();
      lookahead1W(215);             // S^WS | EOF | '!=' | '(:' | ')' | ',' | ':' | ';' | '<' | '<<' | '<=' | '=' |
      if (l1 == 199)                // 'occurs'
      {
        try_FTTimes();
      }
    }
  }

  function parse_FTWords()
  {
    eventHandler.startNonterminal("FTWords", e0);
    parse_FTWordsValue();
    lookahead1W(221);               // S^WS | EOF | '!=' | '(:' | ')' | ',' | ':' | ';' | '<' | '<<' | '<=' | '=' |
    if (l1 == 72                    // 'all'
     || l1 == 77                    // 'any'
     || l1 == 214)                  // 'phrase'
    {
      whitespace();
      parse_FTAnyallOption();
    }
    eventHandler.endNonterminal("FTWords", e0);
  }

  function try_FTWords()
  {
    try_FTWordsValue();
    lookahead1W(221);               // S^WS | EOF | '!=' | '(:' | ')' | ',' | ':' | ';' | '<' | '<<' | '<=' | '=' |
    if (l1 == 72                    // 'all'
     || l1 == 77                    // 'any'
     || l1 == 214)                  // 'phrase'
    {
      try_FTAnyallOption();
    }
  }

  function parse_FTWordsValue()
  {
    eventHandler.startNonterminal("FTWordsValue", e0);
    switch (l1)
    {
    case 11:                        // StringLiteral
      shift(11);                    // StringLiteral
      break;
    default:
      shift(281);                   // '{'
      lookahead1W(267);             // EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral | StringLiteral |
      whitespace();
      parse_Expr();
      shift(287);                   // '}'
    }
    eventHandler.endNonterminal("FTWordsValue", e0);
  }

  function try_FTWordsValue()
  {
    switch (l1)
    {
    case 11:                        // StringLiteral
      shiftT(11);                   // StringLiteral
      break;
    default:
      shiftT(281);                  // '{'
      lookahead1W(267);             // EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral | StringLiteral |
      try_Expr();
      shiftT(287);                  // '}'
    }
  }

  function parse_FTExtensionSelection()
  {
    eventHandler.startNonterminal("FTExtensionSelection", e0);
    for (;;)
    {
      whitespace();
      parse_Pragma();
      lookahead1W(104);             // S^WS | '(#' | '(:' | '{'
      if (l1 != 36)                 // '(#'
      {
        break;
      }
    }
    shift(281);                     // '{'
    lookahead1W(184);               // StringLiteral | S^WS | '(' | '(#' | '(:' | 'ftnot' | '{' | '}'
    if (l1 != 287)                  // '}'
    {
      whitespace();
      parse_FTSelection();
    }
    shift(287);                     // '}'
    eventHandler.endNonterminal("FTExtensionSelection", e0);
  }

  function try_FTExtensionSelection()
  {
    for (;;)
    {
      try_Pragma();
      lookahead1W(104);             // S^WS | '(#' | '(:' | '{'
      if (l1 != 36)                 // '(#'
      {
        break;
      }
    }
    shiftT(281);                    // '{'
    lookahead1W(184);               // StringLiteral | S^WS | '(' | '(#' | '(:' | 'ftnot' | '{' | '}'
    if (l1 != 287)                  // '}'
    {
      try_FTSelection();
    }
    shiftT(287);                    // '}'
  }

  function parse_FTAnyallOption()
  {
    eventHandler.startNonterminal("FTAnyallOption", e0);
    switch (l1)
    {
    case 77:                        // 'any'
      shift(77);                    // 'any'
      lookahead1W(218);             // S^WS | EOF | '!=' | '(:' | ')' | ',' | ':' | ';' | '<' | '<<' | '<=' | '=' |
      if (l1 == 278)                // 'word'
      {
        shift(278);                 // 'word'
      }
      break;
    case 72:                        // 'all'
      shift(72);                    // 'all'
      lookahead1W(219);             // S^WS | EOF | '!=' | '(:' | ')' | ',' | ':' | ';' | '<' | '<<' | '<=' | '=' |
      if (l1 == 279)                // 'words'
      {
        shift(279);                 // 'words'
      }
      break;
    default:
      shift(214);                   // 'phrase'
    }
    eventHandler.endNonterminal("FTAnyallOption", e0);
  }

  function try_FTAnyallOption()
  {
    switch (l1)
    {
    case 77:                        // 'any'
      shiftT(77);                   // 'any'
      lookahead1W(218);             // S^WS | EOF | '!=' | '(:' | ')' | ',' | ':' | ';' | '<' | '<<' | '<=' | '=' |
      if (l1 == 278)                // 'word'
      {
        shiftT(278);                // 'word'
      }
      break;
    case 72:                        // 'all'
      shiftT(72);                   // 'all'
      lookahead1W(219);             // S^WS | EOF | '!=' | '(:' | ')' | ',' | ':' | ';' | '<' | '<<' | '<=' | '=' |
      if (l1 == 279)                // 'words'
      {
        shiftT(279);                // 'words'
      }
      break;
    default:
      shiftT(214);                  // 'phrase'
    }
  }

  function parse_FTTimes()
  {
    eventHandler.startNonterminal("FTTimes", e0);
    shift(199);                     // 'occurs'
    lookahead1W(159);               // S^WS | '(:' | 'at' | 'exactly' | 'from'
    whitespace();
    parse_FTRange();
    shift(252);                     // 'times'
    eventHandler.endNonterminal("FTTimes", e0);
  }

  function try_FTTimes()
  {
    shiftT(199);                    // 'occurs'
    lookahead1W(159);               // S^WS | '(:' | 'at' | 'exactly' | 'from'
    try_FTRange();
    shiftT(252);                    // 'times'
  }

  function parse_FTRange()
  {
    eventHandler.startNonterminal("FTRange", e0);
    switch (l1)
    {
    case 131:                       // 'exactly'
      shift(131);                   // 'exactly'
      lookahead1W(266);             // EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral | StringLiteral |
      whitespace();
      parse_AdditiveExpr();
      break;
    case 82:                        // 'at'
      shift(82);                    // 'at'
      lookahead1W(129);             // S^WS | '(:' | 'least' | 'most'
      switch (l1)
      {
      case 176:                     // 'least'
        shift(176);                 // 'least'
        lookahead1W(266);           // EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral | StringLiteral |
        whitespace();
        parse_AdditiveExpr();
        break;
      default:
        shift(186);                 // 'most'
        lookahead1W(266);           // EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral | StringLiteral |
        whitespace();
        parse_AdditiveExpr();
      }
      break;
    default:
      shift(142);                   // 'from'
      lookahead1W(266);             // EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral | StringLiteral |
      whitespace();
      parse_AdditiveExpr();
      shift(253);                   // 'to'
      lookahead1W(266);             // EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral | StringLiteral |
      whitespace();
      parse_AdditiveExpr();
    }
    eventHandler.endNonterminal("FTRange", e0);
  }

  function try_FTRange()
  {
    switch (l1)
    {
    case 131:                       // 'exactly'
      shiftT(131);                  // 'exactly'
      lookahead1W(266);             // EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral | StringLiteral |
      try_AdditiveExpr();
      break;
    case 82:                        // 'at'
      shiftT(82);                   // 'at'
      lookahead1W(129);             // S^WS | '(:' | 'least' | 'most'
      switch (l1)
      {
      case 176:                     // 'least'
        shiftT(176);                // 'least'
        lookahead1W(266);           // EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral | StringLiteral |
        try_AdditiveExpr();
        break;
      default:
        shiftT(186);                // 'most'
        lookahead1W(266);           // EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral | StringLiteral |
        try_AdditiveExpr();
      }
      break;
    default:
      shiftT(142);                  // 'from'
      lookahead1W(266);             // EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral | StringLiteral |
      try_AdditiveExpr();
      shiftT(253);                  // 'to'
      lookahead1W(266);             // EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral | StringLiteral |
      try_AdditiveExpr();
    }
  }

  function parse_FTPosFilter()
  {
    eventHandler.startNonterminal("FTPosFilter", e0);
    switch (l1)
    {
    case 206:                       // 'ordered'
      parse_FTOrder();
      break;
    case 275:                       // 'window'
      parse_FTWindow();
      break;
    case 118:                       // 'distance'
      parse_FTDistance();
      break;
    case 116:                       // 'different'
    case 227:                       // 'same'
      parse_FTScope();
      break;
    default:
      parse_FTContent();
    }
    eventHandler.endNonterminal("FTPosFilter", e0);
  }

  function try_FTPosFilter()
  {
    switch (l1)
    {
    case 206:                       // 'ordered'
      try_FTOrder();
      break;
    case 275:                       // 'window'
      try_FTWindow();
      break;
    case 118:                       // 'distance'
      try_FTDistance();
      break;
    case 116:                       // 'different'
    case 227:                       // 'same'
      try_FTScope();
      break;
    default:
      try_FTContent();
    }
  }

  function parse_FTOrder()
  {
    eventHandler.startNonterminal("FTOrder", e0);
    shift(206);                     // 'ordered'
    eventHandler.endNonterminal("FTOrder", e0);
  }

  function try_FTOrder()
  {
    shiftT(206);                    // 'ordered'
  }

  function parse_FTWindow()
  {
    eventHandler.startNonterminal("FTWindow", e0);
    shift(275);                     // 'window'
    lookahead1W(266);               // EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral | StringLiteral |
    whitespace();
    parse_AdditiveExpr();
    whitespace();
    parse_FTUnit();
    eventHandler.endNonterminal("FTWindow", e0);
  }

  function try_FTWindow()
  {
    shiftT(275);                    // 'window'
    lookahead1W(266);               // EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral | StringLiteral |
    try_AdditiveExpr();
    try_FTUnit();
  }

  function parse_FTDistance()
  {
    eventHandler.startNonterminal("FTDistance", e0);
    shift(118);                     // 'distance'
    lookahead1W(159);               // S^WS | '(:' | 'at' | 'exactly' | 'from'
    whitespace();
    parse_FTRange();
    whitespace();
    parse_FTUnit();
    eventHandler.endNonterminal("FTDistance", e0);
  }

  function try_FTDistance()
  {
    shiftT(118);                    // 'distance'
    lookahead1W(159);               // S^WS | '(:' | 'at' | 'exactly' | 'from'
    try_FTRange();
    try_FTUnit();
  }

  function parse_FTUnit()
  {
    eventHandler.startNonterminal("FTUnit", e0);
    switch (l1)
    {
    case 279:                       // 'words'
      shift(279);                   // 'words'
      break;
    case 237:                       // 'sentences'
      shift(237);                   // 'sentences'
      break;
    default:
      shift(209);                   // 'paragraphs'
    }
    eventHandler.endNonterminal("FTUnit", e0);
  }

  function try_FTUnit()
  {
    switch (l1)
    {
    case 279:                       // 'words'
      shiftT(279);                  // 'words'
      break;
    case 237:                       // 'sentences'
      shiftT(237);                  // 'sentences'
      break;
    default:
      shiftT(209);                  // 'paragraphs'
    }
  }

  function parse_FTScope()
  {
    eventHandler.startNonterminal("FTScope", e0);
    switch (l1)
    {
    case 227:                       // 'same'
      shift(227);                   // 'same'
      break;
    default:
      shift(116);                   // 'different'
    }
    lookahead1W(136);               // S^WS | '(:' | 'paragraph' | 'sentence'
    whitespace();
    parse_FTBigUnit();
    eventHandler.endNonterminal("FTScope", e0);
  }

  function try_FTScope()
  {
    switch (l1)
    {
    case 227:                       // 'same'
      shiftT(227);                  // 'same'
      break;
    default:
      shiftT(116);                  // 'different'
    }
    lookahead1W(136);               // S^WS | '(:' | 'paragraph' | 'sentence'
    try_FTBigUnit();
  }

  function parse_FTBigUnit()
  {
    eventHandler.startNonterminal("FTBigUnit", e0);
    switch (l1)
    {
    case 236:                       // 'sentence'
      shift(236);                   // 'sentence'
      break;
    default:
      shift(208);                   // 'paragraph'
    }
    eventHandler.endNonterminal("FTBigUnit", e0);
  }

  function try_FTBigUnit()
  {
    switch (l1)
    {
    case 236:                       // 'sentence'
      shiftT(236);                  // 'sentence'
      break;
    default:
      shiftT(208);                  // 'paragraph'
    }
  }

  function parse_FTContent()
  {
    eventHandler.startNonterminal("FTContent", e0);
    switch (l1)
    {
    case 82:                        // 'at'
      shift(82);                    // 'at'
      lookahead1W(121);             // S^WS | '(:' | 'end' | 'start'
      switch (l1)
      {
      case 242:                     // 'start'
        shift(242);                 // 'start'
        break;
      default:
        shift(127);                 // 'end'
      }
      break;
    default:
      shift(128);                   // 'entire'
      lookahead1W(45);              // S^WS | '(:' | 'content'
      shift(101);                   // 'content'
    }
    eventHandler.endNonterminal("FTContent", e0);
  }

  function try_FTContent()
  {
    switch (l1)
    {
    case 82:                        // 'at'
      shiftT(82);                   // 'at'
      lookahead1W(121);             // S^WS | '(:' | 'end' | 'start'
      switch (l1)
      {
      case 242:                     // 'start'
        shiftT(242);                // 'start'
        break;
      default:
        shiftT(127);                // 'end'
      }
      break;
    default:
      shiftT(128);                  // 'entire'
      lookahead1W(45);              // S^WS | '(:' | 'content'
      shiftT(101);                  // 'content'
    }
  }

  function parse_FTMatchOptions()
  {
    eventHandler.startNonterminal("FTMatchOptions", e0);
    for (;;)
    {
      shift(265);                   // 'using'
      lookahead1W(205);             // S^WS | '(:' | 'case' | 'diacritics' | 'language' | 'lowercase' | 'no' |
      whitespace();
      parse_FTMatchOption();
      lookahead1W(214);             // S^WS | EOF | '!=' | '(:' | ')' | ',' | ':' | ';' | '<' | '<<' | '<=' | '=' |
      if (l1 != 265)                // 'using'
      {
        break;
      }
    }
    eventHandler.endNonterminal("FTMatchOptions", e0);
  }

  function try_FTMatchOptions()
  {
    for (;;)
    {
      shiftT(265);                  // 'using'
      lookahead1W(205);             // S^WS | '(:' | 'case' | 'diacritics' | 'language' | 'lowercase' | 'no' |
      try_FTMatchOption();
      lookahead1W(214);             // S^WS | EOF | '!=' | '(:' | ')' | ',' | ':' | ';' | '<' | '<<' | '<=' | '=' |
      if (l1 != 265)                // 'using'
      {
        break;
      }
    }
  }

  function parse_FTMatchOption()
  {
    eventHandler.startNonterminal("FTMatchOption", e0);
    switch (l1)
    {
    case 191:                       // 'no'
      lookahead2W(176);             // S^WS | '(:' | 'stemming' | 'stop' | 'thesaurus' | 'wildcards'
      break;
    default:
      lk = l1;
    }
    switch (lk)
    {
    case 172:                       // 'language'
      parse_FTLanguageOption();
      break;
    case 274:                       // 'wildcards'
    case 140479:                    // 'no' 'wildcards'
      parse_FTWildCardOption();
      break;
    case 251:                       // 'thesaurus'
    case 128703:                    // 'no' 'thesaurus'
      parse_FTThesaurusOption();
      break;
    case 243:                       // 'stemming'
    case 124607:                    // 'no' 'stemming'
      parse_FTStemOption();
      break;
    case 115:                       // 'diacritics'
      parse_FTDiacriticsOption();
      break;
    case 244:                       // 'stop'
    case 125119:                    // 'no' 'stop'
      parse_FTStopWordOption();
      break;
    case 203:                       // 'option'
      parse_FTExtensionOption();
      break;
    default:
      parse_FTCaseOption();
    }
    eventHandler.endNonterminal("FTMatchOption", e0);
  }

  function try_FTMatchOption()
  {
    switch (l1)
    {
    case 191:                       // 'no'
      lookahead2W(176);             // S^WS | '(:' | 'stemming' | 'stop' | 'thesaurus' | 'wildcards'
      break;
    default:
      lk = l1;
    }
    switch (lk)
    {
    case 172:                       // 'language'
      try_FTLanguageOption();
      break;
    case 274:                       // 'wildcards'
    case 140479:                    // 'no' 'wildcards'
      try_FTWildCardOption();
      break;
    case 251:                       // 'thesaurus'
    case 128703:                    // 'no' 'thesaurus'
      try_FTThesaurusOption();
      break;
    case 243:                       // 'stemming'
    case 124607:                    // 'no' 'stemming'
      try_FTStemOption();
      break;
    case 115:                       // 'diacritics'
      try_FTDiacriticsOption();
      break;
    case 244:                       // 'stop'
    case 125119:                    // 'no' 'stop'
      try_FTStopWordOption();
      break;
    case 203:                       // 'option'
      try_FTExtensionOption();
      break;
    default:
      try_FTCaseOption();
    }
  }

  function parse_FTCaseOption()
  {
    eventHandler.startNonterminal("FTCaseOption", e0);
    switch (l1)
    {
    case 89:                        // 'case'
      shift(89);                    // 'case'
      lookahead1W(128);             // S^WS | '(:' | 'insensitive' | 'sensitive'
      switch (l1)
      {
      case 160:                     // 'insensitive'
        shift(160);                 // 'insensitive'
        break;
      default:
        shift(235);                 // 'sensitive'
      }
      break;
    case 180:                       // 'lowercase'
      shift(180);                   // 'lowercase'
      break;
    default:
      shift(264);                   // 'uppercase'
    }
    eventHandler.endNonterminal("FTCaseOption", e0);
  }

  function try_FTCaseOption()
  {
    switch (l1)
    {
    case 89:                        // 'case'
      shiftT(89);                   // 'case'
      lookahead1W(128);             // S^WS | '(:' | 'insensitive' | 'sensitive'
      switch (l1)
      {
      case 160:                     // 'insensitive'
        shiftT(160);                // 'insensitive'
        break;
      default:
        shiftT(235);                // 'sensitive'
      }
      break;
    case 180:                       // 'lowercase'
      shiftT(180);                  // 'lowercase'
      break;
    default:
      shiftT(264);                  // 'uppercase'
    }
  }

  function parse_FTDiacriticsOption()
  {
    eventHandler.startNonterminal("FTDiacriticsOption", e0);
    shift(115);                     // 'diacritics'
    lookahead1W(128);               // S^WS | '(:' | 'insensitive' | 'sensitive'
    switch (l1)
    {
    case 160:                       // 'insensitive'
      shift(160);                   // 'insensitive'
      break;
    default:
      shift(235);                   // 'sensitive'
    }
    eventHandler.endNonterminal("FTDiacriticsOption", e0);
  }

  function try_FTDiacriticsOption()
  {
    shiftT(115);                    // 'diacritics'
    lookahead1W(128);               // S^WS | '(:' | 'insensitive' | 'sensitive'
    switch (l1)
    {
    case 160:                       // 'insensitive'
      shiftT(160);                  // 'insensitive'
      break;
    default:
      shiftT(235);                  // 'sensitive'
    }
  }

  function parse_FTStemOption()
  {
    eventHandler.startNonterminal("FTStemOption", e0);
    switch (l1)
    {
    case 243:                       // 'stemming'
      shift(243);                   // 'stemming'
      break;
    default:
      shift(191);                   // 'no'
      lookahead1W(77);              // S^WS | '(:' | 'stemming'
      shift(243);                   // 'stemming'
    }
    eventHandler.endNonterminal("FTStemOption", e0);
  }

  function try_FTStemOption()
  {
    switch (l1)
    {
    case 243:                       // 'stemming'
      shiftT(243);                  // 'stemming'
      break;
    default:
      shiftT(191);                  // 'no'
      lookahead1W(77);              // S^WS | '(:' | 'stemming'
      shiftT(243);                  // 'stemming'
    }
  }

  function parse_FTThesaurusOption()
  {
    eventHandler.startNonterminal("FTThesaurusOption", e0);
    switch (l1)
    {
    case 251:                       // 'thesaurus'
      shift(251);                   // 'thesaurus'
      lookahead1W(152);             // S^WS | '(' | '(:' | 'at' | 'default'
      switch (l1)
      {
      case 82:                      // 'at'
        whitespace();
        parse_FTThesaurusID();
        break;
      case 110:                     // 'default'
        shift(110);                 // 'default'
        break;
      default:
        shift(35);                  // '('
        lookahead1W(116);           // S^WS | '(:' | 'at' | 'default'
        switch (l1)
        {
        case 82:                    // 'at'
          whitespace();
          parse_FTThesaurusID();
          break;
        default:
          shift(110);               // 'default'
        }
        for (;;)
        {
          lookahead1W(105);         // S^WS | '(:' | ')' | ','
          if (l1 != 42)             // ','
          {
            break;
          }
          shift(42);                // ','
          lookahead1W(34);          // S^WS | '(:' | 'at'
          whitespace();
          parse_FTThesaurusID();
        }
        shift(38);                  // ')'
      }
      break;
    default:
      shift(191);                   // 'no'
      lookahead1W(81);              // S^WS | '(:' | 'thesaurus'
      shift(251);                   // 'thesaurus'
    }
    eventHandler.endNonterminal("FTThesaurusOption", e0);
  }

  function try_FTThesaurusOption()
  {
    switch (l1)
    {
    case 251:                       // 'thesaurus'
      shiftT(251);                  // 'thesaurus'
      lookahead1W(152);             // S^WS | '(' | '(:' | 'at' | 'default'
      switch (l1)
      {
      case 82:                      // 'at'
        try_FTThesaurusID();
        break;
      case 110:                     // 'default'
        shiftT(110);                // 'default'
        break;
      default:
        shiftT(35);                 // '('
        lookahead1W(116);           // S^WS | '(:' | 'at' | 'default'
        switch (l1)
        {
        case 82:                    // 'at'
          try_FTThesaurusID();
          break;
        default:
          shiftT(110);              // 'default'
        }
        for (;;)
        {
          lookahead1W(105);         // S^WS | '(:' | ')' | ','
          if (l1 != 42)             // ','
          {
            break;
          }
          shiftT(42);               // ','
          lookahead1W(34);          // S^WS | '(:' | 'at'
          try_FTThesaurusID();
        }
        shiftT(38);                 // ')'
      }
      break;
    default:
      shiftT(191);                  // 'no'
      lookahead1W(81);              // S^WS | '(:' | 'thesaurus'
      shiftT(251);                  // 'thesaurus'
    }
  }

  function parse_FTThesaurusID()
  {
    eventHandler.startNonterminal("FTThesaurusID", e0);
    shift(82);                      // 'at'
    lookahead1W(15);                // URILiteral | S^WS | '(:'
    shift(7);                       // URILiteral
    lookahead1W(220);               // S^WS | EOF | '!=' | '(:' | ')' | ',' | ':' | ';' | '<' | '<<' | '<=' | '=' |
    if (l1 == 221)                  // 'relationship'
    {
      shift(221);                   // 'relationship'
      lookahead1W(17);              // StringLiteral | S^WS | '(:'
      shift(11);                    // StringLiteral
    }
    lookahead1W(216);               // S^WS | EOF | '!=' | '(:' | ')' | ',' | ':' | ';' | '<' | '<<' | '<=' | '=' |
    switch (l1)
    {
    case 82:                        // 'at'
      lookahead2W(183);             // S^WS | '(:' | 'end' | 'least' | 'most' | 'position' | 'start'
      break;
    default:
      lk = l1;
    }
    if (lk == 131                   // 'exactly'
     || lk == 142                   // 'from'
     || lk == 90194                 // 'at' 'least'
     || lk == 95314)                // 'at' 'most'
    {
      whitespace();
      parse_FTLiteralRange();
      lookahead1W(61);              // S^WS | '(:' | 'levels'
      shift(178);                   // 'levels'
    }
    eventHandler.endNonterminal("FTThesaurusID", e0);
  }

  function try_FTThesaurusID()
  {
    shiftT(82);                     // 'at'
    lookahead1W(15);                // URILiteral | S^WS | '(:'
    shiftT(7);                      // URILiteral
    lookahead1W(220);               // S^WS | EOF | '!=' | '(:' | ')' | ',' | ':' | ';' | '<' | '<<' | '<=' | '=' |
    if (l1 == 221)                  // 'relationship'
    {
      shiftT(221);                  // 'relationship'
      lookahead1W(17);              // StringLiteral | S^WS | '(:'
      shiftT(11);                   // StringLiteral
    }
    lookahead1W(216);               // S^WS | EOF | '!=' | '(:' | ')' | ',' | ':' | ';' | '<' | '<<' | '<=' | '=' |
    switch (l1)
    {
    case 82:                        // 'at'
      lookahead2W(183);             // S^WS | '(:' | 'end' | 'least' | 'most' | 'position' | 'start'
      break;
    default:
      lk = l1;
    }
    if (lk == 131                   // 'exactly'
     || lk == 142                   // 'from'
     || lk == 90194                 // 'at' 'least'
     || lk == 95314)                // 'at' 'most'
    {
      try_FTLiteralRange();
      lookahead1W(61);              // S^WS | '(:' | 'levels'
      shiftT(178);                  // 'levels'
    }
  }

  function parse_FTLiteralRange()
  {
    eventHandler.startNonterminal("FTLiteralRange", e0);
    switch (l1)
    {
    case 131:                       // 'exactly'
      shift(131);                   // 'exactly'
      lookahead1W(16);              // IntegerLiteral | S^WS | '(:'
      shift(8);                     // IntegerLiteral
      break;
    case 82:                        // 'at'
      shift(82);                    // 'at'
      lookahead1W(129);             // S^WS | '(:' | 'least' | 'most'
      switch (l1)
      {
      case 176:                     // 'least'
        shift(176);                 // 'least'
        lookahead1W(16);            // IntegerLiteral | S^WS | '(:'
        shift(8);                   // IntegerLiteral
        break;
      default:
        shift(186);                 // 'most'
        lookahead1W(16);            // IntegerLiteral | S^WS | '(:'
        shift(8);                   // IntegerLiteral
      }
      break;
    default:
      shift(142);                   // 'from'
      lookahead1W(16);              // IntegerLiteral | S^WS | '(:'
      shift(8);                     // IntegerLiteral
      lookahead1W(82);              // S^WS | '(:' | 'to'
      shift(253);                   // 'to'
      lookahead1W(16);              // IntegerLiteral | S^WS | '(:'
      shift(8);                     // IntegerLiteral
    }
    eventHandler.endNonterminal("FTLiteralRange", e0);
  }

  function try_FTLiteralRange()
  {
    switch (l1)
    {
    case 131:                       // 'exactly'
      shiftT(131);                  // 'exactly'
      lookahead1W(16);              // IntegerLiteral | S^WS | '(:'
      shiftT(8);                    // IntegerLiteral
      break;
    case 82:                        // 'at'
      shiftT(82);                   // 'at'
      lookahead1W(129);             // S^WS | '(:' | 'least' | 'most'
      switch (l1)
      {
      case 176:                     // 'least'
        shiftT(176);                // 'least'
        lookahead1W(16);            // IntegerLiteral | S^WS | '(:'
        shiftT(8);                  // IntegerLiteral
        break;
      default:
        shiftT(186);                // 'most'
        lookahead1W(16);            // IntegerLiteral | S^WS | '(:'
        shiftT(8);                  // IntegerLiteral
      }
      break;
    default:
      shiftT(142);                  // 'from'
      lookahead1W(16);              // IntegerLiteral | S^WS | '(:'
      shiftT(8);                    // IntegerLiteral
      lookahead1W(82);              // S^WS | '(:' | 'to'
      shiftT(253);                  // 'to'
      lookahead1W(16);              // IntegerLiteral | S^WS | '(:'
      shiftT(8);                    // IntegerLiteral
    }
  }

  function parse_FTStopWordOption()
  {
    eventHandler.startNonterminal("FTStopWordOption", e0);
    switch (l1)
    {
    case 244:                       // 'stop'
      shift(244);                   // 'stop'
      lookahead1W(89);              // S^WS | '(:' | 'words'
      shift(279);                   // 'words'
      lookahead1W(152);             // S^WS | '(' | '(:' | 'at' | 'default'
      switch (l1)
      {
      case 110:                     // 'default'
        shift(110);                 // 'default'
        for (;;)
        {
          lookahead1W(217);         // S^WS | EOF | '!=' | '(:' | ')' | ',' | ':' | ';' | '<' | '<<' | '<=' | '=' |
          if (l1 != 132             // 'except'
           && l1 != 260)            // 'union'
          {
            break;
          }
          whitespace();
          parse_FTStopWordsInclExcl();
        }
        break;
      default:
        whitespace();
        parse_FTStopWords();
        for (;;)
        {
          lookahead1W(217);         // S^WS | EOF | '!=' | '(:' | ')' | ',' | ':' | ';' | '<' | '<<' | '<=' | '=' |
          if (l1 != 132             // 'except'
           && l1 != 260)            // 'union'
          {
            break;
          }
          whitespace();
          parse_FTStopWordsInclExcl();
        }
      }
      break;
    default:
      shift(191);                   // 'no'
      lookahead1W(78);              // S^WS | '(:' | 'stop'
      shift(244);                   // 'stop'
      lookahead1W(89);              // S^WS | '(:' | 'words'
      shift(279);                   // 'words'
    }
    eventHandler.endNonterminal("FTStopWordOption", e0);
  }

  function try_FTStopWordOption()
  {
    switch (l1)
    {
    case 244:                       // 'stop'
      shiftT(244);                  // 'stop'
      lookahead1W(89);              // S^WS | '(:' | 'words'
      shiftT(279);                  // 'words'
      lookahead1W(152);             // S^WS | '(' | '(:' | 'at' | 'default'
      switch (l1)
      {
      case 110:                     // 'default'
        shiftT(110);                // 'default'
        for (;;)
        {
          lookahead1W(217);         // S^WS | EOF | '!=' | '(:' | ')' | ',' | ':' | ';' | '<' | '<<' | '<=' | '=' |
          if (l1 != 132             // 'except'
           && l1 != 260)            // 'union'
          {
            break;
          }
          try_FTStopWordsInclExcl();
        }
        break;
      default:
        try_FTStopWords();
        for (;;)
        {
          lookahead1W(217);         // S^WS | EOF | '!=' | '(:' | ')' | ',' | ':' | ';' | '<' | '<<' | '<=' | '=' |
          if (l1 != 132             // 'except'
           && l1 != 260)            // 'union'
          {
            break;
          }
          try_FTStopWordsInclExcl();
        }
      }
      break;
    default:
      shiftT(191);                  // 'no'
      lookahead1W(78);              // S^WS | '(:' | 'stop'
      shiftT(244);                  // 'stop'
      lookahead1W(89);              // S^WS | '(:' | 'words'
      shiftT(279);                  // 'words'
    }
  }

  function parse_FTStopWords()
  {
    eventHandler.startNonterminal("FTStopWords", e0);
    switch (l1)
    {
    case 82:                        // 'at'
      shift(82);                    // 'at'
      lookahead1W(15);              // URILiteral | S^WS | '(:'
      shift(7);                     // URILiteral
      break;
    default:
      shift(35);                    // '('
      lookahead1W(17);              // StringLiteral | S^WS | '(:'
      shift(11);                    // StringLiteral
      for (;;)
      {
        lookahead1W(105);           // S^WS | '(:' | ')' | ','
        if (l1 != 42)               // ','
        {
          break;
        }
        shift(42);                  // ','
        lookahead1W(17);            // StringLiteral | S^WS | '(:'
        shift(11);                  // StringLiteral
      }
      shift(38);                    // ')'
    }
    eventHandler.endNonterminal("FTStopWords", e0);
  }

  function try_FTStopWords()
  {
    switch (l1)
    {
    case 82:                        // 'at'
      shiftT(82);                   // 'at'
      lookahead1W(15);              // URILiteral | S^WS | '(:'
      shiftT(7);                    // URILiteral
      break;
    default:
      shiftT(35);                   // '('
      lookahead1W(17);              // StringLiteral | S^WS | '(:'
      shiftT(11);                   // StringLiteral
      for (;;)
      {
        lookahead1W(105);           // S^WS | '(:' | ')' | ','
        if (l1 != 42)               // ','
        {
          break;
        }
        shiftT(42);                 // ','
        lookahead1W(17);            // StringLiteral | S^WS | '(:'
        shiftT(11);                 // StringLiteral
      }
      shiftT(38);                   // ')'
    }
  }

  function parse_FTStopWordsInclExcl()
  {
    eventHandler.startNonterminal("FTStopWordsInclExcl", e0);
    switch (l1)
    {
    case 260:                       // 'union'
      shift(260);                   // 'union'
      break;
    default:
      shift(132);                   // 'except'
    }
    lookahead1W(103);               // S^WS | '(' | '(:' | 'at'
    whitespace();
    parse_FTStopWords();
    eventHandler.endNonterminal("FTStopWordsInclExcl", e0);
  }

  function try_FTStopWordsInclExcl()
  {
    switch (l1)
    {
    case 260:                       // 'union'
      shiftT(260);                  // 'union'
      break;
    default:
      shiftT(132);                  // 'except'
    }
    lookahead1W(103);               // S^WS | '(' | '(:' | 'at'
    try_FTStopWords();
  }

  function parse_FTLanguageOption()
  {
    eventHandler.startNonterminal("FTLanguageOption", e0);
    shift(172);                     // 'language'
    lookahead1W(17);                // StringLiteral | S^WS | '(:'
    shift(11);                      // StringLiteral
    eventHandler.endNonterminal("FTLanguageOption", e0);
  }

  function try_FTLanguageOption()
  {
    shiftT(172);                    // 'language'
    lookahead1W(17);                // StringLiteral | S^WS | '(:'
    shiftT(11);                     // StringLiteral
  }

  function parse_FTWildCardOption()
  {
    eventHandler.startNonterminal("FTWildCardOption", e0);
    switch (l1)
    {
    case 274:                       // 'wildcards'
      shift(274);                   // 'wildcards'
      break;
    default:
      shift(191);                   // 'no'
      lookahead1W(87);              // S^WS | '(:' | 'wildcards'
      shift(274);                   // 'wildcards'
    }
    eventHandler.endNonterminal("FTWildCardOption", e0);
  }

  function try_FTWildCardOption()
  {
    switch (l1)
    {
    case 274:                       // 'wildcards'
      shiftT(274);                  // 'wildcards'
      break;
    default:
      shiftT(191);                  // 'no'
      lookahead1W(87);              // S^WS | '(:' | 'wildcards'
      shiftT(274);                  // 'wildcards'
    }
  }

  function parse_FTExtensionOption()
  {
    eventHandler.startNonterminal("FTExtensionOption", e0);
    shift(203);                     // 'option'
    lookahead1W(246);               // EQName^Token | S^WS | '(:' | 'after' | 'allowing' | 'ancestor' |
    whitespace();
    parse_EQName();
    lookahead1W(17);                // StringLiteral | S^WS | '(:'
    shift(11);                      // StringLiteral
    eventHandler.endNonterminal("FTExtensionOption", e0);
  }

  function try_FTExtensionOption()
  {
    shiftT(203);                    // 'option'
    lookahead1W(246);               // EQName^Token | S^WS | '(:' | 'after' | 'allowing' | 'ancestor' |
    try_EQName();
    lookahead1W(17);                // StringLiteral | S^WS | '(:'
    shiftT(11);                     // StringLiteral
  }

  function parse_FTIgnoreOption()
  {
    eventHandler.startNonterminal("FTIgnoreOption", e0);
    shift(277);                     // 'without'
    lookahead1W(45);                // S^WS | '(:' | 'content'
    shift(101);                     // 'content'
    lookahead1W(266);               // EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral | StringLiteral |
    whitespace();
    parse_UnionExpr();
    eventHandler.endNonterminal("FTIgnoreOption", e0);
  }

  function try_FTIgnoreOption()
  {
    shiftT(277);                    // 'without'
    lookahead1W(45);                // S^WS | '(:' | 'content'
    shiftT(101);                    // 'content'
    lookahead1W(266);               // EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral | StringLiteral |
    try_UnionExpr();
  }

  function parse_CollectionDecl()
  {
    eventHandler.startNonterminal("CollectionDecl", e0);
    shift(96);                      // 'collection'
    lookahead1W(246);               // EQName^Token | S^WS | '(:' | 'after' | 'allowing' | 'ancestor' |
    whitespace();
    parse_EQName();
    lookahead1W(111);               // S^WS | '(:' | ';' | 'as'
    if (l1 == 80)                   // 'as'
    {
      whitespace();
      parse_CollectionTypeDecl();
    }
    eventHandler.endNonterminal("CollectionDecl", e0);
  }

  function parse_CollectionTypeDecl()
  {
    eventHandler.startNonterminal("CollectionTypeDecl", e0);
    shift(80);                      // 'as'
    lookahead1W(200);               // S^WS | '(:' | 'attribute' | 'comment' | 'document-node' | 'element' |
    whitespace();
    parse_KindTest();
    lookahead1W(171);               // S^WS | '(:' | '*' | '+' | ';' | '?'
    if (l1 != 54)                   // ';'
    {
      whitespace();
      parse_OccurrenceIndicator();
    }
    eventHandler.endNonterminal("CollectionTypeDecl", e0);
  }

  function parse_IndexName()
  {
    eventHandler.startNonterminal("IndexName", e0);
    parse_EQName();
    eventHandler.endNonterminal("IndexName", e0);
  }

  function parse_IndexDomainExpr()
  {
    eventHandler.startNonterminal("IndexDomainExpr", e0);
    parse_PathExpr();
    eventHandler.endNonterminal("IndexDomainExpr", e0);
  }

  function parse_IndexKeySpec()
  {
    eventHandler.startNonterminal("IndexKeySpec", e0);
    parse_IndexKeyExpr();
    if (l1 == 80)                   // 'as'
    {
      whitespace();
      parse_IndexKeyTypeDecl();
    }
    lookahead1W(156);               // S^WS | '(:' | ',' | ';' | 'collation'
    if (l1 == 95)                   // 'collation'
    {
      whitespace();
      parse_IndexKeyCollation();
    }
    eventHandler.endNonterminal("IndexKeySpec", e0);
  }

  function parse_IndexKeyExpr()
  {
    eventHandler.startNonterminal("IndexKeyExpr", e0);
    parse_PathExpr();
    eventHandler.endNonterminal("IndexKeyExpr", e0);
  }

  function parse_IndexKeyTypeDecl()
  {
    eventHandler.startNonterminal("IndexKeyTypeDecl", e0);
    shift(80);                      // 'as'
    lookahead1W(246);               // EQName^Token | S^WS | '(:' | 'after' | 'allowing' | 'ancestor' |
    whitespace();
    parse_AtomicType();
    lookahead1W(189);               // S^WS | '(:' | '*' | '+' | ',' | ';' | '?' | 'collation'
    if (l1 == 40                    // '*'
     || l1 == 41                    // '+'
     || l1 == 65)                   // '?'
    {
      whitespace();
      parse_OccurrenceIndicator();
    }
    eventHandler.endNonterminal("IndexKeyTypeDecl", e0);
  }

  function parse_AtomicType()
  {
    eventHandler.startNonterminal("AtomicType", e0);
    parse_EQName();
    eventHandler.endNonterminal("AtomicType", e0);
  }

  function parse_IndexKeyCollation()
  {
    eventHandler.startNonterminal("IndexKeyCollation", e0);
    shift(95);                      // 'collation'
    lookahead1W(15);                // URILiteral | S^WS | '(:'
    shift(7);                       // URILiteral
    eventHandler.endNonterminal("IndexKeyCollation", e0);
  }

  function parse_IndexDecl()
  {
    eventHandler.startNonterminal("IndexDecl", e0);
    shift(157);                     // 'index'
    lookahead1W(246);               // EQName^Token | S^WS | '(:' | 'after' | 'allowing' | 'ancestor' |
    whitespace();
    parse_IndexName();
    lookahead1W(68);                // S^WS | '(:' | 'on'
    shift(201);                     // 'on'
    lookahead1W(66);                // S^WS | '(:' | 'nodes'
    shift(195);                     // 'nodes'
    lookahead1W(263);               // EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral | StringLiteral |
    whitespace();
    parse_IndexDomainExpr();
    shift(88);                      // 'by'
    lookahead1W(263);               // EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral | StringLiteral |
    whitespace();
    parse_IndexKeySpec();
    for (;;)
    {
      lookahead1W(107);             // S^WS | '(:' | ',' | ';'
      if (l1 != 42)                 // ','
      {
        break;
      }
      shift(42);                    // ','
      lookahead1W(263);             // EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral | StringLiteral |
      whitespace();
      parse_IndexKeySpec();
    }
    eventHandler.endNonterminal("IndexDecl", e0);
  }

  function parse_ICDecl()
  {
    eventHandler.startNonterminal("ICDecl", e0);
    shift(163);                     // 'integrity'
    lookahead1W(43);                // S^WS | '(:' | 'constraint'
    shift(98);                      // 'constraint'
    lookahead1W(246);               // EQName^Token | S^WS | '(:' | 'after' | 'allowing' | 'ancestor' |
    whitespace();
    parse_EQName();
    lookahead1W(124);               // S^WS | '(:' | 'foreign' | 'on'
    switch (l1)
    {
    case 201:                       // 'on'
      whitespace();
      parse_ICCollection();
      break;
    default:
      whitespace();
      parse_ICForeignKey();
    }
    eventHandler.endNonterminal("ICDecl", e0);
  }

  function parse_ICCollection()
  {
    eventHandler.startNonterminal("ICCollection", e0);
    shift(201);                     // 'on'
    lookahead1W(42);                // S^WS | '(:' | 'collection'
    shift(96);                      // 'collection'
    lookahead1W(246);               // EQName^Token | S^WS | '(:' | 'after' | 'allowing' | 'ancestor' |
    whitespace();
    parse_EQName();
    lookahead1W(150);               // S^WS | '$' | '(:' | 'foreach' | 'node'
    switch (l1)
    {
    case 31:                        // '$'
      whitespace();
      parse_ICCollSequence();
      break;
    case 194:                       // 'node'
      whitespace();
      parse_ICCollSequenceUnique();
      break;
    default:
      whitespace();
      parse_ICCollNode();
    }
    eventHandler.endNonterminal("ICCollection", e0);
  }

  function parse_ICCollSequence()
  {
    eventHandler.startNonterminal("ICCollSequence", e0);
    parse_VarRef();
    lookahead1W(40);                // S^WS | '(:' | 'check'
    shift(93);                      // 'check'
    lookahead1W(267);               // EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral | StringLiteral |
    whitespace();
    parse_ExprSingle();
    eventHandler.endNonterminal("ICCollSequence", e0);
  }

  function parse_ICCollSequenceUnique()
  {
    eventHandler.startNonterminal("ICCollSequenceUnique", e0);
    shift(194);                     // 'node'
    lookahead1W(21);                // S^WS | '$' | '(:'
    whitespace();
    parse_VarRef();
    lookahead1W(40);                // S^WS | '(:' | 'check'
    shift(93);                      // 'check'
    lookahead1W(83);                // S^WS | '(:' | 'unique'
    shift(261);                     // 'unique'
    lookahead1W(60);                // S^WS | '(:' | 'key'
    shift(171);                     // 'key'
    lookahead1W(263);               // EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral | StringLiteral |
    whitespace();
    parse_PathExpr();
    eventHandler.endNonterminal("ICCollSequenceUnique", e0);
  }

  function parse_ICCollNode()
  {
    eventHandler.startNonterminal("ICCollNode", e0);
    shift(140);                     // 'foreach'
    lookahead1W(65);                // S^WS | '(:' | 'node'
    shift(194);                     // 'node'
    lookahead1W(21);                // S^WS | '$' | '(:'
    whitespace();
    parse_VarRef();
    lookahead1W(40);                // S^WS | '(:' | 'check'
    shift(93);                      // 'check'
    lookahead1W(267);               // EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral | StringLiteral |
    whitespace();
    parse_ExprSingle();
    eventHandler.endNonterminal("ICCollNode", e0);
  }

  function parse_ICForeignKey()
  {
    eventHandler.startNonterminal("ICForeignKey", e0);
    shift(141);                     // 'foreign'
    lookahead1W(60);                // S^WS | '(:' | 'key'
    shift(171);                     // 'key'
    lookahead1W(54);                // S^WS | '(:' | 'from'
    whitespace();
    parse_ICForeignKeySource();
    whitespace();
    parse_ICForeignKeyTarget();
    eventHandler.endNonterminal("ICForeignKey", e0);
  }

  function parse_ICForeignKeySource()
  {
    eventHandler.startNonterminal("ICForeignKeySource", e0);
    shift(142);                     // 'from'
    lookahead1W(42);                // S^WS | '(:' | 'collection'
    whitespace();
    parse_ICForeignKeyValues();
    eventHandler.endNonterminal("ICForeignKeySource", e0);
  }

  function parse_ICForeignKeyTarget()
  {
    eventHandler.startNonterminal("ICForeignKeyTarget", e0);
    shift(253);                     // 'to'
    lookahead1W(42);                // S^WS | '(:' | 'collection'
    whitespace();
    parse_ICForeignKeyValues();
    eventHandler.endNonterminal("ICForeignKeyTarget", e0);
  }

  function parse_ICForeignKeyValues()
  {
    eventHandler.startNonterminal("ICForeignKeyValues", e0);
    shift(96);                      // 'collection'
    lookahead1W(246);               // EQName^Token | S^WS | '(:' | 'after' | 'allowing' | 'ancestor' |
    whitespace();
    parse_EQName();
    lookahead1W(65);                // S^WS | '(:' | 'node'
    shift(194);                     // 'node'
    lookahead1W(21);                // S^WS | '$' | '(:'
    whitespace();
    parse_VarRef();
    lookahead1W(60);                // S^WS | '(:' | 'key'
    shift(171);                     // 'key'
    lookahead1W(263);               // EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral | StringLiteral |
    whitespace();
    parse_PathExpr();
    eventHandler.endNonterminal("ICForeignKeyValues", e0);
  }

  function try_Comment()
  {
    shiftT(37);                     // '(:'
    for (;;)
    {
      lookahead1(92);               // CommentContents | '(:' | ':)'
      if (l1 == 51)                 // ':)'
      {
        break;
      }
      switch (l1)
      {
      case 24:                      // CommentContents
        shiftT(24);                 // CommentContents
        break;
      default:
        try_Comment();
      }
    }
    shiftT(51);                     // ':)'
  }

  function try_Whitespace()
  {
    switch (l1)
    {
    case 22:                        // S^WS
      shiftT(22);                   // S^WS
      break;
    default:
      try_Comment();
    }
  }

  function parse_EQName()
  {
    eventHandler.startNonterminal("EQName", e0);
    lookahead1(241);                // EQName^Token | 'after' | 'allowing' | 'ancestor' | 'ancestor-or-self' | 'and' |
    switch (l1)
    {
    case 83:                        // 'attribute'
      shift(83);                    // 'attribute'
      break;
    case 97:                        // 'comment'
      shift(97);                    // 'comment'
      break;
    case 121:                       // 'document-node'
      shift(121);                   // 'document-node'
      break;
    case 122:                       // 'element'
      shift(122);                   // 'element'
      break;
    case 125:                       // 'empty-sequence'
      shift(125);                   // 'empty-sequence'
      break;
    case 147:                       // 'function'
      shift(147);                   // 'function'
      break;
    case 154:                       // 'if'
      shift(154);                   // 'if'
      break;
    case 167:                       // 'item'
      shift(167);                   // 'item'
      break;
    case 188:                       // 'namespace-node'
      shift(188);                   // 'namespace-node'
      break;
    case 194:                       // 'node'
      shift(194);                   // 'node'
      break;
    case 220:                       // 'processing-instruction'
      shift(220);                   // 'processing-instruction'
      break;
    case 230:                       // 'schema-attribute'
      shift(230);                   // 'schema-attribute'
      break;
    case 231:                       // 'schema-element'
      shift(231);                   // 'schema-element'
      break;
    case 248:                       // 'switch'
      shift(248);                   // 'switch'
      break;
    case 249:                       // 'text'
      shift(249);                   // 'text'
      break;
    case 259:                       // 'typeswitch'
      shift(259);                   // 'typeswitch'
      break;
    case 79:                        // 'array'
      shift(79);                    // 'array'
      break;
    case 169:                       // 'json-item'
      shift(169);                   // 'json-item'
      break;
    case 247:                       // 'structured-item'
      shift(247);                   // 'structured-item'
      break;
    default:
      parse_FunctionName();
    }
    eventHandler.endNonterminal("EQName", e0);
  }

  function try_EQName()
  {
    lookahead1(241);                // EQName^Token | 'after' | 'allowing' | 'ancestor' | 'ancestor-or-self' | 'and' |
    switch (l1)
    {
    case 83:                        // 'attribute'
      shiftT(83);                   // 'attribute'
      break;
    case 97:                        // 'comment'
      shiftT(97);                   // 'comment'
      break;
    case 121:                       // 'document-node'
      shiftT(121);                  // 'document-node'
      break;
    case 122:                       // 'element'
      shiftT(122);                  // 'element'
      break;
    case 125:                       // 'empty-sequence'
      shiftT(125);                  // 'empty-sequence'
      break;
    case 147:                       // 'function'
      shiftT(147);                  // 'function'
      break;
    case 154:                       // 'if'
      shiftT(154);                  // 'if'
      break;
    case 167:                       // 'item'
      shiftT(167);                  // 'item'
      break;
    case 188:                       // 'namespace-node'
      shiftT(188);                  // 'namespace-node'
      break;
    case 194:                       // 'node'
      shiftT(194);                  // 'node'
      break;
    case 220:                       // 'processing-instruction'
      shiftT(220);                  // 'processing-instruction'
      break;
    case 230:                       // 'schema-attribute'
      shiftT(230);                  // 'schema-attribute'
      break;
    case 231:                       // 'schema-element'
      shiftT(231);                  // 'schema-element'
      break;
    case 248:                       // 'switch'
      shiftT(248);                  // 'switch'
      break;
    case 249:                       // 'text'
      shiftT(249);                  // 'text'
      break;
    case 259:                       // 'typeswitch'
      shiftT(259);                  // 'typeswitch'
      break;
    case 79:                        // 'array'
      shiftT(79);                   // 'array'
      break;
    case 169:                       // 'json-item'
      shiftT(169);                  // 'json-item'
      break;
    case 247:                       // 'structured-item'
      shiftT(247);                  // 'structured-item'
      break;
    default:
      try_FunctionName();
    }
  }

  function parse_FunctionName()
  {
    eventHandler.startNonterminal("FunctionName", e0);
    switch (l1)
    {
    case 6:                         // EQName^Token
      shift(6);                     // EQName^Token
      break;
    case 71:                        // 'after'
      shift(71);                    // 'after'
      break;
    case 74:                        // 'ancestor'
      shift(74);                    // 'ancestor'
      break;
    case 75:                        // 'ancestor-or-self'
      shift(75);                    // 'ancestor-or-self'
      break;
    case 76:                        // 'and'
      shift(76);                    // 'and'
      break;
    case 80:                        // 'as'
      shift(80);                    // 'as'
      break;
    case 81:                        // 'ascending'
      shift(81);                    // 'ascending'
      break;
    case 85:                        // 'before'
      shift(85);                    // 'before'
      break;
    case 89:                        // 'case'
      shift(89);                    // 'case'
      break;
    case 90:                        // 'cast'
      shift(90);                    // 'cast'
      break;
    case 91:                        // 'castable'
      shift(91);                    // 'castable'
      break;
    case 94:                        // 'child'
      shift(94);                    // 'child'
      break;
    case 95:                        // 'collation'
      shift(95);                    // 'collation'
      break;
    case 104:                       // 'copy'
      shift(104);                   // 'copy'
      break;
    case 106:                       // 'count'
      shift(106);                   // 'count'
      break;
    case 109:                       // 'declare'
      shift(109);                   // 'declare'
      break;
    case 110:                       // 'default'
      shift(110);                   // 'default'
      break;
    case 111:                       // 'delete'
      shift(111);                   // 'delete'
      break;
    case 112:                       // 'descendant'
      shift(112);                   // 'descendant'
      break;
    case 113:                       // 'descendant-or-self'
      shift(113);                   // 'descendant-or-self'
      break;
    case 114:                       // 'descending'
      shift(114);                   // 'descending'
      break;
    case 119:                       // 'div'
      shift(119);                   // 'div'
      break;
    case 120:                       // 'document'
      shift(120);                   // 'document'
      break;
    case 123:                       // 'else'
      shift(123);                   // 'else'
      break;
    case 124:                       // 'empty'
      shift(124);                   // 'empty'
      break;
    case 127:                       // 'end'
      shift(127);                   // 'end'
      break;
    case 129:                       // 'eq'
      shift(129);                   // 'eq'
      break;
    case 130:                       // 'every'
      shift(130);                   // 'every'
      break;
    case 132:                       // 'except'
      shift(132);                   // 'except'
      break;
    case 136:                       // 'first'
      shift(136);                   // 'first'
      break;
    case 137:                       // 'following'
      shift(137);                   // 'following'
      break;
    case 138:                       // 'following-sibling'
      shift(138);                   // 'following-sibling'
      break;
    case 139:                       // 'for'
      shift(139);                   // 'for'
      break;
    case 148:                       // 'ge'
      shift(148);                   // 'ge'
      break;
    case 150:                       // 'group'
      shift(150);                   // 'group'
      break;
    case 152:                       // 'gt'
      shift(152);                   // 'gt'
      break;
    case 153:                       // 'idiv'
      shift(153);                   // 'idiv'
      break;
    case 155:                       // 'import'
      shift(155);                   // 'import'
      break;
    case 161:                       // 'insert'
      shift(161);                   // 'insert'
      break;
    case 162:                       // 'instance'
      shift(162);                   // 'instance'
      break;
    case 164:                       // 'intersect'
      shift(164);                   // 'intersect'
      break;
    case 165:                       // 'into'
      shift(165);                   // 'into'
      break;
    case 166:                       // 'is'
      shift(166);                   // 'is'
      break;
    case 173:                       // 'last'
      shift(173);                   // 'last'
      break;
    case 175:                       // 'le'
      shift(175);                   // 'le'
      break;
    case 177:                       // 'let'
      shift(177);                   // 'let'
      break;
    case 181:                       // 'lt'
      shift(181);                   // 'lt'
      break;
    case 183:                       // 'mod'
      shift(183);                   // 'mod'
      break;
    case 184:                       // 'modify'
      shift(184);                   // 'modify'
      break;
    case 185:                       // 'module'
      shift(185);                   // 'module'
      break;
    case 187:                       // 'namespace'
      shift(187);                   // 'namespace'
      break;
    case 189:                       // 'ne'
      shift(189);                   // 'ne'
      break;
    case 202:                       // 'only'
      shift(202);                   // 'only'
      break;
    case 204:                       // 'or'
      shift(204);                   // 'or'
      break;
    case 205:                       // 'order'
      shift(205);                   // 'order'
      break;
    case 206:                       // 'ordered'
      shift(206);                   // 'ordered'
      break;
    case 210:                       // 'parent'
      shift(210);                   // 'parent'
      break;
    case 216:                       // 'preceding'
      shift(216);                   // 'preceding'
      break;
    case 217:                       // 'preceding-sibling'
      shift(217);                   // 'preceding-sibling'
      break;
    case 222:                       // 'rename'
      shift(222);                   // 'rename'
      break;
    case 223:                       // 'replace'
      shift(223);                   // 'replace'
      break;
    case 224:                       // 'return'
      shift(224);                   // 'return'
      break;
    case 228:                       // 'satisfies'
      shift(228);                   // 'satisfies'
      break;
    case 234:                       // 'self'
      shift(234);                   // 'self'
      break;
    case 240:                       // 'some'
      shift(240);                   // 'some'
      break;
    case 241:                       // 'stable'
      shift(241);                   // 'stable'
      break;
    case 242:                       // 'start'
      shift(242);                   // 'start'
      break;
    case 253:                       // 'to'
      shift(253);                   // 'to'
      break;
    case 254:                       // 'treat'
      shift(254);                   // 'treat'
      break;
    case 256:                       // 'try'
      shift(256);                   // 'try'
      break;
    case 260:                       // 'union'
      shift(260);                   // 'union'
      break;
    case 262:                       // 'unordered'
      shift(262);                   // 'unordered'
      break;
    case 266:                       // 'validate'
      shift(266);                   // 'validate'
      break;
    case 272:                       // 'where'
      shift(272);                   // 'where'
      break;
    case 276:                       // 'with'
      shift(276);                   // 'with'
      break;
    case 170:                       // 'jsoniq'
      shift(170);                   // 'jsoniq'
      break;
    case 73:                        // 'allowing'
      shift(73);                    // 'allowing'
      break;
    case 82:                        // 'at'
      shift(82);                    // 'at'
      break;
    case 84:                        // 'base-uri'
      shift(84);                    // 'base-uri'
      break;
    case 86:                        // 'boundary-space'
      shift(86);                    // 'boundary-space'
      break;
    case 87:                        // 'break'
      shift(87);                    // 'break'
      break;
    case 92:                        // 'catch'
      shift(92);                    // 'catch'
      break;
    case 99:                        // 'construction'
      shift(99);                    // 'construction'
      break;
    case 102:                       // 'context'
      shift(102);                   // 'context'
      break;
    case 103:                       // 'continue'
      shift(103);                   // 'continue'
      break;
    case 105:                       // 'copy-namespaces'
      shift(105);                   // 'copy-namespaces'
      break;
    case 107:                       // 'decimal-format'
      shift(107);                   // 'decimal-format'
      break;
    case 126:                       // 'encoding'
      shift(126);                   // 'encoding'
      break;
    case 133:                       // 'exit'
      shift(133);                   // 'exit'
      break;
    case 134:                       // 'external'
      shift(134);                   // 'external'
      break;
    case 143:                       // 'ft-option'
      shift(143);                   // 'ft-option'
      break;
    case 156:                       // 'in'
      shift(156);                   // 'in'
      break;
    case 157:                       // 'index'
      shift(157);                   // 'index'
      break;
    case 163:                       // 'integrity'
      shift(163);                   // 'integrity'
      break;
    case 174:                       // 'lax'
      shift(174);                   // 'lax'
      break;
    case 195:                       // 'nodes'
      shift(195);                   // 'nodes'
      break;
    case 203:                       // 'option'
      shift(203);                   // 'option'
      break;
    case 207:                       // 'ordering'
      shift(207);                   // 'ordering'
      break;
    case 226:                       // 'revalidation'
      shift(226);                   // 'revalidation'
      break;
    case 229:                       // 'schema'
      shift(229);                   // 'schema'
      break;
    case 232:                       // 'score'
      shift(232);                   // 'score'
      break;
    case 239:                       // 'sliding'
      shift(239);                   // 'sliding'
      break;
    case 245:                       // 'strict'
      shift(245);                   // 'strict'
      break;
    case 257:                       // 'tumbling'
      shift(257);                   // 'tumbling'
      break;
    case 258:                       // 'type'
      shift(258);                   // 'type'
      break;
    case 263:                       // 'updating'
      shift(263);                   // 'updating'
      break;
    case 267:                       // 'value'
      shift(267);                   // 'value'
      break;
    case 268:                       // 'variable'
      shift(268);                   // 'variable'
      break;
    case 269:                       // 'version'
      shift(269);                   // 'version'
      break;
    case 273:                       // 'while'
      shift(273);                   // 'while'
      break;
    case 98:                        // 'constraint'
      shift(98);                    // 'constraint'
      break;
    case 179:                       // 'loop'
      shift(179);                   // 'loop'
      break;
    case 225:                       // 'returning'
      shift(225);                   // 'returning'
      break;
    case 78:                        // 'append'
      shift(78);                    // 'append'
      break;
    case 135:                       // 'false'
      shift(135);                   // 'false'
      break;
    case 142:                       // 'from'
      shift(142);                   // 'from'
      break;
    case 197:                       // 'null'
      shift(197);                   // 'null'
      break;
    case 168:                       // 'json'
      shift(168);                   // 'json'
      break;
    case 198:                       // 'object'
      shift(198);                   // 'object'
      break;
    case 233:                       // 'select'
      shift(233);                   // 'select'
      break;
    default:
      shift(255);                   // 'true'
    }
    eventHandler.endNonterminal("FunctionName", e0);
  }

  function try_FunctionName()
  {
    switch (l1)
    {
    case 6:                         // EQName^Token
      shiftT(6);                    // EQName^Token
      break;
    case 71:                        // 'after'
      shiftT(71);                   // 'after'
      break;
    case 74:                        // 'ancestor'
      shiftT(74);                   // 'ancestor'
      break;
    case 75:                        // 'ancestor-or-self'
      shiftT(75);                   // 'ancestor-or-self'
      break;
    case 76:                        // 'and'
      shiftT(76);                   // 'and'
      break;
    case 80:                        // 'as'
      shiftT(80);                   // 'as'
      break;
    case 81:                        // 'ascending'
      shiftT(81);                   // 'ascending'
      break;
    case 85:                        // 'before'
      shiftT(85);                   // 'before'
      break;
    case 89:                        // 'case'
      shiftT(89);                   // 'case'
      break;
    case 90:                        // 'cast'
      shiftT(90);                   // 'cast'
      break;
    case 91:                        // 'castable'
      shiftT(91);                   // 'castable'
      break;
    case 94:                        // 'child'
      shiftT(94);                   // 'child'
      break;
    case 95:                        // 'collation'
      shiftT(95);                   // 'collation'
      break;
    case 104:                       // 'copy'
      shiftT(104);                  // 'copy'
      break;
    case 106:                       // 'count'
      shiftT(106);                  // 'count'
      break;
    case 109:                       // 'declare'
      shiftT(109);                  // 'declare'
      break;
    case 110:                       // 'default'
      shiftT(110);                  // 'default'
      break;
    case 111:                       // 'delete'
      shiftT(111);                  // 'delete'
      break;
    case 112:                       // 'descendant'
      shiftT(112);                  // 'descendant'
      break;
    case 113:                       // 'descendant-or-self'
      shiftT(113);                  // 'descendant-or-self'
      break;
    case 114:                       // 'descending'
      shiftT(114);                  // 'descending'
      break;
    case 119:                       // 'div'
      shiftT(119);                  // 'div'
      break;
    case 120:                       // 'document'
      shiftT(120);                  // 'document'
      break;
    case 123:                       // 'else'
      shiftT(123);                  // 'else'
      break;
    case 124:                       // 'empty'
      shiftT(124);                  // 'empty'
      break;
    case 127:                       // 'end'
      shiftT(127);                  // 'end'
      break;
    case 129:                       // 'eq'
      shiftT(129);                  // 'eq'
      break;
    case 130:                       // 'every'
      shiftT(130);                  // 'every'
      break;
    case 132:                       // 'except'
      shiftT(132);                  // 'except'
      break;
    case 136:                       // 'first'
      shiftT(136);                  // 'first'
      break;
    case 137:                       // 'following'
      shiftT(137);                  // 'following'
      break;
    case 138:                       // 'following-sibling'
      shiftT(138);                  // 'following-sibling'
      break;
    case 139:                       // 'for'
      shiftT(139);                  // 'for'
      break;
    case 148:                       // 'ge'
      shiftT(148);                  // 'ge'
      break;
    case 150:                       // 'group'
      shiftT(150);                  // 'group'
      break;
    case 152:                       // 'gt'
      shiftT(152);                  // 'gt'
      break;
    case 153:                       // 'idiv'
      shiftT(153);                  // 'idiv'
      break;
    case 155:                       // 'import'
      shiftT(155);                  // 'import'
      break;
    case 161:                       // 'insert'
      shiftT(161);                  // 'insert'
      break;
    case 162:                       // 'instance'
      shiftT(162);                  // 'instance'
      break;
    case 164:                       // 'intersect'
      shiftT(164);                  // 'intersect'
      break;
    case 165:                       // 'into'
      shiftT(165);                  // 'into'
      break;
    case 166:                       // 'is'
      shiftT(166);                  // 'is'
      break;
    case 173:                       // 'last'
      shiftT(173);                  // 'last'
      break;
    case 175:                       // 'le'
      shiftT(175);                  // 'le'
      break;
    case 177:                       // 'let'
      shiftT(177);                  // 'let'
      break;
    case 181:                       // 'lt'
      shiftT(181);                  // 'lt'
      break;
    case 183:                       // 'mod'
      shiftT(183);                  // 'mod'
      break;
    case 184:                       // 'modify'
      shiftT(184);                  // 'modify'
      break;
    case 185:                       // 'module'
      shiftT(185);                  // 'module'
      break;
    case 187:                       // 'namespace'
      shiftT(187);                  // 'namespace'
      break;
    case 189:                       // 'ne'
      shiftT(189);                  // 'ne'
      break;
    case 202:                       // 'only'
      shiftT(202);                  // 'only'
      break;
    case 204:                       // 'or'
      shiftT(204);                  // 'or'
      break;
    case 205:                       // 'order'
      shiftT(205);                  // 'order'
      break;
    case 206:                       // 'ordered'
      shiftT(206);                  // 'ordered'
      break;
    case 210:                       // 'parent'
      shiftT(210);                  // 'parent'
      break;
    case 216:                       // 'preceding'
      shiftT(216);                  // 'preceding'
      break;
    case 217:                       // 'preceding-sibling'
      shiftT(217);                  // 'preceding-sibling'
      break;
    case 222:                       // 'rename'
      shiftT(222);                  // 'rename'
      break;
    case 223:                       // 'replace'
      shiftT(223);                  // 'replace'
      break;
    case 224:                       // 'return'
      shiftT(224);                  // 'return'
      break;
    case 228:                       // 'satisfies'
      shiftT(228);                  // 'satisfies'
      break;
    case 234:                       // 'self'
      shiftT(234);                  // 'self'
      break;
    case 240:                       // 'some'
      shiftT(240);                  // 'some'
      break;
    case 241:                       // 'stable'
      shiftT(241);                  // 'stable'
      break;
    case 242:                       // 'start'
      shiftT(242);                  // 'start'
      break;
    case 253:                       // 'to'
      shiftT(253);                  // 'to'
      break;
    case 254:                       // 'treat'
      shiftT(254);                  // 'treat'
      break;
    case 256:                       // 'try'
      shiftT(256);                  // 'try'
      break;
    case 260:                       // 'union'
      shiftT(260);                  // 'union'
      break;
    case 262:                       // 'unordered'
      shiftT(262);                  // 'unordered'
      break;
    case 266:                       // 'validate'
      shiftT(266);                  // 'validate'
      break;
    case 272:                       // 'where'
      shiftT(272);                  // 'where'
      break;
    case 276:                       // 'with'
      shiftT(276);                  // 'with'
      break;
    case 170:                       // 'jsoniq'
      shiftT(170);                  // 'jsoniq'
      break;
    case 73:                        // 'allowing'
      shiftT(73);                   // 'allowing'
      break;
    case 82:                        // 'at'
      shiftT(82);                   // 'at'
      break;
    case 84:                        // 'base-uri'
      shiftT(84);                   // 'base-uri'
      break;
    case 86:                        // 'boundary-space'
      shiftT(86);                   // 'boundary-space'
      break;
    case 87:                        // 'break'
      shiftT(87);                   // 'break'
      break;
    case 92:                        // 'catch'
      shiftT(92);                   // 'catch'
      break;
    case 99:                        // 'construction'
      shiftT(99);                   // 'construction'
      break;
    case 102:                       // 'context'
      shiftT(102);                  // 'context'
      break;
    case 103:                       // 'continue'
      shiftT(103);                  // 'continue'
      break;
    case 105:                       // 'copy-namespaces'
      shiftT(105);                  // 'copy-namespaces'
      break;
    case 107:                       // 'decimal-format'
      shiftT(107);                  // 'decimal-format'
      break;
    case 126:                       // 'encoding'
      shiftT(126);                  // 'encoding'
      break;
    case 133:                       // 'exit'
      shiftT(133);                  // 'exit'
      break;
    case 134:                       // 'external'
      shiftT(134);                  // 'external'
      break;
    case 143:                       // 'ft-option'
      shiftT(143);                  // 'ft-option'
      break;
    case 156:                       // 'in'
      shiftT(156);                  // 'in'
      break;
    case 157:                       // 'index'
      shiftT(157);                  // 'index'
      break;
    case 163:                       // 'integrity'
      shiftT(163);                  // 'integrity'
      break;
    case 174:                       // 'lax'
      shiftT(174);                  // 'lax'
      break;
    case 195:                       // 'nodes'
      shiftT(195);                  // 'nodes'
      break;
    case 203:                       // 'option'
      shiftT(203);                  // 'option'
      break;
    case 207:                       // 'ordering'
      shiftT(207);                  // 'ordering'
      break;
    case 226:                       // 'revalidation'
      shiftT(226);                  // 'revalidation'
      break;
    case 229:                       // 'schema'
      shiftT(229);                  // 'schema'
      break;
    case 232:                       // 'score'
      shiftT(232);                  // 'score'
      break;
    case 239:                       // 'sliding'
      shiftT(239);                  // 'sliding'
      break;
    case 245:                       // 'strict'
      shiftT(245);                  // 'strict'
      break;
    case 257:                       // 'tumbling'
      shiftT(257);                  // 'tumbling'
      break;
    case 258:                       // 'type'
      shiftT(258);                  // 'type'
      break;
    case 263:                       // 'updating'
      shiftT(263);                  // 'updating'
      break;
    case 267:                       // 'value'
      shiftT(267);                  // 'value'
      break;
    case 268:                       // 'variable'
      shiftT(268);                  // 'variable'
      break;
    case 269:                       // 'version'
      shiftT(269);                  // 'version'
      break;
    case 273:                       // 'while'
      shiftT(273);                  // 'while'
      break;
    case 98:                        // 'constraint'
      shiftT(98);                   // 'constraint'
      break;
    case 179:                       // 'loop'
      shiftT(179);                  // 'loop'
      break;
    case 225:                       // 'returning'
      shiftT(225);                  // 'returning'
      break;
    case 78:                        // 'append'
      shiftT(78);                   // 'append'
      break;
    case 135:                       // 'false'
      shiftT(135);                  // 'false'
      break;
    case 142:                       // 'from'
      shiftT(142);                  // 'from'
      break;
    case 197:                       // 'null'
      shiftT(197);                  // 'null'
      break;
    case 168:                       // 'json'
      shiftT(168);                  // 'json'
      break;
    case 198:                       // 'object'
      shiftT(198);                  // 'object'
      break;
    case 233:                       // 'select'
      shiftT(233);                  // 'select'
      break;
    default:
      shiftT(255);                  // 'true'
    }
  }

  function parse_NCName()
  {
    eventHandler.startNonterminal("NCName", e0);
    switch (l1)
    {
    case 19:                        // NCName^Token
      shift(19);                    // NCName^Token
      break;
    case 71:                        // 'after'
      shift(71);                    // 'after'
      break;
    case 76:                        // 'and'
      shift(76);                    // 'and'
      break;
    case 80:                        // 'as'
      shift(80);                    // 'as'
      break;
    case 81:                        // 'ascending'
      shift(81);                    // 'ascending'
      break;
    case 85:                        // 'before'
      shift(85);                    // 'before'
      break;
    case 89:                        // 'case'
      shift(89);                    // 'case'
      break;
    case 90:                        // 'cast'
      shift(90);                    // 'cast'
      break;
    case 91:                        // 'castable'
      shift(91);                    // 'castable'
      break;
    case 95:                        // 'collation'
      shift(95);                    // 'collation'
      break;
    case 106:                       // 'count'
      shift(106);                   // 'count'
      break;
    case 110:                       // 'default'
      shift(110);                   // 'default'
      break;
    case 114:                       // 'descending'
      shift(114);                   // 'descending'
      break;
    case 119:                       // 'div'
      shift(119);                   // 'div'
      break;
    case 123:                       // 'else'
      shift(123);                   // 'else'
      break;
    case 124:                       // 'empty'
      shift(124);                   // 'empty'
      break;
    case 127:                       // 'end'
      shift(127);                   // 'end'
      break;
    case 129:                       // 'eq'
      shift(129);                   // 'eq'
      break;
    case 132:                       // 'except'
      shift(132);                   // 'except'
      break;
    case 139:                       // 'for'
      shift(139);                   // 'for'
      break;
    case 148:                       // 'ge'
      shift(148);                   // 'ge'
      break;
    case 150:                       // 'group'
      shift(150);                   // 'group'
      break;
    case 152:                       // 'gt'
      shift(152);                   // 'gt'
      break;
    case 153:                       // 'idiv'
      shift(153);                   // 'idiv'
      break;
    case 162:                       // 'instance'
      shift(162);                   // 'instance'
      break;
    case 164:                       // 'intersect'
      shift(164);                   // 'intersect'
      break;
    case 165:                       // 'into'
      shift(165);                   // 'into'
      break;
    case 166:                       // 'is'
      shift(166);                   // 'is'
      break;
    case 175:                       // 'le'
      shift(175);                   // 'le'
      break;
    case 177:                       // 'let'
      shift(177);                   // 'let'
      break;
    case 181:                       // 'lt'
      shift(181);                   // 'lt'
      break;
    case 183:                       // 'mod'
      shift(183);                   // 'mod'
      break;
    case 184:                       // 'modify'
      shift(184);                   // 'modify'
      break;
    case 189:                       // 'ne'
      shift(189);                   // 'ne'
      break;
    case 202:                       // 'only'
      shift(202);                   // 'only'
      break;
    case 204:                       // 'or'
      shift(204);                   // 'or'
      break;
    case 205:                       // 'order'
      shift(205);                   // 'order'
      break;
    case 224:                       // 'return'
      shift(224);                   // 'return'
      break;
    case 228:                       // 'satisfies'
      shift(228);                   // 'satisfies'
      break;
    case 241:                       // 'stable'
      shift(241);                   // 'stable'
      break;
    case 242:                       // 'start'
      shift(242);                   // 'start'
      break;
    case 253:                       // 'to'
      shift(253);                   // 'to'
      break;
    case 254:                       // 'treat'
      shift(254);                   // 'treat'
      break;
    case 260:                       // 'union'
      shift(260);                   // 'union'
      break;
    case 272:                       // 'where'
      shift(272);                   // 'where'
      break;
    case 276:                       // 'with'
      shift(276);                   // 'with'
      break;
    case 74:                        // 'ancestor'
      shift(74);                    // 'ancestor'
      break;
    case 75:                        // 'ancestor-or-self'
      shift(75);                    // 'ancestor-or-self'
      break;
    case 83:                        // 'attribute'
      shift(83);                    // 'attribute'
      break;
    case 94:                        // 'child'
      shift(94);                    // 'child'
      break;
    case 97:                        // 'comment'
      shift(97);                    // 'comment'
      break;
    case 104:                       // 'copy'
      shift(104);                   // 'copy'
      break;
    case 109:                       // 'declare'
      shift(109);                   // 'declare'
      break;
    case 111:                       // 'delete'
      shift(111);                   // 'delete'
      break;
    case 112:                       // 'descendant'
      shift(112);                   // 'descendant'
      break;
    case 113:                       // 'descendant-or-self'
      shift(113);                   // 'descendant-or-self'
      break;
    case 120:                       // 'document'
      shift(120);                   // 'document'
      break;
    case 121:                       // 'document-node'
      shift(121);                   // 'document-node'
      break;
    case 122:                       // 'element'
      shift(122);                   // 'element'
      break;
    case 125:                       // 'empty-sequence'
      shift(125);                   // 'empty-sequence'
      break;
    case 130:                       // 'every'
      shift(130);                   // 'every'
      break;
    case 136:                       // 'first'
      shift(136);                   // 'first'
      break;
    case 137:                       // 'following'
      shift(137);                   // 'following'
      break;
    case 138:                       // 'following-sibling'
      shift(138);                   // 'following-sibling'
      break;
    case 147:                       // 'function'
      shift(147);                   // 'function'
      break;
    case 154:                       // 'if'
      shift(154);                   // 'if'
      break;
    case 155:                       // 'import'
      shift(155);                   // 'import'
      break;
    case 161:                       // 'insert'
      shift(161);                   // 'insert'
      break;
    case 167:                       // 'item'
      shift(167);                   // 'item'
      break;
    case 173:                       // 'last'
      shift(173);                   // 'last'
      break;
    case 185:                       // 'module'
      shift(185);                   // 'module'
      break;
    case 187:                       // 'namespace'
      shift(187);                   // 'namespace'
      break;
    case 188:                       // 'namespace-node'
      shift(188);                   // 'namespace-node'
      break;
    case 194:                       // 'node'
      shift(194);                   // 'node'
      break;
    case 206:                       // 'ordered'
      shift(206);                   // 'ordered'
      break;
    case 210:                       // 'parent'
      shift(210);                   // 'parent'
      break;
    case 216:                       // 'preceding'
      shift(216);                   // 'preceding'
      break;
    case 217:                       // 'preceding-sibling'
      shift(217);                   // 'preceding-sibling'
      break;
    case 220:                       // 'processing-instruction'
      shift(220);                   // 'processing-instruction'
      break;
    case 222:                       // 'rename'
      shift(222);                   // 'rename'
      break;
    case 223:                       // 'replace'
      shift(223);                   // 'replace'
      break;
    case 230:                       // 'schema-attribute'
      shift(230);                   // 'schema-attribute'
      break;
    case 231:                       // 'schema-element'
      shift(231);                   // 'schema-element'
      break;
    case 234:                       // 'self'
      shift(234);                   // 'self'
      break;
    case 240:                       // 'some'
      shift(240);                   // 'some'
      break;
    case 248:                       // 'switch'
      shift(248);                   // 'switch'
      break;
    case 249:                       // 'text'
      shift(249);                   // 'text'
      break;
    case 256:                       // 'try'
      shift(256);                   // 'try'
      break;
    case 259:                       // 'typeswitch'
      shift(259);                   // 'typeswitch'
      break;
    case 262:                       // 'unordered'
      shift(262);                   // 'unordered'
      break;
    case 266:                       // 'validate'
      shift(266);                   // 'validate'
      break;
    case 268:                       // 'variable'
      shift(268);                   // 'variable'
      break;
    case 170:                       // 'jsoniq'
      shift(170);                   // 'jsoniq'
      break;
    case 73:                        // 'allowing'
      shift(73);                    // 'allowing'
      break;
    case 82:                        // 'at'
      shift(82);                    // 'at'
      break;
    case 84:                        // 'base-uri'
      shift(84);                    // 'base-uri'
      break;
    case 86:                        // 'boundary-space'
      shift(86);                    // 'boundary-space'
      break;
    case 87:                        // 'break'
      shift(87);                    // 'break'
      break;
    case 92:                        // 'catch'
      shift(92);                    // 'catch'
      break;
    case 99:                        // 'construction'
      shift(99);                    // 'construction'
      break;
    case 102:                       // 'context'
      shift(102);                   // 'context'
      break;
    case 103:                       // 'continue'
      shift(103);                   // 'continue'
      break;
    case 105:                       // 'copy-namespaces'
      shift(105);                   // 'copy-namespaces'
      break;
    case 107:                       // 'decimal-format'
      shift(107);                   // 'decimal-format'
      break;
    case 126:                       // 'encoding'
      shift(126);                   // 'encoding'
      break;
    case 133:                       // 'exit'
      shift(133);                   // 'exit'
      break;
    case 134:                       // 'external'
      shift(134);                   // 'external'
      break;
    case 143:                       // 'ft-option'
      shift(143);                   // 'ft-option'
      break;
    case 156:                       // 'in'
      shift(156);                   // 'in'
      break;
    case 157:                       // 'index'
      shift(157);                   // 'index'
      break;
    case 163:                       // 'integrity'
      shift(163);                   // 'integrity'
      break;
    case 174:                       // 'lax'
      shift(174);                   // 'lax'
      break;
    case 195:                       // 'nodes'
      shift(195);                   // 'nodes'
      break;
    case 203:                       // 'option'
      shift(203);                   // 'option'
      break;
    case 207:                       // 'ordering'
      shift(207);                   // 'ordering'
      break;
    case 226:                       // 'revalidation'
      shift(226);                   // 'revalidation'
      break;
    case 229:                       // 'schema'
      shift(229);                   // 'schema'
      break;
    case 232:                       // 'score'
      shift(232);                   // 'score'
      break;
    case 239:                       // 'sliding'
      shift(239);                   // 'sliding'
      break;
    case 245:                       // 'strict'
      shift(245);                   // 'strict'
      break;
    case 257:                       // 'tumbling'
      shift(257);                   // 'tumbling'
      break;
    case 258:                       // 'type'
      shift(258);                   // 'type'
      break;
    case 263:                       // 'updating'
      shift(263);                   // 'updating'
      break;
    case 267:                       // 'value'
      shift(267);                   // 'value'
      break;
    case 269:                       // 'version'
      shift(269);                   // 'version'
      break;
    case 273:                       // 'while'
      shift(273);                   // 'while'
      break;
    case 98:                        // 'constraint'
      shift(98);                    // 'constraint'
      break;
    case 179:                       // 'loop'
      shift(179);                   // 'loop'
      break;
    case 225:                       // 'returning'
      shift(225);                   // 'returning'
      break;
    case 78:                        // 'append'
      shift(78);                    // 'append'
      break;
    case 135:                       // 'false'
      shift(135);                   // 'false'
      break;
    case 142:                       // 'from'
      shift(142);                   // 'from'
      break;
    case 197:                       // 'null'
      shift(197);                   // 'null'
      break;
    case 168:                       // 'json'
      shift(168);                   // 'json'
      break;
    case 198:                       // 'object'
      shift(198);                   // 'object'
      break;
    case 233:                       // 'select'
      shift(233);                   // 'select'
      break;
    default:
      shift(255);                   // 'true'
    }
    eventHandler.endNonterminal("NCName", e0);
  }

  function try_NCName()
  {
    switch (l1)
    {
    case 19:                        // NCName^Token
      shiftT(19);                   // NCName^Token
      break;
    case 71:                        // 'after'
      shiftT(71);                   // 'after'
      break;
    case 76:                        // 'and'
      shiftT(76);                   // 'and'
      break;
    case 80:                        // 'as'
      shiftT(80);                   // 'as'
      break;
    case 81:                        // 'ascending'
      shiftT(81);                   // 'ascending'
      break;
    case 85:                        // 'before'
      shiftT(85);                   // 'before'
      break;
    case 89:                        // 'case'
      shiftT(89);                   // 'case'
      break;
    case 90:                        // 'cast'
      shiftT(90);                   // 'cast'
      break;
    case 91:                        // 'castable'
      shiftT(91);                   // 'castable'
      break;
    case 95:                        // 'collation'
      shiftT(95);                   // 'collation'
      break;
    case 106:                       // 'count'
      shiftT(106);                  // 'count'
      break;
    case 110:                       // 'default'
      shiftT(110);                  // 'default'
      break;
    case 114:                       // 'descending'
      shiftT(114);                  // 'descending'
      break;
    case 119:                       // 'div'
      shiftT(119);                  // 'div'
      break;
    case 123:                       // 'else'
      shiftT(123);                  // 'else'
      break;
    case 124:                       // 'empty'
      shiftT(124);                  // 'empty'
      break;
    case 127:                       // 'end'
      shiftT(127);                  // 'end'
      break;
    case 129:                       // 'eq'
      shiftT(129);                  // 'eq'
      break;
    case 132:                       // 'except'
      shiftT(132);                  // 'except'
      break;
    case 139:                       // 'for'
      shiftT(139);                  // 'for'
      break;
    case 148:                       // 'ge'
      shiftT(148);                  // 'ge'
      break;
    case 150:                       // 'group'
      shiftT(150);                  // 'group'
      break;
    case 152:                       // 'gt'
      shiftT(152);                  // 'gt'
      break;
    case 153:                       // 'idiv'
      shiftT(153);                  // 'idiv'
      break;
    case 162:                       // 'instance'
      shiftT(162);                  // 'instance'
      break;
    case 164:                       // 'intersect'
      shiftT(164);                  // 'intersect'
      break;
    case 165:                       // 'into'
      shiftT(165);                  // 'into'
      break;
    case 166:                       // 'is'
      shiftT(166);                  // 'is'
      break;
    case 175:                       // 'le'
      shiftT(175);                  // 'le'
      break;
    case 177:                       // 'let'
      shiftT(177);                  // 'let'
      break;
    case 181:                       // 'lt'
      shiftT(181);                  // 'lt'
      break;
    case 183:                       // 'mod'
      shiftT(183);                  // 'mod'
      break;
    case 184:                       // 'modify'
      shiftT(184);                  // 'modify'
      break;
    case 189:                       // 'ne'
      shiftT(189);                  // 'ne'
      break;
    case 202:                       // 'only'
      shiftT(202);                  // 'only'
      break;
    case 204:                       // 'or'
      shiftT(204);                  // 'or'
      break;
    case 205:                       // 'order'
      shiftT(205);                  // 'order'
      break;
    case 224:                       // 'return'
      shiftT(224);                  // 'return'
      break;
    case 228:                       // 'satisfies'
      shiftT(228);                  // 'satisfies'
      break;
    case 241:                       // 'stable'
      shiftT(241);                  // 'stable'
      break;
    case 242:                       // 'start'
      shiftT(242);                  // 'start'
      break;
    case 253:                       // 'to'
      shiftT(253);                  // 'to'
      break;
    case 254:                       // 'treat'
      shiftT(254);                  // 'treat'
      break;
    case 260:                       // 'union'
      shiftT(260);                  // 'union'
      break;
    case 272:                       // 'where'
      shiftT(272);                  // 'where'
      break;
    case 276:                       // 'with'
      shiftT(276);                  // 'with'
      break;
    case 74:                        // 'ancestor'
      shiftT(74);                   // 'ancestor'
      break;
    case 75:                        // 'ancestor-or-self'
      shiftT(75);                   // 'ancestor-or-self'
      break;
    case 83:                        // 'attribute'
      shiftT(83);                   // 'attribute'
      break;
    case 94:                        // 'child'
      shiftT(94);                   // 'child'
      break;
    case 97:                        // 'comment'
      shiftT(97);                   // 'comment'
      break;
    case 104:                       // 'copy'
      shiftT(104);                  // 'copy'
      break;
    case 109:                       // 'declare'
      shiftT(109);                  // 'declare'
      break;
    case 111:                       // 'delete'
      shiftT(111);                  // 'delete'
      break;
    case 112:                       // 'descendant'
      shiftT(112);                  // 'descendant'
      break;
    case 113:                       // 'descendant-or-self'
      shiftT(113);                  // 'descendant-or-self'
      break;
    case 120:                       // 'document'
      shiftT(120);                  // 'document'
      break;
    case 121:                       // 'document-node'
      shiftT(121);                  // 'document-node'
      break;
    case 122:                       // 'element'
      shiftT(122);                  // 'element'
      break;
    case 125:                       // 'empty-sequence'
      shiftT(125);                  // 'empty-sequence'
      break;
    case 130:                       // 'every'
      shiftT(130);                  // 'every'
      break;
    case 136:                       // 'first'
      shiftT(136);                  // 'first'
      break;
    case 137:                       // 'following'
      shiftT(137);                  // 'following'
      break;
    case 138:                       // 'following-sibling'
      shiftT(138);                  // 'following-sibling'
      break;
    case 147:                       // 'function'
      shiftT(147);                  // 'function'
      break;
    case 154:                       // 'if'
      shiftT(154);                  // 'if'
      break;
    case 155:                       // 'import'
      shiftT(155);                  // 'import'
      break;
    case 161:                       // 'insert'
      shiftT(161);                  // 'insert'
      break;
    case 167:                       // 'item'
      shiftT(167);                  // 'item'
      break;
    case 173:                       // 'last'
      shiftT(173);                  // 'last'
      break;
    case 185:                       // 'module'
      shiftT(185);                  // 'module'
      break;
    case 187:                       // 'namespace'
      shiftT(187);                  // 'namespace'
      break;
    case 188:                       // 'namespace-node'
      shiftT(188);                  // 'namespace-node'
      break;
    case 194:                       // 'node'
      shiftT(194);                  // 'node'
      break;
    case 206:                       // 'ordered'
      shiftT(206);                  // 'ordered'
      break;
    case 210:                       // 'parent'
      shiftT(210);                  // 'parent'
      break;
    case 216:                       // 'preceding'
      shiftT(216);                  // 'preceding'
      break;
    case 217:                       // 'preceding-sibling'
      shiftT(217);                  // 'preceding-sibling'
      break;
    case 220:                       // 'processing-instruction'
      shiftT(220);                  // 'processing-instruction'
      break;
    case 222:                       // 'rename'
      shiftT(222);                  // 'rename'
      break;
    case 223:                       // 'replace'
      shiftT(223);                  // 'replace'
      break;
    case 230:                       // 'schema-attribute'
      shiftT(230);                  // 'schema-attribute'
      break;
    case 231:                       // 'schema-element'
      shiftT(231);                  // 'schema-element'
      break;
    case 234:                       // 'self'
      shiftT(234);                  // 'self'
      break;
    case 240:                       // 'some'
      shiftT(240);                  // 'some'
      break;
    case 248:                       // 'switch'
      shiftT(248);                  // 'switch'
      break;
    case 249:                       // 'text'
      shiftT(249);                  // 'text'
      break;
    case 256:                       // 'try'
      shiftT(256);                  // 'try'
      break;
    case 259:                       // 'typeswitch'
      shiftT(259);                  // 'typeswitch'
      break;
    case 262:                       // 'unordered'
      shiftT(262);                  // 'unordered'
      break;
    case 266:                       // 'validate'
      shiftT(266);                  // 'validate'
      break;
    case 268:                       // 'variable'
      shiftT(268);                  // 'variable'
      break;
    case 170:                       // 'jsoniq'
      shiftT(170);                  // 'jsoniq'
      break;
    case 73:                        // 'allowing'
      shiftT(73);                   // 'allowing'
      break;
    case 82:                        // 'at'
      shiftT(82);                   // 'at'
      break;
    case 84:                        // 'base-uri'
      shiftT(84);                   // 'base-uri'
      break;
    case 86:                        // 'boundary-space'
      shiftT(86);                   // 'boundary-space'
      break;
    case 87:                        // 'break'
      shiftT(87);                   // 'break'
      break;
    case 92:                        // 'catch'
      shiftT(92);                   // 'catch'
      break;
    case 99:                        // 'construction'
      shiftT(99);                   // 'construction'
      break;
    case 102:                       // 'context'
      shiftT(102);                  // 'context'
      break;
    case 103:                       // 'continue'
      shiftT(103);                  // 'continue'
      break;
    case 105:                       // 'copy-namespaces'
      shiftT(105);                  // 'copy-namespaces'
      break;
    case 107:                       // 'decimal-format'
      shiftT(107);                  // 'decimal-format'
      break;
    case 126:                       // 'encoding'
      shiftT(126);                  // 'encoding'
      break;
    case 133:                       // 'exit'
      shiftT(133);                  // 'exit'
      break;
    case 134:                       // 'external'
      shiftT(134);                  // 'external'
      break;
    case 143:                       // 'ft-option'
      shiftT(143);                  // 'ft-option'
      break;
    case 156:                       // 'in'
      shiftT(156);                  // 'in'
      break;
    case 157:                       // 'index'
      shiftT(157);                  // 'index'
      break;
    case 163:                       // 'integrity'
      shiftT(163);                  // 'integrity'
      break;
    case 174:                       // 'lax'
      shiftT(174);                  // 'lax'
      break;
    case 195:                       // 'nodes'
      shiftT(195);                  // 'nodes'
      break;
    case 203:                       // 'option'
      shiftT(203);                  // 'option'
      break;
    case 207:                       // 'ordering'
      shiftT(207);                  // 'ordering'
      break;
    case 226:                       // 'revalidation'
      shiftT(226);                  // 'revalidation'
      break;
    case 229:                       // 'schema'
      shiftT(229);                  // 'schema'
      break;
    case 232:                       // 'score'
      shiftT(232);                  // 'score'
      break;
    case 239:                       // 'sliding'
      shiftT(239);                  // 'sliding'
      break;
    case 245:                       // 'strict'
      shiftT(245);                  // 'strict'
      break;
    case 257:                       // 'tumbling'
      shiftT(257);                  // 'tumbling'
      break;
    case 258:                       // 'type'
      shiftT(258);                  // 'type'
      break;
    case 263:                       // 'updating'
      shiftT(263);                  // 'updating'
      break;
    case 267:                       // 'value'
      shiftT(267);                  // 'value'
      break;
    case 269:                       // 'version'
      shiftT(269);                  // 'version'
      break;
    case 273:                       // 'while'
      shiftT(273);                  // 'while'
      break;
    case 98:                        // 'constraint'
      shiftT(98);                   // 'constraint'
      break;
    case 179:                       // 'loop'
      shiftT(179);                  // 'loop'
      break;
    case 225:                       // 'returning'
      shiftT(225);                  // 'returning'
      break;
    case 78:                        // 'append'
      shiftT(78);                   // 'append'
      break;
    case 135:                       // 'false'
      shiftT(135);                  // 'false'
      break;
    case 142:                       // 'from'
      shiftT(142);                  // 'from'
      break;
    case 197:                       // 'null'
      shiftT(197);                  // 'null'
      break;
    case 168:                       // 'json'
      shiftT(168);                  // 'json'
      break;
    case 198:                       // 'object'
      shiftT(198);                  // 'object'
      break;
    case 233:                       // 'select'
      shiftT(233);                  // 'select'
      break;
    default:
      shiftT(255);                  // 'true'
    }
  }

  function parse_MainModule()
  {
    eventHandler.startNonterminal("MainModule", e0);
    parse_Prolog();
    whitespace();
    parse_Program();
    eventHandler.endNonterminal("MainModule", e0);
  }

  function parse_Program()
  {
    eventHandler.startNonterminal("Program", e0);
    parse_StatementsAndOptionalExpr();
    eventHandler.endNonterminal("Program", e0);
  }

  function parse_Statements()
  {
    eventHandler.startNonterminal("Statements", e0);
    for (;;)
    {
      lookahead1W(284);             // EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral | StringLiteral |
      switch (l1)
      {
      case 35:                      // '('
        lookahead2W(270);           // EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral | StringLiteral |
        break;
      case 36:                      // '(#'
        lookahead2(243);            // EQName^Token | S | 'after' | 'allowing' | 'ancestor' | 'ancestor-or-self' |
        break;
      case 47:                      // '/'
        lookahead2W(286);           // EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral | StringLiteral |
        break;
      case 48:                      // '//'
        lookahead2W(260);           // EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral | StringLiteral |
        break;
      case 55:                      // '<'
        lookahead2(4);              // QName
        break;
      case 56:                      // '<!--'
        lookahead2(1);              // DirCommentContents
        break;
      case 60:                      // '<?'
        lookahead2(3);              // PITarget
        break;
      case 69:                      // '['
        lookahead2W(273);           // EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral | StringLiteral |
        break;
      case 78:                      // 'append'
        lookahead2W(269);           // EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral | StringLiteral |
        break;
      case 133:                     // 'exit'
        lookahead2W(147);           // S^WS | '#' | '(' | '(:' | 'returning'
        break;
      case 139:                     // 'for'
        lookahead2W(179);           // S^WS | '#' | '$' | '(' | '(:' | 'sliding' | 'tumbling'
        break;
      case 161:                     // 'insert'
        lookahead2W(276);           // EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral | StringLiteral |
        break;
      case 177:                     // 'let'
        lookahead2W(166);           // S^WS | '#' | '$' | '(' | '(:' | 'score'
        break;
      case 187:                     // 'namespace'
        lookahead2W(247);           // NCName^Token | S^WS | '#' | '(' | '(:' | 'after' | 'allowing' | 'ancestor' |
        break;
      case 220:                     // 'processing-instruction'
        lookahead2W(245);           // NCName^Token | S^WS | '#' | '(:' | 'after' | 'allowing' | 'ancestor' |
        break;
      case 223:                     // 'replace'
        lookahead2W(170);           // S^WS | '#' | '(' | '(:' | 'node' | 'value'
        break;
      case 266:                     // 'validate'
        lookahead2W(188);           // S^WS | '#' | '(' | '(:' | 'lax' | 'strict' | 'type' | '{'
        break;
      case 281:                     // '{'
        lookahead2W(283);           // EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral | StringLiteral |
        break;
      case 283:                     // '{|'
        lookahead2W(274);           // EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral | StringLiteral |
        break;
      case 31:                      // '$'
      case 33:                      // '%'
        lookahead2W(246);           // EQName^Token | S^WS | '(:' | 'after' | 'allowing' | 'ancestor' |
        break;
      case 83:                      // 'attribute'
      case 122:                     // 'element'
        lookahead2W(253);           // EQName^Token | S^WS | '#' | '(:' | 'after' | 'allowing' | 'ancestor' |
        break;
      case 87:                      // 'break'
      case 103:                     // 'continue'
        lookahead2W(145);           // S^WS | '#' | '(' | '(:' | 'loop'
        break;
      case 97:                      // 'comment'
      case 249:                     // 'text'
        lookahead2W(97);            // S^WS | '#' | '(:' | '{'
        break;
      case 111:                     // 'delete'
      case 222:                     // 'rename'
        lookahead2W(261);           // EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral | StringLiteral |
        break;
      case 41:                      // '+'
      case 43:                      // '-'
      case 196:                     // 'not'
        lookahead2W(266);           // EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral | StringLiteral |
        break;
      case 135:                     // 'false'
      case 197:                     // 'null'
      case 255:                     // 'true'
        lookahead2W(211);           // S^WS | EOF | '!' | '!=' | '#' | '(' | '(:' | '*' | '+' | ',' | '-' | '.' | '/' |
        break;
      case 104:                     // 'copy'
      case 130:                     // 'every'
      case 240:                     // 'some'
      case 268:                     // 'variable'
        lookahead2W(143);           // S^WS | '#' | '$' | '(' | '(:'
        break;
      case 120:                     // 'document'
      case 206:                     // 'ordered'
      case 256:                     // 'try'
      case 262:                     // 'unordered'
        lookahead2W(148);           // S^WS | '#' | '(' | '(:' | '{'
        break;
      case 8:                       // IntegerLiteral
      case 9:                       // DecimalLiteral
      case 10:                      // DoubleLiteral
      case 11:                      // StringLiteral
      case 32:                      // '$$'
        lookahead2W(210);           // S^WS | EOF | '!' | '!=' | '(' | '(:' | '*' | '+' | ',' | '-' | '.' | '/' | '//' |
        break;
      case 79:                      // 'array'
      case 121:                     // 'document-node'
      case 125:                     // 'empty-sequence'
      case 167:                     // 'item'
      case 169:                     // 'json-item'
      case 188:                     // 'namespace-node'
      case 194:                     // 'node'
      case 230:                     // 'schema-attribute'
      case 231:                     // 'schema-element'
      case 247:                     // 'structured-item'
        lookahead2W(20);            // S^WS | '#' | '(:'
        break;
      case 6:                       // EQName^Token
      case 71:                      // 'after'
      case 73:                      // 'allowing'
      case 74:                      // 'ancestor'
      case 75:                      // 'ancestor-or-self'
      case 76:                      // 'and'
      case 80:                      // 'as'
      case 81:                      // 'ascending'
      case 82:                      // 'at'
      case 84:                      // 'base-uri'
      case 85:                      // 'before'
      case 86:                      // 'boundary-space'
      case 89:                      // 'case'
      case 90:                      // 'cast'
      case 91:                      // 'castable'
      case 92:                      // 'catch'
      case 94:                      // 'child'
      case 95:                      // 'collation'
      case 98:                      // 'constraint'
      case 99:                      // 'construction'
      case 102:                     // 'context'
      case 105:                     // 'copy-namespaces'
      case 106:                     // 'count'
      case 107:                     // 'decimal-format'
      case 109:                     // 'declare'
      case 110:                     // 'default'
      case 112:                     // 'descendant'
      case 113:                     // 'descendant-or-self'
      case 114:                     // 'descending'
      case 119:                     // 'div'
      case 123:                     // 'else'
      case 124:                     // 'empty'
      case 126:                     // 'encoding'
      case 127:                     // 'end'
      case 129:                     // 'eq'
      case 132:                     // 'except'
      case 134:                     // 'external'
      case 136:                     // 'first'
      case 137:                     // 'following'
      case 138:                     // 'following-sibling'
      case 142:                     // 'from'
      case 143:                     // 'ft-option'
      case 147:                     // 'function'
      case 148:                     // 'ge'
      case 150:                     // 'group'
      case 152:                     // 'gt'
      case 153:                     // 'idiv'
      case 154:                     // 'if'
      case 155:                     // 'import'
      case 156:                     // 'in'
      case 157:                     // 'index'
      case 162:                     // 'instance'
      case 163:                     // 'integrity'
      case 164:                     // 'intersect'
      case 165:                     // 'into'
      case 166:                     // 'is'
      case 168:                     // 'json'
      case 170:                     // 'jsoniq'
      case 173:                     // 'last'
      case 174:                     // 'lax'
      case 175:                     // 'le'
      case 179:                     // 'loop'
      case 181:                     // 'lt'
      case 183:                     // 'mod'
      case 184:                     // 'modify'
      case 185:                     // 'module'
      case 189:                     // 'ne'
      case 195:                     // 'nodes'
      case 198:                     // 'object'
      case 202:                     // 'only'
      case 203:                     // 'option'
      case 204:                     // 'or'
      case 205:                     // 'order'
      case 207:                     // 'ordering'
      case 210:                     // 'parent'
      case 216:                     // 'preceding'
      case 217:                     // 'preceding-sibling'
      case 224:                     // 'return'
      case 225:                     // 'returning'
      case 226:                     // 'revalidation'
      case 228:                     // 'satisfies'
      case 229:                     // 'schema'
      case 232:                     // 'score'
      case 233:                     // 'select'
      case 234:                     // 'self'
      case 239:                     // 'sliding'
      case 241:                     // 'stable'
      case 242:                     // 'start'
      case 245:                     // 'strict'
      case 248:                     // 'switch'
      case 253:                     // 'to'
      case 254:                     // 'treat'
      case 257:                     // 'tumbling'
      case 258:                     // 'type'
      case 259:                     // 'typeswitch'
      case 260:                     // 'union'
      case 263:                     // 'updating'
      case 267:                     // 'value'
      case 269:                     // 'version'
      case 272:                     // 'where'
      case 273:                     // 'while'
      case 276:                     // 'with'
        lookahead2W(95);            // S^WS | '#' | '(' | '(:'
        break;
      default:
        lk = l1;
      }
      if (lk != 25                  // EOF
       && lk != 54                  // ';'
       && lk != 287                 // '}'
       && lk != 12808               // IntegerLiteral EOF
       && lk != 12809               // DecimalLiteral EOF
       && lk != 12810               // DoubleLiteral EOF
       && lk != 12811               // StringLiteral EOF
       && lk != 12832               // '$$' EOF
       && lk != 12847               // '/' EOF
       && lk != 12935               // 'false' EOF
       && lk != 12997               // 'null' EOF
       && lk != 13055               // 'true' EOF
       && lk != 16140               // 'variable' '$'
       && lk != 21512               // IntegerLiteral ','
       && lk != 21513               // DecimalLiteral ','
       && lk != 21514               // DoubleLiteral ','
       && lk != 21515               // StringLiteral ','
       && lk != 21536               // '$$' ','
       && lk != 21551               // '/' ','
       && lk != 21639               // 'false' ','
       && lk != 21701               // 'null' ','
       && lk != 21759               // 'true' ','
       && lk != 27656               // IntegerLiteral ';'
       && lk != 27657               // DecimalLiteral ';'
       && lk != 27658               // DoubleLiteral ';'
       && lk != 27659               // StringLiteral ';'
       && lk != 27680               // '$$' ';'
       && lk != 27695               // '/' ';'
       && lk != 27783               // 'false' ';'
       && lk != 27845               // 'null' ';'
       && lk != 27903               // 'true' ';'
       && lk != 91735               // 'break' 'loop'
       && lk != 91751               // 'continue' 'loop'
       && lk != 115333              // 'exit' 'returning'
       && lk != 146952              // IntegerLiteral '}'
       && lk != 146953              // DecimalLiteral '}'
       && lk != 146954              // DoubleLiteral '}'
       && lk != 146955              // StringLiteral '}'
       && lk != 146976              // '$$' '}'
       && lk != 146991              // '/' '}'
       && lk != 147079              // 'false' '}'
       && lk != 147141              // 'null' '}'
       && lk != 147199)             // 'true' '}'
      {
        lk = memoized(8, e0);
        if (lk == 0)
        {
          var b0A = b0; var e0A = e0; var l1A = l1;
          var b1A = b1; var e1A = e1; var l2A = l2;
          var b2A = b2; var e2A = e2;
          try
          {
            try_Statement();
            lk = -1;
          }
          catch (p1A)
          {
            lk = -2;
          }
          b0 = b0A; e0 = e0A; l1 = l1A; if (l1 == 0) {end = e0A;} else {
          b1 = b1A; e1 = e1A; l2 = l2A; if (l2 == 0) {end = e1A;} else {
          b2 = b2A; e2 = e2A; end = e2A; }}
          memoize(8, e0, lk);
        }
      }
      if (lk != -1
       && lk != 54                  // ';'
       && lk != 16140               // 'variable' '$'
       && lk != 27656               // IntegerLiteral ';'
       && lk != 27657               // DecimalLiteral ';'
       && lk != 27658               // DoubleLiteral ';'
       && lk != 27659               // StringLiteral ';'
       && lk != 27680               // '$$' ';'
       && lk != 27695               // '/' ';'
       && lk != 27783               // 'false' ';'
       && lk != 27845               // 'null' ';'
       && lk != 27903               // 'true' ';'
       && lk != 91735               // 'break' 'loop'
       && lk != 91751               // 'continue' 'loop'
       && lk != 115333)             // 'exit' 'returning'
      {
        break;
      }
      whitespace();
      parse_Statement();
    }
    eventHandler.endNonterminal("Statements", e0);
  }

  function try_Statements()
  {
    for (;;)
    {
      lookahead1W(284);             // EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral | StringLiteral |
      switch (l1)
      {
      case 35:                      // '('
        lookahead2W(270);           // EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral | StringLiteral |
        break;
      case 36:                      // '(#'
        lookahead2(243);            // EQName^Token | S | 'after' | 'allowing' | 'ancestor' | 'ancestor-or-self' |
        break;
      case 47:                      // '/'
        lookahead2W(286);           // EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral | StringLiteral |
        break;
      case 48:                      // '//'
        lookahead2W(260);           // EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral | StringLiteral |
        break;
      case 55:                      // '<'
        lookahead2(4);              // QName
        break;
      case 56:                      // '<!--'
        lookahead2(1);              // DirCommentContents
        break;
      case 60:                      // '<?'
        lookahead2(3);              // PITarget
        break;
      case 69:                      // '['
        lookahead2W(273);           // EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral | StringLiteral |
        break;
      case 78:                      // 'append'
        lookahead2W(269);           // EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral | StringLiteral |
        break;
      case 133:                     // 'exit'
        lookahead2W(147);           // S^WS | '#' | '(' | '(:' | 'returning'
        break;
      case 139:                     // 'for'
        lookahead2W(179);           // S^WS | '#' | '$' | '(' | '(:' | 'sliding' | 'tumbling'
        break;
      case 161:                     // 'insert'
        lookahead2W(276);           // EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral | StringLiteral |
        break;
      case 177:                     // 'let'
        lookahead2W(166);           // S^WS | '#' | '$' | '(' | '(:' | 'score'
        break;
      case 187:                     // 'namespace'
        lookahead2W(247);           // NCName^Token | S^WS | '#' | '(' | '(:' | 'after' | 'allowing' | 'ancestor' |
        break;
      case 220:                     // 'processing-instruction'
        lookahead2W(245);           // NCName^Token | S^WS | '#' | '(:' | 'after' | 'allowing' | 'ancestor' |
        break;
      case 223:                     // 'replace'
        lookahead2W(170);           // S^WS | '#' | '(' | '(:' | 'node' | 'value'
        break;
      case 266:                     // 'validate'
        lookahead2W(188);           // S^WS | '#' | '(' | '(:' | 'lax' | 'strict' | 'type' | '{'
        break;
      case 281:                     // '{'
        lookahead2W(283);           // EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral | StringLiteral |
        break;
      case 283:                     // '{|'
        lookahead2W(274);           // EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral | StringLiteral |
        break;
      case 31:                      // '$'
      case 33:                      // '%'
        lookahead2W(246);           // EQName^Token | S^WS | '(:' | 'after' | 'allowing' | 'ancestor' |
        break;
      case 83:                      // 'attribute'
      case 122:                     // 'element'
        lookahead2W(253);           // EQName^Token | S^WS | '#' | '(:' | 'after' | 'allowing' | 'ancestor' |
        break;
      case 87:                      // 'break'
      case 103:                     // 'continue'
        lookahead2W(145);           // S^WS | '#' | '(' | '(:' | 'loop'
        break;
      case 97:                      // 'comment'
      case 249:                     // 'text'
        lookahead2W(97);            // S^WS | '#' | '(:' | '{'
        break;
      case 111:                     // 'delete'
      case 222:                     // 'rename'
        lookahead2W(261);           // EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral | StringLiteral |
        break;
      case 41:                      // '+'
      case 43:                      // '-'
      case 196:                     // 'not'
        lookahead2W(266);           // EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral | StringLiteral |
        break;
      case 135:                     // 'false'
      case 197:                     // 'null'
      case 255:                     // 'true'
        lookahead2W(211);           // S^WS | EOF | '!' | '!=' | '#' | '(' | '(:' | '*' | '+' | ',' | '-' | '.' | '/' |
        break;
      case 104:                     // 'copy'
      case 130:                     // 'every'
      case 240:                     // 'some'
      case 268:                     // 'variable'
        lookahead2W(143);           // S^WS | '#' | '$' | '(' | '(:'
        break;
      case 120:                     // 'document'
      case 206:                     // 'ordered'
      case 256:                     // 'try'
      case 262:                     // 'unordered'
        lookahead2W(148);           // S^WS | '#' | '(' | '(:' | '{'
        break;
      case 8:                       // IntegerLiteral
      case 9:                       // DecimalLiteral
      case 10:                      // DoubleLiteral
      case 11:                      // StringLiteral
      case 32:                      // '$$'
        lookahead2W(210);           // S^WS | EOF | '!' | '!=' | '(' | '(:' | '*' | '+' | ',' | '-' | '.' | '/' | '//' |
        break;
      case 79:                      // 'array'
      case 121:                     // 'document-node'
      case 125:                     // 'empty-sequence'
      case 167:                     // 'item'
      case 169:                     // 'json-item'
      case 188:                     // 'namespace-node'
      case 194:                     // 'node'
      case 230:                     // 'schema-attribute'
      case 231:                     // 'schema-element'
      case 247:                     // 'structured-item'
        lookahead2W(20);            // S^WS | '#' | '(:'
        break;
      case 6:                       // EQName^Token
      case 71:                      // 'after'
      case 73:                      // 'allowing'
      case 74:                      // 'ancestor'
      case 75:                      // 'ancestor-or-self'
      case 76:                      // 'and'
      case 80:                      // 'as'
      case 81:                      // 'ascending'
      case 82:                      // 'at'
      case 84:                      // 'base-uri'
      case 85:                      // 'before'
      case 86:                      // 'boundary-space'
      case 89:                      // 'case'
      case 90:                      // 'cast'
      case 91:                      // 'castable'
      case 92:                      // 'catch'
      case 94:                      // 'child'
      case 95:                      // 'collation'
      case 98:                      // 'constraint'
      case 99:                      // 'construction'
      case 102:                     // 'context'
      case 105:                     // 'copy-namespaces'
      case 106:                     // 'count'
      case 107:                     // 'decimal-format'
      case 109:                     // 'declare'
      case 110:                     // 'default'
      case 112:                     // 'descendant'
      case 113:                     // 'descendant-or-self'
      case 114:                     // 'descending'
      case 119:                     // 'div'
      case 123:                     // 'else'
      case 124:                     // 'empty'
      case 126:                     // 'encoding'
      case 127:                     // 'end'
      case 129:                     // 'eq'
      case 132:                     // 'except'
      case 134:                     // 'external'
      case 136:                     // 'first'
      case 137:                     // 'following'
      case 138:                     // 'following-sibling'
      case 142:                     // 'from'
      case 143:                     // 'ft-option'
      case 147:                     // 'function'
      case 148:                     // 'ge'
      case 150:                     // 'group'
      case 152:                     // 'gt'
      case 153:                     // 'idiv'
      case 154:                     // 'if'
      case 155:                     // 'import'
      case 156:                     // 'in'
      case 157:                     // 'index'
      case 162:                     // 'instance'
      case 163:                     // 'integrity'
      case 164:                     // 'intersect'
      case 165:                     // 'into'
      case 166:                     // 'is'
      case 168:                     // 'json'
      case 170:                     // 'jsoniq'
      case 173:                     // 'last'
      case 174:                     // 'lax'
      case 175:                     // 'le'
      case 179:                     // 'loop'
      case 181:                     // 'lt'
      case 183:                     // 'mod'
      case 184:                     // 'modify'
      case 185:                     // 'module'
      case 189:                     // 'ne'
      case 195:                     // 'nodes'
      case 198:                     // 'object'
      case 202:                     // 'only'
      case 203:                     // 'option'
      case 204:                     // 'or'
      case 205:                     // 'order'
      case 207:                     // 'ordering'
      case 210:                     // 'parent'
      case 216:                     // 'preceding'
      case 217:                     // 'preceding-sibling'
      case 224:                     // 'return'
      case 225:                     // 'returning'
      case 226:                     // 'revalidation'
      case 228:                     // 'satisfies'
      case 229:                     // 'schema'
      case 232:                     // 'score'
      case 233:                     // 'select'
      case 234:                     // 'self'
      case 239:                     // 'sliding'
      case 241:                     // 'stable'
      case 242:                     // 'start'
      case 245:                     // 'strict'
      case 248:                     // 'switch'
      case 253:                     // 'to'
      case 254:                     // 'treat'
      case 257:                     // 'tumbling'
      case 258:                     // 'type'
      case 259:                     // 'typeswitch'
      case 260:                     // 'union'
      case 263:                     // 'updating'
      case 267:                     // 'value'
      case 269:                     // 'version'
      case 272:                     // 'where'
      case 273:                     // 'while'
      case 276:                     // 'with'
        lookahead2W(95);            // S^WS | '#' | '(' | '(:'
        break;
      default:
        lk = l1;
      }
      if (lk != 25                  // EOF
       && lk != 54                  // ';'
       && lk != 287                 // '}'
       && lk != 12808               // IntegerLiteral EOF
       && lk != 12809               // DecimalLiteral EOF
       && lk != 12810               // DoubleLiteral EOF
       && lk != 12811               // StringLiteral EOF
       && lk != 12832               // '$$' EOF
       && lk != 12847               // '/' EOF
       && lk != 12935               // 'false' EOF
       && lk != 12997               // 'null' EOF
       && lk != 13055               // 'true' EOF
       && lk != 16140               // 'variable' '$'
       && lk != 21512               // IntegerLiteral ','
       && lk != 21513               // DecimalLiteral ','
       && lk != 21514               // DoubleLiteral ','
       && lk != 21515               // StringLiteral ','
       && lk != 21536               // '$$' ','
       && lk != 21551               // '/' ','
       && lk != 21639               // 'false' ','
       && lk != 21701               // 'null' ','
       && lk != 21759               // 'true' ','
       && lk != 27656               // IntegerLiteral ';'
       && lk != 27657               // DecimalLiteral ';'
       && lk != 27658               // DoubleLiteral ';'
       && lk != 27659               // StringLiteral ';'
       && lk != 27680               // '$$' ';'
       && lk != 27695               // '/' ';'
       && lk != 27783               // 'false' ';'
       && lk != 27845               // 'null' ';'
       && lk != 27903               // 'true' ';'
       && lk != 91735               // 'break' 'loop'
       && lk != 91751               // 'continue' 'loop'
       && lk != 115333              // 'exit' 'returning'
       && lk != 146952              // IntegerLiteral '}'
       && lk != 146953              // DecimalLiteral '}'
       && lk != 146954              // DoubleLiteral '}'
       && lk != 146955              // StringLiteral '}'
       && lk != 146976              // '$$' '}'
       && lk != 146991              // '/' '}'
       && lk != 147079              // 'false' '}'
       && lk != 147141              // 'null' '}'
       && lk != 147199)             // 'true' '}'
      {
        lk = memoized(8, e0);
        if (lk == 0)
        {
          var b0A = b0; var e0A = e0; var l1A = l1;
          var b1A = b1; var e1A = e1; var l2A = l2;
          var b2A = b2; var e2A = e2;
          try
          {
            try_Statement();
            memoize(8, e0A, -1);
            continue;
          }
          catch (p1A)
          {
            b0 = b0A; e0 = e0A; l1 = l1A; if (l1 == 0) {end = e0A;} else {
            b1 = b1A; e1 = e1A; l2 = l2A; if (l2 == 0) {end = e1A;} else {
            b2 = b2A; e2 = e2A; end = e2A; }}
            memoize(8, e0A, -2);
            break;
          }
        }
      }
      if (lk != -1
       && lk != 54                  // ';'
       && lk != 16140               // 'variable' '$'
       && lk != 27656               // IntegerLiteral ';'
       && lk != 27657               // DecimalLiteral ';'
       && lk != 27658               // DoubleLiteral ';'
       && lk != 27659               // StringLiteral ';'
       && lk != 27680               // '$$' ';'
       && lk != 27695               // '/' ';'
       && lk != 27783               // 'false' ';'
       && lk != 27845               // 'null' ';'
       && lk != 27903               // 'true' ';'
       && lk != 91735               // 'break' 'loop'
       && lk != 91751               // 'continue' 'loop'
       && lk != 115333)             // 'exit' 'returning'
      {
        break;
      }
      try_Statement();
    }
  }

  function parse_StatementsAndExpr()
  {
    eventHandler.startNonterminal("StatementsAndExpr", e0);
    parse_Statements();
    whitespace();
    parse_Expr();
    eventHandler.endNonterminal("StatementsAndExpr", e0);
  }

  function try_StatementsAndExpr()
  {
    try_Statements();
    try_Expr();
  }

  function parse_StatementsAndOptionalExpr()
  {
    eventHandler.startNonterminal("StatementsAndOptionalExpr", e0);
    parse_Statements();
    if (l1 != 25                    // EOF
     && l1 != 287)                  // '}'
    {
      whitespace();
      parse_Expr();
    }
    eventHandler.endNonterminal("StatementsAndOptionalExpr", e0);
  }

  function try_StatementsAndOptionalExpr()
  {
    try_Statements();
    if (l1 != 25                    // EOF
     && l1 != 287)                  // '}'
    {
      try_Expr();
    }
  }

  function parse_Statement()
  {
    eventHandler.startNonterminal("Statement", e0);
    switch (l1)
    {
    case 133:                       // 'exit'
      lookahead2W(147);             // S^WS | '#' | '(' | '(:' | 'returning'
      break;
    case 139:                       // 'for'
      lookahead2W(179);             // S^WS | '#' | '$' | '(' | '(:' | 'sliding' | 'tumbling'
      break;
    case 177:                       // 'let'
      lookahead2W(166);             // S^WS | '#' | '$' | '(' | '(:' | 'score'
      break;
    case 256:                       // 'try'
      lookahead2W(148);             // S^WS | '#' | '(' | '(:' | '{'
      break;
    case 268:                       // 'variable'
      lookahead2W(143);             // S^WS | '#' | '$' | '(' | '(:'
      break;
    case 281:                       // '{'
      lookahead2W(283);             // EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral | StringLiteral |
      break;
    case 31:                        // '$'
    case 33:                        // '%'
      lookahead2W(246);             // EQName^Token | S^WS | '(:' | 'after' | 'allowing' | 'ancestor' |
      break;
    case 87:                        // 'break'
    case 103:                       // 'continue'
      lookahead2W(145);             // S^WS | '#' | '(' | '(:' | 'loop'
      break;
    case 154:                       // 'if'
    case 248:                       // 'switch'
    case 259:                       // 'typeswitch'
    case 273:                       // 'while'
      lookahead2W(95);              // S^WS | '#' | '(' | '(:'
      break;
    default:
      lk = l1;
    }
    if (lk != 6                     // EQName^Token
     && lk != 8                     // IntegerLiteral
     && lk != 9                     // DecimalLiteral
     && lk != 10                    // DoubleLiteral
     && lk != 11                    // StringLiteral
     && lk != 32                    // '$$'
     && lk != 35                    // '('
     && lk != 36                    // '(#'
     && lk != 41                    // '+'
     && lk != 43                    // '-'
     && lk != 47                    // '/'
     && lk != 48                    // '//'
     && lk != 54                    // ';'
     && lk != 55                    // '<'
     && lk != 56                    // '<!--'
     && lk != 60                    // '<?'
     && lk != 69                    // '['
     && lk != 71                    // 'after'
     && lk != 73                    // 'allowing'
     && lk != 74                    // 'ancestor'
     && lk != 75                    // 'ancestor-or-self'
     && lk != 76                    // 'and'
     && lk != 78                    // 'append'
     && lk != 79                    // 'array'
     && lk != 80                    // 'as'
     && lk != 81                    // 'ascending'
     && lk != 82                    // 'at'
     && lk != 83                    // 'attribute'
     && lk != 84                    // 'base-uri'
     && lk != 85                    // 'before'
     && lk != 86                    // 'boundary-space'
     && lk != 89                    // 'case'
     && lk != 90                    // 'cast'
     && lk != 91                    // 'castable'
     && lk != 92                    // 'catch'
     && lk != 94                    // 'child'
     && lk != 95                    // 'collation'
     && lk != 97                    // 'comment'
     && lk != 98                    // 'constraint'
     && lk != 99                    // 'construction'
     && lk != 102                   // 'context'
     && lk != 104                   // 'copy'
     && lk != 105                   // 'copy-namespaces'
     && lk != 106                   // 'count'
     && lk != 107                   // 'decimal-format'
     && lk != 109                   // 'declare'
     && lk != 110                   // 'default'
     && lk != 111                   // 'delete'
     && lk != 112                   // 'descendant'
     && lk != 113                   // 'descendant-or-self'
     && lk != 114                   // 'descending'
     && lk != 119                   // 'div'
     && lk != 120                   // 'document'
     && lk != 121                   // 'document-node'
     && lk != 122                   // 'element'
     && lk != 123                   // 'else'
     && lk != 124                   // 'empty'
     && lk != 125                   // 'empty-sequence'
     && lk != 126                   // 'encoding'
     && lk != 127                   // 'end'
     && lk != 129                   // 'eq'
     && lk != 130                   // 'every'
     && lk != 132                   // 'except'
     && lk != 134                   // 'external'
     && lk != 135                   // 'false'
     && lk != 136                   // 'first'
     && lk != 137                   // 'following'
     && lk != 138                   // 'following-sibling'
     && lk != 142                   // 'from'
     && lk != 143                   // 'ft-option'
     && lk != 147                   // 'function'
     && lk != 148                   // 'ge'
     && lk != 150                   // 'group'
     && lk != 152                   // 'gt'
     && lk != 153                   // 'idiv'
     && lk != 155                   // 'import'
     && lk != 156                   // 'in'
     && lk != 157                   // 'index'
     && lk != 161                   // 'insert'
     && lk != 162                   // 'instance'
     && lk != 163                   // 'integrity'
     && lk != 164                   // 'intersect'
     && lk != 165                   // 'into'
     && lk != 166                   // 'is'
     && lk != 167                   // 'item'
     && lk != 168                   // 'json'
     && lk != 169                   // 'json-item'
     && lk != 170                   // 'jsoniq'
     && lk != 173                   // 'last'
     && lk != 174                   // 'lax'
     && lk != 175                   // 'le'
     && lk != 179                   // 'loop'
     && lk != 181                   // 'lt'
     && lk != 183                   // 'mod'
     && lk != 184                   // 'modify'
     && lk != 185                   // 'module'
     && lk != 187                   // 'namespace'
     && lk != 188                   // 'namespace-node'
     && lk != 189                   // 'ne'
     && lk != 194                   // 'node'
     && lk != 195                   // 'nodes'
     && lk != 196                   // 'not'
     && lk != 197                   // 'null'
     && lk != 198                   // 'object'
     && lk != 202                   // 'only'
     && lk != 203                   // 'option'
     && lk != 204                   // 'or'
     && lk != 205                   // 'order'
     && lk != 206                   // 'ordered'
     && lk != 207                   // 'ordering'
     && lk != 210                   // 'parent'
     && lk != 216                   // 'preceding'
     && lk != 217                   // 'preceding-sibling'
     && lk != 220                   // 'processing-instruction'
     && lk != 222                   // 'rename'
     && lk != 223                   // 'replace'
     && lk != 224                   // 'return'
     && lk != 225                   // 'returning'
     && lk != 226                   // 'revalidation'
     && lk != 228                   // 'satisfies'
     && lk != 229                   // 'schema'
     && lk != 230                   // 'schema-attribute'
     && lk != 231                   // 'schema-element'
     && lk != 232                   // 'score'
     && lk != 233                   // 'select'
     && lk != 234                   // 'self'
     && lk != 239                   // 'sliding'
     && lk != 240                   // 'some'
     && lk != 241                   // 'stable'
     && lk != 242                   // 'start'
     && lk != 245                   // 'strict'
     && lk != 247                   // 'structured-item'
     && lk != 249                   // 'text'
     && lk != 253                   // 'to'
     && lk != 254                   // 'treat'
     && lk != 255                   // 'true'
     && lk != 257                   // 'tumbling'
     && lk != 258                   // 'type'
     && lk != 260                   // 'union'
     && lk != 262                   // 'unordered'
     && lk != 263                   // 'updating'
     && lk != 266                   // 'validate'
     && lk != 267                   // 'value'
     && lk != 269                   // 'version'
     && lk != 272                   // 'where'
     && lk != 276                   // 'with'
     && lk != 283                   // '{|'
     && lk != 10009                 // '{' NCName^Token
     && lk != 14935                 // 'break' '#'
     && lk != 14951                 // 'continue' '#'
     && lk != 14981                 // 'exit' '#'
     && lk != 14987                 // 'for' '#'
     && lk != 15002                 // 'if' '#'
     && lk != 15025                 // 'let' '#'
     && lk != 15096                 // 'switch' '#'
     && lk != 15104                 // 'try' '#'
     && lk != 15107                 // 'typeswitch' '#'
     && lk != 15116                 // 'variable' '#'
     && lk != 15121                 // 'while' '#'
     && lk != 16011                 // 'for' '$'
     && lk != 16049                 // 'let' '$'
     && lk != 16140                 // 'variable' '$'
     && lk != 18007                 // 'break' '('
     && lk != 18023                 // 'continue' '('
     && lk != 18053                 // 'exit' '('
     && lk != 18059                 // 'for' '('
     && lk != 18074                 // 'if' '('
     && lk != 18097                 // 'let' '('
     && lk != 18168                 // 'switch' '('
     && lk != 18176                 // 'try' '('
     && lk != 18179                 // 'typeswitch' '('
     && lk != 18188                 // 'variable' '('
     && lk != 91735                 // 'break' 'loop'
     && lk != 91751                 // 'continue' 'loop'
     && lk != 115333                // 'exit' 'returning'
     && lk != 118961                // 'let' 'score'
     && lk != 122507                // 'for' 'sliding'
     && lk != 131723                // 'for' 'tumbling'
     && lk != 144128                // 'try' '{'
     && lk != 147225)               // '{' '}'
    {
      lk = memoized(9, e0);
      if (lk == 0)
      {
        var b0A = b0; var e0A = e0; var l1A = l1;
        var b1A = b1; var e1A = e1; var l2A = l2;
        var b2A = b2; var e2A = e2;
        try
        {
          try_ApplyStatement();
          lk = -1;
        }
        catch (p1A)
        {
          try
          {
            b0 = b0A; e0 = e0A; l1 = l1A; if (l1 == 0) {end = e0A;} else {
            b1 = b1A; e1 = e1A; l2 = l2A; if (l2 == 0) {end = e1A;} else {
            b2 = b2A; e2 = e2A; end = e2A; }}
            try_AssignStatement();
            lk = -2;
          }
          catch (p2A)
          {
            try
            {
              b0 = b0A; e0 = e0A; l1 = l1A; if (l1 == 0) {end = e0A;} else {
              b1 = b1A; e1 = e1A; l2 = l2A; if (l2 == 0) {end = e1A;} else {
              b2 = b2A; e2 = e2A; end = e2A; }}
              try_BlockStatement();
              lk = -3;
            }
            catch (p3A)
            {
              try
              {
                b0 = b0A; e0 = e0A; l1 = l1A; if (l1 == 0) {end = e0A;} else {
                b1 = b1A; e1 = e1A; l2 = l2A; if (l2 == 0) {end = e1A;} else {
                b2 = b2A; e2 = e2A; end = e2A; }}
                try_VarDeclStatement();
                lk = -12;
              }
              catch (p12A)
              {
                lk = -13;
              }
            }
          }
        }
        b0 = b0A; e0 = e0A; l1 = l1A; if (l1 == 0) {end = e0A;} else {
        b1 = b1A; e1 = e1A; l2 = l2A; if (l2 == 0) {end = e1A;} else {
        b2 = b2A; e2 = e2A; end = e2A; }}
        memoize(9, e0, lk);
      }
    }
    switch (lk)
    {
    case -2:
      parse_AssignStatement();
      break;
    case -3:
      parse_BlockStatement();
      break;
    case 91735:                     // 'break' 'loop'
      parse_BreakStatement();
      break;
    case 91751:                     // 'continue' 'loop'
      parse_ContinueStatement();
      break;
    case 115333:                    // 'exit' 'returning'
      parse_ExitStatement();
      break;
    case 16011:                     // 'for' '$'
    case 16049:                     // 'let' '$'
    case 118961:                    // 'let' 'score'
    case 122507:                    // 'for' 'sliding'
    case 131723:                    // 'for' 'tumbling'
      parse_FLWORStatement();
      break;
    case 18074:                     // 'if' '('
      parse_IfStatement();
      break;
    case 18168:                     // 'switch' '('
      parse_SwitchStatement();
      break;
    case 144128:                    // 'try' '{'
      parse_TryCatchStatement();
      break;
    case 18179:                     // 'typeswitch' '('
      parse_TypeswitchStatement();
      break;
    case -12:
    case 16140:                     // 'variable' '$'
      parse_VarDeclStatement();
      break;
    case -13:
      parse_WhileStatement();
      break;
    case 54:                        // ';'
      parse_VoidStatement();
      break;
    default:
      parse_ApplyStatement();
    }
    eventHandler.endNonterminal("Statement", e0);
  }

  function try_Statement()
  {
    switch (l1)
    {
    case 133:                       // 'exit'
      lookahead2W(147);             // S^WS | '#' | '(' | '(:' | 'returning'
      break;
    case 139:                       // 'for'
      lookahead2W(179);             // S^WS | '#' | '$' | '(' | '(:' | 'sliding' | 'tumbling'
      break;
    case 177:                       // 'let'
      lookahead2W(166);             // S^WS | '#' | '$' | '(' | '(:' | 'score'
      break;
    case 256:                       // 'try'
      lookahead2W(148);             // S^WS | '#' | '(' | '(:' | '{'
      break;
    case 268:                       // 'variable'
      lookahead2W(143);             // S^WS | '#' | '$' | '(' | '(:'
      break;
    case 281:                       // '{'
      lookahead2W(283);             // EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral | StringLiteral |
      break;
    case 31:                        // '$'
    case 33:                        // '%'
      lookahead2W(246);             // EQName^Token | S^WS | '(:' | 'after' | 'allowing' | 'ancestor' |
      break;
    case 87:                        // 'break'
    case 103:                       // 'continue'
      lookahead2W(145);             // S^WS | '#' | '(' | '(:' | 'loop'
      break;
    case 154:                       // 'if'
    case 248:                       // 'switch'
    case 259:                       // 'typeswitch'
    case 273:                       // 'while'
      lookahead2W(95);              // S^WS | '#' | '(' | '(:'
      break;
    default:
      lk = l1;
    }
    if (lk != 6                     // EQName^Token
     && lk != 8                     // IntegerLiteral
     && lk != 9                     // DecimalLiteral
     && lk != 10                    // DoubleLiteral
     && lk != 11                    // StringLiteral
     && lk != 32                    // '$$'
     && lk != 35                    // '('
     && lk != 36                    // '(#'
     && lk != 41                    // '+'
     && lk != 43                    // '-'
     && lk != 47                    // '/'
     && lk != 48                    // '//'
     && lk != 54                    // ';'
     && lk != 55                    // '<'
     && lk != 56                    // '<!--'
     && lk != 60                    // '<?'
     && lk != 69                    // '['
     && lk != 71                    // 'after'
     && lk != 73                    // 'allowing'
     && lk != 74                    // 'ancestor'
     && lk != 75                    // 'ancestor-or-self'
     && lk != 76                    // 'and'
     && lk != 78                    // 'append'
     && lk != 79                    // 'array'
     && lk != 80                    // 'as'
     && lk != 81                    // 'ascending'
     && lk != 82                    // 'at'
     && lk != 83                    // 'attribute'
     && lk != 84                    // 'base-uri'
     && lk != 85                    // 'before'
     && lk != 86                    // 'boundary-space'
     && lk != 89                    // 'case'
     && lk != 90                    // 'cast'
     && lk != 91                    // 'castable'
     && lk != 92                    // 'catch'
     && lk != 94                    // 'child'
     && lk != 95                    // 'collation'
     && lk != 97                    // 'comment'
     && lk != 98                    // 'constraint'
     && lk != 99                    // 'construction'
     && lk != 102                   // 'context'
     && lk != 104                   // 'copy'
     && lk != 105                   // 'copy-namespaces'
     && lk != 106                   // 'count'
     && lk != 107                   // 'decimal-format'
     && lk != 109                   // 'declare'
     && lk != 110                   // 'default'
     && lk != 111                   // 'delete'
     && lk != 112                   // 'descendant'
     && lk != 113                   // 'descendant-or-self'
     && lk != 114                   // 'descending'
     && lk != 119                   // 'div'
     && lk != 120                   // 'document'
     && lk != 121                   // 'document-node'
     && lk != 122                   // 'element'
     && lk != 123                   // 'else'
     && lk != 124                   // 'empty'
     && lk != 125                   // 'empty-sequence'
     && lk != 126                   // 'encoding'
     && lk != 127                   // 'end'
     && lk != 129                   // 'eq'
     && lk != 130                   // 'every'
     && lk != 132                   // 'except'
     && lk != 134                   // 'external'
     && lk != 135                   // 'false'
     && lk != 136                   // 'first'
     && lk != 137                   // 'following'
     && lk != 138                   // 'following-sibling'
     && lk != 142                   // 'from'
     && lk != 143                   // 'ft-option'
     && lk != 147                   // 'function'
     && lk != 148                   // 'ge'
     && lk != 150                   // 'group'
     && lk != 152                   // 'gt'
     && lk != 153                   // 'idiv'
     && lk != 155                   // 'import'
     && lk != 156                   // 'in'
     && lk != 157                   // 'index'
     && lk != 161                   // 'insert'
     && lk != 162                   // 'instance'
     && lk != 163                   // 'integrity'
     && lk != 164                   // 'intersect'
     && lk != 165                   // 'into'
     && lk != 166                   // 'is'
     && lk != 167                   // 'item'
     && lk != 168                   // 'json'
     && lk != 169                   // 'json-item'
     && lk != 170                   // 'jsoniq'
     && lk != 173                   // 'last'
     && lk != 174                   // 'lax'
     && lk != 175                   // 'le'
     && lk != 179                   // 'loop'
     && lk != 181                   // 'lt'
     && lk != 183                   // 'mod'
     && lk != 184                   // 'modify'
     && lk != 185                   // 'module'
     && lk != 187                   // 'namespace'
     && lk != 188                   // 'namespace-node'
     && lk != 189                   // 'ne'
     && lk != 194                   // 'node'
     && lk != 195                   // 'nodes'
     && lk != 196                   // 'not'
     && lk != 197                   // 'null'
     && lk != 198                   // 'object'
     && lk != 202                   // 'only'
     && lk != 203                   // 'option'
     && lk != 204                   // 'or'
     && lk != 205                   // 'order'
     && lk != 206                   // 'ordered'
     && lk != 207                   // 'ordering'
     && lk != 210                   // 'parent'
     && lk != 216                   // 'preceding'
     && lk != 217                   // 'preceding-sibling'
     && lk != 220                   // 'processing-instruction'
     && lk != 222                   // 'rename'
     && lk != 223                   // 'replace'
     && lk != 224                   // 'return'
     && lk != 225                   // 'returning'
     && lk != 226                   // 'revalidation'
     && lk != 228                   // 'satisfies'
     && lk != 229                   // 'schema'
     && lk != 230                   // 'schema-attribute'
     && lk != 231                   // 'schema-element'
     && lk != 232                   // 'score'
     && lk != 233                   // 'select'
     && lk != 234                   // 'self'
     && lk != 239                   // 'sliding'
     && lk != 240                   // 'some'
     && lk != 241                   // 'stable'
     && lk != 242                   // 'start'
     && lk != 245                   // 'strict'
     && lk != 247                   // 'structured-item'
     && lk != 249                   // 'text'
     && lk != 253                   // 'to'
     && lk != 254                   // 'treat'
     && lk != 255                   // 'true'
     && lk != 257                   // 'tumbling'
     && lk != 258                   // 'type'
     && lk != 260                   // 'union'
     && lk != 262                   // 'unordered'
     && lk != 263                   // 'updating'
     && lk != 266                   // 'validate'
     && lk != 267                   // 'value'
     && lk != 269                   // 'version'
     && lk != 272                   // 'where'
     && lk != 276                   // 'with'
     && lk != 283                   // '{|'
     && lk != 10009                 // '{' NCName^Token
     && lk != 14935                 // 'break' '#'
     && lk != 14951                 // 'continue' '#'
     && lk != 14981                 // 'exit' '#'
     && lk != 14987                 // 'for' '#'
     && lk != 15002                 // 'if' '#'
     && lk != 15025                 // 'let' '#'
     && lk != 15096                 // 'switch' '#'
     && lk != 15104                 // 'try' '#'
     && lk != 15107                 // 'typeswitch' '#'
     && lk != 15116                 // 'variable' '#'
     && lk != 15121                 // 'while' '#'
     && lk != 16011                 // 'for' '$'
     && lk != 16049                 // 'let' '$'
     && lk != 16140                 // 'variable' '$'
     && lk != 18007                 // 'break' '('
     && lk != 18023                 // 'continue' '('
     && lk != 18053                 // 'exit' '('
     && lk != 18059                 // 'for' '('
     && lk != 18074                 // 'if' '('
     && lk != 18097                 // 'let' '('
     && lk != 18168                 // 'switch' '('
     && lk != 18176                 // 'try' '('
     && lk != 18179                 // 'typeswitch' '('
     && lk != 18188                 // 'variable' '('
     && lk != 91735                 // 'break' 'loop'
     && lk != 91751                 // 'continue' 'loop'
     && lk != 115333                // 'exit' 'returning'
     && lk != 118961                // 'let' 'score'
     && lk != 122507                // 'for' 'sliding'
     && lk != 131723                // 'for' 'tumbling'
     && lk != 144128                // 'try' '{'
     && lk != 147225)               // '{' '}'
    {
      lk = memoized(9, e0);
      if (lk == 0)
      {
        var b0A = b0; var e0A = e0; var l1A = l1;
        var b1A = b1; var e1A = e1; var l2A = l2;
        var b2A = b2; var e2A = e2;
        try
        {
          try_ApplyStatement();
          memoize(9, e0A, -1);
          lk = -15;
        }
        catch (p1A)
        {
          try
          {
            b0 = b0A; e0 = e0A; l1 = l1A; if (l1 == 0) {end = e0A;} else {
            b1 = b1A; e1 = e1A; l2 = l2A; if (l2 == 0) {end = e1A;} else {
            b2 = b2A; e2 = e2A; end = e2A; }}
            try_AssignStatement();
            memoize(9, e0A, -2);
            lk = -15;
          }
          catch (p2A)
          {
            try
            {
              b0 = b0A; e0 = e0A; l1 = l1A; if (l1 == 0) {end = e0A;} else {
              b1 = b1A; e1 = e1A; l2 = l2A; if (l2 == 0) {end = e1A;} else {
              b2 = b2A; e2 = e2A; end = e2A; }}
              try_BlockStatement();
              memoize(9, e0A, -3);
              lk = -15;
            }
            catch (p3A)
            {
              try
              {
                b0 = b0A; e0 = e0A; l1 = l1A; if (l1 == 0) {end = e0A;} else {
                b1 = b1A; e1 = e1A; l2 = l2A; if (l2 == 0) {end = e1A;} else {
                b2 = b2A; e2 = e2A; end = e2A; }}
                try_VarDeclStatement();
                memoize(9, e0A, -12);
                lk = -15;
              }
              catch (p12A)
              {
                lk = -13;
                b0 = b0A; e0 = e0A; l1 = l1A; if (l1 == 0) {end = e0A;} else {
                b1 = b1A; e1 = e1A; l2 = l2A; if (l2 == 0) {end = e1A;} else {
                b2 = b2A; e2 = e2A; end = e2A; }}
                memoize(9, e0A, -13);
              }
            }
          }
        }
      }
    }
    switch (lk)
    {
    case -2:
      try_AssignStatement();
      break;
    case -3:
      try_BlockStatement();
      break;
    case 91735:                     // 'break' 'loop'
      try_BreakStatement();
      break;
    case 91751:                     // 'continue' 'loop'
      try_ContinueStatement();
      break;
    case 115333:                    // 'exit' 'returning'
      try_ExitStatement();
      break;
    case 16011:                     // 'for' '$'
    case 16049:                     // 'let' '$'
    case 118961:                    // 'let' 'score'
    case 122507:                    // 'for' 'sliding'
    case 131723:                    // 'for' 'tumbling'
      try_FLWORStatement();
      break;
    case 18074:                     // 'if' '('
      try_IfStatement();
      break;
    case 18168:                     // 'switch' '('
      try_SwitchStatement();
      break;
    case 144128:                    // 'try' '{'
      try_TryCatchStatement();
      break;
    case 18179:                     // 'typeswitch' '('
      try_TypeswitchStatement();
      break;
    case -12:
    case 16140:                     // 'variable' '$'
      try_VarDeclStatement();
      break;
    case -13:
      try_WhileStatement();
      break;
    case 54:                        // ';'
      try_VoidStatement();
      break;
    case -15:
      break;
    default:
      try_ApplyStatement();
    }
  }

  function parse_ApplyStatement()
  {
    eventHandler.startNonterminal("ApplyStatement", e0);
    parse_ExprSimple();
    shift(54);                      // ';'
    eventHandler.endNonterminal("ApplyStatement", e0);
  }

  function try_ApplyStatement()
  {
    try_ExprSimple();
    shiftT(54);                     // ';'
  }

  function parse_AssignStatement()
  {
    eventHandler.startNonterminal("AssignStatement", e0);
    shift(31);                      // '$'
    lookahead1W(246);               // EQName^Token | S^WS | '(:' | 'after' | 'allowing' | 'ancestor' |
    whitespace();
    parse_VarName();
    lookahead1W(28);                // S^WS | '(:' | ':='
    shift(53);                      // ':='
    lookahead1W(267);               // EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral | StringLiteral |
    whitespace();
    parse_ExprSingle();
    shift(54);                      // ';'
    eventHandler.endNonterminal("AssignStatement", e0);
  }

  function try_AssignStatement()
  {
    shiftT(31);                     // '$'
    lookahead1W(246);               // EQName^Token | S^WS | '(:' | 'after' | 'allowing' | 'ancestor' |
    try_VarName();
    lookahead1W(28);                // S^WS | '(:' | ':='
    shiftT(53);                     // ':='
    lookahead1W(267);               // EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral | StringLiteral |
    try_ExprSingle();
    shiftT(54);                     // ';'
  }

  function parse_BlockStatement()
  {
    eventHandler.startNonterminal("BlockStatement", e0);
    shift(281);                     // '{'
    lookahead1W(271);               // EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral | StringLiteral |
    whitespace();
    parse_Statement();
    lookahead1W(281);               // EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral | StringLiteral |
    whitespace();
    parse_Statements();
    shift(287);                     // '}'
    eventHandler.endNonterminal("BlockStatement", e0);
  }

  function try_BlockStatement()
  {
    shiftT(281);                    // '{'
    lookahead1W(271);               // EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral | StringLiteral |
    try_Statement();
    lookahead1W(281);               // EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral | StringLiteral |
    try_Statements();
    shiftT(287);                    // '}'
  }

  function parse_BreakStatement()
  {
    eventHandler.startNonterminal("BreakStatement", e0);
    shift(87);                      // 'break'
    lookahead1W(62);                // S^WS | '(:' | 'loop'
    shift(179);                     // 'loop'
    lookahead1W(29);                // S^WS | '(:' | ';'
    shift(54);                      // ';'
    eventHandler.endNonterminal("BreakStatement", e0);
  }

  function try_BreakStatement()
  {
    shiftT(87);                     // 'break'
    lookahead1W(62);                // S^WS | '(:' | 'loop'
    shiftT(179);                    // 'loop'
    lookahead1W(29);                // S^WS | '(:' | ';'
    shiftT(54);                     // ';'
  }

  function parse_ContinueStatement()
  {
    eventHandler.startNonterminal("ContinueStatement", e0);
    shift(103);                     // 'continue'
    lookahead1W(62);                // S^WS | '(:' | 'loop'
    shift(179);                     // 'loop'
    lookahead1W(29);                // S^WS | '(:' | ';'
    shift(54);                      // ';'
    eventHandler.endNonterminal("ContinueStatement", e0);
  }

  function try_ContinueStatement()
  {
    shiftT(103);                    // 'continue'
    lookahead1W(62);                // S^WS | '(:' | 'loop'
    shiftT(179);                    // 'loop'
    lookahead1W(29);                // S^WS | '(:' | ';'
    shiftT(54);                     // ';'
  }

  function parse_ExitStatement()
  {
    eventHandler.startNonterminal("ExitStatement", e0);
    shift(133);                     // 'exit'
    lookahead1W(74);                // S^WS | '(:' | 'returning'
    shift(225);                     // 'returning'
    lookahead1W(267);               // EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral | StringLiteral |
    whitespace();
    parse_ExprSingle();
    shift(54);                      // ';'
    eventHandler.endNonterminal("ExitStatement", e0);
  }

  function try_ExitStatement()
  {
    shiftT(133);                    // 'exit'
    lookahead1W(74);                // S^WS | '(:' | 'returning'
    shiftT(225);                    // 'returning'
    lookahead1W(267);               // EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral | StringLiteral |
    try_ExprSingle();
    shiftT(54);                     // ';'
  }

  function parse_FLWORStatement()
  {
    eventHandler.startNonterminal("FLWORStatement", e0);
    parse_InitialClause();
    for (;;)
    {
      lookahead1W(195);             // S^WS | '(:' | 'count' | 'for' | 'group' | 'let' | 'order' | 'return' | 'stable' |
      if (l1 == 224)                // 'return'
      {
        break;
      }
      whitespace();
      parse_IntermediateClause();
    }
    whitespace();
    parse_ReturnStatement();
    eventHandler.endNonterminal("FLWORStatement", e0);
  }

  function try_FLWORStatement()
  {
    try_InitialClause();
    for (;;)
    {
      lookahead1W(195);             // S^WS | '(:' | 'count' | 'for' | 'group' | 'let' | 'order' | 'return' | 'stable' |
      if (l1 == 224)                // 'return'
      {
        break;
      }
      try_IntermediateClause();
    }
    try_ReturnStatement();
  }

  function parse_ReturnStatement()
  {
    eventHandler.startNonterminal("ReturnStatement", e0);
    shift(224);                     // 'return'
    lookahead1W(271);               // EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral | StringLiteral |
    whitespace();
    parse_Statement();
    eventHandler.endNonterminal("ReturnStatement", e0);
  }

  function try_ReturnStatement()
  {
    shiftT(224);                    // 'return'
    lookahead1W(271);               // EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral | StringLiteral |
    try_Statement();
  }

  function parse_IfStatement()
  {
    eventHandler.startNonterminal("IfStatement", e0);
    shift(154);                     // 'if'
    lookahead1W(22);                // S^WS | '(' | '(:'
    shift(35);                      // '('
    lookahead1W(267);               // EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral | StringLiteral |
    whitespace();
    parse_Expr();
    shift(38);                      // ')'
    lookahead1W(80);                // S^WS | '(:' | 'then'
    shift(250);                     // 'then'
    lookahead1W(271);               // EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral | StringLiteral |
    whitespace();
    parse_Statement();
    lookahead1W(51);                // S^WS | '(:' | 'else'
    shift(123);                     // 'else'
    lookahead1W(271);               // EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral | StringLiteral |
    whitespace();
    parse_Statement();
    eventHandler.endNonterminal("IfStatement", e0);
  }

  function try_IfStatement()
  {
    shiftT(154);                    // 'if'
    lookahead1W(22);                // S^WS | '(' | '(:'
    shiftT(35);                     // '('
    lookahead1W(267);               // EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral | StringLiteral |
    try_Expr();
    shiftT(38);                     // ')'
    lookahead1W(80);                // S^WS | '(:' | 'then'
    shiftT(250);                    // 'then'
    lookahead1W(271);               // EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral | StringLiteral |
    try_Statement();
    lookahead1W(51);                // S^WS | '(:' | 'else'
    shiftT(123);                    // 'else'
    lookahead1W(271);               // EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral | StringLiteral |
    try_Statement();
  }

  function parse_SwitchStatement()
  {
    eventHandler.startNonterminal("SwitchStatement", e0);
    shift(248);                     // 'switch'
    lookahead1W(22);                // S^WS | '(' | '(:'
    shift(35);                      // '('
    lookahead1W(267);               // EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral | StringLiteral |
    whitespace();
    parse_Expr();
    shift(38);                      // ')'
    for (;;)
    {
      lookahead1W(38);              // S^WS | '(:' | 'case'
      whitespace();
      parse_SwitchCaseStatement();
      lookahead1W(117);             // S^WS | '(:' | 'case' | 'default'
      if (l1 != 89)                 // 'case'
      {
        break;
      }
    }
    shift(110);                     // 'default'
    lookahead1W(73);                // S^WS | '(:' | 'return'
    shift(224);                     // 'return'
    lookahead1W(271);               // EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral | StringLiteral |
    whitespace();
    parse_Statement();
    eventHandler.endNonterminal("SwitchStatement", e0);
  }

  function try_SwitchStatement()
  {
    shiftT(248);                    // 'switch'
    lookahead1W(22);                // S^WS | '(' | '(:'
    shiftT(35);                     // '('
    lookahead1W(267);               // EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral | StringLiteral |
    try_Expr();
    shiftT(38);                     // ')'
    for (;;)
    {
      lookahead1W(38);              // S^WS | '(:' | 'case'
      try_SwitchCaseStatement();
      lookahead1W(117);             // S^WS | '(:' | 'case' | 'default'
      if (l1 != 89)                 // 'case'
      {
        break;
      }
    }
    shiftT(110);                    // 'default'
    lookahead1W(73);                // S^WS | '(:' | 'return'
    shiftT(224);                    // 'return'
    lookahead1W(271);               // EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral | StringLiteral |
    try_Statement();
  }

  function parse_SwitchCaseStatement()
  {
    eventHandler.startNonterminal("SwitchCaseStatement", e0);
    for (;;)
    {
      shift(89);                    // 'case'
      lookahead1W(267);             // EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral | StringLiteral |
      whitespace();
      parse_SwitchCaseOperand();
      if (l1 != 89)                 // 'case'
      {
        break;
      }
    }
    shift(224);                     // 'return'
    lookahead1W(271);               // EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral | StringLiteral |
    whitespace();
    parse_Statement();
    eventHandler.endNonterminal("SwitchCaseStatement", e0);
  }

  function try_SwitchCaseStatement()
  {
    for (;;)
    {
      shiftT(89);                   // 'case'
      lookahead1W(267);             // EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral | StringLiteral |
      try_SwitchCaseOperand();
      if (l1 != 89)                 // 'case'
      {
        break;
      }
    }
    shiftT(224);                    // 'return'
    lookahead1W(271);               // EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral | StringLiteral |
    try_Statement();
  }

  function parse_TryCatchStatement()
  {
    eventHandler.startNonterminal("TryCatchStatement", e0);
    shift(256);                     // 'try'
    lookahead1W(90);                // S^WS | '(:' | '{'
    whitespace();
    parse_BlockStatement();
    for (;;)
    {
      lookahead1W(39);              // S^WS | '(:' | 'catch'
      shift(92);                    // 'catch'
      lookahead1W(249);             // Wildcard | EQName^Token | S^WS | '(:' | 'after' | 'allowing' | 'ancestor' |
      whitespace();
      parse_CatchErrorList();
      whitespace();
      parse_BlockStatement();
      lookahead1W(284);             // EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral | StringLiteral |
      switch (l1)
      {
      case 92:                      // 'catch'
        lookahead2W(256);           // Wildcard | EQName^Token | S^WS | '#' | '(' | '(:' | 'after' | 'allowing' |
        break;
      default:
        lk = l1;
      }
      if (lk != 2652                // 'catch' Wildcard
       && lk != 3164                // 'catch' EQName^Token
       && lk != 36444               // 'catch' 'after'
       && lk != 37468               // 'catch' 'allowing'
       && lk != 37980               // 'catch' 'ancestor'
       && lk != 38492               // 'catch' 'ancestor-or-self'
       && lk != 39004               // 'catch' 'and'
       && lk != 40028               // 'catch' 'append'
       && lk != 40540               // 'catch' 'array'
       && lk != 41052               // 'catch' 'as'
       && lk != 41564               // 'catch' 'ascending'
       && lk != 42076               // 'catch' 'at'
       && lk != 42588               // 'catch' 'attribute'
       && lk != 43100               // 'catch' 'base-uri'
       && lk != 43612               // 'catch' 'before'
       && lk != 44124               // 'catch' 'boundary-space'
       && lk != 44636               // 'catch' 'break'
       && lk != 45660               // 'catch' 'case'
       && lk != 46172               // 'catch' 'cast'
       && lk != 46684               // 'catch' 'castable'
       && lk != 47196               // 'catch' 'catch'
       && lk != 48220               // 'catch' 'child'
       && lk != 48732               // 'catch' 'collation'
       && lk != 49756               // 'catch' 'comment'
       && lk != 50268               // 'catch' 'constraint'
       && lk != 50780               // 'catch' 'construction'
       && lk != 52316               // 'catch' 'context'
       && lk != 52828               // 'catch' 'continue'
       && lk != 53340               // 'catch' 'copy'
       && lk != 53852               // 'catch' 'copy-namespaces'
       && lk != 54364               // 'catch' 'count'
       && lk != 54876               // 'catch' 'decimal-format'
       && lk != 55900               // 'catch' 'declare'
       && lk != 56412               // 'catch' 'default'
       && lk != 56924               // 'catch' 'delete'
       && lk != 57436               // 'catch' 'descendant'
       && lk != 57948               // 'catch' 'descendant-or-self'
       && lk != 58460               // 'catch' 'descending'
       && lk != 61020               // 'catch' 'div'
       && lk != 61532               // 'catch' 'document'
       && lk != 62044               // 'catch' 'document-node'
       && lk != 62556               // 'catch' 'element'
       && lk != 63068               // 'catch' 'else'
       && lk != 63580               // 'catch' 'empty'
       && lk != 64092               // 'catch' 'empty-sequence'
       && lk != 64604               // 'catch' 'encoding'
       && lk != 65116               // 'catch' 'end'
       && lk != 66140               // 'catch' 'eq'
       && lk != 66652               // 'catch' 'every'
       && lk != 67676               // 'catch' 'except'
       && lk != 68188               // 'catch' 'exit'
       && lk != 68700               // 'catch' 'external'
       && lk != 69212               // 'catch' 'false'
       && lk != 69724               // 'catch' 'first'
       && lk != 70236               // 'catch' 'following'
       && lk != 70748               // 'catch' 'following-sibling'
       && lk != 71260               // 'catch' 'for'
       && lk != 72796               // 'catch' 'from'
       && lk != 73308               // 'catch' 'ft-option'
       && lk != 75356               // 'catch' 'function'
       && lk != 75868               // 'catch' 'ge'
       && lk != 76892               // 'catch' 'group'
       && lk != 77916               // 'catch' 'gt'
       && lk != 78428               // 'catch' 'idiv'
       && lk != 78940               // 'catch' 'if'
       && lk != 79452               // 'catch' 'import'
       && lk != 79964               // 'catch' 'in'
       && lk != 80476               // 'catch' 'index'
       && lk != 82524               // 'catch' 'insert'
       && lk != 83036               // 'catch' 'instance'
       && lk != 83548               // 'catch' 'integrity'
       && lk != 84060               // 'catch' 'intersect'
       && lk != 84572               // 'catch' 'into'
       && lk != 85084               // 'catch' 'is'
       && lk != 85596               // 'catch' 'item'
       && lk != 86108               // 'catch' 'json'
       && lk != 86620               // 'catch' 'json-item'
       && lk != 87132               // 'catch' 'jsoniq'
       && lk != 88668               // 'catch' 'last'
       && lk != 89180               // 'catch' 'lax'
       && lk != 89692               // 'catch' 'le'
       && lk != 90716               // 'catch' 'let'
       && lk != 91740               // 'catch' 'loop'
       && lk != 92764               // 'catch' 'lt'
       && lk != 93788               // 'catch' 'mod'
       && lk != 94300               // 'catch' 'modify'
       && lk != 94812               // 'catch' 'module'
       && lk != 95836               // 'catch' 'namespace'
       && lk != 96348               // 'catch' 'namespace-node'
       && lk != 96860               // 'catch' 'ne'
       && lk != 99420               // 'catch' 'node'
       && lk != 99932               // 'catch' 'nodes'
       && lk != 100956              // 'catch' 'null'
       && lk != 101468              // 'catch' 'object'
       && lk != 103516              // 'catch' 'only'
       && lk != 104028              // 'catch' 'option'
       && lk != 104540              // 'catch' 'or'
       && lk != 105052              // 'catch' 'order'
       && lk != 105564              // 'catch' 'ordered'
       && lk != 106076              // 'catch' 'ordering'
       && lk != 107612              // 'catch' 'parent'
       && lk != 110684              // 'catch' 'preceding'
       && lk != 111196              // 'catch' 'preceding-sibling'
       && lk != 112732              // 'catch' 'processing-instruction'
       && lk != 113756              // 'catch' 'rename'
       && lk != 114268              // 'catch' 'replace'
       && lk != 114780              // 'catch' 'return'
       && lk != 115292              // 'catch' 'returning'
       && lk != 115804              // 'catch' 'revalidation'
       && lk != 116828              // 'catch' 'satisfies'
       && lk != 117340              // 'catch' 'schema'
       && lk != 117852              // 'catch' 'schema-attribute'
       && lk != 118364              // 'catch' 'schema-element'
       && lk != 118876              // 'catch' 'score'
       && lk != 119388              // 'catch' 'select'
       && lk != 119900              // 'catch' 'self'
       && lk != 122460              // 'catch' 'sliding'
       && lk != 122972              // 'catch' 'some'
       && lk != 123484              // 'catch' 'stable'
       && lk != 123996              // 'catch' 'start'
       && lk != 125532              // 'catch' 'strict'
       && lk != 126556              // 'catch' 'structured-item'
       && lk != 127068              // 'catch' 'switch'
       && lk != 127580              // 'catch' 'text'
       && lk != 129628              // 'catch' 'to'
       && lk != 130140              // 'catch' 'treat'
       && lk != 130652              // 'catch' 'true'
       && lk != 131164              // 'catch' 'try'
       && lk != 131676              // 'catch' 'tumbling'
       && lk != 132188              // 'catch' 'type'
       && lk != 132700              // 'catch' 'typeswitch'
       && lk != 133212              // 'catch' 'union'
       && lk != 134236              // 'catch' 'unordered'
       && lk != 134748              // 'catch' 'updating'
       && lk != 136284              // 'catch' 'validate'
       && lk != 136796              // 'catch' 'value'
       && lk != 137308              // 'catch' 'variable'
       && lk != 137820              // 'catch' 'version'
       && lk != 139356              // 'catch' 'where'
       && lk != 139868              // 'catch' 'while'
       && lk != 141404)             // 'catch' 'with'
      {
        break;
      }
    }
    eventHandler.endNonterminal("TryCatchStatement", e0);
  }

  function try_TryCatchStatement()
  {
    shiftT(256);                    // 'try'
    lookahead1W(90);                // S^WS | '(:' | '{'
    try_BlockStatement();
    for (;;)
    {
      lookahead1W(39);              // S^WS | '(:' | 'catch'
      shiftT(92);                   // 'catch'
      lookahead1W(249);             // Wildcard | EQName^Token | S^WS | '(:' | 'after' | 'allowing' | 'ancestor' |
      try_CatchErrorList();
      try_BlockStatement();
      lookahead1W(284);             // EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral | StringLiteral |
      switch (l1)
      {
      case 92:                      // 'catch'
        lookahead2W(256);           // Wildcard | EQName^Token | S^WS | '#' | '(' | '(:' | 'after' | 'allowing' |
        break;
      default:
        lk = l1;
      }
      if (lk != 2652                // 'catch' Wildcard
       && lk != 3164                // 'catch' EQName^Token
       && lk != 36444               // 'catch' 'after'
       && lk != 37468               // 'catch' 'allowing'
       && lk != 37980               // 'catch' 'ancestor'
       && lk != 38492               // 'catch' 'ancestor-or-self'
       && lk != 39004               // 'catch' 'and'
       && lk != 40028               // 'catch' 'append'
       && lk != 40540               // 'catch' 'array'
       && lk != 41052               // 'catch' 'as'
       && lk != 41564               // 'catch' 'ascending'
       && lk != 42076               // 'catch' 'at'
       && lk != 42588               // 'catch' 'attribute'
       && lk != 43100               // 'catch' 'base-uri'
       && lk != 43612               // 'catch' 'before'
       && lk != 44124               // 'catch' 'boundary-space'
       && lk != 44636               // 'catch' 'break'
       && lk != 45660               // 'catch' 'case'
       && lk != 46172               // 'catch' 'cast'
       && lk != 46684               // 'catch' 'castable'
       && lk != 47196               // 'catch' 'catch'
       && lk != 48220               // 'catch' 'child'
       && lk != 48732               // 'catch' 'collation'
       && lk != 49756               // 'catch' 'comment'
       && lk != 50268               // 'catch' 'constraint'
       && lk != 50780               // 'catch' 'construction'
       && lk != 52316               // 'catch' 'context'
       && lk != 52828               // 'catch' 'continue'
       && lk != 53340               // 'catch' 'copy'
       && lk != 53852               // 'catch' 'copy-namespaces'
       && lk != 54364               // 'catch' 'count'
       && lk != 54876               // 'catch' 'decimal-format'
       && lk != 55900               // 'catch' 'declare'
       && lk != 56412               // 'catch' 'default'
       && lk != 56924               // 'catch' 'delete'
       && lk != 57436               // 'catch' 'descendant'
       && lk != 57948               // 'catch' 'descendant-or-self'
       && lk != 58460               // 'catch' 'descending'
       && lk != 61020               // 'catch' 'div'
       && lk != 61532               // 'catch' 'document'
       && lk != 62044               // 'catch' 'document-node'
       && lk != 62556               // 'catch' 'element'
       && lk != 63068               // 'catch' 'else'
       && lk != 63580               // 'catch' 'empty'
       && lk != 64092               // 'catch' 'empty-sequence'
       && lk != 64604               // 'catch' 'encoding'
       && lk != 65116               // 'catch' 'end'
       && lk != 66140               // 'catch' 'eq'
       && lk != 66652               // 'catch' 'every'
       && lk != 67676               // 'catch' 'except'
       && lk != 68188               // 'catch' 'exit'
       && lk != 68700               // 'catch' 'external'
       && lk != 69212               // 'catch' 'false'
       && lk != 69724               // 'catch' 'first'
       && lk != 70236               // 'catch' 'following'
       && lk != 70748               // 'catch' 'following-sibling'
       && lk != 71260               // 'catch' 'for'
       && lk != 72796               // 'catch' 'from'
       && lk != 73308               // 'catch' 'ft-option'
       && lk != 75356               // 'catch' 'function'
       && lk != 75868               // 'catch' 'ge'
       && lk != 76892               // 'catch' 'group'
       && lk != 77916               // 'catch' 'gt'
       && lk != 78428               // 'catch' 'idiv'
       && lk != 78940               // 'catch' 'if'
       && lk != 79452               // 'catch' 'import'
       && lk != 79964               // 'catch' 'in'
       && lk != 80476               // 'catch' 'index'
       && lk != 82524               // 'catch' 'insert'
       && lk != 83036               // 'catch' 'instance'
       && lk != 83548               // 'catch' 'integrity'
       && lk != 84060               // 'catch' 'intersect'
       && lk != 84572               // 'catch' 'into'
       && lk != 85084               // 'catch' 'is'
       && lk != 85596               // 'catch' 'item'
       && lk != 86108               // 'catch' 'json'
       && lk != 86620               // 'catch' 'json-item'
       && lk != 87132               // 'catch' 'jsoniq'
       && lk != 88668               // 'catch' 'last'
       && lk != 89180               // 'catch' 'lax'
       && lk != 89692               // 'catch' 'le'
       && lk != 90716               // 'catch' 'let'
       && lk != 91740               // 'catch' 'loop'
       && lk != 92764               // 'catch' 'lt'
       && lk != 93788               // 'catch' 'mod'
       && lk != 94300               // 'catch' 'modify'
       && lk != 94812               // 'catch' 'module'
       && lk != 95836               // 'catch' 'namespace'
       && lk != 96348               // 'catch' 'namespace-node'
       && lk != 96860               // 'catch' 'ne'
       && lk != 99420               // 'catch' 'node'
       && lk != 99932               // 'catch' 'nodes'
       && lk != 100956              // 'catch' 'null'
       && lk != 101468              // 'catch' 'object'
       && lk != 103516              // 'catch' 'only'
       && lk != 104028              // 'catch' 'option'
       && lk != 104540              // 'catch' 'or'
       && lk != 105052              // 'catch' 'order'
       && lk != 105564              // 'catch' 'ordered'
       && lk != 106076              // 'catch' 'ordering'
       && lk != 107612              // 'catch' 'parent'
       && lk != 110684              // 'catch' 'preceding'
       && lk != 111196              // 'catch' 'preceding-sibling'
       && lk != 112732              // 'catch' 'processing-instruction'
       && lk != 113756              // 'catch' 'rename'
       && lk != 114268              // 'catch' 'replace'
       && lk != 114780              // 'catch' 'return'
       && lk != 115292              // 'catch' 'returning'
       && lk != 115804              // 'catch' 'revalidation'
       && lk != 116828              // 'catch' 'satisfies'
       && lk != 117340              // 'catch' 'schema'
       && lk != 117852              // 'catch' 'schema-attribute'
       && lk != 118364              // 'catch' 'schema-element'
       && lk != 118876              // 'catch' 'score'
       && lk != 119388              // 'catch' 'select'
       && lk != 119900              // 'catch' 'self'
       && lk != 122460              // 'catch' 'sliding'
       && lk != 122972              // 'catch' 'some'
       && lk != 123484              // 'catch' 'stable'
       && lk != 123996              // 'catch' 'start'
       && lk != 125532              // 'catch' 'strict'
       && lk != 126556              // 'catch' 'structured-item'
       && lk != 127068              // 'catch' 'switch'
       && lk != 127580              // 'catch' 'text'
       && lk != 129628              // 'catch' 'to'
       && lk != 130140              // 'catch' 'treat'
       && lk != 130652              // 'catch' 'true'
       && lk != 131164              // 'catch' 'try'
       && lk != 131676              // 'catch' 'tumbling'
       && lk != 132188              // 'catch' 'type'
       && lk != 132700              // 'catch' 'typeswitch'
       && lk != 133212              // 'catch' 'union'
       && lk != 134236              // 'catch' 'unordered'
       && lk != 134748              // 'catch' 'updating'
       && lk != 136284              // 'catch' 'validate'
       && lk != 136796              // 'catch' 'value'
       && lk != 137308              // 'catch' 'variable'
       && lk != 137820              // 'catch' 'version'
       && lk != 139356              // 'catch' 'where'
       && lk != 139868              // 'catch' 'while'
       && lk != 141404)             // 'catch' 'with'
      {
        break;
      }
    }
  }

  function parse_TypeswitchStatement()
  {
    eventHandler.startNonterminal("TypeswitchStatement", e0);
    shift(259);                     // 'typeswitch'
    lookahead1W(22);                // S^WS | '(' | '(:'
    shift(35);                      // '('
    lookahead1W(267);               // EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral | StringLiteral |
    whitespace();
    parse_Expr();
    shift(38);                      // ')'
    for (;;)
    {
      lookahead1W(38);              // S^WS | '(:' | 'case'
      whitespace();
      parse_CaseStatement();
      lookahead1W(117);             // S^WS | '(:' | 'case' | 'default'
      if (l1 != 89)                 // 'case'
      {
        break;
      }
    }
    shift(110);                     // 'default'
    lookahead1W(99);                // S^WS | '$' | '(:' | 'return'
    if (l1 == 31)                   // '$'
    {
      shift(31);                    // '$'
      lookahead1W(246);             // EQName^Token | S^WS | '(:' | 'after' | 'allowing' | 'ancestor' |
      whitespace();
      parse_VarName();
    }
    lookahead1W(73);                // S^WS | '(:' | 'return'
    shift(224);                     // 'return'
    lookahead1W(271);               // EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral | StringLiteral |
    whitespace();
    parse_Statement();
    eventHandler.endNonterminal("TypeswitchStatement", e0);
  }

  function try_TypeswitchStatement()
  {
    shiftT(259);                    // 'typeswitch'
    lookahead1W(22);                // S^WS | '(' | '(:'
    shiftT(35);                     // '('
    lookahead1W(267);               // EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral | StringLiteral |
    try_Expr();
    shiftT(38);                     // ')'
    for (;;)
    {
      lookahead1W(38);              // S^WS | '(:' | 'case'
      try_CaseStatement();
      lookahead1W(117);             // S^WS | '(:' | 'case' | 'default'
      if (l1 != 89)                 // 'case'
      {
        break;
      }
    }
    shiftT(110);                    // 'default'
    lookahead1W(99);                // S^WS | '$' | '(:' | 'return'
    if (l1 == 31)                   // '$'
    {
      shiftT(31);                   // '$'
      lookahead1W(246);             // EQName^Token | S^WS | '(:' | 'after' | 'allowing' | 'ancestor' |
      try_VarName();
    }
    lookahead1W(73);                // S^WS | '(:' | 'return'
    shiftT(224);                    // 'return'
    lookahead1W(271);               // EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral | StringLiteral |
    try_Statement();
  }

  function parse_CaseStatement()
  {
    eventHandler.startNonterminal("CaseStatement", e0);
    shift(89);                      // 'case'
    lookahead1W(258);               // EQName^Token | S^WS | '$' | '%' | '(' | '(:' | 'after' | 'allowing' |
    if (l1 == 31)                   // '$'
    {
      shift(31);                    // '$'
      lookahead1W(246);             // EQName^Token | S^WS | '(:' | 'after' | 'allowing' | 'ancestor' |
      whitespace();
      parse_VarName();
      lookahead1W(33);              // S^WS | '(:' | 'as'
      shift(80);                    // 'as'
    }
    lookahead1W(254);               // EQName^Token | S^WS | '%' | '(' | '(:' | 'after' | 'allowing' | 'ancestor' |
    whitespace();
    parse_SequenceType();
    lookahead1W(73);                // S^WS | '(:' | 'return'
    shift(224);                     // 'return'
    lookahead1W(271);               // EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral | StringLiteral |
    whitespace();
    parse_Statement();
    eventHandler.endNonterminal("CaseStatement", e0);
  }

  function try_CaseStatement()
  {
    shiftT(89);                     // 'case'
    lookahead1W(258);               // EQName^Token | S^WS | '$' | '%' | '(' | '(:' | 'after' | 'allowing' |
    if (l1 == 31)                   // '$'
    {
      shiftT(31);                   // '$'
      lookahead1W(246);             // EQName^Token | S^WS | '(:' | 'after' | 'allowing' | 'ancestor' |
      try_VarName();
      lookahead1W(33);              // S^WS | '(:' | 'as'
      shiftT(80);                   // 'as'
    }
    lookahead1W(254);               // EQName^Token | S^WS | '%' | '(' | '(:' | 'after' | 'allowing' | 'ancestor' |
    try_SequenceType();
    lookahead1W(73);                // S^WS | '(:' | 'return'
    shiftT(224);                    // 'return'
    lookahead1W(271);               // EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral | StringLiteral |
    try_Statement();
  }

  function parse_VarDeclStatement()
  {
    eventHandler.startNonterminal("VarDeclStatement", e0);
    for (;;)
    {
      lookahead1W(102);             // S^WS | '%' | '(:' | 'variable'
      if (l1 != 33)                 // '%'
      {
        break;
      }
      whitespace();
      parse_Annotation();
    }
    shift(268);                     // 'variable'
    lookahead1W(21);                // S^WS | '$' | '(:'
    shift(31);                      // '$'
    lookahead1W(246);               // EQName^Token | S^WS | '(:' | 'after' | 'allowing' | 'ancestor' |
    whitespace();
    parse_VarName();
    lookahead1W(172);               // S^WS | '(:' | ',' | ':=' | ';' | 'as'
    if (l1 == 80)                   // 'as'
    {
      whitespace();
      parse_TypeDeclaration();
    }
    lookahead1W(155);               // S^WS | '(:' | ',' | ':=' | ';'
    if (l1 == 53)                   // ':='
    {
      shift(53);                    // ':='
      lookahead1W(267);             // EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral | StringLiteral |
      whitespace();
      parse_ExprSingle();
    }
    for (;;)
    {
      if (l1 != 42)                 // ','
      {
        break;
      }
      shift(42);                    // ','
      lookahead1W(21);              // S^WS | '$' | '(:'
      shift(31);                    // '$'
      lookahead1W(246);             // EQName^Token | S^WS | '(:' | 'after' | 'allowing' | 'ancestor' |
      whitespace();
      parse_VarName();
      lookahead1W(172);             // S^WS | '(:' | ',' | ':=' | ';' | 'as'
      if (l1 == 80)                 // 'as'
      {
        whitespace();
        parse_TypeDeclaration();
      }
      lookahead1W(155);             // S^WS | '(:' | ',' | ':=' | ';'
      if (l1 == 53)                 // ':='
      {
        shift(53);                  // ':='
        lookahead1W(267);           // EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral | StringLiteral |
        whitespace();
        parse_ExprSingle();
      }
    }
    shift(54);                      // ';'
    eventHandler.endNonterminal("VarDeclStatement", e0);
  }

  function try_VarDeclStatement()
  {
    for (;;)
    {
      lookahead1W(102);             // S^WS | '%' | '(:' | 'variable'
      if (l1 != 33)                 // '%'
      {
        break;
      }
      try_Annotation();
    }
    shiftT(268);                    // 'variable'
    lookahead1W(21);                // S^WS | '$' | '(:'
    shiftT(31);                     // '$'
    lookahead1W(246);               // EQName^Token | S^WS | '(:' | 'after' | 'allowing' | 'ancestor' |
    try_VarName();
    lookahead1W(172);               // S^WS | '(:' | ',' | ':=' | ';' | 'as'
    if (l1 == 80)                   // 'as'
    {
      try_TypeDeclaration();
    }
    lookahead1W(155);               // S^WS | '(:' | ',' | ':=' | ';'
    if (l1 == 53)                   // ':='
    {
      shiftT(53);                   // ':='
      lookahead1W(267);             // EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral | StringLiteral |
      try_ExprSingle();
    }
    for (;;)
    {
      if (l1 != 42)                 // ','
      {
        break;
      }
      shiftT(42);                   // ','
      lookahead1W(21);              // S^WS | '$' | '(:'
      shiftT(31);                   // '$'
      lookahead1W(246);             // EQName^Token | S^WS | '(:' | 'after' | 'allowing' | 'ancestor' |
      try_VarName();
      lookahead1W(172);             // S^WS | '(:' | ',' | ':=' | ';' | 'as'
      if (l1 == 80)                 // 'as'
      {
        try_TypeDeclaration();
      }
      lookahead1W(155);             // S^WS | '(:' | ',' | ':=' | ';'
      if (l1 == 53)                 // ':='
      {
        shiftT(53);                 // ':='
        lookahead1W(267);           // EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral | StringLiteral |
        try_ExprSingle();
      }
    }
    shiftT(54);                     // ';'
  }

  function parse_WhileStatement()
  {
    eventHandler.startNonterminal("WhileStatement", e0);
    shift(273);                     // 'while'
    lookahead1W(22);                // S^WS | '(' | '(:'
    shift(35);                      // '('
    lookahead1W(267);               // EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral | StringLiteral |
    whitespace();
    parse_Expr();
    shift(38);                      // ')'
    lookahead1W(271);               // EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral | StringLiteral |
    whitespace();
    parse_Statement();
    eventHandler.endNonterminal("WhileStatement", e0);
  }

  function try_WhileStatement()
  {
    shiftT(273);                    // 'while'
    lookahead1W(22);                // S^WS | '(' | '(:'
    shiftT(35);                     // '('
    lookahead1W(267);               // EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral | StringLiteral |
    try_Expr();
    shiftT(38);                     // ')'
    lookahead1W(271);               // EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral | StringLiteral |
    try_Statement();
  }

  function parse_VoidStatement()
  {
    eventHandler.startNonterminal("VoidStatement", e0);
    shift(54);                      // ';'
    eventHandler.endNonterminal("VoidStatement", e0);
  }

  function try_VoidStatement()
  {
    shiftT(54);                     // ';'
  }

  function parse_ExprSingle()
  {
    eventHandler.startNonterminal("ExprSingle", e0);
    switch (l1)
    {
    case 139:                       // 'for'
      lookahead2W(179);             // S^WS | '#' | '$' | '(' | '(:' | 'sliding' | 'tumbling'
      break;
    case 177:                       // 'let'
      lookahead2W(166);             // S^WS | '#' | '$' | '(' | '(:' | 'score'
      break;
    case 256:                       // 'try'
      lookahead2W(148);             // S^WS | '#' | '(' | '(:' | '{'
      break;
    case 154:                       // 'if'
    case 248:                       // 'switch'
    case 259:                       // 'typeswitch'
      lookahead2W(95);              // S^WS | '#' | '(' | '(:'
      break;
    default:
      lk = l1;
    }
    switch (lk)
    {
    case 16011:                     // 'for' '$'
    case 16049:                     // 'let' '$'
    case 118961:                    // 'let' 'score'
    case 122507:                    // 'for' 'sliding'
    case 131723:                    // 'for' 'tumbling'
      parse_FLWORExpr();
      break;
    case 18074:                     // 'if' '('
      parse_IfExpr();
      break;
    case 18168:                     // 'switch' '('
      parse_SwitchExpr();
      break;
    case 144128:                    // 'try' '{'
      parse_TryCatchExpr();
      break;
    case 18179:                     // 'typeswitch' '('
      parse_TypeswitchExpr();
      break;
    default:
      parse_ExprSimple();
    }
    eventHandler.endNonterminal("ExprSingle", e0);
  }

  function try_ExprSingle()
  {
    switch (l1)
    {
    case 139:                       // 'for'
      lookahead2W(179);             // S^WS | '#' | '$' | '(' | '(:' | 'sliding' | 'tumbling'
      break;
    case 177:                       // 'let'
      lookahead2W(166);             // S^WS | '#' | '$' | '(' | '(:' | 'score'
      break;
    case 256:                       // 'try'
      lookahead2W(148);             // S^WS | '#' | '(' | '(:' | '{'
      break;
    case 154:                       // 'if'
    case 248:                       // 'switch'
    case 259:                       // 'typeswitch'
      lookahead2W(95);              // S^WS | '#' | '(' | '(:'
      break;
    default:
      lk = l1;
    }
    switch (lk)
    {
    case 16011:                     // 'for' '$'
    case 16049:                     // 'let' '$'
    case 118961:                    // 'let' 'score'
    case 122507:                    // 'for' 'sliding'
    case 131723:                    // 'for' 'tumbling'
      try_FLWORExpr();
      break;
    case 18074:                     // 'if' '('
      try_IfExpr();
      break;
    case 18168:                     // 'switch' '('
      try_SwitchExpr();
      break;
    case 144128:                    // 'try' '{'
      try_TryCatchExpr();
      break;
    case 18179:                     // 'typeswitch' '('
      try_TypeswitchExpr();
      break;
    default:
      try_ExprSimple();
    }
  }

  function parse_ExprSimple()
  {
    eventHandler.startNonterminal("ExprSimple", e0);
    switch (l1)
    {
    case 78:                        // 'append'
      lookahead2W(269);             // EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral | StringLiteral |
      break;
    case 161:                       // 'insert'
      lookahead2W(276);             // EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral | StringLiteral |
      break;
    case 223:                       // 'replace'
      lookahead2W(170);             // S^WS | '#' | '(' | '(:' | 'node' | 'value'
      break;
    case 111:                       // 'delete'
    case 222:                       // 'rename'
      lookahead2W(261);             // EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral | StringLiteral |
      break;
    case 104:                       // 'copy'
    case 130:                       // 'every'
    case 240:                       // 'some'
      lookahead2W(143);             // S^WS | '#' | '$' | '(' | '(:'
      break;
    default:
      lk = l1;
    }
    if (lk == 17998                 // 'append' '('
     || lk == 18031                 // 'delete' '('
     || lk == 18081                 // 'insert' '('
     || lk == 18142                 // 'rename' '('
     || lk == 99439                 // 'delete' 'node'
     || lk == 99489                 // 'insert' 'node'
     || lk == 99550                 // 'rename' 'node'
     || lk == 99951                 // 'delete' 'nodes'
     || lk == 100001                // 'insert' 'nodes'
     || lk == 136927)               // 'replace' 'value'
    {
      lk = memoized(10, e0);
      if (lk == 0)
      {
        var b0A = b0; var e0A = e0; var l1A = l1;
        var b1A = b1; var e1A = e1; var l2A = l2;
        var b2A = b2; var e2A = e2;
        try
        {
          try_OrExpr();
          lk = -2;
        }
        catch (p2A)
        {
          try
          {
            b0 = b0A; e0 = e0A; l1 = l1A; if (l1 == 0) {end = e0A;} else {
            b1 = b1A; e1 = e1A; l2 = l2A; if (l2 == 0) {end = e1A;} else {
            b2 = b2A; e2 = e2A; end = e2A; }}
            try_InsertExpr();
            lk = -3;
          }
          catch (p3A)
          {
            try
            {
              b0 = b0A; e0 = e0A; l1 = l1A; if (l1 == 0) {end = e0A;} else {
              b1 = b1A; e1 = e1A; l2 = l2A; if (l2 == 0) {end = e1A;} else {
              b2 = b2A; e2 = e2A; end = e2A; }}
              try_DeleteExpr();
              lk = -4;
            }
            catch (p4A)
            {
              try
              {
                b0 = b0A; e0 = e0A; l1 = l1A; if (l1 == 0) {end = e0A;} else {
                b1 = b1A; e1 = e1A; l2 = l2A; if (l2 == 0) {end = e1A;} else {
                b2 = b2A; e2 = e2A; end = e2A; }}
                try_RenameExpr();
                lk = -5;
              }
              catch (p5A)
              {
                try
                {
                  b0 = b0A; e0 = e0A; l1 = l1A; if (l1 == 0) {end = e0A;} else {
                  b1 = b1A; e1 = e1A; l2 = l2A; if (l2 == 0) {end = e1A;} else {
                  b2 = b2A; e2 = e2A; end = e2A; }}
                  try_ReplaceExpr();
                  lk = -6;
                }
                catch (p6A)
                {
                  try
                  {
                    b0 = b0A; e0 = e0A; l1 = l1A; if (l1 == 0) {end = e0A;} else {
                    b1 = b1A; e1 = e1A; l2 = l2A; if (l2 == 0) {end = e1A;} else {
                    b2 = b2A; e2 = e2A; end = e2A; }}
                    try_JSONDeleteExpr();
                    lk = -8;
                  }
                  catch (p8A)
                  {
                    try
                    {
                      b0 = b0A; e0 = e0A; l1 = l1A; if (l1 == 0) {end = e0A;} else {
                      b1 = b1A; e1 = e1A; l2 = l2A; if (l2 == 0) {end = e1A;} else {
                      b2 = b2A; e2 = e2A; end = e2A; }}
                      try_JSONInsertExpr();
                      lk = -9;
                    }
                    catch (p9A)
                    {
                      try
                      {
                        b0 = b0A; e0 = e0A; l1 = l1A; if (l1 == 0) {end = e0A;} else {
                        b1 = b1A; e1 = e1A; l2 = l2A; if (l2 == 0) {end = e1A;} else {
                        b2 = b2A; e2 = e2A; end = e2A; }}
                        try_JSONRenameExpr();
                        lk = -10;
                      }
                      catch (p10A)
                      {
                        try
                        {
                          b0 = b0A; e0 = e0A; l1 = l1A; if (l1 == 0) {end = e0A;} else {
                          b1 = b1A; e1 = e1A; l2 = l2A; if (l2 == 0) {end = e1A;} else {
                          b2 = b2A; e2 = e2A; end = e2A; }}
                          try_JSONReplaceExpr();
                          lk = -11;
                        }
                        catch (p11A)
                        {
                          lk = -12;
                        }
                      }
                    }
                  }
                }
              }
            }
          }
        }
        b0 = b0A; e0 = e0A; l1 = l1A; if (l1 == 0) {end = e0A;} else {
        b1 = b1A; e1 = e1A; l2 = l2A; if (l2 == 0) {end = e1A;} else {
        b2 = b2A; e2 = e2A; end = e2A; }}
        memoize(10, e0, lk);
      }
    }
    switch (lk)
    {
    case 16002:                     // 'every' '$'
    case 16112:                     // 'some' '$'
      parse_QuantifiedExpr();
      break;
    case -3:
      parse_InsertExpr();
      break;
    case -4:
      parse_DeleteExpr();
      break;
    case -5:
      parse_RenameExpr();
      break;
    case -6:
    case 99551:                     // 'replace' 'node'
      parse_ReplaceExpr();
      break;
    case 15976:                     // 'copy' '$'
      parse_TransformExpr();
      break;
    case -8:
    case 3183:                      // 'delete' EQName^Token
    case 4207:                      // 'delete' IntegerLiteral
    case 4719:                      // 'delete' DecimalLiteral
    case 5231:                      // 'delete' DoubleLiteral
    case 5743:                      // 'delete' StringLiteral
    case 15983:                     // 'delete' '$'
    case 16495:                     // 'delete' '$$'
    case 17007:                     // 'delete' '%'
    case 28271:                     // 'delete' '<'
    case 28783:                     // 'delete' '<!--'
    case 30831:                     // 'delete' '<?'
    case 35439:                     // 'delete' '['
    case 36463:                     // 'delete' 'after'
    case 37487:                     // 'delete' 'allowing'
    case 37999:                     // 'delete' 'ancestor'
    case 38511:                     // 'delete' 'ancestor-or-self'
    case 39023:                     // 'delete' 'and'
    case 40047:                     // 'delete' 'append'
    case 40559:                     // 'delete' 'array'
    case 41071:                     // 'delete' 'as'
    case 41583:                     // 'delete' 'ascending'
    case 42095:                     // 'delete' 'at'
    case 42607:                     // 'delete' 'attribute'
    case 43119:                     // 'delete' 'base-uri'
    case 43631:                     // 'delete' 'before'
    case 44143:                     // 'delete' 'boundary-space'
    case 44655:                     // 'delete' 'break'
    case 45679:                     // 'delete' 'case'
    case 46191:                     // 'delete' 'cast'
    case 46703:                     // 'delete' 'castable'
    case 47215:                     // 'delete' 'catch'
    case 48239:                     // 'delete' 'child'
    case 48751:                     // 'delete' 'collation'
    case 49775:                     // 'delete' 'comment'
    case 50287:                     // 'delete' 'constraint'
    case 50799:                     // 'delete' 'construction'
    case 52335:                     // 'delete' 'context'
    case 52847:                     // 'delete' 'continue'
    case 53359:                     // 'delete' 'copy'
    case 53871:                     // 'delete' 'copy-namespaces'
    case 54383:                     // 'delete' 'count'
    case 54895:                     // 'delete' 'decimal-format'
    case 55919:                     // 'delete' 'declare'
    case 56431:                     // 'delete' 'default'
    case 56943:                     // 'delete' 'delete'
    case 57455:                     // 'delete' 'descendant'
    case 57967:                     // 'delete' 'descendant-or-self'
    case 58479:                     // 'delete' 'descending'
    case 61039:                     // 'delete' 'div'
    case 61551:                     // 'delete' 'document'
    case 62063:                     // 'delete' 'document-node'
    case 62575:                     // 'delete' 'element'
    case 63087:                     // 'delete' 'else'
    case 63599:                     // 'delete' 'empty'
    case 64111:                     // 'delete' 'empty-sequence'
    case 64623:                     // 'delete' 'encoding'
    case 65135:                     // 'delete' 'end'
    case 66159:                     // 'delete' 'eq'
    case 66671:                     // 'delete' 'every'
    case 67695:                     // 'delete' 'except'
    case 68207:                     // 'delete' 'exit'
    case 68719:                     // 'delete' 'external'
    case 69231:                     // 'delete' 'false'
    case 69743:                     // 'delete' 'first'
    case 70255:                     // 'delete' 'following'
    case 70767:                     // 'delete' 'following-sibling'
    case 71279:                     // 'delete' 'for'
    case 72815:                     // 'delete' 'from'
    case 73327:                     // 'delete' 'ft-option'
    case 75375:                     // 'delete' 'function'
    case 75887:                     // 'delete' 'ge'
    case 76911:                     // 'delete' 'group'
    case 77935:                     // 'delete' 'gt'
    case 78447:                     // 'delete' 'idiv'
    case 78959:                     // 'delete' 'if'
    case 79471:                     // 'delete' 'import'
    case 79983:                     // 'delete' 'in'
    case 80495:                     // 'delete' 'index'
    case 82543:                     // 'delete' 'insert'
    case 83055:                     // 'delete' 'instance'
    case 83567:                     // 'delete' 'integrity'
    case 84079:                     // 'delete' 'intersect'
    case 84591:                     // 'delete' 'into'
    case 85103:                     // 'delete' 'is'
    case 85615:                     // 'delete' 'item'
    case 86127:                     // 'delete' 'json'
    case 86639:                     // 'delete' 'json-item'
    case 87151:                     // 'delete' 'jsoniq'
    case 88687:                     // 'delete' 'last'
    case 89199:                     // 'delete' 'lax'
    case 89711:                     // 'delete' 'le'
    case 90735:                     // 'delete' 'let'
    case 91759:                     // 'delete' 'loop'
    case 92783:                     // 'delete' 'lt'
    case 93807:                     // 'delete' 'mod'
    case 94319:                     // 'delete' 'modify'
    case 94831:                     // 'delete' 'module'
    case 95855:                     // 'delete' 'namespace'
    case 96367:                     // 'delete' 'namespace-node'
    case 96879:                     // 'delete' 'ne'
    case 100975:                    // 'delete' 'null'
    case 101487:                    // 'delete' 'object'
    case 103535:                    // 'delete' 'only'
    case 104047:                    // 'delete' 'option'
    case 104559:                    // 'delete' 'or'
    case 105071:                    // 'delete' 'order'
    case 105583:                    // 'delete' 'ordered'
    case 106095:                    // 'delete' 'ordering'
    case 107631:                    // 'delete' 'parent'
    case 110703:                    // 'delete' 'preceding'
    case 111215:                    // 'delete' 'preceding-sibling'
    case 112751:                    // 'delete' 'processing-instruction'
    case 113775:                    // 'delete' 'rename'
    case 114287:                    // 'delete' 'replace'
    case 114799:                    // 'delete' 'return'
    case 115311:                    // 'delete' 'returning'
    case 115823:                    // 'delete' 'revalidation'
    case 116847:                    // 'delete' 'satisfies'
    case 117359:                    // 'delete' 'schema'
    case 117871:                    // 'delete' 'schema-attribute'
    case 118383:                    // 'delete' 'schema-element'
    case 118895:                    // 'delete' 'score'
    case 119407:                    // 'delete' 'select'
    case 119919:                    // 'delete' 'self'
    case 122479:                    // 'delete' 'sliding'
    case 122991:                    // 'delete' 'some'
    case 123503:                    // 'delete' 'stable'
    case 124015:                    // 'delete' 'start'
    case 125551:                    // 'delete' 'strict'
    case 126575:                    // 'delete' 'structured-item'
    case 127087:                    // 'delete' 'switch'
    case 127599:                    // 'delete' 'text'
    case 129647:                    // 'delete' 'to'
    case 130159:                    // 'delete' 'treat'
    case 130671:                    // 'delete' 'true'
    case 131183:                    // 'delete' 'try'
    case 131695:                    // 'delete' 'tumbling'
    case 132207:                    // 'delete' 'type'
    case 132719:                    // 'delete' 'typeswitch'
    case 133231:                    // 'delete' 'union'
    case 134255:                    // 'delete' 'unordered'
    case 134767:                    // 'delete' 'updating'
    case 136303:                    // 'delete' 'validate'
    case 136815:                    // 'delete' 'value'
    case 137327:                    // 'delete' 'variable'
    case 137839:                    // 'delete' 'version'
    case 139375:                    // 'delete' 'where'
    case 139887:                    // 'delete' 'while'
    case 141423:                    // 'delete' 'with'
    case 143983:                    // 'delete' '{'
    case 145007:                    // 'delete' '{|'
      parse_JSONDeleteExpr();
      break;
    case -9:
    case 3233:                      // 'insert' EQName^Token
    case 4257:                      // 'insert' IntegerLiteral
    case 4769:                      // 'insert' DecimalLiteral
    case 5281:                      // 'insert' DoubleLiteral
    case 5793:                      // 'insert' StringLiteral
    case 9889:                      // 'insert' NCName^Token
    case 16033:                     // 'insert' '$'
    case 16545:                     // 'insert' '$$'
    case 17057:                     // 'insert' '%'
    case 18593:                     // 'insert' '(#'
    case 21153:                     // 'insert' '+'
    case 22177:                     // 'insert' '-'
    case 24225:                     // 'insert' '/'
    case 24737:                     // 'insert' '//'
    case 28321:                     // 'insert' '<'
    case 28833:                     // 'insert' '<!--'
    case 30881:                     // 'insert' '<?'
    case 35489:                     // 'insert' '['
    case 36513:                     // 'insert' 'after'
    case 37537:                     // 'insert' 'allowing'
    case 38049:                     // 'insert' 'ancestor'
    case 38561:                     // 'insert' 'ancestor-or-self'
    case 39073:                     // 'insert' 'and'
    case 40097:                     // 'insert' 'append'
    case 40609:                     // 'insert' 'array'
    case 41121:                     // 'insert' 'as'
    case 41633:                     // 'insert' 'ascending'
    case 42145:                     // 'insert' 'at'
    case 42657:                     // 'insert' 'attribute'
    case 43169:                     // 'insert' 'base-uri'
    case 43681:                     // 'insert' 'before'
    case 44193:                     // 'insert' 'boundary-space'
    case 44705:                     // 'insert' 'break'
    case 45729:                     // 'insert' 'case'
    case 46241:                     // 'insert' 'cast'
    case 46753:                     // 'insert' 'castable'
    case 47265:                     // 'insert' 'catch'
    case 48289:                     // 'insert' 'child'
    case 48801:                     // 'insert' 'collation'
    case 49825:                     // 'insert' 'comment'
    case 50337:                     // 'insert' 'constraint'
    case 50849:                     // 'insert' 'construction'
    case 52385:                     // 'insert' 'context'
    case 52897:                     // 'insert' 'continue'
    case 53409:                     // 'insert' 'copy'
    case 53921:                     // 'insert' 'copy-namespaces'
    case 54433:                     // 'insert' 'count'
    case 54945:                     // 'insert' 'decimal-format'
    case 55969:                     // 'insert' 'declare'
    case 56481:                     // 'insert' 'default'
    case 56993:                     // 'insert' 'delete'
    case 57505:                     // 'insert' 'descendant'
    case 58017:                     // 'insert' 'descendant-or-self'
    case 58529:                     // 'insert' 'descending'
    case 61089:                     // 'insert' 'div'
    case 61601:                     // 'insert' 'document'
    case 62113:                     // 'insert' 'document-node'
    case 62625:                     // 'insert' 'element'
    case 63137:                     // 'insert' 'else'
    case 63649:                     // 'insert' 'empty'
    case 64161:                     // 'insert' 'empty-sequence'
    case 64673:                     // 'insert' 'encoding'
    case 65185:                     // 'insert' 'end'
    case 66209:                     // 'insert' 'eq'
    case 66721:                     // 'insert' 'every'
    case 67745:                     // 'insert' 'except'
    case 68257:                     // 'insert' 'exit'
    case 68769:                     // 'insert' 'external'
    case 69281:                     // 'insert' 'false'
    case 69793:                     // 'insert' 'first'
    case 70305:                     // 'insert' 'following'
    case 70817:                     // 'insert' 'following-sibling'
    case 71329:                     // 'insert' 'for'
    case 72865:                     // 'insert' 'from'
    case 73377:                     // 'insert' 'ft-option'
    case 75425:                     // 'insert' 'function'
    case 75937:                     // 'insert' 'ge'
    case 76961:                     // 'insert' 'group'
    case 77985:                     // 'insert' 'gt'
    case 78497:                     // 'insert' 'idiv'
    case 79009:                     // 'insert' 'if'
    case 79521:                     // 'insert' 'import'
    case 80033:                     // 'insert' 'in'
    case 80545:                     // 'insert' 'index'
    case 82593:                     // 'insert' 'insert'
    case 83105:                     // 'insert' 'instance'
    case 83617:                     // 'insert' 'integrity'
    case 84129:                     // 'insert' 'intersect'
    case 84641:                     // 'insert' 'into'
    case 85153:                     // 'insert' 'is'
    case 85665:                     // 'insert' 'item'
    case 86177:                     // 'insert' 'json'
    case 86689:                     // 'insert' 'json-item'
    case 87201:                     // 'insert' 'jsoniq'
    case 88737:                     // 'insert' 'last'
    case 89249:                     // 'insert' 'lax'
    case 89761:                     // 'insert' 'le'
    case 90785:                     // 'insert' 'let'
    case 91809:                     // 'insert' 'loop'
    case 92833:                     // 'insert' 'lt'
    case 93857:                     // 'insert' 'mod'
    case 94369:                     // 'insert' 'modify'
    case 94881:                     // 'insert' 'module'
    case 95905:                     // 'insert' 'namespace'
    case 96417:                     // 'insert' 'namespace-node'
    case 96929:                     // 'insert' 'ne'
    case 100513:                    // 'insert' 'not'
    case 101025:                    // 'insert' 'null'
    case 101537:                    // 'insert' 'object'
    case 103585:                    // 'insert' 'only'
    case 104097:                    // 'insert' 'option'
    case 104609:                    // 'insert' 'or'
    case 105121:                    // 'insert' 'order'
    case 105633:                    // 'insert' 'ordered'
    case 106145:                    // 'insert' 'ordering'
    case 107681:                    // 'insert' 'parent'
    case 110753:                    // 'insert' 'preceding'
    case 111265:                    // 'insert' 'preceding-sibling'
    case 112801:                    // 'insert' 'processing-instruction'
    case 113825:                    // 'insert' 'rename'
    case 114337:                    // 'insert' 'replace'
    case 114849:                    // 'insert' 'return'
    case 115361:                    // 'insert' 'returning'
    case 115873:                    // 'insert' 'revalidation'
    case 116897:                    // 'insert' 'satisfies'
    case 117409:                    // 'insert' 'schema'
    case 117921:                    // 'insert' 'schema-attribute'
    case 118433:                    // 'insert' 'schema-element'
    case 118945:                    // 'insert' 'score'
    case 119457:                    // 'insert' 'select'
    case 119969:                    // 'insert' 'self'
    case 122529:                    // 'insert' 'sliding'
    case 123041:                    // 'insert' 'some'
    case 123553:                    // 'insert' 'stable'
    case 124065:                    // 'insert' 'start'
    case 125601:                    // 'insert' 'strict'
    case 126625:                    // 'insert' 'structured-item'
    case 127137:                    // 'insert' 'switch'
    case 127649:                    // 'insert' 'text'
    case 129697:                    // 'insert' 'to'
    case 130209:                    // 'insert' 'treat'
    case 130721:                    // 'insert' 'true'
    case 131233:                    // 'insert' 'try'
    case 131745:                    // 'insert' 'tumbling'
    case 132257:                    // 'insert' 'type'
    case 132769:                    // 'insert' 'typeswitch'
    case 133281:                    // 'insert' 'union'
    case 134305:                    // 'insert' 'unordered'
    case 134817:                    // 'insert' 'updating'
    case 136353:                    // 'insert' 'validate'
    case 136865:                    // 'insert' 'value'
    case 137377:                    // 'insert' 'variable'
    case 137889:                    // 'insert' 'version'
    case 139425:                    // 'insert' 'where'
    case 139937:                    // 'insert' 'while'
    case 141473:                    // 'insert' 'with'
    case 144033:                    // 'insert' '{'
    case 145057:                    // 'insert' '{|'
      parse_JSONInsertExpr();
      break;
    case -10:
    case 3294:                      // 'rename' EQName^Token
    case 4318:                      // 'rename' IntegerLiteral
    case 4830:                      // 'rename' DecimalLiteral
    case 5342:                      // 'rename' DoubleLiteral
    case 5854:                      // 'rename' StringLiteral
    case 16094:                     // 'rename' '$'
    case 16606:                     // 'rename' '$$'
    case 17118:                     // 'rename' '%'
    case 28382:                     // 'rename' '<'
    case 28894:                     // 'rename' '<!--'
    case 30942:                     // 'rename' '<?'
    case 35550:                     // 'rename' '['
    case 36574:                     // 'rename' 'after'
    case 37598:                     // 'rename' 'allowing'
    case 38110:                     // 'rename' 'ancestor'
    case 38622:                     // 'rename' 'ancestor-or-self'
    case 39134:                     // 'rename' 'and'
    case 40158:                     // 'rename' 'append'
    case 40670:                     // 'rename' 'array'
    case 41182:                     // 'rename' 'as'
    case 41694:                     // 'rename' 'ascending'
    case 42206:                     // 'rename' 'at'
    case 42718:                     // 'rename' 'attribute'
    case 43230:                     // 'rename' 'base-uri'
    case 43742:                     // 'rename' 'before'
    case 44254:                     // 'rename' 'boundary-space'
    case 44766:                     // 'rename' 'break'
    case 45790:                     // 'rename' 'case'
    case 46302:                     // 'rename' 'cast'
    case 46814:                     // 'rename' 'castable'
    case 47326:                     // 'rename' 'catch'
    case 48350:                     // 'rename' 'child'
    case 48862:                     // 'rename' 'collation'
    case 49886:                     // 'rename' 'comment'
    case 50398:                     // 'rename' 'constraint'
    case 50910:                     // 'rename' 'construction'
    case 52446:                     // 'rename' 'context'
    case 52958:                     // 'rename' 'continue'
    case 53470:                     // 'rename' 'copy'
    case 53982:                     // 'rename' 'copy-namespaces'
    case 54494:                     // 'rename' 'count'
    case 55006:                     // 'rename' 'decimal-format'
    case 56030:                     // 'rename' 'declare'
    case 56542:                     // 'rename' 'default'
    case 57054:                     // 'rename' 'delete'
    case 57566:                     // 'rename' 'descendant'
    case 58078:                     // 'rename' 'descendant-or-self'
    case 58590:                     // 'rename' 'descending'
    case 61150:                     // 'rename' 'div'
    case 61662:                     // 'rename' 'document'
    case 62174:                     // 'rename' 'document-node'
    case 62686:                     // 'rename' 'element'
    case 63198:                     // 'rename' 'else'
    case 63710:                     // 'rename' 'empty'
    case 64222:                     // 'rename' 'empty-sequence'
    case 64734:                     // 'rename' 'encoding'
    case 65246:                     // 'rename' 'end'
    case 66270:                     // 'rename' 'eq'
    case 66782:                     // 'rename' 'every'
    case 67806:                     // 'rename' 'except'
    case 68318:                     // 'rename' 'exit'
    case 68830:                     // 'rename' 'external'
    case 69342:                     // 'rename' 'false'
    case 69854:                     // 'rename' 'first'
    case 70366:                     // 'rename' 'following'
    case 70878:                     // 'rename' 'following-sibling'
    case 71390:                     // 'rename' 'for'
    case 72926:                     // 'rename' 'from'
    case 73438:                     // 'rename' 'ft-option'
    case 75486:                     // 'rename' 'function'
    case 75998:                     // 'rename' 'ge'
    case 77022:                     // 'rename' 'group'
    case 78046:                     // 'rename' 'gt'
    case 78558:                     // 'rename' 'idiv'
    case 79070:                     // 'rename' 'if'
    case 79582:                     // 'rename' 'import'
    case 80094:                     // 'rename' 'in'
    case 80606:                     // 'rename' 'index'
    case 82654:                     // 'rename' 'insert'
    case 83166:                     // 'rename' 'instance'
    case 83678:                     // 'rename' 'integrity'
    case 84190:                     // 'rename' 'intersect'
    case 84702:                     // 'rename' 'into'
    case 85214:                     // 'rename' 'is'
    case 85726:                     // 'rename' 'item'
    case 86238:                     // 'rename' 'json'
    case 86750:                     // 'rename' 'json-item'
    case 87262:                     // 'rename' 'jsoniq'
    case 88798:                     // 'rename' 'last'
    case 89310:                     // 'rename' 'lax'
    case 89822:                     // 'rename' 'le'
    case 90846:                     // 'rename' 'let'
    case 91870:                     // 'rename' 'loop'
    case 92894:                     // 'rename' 'lt'
    case 93918:                     // 'rename' 'mod'
    case 94430:                     // 'rename' 'modify'
    case 94942:                     // 'rename' 'module'
    case 95966:                     // 'rename' 'namespace'
    case 96478:                     // 'rename' 'namespace-node'
    case 96990:                     // 'rename' 'ne'
    case 100062:                    // 'rename' 'nodes'
    case 101086:                    // 'rename' 'null'
    case 101598:                    // 'rename' 'object'
    case 103646:                    // 'rename' 'only'
    case 104158:                    // 'rename' 'option'
    case 104670:                    // 'rename' 'or'
    case 105182:                    // 'rename' 'order'
    case 105694:                    // 'rename' 'ordered'
    case 106206:                    // 'rename' 'ordering'
    case 107742:                    // 'rename' 'parent'
    case 110814:                    // 'rename' 'preceding'
    case 111326:                    // 'rename' 'preceding-sibling'
    case 112862:                    // 'rename' 'processing-instruction'
    case 113886:                    // 'rename' 'rename'
    case 114398:                    // 'rename' 'replace'
    case 114910:                    // 'rename' 'return'
    case 115422:                    // 'rename' 'returning'
    case 115934:                    // 'rename' 'revalidation'
    case 116958:                    // 'rename' 'satisfies'
    case 117470:                    // 'rename' 'schema'
    case 117982:                    // 'rename' 'schema-attribute'
    case 118494:                    // 'rename' 'schema-element'
    case 119006:                    // 'rename' 'score'
    case 119518:                    // 'rename' 'select'
    case 120030:                    // 'rename' 'self'
    case 122590:                    // 'rename' 'sliding'
    case 123102:                    // 'rename' 'some'
    case 123614:                    // 'rename' 'stable'
    case 124126:                    // 'rename' 'start'
    case 125662:                    // 'rename' 'strict'
    case 126686:                    // 'rename' 'structured-item'
    case 127198:                    // 'rename' 'switch'
    case 127710:                    // 'rename' 'text'
    case 129758:                    // 'rename' 'to'
    case 130270:                    // 'rename' 'treat'
    case 130782:                    // 'rename' 'true'
    case 131294:                    // 'rename' 'try'
    case 131806:                    // 'rename' 'tumbling'
    case 132318:                    // 'rename' 'type'
    case 132830:                    // 'rename' 'typeswitch'
    case 133342:                    // 'rename' 'union'
    case 134366:                    // 'rename' 'unordered'
    case 134878:                    // 'rename' 'updating'
    case 136414:                    // 'rename' 'validate'
    case 136926:                    // 'rename' 'value'
    case 137438:                    // 'rename' 'variable'
    case 137950:                    // 'rename' 'version'
    case 139486:                    // 'rename' 'where'
    case 139998:                    // 'rename' 'while'
    case 141534:                    // 'rename' 'with'
    case 144094:                    // 'rename' '{'
    case 145118:                    // 'rename' '{|'
      parse_JSONRenameExpr();
      break;
    case -11:
      parse_JSONReplaceExpr();
      break;
    case -12:
    case 3150:                      // 'append' EQName^Token
    case 4174:                      // 'append' IntegerLiteral
    case 4686:                      // 'append' DecimalLiteral
    case 5198:                      // 'append' DoubleLiteral
    case 5710:                      // 'append' StringLiteral
    case 15950:                     // 'append' '$'
    case 16462:                     // 'append' '$$'
    case 16974:                     // 'append' '%'
    case 18510:                     // 'append' '(#'
    case 21070:                     // 'append' '+'
    case 22094:                     // 'append' '-'
    case 24142:                     // 'append' '/'
    case 24654:                     // 'append' '//'
    case 28238:                     // 'append' '<'
    case 28750:                     // 'append' '<!--'
    case 30798:                     // 'append' '<?'
    case 35406:                     // 'append' '['
    case 36430:                     // 'append' 'after'
    case 37454:                     // 'append' 'allowing'
    case 37966:                     // 'append' 'ancestor'
    case 38478:                     // 'append' 'ancestor-or-self'
    case 38990:                     // 'append' 'and'
    case 40014:                     // 'append' 'append'
    case 40526:                     // 'append' 'array'
    case 41038:                     // 'append' 'as'
    case 41550:                     // 'append' 'ascending'
    case 42062:                     // 'append' 'at'
    case 42574:                     // 'append' 'attribute'
    case 43086:                     // 'append' 'base-uri'
    case 43598:                     // 'append' 'before'
    case 44110:                     // 'append' 'boundary-space'
    case 44622:                     // 'append' 'break'
    case 45646:                     // 'append' 'case'
    case 46158:                     // 'append' 'cast'
    case 46670:                     // 'append' 'castable'
    case 47182:                     // 'append' 'catch'
    case 48206:                     // 'append' 'child'
    case 48718:                     // 'append' 'collation'
    case 49742:                     // 'append' 'comment'
    case 50254:                     // 'append' 'constraint'
    case 50766:                     // 'append' 'construction'
    case 52302:                     // 'append' 'context'
    case 52814:                     // 'append' 'continue'
    case 53326:                     // 'append' 'copy'
    case 53838:                     // 'append' 'copy-namespaces'
    case 54350:                     // 'append' 'count'
    case 54862:                     // 'append' 'decimal-format'
    case 55886:                     // 'append' 'declare'
    case 56398:                     // 'append' 'default'
    case 56910:                     // 'append' 'delete'
    case 57422:                     // 'append' 'descendant'
    case 57934:                     // 'append' 'descendant-or-self'
    case 58446:                     // 'append' 'descending'
    case 61006:                     // 'append' 'div'
    case 61518:                     // 'append' 'document'
    case 62030:                     // 'append' 'document-node'
    case 62542:                     // 'append' 'element'
    case 63054:                     // 'append' 'else'
    case 63566:                     // 'append' 'empty'
    case 64078:                     // 'append' 'empty-sequence'
    case 64590:                     // 'append' 'encoding'
    case 65102:                     // 'append' 'end'
    case 66126:                     // 'append' 'eq'
    case 66638:                     // 'append' 'every'
    case 67662:                     // 'append' 'except'
    case 68174:                     // 'append' 'exit'
    case 68686:                     // 'append' 'external'
    case 69198:                     // 'append' 'false'
    case 69710:                     // 'append' 'first'
    case 70222:                     // 'append' 'following'
    case 70734:                     // 'append' 'following-sibling'
    case 71246:                     // 'append' 'for'
    case 72782:                     // 'append' 'from'
    case 73294:                     // 'append' 'ft-option'
    case 75342:                     // 'append' 'function'
    case 75854:                     // 'append' 'ge'
    case 76878:                     // 'append' 'group'
    case 77902:                     // 'append' 'gt'
    case 78414:                     // 'append' 'idiv'
    case 78926:                     // 'append' 'if'
    case 79438:                     // 'append' 'import'
    case 79950:                     // 'append' 'in'
    case 80462:                     // 'append' 'index'
    case 82510:                     // 'append' 'insert'
    case 83022:                     // 'append' 'instance'
    case 83534:                     // 'append' 'integrity'
    case 84046:                     // 'append' 'intersect'
    case 84558:                     // 'append' 'into'
    case 85070:                     // 'append' 'is'
    case 85582:                     // 'append' 'item'
    case 86094:                     // 'append' 'json'
    case 86606:                     // 'append' 'json-item'
    case 87118:                     // 'append' 'jsoniq'
    case 88654:                     // 'append' 'last'
    case 89166:                     // 'append' 'lax'
    case 89678:                     // 'append' 'le'
    case 90702:                     // 'append' 'let'
    case 91726:                     // 'append' 'loop'
    case 92750:                     // 'append' 'lt'
    case 93774:                     // 'append' 'mod'
    case 94286:                     // 'append' 'modify'
    case 94798:                     // 'append' 'module'
    case 95822:                     // 'append' 'namespace'
    case 96334:                     // 'append' 'namespace-node'
    case 96846:                     // 'append' 'ne'
    case 99406:                     // 'append' 'node'
    case 99918:                     // 'append' 'nodes'
    case 100430:                    // 'append' 'not'
    case 100942:                    // 'append' 'null'
    case 101454:                    // 'append' 'object'
    case 103502:                    // 'append' 'only'
    case 104014:                    // 'append' 'option'
    case 104526:                    // 'append' 'or'
    case 105038:                    // 'append' 'order'
    case 105550:                    // 'append' 'ordered'
    case 106062:                    // 'append' 'ordering'
    case 107598:                    // 'append' 'parent'
    case 110670:                    // 'append' 'preceding'
    case 111182:                    // 'append' 'preceding-sibling'
    case 112718:                    // 'append' 'processing-instruction'
    case 113742:                    // 'append' 'rename'
    case 114254:                    // 'append' 'replace'
    case 114766:                    // 'append' 'return'
    case 115278:                    // 'append' 'returning'
    case 115790:                    // 'append' 'revalidation'
    case 116814:                    // 'append' 'satisfies'
    case 117326:                    // 'append' 'schema'
    case 117838:                    // 'append' 'schema-attribute'
    case 118350:                    // 'append' 'schema-element'
    case 118862:                    // 'append' 'score'
    case 119374:                    // 'append' 'select'
    case 119886:                    // 'append' 'self'
    case 122446:                    // 'append' 'sliding'
    case 122958:                    // 'append' 'some'
    case 123470:                    // 'append' 'stable'
    case 123982:                    // 'append' 'start'
    case 125518:                    // 'append' 'strict'
    case 126542:                    // 'append' 'structured-item'
    case 127054:                    // 'append' 'switch'
    case 127566:                    // 'append' 'text'
    case 129614:                    // 'append' 'to'
    case 130126:                    // 'append' 'treat'
    case 130638:                    // 'append' 'true'
    case 131150:                    // 'append' 'try'
    case 131662:                    // 'append' 'tumbling'
    case 132174:                    // 'append' 'type'
    case 132686:                    // 'append' 'typeswitch'
    case 133198:                    // 'append' 'union'
    case 134222:                    // 'append' 'unordered'
    case 134734:                    // 'append' 'updating'
    case 136270:                    // 'append' 'validate'
    case 136782:                    // 'append' 'value'
    case 137294:                    // 'append' 'variable'
    case 137806:                    // 'append' 'version'
    case 139342:                    // 'append' 'where'
    case 139854:                    // 'append' 'while'
    case 141390:                    // 'append' 'with'
    case 143950:                    // 'append' '{'
    case 144974:                    // 'append' '{|'
      parse_JSONAppendExpr();
      break;
    default:
      parse_OrExpr();
    }
    eventHandler.endNonterminal("ExprSimple", e0);
  }

  function try_ExprSimple()
  {
    switch (l1)
    {
    case 78:                        // 'append'
      lookahead2W(269);             // EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral | StringLiteral |
      break;
    case 161:                       // 'insert'
      lookahead2W(276);             // EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral | StringLiteral |
      break;
    case 223:                       // 'replace'
      lookahead2W(170);             // S^WS | '#' | '(' | '(:' | 'node' | 'value'
      break;
    case 111:                       // 'delete'
    case 222:                       // 'rename'
      lookahead2W(261);             // EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral | StringLiteral |
      break;
    case 104:                       // 'copy'
    case 130:                       // 'every'
    case 240:                       // 'some'
      lookahead2W(143);             // S^WS | '#' | '$' | '(' | '(:'
      break;
    default:
      lk = l1;
    }
    if (lk == 17998                 // 'append' '('
     || lk == 18031                 // 'delete' '('
     || lk == 18081                 // 'insert' '('
     || lk == 18142                 // 'rename' '('
     || lk == 99439                 // 'delete' 'node'
     || lk == 99489                 // 'insert' 'node'
     || lk == 99550                 // 'rename' 'node'
     || lk == 99951                 // 'delete' 'nodes'
     || lk == 100001                // 'insert' 'nodes'
     || lk == 136927)               // 'replace' 'value'
    {
      lk = memoized(10, e0);
      if (lk == 0)
      {
        var b0A = b0; var e0A = e0; var l1A = l1;
        var b1A = b1; var e1A = e1; var l2A = l2;
        var b2A = b2; var e2A = e2;
        try
        {
          try_OrExpr();
          memoize(10, e0A, -2);
          lk = -13;
        }
        catch (p2A)
        {
          try
          {
            b0 = b0A; e0 = e0A; l1 = l1A; if (l1 == 0) {end = e0A;} else {
            b1 = b1A; e1 = e1A; l2 = l2A; if (l2 == 0) {end = e1A;} else {
            b2 = b2A; e2 = e2A; end = e2A; }}
            try_InsertExpr();
            memoize(10, e0A, -3);
            lk = -13;
          }
          catch (p3A)
          {
            try
            {
              b0 = b0A; e0 = e0A; l1 = l1A; if (l1 == 0) {end = e0A;} else {
              b1 = b1A; e1 = e1A; l2 = l2A; if (l2 == 0) {end = e1A;} else {
              b2 = b2A; e2 = e2A; end = e2A; }}
              try_DeleteExpr();
              memoize(10, e0A, -4);
              lk = -13;
            }
            catch (p4A)
            {
              try
              {
                b0 = b0A; e0 = e0A; l1 = l1A; if (l1 == 0) {end = e0A;} else {
                b1 = b1A; e1 = e1A; l2 = l2A; if (l2 == 0) {end = e1A;} else {
                b2 = b2A; e2 = e2A; end = e2A; }}
                try_RenameExpr();
                memoize(10, e0A, -5);
                lk = -13;
              }
              catch (p5A)
              {
                try
                {
                  b0 = b0A; e0 = e0A; l1 = l1A; if (l1 == 0) {end = e0A;} else {
                  b1 = b1A; e1 = e1A; l2 = l2A; if (l2 == 0) {end = e1A;} else {
                  b2 = b2A; e2 = e2A; end = e2A; }}
                  try_ReplaceExpr();
                  memoize(10, e0A, -6);
                  lk = -13;
                }
                catch (p6A)
                {
                  try
                  {
                    b0 = b0A; e0 = e0A; l1 = l1A; if (l1 == 0) {end = e0A;} else {
                    b1 = b1A; e1 = e1A; l2 = l2A; if (l2 == 0) {end = e1A;} else {
                    b2 = b2A; e2 = e2A; end = e2A; }}
                    try_JSONDeleteExpr();
                    memoize(10, e0A, -8);
                    lk = -13;
                  }
                  catch (p8A)
                  {
                    try
                    {
                      b0 = b0A; e0 = e0A; l1 = l1A; if (l1 == 0) {end = e0A;} else {
                      b1 = b1A; e1 = e1A; l2 = l2A; if (l2 == 0) {end = e1A;} else {
                      b2 = b2A; e2 = e2A; end = e2A; }}
                      try_JSONInsertExpr();
                      memoize(10, e0A, -9);
                      lk = -13;
                    }
                    catch (p9A)
                    {
                      try
                      {
                        b0 = b0A; e0 = e0A; l1 = l1A; if (l1 == 0) {end = e0A;} else {
                        b1 = b1A; e1 = e1A; l2 = l2A; if (l2 == 0) {end = e1A;} else {
                        b2 = b2A; e2 = e2A; end = e2A; }}
                        try_JSONRenameExpr();
                        memoize(10, e0A, -10);
                        lk = -13;
                      }
                      catch (p10A)
                      {
                        try
                        {
                          b0 = b0A; e0 = e0A; l1 = l1A; if (l1 == 0) {end = e0A;} else {
                          b1 = b1A; e1 = e1A; l2 = l2A; if (l2 == 0) {end = e1A;} else {
                          b2 = b2A; e2 = e2A; end = e2A; }}
                          try_JSONReplaceExpr();
                          memoize(10, e0A, -11);
                          lk = -13;
                        }
                        catch (p11A)
                        {
                          lk = -12;
                          b0 = b0A; e0 = e0A; l1 = l1A; if (l1 == 0) {end = e0A;} else {
                          b1 = b1A; e1 = e1A; l2 = l2A; if (l2 == 0) {end = e1A;} else {
                          b2 = b2A; e2 = e2A; end = e2A; }}
                          memoize(10, e0A, -12);
                        }
                      }
                    }
                  }
                }
              }
            }
          }
        }
      }
    }
    switch (lk)
    {
    case 16002:                     // 'every' '$'
    case 16112:                     // 'some' '$'
      try_QuantifiedExpr();
      break;
    case -3:
      try_InsertExpr();
      break;
    case -4:
      try_DeleteExpr();
      break;
    case -5:
      try_RenameExpr();
      break;
    case -6:
    case 99551:                     // 'replace' 'node'
      try_ReplaceExpr();
      break;
    case 15976:                     // 'copy' '$'
      try_TransformExpr();
      break;
    case -8:
    case 3183:                      // 'delete' EQName^Token
    case 4207:                      // 'delete' IntegerLiteral
    case 4719:                      // 'delete' DecimalLiteral
    case 5231:                      // 'delete' DoubleLiteral
    case 5743:                      // 'delete' StringLiteral
    case 15983:                     // 'delete' '$'
    case 16495:                     // 'delete' '$$'
    case 17007:                     // 'delete' '%'
    case 28271:                     // 'delete' '<'
    case 28783:                     // 'delete' '<!--'
    case 30831:                     // 'delete' '<?'
    case 35439:                     // 'delete' '['
    case 36463:                     // 'delete' 'after'
    case 37487:                     // 'delete' 'allowing'
    case 37999:                     // 'delete' 'ancestor'
    case 38511:                     // 'delete' 'ancestor-or-self'
    case 39023:                     // 'delete' 'and'
    case 40047:                     // 'delete' 'append'
    case 40559:                     // 'delete' 'array'
    case 41071:                     // 'delete' 'as'
    case 41583:                     // 'delete' 'ascending'
    case 42095:                     // 'delete' 'at'
    case 42607:                     // 'delete' 'attribute'
    case 43119:                     // 'delete' 'base-uri'
    case 43631:                     // 'delete' 'before'
    case 44143:                     // 'delete' 'boundary-space'
    case 44655:                     // 'delete' 'break'
    case 45679:                     // 'delete' 'case'
    case 46191:                     // 'delete' 'cast'
    case 46703:                     // 'delete' 'castable'
    case 47215:                     // 'delete' 'catch'
    case 48239:                     // 'delete' 'child'
    case 48751:                     // 'delete' 'collation'
    case 49775:                     // 'delete' 'comment'
    case 50287:                     // 'delete' 'constraint'
    case 50799:                     // 'delete' 'construction'
    case 52335:                     // 'delete' 'context'
    case 52847:                     // 'delete' 'continue'
    case 53359:                     // 'delete' 'copy'
    case 53871:                     // 'delete' 'copy-namespaces'
    case 54383:                     // 'delete' 'count'
    case 54895:                     // 'delete' 'decimal-format'
    case 55919:                     // 'delete' 'declare'
    case 56431:                     // 'delete' 'default'
    case 56943:                     // 'delete' 'delete'
    case 57455:                     // 'delete' 'descendant'
    case 57967:                     // 'delete' 'descendant-or-self'
    case 58479:                     // 'delete' 'descending'
    case 61039:                     // 'delete' 'div'
    case 61551:                     // 'delete' 'document'
    case 62063:                     // 'delete' 'document-node'
    case 62575:                     // 'delete' 'element'
    case 63087:                     // 'delete' 'else'
    case 63599:                     // 'delete' 'empty'
    case 64111:                     // 'delete' 'empty-sequence'
    case 64623:                     // 'delete' 'encoding'
    case 65135:                     // 'delete' 'end'
    case 66159:                     // 'delete' 'eq'
    case 66671:                     // 'delete' 'every'
    case 67695:                     // 'delete' 'except'
    case 68207:                     // 'delete' 'exit'
    case 68719:                     // 'delete' 'external'
    case 69231:                     // 'delete' 'false'
    case 69743:                     // 'delete' 'first'
    case 70255:                     // 'delete' 'following'
    case 70767:                     // 'delete' 'following-sibling'
    case 71279:                     // 'delete' 'for'
    case 72815:                     // 'delete' 'from'
    case 73327:                     // 'delete' 'ft-option'
    case 75375:                     // 'delete' 'function'
    case 75887:                     // 'delete' 'ge'
    case 76911:                     // 'delete' 'group'
    case 77935:                     // 'delete' 'gt'
    case 78447:                     // 'delete' 'idiv'
    case 78959:                     // 'delete' 'if'
    case 79471:                     // 'delete' 'import'
    case 79983:                     // 'delete' 'in'
    case 80495:                     // 'delete' 'index'
    case 82543:                     // 'delete' 'insert'
    case 83055:                     // 'delete' 'instance'
    case 83567:                     // 'delete' 'integrity'
    case 84079:                     // 'delete' 'intersect'
    case 84591:                     // 'delete' 'into'
    case 85103:                     // 'delete' 'is'
    case 85615:                     // 'delete' 'item'
    case 86127:                     // 'delete' 'json'
    case 86639:                     // 'delete' 'json-item'
    case 87151:                     // 'delete' 'jsoniq'
    case 88687:                     // 'delete' 'last'
    case 89199:                     // 'delete' 'lax'
    case 89711:                     // 'delete' 'le'
    case 90735:                     // 'delete' 'let'
    case 91759:                     // 'delete' 'loop'
    case 92783:                     // 'delete' 'lt'
    case 93807:                     // 'delete' 'mod'
    case 94319:                     // 'delete' 'modify'
    case 94831:                     // 'delete' 'module'
    case 95855:                     // 'delete' 'namespace'
    case 96367:                     // 'delete' 'namespace-node'
    case 96879:                     // 'delete' 'ne'
    case 100975:                    // 'delete' 'null'
    case 101487:                    // 'delete' 'object'
    case 103535:                    // 'delete' 'only'
    case 104047:                    // 'delete' 'option'
    case 104559:                    // 'delete' 'or'
    case 105071:                    // 'delete' 'order'
    case 105583:                    // 'delete' 'ordered'
    case 106095:                    // 'delete' 'ordering'
    case 107631:                    // 'delete' 'parent'
    case 110703:                    // 'delete' 'preceding'
    case 111215:                    // 'delete' 'preceding-sibling'
    case 112751:                    // 'delete' 'processing-instruction'
    case 113775:                    // 'delete' 'rename'
    case 114287:                    // 'delete' 'replace'
    case 114799:                    // 'delete' 'return'
    case 115311:                    // 'delete' 'returning'
    case 115823:                    // 'delete' 'revalidation'
    case 116847:                    // 'delete' 'satisfies'
    case 117359:                    // 'delete' 'schema'
    case 117871:                    // 'delete' 'schema-attribute'
    case 118383:                    // 'delete' 'schema-element'
    case 118895:                    // 'delete' 'score'
    case 119407:                    // 'delete' 'select'
    case 119919:                    // 'delete' 'self'
    case 122479:                    // 'delete' 'sliding'
    case 122991:                    // 'delete' 'some'
    case 123503:                    // 'delete' 'stable'
    case 124015:                    // 'delete' 'start'
    case 125551:                    // 'delete' 'strict'
    case 126575:                    // 'delete' 'structured-item'
    case 127087:                    // 'delete' 'switch'
    case 127599:                    // 'delete' 'text'
    case 129647:                    // 'delete' 'to'
    case 130159:                    // 'delete' 'treat'
    case 130671:                    // 'delete' 'true'
    case 131183:                    // 'delete' 'try'
    case 131695:                    // 'delete' 'tumbling'
    case 132207:                    // 'delete' 'type'
    case 132719:                    // 'delete' 'typeswitch'
    case 133231:                    // 'delete' 'union'
    case 134255:                    // 'delete' 'unordered'
    case 134767:                    // 'delete' 'updating'
    case 136303:                    // 'delete' 'validate'
    case 136815:                    // 'delete' 'value'
    case 137327:                    // 'delete' 'variable'
    case 137839:                    // 'delete' 'version'
    case 139375:                    // 'delete' 'where'
    case 139887:                    // 'delete' 'while'
    case 141423:                    // 'delete' 'with'
    case 143983:                    // 'delete' '{'
    case 145007:                    // 'delete' '{|'
      try_JSONDeleteExpr();
      break;
    case -9:
    case 3233:                      // 'insert' EQName^Token
    case 4257:                      // 'insert' IntegerLiteral
    case 4769:                      // 'insert' DecimalLiteral
    case 5281:                      // 'insert' DoubleLiteral
    case 5793:                      // 'insert' StringLiteral
    case 9889:                      // 'insert' NCName^Token
    case 16033:                     // 'insert' '$'
    case 16545:                     // 'insert' '$$'
    case 17057:                     // 'insert' '%'
    case 18593:                     // 'insert' '(#'
    case 21153:                     // 'insert' '+'
    case 22177:                     // 'insert' '-'
    case 24225:                     // 'insert' '/'
    case 24737:                     // 'insert' '//'
    case 28321:                     // 'insert' '<'
    case 28833:                     // 'insert' '<!--'
    case 30881:                     // 'insert' '<?'
    case 35489:                     // 'insert' '['
    case 36513:                     // 'insert' 'after'
    case 37537:                     // 'insert' 'allowing'
    case 38049:                     // 'insert' 'ancestor'
    case 38561:                     // 'insert' 'ancestor-or-self'
    case 39073:                     // 'insert' 'and'
    case 40097:                     // 'insert' 'append'
    case 40609:                     // 'insert' 'array'
    case 41121:                     // 'insert' 'as'
    case 41633:                     // 'insert' 'ascending'
    case 42145:                     // 'insert' 'at'
    case 42657:                     // 'insert' 'attribute'
    case 43169:                     // 'insert' 'base-uri'
    case 43681:                     // 'insert' 'before'
    case 44193:                     // 'insert' 'boundary-space'
    case 44705:                     // 'insert' 'break'
    case 45729:                     // 'insert' 'case'
    case 46241:                     // 'insert' 'cast'
    case 46753:                     // 'insert' 'castable'
    case 47265:                     // 'insert' 'catch'
    case 48289:                     // 'insert' 'child'
    case 48801:                     // 'insert' 'collation'
    case 49825:                     // 'insert' 'comment'
    case 50337:                     // 'insert' 'constraint'
    case 50849:                     // 'insert' 'construction'
    case 52385:                     // 'insert' 'context'
    case 52897:                     // 'insert' 'continue'
    case 53409:                     // 'insert' 'copy'
    case 53921:                     // 'insert' 'copy-namespaces'
    case 54433:                     // 'insert' 'count'
    case 54945:                     // 'insert' 'decimal-format'
    case 55969:                     // 'insert' 'declare'
    case 56481:                     // 'insert' 'default'
    case 56993:                     // 'insert' 'delete'
    case 57505:                     // 'insert' 'descendant'
    case 58017:                     // 'insert' 'descendant-or-self'
    case 58529:                     // 'insert' 'descending'
    case 61089:                     // 'insert' 'div'
    case 61601:                     // 'insert' 'document'
    case 62113:                     // 'insert' 'document-node'
    case 62625:                     // 'insert' 'element'
    case 63137:                     // 'insert' 'else'
    case 63649:                     // 'insert' 'empty'
    case 64161:                     // 'insert' 'empty-sequence'
    case 64673:                     // 'insert' 'encoding'
    case 65185:                     // 'insert' 'end'
    case 66209:                     // 'insert' 'eq'
    case 66721:                     // 'insert' 'every'
    case 67745:                     // 'insert' 'except'
    case 68257:                     // 'insert' 'exit'
    case 68769:                     // 'insert' 'external'
    case 69281:                     // 'insert' 'false'
    case 69793:                     // 'insert' 'first'
    case 70305:                     // 'insert' 'following'
    case 70817:                     // 'insert' 'following-sibling'
    case 71329:                     // 'insert' 'for'
    case 72865:                     // 'insert' 'from'
    case 73377:                     // 'insert' 'ft-option'
    case 75425:                     // 'insert' 'function'
    case 75937:                     // 'insert' 'ge'
    case 76961:                     // 'insert' 'group'
    case 77985:                     // 'insert' 'gt'
    case 78497:                     // 'insert' 'idiv'
    case 79009:                     // 'insert' 'if'
    case 79521:                     // 'insert' 'import'
    case 80033:                     // 'insert' 'in'
    case 80545:                     // 'insert' 'index'
    case 82593:                     // 'insert' 'insert'
    case 83105:                     // 'insert' 'instance'
    case 83617:                     // 'insert' 'integrity'
    case 84129:                     // 'insert' 'intersect'
    case 84641:                     // 'insert' 'into'
    case 85153:                     // 'insert' 'is'
    case 85665:                     // 'insert' 'item'
    case 86177:                     // 'insert' 'json'
    case 86689:                     // 'insert' 'json-item'
    case 87201:                     // 'insert' 'jsoniq'
    case 88737:                     // 'insert' 'last'
    case 89249:                     // 'insert' 'lax'
    case 89761:                     // 'insert' 'le'
    case 90785:                     // 'insert' 'let'
    case 91809:                     // 'insert' 'loop'
    case 92833:                     // 'insert' 'lt'
    case 93857:                     // 'insert' 'mod'
    case 94369:                     // 'insert' 'modify'
    case 94881:                     // 'insert' 'module'
    case 95905:                     // 'insert' 'namespace'
    case 96417:                     // 'insert' 'namespace-node'
    case 96929:                     // 'insert' 'ne'
    case 100513:                    // 'insert' 'not'
    case 101025:                    // 'insert' 'null'
    case 101537:                    // 'insert' 'object'
    case 103585:                    // 'insert' 'only'
    case 104097:                    // 'insert' 'option'
    case 104609:                    // 'insert' 'or'
    case 105121:                    // 'insert' 'order'
    case 105633:                    // 'insert' 'ordered'
    case 106145:                    // 'insert' 'ordering'
    case 107681:                    // 'insert' 'parent'
    case 110753:                    // 'insert' 'preceding'
    case 111265:                    // 'insert' 'preceding-sibling'
    case 112801:                    // 'insert' 'processing-instruction'
    case 113825:                    // 'insert' 'rename'
    case 114337:                    // 'insert' 'replace'
    case 114849:                    // 'insert' 'return'
    case 115361:                    // 'insert' 'returning'
    case 115873:                    // 'insert' 'revalidation'
    case 116897:                    // 'insert' 'satisfies'
    case 117409:                    // 'insert' 'schema'
    case 117921:                    // 'insert' 'schema-attribute'
    case 118433:                    // 'insert' 'schema-element'
    case 118945:                    // 'insert' 'score'
    case 119457:                    // 'insert' 'select'
    case 119969:                    // 'insert' 'self'
    case 122529:                    // 'insert' 'sliding'
    case 123041:                    // 'insert' 'some'
    case 123553:                    // 'insert' 'stable'
    case 124065:                    // 'insert' 'start'
    case 125601:                    // 'insert' 'strict'
    case 126625:                    // 'insert' 'structured-item'
    case 127137:                    // 'insert' 'switch'
    case 127649:                    // 'insert' 'text'
    case 129697:                    // 'insert' 'to'
    case 130209:                    // 'insert' 'treat'
    case 130721:                    // 'insert' 'true'
    case 131233:                    // 'insert' 'try'
    case 131745:                    // 'insert' 'tumbling'
    case 132257:                    // 'insert' 'type'
    case 132769:                    // 'insert' 'typeswitch'
    case 133281:                    // 'insert' 'union'
    case 134305:                    // 'insert' 'unordered'
    case 134817:                    // 'insert' 'updating'
    case 136353:                    // 'insert' 'validate'
    case 136865:                    // 'insert' 'value'
    case 137377:                    // 'insert' 'variable'
    case 137889:                    // 'insert' 'version'
    case 139425:                    // 'insert' 'where'
    case 139937:                    // 'insert' 'while'
    case 141473:                    // 'insert' 'with'
    case 144033:                    // 'insert' '{'
    case 145057:                    // 'insert' '{|'
      try_JSONInsertExpr();
      break;
    case -10:
    case 3294:                      // 'rename' EQName^Token
    case 4318:                      // 'rename' IntegerLiteral
    case 4830:                      // 'rename' DecimalLiteral
    case 5342:                      // 'rename' DoubleLiteral
    case 5854:                      // 'rename' StringLiteral
    case 16094:                     // 'rename' '$'
    case 16606:                     // 'rename' '$$'
    case 17118:                     // 'rename' '%'
    case 28382:                     // 'rename' '<'
    case 28894:                     // 'rename' '<!--'
    case 30942:                     // 'rename' '<?'
    case 35550:                     // 'rename' '['
    case 36574:                     // 'rename' 'after'
    case 37598:                     // 'rename' 'allowing'
    case 38110:                     // 'rename' 'ancestor'
    case 38622:                     // 'rename' 'ancestor-or-self'
    case 39134:                     // 'rename' 'and'
    case 40158:                     // 'rename' 'append'
    case 40670:                     // 'rename' 'array'
    case 41182:                     // 'rename' 'as'
    case 41694:                     // 'rename' 'ascending'
    case 42206:                     // 'rename' 'at'
    case 42718:                     // 'rename' 'attribute'
    case 43230:                     // 'rename' 'base-uri'
    case 43742:                     // 'rename' 'before'
    case 44254:                     // 'rename' 'boundary-space'
    case 44766:                     // 'rename' 'break'
    case 45790:                     // 'rename' 'case'
    case 46302:                     // 'rename' 'cast'
    case 46814:                     // 'rename' 'castable'
    case 47326:                     // 'rename' 'catch'
    case 48350:                     // 'rename' 'child'
    case 48862:                     // 'rename' 'collation'
    case 49886:                     // 'rename' 'comment'
    case 50398:                     // 'rename' 'constraint'
    case 50910:                     // 'rename' 'construction'
    case 52446:                     // 'rename' 'context'
    case 52958:                     // 'rename' 'continue'
    case 53470:                     // 'rename' 'copy'
    case 53982:                     // 'rename' 'copy-namespaces'
    case 54494:                     // 'rename' 'count'
    case 55006:                     // 'rename' 'decimal-format'
    case 56030:                     // 'rename' 'declare'
    case 56542:                     // 'rename' 'default'
    case 57054:                     // 'rename' 'delete'
    case 57566:                     // 'rename' 'descendant'
    case 58078:                     // 'rename' 'descendant-or-self'
    case 58590:                     // 'rename' 'descending'
    case 61150:                     // 'rename' 'div'
    case 61662:                     // 'rename' 'document'
    case 62174:                     // 'rename' 'document-node'
    case 62686:                     // 'rename' 'element'
    case 63198:                     // 'rename' 'else'
    case 63710:                     // 'rename' 'empty'
    case 64222:                     // 'rename' 'empty-sequence'
    case 64734:                     // 'rename' 'encoding'
    case 65246:                     // 'rename' 'end'
    case 66270:                     // 'rename' 'eq'
    case 66782:                     // 'rename' 'every'
    case 67806:                     // 'rename' 'except'
    case 68318:                     // 'rename' 'exit'
    case 68830:                     // 'rename' 'external'
    case 69342:                     // 'rename' 'false'
    case 69854:                     // 'rename' 'first'
    case 70366:                     // 'rename' 'following'
    case 70878:                     // 'rename' 'following-sibling'
    case 71390:                     // 'rename' 'for'
    case 72926:                     // 'rename' 'from'
    case 73438:                     // 'rename' 'ft-option'
    case 75486:                     // 'rename' 'function'
    case 75998:                     // 'rename' 'ge'
    case 77022:                     // 'rename' 'group'
    case 78046:                     // 'rename' 'gt'
    case 78558:                     // 'rename' 'idiv'
    case 79070:                     // 'rename' 'if'
    case 79582:                     // 'rename' 'import'
    case 80094:                     // 'rename' 'in'
    case 80606:                     // 'rename' 'index'
    case 82654:                     // 'rename' 'insert'
    case 83166:                     // 'rename' 'instance'
    case 83678:                     // 'rename' 'integrity'
    case 84190:                     // 'rename' 'intersect'
    case 84702:                     // 'rename' 'into'
    case 85214:                     // 'rename' 'is'
    case 85726:                     // 'rename' 'item'
    case 86238:                     // 'rename' 'json'
    case 86750:                     // 'rename' 'json-item'
    case 87262:                     // 'rename' 'jsoniq'
    case 88798:                     // 'rename' 'last'
    case 89310:                     // 'rename' 'lax'
    case 89822:                     // 'rename' 'le'
    case 90846:                     // 'rename' 'let'
    case 91870:                     // 'rename' 'loop'
    case 92894:                     // 'rename' 'lt'
    case 93918:                     // 'rename' 'mod'
    case 94430:                     // 'rename' 'modify'
    case 94942:                     // 'rename' 'module'
    case 95966:                     // 'rename' 'namespace'
    case 96478:                     // 'rename' 'namespace-node'
    case 96990:                     // 'rename' 'ne'
    case 100062:                    // 'rename' 'nodes'
    case 101086:                    // 'rename' 'null'
    case 101598:                    // 'rename' 'object'
    case 103646:                    // 'rename' 'only'
    case 104158:                    // 'rename' 'option'
    case 104670:                    // 'rename' 'or'
    case 105182:                    // 'rename' 'order'
    case 105694:                    // 'rename' 'ordered'
    case 106206:                    // 'rename' 'ordering'
    case 107742:                    // 'rename' 'parent'
    case 110814:                    // 'rename' 'preceding'
    case 111326:                    // 'rename' 'preceding-sibling'
    case 112862:                    // 'rename' 'processing-instruction'
    case 113886:                    // 'rename' 'rename'
    case 114398:                    // 'rename' 'replace'
    case 114910:                    // 'rename' 'return'
    case 115422:                    // 'rename' 'returning'
    case 115934:                    // 'rename' 'revalidation'
    case 116958:                    // 'rename' 'satisfies'
    case 117470:                    // 'rename' 'schema'
    case 117982:                    // 'rename' 'schema-attribute'
    case 118494:                    // 'rename' 'schema-element'
    case 119006:                    // 'rename' 'score'
    case 119518:                    // 'rename' 'select'
    case 120030:                    // 'rename' 'self'
    case 122590:                    // 'rename' 'sliding'
    case 123102:                    // 'rename' 'some'
    case 123614:                    // 'rename' 'stable'
    case 124126:                    // 'rename' 'start'
    case 125662:                    // 'rename' 'strict'
    case 126686:                    // 'rename' 'structured-item'
    case 127198:                    // 'rename' 'switch'
    case 127710:                    // 'rename' 'text'
    case 129758:                    // 'rename' 'to'
    case 130270:                    // 'rename' 'treat'
    case 130782:                    // 'rename' 'true'
    case 131294:                    // 'rename' 'try'
    case 131806:                    // 'rename' 'tumbling'
    case 132318:                    // 'rename' 'type'
    case 132830:                    // 'rename' 'typeswitch'
    case 133342:                    // 'rename' 'union'
    case 134366:                    // 'rename' 'unordered'
    case 134878:                    // 'rename' 'updating'
    case 136414:                    // 'rename' 'validate'
    case 136926:                    // 'rename' 'value'
    case 137438:                    // 'rename' 'variable'
    case 137950:                    // 'rename' 'version'
    case 139486:                    // 'rename' 'where'
    case 139998:                    // 'rename' 'while'
    case 141534:                    // 'rename' 'with'
    case 144094:                    // 'rename' '{'
    case 145118:                    // 'rename' '{|'
      try_JSONRenameExpr();
      break;
    case -11:
      try_JSONReplaceExpr();
      break;
    case -12:
    case 3150:                      // 'append' EQName^Token
    case 4174:                      // 'append' IntegerLiteral
    case 4686:                      // 'append' DecimalLiteral
    case 5198:                      // 'append' DoubleLiteral
    case 5710:                      // 'append' StringLiteral
    case 15950:                     // 'append' '$'
    case 16462:                     // 'append' '$$'
    case 16974:                     // 'append' '%'
    case 18510:                     // 'append' '(#'
    case 21070:                     // 'append' '+'
    case 22094:                     // 'append' '-'
    case 24142:                     // 'append' '/'
    case 24654:                     // 'append' '//'
    case 28238:                     // 'append' '<'
    case 28750:                     // 'append' '<!--'
    case 30798:                     // 'append' '<?'
    case 35406:                     // 'append' '['
    case 36430:                     // 'append' 'after'
    case 37454:                     // 'append' 'allowing'
    case 37966:                     // 'append' 'ancestor'
    case 38478:                     // 'append' 'ancestor-or-self'
    case 38990:                     // 'append' 'and'
    case 40014:                     // 'append' 'append'
    case 40526:                     // 'append' 'array'
    case 41038:                     // 'append' 'as'
    case 41550:                     // 'append' 'ascending'
    case 42062:                     // 'append' 'at'
    case 42574:                     // 'append' 'attribute'
    case 43086:                     // 'append' 'base-uri'
    case 43598:                     // 'append' 'before'
    case 44110:                     // 'append' 'boundary-space'
    case 44622:                     // 'append' 'break'
    case 45646:                     // 'append' 'case'
    case 46158:                     // 'append' 'cast'
    case 46670:                     // 'append' 'castable'
    case 47182:                     // 'append' 'catch'
    case 48206:                     // 'append' 'child'
    case 48718:                     // 'append' 'collation'
    case 49742:                     // 'append' 'comment'
    case 50254:                     // 'append' 'constraint'
    case 50766:                     // 'append' 'construction'
    case 52302:                     // 'append' 'context'
    case 52814:                     // 'append' 'continue'
    case 53326:                     // 'append' 'copy'
    case 53838:                     // 'append' 'copy-namespaces'
    case 54350:                     // 'append' 'count'
    case 54862:                     // 'append' 'decimal-format'
    case 55886:                     // 'append' 'declare'
    case 56398:                     // 'append' 'default'
    case 56910:                     // 'append' 'delete'
    case 57422:                     // 'append' 'descendant'
    case 57934:                     // 'append' 'descendant-or-self'
    case 58446:                     // 'append' 'descending'
    case 61006:                     // 'append' 'div'
    case 61518:                     // 'append' 'document'
    case 62030:                     // 'append' 'document-node'
    case 62542:                     // 'append' 'element'
    case 63054:                     // 'append' 'else'
    case 63566:                     // 'append' 'empty'
    case 64078:                     // 'append' 'empty-sequence'
    case 64590:                     // 'append' 'encoding'
    case 65102:                     // 'append' 'end'
    case 66126:                     // 'append' 'eq'
    case 66638:                     // 'append' 'every'
    case 67662:                     // 'append' 'except'
    case 68174:                     // 'append' 'exit'
    case 68686:                     // 'append' 'external'
    case 69198:                     // 'append' 'false'
    case 69710:                     // 'append' 'first'
    case 70222:                     // 'append' 'following'
    case 70734:                     // 'append' 'following-sibling'
    case 71246:                     // 'append' 'for'
    case 72782:                     // 'append' 'from'
    case 73294:                     // 'append' 'ft-option'
    case 75342:                     // 'append' 'function'
    case 75854:                     // 'append' 'ge'
    case 76878:                     // 'append' 'group'
    case 77902:                     // 'append' 'gt'
    case 78414:                     // 'append' 'idiv'
    case 78926:                     // 'append' 'if'
    case 79438:                     // 'append' 'import'
    case 79950:                     // 'append' 'in'
    case 80462:                     // 'append' 'index'
    case 82510:                     // 'append' 'insert'
    case 83022:                     // 'append' 'instance'
    case 83534:                     // 'append' 'integrity'
    case 84046:                     // 'append' 'intersect'
    case 84558:                     // 'append' 'into'
    case 85070:                     // 'append' 'is'
    case 85582:                     // 'append' 'item'
    case 86094:                     // 'append' 'json'
    case 86606:                     // 'append' 'json-item'
    case 87118:                     // 'append' 'jsoniq'
    case 88654:                     // 'append' 'last'
    case 89166:                     // 'append' 'lax'
    case 89678:                     // 'append' 'le'
    case 90702:                     // 'append' 'let'
    case 91726:                     // 'append' 'loop'
    case 92750:                     // 'append' 'lt'
    case 93774:                     // 'append' 'mod'
    case 94286:                     // 'append' 'modify'
    case 94798:                     // 'append' 'module'
    case 95822:                     // 'append' 'namespace'
    case 96334:                     // 'append' 'namespace-node'
    case 96846:                     // 'append' 'ne'
    case 99406:                     // 'append' 'node'
    case 99918:                     // 'append' 'nodes'
    case 100430:                    // 'append' 'not'
    case 100942:                    // 'append' 'null'
    case 101454:                    // 'append' 'object'
    case 103502:                    // 'append' 'only'
    case 104014:                    // 'append' 'option'
    case 104526:                    // 'append' 'or'
    case 105038:                    // 'append' 'order'
    case 105550:                    // 'append' 'ordered'
    case 106062:                    // 'append' 'ordering'
    case 107598:                    // 'append' 'parent'
    case 110670:                    // 'append' 'preceding'
    case 111182:                    // 'append' 'preceding-sibling'
    case 112718:                    // 'append' 'processing-instruction'
    case 113742:                    // 'append' 'rename'
    case 114254:                    // 'append' 'replace'
    case 114766:                    // 'append' 'return'
    case 115278:                    // 'append' 'returning'
    case 115790:                    // 'append' 'revalidation'
    case 116814:                    // 'append' 'satisfies'
    case 117326:                    // 'append' 'schema'
    case 117838:                    // 'append' 'schema-attribute'
    case 118350:                    // 'append' 'schema-element'
    case 118862:                    // 'append' 'score'
    case 119374:                    // 'append' 'select'
    case 119886:                    // 'append' 'self'
    case 122446:                    // 'append' 'sliding'
    case 122958:                    // 'append' 'some'
    case 123470:                    // 'append' 'stable'
    case 123982:                    // 'append' 'start'
    case 125518:                    // 'append' 'strict'
    case 126542:                    // 'append' 'structured-item'
    case 127054:                    // 'append' 'switch'
    case 127566:                    // 'append' 'text'
    case 129614:                    // 'append' 'to'
    case 130126:                    // 'append' 'treat'
    case 130638:                    // 'append' 'true'
    case 131150:                    // 'append' 'try'
    case 131662:                    // 'append' 'tumbling'
    case 132174:                    // 'append' 'type'
    case 132686:                    // 'append' 'typeswitch'
    case 133198:                    // 'append' 'union'
    case 134222:                    // 'append' 'unordered'
    case 134734:                    // 'append' 'updating'
    case 136270:                    // 'append' 'validate'
    case 136782:                    // 'append' 'value'
    case 137294:                    // 'append' 'variable'
    case 137806:                    // 'append' 'version'
    case 139342:                    // 'append' 'where'
    case 139854:                    // 'append' 'while'
    case 141390:                    // 'append' 'with'
    case 143950:                    // 'append' '{'
    case 144974:                    // 'append' '{|'
      try_JSONAppendExpr();
      break;
    case -13:
      break;
    default:
      try_OrExpr();
    }
  }

  function parse_JSONDeleteExpr()
  {
    eventHandler.startNonterminal("JSONDeleteExpr", e0);
    shift(111);                     // 'delete'
    lookahead1W(260);               // EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral | StringLiteral |
    switch (l1)
    {
    case 168:                       // 'json'
      lookahead2W(261);             // EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral | StringLiteral |
      break;
    default:
      lk = l1;
    }
    if (lk == 18088)                // 'json' '('
    {
      lk = memoized(11, e0);
      if (lk == 0)
      {
        var b0A = b0; var e0A = e0; var l1A = l1;
        var b1A = b1; var e1A = e1; var l2A = l2;
        var b2A = b2; var e2A = e2;
        try
        {
          shiftT(168);              // 'json'
          lk = -1;
        }
        catch (p1A)
        {
          lk = -2;
        }
        b0 = b0A; e0 = e0A; l1 = l1A; if (l1 == 0) {end = e0A;} else {
        b1 = b1A; e1 = e1A; l2 = l2A; if (l2 == 0) {end = e1A;} else {
        b2 = b2A; e2 = e2A; end = e2A; }}
        memoize(11, e0, lk);
      }
    }
    if (lk == -1
     || lk == 3240                  // 'json' EQName^Token
     || lk == 4264                  // 'json' IntegerLiteral
     || lk == 4776                  // 'json' DecimalLiteral
     || lk == 5288                  // 'json' DoubleLiteral
     || lk == 5800                  // 'json' StringLiteral
     || lk == 16040                 // 'json' '$'
     || lk == 16552                 // 'json' '$$'
     || lk == 17064                 // 'json' '%'
     || lk == 28328                 // 'json' '<'
     || lk == 28840                 // 'json' '<!--'
     || lk == 30888                 // 'json' '<?'
     || lk == 35496                 // 'json' '['
     || lk == 36520                 // 'json' 'after'
     || lk == 37544                 // 'json' 'allowing'
     || lk == 38056                 // 'json' 'ancestor'
     || lk == 38568                 // 'json' 'ancestor-or-self'
     || lk == 39080                 // 'json' 'and'
     || lk == 40104                 // 'json' 'append'
     || lk == 40616                 // 'json' 'array'
     || lk == 41128                 // 'json' 'as'
     || lk == 41640                 // 'json' 'ascending'
     || lk == 42152                 // 'json' 'at'
     || lk == 42664                 // 'json' 'attribute'
     || lk == 43176                 // 'json' 'base-uri'
     || lk == 43688                 // 'json' 'before'
     || lk == 44200                 // 'json' 'boundary-space'
     || lk == 44712                 // 'json' 'break'
     || lk == 45736                 // 'json' 'case'
     || lk == 46248                 // 'json' 'cast'
     || lk == 46760                 // 'json' 'castable'
     || lk == 47272                 // 'json' 'catch'
     || lk == 48296                 // 'json' 'child'
     || lk == 48808                 // 'json' 'collation'
     || lk == 49832                 // 'json' 'comment'
     || lk == 50344                 // 'json' 'constraint'
     || lk == 50856                 // 'json' 'construction'
     || lk == 52392                 // 'json' 'context'
     || lk == 52904                 // 'json' 'continue'
     || lk == 53416                 // 'json' 'copy'
     || lk == 53928                 // 'json' 'copy-namespaces'
     || lk == 54440                 // 'json' 'count'
     || lk == 54952                 // 'json' 'decimal-format'
     || lk == 55976                 // 'json' 'declare'
     || lk == 56488                 // 'json' 'default'
     || lk == 57000                 // 'json' 'delete'
     || lk == 57512                 // 'json' 'descendant'
     || lk == 58024                 // 'json' 'descendant-or-self'
     || lk == 58536                 // 'json' 'descending'
     || lk == 61096                 // 'json' 'div'
     || lk == 61608                 // 'json' 'document'
     || lk == 62120                 // 'json' 'document-node'
     || lk == 62632                 // 'json' 'element'
     || lk == 63144                 // 'json' 'else'
     || lk == 63656                 // 'json' 'empty'
     || lk == 64168                 // 'json' 'empty-sequence'
     || lk == 64680                 // 'json' 'encoding'
     || lk == 65192                 // 'json' 'end'
     || lk == 66216                 // 'json' 'eq'
     || lk == 66728                 // 'json' 'every'
     || lk == 67752                 // 'json' 'except'
     || lk == 68264                 // 'json' 'exit'
     || lk == 68776                 // 'json' 'external'
     || lk == 69288                 // 'json' 'false'
     || lk == 69800                 // 'json' 'first'
     || lk == 70312                 // 'json' 'following'
     || lk == 70824                 // 'json' 'following-sibling'
     || lk == 71336                 // 'json' 'for'
     || lk == 72872                 // 'json' 'from'
     || lk == 73384                 // 'json' 'ft-option'
     || lk == 75432                 // 'json' 'function'
     || lk == 75944                 // 'json' 'ge'
     || lk == 76968                 // 'json' 'group'
     || lk == 77992                 // 'json' 'gt'
     || lk == 78504                 // 'json' 'idiv'
     || lk == 79016                 // 'json' 'if'
     || lk == 79528                 // 'json' 'import'
     || lk == 80040                 // 'json' 'in'
     || lk == 80552                 // 'json' 'index'
     || lk == 82600                 // 'json' 'insert'
     || lk == 83112                 // 'json' 'instance'
     || lk == 83624                 // 'json' 'integrity'
     || lk == 84136                 // 'json' 'intersect'
     || lk == 84648                 // 'json' 'into'
     || lk == 85160                 // 'json' 'is'
     || lk == 85672                 // 'json' 'item'
     || lk == 86184                 // 'json' 'json'
     || lk == 86696                 // 'json' 'json-item'
     || lk == 87208                 // 'json' 'jsoniq'
     || lk == 88744                 // 'json' 'last'
     || lk == 89256                 // 'json' 'lax'
     || lk == 89768                 // 'json' 'le'
     || lk == 90792                 // 'json' 'let'
     || lk == 91816                 // 'json' 'loop'
     || lk == 92840                 // 'json' 'lt'
     || lk == 93864                 // 'json' 'mod'
     || lk == 94376                 // 'json' 'modify'
     || lk == 94888                 // 'json' 'module'
     || lk == 95912                 // 'json' 'namespace'
     || lk == 96424                 // 'json' 'namespace-node'
     || lk == 96936                 // 'json' 'ne'
     || lk == 99496                 // 'json' 'node'
     || lk == 100008                // 'json' 'nodes'
     || lk == 101032                // 'json' 'null'
     || lk == 101544                // 'json' 'object'
     || lk == 103592                // 'json' 'only'
     || lk == 104104                // 'json' 'option'
     || lk == 104616                // 'json' 'or'
     || lk == 105128                // 'json' 'order'
     || lk == 105640                // 'json' 'ordered'
     || lk == 106152                // 'json' 'ordering'
     || lk == 107688                // 'json' 'parent'
     || lk == 110760                // 'json' 'preceding'
     || lk == 111272                // 'json' 'preceding-sibling'
     || lk == 112808                // 'json' 'processing-instruction'
     || lk == 113832                // 'json' 'rename'
     || lk == 114344                // 'json' 'replace'
     || lk == 114856                // 'json' 'return'
     || lk == 115368                // 'json' 'returning'
     || lk == 115880                // 'json' 'revalidation'
     || lk == 116904                // 'json' 'satisfies'
     || lk == 117416                // 'json' 'schema'
     || lk == 117928                // 'json' 'schema-attribute'
     || lk == 118440                // 'json' 'schema-element'
     || lk == 118952                // 'json' 'score'
     || lk == 119464                // 'json' 'select'
     || lk == 119976                // 'json' 'self'
     || lk == 122536                // 'json' 'sliding'
     || lk == 123048                // 'json' 'some'
     || lk == 123560                // 'json' 'stable'
     || lk == 124072                // 'json' 'start'
     || lk == 125608                // 'json' 'strict'
     || lk == 126632                // 'json' 'structured-item'
     || lk == 127144                // 'json' 'switch'
     || lk == 127656                // 'json' 'text'
     || lk == 129704                // 'json' 'to'
     || lk == 130216                // 'json' 'treat'
     || lk == 130728                // 'json' 'true'
     || lk == 131240                // 'json' 'try'
     || lk == 131752                // 'json' 'tumbling'
     || lk == 132264                // 'json' 'type'
     || lk == 132776                // 'json' 'typeswitch'
     || lk == 133288                // 'json' 'union'
     || lk == 134312                // 'json' 'unordered'
     || lk == 134824                // 'json' 'updating'
     || lk == 136360                // 'json' 'validate'
     || lk == 136872                // 'json' 'value'
     || lk == 137384                // 'json' 'variable'
     || lk == 137896                // 'json' 'version'
     || lk == 139432                // 'json' 'where'
     || lk == 139944                // 'json' 'while'
     || lk == 141480                // 'json' 'with'
     || lk == 144040                // 'json' '{'
     || lk == 145064)               // 'json' '{|'
    {
      shift(168);                   // 'json'
    }
    lookahead1W(260);               // EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral | StringLiteral |
    whitespace();
    parse_PostfixExpr();
    eventHandler.endNonterminal("JSONDeleteExpr", e0);
  }

  function try_JSONDeleteExpr()
  {
    shiftT(111);                    // 'delete'
    lookahead1W(260);               // EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral | StringLiteral |
    switch (l1)
    {
    case 168:                       // 'json'
      lookahead2W(261);             // EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral | StringLiteral |
      break;
    default:
      lk = l1;
    }
    if (lk == 18088)                // 'json' '('
    {
      lk = memoized(11, e0);
      if (lk == 0)
      {
        var b0A = b0; var e0A = e0; var l1A = l1;
        var b1A = b1; var e1A = e1; var l2A = l2;
        var b2A = b2; var e2A = e2;
        try
        {
          shiftT(168);              // 'json'
          memoize(11, e0A, -1);
        }
        catch (p1A)
        {
          b0 = b0A; e0 = e0A; l1 = l1A; if (l1 == 0) {end = e0A;} else {
          b1 = b1A; e1 = e1A; l2 = l2A; if (l2 == 0) {end = e1A;} else {
          b2 = b2A; e2 = e2A; end = e2A; }}
          memoize(11, e0A, -2);
        }
        lk = -2;
      }
    }
    if (lk == -1
     || lk == 3240                  // 'json' EQName^Token
     || lk == 4264                  // 'json' IntegerLiteral
     || lk == 4776                  // 'json' DecimalLiteral
     || lk == 5288                  // 'json' DoubleLiteral
     || lk == 5800                  // 'json' StringLiteral
     || lk == 16040                 // 'json' '$'
     || lk == 16552                 // 'json' '$$'
     || lk == 17064                 // 'json' '%'
     || lk == 28328                 // 'json' '<'
     || lk == 28840                 // 'json' '<!--'
     || lk == 30888                 // 'json' '<?'
     || lk == 35496                 // 'json' '['
     || lk == 36520                 // 'json' 'after'
     || lk == 37544                 // 'json' 'allowing'
     || lk == 38056                 // 'json' 'ancestor'
     || lk == 38568                 // 'json' 'ancestor-or-self'
     || lk == 39080                 // 'json' 'and'
     || lk == 40104                 // 'json' 'append'
     || lk == 40616                 // 'json' 'array'
     || lk == 41128                 // 'json' 'as'
     || lk == 41640                 // 'json' 'ascending'
     || lk == 42152                 // 'json' 'at'
     || lk == 42664                 // 'json' 'attribute'
     || lk == 43176                 // 'json' 'base-uri'
     || lk == 43688                 // 'json' 'before'
     || lk == 44200                 // 'json' 'boundary-space'
     || lk == 44712                 // 'json' 'break'
     || lk == 45736                 // 'json' 'case'
     || lk == 46248                 // 'json' 'cast'
     || lk == 46760                 // 'json' 'castable'
     || lk == 47272                 // 'json' 'catch'
     || lk == 48296                 // 'json' 'child'
     || lk == 48808                 // 'json' 'collation'
     || lk == 49832                 // 'json' 'comment'
     || lk == 50344                 // 'json' 'constraint'
     || lk == 50856                 // 'json' 'construction'
     || lk == 52392                 // 'json' 'context'
     || lk == 52904                 // 'json' 'continue'
     || lk == 53416                 // 'json' 'copy'
     || lk == 53928                 // 'json' 'copy-namespaces'
     || lk == 54440                 // 'json' 'count'
     || lk == 54952                 // 'json' 'decimal-format'
     || lk == 55976                 // 'json' 'declare'
     || lk == 56488                 // 'json' 'default'
     || lk == 57000                 // 'json' 'delete'
     || lk == 57512                 // 'json' 'descendant'
     || lk == 58024                 // 'json' 'descendant-or-self'
     || lk == 58536                 // 'json' 'descending'
     || lk == 61096                 // 'json' 'div'
     || lk == 61608                 // 'json' 'document'
     || lk == 62120                 // 'json' 'document-node'
     || lk == 62632                 // 'json' 'element'
     || lk == 63144                 // 'json' 'else'
     || lk == 63656                 // 'json' 'empty'
     || lk == 64168                 // 'json' 'empty-sequence'
     || lk == 64680                 // 'json' 'encoding'
     || lk == 65192                 // 'json' 'end'
     || lk == 66216                 // 'json' 'eq'
     || lk == 66728                 // 'json' 'every'
     || lk == 67752                 // 'json' 'except'
     || lk == 68264                 // 'json' 'exit'
     || lk == 68776                 // 'json' 'external'
     || lk == 69288                 // 'json' 'false'
     || lk == 69800                 // 'json' 'first'
     || lk == 70312                 // 'json' 'following'
     || lk == 70824                 // 'json' 'following-sibling'
     || lk == 71336                 // 'json' 'for'
     || lk == 72872                 // 'json' 'from'
     || lk == 73384                 // 'json' 'ft-option'
     || lk == 75432                 // 'json' 'function'
     || lk == 75944                 // 'json' 'ge'
     || lk == 76968                 // 'json' 'group'
     || lk == 77992                 // 'json' 'gt'
     || lk == 78504                 // 'json' 'idiv'
     || lk == 79016                 // 'json' 'if'
     || lk == 79528                 // 'json' 'import'
     || lk == 80040                 // 'json' 'in'
     || lk == 80552                 // 'json' 'index'
     || lk == 82600                 // 'json' 'insert'
     || lk == 83112                 // 'json' 'instance'
     || lk == 83624                 // 'json' 'integrity'
     || lk == 84136                 // 'json' 'intersect'
     || lk == 84648                 // 'json' 'into'
     || lk == 85160                 // 'json' 'is'
     || lk == 85672                 // 'json' 'item'
     || lk == 86184                 // 'json' 'json'
     || lk == 86696                 // 'json' 'json-item'
     || lk == 87208                 // 'json' 'jsoniq'
     || lk == 88744                 // 'json' 'last'
     || lk == 89256                 // 'json' 'lax'
     || lk == 89768                 // 'json' 'le'
     || lk == 90792                 // 'json' 'let'
     || lk == 91816                 // 'json' 'loop'
     || lk == 92840                 // 'json' 'lt'
     || lk == 93864                 // 'json' 'mod'
     || lk == 94376                 // 'json' 'modify'
     || lk == 94888                 // 'json' 'module'
     || lk == 95912                 // 'json' 'namespace'
     || lk == 96424                 // 'json' 'namespace-node'
     || lk == 96936                 // 'json' 'ne'
     || lk == 99496                 // 'json' 'node'
     || lk == 100008                // 'json' 'nodes'
     || lk == 101032                // 'json' 'null'
     || lk == 101544                // 'json' 'object'
     || lk == 103592                // 'json' 'only'
     || lk == 104104                // 'json' 'option'
     || lk == 104616                // 'json' 'or'
     || lk == 105128                // 'json' 'order'
     || lk == 105640                // 'json' 'ordered'
     || lk == 106152                // 'json' 'ordering'
     || lk == 107688                // 'json' 'parent'
     || lk == 110760                // 'json' 'preceding'
     || lk == 111272                // 'json' 'preceding-sibling'
     || lk == 112808                // 'json' 'processing-instruction'
     || lk == 113832                // 'json' 'rename'
     || lk == 114344                // 'json' 'replace'
     || lk == 114856                // 'json' 'return'
     || lk == 115368                // 'json' 'returning'
     || lk == 115880                // 'json' 'revalidation'
     || lk == 116904                // 'json' 'satisfies'
     || lk == 117416                // 'json' 'schema'
     || lk == 117928                // 'json' 'schema-attribute'
     || lk == 118440                // 'json' 'schema-element'
     || lk == 118952                // 'json' 'score'
     || lk == 119464                // 'json' 'select'
     || lk == 119976                // 'json' 'self'
     || lk == 122536                // 'json' 'sliding'
     || lk == 123048                // 'json' 'some'
     || lk == 123560                // 'json' 'stable'
     || lk == 124072                // 'json' 'start'
     || lk == 125608                // 'json' 'strict'
     || lk == 126632                // 'json' 'structured-item'
     || lk == 127144                // 'json' 'switch'
     || lk == 127656                // 'json' 'text'
     || lk == 129704                // 'json' 'to'
     || lk == 130216                // 'json' 'treat'
     || lk == 130728                // 'json' 'true'
     || lk == 131240                // 'json' 'try'
     || lk == 131752                // 'json' 'tumbling'
     || lk == 132264                // 'json' 'type'
     || lk == 132776                // 'json' 'typeswitch'
     || lk == 133288                // 'json' 'union'
     || lk == 134312                // 'json' 'unordered'
     || lk == 134824                // 'json' 'updating'
     || lk == 136360                // 'json' 'validate'
     || lk == 136872                // 'json' 'value'
     || lk == 137384                // 'json' 'variable'
     || lk == 137896                // 'json' 'version'
     || lk == 139432                // 'json' 'where'
     || lk == 139944                // 'json' 'while'
     || lk == 141480                // 'json' 'with'
     || lk == 144040                // 'json' '{'
     || lk == 145064)               // 'json' '{|'
    {
      shiftT(168);                  // 'json'
    }
    lookahead1W(260);               // EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral | StringLiteral |
    try_PostfixExpr();
  }

  function parse_JSONInsertExpr()
  {
    eventHandler.startNonterminal("JSONInsertExpr", e0);
    switch (l1)
    {
    case 161:                       // 'insert'
      lookahead2W(268);             // EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral | StringLiteral |
      break;
    default:
      lk = l1;
    }
    if (lk != 9889)                 // 'insert' NCName^Token
    {
      lk = memoized(12, e0);
      if (lk == 0)
      {
        var b0A = b0; var e0A = e0; var l1A = l1;
        var b1A = b1; var e1A = e1; var l2A = l2;
        var b2A = b2; var e2A = e2;
        try
        {
          shiftT(161);              // 'insert'
          lookahead1W(267);         // EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral | StringLiteral |
          switch (l1)
          {
          case 168:                 // 'json'
            lookahead2W(269);       // EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral | StringLiteral |
            break;
          default:
            lk = l1;
          }
          if (lk == 18088)          // 'json' '('
          {
            lk = memoized(13, e0);
            if (lk == 0)
            {
              var b0B = b0; var e0B = e0; var l1B = l1;
              var b1B = b1; var e1B = e1; var l2B = l2;
              var b2B = b2; var e2B = e2;
              try
              {
                shiftT(168);        // 'json'
                memoize(13, e0B, -1);
              }
              catch (p1B)
              {
                b0 = b0B; e0 = e0B; l1 = l1B; if (l1 == 0) {end = e0B;} else {
                b1 = b1B; e1 = e1B; l2 = l2B; if (l2 == 0) {end = e1B;} else {
                b2 = b2B; e2 = e2B; end = e2B; }}
                memoize(13, e0B, -2);
              }
              lk = -2;
            }
          }
          if (lk == -1
           || lk == 3240            // 'json' EQName^Token
           || lk == 4264            // 'json' IntegerLiteral
           || lk == 4776            // 'json' DecimalLiteral
           || lk == 5288            // 'json' DoubleLiteral
           || lk == 5800            // 'json' StringLiteral
           || lk == 16040           // 'json' '$'
           || lk == 16552           // 'json' '$$'
           || lk == 17064           // 'json' '%'
           || lk == 18600           // 'json' '(#'
           || lk == 21160           // 'json' '+'
           || lk == 22184           // 'json' '-'
           || lk == 24232           // 'json' '/'
           || lk == 24744           // 'json' '//'
           || lk == 28328           // 'json' '<'
           || lk == 28840           // 'json' '<!--'
           || lk == 30888           // 'json' '<?'
           || lk == 35496           // 'json' '['
           || lk == 36520           // 'json' 'after'
           || lk == 37544           // 'json' 'allowing'
           || lk == 38056           // 'json' 'ancestor'
           || lk == 38568           // 'json' 'ancestor-or-self'
           || lk == 39080           // 'json' 'and'
           || lk == 40104           // 'json' 'append'
           || lk == 40616           // 'json' 'array'
           || lk == 41128           // 'json' 'as'
           || lk == 41640           // 'json' 'ascending'
           || lk == 42152           // 'json' 'at'
           || lk == 42664           // 'json' 'attribute'
           || lk == 43176           // 'json' 'base-uri'
           || lk == 43688           // 'json' 'before'
           || lk == 44200           // 'json' 'boundary-space'
           || lk == 44712           // 'json' 'break'
           || lk == 45736           // 'json' 'case'
           || lk == 46248           // 'json' 'cast'
           || lk == 46760           // 'json' 'castable'
           || lk == 47272           // 'json' 'catch'
           || lk == 48296           // 'json' 'child'
           || lk == 48808           // 'json' 'collation'
           || lk == 49832           // 'json' 'comment'
           || lk == 50344           // 'json' 'constraint'
           || lk == 50856           // 'json' 'construction'
           || lk == 52392           // 'json' 'context'
           || lk == 52904           // 'json' 'continue'
           || lk == 53416           // 'json' 'copy'
           || lk == 53928           // 'json' 'copy-namespaces'
           || lk == 54440           // 'json' 'count'
           || lk == 54952           // 'json' 'decimal-format'
           || lk == 55976           // 'json' 'declare'
           || lk == 56488           // 'json' 'default'
           || lk == 57000           // 'json' 'delete'
           || lk == 57512           // 'json' 'descendant'
           || lk == 58024           // 'json' 'descendant-or-self'
           || lk == 58536           // 'json' 'descending'
           || lk == 61096           // 'json' 'div'
           || lk == 61608           // 'json' 'document'
           || lk == 62120           // 'json' 'document-node'
           || lk == 62632           // 'json' 'element'
           || lk == 63144           // 'json' 'else'
           || lk == 63656           // 'json' 'empty'
           || lk == 64168           // 'json' 'empty-sequence'
           || lk == 64680           // 'json' 'encoding'
           || lk == 65192           // 'json' 'end'
           || lk == 66216           // 'json' 'eq'
           || lk == 66728           // 'json' 'every'
           || lk == 67752           // 'json' 'except'
           || lk == 68264           // 'json' 'exit'
           || lk == 68776           // 'json' 'external'
           || lk == 69288           // 'json' 'false'
           || lk == 69800           // 'json' 'first'
           || lk == 70312           // 'json' 'following'
           || lk == 70824           // 'json' 'following-sibling'
           || lk == 71336           // 'json' 'for'
           || lk == 72872           // 'json' 'from'
           || lk == 73384           // 'json' 'ft-option'
           || lk == 75432           // 'json' 'function'
           || lk == 75944           // 'json' 'ge'
           || lk == 76968           // 'json' 'group'
           || lk == 77992           // 'json' 'gt'
           || lk == 78504           // 'json' 'idiv'
           || lk == 79016           // 'json' 'if'
           || lk == 79528           // 'json' 'import'
           || lk == 80040           // 'json' 'in'
           || lk == 80552           // 'json' 'index'
           || lk == 82600           // 'json' 'insert'
           || lk == 83112           // 'json' 'instance'
           || lk == 83624           // 'json' 'integrity'
           || lk == 84136           // 'json' 'intersect'
           || lk == 84648           // 'json' 'into'
           || lk == 85160           // 'json' 'is'
           || lk == 85672           // 'json' 'item'
           || lk == 86184           // 'json' 'json'
           || lk == 86696           // 'json' 'json-item'
           || lk == 87208           // 'json' 'jsoniq'
           || lk == 88744           // 'json' 'last'
           || lk == 89256           // 'json' 'lax'
           || lk == 89768           // 'json' 'le'
           || lk == 90792           // 'json' 'let'
           || lk == 91816           // 'json' 'loop'
           || lk == 92840           // 'json' 'lt'
           || lk == 93864           // 'json' 'mod'
           || lk == 94376           // 'json' 'modify'
           || lk == 94888           // 'json' 'module'
           || lk == 95912           // 'json' 'namespace'
           || lk == 96424           // 'json' 'namespace-node'
           || lk == 96936           // 'json' 'ne'
           || lk == 99496           // 'json' 'node'
           || lk == 100008          // 'json' 'nodes'
           || lk == 100520          // 'json' 'not'
           || lk == 101032          // 'json' 'null'
           || lk == 101544          // 'json' 'object'
           || lk == 103592          // 'json' 'only'
           || lk == 104104          // 'json' 'option'
           || lk == 104616          // 'json' 'or'
           || lk == 105128          // 'json' 'order'
           || lk == 105640          // 'json' 'ordered'
           || lk == 106152          // 'json' 'ordering'
           || lk == 107688          // 'json' 'parent'
           || lk == 110760          // 'json' 'preceding'
           || lk == 111272          // 'json' 'preceding-sibling'
           || lk == 112808          // 'json' 'processing-instruction'
           || lk == 113832          // 'json' 'rename'
           || lk == 114344          // 'json' 'replace'
           || lk == 114856          // 'json' 'return'
           || lk == 115368          // 'json' 'returning'
           || lk == 115880          // 'json' 'revalidation'
           || lk == 116904          // 'json' 'satisfies'
           || lk == 117416          // 'json' 'schema'
           || lk == 117928          // 'json' 'schema-attribute'
           || lk == 118440          // 'json' 'schema-element'
           || lk == 118952          // 'json' 'score'
           || lk == 119464          // 'json' 'select'
           || lk == 119976          // 'json' 'self'
           || lk == 122536          // 'json' 'sliding'
           || lk == 123048          // 'json' 'some'
           || lk == 123560          // 'json' 'stable'
           || lk == 124072          // 'json' 'start'
           || lk == 125608          // 'json' 'strict'
           || lk == 126632          // 'json' 'structured-item'
           || lk == 127144          // 'json' 'switch'
           || lk == 127656          // 'json' 'text'
           || lk == 129704          // 'json' 'to'
           || lk == 130216          // 'json' 'treat'
           || lk == 130728          // 'json' 'true'
           || lk == 131240          // 'json' 'try'
           || lk == 131752          // 'json' 'tumbling'
           || lk == 132264          // 'json' 'type'
           || lk == 132776          // 'json' 'typeswitch'
           || lk == 133288          // 'json' 'union'
           || lk == 134312          // 'json' 'unordered'
           || lk == 134824          // 'json' 'updating'
           || lk == 136360          // 'json' 'validate'
           || lk == 136872          // 'json' 'value'
           || lk == 137384          // 'json' 'variable'
           || lk == 137896          // 'json' 'version'
           || lk == 139432          // 'json' 'where'
           || lk == 139944          // 'json' 'while'
           || lk == 141480          // 'json' 'with'
           || lk == 144040          // 'json' '{'
           || lk == 145064)         // 'json' '{|'
          {
            shiftT(168);            // 'json'
          }
          lookahead1W(267);         // EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral | StringLiteral |
          try_ExprSingle();
          shiftT(165);              // 'into'
          lookahead1W(267);         // EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral | StringLiteral |
          try_ExprSingle();
          switch (l1)
          {
          case 82:                  // 'at'
            lookahead2W(72);        // S^WS | '(:' | 'position'
            break;
          default:
            lk = l1;
          }
          if (lk == 110162)         // 'at' 'position'
          {
            lk = memoized(14, e0);
            if (lk == 0)
            {
              var b0B = b0; var e0B = e0; var l1B = l1;
              var b1B = b1; var e1B = e1; var l2B = l2;
              var b2B = b2; var e2B = e2;
              try
              {
                shiftT(82);         // 'at'
                lookahead1W(72);    // S^WS | '(:' | 'position'
                shiftT(215);        // 'position'
                lookahead1W(267);   // EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral | StringLiteral |
                try_ExprSingle();
                memoize(14, e0B, -1);
              }
              catch (p1B)
              {
                b0 = b0B; e0 = e0B; l1 = l1B; if (l1 == 0) {end = e0B;} else {
                b1 = b1B; e1 = e1B; l2 = l2B; if (l2 == 0) {end = e1B;} else {
                b2 = b2B; e2 = e2B; end = e2B; }}
                memoize(14, e0B, -2);
              }
              lk = -2;
            }
          }
          if (lk == -1)
          {
            shiftT(82);             // 'at'
            lookahead1W(72);        // S^WS | '(:' | 'position'
            shiftT(215);            // 'position'
            lookahead1W(267);       // EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral | StringLiteral |
            try_ExprSingle();
          }
          lk = -1;
        }
        catch (p1A)
        {
          lk = -2;
        }
        b0 = b0A; e0 = e0A; l1 = l1A; if (l1 == 0) {end = e0A;} else {
        b1 = b1A; e1 = e1A; l2 = l2A; if (l2 == 0) {end = e1A;} else {
        b2 = b2A; e2 = e2A; end = e2A; }}
        memoize(12, e0, lk);
      }
    }
    switch (lk)
    {
    case -1:
      shift(161);                   // 'insert'
      lookahead1W(267);             // EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral | StringLiteral |
      switch (l1)
      {
      case 168:                     // 'json'
        lookahead2W(269);           // EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral | StringLiteral |
        break;
      default:
        lk = l1;
      }
      if (lk == 18088)              // 'json' '('
      {
        lk = memoized(13, e0);
        if (lk == 0)
        {
          var b0B = b0; var e0B = e0; var l1B = l1;
          var b1B = b1; var e1B = e1; var l2B = l2;
          var b2B = b2; var e2B = e2;
          try
          {
            shiftT(168);            // 'json'
            lk = -1;
          }
          catch (p1B)
          {
            lk = -2;
          }
          b0 = b0B; e0 = e0B; l1 = l1B; if (l1 == 0) {end = e0B;} else {
          b1 = b1B; e1 = e1B; l2 = l2B; if (l2 == 0) {end = e1B;} else {
          b2 = b2B; e2 = e2B; end = e2B; }}
          memoize(13, e0, lk);
        }
      }
      if (lk == -1
       || lk == 3240                // 'json' EQName^Token
       || lk == 4264                // 'json' IntegerLiteral
       || lk == 4776                // 'json' DecimalLiteral
       || lk == 5288                // 'json' DoubleLiteral
       || lk == 5800                // 'json' StringLiteral
       || lk == 16040               // 'json' '$'
       || lk == 16552               // 'json' '$$'
       || lk == 17064               // 'json' '%'
       || lk == 18600               // 'json' '(#'
       || lk == 21160               // 'json' '+'
       || lk == 22184               // 'json' '-'
       || lk == 24232               // 'json' '/'
       || lk == 24744               // 'json' '//'
       || lk == 28328               // 'json' '<'
       || lk == 28840               // 'json' '<!--'
       || lk == 30888               // 'json' '<?'
       || lk == 35496               // 'json' '['
       || lk == 36520               // 'json' 'after'
       || lk == 37544               // 'json' 'allowing'
       || lk == 38056               // 'json' 'ancestor'
       || lk == 38568               // 'json' 'ancestor-or-self'
       || lk == 39080               // 'json' 'and'
       || lk == 40104               // 'json' 'append'
       || lk == 40616               // 'json' 'array'
       || lk == 41128               // 'json' 'as'
       || lk == 41640               // 'json' 'ascending'
       || lk == 42152               // 'json' 'at'
       || lk == 42664               // 'json' 'attribute'
       || lk == 43176               // 'json' 'base-uri'
       || lk == 43688               // 'json' 'before'
       || lk == 44200               // 'json' 'boundary-space'
       || lk == 44712               // 'json' 'break'
       || lk == 45736               // 'json' 'case'
       || lk == 46248               // 'json' 'cast'
       || lk == 46760               // 'json' 'castable'
       || lk == 47272               // 'json' 'catch'
       || lk == 48296               // 'json' 'child'
       || lk == 48808               // 'json' 'collation'
       || lk == 49832               // 'json' 'comment'
       || lk == 50344               // 'json' 'constraint'
       || lk == 50856               // 'json' 'construction'
       || lk == 52392               // 'json' 'context'
       || lk == 52904               // 'json' 'continue'
       || lk == 53416               // 'json' 'copy'
       || lk == 53928               // 'json' 'copy-namespaces'
       || lk == 54440               // 'json' 'count'
       || lk == 54952               // 'json' 'decimal-format'
       || lk == 55976               // 'json' 'declare'
       || lk == 56488               // 'json' 'default'
       || lk == 57000               // 'json' 'delete'
       || lk == 57512               // 'json' 'descendant'
       || lk == 58024               // 'json' 'descendant-or-self'
       || lk == 58536               // 'json' 'descending'
       || lk == 61096               // 'json' 'div'
       || lk == 61608               // 'json' 'document'
       || lk == 62120               // 'json' 'document-node'
       || lk == 62632               // 'json' 'element'
       || lk == 63144               // 'json' 'else'
       || lk == 63656               // 'json' 'empty'
       || lk == 64168               // 'json' 'empty-sequence'
       || lk == 64680               // 'json' 'encoding'
       || lk == 65192               // 'json' 'end'
       || lk == 66216               // 'json' 'eq'
       || lk == 66728               // 'json' 'every'
       || lk == 67752               // 'json' 'except'
       || lk == 68264               // 'json' 'exit'
       || lk == 68776               // 'json' 'external'
       || lk == 69288               // 'json' 'false'
       || lk == 69800               // 'json' 'first'
       || lk == 70312               // 'json' 'following'
       || lk == 70824               // 'json' 'following-sibling'
       || lk == 71336               // 'json' 'for'
       || lk == 72872               // 'json' 'from'
       || lk == 73384               // 'json' 'ft-option'
       || lk == 75432               // 'json' 'function'
       || lk == 75944               // 'json' 'ge'
       || lk == 76968               // 'json' 'group'
       || lk == 77992               // 'json' 'gt'
       || lk == 78504               // 'json' 'idiv'
       || lk == 79016               // 'json' 'if'
       || lk == 79528               // 'json' 'import'
       || lk == 80040               // 'json' 'in'
       || lk == 80552               // 'json' 'index'
       || lk == 82600               // 'json' 'insert'
       || lk == 83112               // 'json' 'instance'
       || lk == 83624               // 'json' 'integrity'
       || lk == 84136               // 'json' 'intersect'
       || lk == 84648               // 'json' 'into'
       || lk == 85160               // 'json' 'is'
       || lk == 85672               // 'json' 'item'
       || lk == 86184               // 'json' 'json'
       || lk == 86696               // 'json' 'json-item'
       || lk == 87208               // 'json' 'jsoniq'
       || lk == 88744               // 'json' 'last'
       || lk == 89256               // 'json' 'lax'
       || lk == 89768               // 'json' 'le'
       || lk == 90792               // 'json' 'let'
       || lk == 91816               // 'json' 'loop'
       || lk == 92840               // 'json' 'lt'
       || lk == 93864               // 'json' 'mod'
       || lk == 94376               // 'json' 'modify'
       || lk == 94888               // 'json' 'module'
       || lk == 95912               // 'json' 'namespace'
       || lk == 96424               // 'json' 'namespace-node'
       || lk == 96936               // 'json' 'ne'
       || lk == 99496               // 'json' 'node'
       || lk == 100008              // 'json' 'nodes'
       || lk == 100520              // 'json' 'not'
       || lk == 101032              // 'json' 'null'
       || lk == 101544              // 'json' 'object'
       || lk == 103592              // 'json' 'only'
       || lk == 104104              // 'json' 'option'
       || lk == 104616              // 'json' 'or'
       || lk == 105128              // 'json' 'order'
       || lk == 105640              // 'json' 'ordered'
       || lk == 106152              // 'json' 'ordering'
       || lk == 107688              // 'json' 'parent'
       || lk == 110760              // 'json' 'preceding'
       || lk == 111272              // 'json' 'preceding-sibling'
       || lk == 112808              // 'json' 'processing-instruction'
       || lk == 113832              // 'json' 'rename'
       || lk == 114344              // 'json' 'replace'
       || lk == 114856              // 'json' 'return'
       || lk == 115368              // 'json' 'returning'
       || lk == 115880              // 'json' 'revalidation'
       || lk == 116904              // 'json' 'satisfies'
       || lk == 117416              // 'json' 'schema'
       || lk == 117928              // 'json' 'schema-attribute'
       || lk == 118440              // 'json' 'schema-element'
       || lk == 118952              // 'json' 'score'
       || lk == 119464              // 'json' 'select'
       || lk == 119976              // 'json' 'self'
       || lk == 122536              // 'json' 'sliding'
       || lk == 123048              // 'json' 'some'
       || lk == 123560              // 'json' 'stable'
       || lk == 124072              // 'json' 'start'
       || lk == 125608              // 'json' 'strict'
       || lk == 126632              // 'json' 'structured-item'
       || lk == 127144              // 'json' 'switch'
       || lk == 127656              // 'json' 'text'
       || lk == 129704              // 'json' 'to'
       || lk == 130216              // 'json' 'treat'
       || lk == 130728              // 'json' 'true'
       || lk == 131240              // 'json' 'try'
       || lk == 131752              // 'json' 'tumbling'
       || lk == 132264              // 'json' 'type'
       || lk == 132776              // 'json' 'typeswitch'
       || lk == 133288              // 'json' 'union'
       || lk == 134312              // 'json' 'unordered'
       || lk == 134824              // 'json' 'updating'
       || lk == 136360              // 'json' 'validate'
       || lk == 136872              // 'json' 'value'
       || lk == 137384              // 'json' 'variable'
       || lk == 137896              // 'json' 'version'
       || lk == 139432              // 'json' 'where'
       || lk == 139944              // 'json' 'while'
       || lk == 141480              // 'json' 'with'
       || lk == 144040              // 'json' '{'
       || lk == 145064)             // 'json' '{|'
      {
        shift(168);                 // 'json'
      }
      lookahead1W(267);             // EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral | StringLiteral |
      whitespace();
      parse_ExprSingle();
      shift(165);                   // 'into'
      lookahead1W(267);             // EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral | StringLiteral |
      whitespace();
      parse_ExprSingle();
      switch (l1)
      {
      case 82:                      // 'at'
        lookahead2W(72);            // S^WS | '(:' | 'position'
        break;
      default:
        lk = l1;
      }
      if (lk == 110162)             // 'at' 'position'
      {
        lk = memoized(14, e0);
        if (lk == 0)
        {
          var b0B = b0; var e0B = e0; var l1B = l1;
          var b1B = b1; var e1B = e1; var l2B = l2;
          var b2B = b2; var e2B = e2;
          try
          {
            shiftT(82);             // 'at'
            lookahead1W(72);        // S^WS | '(:' | 'position'
            shiftT(215);            // 'position'
            lookahead1W(267);       // EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral | StringLiteral |
            try_ExprSingle();
            lk = -1;
          }
          catch (p1B)
          {
            lk = -2;
          }
          b0 = b0B; e0 = e0B; l1 = l1B; if (l1 == 0) {end = e0B;} else {
          b1 = b1B; e1 = e1B; l2 = l2B; if (l2 == 0) {end = e1B;} else {
          b2 = b2B; e2 = e2B; end = e2B; }}
          memoize(14, e0, lk);
        }
      }
      if (lk == -1)
      {
        shift(82);                  // 'at'
        lookahead1W(72);            // S^WS | '(:' | 'position'
        shift(215);                 // 'position'
        lookahead1W(267);           // EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral | StringLiteral |
        whitespace();
        parse_ExprSingle();
      }
      break;
    default:
      shift(161);                   // 'insert'
      lookahead1W(268);             // EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral | StringLiteral |
      switch (l1)
      {
      case 168:                     // 'json'
        lookahead2W(282);           // EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral | StringLiteral |
        break;
      default:
        lk = l1;
      }
      if (lk == 18088)              // 'json' '('
      {
        lk = memoized(15, e0);
        if (lk == 0)
        {
          var b0B = b0; var e0B = e0; var l1B = l1;
          var b1B = b1; var e1B = e1; var l2B = l2;
          var b2B = b2; var e2B = e2;
          try
          {
            shiftT(168);            // 'json'
            lk = -1;
          }
          catch (p1B)
          {
            lk = -2;
          }
          b0 = b0B; e0 = e0B; l1 = l1B; if (l1 == 0) {end = e0B;} else {
          b1 = b1B; e1 = e1B; l2 = l2B; if (l2 == 0) {end = e1B;} else {
          b2 = b2B; e2 = e2B; end = e2B; }}
          memoize(15, e0, lk);
        }
      }
      if (lk == -1
       || lk == 3240                // 'json' EQName^Token
       || lk == 4264                // 'json' IntegerLiteral
       || lk == 4776                // 'json' DecimalLiteral
       || lk == 5288                // 'json' DoubleLiteral
       || lk == 5800                // 'json' StringLiteral
       || lk == 9896                // 'json' NCName^Token
       || lk == 16040               // 'json' '$'
       || lk == 16552               // 'json' '$$'
       || lk == 17064               // 'json' '%'
       || lk == 18600               // 'json' '(#'
       || lk == 21160               // 'json' '+'
       || lk == 22184               // 'json' '-'
       || lk == 24232               // 'json' '/'
       || lk == 24744               // 'json' '//'
       || lk == 28328               // 'json' '<'
       || lk == 28840               // 'json' '<!--'
       || lk == 30888               // 'json' '<?'
       || lk == 35496               // 'json' '['
       || lk == 36520               // 'json' 'after'
       || lk == 37544               // 'json' 'allowing'
       || lk == 38056               // 'json' 'ancestor'
       || lk == 38568               // 'json' 'ancestor-or-self'
       || lk == 39080               // 'json' 'and'
       || lk == 40104               // 'json' 'append'
       || lk == 40616               // 'json' 'array'
       || lk == 41128               // 'json' 'as'
       || lk == 41640               // 'json' 'ascending'
       || lk == 42152               // 'json' 'at'
       || lk == 42664               // 'json' 'attribute'
       || lk == 43176               // 'json' 'base-uri'
       || lk == 43688               // 'json' 'before'
       || lk == 44200               // 'json' 'boundary-space'
       || lk == 44712               // 'json' 'break'
       || lk == 45736               // 'json' 'case'
       || lk == 46248               // 'json' 'cast'
       || lk == 46760               // 'json' 'castable'
       || lk == 47272               // 'json' 'catch'
       || lk == 48296               // 'json' 'child'
       || lk == 48808               // 'json' 'collation'
       || lk == 49832               // 'json' 'comment'
       || lk == 50344               // 'json' 'constraint'
       || lk == 50856               // 'json' 'construction'
       || lk == 52392               // 'json' 'context'
       || lk == 52904               // 'json' 'continue'
       || lk == 53416               // 'json' 'copy'
       || lk == 53928               // 'json' 'copy-namespaces'
       || lk == 54440               // 'json' 'count'
       || lk == 54952               // 'json' 'decimal-format'
       || lk == 55976               // 'json' 'declare'
       || lk == 56488               // 'json' 'default'
       || lk == 57000               // 'json' 'delete'
       || lk == 57512               // 'json' 'descendant'
       || lk == 58024               // 'json' 'descendant-or-self'
       || lk == 58536               // 'json' 'descending'
       || lk == 61096               // 'json' 'div'
       || lk == 61608               // 'json' 'document'
       || lk == 62120               // 'json' 'document-node'
       || lk == 62632               // 'json' 'element'
       || lk == 63144               // 'json' 'else'
       || lk == 63656               // 'json' 'empty'
       || lk == 64168               // 'json' 'empty-sequence'
       || lk == 64680               // 'json' 'encoding'
       || lk == 65192               // 'json' 'end'
       || lk == 66216               // 'json' 'eq'
       || lk == 66728               // 'json' 'every'
       || lk == 67752               // 'json' 'except'
       || lk == 68264               // 'json' 'exit'
       || lk == 68776               // 'json' 'external'
       || lk == 69288               // 'json' 'false'
       || lk == 69800               // 'json' 'first'
       || lk == 70312               // 'json' 'following'
       || lk == 70824               // 'json' 'following-sibling'
       || lk == 71336               // 'json' 'for'
       || lk == 72872               // 'json' 'from'
       || lk == 73384               // 'json' 'ft-option'
       || lk == 75432               // 'json' 'function'
       || lk == 75944               // 'json' 'ge'
       || lk == 76968               // 'json' 'group'
       || lk == 77992               // 'json' 'gt'
       || lk == 78504               // 'json' 'idiv'
       || lk == 79016               // 'json' 'if'
       || lk == 79528               // 'json' 'import'
       || lk == 80040               // 'json' 'in'
       || lk == 80552               // 'json' 'index'
       || lk == 82600               // 'json' 'insert'
       || lk == 83112               // 'json' 'instance'
       || lk == 83624               // 'json' 'integrity'
       || lk == 84136               // 'json' 'intersect'
       || lk == 84648               // 'json' 'into'
       || lk == 85160               // 'json' 'is'
       || lk == 85672               // 'json' 'item'
       || lk == 86184               // 'json' 'json'
       || lk == 86696               // 'json' 'json-item'
       || lk == 87208               // 'json' 'jsoniq'
       || lk == 88744               // 'json' 'last'
       || lk == 89256               // 'json' 'lax'
       || lk == 89768               // 'json' 'le'
       || lk == 90792               // 'json' 'let'
       || lk == 91816               // 'json' 'loop'
       || lk == 92840               // 'json' 'lt'
       || lk == 93864               // 'json' 'mod'
       || lk == 94376               // 'json' 'modify'
       || lk == 94888               // 'json' 'module'
       || lk == 95912               // 'json' 'namespace'
       || lk == 96424               // 'json' 'namespace-node'
       || lk == 96936               // 'json' 'ne'
       || lk == 99496               // 'json' 'node'
       || lk == 100008              // 'json' 'nodes'
       || lk == 100520              // 'json' 'not'
       || lk == 101032              // 'json' 'null'
       || lk == 101544              // 'json' 'object'
       || lk == 103592              // 'json' 'only'
       || lk == 104104              // 'json' 'option'
       || lk == 104616              // 'json' 'or'
       || lk == 105128              // 'json' 'order'
       || lk == 105640              // 'json' 'ordered'
       || lk == 106152              // 'json' 'ordering'
       || lk == 107688              // 'json' 'parent'
       || lk == 110760              // 'json' 'preceding'
       || lk == 111272              // 'json' 'preceding-sibling'
       || lk == 112808              // 'json' 'processing-instruction'
       || lk == 113832              // 'json' 'rename'
       || lk == 114344              // 'json' 'replace'
       || lk == 114856              // 'json' 'return'
       || lk == 115368              // 'json' 'returning'
       || lk == 115880              // 'json' 'revalidation'
       || lk == 116904              // 'json' 'satisfies'
       || lk == 117416              // 'json' 'schema'
       || lk == 117928              // 'json' 'schema-attribute'
       || lk == 118440              // 'json' 'schema-element'
       || lk == 118952              // 'json' 'score'
       || lk == 119464              // 'json' 'select'
       || lk == 119976              // 'json' 'self'
       || lk == 122536              // 'json' 'sliding'
       || lk == 123048              // 'json' 'some'
       || lk == 123560              // 'json' 'stable'
       || lk == 124072              // 'json' 'start'
       || lk == 125608              // 'json' 'strict'
       || lk == 126632              // 'json' 'structured-item'
       || lk == 127144              // 'json' 'switch'
       || lk == 127656              // 'json' 'text'
       || lk == 129704              // 'json' 'to'
       || lk == 130216              // 'json' 'treat'
       || lk == 130728              // 'json' 'true'
       || lk == 131240              // 'json' 'try'
       || lk == 131752              // 'json' 'tumbling'
       || lk == 132264              // 'json' 'type'
       || lk == 132776              // 'json' 'typeswitch'
       || lk == 133288              // 'json' 'union'
       || lk == 134312              // 'json' 'unordered'
       || lk == 134824              // 'json' 'updating'
       || lk == 136360              // 'json' 'validate'
       || lk == 136872              // 'json' 'value'
       || lk == 137384              // 'json' 'variable'
       || lk == 137896              // 'json' 'version'
       || lk == 139432              // 'json' 'where'
       || lk == 139944              // 'json' 'while'
       || lk == 141480              // 'json' 'with'
       || lk == 144040              // 'json' '{'
       || lk == 145064)             // 'json' '{|'
      {
        shift(168);                 // 'json'
      }
      lookahead1W(268);             // EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral | StringLiteral |
      whitespace();
      parse_PairConstructorList();
      shift(165);                   // 'into'
      lookahead1W(267);             // EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral | StringLiteral |
      whitespace();
      parse_ExprSingle();
    }
    eventHandler.endNonterminal("JSONInsertExpr", e0);
  }

  function try_JSONInsertExpr()
  {
    switch (l1)
    {
    case 161:                       // 'insert'
      lookahead2W(268);             // EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral | StringLiteral |
      break;
    default:
      lk = l1;
    }
    if (lk != 9889)                 // 'insert' NCName^Token
    {
      lk = memoized(12, e0);
      if (lk == 0)
      {
        var b0A = b0; var e0A = e0; var l1A = l1;
        var b1A = b1; var e1A = e1; var l2A = l2;
        var b2A = b2; var e2A = e2;
        try
        {
          shiftT(161);              // 'insert'
          lookahead1W(267);         // EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral | StringLiteral |
          switch (l1)
          {
          case 168:                 // 'json'
            lookahead2W(269);       // EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral | StringLiteral |
            break;
          default:
            lk = l1;
          }
          if (lk == 18088)          // 'json' '('
          {
            lk = memoized(13, e0);
            if (lk == 0)
            {
              var b0B = b0; var e0B = e0; var l1B = l1;
              var b1B = b1; var e1B = e1; var l2B = l2;
              var b2B = b2; var e2B = e2;
              try
              {
                shiftT(168);        // 'json'
                memoize(13, e0B, -1);
              }
              catch (p1B)
              {
                b0 = b0B; e0 = e0B; l1 = l1B; if (l1 == 0) {end = e0B;} else {
                b1 = b1B; e1 = e1B; l2 = l2B; if (l2 == 0) {end = e1B;} else {
                b2 = b2B; e2 = e2B; end = e2B; }}
                memoize(13, e0B, -2);
              }
              lk = -2;
            }
          }
          if (lk == -1
           || lk == 3240            // 'json' EQName^Token
           || lk == 4264            // 'json' IntegerLiteral
           || lk == 4776            // 'json' DecimalLiteral
           || lk == 5288            // 'json' DoubleLiteral
           || lk == 5800            // 'json' StringLiteral
           || lk == 16040           // 'json' '$'
           || lk == 16552           // 'json' '$$'
           || lk == 17064           // 'json' '%'
           || lk == 18600           // 'json' '(#'
           || lk == 21160           // 'json' '+'
           || lk == 22184           // 'json' '-'
           || lk == 24232           // 'json' '/'
           || lk == 24744           // 'json' '//'
           || lk == 28328           // 'json' '<'
           || lk == 28840           // 'json' '<!--'
           || lk == 30888           // 'json' '<?'
           || lk == 35496           // 'json' '['
           || lk == 36520           // 'json' 'after'
           || lk == 37544           // 'json' 'allowing'
           || lk == 38056           // 'json' 'ancestor'
           || lk == 38568           // 'json' 'ancestor-or-self'
           || lk == 39080           // 'json' 'and'
           || lk == 40104           // 'json' 'append'
           || lk == 40616           // 'json' 'array'
           || lk == 41128           // 'json' 'as'
           || lk == 41640           // 'json' 'ascending'
           || lk == 42152           // 'json' 'at'
           || lk == 42664           // 'json' 'attribute'
           || lk == 43176           // 'json' 'base-uri'
           || lk == 43688           // 'json' 'before'
           || lk == 44200           // 'json' 'boundary-space'
           || lk == 44712           // 'json' 'break'
           || lk == 45736           // 'json' 'case'
           || lk == 46248           // 'json' 'cast'
           || lk == 46760           // 'json' 'castable'
           || lk == 47272           // 'json' 'catch'
           || lk == 48296           // 'json' 'child'
           || lk == 48808           // 'json' 'collation'
           || lk == 49832           // 'json' 'comment'
           || lk == 50344           // 'json' 'constraint'
           || lk == 50856           // 'json' 'construction'
           || lk == 52392           // 'json' 'context'
           || lk == 52904           // 'json' 'continue'
           || lk == 53416           // 'json' 'copy'
           || lk == 53928           // 'json' 'copy-namespaces'
           || lk == 54440           // 'json' 'count'
           || lk == 54952           // 'json' 'decimal-format'
           || lk == 55976           // 'json' 'declare'
           || lk == 56488           // 'json' 'default'
           || lk == 57000           // 'json' 'delete'
           || lk == 57512           // 'json' 'descendant'
           || lk == 58024           // 'json' 'descendant-or-self'
           || lk == 58536           // 'json' 'descending'
           || lk == 61096           // 'json' 'div'
           || lk == 61608           // 'json' 'document'
           || lk == 62120           // 'json' 'document-node'
           || lk == 62632           // 'json' 'element'
           || lk == 63144           // 'json' 'else'
           || lk == 63656           // 'json' 'empty'
           || lk == 64168           // 'json' 'empty-sequence'
           || lk == 64680           // 'json' 'encoding'
           || lk == 65192           // 'json' 'end'
           || lk == 66216           // 'json' 'eq'
           || lk == 66728           // 'json' 'every'
           || lk == 67752           // 'json' 'except'
           || lk == 68264           // 'json' 'exit'
           || lk == 68776           // 'json' 'external'
           || lk == 69288           // 'json' 'false'
           || lk == 69800           // 'json' 'first'
           || lk == 70312           // 'json' 'following'
           || lk == 70824           // 'json' 'following-sibling'
           || lk == 71336           // 'json' 'for'
           || lk == 72872           // 'json' 'from'
           || lk == 73384           // 'json' 'ft-option'
           || lk == 75432           // 'json' 'function'
           || lk == 75944           // 'json' 'ge'
           || lk == 76968           // 'json' 'group'
           || lk == 77992           // 'json' 'gt'
           || lk == 78504           // 'json' 'idiv'
           || lk == 79016           // 'json' 'if'
           || lk == 79528           // 'json' 'import'
           || lk == 80040           // 'json' 'in'
           || lk == 80552           // 'json' 'index'
           || lk == 82600           // 'json' 'insert'
           || lk == 83112           // 'json' 'instance'
           || lk == 83624           // 'json' 'integrity'
           || lk == 84136           // 'json' 'intersect'
           || lk == 84648           // 'json' 'into'
           || lk == 85160           // 'json' 'is'
           || lk == 85672           // 'json' 'item'
           || lk == 86184           // 'json' 'json'
           || lk == 86696           // 'json' 'json-item'
           || lk == 87208           // 'json' 'jsoniq'
           || lk == 88744           // 'json' 'last'
           || lk == 89256           // 'json' 'lax'
           || lk == 89768           // 'json' 'le'
           || lk == 90792           // 'json' 'let'
           || lk == 91816           // 'json' 'loop'
           || lk == 92840           // 'json' 'lt'
           || lk == 93864           // 'json' 'mod'
           || lk == 94376           // 'json' 'modify'
           || lk == 94888           // 'json' 'module'
           || lk == 95912           // 'json' 'namespace'
           || lk == 96424           // 'json' 'namespace-node'
           || lk == 96936           // 'json' 'ne'
           || lk == 99496           // 'json' 'node'
           || lk == 100008          // 'json' 'nodes'
           || lk == 100520          // 'json' 'not'
           || lk == 101032          // 'json' 'null'
           || lk == 101544          // 'json' 'object'
           || lk == 103592          // 'json' 'only'
           || lk == 104104          // 'json' 'option'
           || lk == 104616          // 'json' 'or'
           || lk == 105128          // 'json' 'order'
           || lk == 105640          // 'json' 'ordered'
           || lk == 106152          // 'json' 'ordering'
           || lk == 107688          // 'json' 'parent'
           || lk == 110760          // 'json' 'preceding'
           || lk == 111272          // 'json' 'preceding-sibling'
           || lk == 112808          // 'json' 'processing-instruction'
           || lk == 113832          // 'json' 'rename'
           || lk == 114344          // 'json' 'replace'
           || lk == 114856          // 'json' 'return'
           || lk == 115368          // 'json' 'returning'
           || lk == 115880          // 'json' 'revalidation'
           || lk == 116904          // 'json' 'satisfies'
           || lk == 117416          // 'json' 'schema'
           || lk == 117928          // 'json' 'schema-attribute'
           || lk == 118440          // 'json' 'schema-element'
           || lk == 118952          // 'json' 'score'
           || lk == 119464          // 'json' 'select'
           || lk == 119976          // 'json' 'self'
           || lk == 122536          // 'json' 'sliding'
           || lk == 123048          // 'json' 'some'
           || lk == 123560          // 'json' 'stable'
           || lk == 124072          // 'json' 'start'
           || lk == 125608          // 'json' 'strict'
           || lk == 126632          // 'json' 'structured-item'
           || lk == 127144          // 'json' 'switch'
           || lk == 127656          // 'json' 'text'
           || lk == 129704          // 'json' 'to'
           || lk == 130216          // 'json' 'treat'
           || lk == 130728          // 'json' 'true'
           || lk == 131240          // 'json' 'try'
           || lk == 131752          // 'json' 'tumbling'
           || lk == 132264          // 'json' 'type'
           || lk == 132776          // 'json' 'typeswitch'
           || lk == 133288          // 'json' 'union'
           || lk == 134312          // 'json' 'unordered'
           || lk == 134824          // 'json' 'updating'
           || lk == 136360          // 'json' 'validate'
           || lk == 136872          // 'json' 'value'
           || lk == 137384          // 'json' 'variable'
           || lk == 137896          // 'json' 'version'
           || lk == 139432          // 'json' 'where'
           || lk == 139944          // 'json' 'while'
           || lk == 141480          // 'json' 'with'
           || lk == 144040          // 'json' '{'
           || lk == 145064)         // 'json' '{|'
          {
            shiftT(168);            // 'json'
          }
          lookahead1W(267);         // EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral | StringLiteral |
          try_ExprSingle();
          shiftT(165);              // 'into'
          lookahead1W(267);         // EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral | StringLiteral |
          try_ExprSingle();
          switch (l1)
          {
          case 82:                  // 'at'
            lookahead2W(72);        // S^WS | '(:' | 'position'
            break;
          default:
            lk = l1;
          }
          if (lk == 110162)         // 'at' 'position'
          {
            lk = memoized(14, e0);
            if (lk == 0)
            {
              var b0B = b0; var e0B = e0; var l1B = l1;
              var b1B = b1; var e1B = e1; var l2B = l2;
              var b2B = b2; var e2B = e2;
              try
              {
                shiftT(82);         // 'at'
                lookahead1W(72);    // S^WS | '(:' | 'position'
                shiftT(215);        // 'position'
                lookahead1W(267);   // EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral | StringLiteral |
                try_ExprSingle();
                memoize(14, e0B, -1);
              }
              catch (p1B)
              {
                b0 = b0B; e0 = e0B; l1 = l1B; if (l1 == 0) {end = e0B;} else {
                b1 = b1B; e1 = e1B; l2 = l2B; if (l2 == 0) {end = e1B;} else {
                b2 = b2B; e2 = e2B; end = e2B; }}
                memoize(14, e0B, -2);
              }
              lk = -2;
            }
          }
          if (lk == -1)
          {
            shiftT(82);             // 'at'
            lookahead1W(72);        // S^WS | '(:' | 'position'
            shiftT(215);            // 'position'
            lookahead1W(267);       // EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral | StringLiteral |
            try_ExprSingle();
          }
          memoize(12, e0A, -1);
          lk = -3;
        }
        catch (p1A)
        {
          lk = -2;
          b0 = b0A; e0 = e0A; l1 = l1A; if (l1 == 0) {end = e0A;} else {
          b1 = b1A; e1 = e1A; l2 = l2A; if (l2 == 0) {end = e1A;} else {
          b2 = b2A; e2 = e2A; end = e2A; }}
          memoize(12, e0A, -2);
        }
      }
    }
    switch (lk)
    {
    case -1:
      shiftT(161);                  // 'insert'
      lookahead1W(267);             // EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral | StringLiteral |
      switch (l1)
      {
      case 168:                     // 'json'
        lookahead2W(269);           // EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral | StringLiteral |
        break;
      default:
        lk = l1;
      }
      if (lk == 18088)              // 'json' '('
      {
        lk = memoized(13, e0);
        if (lk == 0)
        {
          var b0B = b0; var e0B = e0; var l1B = l1;
          var b1B = b1; var e1B = e1; var l2B = l2;
          var b2B = b2; var e2B = e2;
          try
          {
            shiftT(168);            // 'json'
            memoize(13, e0B, -1);
          }
          catch (p1B)
          {
            b0 = b0B; e0 = e0B; l1 = l1B; if (l1 == 0) {end = e0B;} else {
            b1 = b1B; e1 = e1B; l2 = l2B; if (l2 == 0) {end = e1B;} else {
            b2 = b2B; e2 = e2B; end = e2B; }}
            memoize(13, e0B, -2);
          }
          lk = -2;
        }
      }
      if (lk == -1
       || lk == 3240                // 'json' EQName^Token
       || lk == 4264                // 'json' IntegerLiteral
       || lk == 4776                // 'json' DecimalLiteral
       || lk == 5288                // 'json' DoubleLiteral
       || lk == 5800                // 'json' StringLiteral
       || lk == 16040               // 'json' '$'
       || lk == 16552               // 'json' '$$'
       || lk == 17064               // 'json' '%'
       || lk == 18600               // 'json' '(#'
       || lk == 21160               // 'json' '+'
       || lk == 22184               // 'json' '-'
       || lk == 24232               // 'json' '/'
       || lk == 24744               // 'json' '//'
       || lk == 28328               // 'json' '<'
       || lk == 28840               // 'json' '<!--'
       || lk == 30888               // 'json' '<?'
       || lk == 35496               // 'json' '['
       || lk == 36520               // 'json' 'after'
       || lk == 37544               // 'json' 'allowing'
       || lk == 38056               // 'json' 'ancestor'
       || lk == 38568               // 'json' 'ancestor-or-self'
       || lk == 39080               // 'json' 'and'
       || lk == 40104               // 'json' 'append'
       || lk == 40616               // 'json' 'array'
       || lk == 41128               // 'json' 'as'
       || lk == 41640               // 'json' 'ascending'
       || lk == 42152               // 'json' 'at'
       || lk == 42664               // 'json' 'attribute'
       || lk == 43176               // 'json' 'base-uri'
       || lk == 43688               // 'json' 'before'
       || lk == 44200               // 'json' 'boundary-space'
       || lk == 44712               // 'json' 'break'
       || lk == 45736               // 'json' 'case'
       || lk == 46248               // 'json' 'cast'
       || lk == 46760               // 'json' 'castable'
       || lk == 47272               // 'json' 'catch'
       || lk == 48296               // 'json' 'child'
       || lk == 48808               // 'json' 'collation'
       || lk == 49832               // 'json' 'comment'
       || lk == 50344               // 'json' 'constraint'
       || lk == 50856               // 'json' 'construction'
       || lk == 52392               // 'json' 'context'
       || lk == 52904               // 'json' 'continue'
       || lk == 53416               // 'json' 'copy'
       || lk == 53928               // 'json' 'copy-namespaces'
       || lk == 54440               // 'json' 'count'
       || lk == 54952               // 'json' 'decimal-format'
       || lk == 55976               // 'json' 'declare'
       || lk == 56488               // 'json' 'default'
       || lk == 57000               // 'json' 'delete'
       || lk == 57512               // 'json' 'descendant'
       || lk == 58024               // 'json' 'descendant-or-self'
       || lk == 58536               // 'json' 'descending'
       || lk == 61096               // 'json' 'div'
       || lk == 61608               // 'json' 'document'
       || lk == 62120               // 'json' 'document-node'
       || lk == 62632               // 'json' 'element'
       || lk == 63144               // 'json' 'else'
       || lk == 63656               // 'json' 'empty'
       || lk == 64168               // 'json' 'empty-sequence'
       || lk == 64680               // 'json' 'encoding'
       || lk == 65192               // 'json' 'end'
       || lk == 66216               // 'json' 'eq'
       || lk == 66728               // 'json' 'every'
       || lk == 67752               // 'json' 'except'
       || lk == 68264               // 'json' 'exit'
       || lk == 68776               // 'json' 'external'
       || lk == 69288               // 'json' 'false'
       || lk == 69800               // 'json' 'first'
       || lk == 70312               // 'json' 'following'
       || lk == 70824               // 'json' 'following-sibling'
       || lk == 71336               // 'json' 'for'
       || lk == 72872               // 'json' 'from'
       || lk == 73384               // 'json' 'ft-option'
       || lk == 75432               // 'json' 'function'
       || lk == 75944               // 'json' 'ge'
       || lk == 76968               // 'json' 'group'
       || lk == 77992               // 'json' 'gt'
       || lk == 78504               // 'json' 'idiv'
       || lk == 79016               // 'json' 'if'
       || lk == 79528               // 'json' 'import'
       || lk == 80040               // 'json' 'in'
       || lk == 80552               // 'json' 'index'
       || lk == 82600               // 'json' 'insert'
       || lk == 83112               // 'json' 'instance'
       || lk == 83624               // 'json' 'integrity'
       || lk == 84136               // 'json' 'intersect'
       || lk == 84648               // 'json' 'into'
       || lk == 85160               // 'json' 'is'
       || lk == 85672               // 'json' 'item'
       || lk == 86184               // 'json' 'json'
       || lk == 86696               // 'json' 'json-item'
       || lk == 87208               // 'json' 'jsoniq'
       || lk == 88744               // 'json' 'last'
       || lk == 89256               // 'json' 'lax'
       || lk == 89768               // 'json' 'le'
       || lk == 90792               // 'json' 'let'
       || lk == 91816               // 'json' 'loop'
       || lk == 92840               // 'json' 'lt'
       || lk == 93864               // 'json' 'mod'
       || lk == 94376               // 'json' 'modify'
       || lk == 94888               // 'json' 'module'
       || lk == 95912               // 'json' 'namespace'
       || lk == 96424               // 'json' 'namespace-node'
       || lk == 96936               // 'json' 'ne'
       || lk == 99496               // 'json' 'node'
       || lk == 100008              // 'json' 'nodes'
       || lk == 100520              // 'json' 'not'
       || lk == 101032              // 'json' 'null'
       || lk == 101544              // 'json' 'object'
       || lk == 103592              // 'json' 'only'
       || lk == 104104              // 'json' 'option'
       || lk == 104616              // 'json' 'or'
       || lk == 105128              // 'json' 'order'
       || lk == 105640              // 'json' 'ordered'
       || lk == 106152              // 'json' 'ordering'
       || lk == 107688              // 'json' 'parent'
       || lk == 110760              // 'json' 'preceding'
       || lk == 111272              // 'json' 'preceding-sibling'
       || lk == 112808              // 'json' 'processing-instruction'
       || lk == 113832              // 'json' 'rename'
       || lk == 114344              // 'json' 'replace'
       || lk == 114856              // 'json' 'return'
       || lk == 115368              // 'json' 'returning'
       || lk == 115880              // 'json' 'revalidation'
       || lk == 116904              // 'json' 'satisfies'
       || lk == 117416              // 'json' 'schema'
       || lk == 117928              // 'json' 'schema-attribute'
       || lk == 118440              // 'json' 'schema-element'
       || lk == 118952              // 'json' 'score'
       || lk == 119464              // 'json' 'select'
       || lk == 119976              // 'json' 'self'
       || lk == 122536              // 'json' 'sliding'
       || lk == 123048              // 'json' 'some'
       || lk == 123560              // 'json' 'stable'
       || lk == 124072              // 'json' 'start'
       || lk == 125608              // 'json' 'strict'
       || lk == 126632              // 'json' 'structured-item'
       || lk == 127144              // 'json' 'switch'
       || lk == 127656              // 'json' 'text'
       || lk == 129704              // 'json' 'to'
       || lk == 130216              // 'json' 'treat'
       || lk == 130728              // 'json' 'true'
       || lk == 131240              // 'json' 'try'
       || lk == 131752              // 'json' 'tumbling'
       || lk == 132264              // 'json' 'type'
       || lk == 132776              // 'json' 'typeswitch'
       || lk == 133288              // 'json' 'union'
       || lk == 134312              // 'json' 'unordered'
       || lk == 134824              // 'json' 'updating'
       || lk == 136360              // 'json' 'validate'
       || lk == 136872              // 'json' 'value'
       || lk == 137384              // 'json' 'variable'
       || lk == 137896              // 'json' 'version'
       || lk == 139432              // 'json' 'where'
       || lk == 139944              // 'json' 'while'
       || lk == 141480              // 'json' 'with'
       || lk == 144040              // 'json' '{'
       || lk == 145064)             // 'json' '{|'
      {
        shiftT(168);                // 'json'
      }
      lookahead1W(267);             // EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral | StringLiteral |
      try_ExprSingle();
      shiftT(165);                  // 'into'
      lookahead1W(267);             // EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral | StringLiteral |
      try_ExprSingle();
      switch (l1)
      {
      case 82:                      // 'at'
        lookahead2W(72);            // S^WS | '(:' | 'position'
        break;
      default:
        lk = l1;
      }
      if (lk == 110162)             // 'at' 'position'
      {
        lk = memoized(14, e0);
        if (lk == 0)
        {
          var b0B = b0; var e0B = e0; var l1B = l1;
          var b1B = b1; var e1B = e1; var l2B = l2;
          var b2B = b2; var e2B = e2;
          try
          {
            shiftT(82);             // 'at'
            lookahead1W(72);        // S^WS | '(:' | 'position'
            shiftT(215);            // 'position'
            lookahead1W(267);       // EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral | StringLiteral |
            try_ExprSingle();
            memoize(14, e0B, -1);
          }
          catch (p1B)
          {
            b0 = b0B; e0 = e0B; l1 = l1B; if (l1 == 0) {end = e0B;} else {
            b1 = b1B; e1 = e1B; l2 = l2B; if (l2 == 0) {end = e1B;} else {
            b2 = b2B; e2 = e2B; end = e2B; }}
            memoize(14, e0B, -2);
          }
          lk = -2;
        }
      }
      if (lk == -1)
      {
        shiftT(82);                 // 'at'
        lookahead1W(72);            // S^WS | '(:' | 'position'
        shiftT(215);                // 'position'
        lookahead1W(267);           // EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral | StringLiteral |
        try_ExprSingle();
      }
      break;
    case -3:
      break;
    default:
      shiftT(161);                  // 'insert'
      lookahead1W(268);             // EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral | StringLiteral |
      switch (l1)
      {
      case 168:                     // 'json'
        lookahead2W(282);           // EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral | StringLiteral |
        break;
      default:
        lk = l1;
      }
      if (lk == 18088)              // 'json' '('
      {
        lk = memoized(15, e0);
        if (lk == 0)
        {
          var b0B = b0; var e0B = e0; var l1B = l1;
          var b1B = b1; var e1B = e1; var l2B = l2;
          var b2B = b2; var e2B = e2;
          try
          {
            shiftT(168);            // 'json'
            memoize(15, e0B, -1);
          }
          catch (p1B)
          {
            b0 = b0B; e0 = e0B; l1 = l1B; if (l1 == 0) {end = e0B;} else {
            b1 = b1B; e1 = e1B; l2 = l2B; if (l2 == 0) {end = e1B;} else {
            b2 = b2B; e2 = e2B; end = e2B; }}
            memoize(15, e0B, -2);
          }
          lk = -2;
        }
      }
      if (lk == -1
       || lk == 3240                // 'json' EQName^Token
       || lk == 4264                // 'json' IntegerLiteral
       || lk == 4776                // 'json' DecimalLiteral
       || lk == 5288                // 'json' DoubleLiteral
       || lk == 5800                // 'json' StringLiteral
       || lk == 9896                // 'json' NCName^Token
       || lk == 16040               // 'json' '$'
       || lk == 16552               // 'json' '$$'
       || lk == 17064               // 'json' '%'
       || lk == 18600               // 'json' '(#'
       || lk == 21160               // 'json' '+'
       || lk == 22184               // 'json' '-'
       || lk == 24232               // 'json' '/'
       || lk == 24744               // 'json' '//'
       || lk == 28328               // 'json' '<'
       || lk == 28840               // 'json' '<!--'
       || lk == 30888               // 'json' '<?'
       || lk == 35496               // 'json' '['
       || lk == 36520               // 'json' 'after'
       || lk == 37544               // 'json' 'allowing'
       || lk == 38056               // 'json' 'ancestor'
       || lk == 38568               // 'json' 'ancestor-or-self'
       || lk == 39080               // 'json' 'and'
       || lk == 40104               // 'json' 'append'
       || lk == 40616               // 'json' 'array'
       || lk == 41128               // 'json' 'as'
       || lk == 41640               // 'json' 'ascending'
       || lk == 42152               // 'json' 'at'
       || lk == 42664               // 'json' 'attribute'
       || lk == 43176               // 'json' 'base-uri'
       || lk == 43688               // 'json' 'before'
       || lk == 44200               // 'json' 'boundary-space'
       || lk == 44712               // 'json' 'break'
       || lk == 45736               // 'json' 'case'
       || lk == 46248               // 'json' 'cast'
       || lk == 46760               // 'json' 'castable'
       || lk == 47272               // 'json' 'catch'
       || lk == 48296               // 'json' 'child'
       || lk == 48808               // 'json' 'collation'
       || lk == 49832               // 'json' 'comment'
       || lk == 50344               // 'json' 'constraint'
       || lk == 50856               // 'json' 'construction'
       || lk == 52392               // 'json' 'context'
       || lk == 52904               // 'json' 'continue'
       || lk == 53416               // 'json' 'copy'
       || lk == 53928               // 'json' 'copy-namespaces'
       || lk == 54440               // 'json' 'count'
       || lk == 54952               // 'json' 'decimal-format'
       || lk == 55976               // 'json' 'declare'
       || lk == 56488               // 'json' 'default'
       || lk == 57000               // 'json' 'delete'
       || lk == 57512               // 'json' 'descendant'
       || lk == 58024               // 'json' 'descendant-or-self'
       || lk == 58536               // 'json' 'descending'
       || lk == 61096               // 'json' 'div'
       || lk == 61608               // 'json' 'document'
       || lk == 62120               // 'json' 'document-node'
       || lk == 62632               // 'json' 'element'
       || lk == 63144               // 'json' 'else'
       || lk == 63656               // 'json' 'empty'
       || lk == 64168               // 'json' 'empty-sequence'
       || lk == 64680               // 'json' 'encoding'
       || lk == 65192               // 'json' 'end'
       || lk == 66216               // 'json' 'eq'
       || lk == 66728               // 'json' 'every'
       || lk == 67752               // 'json' 'except'
       || lk == 68264               // 'json' 'exit'
       || lk == 68776               // 'json' 'external'
       || lk == 69288               // 'json' 'false'
       || lk == 69800               // 'json' 'first'
       || lk == 70312               // 'json' 'following'
       || lk == 70824               // 'json' 'following-sibling'
       || lk == 71336               // 'json' 'for'
       || lk == 72872               // 'json' 'from'
       || lk == 73384               // 'json' 'ft-option'
       || lk == 75432               // 'json' 'function'
       || lk == 75944               // 'json' 'ge'
       || lk == 76968               // 'json' 'group'
       || lk == 77992               // 'json' 'gt'
       || lk == 78504               // 'json' 'idiv'
       || lk == 79016               // 'json' 'if'
       || lk == 79528               // 'json' 'import'
       || lk == 80040               // 'json' 'in'
       || lk == 80552               // 'json' 'index'
       || lk == 82600               // 'json' 'insert'
       || lk == 83112               // 'json' 'instance'
       || lk == 83624               // 'json' 'integrity'
       || lk == 84136               // 'json' 'intersect'
       || lk == 84648               // 'json' 'into'
       || lk == 85160               // 'json' 'is'
       || lk == 85672               // 'json' 'item'
       || lk == 86184               // 'json' 'json'
       || lk == 86696               // 'json' 'json-item'
       || lk == 87208               // 'json' 'jsoniq'
       || lk == 88744               // 'json' 'last'
       || lk == 89256               // 'json' 'lax'
       || lk == 89768               // 'json' 'le'
       || lk == 90792               // 'json' 'let'
       || lk == 91816               // 'json' 'loop'
       || lk == 92840               // 'json' 'lt'
       || lk == 93864               // 'json' 'mod'
       || lk == 94376               // 'json' 'modify'
       || lk == 94888               // 'json' 'module'
       || lk == 95912               // 'json' 'namespace'
       || lk == 96424               // 'json' 'namespace-node'
       || lk == 96936               // 'json' 'ne'
       || lk == 99496               // 'json' 'node'
       || lk == 100008              // 'json' 'nodes'
       || lk == 100520              // 'json' 'not'
       || lk == 101032              // 'json' 'null'
       || lk == 101544              // 'json' 'object'
       || lk == 103592              // 'json' 'only'
       || lk == 104104              // 'json' 'option'
       || lk == 104616              // 'json' 'or'
       || lk == 105128              // 'json' 'order'
       || lk == 105640              // 'json' 'ordered'
       || lk == 106152              // 'json' 'ordering'
       || lk == 107688              // 'json' 'parent'
       || lk == 110760              // 'json' 'preceding'
       || lk == 111272              // 'json' 'preceding-sibling'
       || lk == 112808              // 'json' 'processing-instruction'
       || lk == 113832              // 'json' 'rename'
       || lk == 114344              // 'json' 'replace'
       || lk == 114856              // 'json' 'return'
       || lk == 115368              // 'json' 'returning'
       || lk == 115880              // 'json' 'revalidation'
       || lk == 116904              // 'json' 'satisfies'
       || lk == 117416              // 'json' 'schema'
       || lk == 117928              // 'json' 'schema-attribute'
       || lk == 118440              // 'json' 'schema-element'
       || lk == 118952              // 'json' 'score'
       || lk == 119464              // 'json' 'select'
       || lk == 119976              // 'json' 'self'
       || lk == 122536              // 'json' 'sliding'
       || lk == 123048              // 'json' 'some'
       || lk == 123560              // 'json' 'stable'
       || lk == 124072              // 'json' 'start'
       || lk == 125608              // 'json' 'strict'
       || lk == 126632              // 'json' 'structured-item'
       || lk == 127144              // 'json' 'switch'
       || lk == 127656              // 'json' 'text'
       || lk == 129704              // 'json' 'to'
       || lk == 130216              // 'json' 'treat'
       || lk == 130728              // 'json' 'true'
       || lk == 131240              // 'json' 'try'
       || lk == 131752              // 'json' 'tumbling'
       || lk == 132264              // 'json' 'type'
       || lk == 132776              // 'json' 'typeswitch'
       || lk == 133288              // 'json' 'union'
       || lk == 134312              // 'json' 'unordered'
       || lk == 134824              // 'json' 'updating'
       || lk == 136360              // 'json' 'validate'
       || lk == 136872              // 'json' 'value'
       || lk == 137384              // 'json' 'variable'
       || lk == 137896              // 'json' 'version'
       || lk == 139432              // 'json' 'where'
       || lk == 139944              // 'json' 'while'
       || lk == 141480              // 'json' 'with'
       || lk == 144040              // 'json' '{'
       || lk == 145064)             // 'json' '{|'
      {
        shiftT(168);                // 'json'
      }
      lookahead1W(268);             // EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral | StringLiteral |
      try_PairConstructorList();
      shiftT(165);                  // 'into'
      lookahead1W(267);             // EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral | StringLiteral |
      try_ExprSingle();
    }
  }

  function parse_JSONRenameExpr()
  {
    eventHandler.startNonterminal("JSONRenameExpr", e0);
    shift(222);                     // 'rename'
    lookahead1W(260);               // EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral | StringLiteral |
    switch (l1)
    {
    case 168:                       // 'json'
      lookahead2W(261);             // EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral | StringLiteral |
      break;
    default:
      lk = l1;
    }
    if (lk == 18088)                // 'json' '('
    {
      lk = memoized(16, e0);
      if (lk == 0)
      {
        var b0A = b0; var e0A = e0; var l1A = l1;
        var b1A = b1; var e1A = e1; var l2A = l2;
        var b2A = b2; var e2A = e2;
        try
        {
          shiftT(168);              // 'json'
          lk = -1;
        }
        catch (p1A)
        {
          lk = -2;
        }
        b0 = b0A; e0 = e0A; l1 = l1A; if (l1 == 0) {end = e0A;} else {
        b1 = b1A; e1 = e1A; l2 = l2A; if (l2 == 0) {end = e1A;} else {
        b2 = b2A; e2 = e2A; end = e2A; }}
        memoize(16, e0, lk);
      }
    }
    if (lk == -1
     || lk == 3240                  // 'json' EQName^Token
     || lk == 4264                  // 'json' IntegerLiteral
     || lk == 4776                  // 'json' DecimalLiteral
     || lk == 5288                  // 'json' DoubleLiteral
     || lk == 5800                  // 'json' StringLiteral
     || lk == 16040                 // 'json' '$'
     || lk == 16552                 // 'json' '$$'
     || lk == 17064                 // 'json' '%'
     || lk == 28328                 // 'json' '<'
     || lk == 28840                 // 'json' '<!--'
     || lk == 30888                 // 'json' '<?'
     || lk == 35496                 // 'json' '['
     || lk == 36520                 // 'json' 'after'
     || lk == 37544                 // 'json' 'allowing'
     || lk == 38056                 // 'json' 'ancestor'
     || lk == 38568                 // 'json' 'ancestor-or-self'
     || lk == 39080                 // 'json' 'and'
     || lk == 40104                 // 'json' 'append'
     || lk == 40616                 // 'json' 'array'
     || lk == 41128                 // 'json' 'as'
     || lk == 41640                 // 'json' 'ascending'
     || lk == 42152                 // 'json' 'at'
     || lk == 42664                 // 'json' 'attribute'
     || lk == 43176                 // 'json' 'base-uri'
     || lk == 43688                 // 'json' 'before'
     || lk == 44200                 // 'json' 'boundary-space'
     || lk == 44712                 // 'json' 'break'
     || lk == 45736                 // 'json' 'case'
     || lk == 46248                 // 'json' 'cast'
     || lk == 46760                 // 'json' 'castable'
     || lk == 47272                 // 'json' 'catch'
     || lk == 48296                 // 'json' 'child'
     || lk == 48808                 // 'json' 'collation'
     || lk == 49832                 // 'json' 'comment'
     || lk == 50344                 // 'json' 'constraint'
     || lk == 50856                 // 'json' 'construction'
     || lk == 52392                 // 'json' 'context'
     || lk == 52904                 // 'json' 'continue'
     || lk == 53416                 // 'json' 'copy'
     || lk == 53928                 // 'json' 'copy-namespaces'
     || lk == 54440                 // 'json' 'count'
     || lk == 54952                 // 'json' 'decimal-format'
     || lk == 55976                 // 'json' 'declare'
     || lk == 56488                 // 'json' 'default'
     || lk == 57000                 // 'json' 'delete'
     || lk == 57512                 // 'json' 'descendant'
     || lk == 58024                 // 'json' 'descendant-or-self'
     || lk == 58536                 // 'json' 'descending'
     || lk == 61096                 // 'json' 'div'
     || lk == 61608                 // 'json' 'document'
     || lk == 62120                 // 'json' 'document-node'
     || lk == 62632                 // 'json' 'element'
     || lk == 63144                 // 'json' 'else'
     || lk == 63656                 // 'json' 'empty'
     || lk == 64168                 // 'json' 'empty-sequence'
     || lk == 64680                 // 'json' 'encoding'
     || lk == 65192                 // 'json' 'end'
     || lk == 66216                 // 'json' 'eq'
     || lk == 66728                 // 'json' 'every'
     || lk == 67752                 // 'json' 'except'
     || lk == 68264                 // 'json' 'exit'
     || lk == 68776                 // 'json' 'external'
     || lk == 69288                 // 'json' 'false'
     || lk == 69800                 // 'json' 'first'
     || lk == 70312                 // 'json' 'following'
     || lk == 70824                 // 'json' 'following-sibling'
     || lk == 71336                 // 'json' 'for'
     || lk == 72872                 // 'json' 'from'
     || lk == 73384                 // 'json' 'ft-option'
     || lk == 75432                 // 'json' 'function'
     || lk == 75944                 // 'json' 'ge'
     || lk == 76968                 // 'json' 'group'
     || lk == 77992                 // 'json' 'gt'
     || lk == 78504                 // 'json' 'idiv'
     || lk == 79016                 // 'json' 'if'
     || lk == 79528                 // 'json' 'import'
     || lk == 80040                 // 'json' 'in'
     || lk == 80552                 // 'json' 'index'
     || lk == 82600                 // 'json' 'insert'
     || lk == 83112                 // 'json' 'instance'
     || lk == 83624                 // 'json' 'integrity'
     || lk == 84136                 // 'json' 'intersect'
     || lk == 84648                 // 'json' 'into'
     || lk == 85160                 // 'json' 'is'
     || lk == 85672                 // 'json' 'item'
     || lk == 86184                 // 'json' 'json'
     || lk == 86696                 // 'json' 'json-item'
     || lk == 87208                 // 'json' 'jsoniq'
     || lk == 88744                 // 'json' 'last'
     || lk == 89256                 // 'json' 'lax'
     || lk == 89768                 // 'json' 'le'
     || lk == 90792                 // 'json' 'let'
     || lk == 91816                 // 'json' 'loop'
     || lk == 92840                 // 'json' 'lt'
     || lk == 93864                 // 'json' 'mod'
     || lk == 94376                 // 'json' 'modify'
     || lk == 94888                 // 'json' 'module'
     || lk == 95912                 // 'json' 'namespace'
     || lk == 96424                 // 'json' 'namespace-node'
     || lk == 96936                 // 'json' 'ne'
     || lk == 99496                 // 'json' 'node'
     || lk == 100008                // 'json' 'nodes'
     || lk == 101032                // 'json' 'null'
     || lk == 101544                // 'json' 'object'
     || lk == 103592                // 'json' 'only'
     || lk == 104104                // 'json' 'option'
     || lk == 104616                // 'json' 'or'
     || lk == 105128                // 'json' 'order'
     || lk == 105640                // 'json' 'ordered'
     || lk == 106152                // 'json' 'ordering'
     || lk == 107688                // 'json' 'parent'
     || lk == 110760                // 'json' 'preceding'
     || lk == 111272                // 'json' 'preceding-sibling'
     || lk == 112808                // 'json' 'processing-instruction'
     || lk == 113832                // 'json' 'rename'
     || lk == 114344                // 'json' 'replace'
     || lk == 114856                // 'json' 'return'
     || lk == 115368                // 'json' 'returning'
     || lk == 115880                // 'json' 'revalidation'
     || lk == 116904                // 'json' 'satisfies'
     || lk == 117416                // 'json' 'schema'
     || lk == 117928                // 'json' 'schema-attribute'
     || lk == 118440                // 'json' 'schema-element'
     || lk == 118952                // 'json' 'score'
     || lk == 119464                // 'json' 'select'
     || lk == 119976                // 'json' 'self'
     || lk == 122536                // 'json' 'sliding'
     || lk == 123048                // 'json' 'some'
     || lk == 123560                // 'json' 'stable'
     || lk == 124072                // 'json' 'start'
     || lk == 125608                // 'json' 'strict'
     || lk == 126632                // 'json' 'structured-item'
     || lk == 127144                // 'json' 'switch'
     || lk == 127656                // 'json' 'text'
     || lk == 129704                // 'json' 'to'
     || lk == 130216                // 'json' 'treat'
     || lk == 130728                // 'json' 'true'
     || lk == 131240                // 'json' 'try'
     || lk == 131752                // 'json' 'tumbling'
     || lk == 132264                // 'json' 'type'
     || lk == 132776                // 'json' 'typeswitch'
     || lk == 133288                // 'json' 'union'
     || lk == 134312                // 'json' 'unordered'
     || lk == 134824                // 'json' 'updating'
     || lk == 136360                // 'json' 'validate'
     || lk == 136872                // 'json' 'value'
     || lk == 137384                // 'json' 'variable'
     || lk == 137896                // 'json' 'version'
     || lk == 139432                // 'json' 'where'
     || lk == 139944                // 'json' 'while'
     || lk == 141480                // 'json' 'with'
     || lk == 144040                // 'json' '{'
     || lk == 145064)               // 'json' '{|'
    {
      shift(168);                   // 'json'
    }
    lookahead1W(260);               // EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral | StringLiteral |
    whitespace();
    parse_PostfixExpr();
    shift(80);                      // 'as'
    lookahead1W(267);               // EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral | StringLiteral |
    whitespace();
    parse_ExprSingle();
    eventHandler.endNonterminal("JSONRenameExpr", e0);
  }

  function try_JSONRenameExpr()
  {
    shiftT(222);                    // 'rename'
    lookahead1W(260);               // EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral | StringLiteral |
    switch (l1)
    {
    case 168:                       // 'json'
      lookahead2W(261);             // EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral | StringLiteral |
      break;
    default:
      lk = l1;
    }
    if (lk == 18088)                // 'json' '('
    {
      lk = memoized(16, e0);
      if (lk == 0)
      {
        var b0A = b0; var e0A = e0; var l1A = l1;
        var b1A = b1; var e1A = e1; var l2A = l2;
        var b2A = b2; var e2A = e2;
        try
        {
          shiftT(168);              // 'json'
          memoize(16, e0A, -1);
        }
        catch (p1A)
        {
          b0 = b0A; e0 = e0A; l1 = l1A; if (l1 == 0) {end = e0A;} else {
          b1 = b1A; e1 = e1A; l2 = l2A; if (l2 == 0) {end = e1A;} else {
          b2 = b2A; e2 = e2A; end = e2A; }}
          memoize(16, e0A, -2);
        }
        lk = -2;
      }
    }
    if (lk == -1
     || lk == 3240                  // 'json' EQName^Token
     || lk == 4264                  // 'json' IntegerLiteral
     || lk == 4776                  // 'json' DecimalLiteral
     || lk == 5288                  // 'json' DoubleLiteral
     || lk == 5800                  // 'json' StringLiteral
     || lk == 16040                 // 'json' '$'
     || lk == 16552                 // 'json' '$$'
     || lk == 17064                 // 'json' '%'
     || lk == 28328                 // 'json' '<'
     || lk == 28840                 // 'json' '<!--'
     || lk == 30888                 // 'json' '<?'
     || lk == 35496                 // 'json' '['
     || lk == 36520                 // 'json' 'after'
     || lk == 37544                 // 'json' 'allowing'
     || lk == 38056                 // 'json' 'ancestor'
     || lk == 38568                 // 'json' 'ancestor-or-self'
     || lk == 39080                 // 'json' 'and'
     || lk == 40104                 // 'json' 'append'
     || lk == 40616                 // 'json' 'array'
     || lk == 41128                 // 'json' 'as'
     || lk == 41640                 // 'json' 'ascending'
     || lk == 42152                 // 'json' 'at'
     || lk == 42664                 // 'json' 'attribute'
     || lk == 43176                 // 'json' 'base-uri'
     || lk == 43688                 // 'json' 'before'
     || lk == 44200                 // 'json' 'boundary-space'
     || lk == 44712                 // 'json' 'break'
     || lk == 45736                 // 'json' 'case'
     || lk == 46248                 // 'json' 'cast'
     || lk == 46760                 // 'json' 'castable'
     || lk == 47272                 // 'json' 'catch'
     || lk == 48296                 // 'json' 'child'
     || lk == 48808                 // 'json' 'collation'
     || lk == 49832                 // 'json' 'comment'
     || lk == 50344                 // 'json' 'constraint'
     || lk == 50856                 // 'json' 'construction'
     || lk == 52392                 // 'json' 'context'
     || lk == 52904                 // 'json' 'continue'
     || lk == 53416                 // 'json' 'copy'
     || lk == 53928                 // 'json' 'copy-namespaces'
     || lk == 54440                 // 'json' 'count'
     || lk == 54952                 // 'json' 'decimal-format'
     || lk == 55976                 // 'json' 'declare'
     || lk == 56488                 // 'json' 'default'
     || lk == 57000                 // 'json' 'delete'
     || lk == 57512                 // 'json' 'descendant'
     || lk == 58024                 // 'json' 'descendant-or-self'
     || lk == 58536                 // 'json' 'descending'
     || lk == 61096                 // 'json' 'div'
     || lk == 61608                 // 'json' 'document'
     || lk == 62120                 // 'json' 'document-node'
     || lk == 62632                 // 'json' 'element'
     || lk == 63144                 // 'json' 'else'
     || lk == 63656                 // 'json' 'empty'
     || lk == 64168                 // 'json' 'empty-sequence'
     || lk == 64680                 // 'json' 'encoding'
     || lk == 65192                 // 'json' 'end'
     || lk == 66216                 // 'json' 'eq'
     || lk == 66728                 // 'json' 'every'
     || lk == 67752                 // 'json' 'except'
     || lk == 68264                 // 'json' 'exit'
     || lk == 68776                 // 'json' 'external'
     || lk == 69288                 // 'json' 'false'
     || lk == 69800                 // 'json' 'first'
     || lk == 70312                 // 'json' 'following'
     || lk == 70824                 // 'json' 'following-sibling'
     || lk == 71336                 // 'json' 'for'
     || lk == 72872                 // 'json' 'from'
     || lk == 73384                 // 'json' 'ft-option'
     || lk == 75432                 // 'json' 'function'
     || lk == 75944                 // 'json' 'ge'
     || lk == 76968                 // 'json' 'group'
     || lk == 77992                 // 'json' 'gt'
     || lk == 78504                 // 'json' 'idiv'
     || lk == 79016                 // 'json' 'if'
     || lk == 79528                 // 'json' 'import'
     || lk == 80040                 // 'json' 'in'
     || lk == 80552                 // 'json' 'index'
     || lk == 82600                 // 'json' 'insert'
     || lk == 83112                 // 'json' 'instance'
     || lk == 83624                 // 'json' 'integrity'
     || lk == 84136                 // 'json' 'intersect'
     || lk == 84648                 // 'json' 'into'
     || lk == 85160                 // 'json' 'is'
     || lk == 85672                 // 'json' 'item'
     || lk == 86184                 // 'json' 'json'
     || lk == 86696                 // 'json' 'json-item'
     || lk == 87208                 // 'json' 'jsoniq'
     || lk == 88744                 // 'json' 'last'
     || lk == 89256                 // 'json' 'lax'
     || lk == 89768                 // 'json' 'le'
     || lk == 90792                 // 'json' 'let'
     || lk == 91816                 // 'json' 'loop'
     || lk == 92840                 // 'json' 'lt'
     || lk == 93864                 // 'json' 'mod'
     || lk == 94376                 // 'json' 'modify'
     || lk == 94888                 // 'json' 'module'
     || lk == 95912                 // 'json' 'namespace'
     || lk == 96424                 // 'json' 'namespace-node'
     || lk == 96936                 // 'json' 'ne'
     || lk == 99496                 // 'json' 'node'
     || lk == 100008                // 'json' 'nodes'
     || lk == 101032                // 'json' 'null'
     || lk == 101544                // 'json' 'object'
     || lk == 103592                // 'json' 'only'
     || lk == 104104                // 'json' 'option'
     || lk == 104616                // 'json' 'or'
     || lk == 105128                // 'json' 'order'
     || lk == 105640                // 'json' 'ordered'
     || lk == 106152                // 'json' 'ordering'
     || lk == 107688                // 'json' 'parent'
     || lk == 110760                // 'json' 'preceding'
     || lk == 111272                // 'json' 'preceding-sibling'
     || lk == 112808                // 'json' 'processing-instruction'
     || lk == 113832                // 'json' 'rename'
     || lk == 114344                // 'json' 'replace'
     || lk == 114856                // 'json' 'return'
     || lk == 115368                // 'json' 'returning'
     || lk == 115880                // 'json' 'revalidation'
     || lk == 116904                // 'json' 'satisfies'
     || lk == 117416                // 'json' 'schema'
     || lk == 117928                // 'json' 'schema-attribute'
     || lk == 118440                // 'json' 'schema-element'
     || lk == 118952                // 'json' 'score'
     || lk == 119464                // 'json' 'select'
     || lk == 119976                // 'json' 'self'
     || lk == 122536                // 'json' 'sliding'
     || lk == 123048                // 'json' 'some'
     || lk == 123560                // 'json' 'stable'
     || lk == 124072                // 'json' 'start'
     || lk == 125608                // 'json' 'strict'
     || lk == 126632                // 'json' 'structured-item'
     || lk == 127144                // 'json' 'switch'
     || lk == 127656                // 'json' 'text'
     || lk == 129704                // 'json' 'to'
     || lk == 130216                // 'json' 'treat'
     || lk == 130728                // 'json' 'true'
     || lk == 131240                // 'json' 'try'
     || lk == 131752                // 'json' 'tumbling'
     || lk == 132264                // 'json' 'type'
     || lk == 132776                // 'json' 'typeswitch'
     || lk == 133288                // 'json' 'union'
     || lk == 134312                // 'json' 'unordered'
     || lk == 134824                // 'json' 'updating'
     || lk == 136360                // 'json' 'validate'
     || lk == 136872                // 'json' 'value'
     || lk == 137384                // 'json' 'variable'
     || lk == 137896                // 'json' 'version'
     || lk == 139432                // 'json' 'where'
     || lk == 139944                // 'json' 'while'
     || lk == 141480                // 'json' 'with'
     || lk == 144040                // 'json' '{'
     || lk == 145064)               // 'json' '{|'
    {
      shiftT(168);                  // 'json'
    }
    lookahead1W(260);               // EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral | StringLiteral |
    try_PostfixExpr();
    shiftT(80);                     // 'as'
    lookahead1W(267);               // EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral | StringLiteral |
    try_ExprSingle();
  }

  function parse_JSONReplaceExpr()
  {
    eventHandler.startNonterminal("JSONReplaceExpr", e0);
    shift(223);                     // 'replace'
    lookahead1W(85);                // S^WS | '(:' | 'value'
    shift(267);                     // 'value'
    lookahead1W(67);                // S^WS | '(:' | 'of'
    shift(200);                     // 'of'
    lookahead1W(59);                // S^WS | '(:' | 'json'
    shift(168);                     // 'json'
    lookahead1W(260);               // EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral | StringLiteral |
    whitespace();
    parse_PostfixExpr();
    shift(276);                     // 'with'
    lookahead1W(267);               // EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral | StringLiteral |
    whitespace();
    parse_ExprSingle();
    eventHandler.endNonterminal("JSONReplaceExpr", e0);
  }

  function try_JSONReplaceExpr()
  {
    shiftT(223);                    // 'replace'
    lookahead1W(85);                // S^WS | '(:' | 'value'
    shiftT(267);                    // 'value'
    lookahead1W(67);                // S^WS | '(:' | 'of'
    shiftT(200);                    // 'of'
    lookahead1W(59);                // S^WS | '(:' | 'json'
    shiftT(168);                    // 'json'
    lookahead1W(260);               // EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral | StringLiteral |
    try_PostfixExpr();
    shiftT(276);                    // 'with'
    lookahead1W(267);               // EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral | StringLiteral |
    try_ExprSingle();
  }

  function parse_JSONAppendExpr()
  {
    eventHandler.startNonterminal("JSONAppendExpr", e0);
    shift(78);                      // 'append'
    lookahead1W(267);               // EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral | StringLiteral |
    switch (l1)
    {
    case 168:                       // 'json'
      lookahead2W(269);             // EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral | StringLiteral |
      break;
    default:
      lk = l1;
    }
    if (lk == 18088)                // 'json' '('
    {
      lk = memoized(17, e0);
      if (lk == 0)
      {
        var b0A = b0; var e0A = e0; var l1A = l1;
        var b1A = b1; var e1A = e1; var l2A = l2;
        var b2A = b2; var e2A = e2;
        try
        {
          shiftT(168);              // 'json'
          lk = -1;
        }
        catch (p1A)
        {
          lk = -2;
        }
        b0 = b0A; e0 = e0A; l1 = l1A; if (l1 == 0) {end = e0A;} else {
        b1 = b1A; e1 = e1A; l2 = l2A; if (l2 == 0) {end = e1A;} else {
        b2 = b2A; e2 = e2A; end = e2A; }}
        memoize(17, e0, lk);
      }
    }
    if (lk == -1
     || lk == 3240                  // 'json' EQName^Token
     || lk == 4264                  // 'json' IntegerLiteral
     || lk == 4776                  // 'json' DecimalLiteral
     || lk == 5288                  // 'json' DoubleLiteral
     || lk == 5800                  // 'json' StringLiteral
     || lk == 16040                 // 'json' '$'
     || lk == 16552                 // 'json' '$$'
     || lk == 17064                 // 'json' '%'
     || lk == 18600                 // 'json' '(#'
     || lk == 21160                 // 'json' '+'
     || lk == 22184                 // 'json' '-'
     || lk == 24232                 // 'json' '/'
     || lk == 24744                 // 'json' '//'
     || lk == 28328                 // 'json' '<'
     || lk == 28840                 // 'json' '<!--'
     || lk == 30888                 // 'json' '<?'
     || lk == 35496                 // 'json' '['
     || lk == 36520                 // 'json' 'after'
     || lk == 37544                 // 'json' 'allowing'
     || lk == 38056                 // 'json' 'ancestor'
     || lk == 38568                 // 'json' 'ancestor-or-self'
     || lk == 39080                 // 'json' 'and'
     || lk == 40104                 // 'json' 'append'
     || lk == 40616                 // 'json' 'array'
     || lk == 41128                 // 'json' 'as'
     || lk == 41640                 // 'json' 'ascending'
     || lk == 42152                 // 'json' 'at'
     || lk == 42664                 // 'json' 'attribute'
     || lk == 43176                 // 'json' 'base-uri'
     || lk == 43688                 // 'json' 'before'
     || lk == 44200                 // 'json' 'boundary-space'
     || lk == 44712                 // 'json' 'break'
     || lk == 45736                 // 'json' 'case'
     || lk == 46248                 // 'json' 'cast'
     || lk == 46760                 // 'json' 'castable'
     || lk == 47272                 // 'json' 'catch'
     || lk == 48296                 // 'json' 'child'
     || lk == 48808                 // 'json' 'collation'
     || lk == 49832                 // 'json' 'comment'
     || lk == 50344                 // 'json' 'constraint'
     || lk == 50856                 // 'json' 'construction'
     || lk == 52392                 // 'json' 'context'
     || lk == 52904                 // 'json' 'continue'
     || lk == 53416                 // 'json' 'copy'
     || lk == 53928                 // 'json' 'copy-namespaces'
     || lk == 54440                 // 'json' 'count'
     || lk == 54952                 // 'json' 'decimal-format'
     || lk == 55976                 // 'json' 'declare'
     || lk == 56488                 // 'json' 'default'
     || lk == 57000                 // 'json' 'delete'
     || lk == 57512                 // 'json' 'descendant'
     || lk == 58024                 // 'json' 'descendant-or-self'
     || lk == 58536                 // 'json' 'descending'
     || lk == 61096                 // 'json' 'div'
     || lk == 61608                 // 'json' 'document'
     || lk == 62120                 // 'json' 'document-node'
     || lk == 62632                 // 'json' 'element'
     || lk == 63144                 // 'json' 'else'
     || lk == 63656                 // 'json' 'empty'
     || lk == 64168                 // 'json' 'empty-sequence'
     || lk == 64680                 // 'json' 'encoding'
     || lk == 65192                 // 'json' 'end'
     || lk == 66216                 // 'json' 'eq'
     || lk == 66728                 // 'json' 'every'
     || lk == 67752                 // 'json' 'except'
     || lk == 68264                 // 'json' 'exit'
     || lk == 68776                 // 'json' 'external'
     || lk == 69288                 // 'json' 'false'
     || lk == 69800                 // 'json' 'first'
     || lk == 70312                 // 'json' 'following'
     || lk == 70824                 // 'json' 'following-sibling'
     || lk == 71336                 // 'json' 'for'
     || lk == 72872                 // 'json' 'from'
     || lk == 73384                 // 'json' 'ft-option'
     || lk == 75432                 // 'json' 'function'
     || lk == 75944                 // 'json' 'ge'
     || lk == 76968                 // 'json' 'group'
     || lk == 77992                 // 'json' 'gt'
     || lk == 78504                 // 'json' 'idiv'
     || lk == 79016                 // 'json' 'if'
     || lk == 79528                 // 'json' 'import'
     || lk == 80040                 // 'json' 'in'
     || lk == 80552                 // 'json' 'index'
     || lk == 82600                 // 'json' 'insert'
     || lk == 83112                 // 'json' 'instance'
     || lk == 83624                 // 'json' 'integrity'
     || lk == 84136                 // 'json' 'intersect'
     || lk == 84648                 // 'json' 'into'
     || lk == 85160                 // 'json' 'is'
     || lk == 85672                 // 'json' 'item'
     || lk == 86184                 // 'json' 'json'
     || lk == 86696                 // 'json' 'json-item'
     || lk == 87208                 // 'json' 'jsoniq'
     || lk == 88744                 // 'json' 'last'
     || lk == 89256                 // 'json' 'lax'
     || lk == 89768                 // 'json' 'le'
     || lk == 90792                 // 'json' 'let'
     || lk == 91816                 // 'json' 'loop'
     || lk == 92840                 // 'json' 'lt'
     || lk == 93864                 // 'json' 'mod'
     || lk == 94376                 // 'json' 'modify'
     || lk == 94888                 // 'json' 'module'
     || lk == 95912                 // 'json' 'namespace'
     || lk == 96424                 // 'json' 'namespace-node'
     || lk == 96936                 // 'json' 'ne'
     || lk == 99496                 // 'json' 'node'
     || lk == 100008                // 'json' 'nodes'
     || lk == 100520                // 'json' 'not'
     || lk == 101032                // 'json' 'null'
     || lk == 101544                // 'json' 'object'
     || lk == 103592                // 'json' 'only'
     || lk == 104104                // 'json' 'option'
     || lk == 104616                // 'json' 'or'
     || lk == 105128                // 'json' 'order'
     || lk == 105640                // 'json' 'ordered'
     || lk == 106152                // 'json' 'ordering'
     || lk == 107688                // 'json' 'parent'
     || lk == 110760                // 'json' 'preceding'
     || lk == 111272                // 'json' 'preceding-sibling'
     || lk == 112808                // 'json' 'processing-instruction'
     || lk == 113832                // 'json' 'rename'
     || lk == 114344                // 'json' 'replace'
     || lk == 114856                // 'json' 'return'
     || lk == 115368                // 'json' 'returning'
     || lk == 115880                // 'json' 'revalidation'
     || lk == 116904                // 'json' 'satisfies'
     || lk == 117416                // 'json' 'schema'
     || lk == 117928                // 'json' 'schema-attribute'
     || lk == 118440                // 'json' 'schema-element'
     || lk == 118952                // 'json' 'score'
     || lk == 119464                // 'json' 'select'
     || lk == 119976                // 'json' 'self'
     || lk == 122536                // 'json' 'sliding'
     || lk == 123048                // 'json' 'some'
     || lk == 123560                // 'json' 'stable'
     || lk == 124072                // 'json' 'start'
     || lk == 125608                // 'json' 'strict'
     || lk == 126632                // 'json' 'structured-item'
     || lk == 127144                // 'json' 'switch'
     || lk == 127656                // 'json' 'text'
     || lk == 129704                // 'json' 'to'
     || lk == 130216                // 'json' 'treat'
     || lk == 130728                // 'json' 'true'
     || lk == 131240                // 'json' 'try'
     || lk == 131752                // 'json' 'tumbling'
     || lk == 132264                // 'json' 'type'
     || lk == 132776                // 'json' 'typeswitch'
     || lk == 133288                // 'json' 'union'
     || lk == 134312                // 'json' 'unordered'
     || lk == 134824                // 'json' 'updating'
     || lk == 136360                // 'json' 'validate'
     || lk == 136872                // 'json' 'value'
     || lk == 137384                // 'json' 'variable'
     || lk == 137896                // 'json' 'version'
     || lk == 139432                // 'json' 'where'
     || lk == 139944                // 'json' 'while'
     || lk == 141480                // 'json' 'with'
     || lk == 144040                // 'json' '{'
     || lk == 145064)               // 'json' '{|'
    {
      shift(168);                   // 'json'
    }
    lookahead1W(267);               // EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral | StringLiteral |
    whitespace();
    parse_ExprSingle();
    shift(165);                     // 'into'
    lookahead1W(267);               // EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral | StringLiteral |
    whitespace();
    parse_ExprSingle();
    eventHandler.endNonterminal("JSONAppendExpr", e0);
  }

  function try_JSONAppendExpr()
  {
    shiftT(78);                     // 'append'
    lookahead1W(267);               // EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral | StringLiteral |
    switch (l1)
    {
    case 168:                       // 'json'
      lookahead2W(269);             // EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral | StringLiteral |
      break;
    default:
      lk = l1;
    }
    if (lk == 18088)                // 'json' '('
    {
      lk = memoized(17, e0);
      if (lk == 0)
      {
        var b0A = b0; var e0A = e0; var l1A = l1;
        var b1A = b1; var e1A = e1; var l2A = l2;
        var b2A = b2; var e2A = e2;
        try
        {
          shiftT(168);              // 'json'
          memoize(17, e0A, -1);
        }
        catch (p1A)
        {
          b0 = b0A; e0 = e0A; l1 = l1A; if (l1 == 0) {end = e0A;} else {
          b1 = b1A; e1 = e1A; l2 = l2A; if (l2 == 0) {end = e1A;} else {
          b2 = b2A; e2 = e2A; end = e2A; }}
          memoize(17, e0A, -2);
        }
        lk = -2;
      }
    }
    if (lk == -1
     || lk == 3240                  // 'json' EQName^Token
     || lk == 4264                  // 'json' IntegerLiteral
     || lk == 4776                  // 'json' DecimalLiteral
     || lk == 5288                  // 'json' DoubleLiteral
     || lk == 5800                  // 'json' StringLiteral
     || lk == 16040                 // 'json' '$'
     || lk == 16552                 // 'json' '$$'
     || lk == 17064                 // 'json' '%'
     || lk == 18600                 // 'json' '(#'
     || lk == 21160                 // 'json' '+'
     || lk == 22184                 // 'json' '-'
     || lk == 24232                 // 'json' '/'
     || lk == 24744                 // 'json' '//'
     || lk == 28328                 // 'json' '<'
     || lk == 28840                 // 'json' '<!--'
     || lk == 30888                 // 'json' '<?'
     || lk == 35496                 // 'json' '['
     || lk == 36520                 // 'json' 'after'
     || lk == 37544                 // 'json' 'allowing'
     || lk == 38056                 // 'json' 'ancestor'
     || lk == 38568                 // 'json' 'ancestor-or-self'
     || lk == 39080                 // 'json' 'and'
     || lk == 40104                 // 'json' 'append'
     || lk == 40616                 // 'json' 'array'
     || lk == 41128                 // 'json' 'as'
     || lk == 41640                 // 'json' 'ascending'
     || lk == 42152                 // 'json' 'at'
     || lk == 42664                 // 'json' 'attribute'
     || lk == 43176                 // 'json' 'base-uri'
     || lk == 43688                 // 'json' 'before'
     || lk == 44200                 // 'json' 'boundary-space'
     || lk == 44712                 // 'json' 'break'
     || lk == 45736                 // 'json' 'case'
     || lk == 46248                 // 'json' 'cast'
     || lk == 46760                 // 'json' 'castable'
     || lk == 47272                 // 'json' 'catch'
     || lk == 48296                 // 'json' 'child'
     || lk == 48808                 // 'json' 'collation'
     || lk == 49832                 // 'json' 'comment'
     || lk == 50344                 // 'json' 'constraint'
     || lk == 50856                 // 'json' 'construction'
     || lk == 52392                 // 'json' 'context'
     || lk == 52904                 // 'json' 'continue'
     || lk == 53416                 // 'json' 'copy'
     || lk == 53928                 // 'json' 'copy-namespaces'
     || lk == 54440                 // 'json' 'count'
     || lk == 54952                 // 'json' 'decimal-format'
     || lk == 55976                 // 'json' 'declare'
     || lk == 56488                 // 'json' 'default'
     || lk == 57000                 // 'json' 'delete'
     || lk == 57512                 // 'json' 'descendant'
     || lk == 58024                 // 'json' 'descendant-or-self'
     || lk == 58536                 // 'json' 'descending'
     || lk == 61096                 // 'json' 'div'
     || lk == 61608                 // 'json' 'document'
     || lk == 62120                 // 'json' 'document-node'
     || lk == 62632                 // 'json' 'element'
     || lk == 63144                 // 'json' 'else'
     || lk == 63656                 // 'json' 'empty'
     || lk == 64168                 // 'json' 'empty-sequence'
     || lk == 64680                 // 'json' 'encoding'
     || lk == 65192                 // 'json' 'end'
     || lk == 66216                 // 'json' 'eq'
     || lk == 66728                 // 'json' 'every'
     || lk == 67752                 // 'json' 'except'
     || lk == 68264                 // 'json' 'exit'
     || lk == 68776                 // 'json' 'external'
     || lk == 69288                 // 'json' 'false'
     || lk == 69800                 // 'json' 'first'
     || lk == 70312                 // 'json' 'following'
     || lk == 70824                 // 'json' 'following-sibling'
     || lk == 71336                 // 'json' 'for'
     || lk == 72872                 // 'json' 'from'
     || lk == 73384                 // 'json' 'ft-option'
     || lk == 75432                 // 'json' 'function'
     || lk == 75944                 // 'json' 'ge'
     || lk == 76968                 // 'json' 'group'
     || lk == 77992                 // 'json' 'gt'
     || lk == 78504                 // 'json' 'idiv'
     || lk == 79016                 // 'json' 'if'
     || lk == 79528                 // 'json' 'import'
     || lk == 80040                 // 'json' 'in'
     || lk == 80552                 // 'json' 'index'
     || lk == 82600                 // 'json' 'insert'
     || lk == 83112                 // 'json' 'instance'
     || lk == 83624                 // 'json' 'integrity'
     || lk == 84136                 // 'json' 'intersect'
     || lk == 84648                 // 'json' 'into'
     || lk == 85160                 // 'json' 'is'
     || lk == 85672                 // 'json' 'item'
     || lk == 86184                 // 'json' 'json'
     || lk == 86696                 // 'json' 'json-item'
     || lk == 87208                 // 'json' 'jsoniq'
     || lk == 88744                 // 'json' 'last'
     || lk == 89256                 // 'json' 'lax'
     || lk == 89768                 // 'json' 'le'
     || lk == 90792                 // 'json' 'let'
     || lk == 91816                 // 'json' 'loop'
     || lk == 92840                 // 'json' 'lt'
     || lk == 93864                 // 'json' 'mod'
     || lk == 94376                 // 'json' 'modify'
     || lk == 94888                 // 'json' 'module'
     || lk == 95912                 // 'json' 'namespace'
     || lk == 96424                 // 'json' 'namespace-node'
     || lk == 96936                 // 'json' 'ne'
     || lk == 99496                 // 'json' 'node'
     || lk == 100008                // 'json' 'nodes'
     || lk == 100520                // 'json' 'not'
     || lk == 101032                // 'json' 'null'
     || lk == 101544                // 'json' 'object'
     || lk == 103592                // 'json' 'only'
     || lk == 104104                // 'json' 'option'
     || lk == 104616                // 'json' 'or'
     || lk == 105128                // 'json' 'order'
     || lk == 105640                // 'json' 'ordered'
     || lk == 106152                // 'json' 'ordering'
     || lk == 107688                // 'json' 'parent'
     || lk == 110760                // 'json' 'preceding'
     || lk == 111272                // 'json' 'preceding-sibling'
     || lk == 112808                // 'json' 'processing-instruction'
     || lk == 113832                // 'json' 'rename'
     || lk == 114344                // 'json' 'replace'
     || lk == 114856                // 'json' 'return'
     || lk == 115368                // 'json' 'returning'
     || lk == 115880                // 'json' 'revalidation'
     || lk == 116904                // 'json' 'satisfies'
     || lk == 117416                // 'json' 'schema'
     || lk == 117928                // 'json' 'schema-attribute'
     || lk == 118440                // 'json' 'schema-element'
     || lk == 118952                // 'json' 'score'
     || lk == 119464                // 'json' 'select'
     || lk == 119976                // 'json' 'self'
     || lk == 122536                // 'json' 'sliding'
     || lk == 123048                // 'json' 'some'
     || lk == 123560                // 'json' 'stable'
     || lk == 124072                // 'json' 'start'
     || lk == 125608                // 'json' 'strict'
     || lk == 126632                // 'json' 'structured-item'
     || lk == 127144                // 'json' 'switch'
     || lk == 127656                // 'json' 'text'
     || lk == 129704                // 'json' 'to'
     || lk == 130216                // 'json' 'treat'
     || lk == 130728                // 'json' 'true'
     || lk == 131240                // 'json' 'try'
     || lk == 131752                // 'json' 'tumbling'
     || lk == 132264                // 'json' 'type'
     || lk == 132776                // 'json' 'typeswitch'
     || lk == 133288                // 'json' 'union'
     || lk == 134312                // 'json' 'unordered'
     || lk == 134824                // 'json' 'updating'
     || lk == 136360                // 'json' 'validate'
     || lk == 136872                // 'json' 'value'
     || lk == 137384                // 'json' 'variable'
     || lk == 137896                // 'json' 'version'
     || lk == 139432                // 'json' 'where'
     || lk == 139944                // 'json' 'while'
     || lk == 141480                // 'json' 'with'
     || lk == 144040                // 'json' '{'
     || lk == 145064)               // 'json' '{|'
    {
      shiftT(168);                  // 'json'
    }
    lookahead1W(267);               // EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral | StringLiteral |
    try_ExprSingle();
    shiftT(165);                    // 'into'
    lookahead1W(267);               // EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral | StringLiteral |
    try_ExprSingle();
  }

  function parse_CommonContent()
  {
    eventHandler.startNonterminal("CommonContent", e0);
    switch (l1)
    {
    case 12:                        // PredefinedEntityRef
      shift(12);                    // PredefinedEntityRef
      break;
    case 23:                        // CharRef
      shift(23);                    // CharRef
      break;
    case 282:                       // '{{'
      shift(282);                   // '{{'
      break;
    case 288:                       // '}}'
      shift(288);                   // '}}'
      break;
    default:
      parse_BlockExpr();
    }
    eventHandler.endNonterminal("CommonContent", e0);
  }

  function try_CommonContent()
  {
    switch (l1)
    {
    case 12:                        // PredefinedEntityRef
      shiftT(12);                   // PredefinedEntityRef
      break;
    case 23:                        // CharRef
      shiftT(23);                   // CharRef
      break;
    case 282:                       // '{{'
      shiftT(282);                  // '{{'
      break;
    case 288:                       // '}}'
      shiftT(288);                  // '}}'
      break;
    default:
      try_BlockExpr();
    }
  }

  function parse_ContentExpr()
  {
    eventHandler.startNonterminal("ContentExpr", e0);
    parse_StatementsAndExpr();
    eventHandler.endNonterminal("ContentExpr", e0);
  }

  function try_ContentExpr()
  {
    try_StatementsAndExpr();
  }

  function parse_CompDocConstructor()
  {
    eventHandler.startNonterminal("CompDocConstructor", e0);
    shift(120);                     // 'document'
    lookahead1W(90);                // S^WS | '(:' | '{'
    whitespace();
    parse_BlockExpr();
    eventHandler.endNonterminal("CompDocConstructor", e0);
  }

  function try_CompDocConstructor()
  {
    shiftT(120);                    // 'document'
    lookahead1W(90);                // S^WS | '(:' | '{'
    try_BlockExpr();
  }

  function parse_CompAttrConstructor()
  {
    eventHandler.startNonterminal("CompAttrConstructor", e0);
    shift(83);                      // 'attribute'
    lookahead1W(250);               // EQName^Token | S^WS | '(:' | 'after' | 'allowing' | 'ancestor' |
    switch (l1)
    {
    case 281:                       // '{'
      shift(281);                   // '{'
      lookahead1W(267);             // EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral | StringLiteral |
      whitespace();
      parse_Expr();
      shift(287);                   // '}'
      break;
    default:
      whitespace();
      parse_EQName();
    }
    lookahead1W(90);                // S^WS | '(:' | '{'
    switch (l1)
    {
    case 281:                       // '{'
      lookahead2W(281);             // EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral | StringLiteral |
      break;
    default:
      lk = l1;
    }
    if (lk == 147225)               // '{' '}'
    {
      lk = memoized(18, e0);
      if (lk == 0)
      {
        var b0A = b0; var e0A = e0; var l1A = l1;
        var b1A = b1; var e1A = e1; var l2A = l2;
        var b2A = b2; var e2A = e2;
        try
        {
          shiftT(281);              // '{'
          lookahead1W(91);          // S^WS | '(:' | '}'
          shiftT(287);              // '}'
          lk = -1;
        }
        catch (p1A)
        {
          lk = -2;
        }
        b0 = b0A; e0 = e0A; l1 = l1A; if (l1 == 0) {end = e0A;} else {
        b1 = b1A; e1 = e1A; l2 = l2A; if (l2 == 0) {end = e1A;} else {
        b2 = b2A; e2 = e2A; end = e2A; }}
        memoize(18, e0, lk);
      }
    }
    switch (lk)
    {
    case -1:
      shift(281);                   // '{'
      lookahead1W(91);              // S^WS | '(:' | '}'
      shift(287);                   // '}'
      break;
    default:
      whitespace();
      parse_BlockExpr();
    }
    eventHandler.endNonterminal("CompAttrConstructor", e0);
  }

  function try_CompAttrConstructor()
  {
    shiftT(83);                     // 'attribute'
    lookahead1W(250);               // EQName^Token | S^WS | '(:' | 'after' | 'allowing' | 'ancestor' |
    switch (l1)
    {
    case 281:                       // '{'
      shiftT(281);                  // '{'
      lookahead1W(267);             // EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral | StringLiteral |
      try_Expr();
      shiftT(287);                  // '}'
      break;
    default:
      try_EQName();
    }
    lookahead1W(90);                // S^WS | '(:' | '{'
    switch (l1)
    {
    case 281:                       // '{'
      lookahead2W(281);             // EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral | StringLiteral |
      break;
    default:
      lk = l1;
    }
    if (lk == 147225)               // '{' '}'
    {
      lk = memoized(18, e0);
      if (lk == 0)
      {
        var b0A = b0; var e0A = e0; var l1A = l1;
        var b1A = b1; var e1A = e1; var l2A = l2;
        var b2A = b2; var e2A = e2;
        try
        {
          shiftT(281);              // '{'
          lookahead1W(91);          // S^WS | '(:' | '}'
          shiftT(287);              // '}'
          memoize(18, e0A, -1);
          lk = -3;
        }
        catch (p1A)
        {
          lk = -2;
          b0 = b0A; e0 = e0A; l1 = l1A; if (l1 == 0) {end = e0A;} else {
          b1 = b1A; e1 = e1A; l2 = l2A; if (l2 == 0) {end = e1A;} else {
          b2 = b2A; e2 = e2A; end = e2A; }}
          memoize(18, e0A, -2);
        }
      }
    }
    switch (lk)
    {
    case -1:
      shiftT(281);                  // '{'
      lookahead1W(91);              // S^WS | '(:' | '}'
      shiftT(287);                  // '}'
      break;
    case -3:
      break;
    default:
      try_BlockExpr();
    }
  }

  function parse_CompPIConstructor()
  {
    eventHandler.startNonterminal("CompPIConstructor", e0);
    shift(220);                     // 'processing-instruction'
    lookahead1W(242);               // NCName^Token | S^WS | '(:' | 'after' | 'allowing' | 'ancestor' |
    switch (l1)
    {
    case 281:                       // '{'
      shift(281);                   // '{'
      lookahead1W(267);             // EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral | StringLiteral |
      whitespace();
      parse_Expr();
      shift(287);                   // '}'
      break;
    default:
      whitespace();
      parse_NCName();
    }
    lookahead1W(90);                // S^WS | '(:' | '{'
    switch (l1)
    {
    case 281:                       // '{'
      lookahead2W(281);             // EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral | StringLiteral |
      break;
    default:
      lk = l1;
    }
    if (lk == 147225)               // '{' '}'
    {
      lk = memoized(19, e0);
      if (lk == 0)
      {
        var b0A = b0; var e0A = e0; var l1A = l1;
        var b1A = b1; var e1A = e1; var l2A = l2;
        var b2A = b2; var e2A = e2;
        try
        {
          shiftT(281);              // '{'
          lookahead1W(91);          // S^WS | '(:' | '}'
          shiftT(287);              // '}'
          lk = -1;
        }
        catch (p1A)
        {
          lk = -2;
        }
        b0 = b0A; e0 = e0A; l1 = l1A; if (l1 == 0) {end = e0A;} else {
        b1 = b1A; e1 = e1A; l2 = l2A; if (l2 == 0) {end = e1A;} else {
        b2 = b2A; e2 = e2A; end = e2A; }}
        memoize(19, e0, lk);
      }
    }
    switch (lk)
    {
    case -1:
      shift(281);                   // '{'
      lookahead1W(91);              // S^WS | '(:' | '}'
      shift(287);                   // '}'
      break;
    default:
      whitespace();
      parse_BlockExpr();
    }
    eventHandler.endNonterminal("CompPIConstructor", e0);
  }

  function try_CompPIConstructor()
  {
    shiftT(220);                    // 'processing-instruction'
    lookahead1W(242);               // NCName^Token | S^WS | '(:' | 'after' | 'allowing' | 'ancestor' |
    switch (l1)
    {
    case 281:                       // '{'
      shiftT(281);                  // '{'
      lookahead1W(267);             // EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral | StringLiteral |
      try_Expr();
      shiftT(287);                  // '}'
      break;
    default:
      try_NCName();
    }
    lookahead1W(90);                // S^WS | '(:' | '{'
    switch (l1)
    {
    case 281:                       // '{'
      lookahead2W(281);             // EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral | StringLiteral |
      break;
    default:
      lk = l1;
    }
    if (lk == 147225)               // '{' '}'
    {
      lk = memoized(19, e0);
      if (lk == 0)
      {
        var b0A = b0; var e0A = e0; var l1A = l1;
        var b1A = b1; var e1A = e1; var l2A = l2;
        var b2A = b2; var e2A = e2;
        try
        {
          shiftT(281);              // '{'
          lookahead1W(91);          // S^WS | '(:' | '}'
          shiftT(287);              // '}'
          memoize(19, e0A, -1);
          lk = -3;
        }
        catch (p1A)
        {
          lk = -2;
          b0 = b0A; e0 = e0A; l1 = l1A; if (l1 == 0) {end = e0A;} else {
          b1 = b1A; e1 = e1A; l2 = l2A; if (l2 == 0) {end = e1A;} else {
          b2 = b2A; e2 = e2A; end = e2A; }}
          memoize(19, e0A, -2);
        }
      }
    }
    switch (lk)
    {
    case -1:
      shiftT(281);                  // '{'
      lookahead1W(91);              // S^WS | '(:' | '}'
      shiftT(287);                  // '}'
      break;
    case -3:
      break;
    default:
      try_BlockExpr();
    }
  }

  function parse_CompCommentConstructor()
  {
    eventHandler.startNonterminal("CompCommentConstructor", e0);
    shift(97);                      // 'comment'
    lookahead1W(90);                // S^WS | '(:' | '{'
    whitespace();
    parse_BlockExpr();
    eventHandler.endNonterminal("CompCommentConstructor", e0);
  }

  function try_CompCommentConstructor()
  {
    shiftT(97);                     // 'comment'
    lookahead1W(90);                // S^WS | '(:' | '{'
    try_BlockExpr();
  }

  function parse_CompTextConstructor()
  {
    eventHandler.startNonterminal("CompTextConstructor", e0);
    shift(249);                     // 'text'
    lookahead1W(90);                // S^WS | '(:' | '{'
    whitespace();
    parse_BlockExpr();
    eventHandler.endNonterminal("CompTextConstructor", e0);
  }

  function try_CompTextConstructor()
  {
    shiftT(249);                    // 'text'
    lookahead1W(90);                // S^WS | '(:' | '{'
    try_BlockExpr();
  }

  function parse_PrimaryExpr()
  {
    eventHandler.startNonterminal("PrimaryExpr", e0);
    switch (l1)
    {
    case 187:                       // 'namespace'
      lookahead2W(247);             // NCName^Token | S^WS | '#' | '(' | '(:' | 'after' | 'allowing' | 'ancestor' |
      break;
    case 220:                       // 'processing-instruction'
      lookahead2W(245);             // NCName^Token | S^WS | '#' | '(:' | 'after' | 'allowing' | 'ancestor' |
      break;
    case 281:                       // '{'
      lookahead2W(283);             // EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral | StringLiteral |
      break;
    case 83:                        // 'attribute'
    case 122:                       // 'element'
      lookahead2W(253);             // EQName^Token | S^WS | '#' | '(:' | 'after' | 'allowing' | 'ancestor' |
      break;
    case 97:                        // 'comment'
    case 249:                       // 'text'
      lookahead2W(97);              // S^WS | '#' | '(:' | '{'
      break;
    case 120:                       // 'document'
    case 206:                       // 'ordered'
    case 262:                       // 'unordered'
      lookahead2W(148);             // S^WS | '#' | '(' | '(:' | '{'
      break;
    case 135:                       // 'false'
    case 197:                       // 'null'
    case 255:                       // 'true'
      lookahead2W(237);             // S^WS | EOF | '!' | '!=' | '#' | '(' | '(:' | ')' | '*' | '+' | ',' | '-' | '.' |
      break;
    case 6:                         // EQName^Token
    case 71:                        // 'after'
    case 73:                        // 'allowing'
    case 74:                        // 'ancestor'
    case 75:                        // 'ancestor-or-self'
    case 76:                        // 'and'
    case 78:                        // 'append'
    case 80:                        // 'as'
    case 81:                        // 'ascending'
    case 82:                        // 'at'
    case 84:                        // 'base-uri'
    case 85:                        // 'before'
    case 86:                        // 'boundary-space'
    case 87:                        // 'break'
    case 89:                        // 'case'
    case 90:                        // 'cast'
    case 91:                        // 'castable'
    case 92:                        // 'catch'
    case 94:                        // 'child'
    case 95:                        // 'collation'
    case 98:                        // 'constraint'
    case 99:                        // 'construction'
    case 102:                       // 'context'
    case 103:                       // 'continue'
    case 104:                       // 'copy'
    case 105:                       // 'copy-namespaces'
    case 106:                       // 'count'
    case 107:                       // 'decimal-format'
    case 109:                       // 'declare'
    case 110:                       // 'default'
    case 111:                       // 'delete'
    case 112:                       // 'descendant'
    case 113:                       // 'descendant-or-self'
    case 114:                       // 'descending'
    case 119:                       // 'div'
    case 123:                       // 'else'
    case 124:                       // 'empty'
    case 126:                       // 'encoding'
    case 127:                       // 'end'
    case 129:                       // 'eq'
    case 130:                       // 'every'
    case 132:                       // 'except'
    case 133:                       // 'exit'
    case 134:                       // 'external'
    case 136:                       // 'first'
    case 137:                       // 'following'
    case 138:                       // 'following-sibling'
    case 139:                       // 'for'
    case 142:                       // 'from'
    case 143:                       // 'ft-option'
    case 148:                       // 'ge'
    case 150:                       // 'group'
    case 152:                       // 'gt'
    case 153:                       // 'idiv'
    case 155:                       // 'import'
    case 156:                       // 'in'
    case 157:                       // 'index'
    case 161:                       // 'insert'
    case 162:                       // 'instance'
    case 163:                       // 'integrity'
    case 164:                       // 'intersect'
    case 165:                       // 'into'
    case 166:                       // 'is'
    case 168:                       // 'json'
    case 170:                       // 'jsoniq'
    case 173:                       // 'last'
    case 174:                       // 'lax'
    case 175:                       // 'le'
    case 177:                       // 'let'
    case 179:                       // 'loop'
    case 181:                       // 'lt'
    case 183:                       // 'mod'
    case 184:                       // 'modify'
    case 185:                       // 'module'
    case 189:                       // 'ne'
    case 195:                       // 'nodes'
    case 198:                       // 'object'
    case 202:                       // 'only'
    case 203:                       // 'option'
    case 204:                       // 'or'
    case 205:                       // 'order'
    case 207:                       // 'ordering'
    case 210:                       // 'parent'
    case 216:                       // 'preceding'
    case 217:                       // 'preceding-sibling'
    case 222:                       // 'rename'
    case 223:                       // 'replace'
    case 224:                       // 'return'
    case 225:                       // 'returning'
    case 226:                       // 'revalidation'
    case 228:                       // 'satisfies'
    case 229:                       // 'schema'
    case 232:                       // 'score'
    case 233:                       // 'select'
    case 234:                       // 'self'
    case 239:                       // 'sliding'
    case 240:                       // 'some'
    case 241:                       // 'stable'
    case 242:                       // 'start'
    case 245:                       // 'strict'
    case 253:                       // 'to'
    case 254:                       // 'treat'
    case 256:                       // 'try'
    case 257:                       // 'tumbling'
    case 258:                       // 'type'
    case 260:                       // 'union'
    case 263:                       // 'updating'
    case 266:                       // 'validate'
    case 267:                       // 'value'
    case 268:                       // 'variable'
    case 269:                       // 'version'
    case 272:                       // 'where'
    case 273:                       // 'while'
    case 276:                       // 'with'
      lookahead2W(95);              // S^WS | '#' | '(' | '(:'
      break;
    default:
      lk = l1;
    }
    if (lk == 3353                  // '{' EQName^Token
     || lk == 4377                  // '{' IntegerLiteral
     || lk == 4889                  // '{' DecimalLiteral
     || lk == 5401                  // '{' DoubleLiteral
     || lk == 5913                  // '{' StringLiteral
     || lk == 16153                 // '{' '$'
     || lk == 16665                 // '{' '$$'
     || lk == 17177                 // '{' '%'
     || lk == 18055                 // 'false' '('
     || lk == 18117                 // 'null' '('
     || lk == 18175                 // 'true' '('
     || lk == 18201                 // '{' '('
     || lk == 18713                 // '{' '(#'
     || lk == 21273                 // '{' '+'
     || lk == 22297                 // '{' '-'
     || lk == 24345                 // '{' '/'
     || lk == 24857                 // '{' '//'
     || lk == 28441                 // '{' '<'
     || lk == 28953                 // '{' '<!--'
     || lk == 31001                 // '{' '<?'
     || lk == 35609                 // '{' '['
     || lk == 36633                 // '{' 'after'
     || lk == 37657                 // '{' 'allowing'
     || lk == 38169                 // '{' 'ancestor'
     || lk == 38681                 // '{' 'ancestor-or-self'
     || lk == 39193                 // '{' 'and'
     || lk == 40217                 // '{' 'append'
     || lk == 40729                 // '{' 'array'
     || lk == 41241                 // '{' 'as'
     || lk == 41753                 // '{' 'ascending'
     || lk == 42265                 // '{' 'at'
     || lk == 42777                 // '{' 'attribute'
     || lk == 43289                 // '{' 'base-uri'
     || lk == 43801                 // '{' 'before'
     || lk == 44313                 // '{' 'boundary-space'
     || lk == 44825                 // '{' 'break'
     || lk == 45849                 // '{' 'case'
     || lk == 46361                 // '{' 'cast'
     || lk == 46873                 // '{' 'castable'
     || lk == 47385                 // '{' 'catch'
     || lk == 48409                 // '{' 'child'
     || lk == 48921                 // '{' 'collation'
     || lk == 49945                 // '{' 'comment'
     || lk == 50457                 // '{' 'constraint'
     || lk == 50969                 // '{' 'construction'
     || lk == 52505                 // '{' 'context'
     || lk == 53017                 // '{' 'continue'
     || lk == 53529                 // '{' 'copy'
     || lk == 54041                 // '{' 'copy-namespaces'
     || lk == 54553                 // '{' 'count'
     || lk == 55065                 // '{' 'decimal-format'
     || lk == 56089                 // '{' 'declare'
     || lk == 56601                 // '{' 'default'
     || lk == 57113                 // '{' 'delete'
     || lk == 57625                 // '{' 'descendant'
     || lk == 58137                 // '{' 'descendant-or-self'
     || lk == 58649                 // '{' 'descending'
     || lk == 61209                 // '{' 'div'
     || lk == 61721                 // '{' 'document'
     || lk == 62233                 // '{' 'document-node'
     || lk == 62745                 // '{' 'element'
     || lk == 63257                 // '{' 'else'
     || lk == 63769                 // '{' 'empty'
     || lk == 64281                 // '{' 'empty-sequence'
     || lk == 64793                 // '{' 'encoding'
     || lk == 65305                 // '{' 'end'
     || lk == 66329                 // '{' 'eq'
     || lk == 66841                 // '{' 'every'
     || lk == 67865                 // '{' 'except'
     || lk == 68377                 // '{' 'exit'
     || lk == 68889                 // '{' 'external'
     || lk == 69401                 // '{' 'false'
     || lk == 69913                 // '{' 'first'
     || lk == 70425                 // '{' 'following'
     || lk == 70937                 // '{' 'following-sibling'
     || lk == 71449                 // '{' 'for'
     || lk == 72985                 // '{' 'from'
     || lk == 73497                 // '{' 'ft-option'
     || lk == 75545                 // '{' 'function'
     || lk == 76057                 // '{' 'ge'
     || lk == 77081                 // '{' 'group'
     || lk == 78105                 // '{' 'gt'
     || lk == 78617                 // '{' 'idiv'
     || lk == 79129                 // '{' 'if'
     || lk == 79641                 // '{' 'import'
     || lk == 80153                 // '{' 'in'
     || lk == 80665                 // '{' 'index'
     || lk == 82713                 // '{' 'insert'
     || lk == 83225                 // '{' 'instance'
     || lk == 83737                 // '{' 'integrity'
     || lk == 84249                 // '{' 'intersect'
     || lk == 84761                 // '{' 'into'
     || lk == 85273                 // '{' 'is'
     || lk == 85785                 // '{' 'item'
     || lk == 86297                 // '{' 'json'
     || lk == 86809                 // '{' 'json-item'
     || lk == 87321                 // '{' 'jsoniq'
     || lk == 88857                 // '{' 'last'
     || lk == 89369                 // '{' 'lax'
     || lk == 89881                 // '{' 'le'
     || lk == 90905                 // '{' 'let'
     || lk == 91929                 // '{' 'loop'
     || lk == 92953                 // '{' 'lt'
     || lk == 93977                 // '{' 'mod'
     || lk == 94489                 // '{' 'modify'
     || lk == 95001                 // '{' 'module'
     || lk == 96025                 // '{' 'namespace'
     || lk == 96537                 // '{' 'namespace-node'
     || lk == 97049                 // '{' 'ne'
     || lk == 99609                 // '{' 'node'
     || lk == 100121                // '{' 'nodes'
     || lk == 100633                // '{' 'not'
     || lk == 101145                // '{' 'null'
     || lk == 101657                // '{' 'object'
     || lk == 103705                // '{' 'only'
     || lk == 104217                // '{' 'option'
     || lk == 104729                // '{' 'or'
     || lk == 105241                // '{' 'order'
     || lk == 105753                // '{' 'ordered'
     || lk == 106265                // '{' 'ordering'
     || lk == 107801                // '{' 'parent'
     || lk == 110873                // '{' 'preceding'
     || lk == 111385                // '{' 'preceding-sibling'
     || lk == 112921                // '{' 'processing-instruction'
     || lk == 113945                // '{' 'rename'
     || lk == 114457                // '{' 'replace'
     || lk == 114969                // '{' 'return'
     || lk == 115481                // '{' 'returning'
     || lk == 115993                // '{' 'revalidation'
     || lk == 117017                // '{' 'satisfies'
     || lk == 117529                // '{' 'schema'
     || lk == 118041                // '{' 'schema-attribute'
     || lk == 118553                // '{' 'schema-element'
     || lk == 119065                // '{' 'score'
     || lk == 119577                // '{' 'select'
     || lk == 120089                // '{' 'self'
     || lk == 122649                // '{' 'sliding'
     || lk == 123161                // '{' 'some'
     || lk == 123673                // '{' 'stable'
     || lk == 124185                // '{' 'start'
     || lk == 125721                // '{' 'strict'
     || lk == 126745                // '{' 'structured-item'
     || lk == 127257                // '{' 'switch'
     || lk == 127769                // '{' 'text'
     || lk == 129817                // '{' 'to'
     || lk == 130329                // '{' 'treat'
     || lk == 130841                // '{' 'true'
     || lk == 131353                // '{' 'try'
     || lk == 131865                // '{' 'tumbling'
     || lk == 132377                // '{' 'type'
     || lk == 132889                // '{' 'typeswitch'
     || lk == 133401                // '{' 'union'
     || lk == 134425                // '{' 'unordered'
     || lk == 134937                // '{' 'updating'
     || lk == 136473                // '{' 'validate'
     || lk == 136985                // '{' 'value'
     || lk == 137497                // '{' 'variable'
     || lk == 138009                // '{' 'version'
     || lk == 139545                // '{' 'where'
     || lk == 140057                // '{' 'while'
     || lk == 141593                // '{' 'with'
     || lk == 144153                // '{' '{'
     || lk == 145177                // '{' '{|'
     || lk == 147225)               // '{' '}'
    {
      lk = memoized(20, e0);
      if (lk == 0)
      {
        var b0A = b0; var e0A = e0; var l1A = l1;
        var b1A = b1; var e1A = e1; var l2A = l2;
        var b2A = b2; var e2A = e2;
        try
        {
          try_Literal();
          lk = -1;
        }
        catch (p1A)
        {
          try
          {
            b0 = b0A; e0 = e0A; l1 = l1A; if (l1 == 0) {end = e0A;} else {
            b1 = b1A; e1 = e1A; l2 = l2A; if (l2 == 0) {end = e1A;} else {
            b2 = b2A; e2 = e2A; end = e2A; }}
            try_FunctionCall();
            lk = -5;
          }
          catch (p5A)
          {
            try
            {
              b0 = b0A; e0 = e0A; l1 = l1A; if (l1 == 0) {end = e0A;} else {
              b1 = b1A; e1 = e1A; l2 = l2A; if (l2 == 0) {end = e1A;} else {
              b2 = b2A; e2 = e2A; end = e2A; }}
              try_BlockExpr();
              lk = -10;
            }
            catch (p10A)
            {
              lk = -11;
            }
          }
        }
        b0 = b0A; e0 = e0A; l1 = l1A; if (l1 == 0) {end = e0A;} else {
        b1 = b1A; e1 = e1A; l2 = l2A; if (l2 == 0) {end = e1A;} else {
        b2 = b2A; e2 = e2A; end = e2A; }}
        memoize(20, e0, lk);
      }
    }
    switch (lk)
    {
    case -1:
    case 8:                         // IntegerLiteral
    case 9:                         // DecimalLiteral
    case 10:                        // DoubleLiteral
    case 11:                        // StringLiteral
    case 12935:                     // 'false' EOF
    case 12997:                     // 'null' EOF
    case 13055:                     // 'true' EOF
    case 13447:                     // 'false' '!'
    case 13509:                     // 'null' '!'
    case 13567:                     // 'true' '!'
    case 13959:                     // 'false' '!='
    case 14021:                     // 'null' '!='
    case 14079:                     // 'true' '!='
    case 19591:                     // 'false' ')'
    case 19653:                     // 'null' ')'
    case 19711:                     // 'true' ')'
    case 20103:                     // 'false' '*'
    case 20165:                     // 'null' '*'
    case 20223:                     // 'true' '*'
    case 21127:                     // 'false' '+'
    case 21189:                     // 'null' '+'
    case 21247:                     // 'true' '+'
    case 21639:                     // 'false' ','
    case 21701:                     // 'null' ','
    case 21759:                     // 'true' ','
    case 22151:                     // 'false' '-'
    case 22213:                     // 'null' '-'
    case 22271:                     // 'true' '-'
    case 23175:                     // 'false' '.'
    case 23237:                     // 'null' '.'
    case 23295:                     // 'true' '.'
    case 24199:                     // 'false' '/'
    case 24261:                     // 'null' '/'
    case 24319:                     // 'true' '/'
    case 24711:                     // 'false' '//'
    case 24773:                     // 'null' '//'
    case 24831:                     // 'true' '//'
    case 25735:                     // 'false' ':'
    case 25797:                     // 'null' ':'
    case 25855:                     // 'true' ':'
    case 27783:                     // 'false' ';'
    case 27845:                     // 'null' ';'
    case 27903:                     // 'true' ';'
    case 28295:                     // 'false' '<'
    case 28357:                     // 'null' '<'
    case 28415:                     // 'true' '<'
    case 29831:                     // 'false' '<<'
    case 29893:                     // 'null' '<<'
    case 29951:                     // 'true' '<<'
    case 30343:                     // 'false' '<='
    case 30405:                     // 'null' '<='
    case 30463:                     // 'true' '<='
    case 31367:                     // 'false' '='
    case 31429:                     // 'null' '='
    case 31487:                     // 'true' '='
    case 31879:                     // 'false' '>'
    case 31941:                     // 'null' '>'
    case 31999:                     // 'true' '>'
    case 32391:                     // 'false' '>='
    case 32453:                     // 'null' '>='
    case 32511:                     // 'true' '>='
    case 32903:                     // 'false' '>>'
    case 32965:                     // 'null' '>>'
    case 33023:                     // 'true' '>>'
    case 35463:                     // 'false' '['
    case 35525:                     // 'null' '['
    case 35583:                     // 'true' '['
    case 35975:                     // 'false' ']'
    case 36037:                     // 'null' ']'
    case 36095:                     // 'true' ']'
    case 36487:                     // 'false' 'after'
    case 36549:                     // 'null' 'after'
    case 36607:                     // 'true' 'after'
    case 39047:                     // 'false' 'and'
    case 39109:                     // 'null' 'and'
    case 39167:                     // 'true' 'and'
    case 41095:                     // 'false' 'as'
    case 41157:                     // 'null' 'as'
    case 41215:                     // 'true' 'as'
    case 41607:                     // 'false' 'ascending'
    case 41669:                     // 'null' 'ascending'
    case 41727:                     // 'true' 'ascending'
    case 42119:                     // 'false' 'at'
    case 42181:                     // 'null' 'at'
    case 42239:                     // 'true' 'at'
    case 43655:                     // 'false' 'before'
    case 43717:                     // 'null' 'before'
    case 43775:                     // 'true' 'before'
    case 45191:                     // 'false' 'by'
    case 45253:                     // 'null' 'by'
    case 45311:                     // 'true' 'by'
    case 45703:                     // 'false' 'case'
    case 45765:                     // 'null' 'case'
    case 45823:                     // 'true' 'case'
    case 46215:                     // 'false' 'cast'
    case 46277:                     // 'null' 'cast'
    case 46335:                     // 'true' 'cast'
    case 46727:                     // 'false' 'castable'
    case 46789:                     // 'null' 'castable'
    case 46847:                     // 'true' 'castable'
    case 48775:                     // 'false' 'collation'
    case 48837:                     // 'null' 'collation'
    case 48895:                     // 'true' 'collation'
    case 51335:                     // 'false' 'contains'
    case 51397:                     // 'null' 'contains'
    case 51455:                     // 'true' 'contains'
    case 54407:                     // 'false' 'count'
    case 54469:                     // 'null' 'count'
    case 54527:                     // 'true' 'count'
    case 56455:                     // 'false' 'default'
    case 56517:                     // 'null' 'default'
    case 56575:                     // 'true' 'default'
    case 58503:                     // 'false' 'descending'
    case 58565:                     // 'null' 'descending'
    case 58623:                     // 'true' 'descending'
    case 61063:                     // 'false' 'div'
    case 61125:                     // 'null' 'div'
    case 61183:                     // 'true' 'div'
    case 63111:                     // 'false' 'else'
    case 63173:                     // 'null' 'else'
    case 63231:                     // 'true' 'else'
    case 63623:                     // 'false' 'empty'
    case 63685:                     // 'null' 'empty'
    case 63743:                     // 'true' 'empty'
    case 65159:                     // 'false' 'end'
    case 65221:                     // 'null' 'end'
    case 65279:                     // 'true' 'end'
    case 66183:                     // 'false' 'eq'
    case 66245:                     // 'null' 'eq'
    case 66303:                     // 'true' 'eq'
    case 67719:                     // 'false' 'except'
    case 67781:                     // 'null' 'except'
    case 67839:                     // 'true' 'except'
    case 71303:                     // 'false' 'for'
    case 71365:                     // 'null' 'for'
    case 71423:                     // 'true' 'for'
    case 75911:                     // 'false' 'ge'
    case 75973:                     // 'null' 'ge'
    case 76031:                     // 'true' 'ge'
    case 76935:                     // 'false' 'group'
    case 76997:                     // 'null' 'group'
    case 77055:                     // 'true' 'group'
    case 77959:                     // 'false' 'gt'
    case 78021:                     // 'null' 'gt'
    case 78079:                     // 'true' 'gt'
    case 78471:                     // 'false' 'idiv'
    case 78533:                     // 'null' 'idiv'
    case 78591:                     // 'true' 'idiv'
    case 83079:                     // 'false' 'instance'
    case 83141:                     // 'null' 'instance'
    case 83199:                     // 'true' 'instance'
    case 84103:                     // 'false' 'intersect'
    case 84165:                     // 'null' 'intersect'
    case 84223:                     // 'true' 'intersect'
    case 84615:                     // 'false' 'into'
    case 84677:                     // 'null' 'into'
    case 84735:                     // 'true' 'into'
    case 85127:                     // 'false' 'is'
    case 85189:                     // 'null' 'is'
    case 85247:                     // 'true' 'is'
    case 89735:                     // 'false' 'le'
    case 89797:                     // 'null' 'le'
    case 89855:                     // 'true' 'le'
    case 90759:                     // 'false' 'let'
    case 90821:                     // 'null' 'let'
    case 90879:                     // 'true' 'let'
    case 92807:                     // 'false' 'lt'
    case 92869:                     // 'null' 'lt'
    case 92927:                     // 'true' 'lt'
    case 93831:                     // 'false' 'mod'
    case 93893:                     // 'null' 'mod'
    case 93951:                     // 'true' 'mod'
    case 94343:                     // 'false' 'modify'
    case 94405:                     // 'null' 'modify'
    case 94463:                     // 'true' 'modify'
    case 96903:                     // 'false' 'ne'
    case 96965:                     // 'null' 'ne'
    case 97023:                     // 'true' 'ne'
    case 103559:                    // 'false' 'only'
    case 103621:                    // 'null' 'only'
    case 103679:                    // 'true' 'only'
    case 104583:                    // 'false' 'or'
    case 104645:                    // 'null' 'or'
    case 104703:                    // 'true' 'or'
    case 105095:                    // 'false' 'order'
    case 105157:                    // 'null' 'order'
    case 105215:                    // 'true' 'order'
    case 107143:                    // 'false' 'paragraphs'
    case 107205:                    // 'null' 'paragraphs'
    case 107263:                    // 'true' 'paragraphs'
    case 114823:                    // 'false' 'return'
    case 114885:                    // 'null' 'return'
    case 114943:                    // 'true' 'return'
    case 116871:                    // 'false' 'satisfies'
    case 116933:                    // 'null' 'satisfies'
    case 116991:                    // 'true' 'satisfies'
    case 121479:                    // 'false' 'sentences'
    case 121541:                    // 'null' 'sentences'
    case 121599:                    // 'true' 'sentences'
    case 123527:                    // 'false' 'stable'
    case 123589:                    // 'null' 'stable'
    case 123647:                    // 'true' 'stable'
    case 124039:                    // 'false' 'start'
    case 124101:                    // 'null' 'start'
    case 124159:                    // 'true' 'start'
    case 129159:                    // 'false' 'times'
    case 129221:                    // 'null' 'times'
    case 129279:                    // 'true' 'times'
    case 129671:                    // 'false' 'to'
    case 129733:                    // 'null' 'to'
    case 129791:                    // 'true' 'to'
    case 130183:                    // 'false' 'treat'
    case 130245:                    // 'null' 'treat'
    case 130303:                    // 'true' 'treat'
    case 133255:                    // 'false' 'union'
    case 133317:                    // 'null' 'union'
    case 133375:                    // 'true' 'union'
    case 139399:                    // 'false' 'where'
    case 139461:                    // 'null' 'where'
    case 139519:                    // 'true' 'where'
    case 141447:                    // 'false' 'with'
    case 141509:                    // 'null' 'with'
    case 141567:                    // 'true' 'with'
    case 142983:                    // 'false' 'words'
    case 143045:                    // 'null' 'words'
    case 143103:                    // 'true' 'words'
    case 145543:                    // 'false' '|'
    case 145605:                    // 'null' '|'
    case 145663:                    // 'true' '|'
    case 146055:                    // 'false' '||'
    case 146117:                    // 'null' '||'
    case 146175:                    // 'true' '||'
    case 146567:                    // 'false' '|}'
    case 146629:                    // 'null' '|}'
    case 146687:                    // 'true' '|}'
    case 147079:                    // 'false' '}'
    case 147141:                    // 'null' '}'
    case 147199:                    // 'true' '}'
      parse_Literal();
      break;
    case 31:                        // '$'
      parse_VarRef();
      break;
    case 35:                        // '('
      parse_ParenthesizedExpr();
      break;
    case 32:                        // '$$'
      parse_ContextItemExpr();
      break;
    case -5:
    case 17926:                     // EQName^Token '('
    case 17991:                     // 'after' '('
    case 17993:                     // 'allowing' '('
    case 17994:                     // 'ancestor' '('
    case 17995:                     // 'ancestor-or-self' '('
    case 17996:                     // 'and' '('
    case 17998:                     // 'append' '('
    case 18000:                     // 'as' '('
    case 18001:                     // 'ascending' '('
    case 18002:                     // 'at' '('
    case 18004:                     // 'base-uri' '('
    case 18005:                     // 'before' '('
    case 18006:                     // 'boundary-space' '('
    case 18007:                     // 'break' '('
    case 18009:                     // 'case' '('
    case 18010:                     // 'cast' '('
    case 18011:                     // 'castable' '('
    case 18012:                     // 'catch' '('
    case 18014:                     // 'child' '('
    case 18015:                     // 'collation' '('
    case 18018:                     // 'constraint' '('
    case 18019:                     // 'construction' '('
    case 18022:                     // 'context' '('
    case 18023:                     // 'continue' '('
    case 18024:                     // 'copy' '('
    case 18025:                     // 'copy-namespaces' '('
    case 18026:                     // 'count' '('
    case 18027:                     // 'decimal-format' '('
    case 18029:                     // 'declare' '('
    case 18030:                     // 'default' '('
    case 18031:                     // 'delete' '('
    case 18032:                     // 'descendant' '('
    case 18033:                     // 'descendant-or-self' '('
    case 18034:                     // 'descending' '('
    case 18039:                     // 'div' '('
    case 18040:                     // 'document' '('
    case 18043:                     // 'else' '('
    case 18044:                     // 'empty' '('
    case 18046:                     // 'encoding' '('
    case 18047:                     // 'end' '('
    case 18049:                     // 'eq' '('
    case 18050:                     // 'every' '('
    case 18052:                     // 'except' '('
    case 18053:                     // 'exit' '('
    case 18054:                     // 'external' '('
    case 18056:                     // 'first' '('
    case 18057:                     // 'following' '('
    case 18058:                     // 'following-sibling' '('
    case 18059:                     // 'for' '('
    case 18062:                     // 'from' '('
    case 18063:                     // 'ft-option' '('
    case 18068:                     // 'ge' '('
    case 18070:                     // 'group' '('
    case 18072:                     // 'gt' '('
    case 18073:                     // 'idiv' '('
    case 18075:                     // 'import' '('
    case 18076:                     // 'in' '('
    case 18077:                     // 'index' '('
    case 18081:                     // 'insert' '('
    case 18082:                     // 'instance' '('
    case 18083:                     // 'integrity' '('
    case 18084:                     // 'intersect' '('
    case 18085:                     // 'into' '('
    case 18086:                     // 'is' '('
    case 18088:                     // 'json' '('
    case 18090:                     // 'jsoniq' '('
    case 18093:                     // 'last' '('
    case 18094:                     // 'lax' '('
    case 18095:                     // 'le' '('
    case 18097:                     // 'let' '('
    case 18099:                     // 'loop' '('
    case 18101:                     // 'lt' '('
    case 18103:                     // 'mod' '('
    case 18104:                     // 'modify' '('
    case 18105:                     // 'module' '('
    case 18107:                     // 'namespace' '('
    case 18109:                     // 'ne' '('
    case 18115:                     // 'nodes' '('
    case 18118:                     // 'object' '('
    case 18122:                     // 'only' '('
    case 18123:                     // 'option' '('
    case 18124:                     // 'or' '('
    case 18125:                     // 'order' '('
    case 18126:                     // 'ordered' '('
    case 18127:                     // 'ordering' '('
    case 18130:                     // 'parent' '('
    case 18136:                     // 'preceding' '('
    case 18137:                     // 'preceding-sibling' '('
    case 18142:                     // 'rename' '('
    case 18143:                     // 'replace' '('
    case 18144:                     // 'return' '('
    case 18145:                     // 'returning' '('
    case 18146:                     // 'revalidation' '('
    case 18148:                     // 'satisfies' '('
    case 18149:                     // 'schema' '('
    case 18152:                     // 'score' '('
    case 18153:                     // 'select' '('
    case 18154:                     // 'self' '('
    case 18159:                     // 'sliding' '('
    case 18160:                     // 'some' '('
    case 18161:                     // 'stable' '('
    case 18162:                     // 'start' '('
    case 18165:                     // 'strict' '('
    case 18173:                     // 'to' '('
    case 18174:                     // 'treat' '('
    case 18176:                     // 'try' '('
    case 18177:                     // 'tumbling' '('
    case 18178:                     // 'type' '('
    case 18180:                     // 'union' '('
    case 18182:                     // 'unordered' '('
    case 18183:                     // 'updating' '('
    case 18186:                     // 'validate' '('
    case 18187:                     // 'value' '('
    case 18188:                     // 'variable' '('
    case 18189:                     // 'version' '('
    case 18192:                     // 'where' '('
    case 18193:                     // 'while' '('
    case 18196:                     // 'with' '('
      parse_FunctionCall();
      break;
    case 144078:                    // 'ordered' '{'
      parse_OrderedExpr();
      break;
    case 144134:                    // 'unordered' '{'
      parse_UnorderedExpr();
      break;
    case 33:                        // '%'
    case 79:                        // 'array'
    case 121:                       // 'document-node'
    case 125:                       // 'empty-sequence'
    case 147:                       // 'function'
    case 154:                       // 'if'
    case 167:                       // 'item'
    case 169:                       // 'json-item'
    case 188:                       // 'namespace-node'
    case 194:                       // 'node'
    case 230:                       // 'schema-attribute'
    case 231:                       // 'schema-element'
    case 247:                       // 'structured-item'
    case 248:                       // 'switch'
    case 259:                       // 'typeswitch'
    case 14854:                     // EQName^Token '#'
    case 14919:                     // 'after' '#'
    case 14921:                     // 'allowing' '#'
    case 14922:                     // 'ancestor' '#'
    case 14923:                     // 'ancestor-or-self' '#'
    case 14924:                     // 'and' '#'
    case 14926:                     // 'append' '#'
    case 14928:                     // 'as' '#'
    case 14929:                     // 'ascending' '#'
    case 14930:                     // 'at' '#'
    case 14931:                     // 'attribute' '#'
    case 14932:                     // 'base-uri' '#'
    case 14933:                     // 'before' '#'
    case 14934:                     // 'boundary-space' '#'
    case 14935:                     // 'break' '#'
    case 14937:                     // 'case' '#'
    case 14938:                     // 'cast' '#'
    case 14939:                     // 'castable' '#'
    case 14940:                     // 'catch' '#'
    case 14942:                     // 'child' '#'
    case 14943:                     // 'collation' '#'
    case 14945:                     // 'comment' '#'
    case 14946:                     // 'constraint' '#'
    case 14947:                     // 'construction' '#'
    case 14950:                     // 'context' '#'
    case 14951:                     // 'continue' '#'
    case 14952:                     // 'copy' '#'
    case 14953:                     // 'copy-namespaces' '#'
    case 14954:                     // 'count' '#'
    case 14955:                     // 'decimal-format' '#'
    case 14957:                     // 'declare' '#'
    case 14958:                     // 'default' '#'
    case 14959:                     // 'delete' '#'
    case 14960:                     // 'descendant' '#'
    case 14961:                     // 'descendant-or-self' '#'
    case 14962:                     // 'descending' '#'
    case 14967:                     // 'div' '#'
    case 14968:                     // 'document' '#'
    case 14970:                     // 'element' '#'
    case 14971:                     // 'else' '#'
    case 14972:                     // 'empty' '#'
    case 14974:                     // 'encoding' '#'
    case 14975:                     // 'end' '#'
    case 14977:                     // 'eq' '#'
    case 14978:                     // 'every' '#'
    case 14980:                     // 'except' '#'
    case 14981:                     // 'exit' '#'
    case 14982:                     // 'external' '#'
    case 14983:                     // 'false' '#'
    case 14984:                     // 'first' '#'
    case 14985:                     // 'following' '#'
    case 14986:                     // 'following-sibling' '#'
    case 14987:                     // 'for' '#'
    case 14990:                     // 'from' '#'
    case 14991:                     // 'ft-option' '#'
    case 14996:                     // 'ge' '#'
    case 14998:                     // 'group' '#'
    case 15000:                     // 'gt' '#'
    case 15001:                     // 'idiv' '#'
    case 15003:                     // 'import' '#'
    case 15004:                     // 'in' '#'
    case 15005:                     // 'index' '#'
    case 15009:                     // 'insert' '#'
    case 15010:                     // 'instance' '#'
    case 15011:                     // 'integrity' '#'
    case 15012:                     // 'intersect' '#'
    case 15013:                     // 'into' '#'
    case 15014:                     // 'is' '#'
    case 15016:                     // 'json' '#'
    case 15018:                     // 'jsoniq' '#'
    case 15021:                     // 'last' '#'
    case 15022:                     // 'lax' '#'
    case 15023:                     // 'le' '#'
    case 15025:                     // 'let' '#'
    case 15027:                     // 'loop' '#'
    case 15029:                     // 'lt' '#'
    case 15031:                     // 'mod' '#'
    case 15032:                     // 'modify' '#'
    case 15033:                     // 'module' '#'
    case 15035:                     // 'namespace' '#'
    case 15037:                     // 'ne' '#'
    case 15043:                     // 'nodes' '#'
    case 15045:                     // 'null' '#'
    case 15046:                     // 'object' '#'
    case 15050:                     // 'only' '#'
    case 15051:                     // 'option' '#'
    case 15052:                     // 'or' '#'
    case 15053:                     // 'order' '#'
    case 15054:                     // 'ordered' '#'
    case 15055:                     // 'ordering' '#'
    case 15058:                     // 'parent' '#'
    case 15064:                     // 'preceding' '#'
    case 15065:                     // 'preceding-sibling' '#'
    case 15068:                     // 'processing-instruction' '#'
    case 15070:                     // 'rename' '#'
    case 15071:                     // 'replace' '#'
    case 15072:                     // 'return' '#'
    case 15073:                     // 'returning' '#'
    case 15074:                     // 'revalidation' '#'
    case 15076:                     // 'satisfies' '#'
    case 15077:                     // 'schema' '#'
    case 15080:                     // 'score' '#'
    case 15081:                     // 'select' '#'
    case 15082:                     // 'self' '#'
    case 15087:                     // 'sliding' '#'
    case 15088:                     // 'some' '#'
    case 15089:                     // 'stable' '#'
    case 15090:                     // 'start' '#'
    case 15093:                     // 'strict' '#'
    case 15097:                     // 'text' '#'
    case 15101:                     // 'to' '#'
    case 15102:                     // 'treat' '#'
    case 15103:                     // 'true' '#'
    case 15104:                     // 'try' '#'
    case 15105:                     // 'tumbling' '#'
    case 15106:                     // 'type' '#'
    case 15108:                     // 'union' '#'
    case 15110:                     // 'unordered' '#'
    case 15111:                     // 'updating' '#'
    case 15114:                     // 'validate' '#'
    case 15115:                     // 'value' '#'
    case 15116:                     // 'variable' '#'
    case 15117:                     // 'version' '#'
    case 15120:                     // 'where' '#'
    case 15121:                     // 'while' '#'
    case 15124:                     // 'with' '#'
      parse_FunctionItemExpr();
      break;
    case -10:
    case 27929:                     // '{' ';'
      parse_BlockExpr();
      break;
    case -11:
    case 10009:                     // '{' NCName^Token
      parse_ObjectConstructor();
      break;
    case 69:                        // '['
      parse_ArrayConstructor();
      break;
    case 283:                       // '{|'
      parse_JSONSimpleObjectUnion();
      break;
    default:
      parse_Constructor();
    }
    eventHandler.endNonterminal("PrimaryExpr", e0);
  }

  function try_PrimaryExpr()
  {
    switch (l1)
    {
    case 187:                       // 'namespace'
      lookahead2W(247);             // NCName^Token | S^WS | '#' | '(' | '(:' | 'after' | 'allowing' | 'ancestor' |
      break;
    case 220:                       // 'processing-instruction'
      lookahead2W(245);             // NCName^Token | S^WS | '#' | '(:' | 'after' | 'allowing' | 'ancestor' |
      break;
    case 281:                       // '{'
      lookahead2W(283);             // EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral | StringLiteral |
      break;
    case 83:                        // 'attribute'
    case 122:                       // 'element'
      lookahead2W(253);             // EQName^Token | S^WS | '#' | '(:' | 'after' | 'allowing' | 'ancestor' |
      break;
    case 97:                        // 'comment'
    case 249:                       // 'text'
      lookahead2W(97);              // S^WS | '#' | '(:' | '{'
      break;
    case 120:                       // 'document'
    case 206:                       // 'ordered'
    case 262:                       // 'unordered'
      lookahead2W(148);             // S^WS | '#' | '(' | '(:' | '{'
      break;
    case 135:                       // 'false'
    case 197:                       // 'null'
    case 255:                       // 'true'
      lookahead2W(237);             // S^WS | EOF | '!' | '!=' | '#' | '(' | '(:' | ')' | '*' | '+' | ',' | '-' | '.' |
      break;
    case 6:                         // EQName^Token
    case 71:                        // 'after'
    case 73:                        // 'allowing'
    case 74:                        // 'ancestor'
    case 75:                        // 'ancestor-or-self'
    case 76:                        // 'and'
    case 78:                        // 'append'
    case 80:                        // 'as'
    case 81:                        // 'ascending'
    case 82:                        // 'at'
    case 84:                        // 'base-uri'
    case 85:                        // 'before'
    case 86:                        // 'boundary-space'
    case 87:                        // 'break'
    case 89:                        // 'case'
    case 90:                        // 'cast'
    case 91:                        // 'castable'
    case 92:                        // 'catch'
    case 94:                        // 'child'
    case 95:                        // 'collation'
    case 98:                        // 'constraint'
    case 99:                        // 'construction'
    case 102:                       // 'context'
    case 103:                       // 'continue'
    case 104:                       // 'copy'
    case 105:                       // 'copy-namespaces'
    case 106:                       // 'count'
    case 107:                       // 'decimal-format'
    case 109:                       // 'declare'
    case 110:                       // 'default'
    case 111:                       // 'delete'
    case 112:                       // 'descendant'
    case 113:                       // 'descendant-or-self'
    case 114:                       // 'descending'
    case 119:                       // 'div'
    case 123:                       // 'else'
    case 124:                       // 'empty'
    case 126:                       // 'encoding'
    case 127:                       // 'end'
    case 129:                       // 'eq'
    case 130:                       // 'every'
    case 132:                       // 'except'
    case 133:                       // 'exit'
    case 134:                       // 'external'
    case 136:                       // 'first'
    case 137:                       // 'following'
    case 138:                       // 'following-sibling'
    case 139:                       // 'for'
    case 142:                       // 'from'
    case 143:                       // 'ft-option'
    case 148:                       // 'ge'
    case 150:                       // 'group'
    case 152:                       // 'gt'
    case 153:                       // 'idiv'
    case 155:                       // 'import'
    case 156:                       // 'in'
    case 157:                       // 'index'
    case 161:                       // 'insert'
    case 162:                       // 'instance'
    case 163:                       // 'integrity'
    case 164:                       // 'intersect'
    case 165:                       // 'into'
    case 166:                       // 'is'
    case 168:                       // 'json'
    case 170:                       // 'jsoniq'
    case 173:                       // 'last'
    case 174:                       // 'lax'
    case 175:                       // 'le'
    case 177:                       // 'let'
    case 179:                       // 'loop'
    case 181:                       // 'lt'
    case 183:                       // 'mod'
    case 184:                       // 'modify'
    case 185:                       // 'module'
    case 189:                       // 'ne'
    case 195:                       // 'nodes'
    case 198:                       // 'object'
    case 202:                       // 'only'
    case 203:                       // 'option'
    case 204:                       // 'or'
    case 205:                       // 'order'
    case 207:                       // 'ordering'
    case 210:                       // 'parent'
    case 216:                       // 'preceding'
    case 217:                       // 'preceding-sibling'
    case 222:                       // 'rename'
    case 223:                       // 'replace'
    case 224:                       // 'return'
    case 225:                       // 'returning'
    case 226:                       // 'revalidation'
    case 228:                       // 'satisfies'
    case 229:                       // 'schema'
    case 232:                       // 'score'
    case 233:                       // 'select'
    case 234:                       // 'self'
    case 239:                       // 'sliding'
    case 240:                       // 'some'
    case 241:                       // 'stable'
    case 242:                       // 'start'
    case 245:                       // 'strict'
    case 253:                       // 'to'
    case 254:                       // 'treat'
    case 256:                       // 'try'
    case 257:                       // 'tumbling'
    case 258:                       // 'type'
    case 260:                       // 'union'
    case 263:                       // 'updating'
    case 266:                       // 'validate'
    case 267:                       // 'value'
    case 268:                       // 'variable'
    case 269:                       // 'version'
    case 272:                       // 'where'
    case 273:                       // 'while'
    case 276:                       // 'with'
      lookahead2W(95);              // S^WS | '#' | '(' | '(:'
      break;
    default:
      lk = l1;
    }
    if (lk == 3353                  // '{' EQName^Token
     || lk == 4377                  // '{' IntegerLiteral
     || lk == 4889                  // '{' DecimalLiteral
     || lk == 5401                  // '{' DoubleLiteral
     || lk == 5913                  // '{' StringLiteral
     || lk == 16153                 // '{' '$'
     || lk == 16665                 // '{' '$$'
     || lk == 17177                 // '{' '%'
     || lk == 18055                 // 'false' '('
     || lk == 18117                 // 'null' '('
     || lk == 18175                 // 'true' '('
     || lk == 18201                 // '{' '('
     || lk == 18713                 // '{' '(#'
     || lk == 21273                 // '{' '+'
     || lk == 22297                 // '{' '-'
     || lk == 24345                 // '{' '/'
     || lk == 24857                 // '{' '//'
     || lk == 28441                 // '{' '<'
     || lk == 28953                 // '{' '<!--'
     || lk == 31001                 // '{' '<?'
     || lk == 35609                 // '{' '['
     || lk == 36633                 // '{' 'after'
     || lk == 37657                 // '{' 'allowing'
     || lk == 38169                 // '{' 'ancestor'
     || lk == 38681                 // '{' 'ancestor-or-self'
     || lk == 39193                 // '{' 'and'
     || lk == 40217                 // '{' 'append'
     || lk == 40729                 // '{' 'array'
     || lk == 41241                 // '{' 'as'
     || lk == 41753                 // '{' 'ascending'
     || lk == 42265                 // '{' 'at'
     || lk == 42777                 // '{' 'attribute'
     || lk == 43289                 // '{' 'base-uri'
     || lk == 43801                 // '{' 'before'
     || lk == 44313                 // '{' 'boundary-space'
     || lk == 44825                 // '{' 'break'
     || lk == 45849                 // '{' 'case'
     || lk == 46361                 // '{' 'cast'
     || lk == 46873                 // '{' 'castable'
     || lk == 47385                 // '{' 'catch'
     || lk == 48409                 // '{' 'child'
     || lk == 48921                 // '{' 'collation'
     || lk == 49945                 // '{' 'comment'
     || lk == 50457                 // '{' 'constraint'
     || lk == 50969                 // '{' 'construction'
     || lk == 52505                 // '{' 'context'
     || lk == 53017                 // '{' 'continue'
     || lk == 53529                 // '{' 'copy'
     || lk == 54041                 // '{' 'copy-namespaces'
     || lk == 54553                 // '{' 'count'
     || lk == 55065                 // '{' 'decimal-format'
     || lk == 56089                 // '{' 'declare'
     || lk == 56601                 // '{' 'default'
     || lk == 57113                 // '{' 'delete'
     || lk == 57625                 // '{' 'descendant'
     || lk == 58137                 // '{' 'descendant-or-self'
     || lk == 58649                 // '{' 'descending'
     || lk == 61209                 // '{' 'div'
     || lk == 61721                 // '{' 'document'
     || lk == 62233                 // '{' 'document-node'
     || lk == 62745                 // '{' 'element'
     || lk == 63257                 // '{' 'else'
     || lk == 63769                 // '{' 'empty'
     || lk == 64281                 // '{' 'empty-sequence'
     || lk == 64793                 // '{' 'encoding'
     || lk == 65305                 // '{' 'end'
     || lk == 66329                 // '{' 'eq'
     || lk == 66841                 // '{' 'every'
     || lk == 67865                 // '{' 'except'
     || lk == 68377                 // '{' 'exit'
     || lk == 68889                 // '{' 'external'
     || lk == 69401                 // '{' 'false'
     || lk == 69913                 // '{' 'first'
     || lk == 70425                 // '{' 'following'
     || lk == 70937                 // '{' 'following-sibling'
     || lk == 71449                 // '{' 'for'
     || lk == 72985                 // '{' 'from'
     || lk == 73497                 // '{' 'ft-option'
     || lk == 75545                 // '{' 'function'
     || lk == 76057                 // '{' 'ge'
     || lk == 77081                 // '{' 'group'
     || lk == 78105                 // '{' 'gt'
     || lk == 78617                 // '{' 'idiv'
     || lk == 79129                 // '{' 'if'
     || lk == 79641                 // '{' 'import'
     || lk == 80153                 // '{' 'in'
     || lk == 80665                 // '{' 'index'
     || lk == 82713                 // '{' 'insert'
     || lk == 83225                 // '{' 'instance'
     || lk == 83737                 // '{' 'integrity'
     || lk == 84249                 // '{' 'intersect'
     || lk == 84761                 // '{' 'into'
     || lk == 85273                 // '{' 'is'
     || lk == 85785                 // '{' 'item'
     || lk == 86297                 // '{' 'json'
     || lk == 86809                 // '{' 'json-item'
     || lk == 87321                 // '{' 'jsoniq'
     || lk == 88857                 // '{' 'last'
     || lk == 89369                 // '{' 'lax'
     || lk == 89881                 // '{' 'le'
     || lk == 90905                 // '{' 'let'
     || lk == 91929                 // '{' 'loop'
     || lk == 92953                 // '{' 'lt'
     || lk == 93977                 // '{' 'mod'
     || lk == 94489                 // '{' 'modify'
     || lk == 95001                 // '{' 'module'
     || lk == 96025                 // '{' 'namespace'
     || lk == 96537                 // '{' 'namespace-node'
     || lk == 97049                 // '{' 'ne'
     || lk == 99609                 // '{' 'node'
     || lk == 100121                // '{' 'nodes'
     || lk == 100633                // '{' 'not'
     || lk == 101145                // '{' 'null'
     || lk == 101657                // '{' 'object'
     || lk == 103705                // '{' 'only'
     || lk == 104217                // '{' 'option'
     || lk == 104729                // '{' 'or'
     || lk == 105241                // '{' 'order'
     || lk == 105753                // '{' 'ordered'
     || lk == 106265                // '{' 'ordering'
     || lk == 107801                // '{' 'parent'
     || lk == 110873                // '{' 'preceding'
     || lk == 111385                // '{' 'preceding-sibling'
     || lk == 112921                // '{' 'processing-instruction'
     || lk == 113945                // '{' 'rename'
     || lk == 114457                // '{' 'replace'
     || lk == 114969                // '{' 'return'
     || lk == 115481                // '{' 'returning'
     || lk == 115993                // '{' 'revalidation'
     || lk == 117017                // '{' 'satisfies'
     || lk == 117529                // '{' 'schema'
     || lk == 118041                // '{' 'schema-attribute'
     || lk == 118553                // '{' 'schema-element'
     || lk == 119065                // '{' 'score'
     || lk == 119577                // '{' 'select'
     || lk == 120089                // '{' 'self'
     || lk == 122649                // '{' 'sliding'
     || lk == 123161                // '{' 'some'
     || lk == 123673                // '{' 'stable'
     || lk == 124185                // '{' 'start'
     || lk == 125721                // '{' 'strict'
     || lk == 126745                // '{' 'structured-item'
     || lk == 127257                // '{' 'switch'
     || lk == 127769                // '{' 'text'
     || lk == 129817                // '{' 'to'
     || lk == 130329                // '{' 'treat'
     || lk == 130841                // '{' 'true'
     || lk == 131353                // '{' 'try'
     || lk == 131865                // '{' 'tumbling'
     || lk == 132377                // '{' 'type'
     || lk == 132889                // '{' 'typeswitch'
     || lk == 133401                // '{' 'union'
     || lk == 134425                // '{' 'unordered'
     || lk == 134937                // '{' 'updating'
     || lk == 136473                // '{' 'validate'
     || lk == 136985                // '{' 'value'
     || lk == 137497                // '{' 'variable'
     || lk == 138009                // '{' 'version'
     || lk == 139545                // '{' 'where'
     || lk == 140057                // '{' 'while'
     || lk == 141593                // '{' 'with'
     || lk == 144153                // '{' '{'
     || lk == 145177                // '{' '{|'
     || lk == 147225)               // '{' '}'
    {
      lk = memoized(20, e0);
      if (lk == 0)
      {
        var b0A = b0; var e0A = e0; var l1A = l1;
        var b1A = b1; var e1A = e1; var l2A = l2;
        var b2A = b2; var e2A = e2;
        try
        {
          try_Literal();
          memoize(20, e0A, -1);
          lk = -14;
        }
        catch (p1A)
        {
          try
          {
            b0 = b0A; e0 = e0A; l1 = l1A; if (l1 == 0) {end = e0A;} else {
            b1 = b1A; e1 = e1A; l2 = l2A; if (l2 == 0) {end = e1A;} else {
            b2 = b2A; e2 = e2A; end = e2A; }}
            try_FunctionCall();
            memoize(20, e0A, -5);
            lk = -14;
          }
          catch (p5A)
          {
            try
            {
              b0 = b0A; e0 = e0A; l1 = l1A; if (l1 == 0) {end = e0A;} else {
              b1 = b1A; e1 = e1A; l2 = l2A; if (l2 == 0) {end = e1A;} else {
              b2 = b2A; e2 = e2A; end = e2A; }}
              try_BlockExpr();
              memoize(20, e0A, -10);
              lk = -14;
            }
            catch (p10A)
            {
              lk = -11;
              b0 = b0A; e0 = e0A; l1 = l1A; if (l1 == 0) {end = e0A;} else {
              b1 = b1A; e1 = e1A; l2 = l2A; if (l2 == 0) {end = e1A;} else {
              b2 = b2A; e2 = e2A; end = e2A; }}
              memoize(20, e0A, -11);
            }
          }
        }
      }
    }
    switch (lk)
    {
    case -1:
    case 8:                         // IntegerLiteral
    case 9:                         // DecimalLiteral
    case 10:                        // DoubleLiteral
    case 11:                        // StringLiteral
    case 12935:                     // 'false' EOF
    case 12997:                     // 'null' EOF
    case 13055:                     // 'true' EOF
    case 13447:                     // 'false' '!'
    case 13509:                     // 'null' '!'
    case 13567:                     // 'true' '!'
    case 13959:                     // 'false' '!='
    case 14021:                     // 'null' '!='
    case 14079:                     // 'true' '!='
    case 19591:                     // 'false' ')'
    case 19653:                     // 'null' ')'
    case 19711:                     // 'true' ')'
    case 20103:                     // 'false' '*'
    case 20165:                     // 'null' '*'
    case 20223:                     // 'true' '*'
    case 21127:                     // 'false' '+'
    case 21189:                     // 'null' '+'
    case 21247:                     // 'true' '+'
    case 21639:                     // 'false' ','
    case 21701:                     // 'null' ','
    case 21759:                     // 'true' ','
    case 22151:                     // 'false' '-'
    case 22213:                     // 'null' '-'
    case 22271:                     // 'true' '-'
    case 23175:                     // 'false' '.'
    case 23237:                     // 'null' '.'
    case 23295:                     // 'true' '.'
    case 24199:                     // 'false' '/'
    case 24261:                     // 'null' '/'
    case 24319:                     // 'true' '/'
    case 24711:                     // 'false' '//'
    case 24773:                     // 'null' '//'
    case 24831:                     // 'true' '//'
    case 25735:                     // 'false' ':'
    case 25797:                     // 'null' ':'
    case 25855:                     // 'true' ':'
    case 27783:                     // 'false' ';'
    case 27845:                     // 'null' ';'
    case 27903:                     // 'true' ';'
    case 28295:                     // 'false' '<'
    case 28357:                     // 'null' '<'
    case 28415:                     // 'true' '<'
    case 29831:                     // 'false' '<<'
    case 29893:                     // 'null' '<<'
    case 29951:                     // 'true' '<<'
    case 30343:                     // 'false' '<='
    case 30405:                     // 'null' '<='
    case 30463:                     // 'true' '<='
    case 31367:                     // 'false' '='
    case 31429:                     // 'null' '='
    case 31487:                     // 'true' '='
    case 31879:                     // 'false' '>'
    case 31941:                     // 'null' '>'
    case 31999:                     // 'true' '>'
    case 32391:                     // 'false' '>='
    case 32453:                     // 'null' '>='
    case 32511:                     // 'true' '>='
    case 32903:                     // 'false' '>>'
    case 32965:                     // 'null' '>>'
    case 33023:                     // 'true' '>>'
    case 35463:                     // 'false' '['
    case 35525:                     // 'null' '['
    case 35583:                     // 'true' '['
    case 35975:                     // 'false' ']'
    case 36037:                     // 'null' ']'
    case 36095:                     // 'true' ']'
    case 36487:                     // 'false' 'after'
    case 36549:                     // 'null' 'after'
    case 36607:                     // 'true' 'after'
    case 39047:                     // 'false' 'and'
    case 39109:                     // 'null' 'and'
    case 39167:                     // 'true' 'and'
    case 41095:                     // 'false' 'as'
    case 41157:                     // 'null' 'as'
    case 41215:                     // 'true' 'as'
    case 41607:                     // 'false' 'ascending'
    case 41669:                     // 'null' 'ascending'
    case 41727:                     // 'true' 'ascending'
    case 42119:                     // 'false' 'at'
    case 42181:                     // 'null' 'at'
    case 42239:                     // 'true' 'at'
    case 43655:                     // 'false' 'before'
    case 43717:                     // 'null' 'before'
    case 43775:                     // 'true' 'before'
    case 45191:                     // 'false' 'by'
    case 45253:                     // 'null' 'by'
    case 45311:                     // 'true' 'by'
    case 45703:                     // 'false' 'case'
    case 45765:                     // 'null' 'case'
    case 45823:                     // 'true' 'case'
    case 46215:                     // 'false' 'cast'
    case 46277:                     // 'null' 'cast'
    case 46335:                     // 'true' 'cast'
    case 46727:                     // 'false' 'castable'
    case 46789:                     // 'null' 'castable'
    case 46847:                     // 'true' 'castable'
    case 48775:                     // 'false' 'collation'
    case 48837:                     // 'null' 'collation'
    case 48895:                     // 'true' 'collation'
    case 51335:                     // 'false' 'contains'
    case 51397:                     // 'null' 'contains'
    case 51455:                     // 'true' 'contains'
    case 54407:                     // 'false' 'count'
    case 54469:                     // 'null' 'count'
    case 54527:                     // 'true' 'count'
    case 56455:                     // 'false' 'default'
    case 56517:                     // 'null' 'default'
    case 56575:                     // 'true' 'default'
    case 58503:                     // 'false' 'descending'
    case 58565:                     // 'null' 'descending'
    case 58623:                     // 'true' 'descending'
    case 61063:                     // 'false' 'div'
    case 61125:                     // 'null' 'div'
    case 61183:                     // 'true' 'div'
    case 63111:                     // 'false' 'else'
    case 63173:                     // 'null' 'else'
    case 63231:                     // 'true' 'else'
    case 63623:                     // 'false' 'empty'
    case 63685:                     // 'null' 'empty'
    case 63743:                     // 'true' 'empty'
    case 65159:                     // 'false' 'end'
    case 65221:                     // 'null' 'end'
    case 65279:                     // 'true' 'end'
    case 66183:                     // 'false' 'eq'
    case 66245:                     // 'null' 'eq'
    case 66303:                     // 'true' 'eq'
    case 67719:                     // 'false' 'except'
    case 67781:                     // 'null' 'except'
    case 67839:                     // 'true' 'except'
    case 71303:                     // 'false' 'for'
    case 71365:                     // 'null' 'for'
    case 71423:                     // 'true' 'for'
    case 75911:                     // 'false' 'ge'
    case 75973:                     // 'null' 'ge'
    case 76031:                     // 'true' 'ge'
    case 76935:                     // 'false' 'group'
    case 76997:                     // 'null' 'group'
    case 77055:                     // 'true' 'group'
    case 77959:                     // 'false' 'gt'
    case 78021:                     // 'null' 'gt'
    case 78079:                     // 'true' 'gt'
    case 78471:                     // 'false' 'idiv'
    case 78533:                     // 'null' 'idiv'
    case 78591:                     // 'true' 'idiv'
    case 83079:                     // 'false' 'instance'
    case 83141:                     // 'null' 'instance'
    case 83199:                     // 'true' 'instance'
    case 84103:                     // 'false' 'intersect'
    case 84165:                     // 'null' 'intersect'
    case 84223:                     // 'true' 'intersect'
    case 84615:                     // 'false' 'into'
    case 84677:                     // 'null' 'into'
    case 84735:                     // 'true' 'into'
    case 85127:                     // 'false' 'is'
    case 85189:                     // 'null' 'is'
    case 85247:                     // 'true' 'is'
    case 89735:                     // 'false' 'le'
    case 89797:                     // 'null' 'le'
    case 89855:                     // 'true' 'le'
    case 90759:                     // 'false' 'let'
    case 90821:                     // 'null' 'let'
    case 90879:                     // 'true' 'let'
    case 92807:                     // 'false' 'lt'
    case 92869:                     // 'null' 'lt'
    case 92927:                     // 'true' 'lt'
    case 93831:                     // 'false' 'mod'
    case 93893:                     // 'null' 'mod'
    case 93951:                     // 'true' 'mod'
    case 94343:                     // 'false' 'modify'
    case 94405:                     // 'null' 'modify'
    case 94463:                     // 'true' 'modify'
    case 96903:                     // 'false' 'ne'
    case 96965:                     // 'null' 'ne'
    case 97023:                     // 'true' 'ne'
    case 103559:                    // 'false' 'only'
    case 103621:                    // 'null' 'only'
    case 103679:                    // 'true' 'only'
    case 104583:                    // 'false' 'or'
    case 104645:                    // 'null' 'or'
    case 104703:                    // 'true' 'or'
    case 105095:                    // 'false' 'order'
    case 105157:                    // 'null' 'order'
    case 105215:                    // 'true' 'order'
    case 107143:                    // 'false' 'paragraphs'
    case 107205:                    // 'null' 'paragraphs'
    case 107263:                    // 'true' 'paragraphs'
    case 114823:                    // 'false' 'return'
    case 114885:                    // 'null' 'return'
    case 114943:                    // 'true' 'return'
    case 116871:                    // 'false' 'satisfies'
    case 116933:                    // 'null' 'satisfies'
    case 116991:                    // 'true' 'satisfies'
    case 121479:                    // 'false' 'sentences'
    case 121541:                    // 'null' 'sentences'
    case 121599:                    // 'true' 'sentences'
    case 123527:                    // 'false' 'stable'
    case 123589:                    // 'null' 'stable'
    case 123647:                    // 'true' 'stable'
    case 124039:                    // 'false' 'start'
    case 124101:                    // 'null' 'start'
    case 124159:                    // 'true' 'start'
    case 129159:                    // 'false' 'times'
    case 129221:                    // 'null' 'times'
    case 129279:                    // 'true' 'times'
    case 129671:                    // 'false' 'to'
    case 129733:                    // 'null' 'to'
    case 129791:                    // 'true' 'to'
    case 130183:                    // 'false' 'treat'
    case 130245:                    // 'null' 'treat'
    case 130303:                    // 'true' 'treat'
    case 133255:                    // 'false' 'union'
    case 133317:                    // 'null' 'union'
    case 133375:                    // 'true' 'union'
    case 139399:                    // 'false' 'where'
    case 139461:                    // 'null' 'where'
    case 139519:                    // 'true' 'where'
    case 141447:                    // 'false' 'with'
    case 141509:                    // 'null' 'with'
    case 141567:                    // 'true' 'with'
    case 142983:                    // 'false' 'words'
    case 143045:                    // 'null' 'words'
    case 143103:                    // 'true' 'words'
    case 145543:                    // 'false' '|'
    case 145605:                    // 'null' '|'
    case 145663:                    // 'true' '|'
    case 146055:                    // 'false' '||'
    case 146117:                    // 'null' '||'
    case 146175:                    // 'true' '||'
    case 146567:                    // 'false' '|}'
    case 146629:                    // 'null' '|}'
    case 146687:                    // 'true' '|}'
    case 147079:                    // 'false' '}'
    case 147141:                    // 'null' '}'
    case 147199:                    // 'true' '}'
      try_Literal();
      break;
    case 31:                        // '$'
      try_VarRef();
      break;
    case 35:                        // '('
      try_ParenthesizedExpr();
      break;
    case 32:                        // '$$'
      try_ContextItemExpr();
      break;
    case -5:
    case 17926:                     // EQName^Token '('
    case 17991:                     // 'after' '('
    case 17993:                     // 'allowing' '('
    case 17994:                     // 'ancestor' '('
    case 17995:                     // 'ancestor-or-self' '('
    case 17996:                     // 'and' '('
    case 17998:                     // 'append' '('
    case 18000:                     // 'as' '('
    case 18001:                     // 'ascending' '('
    case 18002:                     // 'at' '('
    case 18004:                     // 'base-uri' '('
    case 18005:                     // 'before' '('
    case 18006:                     // 'boundary-space' '('
    case 18007:                     // 'break' '('
    case 18009:                     // 'case' '('
    case 18010:                     // 'cast' '('
    case 18011:                     // 'castable' '('
    case 18012:                     // 'catch' '('
    case 18014:                     // 'child' '('
    case 18015:                     // 'collation' '('
    case 18018:                     // 'constraint' '('
    case 18019:                     // 'construction' '('
    case 18022:                     // 'context' '('
    case 18023:                     // 'continue' '('
    case 18024:                     // 'copy' '('
    case 18025:                     // 'copy-namespaces' '('
    case 18026:                     // 'count' '('
    case 18027:                     // 'decimal-format' '('
    case 18029:                     // 'declare' '('
    case 18030:                     // 'default' '('
    case 18031:                     // 'delete' '('
    case 18032:                     // 'descendant' '('
    case 18033:                     // 'descendant-or-self' '('
    case 18034:                     // 'descending' '('
    case 18039:                     // 'div' '('
    case 18040:                     // 'document' '('
    case 18043:                     // 'else' '('
    case 18044:                     // 'empty' '('
    case 18046:                     // 'encoding' '('
    case 18047:                     // 'end' '('
    case 18049:                     // 'eq' '('
    case 18050:                     // 'every' '('
    case 18052:                     // 'except' '('
    case 18053:                     // 'exit' '('
    case 18054:                     // 'external' '('
    case 18056:                     // 'first' '('
    case 18057:                     // 'following' '('
    case 18058:                     // 'following-sibling' '('
    case 18059:                     // 'for' '('
    case 18062:                     // 'from' '('
    case 18063:                     // 'ft-option' '('
    case 18068:                     // 'ge' '('
    case 18070:                     // 'group' '('
    case 18072:                     // 'gt' '('
    case 18073:                     // 'idiv' '('
    case 18075:                     // 'import' '('
    case 18076:                     // 'in' '('
    case 18077:                     // 'index' '('
    case 18081:                     // 'insert' '('
    case 18082:                     // 'instance' '('
    case 18083:                     // 'integrity' '('
    case 18084:                     // 'intersect' '('
    case 18085:                     // 'into' '('
    case 18086:                     // 'is' '('
    case 18088:                     // 'json' '('
    case 18090:                     // 'jsoniq' '('
    case 18093:                     // 'last' '('
    case 18094:                     // 'lax' '('
    case 18095:                     // 'le' '('
    case 18097:                     // 'let' '('
    case 18099:                     // 'loop' '('
    case 18101:                     // 'lt' '('
    case 18103:                     // 'mod' '('
    case 18104:                     // 'modify' '('
    case 18105:                     // 'module' '('
    case 18107:                     // 'namespace' '('
    case 18109:                     // 'ne' '('
    case 18115:                     // 'nodes' '('
    case 18118:                     // 'object' '('
    case 18122:                     // 'only' '('
    case 18123:                     // 'option' '('
    case 18124:                     // 'or' '('
    case 18125:                     // 'order' '('
    case 18126:                     // 'ordered' '('
    case 18127:                     // 'ordering' '('
    case 18130:                     // 'parent' '('
    case 18136:                     // 'preceding' '('
    case 18137:                     // 'preceding-sibling' '('
    case 18142:                     // 'rename' '('
    case 18143:                     // 'replace' '('
    case 18144:                     // 'return' '('
    case 18145:                     // 'returning' '('
    case 18146:                     // 'revalidation' '('
    case 18148:                     // 'satisfies' '('
    case 18149:                     // 'schema' '('
    case 18152:                     // 'score' '('
    case 18153:                     // 'select' '('
    case 18154:                     // 'self' '('
    case 18159:                     // 'sliding' '('
    case 18160:                     // 'some' '('
    case 18161:                     // 'stable' '('
    case 18162:                     // 'start' '('
    case 18165:                     // 'strict' '('
    case 18173:                     // 'to' '('
    case 18174:                     // 'treat' '('
    case 18176:                     // 'try' '('
    case 18177:                     // 'tumbling' '('
    case 18178:                     // 'type' '('
    case 18180:                     // 'union' '('
    case 18182:                     // 'unordered' '('
    case 18183:                     // 'updating' '('
    case 18186:                     // 'validate' '('
    case 18187:                     // 'value' '('
    case 18188:                     // 'variable' '('
    case 18189:                     // 'version' '('
    case 18192:                     // 'where' '('
    case 18193:                     // 'while' '('
    case 18196:                     // 'with' '('
      try_FunctionCall();
      break;
    case 144078:                    // 'ordered' '{'
      try_OrderedExpr();
      break;
    case 144134:                    // 'unordered' '{'
      try_UnorderedExpr();
      break;
    case 33:                        // '%'
    case 79:                        // 'array'
    case 121:                       // 'document-node'
    case 125:                       // 'empty-sequence'
    case 147:                       // 'function'
    case 154:                       // 'if'
    case 167:                       // 'item'
    case 169:                       // 'json-item'
    case 188:                       // 'namespace-node'
    case 194:                       // 'node'
    case 230:                       // 'schema-attribute'
    case 231:                       // 'schema-element'
    case 247:                       // 'structured-item'
    case 248:                       // 'switch'
    case 259:                       // 'typeswitch'
    case 14854:                     // EQName^Token '#'
    case 14919:                     // 'after' '#'
    case 14921:                     // 'allowing' '#'
    case 14922:                     // 'ancestor' '#'
    case 14923:                     // 'ancestor-or-self' '#'
    case 14924:                     // 'and' '#'
    case 14926:                     // 'append' '#'
    case 14928:                     // 'as' '#'
    case 14929:                     // 'ascending' '#'
    case 14930:                     // 'at' '#'
    case 14931:                     // 'attribute' '#'
    case 14932:                     // 'base-uri' '#'
    case 14933:                     // 'before' '#'
    case 14934:                     // 'boundary-space' '#'
    case 14935:                     // 'break' '#'
    case 14937:                     // 'case' '#'
    case 14938:                     // 'cast' '#'
    case 14939:                     // 'castable' '#'
    case 14940:                     // 'catch' '#'
    case 14942:                     // 'child' '#'
    case 14943:                     // 'collation' '#'
    case 14945:                     // 'comment' '#'
    case 14946:                     // 'constraint' '#'
    case 14947:                     // 'construction' '#'
    case 14950:                     // 'context' '#'
    case 14951:                     // 'continue' '#'
    case 14952:                     // 'copy' '#'
    case 14953:                     // 'copy-namespaces' '#'
    case 14954:                     // 'count' '#'
    case 14955:                     // 'decimal-format' '#'
    case 14957:                     // 'declare' '#'
    case 14958:                     // 'default' '#'
    case 14959:                     // 'delete' '#'
    case 14960:                     // 'descendant' '#'
    case 14961:                     // 'descendant-or-self' '#'
    case 14962:                     // 'descending' '#'
    case 14967:                     // 'div' '#'
    case 14968:                     // 'document' '#'
    case 14970:                     // 'element' '#'
    case 14971:                     // 'else' '#'
    case 14972:                     // 'empty' '#'
    case 14974:                     // 'encoding' '#'
    case 14975:                     // 'end' '#'
    case 14977:                     // 'eq' '#'
    case 14978:                     // 'every' '#'
    case 14980:                     // 'except' '#'
    case 14981:                     // 'exit' '#'
    case 14982:                     // 'external' '#'
    case 14983:                     // 'false' '#'
    case 14984:                     // 'first' '#'
    case 14985:                     // 'following' '#'
    case 14986:                     // 'following-sibling' '#'
    case 14987:                     // 'for' '#'
    case 14990:                     // 'from' '#'
    case 14991:                     // 'ft-option' '#'
    case 14996:                     // 'ge' '#'
    case 14998:                     // 'group' '#'
    case 15000:                     // 'gt' '#'
    case 15001:                     // 'idiv' '#'
    case 15003:                     // 'import' '#'
    case 15004:                     // 'in' '#'
    case 15005:                     // 'index' '#'
    case 15009:                     // 'insert' '#'
    case 15010:                     // 'instance' '#'
    case 15011:                     // 'integrity' '#'
    case 15012:                     // 'intersect' '#'
    case 15013:                     // 'into' '#'
    case 15014:                     // 'is' '#'
    case 15016:                     // 'json' '#'
    case 15018:                     // 'jsoniq' '#'
    case 15021:                     // 'last' '#'
    case 15022:                     // 'lax' '#'
    case 15023:                     // 'le' '#'
    case 15025:                     // 'let' '#'
    case 15027:                     // 'loop' '#'
    case 15029:                     // 'lt' '#'
    case 15031:                     // 'mod' '#'
    case 15032:                     // 'modify' '#'
    case 15033:                     // 'module' '#'
    case 15035:                     // 'namespace' '#'
    case 15037:                     // 'ne' '#'
    case 15043:                     // 'nodes' '#'
    case 15045:                     // 'null' '#'
    case 15046:                     // 'object' '#'
    case 15050:                     // 'only' '#'
    case 15051:                     // 'option' '#'
    case 15052:                     // 'or' '#'
    case 15053:                     // 'order' '#'
    case 15054:                     // 'ordered' '#'
    case 15055:                     // 'ordering' '#'
    case 15058:                     // 'parent' '#'
    case 15064:                     // 'preceding' '#'
    case 15065:                     // 'preceding-sibling' '#'
    case 15068:                     // 'processing-instruction' '#'
    case 15070:                     // 'rename' '#'
    case 15071:                     // 'replace' '#'
    case 15072:                     // 'return' '#'
    case 15073:                     // 'returning' '#'
    case 15074:                     // 'revalidation' '#'
    case 15076:                     // 'satisfies' '#'
    case 15077:                     // 'schema' '#'
    case 15080:                     // 'score' '#'
    case 15081:                     // 'select' '#'
    case 15082:                     // 'self' '#'
    case 15087:                     // 'sliding' '#'
    case 15088:                     // 'some' '#'
    case 15089:                     // 'stable' '#'
    case 15090:                     // 'start' '#'
    case 15093:                     // 'strict' '#'
    case 15097:                     // 'text' '#'
    case 15101:                     // 'to' '#'
    case 15102:                     // 'treat' '#'
    case 15103:                     // 'true' '#'
    case 15104:                     // 'try' '#'
    case 15105:                     // 'tumbling' '#'
    case 15106:                     // 'type' '#'
    case 15108:                     // 'union' '#'
    case 15110:                     // 'unordered' '#'
    case 15111:                     // 'updating' '#'
    case 15114:                     // 'validate' '#'
    case 15115:                     // 'value' '#'
    case 15116:                     // 'variable' '#'
    case 15117:                     // 'version' '#'
    case 15120:                     // 'where' '#'
    case 15121:                     // 'while' '#'
    case 15124:                     // 'with' '#'
      try_FunctionItemExpr();
      break;
    case -10:
    case 27929:                     // '{' ';'
      try_BlockExpr();
      break;
    case -11:
    case 10009:                     // '{' NCName^Token
      try_ObjectConstructor();
      break;
    case 69:                        // '['
      try_ArrayConstructor();
      break;
    case 283:                       // '{|'
      try_JSONSimpleObjectUnion();
      break;
    case -14:
      break;
    default:
      try_Constructor();
    }
  }

  function parse_JSONSimpleObjectUnion()
  {
    eventHandler.startNonterminal("JSONSimpleObjectUnion", e0);
    shift(283);                     // '{|'
    lookahead1W(274);               // EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral | StringLiteral |
    if (l1 != 286)                  // '|}'
    {
      whitespace();
      parse_Expr();
    }
    shift(286);                     // '|}'
    eventHandler.endNonterminal("JSONSimpleObjectUnion", e0);
  }

  function try_JSONSimpleObjectUnion()
  {
    shiftT(283);                    // '{|'
    lookahead1W(274);               // EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral | StringLiteral |
    if (l1 != 286)                  // '|}'
    {
      try_Expr();
    }
    shiftT(286);                    // '|}'
  }

  function parse_ObjectConstructor()
  {
    eventHandler.startNonterminal("ObjectConstructor", e0);
    shift(281);                     // '{'
    lookahead1W(277);               // EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral | StringLiteral |
    if (l1 != 287)                  // '}'
    {
      whitespace();
      parse_PairConstructorList();
    }
    shift(287);                     // '}'
    eventHandler.endNonterminal("ObjectConstructor", e0);
  }

  function try_ObjectConstructor()
  {
    shiftT(281);                    // '{'
    lookahead1W(277);               // EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral | StringLiteral |
    if (l1 != 287)                  // '}'
    {
      try_PairConstructorList();
    }
    shiftT(287);                    // '}'
  }

  function parse_PairConstructorList()
  {
    eventHandler.startNonterminal("PairConstructorList", e0);
    parse_PairConstructor();
    for (;;)
    {
      if (l1 != 42)                 // ','
      {
        break;
      }
      shift(42);                    // ','
      lookahead1W(268);             // EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral | StringLiteral |
      whitespace();
      parse_PairConstructor();
    }
    eventHandler.endNonterminal("PairConstructorList", e0);
  }

  function try_PairConstructorList()
  {
    try_PairConstructor();
    for (;;)
    {
      if (l1 != 42)                 // ','
      {
        break;
      }
      shiftT(42);                   // ','
      lookahead1W(268);             // EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral | StringLiteral |
      try_PairConstructor();
    }
  }

  function parse_PairConstructor()
  {
    eventHandler.startNonterminal("PairConstructor", e0);
    switch (l1)
    {
    case 78:                        // 'append'
      lookahead2W(279);             // EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral | StringLiteral |
      break;
    case 139:                       // 'for'
      lookahead2W(187);             // S^WS | '#' | '$' | '(' | '(:' | ':' | 'sliding' | 'tumbling'
      break;
    case 161:                       // 'insert'
      lookahead2W(282);             // EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral | StringLiteral |
      break;
    case 177:                       // 'let'
      lookahead2W(178);             // S^WS | '#' | '$' | '(' | '(:' | ':' | 'score'
      break;
    case 187:                       // 'namespace'
      lookahead2W(252);             // NCName^Token | S^WS | '#' | '(' | '(:' | ':' | 'after' | 'allowing' |
      break;
    case 220:                       // 'processing-instruction'
      lookahead2W(248);             // NCName^Token | S^WS | '#' | '(:' | ':' | 'after' | 'allowing' | 'ancestor' |
      break;
    case 223:                       // 'replace'
      lookahead2W(180);             // S^WS | '#' | '(' | '(:' | ':' | 'node' | 'value'
      break;
    case 266:                       // 'validate'
      lookahead2W(191);             // S^WS | '#' | '(' | '(:' | ':' | 'lax' | 'strict' | 'type' | '{'
      break;
    case 83:                        // 'attribute'
    case 122:                       // 'element'
      lookahead2W(257);             // EQName^Token | S^WS | '#' | '(:' | ':' | 'after' | 'allowing' | 'ancestor' |
      break;
    case 97:                        // 'comment'
    case 249:                       // 'text'
      lookahead2W(149);             // S^WS | '#' | '(:' | ':' | '{'
      break;
    case 111:                       // 'delete'
    case 222:                       // 'rename'
      lookahead2W(262);             // EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral | StringLiteral |
      break;
    case 104:                       // 'copy'
    case 130:                       // 'every'
    case 240:                       // 'some'
      lookahead2W(165);             // S^WS | '#' | '$' | '(' | '(:' | ':'
      break;
    case 135:                       // 'false'
    case 197:                       // 'null'
    case 255:                       // 'true'
      lookahead2W(209);             // S^WS | '!' | '!=' | '#' | '(' | '(:' | '*' | '+' | '-' | '.' | '/' | '//' | ':' |
      break;
    case 120:                       // 'document'
    case 206:                       // 'ordered'
    case 256:                       // 'try'
    case 262:                       // 'unordered'
      lookahead2W(167);             // S^WS | '#' | '(' | '(:' | ':' | '{'
      break;
    case 121:                       // 'document-node'
    case 125:                       // 'empty-sequence'
    case 167:                       // 'item'
    case 188:                       // 'namespace-node'
    case 194:                       // 'node'
    case 230:                       // 'schema-attribute'
    case 231:                       // 'schema-element'
      lookahead2W(96);              // S^WS | '#' | '(:' | ':'
      break;
    case 71:                        // 'after'
    case 73:                        // 'allowing'
    case 74:                        // 'ancestor'
    case 75:                        // 'ancestor-or-self'
    case 76:                        // 'and'
    case 80:                        // 'as'
    case 81:                        // 'ascending'
    case 82:                        // 'at'
    case 84:                        // 'base-uri'
    case 85:                        // 'before'
    case 86:                        // 'boundary-space'
    case 87:                        // 'break'
    case 89:                        // 'case'
    case 90:                        // 'cast'
    case 91:                        // 'castable'
    case 92:                        // 'catch'
    case 94:                        // 'child'
    case 95:                        // 'collation'
    case 98:                        // 'constraint'
    case 99:                        // 'construction'
    case 102:                       // 'context'
    case 103:                       // 'continue'
    case 105:                       // 'copy-namespaces'
    case 106:                       // 'count'
    case 107:                       // 'decimal-format'
    case 109:                       // 'declare'
    case 110:                       // 'default'
    case 112:                       // 'descendant'
    case 113:                       // 'descendant-or-self'
    case 114:                       // 'descending'
    case 119:                       // 'div'
    case 123:                       // 'else'
    case 124:                       // 'empty'
    case 126:                       // 'encoding'
    case 127:                       // 'end'
    case 129:                       // 'eq'
    case 132:                       // 'except'
    case 133:                       // 'exit'
    case 134:                       // 'external'
    case 136:                       // 'first'
    case 137:                       // 'following'
    case 138:                       // 'following-sibling'
    case 142:                       // 'from'
    case 143:                       // 'ft-option'
    case 147:                       // 'function'
    case 148:                       // 'ge'
    case 150:                       // 'group'
    case 152:                       // 'gt'
    case 153:                       // 'idiv'
    case 154:                       // 'if'
    case 155:                       // 'import'
    case 156:                       // 'in'
    case 157:                       // 'index'
    case 162:                       // 'instance'
    case 163:                       // 'integrity'
    case 164:                       // 'intersect'
    case 165:                       // 'into'
    case 166:                       // 'is'
    case 168:                       // 'json'
    case 170:                       // 'jsoniq'
    case 173:                       // 'last'
    case 174:                       // 'lax'
    case 175:                       // 'le'
    case 179:                       // 'loop'
    case 181:                       // 'lt'
    case 183:                       // 'mod'
    case 184:                       // 'modify'
    case 185:                       // 'module'
    case 189:                       // 'ne'
    case 195:                       // 'nodes'
    case 198:                       // 'object'
    case 202:                       // 'only'
    case 203:                       // 'option'
    case 204:                       // 'or'
    case 205:                       // 'order'
    case 207:                       // 'ordering'
    case 210:                       // 'parent'
    case 216:                       // 'preceding'
    case 217:                       // 'preceding-sibling'
    case 224:                       // 'return'
    case 225:                       // 'returning'
    case 226:                       // 'revalidation'
    case 228:                       // 'satisfies'
    case 229:                       // 'schema'
    case 232:                       // 'score'
    case 233:                       // 'select'
    case 234:                       // 'self'
    case 239:                       // 'sliding'
    case 241:                       // 'stable'
    case 242:                       // 'start'
    case 245:                       // 'strict'
    case 248:                       // 'switch'
    case 253:                       // 'to'
    case 254:                       // 'treat'
    case 257:                       // 'tumbling'
    case 258:                       // 'type'
    case 259:                       // 'typeswitch'
    case 260:                       // 'union'
    case 263:                       // 'updating'
    case 267:                       // 'value'
    case 268:                       // 'variable'
    case 269:                       // 'version'
    case 272:                       // 'where'
    case 273:                       // 'while'
    case 276:                       // 'with'
      lookahead2W(144);             // S^WS | '#' | '(' | '(:' | ':'
      break;
    default:
      lk = l1;
    }
    if (lk == 25735                 // 'false' ':'
     || lk == 25797                 // 'null' ':'
     || lk == 25855)                // 'true' ':'
    {
      lk = memoized(21, e0);
      if (lk == 0)
      {
        var b0A = b0; var e0A = e0; var l1A = l1;
        var b1A = b1; var e1A = e1; var l2A = l2;
        var b2A = b2; var e2A = e2;
        try
        {
          try_ExprSingle();
          lk = -1;
        }
        catch (p1A)
        {
          lk = -2;
        }
        b0 = b0A; e0 = e0A; l1 = l1A; if (l1 == 0) {end = e0A;} else {
        b1 = b1A; e1 = e1A; l2 = l2A; if (l2 == 0) {end = e1A;} else {
        b2 = b2A; e2 = e2A; end = e2A; }}
        memoize(21, e0, lk);
      }
    }
    switch (lk)
    {
    case -2:
    case 19:                        // NCName^Token
    case 25671:                     // 'after' ':'
    case 25673:                     // 'allowing' ':'
    case 25674:                     // 'ancestor' ':'
    case 25675:                     // 'ancestor-or-self' ':'
    case 25676:                     // 'and' ':'
    case 25678:                     // 'append' ':'
    case 25680:                     // 'as' ':'
    case 25681:                     // 'ascending' ':'
    case 25682:                     // 'at' ':'
    case 25683:                     // 'attribute' ':'
    case 25684:                     // 'base-uri' ':'
    case 25685:                     // 'before' ':'
    case 25686:                     // 'boundary-space' ':'
    case 25687:                     // 'break' ':'
    case 25689:                     // 'case' ':'
    case 25690:                     // 'cast' ':'
    case 25691:                     // 'castable' ':'
    case 25692:                     // 'catch' ':'
    case 25694:                     // 'child' ':'
    case 25695:                     // 'collation' ':'
    case 25697:                     // 'comment' ':'
    case 25698:                     // 'constraint' ':'
    case 25699:                     // 'construction' ':'
    case 25702:                     // 'context' ':'
    case 25703:                     // 'continue' ':'
    case 25704:                     // 'copy' ':'
    case 25705:                     // 'copy-namespaces' ':'
    case 25706:                     // 'count' ':'
    case 25707:                     // 'decimal-format' ':'
    case 25709:                     // 'declare' ':'
    case 25710:                     // 'default' ':'
    case 25711:                     // 'delete' ':'
    case 25712:                     // 'descendant' ':'
    case 25713:                     // 'descendant-or-self' ':'
    case 25714:                     // 'descending' ':'
    case 25719:                     // 'div' ':'
    case 25720:                     // 'document' ':'
    case 25721:                     // 'document-node' ':'
    case 25722:                     // 'element' ':'
    case 25723:                     // 'else' ':'
    case 25724:                     // 'empty' ':'
    case 25725:                     // 'empty-sequence' ':'
    case 25726:                     // 'encoding' ':'
    case 25727:                     // 'end' ':'
    case 25729:                     // 'eq' ':'
    case 25730:                     // 'every' ':'
    case 25732:                     // 'except' ':'
    case 25733:                     // 'exit' ':'
    case 25734:                     // 'external' ':'
    case 25736:                     // 'first' ':'
    case 25737:                     // 'following' ':'
    case 25738:                     // 'following-sibling' ':'
    case 25739:                     // 'for' ':'
    case 25742:                     // 'from' ':'
    case 25743:                     // 'ft-option' ':'
    case 25747:                     // 'function' ':'
    case 25748:                     // 'ge' ':'
    case 25750:                     // 'group' ':'
    case 25752:                     // 'gt' ':'
    case 25753:                     // 'idiv' ':'
    case 25754:                     // 'if' ':'
    case 25755:                     // 'import' ':'
    case 25756:                     // 'in' ':'
    case 25757:                     // 'index' ':'
    case 25761:                     // 'insert' ':'
    case 25762:                     // 'instance' ':'
    case 25763:                     // 'integrity' ':'
    case 25764:                     // 'intersect' ':'
    case 25765:                     // 'into' ':'
    case 25766:                     // 'is' ':'
    case 25767:                     // 'item' ':'
    case 25768:                     // 'json' ':'
    case 25770:                     // 'jsoniq' ':'
    case 25773:                     // 'last' ':'
    case 25774:                     // 'lax' ':'
    case 25775:                     // 'le' ':'
    case 25777:                     // 'let' ':'
    case 25779:                     // 'loop' ':'
    case 25781:                     // 'lt' ':'
    case 25783:                     // 'mod' ':'
    case 25784:                     // 'modify' ':'
    case 25785:                     // 'module' ':'
    case 25787:                     // 'namespace' ':'
    case 25788:                     // 'namespace-node' ':'
    case 25789:                     // 'ne' ':'
    case 25794:                     // 'node' ':'
    case 25795:                     // 'nodes' ':'
    case 25798:                     // 'object' ':'
    case 25802:                     // 'only' ':'
    case 25803:                     // 'option' ':'
    case 25804:                     // 'or' ':'
    case 25805:                     // 'order' ':'
    case 25806:                     // 'ordered' ':'
    case 25807:                     // 'ordering' ':'
    case 25810:                     // 'parent' ':'
    case 25816:                     // 'preceding' ':'
    case 25817:                     // 'preceding-sibling' ':'
    case 25820:                     // 'processing-instruction' ':'
    case 25822:                     // 'rename' ':'
    case 25823:                     // 'replace' ':'
    case 25824:                     // 'return' ':'
    case 25825:                     // 'returning' ':'
    case 25826:                     // 'revalidation' ':'
    case 25828:                     // 'satisfies' ':'
    case 25829:                     // 'schema' ':'
    case 25830:                     // 'schema-attribute' ':'
    case 25831:                     // 'schema-element' ':'
    case 25832:                     // 'score' ':'
    case 25833:                     // 'select' ':'
    case 25834:                     // 'self' ':'
    case 25839:                     // 'sliding' ':'
    case 25840:                     // 'some' ':'
    case 25841:                     // 'stable' ':'
    case 25842:                     // 'start' ':'
    case 25845:                     // 'strict' ':'
    case 25848:                     // 'switch' ':'
    case 25849:                     // 'text' ':'
    case 25853:                     // 'to' ':'
    case 25854:                     // 'treat' ':'
    case 25856:                     // 'try' ':'
    case 25857:                     // 'tumbling' ':'
    case 25858:                     // 'type' ':'
    case 25859:                     // 'typeswitch' ':'
    case 25860:                     // 'union' ':'
    case 25862:                     // 'unordered' ':'
    case 25863:                     // 'updating' ':'
    case 25866:                     // 'validate' ':'
    case 25867:                     // 'value' ':'
    case 25868:                     // 'variable' ':'
    case 25869:                     // 'version' ':'
    case 25872:                     // 'where' ':'
    case 25873:                     // 'while' ':'
    case 25876:                     // 'with' ':'
      parse_NCName();
      break;
    default:
      parse_ExprSingle();
    }
    lookahead1W(26);                // S^WS | '(:' | ':'
    shift(50);                      // ':'
    lookahead1W(267);               // EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral | StringLiteral |
    whitespace();
    parse_ExprSingle();
    eventHandler.endNonterminal("PairConstructor", e0);
  }

  function try_PairConstructor()
  {
    switch (l1)
    {
    case 78:                        // 'append'
      lookahead2W(279);             // EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral | StringLiteral |
      break;
    case 139:                       // 'for'
      lookahead2W(187);             // S^WS | '#' | '$' | '(' | '(:' | ':' | 'sliding' | 'tumbling'
      break;
    case 161:                       // 'insert'
      lookahead2W(282);             // EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral | StringLiteral |
      break;
    case 177:                       // 'let'
      lookahead2W(178);             // S^WS | '#' | '$' | '(' | '(:' | ':' | 'score'
      break;
    case 187:                       // 'namespace'
      lookahead2W(252);             // NCName^Token | S^WS | '#' | '(' | '(:' | ':' | 'after' | 'allowing' |
      break;
    case 220:                       // 'processing-instruction'
      lookahead2W(248);             // NCName^Token | S^WS | '#' | '(:' | ':' | 'after' | 'allowing' | 'ancestor' |
      break;
    case 223:                       // 'replace'
      lookahead2W(180);             // S^WS | '#' | '(' | '(:' | ':' | 'node' | 'value'
      break;
    case 266:                       // 'validate'
      lookahead2W(191);             // S^WS | '#' | '(' | '(:' | ':' | 'lax' | 'strict' | 'type' | '{'
      break;
    case 83:                        // 'attribute'
    case 122:                       // 'element'
      lookahead2W(257);             // EQName^Token | S^WS | '#' | '(:' | ':' | 'after' | 'allowing' | 'ancestor' |
      break;
    case 97:                        // 'comment'
    case 249:                       // 'text'
      lookahead2W(149);             // S^WS | '#' | '(:' | ':' | '{'
      break;
    case 111:                       // 'delete'
    case 222:                       // 'rename'
      lookahead2W(262);             // EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral | StringLiteral |
      break;
    case 104:                       // 'copy'
    case 130:                       // 'every'
    case 240:                       // 'some'
      lookahead2W(165);             // S^WS | '#' | '$' | '(' | '(:' | ':'
      break;
    case 135:                       // 'false'
    case 197:                       // 'null'
    case 255:                       // 'true'
      lookahead2W(209);             // S^WS | '!' | '!=' | '#' | '(' | '(:' | '*' | '+' | '-' | '.' | '/' | '//' | ':' |
      break;
    case 120:                       // 'document'
    case 206:                       // 'ordered'
    case 256:                       // 'try'
    case 262:                       // 'unordered'
      lookahead2W(167);             // S^WS | '#' | '(' | '(:' | ':' | '{'
      break;
    case 121:                       // 'document-node'
    case 125:                       // 'empty-sequence'
    case 167:                       // 'item'
    case 188:                       // 'namespace-node'
    case 194:                       // 'node'
    case 230:                       // 'schema-attribute'
    case 231:                       // 'schema-element'
      lookahead2W(96);              // S^WS | '#' | '(:' | ':'
      break;
    case 71:                        // 'after'
    case 73:                        // 'allowing'
    case 74:                        // 'ancestor'
    case 75:                        // 'ancestor-or-self'
    case 76:                        // 'and'
    case 80:                        // 'as'
    case 81:                        // 'ascending'
    case 82:                        // 'at'
    case 84:                        // 'base-uri'
    case 85:                        // 'before'
    case 86:                        // 'boundary-space'
    case 87:                        // 'break'
    case 89:                        // 'case'
    case 90:                        // 'cast'
    case 91:                        // 'castable'
    case 92:                        // 'catch'
    case 94:                        // 'child'
    case 95:                        // 'collation'
    case 98:                        // 'constraint'
    case 99:                        // 'construction'
    case 102:                       // 'context'
    case 103:                       // 'continue'
    case 105:                       // 'copy-namespaces'
    case 106:                       // 'count'
    case 107:                       // 'decimal-format'
    case 109:                       // 'declare'
    case 110:                       // 'default'
    case 112:                       // 'descendant'
    case 113:                       // 'descendant-or-self'
    case 114:                       // 'descending'
    case 119:                       // 'div'
    case 123:                       // 'else'
    case 124:                       // 'empty'
    case 126:                       // 'encoding'
    case 127:                       // 'end'
    case 129:                       // 'eq'
    case 132:                       // 'except'
    case 133:                       // 'exit'
    case 134:                       // 'external'
    case 136:                       // 'first'
    case 137:                       // 'following'
    case 138:                       // 'following-sibling'
    case 142:                       // 'from'
    case 143:                       // 'ft-option'
    case 147:                       // 'function'
    case 148:                       // 'ge'
    case 150:                       // 'group'
    case 152:                       // 'gt'
    case 153:                       // 'idiv'
    case 154:                       // 'if'
    case 155:                       // 'import'
    case 156:                       // 'in'
    case 157:                       // 'index'
    case 162:                       // 'instance'
    case 163:                       // 'integrity'
    case 164:                       // 'intersect'
    case 165:                       // 'into'
    case 166:                       // 'is'
    case 168:                       // 'json'
    case 170:                       // 'jsoniq'
    case 173:                       // 'last'
    case 174:                       // 'lax'
    case 175:                       // 'le'
    case 179:                       // 'loop'
    case 181:                       // 'lt'
    case 183:                       // 'mod'
    case 184:                       // 'modify'
    case 185:                       // 'module'
    case 189:                       // 'ne'
    case 195:                       // 'nodes'
    case 198:                       // 'object'
    case 202:                       // 'only'
    case 203:                       // 'option'
    case 204:                       // 'or'
    case 205:                       // 'order'
    case 207:                       // 'ordering'
    case 210:                       // 'parent'
    case 216:                       // 'preceding'
    case 217:                       // 'preceding-sibling'
    case 224:                       // 'return'
    case 225:                       // 'returning'
    case 226:                       // 'revalidation'
    case 228:                       // 'satisfies'
    case 229:                       // 'schema'
    case 232:                       // 'score'
    case 233:                       // 'select'
    case 234:                       // 'self'
    case 239:                       // 'sliding'
    case 241:                       // 'stable'
    case 242:                       // 'start'
    case 245:                       // 'strict'
    case 248:                       // 'switch'
    case 253:                       // 'to'
    case 254:                       // 'treat'
    case 257:                       // 'tumbling'
    case 258:                       // 'type'
    case 259:                       // 'typeswitch'
    case 260:                       // 'union'
    case 263:                       // 'updating'
    case 267:                       // 'value'
    case 268:                       // 'variable'
    case 269:                       // 'version'
    case 272:                       // 'where'
    case 273:                       // 'while'
    case 276:                       // 'with'
      lookahead2W(144);             // S^WS | '#' | '(' | '(:' | ':'
      break;
    default:
      lk = l1;
    }
    if (lk == 25735                 // 'false' ':'
     || lk == 25797                 // 'null' ':'
     || lk == 25855)                // 'true' ':'
    {
      lk = memoized(21, e0);
      if (lk == 0)
      {
        var b0A = b0; var e0A = e0; var l1A = l1;
        var b1A = b1; var e1A = e1; var l2A = l2;
        var b2A = b2; var e2A = e2;
        try
        {
          try_ExprSingle();
          memoize(21, e0A, -1);
          lk = -3;
        }
        catch (p1A)
        {
          lk = -2;
          b0 = b0A; e0 = e0A; l1 = l1A; if (l1 == 0) {end = e0A;} else {
          b1 = b1A; e1 = e1A; l2 = l2A; if (l2 == 0) {end = e1A;} else {
          b2 = b2A; e2 = e2A; end = e2A; }}
          memoize(21, e0A, -2);
        }
      }
    }
    switch (lk)
    {
    case -2:
    case 19:                        // NCName^Token
    case 25671:                     // 'after' ':'
    case 25673:                     // 'allowing' ':'
    case 25674:                     // 'ancestor' ':'
    case 25675:                     // 'ancestor-or-self' ':'
    case 25676:                     // 'and' ':'
    case 25678:                     // 'append' ':'
    case 25680:                     // 'as' ':'
    case 25681:                     // 'ascending' ':'
    case 25682:                     // 'at' ':'
    case 25683:                     // 'attribute' ':'
    case 25684:                     // 'base-uri' ':'
    case 25685:                     // 'before' ':'
    case 25686:                     // 'boundary-space' ':'
    case 25687:                     // 'break' ':'
    case 25689:                     // 'case' ':'
    case 25690:                     // 'cast' ':'
    case 25691:                     // 'castable' ':'
    case 25692:                     // 'catch' ':'
    case 25694:                     // 'child' ':'
    case 25695:                     // 'collation' ':'
    case 25697:                     // 'comment' ':'
    case 25698:                     // 'constraint' ':'
    case 25699:                     // 'construction' ':'
    case 25702:                     // 'context' ':'
    case 25703:                     // 'continue' ':'
    case 25704:                     // 'copy' ':'
    case 25705:                     // 'copy-namespaces' ':'
    case 25706:                     // 'count' ':'
    case 25707:                     // 'decimal-format' ':'
    case 25709:                     // 'declare' ':'
    case 25710:                     // 'default' ':'
    case 25711:                     // 'delete' ':'
    case 25712:                     // 'descendant' ':'
    case 25713:                     // 'descendant-or-self' ':'
    case 25714:                     // 'descending' ':'
    case 25719:                     // 'div' ':'
    case 25720:                     // 'document' ':'
    case 25721:                     // 'document-node' ':'
    case 25722:                     // 'element' ':'
    case 25723:                     // 'else' ':'
    case 25724:                     // 'empty' ':'
    case 25725:                     // 'empty-sequence' ':'
    case 25726:                     // 'encoding' ':'
    case 25727:                     // 'end' ':'
    case 25729:                     // 'eq' ':'
    case 25730:                     // 'every' ':'
    case 25732:                     // 'except' ':'
    case 25733:                     // 'exit' ':'
    case 25734:                     // 'external' ':'
    case 25736:                     // 'first' ':'
    case 25737:                     // 'following' ':'
    case 25738:                     // 'following-sibling' ':'
    case 25739:                     // 'for' ':'
    case 25742:                     // 'from' ':'
    case 25743:                     // 'ft-option' ':'
    case 25747:                     // 'function' ':'
    case 25748:                     // 'ge' ':'
    case 25750:                     // 'group' ':'
    case 25752:                     // 'gt' ':'
    case 25753:                     // 'idiv' ':'
    case 25754:                     // 'if' ':'
    case 25755:                     // 'import' ':'
    case 25756:                     // 'in' ':'
    case 25757:                     // 'index' ':'
    case 25761:                     // 'insert' ':'
    case 25762:                     // 'instance' ':'
    case 25763:                     // 'integrity' ':'
    case 25764:                     // 'intersect' ':'
    case 25765:                     // 'into' ':'
    case 25766:                     // 'is' ':'
    case 25767:                     // 'item' ':'
    case 25768:                     // 'json' ':'
    case 25770:                     // 'jsoniq' ':'
    case 25773:                     // 'last' ':'
    case 25774:                     // 'lax' ':'
    case 25775:                     // 'le' ':'
    case 25777:                     // 'let' ':'
    case 25779:                     // 'loop' ':'
    case 25781:                     // 'lt' ':'
    case 25783:                     // 'mod' ':'
    case 25784:                     // 'modify' ':'
    case 25785:                     // 'module' ':'
    case 25787:                     // 'namespace' ':'
    case 25788:                     // 'namespace-node' ':'
    case 25789:                     // 'ne' ':'
    case 25794:                     // 'node' ':'
    case 25795:                     // 'nodes' ':'
    case 25798:                     // 'object' ':'
    case 25802:                     // 'only' ':'
    case 25803:                     // 'option' ':'
    case 25804:                     // 'or' ':'
    case 25805:                     // 'order' ':'
    case 25806:                     // 'ordered' ':'
    case 25807:                     // 'ordering' ':'
    case 25810:                     // 'parent' ':'
    case 25816:                     // 'preceding' ':'
    case 25817:                     // 'preceding-sibling' ':'
    case 25820:                     // 'processing-instruction' ':'
    case 25822:                     // 'rename' ':'
    case 25823:                     // 'replace' ':'
    case 25824:                     // 'return' ':'
    case 25825:                     // 'returning' ':'
    case 25826:                     // 'revalidation' ':'
    case 25828:                     // 'satisfies' ':'
    case 25829:                     // 'schema' ':'
    case 25830:                     // 'schema-attribute' ':'
    case 25831:                     // 'schema-element' ':'
    case 25832:                     // 'score' ':'
    case 25833:                     // 'select' ':'
    case 25834:                     // 'self' ':'
    case 25839:                     // 'sliding' ':'
    case 25840:                     // 'some' ':'
    case 25841:                     // 'stable' ':'
    case 25842:                     // 'start' ':'
    case 25845:                     // 'strict' ':'
    case 25848:                     // 'switch' ':'
    case 25849:                     // 'text' ':'
    case 25853:                     // 'to' ':'
    case 25854:                     // 'treat' ':'
    case 25856:                     // 'try' ':'
    case 25857:                     // 'tumbling' ':'
    case 25858:                     // 'type' ':'
    case 25859:                     // 'typeswitch' ':'
    case 25860:                     // 'union' ':'
    case 25862:                     // 'unordered' ':'
    case 25863:                     // 'updating' ':'
    case 25866:                     // 'validate' ':'
    case 25867:                     // 'value' ':'
    case 25868:                     // 'variable' ':'
    case 25869:                     // 'version' ':'
    case 25872:                     // 'where' ':'
    case 25873:                     // 'while' ':'
    case 25876:                     // 'with' ':'
      try_NCName();
      break;
    case -3:
      break;
    default:
      try_ExprSingle();
    }
    lookahead1W(26);                // S^WS | '(:' | ':'
    shiftT(50);                     // ':'
    lookahead1W(267);               // EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral | StringLiteral |
    try_ExprSingle();
  }

  function parse_ArrayConstructor()
  {
    eventHandler.startNonterminal("ArrayConstructor", e0);
    shift(69);                      // '['
    lookahead1W(273);               // EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral | StringLiteral |
    if (l1 != 70)                   // ']'
    {
      whitespace();
      parse_Expr();
    }
    shift(70);                      // ']'
    eventHandler.endNonterminal("ArrayConstructor", e0);
  }

  function try_ArrayConstructor()
  {
    shiftT(69);                     // '['
    lookahead1W(273);               // EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral | StringLiteral |
    if (l1 != 70)                   // ']'
    {
      try_Expr();
    }
    shiftT(70);                     // ']'
  }

  function parse_BlockExpr()
  {
    eventHandler.startNonterminal("BlockExpr", e0);
    shift(281);                     // '{'
    lookahead1W(281);               // EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral | StringLiteral |
    whitespace();
    parse_StatementsAndOptionalExpr();
    shift(287);                     // '}'
    eventHandler.endNonterminal("BlockExpr", e0);
  }

  function try_BlockExpr()
  {
    shiftT(281);                    // '{'
    lookahead1W(281);               // EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral | StringLiteral |
    try_StatementsAndOptionalExpr();
    shiftT(287);                    // '}'
  }

  function parse_FunctionDecl()
  {
    eventHandler.startNonterminal("FunctionDecl", e0);
    shift(147);                     // 'function'
    lookahead1W(246);               // EQName^Token | S^WS | '(:' | 'after' | 'allowing' | 'ancestor' |
    whitespace();
    parse_EQName();
    lookahead1W(22);                // S^WS | '(' | '(:'
    shift(35);                      // '('
    lookahead1W(98);                // S^WS | '$' | '(:' | ')'
    if (l1 == 31)                   // '$'
    {
      whitespace();
      parse_ParamList();
    }
    shift(38);                      // ')'
    lookahead1W(158);               // S^WS | '(:' | 'as' | 'external' | '{'
    if (l1 == 80)                   // 'as'
    {
      whitespace();
      parse_ReturnType();
    }
    lookahead1W(122);               // S^WS | '(:' | 'external' | '{'
    switch (l1)
    {
    case 281:                       // '{'
      shift(281);                   // '{'
      lookahead1W(281);             // EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral | StringLiteral |
      whitespace();
      parse_StatementsAndOptionalExpr();
      shift(287);                   // '}'
      break;
    default:
      shift(134);                   // 'external'
    }
    eventHandler.endNonterminal("FunctionDecl", e0);
  }

  function parse_ReturnType()
  {
    eventHandler.startNonterminal("ReturnType", e0);
    shift(80);                      // 'as'
    lookahead1W(254);               // EQName^Token | S^WS | '%' | '(' | '(:' | 'after' | 'allowing' | 'ancestor' |
    whitespace();
    parse_SequenceType();
    eventHandler.endNonterminal("ReturnType", e0);
  }

  function shift(t)
  {
    if (l1 == t)
    {
      whitespace();
      eventHandler.terminal(JSONiqParser.TOKEN[l1], b1, e1 > size ? size : e1);
      b0 = b1; e0 = e1; l1 = l2; if (l1 != 0) {
      b1 = b2; e1 = e2; l2 = 0; }
    }
    else
    {
      error(b1, e1, 0, l1, t);
    }
  }

  function shiftT(t)
  {
    if (l1 == t)
    {
      b0 = b1; e0 = e1; l1 = l2; if (l1 != 0) {
      b1 = b2; e1 = e2; l2 = 0; }
    }
    else
    {
      error(b1, e1, 0, l1, t);
    }
  }

  function skip(code)
  {
    var b0W = b0; var e0W = e0; var l1W = l1;
    var b1W = b1; var e1W = e1;

    l1 = code; b1 = begin; e1 = end;
    l2 = 0;

    try_Whitespace();

    b0 = b0W; e0 = e0W; l1 = l1W; if (l1 != 0) {
    b1 = b1W; e1 = e1W; }
  }

  function whitespace()
  {
    if (e0 != b1)
    {
      b0 = e0;
      e0 = b1;
      eventHandler.whitespace(b0, e0);
    }
  }

  function matchW(set)
  {
    var code;
    for (;;)
    {
      code = match(set);
      if (code != 22)               // S^WS
      {
        if (code != 37)             // '(:'
        {
          break;
        }
        skip(code);
      }
    }
    return code;
  }

  function lookahead1W(set)
  {
    if (l1 == 0)
    {
      l1 = matchW(set);
      b1 = begin;
      e1 = end;
    }
  }

  function lookahead2W(set)
  {
    if (l2 == 0)
    {
      l2 = matchW(set);
      b2 = begin;
      e2 = end;
    }
    lk = (l2 << 9) | l1;
  }

  function lookahead1(set)
  {
    if (l1 == 0)
    {
      l1 = match(set);
      b1 = begin;
      e1 = end;
    }
  }

  function lookahead2(set)
  {
    if (l2 == 0)
    {
      l2 = match(set);
      b2 = begin;
      e2 = end;
    }
    lk = (l2 << 9) | l1;
  }

  function error(b, e, s, l, t)
  {
    if (e > ex)
    {
      bx = b;
      ex = e;
      sx = s;
      lx = l;
      tx = t;
    }
    throw new self.ParseException(bx, ex, sx, lx, tx);
  }

  var lk, b0, e0;
  var l1, b1, e1;
  var l2, b2, e2;
  var bx, ex, sx, lx, tx;
  var eventHandler;
  var memo;

  function memoize(i, e, v)
  {
    memo[(e << 5) + i] = v;
  }

  function memoized(i, e)
  {
    var v = memo[(e << 5) + i];
    return typeof v != "undefined" ? v : 0;
  }

  var input;
  var size;
  var begin;
  var end;

  function match(tokenSetId)
  {
    var nonbmp = false;
    begin = end;
    var current = end;
    var result = JSONiqParser.INITIAL[tokenSetId];
    var state = 0;

    for (var code = result & 8191; code != 0; )
    {
      var charclass;
      var c0 = current < size ? input.charCodeAt(current) : 0;
      ++current;
      if (c0 < 0x80)
      {
        charclass = JSONiqParser.MAP0[c0];
      }
      else if (c0 < 0xd800)
      {
        var c1 = c0 >> 4;
        charclass = JSONiqParser.MAP1[(c0 & 15) + JSONiqParser.MAP1[(c1 & 31) + JSONiqParser.MAP1[c1 >> 5]]];
      }
      else
      {
        if (c0 < 0xdc00)
        {
          var c1 = current < size ? input.charCodeAt(current) : 0;
          if (c1 >= 0xdc00 && c1 < 0xe000)
          {
            ++current;
            c0 = ((c0 & 0x3ff) << 10) + (c1 & 0x3ff) + 0x10000;
            nonbmp = true;
          }
        }
        var lo = 0, hi = 5;
        for (var m = 3; ; m = (hi + lo) >> 1)
        {
          if (JSONiqParser.MAP2[m] > c0) hi = m - 1;
          else if (JSONiqParser.MAP2[6 + m] < c0) lo = m + 1;
          else {charclass = JSONiqParser.MAP2[12 + m]; break;}
          if (lo > hi) {charclass = 0; break;}
        }
      }

      state = code;
      var i0 = (charclass << 13) + code - 1;
      code = JSONiqParser.TRANSITION[(i0 & 15) + JSONiqParser.TRANSITION[i0 >> 4]];

      if (code > 8191)
      {
        result = code;
        code &= 8191;
        end = current;
      }
    }

    result >>= 13;
    if (result == 0)
    {
      end = current - 1;
      var c1 = end < size ? input.charCodeAt(end) : 0;
      if (c1 >= 0xdc00 && c1 < 0xe000) --end;
      return error(begin, end, state, -1, -1);
    }

    if (nonbmp)
    {
      for (var i = result >> 9; i > 0; --i)
      {
        --end;
        var c1 = end < size ? input.charCodeAt(end) : 0;
        if (c1 >= 0xdc00 && c1 < 0xe000) --end;
      }
    }
    else
    {
      end -= result >> 9;
    }

    return (result & 511) - 1;
  }
}

JSONiqParser.getTokenSet = function(tokenSetId)
{
  var set = [];
  var s = tokenSetId < 0 ? - tokenSetId : INITIAL[tokenSetId] & 8191;
  for (var i = 0; i < 289; i += 32)
  {
    var j = i;
    var i0 = (i >> 5) * 4317 + s - 1;
    var i1 = i0 >> 2;
    var i2 = i1 >> 2;
    var f = JSONiqParser.EXPECTED[(i0 & 3) + JSONiqParser.EXPECTED[(i1 & 3) + JSONiqParser.EXPECTED[(i2 & 15) + JSONiqParser.EXPECTED[i2 >> 4]]]];
    for ( ; f != 0; f >>>= 1, ++j)
    {
      if ((f & 1) != 0)
      {
        set.push(JSONiqParser.TOKEN[j]);
      }
    }
  }
  return set;
};

JSONiqParser.MAP0 =
[ 71, 0, 0, 0, 0, 0, 0, 0, 0, 1, 1, 0, 0, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 17, 17, 17, 17, 17, 17, 17, 17, 17, 18, 19, 20, 21, 22, 23, 24, 25, 26, 27, 28, 29, 26, 30, 30, 30, 30, 30, 31, 32, 33, 30, 30, 34, 30, 30, 35, 30, 30, 30, 36, 30, 30, 37, 38, 39, 40, 30, 40, 41, 42, 43, 44, 45, 46, 47, 48, 49, 50, 51, 52, 53, 54, 55, 56, 57, 58, 59, 60, 61, 62, 63, 64, 65, 66, 67, 68, 69, 40, 40
];

JSONiqParser.MAP1 =
[ 108, 124, 214, 214, 214, 214, 214, 214, 214, 214, 214, 214, 214, 214, 214, 214, 156, 181, 181, 181, 181, 181, 214, 215, 213, 214, 214, 214, 214, 214, 214, 214, 214, 214, 214, 214, 214, 214, 214, 214, 214, 214, 214, 214, 214, 214, 214, 214, 214, 214, 214, 214, 214, 214, 214, 214, 214, 214, 214, 214, 214, 214, 214, 214, 214, 214, 214, 214, 214, 214, 214, 214, 214, 214, 214, 214, 214, 214, 214, 214, 214, 214, 214, 214, 214, 214, 214, 214, 214, 214, 214, 214, 214, 214, 214, 214, 214, 214, 214, 214, 214, 214, 214, 214, 214, 214, 214, 214, 247, 261, 277, 293, 309, 355, 371, 387, 423, 423, 423, 415, 339, 331, 339, 331, 339, 339, 339, 339, 339, 339, 339, 339, 339, 339, 339, 339, 339, 339, 339, 339, 440, 440, 440, 440, 440, 440, 440, 324, 339, 339, 339, 339, 339, 339, 339, 339, 401, 423, 423, 424, 422, 423, 423, 339, 339, 339, 339, 339, 339, 339, 339, 339, 339, 339, 339, 339, 339, 339, 339, 339, 339, 423, 423, 423, 423, 423, 423, 423, 423, 423, 423, 423, 423, 423, 423, 423, 423, 423, 423, 423, 423, 423, 423, 423, 423, 423, 423, 423, 423, 423, 423, 423, 423, 338, 339, 339, 339, 339, 339, 339, 339, 339, 339, 339, 339, 339, 339, 339, 339, 339, 339, 339, 339, 339, 339, 339, 339, 339, 339, 339, 339, 339, 339, 339, 339, 339, 423, 71, 0, 0, 0, 0, 0, 0, 0, 0, 1, 1, 0, 0, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 17, 17, 17, 17, 17, 17, 17, 17, 17, 18, 19, 20, 21, 22, 23, 24, 25, 26, 27, 28, 29, 26, 30, 30, 30, 30, 30, 31, 32, 33, 30, 30, 30, 30, 30, 30, 30, 30, 30, 30, 30, 30, 30, 30, 40, 30, 30, 30, 30, 30, 30, 30, 30, 30, 30, 30, 30, 30, 30, 30, 30, 30, 34, 30, 30, 35, 30, 30, 30, 36, 30, 30, 37, 38, 39, 40, 30, 40, 41, 42, 43, 44, 45, 46, 47, 48, 49, 50, 51, 52, 53, 54, 55, 56, 57, 58, 59, 60, 61, 62, 63, 64, 65, 66, 67, 68, 69, 40, 40, 40, 40, 40, 40, 40, 40, 40, 40, 40, 40, 30, 30, 40, 40, 40, 40, 40, 40, 40, 70, 40, 40, 40, 40, 40, 40, 40, 40, 40, 40, 40, 40, 40, 40, 40, 40, 70, 70, 70, 70, 70, 70, 70, 70, 70, 70, 70, 70, 70, 70, 70, 70, 70
];

JSONiqParser.MAP2 =
[ 57344, 63744, 64976, 65008, 65536, 983040, 63743, 64975, 65007, 65533, 983039, 1114111, 40, 30, 40, 30, 30, 40
];

JSONiqParser.INITIAL =
[ 1, 24578, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20, 21, 22, 23, 24, 25, 26, 27, 28, 29, 30, 31, 32, 33, 34, 35, 36, 37, 38, 39, 40, 41, 42, 43, 44, 45, 46, 47, 48, 49, 50, 51, 52, 53, 54, 55, 56, 57, 58, 59, 60, 61, 62, 63, 64, 65, 66, 67, 68, 69, 70, 71, 72, 73, 74, 75, 76, 77, 78, 79, 80, 81, 82, 83, 84, 85, 86, 87, 88, 89, 90, 91, 92, 93, 94, 95, 96, 97, 98, 99, 100, 101, 102, 103, 104, 105, 106, 107, 108, 109, 110, 111, 112, 113, 114, 115, 116, 117, 118, 119, 120, 121, 122, 123, 124, 125, 126, 127, 128, 129, 130, 131, 132, 133, 134, 135, 136, 137, 138, 139, 140, 141, 142, 143, 144, 145, 146, 147, 148, 149, 150, 151, 152, 153, 154, 155, 156, 157, 158, 159, 160, 161, 162, 163, 164, 165, 166, 167, 168, 169, 170, 171, 172, 173, 174, 175, 176, 177, 178, 179, 180, 181, 182, 183, 184, 185, 186, 187, 188, 189, 190, 191, 192, 193, 194, 195, 196, 197, 198, 199, 200, 201, 202, 203, 204, 205, 206, 207, 208, 209, 210, 211, 212, 213, 214, 215, 216, 217, 218, 219, 220, 221, 222, 223, 224, 225, 226, 227, 228, 229, 230, 231, 232, 233, 234, 235, 236, 237, 238, 239, 240, 241, 242, 243, 244, 245, 246, 247, 248, 249, 250, 251, 252, 253, 254, 255, 256, 257, 258, 259, 260, 261, 262, 263, 264, 265, 266, 267, 268, 269, 270, 271, 272, 273, 274, 275, 276, 277, 278, 279, 280, 281, 282, 283, 284, 285, 286, 287, 288, 289, 290
];

JSONiqParser.TRANSITION =
[ 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 36864, 36899, 36883, 36883, 36883, 36903, 36883, 36883, 36883, 36883, 36883, 36926, 36919, 36883, 36883, 36879, 36883, 36883, 36942, 36991, 37318, 37318, 41289, 37087, 37318, 37318, 37010, 37318, 37318, 37318, 37318, 37318, 65657, 40192, 38525, 37318, 37318, 39335, 40192, 38527, 37031, 44584, 37054, 37076, 37318, 37318, 74250, 37318, 69924, 37107, 37136, 37152, 65815, 69390, 37247, 37265, 53482, 37182, 37116, 37202, 37162, 37222, 37186, 37120, 37206, 37166, 74726, 69388, 37245, 37263, 53480, 37281, 37318, 41092, 37317, 37318, 37318, 45006, 57380, 37335, 37318, 74671, 64449, 74683, 40193, 37351, 37378, 40192, 39199, 37741, 69458, 41098, 41041, 37318, 46044, 37882, 69459, 41099, 41042, 37318, 46045, 37371, 74376, 37355, 37382, 40192, 39202, 38523, 74469, 50180, 37398, 37414, 37436, 50351, 37475, 37318, 58014, 37496, 39340, 37628, 37651, 37674, 55492, 37717, 38230, 37546, 37566, 58028, 41986, 37605, 37523, 37542, 37562, 58024, 37582, 37601, 39304, 37621, 37644, 37667, 37690, 37709, 37733, 74273, 57770, 38317, 37757, 37779, 37822, 46961, 37943, 37979, 37995, 38150, 38170, 38011, 38093, 38038, 38061, 37792, 39434, 38084, 38100, 38045, 38068, 37799, 39441, 38116, 37992, 38147, 38166, 38186, 38022, 38246, 38263, 37318, 38304, 37318, 38354, 38370, 45231, 38397, 38413, 37861, 37459, 38432, 38451, 68861, 37456, 38429, 38448, 38467, 45226, 38483, 38407, 38516, 57783, 51848, 38545, 38580, 73050, 38677, 38610, 38626, 49222, 38643, 38660, 49221, 38642, 38659, 38676, 38693, 38624, 38714, 38739, 38791, 38841, 38876, 38942, 38892, 60443, 38908, 60433, 60449, 38914, 38930, 38946, 38962, 38978, 39013, 39048, 39078, 39105, 39169, 39156, 39172, 39188, 39089, 39218, 39249, 55484, 39265, 39291, 55476, 39311, 39327, 37693, 37318, 65659, 38698, 39356, 39410, 37835, 37848, 39457, 45051, 38498, 38500, 45058, 39494, 39520, 39559, 39504, 39575, 59063, 39606, 39616, 39632, 39681, 39692, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 39708, 37318, 37318, 37318, 37318, 48276, 37318, 37318, 37318, 37318, 37318, 67919, 57964, 39727, 39739, 37318, 37318, 72403, 39755, 36991, 37318, 37318, 41289, 37087, 37318, 37318, 37010, 39794, 37318, 37318, 37318, 37318, 65657, 40192, 38525, 37318, 37318, 39813, 40192, 39835, 37031, 44584, 37054, 37076, 37318, 37318, 74250, 37318, 69924, 37107, 37136, 37152, 65815, 69390, 37247, 37265, 53482, 37182, 37116, 37202, 37162, 37222, 37186, 37120, 37206, 37166, 74726, 69388, 37245, 37263, 53480, 37281, 37318, 41092, 37317, 37318, 37318, 45006, 57380, 37335, 37318, 74671, 64449, 74683, 40193, 37351, 37378, 40192, 39199, 37741, 69458, 41098, 41041, 37318, 46044, 37882, 69459, 41099, 41042, 37318, 46045, 37371, 74376, 37355, 37382, 40192, 39202, 38523, 74469, 50180, 37398, 37414, 37436, 50351, 37475, 37318, 58014, 37496, 39340, 37628, 37651, 37674, 55492, 37717, 38230, 37546, 37566, 58028, 41986, 37605, 37523, 37542, 37562, 58024, 37582, 37601, 39304, 37621, 37644, 37667, 37690, 37709, 37733, 74273, 57770, 38317, 37757, 37779, 37822, 46961, 37943, 37979, 37995, 38150, 38170, 38011, 38093, 38038, 38061, 37792, 39434, 38084, 38100, 38045, 38068, 37799, 39441, 38116, 37992, 38147, 38166, 38186, 38022, 38246, 38263, 37318, 38304, 37318, 38354, 38370, 45231, 38397, 38413, 37861, 37459, 38432, 38451, 68861, 37456, 38429, 38448, 38467, 45226, 38483, 38407, 38516, 57783, 51848, 38545, 38580, 73050, 38677, 38610, 38626, 49222, 38643, 38660, 49221, 38642, 38659, 38676, 38693, 38624, 38714, 38739, 38791, 38841, 38876, 38942, 38892, 60443, 38908, 60433, 60449, 38914, 38930, 38946, 38962, 38978, 39013, 39048, 39078, 39105, 39169, 39156, 39172, 39188, 39089, 39218, 39249, 55484, 39265, 39291, 55476, 39311, 39327, 37693, 37318, 65659, 38698, 39356, 39410, 37835, 37848, 39457, 45051, 38498, 38500, 45058, 39494, 39520, 39559, 39504, 39575, 59063, 39606, 39616, 39632, 39681, 39692, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 39854, 39870, 37318, 37318, 37318, 46325, 37318, 37318, 36994, 37318, 61657, 45847, 57964, 37318, 37318, 61650, 62405, 62409, 50496, 39889, 37318, 37318, 41289, 37087, 37318, 44009, 37010, 37318, 37318, 37318, 37318, 37318, 65657, 40192, 38525, 37318, 37318, 39335, 40192, 45833, 37031, 44584, 37054, 37076, 37318, 37318, 74250, 37318, 69924, 37107, 37136, 37152, 65815, 69390, 37247, 37265, 53482, 37182, 37116, 37202, 37162, 37222, 37186, 37120, 37206, 37166, 74726, 69388, 37245, 37263, 53480, 37281, 37318, 41092, 37317, 37318, 37318, 45006, 57380, 37335, 37318, 74671, 64449, 74683, 40193, 37351, 37378, 40192, 39199, 37741, 69458, 41098, 41041, 37318, 46044, 37882, 69459, 41099, 41042, 37318, 46045, 37371, 74376, 37355, 37382, 40192, 39202, 38523, 74469, 50180, 37398, 37414, 37436, 50351, 37475, 37318, 58014, 37496, 39340, 37628, 37651, 37674, 55492, 37717, 38230, 37546, 37566, 58028, 41986, 37605, 37523, 37542, 37562, 58024, 37582, 37601, 39304, 37621, 37644, 37667, 37690, 37709, 37733, 74273, 57770, 38317, 37757, 37779, 37822, 46961, 37943, 37979, 37995, 38150, 38170, 38011, 38093, 38038, 38061, 37792, 39434, 38084, 38100, 38045, 38068, 37799, 39441, 38116, 37992, 38147, 38166, 38186, 38022, 38246, 38263, 37318, 38304, 37318, 38354, 38370, 45231, 38397, 38413, 37861, 37459, 38432, 38451, 68861, 37456, 38429, 38448, 38467, 45226, 38483, 38407, 38516, 57783, 51848, 38545, 38580, 73050, 38677, 38610, 38626, 49222, 38643, 38660, 49221, 38642, 38659, 38676, 38693, 38624, 38714, 38739, 38791, 38841, 38876, 38942, 38892, 60443, 38908, 60433, 60449, 38914, 38930, 38946, 38962, 38978, 39013, 39048, 39078, 39105, 39169, 39156, 39172, 39188, 39089, 39218, 39249, 55484, 39265, 39291, 55476, 39311, 39327, 37693, 37318, 65659, 38698, 39356, 39410, 37835, 37848, 39457, 45051, 38498, 38500, 45058, 39494, 39520, 39559, 39504, 39575, 59063, 39606, 39616, 39632, 39681, 39692, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 39908, 62747, 37318, 37318, 37318, 42026, 65655, 37318, 73568, 65651, 65646, 39935, 62736, 68791, 45102, 45088, 45095, 68779, 39950, 36991, 37318, 37318, 41289, 39994, 37318, 45426, 37010, 37318, 37318, 37318, 37318, 37318, 65657, 40192, 38525, 37318, 37318, 39335, 40192, 39370, 40016, 44584, 37054, 37076, 37318, 37318, 74250, 37318, 69924, 37107, 37136, 37152, 65815, 69390, 37247, 37265, 53482, 37182, 37116, 37202, 37162, 37222, 37186, 37120, 37206, 37166, 74726, 69388, 37245, 37263, 53480, 37281, 37318, 41092, 37317, 37318, 37318, 45006, 57380, 37335, 37318, 74671, 64449, 74683, 40193, 37351, 37378, 40192, 39199, 40039, 69458, 41098, 41041, 37318, 46044, 44842, 69459, 41099, 41042, 37318, 46045, 37371, 74376, 37355, 37382, 40192, 39202, 38523, 74469, 50180, 37398, 37414, 37436, 50351, 37475, 37318, 58014, 37496, 39340, 37628, 37651, 37674, 55492, 37717, 38230, 37546, 37566, 58028, 41986, 37605, 37523, 37542, 37562, 58024, 37582, 37601, 39304, 37621, 37644, 37667, 37690, 37709, 37733, 74273, 57770, 38317, 37757, 37779, 37822, 46961, 37943, 37979, 37995, 38150, 38170, 38011, 38093, 38038, 38061, 37792, 39434, 38084, 38100, 38045, 38068, 37799, 39441, 38116, 37992, 38147, 38166, 38186, 38022, 38246, 38263, 37318, 38304, 37318, 38354, 38370, 45231, 38397, 38413, 37861, 37459, 38432, 38451, 68861, 37456, 38429, 38448, 38467, 45226, 38483, 38407, 38516, 57783, 51848, 38545, 38580, 73050, 38677, 38610, 38626, 49222, 38643, 38660, 49221, 38642, 38659, 38676, 38693, 38624, 38714, 38739, 38791, 38841, 38876, 38942, 38892, 60443, 38908, 60433, 60449, 38914, 38930, 38946, 38962, 38978, 39013, 39048, 39078, 39105, 39169, 39156, 39172, 39188, 39089, 39218, 39249, 55484, 39265, 39291, 55476, 39311, 39327, 37693, 37318, 65659, 38698, 39356, 39410, 37835, 37848, 39457, 45051, 38498, 38500, 45058, 39494, 39520, 39559, 39504, 39575, 59063, 39606, 39616, 39632, 39681, 39692, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 39708, 47580, 37318, 37318, 37318, 48276, 40073, 37318, 37015, 71422, 71423, 40094, 57964, 37318, 37318, 67783, 40130, 40134, 40150, 36991, 37318, 37318, 41289, 37087, 37318, 37318, 37010, 37318, 37318, 37318, 37318, 37318, 65657, 40192, 38525, 37318, 37318, 40186, 40192, 38527, 37031, 44584, 37054, 37076, 37318, 37318, 74250, 37318, 69924, 37107, 37136, 37152, 65815, 69390, 37247, 37265, 53482, 37182, 37116, 37202, 37162, 37222, 37186, 37120, 37206, 37166, 74726, 69388, 37245, 37263, 53480, 37281, 37318, 41092, 37317, 37318, 37318, 45006, 57380, 37335, 37318, 74671, 64449, 74683, 40193, 37351, 37378, 40192, 39199, 37741, 69458, 41098, 41041, 37318, 46044, 37882, 69459, 41099, 41042, 37318, 46045, 37371, 74376, 37355, 37382, 40192, 39202, 38523, 74469, 50180, 37398, 37414, 37436, 50351, 37475, 37318, 58014, 37496, 39340, 37628, 37651, 37674, 55492, 37717, 38230, 37546, 37566, 58028, 41986, 37605, 37523, 37542, 37562, 58024, 37582, 37601, 39304, 37621, 37644, 37667, 37690, 37709, 37733, 74273, 57770, 38317, 37757, 37779, 37822, 46961, 37943, 37979, 37995, 38150, 38170, 38011, 38093, 38038, 38061, 37792, 39434, 38084, 38100, 38045, 38068, 37799, 39441, 38116, 37992, 38147, 38166, 38186, 38022, 38246, 38263, 37318, 38304, 37318, 38354, 38370, 45231, 38397, 38413, 37861, 37459, 38432, 38451, 68861, 37456, 38429, 38448, 38467, 45226, 38483, 38407, 38516, 57783, 51848, 38545, 38580, 73050, 38677, 38610, 38626, 49222, 38643, 38660, 49221, 38642, 38659, 38676, 38693, 38624, 38714, 38739, 38791, 38841, 38876, 38942, 38892, 60443, 38908, 60433, 60449, 38914, 38930, 38946, 38962, 38978, 39013, 39048, 39078, 39105, 39169, 39156, 39172, 39188, 39089, 39218, 39249, 55484, 39265, 39291, 55476, 39311, 39327, 37693, 37318, 65659, 38698, 39356, 39410, 37835, 37848, 39457, 45051, 38498, 38500, 45058, 39494, 39520, 39559, 39504, 39575, 59063, 39606, 39616, 39632, 39681, 39692, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 39708, 37318, 37318, 37318, 37318, 48276, 71599, 37318, 37318, 37318, 37318, 67919, 40209, 37318, 37318, 66772, 40222, 40225, 40241, 36991, 37318, 37318, 41289, 37087, 37318, 37318, 37010, 37318, 37318, 37318, 37318, 37318, 65657, 40192, 38525, 37318, 37318, 39335, 40192, 38527, 37031, 44584, 37054, 37076, 37318, 37318, 74250, 37318, 69924, 37107, 37136, 37152, 65815, 69390, 37247, 37265, 53482, 37182, 37116, 37202, 37162, 37222, 37186, 37120, 37206, 37166, 74726, 69388, 37245, 37263, 53480, 37281, 37318, 41092, 37317, 37318, 37318, 45006, 57380, 37335, 37318, 74671, 64449, 74683, 40193, 37351, 37378, 40192, 39199, 37741, 69458, 41098, 41041, 37318, 46044, 37882, 69459, 41099, 41042, 37318, 46045, 37371, 74376, 37355, 37382, 40192, 39202, 38523, 74469, 50180, 37398, 37414, 37436, 50351, 37475, 37318, 58014, 37496, 39340, 37628, 37651, 37674, 55492, 37717, 38230, 37546, 37566, 58028, 41986, 37605, 37523, 37542, 37562, 58024, 37582, 37601, 39304, 37621, 37644, 37667, 37690, 37709, 37733, 74273, 57770, 38317, 37757, 37779, 37822, 46961, 37943, 37979, 37995, 38150, 38170, 38011, 38093, 38038, 38061, 37792, 39434, 38084, 38100, 38045, 38068, 37799, 39441, 38116, 37992, 38147, 38166, 38186, 38022, 38246, 38263, 37318, 38304, 37318, 38354, 38370, 45231, 38397, 38413, 37861, 37459, 38432, 38451, 68861, 37456, 38429, 38448, 38467, 45226, 38483, 38407, 38516, 57783, 51848, 38545, 38580, 73050, 38677, 38610, 38626, 49222, 38643, 38660, 49221, 38642, 38659, 38676, 38693, 38624, 38714, 38739, 38791, 38841, 38876, 38942, 38892, 60443, 38908, 60433, 60449, 38914, 38930, 38946, 38962, 38978, 39013, 39048, 39078, 39105, 39169, 39156, 39172, 39188, 39089, 39218, 39249, 55484, 39265, 39291, 55476, 39311, 39327, 37693, 37318, 65659, 38698, 39356, 39410, 37835, 37848, 39457, 45051, 38498, 38500, 45058, 39494, 39520, 39559, 39504, 39575, 59063, 39606, 39616, 39632, 39681, 39692, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 39708, 37318, 37318, 37318, 37318, 48276, 37318, 37318, 37318, 37318, 37318, 71606, 73347, 37318, 37318, 37318, 37318, 37318, 40283, 40324, 37318, 37318, 41289, 37087, 37318, 37318, 37010, 37318, 37318, 37318, 37318, 37318, 65657, 40192, 38525, 37318, 37318, 39335, 40192, 38527, 37031, 44584, 37054, 37076, 37318, 37318, 74250, 37318, 69924, 37107, 37136, 37152, 65815, 69390, 37247, 37265, 53482, 40344, 37116, 37202, 37162, 40378, 37186, 37120, 37206, 37166, 74726, 69388, 37245, 37263, 53480, 37281, 37318, 41092, 37317, 37318, 37318, 45006, 57380, 37335, 37318, 74671, 64449, 74683, 40193, 37351, 37378, 40192, 39199, 37741, 69458, 41098, 41041, 37318, 46044, 37882, 69459, 41099, 41042, 37318, 46045, 37371, 74376, 37355, 37382, 40192, 39202, 38523, 74469, 50180, 37398, 37414, 37436, 50351, 37475, 37318, 58014, 37496, 39340, 37628, 37651, 37674, 55492, 37717, 38230, 37546, 37566, 58028, 41986, 37605, 37523, 37542, 37562, 58024, 37582, 37601, 39304, 37621, 37644, 37667, 37690, 37709, 37733, 74273, 57770, 38317, 37757, 37779, 37822, 46961, 37943, 37979, 37995, 38150, 38170, 38011, 38093, 38038, 38061, 37792, 39434, 38084, 38100, 38045, 38068, 37799, 39441, 38116, 37992, 38147, 38166, 38186, 38022, 38246, 38263, 37318, 38304, 37318, 38354, 38370, 45231, 38397, 38413, 37861, 37459, 38432, 38451, 68861, 37456, 38429, 38448, 38467, 45226, 38483, 38407, 38516, 57783, 51848, 38545, 38580, 73050, 38677, 38610, 38626, 49222, 38643, 38660, 49221, 38642, 38659, 38676, 38693, 38624, 38714, 38739, 38791, 38841, 38876, 38942, 38892, 60443, 38908, 60433, 60449, 38914, 38930, 38946, 38962, 38978, 39013, 39048, 39078, 39105, 39169, 39156, 39172, 39188, 39089, 39218, 39249, 55484, 39265, 39291, 55476, 39311, 39327, 37693, 37318, 65659, 38698, 39356, 39410, 37835, 37848, 39457, 45051, 38498, 38500, 45058, 39494, 39520, 39559, 39504, 39575, 59063, 39606, 39616, 39632, 39681, 39692, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 40416, 40432, 37318, 37318, 37318, 48315, 37318, 37318, 37526, 37318, 62612, 40451, 57964, 37318, 37318, 62605, 65082, 65086, 40487, 40535, 37318, 37318, 41289, 37087, 37318, 37318, 40554, 37318, 37318, 37318, 37318, 37318, 65657, 40192, 38525, 37318, 37318, 39335, 40192, 38131, 40575, 44584, 37054, 37076, 37318, 37318, 74250, 37318, 69924, 37107, 37136, 37152, 65815, 69390, 37247, 37265, 53482, 37182, 37116, 37202, 37162, 37222, 37186, 37120, 37206, 37166, 74726, 69388, 37245, 37263, 53480, 37281, 37318, 41092, 37317, 37318, 37318, 45006, 57380, 37335, 37318, 74671, 64449, 74683, 40193, 37351, 37378, 40192, 39199, 37741, 69458, 41098, 41041, 37318, 46044, 37882, 69459, 41099, 41042, 37318, 46045, 37371, 74376, 37355, 37382, 40192, 39202, 38523, 74469, 50180, 37398, 37414, 37436, 50351, 37475, 37318, 58014, 37496, 39340, 37628, 37651, 37674, 55492, 37717, 38230, 37546, 37566, 58028, 41986, 37605, 37523, 37542, 37562, 58024, 37582, 37601, 39304, 37621, 37644, 37667, 37690, 37709, 37733, 74273, 57770, 38317, 37757, 37779, 37822, 46961, 37943, 37979, 37995, 38150, 38170, 38011, 38093, 38038, 38061, 37792, 39434, 38084, 38100, 38045, 38068, 37799, 39441, 38116, 37992, 38147, 38166, 38186, 38022, 38246, 38263, 37318, 38304, 37318, 38354, 38370, 45231, 38397, 38413, 37861, 37459, 38432, 38451, 68861, 37456, 38429, 38448, 38467, 45226, 38483, 38407, 38516, 57783, 51848, 38545, 38580, 73050, 38677, 38610, 38626, 49222, 38643, 38660, 49221, 38642, 38659, 38676, 38693, 38624, 38714, 38739, 38791, 38841, 38876, 38942, 38892, 60443, 38908, 60433, 60449, 38914, 38930, 38946, 38962, 38978, 39013, 39048, 39078, 39105, 39169, 39156, 39172, 39188, 39089, 39218, 39249, 55484, 39265, 39291, 55476, 39311, 39327, 37693, 37318, 65659, 38698, 39356, 39410, 37835, 37848, 39457, 45051, 38498, 38500, 45058, 39494, 39520, 39559, 39504, 39575, 59063, 39606, 39616, 39632, 39681, 39692, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 40598, 40656, 40697, 40697, 40697, 40663, 40736, 40697, 40698, 40727, 40642, 40679, 40626, 40693, 40745, 40613, 40752, 40714, 40768, 36991, 37318, 37318, 40817, 40815, 37318, 37318, 37010, 37318, 37318, 37318, 37318, 37318, 65657, 40192, 38525, 37318, 37318, 39335, 40192, 38527, 37031, 44584, 37054, 40804, 37318, 37318, 74250, 37318, 69924, 37107, 37136, 37152, 65815, 69390, 37247, 37265, 53482, 37182, 37116, 37202, 37162, 37222, 37186, 37120, 37206, 37166, 74726, 69388, 37245, 37263, 53480, 37281, 37318, 41092, 37317, 37318, 37318, 45006, 57380, 37335, 37318, 74671, 64449, 74683, 40193, 37351, 37378, 40192, 39199, 37741, 69458, 41098, 41041, 37318, 46044, 37882, 69459, 41099, 41042, 37318, 46045, 37371, 74376, 37355, 37382, 40192, 39202, 38523, 74469, 50180, 37398, 37414, 37436, 50351, 37475, 37318, 58014, 37496, 39340, 37628, 37651, 37674, 55492, 37717, 38230, 37546, 37566, 58028, 41986, 37605, 37523, 37542, 37562, 58024, 37582, 37601, 39304, 37621, 37644, 37667, 37690, 37709, 37733, 74273, 57770, 38317, 37757, 37779, 37822, 46961, 37943, 37979, 37995, 38150, 38170, 38011, 38093, 38038, 38061, 37792, 39434, 38084, 38100, 38045, 38068, 37799, 39441, 38116, 37992, 38147, 38166, 38186, 38022, 38246, 38263, 37318, 38304, 37318, 38354, 38370, 45231, 38397, 38413, 37861, 37459, 38432, 38451, 68861, 37456, 38429, 38448, 38467, 45226, 38483, 38407, 38516, 57783, 51848, 38545, 38580, 73050, 38677, 38610, 38626, 49222, 38643, 38660, 49221, 38642, 38659, 38676, 38693, 38624, 38714, 38739, 38791, 38841, 38876, 38942, 38892, 60443, 38908, 60433, 60449, 38914, 38930, 38946, 38962, 38978, 39013, 39048, 39078, 39105, 39169, 39156, 39172, 39188, 39089, 39218, 39249, 55484, 39265, 39291, 55476, 39311, 39327, 37693, 37318, 65659, 38698, 39356, 39410, 37835, 37848, 39457, 45051, 38498, 38500, 45058, 39494, 39520, 39559, 39504, 39575, 59063, 39606, 39616, 39632, 39681, 39692, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 39708, 43152, 37318, 37318, 37318, 48276, 40833, 37318, 37318, 73511, 37318, 67919, 57964, 40868, 40872, 43155, 43156, 43784, 40888, 36991, 37318, 37318, 41289, 40924, 37318, 37318, 37010, 37318, 37318, 37318, 37318, 37318, 65657, 40192, 38525, 37318, 37318, 39335, 40192, 38527, 37031, 44584, 37054, 40942, 37318, 37318, 74250, 37318, 69924, 37107, 37136, 37152, 65815, 69390, 37247, 37265, 53482, 37182, 37116, 37202, 37162, 37222, 37186, 37120, 37206, 37166, 74726, 69388, 37245, 37263, 53480, 37281, 37318, 41092, 37317, 37318, 37318, 45006, 57380, 37335, 37318, 74671, 64449, 74683, 40193, 37351, 37378, 40192, 39199, 37741, 69458, 41098, 41041, 37318, 46044, 37882, 69459, 41099, 41042, 37318, 46045, 37371, 74376, 37355, 37382, 40192, 39202, 38523, 74469, 50180, 37398, 37414, 37436, 50351, 37475, 37318, 58014, 37496, 39340, 37628, 37651, 37674, 55492, 37717, 38230, 37546, 37566, 58028, 41986, 37605, 37523, 37542, 37562, 58024, 37582, 37601, 39304, 37621, 37644, 37667, 37690, 37709, 37733, 74273, 57770, 38317, 37757, 37779, 37822, 46961, 37943, 37979, 37995, 38150, 38170, 38011, 38093, 38038, 38061, 37792, 39434, 38084, 38100, 38045, 38068, 37799, 39441, 38116, 37992, 38147, 38166, 38186, 38022, 38246, 38263, 37318, 38304, 37318, 38354, 38370, 45231, 38397, 38413, 37861, 37459, 38432, 38451, 68861, 37456, 38429, 38448, 38467, 45226, 38483, 38407, 38516, 57783, 51848, 38545, 38580, 73050, 38677, 38610, 38626, 49222, 38643, 38660, 49221, 38642, 38659, 38676, 38693, 38624, 38714, 38739, 38791, 38841, 38876, 38942, 38892, 60443, 38908, 60433, 60449, 38914, 38930, 38946, 38962, 38978, 39013, 39048, 39078, 39105, 39169, 39156, 39172, 39188, 39089, 39218, 39249, 55484, 39265, 39291, 55476, 39311, 39327, 37693, 37318, 65659, 38698, 39356, 39410, 37835, 37848, 39457, 45051, 38498, 38500, 45058, 39494, 39520, 39559, 39504, 39575, 59063, 39606, 39616, 39632, 39681, 39692, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 39708, 43775, 37318, 37318, 37318, 48276, 37318, 37318, 37318, 37318, 69945, 73273, 57964, 40977, 41007, 44794, 41023, 73756, 40991, 36991, 37318, 37318, 41289, 37087, 37318, 37318, 37010, 37318, 37318, 37318, 37318, 37318, 65657, 40192, 38525, 37318, 37318, 39335, 40192, 38527, 37031, 44584, 37054, 37076, 37318, 37318, 74250, 37318, 69924, 37107, 37136, 37152, 65815, 69390, 37247, 37265, 53482, 37182, 37116, 37202, 37162, 41058, 37186, 37120, 37206, 37166, 74726, 69388, 37245, 37263, 53480, 37281, 37318, 41092, 37317, 37318, 37318, 45006, 57380, 37335, 37318, 74671, 64449, 74683, 40193, 37351, 37378, 40192, 39199, 37741, 69458, 41098, 41041, 37318, 46044, 37882, 69459, 41099, 41042, 37318, 46045, 37371, 74376, 37355, 37382, 40192, 39202, 38523, 74469, 50180, 37398, 37414, 37436, 50351, 37475, 37318, 58014, 37496, 39340, 37628, 37651, 37674, 55492, 37717, 38230, 37546, 37566, 58028, 41986, 37605, 37523, 37542, 37562, 58024, 37582, 37601, 39304, 37621, 37644, 37667, 37690, 37709, 37733, 74273, 57770, 38317, 37757, 37779, 37822, 46961, 37943, 37979, 37995, 38150, 38170, 38011, 38093, 38038, 38061, 37792, 39434, 38084, 38100, 38045, 38068, 37799, 39441, 38116, 37992, 38147, 38166, 38186, 38022, 38246, 38263, 37318, 38304, 37318, 38354, 38370, 45231, 38397, 38413, 37861, 37459, 38432, 38451, 68861, 37456, 38429, 38448, 38467, 45226, 38483, 38407, 38516, 57783, 51848, 38545, 38580, 73050, 38677, 38610, 38626, 49222, 38643, 38660, 49221, 38642, 38659, 38676, 38693, 38624, 38714, 38739, 38791, 38841, 38876, 38942, 38892, 60443, 38908, 60433, 60449, 38914, 38930, 38946, 38962, 38978, 39013, 39048, 39078, 39105, 39169, 39156, 39172, 39188, 39089, 39218, 39249, 55484, 39265, 39291, 55476, 39311, 39327, 37693, 37318, 65659, 38698, 39356, 39410, 37835, 37848, 39457, 45051, 38498, 38500, 45058, 39494, 39520, 39559, 39504, 39575, 59063, 39606, 39616, 39632, 39681, 39692, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 39708, 37318, 37318, 37318, 37318, 48276, 37318, 37318, 37318, 37318, 45645, 38564, 57964, 41115, 41135, 37318, 41119, 41135, 41149, 36991, 37318, 37318, 41289, 37087, 37318, 37318, 37010, 37318, 37318, 37318, 37318, 37318, 65657, 40192, 38525, 37318, 37318, 39335, 40192, 38527, 37031, 44584, 37054, 37076, 37318, 37318, 37301, 37318, 69924, 37107, 37136, 37152, 65815, 69390, 37247, 37265, 53482, 37182, 37116, 37202, 37162, 37222, 37186, 37120, 37206, 37166, 74726, 69388, 37245, 37263, 53480, 37281, 37318, 41092, 37317, 37318, 37318, 45006, 57380, 37335, 37318, 74671, 64449, 74683, 40193, 37351, 37378, 40192, 39199, 37741, 69458, 41098, 41041, 37318, 46044, 37882, 69459, 41099, 41042, 37318, 46045, 37371, 74376, 37355, 37382, 40192, 39202, 38523, 74469, 50180, 37398, 37414, 37436, 50351, 37475, 37318, 58014, 37496, 39340, 37628, 37651, 37674, 55492, 37717, 38230, 37546, 37566, 58028, 41986, 37605, 37523, 37542, 37562, 58024, 37582, 37601, 39304, 37621, 37644, 37667, 37690, 37709, 37733, 74273, 57770, 38317, 37757, 37779, 37822, 46961, 37943, 37979, 37995, 38150, 38170, 38011, 38093, 38038, 38061, 37792, 39434, 38084, 38100, 38045, 38068, 37799, 39441, 38116, 37992, 38147, 38166, 38186, 38022, 38246, 38263, 37318, 38304, 37318, 38354, 38370, 45231, 38397, 38413, 37861, 37459, 38432, 38451, 68861, 37456, 38429, 38448, 38467, 45226, 38483, 38407, 38516, 57783, 51848, 38545, 38580, 73050, 38677, 38610, 38626, 49222, 38643, 38660, 49221, 38642, 38659, 38676, 38693, 38624, 38714, 38739, 38791, 38841, 38876, 38942, 38892, 60443, 38908, 60433, 60449, 38914, 38930, 38946, 38962, 38978, 39013, 39048, 39078, 39105, 39169, 39156, 39172, 39188, 39089, 39218, 39249, 55484, 39265, 39291, 55476, 39311, 39327, 37693, 37318, 65659, 38698, 39356, 39410, 37835, 37848, 39457, 45051, 38498, 38500, 45058, 39494, 39520, 39559, 39504, 39575, 59063, 39606, 39616, 39632, 39681, 39692, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 39708, 62620, 37318, 37318, 37318, 48276, 62315, 37318, 37318, 42112, 62617, 51150, 68655, 68669, 68671, 37318, 37318, 74000, 41165, 36991, 37318, 37318, 41289, 37087, 37318, 37318, 37010, 37318, 37318, 37318, 37318, 37318, 65657, 40192, 38525, 37318, 37318, 39335, 40192, 38527, 37031, 44584, 37054, 37076, 37318, 37318, 74250, 37318, 69924, 37107, 37136, 37152, 65815, 69390, 37247, 37265, 53482, 37182, 37116, 37202, 37162, 37222, 37186, 37120, 37206, 37166, 74726, 69388, 37245, 37263, 53480, 37281, 37318, 41092, 37317, 37318, 37318, 45006, 57380, 37335, 37318, 74671, 64449, 74683, 40193, 37351, 37378, 40192, 39199, 37741, 69458, 41098, 41041, 37318, 46044, 37882, 69459, 41099, 41042, 37318, 46045, 37371, 74376, 37355, 37382, 40192, 39202, 38523, 74469, 50180, 37398, 37414, 37436, 50351, 37475, 37318, 58014, 37496, 39340, 37628, 37651, 37674, 55492, 37717, 38230, 37546, 37566, 58028, 41986, 37605, 37523, 37542, 37562, 58024, 37582, 37601, 39304, 37621, 37644, 37667, 37690, 37709, 37733, 74273, 57770, 38317, 37757, 37779, 37822, 46961, 37943, 37979, 37995, 38150, 38170, 38011, 38093, 38038, 38061, 37792, 39434, 38084, 38100, 38045, 38068, 37799, 39441, 38116, 37992, 38147, 38166, 38186, 38022, 38246, 38263, 37318, 38304, 37318, 38354, 38370, 45231, 38397, 38413, 37861, 37459, 38432, 38451, 68861, 37456, 38429, 38448, 38467, 45226, 38483, 38407, 38516, 57783, 51848, 38545, 38580, 73050, 38677, 38610, 38626, 49222, 38643, 38660, 49221, 38642, 38659, 38676, 38693, 38624, 38714, 38739, 38791, 38841, 38876, 38942, 38892, 60443, 38908, 60433, 60449, 38914, 38930, 38946, 38962, 38978, 39013, 39048, 39078, 39105, 39169, 39156, 39172, 39188, 39089, 39218, 39249, 55484, 39265, 39291, 55476, 39311, 39327, 37693, 37318, 65659, 38698, 39356, 39410, 37835, 37848, 39457, 45051, 38498, 38500, 45058, 39494, 39520, 39559, 39504, 39575, 59063, 39606, 39616, 39632, 39681, 39692, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 41215, 37318, 37318, 37318, 37318, 48276, 37318, 37318, 37318, 37318, 37318, 67919, 57964, 41238, 41258, 37318, 41242, 41258, 41272, 36991, 37318, 37318, 41289, 37087, 37318, 37318, 37010, 37318, 37318, 37318, 37318, 37318, 44404, 72644, 50028, 41369, 65498, 47282, 62003, 69205, 37318, 62433, 37318, 68839, 45752, 41288, 37301, 44633, 37318, 37318, 37318, 37318, 73164, 72644, 41305, 72644, 72644, 41324, 38775, 41542, 41542, 58552, 65498, 52179, 65498, 65498, 52319, 41461, 41342, 41461, 41461, 60376, 37318, 37318, 37318, 37318, 37318, 37318, 57380, 44576, 38329, 37318, 37318, 73164, 72644, 72644, 72644, 72644, 72644, 41363, 41542, 41542, 41542, 41542, 41542, 65493, 65498, 65498, 65498, 65498, 65498, 46825, 41461, 41461, 41461, 41461, 41461, 47041, 57802, 45001, 37318, 37318, 37318, 37318, 37318, 62575, 37318, 37318, 71516, 41385, 72644, 72644, 72644, 72645, 41405, 67865, 41542, 58857, 41542, 41542, 46917, 69302, 65498, 55881, 65498, 65498, 53308, 58621, 41461, 41722, 41461, 41461, 47863, 54854, 37318, 37318, 37318, 37318, 64802, 37318, 37318, 72642, 48855, 72644, 72644, 72644, 38767, 41542, 41439, 41542, 41542, 41444, 65497, 64981, 65498, 65498, 65498, 59803, 41461, 41460, 41461, 41461, 47863, 54854, 37318, 37318, 69109, 41478, 37318, 60498, 72644, 72644, 70521, 58547, 41542, 41542, 60742, 46770, 65498, 65498, 73471, 59798, 41461, 41461, 70944, 47288, 64766, 37318, 37318, 37507, 37318, 71005, 72644, 60920, 59569, 41542, 41542, 72922, 65498, 65498, 48896, 41461, 41461, 46878, 37318, 60844, 41498, 41222, 67506, 72644, 41518, 41541, 58729, 41559, 65498, 46559, 41593, 41462, 56939, 50451, 37318, 66947, 41614, 41423, 73664, 58829, 59801, 41638, 49498, 41664, 52412, 41686, 41702, 41738, 63218, 38594, 66255, 64873, 73961, 46828, 73161, 59759, 73663, 59797, 47605, 39543, 41802, 41825, 41848, 60496, 73658, 47451, 47606, 53268, 62159, 62339, 66862, 41867, 41898, 41909, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 39708, 37318, 37318, 37318, 37318, 48276, 37318, 37318, 37318, 37318, 37318, 45787, 57964, 41925, 72170, 37318, 38278, 38288, 41945, 36991, 37318, 37318, 41289, 37087, 37318, 37318, 41981, 37318, 37318, 37318, 37318, 37318, 65657, 40192, 38525, 37318, 37318, 69382, 40192, 38527, 37031, 44584, 37054, 37076, 37318, 37318, 74250, 37318, 69924, 37107, 37136, 37152, 65815, 69390, 37247, 37265, 53482, 37182, 37116, 37202, 37162, 37222, 37186, 37120, 37206, 37166, 74726, 69388, 37245, 37263, 53480, 37281, 37318, 41092, 37317, 37318, 37318, 45006, 57380, 37335, 37318, 74671, 64449, 74683, 40193, 37351, 37378, 40192, 39199, 37741, 69458, 41098, 41041, 37318, 46044, 37882, 69459, 41099, 41042, 37318, 46045, 37371, 74376, 37355, 37382, 40192, 39202, 38523, 74469, 50180, 37398, 37414, 37436, 50351, 37475, 37318, 58014, 37496, 39340, 37628, 37651, 37674, 55492, 37717, 38230, 37546, 37566, 58028, 41986, 37605, 37523, 37542, 37562, 58024, 37582, 37601, 39304, 37621, 37644, 37667, 37690, 37709, 37733, 74273, 57770, 38317, 37757, 37779, 37822, 46961, 37943, 37979, 37995, 38150, 38170, 38011, 38093, 38038, 38061, 37792, 39434, 38084, 38100, 38045, 38068, 37799, 39441, 38116, 37992, 38147, 38166, 38186, 38022, 38246, 38263, 37318, 38304, 37318, 38354, 38370, 45231, 38397, 38413, 37861, 37459, 38432, 38451, 68861, 37456, 38429, 38448, 38467, 45226, 38483, 38407, 38516, 57783, 51848, 38545, 38580, 73050, 38677, 38610, 38626, 49222, 38643, 38660, 49221, 38642, 38659, 38676, 38693, 38624, 38714, 38739, 38791, 38841, 38876, 38942, 38892, 60443, 38908, 60433, 60449, 38914, 38930, 38946, 38962, 38978, 39013, 39048, 39078, 39105, 39169, 39156, 39172, 39188, 39089, 39218, 39249, 55484, 39265, 39291, 55476, 39311, 39327, 37693, 37318, 65659, 38698, 39356, 39410, 37835, 37848, 39457, 45051, 38498, 38500, 45058, 39494, 39520, 39559, 39504, 39575, 59063, 39606, 39616, 39632, 39681, 39692, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 39708, 42002, 37318, 37318, 37318, 45026, 37318, 37318, 37318, 37318, 37318, 67919, 57964, 42022, 71555, 37318, 43019, 43028, 71570, 36991, 37318, 37318, 41289, 37087, 37318, 37318, 37010, 42042, 37318, 73045, 37318, 37318, 65657, 40192, 38525, 37318, 37318, 39335, 40192, 45833, 37031, 44584, 37054, 37076, 37318, 37318, 74250, 37318, 69924, 37107, 37136, 37152, 65815, 69390, 37247, 37265, 53482, 37182, 37116, 37202, 37162, 37222, 37186, 37120, 37206, 37166, 74726, 69388, 37245, 37263, 53480, 37281, 37318, 41092, 37317, 37318, 37318, 45006, 57380, 37335, 37318, 74671, 64449, 74683, 40193, 37351, 37378, 40192, 39199, 37741, 69458, 41098, 41041, 37318, 46044, 37882, 69459, 41099, 41042, 37318, 46045, 37371, 74376, 37355, 37382, 40192, 39202, 38523, 74469, 50180, 37398, 37414, 37436, 50351, 37475, 37318, 58014, 37496, 39340, 37628, 37651, 37674, 55492, 37717, 38230, 37546, 37566, 58028, 41986, 37605, 37523, 37542, 37562, 58024, 37582, 37601, 39304, 37621, 37644, 37667, 37690, 37709, 37733, 74273, 57770, 38317, 37757, 37779, 37822, 46961, 37943, 37979, 37995, 38150, 38170, 38011, 38093, 38038, 38061, 37792, 39434, 38084, 38100, 38045, 38068, 37799, 39441, 38116, 37992, 38147, 38166, 38186, 38022, 38246, 38263, 37318, 38304, 37318, 38354, 38370, 45231, 38397, 38413, 37861, 37459, 38432, 38451, 68861, 37456, 38429, 38448, 38467, 45226, 38483, 38407, 38516, 57783, 51848, 38545, 38580, 73050, 38677, 38610, 38626, 49222, 38643, 38660, 49221, 38642, 38659, 38676, 38693, 38624, 38714, 38739, 38791, 38841, 38876, 38942, 38892, 60443, 38908, 60433, 60449, 38914, 38930, 38946, 38962, 38978, 39013, 39048, 39078, 39105, 39169, 39156, 39172, 39188, 39089, 39218, 39249, 55484, 39265, 39291, 55476, 39311, 39327, 37693, 37318, 65659, 38698, 39356, 39410, 37835, 37848, 39457, 45051, 38498, 38500, 45058, 39494, 39520, 39559, 39504, 39575, 59063, 39606, 39616, 39632, 39681, 39692, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 39708, 42061, 37318, 37318, 37318, 48276, 37318, 37318, 37318, 37318, 37318, 50713, 57964, 37318, 37318, 37318, 38993, 38997, 42078, 42128, 37318, 37318, 41289, 37087, 37318, 37318, 42147, 37318, 37318, 37318, 37318, 37318, 44404, 72644, 50028, 41369, 65498, 42168, 62003, 69205, 37318, 37318, 37318, 68839, 37318, 37318, 39275, 37318, 37318, 37318, 37318, 37318, 73164, 72644, 72644, 72644, 72644, 41324, 41542, 41542, 41542, 58552, 65498, 65498, 65498, 65498, 57854, 41461, 41461, 41461, 41461, 64567, 42190, 37318, 37318, 37318, 37318, 37318, 42212, 37318, 37318, 37318, 37318, 73164, 72644, 72644, 72644, 72644, 72644, 41363, 41542, 41542, 41542, 41542, 41542, 65493, 65498, 65498, 65498, 65498, 65498, 46825, 41461, 41461, 41461, 41461, 41461, 67436, 42249, 37318, 37318, 37318, 37318, 37318, 44403, 37318, 37318, 37318, 72643, 72644, 72644, 72644, 72644, 72187, 38772, 41542, 41542, 41542, 41542, 41542, 72266, 65498, 65498, 65498, 65498, 65498, 59800, 41461, 41461, 41461, 41461, 41461, 47863, 51497, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 72642, 72644, 72644, 72644, 72644, 42270, 41542, 41542, 41542, 41542, 49331, 65497, 65498, 65498, 65498, 65498, 59803, 41461, 41461, 41461, 41461, 47863, 47501, 37318, 37318, 37318, 37318, 37318, 60498, 72644, 72644, 72644, 42291, 41542, 41542, 41542, 59595, 65498, 65498, 65498, 59798, 41461, 41461, 41461, 47288, 37318, 37318, 37318, 37318, 37318, 72644, 72644, 60920, 41542, 41542, 41542, 65498, 65498, 65498, 63673, 41461, 41461, 56936, 37318, 37318, 37318, 73163, 72644, 72644, 50031, 41542, 73662, 65498, 65498, 59802, 41461, 41462, 56939, 61615, 37318, 72643, 60923, 41542, 73664, 65498, 59801, 41461, 49498, 72946, 72644, 50030, 54383, 59799, 54339, 61620, 60922, 73660, 47450, 46828, 73161, 59759, 73663, 59797, 47605, 60497, 50030, 47449, 46829, 60496, 73658, 47451, 47606, 53268, 62159, 62339, 66862, 41867, 41898, 41909, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 39708, 63809, 37318, 37318, 37318, 49242, 42389, 37318, 37318, 42312, 41767, 44524, 59934, 42356, 42367, 45136, 42383, 44513, 42405, 42445, 37318, 37318, 40926, 42420, 37318, 37318, 37010, 37318, 37318, 37318, 37318, 37318, 65657, 40192, 44142, 43528, 42807, 74542, 43714, 44329, 37031, 44584, 37054, 42466, 37318, 37318, 74250, 37318, 69924, 37107, 37136, 37152, 65815, 69390, 37247, 37265, 53482, 42495, 44053, 42520, 44234, 42536, 42631, 42566, 42582, 42617, 43323, 74548, 42732, 42652, 43181, 43457, 37318, 41092, 37317, 37318, 37318, 45006, 57380, 37335, 37318, 74671, 64449, 74683, 40193, 37351, 37378, 40192, 39199, 74695, 44148, 43301, 42504, 44146, 43812, 43314, 42636, 43906, 44107, 42807, 42594, 42699, 42706, 42722, 43750, 42901, 42677, 43141, 74469, 50180, 37398, 37414, 37436, 50351, 37475, 37318, 58014, 37496, 39340, 37628, 37651, 37674, 55492, 37717, 42748, 42962, 43618, 43603, 43820, 43354, 42764, 44271, 43892, 42788, 42804, 42826, 43918, 42855, 43210, 42882, 42898, 42920, 44134, 44337, 57770, 38317, 37757, 37779, 37822, 46961, 37943, 37979, 37995, 38150, 38170, 38011, 42949, 44040, 42992, 42978, 43271, 43008, 43044, 42839, 43084, 44099, 43110, 43126, 43172, 43197, 43740, 43442, 43226, 43242, 38263, 37318, 38304, 37318, 38354, 38370, 45231, 38397, 38413, 43259, 44205, 43287, 43339, 74705, 42601, 43379, 43395, 43411, 42683, 43427, 43493, 43509, 57783, 51848, 38545, 38580, 73050, 38677, 38610, 43544, 43363, 43588, 43648, 42772, 43664, 43059, 43680, 43709, 43730, 43766, 38739, 38791, 38841, 38876, 38942, 38892, 43800, 43836, 43632, 43878, 43941, 43957, 42665, 43973, 43989, 39013, 39048, 39078, 44025, 44069, 44085, 43094, 44123, 43693, 44164, 39249, 55484, 44195, 43559, 43068, 43925, 39327, 39819, 44240, 42810, 42904, 39356, 44221, 43849, 44282, 42866, 45051, 43520, 43862, 42933, 39494, 44256, 44298, 44314, 39590, 43572, 44353, 42550, 44369, 44420, 44431, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 39708, 37091, 37318, 37318, 37318, 48276, 39140, 44447, 37318, 67184, 67184, 50155, 62190, 71955, 71957, 37318, 37089, 71946, 44465, 36991, 37318, 37318, 41289, 37087, 37318, 37318, 37010, 37318, 37318, 37318, 37318, 37318, 65657, 40192, 38525, 37318, 37318, 39335, 40192, 38527, 37031, 44584, 37054, 37076, 37318, 37318, 74250, 37318, 69924, 37107, 37136, 37152, 65815, 69390, 37247, 37265, 53482, 37182, 37116, 37202, 37162, 37222, 37186, 37120, 37206, 37166, 74726, 69388, 37245, 37263, 53480, 37281, 37318, 41092, 37317, 37318, 37318, 45006, 44502, 37335, 37318, 74671, 64449, 74683, 40193, 37351, 37378, 40192, 39199, 37741, 69458, 41098, 41041, 37318, 46044, 37882, 69459, 41099, 41042, 37318, 46045, 37371, 74376, 37355, 37382, 40192, 39202, 38381, 36956, 50180, 37398, 37414, 37436, 50351, 44540, 37318, 58014, 37496, 39340, 37628, 37651, 37674, 55492, 37717, 38230, 37546, 37566, 58028, 41986, 37605, 37523, 37542, 37562, 58024, 37582, 37601, 39304, 37621, 37644, 37667, 37690, 37709, 37733, 44565, 57770, 38317, 37757, 37779, 37822, 46961, 37943, 37979, 37995, 38150, 38170, 38011, 44600, 38038, 38061, 37792, 39434, 44649, 38100, 38045, 38068, 37799, 39441, 38116, 37992, 38147, 38166, 38186, 38022, 38246, 38263, 37318, 38304, 37318, 38354, 38370, 45231, 38397, 38413, 44665, 37459, 38432, 38451, 37229, 37456, 38429, 38448, 38467, 45226, 38483, 38407, 38516, 57783, 51848, 38545, 38580, 73050, 38677, 38610, 38626, 49222, 38643, 38660, 49221, 38642, 38659, 38676, 38693, 38624, 38714, 38739, 38791, 38841, 38876, 38942, 38892, 60443, 38908, 60433, 60449, 38914, 38930, 38946, 38962, 38978, 39013, 39048, 39078, 39105, 39169, 39156, 39172, 39188, 39089, 39218, 39249, 55484, 39265, 39291, 55476, 39311, 39327, 37693, 37318, 65659, 38698, 39356, 39410, 37835, 37848, 39457, 45051, 38498, 38500, 45058, 39494, 39520, 39559, 39504, 39575, 59063, 39606, 39616, 39632, 39681, 39692, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 39708, 37318, 37318, 37318, 37318, 48276, 37318, 37318, 37318, 37318, 37318, 37318, 73156, 44700, 44701, 37318, 74225, 74229, 44717, 36991, 37318, 37318, 41289, 37087, 37318, 37318, 37010, 37318, 37318, 46650, 37318, 37318, 65657, 40192, 38525, 37318, 37318, 39335, 40192, 44756, 37031, 44584, 37054, 37076, 37318, 37318, 74250, 37318, 69924, 37107, 37136, 37152, 65815, 69390, 37247, 37265, 53482, 37182, 37116, 37202, 37162, 37222, 37186, 37120, 37206, 37166, 74726, 69388, 37245, 37263, 53480, 37281, 37318, 41092, 37317, 37318, 37318, 45006, 57380, 37335, 37318, 74671, 64449, 74683, 40193, 37351, 37378, 40192, 39199, 37741, 69458, 41098, 41041, 37318, 46044, 37882, 69459, 41099, 41042, 37318, 46045, 37371, 74376, 37355, 37382, 40192, 39202, 38523, 74469, 50180, 37398, 37414, 37436, 50351, 37475, 37318, 58014, 37496, 39340, 37628, 37651, 37674, 55492, 37717, 38230, 37546, 37566, 58028, 41986, 37605, 37523, 37542, 37562, 58024, 37582, 37601, 39304, 37621, 37644, 37667, 37690, 37709, 37733, 74273, 57770, 38317, 37757, 37779, 37822, 46961, 37943, 37979, 37995, 38150, 38170, 38011, 38093, 38038, 38061, 37792, 39434, 38084, 38100, 38045, 38068, 37799, 39441, 38116, 37992, 38147, 38166, 38186, 38022, 38246, 38263, 37318, 38304, 37318, 38354, 38370, 45231, 38397, 38413, 37861, 37459, 38432, 38451, 68861, 37456, 38429, 38448, 38467, 45226, 38483, 38407, 38516, 57783, 51848, 38545, 38580, 73050, 38677, 38610, 38626, 49222, 38643, 38660, 49221, 38642, 38659, 38676, 38693, 38624, 38714, 38739, 38791, 38841, 38876, 38942, 38892, 60443, 38908, 60433, 60449, 38914, 38930, 38946, 38962, 38978, 39013, 39048, 39078, 39105, 39169, 39156, 39172, 39188, 39089, 39218, 39249, 55484, 39265, 39291, 55476, 39311, 39327, 37693, 37318, 65659, 38698, 39356, 39410, 37835, 37848, 39457, 45051, 38498, 38500, 45058, 39494, 39520, 39559, 39504, 39575, 59063, 39606, 39616, 39632, 39681, 39692, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 44775, 67647, 37318, 37318, 37318, 48276, 37318, 37318, 37318, 37318, 37318, 67919, 57964, 44810, 44811, 37318, 37318, 74488, 44827, 44876, 37318, 37318, 41289, 37087, 37318, 37318, 37010, 37318, 37318, 44898, 37318, 37318, 65657, 40192, 38525, 37318, 37318, 39335, 40192, 44921, 37031, 44584, 37054, 37076, 37318, 37318, 74250, 37318, 69924, 37107, 37136, 37152, 65815, 69390, 37247, 37265, 53482, 37182, 37116, 37202, 37162, 37222, 37186, 37120, 37206, 37166, 74726, 69388, 37245, 37263, 53480, 37281, 37318, 41092, 37317, 37318, 37318, 45006, 57380, 37335, 37318, 74671, 64449, 74683, 40193, 37351, 37378, 40192, 39199, 37741, 69458, 41098, 41041, 37318, 46044, 37882, 69459, 41099, 41042, 37318, 46045, 37371, 74376, 37355, 37382, 40192, 39202, 38523, 74469, 50180, 37398, 37414, 37436, 50351, 37475, 37318, 58014, 37496, 39340, 37628, 37651, 37674, 55492, 37717, 38230, 37546, 37566, 58028, 41986, 37605, 37523, 37542, 37562, 58024, 37582, 37601, 39304, 37621, 37644, 37667, 37690, 37709, 37733, 74273, 57770, 38317, 37757, 37779, 37822, 46961, 37943, 37979, 37995, 38150, 38170, 38011, 38093, 38038, 38061, 37792, 39434, 38084, 38100, 38045, 38068, 37799, 39441, 38116, 37992, 38147, 38166, 38186, 38022, 38246, 38263, 37318, 38304, 37318, 38354, 38370, 45231, 38397, 38413, 37861, 37459, 38432, 38451, 68861, 37456, 38429, 38448, 38467, 45226, 38483, 38407, 38516, 57783, 51848, 38545, 38580, 73050, 38677, 38610, 38626, 49222, 38643, 38660, 49221, 38642, 38659, 38676, 38693, 38624, 38714, 38739, 38791, 38841, 38876, 38942, 38892, 60443, 38908, 60433, 60449, 38914, 38930, 38946, 38962, 38978, 39013, 39048, 39078, 39105, 39169, 39156, 39172, 39188, 39089, 39218, 39249, 55484, 39265, 39291, 55476, 39311, 39327, 37693, 37318, 65659, 38698, 39356, 39410, 37835, 37848, 39457, 45051, 38498, 38500, 45058, 39494, 39520, 39559, 39504, 39575, 59063, 39606, 39616, 39632, 39681, 39692, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 44940, 44949, 37318, 37318, 37318, 45580, 37318, 37318, 37318, 37318, 37318, 67919, 57964, 44969, 44970, 37318, 37318, 68795, 44986, 45022, 37318, 37318, 41289, 37087, 37318, 37318, 37010, 37318, 37318, 72142, 37318, 37318, 65657, 40192, 38525, 37318, 37318, 39335, 40192, 38201, 37031, 44584, 37054, 37076, 37318, 37318, 74250, 37318, 69924, 37107, 37136, 37152, 65815, 69390, 37247, 37265, 53482, 37182, 37116, 37202, 37162, 37222, 37186, 37120, 37206, 37166, 74726, 69388, 37245, 37263, 53480, 37281, 37318, 41092, 37317, 37318, 37318, 45006, 57380, 37335, 37318, 74671, 64449, 74683, 40193, 37351, 37378, 40192, 39199, 37741, 69458, 41098, 41041, 37318, 46044, 37882, 69459, 41099, 41042, 37318, 46045, 37371, 74376, 37355, 37382, 40192, 39202, 38523, 74469, 50180, 37398, 37414, 37436, 50351, 37475, 37318, 58014, 37496, 39340, 37628, 37651, 37674, 55492, 37717, 38230, 37546, 37566, 58028, 41986, 37605, 37523, 37542, 37562, 58024, 37582, 37601, 39304, 37621, 37644, 37667, 37690, 37709, 37733, 74273, 57770, 38317, 37757, 37779, 37822, 46961, 37943, 37979, 37995, 38150, 38170, 38011, 38093, 38038, 38061, 37792, 39434, 38084, 38100, 38045, 38068, 37799, 39441, 38116, 37992, 38147, 38166, 38186, 38022, 38246, 38263, 37318, 38304, 37318, 38354, 38370, 45231, 38397, 38413, 37861, 37459, 38432, 38451, 68861, 37456, 38429, 38448, 38467, 45226, 38483, 38407, 38516, 57783, 51848, 38545, 38580, 73050, 38677, 38610, 38626, 49222, 38643, 38660, 49221, 38642, 38659, 38676, 38693, 38624, 38714, 38739, 38791, 38841, 38876, 38942, 38892, 60443, 38908, 60433, 60449, 38914, 38930, 38946, 38962, 38978, 39013, 39048, 39078, 39105, 39169, 39156, 39172, 39188, 39089, 39218, 39249, 55484, 39265, 39291, 55476, 39311, 45042, 37693, 37318, 65659, 38698, 39356, 39410, 37835, 37848, 39457, 45051, 38498, 38500, 45058, 39494, 39520, 39559, 39504, 39575, 59063, 39606, 39616, 39632, 39681, 39692, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 45074, 37318, 37318, 37318, 37318, 48276, 59955, 37318, 37318, 37318, 59954, 73317, 57964, 37318, 45118, 37318, 37318, 45127, 45152, 36991, 37318, 37318, 41289, 37087, 37318, 37318, 37010, 45201, 37318, 37318, 37318, 37318, 65657, 40192, 38525, 37318, 37318, 45220, 40192, 45247, 37031, 44584, 37054, 37076, 37318, 37318, 74250, 37318, 69924, 37107, 37136, 37152, 65815, 69390, 37247, 37265, 53482, 37182, 37116, 37202, 37162, 37222, 37186, 37120, 37206, 37166, 74726, 69388, 37245, 37263, 53480, 37281, 37318, 41092, 37317, 37318, 37318, 45006, 57380, 37335, 37318, 74671, 64449, 74683, 40193, 37351, 37378, 40192, 39199, 37741, 69458, 41098, 41041, 37318, 46044, 37882, 69459, 41099, 41042, 37318, 46045, 37371, 74376, 37355, 37382, 40192, 39202, 38523, 74469, 50180, 37398, 37414, 37436, 50351, 37475, 37318, 58014, 37496, 39340, 37628, 37651, 37674, 55492, 37717, 38230, 37546, 37566, 58028, 41986, 37605, 37523, 37542, 37562, 58024, 37582, 37601, 39304, 37621, 37644, 37667, 37690, 37709, 37733, 74273, 57770, 38317, 37757, 37779, 37822, 46961, 37943, 37979, 37995, 38150, 38170, 38011, 38093, 38038, 38061, 37792, 39434, 38084, 38100, 38045, 38068, 37799, 39441, 38116, 37992, 38147, 38166, 38186, 38022, 38246, 38263, 37318, 38304, 37318, 38354, 38370, 45231, 38397, 38413, 37861, 37459, 38432, 38451, 68861, 37456, 38429, 38448, 38467, 45226, 38483, 38407, 38516, 57783, 51848, 38545, 38580, 73050, 38677, 38610, 38626, 49222, 38643, 38660, 49221, 38642, 38659, 38676, 38693, 38624, 38714, 38739, 38791, 38841, 38876, 38942, 38892, 60443, 38908, 60433, 60449, 38914, 38930, 38946, 38962, 38978, 39013, 39048, 39078, 39105, 39169, 39156, 39172, 39188, 39089, 39218, 39249, 55484, 39265, 39291, 55476, 39311, 39327, 37693, 37318, 65659, 38698, 39356, 39410, 37835, 37848, 39457, 45051, 38498, 38500, 45058, 39494, 39520, 39559, 39504, 39575, 59063, 39606, 39616, 39632, 39681, 39692, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 39708, 37318, 37318, 37318, 37318, 48276, 37318, 37318, 37318, 37318, 37318, 67919, 57964, 37318, 37318, 37318, 46105, 37318, 45266, 36991, 37318, 37318, 41289, 37087, 37318, 37318, 37010, 37318, 37318, 37318, 37318, 37318, 65657, 40192, 38525, 37318, 37318, 39335, 40192, 38527, 37031, 44584, 37054, 37076, 37318, 37318, 74250, 37318, 69924, 37107, 37136, 37152, 65815, 69390, 37247, 37265, 53482, 37182, 37116, 37202, 37162, 37222, 37186, 37120, 37206, 37166, 74726, 69388, 37245, 37263, 53480, 37281, 37318, 41092, 37317, 37318, 37318, 45006, 57380, 37335, 37318, 74671, 64449, 74683, 40193, 37351, 37378, 40192, 39199, 37741, 69458, 41098, 41041, 37318, 46044, 37882, 69459, 41099, 41042, 37318, 46045, 37371, 74376, 37355, 37382, 40192, 39202, 38523, 74469, 50180, 37398, 37414, 37436, 50351, 37475, 37318, 58014, 37496, 39340, 37628, 37651, 37674, 55492, 37717, 38230, 37546, 37566, 58028, 41986, 37605, 37523, 37542, 37562, 58024, 37582, 37601, 39304, 37621, 37644, 37667, 37690, 37709, 37733, 74273, 57770, 38317, 37757, 37779, 37822, 46961, 37943, 37979, 37995, 38150, 38170, 38011, 38093, 38038, 38061, 37792, 39434, 38084, 38100, 38045, 38068, 37799, 39441, 38116, 37992, 38147, 38166, 38186, 38022, 38246, 38263, 37318, 38304, 37318, 38354, 38370, 45231, 38397, 38413, 37861, 37459, 38432, 38451, 68861, 37456, 38429, 38448, 38467, 45226, 38483, 38407, 38516, 57783, 51848, 38545, 38580, 73050, 38677, 38610, 38626, 49222, 38643, 38660, 49221, 38642, 38659, 38676, 38693, 38624, 38714, 38739, 38791, 38841, 38876, 38942, 38892, 60443, 38908, 60433, 60449, 38914, 38930, 38946, 38962, 38978, 39013, 39048, 39078, 39105, 39169, 39156, 39172, 39188, 39089, 39218, 39249, 55484, 39265, 39291, 55476, 39311, 39327, 37693, 37318, 65659, 38698, 39356, 39410, 37835, 37848, 39457, 45051, 38498, 38500, 45058, 39494, 39520, 39559, 39504, 39575, 59063, 39606, 39616, 39632, 39681, 39692, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 45302, 37318, 37318, 37318, 37318, 44953, 37318, 37318, 37318, 37318, 37318, 67919, 57964, 37318, 37318, 45323, 45350, 63443, 47000, 65689, 37318, 37318, 41289, 37087, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 44404, 72644, 50028, 41369, 65498, 47282, 62003, 72217, 37318, 37318, 37318, 68839, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 73164, 72644, 72644, 72644, 72644, 45386, 41542, 41542, 41542, 52709, 65498, 65498, 65498, 65498, 57854, 41461, 41461, 41461, 41461, 64567, 41960, 37318, 37318, 37318, 37318, 37318, 44399, 37318, 37318, 37318, 37318, 73164, 72644, 72644, 72644, 72644, 72644, 41363, 41542, 41542, 41542, 41542, 41542, 65493, 65498, 65498, 65498, 65498, 65498, 46825, 41461, 41461, 41461, 41461, 41461, 63696, 45404, 37318, 37318, 37318, 37318, 37318, 44403, 37318, 37318, 37318, 72643, 72644, 72644, 72644, 72644, 72645, 38772, 41542, 41542, 41542, 41542, 41542, 45363, 65498, 65498, 65498, 65498, 65498, 59800, 41461, 41461, 41461, 41461, 41461, 47863, 51497, 37318, 37318, 37318, 37318, 47785, 37318, 37318, 72642, 72644, 72644, 72644, 72644, 61913, 41542, 41542, 41542, 41542, 48876, 65497, 65498, 65498, 65498, 65498, 59803, 41461, 41461, 41461, 41461, 47863, 47501, 37318, 37318, 37318, 37318, 37318, 60498, 72644, 72644, 72644, 42291, 41542, 41542, 41542, 59595, 65498, 65498, 65498, 59798, 41461, 41461, 41461, 47288, 37318, 37318, 37318, 45425, 37318, 72644, 72644, 60920, 41542, 41542, 41542, 65498, 65498, 65498, 63673, 41461, 41461, 56936, 37318, 37318, 37318, 73163, 72644, 72644, 50031, 41542, 73662, 65498, 65498, 59802, 41461, 41462, 56939, 61615, 37318, 72643, 60923, 41542, 73664, 65498, 59801, 41461, 49498, 72946, 72644, 50030, 54383, 59799, 54339, 61620, 60922, 73660, 47450, 46828, 73161, 59759, 73663, 59797, 47605, 60497, 50030, 47449, 46829, 60496, 73658, 47451, 47606, 53268, 62159, 62339, 66862, 41867, 41898, 41909, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 45302, 37318, 37318, 37318, 37318, 44953, 37318, 37318, 37318, 37318, 37318, 67919, 57964, 37318, 37318, 45323, 45350, 63443, 47000, 65689, 37318, 37318, 41289, 37087, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 44404, 72644, 50028, 41369, 65498, 47282, 62003, 72217, 37318, 37318, 37318, 68839, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 73164, 72644, 72644, 72644, 72644, 45386, 41542, 41542, 41542, 52709, 65498, 65498, 65498, 65498, 57854, 41461, 41461, 41461, 41461, 64567, 41960, 37318, 37318, 37318, 37318, 37318, 44399, 37318, 37318, 37318, 37318, 73164, 72644, 72644, 72644, 72644, 72644, 41363, 41542, 41542, 41542, 41542, 41542, 65493, 65498, 65498, 65498, 65498, 65498, 46825, 41461, 41461, 41461, 41461, 41461, 63696, 45404, 37318, 37318, 37318, 37318, 37318, 44403, 37318, 37318, 37318, 72643, 72644, 72644, 72644, 72644, 72645, 38772, 41542, 41542, 41542, 41542, 41542, 45363, 65498, 65498, 65498, 65498, 65498, 59800, 41461, 41461, 41461, 41461, 41461, 47863, 51497, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 72642, 72644, 72644, 72644, 72644, 61913, 41542, 41542, 41542, 41542, 48876, 65497, 65498, 65498, 65498, 65498, 59803, 41461, 41461, 41461, 41461, 47863, 47501, 37318, 37318, 37318, 37318, 37318, 60498, 72644, 72644, 72644, 42291, 41542, 41542, 41542, 59595, 65498, 65498, 65498, 59798, 41461, 41461, 41461, 47288, 37318, 37318, 37318, 37318, 37318, 72644, 72644, 60920, 41542, 41542, 41542, 65498, 65498, 65498, 63673, 41461, 41461, 56936, 37318, 37318, 37318, 73163, 72644, 72644, 50031, 41542, 73662, 65498, 65498, 59802, 41461, 41462, 56939, 61615, 37318, 72643, 60923, 41542, 73664, 65498, 59801, 41461, 49498, 72946, 72644, 50030, 54383, 59799, 54339, 61620, 60922, 73660, 47450, 46828, 73161, 59759, 73663, 59797, 47605, 60497, 50030, 47449, 46829, 60496, 73658, 47451, 47606, 53268, 62159, 62339, 66862, 41867, 41898, 41909, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 45302, 37318, 37318, 37318, 37318, 44953, 37318, 37318, 37318, 37318, 37318, 67919, 57964, 37318, 37318, 45323, 45350, 63443, 47000, 65689, 37318, 37318, 41289, 37087, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 44404, 72644, 50028, 41369, 65498, 47282, 62003, 72217, 37318, 37318, 37318, 68839, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 73164, 72644, 72644, 72644, 72644, 45386, 41542, 41542, 41542, 52709, 65498, 65498, 65498, 65498, 57854, 41461, 41461, 41461, 41461, 64567, 41960, 37318, 37318, 37318, 37318, 37318, 44399, 68819, 37318, 37318, 37318, 73164, 72644, 72644, 72644, 72644, 72644, 41363, 41542, 41542, 41542, 41542, 41542, 65493, 65498, 65498, 65498, 65498, 65498, 46825, 41461, 41461, 41461, 41461, 41461, 63696, 45404, 37318, 37318, 37318, 37318, 37318, 44403, 37318, 37318, 37318, 72643, 72644, 72644, 72644, 72644, 72645, 38772, 41542, 41542, 41542, 41542, 41542, 45363, 65498, 65498, 65498, 65498, 65498, 59800, 41461, 41461, 41461, 41461, 41461, 47863, 51497, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 72642, 72644, 72644, 72644, 72644, 61913, 41542, 41542, 41542, 41542, 48876, 65497, 65498, 65498, 65498, 65498, 59803, 41461, 41461, 41461, 41461, 47863, 47501, 37318, 37318, 37318, 37318, 37318, 60498, 72644, 72644, 72644, 42291, 41542, 41542, 41542, 59595, 65498, 65498, 65498, 59798, 41461, 41461, 41461, 47288, 37318, 37318, 37318, 37318, 37318, 72644, 72644, 60920, 41542, 41542, 41542, 65498, 65498, 65498, 63673, 41461, 41461, 56936, 37318, 37318, 37318, 73163, 72644, 72644, 50031, 41542, 73662, 65498, 65498, 59802, 41461, 41462, 56939, 61615, 37318, 72643, 60923, 41542, 73664, 65498, 59801, 41461, 49498, 72946, 72644, 50030, 54383, 59799, 54339, 61620, 60922, 73660, 47450, 46828, 73161, 59759, 73663, 59797, 47605, 60497, 50030, 47449, 46829, 60496, 73658, 47451, 47606, 53268, 62159, 62339, 66862, 41867, 41898, 41909, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 45302, 37318, 37318, 37318, 37318, 44953, 37318, 37318, 37318, 37318, 37318, 67919, 57964, 37318, 37318, 45323, 45350, 63443, 47000, 65689, 37318, 37318, 41289, 37087, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 44404, 72644, 50028, 41369, 65498, 47282, 62003, 72217, 37318, 37318, 37318, 68839, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 73164, 72644, 72644, 72644, 72644, 45386, 41542, 41542, 41542, 52709, 65498, 65498, 65498, 65498, 57854, 41461, 41461, 41461, 41461, 64567, 41960, 37318, 37318, 37318, 37318, 37318, 44399, 37318, 37318, 37318, 37318, 73164, 72644, 72644, 72644, 72644, 72644, 41363, 41542, 41542, 41542, 41542, 41542, 65493, 65498, 65498, 65498, 65498, 65498, 46825, 41461, 41461, 41461, 41461, 41461, 63696, 45404, 37318, 37318, 37318, 37318, 37318, 45442, 37318, 37318, 37318, 72643, 72644, 72644, 72644, 72644, 72645, 38772, 41542, 41542, 41542, 41542, 41542, 45363, 65498, 65498, 65498, 65498, 65498, 59800, 41461, 41461, 41461, 41461, 41461, 47863, 51497, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 72642, 72644, 72644, 72644, 72644, 61913, 41542, 41542, 41542, 41542, 48876, 65497, 65498, 65498, 65498, 65498, 59803, 41461, 41461, 41461, 41461, 47863, 47501, 37318, 37318, 37318, 37318, 37318, 60498, 72644, 72644, 72644, 42291, 41542, 41542, 41542, 59595, 65498, 65498, 65498, 59798, 41461, 41461, 41461, 47288, 37318, 37318, 37318, 37318, 37318, 72644, 72644, 60920, 41542, 41542, 41542, 65498, 65498, 65498, 63673, 41461, 41461, 56936, 37318, 37318, 37318, 73163, 72644, 72644, 50031, 41542, 73662, 65498, 65498, 59802, 41461, 41462, 56939, 61615, 37318, 72643, 60923, 41542, 73664, 65498, 59801, 41461, 49498, 72946, 72644, 50030, 54383, 59799, 54339, 61620, 60922, 73660, 47450, 46828, 73161, 59759, 73663, 59797, 47605, 60497, 50030, 47449, 46829, 60496, 73658, 47451, 47606, 53268, 62159, 62339, 66862, 41867, 41898, 41909, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 45302, 37318, 37318, 37318, 37318, 44953, 37318, 37318, 37318, 37318, 37318, 67919, 57964, 37318, 37318, 45323, 45350, 63443, 47000, 65689, 37318, 37318, 41289, 37087, 37318, 37318, 50781, 37318, 37318, 37318, 37318, 37318, 44404, 72644, 50028, 41369, 65498, 47282, 62003, 72217, 37318, 37318, 37318, 68839, 37318, 37318, 53518, 37318, 37318, 37318, 37318, 37318, 73164, 72644, 72644, 72644, 72644, 45386, 41542, 41542, 41542, 52709, 65498, 65498, 65498, 65498, 57854, 41461, 41461, 41461, 41461, 64567, 41960, 37318, 37318, 37318, 37318, 37318, 44399, 37318, 37318, 37318, 37318, 73164, 72644, 72644, 72644, 72644, 72644, 41363, 41542, 41542, 41542, 41542, 41542, 65493, 65498, 65498, 65498, 65498, 65498, 46825, 41461, 41461, 41461, 41461, 41461, 63696, 45404, 37318, 37318, 37318, 37318, 37318, 44403, 37318, 37318, 37318, 72643, 72644, 72644, 72644, 72644, 72645, 38772, 41542, 41542, 41542, 41542, 41542, 45363, 65498, 65498, 65498, 65498, 65498, 59800, 41461, 41461, 41461, 41461, 41461, 47863, 51497, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 72642, 72644, 72644, 72644, 72644, 61913, 41542, 41542, 41542, 41542, 48876, 65497, 65498, 65498, 65498, 65498, 59803, 41461, 41461, 41461, 41461, 47863, 47501, 37318, 37318, 37318, 37318, 37318, 60498, 72644, 72644, 72644, 42291, 41542, 41542, 41542, 59595, 65498, 65498, 65498, 59798, 41461, 41461, 41461, 47288, 37318, 37318, 37318, 37318, 37318, 72644, 72644, 60920, 41542, 41542, 41542, 65498, 65498, 65498, 63673, 41461, 41461, 56936, 37318, 37318, 37318, 73163, 72644, 72644, 50031, 41542, 73662, 65498, 65498, 59802, 41461, 41462, 56939, 61615, 37318, 72643, 60923, 41542, 73664, 65498, 59801, 41461, 49498, 72946, 72644, 50030, 54383, 59799, 54339, 61620, 60922, 73660, 47450, 46828, 73161, 59759, 73663, 59797, 47605, 60497, 50030, 47449, 46829, 60496, 73658, 47451, 47606, 53268, 62159, 62339, 66862, 41867, 41898, 41909, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 45302, 37318, 37318, 37318, 37318, 44953, 37318, 37318, 37318, 37318, 37318, 67919, 57964, 37318, 37318, 45323, 45350, 63443, 47000, 65689, 37318, 37318, 41289, 37087, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 44404, 72644, 50028, 41369, 65498, 47282, 62003, 72217, 37318, 37318, 37318, 68839, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 73164, 72644, 72644, 72644, 72644, 45386, 41542, 41542, 41542, 52709, 65498, 65498, 65498, 65498, 57854, 41461, 41461, 41461, 41461, 60376, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 73164, 72644, 72644, 72644, 72644, 72644, 41363, 41542, 41542, 41542, 41542, 41542, 65493, 65498, 65498, 65498, 65498, 65498, 46825, 41461, 41461, 41461, 41461, 41461, 47041, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 72643, 72644, 72644, 72644, 72644, 72645, 38772, 41542, 41542, 41542, 41542, 41542, 45363, 65498, 65498, 65498, 65498, 65498, 59800, 41461, 41461, 41461, 41461, 41461, 47863, 54854, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 72642, 72644, 72644, 72644, 72644, 38767, 41542, 41542, 41542, 41542, 41444, 65497, 65498, 65498, 65498, 65498, 59803, 41461, 41461, 41461, 41461, 47863, 54854, 37318, 37318, 37318, 37318, 37318, 60498, 72644, 72644, 72644, 58547, 41542, 41542, 41542, 46770, 65498, 65498, 65498, 59798, 41461, 41461, 41461, 47288, 37318, 37318, 37318, 37318, 37318, 72644, 72644, 60920, 41542, 41542, 41542, 65498, 65498, 65498, 63673, 41461, 41461, 56936, 37318, 37318, 37318, 73163, 72644, 72644, 50031, 41542, 73662, 65498, 65498, 59802, 41461, 41462, 56939, 61615, 37318, 72643, 60923, 41542, 73664, 65498, 59801, 41461, 49498, 72946, 72644, 50030, 54383, 59799, 54339, 61620, 60922, 73660, 47450, 46828, 73161, 59759, 73663, 59797, 47605, 60497, 50030, 47449, 46829, 60496, 73658, 47451, 47606, 53268, 62159, 62339, 66862, 41867, 41898, 41909, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 45302, 37318, 37318, 37318, 37318, 44953, 37318, 37318, 37318, 37318, 37318, 67919, 57964, 37318, 37318, 45323, 45350, 63443, 47000, 65689, 37318, 37318, 41289, 37087, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 44404, 72644, 50028, 41369, 65498, 47282, 62003, 65804, 37318, 37318, 37318, 68839, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 73164, 72644, 72644, 72644, 72644, 45386, 41542, 41542, 41542, 52709, 65498, 65498, 65498, 65498, 57854, 41461, 41461, 41461, 41461, 60376, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 73164, 72644, 72644, 72644, 72644, 72644, 41363, 41542, 41542, 41542, 41542, 41542, 65493, 65498, 65498, 65498, 65498, 65498, 46825, 41461, 41461, 41461, 41461, 41461, 47041, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 72643, 72644, 72644, 72644, 72644, 72645, 38772, 41542, 41542, 41542, 41542, 41542, 45363, 65498, 65498, 65498, 65498, 65498, 59800, 41461, 41461, 41461, 41461, 41461, 47863, 54854, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 72642, 72644, 72644, 72644, 72644, 38767, 41542, 41542, 41542, 41542, 41444, 65497, 65498, 65498, 65498, 65498, 59803, 41461, 41461, 41461, 41461, 47863, 54854, 37318, 37318, 37318, 37318, 37318, 60498, 72644, 72644, 72644, 58547, 41542, 41542, 41542, 46770, 65498, 65498, 65498, 59798, 41461, 41461, 41461, 47288, 37318, 37318, 37318, 37318, 37318, 72644, 72644, 60920, 41542, 41542, 41542, 65498, 65498, 65498, 63673, 41461, 41461, 56936, 37318, 37318, 37318, 73163, 72644, 72644, 50031, 41542, 73662, 65498, 65498, 59802, 41461, 41462, 56939, 61615, 37318, 72643, 60923, 41542, 73664, 65498, 59801, 41461, 49498, 72946, 72644, 50030, 54383, 59799, 54339, 61620, 60922, 73660, 47450, 46828, 73161, 59759, 73663, 59797, 47605, 60497, 50030, 47449, 46829, 60496, 73658, 47451, 47606, 53268, 62159, 62339, 66862, 41867, 41898, 41909, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 45302, 37318, 37318, 37318, 37318, 44953, 37318, 37318, 37318, 37318, 37318, 67919, 57964, 37318, 37318, 45323, 45350, 63443, 48653, 65689, 37318, 37318, 41289, 37087, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 44404, 72644, 50028, 41369, 65498, 47282, 62003, 72217, 37318, 37318, 37318, 68839, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 73164, 72644, 72644, 72644, 72644, 45386, 41542, 41542, 41542, 52709, 65498, 65498, 65498, 65498, 57854, 41461, 41461, 41461, 41461, 60376, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 73164, 72644, 72644, 72644, 72644, 72644, 41363, 41542, 41542, 41542, 41542, 41542, 65493, 65498, 65498, 65498, 65498, 65498, 46825, 41461, 41461, 41461, 41461, 41461, 47041, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 72643, 72644, 72644, 72644, 72644, 72645, 38772, 41542, 41542, 41542, 41542, 41542, 45363, 65498, 65498, 65498, 65498, 65498, 59800, 41461, 41461, 41461, 41461, 41461, 47863, 54854, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 72642, 72644, 72644, 72644, 72644, 38767, 41542, 41542, 41542, 41542, 41444, 65497, 65498, 65498, 65498, 65498, 59803, 41461, 41461, 41461, 41461, 47863, 54854, 37318, 37318, 37318, 37318, 37318, 60498, 72644, 72644, 72644, 58547, 41542, 41542, 41542, 46770, 65498, 65498, 65498, 59798, 41461, 41461, 41461, 47288, 37318, 37318, 37318, 37318, 37318, 72644, 72644, 60920, 41542, 41542, 41542, 65498, 65498, 65498, 63673, 41461, 41461, 56936, 37318, 37318, 37318, 73163, 72644, 72644, 50031, 41542, 73662, 65498, 65498, 59802, 41461, 41462, 56939, 61615, 37318, 72643, 60923, 41542, 73664, 65498, 59801, 41461, 49498, 72946, 72644, 50030, 54383, 59799, 54339, 61620, 60922, 73660, 47450, 46828, 73161, 59759, 73663, 59797, 47605, 60497, 50030, 47449, 46829, 60496, 73658, 47451, 47606, 53268, 62159, 62339, 66862, 41867, 41898, 41909, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 45302, 37318, 37318, 37318, 37318, 44953, 37318, 37318, 37318, 37318, 37318, 67919, 60153, 37318, 37318, 45323, 45350, 63443, 47000, 65689, 37318, 37318, 41289, 37087, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 44404, 72644, 50028, 41369, 65498, 47282, 62003, 72217, 37318, 37318, 37318, 68839, 37318, 37318, 37318, 37318, 45477, 37318, 37318, 37318, 73164, 72644, 72644, 72644, 72644, 45386, 41542, 41542, 41542, 52709, 65498, 65498, 65498, 65498, 57854, 41461, 41461, 41461, 41461, 60376, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 73164, 72644, 72644, 72644, 72644, 72644, 41363, 41542, 41542, 41542, 41542, 41542, 65493, 65498, 65498, 65498, 65498, 65498, 46825, 41461, 41461, 41461, 41461, 41461, 47041, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 72643, 72644, 72644, 72644, 72644, 72645, 38772, 41542, 41542, 41542, 41542, 41542, 45363, 65498, 65498, 65498, 65498, 65498, 59800, 41461, 41461, 41461, 41461, 41461, 47863, 54854, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 72642, 72644, 72644, 72644, 72644, 38767, 41542, 41542, 41542, 41542, 41444, 65497, 65498, 65498, 65498, 65498, 59803, 41461, 41461, 41461, 41461, 47863, 54854, 37318, 37318, 37318, 37318, 37318, 60498, 72644, 72644, 72644, 58547, 41542, 41542, 41542, 46770, 65498, 65498, 65498, 59798, 41461, 41461, 41461, 47288, 37318, 37318, 37318, 37318, 37318, 72644, 72644, 60920, 41542, 41542, 41542, 65498, 65498, 65498, 63673, 41461, 41461, 56936, 37318, 37318, 37318, 73163, 72644, 72644, 50031, 41542, 73662, 65498, 65498, 59802, 41461, 41462, 56939, 61615, 37318, 72643, 60923, 41542, 73664, 65498, 59801, 41461, 49498, 72946, 72644, 50030, 54383, 59799, 54339, 61620, 60922, 73660, 47450, 46828, 73161, 59759, 73663, 59797, 47605, 60497, 50030, 47449, 46829, 60496, 73658, 47451, 47606, 53268, 62159, 62339, 66862, 41867, 41898, 41909, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 45302, 37318, 37318, 37318, 37318, 44953, 37318, 37318, 37318, 37318, 37318, 67919, 57964, 37318, 37318, 45496, 45525, 45509, 45539, 65689, 37318, 37318, 41289, 37087, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 44404, 72644, 50028, 41369, 65498, 47282, 62003, 72217, 37318, 37318, 37318, 68839, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 73164, 72644, 72644, 72644, 72644, 45386, 41542, 41542, 41542, 52709, 65498, 65498, 65498, 65498, 57854, 41461, 41461, 41461, 41461, 60376, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 73164, 72644, 72644, 72644, 72644, 72644, 41363, 41542, 41542, 41542, 41542, 41542, 65493, 65498, 65498, 65498, 65498, 65498, 46825, 41461, 41461, 41461, 41461, 41461, 47041, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 72643, 72644, 72644, 72644, 72644, 72645, 38772, 41542, 41542, 41542, 41542, 41542, 45363, 65498, 65498, 65498, 65498, 65498, 59800, 41461, 41461, 41461, 41461, 41461, 47863, 54854, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 72642, 72644, 72644, 72644, 72644, 38767, 41542, 41542, 41542, 41542, 41444, 65497, 65498, 65498, 65498, 65498, 59803, 41461, 41461, 41461, 41461, 47863, 54854, 37318, 37318, 37318, 37318, 37318, 60498, 72644, 72644, 72644, 58547, 41542, 41542, 41542, 46770, 65498, 65498, 65498, 59798, 41461, 41461, 41461, 47288, 37318, 37318, 37318, 37318, 37318, 72644, 72644, 60920, 41542, 41542, 41542, 65498, 65498, 65498, 63673, 41461, 41461, 56936, 37318, 37318, 37318, 73163, 72644, 72644, 50031, 41542, 73662, 65498, 65498, 59802, 41461, 41462, 56939, 61615, 37318, 72643, 60923, 41542, 73664, 65498, 59801, 41461, 49498, 72946, 72644, 50030, 54383, 59799, 54339, 61620, 60922, 73660, 47450, 46828, 73161, 59759, 73663, 59797, 47605, 60497, 50030, 47449, 46829, 60496, 73658, 47451, 47606, 53268, 62159, 62339, 66862, 41867, 41898, 41909, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 45302, 37318, 37318, 37318, 37318, 44953, 37318, 37318, 37318, 37318, 37318, 67919, 57964, 37318, 37318, 45323, 45350, 63443, 47000, 65689, 37318, 37318, 41289, 37087, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 44404, 72644, 50028, 41369, 65498, 47282, 62003, 72217, 37318, 37318, 37318, 68839, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 73164, 72644, 72644, 72644, 72644, 45386, 41542, 41542, 41542, 52709, 65498, 65498, 65498, 65498, 57854, 41461, 41461, 41461, 41461, 60376, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 73164, 72644, 72644, 72644, 72644, 72644, 41363, 41542, 41542, 41542, 41542, 41542, 65493, 65498, 65498, 65498, 65498, 65498, 46825, 41461, 41461, 41461, 41461, 41461, 47041, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 72643, 72644, 72644, 72644, 72644, 72645, 38772, 41542, 41542, 41542, 41542, 41542, 45363, 65498, 65498, 65498, 65498, 65498, 59800, 41461, 41461, 41461, 41461, 41461, 47863, 54854, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 72642, 72644, 72644, 72644, 72644, 38767, 41542, 41542, 41542, 41542, 41444, 65497, 65498, 65498, 65498, 65498, 59803, 41461, 41461, 41461, 41461, 47863, 54854, 37318, 37318, 64785, 37318, 37318, 60498, 72644, 72644, 72644, 58547, 41542, 41542, 41542, 46770, 65498, 65498, 65498, 59798, 41461, 41461, 41461, 47288, 37318, 37318, 37318, 37318, 37318, 72644, 72644, 60920, 41542, 41542, 41542, 65498, 65498, 65498, 63673, 41461, 41461, 56936, 37318, 37318, 37318, 73163, 72644, 72644, 50031, 41542, 73662, 65498, 65498, 59802, 41461, 41462, 56939, 61615, 37318, 72643, 60923, 41542, 73664, 65498, 59801, 41461, 49498, 72946, 72644, 50030, 54383, 59799, 54339, 61620, 60922, 73660, 47450, 46828, 73161, 59759, 73663, 59797, 47605, 60497, 50030, 47449, 46829, 60496, 73658, 47451, 47606, 53268, 62159, 62339, 66862, 41867, 41898, 41909, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 45555, 37318, 37318, 37318, 37318, 44953, 37318, 37318, 37318, 37318, 37318, 67919, 57964, 37318, 37318, 45323, 45350, 63443, 47000, 65689, 37318, 37318, 41289, 37087, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 44404, 72644, 50028, 41369, 65498, 47282, 62003, 72217, 37318, 37318, 37318, 68839, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 73164, 72644, 72644, 72644, 72644, 45386, 41542, 41542, 41542, 52709, 65498, 65498, 65498, 65498, 57854, 41461, 41461, 41461, 41461, 60376, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 73164, 72644, 72644, 72644, 72644, 72644, 41363, 41542, 41542, 41542, 41542, 41542, 65493, 65498, 65498, 65498, 65498, 65498, 46825, 41461, 41461, 41461, 41461, 41461, 47041, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 72643, 72644, 72644, 72644, 72644, 72645, 38772, 41542, 41542, 41542, 41542, 41542, 45363, 65498, 65498, 65498, 65498, 65498, 59800, 41461, 41461, 41461, 41461, 41461, 47863, 54854, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 72642, 72644, 72644, 72644, 72644, 38767, 41542, 41542, 41542, 41542, 41444, 65497, 65498, 65498, 65498, 65498, 59803, 41461, 41461, 41461, 41461, 47863, 54854, 37318, 37318, 37318, 37318, 37318, 60498, 72644, 72644, 72644, 58547, 41542, 41542, 41542, 46770, 65498, 65498, 65498, 59798, 41461, 41461, 41461, 47288, 37318, 37318, 37318, 37318, 37318, 72644, 72644, 60920, 41542, 41542, 41542, 65498, 65498, 65498, 63673, 41461, 41461, 56936, 37318, 37318, 37318, 73163, 72644, 72644, 50031, 41542, 73662, 65498, 65498, 59802, 41461, 41462, 56939, 61615, 37318, 72643, 60923, 41542, 73664, 65498, 59801, 41461, 49498, 72946, 72644, 50030, 54383, 59799, 54339, 61620, 60922, 73660, 47450, 46828, 73161, 59759, 73663, 59797, 47605, 60497, 50030, 47449, 46829, 60496, 73658, 47451, 47606, 53268, 62159, 62339, 66862, 41867, 41898, 41909, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 39708, 38529, 37318, 37318, 37318, 48276, 37318, 37318, 37318, 37318, 37318, 67919, 57964, 45576, 74434, 37318, 74579, 74583, 74448, 36991, 37318, 37318, 41289, 37087, 37318, 37318, 37010, 37318, 37318, 37318, 37318, 37318, 65657, 40192, 38525, 37318, 37318, 39335, 40192, 38527, 37031, 44584, 37054, 37076, 37318, 37318, 74250, 68588, 69924, 37107, 37136, 37152, 65815, 69390, 37247, 37265, 53482, 37182, 37116, 37202, 37162, 37222, 37186, 37120, 37206, 37166, 74726, 69388, 37245, 37263, 53480, 37281, 37318, 41092, 37317, 37318, 37318, 45006, 57380, 37335, 37318, 74671, 64449, 74683, 40193, 37351, 37378, 40192, 39199, 37741, 69458, 41098, 41041, 37318, 46044, 37882, 69459, 41099, 41042, 37318, 46045, 37371, 74376, 37355, 37382, 40192, 39202, 38523, 74469, 50180, 37398, 37414, 37436, 50351, 37475, 37318, 58014, 37496, 39340, 37628, 37651, 37674, 55492, 37717, 38230, 37546, 37566, 58028, 41986, 37605, 37523, 37542, 37562, 58024, 37582, 37601, 39304, 37621, 37644, 37667, 37690, 37709, 37733, 74273, 57770, 38317, 37757, 37779, 37822, 46961, 37943, 37979, 37995, 38150, 38170, 38011, 38093, 38038, 38061, 37792, 39434, 38084, 38100, 38045, 38068, 37799, 39441, 38116, 37992, 38147, 38166, 38186, 38022, 38246, 38263, 37318, 38304, 37318, 38354, 38370, 45231, 38397, 38413, 37861, 37459, 38432, 38451, 68861, 37456, 38429, 38448, 38467, 45226, 38483, 38407, 38516, 57783, 51848, 38545, 38580, 73050, 38677, 38610, 38626, 49222, 38643, 38660, 49221, 38642, 38659, 38676, 38693, 38624, 38714, 38739, 45596, 38841, 38876, 38942, 38892, 60443, 38908, 60433, 60449, 38914, 38930, 38946, 38962, 38978, 39013, 39048, 39078, 39105, 39169, 39156, 39172, 39188, 39089, 39218, 39249, 55484, 39265, 39291, 55476, 39311, 39327, 37693, 37318, 65659, 38698, 39356, 39410, 37835, 37848, 39457, 45051, 38498, 38500, 45058, 39494, 39520, 39559, 39504, 39575, 59063, 39606, 39616, 39632, 39681, 39692, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 39708, 37318, 37318, 37318, 37318, 48276, 37318, 37318, 37318, 37318, 37318, 67919, 57964, 37318, 37318, 37318, 37318, 37318, 45625, 45661, 37318, 37318, 41289, 37087, 37318, 37318, 37010, 37318, 37318, 37318, 37318, 37318, 65657, 40192, 38525, 37318, 37318, 39335, 40192, 45833, 37031, 44584, 37054, 37076, 37318, 37318, 74250, 37318, 69924, 37107, 37136, 37152, 65815, 69390, 37247, 37265, 53482, 37182, 37116, 37202, 37162, 37222, 37186, 37120, 37206, 37166, 74726, 69388, 37245, 37263, 53480, 37281, 37318, 41092, 37317, 37318, 37318, 45006, 57380, 37335, 37318, 74671, 64449, 74683, 40193, 37351, 37378, 40192, 39199, 37741, 69458, 41098, 41041, 37318, 46044, 37882, 69459, 41099, 41042, 37318, 46045, 37371, 74376, 37355, 37382, 40192, 39202, 38523, 74469, 50180, 37398, 37414, 37436, 50351, 37475, 37318, 58014, 37496, 39340, 37628, 37651, 37674, 55492, 37717, 38230, 37546, 37566, 58028, 41986, 37605, 37523, 37542, 37562, 58024, 37582, 37601, 39304, 37621, 37644, 37667, 37690, 37709, 37733, 74273, 57770, 38317, 37757, 37779, 37822, 46961, 37943, 37979, 37995, 38150, 38170, 38011, 38093, 38038, 38061, 37792, 39434, 38084, 38100, 38045, 38068, 37799, 39441, 38116, 37992, 38147, 38166, 38186, 38022, 38246, 38263, 37318, 38304, 37318, 38354, 38370, 45231, 38397, 38413, 37861, 37459, 38432, 38451, 68861, 37456, 38429, 38448, 38467, 45226, 38483, 38407, 38516, 57783, 51848, 38545, 38580, 73050, 38677, 38610, 38626, 49222, 38643, 38660, 49221, 38642, 38659, 38676, 38693, 38624, 38714, 38739, 38791, 38841, 38876, 38942, 38892, 60443, 38908, 60433, 60449, 38914, 38930, 38946, 38962, 38978, 39013, 39048, 39078, 39105, 39169, 39156, 39172, 39188, 39089, 39218, 39249, 55484, 39265, 39291, 55476, 39311, 39327, 37693, 37318, 65659, 38698, 39356, 39410, 37835, 37848, 39457, 45051, 38498, 38500, 45058, 39494, 39520, 39559, 39504, 39575, 59063, 39606, 39616, 39632, 39681, 39692, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 39708, 37318, 45715, 37318, 37318, 48276, 37318, 37318, 37318, 37318, 37318, 67919, 57964, 45680, 45684, 37318, 37318, 45700, 45732, 36991, 37318, 37318, 41289, 37087, 37318, 37318, 37010, 37318, 37318, 37318, 37318, 37318, 65657, 40192, 38525, 37318, 37318, 39335, 40192, 38527, 37031, 44584, 37054, 37076, 37318, 37318, 74250, 37318, 69924, 37107, 37136, 37152, 65815, 69390, 37247, 37265, 53482, 37182, 37116, 37202, 37162, 37222, 37186, 37120, 37206, 37166, 74726, 69388, 37245, 37263, 53480, 37281, 37318, 41092, 37317, 37318, 37318, 45006, 57380, 37335, 37318, 74671, 64449, 74683, 40193, 37351, 37378, 40192, 39199, 37741, 69458, 41098, 41041, 37318, 46044, 37882, 69459, 41099, 41042, 37318, 46045, 37371, 74376, 37355, 37382, 40192, 39202, 38523, 74469, 50180, 37398, 37414, 37436, 50351, 37475, 37318, 58014, 37496, 39340, 37628, 37651, 37674, 55492, 37717, 38230, 37546, 37566, 58028, 41986, 37605, 37523, 37542, 37562, 58024, 37582, 37601, 39304, 37621, 37644, 37667, 37690, 37709, 37733, 74273, 57770, 38317, 37757, 37779, 37822, 46961, 37943, 37979, 37995, 38150, 38170, 38011, 38093, 38038, 38061, 37792, 39434, 38084, 38100, 38045, 38068, 37799, 39441, 38116, 37992, 38147, 38166, 38186, 38022, 38246, 38263, 37318, 38304, 37318, 38354, 38370, 45231, 38397, 38413, 37861, 37459, 38432, 38451, 68861, 37456, 38429, 38448, 38467, 45226, 38483, 38407, 38516, 57783, 51848, 38545, 38580, 73050, 38677, 38610, 38626, 49222, 38643, 38660, 49221, 38642, 38659, 38676, 38693, 38624, 38714, 38739, 38791, 38841, 38876, 38942, 38892, 60443, 38908, 60433, 60449, 38914, 38930, 38946, 38962, 38978, 45768, 39048, 39078, 39105, 39169, 39156, 39172, 39188, 39089, 39218, 45803, 55484, 39265, 39291, 55476, 39311, 45819, 37693, 37318, 65659, 38698, 39356, 39410, 37835, 37848, 39457, 45051, 38498, 38500, 45058, 39494, 39520, 39559, 39504, 39575, 59063, 39606, 39616, 39632, 39681, 39692, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 39708, 37318, 37318, 37318, 37318, 48276, 37318, 37318, 37318, 37318, 37318, 67919, 57964, 37318, 37318, 37318, 37318, 37318, 45266, 36991, 37318, 37318, 41289, 37087, 37318, 37318, 37010, 37318, 37318, 37318, 37318, 37318, 65657, 40192, 38525, 37318, 37318, 39335, 40192, 38527, 37031, 44584, 37054, 37076, 37318, 37318, 74250, 37318, 69924, 37107, 37136, 37152, 65815, 69390, 37247, 37265, 53482, 37182, 37116, 37202, 37162, 37222, 37186, 37120, 37206, 37166, 74726, 69388, 37245, 37263, 53480, 37281, 37318, 41092, 37317, 37318, 37318, 45006, 57380, 37335, 37318, 74671, 64449, 74683, 40193, 37351, 37378, 40192, 39199, 37741, 69458, 41098, 41041, 37318, 46044, 37882, 69459, 41099, 41042, 37318, 46045, 37371, 74376, 37355, 37382, 40192, 39202, 38523, 74469, 50180, 37398, 37414, 37436, 50351, 37475, 37318, 58014, 37496, 39340, 37628, 37651, 37674, 55492, 37717, 38230, 37546, 37566, 58028, 41986, 37605, 37523, 37542, 37562, 58024, 37582, 37601, 39304, 37621, 37644, 37667, 37690, 37709, 37733, 74273, 57770, 38317, 37757, 37779, 37822, 46961, 37943, 37979, 37995, 38150, 38170, 38011, 38093, 38038, 38061, 37792, 39434, 38084, 38100, 38045, 38068, 37799, 39441, 38116, 37992, 38147, 38166, 38186, 38022, 38246, 38263, 37318, 38304, 37318, 38354, 38370, 45231, 38397, 38413, 37861, 37459, 38432, 38451, 68861, 37456, 38429, 38448, 38467, 45226, 38483, 38407, 38516, 57783, 51848, 38545, 38580, 73050, 38677, 38610, 38626, 49222, 38643, 38660, 49221, 38642, 38659, 38676, 38693, 38624, 38714, 38739, 38791, 38841, 38876, 38942, 38892, 60443, 38908, 60433, 60449, 38914, 38930, 38946, 38962, 38978, 39013, 39048, 39078, 39105, 39169, 39156, 39172, 39188, 39089, 39218, 39249, 55484, 39265, 39291, 55476, 39311, 39327, 37693, 37318, 65659, 38698, 39356, 39410, 37835, 37848, 39457, 45051, 38498, 38500, 45058, 39494, 39520, 39559, 39504, 39575, 59063, 39606, 39616, 39632, 39681, 39692, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 45302, 37318, 45863, 37318, 37318, 44953, 53155, 45882, 37318, 52352, 45886, 39423, 67164, 45902, 45916, 45932, 45961, 45945, 45975, 45991, 37318, 45455, 41670, 46020, 50786, 46004, 46033, 45185, 46061, 46096, 44613, 41965, 46121, 46173, 46189, 46205, 46221, 46237, 46288, 65632, 46304, 37318, 37318, 68839, 40298, 50758, 37318, 41033, 53889, 69149, 37318, 46321, 73164, 72644, 72644, 72644, 46341, 45386, 41542, 41542, 59697, 52709, 65498, 65498, 65498, 64013, 57854, 41461, 41461, 41461, 53427, 64567, 41960, 63943, 65692, 37318, 55390, 46361, 44399, 46380, 37318, 58658, 46936, 46397, 46417, 72644, 72644, 59240, 58293, 46453, 46492, 41542, 41542, 52156, 55833, 61582, 46520, 65498, 65498, 64241, 46547, 52866, 68024, 41461, 41461, 68015, 46594, 47566, 46621, 37318, 37318, 46672, 37318, 46264, 44403, 37318, 40057, 46707, 46725, 63333, 41308, 72644, 46741, 63340, 42275, 46760, 41542, 46793, 66052, 61720, 45363, 46811, 65498, 49417, 68460, 65498, 46845, 46864, 41461, 51002, 52264, 41461, 46904, 52111, 46933, 46952, 37318, 46684, 52530, 67711, 37318, 59166, 64847, 72644, 54549, 51902, 61913, 46986, 41542, 41542, 73854, 71759, 72272, 41570, 65498, 54158, 68314, 46848, 58449, 41461, 41461, 47016, 47035, 51372, 47057, 47076, 37318, 47124, 37318, 47140, 47156, 71524, 72644, 42291, 47208, 67563, 41542, 59595, 47244, 47267, 65498, 59798, 47304, 47328, 41461, 53433, 37318, 47344, 37318, 61381, 37318, 72644, 47362, 47380, 41542, 47407, 47424, 65498, 47447, 64139, 63673, 47467, 64041, 56936, 37318, 37318, 37318, 73163, 72644, 72644, 50031, 41542, 73662, 65498, 65498, 59802, 41461, 41462, 56939, 61615, 37318, 72643, 60923, 41542, 73664, 65498, 59801, 41461, 49498, 72946, 72644, 50030, 54383, 59799, 41347, 39062, 47486, 47520, 47551, 47602, 47622, 61702, 70577, 48688, 47605, 47638, 50030, 47449, 46829, 57656, 73658, 47451, 47606, 53268, 62159, 62339, 66862, 41867, 41898, 41909, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 45302, 37318, 47659, 37318, 37318, 44953, 37318, 37318, 37318, 37318, 37318, 67919, 47681, 47697, 47709, 47725, 47754, 47738, 47770, 65689, 37318, 37318, 41289, 37087, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 44404, 67827, 50028, 47806, 47251, 47282, 47858, 50482, 37318, 37318, 37318, 68839, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 73164, 72644, 72644, 72644, 72644, 45386, 41542, 41542, 41542, 52709, 65498, 65498, 65498, 65498, 57854, 41461, 41461, 41461, 41461, 64567, 41960, 37318, 37318, 37318, 37318, 73633, 44399, 56082, 37318, 56083, 37318, 73164, 72644, 72644, 72644, 72644, 47879, 41363, 41542, 41542, 41542, 41542, 47922, 65493, 65498, 65498, 65498, 65498, 47955, 46825, 41461, 41461, 41461, 67629, 66428, 63696, 45404, 37318, 37318, 37318, 37318, 37318, 44403, 37318, 37318, 37318, 72643, 72644, 72644, 72644, 72644, 72645, 38772, 41542, 41542, 41542, 41542, 41542, 45363, 65498, 65498, 65498, 65498, 65498, 59800, 41461, 41461, 41461, 41461, 41461, 47863, 51497, 37318, 58926, 37318, 37318, 64306, 57609, 37318, 65431, 72644, 72644, 72644, 56125, 63073, 47979, 41542, 41542, 41542, 70807, 58352, 65498, 65498, 65498, 65498, 47999, 41461, 41461, 41461, 41461, 48015, 47501, 37318, 37318, 37318, 37318, 37318, 60498, 72644, 72644, 72644, 42291, 41542, 41542, 41542, 59595, 65498, 65498, 65498, 59798, 41461, 41461, 41461, 47288, 37318, 37318, 37318, 37318, 37318, 72644, 72644, 60920, 41542, 41542, 41542, 65498, 65498, 65498, 63673, 41461, 41461, 56936, 37318, 37318, 37318, 73163, 72644, 72644, 50031, 41542, 73662, 65498, 65498, 59802, 41461, 41462, 56939, 61615, 37318, 72643, 60923, 41542, 73664, 65498, 59801, 41461, 49498, 72946, 72644, 50030, 54383, 59799, 54339, 61620, 60922, 73660, 47450, 46828, 65375, 48037, 48062, 48088, 48113, 60497, 50030, 47449, 46829, 60496, 73658, 47451, 47606, 53268, 62159, 62339, 66862, 41867, 41898, 41909, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 45302, 37318, 49548, 37318, 37318, 44953, 37318, 41780, 37318, 41482, 37318, 54642, 48139, 48155, 48170, 48186, 48215, 48199, 48231, 65689, 37318, 37318, 48273, 48292, 37318, 48311, 37318, 51503, 37318, 37318, 37318, 48331, 46709, 48623, 50028, 58342, 50938, 47282, 52495, 55465, 37318, 48349, 37318, 68839, 48369, 47665, 37318, 51866, 48388, 48408, 48427, 73007, 67026, 70205, 72644, 72644, 72644, 48444, 48468, 41542, 41542, 52709, 48501, 48532, 65498, 65498, 51441, 56912, 41461, 41461, 41461, 64567, 71585, 37318, 37318, 65402, 37318, 56580, 44399, 46364, 37873, 48553, 37318, 73164, 48570, 72644, 48622, 56986, 72644, 41363, 48639, 50032, 41542, 55261, 41542, 65493, 48669, 57135, 65498, 73731, 65498, 46825, 48704, 41598, 41461, 48739, 41461, 63696, 45404, 37318, 37318, 39025, 37318, 62106, 44403, 37318, 37318, 37318, 72643, 72644, 72644, 60958, 57260, 72645, 38772, 41542, 41542, 54046, 68350, 48760, 45363, 65498, 65498, 65498, 48780, 52828, 59800, 41461, 41461, 41461, 48796, 66903, 47863, 51534, 37318, 37318, 37318, 64199, 37318, 64441, 37318, 72642, 72644, 72644, 64854, 72644, 61913, 41542, 41542, 41542, 48812, 48876, 65497, 65498, 65498, 71839, 65498, 59803, 41461, 41461, 47470, 41461, 47863, 47501, 37318, 65142, 37318, 37318, 38723, 48831, 72644, 48851, 72644, 42291, 41542, 48871, 41542, 59595, 65498, 70398, 65498, 59798, 41461, 61249, 41461, 42174, 72114, 37318, 37318, 53513, 66795, 54283, 70249, 60920, 63484, 67969, 41542, 58586, 58373, 65498, 49897, 57220, 41461, 56936, 37318, 37318, 67331, 73163, 72644, 72800, 50031, 41542, 52475, 65498, 65498, 48892, 41461, 58211, 56939, 61615, 37318, 72643, 60923, 41542, 73664, 65498, 59801, 41461, 49498, 72946, 72644, 50030, 54383, 59799, 54339, 61620, 60922, 73660, 47450, 47019, 45307, 48912, 48959, 48989, 49019, 60497, 50030, 47449, 46829, 60496, 73658, 47451, 47606, 53268, 62159, 62339, 70340, 49053, 41898, 41909, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 45302, 37318, 37318, 49089, 37318, 44953, 37318, 48599, 37585, 39665, 37318, 67919, 49107, 49123, 49137, 49153, 49182, 49166, 49196, 65689, 37318, 37318, 41289, 37087, 37318, 37318, 37318, 37318, 37318, 48353, 37318, 48351, 60095, 51266, 41622, 41369, 62908, 47535, 62003, 72217, 37318, 62994, 49212, 68839, 37318, 49238, 37318, 49258, 37318, 51050, 39778, 49276, 57933, 60709, 68980, 49292, 71214, 49326, 70571, 70834, 49347, 57459, 49369, 49386, 49404, 49439, 49457, 61552, 61800, 49485, 58425, 64567, 41960, 37318, 37318, 68084, 37318, 56022, 44399, 37318, 37318, 37318, 70710, 73164, 72644, 72644, 72644, 72644, 49520, 41363, 41542, 41542, 41542, 52429, 41542, 65493, 65498, 65498, 65498, 59621, 65498, 46825, 41461, 41461, 41461, 65942, 41461, 71902, 49537, 37318, 37318, 48372, 63307, 37318, 44403, 37318, 37318, 38213, 70690, 72446, 72644, 72644, 72644, 64634, 56631, 49564, 49582, 41542, 41542, 56653, 45363, 49600, 48682, 65498, 65498, 64927, 49622, 49640, 70938, 41461, 41461, 62085, 49662, 51497, 37318, 37318, 37318, 37318, 37318, 49684, 37318, 68617, 72644, 72644, 69970, 72644, 65479, 41542, 41542, 41326, 41542, 48876, 49710, 65498, 65498, 47842, 65498, 59286, 41461, 41461, 58267, 41461, 47863, 47501, 37918, 62752, 37318, 37318, 49733, 60498, 55203, 65029, 49752, 42291, 67741, 65203, 74026, 59595, 66024, 65498, 49773, 59798, 49646, 41461, 49825, 47288, 37318, 51997, 37318, 37318, 37318, 72644, 72644, 60920, 41542, 41542, 41542, 65498, 65498, 65498, 63673, 41461, 41461, 56936, 37318, 49852, 37318, 73163, 72644, 60539, 50031, 41542, 72428, 65498, 65498, 49872, 41461, 70461, 56939, 61615, 37318, 72643, 60923, 41542, 57845, 65498, 49892, 41461, 49913, 72946, 72644, 50030, 54383, 59799, 54339, 40392, 49935, 49960, 49980, 50002, 57326, 50022, 50048, 50069, 50089, 60497, 50030, 47449, 46829, 60496, 73658, 47451, 47606, 53268, 62159, 62339, 66862, 41867, 41898, 41909, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 45302, 37318, 37318, 50115, 37318, 44953, 44449, 50137, 37318, 37060, 50171, 54108, 64429, 50196, 50210, 50226, 50255, 50239, 50269, 65689, 44626, 50285, 50301, 50316, 50332, 50367, 39233, 53108, 50384, 61398, 55344, 50419, 50467, 50512, 50528, 50544, 50594, 50610, 50651, 50667, 73014, 50694, 49260, 50729, 62653, 50756, 50774, 72085, 37318, 37318, 49073, 37318, 73164, 50802, 70730, 47643, 67373, 50837, 50866, 47931, 48477, 50890, 64147, 50934, 50954, 50974, 61240, 51024, 54206, 54678, 60300, 51066, 42093, 59527, 51082, 54503, 44179, 51132, 51166, 69514, 50121, 44486, 44480, 51182, 51198, 51247, 51290, 51306, 51322, 51357, 51407, 51457, 51482, 51519, 51581, 60231, 51607, 51635, 51659, 51675, 51432, 54188, 73942, 51691, 51713, 51750, 51800, 41753, 51824, 51840, 37318, 45167, 74354, 51864, 51882, 51918, 51962, 51982, 72643, 52018, 52052, 68101, 52072, 70228, 38772, 52096, 72897, 41542, 52136, 57279, 52172, 66566, 52195, 65498, 64541, 65498, 52222, 56902, 52245, 41461, 67612, 41461, 52280, 51497, 37891, 64079, 46636, 37318, 52344, 55168, 73701, 70986, 47906, 72644, 52368, 52401, 61913, 52428, 52445, 53191, 52464, 52511, 46777, 66351, 65498, 52546, 52580, 55946, 51008, 41461, 52615, 49469, 47863, 47501, 37318, 39657, 52645, 37318, 58484, 60498, 52664, 60195, 52685, 42291, 67536, 73887, 52704, 59595, 49809, 69760, 70602, 59798, 46578, 54457, 71795, 47288, 37318, 68581, 52725, 60872, 52759, 71031, 52775, 65989, 69245, 52802, 72675, 52206, 52852, 55908, 48097, 54237, 46605, 52882, 52918, 52936, 39919, 39478, 52952, 52973, 52992, 65449, 53038, 56792, 66087, 71786, 53088, 56765, 56939, 40362, 53104, 72643, 69978, 41542, 52310, 65498, 53124, 53817, 55637, 53146, 55211, 53171, 53289, 59799, 53207, 55668, 73239, 68122, 73294, 50578, 53242, 53264, 53284, 53305, 53324, 42326, 53340, 53356, 53372, 60496, 73658, 47451, 66435, 53400, 53449, 62339, 66862, 41867, 41898, 41909, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 45302, 37318, 37318, 51938, 37318, 44953, 62203, 56053, 37318, 51929, 37318, 53498, 53534, 53550, 53564, 53580, 53609, 53593, 53623, 65689, 37318, 68909, 41289, 37087, 37318, 37318, 37318, 37318, 61388, 37318, 61393, 61388, 52002, 53639, 47391, 58863, 63536, 59869, 62003, 50482, 37318, 53663, 37318, 68839, 53663, 37318, 37318, 37318, 69142, 53682, 45250, 37318, 55674, 71208, 72644, 72644, 72644, 53704, 53738, 41542, 41542, 52709, 48072, 65498, 65498, 65498, 57854, 53755, 41461, 41461, 41461, 64567, 41960, 37318, 37318, 37318, 37318, 37318, 44399, 37318, 37318, 37318, 53779, 73164, 72644, 72644, 72644, 63735, 72644, 41363, 41542, 41542, 41542, 60760, 41542, 65493, 65498, 65498, 65498, 61277, 65498, 46825, 41461, 41461, 41461, 48744, 41461, 63696, 45404, 37318, 37318, 37318, 37318, 37318, 44403, 37318, 45747, 37318, 72643, 72644, 72644, 50815, 72644, 72645, 38772, 41542, 41542, 71247, 41542, 41542, 45363, 65498, 65498, 48537, 65498, 65498, 59800, 41461, 41461, 67762, 41461, 41461, 47863, 51497, 37318, 37318, 37318, 37318, 37318, 67308, 37318, 72642, 72644, 72644, 56359, 72644, 61913, 41542, 41542, 41542, 70112, 48876, 65497, 65498, 65498, 66735, 65498, 59803, 41461, 41461, 41461, 53797, 47863, 47501, 37318, 37318, 37318, 37318, 37318, 60498, 72644, 72644, 72644, 42291, 41542, 41542, 41542, 59595, 65498, 65498, 65498, 59798, 41461, 41461, 41461, 47288, 37318, 37318, 37318, 37318, 37318, 72644, 72644, 60920, 41542, 41542, 41542, 65498, 65498, 65498, 63673, 41461, 41461, 54260, 37318, 37318, 37318, 73163, 67467, 72644, 48046, 41542, 73662, 48935, 65498, 59802, 53815, 41462, 56939, 61615, 37318, 72643, 60923, 41542, 73664, 65498, 59801, 41461, 49498, 72946, 72644, 50030, 54383, 59799, 54339, 61620, 60922, 73660, 47450, 46828, 73161, 59759, 73663, 59797, 47605, 60497, 50030, 47449, 46829, 60496, 73658, 47451, 41851, 61054, 66378, 62339, 68189, 53833, 41898, 41909, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 45302, 37318, 37318, 37318, 37318, 44953, 37318, 39797, 37318, 37318, 37318, 67919, 53869, 53905, 53909, 53925, 53954, 53938, 53968, 65689, 37318, 37318, 41289, 37087, 37318, 45716, 37318, 37318, 37318, 37318, 37318, 37318, 44404, 72644, 50028, 41369, 65498, 47282, 62003, 46136, 46156, 37318, 37318, 68839, 37318, 37318, 37318, 37318, 40903, 37318, 37318, 37318, 73164, 72644, 72644, 72644, 72644, 45386, 41542, 41542, 41542, 52709, 65498, 65498, 65498, 65498, 57854, 41461, 41461, 41461, 41461, 60376, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 36975, 37318, 64085, 73164, 72644, 72644, 72644, 72644, 72644, 53984, 41542, 41542, 41542, 41542, 41542, 62136, 65498, 65498, 65498, 65498, 65498, 46825, 41461, 41461, 41461, 41461, 41461, 47041, 37318, 37318, 37318, 54006, 37318, 54026, 36968, 37318, 37318, 53666, 72643, 72644, 65774, 72644, 72644, 72645, 38772, 41542, 41542, 54043, 41542, 41542, 45363, 65498, 65498, 53015, 65498, 65498, 59800, 41461, 41461, 66593, 41461, 41461, 71140, 54854, 37318, 37318, 54062, 37318, 37318, 37318, 37318, 72642, 72644, 72644, 72644, 72644, 38767, 41542, 41542, 41542, 41542, 41444, 65497, 65498, 65498, 65498, 65498, 59803, 41461, 41461, 41461, 41461, 47863, 54854, 37318, 37318, 54082, 64821, 37318, 60498, 72644, 72644, 62830, 58547, 41542, 41542, 58315, 46770, 65498, 65498, 71297, 59798, 41461, 41461, 54740, 47288, 59033, 54099, 69440, 44860, 37318, 54124, 64478, 44740, 41542, 54141, 70297, 54157, 61206, 54174, 54204, 58614, 54222, 56936, 53884, 37318, 41072, 40582, 71978, 54276, 54299, 54319, 56380, 73391, 73919, 41714, 55599, 54335, 56939, 50403, 59838, 42233, 54355, 53715, 54379, 54399, 59801, 54439, 49498, 72946, 72644, 50030, 54383, 59799, 54339, 61620, 60922, 73660, 47450, 46828, 73161, 59759, 73663, 59797, 47605, 60497, 50030, 47449, 46829, 60496, 73658, 47451, 47606, 53268, 62159, 57408, 54473, 41867, 41898, 41909, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 45302, 37318, 37318, 37318, 37318, 44953, 37318, 37318, 37318, 37318, 37318, 67919, 57964, 37318, 37318, 45323, 45350, 63443, 47000, 59399, 37318, 37318, 60166, 37087, 37318, 37318, 37318, 54501, 37318, 54501, 42045, 54519, 39472, 54546, 54565, 67872, 65498, 54590, 62375, 54624, 37318, 37318, 47504, 68839, 67717, 44759, 37318, 37318, 54658, 37318, 37318, 37318, 73164, 72644, 72644, 58045, 72644, 45386, 41542, 41542, 67542, 52709, 65498, 65498, 65498, 56180, 57854, 41461, 41461, 41461, 54676, 60376, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 68532, 57795, 73164, 72644, 72644, 72644, 72644, 72644, 54694, 41542, 41542, 41542, 41542, 41542, 54716, 65498, 65498, 65498, 65498, 65498, 54737, 41461, 41461, 41461, 41461, 41461, 54756, 71731, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 74625, 57661, 72644, 72644, 72644, 72644, 72645, 38772, 54791, 41542, 41542, 41542, 41542, 45363, 54808, 65498, 65498, 65498, 65498, 59800, 54829, 41461, 41461, 41461, 41461, 47863, 54854, 37318, 37318, 66657, 37318, 37318, 37318, 37318, 72642, 72644, 72644, 72644, 58513, 38767, 41542, 41542, 41542, 63753, 41444, 65497, 65498, 65498, 65498, 70365, 59803, 41461, 41461, 41461, 60263, 47863, 54854, 37318, 37318, 49068, 37318, 37318, 60498, 72644, 72644, 72644, 58547, 41542, 41542, 41542, 46770, 65498, 65498, 65498, 59798, 41461, 41461, 41461, 47288, 37318, 37318, 37318, 37318, 37318, 72644, 72644, 60920, 41542, 41542, 41542, 65498, 65498, 65498, 63673, 41461, 41461, 56936, 37318, 54946, 37318, 40844, 72644, 72644, 50031, 41542, 73662, 65498, 65498, 59802, 41461, 41462, 54850, 61615, 65734, 72643, 61015, 41542, 72843, 65498, 54871, 41461, 54893, 72946, 72644, 50030, 54383, 59799, 54339, 61620, 60922, 73660, 47450, 46828, 73161, 59759, 73663, 59797, 47605, 60497, 50030, 47449, 46829, 60496, 73658, 47451, 47606, 53268, 62159, 62339, 66862, 41867, 41898, 41909, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 45302, 37318, 37318, 51946, 37318, 44953, 37318, 54915, 54931, 37318, 54967, 54974, 54990, 55006, 55021, 55037, 55066, 55050, 55080, 65689, 37318, 37318, 51966, 38825, 37318, 37318, 37318, 54010, 55096, 55115, 48257, 55149, 55184, 55227, 55249, 41525, 55284, 55303, 63265, 55333, 37318, 37318, 37318, 40256, 37318, 55133, 55360, 37318, 37318, 37318, 55377, 52524, 73164, 55412, 59232, 72644, 55450, 55508, 48452, 55571, 48764, 46504, 61212, 71358, 65498, 52836, 55590, 72384, 66401, 41461, 55624, 55653, 37318, 55162, 48123, 45480, 68146, 44384, 37318, 37318, 55690, 55710, 55730, 55750, 50821, 72644, 73775, 55770, 55805, 41363, 53990, 41542, 54303, 55821, 55849, 65493, 53022, 65498, 61780, 55897, 55924, 46825, 56459, 41461, 71394, 55962, 55996, 47041, 37318, 37318, 37318, 54660, 56044, 56069, 49736, 37318, 37318, 37318, 56099, 56122, 72644, 56141, 70528, 58519, 56160, 58143, 41542, 63117, 55574, 66717, 50850, 52564, 65498, 63146, 65498, 56203, 56221, 51808, 41461, 56467, 41461, 56251, 56269, 54854, 55099, 56307, 46073, 56323, 59679, 56339, 62046, 56355, 52688, 61008, 58074, 70003, 51212, 41542, 66283, 70084, 56375, 56396, 56430, 65125, 61593, 46531, 69280, 62369, 41461, 56449, 56483, 56517, 56546, 63230, 66178, 37927, 69494, 47228, 56575, 66873, 56144, 56596, 56612, 70031, 45388, 56647, 56669, 56693, 56729, 56747, 56781, 56815, 56873, 56892, 51765, 56928, 56955, 37318, 37318, 61883, 53848, 56976, 57010, 57028, 57047, 57072, 57089, 57107, 57133, 57151, 57170, 57197, 57216, 56936, 60597, 37318, 40353, 73163, 72644, 72644, 50031, 41542, 73662, 65498, 65498, 59802, 41461, 41462, 57236, 61615, 37318, 57252, 51333, 57276, 57295, 62141, 50073, 54877, 49498, 57311, 72644, 50030, 54383, 59799, 54339, 39384, 52786, 55523, 72337, 68506, 73161, 59759, 73663, 59797, 47605, 60497, 50030, 47449, 46829, 57331, 57347, 55938, 57396, 53268, 62159, 62339, 66862, 41867, 57424, 41909, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 45302, 37318, 37318, 42152, 37318, 44953, 37318, 37318, 37318, 37318, 37318, 67919, 57964, 37318, 37318, 57475, 57504, 57488, 57518, 65689, 37318, 37318, 41289, 37087, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 44404, 72644, 50028, 41369, 65498, 47282, 62003, 72217, 37318, 37318, 37318, 68839, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 73164, 72644, 72644, 66211, 72644, 45386, 41542, 41542, 57534, 52709, 65498, 65498, 49388, 65498, 57854, 41461, 41461, 51784, 41461, 60376, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 73164, 72644, 72644, 72644, 72644, 72644, 41363, 41542, 41542, 41542, 41542, 41542, 65493, 65498, 65498, 65498, 65498, 65498, 46825, 41461, 41461, 41461, 41461, 41461, 47041, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 72643, 72644, 72644, 72644, 72644, 72645, 38772, 41542, 41542, 41542, 41542, 41542, 45363, 65498, 65498, 65498, 65498, 65498, 59800, 41461, 41461, 41461, 41461, 41461, 47863, 54854, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 72642, 72644, 72644, 72644, 72644, 38767, 41542, 41542, 41542, 41542, 41444, 65497, 65498, 65498, 65498, 65498, 59803, 41461, 41461, 41461, 41461, 47863, 54854, 37318, 37318, 37318, 37318, 37318, 60498, 72644, 72644, 72644, 58547, 41542, 41542, 41542, 46770, 65498, 65498, 65498, 59798, 41461, 41461, 41461, 47288, 37318, 37318, 37318, 37318, 37318, 72644, 72644, 60920, 41542, 41542, 41542, 65498, 65498, 65498, 63673, 41461, 41461, 56936, 37318, 37318, 37318, 73163, 72644, 72644, 50031, 41542, 73662, 65498, 65498, 59802, 41461, 41462, 56939, 61615, 37318, 72643, 60923, 41542, 73664, 65498, 59801, 41461, 49498, 72946, 72644, 50030, 54383, 59799, 54339, 61620, 60922, 73660, 47450, 46828, 73161, 59759, 73663, 59797, 47605, 60497, 50030, 47449, 46829, 60496, 73658, 47451, 47606, 53268, 62159, 62339, 66862, 41867, 41898, 41909, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 45302, 37318, 37318, 41502, 37318, 44953, 37318, 37318, 37318, 37318, 37318, 67919, 57964, 37318, 37318, 45323, 45350, 63443, 47000, 65689, 37318, 37318, 41289, 37087, 37318, 52736, 37318, 37318, 37318, 37318, 37318, 37318, 44404, 72644, 50028, 41369, 65498, 47282, 62003, 72217, 37318, 37318, 37318, 68839, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 73164, 72644, 72644, 72644, 72644, 45386, 41542, 41542, 41542, 52709, 65498, 65498, 65498, 65498, 57854, 41461, 41461, 41461, 41461, 60376, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 73164, 72644, 72644, 72644, 72644, 72644, 41363, 41542, 41542, 41542, 41542, 41542, 65493, 65498, 65498, 65498, 65498, 65498, 46825, 41461, 41461, 41461, 41461, 41461, 47041, 47060, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 57438, 72644, 72644, 72644, 72644, 72645, 61918, 41542, 41542, 41542, 41542, 41542, 45363, 57553, 65498, 65498, 65498, 65498, 59800, 57571, 41461, 41461, 41461, 41461, 47863, 54854, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 72642, 72644, 72644, 72644, 72644, 38767, 41542, 41542, 41542, 41542, 41444, 65497, 65498, 65498, 65498, 65498, 59803, 41461, 41461, 41461, 41461, 47863, 54854, 37318, 37318, 37318, 37318, 37318, 60498, 72644, 72644, 72644, 58547, 41542, 41542, 41542, 46770, 65498, 65498, 65498, 59798, 41461, 41461, 41461, 47288, 37318, 37318, 37318, 37318, 37318, 72644, 72644, 60920, 41542, 41542, 41542, 65498, 65498, 65498, 63673, 41461, 41461, 56936, 37318, 37318, 37318, 73163, 72644, 72644, 50031, 41542, 73662, 65498, 65498, 59802, 41461, 41462, 56939, 61615, 37318, 72643, 60923, 41542, 73664, 65498, 59801, 41461, 49498, 72946, 72644, 50030, 54383, 59799, 54339, 61620, 60922, 73660, 47450, 46828, 73161, 59759, 73663, 59797, 47605, 60497, 50030, 47449, 46829, 60496, 73658, 47451, 47606, 53268, 62159, 62339, 66862, 41867, 41898, 41909, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 45302, 37318, 37318, 39711, 37318, 44953, 37318, 42450, 57589, 57607, 57625, 51565, 57641, 57677, 57681, 57697, 57726, 57710, 57740, 65689, 51556, 37318, 41289, 37087, 47346, 60068, 37318, 37318, 39892, 37318, 40511, 57756, 45560, 57818, 57834, 63977, 57879, 60331, 41648, 58000, 57914, 37318, 37318, 38814, 37318, 37318, 57949, 44882, 37318, 57925, 37318, 67807, 49037, 57985, 58044, 51257, 58061, 58115, 58131, 41542, 58165, 54574, 59660, 56705, 57154, 47834, 58193, 52256, 58227, 57863, 58246, 50918, 53781, 37318, 37318, 37318, 37318, 49029, 67667, 37318, 37318, 37318, 37318, 73164, 60986, 49757, 72644, 58283, 72644, 58309, 58331, 63863, 47408, 61105, 41542, 58368, 58389, 67585, 57555, 49801, 65498, 58422, 58441, 68033, 65185, 60256, 69351, 47041, 37318, 61839, 58465, 37318, 37318, 45281, 53688, 37318, 37318, 37318, 72643, 72644, 72644, 63835, 58500, 58535, 38772, 41542, 41542, 55268, 69704, 69729, 45363, 65498, 65498, 61489, 63774, 58568, 59800, 41461, 41461, 66754, 51734, 58602, 47863, 54854, 58637, 37318, 37318, 68901, 37318, 37318, 37318, 72642, 66219, 72644, 72644, 72644, 38767, 67986, 41542, 41542, 41542, 41444, 65497, 56713, 65498, 65498, 65498, 59803, 71875, 41461, 41461, 41461, 47863, 54899, 37318, 58657, 37318, 51097, 69488, 55754, 58674, 72644, 72644, 58694, 58724, 41542, 41542, 58745, 58769, 65498, 65498, 61143, 58793, 41461, 41461, 71804, 37318, 68935, 37318, 73352, 37318, 72644, 49303, 60920, 41542, 72302, 41542, 65498, 52557, 65498, 63673, 46570, 41461, 56936, 37318, 38852, 37318, 73163, 72644, 56106, 50031, 41542, 58818, 65498, 73828, 59802, 41461, 48715, 56939, 61615, 37318, 72643, 60923, 41542, 73664, 65498, 59801, 41461, 49498, 72946, 72644, 50030, 54383, 59799, 54339, 61620, 60922, 73660, 47450, 46828, 73161, 59759, 73663, 59797, 47605, 53248, 58845, 64251, 68569, 67094, 58879, 59108, 47606, 53268, 62159, 58907, 58895, 41867, 41898, 41909, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 45302, 37318, 37318, 37319, 37318, 44953, 37318, 37318, 58923, 37318, 37038, 61423, 60807, 58942, 58956, 58972, 59001, 58985, 59015, 65689, 38336, 37318, 41289, 37087, 37318, 59031, 37318, 37318, 40327, 37318, 38338, 59049, 40328, 59079, 54363, 41369, 59103, 48516, 62003, 72217, 37318, 37318, 59124, 68839, 37318, 63001, 62951, 44924, 37318, 37318, 37318, 59142, 59160, 59182, 72644, 59203, 59223, 59256, 41542, 57091, 48815, 59272, 59718, 65498, 52599, 56799, 57854, 54450, 41461, 63259, 59312, 52629, 59347, 40308, 54027, 37318, 37318, 37318, 37318, 56028, 37318, 37318, 37318, 73164, 56849, 59370, 68264, 72644, 72644, 41363, 63477, 59429, 70290, 41542, 41542, 65493, 64678, 69828, 70882, 65498, 65498, 46825, 59459, 66600, 59489, 41461, 41461, 47041, 37318, 60820, 72604, 37318, 37318, 56414, 63654, 59509, 37318, 37318, 72643, 61043, 72644, 72644, 59543, 72645, 59566, 72479, 41542, 41542, 59585, 41542, 65528, 73531, 59619, 65498, 70373, 65498, 59800, 54834, 59637, 41461, 64050, 41461, 47863, 54854, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 72642, 72644, 72644, 72644, 72644, 38767, 41542, 41542, 41542, 41542, 41444, 65497, 65498, 65498, 65498, 65498, 59803, 41461, 41461, 41461, 41461, 47863, 54854, 37318, 37318, 37318, 37318, 37318, 60498, 72644, 72644, 72644, 58547, 41542, 41542, 41542, 46770, 65498, 65498, 65498, 59798, 41461, 41461, 41461, 47288, 44851, 37318, 37318, 37318, 37318, 59207, 72644, 60920, 61740, 41542, 41542, 73606, 65498, 65498, 49876, 41461, 41461, 56936, 37318, 37318, 37318, 73163, 72644, 72644, 50031, 61445, 73662, 65498, 59655, 59802, 56876, 41462, 56939, 61615, 37318, 72643, 60923, 41542, 73664, 65498, 59801, 41461, 49498, 59676, 72644, 59695, 59713, 71108, 58802, 59734, 59756, 59775, 59794, 59819, 73161, 59759, 73663, 59797, 47605, 60497, 50030, 47449, 46829, 60496, 59854, 73737, 67066, 53268, 62159, 62339, 66862, 41867, 41898, 41909, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 45302, 37318, 37318, 37318, 67305, 48392, 37318, 54855, 59885, 59896, 59919, 66152, 67290, 59971, 59985, 60001, 60031, 60044, 60015, 65689, 47586, 44905, 38860, 60060, 60084, 37318, 60579, 48333, 40435, 72237, 60111, 60138, 60122, 60182, 60217, 60279, 60316, 65566, 60366, 50482, 58641, 37318, 37318, 48606, 72949, 68216, 73429, 60392, 69215, 60419, 37318, 60465, 60494, 60514, 60535, 72871, 72644, 60555, 49584, 41542, 47939, 52709, 70645, 70888, 65498, 70640, 57854, 54601, 60344, 63674, 41461, 59473, 64205, 46157, 37318, 60571, 37318, 37318, 37318, 60595, 37318, 37318, 45177, 73164, 60613, 72644, 53647, 72644, 72644, 41363, 60640, 41542, 61097, 41542, 41542, 58708, 70406, 65498, 55874, 65498, 65498, 50904, 58204, 41461, 60666, 41461, 41461, 47041, 37318, 37318, 37318, 37318, 40106, 37318, 37318, 44677, 60687, 37318, 60703, 72644, 72644, 46744, 72644, 58087, 60725, 41542, 41542, 41542, 60758, 74033, 52815, 65498, 65498, 65498, 52593, 61759, 41832, 41461, 41461, 41461, 53066, 69906, 47863, 49504, 60776, 60792, 64608, 60842, 51384, 60860, 60895, 60918, 60939, 60974, 61031, 61070, 38767, 70056, 61088, 61121, 61167, 41444, 65497, 61193, 61228, 55536, 61274, 59803, 61293, 61309, 61258, 72531, 61332, 54854, 48246, 61348, 61369, 61414, 58477, 51897, 68272, 58678, 65107, 61439, 53722, 46795, 49566, 61461, 54813, 65498, 61486, 61505, 72623, 61531, 61548, 61568, 67348, 61609, 37318, 37318, 61636, 61678, 47889, 61697, 61718, 52147, 61736, 61756, 56291, 61775, 61796, 55608, 61816, 46251, 61838, 37318, 61855, 73163, 61899, 68974, 61934, 52295, 73662, 61950, 58777, 71058, 61966, 62000, 62019, 63023, 62043, 40852, 60923, 72471, 73664, 62062, 59801, 62083, 49498, 62101, 55233, 62122, 54383, 62157, 62175, 62225, 62247, 62263, 62279, 62295, 62331, 57449, 55864, 54413, 66647, 60497, 50030, 47449, 46829, 60496, 73658, 47451, 47606, 62355, 62391, 62339, 66862, 41867, 41898, 57361, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 45302, 37318, 37318, 37318, 62425, 44953, 37318, 41929, 62645, 37318, 37318, 67919, 62449, 62465, 62479, 62495, 62524, 62508, 62538, 62554, 62570, 62591, 51116, 40267, 62636, 62669, 37763, 62688, 62721, 40114, 62768, 62784, 62800, 62846, 62862, 62878, 62894, 62929, 62967, 62983, 37318, 74513, 46381, 42254, 37318, 40466, 50397, 50445, 63017, 37318, 37318, 63039, 73164, 72644, 63059, 63089, 55789, 45386, 47983, 63105, 51466, 69257, 65498, 47963, 66977, 63139, 63162, 41461, 63187, 63203, 63246, 60376, 37318, 65243, 37318, 54083, 37318, 65923, 63281, 37318, 49694, 37318, 63306, 63323, 63356, 63373, 63389, 72644, 67473, 63412, 41542, 63428, 63500, 41542, 54700, 59443, 65498, 63522, 63557, 65498, 56187, 53414, 41461, 63578, 63594, 41461, 68051, 48021, 46148, 37318, 59832, 55125, 37318, 37318, 37318, 37318, 37318, 63803, 72643, 72644, 70758, 59187, 72644, 72645, 38772, 41542, 55434, 57073, 41542, 41542, 45363, 65498, 74070, 65498, 63613, 65498, 59800, 41461, 60671, 41461, 63633, 41461, 47863, 54854, 37318, 37318, 63653, 44684, 37318, 37318, 37318, 72642, 72644, 72644, 72644, 57012, 55426, 41542, 41542, 41542, 41542, 66835, 62140, 65498, 65498, 65498, 65498, 63670, 41461, 41461, 41461, 41461, 63690, 54854, 67948, 37318, 37318, 37318, 37318, 63712, 72644, 63734, 72644, 46430, 41542, 63751, 41542, 56170, 65498, 63769, 65498, 41577, 41461, 51728, 41461, 47288, 63790, 37318, 37318, 37318, 37318, 62822, 66520, 60920, 41416, 71239, 41542, 47825, 58753, 65498, 61151, 71116, 41461, 72584, 37318, 37318, 37318, 73163, 72644, 72644, 50031, 41542, 73662, 65498, 65498, 59802, 41461, 41462, 45334, 61615, 37318, 63825, 73373, 63858, 64501, 63879, 65885, 41461, 56530, 63897, 63917, 63966, 63993, 64029, 64066, 39534, 64101, 64126, 64163, 64186, 73161, 59759, 73663, 59797, 47605, 60497, 50030, 47449, 46829, 60496, 73658, 47451, 50006, 53268, 62159, 64221, 66862, 41867, 64277, 64293, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 45302, 37318, 37318, 37318, 66665, 44953, 37318, 37318, 68891, 37318, 64328, 64337, 67698, 39838, 64353, 64369, 64398, 64382, 64414, 65689, 37318, 69115, 41289, 37087, 37318, 37318, 63950, 37318, 51547, 37318, 37318, 37318, 44404, 64465, 64494, 64517, 49717, 67057, 64557, 72217, 37318, 64583, 37318, 68839, 37318, 40908, 40502, 64605, 50368, 64607, 37318, 37318, 62231, 64624, 67101, 73183, 63396, 64650, 67404, 68345, 48485, 71344, 64666, 64694, 64712, 71834, 64730, 57181, 60350, 61532, 53072, 55317, 64759, 37318, 64782, 64801, 45664, 45866, 38224, 37318, 37318, 37318, 37318, 73164, 72644, 72644, 52669, 72644, 72644, 41363, 41542, 41542, 67409, 41542, 41542, 65493, 65498, 65498, 55555, 65498, 65498, 46825, 41461, 41461, 68042, 41461, 41461, 49668, 53220, 61353, 37318, 37318, 37318, 64818, 59354, 57374, 68538, 37318, 72643, 72644, 64837, 72644, 72644, 72645, 64870, 41542, 64889, 41542, 41542, 41542, 68453, 65498, 51231, 64925, 65498, 65498, 59800, 41461, 61515, 64943, 41461, 41461, 47863, 54854, 59903, 37318, 37318, 37318, 37909, 37318, 37318, 72642, 72644, 72644, 64960, 72644, 56626, 41542, 41542, 73894, 41542, 42296, 65497, 65498, 65498, 64978, 65498, 59803, 41461, 41461, 71869, 41461, 47863, 54854, 37318, 37318, 37318, 37318, 37318, 60498, 72644, 72644, 72644, 58547, 41542, 41542, 41542, 46770, 65498, 65498, 65498, 59798, 41461, 41461, 41461, 47288, 37318, 37318, 52897, 37318, 59144, 72644, 72644, 60920, 41542, 41542, 41542, 65498, 65498, 65498, 63673, 41461, 41461, 62943, 37318, 37318, 37318, 73163, 72644, 72644, 50031, 41542, 73662, 65498, 65498, 59802, 41461, 41462, 56939, 61615, 37318, 72643, 60923, 41542, 73664, 65498, 59801, 41461, 56828, 64997, 65024, 65045, 64231, 59603, 54339, 38753, 60922, 73660, 47450, 46828, 73161, 59759, 73663, 59797, 47605, 60497, 50030, 47449, 46829, 60496, 73658, 47451, 47606, 53268, 62159, 62339, 66862, 41867, 41898, 41909, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 45302, 37318, 37318, 37318, 37318, 44953, 37318, 37318, 37318, 37318, 37318, 67919, 57964, 37318, 37318, 45323, 45350, 63443, 47000, 65689, 37318, 37318, 41289, 37087, 37318, 46305, 37318, 37318, 37318, 42426, 42429, 65068, 54066, 65102, 51341, 41369, 65123, 50559, 62003, 48585, 65141, 37318, 37318, 68839, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 73164, 72644, 72644, 72644, 72644, 45386, 41542, 41542, 41542, 52709, 65498, 65498, 65498, 65498, 57854, 41461, 41461, 41461, 41461, 60376, 37318, 37318, 37318, 43472, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 73164, 72644, 72644, 72644, 72644, 72644, 65158, 41542, 41542, 41542, 41542, 41542, 66019, 65498, 65498, 65498, 65498, 65498, 46825, 41461, 41461, 41461, 41461, 41461, 47041, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 72643, 72644, 72644, 72644, 72644, 72645, 38772, 41542, 41542, 41542, 41542, 41542, 45363, 65498, 65498, 65498, 65498, 65498, 59800, 41461, 41461, 41461, 41461, 41461, 47863, 54854, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 72642, 72644, 52976, 72644, 72644, 38767, 41542, 41542, 73801, 41542, 41444, 65497, 65498, 49370, 65498, 65498, 59803, 41461, 41461, 65182, 41461, 47863, 54854, 37318, 37318, 37318, 37318, 37318, 60498, 72644, 72644, 72644, 58547, 41542, 41542, 41542, 46770, 65498, 65498, 65498, 59798, 41461, 41461, 41461, 47288, 37318, 37318, 37318, 37318, 37318, 72644, 72644, 60920, 41542, 41542, 41542, 65498, 65498, 65498, 63673, 41461, 41461, 56936, 37318, 37318, 37318, 73163, 51274, 72644, 50031, 65201, 73662, 71079, 65498, 59802, 58261, 41462, 56939, 61615, 37318, 72643, 60923, 41542, 73664, 65498, 59801, 41461, 49498, 72946, 72644, 50030, 54383, 59799, 54339, 61620, 60922, 73660, 47450, 46828, 73161, 59759, 73663, 59797, 47605, 60497, 50030, 47449, 46829, 60496, 73658, 47451, 47606, 53268, 62159, 62339, 66862, 41867, 41898, 41909, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 45302, 37318, 37318, 37318, 37806, 44953, 65219, 37318, 37420, 65239, 37318, 67919, 65259, 65275, 65288, 65304, 65333, 65317, 65347, 65689, 45409, 47790, 41289, 50740, 71165, 72077, 54775, 46691, 68943, 37318, 65363, 65398, 65418, 65465, 65515, 65551, 65582, 68491, 65617, 65675, 37318, 37318, 37318, 55396, 65708, 50432, 65732, 40783, 60879, 37318, 37318, 46888, 65750, 54125, 65773, 60201, 65790, 65831, 49353, 41542, 61177, 57056, 65855, 49783, 65498, 65874, 65901, 41461, 65939, 58230, 64743, 49003, 37318, 37318, 37318, 54951, 44007, 65958, 37318, 73691, 37318, 44004, 69414, 59740, 72644, 52378, 72644, 64962, 65976, 66005, 41542, 53183, 41542, 52448, 66040, 66074, 65498, 73466, 65498, 70854, 66103, 60245, 41461, 66119, 41461, 72537, 66911, 47041, 37318, 37318, 38803, 72980, 60478, 40954, 37318, 37480, 66138, 66168, 66202, 72644, 66235, 46345, 66251, 72645, 66271, 41542, 58177, 66299, 66320, 41542, 56559, 65498, 65535, 66344, 66367, 65498, 69808, 41461, 53763, 66394, 66417, 41461, 47863, 49919, 37318, 37318, 52902, 66451, 37318, 66495, 39978, 72642, 66512, 60519, 72644, 72644, 38767, 65839, 41542, 66536, 41542, 41444, 65497, 66557, 54721, 65498, 65498, 59803, 66582, 73488, 41461, 41461, 52328, 47223, 43477, 62672, 37318, 37318, 37318, 65382, 72644, 72644, 61681, 58099, 41542, 41542, 41542, 66616, 65498, 65498, 65498, 66632, 66681, 41461, 41461, 54252, 37318, 70963, 37318, 37318, 37318, 66698, 72644, 60920, 66716, 41542, 67896, 66733, 65498, 69783, 66751, 41461, 54608, 56936, 37318, 66770, 37318, 73163, 72644, 72644, 50031, 41542, 73662, 65498, 65498, 59802, 41461, 41462, 56939, 61615, 37318, 72643, 60923, 41542, 73664, 65498, 59801, 41461, 56009, 66788, 66811, 66827, 66851, 66889, 49836, 61620, 60922, 73660, 47450, 46828, 42223, 66927, 59778, 49606, 47605, 66943, 50030, 47449, 46829, 60496, 73658, 47451, 52229, 66963, 67001, 67042, 47192, 41867, 41898, 41909, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 45302, 37318, 37318, 37318, 42006, 44953, 41194, 40023, 67082, 63290, 67117, 67133, 67149, 67200, 67214, 67230, 67259, 67243, 67275, 67324, 37318, 67347, 72149, 37087, 37318, 39873, 37318, 37318, 73640, 55694, 39647, 45609, 60403, 67364, 67389, 66058, 64007, 71278, 67425, 72217, 39770, 40165, 39032, 68839, 68408, 37318, 37318, 37318, 37318, 39965, 41180, 37318, 67452, 67489, 70764, 72644, 72644, 67522, 67558, 63467, 41542, 52709, 51619, 65498, 67579, 65498, 57854, 67601, 63637, 67628, 41461, 60376, 37318, 67645, 37318, 67663, 67683, 37318, 56960, 37318, 65223, 37318, 37318, 73164, 52080, 69630, 72644, 72644, 72644, 67733, 60734, 73861, 41542, 41542, 53739, 65493, 58399, 51643, 65498, 65498, 63881, 46825, 67757, 59323, 41461, 41461, 66122, 50625, 37318, 37318, 67778, 47108, 48295, 37318, 38228, 67799, 44787, 57969, 67823, 72644, 72644, 69591, 55783, 67843, 67888, 41542, 41542, 63123, 65166, 73809, 53005, 65498, 65498, 64714, 62913, 63541, 72852, 41461, 41461, 59639, 69865, 70909, 48723, 67912, 59519, 37318, 37900, 37318, 51391, 37318, 67935, 72642, 72644, 52056, 59087, 72644, 38767, 41542, 41542, 67964, 67985, 41444, 65497, 65498, 65858, 58406, 65498, 59803, 41461, 41461, 68002, 41461, 47863, 54854, 37318, 68397, 37318, 68076, 37318, 60498, 68624, 72644, 68100, 65444, 66541, 41542, 68117, 51222, 72818, 65498, 55549, 59798, 53799, 41461, 51777, 47288, 37318, 37318, 40078, 37318, 68138, 72644, 72644, 60920, 41542, 41542, 41542, 65498, 65498, 65498, 63673, 41461, 41461, 65915, 37954, 37318, 68162, 56840, 72644, 69566, 50031, 41542, 68178, 65498, 72029, 59802, 41461, 61324, 68205, 61615, 68239, 68255, 57031, 68288, 49964, 68312, 64170, 41461, 72350, 72946, 63842, 68330, 50053, 68366, 68384, 61620, 60922, 73660, 47450, 46828, 40170, 49310, 63506, 57898, 47605, 68424, 68440, 45370, 63171, 40400, 68476, 68554, 68604, 53268, 62159, 62339, 66862, 41867, 41898, 41909, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 45302, 37318, 37318, 37318, 38247, 68640, 37318, 37318, 44545, 44549, 42062, 66479, 66466, 68687, 68703, 68719, 68748, 68732, 68764, 68811, 49856, 37318, 68835, 73418, 40538, 54530, 37318, 68855, 37963, 68877, 39120, 68925, 68959, 68996, 69012, 46476, 69028, 71470, 69063, 59385, 62027, 45461, 65008, 68839, 69079, 37318, 38220, 69096, 65716, 69131, 41882, 69165, 69175, 69191, 63718, 59550, 63357, 69231, 50874, 46465, 70280, 70118, 64531, 65596, 69273, 69296, 57117, 69318, 73551, 69334, 69350, 69367, 69406, 69430, 37318, 69456, 69475, 69510, 37318, 37318, 68519, 50099, 69530, 44732, 69549, 69582, 60998, 72644, 69624, 41363, 69646, 69670, 69695, 41542, 69720, 65493, 69753, 69776, 69799, 65498, 69824, 48973, 69844, 69860, 69881, 41461, 69903, 51039, 37318, 69922, 62705, 62209, 69940, 52743, 51107, 45640, 57591, 40471, 72643, 69961, 69994, 72644, 48835, 70019, 70047, 70072, 69679, 41542, 41542, 70100, 64902, 48943, 63562, 65498, 65498, 70134, 59800, 68060, 61822, 41461, 41461, 70150, 47863, 72356, 45780, 37318, 73436, 47091, 37318, 37318, 70166, 72704, 72644, 70194, 70221, 70244, 47170, 73209, 73218, 70265, 66328, 41444, 70313, 65498, 70329, 70356, 70389, 60293, 70422, 70441, 54423, 70477, 47863, 50635, 37318, 50706, 66496, 53384, 69533, 46401, 70511, 72644, 72644, 70544, 70560, 70789, 41542, 70593, 70626, 69047, 65498, 52486, 70661, 69887, 41461, 70487, 37318, 41199, 59126, 70706, 37318, 70726, 70746, 56994, 70780, 70823, 72308, 70850, 70870, 65601, 70904, 70925, 51697, 70495, 37450, 37318, 70960, 70979, 71002, 71021, 63456, 46437, 71047, 71074, 71095, 49624, 59493, 71132, 71156, 71181, 41086, 71197, 69608, 71230, 71263, 71294, 71313, 57200, 49498, 72946, 72644, 50030, 54383, 59799, 54339, 61620, 60922, 73660, 47450, 46828, 73161, 59759, 65052, 64909, 59331, 55194, 71329, 71374, 71410, 71439, 71455, 66985, 70676, 53268, 62159, 62339, 66862, 71486, 71502, 41909, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 45302, 37318, 37318, 37318, 37318, 71540, 37318, 37318, 62697, 37318, 37318, 67919, 71622, 71638, 71652, 71668, 71697, 71681, 71711, 65689, 37318, 37318, 41289, 71727, 43243, 37318, 54635, 37318, 47100, 37318, 37318, 37318, 44404, 60624, 71747, 71775, 71820, 71855, 71891, 71918, 50345, 37318, 37318, 68839, 37318, 37318, 64312, 71934, 37318, 37318, 37318, 63901, 73164, 71973, 72644, 72644, 67498, 71994, 41542, 41542, 41542, 72010, 61470, 65498, 65498, 49441, 57854, 72045, 41461, 41461, 72052, 56235, 72068, 37318, 72101, 73266, 37318, 37318, 37318, 72130, 72165, 37318, 45286, 73164, 72644, 72186, 72203, 52036, 66700, 41363, 41543, 41542, 72253, 47431, 72288, 65493, 56731, 65499, 62067, 70610, 72324, 46825, 73981, 63597, 47312, 56493, 72372, 47041, 37318, 38557, 37318, 72400, 37318, 37318, 37318, 52648, 37318, 37318, 72643, 69558, 72644, 72644, 72644, 72645, 72419, 58149, 41542, 41542, 41542, 41542, 61134, 64696, 65498, 65498, 65498, 65498, 59800, 57573, 41461, 41461, 41461, 41461, 47863, 56406, 37318, 67179, 37318, 37318, 37318, 37318, 60902, 62813, 72644, 72644, 72644, 72644, 42340, 41542, 41542, 41542, 41542, 41444, 47816, 65498, 65498, 65498, 65498, 64261, 41461, 41461, 41461, 41461, 47863, 54766, 37318, 37318, 46970, 37318, 37318, 39394, 72444, 72644, 72644, 67855, 72462, 41542, 57537, 51591, 72495, 65498, 65498, 72517, 72553, 41461, 41461, 72576, 37318, 72600, 37318, 37318, 37318, 72644, 72644, 60920, 41542, 41542, 41542, 65498, 65498, 65498, 63673, 41461, 41461, 56936, 37318, 37318, 37318, 73163, 72644, 72644, 50031, 41542, 73662, 65498, 65498, 59802, 41461, 41462, 56939, 61615, 37318, 65757, 60923, 69654, 73664, 69041, 59801, 72620, 49498, 72946, 72644, 50030, 54383, 59799, 54339, 61620, 60922, 73660, 47450, 46828, 73161, 59759, 73663, 59797, 47605, 72639, 72661, 49423, 56501, 60496, 73658, 47451, 47606, 53268, 68368, 54485, 72691, 41867, 41898, 41909, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 45302, 37318, 37318, 37318, 37318, 42479, 51143, 66186, 50148, 37318, 40519, 67015, 72720, 37318, 37318, 72736, 72765, 72749, 72779, 65689, 37318, 37318, 41289, 37087, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 44404, 72795, 49944, 41369, 72816, 50989, 62003, 72217, 37318, 49091, 63043, 68839, 37318, 37318, 37318, 37318, 37318, 52920, 37318, 37318, 73164, 47899, 72644, 73587, 72644, 45386, 72834, 41542, 56677, 52709, 56433, 65498, 65498, 72024, 57854, 55980, 41461, 64944, 41461, 60376, 37318, 37318, 37318, 37318, 37318, 55361, 37318, 37318, 37318, 53853, 37318, 73164, 72644, 72644, 72868, 72644, 72644, 41363, 41542, 41542, 72887, 41542, 41542, 65493, 65498, 65498, 72920, 65498, 65498, 46825, 41461, 56253, 41461, 41461, 41461, 47041, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 72643, 72644, 72644, 72644, 72644, 72645, 38772, 41542, 41542, 41542, 41542, 41542, 45363, 65498, 65498, 65498, 65498, 65498, 59800, 41461, 41461, 41461, 41461, 41461, 47863, 54854, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 72642, 72644, 72644, 72644, 72644, 38767, 41542, 41542, 41542, 41542, 41444, 65497, 65498, 65498, 65498, 65498, 59803, 41461, 41461, 41461, 41461, 47863, 54854, 37318, 40788, 37318, 37318, 37318, 60498, 72644, 72644, 72644, 58547, 41542, 41542, 41542, 46770, 65498, 65498, 65498, 59798, 41461, 41461, 41461, 47288, 37318, 69080, 37318, 37318, 37318, 72644, 72644, 60920, 41542, 41542, 41542, 65498, 65498, 65498, 63673, 41461, 41461, 56936, 37318, 37318, 37318, 73163, 72644, 72644, 50031, 41542, 73662, 65498, 65498, 59802, 41461, 41462, 56939, 72938, 37318, 72643, 60923, 41542, 73664, 65498, 59801, 41461, 49498, 72946, 72644, 50030, 54383, 59799, 54339, 61620, 60922, 73660, 47450, 46828, 73161, 59759, 73663, 59797, 47605, 60497, 50030, 47449, 46829, 60496, 73658, 47451, 47606, 53268, 62159, 62339, 66862, 41867, 41898, 41909, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 45302, 37318, 37318, 37318, 37318, 50678, 37318, 37318, 72976, 37318, 72965, 72996, 73030, 73066, 73080, 73096, 73125, 73109, 73141, 65689, 37318, 37318, 41289, 37087, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 44404, 49521, 50028, 47181, 50958, 47282, 59296, 63932, 37318, 37318, 37318, 68839, 37318, 37318, 37318, 37318, 45204, 37318, 37318, 37318, 73164, 72644, 72644, 72644, 72644, 45386, 41542, 41542, 41542, 52709, 65498, 65498, 65498, 65498, 57854, 41461, 41461, 41461, 41461, 60376, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 73164, 72644, 72644, 72644, 72644, 72644, 41363, 41542, 41542, 41542, 41542, 41542, 65493, 65498, 65498, 65498, 65498, 65498, 46825, 41461, 41461, 41461, 41461, 41461, 47041, 37318, 37318, 37318, 37318, 37318, 48411, 37318, 37318, 37318, 37318, 73180, 72644, 72644, 72644, 72644, 72645, 73199, 41542, 41542, 41542, 41542, 41542, 56282, 65498, 65498, 65498, 65498, 65498, 56756, 41461, 41461, 41461, 41461, 41461, 47863, 54854, 37318, 68223, 37318, 37318, 37318, 37318, 37318, 72642, 72644, 73234, 72644, 56857, 38767, 41542, 66304, 41542, 41542, 73255, 65497, 65498, 73289, 65498, 72501, 59803, 41461, 61976, 41461, 70425, 47863, 54854, 37318, 37318, 37318, 37318, 37318, 60498, 72644, 72644, 72644, 58547, 41542, 41542, 41542, 46770, 65498, 65498, 65498, 59798, 41461, 41461, 41461, 47288, 37318, 37318, 37318, 37318, 37318, 72644, 72644, 60920, 41542, 41542, 41542, 65498, 65498, 65498, 63673, 41461, 41461, 56936, 37318, 37318, 37318, 73163, 72644, 72644, 50031, 41542, 73662, 65498, 65498, 59802, 41461, 41462, 56939, 61615, 37318, 72643, 60923, 41542, 73664, 65498, 59801, 41461, 49498, 72946, 72644, 50030, 54383, 59799, 54339, 61620, 60922, 73660, 47450, 46828, 73161, 59759, 73663, 59797, 47605, 60497, 50030, 47449, 46829, 60496, 73658, 47451, 47606, 53268, 62159, 62339, 66862, 41867, 41898, 41909, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 45555, 37318, 37318, 37318, 37318, 44953, 37318, 37318, 37318, 37318, 37318, 67919, 57964, 37318, 37318, 45323, 45350, 63443, 47000, 65689, 37318, 37318, 41289, 73310, 37318, 39135, 37318, 37318, 37318, 46656, 37318, 73333, 55714, 73368, 64110, 41369, 73389, 53053, 62003, 72217, 37318, 37318, 37318, 73407, 48428, 48554, 40049, 37318, 37318, 37318, 37318, 37318, 73164, 72644, 61072, 72644, 52030, 45386, 41542, 41809, 41542, 73452, 65498, 65498, 58580, 63617, 57854, 41461, 41461, 73487, 55972, 60376, 73504, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 73164, 72644, 72644, 72644, 72644, 72644, 41363, 41542, 41542, 41542, 41542, 41542, 65493, 65498, 65498, 65498, 65498, 65498, 46825, 41461, 41461, 41461, 41461, 41461, 47041, 37318, 37318, 37318, 37318, 37318, 37318, 62308, 37318, 37318, 37318, 72643, 72644, 52957, 72644, 72644, 74050, 38772, 41542, 54792, 41542, 41542, 41542, 48926, 65498, 65498, 73527, 65498, 65498, 59800, 41461, 41461, 73547, 41461, 41461, 47863, 54854, 73567, 37318, 37318, 37318, 37318, 37318, 37318, 72642, 73584, 72644, 72644, 72644, 38767, 72904, 41542, 41542, 41542, 41444, 65497, 73603, 65498, 65498, 65498, 59803, 70455, 41461, 41461, 41461, 47863, 54854, 37318, 37318, 37318, 37318, 37318, 60498, 72644, 72644, 72644, 58547, 41542, 41542, 41542, 46770, 65498, 65498, 65498, 59798, 41461, 41461, 41461, 47288, 37318, 37318, 37318, 37318, 37318, 72644, 72644, 60920, 41542, 41542, 41542, 65498, 65498, 65498, 63673, 41461, 41461, 56936, 37318, 37318, 37318, 73163, 72644, 72644, 50031, 41542, 73662, 65498, 65498, 59802, 41461, 41462, 56939, 61615, 37318, 72643, 60923, 41542, 73664, 65498, 59801, 41461, 49498, 72946, 72644, 50030, 54383, 59799, 54339, 61620, 60922, 73660, 47450, 46828, 73161, 59759, 73663, 59797, 47605, 60497, 50030, 47449, 46829, 60496, 73658, 47451, 47606, 53268, 62159, 62339, 66862, 41867, 41898, 41909, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 45302, 37318, 37318, 37318, 37318, 44953, 37318, 37318, 37318, 37318, 37318, 67919, 57964, 37318, 37318, 45323, 45350, 63443, 47000, 73622, 37318, 37318, 41289, 37087, 37318, 42196, 37318, 37318, 37318, 37318, 37318, 37318, 40000, 72644, 73656, 60650, 55287, 47282, 61984, 73680, 37318, 42131, 37318, 68839, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 41786, 73164, 72644, 72644, 72644, 69600, 45386, 41542, 41542, 41542, 73717, 65498, 65498, 65498, 56205, 57854, 41461, 41461, 41461, 72560, 60376, 37318, 59949, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 73753, 37318, 73164, 52385, 72644, 72644, 73772, 72644, 41363, 69737, 41542, 41542, 73791, 41542, 65493, 49792, 65498, 65498, 73825, 65498, 46825, 50572, 41461, 66682, 41461, 41461, 47041, 37318, 46080, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 72643, 41389, 72644, 72644, 72644, 72645, 73844, 41542, 73877, 41542, 41542, 41542, 51420, 65498, 73910, 65498, 65498, 65498, 49986, 41461, 73935, 41461, 41461, 41461, 47863, 54854, 37318, 37318, 37318, 37318, 37318, 61662, 37318, 72642, 72644, 47364, 72644, 72644, 38767, 41542, 41542, 70798, 41542, 41444, 65497, 65498, 65498, 73958, 65498, 59803, 41461, 41461, 73977, 41461, 47863, 54854, 37318, 37318, 42105, 37318, 37318, 60498, 72644, 72644, 72644, 58547, 41542, 41542, 41542, 46770, 65498, 65498, 65498, 59798, 41461, 41461, 41461, 47288, 73997, 37318, 37318, 40961, 37318, 60952, 72644, 60920, 74016, 41542, 41542, 57892, 65498, 65498, 71387, 41461, 41461, 56936, 37318, 60826, 37318, 73163, 72644, 74049, 50031, 68296, 73662, 65498, 74066, 59802, 53130, 41462, 56939, 61615, 37318, 72643, 60923, 41542, 73664, 65498, 59801, 41461, 49498, 72946, 72644, 50030, 54383, 59799, 54339, 61620, 60922, 73660, 47450, 46828, 73161, 59759, 73663, 59797, 47605, 60497, 50030, 47449, 46829, 60496, 73658, 47451, 47606, 53268, 62159, 62339, 66862, 41867, 41898, 41909, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 45302, 37318, 37318, 37318, 37318, 44953, 37318, 37318, 37318, 37318, 37318, 67919, 61870, 37318, 37318, 45323, 45350, 63443, 47000, 65689, 37318, 37318, 41289, 37087, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 44404, 72644, 50028, 41369, 65498, 47282, 62003, 72217, 37318, 37318, 37318, 68839, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 73164, 72644, 72644, 72644, 72644, 45386, 41542, 41542, 41542, 52709, 65498, 65498, 65498, 65498, 57854, 41461, 41461, 41461, 41461, 60376, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 73164, 72644, 72644, 72644, 72644, 72644, 41363, 41542, 41542, 41542, 41542, 41542, 65493, 65498, 65498, 65498, 65498, 65498, 46825, 41461, 41461, 41461, 41461, 41461, 47041, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 72643, 72644, 72644, 72644, 72644, 72645, 38772, 41542, 41542, 41542, 41542, 41542, 45363, 65498, 65498, 65498, 65498, 65498, 59800, 41461, 41461, 41461, 41461, 41461, 47863, 54854, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 72642, 72644, 72644, 72644, 72644, 38767, 41542, 41542, 41542, 41542, 41444, 65497, 65498, 65498, 65498, 65498, 59803, 41461, 41461, 41461, 41461, 47863, 54854, 37318, 37318, 37318, 37318, 37318, 60498, 72644, 72644, 72644, 58547, 41542, 41542, 41542, 46770, 65498, 65498, 65498, 59798, 41461, 41461, 41461, 47288, 37318, 37318, 37318, 37318, 37318, 72644, 72644, 60920, 41542, 41542, 41542, 65498, 65498, 65498, 63673, 41461, 41461, 56936, 37318, 37318, 37318, 73163, 72644, 72644, 50031, 41542, 73662, 65498, 65498, 59802, 41461, 41462, 56939, 61615, 37318, 72643, 60923, 41542, 73664, 65498, 59801, 41461, 49498, 72946, 72644, 50030, 54383, 59799, 54339, 61620, 60922, 73660, 47450, 46828, 73161, 59759, 73663, 59797, 47605, 60497, 50030, 47449, 46829, 60496, 73658, 47451, 47606, 53268, 62159, 62339, 66862, 41867, 41898, 41909, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 39708, 37318, 37318, 37318, 37318, 64589, 74088, 74086, 40559, 74115, 74104, 74140, 74169, 37318, 74124, 74153, 74190, 74194, 74210, 36991, 37318, 37318, 41289, 37087, 37318, 37318, 74245, 37318, 37318, 37318, 37318, 37318, 65657, 40192, 74266, 55734, 37318, 53472, 40192, 38527, 37031, 44584, 37054, 37076, 37318, 37318, 74250, 37318, 69924, 37107, 37136, 37152, 65815, 69390, 37247, 37265, 53482, 72228, 37116, 37202, 37162, 74643, 37186, 37120, 37206, 37166, 74726, 69388, 37245, 37263, 53480, 37281, 37318, 41092, 37317, 37318, 37318, 45006, 57380, 37335, 37318, 74671, 64449, 74683, 40193, 37351, 37378, 40192, 39199, 37741, 69458, 41098, 41041, 37318, 46044, 37882, 69459, 41099, 41042, 37318, 46045, 37371, 74376, 37355, 37382, 40192, 39202, 38523, 74469, 50180, 37398, 37414, 37436, 50351, 37475, 37318, 58014, 37496, 39340, 37628, 37651, 37674, 55492, 37717, 38230, 37546, 37566, 58028, 41986, 37605, 37523, 37542, 37562, 58024, 37582, 37601, 39304, 37621, 37644, 37667, 37690, 37709, 37733, 74273, 57770, 38317, 37757, 37779, 37822, 46961, 37943, 37979, 37995, 38150, 38170, 38011, 38093, 38038, 38061, 37792, 39434, 38084, 38100, 38045, 38068, 37799, 39441, 38116, 37992, 38147, 38166, 38186, 38022, 38246, 38263, 37318, 38304, 37318, 38354, 38370, 45231, 38397, 38413, 37861, 37459, 38432, 38451, 68861, 37456, 38429, 38448, 38467, 45226, 38483, 38407, 38516, 57783, 51848, 38545, 38580, 73050, 38677, 38610, 38626, 49222, 38643, 38660, 49221, 38642, 38659, 38676, 38693, 38624, 38714, 38739, 38791, 38841, 38876, 38942, 38892, 60443, 38908, 60433, 60449, 38914, 38930, 38946, 38962, 38978, 39013, 39048, 39078, 39105, 39169, 39156, 39172, 39188, 39089, 39218, 39249, 55484, 39265, 39291, 55476, 39311, 39327, 37693, 37318, 65659, 38698, 39356, 39410, 37835, 37848, 39457, 45051, 38498, 38500, 45058, 39494, 39520, 39559, 39504, 39575, 59063, 39606, 39616, 39632, 39681, 39692, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 39708, 37318, 37318, 37318, 37318, 48276, 37318, 37318, 53226, 37318, 37318, 67919, 57964, 74289, 74334, 37318, 37318, 74319, 74303, 36991, 37318, 37318, 41289, 37087, 37318, 37318, 37010, 37318, 37318, 37318, 74350, 37318, 53463, 40192, 38525, 37318, 37318, 74370, 40192, 38527, 37031, 44584, 37054, 37076, 37318, 37318, 74250, 37318, 69924, 37107, 37136, 37152, 65815, 69390, 37247, 37265, 53482, 37182, 37116, 37202, 37162, 37222, 37186, 37120, 37206, 37166, 74726, 69388, 37245, 37263, 53480, 37281, 37318, 41092, 37317, 37318, 37318, 45006, 57380, 37335, 37318, 74671, 64449, 74683, 40193, 37351, 37378, 40192, 39199, 37741, 69458, 41098, 41041, 37318, 46044, 37882, 69459, 41099, 41042, 37318, 46045, 37371, 74376, 37355, 37382, 40192, 39202, 38523, 74469, 50180, 37398, 37414, 37436, 50351, 37475, 37318, 58014, 37496, 39340, 37628, 37651, 37674, 55492, 37717, 38230, 37546, 37566, 58028, 41986, 37605, 37523, 37542, 37562, 58024, 37582, 37601, 39304, 37621, 37644, 37667, 37690, 37709, 37733, 74273, 57770, 38317, 37757, 37779, 37822, 46961, 37943, 37979, 37995, 38150, 38170, 38011, 38093, 38038, 38061, 37792, 39434, 38084, 38100, 38045, 38068, 37799, 39441, 38116, 37992, 38147, 38166, 38186, 38022, 38246, 38263, 37318, 38304, 37318, 38354, 38370, 45231, 38397, 38413, 37861, 37459, 38432, 38451, 68861, 37456, 38429, 38448, 38467, 45226, 38483, 38407, 38516, 57783, 51848, 38545, 38580, 73050, 38677, 38610, 38626, 49222, 38643, 38660, 49221, 38642, 38659, 38676, 38693, 38624, 38714, 38739, 38791, 38841, 38876, 38942, 38892, 60443, 38908, 60433, 60449, 38914, 38930, 38946, 38962, 38978, 39013, 39048, 39078, 39105, 39169, 39156, 39172, 39188, 39089, 39218, 39249, 55484, 39265, 39291, 55476, 39311, 39327, 37693, 37318, 65659, 38698, 39356, 39410, 37835, 37848, 39457, 45051, 38498, 38500, 45058, 39494, 39520, 39559, 39504, 39575, 59063, 39606, 39616, 39632, 39681, 39692, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 39708, 37318, 37318, 37318, 37318, 74174, 37318, 37318, 37318, 37318, 37318, 52120, 37296, 74401, 74403, 37318, 37318, 74392, 74419, 36991, 37318, 37318, 41289, 37087, 37318, 37318, 74464, 37318, 37318, 74485, 37318, 37318, 59413, 40192, 38525, 37318, 37318, 39335, 40192, 38527, 37031, 44584, 37054, 37076, 37318, 37318, 74250, 37318, 69924, 37107, 37136, 37152, 65815, 69390, 37247, 37265, 53482, 74504, 37116, 37202, 37162, 74529, 37186, 37120, 37206, 37166, 74726, 69388, 37245, 37263, 53480, 37281, 37318, 41092, 37317, 37318, 37318, 45006, 57380, 37335, 37318, 74671, 64449, 74683, 40193, 37351, 37378, 40192, 39199, 37741, 69458, 41098, 41041, 37318, 46044, 37882, 69459, 41099, 41042, 37318, 46045, 37371, 74376, 37355, 37382, 40192, 39202, 38523, 74469, 50180, 37398, 37414, 37436, 50351, 37475, 37318, 58014, 37496, 39340, 37628, 37651, 37674, 55492, 37717, 38230, 37546, 37566, 58028, 41986, 37605, 37523, 37542, 37562, 58024, 37582, 37601, 39304, 37621, 37644, 37667, 37690, 37709, 37733, 74273, 57770, 38317, 37757, 37779, 37822, 46961, 37943, 37979, 37995, 38150, 38170, 38011, 38093, 38038, 38061, 37792, 39434, 38084, 38100, 38045, 38068, 37799, 39441, 38116, 37992, 38147, 38166, 38186, 38022, 38246, 38263, 37318, 38304, 37318, 38354, 38370, 45231, 38397, 38413, 37861, 37459, 38432, 38451, 68861, 37456, 38429, 38448, 38467, 45226, 38483, 38407, 38516, 57783, 51848, 38545, 38580, 73050, 38677, 38610, 38626, 49222, 38643, 38660, 49221, 38642, 38659, 38676, 38693, 38624, 38714, 38739, 38791, 38841, 38876, 38942, 38892, 60443, 38908, 60433, 60449, 38914, 38930, 38946, 38962, 38978, 39013, 39048, 39078, 39105, 39169, 39156, 39172, 39188, 39089, 39218, 39249, 55484, 39265, 39291, 55476, 39311, 39327, 37693, 37318, 65659, 38698, 39356, 39410, 37835, 37848, 39457, 45051, 38498, 38500, 45058, 39494, 39520, 39559, 39504, 39575, 59063, 39606, 39616, 39632, 39681, 39692, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 39708, 37318, 37318, 37318, 37318, 48276, 37318, 37318, 37318, 37318, 37318, 67919, 57964, 37318, 37318, 37318, 37318, 37318, 74564, 65689, 37318, 37318, 41289, 37087, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 44404, 72644, 50028, 41369, 65498, 47282, 62003, 69205, 37318, 37318, 37318, 68839, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 73164, 72644, 72644, 72644, 72644, 41324, 41542, 41542, 41542, 58552, 65498, 65498, 65498, 65498, 57854, 41461, 41461, 41461, 41461, 60376, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 73164, 72644, 72644, 72644, 72644, 72644, 41363, 41542, 41542, 41542, 41542, 41542, 65493, 65498, 65498, 65498, 65498, 65498, 46825, 41461, 41461, 41461, 41461, 41461, 47041, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 72643, 72644, 72644, 72644, 72644, 72645, 38772, 41542, 41542, 41542, 41542, 41542, 45363, 65498, 65498, 65498, 65498, 65498, 59800, 41461, 41461, 41461, 41461, 41461, 47863, 54854, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 72642, 72644, 72644, 72644, 72644, 38767, 41542, 41542, 41542, 41542, 41444, 65497, 65498, 65498, 65498, 65498, 59803, 41461, 41461, 41461, 41461, 47863, 54854, 37318, 37318, 37318, 37318, 37318, 60498, 72644, 72644, 72644, 58547, 41542, 41542, 41542, 46770, 65498, 65498, 65498, 59798, 41461, 41461, 41461, 47288, 37318, 37318, 37318, 37318, 37318, 72644, 72644, 60920, 41542, 41542, 41542, 65498, 65498, 65498, 63673, 41461, 41461, 56936, 37318, 37318, 37318, 73163, 72644, 72644, 50031, 41542, 73662, 65498, 65498, 59802, 41461, 41462, 56939, 61615, 37318, 72643, 60923, 41542, 73664, 65498, 59801, 41461, 49498, 72946, 72644, 50030, 54383, 59799, 54339, 61620, 60922, 73660, 47450, 46828, 73161, 59759, 73663, 59797, 47605, 60497, 50030, 47449, 46829, 60496, 73658, 47451, 47606, 53268, 62159, 62339, 66862, 41867, 41898, 41909, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 65960, 37318, 37318, 37318, 37318, 74599, 74601, 37318, 37318, 70178, 74617, 37014, 37318, 37318, 37318, 37318, 37318, 37318, 37010, 37318, 37318, 37318, 37318, 37318, 65657, 40192, 38525, 37318, 37318, 39335, 40192, 38527, 37031, 44584, 37054, 74641, 37318, 37318, 74250, 37318, 69924, 37107, 37136, 37152, 65815, 69390, 37247, 37265, 53482, 72228, 37116, 37202, 37162, 74643, 37186, 37120, 37206, 37166, 74726, 69388, 37245, 37263, 53480, 37281, 37318, 41092, 37317, 37318, 37318, 45006, 57380, 37335, 37318, 74671, 64449, 74683, 40193, 37351, 37378, 40192, 39199, 37741, 69458, 41098, 41041, 37318, 46044, 37882, 69459, 41099, 41042, 37318, 46045, 37371, 74376, 37355, 37382, 40192, 39202, 38523, 74469, 50180, 37398, 37414, 37436, 50351, 37475, 37318, 58014, 37496, 39340, 37628, 37651, 37674, 55492, 37717, 38230, 37546, 37566, 58028, 41986, 37605, 37523, 37542, 37562, 58024, 37582, 37601, 39304, 37621, 37644, 37667, 37690, 37709, 37733, 74273, 57770, 38317, 37757, 37779, 37822, 46961, 37943, 37979, 37995, 38150, 38170, 38011, 38093, 38038, 38061, 37792, 39434, 38084, 38100, 38045, 38068, 37799, 39441, 38116, 37992, 38147, 38166, 38186, 38022, 38246, 38263, 37318, 38304, 37318, 38354, 38370, 45231, 38397, 38413, 37861, 37459, 38432, 38451, 68861, 37456, 38429, 38448, 38467, 45226, 38483, 38407, 38516, 57783, 51848, 38545, 38580, 73050, 38677, 38610, 38626, 49222, 38643, 38660, 49221, 38642, 38659, 38676, 38693, 38624, 38714, 38739, 38791, 38841, 38876, 38942, 38892, 60443, 38908, 60433, 60449, 38914, 38930, 38946, 38962, 38978, 74659, 39048, 39078, 39105, 39169, 39156, 39172, 39188, 39089, 39218, 46272, 55484, 39265, 39291, 55476, 39311, 74721, 37693, 37318, 65659, 38698, 39356, 39410, 37835, 37848, 39457, 45051, 38498, 38500, 45058, 39494, 39520, 39559, 39504, 39575, 59063, 39606, 39616, 39632, 39681, 39692, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 37318, 1, 24578, 3, 0, 0, 0, 0, 0, 0, 0, 180524, 180524, 180524, 180524, 0, 188717, 0, 188717, 180524, 188717, 188717, 188717, 188717, 188717, 188717, 188717, 188717, 188717, 188717, 188717, 188717, 188717, 188717, 188717, 188717, 188717, 188717, 180524, 180524, 188717, 188717, 188717, 188717, 188717, 188717, 188717, 188717, 188717, 188717, 188717, 188717, 368, 188717, 180524, 188717, 188717, 188717, 188717, 188717, 131072, 188717, 188717, 188717, 188717, 188717, 188717, 188717, 188717, 188717, 188717, 188717, 139264, 147456, 188717, 188717, 188717, 188717, 188717, 188717, 188717, 1, 24578, 3, 0, 0, 4366336, 0, 0, 0, 180524, 188717, 302, 303, 0, 0, 307, 0, 0, 0, 307, 0, 0, 0, 4931584, 0, 0, 0, 0, 0, 0, 2367, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1854, 0, 0, 0, 0, 0, 0, 4268032, 306, 307, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 302, 0, 0, 0, 0, 0, 0, 4268032, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 262144, 0, 0, 4857856, 4874240, 0, 0, 4923392, 0, 0, 0, 0, 0, 0, 0, 0, 0, 341, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 5840896, 5849088, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 328, 0, 0, 375, 375, 405, 0, 0, 0, 6275072, 0, 0, 0, 0, 0, 0, 0, 368, 368, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 450560, 0, 0, 0, 0, 4857856, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 5259264, 0, 0, 0, 0, 0, 0, 0, 0, 5414912, 0, 5447680, 0, 5464064, 0, 5414912, 5447680, 0, 0, 5562368, 5636096, 5685248, 0, 5750784, 5873664, 0, 0, 0, 0, 5636096, 5873664, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 5873664, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 6275072, 0, 0, 0, 914, 0, 0, 0, 0, 0, 4857856, 4874240, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 5447680, 0, 5464064, 0, 5480448, 5562368, 0, 0, 0, 5636096, 0, 5685248, 0, 0, 5750784, 0, 0, 0, 0, 0, 6275072, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 990, 0, 0, 0, 4841472, 0, 0, 0, 4898816, 0, 4358144, 4358144, 4358144, 4358144, 4358144, 4358144, 5414912, 4358144, 5447680, 4358144, 5464064, 4358144, 5480448, 5562368, 4358144, 4358144, 4358144, 5636096, 4358144, 5636096, 4358144, 5685248, 4358144, 4358144, 5750784, 4358144, 4358144, 4358144, 4358144, 4358144, 5873664, 4358144, 4358144, 4358144, 4358144, 4358144, 4358144, 4358144, 4923392, 4358144, 4358144, 4358144, 4358144, 4358144, 0, 4923392, 0, 0, 0, 0, 4366336, 0, 0, 0, 0, 418, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 4276224, 1258, 0, 0, 0, 5603328, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 341, 0, 0, 5341184, 0, 5652480, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 4759552, 4358144, 4358144, 4358144, 4358144, 4358144, 5242880, 4358144, 4358144, 4358144, 4358144, 4358144, 4358144, 4358144, 5341184, 4358144, 4358144, 4358144, 4358144, 4358144, 4358144, 0, 0, 5808128, 4358144, 4358144, 4358144, 4825088, 4358144, 4358144, 4358144, 4358144, 4358144, 4358144, 4358144, 4358144, 4358144, 4358144, 5627904, 5652480, 4358144, 5701632, 4358144, 4358144, 5808128, 4358144, 4358144, 4358144, 5668864, 0, 0, 5791744, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 6201344, 6242304, 6250496, 0, 0, 0, 0, 6422528, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 351, 0, 0, 0, 0, 0, 0, 5619712, 0, 0, 0, 0, 0, 0, 0, 5726208, 5758976, 0, 0, 5791744, 0, 0, 0, 0, 0, 3664, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 5111808, 0, 0, 0, 0, 0, 5283840, 0, 0, 0, 0, 0, 5816320, 6291456, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 2389, 0, 2391, 2392, 0, 0, 0, 0, 6062080, 6463488, 0, 5398528, 0, 0, 6479872, 0, 0, 0, 0, 0, 0, 0, 3459, 0, 0, 0, 3463, 0, 0, 0, 0, 0, 0, 6463488, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 303, 0, 0, 0, 0, 4931584, 4939776, 0, 0, 0, 0, 0, 0, 5054464, 0, 0, 0, 0, 0, 0, 0, 0, 5210112, 0, 0, 0, 5210112, 0, 0, 0, 0, 5292032, 0, 0, 0, 0, 5365760, 0, 0, 0, 5455872, 0, 0, 5816320, 0, 5857280, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 327, 401, 0, 6119424, 0, 6168576, 0, 0, 0, 0, 0, 6242304, 0, 6291456, 0, 6316032, 0, 0, 0, 0, 0, 0, 0, 4358144, 4358144, 4931584, 4939776, 4358144, 4358144, 4358144, 4358144, 4358144, 4358144, 5054464, 4358144, 4358144, 4358144, 4358144, 4358144, 4358144, 4358144, 4358144, 5210112, 4358144, 4358144, 4358144, 4358144, 4358144, 4358144, 5210112, 4358144, 4358144, 4358144, 4358144, 5292032, 4358144, 4358144, 4358144, 4358144, 5365760, 4358144, 4358144, 4358144, 5455872, 4358144, 4358144, 4358144, 4358144, 4358144, 4358144, 5455872, 4358144, 4358144, 4358144, 4358144, 4358144, 5554176, 5570560, 5578752, 5619712, 5668864, 4358144, 4358144, 4358144, 5791744, 5816320, 4358144, 5857280, 4358144, 4358144, 4358144, 4358144, 5816320, 4358144, 5857280, 4358144, 4358144, 4358144, 4358144, 4358144, 4358144, 4358144, 4358144, 4358144, 4358144, 4358144, 4358144, 4358144, 0, 0, 0, 6119424, 4358144, 6168576, 4358144, 4358144, 4358144, 4358144, 4358144, 6242304, 4358144, 6291456, 4358144, 6316032, 4358144, 4358144, 4358144, 4358144, 4358144, 4358144, 4358144, 4358144, 4358144, 6463488, 0, 4358144, 4358144, 4358144, 4358144, 4358144, 4358144, 6463488, 4358144, 4358144, 4358144, 4358144, 4358144, 0, 0, 0, 0, 0, 4825088, 0, 0, 0, 0, 0, 0, 0, 6184960, 5316608, 0, 0, 5644288, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 646, 755, 756, 757, 0, 0, 6217728, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 5390336, 5308416, 5488640, 0, 0, 0, 0, 0, 0, 0, 0, 0, 5799936, 0, 0, 5881856, 0, 0, 0, 0, 0, 0, 0, 0, 0, 351, 352, 353, 0, 0, 0, 0, 5070848, 5431296, 0, 6430720, 0, 0, 0, 0, 0, 0, 0, 0, 5160960, 0, 0, 0, 0, 0, 0, 6053888, 0, 0, 0, 0, 0, 5013504, 0, 0, 0, 0, 0, 0, 6053888, 0, 0, 0, 4358144, 4358144, 5013504, 4358144, 4358144, 4358144, 0, 0, 0, 0, 4841472, 0, 0, 0, 4898816, 0, 0, 0, 0, 0, 0, 1851, 0, 1853, 0, 0, 0, 0, 0, 0, 0, 0, 4825088, 0, 0, 0, 0, 0, 0, 0, 0, 2786, 0, 0, 0, 0, 0, 0, 0, 0, 2809, 0, 0, 0, 0, 0, 0, 0, 0, 2834, 0, 0, 0, 0, 0, 0, 0, 0, 3154, 0, 0, 0, 0, 0, 0, 0, 0, 3167, 3168, 0, 0, 0, 0, 0, 0, 0, 0, 0, 6283264, 6332416, 0, 0, 0, 5881856, 0, 5382144, 0, 0, 0, 0, 0, 0, 0, 3665, 3666, 0, 0, 0, 0, 0, 0, 0, 665, 0, 644, 0, 0, 0, 0, 0, 0, 0, 6266880, 4784128, 4358144, 4358144, 4358144, 4358144, 4358144, 4358144, 4358144, 4358144, 4915200, 4358144, 4956160, 4972544, 4358144, 4358144, 4358144, 4358144, 4358144, 4358144, 5070848, 4358144, 4358144, 4358144, 4358144, 4358144, 4358144, 4358144, 5218304, 4358144, 5267456, 4358144, 4358144, 4358144, 6184960, 4358144, 4358144, 6283264, 4358144, 4358144, 6332416, 4358144, 4358144, 4358144, 6389760, 4358144, 4358144, 6430720, 6438912, 4358144, 4358144, 4358144, 6266880, 6488064, 0, 0, 0, 6266880, 0, 0, 4915200, 0, 4956160, 4972544, 0, 0, 0, 0, 0, 0, 5070848, 0, 0, 0, 0, 0, 0, 0, 5218304, 0, 5267456, 0, 0, 0, 0, 5218304, 0, 5267456, 0, 0, 5308416, 5316608, 0, 0, 0, 5431296, 0, 5488640, 0, 0, 0, 0, 0, 0, 0, 0, 6332416, 0, 0, 0, 6389760, 0, 0, 6430720, 6438912, 0, 0, 0, 0, 0, 0, 4784128, 0, 0, 0, 4849664, 0, 0, 0, 0, 0, 4915200, 0, 4956160, 4972544, 0, 0, 0, 6430720, 6438912, 4784128, 4358144, 4358144, 4358144, 4849664, 4358144, 4358144, 4358144, 4358144, 4358144, 4915200, 4358144, 4358144, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 302, 0, 303, 306, 5218304, 4358144, 5267456, 4358144, 4358144, 5308416, 5316608, 4358144, 4358144, 4358144, 5431296, 4358144, 5488640, 4358144, 4358144, 4358144, 4358144, 4358144, 4358144, 4358144, 4358144, 4358144, 4358144, 4358144, 4358144, 5799936, 4358144, 4358144, 5881856, 4358144, 4358144, 4358144, 4358144, 4358144, 4358144, 4358144, 4358144, 4358144, 6103040, 4358144, 4358144, 4358144, 6103040, 4358144, 4358144, 4358144, 6184960, 4358144, 4358144, 4358144, 6283264, 4358144, 4358144, 6332416, 4358144, 4358144, 0, 0, 0, 0, 0, 0, 0, 0, 0, 368640, 0, 0, 0, 0, 0, 0, 2411, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1256, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 4931584, 4939776, 6488064, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 357, 5660672, 5718016, 0, 5865472, 0, 0, 6037504, 0, 0, 6078464, 0, 0, 6340608, 0, 6455296, 0, 0, 0, 0, 420, 420, 420, 420, 597, 597, 420, 420, 420, 420, 420, 420, 420, 420, 420, 420, 420, 420, 420, 0, 420, 0, 0, 0, 0, 5472256, 0, 0, 0, 6209536, 0, 0, 0, 0, 6176768, 0, 0, 0, 0, 0, 0, 6373376, 6389760, 0, 0, 6488064, 6103040, 0, 0, 0, 0, 0, 0, 1850, 0, 0, 0, 0, 0, 0, 0, 0, 0, 662, 0, 0, 0, 0, 0, 0, 0, 0, 0, 4898816, 0, 5709824, 0, 0, 0, 5283840, 0, 0, 0, 0, 5251072, 0, 6414336, 5832704, 0, 5955584, 0, 0, 4358144, 4358144, 4841472, 4358144, 4358144, 4358144, 4898816, 4358144, 4358144, 4358144, 4358144, 4358144, 4358144, 0, 0, 0, 0, 0, 0, 303, 0, 0, 0, 4358144, 5472256, 5521408, 4358144, 4358144, 4358144, 5595136, 5709824, 5718016, 4358144, 5824512, 5865472, 4358144, 4358144, 5922816, 4358144, 4358144, 6021120, 4358144, 6037504, 4358144, 4358144, 6078464, 6111232, 4358144, 6176768, 6209536, 6234112, 4358144, 4358144, 4358144, 4358144, 5283840, 0, 0, 0, 0, 5472256, 5521408, 0, 0, 0, 0, 5595136, 5709824, 5718016, 0, 5824512, 5865472, 0, 0, 5865472, 0, 0, 5922816, 0, 0, 6021120, 0, 6037504, 0, 0, 6078464, 6111232, 0, 6176768, 6209536, 0, 6234112, 0, 0, 6234112, 0, 0, 0, 0, 0, 0, 0, 4358144, 4358144, 4841472, 4358144, 4358144, 4358144, 4898816, 4358144, 5283840, 4358144, 4358144, 4358144, 4358144, 5472256, 5521408, 4358144, 4358144, 4358144, 4358144, 5595136, 5709824, 5718016, 4358144, 4358144, 0, 0, 0, 0, 0, 5193728, 0, 0, 0, 0, 0, 0, 0, 0, 4358144, 4358144, 6209536, 4358144, 6234112, 4358144, 4358144, 4358144, 4358144, 4358144, 4358144, 4358144, 4358144, 4358144, 4358144, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 573440, 0, 0, 5890048, 0, 0, 0, 6160384, 0, 5095424, 5349376, 0, 5275648, 0, 0, 0, 0, 0, 0, 2308, 0, 0, 0, 0, 0, 0, 0, 0, 0, 139264, 147456, 0, 0, 344064, 0, 0, 0, 0, 4997120, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 5947392, 0, 0, 0, 0, 0, 3675, 0, 0, 0, 0, 0, 0, 0, 0, 4012, 528, 5103616, 4358144, 4358144, 5201920, 4358144, 4358144, 4358144, 4358144, 4358144, 4358144, 4358144, 4358144, 4358144, 4358144, 5890048, 4358144, 4358144, 4358144, 6029312, 4358144, 4358144, 4358144, 4358144, 6160384, 4358144, 4358144, 4358144, 4358144, 4358144, 4358144, 6406144, 0, 5103616, 0, 0, 5201920, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 5890048, 0, 0, 0, 0, 6029312, 0, 0, 0, 0, 6160384, 0, 0, 0, 0, 0, 0, 0, 6406144, 6406144, 4358144, 4358144, 4358144, 4358144, 4358144, 4358144, 4358144, 4358144, 4997120, 4358144, 4358144, 5038080, 4358144, 4358144, 4358144, 5095424, 5095424, 5103616, 4358144, 4358144, 5201920, 4358144, 4358144, 4358144, 4358144, 4358144, 4358144, 4358144, 4358144, 4358144, 4358144, 4358144, 4358144, 4358144, 0, 5013504, 0, 4358144, 6406144, 4358144, 4358144, 4358144, 0, 0, 0, 4890624, 0, 0, 0, 0, 0, 0, 0, 0, 3209, 0, 0, 3212, 0, 0, 0, 0, 0, 0, 5898240, 5963776, 0, 0, 6193152, 0, 0, 5406720, 6397952, 5300224, 5234688, 5423104, 0, 0, 0, 0, 0, 3675, 0, 0, 0, 0, 0, 4009, 0, 4011, 528, 528, 0, 0, 0, 0, 0, 0, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 58815, 57893, 57893, 0, 0, 5988352, 0, 0, 6135808, 6307840, 0, 5996544, 4800512, 0, 6356992, 0, 0, 0, 0, 0, 0, 2317, 2318, 0, 2320, 2321, 0, 0, 0, 0, 0, 0, 1207, 0, 1209, 0, 0, 368, 368, 0, 0, 0, 0, 0, 0, 0, 0, 0, 710, 0, 0, 0, 0, 0, 0, 0, 0, 5496832, 0, 0, 0, 0, 0, 5611520, 0, 0, 0, 0, 0, 0, 0, 3673, 0, 0, 0, 0, 0, 0, 0, 0, 693, 0, 0, 0, 0, 0, 0, 368, 4947968, 5021696, 5529600, 0, 0, 5169152, 0, 0, 0, 4800512, 4808704, 4358144, 4358144, 4890624, 4358144, 4947968, 4358144, 4358144, 4358144, 5898240, 4358144, 4358144, 4358144, 4358144, 4358144, 4358144, 4358144, 6307840, 4358144, 4358144, 6356992, 6381568, 5185536, 0, 5234688, 5300224, 0, 0, 5406720, 5529600, 0, 0, 0, 0, 5898240, 0, 0, 0, 0, 0, 0, 0, 0, 6307840, 0, 0, 6356992, 6381568, 6397952, 4800512, 4808704, 4358144, 4358144, 4890624, 4358144, 4947968, 4358144, 4358144, 4358144, 5046272, 4358144, 4358144, 4358144, 4358144, 5185536, 4358144, 5234688, 5300224, 4358144, 4358144, 5406720, 5529600, 4358144, 4358144, 4358144, 4358144, 5898240, 4358144, 4358144, 4358144, 4358144, 4358144, 4358144, 4358144, 4358144, 6307840, 4358144, 4358144, 6356992, 6381568, 6397952, 5021696, 4358144, 4358144, 5021696, 0, 0, 0, 4980736, 0, 0, 0, 0, 0, 5373952, 5734400, 6045696, 0, 0, 0, 0, 421, 421, 421, 421, 421, 421, 421, 421, 421, 421, 421, 421, 421, 0, 421, 0, 6258688, 6447104, 0, 0, 6127616, 0, 6348800, 5906432, 0, 5537792, 3675, 4882432, 0, 0, 0, 0, 0, 0, 2329, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1195, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 5939200, 0, 0, 5677056, 6365184, 4866048, 0, 6070272, 5545984, 5152768, 0, 0, 0, 0, 0, 3675, 0, 0, 0, 0, 4008, 0, 4010, 0, 528, 4013, 6144000, 4358144, 4866048, 4882432, 4358144, 4980736, 4358144, 4358144, 4358144, 4358144, 4358144, 4358144, 4358144, 4358144, 5324800, 5373952, 5537792, 5545984, 5586944, 5734400, 5971968, 4358144, 6045696, 4358144, 6070272, 4358144, 4358144, 5537792, 5545984, 5734400, 5971968, 4358144, 6045696, 4358144, 6070272, 4358144, 4358144, 4358144, 6348800, 0, 4866048, 4882432, 0, 0, 0, 0, 644, 0, 0, 0, 803, 0, 808, 0, 794, 0, 820, 0, 0, 0, 0, 735, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 450560, 450560, 0, 0, 450560, 0, 6045696, 0, 6070272, 0, 0, 0, 0, 6348800, 0, 4866048, 4882432, 0, 4980736, 0, 0, 0, 0, 0, 0, 0, 0, 5324800, 5373952, 5537792, 5545984, 5586944, 5734400, 5971968, 0, 6045696, 0, 6070272, 0, 0, 0, 0, 6348800, 4358144, 4866048, 4882432, 4358144, 4980736, 4358144, 4358144, 4358144, 4358144, 4358144, 4358144, 4358144, 4358144, 6299648, 4358144, 4358144, 4358144, 4358144, 4358144, 4358144, 4358144, 4358144, 4358144, 4358144, 4358144, 4358144, 6348800, 4358144, 6144000, 0, 6144000, 0, 4988928, 5005312, 0, 0, 0, 0, 5775360, 0, 0, 0, 0, 749, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 759, 0, 0, 3675, 0, 0, 0, 0, 0, 0, 5693440, 0, 6496256, 5144576, 5136384, 0, 5914624, 4358144, 6324224, 0, 0, 5005312, 0, 0, 0, 5120000, 5136384, 0, 0, 0, 0, 0, 0, 0, 1252, 0, 0, 0, 748, 1259, 0, 0, 0, 0, 0, 0, 0, 6324224, 0, 0, 5005312, 0, 0, 0, 5120000, 5136384, 0, 0, 0, 0, 0, 0, 6463488, 4358144, 4358144, 4358144, 4358144, 4358144, 4358144, 4358144, 4358144, 4358144, 4358144, 6324224, 5914624, 5914624, 0, 0, 0, 0, 0, 5513216, 5783552, 0, 3675, 0, 0, 0, 0, 0, 0, 0, 0, 4358144, 4358144, 4358144, 4358144, 4358144, 4358144, 4358144, 4358144, 4358144, 4358144, 4931584, 4939776, 4358144, 4358144, 4358144, 0, 6053888, 0, 0, 0, 0, 0, 0, 0, 0, 6012928, 4358144, 4358144, 5013504, 4358144, 4358144, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1153, 0, 0, 0, 0, 0, 3675, 0, 0, 4006, 4007, 0, 0, 0, 0, 528, 528, 528, 528, 3220, 528, 528, 528, 528, 528, 528, 3227, 4358144, 4358144, 4358144, 4358144, 4358144, 4358144, 4358144, 6053888, 4358144, 4358144, 0, 0, 5013504, 0, 0, 0, 0, 0, 312, 414, 0, 0, 139264, 147456, 0, 0, 0, 0, 0, 0, 0, 6103040, 0, 0, 0, 6184960, 0, 0, 0, 6283264, 0, 0, 6332416, 0, 0, 0, 6389760, 4358144, 4358144, 4358144, 4358144, 4358144, 4358144, 6053888, 4358144, 4358144, 4358144, 0, 0, 0, 0, 5193728, 0, 0, 0, 0, 761, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 528, 528, 528, 3700, 528, 528, 528, 5742592, 0, 0, 0, 0, 0, 4358144, 4907008, 4358144, 5079040, 4358144, 5226496, 4358144, 5742592, 4358144, 4358144, 4358144, 6094848, 4358144, 5062656, 0, 0, 0, 0, 0, 4358144, 4358144, 6094848, 0, 4907008, 0, 5079040, 0, 5226496, 0, 5742592, 0, 0, 0, 6094848, 0, 0, 0, 0, 0, 3675, 0, 4005, 0, 0, 0, 0, 0, 0, 528, 528, 528, 528, 4155, 528, 528, 528, 528, 528, 528, 4907008, 0, 5079040, 0, 5226496, 0, 5742592, 0, 0, 0, 6094848, 0, 4358144, 4907008, 4358144, 5079040, 5062656, 4358144, 4358144, 4358144, 4358144, 4358144, 0, 5062656, 0, 0, 0, 0, 0, 6225920, 0, 5062656, 4358144, 4358144, 4358144, 4358144, 4358144, 913, 5063569, 913, 913, 913, 913, 913, 6226833, 989, 5063645, 0, 0, 0, 4816896, 4358144, 4358144, 4358144, 4358144, 6086656, 4816896, 0, 0, 0, 0, 6086656, 4816896, 4358144, 4358144, 4358144, 4358144, 6086656, 0, 5087232, 0, 5931008, 4358144, 5332992, 5980160, 4358144, 0, 5332992, 5980160, 0, 0, 5332992, 5980160, 0, 4358144, 5332992, 5980160, 4358144, 0, 0, 0, 0, 787, 0, 0, 0, 0, 806, 0, 0, 0, 0, 0, 0, 0, 3166, 0, 0, 0, 0, 0, 0, 0, 0, 327, 0, 0, 0, 0, 0, 0, 0, 5439488, 5128192, 4358144, 5128192, 0, 5128192, 0, 5128192, 4358144, 0, 4358144, 0, 0, 4358144, 0, 4358144, 0, 0, 4358144, 6004736, 6004736, 6004736, 6004736, 6004736, 0, 0, 0, 1, 24578, 3, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 339, 340, 0, 0, 221652, 221652, 221652, 468, 468, 468, 468, 468, 468, 468, 468, 468, 468, 468, 468, 221652, 468, 221652, 221652, 221652, 468, 221652, 221652, 221652, 221652, 221652, 221652, 221652, 221652, 1, 24578, 3, 0, 0, 4366336, 0, 0, 0, 0, 0, 302, 303, 0, 0, 0, 0, 1162, 0, 0, 1164, 0, 0, 0, 0, 0, 0, 0, 0, 1311, 0, 0, 0, 0, 0, 1179, 0, 0, 0, 762, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 383, 0, 0, 0, 0, 1064, 0, 0, 0, 4358144, 4358144, 4358144, 4358144, 4358144, 4358144, 4358144, 4358144, 4358144, 4358144, 4358144, 4358144, 4358144, 913, 913, 913, 4358144, 4358144, 1064, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 507, 514, 514, 1, 24578, 3, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 237568, 302, 0, 306, 237568, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 643, 0, 0, 4268032, 98304, 307, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 661, 0, 0, 4210979, 24578, 3, 0, 0, 297, 0, 0, 0, 0, 297, 0, 0, 0, 0, 0, 0, 0, 3685, 0, 0, 0, 3688, 3689, 0, 0, 0, 0, 0, 245760, 245760, 245760, 0, 0, 0, 0, 139264, 147456, 245760, 245760, 0, 0, 245760, 0, 4210979, 24578, 3, 0, 0, 4366336, 0, 0, 0, 0, 0, 302, 303, 0, 0, 0, 0, 1164, 1297, 0, 0, 0, 0, 0, 0, 1304, 0, 0, 0, 0, 0, 2859, 0, 0, 0, 0, 0, 0, 0, 0, 0, 2867, 368, 368, 0, 0, 0, 303104, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 647, 0, 0, 0, 0, 528, 1158, 0, 4857856, 4874240, 0, 0, 4923392, 0, 0, 0, 0, 0, 0, 0, 0, 0, 379, 0, 0, 0, 0, 371, 0, 4358144, 4358144, 4358144, 4358144, 0, 1984, 0, 0, 0, 4825088, 0, 0, 0, 0, 0, 0, 0, 1253, 0, 0, 0, 0, 0, 0, 0, 0, 2399, 2400, 0, 0, 2402, 0, 0, 0, 0, 0, 262144, 262144, 262144, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 3449, 0, 0, 0, 0, 0, 262144, 262144, 0, 262144, 0, 0, 0, 139264, 147456, 262144, 0, 0, 0, 0, 0, 0, 2344, 2345, 0, 0, 0, 0, 0, 0, 0, 0, 798, 0, 0, 0, 0, 0, 809, 0, 0, 0, 262144, 0, 262738, 262738, 262738, 262738, 262738, 262738, 262738, 262738, 262738, 262738, 262738, 262738, 262738, 0, 262738, 0, 0, 262738, 1, 24578, 3, 0, 0, 4366336, 0, 0, 0, 0, 0, 302, 303, 0, 0, 0, 0, 1177, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 4086, 528, 528, 528, 528, 0, 270336, 0, 0, 0, 0, 4358144, 4358144, 4358144, 4358144, 4358144, 4358144, 4358144, 4358144, 4358144, 4358144, 4358144, 4358144, 4358144, 4358144, 4358144, 4358144, 5177344, 278528, 278528, 0, 0, 131072, 278528, 0, 0, 0, 0, 278528, 0, 0, 0, 0, 278528, 278528, 278528, 278528, 278528, 278528, 278528, 278528, 278528, 278528, 278528, 278528, 278528, 278528, 0, 278528, 0, 0, 278528, 1, 24578, 3, 0, 0, 4366336, 0, 0, 0, 0, 0, 302, 303, 0, 0, 0, 0, 1205, 1206, 0, 0, 0, 0, 0, 368, 368, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 712, 0, 0, 0, 0, 1, 24578, 3, 0, 0, 4366336, 0, 0, 0, 0, 0, 302, 638, 0, 0, 0, 0, 1218, 0, 0, 0, 0, 1223, 0, 0, 0, 0, 0, 0, 0, 1752, 0, 0, 0, 1755, 0, 0, 1758, 0, 4268032, 306, 641, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 662, 0, 0, 528, 0, 1430, 0, 0, 0, 0, 0, 4857856, 4874240, 0, 0, 0, 0, 0, 0, 0, 0, 3686, 0, 0, 0, 0, 0, 0, 0, 0, 3826, 0, 3675, 0, 0, 0, 0, 0, 0, 6275072, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1525, 0, 0, 0, 0, 0, 3675, 4004, 0, 0, 0, 0, 0, 0, 0, 528, 528, 4209, 528, 4210, 528, 528, 528, 528, 528, 1, 24578, 3, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 286720, 303, 0, 307, 286720, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 663, 0, 792, 0, 307, 0, 0, 0, 0, 0, 0, 307, 139264, 287139, 0, 0, 0, 307, 0, 0, 0, 0, 1234, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 2414, 0, 0, 0, 0, 0, 307, 1, 24578, 3, 0, 0, 4366336, 0, 0, 0, 0, 0, 302, 66175, 0, 0, 0, 0, 1250, 0, 0, 0, 1255, 0, 0, 0, 0, 0, 0, 0, 661, 0, 0, 0, 0, 0, 0, 0, 0, 378, 0, 363, 0, 0, 0, 0, 0, 4268032, 306, 98946, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 728, 0, 0, 0, 0, 122880, 0, 4268032, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 2310144, 0, 0, 0, 0, 307, 4857856, 4874240, 0, 0, 4923392, 0, 0, 0, 0, 0, 0, 0, 0, 0, 528, 528, 3699, 528, 528, 528, 528, 1, 24578, 3, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 304, 0, 304, 0, 304, 304, 304, 295216, 304, 304, 304, 295216, 295216, 304, 295216, 304, 304, 131072, 304, 304, 304, 304, 304, 295216, 304, 304, 304, 304, 295216, 304, 304, 304, 304, 295286, 295216, 295216, 295216, 295216, 295216, 295216, 304, 304, 304, 304, 304, 0, 0, 304, 304, 295216, 304, 304, 304, 304, 304, 304, 304, 304, 304, 304, 304, 304, 369, 304, 0, 295216, 304, 295286, 295216, 295216, 295216, 304, 304, 304, 295286, 139264, 147456, 295216, 295216, 304, 304, 295216, 295216, 295216, 304, 304, 304, 304, 304, 304, 304, 304, 304, 304, 304, 304, 304, 304, 304, 304, 295216, 295286, 295286, 295286, 295286, 295286, 295286, 295286, 295286, 295286, 295286, 295286, 295286, 295286, 295216, 295216, 295216, 295216, 295216, 304, 304, 304, 295216, 304, 304, 304, 304, 304, 304, 304, 295216, 374, 304, 304, 304, 304, 304, 304, 304, 295216, 304, 295216, 295216, 295216, 295216, 295216, 295216, 295216, 295216, 295286, 295286, 295286, 295286, 295286, 295286, 295216, 295216, 1, 24578, 3, 0, 0, 4366336, 0, 0, 0, 0, 0, 302, 303, 0, 0, 0, 0, 1267, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 3170, 0, 0, 0, 0, 0, 0, 0, 6275072, 0, 0, 0, 0, 0, 0, 0, 4399804, 4399804, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 4399804, 0, 0, 319488, 0, 0, 0, 0, 0, 0, 319488, 319488, 0, 0, 0, 0, 0, 0, 0, 3696, 0, 528, 528, 528, 528, 528, 528, 528, 528, 528, 528, 528, 3849, 528, 528, 528, 319488, 0, 0, 0, 319488, 319488, 319488, 319488, 319488, 319488, 319488, 319488, 319488, 319488, 319488, 319488, 319488, 319488, 319488, 319488, 319488, 319488, 0, 24578, 3, 0, 0, 4366336, 253952, 0, 0, 0, 0, 302, 303, 0, 0, 0, 0, 1282, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1241, 0, 0, 0, 0, 368, 425984, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 4399805, 0, 0, 0, 6275072, 0, 0, 0, 0, 0, 0, 0, 368, 0, 0, 0, 0, 0, 0, 2357, 0, 0, 0, 0, 0, 0, 0, 0, 0, 3461, 0, 0, 0, 0, 0, 0, 0, 327680, 327680, 327680, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 327680, 327680, 1, 24578, 3, 0, 0, 4366336, 0, 0, 0, 0, 0, 302, 303, 0, 327680, 327680, 327680, 327680, 327680, 335872, 327680, 327680, 327680, 335872, 327680, 327680, 327680, 327680, 327680, 327680, 49723, 0, 0, 0, 0, 0, 0, 0, 49723, 49723, 0, 0, 0, 0, 0, 0, 0, 1270, 0, 0, 0, 0, 0, 0, 0, 0, 0, 5627904, 5652480, 0, 5701632, 0, 0, 0, 0, 0, 6275072, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 49152, 990, 0, 0, 0, 0, 0, 3683, 0, 0, 0, 0, 0, 0, 0, 3690, 0, 0, 0, 0, 0, 3836, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 5242880, 0, 0, 0, 0, 0, 0, 0, 5341184, 0, 0, 0, 0, 0, 344064, 344064, 344064, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 344064, 344064, 344064, 344064, 344064, 344064, 344064, 344064, 344064, 344064, 344064, 344064, 344064, 344064, 344064, 344064, 344064, 344064, 344064, 344064, 344064, 344064, 1, 24578, 3, 0, 0, 4366336, 0, 0, 0, 0, 0, 302, 303, 0, 352256, 352256, 1, 24578, 3, 0, 0, 4366336, 0, 0, 0, 0, 0, 302, 303, 0, 0, 0, 0, 1309, 0, 0, 0, 0, 0, 0, 0, 0, 1315, 0, 0, 0, 0, 371, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 3438, 0, 0, 3440, 0, 1, 292, 3, 0, 0, 0, 298, 0, 0, 0, 0, 0, 0, 0, 0, 0, 528, 3698, 528, 528, 528, 3701, 528, 0, 360448, 360448, 360448, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 360448, 360448, 360448, 360448, 360448, 360448, 360448, 360448, 360448, 360448, 360448, 360448, 360448, 360448, 360448, 360448, 360448, 360448, 360448, 360448, 360448, 360448, 1, 0, 3, 155942, 155942, 296, 0, 636, 0, 0, 0, 302, 303, 0, 1230, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 368, 528, 528, 1376, 528, 528, 528, 528, 528, 528, 528, 528, 528, 528, 528, 528, 528, 528, 2460, 528, 0, 914, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 60322, 57943, 57943, 57943, 57943, 59004, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 528, 57893, 3998, 0, 4000, 528, 528, 528, 528, 58773, 0, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 0, 57916, 57916, 57916, 57916, 57916, 528, 528, 528, 2437, 528, 528, 528, 528, 528, 528, 528, 528, 528, 528, 528, 528, 528, 2446, 528, 2448, 0, 0, 0, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 59856, 57893, 57893, 57893, 57893, 57893, 57893, 60873, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 61213, 57893, 57893, 57893, 57893, 61214, 61215, 57893, 57893, 57893, 57893, 60300, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 0, 0, 0, 0, 0, 60432, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 528, 0, 0, 0, 3191, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 319, 0, 0, 0, 0, 0, 0, 3682, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 338, 0, 0, 0, 528, 57893, 61069, 57893, 57893, 57893, 61072, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 58331, 0, 57916, 57916, 57916, 57916, 57916, 61080, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 59358, 61101, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 61109, 57916, 57916, 57916, 57916, 57916, 57916, 60360, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57943, 60707, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 61138, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 59568, 57943, 57943, 57943, 528, 528, 3852, 3853, 528, 528, 528, 528, 528, 528, 528, 528, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 58291, 57893, 57893, 57943, 57943, 57943, 61251, 57943, 57943, 57943, 57943, 61252, 61253, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 58468, 57943, 57943, 57943, 57943, 528, 528, 528, 898, 0, 0, 3675, 0, 0, 3929, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 695, 0, 0, 0, 0, 368, 528, 528, 57893, 57893, 57893, 57893, 57893, 57893, 61299, 57893, 57893, 57893, 57893, 57893, 57893, 61305, 57893, 57893, 57893, 61309, 57893, 57916, 57916, 57916, 57916, 57916, 57916, 61315, 57916, 57916, 57916, 57916, 57916, 57943, 57943, 61128, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 60066, 57943, 57943, 57943, 57943, 57943, 57943, 57916, 57916, 61321, 57916, 57916, 57916, 61325, 57916, 57943, 57943, 57943, 57943, 57943, 57943, 61331, 57943, 528, 1951, 528, 2283, 528, 57893, 59394, 57893, 59632, 57893, 2290, 0, 2291, 0, 0, 0, 0, 0, 417792, 0, 417792, 0, 0, 0, 0, 310, 0, 0, 0, 0, 0, 316, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1328, 0, 0, 0, 0, 0, 528, 528, 57893, 57893, 57893, 57893, 61511, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 58826, 57893, 57893, 57893, 57893, 57893, 57893, 57916, 57916, 57916, 57916, 61524, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57943, 57943, 57943, 60024, 57943, 60026, 57943, 57943, 57943, 57943, 57943, 61537, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 0, 0, 0, 0, 0, 0, 4244, 528, 528, 528, 57893, 57893, 57893, 57893, 57916, 57916, 57916, 57916, 57943, 57943, 57943, 57943, 0, 0, 0, 0, 1310, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1316, 1269, 0, 528, 528, 57893, 57893, 57916, 57916, 57943, 57943, 0, 528, 57893, 57916, 57943, 0, 528, 57893, 57916, 57943, 0, 528, 57893, 57916, 57943, 0, 0, 0, 0, 376832, 376832, 376832, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 346, 0, 0, 0, 0, 420, 1, 24578, 3, 0, 0, 4366336, 0, 0, 0, 0, 0, 302, 303, 0, 0, 0, 0, 1731, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 648, 0, 0, 0, 0, 0, 0, 0, 0, 4268780, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 6119424, 0, 6168576, 0, 0, 0, 0, 308, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 354, 355, 356, 0, 0, 393685, 393685, 393685, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 368, 0, 0, 245760, 0, 0, 475136, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 761, 0, 0, 305, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 411, 0, 421, 1, 24578, 3, 155942, 155942, 296, 0, 0, 0, 0, 0, 302, 303, 0, 0, 0, 0, 1731, 0, 0, 0, 0, 1738, 0, 1740, 0, 0, 0, 0, 0, 0, 3180, 0, 0, 0, 0, 0, 0, 0, 0, 0, 352256, 0, 352256, 352256, 0, 0, 0, 305, 306, 307, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1185, 0, 0, 0, 0, 0, 748, 421, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 337, 0, 0, 0, 0, 57916, 0, 0, 0, 748, 57893, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 528, 528, 3413, 57893, 57893, 60760, 1725, 0, 0, 0, 1731, 1732, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 741, 0, 0, 0, 0, 0, 0, 0, 0, 1252, 1825, 0, 0, 0, 0, 1259, 1259, 0, 0, 0, 0, 0, 0, 0, 4083, 0, 4085, 0, 528, 528, 528, 528, 528, 528, 528, 528, 528, 528, 3848, 528, 528, 528, 528, 0, 0, 2293, 1732, 2294, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 368, 368, 1212, 0, 0, 528, 528, 2499, 2932, 0, 0, 0, 0, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 59860, 528, 528, 528, 2932, 0, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 0, 0, 2592, 0, 0, 417792, 0, 0, 0, 0, 417792, 0, 0, 0, 0, 0, 310, 0, 310, 0, 0, 0, 0, 0, 4151, 4152, 4153, 528, 528, 528, 4156, 528, 4158, 528, 528, 0, 0, 0, 0, 0, 0, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 60283, 417792, 417792, 0, 0, 417792, 417792, 417792, 417792, 417792, 417792, 417792, 417792, 417792, 417792, 417792, 417792, 418102, 417792, 417792, 418101, 418102, 417792, 417792, 418101, 417792, 418101, 417792, 0, 417792, 0, 0, 0, 0, 417792, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 310, 310, 310, 0, 418101, 417792, 1, 24578, 3, 0, 0, 4366971, 0, 0, 0, 0, 0, 302, 303, 311296, 4399805, 0, 0, 0, 311296, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 800, 0, 0, 0, 0, 0, 0, 0, 0, 4268032, 306, 307, 0, 434176, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 381, 0, 384, 0, 0, 0, 0, 0, 6275072, 0, 0, 0, 0, 0, 0, 0, 0, 4399805, 0, 0, 0, 0, 0, 363, 0, 0, 0, 0, 0, 0, 368, 0, 296, 0, 0, 914, 913, 913, 913, 913, 913, 4858769, 4875153, 913, 913, 913, 913, 913, 913, 913, 913, 913, 5628817, 5653393, 913, 5702545, 913, 913, 913, 913, 5448593, 913, 5464977, 913, 5481361, 5563281, 913, 913, 913, 5637009, 913, 5686161, 913, 913, 5751697, 913, 6275985, 913, 913, 913, 913, 913, 913, 913, 913, 913, 0, 0, 990, 989, 989, 989, 989, 6087645, 4817809, 4359057, 4359057, 4359057, 4359057, 6087569, 0, 5087232, 0, 5931008, 4358144, 989, 989, 5260253, 989, 989, 989, 989, 989, 989, 989, 989, 5415901, 989, 5448669, 989, 5465053, 989, 5481437, 5563357, 989, 989, 989, 5637085, 989, 5686237, 989, 989, 5751773, 989, 989, 989, 989, 989, 989, 6300637, 989, 989, 989, 989, 989, 989, 989, 989, 989, 989, 5112797, 989, 989, 989, 989, 989, 989, 5874653, 989, 989, 989, 989, 989, 989, 989, 989, 989, 989, 989, 6276061, 989, 989, 989, 4858845, 4875229, 989, 989, 989, 989, 989, 989, 989, 989, 989, 989, 989, 989, 989, 5178333, 989, 989, 4359057, 5637009, 4359057, 5686161, 4359057, 4359057, 5751697, 4359057, 4359057, 4359057, 4359057, 4359057, 5874577, 4359057, 4359057, 4359057, 4359057, 5186449, 4359057, 5235601, 5301137, 4359057, 4359057, 5407633, 5530513, 4359057, 4359057, 4359057, 4359057, 4359057, 6300561, 4359057, 4359057, 4359057, 4359057, 4359057, 4359057, 4359057, 4359057, 4359057, 4359057, 4359057, 5112721, 4359057, 4359057, 4359057, 4359057, 989, 989, 5809041, 4359057, 4359057, 4359057, 4826001, 4359057, 4359057, 4359057, 4359057, 4359057, 4359057, 4359057, 4359057, 4359057, 4359057, 4359057, 5178257, 4359057, 4359057, 4359057, 4359057, 4359057, 5243793, 4359057, 4359057, 4359057, 4359057, 4359057, 4359057, 4359057, 5342097, 4359057, 4359057, 4359057, 4359057, 4359057, 4359057, 5415825, 4359057, 5448593, 4359057, 5464977, 4359057, 5481361, 5563281, 4359057, 4359057, 0, 0, 0, 913, 913, 913, 913, 913, 913, 913, 913, 913, 913, 913, 4932497, 4940689, 913, 913, 6464401, 0, 0, 0, 0, 989, 989, 989, 989, 989, 989, 989, 989, 989, 4998109, 989, 989, 5039069, 989, 989, 989, 5096413, 989, 5456861, 989, 989, 989, 989, 989, 5555165, 5571549, 5579741, 5620701, 5669853, 989, 989, 989, 5792733, 5817309, 989, 5858269, 989, 989, 989, 989, 989, 989, 989, 989, 989, 989, 989, 989, 989, 989, 989, 989, 4359057, 4359057, 4359057, 6120413, 989, 6169565, 989, 989, 989, 989, 989, 6243293, 989, 6292445, 989, 6317021, 989, 989, 989, 989, 989, 5071837, 989, 989, 989, 989, 989, 989, 989, 5219293, 989, 5268445, 4359057, 4359057, 4932497, 4940689, 4359057, 4359057, 4359057, 4359057, 4359057, 4359057, 5055377, 4359057, 4359057, 4359057, 4359057, 4359057, 4359057, 6054801, 4359057, 4359057, 4359057, 0, 0, 0, 0, 5193728, 0, 4359057, 5456785, 4359057, 4359057, 4359057, 4359057, 4359057, 5555089, 5571473, 5579665, 5620625, 5669777, 4359057, 4359057, 4359057, 5792657, 5817233, 4359057, 5858193, 4359057, 4359057, 4359057, 4359057, 4359057, 4359057, 4359057, 4359057, 4359057, 4359057, 4359057, 4359057, 4359057, 4359057, 4359057, 4359057, 0, 5013504, 0, 6120337, 4359057, 6169489, 4359057, 4359057, 4359057, 4359057, 4359057, 6243217, 4359057, 6292369, 4359057, 6316945, 4359057, 4359057, 4359057, 5194641, 4359057, 4359057, 4359057, 4359057, 4359057, 4359057, 4359057, 4359057, 4907008, 0, 5079040, 6094848, 6430720, 6438912, 0, 0, 0, 0, 0, 0, 4785041, 913, 913, 913, 4850577, 913, 913, 913, 913, 913, 913, 5055377, 913, 913, 913, 913, 913, 913, 913, 913, 5211025, 5489553, 913, 913, 913, 913, 913, 913, 913, 913, 913, 5800849, 913, 913, 5882769, 913, 913, 913, 913, 5219217, 913, 5268369, 913, 913, 5309329, 5317521, 913, 913, 913, 5432209, 913, 913, 913, 6333329, 913, 913, 913, 6390673, 913, 913, 6431633, 6439825, 0, 0, 0, 0, 0, 0, 0, 393685, 0, 393685, 393685, 393685, 393685, 393685, 393685, 393685, 393685, 393685, 393685, 393685, 393685, 393685, 393685, 0, 393685, 0, 4785117, 989, 989, 989, 4850653, 989, 989, 989, 989, 989, 4916189, 989, 4957149, 4973533, 989, 989, 989, 6030301, 989, 989, 989, 989, 6161373, 989, 989, 989, 989, 989, 989, 989, 6325213, 4359057, 4359057, 5006225, 4359057, 4359057, 4359057, 5120913, 5137297, 989, 989, 5309405, 5317597, 989, 989, 989, 5432285, 989, 5489629, 989, 989, 989, 989, 989, 989, 5325789, 5374941, 5538781, 5546973, 5587933, 5735389, 5972957, 989, 6046685, 989, 6104029, 989, 989, 989, 6185949, 989, 989, 989, 6284253, 989, 989, 6333405, 989, 989, 989, 6390749, 989, 989, 6431709, 6439901, 4785041, 4359057, 4359057, 4359057, 4850577, 4359057, 4359057, 4359057, 4359057, 4359057, 4916113, 4359057, 4358144, 4358144, 4358144, 4358144, 4358144, 913, 913, 913, 913, 913, 0, 0, 0, 0, 0, 0, 0, 319488, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 319488, 0, 4957073, 4973457, 4359057, 4359057, 4359057, 4359057, 4359057, 4359057, 5071761, 4359057, 4359057, 4359057, 4359057, 4359057, 4359057, 4359057, 4359057, 6275985, 4359057, 4359057, 4359057, 4359057, 4359057, 4359057, 4359057, 5219217, 4359057, 5268369, 4359057, 4359057, 5309329, 5317521, 4359057, 4359057, 4359057, 5432209, 4359057, 5489553, 4359057, 4359057, 4359057, 5211025, 4359057, 4359057, 4359057, 4359057, 5292945, 4359057, 4359057, 4359057, 4359057, 5366673, 4359057, 4359057, 4359057, 4359057, 6390673, 4359057, 4359057, 6431633, 6439825, 4358144, 4358144, 4358144, 6266880, 6488064, 913, 913, 913, 6267793, 6488977, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 730, 4358144, 4358144, 4358144, 0, 0, 913, 913, 4842385, 913, 913, 913, 4899729, 913, 913, 913, 913, 913, 913, 913, 6103953, 913, 913, 913, 6185873, 913, 913, 913, 6284177, 913, 913, 5473169, 5522321, 913, 913, 913, 913, 5596049, 5710737, 5718929, 913, 5825425, 5866385, 913, 913, 913, 913, 5243793, 913, 913, 913, 913, 913, 913, 913, 5342097, 913, 913, 913, 0, 0, 989, 989, 989, 4826077, 989, 989, 989, 989, 989, 989, 989, 0, 913, 4359057, 4359057, 4359057, 4359057, 4359057, 4858769, 4875153, 5923729, 913, 913, 6022033, 913, 6038417, 913, 913, 6079377, 6112145, 913, 6177681, 6210449, 913, 6235025, 913, 913, 913, 913, 6243217, 913, 6292369, 913, 6316945, 913, 913, 913, 913, 913, 913, 913, 4998033, 913, 913, 5038993, 913, 913, 913, 5096337, 5104529, 5284829, 989, 989, 989, 989, 5473245, 5522397, 989, 989, 989, 989, 5596125, 5710813, 5719005, 989, 5825501, 5866461, 989, 989, 5923805, 989, 989, 6022109, 989, 6038493, 989, 989, 6079453, 6112221, 989, 6177757, 6210525, 989, 6235101, 989, 989, 989, 989, 989, 989, 989, 4359057, 4359057, 4842385, 4359057, 4359057, 4359057, 4899729, 4359057, 5284753, 4359057, 4359057, 4359057, 4359057, 5473169, 5522321, 4359057, 4359057, 4359057, 4359057, 5596049, 5710737, 5718929, 4359057, 4359057, 4359057, 6103953, 4359057, 4359057, 4359057, 6185873, 4359057, 4359057, 4359057, 6284177, 4359057, 4359057, 6333329, 4359057, 4359057, 4923392, 4358144, 4358144, 4358144, 4358144, 4358144, 913, 4924305, 913, 913, 913, 913, 4366336, 0, 0, 0, 0, 1779, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 3156, 0, 3157, 0, 0, 5825425, 5866385, 4359057, 4359057, 5923729, 4359057, 4359057, 6022033, 4359057, 6038417, 4359057, 4359057, 6079377, 6112145, 4359057, 6177681, 6210449, 4359057, 6235025, 4359057, 4359057, 4359057, 4359057, 4359057, 4359057, 4359057, 4358144, 4358144, 4358144, 913, 913, 913, 913, 913, 5194641, 913, 913, 913, 913, 913, 913, 913, 913, 913, 913, 988, 989, 989, 989, 989, 989, 4358144, 4358144, 6029312, 4358144, 4358144, 4358144, 4358144, 6160384, 4358144, 4358144, 4358144, 4358144, 4358144, 4358144, 6406144, 913, 913, 913, 913, 6325137, 989, 989, 5006301, 989, 989, 989, 5120989, 5137373, 989, 989, 989, 989, 989, 6226909, 4359057, 5063569, 4359057, 4359057, 4359057, 4359057, 4359057, 6226833, 0, 6086656, 913, 913, 5202833, 913, 913, 913, 913, 913, 913, 913, 913, 913, 913, 913, 5890961, 913, 913, 913, 5555089, 5571473, 5579665, 5620625, 5669777, 913, 913, 913, 5792657, 5817233, 913, 5858193, 913, 913, 913, 913, 5292945, 913, 913, 913, 913, 5366673, 913, 913, 913, 5456785, 913, 913, 913, 913, 913, 6308753, 913, 913, 6357905, 6382481, 6398865, 4801501, 4809693, 989, 989, 4891613, 913, 913, 6030225, 913, 913, 913, 913, 6161297, 913, 913, 913, 913, 913, 913, 913, 6407057, 5104605, 989, 989, 5202909, 989, 989, 989, 989, 989, 989, 989, 989, 989, 989, 989, 5891037, 6407133, 4359057, 4359057, 4359057, 4359057, 4359057, 4359057, 4359057, 4359057, 4998033, 4359057, 4359057, 5038993, 4359057, 4359057, 4359057, 5325713, 5374865, 5538705, 5546897, 5587857, 5735313, 5972881, 4359057, 6046609, 4359057, 6071185, 4359057, 4359057, 5096337, 5104529, 4359057, 4359057, 5202833, 4359057, 4359057, 4359057, 4359057, 4359057, 4359057, 4359057, 4359057, 4359057, 4359057, 4359057, 4359057, 4358144, 4358144, 4358144, 4358144, 5890961, 4359057, 4359057, 4359057, 6030225, 4359057, 4359057, 4359057, 4359057, 6161297, 4359057, 4359057, 4359057, 4359057, 4359057, 4359057, 5800849, 4359057, 4359057, 5882769, 4359057, 4359057, 4359057, 4359057, 4359057, 4359057, 5628817, 5653393, 4359057, 5702545, 4359057, 4359057, 5809041, 4359057, 4359057, 4359057, 4359057, 6407057, 4358144, 4358144, 4358144, 913, 913, 913, 4890624, 0, 0, 0, 0, 0, 0, 0, 0, 327680, 0, 0, 0, 0, 0, 0, 0, 0, 319488, 0, 0, 0, 0, 319488, 0, 319488, 6397952, 4801425, 4809617, 913, 913, 4891537, 913, 4948881, 913, 913, 913, 5047185, 913, 913, 913, 913, 913, 913, 913, 6300561, 913, 913, 913, 913, 913, 913, 913, 913, 913, 913, 913, 913, 6120337, 913, 6169489, 913, 5186449, 913, 5235601, 5301137, 913, 913, 5407633, 5530513, 913, 913, 913, 913, 5899153, 913, 913, 913, 913, 913, 913, 6054801, 913, 913, 913, 989, 989, 5014493, 989, 989, 989, 989, 989, 5194717, 989, 989, 989, 989, 989, 989, 989, 989, 4359057, 4359057, 989, 4948957, 989, 989, 989, 5047261, 989, 989, 989, 989, 5186525, 989, 5235677, 5301213, 989, 989, 989, 5211101, 989, 989, 989, 989, 5293021, 989, 989, 989, 989, 5366749, 989, 989, 989, 5243869, 989, 989, 989, 989, 989, 989, 989, 5342173, 989, 989, 989, 989, 989, 989, 6464477, 4359057, 4359057, 4359057, 4359057, 4359057, 4359057, 4359057, 4359057, 4359057, 4359057, 6325137, 5914624, 5915537, 0, 0, 0, 5407709, 5530589, 989, 989, 989, 989, 5899229, 989, 989, 989, 989, 989, 989, 989, 989, 6308829, 989, 989, 6357981, 6382557, 6398941, 4801425, 4809617, 4359057, 4359057, 4891537, 4359057, 4948881, 4359057, 4359057, 4359057, 5047185, 5899153, 4359057, 4359057, 4359057, 4359057, 4359057, 4359057, 4359057, 4359057, 6308753, 4359057, 4359057, 6357905, 6382481, 6398865, 5021696, 4358144, 4358144, 5022609, 913, 913, 0, 4980736, 0, 0, 0, 0, 0, 5373952, 5734400, 6045696, 0, 0, 0, 0, 1792, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 114688, 0, 5537792, 5545984, 5734400, 5971968, 4358144, 6045696, 4358144, 6070272, 4358144, 4358144, 4358144, 6348800, 913, 4866961, 4883345, 913, 913, 4916113, 913, 4957073, 4973457, 913, 913, 913, 913, 913, 913, 5071761, 913, 913, 913, 913, 913, 913, 5260177, 913, 913, 913, 913, 913, 913, 913, 913, 5415825, 4981649, 913, 913, 913, 913, 913, 913, 913, 913, 5325713, 5374865, 5538705, 5546897, 5587857, 5735313, 5972881, 913, 6046609, 913, 6071185, 913, 913, 913, 913, 6349713, 989, 4867037, 4883421, 989, 4981725, 989, 989, 989, 5800925, 989, 989, 5882845, 989, 989, 989, 989, 989, 989, 989, 989, 989, 5628893, 5653469, 989, 5702621, 989, 989, 989, 989, 6071261, 989, 989, 989, 989, 6349789, 4359057, 4866961, 4883345, 4359057, 4981649, 4359057, 4359057, 4359057, 4359057, 4359057, 4359057, 6464401, 4358144, 4358144, 4358144, 4358144, 4358144, 913, 913, 913, 913, 913, 913, 913, 913, 913, 913, 913, 913, 913, 913, 913, 913, 5178257, 913, 4359057, 4359057, 6349713, 4358144, 6144000, 913, 6144913, 0, 4988928, 5005312, 0, 0, 0, 0, 5775360, 0, 0, 0, 0, 1795, 0, 0, 1798, 0, 1800, 0, 0, 0, 0, 0, 1806, 4358144, 6324224, 913, 913, 5006225, 913, 913, 913, 5120913, 5137297, 913, 913, 913, 913, 913, 913, 913, 5112721, 913, 913, 913, 913, 913, 5284753, 913, 913, 4358144, 4358144, 4358144, 4358144, 4358144, 4358144, 4358144, 6053888, 4358144, 4358144, 913, 913, 5014417, 913, 913, 913, 913, 913, 5874577, 913, 913, 913, 913, 913, 913, 913, 913, 913, 913, 913, 913, 913, 989, 989, 989, 4358144, 6094848, 913, 4907921, 913, 5079953, 913, 5227409, 913, 5743505, 913, 913, 913, 6095761, 913, 989, 989, 4932573, 4940765, 989, 989, 989, 989, 989, 989, 5055453, 989, 989, 989, 989, 989, 989, 6054877, 989, 989, 989, 4359057, 4359057, 5014417, 4359057, 4359057, 4359057, 4907997, 989, 5080029, 989, 5227485, 989, 5743581, 989, 989, 989, 6095837, 989, 4359057, 4907921, 4359057, 5079953, 4359057, 5227409, 4359057, 5743505, 4359057, 4359057, 4359057, 6095761, 4359057, 5062656, 0, 0, 0, 0, 0, 4358144, 4358144, 0, 913, 913, 913, 913, 913, 913, 0, 0, 0, 0, 0, 0, 0, 0, 4956160, 4964352, 0, 0, 0, 0, 0, 0, 0, 0, 4816896, 4358144, 4358144, 4358144, 4358144, 6086656, 4817809, 913, 913, 913, 913, 6087569, 4817885, 5332992, 5980160, 4358144, 913, 5333905, 5981073, 913, 989, 5333981, 5981149, 989, 4359057, 5333905, 5981073, 4359057, 0, 0, 0, 0, 1811, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1820, 0, 0, 0, 0, 1825, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 528, 5439488, 5128192, 4358144, 5129105, 913, 5129181, 989, 5129105, 4359057, 0, 4358144, 913, 989, 4359057, 0, 4358144, 913, 989, 4359057, 6004736, 6004736, 6005649, 6005725, 6005649, 0, 0, 0, 450560, 450560, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 375, 0, 450560, 450560, 1, 24578, 3, 0, 0, 4366336, 0, 0, 0, 0, 0, 302, 303, 0, 0, 0, 0, 1876, 1877, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1867, 0, 0, 1870, 0, 0, 0, 0, 0, 196608, 0, 0, 0, 106496, 0, 0, 4284416, 0, 0, 0, 0, 0, 0, 0, 417792, 0, 0, 417792, 0, 0, 417792, 0, 417792, 0, 0, 0, 0, 139264, 147456, 417792, 0, 0, 0, 417792, 196608, 0, 0, 5816320, 6291456, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 400, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 303, 0, 0, 307, 0, 0, 0, 4956160, 4964352, 0, 0, 0, 0, 0, 0, 0, 466944, 0, 0, 0, 0, 0, 0, 0, 0, 0, 5480448, 0, 0, 0, 0, 0, 0, 6430720, 6438912, 914, 0, 0, 0, 914, 0, 4784128, 0, 0, 0, 4849664, 0, 0, 0, 0, 0, 648, 0, 0, 0, 0, 0, 0, 818, 0, 0, 0, 0, 0, 658, 659, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1272, 0, 0, 0, 0, 0, 0, 0, 0, 6332416, 0, 0, 0, 6389760, 0, 0, 6430720, 6438912, 990, 0, 0, 0, 990, 4358144, 4358144, 4358144, 914, 0, 0, 0, 4841472, 0, 0, 0, 4898816, 0, 0, 0, 0, 0, 0, 2383, 0, 0, 0, 0, 0, 0, 0, 0, 0, 2825, 0, 0, 0, 0, 0, 0, 0, 459222, 459222, 459222, 459222, 459222, 459222, 459222, 459222, 459222, 459222, 459222, 459222, 459222, 459222, 459222, 459222, 459222, 459379, 1, 24578, 3, 0, 0, 4366336, 0, 0, 0, 0, 0, 302, 303, 0, 0, 0, 0, 1887, 0, 1889, 1865, 528, 528, 528, 528, 528, 528, 528, 528, 3516, 528, 528, 3519, 528, 528, 528, 60866, 4358144, 4358144, 483328, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1243, 0, 0, 1, 24578, 3, 0, 0, 0, 0, 507904, 0, 0, 0, 507904, 0, 0, 0, 0, 0, 0, 2398, 0, 0, 0, 0, 0, 0, 0, 0, 0, 49723, 0, 0, 0, 0, 0, 327680, 0, 507904, 507904, 507904, 507904, 507904, 507904, 507904, 507904, 507904, 507904, 507904, 507904, 507904, 507904, 507904, 507904, 507904, 507904, 1, 24578, 3, 0, 0, 4366336, 0, 0, 0, 0, 0, 302, 303, 0, 0, 0, 0, 2084, 0, 0, 0, 4825088, 0, 0, 0, 0, 0, 0, 0, 0, 3423, 0, 0, 0, 0, 0, 0, 0, 0, 3460, 0, 0, 0, 0, 0, 3465, 0, 4268032, 306, 307, 0, 0, 442368, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1168, 1167, 0, 0, 0, 0, 0, 0, 0, 229376, 0, 491520, 524288, 0, 0, 0, 0, 0, 0, 0, 0, 0, 678, 0, 0, 0, 0, 0, 0, 4358144, 4358144, 491520, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1275, 0, 1277, 1, 24578, 3, 0, 0, 0, 0, 0, 516096, 0, 0, 0, 516096, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 368, 0, 296, 0, 0, 516567, 516567, 516567, 516567, 516567, 516567, 516567, 516567, 516567, 516567, 516567, 516567, 516567, 516567, 516567, 516567, 516567, 516567, 1, 24578, 0, 0, 0, 4366336, 0, 0, 548864, 0, 0, 302, 303, 0, 0, 0, 0, 2306, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 5627904, 0, 0, 0, 4268032, 306, 307, 409600, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 368, 0, 308, 0, 0, 0, 5513216, 5783552, 0, 40960, 0, 0, 0, 0, 0, 0, 0, 0, 4358144, 4358144, 4358144, 4358144, 4358144, 5193728, 4358144, 4358144, 4358144, 4358144, 4358144, 4358144, 4358144, 4358144, 4907008, 0, 5079040, 6094848, 1, 24578, 4227365, 0, 0, 0, 0, 0, 0, 299, 0, 0, 0, 299, 0, 0, 0, 0, 0, 245760, 0, 245760, 245760, 0, 0, 0, 245760, 245760, 0, 0, 0, 0, 0, 0, 245760, 0, 0, 0, 245760, 0, 0, 245760, 245760, 245760, 0, 0, 540672, 0, 0, 540672, 0, 0, 0, 540672, 0, 0, 0, 0, 0, 0, 0, 540672, 0, 0, 0, 0, 0, 0, 0, 0, 417792, 0, 0, 0, 417792, 0, 0, 0, 0, 0, 1, 24578, 4227365, 0, 0, 4366336, 0, 0, 0, 0, 0, 302, 303, 0, 0, 0, 0, 2328, 0, 0, 0, 0, 2332, 0, 0, 0, 0, 0, 0, 0, 1879, 0, 0, 0, 0, 0, 0, 0, 0, 768, 0, 0, 771, 0, 0, 0, 0, 0, 0, 499712, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1291, 0, 0, 0, 0, 499712, 0, 0, 0, 4358144, 4358144, 4358144, 4358144, 4358144, 4358144, 4358144, 4358144, 4358144, 4358144, 4358144, 5111808, 4358144, 4358144, 4358144, 4358144, 4358144, 5283840, 4358144, 4358144, 4358144, 4358144, 4358144, 499712, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1314, 0, 0, 0, 0, 1, 24578, 3, 0, 0, 4366336, 0, 0, 0, 0, 0, 302, 303, 0, 0, 0, 0, 2355, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1882, 0, 0, 0, 0, 1, 24578, 3, 155942, 296, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 528, 4087, 528, 4088, 528, 528, 57893, 528, 57893, 528, 528, 57893, 528, 528, 57916, 57893, 528, 528, 57893, 57893, 57893, 0, 0, 0, 0, 0, 0, 3821, 0, 0, 0, 0, 57916, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57916, 57916, 57893, 57893, 57943, 57893, 57893, 57893, 0, 0, 0, 0, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 61529, 57916, 57916, 57916, 57943, 57943, 58773, 914, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 60631, 0, 0, 2293, 0, 2294, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 664, 0, 0, 0, 0, 3453, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 744, 1825, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 2373, 0, 0, 0, 0, 0, 674, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1182, 0, 0, 0, 0, 0, 0, 0, 1280, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1788, 0, 0, 528, 57894, 528, 57894, 528, 528, 57894, 528, 528, 57917, 57894, 528, 528, 57894, 57894, 57894, 57894, 57944, 57944, 57894, 57894, 57894, 57894, 57944, 57944, 57894, 528, 57894, 57894, 57917, 57894, 57894, 57894, 57894, 57894, 57894, 57894, 57917, 57917, 57894, 57894, 57944, 57894, 57894, 57894, 1, 24578, 3, 155942, 155942, 296, 0, 0, 0, 0, 0, 302, 303, 0, 1, 24578, 3, 155943, 296, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 740, 661, 0, 0, 528, 0, 573440, 573440, 573440, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 368, 0, 516096, 0, 0, 0, 5988352, 0, 0, 6135808, 6307840, 0, 5996544, 4800512, 0, 6356992, 3675, 0, 0, 0, 0, 0, 694, 0, 0, 0, 787, 0, 0, 0, 0, 806, 0, 0, 0, 1, 24578, 3, 0, 0, 4366336, 0, 0, 0, 0, 0, 637, 303, 0, 0, 0, 0, 2381, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 344064, 0, 0, 0, 0, 4268032, 640, 307, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1804, 0, 0, 581632, 0, 0, 0, 581632, 581632, 581632, 581632, 581632, 581632, 581632, 581632, 581632, 581632, 581632, 581632, 581632, 581632, 581632, 581632, 0, 581632, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 581632, 0, 581632, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 746, 581632, 581632, 1, 24578, 3, 0, 0, 4366336, 0, 0, 0, 0, 0, 302, 303, 0, 0, 0, 0, 2397, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1225, 0, 0, 0, 0, 6258688, 6447104, 0, 0, 6127616, 0, 6348800, 5906432, 0, 5537792, 3827, 4882432, 0, 0, 0, 0, 0, 0, 2784, 0, 0, 0, 0, 0, 0, 0, 0, 0, 139264, 147456, 0, 0, 0, 420, 0, 0, 0, 3926, 0, 0, 0, 0, 0, 0, 5693440, 0, 6496256, 5144576, 5136384, 0, 5914624, 0, 0, 5513216, 5783552, 0, 3926, 0, 0, 0, 0, 0, 0, 0, 0, 4358144, 4358144, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 302, 0, 0, 306, 0, 0, 0, 0, 0, 0, 306, 237983, 147456, 0, 0, 0, 306, 0, 0, 311, 312, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1819, 0, 0, 312, 0, 311, 311, 312, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 311, 409, 312, 0, 458, 472, 472, 472, 485, 485, 485, 485, 485, 485, 485, 485, 485, 506, 485, 485, 485, 485, 485, 524, 485, 485, 485, 524, 485, 485, 485, 485, 485, 485, 529, 57895, 529, 57895, 529, 529, 57895, 529, 529, 57918, 57895, 529, 529, 57895, 57895, 57895, 57895, 57945, 57945, 57895, 57895, 57895, 57895, 57945, 57945, 57895, 529, 57895, 57895, 57918, 57895, 57895, 57895, 57895, 57895, 57895, 57895, 57918, 57918, 57895, 57895, 57945, 57895, 57895, 57895, 1, 24578, 3, 155942, 155942, 296, 0, 0, 0, 0, 0, 302, 303, 0, 0, 306, 307, 0, 0, 0, 0, 0, 645, 0, 0, 648, 649, 0, 0, 0, 0, 0, 736, 0, 0, 0, 0, 0, 0, 0, 0, 0, 745, 368, 368, 0, 0, 704, 0, 0, 0, 0, 0, 0, 0, 711, 0, 0, 0, 0, 0, 750, 0, 0, 0, 0, 645, 0, 0, 0, 0, 0, 0, 0, 6299648, 0, 0, 0, 0, 0, 0, 0, 0, 0, 777, 0, 0, 781, 0, 0, 0, 0, 0, 0, 0, 789, 0, 0, 0, 0, 0, 0, 2807, 0, 0, 0, 0, 0, 0, 0, 0, 0, 2310, 0, 0, 0, 0, 0, 0, 793, 0, 0, 0, 0, 0, 0, 0, 797, 0, 0, 0, 0, 0, 0, 0, 0, 557056, 557056, 0, 0, 0, 0, 0, 0, 0, 833, 793, 0, 0, 0, 0, 0, 837, 838, 0, 0, 0, 0, 793, 528, 528, 0, 57893, 57893, 57893, 57893, 57893, 57893, 155942, 1151, 0, 0, 1155, 0, 0, 0, 0, 2295, 0, 1160, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1759, 528, 847, 851, 528, 528, 863, 528, 528, 528, 879, 528, 884, 528, 892, 528, 895, 528, 528, 909, 528, 57893, 57893, 57893, 58266, 58270, 57893, 57893, 58282, 57893, 57893, 57893, 58298, 57893, 58303, 57893, 58311, 57893, 58314, 57893, 57893, 58328, 57893, 0, 57916, 57916, 57916, 58342, 58346, 57916, 57916, 58358, 57916, 57916, 57916, 58374, 57916, 58379, 57916, 58387, 57916, 58390, 57916, 57916, 58404, 57916, 0, 0, 0, 0, 58303, 57943, 57943, 57943, 58417, 58421, 57943, 57943, 58433, 57943, 57943, 528, 528, 528, 57893, 57893, 57893, 0, 0, 3655, 0, 3657, 0, 0, 0, 0, 0, 2356, 0, 2358, 0, 0, 0, 0, 0, 0, 0, 0, 0, 5693440, 0, 6496256, 5144576, 5136384, 0, 5914624, 57943, 58449, 57943, 58454, 57943, 58462, 57943, 58465, 57943, 57943, 58479, 57943, 847, 851, 1141, 895, 1159, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 747, 0, 0, 0, 1320, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 368, 302, 0, 0, 528, 528, 528, 1411, 528, 528, 528, 528, 528, 528, 528, 528, 528, 528, 528, 528, 528, 2470, 528, 528, 0, 0, 1809, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1842, 0, 0, 1832, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1201, 1885, 0, 0, 1886, 0, 0, 0, 0, 528, 528, 528, 528, 528, 528, 528, 528, 528, 528, 3225, 528, 528, 528, 1899, 528, 528, 528, 528, 528, 528, 528, 528, 528, 1912, 528, 528, 528, 0, 0, 57893, 60609, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 61085, 57893, 57893, 57893, 57893, 57893, 57893, 528, 528, 528, 528, 58773, 1985, 57893, 57893, 57893, 57893, 57893, 59336, 57893, 57893, 57893, 57893, 57893, 58822, 57893, 57893, 57893, 57893, 58827, 57893, 57893, 57893, 57893, 57893, 58319, 57893, 57893, 57893, 57893, 0, 57916, 57916, 58341, 57916, 57916, 57893, 59342, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 59355, 57893, 57893, 57893, 57893, 57893, 58859, 57893, 57893, 57893, 58866, 57893, 50676, 58773, 990, 57916, 57916, 59442, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 59455, 57916, 57916, 57916, 57916, 57916, 57916, 60387, 57916, 57916, 57916, 57916, 57916, 57916, 60394, 57916, 57916, 57916, 57916, 57916, 57916, 59514, 57916, 57916, 57916, 57916, 57916, 57916, 59520, 57916, 57916, 57916, 57916, 57916, 57943, 61127, 57943, 57943, 57943, 61130, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 60968, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 60721, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 59610, 57943, 57943, 57943, 57943, 57943, 57943, 59616, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 60982, 60984, 57943, 57943, 57943, 57943, 60988, 57943, 60990, 0, 0, 2293, 0, 2294, 0, 0, 0, 0, 0, 0, 0, 0, 0, 2300, 0, 0, 0, 0, 2806, 0, 0, 0, 0, 0, 2811, 0, 2813, 2814, 0, 0, 0, 0, 0, 483328, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 801, 0, 0, 0, 0, 0, 0, 2325, 0, 0, 0, 0, 0, 0, 0, 0, 0, 2334, 0, 0, 0, 0, 0, 0, 2822, 0, 0, 0, 0, 0, 0, 0, 0, 0, 769, 0, 0, 0, 774, 0, 0, 0, 2407, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 702, 528, 2399, 528, 528, 528, 528, 528, 528, 528, 528, 528, 528, 528, 2429, 528, 528, 2432, 528, 528, 2475, 528, 528, 528, 528, 528, 528, 528, 528, 528, 528, 528, 528, 528, 528, 2471, 528, 57893, 57893, 59863, 57893, 57893, 57893, 57893, 57893, 57893, 59871, 57893, 57893, 57893, 57893, 57893, 57893, 0, 0, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 60351, 57916, 57916, 57916, 57916, 57916, 57916, 57893, 59891, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 60640, 57916, 57916, 57916, 59949, 57916, 57916, 59952, 57916, 57916, 57916, 57916, 57916, 57916, 59960, 57916, 57916, 57893, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 0, 0, 0, 0, 57916, 57916, 60017, 57916, 57916, 57916, 57916, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 60417, 57943, 57943, 57943, 57943, 57943, 60032, 57943, 57943, 60035, 57943, 57943, 57943, 57943, 57943, 57943, 60043, 57943, 57943, 528, 528, 528, 57893, 57893, 57893, 0, 3654, 0, 0, 0, 0, 0, 0, 0, 1325, 0, 0, 0, 1329, 0, 0, 0, 0, 57943, 57943, 60100, 57943, 57943, 57943, 57943, 2760, 528, 528, 528, 528, 60109, 57893, 57893, 57893, 0, 0, 0, 0, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 59945, 57916, 0, 0, 2781, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1884, 0, 0, 0, 0, 2794, 0, 0, 0, 0, 0, 2798, 0, 0, 0, 0, 0, 0, 0, 0, 4784128, 0, 0, 0, 0, 0, 0, 0, 0, 3181, 0, 0, 0, 0, 0, 3186, 0, 57893, 60285, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 60293, 57893, 57893, 1, 24578, 3, 155942, 155942, 296, 0, 0, 0, 0, 0, 302, 303, 0, 57943, 57943, 60461, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 4075, 0, 4076, 57943, 60472, 57943, 57943, 57943, 57943, 57943, 528, 528, 528, 528, 528, 57893, 57893, 57893, 57893, 57893, 0, 0, 0, 0, 0, 0, 0, 3150, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 2299, 0, 0, 0, 3160, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 3173, 0, 0, 0, 0, 2820, 0, 0, 0, 2824, 0, 0, 0, 0, 0, 0, 0, 703, 0, 0, 0, 0, 0, 0, 0, 0, 2331, 0, 0, 0, 0, 0, 0, 0, 0, 3189, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 3202, 3203, 0, 0, 3215, 0, 528, 528, 528, 528, 528, 528, 528, 528, 528, 528, 528, 3226, 528, 528, 3230, 528, 528, 528, 528, 528, 528, 528, 528, 528, 528, 3239, 528, 528, 0, 0, 0, 0, 0, 0, 57893, 57893, 60279, 57893, 57893, 57893, 57893, 57893, 58320, 57893, 57893, 57893, 57893, 0, 57916, 57916, 57916, 57916, 57916, 57943, 57943, 57943, 57943, 57943, 57943, 0, 0, 4289, 0, 528, 60617, 57893, 57893, 60621, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 60630, 57893, 0, 0, 3137, 3138, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 3199, 0, 0, 0, 0, 57916, 57916, 57916, 60666, 57916, 57916, 60670, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 58383, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 60679, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 60687, 57916, 0, 0, 0, 0, 57893, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 528, 528, 528, 57893, 57893, 57893, 57943, 57943, 57943, 57943, 60715, 57943, 57943, 60719, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 59582, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 60728, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 60736, 0, 3430, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 729, 0, 528, 3495, 528, 528, 528, 528, 528, 528, 528, 528, 528, 528, 528, 528, 528, 528, 528, 2904, 528, 528, 528, 528, 3512, 528, 3514, 528, 528, 528, 528, 528, 528, 528, 528, 57893, 57893, 58259, 57893, 57893, 57893, 57893, 57893, 57893, 58292, 57893, 57893, 60879, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 59389, 57893, 57893, 57893, 57893, 60897, 57893, 60899, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 59399, 57893, 57893, 57893, 57893, 57893, 57893, 57916, 60921, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57943, 57943, 57943, 57943, 57943, 57943, 60963, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 60457, 57943, 528, 4015, 528, 528, 528, 528, 528, 528, 528, 528, 528, 528, 528, 57893, 61372, 57893, 302, 306, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1199, 0, 0, 61374, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57916, 61388, 57916, 0, 0, 0, 0, 57893, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 58442, 61390, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57943, 61404, 57943, 528, 2281, 528, 528, 528, 57893, 59630, 57893, 57893, 57893, 2290, 0, 2291, 0, 0, 0, 0, 0, 262144, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 663, 0, 0, 666, 667, 0, 61406, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 0, 0, 0, 0, 0, 0, 528, 4077, 0, 0, 0, 0, 0, 0, 0, 4084, 0, 0, 528, 528, 528, 528, 4089, 0, 0, 4148, 0, 4150, 528, 528, 528, 528, 528, 528, 528, 528, 528, 528, 528, 528, 1399, 528, 528, 528, 0, 0, 0, 313, 314, 315, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1240, 0, 0, 0, 0, 0, 0, 0, 427, 0, 131072, 0, 0, 0, 0, 427, 0, 0, 0, 0, 0, 427, 459, 0, 0, 0, 459, 459, 459, 459, 459, 459, 459, 459, 459, 459, 459, 459, 523, 459, 523, 523, 523, 459, 523, 523, 523, 523, 523, 523, 530, 57896, 530, 57896, 530, 530, 57896, 530, 530, 57919, 57896, 530, 530, 57896, 57896, 57896, 57896, 57946, 57946, 57896, 57896, 57896, 57896, 57946, 57946, 57896, 621, 57896, 57973, 57919, 57896, 57896, 57896, 57896, 57896, 57896, 57896, 57919, 57919, 57896, 57896, 57946, 57896, 57896, 57896, 57973, 57973, 1, 24578, 3, 155942, 155942, 296, 0, 0, 0, 0, 0, 302, 303, 0, 0, 0, 0, 2830, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 680, 681, 0, 0, 0, 57893, 57893, 58307, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 0, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 60350, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 60915, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 58942, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 60389, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57943, 57943, 57943, 57943, 58458, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 528, 528, 528, 528, 528, 57893, 57893, 57893, 57893, 528, 528, 1965, 528, 528, 528, 528, 528, 528, 1972, 528, 528, 528, 528, 528, 528, 528, 3501, 528, 3503, 528, 528, 528, 528, 528, 528, 1360, 528, 528, 528, 528, 528, 528, 528, 528, 528, 2887, 528, 2889, 528, 528, 528, 528, 57893, 59408, 57893, 57893, 57893, 57893, 57893, 57893, 59416, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 58823, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 58840, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 59508, 57916, 57916, 57916, 57916, 57916, 57916, 59516, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 58909, 57916, 57916, 57916, 58912, 57916, 57916, 57916, 57893, 57893, 57893, 60286, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 58814, 57893, 57893, 57893, 60408, 57916, 57916, 57916, 57943, 57943, 57943, 57943, 57943, 57943, 60414, 57943, 57943, 57943, 57943, 60418, 57943, 57943, 57943, 60473, 57943, 57943, 57943, 528, 528, 528, 528, 528, 57893, 57893, 57893, 57893, 57893, 0, 0, 0, 0, 2292, 528, 528, 528, 4093, 528, 4095, 528, 528, 4097, 528, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 61077, 57893, 57893, 57893, 57893, 61450, 57893, 61452, 57893, 57893, 61454, 57893, 57893, 57916, 57916, 57916, 57916, 57916, 57916, 58879, 57916, 57916, 57916, 57916, 57916, 58891, 57916, 57916, 57916, 57916, 57916, 61464, 57916, 61466, 57916, 57916, 61468, 57916, 57916, 57943, 57943, 57943, 57943, 57943, 57943, 60956, 57943, 57943, 57943, 57943, 57943, 60960, 57943, 57943, 57943, 57943, 61478, 57943, 61480, 57943, 57943, 61482, 57943, 57943, 0, 0, 0, 0, 0, 0, 0, 1766, 0, 0, 1769, 0, 0, 0, 0, 0, 320, 320, 428, 429, 131072, 436, 429, 437, 439, 428, 436, 437, 0, 316, 437, 455, 460, 473, 473, 473, 486, 486, 486, 486, 486, 486, 486, 486, 486, 486, 508, 508, 521, 521, 522, 522, 508, 522, 522, 522, 508, 522, 522, 522, 522, 522, 522, 531, 57897, 531, 57897, 531, 531, 57897, 531, 531, 57920, 57897, 531, 531, 57897, 57897, 57897, 57897, 57947, 57947, 57897, 57897, 57897, 57897, 57947, 57947, 57897, 622, 57972, 57972, 57920, 57897, 57897, 57897, 57897, 57897, 57897, 57897, 57920, 57920, 57897, 57897, 57947, 57897, 57897, 57897, 57972, 57972, 1, 24578, 3, 155942, 155942, 296, 0, 0, 0, 0, 0, 302, 303, 0, 0, 0, 0, 3151, 0, 0, 0, 0, 0, 3155, 0, 0, 0, 0, 0, 0, 814, 0, 0, 0, 0, 0, 0, 819, 0, 0, 0, 686, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 368, 0, 0, 0, 368, 368, 702, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 2315, 0, 0, 0, 0, 0, 734, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 368, 303, 0, 0, 0, 823, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 775, 0, 0, 1174, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 804, 0, 0, 0, 0, 1216, 1174, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 2336, 0, 0, 0, 0, 0, 1281, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 368, 342, 296, 0, 0, 0, 1295, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 2363, 0, 0, 1307, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1229, 58773, 914, 57893, 57893, 58777, 57893, 57893, 58781, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 58808, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 58795, 57893, 58800, 57893, 57893, 58804, 57893, 57893, 58807, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 58838, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 58841, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 58872, 57916, 57916, 58876, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 58890, 57916, 58895, 57916, 0, 0, 0, 0, 57893, 57943, 57943, 57943, 57943, 57943, 57943, 58428, 57943, 57943, 58444, 57916, 58899, 57916, 57916, 58902, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 59985, 57916, 57916, 57916, 1860, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1245, 528, 528, 528, 528, 1902, 528, 528, 528, 528, 528, 528, 528, 528, 528, 1914, 528, 528, 0, 57893, 57893, 57893, 57893, 57893, 57893, 155942, 1151, 0, 0, 1156, 0, 0, 0, 0, 327, 327, 377, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1210, 0, 368, 368, 0, 1213, 0, 1929, 528, 528, 528, 528, 528, 528, 528, 528, 528, 528, 528, 528, 528, 528, 528, 896, 57893, 57893, 57893, 59345, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 59357, 57893, 57893, 1, 24578, 3, 155942, 156282, 296, 0, 0, 0, 0, 0, 302, 303, 0, 57916, 57916, 59445, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 59457, 57916, 57916, 57916, 57916, 57916, 59967, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57943, 57943, 57943, 57943, 61474, 57943, 59541, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 59553, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 61152, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 528, 528, 528, 2763, 2764, 57893, 57893, 57893, 60112, 57943, 57943, 57943, 59591, 59592, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 59601, 57943, 57943, 57943, 57893, 59917, 59918, 59919, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 58848, 57893, 57893, 58852, 57916, 59989, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 60002, 57943, 60072, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 60085, 57893, 57893, 60325, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 58849, 57893, 57893, 0, 0, 0, 3216, 528, 528, 528, 528, 528, 528, 528, 528, 528, 528, 528, 528, 528, 2484, 528, 528, 528, 528, 528, 3243, 528, 528, 528, 528, 528, 528, 528, 528, 528, 528, 528, 528, 528, 2891, 528, 528, 57893, 57893, 57893, 57893, 60634, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 0, 2999, 0, 0, 0, 61125, 57916, 57916, 57916, 57916, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 60962, 528, 528, 4092, 528, 528, 528, 528, 528, 528, 528, 57893, 61444, 57893, 61445, 57893, 57893, 57893, 2589, 0, 0, 0, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 61106, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 59954, 57916, 57916, 59958, 57916, 57916, 57916, 59962, 57893, 61449, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57916, 61458, 57916, 61459, 57916, 57916, 57893, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 59540, 57916, 61463, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57943, 61472, 57943, 61473, 57943, 57943, 528, 528, 1715, 528, 528, 1718, 57893, 57893, 59064, 57893, 57893, 59067, 1151, 0, 57943, 61477, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 0, 0, 0, 0, 0, 0, 0, 1814, 0, 0, 0, 0, 0, 0, 0, 0, 528, 1338, 528, 528, 528, 528, 528, 528, 528, 528, 4291, 57893, 57893, 57893, 61637, 57916, 57916, 57916, 61639, 57943, 57943, 57943, 61641, 0, 0, 0, 0, 3178, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1312, 0, 0, 0, 0, 326, 327, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1186, 0, 0, 0, 377, 0, 131072, 0, 0, 0, 440, 377, 0, 0, 445, 451, 0, 377, 461, 474, 474, 474, 487, 487, 487, 487, 487, 487, 487, 487, 487, 487, 509, 509, 509, 509, 509, 509, 509, 509, 509, 509, 509, 509, 509, 509, 509, 509, 532, 57898, 532, 57898, 532, 532, 57898, 532, 532, 57921, 57898, 532, 532, 57898, 57898, 57898, 57898, 57948, 57948, 57898, 57898, 57898, 57898, 57948, 57948, 57898, 532, 57898, 57898, 57921, 57898, 57898, 57898, 57898, 57898, 57898, 57898, 57921, 57921, 57898, 57898, 57948, 57898, 57898, 57898, 1, 24578, 3, 155942, 155942, 296, 0, 0, 0, 0, 0, 302, 303, 0, 1188, 0, 1190, 1191, 0, 0, 0, 1193, 1194, 0, 0, 0, 0, 0, 0, 0, 0, 4997120, 0, 0, 5038080, 0, 0, 0, 5095424, 5103616, 0, 0, 1232, 1233, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 370, 0, 0, 0, 1263, 1265, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1200, 0, 1317, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1303, 0, 0, 0, 1331, 1193, 528, 528, 1390, 528, 528, 1392, 528, 528, 528, 528, 1397, 528, 528, 528, 528, 528, 528, 3500, 528, 528, 528, 528, 528, 528, 528, 528, 528, 528, 61443, 57893, 57893, 57893, 57893, 57893, 58773, 914, 57893, 57893, 58778, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 2588, 2999, 0, 0, 0, 58831, 57893, 57893, 57893, 57893, 58836, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 58811, 58813, 57893, 57893, 57893, 57893, 58873, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 60380, 57916, 58900, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 58928, 57916, 57916, 58915, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 58924, 57916, 57916, 58926, 57916, 57916, 57916, 57916, 57916, 59980, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 61530, 57916, 57916, 57943, 57943, 57916, 58931, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 58950, 57916, 57916, 57916, 58956, 57916, 57916, 57916, 57916, 0, 58831, 57943, 57943, 58967, 57943, 57943, 57943, 57943, 57943, 57943, 60464, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 60470, 57943, 57943, 57943, 57943, 59018, 57943, 57943, 59020, 57943, 57943, 57943, 57943, 59026, 57943, 57943, 57943, 528, 528, 57893, 57893, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 2778, 1963, 528, 528, 528, 528, 528, 528, 528, 528, 528, 528, 528, 528, 528, 528, 528, 901, 0, 0, 2293, 0, 2294, 0, 0, 0, 0, 0, 2297, 0, 0, 0, 0, 0, 0, 316, 317, 318, 319, 320, 321, 322, 323, 324, 325, 57893, 59862, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 60650, 57893, 59878, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 58816, 57893, 59947, 57916, 57916, 57916, 57916, 59951, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57943, 57943, 57943, 57943, 57943, 61475, 60015, 57916, 57916, 57916, 57916, 57916, 57916, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 61133, 57943, 60030, 57943, 57943, 57943, 57943, 60034, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 60723, 57943, 57943, 57943, 60098, 57943, 57943, 57943, 57943, 57943, 57943, 528, 528, 528, 528, 528, 57893, 57893, 57893, 57893, 57893, 0, 0, 0, 1729, 0, 0, 0, 0, 2844, 0, 0, 0, 0, 0, 2849, 0, 0, 0, 0, 0, 0, 0, 1852, 0, 0, 0, 0, 0, 0, 0, 1859, 0, 57916, 57916, 57916, 60347, 57916, 60348, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 58385, 57916, 57916, 57916, 57916, 58403, 57916, 0, 0, 3205, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 2374, 0, 0, 528, 528, 528, 528, 3253, 528, 528, 528, 528, 528, 528, 528, 528, 528, 528, 528, 528, 1926, 528, 528, 528, 57916, 60688, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 60694, 57916, 57916, 57916, 57916, 57916, 57916, 58906, 58908, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 59451, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 59498, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 60672, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57943, 57943, 60737, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 60743, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 61339, 57943, 57943, 57943, 528, 57893, 0, 0, 0, 0, 0, 0, 3671, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 665, 0, 0, 668, 57916, 61126, 57916, 57916, 57916, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 60961, 57943, 57916, 57916, 57916, 61239, 57916, 57916, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 60959, 57943, 57943, 57943, 57943, 61258, 57943, 57943, 528, 528, 57893, 57893, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 2776, 2777, 0, 528, 528, 528, 528, 4018, 528, 528, 528, 528, 528, 528, 528, 528, 57893, 57893, 57893, 57893, 57893, 57893, 58280, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 61377, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57916, 57916, 57916, 61223, 57916, 57916, 57916, 57916, 57916, 57916, 61393, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57943, 57943, 57943, 57943, 60025, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 61409, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 0, 0, 0, 4241, 0, 4243, 528, 528, 528, 528, 528, 4094, 528, 528, 528, 528, 528, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 59372, 57893, 57893, 57893, 61451, 57893, 57893, 57893, 57893, 57893, 57893, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 61318, 57916, 57916, 57916, 61465, 57916, 57916, 57916, 57916, 57916, 57916, 57943, 57943, 57943, 57943, 57943, 57943, 61244, 57943, 57943, 57943, 57943, 57943, 57943, 61479, 57943, 57943, 57943, 57943, 57943, 57943, 0, 0, 0, 0, 0, 0, 0, 1865, 1866, 0, 0, 0, 1869, 0, 0, 0, 0, 0, 328, 329, 330, 331, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1855, 1856, 0, 1857, 1858, 0, 0, 376, 0, 0, 0, 0, 0, 328, 376, 331, 375, 0, 0, 0, 0, 0, 0, 363, 0, 0, 0, 0, 0, 0, 0, 0, 0, 139264, 147456, 0, 0, 450560, 0, 0, 0, 331, 0, 0, 0, 0, 0, 0, 376, 0, 0, 0, 0, 0, 0, 0, 0, 5210112, 0, 5365760, 0, 5554176, 5570560, 5578752, 0, 462, 475, 475, 475, 488, 488, 488, 488, 499, 501, 488, 488, 499, 488, 510, 510, 510, 510, 510, 525, 510, 510, 510, 525, 510, 510, 510, 510, 510, 510, 533, 57899, 533, 57899, 533, 533, 57899, 533, 533, 57922, 57899, 533, 533, 57899, 57899, 57899, 57899, 57949, 57949, 57899, 57899, 57899, 57899, 57949, 57949, 57899, 533, 57899, 57899, 57922, 57899, 57899, 57899, 57899, 57899, 57899, 57899, 57922, 57922, 57899, 57899, 57949, 57899, 57899, 57899, 1, 24578, 3, 155942, 155942, 296, 0, 0, 0, 0, 0, 302, 303, 0, 0, 670, 671, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 683, 684, 685, 0, 0, 0, 689, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 368, 368, 0, 0, 0, 0, 0, 0, 707, 708, 0, 0, 0, 0, 0, 714, 0, 0, 0, 718, 0, 720, 0, 0, 0, 0, 0, 0, 727, 0, 0, 0, 0, 0, 1163, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 6152192, 0, 0, 0, 6316032, 0, 731, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1292, 778, 779, 0, 0, 0, 0, 0, 0, 0, 0, 788, 0, 790, 0, 0, 0, 0, 0, 1201, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 3675, 0, 0, 0, 3830, 0, 822, 0, 0, 0, 0, 0, 822, 822, 825, 0, 0, 0, 790, 0, 0, 0, 0, 0, 1235, 0, 1237, 0, 0, 0, 0, 1242, 0, 0, 0, 0, 0, 1268, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 3675, 0, 0, 3829, 0, 0, 0, 0, 834, 0, 0, 0, 0, 0, 0, 0, 788, 0, 0, 0, 834, 528, 528, 0, 57893, 57893, 57893, 57893, 57893, 57893, 155942, 1151, 0, 302, 0, 0, 306, 1, 24578, 3, 0, 0, 4366336, 0, 0, 0, 0, 0, 65536, 303, 0, 528, 848, 528, 854, 528, 528, 869, 528, 528, 880, 528, 885, 528, 528, 894, 897, 902, 528, 910, 528, 57893, 57893, 57893, 58267, 57893, 58273, 57893, 57893, 58288, 57893, 57893, 58299, 57893, 58304, 57893, 57893, 58313, 58316, 58321, 57893, 58329, 57893, 0, 57916, 57916, 57916, 58343, 57916, 0, 0, 0, 0, 57893, 57943, 57943, 57943, 57943, 57943, 57943, 58430, 57943, 57943, 57943, 57943, 57943, 59547, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 61417, 57943, 0, 0, 0, 58349, 57916, 57916, 58364, 57916, 57916, 58375, 57916, 58380, 57916, 57916, 58389, 58392, 58397, 57916, 58405, 57916, 0, 0, 0, 0, 58304, 57943, 57943, 57943, 58418, 57943, 58424, 57943, 57943, 58439, 57943, 1906, 528, 528, 528, 528, 59349, 57893, 57893, 57893, 57893, 0, 0, 0, 0, 0, 3140, 0, 0, 3143, 3144, 0, 0, 3147, 3148, 0, 57943, 58450, 57943, 58455, 57943, 57943, 58464, 58467, 58472, 57943, 58480, 57943, 848, 528, 528, 1142, 902, 528, 0, 57893, 58267, 57893, 58492, 58321, 57893, 155942, 1151, 0, 0, 0, 0, 0, 0, 364, 365, 366, 367, 0, 0, 368, 0, 296, 0, 0, 0, 0, 1176, 0, 0, 0, 0, 0, 0, 0, 1183, 0, 0, 0, 0, 0, 0, 3165, 0, 0, 0, 0, 0, 0, 0, 0, 0, 139264, 147456, 0, 0, 0, 421, 0, 0, 1203, 1204, 0, 0, 0, 0, 1208, 0, 0, 0, 368, 368, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 713, 0, 0, 1231, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1244, 0, 0, 0, 1248, 0, 0, 1200, 0, 0, 0, 0, 0, 749, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 726, 0, 0, 0, 0, 528, 1347, 528, 528, 528, 528, 528, 528, 1362, 528, 528, 528, 1367, 528, 528, 528, 528, 528, 2464, 528, 528, 528, 528, 528, 528, 528, 528, 528, 528, 528, 1910, 528, 528, 528, 528, 58773, 914, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 58786, 57893, 57893, 57893, 0, 0, 0, 0, 57916, 57916, 57916, 57916, 57916, 57916, 59944, 57916, 57916, 57893, 57893, 57893, 58801, 57893, 57893, 57893, 58806, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 58809, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 58854, 57893, 57893, 57893, 57893, 57893, 57893, 58865, 57893, 50676, 58773, 990, 57916, 57916, 57893, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 59537, 57943, 57943, 528, 1351, 528, 1408, 528, 528, 58790, 57893, 57893, 58847, 57893, 57893, 1151, 0, 57916, 57916, 57916, 58901, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 58391, 57916, 57916, 57916, 57916, 57916, 57916, 58918, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 58396, 57916, 57916, 57916, 57916, 57916, 57916, 58933, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 58949, 57916, 0, 0, 0, 0, 57893, 57943, 57943, 57943, 57943, 57943, 57943, 58431, 57943, 57943, 57943, 57943, 57943, 60063, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 60428, 57943, 60430, 57943, 57943, 57943, 57943, 58975, 57943, 57943, 57943, 57943, 57943, 57943, 58990, 57943, 57943, 57943, 58995, 57943, 2280, 528, 2282, 528, 528, 59629, 57893, 59631, 57893, 57893, 0, 0, 0, 0, 0, 0, 1298, 0, 0, 0, 0, 1303, 0, 0, 0, 0, 59055, 57943, 528, 528, 528, 528, 528, 528, 57893, 57893, 57893, 57893, 57893, 57893, 1151, 1724, 0, 1760, 0, 0, 1763, 1764, 1765, 0, 1767, 1768, 0, 0, 0, 0, 1773, 0, 0, 0, 0, 3192, 0, 0, 0, 0, 3197, 0, 0, 0, 0, 0, 0, 0, 2368, 2369, 0, 0, 0, 0, 0, 0, 0, 692, 0, 0, 0, 0, 0, 0, 699, 368, 1807, 0, 0, 0, 0, 0, 0, 0, 0, 1816, 1817, 0, 0, 0, 0, 0, 0, 373, 0, 0, 0, 0, 0, 0, 0, 0, 0, 139264, 147456, 0, 0, 352256, 0, 0, 0, 1823, 0, 0, 1825, 0, 0, 0, 0, 0, 0, 0, 0, 1829, 1830, 1831, 0, 1870, 0, 0, 0, 1888, 1740, 1740, 1890, 528, 1892, 528, 1893, 1894, 528, 1896, 528, 528, 528, 1900, 528, 528, 528, 528, 528, 528, 528, 528, 528, 1913, 528, 528, 0, 0, 0, 0, 0, 0, 57893, 60278, 57893, 57893, 57893, 57893, 57893, 57893, 0, 2592, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 59969, 57916, 57916, 57916, 57916, 57916, 57916, 59974, 57916, 528, 528, 1917, 528, 528, 528, 528, 1921, 528, 1923, 528, 528, 528, 528, 528, 528, 1393, 528, 1395, 528, 528, 528, 528, 528, 528, 528, 872, 528, 528, 528, 528, 528, 528, 528, 528, 528, 3710, 528, 528, 528, 528, 528, 528, 528, 528, 528, 528, 1933, 1934, 1936, 528, 528, 528, 528, 528, 528, 528, 1944, 1945, 528, 1947, 528, 528, 1950, 1951, 528, 528, 528, 528, 528, 528, 528, 1959, 528, 1961, 528, 1964, 528, 528, 528, 528, 528, 1971, 528, 528, 1973, 528, 528, 528, 528, 528, 528, 3855, 528, 528, 528, 528, 528, 57893, 57893, 57893, 57893, 57893, 57893, 58279, 57893, 57893, 57893, 57893, 57893, 528, 528, 528, 528, 58773, 0, 59332, 57893, 59334, 57893, 59335, 57893, 59337, 57893, 59339, 57893, 302, 306, 0, 0, 0, 0, 3141, 0, 0, 0, 3145, 0, 0, 0, 0, 0, 0, 2832, 0, 0, 0, 0, 0, 0, 0, 0, 0, 2835, 0, 0, 0, 0, 0, 0, 57893, 57893, 59343, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 59356, 57893, 57893, 57893, 0, 0, 0, 0, 57916, 57916, 57916, 57916, 59942, 57916, 57916, 57916, 57916, 57916, 59515, 57916, 57916, 59517, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 0, 57893, 57943, 57943, 58966, 57943, 57943, 58970, 57943, 57893, 59360, 57893, 57893, 57893, 57893, 59364, 57893, 59366, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 58839, 57893, 57893, 58846, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 59376, 59377, 59379, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 59387, 59388, 57893, 2770, 2291, 0, 2771, 2294, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 770, 0, 0, 0, 0, 0, 59390, 57893, 57893, 59393, 59394, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 59402, 57893, 59404, 57893, 2770, 2291, 0, 2771, 2294, 0, 0, 0, 0, 0, 0, 2775, 0, 0, 0, 0, 0, 784, 0, 0, 679, 0, 0, 0, 0, 0, 0, 0, 660, 661, 0, 0, 0, 0, 0, 0, 0, 384, 0, 139264, 147456, 0, 406, 0, 0, 406, 59407, 57893, 57893, 57893, 57893, 57893, 59415, 57893, 57893, 59417, 57893, 57893, 57893, 57893, 57893, 57893, 0, 0, 57916, 57916, 57916, 57916, 60660, 57916, 57916, 57916, 57916, 59443, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 59456, 57916, 57916, 57916, 57916, 57916, 58878, 57916, 57916, 57916, 58882, 57916, 57916, 58893, 57916, 57916, 58897, 59460, 57916, 57916, 57916, 57916, 59464, 57916, 59466, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 59467, 59468, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 59476, 59477, 59479, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 59487, 59488, 57916, 59490, 57916, 57916, 59493, 59494, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 59502, 57916, 59504, 57916, 59507, 57943, 57943, 57943, 59560, 57943, 59562, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 60987, 57943, 57943, 57943, 59572, 59573, 59575, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 59583, 59584, 57943, 57943, 59586, 57943, 57943, 57943, 57943, 57943, 60731, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 60081, 57943, 57943, 57943, 57943, 57943, 59589, 59590, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 59598, 57943, 59600, 57943, 59603, 57943, 57943, 57943, 57943, 57943, 60739, 57943, 57943, 57943, 60742, 57943, 60744, 57943, 57943, 57943, 57943, 57943, 57943, 60740, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 59023, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 59611, 57943, 57943, 59613, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 60039, 57943, 57943, 57943, 57943, 57943, 57943, 0, 0, 2293, 0, 2294, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 2301, 0, 0, 2304, 2305, 0, 0, 0, 2309, 0, 0, 0, 0, 0, 0, 0, 0, 0, 6406144, 5357568, 0, 5505024, 0, 0, 0, 0, 2352, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1276, 0, 1825, 0, 2365, 0, 0, 0, 0, 0, 0, 0, 0, 2372, 0, 0, 2375, 0, 0, 0, 0, 3217, 528, 528, 528, 528, 528, 528, 528, 528, 528, 528, 528, 528, 2929, 528, 528, 528, 0, 2378, 0, 0, 0, 0, 0, 0, 2385, 0, 2387, 0, 0, 0, 0, 0, 0, 403, 0, 0, 0, 0, 0, 0, 0, 0, 332, 333, 0, 0, 0, 0, 0, 0, 0, 0, 334, 335, 336, 0, 0, 0, 0, 0, 0, 0, 0, 2396, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 697, 698, 0, 368, 2406, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 2417, 0, 0, 0, 0, 3433, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 786, 0, 0, 0, 528, 2433, 528, 2435, 528, 528, 528, 528, 528, 528, 2443, 528, 2445, 528, 528, 528, 528, 528, 1414, 528, 528, 528, 528, 528, 528, 528, 528, 528, 528, 1956, 528, 528, 528, 528, 528, 528, 528, 528, 2451, 528, 528, 528, 528, 528, 528, 528, 528, 528, 528, 528, 528, 528, 2902, 528, 528, 2473, 528, 528, 528, 528, 528, 528, 2480, 528, 528, 528, 528, 528, 528, 528, 528, 1906, 528, 528, 528, 528, 528, 528, 528, 57893, 57893, 57893, 59864, 57893, 59866, 57893, 57893, 57893, 57893, 57893, 57893, 59874, 57893, 59876, 57893, 2770, 2291, 0, 2771, 2294, 0, 0, 2773, 0, 0, 0, 0, 0, 0, 0, 0, 2359296, 418, 418, 0, 0, 0, 0, 0, 57893, 57893, 59904, 59905, 57893, 57893, 57893, 57893, 57893, 57893, 59912, 57893, 57893, 57893, 57893, 57893, 57893, 60885, 57893, 60887, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 59397, 57893, 57893, 59400, 57893, 57893, 57893, 57893, 57893, 59930, 59931, 57893, 0, 0, 0, 0, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 58910, 57916, 57916, 57916, 57916, 57916, 57916, 59963, 57916, 59965, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 59971, 57916, 57916, 57916, 57916, 57916, 57916, 60914, 57916, 57916, 57916, 57916, 57916, 60918, 57916, 57916, 57916, 57916, 60016, 57916, 57916, 60019, 60020, 57916, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 0, 4239, 0, 0, 0, 0, 528, 60046, 57943, 60048, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 60054, 57943, 57943, 57943, 57943, 57943, 57943, 58979, 58986, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 60079, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 60099, 57943, 57943, 60102, 60103, 57943, 2435, 528, 2762, 528, 528, 59866, 57893, 60111, 57893, 57893, 57893, 57893, 57893, 61083, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 61089, 57893, 57893, 57893, 57893, 57893, 61219, 57893, 57893, 57893, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 1272, 57893, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 528, 3132, 528, 528, 528, 57893, 60479, 57893, 57893, 0, 0, 2829, 0, 0, 0, 0, 2833, 0, 0, 0, 0, 0, 0, 0, 0, 312, 311, 0, 0, 0, 311, 311, 312, 2905, 528, 528, 528, 528, 2909, 528, 528, 528, 2914, 528, 528, 528, 528, 528, 528, 1920, 528, 528, 528, 528, 528, 528, 528, 528, 528, 1908, 528, 528, 528, 528, 528, 528, 528, 528, 2922, 528, 528, 528, 528, 528, 528, 528, 2927, 528, 528, 528, 528, 528, 528, 3940, 528, 528, 528, 528, 528, 528, 3946, 528, 528, 60284, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 59406, 60296, 57893, 60298, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 59403, 57893, 57893, 57893, 60324, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 60332, 57893, 57893, 57893, 57893, 57893, 57893, 61096, 57893, 57893, 57893, 57893, 57916, 57916, 57916, 57916, 57916, 57916, 60703, 57916, 57916, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 58466, 57943, 57943, 57943, 57943, 528, 528, 528, 896, 57893, 57893, 57893, 60338, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 0, 2999, 0, 0, 0, 0, 0, 1323, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 2836, 2837, 0, 0, 0, 0, 57916, 60382, 57916, 57916, 57916, 57916, 60386, 57916, 57916, 57916, 60391, 57916, 57916, 57916, 57916, 57916, 57916, 60926, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 59956, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 60399, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 60405, 57916, 57916, 57916, 57916, 57916, 59992, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 58925, 57916, 57916, 57916, 57916, 57916, 57943, 57943, 57943, 57943, 60447, 57943, 57943, 57943, 57943, 60451, 57943, 57943, 57943, 60456, 57943, 57943, 528, 1352, 528, 528, 1717, 528, 58791, 57893, 57893, 57893, 59066, 57893, 1151, 0, 0, 0, 3177, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 2390, 0, 0, 528, 528, 528, 528, 3232, 528, 528, 528, 528, 528, 528, 528, 528, 528, 528, 528, 528, 1941, 528, 528, 528, 528, 528, 3251, 528, 528, 528, 528, 528, 528, 528, 528, 528, 528, 528, 528, 528, 528, 2892, 528, 57893, 57893, 57893, 57893, 60643, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 50676, 58773, 990, 57916, 57916, 3442, 3443, 0, 3444, 0, 3446, 0, 0, 0, 0, 3448, 0, 0, 0, 0, 0, 0, 737, 0, 0, 0, 0, 0, 0, 0, 0, 0, 2360, 0, 0, 0, 0, 0, 2364, 3467, 0, 0, 0, 0, 3472, 3473, 0, 3475, 0, 0, 3478, 0, 0, 0, 3481, 528, 528, 528, 528, 3498, 528, 528, 528, 528, 528, 3504, 528, 528, 528, 528, 528, 528, 4020, 528, 4022, 4023, 528, 4025, 528, 57893, 57893, 57893, 57893, 57893, 57893, 60882, 57893, 57893, 57893, 57893, 57893, 60888, 57893, 57893, 60891, 57893, 57893, 57893, 0, 0, 0, 0, 57916, 57916, 57916, 59941, 57916, 59943, 57916, 57916, 57916, 57916, 57916, 60006, 60007, 60008, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 58943, 57916, 57916, 58947, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 60924, 57916, 57916, 57916, 57916, 57916, 60930, 57916, 57916, 60933, 57916, 57916, 57893, 57943, 57943, 57943, 57943, 57943, 59532, 57943, 57943, 57943, 57943, 57943, 59538, 57943, 60991, 57943, 528, 528, 3650, 57893, 57893, 60997, 0, 0, 0, 0, 0, 0, 3659, 0, 0, 0, 0, 3445, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 2812, 0, 0, 2815, 0, 0, 3662, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1306, 0, 3669, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 3676, 0, 3678, 528, 528, 528, 528, 3705, 528, 528, 528, 528, 528, 528, 528, 528, 528, 528, 528, 528, 2458, 528, 528, 528, 528, 528, 3715, 528, 528, 528, 528, 528, 528, 528, 528, 528, 528, 528, 528, 528, 528, 2903, 528, 528, 57893, 57893, 57893, 61071, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 61076, 57893, 57893, 57893, 0, 0, 0, 0, 57916, 57916, 59940, 57916, 57916, 57916, 57916, 57916, 57916, 59981, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 59453, 57916, 57916, 57916, 57916, 57916, 57916, 57893, 57893, 57893, 57893, 61095, 57893, 57893, 57893, 57893, 57893, 57893, 57916, 57916, 57916, 61100, 57916, 0, 0, 0, 0, 57893, 57943, 57943, 57943, 57943, 57943, 57943, 58432, 57943, 57943, 57943, 57943, 57943, 60075, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 59048, 57943, 57943, 57943, 57943, 61134, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 61145, 3832, 0, 0, 3835, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 772, 0, 0, 0, 57916, 57916, 61238, 57916, 57916, 57916, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 61142, 57943, 57943, 57943, 3924, 0, 3675, 0, 0, 0, 0, 0, 3932, 0, 0, 0, 0, 0, 0, 0, 312, 0, 0, 0, 0, 0, 311, 0, 311, 528, 528, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 61301, 57893, 57893, 57893, 57893, 57893, 59363, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 60315, 57893, 57893, 57893, 57893, 60319, 57893, 57893, 57943, 61333, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 528, 57893, 0, 0, 0, 0, 0, 1736, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 2334720, 0, 2334720, 0, 0, 0, 0, 0, 0, 4079, 0, 0, 0, 0, 0, 0, 0, 528, 528, 528, 528, 528, 528, 528, 4157, 528, 4159, 528, 528, 4091, 528, 528, 528, 528, 528, 528, 528, 528, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57916, 57916, 61448, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 61317, 57916, 61462, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 60028, 57943, 61476, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 4141, 4142, 0, 0, 0, 4145, 528, 528, 61507, 61508, 61509, 57893, 57893, 57893, 61512, 57893, 61514, 57893, 57893, 57893, 57893, 61519, 61520, 61521, 61522, 57916, 57916, 57916, 61525, 57916, 61527, 57916, 57916, 57916, 57916, 61532, 61533, 61534, 61535, 57943, 57943, 57943, 61538, 57943, 61540, 57943, 57943, 57943, 57943, 61545, 0, 0, 0, 0, 0, 0, 3194, 0, 0, 0, 3198, 0, 3200, 0, 0, 0, 528, 4245, 528, 528, 528, 4249, 57893, 57893, 61595, 57893, 57893, 57893, 61599, 57893, 57916, 57916, 57893, 57943, 59529, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 59536, 57943, 57943, 57943, 57943, 57943, 59040, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 528, 3412, 528, 57893, 60759, 57893, 61601, 57916, 57916, 57916, 61605, 57916, 57943, 57943, 61607, 57943, 57943, 57943, 61611, 57943, 0, 0, 0, 0, 0, 2342912, 0, 0, 0, 0, 0, 0, 0, 0, 0, 4358144, 4359058, 4358144, 4358144, 4358144, 4358144, 4358144, 4358144, 4358144, 4358144, 6275072, 4358144, 4358144, 4358144, 4358144, 4358144, 4358144, 4358144, 4358144, 4358144, 0, 413, 0, 0, 0, 0, 0, 0, 413, 139264, 147456, 0, 0, 0, 422, 0, 0, 0, 0, 3456, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 749, 0, 0, 0, 0, 372, 372, 0, 430, 131072, 372, 430, 430, 0, 333, 372, 430, 0, 0, 430, 456, 430, 0, 0, 0, 430, 495, 495, 495, 500, 495, 495, 495, 500, 495, 430, 430, 430, 430, 430, 430, 430, 430, 430, 430, 430, 430, 430, 430, 430, 430, 534, 57900, 534, 57900, 534, 534, 57900, 534, 534, 57923, 57900, 534, 534, 57900, 57900, 57900, 57900, 57950, 57950, 57900, 57900, 57900, 57900, 57950, 57950, 57900, 534, 57900, 57900, 57923, 57900, 57900, 57900, 57900, 57900, 57900, 57900, 57923, 57923, 57900, 57900, 57950, 57900, 57900, 57900, 1, 24578, 3, 155942, 155942, 296, 0, 0, 0, 0, 0, 302, 303, 0, 841, 528, 528, 528, 528, 528, 528, 873, 528, 528, 528, 528, 528, 528, 528, 528, 1939, 528, 528, 528, 528, 528, 528, 528, 0, 0, 1175, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 2416, 0, 0, 0, 0, 0, 1296, 0, 1175, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 2371, 0, 0, 0, 0, 0, 58773, 914, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 58784, 57893, 57893, 57893, 57893, 57893, 57893, 61210, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 60626, 57893, 57893, 60629, 57893, 57893, 57893, 58796, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 59424, 57943, 58973, 57943, 57943, 57943, 57943, 57943, 58985, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 60053, 57943, 57943, 57943, 57943, 57943, 57943, 0, 1873, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1743, 1744, 57943, 60460, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 60726, 57943, 61135, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 61257, 528, 528, 528, 61636, 57893, 57893, 57893, 61638, 57916, 57916, 57916, 61640, 57943, 57943, 57943, 0, 0, 0, 0, 3471, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1868, 0, 0, 0, 0, 0, 0, 0, 431, 131072, 0, 431, 431, 0, 0, 0, 431, 446, 0, 431, 0, 0, 0, 0, 3663, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1289, 0, 0, 0, 0, 431, 476, 476, 476, 489, 489, 489, 489, 489, 489, 489, 489, 489, 489, 489, 489, 489, 489, 489, 489, 535, 57901, 535, 57901, 535, 535, 57901, 535, 535, 57924, 57901, 535, 535, 57901, 57901, 57901, 57901, 57951, 57951, 57901, 57901, 57901, 57901, 57951, 57951, 57901, 535, 57901, 57901, 57924, 57901, 57901, 57901, 57901, 57901, 57901, 57901, 57924, 57924, 57901, 57901, 57951, 57901, 57901, 57901, 1, 24578, 3, 155942, 155942, 296, 0, 0, 0, 0, 0, 302, 303, 0, 528, 528, 528, 528, 58773, 1986, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 59353, 57893, 57893, 57893, 57893, 57893, 0, 0, 0, 2327, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 773, 0, 0, 776, 2351, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1774, 57893, 57893, 59892, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 59900, 57893, 57893, 0, 0, 0, 2805, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 800, 0, 0, 528, 3175, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1790, 0, 0, 3431, 0, 0, 0, 0, 0, 3437, 0, 0, 0, 0, 0, 0, 0, 331, 0, 139264, 147456, 0, 0, 0, 0, 0, 3482, 528, 528, 528, 528, 528, 528, 528, 528, 528, 528, 528, 528, 528, 528, 528, 1372, 57893, 57893, 57893, 57893, 60883, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 60893, 60908, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 60396, 60935, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 60944, 57916, 57916, 60947, 57916, 57916, 57893, 59528, 57943, 59530, 57943, 59531, 57943, 59533, 57943, 59535, 57943, 57943, 57943, 59539, 57916, 60950, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 59012, 57943, 57943, 60977, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 60986, 57943, 57943, 60989, 57943, 57943, 57943, 57943, 57943, 60966, 57943, 57943, 57943, 57943, 57943, 60972, 57943, 57943, 60975, 57943, 57943, 57943, 57943, 57943, 60750, 57943, 57943, 57943, 57943, 528, 528, 528, 57893, 57893, 57893, 0, 0, 0, 0, 0, 0, 0, 3660, 528, 528, 528, 528, 3716, 528, 3718, 528, 528, 528, 528, 528, 528, 528, 528, 528, 528, 3491, 528, 528, 528, 528, 528, 528, 57893, 57893, 61070, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 59385, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 61082, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 61088, 57893, 61090, 57943, 61146, 57943, 61148, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 528, 57893, 0, 0, 0, 528, 528, 528, 528, 3854, 528, 528, 528, 528, 528, 528, 528, 57893, 57893, 57893, 57893, 57893, 57893, 58277, 57893, 57893, 58293, 57893, 57893, 61216, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 61229, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 61235, 57916, 57916, 57916, 57916, 57916, 61467, 57916, 57916, 61469, 57916, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 60449, 57943, 57943, 60453, 57943, 57943, 57943, 57943, 57943, 61248, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 61254, 57943, 57943, 57943, 57943, 57943, 57943, 58980, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 60734, 57943, 57943, 57943, 57943, 57943, 57916, 61625, 61626, 57916, 57916, 57943, 57943, 61629, 61630, 57943, 57943, 0, 0, 0, 0, 528, 528, 528, 528, 4275, 528, 57893, 57893, 57893, 57893, 61623, 57893, 57916, 0, 761, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1789, 0, 761, 0, 0, 0, 0, 0, 761, 761, 0, 0, 828, 0, 0, 0, 0, 0, 0, 738, 0, 644, 738, 0, 742, 743, 644, 0, 0, 528, 528, 852, 528, 528, 528, 528, 528, 528, 528, 528, 528, 528, 528, 528, 528, 528, 2919, 528, 528, 528, 528, 911, 57893, 57893, 57893, 57893, 58271, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 58862, 57893, 57893, 57893, 50676, 58773, 990, 57916, 58871, 58406, 0, 0, 0, 0, 57893, 57943, 57943, 57943, 57943, 58422, 57943, 57943, 57943, 57943, 57943, 57943, 58981, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 60985, 57943, 57943, 57943, 57943, 57943, 528, 911, 0, 58271, 57893, 57893, 57893, 57893, 58330, 155942, 1151, 0, 0, 0, 0, 0, 0, 751, 0, 0, 0, 0, 0, 0, 0, 0, 0, 139264, 147456, 0, 0, 319, 0, 0, 0, 1279, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 2337, 0, 57943, 59035, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 59028, 57943, 528, 528, 528, 1983, 58773, 0, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 59418, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 59427, 50676, 0, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 60377, 57916, 57916, 57916, 57916, 59527, 57893, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 60746, 57943, 59623, 528, 528, 528, 528, 528, 57893, 57893, 57893, 57893, 57893, 0, 0, 0, 0, 0, 0, 0, 3142, 0, 0, 0, 0, 0, 0, 0, 752, 0, 0, 0, 0, 0, 0, 758, 0, 59861, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 59889, 57916, 57916, 57916, 57916, 59950, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 60675, 57916, 57916, 60678, 57943, 57943, 57943, 57943, 60033, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 60042, 57943, 57943, 57943, 3812, 528, 57893, 61158, 57893, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 386, 57916, 57916, 57916, 57916, 61240, 57916, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 61255, 57943, 57943, 57943, 57943, 61259, 57943, 528, 528, 57893, 57893, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 3146, 0, 0, 0, 0, 0, 334, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 334, 385, 387, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 402, 0, 0, 0, 0, 3672, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1786, 0, 0, 0, 0, 334, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 334, 0, 0, 139264, 147456, 0, 0, 0, 0, 0, 425, 425, 0, 0, 131072, 425, 0, 0, 0, 0, 425, 0, 447, 0, 0, 425, 335, 477, 477, 477, 490, 490, 490, 490, 490, 490, 490, 490, 490, 490, 511, 519, 519, 519, 519, 519, 526, 519, 519, 519, 526, 519, 519, 519, 519, 519, 519, 536, 57902, 536, 57902, 536, 536, 57902, 536, 536, 57925, 57902, 536, 536, 57902, 57902, 57902, 57902, 57952, 57952, 57902, 57902, 57902, 57902, 57952, 57952, 57902, 536, 57902, 57902, 57925, 57902, 57902, 57902, 57902, 57902, 57902, 57902, 57925, 57925, 57902, 57902, 57952, 57902, 57902, 57902, 1, 24578, 3, 155942, 155942, 296, 0, 0, 0, 0, 0, 302, 303, 0, 0, 0, 780, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 2789, 2790, 0, 0, 795, 0, 0, 0, 0, 0, 0, 0, 799, 0, 0, 0, 0, 0, 0, 0, 2330, 0, 0, 0, 0, 0, 0, 0, 0, 1238, 0, 0, 0, 0, 0, 0, 0, 819, 0, 0, 0, 0, 0, 819, 819, 0, 0, 0, 0, 799, 0, 0, 0, 0, 0, 1750, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 2850, 0, 0, 0, 0, 0, 0, 0, 0, 835, 795, 0, 0, 835, 0, 0, 0, 0, 0, 0, 0, 528, 528, 528, 4154, 528, 528, 528, 528, 528, 528, 528, 3234, 528, 528, 528, 528, 528, 528, 528, 528, 528, 3942, 528, 528, 528, 528, 528, 528, 528, 528, 528, 855, 528, 864, 528, 528, 528, 528, 528, 528, 528, 528, 528, 528, 528, 3944, 528, 528, 528, 528, 528, 528, 528, 912, 57893, 57893, 57893, 57893, 57893, 58274, 57893, 58283, 57893, 57893, 57893, 57893, 57893, 59395, 59396, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 59897, 57893, 57893, 57893, 57893, 57893, 57893, 58350, 57916, 58359, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 58401, 57916, 57916, 58407, 0, 0, 0, 0, 57893, 57943, 57943, 57943, 57943, 57943, 58425, 57943, 58434, 57943, 57943, 528, 1354, 528, 528, 528, 528, 58793, 57893, 57893, 57893, 57893, 57893, 1151, 0, 1143, 912, 0, 57893, 57893, 57893, 57893, 58493, 58331, 155942, 1151, 0, 0, 0, 0, 0, 0, 790, 0, 802, 0, 816, 0, 0, 0, 0, 810, 1246, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1821, 0, 0, 0, 1308, 0, 0, 0, 0, 0, 0, 0, 0, 1313, 0, 0, 0, 0, 0, 1796, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1211, 368, 368, 0, 0, 1214, 528, 528, 528, 1350, 528, 528, 528, 528, 528, 528, 528, 528, 528, 1369, 528, 528, 0, 0, 0, 2933, 0, 1986, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 59884, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 528, 1409, 528, 528, 1413, 528, 528, 528, 528, 528, 1420, 528, 528, 528, 1427, 528, 528, 0, 57893, 57893, 57893, 58315, 57893, 57893, 155942, 1151, 0, 0, 0, 0, 0, 0, 0, 6324224, 4358144, 4358144, 5005312, 4358144, 4358144, 4358144, 5120000, 5136384, 4358144, 4358144, 4358144, 4358144, 4358144, 4358144, 4358144, 4358144, 4358144, 6119424, 4358144, 6168576, 4358144, 4358144, 4358144, 4358144, 58773, 914, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 58789, 57893, 57893, 57893, 57893, 57893, 61379, 57893, 61381, 61382, 57893, 61384, 57893, 61386, 57916, 57916, 57916, 57916, 57916, 60385, 57916, 57916, 57916, 57916, 57916, 57916, 60393, 57916, 57916, 57916, 57916, 57916, 60691, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 59484, 57916, 57916, 57916, 57916, 57916, 57893, 57893, 58818, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 59914, 57893, 57893, 57916, 58954, 57916, 57916, 57916, 58961, 57916, 0, 57893, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 61140, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 60969, 57943, 60971, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 59038, 57943, 57943, 59042, 57943, 57943, 57943, 57943, 57943, 59049, 57943, 57943, 57943, 528, 528, 57893, 57893, 0, 0, 0, 0, 0, 0, 0, 0, 3923, 59056, 57943, 528, 528, 528, 528, 528, 528, 57893, 57893, 57893, 57893, 57893, 57893, 1151, 0, 0, 0, 0, 4003, 3675, 0, 0, 0, 0, 0, 0, 0, 0, 528, 528, 528, 528, 528, 528, 528, 1345, 1844, 1845, 0, 1847, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 806, 0, 0, 0, 0, 1861, 0, 1863, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 839, 0, 0, 528, 0, 0, 0, 1875, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 990, 0, 0, 0, 0, 0, 1861, 0, 0, 0, 0, 0, 528, 528, 528, 528, 528, 528, 528, 528, 3223, 528, 528, 528, 528, 528, 528, 1949, 528, 528, 528, 528, 528, 528, 528, 528, 1958, 528, 528, 528, 528, 528, 2478, 528, 528, 528, 528, 528, 528, 528, 528, 528, 528, 1421, 528, 528, 528, 528, 528, 528, 528, 528, 1967, 528, 528, 528, 528, 528, 528, 528, 528, 528, 528, 1977, 1979, 57893, 57893, 59392, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 59401, 57893, 57893, 57893, 57893, 57893, 59414, 57893, 57893, 57893, 57893, 57893, 57893, 59420, 57893, 57893, 57893, 57893, 57893, 59410, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 59421, 59423, 57893, 57893, 57893, 57893, 57893, 61453, 57893, 57893, 61455, 57893, 57916, 57916, 57916, 57916, 57916, 57916, 59482, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 59983, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 59492, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 59501, 57916, 57916, 57916, 57916, 57916, 57916, 60940, 60942, 57916, 57916, 57916, 57916, 60946, 57916, 60948, 60949, 57916, 59510, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 59521, 59523, 57916, 57916, 57916, 57916, 57916, 61570, 57916, 61571, 57916, 57916, 57916, 57916, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 60416, 57943, 57943, 57943, 59588, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 59597, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 59043, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 58988, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 59606, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 59617, 59619, 57943, 57943, 57943, 528, 528, 57893, 57893, 0, 0, 0, 0, 0, 3921, 0, 0, 0, 0, 0, 1812, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1839, 0, 0, 0, 0, 0, 0, 0, 2340, 0, 0, 0, 0, 0, 2346, 0, 0, 0, 0, 0, 0, 0, 372, 0, 0, 0, 380, 382, 0, 0, 0, 0, 0, 0, 2354, 0, 0, 0, 0, 0, 0, 0, 0, 2362, 0, 0, 0, 0, 0, 1835, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 528, 528, 528, 528, 528, 2424, 528, 528, 528, 528, 528, 528, 528, 528, 528, 528, 3722, 528, 528, 528, 528, 528, 528, 528, 2436, 528, 528, 528, 528, 528, 528, 528, 528, 528, 528, 528, 528, 528, 528, 2930, 528, 528, 528, 2463, 528, 528, 528, 528, 528, 528, 528, 528, 528, 528, 528, 528, 528, 528, 3240, 3241, 0, 0, 0, 57893, 57893, 57893, 57893, 57893, 57893, 59855, 57893, 57893, 57893, 57893, 57893, 57893, 0, 0, 57916, 60658, 57916, 57916, 57916, 57916, 57916, 57916, 58940, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 59518, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 60003, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 58951, 57916, 57916, 57916, 57916, 60018, 57916, 57916, 57916, 57943, 57943, 57943, 57943, 57943, 57943, 60027, 57943, 57943, 528, 1355, 528, 528, 528, 528, 58794, 57893, 57893, 57893, 57893, 57893, 1151, 0, 57943, 60086, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 59570, 57943, 57943, 57943, 57943, 60101, 57943, 57943, 57943, 2436, 528, 528, 528, 528, 59867, 57893, 57893, 57893, 0, 0, 0, 0, 57916, 59939, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 60927, 57916, 60929, 57916, 57916, 57916, 57916, 57916, 57916, 0, 2793, 0, 2795, 0, 0, 0, 0, 0, 0, 0, 0, 2800, 0, 0, 2803, 0, 0, 2818, 0, 0, 0, 0, 0, 0, 0, 2826, 0, 0, 0, 0, 2828, 0, 2842, 0, 0, 2845, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 2855, 0, 0, 528, 2869, 528, 528, 528, 528, 528, 528, 528, 528, 528, 528, 528, 528, 528, 2918, 528, 528, 57893, 57893, 57893, 57893, 60327, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57916, 57916, 61099, 57916, 57916, 60336, 57893, 57893, 57893, 60339, 57893, 57893, 57893, 57893, 57893, 57893, 0, 0, 0, 0, 0, 0, 2772, 0, 0, 0, 0, 0, 0, 0, 0, 2359, 0, 0, 0, 0, 0, 0, 0, 0, 57916, 60345, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 58894, 57916, 57916, 57943, 60433, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 60438, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 59549, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 60067, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 60444, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 60452, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 59595, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 61543, 57943, 57943, 0, 0, 0, 0, 60459, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 60468, 57943, 57943, 57943, 528, 528, 57893, 57893, 0, 0, 0, 3919, 0, 0, 3922, 0, 0, 60471, 57943, 57943, 57943, 57943, 57943, 57943, 3131, 528, 528, 528, 528, 60478, 57893, 57893, 57893, 0, 0, 0, 0, 59938, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 59946, 3204, 0, 0, 0, 3206, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1818, 0, 0, 0, 0, 528, 528, 528, 528, 3244, 528, 528, 528, 528, 528, 528, 528, 528, 528, 528, 3249, 528, 528, 528, 3252, 528, 3254, 528, 528, 528, 528, 528, 528, 528, 3258, 528, 528, 0, 0, 2503, 0, 0, 0, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 59858, 57893, 57893, 57893, 60632, 57893, 57893, 57893, 57893, 60635, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 59924, 57893, 59926, 57893, 57893, 57893, 57893, 60641, 57893, 57893, 57893, 60644, 57893, 60646, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 58843, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 60651, 57893, 57893, 57893, 57893, 57893, 0, 0, 57916, 57916, 57916, 60659, 57916, 57916, 57916, 57916, 57916, 58905, 57916, 58907, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 60362, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 60664, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 59458, 57916, 57916, 57916, 60680, 60681, 57916, 57916, 57916, 57916, 60684, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57943, 60022, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 61153, 57943, 57943, 57943, 57943, 57943, 57943, 528, 57916, 57916, 57916, 57916, 60690, 57916, 57916, 57916, 60693, 57916, 60695, 57916, 57916, 57916, 57916, 57916, 57916, 61105, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 58944, 57916, 57916, 57916, 57916, 57916, 58952, 57916, 57916, 57916, 60700, 57916, 57916, 57916, 57916, 57916, 57943, 57943, 57943, 60708, 57943, 57943, 57943, 528, 528, 57893, 57893, 3918, 0, 0, 0, 3920, 0, 0, 0, 0, 0, 0, 3695, 0, 3697, 528, 528, 528, 528, 528, 528, 528, 1905, 528, 528, 528, 528, 528, 528, 528, 528, 2926, 528, 528, 528, 528, 528, 528, 528, 57943, 57943, 60713, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 61144, 57943, 57943, 57943, 57943, 60729, 60730, 57943, 57943, 57943, 57943, 60733, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 60036, 57943, 60038, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 58984, 57943, 58989, 57943, 57943, 58993, 57943, 57943, 58996, 57943, 57943, 57943, 57943, 60749, 57943, 57943, 57943, 57943, 57943, 528, 528, 528, 57893, 57893, 57893, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 3417, 0, 0, 3420, 3421, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1827, 0, 0, 0, 0, 528, 528, 528, 528, 3486, 528, 528, 528, 528, 3490, 528, 528, 528, 528, 528, 528, 1952, 1953, 528, 528, 528, 528, 528, 528, 528, 528, 528, 3517, 528, 528, 528, 528, 528, 57893, 528, 3496, 528, 528, 528, 528, 528, 528, 528, 528, 528, 528, 528, 528, 528, 528, 528, 2931, 528, 3510, 528, 528, 528, 528, 528, 528, 528, 528, 528, 528, 528, 528, 528, 57893, 57893, 57893, 61204, 57893, 57893, 57893, 60870, 57893, 57893, 57893, 57893, 60874, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 58863, 58864, 57893, 57893, 50676, 58773, 990, 57916, 57916, 60880, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 59901, 57893, 60895, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 58830, 57893, 57916, 57916, 57916, 57916, 60912, 57916, 57916, 57916, 57916, 60916, 57916, 57916, 57916, 57916, 57916, 57916, 58962, 0, 58963, 58964, 57943, 57943, 57943, 57943, 57943, 58971, 57916, 60922, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 59472, 57916, 57916, 57916, 60937, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 58927, 57916, 58929, 57916, 57943, 57943, 57943, 57943, 60954, 57943, 57943, 57943, 57943, 60958, 57943, 57943, 57943, 57943, 57943, 57943, 58982, 57943, 57943, 57943, 57943, 58992, 57943, 57943, 57943, 57943, 57943, 57943, 60964, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 61256, 57943, 57943, 57943, 57943, 60979, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 60974, 57943, 60976, 528, 528, 57893, 57893, 57893, 0, 0, 0, 0, 3819, 0, 0, 0, 0, 0, 3822, 0, 528, 528, 528, 528, 528, 528, 3844, 528, 528, 528, 528, 528, 528, 528, 528, 2481, 528, 528, 528, 528, 528, 2485, 2486, 57893, 57893, 61206, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 59927, 57893, 57893, 57893, 57893, 61217, 57893, 57893, 57893, 57893, 57893, 57893, 57916, 57916, 57916, 57916, 57916, 57916, 61225, 0, 0, 3675, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 3934, 0, 0, 0, 0, 4080, 0, 0, 0, 0, 0, 0, 528, 528, 528, 528, 528, 528, 4211, 528, 4212, 528, 528, 528, 57893, 57893, 57893, 57893, 57893, 57893, 61562, 57893, 61563, 57893, 57893, 57893, 57893, 57916, 57943, 4313, 4314, 61659, 61660, 61661, 0, 528, 57893, 57916, 57943, 0, 0, 0, 0, 0, 2382, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 4284416, 0, 0, 0, 0, 0, 57943, 57943, 61578, 57943, 61579, 57943, 57943, 57943, 57943, 0, 0, 0, 0, 0, 0, 528, 528, 4273, 4274, 528, 528, 57893, 57893, 61621, 61622, 57893, 57893, 57916, 0, 528, 528, 57893, 57893, 57916, 57916, 57943, 57943, 4303, 4304, 61649, 61650, 61651, 0, 528, 528, 528, 528, 528, 528, 528, 528, 528, 2428, 528, 528, 528, 528, 528, 528, 4096, 528, 528, 4098, 57893, 57893, 57893, 57893, 57893, 57893, 58861, 57893, 57893, 57893, 57893, 50676, 58773, 990, 57916, 57916, 537, 57903, 537, 57903, 537, 537, 57903, 537, 537, 57926, 57903, 537, 537, 57903, 57903, 57903, 57903, 57953, 57953, 57903, 57903, 57903, 57903, 57953, 57953, 57903, 537, 57903, 57903, 57926, 57903, 57903, 57903, 57903, 57903, 57903, 57903, 57926, 57926, 57903, 57903, 57953, 57903, 57903, 57903, 1, 24578, 3, 155942, 155942, 296, 0, 0, 0, 0, 0, 302, 303, 0, 57893, 57893, 58833, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 60649, 57893, 57893, 57916, 59948, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 59489, 57916, 57943, 60031, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 60044, 57943, 0, 384, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 2404, 0, 0, 340, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 2854, 0, 0, 0, 406, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 406, 0, 0, 0, 432, 131072, 0, 432, 432, 0, 0, 0, 432, 0, 452, 432, 0, 0, 0, 0, 4206, 0, 528, 528, 528, 528, 528, 528, 528, 528, 528, 528, 528, 528, 2430, 528, 528, 432, 478, 478, 478, 491, 491, 491, 491, 491, 491, 491, 491, 491, 491, 491, 491, 491, 491, 491, 491, 538, 57904, 538, 57904, 538, 538, 57904, 538, 538, 57927, 57904, 538, 538, 57904, 57904, 57904, 57904, 57954, 57954, 57904, 57904, 57904, 57904, 57954, 57954, 57904, 538, 57904, 57904, 57927, 57904, 57904, 57904, 57904, 57904, 57904, 57904, 57927, 57927, 57904, 57904, 57954, 57904, 57904, 57904, 1, 24578, 3, 155942, 155942, 296, 0, 0, 0, 0, 0, 302, 303, 0, 0, 0, 661, 0, 661, 0, 0, 0, 0, 826, 0, 0, 0, 661, 0, 0, 0, 0, 0, 5218304, 0, 0, 0, 0, 5799936, 0, 5881856, 0, 0, 0, 0, 0, 0, 5029888, 5038080, 0, 0, 5103616, 5201920, 0, 0, 0, 0, 0, 0, 1878, 0, 0, 0, 0, 0, 0, 0, 0, 0, 2296, 0, 0, 0, 0, 0, 0, 842, 528, 528, 528, 857, 528, 528, 528, 528, 528, 528, 528, 528, 528, 528, 898, 528, 528, 528, 528, 57893, 57893, 58260, 57893, 57893, 57893, 58276, 57893, 57893, 57893, 57893, 57893, 57893, 61220, 57893, 57893, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 0, 57893, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 59022, 57943, 59024, 57943, 57943, 57943, 57943, 57943, 57916, 58352, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 58393, 57916, 57916, 57916, 57916, 57916, 60913, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 61471, 57943, 57943, 57943, 57943, 57943, 1160, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1167, 1168, 0, 0, 0, 0, 0, 0, 1299, 0, 0, 0, 0, 0, 0, 0, 0, 528, 528, 1340, 528, 528, 528, 528, 528, 0, 0, 0, 1249, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1260, 1261, 0, 0, 0, 0, 131072, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 2415, 0, 0, 2418, 528, 528, 528, 528, 1351, 1358, 528, 528, 528, 528, 528, 528, 528, 528, 1371, 528, 528, 0, 57893, 57893, 57893, 58317, 57893, 57893, 294, 1151, 0, 0, 1155, 0, 0, 0, 0, 0, 5857280, 0, 6463488, 4939776, 0, 0, 5455872, 0, 0, 0, 0, 0, 5554176, 5570560, 5578752, 5619712, 5668864, 0, 0, 0, 5791744, 5816320, 0, 5857280, 0, 1373, 528, 528, 528, 528, 528, 528, 528, 528, 528, 528, 528, 528, 528, 528, 528, 1406, 1408, 528, 528, 528, 528, 528, 528, 528, 528, 528, 528, 528, 1423, 528, 528, 528, 528, 528, 2910, 528, 528, 528, 528, 528, 528, 2917, 528, 528, 528, 528, 528, 2490, 528, 528, 528, 528, 528, 528, 528, 528, 528, 0, 0, 57893, 57893, 57893, 57893, 57893, 60612, 57893, 60613, 57893, 57893, 57893, 58773, 914, 57893, 58776, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 58790, 58797, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 58810, 57893, 58812, 57893, 57893, 57893, 57893, 57893, 59867, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 59872, 57893, 57893, 57893, 57893, 57893, 57893, 58832, 57893, 58834, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 58847, 57893, 57893, 57893, 57893, 57893, 59881, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 59888, 57893, 57916, 57916, 57916, 58957, 57916, 57916, 57916, 0, 57893, 57943, 58965, 57943, 57943, 57943, 57943, 57943, 57943, 59548, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 61154, 57943, 57943, 57943, 57943, 528, 58999, 57943, 59001, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 59027, 57943, 57943, 57943, 57943, 59037, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 59052, 57943, 57943, 57943, 57943, 57943, 61139, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 60454, 57943, 57943, 57943, 57943, 1946, 528, 528, 528, 528, 528, 528, 528, 528, 1955, 528, 528, 528, 528, 528, 528, 1970, 528, 528, 528, 528, 528, 528, 1976, 528, 528, 528, 528, 1982, 528, 58773, 1986, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 60648, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 59346, 59347, 57893, 57893, 57893, 57893, 59354, 57893, 57893, 57893, 57893, 57893, 58315, 57893, 57893, 57893, 57893, 0, 57916, 57916, 57916, 57916, 57916, 57916, 60349, 57916, 57916, 57916, 57916, 60353, 57916, 57916, 57916, 57893, 59426, 57893, 50676, 2086, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 60932, 57916, 60934, 57916, 57916, 57916, 57916, 59446, 59447, 57916, 57916, 57916, 57916, 59454, 57916, 57916, 57916, 57916, 57916, 57916, 59449, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 60390, 57916, 57916, 57916, 57916, 57916, 57916, 59526, 57916, 57893, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 59051, 57943, 57943, 57943, 59542, 59543, 57943, 57943, 57943, 57943, 59550, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 60425, 57943, 57943, 57943, 57943, 57943, 57943, 0, 2313, 0, 0, 0, 0, 0, 0, 0, 0, 0, 2322, 0, 0, 0, 0, 0, 0, 3208, 0, 0, 0, 0, 0, 0, 0, 0, 0, 3210, 3211, 0, 0, 0, 0, 0, 528, 528, 528, 528, 2477, 528, 528, 528, 528, 528, 528, 528, 2483, 528, 528, 528, 528, 528, 2924, 528, 528, 528, 528, 528, 528, 528, 528, 528, 528, 528, 2496, 528, 528, 528, 0, 528, 528, 528, 2488, 528, 528, 528, 528, 528, 528, 528, 528, 528, 528, 528, 0, 0, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 0, 0, 990, 57916, 57916, 57916, 57916, 57916, 60004, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 60010, 57916, 57916, 57916, 57916, 57916, 58921, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 60917, 57916, 57916, 57916, 57916, 57916, 57943, 57943, 57943, 60087, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 60093, 57943, 57943, 57943, 57943, 57943, 57943, 60967, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 60040, 57943, 57943, 57943, 57943, 57943, 0, 0, 0, 2782, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1169, 1170, 1171, 1172, 3159, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1871, 528, 528, 528, 3231, 528, 528, 528, 528, 528, 528, 528, 528, 528, 528, 528, 528, 528, 3248, 528, 528, 528, 3262, 528, 0, 0, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 60614, 57893, 57893, 57893, 50676, 0, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 59441, 57893, 57893, 57893, 57893, 60622, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57916, 61098, 57916, 57916, 57916, 57893, 57893, 57893, 57893, 60655, 57893, 0, 0, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 60928, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 60663, 57916, 57916, 57916, 57916, 57916, 57916, 60671, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 61118, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57943, 60712, 57943, 57943, 57943, 57943, 57943, 57943, 60720, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 61340, 57943, 57943, 528, 57893, 0, 0, 0, 57893, 57893, 57893, 61094, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57916, 57916, 57916, 57916, 57916, 57916, 61232, 57916, 57916, 57916, 57916, 61233, 61234, 57916, 57916, 57916, 528, 528, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 61513, 57893, 61515, 57893, 57893, 57893, 57893, 57893, 59894, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 0, 57916, 57916, 58335, 57916, 57916, 528, 528, 61559, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 61567, 61624, 57916, 57916, 57916, 57916, 57943, 61628, 57943, 57943, 57943, 57943, 0, 0, 0, 0, 528, 4272, 528, 528, 528, 528, 57893, 61620, 57893, 57893, 57893, 57893, 57916, 0, 389, 341, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 2801, 0, 0, 463, 479, 479, 479, 463, 463, 463, 463, 463, 463, 463, 463, 463, 463, 512, 512, 512, 512, 512, 512, 512, 512, 512, 512, 512, 512, 512, 512, 512, 512, 539, 57905, 539, 57905, 539, 539, 57905, 539, 539, 57928, 57905, 539, 539, 57905, 57905, 57905, 57905, 57955, 57955, 57905, 57905, 57905, 57905, 57955, 57955, 57905, 539, 57905, 57905, 57928, 57905, 57905, 57905, 57905, 57905, 57905, 57905, 57928, 57928, 57905, 57905, 57955, 57905, 57905, 57905, 1, 24578, 3, 155942, 156282, 296, 0, 0, 0, 0, 0, 302, 303, 0, 0, 732, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 3427, 0, 0, 0, 662, 0, 662, 0, 0, 0, 0, 0, 0, 0, 0, 662, 0, 0, 0, 0, 0, 6225920, 4358144, 5062656, 4358144, 4358144, 4358144, 4358144, 4358144, 6225920, 0, 6086656, 528, 528, 528, 528, 858, 528, 528, 874, 528, 528, 528, 528, 528, 528, 528, 528, 2913, 528, 528, 528, 528, 528, 528, 528, 57916, 58353, 57916, 57916, 58369, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 61575, 57943, 57943, 57943, 0, 1189, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 3451, 0, 0, 1318, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 3480, 0, 0, 0, 1334, 0, 0, 0, 0, 0, 528, 528, 528, 528, 528, 528, 528, 528, 2876, 528, 528, 528, 528, 528, 528, 528, 528, 528, 1352, 528, 528, 528, 528, 528, 528, 528, 528, 528, 528, 528, 528, 2469, 528, 528, 528, 528, 528, 528, 1391, 528, 528, 528, 528, 528, 528, 528, 528, 528, 528, 528, 528, 528, 3493, 528, 528, 528, 528, 1410, 528, 528, 528, 528, 528, 1418, 528, 528, 528, 528, 528, 528, 528, 1379, 528, 528, 528, 528, 528, 528, 528, 528, 1954, 528, 528, 1957, 528, 528, 528, 528, 58773, 914, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 58791, 57893, 57893, 57893, 58857, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 50676, 58773, 990, 57916, 57916, 57916, 57916, 57943, 57943, 57943, 60412, 57943, 60413, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 58471, 57943, 57943, 57943, 57943, 528, 528, 528, 901, 57943, 57943, 57943, 57943, 59039, 57943, 57943, 57943, 57943, 57943, 59047, 57943, 57943, 57943, 57943, 57943, 57943, 59563, 59564, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 61484, 0, 0, 4143, 4144, 0, 0, 0, 1727, 0, 0, 0, 0, 1734, 0, 0, 0, 0, 0, 0, 0, 0, 0, 2370, 0, 0, 0, 0, 0, 0, 528, 1916, 528, 528, 528, 528, 528, 528, 528, 528, 528, 528, 528, 528, 1927, 528, 528, 0, 57893, 57893, 57893, 58319, 57893, 57893, 155942, 1151, 0, 302, 0, 0, 306, 307, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 650, 0, 0, 0, 0, 0, 2351104, 0, 0, 0, 0, 0, 0, 0, 0, 0, 4358144, 59359, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 59370, 57893, 57893, 57893, 50676, 0, 57916, 59433, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 59440, 57916, 57943, 57943, 57943, 59544, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 59555, 57943, 57943, 528, 1714, 528, 1716, 528, 528, 59063, 57893, 57893, 59065, 57893, 57893, 1151, 0, 57943, 57943, 57943, 59577, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 61143, 57943, 57943, 0, 0, 2379, 2380, 0, 0, 0, 0, 0, 2386, 0, 0, 0, 0, 0, 0, 0, 2785, 0, 0, 0, 0, 0, 0, 0, 0, 1753, 0, 0, 0, 0, 0, 0, 0, 528, 2474, 528, 528, 528, 528, 2479, 528, 528, 528, 528, 528, 528, 528, 528, 528, 1396, 528, 528, 528, 1403, 1405, 528, 2501, 0, 0, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 60878, 57893, 57893, 57893, 57893, 57893, 57893, 59906, 57893, 57893, 57893, 57893, 59911, 57893, 57893, 57893, 57893, 57893, 57893, 2999, 0, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57943, 61327, 57943, 57943, 61329, 57943, 57943, 57943, 57916, 59964, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 59506, 57916, 57943, 60047, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 60070, 60071, 57916, 57916, 57916, 57916, 61115, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 58885, 58892, 57916, 57916, 57916, 0, 3925, 3675, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 2838, 2839, 0, 3949, 528, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 58850, 57893, 57893, 57893, 61308, 57893, 57893, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 58886, 57916, 57916, 57916, 57916, 4001, 0, 0, 0, 0, 3675, 0, 0, 0, 0, 0, 0, 0, 0, 528, 528, 528, 528, 528, 528, 1895, 528, 528, 528, 4016, 528, 528, 528, 528, 528, 528, 528, 528, 528, 528, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 61375, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57916, 57916, 57916, 57916, 57916, 61461, 57916, 61391, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 61407, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 0, 0, 0, 0, 0, 2316, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 3838, 0, 0, 0, 0, 0, 528, 528, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 61566, 57916, 0, 0, 0, 0, 57893, 57943, 57943, 58410, 57943, 57943, 57943, 57943, 57943, 57943, 58443, 0, 0, 0, 390, 391, 393, 343, 0, 0, 0, 0, 0, 0, 342, 0, 0, 0, 343, 0, 0, 0, 0, 0, 0, 0, 0, 0, 2787, 0, 0, 0, 0, 0, 0, 0, 0, 0, 390, 0, 0, 0, 0, 0, 0, 343, 0, 0, 0, 390, 0, 0, 0, 0, 131072, 0, 0, 0, 0, 0, 0, 0, 0, 0, 310, 0, 0, 0, 0, 1749, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 540672, 0, 0, 0, 0, 0, 0, 480, 480, 480, 480, 496, 496, 496, 496, 496, 496, 496, 496, 496, 480, 480, 480, 480, 480, 480, 480, 480, 480, 480, 480, 480, 480, 480, 480, 480, 540, 57906, 540, 57906, 540, 540, 57906, 540, 540, 57929, 57906, 540, 540, 57906, 57906, 57906, 1, 24578, 3, 155942, 155942, 296, 0, 0, 0, 0, 0, 302, 303, 0, 57929, 57906, 57906, 57906, 57906, 57906, 57906, 57906, 57929, 57929, 57906, 57942, 57956, 57942, 57942, 57942, 57942, 57956, 57956, 57942, 57942, 57942, 57942, 57956, 57956, 57942, 540, 57906, 57906, 368, 368, 0, 0, 0, 0, 0, 706, 0, 0, 0, 0, 0, 0, 0, 0, 740, 0, 0, 0, 0, 740, 0, 746, 715, 0, 717, 0, 0, 0, 0, 0, 0, 0, 725, 0, 0, 0, 0, 0, 0, 804, 0, 0, 0, 0, 0, 0, 804, 0, 528, 0, 0, 813, 0, 796, 0, 0, 815, 0, 667, 0, 792, 0, 0, 0, 0, 0, 836, 0, 0, 0, 0, 796, 663, 840, 0, 528, 0, 792, 815, 0, 815, 813, 0, 0, 0, 827, 0, 0, 0, 663, 830, 0, 0, 0, 0, 131072, 0, 0, 0, 0, 0, 0, 0, 444, 0, 0, 0, 0, 0, 690, 691, 0, 0, 0, 0, 696, 0, 0, 0, 368, 843, 528, 528, 528, 859, 528, 528, 875, 528, 528, 528, 528, 889, 528, 528, 528, 528, 528, 3245, 528, 528, 528, 528, 528, 528, 528, 528, 528, 528, 528, 1398, 528, 528, 528, 528, 528, 907, 528, 528, 57893, 57893, 58261, 57893, 57893, 57893, 58278, 57893, 57893, 58294, 57893, 57893, 57893, 50676, 0, 59432, 57916, 59434, 57916, 59435, 57916, 59437, 57916, 59439, 57916, 57916, 57893, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 59534, 57943, 57943, 57943, 57943, 57943, 57943, 59594, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 60467, 57943, 57943, 57943, 57943, 57943, 57893, 57893, 58308, 57893, 57893, 57893, 57893, 58326, 57893, 57893, 0, 57916, 57916, 58337, 57916, 57916, 57916, 57916, 57943, 57943, 60411, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 59044, 57943, 57943, 57943, 57943, 57943, 57943, 57916, 58354, 57916, 57916, 58370, 57916, 57916, 57916, 57916, 58384, 57916, 57916, 57916, 57916, 58402, 57916, 0, 0, 0, 0, 57893, 57943, 57943, 58411, 57943, 57943, 57943, 58427, 57943, 57943, 57943, 57943, 57943, 59005, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 59008, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 58459, 57943, 57943, 57943, 57943, 58477, 57943, 57943, 528, 528, 528, 528, 528, 528, 57893, 57893, 57893, 57893, 57893, 57893, 1151, 0, 0, 0, 1170, 0, 0, 0, 0, 0, 0, 0, 1172, 0, 0, 0, 0, 0, 0, 806, 0, 0, 0, 0, 787, 0, 806, 0, 528, 1293, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1305, 0, 0, 0, 0, 0, 6307840, 0, 0, 6356992, 6381568, 6397952, 4800512, 4808704, 0, 0, 4890624, 0, 4947968, 0, 0, 0, 5046272, 0, 0, 0, 0, 5185536, 0, 5234688, 5300224, 0, 0, 0, 0, 1210, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1305, 0, 0, 0, 0, 0, 2343, 0, 0, 0, 0, 0, 2347, 0, 0, 2349, 0, 0, 1333, 0, 0, 0, 0, 0, 0, 528, 528, 528, 528, 528, 528, 528, 528, 528, 528, 528, 528, 528, 528, 528, 528, 1353, 528, 528, 528, 528, 528, 528, 528, 528, 528, 528, 528, 528, 2901, 528, 528, 528, 528, 528, 528, 1377, 528, 528, 528, 528, 528, 528, 528, 528, 528, 528, 528, 528, 528, 3724, 528, 528, 58773, 914, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 58792, 0, 0, 1777, 0, 0, 1780, 0, 1782, 0, 0, 0, 0, 0, 0, 0, 0, 753, 0, 0, 0, 0, 0, 0, 0, 0, 1833, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 3667, 3668, 528, 1898, 528, 528, 528, 528, 528, 528, 528, 528, 1909, 528, 528, 528, 528, 528, 868, 528, 528, 528, 528, 528, 887, 528, 528, 528, 528, 59341, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 59352, 57893, 57893, 57893, 57893, 57893, 57893, 58325, 57893, 57893, 57893, 0, 57916, 57916, 57916, 57916, 57916, 57943, 57943, 57943, 57943, 59578, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 60056, 57943, 57943, 57943, 0, 2394, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 2405, 0, 528, 528, 528, 2422, 2423, 528, 528, 528, 528, 528, 528, 528, 528, 528, 528, 1366, 528, 528, 528, 528, 528, 0, 0, 0, 57893, 57893, 57893, 59852, 57893, 59854, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 59349, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 60647, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 59903, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 59405, 57893, 0, 2780, 0, 0, 2783, 0, 0, 0, 0, 0, 0, 2788, 0, 0, 0, 2791, 2792, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 2802, 0, 0, 0, 0, 131072, 0, 0, 0, 0, 0, 0, 0, 448, 0, 0, 0, 0, 0, 2307, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 3674, 0, 0, 0, 0, 0, 0, 2817, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 3677, 0, 0, 0, 2843, 0, 0, 0, 0, 0, 0, 0, 0, 2851, 0, 0, 0, 0, 0, 0, 3448, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1287, 1288, 0, 0, 0, 0, 0, 0, 2857, 0, 0, 0, 0, 2860, 0, 0, 0, 0, 0, 0, 0, 0, 0, 2862, 0, 0, 0, 0, 0, 0, 2868, 0, 528, 528, 528, 528, 528, 528, 528, 528, 528, 528, 528, 528, 528, 528, 528, 57893, 57893, 57893, 57893, 2879, 528, 528, 2882, 2883, 528, 528, 528, 528, 528, 2888, 528, 2890, 528, 528, 528, 528, 528, 3487, 528, 528, 528, 528, 528, 528, 528, 528, 528, 528, 2468, 528, 528, 528, 528, 528, 528, 2894, 528, 528, 528, 528, 528, 528, 528, 528, 528, 2900, 528, 528, 528, 528, 528, 1903, 1904, 528, 528, 528, 528, 1911, 528, 528, 528, 528, 528, 1935, 528, 528, 528, 1940, 528, 528, 528, 528, 528, 528, 2897, 528, 528, 528, 528, 528, 528, 528, 528, 528, 528, 3858, 528, 57893, 57893, 57893, 57893, 528, 528, 528, 528, 2908, 528, 528, 528, 528, 528, 528, 2916, 528, 528, 528, 528, 528, 2439, 528, 528, 528, 528, 2444, 528, 528, 528, 528, 528, 528, 61594, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 61600, 57916, 528, 2921, 528, 528, 528, 528, 528, 528, 528, 528, 528, 528, 528, 528, 528, 528, 1387, 528, 57893, 60297, 57893, 60299, 57893, 57893, 57893, 57893, 60303, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 59382, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 59398, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 60309, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 60318, 57893, 57893, 57893, 0, 0, 0, 2593, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 60704, 57916, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 60957, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57893, 57893, 57893, 60326, 57893, 57893, 57893, 57893, 57893, 60331, 57893, 57893, 57893, 57893, 57893, 57893, 58837, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 58851, 57893, 60355, 57916, 57916, 60358, 60359, 57916, 57916, 57916, 57916, 57916, 60364, 57916, 60366, 57916, 57916, 57916, 57916, 57916, 60925, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 58884, 57916, 57916, 57916, 57916, 57916, 57916, 60370, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 60376, 57916, 57916, 57916, 57916, 57916, 58960, 57916, 0, 57893, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 60732, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 60450, 57943, 57943, 57943, 57943, 57943, 57943, 60458, 57916, 57916, 60398, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 59505, 57916, 57916, 57943, 57943, 57943, 60420, 57943, 57943, 60423, 60424, 57943, 57943, 57943, 57943, 57943, 60429, 57943, 60431, 57943, 57943, 57943, 57943, 60435, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 60441, 57943, 57943, 57943, 57943, 57943, 61150, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 528, 528, 3133, 528, 528, 57893, 57893, 60480, 57893, 0, 0, 3161, 3162, 3163, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 2311, 0, 0, 0, 0, 0, 3176, 0, 0, 0, 0, 0, 0, 0, 3182, 0, 3184, 0, 0, 0, 0, 0, 0, 3458, 0, 0, 0, 0, 0, 0, 0, 0, 0, 786, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 802, 0, 807, 0, 810, 3188, 0, 0, 0, 0, 3193, 0, 0, 3196, 0, 0, 0, 0, 0, 0, 0, 389, 0, 139264, 147456, 0, 0, 0, 0, 0, 528, 528, 3263, 0, 0, 60608, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 61086, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 60653, 57893, 57893, 60656, 0, 0, 60657, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 58880, 57916, 57916, 57916, 58889, 57916, 57916, 57916, 57916, 57916, 57916, 60689, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 59986, 57916, 57916, 57916, 57916, 60699, 57916, 57916, 60702, 57916, 57916, 60705, 60706, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 60052, 57943, 57943, 57943, 57943, 57943, 57943, 60057, 57943, 60727, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 59031, 57943, 57943, 57943, 60738, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 58994, 57943, 57943, 57943, 57943, 57943, 57943, 60748, 57943, 57943, 60751, 57943, 57943, 60754, 3411, 528, 528, 60758, 57893, 57893, 57893, 50676, 2085, 57916, 57916, 57916, 57916, 57916, 59436, 57916, 57916, 57916, 57916, 57916, 57916, 60373, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 60379, 57916, 3429, 0, 0, 0, 0, 3434, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 3675, 0, 0, 0, 0, 0, 0, 0, 0, 528, 528, 0, 0, 0, 3470, 0, 0, 0, 0, 0, 0, 3477, 0, 0, 3479, 0, 0, 0, 0, 306, 0, 0, 0, 0, 0, 0, 306, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 2852, 0, 0, 0, 528, 528, 3484, 528, 528, 528, 528, 528, 528, 528, 528, 528, 528, 528, 528, 528, 528, 3259, 528, 3509, 528, 528, 3511, 528, 528, 528, 528, 528, 528, 528, 528, 528, 528, 528, 57893, 57893, 57893, 57893, 61446, 57893, 57893, 60868, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 59928, 57893, 60894, 57893, 57893, 60896, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 60877, 57893, 57893, 57893, 57916, 57916, 60910, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 60012, 57916, 57916, 57916, 60936, 57916, 57916, 60938, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 59485, 57916, 57916, 57916, 57916, 57916, 57943, 57943, 60952, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 59009, 57943, 57943, 57943, 57943, 57943, 60978, 57943, 57943, 60980, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 60055, 57943, 57943, 57943, 60058, 3661, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 2312, 3679, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 3691, 0, 0, 0, 0, 131072, 0, 0, 0, 0, 0, 0, 0, 450, 0, 0, 0, 0, 0, 3457, 0, 0, 0, 0, 3462, 0, 0, 0, 0, 3466, 3702, 3703, 528, 528, 528, 528, 3707, 3708, 528, 528, 528, 528, 528, 3712, 528, 528, 0, 2932, 0, 0, 0, 0, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 59859, 57893, 57893, 528, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 61073, 61074, 57893, 57893, 57893, 57893, 61078, 61079, 57916, 57916, 61102, 61103, 57916, 57916, 57916, 57916, 61107, 61108, 57916, 57916, 57916, 57916, 57916, 61112, 57943, 57943, 61136, 61137, 57943, 57943, 57943, 57943, 57943, 61141, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 60436, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 58476, 57943, 57943, 57943, 1139, 528, 528, 528, 57943, 57943, 61147, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 528, 528, 528, 528, 528, 528, 57893, 57893, 57893, 0, 0, 3817, 0, 0, 0, 0, 0, 0, 0, 0, 1165, 0, 0, 0, 0, 0, 0, 0, 0, 0, 3834, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 2865, 0, 0, 57916, 57916, 57916, 57916, 61230, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 59486, 57916, 57916, 57916, 57916, 57943, 61249, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 60096, 57943, 0, 0, 3675, 0, 3928, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 2361, 0, 0, 0, 0, 528, 528, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 61303, 57893, 57893, 57893, 50676, 2086, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 61236, 61319, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 0, 0, 57943, 57943, 57943, 61335, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 528, 57893, 0, 3999, 0, 0, 0, 0, 131072, 0, 0, 0, 0, 0, 0, 0, 450560, 0, 0, 0, 0, 0, 372, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 2333, 0, 0, 0, 0, 2338, 0, 4002, 0, 0, 0, 3675, 0, 0, 0, 0, 0, 0, 0, 0, 528, 528, 528, 1341, 528, 528, 528, 528, 4014, 528, 528, 528, 528, 4019, 528, 528, 528, 528, 4024, 528, 528, 57893, 57893, 61373, 57893, 57893, 57893, 57893, 61378, 57893, 57893, 57893, 57893, 61383, 57893, 57893, 57893, 57916, 57916, 61389, 57916, 57916, 57916, 57916, 61394, 57916, 57916, 57916, 57916, 61399, 57916, 57916, 57916, 57943, 57943, 61405, 57943, 57943, 57943, 57943, 61410, 57943, 57943, 57943, 57943, 61415, 57943, 57943, 57943, 0, 0, 0, 0, 0, 2366, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 352256, 0, 352256, 0, 0, 0, 0, 0, 0, 4078, 0, 0, 4081, 0, 0, 0, 0, 0, 528, 528, 528, 528, 528, 528, 57893, 57893, 57893, 57893, 57893, 57893, 57916, 528, 528, 4246, 4247, 528, 528, 57893, 57893, 57893, 61596, 61597, 57893, 57893, 57893, 57916, 57916, 57916, 57916, 57943, 60410, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 58481, 528, 852, 528, 528, 57916, 61602, 61603, 57916, 57916, 57916, 57943, 57943, 57943, 61608, 61609, 57943, 57943, 57943, 0, 0, 0, 0, 306, 306, 306, 306, 306, 306, 306, 306, 306, 306, 306, 306, 306, 0, 306, 0, 0, 0, 0, 345, 346, 347, 348, 349, 0, 0, 0, 0, 0, 0, 0, 0, 1181, 0, 0, 0, 0, 0, 0, 0, 0, 0, 349, 348, 131072, 347, 348, 348, 0, 349, 347, 348, 0, 347, 348, 457, 464, 481, 481, 481, 492, 492, 492, 498, 492, 492, 498, 498, 492, 498, 513, 513, 513, 513, 513, 513, 513, 513, 513, 513, 513, 513, 513, 513, 513, 513, 541, 57907, 541, 57907, 541, 541, 57907, 541, 541, 57930, 57907, 541, 541, 57907, 57907, 57907, 57907, 57957, 57957, 57907, 57907, 57907, 57907, 57957, 57957, 57907, 541, 57907, 57907, 57930, 57907, 57907, 57907, 57907, 57907, 57907, 57907, 57930, 57930, 57907, 57907, 57957, 57907, 57907, 57907, 1, 24578, 3, 155942, 155942, 296, 0, 0, 0, 0, 0, 302, 303, 0, 0, 306, 307, 0, 0, 0, 0, 0, 0, 646, 0, 0, 0, 0, 651, 652, 653, 654, 655, 656, 657, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 2388, 0, 0, 0, 0, 0, 0, 0, 672, 673, 0, 675, 676, 0, 0, 0, 0, 0, 682, 0, 0, 0, 0, 307, 0, 0, 0, 0, 0, 0, 307, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 352256, 0, 0, 0, 0, 0, 0, 0, 716, 0, 0, 719, 0, 721, 0, 723, 0, 0, 0, 0, 0, 0, 0, 394, 0, 0, 0, 0, 0, 0, 0, 0, 1222, 0, 0, 0, 0, 0, 0, 0, 0, 0, 733, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 3172, 0, 0, 0, 0, 0, 763, 764, 0, 766, 767, 675, 0, 0, 0, 0, 0, 0, 0, 395, 0, 0, 0, 0, 0, 0, 0, 0, 2319, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 782, 783, 0, 785, 0, 0, 0, 0, 764, 0, 0, 791, 0, 0, 0, 0, 131072, 0, 0, 0, 0, 0, 245760, 0, 0, 0, 0, 245760, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 3171, 0, 0, 0, 0, 692, 0, 0, 0, 764, 0, 0, 0, 0, 0, 0, 0, 0, 757, 821, 0, 0, 0, 757, 0, 0, 824, 699, 0, 0, 0, 829, 0, 0, 0, 831, 0, 0, 0, 692, 699, 0, 0, 692, 829, 829, 0, 0, 0, 0, 0, 528, 528, 528, 528, 528, 528, 2874, 528, 528, 528, 528, 528, 528, 528, 3489, 528, 528, 528, 528, 528, 528, 528, 528, 3256, 528, 528, 528, 528, 528, 528, 528, 528, 849, 853, 856, 528, 865, 528, 528, 528, 881, 883, 886, 528, 528, 528, 899, 903, 528, 528, 528, 57893, 57893, 57893, 58268, 58272, 58275, 57893, 58284, 57893, 57893, 57893, 58300, 58302, 58305, 57893, 57893, 57893, 58318, 58322, 57893, 57893, 57893, 0, 57916, 57916, 57916, 58344, 58348, 58351, 57916, 58360, 57916, 57916, 57916, 58376, 58378, 58381, 57916, 57916, 57916, 58394, 58398, 57916, 57916, 57916, 57916, 58367, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 59999, 57916, 57916, 57916, 57916, 0, 0, 0, 0, 58409, 57943, 57943, 57943, 58419, 58423, 58426, 57943, 58435, 57943, 57943, 528, 3649, 528, 57893, 60996, 57893, 0, 0, 0, 0, 0, 0, 0, 0, 1254, 0, 0, 0, 0, 0, 0, 0, 57943, 58451, 58453, 58456, 57943, 57943, 57943, 58469, 58473, 57943, 57943, 57943, 849, 1140, 528, 899, 903, 1144, 0, 58489, 58268, 57893, 58318, 58322, 58494, 155942, 1151, 0, 0, 0, 0, 0, 0, 1179, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1239, 0, 0, 0, 0, 0, 0, 1278, 0, 0, 0, 0, 1283, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 3675, 0, 3828, 0, 0, 0, 0, 0, 0, 1321, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1198, 0, 0, 0, 528, 1375, 528, 528, 528, 1378, 528, 528, 528, 528, 528, 528, 528, 1385, 528, 528, 0, 2932, 0, 0, 0, 0, 57893, 57893, 57893, 57893, 57893, 57893, 60282, 57893, 1389, 528, 528, 528, 528, 528, 528, 528, 528, 528, 528, 528, 1400, 528, 528, 1407, 58817, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 58824, 57893, 57893, 58828, 57893, 57893, 57893, 57893, 57893, 59895, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 59898, 59899, 57893, 57893, 57893, 57893, 57916, 57916, 57916, 58934, 57916, 57916, 58941, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 59984, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 58955, 57916, 57916, 57916, 57916, 57916, 0, 57893, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 61542, 57943, 57943, 57943, 0, 4202, 0, 0, 57943, 57943, 57943, 59003, 57943, 57943, 57943, 59006, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 59013, 57943, 57943, 59017, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 59029, 57943, 57943, 57943, 57943, 57943, 61337, 57943, 57943, 57943, 61341, 57943, 528, 57893, 0, 0, 0, 0, 3139, 0, 0, 0, 0, 0, 0, 0, 0, 0, 3149, 57943, 59036, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 59050, 57943, 57943, 57943, 57943, 57943, 59019, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 58482, 528, 528, 528, 528, 1822, 0, 1824, 0, 0, 0, 1826, 0, 746, 0, 0, 0, 0, 0, 0, 0, 399, 0, 0, 404, 0, 0, 0, 0, 0, 1872, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 2350, 0, 1757, 0, 0, 0, 0, 0, 0, 528, 1891, 528, 528, 528, 528, 528, 528, 2440, 528, 528, 528, 528, 528, 528, 528, 528, 528, 528, 2495, 528, 528, 528, 528, 0, 1897, 528, 528, 528, 528, 528, 528, 528, 528, 528, 528, 528, 528, 528, 528, 528, 1428, 528, 528, 528, 528, 1919, 528, 528, 528, 528, 528, 528, 528, 528, 528, 528, 1928, 528, 528, 528, 1932, 528, 528, 1937, 528, 528, 528, 528, 528, 528, 528, 528, 528, 1419, 528, 528, 528, 528, 528, 528, 528, 528, 528, 528, 58773, 0, 57893, 59333, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 59340, 57893, 57893, 57893, 59362, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 59371, 57893, 57893, 57893, 57893, 57943, 57943, 57893, 57893, 57893, 57893, 57943, 57943, 57893, 528, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 61075, 57893, 57893, 57893, 57893, 57893, 58821, 57893, 57893, 57893, 58825, 57893, 57893, 57893, 57893, 57893, 57893, 59348, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 60875, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 59375, 57893, 57893, 59380, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 61457, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 59462, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 59471, 57916, 57916, 57916, 57916, 58368, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 60011, 57916, 57916, 57916, 57916, 59475, 57916, 57916, 59480, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 59972, 57916, 57916, 57916, 59975, 59558, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 59567, 57943, 57943, 57943, 59571, 57943, 57943, 59576, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 59569, 57943, 57943, 57916, 57916, 57916, 59990, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 58948, 57916, 57916, 57916, 57943, 57943, 57943, 60073, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 59010, 57943, 57943, 57943, 2804, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 2376, 57916, 60409, 57916, 57916, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 59030, 57943, 57943, 57943, 57943, 60474, 57943, 57943, 528, 528, 528, 528, 528, 57893, 57893, 57893, 57893, 57893, 2290, 0, 2291, 0, 0, 3214, 0, 0, 0, 528, 3218, 528, 528, 528, 528, 528, 528, 528, 528, 528, 528, 1383, 528, 528, 528, 528, 1388, 3242, 528, 528, 528, 528, 528, 528, 528, 528, 528, 528, 528, 528, 528, 528, 528, 1962, 57893, 60633, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 60335, 57893, 57916, 57916, 57916, 57916, 60682, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 59998, 57916, 57916, 57916, 57916, 0, 0, 3419, 0, 0, 0, 0, 0, 0, 0, 0, 0, 3425, 0, 0, 0, 0, 0, 2410, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 417792, 309, 310, 0, 0, 0, 0, 3841, 528, 528, 528, 528, 528, 528, 528, 3846, 528, 528, 528, 528, 528, 528, 2465, 528, 528, 528, 528, 528, 528, 528, 528, 528, 528, 3943, 528, 3945, 528, 528, 528, 57893, 57893, 57893, 57893, 61208, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 59369, 57893, 57893, 57893, 57893, 57916, 61227, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 59524, 57916, 0, 0, 3675, 3927, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1266, 0, 0, 0, 528, 528, 528, 3937, 528, 528, 528, 528, 3941, 528, 528, 528, 528, 528, 3947, 528, 528, 0, 57893, 57893, 57893, 58320, 57893, 57893, 155942, 1151, 0, 0, 0, 0, 0, 0, 1751, 0, 0, 0, 0, 0, 0, 0, 0, 0, 754, 0, 0, 0, 0, 0, 0, 528, 528, 57893, 57893, 57893, 61296, 57893, 57893, 57893, 57893, 61300, 57893, 57893, 57893, 57893, 57893, 58317, 57893, 57893, 57893, 57893, 0, 57916, 57916, 58336, 57916, 57916, 61306, 57893, 57893, 57893, 57893, 57916, 57916, 57916, 61312, 57916, 57916, 57916, 57916, 61316, 57916, 57916, 57916, 57916, 58371, 58373, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 58945, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 61322, 57916, 57916, 57916, 57916, 57943, 57943, 57943, 61328, 57943, 57943, 57943, 57943, 57943, 57943, 60981, 57943, 60983, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 60078, 57943, 57943, 57943, 57943, 60083, 57943, 57943, 61332, 57943, 57943, 57943, 57943, 57943, 61338, 57943, 57943, 57943, 57943, 528, 57893, 0, 0, 0, 0, 0, 2797, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1881, 0, 0, 0, 0, 0, 528, 528, 528, 4017, 528, 528, 528, 4021, 528, 528, 528, 528, 528, 57893, 57893, 57893, 57893, 57893, 57893, 58281, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 61376, 57893, 57893, 57893, 61380, 57893, 57893, 57893, 57893, 57893, 57893, 57916, 57916, 57916, 57916, 57916, 60939, 57916, 60941, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 58881, 57916, 57916, 57916, 57916, 57916, 57916, 58896, 57916, 57916, 61392, 57916, 57916, 57916, 61396, 57916, 57916, 57916, 57916, 57916, 57916, 57943, 57943, 57943, 61242, 57943, 57943, 57943, 61245, 57943, 57943, 57943, 57943, 61408, 57943, 57943, 57943, 61412, 57943, 57943, 57943, 57943, 57943, 57943, 0, 0, 0, 0, 0, 2821, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1739, 0, 0, 0, 0, 0, 0, 4270, 0, 528, 528, 528, 528, 528, 528, 57893, 57893, 57893, 57893, 57893, 57893, 57916, 61311, 57916, 57916, 61313, 57916, 57916, 57916, 57916, 57916, 57916, 59497, 57916, 57916, 59500, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 61526, 57916, 61528, 57916, 57916, 57916, 57916, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 60415, 57943, 57943, 57943, 57943, 0, 528, 528, 57893, 57893, 57916, 57916, 57943, 57943, 0, 528, 57893, 57916, 57943, 4308, 4309, 61654, 61655, 61656, 0, 528, 57893, 57916, 57943, 0, 528, 57893, 57916, 57943, 0, 0, 0, 0, 0, 2831, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1257, 0, 0, 0, 0, 1262, 0, 350, 0, 408, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 408, 0, 350, 0, 139264, 147456, 0, 0, 0, 0, 0, 514, 514, 514, 514, 514, 514, 514, 514, 514, 514, 514, 514, 514, 514, 514, 514, 542, 57908, 542, 57908, 542, 542, 57908, 542, 542, 57931, 57908, 542, 542, 57908, 57908, 57908, 57908, 57958, 57958, 57908, 57908, 57908, 57908, 57958, 57958, 57908, 623, 57908, 57974, 57931, 57908, 57908, 57908, 57908, 57908, 57908, 57908, 57931, 57931, 57908, 57908, 57958, 57908, 57908, 57908, 57974, 57974, 1, 24578, 3, 155942, 155942, 296, 0, 0, 0, 0, 0, 302, 303, 0, 0, 0, 0, 131072, 0, 0, 0, 328, 0, 0, 330, 0, 0, 0, 0, 0, 0, 2847, 2848, 0, 0, 0, 0, 0, 0, 0, 0, 0, 5808128, 0, 0, 0, 0, 4792320, 4833280, 844, 528, 528, 528, 528, 528, 528, 528, 528, 528, 528, 528, 890, 528, 528, 528, 528, 528, 3499, 528, 528, 528, 528, 528, 528, 528, 528, 528, 3508, 528, 908, 528, 528, 57893, 57893, 58262, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 61222, 57916, 57916, 57916, 57916, 57916, 57916, 57893, 57893, 58309, 57893, 57893, 57893, 57893, 58327, 57893, 57893, 0, 57916, 57916, 58338, 57916, 57916, 57916, 57916, 58877, 57916, 57916, 57916, 57916, 58883, 57916, 57916, 57916, 57916, 57916, 57916, 59993, 59994, 57916, 57916, 57916, 57916, 57916, 57916, 60001, 57916, 57943, 57943, 57943, 57943, 58460, 57943, 57943, 57943, 57943, 58478, 57943, 57943, 528, 528, 528, 528, 528, 528, 57893, 57893, 57893, 57893, 57893, 57893, 1151, 1724, 1173, 0, 0, 0, 0, 1178, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 2310144, 0, 368, 0, 0, 0, 0, 0, 1173, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 2816, 528, 528, 528, 528, 1354, 528, 528, 528, 528, 1364, 528, 528, 528, 528, 528, 528, 2491, 528, 2493, 528, 528, 528, 528, 528, 528, 0, 58773, 914, 57893, 57893, 57893, 58779, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 58793, 57916, 58874, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 58888, 57916, 57916, 57916, 57916, 57916, 59448, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 59459, 58898, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 59961, 57916, 58914, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 59987, 59988, 58953, 57916, 57916, 57916, 57916, 57916, 57916, 0, 57893, 57943, 57943, 57943, 58968, 57943, 57943, 57943, 57943, 57943, 59041, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 59053, 59054, 0, 1728, 0, 0, 0, 0, 1735, 0, 0, 0, 0, 0, 0, 0, 0, 0, 3424, 0, 0, 0, 0, 0, 0, 0, 0, 1761, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 3185, 0, 0, 1775, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 2840, 0, 0, 2353, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 3201, 0, 0, 2449, 528, 528, 528, 528, 528, 528, 2454, 528, 2456, 528, 528, 528, 528, 528, 528, 2884, 528, 528, 528, 528, 528, 528, 528, 528, 528, 528, 2915, 528, 528, 528, 528, 528, 2502, 0, 0, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 61387, 57916, 57916, 57893, 57893, 57893, 59880, 57893, 57893, 57893, 57893, 57893, 57893, 59885, 57893, 59887, 57893, 57893, 57893, 0, 0, 2592, 0, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 61470, 57943, 57943, 57943, 57943, 57943, 57943, 59976, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 60013, 57916, 60059, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 59033, 528, 2906, 528, 528, 528, 528, 528, 528, 528, 528, 528, 528, 528, 528, 528, 528, 1960, 528, 57916, 57916, 60383, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 60367, 57916, 57916, 0, 0, 3675, 0, 0, 0, 0, 3931, 0, 0, 3933, 0, 0, 0, 0, 0, 0, 1192, 0, 0, 0, 1196, 1197, 0, 0, 0, 0, 528, 3936, 528, 528, 3938, 528, 528, 528, 528, 528, 528, 528, 528, 528, 528, 528, 528, 3247, 528, 528, 528, 528, 528, 57893, 61295, 57893, 57893, 61297, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 61456, 57916, 57916, 57916, 57916, 57916, 57916, 0, 0, 800, 0, 800, 0, 0, 0, 0, 0, 0, 0, 0, 800, 0, 0, 0, 0, 307, 307, 307, 307, 307, 307, 307, 307, 307, 307, 307, 307, 307, 0, 307, 0, 528, 528, 528, 528, 860, 528, 528, 528, 528, 528, 528, 528, 528, 528, 528, 528, 528, 3257, 528, 528, 3260, 57916, 58355, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 60368, 57916, 1161, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 3174, 528, 528, 528, 528, 58773, 1987, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 59910, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57943, 57943, 60445, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 59585, 57943, 57943, 57893, 61081, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 60639, 57893, 0, 0, 0, 351, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1746, 0, 0, 0, 0, 0, 0, 352, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1756, 1757, 0, 0, 0, 0, 353, 351, 131072, 0, 351, 351, 0, 353, 0, 351, 0, 0, 351, 353, 351, 0, 0, 0, 351, 351, 351, 351, 351, 351, 351, 351, 505, 351, 351, 351, 351, 351, 351, 351, 351, 351, 351, 351, 351, 351, 351, 351, 351, 543, 57909, 543, 57909, 543, 543, 57909, 543, 543, 57932, 57909, 543, 543, 57909, 57909, 57909, 57909, 57959, 57959, 57909, 57909, 57909, 57909, 57959, 57959, 57909, 543, 57909, 57909, 57932, 57909, 57909, 57909, 57909, 57909, 57909, 57909, 57932, 57932, 57909, 57909, 57959, 57909, 57909, 57909, 1, 24578, 3, 155942, 155942, 296, 0, 0, 0, 0, 0, 302, 303, 0, 811, 812, 0, 0, 0, 0, 0, 0, 758, 0, 0, 817, 0, 0, 0, 0, 0, 0, 4082, 0, 0, 0, 0, 528, 528, 528, 528, 528, 3221, 528, 3222, 528, 528, 528, 528, 0, 817, 0, 664, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1787, 0, 0, 0, 832, 0, 0, 0, 0, 0, 0, 812, 0, 0, 0, 0, 0, 0, 0, 528, 528, 528, 528, 528, 2873, 528, 528, 528, 528, 2877, 528, 528, 528, 0, 2503, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 61087, 57893, 57893, 57893, 57893, 528, 850, 528, 528, 528, 866, 870, 528, 528, 528, 528, 528, 891, 893, 528, 528, 0, 2932, 0, 0, 0, 0, 57893, 57893, 57893, 60280, 57893, 60281, 57893, 57893, 57893, 50676, 0, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 59473, 904, 528, 528, 528, 57893, 57893, 58263, 58269, 57893, 57893, 57893, 58285, 58289, 57893, 57893, 57893, 0, 2590, 0, 0, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 59970, 57916, 57916, 57916, 57916, 57916, 57916, 57893, 57893, 58310, 58312, 57893, 57893, 58323, 57893, 57893, 57893, 0, 57916, 57916, 58339, 58345, 57916, 0, 0, 0, 0, 57893, 57943, 57943, 58412, 57943, 57943, 57943, 58429, 57943, 57943, 58445, 57916, 57916, 58361, 58365, 57916, 57916, 57916, 57916, 57916, 58386, 58388, 57916, 57916, 58399, 57916, 57916, 57916, 57916, 58904, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 60945, 57916, 57916, 57916, 57916, 57943, 57943, 57943, 57943, 58461, 58463, 57943, 57943, 58474, 57943, 57943, 57943, 850, 528, 893, 528, 528, 0, 58270, 58266, 58491, 58314, 57893, 57893, 155942, 1151, 0, 0, 1154, 0, 0, 0, 0, 0, 245760, 245760, 245760, 245760, 245760, 245760, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 4358144, 4358144, 4358144, 904, 528, 0, 57893, 58269, 58312, 57893, 58323, 57893, 155942, 1151, 0, 302, 0, 0, 306, 307, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1772, 0, 0, 0, 0, 0, 1217, 0, 1219, 0, 1221, 0, 0, 0, 0, 0, 0, 0, 0, 1286, 0, 0, 0, 0, 0, 0, 0, 0, 1247, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 3839, 0, 1332, 0, 0, 0, 0, 0, 0, 0, 528, 528, 528, 528, 528, 528, 528, 528, 528, 528, 528, 528, 3850, 528, 528, 1374, 528, 528, 528, 528, 528, 528, 528, 528, 528, 528, 528, 528, 528, 528, 528, 2461, 528, 528, 528, 1412, 528, 528, 528, 528, 528, 528, 528, 528, 1424, 1425, 528, 528, 0, 57893, 57893, 57893, 57893, 57893, 57893, 294, 1151, 0, 0, 0, 0, 0, 0, 0, 5480448, 4358144, 4358144, 4358144, 4358144, 4857856, 4874240, 4358144, 4358144, 58773, 914, 57893, 57893, 57893, 57893, 58780, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 60289, 57893, 57893, 57893, 57893, 57893, 60294, 57893, 57916, 57916, 58875, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 60378, 57916, 57916, 57916, 57916, 58932, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 58946, 57916, 57916, 57916, 57916, 57916, 57916, 61241, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 61246, 57943, 57916, 57916, 57916, 58958, 58959, 57916, 57916, 0, 57893, 57943, 57943, 57943, 57943, 58969, 57943, 57943, 3648, 528, 528, 60995, 57893, 57893, 0, 0, 0, 0, 0, 0, 0, 0, 1815, 0, 0, 0, 0, 0, 0, 0, 57943, 59000, 59002, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 59602, 57943, 57943, 0, 1808, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 212992, 0, 528, 528, 1966, 528, 528, 528, 528, 528, 528, 528, 528, 528, 1975, 528, 528, 528, 528, 528, 3513, 3515, 528, 528, 528, 3518, 528, 3520, 3521, 528, 57893, 528, 1981, 528, 528, 58773, 0, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 59338, 57893, 57893, 57893, 50676, 2087, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 60674, 57916, 57916, 57916, 57916, 57893, 59409, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 59419, 57893, 57893, 57893, 57893, 57893, 59907, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 0, 57916, 57916, 58340, 57916, 57916, 59425, 57893, 57893, 50676, 0, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 59438, 57916, 57916, 57916, 57916, 57916, 61116, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 61124, 57916, 59509, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 59519, 57916, 57916, 57916, 57916, 59525, 57943, 57943, 59559, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 59620, 57943, 57943, 2393, 0, 2395, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 2403, 0, 0, 0, 0, 343, 390, 0, 0, 0, 139264, 147456, 0, 0, 0, 423, 0, 0, 0, 2408, 2409, 0, 0, 0, 0, 0, 2413, 0, 0, 0, 0, 0, 0, 0, 3153, 0, 0, 0, 0, 0, 0, 0, 0, 378, 0, 0, 0, 0, 0, 0, 0, 0, 2419, 528, 528, 528, 528, 528, 528, 2426, 528, 528, 528, 528, 528, 528, 528, 1394, 528, 528, 528, 528, 528, 528, 528, 528, 2886, 528, 528, 528, 528, 528, 528, 528, 528, 528, 2450, 528, 528, 528, 528, 528, 528, 528, 528, 2457, 528, 2459, 528, 2462, 528, 528, 528, 2476, 528, 528, 528, 528, 528, 528, 528, 528, 528, 528, 528, 528, 528, 61371, 57893, 57893, 0, 0, 0, 59849, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 59857, 57893, 57893, 57893, 57893, 57893, 60301, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 60306, 57893, 57893, 59890, 57893, 59893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 60304, 57893, 57893, 57893, 57893, 59902, 57893, 57893, 57893, 57893, 57893, 59908, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 60330, 57893, 57893, 57893, 60333, 60334, 57893, 57893, 57916, 57916, 59977, 57916, 59979, 57916, 59982, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 60363, 57916, 60365, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 59991, 57916, 57916, 57916, 57916, 57916, 59997, 57916, 57916, 57916, 57916, 57916, 57916, 61606, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 0, 0, 57943, 57943, 60060, 57943, 60062, 57943, 60065, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 59007, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 60074, 57943, 57943, 57943, 57943, 57943, 60080, 57943, 57943, 57943, 57943, 57943, 57943, 59612, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 0, 0, 4240, 0, 0, 0, 528, 0, 0, 0, 2819, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 2827, 0, 0, 0, 0, 131072, 0, 0, 0, 357, 0, 0, 0, 0, 359, 0, 0, 400, 0, 0, 0, 0, 0, 139264, 147456, 400, 411, 0, 424, 411, 2841, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 3187, 528, 2880, 528, 528, 528, 528, 528, 2885, 528, 528, 528, 528, 528, 528, 528, 528, 3502, 528, 528, 528, 528, 528, 528, 528, 57893, 57893, 57893, 60310, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 60628, 57893, 57893, 57893, 57893, 57916, 60356, 57916, 57916, 57916, 57916, 57916, 60361, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 59953, 57916, 59955, 57916, 57916, 57916, 57916, 57916, 57916, 57943, 57943, 57943, 57943, 60421, 57943, 57943, 57943, 57943, 57943, 60426, 57943, 57943, 57943, 57943, 57943, 57943, 60064, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 59566, 57943, 57943, 57943, 57943, 57943, 57893, 60652, 57893, 57893, 57893, 57893, 0, 0, 57916, 57916, 57916, 57916, 57916, 60661, 57916, 60662, 57916, 57916, 57916, 57916, 60701, 57916, 57916, 57916, 57916, 57943, 57943, 57943, 57943, 57943, 60710, 57943, 57943, 57943, 57943, 57943, 61481, 57943, 57943, 61483, 57943, 0, 0, 0, 0, 0, 0, 0, 2808, 0, 0, 0, 0, 0, 0, 0, 0, 350, 0, 0, 0, 0, 0, 0, 0, 60711, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 59587, 528, 3483, 528, 528, 528, 528, 528, 528, 528, 528, 528, 528, 528, 528, 528, 528, 1978, 528, 60867, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 59929, 57916, 60909, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 60395, 57916, 57916, 57943, 60951, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 60069, 57943, 57943, 0, 3670, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 278528, 0, 0, 0, 3675, 0, 0, 0, 3930, 0, 0, 0, 0, 0, 0, 0, 0, 0, 3476, 0, 0, 0, 0, 0, 0, 3935, 528, 528, 528, 528, 3939, 528, 528, 528, 528, 528, 528, 528, 528, 528, 3948, 528, 528, 61294, 57893, 57893, 57893, 57893, 61298, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 60342, 57893, 57893, 0, 0, 0, 3000, 0, 57893, 61307, 57893, 57893, 57893, 61310, 57916, 57916, 57916, 57916, 61314, 57916, 57916, 57916, 57916, 57916, 57943, 57943, 57943, 57943, 57943, 57943, 0, 0, 0, 0, 528, 528, 528, 3219, 528, 528, 528, 528, 528, 3224, 528, 528, 57916, 57916, 57916, 57916, 61323, 57916, 57916, 57916, 61326, 57943, 57943, 57943, 57943, 61330, 57943, 57943, 57943, 57943, 57943, 60089, 60090, 60091, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 59615, 57943, 57943, 57943, 57943, 59621, 57943, 4090, 528, 528, 528, 528, 528, 528, 528, 528, 528, 57893, 57893, 57893, 57893, 57893, 61447, 0, 0, 0, 4149, 0, 528, 528, 528, 528, 528, 528, 528, 528, 528, 528, 528, 528, 528, 3851, 528, 528, 528, 528, 528, 4248, 528, 57893, 57893, 57893, 57893, 57893, 61598, 57893, 57893, 57916, 57916, 57916, 57916, 58919, 57916, 57916, 58923, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 61572, 61573, 57916, 57916, 57943, 57943, 57943, 57943, 57916, 57916, 57916, 61604, 57916, 57916, 57943, 57943, 57943, 57943, 57943, 61610, 57943, 57943, 0, 0, 0, 0, 363, 0, 0, 0, 0, 139264, 147456, 0, 0, 0, 0, 0, 0, 1307, 0, 528, 528, 1339, 528, 1342, 528, 528, 528, 4269, 0, 4271, 528, 528, 528, 528, 528, 528, 57893, 57893, 57893, 57893, 57893, 57893, 57916, 0, 0, 0, 0, 57893, 57943, 57943, 58413, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 61582, 0, 0, 0, 0, 0, 0, 528, 388, 0, 354, 0, 0, 0, 0, 0, 397, 398, 0, 399, 0, 0, 0, 0, 0, 0, 4208, 528, 528, 528, 528, 528, 528, 528, 528, 528, 1380, 528, 528, 528, 528, 528, 528, 371, 379, 407, 0, 0, 0, 371, 0, 0, 354, 0, 0, 0, 371, 0, 410, 412, 0, 371, 399, 0, 0, 371, 379, 0, 139264, 147456, 399, 410, 0, 0, 410, 0, 0, 0, 433, 131072, 0, 433, 433, 443, 0, 0, 433, 0, 412, 433, 0, 0, 0, 0, 131072, 0, 0, 0, 438, 0, 0, 0, 0, 0, 311, 0, 0, 0, 0, 2796, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 450560, 450560, 0, 0, 0, 465, 0, 0, 0, 493, 493, 493, 493, 493, 493, 493, 493, 493, 493, 515, 515, 515, 515, 515, 527, 515, 515, 515, 527, 515, 515, 515, 515, 515, 515, 544, 57910, 544, 57910, 544, 544, 57910, 544, 544, 57933, 57910, 544, 544, 57910, 57910, 57910, 57910, 57960, 57960, 57910, 57910, 57910, 57910, 57960, 57960, 57910, 624, 57910, 57975, 57933, 57910, 57910, 57910, 57910, 57910, 57910, 57910, 57933, 57933, 57910, 57910, 57960, 57910, 57910, 57910, 57975, 57975, 1, 24578, 3, 155942, 155942, 296, 0, 0, 0, 0, 0, 302, 303, 0, 0, 0, 0, 131072, 0, 0, 0, 441, 342, 0, 0, 0, 453, 0, 342, 343, 344, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 2853, 0, 0, 0, 306, 307, 0, 0, 0, 643, 0, 0, 0, 0, 0, 0, 0, 0, 0, 3687, 0, 0, 0, 0, 0, 0, 669, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 3428, 845, 528, 528, 528, 528, 528, 528, 876, 878, 528, 528, 528, 528, 528, 528, 528, 1415, 528, 528, 528, 528, 528, 528, 1426, 528, 528, 528, 528, 528, 57893, 57893, 58264, 57893, 57893, 57893, 57893, 57893, 57893, 58295, 58297, 57893, 57893, 57893, 57893, 58803, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 59384, 57893, 57893, 57893, 57893, 58448, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 528, 528, 528, 528, 528, 57893, 57893, 57893, 57893, 57893, 2290, 1725, 2291, 0, 0, 0, 0, 0, 1309, 1335, 1336, 0, 1309, 528, 528, 528, 528, 528, 528, 1344, 528, 528, 528, 528, 528, 3706, 528, 528, 528, 528, 528, 528, 528, 528, 528, 528, 528, 1974, 528, 528, 528, 528, 528, 528, 1348, 528, 528, 1359, 528, 528, 1363, 528, 528, 528, 528, 528, 528, 528, 1416, 528, 528, 528, 528, 528, 528, 528, 528, 3709, 528, 528, 528, 528, 528, 528, 528, 58773, 914, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 58783, 57893, 57893, 57893, 58787, 57893, 57893, 57893, 57893, 57893, 60623, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 58845, 57893, 57893, 57893, 57893, 57893, 58798, 57893, 57893, 58802, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 60638, 57893, 57893, 57893, 57893, 57916, 58916, 57916, 57916, 57916, 58920, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 59469, 57916, 57916, 57916, 57916, 57916, 58972, 57943, 57943, 57943, 58976, 57943, 57943, 58987, 57943, 57943, 58991, 57943, 57943, 57943, 57943, 57943, 57943, 60076, 60077, 57943, 57943, 57943, 57943, 57943, 57943, 60084, 57943, 59014, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 59604, 1745, 1746, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 507904, 0, 0, 0, 0, 1778, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1828, 0, 0, 0, 1791, 0, 1793, 0, 0, 0, 1797, 0, 0, 0, 1801, 0, 0, 0, 1805, 0, 0, 0, 0, 131072, 0, 0, 0, 442, 0, 0, 0, 449, 0, 0, 0, 0, 0, 2846, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1224, 0, 0, 0, 0, 0, 1980, 528, 528, 528, 58773, 0, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 60625, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57943, 57943, 57943, 57943, 59545, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 60068, 57943, 57943, 57943, 0, 0, 2314, 0, 2315, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 262738, 0, 0, 0, 0, 2377, 0, 0, 0, 0, 0, 0, 2384, 0, 0, 0, 0, 0, 0, 0, 0, 1326, 1327, 0, 0, 1167, 0, 0, 0, 0, 528, 528, 2421, 528, 528, 528, 528, 528, 528, 528, 528, 528, 528, 528, 528, 888, 528, 528, 528, 528, 528, 528, 528, 2489, 528, 528, 528, 528, 528, 528, 528, 528, 528, 528, 0, 0, 57893, 57893, 57893, 57893, 60611, 57893, 57893, 57893, 57893, 57893, 57893, 59868, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 58330, 0, 57916, 57916, 57916, 57916, 58347, 0, 0, 0, 57893, 57893, 59851, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 60901, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 60113, 0, 0, 1729, 0, 0, 1736, 0, 0, 0, 0, 0, 0, 0, 0, 0, 139264, 147456, 0, 0, 0, 0, 0, 0, 0, 2858, 0, 0, 0, 0, 0, 0, 0, 0, 0, 2864, 0, 0, 0, 0, 0, 3152, 0, 0, 0, 0, 0, 0, 0, 0, 0, 3158, 57893, 57893, 57893, 57893, 60311, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 60890, 57893, 60892, 57893, 57893, 60323, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 60295, 60443, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 60455, 57943, 57943, 57943, 57943, 57943, 59593, 57943, 57943, 59596, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 59551, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 59565, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 59580, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 59614, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 60037, 57943, 57943, 60041, 57943, 57943, 57943, 60045, 0, 0, 3190, 0, 0, 0, 0, 3195, 0, 0, 0, 0, 0, 0, 0, 0, 1783, 1784, 1785, 0, 0, 0, 0, 0, 3250, 528, 528, 528, 528, 528, 528, 528, 528, 528, 528, 528, 528, 528, 528, 528, 2472, 57893, 57893, 60642, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 61385, 57893, 57916, 57916, 57916, 0, 3468, 3469, 0, 0, 0, 0, 3474, 0, 0, 0, 0, 0, 0, 0, 0, 1799, 0, 0, 0, 0, 0, 0, 0, 0, 0, 3681, 0, 0, 0, 3684, 0, 0, 0, 0, 0, 0, 0, 0, 3692, 57893, 61092, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57916, 57916, 57916, 57916, 57916, 57943, 57943, 57943, 57943, 57943, 57943, 0, 0, 0, 0, 4290, 528, 3813, 57893, 57893, 61159, 3816, 0, 0, 0, 0, 3820, 0, 0, 0, 0, 0, 0, 1236, 0, 0, 0, 0, 0, 0, 0, 0, 0, 2799, 0, 0, 0, 0, 0, 0, 0, 3833, 0, 0, 0, 0, 3837, 0, 0, 0, 0, 0, 0, 0, 0, 3840, 0, 528, 528, 528, 3842, 528, 528, 528, 3845, 528, 528, 528, 528, 528, 528, 528, 1938, 528, 528, 528, 528, 528, 528, 528, 528, 3235, 528, 528, 3238, 528, 528, 528, 528, 57893, 57893, 57893, 61207, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 61084, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 61226, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 60407, 57916, 528, 528, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 61302, 57893, 61304, 57893, 57893, 57893, 57893, 58819, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 59913, 57893, 57893, 57893, 57893, 57916, 61320, 57916, 57916, 57916, 57916, 57916, 57916, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 4268, 0, 57943, 57943, 61334, 57943, 61336, 57943, 57943, 57943, 57943, 57943, 57943, 528, 57893, 0, 0, 0, 0, 0, 3164, 0, 0, 0, 0, 3169, 0, 0, 0, 0, 0, 0, 1220, 0, 0, 0, 0, 0, 1226, 0, 1228, 0, 4146, 0, 0, 0, 0, 528, 528, 528, 528, 528, 528, 528, 528, 528, 528, 4160, 528, 528, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 61516, 57893, 57893, 57893, 0, 2591, 0, 0, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 59996, 57916, 57916, 57916, 57916, 57916, 57916, 528, 528, 57893, 57893, 61560, 57893, 61561, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57916, 0, 0, 0, 0, 57893, 57943, 57943, 58414, 58420, 57943, 57943, 57943, 58436, 58440, 57943, 57943, 57943, 57943, 57943, 61411, 57943, 61413, 61414, 57943, 61416, 57943, 61418, 0, 0, 0, 0, 0, 1849, 0, 0, 0, 0, 0, 0, 1748, 0, 0, 0, 0, 0, 1864, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 2401, 0, 0, 0, 0, 0, 57916, 61568, 57916, 61569, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57943, 57943, 61576, 57943, 57943, 57943, 57943, 57943, 61539, 57943, 61541, 57943, 57943, 57943, 57943, 0, 0, 0, 0, 0, 0, 3435, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1273, 0, 0, 0, 0, 0, 0, 61577, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 0, 0, 0, 0, 0, 0, 528, 528, 528, 2871, 2872, 528, 528, 528, 528, 528, 528, 528, 528, 528, 528, 3237, 528, 528, 528, 528, 528, 358, 359, 360, 0, 0, 0, 0, 0, 0, 0, 0, 0, 368, 0, 296, 0, 0, 0, 0, 131072, 0, 352256, 352256, 0, 0, 0, 352256, 0, 0, 352256, 0, 352256, 352256, 352256, 352256, 352256, 352256, 352256, 352256, 352256, 352256, 352256, 352256, 352256, 352256, 352256, 352256, 0, 482, 482, 482, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 516, 516, 520, 520, 520, 520, 520, 516, 520, 520, 520, 516, 520, 520, 520, 520, 520, 520, 545, 57911, 545, 57911, 545, 545, 57911, 545, 545, 57934, 57911, 545, 545, 57911, 57911, 57911, 57911, 57961, 57961, 57911, 57911, 57911, 57911, 57961, 57961, 57911, 625, 57911, 57976, 57934, 57911, 57911, 57911, 57911, 57911, 57911, 57911, 57934, 57934, 57911, 57911, 57961, 57911, 57911, 57911, 57976, 57976, 1, 24578, 3, 155942, 155942, 296, 0, 0, 0, 0, 0, 302, 303, 0, 0, 0, 0, 245760, 0, 0, 245760, 0, 0, 245760, 0, 0, 245760, 0, 245760, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 516567, 516567, 516567, 0, 306, 307, 0, 0, 0, 0, 644, 0, 0, 0, 0, 0, 0, 0, 0, 1837, 0, 0, 0, 0, 0, 0, 0, 0, 0, 687, 688, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 368, 368, 0, 0, 0, 760, 0, 0, 0, 0, 765, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 4841472, 0, 0, 0, 4898816, 0, 794, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 803, 0, 808, 0, 0, 0, 0, 392, 0, 0, 0, 396, 392, 0, 0, 0, 0, 0, 0, 0, 2823, 0, 0, 0, 0, 0, 0, 0, 0, 677, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 820, 0, 0, 0, 0, 0, 644, 0, 0, 0, 0, 0, 0, 0, 3436, 0, 0, 0, 0, 0, 0, 0, 0, 681, 0, 0, 0, 0, 0, 0, 680, 0, 0, 794, 0, 0, 0, 0, 0, 0, 0, 0, 644, 0, 0, 794, 528, 528, 528, 528, 528, 3717, 528, 528, 528, 528, 528, 528, 528, 528, 528, 528, 1381, 528, 528, 528, 528, 528, 846, 528, 528, 528, 528, 867, 871, 877, 528, 882, 528, 528, 528, 528, 528, 900, 528, 528, 528, 528, 57893, 57893, 58265, 57893, 57893, 57893, 57893, 58286, 58290, 58296, 57893, 58301, 57916, 57916, 58362, 58366, 58372, 57916, 58377, 57916, 57916, 57916, 57916, 57916, 58395, 57916, 57916, 57916, 57916, 57916, 61231, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 60686, 57916, 57916, 57916, 57916, 57916, 57943, 58452, 57943, 57943, 57943, 57943, 57943, 58470, 57943, 57943, 57943, 57943, 528, 528, 528, 900, 1215, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 3441, 1264, 0, 0, 0, 0, 0, 1269, 0, 0, 0, 0, 0, 1274, 0, 0, 0, 0, 0, 3179, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 679, 0, 0, 0, 0, 0, 0, 1294, 0, 0, 1165, 0, 0, 0, 1300, 0, 1302, 0, 0, 0, 0, 0, 0, 1284, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1301, 0, 0, 0, 0, 0, 0, 0, 1300, 1319, 0, 1322, 0, 0, 0, 1196, 0, 0, 0, 0, 1330, 0, 0, 1215, 1330, 1337, 528, 528, 528, 528, 1343, 528, 528, 528, 528, 1349, 528, 528, 528, 528, 528, 528, 528, 528, 528, 528, 1370, 528, 528, 0, 57893, 57893, 57893, 57893, 57893, 57893, 155942, 0, 0, 0, 0, 0, 0, 0, 1285, 0, 0, 0, 0, 1290, 0, 0, 0, 58773, 914, 58775, 57893, 57893, 57893, 57893, 57893, 58782, 57893, 57893, 57893, 57893, 58788, 57893, 57893, 57893, 57893, 57893, 60872, 57893, 57893, 57893, 57893, 57893, 60876, 57893, 57893, 57893, 57893, 57893, 58860, 57893, 57893, 57893, 57893, 57893, 50676, 58773, 990, 57916, 57916, 57916, 58917, 57916, 57916, 57916, 57916, 58922, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 60403, 57916, 57916, 57916, 60406, 57916, 57916, 58930, 57916, 57916, 57916, 58937, 58939, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 59957, 57916, 57916, 57916, 57916, 57916, 57943, 57943, 57943, 57943, 58977, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 58998, 57943, 59016, 57943, 57943, 57943, 57943, 57943, 59021, 57943, 57943, 57943, 59025, 57943, 57943, 57943, 59032, 59034, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 59622, 57943, 59057, 528, 528, 528, 528, 528, 528, 57893, 57893, 57893, 57893, 57893, 57893, 1151, 0, 0, 0, 0, 385024, 0, 4358144, 4358144, 4358144, 4358144, 4358144, 4358144, 4358144, 4358144, 4358144, 4358144, 4358144, 4358144, 4358144, 5259264, 4358144, 4358144, 4358144, 4358144, 0, 0, 1729, 0, 0, 0, 0, 1736, 0, 0, 0, 0, 0, 0, 0, 0, 1880, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1747, 1748, 0, 0, 0, 0, 0, 1754, 0, 0, 0, 0, 0, 0, 0, 3447, 0, 0, 0, 0, 0, 3450, 0, 3452, 0, 1776, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 5177344, 0, 0, 0, 0, 0, 1794, 0, 0, 0, 0, 0, 0, 0, 1802, 1803, 0, 0, 0, 0, 0, 3207, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 3183, 0, 0, 0, 0, 0, 0, 0, 0, 1810, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1841, 0, 1843, 0, 0, 0, 1874, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 3213, 0, 0, 528, 528, 528, 1901, 528, 528, 528, 528, 1907, 528, 528, 528, 528, 528, 528, 528, 2441, 528, 528, 528, 528, 528, 528, 528, 528, 3720, 528, 528, 528, 528, 528, 528, 528, 528, 528, 528, 1918, 528, 528, 528, 528, 1922, 528, 528, 528, 528, 528, 528, 528, 2466, 2467, 528, 528, 528, 528, 528, 528, 528, 1417, 528, 528, 528, 528, 528, 528, 528, 528, 3856, 528, 528, 528, 57893, 57893, 57893, 57893, 528, 528, 528, 528, 1968, 1969, 528, 528, 528, 528, 528, 528, 528, 528, 528, 528, 1924, 1925, 528, 528, 528, 528, 57893, 57893, 59344, 57893, 57893, 57893, 57893, 59350, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 61212, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 59361, 57893, 57893, 57893, 57893, 59365, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 59883, 57893, 57893, 57893, 59886, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 59378, 57893, 57893, 57893, 59383, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 59909, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 59915, 57893, 57893, 57893, 59412, 59413, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 59921, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 59351, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57916, 59444, 57916, 57916, 57916, 57916, 59450, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 60685, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 59461, 57916, 57916, 57916, 57916, 59465, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 60943, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 59478, 57916, 57916, 57916, 59483, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 60021, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 60029, 57916, 57916, 59512, 59513, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 59470, 57916, 57916, 57916, 57943, 57943, 57943, 57943, 59546, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 59557, 57943, 57943, 57943, 57943, 59561, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 60082, 57943, 57943, 57943, 57943, 59574, 57943, 57943, 57943, 59579, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 60735, 57943, 57943, 57943, 57943, 57943, 59608, 59609, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 60095, 57943, 57943, 2302, 2303, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 5767168, 0, 2339, 0, 0, 2341, 2342, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 335872, 0, 0, 0, 0, 528, 2434, 528, 528, 2438, 528, 528, 528, 2442, 528, 528, 528, 528, 528, 528, 528, 2912, 528, 528, 528, 528, 528, 528, 528, 528, 528, 3857, 528, 528, 57893, 57893, 57893, 57893, 528, 528, 528, 528, 2452, 528, 528, 528, 2455, 528, 528, 528, 528, 528, 528, 528, 2925, 528, 528, 528, 2928, 528, 528, 528, 528, 528, 2487, 528, 528, 528, 528, 528, 2492, 528, 528, 528, 528, 528, 528, 528, 0, 0, 57893, 57893, 57893, 60610, 57893, 57893, 57893, 57893, 57893, 60615, 57893, 0, 2503, 0, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 60288, 57893, 57893, 60291, 60292, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 59865, 57893, 57893, 59869, 57893, 57893, 57893, 59873, 57893, 57893, 57893, 57893, 57893, 60312, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 60320, 57893, 57893, 59916, 57893, 57893, 57893, 57893, 59920, 57893, 57893, 57893, 57893, 57893, 59925, 57893, 57893, 57893, 57893, 57893, 60328, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 58867, 50676, 58773, 990, 58870, 57916, 57916, 57916, 57916, 57916, 60005, 57916, 57916, 57916, 57916, 60009, 57916, 57916, 57916, 57916, 57916, 60014, 57943, 57943, 57943, 57943, 60088, 57943, 57943, 57943, 57943, 60092, 57943, 57943, 57943, 57943, 57943, 60097, 2856, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 2863, 0, 0, 0, 0, 0, 0, 212992, 0, 0, 0, 0, 0, 212992, 212992, 212992, 212992, 2893, 528, 528, 528, 528, 2896, 528, 528, 2898, 528, 2899, 528, 528, 528, 528, 528, 1356, 528, 1361, 528, 528, 1365, 528, 528, 1368, 528, 528, 528, 528, 528, 2907, 528, 528, 2911, 528, 528, 528, 528, 528, 528, 528, 528, 528, 2494, 528, 528, 2497, 2498, 528, 0, 2920, 528, 528, 528, 2923, 528, 528, 528, 528, 528, 528, 528, 528, 528, 528, 528, 528, 3506, 3507, 528, 528, 57893, 60308, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 60317, 57893, 57893, 60321, 57893, 57893, 57893, 57893, 58835, 57893, 57893, 57893, 58842, 58844, 57893, 57893, 57893, 57893, 57893, 57893, 59381, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 60902, 57893, 57893, 60905, 57893, 57893, 57893, 0, 57916, 57916, 60346, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 60354, 60369, 57916, 57916, 57916, 57916, 60372, 57916, 57916, 60374, 57916, 60375, 57916, 57916, 57916, 57916, 57916, 57943, 57943, 57943, 57943, 57943, 57943, 4288, 0, 0, 0, 528, 57916, 57916, 57916, 57916, 60384, 57916, 57916, 60388, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 60402, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 59995, 57916, 57916, 57916, 57916, 60000, 57916, 57916, 57916, 60397, 57916, 57916, 57916, 60400, 60401, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 60683, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 59452, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57943, 57943, 60419, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 60469, 57943, 57943, 57943, 57943, 57943, 60434, 57943, 57943, 57943, 57943, 60437, 57943, 57943, 60439, 57943, 60440, 57943, 57943, 57943, 57943, 57943, 60422, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 61155, 57943, 57943, 57943, 528, 57943, 57943, 57943, 57943, 60462, 57943, 57943, 57943, 60465, 60466, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 60752, 57943, 57943, 528, 528, 528, 57893, 57893, 57893, 0, 0, 0, 0, 0, 3658, 0, 0, 3228, 528, 528, 528, 528, 3233, 528, 528, 528, 3236, 528, 528, 528, 528, 528, 528, 3255, 528, 528, 528, 528, 528, 528, 528, 528, 528, 528, 2482, 528, 528, 528, 528, 528, 3261, 528, 528, 0, 0, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 60616, 57893, 60619, 57893, 57893, 57893, 57893, 60624, 57893, 57893, 57893, 60627, 57893, 57893, 57893, 57893, 57893, 58805, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57916, 57916, 57916, 57916, 61460, 57916, 57893, 57893, 57893, 60654, 57893, 57893, 0, 0, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 60692, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 59499, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 60665, 57916, 60668, 57916, 57916, 57916, 57916, 60673, 57916, 57916, 57916, 60676, 57916, 57916, 57916, 57916, 58935, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 58887, 57916, 57916, 57916, 57916, 57943, 57943, 57943, 60714, 57943, 60717, 57943, 57943, 57943, 57943, 60722, 57943, 57943, 57943, 60725, 57943, 57943, 57943, 57943, 57943, 61580, 61581, 57943, 57943, 0, 0, 0, 0, 4242, 0, 528, 528, 528, 528, 528, 528, 528, 528, 2427, 528, 528, 528, 528, 2431, 528, 0, 3454, 0, 3455, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1883, 0, 0, 0, 528, 528, 528, 3485, 528, 528, 528, 528, 528, 528, 528, 528, 528, 528, 528, 528, 1384, 528, 528, 528, 528, 528, 3497, 528, 528, 528, 528, 528, 528, 528, 528, 3505, 528, 528, 528, 528, 528, 2453, 528, 528, 528, 528, 528, 528, 528, 528, 528, 528, 1382, 528, 528, 528, 1386, 528, 57893, 57893, 60869, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 60637, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 60314, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 60341, 57893, 57893, 57893, 0, 2999, 0, 0, 0, 57893, 60881, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 60889, 57893, 57893, 57893, 57893, 57893, 58820, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 58829, 57893, 57893, 57916, 57916, 57916, 60911, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 59503, 57916, 57916, 57916, 57916, 57916, 60923, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 60931, 57916, 57916, 57916, 57916, 57916, 59481, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 58911, 57916, 57916, 57916, 57916, 57916, 57916, 57943, 57943, 57943, 60953, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 60094, 57943, 57943, 57943, 57943, 57943, 57943, 60965, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 60973, 57943, 57943, 57943, 57943, 57943, 60050, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 60745, 57943, 57943, 57943, 0, 3680, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 3439, 0, 0, 0, 0, 0, 3693, 3694, 0, 0, 0, 0, 528, 528, 528, 528, 528, 528, 528, 2875, 528, 528, 528, 528, 528, 528, 528, 528, 3704, 528, 528, 528, 528, 528, 528, 528, 528, 528, 528, 528, 528, 528, 528, 3494, 528, 528, 3714, 528, 528, 528, 528, 528, 3719, 528, 3721, 528, 528, 528, 528, 528, 528, 3488, 528, 528, 528, 528, 528, 3492, 528, 528, 528, 61091, 57893, 61093, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57916, 57916, 57916, 57916, 57916, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 61131, 61132, 57943, 57943, 57916, 57916, 57916, 57916, 61104, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 61110, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 61114, 57916, 57916, 57916, 57916, 57916, 57916, 61120, 57916, 61122, 57916, 57916, 57916, 57916, 57916, 61324, 57916, 57916, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 60970, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 61149, 57943, 61151, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 528, 2761, 528, 528, 528, 57893, 60110, 57893, 57893, 528, 528, 57893, 57893, 57893, 0, 0, 0, 3818, 0, 0, 0, 0, 0, 0, 0, 722, 0, 724, 0, 0, 0, 0, 0, 0, 0, 0, 3823, 0, 0, 0, 0, 0, 0, 0, 3675, 0, 0, 0, 0, 3831, 0, 528, 528, 528, 528, 528, 3843, 528, 528, 528, 3847, 528, 528, 528, 528, 528, 1357, 528, 528, 528, 528, 528, 528, 528, 528, 528, 528, 528, 1422, 528, 528, 528, 528, 57893, 61205, 57893, 57893, 57893, 61209, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 60886, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 59896, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 61218, 57893, 57893, 57893, 57893, 57916, 57916, 57916, 57916, 57916, 61224, 57916, 0, 0, 0, 0, 57893, 57943, 57943, 58415, 57943, 57943, 57943, 57943, 57943, 57943, 58446, 57916, 57916, 61228, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 60697, 57916, 57916, 57916, 61237, 57916, 57916, 57916, 57916, 57943, 57943, 57943, 57943, 57943, 61243, 57943, 57943, 57943, 61247, 528, 4162, 57893, 57893, 57893, 61510, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 61518, 57893, 57893, 57893, 57893, 58858, 57893, 57893, 57893, 57893, 57893, 57893, 50676, 58773, 990, 57916, 57916, 57916, 57916, 58903, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 58913, 57916, 57916, 57916, 57916, 61523, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 61531, 57916, 57943, 57943, 57943, 57943, 57943, 60955, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 59581, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 61536, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 61544, 57943, 0, 0, 0, 0, 0, 0, 262144, 262144, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 4203, 4204, 4205, 0, 4207, 528, 528, 528, 528, 528, 528, 528, 528, 528, 4213, 4214, 528, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 61564, 61565, 57893, 57893, 57916, 0, 0, 0, 0, 57893, 57943, 57943, 58416, 57943, 57943, 57943, 57943, 58437, 58441, 58447, 528, 528, 528, 57893, 57893, 57893, 57893, 57916, 57916, 57916, 57916, 57943, 57943, 57943, 57943, 4298, 0, 528, 4299, 57893, 61644, 57916, 61645, 57943, 61646, 0, 528, 57893, 57916, 57943, 0, 528, 528, 528, 528, 528, 528, 2425, 528, 528, 528, 528, 528, 528, 528, 528, 528, 3246, 528, 528, 528, 528, 528, 528, 0, 0, 0, 361, 362, 0, 0, 0, 0, 0, 0, 0, 368, 0, 296, 0, 0, 0, 0, 393685, 0, 393685, 393685, 393685, 0, 393685, 393685, 393685, 393685, 393685, 393685, 0, 1, 24578, 3, 0, 0, 4366336, 0, 0, 0, 0, 0, 302, 303, 0, 0, 0, 0, 1731, 0, 0, 0, 0, 0, 0, 0, 1741, 1742, 0, 0, 0, 0, 0, 278528, 278528, 0, 0, 0, 0, 0, 0, 0, 0, 0, 416, 416, 0, 0, 0, 0, 0, 426, 426, 0, 0, 131072, 426, 0, 0, 0, 0, 426, 0, 0, 454, 0, 426, 0, 483, 483, 483, 0, 0, 362, 362, 362, 502, 362, 362, 362, 362, 483, 483, 483, 483, 483, 483, 483, 483, 483, 483, 483, 483, 483, 483, 483, 483, 546, 57912, 546, 57912, 546, 546, 57912, 546, 546, 57935, 57912, 546, 546, 57912, 57912, 57912, 57912, 57962, 57962, 57912, 57912, 57912, 57912, 57962, 57962, 57912, 546, 57912, 57912, 57935, 57912, 57912, 57912, 57912, 57912, 57912, 57912, 57935, 57935, 57912, 57912, 57962, 57912, 57912, 57912, 1, 24578, 3, 155942, 155942, 296, 0, 0, 0, 0, 0, 302, 303, 0, 368, 368, 0, 703, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 2298, 0, 0, 0, 905, 528, 528, 528, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 58287, 57893, 57893, 57893, 57893, 57893, 60340, 57893, 57893, 57893, 57893, 57893, 0, 2999, 0, 0, 0, 57893, 58306, 57893, 57893, 57893, 57893, 58324, 57893, 57893, 57893, 0, 57916, 57916, 57916, 57916, 57916, 57943, 57943, 57943, 61129, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 60741, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 60753, 57943, 528, 528, 528, 57893, 57893, 57893, 57916, 57916, 58363, 57916, 57916, 57916, 57916, 57916, 58382, 57916, 57916, 57916, 57916, 58400, 57916, 57916, 57916, 57916, 58936, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 60392, 57916, 57916, 57916, 57916, 57916, 0, 0, 0, 0, 58306, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 58438, 57943, 57943, 57943, 57943, 57943, 60448, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 60427, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 58457, 57943, 57943, 57943, 57943, 58475, 57943, 57943, 57943, 528, 528, 528, 528, 2284, 57893, 57893, 57893, 57893, 59633, 2290, 0, 2291, 0, 0, 905, 528, 0, 57893, 57893, 57893, 57893, 58324, 57893, 155942, 1151, 0, 1152, 0, 0, 1157, 0, 0, 0, 1266, 0, 0, 0, 0, 0, 0, 0, 1266, 0, 0, 0, 0, 0, 0, 450560, 0, 0, 450560, 0, 450560, 450560, 450560, 450560, 450560, 450560, 450560, 450560, 450560, 450560, 450560, 450560, 450560, 450560, 450560, 450560, 1346, 528, 528, 528, 1355, 528, 528, 528, 528, 528, 528, 528, 528, 528, 528, 528, 528, 3711, 528, 528, 528, 58773, 914, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 58785, 57893, 57893, 57893, 58794, 57893, 57893, 58855, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 50676, 58773, 990, 57916, 57916, 57916, 57916, 58938, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 61121, 57916, 57916, 57916, 57916, 57943, 57943, 58974, 57943, 57943, 57943, 58983, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 59045, 57943, 57943, 57943, 57943, 57943, 57943, 0, 0, 0, 1730, 0, 0, 0, 0, 1737, 0, 0, 0, 0, 0, 0, 0, 739, 0, 0, 0, 0, 0, 0, 0, 0, 1271, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1762, 0, 0, 0, 0, 0, 0, 0, 1770, 1771, 0, 0, 0, 0, 0, 3422, 0, 0, 0, 0, 0, 0, 0, 3426, 0, 0, 0, 0, 0, 1834, 0, 0, 0, 0, 0, 0, 0, 1840, 0, 0, 0, 0, 0, 0, 532480, 0, 0, 0, 0, 0, 0, 0, 0, 0, 694, 0, 0, 0, 0, 0, 368, 0, 0, 1846, 0, 1848, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 376832, 0, 376832, 0, 0, 1915, 528, 528, 528, 528, 528, 528, 528, 528, 528, 528, 528, 528, 528, 528, 528, 2499, 528, 1930, 528, 528, 528, 528, 528, 528, 528, 528, 528, 528, 528, 1943, 528, 528, 0, 57893, 57893, 57893, 57893, 57893, 57893, 155942, 1151, 0, 0, 0, 0, 0, 0, 0, 4857856, 4874240, 0, 0, 0, 0, 0, 0, 0, 796, 0, 0, 0, 0, 805, 0, 0, 0, 59373, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 59386, 57893, 57893, 57893, 2588, 0, 0, 0, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 60352, 57916, 57916, 57916, 57916, 57916, 57893, 57893, 59411, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 59422, 57893, 57893, 57893, 57893, 57893, 60884, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 60903, 57893, 57893, 57893, 57893, 57893, 57916, 59511, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 59522, 57916, 57916, 57916, 57916, 57916, 61395, 57916, 61397, 61398, 57916, 61400, 57916, 61402, 57943, 57943, 57943, 3916, 528, 61261, 57893, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 2774, 0, 0, 0, 0, 59607, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 59618, 57943, 57943, 57943, 57943, 57943, 58978, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 58997, 0, 0, 2326, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 221652, 221652, 221652, 0, 0, 2504, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 61097, 57893, 57893, 57893, 57916, 57916, 57916, 57916, 57916, 528, 3229, 528, 528, 528, 528, 528, 528, 528, 528, 528, 528, 528, 528, 528, 528, 2447, 528, 60618, 57893, 60620, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 61211, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 59870, 57893, 57893, 57893, 57893, 59875, 57893, 57893, 57916, 57916, 57916, 60667, 57916, 60669, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 60404, 57916, 57916, 57916, 57916, 57916, 60698, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57943, 57943, 57943, 57943, 60709, 57943, 57943, 57943, 57943, 57943, 60463, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 59599, 57943, 57943, 57943, 59605, 57943, 57943, 57943, 57943, 60716, 57943, 60718, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 59046, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 60747, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 528, 528, 528, 57893, 57893, 57893, 0, 0, 0, 3656, 0, 0, 0, 0, 0, 0, 0, 3432, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 2323, 2324, 0, 0, 57943, 57943, 61250, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 60724, 57943, 57943, 0, 4147, 0, 0, 0, 528, 528, 528, 528, 528, 528, 528, 528, 528, 528, 528, 528, 528, 528, 528, 528, 0, 4161, 528, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 61517, 57893, 57893, 57893, 57893, 57893, 60898, 60900, 57893, 57893, 57893, 57893, 60904, 57893, 60906, 60907, 57893, 57916, 57916, 57916, 61627, 57916, 57943, 57943, 57943, 57943, 61631, 57943, 0, 0, 0, 0, 528, 528, 2870, 528, 528, 528, 528, 528, 528, 528, 528, 528, 528, 2878, 373, 373, 0, 0, 131072, 373, 0, 0, 0, 0, 373, 0, 0, 0, 0, 373, 547, 57913, 547, 57913, 547, 547, 57913, 547, 547, 57936, 57913, 547, 547, 57913, 57913, 57913, 57913, 57963, 57963, 57913, 57913, 57913, 57913, 57963, 57963, 57913, 547, 57913, 57913, 57936, 57913, 57913, 57913, 57913, 57913, 57913, 57913, 57936, 57936, 57913, 57913, 57963, 57913, 57913, 57913, 1, 24578, 3, 155942, 155942, 296, 0, 0, 0, 0, 0, 302, 303, 0, 528, 528, 528, 528, 861, 528, 528, 528, 528, 528, 528, 528, 528, 528, 528, 528, 528, 3723, 528, 528, 528, 57916, 58356, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 60677, 57916, 57893, 58799, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 61221, 57893, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57943, 57943, 60023, 57943, 57943, 57943, 57943, 57943, 57943, 528, 528, 1931, 528, 528, 528, 528, 528, 528, 528, 528, 528, 528, 528, 528, 528, 1401, 528, 528, 57893, 59374, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 59882, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 60290, 57893, 57893, 57893, 57893, 57893, 57893, 59474, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 60920, 57916, 0, 0, 0, 3824, 0, 3825, 0, 0, 0, 0, 3675, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1227, 0, 0, 0, 0, 0, 364, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 364, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 2335, 0, 0, 0, 365, 0, 0, 0, 0, 364, 0, 0, 0, 139264, 147456, 0, 0, 0, 0, 0, 0, 1324, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1166, 0, 0, 0, 0, 0, 0, 0, 0, 0, 434, 131072, 0, 434, 434, 0, 0, 0, 434, 0, 365, 434, 0, 0, 0, 0, 401408, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 6471680, 0, 0, 0, 466, 0, 0, 0, 494, 494, 497, 497, 497, 497, 503, 504, 497, 497, 517, 517, 517, 517, 517, 517, 517, 517, 517, 517, 517, 517, 517, 517, 517, 517, 548, 57914, 548, 57914, 548, 548, 57914, 548, 548, 57937, 57914, 548, 548, 57914, 57914, 57914, 57914, 57964, 57964, 57914, 57914, 57914, 57914, 57964, 57964, 57914, 626, 57914, 57977, 57937, 57914, 57914, 57914, 57914, 57914, 57914, 57914, 57937, 57937, 57914, 57914, 57964, 57914, 57914, 57914, 57977, 57977, 1, 24578, 3, 155942, 155942, 296, 0, 0, 0, 0, 0, 302, 303, 0, 0, 0, 0, 459187, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 528, 528, 528, 528, 528, 528, 528, 528, 0, 528, 2420, 528, 528, 528, 528, 528, 528, 528, 528, 528, 528, 528, 528, 528, 1402, 528, 528, 0, 0, 0, 57893, 59850, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 60287, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 60302, 57893, 57893, 57893, 57893, 60305, 57893, 57893, 60307, 528, 528, 528, 528, 2895, 528, 528, 528, 528, 528, 528, 528, 528, 528, 528, 528, 528, 4026, 57893, 57893, 57893, 57893, 60337, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 0, 0, 0, 0, 0, 0, 1781, 0, 0, 0, 0, 0, 0, 0, 0, 0, 139264, 147456, 0, 0, 335872, 0, 0, 57916, 57916, 57916, 57916, 60371, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 61401, 57916, 57943, 57943, 57943, 368, 368, 0, 0, 0, 0, 705, 0, 0, 0, 0, 0, 0, 0, 0, 0, 139264, 147456, 0, 0, 540672, 0, 0, 0, 0, 735, 0, 801, 0, 0, 0, 0, 0, 0, 0, 0, 801, 0, 0, 0, 0, 416, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 3464, 0, 0, 0, 528, 528, 528, 528, 862, 528, 528, 528, 528, 528, 528, 528, 528, 528, 528, 528, 528, 61203, 57893, 57893, 57893, 57916, 58357, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 61111, 57916, 1202, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 368, 368, 0, 0, 0, 0, 0, 0, 0, 0, 709, 0, 0, 0, 0, 0, 0, 1251, 0, 0, 0, 0, 0, 0, 0, 0, 0, 2810, 0, 0, 0, 0, 0, 0, 58853, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 50676, 58773, 990, 57916, 57916, 57916, 57916, 59463, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 60696, 57916, 57916, 57916, 57916, 59015, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 60442, 1726, 0, 0, 0, 0, 1733, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 319488, 319488, 0, 0, 0, 0, 0, 57916, 57916, 57916, 59978, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 59959, 57916, 57916, 57916, 57943, 57943, 57943, 60061, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 59011, 57943, 57943, 57943, 2779, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 245760, 528, 528, 2881, 528, 528, 528, 528, 528, 528, 528, 528, 528, 528, 528, 528, 528, 1404, 528, 528, 57916, 57916, 60357, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 60919, 57916, 57916, 0, 306, 307, 0, 0, 0, 0, 0, 0, 0, 647, 0, 0, 0, 0, 0, 0, 1813, 0, 0, 0, 0, 0, 0, 0, 0, 0, 787, 0, 0, 0, 0, 0, 0, 906, 528, 528, 528, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 906, 528, 0, 57893, 58490, 57893, 57893, 58325, 57893, 155942, 1151, 0, 0, 0, 0, 0, 0, 1836, 0, 0, 1838, 0, 0, 0, 0, 0, 0, 0, 2861, 2813, 0, 0, 0, 0, 0, 2866, 0, 57893, 57893, 58856, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 50676, 58773, 990, 57916, 57916, 57916, 57916, 59495, 59496, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 61574, 57943, 57943, 57943, 57943, 0, 0, 1862, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 327680, 327680, 327680, 528, 528, 1948, 528, 528, 528, 528, 528, 528, 528, 528, 528, 528, 528, 528, 528, 1942, 528, 528, 57893, 59391, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 60313, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 59922, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 59491, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 61123, 57916, 57916, 0, 0, 0, 57893, 57893, 57893, 57893, 59853, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 60329, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 59367, 59368, 57893, 57893, 57893, 57893, 57893, 59877, 57893, 59879, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 60636, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 60316, 57893, 57893, 57893, 57893, 57893, 57893, 57916, 57916, 57916, 57916, 59966, 57916, 59968, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 61117, 57916, 61119, 57916, 57916, 57916, 57916, 57916, 57916, 57943, 57943, 57943, 57943, 60049, 57943, 60051, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 59552, 57943, 57943, 57943, 57943, 59556, 57943, 60381, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 61403, 57943, 57943, 57943, 57943, 57943, 60446, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 57943, 59554, 57943, 57943, 57943, 0, 3418, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 352256, 352256, 352256, 57893, 57893, 57893, 57893, 60871, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 60645, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 57893, 59923, 57893, 57893, 57893, 57893, 57893, 57893, 3713, 528, 528, 528, 528, 528, 528, 528, 528, 528, 528, 528, 528, 528, 528, 528, 2500, 57916, 57916, 61113, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 57916, 59973, 57916, 57916, 57916, 0, 0, 0, 2310144, 0, 0, 0, 0, 0, 0, 2310144, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 2310144, 0, 0, 2310144, 0, 0, 0, 0, 0, 0, 0, 2310144, 2310144, 0, 0, 0, 0, 0, 0, 0, 0, 2310144, 0, 0, 0, 2310144, 0, 0, 0, 0, 0, 2310144, 0, 2310144, 0, 0, 0, 0, 0, 0, 2310144, 2310561, 2310561, 0, 2310144, 0, 0, 2310144, 0, 0, 2310144, 0, 2310144, 2310144, 0, 2310144, 0, 2310144, 2310144, 0, 0, 0, 0, 0, 0, 2310561, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 2359296, 368, 0, 0, 0, 0, 2310144, 0, 0, 2310740, 2310740, 2310740, 2310740, 2310740, 2310740, 2310740, 2310740, 2310740, 2310740, 2310740, 2310740, 2310740, 2310144, 2310740, 2310144, 2310144, 2310740, 1, 24578, 3, 0, 0, 4366336, 0, 0, 0, 0, 0, 302, 303, 0, 0, 0, 0, 459347, 459347, 459347, 459347, 459347, 459347, 459347, 459347, 459347, 459347, 459347, 459347, 459347, 459222, 459379, 459222, 2318336, 0, 0, 0, 4268032, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 4276224, 0, 0, 0, 0, 4358144, 4358144, 4358144, 4358144, 0, 914, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 4956160, 4964352, 0, 0, 0, 0, 0, 467, 2335204, 2335204, 2335204, 467, 467, 467, 467, 467, 467, 467, 467, 467, 467, 2335238, 2335238, 1, 24578, 3, 0, 0, 4366336, 0, 0, 0, 0, 0, 302, 303, 0, 0, 0, 467, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 2335238, 2335204, 2335238, 2335238, 2335238, 2335238, 2335238, 2335238, 2335238, 2335238, 2335238, 2335238, 2335238, 2335238, 2335238, 2335238, 2335238, 2335238, 0, 0, 0, 2342912, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 2348, 0, 0, 0, 0, 0, 0, 2326528, 0, 0, 4358144, 4358144, 4358144, 4358144, 4358144, 4358144, 4358144, 4358144, 4358144, 4358144, 4358144, 5177344, 4358144, 4358144, 4358144, 4358144, 0, 0, 0, 2359296, 0, 2359296, 0, 0, 0, 2359296, 0, 2359296, 2359296, 2359296, 2359296, 2359296, 2359296, 2359296, 2359296, 2359296, 2359296, 2359296, 2359296, 2359296, 2359296, 2359296, 2359296, 2359296, 2359296, 1, 24578, 3, 0, 0, 4366336, 0, 0, 0, 0, 0, 302, 303, 0, 0, 0, 0, 573440, 0, 573440, 573440, 573440, 0, 573440, 573440, 573440, 573440, 573440, 573440, 1, 24578, 3, 0, 0, 4366336, 0, 0, 0, 0, 0, 302, 303, 0, 0, 2367488, 0, 0, 4268032, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 4931584, 0, 0, 0, 0, 0, 0, 2351104, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 507904, 507904, 507904, 0, 913, 0, 0, 0, 0, 0, 4857856, 4874240, 0, 0, 0, 0, 0, 0, 0, 1180, 0, 0, 0, 0, 1184, 0, 0, 1187, 0, 6275072, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 989, 0, 0, 0, 0, 913, 4359057, 4359057, 4359057, 4359057, 4359057, 4359057, 4359057, 4359057, 4359057, 4359057, 4359057, 4359057, 4359057, 5260177, 4359057, 4359057, 0, 0, 1, 24578, 3, 155942, 155942, 296, 0, 0, 0, 0, 0, 302, 303, 0, 0, 0, 0, 573440, 573440, 573440, 573440, 573440, 573440, 573440, 573440, 573440, 573440, 573440, 573440, 573440, 573440, 573440, 573440, 212992, 0, 212992, 212992, 212992, 212992, 212992, 212992, 212992, 212992, 212992, 212992, 212992, 212992, 212992, 212992, 212992, 212992, 212992, 212992, 0, 0, 0, 0, 0, 4366336, 0, 0, 0, 0, 0, 0, 0, 0, 2412, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 6275072, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 6258688, 6447104, 0, 0, 6127616, 0, 6348800, 5906432, 0, 5537792, 0, 4882432, 0, 0, 0, 0, 0, 0, 4825088, 0, 0, 5177344, 0, 0, 0, 0, 5701632, 0, 0, 0, 0, 0, 4358144, 4358144, 4358144, 4825088, 4358144, 4358144, 4358144, 4358144, 0, 0, 913, 913, 913, 4826001, 913, 913, 913, 913, 913, 913, 0, 0, 989, 989, 4842461, 989, 989, 989, 4899805, 989, 0, 0, 5513216, 5783552, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 4358144, 4358144, 4358144, 4358144, 4358144, 4857856, 4874240
];

JSONiqParser.EXPECTED =
[ 169, 185, 215, 1326, 231, 1001, 278, 928, 942, 1085, 320, 247, 263, 307, 336, 352, 368, 384, 400, 914, 2264, 430, 2264, 2264, 2264, 2264, 2264, 2264, 2264, 2264, 2264, 2264, 2264, 2268, 1707, 1603, 1403, 460, 1356, 1591, 491, 507, 517, 533, 1816, 1031, 2119, 564, 475, 580, 596, 625, 684, 700, 1905, 716, 1114, 747, 778, 788, 1693, 1145, 762, 804, 820, 836, 852, 868, 884, 900, 958, 988, 1017, 972, 1997, 1047, 1057, 1073, 1101, 1130, 2028, 1562, 1190, 1205, 1221, 1237, 1253, 1269, 1298, 1314, 1342, 1174, 1446, 1372, 1876, 2073, 1633, 668, 1388, 1419, 1434, 1467, 1483, 1499, 1515, 609, 1531, 1547, 1578, 444, 654, 640, 1619, 1649, 1662, 1678, 1723, 1738, 1754, 1770, 1786, 1802, 1832, 1848, 291, 1864, 2228, 1892, 1921, 1937, 199, 2149, 1953, 1969, 1985, 1282, 2013, 2044, 2060, 2089, 2105, 548, 2135, 2165, 2181, 2197, 2213, 1160, 2244, 731, 2260, 2264, 1457, 412, 2264, 2264, 2264, 2264, 2264, 2264, 2264, 2264, 2264, 2264, 2264, 2264, 2264, 2264, 419, 2284, 2288, 2290, 2290, 2290, 2291, 2295, 2290, 2297, 2301, 2304, 2311, 2307, 2315, 2318, 2322, 2326, 2330, 2334, 2338, 2345, 2345, 3446, 2344, 2345, 3427, 2352, 3605, 2345, 4169, 2345, 2345, 2345, 3950, 4664, 4990, 4059, 4582, 2828, 4588, 2345, 4069, 4598, 2345, 4069, 4603, 2734, 3100, 2397, 2497, 2359, 2366, 2466, 2372, 2376, 2345, 2345, 4266, 2345, 2345, 2383, 3820, 2359, 2359, 2392, 2419, 2419, 2419, 2419, 2405, 2427, 2345, 2345, 2345, 2345, 2345, 2409, 2455, 3819, 2345, 2345, 3099, 3100, 3100, 3100, 2495, 2398, 2398, 2398, 2504, 2359, 2359, 2359, 2482, 2419, 2419, 2419, 2517, 2345, 2345, 2345, 2457, 2345, 3100, 3100, 3101, 2398, 2398, 2398, 2359, 2359, 2359, 2359, 2359, 2418, 2419, 2419, 2419, 2419, 2419, 2424, 2437, 2345, 2345, 2345, 2345, 4505, 2729, 2345, 2345, 2345, 2345, 4509, 5101, 4513, 2345, 2345, 3904, 2359, 2359, 2484, 2419, 2419, 2519, 2345, 3864, 2345, 4865, 3100, 3100, 2501, 2398, 2538, 2359, 2359, 2359, 2359, 2359, 2548, 2419, 2419, 2419, 2419, 2395, 2490, 2345, 2345, 2359, 2483, 2419, 2549, 2508, 3755, 2345, 4867, 4911, 2398, 2400, 2359, 2515, 2419, 2525, 3605, 3100, 2476, 2401, 2547, 2550, 2816, 4868, 2399, 2481, 2485, 2735, 2532, 2503, 2361, 2368, 4866, 2476, 2360, 2420, 4909, 2537, 2546, 2486, 2533, 2362, 2542, 2554, 2567, 2571, 2575, 2579, 2583, 2586, 2586, 2586, 2598, 2591, 2586, 2587, 2595, 2602, 2606, 2610, 2614, 2618, 2622, 2626, 2630, 4594, 2634, 2345, 2345, 2903, 2640, 2345, 2345, 2999, 2345, 2345, 2645, 2345, 2345, 2345, 2345, 2345, 2345, 4208, 2345, 2345, 2345, 2345, 2345, 2345, 2345, 2345, 2345, 2345, 4460, 2655, 2345, 2345, 2345, 2345, 2345, 2345, 2345, 2345, 2345, 2345, 2345, 2345, 2655, 2345, 2345, 2636, 4128, 2345, 2345, 2345, 2345, 2635, 4127, 2345, 2345, 2345, 2345, 2635, 4127, 2794, 2345, 2345, 2345, 3016, 2798, 2345, 2345, 2345, 3017, 2799, 2345, 2345, 2345, 2806, 2729, 2876, 3308, 2345, 2915, 2345, 3310, 2345, 2922, 2345, 3536, 2683, 4475, 3831, 4927, 2685, 4422, 2850, 2345, 2345, 2345, 2345, 2345, 2345, 5004, 2345, 2345, 2345, 2345, 4417, 3024, 3547, 2854, 5007, 2861, 2345, 2345, 2345, 2345, 4422, 2869, 2345, 2345, 2345, 2345, 2345, 2868, 2729, 2345, 2345, 2345, 2345, 2345, 2873, 2345, 2345, 2345, 4443, 2345, 3549, 3338, 2880, 2345, 2345, 2345, 2345, 2887, 2729, 2345, 2345, 2345, 2883, 2880, 2345, 2345, 4951, 4956, 2345, 2345, 2345, 2345, 4604, 4971, 4982, 2345, 2345, 4805, 4988, 3762, 2345, 3965, 2345, 4436, 4591, 2907, 2345, 3192, 2909, 2345, 4490, 2345, 3191, 2908, 2345, 3491, 4849, 4476, 4920, 4920, 4920, 4849, 2720, 4477, 4477, 4477, 4918, 4922, 2683, 4476, 4863, 4478, 2719, 2684, 2432, 2429, 2431, 2563, 2345, 2345, 2345, 2345, 2345, 2345, 2984, 2345, 2345, 2345, 2511, 4068, 4075, 2345, 2345, 4081, 2345, 4085, 2345, 2345, 4099, 2345, 3067, 3768, 2729, 4088, 2928, 3046, 2947, 2932, 2935, 2938, 2940, 2940, 2944, 2345, 2951, 2956, 2345, 2345, 5022, 2729, 2345, 2345, 2345, 2345, 4180, 2345, 2345, 2345, 2345, 5023, 2345, 4654, 2345, 4496, 2345, 3077, 4168, 3570, 5100, 4176, 2345, 2345, 2345, 5024, 2345, 2345, 4147, 2345, 2345, 4258, 2345, 2345, 2339, 3845, 2345, 3726, 3911, 3122, 3918, 2345, 4757, 2345, 4949, 2345, 2961, 2974, 2978, 3641, 3718, 2988, 2996, 2722, 3003, 3008, 2345, 2998, 4205, 2345, 2952, 2345, 2345, 3697, 2345, 3015, 4630, 3021, 3029, 3033, 2412, 2991, 2723, 3040, 3497, 2345, 2951, 2956, 2345, 5067, 4831, 2345, 3623, 2964, 3050, 4101, 2345, 3156, 3059, 2345, 2345, 5115, 3712, 2387, 4578, 2345, 5120, 2458, 2509, 4221, 2345, 5107, 2345, 3202, 2763, 3060, 2345, 2345, 2345, 3733, 2345, 2951, 2733, 2345, 4755, 4856, 3111, 4236, 2557, 3086, 2345, 2348, 3162, 2345, 2345, 2527, 3154, 2345, 2345, 2347, 3161, 2345, 2345, 4684, 3167, 3432, 4676, 3096, 2345, 2345, 2345, 2345, 2472, 3106, 2345, 2345, 2345, 2345, 2346, 3105, 2345, 2345, 2345, 4878, 3110, 3119, 2345, 5067, 4832, 4234, 3023, 3126, 4109, 3602, 3171, 3176, 2345, 4069, 3180, 2345, 4170, 3176, 2345, 2345, 3187, 2345, 4682, 3217, 4094, 3257, 3196, 3972, 3190, 2345, 3974, 2345, 3251, 3209, 2345, 3973, 2345, 3214, 3218, 3223, 3227, 3231, 4192, 2345, 3229, 2345, 3237, 3606, 3242, 3246, 4319, 3250, 3238, 3230, 3255, 3261, 3266, 3266, 3266, 3274, 3278, 3262, 3262, 3262, 3285, 3289, 4198, 3290, 4959, 3294, 4246, 5038, 3298, 3302, 3306, 3316, 3322, 3329, 2345, 2345, 2345, 2345, 3926, 2345, 2345, 2875, 3333, 3337, 4593, 3342, 3347, 4397, 3357, 3361, 3365, 3366, 3366, 3370, 2345, 4118, 3376, 2345, 4445, 3382, 4945, 3392, 2721, 3398, 2829, 3406, 3410, 3417, 3424, 3004, 3431, 3947, 2345, 3426, 2345, 2345, 3115, 3658, 4038, 4560, 2345, 2345, 2345, 2345, 2345, 2345, 2345, 2650, 2345, 2345, 2454, 2345, 2345, 2345, 4867, 3100, 3100, 3100, 3100, 3101, 2398, 2398, 2398, 2398, 2545, 2359, 2359, 2359, 2359, 2359, 2464, 2419, 2419, 2419, 2419, 2419, 2395, 2470, 4119, 2345, 2345, 3437, 3706, 3578, 3394, 4115, 2345, 5085, 3441, 3522, 4170, 3451, 2345, 2345, 3132, 3489, 2345, 2345, 2345, 2345, 3143, 3495, 2345, 2345, 2345, 2345, 4516, 3484, 2345, 4070, 3455, 2345, 2345, 2345, 4071, 3456, 2345, 2345, 2345, 3460, 3464, 2345, 2345, 2345, 3098, 3100, 3100, 3100, 3100, 3100, 2397, 2398, 2398, 2398, 2398, 2398, 2480, 4118, 3830, 2345, 3469, 3584, 3579, 3476, 4935, 4215, 3011, 2345, 2345, 4515, 3483, 2345, 2345, 3191, 2898, 2345, 2345, 2345, 2892, 2345, 2345, 2345, 2897, 2345, 2345, 2345, 3963, 3232, 2345, 2345, 2345, 2345, 3517, 3490, 2345, 2345, 2345, 2345, 4515, 3520, 2345, 2345, 2345, 2345, 3982, 2345, 3526, 3585, 3535, 4117, 2786, 2345, 2345, 3541, 2345, 2345, 2345, 2345, 4530, 3545, 2345, 2345, 2345, 2345, 3541, 2345, 2345, 2345, 2345, 2456, 2345, 2345, 4910, 3100, 3100, 3100, 3100, 2476, 2398, 2398, 2398, 4531, 3232, 2345, 2345, 2345, 3560, 3565, 3576, 4115, 3204, 3583, 2345, 5095, 2345, 2345, 2345, 3163, 3064, 2345, 2345, 2345, 2345, 3219, 3071, 2345, 2345, 2345, 2345, 3157, 5054, 2345, 2345, 2345, 5094, 2345, 2345, 2345, 5053, 2345, 2345, 2345, 3590, 3966, 3561, 2345, 2521, 3137, 2345, 2345, 3754, 2981, 3141, 5078, 4842, 4667, 2967, 3147, 2528, 3155, 2345, 2751, 5076, 2686, 5082, 4996, 5091, 5099, 2345, 2345, 5072, 2345, 2345, 5105, 2345, 2345, 4450, 3818, 2345, 3825, 2345, 2460, 3835, 3839, 2345, 2345, 2345, 4069, 3843, 2345, 4928, 2345, 3443, 2345, 3828, 2345, 4052, 3148, 2345, 2386, 3114, 3150, 4047, 3149, 3635, 3635, 4052, 3113, 3150, 3150, 3150, 3980, 3634, 3112, 3635, 4053, 3622, 3635, 3150, 4974, 4976, 3627, 4978, 3631, 3640, 2345, 2345, 2345, 2345, 4428, 3779, 3723, 2345, 3595, 3645, 3649, 4091, 3655, 3663, 3667, 3671, 3675, 3679, 3680, 3684, 2345, 2801, 3689, 2345, 4841, 3695, 4295, 2763, 3350, 3199, 3691, 5040, 3485, 3701, 4584, 3465, 3705, 4823, 3710, 3716, 2345, 2345, 2802, 3690, 2345, 4616, 4756, 3722, 3731, 3957, 3201, 3737, 3750, 3759, 2345, 3940, 3766, 2345, 2345, 2345, 3312, 4745, 4749, 4761, 4765, 4769, 4773, 4777, 4781, 4785, 4788, 4792, 4795, 3504, 3232, 2345, 2345, 2345, 3505, 2345, 2345, 2345, 2646, 3772, 2345, 2345, 2345, 4427, 3778, 2345, 4164, 4095, 2345, 3785, 4903, 2651, 2345, 3753, 2345, 2345, 3793, 2345, 2345, 2345, 2345, 3098, 3100, 3100, 3100, 3100, 2398, 2398, 2398, 2398, 2478, 2359, 2359, 2345, 3797, 2345, 2345, 2345, 2345, 2345, 3801, 2345, 2345, 2345, 2345, 2345, 3805, 2345, 2345, 3846, 2810, 5004, 2345, 2345, 2345, 2345, 4151, 2345, 2815, 2820, 2833, 4077, 2838, 3850, 2345, 2345, 2345, 2345, 4737, 2345, 3855, 2458, 4716, 3837, 3863, 2345, 2345, 3172, 3871, 3924, 2345, 2340, 2345, 2345, 4149, 2345, 2339, 3845, 3723, 3089, 3789, 2345, 3930, 4129, 2345, 2776, 3914, 4135, 2728, 2345, 2345, 2345, 2345, 3210, 2345, 2784, 3082, 3938, 2345, 2790, 5046, 2345, 3935, 2345, 3944, 2730, 3877, 2345, 3352, 2732, 3955, 2731, 4157, 4157, 4157, 3351, 2732, 2732, 2732, 3531, 4156, 4716, 4157, 4718, 3878, 4157, 2732, 2345, 2345, 2345, 2345, 3851, 2345, 2345, 2345, 2345, 2345, 3850, 2345, 2345, 2345, 2345, 2345, 2345, 2345, 3612, 2663, 2345, 2345, 2345, 2345, 2345, 2345, 2345, 3780, 3961, 2345, 2345, 3920, 3970, 3978, 3986, 3990, 3994, 3998, 4001, 4005, 4007, 4011, 2345, 2345, 2345, 4015, 2345, 3606, 4752, 4107, 3553, 4020, 4024, 4643, 4888, 4028, 2345, 4851, 2811, 4032, 4826, 4037, 4042, 4046, 2345, 2345, 4016, 2345, 3746, 4051, 3551, 3877, 4057, 3636, 4570, 4063, 2345, 2345, 2345, 2780, 2345, 2345, 2345, 2510, 4067, 4105, 2924, 4156, 4113, 2386, 4123, 4538, 2345, 2345, 2635, 4127, 2345, 2345, 2345, 2345, 2641, 4133, 2345, 2345, 2345, 2345, 2778, 4181, 2345, 2457, 2345, 2345, 4984, 5025, 2345, 4139, 2345, 2855, 3611, 2345, 2857, 2345, 2345, 3386, 2345, 2856, 2345, 2345, 3830, 3616, 2345, 3445, 2345, 4145, 2345, 4670, 3724, 4155, 3572, 3114, 4161, 2345, 2345, 2345, 3472, 2729, 2345, 2345, 2345, 2345, 2842, 2345, 2345, 2345, 2345, 3081, 2846, 2345, 2345, 2345, 2345, 2345, 2739, 2692, 3821, 2749, 2757, 2345, 2761, 2767, 2345, 2771, 4802, 2345, 4185, 3077, 3725, 4231, 3041, 4220, 2345, 5087, 2729, 2345, 2345, 2911, 4134, 2345, 2345, 4172, 2345, 2345, 2345, 2354, 3845, 2345, 3723, 3606, 2458, 3787, 3901, 2345, 3781, 3537, 4190, 2345, 2345, 2910, 4196, 3378, 3685, 3607, 4673, 4202, 2823, 2345, 4033, 2729, 2345, 3931, 2956, 2345, 3925, 4214, 4219, 4225, 4240, 4244, 4264, 2345, 4250, 2345, 4252, 2345, 2345, 4256, 2345, 4251, 3343, 4538, 4228, 4262, 2345, 4270, 3353, 2345, 4610, 2345, 3183, 3130, 2345, 2345, 2345, 2346, 3136, 2345, 2345, 2345, 2345, 3183, 3130, 2345, 2345, 2673, 2345, 2345, 2677, 3814, 2690, 2696, 2700, 2704, 2708, 2712, 2716, 3744, 2727, 4275, 4284, 4288, 4419, 2668, 3659, 2733, 4292, 4308, 4420, 3556, 3556, 3556, 4317, 3530, 4421, 4335, 4323, 3555, 3529, 3568, 3025, 2669, 3556, 4330, 4339, 4341, 4333, 4326, 4345, 4349, 4351, 2345, 2345, 2345, 2345, 3897, 2992, 2345, 4313, 4355, 4359, 4390, 4363, 4367, 4371, 4375, 4379, 4381, 4383, 2345, 2345, 2345, 4937, 4387, 4657, 4394, 4401, 4460, 2827, 4636, 4599, 4278, 4407, 4411, 5122, 3727, 4426, 4432, 2345, 4440, 4449, 2345, 2345, 4938, 3774, 2657, 4454, 4458, 3325, 2345, 3053, 4464, 4468, 2345, 2345, 2345, 5128, 4474, 2345, 2345, 2345, 4482, 2345, 2345, 4271, 2888, 2345, 2345, 2345, 2345, 3233, 2345, 2345, 2345, 4962, 2864, 3650, 2893, 2345, 3727, 4483, 2345, 2345, 2345, 3055, 4487, 4494, 2345, 2345, 3890, 4311, 2433, 2446, 3618, 2826, 4186, 5066, 4947, 2733, 2345, 2345, 2345, 3413, 4500, 2345, 2345, 2345, 2345, 3420, 2800, 4414, 4622, 4520, 3318, 2345, 2970, 5060, 2345, 2345, 2345, 2377, 4524, 2345, 2345, 2345, 2345, 3869, 2345, 2345, 2345, 2345, 4171, 3876, 2345, 2345, 2345, 4069, 3870, 2345, 4952, 4311, 2443, 4535, 2827, 3865, 3433, 2345, 2345, 2345, 2379, 4543, 2345, 2345, 2345, 3858, 2345, 2345, 2345, 2345, 3859, 2345, 2345, 2345, 2957, 3045, 2345, 2345, 2377, 4548, 2345, 2345, 2345, 2345, 3281, 4553, 2345, 2345, 2345, 2378, 4549, 4470, 2345, 4558, 2440, 4564, 3203, 4831, 2345, 2345, 2377, 4568, 2345, 2345, 2345, 3951, 2345, 2345, 2345, 4574, 2345, 4640, 2345, 4069, 4647, 2560, 4651, 4661, 2345, 4680, 4539, 4642, 5031, 2345, 5014, 4688, 4830, 4815, 4403, 3883, 4642, 4692, 4628, 4816, 4696, 4696, 4702, 4830, 4706, 4714, 4714, 4722, 2414, 2449, 3808, 4731, 2414, 4906, 2450, 4735, 4725, 4727, 4741, 4829, 2345, 2345, 2345, 2345, 3925, 2345, 2772, 3501, 4141, 3509, 4115, 4215, 3074, 2345, 2345, 3513, 4799, 2345, 2345, 2345, 2345, 4809, 3015, 4813, 4820, 4967, 2388, 2680, 4210, 4836, 4554, 2345, 3372, 2345, 2345, 3384, 2345, 2345, 3372, 2345, 2345, 2856, 2345, 2345, 3589, 3594, 3599, 4708, 3872, 4840, 4846, 2345, 4710, 4855, 2345, 2345, 4544, 4860, 2834, 3742, 4965, 3080, 4872, 4877, 4882, 2345, 2345, 2345, 2345, 3606, 4886, 2345, 2345, 2345, 3478, 4892, 2345, 2345, 2345, 3882, 2663, 2459, 3092, 3887, 2345, 2345, 3894, 2345, 2345, 2345, 2355, 2345, 3479, 2345, 2345, 2345, 2345, 4897, 3616, 2345, 2345, 4544, 4915, 3035, 4280, 2918, 3446, 4873, 4698, 4926, 2345, 2345, 2345, 2345, 2345, 4932, 2345, 2345, 2345, 2345, 3606, 4942, 2345, 2345, 4435, 3377, 3907, 2902, 2345, 4489, 2729, 2345, 3651, 2902, 2345, 3192, 2909, 2345, 4994, 5020, 5001, 3205, 5011, 2345, 2345, 2345, 2345, 2741, 2745, 2345, 2345, 2345, 2345, 4170, 4608, 2345, 2345, 4614, 4989, 4620, 4626, 3755, 4950, 2345, 4634, 2345, 5013, 2345, 2744, 2345, 2345, 2345, 2345, 2345, 2743, 2345, 2345, 2345, 2345, 2345, 2743, 4900, 2345, 4501, 4915, 3036, 5018, 3447, 3864, 5029, 2345, 2345, 2345, 4170, 5035, 2345, 2345, 2345, 2345, 4303, 5044, 2345, 2345, 2345, 2345, 5050, 2345, 2345, 2345, 2345, 4304, 4900, 2345, 3811, 3740, 4997, 5058, 5064, 2345, 2345, 2345, 5071, 2345, 2345, 2345, 3269, 2345, 2345, 2345, 4297, 4301, 2345, 3402, 2800, 2345, 2345, 2345, 2345, 3401, 4528, 2345, 2345, 2345, 2345, 3401, 4528, 4470, 4299, 2345, 2345, 3270, 2345, 5111, 4997, 4577, 2345, 2345, 2753, 2345, 4893, 5116, 2345, 2752, 2491, 2458, 2345, 5126, 2345, 2345, 2345, 2345, 2345, 2345, 2345, 2345, 2345, 2345, 2345, 2345, 2345, 2345, 2345, 2345, 2661, 3388, 2667, 2345, 7267, 5893, 5132, 5144, 5141, 5151, 5153, 5153, 5153, 5153, 5148, 5176, 5152, 5153, 5153, 5153, 5159, 5163, 5176, 5153, 5153, 5169, 5164, 5153, 5180, 5155, 5154, 5168, 5151, 6159, 5173, 5199, 5205, 5205, 5205, 5209, 5209, 5201, 5213, 5217, 5220, 5224, 5228, 5245, 5232, 5242, 5234, 5238, 5236, 5249, 5252, 7269, 7026, 5260, 5273, 5276, 5276, 5276, 5255, 5821, 5295, 5276, 5276, 5276, 5276, 5268, 5730, 6588, 6270, 5301, 5276, 5276, 5254, 6597, 5821, 6646, 6646, 6646, 6646, 5394, 5396, 5325, 5449, 5322, 5396, 5396, 5325, 5276, 5384, 5428, 5333, 5409, 5274, 5276, 5276, 5276, 5286, 5290, 7174, 5276, 6844, 7218, 6005, 5276, 5276, 5276, 6390, 6115, 6646, 6647, 5323, 5396, 5397, 5400, 5428, 5428, 5428, 5428, 6645, 6646, 6646, 5398, 5400, 5428, 5345, 6844, 7217, 5353, 5276, 5266, 5276, 5276, 5388, 7455, 5457, 5396, 5396, 5396, 5396, 5276, 5399, 5402, 6779, 5408, 6667, 5275, 5276, 5275, 5276, 5276, 6239, 5276, 6839, 5410, 6667, 5275, 5276, 5275, 6561, 5276, 5275, 6839, 6843, 7454, 7155, 5276, 5276, 7229, 5276, 5276, 6466, 5276, 5276, 6526, 5276, 5276, 5276, 5302, 5276, 5276, 6646, 5393, 5396, 5396, 5396, 5400, 5407, 6668, 5276, 5276, 5268, 6066, 5402, 5428, 5428, 5428, 5429, 6646, 6646, 6646, 5395, 5396, 5396, 5396, 5326, 5368, 5419, 5276, 5276, 5276, 5305, 5401, 5428, 5428, 5428, 5430, 6646, 5427, 5428, 5428, 5428, 6644, 6646, 6646, 5402, 5387, 5276, 5276, 5276, 5307, 6771, 6646, 5394, 5396, 5396, 5398, 5427, 5276, 5276, 5268, 6630, 5397, 5403, 5276, 5276, 5269, 5731, 6036, 5400, 5400, 5402, 5428, 6644, 5402, 5428, 5428, 6643, 6646, 5368, 5400, 5427, 6643, 6646, 6646, 6646, 5396, 5396, 5396, 5397, 5387, 6646, 5395, 5324, 5368, 5276, 5999, 5276, 5276, 7234, 6237, 5276, 6239, 6237, 5401, 6643, 5393, 5324, 5415, 5434, 5460, 5464, 5463, 5462, 5461, 5326, 7445, 6108, 6418, 5509, 5541, 5438, 5442, 5508, 5508, 5508, 5508, 5486, 5515, 5454, 5468, 5472, 5491, 5543, 5496, 5508, 5508, 5538, 5446, 5582, 5487, 5506, 5581, 5513, 5475, 5519, 5535, 6805, 6808, 5547, 6811, 6814, 6815, 6816, 5551, 5554, 5558, 5569, 5573, 5579, 5492, 5586, 5590, 5594, 5598, 5605, 5604, 5601, 5609, 5613, 5561, 6981, 5276, 5276, 5276, 5308, 5312, 7372, 5276, 5276, 5276, 5309, 7137, 5276, 5276, 5276, 5329, 5276, 6389, 5276, 5276, 6557, 5276, 6391, 5276, 5276, 5275, 5276, 6392, 6392, 5276, 5276, 5279, 5276, 6282, 5276, 5276, 5276, 5366, 5276, 6000, 6885, 5910, 7281, 5626, 5297, 5629, 5276, 5276, 7348, 5276, 5276, 7354, 5276, 5276, 5276, 5631, 5640, 5899, 5276, 5647, 5276, 6987, 5263, 5186, 5654, 5657, 5659, 5660, 5664, 5668, 5673, 5672, 5677, 5679, 5680, 5680, 5684, 5686, 5693, 5688, 5700, 5688, 5697, 5689, 5704, 5276, 5276, 7355, 5276, 5276, 5276, 5648, 6027, 6933, 6281, 5898, 5276, 5276, 5276, 5367, 5276, 5276, 5276, 5368, 5400, 5708, 6023, 5276, 5276, 5281, 7411, 7471, 7426, 5276, 5276, 6043, 5135, 5276, 5276, 5281, 7468, 6467, 5276, 5716, 5276, 5276, 5721, 6061, 5339, 5276, 5276, 5304, 5276, 5341, 5737, 5745, 5751, 5749, 5276, 5276, 5276, 5369, 6219, 5751, 5276, 5276, 5306, 5310, 6765, 5276, 5276, 6988, 5276, 6526, 5277, 6886, 6113, 5784, 5276, 5793, 7239, 5808, 5855, 5812, 5831, 7055, 5318, 5276, 5276, 5276, 5372, 5348, 7452, 5827, 5832, 7056, 5137, 5276, 5276, 5276, 5376, 5368, 5276, 6525, 5276, 5414, 5276, 7452, 7443, 5276, 5276, 7419, 6200, 5276, 5925, 5276, 5276, 5276, 7249, 5717, 5276, 5276, 5276, 5387, 5795, 7243, 5854, 5845, 7241, 5851, 5855, 5846, 7242, 5852, 5856, 5317, 5850, 5854, 5845, 5898, 5860, 5276, 5276, 5276, 5388, 6362, 5276, 5854, 5876, 5318, 5276, 5276, 7441, 6519, 7240, 5851, 5855, 5877, 5898, 5881, 5839, 5276, 5276, 5370, 5276, 5276, 5853, 5885, 5318, 5276, 5276, 7452, 6134, 7239, 6135, 5854, 5886, 5898, 6404, 5870, 6865, 5276, 5276, 6117, 5869, 6864, 5898, 5276, 6867, 5276, 5276, 5276, 5617, 6117, 5890, 6866, 5276, 5276, 5276, 5620, 6857, 5276, 6719, 6723, 5276, 5276, 7455, 6886, 6721, 5898, 5276, 5276, 5373, 5276, 5914, 5276, 5898, 7023, 5934, 5938, 5942, 5945, 5948, 5949, 5949, 5950, 5954, 5954, 5954, 5954, 5954, 5958, 5961, 5276, 5277, 5921, 5930, 5276, 6659, 6079, 5966, 6872, 6872, 5276, 5276, 5276, 5650, 6487, 5276, 5711, 6194, 7445, 7444, 5368, 5276, 6000, 6525, 5276, 7025, 5302, 6259, 5276, 5370, 5979, 6510, 5997, 5276, 6523, 5276, 5276, 6660, 6080, 5967, 5898, 6890, 5997, 5276, 6889, 5276, 5276, 5276, 5841, 6010, 5276, 6606, 5276, 5276, 5276, 5822, 6016, 5276, 5276, 5276, 5787, 5276, 6605, 6017, 5276, 5277, 5962, 6001, 7450, 5276, 5276, 5276, 5826, 5831, 5712, 6195, 7052, 5276, 5368, 5276, 5276, 5366, 5369, 6121, 5276, 6513, 5276, 6559, 5276, 5276, 5386, 5282, 5276, 6093, 5276, 5276, 5276, 5864, 6029, 5276, 5276, 5276, 5896, 6848, 5998, 5276, 6523, 6792, 5276, 5276, 6397, 7112, 6033, 6047, 6053, 5898, 5276, 6034, 6048, 6054, 5276, 5278, 5281, 6211, 6035, 6049, 6055, 5276, 5278, 5909, 5276, 5279, 6836, 6888, 5276, 5276, 5276, 7452, 5761, 6526, 6233, 6268, 5276, 5280, 5303, 6536, 5276, 6268, 6140, 6038, 6070, 5276, 5276, 5400, 5400, 5400, 5400, 5401, 6066, 6588, 6039, 5894, 5276, 7456, 5276, 5276, 5276, 6002, 5276, 5276, 5276, 5975, 5277, 6078, 6084, 5367, 6266, 7445, 6236, 5999, 6524, 5276, 6100, 6037, 6070, 5276, 5276, 5477, 5481, 6630, 6587, 6090, 5894, 5276, 6080, 6086, 5276, 5276, 5478, 5482, 6099, 5276, 5276, 5276, 6003, 5276, 5276, 6036, 6107, 5276, 5276, 5276, 6072, 6033, 6588, 6106, 5276, 5276, 5276, 6073, 6085, 5276, 5276, 7023, 6099, 5276, 5276, 5281, 6583, 5728, 5732, 6105, 5894, 5729, 5733, 6106, 5276, 5281, 6629, 6586, 5727, 5731, 6104, 6108, 5276, 5276, 5276, 6117, 5890, 5371, 6000, 6125, 6268, 6456, 5276, 6388, 5276, 5276, 5276, 6523, 5276, 7417, 6416, 5276, 5276, 5276, 6119, 5276, 6524, 6525, 6659, 6322, 5276, 5276, 5276, 6074, 5363, 7445, 5276, 5370, 6000, 6236, 5276, 6320, 5799, 5803, 5276, 5276, 5276, 6127, 6319, 5798, 5802, 5276, 5276, 6321, 5276, 7260, 7446, 5368, 6001, 6320, 5799, 5801, 5276, 5276, 6319, 5797, 5276, 7073, 5276, 7258, 7447, 5276, 7447, 5302, 7075, 5803, 5276, 5276, 7072, 7076, 5276, 5281, 7468, 7477, 5276, 5276, 7073, 7260, 7446, 5276, 7074, 5802, 5276, 5285, 5289, 7173, 6339, 7258, 7447, 6339, 7077, 5276, 6117, 7076, 5276, 6342, 5276, 6117, 6342, 6139, 6117, 6200, 6978, 5276, 6978, 5276, 6978, 5276, 7445, 6197, 5276, 6237, 6722, 5276, 5276, 5372, 7365, 6199, 6197, 5276, 6199, 5276, 5925, 7445, 6198, 6198, 6198, 5276, 5276, 5926, 6198, 5276, 6896, 6896, 5389, 6536, 5368, 5525, 6145, 5276, 5276, 5276, 6133, 6358, 5367, 5276, 5276, 6198, 6537, 6199, 6536, 6200, 5276, 6537, 5276, 5276, 5276, 6480, 6167, 6152, 6171, 6175, 6179, 6181, 6185, 6185, 6181, 6189, 6189, 6189, 6189, 6189, 6191, 5276, 5276, 5564, 6363, 6206, 5276, 5276, 5276, 6199, 6200, 6251, 5898, 5276, 5276, 5565, 5371, 5276, 5276, 5575, 6934, 6200, 5276, 7454, 5803, 7356, 5276, 6293, 5276, 5804, 5276, 5287, 5291, 7160, 7066, 6196, 5276, 5285, 6217, 6223, 7106, 6232, 5276, 5287, 7149, 7129, 6243, 5276, 7260, 5276, 5288, 7150, 7141, 6257, 5276, 5789, 5276, 5276, 5276, 6202, 6263, 5276, 5276, 5276, 6201, 5367, 5369, 5387, 6274, 6253, 5278, 6018, 5276, 5276, 5642, 5276, 5276, 5276, 6388, 6113, 5193, 6286, 6935, 6040, 6287, 6936, 6041, 5276, 5276, 5191, 5195, 6288, 6937, 6042, 5276, 5276, 5276, 6226, 5385, 6114, 6252, 5276, 5307, 5311, 5315, 7420, 7354, 5276, 5276, 5739, 7397, 7403, 6298, 6329, 5803, 5276, 5276, 6006, 6299, 6330, 5276, 5276, 5276, 6265, 6300, 6331, 5276, 5276, 5753, 5752, 5387, 6275, 5897, 5276, 5328, 6552, 6547, 5803, 6197, 7453, 7421, 6267, 6392, 5479, 6312, 6329, 5276, 6458, 5480, 6313, 6330, 5276, 5276, 5764, 5276, 5385, 6246, 6253, 5276, 5363, 5367, 5276, 5276, 6536, 6317, 6267, 5276, 5276, 5276, 5621, 6392, 7412, 6327, 6332, 6301, 5803, 5276, 5276, 5815, 7444, 6522, 5276, 5276, 5375, 5276, 5276, 5365, 5276, 5276, 5916, 5276, 5276, 5276, 6294, 5368, 5386, 6336, 5276, 5365, 5276, 6535, 5276, 6820, 7444, 5276, 5276, 6519, 6308, 5276, 6198, 5276, 7452, 5280, 5276, 5276, 5276, 6307, 6197, 5276, 6843, 5276, 6356, 6250, 6678, 5276, 5276, 5276, 6368, 5276, 6844, 5894, 5276, 5369, 5998, 6524, 5276, 5276, 5276, 5277, 6208, 6362, 5276, 5276, 5276, 6371, 5276, 6846, 5276, 5276, 5898, 5276, 6005, 5276, 5276, 6005, 5276, 5276, 6846, 6004, 6002, 6846, 6005, 6005, 6005, 5276, 5276, 6004, 5276, 5276, 6005, 6847, 5276, 5276, 5995, 6375, 5894, 5276, 6520, 6383, 5276, 5276, 5276, 6404, 5871, 6112, 7024, 6387, 6111, 5276, 5276, 5276, 5363, 6401, 6761, 6409, 6972, 6429, 6432, 6433, 6434, 6438, 6438, 6438, 6438, 6441, 6443, 6444, 6444, 6451, 6451, 6451, 6451, 6448, 6452, 5276, 5276, 5276, 6425, 5348, 6566, 5276, 5276, 5276, 6463, 6370, 6494, 5276, 5276, 5973, 5276, 5362, 6468, 5276, 6469, 6475, 5276, 5276, 5276, 6519, 5276, 6484, 5276, 5276, 6000, 5389, 6228, 6477, 5276, 5276, 6012, 5276, 6112, 5276, 5276, 5276, 6520, 5276, 5276, 5276, 6396, 5304, 6197, 5276, 5276, 6060, 6059, 5276, 6499, 6507, 5276, 5386, 5283, 5276, 5276, 6395, 5276, 5276, 5279, 6208, 6518, 5276, 5276, 5359, 5276, 5276, 5276, 6524, 5276, 5276, 6534, 6534, 5276, 5388, 5276, 5284, 6546, 6618, 5276, 5276, 6109, 5903, 6553, 6548, 5276, 5276, 6116, 5998, 5349, 6567, 5276, 5276, 5276, 6392, 6611, 5305, 6199, 5276, 6537, 6265, 5276, 6236, 6525, 5276, 6830, 6573, 5818, 5276, 6831, 5501, 5819, 5276, 6832, 5502, 5820, 6830, 5500, 5818, 5276, 5422, 6847, 5276, 5522, 7448, 5276, 5636, 6001, 5998, 6565, 5276, 5276, 5276, 6525, 5276, 5276, 7449, 6887, 6493, 5276, 5641, 6843, 5276, 5276, 5276, 6237, 6537, 5276, 6268, 6140, 5276, 6390, 5276, 6459, 6571, 6577, 5821, 5276, 5276, 5276, 6657, 5276, 7285, 5499, 6578, 5276, 5276, 6886, 5279, 5276, 5649, 6028, 6094, 5276, 6458, 5276, 5276, 5276, 6526, 7024, 6877, 5256, 6579, 5276, 5276, 5276, 6528, 6598, 5276, 5276, 5276, 6536, 5276, 6592, 5276, 5276, 5276, 6620, 5276, 6391, 6459, 5276, 5739, 7123, 6938, 5281, 6583, 6579, 5276, 5741, 7125, 6332, 6602, 5276, 6459, 5276, 5741, 7125, 6522, 5276, 6404, 5871, 5280, 5276, 5302, 5276, 5757, 5276, 5356, 6392, 6526, 5276, 5276, 6118, 5279, 6610, 5276, 5276, 5276, 6783, 5367, 6615, 5276, 5276, 5276, 6784, 5276, 6919, 6923, 5276, 5777, 5276, 5276, 5327, 6542, 5280, 6537, 6266, 5276, 5788, 6264, 5276, 5288, 7184, 7189, 5276, 5277, 5276, 5367, 5276, 6268, 6141, 6712, 6424, 5276, 5276, 6126, 5898, 5276, 5276, 5632, 6626, 6634, 5276, 5276, 6131, 6414, 6108, 5276, 5276, 6395, 5276, 6844, 5276, 5276, 5917, 5276, 6521, 5276, 6395, 6522, 6729, 6846, 5276, 5375, 6502, 6651, 6664, 6672, 6682, 6685, 6689, 6693, 6693, 6693, 6695, 6697, 6697, 6697, 6697, 6701, 6702, 6702, 6702, 6703, 6707, 5276, 6711, 6423, 5276, 5276, 5276, 6735, 5276, 6716, 5276, 7351, 5276, 6727, 5276, 7420, 6268, 6969, 6745, 5276, 5276, 5276, 6785, 5450, 5276, 5276, 5276, 6793, 6743, 6094, 5276, 6750, 6751, 5276, 5276, 5276, 6844, 6212, 5276, 5276, 5276, 6847, 5276, 5276, 6755, 5276, 5276, 6238, 5276, 6524, 6798, 6267, 6759, 6771, 6766, 5276, 5276, 5276, 6877, 5194, 6287, 5276, 7275, 5276, 5276, 6248, 5276, 5276, 6776, 6772, 6767, 6790, 5276, 6792, 5276, 5907, 5276, 6269, 5894, 7260, 7023, 5276, 5276, 5276, 6111, 6392, 6422, 6847, 5276, 6100, 5276, 6888, 5276, 5276, 6521, 5276, 5276, 6194, 7446, 5368, 7418, 5276, 5276, 6293, 5276, 5276, 5276, 6269, 6206, 7033, 6523, 6797, 6266, 5312, 5316, 5276, 5276, 5276, 6921, 5313, 5317, 5276, 5276, 5276, 6933, 6792, 6791, 5276, 5276, 6305, 5304, 6420, 6424, 5276, 5276, 6319, 6922, 5276, 5276, 6118, 5276, 5374, 5276, 5276, 6535, 5276, 5276, 7034, 7452, 6826, 5276, 5908, 5278, 6495, 5276, 6519, 5276, 5276, 5276, 5281, 5255, 6598, 5276, 7033, 5863, 6828, 6852, 5314, 5318, 5276, 5276, 6913, 5276, 5276, 5276, 6958, 6858, 5318, 5276, 5276, 6321, 5800, 6857, 5317, 5276, 5276, 6340, 5802, 6535, 5276, 6862, 5276, 6022, 5753, 5276, 5278, 5276, 5276, 7210, 5276, 6871, 5276, 5276, 5276, 7191, 6199, 6200, 5276, 5276, 5276, 7042, 6425, 5276, 5276, 6876, 6887, 5276, 5367, 6820, 7444, 5276, 6002, 5276, 6194, 7052, 5276, 6637, 6520, 5276, 6536, 5276, 6822, 5276, 5276, 6341, 5276, 7204, 5276, 5276, 5276, 7203, 5367, 5276, 6786, 5276, 5276, 6392, 6921, 6821, 5276, 5276, 6904, 5276, 5276, 6388, 6391, 6480, 5276, 5276, 5276, 7239, 5276, 6478, 5367, 5276, 6095, 5276, 5276, 6394, 5276, 6897, 5276, 5276, 6883, 6535, 5276, 5371, 6904, 5364, 5276, 6894, 5276, 6110, 5276, 5276, 5981, 7461, 7451, 5276, 5276, 5276, 5983, 7470, 5990, 5283, 6536, 6901, 5276, 6114, 5276, 6001, 6911, 5374, 6535, 5368, 5803, 5276, 6392, 6412, 5369, 5276, 5276, 5366, 5364, 6537, 6364, 5364, 5276, 5367, 5371, 5366, 5364, 5276, 5276, 6536, 5365, 5276, 5366, 5276, 5366, 5367, 6538, 6538, 6538, 5773, 5767, 5769, 5771, 5771, 5772, 5276, 7445, 5183, 5276, 5282, 5917, 7454, 6113, 6917, 6942, 7063, 5336, 6951, 6962, 6966, 5531, 6992, 6995, 6995, 6998, 7003, 7002, 7002, 7004, 7008, 7009, 7015, 7013, 7013, 7013, 7013, 7019, 7022, 7030, 5276, 5276, 6115, 5375, 5528, 6927, 5276, 7445, 6842, 7453, 6149, 6156, 6163, 7038, 6983, 5276, 5276, 6392, 7229, 5277, 5276, 5276, 7046, 7060, 5276, 7070, 5276, 6116, 5999, 5276, 6117, 5276, 5276, 5364, 5276, 5276, 5276, 7240, 7085, 5276, 5276, 5276, 7337, 5349, 5276, 6906, 7086, 5276, 6117, 5368, 5276, 5276, 6907, 5276, 7090, 5276, 6120, 5276, 5368, 5389, 6113, 7094, 5276, 5276, 5276, 7338, 6840, 5276, 6782, 7444, 6985, 5276, 5896, 5276, 6197, 5276, 5276, 7024, 5276, 7104, 5277, 5276, 5276, 6947, 6946, 5276, 6378, 5276, 5276, 5276, 7356, 5276, 5276, 7356, 6396, 7111, 7118, 6379, 5276, 7119, 6094, 5276, 5276, 6405, 5872, 5898, 6956, 6954, 5276, 5276, 6422, 6847, 7144, 5276, 5276, 5276, 7363, 5285, 5289, 7113, 7142, 5286, 7148, 7114, 7143, 6201, 5367, 5276, 5276, 6458, 5480, 6298, 7452, 7154, 5276, 5895, 5290, 7159, 7164, 5318, 7066, 5318, 5276, 5276, 6458, 5481, 6301, 6676, 5276, 5276, 6199, 5276, 5276, 5276, 6879, 7180, 5276, 5276, 5276, 7364, 5290, 7174, 7179, 5276, 5276, 7178, 5276, 5276, 5276, 7380, 5740, 7169, 5276, 5276, 6467, 5276, 6677, 6197, 5276, 5924, 7196, 6975, 5276, 5276, 6467, 7049, 5285, 7195, 7185, 6391, 6116, 5276, 5276, 5276, 7208, 7454, 5276, 5276, 6473, 6477, 6526, 5276, 7449, 5276, 6197, 6520, 5276, 7446, 5276, 5375, 7224, 5276, 5276, 5276, 7381, 7214, 5276, 5276, 5276, 7408, 7223, 6847, 5276, 5276, 6479, 5276, 7222, 6425, 5276, 6201, 6886, 6492, 7200, 5276, 6237, 5276, 5276, 6841, 6675, 5276, 5276, 5387, 5276, 5276, 6488, 5276, 6878, 6781, 5276, 5276, 6514, 5276, 5276, 6879, 7230, 5276, 5276, 5276, 7446, 7228, 5276, 5276, 6249, 7452, 5276, 5389, 5276, 6198, 6197, 5276, 6000, 5276, 6238, 5276, 6524, 6201, 5276, 6201, 7449, 5276, 6193, 7445, 5276, 5280, 6209, 6213, 5276, 6520, 5276, 5281, 6065, 6587, 6879, 7455, 5276, 5276, 6525, 6526, 6659, 6079, 7450, 5276, 6395, 7455, 6621, 6847, 5276, 7238, 5276, 6622, 5276, 5276, 6526, 7416, 5276, 6622, 5276, 7247, 6620, 7455, 5276, 5276, 6530, 5276, 7391, 6527, 6621, 6847, 5276, 5276, 6537, 5276, 5276, 6536, 6621, 6847, 7455, 5387, 5388, 6847, 5388, 7344, 5276, 5276, 7229, 7453, 5386, 5423, 5389, 5276, 5276, 6593, 5276, 5388, 5276, 5386, 5388, 7253, 5276, 7258, 7446, 6196, 5276, 5363, 5276, 6210, 6887, 5276, 5894, 5276, 5276, 5276, 5971, 7257, 5276, 5916, 6640, 7264, 6258, 5276, 5189, 5276, 7260, 5916, 7259, 6503, 7097, 7283, 7273, 5724, 7100, 7135, 7279, 6930, 7132, 7289, 7293, 7302, 7295, 7297, 7306, 7298, 7317, 7310, 7311, 7315, 7318, 7326, 7326, 7328, 7324, 7322, 7332, 7335, 5276, 6218, 5750, 5276, 5276, 7362, 7366, 7364, 7342, 5276, 6467, 5389, 5281, 5276, 5276, 6621, 5276, 5276, 6393, 5276, 5915, 5276, 6227, 6476, 5276, 5377, 6746, 5276, 5386, 5276, 5276, 5276, 7449, 5276, 5276, 7360, 7370, 7376, 7385, 5276, 5276, 5276, 7450, 5276, 5276, 6529, 7386, 5276, 6265, 5276, 5276, 5378, 6094, 7392, 5276, 5276, 5276, 7451, 7342, 7445, 6201, 5276, 6266, 5276, 5276, 5382, 5400, 5400, 5400, 5427, 6389, 6113, 5276, 5276, 6235, 6234, 5276, 5276, 5276, 7457, 5276, 6526, 6716, 7433, 7396, 7402, 5276, 5276, 6733, 6739, 7403, 5276, 5276, 5276, 7466, 7408, 7398, 7404, 6844, 5276, 6847, 5276, 6267, 6140, 5276, 5388, 7230, 5276, 5414, 5400, 5400, 5400, 5428, 7342, 7447, 7451, 5276, 6268, 5276, 6237, 6268, 5276, 5276, 7356, 6110, 6268, 5276, 5276, 5276, 5643, 7409, 5986, 7425, 5276, 6292, 5276, 5276, 5740, 7124, 7030, 7410, 5987, 7426, 5276, 6306, 5305, 5276, 5276, 6201, 5276, 5276, 5276, 5739, 7168, 7411, 5988, 7427, 5276, 6323, 7446, 6117, 5276, 5371, 5276, 5276, 7454, 5276, 6887, 5276, 5985, 5989, 6236, 6846, 6003, 5276, 6004, 5276, 6004, 6002, 5276, 6845, 5276, 5276, 5276, 6802, 5311, 6560, 7449, 5276, 5276, 5281, 7200, 5276, 6393, 5276, 7452, 5276, 5276, 5276, 6388, 6113, 5276, 6233, 5836, 5898, 5276, 5276, 7387, 5868, 7434, 5276, 5276, 5276, 6877, 7228, 6201, 5276, 7452, 5962, 5276, 5276, 5276, 6853, 5315, 5898, 5276, 5276, 7431, 5276, 5276, 5276, 6878, 7229, 5984, 7471, 5991, 5276, 6340, 5803, 5276, 5780, 5276, 5990, 5276, 5276, 5276, 6920, 5821, 5982, 7469, 5989, 5276, 6346, 6351, 5276, 5276, 6388, 6113, 5276, 5276, 6946, 5276, 5276, 7438, 5276, 5276, 7024, 5276, 5276, 5276, 7467, 7476, 5276, 5276, 7477, 5276, 5276, 5276, 7025, 5276, 7450, 5276, 5388, 5281, 6279, 5276, 5276, 5622, 5315, 5276, 6389, 6114, 5276, 6347, 6352, 5276, 5276, 5276, 7444, 5276, 5276, 5276, 6945, 7466, 7475, 5276, 5276, 7041, 5276, 5631, 5276, 5388, 5282, 7466, 7462, 5276, 5276, 5276, 7040, 5276, 5276, 5276, 7081, 5276, 5303, 5276, 5276, 5276, 7110, 6654, 0, 0, 1075838976, 2097152, -1845493760, 0, 0, 0x80000000, 16777216, 4194560, 4196352, 270532608, 2097152, 2097152, 268435456, 4194432, 16777216, 4194432, 3145728, 541065216, -2143289344, 4194304, 4194304, 4194304, 4194304, 541065216, 4194304, 4194304, 4194432, 37748736, -1606418432, 541065216, 541065216, 541065216, 541065216, 4194304, 4194304, 4196352, -1606418432, -1606418432, 541065216, 541065216, 4194304, 4198144, 541065216, 541065216, -2143289344, -2143289344, 8425488, 4194304, 4194304, 4194304, 1, 32768, 0, 0x80000000, 16, 33554432, 33554432, 0, 2, 4, 112, 128, 256, 3584, 16384, 37748736, 742391808, 239075328, 775946240, 775946240, 775946240, 171966464, 171966464, 171966464, 171966464, 239075328, 171966464, 775946240, 239075328, 4718592, 64, 4718592, 2097216, 4720640, 541589504, 4194368, 541589504, 4194400, 4194368, -2142763008, 541589504, 541065280, 4194368, 4194368, 541065312, 541065280, -2143289280, 4194368, -2143285408, -2143285408, -2143285440, -2143285440, -2143285440, -2143285440, -1605890240, -2142761152, -2109731008, -1606414528, -2142761152, -1606414528, -2143285440, -2143285440, -1606414528, -1606414528, -2143285440, -2109731008, 776470528, -1908404416, 775946304, -1908404416, 2, 4, 8, 16, 512, 1024, 4194304, 128, 128, 0, 0x80000000, 524288, 5242880, 0x80000000, 0, 0, 2, 12, 64, 256, 2048, 2048, 0, 0, 0, 0, 1, 0, 0, 0, 2, 0, 0, 0, 3, 4, 16, 224, 256, 512, 1024, 16777216, 16777216, 0, 0, 0x80000000, 65536, 1792, 0, 0, 0, 8, 0, 0, 0, 12, 32, 64, 1024, 2048, 57344, 262144, 50331648, 268435456, 1073741824, 0x80000000, 0, 0, 1536, 64, 524352, 524352, 524352, 0, 0, 0, 62, 64, 128, 64, 262144, 1048576, 0, 0x80000000, 2097152, 0, -2113929216, 0, 0, -1979711488, -1912602624, 64, 64, 1048576, 128, 256, 2048, 262144, 524288, 4096, 1024, 1024, 0, -570425344, 32505856, 0, 0, 48, 25165824, 0, 0, 0, 33554432, 268435456, 0, 0, 0, 524288, 0, 0, 0, 32, 0, 0, 0, 44, 64576, 319029248, 0, 524288, 524288, 524288, 0, 64, 0, 0, 0, 64, 0, 96, 96, 96, 524352, 524352, 524352, 524352, 524288, 524288, 524288, 524288, 64, 64, 0, 64, 128, 128, 128, 128, 2048, 2048, 0, 0, 524288, 524288, 64, 64, 128, 2048, 0, 0, 64, 128, 8388608, 524288, 64, 64, 64, 64, 32, 96, 64, 96, 96, 524352, 96, 160, 1056, 262176, 1048608, 2097184, 4194336, 536870944, 40, 262176, 32, 96, 0, 0, 0, 60, 40, 48, 1120, 96, 96, 64, 524352, 0, 524288, 64, 96, 524352, 0, 524288, 4195360, 6291488, 2097184, 2097184, 4194336, 4194336, 4194336, 32, 56, 0, 4, 16, 32, 64, 128, 256, 1536, 2048, 40, 262184, 40, 40, 40, 40, 40, 262176, 32, 32, 6292512, 4195360, 2097184, 32, 128, 1792, 8192, 16384, 131072, 524288, 4195104, 6292512, 32, 32, 32, 32, 4, 40, 262184, 32, 32, 34, 34, 262184, 40, 4196128, 32, 262144, 524288, 0, 0, 64, 256, 0, 2097152, 135790592, 131073, 4, 393233, 262184, 34, 42, 32, 524320, 32, 1073872896, 32, 32, 40, 1120, 96, 1056, 4194336, 32, 2098208, -322695456, -322695456, -322695456, -322695456, -322597152, -320598176, -322597152, -322597144, -321548576, -320598168, -322597144, -321548568, -37482773, 0, 0, 64, 1536, 32768, -322588952, -321548568, -322588952, -321548568, -322597144, 32, 0, 32, 64, 65536, 0, 96, 32, 32, 56, 262184, 40, 41, 262184, 32, 42, 224, 40, 262176, 42, 106, 293601323, 293601323, 293863467, 293699627, 293617707, 293716011, 293702203, 293702203, 293702203, 297896507, 293702203, 293702203, 293702203, 293702267, 293964347, 293702267, 297896507, 293964347, 297896507, 297896507, -322597144, -37744981, 0, 32, 524288, 0, 0, 64, 2048, 16384, 32768, 0, 0, 262144, 65536, 262144, 262144, 0, 4096, 0, 8, 0, 2, 65536, 262656, 328192, 0, 0, 0, 1536, 32768, 0, 5242880, 0, 0, 0, 1998, 518144, 8388608, -2147418112, 5242880, -1842937664, 201330721, 201330721, -2111369023, -2111369023, -2111369023, -2111369023, -2111360575, -2111369023, -2111369023, -2111369023, -1977151295, -1977151293, -1910042431, -1893265183, -2111368509, -1893265183, -1893265183, -1893265183, -1893265183, -2111368509, -1893265183, -1893265183, -553689472, -553656704, -553689472, -553689472, -553656704, -553656704, -553656704, -553656704, -553656704, -553656672, -553656672, -553656672, -553656672, -536912159, -553656672, -553656672, -553656664, -553656664, -553656672, -553656672, -553656670, -553656672, -553656672, -553656670, -553656608, -553656671, -536879391, -536879391, -536879391, 0, 0, 262656, 0, 0, 65, 1024, 0, 1, 4096, 201326592, 0, 0, 0, 0, 462976, -2113929216, 100663296, 100663296, 2, 4, 8, 64, 128, 512, 2048, 8192, 16384, 458752, 18874368, 463488, 0, 0, 1, 2, 4, 32, 0, 0, 1007232, 15728640, 1040000, 15728640, -570425344, 0, 0, 0, 2014, 0, 0, 0, 32505856, -570425344, 196608, 2097152, 301989888, 0, 0, 80, 268435456, 0, 268435456, 0, 268435456, 268435456, 268435456, 268435456, 0, 0, 0, 4096, 201326592, 0, 0, 96, 2260992, 12288, 0, 0x80000000, 0, 0, 118, 577408, 22020096, 1040187392, 0, 0, 167772160, 234881024, 128, 512, 2048, 196608, 262144, 33554432, 536870912, 0, 0, 0, 557056, 7168, 16384, 196608, 786432, 503316480, 1073741824, 0x80000000, 0, 0, 128, 131072, 524288, 58720256, 402653184, 0, 0, 0, 318767104, 128, 512, 7168, 16384, 32768, 32768, 196608, 786432, 1048576, 2097152, 4194304, 33554432, 268435456, 536870912, 0x80000000, 0, 0, 1, 256, 8388608, 234881024, 268435456, 1073741824, 0x80000000, 4096, 16384, 32768, 131072, 524288, 1048576, 2097152, 4194304, 8388608, 234881024, 0, 0, 201326592, 0, 0, 128, 536870912, 4194304, 512, 3072, 16384, 131072, 524288, 1048576, 4194304, 134217728, 8388608, 33554432, 201326592, 268435456, 1073741824, 0, 1048576, 4194304, 268435456, 4194304, 8388608, 134217728, 268435456, 1073741824, 3072, 131072, 524288, 1048576, 1073741824, 0, 0, 0, 0x80000000, 0, 0, 0, -2147483646, 16384, 18432, 67108864, 1073741824, 16384, 8192, 0, 0, 65536, 262144, 0, 0, 67108864, 0, 0, 0, 32768, 0, 1, 18952, 1024, 0, 0, 192, 0, 0, 0, 65, 1024, 1024, 100663298, 18952, 65, 268436480, 2101248, 524288, 1024, 19017, -1744550912, 8388624, 8388624, 8388624, -1739308032, -1739308032, -1739308032, -1739308032, -1736162288, -1736162288, -1736162288, -1736162288, -7868466, -7868466, -7868466, -7868466, -7868466, -7868450, -7868450, -7868450, -7868450, 0, 0, 0, 65536, 2048, 16384, 67108864, 134217728, 268435456, 0, 0, 1073741824, 18432, 0, 0, 1, 285212672, 0, 585, 0, 0, 2, 8, 16, 64, 128, 3072, 4096, 8192, 65536, 131072, 0, 0, 1024, 5521408, -1744830464, 0, 0, 0, 262144, 0, 0, 0, 1024, 0, 0, 0, 112, 1040, 0, 0, -1744830464, 0, -1744830464, 59238400, -67108864, 0, 0, 0, 327680, 2014, 0, 0, 0, 328192, 518144, 8388608, 50331648, 201326592, 805306368, -1073741824, 768, 1024, 10240, 16384, 32768, 458752, 50331648, 67108864, 134217728, 805306368, 0, 0, 0, 458880, 32768, 458752, 8388608, 50331648, 67108864, 134217728, 67108864, 134217728, 805306368, 1073741824, 0x80000000, 0, 0, 220, 0, 0, 0, 471424, 12, 192, 768, 1024, 2048, 805306368, 1073741824, 0, 0, 2, 204, 768, 1024, 4, 8, 32, 64, 512, 2048, 512, 2048, 16384, 67108864, 0, 0, 458752, 50331648, 67108864, 805306368, -1073741824, 0, 0, 0, 393240, 0, 1048576, 4194304, 0, 0, 16384, 458752, 50331648, 67108864, 536870912, 1073741824, 0, 0, 16384, 0, 0, 0, 256, 0, 0, 0, 512, 0, 0, 0, 585, 16, 0, 0, 1048576, 4194304, 0x80000000, 4, 8, 128, 512, 3072, 16384, 32768, 131072, 512, 0, 0, 4096, 1048576, 0x80000000, 8192, 2097152, 268435456, 0x80000000, 537395200, 537395200, 0, 4196352, 51380242, 51380242, 51380242, 0, 537395200, 4196352, 4196352, 276901888, 8540160, -1606418432, 0, 32768, 537395200, 4196352, 1082130432, 0, 4196352, 537427968, 22022147, 22349827, 22349827, 22349827, 22366219, 22349843, 22349827, 22349827, 22366219, 22349827, 55576594, 55576594, 55576594, 55576594, 55576594, 324012114, 55576594, 55576594, 1062785014, 1062785014, 1062785014, 1062785014, 0, 0, 2, 33554432, 0, 0, 0, 131072, 0, 0, 0, 8192, 8392704, 32768, 268435456, 0, 0, 2, 67108864, 12, 16384, 0, 65536, 0, 22020096, 0, 0, 0, 1040000, 15728640, 0, 0, 329728, 0, 0, 254, 1792, 2809856, 58720256, 19, 0, 0, 0, 1048576, 0, 0, 0, 2048, 0, 0, 0, 18, 33554432, 0, 0, 256, 8192, 0, 0, 8192, 2097152, 0, 0x80000000, 0, 82, 301989888, 0, 0, 0, 2101248, 22020096, 1040187392, 0, 0, 0, 4194304, 0, 0, 0, 16384, 1536, 0, 256, 0, 8192, 2097152, 16, 1048576, 16777216, 33554432, 268435456, 536870912, 0x80000000, 3584, 16384, 32768, 524288, 1048576, 4194304, 0, 0, 8388608, 1073741824, 0, 0, 1536, 2048, 16384, 32768, 524288, 4194304, 134217728, 0, 0, 134217728, 4096, 0, 8, 0, 256, 1536, 16384, 32768, 524288, 128, 536870912, 0, 0, 4, 8, 512, 2048, 0, 0, 1536, 32768, 524288, 4194304, 33554432, 134217728, 536870912, 0, 0, 8192, 2097152, 0x80000000, 0, 0, 512, 2048, 131072, 536870912, 0, 0, 16, 64, 1536, 32768, 32768, 524288, 134217728, 0, 0, 0, 524288, 0, 64, 64, 16392, 1536, 32768, 524288, 0, 0, 33554432, 8192, 0, 65536, 0, 0, 1, 1, 0, 1, 67174400, 33554432, 536870912, -1073741824, 0, 0, 0, 0, 524288, 134217728, 67174400, 0, 0, 0, 16777216, 0, 0, 0, 4, 0, 0, 0, 7, 16, 16384, 8, 8, 0, 0, 512, 3072, 131072, 131072, 268435456, 134217728, 8, 512, 2048, 196608, 262144, 50331648, 536870912, 1073741824, 0, 4, 8, 2048, 8192, 32768, 8388608, 0, 0, 134217736, 16908320, 547389524, 547389524, 555909216, 555909216, 555909216, 555909216, 564297840, 564297844, 564297844, 564297844, 564297844, 1001055742, 1001056254, 1001055742, 1001055742, 1001056254, 1001056254, 1001056254, 1001056254, 1001055742, 1001056254, 1001056254, 1001056254, 1001056254, 0, 1052672, 0x80000000, 0, 0, 4, 16, 0, 0, 84, 2129920, 8388608, 4096, 0, 0, 0, 116, 0, 254, 1280, 2809856, 58720256, 939524096, 0, 0, 0, 50331648, 268435456, 0, 0, 0, 939524096, 0, 0, 520, 1024, 0, 0, 1, 0, 67108864, 1073741824, 0, 0, 0, 20, 64, 32768, 8192, 0, 2048, 0, 2097152, 8388608, 536870912, 0, 0, 1024, 278528, 0, 0, 0, 393232, 163840, 0, 0, 0, 134217728, 0, 0, 0, 16, 0, 0, 0, 15, 208, 15360, 1245184, 52, 0, 0, 0, 268435456, 0, 0, 33554432, 64, 128, 1280, 24576, 163840, 524288, 2097152, 58720256, 402653184, 536870912, 128, 1792, 24576, 163840, 524288, 0, 4, 16, 8388608, 0, 0, 4096, 32768, 262144, 524288, 33554432, 134217728, 0, 0, 24, 32, 128, 1280, 8192, 16384, 8192, 524288, 16777216, 33554432, 402653184, 0, 4, 8, 16, 1024, 2048, 8192, 16384, 32768, 458752, 0, 262144, 33554432, 134217728, 0, 512, 1024, 16777216, 33554432, 402653184, 0, 4096, 1048576, 0, 0, 1998, 59238400, -67108864, 4, 8, 16, 402653184, 0, 0, 8, 16, 402653184, 536870912, 0, 0, 4, 64, 128, 8388608, 0, 0, 67108866, 12, 64, 128, 512, 1024, 4, 16384, 65536, 67108864, 0, 65536, 0, 0, 2048, 64, 64, 64, 96, 96, 96, 96, 0, 8192, 8192, 268435460, 32768, 65536, 2490368, 16777216, 0x80000000, 0, 0, 1, 4, 8, 2048, 8192, 3670016, 2048, 2048, 2048, 2048, 0, 8192, 34816, 9216, 4096, 0, 128, 0, 2097152, 0, 0, 4096, 4096, 29696, 29712, 29840, 29712, 29712, 29840, 29840, 536900624, 4224144, 144384, 144384, 144384, 144384, -754647956, -754647956, -754647956, -754647956, -754647956, -754647940, -754647940, -754647940, -754647940, -754516884, -754647956, -754516884, -754516884, -754516884, 4, 8, 256, 512, 2048, 0, 0, 3670016, 0, 0, 2048, 131072, 524288, 4194304, 0x80000000, 0, 0, 9216, 0, 0, 4, 134217728, 0, 29696, 0, 0, 4, 268435456, 0, 16, 0, 29824, 0, 60, 64576, 319029248, -1073741824, 0, 0, 319160320, 0, 0, 0, 319160320, 0, 0, 524288, 3145728, 0, 12288, 131072, 0, 8, 131072, 61440, 262144, 318767104, -1073741824, 0, 0, 64, 1024, 2048, 61440, 262144, 0, 28, 32, 64, 64, 64, 128, 0, 0, 16384, 32768, 50331648, 268435456, 0, 0, 0, 393216, 0, 0, 0, 486539264, 0, 128, 0, 536870912, 0, 0, 12, 16, 32, 327155712, 34, 1056, 32, 32, 42, 4457568, -326784344, -322851160, -322698144, -322698144, -322698144, -322698144, -322695456, 0, 0, 524288, 1048576, 0, 0, 0, 536870912, 4194304, 131072, 0, 0, 6, 56, 128, 1792, 2, 67108864, 16384, 0, 0, 4096, 4194304, 32768, 0, 0, 0, 8388608, 0, 0, 0, 72, 0, 8, 64, 2048, 57344, 16384, 32768, 262144, 50331648, 268435456, 0, 524288, 1048576, 2097152, 4194304, 134217728, 0x80000000, 0, 0, 50331648, 268435456, 0x80000000, 0, 0, 1, 0, 0, 2, 4, 16, 64, 1, 0, 2, 0, 0, 65536, 0, 0, 0, 1040, 8667136, 0, 131072, 131072, 0, 131072, 0, 131072, 0, 0, 524288, 536870912, 131072, 0, 0, 7, 27756528, -503316480, 0, 256, 0, 2048, 32768, 8388608, 262144, 2113536, 0, 0, 8, 16, 512, 402653184, 0, 0, 0, 256, 32768, 0, 0, 4224, 65536, 262144, 1048576, 4194304, 16777216, 33554432, 67108864, 134217728, 0, 0, 256, 262144, 0, 0, 8192, 268435456, 0, 0, 0, 4, 131073, 0, 0, 9728, 268435456, 0, 0, 16, 393216, 0, 131073, 131073, 33554624, 4, 0, 131073, 0, 0, 13312, 131072, 4194304, -2146430976, 131072, 2097152, 16777216, 0, 0, 512, 131072, 1048576, 2097152, 0, 1572864, 0, 0, 0, 0x80000000, 524288, 0, 1610612736, 1610612736, 1610612736, 393241, 393241, 393241, 393241, 805707793, 805707793, 1879449617, 805708049, 1879449617, 1879449617, 1879449617, 1879449617, -483948553, -475559945, -483948553, -475559945, -483948553, -483948553, -475559945, -475559945, -475559945, -475559945, -483948553, -483948553, -215504905, -475559945, -207116297, -207116297, 0, 0, 0, 1073741824, 0, 0, 2097152, 67108864, 134217728, 536870912, 0, 0, 24576, 0, 0, 0, 2113536, 0, 0, 8, 64, 0, 0, 0, 401424, 805306368, 0, 0, 28672, 0, 0, 4096, 2097152, 4194304, 8388608, 503316480, 1073741824, 0, 0, 1879048192, 0, 0, 32768, 2097152, 8388608, 16777216, 33554432, 0, 401680, 0, 0, 8, 512, 2048, 131072, 33554432, 536870912, 0, 0, 7, 19367920, -503316480, 27756528, -503316480, 0, 0, 0, 0, 0, 19376112, -234881024, 0, 27764720, -234881024, 0, 0, 32768, 33554436, 0, 0, 33554436, 24, 0, 0, 0, 11, 346112, 7, 16, 480, 1536, 32768, 65536, 393216, 10485760, 65536, 10878976, 16777216, 33554432, 536870912, 4, 32, 524288, 1048576, 33554432, 67108864, 65536, 393216, 2097152, 16777216, 262400, 65536, 4224, 4224, 0, 65536, 201326592, 0x80000000, 393216, 10485760, 16777216, 33554432, 1073741824, 0x80000000, 0, 16, 224, 256, 1536, 32768, 65536, 0, 16384, 2097152, 0, 0, 1024, 32768, 65536, 131072, 262144, 262144, 2097152, 16777216, 33554432, 4, 32, 524288, 134217728, 0, 512, 32768, 131072, 262144, 2097152, 2097152, 8388608, 16777216, 1073741824, 0, 0, 512, 32768, 131072, 2097152, 8388608, 8388608, 16777216, 0, 0, 16, 33554432, 4, 16, 224, 512, 32768, 4, 524288, 134217728, 0, 0, 32768, 50331648, 268435456, 4096, 32768, 0, 0, 16, 536870912, 16, 192, 32768, 8388608, 4096, 4096, 4096, 1536, 2, 4, 16, 192, 32768, 0, 16, 64, 128, 8388608, 0, 0, 0, 2, 4, 134217728, 4, 0, 0, 128, 512, 3072, 4096, 16384, 131072, 4, 128, 0, 0, 18, 17825792, 524288, 8388608, 33554432, 0x80000000, 8192, 0, 33554432, 0, 0, 33554432, 0, 0, 0, 268435456, 2, 4, 8, 262144, 262144, 1048576, 2048, 32768, 0, 0, 28, 0, 100663296, 4224, 65536, 65536, 262144, 33554432, 0, 2, 4, 24, -1072627712, 805306384, -1342177264, -1342177264, -1070006272, -1070006272, -1069989376, -1069989376, -258932720, -258932720, -258932720, -258932720, -225378288, -1069989376, -1069989360, -1065795072, -1061600768, -258932720, -225378288, -258932720, -258932720, 1260767, 34815199, 1260767, 34815199, 1260767, 34815199, 34815199, 1260767, 1260767, 34815199, 1260767, 1260767, 169032927, -1978450721, 169032927, -1978450721, 169032927, 169032927, 169032927, 169032927, 1242774751, -1978450721, -1978450721, -225231649, -1173144353, -225231649, -91013921, 0, 0, 32, 128, 256, 262144, 524288, 8388608, 0, 64, 0, 0, 1114112, 1073741824, 0, 0, 34816, 0, 0, 2048, 4194304, 0, 0, 0, 3735552, 0, 0, 32, 512, 2048, 32768, 262144, 524288, 3751936, 0, 0, 0, 48, 0, 0, 528, 7946240, 12140544, 9502720, 1610612736, 0, 0, 0, 15360, 1245184, 0, 0, 134217728, 128, 15, 9633792, 0, 0, 0, 2, 12, 80, 128, 7168, 8192, 7168, 8192, 196608, 1048576, 0, 0, 0, 1, 2, 12, 16, 64, 128, 256, 0, 0, 3145728, 0, 0, 0, 536870912, 0, 0, 8192, 65536, 131072, 1048576, 0, 0, 0, 2097152, 0, 16384, 0, 4194304, 0, 0, 2097152, 16384, 0, 0, 131072, 2097152, 0, 0, 0, 4096, 0, 0, 8192, 0, 0, 0, 128, 0, 0, 0, 208, 0, 64, 128, 1024, 4096, 0, 0, 2, 8, 64, 128, 1024, 2048, 4096, 8192, 128, 1024, 4096, 8192, 0, 0
];

JSONiqParser.TOKEN =
[
  "(0)",
  "PragmaContents",
  "DirCommentContents",
  "DirPIContents",
  "CDataSection",
  "Wildcard",
  "EQName",
  "URILiteral",
  "IntegerLiteral",
  "DecimalLiteral",
  "DoubleLiteral",
  "StringLiteral",
  "PredefinedEntityRef",
  "'\"\"'",
  "EscapeApos",
  "ElementContentChar",
  "QuotAttrContentChar",
  "AposAttrContentChar",
  "PITarget",
  "NCName",
  "QName",
  "S",
  "S",
  "CharRef",
  "CommentContents",
  "EOF",
  "'!'",
  "'!='",
  "'\"'",
  "'#'",
  "'#)'",
  "'$'",
  "'$$'",
  "'%'",
  "''''",
  "'('",
  "'(#'",
  "'(:'",
  "')'",
  "'*'",
  "'*'",
  "'+'",
  "','",
  "'-'",
  "'-->'",
  "'.'",
  "'..'",
  "'/'",
  "'//'",
  "'/>'",
  "':'",
  "':)'",
  "'::'",
  "':='",
  "';'",
  "'<'",
  "'<!--'",
  "'</'",
  "'<<'",
  "'<='",
  "'<?'",
  "'='",
  "'>'",
  "'>='",
  "'>>'",
  "'?'",
  "'?>'",
  "'@'",
  "'NaN'",
  "'['",
  "']'",
  "'after'",
  "'all'",
  "'allowing'",
  "'ancestor'",
  "'ancestor-or-self'",
  "'and'",
  "'any'",
  "'append'",
  "'array'",
  "'as'",
  "'ascending'",
  "'at'",
  "'attribute'",
  "'base-uri'",
  "'before'",
  "'boundary-space'",
  "'break'",
  "'by'",
  "'case'",
  "'cast'",
  "'castable'",
  "'catch'",
  "'check'",
  "'child'",
  "'collation'",
  "'collection'",
  "'comment'",
  "'constraint'",
  "'construction'",
  "'contains'",
  "'content'",
  "'context'",
  "'continue'",
  "'copy'",
  "'copy-namespaces'",
  "'count'",
  "'decimal-format'",
  "'decimal-separator'",
  "'declare'",
  "'default'",
  "'delete'",
  "'descendant'",
  "'descendant-or-self'",
  "'descending'",
  "'diacritics'",
  "'different'",
  "'digit'",
  "'distance'",
  "'div'",
  "'document'",
  "'document-node'",
  "'element'",
  "'else'",
  "'empty'",
  "'empty-sequence'",
  "'encoding'",
  "'end'",
  "'entire'",
  "'eq'",
  "'every'",
  "'exactly'",
  "'except'",
  "'exit'",
  "'external'",
  "'false'",
  "'first'",
  "'following'",
  "'following-sibling'",
  "'for'",
  "'foreach'",
  "'foreign'",
  "'from'",
  "'ft-option'",
  "'ftand'",
  "'ftnot'",
  "'ftor'",
  "'function'",
  "'ge'",
  "'greatest'",
  "'group'",
  "'grouping-separator'",
  "'gt'",
  "'idiv'",
  "'if'",
  "'import'",
  "'in'",
  "'index'",
  "'infinity'",
  "'inherit'",
  "'insensitive'",
  "'insert'",
  "'instance'",
  "'integrity'",
  "'intersect'",
  "'into'",
  "'is'",
  "'item'",
  "'json'",
  "'json-item'",
  "'jsoniq'",
  "'key'",
  "'language'",
  "'last'",
  "'lax'",
  "'le'",
  "'least'",
  "'let'",
  "'levels'",
  "'loop'",
  "'lowercase'",
  "'lt'",
  "'minus-sign'",
  "'mod'",
  "'modify'",
  "'module'",
  "'most'",
  "'namespace'",
  "'namespace-node'",
  "'ne'",
  "'next'",
  "'no'",
  "'no-inherit'",
  "'no-preserve'",
  "'node'",
  "'nodes'",
  "'not'",
  "'null'",
  "'object'",
  "'occurs'",
  "'of'",
  "'on'",
  "'only'",
  "'option'",
  "'or'",
  "'order'",
  "'ordered'",
  "'ordering'",
  "'paragraph'",
  "'paragraphs'",
  "'parent'",
  "'pattern-separator'",
  "'per-mille'",
  "'percent'",
  "'phrase'",
  "'position'",
  "'preceding'",
  "'preceding-sibling'",
  "'preserve'",
  "'previous'",
  "'processing-instruction'",
  "'relationship'",
  "'rename'",
  "'replace'",
  "'return'",
  "'returning'",
  "'revalidation'",
  "'same'",
  "'satisfies'",
  "'schema'",
  "'schema-attribute'",
  "'schema-element'",
  "'score'",
  "'select'",
  "'self'",
  "'sensitive'",
  "'sentence'",
  "'sentences'",
  "'skip'",
  "'sliding'",
  "'some'",
  "'stable'",
  "'start'",
  "'stemming'",
  "'stop'",
  "'strict'",
  "'strip'",
  "'structured-item'",
  "'switch'",
  "'text'",
  "'then'",
  "'thesaurus'",
  "'times'",
  "'to'",
  "'treat'",
  "'true'",
  "'try'",
  "'tumbling'",
  "'type'",
  "'typeswitch'",
  "'union'",
  "'unique'",
  "'unordered'",
  "'updating'",
  "'uppercase'",
  "'using'",
  "'validate'",
  "'value'",
  "'variable'",
  "'version'",
  "'weight'",
  "'when'",
  "'where'",
  "'while'",
  "'wildcards'",
  "'window'",
  "'with'",
  "'without'",
  "'word'",
  "'words'",
  "'zero-digit'",
  "'{'",
  "'{{'",
  "'{|'",
  "'|'",
  "'||'",
  "'|}'",
  "'}'",
  "'}}'"
];

},
{}],
10:[function(_dereq_,module,exports){
                                                            var XQueryParser = exports.XQueryParser = function XQueryParser(string, parsingEventHandler)
                                                            {
                                                              init(string, parsingEventHandler);
  var self = this;

  this.ParseException = function(b, e, s, o, x)
  {
    var
      begin = b,
      end = e,
      state = s,
      offending = o,
      expected = x;

    this.getBegin = function() {return begin;};
    this.getEnd = function() {return end;};
    this.getState = function() {return state;};
    this.getExpected = function() {return expected;};
    this.getOffending = function() {return offending;};

    this.getMessage = function()
    {
      return offending < 0 ? "lexical analysis failed" : "syntax error";
    };
  };

  function init(string, parsingEventHandler)
  {
    eventHandler = parsingEventHandler;
    input = string;
    size = string.length;
    reset(0, 0, 0);
  }

  this.getInput = function()
  {
    return input;
  };

  function reset(l, b, e)
  {
            b0 = b; e0 = b;
    l1 = l; b1 = b; e1 = e;
    l2 = 0;
    end = e;
    ex = -1;
    memo = {};
    eventHandler.reset(input);
  }

  this.getOffendingToken = function(e)
  {
    var o = e.getOffending();
    return o >= 0 ? XQueryParser.TOKEN[o] : null;
  };

  this.getExpectedTokenSet = function(e)
  {
    var expected;
    if (e.getExpected() < 0)
    {
      expected = XQueryParser.getTokenSet(- e.getState());
    }
    else
    {
      expected = [XQueryParser.TOKEN[e.getExpected()]];
    }
    return expected;
  };

  this.getErrorMessage = function(e)
  {
    var tokenSet = this.getExpectedTokenSet(e);
    var found = this.getOffendingToken(e);
    var prefix = input.substring(0, e.getBegin());
    var lines = prefix.split("\n");
    var line = lines.length;
    var column = lines[line - 1].length + 1;
    var size = e.getEnd() - e.getBegin();
    return e.getMessage()
         + (found == null ? "" : ", found " + found)
         + "\nwhile expecting "
         + (tokenSet.length == 1 ? tokenSet[0] : ("[" + tokenSet.join(", ") + "]"))
         + "\n"
         + (size == 0 || found != null ? "" : "after successfully scanning " + size + " characters beginning ")
         + "at line " + line + ", column " + column + ":\n..."
         + input.substring(e.getBegin(), Math.min(input.length, e.getBegin() + 64))
         + "...";
  };

  this.parse_XQuery = function()
  {
    eventHandler.startNonterminal("XQuery", e0);
    lookahead1W(275);               // Wildcard | EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral |
    whitespace();
    parse_Module();
    shift(25);                      // EOF
    eventHandler.endNonterminal("XQuery", e0);
  };

  function parse_Module()
  {
    eventHandler.startNonterminal("Module", e0);
    switch (l1)
    {
    case 274:                       // 'xquery'
      lookahead2W(199);             // S^WS | EOF | '!' | '!=' | '#' | '(' | '(:' | '*' | '+' | ',' | '-' | '/' | '//' |
      break;
    default:
      lk = l1;
    }
    if (lk == 64274                 // 'xquery' 'encoding'
     || lk == 134930)               // 'xquery' 'version'
    {
      parse_VersionDecl();
    }
    lookahead1W(275);               // Wildcard | EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral |
    switch (l1)
    {
    case 182:                       // 'module'
      lookahead2W(194);             // S^WS | EOF | '!' | '!=' | '#' | '(' | '(:' | '*' | '+' | ',' | '-' | '/' | '//' |
      break;
    default:
      lk = l1;
    }
    switch (lk)
    {
    case 94390:                     // 'module' 'namespace'
      whitespace();
      parse_LibraryModule();
      break;
    default:
      whitespace();
      parse_MainModule();
    }
    eventHandler.endNonterminal("Module", e0);
  }

  function parse_VersionDecl()
  {
    eventHandler.startNonterminal("VersionDecl", e0);
    shift(274);                     // 'xquery'
    lookahead1W(116);               // S^WS | '(:' | 'encoding' | 'version'
    switch (l1)
    {
    case 125:                       // 'encoding'
      shift(125);                   // 'encoding'
      lookahead1W(17);              // StringLiteral | S^WS | '(:'
      shift(11);                    // StringLiteral
      break;
    default:
      shift(263);                   // 'version'
      lookahead1W(17);              // StringLiteral | S^WS | '(:'
      shift(11);                    // StringLiteral
      lookahead1W(109);             // S^WS | '(:' | ';' | 'encoding'
      if (l1 == 125)                // 'encoding'
      {
        shift(125);                 // 'encoding'
        lookahead1W(17);            // StringLiteral | S^WS | '(:'
        shift(11);                  // StringLiteral
      }
    }
    lookahead1W(28);                // S^WS | '(:' | ';'
    whitespace();
    parse_Separator();
    eventHandler.endNonterminal("VersionDecl", e0);
  }

  function parse_LibraryModule()
  {
    eventHandler.startNonterminal("LibraryModule", e0);
    parse_ModuleDecl();
    lookahead1W(138);               // S^WS | EOF | '(:' | 'declare' | 'import'
    whitespace();
    parse_Prolog();
    eventHandler.endNonterminal("LibraryModule", e0);
  }

  function parse_ModuleDecl()
  {
    eventHandler.startNonterminal("ModuleDecl", e0);
    shift(182);                     // 'module'
    lookahead1W(61);                // S^WS | '(:' | 'namespace'
    shift(184);                     // 'namespace'
    lookahead1W(249);               // NCName^Token | S^WS | '(:' | 'after' | 'allowing' | 'ancestor' |
    whitespace();
    parse_NCName();
    lookahead1W(29);                // S^WS | '(:' | '='
    shift(60);                      // '='
    lookahead1W(15);                // URILiteral | S^WS | '(:'
    shift(7);                       // URILiteral
    lookahead1W(28);                // S^WS | '(:' | ';'
    whitespace();
    parse_Separator();
    eventHandler.endNonterminal("ModuleDecl", e0);
  }

  function parse_Prolog()
  {
    eventHandler.startNonterminal("Prolog", e0);
    for (;;)
    {
      lookahead1W(275);             // Wildcard | EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral |
      switch (l1)
      {
      case 108:                     // 'declare'
        lookahead2W(214);           // S^WS | EOF | '!' | '!=' | '#' | '%' | '(' | '(:' | '*' | '+' | ',' | '-' | '/' |
        break;
      case 153:                     // 'import'
        lookahead2W(202);           // S^WS | EOF | '!' | '!=' | '#' | '(' | '(:' | '*' | '+' | ',' | '-' | '/' | '//' |
        break;
      default:
        lk = l1;
      }
      if (lk != 42604               // 'declare' 'base-uri'
       && lk != 43628               // 'declare' 'boundary-space'
       && lk != 50284               // 'declare' 'construction'
       && lk != 53356               // 'declare' 'copy-namespaces'
       && lk != 54380               // 'declare' 'decimal-format'
       && lk != 55916               // 'declare' 'default'
       && lk != 72300               // 'declare' 'ft-option'
       && lk != 93337               // 'import' 'module'
       && lk != 94316               // 'declare' 'namespace'
       && lk != 104044              // 'declare' 'ordering'
       && lk != 113772              // 'declare' 'revalidation'
       && lk != 115353)             // 'import' 'schema'
      {
        break;
      }
      switch (l1)
      {
      case 108:                     // 'declare'
        lookahead2W(179);           // S^WS | '(:' | 'base-uri' | 'boundary-space' | 'construction' |
        break;
      default:
        lk = l1;
      }
      if (lk == 55916)              // 'declare' 'default'
      {
        lk = memoized(0, e0);
        if (lk == 0)
        {
          var b0A = b0; var e0A = e0; var l1A = l1;
          var b1A = b1; var e1A = e1; var l2A = l2;
          var b2A = b2; var e2A = e2;
          try
          {
            try_DefaultNamespaceDecl();
            lk = -1;
          }
          catch (p1A)
          {
            lk = -2;
          }
          b0 = b0A; e0 = e0A; l1 = l1A; if (l1 == 0) {end = e0A;} else {
          b1 = b1A; e1 = e1A; l2 = l2A; if (l2 == 0) {end = e1A;} else {
          b2 = b2A; e2 = e2A; end = e2A; }}
          memoize(0, e0, lk);
        }
      }
      switch (lk)
      {
      case -1:
        whitespace();
        parse_DefaultNamespaceDecl();
        break;
      case 94316:                   // 'declare' 'namespace'
        whitespace();
        parse_NamespaceDecl();
        break;
      case 153:                     // 'import'
        whitespace();
        parse_Import();
        break;
      case 72300:                   // 'declare' 'ft-option'
        whitespace();
        parse_FTOptionDecl();
        break;
      default:
        whitespace();
        parse_Setter();
      }
      lookahead1W(28);              // S^WS | '(:' | ';'
      whitespace();
      parse_Separator();
    }
    for (;;)
    {
      lookahead1W(275);             // Wildcard | EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral |
      switch (l1)
      {
      case 108:                     // 'declare'
        lookahead2W(211);           // S^WS | EOF | '!' | '!=' | '#' | '%' | '(' | '(:' | '*' | '+' | ',' | '-' | '/' |
        break;
      default:
        lk = l1;
      }
      if (lk != 16492               // 'declare' '%'
       && lk != 48748               // 'declare' 'collection'
       && lk != 51820               // 'declare' 'context'
       && lk != 74348               // 'declare' 'function'
       && lk != 79468               // 'declare' 'index'
       && lk != 82540               // 'declare' 'integrity'
       && lk != 101996              // 'declare' 'option'
       && lk != 131692              // 'declare' 'updating'
       && lk != 134252)             // 'declare' 'variable'
      {
        break;
      }
      switch (l1)
      {
      case 108:                     // 'declare'
        lookahead2W(175);           // S^WS | '%' | '(:' | 'collection' | 'context' | 'function' | 'index' |
        break;
      default:
        lk = l1;
      }
      switch (lk)
      {
      case 51820:                   // 'declare' 'context'
        whitespace();
        parse_ContextItemDecl();
        break;
      case 101996:                  // 'declare' 'option'
        whitespace();
        parse_OptionDecl();
        break;
      default:
        whitespace();
        parse_AnnotatedDecl();
      }
      lookahead1W(28);              // S^WS | '(:' | ';'
      whitespace();
      parse_Separator();
    }
    eventHandler.endNonterminal("Prolog", e0);
  }

  function parse_Separator()
  {
    eventHandler.startNonterminal("Separator", e0);
    shift(53);                      // ';'
    eventHandler.endNonterminal("Separator", e0);
  }

  function parse_Setter()
  {
    eventHandler.startNonterminal("Setter", e0);
    switch (l1)
    {
    case 108:                       // 'declare'
      lookahead2W(172);             // S^WS | '(:' | 'base-uri' | 'boundary-space' | 'construction' |
      break;
    default:
      lk = l1;
    }
    if (lk == 55916)                // 'declare' 'default'
    {
      lk = memoized(1, e0);
      if (lk == 0)
      {
        var b0A = b0; var e0A = e0; var l1A = l1;
        var b1A = b1; var e1A = e1; var l2A = l2;
        var b2A = b2; var e2A = e2;
        try
        {
          try_DefaultCollationDecl();
          lk = -2;
        }
        catch (p2A)
        {
          try
          {
            b0 = b0A; e0 = e0A; l1 = l1A; if (l1 == 0) {end = e0A;} else {
            b1 = b1A; e1 = e1A; l2 = l2A; if (l2 == 0) {end = e1A;} else {
            b2 = b2A; e2 = e2A; end = e2A; }}
            try_EmptyOrderDecl();
            lk = -6;
          }
          catch (p6A)
          {
            lk = -9;
          }
        }
        b0 = b0A; e0 = e0A; l1 = l1A; if (l1 == 0) {end = e0A;} else {
        b1 = b1A; e1 = e1A; l2 = l2A; if (l2 == 0) {end = e1A;} else {
        b2 = b2A; e2 = e2A; end = e2A; }}
        memoize(1, e0, lk);
      }
    }
    switch (lk)
    {
    case 43628:                     // 'declare' 'boundary-space'
      parse_BoundarySpaceDecl();
      break;
    case -2:
      parse_DefaultCollationDecl();
      break;
    case 42604:                     // 'declare' 'base-uri'
      parse_BaseURIDecl();
      break;
    case 50284:                     // 'declare' 'construction'
      parse_ConstructionDecl();
      break;
    case 104044:                    // 'declare' 'ordering'
      parse_OrderingModeDecl();
      break;
    case -6:
      parse_EmptyOrderDecl();
      break;
    case 113772:                    // 'declare' 'revalidation'
      parse_RevalidationDecl();
      break;
    case 53356:                     // 'declare' 'copy-namespaces'
      parse_CopyNamespacesDecl();
      break;
    default:
      parse_DecimalFormatDecl();
    }
    eventHandler.endNonterminal("Setter", e0);
  }

  function parse_BoundarySpaceDecl()
  {
    eventHandler.startNonterminal("BoundarySpaceDecl", e0);
    shift(108);                     // 'declare'
    lookahead1W(33);                // S^WS | '(:' | 'boundary-space'
    shift(85);                      // 'boundary-space'
    lookahead1W(133);               // S^WS | '(:' | 'preserve' | 'strip'
    switch (l1)
    {
    case 214:                       // 'preserve'
      shift(214);                   // 'preserve'
      break;
    default:
      shift(241);                   // 'strip'
    }
    eventHandler.endNonterminal("BoundarySpaceDecl", e0);
  }

  function parse_DefaultCollationDecl()
  {
    eventHandler.startNonterminal("DefaultCollationDecl", e0);
    shift(108);                     // 'declare'
    lookahead1W(46);                // S^WS | '(:' | 'default'
    shift(109);                     // 'default'
    lookahead1W(38);                // S^WS | '(:' | 'collation'
    shift(94);                      // 'collation'
    lookahead1W(15);                // URILiteral | S^WS | '(:'
    shift(7);                       // URILiteral
    eventHandler.endNonterminal("DefaultCollationDecl", e0);
  }

  function try_DefaultCollationDecl()
  {
    shiftT(108);                    // 'declare'
    lookahead1W(46);                // S^WS | '(:' | 'default'
    shiftT(109);                    // 'default'
    lookahead1W(38);                // S^WS | '(:' | 'collation'
    shiftT(94);                     // 'collation'
    lookahead1W(15);                // URILiteral | S^WS | '(:'
    shiftT(7);                      // URILiteral
  }

  function parse_BaseURIDecl()
  {
    eventHandler.startNonterminal("BaseURIDecl", e0);
    shift(108);                     // 'declare'
    lookahead1W(32);                // S^WS | '(:' | 'base-uri'
    shift(83);                      // 'base-uri'
    lookahead1W(15);                // URILiteral | S^WS | '(:'
    shift(7);                       // URILiteral
    eventHandler.endNonterminal("BaseURIDecl", e0);
  }

  function parse_ConstructionDecl()
  {
    eventHandler.startNonterminal("ConstructionDecl", e0);
    shift(108);                     // 'declare'
    lookahead1W(41);                // S^WS | '(:' | 'construction'
    shift(98);                      // 'construction'
    lookahead1W(133);               // S^WS | '(:' | 'preserve' | 'strip'
    switch (l1)
    {
    case 241:                       // 'strip'
      shift(241);                   // 'strip'
      break;
    default:
      shift(214);                   // 'preserve'
    }
    eventHandler.endNonterminal("ConstructionDecl", e0);
  }

  function parse_OrderingModeDecl()
  {
    eventHandler.startNonterminal("OrderingModeDecl", e0);
    shift(108);                     // 'declare'
    lookahead1W(68);                // S^WS | '(:' | 'ordering'
    shift(203);                     // 'ordering'
    lookahead1W(131);               // S^WS | '(:' | 'ordered' | 'unordered'
    switch (l1)
    {
    case 202:                       // 'ordered'
      shift(202);                   // 'ordered'
      break;
    default:
      shift(256);                   // 'unordered'
    }
    eventHandler.endNonterminal("OrderingModeDecl", e0);
  }

  function parse_EmptyOrderDecl()
  {
    eventHandler.startNonterminal("EmptyOrderDecl", e0);
    shift(108);                     // 'declare'
    lookahead1W(46);                // S^WS | '(:' | 'default'
    shift(109);                     // 'default'
    lookahead1W(67);                // S^WS | '(:' | 'order'
    shift(201);                     // 'order'
    lookahead1W(49);                // S^WS | '(:' | 'empty'
    shift(123);                     // 'empty'
    lookahead1W(121);               // S^WS | '(:' | 'greatest' | 'least'
    switch (l1)
    {
    case 147:                       // 'greatest'
      shift(147);                   // 'greatest'
      break;
    default:
      shift(173);                   // 'least'
    }
    eventHandler.endNonterminal("EmptyOrderDecl", e0);
  }

  function try_EmptyOrderDecl()
  {
    shiftT(108);                    // 'declare'
    lookahead1W(46);                // S^WS | '(:' | 'default'
    shiftT(109);                    // 'default'
    lookahead1W(67);                // S^WS | '(:' | 'order'
    shiftT(201);                    // 'order'
    lookahead1W(49);                // S^WS | '(:' | 'empty'
    shiftT(123);                    // 'empty'
    lookahead1W(121);               // S^WS | '(:' | 'greatest' | 'least'
    switch (l1)
    {
    case 147:                       // 'greatest'
      shiftT(147);                  // 'greatest'
      break;
    default:
      shiftT(173);                  // 'least'
    }
  }

  function parse_CopyNamespacesDecl()
  {
    eventHandler.startNonterminal("CopyNamespacesDecl", e0);
    shift(108);                     // 'declare'
    lookahead1W(44);                // S^WS | '(:' | 'copy-namespaces'
    shift(104);                     // 'copy-namespaces'
    lookahead1W(128);               // S^WS | '(:' | 'no-preserve' | 'preserve'
    whitespace();
    parse_PreserveMode();
    lookahead1W(25);                // S^WS | '(:' | ','
    shift(41);                      // ','
    lookahead1W(123);               // S^WS | '(:' | 'inherit' | 'no-inherit'
    whitespace();
    parse_InheritMode();
    eventHandler.endNonterminal("CopyNamespacesDecl", e0);
  }

  function parse_PreserveMode()
  {
    eventHandler.startNonterminal("PreserveMode", e0);
    switch (l1)
    {
    case 214:                       // 'preserve'
      shift(214);                   // 'preserve'
      break;
    default:
      shift(190);                   // 'no-preserve'
    }
    eventHandler.endNonterminal("PreserveMode", e0);
  }

  function parse_InheritMode()
  {
    eventHandler.startNonterminal("InheritMode", e0);
    switch (l1)
    {
    case 157:                       // 'inherit'
      shift(157);                   // 'inherit'
      break;
    default:
      shift(189);                   // 'no-inherit'
    }
    eventHandler.endNonterminal("InheritMode", e0);
  }

  function parse_DecimalFormatDecl()
  {
    eventHandler.startNonterminal("DecimalFormatDecl", e0);
    shift(108);                     // 'declare'
    lookahead1W(114);               // S^WS | '(:' | 'decimal-format' | 'default'
    switch (l1)
    {
    case 106:                       // 'decimal-format'
      shift(106);                   // 'decimal-format'
      lookahead1W(255);             // EQName^Token | S^WS | '(:' | 'after' | 'allowing' | 'ancestor' |
      whitespace();
      parse_EQName();
      break;
    default:
      shift(109);                   // 'default'
      lookahead1W(45);              // S^WS | '(:' | 'decimal-format'
      shift(106);                   // 'decimal-format'
    }
    for (;;)
    {
      lookahead1W(181);             // S^WS | '(:' | ';' | 'NaN' | 'decimal-separator' | 'digit' |
      if (l1 == 53)                 // ';'
      {
        break;
      }
      whitespace();
      parse_DFPropertyName();
      lookahead1W(29);              // S^WS | '(:' | '='
      shift(60);                    // '='
      lookahead1W(17);              // StringLiteral | S^WS | '(:'
      shift(11);                    // StringLiteral
    }
    eventHandler.endNonterminal("DecimalFormatDecl", e0);
  }

  function parse_DFPropertyName()
  {
    eventHandler.startNonterminal("DFPropertyName", e0);
    switch (l1)
    {
    case 107:                       // 'decimal-separator'
      shift(107);                   // 'decimal-separator'
      break;
    case 149:                       // 'grouping-separator'
      shift(149);                   // 'grouping-separator'
      break;
    case 156:                       // 'infinity'
      shift(156);                   // 'infinity'
      break;
    case 179:                       // 'minus-sign'
      shift(179);                   // 'minus-sign'
      break;
    case 67:                        // 'NaN'
      shift(67);                    // 'NaN'
      break;
    case 209:                       // 'percent'
      shift(209);                   // 'percent'
      break;
    case 208:                       // 'per-mille'
      shift(208);                   // 'per-mille'
      break;
    case 275:                       // 'zero-digit'
      shift(275);                   // 'zero-digit'
      break;
    case 116:                       // 'digit'
      shift(116);                   // 'digit'
      break;
    default:
      shift(207);                   // 'pattern-separator'
    }
    eventHandler.endNonterminal("DFPropertyName", e0);
  }

  function parse_Import()
  {
    eventHandler.startNonterminal("Import", e0);
    switch (l1)
    {
    case 153:                       // 'import'
      lookahead2W(126);             // S^WS | '(:' | 'module' | 'schema'
      break;
    default:
      lk = l1;
    }
    switch (lk)
    {
    case 115353:                    // 'import' 'schema'
      parse_SchemaImport();
      break;
    default:
      parse_ModuleImport();
    }
    eventHandler.endNonterminal("Import", e0);
  }

  function parse_SchemaImport()
  {
    eventHandler.startNonterminal("SchemaImport", e0);
    shift(153);                     // 'import'
    lookahead1W(73);                // S^WS | '(:' | 'schema'
    shift(225);                     // 'schema'
    lookahead1W(137);               // URILiteral | S^WS | '(:' | 'default' | 'namespace'
    if (l1 != 7)                    // URILiteral
    {
      whitespace();
      parse_SchemaPrefix();
    }
    lookahead1W(15);                // URILiteral | S^WS | '(:'
    shift(7);                       // URILiteral
    lookahead1W(108);               // S^WS | '(:' | ';' | 'at'
    if (l1 == 81)                   // 'at'
    {
      shift(81);                    // 'at'
      lookahead1W(15);              // URILiteral | S^WS | '(:'
      shift(7);                     // URILiteral
      for (;;)
      {
        lookahead1W(103);           // S^WS | '(:' | ',' | ';'
        if (l1 != 41)               // ','
        {
          break;
        }
        shift(41);                  // ','
        lookahead1W(15);            // URILiteral | S^WS | '(:'
        shift(7);                   // URILiteral
      }
    }
    eventHandler.endNonterminal("SchemaImport", e0);
  }

  function parse_SchemaPrefix()
  {
    eventHandler.startNonterminal("SchemaPrefix", e0);
    switch (l1)
    {
    case 184:                       // 'namespace'
      shift(184);                   // 'namespace'
      lookahead1W(249);             // NCName^Token | S^WS | '(:' | 'after' | 'allowing' | 'ancestor' |
      whitespace();
      parse_NCName();
      lookahead1W(29);              // S^WS | '(:' | '='
      shift(60);                    // '='
      break;
    default:
      shift(109);                   // 'default'
      lookahead1W(47);              // S^WS | '(:' | 'element'
      shift(121);                   // 'element'
      lookahead1W(61);              // S^WS | '(:' | 'namespace'
      shift(184);                   // 'namespace'
    }
    eventHandler.endNonterminal("SchemaPrefix", e0);
  }

  function parse_ModuleImport()
  {
    eventHandler.startNonterminal("ModuleImport", e0);
    shift(153);                     // 'import'
    lookahead1W(60);                // S^WS | '(:' | 'module'
    shift(182);                     // 'module'
    lookahead1W(90);                // URILiteral | S^WS | '(:' | 'namespace'
    if (l1 == 184)                  // 'namespace'
    {
      shift(184);                   // 'namespace'
      lookahead1W(249);             // NCName^Token | S^WS | '(:' | 'after' | 'allowing' | 'ancestor' |
      whitespace();
      parse_NCName();
      lookahead1W(29);              // S^WS | '(:' | '='
      shift(60);                    // '='
    }
    lookahead1W(15);                // URILiteral | S^WS | '(:'
    shift(7);                       // URILiteral
    lookahead1W(108);               // S^WS | '(:' | ';' | 'at'
    if (l1 == 81)                   // 'at'
    {
      shift(81);                    // 'at'
      lookahead1W(15);              // URILiteral | S^WS | '(:'
      shift(7);                     // URILiteral
      for (;;)
      {
        lookahead1W(103);           // S^WS | '(:' | ',' | ';'
        if (l1 != 41)               // ','
        {
          break;
        }
        shift(41);                  // ','
        lookahead1W(15);            // URILiteral | S^WS | '(:'
        shift(7);                   // URILiteral
      }
    }
    eventHandler.endNonterminal("ModuleImport", e0);
  }

  function parse_NamespaceDecl()
  {
    eventHandler.startNonterminal("NamespaceDecl", e0);
    shift(108);                     // 'declare'
    lookahead1W(61);                // S^WS | '(:' | 'namespace'
    shift(184);                     // 'namespace'
    lookahead1W(249);               // NCName^Token | S^WS | '(:' | 'after' | 'allowing' | 'ancestor' |
    whitespace();
    parse_NCName();
    lookahead1W(29);                // S^WS | '(:' | '='
    shift(60);                      // '='
    lookahead1W(15);                // URILiteral | S^WS | '(:'
    shift(7);                       // URILiteral
    eventHandler.endNonterminal("NamespaceDecl", e0);
  }

  function parse_DefaultNamespaceDecl()
  {
    eventHandler.startNonterminal("DefaultNamespaceDecl", e0);
    shift(108);                     // 'declare'
    lookahead1W(46);                // S^WS | '(:' | 'default'
    shift(109);                     // 'default'
    lookahead1W(115);               // S^WS | '(:' | 'element' | 'function'
    switch (l1)
    {
    case 121:                       // 'element'
      shift(121);                   // 'element'
      break;
    default:
      shift(145);                   // 'function'
    }
    lookahead1W(61);                // S^WS | '(:' | 'namespace'
    shift(184);                     // 'namespace'
    lookahead1W(15);                // URILiteral | S^WS | '(:'
    shift(7);                       // URILiteral
    eventHandler.endNonterminal("DefaultNamespaceDecl", e0);
  }

  function try_DefaultNamespaceDecl()
  {
    shiftT(108);                    // 'declare'
    lookahead1W(46);                // S^WS | '(:' | 'default'
    shiftT(109);                    // 'default'
    lookahead1W(115);               // S^WS | '(:' | 'element' | 'function'
    switch (l1)
    {
    case 121:                       // 'element'
      shiftT(121);                  // 'element'
      break;
    default:
      shiftT(145);                  // 'function'
    }
    lookahead1W(61);                // S^WS | '(:' | 'namespace'
    shiftT(184);                    // 'namespace'
    lookahead1W(15);                // URILiteral | S^WS | '(:'
    shiftT(7);                      // URILiteral
  }

  function parse_FTOptionDecl()
  {
    eventHandler.startNonterminal("FTOptionDecl", e0);
    shift(108);                     // 'declare'
    lookahead1W(52);                // S^WS | '(:' | 'ft-option'
    shift(141);                     // 'ft-option'
    lookahead1W(81);                // S^WS | '(:' | 'using'
    whitespace();
    parse_FTMatchOptions();
    eventHandler.endNonterminal("FTOptionDecl", e0);
  }

  function parse_AnnotatedDecl()
  {
    eventHandler.startNonterminal("AnnotatedDecl", e0);
    shift(108);                     // 'declare'
    for (;;)
    {
      lookahead1W(170);             // S^WS | '%' | '(:' | 'collection' | 'function' | 'index' | 'integrity' |
      if (l1 != 32                  // '%'
       && l1 != 257)                // 'updating'
      {
        break;
      }
      switch (l1)
      {
      case 257:                     // 'updating'
        whitespace();
        parse_CompatibilityAnnotation();
        break;
      default:
        whitespace();
        parse_Annotation();
      }
    }
    switch (l1)
    {
    case 262:                       // 'variable'
      whitespace();
      parse_VarDecl();
      break;
    case 145:                       // 'function'
      whitespace();
      parse_FunctionDecl();
      break;
    case 95:                        // 'collection'
      whitespace();
      parse_CollectionDecl();
      break;
    case 155:                       // 'index'
      whitespace();
      parse_IndexDecl();
      break;
    default:
      whitespace();
      parse_ICDecl();
    }
    eventHandler.endNonterminal("AnnotatedDecl", e0);
  }

  function parse_CompatibilityAnnotation()
  {
    eventHandler.startNonterminal("CompatibilityAnnotation", e0);
    shift(257);                     // 'updating'
    eventHandler.endNonterminal("CompatibilityAnnotation", e0);
  }

  function parse_Annotation()
  {
    eventHandler.startNonterminal("Annotation", e0);
    shift(32);                      // '%'
    lookahead1W(255);               // EQName^Token | S^WS | '(:' | 'after' | 'allowing' | 'ancestor' |
    whitespace();
    parse_EQName();
    lookahead1W(171);               // S^WS | '%' | '(' | '(:' | 'collection' | 'function' | 'index' | 'integrity' |
    if (l1 == 34)                   // '('
    {
      shift(34);                    // '('
      lookahead1W(154);             // IntegerLiteral | DecimalLiteral | DoubleLiteral | StringLiteral | S^WS | '(:'
      whitespace();
      parse_Literal();
      for (;;)
      {
        lookahead1W(101);           // S^WS | '(:' | ')' | ','
        if (l1 != 41)               // ','
        {
          break;
        }
        shift(41);                  // ','
        lookahead1W(154);           // IntegerLiteral | DecimalLiteral | DoubleLiteral | StringLiteral | S^WS | '(:'
        whitespace();
        parse_Literal();
      }
      shift(37);                    // ')'
    }
    eventHandler.endNonterminal("Annotation", e0);
  }

  function try_Annotation()
  {
    shiftT(32);                     // '%'
    lookahead1W(255);               // EQName^Token | S^WS | '(:' | 'after' | 'allowing' | 'ancestor' |
    try_EQName();
    lookahead1W(171);               // S^WS | '%' | '(' | '(:' | 'collection' | 'function' | 'index' | 'integrity' |
    if (l1 == 34)                   // '('
    {
      shiftT(34);                   // '('
      lookahead1W(154);             // IntegerLiteral | DecimalLiteral | DoubleLiteral | StringLiteral | S^WS | '(:'
      try_Literal();
      for (;;)
      {
        lookahead1W(101);           // S^WS | '(:' | ')' | ','
        if (l1 != 41)               // ','
        {
          break;
        }
        shiftT(41);                 // ','
        lookahead1W(154);           // IntegerLiteral | DecimalLiteral | DoubleLiteral | StringLiteral | S^WS | '(:'
        try_Literal();
      }
      shiftT(37);                   // ')'
    }
  }

  function parse_VarDecl()
  {
    eventHandler.startNonterminal("VarDecl", e0);
    shift(262);                     // 'variable'
    lookahead1W(21);                // S^WS | '$' | '(:'
    shift(31);                      // '$'
    lookahead1W(255);               // EQName^Token | S^WS | '(:' | 'after' | 'allowing' | 'ancestor' |
    whitespace();
    parse_VarName();
    lookahead1W(147);               // S^WS | '(:' | ':=' | 'as' | 'external'
    if (l1 == 79)                   // 'as'
    {
      whitespace();
      parse_TypeDeclaration();
    }
    lookahead1W(106);               // S^WS | '(:' | ':=' | 'external'
    switch (l1)
    {
    case 52:                        // ':='
      shift(52);                    // ':='
      lookahead1W(267);             // Wildcard | EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral |
      whitespace();
      parse_VarValue();
      break;
    default:
      shift(133);                   // 'external'
      lookahead1W(104);             // S^WS | '(:' | ':=' | ';'
      if (l1 == 52)                 // ':='
      {
        shift(52);                  // ':='
        lookahead1W(267);           // Wildcard | EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral |
        whitespace();
        parse_VarDefaultValue();
      }
    }
    eventHandler.endNonterminal("VarDecl", e0);
  }

  function parse_VarValue()
  {
    eventHandler.startNonterminal("VarValue", e0);
    parse_ExprSingle();
    eventHandler.endNonterminal("VarValue", e0);
  }

  function parse_VarDefaultValue()
  {
    eventHandler.startNonterminal("VarDefaultValue", e0);
    parse_ExprSingle();
    eventHandler.endNonterminal("VarDefaultValue", e0);
  }

  function parse_ContextItemDecl()
  {
    eventHandler.startNonterminal("ContextItemDecl", e0);
    shift(108);                     // 'declare'
    lookahead1W(43);                // S^WS | '(:' | 'context'
    shift(101);                     // 'context'
    lookahead1W(55);                // S^WS | '(:' | 'item'
    shift(165);                     // 'item'
    lookahead1W(147);               // S^WS | '(:' | ':=' | 'as' | 'external'
    if (l1 == 79)                   // 'as'
    {
      shift(79);                    // 'as'
      lookahead1W(260);             // EQName^Token | S^WS | '%' | '(' | '(:' | 'after' | 'allowing' | 'ancestor' |
      whitespace();
      parse_ItemType();
    }
    lookahead1W(106);               // S^WS | '(:' | ':=' | 'external'
    switch (l1)
    {
    case 52:                        // ':='
      shift(52);                    // ':='
      lookahead1W(267);             // Wildcard | EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral |
      whitespace();
      parse_VarValue();
      break;
    default:
      shift(133);                   // 'external'
      lookahead1W(104);             // S^WS | '(:' | ':=' | ';'
      if (l1 == 52)                 // ':='
      {
        shift(52);                  // ':='
        lookahead1W(267);           // Wildcard | EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral |
        whitespace();
        parse_VarDefaultValue();
      }
    }
    eventHandler.endNonterminal("ContextItemDecl", e0);
  }

  function parse_ParamList()
  {
    eventHandler.startNonterminal("ParamList", e0);
    parse_Param();
    for (;;)
    {
      lookahead1W(101);             // S^WS | '(:' | ')' | ','
      if (l1 != 41)                 // ','
      {
        break;
      }
      shift(41);                    // ','
      lookahead1W(21);              // S^WS | '$' | '(:'
      whitespace();
      parse_Param();
    }
    eventHandler.endNonterminal("ParamList", e0);
  }

  function try_ParamList()
  {
    try_Param();
    for (;;)
    {
      lookahead1W(101);             // S^WS | '(:' | ')' | ','
      if (l1 != 41)                 // ','
      {
        break;
      }
      shiftT(41);                   // ','
      lookahead1W(21);              // S^WS | '$' | '(:'
      try_Param();
    }
  }

  function parse_Param()
  {
    eventHandler.startNonterminal("Param", e0);
    shift(31);                      // '$'
    lookahead1W(255);               // EQName^Token | S^WS | '(:' | 'after' | 'allowing' | 'ancestor' |
    whitespace();
    parse_EQName();
    lookahead1W(143);               // S^WS | '(:' | ')' | ',' | 'as'
    if (l1 == 79)                   // 'as'
    {
      whitespace();
      parse_TypeDeclaration();
    }
    eventHandler.endNonterminal("Param", e0);
  }

  function try_Param()
  {
    shiftT(31);                     // '$'
    lookahead1W(255);               // EQName^Token | S^WS | '(:' | 'after' | 'allowing' | 'ancestor' |
    try_EQName();
    lookahead1W(143);               // S^WS | '(:' | ')' | ',' | 'as'
    if (l1 == 79)                   // 'as'
    {
      try_TypeDeclaration();
    }
  }

  function parse_FunctionBody()
  {
    eventHandler.startNonterminal("FunctionBody", e0);
    parse_EnclosedExpr();
    eventHandler.endNonterminal("FunctionBody", e0);
  }

  function try_FunctionBody()
  {
    try_EnclosedExpr();
  }

  function parse_EnclosedExpr()
  {
    eventHandler.startNonterminal("EnclosedExpr", e0);
    shift(276);                     // '{'
    lookahead1W(267);               // Wildcard | EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral |
    whitespace();
    parse_Expr();
    shift(282);                     // '}'
    eventHandler.endNonterminal("EnclosedExpr", e0);
  }

  function try_EnclosedExpr()
  {
    shiftT(276);                    // '{'
    lookahead1W(267);               // Wildcard | EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral |
    try_Expr();
    shiftT(282);                    // '}'
  }

  function parse_OptionDecl()
  {
    eventHandler.startNonterminal("OptionDecl", e0);
    shift(108);                     // 'declare'
    lookahead1W(66);                // S^WS | '(:' | 'option'
    shift(199);                     // 'option'
    lookahead1W(255);               // EQName^Token | S^WS | '(:' | 'after' | 'allowing' | 'ancestor' |
    whitespace();
    parse_EQName();
    lookahead1W(17);                // StringLiteral | S^WS | '(:'
    shift(11);                      // StringLiteral
    eventHandler.endNonterminal("OptionDecl", e0);
  }

  function parse_Expr()
  {
    eventHandler.startNonterminal("Expr", e0);
    parse_ExprSingle();
    for (;;)
    {
      if (l1 != 41)                 // ','
      {
        break;
      }
      shift(41);                    // ','
      lookahead1W(267);             // Wildcard | EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral |
      whitespace();
      parse_ExprSingle();
    }
    eventHandler.endNonterminal("Expr", e0);
  }

  function try_Expr()
  {
    try_ExprSingle();
    for (;;)
    {
      if (l1 != 41)                 // ','
      {
        break;
      }
      shiftT(41);                   // ','
      lookahead1W(267);             // Wildcard | EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral |
      try_ExprSingle();
    }
  }

  function parse_FLWORExpr()
  {
    eventHandler.startNonterminal("FLWORExpr", e0);
    parse_InitialClause();
    for (;;)
    {
      lookahead1W(173);             // S^WS | '(:' | 'count' | 'for' | 'group' | 'let' | 'order' | 'return' | 'stable' |
      if (l1 == 220)                // 'return'
      {
        break;
      }
      whitespace();
      parse_IntermediateClause();
    }
    whitespace();
    parse_ReturnClause();
    eventHandler.endNonterminal("FLWORExpr", e0);
  }

  function try_FLWORExpr()
  {
    try_InitialClause();
    for (;;)
    {
      lookahead1W(173);             // S^WS | '(:' | 'count' | 'for' | 'group' | 'let' | 'order' | 'return' | 'stable' |
      if (l1 == 220)                // 'return'
      {
        break;
      }
      try_IntermediateClause();
    }
    try_ReturnClause();
  }

  function parse_InitialClause()
  {
    eventHandler.startNonterminal("InitialClause", e0);
    switch (l1)
    {
    case 137:                       // 'for'
      lookahead2W(141);             // S^WS | '$' | '(:' | 'sliding' | 'tumbling'
      break;
    default:
      lk = l1;
    }
    switch (lk)
    {
    case 16009:                     // 'for' '$'
      parse_ForClause();
      break;
    case 174:                       // 'let'
      parse_LetClause();
      break;
    default:
      parse_WindowClause();
    }
    eventHandler.endNonterminal("InitialClause", e0);
  }

  function try_InitialClause()
  {
    switch (l1)
    {
    case 137:                       // 'for'
      lookahead2W(141);             // S^WS | '$' | '(:' | 'sliding' | 'tumbling'
      break;
    default:
      lk = l1;
    }
    switch (lk)
    {
    case 16009:                     // 'for' '$'
      try_ForClause();
      break;
    case 174:                       // 'let'
      try_LetClause();
      break;
    default:
      try_WindowClause();
    }
  }

  function parse_IntermediateClause()
  {
    eventHandler.startNonterminal("IntermediateClause", e0);
    switch (l1)
    {
    case 137:                       // 'for'
    case 174:                       // 'let'
      parse_InitialClause();
      break;
    case 266:                       // 'where'
      parse_WhereClause();
      break;
    case 148:                       // 'group'
      parse_GroupByClause();
      break;
    case 105:                       // 'count'
      parse_CountClause();
      break;
    default:
      parse_OrderByClause();
    }
    eventHandler.endNonterminal("IntermediateClause", e0);
  }

  function try_IntermediateClause()
  {
    switch (l1)
    {
    case 137:                       // 'for'
    case 174:                       // 'let'
      try_InitialClause();
      break;
    case 266:                       // 'where'
      try_WhereClause();
      break;
    case 148:                       // 'group'
      try_GroupByClause();
      break;
    case 105:                       // 'count'
      try_CountClause();
      break;
    default:
      try_OrderByClause();
    }
  }

  function parse_ForClause()
  {
    eventHandler.startNonterminal("ForClause", e0);
    shift(137);                     // 'for'
    lookahead1W(21);                // S^WS | '$' | '(:'
    whitespace();
    parse_ForBinding();
    for (;;)
    {
      if (l1 != 41)                 // ','
      {
        break;
      }
      shift(41);                    // ','
      lookahead1W(21);              // S^WS | '$' | '(:'
      whitespace();
      parse_ForBinding();
    }
    eventHandler.endNonterminal("ForClause", e0);
  }

  function try_ForClause()
  {
    shiftT(137);                    // 'for'
    lookahead1W(21);                // S^WS | '$' | '(:'
    try_ForBinding();
    for (;;)
    {
      if (l1 != 41)                 // ','
      {
        break;
      }
      shiftT(41);                   // ','
      lookahead1W(21);              // S^WS | '$' | '(:'
      try_ForBinding();
    }
  }

  function parse_ForBinding()
  {
    eventHandler.startNonterminal("ForBinding", e0);
    shift(31);                      // '$'
    lookahead1W(255);               // EQName^Token | S^WS | '(:' | 'after' | 'allowing' | 'ancestor' |
    whitespace();
    parse_VarName();
    lookahead1W(164);               // S^WS | '(:' | 'allowing' | 'as' | 'at' | 'in' | 'score'
    if (l1 == 79)                   // 'as'
    {
      whitespace();
      parse_TypeDeclaration();
    }
    lookahead1W(158);               // S^WS | '(:' | 'allowing' | 'at' | 'in' | 'score'
    if (l1 == 72)                   // 'allowing'
    {
      whitespace();
      parse_AllowingEmpty();
    }
    lookahead1W(150);               // S^WS | '(:' | 'at' | 'in' | 'score'
    if (l1 == 81)                   // 'at'
    {
      whitespace();
      parse_PositionalVar();
    }
    lookahead1W(122);               // S^WS | '(:' | 'in' | 'score'
    if (l1 == 228)                  // 'score'
    {
      whitespace();
      parse_FTScoreVar();
    }
    lookahead1W(53);                // S^WS | '(:' | 'in'
    shift(154);                     // 'in'
    lookahead1W(267);               // Wildcard | EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral |
    whitespace();
    parse_ExprSingle();
    eventHandler.endNonterminal("ForBinding", e0);
  }

  function try_ForBinding()
  {
    shiftT(31);                     // '$'
    lookahead1W(255);               // EQName^Token | S^WS | '(:' | 'after' | 'allowing' | 'ancestor' |
    try_VarName();
    lookahead1W(164);               // S^WS | '(:' | 'allowing' | 'as' | 'at' | 'in' | 'score'
    if (l1 == 79)                   // 'as'
    {
      try_TypeDeclaration();
    }
    lookahead1W(158);               // S^WS | '(:' | 'allowing' | 'at' | 'in' | 'score'
    if (l1 == 72)                   // 'allowing'
    {
      try_AllowingEmpty();
    }
    lookahead1W(150);               // S^WS | '(:' | 'at' | 'in' | 'score'
    if (l1 == 81)                   // 'at'
    {
      try_PositionalVar();
    }
    lookahead1W(122);               // S^WS | '(:' | 'in' | 'score'
    if (l1 == 228)                  // 'score'
    {
      try_FTScoreVar();
    }
    lookahead1W(53);                // S^WS | '(:' | 'in'
    shiftT(154);                    // 'in'
    lookahead1W(267);               // Wildcard | EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral |
    try_ExprSingle();
  }

  function parse_AllowingEmpty()
  {
    eventHandler.startNonterminal("AllowingEmpty", e0);
    shift(72);                      // 'allowing'
    lookahead1W(49);                // S^WS | '(:' | 'empty'
    shift(123);                     // 'empty'
    eventHandler.endNonterminal("AllowingEmpty", e0);
  }

  function try_AllowingEmpty()
  {
    shiftT(72);                     // 'allowing'
    lookahead1W(49);                // S^WS | '(:' | 'empty'
    shiftT(123);                    // 'empty'
  }

  function parse_PositionalVar()
  {
    eventHandler.startNonterminal("PositionalVar", e0);
    shift(81);                      // 'at'
    lookahead1W(21);                // S^WS | '$' | '(:'
    shift(31);                      // '$'
    lookahead1W(255);               // EQName^Token | S^WS | '(:' | 'after' | 'allowing' | 'ancestor' |
    whitespace();
    parse_VarName();
    eventHandler.endNonterminal("PositionalVar", e0);
  }

  function try_PositionalVar()
  {
    shiftT(81);                     // 'at'
    lookahead1W(21);                // S^WS | '$' | '(:'
    shiftT(31);                     // '$'
    lookahead1W(255);               // EQName^Token | S^WS | '(:' | 'after' | 'allowing' | 'ancestor' |
    try_VarName();
  }

  function parse_FTScoreVar()
  {
    eventHandler.startNonterminal("FTScoreVar", e0);
    shift(228);                     // 'score'
    lookahead1W(21);                // S^WS | '$' | '(:'
    shift(31);                      // '$'
    lookahead1W(255);               // EQName^Token | S^WS | '(:' | 'after' | 'allowing' | 'ancestor' |
    whitespace();
    parse_VarName();
    eventHandler.endNonterminal("FTScoreVar", e0);
  }

  function try_FTScoreVar()
  {
    shiftT(228);                    // 'score'
    lookahead1W(21);                // S^WS | '$' | '(:'
    shiftT(31);                     // '$'
    lookahead1W(255);               // EQName^Token | S^WS | '(:' | 'after' | 'allowing' | 'ancestor' |
    try_VarName();
  }

  function parse_LetClause()
  {
    eventHandler.startNonterminal("LetClause", e0);
    shift(174);                     // 'let'
    lookahead1W(96);                // S^WS | '$' | '(:' | 'score'
    whitespace();
    parse_LetBinding();
    for (;;)
    {
      if (l1 != 41)                 // ','
      {
        break;
      }
      shift(41);                    // ','
      lookahead1W(96);              // S^WS | '$' | '(:' | 'score'
      whitespace();
      parse_LetBinding();
    }
    eventHandler.endNonterminal("LetClause", e0);
  }

  function try_LetClause()
  {
    shiftT(174);                    // 'let'
    lookahead1W(96);                // S^WS | '$' | '(:' | 'score'
    try_LetBinding();
    for (;;)
    {
      if (l1 != 41)                 // ','
      {
        break;
      }
      shiftT(41);                   // ','
      lookahead1W(96);              // S^WS | '$' | '(:' | 'score'
      try_LetBinding();
    }
  }

  function parse_LetBinding()
  {
    eventHandler.startNonterminal("LetBinding", e0);
    switch (l1)
    {
    case 31:                        // '$'
      shift(31);                    // '$'
      lookahead1W(255);             // EQName^Token | S^WS | '(:' | 'after' | 'allowing' | 'ancestor' |
      whitespace();
      parse_VarName();
      lookahead1W(105);             // S^WS | '(:' | ':=' | 'as'
      if (l1 == 79)                 // 'as'
      {
        whitespace();
        parse_TypeDeclaration();
      }
      break;
    default:
      parse_FTScoreVar();
    }
    lookahead1W(27);                // S^WS | '(:' | ':='
    shift(52);                      // ':='
    lookahead1W(267);               // Wildcard | EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral |
    whitespace();
    parse_ExprSingle();
    eventHandler.endNonterminal("LetBinding", e0);
  }

  function try_LetBinding()
  {
    switch (l1)
    {
    case 31:                        // '$'
      shiftT(31);                   // '$'
      lookahead1W(255);             // EQName^Token | S^WS | '(:' | 'after' | 'allowing' | 'ancestor' |
      try_VarName();
      lookahead1W(105);             // S^WS | '(:' | ':=' | 'as'
      if (l1 == 79)                 // 'as'
      {
        try_TypeDeclaration();
      }
      break;
    default:
      try_FTScoreVar();
    }
    lookahead1W(27);                // S^WS | '(:' | ':='
    shiftT(52);                     // ':='
    lookahead1W(267);               // Wildcard | EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral |
    try_ExprSingle();
  }

  function parse_WindowClause()
  {
    eventHandler.startNonterminal("WindowClause", e0);
    shift(137);                     // 'for'
    lookahead1W(135);               // S^WS | '(:' | 'sliding' | 'tumbling'
    switch (l1)
    {
    case 251:                       // 'tumbling'
      whitespace();
      parse_TumblingWindowClause();
      break;
    default:
      whitespace();
      parse_SlidingWindowClause();
    }
    eventHandler.endNonterminal("WindowClause", e0);
  }

  function try_WindowClause()
  {
    shiftT(137);                    // 'for'
    lookahead1W(135);               // S^WS | '(:' | 'sliding' | 'tumbling'
    switch (l1)
    {
    case 251:                       // 'tumbling'
      try_TumblingWindowClause();
      break;
    default:
      try_SlidingWindowClause();
    }
  }

  function parse_TumblingWindowClause()
  {
    eventHandler.startNonterminal("TumblingWindowClause", e0);
    shift(251);                     // 'tumbling'
    lookahead1W(85);                // S^WS | '(:' | 'window'
    shift(269);                     // 'window'
    lookahead1W(21);                // S^WS | '$' | '(:'
    shift(31);                      // '$'
    lookahead1W(255);               // EQName^Token | S^WS | '(:' | 'after' | 'allowing' | 'ancestor' |
    whitespace();
    parse_VarName();
    lookahead1W(110);               // S^WS | '(:' | 'as' | 'in'
    if (l1 == 79)                   // 'as'
    {
      whitespace();
      parse_TypeDeclaration();
    }
    lookahead1W(53);                // S^WS | '(:' | 'in'
    shift(154);                     // 'in'
    lookahead1W(267);               // Wildcard | EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral |
    whitespace();
    parse_ExprSingle();
    whitespace();
    parse_WindowStartCondition();
    if (l1 == 126                   // 'end'
     || l1 == 198)                  // 'only'
    {
      whitespace();
      parse_WindowEndCondition();
    }
    eventHandler.endNonterminal("TumblingWindowClause", e0);
  }

  function try_TumblingWindowClause()
  {
    shiftT(251);                    // 'tumbling'
    lookahead1W(85);                // S^WS | '(:' | 'window'
    shiftT(269);                    // 'window'
    lookahead1W(21);                // S^WS | '$' | '(:'
    shiftT(31);                     // '$'
    lookahead1W(255);               // EQName^Token | S^WS | '(:' | 'after' | 'allowing' | 'ancestor' |
    try_VarName();
    lookahead1W(110);               // S^WS | '(:' | 'as' | 'in'
    if (l1 == 79)                   // 'as'
    {
      try_TypeDeclaration();
    }
    lookahead1W(53);                // S^WS | '(:' | 'in'
    shiftT(154);                    // 'in'
    lookahead1W(267);               // Wildcard | EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral |
    try_ExprSingle();
    try_WindowStartCondition();
    if (l1 == 126                   // 'end'
     || l1 == 198)                  // 'only'
    {
      try_WindowEndCondition();
    }
  }

  function parse_SlidingWindowClause()
  {
    eventHandler.startNonterminal("SlidingWindowClause", e0);
    shift(234);                     // 'sliding'
    lookahead1W(85);                // S^WS | '(:' | 'window'
    shift(269);                     // 'window'
    lookahead1W(21);                // S^WS | '$' | '(:'
    shift(31);                      // '$'
    lookahead1W(255);               // EQName^Token | S^WS | '(:' | 'after' | 'allowing' | 'ancestor' |
    whitespace();
    parse_VarName();
    lookahead1W(110);               // S^WS | '(:' | 'as' | 'in'
    if (l1 == 79)                   // 'as'
    {
      whitespace();
      parse_TypeDeclaration();
    }
    lookahead1W(53);                // S^WS | '(:' | 'in'
    shift(154);                     // 'in'
    lookahead1W(267);               // Wildcard | EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral |
    whitespace();
    parse_ExprSingle();
    whitespace();
    parse_WindowStartCondition();
    whitespace();
    parse_WindowEndCondition();
    eventHandler.endNonterminal("SlidingWindowClause", e0);
  }

  function try_SlidingWindowClause()
  {
    shiftT(234);                    // 'sliding'
    lookahead1W(85);                // S^WS | '(:' | 'window'
    shiftT(269);                    // 'window'
    lookahead1W(21);                // S^WS | '$' | '(:'
    shiftT(31);                     // '$'
    lookahead1W(255);               // EQName^Token | S^WS | '(:' | 'after' | 'allowing' | 'ancestor' |
    try_VarName();
    lookahead1W(110);               // S^WS | '(:' | 'as' | 'in'
    if (l1 == 79)                   // 'as'
    {
      try_TypeDeclaration();
    }
    lookahead1W(53);                // S^WS | '(:' | 'in'
    shiftT(154);                    // 'in'
    lookahead1W(267);               // Wildcard | EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral |
    try_ExprSingle();
    try_WindowStartCondition();
    try_WindowEndCondition();
  }

  function parse_WindowStartCondition()
  {
    eventHandler.startNonterminal("WindowStartCondition", e0);
    shift(237);                     // 'start'
    lookahead1W(163);               // S^WS | '$' | '(:' | 'at' | 'next' | 'previous' | 'when'
    whitespace();
    parse_WindowVars();
    lookahead1W(83);                // S^WS | '(:' | 'when'
    shift(265);                     // 'when'
    lookahead1W(267);               // Wildcard | EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral |
    whitespace();
    parse_ExprSingle();
    eventHandler.endNonterminal("WindowStartCondition", e0);
  }

  function try_WindowStartCondition()
  {
    shiftT(237);                    // 'start'
    lookahead1W(163);               // S^WS | '$' | '(:' | 'at' | 'next' | 'previous' | 'when'
    try_WindowVars();
    lookahead1W(83);                // S^WS | '(:' | 'when'
    shiftT(265);                    // 'when'
    lookahead1W(267);               // Wildcard | EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral |
    try_ExprSingle();
  }

  function parse_WindowEndCondition()
  {
    eventHandler.startNonterminal("WindowEndCondition", e0);
    if (l1 == 198)                  // 'only'
    {
      shift(198);                   // 'only'
    }
    lookahead1W(50);                // S^WS | '(:' | 'end'
    shift(126);                     // 'end'
    lookahead1W(163);               // S^WS | '$' | '(:' | 'at' | 'next' | 'previous' | 'when'
    whitespace();
    parse_WindowVars();
    lookahead1W(83);                // S^WS | '(:' | 'when'
    shift(265);                     // 'when'
    lookahead1W(267);               // Wildcard | EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral |
    whitespace();
    parse_ExprSingle();
    eventHandler.endNonterminal("WindowEndCondition", e0);
  }

  function try_WindowEndCondition()
  {
    if (l1 == 198)                  // 'only'
    {
      shiftT(198);                  // 'only'
    }
    lookahead1W(50);                // S^WS | '(:' | 'end'
    shiftT(126);                    // 'end'
    lookahead1W(163);               // S^WS | '$' | '(:' | 'at' | 'next' | 'previous' | 'when'
    try_WindowVars();
    lookahead1W(83);                // S^WS | '(:' | 'when'
    shiftT(265);                    // 'when'
    lookahead1W(267);               // Wildcard | EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral |
    try_ExprSingle();
  }

  function parse_WindowVars()
  {
    eventHandler.startNonterminal("WindowVars", e0);
    if (l1 == 31)                   // '$'
    {
      shift(31);                    // '$'
      lookahead1W(255);             // EQName^Token | S^WS | '(:' | 'after' | 'allowing' | 'ancestor' |
      whitespace();
      parse_CurrentItem();
    }
    lookahead1W(159);               // S^WS | '(:' | 'at' | 'next' | 'previous' | 'when'
    if (l1 == 81)                   // 'at'
    {
      whitespace();
      parse_PositionalVar();
    }
    lookahead1W(153);               // S^WS | '(:' | 'next' | 'previous' | 'when'
    if (l1 == 215)                  // 'previous'
    {
      shift(215);                   // 'previous'
      lookahead1W(21);              // S^WS | '$' | '(:'
      shift(31);                    // '$'
      lookahead1W(255);             // EQName^Token | S^WS | '(:' | 'after' | 'allowing' | 'ancestor' |
      whitespace();
      parse_PreviousItem();
    }
    lookahead1W(127);               // S^WS | '(:' | 'next' | 'when'
    if (l1 == 187)                  // 'next'
    {
      shift(187);                   // 'next'
      lookahead1W(21);              // S^WS | '$' | '(:'
      shift(31);                    // '$'
      lookahead1W(255);             // EQName^Token | S^WS | '(:' | 'after' | 'allowing' | 'ancestor' |
      whitespace();
      parse_NextItem();
    }
    eventHandler.endNonterminal("WindowVars", e0);
  }

  function try_WindowVars()
  {
    if (l1 == 31)                   // '$'
    {
      shiftT(31);                   // '$'
      lookahead1W(255);             // EQName^Token | S^WS | '(:' | 'after' | 'allowing' | 'ancestor' |
      try_CurrentItem();
    }
    lookahead1W(159);               // S^WS | '(:' | 'at' | 'next' | 'previous' | 'when'
    if (l1 == 81)                   // 'at'
    {
      try_PositionalVar();
    }
    lookahead1W(153);               // S^WS | '(:' | 'next' | 'previous' | 'when'
    if (l1 == 215)                  // 'previous'
    {
      shiftT(215);                  // 'previous'
      lookahead1W(21);              // S^WS | '$' | '(:'
      shiftT(31);                   // '$'
      lookahead1W(255);             // EQName^Token | S^WS | '(:' | 'after' | 'allowing' | 'ancestor' |
      try_PreviousItem();
    }
    lookahead1W(127);               // S^WS | '(:' | 'next' | 'when'
    if (l1 == 187)                  // 'next'
    {
      shiftT(187);                  // 'next'
      lookahead1W(21);              // S^WS | '$' | '(:'
      shiftT(31);                   // '$'
      lookahead1W(255);             // EQName^Token | S^WS | '(:' | 'after' | 'allowing' | 'ancestor' |
      try_NextItem();
    }
  }

  function parse_CurrentItem()
  {
    eventHandler.startNonterminal("CurrentItem", e0);
    parse_EQName();
    eventHandler.endNonterminal("CurrentItem", e0);
  }

  function try_CurrentItem()
  {
    try_EQName();
  }

  function parse_PreviousItem()
  {
    eventHandler.startNonterminal("PreviousItem", e0);
    parse_EQName();
    eventHandler.endNonterminal("PreviousItem", e0);
  }

  function try_PreviousItem()
  {
    try_EQName();
  }

  function parse_NextItem()
  {
    eventHandler.startNonterminal("NextItem", e0);
    parse_EQName();
    eventHandler.endNonterminal("NextItem", e0);
  }

  function try_NextItem()
  {
    try_EQName();
  }

  function parse_CountClause()
  {
    eventHandler.startNonterminal("CountClause", e0);
    shift(105);                     // 'count'
    lookahead1W(21);                // S^WS | '$' | '(:'
    shift(31);                      // '$'
    lookahead1W(255);               // EQName^Token | S^WS | '(:' | 'after' | 'allowing' | 'ancestor' |
    whitespace();
    parse_VarName();
    eventHandler.endNonterminal("CountClause", e0);
  }

  function try_CountClause()
  {
    shiftT(105);                    // 'count'
    lookahead1W(21);                // S^WS | '$' | '(:'
    shiftT(31);                     // '$'
    lookahead1W(255);               // EQName^Token | S^WS | '(:' | 'after' | 'allowing' | 'ancestor' |
    try_VarName();
  }

  function parse_WhereClause()
  {
    eventHandler.startNonterminal("WhereClause", e0);
    shift(266);                     // 'where'
    lookahead1W(267);               // Wildcard | EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral |
    whitespace();
    parse_ExprSingle();
    eventHandler.endNonterminal("WhereClause", e0);
  }

  function try_WhereClause()
  {
    shiftT(266);                    // 'where'
    lookahead1W(267);               // Wildcard | EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral |
    try_ExprSingle();
  }

  function parse_GroupByClause()
  {
    eventHandler.startNonterminal("GroupByClause", e0);
    shift(148);                     // 'group'
    lookahead1W(34);                // S^WS | '(:' | 'by'
    shift(87);                      // 'by'
    lookahead1W(267);               // Wildcard | EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral |
    whitespace();
    parse_GroupingSpecList();
    eventHandler.endNonterminal("GroupByClause", e0);
  }

  function try_GroupByClause()
  {
    shiftT(148);                    // 'group'
    lookahead1W(34);                // S^WS | '(:' | 'by'
    shiftT(87);                     // 'by'
    lookahead1W(267);               // Wildcard | EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral |
    try_GroupingSpecList();
  }

  function parse_GroupingSpecList()
  {
    eventHandler.startNonterminal("GroupingSpecList", e0);
    parse_GroupingSpec();
    for (;;)
    {
      lookahead1W(176);             // S^WS | '(:' | ',' | 'count' | 'for' | 'group' | 'let' | 'order' | 'return' |
      if (l1 != 41)                 // ','
      {
        break;
      }
      shift(41);                    // ','
      lookahead1W(267);             // Wildcard | EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral |
      whitespace();
      parse_GroupingSpec();
    }
    eventHandler.endNonterminal("GroupingSpecList", e0);
  }

  function try_GroupingSpecList()
  {
    try_GroupingSpec();
    for (;;)
    {
      lookahead1W(176);             // S^WS | '(:' | ',' | 'count' | 'for' | 'group' | 'let' | 'order' | 'return' |
      if (l1 != 41)                 // ','
      {
        break;
      }
      shiftT(41);                   // ','
      lookahead1W(267);             // Wildcard | EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral |
      try_GroupingSpec();
    }
  }

  function parse_GroupingSpec()
  {
    eventHandler.startNonterminal("GroupingSpec", e0);
    switch (l1)
    {
    case 31:                        // '$'
      lookahead2W(255);             // EQName^Token | S^WS | '(:' | 'after' | 'allowing' | 'ancestor' |
      break;
    default:
      lk = l1;
    }
    if (lk == 3103                  // '$' EQName^Token
     || lk == 35871                 // '$' 'after'
     || lk == 36895                 // '$' 'allowing'
     || lk == 37407                 // '$' 'ancestor'
     || lk == 37919                 // '$' 'ancestor-or-self'
     || lk == 38431                 // '$' 'and'
     || lk == 39455                 // '$' 'append'
     || lk == 39967                 // '$' 'array'
     || lk == 40479                 // '$' 'as'
     || lk == 40991                 // '$' 'ascending'
     || lk == 41503                 // '$' 'at'
     || lk == 42015                 // '$' 'attribute'
     || lk == 42527                 // '$' 'base-uri'
     || lk == 43039                 // '$' 'before'
     || lk == 43551                 // '$' 'boundary-space'
     || lk == 44063                 // '$' 'break'
     || lk == 45087                 // '$' 'case'
     || lk == 45599                 // '$' 'cast'
     || lk == 46111                 // '$' 'castable'
     || lk == 46623                 // '$' 'catch'
     || lk == 47647                 // '$' 'child'
     || lk == 48159                 // '$' 'collation'
     || lk == 49183                 // '$' 'comment'
     || lk == 49695                 // '$' 'constraint'
     || lk == 50207                 // '$' 'construction'
     || lk == 51743                 // '$' 'context'
     || lk == 52255                 // '$' 'continue'
     || lk == 52767                 // '$' 'copy'
     || lk == 53279                 // '$' 'copy-namespaces'
     || lk == 53791                 // '$' 'count'
     || lk == 54303                 // '$' 'decimal-format'
     || lk == 55327                 // '$' 'declare'
     || lk == 55839                 // '$' 'default'
     || lk == 56351                 // '$' 'delete'
     || lk == 56863                 // '$' 'descendant'
     || lk == 57375                 // '$' 'descendant-or-self'
     || lk == 57887                 // '$' 'descending'
     || lk == 60447                 // '$' 'div'
     || lk == 60959                 // '$' 'document'
     || lk == 61471                 // '$' 'document-node'
     || lk == 61983                 // '$' 'element'
     || lk == 62495                 // '$' 'else'
     || lk == 63007                 // '$' 'empty'
     || lk == 63519                 // '$' 'empty-sequence'
     || lk == 64031                 // '$' 'encoding'
     || lk == 64543                 // '$' 'end'
     || lk == 65567                 // '$' 'eq'
     || lk == 66079                 // '$' 'every'
     || lk == 67103                 // '$' 'except'
     || lk == 67615                 // '$' 'exit'
     || lk == 68127                 // '$' 'external'
     || lk == 68639                 // '$' 'first'
     || lk == 69151                 // '$' 'following'
     || lk == 69663                 // '$' 'following-sibling'
     || lk == 70175                 // '$' 'for'
     || lk == 72223                 // '$' 'ft-option'
     || lk == 74271                 // '$' 'function'
     || lk == 74783                 // '$' 'ge'
     || lk == 75807                 // '$' 'group'
     || lk == 76831                 // '$' 'gt'
     || lk == 77343                 // '$' 'idiv'
     || lk == 77855                 // '$' 'if'
     || lk == 78367                 // '$' 'import'
     || lk == 78879                 // '$' 'in'
     || lk == 79391                 // '$' 'index'
     || lk == 81439                 // '$' 'insert'
     || lk == 81951                 // '$' 'instance'
     || lk == 82463                 // '$' 'integrity'
     || lk == 82975                 // '$' 'intersect'
     || lk == 83487                 // '$' 'into'
     || lk == 83999                 // '$' 'is'
     || lk == 84511                 // '$' 'item'
     || lk == 85023                 // '$' 'json'
     || lk == 85535                 // '$' 'json-item'
     || lk == 87071                 // '$' 'last'
     || lk == 87583                 // '$' 'lax'
     || lk == 88095                 // '$' 'le'
     || lk == 89119                 // '$' 'let'
     || lk == 90143                 // '$' 'loop'
     || lk == 91167                 // '$' 'lt'
     || lk == 92191                 // '$' 'mod'
     || lk == 92703                 // '$' 'modify'
     || lk == 93215                 // '$' 'module'
     || lk == 94239                 // '$' 'namespace'
     || lk == 94751                 // '$' 'namespace-node'
     || lk == 95263                 // '$' 'ne'
     || lk == 97823                 // '$' 'node'
     || lk == 98335                 // '$' 'nodes'
     || lk == 99359                 // '$' 'object'
     || lk == 101407                // '$' 'only'
     || lk == 101919                // '$' 'option'
     || lk == 102431                // '$' 'or'
     || lk == 102943                // '$' 'order'
     || lk == 103455                // '$' 'ordered'
     || lk == 103967                // '$' 'ordering'
     || lk == 105503                // '$' 'parent'
     || lk == 108575                // '$' 'preceding'
     || lk == 109087                // '$' 'preceding-sibling'
     || lk == 110623                // '$' 'processing-instruction'
     || lk == 111647                // '$' 'rename'
     || lk == 112159                // '$' 'replace'
     || lk == 112671                // '$' 'return'
     || lk == 113183                // '$' 'returning'
     || lk == 113695                // '$' 'revalidation'
     || lk == 114719                // '$' 'satisfies'
     || lk == 115231                // '$' 'schema'
     || lk == 115743                // '$' 'schema-attribute'
     || lk == 116255                // '$' 'schema-element'
     || lk == 116767                // '$' 'score'
     || lk == 117279                // '$' 'self'
     || lk == 119839                // '$' 'sliding'
     || lk == 120351                // '$' 'some'
     || lk == 120863                // '$' 'stable'
     || lk == 121375                // '$' 'start'
     || lk == 122911                // '$' 'strict'
     || lk == 123935                // '$' 'structured-item'
     || lk == 124447                // '$' 'switch'
     || lk == 124959                // '$' 'text'
     || lk == 127007                // '$' 'to'
     || lk == 127519                // '$' 'treat'
     || lk == 128031                // '$' 'try'
     || lk == 128543                // '$' 'tumbling'
     || lk == 129055                // '$' 'type'
     || lk == 129567                // '$' 'typeswitch'
     || lk == 130079                // '$' 'union'
     || lk == 131103                // '$' 'unordered'
     || lk == 131615                // '$' 'updating'
     || lk == 133151                // '$' 'validate'
     || lk == 133663                // '$' 'value'
     || lk == 134175                // '$' 'variable'
     || lk == 134687                // '$' 'version'
     || lk == 136223                // '$' 'where'
     || lk == 136735                // '$' 'while'
     || lk == 138271                // '$' 'with'
     || lk == 140319)               // '$' 'xquery'
    {
      lk = memoized(2, e0);
      if (lk == 0)
      {
        var b0A = b0; var e0A = e0; var l1A = l1;
        var b1A = b1; var e1A = e1; var l2A = l2;
        var b2A = b2; var e2A = e2;
        try
        {
          try_GroupingVariable();
          lookahead1W(183);         // S^WS | '(:' | ',' | ':=' | 'as' | 'collation' | 'count' | 'for' | 'group' |
          if (l1 == 52              // ':='
           || l1 == 79)             // 'as'
          {
            if (l1 == 79)           // 'as'
            {
              try_TypeDeclaration();
            }
            lookahead1W(27);        // S^WS | '(:' | ':='
            shiftT(52);             // ':='
            lookahead1W(267);       // Wildcard | EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral |
            try_ExprSingle();
          }
          if (l1 == 94)             // 'collation'
          {
            shiftT(94);             // 'collation'
            lookahead1W(15);        // URILiteral | S^WS | '(:'
            shiftT(7);              // URILiteral
          }
          lk = -1;
        }
        catch (p1A)
        {
          lk = -2;
        }
        b0 = b0A; e0 = e0A; l1 = l1A; if (l1 == 0) {end = e0A;} else {
        b1 = b1A; e1 = e1A; l2 = l2A; if (l2 == 0) {end = e1A;} else {
        b2 = b2A; e2 = e2A; end = e2A; }}
        memoize(2, e0, lk);
      }
    }
    switch (lk)
    {
    case -1:
      parse_GroupingVariable();
      lookahead1W(183);             // S^WS | '(:' | ',' | ':=' | 'as' | 'collation' | 'count' | 'for' | 'group' |
      if (l1 == 52                  // ':='
       || l1 == 79)                 // 'as'
      {
        if (l1 == 79)               // 'as'
        {
          whitespace();
          parse_TypeDeclaration();
        }
        lookahead1W(27);            // S^WS | '(:' | ':='
        shift(52);                  // ':='
        lookahead1W(267);           // Wildcard | EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral |
        whitespace();
        parse_ExprSingle();
      }
      if (l1 == 94)                 // 'collation'
      {
        shift(94);                  // 'collation'
        lookahead1W(15);            // URILiteral | S^WS | '(:'
        shift(7);                   // URILiteral
      }
      break;
    default:
      parse_ExprSingle();
    }
    eventHandler.endNonterminal("GroupingSpec", e0);
  }

  function try_GroupingSpec()
  {
    switch (l1)
    {
    case 31:                        // '$'
      lookahead2W(255);             // EQName^Token | S^WS | '(:' | 'after' | 'allowing' | 'ancestor' |
      break;
    default:
      lk = l1;
    }
    if (lk == 3103                  // '$' EQName^Token
     || lk == 35871                 // '$' 'after'
     || lk == 36895                 // '$' 'allowing'
     || lk == 37407                 // '$' 'ancestor'
     || lk == 37919                 // '$' 'ancestor-or-self'
     || lk == 38431                 // '$' 'and'
     || lk == 39455                 // '$' 'append'
     || lk == 39967                 // '$' 'array'
     || lk == 40479                 // '$' 'as'
     || lk == 40991                 // '$' 'ascending'
     || lk == 41503                 // '$' 'at'
     || lk == 42015                 // '$' 'attribute'
     || lk == 42527                 // '$' 'base-uri'
     || lk == 43039                 // '$' 'before'
     || lk == 43551                 // '$' 'boundary-space'
     || lk == 44063                 // '$' 'break'
     || lk == 45087                 // '$' 'case'
     || lk == 45599                 // '$' 'cast'
     || lk == 46111                 // '$' 'castable'
     || lk == 46623                 // '$' 'catch'
     || lk == 47647                 // '$' 'child'
     || lk == 48159                 // '$' 'collation'
     || lk == 49183                 // '$' 'comment'
     || lk == 49695                 // '$' 'constraint'
     || lk == 50207                 // '$' 'construction'
     || lk == 51743                 // '$' 'context'
     || lk == 52255                 // '$' 'continue'
     || lk == 52767                 // '$' 'copy'
     || lk == 53279                 // '$' 'copy-namespaces'
     || lk == 53791                 // '$' 'count'
     || lk == 54303                 // '$' 'decimal-format'
     || lk == 55327                 // '$' 'declare'
     || lk == 55839                 // '$' 'default'
     || lk == 56351                 // '$' 'delete'
     || lk == 56863                 // '$' 'descendant'
     || lk == 57375                 // '$' 'descendant-or-self'
     || lk == 57887                 // '$' 'descending'
     || lk == 60447                 // '$' 'div'
     || lk == 60959                 // '$' 'document'
     || lk == 61471                 // '$' 'document-node'
     || lk == 61983                 // '$' 'element'
     || lk == 62495                 // '$' 'else'
     || lk == 63007                 // '$' 'empty'
     || lk == 63519                 // '$' 'empty-sequence'
     || lk == 64031                 // '$' 'encoding'
     || lk == 64543                 // '$' 'end'
     || lk == 65567                 // '$' 'eq'
     || lk == 66079                 // '$' 'every'
     || lk == 67103                 // '$' 'except'
     || lk == 67615                 // '$' 'exit'
     || lk == 68127                 // '$' 'external'
     || lk == 68639                 // '$' 'first'
     || lk == 69151                 // '$' 'following'
     || lk == 69663                 // '$' 'following-sibling'
     || lk == 70175                 // '$' 'for'
     || lk == 72223                 // '$' 'ft-option'
     || lk == 74271                 // '$' 'function'
     || lk == 74783                 // '$' 'ge'
     || lk == 75807                 // '$' 'group'
     || lk == 76831                 // '$' 'gt'
     || lk == 77343                 // '$' 'idiv'
     || lk == 77855                 // '$' 'if'
     || lk == 78367                 // '$' 'import'
     || lk == 78879                 // '$' 'in'
     || lk == 79391                 // '$' 'index'
     || lk == 81439                 // '$' 'insert'
     || lk == 81951                 // '$' 'instance'
     || lk == 82463                 // '$' 'integrity'
     || lk == 82975                 // '$' 'intersect'
     || lk == 83487                 // '$' 'into'
     || lk == 83999                 // '$' 'is'
     || lk == 84511                 // '$' 'item'
     || lk == 85023                 // '$' 'json'
     || lk == 85535                 // '$' 'json-item'
     || lk == 87071                 // '$' 'last'
     || lk == 87583                 // '$' 'lax'
     || lk == 88095                 // '$' 'le'
     || lk == 89119                 // '$' 'let'
     || lk == 90143                 // '$' 'loop'
     || lk == 91167                 // '$' 'lt'
     || lk == 92191                 // '$' 'mod'
     || lk == 92703                 // '$' 'modify'
     || lk == 93215                 // '$' 'module'
     || lk == 94239                 // '$' 'namespace'
     || lk == 94751                 // '$' 'namespace-node'
     || lk == 95263                 // '$' 'ne'
     || lk == 97823                 // '$' 'node'
     || lk == 98335                 // '$' 'nodes'
     || lk == 99359                 // '$' 'object'
     || lk == 101407                // '$' 'only'
     || lk == 101919                // '$' 'option'
     || lk == 102431                // '$' 'or'
     || lk == 102943                // '$' 'order'
     || lk == 103455                // '$' 'ordered'
     || lk == 103967                // '$' 'ordering'
     || lk == 105503                // '$' 'parent'
     || lk == 108575                // '$' 'preceding'
     || lk == 109087                // '$' 'preceding-sibling'
     || lk == 110623                // '$' 'processing-instruction'
     || lk == 111647                // '$' 'rename'
     || lk == 112159                // '$' 'replace'
     || lk == 112671                // '$' 'return'
     || lk == 113183                // '$' 'returning'
     || lk == 113695                // '$' 'revalidation'
     || lk == 114719                // '$' 'satisfies'
     || lk == 115231                // '$' 'schema'
     || lk == 115743                // '$' 'schema-attribute'
     || lk == 116255                // '$' 'schema-element'
     || lk == 116767                // '$' 'score'
     || lk == 117279                // '$' 'self'
     || lk == 119839                // '$' 'sliding'
     || lk == 120351                // '$' 'some'
     || lk == 120863                // '$' 'stable'
     || lk == 121375                // '$' 'start'
     || lk == 122911                // '$' 'strict'
     || lk == 123935                // '$' 'structured-item'
     || lk == 124447                // '$' 'switch'
     || lk == 124959                // '$' 'text'
     || lk == 127007                // '$' 'to'
     || lk == 127519                // '$' 'treat'
     || lk == 128031                // '$' 'try'
     || lk == 128543                // '$' 'tumbling'
     || lk == 129055                // '$' 'type'
     || lk == 129567                // '$' 'typeswitch'
     || lk == 130079                // '$' 'union'
     || lk == 131103                // '$' 'unordered'
     || lk == 131615                // '$' 'updating'
     || lk == 133151                // '$' 'validate'
     || lk == 133663                // '$' 'value'
     || lk == 134175                // '$' 'variable'
     || lk == 134687                // '$' 'version'
     || lk == 136223                // '$' 'where'
     || lk == 136735                // '$' 'while'
     || lk == 138271                // '$' 'with'
     || lk == 140319)               // '$' 'xquery'
    {
      lk = memoized(2, e0);
      if (lk == 0)
      {
        var b0A = b0; var e0A = e0; var l1A = l1;
        var b1A = b1; var e1A = e1; var l2A = l2;
        var b2A = b2; var e2A = e2;
        try
        {
          try_GroupingVariable();
          lookahead1W(183);         // S^WS | '(:' | ',' | ':=' | 'as' | 'collation' | 'count' | 'for' | 'group' |
          if (l1 == 52              // ':='
           || l1 == 79)             // 'as'
          {
            if (l1 == 79)           // 'as'
            {
              try_TypeDeclaration();
            }
            lookahead1W(27);        // S^WS | '(:' | ':='
            shiftT(52);             // ':='
            lookahead1W(267);       // Wildcard | EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral |
            try_ExprSingle();
          }
          if (l1 == 94)             // 'collation'
          {
            shiftT(94);             // 'collation'
            lookahead1W(15);        // URILiteral | S^WS | '(:'
            shiftT(7);              // URILiteral
          }
          memoize(2, e0A, -1);
          lk = -3;
        }
        catch (p1A)
        {
          lk = -2;
          b0 = b0A; e0 = e0A; l1 = l1A; if (l1 == 0) {end = e0A;} else {
          b1 = b1A; e1 = e1A; l2 = l2A; if (l2 == 0) {end = e1A;} else {
          b2 = b2A; e2 = e2A; end = e2A; }}
          memoize(2, e0A, -2);
        }
      }
    }
    switch (lk)
    {
    case -1:
      try_GroupingVariable();
      lookahead1W(183);             // S^WS | '(:' | ',' | ':=' | 'as' | 'collation' | 'count' | 'for' | 'group' |
      if (l1 == 52                  // ':='
       || l1 == 79)                 // 'as'
      {
        if (l1 == 79)               // 'as'
        {
          try_TypeDeclaration();
        }
        lookahead1W(27);            // S^WS | '(:' | ':='
        shiftT(52);                 // ':='
        lookahead1W(267);           // Wildcard | EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral |
        try_ExprSingle();
      }
      if (l1 == 94)                 // 'collation'
      {
        shiftT(94);                 // 'collation'
        lookahead1W(15);            // URILiteral | S^WS | '(:'
        shiftT(7);                  // URILiteral
      }
      break;
    case -3:
      break;
    default:
      try_ExprSingle();
    }
  }

  function parse_GroupingVariable()
  {
    eventHandler.startNonterminal("GroupingVariable", e0);
    shift(31);                      // '$'
    lookahead1W(255);               // EQName^Token | S^WS | '(:' | 'after' | 'allowing' | 'ancestor' |
    whitespace();
    parse_VarName();
    eventHandler.endNonterminal("GroupingVariable", e0);
  }

  function try_GroupingVariable()
  {
    shiftT(31);                     // '$'
    lookahead1W(255);               // EQName^Token | S^WS | '(:' | 'after' | 'allowing' | 'ancestor' |
    try_VarName();
  }

  function parse_OrderByClause()
  {
    eventHandler.startNonterminal("OrderByClause", e0);
    switch (l1)
    {
    case 201:                       // 'order'
      shift(201);                   // 'order'
      lookahead1W(34);              // S^WS | '(:' | 'by'
      shift(87);                    // 'by'
      break;
    default:
      shift(236);                   // 'stable'
      lookahead1W(67);              // S^WS | '(:' | 'order'
      shift(201);                   // 'order'
      lookahead1W(34);              // S^WS | '(:' | 'by'
      shift(87);                    // 'by'
    }
    lookahead1W(267);               // Wildcard | EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral |
    whitespace();
    parse_OrderSpecList();
    eventHandler.endNonterminal("OrderByClause", e0);
  }

  function try_OrderByClause()
  {
    switch (l1)
    {
    case 201:                       // 'order'
      shiftT(201);                  // 'order'
      lookahead1W(34);              // S^WS | '(:' | 'by'
      shiftT(87);                   // 'by'
      break;
    default:
      shiftT(236);                  // 'stable'
      lookahead1W(67);              // S^WS | '(:' | 'order'
      shiftT(201);                  // 'order'
      lookahead1W(34);              // S^WS | '(:' | 'by'
      shiftT(87);                   // 'by'
    }
    lookahead1W(267);               // Wildcard | EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral |
    try_OrderSpecList();
  }

  function parse_OrderSpecList()
  {
    eventHandler.startNonterminal("OrderSpecList", e0);
    parse_OrderSpec();
    for (;;)
    {
      lookahead1W(176);             // S^WS | '(:' | ',' | 'count' | 'for' | 'group' | 'let' | 'order' | 'return' |
      if (l1 != 41)                 // ','
      {
        break;
      }
      shift(41);                    // ','
      lookahead1W(267);             // Wildcard | EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral |
      whitespace();
      parse_OrderSpec();
    }
    eventHandler.endNonterminal("OrderSpecList", e0);
  }

  function try_OrderSpecList()
  {
    try_OrderSpec();
    for (;;)
    {
      lookahead1W(176);             // S^WS | '(:' | ',' | 'count' | 'for' | 'group' | 'let' | 'order' | 'return' |
      if (l1 != 41)                 // ','
      {
        break;
      }
      shiftT(41);                   // ','
      lookahead1W(267);             // Wildcard | EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral |
      try_OrderSpec();
    }
  }

  function parse_OrderSpec()
  {
    eventHandler.startNonterminal("OrderSpec", e0);
    parse_ExprSingle();
    whitespace();
    parse_OrderModifier();
    eventHandler.endNonterminal("OrderSpec", e0);
  }

  function try_OrderSpec()
  {
    try_ExprSingle();
    try_OrderModifier();
  }

  function parse_OrderModifier()
  {
    eventHandler.startNonterminal("OrderModifier", e0);
    if (l1 == 80                    // 'ascending'
     || l1 == 113)                  // 'descending'
    {
      switch (l1)
      {
      case 80:                      // 'ascending'
        shift(80);                  // 'ascending'
        break;
      default:
        shift(113);                 // 'descending'
      }
    }
    lookahead1W(180);               // S^WS | '(:' | ',' | 'collation' | 'count' | 'empty' | 'for' | 'group' | 'let' |
    if (l1 == 123)                  // 'empty'
    {
      shift(123);                   // 'empty'
      lookahead1W(121);             // S^WS | '(:' | 'greatest' | 'least'
      switch (l1)
      {
      case 147:                     // 'greatest'
        shift(147);                 // 'greatest'
        break;
      default:
        shift(173);                 // 'least'
      }
    }
    lookahead1W(177);               // S^WS | '(:' | ',' | 'collation' | 'count' | 'for' | 'group' | 'let' | 'order' |
    if (l1 == 94)                   // 'collation'
    {
      shift(94);                    // 'collation'
      lookahead1W(15);              // URILiteral | S^WS | '(:'
      shift(7);                     // URILiteral
    }
    eventHandler.endNonterminal("OrderModifier", e0);
  }

  function try_OrderModifier()
  {
    if (l1 == 80                    // 'ascending'
     || l1 == 113)                  // 'descending'
    {
      switch (l1)
      {
      case 80:                      // 'ascending'
        shiftT(80);                 // 'ascending'
        break;
      default:
        shiftT(113);                // 'descending'
      }
    }
    lookahead1W(180);               // S^WS | '(:' | ',' | 'collation' | 'count' | 'empty' | 'for' | 'group' | 'let' |
    if (l1 == 123)                  // 'empty'
    {
      shiftT(123);                  // 'empty'
      lookahead1W(121);             // S^WS | '(:' | 'greatest' | 'least'
      switch (l1)
      {
      case 147:                     // 'greatest'
        shiftT(147);                // 'greatest'
        break;
      default:
        shiftT(173);                // 'least'
      }
    }
    lookahead1W(177);               // S^WS | '(:' | ',' | 'collation' | 'count' | 'for' | 'group' | 'let' | 'order' |
    if (l1 == 94)                   // 'collation'
    {
      shiftT(94);                   // 'collation'
      lookahead1W(15);              // URILiteral | S^WS | '(:'
      shiftT(7);                    // URILiteral
    }
  }

  function parse_ReturnClause()
  {
    eventHandler.startNonterminal("ReturnClause", e0);
    shift(220);                     // 'return'
    lookahead1W(267);               // Wildcard | EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral |
    whitespace();
    parse_ExprSingle();
    eventHandler.endNonterminal("ReturnClause", e0);
  }

  function try_ReturnClause()
  {
    shiftT(220);                    // 'return'
    lookahead1W(267);               // Wildcard | EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral |
    try_ExprSingle();
  }

  function parse_QuantifiedExpr()
  {
    eventHandler.startNonterminal("QuantifiedExpr", e0);
    switch (l1)
    {
    case 235:                       // 'some'
      shift(235);                   // 'some'
      break;
    default:
      shift(129);                   // 'every'
    }
    lookahead1W(21);                // S^WS | '$' | '(:'
    whitespace();
    parse_QuantifiedVarDecl();
    for (;;)
    {
      if (l1 != 41)                 // ','
      {
        break;
      }
      shift(41);                    // ','
      lookahead1W(21);              // S^WS | '$' | '(:'
      whitespace();
      parse_QuantifiedVarDecl();
    }
    shift(224);                     // 'satisfies'
    lookahead1W(267);               // Wildcard | EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral |
    whitespace();
    parse_ExprSingle();
    eventHandler.endNonterminal("QuantifiedExpr", e0);
  }

  function try_QuantifiedExpr()
  {
    switch (l1)
    {
    case 235:                       // 'some'
      shiftT(235);                  // 'some'
      break;
    default:
      shiftT(129);                  // 'every'
    }
    lookahead1W(21);                // S^WS | '$' | '(:'
    try_QuantifiedVarDecl();
    for (;;)
    {
      if (l1 != 41)                 // ','
      {
        break;
      }
      shiftT(41);                   // ','
      lookahead1W(21);              // S^WS | '$' | '(:'
      try_QuantifiedVarDecl();
    }
    shiftT(224);                    // 'satisfies'
    lookahead1W(267);               // Wildcard | EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral |
    try_ExprSingle();
  }

  function parse_QuantifiedVarDecl()
  {
    eventHandler.startNonterminal("QuantifiedVarDecl", e0);
    shift(31);                      // '$'
    lookahead1W(255);               // EQName^Token | S^WS | '(:' | 'after' | 'allowing' | 'ancestor' |
    whitespace();
    parse_VarName();
    lookahead1W(110);               // S^WS | '(:' | 'as' | 'in'
    if (l1 == 79)                   // 'as'
    {
      whitespace();
      parse_TypeDeclaration();
    }
    lookahead1W(53);                // S^WS | '(:' | 'in'
    shift(154);                     // 'in'
    lookahead1W(267);               // Wildcard | EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral |
    whitespace();
    parse_ExprSingle();
    eventHandler.endNonterminal("QuantifiedVarDecl", e0);
  }

  function try_QuantifiedVarDecl()
  {
    shiftT(31);                     // '$'
    lookahead1W(255);               // EQName^Token | S^WS | '(:' | 'after' | 'allowing' | 'ancestor' |
    try_VarName();
    lookahead1W(110);               // S^WS | '(:' | 'as' | 'in'
    if (l1 == 79)                   // 'as'
    {
      try_TypeDeclaration();
    }
    lookahead1W(53);                // S^WS | '(:' | 'in'
    shiftT(154);                    // 'in'
    lookahead1W(267);               // Wildcard | EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral |
    try_ExprSingle();
  }

  function parse_SwitchExpr()
  {
    eventHandler.startNonterminal("SwitchExpr", e0);
    shift(243);                     // 'switch'
    lookahead1W(22);                // S^WS | '(' | '(:'
    shift(34);                      // '('
    lookahead1W(267);               // Wildcard | EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral |
    whitespace();
    parse_Expr();
    shift(37);                      // ')'
    for (;;)
    {
      lookahead1W(35);              // S^WS | '(:' | 'case'
      whitespace();
      parse_SwitchCaseClause();
      if (l1 != 88)                 // 'case'
      {
        break;
      }
    }
    shift(109);                     // 'default'
    lookahead1W(70);                // S^WS | '(:' | 'return'
    shift(220);                     // 'return'
    lookahead1W(267);               // Wildcard | EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral |
    whitespace();
    parse_ExprSingle();
    eventHandler.endNonterminal("SwitchExpr", e0);
  }

  function try_SwitchExpr()
  {
    shiftT(243);                    // 'switch'
    lookahead1W(22);                // S^WS | '(' | '(:'
    shiftT(34);                     // '('
    lookahead1W(267);               // Wildcard | EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral |
    try_Expr();
    shiftT(37);                     // ')'
    for (;;)
    {
      lookahead1W(35);              // S^WS | '(:' | 'case'
      try_SwitchCaseClause();
      if (l1 != 88)                 // 'case'
      {
        break;
      }
    }
    shiftT(109);                    // 'default'
    lookahead1W(70);                // S^WS | '(:' | 'return'
    shiftT(220);                    // 'return'
    lookahead1W(267);               // Wildcard | EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral |
    try_ExprSingle();
  }

  function parse_SwitchCaseClause()
  {
    eventHandler.startNonterminal("SwitchCaseClause", e0);
    for (;;)
    {
      shift(88);                    // 'case'
      lookahead1W(267);             // Wildcard | EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral |
      whitespace();
      parse_SwitchCaseOperand();
      if (l1 != 88)                 // 'case'
      {
        break;
      }
    }
    shift(220);                     // 'return'
    lookahead1W(267);               // Wildcard | EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral |
    whitespace();
    parse_ExprSingle();
    eventHandler.endNonterminal("SwitchCaseClause", e0);
  }

  function try_SwitchCaseClause()
  {
    for (;;)
    {
      shiftT(88);                   // 'case'
      lookahead1W(267);             // Wildcard | EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral |
      try_SwitchCaseOperand();
      if (l1 != 88)                 // 'case'
      {
        break;
      }
    }
    shiftT(220);                    // 'return'
    lookahead1W(267);               // Wildcard | EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral |
    try_ExprSingle();
  }

  function parse_SwitchCaseOperand()
  {
    eventHandler.startNonterminal("SwitchCaseOperand", e0);
    parse_ExprSingle();
    eventHandler.endNonterminal("SwitchCaseOperand", e0);
  }

  function try_SwitchCaseOperand()
  {
    try_ExprSingle();
  }

  function parse_TypeswitchExpr()
  {
    eventHandler.startNonterminal("TypeswitchExpr", e0);
    shift(253);                     // 'typeswitch'
    lookahead1W(22);                // S^WS | '(' | '(:'
    shift(34);                      // '('
    lookahead1W(267);               // Wildcard | EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral |
    whitespace();
    parse_Expr();
    shift(37);                      // ')'
    for (;;)
    {
      lookahead1W(35);              // S^WS | '(:' | 'case'
      whitespace();
      parse_CaseClause();
      if (l1 != 88)                 // 'case'
      {
        break;
      }
    }
    shift(109);                     // 'default'
    lookahead1W(95);                // S^WS | '$' | '(:' | 'return'
    if (l1 == 31)                   // '$'
    {
      shift(31);                    // '$'
      lookahead1W(255);             // EQName^Token | S^WS | '(:' | 'after' | 'allowing' | 'ancestor' |
      whitespace();
      parse_VarName();
    }
    lookahead1W(70);                // S^WS | '(:' | 'return'
    shift(220);                     // 'return'
    lookahead1W(267);               // Wildcard | EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral |
    whitespace();
    parse_ExprSingle();
    eventHandler.endNonterminal("TypeswitchExpr", e0);
  }

  function try_TypeswitchExpr()
  {
    shiftT(253);                    // 'typeswitch'
    lookahead1W(22);                // S^WS | '(' | '(:'
    shiftT(34);                     // '('
    lookahead1W(267);               // Wildcard | EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral |
    try_Expr();
    shiftT(37);                     // ')'
    for (;;)
    {
      lookahead1W(35);              // S^WS | '(:' | 'case'
      try_CaseClause();
      if (l1 != 88)                 // 'case'
      {
        break;
      }
    }
    shiftT(109);                    // 'default'
    lookahead1W(95);                // S^WS | '$' | '(:' | 'return'
    if (l1 == 31)                   // '$'
    {
      shiftT(31);                   // '$'
      lookahead1W(255);             // EQName^Token | S^WS | '(:' | 'after' | 'allowing' | 'ancestor' |
      try_VarName();
    }
    lookahead1W(70);                // S^WS | '(:' | 'return'
    shiftT(220);                    // 'return'
    lookahead1W(267);               // Wildcard | EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral |
    try_ExprSingle();
  }

  function parse_CaseClause()
  {
    eventHandler.startNonterminal("CaseClause", e0);
    shift(88);                      // 'case'
    lookahead1W(262);               // EQName^Token | S^WS | '$' | '%' | '(' | '(:' | 'after' | 'allowing' |
    if (l1 == 31)                   // '$'
    {
      shift(31);                    // '$'
      lookahead1W(255);             // EQName^Token | S^WS | '(:' | 'after' | 'allowing' | 'ancestor' |
      whitespace();
      parse_VarName();
      lookahead1W(30);              // S^WS | '(:' | 'as'
      shift(79);                    // 'as'
    }
    lookahead1W(260);               // EQName^Token | S^WS | '%' | '(' | '(:' | 'after' | 'allowing' | 'ancestor' |
    whitespace();
    parse_SequenceTypeUnion();
    shift(220);                     // 'return'
    lookahead1W(267);               // Wildcard | EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral |
    whitespace();
    parse_ExprSingle();
    eventHandler.endNonterminal("CaseClause", e0);
  }

  function try_CaseClause()
  {
    shiftT(88);                     // 'case'
    lookahead1W(262);               // EQName^Token | S^WS | '$' | '%' | '(' | '(:' | 'after' | 'allowing' |
    if (l1 == 31)                   // '$'
    {
      shiftT(31);                   // '$'
      lookahead1W(255);             // EQName^Token | S^WS | '(:' | 'after' | 'allowing' | 'ancestor' |
      try_VarName();
      lookahead1W(30);              // S^WS | '(:' | 'as'
      shiftT(79);                   // 'as'
    }
    lookahead1W(260);               // EQName^Token | S^WS | '%' | '(' | '(:' | 'after' | 'allowing' | 'ancestor' |
    try_SequenceTypeUnion();
    shiftT(220);                    // 'return'
    lookahead1W(267);               // Wildcard | EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral |
    try_ExprSingle();
  }

  function parse_SequenceTypeUnion()
  {
    eventHandler.startNonterminal("SequenceTypeUnion", e0);
    parse_SequenceType();
    for (;;)
    {
      lookahead1W(134);             // S^WS | '(:' | 'return' | '|'
      if (l1 != 279)                // '|'
      {
        break;
      }
      shift(279);                   // '|'
      lookahead1W(260);             // EQName^Token | S^WS | '%' | '(' | '(:' | 'after' | 'allowing' | 'ancestor' |
      whitespace();
      parse_SequenceType();
    }
    eventHandler.endNonterminal("SequenceTypeUnion", e0);
  }

  function try_SequenceTypeUnion()
  {
    try_SequenceType();
    for (;;)
    {
      lookahead1W(134);             // S^WS | '(:' | 'return' | '|'
      if (l1 != 279)                // '|'
      {
        break;
      }
      shiftT(279);                  // '|'
      lookahead1W(260);             // EQName^Token | S^WS | '%' | '(' | '(:' | 'after' | 'allowing' | 'ancestor' |
      try_SequenceType();
    }
  }

  function parse_IfExpr()
  {
    eventHandler.startNonterminal("IfExpr", e0);
    shift(152);                     // 'if'
    lookahead1W(22);                // S^WS | '(' | '(:'
    shift(34);                      // '('
    lookahead1W(267);               // Wildcard | EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral |
    whitespace();
    parse_Expr();
    shift(37);                      // ')'
    lookahead1W(77);                // S^WS | '(:' | 'then'
    shift(245);                     // 'then'
    lookahead1W(267);               // Wildcard | EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral |
    whitespace();
    parse_ExprSingle();
    shift(122);                     // 'else'
    lookahead1W(267);               // Wildcard | EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral |
    whitespace();
    parse_ExprSingle();
    eventHandler.endNonterminal("IfExpr", e0);
  }

  function try_IfExpr()
  {
    shiftT(152);                    // 'if'
    lookahead1W(22);                // S^WS | '(' | '(:'
    shiftT(34);                     // '('
    lookahead1W(267);               // Wildcard | EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral |
    try_Expr();
    shiftT(37);                     // ')'
    lookahead1W(77);                // S^WS | '(:' | 'then'
    shiftT(245);                    // 'then'
    lookahead1W(267);               // Wildcard | EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral |
    try_ExprSingle();
    shiftT(122);                    // 'else'
    lookahead1W(267);               // Wildcard | EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral |
    try_ExprSingle();
  }

  function parse_TryCatchExpr()
  {
    eventHandler.startNonterminal("TryCatchExpr", e0);
    parse_TryClause();
    for (;;)
    {
      lookahead1W(36);              // S^WS | '(:' | 'catch'
      whitespace();
      parse_CatchClause();
      lookahead1W(184);             // S^WS | EOF | '(:' | ')' | ',' | ':' | ';' | ']' | 'after' | 'as' | 'ascending' |
      if (l1 != 91)                 // 'catch'
      {
        break;
      }
    }
    eventHandler.endNonterminal("TryCatchExpr", e0);
  }

  function try_TryCatchExpr()
  {
    try_TryClause();
    for (;;)
    {
      lookahead1W(36);              // S^WS | '(:' | 'catch'
      try_CatchClause();
      lookahead1W(184);             // S^WS | EOF | '(:' | ')' | ',' | ':' | ';' | ']' | 'after' | 'as' | 'ascending' |
      if (l1 != 91)                 // 'catch'
      {
        break;
      }
    }
  }

  function parse_TryClause()
  {
    eventHandler.startNonterminal("TryClause", e0);
    shift(250);                     // 'try'
    lookahead1W(87);                // S^WS | '(:' | '{'
    shift(276);                     // '{'
    lookahead1W(267);               // Wildcard | EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral |
    whitespace();
    parse_TryTargetExpr();
    shift(282);                     // '}'
    eventHandler.endNonterminal("TryClause", e0);
  }

  function try_TryClause()
  {
    shiftT(250);                    // 'try'
    lookahead1W(87);                // S^WS | '(:' | '{'
    shiftT(276);                    // '{'
    lookahead1W(267);               // Wildcard | EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral |
    try_TryTargetExpr();
    shiftT(282);                    // '}'
  }

  function parse_TryTargetExpr()
  {
    eventHandler.startNonterminal("TryTargetExpr", e0);
    parse_Expr();
    eventHandler.endNonterminal("TryTargetExpr", e0);
  }

  function try_TryTargetExpr()
  {
    try_Expr();
  }

  function parse_CatchClause()
  {
    eventHandler.startNonterminal("CatchClause", e0);
    shift(91);                      // 'catch'
    lookahead1W(257);               // Wildcard | EQName^Token | S^WS | '(:' | 'after' | 'allowing' | 'ancestor' |
    whitespace();
    parse_CatchErrorList();
    shift(276);                     // '{'
    lookahead1W(267);               // Wildcard | EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral |
    whitespace();
    parse_Expr();
    shift(282);                     // '}'
    eventHandler.endNonterminal("CatchClause", e0);
  }

  function try_CatchClause()
  {
    shiftT(91);                     // 'catch'
    lookahead1W(257);               // Wildcard | EQName^Token | S^WS | '(:' | 'after' | 'allowing' | 'ancestor' |
    try_CatchErrorList();
    shiftT(276);                    // '{'
    lookahead1W(267);               // Wildcard | EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral |
    try_Expr();
    shiftT(282);                    // '}'
  }

  function parse_CatchErrorList()
  {
    eventHandler.startNonterminal("CatchErrorList", e0);
    parse_NameTest();
    for (;;)
    {
      lookahead1W(136);             // S^WS | '(:' | '{' | '|'
      if (l1 != 279)                // '|'
      {
        break;
      }
      shift(279);                   // '|'
      lookahead1W(257);             // Wildcard | EQName^Token | S^WS | '(:' | 'after' | 'allowing' | 'ancestor' |
      whitespace();
      parse_NameTest();
    }
    eventHandler.endNonterminal("CatchErrorList", e0);
  }

  function try_CatchErrorList()
  {
    try_NameTest();
    for (;;)
    {
      lookahead1W(136);             // S^WS | '(:' | '{' | '|'
      if (l1 != 279)                // '|'
      {
        break;
      }
      shiftT(279);                  // '|'
      lookahead1W(257);             // Wildcard | EQName^Token | S^WS | '(:' | 'after' | 'allowing' | 'ancestor' |
      try_NameTest();
    }
  }

  function parse_OrExpr()
  {
    eventHandler.startNonterminal("OrExpr", e0);
    parse_AndExpr();
    for (;;)
    {
      if (l1 != 200)                // 'or'
      {
        break;
      }
      shift(200);                   // 'or'
      lookahead1W(267);             // Wildcard | EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral |
      whitespace();
      parse_AndExpr();
    }
    eventHandler.endNonterminal("OrExpr", e0);
  }

  function try_OrExpr()
  {
    try_AndExpr();
    for (;;)
    {
      if (l1 != 200)                // 'or'
      {
        break;
      }
      shiftT(200);                  // 'or'
      lookahead1W(267);             // Wildcard | EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral |
      try_AndExpr();
    }
  }

  function parse_AndExpr()
  {
    eventHandler.startNonterminal("AndExpr", e0);
    parse_ComparisonExpr();
    for (;;)
    {
      if (l1 != 75)                 // 'and'
      {
        break;
      }
      shift(75);                    // 'and'
      lookahead1W(267);             // Wildcard | EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral |
      whitespace();
      parse_ComparisonExpr();
    }
    eventHandler.endNonterminal("AndExpr", e0);
  }

  function try_AndExpr()
  {
    try_ComparisonExpr();
    for (;;)
    {
      if (l1 != 75)                 // 'and'
      {
        break;
      }
      shiftT(75);                   // 'and'
      lookahead1W(267);             // Wildcard | EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral |
      try_ComparisonExpr();
    }
  }

  function parse_ComparisonExpr()
  {
    eventHandler.startNonterminal("ComparisonExpr", e0);
    parse_FTContainsExpr();
    if (l1 == 27                    // '!='
     || l1 == 54                    // '<'
     || l1 == 57                    // '<<'
     || l1 == 58                    // '<='
     || l1 == 60                    // '='
     || l1 == 61                    // '>'
     || l1 == 62                    // '>='
     || l1 == 63                    // '>>'
     || l1 == 128                   // 'eq'
     || l1 == 146                   // 'ge'
     || l1 == 150                   // 'gt'
     || l1 == 164                   // 'is'
     || l1 == 172                   // 'le'
     || l1 == 178                   // 'lt'
     || l1 == 186)                  // 'ne'
    {
      switch (l1)
      {
      case 128:                     // 'eq'
      case 146:                     // 'ge'
      case 150:                     // 'gt'
      case 172:                     // 'le'
      case 178:                     // 'lt'
      case 186:                     // 'ne'
        whitespace();
        parse_ValueComp();
        break;
      case 57:                      // '<<'
      case 63:                      // '>>'
      case 164:                     // 'is'
        whitespace();
        parse_NodeComp();
        break;
      default:
        whitespace();
        parse_GeneralComp();
      }
      lookahead1W(267);             // Wildcard | EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral |
      whitespace();
      parse_FTContainsExpr();
    }
    eventHandler.endNonterminal("ComparisonExpr", e0);
  }

  function try_ComparisonExpr()
  {
    try_FTContainsExpr();
    if (l1 == 27                    // '!='
     || l1 == 54                    // '<'
     || l1 == 57                    // '<<'
     || l1 == 58                    // '<='
     || l1 == 60                    // '='
     || l1 == 61                    // '>'
     || l1 == 62                    // '>='
     || l1 == 63                    // '>>'
     || l1 == 128                   // 'eq'
     || l1 == 146                   // 'ge'
     || l1 == 150                   // 'gt'
     || l1 == 164                   // 'is'
     || l1 == 172                   // 'le'
     || l1 == 178                   // 'lt'
     || l1 == 186)                  // 'ne'
    {
      switch (l1)
      {
      case 128:                     // 'eq'
      case 146:                     // 'ge'
      case 150:                     // 'gt'
      case 172:                     // 'le'
      case 178:                     // 'lt'
      case 186:                     // 'ne'
        try_ValueComp();
        break;
      case 57:                      // '<<'
      case 63:                      // '>>'
      case 164:                     // 'is'
        try_NodeComp();
        break;
      default:
        try_GeneralComp();
      }
      lookahead1W(267);             // Wildcard | EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral |
      try_FTContainsExpr();
    }
  }

  function parse_FTContainsExpr()
  {
    eventHandler.startNonterminal("FTContainsExpr", e0);
    parse_StringConcatExpr();
    if (l1 == 99)                   // 'contains'
    {
      shift(99);                    // 'contains'
      lookahead1W(76);              // S^WS | '(:' | 'text'
      shift(244);                   // 'text'
      lookahead1W(162);             // StringLiteral | S^WS | '(' | '(#' | '(:' | 'ftnot' | '{'
      whitespace();
      parse_FTSelection();
      if (l1 == 271)                // 'without'
      {
        whitespace();
        parse_FTIgnoreOption();
      }
    }
    eventHandler.endNonterminal("FTContainsExpr", e0);
  }

  function try_FTContainsExpr()
  {
    try_StringConcatExpr();
    if (l1 == 99)                   // 'contains'
    {
      shiftT(99);                   // 'contains'
      lookahead1W(76);              // S^WS | '(:' | 'text'
      shiftT(244);                  // 'text'
      lookahead1W(162);             // StringLiteral | S^WS | '(' | '(#' | '(:' | 'ftnot' | '{'
      try_FTSelection();
      if (l1 == 271)                // 'without'
      {
        try_FTIgnoreOption();
      }
    }
  }

  function parse_StringConcatExpr()
  {
    eventHandler.startNonterminal("StringConcatExpr", e0);
    parse_RangeExpr();
    for (;;)
    {
      if (l1 != 280)                // '||'
      {
        break;
      }
      shift(280);                   // '||'
      lookahead1W(267);             // Wildcard | EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral |
      whitespace();
      parse_RangeExpr();
    }
    eventHandler.endNonterminal("StringConcatExpr", e0);
  }

  function try_StringConcatExpr()
  {
    try_RangeExpr();
    for (;;)
    {
      if (l1 != 280)                // '||'
      {
        break;
      }
      shiftT(280);                  // '||'
      lookahead1W(267);             // Wildcard | EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral |
      try_RangeExpr();
    }
  }

  function parse_RangeExpr()
  {
    eventHandler.startNonterminal("RangeExpr", e0);
    parse_AdditiveExpr();
    if (l1 == 248)                  // 'to'
    {
      shift(248);                   // 'to'
      lookahead1W(267);             // Wildcard | EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral |
      whitespace();
      parse_AdditiveExpr();
    }
    eventHandler.endNonterminal("RangeExpr", e0);
  }

  function try_RangeExpr()
  {
    try_AdditiveExpr();
    if (l1 == 248)                  // 'to'
    {
      shiftT(248);                  // 'to'
      lookahead1W(267);             // Wildcard | EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral |
      try_AdditiveExpr();
    }
  }

  function parse_AdditiveExpr()
  {
    eventHandler.startNonterminal("AdditiveExpr", e0);
    parse_MultiplicativeExpr();
    for (;;)
    {
      if (l1 != 40                  // '+'
       && l1 != 42)                 // '-'
      {
        break;
      }
      switch (l1)
      {
      case 40:                      // '+'
        shift(40);                  // '+'
        break;
      default:
        shift(42);                  // '-'
      }
      lookahead1W(267);             // Wildcard | EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral |
      whitespace();
      parse_MultiplicativeExpr();
    }
    eventHandler.endNonterminal("AdditiveExpr", e0);
  }

  function try_AdditiveExpr()
  {
    try_MultiplicativeExpr();
    for (;;)
    {
      if (l1 != 40                  // '+'
       && l1 != 42)                 // '-'
      {
        break;
      }
      switch (l1)
      {
      case 40:                      // '+'
        shiftT(40);                 // '+'
        break;
      default:
        shiftT(42);                 // '-'
      }
      lookahead1W(267);             // Wildcard | EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral |
      try_MultiplicativeExpr();
    }
  }

  function parse_MultiplicativeExpr()
  {
    eventHandler.startNonterminal("MultiplicativeExpr", e0);
    parse_UnionExpr();
    for (;;)
    {
      if (l1 != 38                  // '*'
       && l1 != 118                 // 'div'
       && l1 != 151                 // 'idiv'
       && l1 != 180)                // 'mod'
      {
        break;
      }
      switch (l1)
      {
      case 38:                      // '*'
        shift(38);                  // '*'
        break;
      case 118:                     // 'div'
        shift(118);                 // 'div'
        break;
      case 151:                     // 'idiv'
        shift(151);                 // 'idiv'
        break;
      default:
        shift(180);                 // 'mod'
      }
      lookahead1W(267);             // Wildcard | EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral |
      whitespace();
      parse_UnionExpr();
    }
    eventHandler.endNonterminal("MultiplicativeExpr", e0);
  }

  function try_MultiplicativeExpr()
  {
    try_UnionExpr();
    for (;;)
    {
      if (l1 != 38                  // '*'
       && l1 != 118                 // 'div'
       && l1 != 151                 // 'idiv'
       && l1 != 180)                // 'mod'
      {
        break;
      }
      switch (l1)
      {
      case 38:                      // '*'
        shiftT(38);                 // '*'
        break;
      case 118:                     // 'div'
        shiftT(118);                // 'div'
        break;
      case 151:                     // 'idiv'
        shiftT(151);                // 'idiv'
        break;
      default:
        shiftT(180);                // 'mod'
      }
      lookahead1W(267);             // Wildcard | EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral |
      try_UnionExpr();
    }
  }

  function parse_UnionExpr()
  {
    eventHandler.startNonterminal("UnionExpr", e0);
    parse_IntersectExceptExpr();
    for (;;)
    {
      if (l1 != 254                 // 'union'
       && l1 != 279)                // '|'
      {
        break;
      }
      switch (l1)
      {
      case 254:                     // 'union'
        shift(254);                 // 'union'
        break;
      default:
        shift(279);                 // '|'
      }
      lookahead1W(267);             // Wildcard | EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral |
      whitespace();
      parse_IntersectExceptExpr();
    }
    eventHandler.endNonterminal("UnionExpr", e0);
  }

  function try_UnionExpr()
  {
    try_IntersectExceptExpr();
    for (;;)
    {
      if (l1 != 254                 // 'union'
       && l1 != 279)                // '|'
      {
        break;
      }
      switch (l1)
      {
      case 254:                     // 'union'
        shiftT(254);                // 'union'
        break;
      default:
        shiftT(279);                // '|'
      }
      lookahead1W(267);             // Wildcard | EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral |
      try_IntersectExceptExpr();
    }
  }

  function parse_IntersectExceptExpr()
  {
    eventHandler.startNonterminal("IntersectExceptExpr", e0);
    parse_InstanceofExpr();
    for (;;)
    {
      lookahead1W(223);             // S^WS | EOF | '!=' | '(:' | ')' | '*' | '+' | ',' | '-' | ':' | ';' | '<' | '<<' |
      if (l1 != 131                 // 'except'
       && l1 != 162)                // 'intersect'
      {
        break;
      }
      switch (l1)
      {
      case 162:                     // 'intersect'
        shift(162);                 // 'intersect'
        break;
      default:
        shift(131);                 // 'except'
      }
      lookahead1W(267);             // Wildcard | EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral |
      whitespace();
      parse_InstanceofExpr();
    }
    eventHandler.endNonterminal("IntersectExceptExpr", e0);
  }

  function try_IntersectExceptExpr()
  {
    try_InstanceofExpr();
    for (;;)
    {
      lookahead1W(223);             // S^WS | EOF | '!=' | '(:' | ')' | '*' | '+' | ',' | '-' | ':' | ';' | '<' | '<<' |
      if (l1 != 131                 // 'except'
       && l1 != 162)                // 'intersect'
      {
        break;
      }
      switch (l1)
      {
      case 162:                     // 'intersect'
        shiftT(162);                // 'intersect'
        break;
      default:
        shiftT(131);                // 'except'
      }
      lookahead1W(267);             // Wildcard | EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral |
      try_InstanceofExpr();
    }
  }

  function parse_InstanceofExpr()
  {
    eventHandler.startNonterminal("InstanceofExpr", e0);
    parse_TreatExpr();
    lookahead1W(224);               // S^WS | EOF | '!=' | '(:' | ')' | '*' | '+' | ',' | '-' | ':' | ';' | '<' | '<<' |
    if (l1 == 160)                  // 'instance'
    {
      shift(160);                   // 'instance'
      lookahead1W(64);              // S^WS | '(:' | 'of'
      shift(196);                   // 'of'
      lookahead1W(260);             // EQName^Token | S^WS | '%' | '(' | '(:' | 'after' | 'allowing' | 'ancestor' |
      whitespace();
      parse_SequenceType();
    }
    eventHandler.endNonterminal("InstanceofExpr", e0);
  }

  function try_InstanceofExpr()
  {
    try_TreatExpr();
    lookahead1W(224);               // S^WS | EOF | '!=' | '(:' | ')' | '*' | '+' | ',' | '-' | ':' | ';' | '<' | '<<' |
    if (l1 == 160)                  // 'instance'
    {
      shiftT(160);                  // 'instance'
      lookahead1W(64);              // S^WS | '(:' | 'of'
      shiftT(196);                  // 'of'
      lookahead1W(260);             // EQName^Token | S^WS | '%' | '(' | '(:' | 'after' | 'allowing' | 'ancestor' |
      try_SequenceType();
    }
  }

  function parse_TreatExpr()
  {
    eventHandler.startNonterminal("TreatExpr", e0);
    parse_CastableExpr();
    lookahead1W(225);               // S^WS | EOF | '!=' | '(:' | ')' | '*' | '+' | ',' | '-' | ':' | ';' | '<' | '<<' |
    if (l1 == 249)                  // 'treat'
    {
      shift(249);                   // 'treat'
      lookahead1W(30);              // S^WS | '(:' | 'as'
      shift(79);                    // 'as'
      lookahead1W(260);             // EQName^Token | S^WS | '%' | '(' | '(:' | 'after' | 'allowing' | 'ancestor' |
      whitespace();
      parse_SequenceType();
    }
    eventHandler.endNonterminal("TreatExpr", e0);
  }

  function try_TreatExpr()
  {
    try_CastableExpr();
    lookahead1W(225);               // S^WS | EOF | '!=' | '(:' | ')' | '*' | '+' | ',' | '-' | ':' | ';' | '<' | '<<' |
    if (l1 == 249)                  // 'treat'
    {
      shiftT(249);                  // 'treat'
      lookahead1W(30);              // S^WS | '(:' | 'as'
      shiftT(79);                   // 'as'
      lookahead1W(260);             // EQName^Token | S^WS | '%' | '(' | '(:' | 'after' | 'allowing' | 'ancestor' |
      try_SequenceType();
    }
  }

  function parse_CastableExpr()
  {
    eventHandler.startNonterminal("CastableExpr", e0);
    parse_CastExpr();
    lookahead1W(226);               // S^WS | EOF | '!=' | '(:' | ')' | '*' | '+' | ',' | '-' | ':' | ';' | '<' | '<<' |
    if (l1 == 90)                   // 'castable'
    {
      shift(90);                    // 'castable'
      lookahead1W(30);              // S^WS | '(:' | 'as'
      shift(79);                    // 'as'
      lookahead1W(255);             // EQName^Token | S^WS | '(:' | 'after' | 'allowing' | 'ancestor' |
      whitespace();
      parse_SingleType();
    }
    eventHandler.endNonterminal("CastableExpr", e0);
  }

  function try_CastableExpr()
  {
    try_CastExpr();
    lookahead1W(226);               // S^WS | EOF | '!=' | '(:' | ')' | '*' | '+' | ',' | '-' | ':' | ';' | '<' | '<<' |
    if (l1 == 90)                   // 'castable'
    {
      shiftT(90);                   // 'castable'
      lookahead1W(30);              // S^WS | '(:' | 'as'
      shiftT(79);                   // 'as'
      lookahead1W(255);             // EQName^Token | S^WS | '(:' | 'after' | 'allowing' | 'ancestor' |
      try_SingleType();
    }
  }

  function parse_CastExpr()
  {
    eventHandler.startNonterminal("CastExpr", e0);
    parse_UnaryExpr();
    lookahead1W(228);               // S^WS | EOF | '!=' | '(:' | ')' | '*' | '+' | ',' | '-' | ':' | ';' | '<' | '<<' |
    if (l1 == 89)                   // 'cast'
    {
      shift(89);                    // 'cast'
      lookahead1W(30);              // S^WS | '(:' | 'as'
      shift(79);                    // 'as'
      lookahead1W(255);             // EQName^Token | S^WS | '(:' | 'after' | 'allowing' | 'ancestor' |
      whitespace();
      parse_SingleType();
    }
    eventHandler.endNonterminal("CastExpr", e0);
  }

  function try_CastExpr()
  {
    try_UnaryExpr();
    lookahead1W(228);               // S^WS | EOF | '!=' | '(:' | ')' | '*' | '+' | ',' | '-' | ':' | ';' | '<' | '<<' |
    if (l1 == 89)                   // 'cast'
    {
      shiftT(89);                   // 'cast'
      lookahead1W(30);              // S^WS | '(:' | 'as'
      shiftT(79);                   // 'as'
      lookahead1W(255);             // EQName^Token | S^WS | '(:' | 'after' | 'allowing' | 'ancestor' |
      try_SingleType();
    }
  }

  function parse_UnaryExpr()
  {
    eventHandler.startNonterminal("UnaryExpr", e0);
    for (;;)
    {
      lookahead1W(267);             // Wildcard | EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral |
      if (l1 != 40                  // '+'
       && l1 != 42)                 // '-'
      {
        break;
      }
      switch (l1)
      {
      case 42:                      // '-'
        shift(42);                  // '-'
        break;
      default:
        shift(40);                  // '+'
      }
    }
    whitespace();
    parse_ValueExpr();
    eventHandler.endNonterminal("UnaryExpr", e0);
  }

  function try_UnaryExpr()
  {
    for (;;)
    {
      lookahead1W(267);             // Wildcard | EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral |
      if (l1 != 40                  // '+'
       && l1 != 42)                 // '-'
      {
        break;
      }
      switch (l1)
      {
      case 42:                      // '-'
        shiftT(42);                 // '-'
        break;
      default:
        shiftT(40);                 // '+'
      }
    }
    try_ValueExpr();
  }

  function parse_ValueExpr()
  {
    eventHandler.startNonterminal("ValueExpr", e0);
    switch (l1)
    {
    case 260:                       // 'validate'
      lookahead2W(248);             // S^WS | EOF | '!' | '!=' | '#' | '(' | '(:' | ')' | '*' | '+' | ',' | '-' | '/' |
      break;
    default:
      lk = l1;
    }
    switch (lk)
    {
    case 87812:                     // 'validate' 'lax'
    case 123140:                    // 'validate' 'strict'
    case 129284:                    // 'validate' 'type'
    case 141572:                    // 'validate' '{'
      parse_ValidateExpr();
      break;
    case 35:                        // '(#'
      parse_ExtensionExpr();
      break;
    default:
      parse_SimpleMapExpr();
    }
    eventHandler.endNonterminal("ValueExpr", e0);
  }

  function try_ValueExpr()
  {
    switch (l1)
    {
    case 260:                       // 'validate'
      lookahead2W(248);             // S^WS | EOF | '!' | '!=' | '#' | '(' | '(:' | ')' | '*' | '+' | ',' | '-' | '/' |
      break;
    default:
      lk = l1;
    }
    switch (lk)
    {
    case 87812:                     // 'validate' 'lax'
    case 123140:                    // 'validate' 'strict'
    case 129284:                    // 'validate' 'type'
    case 141572:                    // 'validate' '{'
      try_ValidateExpr();
      break;
    case 35:                        // '(#'
      try_ExtensionExpr();
      break;
    default:
      try_SimpleMapExpr();
    }
  }

  function parse_SimpleMapExpr()
  {
    eventHandler.startNonterminal("SimpleMapExpr", e0);
    parse_PathExpr();
    for (;;)
    {
      if (l1 != 26)                 // '!'
      {
        break;
      }
      shift(26);                    // '!'
      lookahead1W(266);             // Wildcard | EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral |
      whitespace();
      parse_PathExpr();
    }
    eventHandler.endNonterminal("SimpleMapExpr", e0);
  }

  function try_SimpleMapExpr()
  {
    try_PathExpr();
    for (;;)
    {
      if (l1 != 26)                 // '!'
      {
        break;
      }
      shiftT(26);                   // '!'
      lookahead1W(266);             // Wildcard | EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral |
      try_PathExpr();
    }
  }

  function parse_GeneralComp()
  {
    eventHandler.startNonterminal("GeneralComp", e0);
    switch (l1)
    {
    case 60:                        // '='
      shift(60);                    // '='
      break;
    case 27:                        // '!='
      shift(27);                    // '!='
      break;
    case 54:                        // '<'
      shift(54);                    // '<'
      break;
    case 58:                        // '<='
      shift(58);                    // '<='
      break;
    case 61:                        // '>'
      shift(61);                    // '>'
      break;
    default:
      shift(62);                    // '>='
    }
    eventHandler.endNonterminal("GeneralComp", e0);
  }

  function try_GeneralComp()
  {
    switch (l1)
    {
    case 60:                        // '='
      shiftT(60);                   // '='
      break;
    case 27:                        // '!='
      shiftT(27);                   // '!='
      break;
    case 54:                        // '<'
      shiftT(54);                   // '<'
      break;
    case 58:                        // '<='
      shiftT(58);                   // '<='
      break;
    case 61:                        // '>'
      shiftT(61);                   // '>'
      break;
    default:
      shiftT(62);                   // '>='
    }
  }

  function parse_ValueComp()
  {
    eventHandler.startNonterminal("ValueComp", e0);
    switch (l1)
    {
    case 128:                       // 'eq'
      shift(128);                   // 'eq'
      break;
    case 186:                       // 'ne'
      shift(186);                   // 'ne'
      break;
    case 178:                       // 'lt'
      shift(178);                   // 'lt'
      break;
    case 172:                       // 'le'
      shift(172);                   // 'le'
      break;
    case 150:                       // 'gt'
      shift(150);                   // 'gt'
      break;
    default:
      shift(146);                   // 'ge'
    }
    eventHandler.endNonterminal("ValueComp", e0);
  }

  function try_ValueComp()
  {
    switch (l1)
    {
    case 128:                       // 'eq'
      shiftT(128);                  // 'eq'
      break;
    case 186:                       // 'ne'
      shiftT(186);                  // 'ne'
      break;
    case 178:                       // 'lt'
      shiftT(178);                  // 'lt'
      break;
    case 172:                       // 'le'
      shiftT(172);                  // 'le'
      break;
    case 150:                       // 'gt'
      shiftT(150);                  // 'gt'
      break;
    default:
      shiftT(146);                  // 'ge'
    }
  }

  function parse_NodeComp()
  {
    eventHandler.startNonterminal("NodeComp", e0);
    switch (l1)
    {
    case 164:                       // 'is'
      shift(164);                   // 'is'
      break;
    case 57:                        // '<<'
      shift(57);                    // '<<'
      break;
    default:
      shift(63);                    // '>>'
    }
    eventHandler.endNonterminal("NodeComp", e0);
  }

  function try_NodeComp()
  {
    switch (l1)
    {
    case 164:                       // 'is'
      shiftT(164);                  // 'is'
      break;
    case 57:                        // '<<'
      shiftT(57);                   // '<<'
      break;
    default:
      shiftT(63);                   // '>>'
    }
  }

  function parse_ValidateExpr()
  {
    eventHandler.startNonterminal("ValidateExpr", e0);
    shift(260);                     // 'validate'
    lookahead1W(160);               // S^WS | '(:' | 'lax' | 'strict' | 'type' | '{'
    if (l1 != 276)                  // '{'
    {
      switch (l1)
      {
      case 252:                     // 'type'
        shift(252);                 // 'type'
        lookahead1W(255);           // EQName^Token | S^WS | '(:' | 'after' | 'allowing' | 'ancestor' |
        whitespace();
        parse_TypeName();
        break;
      default:
        whitespace();
        parse_ValidationMode();
      }
    }
    lookahead1W(87);                // S^WS | '(:' | '{'
    shift(276);                     // '{'
    lookahead1W(267);               // Wildcard | EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral |
    whitespace();
    parse_Expr();
    shift(282);                     // '}'
    eventHandler.endNonterminal("ValidateExpr", e0);
  }

  function try_ValidateExpr()
  {
    shiftT(260);                    // 'validate'
    lookahead1W(160);               // S^WS | '(:' | 'lax' | 'strict' | 'type' | '{'
    if (l1 != 276)                  // '{'
    {
      switch (l1)
      {
      case 252:                     // 'type'
        shiftT(252);                // 'type'
        lookahead1W(255);           // EQName^Token | S^WS | '(:' | 'after' | 'allowing' | 'ancestor' |
        try_TypeName();
        break;
      default:
        try_ValidationMode();
      }
    }
    lookahead1W(87);                // S^WS | '(:' | '{'
    shiftT(276);                    // '{'
    lookahead1W(267);               // Wildcard | EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral |
    try_Expr();
    shiftT(282);                    // '}'
  }

  function parse_ValidationMode()
  {
    eventHandler.startNonterminal("ValidationMode", e0);
    switch (l1)
    {
    case 171:                       // 'lax'
      shift(171);                   // 'lax'
      break;
    default:
      shift(240);                   // 'strict'
    }
    eventHandler.endNonterminal("ValidationMode", e0);
  }

  function try_ValidationMode()
  {
    switch (l1)
    {
    case 171:                       // 'lax'
      shiftT(171);                  // 'lax'
      break;
    default:
      shiftT(240);                  // 'strict'
    }
  }

  function parse_ExtensionExpr()
  {
    eventHandler.startNonterminal("ExtensionExpr", e0);
    for (;;)
    {
      whitespace();
      parse_Pragma();
      lookahead1W(100);             // S^WS | '(#' | '(:' | '{'
      if (l1 != 35)                 // '(#'
      {
        break;
      }
    }
    shift(276);                     // '{'
    lookahead1W(274);               // Wildcard | EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral |
    if (l1 != 282)                  // '}'
    {
      whitespace();
      parse_Expr();
    }
    shift(282);                     // '}'
    eventHandler.endNonterminal("ExtensionExpr", e0);
  }

  function try_ExtensionExpr()
  {
    for (;;)
    {
      try_Pragma();
      lookahead1W(100);             // S^WS | '(#' | '(:' | '{'
      if (l1 != 35)                 // '(#'
      {
        break;
      }
    }
    shiftT(276);                    // '{'
    lookahead1W(274);               // Wildcard | EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral |
    if (l1 != 282)                  // '}'
    {
      try_Expr();
    }
    shiftT(282);                    // '}'
  }

  function parse_Pragma()
  {
    eventHandler.startNonterminal("Pragma", e0);
    shift(35);                      // '(#'
    lookahead1(252);                // EQName^Token | S | 'after' | 'allowing' | 'ancestor' | 'ancestor-or-self' |
    if (l1 == 21)                   // S
    {
      shift(21);                    // S
    }
    parse_EQName();
    lookahead1(10);                 // S | '#)'
    if (l1 == 21)                   // S
    {
      shift(21);                    // S
      lookahead1(0);                // PragmaContents
      shift(1);                     // PragmaContents
    }
    lookahead1(5);                  // '#)'
    shift(30);                      // '#)'
    eventHandler.endNonterminal("Pragma", e0);
  }

  function try_Pragma()
  {
    shiftT(35);                     // '(#'
    lookahead1(252);                // EQName^Token | S | 'after' | 'allowing' | 'ancestor' | 'ancestor-or-self' |
    if (l1 == 21)                   // S
    {
      shiftT(21);                   // S
    }
    try_EQName();
    lookahead1(10);                 // S | '#)'
    if (l1 == 21)                   // S
    {
      shiftT(21);                   // S
      lookahead1(0);                // PragmaContents
      shiftT(1);                    // PragmaContents
    }
    lookahead1(5);                  // '#)'
    shiftT(30);                     // '#)'
  }

  function parse_PathExpr()
  {
    eventHandler.startNonterminal("PathExpr", e0);
    switch (l1)
    {
    case 46:                        // '/'
      shift(46);                    // '/'
      lookahead1W(286);             // Wildcard | EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral |
      switch (l1)
      {
      case 25:                      // EOF
      case 26:                      // '!'
      case 27:                      // '!='
      case 37:                      // ')'
      case 38:                      // '*'
      case 40:                      // '+'
      case 41:                      // ','
      case 42:                      // '-'
      case 49:                      // ':'
      case 53:                      // ';'
      case 57:                      // '<<'
      case 58:                      // '<='
      case 60:                      // '='
      case 61:                      // '>'
      case 62:                      // '>='
      case 63:                      // '>>'
      case 69:                      // ']'
      case 87:                      // 'by'
      case 99:                      // 'contains'
      case 205:                     // 'paragraphs'
      case 232:                     // 'sentences'
      case 247:                     // 'times'
      case 273:                     // 'words'
      case 279:                     // '|'
      case 280:                     // '||'
      case 281:                     // '|}'
      case 282:                     // '}'
        break;
      default:
        whitespace();
        parse_RelativePathExpr();
      }
      break;
    case 47:                        // '//'
      shift(47);                    // '//'
      lookahead1W(265);             // Wildcard | EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral |
      whitespace();
      parse_RelativePathExpr();
      break;
    default:
      parse_RelativePathExpr();
    }
    eventHandler.endNonterminal("PathExpr", e0);
  }

  function try_PathExpr()
  {
    switch (l1)
    {
    case 46:                        // '/'
      shiftT(46);                   // '/'
      lookahead1W(286);             // Wildcard | EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral |
      switch (l1)
      {
      case 25:                      // EOF
      case 26:                      // '!'
      case 27:                      // '!='
      case 37:                      // ')'
      case 38:                      // '*'
      case 40:                      // '+'
      case 41:                      // ','
      case 42:                      // '-'
      case 49:                      // ':'
      case 53:                      // ';'
      case 57:                      // '<<'
      case 58:                      // '<='
      case 60:                      // '='
      case 61:                      // '>'
      case 62:                      // '>='
      case 63:                      // '>>'
      case 69:                      // ']'
      case 87:                      // 'by'
      case 99:                      // 'contains'
      case 205:                     // 'paragraphs'
      case 232:                     // 'sentences'
      case 247:                     // 'times'
      case 273:                     // 'words'
      case 279:                     // '|'
      case 280:                     // '||'
      case 281:                     // '|}'
      case 282:                     // '}'
        break;
      default:
        try_RelativePathExpr();
      }
      break;
    case 47:                        // '//'
      shiftT(47);                   // '//'
      lookahead1W(265);             // Wildcard | EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral |
      try_RelativePathExpr();
      break;
    default:
      try_RelativePathExpr();
    }
  }

  function parse_RelativePathExpr()
  {
    eventHandler.startNonterminal("RelativePathExpr", e0);
    parse_StepExpr();
    for (;;)
    {
      switch (l1)
      {
      case 26:                      // '!'
        lookahead2W(266);           // Wildcard | EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral |
        break;
      default:
        lk = l1;
      }
      if (lk != 25                  // EOF
       && lk != 27                  // '!='
       && lk != 37                  // ')'
       && lk != 38                  // '*'
       && lk != 40                  // '+'
       && lk != 41                  // ','
       && lk != 42                  // '-'
       && lk != 46                  // '/'
       && lk != 47                  // '//'
       && lk != 49                  // ':'
       && lk != 53                  // ';'
       && lk != 54                  // '<'
       && lk != 57                  // '<<'
       && lk != 58                  // '<='
       && lk != 60                  // '='
       && lk != 61                  // '>'
       && lk != 62                  // '>='
       && lk != 63                  // '>>'
       && lk != 69                  // ']'
       && lk != 70                  // 'after'
       && lk != 75                  // 'and'
       && lk != 79                  // 'as'
       && lk != 80                  // 'ascending'
       && lk != 81                  // 'at'
       && lk != 84                  // 'before'
       && lk != 87                  // 'by'
       && lk != 88                  // 'case'
       && lk != 89                  // 'cast'
       && lk != 90                  // 'castable'
       && lk != 94                  // 'collation'
       && lk != 99                  // 'contains'
       && lk != 105                 // 'count'
       && lk != 109                 // 'default'
       && lk != 113                 // 'descending'
       && lk != 118                 // 'div'
       && lk != 122                 // 'else'
       && lk != 123                 // 'empty'
       && lk != 126                 // 'end'
       && lk != 128                 // 'eq'
       && lk != 131                 // 'except'
       && lk != 137                 // 'for'
       && lk != 146                 // 'ge'
       && lk != 148                 // 'group'
       && lk != 150                 // 'gt'
       && lk != 151                 // 'idiv'
       && lk != 160                 // 'instance'
       && lk != 162                 // 'intersect'
       && lk != 163                 // 'into'
       && lk != 164                 // 'is'
       && lk != 172                 // 'le'
       && lk != 174                 // 'let'
       && lk != 178                 // 'lt'
       && lk != 180                 // 'mod'
       && lk != 181                 // 'modify'
       && lk != 186                 // 'ne'
       && lk != 198                 // 'only'
       && lk != 200                 // 'or'
       && lk != 201                 // 'order'
       && lk != 205                 // 'paragraphs'
       && lk != 220                 // 'return'
       && lk != 224                 // 'satisfies'
       && lk != 232                 // 'sentences'
       && lk != 236                 // 'stable'
       && lk != 237                 // 'start'
       && lk != 247                 // 'times'
       && lk != 248                 // 'to'
       && lk != 249                 // 'treat'
       && lk != 254                 // 'union'
       && lk != 266                 // 'where'
       && lk != 270                 // 'with'
       && lk != 273                 // 'words'
       && lk != 279                 // '|'
       && lk != 280                 // '||'
       && lk != 281                 // '|}'
       && lk != 282                 // '}'
       && lk != 23578               // '!' '/'
       && lk != 24090)              // '!' '//'
      {
        lk = memoized(3, e0);
        if (lk == 0)
        {
          var b0A = b0; var e0A = e0; var l1A = l1;
          var b1A = b1; var e1A = e1; var l2A = l2;
          var b2A = b2; var e2A = e2;
          try
          {
            switch (l1)
            {
            case 46:                // '/'
              shiftT(46);           // '/'
              break;
            case 47:                // '//'
              shiftT(47);           // '//'
              break;
            default:
              shiftT(26);           // '!'
            }
            lookahead1W(265);       // Wildcard | EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral |
            try_StepExpr();
            lk = -1;
          }
          catch (p1A)
          {
            lk = -2;
          }
          b0 = b0A; e0 = e0A; l1 = l1A; if (l1 == 0) {end = e0A;} else {
          b1 = b1A; e1 = e1A; l2 = l2A; if (l2 == 0) {end = e1A;} else {
          b2 = b2A; e2 = e2A; end = e2A; }}
          memoize(3, e0, lk);
        }
      }
      if (lk != -1
       && lk != 46                  // '/'
       && lk != 47)                 // '//'
      {
        break;
      }
      switch (l1)
      {
      case 46:                      // '/'
        shift(46);                  // '/'
        break;
      case 47:                      // '//'
        shift(47);                  // '//'
        break;
      default:
        shift(26);                  // '!'
      }
      lookahead1W(265);             // Wildcard | EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral |
      whitespace();
      parse_StepExpr();
    }
    eventHandler.endNonterminal("RelativePathExpr", e0);
  }

  function try_RelativePathExpr()
  {
    try_StepExpr();
    for (;;)
    {
      switch (l1)
      {
      case 26:                      // '!'
        lookahead2W(266);           // Wildcard | EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral |
        break;
      default:
        lk = l1;
      }
      if (lk != 25                  // EOF
       && lk != 27                  // '!='
       && lk != 37                  // ')'
       && lk != 38                  // '*'
       && lk != 40                  // '+'
       && lk != 41                  // ','
       && lk != 42                  // '-'
       && lk != 46                  // '/'
       && lk != 47                  // '//'
       && lk != 49                  // ':'
       && lk != 53                  // ';'
       && lk != 54                  // '<'
       && lk != 57                  // '<<'
       && lk != 58                  // '<='
       && lk != 60                  // '='
       && lk != 61                  // '>'
       && lk != 62                  // '>='
       && lk != 63                  // '>>'
       && lk != 69                  // ']'
       && lk != 70                  // 'after'
       && lk != 75                  // 'and'
       && lk != 79                  // 'as'
       && lk != 80                  // 'ascending'
       && lk != 81                  // 'at'
       && lk != 84                  // 'before'
       && lk != 87                  // 'by'
       && lk != 88                  // 'case'
       && lk != 89                  // 'cast'
       && lk != 90                  // 'castable'
       && lk != 94                  // 'collation'
       && lk != 99                  // 'contains'
       && lk != 105                 // 'count'
       && lk != 109                 // 'default'
       && lk != 113                 // 'descending'
       && lk != 118                 // 'div'
       && lk != 122                 // 'else'
       && lk != 123                 // 'empty'
       && lk != 126                 // 'end'
       && lk != 128                 // 'eq'
       && lk != 131                 // 'except'
       && lk != 137                 // 'for'
       && lk != 146                 // 'ge'
       && lk != 148                 // 'group'
       && lk != 150                 // 'gt'
       && lk != 151                 // 'idiv'
       && lk != 160                 // 'instance'
       && lk != 162                 // 'intersect'
       && lk != 163                 // 'into'
       && lk != 164                 // 'is'
       && lk != 172                 // 'le'
       && lk != 174                 // 'let'
       && lk != 178                 // 'lt'
       && lk != 180                 // 'mod'
       && lk != 181                 // 'modify'
       && lk != 186                 // 'ne'
       && lk != 198                 // 'only'
       && lk != 200                 // 'or'
       && lk != 201                 // 'order'
       && lk != 205                 // 'paragraphs'
       && lk != 220                 // 'return'
       && lk != 224                 // 'satisfies'
       && lk != 232                 // 'sentences'
       && lk != 236                 // 'stable'
       && lk != 237                 // 'start'
       && lk != 247                 // 'times'
       && lk != 248                 // 'to'
       && lk != 249                 // 'treat'
       && lk != 254                 // 'union'
       && lk != 266                 // 'where'
       && lk != 270                 // 'with'
       && lk != 273                 // 'words'
       && lk != 279                 // '|'
       && lk != 280                 // '||'
       && lk != 281                 // '|}'
       && lk != 282                 // '}'
       && lk != 23578               // '!' '/'
       && lk != 24090)              // '!' '//'
      {
        lk = memoized(3, e0);
        if (lk == 0)
        {
          var b0A = b0; var e0A = e0; var l1A = l1;
          var b1A = b1; var e1A = e1; var l2A = l2;
          var b2A = b2; var e2A = e2;
          try
          {
            switch (l1)
            {
            case 46:                // '/'
              shiftT(46);           // '/'
              break;
            case 47:                // '//'
              shiftT(47);           // '//'
              break;
            default:
              shiftT(26);           // '!'
            }
            lookahead1W(265);       // Wildcard | EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral |
            try_StepExpr();
            memoize(3, e0A, -1);
            continue;
          }
          catch (p1A)
          {
            b0 = b0A; e0 = e0A; l1 = l1A; if (l1 == 0) {end = e0A;} else {
            b1 = b1A; e1 = e1A; l2 = l2A; if (l2 == 0) {end = e1A;} else {
            b2 = b2A; e2 = e2A; end = e2A; }}
            memoize(3, e0A, -2);
            break;
          }
        }
      }
      if (lk != -1
       && lk != 46                  // '/'
       && lk != 47)                 // '//'
      {
        break;
      }
      switch (l1)
      {
      case 46:                      // '/'
        shiftT(46);                 // '/'
        break;
      case 47:                      // '//'
        shiftT(47);                 // '//'
        break;
      default:
        shiftT(26);                 // '!'
      }
      lookahead1W(265);             // Wildcard | EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral |
      try_StepExpr();
    }
  }

  function parse_StepExpr()
  {
    eventHandler.startNonterminal("StepExpr", e0);
    switch (l1)
    {
    case 82:                        // 'attribute'
      lookahead2W(285);             // EQName^Token | S^WS | EOF | '!' | '!=' | '#' | '(' | '(:' | ')' | '*' | '+' |
      break;
    case 121:                       // 'element'
      lookahead2W(283);             // EQName^Token | S^WS | EOF | '!' | '!=' | '#' | '(' | '(:' | ')' | '*' | '+' |
      break;
    case 184:                       // 'namespace'
    case 216:                       // 'processing-instruction'
      lookahead2W(282);             // NCName^Token | S^WS | EOF | '!' | '!=' | '#' | '(' | '(:' | ')' | '*' | '+' |
      break;
    case 96:                        // 'comment'
    case 119:                       // 'document'
    case 202:                       // 'ordered'
    case 244:                       // 'text'
    case 256:                       // 'unordered'
      lookahead2W(247);             // S^WS | EOF | '!' | '!=' | '#' | '(' | '(:' | ')' | '*' | '+' | ',' | '-' | '/' |
      break;
    case 78:                        // 'array'
    case 124:                       // 'empty-sequence'
    case 152:                       // 'if'
    case 165:                       // 'item'
    case 167:                       // 'json-item'
    case 242:                       // 'structured-item'
    case 243:                       // 'switch'
    case 253:                       // 'typeswitch'
      lookahead2W(240);             // S^WS | EOF | '!' | '!=' | '#' | '(:' | ')' | '*' | '+' | ',' | '-' | '/' | '//' |
      break;
    case 73:                        // 'ancestor'
    case 74:                        // 'ancestor-or-self'
    case 93:                        // 'child'
    case 111:                       // 'descendant'
    case 112:                       // 'descendant-or-self'
    case 135:                       // 'following'
    case 136:                       // 'following-sibling'
    case 206:                       // 'parent'
    case 212:                       // 'preceding'
    case 213:                       // 'preceding-sibling'
    case 229:                       // 'self'
      lookahead2W(246);             // S^WS | EOF | '!' | '!=' | '#' | '(' | '(:' | ')' | '*' | '+' | ',' | '-' | '/' |
      break;
    case 6:                         // EQName^Token
    case 70:                        // 'after'
    case 72:                        // 'allowing'
    case 75:                        // 'and'
    case 77:                        // 'append'
    case 79:                        // 'as'
    case 80:                        // 'ascending'
    case 81:                        // 'at'
    case 83:                        // 'base-uri'
    case 84:                        // 'before'
    case 85:                        // 'boundary-space'
    case 86:                        // 'break'
    case 88:                        // 'case'
    case 89:                        // 'cast'
    case 90:                        // 'castable'
    case 91:                        // 'catch'
    case 94:                        // 'collation'
    case 97:                        // 'constraint'
    case 98:                        // 'construction'
    case 101:                       // 'context'
    case 102:                       // 'continue'
    case 103:                       // 'copy'
    case 104:                       // 'copy-namespaces'
    case 105:                       // 'count'
    case 106:                       // 'decimal-format'
    case 108:                       // 'declare'
    case 109:                       // 'default'
    case 110:                       // 'delete'
    case 113:                       // 'descending'
    case 118:                       // 'div'
    case 120:                       // 'document-node'
    case 122:                       // 'else'
    case 123:                       // 'empty'
    case 125:                       // 'encoding'
    case 126:                       // 'end'
    case 128:                       // 'eq'
    case 129:                       // 'every'
    case 131:                       // 'except'
    case 132:                       // 'exit'
    case 133:                       // 'external'
    case 134:                       // 'first'
    case 137:                       // 'for'
    case 141:                       // 'ft-option'
    case 145:                       // 'function'
    case 146:                       // 'ge'
    case 148:                       // 'group'
    case 150:                       // 'gt'
    case 151:                       // 'idiv'
    case 153:                       // 'import'
    case 154:                       // 'in'
    case 155:                       // 'index'
    case 159:                       // 'insert'
    case 160:                       // 'instance'
    case 161:                       // 'integrity'
    case 162:                       // 'intersect'
    case 163:                       // 'into'
    case 164:                       // 'is'
    case 166:                       // 'json'
    case 170:                       // 'last'
    case 171:                       // 'lax'
    case 172:                       // 'le'
    case 174:                       // 'let'
    case 176:                       // 'loop'
    case 178:                       // 'lt'
    case 180:                       // 'mod'
    case 181:                       // 'modify'
    case 182:                       // 'module'
    case 185:                       // 'namespace-node'
    case 186:                       // 'ne'
    case 191:                       // 'node'
    case 192:                       // 'nodes'
    case 194:                       // 'object'
    case 198:                       // 'only'
    case 199:                       // 'option'
    case 200:                       // 'or'
    case 201:                       // 'order'
    case 203:                       // 'ordering'
    case 218:                       // 'rename'
    case 219:                       // 'replace'
    case 220:                       // 'return'
    case 221:                       // 'returning'
    case 222:                       // 'revalidation'
    case 224:                       // 'satisfies'
    case 225:                       // 'schema'
    case 226:                       // 'schema-attribute'
    case 227:                       // 'schema-element'
    case 228:                       // 'score'
    case 234:                       // 'sliding'
    case 235:                       // 'some'
    case 236:                       // 'stable'
    case 237:                       // 'start'
    case 240:                       // 'strict'
    case 248:                       // 'to'
    case 249:                       // 'treat'
    case 250:                       // 'try'
    case 251:                       // 'tumbling'
    case 252:                       // 'type'
    case 254:                       // 'union'
    case 257:                       // 'updating'
    case 260:                       // 'validate'
    case 261:                       // 'value'
    case 262:                       // 'variable'
    case 263:                       // 'version'
    case 266:                       // 'where'
    case 267:                       // 'while'
    case 270:                       // 'with'
    case 274:                       // 'xquery'
      lookahead2W(244);             // S^WS | EOF | '!' | '!=' | '#' | '(' | '(:' | ')' | '*' | '+' | ',' | '-' | '/' |
      break;
    default:
      lk = l1;
    }
    if (lk == 35922                 // 'attribute' 'after'
     || lk == 35961                 // 'element' 'after'
     || lk == 36024                 // 'namespace' 'after'
     || lk == 36056                 // 'processing-instruction' 'after'
     || lk == 38482                 // 'attribute' 'and'
     || lk == 38521                 // 'element' 'and'
     || lk == 38584                 // 'namespace' 'and'
     || lk == 38616                 // 'processing-instruction' 'and'
     || lk == 40530                 // 'attribute' 'as'
     || lk == 40569                 // 'element' 'as'
     || lk == 40632                 // 'namespace' 'as'
     || lk == 40664                 // 'processing-instruction' 'as'
     || lk == 41042                 // 'attribute' 'ascending'
     || lk == 41081                 // 'element' 'ascending'
     || lk == 41144                 // 'namespace' 'ascending'
     || lk == 41176                 // 'processing-instruction' 'ascending'
     || lk == 41554                 // 'attribute' 'at'
     || lk == 41593                 // 'element' 'at'
     || lk == 41656                 // 'namespace' 'at'
     || lk == 41688                 // 'processing-instruction' 'at'
     || lk == 43090                 // 'attribute' 'before'
     || lk == 43129                 // 'element' 'before'
     || lk == 43192                 // 'namespace' 'before'
     || lk == 43224                 // 'processing-instruction' 'before'
     || lk == 45138                 // 'attribute' 'case'
     || lk == 45177                 // 'element' 'case'
     || lk == 45240                 // 'namespace' 'case'
     || lk == 45272                 // 'processing-instruction' 'case'
     || lk == 45650                 // 'attribute' 'cast'
     || lk == 45689                 // 'element' 'cast'
     || lk == 45752                 // 'namespace' 'cast'
     || lk == 45784                 // 'processing-instruction' 'cast'
     || lk == 46162                 // 'attribute' 'castable'
     || lk == 46201                 // 'element' 'castable'
     || lk == 46264                 // 'namespace' 'castable'
     || lk == 46296                 // 'processing-instruction' 'castable'
     || lk == 48210                 // 'attribute' 'collation'
     || lk == 48249                 // 'element' 'collation'
     || lk == 48312                 // 'namespace' 'collation'
     || lk == 48344                 // 'processing-instruction' 'collation'
     || lk == 53842                 // 'attribute' 'count'
     || lk == 53881                 // 'element' 'count'
     || lk == 53944                 // 'namespace' 'count'
     || lk == 53976                 // 'processing-instruction' 'count'
     || lk == 55890                 // 'attribute' 'default'
     || lk == 55929                 // 'element' 'default'
     || lk == 55992                 // 'namespace' 'default'
     || lk == 56024                 // 'processing-instruction' 'default'
     || lk == 57938                 // 'attribute' 'descending'
     || lk == 57977                 // 'element' 'descending'
     || lk == 58040                 // 'namespace' 'descending'
     || lk == 58072                 // 'processing-instruction' 'descending'
     || lk == 60498                 // 'attribute' 'div'
     || lk == 60537                 // 'element' 'div'
     || lk == 60600                 // 'namespace' 'div'
     || lk == 60632                 // 'processing-instruction' 'div'
     || lk == 62546                 // 'attribute' 'else'
     || lk == 62585                 // 'element' 'else'
     || lk == 62648                 // 'namespace' 'else'
     || lk == 62680                 // 'processing-instruction' 'else'
     || lk == 63058                 // 'attribute' 'empty'
     || lk == 63097                 // 'element' 'empty'
     || lk == 63160                 // 'namespace' 'empty'
     || lk == 63192                 // 'processing-instruction' 'empty'
     || lk == 64594                 // 'attribute' 'end'
     || lk == 64633                 // 'element' 'end'
     || lk == 64696                 // 'namespace' 'end'
     || lk == 64728                 // 'processing-instruction' 'end'
     || lk == 65618                 // 'attribute' 'eq'
     || lk == 65657                 // 'element' 'eq'
     || lk == 65720                 // 'namespace' 'eq'
     || lk == 65752                 // 'processing-instruction' 'eq'
     || lk == 67154                 // 'attribute' 'except'
     || lk == 67193                 // 'element' 'except'
     || lk == 67256                 // 'namespace' 'except'
     || lk == 67288                 // 'processing-instruction' 'except'
     || lk == 70226                 // 'attribute' 'for'
     || lk == 70265                 // 'element' 'for'
     || lk == 70328                 // 'namespace' 'for'
     || lk == 70360                 // 'processing-instruction' 'for'
     || lk == 74834                 // 'attribute' 'ge'
     || lk == 74873                 // 'element' 'ge'
     || lk == 74936                 // 'namespace' 'ge'
     || lk == 74968                 // 'processing-instruction' 'ge'
     || lk == 75858                 // 'attribute' 'group'
     || lk == 75897                 // 'element' 'group'
     || lk == 75960                 // 'namespace' 'group'
     || lk == 75992                 // 'processing-instruction' 'group'
     || lk == 76882                 // 'attribute' 'gt'
     || lk == 76921                 // 'element' 'gt'
     || lk == 76984                 // 'namespace' 'gt'
     || lk == 77016                 // 'processing-instruction' 'gt'
     || lk == 77394                 // 'attribute' 'idiv'
     || lk == 77433                 // 'element' 'idiv'
     || lk == 77496                 // 'namespace' 'idiv'
     || lk == 77528                 // 'processing-instruction' 'idiv'
     || lk == 82002                 // 'attribute' 'instance'
     || lk == 82041                 // 'element' 'instance'
     || lk == 82104                 // 'namespace' 'instance'
     || lk == 82136                 // 'processing-instruction' 'instance'
     || lk == 83026                 // 'attribute' 'intersect'
     || lk == 83065                 // 'element' 'intersect'
     || lk == 83128                 // 'namespace' 'intersect'
     || lk == 83160                 // 'processing-instruction' 'intersect'
     || lk == 83538                 // 'attribute' 'into'
     || lk == 83577                 // 'element' 'into'
     || lk == 83640                 // 'namespace' 'into'
     || lk == 83672                 // 'processing-instruction' 'into'
     || lk == 84050                 // 'attribute' 'is'
     || lk == 84089                 // 'element' 'is'
     || lk == 84152                 // 'namespace' 'is'
     || lk == 84184                 // 'processing-instruction' 'is'
     || lk == 88146                 // 'attribute' 'le'
     || lk == 88185                 // 'element' 'le'
     || lk == 88248                 // 'namespace' 'le'
     || lk == 88280                 // 'processing-instruction' 'le'
     || lk == 89170                 // 'attribute' 'let'
     || lk == 89209                 // 'element' 'let'
     || lk == 89272                 // 'namespace' 'let'
     || lk == 89304                 // 'processing-instruction' 'let'
     || lk == 91218                 // 'attribute' 'lt'
     || lk == 91257                 // 'element' 'lt'
     || lk == 91320                 // 'namespace' 'lt'
     || lk == 91352                 // 'processing-instruction' 'lt'
     || lk == 92242                 // 'attribute' 'mod'
     || lk == 92281                 // 'element' 'mod'
     || lk == 92344                 // 'namespace' 'mod'
     || lk == 92376                 // 'processing-instruction' 'mod'
     || lk == 92754                 // 'attribute' 'modify'
     || lk == 92793                 // 'element' 'modify'
     || lk == 92856                 // 'namespace' 'modify'
     || lk == 92888                 // 'processing-instruction' 'modify'
     || lk == 95314                 // 'attribute' 'ne'
     || lk == 95353                 // 'element' 'ne'
     || lk == 95416                 // 'namespace' 'ne'
     || lk == 95448                 // 'processing-instruction' 'ne'
     || lk == 101458                // 'attribute' 'only'
     || lk == 101497                // 'element' 'only'
     || lk == 101560                // 'namespace' 'only'
     || lk == 101592                // 'processing-instruction' 'only'
     || lk == 102482                // 'attribute' 'or'
     || lk == 102521                // 'element' 'or'
     || lk == 102584                // 'namespace' 'or'
     || lk == 102616                // 'processing-instruction' 'or'
     || lk == 102994                // 'attribute' 'order'
     || lk == 103033                // 'element' 'order'
     || lk == 103096                // 'namespace' 'order'
     || lk == 103128                // 'processing-instruction' 'order'
     || lk == 112722                // 'attribute' 'return'
     || lk == 112761                // 'element' 'return'
     || lk == 112824                // 'namespace' 'return'
     || lk == 112856                // 'processing-instruction' 'return'
     || lk == 114770                // 'attribute' 'satisfies'
     || lk == 114809                // 'element' 'satisfies'
     || lk == 114872                // 'namespace' 'satisfies'
     || lk == 114904                // 'processing-instruction' 'satisfies'
     || lk == 120914                // 'attribute' 'stable'
     || lk == 120953                // 'element' 'stable'
     || lk == 121016                // 'namespace' 'stable'
     || lk == 121048                // 'processing-instruction' 'stable'
     || lk == 121426                // 'attribute' 'start'
     || lk == 121465                // 'element' 'start'
     || lk == 121528                // 'namespace' 'start'
     || lk == 121560                // 'processing-instruction' 'start'
     || lk == 127058                // 'attribute' 'to'
     || lk == 127097                // 'element' 'to'
     || lk == 127160                // 'namespace' 'to'
     || lk == 127192                // 'processing-instruction' 'to'
     || lk == 127570                // 'attribute' 'treat'
     || lk == 127609                // 'element' 'treat'
     || lk == 127672                // 'namespace' 'treat'
     || lk == 127704                // 'processing-instruction' 'treat'
     || lk == 130130                // 'attribute' 'union'
     || lk == 130169                // 'element' 'union'
     || lk == 130232                // 'namespace' 'union'
     || lk == 130264                // 'processing-instruction' 'union'
     || lk == 136274                // 'attribute' 'where'
     || lk == 136313                // 'element' 'where'
     || lk == 136376                // 'namespace' 'where'
     || lk == 136408                // 'processing-instruction' 'where'
     || lk == 138322                // 'attribute' 'with'
     || lk == 138361                // 'element' 'with'
     || lk == 138424                // 'namespace' 'with'
     || lk == 138456)               // 'processing-instruction' 'with'
    {
      lk = memoized(4, e0);
      if (lk == 0)
      {
        var b0A = b0; var e0A = e0; var l1A = l1;
        var b1A = b1; var e1A = e1; var l2A = l2;
        var b2A = b2; var e2A = e2;
        try
        {
          try_PostfixExpr();
          lk = -1;
        }
        catch (p1A)
        {
          lk = -2;
        }
        b0 = b0A; e0 = e0A; l1 = l1A; if (l1 == 0) {end = e0A;} else {
        b1 = b1A; e1 = e1A; l2 = l2A; if (l2 == 0) {end = e1A;} else {
        b2 = b2A; e2 = e2A; end = e2A; }}
        memoize(4, e0, lk);
      }
    }
    switch (lk)
    {
    case -1:
    case 8:                         // IntegerLiteral
    case 9:                         // DecimalLiteral
    case 10:                        // DoubleLiteral
    case 11:                        // StringLiteral
    case 31:                        // '$'
    case 32:                        // '%'
    case 34:                        // '('
    case 44:                        // '.'
    case 54:                        // '<'
    case 55:                        // '<!--'
    case 59:                        // '<?'
    case 68:                        // '['
    case 276:                       // '{'
    case 278:                       // '{|'
    case 3154:                      // 'attribute' EQName^Token
    case 3193:                      // 'element' EQName^Token
    case 9912:                      // 'namespace' NCName^Token
    case 9944:                      // 'processing-instruction' NCName^Token
    case 14854:                     // EQName^Token '#'
    case 14918:                     // 'after' '#'
    case 14920:                     // 'allowing' '#'
    case 14921:                     // 'ancestor' '#'
    case 14922:                     // 'ancestor-or-self' '#'
    case 14923:                     // 'and' '#'
    case 14925:                     // 'append' '#'
    case 14926:                     // 'array' '#'
    case 14927:                     // 'as' '#'
    case 14928:                     // 'ascending' '#'
    case 14929:                     // 'at' '#'
    case 14930:                     // 'attribute' '#'
    case 14931:                     // 'base-uri' '#'
    case 14932:                     // 'before' '#'
    case 14933:                     // 'boundary-space' '#'
    case 14934:                     // 'break' '#'
    case 14936:                     // 'case' '#'
    case 14937:                     // 'cast' '#'
    case 14938:                     // 'castable' '#'
    case 14939:                     // 'catch' '#'
    case 14941:                     // 'child' '#'
    case 14942:                     // 'collation' '#'
    case 14944:                     // 'comment' '#'
    case 14945:                     // 'constraint' '#'
    case 14946:                     // 'construction' '#'
    case 14949:                     // 'context' '#'
    case 14950:                     // 'continue' '#'
    case 14951:                     // 'copy' '#'
    case 14952:                     // 'copy-namespaces' '#'
    case 14953:                     // 'count' '#'
    case 14954:                     // 'decimal-format' '#'
    case 14956:                     // 'declare' '#'
    case 14957:                     // 'default' '#'
    case 14958:                     // 'delete' '#'
    case 14959:                     // 'descendant' '#'
    case 14960:                     // 'descendant-or-self' '#'
    case 14961:                     // 'descending' '#'
    case 14966:                     // 'div' '#'
    case 14967:                     // 'document' '#'
    case 14968:                     // 'document-node' '#'
    case 14969:                     // 'element' '#'
    case 14970:                     // 'else' '#'
    case 14971:                     // 'empty' '#'
    case 14972:                     // 'empty-sequence' '#'
    case 14973:                     // 'encoding' '#'
    case 14974:                     // 'end' '#'
    case 14976:                     // 'eq' '#'
    case 14977:                     // 'every' '#'
    case 14979:                     // 'except' '#'
    case 14980:                     // 'exit' '#'
    case 14981:                     // 'external' '#'
    case 14982:                     // 'first' '#'
    case 14983:                     // 'following' '#'
    case 14984:                     // 'following-sibling' '#'
    case 14985:                     // 'for' '#'
    case 14989:                     // 'ft-option' '#'
    case 14993:                     // 'function' '#'
    case 14994:                     // 'ge' '#'
    case 14996:                     // 'group' '#'
    case 14998:                     // 'gt' '#'
    case 14999:                     // 'idiv' '#'
    case 15000:                     // 'if' '#'
    case 15001:                     // 'import' '#'
    case 15002:                     // 'in' '#'
    case 15003:                     // 'index' '#'
    case 15007:                     // 'insert' '#'
    case 15008:                     // 'instance' '#'
    case 15009:                     // 'integrity' '#'
    case 15010:                     // 'intersect' '#'
    case 15011:                     // 'into' '#'
    case 15012:                     // 'is' '#'
    case 15013:                     // 'item' '#'
    case 15014:                     // 'json' '#'
    case 15015:                     // 'json-item' '#'
    case 15018:                     // 'last' '#'
    case 15019:                     // 'lax' '#'
    case 15020:                     // 'le' '#'
    case 15022:                     // 'let' '#'
    case 15024:                     // 'loop' '#'
    case 15026:                     // 'lt' '#'
    case 15028:                     // 'mod' '#'
    case 15029:                     // 'modify' '#'
    case 15030:                     // 'module' '#'
    case 15032:                     // 'namespace' '#'
    case 15033:                     // 'namespace-node' '#'
    case 15034:                     // 'ne' '#'
    case 15039:                     // 'node' '#'
    case 15040:                     // 'nodes' '#'
    case 15042:                     // 'object' '#'
    case 15046:                     // 'only' '#'
    case 15047:                     // 'option' '#'
    case 15048:                     // 'or' '#'
    case 15049:                     // 'order' '#'
    case 15050:                     // 'ordered' '#'
    case 15051:                     // 'ordering' '#'
    case 15054:                     // 'parent' '#'
    case 15060:                     // 'preceding' '#'
    case 15061:                     // 'preceding-sibling' '#'
    case 15064:                     // 'processing-instruction' '#'
    case 15066:                     // 'rename' '#'
    case 15067:                     // 'replace' '#'
    case 15068:                     // 'return' '#'
    case 15069:                     // 'returning' '#'
    case 15070:                     // 'revalidation' '#'
    case 15072:                     // 'satisfies' '#'
    case 15073:                     // 'schema' '#'
    case 15074:                     // 'schema-attribute' '#'
    case 15075:                     // 'schema-element' '#'
    case 15076:                     // 'score' '#'
    case 15077:                     // 'self' '#'
    case 15082:                     // 'sliding' '#'
    case 15083:                     // 'some' '#'
    case 15084:                     // 'stable' '#'
    case 15085:                     // 'start' '#'
    case 15088:                     // 'strict' '#'
    case 15090:                     // 'structured-item' '#'
    case 15091:                     // 'switch' '#'
    case 15092:                     // 'text' '#'
    case 15096:                     // 'to' '#'
    case 15097:                     // 'treat' '#'
    case 15098:                     // 'try' '#'
    case 15099:                     // 'tumbling' '#'
    case 15100:                     // 'type' '#'
    case 15101:                     // 'typeswitch' '#'
    case 15102:                     // 'union' '#'
    case 15104:                     // 'unordered' '#'
    case 15105:                     // 'updating' '#'
    case 15108:                     // 'validate' '#'
    case 15109:                     // 'value' '#'
    case 15110:                     // 'variable' '#'
    case 15111:                     // 'version' '#'
    case 15114:                     // 'where' '#'
    case 15115:                     // 'while' '#'
    case 15118:                     // 'with' '#'
    case 15122:                     // 'xquery' '#'
    case 17414:                     // EQName^Token '('
    case 17478:                     // 'after' '('
    case 17480:                     // 'allowing' '('
    case 17481:                     // 'ancestor' '('
    case 17482:                     // 'ancestor-or-self' '('
    case 17483:                     // 'and' '('
    case 17485:                     // 'append' '('
    case 17487:                     // 'as' '('
    case 17488:                     // 'ascending' '('
    case 17489:                     // 'at' '('
    case 17491:                     // 'base-uri' '('
    case 17492:                     // 'before' '('
    case 17493:                     // 'boundary-space' '('
    case 17494:                     // 'break' '('
    case 17496:                     // 'case' '('
    case 17497:                     // 'cast' '('
    case 17498:                     // 'castable' '('
    case 17499:                     // 'catch' '('
    case 17501:                     // 'child' '('
    case 17502:                     // 'collation' '('
    case 17505:                     // 'constraint' '('
    case 17506:                     // 'construction' '('
    case 17509:                     // 'context' '('
    case 17510:                     // 'continue' '('
    case 17511:                     // 'copy' '('
    case 17512:                     // 'copy-namespaces' '('
    case 17513:                     // 'count' '('
    case 17514:                     // 'decimal-format' '('
    case 17516:                     // 'declare' '('
    case 17517:                     // 'default' '('
    case 17518:                     // 'delete' '('
    case 17519:                     // 'descendant' '('
    case 17520:                     // 'descendant-or-self' '('
    case 17521:                     // 'descending' '('
    case 17526:                     // 'div' '('
    case 17527:                     // 'document' '('
    case 17530:                     // 'else' '('
    case 17531:                     // 'empty' '('
    case 17533:                     // 'encoding' '('
    case 17534:                     // 'end' '('
    case 17536:                     // 'eq' '('
    case 17537:                     // 'every' '('
    case 17539:                     // 'except' '('
    case 17540:                     // 'exit' '('
    case 17541:                     // 'external' '('
    case 17542:                     // 'first' '('
    case 17543:                     // 'following' '('
    case 17544:                     // 'following-sibling' '('
    case 17545:                     // 'for' '('
    case 17549:                     // 'ft-option' '('
    case 17553:                     // 'function' '('
    case 17554:                     // 'ge' '('
    case 17556:                     // 'group' '('
    case 17558:                     // 'gt' '('
    case 17559:                     // 'idiv' '('
    case 17561:                     // 'import' '('
    case 17562:                     // 'in' '('
    case 17563:                     // 'index' '('
    case 17567:                     // 'insert' '('
    case 17568:                     // 'instance' '('
    case 17569:                     // 'integrity' '('
    case 17570:                     // 'intersect' '('
    case 17571:                     // 'into' '('
    case 17572:                     // 'is' '('
    case 17574:                     // 'json' '('
    case 17578:                     // 'last' '('
    case 17579:                     // 'lax' '('
    case 17580:                     // 'le' '('
    case 17582:                     // 'let' '('
    case 17584:                     // 'loop' '('
    case 17586:                     // 'lt' '('
    case 17588:                     // 'mod' '('
    case 17589:                     // 'modify' '('
    case 17590:                     // 'module' '('
    case 17592:                     // 'namespace' '('
    case 17594:                     // 'ne' '('
    case 17600:                     // 'nodes' '('
    case 17602:                     // 'object' '('
    case 17606:                     // 'only' '('
    case 17607:                     // 'option' '('
    case 17608:                     // 'or' '('
    case 17609:                     // 'order' '('
    case 17610:                     // 'ordered' '('
    case 17611:                     // 'ordering' '('
    case 17614:                     // 'parent' '('
    case 17620:                     // 'preceding' '('
    case 17621:                     // 'preceding-sibling' '('
    case 17626:                     // 'rename' '('
    case 17627:                     // 'replace' '('
    case 17628:                     // 'return' '('
    case 17629:                     // 'returning' '('
    case 17630:                     // 'revalidation' '('
    case 17632:                     // 'satisfies' '('
    case 17633:                     // 'schema' '('
    case 17636:                     // 'score' '('
    case 17637:                     // 'self' '('
    case 17642:                     // 'sliding' '('
    case 17643:                     // 'some' '('
    case 17644:                     // 'stable' '('
    case 17645:                     // 'start' '('
    case 17648:                     // 'strict' '('
    case 17656:                     // 'to' '('
    case 17657:                     // 'treat' '('
    case 17658:                     // 'try' '('
    case 17659:                     // 'tumbling' '('
    case 17660:                     // 'type' '('
    case 17662:                     // 'union' '('
    case 17664:                     // 'unordered' '('
    case 17665:                     // 'updating' '('
    case 17668:                     // 'validate' '('
    case 17669:                     // 'value' '('
    case 17670:                     // 'variable' '('
    case 17671:                     // 'version' '('
    case 17674:                     // 'where' '('
    case 17675:                     // 'while' '('
    case 17678:                     // 'with' '('
    case 17682:                     // 'xquery' '('
    case 36946:                     // 'attribute' 'allowing'
    case 36985:                     // 'element' 'allowing'
    case 37048:                     // 'namespace' 'allowing'
    case 37080:                     // 'processing-instruction' 'allowing'
    case 37458:                     // 'attribute' 'ancestor'
    case 37497:                     // 'element' 'ancestor'
    case 37560:                     // 'namespace' 'ancestor'
    case 37592:                     // 'processing-instruction' 'ancestor'
    case 37970:                     // 'attribute' 'ancestor-or-self'
    case 38009:                     // 'element' 'ancestor-or-self'
    case 38072:                     // 'namespace' 'ancestor-or-self'
    case 38104:                     // 'processing-instruction' 'ancestor-or-self'
    case 39506:                     // 'attribute' 'append'
    case 39545:                     // 'element' 'append'
    case 39608:                     // 'namespace' 'append'
    case 39640:                     // 'processing-instruction' 'append'
    case 40018:                     // 'attribute' 'array'
    case 40057:                     // 'element' 'array'
    case 42066:                     // 'attribute' 'attribute'
    case 42105:                     // 'element' 'attribute'
    case 42168:                     // 'namespace' 'attribute'
    case 42200:                     // 'processing-instruction' 'attribute'
    case 42578:                     // 'attribute' 'base-uri'
    case 42617:                     // 'element' 'base-uri'
    case 42680:                     // 'namespace' 'base-uri'
    case 42712:                     // 'processing-instruction' 'base-uri'
    case 43602:                     // 'attribute' 'boundary-space'
    case 43641:                     // 'element' 'boundary-space'
    case 43704:                     // 'namespace' 'boundary-space'
    case 43736:                     // 'processing-instruction' 'boundary-space'
    case 44114:                     // 'attribute' 'break'
    case 44153:                     // 'element' 'break'
    case 44216:                     // 'namespace' 'break'
    case 44248:                     // 'processing-instruction' 'break'
    case 46674:                     // 'attribute' 'catch'
    case 46713:                     // 'element' 'catch'
    case 46776:                     // 'namespace' 'catch'
    case 46808:                     // 'processing-instruction' 'catch'
    case 47698:                     // 'attribute' 'child'
    case 47737:                     // 'element' 'child'
    case 47800:                     // 'namespace' 'child'
    case 47832:                     // 'processing-instruction' 'child'
    case 49234:                     // 'attribute' 'comment'
    case 49273:                     // 'element' 'comment'
    case 49336:                     // 'namespace' 'comment'
    case 49368:                     // 'processing-instruction' 'comment'
    case 49746:                     // 'attribute' 'constraint'
    case 49785:                     // 'element' 'constraint'
    case 49848:                     // 'namespace' 'constraint'
    case 49880:                     // 'processing-instruction' 'constraint'
    case 50258:                     // 'attribute' 'construction'
    case 50297:                     // 'element' 'construction'
    case 50360:                     // 'namespace' 'construction'
    case 50392:                     // 'processing-instruction' 'construction'
    case 51794:                     // 'attribute' 'context'
    case 51833:                     // 'element' 'context'
    case 51896:                     // 'namespace' 'context'
    case 51928:                     // 'processing-instruction' 'context'
    case 52306:                     // 'attribute' 'continue'
    case 52345:                     // 'element' 'continue'
    case 52408:                     // 'namespace' 'continue'
    case 52440:                     // 'processing-instruction' 'continue'
    case 52818:                     // 'attribute' 'copy'
    case 52857:                     // 'element' 'copy'
    case 52920:                     // 'namespace' 'copy'
    case 52952:                     // 'processing-instruction' 'copy'
    case 53330:                     // 'attribute' 'copy-namespaces'
    case 53369:                     // 'element' 'copy-namespaces'
    case 53432:                     // 'namespace' 'copy-namespaces'
    case 53464:                     // 'processing-instruction' 'copy-namespaces'
    case 54354:                     // 'attribute' 'decimal-format'
    case 54393:                     // 'element' 'decimal-format'
    case 54456:                     // 'namespace' 'decimal-format'
    case 54488:                     // 'processing-instruction' 'decimal-format'
    case 55378:                     // 'attribute' 'declare'
    case 55417:                     // 'element' 'declare'
    case 55480:                     // 'namespace' 'declare'
    case 55512:                     // 'processing-instruction' 'declare'
    case 56402:                     // 'attribute' 'delete'
    case 56441:                     // 'element' 'delete'
    case 56504:                     // 'namespace' 'delete'
    case 56536:                     // 'processing-instruction' 'delete'
    case 56914:                     // 'attribute' 'descendant'
    case 56953:                     // 'element' 'descendant'
    case 57016:                     // 'namespace' 'descendant'
    case 57048:                     // 'processing-instruction' 'descendant'
    case 57426:                     // 'attribute' 'descendant-or-self'
    case 57465:                     // 'element' 'descendant-or-self'
    case 57528:                     // 'namespace' 'descendant-or-self'
    case 57560:                     // 'processing-instruction' 'descendant-or-self'
    case 61010:                     // 'attribute' 'document'
    case 61049:                     // 'element' 'document'
    case 61112:                     // 'namespace' 'document'
    case 61144:                     // 'processing-instruction' 'document'
    case 61522:                     // 'attribute' 'document-node'
    case 61561:                     // 'element' 'document-node'
    case 61624:                     // 'namespace' 'document-node'
    case 61656:                     // 'processing-instruction' 'document-node'
    case 62034:                     // 'attribute' 'element'
    case 62073:                     // 'element' 'element'
    case 62136:                     // 'namespace' 'element'
    case 62168:                     // 'processing-instruction' 'element'
    case 63570:                     // 'attribute' 'empty-sequence'
    case 63609:                     // 'element' 'empty-sequence'
    case 63672:                     // 'namespace' 'empty-sequence'
    case 63704:                     // 'processing-instruction' 'empty-sequence'
    case 64082:                     // 'attribute' 'encoding'
    case 64121:                     // 'element' 'encoding'
    case 64184:                     // 'namespace' 'encoding'
    case 64216:                     // 'processing-instruction' 'encoding'
    case 66130:                     // 'attribute' 'every'
    case 66169:                     // 'element' 'every'
    case 66232:                     // 'namespace' 'every'
    case 66264:                     // 'processing-instruction' 'every'
    case 67666:                     // 'attribute' 'exit'
    case 67705:                     // 'element' 'exit'
    case 67768:                     // 'namespace' 'exit'
    case 67800:                     // 'processing-instruction' 'exit'
    case 68178:                     // 'attribute' 'external'
    case 68217:                     // 'element' 'external'
    case 68280:                     // 'namespace' 'external'
    case 68312:                     // 'processing-instruction' 'external'
    case 68690:                     // 'attribute' 'first'
    case 68729:                     // 'element' 'first'
    case 68792:                     // 'namespace' 'first'
    case 68824:                     // 'processing-instruction' 'first'
    case 69202:                     // 'attribute' 'following'
    case 69241:                     // 'element' 'following'
    case 69304:                     // 'namespace' 'following'
    case 69336:                     // 'processing-instruction' 'following'
    case 69714:                     // 'attribute' 'following-sibling'
    case 69753:                     // 'element' 'following-sibling'
    case 69816:                     // 'namespace' 'following-sibling'
    case 69848:                     // 'processing-instruction' 'following-sibling'
    case 72274:                     // 'attribute' 'ft-option'
    case 72313:                     // 'element' 'ft-option'
    case 72376:                     // 'namespace' 'ft-option'
    case 72408:                     // 'processing-instruction' 'ft-option'
    case 74322:                     // 'attribute' 'function'
    case 74361:                     // 'element' 'function'
    case 74424:                     // 'namespace' 'function'
    case 74456:                     // 'processing-instruction' 'function'
    case 77906:                     // 'attribute' 'if'
    case 77945:                     // 'element' 'if'
    case 78008:                     // 'namespace' 'if'
    case 78040:                     // 'processing-instruction' 'if'
    case 78418:                     // 'attribute' 'import'
    case 78457:                     // 'element' 'import'
    case 78520:                     // 'namespace' 'import'
    case 78552:                     // 'processing-instruction' 'import'
    case 78930:                     // 'attribute' 'in'
    case 78969:                     // 'element' 'in'
    case 79032:                     // 'namespace' 'in'
    case 79064:                     // 'processing-instruction' 'in'
    case 79442:                     // 'attribute' 'index'
    case 79481:                     // 'element' 'index'
    case 79544:                     // 'namespace' 'index'
    case 79576:                     // 'processing-instruction' 'index'
    case 81490:                     // 'attribute' 'insert'
    case 81529:                     // 'element' 'insert'
    case 81592:                     // 'namespace' 'insert'
    case 81624:                     // 'processing-instruction' 'insert'
    case 82514:                     // 'attribute' 'integrity'
    case 82553:                     // 'element' 'integrity'
    case 82616:                     // 'namespace' 'integrity'
    case 82648:                     // 'processing-instruction' 'integrity'
    case 84562:                     // 'attribute' 'item'
    case 84601:                     // 'element' 'item'
    case 84664:                     // 'namespace' 'item'
    case 84696:                     // 'processing-instruction' 'item'
    case 85074:                     // 'attribute' 'json'
    case 85113:                     // 'element' 'json'
    case 85176:                     // 'namespace' 'json'
    case 85208:                     // 'processing-instruction' 'json'
    case 85586:                     // 'attribute' 'json-item'
    case 85625:                     // 'element' 'json-item'
    case 87122:                     // 'attribute' 'last'
    case 87161:                     // 'element' 'last'
    case 87224:                     // 'namespace' 'last'
    case 87256:                     // 'processing-instruction' 'last'
    case 87634:                     // 'attribute' 'lax'
    case 87673:                     // 'element' 'lax'
    case 87736:                     // 'namespace' 'lax'
    case 87768:                     // 'processing-instruction' 'lax'
    case 90194:                     // 'attribute' 'loop'
    case 90233:                     // 'element' 'loop'
    case 90296:                     // 'namespace' 'loop'
    case 90328:                     // 'processing-instruction' 'loop'
    case 93266:                     // 'attribute' 'module'
    case 93305:                     // 'element' 'module'
    case 93368:                     // 'namespace' 'module'
    case 93400:                     // 'processing-instruction' 'module'
    case 94290:                     // 'attribute' 'namespace'
    case 94329:                     // 'element' 'namespace'
    case 94392:                     // 'namespace' 'namespace'
    case 94424:                     // 'processing-instruction' 'namespace'
    case 94802:                     // 'attribute' 'namespace-node'
    case 94841:                     // 'element' 'namespace-node'
    case 94904:                     // 'namespace' 'namespace-node'
    case 94936:                     // 'processing-instruction' 'namespace-node'
    case 97874:                     // 'attribute' 'node'
    case 97913:                     // 'element' 'node'
    case 97976:                     // 'namespace' 'node'
    case 98008:                     // 'processing-instruction' 'node'
    case 98386:                     // 'attribute' 'nodes'
    case 98425:                     // 'element' 'nodes'
    case 98488:                     // 'namespace' 'nodes'
    case 98520:                     // 'processing-instruction' 'nodes'
    case 99410:                     // 'attribute' 'object'
    case 99449:                     // 'element' 'object'
    case 99512:                     // 'namespace' 'object'
    case 99544:                     // 'processing-instruction' 'object'
    case 101970:                    // 'attribute' 'option'
    case 102009:                    // 'element' 'option'
    case 102072:                    // 'namespace' 'option'
    case 102104:                    // 'processing-instruction' 'option'
    case 103506:                    // 'attribute' 'ordered'
    case 103545:                    // 'element' 'ordered'
    case 103608:                    // 'namespace' 'ordered'
    case 103640:                    // 'processing-instruction' 'ordered'
    case 104018:                    // 'attribute' 'ordering'
    case 104057:                    // 'element' 'ordering'
    case 104120:                    // 'namespace' 'ordering'
    case 104152:                    // 'processing-instruction' 'ordering'
    case 105554:                    // 'attribute' 'parent'
    case 105593:                    // 'element' 'parent'
    case 105656:                    // 'namespace' 'parent'
    case 105688:                    // 'processing-instruction' 'parent'
    case 108626:                    // 'attribute' 'preceding'
    case 108665:                    // 'element' 'preceding'
    case 108728:                    // 'namespace' 'preceding'
    case 108760:                    // 'processing-instruction' 'preceding'
    case 109138:                    // 'attribute' 'preceding-sibling'
    case 109177:                    // 'element' 'preceding-sibling'
    case 109240:                    // 'namespace' 'preceding-sibling'
    case 109272:                    // 'processing-instruction' 'preceding-sibling'
    case 110674:                    // 'attribute' 'processing-instruction'
    case 110713:                    // 'element' 'processing-instruction'
    case 110776:                    // 'namespace' 'processing-instruction'
    case 110808:                    // 'processing-instruction' 'processing-instruction'
    case 111698:                    // 'attribute' 'rename'
    case 111737:                    // 'element' 'rename'
    case 111800:                    // 'namespace' 'rename'
    case 111832:                    // 'processing-instruction' 'rename'
    case 112210:                    // 'attribute' 'replace'
    case 112249:                    // 'element' 'replace'
    case 112312:                    // 'namespace' 'replace'
    case 112344:                    // 'processing-instruction' 'replace'
    case 113234:                    // 'attribute' 'returning'
    case 113273:                    // 'element' 'returning'
    case 113336:                    // 'namespace' 'returning'
    case 113368:                    // 'processing-instruction' 'returning'
    case 113746:                    // 'attribute' 'revalidation'
    case 113785:                    // 'element' 'revalidation'
    case 113848:                    // 'namespace' 'revalidation'
    case 113880:                    // 'processing-instruction' 'revalidation'
    case 115282:                    // 'attribute' 'schema'
    case 115321:                    // 'element' 'schema'
    case 115384:                    // 'namespace' 'schema'
    case 115416:                    // 'processing-instruction' 'schema'
    case 115794:                    // 'attribute' 'schema-attribute'
    case 115833:                    // 'element' 'schema-attribute'
    case 115896:                    // 'namespace' 'schema-attribute'
    case 115928:                    // 'processing-instruction' 'schema-attribute'
    case 116306:                    // 'attribute' 'schema-element'
    case 116345:                    // 'element' 'schema-element'
    case 116408:                    // 'namespace' 'schema-element'
    case 116440:                    // 'processing-instruction' 'schema-element'
    case 116818:                    // 'attribute' 'score'
    case 116857:                    // 'element' 'score'
    case 116920:                    // 'namespace' 'score'
    case 116952:                    // 'processing-instruction' 'score'
    case 117330:                    // 'attribute' 'self'
    case 117369:                    // 'element' 'self'
    case 117432:                    // 'namespace' 'self'
    case 117464:                    // 'processing-instruction' 'self'
    case 119890:                    // 'attribute' 'sliding'
    case 119929:                    // 'element' 'sliding'
    case 119992:                    // 'namespace' 'sliding'
    case 120024:                    // 'processing-instruction' 'sliding'
    case 120402:                    // 'attribute' 'some'
    case 120441:                    // 'element' 'some'
    case 120504:                    // 'namespace' 'some'
    case 120536:                    // 'processing-instruction' 'some'
    case 122962:                    // 'attribute' 'strict'
    case 123001:                    // 'element' 'strict'
    case 123064:                    // 'namespace' 'strict'
    case 123096:                    // 'processing-instruction' 'strict'
    case 123986:                    // 'attribute' 'structured-item'
    case 124025:                    // 'element' 'structured-item'
    case 124498:                    // 'attribute' 'switch'
    case 124537:                    // 'element' 'switch'
    case 124600:                    // 'namespace' 'switch'
    case 124632:                    // 'processing-instruction' 'switch'
    case 125010:                    // 'attribute' 'text'
    case 125049:                    // 'element' 'text'
    case 125112:                    // 'namespace' 'text'
    case 125144:                    // 'processing-instruction' 'text'
    case 128082:                    // 'attribute' 'try'
    case 128121:                    // 'element' 'try'
    case 128184:                    // 'namespace' 'try'
    case 128216:                    // 'processing-instruction' 'try'
    case 128594:                    // 'attribute' 'tumbling'
    case 128633:                    // 'element' 'tumbling'
    case 128696:                    // 'namespace' 'tumbling'
    case 128728:                    // 'processing-instruction' 'tumbling'
    case 129106:                    // 'attribute' 'type'
    case 129145:                    // 'element' 'type'
    case 129208:                    // 'namespace' 'type'
    case 129240:                    // 'processing-instruction' 'type'
    case 129618:                    // 'attribute' 'typeswitch'
    case 129657:                    // 'element' 'typeswitch'
    case 129720:                    // 'namespace' 'typeswitch'
    case 129752:                    // 'processing-instruction' 'typeswitch'
    case 131154:                    // 'attribute' 'unordered'
    case 131193:                    // 'element' 'unordered'
    case 131256:                    // 'namespace' 'unordered'
    case 131288:                    // 'processing-instruction' 'unordered'
    case 131666:                    // 'attribute' 'updating'
    case 131705:                    // 'element' 'updating'
    case 131768:                    // 'namespace' 'updating'
    case 131800:                    // 'processing-instruction' 'updating'
    case 133202:                    // 'attribute' 'validate'
    case 133241:                    // 'element' 'validate'
    case 133304:                    // 'namespace' 'validate'
    case 133336:                    // 'processing-instruction' 'validate'
    case 133714:                    // 'attribute' 'value'
    case 133753:                    // 'element' 'value'
    case 133816:                    // 'namespace' 'value'
    case 133848:                    // 'processing-instruction' 'value'
    case 134226:                    // 'attribute' 'variable'
    case 134265:                    // 'element' 'variable'
    case 134328:                    // 'namespace' 'variable'
    case 134360:                    // 'processing-instruction' 'variable'
    case 134738:                    // 'attribute' 'version'
    case 134777:                    // 'element' 'version'
    case 134840:                    // 'namespace' 'version'
    case 134872:                    // 'processing-instruction' 'version'
    case 136786:                    // 'attribute' 'while'
    case 136825:                    // 'element' 'while'
    case 136888:                    // 'namespace' 'while'
    case 136920:                    // 'processing-instruction' 'while'
    case 140370:                    // 'attribute' 'xquery'
    case 140409:                    // 'element' 'xquery'
    case 140472:                    // 'namespace' 'xquery'
    case 140504:                    // 'processing-instruction' 'xquery'
    case 141394:                    // 'attribute' '{'
    case 141408:                    // 'comment' '{'
    case 141431:                    // 'document' '{'
    case 141433:                    // 'element' '{'
    case 141496:                    // 'namespace' '{'
    case 141514:                    // 'ordered' '{'
    case 141528:                    // 'processing-instruction' '{'
    case 141556:                    // 'text' '{'
    case 141568:                    // 'unordered' '{'
      parse_PostfixExpr();
      break;
    default:
      parse_AxisStep();
    }
    eventHandler.endNonterminal("StepExpr", e0);
  }

  function try_StepExpr()
  {
    switch (l1)
    {
    case 82:                        // 'attribute'
      lookahead2W(285);             // EQName^Token | S^WS | EOF | '!' | '!=' | '#' | '(' | '(:' | ')' | '*' | '+' |
      break;
    case 121:                       // 'element'
      lookahead2W(283);             // EQName^Token | S^WS | EOF | '!' | '!=' | '#' | '(' | '(:' | ')' | '*' | '+' |
      break;
    case 184:                       // 'namespace'
    case 216:                       // 'processing-instruction'
      lookahead2W(282);             // NCName^Token | S^WS | EOF | '!' | '!=' | '#' | '(' | '(:' | ')' | '*' | '+' |
      break;
    case 96:                        // 'comment'
    case 119:                       // 'document'
    case 202:                       // 'ordered'
    case 244:                       // 'text'
    case 256:                       // 'unordered'
      lookahead2W(247);             // S^WS | EOF | '!' | '!=' | '#' | '(' | '(:' | ')' | '*' | '+' | ',' | '-' | '/' |
      break;
    case 78:                        // 'array'
    case 124:                       // 'empty-sequence'
    case 152:                       // 'if'
    case 165:                       // 'item'
    case 167:                       // 'json-item'
    case 242:                       // 'structured-item'
    case 243:                       // 'switch'
    case 253:                       // 'typeswitch'
      lookahead2W(240);             // S^WS | EOF | '!' | '!=' | '#' | '(:' | ')' | '*' | '+' | ',' | '-' | '/' | '//' |
      break;
    case 73:                        // 'ancestor'
    case 74:                        // 'ancestor-or-self'
    case 93:                        // 'child'
    case 111:                       // 'descendant'
    case 112:                       // 'descendant-or-self'
    case 135:                       // 'following'
    case 136:                       // 'following-sibling'
    case 206:                       // 'parent'
    case 212:                       // 'preceding'
    case 213:                       // 'preceding-sibling'
    case 229:                       // 'self'
      lookahead2W(246);             // S^WS | EOF | '!' | '!=' | '#' | '(' | '(:' | ')' | '*' | '+' | ',' | '-' | '/' |
      break;
    case 6:                         // EQName^Token
    case 70:                        // 'after'
    case 72:                        // 'allowing'
    case 75:                        // 'and'
    case 77:                        // 'append'
    case 79:                        // 'as'
    case 80:                        // 'ascending'
    case 81:                        // 'at'
    case 83:                        // 'base-uri'
    case 84:                        // 'before'
    case 85:                        // 'boundary-space'
    case 86:                        // 'break'
    case 88:                        // 'case'
    case 89:                        // 'cast'
    case 90:                        // 'castable'
    case 91:                        // 'catch'
    case 94:                        // 'collation'
    case 97:                        // 'constraint'
    case 98:                        // 'construction'
    case 101:                       // 'context'
    case 102:                       // 'continue'
    case 103:                       // 'copy'
    case 104:                       // 'copy-namespaces'
    case 105:                       // 'count'
    case 106:                       // 'decimal-format'
    case 108:                       // 'declare'
    case 109:                       // 'default'
    case 110:                       // 'delete'
    case 113:                       // 'descending'
    case 118:                       // 'div'
    case 120:                       // 'document-node'
    case 122:                       // 'else'
    case 123:                       // 'empty'
    case 125:                       // 'encoding'
    case 126:                       // 'end'
    case 128:                       // 'eq'
    case 129:                       // 'every'
    case 131:                       // 'except'
    case 132:                       // 'exit'
    case 133:                       // 'external'
    case 134:                       // 'first'
    case 137:                       // 'for'
    case 141:                       // 'ft-option'
    case 145:                       // 'function'
    case 146:                       // 'ge'
    case 148:                       // 'group'
    case 150:                       // 'gt'
    case 151:                       // 'idiv'
    case 153:                       // 'import'
    case 154:                       // 'in'
    case 155:                       // 'index'
    case 159:                       // 'insert'
    case 160:                       // 'instance'
    case 161:                       // 'integrity'
    case 162:                       // 'intersect'
    case 163:                       // 'into'
    case 164:                       // 'is'
    case 166:                       // 'json'
    case 170:                       // 'last'
    case 171:                       // 'lax'
    case 172:                       // 'le'
    case 174:                       // 'let'
    case 176:                       // 'loop'
    case 178:                       // 'lt'
    case 180:                       // 'mod'
    case 181:                       // 'modify'
    case 182:                       // 'module'
    case 185:                       // 'namespace-node'
    case 186:                       // 'ne'
    case 191:                       // 'node'
    case 192:                       // 'nodes'
    case 194:                       // 'object'
    case 198:                       // 'only'
    case 199:                       // 'option'
    case 200:                       // 'or'
    case 201:                       // 'order'
    case 203:                       // 'ordering'
    case 218:                       // 'rename'
    case 219:                       // 'replace'
    case 220:                       // 'return'
    case 221:                       // 'returning'
    case 222:                       // 'revalidation'
    case 224:                       // 'satisfies'
    case 225:                       // 'schema'
    case 226:                       // 'schema-attribute'
    case 227:                       // 'schema-element'
    case 228:                       // 'score'
    case 234:                       // 'sliding'
    case 235:                       // 'some'
    case 236:                       // 'stable'
    case 237:                       // 'start'
    case 240:                       // 'strict'
    case 248:                       // 'to'
    case 249:                       // 'treat'
    case 250:                       // 'try'
    case 251:                       // 'tumbling'
    case 252:                       // 'type'
    case 254:                       // 'union'
    case 257:                       // 'updating'
    case 260:                       // 'validate'
    case 261:                       // 'value'
    case 262:                       // 'variable'
    case 263:                       // 'version'
    case 266:                       // 'where'
    case 267:                       // 'while'
    case 270:                       // 'with'
    case 274:                       // 'xquery'
      lookahead2W(244);             // S^WS | EOF | '!' | '!=' | '#' | '(' | '(:' | ')' | '*' | '+' | ',' | '-' | '/' |
      break;
    default:
      lk = l1;
    }
    if (lk == 35922                 // 'attribute' 'after'
     || lk == 35961                 // 'element' 'after'
     || lk == 36024                 // 'namespace' 'after'
     || lk == 36056                 // 'processing-instruction' 'after'
     || lk == 38482                 // 'attribute' 'and'
     || lk == 38521                 // 'element' 'and'
     || lk == 38584                 // 'namespace' 'and'
     || lk == 38616                 // 'processing-instruction' 'and'
     || lk == 40530                 // 'attribute' 'as'
     || lk == 40569                 // 'element' 'as'
     || lk == 40632                 // 'namespace' 'as'
     || lk == 40664                 // 'processing-instruction' 'as'
     || lk == 41042                 // 'attribute' 'ascending'
     || lk == 41081                 // 'element' 'ascending'
     || lk == 41144                 // 'namespace' 'ascending'
     || lk == 41176                 // 'processing-instruction' 'ascending'
     || lk == 41554                 // 'attribute' 'at'
     || lk == 41593                 // 'element' 'at'
     || lk == 41656                 // 'namespace' 'at'
     || lk == 41688                 // 'processing-instruction' 'at'
     || lk == 43090                 // 'attribute' 'before'
     || lk == 43129                 // 'element' 'before'
     || lk == 43192                 // 'namespace' 'before'
     || lk == 43224                 // 'processing-instruction' 'before'
     || lk == 45138                 // 'attribute' 'case'
     || lk == 45177                 // 'element' 'case'
     || lk == 45240                 // 'namespace' 'case'
     || lk == 45272                 // 'processing-instruction' 'case'
     || lk == 45650                 // 'attribute' 'cast'
     || lk == 45689                 // 'element' 'cast'
     || lk == 45752                 // 'namespace' 'cast'
     || lk == 45784                 // 'processing-instruction' 'cast'
     || lk == 46162                 // 'attribute' 'castable'
     || lk == 46201                 // 'element' 'castable'
     || lk == 46264                 // 'namespace' 'castable'
     || lk == 46296                 // 'processing-instruction' 'castable'
     || lk == 48210                 // 'attribute' 'collation'
     || lk == 48249                 // 'element' 'collation'
     || lk == 48312                 // 'namespace' 'collation'
     || lk == 48344                 // 'processing-instruction' 'collation'
     || lk == 53842                 // 'attribute' 'count'
     || lk == 53881                 // 'element' 'count'
     || lk == 53944                 // 'namespace' 'count'
     || lk == 53976                 // 'processing-instruction' 'count'
     || lk == 55890                 // 'attribute' 'default'
     || lk == 55929                 // 'element' 'default'
     || lk == 55992                 // 'namespace' 'default'
     || lk == 56024                 // 'processing-instruction' 'default'
     || lk == 57938                 // 'attribute' 'descending'
     || lk == 57977                 // 'element' 'descending'
     || lk == 58040                 // 'namespace' 'descending'
     || lk == 58072                 // 'processing-instruction' 'descending'
     || lk == 60498                 // 'attribute' 'div'
     || lk == 60537                 // 'element' 'div'
     || lk == 60600                 // 'namespace' 'div'
     || lk == 60632                 // 'processing-instruction' 'div'
     || lk == 62546                 // 'attribute' 'else'
     || lk == 62585                 // 'element' 'else'
     || lk == 62648                 // 'namespace' 'else'
     || lk == 62680                 // 'processing-instruction' 'else'
     || lk == 63058                 // 'attribute' 'empty'
     || lk == 63097                 // 'element' 'empty'
     || lk == 63160                 // 'namespace' 'empty'
     || lk == 63192                 // 'processing-instruction' 'empty'
     || lk == 64594                 // 'attribute' 'end'
     || lk == 64633                 // 'element' 'end'
     || lk == 64696                 // 'namespace' 'end'
     || lk == 64728                 // 'processing-instruction' 'end'
     || lk == 65618                 // 'attribute' 'eq'
     || lk == 65657                 // 'element' 'eq'
     || lk == 65720                 // 'namespace' 'eq'
     || lk == 65752                 // 'processing-instruction' 'eq'
     || lk == 67154                 // 'attribute' 'except'
     || lk == 67193                 // 'element' 'except'
     || lk == 67256                 // 'namespace' 'except'
     || lk == 67288                 // 'processing-instruction' 'except'
     || lk == 70226                 // 'attribute' 'for'
     || lk == 70265                 // 'element' 'for'
     || lk == 70328                 // 'namespace' 'for'
     || lk == 70360                 // 'processing-instruction' 'for'
     || lk == 74834                 // 'attribute' 'ge'
     || lk == 74873                 // 'element' 'ge'
     || lk == 74936                 // 'namespace' 'ge'
     || lk == 74968                 // 'processing-instruction' 'ge'
     || lk == 75858                 // 'attribute' 'group'
     || lk == 75897                 // 'element' 'group'
     || lk == 75960                 // 'namespace' 'group'
     || lk == 75992                 // 'processing-instruction' 'group'
     || lk == 76882                 // 'attribute' 'gt'
     || lk == 76921                 // 'element' 'gt'
     || lk == 76984                 // 'namespace' 'gt'
     || lk == 77016                 // 'processing-instruction' 'gt'
     || lk == 77394                 // 'attribute' 'idiv'
     || lk == 77433                 // 'element' 'idiv'
     || lk == 77496                 // 'namespace' 'idiv'
     || lk == 77528                 // 'processing-instruction' 'idiv'
     || lk == 82002                 // 'attribute' 'instance'
     || lk == 82041                 // 'element' 'instance'
     || lk == 82104                 // 'namespace' 'instance'
     || lk == 82136                 // 'processing-instruction' 'instance'
     || lk == 83026                 // 'attribute' 'intersect'
     || lk == 83065                 // 'element' 'intersect'
     || lk == 83128                 // 'namespace' 'intersect'
     || lk == 83160                 // 'processing-instruction' 'intersect'
     || lk == 83538                 // 'attribute' 'into'
     || lk == 83577                 // 'element' 'into'
     || lk == 83640                 // 'namespace' 'into'
     || lk == 83672                 // 'processing-instruction' 'into'
     || lk == 84050                 // 'attribute' 'is'
     || lk == 84089                 // 'element' 'is'
     || lk == 84152                 // 'namespace' 'is'
     || lk == 84184                 // 'processing-instruction' 'is'
     || lk == 88146                 // 'attribute' 'le'
     || lk == 88185                 // 'element' 'le'
     || lk == 88248                 // 'namespace' 'le'
     || lk == 88280                 // 'processing-instruction' 'le'
     || lk == 89170                 // 'attribute' 'let'
     || lk == 89209                 // 'element' 'let'
     || lk == 89272                 // 'namespace' 'let'
     || lk == 89304                 // 'processing-instruction' 'let'
     || lk == 91218                 // 'attribute' 'lt'
     || lk == 91257                 // 'element' 'lt'
     || lk == 91320                 // 'namespace' 'lt'
     || lk == 91352                 // 'processing-instruction' 'lt'
     || lk == 92242                 // 'attribute' 'mod'
     || lk == 92281                 // 'element' 'mod'
     || lk == 92344                 // 'namespace' 'mod'
     || lk == 92376                 // 'processing-instruction' 'mod'
     || lk == 92754                 // 'attribute' 'modify'
     || lk == 92793                 // 'element' 'modify'
     || lk == 92856                 // 'namespace' 'modify'
     || lk == 92888                 // 'processing-instruction' 'modify'
     || lk == 95314                 // 'attribute' 'ne'
     || lk == 95353                 // 'element' 'ne'
     || lk == 95416                 // 'namespace' 'ne'
     || lk == 95448                 // 'processing-instruction' 'ne'
     || lk == 101458                // 'attribute' 'only'
     || lk == 101497                // 'element' 'only'
     || lk == 101560                // 'namespace' 'only'
     || lk == 101592                // 'processing-instruction' 'only'
     || lk == 102482                // 'attribute' 'or'
     || lk == 102521                // 'element' 'or'
     || lk == 102584                // 'namespace' 'or'
     || lk == 102616                // 'processing-instruction' 'or'
     || lk == 102994                // 'attribute' 'order'
     || lk == 103033                // 'element' 'order'
     || lk == 103096                // 'namespace' 'order'
     || lk == 103128                // 'processing-instruction' 'order'
     || lk == 112722                // 'attribute' 'return'
     || lk == 112761                // 'element' 'return'
     || lk == 112824                // 'namespace' 'return'
     || lk == 112856                // 'processing-instruction' 'return'
     || lk == 114770                // 'attribute' 'satisfies'
     || lk == 114809                // 'element' 'satisfies'
     || lk == 114872                // 'namespace' 'satisfies'
     || lk == 114904                // 'processing-instruction' 'satisfies'
     || lk == 120914                // 'attribute' 'stable'
     || lk == 120953                // 'element' 'stable'
     || lk == 121016                // 'namespace' 'stable'
     || lk == 121048                // 'processing-instruction' 'stable'
     || lk == 121426                // 'attribute' 'start'
     || lk == 121465                // 'element' 'start'
     || lk == 121528                // 'namespace' 'start'
     || lk == 121560                // 'processing-instruction' 'start'
     || lk == 127058                // 'attribute' 'to'
     || lk == 127097                // 'element' 'to'
     || lk == 127160                // 'namespace' 'to'
     || lk == 127192                // 'processing-instruction' 'to'
     || lk == 127570                // 'attribute' 'treat'
     || lk == 127609                // 'element' 'treat'
     || lk == 127672                // 'namespace' 'treat'
     || lk == 127704                // 'processing-instruction' 'treat'
     || lk == 130130                // 'attribute' 'union'
     || lk == 130169                // 'element' 'union'
     || lk == 130232                // 'namespace' 'union'
     || lk == 130264                // 'processing-instruction' 'union'
     || lk == 136274                // 'attribute' 'where'
     || lk == 136313                // 'element' 'where'
     || lk == 136376                // 'namespace' 'where'
     || lk == 136408                // 'processing-instruction' 'where'
     || lk == 138322                // 'attribute' 'with'
     || lk == 138361                // 'element' 'with'
     || lk == 138424                // 'namespace' 'with'
     || lk == 138456)               // 'processing-instruction' 'with'
    {
      lk = memoized(4, e0);
      if (lk == 0)
      {
        var b0A = b0; var e0A = e0; var l1A = l1;
        var b1A = b1; var e1A = e1; var l2A = l2;
        var b2A = b2; var e2A = e2;
        try
        {
          try_PostfixExpr();
          memoize(4, e0A, -1);
          lk = -3;
        }
        catch (p1A)
        {
          lk = -2;
          b0 = b0A; e0 = e0A; l1 = l1A; if (l1 == 0) {end = e0A;} else {
          b1 = b1A; e1 = e1A; l2 = l2A; if (l2 == 0) {end = e1A;} else {
          b2 = b2A; e2 = e2A; end = e2A; }}
          memoize(4, e0A, -2);
        }
      }
    }
    switch (lk)
    {
    case -1:
    case 8:                         // IntegerLiteral
    case 9:                         // DecimalLiteral
    case 10:                        // DoubleLiteral
    case 11:                        // StringLiteral
    case 31:                        // '$'
    case 32:                        // '%'
    case 34:                        // '('
    case 44:                        // '.'
    case 54:                        // '<'
    case 55:                        // '<!--'
    case 59:                        // '<?'
    case 68:                        // '['
    case 276:                       // '{'
    case 278:                       // '{|'
    case 3154:                      // 'attribute' EQName^Token
    case 3193:                      // 'element' EQName^Token
    case 9912:                      // 'namespace' NCName^Token
    case 9944:                      // 'processing-instruction' NCName^Token
    case 14854:                     // EQName^Token '#'
    case 14918:                     // 'after' '#'
    case 14920:                     // 'allowing' '#'
    case 14921:                     // 'ancestor' '#'
    case 14922:                     // 'ancestor-or-self' '#'
    case 14923:                     // 'and' '#'
    case 14925:                     // 'append' '#'
    case 14926:                     // 'array' '#'
    case 14927:                     // 'as' '#'
    case 14928:                     // 'ascending' '#'
    case 14929:                     // 'at' '#'
    case 14930:                     // 'attribute' '#'
    case 14931:                     // 'base-uri' '#'
    case 14932:                     // 'before' '#'
    case 14933:                     // 'boundary-space' '#'
    case 14934:                     // 'break' '#'
    case 14936:                     // 'case' '#'
    case 14937:                     // 'cast' '#'
    case 14938:                     // 'castable' '#'
    case 14939:                     // 'catch' '#'
    case 14941:                     // 'child' '#'
    case 14942:                     // 'collation' '#'
    case 14944:                     // 'comment' '#'
    case 14945:                     // 'constraint' '#'
    case 14946:                     // 'construction' '#'
    case 14949:                     // 'context' '#'
    case 14950:                     // 'continue' '#'
    case 14951:                     // 'copy' '#'
    case 14952:                     // 'copy-namespaces' '#'
    case 14953:                     // 'count' '#'
    case 14954:                     // 'decimal-format' '#'
    case 14956:                     // 'declare' '#'
    case 14957:                     // 'default' '#'
    case 14958:                     // 'delete' '#'
    case 14959:                     // 'descendant' '#'
    case 14960:                     // 'descendant-or-self' '#'
    case 14961:                     // 'descending' '#'
    case 14966:                     // 'div' '#'
    case 14967:                     // 'document' '#'
    case 14968:                     // 'document-node' '#'
    case 14969:                     // 'element' '#'
    case 14970:                     // 'else' '#'
    case 14971:                     // 'empty' '#'
    case 14972:                     // 'empty-sequence' '#'
    case 14973:                     // 'encoding' '#'
    case 14974:                     // 'end' '#'
    case 14976:                     // 'eq' '#'
    case 14977:                     // 'every' '#'
    case 14979:                     // 'except' '#'
    case 14980:                     // 'exit' '#'
    case 14981:                     // 'external' '#'
    case 14982:                     // 'first' '#'
    case 14983:                     // 'following' '#'
    case 14984:                     // 'following-sibling' '#'
    case 14985:                     // 'for' '#'
    case 14989:                     // 'ft-option' '#'
    case 14993:                     // 'function' '#'
    case 14994:                     // 'ge' '#'
    case 14996:                     // 'group' '#'
    case 14998:                     // 'gt' '#'
    case 14999:                     // 'idiv' '#'
    case 15000:                     // 'if' '#'
    case 15001:                     // 'import' '#'
    case 15002:                     // 'in' '#'
    case 15003:                     // 'index' '#'
    case 15007:                     // 'insert' '#'
    case 15008:                     // 'instance' '#'
    case 15009:                     // 'integrity' '#'
    case 15010:                     // 'intersect' '#'
    case 15011:                     // 'into' '#'
    case 15012:                     // 'is' '#'
    case 15013:                     // 'item' '#'
    case 15014:                     // 'json' '#'
    case 15015:                     // 'json-item' '#'
    case 15018:                     // 'last' '#'
    case 15019:                     // 'lax' '#'
    case 15020:                     // 'le' '#'
    case 15022:                     // 'let' '#'
    case 15024:                     // 'loop' '#'
    case 15026:                     // 'lt' '#'
    case 15028:                     // 'mod' '#'
    case 15029:                     // 'modify' '#'
    case 15030:                     // 'module' '#'
    case 15032:                     // 'namespace' '#'
    case 15033:                     // 'namespace-node' '#'
    case 15034:                     // 'ne' '#'
    case 15039:                     // 'node' '#'
    case 15040:                     // 'nodes' '#'
    case 15042:                     // 'object' '#'
    case 15046:                     // 'only' '#'
    case 15047:                     // 'option' '#'
    case 15048:                     // 'or' '#'
    case 15049:                     // 'order' '#'
    case 15050:                     // 'ordered' '#'
    case 15051:                     // 'ordering' '#'
    case 15054:                     // 'parent' '#'
    case 15060:                     // 'preceding' '#'
    case 15061:                     // 'preceding-sibling' '#'
    case 15064:                     // 'processing-instruction' '#'
    case 15066:                     // 'rename' '#'
    case 15067:                     // 'replace' '#'
    case 15068:                     // 'return' '#'
    case 15069:                     // 'returning' '#'
    case 15070:                     // 'revalidation' '#'
    case 15072:                     // 'satisfies' '#'
    case 15073:                     // 'schema' '#'
    case 15074:                     // 'schema-attribute' '#'
    case 15075:                     // 'schema-element' '#'
    case 15076:                     // 'score' '#'
    case 15077:                     // 'self' '#'
    case 15082:                     // 'sliding' '#'
    case 15083:                     // 'some' '#'
    case 15084:                     // 'stable' '#'
    case 15085:                     // 'start' '#'
    case 15088:                     // 'strict' '#'
    case 15090:                     // 'structured-item' '#'
    case 15091:                     // 'switch' '#'
    case 15092:                     // 'text' '#'
    case 15096:                     // 'to' '#'
    case 15097:                     // 'treat' '#'
    case 15098:                     // 'try' '#'
    case 15099:                     // 'tumbling' '#'
    case 15100:                     // 'type' '#'
    case 15101:                     // 'typeswitch' '#'
    case 15102:                     // 'union' '#'
    case 15104:                     // 'unordered' '#'
    case 15105:                     // 'updating' '#'
    case 15108:                     // 'validate' '#'
    case 15109:                     // 'value' '#'
    case 15110:                     // 'variable' '#'
    case 15111:                     // 'version' '#'
    case 15114:                     // 'where' '#'
    case 15115:                     // 'while' '#'
    case 15118:                     // 'with' '#'
    case 15122:                     // 'xquery' '#'
    case 17414:                     // EQName^Token '('
    case 17478:                     // 'after' '('
    case 17480:                     // 'allowing' '('
    case 17481:                     // 'ancestor' '('
    case 17482:                     // 'ancestor-or-self' '('
    case 17483:                     // 'and' '('
    case 17485:                     // 'append' '('
    case 17487:                     // 'as' '('
    case 17488:                     // 'ascending' '('
    case 17489:                     // 'at' '('
    case 17491:                     // 'base-uri' '('
    case 17492:                     // 'before' '('
    case 17493:                     // 'boundary-space' '('
    case 17494:                     // 'break' '('
    case 17496:                     // 'case' '('
    case 17497:                     // 'cast' '('
    case 17498:                     // 'castable' '('
    case 17499:                     // 'catch' '('
    case 17501:                     // 'child' '('
    case 17502:                     // 'collation' '('
    case 17505:                     // 'constraint' '('
    case 17506:                     // 'construction' '('
    case 17509:                     // 'context' '('
    case 17510:                     // 'continue' '('
    case 17511:                     // 'copy' '('
    case 17512:                     // 'copy-namespaces' '('
    case 17513:                     // 'count' '('
    case 17514:                     // 'decimal-format' '('
    case 17516:                     // 'declare' '('
    case 17517:                     // 'default' '('
    case 17518:                     // 'delete' '('
    case 17519:                     // 'descendant' '('
    case 17520:                     // 'descendant-or-self' '('
    case 17521:                     // 'descending' '('
    case 17526:                     // 'div' '('
    case 17527:                     // 'document' '('
    case 17530:                     // 'else' '('
    case 17531:                     // 'empty' '('
    case 17533:                     // 'encoding' '('
    case 17534:                     // 'end' '('
    case 17536:                     // 'eq' '('
    case 17537:                     // 'every' '('
    case 17539:                     // 'except' '('
    case 17540:                     // 'exit' '('
    case 17541:                     // 'external' '('
    case 17542:                     // 'first' '('
    case 17543:                     // 'following' '('
    case 17544:                     // 'following-sibling' '('
    case 17545:                     // 'for' '('
    case 17549:                     // 'ft-option' '('
    case 17553:                     // 'function' '('
    case 17554:                     // 'ge' '('
    case 17556:                     // 'group' '('
    case 17558:                     // 'gt' '('
    case 17559:                     // 'idiv' '('
    case 17561:                     // 'import' '('
    case 17562:                     // 'in' '('
    case 17563:                     // 'index' '('
    case 17567:                     // 'insert' '('
    case 17568:                     // 'instance' '('
    case 17569:                     // 'integrity' '('
    case 17570:                     // 'intersect' '('
    case 17571:                     // 'into' '('
    case 17572:                     // 'is' '('
    case 17574:                     // 'json' '('
    case 17578:                     // 'last' '('
    case 17579:                     // 'lax' '('
    case 17580:                     // 'le' '('
    case 17582:                     // 'let' '('
    case 17584:                     // 'loop' '('
    case 17586:                     // 'lt' '('
    case 17588:                     // 'mod' '('
    case 17589:                     // 'modify' '('
    case 17590:                     // 'module' '('
    case 17592:                     // 'namespace' '('
    case 17594:                     // 'ne' '('
    case 17600:                     // 'nodes' '('
    case 17602:                     // 'object' '('
    case 17606:                     // 'only' '('
    case 17607:                     // 'option' '('
    case 17608:                     // 'or' '('
    case 17609:                     // 'order' '('
    case 17610:                     // 'ordered' '('
    case 17611:                     // 'ordering' '('
    case 17614:                     // 'parent' '('
    case 17620:                     // 'preceding' '('
    case 17621:                     // 'preceding-sibling' '('
    case 17626:                     // 'rename' '('
    case 17627:                     // 'replace' '('
    case 17628:                     // 'return' '('
    case 17629:                     // 'returning' '('
    case 17630:                     // 'revalidation' '('
    case 17632:                     // 'satisfies' '('
    case 17633:                     // 'schema' '('
    case 17636:                     // 'score' '('
    case 17637:                     // 'self' '('
    case 17642:                     // 'sliding' '('
    case 17643:                     // 'some' '('
    case 17644:                     // 'stable' '('
    case 17645:                     // 'start' '('
    case 17648:                     // 'strict' '('
    case 17656:                     // 'to' '('
    case 17657:                     // 'treat' '('
    case 17658:                     // 'try' '('
    case 17659:                     // 'tumbling' '('
    case 17660:                     // 'type' '('
    case 17662:                     // 'union' '('
    case 17664:                     // 'unordered' '('
    case 17665:                     // 'updating' '('
    case 17668:                     // 'validate' '('
    case 17669:                     // 'value' '('
    case 17670:                     // 'variable' '('
    case 17671:                     // 'version' '('
    case 17674:                     // 'where' '('
    case 17675:                     // 'while' '('
    case 17678:                     // 'with' '('
    case 17682:                     // 'xquery' '('
    case 36946:                     // 'attribute' 'allowing'
    case 36985:                     // 'element' 'allowing'
    case 37048:                     // 'namespace' 'allowing'
    case 37080:                     // 'processing-instruction' 'allowing'
    case 37458:                     // 'attribute' 'ancestor'
    case 37497:                     // 'element' 'ancestor'
    case 37560:                     // 'namespace' 'ancestor'
    case 37592:                     // 'processing-instruction' 'ancestor'
    case 37970:                     // 'attribute' 'ancestor-or-self'
    case 38009:                     // 'element' 'ancestor-or-self'
    case 38072:                     // 'namespace' 'ancestor-or-self'
    case 38104:                     // 'processing-instruction' 'ancestor-or-self'
    case 39506:                     // 'attribute' 'append'
    case 39545:                     // 'element' 'append'
    case 39608:                     // 'namespace' 'append'
    case 39640:                     // 'processing-instruction' 'append'
    case 40018:                     // 'attribute' 'array'
    case 40057:                     // 'element' 'array'
    case 42066:                     // 'attribute' 'attribute'
    case 42105:                     // 'element' 'attribute'
    case 42168:                     // 'namespace' 'attribute'
    case 42200:                     // 'processing-instruction' 'attribute'
    case 42578:                     // 'attribute' 'base-uri'
    case 42617:                     // 'element' 'base-uri'
    case 42680:                     // 'namespace' 'base-uri'
    case 42712:                     // 'processing-instruction' 'base-uri'
    case 43602:                     // 'attribute' 'boundary-space'
    case 43641:                     // 'element' 'boundary-space'
    case 43704:                     // 'namespace' 'boundary-space'
    case 43736:                     // 'processing-instruction' 'boundary-space'
    case 44114:                     // 'attribute' 'break'
    case 44153:                     // 'element' 'break'
    case 44216:                     // 'namespace' 'break'
    case 44248:                     // 'processing-instruction' 'break'
    case 46674:                     // 'attribute' 'catch'
    case 46713:                     // 'element' 'catch'
    case 46776:                     // 'namespace' 'catch'
    case 46808:                     // 'processing-instruction' 'catch'
    case 47698:                     // 'attribute' 'child'
    case 47737:                     // 'element' 'child'
    case 47800:                     // 'namespace' 'child'
    case 47832:                     // 'processing-instruction' 'child'
    case 49234:                     // 'attribute' 'comment'
    case 49273:                     // 'element' 'comment'
    case 49336:                     // 'namespace' 'comment'
    case 49368:                     // 'processing-instruction' 'comment'
    case 49746:                     // 'attribute' 'constraint'
    case 49785:                     // 'element' 'constraint'
    case 49848:                     // 'namespace' 'constraint'
    case 49880:                     // 'processing-instruction' 'constraint'
    case 50258:                     // 'attribute' 'construction'
    case 50297:                     // 'element' 'construction'
    case 50360:                     // 'namespace' 'construction'
    case 50392:                     // 'processing-instruction' 'construction'
    case 51794:                     // 'attribute' 'context'
    case 51833:                     // 'element' 'context'
    case 51896:                     // 'namespace' 'context'
    case 51928:                     // 'processing-instruction' 'context'
    case 52306:                     // 'attribute' 'continue'
    case 52345:                     // 'element' 'continue'
    case 52408:                     // 'namespace' 'continue'
    case 52440:                     // 'processing-instruction' 'continue'
    case 52818:                     // 'attribute' 'copy'
    case 52857:                     // 'element' 'copy'
    case 52920:                     // 'namespace' 'copy'
    case 52952:                     // 'processing-instruction' 'copy'
    case 53330:                     // 'attribute' 'copy-namespaces'
    case 53369:                     // 'element' 'copy-namespaces'
    case 53432:                     // 'namespace' 'copy-namespaces'
    case 53464:                     // 'processing-instruction' 'copy-namespaces'
    case 54354:                     // 'attribute' 'decimal-format'
    case 54393:                     // 'element' 'decimal-format'
    case 54456:                     // 'namespace' 'decimal-format'
    case 54488:                     // 'processing-instruction' 'decimal-format'
    case 55378:                     // 'attribute' 'declare'
    case 55417:                     // 'element' 'declare'
    case 55480:                     // 'namespace' 'declare'
    case 55512:                     // 'processing-instruction' 'declare'
    case 56402:                     // 'attribute' 'delete'
    case 56441:                     // 'element' 'delete'
    case 56504:                     // 'namespace' 'delete'
    case 56536:                     // 'processing-instruction' 'delete'
    case 56914:                     // 'attribute' 'descendant'
    case 56953:                     // 'element' 'descendant'
    case 57016:                     // 'namespace' 'descendant'
    case 57048:                     // 'processing-instruction' 'descendant'
    case 57426:                     // 'attribute' 'descendant-or-self'
    case 57465:                     // 'element' 'descendant-or-self'
    case 57528:                     // 'namespace' 'descendant-or-self'
    case 57560:                     // 'processing-instruction' 'descendant-or-self'
    case 61010:                     // 'attribute' 'document'
    case 61049:                     // 'element' 'document'
    case 61112:                     // 'namespace' 'document'
    case 61144:                     // 'processing-instruction' 'document'
    case 61522:                     // 'attribute' 'document-node'
    case 61561:                     // 'element' 'document-node'
    case 61624:                     // 'namespace' 'document-node'
    case 61656:                     // 'processing-instruction' 'document-node'
    case 62034:                     // 'attribute' 'element'
    case 62073:                     // 'element' 'element'
    case 62136:                     // 'namespace' 'element'
    case 62168:                     // 'processing-instruction' 'element'
    case 63570:                     // 'attribute' 'empty-sequence'
    case 63609:                     // 'element' 'empty-sequence'
    case 63672:                     // 'namespace' 'empty-sequence'
    case 63704:                     // 'processing-instruction' 'empty-sequence'
    case 64082:                     // 'attribute' 'encoding'
    case 64121:                     // 'element' 'encoding'
    case 64184:                     // 'namespace' 'encoding'
    case 64216:                     // 'processing-instruction' 'encoding'
    case 66130:                     // 'attribute' 'every'
    case 66169:                     // 'element' 'every'
    case 66232:                     // 'namespace' 'every'
    case 66264:                     // 'processing-instruction' 'every'
    case 67666:                     // 'attribute' 'exit'
    case 67705:                     // 'element' 'exit'
    case 67768:                     // 'namespace' 'exit'
    case 67800:                     // 'processing-instruction' 'exit'
    case 68178:                     // 'attribute' 'external'
    case 68217:                     // 'element' 'external'
    case 68280:                     // 'namespace' 'external'
    case 68312:                     // 'processing-instruction' 'external'
    case 68690:                     // 'attribute' 'first'
    case 68729:                     // 'element' 'first'
    case 68792:                     // 'namespace' 'first'
    case 68824:                     // 'processing-instruction' 'first'
    case 69202:                     // 'attribute' 'following'
    case 69241:                     // 'element' 'following'
    case 69304:                     // 'namespace' 'following'
    case 69336:                     // 'processing-instruction' 'following'
    case 69714:                     // 'attribute' 'following-sibling'
    case 69753:                     // 'element' 'following-sibling'
    case 69816:                     // 'namespace' 'following-sibling'
    case 69848:                     // 'processing-instruction' 'following-sibling'
    case 72274:                     // 'attribute' 'ft-option'
    case 72313:                     // 'element' 'ft-option'
    case 72376:                     // 'namespace' 'ft-option'
    case 72408:                     // 'processing-instruction' 'ft-option'
    case 74322:                     // 'attribute' 'function'
    case 74361:                     // 'element' 'function'
    case 74424:                     // 'namespace' 'function'
    case 74456:                     // 'processing-instruction' 'function'
    case 77906:                     // 'attribute' 'if'
    case 77945:                     // 'element' 'if'
    case 78008:                     // 'namespace' 'if'
    case 78040:                     // 'processing-instruction' 'if'
    case 78418:                     // 'attribute' 'import'
    case 78457:                     // 'element' 'import'
    case 78520:                     // 'namespace' 'import'
    case 78552:                     // 'processing-instruction' 'import'
    case 78930:                     // 'attribute' 'in'
    case 78969:                     // 'element' 'in'
    case 79032:                     // 'namespace' 'in'
    case 79064:                     // 'processing-instruction' 'in'
    case 79442:                     // 'attribute' 'index'
    case 79481:                     // 'element' 'index'
    case 79544:                     // 'namespace' 'index'
    case 79576:                     // 'processing-instruction' 'index'
    case 81490:                     // 'attribute' 'insert'
    case 81529:                     // 'element' 'insert'
    case 81592:                     // 'namespace' 'insert'
    case 81624:                     // 'processing-instruction' 'insert'
    case 82514:                     // 'attribute' 'integrity'
    case 82553:                     // 'element' 'integrity'
    case 82616:                     // 'namespace' 'integrity'
    case 82648:                     // 'processing-instruction' 'integrity'
    case 84562:                     // 'attribute' 'item'
    case 84601:                     // 'element' 'item'
    case 84664:                     // 'namespace' 'item'
    case 84696:                     // 'processing-instruction' 'item'
    case 85074:                     // 'attribute' 'json'
    case 85113:                     // 'element' 'json'
    case 85176:                     // 'namespace' 'json'
    case 85208:                     // 'processing-instruction' 'json'
    case 85586:                     // 'attribute' 'json-item'
    case 85625:                     // 'element' 'json-item'
    case 87122:                     // 'attribute' 'last'
    case 87161:                     // 'element' 'last'
    case 87224:                     // 'namespace' 'last'
    case 87256:                     // 'processing-instruction' 'last'
    case 87634:                     // 'attribute' 'lax'
    case 87673:                     // 'element' 'lax'
    case 87736:                     // 'namespace' 'lax'
    case 87768:                     // 'processing-instruction' 'lax'
    case 90194:                     // 'attribute' 'loop'
    case 90233:                     // 'element' 'loop'
    case 90296:                     // 'namespace' 'loop'
    case 90328:                     // 'processing-instruction' 'loop'
    case 93266:                     // 'attribute' 'module'
    case 93305:                     // 'element' 'module'
    case 93368:                     // 'namespace' 'module'
    case 93400:                     // 'processing-instruction' 'module'
    case 94290:                     // 'attribute' 'namespace'
    case 94329:                     // 'element' 'namespace'
    case 94392:                     // 'namespace' 'namespace'
    case 94424:                     // 'processing-instruction' 'namespace'
    case 94802:                     // 'attribute' 'namespace-node'
    case 94841:                     // 'element' 'namespace-node'
    case 94904:                     // 'namespace' 'namespace-node'
    case 94936:                     // 'processing-instruction' 'namespace-node'
    case 97874:                     // 'attribute' 'node'
    case 97913:                     // 'element' 'node'
    case 97976:                     // 'namespace' 'node'
    case 98008:                     // 'processing-instruction' 'node'
    case 98386:                     // 'attribute' 'nodes'
    case 98425:                     // 'element' 'nodes'
    case 98488:                     // 'namespace' 'nodes'
    case 98520:                     // 'processing-instruction' 'nodes'
    case 99410:                     // 'attribute' 'object'
    case 99449:                     // 'element' 'object'
    case 99512:                     // 'namespace' 'object'
    case 99544:                     // 'processing-instruction' 'object'
    case 101970:                    // 'attribute' 'option'
    case 102009:                    // 'element' 'option'
    case 102072:                    // 'namespace' 'option'
    case 102104:                    // 'processing-instruction' 'option'
    case 103506:                    // 'attribute' 'ordered'
    case 103545:                    // 'element' 'ordered'
    case 103608:                    // 'namespace' 'ordered'
    case 103640:                    // 'processing-instruction' 'ordered'
    case 104018:                    // 'attribute' 'ordering'
    case 104057:                    // 'element' 'ordering'
    case 104120:                    // 'namespace' 'ordering'
    case 104152:                    // 'processing-instruction' 'ordering'
    case 105554:                    // 'attribute' 'parent'
    case 105593:                    // 'element' 'parent'
    case 105656:                    // 'namespace' 'parent'
    case 105688:                    // 'processing-instruction' 'parent'
    case 108626:                    // 'attribute' 'preceding'
    case 108665:                    // 'element' 'preceding'
    case 108728:                    // 'namespace' 'preceding'
    case 108760:                    // 'processing-instruction' 'preceding'
    case 109138:                    // 'attribute' 'preceding-sibling'
    case 109177:                    // 'element' 'preceding-sibling'
    case 109240:                    // 'namespace' 'preceding-sibling'
    case 109272:                    // 'processing-instruction' 'preceding-sibling'
    case 110674:                    // 'attribute' 'processing-instruction'
    case 110713:                    // 'element' 'processing-instruction'
    case 110776:                    // 'namespace' 'processing-instruction'
    case 110808:                    // 'processing-instruction' 'processing-instruction'
    case 111698:                    // 'attribute' 'rename'
    case 111737:                    // 'element' 'rename'
    case 111800:                    // 'namespace' 'rename'
    case 111832:                    // 'processing-instruction' 'rename'
    case 112210:                    // 'attribute' 'replace'
    case 112249:                    // 'element' 'replace'
    case 112312:                    // 'namespace' 'replace'
    case 112344:                    // 'processing-instruction' 'replace'
    case 113234:                    // 'attribute' 'returning'
    case 113273:                    // 'element' 'returning'
    case 113336:                    // 'namespace' 'returning'
    case 113368:                    // 'processing-instruction' 'returning'
    case 113746:                    // 'attribute' 'revalidation'
    case 113785:                    // 'element' 'revalidation'
    case 113848:                    // 'namespace' 'revalidation'
    case 113880:                    // 'processing-instruction' 'revalidation'
    case 115282:                    // 'attribute' 'schema'
    case 115321:                    // 'element' 'schema'
    case 115384:                    // 'namespace' 'schema'
    case 115416:                    // 'processing-instruction' 'schema'
    case 115794:                    // 'attribute' 'schema-attribute'
    case 115833:                    // 'element' 'schema-attribute'
    case 115896:                    // 'namespace' 'schema-attribute'
    case 115928:                    // 'processing-instruction' 'schema-attribute'
    case 116306:                    // 'attribute' 'schema-element'
    case 116345:                    // 'element' 'schema-element'
    case 116408:                    // 'namespace' 'schema-element'
    case 116440:                    // 'processing-instruction' 'schema-element'
    case 116818:                    // 'attribute' 'score'
    case 116857:                    // 'element' 'score'
    case 116920:                    // 'namespace' 'score'
    case 116952:                    // 'processing-instruction' 'score'
    case 117330:                    // 'attribute' 'self'
    case 117369:                    // 'element' 'self'
    case 117432:                    // 'namespace' 'self'
    case 117464:                    // 'processing-instruction' 'self'
    case 119890:                    // 'attribute' 'sliding'
    case 119929:                    // 'element' 'sliding'
    case 119992:                    // 'namespace' 'sliding'
    case 120024:                    // 'processing-instruction' 'sliding'
    case 120402:                    // 'attribute' 'some'
    case 120441:                    // 'element' 'some'
    case 120504:                    // 'namespace' 'some'
    case 120536:                    // 'processing-instruction' 'some'
    case 122962:                    // 'attribute' 'strict'
    case 123001:                    // 'element' 'strict'
    case 123064:                    // 'namespace' 'strict'
    case 123096:                    // 'processing-instruction' 'strict'
    case 123986:                    // 'attribute' 'structured-item'
    case 124025:                    // 'element' 'structured-item'
    case 124498:                    // 'attribute' 'switch'
    case 124537:                    // 'element' 'switch'
    case 124600:                    // 'namespace' 'switch'
    case 124632:                    // 'processing-instruction' 'switch'
    case 125010:                    // 'attribute' 'text'
    case 125049:                    // 'element' 'text'
    case 125112:                    // 'namespace' 'text'
    case 125144:                    // 'processing-instruction' 'text'
    case 128082:                    // 'attribute' 'try'
    case 128121:                    // 'element' 'try'
    case 128184:                    // 'namespace' 'try'
    case 128216:                    // 'processing-instruction' 'try'
    case 128594:                    // 'attribute' 'tumbling'
    case 128633:                    // 'element' 'tumbling'
    case 128696:                    // 'namespace' 'tumbling'
    case 128728:                    // 'processing-instruction' 'tumbling'
    case 129106:                    // 'attribute' 'type'
    case 129145:                    // 'element' 'type'
    case 129208:                    // 'namespace' 'type'
    case 129240:                    // 'processing-instruction' 'type'
    case 129618:                    // 'attribute' 'typeswitch'
    case 129657:                    // 'element' 'typeswitch'
    case 129720:                    // 'namespace' 'typeswitch'
    case 129752:                    // 'processing-instruction' 'typeswitch'
    case 131154:                    // 'attribute' 'unordered'
    case 131193:                    // 'element' 'unordered'
    case 131256:                    // 'namespace' 'unordered'
    case 131288:                    // 'processing-instruction' 'unordered'
    case 131666:                    // 'attribute' 'updating'
    case 131705:                    // 'element' 'updating'
    case 131768:                    // 'namespace' 'updating'
    case 131800:                    // 'processing-instruction' 'updating'
    case 133202:                    // 'attribute' 'validate'
    case 133241:                    // 'element' 'validate'
    case 133304:                    // 'namespace' 'validate'
    case 133336:                    // 'processing-instruction' 'validate'
    case 133714:                    // 'attribute' 'value'
    case 133753:                    // 'element' 'value'
    case 133816:                    // 'namespace' 'value'
    case 133848:                    // 'processing-instruction' 'value'
    case 134226:                    // 'attribute' 'variable'
    case 134265:                    // 'element' 'variable'
    case 134328:                    // 'namespace' 'variable'
    case 134360:                    // 'processing-instruction' 'variable'
    case 134738:                    // 'attribute' 'version'
    case 134777:                    // 'element' 'version'
    case 134840:                    // 'namespace' 'version'
    case 134872:                    // 'processing-instruction' 'version'
    case 136786:                    // 'attribute' 'while'
    case 136825:                    // 'element' 'while'
    case 136888:                    // 'namespace' 'while'
    case 136920:                    // 'processing-instruction' 'while'
    case 140370:                    // 'attribute' 'xquery'
    case 140409:                    // 'element' 'xquery'
    case 140472:                    // 'namespace' 'xquery'
    case 140504:                    // 'processing-instruction' 'xquery'
    case 141394:                    // 'attribute' '{'
    case 141408:                    // 'comment' '{'
    case 141431:                    // 'document' '{'
    case 141433:                    // 'element' '{'
    case 141496:                    // 'namespace' '{'
    case 141514:                    // 'ordered' '{'
    case 141528:                    // 'processing-instruction' '{'
    case 141556:                    // 'text' '{'
    case 141568:                    // 'unordered' '{'
      try_PostfixExpr();
      break;
    case -3:
      break;
    default:
      try_AxisStep();
    }
  }

  function parse_AxisStep()
  {
    eventHandler.startNonterminal("AxisStep", e0);
    switch (l1)
    {
    case 73:                        // 'ancestor'
    case 74:                        // 'ancestor-or-self'
    case 206:                       // 'parent'
    case 212:                       // 'preceding'
    case 213:                       // 'preceding-sibling'
      lookahead2W(242);             // S^WS | EOF | '!' | '!=' | '(:' | ')' | '*' | '+' | ',' | '-' | '/' | '//' | ':' |
      break;
    default:
      lk = l1;
    }
    switch (lk)
    {
    case 45:                        // '..'
    case 26185:                     // 'ancestor' '::'
    case 26186:                     // 'ancestor-or-self' '::'
    case 26318:                     // 'parent' '::'
    case 26324:                     // 'preceding' '::'
    case 26325:                     // 'preceding-sibling' '::'
      parse_ReverseStep();
      break;
    default:
      parse_ForwardStep();
    }
    lookahead1W(238);               // S^WS | EOF | '!' | '!=' | '(:' | ')' | '*' | '+' | ',' | '-' | '/' | '//' | ':' |
    whitespace();
    parse_PredicateList();
    eventHandler.endNonterminal("AxisStep", e0);
  }

  function try_AxisStep()
  {
    switch (l1)
    {
    case 73:                        // 'ancestor'
    case 74:                        // 'ancestor-or-self'
    case 206:                       // 'parent'
    case 212:                       // 'preceding'
    case 213:                       // 'preceding-sibling'
      lookahead2W(242);             // S^WS | EOF | '!' | '!=' | '(:' | ')' | '*' | '+' | ',' | '-' | '/' | '//' | ':' |
      break;
    default:
      lk = l1;
    }
    switch (lk)
    {
    case 45:                        // '..'
    case 26185:                     // 'ancestor' '::'
    case 26186:                     // 'ancestor-or-self' '::'
    case 26318:                     // 'parent' '::'
    case 26324:                     // 'preceding' '::'
    case 26325:                     // 'preceding-sibling' '::'
      try_ReverseStep();
      break;
    default:
      try_ForwardStep();
    }
    lookahead1W(238);               // S^WS | EOF | '!' | '!=' | '(:' | ')' | '*' | '+' | ',' | '-' | '/' | '//' | ':' |
    try_PredicateList();
  }

  function parse_ForwardStep()
  {
    eventHandler.startNonterminal("ForwardStep", e0);
    switch (l1)
    {
    case 82:                        // 'attribute'
      lookahead2W(245);             // S^WS | EOF | '!' | '!=' | '(' | '(:' | ')' | '*' | '+' | ',' | '-' | '/' | '//' |
      break;
    case 93:                        // 'child'
    case 111:                       // 'descendant'
    case 112:                       // 'descendant-or-self'
    case 135:                       // 'following'
    case 136:                       // 'following-sibling'
    case 229:                       // 'self'
      lookahead2W(242);             // S^WS | EOF | '!' | '!=' | '(:' | ')' | '*' | '+' | ',' | '-' | '/' | '//' | ':' |
      break;
    default:
      lk = l1;
    }
    switch (lk)
    {
    case 26194:                     // 'attribute' '::'
    case 26205:                     // 'child' '::'
    case 26223:                     // 'descendant' '::'
    case 26224:                     // 'descendant-or-self' '::'
    case 26247:                     // 'following' '::'
    case 26248:                     // 'following-sibling' '::'
    case 26341:                     // 'self' '::'
      parse_ForwardAxis();
      lookahead1W(257);             // Wildcard | EQName^Token | S^WS | '(:' | 'after' | 'allowing' | 'ancestor' |
      whitespace();
      parse_NodeTest();
      break;
    default:
      parse_AbbrevForwardStep();
    }
    eventHandler.endNonterminal("ForwardStep", e0);
  }

  function try_ForwardStep()
  {
    switch (l1)
    {
    case 82:                        // 'attribute'
      lookahead2W(245);             // S^WS | EOF | '!' | '!=' | '(' | '(:' | ')' | '*' | '+' | ',' | '-' | '/' | '//' |
      break;
    case 93:                        // 'child'
    case 111:                       // 'descendant'
    case 112:                       // 'descendant-or-self'
    case 135:                       // 'following'
    case 136:                       // 'following-sibling'
    case 229:                       // 'self'
      lookahead2W(242);             // S^WS | EOF | '!' | '!=' | '(:' | ')' | '*' | '+' | ',' | '-' | '/' | '//' | ':' |
      break;
    default:
      lk = l1;
    }
    switch (lk)
    {
    case 26194:                     // 'attribute' '::'
    case 26205:                     // 'child' '::'
    case 26223:                     // 'descendant' '::'
    case 26224:                     // 'descendant-or-self' '::'
    case 26247:                     // 'following' '::'
    case 26248:                     // 'following-sibling' '::'
    case 26341:                     // 'self' '::'
      try_ForwardAxis();
      lookahead1W(257);             // Wildcard | EQName^Token | S^WS | '(:' | 'after' | 'allowing' | 'ancestor' |
      try_NodeTest();
      break;
    default:
      try_AbbrevForwardStep();
    }
  }

  function parse_ForwardAxis()
  {
    eventHandler.startNonterminal("ForwardAxis", e0);
    switch (l1)
    {
    case 93:                        // 'child'
      shift(93);                    // 'child'
      lookahead1W(26);              // S^WS | '(:' | '::'
      shift(51);                    // '::'
      break;
    case 111:                       // 'descendant'
      shift(111);                   // 'descendant'
      lookahead1W(26);              // S^WS | '(:' | '::'
      shift(51);                    // '::'
      break;
    case 82:                        // 'attribute'
      shift(82);                    // 'attribute'
      lookahead1W(26);              // S^WS | '(:' | '::'
      shift(51);                    // '::'
      break;
    case 229:                       // 'self'
      shift(229);                   // 'self'
      lookahead1W(26);              // S^WS | '(:' | '::'
      shift(51);                    // '::'
      break;
    case 112:                       // 'descendant-or-self'
      shift(112);                   // 'descendant-or-self'
      lookahead1W(26);              // S^WS | '(:' | '::'
      shift(51);                    // '::'
      break;
    case 136:                       // 'following-sibling'
      shift(136);                   // 'following-sibling'
      lookahead1W(26);              // S^WS | '(:' | '::'
      shift(51);                    // '::'
      break;
    default:
      shift(135);                   // 'following'
      lookahead1W(26);              // S^WS | '(:' | '::'
      shift(51);                    // '::'
    }
    eventHandler.endNonterminal("ForwardAxis", e0);
  }

  function try_ForwardAxis()
  {
    switch (l1)
    {
    case 93:                        // 'child'
      shiftT(93);                   // 'child'
      lookahead1W(26);              // S^WS | '(:' | '::'
      shiftT(51);                   // '::'
      break;
    case 111:                       // 'descendant'
      shiftT(111);                  // 'descendant'
      lookahead1W(26);              // S^WS | '(:' | '::'
      shiftT(51);                   // '::'
      break;
    case 82:                        // 'attribute'
      shiftT(82);                   // 'attribute'
      lookahead1W(26);              // S^WS | '(:' | '::'
      shiftT(51);                   // '::'
      break;
    case 229:                       // 'self'
      shiftT(229);                  // 'self'
      lookahead1W(26);              // S^WS | '(:' | '::'
      shiftT(51);                   // '::'
      break;
    case 112:                       // 'descendant-or-self'
      shiftT(112);                  // 'descendant-or-self'
      lookahead1W(26);              // S^WS | '(:' | '::'
      shiftT(51);                   // '::'
      break;
    case 136:                       // 'following-sibling'
      shiftT(136);                  // 'following-sibling'
      lookahead1W(26);              // S^WS | '(:' | '::'
      shiftT(51);                   // '::'
      break;
    default:
      shiftT(135);                  // 'following'
      lookahead1W(26);              // S^WS | '(:' | '::'
      shiftT(51);                   // '::'
    }
  }

  function parse_AbbrevForwardStep()
  {
    eventHandler.startNonterminal("AbbrevForwardStep", e0);
    if (l1 == 66)                   // '@'
    {
      shift(66);                    // '@'
    }
    lookahead1W(257);               // Wildcard | EQName^Token | S^WS | '(:' | 'after' | 'allowing' | 'ancestor' |
    whitespace();
    parse_NodeTest();
    eventHandler.endNonterminal("AbbrevForwardStep", e0);
  }

  function try_AbbrevForwardStep()
  {
    if (l1 == 66)                   // '@'
    {
      shiftT(66);                   // '@'
    }
    lookahead1W(257);               // Wildcard | EQName^Token | S^WS | '(:' | 'after' | 'allowing' | 'ancestor' |
    try_NodeTest();
  }

  function parse_ReverseStep()
  {
    eventHandler.startNonterminal("ReverseStep", e0);
    switch (l1)
    {
    case 45:                        // '..'
      parse_AbbrevReverseStep();
      break;
    default:
      parse_ReverseAxis();
      lookahead1W(257);             // Wildcard | EQName^Token | S^WS | '(:' | 'after' | 'allowing' | 'ancestor' |
      whitespace();
      parse_NodeTest();
    }
    eventHandler.endNonterminal("ReverseStep", e0);
  }

  function try_ReverseStep()
  {
    switch (l1)
    {
    case 45:                        // '..'
      try_AbbrevReverseStep();
      break;
    default:
      try_ReverseAxis();
      lookahead1W(257);             // Wildcard | EQName^Token | S^WS | '(:' | 'after' | 'allowing' | 'ancestor' |
      try_NodeTest();
    }
  }

  function parse_ReverseAxis()
  {
    eventHandler.startNonterminal("ReverseAxis", e0);
    switch (l1)
    {
    case 206:                       // 'parent'
      shift(206);                   // 'parent'
      lookahead1W(26);              // S^WS | '(:' | '::'
      shift(51);                    // '::'
      break;
    case 73:                        // 'ancestor'
      shift(73);                    // 'ancestor'
      lookahead1W(26);              // S^WS | '(:' | '::'
      shift(51);                    // '::'
      break;
    case 213:                       // 'preceding-sibling'
      shift(213);                   // 'preceding-sibling'
      lookahead1W(26);              // S^WS | '(:' | '::'
      shift(51);                    // '::'
      break;
    case 212:                       // 'preceding'
      shift(212);                   // 'preceding'
      lookahead1W(26);              // S^WS | '(:' | '::'
      shift(51);                    // '::'
      break;
    default:
      shift(74);                    // 'ancestor-or-self'
      lookahead1W(26);              // S^WS | '(:' | '::'
      shift(51);                    // '::'
    }
    eventHandler.endNonterminal("ReverseAxis", e0);
  }

  function try_ReverseAxis()
  {
    switch (l1)
    {
    case 206:                       // 'parent'
      shiftT(206);                  // 'parent'
      lookahead1W(26);              // S^WS | '(:' | '::'
      shiftT(51);                   // '::'
      break;
    case 73:                        // 'ancestor'
      shiftT(73);                   // 'ancestor'
      lookahead1W(26);              // S^WS | '(:' | '::'
      shiftT(51);                   // '::'
      break;
    case 213:                       // 'preceding-sibling'
      shiftT(213);                  // 'preceding-sibling'
      lookahead1W(26);              // S^WS | '(:' | '::'
      shiftT(51);                   // '::'
      break;
    case 212:                       // 'preceding'
      shiftT(212);                  // 'preceding'
      lookahead1W(26);              // S^WS | '(:' | '::'
      shiftT(51);                   // '::'
      break;
    default:
      shiftT(74);                   // 'ancestor-or-self'
      lookahead1W(26);              // S^WS | '(:' | '::'
      shiftT(51);                   // '::'
    }
  }

  function parse_AbbrevReverseStep()
  {
    eventHandler.startNonterminal("AbbrevReverseStep", e0);
    shift(45);                      // '..'
    eventHandler.endNonterminal("AbbrevReverseStep", e0);
  }

  function try_AbbrevReverseStep()
  {
    shiftT(45);                     // '..'
  }

  function parse_NodeTest()
  {
    eventHandler.startNonterminal("NodeTest", e0);
    switch (l1)
    {
    case 82:                        // 'attribute'
    case 96:                        // 'comment'
    case 120:                       // 'document-node'
    case 121:                       // 'element'
    case 185:                       // 'namespace-node'
    case 191:                       // 'node'
    case 216:                       // 'processing-instruction'
    case 226:                       // 'schema-attribute'
    case 227:                       // 'schema-element'
    case 244:                       // 'text'
      lookahead2W(241);             // S^WS | EOF | '!' | '!=' | '(' | '(:' | ')' | '*' | '+' | ',' | '-' | '/' | '//' |
      break;
    default:
      lk = l1;
    }
    switch (lk)
    {
    case 17490:                     // 'attribute' '('
    case 17504:                     // 'comment' '('
    case 17528:                     // 'document-node' '('
    case 17529:                     // 'element' '('
    case 17593:                     // 'namespace-node' '('
    case 17599:                     // 'node' '('
    case 17624:                     // 'processing-instruction' '('
    case 17634:                     // 'schema-attribute' '('
    case 17635:                     // 'schema-element' '('
    case 17652:                     // 'text' '('
      parse_KindTest();
      break;
    default:
      parse_NameTest();
    }
    eventHandler.endNonterminal("NodeTest", e0);
  }

  function try_NodeTest()
  {
    switch (l1)
    {
    case 82:                        // 'attribute'
    case 96:                        // 'comment'
    case 120:                       // 'document-node'
    case 121:                       // 'element'
    case 185:                       // 'namespace-node'
    case 191:                       // 'node'
    case 216:                       // 'processing-instruction'
    case 226:                       // 'schema-attribute'
    case 227:                       // 'schema-element'
    case 244:                       // 'text'
      lookahead2W(241);             // S^WS | EOF | '!' | '!=' | '(' | '(:' | ')' | '*' | '+' | ',' | '-' | '/' | '//' |
      break;
    default:
      lk = l1;
    }
    switch (lk)
    {
    case 17490:                     // 'attribute' '('
    case 17504:                     // 'comment' '('
    case 17528:                     // 'document-node' '('
    case 17529:                     // 'element' '('
    case 17593:                     // 'namespace-node' '('
    case 17599:                     // 'node' '('
    case 17624:                     // 'processing-instruction' '('
    case 17634:                     // 'schema-attribute' '('
    case 17635:                     // 'schema-element' '('
    case 17652:                     // 'text' '('
      try_KindTest();
      break;
    default:
      try_NameTest();
    }
  }

  function parse_NameTest()
  {
    eventHandler.startNonterminal("NameTest", e0);
    switch (l1)
    {
    case 5:                         // Wildcard
      shift(5);                     // Wildcard
      break;
    default:
      parse_EQName();
    }
    eventHandler.endNonterminal("NameTest", e0);
  }

  function try_NameTest()
  {
    switch (l1)
    {
    case 5:                         // Wildcard
      shiftT(5);                    // Wildcard
      break;
    default:
      try_EQName();
    }
  }

  function parse_PostfixExpr()
  {
    eventHandler.startNonterminal("PostfixExpr", e0);
    parse_PrimaryExpr();
    for (;;)
    {
      lookahead1W(241);             // S^WS | EOF | '!' | '!=' | '(' | '(:' | ')' | '*' | '+' | ',' | '-' | '/' | '//' |
      if (l1 != 34                  // '('
       && l1 != 68)                 // '['
      {
        break;
      }
      switch (l1)
      {
      case 68:                      // '['
        whitespace();
        parse_Predicate();
        break;
      default:
        whitespace();
        parse_ArgumentList();
      }
    }
    eventHandler.endNonterminal("PostfixExpr", e0);
  }

  function try_PostfixExpr()
  {
    try_PrimaryExpr();
    for (;;)
    {
      lookahead1W(241);             // S^WS | EOF | '!' | '!=' | '(' | '(:' | ')' | '*' | '+' | ',' | '-' | '/' | '//' |
      if (l1 != 34                  // '('
       && l1 != 68)                 // '['
      {
        break;
      }
      switch (l1)
      {
      case 68:                      // '['
        try_Predicate();
        break;
      default:
        try_ArgumentList();
      }
    }
  }

  function parse_ArgumentList()
  {
    eventHandler.startNonterminal("ArgumentList", e0);
    shift(34);                      // '('
    lookahead1W(276);               // Wildcard | EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral |
    if (l1 != 37)                   // ')'
    {
      whitespace();
      parse_Argument();
      for (;;)
      {
        lookahead1W(101);           // S^WS | '(:' | ')' | ','
        if (l1 != 41)               // ','
        {
          break;
        }
        shift(41);                  // ','
        lookahead1W(271);           // Wildcard | EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral |
        whitespace();
        parse_Argument();
      }
    }
    shift(37);                      // ')'
    eventHandler.endNonterminal("ArgumentList", e0);
  }

  function try_ArgumentList()
  {
    shiftT(34);                     // '('
    lookahead1W(276);               // Wildcard | EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral |
    if (l1 != 37)                   // ')'
    {
      try_Argument();
      for (;;)
      {
        lookahead1W(101);           // S^WS | '(:' | ')' | ','
        if (l1 != 41)               // ','
        {
          break;
        }
        shiftT(41);                 // ','
        lookahead1W(271);           // Wildcard | EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral |
        try_Argument();
      }
    }
    shiftT(37);                     // ')'
  }

  function parse_PredicateList()
  {
    eventHandler.startNonterminal("PredicateList", e0);
    for (;;)
    {
      lookahead1W(238);             // S^WS | EOF | '!' | '!=' | '(:' | ')' | '*' | '+' | ',' | '-' | '/' | '//' | ':' |
      if (l1 != 68)                 // '['
      {
        break;
      }
      whitespace();
      parse_Predicate();
    }
    eventHandler.endNonterminal("PredicateList", e0);
  }

  function try_PredicateList()
  {
    for (;;)
    {
      lookahead1W(238);             // S^WS | EOF | '!' | '!=' | '(:' | ')' | '*' | '+' | ',' | '-' | '/' | '//' | ':' |
      if (l1 != 68)                 // '['
      {
        break;
      }
      try_Predicate();
    }
  }

  function parse_Predicate()
  {
    eventHandler.startNonterminal("Predicate", e0);
    shift(68);                      // '['
    lookahead1W(267);               // Wildcard | EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral |
    whitespace();
    parse_Expr();
    shift(69);                      // ']'
    eventHandler.endNonterminal("Predicate", e0);
  }

  function try_Predicate()
  {
    shiftT(68);                     // '['
    lookahead1W(267);               // Wildcard | EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral |
    try_Expr();
    shiftT(69);                     // ']'
  }

  function parse_Literal()
  {
    eventHandler.startNonterminal("Literal", e0);
    switch (l1)
    {
    case 11:                        // StringLiteral
      shift(11);                    // StringLiteral
      break;
    default:
      parse_NumericLiteral();
    }
    eventHandler.endNonterminal("Literal", e0);
  }

  function try_Literal()
  {
    switch (l1)
    {
    case 11:                        // StringLiteral
      shiftT(11);                   // StringLiteral
      break;
    default:
      try_NumericLiteral();
    }
  }

  function parse_NumericLiteral()
  {
    eventHandler.startNonterminal("NumericLiteral", e0);
    switch (l1)
    {
    case 8:                         // IntegerLiteral
      shift(8);                     // IntegerLiteral
      break;
    case 9:                         // DecimalLiteral
      shift(9);                     // DecimalLiteral
      break;
    default:
      shift(10);                    // DoubleLiteral
    }
    eventHandler.endNonterminal("NumericLiteral", e0);
  }

  function try_NumericLiteral()
  {
    switch (l1)
    {
    case 8:                         // IntegerLiteral
      shiftT(8);                    // IntegerLiteral
      break;
    case 9:                         // DecimalLiteral
      shiftT(9);                    // DecimalLiteral
      break;
    default:
      shiftT(10);                   // DoubleLiteral
    }
  }

  function parse_VarRef()
  {
    eventHandler.startNonterminal("VarRef", e0);
    shift(31);                      // '$'
    lookahead1W(255);               // EQName^Token | S^WS | '(:' | 'after' | 'allowing' | 'ancestor' |
    whitespace();
    parse_VarName();
    eventHandler.endNonterminal("VarRef", e0);
  }

  function try_VarRef()
  {
    shiftT(31);                     // '$'
    lookahead1W(255);               // EQName^Token | S^WS | '(:' | 'after' | 'allowing' | 'ancestor' |
    try_VarName();
  }

  function parse_VarName()
  {
    eventHandler.startNonterminal("VarName", e0);
    parse_EQName();
    eventHandler.endNonterminal("VarName", e0);
  }

  function try_VarName()
  {
    try_EQName();
  }

  function parse_ParenthesizedExpr()
  {
    eventHandler.startNonterminal("ParenthesizedExpr", e0);
    shift(34);                      // '('
    lookahead1W(269);               // Wildcard | EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral |
    if (l1 != 37)                   // ')'
    {
      whitespace();
      parse_Expr();
    }
    shift(37);                      // ')'
    eventHandler.endNonterminal("ParenthesizedExpr", e0);
  }

  function try_ParenthesizedExpr()
  {
    shiftT(34);                     // '('
    lookahead1W(269);               // Wildcard | EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral |
    if (l1 != 37)                   // ')'
    {
      try_Expr();
    }
    shiftT(37);                     // ')'
  }

  function parse_ContextItemExpr()
  {
    eventHandler.startNonterminal("ContextItemExpr", e0);
    shift(44);                      // '.'
    eventHandler.endNonterminal("ContextItemExpr", e0);
  }

  function try_ContextItemExpr()
  {
    shiftT(44);                     // '.'
  }

  function parse_OrderedExpr()
  {
    eventHandler.startNonterminal("OrderedExpr", e0);
    shift(202);                     // 'ordered'
    lookahead1W(87);                // S^WS | '(:' | '{'
    shift(276);                     // '{'
    lookahead1W(267);               // Wildcard | EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral |
    whitespace();
    parse_Expr();
    shift(282);                     // '}'
    eventHandler.endNonterminal("OrderedExpr", e0);
  }

  function try_OrderedExpr()
  {
    shiftT(202);                    // 'ordered'
    lookahead1W(87);                // S^WS | '(:' | '{'
    shiftT(276);                    // '{'
    lookahead1W(267);               // Wildcard | EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral |
    try_Expr();
    shiftT(282);                    // '}'
  }

  function parse_UnorderedExpr()
  {
    eventHandler.startNonterminal("UnorderedExpr", e0);
    shift(256);                     // 'unordered'
    lookahead1W(87);                // S^WS | '(:' | '{'
    shift(276);                     // '{'
    lookahead1W(267);               // Wildcard | EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral |
    whitespace();
    parse_Expr();
    shift(282);                     // '}'
    eventHandler.endNonterminal("UnorderedExpr", e0);
  }

  function try_UnorderedExpr()
  {
    shiftT(256);                    // 'unordered'
    lookahead1W(87);                // S^WS | '(:' | '{'
    shiftT(276);                    // '{'
    lookahead1W(267);               // Wildcard | EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral |
    try_Expr();
    shiftT(282);                    // '}'
  }

  function parse_FunctionCall()
  {
    eventHandler.startNonterminal("FunctionCall", e0);
    parse_FunctionName();
    lookahead1W(22);                // S^WS | '(' | '(:'
    whitespace();
    parse_ArgumentList();
    eventHandler.endNonterminal("FunctionCall", e0);
  }

  function try_FunctionCall()
  {
    try_FunctionName();
    lookahead1W(22);                // S^WS | '(' | '(:'
    try_ArgumentList();
  }

  function parse_Argument()
  {
    eventHandler.startNonterminal("Argument", e0);
    switch (l1)
    {
    case 64:                        // '?'
      parse_ArgumentPlaceholder();
      break;
    default:
      parse_ExprSingle();
    }
    eventHandler.endNonterminal("Argument", e0);
  }

  function try_Argument()
  {
    switch (l1)
    {
    case 64:                        // '?'
      try_ArgumentPlaceholder();
      break;
    default:
      try_ExprSingle();
    }
  }

  function parse_ArgumentPlaceholder()
  {
    eventHandler.startNonterminal("ArgumentPlaceholder", e0);
    shift(64);                      // '?'
    eventHandler.endNonterminal("ArgumentPlaceholder", e0);
  }

  function try_ArgumentPlaceholder()
  {
    shiftT(64);                     // '?'
  }

  function parse_Constructor()
  {
    eventHandler.startNonterminal("Constructor", e0);
    switch (l1)
    {
    case 54:                        // '<'
    case 55:                        // '<!--'
    case 59:                        // '<?'
      parse_DirectConstructor();
      break;
    default:
      parse_ComputedConstructor();
    }
    eventHandler.endNonterminal("Constructor", e0);
  }

  function try_Constructor()
  {
    switch (l1)
    {
    case 54:                        // '<'
    case 55:                        // '<!--'
    case 59:                        // '<?'
      try_DirectConstructor();
      break;
    default:
      try_ComputedConstructor();
    }
  }

  function parse_DirectConstructor()
  {
    eventHandler.startNonterminal("DirectConstructor", e0);
    switch (l1)
    {
    case 54:                        // '<'
      parse_DirElemConstructor();
      break;
    case 55:                        // '<!--'
      parse_DirCommentConstructor();
      break;
    default:
      parse_DirPIConstructor();
    }
    eventHandler.endNonterminal("DirectConstructor", e0);
  }

  function try_DirectConstructor()
  {
    switch (l1)
    {
    case 54:                        // '<'
      try_DirElemConstructor();
      break;
    case 55:                        // '<!--'
      try_DirCommentConstructor();
      break;
    default:
      try_DirPIConstructor();
    }
  }

  function parse_DirElemConstructor()
  {
    eventHandler.startNonterminal("DirElemConstructor", e0);
    shift(54);                      // '<'
    lookahead1(4);                  // QName
    shift(20);                      // QName
    parse_DirAttributeList();
    switch (l1)
    {
    case 48:                        // '/>'
      shift(48);                    // '/>'
      break;
    default:
      shift(61);                    // '>'
      for (;;)
      {
        lookahead1(174);            // CDataSection | PredefinedEntityRef | ElementContentChar | CharRef | '<' |
        if (l1 == 56)               // '</'
        {
          break;
        }
        parse_DirElemContent();
      }
      shift(56);                    // '</'
      lookahead1(4);                // QName
      shift(20);                    // QName
      lookahead1(12);               // S | '>'
      if (l1 == 21)                 // S
      {
        shift(21);                  // S
      }
      lookahead1(8);                // '>'
      shift(61);                    // '>'
    }
    eventHandler.endNonterminal("DirElemConstructor", e0);
  }

  function try_DirElemConstructor()
  {
    shiftT(54);                     // '<'
    lookahead1(4);                  // QName
    shiftT(20);                     // QName
    try_DirAttributeList();
    switch (l1)
    {
    case 48:                        // '/>'
      shiftT(48);                   // '/>'
      break;
    default:
      shiftT(61);                   // '>'
      for (;;)
      {
        lookahead1(174);            // CDataSection | PredefinedEntityRef | ElementContentChar | CharRef | '<' |
        if (l1 == 56)               // '</'
        {
          break;
        }
        try_DirElemContent();
      }
      shiftT(56);                   // '</'
      lookahead1(4);                // QName
      shiftT(20);                   // QName
      lookahead1(12);               // S | '>'
      if (l1 == 21)                 // S
      {
        shiftT(21);                 // S
      }
      lookahead1(8);                // '>'
      shiftT(61);                   // '>'
    }
  }

  function parse_DirAttributeList()
  {
    eventHandler.startNonterminal("DirAttributeList", e0);
    for (;;)
    {
      lookahead1(19);               // S | '/>' | '>'
      if (l1 != 21)                 // S
      {
        break;
      }
      shift(21);                    // S
      lookahead1(91);               // QName | S | '/>' | '>'
      if (l1 == 20)                 // QName
      {
        shift(20);                  // QName
        lookahead1(11);             // S | '='
        if (l1 == 21)               // S
        {
          shift(21);                // S
        }
        lookahead1(7);              // '='
        shift(60);                  // '='
        lookahead1(18);             // S | '"' | "'"
        if (l1 == 21)               // S
        {
          shift(21);                // S
        }
        parse_DirAttributeValue();
      }
    }
    eventHandler.endNonterminal("DirAttributeList", e0);
  }

  function try_DirAttributeList()
  {
    for (;;)
    {
      lookahead1(19);               // S | '/>' | '>'
      if (l1 != 21)                 // S
      {
        break;
      }
      shiftT(21);                   // S
      lookahead1(91);               // QName | S | '/>' | '>'
      if (l1 == 20)                 // QName
      {
        shiftT(20);                 // QName
        lookahead1(11);             // S | '='
        if (l1 == 21)               // S
        {
          shiftT(21);               // S
        }
        lookahead1(7);              // '='
        shiftT(60);                 // '='
        lookahead1(18);             // S | '"' | "'"
        if (l1 == 21)               // S
        {
          shiftT(21);               // S
        }
        try_DirAttributeValue();
      }
    }
  }

  function parse_DirAttributeValue()
  {
    eventHandler.startNonterminal("DirAttributeValue", e0);
    lookahead1(14);                 // '"' | "'"
    switch (l1)
    {
    case 28:                        // '"'
      shift(28);                    // '"'
      for (;;)
      {
        lookahead1(167);            // PredefinedEntityRef | EscapeQuot | QuotAttrContentChar | CharRef | '"' | '{' |
        if (l1 == 28)               // '"'
        {
          break;
        }
        switch (l1)
        {
        case 13:                    // EscapeQuot
          shift(13);                // EscapeQuot
          break;
        default:
          parse_QuotAttrValueContent();
        }
      }
      shift(28);                    // '"'
      break;
    default:
      shift(33);                    // "'"
      for (;;)
      {
        lookahead1(168);            // PredefinedEntityRef | EscapeApos | AposAttrContentChar | CharRef | "'" | '{' |
        if (l1 == 33)               // "'"
        {
          break;
        }
        switch (l1)
        {
        case 14:                    // EscapeApos
          shift(14);                // EscapeApos
          break;
        default:
          parse_AposAttrValueContent();
        }
      }
      shift(33);                    // "'"
    }
    eventHandler.endNonterminal("DirAttributeValue", e0);
  }

  function try_DirAttributeValue()
  {
    lookahead1(14);                 // '"' | "'"
    switch (l1)
    {
    case 28:                        // '"'
      shiftT(28);                   // '"'
      for (;;)
      {
        lookahead1(167);            // PredefinedEntityRef | EscapeQuot | QuotAttrContentChar | CharRef | '"' | '{' |
        if (l1 == 28)               // '"'
        {
          break;
        }
        switch (l1)
        {
        case 13:                    // EscapeQuot
          shiftT(13);               // EscapeQuot
          break;
        default:
          try_QuotAttrValueContent();
        }
      }
      shiftT(28);                   // '"'
      break;
    default:
      shiftT(33);                   // "'"
      for (;;)
      {
        lookahead1(168);            // PredefinedEntityRef | EscapeApos | AposAttrContentChar | CharRef | "'" | '{' |
        if (l1 == 33)               // "'"
        {
          break;
        }
        switch (l1)
        {
        case 14:                    // EscapeApos
          shiftT(14);               // EscapeApos
          break;
        default:
          try_AposAttrValueContent();
        }
      }
      shiftT(33);                   // "'"
    }
  }

  function parse_QuotAttrValueContent()
  {
    eventHandler.startNonterminal("QuotAttrValueContent", e0);
    switch (l1)
    {
    case 16:                        // QuotAttrContentChar
      shift(16);                    // QuotAttrContentChar
      break;
    default:
      parse_CommonContent();
    }
    eventHandler.endNonterminal("QuotAttrValueContent", e0);
  }

  function try_QuotAttrValueContent()
  {
    switch (l1)
    {
    case 16:                        // QuotAttrContentChar
      shiftT(16);                   // QuotAttrContentChar
      break;
    default:
      try_CommonContent();
    }
  }

  function parse_AposAttrValueContent()
  {
    eventHandler.startNonterminal("AposAttrValueContent", e0);
    switch (l1)
    {
    case 17:                        // AposAttrContentChar
      shift(17);                    // AposAttrContentChar
      break;
    default:
      parse_CommonContent();
    }
    eventHandler.endNonterminal("AposAttrValueContent", e0);
  }

  function try_AposAttrValueContent()
  {
    switch (l1)
    {
    case 17:                        // AposAttrContentChar
      shiftT(17);                   // AposAttrContentChar
      break;
    default:
      try_CommonContent();
    }
  }

  function parse_DirElemContent()
  {
    eventHandler.startNonterminal("DirElemContent", e0);
    switch (l1)
    {
    case 54:                        // '<'
    case 55:                        // '<!--'
    case 59:                        // '<?'
      parse_DirectConstructor();
      break;
    case 4:                         // CDataSection
      shift(4);                     // CDataSection
      break;
    case 15:                        // ElementContentChar
      shift(15);                    // ElementContentChar
      break;
    default:
      parse_CommonContent();
    }
    eventHandler.endNonterminal("DirElemContent", e0);
  }

  function try_DirElemContent()
  {
    switch (l1)
    {
    case 54:                        // '<'
    case 55:                        // '<!--'
    case 59:                        // '<?'
      try_DirectConstructor();
      break;
    case 4:                         // CDataSection
      shiftT(4);                    // CDataSection
      break;
    case 15:                        // ElementContentChar
      shiftT(15);                   // ElementContentChar
      break;
    default:
      try_CommonContent();
    }
  }

  function parse_DirCommentConstructor()
  {
    eventHandler.startNonterminal("DirCommentConstructor", e0);
    shift(55);                      // '<!--'
    lookahead1(1);                  // DirCommentContents
    shift(2);                       // DirCommentContents
    lookahead1(6);                  // '-->'
    shift(43);                      // '-->'
    eventHandler.endNonterminal("DirCommentConstructor", e0);
  }

  function try_DirCommentConstructor()
  {
    shiftT(55);                     // '<!--'
    lookahead1(1);                  // DirCommentContents
    shiftT(2);                      // DirCommentContents
    lookahead1(6);                  // '-->'
    shiftT(43);                     // '-->'
  }

  function parse_DirPIConstructor()
  {
    eventHandler.startNonterminal("DirPIConstructor", e0);
    shift(59);                      // '<?'
    lookahead1(3);                  // PITarget
    shift(18);                      // PITarget
    lookahead1(13);                 // S | '?>'
    if (l1 == 21)                   // S
    {
      shift(21);                    // S
      lookahead1(2);                // DirPIContents
      shift(3);                     // DirPIContents
    }
    lookahead1(9);                  // '?>'
    shift(65);                      // '?>'
    eventHandler.endNonterminal("DirPIConstructor", e0);
  }

  function try_DirPIConstructor()
  {
    shiftT(59);                     // '<?'
    lookahead1(3);                  // PITarget
    shiftT(18);                     // PITarget
    lookahead1(13);                 // S | '?>'
    if (l1 == 21)                   // S
    {
      shiftT(21);                   // S
      lookahead1(2);                // DirPIContents
      shiftT(3);                    // DirPIContents
    }
    lookahead1(9);                  // '?>'
    shiftT(65);                     // '?>'
  }

  function parse_ComputedConstructor()
  {
    eventHandler.startNonterminal("ComputedConstructor", e0);
    switch (l1)
    {
    case 119:                       // 'document'
      parse_CompDocConstructor();
      break;
    case 121:                       // 'element'
      parse_CompElemConstructor();
      break;
    case 82:                        // 'attribute'
      parse_CompAttrConstructor();
      break;
    case 184:                       // 'namespace'
      parse_CompNamespaceConstructor();
      break;
    case 244:                       // 'text'
      parse_CompTextConstructor();
      break;
    case 96:                        // 'comment'
      parse_CompCommentConstructor();
      break;
    default:
      parse_CompPIConstructor();
    }
    eventHandler.endNonterminal("ComputedConstructor", e0);
  }

  function try_ComputedConstructor()
  {
    switch (l1)
    {
    case 119:                       // 'document'
      try_CompDocConstructor();
      break;
    case 121:                       // 'element'
      try_CompElemConstructor();
      break;
    case 82:                        // 'attribute'
      try_CompAttrConstructor();
      break;
    case 184:                       // 'namespace'
      try_CompNamespaceConstructor();
      break;
    case 244:                       // 'text'
      try_CompTextConstructor();
      break;
    case 96:                        // 'comment'
      try_CompCommentConstructor();
      break;
    default:
      try_CompPIConstructor();
    }
  }

  function parse_CompElemConstructor()
  {
    eventHandler.startNonterminal("CompElemConstructor", e0);
    shift(121);                     // 'element'
    lookahead1W(258);               // EQName^Token | S^WS | '(:' | 'after' | 'allowing' | 'ancestor' |
    switch (l1)
    {
    case 276:                       // '{'
      shift(276);                   // '{'
      lookahead1W(267);             // Wildcard | EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral |
      whitespace();
      parse_Expr();
      shift(282);                   // '}'
      break;
    default:
      whitespace();
      parse_EQName();
    }
    lookahead1W(87);                // S^WS | '(:' | '{'
    shift(276);                     // '{'
    lookahead1W(277);               // Wildcard | EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral |
    if (l1 != 282)                  // '}'
    {
      whitespace();
      parse_ContentExpr();
    }
    shift(282);                     // '}'
    eventHandler.endNonterminal("CompElemConstructor", e0);
  }

  function try_CompElemConstructor()
  {
    shiftT(121);                    // 'element'
    lookahead1W(258);               // EQName^Token | S^WS | '(:' | 'after' | 'allowing' | 'ancestor' |
    switch (l1)
    {
    case 276:                       // '{'
      shiftT(276);                  // '{'
      lookahead1W(267);             // Wildcard | EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral |
      try_Expr();
      shiftT(282);                  // '}'
      break;
    default:
      try_EQName();
    }
    lookahead1W(87);                // S^WS | '(:' | '{'
    shiftT(276);                    // '{'
    lookahead1W(277);               // Wildcard | EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral |
    if (l1 != 282)                  // '}'
    {
      try_ContentExpr();
    }
    shiftT(282);                    // '}'
  }

  function parse_CompNamespaceConstructor()
  {
    eventHandler.startNonterminal("CompNamespaceConstructor", e0);
    shift(184);                     // 'namespace'
    lookahead1W(251);               // NCName^Token | S^WS | '(:' | 'after' | 'allowing' | 'ancestor' |
    switch (l1)
    {
    case 276:                       // '{'
      shift(276);                   // '{'
      lookahead1W(267);             // Wildcard | EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral |
      whitespace();
      parse_PrefixExpr();
      shift(282);                   // '}'
      break;
    default:
      whitespace();
      parse_Prefix();
    }
    lookahead1W(87);                // S^WS | '(:' | '{'
    shift(276);                     // '{'
    lookahead1W(267);               // Wildcard | EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral |
    whitespace();
    parse_URIExpr();
    shift(282);                     // '}'
    eventHandler.endNonterminal("CompNamespaceConstructor", e0);
  }

  function try_CompNamespaceConstructor()
  {
    shiftT(184);                    // 'namespace'
    lookahead1W(251);               // NCName^Token | S^WS | '(:' | 'after' | 'allowing' | 'ancestor' |
    switch (l1)
    {
    case 276:                       // '{'
      shiftT(276);                  // '{'
      lookahead1W(267);             // Wildcard | EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral |
      try_PrefixExpr();
      shiftT(282);                  // '}'
      break;
    default:
      try_Prefix();
    }
    lookahead1W(87);                // S^WS | '(:' | '{'
    shiftT(276);                    // '{'
    lookahead1W(267);               // Wildcard | EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral |
    try_URIExpr();
    shiftT(282);                    // '}'
  }

  function parse_Prefix()
  {
    eventHandler.startNonterminal("Prefix", e0);
    parse_NCName();
    eventHandler.endNonterminal("Prefix", e0);
  }

  function try_Prefix()
  {
    try_NCName();
  }

  function parse_PrefixExpr()
  {
    eventHandler.startNonterminal("PrefixExpr", e0);
    parse_Expr();
    eventHandler.endNonterminal("PrefixExpr", e0);
  }

  function try_PrefixExpr()
  {
    try_Expr();
  }

  function parse_URIExpr()
  {
    eventHandler.startNonterminal("URIExpr", e0);
    parse_Expr();
    eventHandler.endNonterminal("URIExpr", e0);
  }

  function try_URIExpr()
  {
    try_Expr();
  }

  function parse_FunctionItemExpr()
  {
    eventHandler.startNonterminal("FunctionItemExpr", e0);
    switch (l1)
    {
    case 145:                       // 'function'
      lookahead2W(92);              // S^WS | '#' | '(' | '(:'
      break;
    default:
      lk = l1;
    }
    switch (lk)
    {
    case 32:                        // '%'
    case 17553:                     // 'function' '('
      parse_InlineFunctionExpr();
      break;
    default:
      parse_NamedFunctionRef();
    }
    eventHandler.endNonterminal("FunctionItemExpr", e0);
  }

  function try_FunctionItemExpr()
  {
    switch (l1)
    {
    case 145:                       // 'function'
      lookahead2W(92);              // S^WS | '#' | '(' | '(:'
      break;
    default:
      lk = l1;
    }
    switch (lk)
    {
    case 32:                        // '%'
    case 17553:                     // 'function' '('
      try_InlineFunctionExpr();
      break;
    default:
      try_NamedFunctionRef();
    }
  }

  function parse_NamedFunctionRef()
  {
    eventHandler.startNonterminal("NamedFunctionRef", e0);
    parse_EQName();
    lookahead1W(20);                // S^WS | '#' | '(:'
    shift(29);                      // '#'
    lookahead1W(16);                // IntegerLiteral | S^WS | '(:'
    shift(8);                       // IntegerLiteral
    eventHandler.endNonterminal("NamedFunctionRef", e0);
  }

  function try_NamedFunctionRef()
  {
    try_EQName();
    lookahead1W(20);                // S^WS | '#' | '(:'
    shiftT(29);                     // '#'
    lookahead1W(16);                // IntegerLiteral | S^WS | '(:'
    shiftT(8);                      // IntegerLiteral
  }

  function parse_InlineFunctionExpr()
  {
    eventHandler.startNonterminal("InlineFunctionExpr", e0);
    for (;;)
    {
      lookahead1W(97);              // S^WS | '%' | '(:' | 'function'
      if (l1 != 32)                 // '%'
      {
        break;
      }
      whitespace();
      parse_Annotation();
    }
    shift(145);                     // 'function'
    lookahead1W(22);                // S^WS | '(' | '(:'
    shift(34);                      // '('
    lookahead1W(94);                // S^WS | '$' | '(:' | ')'
    if (l1 == 31)                   // '$'
    {
      whitespace();
      parse_ParamList();
    }
    shift(37);                      // ')'
    lookahead1W(111);               // S^WS | '(:' | 'as' | '{'
    if (l1 == 79)                   // 'as'
    {
      shift(79);                    // 'as'
      lookahead1W(260);             // EQName^Token | S^WS | '%' | '(' | '(:' | 'after' | 'allowing' | 'ancestor' |
      whitespace();
      parse_SequenceType();
    }
    lookahead1W(87);                // S^WS | '(:' | '{'
    whitespace();
    parse_FunctionBody();
    eventHandler.endNonterminal("InlineFunctionExpr", e0);
  }

  function try_InlineFunctionExpr()
  {
    for (;;)
    {
      lookahead1W(97);              // S^WS | '%' | '(:' | 'function'
      if (l1 != 32)                 // '%'
      {
        break;
      }
      try_Annotation();
    }
    shiftT(145);                    // 'function'
    lookahead1W(22);                // S^WS | '(' | '(:'
    shiftT(34);                     // '('
    lookahead1W(94);                // S^WS | '$' | '(:' | ')'
    if (l1 == 31)                   // '$'
    {
      try_ParamList();
    }
    shiftT(37);                     // ')'
    lookahead1W(111);               // S^WS | '(:' | 'as' | '{'
    if (l1 == 79)                   // 'as'
    {
      shiftT(79);                   // 'as'
      lookahead1W(260);             // EQName^Token | S^WS | '%' | '(' | '(:' | 'after' | 'allowing' | 'ancestor' |
      try_SequenceType();
    }
    lookahead1W(87);                // S^WS | '(:' | '{'
    try_FunctionBody();
  }

  function parse_SingleType()
  {
    eventHandler.startNonterminal("SingleType", e0);
    parse_SimpleTypeName();
    lookahead1W(227);               // S^WS | EOF | '!=' | '(:' | ')' | '*' | '+' | ',' | '-' | ':' | ';' | '<' | '<<' |
    if (l1 == 64)                   // '?'
    {
      shift(64);                    // '?'
    }
    eventHandler.endNonterminal("SingleType", e0);
  }

  function try_SingleType()
  {
    try_SimpleTypeName();
    lookahead1W(227);               // S^WS | EOF | '!=' | '(:' | ')' | '*' | '+' | ',' | '-' | ':' | ';' | '<' | '<<' |
    if (l1 == 64)                   // '?'
    {
      shiftT(64);                   // '?'
    }
  }

  function parse_TypeDeclaration()
  {
    eventHandler.startNonterminal("TypeDeclaration", e0);
    shift(79);                      // 'as'
    lookahead1W(260);               // EQName^Token | S^WS | '%' | '(' | '(:' | 'after' | 'allowing' | 'ancestor' |
    whitespace();
    parse_SequenceType();
    eventHandler.endNonterminal("TypeDeclaration", e0);
  }

  function try_TypeDeclaration()
  {
    shiftT(79);                     // 'as'
    lookahead1W(260);               // EQName^Token | S^WS | '%' | '(' | '(:' | 'after' | 'allowing' | 'ancestor' |
    try_SequenceType();
  }

  function parse_SequenceType()
  {
    eventHandler.startNonterminal("SequenceType", e0);
    switch (l1)
    {
    case 124:                       // 'empty-sequence'
      lookahead2W(243);             // S^WS | EOF | '!=' | '(' | '(:' | ')' | '*' | '*' | '+' | ',' | '-' | ':' | ':=' |
      break;
    default:
      lk = l1;
    }
    switch (lk)
    {
    case 17532:                     // 'empty-sequence' '('
      shift(124);                   // 'empty-sequence'
      lookahead1W(22);              // S^WS | '(' | '(:'
      shift(34);                    // '('
      lookahead1W(23);              // S^WS | '(:' | ')'
      shift(37);                    // ')'
      break;
    default:
      parse_ItemType();
      lookahead1W(239);             // S^WS | EOF | '!=' | '(:' | ')' | '*' | '*' | '+' | ',' | '-' | ':' | ':=' | ';' |
      switch (l1)
      {
      case 39:                      // '*'
      case 40:                      // '+'
      case 64:                      // '?'
        whitespace();
        parse_OccurrenceIndicator();
        break;
      default:
        break;
      }
    }
    eventHandler.endNonterminal("SequenceType", e0);
  }

  function try_SequenceType()
  {
    switch (l1)
    {
    case 124:                       // 'empty-sequence'
      lookahead2W(243);             // S^WS | EOF | '!=' | '(' | '(:' | ')' | '*' | '*' | '+' | ',' | '-' | ':' | ':=' |
      break;
    default:
      lk = l1;
    }
    switch (lk)
    {
    case 17532:                     // 'empty-sequence' '('
      shiftT(124);                  // 'empty-sequence'
      lookahead1W(22);              // S^WS | '(' | '(:'
      shiftT(34);                   // '('
      lookahead1W(23);              // S^WS | '(:' | ')'
      shiftT(37);                   // ')'
      break;
    default:
      try_ItemType();
      lookahead1W(239);             // S^WS | EOF | '!=' | '(:' | ')' | '*' | '*' | '+' | ',' | '-' | ':' | ':=' | ';' |
      switch (l1)
      {
      case 39:                      // '*'
      case 40:                      // '+'
      case 64:                      // '?'
        try_OccurrenceIndicator();
        break;
      default:
        break;
      }
    }
  }

  function parse_OccurrenceIndicator()
  {
    eventHandler.startNonterminal("OccurrenceIndicator", e0);
    switch (l1)
    {
    case 64:                        // '?'
      shift(64);                    // '?'
      break;
    case 39:                        // '*'
      shift(39);                    // '*'
      break;
    default:
      shift(40);                    // '+'
    }
    eventHandler.endNonterminal("OccurrenceIndicator", e0);
  }

  function try_OccurrenceIndicator()
  {
    switch (l1)
    {
    case 64:                        // '?'
      shiftT(64);                   // '?'
      break;
    case 39:                        // '*'
      shiftT(39);                   // '*'
      break;
    default:
      shiftT(40);                   // '+'
    }
  }

  function parse_ItemType()
  {
    eventHandler.startNonterminal("ItemType", e0);
    switch (l1)
    {
    case 78:                        // 'array'
    case 82:                        // 'attribute'
    case 96:                        // 'comment'
    case 120:                       // 'document-node'
    case 121:                       // 'element'
    case 145:                       // 'function'
    case 165:                       // 'item'
    case 167:                       // 'json-item'
    case 185:                       // 'namespace-node'
    case 191:                       // 'node'
    case 194:                       // 'object'
    case 216:                       // 'processing-instruction'
    case 226:                       // 'schema-attribute'
    case 227:                       // 'schema-element'
    case 242:                       // 'structured-item'
    case 244:                       // 'text'
      lookahead2W(243);             // S^WS | EOF | '!=' | '(' | '(:' | ')' | '*' | '*' | '+' | ',' | '-' | ':' | ':=' |
      break;
    default:
      lk = l1;
    }
    switch (lk)
    {
    case 17490:                     // 'attribute' '('
    case 17504:                     // 'comment' '('
    case 17528:                     // 'document-node' '('
    case 17529:                     // 'element' '('
    case 17593:                     // 'namespace-node' '('
    case 17599:                     // 'node' '('
    case 17624:                     // 'processing-instruction' '('
    case 17634:                     // 'schema-attribute' '('
    case 17635:                     // 'schema-element' '('
    case 17652:                     // 'text' '('
      parse_KindTest();
      break;
    case 17573:                     // 'item' '('
      shift(165);                   // 'item'
      lookahead1W(22);              // S^WS | '(' | '(:'
      shift(34);                    // '('
      lookahead1W(23);              // S^WS | '(:' | ')'
      shift(37);                    // ')'
      break;
    case 32:                        // '%'
    case 17553:                     // 'function' '('
      parse_FunctionTest();
      break;
    case 34:                        // '('
      parse_ParenthesizedItemType();
      break;
    case 17486:                     // 'array' '('
    case 17575:                     // 'json-item' '('
    case 17602:                     // 'object' '('
      parse_JSONTest();
      break;
    case 17650:                     // 'structured-item' '('
      parse_StructuredItemTest();
      break;
    default:
      parse_AtomicOrUnionType();
    }
    eventHandler.endNonterminal("ItemType", e0);
  }

  function try_ItemType()
  {
    switch (l1)
    {
    case 78:                        // 'array'
    case 82:                        // 'attribute'
    case 96:                        // 'comment'
    case 120:                       // 'document-node'
    case 121:                       // 'element'
    case 145:                       // 'function'
    case 165:                       // 'item'
    case 167:                       // 'json-item'
    case 185:                       // 'namespace-node'
    case 191:                       // 'node'
    case 194:                       // 'object'
    case 216:                       // 'processing-instruction'
    case 226:                       // 'schema-attribute'
    case 227:                       // 'schema-element'
    case 242:                       // 'structured-item'
    case 244:                       // 'text'
      lookahead2W(243);             // S^WS | EOF | '!=' | '(' | '(:' | ')' | '*' | '*' | '+' | ',' | '-' | ':' | ':=' |
      break;
    default:
      lk = l1;
    }
    switch (lk)
    {
    case 17490:                     // 'attribute' '('
    case 17504:                     // 'comment' '('
    case 17528:                     // 'document-node' '('
    case 17529:                     // 'element' '('
    case 17593:                     // 'namespace-node' '('
    case 17599:                     // 'node' '('
    case 17624:                     // 'processing-instruction' '('
    case 17634:                     // 'schema-attribute' '('
    case 17635:                     // 'schema-element' '('
    case 17652:                     // 'text' '('
      try_KindTest();
      break;
    case 17573:                     // 'item' '('
      shiftT(165);                  // 'item'
      lookahead1W(22);              // S^WS | '(' | '(:'
      shiftT(34);                   // '('
      lookahead1W(23);              // S^WS | '(:' | ')'
      shiftT(37);                   // ')'
      break;
    case 32:                        // '%'
    case 17553:                     // 'function' '('
      try_FunctionTest();
      break;
    case 34:                        // '('
      try_ParenthesizedItemType();
      break;
    case 17486:                     // 'array' '('
    case 17575:                     // 'json-item' '('
    case 17602:                     // 'object' '('
      try_JSONTest();
      break;
    case 17650:                     // 'structured-item' '('
      try_StructuredItemTest();
      break;
    default:
      try_AtomicOrUnionType();
    }
  }

  function parse_JSONTest()
  {
    eventHandler.startNonterminal("JSONTest", e0);
    switch (l1)
    {
    case 167:                       // 'json-item'
      parse_JSONItemTest();
      break;
    case 194:                       // 'object'
      parse_JSONObjectTest();
      break;
    default:
      parse_JSONArrayTest();
    }
    eventHandler.endNonterminal("JSONTest", e0);
  }

  function try_JSONTest()
  {
    switch (l1)
    {
    case 167:                       // 'json-item'
      try_JSONItemTest();
      break;
    case 194:                       // 'object'
      try_JSONObjectTest();
      break;
    default:
      try_JSONArrayTest();
    }
  }

  function parse_StructuredItemTest()
  {
    eventHandler.startNonterminal("StructuredItemTest", e0);
    shift(242);                     // 'structured-item'
    lookahead1W(22);                // S^WS | '(' | '(:'
    shift(34);                      // '('
    lookahead1W(23);                // S^WS | '(:' | ')'
    shift(37);                      // ')'
    eventHandler.endNonterminal("StructuredItemTest", e0);
  }

  function try_StructuredItemTest()
  {
    shiftT(242);                    // 'structured-item'
    lookahead1W(22);                // S^WS | '(' | '(:'
    shiftT(34);                     // '('
    lookahead1W(23);                // S^WS | '(:' | ')'
    shiftT(37);                     // ')'
  }

  function parse_JSONItemTest()
  {
    eventHandler.startNonterminal("JSONItemTest", e0);
    shift(167);                     // 'json-item'
    lookahead1W(22);                // S^WS | '(' | '(:'
    shift(34);                      // '('
    lookahead1W(23);                // S^WS | '(:' | ')'
    shift(37);                      // ')'
    eventHandler.endNonterminal("JSONItemTest", e0);
  }

  function try_JSONItemTest()
  {
    shiftT(167);                    // 'json-item'
    lookahead1W(22);                // S^WS | '(' | '(:'
    shiftT(34);                     // '('
    lookahead1W(23);                // S^WS | '(:' | ')'
    shiftT(37);                     // ')'
  }

  function parse_JSONObjectTest()
  {
    eventHandler.startNonterminal("JSONObjectTest", e0);
    shift(194);                     // 'object'
    lookahead1W(22);                // S^WS | '(' | '(:'
    shift(34);                      // '('
    lookahead1W(23);                // S^WS | '(:' | ')'
    shift(37);                      // ')'
    eventHandler.endNonterminal("JSONObjectTest", e0);
  }

  function try_JSONObjectTest()
  {
    shiftT(194);                    // 'object'
    lookahead1W(22);                // S^WS | '(' | '(:'
    shiftT(34);                     // '('
    lookahead1W(23);                // S^WS | '(:' | ')'
    shiftT(37);                     // ')'
  }

  function parse_JSONArrayTest()
  {
    eventHandler.startNonterminal("JSONArrayTest", e0);
    shift(78);                      // 'array'
    lookahead1W(22);                // S^WS | '(' | '(:'
    shift(34);                      // '('
    lookahead1W(23);                // S^WS | '(:' | ')'
    shift(37);                      // ')'
    eventHandler.endNonterminal("JSONArrayTest", e0);
  }

  function try_JSONArrayTest()
  {
    shiftT(78);                     // 'array'
    lookahead1W(22);                // S^WS | '(' | '(:'
    shiftT(34);                     // '('
    lookahead1W(23);                // S^WS | '(:' | ')'
    shiftT(37);                     // ')'
  }

  function parse_AtomicOrUnionType()
  {
    eventHandler.startNonterminal("AtomicOrUnionType", e0);
    parse_EQName();
    eventHandler.endNonterminal("AtomicOrUnionType", e0);
  }

  function try_AtomicOrUnionType()
  {
    try_EQName();
  }

  function parse_KindTest()
  {
    eventHandler.startNonterminal("KindTest", e0);
    switch (l1)
    {
    case 120:                       // 'document-node'
      parse_DocumentTest();
      break;
    case 121:                       // 'element'
      parse_ElementTest();
      break;
    case 82:                        // 'attribute'
      parse_AttributeTest();
      break;
    case 227:                       // 'schema-element'
      parse_SchemaElementTest();
      break;
    case 226:                       // 'schema-attribute'
      parse_SchemaAttributeTest();
      break;
    case 216:                       // 'processing-instruction'
      parse_PITest();
      break;
    case 96:                        // 'comment'
      parse_CommentTest();
      break;
    case 244:                       // 'text'
      parse_TextTest();
      break;
    case 185:                       // 'namespace-node'
      parse_NamespaceNodeTest();
      break;
    default:
      parse_AnyKindTest();
    }
    eventHandler.endNonterminal("KindTest", e0);
  }

  function try_KindTest()
  {
    switch (l1)
    {
    case 120:                       // 'document-node'
      try_DocumentTest();
      break;
    case 121:                       // 'element'
      try_ElementTest();
      break;
    case 82:                        // 'attribute'
      try_AttributeTest();
      break;
    case 227:                       // 'schema-element'
      try_SchemaElementTest();
      break;
    case 226:                       // 'schema-attribute'
      try_SchemaAttributeTest();
      break;
    case 216:                       // 'processing-instruction'
      try_PITest();
      break;
    case 96:                        // 'comment'
      try_CommentTest();
      break;
    case 244:                       // 'text'
      try_TextTest();
      break;
    case 185:                       // 'namespace-node'
      try_NamespaceNodeTest();
      break;
    default:
      try_AnyKindTest();
    }
  }

  function parse_AnyKindTest()
  {
    eventHandler.startNonterminal("AnyKindTest", e0);
    shift(191);                     // 'node'
    lookahead1W(22);                // S^WS | '(' | '(:'
    shift(34);                      // '('
    lookahead1W(23);                // S^WS | '(:' | ')'
    shift(37);                      // ')'
    eventHandler.endNonterminal("AnyKindTest", e0);
  }

  function try_AnyKindTest()
  {
    shiftT(191);                    // 'node'
    lookahead1W(22);                // S^WS | '(' | '(:'
    shiftT(34);                     // '('
    lookahead1W(23);                // S^WS | '(:' | ')'
    shiftT(37);                     // ')'
  }

  function parse_DocumentTest()
  {
    eventHandler.startNonterminal("DocumentTest", e0);
    shift(120);                     // 'document-node'
    lookahead1W(22);                // S^WS | '(' | '(:'
    shift(34);                      // '('
    lookahead1W(144);               // S^WS | '(:' | ')' | 'element' | 'schema-element'
    if (l1 != 37)                   // ')'
    {
      switch (l1)
      {
      case 121:                     // 'element'
        whitespace();
        parse_ElementTest();
        break;
      default:
        whitespace();
        parse_SchemaElementTest();
      }
    }
    lookahead1W(23);                // S^WS | '(:' | ')'
    shift(37);                      // ')'
    eventHandler.endNonterminal("DocumentTest", e0);
  }

  function try_DocumentTest()
  {
    shiftT(120);                    // 'document-node'
    lookahead1W(22);                // S^WS | '(' | '(:'
    shiftT(34);                     // '('
    lookahead1W(144);               // S^WS | '(:' | ')' | 'element' | 'schema-element'
    if (l1 != 37)                   // ')'
    {
      switch (l1)
      {
      case 121:                     // 'element'
        try_ElementTest();
        break;
      default:
        try_SchemaElementTest();
      }
    }
    lookahead1W(23);                // S^WS | '(:' | ')'
    shiftT(37);                     // ')'
  }

  function parse_TextTest()
  {
    eventHandler.startNonterminal("TextTest", e0);
    shift(244);                     // 'text'
    lookahead1W(22);                // S^WS | '(' | '(:'
    shift(34);                      // '('
    lookahead1W(23);                // S^WS | '(:' | ')'
    shift(37);                      // ')'
    eventHandler.endNonterminal("TextTest", e0);
  }

  function try_TextTest()
  {
    shiftT(244);                    // 'text'
    lookahead1W(22);                // S^WS | '(' | '(:'
    shiftT(34);                     // '('
    lookahead1W(23);                // S^WS | '(:' | ')'
    shiftT(37);                     // ')'
  }

  function parse_CommentTest()
  {
    eventHandler.startNonterminal("CommentTest", e0);
    shift(96);                      // 'comment'
    lookahead1W(22);                // S^WS | '(' | '(:'
    shift(34);                      // '('
    lookahead1W(23);                // S^WS | '(:' | ')'
    shift(37);                      // ')'
    eventHandler.endNonterminal("CommentTest", e0);
  }

  function try_CommentTest()
  {
    shiftT(96);                     // 'comment'
    lookahead1W(22);                // S^WS | '(' | '(:'
    shiftT(34);                     // '('
    lookahead1W(23);                // S^WS | '(:' | ')'
    shiftT(37);                     // ')'
  }

  function parse_NamespaceNodeTest()
  {
    eventHandler.startNonterminal("NamespaceNodeTest", e0);
    shift(185);                     // 'namespace-node'
    lookahead1W(22);                // S^WS | '(' | '(:'
    shift(34);                      // '('
    lookahead1W(23);                // S^WS | '(:' | ')'
    shift(37);                      // ')'
    eventHandler.endNonterminal("NamespaceNodeTest", e0);
  }

  function try_NamespaceNodeTest()
  {
    shiftT(185);                    // 'namespace-node'
    lookahead1W(22);                // S^WS | '(' | '(:'
    shiftT(34);                     // '('
    lookahead1W(23);                // S^WS | '(:' | ')'
    shiftT(37);                     // ')'
  }

  function parse_PITest()
  {
    eventHandler.startNonterminal("PITest", e0);
    shift(216);                     // 'processing-instruction'
    lookahead1W(22);                // S^WS | '(' | '(:'
    shift(34);                      // '('
    lookahead1W(253);               // StringLiteral | NCName^Token | S^WS | '(:' | ')' | 'after' | 'allowing' |
    if (l1 != 37)                   // ')'
    {
      switch (l1)
      {
      case 11:                      // StringLiteral
        shift(11);                  // StringLiteral
        break;
      default:
        whitespace();
        parse_NCName();
      }
    }
    lookahead1W(23);                // S^WS | '(:' | ')'
    shift(37);                      // ')'
    eventHandler.endNonterminal("PITest", e0);
  }

  function try_PITest()
  {
    shiftT(216);                    // 'processing-instruction'
    lookahead1W(22);                // S^WS | '(' | '(:'
    shiftT(34);                     // '('
    lookahead1W(253);               // StringLiteral | NCName^Token | S^WS | '(:' | ')' | 'after' | 'allowing' |
    if (l1 != 37)                   // ')'
    {
      switch (l1)
      {
      case 11:                      // StringLiteral
        shiftT(11);                 // StringLiteral
        break;
      default:
        try_NCName();
      }
    }
    lookahead1W(23);                // S^WS | '(:' | ')'
    shiftT(37);                     // ')'
  }

  function parse_AttributeTest()
  {
    eventHandler.startNonterminal("AttributeTest", e0);
    shift(82);                      // 'attribute'
    lookahead1W(22);                // S^WS | '(' | '(:'
    shift(34);                      // '('
    lookahead1W(261);               // EQName^Token | S^WS | '(:' | ')' | '*' | 'after' | 'allowing' | 'ancestor' |
    if (l1 != 37)                   // ')'
    {
      whitespace();
      parse_AttribNameOrWildcard();
      lookahead1W(101);             // S^WS | '(:' | ')' | ','
      if (l1 == 41)                 // ','
      {
        shift(41);                  // ','
        lookahead1W(255);           // EQName^Token | S^WS | '(:' | 'after' | 'allowing' | 'ancestor' |
        whitespace();
        parse_TypeName();
      }
    }
    lookahead1W(23);                // S^WS | '(:' | ')'
    shift(37);                      // ')'
    eventHandler.endNonterminal("AttributeTest", e0);
  }

  function try_AttributeTest()
  {
    shiftT(82);                     // 'attribute'
    lookahead1W(22);                // S^WS | '(' | '(:'
    shiftT(34);                     // '('
    lookahead1W(261);               // EQName^Token | S^WS | '(:' | ')' | '*' | 'after' | 'allowing' | 'ancestor' |
    if (l1 != 37)                   // ')'
    {
      try_AttribNameOrWildcard();
      lookahead1W(101);             // S^WS | '(:' | ')' | ','
      if (l1 == 41)                 // ','
      {
        shiftT(41);                 // ','
        lookahead1W(255);           // EQName^Token | S^WS | '(:' | 'after' | 'allowing' | 'ancestor' |
        try_TypeName();
      }
    }
    lookahead1W(23);                // S^WS | '(:' | ')'
    shiftT(37);                     // ')'
  }

  function parse_AttribNameOrWildcard()
  {
    eventHandler.startNonterminal("AttribNameOrWildcard", e0);
    switch (l1)
    {
    case 38:                        // '*'
      shift(38);                    // '*'
      break;
    default:
      parse_AttributeName();
    }
    eventHandler.endNonterminal("AttribNameOrWildcard", e0);
  }

  function try_AttribNameOrWildcard()
  {
    switch (l1)
    {
    case 38:                        // '*'
      shiftT(38);                   // '*'
      break;
    default:
      try_AttributeName();
    }
  }

  function parse_SchemaAttributeTest()
  {
    eventHandler.startNonterminal("SchemaAttributeTest", e0);
    shift(226);                     // 'schema-attribute'
    lookahead1W(22);                // S^WS | '(' | '(:'
    shift(34);                      // '('
    lookahead1W(255);               // EQName^Token | S^WS | '(:' | 'after' | 'allowing' | 'ancestor' |
    whitespace();
    parse_AttributeDeclaration();
    lookahead1W(23);                // S^WS | '(:' | ')'
    shift(37);                      // ')'
    eventHandler.endNonterminal("SchemaAttributeTest", e0);
  }

  function try_SchemaAttributeTest()
  {
    shiftT(226);                    // 'schema-attribute'
    lookahead1W(22);                // S^WS | '(' | '(:'
    shiftT(34);                     // '('
    lookahead1W(255);               // EQName^Token | S^WS | '(:' | 'after' | 'allowing' | 'ancestor' |
    try_AttributeDeclaration();
    lookahead1W(23);                // S^WS | '(:' | ')'
    shiftT(37);                     // ')'
  }

  function parse_AttributeDeclaration()
  {
    eventHandler.startNonterminal("AttributeDeclaration", e0);
    parse_AttributeName();
    eventHandler.endNonterminal("AttributeDeclaration", e0);
  }

  function try_AttributeDeclaration()
  {
    try_AttributeName();
  }

  function parse_ElementTest()
  {
    eventHandler.startNonterminal("ElementTest", e0);
    shift(121);                     // 'element'
    lookahead1W(22);                // S^WS | '(' | '(:'
    shift(34);                      // '('
    lookahead1W(261);               // EQName^Token | S^WS | '(:' | ')' | '*' | 'after' | 'allowing' | 'ancestor' |
    if (l1 != 37)                   // ')'
    {
      whitespace();
      parse_ElementNameOrWildcard();
      lookahead1W(101);             // S^WS | '(:' | ')' | ','
      if (l1 == 41)                 // ','
      {
        shift(41);                  // ','
        lookahead1W(255);           // EQName^Token | S^WS | '(:' | 'after' | 'allowing' | 'ancestor' |
        whitespace();
        parse_TypeName();
        lookahead1W(102);           // S^WS | '(:' | ')' | '?'
        if (l1 == 64)               // '?'
        {
          shift(64);                // '?'
        }
      }
    }
    lookahead1W(23);                // S^WS | '(:' | ')'
    shift(37);                      // ')'
    eventHandler.endNonterminal("ElementTest", e0);
  }

  function try_ElementTest()
  {
    shiftT(121);                    // 'element'
    lookahead1W(22);                // S^WS | '(' | '(:'
    shiftT(34);                     // '('
    lookahead1W(261);               // EQName^Token | S^WS | '(:' | ')' | '*' | 'after' | 'allowing' | 'ancestor' |
    if (l1 != 37)                   // ')'
    {
      try_ElementNameOrWildcard();
      lookahead1W(101);             // S^WS | '(:' | ')' | ','
      if (l1 == 41)                 // ','
      {
        shiftT(41);                 // ','
        lookahead1W(255);           // EQName^Token | S^WS | '(:' | 'after' | 'allowing' | 'ancestor' |
        try_TypeName();
        lookahead1W(102);           // S^WS | '(:' | ')' | '?'
        if (l1 == 64)               // '?'
        {
          shiftT(64);               // '?'
        }
      }
    }
    lookahead1W(23);                // S^WS | '(:' | ')'
    shiftT(37);                     // ')'
  }

  function parse_ElementNameOrWildcard()
  {
    eventHandler.startNonterminal("ElementNameOrWildcard", e0);
    switch (l1)
    {
    case 38:                        // '*'
      shift(38);                    // '*'
      break;
    default:
      parse_ElementName();
    }
    eventHandler.endNonterminal("ElementNameOrWildcard", e0);
  }

  function try_ElementNameOrWildcard()
  {
    switch (l1)
    {
    case 38:                        // '*'
      shiftT(38);                   // '*'
      break;
    default:
      try_ElementName();
    }
  }

  function parse_SchemaElementTest()
  {
    eventHandler.startNonterminal("SchemaElementTest", e0);
    shift(227);                     // 'schema-element'
    lookahead1W(22);                // S^WS | '(' | '(:'
    shift(34);                      // '('
    lookahead1W(255);               // EQName^Token | S^WS | '(:' | 'after' | 'allowing' | 'ancestor' |
    whitespace();
    parse_ElementDeclaration();
    lookahead1W(23);                // S^WS | '(:' | ')'
    shift(37);                      // ')'
    eventHandler.endNonterminal("SchemaElementTest", e0);
  }

  function try_SchemaElementTest()
  {
    shiftT(227);                    // 'schema-element'
    lookahead1W(22);                // S^WS | '(' | '(:'
    shiftT(34);                     // '('
    lookahead1W(255);               // EQName^Token | S^WS | '(:' | 'after' | 'allowing' | 'ancestor' |
    try_ElementDeclaration();
    lookahead1W(23);                // S^WS | '(:' | ')'
    shiftT(37);                     // ')'
  }

  function parse_ElementDeclaration()
  {
    eventHandler.startNonterminal("ElementDeclaration", e0);
    parse_ElementName();
    eventHandler.endNonterminal("ElementDeclaration", e0);
  }

  function try_ElementDeclaration()
  {
    try_ElementName();
  }

  function parse_AttributeName()
  {
    eventHandler.startNonterminal("AttributeName", e0);
    parse_EQName();
    eventHandler.endNonterminal("AttributeName", e0);
  }

  function try_AttributeName()
  {
    try_EQName();
  }

  function parse_ElementName()
  {
    eventHandler.startNonterminal("ElementName", e0);
    parse_EQName();
    eventHandler.endNonterminal("ElementName", e0);
  }

  function try_ElementName()
  {
    try_EQName();
  }

  function parse_SimpleTypeName()
  {
    eventHandler.startNonterminal("SimpleTypeName", e0);
    parse_TypeName();
    eventHandler.endNonterminal("SimpleTypeName", e0);
  }

  function try_SimpleTypeName()
  {
    try_TypeName();
  }

  function parse_TypeName()
  {
    eventHandler.startNonterminal("TypeName", e0);
    parse_EQName();
    eventHandler.endNonterminal("TypeName", e0);
  }

  function try_TypeName()
  {
    try_EQName();
  }

  function parse_FunctionTest()
  {
    eventHandler.startNonterminal("FunctionTest", e0);
    for (;;)
    {
      lookahead1W(97);              // S^WS | '%' | '(:' | 'function'
      if (l1 != 32)                 // '%'
      {
        break;
      }
      whitespace();
      parse_Annotation();
    }
    switch (l1)
    {
    case 145:                       // 'function'
      lookahead2W(22);              // S^WS | '(' | '(:'
      break;
    default:
      lk = l1;
    }
    lk = memoized(5, e0);
    if (lk == 0)
    {
      var b0A = b0; var e0A = e0; var l1A = l1;
      var b1A = b1; var e1A = e1; var l2A = l2;
      var b2A = b2; var e2A = e2;
      try
      {
        try_AnyFunctionTest();
        lk = -1;
      }
      catch (p1A)
      {
        lk = -2;
      }
      b0 = b0A; e0 = e0A; l1 = l1A; if (l1 == 0) {end = e0A;} else {
      b1 = b1A; e1 = e1A; l2 = l2A; if (l2 == 0) {end = e1A;} else {
      b2 = b2A; e2 = e2A; end = e2A; }}
      memoize(5, e0, lk);
    }
    switch (lk)
    {
    case -1:
      whitespace();
      parse_AnyFunctionTest();
      break;
    default:
      whitespace();
      parse_TypedFunctionTest();
    }
    eventHandler.endNonterminal("FunctionTest", e0);
  }

  function try_FunctionTest()
  {
    for (;;)
    {
      lookahead1W(97);              // S^WS | '%' | '(:' | 'function'
      if (l1 != 32)                 // '%'
      {
        break;
      }
      try_Annotation();
    }
    switch (l1)
    {
    case 145:                       // 'function'
      lookahead2W(22);              // S^WS | '(' | '(:'
      break;
    default:
      lk = l1;
    }
    lk = memoized(5, e0);
    if (lk == 0)
    {
      var b0A = b0; var e0A = e0; var l1A = l1;
      var b1A = b1; var e1A = e1; var l2A = l2;
      var b2A = b2; var e2A = e2;
      try
      {
        try_AnyFunctionTest();
        memoize(5, e0A, -1);
        lk = -3;
      }
      catch (p1A)
      {
        lk = -2;
        b0 = b0A; e0 = e0A; l1 = l1A; if (l1 == 0) {end = e0A;} else {
        b1 = b1A; e1 = e1A; l2 = l2A; if (l2 == 0) {end = e1A;} else {
        b2 = b2A; e2 = e2A; end = e2A; }}
        memoize(5, e0A, -2);
      }
    }
    switch (lk)
    {
    case -1:
      try_AnyFunctionTest();
      break;
    case -3:
      break;
    default:
      try_TypedFunctionTest();
    }
  }

  function parse_AnyFunctionTest()
  {
    eventHandler.startNonterminal("AnyFunctionTest", e0);
    shift(145);                     // 'function'
    lookahead1W(22);                // S^WS | '(' | '(:'
    shift(34);                      // '('
    lookahead1W(24);                // S^WS | '(:' | '*'
    shift(38);                      // '*'
    lookahead1W(23);                // S^WS | '(:' | ')'
    shift(37);                      // ')'
    eventHandler.endNonterminal("AnyFunctionTest", e0);
  }

  function try_AnyFunctionTest()
  {
    shiftT(145);                    // 'function'
    lookahead1W(22);                // S^WS | '(' | '(:'
    shiftT(34);                     // '('
    lookahead1W(24);                // S^WS | '(:' | '*'
    shiftT(38);                     // '*'
    lookahead1W(23);                // S^WS | '(:' | ')'
    shiftT(37);                     // ')'
  }

  function parse_TypedFunctionTest()
  {
    eventHandler.startNonterminal("TypedFunctionTest", e0);
    shift(145);                     // 'function'
    lookahead1W(22);                // S^WS | '(' | '(:'
    shift(34);                      // '('
    lookahead1W(263);               // EQName^Token | S^WS | '%' | '(' | '(:' | ')' | 'after' | 'allowing' |
    if (l1 != 37)                   // ')'
    {
      whitespace();
      parse_SequenceType();
      for (;;)
      {
        lookahead1W(101);           // S^WS | '(:' | ')' | ','
        if (l1 != 41)               // ','
        {
          break;
        }
        shift(41);                  // ','
        lookahead1W(260);           // EQName^Token | S^WS | '%' | '(' | '(:' | 'after' | 'allowing' | 'ancestor' |
        whitespace();
        parse_SequenceType();
      }
    }
    shift(37);                      // ')'
    lookahead1W(30);                // S^WS | '(:' | 'as'
    shift(79);                      // 'as'
    lookahead1W(260);               // EQName^Token | S^WS | '%' | '(' | '(:' | 'after' | 'allowing' | 'ancestor' |
    whitespace();
    parse_SequenceType();
    eventHandler.endNonterminal("TypedFunctionTest", e0);
  }

  function try_TypedFunctionTest()
  {
    shiftT(145);                    // 'function'
    lookahead1W(22);                // S^WS | '(' | '(:'
    shiftT(34);                     // '('
    lookahead1W(263);               // EQName^Token | S^WS | '%' | '(' | '(:' | ')' | 'after' | 'allowing' |
    if (l1 != 37)                   // ')'
    {
      try_SequenceType();
      for (;;)
      {
        lookahead1W(101);           // S^WS | '(:' | ')' | ','
        if (l1 != 41)               // ','
        {
          break;
        }
        shiftT(41);                 // ','
        lookahead1W(260);           // EQName^Token | S^WS | '%' | '(' | '(:' | 'after' | 'allowing' | 'ancestor' |
        try_SequenceType();
      }
    }
    shiftT(37);                     // ')'
    lookahead1W(30);                // S^WS | '(:' | 'as'
    shiftT(79);                     // 'as'
    lookahead1W(260);               // EQName^Token | S^WS | '%' | '(' | '(:' | 'after' | 'allowing' | 'ancestor' |
    try_SequenceType();
  }

  function parse_ParenthesizedItemType()
  {
    eventHandler.startNonterminal("ParenthesizedItemType", e0);
    shift(34);                      // '('
    lookahead1W(260);               // EQName^Token | S^WS | '%' | '(' | '(:' | 'after' | 'allowing' | 'ancestor' |
    whitespace();
    parse_ItemType();
    lookahead1W(23);                // S^WS | '(:' | ')'
    shift(37);                      // ')'
    eventHandler.endNonterminal("ParenthesizedItemType", e0);
  }

  function try_ParenthesizedItemType()
  {
    shiftT(34);                     // '('
    lookahead1W(260);               // EQName^Token | S^WS | '%' | '(' | '(:' | 'after' | 'allowing' | 'ancestor' |
    try_ItemType();
    lookahead1W(23);                // S^WS | '(:' | ')'
    shiftT(37);                     // ')'
  }

  function parse_RevalidationDecl()
  {
    eventHandler.startNonterminal("RevalidationDecl", e0);
    shift(108);                     // 'declare'
    lookahead1W(72);                // S^WS | '(:' | 'revalidation'
    shift(222);                     // 'revalidation'
    lookahead1W(152);               // S^WS | '(:' | 'lax' | 'skip' | 'strict'
    switch (l1)
    {
    case 240:                       // 'strict'
      shift(240);                   // 'strict'
      break;
    case 171:                       // 'lax'
      shift(171);                   // 'lax'
      break;
    default:
      shift(233);                   // 'skip'
    }
    eventHandler.endNonterminal("RevalidationDecl", e0);
  }

  function parse_InsertExprTargetChoice()
  {
    eventHandler.startNonterminal("InsertExprTargetChoice", e0);
    switch (l1)
    {
    case 70:                        // 'after'
      shift(70);                    // 'after'
      break;
    case 84:                        // 'before'
      shift(84);                    // 'before'
      break;
    default:
      if (l1 == 79)                 // 'as'
      {
        shift(79);                  // 'as'
        lookahead1W(119);           // S^WS | '(:' | 'first' | 'last'
        switch (l1)
        {
        case 134:                   // 'first'
          shift(134);               // 'first'
          break;
        default:
          shift(170);               // 'last'
        }
      }
      lookahead1W(54);              // S^WS | '(:' | 'into'
      shift(163);                   // 'into'
    }
    eventHandler.endNonterminal("InsertExprTargetChoice", e0);
  }

  function try_InsertExprTargetChoice()
  {
    switch (l1)
    {
    case 70:                        // 'after'
      shiftT(70);                   // 'after'
      break;
    case 84:                        // 'before'
      shiftT(84);                   // 'before'
      break;
    default:
      if (l1 == 79)                 // 'as'
      {
        shiftT(79);                 // 'as'
        lookahead1W(119);           // S^WS | '(:' | 'first' | 'last'
        switch (l1)
        {
        case 134:                   // 'first'
          shiftT(134);              // 'first'
          break;
        default:
          shiftT(170);              // 'last'
        }
      }
      lookahead1W(54);              // S^WS | '(:' | 'into'
      shiftT(163);                  // 'into'
    }
  }

  function parse_InsertExpr()
  {
    eventHandler.startNonterminal("InsertExpr", e0);
    shift(159);                     // 'insert'
    lookahead1W(129);               // S^WS | '(:' | 'node' | 'nodes'
    switch (l1)
    {
    case 191:                       // 'node'
      shift(191);                   // 'node'
      break;
    default:
      shift(192);                   // 'nodes'
    }
    lookahead1W(267);               // Wildcard | EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral |
    whitespace();
    parse_SourceExpr();
    whitespace();
    parse_InsertExprTargetChoice();
    lookahead1W(267);               // Wildcard | EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral |
    whitespace();
    parse_TargetExpr();
    eventHandler.endNonterminal("InsertExpr", e0);
  }

  function try_InsertExpr()
  {
    shiftT(159);                    // 'insert'
    lookahead1W(129);               // S^WS | '(:' | 'node' | 'nodes'
    switch (l1)
    {
    case 191:                       // 'node'
      shiftT(191);                  // 'node'
      break;
    default:
      shiftT(192);                  // 'nodes'
    }
    lookahead1W(267);               // Wildcard | EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral |
    try_SourceExpr();
    try_InsertExprTargetChoice();
    lookahead1W(267);               // Wildcard | EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral |
    try_TargetExpr();
  }

  function parse_DeleteExpr()
  {
    eventHandler.startNonterminal("DeleteExpr", e0);
    shift(110);                     // 'delete'
    lookahead1W(129);               // S^WS | '(:' | 'node' | 'nodes'
    switch (l1)
    {
    case 191:                       // 'node'
      shift(191);                   // 'node'
      break;
    default:
      shift(192);                   // 'nodes'
    }
    lookahead1W(267);               // Wildcard | EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral |
    whitespace();
    parse_TargetExpr();
    eventHandler.endNonterminal("DeleteExpr", e0);
  }

  function try_DeleteExpr()
  {
    shiftT(110);                    // 'delete'
    lookahead1W(129);               // S^WS | '(:' | 'node' | 'nodes'
    switch (l1)
    {
    case 191:                       // 'node'
      shiftT(191);                  // 'node'
      break;
    default:
      shiftT(192);                  // 'nodes'
    }
    lookahead1W(267);               // Wildcard | EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral |
    try_TargetExpr();
  }

  function parse_ReplaceExpr()
  {
    eventHandler.startNonterminal("ReplaceExpr", e0);
    shift(219);                     // 'replace'
    lookahead1W(130);               // S^WS | '(:' | 'node' | 'value'
    if (l1 == 261)                  // 'value'
    {
      shift(261);                   // 'value'
      lookahead1W(64);              // S^WS | '(:' | 'of'
      shift(196);                   // 'of'
    }
    lookahead1W(62);                // S^WS | '(:' | 'node'
    shift(191);                     // 'node'
    lookahead1W(267);               // Wildcard | EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral |
    whitespace();
    parse_TargetExpr();
    shift(270);                     // 'with'
    lookahead1W(267);               // Wildcard | EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral |
    whitespace();
    parse_ExprSingle();
    eventHandler.endNonterminal("ReplaceExpr", e0);
  }

  function try_ReplaceExpr()
  {
    shiftT(219);                    // 'replace'
    lookahead1W(130);               // S^WS | '(:' | 'node' | 'value'
    if (l1 == 261)                  // 'value'
    {
      shiftT(261);                  // 'value'
      lookahead1W(64);              // S^WS | '(:' | 'of'
      shiftT(196);                  // 'of'
    }
    lookahead1W(62);                // S^WS | '(:' | 'node'
    shiftT(191);                    // 'node'
    lookahead1W(267);               // Wildcard | EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral |
    try_TargetExpr();
    shiftT(270);                    // 'with'
    lookahead1W(267);               // Wildcard | EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral |
    try_ExprSingle();
  }

  function parse_RenameExpr()
  {
    eventHandler.startNonterminal("RenameExpr", e0);
    shift(218);                     // 'rename'
    lookahead1W(62);                // S^WS | '(:' | 'node'
    shift(191);                     // 'node'
    lookahead1W(267);               // Wildcard | EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral |
    whitespace();
    parse_TargetExpr();
    shift(79);                      // 'as'
    lookahead1W(267);               // Wildcard | EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral |
    whitespace();
    parse_NewNameExpr();
    eventHandler.endNonterminal("RenameExpr", e0);
  }

  function try_RenameExpr()
  {
    shiftT(218);                    // 'rename'
    lookahead1W(62);                // S^WS | '(:' | 'node'
    shiftT(191);                    // 'node'
    lookahead1W(267);               // Wildcard | EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral |
    try_TargetExpr();
    shiftT(79);                     // 'as'
    lookahead1W(267);               // Wildcard | EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral |
    try_NewNameExpr();
  }

  function parse_SourceExpr()
  {
    eventHandler.startNonterminal("SourceExpr", e0);
    parse_ExprSingle();
    eventHandler.endNonterminal("SourceExpr", e0);
  }

  function try_SourceExpr()
  {
    try_ExprSingle();
  }

  function parse_TargetExpr()
  {
    eventHandler.startNonterminal("TargetExpr", e0);
    parse_ExprSingle();
    eventHandler.endNonterminal("TargetExpr", e0);
  }

  function try_TargetExpr()
  {
    try_ExprSingle();
  }

  function parse_NewNameExpr()
  {
    eventHandler.startNonterminal("NewNameExpr", e0);
    parse_ExprSingle();
    eventHandler.endNonterminal("NewNameExpr", e0);
  }

  function try_NewNameExpr()
  {
    try_ExprSingle();
  }

  function parse_TransformExpr()
  {
    eventHandler.startNonterminal("TransformExpr", e0);
    shift(103);                     // 'copy'
    lookahead1W(21);                // S^WS | '$' | '(:'
    whitespace();
    parse_TransformSpec();
    for (;;)
    {
      if (l1 != 41)                 // ','
      {
        break;
      }
      shift(41);                    // ','
      lookahead1W(21);              // S^WS | '$' | '(:'
      whitespace();
      parse_TransformSpec();
    }
    shift(181);                     // 'modify'
    lookahead1W(267);               // Wildcard | EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral |
    whitespace();
    parse_ExprSingle();
    shift(220);                     // 'return'
    lookahead1W(267);               // Wildcard | EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral |
    whitespace();
    parse_ExprSingle();
    eventHandler.endNonterminal("TransformExpr", e0);
  }

  function try_TransformExpr()
  {
    shiftT(103);                    // 'copy'
    lookahead1W(21);                // S^WS | '$' | '(:'
    try_TransformSpec();
    for (;;)
    {
      if (l1 != 41)                 // ','
      {
        break;
      }
      shiftT(41);                   // ','
      lookahead1W(21);              // S^WS | '$' | '(:'
      try_TransformSpec();
    }
    shiftT(181);                    // 'modify'
    lookahead1W(267);               // Wildcard | EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral |
    try_ExprSingle();
    shiftT(220);                    // 'return'
    lookahead1W(267);               // Wildcard | EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral |
    try_ExprSingle();
  }

  function parse_TransformSpec()
  {
    eventHandler.startNonterminal("TransformSpec", e0);
    shift(31);                      // '$'
    lookahead1W(255);               // EQName^Token | S^WS | '(:' | 'after' | 'allowing' | 'ancestor' |
    whitespace();
    parse_VarName();
    lookahead1W(27);                // S^WS | '(:' | ':='
    shift(52);                      // ':='
    lookahead1W(267);               // Wildcard | EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral |
    whitespace();
    parse_ExprSingle();
    eventHandler.endNonterminal("TransformSpec", e0);
  }

  function try_TransformSpec()
  {
    shiftT(31);                     // '$'
    lookahead1W(255);               // EQName^Token | S^WS | '(:' | 'after' | 'allowing' | 'ancestor' |
    try_VarName();
    lookahead1W(27);                // S^WS | '(:' | ':='
    shiftT(52);                     // ':='
    lookahead1W(267);               // Wildcard | EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral |
    try_ExprSingle();
  }

  function parse_FTSelection()
  {
    eventHandler.startNonterminal("FTSelection", e0);
    parse_FTOr();
    for (;;)
    {
      lookahead1W(212);             // S^WS | EOF | '!=' | '(:' | ')' | ',' | ':' | ';' | '<' | '<<' | '<=' | '=' |
      switch (l1)
      {
      case 81:                      // 'at'
        lookahead2W(151);           // S^WS | '(:' | 'end' | 'position' | 'start'
        break;
      default:
        lk = l1;
      }
      if (lk != 115                 // 'different'
       && lk != 117                 // 'distance'
       && lk != 127                 // 'entire'
       && lk != 202                 // 'ordered'
       && lk != 223                 // 'same'
       && lk != 269                 // 'window'
       && lk != 64593               // 'at' 'end'
       && lk != 121425)             // 'at' 'start'
      {
        break;
      }
      whitespace();
      parse_FTPosFilter();
    }
    eventHandler.endNonterminal("FTSelection", e0);
  }

  function try_FTSelection()
  {
    try_FTOr();
    for (;;)
    {
      lookahead1W(212);             // S^WS | EOF | '!=' | '(:' | ')' | ',' | ':' | ';' | '<' | '<<' | '<=' | '=' |
      switch (l1)
      {
      case 81:                      // 'at'
        lookahead2W(151);           // S^WS | '(:' | 'end' | 'position' | 'start'
        break;
      default:
        lk = l1;
      }
      if (lk != 115                 // 'different'
       && lk != 117                 // 'distance'
       && lk != 127                 // 'entire'
       && lk != 202                 // 'ordered'
       && lk != 223                 // 'same'
       && lk != 269                 // 'window'
       && lk != 64593               // 'at' 'end'
       && lk != 121425)             // 'at' 'start'
      {
        break;
      }
      try_FTPosFilter();
    }
  }

  function parse_FTWeight()
  {
    eventHandler.startNonterminal("FTWeight", e0);
    shift(264);                     // 'weight'
    lookahead1W(87);                // S^WS | '(:' | '{'
    shift(276);                     // '{'
    lookahead1W(267);               // Wildcard | EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral |
    whitespace();
    parse_Expr();
    shift(282);                     // '}'
    eventHandler.endNonterminal("FTWeight", e0);
  }

  function try_FTWeight()
  {
    shiftT(264);                    // 'weight'
    lookahead1W(87);                // S^WS | '(:' | '{'
    shiftT(276);                    // '{'
    lookahead1W(267);               // Wildcard | EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral |
    try_Expr();
    shiftT(282);                    // '}'
  }

  function parse_FTOr()
  {
    eventHandler.startNonterminal("FTOr", e0);
    parse_FTAnd();
    for (;;)
    {
      if (l1 != 144)                // 'ftor'
      {
        break;
      }
      shift(144);                   // 'ftor'
      lookahead1W(162);             // StringLiteral | S^WS | '(' | '(#' | '(:' | 'ftnot' | '{'
      whitespace();
      parse_FTAnd();
    }
    eventHandler.endNonterminal("FTOr", e0);
  }

  function try_FTOr()
  {
    try_FTAnd();
    for (;;)
    {
      if (l1 != 144)                // 'ftor'
      {
        break;
      }
      shiftT(144);                  // 'ftor'
      lookahead1W(162);             // StringLiteral | S^WS | '(' | '(#' | '(:' | 'ftnot' | '{'
      try_FTAnd();
    }
  }

  function parse_FTAnd()
  {
    eventHandler.startNonterminal("FTAnd", e0);
    parse_FTMildNot();
    for (;;)
    {
      if (l1 != 142)                // 'ftand'
      {
        break;
      }
      shift(142);                   // 'ftand'
      lookahead1W(162);             // StringLiteral | S^WS | '(' | '(#' | '(:' | 'ftnot' | '{'
      whitespace();
      parse_FTMildNot();
    }
    eventHandler.endNonterminal("FTAnd", e0);
  }

  function try_FTAnd()
  {
    try_FTMildNot();
    for (;;)
    {
      if (l1 != 142)                // 'ftand'
      {
        break;
      }
      shiftT(142);                  // 'ftand'
      lookahead1W(162);             // StringLiteral | S^WS | '(' | '(#' | '(:' | 'ftnot' | '{'
      try_FTMildNot();
    }
  }

  function parse_FTMildNot()
  {
    eventHandler.startNonterminal("FTMildNot", e0);
    parse_FTUnaryNot();
    for (;;)
    {
      lookahead1W(213);             // S^WS | EOF | '!=' | '(:' | ')' | ',' | ':' | ';' | '<' | '<<' | '<=' | '=' |
      if (l1 != 193)                // 'not'
      {
        break;
      }
      shift(193);                   // 'not'
      lookahead1W(53);              // S^WS | '(:' | 'in'
      shift(154);                   // 'in'
      lookahead1W(162);             // StringLiteral | S^WS | '(' | '(#' | '(:' | 'ftnot' | '{'
      whitespace();
      parse_FTUnaryNot();
    }
    eventHandler.endNonterminal("FTMildNot", e0);
  }

  function try_FTMildNot()
  {
    try_FTUnaryNot();
    for (;;)
    {
      lookahead1W(213);             // S^WS | EOF | '!=' | '(:' | ')' | ',' | ':' | ';' | '<' | '<<' | '<=' | '=' |
      if (l1 != 193)                // 'not'
      {
        break;
      }
      shiftT(193);                  // 'not'
      lookahead1W(53);              // S^WS | '(:' | 'in'
      shiftT(154);                  // 'in'
      lookahead1W(162);             // StringLiteral | S^WS | '(' | '(#' | '(:' | 'ftnot' | '{'
      try_FTUnaryNot();
    }
  }

  function parse_FTUnaryNot()
  {
    eventHandler.startNonterminal("FTUnaryNot", e0);
    if (l1 == 143)                  // 'ftnot'
    {
      shift(143);                   // 'ftnot'
    }
    lookahead1W(155);               // StringLiteral | S^WS | '(' | '(#' | '(:' | '{'
    whitespace();
    parse_FTPrimaryWithOptions();
    eventHandler.endNonterminal("FTUnaryNot", e0);
  }

  function try_FTUnaryNot()
  {
    if (l1 == 143)                  // 'ftnot'
    {
      shiftT(143);                  // 'ftnot'
    }
    lookahead1W(155);               // StringLiteral | S^WS | '(' | '(#' | '(:' | '{'
    try_FTPrimaryWithOptions();
  }

  function parse_FTPrimaryWithOptions()
  {
    eventHandler.startNonterminal("FTPrimaryWithOptions", e0);
    parse_FTPrimary();
    lookahead1W(215);               // S^WS | EOF | '!=' | '(:' | ')' | ',' | ':' | ';' | '<' | '<<' | '<=' | '=' |
    if (l1 == 259)                  // 'using'
    {
      whitespace();
      parse_FTMatchOptions();
    }
    if (l1 == 264)                  // 'weight'
    {
      whitespace();
      parse_FTWeight();
    }
    eventHandler.endNonterminal("FTPrimaryWithOptions", e0);
  }

  function try_FTPrimaryWithOptions()
  {
    try_FTPrimary();
    lookahead1W(215);               // S^WS | EOF | '!=' | '(:' | ')' | ',' | ':' | ';' | '<' | '<<' | '<=' | '=' |
    if (l1 == 259)                  // 'using'
    {
      try_FTMatchOptions();
    }
    if (l1 == 264)                  // 'weight'
    {
      try_FTWeight();
    }
  }

  function parse_FTPrimary()
  {
    eventHandler.startNonterminal("FTPrimary", e0);
    switch (l1)
    {
    case 34:                        // '('
      shift(34);                    // '('
      lookahead1W(162);             // StringLiteral | S^WS | '(' | '(#' | '(:' | 'ftnot' | '{'
      whitespace();
      parse_FTSelection();
      shift(37);                    // ')'
      break;
    case 35:                        // '(#'
      parse_FTExtensionSelection();
      break;
    default:
      parse_FTWords();
      lookahead1W(216);             // S^WS | EOF | '!=' | '(:' | ')' | ',' | ':' | ';' | '<' | '<<' | '<=' | '=' |
      if (l1 == 195)                // 'occurs'
      {
        whitespace();
        parse_FTTimes();
      }
    }
    eventHandler.endNonterminal("FTPrimary", e0);
  }

  function try_FTPrimary()
  {
    switch (l1)
    {
    case 34:                        // '('
      shiftT(34);                   // '('
      lookahead1W(162);             // StringLiteral | S^WS | '(' | '(#' | '(:' | 'ftnot' | '{'
      try_FTSelection();
      shiftT(37);                   // ')'
      break;
    case 35:                        // '(#'
      try_FTExtensionSelection();
      break;
    default:
      try_FTWords();
      lookahead1W(216);             // S^WS | EOF | '!=' | '(:' | ')' | ',' | ':' | ';' | '<' | '<<' | '<=' | '=' |
      if (l1 == 195)                // 'occurs'
      {
        try_FTTimes();
      }
    }
  }

  function parse_FTWords()
  {
    eventHandler.startNonterminal("FTWords", e0);
    parse_FTWordsValue();
    lookahead1W(222);               // S^WS | EOF | '!=' | '(:' | ')' | ',' | ':' | ';' | '<' | '<<' | '<=' | '=' |
    if (l1 == 71                    // 'all'
     || l1 == 76                    // 'any'
     || l1 == 210)                  // 'phrase'
    {
      whitespace();
      parse_FTAnyallOption();
    }
    eventHandler.endNonterminal("FTWords", e0);
  }

  function try_FTWords()
  {
    try_FTWordsValue();
    lookahead1W(222);               // S^WS | EOF | '!=' | '(:' | ')' | ',' | ':' | ';' | '<' | '<<' | '<=' | '=' |
    if (l1 == 71                    // 'all'
     || l1 == 76                    // 'any'
     || l1 == 210)                  // 'phrase'
    {
      try_FTAnyallOption();
    }
  }

  function parse_FTWordsValue()
  {
    eventHandler.startNonterminal("FTWordsValue", e0);
    switch (l1)
    {
    case 11:                        // StringLiteral
      shift(11);                    // StringLiteral
      break;
    default:
      shift(276);                   // '{'
      lookahead1W(267);             // Wildcard | EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral |
      whitespace();
      parse_Expr();
      shift(282);                   // '}'
    }
    eventHandler.endNonterminal("FTWordsValue", e0);
  }

  function try_FTWordsValue()
  {
    switch (l1)
    {
    case 11:                        // StringLiteral
      shiftT(11);                   // StringLiteral
      break;
    default:
      shiftT(276);                  // '{'
      lookahead1W(267);             // Wildcard | EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral |
      try_Expr();
      shiftT(282);                  // '}'
    }
  }

  function parse_FTExtensionSelection()
  {
    eventHandler.startNonterminal("FTExtensionSelection", e0);
    for (;;)
    {
      whitespace();
      parse_Pragma();
      lookahead1W(100);             // S^WS | '(#' | '(:' | '{'
      if (l1 != 35)                 // '(#'
      {
        break;
      }
    }
    shift(276);                     // '{'
    lookahead1W(166);               // StringLiteral | S^WS | '(' | '(#' | '(:' | 'ftnot' | '{' | '}'
    if (l1 != 282)                  // '}'
    {
      whitespace();
      parse_FTSelection();
    }
    shift(282);                     // '}'
    eventHandler.endNonterminal("FTExtensionSelection", e0);
  }

  function try_FTExtensionSelection()
  {
    for (;;)
    {
      try_Pragma();
      lookahead1W(100);             // S^WS | '(#' | '(:' | '{'
      if (l1 != 35)                 // '(#'
      {
        break;
      }
    }
    shiftT(276);                    // '{'
    lookahead1W(166);               // StringLiteral | S^WS | '(' | '(#' | '(:' | 'ftnot' | '{' | '}'
    if (l1 != 282)                  // '}'
    {
      try_FTSelection();
    }
    shiftT(282);                    // '}'
  }

  function parse_FTAnyallOption()
  {
    eventHandler.startNonterminal("FTAnyallOption", e0);
    switch (l1)
    {
    case 76:                        // 'any'
      shift(76);                    // 'any'
      lookahead1W(219);             // S^WS | EOF | '!=' | '(:' | ')' | ',' | ':' | ';' | '<' | '<<' | '<=' | '=' |
      if (l1 == 272)                // 'word'
      {
        shift(272);                 // 'word'
      }
      break;
    case 71:                        // 'all'
      shift(71);                    // 'all'
      lookahead1W(220);             // S^WS | EOF | '!=' | '(:' | ')' | ',' | ':' | ';' | '<' | '<<' | '<=' | '=' |
      if (l1 == 273)                // 'words'
      {
        shift(273);                 // 'words'
      }
      break;
    default:
      shift(210);                   // 'phrase'
    }
    eventHandler.endNonterminal("FTAnyallOption", e0);
  }

  function try_FTAnyallOption()
  {
    switch (l1)
    {
    case 76:                        // 'any'
      shiftT(76);                   // 'any'
      lookahead1W(219);             // S^WS | EOF | '!=' | '(:' | ')' | ',' | ':' | ';' | '<' | '<<' | '<=' | '=' |
      if (l1 == 272)                // 'word'
      {
        shiftT(272);                // 'word'
      }
      break;
    case 71:                        // 'all'
      shiftT(71);                   // 'all'
      lookahead1W(220);             // S^WS | EOF | '!=' | '(:' | ')' | ',' | ':' | ';' | '<' | '<<' | '<=' | '=' |
      if (l1 == 273)                // 'words'
      {
        shiftT(273);                // 'words'
      }
      break;
    default:
      shiftT(210);                  // 'phrase'
    }
  }

  function parse_FTTimes()
  {
    eventHandler.startNonterminal("FTTimes", e0);
    shift(195);                     // 'occurs'
    lookahead1W(149);               // S^WS | '(:' | 'at' | 'exactly' | 'from'
    whitespace();
    parse_FTRange();
    shift(247);                     // 'times'
    eventHandler.endNonterminal("FTTimes", e0);
  }

  function try_FTTimes()
  {
    shiftT(195);                    // 'occurs'
    lookahead1W(149);               // S^WS | '(:' | 'at' | 'exactly' | 'from'
    try_FTRange();
    shiftT(247);                    // 'times'
  }

  function parse_FTRange()
  {
    eventHandler.startNonterminal("FTRange", e0);
    switch (l1)
    {
    case 130:                       // 'exactly'
      shift(130);                   // 'exactly'
      lookahead1W(267);             // Wildcard | EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral |
      whitespace();
      parse_AdditiveExpr();
      break;
    case 81:                        // 'at'
      shift(81);                    // 'at'
      lookahead1W(125);             // S^WS | '(:' | 'least' | 'most'
      switch (l1)
      {
      case 173:                     // 'least'
        shift(173);                 // 'least'
        lookahead1W(267);           // Wildcard | EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral |
        whitespace();
        parse_AdditiveExpr();
        break;
      default:
        shift(183);                 // 'most'
        lookahead1W(267);           // Wildcard | EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral |
        whitespace();
        parse_AdditiveExpr();
      }
      break;
    default:
      shift(140);                   // 'from'
      lookahead1W(267);             // Wildcard | EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral |
      whitespace();
      parse_AdditiveExpr();
      shift(248);                   // 'to'
      lookahead1W(267);             // Wildcard | EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral |
      whitespace();
      parse_AdditiveExpr();
    }
    eventHandler.endNonterminal("FTRange", e0);
  }

  function try_FTRange()
  {
    switch (l1)
    {
    case 130:                       // 'exactly'
      shiftT(130);                  // 'exactly'
      lookahead1W(267);             // Wildcard | EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral |
      try_AdditiveExpr();
      break;
    case 81:                        // 'at'
      shiftT(81);                   // 'at'
      lookahead1W(125);             // S^WS | '(:' | 'least' | 'most'
      switch (l1)
      {
      case 173:                     // 'least'
        shiftT(173);                // 'least'
        lookahead1W(267);           // Wildcard | EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral |
        try_AdditiveExpr();
        break;
      default:
        shiftT(183);                // 'most'
        lookahead1W(267);           // Wildcard | EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral |
        try_AdditiveExpr();
      }
      break;
    default:
      shiftT(140);                  // 'from'
      lookahead1W(267);             // Wildcard | EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral |
      try_AdditiveExpr();
      shiftT(248);                  // 'to'
      lookahead1W(267);             // Wildcard | EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral |
      try_AdditiveExpr();
    }
  }

  function parse_FTPosFilter()
  {
    eventHandler.startNonterminal("FTPosFilter", e0);
    switch (l1)
    {
    case 202:                       // 'ordered'
      parse_FTOrder();
      break;
    case 269:                       // 'window'
      parse_FTWindow();
      break;
    case 117:                       // 'distance'
      parse_FTDistance();
      break;
    case 115:                       // 'different'
    case 223:                       // 'same'
      parse_FTScope();
      break;
    default:
      parse_FTContent();
    }
    eventHandler.endNonterminal("FTPosFilter", e0);
  }

  function try_FTPosFilter()
  {
    switch (l1)
    {
    case 202:                       // 'ordered'
      try_FTOrder();
      break;
    case 269:                       // 'window'
      try_FTWindow();
      break;
    case 117:                       // 'distance'
      try_FTDistance();
      break;
    case 115:                       // 'different'
    case 223:                       // 'same'
      try_FTScope();
      break;
    default:
      try_FTContent();
    }
  }

  function parse_FTOrder()
  {
    eventHandler.startNonterminal("FTOrder", e0);
    shift(202);                     // 'ordered'
    eventHandler.endNonterminal("FTOrder", e0);
  }

  function try_FTOrder()
  {
    shiftT(202);                    // 'ordered'
  }

  function parse_FTWindow()
  {
    eventHandler.startNonterminal("FTWindow", e0);
    shift(269);                     // 'window'
    lookahead1W(267);               // Wildcard | EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral |
    whitespace();
    parse_AdditiveExpr();
    whitespace();
    parse_FTUnit();
    eventHandler.endNonterminal("FTWindow", e0);
  }

  function try_FTWindow()
  {
    shiftT(269);                    // 'window'
    lookahead1W(267);               // Wildcard | EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral |
    try_AdditiveExpr();
    try_FTUnit();
  }

  function parse_FTDistance()
  {
    eventHandler.startNonterminal("FTDistance", e0);
    shift(117);                     // 'distance'
    lookahead1W(149);               // S^WS | '(:' | 'at' | 'exactly' | 'from'
    whitespace();
    parse_FTRange();
    whitespace();
    parse_FTUnit();
    eventHandler.endNonterminal("FTDistance", e0);
  }

  function try_FTDistance()
  {
    shiftT(117);                    // 'distance'
    lookahead1W(149);               // S^WS | '(:' | 'at' | 'exactly' | 'from'
    try_FTRange();
    try_FTUnit();
  }

  function parse_FTUnit()
  {
    eventHandler.startNonterminal("FTUnit", e0);
    switch (l1)
    {
    case 273:                       // 'words'
      shift(273);                   // 'words'
      break;
    case 232:                       // 'sentences'
      shift(232);                   // 'sentences'
      break;
    default:
      shift(205);                   // 'paragraphs'
    }
    eventHandler.endNonterminal("FTUnit", e0);
  }

  function try_FTUnit()
  {
    switch (l1)
    {
    case 273:                       // 'words'
      shiftT(273);                  // 'words'
      break;
    case 232:                       // 'sentences'
      shiftT(232);                  // 'sentences'
      break;
    default:
      shiftT(205);                  // 'paragraphs'
    }
  }

  function parse_FTScope()
  {
    eventHandler.startNonterminal("FTScope", e0);
    switch (l1)
    {
    case 223:                       // 'same'
      shift(223);                   // 'same'
      break;
    default:
      shift(115);                   // 'different'
    }
    lookahead1W(132);               // S^WS | '(:' | 'paragraph' | 'sentence'
    whitespace();
    parse_FTBigUnit();
    eventHandler.endNonterminal("FTScope", e0);
  }

  function try_FTScope()
  {
    switch (l1)
    {
    case 223:                       // 'same'
      shiftT(223);                  // 'same'
      break;
    default:
      shiftT(115);                  // 'different'
    }
    lookahead1W(132);               // S^WS | '(:' | 'paragraph' | 'sentence'
    try_FTBigUnit();
  }

  function parse_FTBigUnit()
  {
    eventHandler.startNonterminal("FTBigUnit", e0);
    switch (l1)
    {
    case 231:                       // 'sentence'
      shift(231);                   // 'sentence'
      break;
    default:
      shift(204);                   // 'paragraph'
    }
    eventHandler.endNonterminal("FTBigUnit", e0);
  }

  function try_FTBigUnit()
  {
    switch (l1)
    {
    case 231:                       // 'sentence'
      shiftT(231);                  // 'sentence'
      break;
    default:
      shiftT(204);                  // 'paragraph'
    }
  }

  function parse_FTContent()
  {
    eventHandler.startNonterminal("FTContent", e0);
    switch (l1)
    {
    case 81:                        // 'at'
      shift(81);                    // 'at'
      lookahead1W(117);             // S^WS | '(:' | 'end' | 'start'
      switch (l1)
      {
      case 237:                     // 'start'
        shift(237);                 // 'start'
        break;
      default:
        shift(126);                 // 'end'
      }
      break;
    default:
      shift(127);                   // 'entire'
      lookahead1W(42);              // S^WS | '(:' | 'content'
      shift(100);                   // 'content'
    }
    eventHandler.endNonterminal("FTContent", e0);
  }

  function try_FTContent()
  {
    switch (l1)
    {
    case 81:                        // 'at'
      shiftT(81);                   // 'at'
      lookahead1W(117);             // S^WS | '(:' | 'end' | 'start'
      switch (l1)
      {
      case 237:                     // 'start'
        shiftT(237);                // 'start'
        break;
      default:
        shiftT(126);                // 'end'
      }
      break;
    default:
      shiftT(127);                  // 'entire'
      lookahead1W(42);              // S^WS | '(:' | 'content'
      shiftT(100);                  // 'content'
    }
  }

  function parse_FTMatchOptions()
  {
    eventHandler.startNonterminal("FTMatchOptions", e0);
    for (;;)
    {
      shift(259);                   // 'using'
      lookahead1W(182);             // S^WS | '(:' | 'case' | 'diacritics' | 'language' | 'lowercase' | 'no' |
      whitespace();
      parse_FTMatchOption();
      lookahead1W(215);             // S^WS | EOF | '!=' | '(:' | ')' | ',' | ':' | ';' | '<' | '<<' | '<=' | '=' |
      if (l1 != 259)                // 'using'
      {
        break;
      }
    }
    eventHandler.endNonterminal("FTMatchOptions", e0);
  }

  function try_FTMatchOptions()
  {
    for (;;)
    {
      shiftT(259);                  // 'using'
      lookahead1W(182);             // S^WS | '(:' | 'case' | 'diacritics' | 'language' | 'lowercase' | 'no' |
      try_FTMatchOption();
      lookahead1W(215);             // S^WS | EOF | '!=' | '(:' | ')' | ',' | ':' | ';' | '<' | '<<' | '<=' | '=' |
      if (l1 != 259)                // 'using'
      {
        break;
      }
    }
  }

  function parse_FTMatchOption()
  {
    eventHandler.startNonterminal("FTMatchOption", e0);
    switch (l1)
    {
    case 188:                       // 'no'
      lookahead2W(161);             // S^WS | '(:' | 'stemming' | 'stop' | 'thesaurus' | 'wildcards'
      break;
    default:
      lk = l1;
    }
    switch (lk)
    {
    case 169:                       // 'language'
      parse_FTLanguageOption();
      break;
    case 268:                       // 'wildcards'
    case 137404:                    // 'no' 'wildcards'
      parse_FTWildCardOption();
      break;
    case 246:                       // 'thesaurus'
    case 126140:                    // 'no' 'thesaurus'
      parse_FTThesaurusOption();
      break;
    case 238:                       // 'stemming'
    case 122044:                    // 'no' 'stemming'
      parse_FTStemOption();
      break;
    case 114:                       // 'diacritics'
      parse_FTDiacriticsOption();
      break;
    case 239:                       // 'stop'
    case 122556:                    // 'no' 'stop'
      parse_FTStopWordOption();
      break;
    case 199:                       // 'option'
      parse_FTExtensionOption();
      break;
    default:
      parse_FTCaseOption();
    }
    eventHandler.endNonterminal("FTMatchOption", e0);
  }

  function try_FTMatchOption()
  {
    switch (l1)
    {
    case 188:                       // 'no'
      lookahead2W(161);             // S^WS | '(:' | 'stemming' | 'stop' | 'thesaurus' | 'wildcards'
      break;
    default:
      lk = l1;
    }
    switch (lk)
    {
    case 169:                       // 'language'
      try_FTLanguageOption();
      break;
    case 268:                       // 'wildcards'
    case 137404:                    // 'no' 'wildcards'
      try_FTWildCardOption();
      break;
    case 246:                       // 'thesaurus'
    case 126140:                    // 'no' 'thesaurus'
      try_FTThesaurusOption();
      break;
    case 238:                       // 'stemming'
    case 122044:                    // 'no' 'stemming'
      try_FTStemOption();
      break;
    case 114:                       // 'diacritics'
      try_FTDiacriticsOption();
      break;
    case 239:                       // 'stop'
    case 122556:                    // 'no' 'stop'
      try_FTStopWordOption();
      break;
    case 199:                       // 'option'
      try_FTExtensionOption();
      break;
    default:
      try_FTCaseOption();
    }
  }

  function parse_FTCaseOption()
  {
    eventHandler.startNonterminal("FTCaseOption", e0);
    switch (l1)
    {
    case 88:                        // 'case'
      shift(88);                    // 'case'
      lookahead1W(124);             // S^WS | '(:' | 'insensitive' | 'sensitive'
      switch (l1)
      {
      case 158:                     // 'insensitive'
        shift(158);                 // 'insensitive'
        break;
      default:
        shift(230);                 // 'sensitive'
      }
      break;
    case 177:                       // 'lowercase'
      shift(177);                   // 'lowercase'
      break;
    default:
      shift(258);                   // 'uppercase'
    }
    eventHandler.endNonterminal("FTCaseOption", e0);
  }

  function try_FTCaseOption()
  {
    switch (l1)
    {
    case 88:                        // 'case'
      shiftT(88);                   // 'case'
      lookahead1W(124);             // S^WS | '(:' | 'insensitive' | 'sensitive'
      switch (l1)
      {
      case 158:                     // 'insensitive'
        shiftT(158);                // 'insensitive'
        break;
      default:
        shiftT(230);                // 'sensitive'
      }
      break;
    case 177:                       // 'lowercase'
      shiftT(177);                  // 'lowercase'
      break;
    default:
      shiftT(258);                  // 'uppercase'
    }
  }

  function parse_FTDiacriticsOption()
  {
    eventHandler.startNonterminal("FTDiacriticsOption", e0);
    shift(114);                     // 'diacritics'
    lookahead1W(124);               // S^WS | '(:' | 'insensitive' | 'sensitive'
    switch (l1)
    {
    case 158:                       // 'insensitive'
      shift(158);                   // 'insensitive'
      break;
    default:
      shift(230);                   // 'sensitive'
    }
    eventHandler.endNonterminal("FTDiacriticsOption", e0);
  }

  function try_FTDiacriticsOption()
  {
    shiftT(114);                    // 'diacritics'
    lookahead1W(124);               // S^WS | '(:' | 'insensitive' | 'sensitive'
    switch (l1)
    {
    case 158:                       // 'insensitive'
      shiftT(158);                  // 'insensitive'
      break;
    default:
      shiftT(230);                  // 'sensitive'
    }
  }

  function parse_FTStemOption()
  {
    eventHandler.startNonterminal("FTStemOption", e0);
    switch (l1)
    {
    case 238:                       // 'stemming'
      shift(238);                   // 'stemming'
      break;
    default:
      shift(188);                   // 'no'
      lookahead1W(74);              // S^WS | '(:' | 'stemming'
      shift(238);                   // 'stemming'
    }
    eventHandler.endNonterminal("FTStemOption", e0);
  }

  function try_FTStemOption()
  {
    switch (l1)
    {
    case 238:                       // 'stemming'
      shiftT(238);                  // 'stemming'
      break;
    default:
      shiftT(188);                  // 'no'
      lookahead1W(74);              // S^WS | '(:' | 'stemming'
      shiftT(238);                  // 'stemming'
    }
  }

  function parse_FTThesaurusOption()
  {
    eventHandler.startNonterminal("FTThesaurusOption", e0);
    switch (l1)
    {
    case 246:                       // 'thesaurus'
      shift(246);                   // 'thesaurus'
      lookahead1W(142);             // S^WS | '(' | '(:' | 'at' | 'default'
      switch (l1)
      {
      case 81:                      // 'at'
        whitespace();
        parse_FTThesaurusID();
        break;
      case 109:                     // 'default'
        shift(109);                 // 'default'
        break;
      default:
        shift(34);                  // '('
        lookahead1W(112);           // S^WS | '(:' | 'at' | 'default'
        switch (l1)
        {
        case 81:                    // 'at'
          whitespace();
          parse_FTThesaurusID();
          break;
        default:
          shift(109);               // 'default'
        }
        for (;;)
        {
          lookahead1W(101);         // S^WS | '(:' | ')' | ','
          if (l1 != 41)             // ','
          {
            break;
          }
          shift(41);                // ','
          lookahead1W(31);          // S^WS | '(:' | 'at'
          whitespace();
          parse_FTThesaurusID();
        }
        shift(37);                  // ')'
      }
      break;
    default:
      shift(188);                   // 'no'
      lookahead1W(78);              // S^WS | '(:' | 'thesaurus'
      shift(246);                   // 'thesaurus'
    }
    eventHandler.endNonterminal("FTThesaurusOption", e0);
  }

  function try_FTThesaurusOption()
  {
    switch (l1)
    {
    case 246:                       // 'thesaurus'
      shiftT(246);                  // 'thesaurus'
      lookahead1W(142);             // S^WS | '(' | '(:' | 'at' | 'default'
      switch (l1)
      {
      case 81:                      // 'at'
        try_FTThesaurusID();
        break;
      case 109:                     // 'default'
        shiftT(109);                // 'default'
        break;
      default:
        shiftT(34);                 // '('
        lookahead1W(112);           // S^WS | '(:' | 'at' | 'default'
        switch (l1)
        {
        case 81:                    // 'at'
          try_FTThesaurusID();
          break;
        default:
          shiftT(109);              // 'default'
        }
        for (;;)
        {
          lookahead1W(101);         // S^WS | '(:' | ')' | ','
          if (l1 != 41)             // ','
          {
            break;
          }
          shiftT(41);               // ','
          lookahead1W(31);          // S^WS | '(:' | 'at'
          try_FTThesaurusID();
        }
        shiftT(37);                 // ')'
      }
      break;
    default:
      shiftT(188);                  // 'no'
      lookahead1W(78);              // S^WS | '(:' | 'thesaurus'
      shiftT(246);                  // 'thesaurus'
    }
  }

  function parse_FTThesaurusID()
  {
    eventHandler.startNonterminal("FTThesaurusID", e0);
    shift(81);                      // 'at'
    lookahead1W(15);                // URILiteral | S^WS | '(:'
    shift(7);                       // URILiteral
    lookahead1W(221);               // S^WS | EOF | '!=' | '(:' | ')' | ',' | ':' | ';' | '<' | '<<' | '<=' | '=' |
    if (l1 == 217)                  // 'relationship'
    {
      shift(217);                   // 'relationship'
      lookahead1W(17);              // StringLiteral | S^WS | '(:'
      shift(11);                    // StringLiteral
    }
    lookahead1W(217);               // S^WS | EOF | '!=' | '(:' | ')' | ',' | ':' | ';' | '<' | '<<' | '<=' | '=' |
    switch (l1)
    {
    case 81:                        // 'at'
      lookahead2W(165);             // S^WS | '(:' | 'end' | 'least' | 'most' | 'position' | 'start'
      break;
    default:
      lk = l1;
    }
    if (lk == 130                   // 'exactly'
     || lk == 140                   // 'from'
     || lk == 88657                 // 'at' 'least'
     || lk == 93777)                // 'at' 'most'
    {
      whitespace();
      parse_FTLiteralRange();
      lookahead1W(58);              // S^WS | '(:' | 'levels'
      shift(175);                   // 'levels'
    }
    eventHandler.endNonterminal("FTThesaurusID", e0);
  }

  function try_FTThesaurusID()
  {
    shiftT(81);                     // 'at'
    lookahead1W(15);                // URILiteral | S^WS | '(:'
    shiftT(7);                      // URILiteral
    lookahead1W(221);               // S^WS | EOF | '!=' | '(:' | ')' | ',' | ':' | ';' | '<' | '<<' | '<=' | '=' |
    if (l1 == 217)                  // 'relationship'
    {
      shiftT(217);                  // 'relationship'
      lookahead1W(17);              // StringLiteral | S^WS | '(:'
      shiftT(11);                   // StringLiteral
    }
    lookahead1W(217);               // S^WS | EOF | '!=' | '(:' | ')' | ',' | ':' | ';' | '<' | '<<' | '<=' | '=' |
    switch (l1)
    {
    case 81:                        // 'at'
      lookahead2W(165);             // S^WS | '(:' | 'end' | 'least' | 'most' | 'position' | 'start'
      break;
    default:
      lk = l1;
    }
    if (lk == 130                   // 'exactly'
     || lk == 140                   // 'from'
     || lk == 88657                 // 'at' 'least'
     || lk == 93777)                // 'at' 'most'
    {
      try_FTLiteralRange();
      lookahead1W(58);              // S^WS | '(:' | 'levels'
      shiftT(175);                  // 'levels'
    }
  }

  function parse_FTLiteralRange()
  {
    eventHandler.startNonterminal("FTLiteralRange", e0);
    switch (l1)
    {
    case 130:                       // 'exactly'
      shift(130);                   // 'exactly'
      lookahead1W(16);              // IntegerLiteral | S^WS | '(:'
      shift(8);                     // IntegerLiteral
      break;
    case 81:                        // 'at'
      shift(81);                    // 'at'
      lookahead1W(125);             // S^WS | '(:' | 'least' | 'most'
      switch (l1)
      {
      case 173:                     // 'least'
        shift(173);                 // 'least'
        lookahead1W(16);            // IntegerLiteral | S^WS | '(:'
        shift(8);                   // IntegerLiteral
        break;
      default:
        shift(183);                 // 'most'
        lookahead1W(16);            // IntegerLiteral | S^WS | '(:'
        shift(8);                   // IntegerLiteral
      }
      break;
    default:
      shift(140);                   // 'from'
      lookahead1W(16);              // IntegerLiteral | S^WS | '(:'
      shift(8);                     // IntegerLiteral
      lookahead1W(79);              // S^WS | '(:' | 'to'
      shift(248);                   // 'to'
      lookahead1W(16);              // IntegerLiteral | S^WS | '(:'
      shift(8);                     // IntegerLiteral
    }
    eventHandler.endNonterminal("FTLiteralRange", e0);
  }

  function try_FTLiteralRange()
  {
    switch (l1)
    {
    case 130:                       // 'exactly'
      shiftT(130);                  // 'exactly'
      lookahead1W(16);              // IntegerLiteral | S^WS | '(:'
      shiftT(8);                    // IntegerLiteral
      break;
    case 81:                        // 'at'
      shiftT(81);                   // 'at'
      lookahead1W(125);             // S^WS | '(:' | 'least' | 'most'
      switch (l1)
      {
      case 173:                     // 'least'
        shiftT(173);                // 'least'
        lookahead1W(16);            // IntegerLiteral | S^WS | '(:'
        shiftT(8);                  // IntegerLiteral
        break;
      default:
        shiftT(183);                // 'most'
        lookahead1W(16);            // IntegerLiteral | S^WS | '(:'
        shiftT(8);                  // IntegerLiteral
      }
      break;
    default:
      shiftT(140);                  // 'from'
      lookahead1W(16);              // IntegerLiteral | S^WS | '(:'
      shiftT(8);                    // IntegerLiteral
      lookahead1W(79);              // S^WS | '(:' | 'to'
      shiftT(248);                  // 'to'
      lookahead1W(16);              // IntegerLiteral | S^WS | '(:'
      shiftT(8);                    // IntegerLiteral
    }
  }

  function parse_FTStopWordOption()
  {
    eventHandler.startNonterminal("FTStopWordOption", e0);
    switch (l1)
    {
    case 239:                       // 'stop'
      shift(239);                   // 'stop'
      lookahead1W(86);              // S^WS | '(:' | 'words'
      shift(273);                   // 'words'
      lookahead1W(142);             // S^WS | '(' | '(:' | 'at' | 'default'
      switch (l1)
      {
      case 109:                     // 'default'
        shift(109);                 // 'default'
        for (;;)
        {
          lookahead1W(218);         // S^WS | EOF | '!=' | '(:' | ')' | ',' | ':' | ';' | '<' | '<<' | '<=' | '=' |
          if (l1 != 131             // 'except'
           && l1 != 254)            // 'union'
          {
            break;
          }
          whitespace();
          parse_FTStopWordsInclExcl();
        }
        break;
      default:
        whitespace();
        parse_FTStopWords();
        for (;;)
        {
          lookahead1W(218);         // S^WS | EOF | '!=' | '(:' | ')' | ',' | ':' | ';' | '<' | '<<' | '<=' | '=' |
          if (l1 != 131             // 'except'
           && l1 != 254)            // 'union'
          {
            break;
          }
          whitespace();
          parse_FTStopWordsInclExcl();
        }
      }
      break;
    default:
      shift(188);                   // 'no'
      lookahead1W(75);              // S^WS | '(:' | 'stop'
      shift(239);                   // 'stop'
      lookahead1W(86);              // S^WS | '(:' | 'words'
      shift(273);                   // 'words'
    }
    eventHandler.endNonterminal("FTStopWordOption", e0);
  }

  function try_FTStopWordOption()
  {
    switch (l1)
    {
    case 239:                       // 'stop'
      shiftT(239);                  // 'stop'
      lookahead1W(86);              // S^WS | '(:' | 'words'
      shiftT(273);                  // 'words'
      lookahead1W(142);             // S^WS | '(' | '(:' | 'at' | 'default'
      switch (l1)
      {
      case 109:                     // 'default'
        shiftT(109);                // 'default'
        for (;;)
        {
          lookahead1W(218);         // S^WS | EOF | '!=' | '(:' | ')' | ',' | ':' | ';' | '<' | '<<' | '<=' | '=' |
          if (l1 != 131             // 'except'
           && l1 != 254)            // 'union'
          {
            break;
          }
          try_FTStopWordsInclExcl();
        }
        break;
      default:
        try_FTStopWords();
        for (;;)
        {
          lookahead1W(218);         // S^WS | EOF | '!=' | '(:' | ')' | ',' | ':' | ';' | '<' | '<<' | '<=' | '=' |
          if (l1 != 131             // 'except'
           && l1 != 254)            // 'union'
          {
            break;
          }
          try_FTStopWordsInclExcl();
        }
      }
      break;
    default:
      shiftT(188);                  // 'no'
      lookahead1W(75);              // S^WS | '(:' | 'stop'
      shiftT(239);                  // 'stop'
      lookahead1W(86);              // S^WS | '(:' | 'words'
      shiftT(273);                  // 'words'
    }
  }

  function parse_FTStopWords()
  {
    eventHandler.startNonterminal("FTStopWords", e0);
    switch (l1)
    {
    case 81:                        // 'at'
      shift(81);                    // 'at'
      lookahead1W(15);              // URILiteral | S^WS | '(:'
      shift(7);                     // URILiteral
      break;
    default:
      shift(34);                    // '('
      lookahead1W(17);              // StringLiteral | S^WS | '(:'
      shift(11);                    // StringLiteral
      for (;;)
      {
        lookahead1W(101);           // S^WS | '(:' | ')' | ','
        if (l1 != 41)               // ','
        {
          break;
        }
        shift(41);                  // ','
        lookahead1W(17);            // StringLiteral | S^WS | '(:'
        shift(11);                  // StringLiteral
      }
      shift(37);                    // ')'
    }
    eventHandler.endNonterminal("FTStopWords", e0);
  }

  function try_FTStopWords()
  {
    switch (l1)
    {
    case 81:                        // 'at'
      shiftT(81);                   // 'at'
      lookahead1W(15);              // URILiteral | S^WS | '(:'
      shiftT(7);                    // URILiteral
      break;
    default:
      shiftT(34);                   // '('
      lookahead1W(17);              // StringLiteral | S^WS | '(:'
      shiftT(11);                   // StringLiteral
      for (;;)
      {
        lookahead1W(101);           // S^WS | '(:' | ')' | ','
        if (l1 != 41)               // ','
        {
          break;
        }
        shiftT(41);                 // ','
        lookahead1W(17);            // StringLiteral | S^WS | '(:'
        shiftT(11);                 // StringLiteral
      }
      shiftT(37);                   // ')'
    }
  }

  function parse_FTStopWordsInclExcl()
  {
    eventHandler.startNonterminal("FTStopWordsInclExcl", e0);
    switch (l1)
    {
    case 254:                       // 'union'
      shift(254);                   // 'union'
      break;
    default:
      shift(131);                   // 'except'
    }
    lookahead1W(99);                // S^WS | '(' | '(:' | 'at'
    whitespace();
    parse_FTStopWords();
    eventHandler.endNonterminal("FTStopWordsInclExcl", e0);
  }

  function try_FTStopWordsInclExcl()
  {
    switch (l1)
    {
    case 254:                       // 'union'
      shiftT(254);                  // 'union'
      break;
    default:
      shiftT(131);                  // 'except'
    }
    lookahead1W(99);                // S^WS | '(' | '(:' | 'at'
    try_FTStopWords();
  }

  function parse_FTLanguageOption()
  {
    eventHandler.startNonterminal("FTLanguageOption", e0);
    shift(169);                     // 'language'
    lookahead1W(17);                // StringLiteral | S^WS | '(:'
    shift(11);                      // StringLiteral
    eventHandler.endNonterminal("FTLanguageOption", e0);
  }

  function try_FTLanguageOption()
  {
    shiftT(169);                    // 'language'
    lookahead1W(17);                // StringLiteral | S^WS | '(:'
    shiftT(11);                     // StringLiteral
  }

  function parse_FTWildCardOption()
  {
    eventHandler.startNonterminal("FTWildCardOption", e0);
    switch (l1)
    {
    case 268:                       // 'wildcards'
      shift(268);                   // 'wildcards'
      break;
    default:
      shift(188);                   // 'no'
      lookahead1W(84);              // S^WS | '(:' | 'wildcards'
      shift(268);                   // 'wildcards'
    }
    eventHandler.endNonterminal("FTWildCardOption", e0);
  }

  function try_FTWildCardOption()
  {
    switch (l1)
    {
    case 268:                       // 'wildcards'
      shiftT(268);                  // 'wildcards'
      break;
    default:
      shiftT(188);                  // 'no'
      lookahead1W(84);              // S^WS | '(:' | 'wildcards'
      shiftT(268);                  // 'wildcards'
    }
  }

  function parse_FTExtensionOption()
  {
    eventHandler.startNonterminal("FTExtensionOption", e0);
    shift(199);                     // 'option'
    lookahead1W(255);               // EQName^Token | S^WS | '(:' | 'after' | 'allowing' | 'ancestor' |
    whitespace();
    parse_EQName();
    lookahead1W(17);                // StringLiteral | S^WS | '(:'
    shift(11);                      // StringLiteral
    eventHandler.endNonterminal("FTExtensionOption", e0);
  }

  function try_FTExtensionOption()
  {
    shiftT(199);                    // 'option'
    lookahead1W(255);               // EQName^Token | S^WS | '(:' | 'after' | 'allowing' | 'ancestor' |
    try_EQName();
    lookahead1W(17);                // StringLiteral | S^WS | '(:'
    shiftT(11);                     // StringLiteral
  }

  function parse_FTIgnoreOption()
  {
    eventHandler.startNonterminal("FTIgnoreOption", e0);
    shift(271);                     // 'without'
    lookahead1W(42);                // S^WS | '(:' | 'content'
    shift(100);                     // 'content'
    lookahead1W(267);               // Wildcard | EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral |
    whitespace();
    parse_UnionExpr();
    eventHandler.endNonterminal("FTIgnoreOption", e0);
  }

  function try_FTIgnoreOption()
  {
    shiftT(271);                    // 'without'
    lookahead1W(42);                // S^WS | '(:' | 'content'
    shiftT(100);                    // 'content'
    lookahead1W(267);               // Wildcard | EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral |
    try_UnionExpr();
  }

  function parse_CollectionDecl()
  {
    eventHandler.startNonterminal("CollectionDecl", e0);
    shift(95);                      // 'collection'
    lookahead1W(255);               // EQName^Token | S^WS | '(:' | 'after' | 'allowing' | 'ancestor' |
    whitespace();
    parse_EQName();
    lookahead1W(107);               // S^WS | '(:' | ';' | 'as'
    if (l1 == 79)                   // 'as'
    {
      whitespace();
      parse_CollectionTypeDecl();
    }
    eventHandler.endNonterminal("CollectionDecl", e0);
  }

  function parse_CollectionTypeDecl()
  {
    eventHandler.startNonterminal("CollectionTypeDecl", e0);
    shift(79);                      // 'as'
    lookahead1W(178);               // S^WS | '(:' | 'attribute' | 'comment' | 'document-node' | 'element' |
    whitespace();
    parse_KindTest();
    lookahead1W(156);               // S^WS | '(:' | '*' | '+' | ';' | '?'
    if (l1 != 53)                   // ';'
    {
      whitespace();
      parse_OccurrenceIndicator();
    }
    eventHandler.endNonterminal("CollectionTypeDecl", e0);
  }

  function parse_IndexName()
  {
    eventHandler.startNonterminal("IndexName", e0);
    parse_EQName();
    eventHandler.endNonterminal("IndexName", e0);
  }

  function parse_IndexDomainExpr()
  {
    eventHandler.startNonterminal("IndexDomainExpr", e0);
    parse_PathExpr();
    eventHandler.endNonterminal("IndexDomainExpr", e0);
  }

  function parse_IndexKeySpec()
  {
    eventHandler.startNonterminal("IndexKeySpec", e0);
    parse_IndexKeyExpr();
    if (l1 == 79)                   // 'as'
    {
      whitespace();
      parse_IndexKeyTypeDecl();
    }
    lookahead1W(146);               // S^WS | '(:' | ',' | ';' | 'collation'
    if (l1 == 94)                   // 'collation'
    {
      whitespace();
      parse_IndexKeyCollation();
    }
    eventHandler.endNonterminal("IndexKeySpec", e0);
  }

  function parse_IndexKeyExpr()
  {
    eventHandler.startNonterminal("IndexKeyExpr", e0);
    parse_PathExpr();
    eventHandler.endNonterminal("IndexKeyExpr", e0);
  }

  function parse_IndexKeyTypeDecl()
  {
    eventHandler.startNonterminal("IndexKeyTypeDecl", e0);
    shift(79);                      // 'as'
    lookahead1W(255);               // EQName^Token | S^WS | '(:' | 'after' | 'allowing' | 'ancestor' |
    whitespace();
    parse_AtomicType();
    lookahead1W(169);               // S^WS | '(:' | '*' | '+' | ',' | ';' | '?' | 'collation'
    if (l1 == 39                    // '*'
     || l1 == 40                    // '+'
     || l1 == 64)                   // '?'
    {
      whitespace();
      parse_OccurrenceIndicator();
    }
    eventHandler.endNonterminal("IndexKeyTypeDecl", e0);
  }

  function parse_AtomicType()
  {
    eventHandler.startNonterminal("AtomicType", e0);
    parse_EQName();
    eventHandler.endNonterminal("AtomicType", e0);
  }

  function parse_IndexKeyCollation()
  {
    eventHandler.startNonterminal("IndexKeyCollation", e0);
    shift(94);                      // 'collation'
    lookahead1W(15);                // URILiteral | S^WS | '(:'
    shift(7);                       // URILiteral
    eventHandler.endNonterminal("IndexKeyCollation", e0);
  }

  function parse_IndexDecl()
  {
    eventHandler.startNonterminal("IndexDecl", e0);
    shift(155);                     // 'index'
    lookahead1W(255);               // EQName^Token | S^WS | '(:' | 'after' | 'allowing' | 'ancestor' |
    whitespace();
    parse_IndexName();
    lookahead1W(65);                // S^WS | '(:' | 'on'
    shift(197);                     // 'on'
    lookahead1W(63);                // S^WS | '(:' | 'nodes'
    shift(192);                     // 'nodes'
    lookahead1W(266);               // Wildcard | EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral |
    whitespace();
    parse_IndexDomainExpr();
    shift(87);                      // 'by'
    lookahead1W(266);               // Wildcard | EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral |
    whitespace();
    parse_IndexKeySpec();
    for (;;)
    {
      lookahead1W(103);             // S^WS | '(:' | ',' | ';'
      if (l1 != 41)                 // ','
      {
        break;
      }
      shift(41);                    // ','
      lookahead1W(266);             // Wildcard | EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral |
      whitespace();
      parse_IndexKeySpec();
    }
    eventHandler.endNonterminal("IndexDecl", e0);
  }

  function parse_ICDecl()
  {
    eventHandler.startNonterminal("ICDecl", e0);
    shift(161);                     // 'integrity'
    lookahead1W(40);                // S^WS | '(:' | 'constraint'
    shift(97);                      // 'constraint'
    lookahead1W(255);               // EQName^Token | S^WS | '(:' | 'after' | 'allowing' | 'ancestor' |
    whitespace();
    parse_EQName();
    lookahead1W(120);               // S^WS | '(:' | 'foreign' | 'on'
    switch (l1)
    {
    case 197:                       // 'on'
      whitespace();
      parse_ICCollection();
      break;
    default:
      whitespace();
      parse_ICForeignKey();
    }
    eventHandler.endNonterminal("ICDecl", e0);
  }

  function parse_ICCollection()
  {
    eventHandler.startNonterminal("ICCollection", e0);
    shift(197);                     // 'on'
    lookahead1W(39);                // S^WS | '(:' | 'collection'
    shift(95);                      // 'collection'
    lookahead1W(255);               // EQName^Token | S^WS | '(:' | 'after' | 'allowing' | 'ancestor' |
    whitespace();
    parse_EQName();
    lookahead1W(140);               // S^WS | '$' | '(:' | 'foreach' | 'node'
    switch (l1)
    {
    case 31:                        // '$'
      whitespace();
      parse_ICCollSequence();
      break;
    case 191:                       // 'node'
      whitespace();
      parse_ICCollSequenceUnique();
      break;
    default:
      whitespace();
      parse_ICCollNode();
    }
    eventHandler.endNonterminal("ICCollection", e0);
  }

  function parse_ICCollSequence()
  {
    eventHandler.startNonterminal("ICCollSequence", e0);
    parse_VarRef();
    lookahead1W(37);                // S^WS | '(:' | 'check'
    shift(92);                      // 'check'
    lookahead1W(267);               // Wildcard | EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral |
    whitespace();
    parse_ExprSingle();
    eventHandler.endNonterminal("ICCollSequence", e0);
  }

  function parse_ICCollSequenceUnique()
  {
    eventHandler.startNonterminal("ICCollSequenceUnique", e0);
    shift(191);                     // 'node'
    lookahead1W(21);                // S^WS | '$' | '(:'
    whitespace();
    parse_VarRef();
    lookahead1W(37);                // S^WS | '(:' | 'check'
    shift(92);                      // 'check'
    lookahead1W(80);                // S^WS | '(:' | 'unique'
    shift(255);                     // 'unique'
    lookahead1W(57);                // S^WS | '(:' | 'key'
    shift(168);                     // 'key'
    lookahead1W(266);               // Wildcard | EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral |
    whitespace();
    parse_PathExpr();
    eventHandler.endNonterminal("ICCollSequenceUnique", e0);
  }

  function parse_ICCollNode()
  {
    eventHandler.startNonterminal("ICCollNode", e0);
    shift(138);                     // 'foreach'
    lookahead1W(62);                // S^WS | '(:' | 'node'
    shift(191);                     // 'node'
    lookahead1W(21);                // S^WS | '$' | '(:'
    whitespace();
    parse_VarRef();
    lookahead1W(37);                // S^WS | '(:' | 'check'
    shift(92);                      // 'check'
    lookahead1W(267);               // Wildcard | EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral |
    whitespace();
    parse_ExprSingle();
    eventHandler.endNonterminal("ICCollNode", e0);
  }

  function parse_ICForeignKey()
  {
    eventHandler.startNonterminal("ICForeignKey", e0);
    shift(139);                     // 'foreign'
    lookahead1W(57);                // S^WS | '(:' | 'key'
    shift(168);                     // 'key'
    lookahead1W(51);                // S^WS | '(:' | 'from'
    whitespace();
    parse_ICForeignKeySource();
    whitespace();
    parse_ICForeignKeyTarget();
    eventHandler.endNonterminal("ICForeignKey", e0);
  }

  function parse_ICForeignKeySource()
  {
    eventHandler.startNonterminal("ICForeignKeySource", e0);
    shift(140);                     // 'from'
    lookahead1W(39);                // S^WS | '(:' | 'collection'
    whitespace();
    parse_ICForeignKeyValues();
    eventHandler.endNonterminal("ICForeignKeySource", e0);
  }

  function parse_ICForeignKeyTarget()
  {
    eventHandler.startNonterminal("ICForeignKeyTarget", e0);
    shift(248);                     // 'to'
    lookahead1W(39);                // S^WS | '(:' | 'collection'
    whitespace();
    parse_ICForeignKeyValues();
    eventHandler.endNonterminal("ICForeignKeyTarget", e0);
  }

  function parse_ICForeignKeyValues()
  {
    eventHandler.startNonterminal("ICForeignKeyValues", e0);
    shift(95);                      // 'collection'
    lookahead1W(255);               // EQName^Token | S^WS | '(:' | 'after' | 'allowing' | 'ancestor' |
    whitespace();
    parse_EQName();
    lookahead1W(62);                // S^WS | '(:' | 'node'
    shift(191);                     // 'node'
    lookahead1W(21);                // S^WS | '$' | '(:'
    whitespace();
    parse_VarRef();
    lookahead1W(57);                // S^WS | '(:' | 'key'
    shift(168);                     // 'key'
    lookahead1W(266);               // Wildcard | EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral |
    whitespace();
    parse_PathExpr();
    eventHandler.endNonterminal("ICForeignKeyValues", e0);
  }

  function try_Comment()
  {
    shiftT(36);                     // '(:'
    for (;;)
    {
      lookahead1(89);               // CommentContents | '(:' | ':)'
      if (l1 == 50)                 // ':)'
      {
        break;
      }
      switch (l1)
      {
      case 24:                      // CommentContents
        shiftT(24);                 // CommentContents
        break;
      default:
        try_Comment();
      }
    }
    shiftT(50);                     // ':)'
  }

  function try_Whitespace()
  {
    switch (l1)
    {
    case 22:                        // S^WS
      shiftT(22);                   // S^WS
      break;
    default:
      try_Comment();
    }
  }

  function parse_EQName()
  {
    eventHandler.startNonterminal("EQName", e0);
    lookahead1(250);                // EQName^Token | 'after' | 'allowing' | 'ancestor' | 'ancestor-or-self' | 'and' |
    switch (l1)
    {
    case 82:                        // 'attribute'
      shift(82);                    // 'attribute'
      break;
    case 96:                        // 'comment'
      shift(96);                    // 'comment'
      break;
    case 120:                       // 'document-node'
      shift(120);                   // 'document-node'
      break;
    case 121:                       // 'element'
      shift(121);                   // 'element'
      break;
    case 124:                       // 'empty-sequence'
      shift(124);                   // 'empty-sequence'
      break;
    case 145:                       // 'function'
      shift(145);                   // 'function'
      break;
    case 152:                       // 'if'
      shift(152);                   // 'if'
      break;
    case 165:                       // 'item'
      shift(165);                   // 'item'
      break;
    case 185:                       // 'namespace-node'
      shift(185);                   // 'namespace-node'
      break;
    case 191:                       // 'node'
      shift(191);                   // 'node'
      break;
    case 216:                       // 'processing-instruction'
      shift(216);                   // 'processing-instruction'
      break;
    case 226:                       // 'schema-attribute'
      shift(226);                   // 'schema-attribute'
      break;
    case 227:                       // 'schema-element'
      shift(227);                   // 'schema-element'
      break;
    case 243:                       // 'switch'
      shift(243);                   // 'switch'
      break;
    case 244:                       // 'text'
      shift(244);                   // 'text'
      break;
    case 253:                       // 'typeswitch'
      shift(253);                   // 'typeswitch'
      break;
    case 78:                        // 'array'
      shift(78);                    // 'array'
      break;
    case 167:                       // 'json-item'
      shift(167);                   // 'json-item'
      break;
    case 242:                       // 'structured-item'
      shift(242);                   // 'structured-item'
      break;
    default:
      parse_FunctionName();
    }
    eventHandler.endNonterminal("EQName", e0);
  }

  function try_EQName()
  {
    lookahead1(250);                // EQName^Token | 'after' | 'allowing' | 'ancestor' | 'ancestor-or-self' | 'and' |
    switch (l1)
    {
    case 82:                        // 'attribute'
      shiftT(82);                   // 'attribute'
      break;
    case 96:                        // 'comment'
      shiftT(96);                   // 'comment'
      break;
    case 120:                       // 'document-node'
      shiftT(120);                  // 'document-node'
      break;
    case 121:                       // 'element'
      shiftT(121);                  // 'element'
      break;
    case 124:                       // 'empty-sequence'
      shiftT(124);                  // 'empty-sequence'
      break;
    case 145:                       // 'function'
      shiftT(145);                  // 'function'
      break;
    case 152:                       // 'if'
      shiftT(152);                  // 'if'
      break;
    case 165:                       // 'item'
      shiftT(165);                  // 'item'
      break;
    case 185:                       // 'namespace-node'
      shiftT(185);                  // 'namespace-node'
      break;
    case 191:                       // 'node'
      shiftT(191);                  // 'node'
      break;
    case 216:                       // 'processing-instruction'
      shiftT(216);                  // 'processing-instruction'
      break;
    case 226:                       // 'schema-attribute'
      shiftT(226);                  // 'schema-attribute'
      break;
    case 227:                       // 'schema-element'
      shiftT(227);                  // 'schema-element'
      break;
    case 243:                       // 'switch'
      shiftT(243);                  // 'switch'
      break;
    case 244:                       // 'text'
      shiftT(244);                  // 'text'
      break;
    case 253:                       // 'typeswitch'
      shiftT(253);                  // 'typeswitch'
      break;
    case 78:                        // 'array'
      shiftT(78);                   // 'array'
      break;
    case 167:                       // 'json-item'
      shiftT(167);                  // 'json-item'
      break;
    case 242:                       // 'structured-item'
      shiftT(242);                  // 'structured-item'
      break;
    default:
      try_FunctionName();
    }
  }

  function parse_FunctionName()
  {
    eventHandler.startNonterminal("FunctionName", e0);
    switch (l1)
    {
    case 6:                         // EQName^Token
      shift(6);                     // EQName^Token
      break;
    case 70:                        // 'after'
      shift(70);                    // 'after'
      break;
    case 73:                        // 'ancestor'
      shift(73);                    // 'ancestor'
      break;
    case 74:                        // 'ancestor-or-self'
      shift(74);                    // 'ancestor-or-self'
      break;
    case 75:                        // 'and'
      shift(75);                    // 'and'
      break;
    case 79:                        // 'as'
      shift(79);                    // 'as'
      break;
    case 80:                        // 'ascending'
      shift(80);                    // 'ascending'
      break;
    case 84:                        // 'before'
      shift(84);                    // 'before'
      break;
    case 88:                        // 'case'
      shift(88);                    // 'case'
      break;
    case 89:                        // 'cast'
      shift(89);                    // 'cast'
      break;
    case 90:                        // 'castable'
      shift(90);                    // 'castable'
      break;
    case 93:                        // 'child'
      shift(93);                    // 'child'
      break;
    case 94:                        // 'collation'
      shift(94);                    // 'collation'
      break;
    case 103:                       // 'copy'
      shift(103);                   // 'copy'
      break;
    case 105:                       // 'count'
      shift(105);                   // 'count'
      break;
    case 108:                       // 'declare'
      shift(108);                   // 'declare'
      break;
    case 109:                       // 'default'
      shift(109);                   // 'default'
      break;
    case 110:                       // 'delete'
      shift(110);                   // 'delete'
      break;
    case 111:                       // 'descendant'
      shift(111);                   // 'descendant'
      break;
    case 112:                       // 'descendant-or-self'
      shift(112);                   // 'descendant-or-self'
      break;
    case 113:                       // 'descending'
      shift(113);                   // 'descending'
      break;
    case 118:                       // 'div'
      shift(118);                   // 'div'
      break;
    case 119:                       // 'document'
      shift(119);                   // 'document'
      break;
    case 122:                       // 'else'
      shift(122);                   // 'else'
      break;
    case 123:                       // 'empty'
      shift(123);                   // 'empty'
      break;
    case 126:                       // 'end'
      shift(126);                   // 'end'
      break;
    case 128:                       // 'eq'
      shift(128);                   // 'eq'
      break;
    case 129:                       // 'every'
      shift(129);                   // 'every'
      break;
    case 131:                       // 'except'
      shift(131);                   // 'except'
      break;
    case 134:                       // 'first'
      shift(134);                   // 'first'
      break;
    case 135:                       // 'following'
      shift(135);                   // 'following'
      break;
    case 136:                       // 'following-sibling'
      shift(136);                   // 'following-sibling'
      break;
    case 137:                       // 'for'
      shift(137);                   // 'for'
      break;
    case 146:                       // 'ge'
      shift(146);                   // 'ge'
      break;
    case 148:                       // 'group'
      shift(148);                   // 'group'
      break;
    case 150:                       // 'gt'
      shift(150);                   // 'gt'
      break;
    case 151:                       // 'idiv'
      shift(151);                   // 'idiv'
      break;
    case 153:                       // 'import'
      shift(153);                   // 'import'
      break;
    case 159:                       // 'insert'
      shift(159);                   // 'insert'
      break;
    case 160:                       // 'instance'
      shift(160);                   // 'instance'
      break;
    case 162:                       // 'intersect'
      shift(162);                   // 'intersect'
      break;
    case 163:                       // 'into'
      shift(163);                   // 'into'
      break;
    case 164:                       // 'is'
      shift(164);                   // 'is'
      break;
    case 170:                       // 'last'
      shift(170);                   // 'last'
      break;
    case 172:                       // 'le'
      shift(172);                   // 'le'
      break;
    case 174:                       // 'let'
      shift(174);                   // 'let'
      break;
    case 178:                       // 'lt'
      shift(178);                   // 'lt'
      break;
    case 180:                       // 'mod'
      shift(180);                   // 'mod'
      break;
    case 181:                       // 'modify'
      shift(181);                   // 'modify'
      break;
    case 182:                       // 'module'
      shift(182);                   // 'module'
      break;
    case 184:                       // 'namespace'
      shift(184);                   // 'namespace'
      break;
    case 186:                       // 'ne'
      shift(186);                   // 'ne'
      break;
    case 198:                       // 'only'
      shift(198);                   // 'only'
      break;
    case 200:                       // 'or'
      shift(200);                   // 'or'
      break;
    case 201:                       // 'order'
      shift(201);                   // 'order'
      break;
    case 202:                       // 'ordered'
      shift(202);                   // 'ordered'
      break;
    case 206:                       // 'parent'
      shift(206);                   // 'parent'
      break;
    case 212:                       // 'preceding'
      shift(212);                   // 'preceding'
      break;
    case 213:                       // 'preceding-sibling'
      shift(213);                   // 'preceding-sibling'
      break;
    case 218:                       // 'rename'
      shift(218);                   // 'rename'
      break;
    case 219:                       // 'replace'
      shift(219);                   // 'replace'
      break;
    case 220:                       // 'return'
      shift(220);                   // 'return'
      break;
    case 224:                       // 'satisfies'
      shift(224);                   // 'satisfies'
      break;
    case 229:                       // 'self'
      shift(229);                   // 'self'
      break;
    case 235:                       // 'some'
      shift(235);                   // 'some'
      break;
    case 236:                       // 'stable'
      shift(236);                   // 'stable'
      break;
    case 237:                       // 'start'
      shift(237);                   // 'start'
      break;
    case 248:                       // 'to'
      shift(248);                   // 'to'
      break;
    case 249:                       // 'treat'
      shift(249);                   // 'treat'
      break;
    case 250:                       // 'try'
      shift(250);                   // 'try'
      break;
    case 254:                       // 'union'
      shift(254);                   // 'union'
      break;
    case 256:                       // 'unordered'
      shift(256);                   // 'unordered'
      break;
    case 260:                       // 'validate'
      shift(260);                   // 'validate'
      break;
    case 266:                       // 'where'
      shift(266);                   // 'where'
      break;
    case 270:                       // 'with'
      shift(270);                   // 'with'
      break;
    case 274:                       // 'xquery'
      shift(274);                   // 'xquery'
      break;
    case 72:                        // 'allowing'
      shift(72);                    // 'allowing'
      break;
    case 81:                        // 'at'
      shift(81);                    // 'at'
      break;
    case 83:                        // 'base-uri'
      shift(83);                    // 'base-uri'
      break;
    case 85:                        // 'boundary-space'
      shift(85);                    // 'boundary-space'
      break;
    case 86:                        // 'break'
      shift(86);                    // 'break'
      break;
    case 91:                        // 'catch'
      shift(91);                    // 'catch'
      break;
    case 98:                        // 'construction'
      shift(98);                    // 'construction'
      break;
    case 101:                       // 'context'
      shift(101);                   // 'context'
      break;
    case 102:                       // 'continue'
      shift(102);                   // 'continue'
      break;
    case 104:                       // 'copy-namespaces'
      shift(104);                   // 'copy-namespaces'
      break;
    case 106:                       // 'decimal-format'
      shift(106);                   // 'decimal-format'
      break;
    case 125:                       // 'encoding'
      shift(125);                   // 'encoding'
      break;
    case 132:                       // 'exit'
      shift(132);                   // 'exit'
      break;
    case 133:                       // 'external'
      shift(133);                   // 'external'
      break;
    case 141:                       // 'ft-option'
      shift(141);                   // 'ft-option'
      break;
    case 154:                       // 'in'
      shift(154);                   // 'in'
      break;
    case 155:                       // 'index'
      shift(155);                   // 'index'
      break;
    case 161:                       // 'integrity'
      shift(161);                   // 'integrity'
      break;
    case 171:                       // 'lax'
      shift(171);                   // 'lax'
      break;
    case 192:                       // 'nodes'
      shift(192);                   // 'nodes'
      break;
    case 199:                       // 'option'
      shift(199);                   // 'option'
      break;
    case 203:                       // 'ordering'
      shift(203);                   // 'ordering'
      break;
    case 222:                       // 'revalidation'
      shift(222);                   // 'revalidation'
      break;
    case 225:                       // 'schema'
      shift(225);                   // 'schema'
      break;
    case 228:                       // 'score'
      shift(228);                   // 'score'
      break;
    case 234:                       // 'sliding'
      shift(234);                   // 'sliding'
      break;
    case 240:                       // 'strict'
      shift(240);                   // 'strict'
      break;
    case 251:                       // 'tumbling'
      shift(251);                   // 'tumbling'
      break;
    case 252:                       // 'type'
      shift(252);                   // 'type'
      break;
    case 257:                       // 'updating'
      shift(257);                   // 'updating'
      break;
    case 261:                       // 'value'
      shift(261);                   // 'value'
      break;
    case 262:                       // 'variable'
      shift(262);                   // 'variable'
      break;
    case 263:                       // 'version'
      shift(263);                   // 'version'
      break;
    case 267:                       // 'while'
      shift(267);                   // 'while'
      break;
    case 97:                        // 'constraint'
      shift(97);                    // 'constraint'
      break;
    case 176:                       // 'loop'
      shift(176);                   // 'loop'
      break;
    case 221:                       // 'returning'
      shift(221);                   // 'returning'
      break;
    case 77:                        // 'append'
      shift(77);                    // 'append'
      break;
    case 166:                       // 'json'
      shift(166);                   // 'json'
      break;
    default:
      shift(194);                   // 'object'
    }
    eventHandler.endNonterminal("FunctionName", e0);
  }

  function try_FunctionName()
  {
    switch (l1)
    {
    case 6:                         // EQName^Token
      shiftT(6);                    // EQName^Token
      break;
    case 70:                        // 'after'
      shiftT(70);                   // 'after'
      break;
    case 73:                        // 'ancestor'
      shiftT(73);                   // 'ancestor'
      break;
    case 74:                        // 'ancestor-or-self'
      shiftT(74);                   // 'ancestor-or-self'
      break;
    case 75:                        // 'and'
      shiftT(75);                   // 'and'
      break;
    case 79:                        // 'as'
      shiftT(79);                   // 'as'
      break;
    case 80:                        // 'ascending'
      shiftT(80);                   // 'ascending'
      break;
    case 84:                        // 'before'
      shiftT(84);                   // 'before'
      break;
    case 88:                        // 'case'
      shiftT(88);                   // 'case'
      break;
    case 89:                        // 'cast'
      shiftT(89);                   // 'cast'
      break;
    case 90:                        // 'castable'
      shiftT(90);                   // 'castable'
      break;
    case 93:                        // 'child'
      shiftT(93);                   // 'child'
      break;
    case 94:                        // 'collation'
      shiftT(94);                   // 'collation'
      break;
    case 103:                       // 'copy'
      shiftT(103);                  // 'copy'
      break;
    case 105:                       // 'count'
      shiftT(105);                  // 'count'
      break;
    case 108:                       // 'declare'
      shiftT(108);                  // 'declare'
      break;
    case 109:                       // 'default'
      shiftT(109);                  // 'default'
      break;
    case 110:                       // 'delete'
      shiftT(110);                  // 'delete'
      break;
    case 111:                       // 'descendant'
      shiftT(111);                  // 'descendant'
      break;
    case 112:                       // 'descendant-or-self'
      shiftT(112);                  // 'descendant-or-self'
      break;
    case 113:                       // 'descending'
      shiftT(113);                  // 'descending'
      break;
    case 118:                       // 'div'
      shiftT(118);                  // 'div'
      break;
    case 119:                       // 'document'
      shiftT(119);                  // 'document'
      break;
    case 122:                       // 'else'
      shiftT(122);                  // 'else'
      break;
    case 123:                       // 'empty'
      shiftT(123);                  // 'empty'
      break;
    case 126:                       // 'end'
      shiftT(126);                  // 'end'
      break;
    case 128:                       // 'eq'
      shiftT(128);                  // 'eq'
      break;
    case 129:                       // 'every'
      shiftT(129);                  // 'every'
      break;
    case 131:                       // 'except'
      shiftT(131);                  // 'except'
      break;
    case 134:                       // 'first'
      shiftT(134);                  // 'first'
      break;
    case 135:                       // 'following'
      shiftT(135);                  // 'following'
      break;
    case 136:                       // 'following-sibling'
      shiftT(136);                  // 'following-sibling'
      break;
    case 137:                       // 'for'
      shiftT(137);                  // 'for'
      break;
    case 146:                       // 'ge'
      shiftT(146);                  // 'ge'
      break;
    case 148:                       // 'group'
      shiftT(148);                  // 'group'
      break;
    case 150:                       // 'gt'
      shiftT(150);                  // 'gt'
      break;
    case 151:                       // 'idiv'
      shiftT(151);                  // 'idiv'
      break;
    case 153:                       // 'import'
      shiftT(153);                  // 'import'
      break;
    case 159:                       // 'insert'
      shiftT(159);                  // 'insert'
      break;
    case 160:                       // 'instance'
      shiftT(160);                  // 'instance'
      break;
    case 162:                       // 'intersect'
      shiftT(162);                  // 'intersect'
      break;
    case 163:                       // 'into'
      shiftT(163);                  // 'into'
      break;
    case 164:                       // 'is'
      shiftT(164);                  // 'is'
      break;
    case 170:                       // 'last'
      shiftT(170);                  // 'last'
      break;
    case 172:                       // 'le'
      shiftT(172);                  // 'le'
      break;
    case 174:                       // 'let'
      shiftT(174);                  // 'let'
      break;
    case 178:                       // 'lt'
      shiftT(178);                  // 'lt'
      break;
    case 180:                       // 'mod'
      shiftT(180);                  // 'mod'
      break;
    case 181:                       // 'modify'
      shiftT(181);                  // 'modify'
      break;
    case 182:                       // 'module'
      shiftT(182);                  // 'module'
      break;
    case 184:                       // 'namespace'
      shiftT(184);                  // 'namespace'
      break;
    case 186:                       // 'ne'
      shiftT(186);                  // 'ne'
      break;
    case 198:                       // 'only'
      shiftT(198);                  // 'only'
      break;
    case 200:                       // 'or'
      shiftT(200);                  // 'or'
      break;
    case 201:                       // 'order'
      shiftT(201);                  // 'order'
      break;
    case 202:                       // 'ordered'
      shiftT(202);                  // 'ordered'
      break;
    case 206:                       // 'parent'
      shiftT(206);                  // 'parent'
      break;
    case 212:                       // 'preceding'
      shiftT(212);                  // 'preceding'
      break;
    case 213:                       // 'preceding-sibling'
      shiftT(213);                  // 'preceding-sibling'
      break;
    case 218:                       // 'rename'
      shiftT(218);                  // 'rename'
      break;
    case 219:                       // 'replace'
      shiftT(219);                  // 'replace'
      break;
    case 220:                       // 'return'
      shiftT(220);                  // 'return'
      break;
    case 224:                       // 'satisfies'
      shiftT(224);                  // 'satisfies'
      break;
    case 229:                       // 'self'
      shiftT(229);                  // 'self'
      break;
    case 235:                       // 'some'
      shiftT(235);                  // 'some'
      break;
    case 236:                       // 'stable'
      shiftT(236);                  // 'stable'
      break;
    case 237:                       // 'start'
      shiftT(237);                  // 'start'
      break;
    case 248:                       // 'to'
      shiftT(248);                  // 'to'
      break;
    case 249:                       // 'treat'
      shiftT(249);                  // 'treat'
      break;
    case 250:                       // 'try'
      shiftT(250);                  // 'try'
      break;
    case 254:                       // 'union'
      shiftT(254);                  // 'union'
      break;
    case 256:                       // 'unordered'
      shiftT(256);                  // 'unordered'
      break;
    case 260:                       // 'validate'
      shiftT(260);                  // 'validate'
      break;
    case 266:                       // 'where'
      shiftT(266);                  // 'where'
      break;
    case 270:                       // 'with'
      shiftT(270);                  // 'with'
      break;
    case 274:                       // 'xquery'
      shiftT(274);                  // 'xquery'
      break;
    case 72:                        // 'allowing'
      shiftT(72);                   // 'allowing'
      break;
    case 81:                        // 'at'
      shiftT(81);                   // 'at'
      break;
    case 83:                        // 'base-uri'
      shiftT(83);                   // 'base-uri'
      break;
    case 85:                        // 'boundary-space'
      shiftT(85);                   // 'boundary-space'
      break;
    case 86:                        // 'break'
      shiftT(86);                   // 'break'
      break;
    case 91:                        // 'catch'
      shiftT(91);                   // 'catch'
      break;
    case 98:                        // 'construction'
      shiftT(98);                   // 'construction'
      break;
    case 101:                       // 'context'
      shiftT(101);                  // 'context'
      break;
    case 102:                       // 'continue'
      shiftT(102);                  // 'continue'
      break;
    case 104:                       // 'copy-namespaces'
      shiftT(104);                  // 'copy-namespaces'
      break;
    case 106:                       // 'decimal-format'
      shiftT(106);                  // 'decimal-format'
      break;
    case 125:                       // 'encoding'
      shiftT(125);                  // 'encoding'
      break;
    case 132:                       // 'exit'
      shiftT(132);                  // 'exit'
      break;
    case 133:                       // 'external'
      shiftT(133);                  // 'external'
      break;
    case 141:                       // 'ft-option'
      shiftT(141);                  // 'ft-option'
      break;
    case 154:                       // 'in'
      shiftT(154);                  // 'in'
      break;
    case 155:                       // 'index'
      shiftT(155);                  // 'index'
      break;
    case 161:                       // 'integrity'
      shiftT(161);                  // 'integrity'
      break;
    case 171:                       // 'lax'
      shiftT(171);                  // 'lax'
      break;
    case 192:                       // 'nodes'
      shiftT(192);                  // 'nodes'
      break;
    case 199:                       // 'option'
      shiftT(199);                  // 'option'
      break;
    case 203:                       // 'ordering'
      shiftT(203);                  // 'ordering'
      break;
    case 222:                       // 'revalidation'
      shiftT(222);                  // 'revalidation'
      break;
    case 225:                       // 'schema'
      shiftT(225);                  // 'schema'
      break;
    case 228:                       // 'score'
      shiftT(228);                  // 'score'
      break;
    case 234:                       // 'sliding'
      shiftT(234);                  // 'sliding'
      break;
    case 240:                       // 'strict'
      shiftT(240);                  // 'strict'
      break;
    case 251:                       // 'tumbling'
      shiftT(251);                  // 'tumbling'
      break;
    case 252:                       // 'type'
      shiftT(252);                  // 'type'
      break;
    case 257:                       // 'updating'
      shiftT(257);                  // 'updating'
      break;
    case 261:                       // 'value'
      shiftT(261);                  // 'value'
      break;
    case 262:                       // 'variable'
      shiftT(262);                  // 'variable'
      break;
    case 263:                       // 'version'
      shiftT(263);                  // 'version'
      break;
    case 267:                       // 'while'
      shiftT(267);                  // 'while'
      break;
    case 97:                        // 'constraint'
      shiftT(97);                   // 'constraint'
      break;
    case 176:                       // 'loop'
      shiftT(176);                  // 'loop'
      break;
    case 221:                       // 'returning'
      shiftT(221);                  // 'returning'
      break;
    case 77:                        // 'append'
      shiftT(77);                   // 'append'
      break;
    case 166:                       // 'json'
      shiftT(166);                  // 'json'
      break;
    default:
      shiftT(194);                  // 'object'
    }
  }

  function parse_NCName()
  {
    eventHandler.startNonterminal("NCName", e0);
    switch (l1)
    {
    case 19:                        // NCName^Token
      shift(19);                    // NCName^Token
      break;
    case 70:                        // 'after'
      shift(70);                    // 'after'
      break;
    case 75:                        // 'and'
      shift(75);                    // 'and'
      break;
    case 79:                        // 'as'
      shift(79);                    // 'as'
      break;
    case 80:                        // 'ascending'
      shift(80);                    // 'ascending'
      break;
    case 84:                        // 'before'
      shift(84);                    // 'before'
      break;
    case 88:                        // 'case'
      shift(88);                    // 'case'
      break;
    case 89:                        // 'cast'
      shift(89);                    // 'cast'
      break;
    case 90:                        // 'castable'
      shift(90);                    // 'castable'
      break;
    case 94:                        // 'collation'
      shift(94);                    // 'collation'
      break;
    case 105:                       // 'count'
      shift(105);                   // 'count'
      break;
    case 109:                       // 'default'
      shift(109);                   // 'default'
      break;
    case 113:                       // 'descending'
      shift(113);                   // 'descending'
      break;
    case 118:                       // 'div'
      shift(118);                   // 'div'
      break;
    case 122:                       // 'else'
      shift(122);                   // 'else'
      break;
    case 123:                       // 'empty'
      shift(123);                   // 'empty'
      break;
    case 126:                       // 'end'
      shift(126);                   // 'end'
      break;
    case 128:                       // 'eq'
      shift(128);                   // 'eq'
      break;
    case 131:                       // 'except'
      shift(131);                   // 'except'
      break;
    case 137:                       // 'for'
      shift(137);                   // 'for'
      break;
    case 146:                       // 'ge'
      shift(146);                   // 'ge'
      break;
    case 148:                       // 'group'
      shift(148);                   // 'group'
      break;
    case 150:                       // 'gt'
      shift(150);                   // 'gt'
      break;
    case 151:                       // 'idiv'
      shift(151);                   // 'idiv'
      break;
    case 160:                       // 'instance'
      shift(160);                   // 'instance'
      break;
    case 162:                       // 'intersect'
      shift(162);                   // 'intersect'
      break;
    case 163:                       // 'into'
      shift(163);                   // 'into'
      break;
    case 164:                       // 'is'
      shift(164);                   // 'is'
      break;
    case 172:                       // 'le'
      shift(172);                   // 'le'
      break;
    case 174:                       // 'let'
      shift(174);                   // 'let'
      break;
    case 178:                       // 'lt'
      shift(178);                   // 'lt'
      break;
    case 180:                       // 'mod'
      shift(180);                   // 'mod'
      break;
    case 181:                       // 'modify'
      shift(181);                   // 'modify'
      break;
    case 186:                       // 'ne'
      shift(186);                   // 'ne'
      break;
    case 198:                       // 'only'
      shift(198);                   // 'only'
      break;
    case 200:                       // 'or'
      shift(200);                   // 'or'
      break;
    case 201:                       // 'order'
      shift(201);                   // 'order'
      break;
    case 220:                       // 'return'
      shift(220);                   // 'return'
      break;
    case 224:                       // 'satisfies'
      shift(224);                   // 'satisfies'
      break;
    case 236:                       // 'stable'
      shift(236);                   // 'stable'
      break;
    case 237:                       // 'start'
      shift(237);                   // 'start'
      break;
    case 248:                       // 'to'
      shift(248);                   // 'to'
      break;
    case 249:                       // 'treat'
      shift(249);                   // 'treat'
      break;
    case 254:                       // 'union'
      shift(254);                   // 'union'
      break;
    case 266:                       // 'where'
      shift(266);                   // 'where'
      break;
    case 270:                       // 'with'
      shift(270);                   // 'with'
      break;
    case 73:                        // 'ancestor'
      shift(73);                    // 'ancestor'
      break;
    case 74:                        // 'ancestor-or-self'
      shift(74);                    // 'ancestor-or-self'
      break;
    case 82:                        // 'attribute'
      shift(82);                    // 'attribute'
      break;
    case 93:                        // 'child'
      shift(93);                    // 'child'
      break;
    case 96:                        // 'comment'
      shift(96);                    // 'comment'
      break;
    case 103:                       // 'copy'
      shift(103);                   // 'copy'
      break;
    case 108:                       // 'declare'
      shift(108);                   // 'declare'
      break;
    case 110:                       // 'delete'
      shift(110);                   // 'delete'
      break;
    case 111:                       // 'descendant'
      shift(111);                   // 'descendant'
      break;
    case 112:                       // 'descendant-or-self'
      shift(112);                   // 'descendant-or-self'
      break;
    case 119:                       // 'document'
      shift(119);                   // 'document'
      break;
    case 120:                       // 'document-node'
      shift(120);                   // 'document-node'
      break;
    case 121:                       // 'element'
      shift(121);                   // 'element'
      break;
    case 124:                       // 'empty-sequence'
      shift(124);                   // 'empty-sequence'
      break;
    case 129:                       // 'every'
      shift(129);                   // 'every'
      break;
    case 134:                       // 'first'
      shift(134);                   // 'first'
      break;
    case 135:                       // 'following'
      shift(135);                   // 'following'
      break;
    case 136:                       // 'following-sibling'
      shift(136);                   // 'following-sibling'
      break;
    case 145:                       // 'function'
      shift(145);                   // 'function'
      break;
    case 152:                       // 'if'
      shift(152);                   // 'if'
      break;
    case 153:                       // 'import'
      shift(153);                   // 'import'
      break;
    case 159:                       // 'insert'
      shift(159);                   // 'insert'
      break;
    case 165:                       // 'item'
      shift(165);                   // 'item'
      break;
    case 170:                       // 'last'
      shift(170);                   // 'last'
      break;
    case 182:                       // 'module'
      shift(182);                   // 'module'
      break;
    case 184:                       // 'namespace'
      shift(184);                   // 'namespace'
      break;
    case 185:                       // 'namespace-node'
      shift(185);                   // 'namespace-node'
      break;
    case 191:                       // 'node'
      shift(191);                   // 'node'
      break;
    case 202:                       // 'ordered'
      shift(202);                   // 'ordered'
      break;
    case 206:                       // 'parent'
      shift(206);                   // 'parent'
      break;
    case 212:                       // 'preceding'
      shift(212);                   // 'preceding'
      break;
    case 213:                       // 'preceding-sibling'
      shift(213);                   // 'preceding-sibling'
      break;
    case 216:                       // 'processing-instruction'
      shift(216);                   // 'processing-instruction'
      break;
    case 218:                       // 'rename'
      shift(218);                   // 'rename'
      break;
    case 219:                       // 'replace'
      shift(219);                   // 'replace'
      break;
    case 226:                       // 'schema-attribute'
      shift(226);                   // 'schema-attribute'
      break;
    case 227:                       // 'schema-element'
      shift(227);                   // 'schema-element'
      break;
    case 229:                       // 'self'
      shift(229);                   // 'self'
      break;
    case 235:                       // 'some'
      shift(235);                   // 'some'
      break;
    case 243:                       // 'switch'
      shift(243);                   // 'switch'
      break;
    case 244:                       // 'text'
      shift(244);                   // 'text'
      break;
    case 250:                       // 'try'
      shift(250);                   // 'try'
      break;
    case 253:                       // 'typeswitch'
      shift(253);                   // 'typeswitch'
      break;
    case 256:                       // 'unordered'
      shift(256);                   // 'unordered'
      break;
    case 260:                       // 'validate'
      shift(260);                   // 'validate'
      break;
    case 262:                       // 'variable'
      shift(262);                   // 'variable'
      break;
    case 274:                       // 'xquery'
      shift(274);                   // 'xquery'
      break;
    case 72:                        // 'allowing'
      shift(72);                    // 'allowing'
      break;
    case 81:                        // 'at'
      shift(81);                    // 'at'
      break;
    case 83:                        // 'base-uri'
      shift(83);                    // 'base-uri'
      break;
    case 85:                        // 'boundary-space'
      shift(85);                    // 'boundary-space'
      break;
    case 86:                        // 'break'
      shift(86);                    // 'break'
      break;
    case 91:                        // 'catch'
      shift(91);                    // 'catch'
      break;
    case 98:                        // 'construction'
      shift(98);                    // 'construction'
      break;
    case 101:                       // 'context'
      shift(101);                   // 'context'
      break;
    case 102:                       // 'continue'
      shift(102);                   // 'continue'
      break;
    case 104:                       // 'copy-namespaces'
      shift(104);                   // 'copy-namespaces'
      break;
    case 106:                       // 'decimal-format'
      shift(106);                   // 'decimal-format'
      break;
    case 125:                       // 'encoding'
      shift(125);                   // 'encoding'
      break;
    case 132:                       // 'exit'
      shift(132);                   // 'exit'
      break;
    case 133:                       // 'external'
      shift(133);                   // 'external'
      break;
    case 141:                       // 'ft-option'
      shift(141);                   // 'ft-option'
      break;
    case 154:                       // 'in'
      shift(154);                   // 'in'
      break;
    case 155:                       // 'index'
      shift(155);                   // 'index'
      break;
    case 161:                       // 'integrity'
      shift(161);                   // 'integrity'
      break;
    case 171:                       // 'lax'
      shift(171);                   // 'lax'
      break;
    case 192:                       // 'nodes'
      shift(192);                   // 'nodes'
      break;
    case 199:                       // 'option'
      shift(199);                   // 'option'
      break;
    case 203:                       // 'ordering'
      shift(203);                   // 'ordering'
      break;
    case 222:                       // 'revalidation'
      shift(222);                   // 'revalidation'
      break;
    case 225:                       // 'schema'
      shift(225);                   // 'schema'
      break;
    case 228:                       // 'score'
      shift(228);                   // 'score'
      break;
    case 234:                       // 'sliding'
      shift(234);                   // 'sliding'
      break;
    case 240:                       // 'strict'
      shift(240);                   // 'strict'
      break;
    case 251:                       // 'tumbling'
      shift(251);                   // 'tumbling'
      break;
    case 252:                       // 'type'
      shift(252);                   // 'type'
      break;
    case 257:                       // 'updating'
      shift(257);                   // 'updating'
      break;
    case 261:                       // 'value'
      shift(261);                   // 'value'
      break;
    case 263:                       // 'version'
      shift(263);                   // 'version'
      break;
    case 267:                       // 'while'
      shift(267);                   // 'while'
      break;
    case 97:                        // 'constraint'
      shift(97);                    // 'constraint'
      break;
    case 176:                       // 'loop'
      shift(176);                   // 'loop'
      break;
    case 221:                       // 'returning'
      shift(221);                   // 'returning'
      break;
    case 77:                        // 'append'
      shift(77);                    // 'append'
      break;
    case 166:                       // 'json'
      shift(166);                   // 'json'
      break;
    default:
      shift(194);                   // 'object'
    }
    eventHandler.endNonterminal("NCName", e0);
  }

  function try_NCName()
  {
    switch (l1)
    {
    case 19:                        // NCName^Token
      shiftT(19);                   // NCName^Token
      break;
    case 70:                        // 'after'
      shiftT(70);                   // 'after'
      break;
    case 75:                        // 'and'
      shiftT(75);                   // 'and'
      break;
    case 79:                        // 'as'
      shiftT(79);                   // 'as'
      break;
    case 80:                        // 'ascending'
      shiftT(80);                   // 'ascending'
      break;
    case 84:                        // 'before'
      shiftT(84);                   // 'before'
      break;
    case 88:                        // 'case'
      shiftT(88);                   // 'case'
      break;
    case 89:                        // 'cast'
      shiftT(89);                   // 'cast'
      break;
    case 90:                        // 'castable'
      shiftT(90);                   // 'castable'
      break;
    case 94:                        // 'collation'
      shiftT(94);                   // 'collation'
      break;
    case 105:                       // 'count'
      shiftT(105);                  // 'count'
      break;
    case 109:                       // 'default'
      shiftT(109);                  // 'default'
      break;
    case 113:                       // 'descending'
      shiftT(113);                  // 'descending'
      break;
    case 118:                       // 'div'
      shiftT(118);                  // 'div'
      break;
    case 122:                       // 'else'
      shiftT(122);                  // 'else'
      break;
    case 123:                       // 'empty'
      shiftT(123);                  // 'empty'
      break;
    case 126:                       // 'end'
      shiftT(126);                  // 'end'
      break;
    case 128:                       // 'eq'
      shiftT(128);                  // 'eq'
      break;
    case 131:                       // 'except'
      shiftT(131);                  // 'except'
      break;
    case 137:                       // 'for'
      shiftT(137);                  // 'for'
      break;
    case 146:                       // 'ge'
      shiftT(146);                  // 'ge'
      break;
    case 148:                       // 'group'
      shiftT(148);                  // 'group'
      break;
    case 150:                       // 'gt'
      shiftT(150);                  // 'gt'
      break;
    case 151:                       // 'idiv'
      shiftT(151);                  // 'idiv'
      break;
    case 160:                       // 'instance'
      shiftT(160);                  // 'instance'
      break;
    case 162:                       // 'intersect'
      shiftT(162);                  // 'intersect'
      break;
    case 163:                       // 'into'
      shiftT(163);                  // 'into'
      break;
    case 164:                       // 'is'
      shiftT(164);                  // 'is'
      break;
    case 172:                       // 'le'
      shiftT(172);                  // 'le'
      break;
    case 174:                       // 'let'
      shiftT(174);                  // 'let'
      break;
    case 178:                       // 'lt'
      shiftT(178);                  // 'lt'
      break;
    case 180:                       // 'mod'
      shiftT(180);                  // 'mod'
      break;
    case 181:                       // 'modify'
      shiftT(181);                  // 'modify'
      break;
    case 186:                       // 'ne'
      shiftT(186);                  // 'ne'
      break;
    case 198:                       // 'only'
      shiftT(198);                  // 'only'
      break;
    case 200:                       // 'or'
      shiftT(200);                  // 'or'
      break;
    case 201:                       // 'order'
      shiftT(201);                  // 'order'
      break;
    case 220:                       // 'return'
      shiftT(220);                  // 'return'
      break;
    case 224:                       // 'satisfies'
      shiftT(224);                  // 'satisfies'
      break;
    case 236:                       // 'stable'
      shiftT(236);                  // 'stable'
      break;
    case 237:                       // 'start'
      shiftT(237);                  // 'start'
      break;
    case 248:                       // 'to'
      shiftT(248);                  // 'to'
      break;
    case 249:                       // 'treat'
      shiftT(249);                  // 'treat'
      break;
    case 254:                       // 'union'
      shiftT(254);                  // 'union'
      break;
    case 266:                       // 'where'
      shiftT(266);                  // 'where'
      break;
    case 270:                       // 'with'
      shiftT(270);                  // 'with'
      break;
    case 73:                        // 'ancestor'
      shiftT(73);                   // 'ancestor'
      break;
    case 74:                        // 'ancestor-or-self'
      shiftT(74);                   // 'ancestor-or-self'
      break;
    case 82:                        // 'attribute'
      shiftT(82);                   // 'attribute'
      break;
    case 93:                        // 'child'
      shiftT(93);                   // 'child'
      break;
    case 96:                        // 'comment'
      shiftT(96);                   // 'comment'
      break;
    case 103:                       // 'copy'
      shiftT(103);                  // 'copy'
      break;
    case 108:                       // 'declare'
      shiftT(108);                  // 'declare'
      break;
    case 110:                       // 'delete'
      shiftT(110);                  // 'delete'
      break;
    case 111:                       // 'descendant'
      shiftT(111);                  // 'descendant'
      break;
    case 112:                       // 'descendant-or-self'
      shiftT(112);                  // 'descendant-or-self'
      break;
    case 119:                       // 'document'
      shiftT(119);                  // 'document'
      break;
    case 120:                       // 'document-node'
      shiftT(120);                  // 'document-node'
      break;
    case 121:                       // 'element'
      shiftT(121);                  // 'element'
      break;
    case 124:                       // 'empty-sequence'
      shiftT(124);                  // 'empty-sequence'
      break;
    case 129:                       // 'every'
      shiftT(129);                  // 'every'
      break;
    case 134:                       // 'first'
      shiftT(134);                  // 'first'
      break;
    case 135:                       // 'following'
      shiftT(135);                  // 'following'
      break;
    case 136:                       // 'following-sibling'
      shiftT(136);                  // 'following-sibling'
      break;
    case 145:                       // 'function'
      shiftT(145);                  // 'function'
      break;
    case 152:                       // 'if'
      shiftT(152);                  // 'if'
      break;
    case 153:                       // 'import'
      shiftT(153);                  // 'import'
      break;
    case 159:                       // 'insert'
      shiftT(159);                  // 'insert'
      break;
    case 165:                       // 'item'
      shiftT(165);                  // 'item'
      break;
    case 170:                       // 'last'
      shiftT(170);                  // 'last'
      break;
    case 182:                       // 'module'
      shiftT(182);                  // 'module'
      break;
    case 184:                       // 'namespace'
      shiftT(184);                  // 'namespace'
      break;
    case 185:                       // 'namespace-node'
      shiftT(185);                  // 'namespace-node'
      break;
    case 191:                       // 'node'
      shiftT(191);                  // 'node'
      break;
    case 202:                       // 'ordered'
      shiftT(202);                  // 'ordered'
      break;
    case 206:                       // 'parent'
      shiftT(206);                  // 'parent'
      break;
    case 212:                       // 'preceding'
      shiftT(212);                  // 'preceding'
      break;
    case 213:                       // 'preceding-sibling'
      shiftT(213);                  // 'preceding-sibling'
      break;
    case 216:                       // 'processing-instruction'
      shiftT(216);                  // 'processing-instruction'
      break;
    case 218:                       // 'rename'
      shiftT(218);                  // 'rename'
      break;
    case 219:                       // 'replace'
      shiftT(219);                  // 'replace'
      break;
    case 226:                       // 'schema-attribute'
      shiftT(226);                  // 'schema-attribute'
      break;
    case 227:                       // 'schema-element'
      shiftT(227);                  // 'schema-element'
      break;
    case 229:                       // 'self'
      shiftT(229);                  // 'self'
      break;
    case 235:                       // 'some'
      shiftT(235);                  // 'some'
      break;
    case 243:                       // 'switch'
      shiftT(243);                  // 'switch'
      break;
    case 244:                       // 'text'
      shiftT(244);                  // 'text'
      break;
    case 250:                       // 'try'
      shiftT(250);                  // 'try'
      break;
    case 253:                       // 'typeswitch'
      shiftT(253);                  // 'typeswitch'
      break;
    case 256:                       // 'unordered'
      shiftT(256);                  // 'unordered'
      break;
    case 260:                       // 'validate'
      shiftT(260);                  // 'validate'
      break;
    case 262:                       // 'variable'
      shiftT(262);                  // 'variable'
      break;
    case 274:                       // 'xquery'
      shiftT(274);                  // 'xquery'
      break;
    case 72:                        // 'allowing'
      shiftT(72);                   // 'allowing'
      break;
    case 81:                        // 'at'
      shiftT(81);                   // 'at'
      break;
    case 83:                        // 'base-uri'
      shiftT(83);                   // 'base-uri'
      break;
    case 85:                        // 'boundary-space'
      shiftT(85);                   // 'boundary-space'
      break;
    case 86:                        // 'break'
      shiftT(86);                   // 'break'
      break;
    case 91:                        // 'catch'
      shiftT(91);                   // 'catch'
      break;
    case 98:                        // 'construction'
      shiftT(98);                   // 'construction'
      break;
    case 101:                       // 'context'
      shiftT(101);                  // 'context'
      break;
    case 102:                       // 'continue'
      shiftT(102);                  // 'continue'
      break;
    case 104:                       // 'copy-namespaces'
      shiftT(104);                  // 'copy-namespaces'
      break;
    case 106:                       // 'decimal-format'
      shiftT(106);                  // 'decimal-format'
      break;
    case 125:                       // 'encoding'
      shiftT(125);                  // 'encoding'
      break;
    case 132:                       // 'exit'
      shiftT(132);                  // 'exit'
      break;
    case 133:                       // 'external'
      shiftT(133);                  // 'external'
      break;
    case 141:                       // 'ft-option'
      shiftT(141);                  // 'ft-option'
      break;
    case 154:                       // 'in'
      shiftT(154);                  // 'in'
      break;
    case 155:                       // 'index'
      shiftT(155);                  // 'index'
      break;
    case 161:                       // 'integrity'
      shiftT(161);                  // 'integrity'
      break;
    case 171:                       // 'lax'
      shiftT(171);                  // 'lax'
      break;
    case 192:                       // 'nodes'
      shiftT(192);                  // 'nodes'
      break;
    case 199:                       // 'option'
      shiftT(199);                  // 'option'
      break;
    case 203:                       // 'ordering'
      shiftT(203);                  // 'ordering'
      break;
    case 222:                       // 'revalidation'
      shiftT(222);                  // 'revalidation'
      break;
    case 225:                       // 'schema'
      shiftT(225);                  // 'schema'
      break;
    case 228:                       // 'score'
      shiftT(228);                  // 'score'
      break;
    case 234:                       // 'sliding'
      shiftT(234);                  // 'sliding'
      break;
    case 240:                       // 'strict'
      shiftT(240);                  // 'strict'
      break;
    case 251:                       // 'tumbling'
      shiftT(251);                  // 'tumbling'
      break;
    case 252:                       // 'type'
      shiftT(252);                  // 'type'
      break;
    case 257:                       // 'updating'
      shiftT(257);                  // 'updating'
      break;
    case 261:                       // 'value'
      shiftT(261);                  // 'value'
      break;
    case 263:                       // 'version'
      shiftT(263);                  // 'version'
      break;
    case 267:                       // 'while'
      shiftT(267);                  // 'while'
      break;
    case 97:                        // 'constraint'
      shiftT(97);                   // 'constraint'
      break;
    case 176:                       // 'loop'
      shiftT(176);                  // 'loop'
      break;
    case 221:                       // 'returning'
      shiftT(221);                  // 'returning'
      break;
    case 77:                        // 'append'
      shiftT(77);                   // 'append'
      break;
    case 166:                       // 'json'
      shiftT(166);                  // 'json'
      break;
    default:
      shiftT(194);                  // 'object'
    }
  }

  function parse_MainModule()
  {
    eventHandler.startNonterminal("MainModule", e0);
    parse_Prolog();
    whitespace();
    parse_Program();
    eventHandler.endNonterminal("MainModule", e0);
  }

  function parse_Program()
  {
    eventHandler.startNonterminal("Program", e0);
    parse_StatementsAndOptionalExpr();
    eventHandler.endNonterminal("Program", e0);
  }

  function parse_Statements()
  {
    eventHandler.startNonterminal("Statements", e0);
    for (;;)
    {
      lookahead1W(278);             // Wildcard | EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral |
      switch (l1)
      {
      case 34:                      // '('
        lookahead2W(269);           // Wildcard | EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral |
        break;
      case 35:                      // '(#'
        lookahead2(252);            // EQName^Token | S | 'after' | 'allowing' | 'ancestor' | 'ancestor-or-self' |
        break;
      case 46:                      // '/'
        lookahead2W(284);           // Wildcard | EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral |
        break;
      case 47:                      // '//'
        lookahead2W(265);           // Wildcard | EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral |
        break;
      case 54:                      // '<'
        lookahead2(4);              // QName
        break;
      case 55:                      // '<!--'
        lookahead2(1);              // DirCommentContents
        break;
      case 59:                      // '<?'
        lookahead2(3);              // PITarget
        break;
      case 66:                      // '@'
        lookahead2W(257);           // Wildcard | EQName^Token | S^WS | '(:' | 'after' | 'allowing' | 'ancestor' |
        break;
      case 68:                      // '['
        lookahead2W(272);           // Wildcard | EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral |
        break;
      case 77:                      // 'append'
        lookahead2W(200);           // S^WS | EOF | '!' | '!=' | '#' | '(' | '(:' | '*' | '+' | ',' | '-' | '/' | '//' |
        break;
      case 82:                      // 'attribute'
        lookahead2W(281);           // EQName^Token | S^WS | EOF | '!' | '!=' | '#' | '(' | '(:' | '*' | '+' | ',' |
        break;
      case 121:                     // 'element'
        lookahead2W(280);           // EQName^Token | S^WS | EOF | '!' | '!=' | '#' | '(' | '(:' | '*' | '+' | ',' |
        break;
      case 132:                     // 'exit'
        lookahead2W(203);           // S^WS | EOF | '!' | '!=' | '#' | '(' | '(:' | '*' | '+' | ',' | '-' | '/' | '//' |
        break;
      case 137:                     // 'for'
        lookahead2W(208);           // S^WS | EOF | '!' | '!=' | '#' | '$' | '(' | '(:' | '*' | '+' | ',' | '-' | '/' |
        break;
      case 174:                     // 'let'
        lookahead2W(205);           // S^WS | EOF | '!' | '!=' | '#' | '$' | '(' | '(:' | '*' | '+' | ',' | '-' | '/' |
        break;
      case 218:                     // 'rename'
        lookahead2W(206);           // S^WS | EOF | '!' | '!=' | '#' | '(' | '(:' | '*' | '+' | ',' | '-' | '/' | '//' |
        break;
      case 219:                     // 'replace'
        lookahead2W(207);           // S^WS | EOF | '!' | '!=' | '#' | '(' | '(:' | '*' | '+' | ',' | '-' | '/' | '//' |
        break;
      case 260:                     // 'validate'
        lookahead2W(210);           // S^WS | EOF | '!' | '!=' | '#' | '(' | '(:' | '*' | '+' | ',' | '-' | '/' | '//' |
        break;
      case 276:                     // '{'
        lookahead2W(277);           // Wildcard | EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral |
        break;
      case 278:                     // '{|'
        lookahead2W(273);           // Wildcard | EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral |
        break;
      case 5:                       // Wildcard
      case 45:                      // '..'
        lookahead2W(186);           // S^WS | EOF | '!' | '!=' | '(:' | '*' | '+' | ',' | '-' | '/' | '//' | ';' | '<' |
        break;
      case 31:                      // '$'
      case 32:                      // '%'
        lookahead2W(255);           // EQName^Token | S^WS | '(:' | 'after' | 'allowing' | 'ancestor' |
        break;
      case 40:                      // '+'
      case 42:                      // '-'
        lookahead2W(267);           // Wildcard | EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral |
        break;
      case 86:                      // 'break'
      case 102:                     // 'continue'
        lookahead2W(201);           // S^WS | EOF | '!' | '!=' | '#' | '(' | '(:' | '*' | '+' | ',' | '-' | '/' | '//' |
        break;
      case 110:                     // 'delete'
      case 159:                     // 'insert'
        lookahead2W(209);           // S^WS | EOF | '!' | '!=' | '#' | '(' | '(:' | '*' | '+' | ',' | '-' | '/' | '//' |
        break;
      case 184:                     // 'namespace'
      case 216:                     // 'processing-instruction'
        lookahead2W(268);           // NCName^Token | S^WS | EOF | '!' | '!=' | '#' | '(' | '(:' | '*' | '+' | ',' |
        break;
      case 103:                     // 'copy'
      case 129:                     // 'every'
      case 235:                     // 'some'
      case 262:                     // 'variable'
        lookahead2W(197);           // S^WS | EOF | '!' | '!=' | '#' | '$' | '(' | '(:' | '*' | '+' | ',' | '-' | '/' |
        break;
      case 8:                       // IntegerLiteral
      case 9:                       // DecimalLiteral
      case 10:                      // DoubleLiteral
      case 11:                      // StringLiteral
      case 44:                      // '.'
        lookahead2W(192);           // S^WS | EOF | '!' | '!=' | '(' | '(:' | '*' | '+' | ',' | '-' | '/' | '//' | ';' |
        break;
      case 78:                      // 'array'
      case 124:                     // 'empty-sequence'
      case 165:                     // 'item'
      case 167:                     // 'json-item'
      case 242:                     // 'structured-item'
        lookahead2W(191);           // S^WS | EOF | '!' | '!=' | '#' | '(:' | '*' | '+' | ',' | '-' | '/' | '//' | ';' |
        break;
      case 96:                      // 'comment'
      case 119:                     // 'document'
      case 202:                     // 'ordered'
      case 244:                     // 'text'
      case 250:                     // 'try'
      case 256:                     // 'unordered'
        lookahead2W(204);           // S^WS | EOF | '!' | '!=' | '#' | '(' | '(:' | '*' | '+' | ',' | '-' | '/' | '//' |
        break;
      case 73:                      // 'ancestor'
      case 74:                      // 'ancestor-or-self'
      case 93:                      // 'child'
      case 111:                     // 'descendant'
      case 112:                     // 'descendant-or-self'
      case 135:                     // 'following'
      case 136:                     // 'following-sibling'
      case 206:                     // 'parent'
      case 212:                     // 'preceding'
      case 213:                     // 'preceding-sibling'
      case 229:                     // 'self'
        lookahead2W(198);           // S^WS | EOF | '!' | '!=' | '#' | '(' | '(:' | '*' | '+' | ',' | '-' | '/' | '//' |
        break;
      case 6:                       // EQName^Token
      case 70:                      // 'after'
      case 72:                      // 'allowing'
      case 75:                      // 'and'
      case 79:                      // 'as'
      case 80:                      // 'ascending'
      case 81:                      // 'at'
      case 83:                      // 'base-uri'
      case 84:                      // 'before'
      case 85:                      // 'boundary-space'
      case 88:                      // 'case'
      case 89:                      // 'cast'
      case 90:                      // 'castable'
      case 91:                      // 'catch'
      case 94:                      // 'collation'
      case 97:                      // 'constraint'
      case 98:                      // 'construction'
      case 101:                     // 'context'
      case 104:                     // 'copy-namespaces'
      case 105:                     // 'count'
      case 106:                     // 'decimal-format'
      case 108:                     // 'declare'
      case 109:                     // 'default'
      case 113:                     // 'descending'
      case 118:                     // 'div'
      case 120:                     // 'document-node'
      case 122:                     // 'else'
      case 123:                     // 'empty'
      case 125:                     // 'encoding'
      case 126:                     // 'end'
      case 128:                     // 'eq'
      case 131:                     // 'except'
      case 133:                     // 'external'
      case 134:                     // 'first'
      case 141:                     // 'ft-option'
      case 145:                     // 'function'
      case 146:                     // 'ge'
      case 148:                     // 'group'
      case 150:                     // 'gt'
      case 151:                     // 'idiv'
      case 152:                     // 'if'
      case 153:                     // 'import'
      case 154:                     // 'in'
      case 155:                     // 'index'
      case 160:                     // 'instance'
      case 161:                     // 'integrity'
      case 162:                     // 'intersect'
      case 163:                     // 'into'
      case 164:                     // 'is'
      case 166:                     // 'json'
      case 170:                     // 'last'
      case 171:                     // 'lax'
      case 172:                     // 'le'
      case 176:                     // 'loop'
      case 178:                     // 'lt'
      case 180:                     // 'mod'
      case 181:                     // 'modify'
      case 182:                     // 'module'
      case 185:                     // 'namespace-node'
      case 186:                     // 'ne'
      case 191:                     // 'node'
      case 192:                     // 'nodes'
      case 194:                     // 'object'
      case 198:                     // 'only'
      case 199:                     // 'option'
      case 200:                     // 'or'
      case 201:                     // 'order'
      case 203:                     // 'ordering'
      case 220:                     // 'return'
      case 221:                     // 'returning'
      case 222:                     // 'revalidation'
      case 224:                     // 'satisfies'
      case 225:                     // 'schema'
      case 226:                     // 'schema-attribute'
      case 227:                     // 'schema-element'
      case 228:                     // 'score'
      case 234:                     // 'sliding'
      case 236:                     // 'stable'
      case 237:                     // 'start'
      case 240:                     // 'strict'
      case 243:                     // 'switch'
      case 248:                     // 'to'
      case 249:                     // 'treat'
      case 251:                     // 'tumbling'
      case 252:                     // 'type'
      case 253:                     // 'typeswitch'
      case 254:                     // 'union'
      case 257:                     // 'updating'
      case 261:                     // 'value'
      case 263:                     // 'version'
      case 266:                     // 'where'
      case 267:                     // 'while'
      case 270:                     // 'with'
      case 274:                     // 'xquery'
        lookahead2W(195);           // S^WS | EOF | '!' | '!=' | '#' | '(' | '(:' | '*' | '+' | ',' | '-' | '/' | '//' |
        break;
      default:
        lk = l1;
      }
      if (lk != 25                  // EOF
       && lk != 53                  // ';'
       && lk != 282                 // '}'
       && lk != 12805               // Wildcard EOF
       && lk != 12806               // EQName^Token EOF
       && lk != 12808               // IntegerLiteral EOF
       && lk != 12809               // DecimalLiteral EOF
       && lk != 12810               // DoubleLiteral EOF
       && lk != 12811               // StringLiteral EOF
       && lk != 12844               // '.' EOF
       && lk != 12845               // '..' EOF
       && lk != 12846               // '/' EOF
       && lk != 12870               // 'after' EOF
       && lk != 12872               // 'allowing' EOF
       && lk != 12873               // 'ancestor' EOF
       && lk != 12874               // 'ancestor-or-self' EOF
       && lk != 12875               // 'and' EOF
       && lk != 12877               // 'append' EOF
       && lk != 12878               // 'array' EOF
       && lk != 12879               // 'as' EOF
       && lk != 12880               // 'ascending' EOF
       && lk != 12881               // 'at' EOF
       && lk != 12882               // 'attribute' EOF
       && lk != 12883               // 'base-uri' EOF
       && lk != 12884               // 'before' EOF
       && lk != 12885               // 'boundary-space' EOF
       && lk != 12886               // 'break' EOF
       && lk != 12888               // 'case' EOF
       && lk != 12889               // 'cast' EOF
       && lk != 12890               // 'castable' EOF
       && lk != 12891               // 'catch' EOF
       && lk != 12893               // 'child' EOF
       && lk != 12894               // 'collation' EOF
       && lk != 12896               // 'comment' EOF
       && lk != 12897               // 'constraint' EOF
       && lk != 12898               // 'construction' EOF
       && lk != 12901               // 'context' EOF
       && lk != 12902               // 'continue' EOF
       && lk != 12903               // 'copy' EOF
       && lk != 12904               // 'copy-namespaces' EOF
       && lk != 12905               // 'count' EOF
       && lk != 12906               // 'decimal-format' EOF
       && lk != 12908               // 'declare' EOF
       && lk != 12909               // 'default' EOF
       && lk != 12910               // 'delete' EOF
       && lk != 12911               // 'descendant' EOF
       && lk != 12912               // 'descendant-or-self' EOF
       && lk != 12913               // 'descending' EOF
       && lk != 12918               // 'div' EOF
       && lk != 12919               // 'document' EOF
       && lk != 12920               // 'document-node' EOF
       && lk != 12921               // 'element' EOF
       && lk != 12922               // 'else' EOF
       && lk != 12923               // 'empty' EOF
       && lk != 12924               // 'empty-sequence' EOF
       && lk != 12925               // 'encoding' EOF
       && lk != 12926               // 'end' EOF
       && lk != 12928               // 'eq' EOF
       && lk != 12929               // 'every' EOF
       && lk != 12931               // 'except' EOF
       && lk != 12932               // 'exit' EOF
       && lk != 12933               // 'external' EOF
       && lk != 12934               // 'first' EOF
       && lk != 12935               // 'following' EOF
       && lk != 12936               // 'following-sibling' EOF
       && lk != 12937               // 'for' EOF
       && lk != 12941               // 'ft-option' EOF
       && lk != 12945               // 'function' EOF
       && lk != 12946               // 'ge' EOF
       && lk != 12948               // 'group' EOF
       && lk != 12950               // 'gt' EOF
       && lk != 12951               // 'idiv' EOF
       && lk != 12952               // 'if' EOF
       && lk != 12953               // 'import' EOF
       && lk != 12954               // 'in' EOF
       && lk != 12955               // 'index' EOF
       && lk != 12959               // 'insert' EOF
       && lk != 12960               // 'instance' EOF
       && lk != 12961               // 'integrity' EOF
       && lk != 12962               // 'intersect' EOF
       && lk != 12963               // 'into' EOF
       && lk != 12964               // 'is' EOF
       && lk != 12965               // 'item' EOF
       && lk != 12966               // 'json' EOF
       && lk != 12967               // 'json-item' EOF
       && lk != 12970               // 'last' EOF
       && lk != 12971               // 'lax' EOF
       && lk != 12972               // 'le' EOF
       && lk != 12974               // 'let' EOF
       && lk != 12976               // 'loop' EOF
       && lk != 12978               // 'lt' EOF
       && lk != 12980               // 'mod' EOF
       && lk != 12981               // 'modify' EOF
       && lk != 12982               // 'module' EOF
       && lk != 12984               // 'namespace' EOF
       && lk != 12985               // 'namespace-node' EOF
       && lk != 12986               // 'ne' EOF
       && lk != 12991               // 'node' EOF
       && lk != 12992               // 'nodes' EOF
       && lk != 12994               // 'object' EOF
       && lk != 12998               // 'only' EOF
       && lk != 12999               // 'option' EOF
       && lk != 13000               // 'or' EOF
       && lk != 13001               // 'order' EOF
       && lk != 13002               // 'ordered' EOF
       && lk != 13003               // 'ordering' EOF
       && lk != 13006               // 'parent' EOF
       && lk != 13012               // 'preceding' EOF
       && lk != 13013               // 'preceding-sibling' EOF
       && lk != 13016               // 'processing-instruction' EOF
       && lk != 13018               // 'rename' EOF
       && lk != 13019               // 'replace' EOF
       && lk != 13020               // 'return' EOF
       && lk != 13021               // 'returning' EOF
       && lk != 13022               // 'revalidation' EOF
       && lk != 13024               // 'satisfies' EOF
       && lk != 13025               // 'schema' EOF
       && lk != 13026               // 'schema-attribute' EOF
       && lk != 13027               // 'schema-element' EOF
       && lk != 13028               // 'score' EOF
       && lk != 13029               // 'self' EOF
       && lk != 13034               // 'sliding' EOF
       && lk != 13035               // 'some' EOF
       && lk != 13036               // 'stable' EOF
       && lk != 13037               // 'start' EOF
       && lk != 13040               // 'strict' EOF
       && lk != 13042               // 'structured-item' EOF
       && lk != 13043               // 'switch' EOF
       && lk != 13044               // 'text' EOF
       && lk != 13048               // 'to' EOF
       && lk != 13049               // 'treat' EOF
       && lk != 13050               // 'try' EOF
       && lk != 13051               // 'tumbling' EOF
       && lk != 13052               // 'type' EOF
       && lk != 13053               // 'typeswitch' EOF
       && lk != 13054               // 'union' EOF
       && lk != 13056               // 'unordered' EOF
       && lk != 13057               // 'updating' EOF
       && lk != 13060               // 'validate' EOF
       && lk != 13061               // 'value' EOF
       && lk != 13062               // 'variable' EOF
       && lk != 13063               // 'version' EOF
       && lk != 13066               // 'where' EOF
       && lk != 13067               // 'while' EOF
       && lk != 13070               // 'with' EOF
       && lk != 13074               // 'xquery' EOF
       && lk != 16134               // 'variable' '$'
       && lk != 20997               // Wildcard ','
       && lk != 20998               // EQName^Token ','
       && lk != 21000               // IntegerLiteral ','
       && lk != 21001               // DecimalLiteral ','
       && lk != 21002               // DoubleLiteral ','
       && lk != 21003               // StringLiteral ','
       && lk != 21036               // '.' ','
       && lk != 21037               // '..' ','
       && lk != 21038               // '/' ','
       && lk != 21062               // 'after' ','
       && lk != 21064               // 'allowing' ','
       && lk != 21065               // 'ancestor' ','
       && lk != 21066               // 'ancestor-or-self' ','
       && lk != 21067               // 'and' ','
       && lk != 21069               // 'append' ','
       && lk != 21070               // 'array' ','
       && lk != 21071               // 'as' ','
       && lk != 21072               // 'ascending' ','
       && lk != 21073               // 'at' ','
       && lk != 21074               // 'attribute' ','
       && lk != 21075               // 'base-uri' ','
       && lk != 21076               // 'before' ','
       && lk != 21077               // 'boundary-space' ','
       && lk != 21078               // 'break' ','
       && lk != 21080               // 'case' ','
       && lk != 21081               // 'cast' ','
       && lk != 21082               // 'castable' ','
       && lk != 21083               // 'catch' ','
       && lk != 21085               // 'child' ','
       && lk != 21086               // 'collation' ','
       && lk != 21088               // 'comment' ','
       && lk != 21089               // 'constraint' ','
       && lk != 21090               // 'construction' ','
       && lk != 21093               // 'context' ','
       && lk != 21094               // 'continue' ','
       && lk != 21095               // 'copy' ','
       && lk != 21096               // 'copy-namespaces' ','
       && lk != 21097               // 'count' ','
       && lk != 21098               // 'decimal-format' ','
       && lk != 21100               // 'declare' ','
       && lk != 21101               // 'default' ','
       && lk != 21102               // 'delete' ','
       && lk != 21103               // 'descendant' ','
       && lk != 21104               // 'descendant-or-self' ','
       && lk != 21105               // 'descending' ','
       && lk != 21110               // 'div' ','
       && lk != 21111               // 'document' ','
       && lk != 21112               // 'document-node' ','
       && lk != 21113               // 'element' ','
       && lk != 21114               // 'else' ','
       && lk != 21115               // 'empty' ','
       && lk != 21116               // 'empty-sequence' ','
       && lk != 21117               // 'encoding' ','
       && lk != 21118               // 'end' ','
       && lk != 21120               // 'eq' ','
       && lk != 21121               // 'every' ','
       && lk != 21123               // 'except' ','
       && lk != 21124               // 'exit' ','
       && lk != 21125               // 'external' ','
       && lk != 21126               // 'first' ','
       && lk != 21127               // 'following' ','
       && lk != 21128               // 'following-sibling' ','
       && lk != 21129               // 'for' ','
       && lk != 21133               // 'ft-option' ','
       && lk != 21137               // 'function' ','
       && lk != 21138               // 'ge' ','
       && lk != 21140               // 'group' ','
       && lk != 21142               // 'gt' ','
       && lk != 21143               // 'idiv' ','
       && lk != 21144               // 'if' ','
       && lk != 21145               // 'import' ','
       && lk != 21146               // 'in' ','
       && lk != 21147               // 'index' ','
       && lk != 21151               // 'insert' ','
       && lk != 21152               // 'instance' ','
       && lk != 21153               // 'integrity' ','
       && lk != 21154               // 'intersect' ','
       && lk != 21155               // 'into' ','
       && lk != 21156               // 'is' ','
       && lk != 21157               // 'item' ','
       && lk != 21158               // 'json' ','
       && lk != 21159               // 'json-item' ','
       && lk != 21162               // 'last' ','
       && lk != 21163               // 'lax' ','
       && lk != 21164               // 'le' ','
       && lk != 21166               // 'let' ','
       && lk != 21168               // 'loop' ','
       && lk != 21170               // 'lt' ','
       && lk != 21172               // 'mod' ','
       && lk != 21173               // 'modify' ','
       && lk != 21174               // 'module' ','
       && lk != 21176               // 'namespace' ','
       && lk != 21177               // 'namespace-node' ','
       && lk != 21178               // 'ne' ','
       && lk != 21183               // 'node' ','
       && lk != 21184               // 'nodes' ','
       && lk != 21186               // 'object' ','
       && lk != 21190               // 'only' ','
       && lk != 21191               // 'option' ','
       && lk != 21192               // 'or' ','
       && lk != 21193               // 'order' ','
       && lk != 21194               // 'ordered' ','
       && lk != 21195               // 'ordering' ','
       && lk != 21198               // 'parent' ','
       && lk != 21204               // 'preceding' ','
       && lk != 21205               // 'preceding-sibling' ','
       && lk != 21208               // 'processing-instruction' ','
       && lk != 21210               // 'rename' ','
       && lk != 21211               // 'replace' ','
       && lk != 21212               // 'return' ','
       && lk != 21213               // 'returning' ','
       && lk != 21214               // 'revalidation' ','
       && lk != 21216               // 'satisfies' ','
       && lk != 21217               // 'schema' ','
       && lk != 21218               // 'schema-attribute' ','
       && lk != 21219               // 'schema-element' ','
       && lk != 21220               // 'score' ','
       && lk != 21221               // 'self' ','
       && lk != 21226               // 'sliding' ','
       && lk != 21227               // 'some' ','
       && lk != 21228               // 'stable' ','
       && lk != 21229               // 'start' ','
       && lk != 21232               // 'strict' ','
       && lk != 21234               // 'structured-item' ','
       && lk != 21235               // 'switch' ','
       && lk != 21236               // 'text' ','
       && lk != 21240               // 'to' ','
       && lk != 21241               // 'treat' ','
       && lk != 21242               // 'try' ','
       && lk != 21243               // 'tumbling' ','
       && lk != 21244               // 'type' ','
       && lk != 21245               // 'typeswitch' ','
       && lk != 21246               // 'union' ','
       && lk != 21248               // 'unordered' ','
       && lk != 21249               // 'updating' ','
       && lk != 21252               // 'validate' ','
       && lk != 21253               // 'value' ','
       && lk != 21254               // 'variable' ','
       && lk != 21255               // 'version' ','
       && lk != 21258               // 'where' ','
       && lk != 21259               // 'while' ','
       && lk != 21262               // 'with' ','
       && lk != 21266               // 'xquery' ','
       && lk != 27141               // Wildcard ';'
       && lk != 27142               // EQName^Token ';'
       && lk != 27144               // IntegerLiteral ';'
       && lk != 27145               // DecimalLiteral ';'
       && lk != 27146               // DoubleLiteral ';'
       && lk != 27147               // StringLiteral ';'
       && lk != 27180               // '.' ';'
       && lk != 27181               // '..' ';'
       && lk != 27182               // '/' ';'
       && lk != 27206               // 'after' ';'
       && lk != 27208               // 'allowing' ';'
       && lk != 27209               // 'ancestor' ';'
       && lk != 27210               // 'ancestor-or-self' ';'
       && lk != 27211               // 'and' ';'
       && lk != 27213               // 'append' ';'
       && lk != 27214               // 'array' ';'
       && lk != 27215               // 'as' ';'
       && lk != 27216               // 'ascending' ';'
       && lk != 27217               // 'at' ';'
       && lk != 27218               // 'attribute' ';'
       && lk != 27219               // 'base-uri' ';'
       && lk != 27220               // 'before' ';'
       && lk != 27221               // 'boundary-space' ';'
       && lk != 27222               // 'break' ';'
       && lk != 27224               // 'case' ';'
       && lk != 27225               // 'cast' ';'
       && lk != 27226               // 'castable' ';'
       && lk != 27227               // 'catch' ';'
       && lk != 27229               // 'child' ';'
       && lk != 27230               // 'collation' ';'
       && lk != 27232               // 'comment' ';'
       && lk != 27233               // 'constraint' ';'
       && lk != 27234               // 'construction' ';'
       && lk != 27237               // 'context' ';'
       && lk != 27238               // 'continue' ';'
       && lk != 27239               // 'copy' ';'
       && lk != 27240               // 'copy-namespaces' ';'
       && lk != 27241               // 'count' ';'
       && lk != 27242               // 'decimal-format' ';'
       && lk != 27244               // 'declare' ';'
       && lk != 27245               // 'default' ';'
       && lk != 27246               // 'delete' ';'
       && lk != 27247               // 'descendant' ';'
       && lk != 27248               // 'descendant-or-self' ';'
       && lk != 27249               // 'descending' ';'
       && lk != 27254               // 'div' ';'
       && lk != 27255               // 'document' ';'
       && lk != 27256               // 'document-node' ';'
       && lk != 27257               // 'element' ';'
       && lk != 27258               // 'else' ';'
       && lk != 27259               // 'empty' ';'
       && lk != 27260               // 'empty-sequence' ';'
       && lk != 27261               // 'encoding' ';'
       && lk != 27262               // 'end' ';'
       && lk != 27264               // 'eq' ';'
       && lk != 27265               // 'every' ';'
       && lk != 27267               // 'except' ';'
       && lk != 27268               // 'exit' ';'
       && lk != 27269               // 'external' ';'
       && lk != 27270               // 'first' ';'
       && lk != 27271               // 'following' ';'
       && lk != 27272               // 'following-sibling' ';'
       && lk != 27273               // 'for' ';'
       && lk != 27277               // 'ft-option' ';'
       && lk != 27281               // 'function' ';'
       && lk != 27282               // 'ge' ';'
       && lk != 27284               // 'group' ';'
       && lk != 27286               // 'gt' ';'
       && lk != 27287               // 'idiv' ';'
       && lk != 27288               // 'if' ';'
       && lk != 27289               // 'import' ';'
       && lk != 27290               // 'in' ';'
       && lk != 27291               // 'index' ';'
       && lk != 27295               // 'insert' ';'
       && lk != 27296               // 'instance' ';'
       && lk != 27297               // 'integrity' ';'
       && lk != 27298               // 'intersect' ';'
       && lk != 27299               // 'into' ';'
       && lk != 27300               // 'is' ';'
       && lk != 27301               // 'item' ';'
       && lk != 27302               // 'json' ';'
       && lk != 27303               // 'json-item' ';'
       && lk != 27306               // 'last' ';'
       && lk != 27307               // 'lax' ';'
       && lk != 27308               // 'le' ';'
       && lk != 27310               // 'let' ';'
       && lk != 27312               // 'loop' ';'
       && lk != 27314               // 'lt' ';'
       && lk != 27316               // 'mod' ';'
       && lk != 27317               // 'modify' ';'
       && lk != 27318               // 'module' ';'
       && lk != 27320               // 'namespace' ';'
       && lk != 27321               // 'namespace-node' ';'
       && lk != 27322               // 'ne' ';'
       && lk != 27327               // 'node' ';'
       && lk != 27328               // 'nodes' ';'
       && lk != 27330               // 'object' ';'
       && lk != 27334               // 'only' ';'
       && lk != 27335               // 'option' ';'
       && lk != 27336               // 'or' ';'
       && lk != 27337               // 'order' ';'
       && lk != 27338               // 'ordered' ';'
       && lk != 27339               // 'ordering' ';'
       && lk != 27342               // 'parent' ';'
       && lk != 27348               // 'preceding' ';'
       && lk != 27349               // 'preceding-sibling' ';'
       && lk != 27352               // 'processing-instruction' ';'
       && lk != 27354               // 'rename' ';'
       && lk != 27355               // 'replace' ';'
       && lk != 27356               // 'return' ';'
       && lk != 27357               // 'returning' ';'
       && lk != 27358               // 'revalidation' ';'
       && lk != 27360               // 'satisfies' ';'
       && lk != 27361               // 'schema' ';'
       && lk != 27362               // 'schema-attribute' ';'
       && lk != 27363               // 'schema-element' ';'
       && lk != 27364               // 'score' ';'
       && lk != 27365               // 'self' ';'
       && lk != 27370               // 'sliding' ';'
       && lk != 27371               // 'some' ';'
       && lk != 27372               // 'stable' ';'
       && lk != 27373               // 'start' ';'
       && lk != 27376               // 'strict' ';'
       && lk != 27378               // 'structured-item' ';'
       && lk != 27379               // 'switch' ';'
       && lk != 27380               // 'text' ';'
       && lk != 27384               // 'to' ';'
       && lk != 27385               // 'treat' ';'
       && lk != 27386               // 'try' ';'
       && lk != 27387               // 'tumbling' ';'
       && lk != 27388               // 'type' ';'
       && lk != 27389               // 'typeswitch' ';'
       && lk != 27390               // 'union' ';'
       && lk != 27392               // 'unordered' ';'
       && lk != 27393               // 'updating' ';'
       && lk != 27396               // 'validate' ';'
       && lk != 27397               // 'value' ';'
       && lk != 27398               // 'variable' ';'
       && lk != 27399               // 'version' ';'
       && lk != 27402               // 'where' ';'
       && lk != 27403               // 'while' ';'
       && lk != 27406               // 'with' ';'
       && lk != 27410               // 'xquery' ';'
       && lk != 90198               // 'break' 'loop'
       && lk != 90214               // 'continue' 'loop'
       && lk != 113284              // 'exit' 'returning'
       && lk != 144389              // Wildcard '}'
       && lk != 144390              // EQName^Token '}'
       && lk != 144392              // IntegerLiteral '}'
       && lk != 144393              // DecimalLiteral '}'
       && lk != 144394              // DoubleLiteral '}'
       && lk != 144395              // StringLiteral '}'
       && lk != 144428              // '.' '}'
       && lk != 144429              // '..' '}'
       && lk != 144430              // '/' '}'
       && lk != 144454              // 'after' '}'
       && lk != 144456              // 'allowing' '}'
       && lk != 144457              // 'ancestor' '}'
       && lk != 144458              // 'ancestor-or-self' '}'
       && lk != 144459              // 'and' '}'
       && lk != 144461              // 'append' '}'
       && lk != 144462              // 'array' '}'
       && lk != 144463              // 'as' '}'
       && lk != 144464              // 'ascending' '}'
       && lk != 144465              // 'at' '}'
       && lk != 144466              // 'attribute' '}'
       && lk != 144467              // 'base-uri' '}'
       && lk != 144468              // 'before' '}'
       && lk != 144469              // 'boundary-space' '}'
       && lk != 144470              // 'break' '}'
       && lk != 144472              // 'case' '}'
       && lk != 144473              // 'cast' '}'
       && lk != 144474              // 'castable' '}'
       && lk != 144475              // 'catch' '}'
       && lk != 144477              // 'child' '}'
       && lk != 144478              // 'collation' '}'
       && lk != 144480              // 'comment' '}'
       && lk != 144481              // 'constraint' '}'
       && lk != 144482              // 'construction' '}'
       && lk != 144485              // 'context' '}'
       && lk != 144486              // 'continue' '}'
       && lk != 144487              // 'copy' '}'
       && lk != 144488              // 'copy-namespaces' '}'
       && lk != 144489              // 'count' '}'
       && lk != 144490              // 'decimal-format' '}'
       && lk != 144492              // 'declare' '}'
       && lk != 144493              // 'default' '}'
       && lk != 144494              // 'delete' '}'
       && lk != 144495              // 'descendant' '}'
       && lk != 144496              // 'descendant-or-self' '}'
       && lk != 144497              // 'descending' '}'
       && lk != 144502              // 'div' '}'
       && lk != 144503              // 'document' '}'
       && lk != 144504              // 'document-node' '}'
       && lk != 144505              // 'element' '}'
       && lk != 144506              // 'else' '}'
       && lk != 144507              // 'empty' '}'
       && lk != 144508              // 'empty-sequence' '}'
       && lk != 144509              // 'encoding' '}'
       && lk != 144510              // 'end' '}'
       && lk != 144512              // 'eq' '}'
       && lk != 144513              // 'every' '}'
       && lk != 144515              // 'except' '}'
       && lk != 144516              // 'exit' '}'
       && lk != 144517              // 'external' '}'
       && lk != 144518              // 'first' '}'
       && lk != 144519              // 'following' '}'
       && lk != 144520              // 'following-sibling' '}'
       && lk != 144521              // 'for' '}'
       && lk != 144525              // 'ft-option' '}'
       && lk != 144529              // 'function' '}'
       && lk != 144530              // 'ge' '}'
       && lk != 144532              // 'group' '}'
       && lk != 144534              // 'gt' '}'
       && lk != 144535              // 'idiv' '}'
       && lk != 144536              // 'if' '}'
       && lk != 144537              // 'import' '}'
       && lk != 144538              // 'in' '}'
       && lk != 144539              // 'index' '}'
       && lk != 144543              // 'insert' '}'
       && lk != 144544              // 'instance' '}'
       && lk != 144545              // 'integrity' '}'
       && lk != 144546              // 'intersect' '}'
       && lk != 144547              // 'into' '}'
       && lk != 144548              // 'is' '}'
       && lk != 144549              // 'item' '}'
       && lk != 144550              // 'json' '}'
       && lk != 144551              // 'json-item' '}'
       && lk != 144554              // 'last' '}'
       && lk != 144555              // 'lax' '}'
       && lk != 144556              // 'le' '}'
       && lk != 144558              // 'let' '}'
       && lk != 144560              // 'loop' '}'
       && lk != 144562              // 'lt' '}'
       && lk != 144564              // 'mod' '}'
       && lk != 144565              // 'modify' '}'
       && lk != 144566              // 'module' '}'
       && lk != 144568              // 'namespace' '}'
       && lk != 144569              // 'namespace-node' '}'
       && lk != 144570              // 'ne' '}'
       && lk != 144575              // 'node' '}'
       && lk != 144576              // 'nodes' '}'
       && lk != 144578              // 'object' '}'
       && lk != 144582              // 'only' '}'
       && lk != 144583              // 'option' '}'
       && lk != 144584              // 'or' '}'
       && lk != 144585              // 'order' '}'
       && lk != 144586              // 'ordered' '}'
       && lk != 144587              // 'ordering' '}'
       && lk != 144590              // 'parent' '}'
       && lk != 144596              // 'preceding' '}'
       && lk != 144597              // 'preceding-sibling' '}'
       && lk != 144600              // 'processing-instruction' '}'
       && lk != 144602              // 'rename' '}'
       && lk != 144603              // 'replace' '}'
       && lk != 144604              // 'return' '}'
       && lk != 144605              // 'returning' '}'
       && lk != 144606              // 'revalidation' '}'
       && lk != 144608              // 'satisfies' '}'
       && lk != 144609              // 'schema' '}'
       && lk != 144610              // 'schema-attribute' '}'
       && lk != 144611              // 'schema-element' '}'
       && lk != 144612              // 'score' '}'
       && lk != 144613              // 'self' '}'
       && lk != 144618              // 'sliding' '}'
       && lk != 144619              // 'some' '}'
       && lk != 144620              // 'stable' '}'
       && lk != 144621              // 'start' '}'
       && lk != 144624              // 'strict' '}'
       && lk != 144626              // 'structured-item' '}'
       && lk != 144627              // 'switch' '}'
       && lk != 144628              // 'text' '}'
       && lk != 144632              // 'to' '}'
       && lk != 144633              // 'treat' '}'
       && lk != 144634              // 'try' '}'
       && lk != 144635              // 'tumbling' '}'
       && lk != 144636              // 'type' '}'
       && lk != 144637              // 'typeswitch' '}'
       && lk != 144638              // 'union' '}'
       && lk != 144640              // 'unordered' '}'
       && lk != 144641              // 'updating' '}'
       && lk != 144644              // 'validate' '}'
       && lk != 144645              // 'value' '}'
       && lk != 144646              // 'variable' '}'
       && lk != 144647              // 'version' '}'
       && lk != 144650              // 'where' '}'
       && lk != 144651              // 'while' '}'
       && lk != 144654              // 'with' '}'
       && lk != 144658)             // 'xquery' '}'
      {
        lk = memoized(6, e0);
        if (lk == 0)
        {
          var b0A = b0; var e0A = e0; var l1A = l1;
          var b1A = b1; var e1A = e1; var l2A = l2;
          var b2A = b2; var e2A = e2;
          try
          {
            try_Statement();
            lk = -1;
          }
          catch (p1A)
          {
            lk = -2;
          }
          b0 = b0A; e0 = e0A; l1 = l1A; if (l1 == 0) {end = e0A;} else {
          b1 = b1A; e1 = e1A; l2 = l2A; if (l2 == 0) {end = e1A;} else {
          b2 = b2A; e2 = e2A; end = e2A; }}
          memoize(6, e0, lk);
        }
      }
      if (lk != -1
       && lk != 53                  // ';'
       && lk != 16134               // 'variable' '$'
       && lk != 27141               // Wildcard ';'
       && lk != 27142               // EQName^Token ';'
       && lk != 27144               // IntegerLiteral ';'
       && lk != 27145               // DecimalLiteral ';'
       && lk != 27146               // DoubleLiteral ';'
       && lk != 27147               // StringLiteral ';'
       && lk != 27180               // '.' ';'
       && lk != 27181               // '..' ';'
       && lk != 27182               // '/' ';'
       && lk != 27206               // 'after' ';'
       && lk != 27208               // 'allowing' ';'
       && lk != 27209               // 'ancestor' ';'
       && lk != 27210               // 'ancestor-or-self' ';'
       && lk != 27211               // 'and' ';'
       && lk != 27213               // 'append' ';'
       && lk != 27214               // 'array' ';'
       && lk != 27215               // 'as' ';'
       && lk != 27216               // 'ascending' ';'
       && lk != 27217               // 'at' ';'
       && lk != 27218               // 'attribute' ';'
       && lk != 27219               // 'base-uri' ';'
       && lk != 27220               // 'before' ';'
       && lk != 27221               // 'boundary-space' ';'
       && lk != 27222               // 'break' ';'
       && lk != 27224               // 'case' ';'
       && lk != 27225               // 'cast' ';'
       && lk != 27226               // 'castable' ';'
       && lk != 27227               // 'catch' ';'
       && lk != 27229               // 'child' ';'
       && lk != 27230               // 'collation' ';'
       && lk != 27232               // 'comment' ';'
       && lk != 27233               // 'constraint' ';'
       && lk != 27234               // 'construction' ';'
       && lk != 27237               // 'context' ';'
       && lk != 27238               // 'continue' ';'
       && lk != 27239               // 'copy' ';'
       && lk != 27240               // 'copy-namespaces' ';'
       && lk != 27241               // 'count' ';'
       && lk != 27242               // 'decimal-format' ';'
       && lk != 27244               // 'declare' ';'
       && lk != 27245               // 'default' ';'
       && lk != 27246               // 'delete' ';'
       && lk != 27247               // 'descendant' ';'
       && lk != 27248               // 'descendant-or-self' ';'
       && lk != 27249               // 'descending' ';'
       && lk != 27254               // 'div' ';'
       && lk != 27255               // 'document' ';'
       && lk != 27256               // 'document-node' ';'
       && lk != 27257               // 'element' ';'
       && lk != 27258               // 'else' ';'
       && lk != 27259               // 'empty' ';'
       && lk != 27260               // 'empty-sequence' ';'
       && lk != 27261               // 'encoding' ';'
       && lk != 27262               // 'end' ';'
       && lk != 27264               // 'eq' ';'
       && lk != 27265               // 'every' ';'
       && lk != 27267               // 'except' ';'
       && lk != 27268               // 'exit' ';'
       && lk != 27269               // 'external' ';'
       && lk != 27270               // 'first' ';'
       && lk != 27271               // 'following' ';'
       && lk != 27272               // 'following-sibling' ';'
       && lk != 27273               // 'for' ';'
       && lk != 27277               // 'ft-option' ';'
       && lk != 27281               // 'function' ';'
       && lk != 27282               // 'ge' ';'
       && lk != 27284               // 'group' ';'
       && lk != 27286               // 'gt' ';'
       && lk != 27287               // 'idiv' ';'
       && lk != 27288               // 'if' ';'
       && lk != 27289               // 'import' ';'
       && lk != 27290               // 'in' ';'
       && lk != 27291               // 'index' ';'
       && lk != 27295               // 'insert' ';'
       && lk != 27296               // 'instance' ';'
       && lk != 27297               // 'integrity' ';'
       && lk != 27298               // 'intersect' ';'
       && lk != 27299               // 'into' ';'
       && lk != 27300               // 'is' ';'
       && lk != 27301               // 'item' ';'
       && lk != 27302               // 'json' ';'
       && lk != 27303               // 'json-item' ';'
       && lk != 27306               // 'last' ';'
       && lk != 27307               // 'lax' ';'
       && lk != 27308               // 'le' ';'
       && lk != 27310               // 'let' ';'
       && lk != 27312               // 'loop' ';'
       && lk != 27314               // 'lt' ';'
       && lk != 27316               // 'mod' ';'
       && lk != 27317               // 'modify' ';'
       && lk != 27318               // 'module' ';'
       && lk != 27320               // 'namespace' ';'
       && lk != 27321               // 'namespace-node' ';'
       && lk != 27322               // 'ne' ';'
       && lk != 27327               // 'node' ';'
       && lk != 27328               // 'nodes' ';'
       && lk != 27330               // 'object' ';'
       && lk != 27334               // 'only' ';'
       && lk != 27335               // 'option' ';'
       && lk != 27336               // 'or' ';'
       && lk != 27337               // 'order' ';'
       && lk != 27338               // 'ordered' ';'
       && lk != 27339               // 'ordering' ';'
       && lk != 27342               // 'parent' ';'
       && lk != 27348               // 'preceding' ';'
       && lk != 27349               // 'preceding-sibling' ';'
       && lk != 27352               // 'processing-instruction' ';'
       && lk != 27354               // 'rename' ';'
       && lk != 27355               // 'replace' ';'
       && lk != 27356               // 'return' ';'
       && lk != 27357               // 'returning' ';'
       && lk != 27358               // 'revalidation' ';'
       && lk != 27360               // 'satisfies' ';'
       && lk != 27361               // 'schema' ';'
       && lk != 27362               // 'schema-attribute' ';'
       && lk != 27363               // 'schema-element' ';'
       && lk != 27364               // 'score' ';'
       && lk != 27365               // 'self' ';'
       && lk != 27370               // 'sliding' ';'
       && lk != 27371               // 'some' ';'
       && lk != 27372               // 'stable' ';'
       && lk != 27373               // 'start' ';'
       && lk != 27376               // 'strict' ';'
       && lk != 27378               // 'structured-item' ';'
       && lk != 27379               // 'switch' ';'
       && lk != 27380               // 'text' ';'
       && lk != 27384               // 'to' ';'
       && lk != 27385               // 'treat' ';'
       && lk != 27386               // 'try' ';'
       && lk != 27387               // 'tumbling' ';'
       && lk != 27388               // 'type' ';'
       && lk != 27389               // 'typeswitch' ';'
       && lk != 27390               // 'union' ';'
       && lk != 27392               // 'unordered' ';'
       && lk != 27393               // 'updating' ';'
       && lk != 27396               // 'validate' ';'
       && lk != 27397               // 'value' ';'
       && lk != 27398               // 'variable' ';'
       && lk != 27399               // 'version' ';'
       && lk != 27402               // 'where' ';'
       && lk != 27403               // 'while' ';'
       && lk != 27406               // 'with' ';'
       && lk != 27410               // 'xquery' ';'
       && lk != 90198               // 'break' 'loop'
       && lk != 90214               // 'continue' 'loop'
       && lk != 113284)             // 'exit' 'returning'
      {
        break;
      }
      whitespace();
      parse_Statement();
    }
    eventHandler.endNonterminal("Statements", e0);
  }

  function try_Statements()
  {
    for (;;)
    {
      lookahead1W(278);             // Wildcard | EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral |
      switch (l1)
      {
      case 34:                      // '('
        lookahead2W(269);           // Wildcard | EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral |
        break;
      case 35:                      // '(#'
        lookahead2(252);            // EQName^Token | S | 'after' | 'allowing' | 'ancestor' | 'ancestor-or-self' |
        break;
      case 46:                      // '/'
        lookahead2W(284);           // Wildcard | EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral |
        break;
      case 47:                      // '//'
        lookahead2W(265);           // Wildcard | EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral |
        break;
      case 54:                      // '<'
        lookahead2(4);              // QName
        break;
      case 55:                      // '<!--'
        lookahead2(1);              // DirCommentContents
        break;
      case 59:                      // '<?'
        lookahead2(3);              // PITarget
        break;
      case 66:                      // '@'
        lookahead2W(257);           // Wildcard | EQName^Token | S^WS | '(:' | 'after' | 'allowing' | 'ancestor' |
        break;
      case 68:                      // '['
        lookahead2W(272);           // Wildcard | EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral |
        break;
      case 77:                      // 'append'
        lookahead2W(200);           // S^WS | EOF | '!' | '!=' | '#' | '(' | '(:' | '*' | '+' | ',' | '-' | '/' | '//' |
        break;
      case 82:                      // 'attribute'
        lookahead2W(281);           // EQName^Token | S^WS | EOF | '!' | '!=' | '#' | '(' | '(:' | '*' | '+' | ',' |
        break;
      case 121:                     // 'element'
        lookahead2W(280);           // EQName^Token | S^WS | EOF | '!' | '!=' | '#' | '(' | '(:' | '*' | '+' | ',' |
        break;
      case 132:                     // 'exit'
        lookahead2W(203);           // S^WS | EOF | '!' | '!=' | '#' | '(' | '(:' | '*' | '+' | ',' | '-' | '/' | '//' |
        break;
      case 137:                     // 'for'
        lookahead2W(208);           // S^WS | EOF | '!' | '!=' | '#' | '$' | '(' | '(:' | '*' | '+' | ',' | '-' | '/' |
        break;
      case 174:                     // 'let'
        lookahead2W(205);           // S^WS | EOF | '!' | '!=' | '#' | '$' | '(' | '(:' | '*' | '+' | ',' | '-' | '/' |
        break;
      case 218:                     // 'rename'
        lookahead2W(206);           // S^WS | EOF | '!' | '!=' | '#' | '(' | '(:' | '*' | '+' | ',' | '-' | '/' | '//' |
        break;
      case 219:                     // 'replace'
        lookahead2W(207);           // S^WS | EOF | '!' | '!=' | '#' | '(' | '(:' | '*' | '+' | ',' | '-' | '/' | '//' |
        break;
      case 260:                     // 'validate'
        lookahead2W(210);           // S^WS | EOF | '!' | '!=' | '#' | '(' | '(:' | '*' | '+' | ',' | '-' | '/' | '//' |
        break;
      case 276:                     // '{'
        lookahead2W(277);           // Wildcard | EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral |
        break;
      case 278:                     // '{|'
        lookahead2W(273);           // Wildcard | EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral |
        break;
      case 5:                       // Wildcard
      case 45:                      // '..'
        lookahead2W(186);           // S^WS | EOF | '!' | '!=' | '(:' | '*' | '+' | ',' | '-' | '/' | '//' | ';' | '<' |
        break;
      case 31:                      // '$'
      case 32:                      // '%'
        lookahead2W(255);           // EQName^Token | S^WS | '(:' | 'after' | 'allowing' | 'ancestor' |
        break;
      case 40:                      // '+'
      case 42:                      // '-'
        lookahead2W(267);           // Wildcard | EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral |
        break;
      case 86:                      // 'break'
      case 102:                     // 'continue'
        lookahead2W(201);           // S^WS | EOF | '!' | '!=' | '#' | '(' | '(:' | '*' | '+' | ',' | '-' | '/' | '//' |
        break;
      case 110:                     // 'delete'
      case 159:                     // 'insert'
        lookahead2W(209);           // S^WS | EOF | '!' | '!=' | '#' | '(' | '(:' | '*' | '+' | ',' | '-' | '/' | '//' |
        break;
      case 184:                     // 'namespace'
      case 216:                     // 'processing-instruction'
        lookahead2W(268);           // NCName^Token | S^WS | EOF | '!' | '!=' | '#' | '(' | '(:' | '*' | '+' | ',' |
        break;
      case 103:                     // 'copy'
      case 129:                     // 'every'
      case 235:                     // 'some'
      case 262:                     // 'variable'
        lookahead2W(197);           // S^WS | EOF | '!' | '!=' | '#' | '$' | '(' | '(:' | '*' | '+' | ',' | '-' | '/' |
        break;
      case 8:                       // IntegerLiteral
      case 9:                       // DecimalLiteral
      case 10:                      // DoubleLiteral
      case 11:                      // StringLiteral
      case 44:                      // '.'
        lookahead2W(192);           // S^WS | EOF | '!' | '!=' | '(' | '(:' | '*' | '+' | ',' | '-' | '/' | '//' | ';' |
        break;
      case 78:                      // 'array'
      case 124:                     // 'empty-sequence'
      case 165:                     // 'item'
      case 167:                     // 'json-item'
      case 242:                     // 'structured-item'
        lookahead2W(191);           // S^WS | EOF | '!' | '!=' | '#' | '(:' | '*' | '+' | ',' | '-' | '/' | '//' | ';' |
        break;
      case 96:                      // 'comment'
      case 119:                     // 'document'
      case 202:                     // 'ordered'
      case 244:                     // 'text'
      case 250:                     // 'try'
      case 256:                     // 'unordered'
        lookahead2W(204);           // S^WS | EOF | '!' | '!=' | '#' | '(' | '(:' | '*' | '+' | ',' | '-' | '/' | '//' |
        break;
      case 73:                      // 'ancestor'
      case 74:                      // 'ancestor-or-self'
      case 93:                      // 'child'
      case 111:                     // 'descendant'
      case 112:                     // 'descendant-or-self'
      case 135:                     // 'following'
      case 136:                     // 'following-sibling'
      case 206:                     // 'parent'
      case 212:                     // 'preceding'
      case 213:                     // 'preceding-sibling'
      case 229:                     // 'self'
        lookahead2W(198);           // S^WS | EOF | '!' | '!=' | '#' | '(' | '(:' | '*' | '+' | ',' | '-' | '/' | '//' |
        break;
      case 6:                       // EQName^Token
      case 70:                      // 'after'
      case 72:                      // 'allowing'
      case 75:                      // 'and'
      case 79:                      // 'as'
      case 80:                      // 'ascending'
      case 81:                      // 'at'
      case 83:                      // 'base-uri'
      case 84:                      // 'before'
      case 85:                      // 'boundary-space'
      case 88:                      // 'case'
      case 89:                      // 'cast'
      case 90:                      // 'castable'
      case 91:                      // 'catch'
      case 94:                      // 'collation'
      case 97:                      // 'constraint'
      case 98:                      // 'construction'
      case 101:                     // 'context'
      case 104:                     // 'copy-namespaces'
      case 105:                     // 'count'
      case 106:                     // 'decimal-format'
      case 108:                     // 'declare'
      case 109:                     // 'default'
      case 113:                     // 'descending'
      case 118:                     // 'div'
      case 120:                     // 'document-node'
      case 122:                     // 'else'
      case 123:                     // 'empty'
      case 125:                     // 'encoding'
      case 126:                     // 'end'
      case 128:                     // 'eq'
      case 131:                     // 'except'
      case 133:                     // 'external'
      case 134:                     // 'first'
      case 141:                     // 'ft-option'
      case 145:                     // 'function'
      case 146:                     // 'ge'
      case 148:                     // 'group'
      case 150:                     // 'gt'
      case 151:                     // 'idiv'
      case 152:                     // 'if'
      case 153:                     // 'import'
      case 154:                     // 'in'
      case 155:                     // 'index'
      case 160:                     // 'instance'
      case 161:                     // 'integrity'
      case 162:                     // 'intersect'
      case 163:                     // 'into'
      case 164:                     // 'is'
      case 166:                     // 'json'
      case 170:                     // 'last'
      case 171:                     // 'lax'
      case 172:                     // 'le'
      case 176:                     // 'loop'
      case 178:                     // 'lt'
      case 180:                     // 'mod'
      case 181:                     // 'modify'
      case 182:                     // 'module'
      case 185:                     // 'namespace-node'
      case 186:                     // 'ne'
      case 191:                     // 'node'
      case 192:                     // 'nodes'
      case 194:                     // 'object'
      case 198:                     // 'only'
      case 199:                     // 'option'
      case 200:                     // 'or'
      case 201:                     // 'order'
      case 203:                     // 'ordering'
      case 220:                     // 'return'
      case 221:                     // 'returning'
      case 222:                     // 'revalidation'
      case 224:                     // 'satisfies'
      case 225:                     // 'schema'
      case 226:                     // 'schema-attribute'
      case 227:                     // 'schema-element'
      case 228:                     // 'score'
      case 234:                     // 'sliding'
      case 236:                     // 'stable'
      case 237:                     // 'start'
      case 240:                     // 'strict'
      case 243:                     // 'switch'
      case 248:                     // 'to'
      case 249:                     // 'treat'
      case 251:                     // 'tumbling'
      case 252:                     // 'type'
      case 253:                     // 'typeswitch'
      case 254:                     // 'union'
      case 257:                     // 'updating'
      case 261:                     // 'value'
      case 263:                     // 'version'
      case 266:                     // 'where'
      case 267:                     // 'while'
      case 270:                     // 'with'
      case 274:                     // 'xquery'
        lookahead2W(195);           // S^WS | EOF | '!' | '!=' | '#' | '(' | '(:' | '*' | '+' | ',' | '-' | '/' | '//' |
        break;
      default:
        lk = l1;
      }
      if (lk != 25                  // EOF
       && lk != 53                  // ';'
       && lk != 282                 // '}'
       && lk != 12805               // Wildcard EOF
       && lk != 12806               // EQName^Token EOF
       && lk != 12808               // IntegerLiteral EOF
       && lk != 12809               // DecimalLiteral EOF
       && lk != 12810               // DoubleLiteral EOF
       && lk != 12811               // StringLiteral EOF
       && lk != 12844               // '.' EOF
       && lk != 12845               // '..' EOF
       && lk != 12846               // '/' EOF
       && lk != 12870               // 'after' EOF
       && lk != 12872               // 'allowing' EOF
       && lk != 12873               // 'ancestor' EOF
       && lk != 12874               // 'ancestor-or-self' EOF
       && lk != 12875               // 'and' EOF
       && lk != 12877               // 'append' EOF
       && lk != 12878               // 'array' EOF
       && lk != 12879               // 'as' EOF
       && lk != 12880               // 'ascending' EOF
       && lk != 12881               // 'at' EOF
       && lk != 12882               // 'attribute' EOF
       && lk != 12883               // 'base-uri' EOF
       && lk != 12884               // 'before' EOF
       && lk != 12885               // 'boundary-space' EOF
       && lk != 12886               // 'break' EOF
       && lk != 12888               // 'case' EOF
       && lk != 12889               // 'cast' EOF
       && lk != 12890               // 'castable' EOF
       && lk != 12891               // 'catch' EOF
       && lk != 12893               // 'child' EOF
       && lk != 12894               // 'collation' EOF
       && lk != 12896               // 'comment' EOF
       && lk != 12897               // 'constraint' EOF
       && lk != 12898               // 'construction' EOF
       && lk != 12901               // 'context' EOF
       && lk != 12902               // 'continue' EOF
       && lk != 12903               // 'copy' EOF
       && lk != 12904               // 'copy-namespaces' EOF
       && lk != 12905               // 'count' EOF
       && lk != 12906               // 'decimal-format' EOF
       && lk != 12908               // 'declare' EOF
       && lk != 12909               // 'default' EOF
       && lk != 12910               // 'delete' EOF
       && lk != 12911               // 'descendant' EOF
       && lk != 12912               // 'descendant-or-self' EOF
       && lk != 12913               // 'descending' EOF
       && lk != 12918               // 'div' EOF
       && lk != 12919               // 'document' EOF
       && lk != 12920               // 'document-node' EOF
       && lk != 12921               // 'element' EOF
       && lk != 12922               // 'else' EOF
       && lk != 12923               // 'empty' EOF
       && lk != 12924               // 'empty-sequence' EOF
       && lk != 12925               // 'encoding' EOF
       && lk != 12926               // 'end' EOF
       && lk != 12928               // 'eq' EOF
       && lk != 12929               // 'every' EOF
       && lk != 12931               // 'except' EOF
       && lk != 12932               // 'exit' EOF
       && lk != 12933               // 'external' EOF
       && lk != 12934               // 'first' EOF
       && lk != 12935               // 'following' EOF
       && lk != 12936               // 'following-sibling' EOF
       && lk != 12937               // 'for' EOF
       && lk != 12941               // 'ft-option' EOF
       && lk != 12945               // 'function' EOF
       && lk != 12946               // 'ge' EOF
       && lk != 12948               // 'group' EOF
       && lk != 12950               // 'gt' EOF
       && lk != 12951               // 'idiv' EOF
       && lk != 12952               // 'if' EOF
       && lk != 12953               // 'import' EOF
       && lk != 12954               // 'in' EOF
       && lk != 12955               // 'index' EOF
       && lk != 12959               // 'insert' EOF
       && lk != 12960               // 'instance' EOF
       && lk != 12961               // 'integrity' EOF
       && lk != 12962               // 'intersect' EOF
       && lk != 12963               // 'into' EOF
       && lk != 12964               // 'is' EOF
       && lk != 12965               // 'item' EOF
       && lk != 12966               // 'json' EOF
       && lk != 12967               // 'json-item' EOF
       && lk != 12970               // 'last' EOF
       && lk != 12971               // 'lax' EOF
       && lk != 12972               // 'le' EOF
       && lk != 12974               // 'let' EOF
       && lk != 12976               // 'loop' EOF
       && lk != 12978               // 'lt' EOF
       && lk != 12980               // 'mod' EOF
       && lk != 12981               // 'modify' EOF
       && lk != 12982               // 'module' EOF
       && lk != 12984               // 'namespace' EOF
       && lk != 12985               // 'namespace-node' EOF
       && lk != 12986               // 'ne' EOF
       && lk != 12991               // 'node' EOF
       && lk != 12992               // 'nodes' EOF
       && lk != 12994               // 'object' EOF
       && lk != 12998               // 'only' EOF
       && lk != 12999               // 'option' EOF
       && lk != 13000               // 'or' EOF
       && lk != 13001               // 'order' EOF
       && lk != 13002               // 'ordered' EOF
       && lk != 13003               // 'ordering' EOF
       && lk != 13006               // 'parent' EOF
       && lk != 13012               // 'preceding' EOF
       && lk != 13013               // 'preceding-sibling' EOF
       && lk != 13016               // 'processing-instruction' EOF
       && lk != 13018               // 'rename' EOF
       && lk != 13019               // 'replace' EOF
       && lk != 13020               // 'return' EOF
       && lk != 13021               // 'returning' EOF
       && lk != 13022               // 'revalidation' EOF
       && lk != 13024               // 'satisfies' EOF
       && lk != 13025               // 'schema' EOF
       && lk != 13026               // 'schema-attribute' EOF
       && lk != 13027               // 'schema-element' EOF
       && lk != 13028               // 'score' EOF
       && lk != 13029               // 'self' EOF
       && lk != 13034               // 'sliding' EOF
       && lk != 13035               // 'some' EOF
       && lk != 13036               // 'stable' EOF
       && lk != 13037               // 'start' EOF
       && lk != 13040               // 'strict' EOF
       && lk != 13042               // 'structured-item' EOF
       && lk != 13043               // 'switch' EOF
       && lk != 13044               // 'text' EOF
       && lk != 13048               // 'to' EOF
       && lk != 13049               // 'treat' EOF
       && lk != 13050               // 'try' EOF
       && lk != 13051               // 'tumbling' EOF
       && lk != 13052               // 'type' EOF
       && lk != 13053               // 'typeswitch' EOF
       && lk != 13054               // 'union' EOF
       && lk != 13056               // 'unordered' EOF
       && lk != 13057               // 'updating' EOF
       && lk != 13060               // 'validate' EOF
       && lk != 13061               // 'value' EOF
       && lk != 13062               // 'variable' EOF
       && lk != 13063               // 'version' EOF
       && lk != 13066               // 'where' EOF
       && lk != 13067               // 'while' EOF
       && lk != 13070               // 'with' EOF
       && lk != 13074               // 'xquery' EOF
       && lk != 16134               // 'variable' '$'
       && lk != 20997               // Wildcard ','
       && lk != 20998               // EQName^Token ','
       && lk != 21000               // IntegerLiteral ','
       && lk != 21001               // DecimalLiteral ','
       && lk != 21002               // DoubleLiteral ','
       && lk != 21003               // StringLiteral ','
       && lk != 21036               // '.' ','
       && lk != 21037               // '..' ','
       && lk != 21038               // '/' ','
       && lk != 21062               // 'after' ','
       && lk != 21064               // 'allowing' ','
       && lk != 21065               // 'ancestor' ','
       && lk != 21066               // 'ancestor-or-self' ','
       && lk != 21067               // 'and' ','
       && lk != 21069               // 'append' ','
       && lk != 21070               // 'array' ','
       && lk != 21071               // 'as' ','
       && lk != 21072               // 'ascending' ','
       && lk != 21073               // 'at' ','
       && lk != 21074               // 'attribute' ','
       && lk != 21075               // 'base-uri' ','
       && lk != 21076               // 'before' ','
       && lk != 21077               // 'boundary-space' ','
       && lk != 21078               // 'break' ','
       && lk != 21080               // 'case' ','
       && lk != 21081               // 'cast' ','
       && lk != 21082               // 'castable' ','
       && lk != 21083               // 'catch' ','
       && lk != 21085               // 'child' ','
       && lk != 21086               // 'collation' ','
       && lk != 21088               // 'comment' ','
       && lk != 21089               // 'constraint' ','
       && lk != 21090               // 'construction' ','
       && lk != 21093               // 'context' ','
       && lk != 21094               // 'continue' ','
       && lk != 21095               // 'copy' ','
       && lk != 21096               // 'copy-namespaces' ','
       && lk != 21097               // 'count' ','
       && lk != 21098               // 'decimal-format' ','
       && lk != 21100               // 'declare' ','
       && lk != 21101               // 'default' ','
       && lk != 21102               // 'delete' ','
       && lk != 21103               // 'descendant' ','
       && lk != 21104               // 'descendant-or-self' ','
       && lk != 21105               // 'descending' ','
       && lk != 21110               // 'div' ','
       && lk != 21111               // 'document' ','
       && lk != 21112               // 'document-node' ','
       && lk != 21113               // 'element' ','
       && lk != 21114               // 'else' ','
       && lk != 21115               // 'empty' ','
       && lk != 21116               // 'empty-sequence' ','
       && lk != 21117               // 'encoding' ','
       && lk != 21118               // 'end' ','
       && lk != 21120               // 'eq' ','
       && lk != 21121               // 'every' ','
       && lk != 21123               // 'except' ','
       && lk != 21124               // 'exit' ','
       && lk != 21125               // 'external' ','
       && lk != 21126               // 'first' ','
       && lk != 21127               // 'following' ','
       && lk != 21128               // 'following-sibling' ','
       && lk != 21129               // 'for' ','
       && lk != 21133               // 'ft-option' ','
       && lk != 21137               // 'function' ','
       && lk != 21138               // 'ge' ','
       && lk != 21140               // 'group' ','
       && lk != 21142               // 'gt' ','
       && lk != 21143               // 'idiv' ','
       && lk != 21144               // 'if' ','
       && lk != 21145               // 'import' ','
       && lk != 21146               // 'in' ','
       && lk != 21147               // 'index' ','
       && lk != 21151               // 'insert' ','
       && lk != 21152               // 'instance' ','
       && lk != 21153               // 'integrity' ','
       && lk != 21154               // 'intersect' ','
       && lk != 21155               // 'into' ','
       && lk != 21156               // 'is' ','
       && lk != 21157               // 'item' ','
       && lk != 21158               // 'json' ','
       && lk != 21159               // 'json-item' ','
       && lk != 21162               // 'last' ','
       && lk != 21163               // 'lax' ','
       && lk != 21164               // 'le' ','
       && lk != 21166               // 'let' ','
       && lk != 21168               // 'loop' ','
       && lk != 21170               // 'lt' ','
       && lk != 21172               // 'mod' ','
       && lk != 21173               // 'modify' ','
       && lk != 21174               // 'module' ','
       && lk != 21176               // 'namespace' ','
       && lk != 21177               // 'namespace-node' ','
       && lk != 21178               // 'ne' ','
       && lk != 21183               // 'node' ','
       && lk != 21184               // 'nodes' ','
       && lk != 21186               // 'object' ','
       && lk != 21190               // 'only' ','
       && lk != 21191               // 'option' ','
       && lk != 21192               // 'or' ','
       && lk != 21193               // 'order' ','
       && lk != 21194               // 'ordered' ','
       && lk != 21195               // 'ordering' ','
       && lk != 21198               // 'parent' ','
       && lk != 21204               // 'preceding' ','
       && lk != 21205               // 'preceding-sibling' ','
       && lk != 21208               // 'processing-instruction' ','
       && lk != 21210               // 'rename' ','
       && lk != 21211               // 'replace' ','
       && lk != 21212               // 'return' ','
       && lk != 21213               // 'returning' ','
       && lk != 21214               // 'revalidation' ','
       && lk != 21216               // 'satisfies' ','
       && lk != 21217               // 'schema' ','
       && lk != 21218               // 'schema-attribute' ','
       && lk != 21219               // 'schema-element' ','
       && lk != 21220               // 'score' ','
       && lk != 21221               // 'self' ','
       && lk != 21226               // 'sliding' ','
       && lk != 21227               // 'some' ','
       && lk != 21228               // 'stable' ','
       && lk != 21229               // 'start' ','
       && lk != 21232               // 'strict' ','
       && lk != 21234               // 'structured-item' ','
       && lk != 21235               // 'switch' ','
       && lk != 21236               // 'text' ','
       && lk != 21240               // 'to' ','
       && lk != 21241               // 'treat' ','
       && lk != 21242               // 'try' ','
       && lk != 21243               // 'tumbling' ','
       && lk != 21244               // 'type' ','
       && lk != 21245               // 'typeswitch' ','
       && lk != 21246               // 'union' ','
       && lk != 21248               // 'unordered' ','
       && lk != 21249               // 'updating' ','
       && lk != 21252               // 'validate' ','
       && lk != 21253               // 'value' ','
       && lk != 21254               // 'variable' ','
       && lk != 21255               // 'version' ','
       && lk != 21258               // 'where' ','
       && lk != 21259               // 'while' ','
       && lk != 21262               // 'with' ','
       && lk != 21266               // 'xquery' ','
       && lk != 27141               // Wildcard ';'
       && lk != 27142               // EQName^Token ';'
       && lk != 27144               // IntegerLiteral ';'
       && lk != 27145               // DecimalLiteral ';'
       && lk != 27146               // DoubleLiteral ';'
       && lk != 27147               // StringLiteral ';'
       && lk != 27180               // '.' ';'
       && lk != 27181               // '..' ';'
       && lk != 27182               // '/' ';'
       && lk != 27206               // 'after' ';'
       && lk != 27208               // 'allowing' ';'
       && lk != 27209               // 'ancestor' ';'
       && lk != 27210               // 'ancestor-or-self' ';'
       && lk != 27211               // 'and' ';'
       && lk != 27213               // 'append' ';'
       && lk != 27214               // 'array' ';'
       && lk != 27215               // 'as' ';'
       && lk != 27216               // 'ascending' ';'
       && lk != 27217               // 'at' ';'
       && lk != 27218               // 'attribute' ';'
       && lk != 27219               // 'base-uri' ';'
       && lk != 27220               // 'before' ';'
       && lk != 27221               // 'boundary-space' ';'
       && lk != 27222               // 'break' ';'
       && lk != 27224               // 'case' ';'
       && lk != 27225               // 'cast' ';'
       && lk != 27226               // 'castable' ';'
       && lk != 27227               // 'catch' ';'
       && lk != 27229               // 'child' ';'
       && lk != 27230               // 'collation' ';'
       && lk != 27232               // 'comment' ';'
       && lk != 27233               // 'constraint' ';'
       && lk != 27234               // 'construction' ';'
       && lk != 27237               // 'context' ';'
       && lk != 27238               // 'continue' ';'
       && lk != 27239               // 'copy' ';'
       && lk != 27240               // 'copy-namespaces' ';'
       && lk != 27241               // 'count' ';'
       && lk != 27242               // 'decimal-format' ';'
       && lk != 27244               // 'declare' ';'
       && lk != 27245               // 'default' ';'
       && lk != 27246               // 'delete' ';'
       && lk != 27247               // 'descendant' ';'
       && lk != 27248               // 'descendant-or-self' ';'
       && lk != 27249               // 'descending' ';'
       && lk != 27254               // 'div' ';'
       && lk != 27255               // 'document' ';'
       && lk != 27256               // 'document-node' ';'
       && lk != 27257               // 'element' ';'
       && lk != 27258               // 'else' ';'
       && lk != 27259               // 'empty' ';'
       && lk != 27260               // 'empty-sequence' ';'
       && lk != 27261               // 'encoding' ';'
       && lk != 27262               // 'end' ';'
       && lk != 27264               // 'eq' ';'
       && lk != 27265               // 'every' ';'
       && lk != 27267               // 'except' ';'
       && lk != 27268               // 'exit' ';'
       && lk != 27269               // 'external' ';'
       && lk != 27270               // 'first' ';'
       && lk != 27271               // 'following' ';'
       && lk != 27272               // 'following-sibling' ';'
       && lk != 27273               // 'for' ';'
       && lk != 27277               // 'ft-option' ';'
       && lk != 27281               // 'function' ';'
       && lk != 27282               // 'ge' ';'
       && lk != 27284               // 'group' ';'
       && lk != 27286               // 'gt' ';'
       && lk != 27287               // 'idiv' ';'
       && lk != 27288               // 'if' ';'
       && lk != 27289               // 'import' ';'
       && lk != 27290               // 'in' ';'
       && lk != 27291               // 'index' ';'
       && lk != 27295               // 'insert' ';'
       && lk != 27296               // 'instance' ';'
       && lk != 27297               // 'integrity' ';'
       && lk != 27298               // 'intersect' ';'
       && lk != 27299               // 'into' ';'
       && lk != 27300               // 'is' ';'
       && lk != 27301               // 'item' ';'
       && lk != 27302               // 'json' ';'
       && lk != 27303               // 'json-item' ';'
       && lk != 27306               // 'last' ';'
       && lk != 27307               // 'lax' ';'
       && lk != 27308               // 'le' ';'
       && lk != 27310               // 'let' ';'
       && lk != 27312               // 'loop' ';'
       && lk != 27314               // 'lt' ';'
       && lk != 27316               // 'mod' ';'
       && lk != 27317               // 'modify' ';'
       && lk != 27318               // 'module' ';'
       && lk != 27320               // 'namespace' ';'
       && lk != 27321               // 'namespace-node' ';'
       && lk != 27322               // 'ne' ';'
       && lk != 27327               // 'node' ';'
       && lk != 27328               // 'nodes' ';'
       && lk != 27330               // 'object' ';'
       && lk != 27334               // 'only' ';'
       && lk != 27335               // 'option' ';'
       && lk != 27336               // 'or' ';'
       && lk != 27337               // 'order' ';'
       && lk != 27338               // 'ordered' ';'
       && lk != 27339               // 'ordering' ';'
       && lk != 27342               // 'parent' ';'
       && lk != 27348               // 'preceding' ';'
       && lk != 27349               // 'preceding-sibling' ';'
       && lk != 27352               // 'processing-instruction' ';'
       && lk != 27354               // 'rename' ';'
       && lk != 27355               // 'replace' ';'
       && lk != 27356               // 'return' ';'
       && lk != 27357               // 'returning' ';'
       && lk != 27358               // 'revalidation' ';'
       && lk != 27360               // 'satisfies' ';'
       && lk != 27361               // 'schema' ';'
       && lk != 27362               // 'schema-attribute' ';'
       && lk != 27363               // 'schema-element' ';'
       && lk != 27364               // 'score' ';'
       && lk != 27365               // 'self' ';'
       && lk != 27370               // 'sliding' ';'
       && lk != 27371               // 'some' ';'
       && lk != 27372               // 'stable' ';'
       && lk != 27373               // 'start' ';'
       && lk != 27376               // 'strict' ';'
       && lk != 27378               // 'structured-item' ';'
       && lk != 27379               // 'switch' ';'
       && lk != 27380               // 'text' ';'
       && lk != 27384               // 'to' ';'
       && lk != 27385               // 'treat' ';'
       && lk != 27386               // 'try' ';'
       && lk != 27387               // 'tumbling' ';'
       && lk != 27388               // 'type' ';'
       && lk != 27389               // 'typeswitch' ';'
       && lk != 27390               // 'union' ';'
       && lk != 27392               // 'unordered' ';'
       && lk != 27393               // 'updating' ';'
       && lk != 27396               // 'validate' ';'
       && lk != 27397               // 'value' ';'
       && lk != 27398               // 'variable' ';'
       && lk != 27399               // 'version' ';'
       && lk != 27402               // 'where' ';'
       && lk != 27403               // 'while' ';'
       && lk != 27406               // 'with' ';'
       && lk != 27410               // 'xquery' ';'
       && lk != 90198               // 'break' 'loop'
       && lk != 90214               // 'continue' 'loop'
       && lk != 113284              // 'exit' 'returning'
       && lk != 144389              // Wildcard '}'
       && lk != 144390              // EQName^Token '}'
       && lk != 144392              // IntegerLiteral '}'
       && lk != 144393              // DecimalLiteral '}'
       && lk != 144394              // DoubleLiteral '}'
       && lk != 144395              // StringLiteral '}'
       && lk != 144428              // '.' '}'
       && lk != 144429              // '..' '}'
       && lk != 144430              // '/' '}'
       && lk != 144454              // 'after' '}'
       && lk != 144456              // 'allowing' '}'
       && lk != 144457              // 'ancestor' '}'
       && lk != 144458              // 'ancestor-or-self' '}'
       && lk != 144459              // 'and' '}'
       && lk != 144461              // 'append' '}'
       && lk != 144462              // 'array' '}'
       && lk != 144463              // 'as' '}'
       && lk != 144464              // 'ascending' '}'
       && lk != 144465              // 'at' '}'
       && lk != 144466              // 'attribute' '}'
       && lk != 144467              // 'base-uri' '}'
       && lk != 144468              // 'before' '}'
       && lk != 144469              // 'boundary-space' '}'
       && lk != 144470              // 'break' '}'
       && lk != 144472              // 'case' '}'
       && lk != 144473              // 'cast' '}'
       && lk != 144474              // 'castable' '}'
       && lk != 144475              // 'catch' '}'
       && lk != 144477              // 'child' '}'
       && lk != 144478              // 'collation' '}'
       && lk != 144480              // 'comment' '}'
       && lk != 144481              // 'constraint' '}'
       && lk != 144482              // 'construction' '}'
       && lk != 144485              // 'context' '}'
       && lk != 144486              // 'continue' '}'
       && lk != 144487              // 'copy' '}'
       && lk != 144488              // 'copy-namespaces' '}'
       && lk != 144489              // 'count' '}'
       && lk != 144490              // 'decimal-format' '}'
       && lk != 144492              // 'declare' '}'
       && lk != 144493              // 'default' '}'
       && lk != 144494              // 'delete' '}'
       && lk != 144495              // 'descendant' '}'
       && lk != 144496              // 'descendant-or-self' '}'
       && lk != 144497              // 'descending' '}'
       && lk != 144502              // 'div' '}'
       && lk != 144503              // 'document' '}'
       && lk != 144504              // 'document-node' '}'
       && lk != 144505              // 'element' '}'
       && lk != 144506              // 'else' '}'
       && lk != 144507              // 'empty' '}'
       && lk != 144508              // 'empty-sequence' '}'
       && lk != 144509              // 'encoding' '}'
       && lk != 144510              // 'end' '}'
       && lk != 144512              // 'eq' '}'
       && lk != 144513              // 'every' '}'
       && lk != 144515              // 'except' '}'
       && lk != 144516              // 'exit' '}'
       && lk != 144517              // 'external' '}'
       && lk != 144518              // 'first' '}'
       && lk != 144519              // 'following' '}'
       && lk != 144520              // 'following-sibling' '}'
       && lk != 144521              // 'for' '}'
       && lk != 144525              // 'ft-option' '}'
       && lk != 144529              // 'function' '}'
       && lk != 144530              // 'ge' '}'
       && lk != 144532              // 'group' '}'
       && lk != 144534              // 'gt' '}'
       && lk != 144535              // 'idiv' '}'
       && lk != 144536              // 'if' '}'
       && lk != 144537              // 'import' '}'
       && lk != 144538              // 'in' '}'
       && lk != 144539              // 'index' '}'
       && lk != 144543              // 'insert' '}'
       && lk != 144544              // 'instance' '}'
       && lk != 144545              // 'integrity' '}'
       && lk != 144546              // 'intersect' '}'
       && lk != 144547              // 'into' '}'
       && lk != 144548              // 'is' '}'
       && lk != 144549              // 'item' '}'
       && lk != 144550              // 'json' '}'
       && lk != 144551              // 'json-item' '}'
       && lk != 144554              // 'last' '}'
       && lk != 144555              // 'lax' '}'
       && lk != 144556              // 'le' '}'
       && lk != 144558              // 'let' '}'
       && lk != 144560              // 'loop' '}'
       && lk != 144562              // 'lt' '}'
       && lk != 144564              // 'mod' '}'
       && lk != 144565              // 'modify' '}'
       && lk != 144566              // 'module' '}'
       && lk != 144568              // 'namespace' '}'
       && lk != 144569              // 'namespace-node' '}'
       && lk != 144570              // 'ne' '}'
       && lk != 144575              // 'node' '}'
       && lk != 144576              // 'nodes' '}'
       && lk != 144578              // 'object' '}'
       && lk != 144582              // 'only' '}'
       && lk != 144583              // 'option' '}'
       && lk != 144584              // 'or' '}'
       && lk != 144585              // 'order' '}'
       && lk != 144586              // 'ordered' '}'
       && lk != 144587              // 'ordering' '}'
       && lk != 144590              // 'parent' '}'
       && lk != 144596              // 'preceding' '}'
       && lk != 144597              // 'preceding-sibling' '}'
       && lk != 144600              // 'processing-instruction' '}'
       && lk != 144602              // 'rename' '}'
       && lk != 144603              // 'replace' '}'
       && lk != 144604              // 'return' '}'
       && lk != 144605              // 'returning' '}'
       && lk != 144606              // 'revalidation' '}'
       && lk != 144608              // 'satisfies' '}'
       && lk != 144609              // 'schema' '}'
       && lk != 144610              // 'schema-attribute' '}'
       && lk != 144611              // 'schema-element' '}'
       && lk != 144612              // 'score' '}'
       && lk != 144613              // 'self' '}'
       && lk != 144618              // 'sliding' '}'
       && lk != 144619              // 'some' '}'
       && lk != 144620              // 'stable' '}'
       && lk != 144621              // 'start' '}'
       && lk != 144624              // 'strict' '}'
       && lk != 144626              // 'structured-item' '}'
       && lk != 144627              // 'switch' '}'
       && lk != 144628              // 'text' '}'
       && lk != 144632              // 'to' '}'
       && lk != 144633              // 'treat' '}'
       && lk != 144634              // 'try' '}'
       && lk != 144635              // 'tumbling' '}'
       && lk != 144636              // 'type' '}'
       && lk != 144637              // 'typeswitch' '}'
       && lk != 144638              // 'union' '}'
       && lk != 144640              // 'unordered' '}'
       && lk != 144641              // 'updating' '}'
       && lk != 144644              // 'validate' '}'
       && lk != 144645              // 'value' '}'
       && lk != 144646              // 'variable' '}'
       && lk != 144647              // 'version' '}'
       && lk != 144650              // 'where' '}'
       && lk != 144651              // 'while' '}'
       && lk != 144654              // 'with' '}'
       && lk != 144658)             // 'xquery' '}'
      {
        lk = memoized(6, e0);
        if (lk == 0)
        {
          var b0A = b0; var e0A = e0; var l1A = l1;
          var b1A = b1; var e1A = e1; var l2A = l2;
          var b2A = b2; var e2A = e2;
          try
          {
            try_Statement();
            memoize(6, e0A, -1);
            continue;
          }
          catch (p1A)
          {
            b0 = b0A; e0 = e0A; l1 = l1A; if (l1 == 0) {end = e0A;} else {
            b1 = b1A; e1 = e1A; l2 = l2A; if (l2 == 0) {end = e1A;} else {
            b2 = b2A; e2 = e2A; end = e2A; }}
            memoize(6, e0A, -2);
            break;
          }
        }
      }
      if (lk != -1
       && lk != 53                  // ';'
       && lk != 16134               // 'variable' '$'
       && lk != 27141               // Wildcard ';'
       && lk != 27142               // EQName^Token ';'
       && lk != 27144               // IntegerLiteral ';'
       && lk != 27145               // DecimalLiteral ';'
       && lk != 27146               // DoubleLiteral ';'
       && lk != 27147               // StringLiteral ';'
       && lk != 27180               // '.' ';'
       && lk != 27181               // '..' ';'
       && lk != 27182               // '/' ';'
       && lk != 27206               // 'after' ';'
       && lk != 27208               // 'allowing' ';'
       && lk != 27209               // 'ancestor' ';'
       && lk != 27210               // 'ancestor-or-self' ';'
       && lk != 27211               // 'and' ';'
       && lk != 27213               // 'append' ';'
       && lk != 27214               // 'array' ';'
       && lk != 27215               // 'as' ';'
       && lk != 27216               // 'ascending' ';'
       && lk != 27217               // 'at' ';'
       && lk != 27218               // 'attribute' ';'
       && lk != 27219               // 'base-uri' ';'
       && lk != 27220               // 'before' ';'
       && lk != 27221               // 'boundary-space' ';'
       && lk != 27222               // 'break' ';'
       && lk != 27224               // 'case' ';'
       && lk != 27225               // 'cast' ';'
       && lk != 27226               // 'castable' ';'
       && lk != 27227               // 'catch' ';'
       && lk != 27229               // 'child' ';'
       && lk != 27230               // 'collation' ';'
       && lk != 27232               // 'comment' ';'
       && lk != 27233               // 'constraint' ';'
       && lk != 27234               // 'construction' ';'
       && lk != 27237               // 'context' ';'
       && lk != 27238               // 'continue' ';'
       && lk != 27239               // 'copy' ';'
       && lk != 27240               // 'copy-namespaces' ';'
       && lk != 27241               // 'count' ';'
       && lk != 27242               // 'decimal-format' ';'
       && lk != 27244               // 'declare' ';'
       && lk != 27245               // 'default' ';'
       && lk != 27246               // 'delete' ';'
       && lk != 27247               // 'descendant' ';'
       && lk != 27248               // 'descendant-or-self' ';'
       && lk != 27249               // 'descending' ';'
       && lk != 27254               // 'div' ';'
       && lk != 27255               // 'document' ';'
       && lk != 27256               // 'document-node' ';'
       && lk != 27257               // 'element' ';'
       && lk != 27258               // 'else' ';'
       && lk != 27259               // 'empty' ';'
       && lk != 27260               // 'empty-sequence' ';'
       && lk != 27261               // 'encoding' ';'
       && lk != 27262               // 'end' ';'
       && lk != 27264               // 'eq' ';'
       && lk != 27265               // 'every' ';'
       && lk != 27267               // 'except' ';'
       && lk != 27268               // 'exit' ';'
       && lk != 27269               // 'external' ';'
       && lk != 27270               // 'first' ';'
       && lk != 27271               // 'following' ';'
       && lk != 27272               // 'following-sibling' ';'
       && lk != 27273               // 'for' ';'
       && lk != 27277               // 'ft-option' ';'
       && lk != 27281               // 'function' ';'
       && lk != 27282               // 'ge' ';'
       && lk != 27284               // 'group' ';'
       && lk != 27286               // 'gt' ';'
       && lk != 27287               // 'idiv' ';'
       && lk != 27288               // 'if' ';'
       && lk != 27289               // 'import' ';'
       && lk != 27290               // 'in' ';'
       && lk != 27291               // 'index' ';'
       && lk != 27295               // 'insert' ';'
       && lk != 27296               // 'instance' ';'
       && lk != 27297               // 'integrity' ';'
       && lk != 27298               // 'intersect' ';'
       && lk != 27299               // 'into' ';'
       && lk != 27300               // 'is' ';'
       && lk != 27301               // 'item' ';'
       && lk != 27302               // 'json' ';'
       && lk != 27303               // 'json-item' ';'
       && lk != 27306               // 'last' ';'
       && lk != 27307               // 'lax' ';'
       && lk != 27308               // 'le' ';'
       && lk != 27310               // 'let' ';'
       && lk != 27312               // 'loop' ';'
       && lk != 27314               // 'lt' ';'
       && lk != 27316               // 'mod' ';'
       && lk != 27317               // 'modify' ';'
       && lk != 27318               // 'module' ';'
       && lk != 27320               // 'namespace' ';'
       && lk != 27321               // 'namespace-node' ';'
       && lk != 27322               // 'ne' ';'
       && lk != 27327               // 'node' ';'
       && lk != 27328               // 'nodes' ';'
       && lk != 27330               // 'object' ';'
       && lk != 27334               // 'only' ';'
       && lk != 27335               // 'option' ';'
       && lk != 27336               // 'or' ';'
       && lk != 27337               // 'order' ';'
       && lk != 27338               // 'ordered' ';'
       && lk != 27339               // 'ordering' ';'
       && lk != 27342               // 'parent' ';'
       && lk != 27348               // 'preceding' ';'
       && lk != 27349               // 'preceding-sibling' ';'
       && lk != 27352               // 'processing-instruction' ';'
       && lk != 27354               // 'rename' ';'
       && lk != 27355               // 'replace' ';'
       && lk != 27356               // 'return' ';'
       && lk != 27357               // 'returning' ';'
       && lk != 27358               // 'revalidation' ';'
       && lk != 27360               // 'satisfies' ';'
       && lk != 27361               // 'schema' ';'
       && lk != 27362               // 'schema-attribute' ';'
       && lk != 27363               // 'schema-element' ';'
       && lk != 27364               // 'score' ';'
       && lk != 27365               // 'self' ';'
       && lk != 27370               // 'sliding' ';'
       && lk != 27371               // 'some' ';'
       && lk != 27372               // 'stable' ';'
       && lk != 27373               // 'start' ';'
       && lk != 27376               // 'strict' ';'
       && lk != 27378               // 'structured-item' ';'
       && lk != 27379               // 'switch' ';'
       && lk != 27380               // 'text' ';'
       && lk != 27384               // 'to' ';'
       && lk != 27385               // 'treat' ';'
       && lk != 27386               // 'try' ';'
       && lk != 27387               // 'tumbling' ';'
       && lk != 27388               // 'type' ';'
       && lk != 27389               // 'typeswitch' ';'
       && lk != 27390               // 'union' ';'
       && lk != 27392               // 'unordered' ';'
       && lk != 27393               // 'updating' ';'
       && lk != 27396               // 'validate' ';'
       && lk != 27397               // 'value' ';'
       && lk != 27398               // 'variable' ';'
       && lk != 27399               // 'version' ';'
       && lk != 27402               // 'where' ';'
       && lk != 27403               // 'while' ';'
       && lk != 27406               // 'with' ';'
       && lk != 27410               // 'xquery' ';'
       && lk != 90198               // 'break' 'loop'
       && lk != 90214               // 'continue' 'loop'
       && lk != 113284)             // 'exit' 'returning'
      {
        break;
      }
      try_Statement();
    }
  }

  function parse_StatementsAndExpr()
  {
    eventHandler.startNonterminal("StatementsAndExpr", e0);
    parse_Statements();
    whitespace();
    parse_Expr();
    eventHandler.endNonterminal("StatementsAndExpr", e0);
  }

  function try_StatementsAndExpr()
  {
    try_Statements();
    try_Expr();
  }

  function parse_StatementsAndOptionalExpr()
  {
    eventHandler.startNonterminal("StatementsAndOptionalExpr", e0);
    parse_Statements();
    if (l1 != 25                    // EOF
     && l1 != 282)                  // '}'
    {
      whitespace();
      parse_Expr();
    }
    eventHandler.endNonterminal("StatementsAndOptionalExpr", e0);
  }

  function try_StatementsAndOptionalExpr()
  {
    try_Statements();
    if (l1 != 25                    // EOF
     && l1 != 282)                  // '}'
    {
      try_Expr();
    }
  }

  function parse_Statement()
  {
    eventHandler.startNonterminal("Statement", e0);
    switch (l1)
    {
    case 132:                       // 'exit'
      lookahead2W(189);             // S^WS | '!' | '!=' | '#' | '(' | '(:' | '*' | '+' | '-' | '/' | '//' | ';' | '<' |
      break;
    case 137:                       // 'for'
      lookahead2W(196);             // S^WS | '!' | '!=' | '#' | '$' | '(' | '(:' | '*' | '+' | '-' | '/' | '//' | ';' |
      break;
    case 174:                       // 'let'
      lookahead2W(193);             // S^WS | '!' | '!=' | '#' | '$' | '(' | '(:' | '*' | '+' | '-' | '/' | '//' | ';' |
      break;
    case 250:                       // 'try'
      lookahead2W(190);             // S^WS | '!' | '!=' | '#' | '(' | '(:' | '*' | '+' | '-' | '/' | '//' | ';' | '<' |
      break;
    case 262:                       // 'variable'
      lookahead2W(187);             // S^WS | '!' | '!=' | '#' | '$' | '(' | '(:' | '*' | '+' | '-' | '/' | '//' | ';' |
      break;
    case 276:                       // '{'
      lookahead2W(277);             // Wildcard | EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral |
      break;
    case 31:                        // '$'
    case 32:                        // '%'
      lookahead2W(255);             // EQName^Token | S^WS | '(:' | 'after' | 'allowing' | 'ancestor' |
      break;
    case 86:                        // 'break'
    case 102:                       // 'continue'
      lookahead2W(188);             // S^WS | '!' | '!=' | '#' | '(' | '(:' | '*' | '+' | '-' | '/' | '//' | ';' | '<' |
      break;
    case 152:                       // 'if'
    case 243:                       // 'switch'
    case 253:                       // 'typeswitch'
    case 267:                       // 'while'
      lookahead2W(185);             // S^WS | '!' | '!=' | '#' | '(' | '(:' | '*' | '+' | '-' | '/' | '//' | ';' | '<' |
      break;
    default:
      lk = l1;
    }
    if (lk == 2836                  // '{' Wildcard
     || lk == 3103                  // '$' EQName^Token
     || lk == 3104                  // '%' EQName^Token
     || lk == 3348                  // '{' EQName^Token
     || lk == 4372                  // '{' IntegerLiteral
     || lk == 4884                  // '{' DecimalLiteral
     || lk == 5396                  // '{' DoubleLiteral
     || lk == 5908                  // '{' StringLiteral
     || lk == 16148                 // '{' '$'
     || lk == 16660                 // '{' '%'
     || lk == 17675                 // 'while' '('
     || lk == 17684                 // '{' '('
     || lk == 18196                 // '{' '(#'
     || lk == 20756                 // '{' '+'
     || lk == 21780                 // '{' '-'
     || lk == 22804                 // '{' '.'
     || lk == 23316                 // '{' '..'
     || lk == 23828                 // '{' '/'
     || lk == 24340                 // '{' '//'
     || lk == 27412                 // '{' ';'
     || lk == 27924                 // '{' '<'
     || lk == 28436                 // '{' '<!--'
     || lk == 30484                 // '{' '<?'
     || lk == 34068                 // '{' '@'
     || lk == 35092                 // '{' '['
     || lk == 35871                 // '$' 'after'
     || lk == 35872                 // '%' 'after'
     || lk == 36116                 // '{' 'after'
     || lk == 36895                 // '$' 'allowing'
     || lk == 36896                 // '%' 'allowing'
     || lk == 37140                 // '{' 'allowing'
     || lk == 37407                 // '$' 'ancestor'
     || lk == 37408                 // '%' 'ancestor'
     || lk == 37652                 // '{' 'ancestor'
     || lk == 37919                 // '$' 'ancestor-or-self'
     || lk == 37920                 // '%' 'ancestor-or-self'
     || lk == 38164                 // '{' 'ancestor-or-self'
     || lk == 38431                 // '$' 'and'
     || lk == 38432                 // '%' 'and'
     || lk == 38676                 // '{' 'and'
     || lk == 39455                 // '$' 'append'
     || lk == 39456                 // '%' 'append'
     || lk == 39700                 // '{' 'append'
     || lk == 39967                 // '$' 'array'
     || lk == 39968                 // '%' 'array'
     || lk == 40212                 // '{' 'array'
     || lk == 40479                 // '$' 'as'
     || lk == 40480                 // '%' 'as'
     || lk == 40724                 // '{' 'as'
     || lk == 40991                 // '$' 'ascending'
     || lk == 40992                 // '%' 'ascending'
     || lk == 41236                 // '{' 'ascending'
     || lk == 41503                 // '$' 'at'
     || lk == 41504                 // '%' 'at'
     || lk == 41748                 // '{' 'at'
     || lk == 42015                 // '$' 'attribute'
     || lk == 42016                 // '%' 'attribute'
     || lk == 42260                 // '{' 'attribute'
     || lk == 42527                 // '$' 'base-uri'
     || lk == 42528                 // '%' 'base-uri'
     || lk == 42772                 // '{' 'base-uri'
     || lk == 43039                 // '$' 'before'
     || lk == 43040                 // '%' 'before'
     || lk == 43284                 // '{' 'before'
     || lk == 43551                 // '$' 'boundary-space'
     || lk == 43552                 // '%' 'boundary-space'
     || lk == 43796                 // '{' 'boundary-space'
     || lk == 44063                 // '$' 'break'
     || lk == 44064                 // '%' 'break'
     || lk == 44308                 // '{' 'break'
     || lk == 45087                 // '$' 'case'
     || lk == 45088                 // '%' 'case'
     || lk == 45332                 // '{' 'case'
     || lk == 45599                 // '$' 'cast'
     || lk == 45600                 // '%' 'cast'
     || lk == 45844                 // '{' 'cast'
     || lk == 46111                 // '$' 'castable'
     || lk == 46112                 // '%' 'castable'
     || lk == 46356                 // '{' 'castable'
     || lk == 46623                 // '$' 'catch'
     || lk == 46624                 // '%' 'catch'
     || lk == 46868                 // '{' 'catch'
     || lk == 47647                 // '$' 'child'
     || lk == 47648                 // '%' 'child'
     || lk == 47892                 // '{' 'child'
     || lk == 48159                 // '$' 'collation'
     || lk == 48160                 // '%' 'collation'
     || lk == 48404                 // '{' 'collation'
     || lk == 49183                 // '$' 'comment'
     || lk == 49184                 // '%' 'comment'
     || lk == 49428                 // '{' 'comment'
     || lk == 49695                 // '$' 'constraint'
     || lk == 49696                 // '%' 'constraint'
     || lk == 49940                 // '{' 'constraint'
     || lk == 50207                 // '$' 'construction'
     || lk == 50208                 // '%' 'construction'
     || lk == 50452                 // '{' 'construction'
     || lk == 51743                 // '$' 'context'
     || lk == 51744                 // '%' 'context'
     || lk == 51988                 // '{' 'context'
     || lk == 52255                 // '$' 'continue'
     || lk == 52256                 // '%' 'continue'
     || lk == 52500                 // '{' 'continue'
     || lk == 52767                 // '$' 'copy'
     || lk == 52768                 // '%' 'copy'
     || lk == 53012                 // '{' 'copy'
     || lk == 53279                 // '$' 'copy-namespaces'
     || lk == 53280                 // '%' 'copy-namespaces'
     || lk == 53524                 // '{' 'copy-namespaces'
     || lk == 53791                 // '$' 'count'
     || lk == 53792                 // '%' 'count'
     || lk == 54036                 // '{' 'count'
     || lk == 54303                 // '$' 'decimal-format'
     || lk == 54304                 // '%' 'decimal-format'
     || lk == 54548                 // '{' 'decimal-format'
     || lk == 55327                 // '$' 'declare'
     || lk == 55328                 // '%' 'declare'
     || lk == 55572                 // '{' 'declare'
     || lk == 55839                 // '$' 'default'
     || lk == 55840                 // '%' 'default'
     || lk == 56084                 // '{' 'default'
     || lk == 56351                 // '$' 'delete'
     || lk == 56352                 // '%' 'delete'
     || lk == 56596                 // '{' 'delete'
     || lk == 56863                 // '$' 'descendant'
     || lk == 56864                 // '%' 'descendant'
     || lk == 57108                 // '{' 'descendant'
     || lk == 57375                 // '$' 'descendant-or-self'
     || lk == 57376                 // '%' 'descendant-or-self'
     || lk == 57620                 // '{' 'descendant-or-self'
     || lk == 57887                 // '$' 'descending'
     || lk == 57888                 // '%' 'descending'
     || lk == 58132                 // '{' 'descending'
     || lk == 60447                 // '$' 'div'
     || lk == 60448                 // '%' 'div'
     || lk == 60692                 // '{' 'div'
     || lk == 60959                 // '$' 'document'
     || lk == 60960                 // '%' 'document'
     || lk == 61204                 // '{' 'document'
     || lk == 61471                 // '$' 'document-node'
     || lk == 61472                 // '%' 'document-node'
     || lk == 61716                 // '{' 'document-node'
     || lk == 61983                 // '$' 'element'
     || lk == 61984                 // '%' 'element'
     || lk == 62228                 // '{' 'element'
     || lk == 62495                 // '$' 'else'
     || lk == 62496                 // '%' 'else'
     || lk == 62740                 // '{' 'else'
     || lk == 63007                 // '$' 'empty'
     || lk == 63008                 // '%' 'empty'
     || lk == 63252                 // '{' 'empty'
     || lk == 63519                 // '$' 'empty-sequence'
     || lk == 63520                 // '%' 'empty-sequence'
     || lk == 63764                 // '{' 'empty-sequence'
     || lk == 64031                 // '$' 'encoding'
     || lk == 64032                 // '%' 'encoding'
     || lk == 64276                 // '{' 'encoding'
     || lk == 64543                 // '$' 'end'
     || lk == 64544                 // '%' 'end'
     || lk == 64788                 // '{' 'end'
     || lk == 65567                 // '$' 'eq'
     || lk == 65568                 // '%' 'eq'
     || lk == 65812                 // '{' 'eq'
     || lk == 66079                 // '$' 'every'
     || lk == 66080                 // '%' 'every'
     || lk == 66324                 // '{' 'every'
     || lk == 67103                 // '$' 'except'
     || lk == 67104                 // '%' 'except'
     || lk == 67348                 // '{' 'except'
     || lk == 67615                 // '$' 'exit'
     || lk == 67616                 // '%' 'exit'
     || lk == 67860                 // '{' 'exit'
     || lk == 68127                 // '$' 'external'
     || lk == 68128                 // '%' 'external'
     || lk == 68372                 // '{' 'external'
     || lk == 68639                 // '$' 'first'
     || lk == 68640                 // '%' 'first'
     || lk == 68884                 // '{' 'first'
     || lk == 69151                 // '$' 'following'
     || lk == 69152                 // '%' 'following'
     || lk == 69396                 // '{' 'following'
     || lk == 69663                 // '$' 'following-sibling'
     || lk == 69664                 // '%' 'following-sibling'
     || lk == 69908                 // '{' 'following-sibling'
     || lk == 70175                 // '$' 'for'
     || lk == 70176                 // '%' 'for'
     || lk == 70420                 // '{' 'for'
     || lk == 72223                 // '$' 'ft-option'
     || lk == 72224                 // '%' 'ft-option'
     || lk == 72468                 // '{' 'ft-option'
     || lk == 74271                 // '$' 'function'
     || lk == 74272                 // '%' 'function'
     || lk == 74516                 // '{' 'function'
     || lk == 74783                 // '$' 'ge'
     || lk == 74784                 // '%' 'ge'
     || lk == 75028                 // '{' 'ge'
     || lk == 75807                 // '$' 'group'
     || lk == 75808                 // '%' 'group'
     || lk == 76052                 // '{' 'group'
     || lk == 76831                 // '$' 'gt'
     || lk == 76832                 // '%' 'gt'
     || lk == 77076                 // '{' 'gt'
     || lk == 77343                 // '$' 'idiv'
     || lk == 77344                 // '%' 'idiv'
     || lk == 77588                 // '{' 'idiv'
     || lk == 77855                 // '$' 'if'
     || lk == 77856                 // '%' 'if'
     || lk == 78100                 // '{' 'if'
     || lk == 78367                 // '$' 'import'
     || lk == 78368                 // '%' 'import'
     || lk == 78612                 // '{' 'import'
     || lk == 78879                 // '$' 'in'
     || lk == 78880                 // '%' 'in'
     || lk == 79124                 // '{' 'in'
     || lk == 79391                 // '$' 'index'
     || lk == 79392                 // '%' 'index'
     || lk == 79636                 // '{' 'index'
     || lk == 81439                 // '$' 'insert'
     || lk == 81440                 // '%' 'insert'
     || lk == 81684                 // '{' 'insert'
     || lk == 81951                 // '$' 'instance'
     || lk == 81952                 // '%' 'instance'
     || lk == 82196                 // '{' 'instance'
     || lk == 82463                 // '$' 'integrity'
     || lk == 82464                 // '%' 'integrity'
     || lk == 82708                 // '{' 'integrity'
     || lk == 82975                 // '$' 'intersect'
     || lk == 82976                 // '%' 'intersect'
     || lk == 83220                 // '{' 'intersect'
     || lk == 83487                 // '$' 'into'
     || lk == 83488                 // '%' 'into'
     || lk == 83732                 // '{' 'into'
     || lk == 83999                 // '$' 'is'
     || lk == 84000                 // '%' 'is'
     || lk == 84244                 // '{' 'is'
     || lk == 84511                 // '$' 'item'
     || lk == 84512                 // '%' 'item'
     || lk == 84756                 // '{' 'item'
     || lk == 85023                 // '$' 'json'
     || lk == 85024                 // '%' 'json'
     || lk == 85268                 // '{' 'json'
     || lk == 85535                 // '$' 'json-item'
     || lk == 85536                 // '%' 'json-item'
     || lk == 85780                 // '{' 'json-item'
     || lk == 87071                 // '$' 'last'
     || lk == 87072                 // '%' 'last'
     || lk == 87316                 // '{' 'last'
     || lk == 87583                 // '$' 'lax'
     || lk == 87584                 // '%' 'lax'
     || lk == 87828                 // '{' 'lax'
     || lk == 88095                 // '$' 'le'
     || lk == 88096                 // '%' 'le'
     || lk == 88340                 // '{' 'le'
     || lk == 89119                 // '$' 'let'
     || lk == 89120                 // '%' 'let'
     || lk == 89364                 // '{' 'let'
     || lk == 90143                 // '$' 'loop'
     || lk == 90144                 // '%' 'loop'
     || lk == 90388                 // '{' 'loop'
     || lk == 91167                 // '$' 'lt'
     || lk == 91168                 // '%' 'lt'
     || lk == 91412                 // '{' 'lt'
     || lk == 92191                 // '$' 'mod'
     || lk == 92192                 // '%' 'mod'
     || lk == 92436                 // '{' 'mod'
     || lk == 92703                 // '$' 'modify'
     || lk == 92704                 // '%' 'modify'
     || lk == 92948                 // '{' 'modify'
     || lk == 93215                 // '$' 'module'
     || lk == 93216                 // '%' 'module'
     || lk == 93460                 // '{' 'module'
     || lk == 94239                 // '$' 'namespace'
     || lk == 94240                 // '%' 'namespace'
     || lk == 94484                 // '{' 'namespace'
     || lk == 94751                 // '$' 'namespace-node'
     || lk == 94752                 // '%' 'namespace-node'
     || lk == 94996                 // '{' 'namespace-node'
     || lk == 95263                 // '$' 'ne'
     || lk == 95264                 // '%' 'ne'
     || lk == 95508                 // '{' 'ne'
     || lk == 97823                 // '$' 'node'
     || lk == 97824                 // '%' 'node'
     || lk == 98068                 // '{' 'node'
     || lk == 98335                 // '$' 'nodes'
     || lk == 98336                 // '%' 'nodes'
     || lk == 98580                 // '{' 'nodes'
     || lk == 99359                 // '$' 'object'
     || lk == 99360                 // '%' 'object'
     || lk == 99604                 // '{' 'object'
     || lk == 101407                // '$' 'only'
     || lk == 101408                // '%' 'only'
     || lk == 101652                // '{' 'only'
     || lk == 101919                // '$' 'option'
     || lk == 101920                // '%' 'option'
     || lk == 102164                // '{' 'option'
     || lk == 102431                // '$' 'or'
     || lk == 102432                // '%' 'or'
     || lk == 102676                // '{' 'or'
     || lk == 102943                // '$' 'order'
     || lk == 102944                // '%' 'order'
     || lk == 103188                // '{' 'order'
     || lk == 103455                // '$' 'ordered'
     || lk == 103456                // '%' 'ordered'
     || lk == 103700                // '{' 'ordered'
     || lk == 103967                // '$' 'ordering'
     || lk == 103968                // '%' 'ordering'
     || lk == 104212                // '{' 'ordering'
     || lk == 105503                // '$' 'parent'
     || lk == 105504                // '%' 'parent'
     || lk == 105748                // '{' 'parent'
     || lk == 108575                // '$' 'preceding'
     || lk == 108576                // '%' 'preceding'
     || lk == 108820                // '{' 'preceding'
     || lk == 109087                // '$' 'preceding-sibling'
     || lk == 109088                // '%' 'preceding-sibling'
     || lk == 109332                // '{' 'preceding-sibling'
     || lk == 110623                // '$' 'processing-instruction'
     || lk == 110624                // '%' 'processing-instruction'
     || lk == 110868                // '{' 'processing-instruction'
     || lk == 111647                // '$' 'rename'
     || lk == 111648                // '%' 'rename'
     || lk == 111892                // '{' 'rename'
     || lk == 112159                // '$' 'replace'
     || lk == 112160                // '%' 'replace'
     || lk == 112404                // '{' 'replace'
     || lk == 112671                // '$' 'return'
     || lk == 112672                // '%' 'return'
     || lk == 112916                // '{' 'return'
     || lk == 113183                // '$' 'returning'
     || lk == 113184                // '%' 'returning'
     || lk == 113428                // '{' 'returning'
     || lk == 113695                // '$' 'revalidation'
     || lk == 113696                // '%' 'revalidation'
     || lk == 113940                // '{' 'revalidation'
     || lk == 114719                // '$' 'satisfies'
     || lk == 114720                // '%' 'satisfies'
     || lk == 114964                // '{' 'satisfies'
     || lk == 115231                // '$' 'schema'
     || lk == 115232                // '%' 'schema'
     || lk == 115476                // '{' 'schema'
     || lk == 115743                // '$' 'schema-attribute'
     || lk == 115744                // '%' 'schema-attribute'
     || lk == 115988                // '{' 'schema-attribute'
     || lk == 116255                // '$' 'schema-element'
     || lk == 116256                // '%' 'schema-element'
     || lk == 116500                // '{' 'schema-element'
     || lk == 116767                // '$' 'score'
     || lk == 116768                // '%' 'score'
     || lk == 117012                // '{' 'score'
     || lk == 117279                // '$' 'self'
     || lk == 117280                // '%' 'self'
     || lk == 117524                // '{' 'self'
     || lk == 119839                // '$' 'sliding'
     || lk == 119840                // '%' 'sliding'
     || lk == 120084                // '{' 'sliding'
     || lk == 120351                // '$' 'some'
     || lk == 120352                // '%' 'some'
     || lk == 120596                // '{' 'some'
     || lk == 120863                // '$' 'stable'
     || lk == 120864                // '%' 'stable'
     || lk == 121108                // '{' 'stable'
     || lk == 121375                // '$' 'start'
     || lk == 121376                // '%' 'start'
     || lk == 121620                // '{' 'start'
     || lk == 122911                // '$' 'strict'
     || lk == 122912                // '%' 'strict'
     || lk == 123156                // '{' 'strict'
     || lk == 123935                // '$' 'structured-item'
     || lk == 123936                // '%' 'structured-item'
     || lk == 124180                // '{' 'structured-item'
     || lk == 124447                // '$' 'switch'
     || lk == 124448                // '%' 'switch'
     || lk == 124692                // '{' 'switch'
     || lk == 124959                // '$' 'text'
     || lk == 124960                // '%' 'text'
     || lk == 125204                // '{' 'text'
     || lk == 127007                // '$' 'to'
     || lk == 127008                // '%' 'to'
     || lk == 127252                // '{' 'to'
     || lk == 127519                // '$' 'treat'
     || lk == 127520                // '%' 'treat'
     || lk == 127764                // '{' 'treat'
     || lk == 128031                // '$' 'try'
     || lk == 128032                // '%' 'try'
     || lk == 128276                // '{' 'try'
     || lk == 128543                // '$' 'tumbling'
     || lk == 128544                // '%' 'tumbling'
     || lk == 128788                // '{' 'tumbling'
     || lk == 129055                // '$' 'type'
     || lk == 129056                // '%' 'type'
     || lk == 129300                // '{' 'type'
     || lk == 129567                // '$' 'typeswitch'
     || lk == 129568                // '%' 'typeswitch'
     || lk == 129812                // '{' 'typeswitch'
     || lk == 130079                // '$' 'union'
     || lk == 130080                // '%' 'union'
     || lk == 130324                // '{' 'union'
     || lk == 131103                // '$' 'unordered'
     || lk == 131104                // '%' 'unordered'
     || lk == 131348                // '{' 'unordered'
     || lk == 131615                // '$' 'updating'
     || lk == 131616                // '%' 'updating'
     || lk == 131860                // '{' 'updating'
     || lk == 133151                // '$' 'validate'
     || lk == 133152                // '%' 'validate'
     || lk == 133396                // '{' 'validate'
     || lk == 133663                // '$' 'value'
     || lk == 133664                // '%' 'value'
     || lk == 133908                // '{' 'value'
     || lk == 134175                // '$' 'variable'
     || lk == 134176                // '%' 'variable'
     || lk == 134420                // '{' 'variable'
     || lk == 134687                // '$' 'version'
     || lk == 134688                // '%' 'version'
     || lk == 134932                // '{' 'version'
     || lk == 136223                // '$' 'where'
     || lk == 136224                // '%' 'where'
     || lk == 136468                // '{' 'where'
     || lk == 136735                // '$' 'while'
     || lk == 136736                // '%' 'while'
     || lk == 136980                // '{' 'while'
     || lk == 138271                // '$' 'with'
     || lk == 138272                // '%' 'with'
     || lk == 138516                // '{' 'with'
     || lk == 140319                // '$' 'xquery'
     || lk == 140320                // '%' 'xquery'
     || lk == 140564                // '{' 'xquery'
     || lk == 141588                // '{' '{'
     || lk == 142612                // '{' '{|'
     || lk == 144660)               // '{' '}'
    {
      lk = memoized(7, e0);
      if (lk == 0)
      {
        var b0A = b0; var e0A = e0; var l1A = l1;
        var b1A = b1; var e1A = e1; var l2A = l2;
        var b2A = b2; var e2A = e2;
        try
        {
          try_ApplyStatement();
          lk = -1;
        }
        catch (p1A)
        {
          try
          {
            b0 = b0A; e0 = e0A; l1 = l1A; if (l1 == 0) {end = e0A;} else {
            b1 = b1A; e1 = e1A; l2 = l2A; if (l2 == 0) {end = e1A;} else {
            b2 = b2A; e2 = e2A; end = e2A; }}
            try_AssignStatement();
            lk = -2;
          }
          catch (p2A)
          {
            try
            {
              b0 = b0A; e0 = e0A; l1 = l1A; if (l1 == 0) {end = e0A;} else {
              b1 = b1A; e1 = e1A; l2 = l2A; if (l2 == 0) {end = e1A;} else {
              b2 = b2A; e2 = e2A; end = e2A; }}
              try_BlockStatement();
              lk = -3;
            }
            catch (p3A)
            {
              try
              {
                b0 = b0A; e0 = e0A; l1 = l1A; if (l1 == 0) {end = e0A;} else {
                b1 = b1A; e1 = e1A; l2 = l2A; if (l2 == 0) {end = e1A;} else {
                b2 = b2A; e2 = e2A; end = e2A; }}
                try_VarDeclStatement();
                lk = -12;
              }
              catch (p12A)
              {
                lk = -13;
              }
            }
          }
        }
        b0 = b0A; e0 = e0A; l1 = l1A; if (l1 == 0) {end = e0A;} else {
        b1 = b1A; e1 = e1A; l2 = l2A; if (l2 == 0) {end = e1A;} else {
        b2 = b2A; e2 = e2A; end = e2A; }}
        memoize(7, e0, lk);
      }
    }
    switch (lk)
    {
    case -2:
      parse_AssignStatement();
      break;
    case -3:
      parse_BlockStatement();
      break;
    case 90198:                     // 'break' 'loop'
      parse_BreakStatement();
      break;
    case 90214:                     // 'continue' 'loop'
      parse_ContinueStatement();
      break;
    case 113284:                    // 'exit' 'returning'
      parse_ExitStatement();
      break;
    case 16009:                     // 'for' '$'
    case 16046:                     // 'let' '$'
    case 116910:                    // 'let' 'score'
    case 119945:                    // 'for' 'sliding'
    case 128649:                    // 'for' 'tumbling'
      parse_FLWORStatement();
      break;
    case 17560:                     // 'if' '('
      parse_IfStatement();
      break;
    case 17651:                     // 'switch' '('
      parse_SwitchStatement();
      break;
    case 141562:                    // 'try' '{'
      parse_TryCatchStatement();
      break;
    case 17661:                     // 'typeswitch' '('
      parse_TypeswitchStatement();
      break;
    case -12:
    case 16134:                     // 'variable' '$'
      parse_VarDeclStatement();
      break;
    case -13:
      parse_WhileStatement();
      break;
    case 53:                        // ';'
      parse_VoidStatement();
      break;
    default:
      parse_ApplyStatement();
    }
    eventHandler.endNonterminal("Statement", e0);
  }

  function try_Statement()
  {
    switch (l1)
    {
    case 132:                       // 'exit'
      lookahead2W(189);             // S^WS | '!' | '!=' | '#' | '(' | '(:' | '*' | '+' | '-' | '/' | '//' | ';' | '<' |
      break;
    case 137:                       // 'for'
      lookahead2W(196);             // S^WS | '!' | '!=' | '#' | '$' | '(' | '(:' | '*' | '+' | '-' | '/' | '//' | ';' |
      break;
    case 174:                       // 'let'
      lookahead2W(193);             // S^WS | '!' | '!=' | '#' | '$' | '(' | '(:' | '*' | '+' | '-' | '/' | '//' | ';' |
      break;
    case 250:                       // 'try'
      lookahead2W(190);             // S^WS | '!' | '!=' | '#' | '(' | '(:' | '*' | '+' | '-' | '/' | '//' | ';' | '<' |
      break;
    case 262:                       // 'variable'
      lookahead2W(187);             // S^WS | '!' | '!=' | '#' | '$' | '(' | '(:' | '*' | '+' | '-' | '/' | '//' | ';' |
      break;
    case 276:                       // '{'
      lookahead2W(277);             // Wildcard | EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral |
      break;
    case 31:                        // '$'
    case 32:                        // '%'
      lookahead2W(255);             // EQName^Token | S^WS | '(:' | 'after' | 'allowing' | 'ancestor' |
      break;
    case 86:                        // 'break'
    case 102:                       // 'continue'
      lookahead2W(188);             // S^WS | '!' | '!=' | '#' | '(' | '(:' | '*' | '+' | '-' | '/' | '//' | ';' | '<' |
      break;
    case 152:                       // 'if'
    case 243:                       // 'switch'
    case 253:                       // 'typeswitch'
    case 267:                       // 'while'
      lookahead2W(185);             // S^WS | '!' | '!=' | '#' | '(' | '(:' | '*' | '+' | '-' | '/' | '//' | ';' | '<' |
      break;
    default:
      lk = l1;
    }
    if (lk == 2836                  // '{' Wildcard
     || lk == 3103                  // '$' EQName^Token
     || lk == 3104                  // '%' EQName^Token
     || lk == 3348                  // '{' EQName^Token
     || lk == 4372                  // '{' IntegerLiteral
     || lk == 4884                  // '{' DecimalLiteral
     || lk == 5396                  // '{' DoubleLiteral
     || lk == 5908                  // '{' StringLiteral
     || lk == 16148                 // '{' '$'
     || lk == 16660                 // '{' '%'
     || lk == 17675                 // 'while' '('
     || lk == 17684                 // '{' '('
     || lk == 18196                 // '{' '(#'
     || lk == 20756                 // '{' '+'
     || lk == 21780                 // '{' '-'
     || lk == 22804                 // '{' '.'
     || lk == 23316                 // '{' '..'
     || lk == 23828                 // '{' '/'
     || lk == 24340                 // '{' '//'
     || lk == 27412                 // '{' ';'
     || lk == 27924                 // '{' '<'
     || lk == 28436                 // '{' '<!--'
     || lk == 30484                 // '{' '<?'
     || lk == 34068                 // '{' '@'
     || lk == 35092                 // '{' '['
     || lk == 35871                 // '$' 'after'
     || lk == 35872                 // '%' 'after'
     || lk == 36116                 // '{' 'after'
     || lk == 36895                 // '$' 'allowing'
     || lk == 36896                 // '%' 'allowing'
     || lk == 37140                 // '{' 'allowing'
     || lk == 37407                 // '$' 'ancestor'
     || lk == 37408                 // '%' 'ancestor'
     || lk == 37652                 // '{' 'ancestor'
     || lk == 37919                 // '$' 'ancestor-or-self'
     || lk == 37920                 // '%' 'ancestor-or-self'
     || lk == 38164                 // '{' 'ancestor-or-self'
     || lk == 38431                 // '$' 'and'
     || lk == 38432                 // '%' 'and'
     || lk == 38676                 // '{' 'and'
     || lk == 39455                 // '$' 'append'
     || lk == 39456                 // '%' 'append'
     || lk == 39700                 // '{' 'append'
     || lk == 39967                 // '$' 'array'
     || lk == 39968                 // '%' 'array'
     || lk == 40212                 // '{' 'array'
     || lk == 40479                 // '$' 'as'
     || lk == 40480                 // '%' 'as'
     || lk == 40724                 // '{' 'as'
     || lk == 40991                 // '$' 'ascending'
     || lk == 40992                 // '%' 'ascending'
     || lk == 41236                 // '{' 'ascending'
     || lk == 41503                 // '$' 'at'
     || lk == 41504                 // '%' 'at'
     || lk == 41748                 // '{' 'at'
     || lk == 42015                 // '$' 'attribute'
     || lk == 42016                 // '%' 'attribute'
     || lk == 42260                 // '{' 'attribute'
     || lk == 42527                 // '$' 'base-uri'
     || lk == 42528                 // '%' 'base-uri'
     || lk == 42772                 // '{' 'base-uri'
     || lk == 43039                 // '$' 'before'
     || lk == 43040                 // '%' 'before'
     || lk == 43284                 // '{' 'before'
     || lk == 43551                 // '$' 'boundary-space'
     || lk == 43552                 // '%' 'boundary-space'
     || lk == 43796                 // '{' 'boundary-space'
     || lk == 44063                 // '$' 'break'
     || lk == 44064                 // '%' 'break'
     || lk == 44308                 // '{' 'break'
     || lk == 45087                 // '$' 'case'
     || lk == 45088                 // '%' 'case'
     || lk == 45332                 // '{' 'case'
     || lk == 45599                 // '$' 'cast'
     || lk == 45600                 // '%' 'cast'
     || lk == 45844                 // '{' 'cast'
     || lk == 46111                 // '$' 'castable'
     || lk == 46112                 // '%' 'castable'
     || lk == 46356                 // '{' 'castable'
     || lk == 46623                 // '$' 'catch'
     || lk == 46624                 // '%' 'catch'
     || lk == 46868                 // '{' 'catch'
     || lk == 47647                 // '$' 'child'
     || lk == 47648                 // '%' 'child'
     || lk == 47892                 // '{' 'child'
     || lk == 48159                 // '$' 'collation'
     || lk == 48160                 // '%' 'collation'
     || lk == 48404                 // '{' 'collation'
     || lk == 49183                 // '$' 'comment'
     || lk == 49184                 // '%' 'comment'
     || lk == 49428                 // '{' 'comment'
     || lk == 49695                 // '$' 'constraint'
     || lk == 49696                 // '%' 'constraint'
     || lk == 49940                 // '{' 'constraint'
     || lk == 50207                 // '$' 'construction'
     || lk == 50208                 // '%' 'construction'
     || lk == 50452                 // '{' 'construction'
     || lk == 51743                 // '$' 'context'
     || lk == 51744                 // '%' 'context'
     || lk == 51988                 // '{' 'context'
     || lk == 52255                 // '$' 'continue'
     || lk == 52256                 // '%' 'continue'
     || lk == 52500                 // '{' 'continue'
     || lk == 52767                 // '$' 'copy'
     || lk == 52768                 // '%' 'copy'
     || lk == 53012                 // '{' 'copy'
     || lk == 53279                 // '$' 'copy-namespaces'
     || lk == 53280                 // '%' 'copy-namespaces'
     || lk == 53524                 // '{' 'copy-namespaces'
     || lk == 53791                 // '$' 'count'
     || lk == 53792                 // '%' 'count'
     || lk == 54036                 // '{' 'count'
     || lk == 54303                 // '$' 'decimal-format'
     || lk == 54304                 // '%' 'decimal-format'
     || lk == 54548                 // '{' 'decimal-format'
     || lk == 55327                 // '$' 'declare'
     || lk == 55328                 // '%' 'declare'
     || lk == 55572                 // '{' 'declare'
     || lk == 55839                 // '$' 'default'
     || lk == 55840                 // '%' 'default'
     || lk == 56084                 // '{' 'default'
     || lk == 56351                 // '$' 'delete'
     || lk == 56352                 // '%' 'delete'
     || lk == 56596                 // '{' 'delete'
     || lk == 56863                 // '$' 'descendant'
     || lk == 56864                 // '%' 'descendant'
     || lk == 57108                 // '{' 'descendant'
     || lk == 57375                 // '$' 'descendant-or-self'
     || lk == 57376                 // '%' 'descendant-or-self'
     || lk == 57620                 // '{' 'descendant-or-self'
     || lk == 57887                 // '$' 'descending'
     || lk == 57888                 // '%' 'descending'
     || lk == 58132                 // '{' 'descending'
     || lk == 60447                 // '$' 'div'
     || lk == 60448                 // '%' 'div'
     || lk == 60692                 // '{' 'div'
     || lk == 60959                 // '$' 'document'
     || lk == 60960                 // '%' 'document'
     || lk == 61204                 // '{' 'document'
     || lk == 61471                 // '$' 'document-node'
     || lk == 61472                 // '%' 'document-node'
     || lk == 61716                 // '{' 'document-node'
     || lk == 61983                 // '$' 'element'
     || lk == 61984                 // '%' 'element'
     || lk == 62228                 // '{' 'element'
     || lk == 62495                 // '$' 'else'
     || lk == 62496                 // '%' 'else'
     || lk == 62740                 // '{' 'else'
     || lk == 63007                 // '$' 'empty'
     || lk == 63008                 // '%' 'empty'
     || lk == 63252                 // '{' 'empty'
     || lk == 63519                 // '$' 'empty-sequence'
     || lk == 63520                 // '%' 'empty-sequence'
     || lk == 63764                 // '{' 'empty-sequence'
     || lk == 64031                 // '$' 'encoding'
     || lk == 64032                 // '%' 'encoding'
     || lk == 64276                 // '{' 'encoding'
     || lk == 64543                 // '$' 'end'
     || lk == 64544                 // '%' 'end'
     || lk == 64788                 // '{' 'end'
     || lk == 65567                 // '$' 'eq'
     || lk == 65568                 // '%' 'eq'
     || lk == 65812                 // '{' 'eq'
     || lk == 66079                 // '$' 'every'
     || lk == 66080                 // '%' 'every'
     || lk == 66324                 // '{' 'every'
     || lk == 67103                 // '$' 'except'
     || lk == 67104                 // '%' 'except'
     || lk == 67348                 // '{' 'except'
     || lk == 67615                 // '$' 'exit'
     || lk == 67616                 // '%' 'exit'
     || lk == 67860                 // '{' 'exit'
     || lk == 68127                 // '$' 'external'
     || lk == 68128                 // '%' 'external'
     || lk == 68372                 // '{' 'external'
     || lk == 68639                 // '$' 'first'
     || lk == 68640                 // '%' 'first'
     || lk == 68884                 // '{' 'first'
     || lk == 69151                 // '$' 'following'
     || lk == 69152                 // '%' 'following'
     || lk == 69396                 // '{' 'following'
     || lk == 69663                 // '$' 'following-sibling'
     || lk == 69664                 // '%' 'following-sibling'
     || lk == 69908                 // '{' 'following-sibling'
     || lk == 70175                 // '$' 'for'
     || lk == 70176                 // '%' 'for'
     || lk == 70420                 // '{' 'for'
     || lk == 72223                 // '$' 'ft-option'
     || lk == 72224                 // '%' 'ft-option'
     || lk == 72468                 // '{' 'ft-option'
     || lk == 74271                 // '$' 'function'
     || lk == 74272                 // '%' 'function'
     || lk == 74516                 // '{' 'function'
     || lk == 74783                 // '$' 'ge'
     || lk == 74784                 // '%' 'ge'
     || lk == 75028                 // '{' 'ge'
     || lk == 75807                 // '$' 'group'
     || lk == 75808                 // '%' 'group'
     || lk == 76052                 // '{' 'group'
     || lk == 76831                 // '$' 'gt'
     || lk == 76832                 // '%' 'gt'
     || lk == 77076                 // '{' 'gt'
     || lk == 77343                 // '$' 'idiv'
     || lk == 77344                 // '%' 'idiv'
     || lk == 77588                 // '{' 'idiv'
     || lk == 77855                 // '$' 'if'
     || lk == 77856                 // '%' 'if'
     || lk == 78100                 // '{' 'if'
     || lk == 78367                 // '$' 'import'
     || lk == 78368                 // '%' 'import'
     || lk == 78612                 // '{' 'import'
     || lk == 78879                 // '$' 'in'
     || lk == 78880                 // '%' 'in'
     || lk == 79124                 // '{' 'in'
     || lk == 79391                 // '$' 'index'
     || lk == 79392                 // '%' 'index'
     || lk == 79636                 // '{' 'index'
     || lk == 81439                 // '$' 'insert'
     || lk == 81440                 // '%' 'insert'
     || lk == 81684                 // '{' 'insert'
     || lk == 81951                 // '$' 'instance'
     || lk == 81952                 // '%' 'instance'
     || lk == 82196                 // '{' 'instance'
     || lk == 82463                 // '$' 'integrity'
     || lk == 82464                 // '%' 'integrity'
     || lk == 82708                 // '{' 'integrity'
     || lk == 82975                 // '$' 'intersect'
     || lk == 82976                 // '%' 'intersect'
     || lk == 83220                 // '{' 'intersect'
     || lk == 83487                 // '$' 'into'
     || lk == 83488                 // '%' 'into'
     || lk == 83732                 // '{' 'into'
     || lk == 83999                 // '$' 'is'
     || lk == 84000                 // '%' 'is'
     || lk == 84244                 // '{' 'is'
     || lk == 84511                 // '$' 'item'
     || lk == 84512                 // '%' 'item'
     || lk == 84756                 // '{' 'item'
     || lk == 85023                 // '$' 'json'
     || lk == 85024                 // '%' 'json'
     || lk == 85268                 // '{' 'json'
     || lk == 85535                 // '$' 'json-item'
     || lk == 85536                 // '%' 'json-item'
     || lk == 85780                 // '{' 'json-item'
     || lk == 87071                 // '$' 'last'
     || lk == 87072                 // '%' 'last'
     || lk == 87316                 // '{' 'last'
     || lk == 87583                 // '$' 'lax'
     || lk == 87584                 // '%' 'lax'
     || lk == 87828                 // '{' 'lax'
     || lk == 88095                 // '$' 'le'
     || lk == 88096                 // '%' 'le'
     || lk == 88340                 // '{' 'le'
     || lk == 89119                 // '$' 'let'
     || lk == 89120                 // '%' 'let'
     || lk == 89364                 // '{' 'let'
     || lk == 90143                 // '$' 'loop'
     || lk == 90144                 // '%' 'loop'
     || lk == 90388                 // '{' 'loop'
     || lk == 91167                 // '$' 'lt'
     || lk == 91168                 // '%' 'lt'
     || lk == 91412                 // '{' 'lt'
     || lk == 92191                 // '$' 'mod'
     || lk == 92192                 // '%' 'mod'
     || lk == 92436                 // '{' 'mod'
     || lk == 92703                 // '$' 'modify'
     || lk == 92704                 // '%' 'modify'
     || lk == 92948                 // '{' 'modify'
     || lk == 93215                 // '$' 'module'
     || lk == 93216                 // '%' 'module'
     || lk == 93460                 // '{' 'module'
     || lk == 94239                 // '$' 'namespace'
     || lk == 94240                 // '%' 'namespace'
     || lk == 94484                 // '{' 'namespace'
     || lk == 94751                 // '$' 'namespace-node'
     || lk == 94752                 // '%' 'namespace-node'
     || lk == 94996                 // '{' 'namespace-node'
     || lk == 95263                 // '$' 'ne'
     || lk == 95264                 // '%' 'ne'
     || lk == 95508                 // '{' 'ne'
     || lk == 97823                 // '$' 'node'
     || lk == 97824                 // '%' 'node'
     || lk == 98068                 // '{' 'node'
     || lk == 98335                 // '$' 'nodes'
     || lk == 98336                 // '%' 'nodes'
     || lk == 98580                 // '{' 'nodes'
     || lk == 99359                 // '$' 'object'
     || lk == 99360                 // '%' 'object'
     || lk == 99604                 // '{' 'object'
     || lk == 101407                // '$' 'only'
     || lk == 101408                // '%' 'only'
     || lk == 101652                // '{' 'only'
     || lk == 101919                // '$' 'option'
     || lk == 101920                // '%' 'option'
     || lk == 102164                // '{' 'option'
     || lk == 102431                // '$' 'or'
     || lk == 102432                // '%' 'or'
     || lk == 102676                // '{' 'or'
     || lk == 102943                // '$' 'order'
     || lk == 102944                // '%' 'order'
     || lk == 103188                // '{' 'order'
     || lk == 103455                // '$' 'ordered'
     || lk == 103456                // '%' 'ordered'
     || lk == 103700                // '{' 'ordered'
     || lk == 103967                // '$' 'ordering'
     || lk == 103968                // '%' 'ordering'
     || lk == 104212                // '{' 'ordering'
     || lk == 105503                // '$' 'parent'
     || lk == 105504                // '%' 'parent'
     || lk == 105748                // '{' 'parent'
     || lk == 108575                // '$' 'preceding'
     || lk == 108576                // '%' 'preceding'
     || lk == 108820                // '{' 'preceding'
     || lk == 109087                // '$' 'preceding-sibling'
     || lk == 109088                // '%' 'preceding-sibling'
     || lk == 109332                // '{' 'preceding-sibling'
     || lk == 110623                // '$' 'processing-instruction'
     || lk == 110624                // '%' 'processing-instruction'
     || lk == 110868                // '{' 'processing-instruction'
     || lk == 111647                // '$' 'rename'
     || lk == 111648                // '%' 'rename'
     || lk == 111892                // '{' 'rename'
     || lk == 112159                // '$' 'replace'
     || lk == 112160                // '%' 'replace'
     || lk == 112404                // '{' 'replace'
     || lk == 112671                // '$' 'return'
     || lk == 112672                // '%' 'return'
     || lk == 112916                // '{' 'return'
     || lk == 113183                // '$' 'returning'
     || lk == 113184                // '%' 'returning'
     || lk == 113428                // '{' 'returning'
     || lk == 113695                // '$' 'revalidation'
     || lk == 113696                // '%' 'revalidation'
     || lk == 113940                // '{' 'revalidation'
     || lk == 114719                // '$' 'satisfies'
     || lk == 114720                // '%' 'satisfies'
     || lk == 114964                // '{' 'satisfies'
     || lk == 115231                // '$' 'schema'
     || lk == 115232                // '%' 'schema'
     || lk == 115476                // '{' 'schema'
     || lk == 115743                // '$' 'schema-attribute'
     || lk == 115744                // '%' 'schema-attribute'
     || lk == 115988                // '{' 'schema-attribute'
     || lk == 116255                // '$' 'schema-element'
     || lk == 116256                // '%' 'schema-element'
     || lk == 116500                // '{' 'schema-element'
     || lk == 116767                // '$' 'score'
     || lk == 116768                // '%' 'score'
     || lk == 117012                // '{' 'score'
     || lk == 117279                // '$' 'self'
     || lk == 117280                // '%' 'self'
     || lk == 117524                // '{' 'self'
     || lk == 119839                // '$' 'sliding'
     || lk == 119840                // '%' 'sliding'
     || lk == 120084                // '{' 'sliding'
     || lk == 120351                // '$' 'some'
     || lk == 120352                // '%' 'some'
     || lk == 120596                // '{' 'some'
     || lk == 120863                // '$' 'stable'
     || lk == 120864                // '%' 'stable'
     || lk == 121108                // '{' 'stable'
     || lk == 121375                // '$' 'start'
     || lk == 121376                // '%' 'start'
     || lk == 121620                // '{' 'start'
     || lk == 122911                // '$' 'strict'
     || lk == 122912                // '%' 'strict'
     || lk == 123156                // '{' 'strict'
     || lk == 123935                // '$' 'structured-item'
     || lk == 123936                // '%' 'structured-item'
     || lk == 124180                // '{' 'structured-item'
     || lk == 124447                // '$' 'switch'
     || lk == 124448                // '%' 'switch'
     || lk == 124692                // '{' 'switch'
     || lk == 124959                // '$' 'text'
     || lk == 124960                // '%' 'text'
     || lk == 125204                // '{' 'text'
     || lk == 127007                // '$' 'to'
     || lk == 127008                // '%' 'to'
     || lk == 127252                // '{' 'to'
     || lk == 127519                // '$' 'treat'
     || lk == 127520                // '%' 'treat'
     || lk == 127764                // '{' 'treat'
     || lk == 128031                // '$' 'try'
     || lk == 128032                // '%' 'try'
     || lk == 128276                // '{' 'try'
     || lk == 128543                // '$' 'tumbling'
     || lk == 128544                // '%' 'tumbling'
     || lk == 128788                // '{' 'tumbling'
     || lk == 129055                // '$' 'type'
     || lk == 129056                // '%' 'type'
     || lk == 129300                // '{' 'type'
     || lk == 129567                // '$' 'typeswitch'
     || lk == 129568                // '%' 'typeswitch'
     || lk == 129812                // '{' 'typeswitch'
     || lk == 130079                // '$' 'union'
     || lk == 130080                // '%' 'union'
     || lk == 130324                // '{' 'union'
     || lk == 131103                // '$' 'unordered'
     || lk == 131104                // '%' 'unordered'
     || lk == 131348                // '{' 'unordered'
     || lk == 131615                // '$' 'updating'
     || lk == 131616                // '%' 'updating'
     || lk == 131860                // '{' 'updating'
     || lk == 133151                // '$' 'validate'
     || lk == 133152                // '%' 'validate'
     || lk == 133396                // '{' 'validate'
     || lk == 133663                // '$' 'value'
     || lk == 133664                // '%' 'value'
     || lk == 133908                // '{' 'value'
     || lk == 134175                // '$' 'variable'
     || lk == 134176                // '%' 'variable'
     || lk == 134420                // '{' 'variable'
     || lk == 134687                // '$' 'version'
     || lk == 134688                // '%' 'version'
     || lk == 134932                // '{' 'version'
     || lk == 136223                // '$' 'where'
     || lk == 136224                // '%' 'where'
     || lk == 136468                // '{' 'where'
     || lk == 136735                // '$' 'while'
     || lk == 136736                // '%' 'while'
     || lk == 136980                // '{' 'while'
     || lk == 138271                // '$' 'with'
     || lk == 138272                // '%' 'with'
     || lk == 138516                // '{' 'with'
     || lk == 140319                // '$' 'xquery'
     || lk == 140320                // '%' 'xquery'
     || lk == 140564                // '{' 'xquery'
     || lk == 141588                // '{' '{'
     || lk == 142612                // '{' '{|'
     || lk == 144660)               // '{' '}'
    {
      lk = memoized(7, e0);
      if (lk == 0)
      {
        var b0A = b0; var e0A = e0; var l1A = l1;
        var b1A = b1; var e1A = e1; var l2A = l2;
        var b2A = b2; var e2A = e2;
        try
        {
          try_ApplyStatement();
          memoize(7, e0A, -1);
          lk = -15;
        }
        catch (p1A)
        {
          try
          {
            b0 = b0A; e0 = e0A; l1 = l1A; if (l1 == 0) {end = e0A;} else {
            b1 = b1A; e1 = e1A; l2 = l2A; if (l2 == 0) {end = e1A;} else {
            b2 = b2A; e2 = e2A; end = e2A; }}
            try_AssignStatement();
            memoize(7, e0A, -2);
            lk = -15;
          }
          catch (p2A)
          {
            try
            {
              b0 = b0A; e0 = e0A; l1 = l1A; if (l1 == 0) {end = e0A;} else {
              b1 = b1A; e1 = e1A; l2 = l2A; if (l2 == 0) {end = e1A;} else {
              b2 = b2A; e2 = e2A; end = e2A; }}
              try_BlockStatement();
              memoize(7, e0A, -3);
              lk = -15;
            }
            catch (p3A)
            {
              try
              {
                b0 = b0A; e0 = e0A; l1 = l1A; if (l1 == 0) {end = e0A;} else {
                b1 = b1A; e1 = e1A; l2 = l2A; if (l2 == 0) {end = e1A;} else {
                b2 = b2A; e2 = e2A; end = e2A; }}
                try_VarDeclStatement();
                memoize(7, e0A, -12);
                lk = -15;
              }
              catch (p12A)
              {
                lk = -13;
                b0 = b0A; e0 = e0A; l1 = l1A; if (l1 == 0) {end = e0A;} else {
                b1 = b1A; e1 = e1A; l2 = l2A; if (l2 == 0) {end = e1A;} else {
                b2 = b2A; e2 = e2A; end = e2A; }}
                memoize(7, e0A, -13);
              }
            }
          }
        }
      }
    }
    switch (lk)
    {
    case -2:
      try_AssignStatement();
      break;
    case -3:
      try_BlockStatement();
      break;
    case 90198:                     // 'break' 'loop'
      try_BreakStatement();
      break;
    case 90214:                     // 'continue' 'loop'
      try_ContinueStatement();
      break;
    case 113284:                    // 'exit' 'returning'
      try_ExitStatement();
      break;
    case 16009:                     // 'for' '$'
    case 16046:                     // 'let' '$'
    case 116910:                    // 'let' 'score'
    case 119945:                    // 'for' 'sliding'
    case 128649:                    // 'for' 'tumbling'
      try_FLWORStatement();
      break;
    case 17560:                     // 'if' '('
      try_IfStatement();
      break;
    case 17651:                     // 'switch' '('
      try_SwitchStatement();
      break;
    case 141562:                    // 'try' '{'
      try_TryCatchStatement();
      break;
    case 17661:                     // 'typeswitch' '('
      try_TypeswitchStatement();
      break;
    case -12:
    case 16134:                     // 'variable' '$'
      try_VarDeclStatement();
      break;
    case -13:
      try_WhileStatement();
      break;
    case 53:                        // ';'
      try_VoidStatement();
      break;
    case -15:
      break;
    default:
      try_ApplyStatement();
    }
  }

  function parse_ApplyStatement()
  {
    eventHandler.startNonterminal("ApplyStatement", e0);
    parse_ExprSimple();
    shift(53);                      // ';'
    eventHandler.endNonterminal("ApplyStatement", e0);
  }

  function try_ApplyStatement()
  {
    try_ExprSimple();
    shiftT(53);                     // ';'
  }

  function parse_AssignStatement()
  {
    eventHandler.startNonterminal("AssignStatement", e0);
    shift(31);                      // '$'
    lookahead1W(255);               // EQName^Token | S^WS | '(:' | 'after' | 'allowing' | 'ancestor' |
    whitespace();
    parse_VarName();
    lookahead1W(27);                // S^WS | '(:' | ':='
    shift(52);                      // ':='
    lookahead1W(267);               // Wildcard | EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral |
    whitespace();
    parse_ExprSingle();
    shift(53);                      // ';'
    eventHandler.endNonterminal("AssignStatement", e0);
  }

  function try_AssignStatement()
  {
    shiftT(31);                     // '$'
    lookahead1W(255);               // EQName^Token | S^WS | '(:' | 'after' | 'allowing' | 'ancestor' |
    try_VarName();
    lookahead1W(27);                // S^WS | '(:' | ':='
    shiftT(52);                     // ':='
    lookahead1W(267);               // Wildcard | EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral |
    try_ExprSingle();
    shiftT(53);                     // ';'
  }

  function parse_BlockStatement()
  {
    eventHandler.startNonterminal("BlockStatement", e0);
    shift(276);                     // '{'
    lookahead1W(277);               // Wildcard | EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral |
    whitespace();
    parse_Statements();
    shift(282);                     // '}'
    eventHandler.endNonterminal("BlockStatement", e0);
  }

  function try_BlockStatement()
  {
    shiftT(276);                    // '{'
    lookahead1W(277);               // Wildcard | EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral |
    try_Statements();
    shiftT(282);                    // '}'
  }

  function parse_BreakStatement()
  {
    eventHandler.startNonterminal("BreakStatement", e0);
    shift(86);                      // 'break'
    lookahead1W(59);                // S^WS | '(:' | 'loop'
    shift(176);                     // 'loop'
    lookahead1W(28);                // S^WS | '(:' | ';'
    shift(53);                      // ';'
    eventHandler.endNonterminal("BreakStatement", e0);
  }

  function try_BreakStatement()
  {
    shiftT(86);                     // 'break'
    lookahead1W(59);                // S^WS | '(:' | 'loop'
    shiftT(176);                    // 'loop'
    lookahead1W(28);                // S^WS | '(:' | ';'
    shiftT(53);                     // ';'
  }

  function parse_ContinueStatement()
  {
    eventHandler.startNonterminal("ContinueStatement", e0);
    shift(102);                     // 'continue'
    lookahead1W(59);                // S^WS | '(:' | 'loop'
    shift(176);                     // 'loop'
    lookahead1W(28);                // S^WS | '(:' | ';'
    shift(53);                      // ';'
    eventHandler.endNonterminal("ContinueStatement", e0);
  }

  function try_ContinueStatement()
  {
    shiftT(102);                    // 'continue'
    lookahead1W(59);                // S^WS | '(:' | 'loop'
    shiftT(176);                    // 'loop'
    lookahead1W(28);                // S^WS | '(:' | ';'
    shiftT(53);                     // ';'
  }

  function parse_ExitStatement()
  {
    eventHandler.startNonterminal("ExitStatement", e0);
    shift(132);                     // 'exit'
    lookahead1W(71);                // S^WS | '(:' | 'returning'
    shift(221);                     // 'returning'
    lookahead1W(267);               // Wildcard | EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral |
    whitespace();
    parse_ExprSingle();
    shift(53);                      // ';'
    eventHandler.endNonterminal("ExitStatement", e0);
  }

  function try_ExitStatement()
  {
    shiftT(132);                    // 'exit'
    lookahead1W(71);                // S^WS | '(:' | 'returning'
    shiftT(221);                    // 'returning'
    lookahead1W(267);               // Wildcard | EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral |
    try_ExprSingle();
    shiftT(53);                     // ';'
  }

  function parse_FLWORStatement()
  {
    eventHandler.startNonterminal("FLWORStatement", e0);
    parse_InitialClause();
    for (;;)
    {
      lookahead1W(173);             // S^WS | '(:' | 'count' | 'for' | 'group' | 'let' | 'order' | 'return' | 'stable' |
      if (l1 == 220)                // 'return'
      {
        break;
      }
      whitespace();
      parse_IntermediateClause();
    }
    whitespace();
    parse_ReturnStatement();
    eventHandler.endNonterminal("FLWORStatement", e0);
  }

  function try_FLWORStatement()
  {
    try_InitialClause();
    for (;;)
    {
      lookahead1W(173);             // S^WS | '(:' | 'count' | 'for' | 'group' | 'let' | 'order' | 'return' | 'stable' |
      if (l1 == 220)                // 'return'
      {
        break;
      }
      try_IntermediateClause();
    }
    try_ReturnStatement();
  }

  function parse_ReturnStatement()
  {
    eventHandler.startNonterminal("ReturnStatement", e0);
    shift(220);                     // 'return'
    lookahead1W(270);               // Wildcard | EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral |
    whitespace();
    parse_Statement();
    eventHandler.endNonterminal("ReturnStatement", e0);
  }

  function try_ReturnStatement()
  {
    shiftT(220);                    // 'return'
    lookahead1W(270);               // Wildcard | EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral |
    try_Statement();
  }

  function parse_IfStatement()
  {
    eventHandler.startNonterminal("IfStatement", e0);
    shift(152);                     // 'if'
    lookahead1W(22);                // S^WS | '(' | '(:'
    shift(34);                      // '('
    lookahead1W(267);               // Wildcard | EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral |
    whitespace();
    parse_Expr();
    shift(37);                      // ')'
    lookahead1W(77);                // S^WS | '(:' | 'then'
    shift(245);                     // 'then'
    lookahead1W(270);               // Wildcard | EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral |
    whitespace();
    parse_Statement();
    lookahead1W(48);                // S^WS | '(:' | 'else'
    shift(122);                     // 'else'
    lookahead1W(270);               // Wildcard | EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral |
    whitespace();
    parse_Statement();
    eventHandler.endNonterminal("IfStatement", e0);
  }

  function try_IfStatement()
  {
    shiftT(152);                    // 'if'
    lookahead1W(22);                // S^WS | '(' | '(:'
    shiftT(34);                     // '('
    lookahead1W(267);               // Wildcard | EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral |
    try_Expr();
    shiftT(37);                     // ')'
    lookahead1W(77);                // S^WS | '(:' | 'then'
    shiftT(245);                    // 'then'
    lookahead1W(270);               // Wildcard | EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral |
    try_Statement();
    lookahead1W(48);                // S^WS | '(:' | 'else'
    shiftT(122);                    // 'else'
    lookahead1W(270);               // Wildcard | EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral |
    try_Statement();
  }

  function parse_SwitchStatement()
  {
    eventHandler.startNonterminal("SwitchStatement", e0);
    shift(243);                     // 'switch'
    lookahead1W(22);                // S^WS | '(' | '(:'
    shift(34);                      // '('
    lookahead1W(267);               // Wildcard | EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral |
    whitespace();
    parse_Expr();
    shift(37);                      // ')'
    for (;;)
    {
      lookahead1W(35);              // S^WS | '(:' | 'case'
      whitespace();
      parse_SwitchCaseStatement();
      lookahead1W(113);             // S^WS | '(:' | 'case' | 'default'
      if (l1 != 88)                 // 'case'
      {
        break;
      }
    }
    shift(109);                     // 'default'
    lookahead1W(70);                // S^WS | '(:' | 'return'
    shift(220);                     // 'return'
    lookahead1W(270);               // Wildcard | EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral |
    whitespace();
    parse_Statement();
    eventHandler.endNonterminal("SwitchStatement", e0);
  }

  function try_SwitchStatement()
  {
    shiftT(243);                    // 'switch'
    lookahead1W(22);                // S^WS | '(' | '(:'
    shiftT(34);                     // '('
    lookahead1W(267);               // Wildcard | EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral |
    try_Expr();
    shiftT(37);                     // ')'
    for (;;)
    {
      lookahead1W(35);              // S^WS | '(:' | 'case'
      try_SwitchCaseStatement();
      lookahead1W(113);             // S^WS | '(:' | 'case' | 'default'
      if (l1 != 88)                 // 'case'
      {
        break;
      }
    }
    shiftT(109);                    // 'default'
    lookahead1W(70);                // S^WS | '(:' | 'return'
    shiftT(220);                    // 'return'
    lookahead1W(270);               // Wildcard | EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral |
    try_Statement();
  }

  function parse_SwitchCaseStatement()
  {
    eventHandler.startNonterminal("SwitchCaseStatement", e0);
    for (;;)
    {
      shift(88);                    // 'case'
      lookahead1W(267);             // Wildcard | EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral |
      whitespace();
      parse_SwitchCaseOperand();
      if (l1 != 88)                 // 'case'
      {
        break;
      }
    }
    shift(220);                     // 'return'
    lookahead1W(270);               // Wildcard | EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral |
    whitespace();
    parse_Statement();
    eventHandler.endNonterminal("SwitchCaseStatement", e0);
  }

  function try_SwitchCaseStatement()
  {
    for (;;)
    {
      shiftT(88);                   // 'case'
      lookahead1W(267);             // Wildcard | EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral |
      try_SwitchCaseOperand();
      if (l1 != 88)                 // 'case'
      {
        break;
      }
    }
    shiftT(220);                    // 'return'
    lookahead1W(270);               // Wildcard | EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral |
    try_Statement();
  }

  function parse_TryCatchStatement()
  {
    eventHandler.startNonterminal("TryCatchStatement", e0);
    shift(250);                     // 'try'
    lookahead1W(87);                // S^WS | '(:' | '{'
    whitespace();
    parse_BlockStatement();
    for (;;)
    {
      lookahead1W(36);              // S^WS | '(:' | 'catch'
      shift(91);                    // 'catch'
      lookahead1W(257);             // Wildcard | EQName^Token | S^WS | '(:' | 'after' | 'allowing' | 'ancestor' |
      whitespace();
      parse_CatchErrorList();
      whitespace();
      parse_BlockStatement();
      lookahead1W(278);             // Wildcard | EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral |
      switch (l1)
      {
      case 91:                      // 'catch'
        lookahead2W(279);           // Wildcard | EQName^Token | S^WS | EOF | '!' | '!=' | '#' | '(' | '(:' | '*' |
        break;
      default:
        lk = l1;
      }
      if (lk == 38491               // 'catch' 'and'
       || lk == 45659               // 'catch' 'cast'
       || lk == 46171               // 'catch' 'castable'
       || lk == 60507               // 'catch' 'div'
       || lk == 65627               // 'catch' 'eq'
       || lk == 67163               // 'catch' 'except'
       || lk == 74843               // 'catch' 'ge'
       || lk == 76891               // 'catch' 'gt'
       || lk == 77403               // 'catch' 'idiv'
       || lk == 82011               // 'catch' 'instance'
       || lk == 83035               // 'catch' 'intersect'
       || lk == 84059               // 'catch' 'is'
       || lk == 88155               // 'catch' 'le'
       || lk == 91227               // 'catch' 'lt'
       || lk == 92251               // 'catch' 'mod'
       || lk == 95323               // 'catch' 'ne'
       || lk == 102491              // 'catch' 'or'
       || lk == 127067              // 'catch' 'to'
       || lk == 127579              // 'catch' 'treat'
       || lk == 130139)             // 'catch' 'union'
      {
        lk = memoized(8, e0);
        if (lk == 0)
        {
          var b0A = b0; var e0A = e0; var l1A = l1;
          var b1A = b1; var e1A = e1; var l2A = l2;
          var b2A = b2; var e2A = e2;
          try
          {
            lookahead1W(36);        // S^WS | '(:' | 'catch'
            shiftT(91);             // 'catch'
            lookahead1W(257);       // Wildcard | EQName^Token | S^WS | '(:' | 'after' | 'allowing' | 'ancestor' |
            try_CatchErrorList();
            try_BlockStatement();
            lk = -1;
          }
          catch (p1A)
          {
            lk = -2;
          }
          b0 = b0A; e0 = e0A; l1 = l1A; if (l1 == 0) {end = e0A;} else {
          b1 = b1A; e1 = e1A; l2 = l2A; if (l2 == 0) {end = e1A;} else {
          b2 = b2A; e2 = e2A; end = e2A; }}
          memoize(8, e0, lk);
        }
      }
      if (lk != -1
       && lk != 2651                // 'catch' Wildcard
       && lk != 3163                // 'catch' EQName^Token
       && lk != 35931               // 'catch' 'after'
       && lk != 36955               // 'catch' 'allowing'
       && lk != 37467               // 'catch' 'ancestor'
       && lk != 37979               // 'catch' 'ancestor-or-self'
       && lk != 39515               // 'catch' 'append'
       && lk != 40027               // 'catch' 'array'
       && lk != 40539               // 'catch' 'as'
       && lk != 41051               // 'catch' 'ascending'
       && lk != 41563               // 'catch' 'at'
       && lk != 42075               // 'catch' 'attribute'
       && lk != 42587               // 'catch' 'base-uri'
       && lk != 43099               // 'catch' 'before'
       && lk != 43611               // 'catch' 'boundary-space'
       && lk != 44123               // 'catch' 'break'
       && lk != 45147               // 'catch' 'case'
       && lk != 46683               // 'catch' 'catch'
       && lk != 47707               // 'catch' 'child'
       && lk != 48219               // 'catch' 'collation'
       && lk != 49243               // 'catch' 'comment'
       && lk != 49755               // 'catch' 'constraint'
       && lk != 50267               // 'catch' 'construction'
       && lk != 51803               // 'catch' 'context'
       && lk != 52315               // 'catch' 'continue'
       && lk != 52827               // 'catch' 'copy'
       && lk != 53339               // 'catch' 'copy-namespaces'
       && lk != 53851               // 'catch' 'count'
       && lk != 54363               // 'catch' 'decimal-format'
       && lk != 55387               // 'catch' 'declare'
       && lk != 55899               // 'catch' 'default'
       && lk != 56411               // 'catch' 'delete'
       && lk != 56923               // 'catch' 'descendant'
       && lk != 57435               // 'catch' 'descendant-or-self'
       && lk != 57947               // 'catch' 'descending'
       && lk != 61019               // 'catch' 'document'
       && lk != 61531               // 'catch' 'document-node'
       && lk != 62043               // 'catch' 'element'
       && lk != 62555               // 'catch' 'else'
       && lk != 63067               // 'catch' 'empty'
       && lk != 63579               // 'catch' 'empty-sequence'
       && lk != 64091               // 'catch' 'encoding'
       && lk != 64603               // 'catch' 'end'
       && lk != 66139               // 'catch' 'every'
       && lk != 67675               // 'catch' 'exit'
       && lk != 68187               // 'catch' 'external'
       && lk != 68699               // 'catch' 'first'
       && lk != 69211               // 'catch' 'following'
       && lk != 69723               // 'catch' 'following-sibling'
       && lk != 70235               // 'catch' 'for'
       && lk != 72283               // 'catch' 'ft-option'
       && lk != 74331               // 'catch' 'function'
       && lk != 75867               // 'catch' 'group'
       && lk != 77915               // 'catch' 'if'
       && lk != 78427               // 'catch' 'import'
       && lk != 78939               // 'catch' 'in'
       && lk != 79451               // 'catch' 'index'
       && lk != 81499               // 'catch' 'insert'
       && lk != 82523               // 'catch' 'integrity'
       && lk != 83547               // 'catch' 'into'
       && lk != 84571               // 'catch' 'item'
       && lk != 85083               // 'catch' 'json'
       && lk != 85595               // 'catch' 'json-item'
       && lk != 87131               // 'catch' 'last'
       && lk != 87643               // 'catch' 'lax'
       && lk != 89179               // 'catch' 'let'
       && lk != 90203               // 'catch' 'loop'
       && lk != 92763               // 'catch' 'modify'
       && lk != 93275               // 'catch' 'module'
       && lk != 94299               // 'catch' 'namespace'
       && lk != 94811               // 'catch' 'namespace-node'
       && lk != 97883               // 'catch' 'node'
       && lk != 98395               // 'catch' 'nodes'
       && lk != 99419               // 'catch' 'object'
       && lk != 101467              // 'catch' 'only'
       && lk != 101979              // 'catch' 'option'
       && lk != 103003              // 'catch' 'order'
       && lk != 103515              // 'catch' 'ordered'
       && lk != 104027              // 'catch' 'ordering'
       && lk != 105563              // 'catch' 'parent'
       && lk != 108635              // 'catch' 'preceding'
       && lk != 109147              // 'catch' 'preceding-sibling'
       && lk != 110683              // 'catch' 'processing-instruction'
       && lk != 111707              // 'catch' 'rename'
       && lk != 112219              // 'catch' 'replace'
       && lk != 112731              // 'catch' 'return'
       && lk != 113243              // 'catch' 'returning'
       && lk != 113755              // 'catch' 'revalidation'
       && lk != 114779              // 'catch' 'satisfies'
       && lk != 115291              // 'catch' 'schema'
       && lk != 115803              // 'catch' 'schema-attribute'
       && lk != 116315              // 'catch' 'schema-element'
       && lk != 116827              // 'catch' 'score'
       && lk != 117339              // 'catch' 'self'
       && lk != 119899              // 'catch' 'sliding'
       && lk != 120411              // 'catch' 'some'
       && lk != 120923              // 'catch' 'stable'
       && lk != 121435              // 'catch' 'start'
       && lk != 122971              // 'catch' 'strict'
       && lk != 123995              // 'catch' 'structured-item'
       && lk != 124507              // 'catch' 'switch'
       && lk != 125019              // 'catch' 'text'
       && lk != 128091              // 'catch' 'try'
       && lk != 128603              // 'catch' 'tumbling'
       && lk != 129115              // 'catch' 'type'
       && lk != 129627              // 'catch' 'typeswitch'
       && lk != 131163              // 'catch' 'unordered'
       && lk != 131675              // 'catch' 'updating'
       && lk != 133211              // 'catch' 'validate'
       && lk != 133723              // 'catch' 'value'
       && lk != 134235              // 'catch' 'variable'
       && lk != 134747              // 'catch' 'version'
       && lk != 136283              // 'catch' 'where'
       && lk != 136795              // 'catch' 'while'
       && lk != 138331              // 'catch' 'with'
       && lk != 140379)             // 'catch' 'xquery'
      {
        break;
      }
    }
    eventHandler.endNonterminal("TryCatchStatement", e0);
  }

  function try_TryCatchStatement()
  {
    shiftT(250);                    // 'try'
    lookahead1W(87);                // S^WS | '(:' | '{'
    try_BlockStatement();
    lookahead1W(36);                // S^WS | '(:' | 'catch'
    shiftT(91);                     // 'catch'
    lookahead1W(257);               // Wildcard | EQName^Token | S^WS | '(:' | 'after' | 'allowing' | 'ancestor' |
    try_CatchErrorList();
    try_BlockStatement();
    for (;;)
    {
      lookahead1W(278);             // Wildcard | EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral |
      switch (l1)
      {
      case 91:                      // 'catch'
        lookahead2W(279);           // Wildcard | EQName^Token | S^WS | EOF | '!' | '!=' | '#' | '(' | '(:' | '*' |
        break;
      default:
        lk = l1;
      }
      if (lk == 38491               // 'catch' 'and'
       || lk == 45659               // 'catch' 'cast'
       || lk == 46171               // 'catch' 'castable'
       || lk == 60507               // 'catch' 'div'
       || lk == 65627               // 'catch' 'eq'
       || lk == 67163               // 'catch' 'except'
       || lk == 74843               // 'catch' 'ge'
       || lk == 76891               // 'catch' 'gt'
       || lk == 77403               // 'catch' 'idiv'
       || lk == 82011               // 'catch' 'instance'
       || lk == 83035               // 'catch' 'intersect'
       || lk == 84059               // 'catch' 'is'
       || lk == 88155               // 'catch' 'le'
       || lk == 91227               // 'catch' 'lt'
       || lk == 92251               // 'catch' 'mod'
       || lk == 95323               // 'catch' 'ne'
       || lk == 102491              // 'catch' 'or'
       || lk == 127067              // 'catch' 'to'
       || lk == 127579              // 'catch' 'treat'
       || lk == 130139)             // 'catch' 'union'
      {
        lk = memoized(8, e0);
        if (lk == 0)
        {
          var b0A = b0; var e0A = e0; var l1A = l1;
          var b1A = b1; var e1A = e1; var l2A = l2;
          var b2A = b2; var e2A = e2;
          try
          {
            lookahead1W(36);        // S^WS | '(:' | 'catch'
            shiftT(91);             // 'catch'
            lookahead1W(257);       // Wildcard | EQName^Token | S^WS | '(:' | 'after' | 'allowing' | 'ancestor' |
            try_CatchErrorList();
            try_BlockStatement();
            memoize(8, e0A, -1);
            continue;
          }
          catch (p1A)
          {
            b0 = b0A; e0 = e0A; l1 = l1A; if (l1 == 0) {end = e0A;} else {
            b1 = b1A; e1 = e1A; l2 = l2A; if (l2 == 0) {end = e1A;} else {
            b2 = b2A; e2 = e2A; end = e2A; }}
            memoize(8, e0A, -2);
            break;
          }
        }
      }
      if (lk != -1
       && lk != 2651                // 'catch' Wildcard
       && lk != 3163                // 'catch' EQName^Token
       && lk != 35931               // 'catch' 'after'
       && lk != 36955               // 'catch' 'allowing'
       && lk != 37467               // 'catch' 'ancestor'
       && lk != 37979               // 'catch' 'ancestor-or-self'
       && lk != 39515               // 'catch' 'append'
       && lk != 40027               // 'catch' 'array'
       && lk != 40539               // 'catch' 'as'
       && lk != 41051               // 'catch' 'ascending'
       && lk != 41563               // 'catch' 'at'
       && lk != 42075               // 'catch' 'attribute'
       && lk != 42587               // 'catch' 'base-uri'
       && lk != 43099               // 'catch' 'before'
       && lk != 43611               // 'catch' 'boundary-space'
       && lk != 44123               // 'catch' 'break'
       && lk != 45147               // 'catch' 'case'
       && lk != 46683               // 'catch' 'catch'
       && lk != 47707               // 'catch' 'child'
       && lk != 48219               // 'catch' 'collation'
       && lk != 49243               // 'catch' 'comment'
       && lk != 49755               // 'catch' 'constraint'
       && lk != 50267               // 'catch' 'construction'
       && lk != 51803               // 'catch' 'context'
       && lk != 52315               // 'catch' 'continue'
       && lk != 52827               // 'catch' 'copy'
       && lk != 53339               // 'catch' 'copy-namespaces'
       && lk != 53851               // 'catch' 'count'
       && lk != 54363               // 'catch' 'decimal-format'
       && lk != 55387               // 'catch' 'declare'
       && lk != 55899               // 'catch' 'default'
       && lk != 56411               // 'catch' 'delete'
       && lk != 56923               // 'catch' 'descendant'
       && lk != 57435               // 'catch' 'descendant-or-self'
       && lk != 57947               // 'catch' 'descending'
       && lk != 61019               // 'catch' 'document'
       && lk != 61531               // 'catch' 'document-node'
       && lk != 62043               // 'catch' 'element'
       && lk != 62555               // 'catch' 'else'
       && lk != 63067               // 'catch' 'empty'
       && lk != 63579               // 'catch' 'empty-sequence'
       && lk != 64091               // 'catch' 'encoding'
       && lk != 64603               // 'catch' 'end'
       && lk != 66139               // 'catch' 'every'
       && lk != 67675               // 'catch' 'exit'
       && lk != 68187               // 'catch' 'external'
       && lk != 68699               // 'catch' 'first'
       && lk != 69211               // 'catch' 'following'
       && lk != 69723               // 'catch' 'following-sibling'
       && lk != 70235               // 'catch' 'for'
       && lk != 72283               // 'catch' 'ft-option'
       && lk != 74331               // 'catch' 'function'
       && lk != 75867               // 'catch' 'group'
       && lk != 77915               // 'catch' 'if'
       && lk != 78427               // 'catch' 'import'
       && lk != 78939               // 'catch' 'in'
       && lk != 79451               // 'catch' 'index'
       && lk != 81499               // 'catch' 'insert'
       && lk != 82523               // 'catch' 'integrity'
       && lk != 83547               // 'catch' 'into'
       && lk != 84571               // 'catch' 'item'
       && lk != 85083               // 'catch' 'json'
       && lk != 85595               // 'catch' 'json-item'
       && lk != 87131               // 'catch' 'last'
       && lk != 87643               // 'catch' 'lax'
       && lk != 89179               // 'catch' 'let'
       && lk != 90203               // 'catch' 'loop'
       && lk != 92763               // 'catch' 'modify'
       && lk != 93275               // 'catch' 'module'
       && lk != 94299               // 'catch' 'namespace'
       && lk != 94811               // 'catch' 'namespace-node'
       && lk != 97883               // 'catch' 'node'
       && lk != 98395               // 'catch' 'nodes'
       && lk != 99419               // 'catch' 'object'
       && lk != 101467              // 'catch' 'only'
       && lk != 101979              // 'catch' 'option'
       && lk != 103003              // 'catch' 'order'
       && lk != 103515              // 'catch' 'ordered'
       && lk != 104027              // 'catch' 'ordering'
       && lk != 105563              // 'catch' 'parent'
       && lk != 108635              // 'catch' 'preceding'
       && lk != 109147              // 'catch' 'preceding-sibling'
       && lk != 110683              // 'catch' 'processing-instruction'
       && lk != 111707              // 'catch' 'rename'
       && lk != 112219              // 'catch' 'replace'
       && lk != 112731              // 'catch' 'return'
       && lk != 113243              // 'catch' 'returning'
       && lk != 113755              // 'catch' 'revalidation'
       && lk != 114779              // 'catch' 'satisfies'
       && lk != 115291              // 'catch' 'schema'
       && lk != 115803              // 'catch' 'schema-attribute'
       && lk != 116315              // 'catch' 'schema-element'
       && lk != 116827              // 'catch' 'score'
       && lk != 117339              // 'catch' 'self'
       && lk != 119899              // 'catch' 'sliding'
       && lk != 120411              // 'catch' 'some'
       && lk != 120923              // 'catch' 'stable'
       && lk != 121435              // 'catch' 'start'
       && lk != 122971              // 'catch' 'strict'
       && lk != 123995              // 'catch' 'structured-item'
       && lk != 124507              // 'catch' 'switch'
       && lk != 125019              // 'catch' 'text'
       && lk != 128091              // 'catch' 'try'
       && lk != 128603              // 'catch' 'tumbling'
       && lk != 129115              // 'catch' 'type'
       && lk != 129627              // 'catch' 'typeswitch'
       && lk != 131163              // 'catch' 'unordered'
       && lk != 131675              // 'catch' 'updating'
       && lk != 133211              // 'catch' 'validate'
       && lk != 133723              // 'catch' 'value'
       && lk != 134235              // 'catch' 'variable'
       && lk != 134747              // 'catch' 'version'
       && lk != 136283              // 'catch' 'where'
       && lk != 136795              // 'catch' 'while'
       && lk != 138331              // 'catch' 'with'
       && lk != 140379)             // 'catch' 'xquery'
      {
        break;
      }
      lookahead1W(36);              // S^WS | '(:' | 'catch'
      shiftT(91);                   // 'catch'
      lookahead1W(257);             // Wildcard | EQName^Token | S^WS | '(:' | 'after' | 'allowing' | 'ancestor' |
      try_CatchErrorList();
      try_BlockStatement();
    }
  }

  function parse_TypeswitchStatement()
  {
    eventHandler.startNonterminal("TypeswitchStatement", e0);
    shift(253);                     // 'typeswitch'
    lookahead1W(22);                // S^WS | '(' | '(:'
    shift(34);                      // '('
    lookahead1W(267);               // Wildcard | EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral |
    whitespace();
    parse_Expr();
    shift(37);                      // ')'
    for (;;)
    {
      lookahead1W(35);              // S^WS | '(:' | 'case'
      whitespace();
      parse_CaseStatement();
      lookahead1W(113);             // S^WS | '(:' | 'case' | 'default'
      if (l1 != 88)                 // 'case'
      {
        break;
      }
    }
    shift(109);                     // 'default'
    lookahead1W(95);                // S^WS | '$' | '(:' | 'return'
    if (l1 == 31)                   // '$'
    {
      shift(31);                    // '$'
      lookahead1W(255);             // EQName^Token | S^WS | '(:' | 'after' | 'allowing' | 'ancestor' |
      whitespace();
      parse_VarName();
    }
    lookahead1W(70);                // S^WS | '(:' | 'return'
    shift(220);                     // 'return'
    lookahead1W(270);               // Wildcard | EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral |
    whitespace();
    parse_Statement();
    eventHandler.endNonterminal("TypeswitchStatement", e0);
  }

  function try_TypeswitchStatement()
  {
    shiftT(253);                    // 'typeswitch'
    lookahead1W(22);                // S^WS | '(' | '(:'
    shiftT(34);                     // '('
    lookahead1W(267);               // Wildcard | EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral |
    try_Expr();
    shiftT(37);                     // ')'
    for (;;)
    {
      lookahead1W(35);              // S^WS | '(:' | 'case'
      try_CaseStatement();
      lookahead1W(113);             // S^WS | '(:' | 'case' | 'default'
      if (l1 != 88)                 // 'case'
      {
        break;
      }
    }
    shiftT(109);                    // 'default'
    lookahead1W(95);                // S^WS | '$' | '(:' | 'return'
    if (l1 == 31)                   // '$'
    {
      shiftT(31);                   // '$'
      lookahead1W(255);             // EQName^Token | S^WS | '(:' | 'after' | 'allowing' | 'ancestor' |
      try_VarName();
    }
    lookahead1W(70);                // S^WS | '(:' | 'return'
    shiftT(220);                    // 'return'
    lookahead1W(270);               // Wildcard | EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral |
    try_Statement();
  }

  function parse_CaseStatement()
  {
    eventHandler.startNonterminal("CaseStatement", e0);
    shift(88);                      // 'case'
    lookahead1W(262);               // EQName^Token | S^WS | '$' | '%' | '(' | '(:' | 'after' | 'allowing' |
    if (l1 == 31)                   // '$'
    {
      shift(31);                    // '$'
      lookahead1W(255);             // EQName^Token | S^WS | '(:' | 'after' | 'allowing' | 'ancestor' |
      whitespace();
      parse_VarName();
      lookahead1W(30);              // S^WS | '(:' | 'as'
      shift(79);                    // 'as'
    }
    lookahead1W(260);               // EQName^Token | S^WS | '%' | '(' | '(:' | 'after' | 'allowing' | 'ancestor' |
    whitespace();
    parse_SequenceType();
    lookahead1W(70);                // S^WS | '(:' | 'return'
    shift(220);                     // 'return'
    lookahead1W(270);               // Wildcard | EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral |
    whitespace();
    parse_Statement();
    eventHandler.endNonterminal("CaseStatement", e0);
  }

  function try_CaseStatement()
  {
    shiftT(88);                     // 'case'
    lookahead1W(262);               // EQName^Token | S^WS | '$' | '%' | '(' | '(:' | 'after' | 'allowing' |
    if (l1 == 31)                   // '$'
    {
      shiftT(31);                   // '$'
      lookahead1W(255);             // EQName^Token | S^WS | '(:' | 'after' | 'allowing' | 'ancestor' |
      try_VarName();
      lookahead1W(30);              // S^WS | '(:' | 'as'
      shiftT(79);                   // 'as'
    }
    lookahead1W(260);               // EQName^Token | S^WS | '%' | '(' | '(:' | 'after' | 'allowing' | 'ancestor' |
    try_SequenceType();
    lookahead1W(70);                // S^WS | '(:' | 'return'
    shiftT(220);                    // 'return'
    lookahead1W(270);               // Wildcard | EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral |
    try_Statement();
  }

  function parse_VarDeclStatement()
  {
    eventHandler.startNonterminal("VarDeclStatement", e0);
    for (;;)
    {
      lookahead1W(98);              // S^WS | '%' | '(:' | 'variable'
      if (l1 != 32)                 // '%'
      {
        break;
      }
      whitespace();
      parse_Annotation();
    }
    shift(262);                     // 'variable'
    lookahead1W(21);                // S^WS | '$' | '(:'
    shift(31);                      // '$'
    lookahead1W(255);               // EQName^Token | S^WS | '(:' | 'after' | 'allowing' | 'ancestor' |
    whitespace();
    parse_VarName();
    lookahead1W(157);               // S^WS | '(:' | ',' | ':=' | ';' | 'as'
    if (l1 == 79)                   // 'as'
    {
      whitespace();
      parse_TypeDeclaration();
    }
    lookahead1W(145);               // S^WS | '(:' | ',' | ':=' | ';'
    if (l1 == 52)                   // ':='
    {
      shift(52);                    // ':='
      lookahead1W(267);             // Wildcard | EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral |
      whitespace();
      parse_ExprSingle();
    }
    for (;;)
    {
      if (l1 != 41)                 // ','
      {
        break;
      }
      shift(41);                    // ','
      lookahead1W(21);              // S^WS | '$' | '(:'
      shift(31);                    // '$'
      lookahead1W(255);             // EQName^Token | S^WS | '(:' | 'after' | 'allowing' | 'ancestor' |
      whitespace();
      parse_VarName();
      lookahead1W(157);             // S^WS | '(:' | ',' | ':=' | ';' | 'as'
      if (l1 == 79)                 // 'as'
      {
        whitespace();
        parse_TypeDeclaration();
      }
      lookahead1W(145);             // S^WS | '(:' | ',' | ':=' | ';'
      if (l1 == 52)                 // ':='
      {
        shift(52);                  // ':='
        lookahead1W(267);           // Wildcard | EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral |
        whitespace();
        parse_ExprSingle();
      }
    }
    shift(53);                      // ';'
    eventHandler.endNonterminal("VarDeclStatement", e0);
  }

  function try_VarDeclStatement()
  {
    for (;;)
    {
      lookahead1W(98);              // S^WS | '%' | '(:' | 'variable'
      if (l1 != 32)                 // '%'
      {
        break;
      }
      try_Annotation();
    }
    shiftT(262);                    // 'variable'
    lookahead1W(21);                // S^WS | '$' | '(:'
    shiftT(31);                     // '$'
    lookahead1W(255);               // EQName^Token | S^WS | '(:' | 'after' | 'allowing' | 'ancestor' |
    try_VarName();
    lookahead1W(157);               // S^WS | '(:' | ',' | ':=' | ';' | 'as'
    if (l1 == 79)                   // 'as'
    {
      try_TypeDeclaration();
    }
    lookahead1W(145);               // S^WS | '(:' | ',' | ':=' | ';'
    if (l1 == 52)                   // ':='
    {
      shiftT(52);                   // ':='
      lookahead1W(267);             // Wildcard | EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral |
      try_ExprSingle();
    }
    for (;;)
    {
      if (l1 != 41)                 // ','
      {
        break;
      }
      shiftT(41);                   // ','
      lookahead1W(21);              // S^WS | '$' | '(:'
      shiftT(31);                   // '$'
      lookahead1W(255);             // EQName^Token | S^WS | '(:' | 'after' | 'allowing' | 'ancestor' |
      try_VarName();
      lookahead1W(157);             // S^WS | '(:' | ',' | ':=' | ';' | 'as'
      if (l1 == 79)                 // 'as'
      {
        try_TypeDeclaration();
      }
      lookahead1W(145);             // S^WS | '(:' | ',' | ':=' | ';'
      if (l1 == 52)                 // ':='
      {
        shiftT(52);                 // ':='
        lookahead1W(267);           // Wildcard | EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral |
        try_ExprSingle();
      }
    }
    shiftT(53);                     // ';'
  }

  function parse_WhileStatement()
  {
    eventHandler.startNonterminal("WhileStatement", e0);
    shift(267);                     // 'while'
    lookahead1W(22);                // S^WS | '(' | '(:'
    shift(34);                      // '('
    lookahead1W(267);               // Wildcard | EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral |
    whitespace();
    parse_Expr();
    shift(37);                      // ')'
    lookahead1W(270);               // Wildcard | EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral |
    whitespace();
    parse_Statement();
    eventHandler.endNonterminal("WhileStatement", e0);
  }

  function try_WhileStatement()
  {
    shiftT(267);                    // 'while'
    lookahead1W(22);                // S^WS | '(' | '(:'
    shiftT(34);                     // '('
    lookahead1W(267);               // Wildcard | EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral |
    try_Expr();
    shiftT(37);                     // ')'
    lookahead1W(270);               // Wildcard | EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral |
    try_Statement();
  }

  function parse_VoidStatement()
  {
    eventHandler.startNonterminal("VoidStatement", e0);
    shift(53);                      // ';'
    eventHandler.endNonterminal("VoidStatement", e0);
  }

  function try_VoidStatement()
  {
    shiftT(53);                     // ';'
  }

  function parse_ExprSingle()
  {
    eventHandler.startNonterminal("ExprSingle", e0);
    switch (l1)
    {
    case 137:                       // 'for'
      lookahead2W(236);             // S^WS | EOF | '!' | '!=' | '#' | '$' | '(' | '(:' | ')' | '*' | '+' | ',' | '-' |
      break;
    case 174:                       // 'let'
      lookahead2W(233);             // S^WS | EOF | '!' | '!=' | '#' | '$' | '(' | '(:' | ')' | '*' | '+' | ',' | '-' |
      break;
    case 250:                       // 'try'
      lookahead2W(232);             // S^WS | EOF | '!' | '!=' | '#' | '(' | '(:' | ')' | '*' | '+' | ',' | '-' | '/' |
      break;
    case 152:                       // 'if'
    case 243:                       // 'switch'
    case 253:                       // 'typeswitch'
      lookahead2W(229);             // S^WS | EOF | '!' | '!=' | '#' | '(' | '(:' | ')' | '*' | '+' | ',' | '-' | '/' |
      break;
    default:
      lk = l1;
    }
    switch (lk)
    {
    case 16009:                     // 'for' '$'
    case 16046:                     // 'let' '$'
    case 116910:                    // 'let' 'score'
    case 119945:                    // 'for' 'sliding'
    case 128649:                    // 'for' 'tumbling'
      parse_FLWORExpr();
      break;
    case 17560:                     // 'if' '('
      parse_IfExpr();
      break;
    case 17651:                     // 'switch' '('
      parse_SwitchExpr();
      break;
    case 141562:                    // 'try' '{'
      parse_TryCatchExpr();
      break;
    case 17661:                     // 'typeswitch' '('
      parse_TypeswitchExpr();
      break;
    default:
      parse_ExprSimple();
    }
    eventHandler.endNonterminal("ExprSingle", e0);
  }

  function try_ExprSingle()
  {
    switch (l1)
    {
    case 137:                       // 'for'
      lookahead2W(236);             // S^WS | EOF | '!' | '!=' | '#' | '$' | '(' | '(:' | ')' | '*' | '+' | ',' | '-' |
      break;
    case 174:                       // 'let'
      lookahead2W(233);             // S^WS | EOF | '!' | '!=' | '#' | '$' | '(' | '(:' | ')' | '*' | '+' | ',' | '-' |
      break;
    case 250:                       // 'try'
      lookahead2W(232);             // S^WS | EOF | '!' | '!=' | '#' | '(' | '(:' | ')' | '*' | '+' | ',' | '-' | '/' |
      break;
    case 152:                       // 'if'
    case 243:                       // 'switch'
    case 253:                       // 'typeswitch'
      lookahead2W(229);             // S^WS | EOF | '!' | '!=' | '#' | '(' | '(:' | ')' | '*' | '+' | ',' | '-' | '/' |
      break;
    default:
      lk = l1;
    }
    switch (lk)
    {
    case 16009:                     // 'for' '$'
    case 16046:                     // 'let' '$'
    case 116910:                    // 'let' 'score'
    case 119945:                    // 'for' 'sliding'
    case 128649:                    // 'for' 'tumbling'
      try_FLWORExpr();
      break;
    case 17560:                     // 'if' '('
      try_IfExpr();
      break;
    case 17651:                     // 'switch' '('
      try_SwitchExpr();
      break;
    case 141562:                    // 'try' '{'
      try_TryCatchExpr();
      break;
    case 17661:                     // 'typeswitch' '('
      try_TypeswitchExpr();
      break;
    default:
      try_ExprSimple();
    }
  }

  function parse_ExprSimple()
  {
    eventHandler.startNonterminal("ExprSimple", e0);
    switch (l1)
    {
    case 77:                        // 'append'
      lookahead2W(231);             // S^WS | EOF | '!' | '!=' | '#' | '(' | '(:' | ')' | '*' | '+' | ',' | '-' | '/' |
      break;
    case 218:                       // 'rename'
      lookahead2W(234);             // S^WS | EOF | '!' | '!=' | '#' | '(' | '(:' | ')' | '*' | '+' | ',' | '-' | '/' |
      break;
    case 219:                       // 'replace'
      lookahead2W(235);             // S^WS | EOF | '!' | '!=' | '#' | '(' | '(:' | ')' | '*' | '+' | ',' | '-' | '/' |
      break;
    case 110:                       // 'delete'
    case 159:                       // 'insert'
      lookahead2W(237);             // S^WS | EOF | '!' | '!=' | '#' | '(' | '(:' | ')' | '*' | '+' | ',' | '-' | '/' |
      break;
    case 103:                       // 'copy'
    case 129:                       // 'every'
    case 235:                       // 'some'
      lookahead2W(230);             // S^WS | EOF | '!' | '!=' | '#' | '$' | '(' | '(:' | ')' | '*' | '+' | ',' | '-' |
      break;
    default:
      lk = l1;
    }
    if (lk == 133851)               // 'replace' 'value'
    {
      lk = memoized(9, e0);
      if (lk == 0)
      {
        var b0A = b0; var e0A = e0; var l1A = l1;
        var b1A = b1; var e1A = e1; var l2A = l2;
        var b2A = b2; var e2A = e2;
        try
        {
          try_ReplaceExpr();
          lk = -6;
        }
        catch (p6A)
        {
          lk = -11;
        }
        b0 = b0A; e0 = e0A; l1 = l1A; if (l1 == 0) {end = e0A;} else {
        b1 = b1A; e1 = e1A; l2 = l2A; if (l2 == 0) {end = e1A;} else {
        b2 = b2A; e2 = e2A; end = e2A; }}
        memoize(9, e0, lk);
      }
    }
    switch (lk)
    {
    case 16001:                     // 'every' '$'
    case 16107:                     // 'some' '$'
      parse_QuantifiedExpr();
      break;
    case 97951:                     // 'insert' 'node'
    case 98463:                     // 'insert' 'nodes'
      parse_InsertExpr();
      break;
    case 97902:                     // 'delete' 'node'
    case 98414:                     // 'delete' 'nodes'
      parse_DeleteExpr();
      break;
    case 98010:                     // 'rename' 'node'
      parse_RenameExpr();
      break;
    case -6:
    case 98011:                     // 'replace' 'node'
      parse_ReplaceExpr();
      break;
    case 15975:                     // 'copy' '$'
      parse_TransformExpr();
      break;
    case 85102:                     // 'delete' 'json'
      parse_JSONDeleteExpr();
      break;
    case 85151:                     // 'insert' 'json'
      parse_JSONInsertExpr();
      break;
    case 85210:                     // 'rename' 'json'
      parse_JSONRenameExpr();
      break;
    case -11:
      parse_JSONReplaceExpr();
      break;
    case 85069:                     // 'append' 'json'
      parse_JSONAppendExpr();
      break;
    default:
      parse_OrExpr();
    }
    eventHandler.endNonterminal("ExprSimple", e0);
  }

  function try_ExprSimple()
  {
    switch (l1)
    {
    case 77:                        // 'append'
      lookahead2W(231);             // S^WS | EOF | '!' | '!=' | '#' | '(' | '(:' | ')' | '*' | '+' | ',' | '-' | '/' |
      break;
    case 218:                       // 'rename'
      lookahead2W(234);             // S^WS | EOF | '!' | '!=' | '#' | '(' | '(:' | ')' | '*' | '+' | ',' | '-' | '/' |
      break;
    case 219:                       // 'replace'
      lookahead2W(235);             // S^WS | EOF | '!' | '!=' | '#' | '(' | '(:' | ')' | '*' | '+' | ',' | '-' | '/' |
      break;
    case 110:                       // 'delete'
    case 159:                       // 'insert'
      lookahead2W(237);             // S^WS | EOF | '!' | '!=' | '#' | '(' | '(:' | ')' | '*' | '+' | ',' | '-' | '/' |
      break;
    case 103:                       // 'copy'
    case 129:                       // 'every'
    case 235:                       // 'some'
      lookahead2W(230);             // S^WS | EOF | '!' | '!=' | '#' | '$' | '(' | '(:' | ')' | '*' | '+' | ',' | '-' |
      break;
    default:
      lk = l1;
    }
    if (lk == 133851)               // 'replace' 'value'
    {
      lk = memoized(9, e0);
      if (lk == 0)
      {
        var b0A = b0; var e0A = e0; var l1A = l1;
        var b1A = b1; var e1A = e1; var l2A = l2;
        var b2A = b2; var e2A = e2;
        try
        {
          try_ReplaceExpr();
          memoize(9, e0A, -6);
          lk = -13;
        }
        catch (p6A)
        {
          lk = -11;
          b0 = b0A; e0 = e0A; l1 = l1A; if (l1 == 0) {end = e0A;} else {
          b1 = b1A; e1 = e1A; l2 = l2A; if (l2 == 0) {end = e1A;} else {
          b2 = b2A; e2 = e2A; end = e2A; }}
          memoize(9, e0A, -11);
        }
      }
    }
    switch (lk)
    {
    case 16001:                     // 'every' '$'
    case 16107:                     // 'some' '$'
      try_QuantifiedExpr();
      break;
    case 97951:                     // 'insert' 'node'
    case 98463:                     // 'insert' 'nodes'
      try_InsertExpr();
      break;
    case 97902:                     // 'delete' 'node'
    case 98414:                     // 'delete' 'nodes'
      try_DeleteExpr();
      break;
    case 98010:                     // 'rename' 'node'
      try_RenameExpr();
      break;
    case -6:
    case 98011:                     // 'replace' 'node'
      try_ReplaceExpr();
      break;
    case 15975:                     // 'copy' '$'
      try_TransformExpr();
      break;
    case 85102:                     // 'delete' 'json'
      try_JSONDeleteExpr();
      break;
    case 85151:                     // 'insert' 'json'
      try_JSONInsertExpr();
      break;
    case 85210:                     // 'rename' 'json'
      try_JSONRenameExpr();
      break;
    case -11:
      try_JSONReplaceExpr();
      break;
    case 85069:                     // 'append' 'json'
      try_JSONAppendExpr();
      break;
    case -13:
      break;
    default:
      try_OrExpr();
    }
  }

  function parse_JSONDeleteExpr()
  {
    eventHandler.startNonterminal("JSONDeleteExpr", e0);
    shift(110);                     // 'delete'
    lookahead1W(56);                // S^WS | '(:' | 'json'
    shift(166);                     // 'json'
    lookahead1W(264);               // EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral | StringLiteral |
    whitespace();
    parse_PostfixExpr();
    eventHandler.endNonterminal("JSONDeleteExpr", e0);
  }

  function try_JSONDeleteExpr()
  {
    shiftT(110);                    // 'delete'
    lookahead1W(56);                // S^WS | '(:' | 'json'
    shiftT(166);                    // 'json'
    lookahead1W(264);               // EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral | StringLiteral |
    try_PostfixExpr();
  }

  function parse_JSONInsertExpr()
  {
    eventHandler.startNonterminal("JSONInsertExpr", e0);
    switch (l1)
    {
    case 159:                       // 'insert'
      lookahead2W(56);              // S^WS | '(:' | 'json'
      break;
    default:
      lk = l1;
    }
    lk = memoized(10, e0);
    if (lk == 0)
    {
      var b0A = b0; var e0A = e0; var l1A = l1;
      var b1A = b1; var e1A = e1; var l2A = l2;
      var b2A = b2; var e2A = e2;
      try
      {
        shiftT(159);                // 'insert'
        lookahead1W(56);            // S^WS | '(:' | 'json'
        shiftT(166);                // 'json'
        lookahead1W(267);           // Wildcard | EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral |
        try_ExprSingle();
        shiftT(163);                // 'into'
        lookahead1W(267);           // Wildcard | EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral |
        try_ExprSingle();
        switch (l1)
        {
        case 81:                    // 'at'
          lookahead2W(69);          // S^WS | '(:' | 'position'
          break;
        default:
          lk = l1;
        }
        if (lk == 108113)           // 'at' 'position'
        {
          lk = memoized(11, e0);
          if (lk == 0)
          {
            var b0B = b0; var e0B = e0; var l1B = l1;
            var b1B = b1; var e1B = e1; var l2B = l2;
            var b2B = b2; var e2B = e2;
            try
            {
              shiftT(81);           // 'at'
              lookahead1W(69);      // S^WS | '(:' | 'position'
              shiftT(211);          // 'position'
              lookahead1W(267);     // Wildcard | EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral |
              try_ExprSingle();
              memoize(11, e0B, -1);
            }
            catch (p1B)
            {
              b0 = b0B; e0 = e0B; l1 = l1B; if (l1 == 0) {end = e0B;} else {
              b1 = b1B; e1 = e1B; l2 = l2B; if (l2 == 0) {end = e1B;} else {
              b2 = b2B; e2 = e2B; end = e2B; }}
              memoize(11, e0B, -2);
            }
            lk = -2;
          }
        }
        if (lk == -1)
        {
          shiftT(81);               // 'at'
          lookahead1W(69);          // S^WS | '(:' | 'position'
          shiftT(211);              // 'position'
          lookahead1W(267);         // Wildcard | EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral |
          try_ExprSingle();
        }
        lk = -1;
      }
      catch (p1A)
      {
        lk = -2;
      }
      b0 = b0A; e0 = e0A; l1 = l1A; if (l1 == 0) {end = e0A;} else {
      b1 = b1A; e1 = e1A; l2 = l2A; if (l2 == 0) {end = e1A;} else {
      b2 = b2A; e2 = e2A; end = e2A; }}
      memoize(10, e0, lk);
    }
    switch (lk)
    {
    case -1:
      shift(159);                   // 'insert'
      lookahead1W(56);              // S^WS | '(:' | 'json'
      shift(166);                   // 'json'
      lookahead1W(267);             // Wildcard | EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral |
      whitespace();
      parse_ExprSingle();
      shift(163);                   // 'into'
      lookahead1W(267);             // Wildcard | EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral |
      whitespace();
      parse_ExprSingle();
      switch (l1)
      {
      case 81:                      // 'at'
        lookahead2W(69);            // S^WS | '(:' | 'position'
        break;
      default:
        lk = l1;
      }
      if (lk == 108113)             // 'at' 'position'
      {
        lk = memoized(11, e0);
        if (lk == 0)
        {
          var b0B = b0; var e0B = e0; var l1B = l1;
          var b1B = b1; var e1B = e1; var l2B = l2;
          var b2B = b2; var e2B = e2;
          try
          {
            shiftT(81);             // 'at'
            lookahead1W(69);        // S^WS | '(:' | 'position'
            shiftT(211);            // 'position'
            lookahead1W(267);       // Wildcard | EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral |
            try_ExprSingle();
            lk = -1;
          }
          catch (p1B)
          {
            lk = -2;
          }
          b0 = b0B; e0 = e0B; l1 = l1B; if (l1 == 0) {end = e0B;} else {
          b1 = b1B; e1 = e1B; l2 = l2B; if (l2 == 0) {end = e1B;} else {
          b2 = b2B; e2 = e2B; end = e2B; }}
          memoize(11, e0, lk);
        }
      }
      if (lk == -1)
      {
        shift(81);                  // 'at'
        lookahead1W(69);            // S^WS | '(:' | 'position'
        shift(211);                 // 'position'
        lookahead1W(267);           // Wildcard | EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral |
        whitespace();
        parse_ExprSingle();
      }
      break;
    default:
      shift(159);                   // 'insert'
      lookahead1W(56);              // S^WS | '(:' | 'json'
      shift(166);                   // 'json'
      lookahead1W(267);             // Wildcard | EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral |
      whitespace();
      parse_PairConstructorList();
      shift(163);                   // 'into'
      lookahead1W(267);             // Wildcard | EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral |
      whitespace();
      parse_ExprSingle();
    }
    eventHandler.endNonterminal("JSONInsertExpr", e0);
  }

  function try_JSONInsertExpr()
  {
    switch (l1)
    {
    case 159:                       // 'insert'
      lookahead2W(56);              // S^WS | '(:' | 'json'
      break;
    default:
      lk = l1;
    }
    lk = memoized(10, e0);
    if (lk == 0)
    {
      var b0A = b0; var e0A = e0; var l1A = l1;
      var b1A = b1; var e1A = e1; var l2A = l2;
      var b2A = b2; var e2A = e2;
      try
      {
        shiftT(159);                // 'insert'
        lookahead1W(56);            // S^WS | '(:' | 'json'
        shiftT(166);                // 'json'
        lookahead1W(267);           // Wildcard | EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral |
        try_ExprSingle();
        shiftT(163);                // 'into'
        lookahead1W(267);           // Wildcard | EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral |
        try_ExprSingle();
        switch (l1)
        {
        case 81:                    // 'at'
          lookahead2W(69);          // S^WS | '(:' | 'position'
          break;
        default:
          lk = l1;
        }
        if (lk == 108113)           // 'at' 'position'
        {
          lk = memoized(11, e0);
          if (lk == 0)
          {
            var b0B = b0; var e0B = e0; var l1B = l1;
            var b1B = b1; var e1B = e1; var l2B = l2;
            var b2B = b2; var e2B = e2;
            try
            {
              shiftT(81);           // 'at'
              lookahead1W(69);      // S^WS | '(:' | 'position'
              shiftT(211);          // 'position'
              lookahead1W(267);     // Wildcard | EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral |
              try_ExprSingle();
              memoize(11, e0B, -1);
            }
            catch (p1B)
            {
              b0 = b0B; e0 = e0B; l1 = l1B; if (l1 == 0) {end = e0B;} else {
              b1 = b1B; e1 = e1B; l2 = l2B; if (l2 == 0) {end = e1B;} else {
              b2 = b2B; e2 = e2B; end = e2B; }}
              memoize(11, e0B, -2);
            }
            lk = -2;
          }
        }
        if (lk == -1)
        {
          shiftT(81);               // 'at'
          lookahead1W(69);          // S^WS | '(:' | 'position'
          shiftT(211);              // 'position'
          lookahead1W(267);         // Wildcard | EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral |
          try_ExprSingle();
        }
        memoize(10, e0A, -1);
        lk = -3;
      }
      catch (p1A)
      {
        lk = -2;
        b0 = b0A; e0 = e0A; l1 = l1A; if (l1 == 0) {end = e0A;} else {
        b1 = b1A; e1 = e1A; l2 = l2A; if (l2 == 0) {end = e1A;} else {
        b2 = b2A; e2 = e2A; end = e2A; }}
        memoize(10, e0A, -2);
      }
    }
    switch (lk)
    {
    case -1:
      shiftT(159);                  // 'insert'
      lookahead1W(56);              // S^WS | '(:' | 'json'
      shiftT(166);                  // 'json'
      lookahead1W(267);             // Wildcard | EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral |
      try_ExprSingle();
      shiftT(163);                  // 'into'
      lookahead1W(267);             // Wildcard | EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral |
      try_ExprSingle();
      switch (l1)
      {
      case 81:                      // 'at'
        lookahead2W(69);            // S^WS | '(:' | 'position'
        break;
      default:
        lk = l1;
      }
      if (lk == 108113)             // 'at' 'position'
      {
        lk = memoized(11, e0);
        if (lk == 0)
        {
          var b0B = b0; var e0B = e0; var l1B = l1;
          var b1B = b1; var e1B = e1; var l2B = l2;
          var b2B = b2; var e2B = e2;
          try
          {
            shiftT(81);             // 'at'
            lookahead1W(69);        // S^WS | '(:' | 'position'
            shiftT(211);            // 'position'
            lookahead1W(267);       // Wildcard | EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral |
            try_ExprSingle();
            memoize(11, e0B, -1);
          }
          catch (p1B)
          {
            b0 = b0B; e0 = e0B; l1 = l1B; if (l1 == 0) {end = e0B;} else {
            b1 = b1B; e1 = e1B; l2 = l2B; if (l2 == 0) {end = e1B;} else {
            b2 = b2B; e2 = e2B; end = e2B; }}
            memoize(11, e0B, -2);
          }
          lk = -2;
        }
      }
      if (lk == -1)
      {
        shiftT(81);                 // 'at'
        lookahead1W(69);            // S^WS | '(:' | 'position'
        shiftT(211);                // 'position'
        lookahead1W(267);           // Wildcard | EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral |
        try_ExprSingle();
      }
      break;
    case -3:
      break;
    default:
      shiftT(159);                  // 'insert'
      lookahead1W(56);              // S^WS | '(:' | 'json'
      shiftT(166);                  // 'json'
      lookahead1W(267);             // Wildcard | EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral |
      try_PairConstructorList();
      shiftT(163);                  // 'into'
      lookahead1W(267);             // Wildcard | EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral |
      try_ExprSingle();
    }
  }

  function parse_JSONRenameExpr()
  {
    eventHandler.startNonterminal("JSONRenameExpr", e0);
    shift(218);                     // 'rename'
    lookahead1W(56);                // S^WS | '(:' | 'json'
    shift(166);                     // 'json'
    lookahead1W(264);               // EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral | StringLiteral |
    whitespace();
    parse_PostfixExpr();
    shift(79);                      // 'as'
    lookahead1W(267);               // Wildcard | EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral |
    whitespace();
    parse_ExprSingle();
    eventHandler.endNonterminal("JSONRenameExpr", e0);
  }

  function try_JSONRenameExpr()
  {
    shiftT(218);                    // 'rename'
    lookahead1W(56);                // S^WS | '(:' | 'json'
    shiftT(166);                    // 'json'
    lookahead1W(264);               // EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral | StringLiteral |
    try_PostfixExpr();
    shiftT(79);                     // 'as'
    lookahead1W(267);               // Wildcard | EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral |
    try_ExprSingle();
  }

  function parse_JSONReplaceExpr()
  {
    eventHandler.startNonterminal("JSONReplaceExpr", e0);
    shift(219);                     // 'replace'
    lookahead1W(82);                // S^WS | '(:' | 'value'
    shift(261);                     // 'value'
    lookahead1W(64);                // S^WS | '(:' | 'of'
    shift(196);                     // 'of'
    lookahead1W(56);                // S^WS | '(:' | 'json'
    shift(166);                     // 'json'
    lookahead1W(264);               // EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral | StringLiteral |
    whitespace();
    parse_PostfixExpr();
    shift(270);                     // 'with'
    lookahead1W(267);               // Wildcard | EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral |
    whitespace();
    parse_ExprSingle();
    eventHandler.endNonterminal("JSONReplaceExpr", e0);
  }

  function try_JSONReplaceExpr()
  {
    shiftT(219);                    // 'replace'
    lookahead1W(82);                // S^WS | '(:' | 'value'
    shiftT(261);                    // 'value'
    lookahead1W(64);                // S^WS | '(:' | 'of'
    shiftT(196);                    // 'of'
    lookahead1W(56);                // S^WS | '(:' | 'json'
    shiftT(166);                    // 'json'
    lookahead1W(264);               // EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral | StringLiteral |
    try_PostfixExpr();
    shiftT(270);                    // 'with'
    lookahead1W(267);               // Wildcard | EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral |
    try_ExprSingle();
  }

  function parse_JSONAppendExpr()
  {
    eventHandler.startNonterminal("JSONAppendExpr", e0);
    shift(77);                      // 'append'
    lookahead1W(56);                // S^WS | '(:' | 'json'
    shift(166);                     // 'json'
    lookahead1W(267);               // Wildcard | EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral |
    whitespace();
    parse_ExprSingle();
    shift(163);                     // 'into'
    lookahead1W(267);               // Wildcard | EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral |
    whitespace();
    parse_ExprSingle();
    eventHandler.endNonterminal("JSONAppendExpr", e0);
  }

  function try_JSONAppendExpr()
  {
    shiftT(77);                     // 'append'
    lookahead1W(56);                // S^WS | '(:' | 'json'
    shiftT(166);                    // 'json'
    lookahead1W(267);               // Wildcard | EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral |
    try_ExprSingle();
    shiftT(163);                    // 'into'
    lookahead1W(267);               // Wildcard | EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral |
    try_ExprSingle();
  }

  function parse_CommonContent()
  {
    eventHandler.startNonterminal("CommonContent", e0);
    switch (l1)
    {
    case 12:                        // PredefinedEntityRef
      shift(12);                    // PredefinedEntityRef
      break;
    case 23:                        // CharRef
      shift(23);                    // CharRef
      break;
    case 277:                       // '{{'
      shift(277);                   // '{{'
      break;
    case 283:                       // '}}'
      shift(283);                   // '}}'
      break;
    default:
      parse_BlockExpr();
    }
    eventHandler.endNonterminal("CommonContent", e0);
  }

  function try_CommonContent()
  {
    switch (l1)
    {
    case 12:                        // PredefinedEntityRef
      shiftT(12);                   // PredefinedEntityRef
      break;
    case 23:                        // CharRef
      shiftT(23);                   // CharRef
      break;
    case 277:                       // '{{'
      shiftT(277);                  // '{{'
      break;
    case 283:                       // '}}'
      shiftT(283);                  // '}}'
      break;
    default:
      try_BlockExpr();
    }
  }

  function parse_ContentExpr()
  {
    eventHandler.startNonterminal("ContentExpr", e0);
    parse_StatementsAndExpr();
    eventHandler.endNonterminal("ContentExpr", e0);
  }

  function try_ContentExpr()
  {
    try_StatementsAndExpr();
  }

  function parse_CompDocConstructor()
  {
    eventHandler.startNonterminal("CompDocConstructor", e0);
    shift(119);                     // 'document'
    lookahead1W(87);                // S^WS | '(:' | '{'
    whitespace();
    parse_BlockExpr();
    eventHandler.endNonterminal("CompDocConstructor", e0);
  }

  function try_CompDocConstructor()
  {
    shiftT(119);                    // 'document'
    lookahead1W(87);                // S^WS | '(:' | '{'
    try_BlockExpr();
  }

  function parse_CompAttrConstructor()
  {
    eventHandler.startNonterminal("CompAttrConstructor", e0);
    shift(82);                      // 'attribute'
    lookahead1W(258);               // EQName^Token | S^WS | '(:' | 'after' | 'allowing' | 'ancestor' |
    switch (l1)
    {
    case 276:                       // '{'
      shift(276);                   // '{'
      lookahead1W(267);             // Wildcard | EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral |
      whitespace();
      parse_Expr();
      shift(282);                   // '}'
      break;
    default:
      whitespace();
      parse_EQName();
    }
    lookahead1W(87);                // S^WS | '(:' | '{'
    switch (l1)
    {
    case 276:                       // '{'
      lookahead2W(277);             // Wildcard | EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral |
      break;
    default:
      lk = l1;
    }
    if (lk == 144660)               // '{' '}'
    {
      lk = memoized(12, e0);
      if (lk == 0)
      {
        var b0A = b0; var e0A = e0; var l1A = l1;
        var b1A = b1; var e1A = e1; var l2A = l2;
        var b2A = b2; var e2A = e2;
        try
        {
          shiftT(276);              // '{'
          lookahead1W(88);          // S^WS | '(:' | '}'
          shiftT(282);              // '}'
          lk = -1;
        }
        catch (p1A)
        {
          lk = -2;
        }
        b0 = b0A; e0 = e0A; l1 = l1A; if (l1 == 0) {end = e0A;} else {
        b1 = b1A; e1 = e1A; l2 = l2A; if (l2 == 0) {end = e1A;} else {
        b2 = b2A; e2 = e2A; end = e2A; }}
        memoize(12, e0, lk);
      }
    }
    switch (lk)
    {
    case -1:
      shift(276);                   // '{'
      lookahead1W(88);              // S^WS | '(:' | '}'
      shift(282);                   // '}'
      break;
    default:
      whitespace();
      parse_BlockExpr();
    }
    eventHandler.endNonterminal("CompAttrConstructor", e0);
  }

  function try_CompAttrConstructor()
  {
    shiftT(82);                     // 'attribute'
    lookahead1W(258);               // EQName^Token | S^WS | '(:' | 'after' | 'allowing' | 'ancestor' |
    switch (l1)
    {
    case 276:                       // '{'
      shiftT(276);                  // '{'
      lookahead1W(267);             // Wildcard | EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral |
      try_Expr();
      shiftT(282);                  // '}'
      break;
    default:
      try_EQName();
    }
    lookahead1W(87);                // S^WS | '(:' | '{'
    switch (l1)
    {
    case 276:                       // '{'
      lookahead2W(277);             // Wildcard | EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral |
      break;
    default:
      lk = l1;
    }
    if (lk == 144660)               // '{' '}'
    {
      lk = memoized(12, e0);
      if (lk == 0)
      {
        var b0A = b0; var e0A = e0; var l1A = l1;
        var b1A = b1; var e1A = e1; var l2A = l2;
        var b2A = b2; var e2A = e2;
        try
        {
          shiftT(276);              // '{'
          lookahead1W(88);          // S^WS | '(:' | '}'
          shiftT(282);              // '}'
          memoize(12, e0A, -1);
          lk = -3;
        }
        catch (p1A)
        {
          lk = -2;
          b0 = b0A; e0 = e0A; l1 = l1A; if (l1 == 0) {end = e0A;} else {
          b1 = b1A; e1 = e1A; l2 = l2A; if (l2 == 0) {end = e1A;} else {
          b2 = b2A; e2 = e2A; end = e2A; }}
          memoize(12, e0A, -2);
        }
      }
    }
    switch (lk)
    {
    case -1:
      shiftT(276);                  // '{'
      lookahead1W(88);              // S^WS | '(:' | '}'
      shiftT(282);                  // '}'
      break;
    case -3:
      break;
    default:
      try_BlockExpr();
    }
  }

  function parse_CompPIConstructor()
  {
    eventHandler.startNonterminal("CompPIConstructor", e0);
    shift(216);                     // 'processing-instruction'
    lookahead1W(251);               // NCName^Token | S^WS | '(:' | 'after' | 'allowing' | 'ancestor' |
    switch (l1)
    {
    case 276:                       // '{'
      shift(276);                   // '{'
      lookahead1W(267);             // Wildcard | EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral |
      whitespace();
      parse_Expr();
      shift(282);                   // '}'
      break;
    default:
      whitespace();
      parse_NCName();
    }
    lookahead1W(87);                // S^WS | '(:' | '{'
    switch (l1)
    {
    case 276:                       // '{'
      lookahead2W(277);             // Wildcard | EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral |
      break;
    default:
      lk = l1;
    }
    if (lk == 144660)               // '{' '}'
    {
      lk = memoized(13, e0);
      if (lk == 0)
      {
        var b0A = b0; var e0A = e0; var l1A = l1;
        var b1A = b1; var e1A = e1; var l2A = l2;
        var b2A = b2; var e2A = e2;
        try
        {
          shiftT(276);              // '{'
          lookahead1W(88);          // S^WS | '(:' | '}'
          shiftT(282);              // '}'
          lk = -1;
        }
        catch (p1A)
        {
          lk = -2;
        }
        b0 = b0A; e0 = e0A; l1 = l1A; if (l1 == 0) {end = e0A;} else {
        b1 = b1A; e1 = e1A; l2 = l2A; if (l2 == 0) {end = e1A;} else {
        b2 = b2A; e2 = e2A; end = e2A; }}
        memoize(13, e0, lk);
      }
    }
    switch (lk)
    {
    case -1:
      shift(276);                   // '{'
      lookahead1W(88);              // S^WS | '(:' | '}'
      shift(282);                   // '}'
      break;
    default:
      whitespace();
      parse_BlockExpr();
    }
    eventHandler.endNonterminal("CompPIConstructor", e0);
  }

  function try_CompPIConstructor()
  {
    shiftT(216);                    // 'processing-instruction'
    lookahead1W(251);               // NCName^Token | S^WS | '(:' | 'after' | 'allowing' | 'ancestor' |
    switch (l1)
    {
    case 276:                       // '{'
      shiftT(276);                  // '{'
      lookahead1W(267);             // Wildcard | EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral |
      try_Expr();
      shiftT(282);                  // '}'
      break;
    default:
      try_NCName();
    }
    lookahead1W(87);                // S^WS | '(:' | '{'
    switch (l1)
    {
    case 276:                       // '{'
      lookahead2W(277);             // Wildcard | EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral |
      break;
    default:
      lk = l1;
    }
    if (lk == 144660)               // '{' '}'
    {
      lk = memoized(13, e0);
      if (lk == 0)
      {
        var b0A = b0; var e0A = e0; var l1A = l1;
        var b1A = b1; var e1A = e1; var l2A = l2;
        var b2A = b2; var e2A = e2;
        try
        {
          shiftT(276);              // '{'
          lookahead1W(88);          // S^WS | '(:' | '}'
          shiftT(282);              // '}'
          memoize(13, e0A, -1);
          lk = -3;
        }
        catch (p1A)
        {
          lk = -2;
          b0 = b0A; e0 = e0A; l1 = l1A; if (l1 == 0) {end = e0A;} else {
          b1 = b1A; e1 = e1A; l2 = l2A; if (l2 == 0) {end = e1A;} else {
          b2 = b2A; e2 = e2A; end = e2A; }}
          memoize(13, e0A, -2);
        }
      }
    }
    switch (lk)
    {
    case -1:
      shiftT(276);                  // '{'
      lookahead1W(88);              // S^WS | '(:' | '}'
      shiftT(282);                  // '}'
      break;
    case -3:
      break;
    default:
      try_BlockExpr();
    }
  }

  function parse_CompCommentConstructor()
  {
    eventHandler.startNonterminal("CompCommentConstructor", e0);
    shift(96);                      // 'comment'
    lookahead1W(87);                // S^WS | '(:' | '{'
    whitespace();
    parse_BlockExpr();
    eventHandler.endNonterminal("CompCommentConstructor", e0);
  }

  function try_CompCommentConstructor()
  {
    shiftT(96);                     // 'comment'
    lookahead1W(87);                // S^WS | '(:' | '{'
    try_BlockExpr();
  }

  function parse_CompTextConstructor()
  {
    eventHandler.startNonterminal("CompTextConstructor", e0);
    shift(244);                     // 'text'
    lookahead1W(87);                // S^WS | '(:' | '{'
    whitespace();
    parse_BlockExpr();
    eventHandler.endNonterminal("CompTextConstructor", e0);
  }

  function try_CompTextConstructor()
  {
    shiftT(244);                    // 'text'
    lookahead1W(87);                // S^WS | '(:' | '{'
    try_BlockExpr();
  }

  function parse_PrimaryExpr()
  {
    eventHandler.startNonterminal("PrimaryExpr", e0);
    switch (l1)
    {
    case 184:                       // 'namespace'
      lookahead2W(256);             // NCName^Token | S^WS | '#' | '(' | '(:' | 'after' | 'allowing' | 'ancestor' |
      break;
    case 216:                       // 'processing-instruction'
      lookahead2W(254);             // NCName^Token | S^WS | '#' | '(:' | 'after' | 'allowing' | 'ancestor' |
      break;
    case 276:                       // '{'
      lookahead2W(277);             // Wildcard | EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral |
      break;
    case 82:                        // 'attribute'
    case 121:                       // 'element'
      lookahead2W(259);             // EQName^Token | S^WS | '#' | '(:' | 'after' | 'allowing' | 'ancestor' |
      break;
    case 96:                        // 'comment'
    case 244:                       // 'text'
      lookahead2W(93);              // S^WS | '#' | '(:' | '{'
      break;
    case 119:                       // 'document'
    case 202:                       // 'ordered'
    case 256:                       // 'unordered'
      lookahead2W(139);             // S^WS | '#' | '(' | '(:' | '{'
      break;
    case 6:                         // EQName^Token
    case 70:                        // 'after'
    case 72:                        // 'allowing'
    case 73:                        // 'ancestor'
    case 74:                        // 'ancestor-or-self'
    case 75:                        // 'and'
    case 77:                        // 'append'
    case 79:                        // 'as'
    case 80:                        // 'ascending'
    case 81:                        // 'at'
    case 83:                        // 'base-uri'
    case 84:                        // 'before'
    case 85:                        // 'boundary-space'
    case 86:                        // 'break'
    case 88:                        // 'case'
    case 89:                        // 'cast'
    case 90:                        // 'castable'
    case 91:                        // 'catch'
    case 93:                        // 'child'
    case 94:                        // 'collation'
    case 97:                        // 'constraint'
    case 98:                        // 'construction'
    case 101:                       // 'context'
    case 102:                       // 'continue'
    case 103:                       // 'copy'
    case 104:                       // 'copy-namespaces'
    case 105:                       // 'count'
    case 106:                       // 'decimal-format'
    case 108:                       // 'declare'
    case 109:                       // 'default'
    case 110:                       // 'delete'
    case 111:                       // 'descendant'
    case 112:                       // 'descendant-or-self'
    case 113:                       // 'descending'
    case 118:                       // 'div'
    case 122:                       // 'else'
    case 123:                       // 'empty'
    case 125:                       // 'encoding'
    case 126:                       // 'end'
    case 128:                       // 'eq'
    case 129:                       // 'every'
    case 131:                       // 'except'
    case 132:                       // 'exit'
    case 133:                       // 'external'
    case 134:                       // 'first'
    case 135:                       // 'following'
    case 136:                       // 'following-sibling'
    case 137:                       // 'for'
    case 141:                       // 'ft-option'
    case 146:                       // 'ge'
    case 148:                       // 'group'
    case 150:                       // 'gt'
    case 151:                       // 'idiv'
    case 153:                       // 'import'
    case 154:                       // 'in'
    case 155:                       // 'index'
    case 159:                       // 'insert'
    case 160:                       // 'instance'
    case 161:                       // 'integrity'
    case 162:                       // 'intersect'
    case 163:                       // 'into'
    case 164:                       // 'is'
    case 166:                       // 'json'
    case 170:                       // 'last'
    case 171:                       // 'lax'
    case 172:                       // 'le'
    case 174:                       // 'let'
    case 176:                       // 'loop'
    case 178:                       // 'lt'
    case 180:                       // 'mod'
    case 181:                       // 'modify'
    case 182:                       // 'module'
    case 186:                       // 'ne'
    case 192:                       // 'nodes'
    case 194:                       // 'object'
    case 198:                       // 'only'
    case 199:                       // 'option'
    case 200:                       // 'or'
    case 201:                       // 'order'
    case 203:                       // 'ordering'
    case 206:                       // 'parent'
    case 212:                       // 'preceding'
    case 213:                       // 'preceding-sibling'
    case 218:                       // 'rename'
    case 219:                       // 'replace'
    case 220:                       // 'return'
    case 221:                       // 'returning'
    case 222:                       // 'revalidation'
    case 224:                       // 'satisfies'
    case 225:                       // 'schema'
    case 228:                       // 'score'
    case 229:                       // 'self'
    case 234:                       // 'sliding'
    case 235:                       // 'some'
    case 236:                       // 'stable'
    case 237:                       // 'start'
    case 240:                       // 'strict'
    case 248:                       // 'to'
    case 249:                       // 'treat'
    case 250:                       // 'try'
    case 251:                       // 'tumbling'
    case 252:                       // 'type'
    case 254:                       // 'union'
    case 257:                       // 'updating'
    case 260:                       // 'validate'
    case 261:                       // 'value'
    case 262:                       // 'variable'
    case 263:                       // 'version'
    case 266:                       // 'where'
    case 267:                       // 'while'
    case 270:                       // 'with'
    case 274:                       // 'xquery'
      lookahead2W(92);              // S^WS | '#' | '(' | '(:'
      break;
    default:
      lk = l1;
    }
    if (lk == 2836                  // '{' Wildcard
     || lk == 3348                  // '{' EQName^Token
     || lk == 4372                  // '{' IntegerLiteral
     || lk == 4884                  // '{' DecimalLiteral
     || lk == 5396                  // '{' DoubleLiteral
     || lk == 5908                  // '{' StringLiteral
     || lk == 16148                 // '{' '$'
     || lk == 16660                 // '{' '%'
     || lk == 17684                 // '{' '('
     || lk == 18196                 // '{' '(#'
     || lk == 20756                 // '{' '+'
     || lk == 21780                 // '{' '-'
     || lk == 22804                 // '{' '.'
     || lk == 23316                 // '{' '..'
     || lk == 23828                 // '{' '/'
     || lk == 24340                 // '{' '//'
     || lk == 27924                 // '{' '<'
     || lk == 28436                 // '{' '<!--'
     || lk == 30484                 // '{' '<?'
     || lk == 34068                 // '{' '@'
     || lk == 35092                 // '{' '['
     || lk == 36116                 // '{' 'after'
     || lk == 37140                 // '{' 'allowing'
     || lk == 37652                 // '{' 'ancestor'
     || lk == 38164                 // '{' 'ancestor-or-self'
     || lk == 38676                 // '{' 'and'
     || lk == 39700                 // '{' 'append'
     || lk == 40212                 // '{' 'array'
     || lk == 40724                 // '{' 'as'
     || lk == 41236                 // '{' 'ascending'
     || lk == 41748                 // '{' 'at'
     || lk == 42260                 // '{' 'attribute'
     || lk == 42772                 // '{' 'base-uri'
     || lk == 43284                 // '{' 'before'
     || lk == 43796                 // '{' 'boundary-space'
     || lk == 44308                 // '{' 'break'
     || lk == 45332                 // '{' 'case'
     || lk == 45844                 // '{' 'cast'
     || lk == 46356                 // '{' 'castable'
     || lk == 46868                 // '{' 'catch'
     || lk == 47892                 // '{' 'child'
     || lk == 48404                 // '{' 'collation'
     || lk == 49428                 // '{' 'comment'
     || lk == 49940                 // '{' 'constraint'
     || lk == 50452                 // '{' 'construction'
     || lk == 51988                 // '{' 'context'
     || lk == 52500                 // '{' 'continue'
     || lk == 53012                 // '{' 'copy'
     || lk == 53524                 // '{' 'copy-namespaces'
     || lk == 54036                 // '{' 'count'
     || lk == 54548                 // '{' 'decimal-format'
     || lk == 55572                 // '{' 'declare'
     || lk == 56084                 // '{' 'default'
     || lk == 56596                 // '{' 'delete'
     || lk == 57108                 // '{' 'descendant'
     || lk == 57620                 // '{' 'descendant-or-self'
     || lk == 58132                 // '{' 'descending'
     || lk == 60692                 // '{' 'div'
     || lk == 61204                 // '{' 'document'
     || lk == 61716                 // '{' 'document-node'
     || lk == 62228                 // '{' 'element'
     || lk == 62740                 // '{' 'else'
     || lk == 63252                 // '{' 'empty'
     || lk == 63764                 // '{' 'empty-sequence'
     || lk == 64276                 // '{' 'encoding'
     || lk == 64788                 // '{' 'end'
     || lk == 65812                 // '{' 'eq'
     || lk == 66324                 // '{' 'every'
     || lk == 67348                 // '{' 'except'
     || lk == 67860                 // '{' 'exit'
     || lk == 68372                 // '{' 'external'
     || lk == 68884                 // '{' 'first'
     || lk == 69396                 // '{' 'following'
     || lk == 69908                 // '{' 'following-sibling'
     || lk == 70420                 // '{' 'for'
     || lk == 72468                 // '{' 'ft-option'
     || lk == 74516                 // '{' 'function'
     || lk == 75028                 // '{' 'ge'
     || lk == 76052                 // '{' 'group'
     || lk == 77076                 // '{' 'gt'
     || lk == 77588                 // '{' 'idiv'
     || lk == 78100                 // '{' 'if'
     || lk == 78612                 // '{' 'import'
     || lk == 79124                 // '{' 'in'
     || lk == 79636                 // '{' 'index'
     || lk == 81684                 // '{' 'insert'
     || lk == 82196                 // '{' 'instance'
     || lk == 82708                 // '{' 'integrity'
     || lk == 83220                 // '{' 'intersect'
     || lk == 83732                 // '{' 'into'
     || lk == 84244                 // '{' 'is'
     || lk == 84756                 // '{' 'item'
     || lk == 85268                 // '{' 'json'
     || lk == 85780                 // '{' 'json-item'
     || lk == 87316                 // '{' 'last'
     || lk == 87828                 // '{' 'lax'
     || lk == 88340                 // '{' 'le'
     || lk == 89364                 // '{' 'let'
     || lk == 90388                 // '{' 'loop'
     || lk == 91412                 // '{' 'lt'
     || lk == 92436                 // '{' 'mod'
     || lk == 92948                 // '{' 'modify'
     || lk == 93460                 // '{' 'module'
     || lk == 94484                 // '{' 'namespace'
     || lk == 94996                 // '{' 'namespace-node'
     || lk == 95508                 // '{' 'ne'
     || lk == 98068                 // '{' 'node'
     || lk == 98580                 // '{' 'nodes'
     || lk == 99604                 // '{' 'object'
     || lk == 101652                // '{' 'only'
     || lk == 102164                // '{' 'option'
     || lk == 102676                // '{' 'or'
     || lk == 103188                // '{' 'order'
     || lk == 103700                // '{' 'ordered'
     || lk == 104212                // '{' 'ordering'
     || lk == 105748                // '{' 'parent'
     || lk == 108820                // '{' 'preceding'
     || lk == 109332                // '{' 'preceding-sibling'
     || lk == 110868                // '{' 'processing-instruction'
     || lk == 111892                // '{' 'rename'
     || lk == 112404                // '{' 'replace'
     || lk == 112916                // '{' 'return'
     || lk == 113428                // '{' 'returning'
     || lk == 113940                // '{' 'revalidation'
     || lk == 114964                // '{' 'satisfies'
     || lk == 115476                // '{' 'schema'
     || lk == 115988                // '{' 'schema-attribute'
     || lk == 116500                // '{' 'schema-element'
     || lk == 117012                // '{' 'score'
     || lk == 117524                // '{' 'self'
     || lk == 120084                // '{' 'sliding'
     || lk == 120596                // '{' 'some'
     || lk == 121108                // '{' 'stable'
     || lk == 121620                // '{' 'start'
     || lk == 123156                // '{' 'strict'
     || lk == 124180                // '{' 'structured-item'
     || lk == 124692                // '{' 'switch'
     || lk == 125204                // '{' 'text'
     || lk == 127252                // '{' 'to'
     || lk == 127764                // '{' 'treat'
     || lk == 128276                // '{' 'try'
     || lk == 128788                // '{' 'tumbling'
     || lk == 129300                // '{' 'type'
     || lk == 129812                // '{' 'typeswitch'
     || lk == 130324                // '{' 'union'
     || lk == 131348                // '{' 'unordered'
     || lk == 131860                // '{' 'updating'
     || lk == 133396                // '{' 'validate'
     || lk == 133908                // '{' 'value'
     || lk == 134420                // '{' 'variable'
     || lk == 134932                // '{' 'version'
     || lk == 136468                // '{' 'where'
     || lk == 136980                // '{' 'while'
     || lk == 138516                // '{' 'with'
     || lk == 140564                // '{' 'xquery'
     || lk == 141588                // '{' '{'
     || lk == 142612                // '{' '{|'
     || lk == 144660)               // '{' '}'
    {
      lk = memoized(14, e0);
      if (lk == 0)
      {
        var b0A = b0; var e0A = e0; var l1A = l1;
        var b1A = b1; var e1A = e1; var l2A = l2;
        var b2A = b2; var e2A = e2;
        try
        {
          try_BlockExpr();
          lk = -10;
        }
        catch (p10A)
        {
          lk = -11;
        }
        b0 = b0A; e0 = e0A; l1 = l1A; if (l1 == 0) {end = e0A;} else {
        b1 = b1A; e1 = e1A; l2 = l2A; if (l2 == 0) {end = e1A;} else {
        b2 = b2A; e2 = e2A; end = e2A; }}
        memoize(14, e0, lk);
      }
    }
    switch (lk)
    {
    case 8:                         // IntegerLiteral
    case 9:                         // DecimalLiteral
    case 10:                        // DoubleLiteral
    case 11:                        // StringLiteral
      parse_Literal();
      break;
    case 31:                        // '$'
      parse_VarRef();
      break;
    case 34:                        // '('
      parse_ParenthesizedExpr();
      break;
    case 44:                        // '.'
      parse_ContextItemExpr();
      break;
    case 17414:                     // EQName^Token '('
    case 17478:                     // 'after' '('
    case 17480:                     // 'allowing' '('
    case 17481:                     // 'ancestor' '('
    case 17482:                     // 'ancestor-or-self' '('
    case 17483:                     // 'and' '('
    case 17485:                     // 'append' '('
    case 17487:                     // 'as' '('
    case 17488:                     // 'ascending' '('
    case 17489:                     // 'at' '('
    case 17491:                     // 'base-uri' '('
    case 17492:                     // 'before' '('
    case 17493:                     // 'boundary-space' '('
    case 17494:                     // 'break' '('
    case 17496:                     // 'case' '('
    case 17497:                     // 'cast' '('
    case 17498:                     // 'castable' '('
    case 17499:                     // 'catch' '('
    case 17501:                     // 'child' '('
    case 17502:                     // 'collation' '('
    case 17505:                     // 'constraint' '('
    case 17506:                     // 'construction' '('
    case 17509:                     // 'context' '('
    case 17510:                     // 'continue' '('
    case 17511:                     // 'copy' '('
    case 17512:                     // 'copy-namespaces' '('
    case 17513:                     // 'count' '('
    case 17514:                     // 'decimal-format' '('
    case 17516:                     // 'declare' '('
    case 17517:                     // 'default' '('
    case 17518:                     // 'delete' '('
    case 17519:                     // 'descendant' '('
    case 17520:                     // 'descendant-or-self' '('
    case 17521:                     // 'descending' '('
    case 17526:                     // 'div' '('
    case 17527:                     // 'document' '('
    case 17530:                     // 'else' '('
    case 17531:                     // 'empty' '('
    case 17533:                     // 'encoding' '('
    case 17534:                     // 'end' '('
    case 17536:                     // 'eq' '('
    case 17537:                     // 'every' '('
    case 17539:                     // 'except' '('
    case 17540:                     // 'exit' '('
    case 17541:                     // 'external' '('
    case 17542:                     // 'first' '('
    case 17543:                     // 'following' '('
    case 17544:                     // 'following-sibling' '('
    case 17545:                     // 'for' '('
    case 17549:                     // 'ft-option' '('
    case 17554:                     // 'ge' '('
    case 17556:                     // 'group' '('
    case 17558:                     // 'gt' '('
    case 17559:                     // 'idiv' '('
    case 17561:                     // 'import' '('
    case 17562:                     // 'in' '('
    case 17563:                     // 'index' '('
    case 17567:                     // 'insert' '('
    case 17568:                     // 'instance' '('
    case 17569:                     // 'integrity' '('
    case 17570:                     // 'intersect' '('
    case 17571:                     // 'into' '('
    case 17572:                     // 'is' '('
    case 17574:                     // 'json' '('
    case 17578:                     // 'last' '('
    case 17579:                     // 'lax' '('
    case 17580:                     // 'le' '('
    case 17582:                     // 'let' '('
    case 17584:                     // 'loop' '('
    case 17586:                     // 'lt' '('
    case 17588:                     // 'mod' '('
    case 17589:                     // 'modify' '('
    case 17590:                     // 'module' '('
    case 17592:                     // 'namespace' '('
    case 17594:                     // 'ne' '('
    case 17600:                     // 'nodes' '('
    case 17602:                     // 'object' '('
    case 17606:                     // 'only' '('
    case 17607:                     // 'option' '('
    case 17608:                     // 'or' '('
    case 17609:                     // 'order' '('
    case 17610:                     // 'ordered' '('
    case 17611:                     // 'ordering' '('
    case 17614:                     // 'parent' '('
    case 17620:                     // 'preceding' '('
    case 17621:                     // 'preceding-sibling' '('
    case 17626:                     // 'rename' '('
    case 17627:                     // 'replace' '('
    case 17628:                     // 'return' '('
    case 17629:                     // 'returning' '('
    case 17630:                     // 'revalidation' '('
    case 17632:                     // 'satisfies' '('
    case 17633:                     // 'schema' '('
    case 17636:                     // 'score' '('
    case 17637:                     // 'self' '('
    case 17642:                     // 'sliding' '('
    case 17643:                     // 'some' '('
    case 17644:                     // 'stable' '('
    case 17645:                     // 'start' '('
    case 17648:                     // 'strict' '('
    case 17656:                     // 'to' '('
    case 17657:                     // 'treat' '('
    case 17658:                     // 'try' '('
    case 17659:                     // 'tumbling' '('
    case 17660:                     // 'type' '('
    case 17662:                     // 'union' '('
    case 17664:                     // 'unordered' '('
    case 17665:                     // 'updating' '('
    case 17668:                     // 'validate' '('
    case 17669:                     // 'value' '('
    case 17670:                     // 'variable' '('
    case 17671:                     // 'version' '('
    case 17674:                     // 'where' '('
    case 17675:                     // 'while' '('
    case 17678:                     // 'with' '('
    case 17682:                     // 'xquery' '('
      parse_FunctionCall();
      break;
    case 141514:                    // 'ordered' '{'
      parse_OrderedExpr();
      break;
    case 141568:                    // 'unordered' '{'
      parse_UnorderedExpr();
      break;
    case 32:                        // '%'
    case 78:                        // 'array'
    case 120:                       // 'document-node'
    case 124:                       // 'empty-sequence'
    case 145:                       // 'function'
    case 152:                       // 'if'
    case 165:                       // 'item'
    case 167:                       // 'json-item'
    case 185:                       // 'namespace-node'
    case 191:                       // 'node'
    case 226:                       // 'schema-attribute'
    case 227:                       // 'schema-element'
    case 242:                       // 'structured-item'
    case 243:                       // 'switch'
    case 253:                       // 'typeswitch'
    case 14854:                     // EQName^Token '#'
    case 14918:                     // 'after' '#'
    case 14920:                     // 'allowing' '#'
    case 14921:                     // 'ancestor' '#'
    case 14922:                     // 'ancestor-or-self' '#'
    case 14923:                     // 'and' '#'
    case 14925:                     // 'append' '#'
    case 14927:                     // 'as' '#'
    case 14928:                     // 'ascending' '#'
    case 14929:                     // 'at' '#'
    case 14930:                     // 'attribute' '#'
    case 14931:                     // 'base-uri' '#'
    case 14932:                     // 'before' '#'
    case 14933:                     // 'boundary-space' '#'
    case 14934:                     // 'break' '#'
    case 14936:                     // 'case' '#'
    case 14937:                     // 'cast' '#'
    case 14938:                     // 'castable' '#'
    case 14939:                     // 'catch' '#'
    case 14941:                     // 'child' '#'
    case 14942:                     // 'collation' '#'
    case 14944:                     // 'comment' '#'
    case 14945:                     // 'constraint' '#'
    case 14946:                     // 'construction' '#'
    case 14949:                     // 'context' '#'
    case 14950:                     // 'continue' '#'
    case 14951:                     // 'copy' '#'
    case 14952:                     // 'copy-namespaces' '#'
    case 14953:                     // 'count' '#'
    case 14954:                     // 'decimal-format' '#'
    case 14956:                     // 'declare' '#'
    case 14957:                     // 'default' '#'
    case 14958:                     // 'delete' '#'
    case 14959:                     // 'descendant' '#'
    case 14960:                     // 'descendant-or-self' '#'
    case 14961:                     // 'descending' '#'
    case 14966:                     // 'div' '#'
    case 14967:                     // 'document' '#'
    case 14969:                     // 'element' '#'
    case 14970:                     // 'else' '#'
    case 14971:                     // 'empty' '#'
    case 14973:                     // 'encoding' '#'
    case 14974:                     // 'end' '#'
    case 14976:                     // 'eq' '#'
    case 14977:                     // 'every' '#'
    case 14979:                     // 'except' '#'
    case 14980:                     // 'exit' '#'
    case 14981:                     // 'external' '#'
    case 14982:                     // 'first' '#'
    case 14983:                     // 'following' '#'
    case 14984:                     // 'following-sibling' '#'
    case 14985:                     // 'for' '#'
    case 14989:                     // 'ft-option' '#'
    case 14994:                     // 'ge' '#'
    case 14996:                     // 'group' '#'
    case 14998:                     // 'gt' '#'
    case 14999:                     // 'idiv' '#'
    case 15001:                     // 'import' '#'
    case 15002:                     // 'in' '#'
    case 15003:                     // 'index' '#'
    case 15007:                     // 'insert' '#'
    case 15008:                     // 'instance' '#'
    case 15009:                     // 'integrity' '#'
    case 15010:                     // 'intersect' '#'
    case 15011:                     // 'into' '#'
    case 15012:                     // 'is' '#'
    case 15014:                     // 'json' '#'
    case 15018:                     // 'last' '#'
    case 15019:                     // 'lax' '#'
    case 15020:                     // 'le' '#'
    case 15022:                     // 'let' '#'
    case 15024:                     // 'loop' '#'
    case 15026:                     // 'lt' '#'
    case 15028:                     // 'mod' '#'
    case 15029:                     // 'modify' '#'
    case 15030:                     // 'module' '#'
    case 15032:                     // 'namespace' '#'
    case 15034:                     // 'ne' '#'
    case 15040:                     // 'nodes' '#'
    case 15042:                     // 'object' '#'
    case 15046:                     // 'only' '#'
    case 15047:                     // 'option' '#'
    case 15048:                     // 'or' '#'
    case 15049:                     // 'order' '#'
    case 15050:                     // 'ordered' '#'
    case 15051:                     // 'ordering' '#'
    case 15054:                     // 'parent' '#'
    case 15060:                     // 'preceding' '#'
    case 15061:                     // 'preceding-sibling' '#'
    case 15064:                     // 'processing-instruction' '#'
    case 15066:                     // 'rename' '#'
    case 15067:                     // 'replace' '#'
    case 15068:                     // 'return' '#'
    case 15069:                     // 'returning' '#'
    case 15070:                     // 'revalidation' '#'
    case 15072:                     // 'satisfies' '#'
    case 15073:                     // 'schema' '#'
    case 15076:                     // 'score' '#'
    case 15077:                     // 'self' '#'
    case 15082:                     // 'sliding' '#'
    case 15083:                     // 'some' '#'
    case 15084:                     // 'stable' '#'
    case 15085:                     // 'start' '#'
    case 15088:                     // 'strict' '#'
    case 15092:                     // 'text' '#'
    case 15096:                     // 'to' '#'
    case 15097:                     // 'treat' '#'
    case 15098:                     // 'try' '#'
    case 15099:                     // 'tumbling' '#'
    case 15100:                     // 'type' '#'
    case 15102:                     // 'union' '#'
    case 15104:                     // 'unordered' '#'
    case 15105:                     // 'updating' '#'
    case 15108:                     // 'validate' '#'
    case 15109:                     // 'value' '#'
    case 15110:                     // 'variable' '#'
    case 15111:                     // 'version' '#'
    case 15114:                     // 'where' '#'
    case 15115:                     // 'while' '#'
    case 15118:                     // 'with' '#'
    case 15122:                     // 'xquery' '#'
      parse_FunctionItemExpr();
      break;
    case -10:
    case 27412:                     // '{' ';'
      parse_BlockExpr();
      break;
    case -11:
      parse_ObjectConstructor();
      break;
    case 68:                        // '['
      parse_ArrayConstructor();
      break;
    case 278:                       // '{|'
      parse_JSONSimpleObjectUnion();
      break;
    default:
      parse_Constructor();
    }
    eventHandler.endNonterminal("PrimaryExpr", e0);
  }

  function try_PrimaryExpr()
  {
    switch (l1)
    {
    case 184:                       // 'namespace'
      lookahead2W(256);             // NCName^Token | S^WS | '#' | '(' | '(:' | 'after' | 'allowing' | 'ancestor' |
      break;
    case 216:                       // 'processing-instruction'
      lookahead2W(254);             // NCName^Token | S^WS | '#' | '(:' | 'after' | 'allowing' | 'ancestor' |
      break;
    case 276:                       // '{'
      lookahead2W(277);             // Wildcard | EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral |
      break;
    case 82:                        // 'attribute'
    case 121:                       // 'element'
      lookahead2W(259);             // EQName^Token | S^WS | '#' | '(:' | 'after' | 'allowing' | 'ancestor' |
      break;
    case 96:                        // 'comment'
    case 244:                       // 'text'
      lookahead2W(93);              // S^WS | '#' | '(:' | '{'
      break;
    case 119:                       // 'document'
    case 202:                       // 'ordered'
    case 256:                       // 'unordered'
      lookahead2W(139);             // S^WS | '#' | '(' | '(:' | '{'
      break;
    case 6:                         // EQName^Token
    case 70:                        // 'after'
    case 72:                        // 'allowing'
    case 73:                        // 'ancestor'
    case 74:                        // 'ancestor-or-self'
    case 75:                        // 'and'
    case 77:                        // 'append'
    case 79:                        // 'as'
    case 80:                        // 'ascending'
    case 81:                        // 'at'
    case 83:                        // 'base-uri'
    case 84:                        // 'before'
    case 85:                        // 'boundary-space'
    case 86:                        // 'break'
    case 88:                        // 'case'
    case 89:                        // 'cast'
    case 90:                        // 'castable'
    case 91:                        // 'catch'
    case 93:                        // 'child'
    case 94:                        // 'collation'
    case 97:                        // 'constraint'
    case 98:                        // 'construction'
    case 101:                       // 'context'
    case 102:                       // 'continue'
    case 103:                       // 'copy'
    case 104:                       // 'copy-namespaces'
    case 105:                       // 'count'
    case 106:                       // 'decimal-format'
    case 108:                       // 'declare'
    case 109:                       // 'default'
    case 110:                       // 'delete'
    case 111:                       // 'descendant'
    case 112:                       // 'descendant-or-self'
    case 113:                       // 'descending'
    case 118:                       // 'div'
    case 122:                       // 'else'
    case 123:                       // 'empty'
    case 125:                       // 'encoding'
    case 126:                       // 'end'
    case 128:                       // 'eq'
    case 129:                       // 'every'
    case 131:                       // 'except'
    case 132:                       // 'exit'
    case 133:                       // 'external'
    case 134:                       // 'first'
    case 135:                       // 'following'
    case 136:                       // 'following-sibling'
    case 137:                       // 'for'
    case 141:                       // 'ft-option'
    case 146:                       // 'ge'
    case 148:                       // 'group'
    case 150:                       // 'gt'
    case 151:                       // 'idiv'
    case 153:                       // 'import'
    case 154:                       // 'in'
    case 155:                       // 'index'
    case 159:                       // 'insert'
    case 160:                       // 'instance'
    case 161:                       // 'integrity'
    case 162:                       // 'intersect'
    case 163:                       // 'into'
    case 164:                       // 'is'
    case 166:                       // 'json'
    case 170:                       // 'last'
    case 171:                       // 'lax'
    case 172:                       // 'le'
    case 174:                       // 'let'
    case 176:                       // 'loop'
    case 178:                       // 'lt'
    case 180:                       // 'mod'
    case 181:                       // 'modify'
    case 182:                       // 'module'
    case 186:                       // 'ne'
    case 192:                       // 'nodes'
    case 194:                       // 'object'
    case 198:                       // 'only'
    case 199:                       // 'option'
    case 200:                       // 'or'
    case 201:                       // 'order'
    case 203:                       // 'ordering'
    case 206:                       // 'parent'
    case 212:                       // 'preceding'
    case 213:                       // 'preceding-sibling'
    case 218:                       // 'rename'
    case 219:                       // 'replace'
    case 220:                       // 'return'
    case 221:                       // 'returning'
    case 222:                       // 'revalidation'
    case 224:                       // 'satisfies'
    case 225:                       // 'schema'
    case 228:                       // 'score'
    case 229:                       // 'self'
    case 234:                       // 'sliding'
    case 235:                       // 'some'
    case 236:                       // 'stable'
    case 237:                       // 'start'
    case 240:                       // 'strict'
    case 248:                       // 'to'
    case 249:                       // 'treat'
    case 250:                       // 'try'
    case 251:                       // 'tumbling'
    case 252:                       // 'type'
    case 254:                       // 'union'
    case 257:                       // 'updating'
    case 260:                       // 'validate'
    case 261:                       // 'value'
    case 262:                       // 'variable'
    case 263:                       // 'version'
    case 266:                       // 'where'
    case 267:                       // 'while'
    case 270:                       // 'with'
    case 274:                       // 'xquery'
      lookahead2W(92);              // S^WS | '#' | '(' | '(:'
      break;
    default:
      lk = l1;
    }
    if (lk == 2836                  // '{' Wildcard
     || lk == 3348                  // '{' EQName^Token
     || lk == 4372                  // '{' IntegerLiteral
     || lk == 4884                  // '{' DecimalLiteral
     || lk == 5396                  // '{' DoubleLiteral
     || lk == 5908                  // '{' StringLiteral
     || lk == 16148                 // '{' '$'
     || lk == 16660                 // '{' '%'
     || lk == 17684                 // '{' '('
     || lk == 18196                 // '{' '(#'
     || lk == 20756                 // '{' '+'
     || lk == 21780                 // '{' '-'
     || lk == 22804                 // '{' '.'
     || lk == 23316                 // '{' '..'
     || lk == 23828                 // '{' '/'
     || lk == 24340                 // '{' '//'
     || lk == 27924                 // '{' '<'
     || lk == 28436                 // '{' '<!--'
     || lk == 30484                 // '{' '<?'
     || lk == 34068                 // '{' '@'
     || lk == 35092                 // '{' '['
     || lk == 36116                 // '{' 'after'
     || lk == 37140                 // '{' 'allowing'
     || lk == 37652                 // '{' 'ancestor'
     || lk == 38164                 // '{' 'ancestor-or-self'
     || lk == 38676                 // '{' 'and'
     || lk == 39700                 // '{' 'append'
     || lk == 40212                 // '{' 'array'
     || lk == 40724                 // '{' 'as'
     || lk == 41236                 // '{' 'ascending'
     || lk == 41748                 // '{' 'at'
     || lk == 42260                 // '{' 'attribute'
     || lk == 42772                 // '{' 'base-uri'
     || lk == 43284                 // '{' 'before'
     || lk == 43796                 // '{' 'boundary-space'
     || lk == 44308                 // '{' 'break'
     || lk == 45332                 // '{' 'case'
     || lk == 45844                 // '{' 'cast'
     || lk == 46356                 // '{' 'castable'
     || lk == 46868                 // '{' 'catch'
     || lk == 47892                 // '{' 'child'
     || lk == 48404                 // '{' 'collation'
     || lk == 49428                 // '{' 'comment'
     || lk == 49940                 // '{' 'constraint'
     || lk == 50452                 // '{' 'construction'
     || lk == 51988                 // '{' 'context'
     || lk == 52500                 // '{' 'continue'
     || lk == 53012                 // '{' 'copy'
     || lk == 53524                 // '{' 'copy-namespaces'
     || lk == 54036                 // '{' 'count'
     || lk == 54548                 // '{' 'decimal-format'
     || lk == 55572                 // '{' 'declare'
     || lk == 56084                 // '{' 'default'
     || lk == 56596                 // '{' 'delete'
     || lk == 57108                 // '{' 'descendant'
     || lk == 57620                 // '{' 'descendant-or-self'
     || lk == 58132                 // '{' 'descending'
     || lk == 60692                 // '{' 'div'
     || lk == 61204                 // '{' 'document'
     || lk == 61716                 // '{' 'document-node'
     || lk == 62228                 // '{' 'element'
     || lk == 62740                 // '{' 'else'
     || lk == 63252                 // '{' 'empty'
     || lk == 63764                 // '{' 'empty-sequence'
     || lk == 64276                 // '{' 'encoding'
     || lk == 64788                 // '{' 'end'
     || lk == 65812                 // '{' 'eq'
     || lk == 66324                 // '{' 'every'
     || lk == 67348                 // '{' 'except'
     || lk == 67860                 // '{' 'exit'
     || lk == 68372                 // '{' 'external'
     || lk == 68884                 // '{' 'first'
     || lk == 69396                 // '{' 'following'
     || lk == 69908                 // '{' 'following-sibling'
     || lk == 70420                 // '{' 'for'
     || lk == 72468                 // '{' 'ft-option'
     || lk == 74516                 // '{' 'function'
     || lk == 75028                 // '{' 'ge'
     || lk == 76052                 // '{' 'group'
     || lk == 77076                 // '{' 'gt'
     || lk == 77588                 // '{' 'idiv'
     || lk == 78100                 // '{' 'if'
     || lk == 78612                 // '{' 'import'
     || lk == 79124                 // '{' 'in'
     || lk == 79636                 // '{' 'index'
     || lk == 81684                 // '{' 'insert'
     || lk == 82196                 // '{' 'instance'
     || lk == 82708                 // '{' 'integrity'
     || lk == 83220                 // '{' 'intersect'
     || lk == 83732                 // '{' 'into'
     || lk == 84244                 // '{' 'is'
     || lk == 84756                 // '{' 'item'
     || lk == 85268                 // '{' 'json'
     || lk == 85780                 // '{' 'json-item'
     || lk == 87316                 // '{' 'last'
     || lk == 87828                 // '{' 'lax'
     || lk == 88340                 // '{' 'le'
     || lk == 89364                 // '{' 'let'
     || lk == 90388                 // '{' 'loop'
     || lk == 91412                 // '{' 'lt'
     || lk == 92436                 // '{' 'mod'
     || lk == 92948                 // '{' 'modify'
     || lk == 93460                 // '{' 'module'
     || lk == 94484                 // '{' 'namespace'
     || lk == 94996                 // '{' 'namespace-node'
     || lk == 95508                 // '{' 'ne'
     || lk == 98068                 // '{' 'node'
     || lk == 98580                 // '{' 'nodes'
     || lk == 99604                 // '{' 'object'
     || lk == 101652                // '{' 'only'
     || lk == 102164                // '{' 'option'
     || lk == 102676                // '{' 'or'
     || lk == 103188                // '{' 'order'
     || lk == 103700                // '{' 'ordered'
     || lk == 104212                // '{' 'ordering'
     || lk == 105748                // '{' 'parent'
     || lk == 108820                // '{' 'preceding'
     || lk == 109332                // '{' 'preceding-sibling'
     || lk == 110868                // '{' 'processing-instruction'
     || lk == 111892                // '{' 'rename'
     || lk == 112404                // '{' 'replace'
     || lk == 112916                // '{' 'return'
     || lk == 113428                // '{' 'returning'
     || lk == 113940                // '{' 'revalidation'
     || lk == 114964                // '{' 'satisfies'
     || lk == 115476                // '{' 'schema'
     || lk == 115988                // '{' 'schema-attribute'
     || lk == 116500                // '{' 'schema-element'
     || lk == 117012                // '{' 'score'
     || lk == 117524                // '{' 'self'
     || lk == 120084                // '{' 'sliding'
     || lk == 120596                // '{' 'some'
     || lk == 121108                // '{' 'stable'
     || lk == 121620                // '{' 'start'
     || lk == 123156                // '{' 'strict'
     || lk == 124180                // '{' 'structured-item'
     || lk == 124692                // '{' 'switch'
     || lk == 125204                // '{' 'text'
     || lk == 127252                // '{' 'to'
     || lk == 127764                // '{' 'treat'
     || lk == 128276                // '{' 'try'
     || lk == 128788                // '{' 'tumbling'
     || lk == 129300                // '{' 'type'
     || lk == 129812                // '{' 'typeswitch'
     || lk == 130324                // '{' 'union'
     || lk == 131348                // '{' 'unordered'
     || lk == 131860                // '{' 'updating'
     || lk == 133396                // '{' 'validate'
     || lk == 133908                // '{' 'value'
     || lk == 134420                // '{' 'variable'
     || lk == 134932                // '{' 'version'
     || lk == 136468                // '{' 'where'
     || lk == 136980                // '{' 'while'
     || lk == 138516                // '{' 'with'
     || lk == 140564                // '{' 'xquery'
     || lk == 141588                // '{' '{'
     || lk == 142612                // '{' '{|'
     || lk == 144660)               // '{' '}'
    {
      lk = memoized(14, e0);
      if (lk == 0)
      {
        var b0A = b0; var e0A = e0; var l1A = l1;
        var b1A = b1; var e1A = e1; var l2A = l2;
        var b2A = b2; var e2A = e2;
        try
        {
          try_BlockExpr();
          memoize(14, e0A, -10);
          lk = -14;
        }
        catch (p10A)
        {
          lk = -11;
          b0 = b0A; e0 = e0A; l1 = l1A; if (l1 == 0) {end = e0A;} else {
          b1 = b1A; e1 = e1A; l2 = l2A; if (l2 == 0) {end = e1A;} else {
          b2 = b2A; e2 = e2A; end = e2A; }}
          memoize(14, e0A, -11);
        }
      }
    }
    switch (lk)
    {
    case 8:                         // IntegerLiteral
    case 9:                         // DecimalLiteral
    case 10:                        // DoubleLiteral
    case 11:                        // StringLiteral
      try_Literal();
      break;
    case 31:                        // '$'
      try_VarRef();
      break;
    case 34:                        // '('
      try_ParenthesizedExpr();
      break;
    case 44:                        // '.'
      try_ContextItemExpr();
      break;
    case 17414:                     // EQName^Token '('
    case 17478:                     // 'after' '('
    case 17480:                     // 'allowing' '('
    case 17481:                     // 'ancestor' '('
    case 17482:                     // 'ancestor-or-self' '('
    case 17483:                     // 'and' '('
    case 17485:                     // 'append' '('
    case 17487:                     // 'as' '('
    case 17488:                     // 'ascending' '('
    case 17489:                     // 'at' '('
    case 17491:                     // 'base-uri' '('
    case 17492:                     // 'before' '('
    case 17493:                     // 'boundary-space' '('
    case 17494:                     // 'break' '('
    case 17496:                     // 'case' '('
    case 17497:                     // 'cast' '('
    case 17498:                     // 'castable' '('
    case 17499:                     // 'catch' '('
    case 17501:                     // 'child' '('
    case 17502:                     // 'collation' '('
    case 17505:                     // 'constraint' '('
    case 17506:                     // 'construction' '('
    case 17509:                     // 'context' '('
    case 17510:                     // 'continue' '('
    case 17511:                     // 'copy' '('
    case 17512:                     // 'copy-namespaces' '('
    case 17513:                     // 'count' '('
    case 17514:                     // 'decimal-format' '('
    case 17516:                     // 'declare' '('
    case 17517:                     // 'default' '('
    case 17518:                     // 'delete' '('
    case 17519:                     // 'descendant' '('
    case 17520:                     // 'descendant-or-self' '('
    case 17521:                     // 'descending' '('
    case 17526:                     // 'div' '('
    case 17527:                     // 'document' '('
    case 17530:                     // 'else' '('
    case 17531:                     // 'empty' '('
    case 17533:                     // 'encoding' '('
    case 17534:                     // 'end' '('
    case 17536:                     // 'eq' '('
    case 17537:                     // 'every' '('
    case 17539:                     // 'except' '('
    case 17540:                     // 'exit' '('
    case 17541:                     // 'external' '('
    case 17542:                     // 'first' '('
    case 17543:                     // 'following' '('
    case 17544:                     // 'following-sibling' '('
    case 17545:                     // 'for' '('
    case 17549:                     // 'ft-option' '('
    case 17554:                     // 'ge' '('
    case 17556:                     // 'group' '('
    case 17558:                     // 'gt' '('
    case 17559:                     // 'idiv' '('
    case 17561:                     // 'import' '('
    case 17562:                     // 'in' '('
    case 17563:                     // 'index' '('
    case 17567:                     // 'insert' '('
    case 17568:                     // 'instance' '('
    case 17569:                     // 'integrity' '('
    case 17570:                     // 'intersect' '('
    case 17571:                     // 'into' '('
    case 17572:                     // 'is' '('
    case 17574:                     // 'json' '('
    case 17578:                     // 'last' '('
    case 17579:                     // 'lax' '('
    case 17580:                     // 'le' '('
    case 17582:                     // 'let' '('
    case 17584:                     // 'loop' '('
    case 17586:                     // 'lt' '('
    case 17588:                     // 'mod' '('
    case 17589:                     // 'modify' '('
    case 17590:                     // 'module' '('
    case 17592:                     // 'namespace' '('
    case 17594:                     // 'ne' '('
    case 17600:                     // 'nodes' '('
    case 17602:                     // 'object' '('
    case 17606:                     // 'only' '('
    case 17607:                     // 'option' '('
    case 17608:                     // 'or' '('
    case 17609:                     // 'order' '('
    case 17610:                     // 'ordered' '('
    case 17611:                     // 'ordering' '('
    case 17614:                     // 'parent' '('
    case 17620:                     // 'preceding' '('
    case 17621:                     // 'preceding-sibling' '('
    case 17626:                     // 'rename' '('
    case 17627:                     // 'replace' '('
    case 17628:                     // 'return' '('
    case 17629:                     // 'returning' '('
    case 17630:                     // 'revalidation' '('
    case 17632:                     // 'satisfies' '('
    case 17633:                     // 'schema' '('
    case 17636:                     // 'score' '('
    case 17637:                     // 'self' '('
    case 17642:                     // 'sliding' '('
    case 17643:                     // 'some' '('
    case 17644:                     // 'stable' '('
    case 17645:                     // 'start' '('
    case 17648:                     // 'strict' '('
    case 17656:                     // 'to' '('
    case 17657:                     // 'treat' '('
    case 17658:                     // 'try' '('
    case 17659:                     // 'tumbling' '('
    case 17660:                     // 'type' '('
    case 17662:                     // 'union' '('
    case 17664:                     // 'unordered' '('
    case 17665:                     // 'updating' '('
    case 17668:                     // 'validate' '('
    case 17669:                     // 'value' '('
    case 17670:                     // 'variable' '('
    case 17671:                     // 'version' '('
    case 17674:                     // 'where' '('
    case 17675:                     // 'while' '('
    case 17678:                     // 'with' '('
    case 17682:                     // 'xquery' '('
      try_FunctionCall();
      break;
    case 141514:                    // 'ordered' '{'
      try_OrderedExpr();
      break;
    case 141568:                    // 'unordered' '{'
      try_UnorderedExpr();
      break;
    case 32:                        // '%'
    case 78:                        // 'array'
    case 120:                       // 'document-node'
    case 124:                       // 'empty-sequence'
    case 145:                       // 'function'
    case 152:                       // 'if'
    case 165:                       // 'item'
    case 167:                       // 'json-item'
    case 185:                       // 'namespace-node'
    case 191:                       // 'node'
    case 226:                       // 'schema-attribute'
    case 227:                       // 'schema-element'
    case 242:                       // 'structured-item'
    case 243:                       // 'switch'
    case 253:                       // 'typeswitch'
    case 14854:                     // EQName^Token '#'
    case 14918:                     // 'after' '#'
    case 14920:                     // 'allowing' '#'
    case 14921:                     // 'ancestor' '#'
    case 14922:                     // 'ancestor-or-self' '#'
    case 14923:                     // 'and' '#'
    case 14925:                     // 'append' '#'
    case 14927:                     // 'as' '#'
    case 14928:                     // 'ascending' '#'
    case 14929:                     // 'at' '#'
    case 14930:                     // 'attribute' '#'
    case 14931:                     // 'base-uri' '#'
    case 14932:                     // 'before' '#'
    case 14933:                     // 'boundary-space' '#'
    case 14934:                     // 'break' '#'
    case 14936:                     // 'case' '#'
    case 14937:                     // 'cast' '#'
    case 14938:                     // 'castable' '#'
    case 14939:                     // 'catch' '#'
    case 14941:                     // 'child' '#'
    case 14942:                     // 'collation' '#'
    case 14944:                     // 'comment' '#'
    case 14945:                     // 'constraint' '#'
    case 14946:                     // 'construction' '#'
    case 14949:                     // 'context' '#'
    case 14950:                     // 'continue' '#'
    case 14951:                     // 'copy' '#'
    case 14952:                     // 'copy-namespaces' '#'
    case 14953:                     // 'count' '#'
    case 14954:                     // 'decimal-format' '#'
    case 14956:                     // 'declare' '#'
    case 14957:                     // 'default' '#'
    case 14958:                     // 'delete' '#'
    case 14959:                     // 'descendant' '#'
    case 14960:                     // 'descendant-or-self' '#'
    case 14961:                     // 'descending' '#'
    case 14966:                     // 'div' '#'
    case 14967:                     // 'document' '#'
    case 14969:                     // 'element' '#'
    case 14970:                     // 'else' '#'
    case 14971:                     // 'empty' '#'
    case 14973:                     // 'encoding' '#'
    case 14974:                     // 'end' '#'
    case 14976:                     // 'eq' '#'
    case 14977:                     // 'every' '#'
    case 14979:                     // 'except' '#'
    case 14980:                     // 'exit' '#'
    case 14981:                     // 'external' '#'
    case 14982:                     // 'first' '#'
    case 14983:                     // 'following' '#'
    case 14984:                     // 'following-sibling' '#'
    case 14985:                     // 'for' '#'
    case 14989:                     // 'ft-option' '#'
    case 14994:                     // 'ge' '#'
    case 14996:                     // 'group' '#'
    case 14998:                     // 'gt' '#'
    case 14999:                     // 'idiv' '#'
    case 15001:                     // 'import' '#'
    case 15002:                     // 'in' '#'
    case 15003:                     // 'index' '#'
    case 15007:                     // 'insert' '#'
    case 15008:                     // 'instance' '#'
    case 15009:                     // 'integrity' '#'
    case 15010:                     // 'intersect' '#'
    case 15011:                     // 'into' '#'
    case 15012:                     // 'is' '#'
    case 15014:                     // 'json' '#'
    case 15018:                     // 'last' '#'
    case 15019:                     // 'lax' '#'
    case 15020:                     // 'le' '#'
    case 15022:                     // 'let' '#'
    case 15024:                     // 'loop' '#'
    case 15026:                     // 'lt' '#'
    case 15028:                     // 'mod' '#'
    case 15029:                     // 'modify' '#'
    case 15030:                     // 'module' '#'
    case 15032:                     // 'namespace' '#'
    case 15034:                     // 'ne' '#'
    case 15040:                     // 'nodes' '#'
    case 15042:                     // 'object' '#'
    case 15046:                     // 'only' '#'
    case 15047:                     // 'option' '#'
    case 15048:                     // 'or' '#'
    case 15049:                     // 'order' '#'
    case 15050:                     // 'ordered' '#'
    case 15051:                     // 'ordering' '#'
    case 15054:                     // 'parent' '#'
    case 15060:                     // 'preceding' '#'
    case 15061:                     // 'preceding-sibling' '#'
    case 15064:                     // 'processing-instruction' '#'
    case 15066:                     // 'rename' '#'
    case 15067:                     // 'replace' '#'
    case 15068:                     // 'return' '#'
    case 15069:                     // 'returning' '#'
    case 15070:                     // 'revalidation' '#'
    case 15072:                     // 'satisfies' '#'
    case 15073:                     // 'schema' '#'
    case 15076:                     // 'score' '#'
    case 15077:                     // 'self' '#'
    case 15082:                     // 'sliding' '#'
    case 15083:                     // 'some' '#'
    case 15084:                     // 'stable' '#'
    case 15085:                     // 'start' '#'
    case 15088:                     // 'strict' '#'
    case 15092:                     // 'text' '#'
    case 15096:                     // 'to' '#'
    case 15097:                     // 'treat' '#'
    case 15098:                     // 'try' '#'
    case 15099:                     // 'tumbling' '#'
    case 15100:                     // 'type' '#'
    case 15102:                     // 'union' '#'
    case 15104:                     // 'unordered' '#'
    case 15105:                     // 'updating' '#'
    case 15108:                     // 'validate' '#'
    case 15109:                     // 'value' '#'
    case 15110:                     // 'variable' '#'
    case 15111:                     // 'version' '#'
    case 15114:                     // 'where' '#'
    case 15115:                     // 'while' '#'
    case 15118:                     // 'with' '#'
    case 15122:                     // 'xquery' '#'
      try_FunctionItemExpr();
      break;
    case -10:
    case 27412:                     // '{' ';'
      try_BlockExpr();
      break;
    case -11:
      try_ObjectConstructor();
      break;
    case 68:                        // '['
      try_ArrayConstructor();
      break;
    case 278:                       // '{|'
      try_JSONSimpleObjectUnion();
      break;
    case -14:
      break;
    default:
      try_Constructor();
    }
  }

  function parse_JSONSimpleObjectUnion()
  {
    eventHandler.startNonterminal("JSONSimpleObjectUnion", e0);
    shift(278);                     // '{|'
    lookahead1W(273);               // Wildcard | EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral |
    if (l1 != 281)                  // '|}'
    {
      whitespace();
      parse_Expr();
    }
    shift(281);                     // '|}'
    eventHandler.endNonterminal("JSONSimpleObjectUnion", e0);
  }

  function try_JSONSimpleObjectUnion()
  {
    shiftT(278);                    // '{|'
    lookahead1W(273);               // Wildcard | EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral |
    if (l1 != 281)                  // '|}'
    {
      try_Expr();
    }
    shiftT(281);                    // '|}'
  }

  function parse_ObjectConstructor()
  {
    eventHandler.startNonterminal("ObjectConstructor", e0);
    shift(276);                     // '{'
    lookahead1W(274);               // Wildcard | EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral |
    if (l1 != 282)                  // '}'
    {
      whitespace();
      parse_PairConstructorList();
    }
    shift(282);                     // '}'
    eventHandler.endNonterminal("ObjectConstructor", e0);
  }

  function try_ObjectConstructor()
  {
    shiftT(276);                    // '{'
    lookahead1W(274);               // Wildcard | EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral |
    if (l1 != 282)                  // '}'
    {
      try_PairConstructorList();
    }
    shiftT(282);                    // '}'
  }

  function parse_PairConstructorList()
  {
    eventHandler.startNonterminal("PairConstructorList", e0);
    parse_PairConstructor();
    for (;;)
    {
      if (l1 != 41)                 // ','
      {
        break;
      }
      shift(41);                    // ','
      lookahead1W(267);             // Wildcard | EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral |
      whitespace();
      parse_PairConstructor();
    }
    eventHandler.endNonterminal("PairConstructorList", e0);
  }

  function try_PairConstructorList()
  {
    try_PairConstructor();
    for (;;)
    {
      if (l1 != 41)                 // ','
      {
        break;
      }
      shiftT(41);                   // ','
      lookahead1W(267);             // Wildcard | EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral |
      try_PairConstructor();
    }
  }

  function parse_PairConstructor()
  {
    eventHandler.startNonterminal("PairConstructor", e0);
    parse_ExprSingle();
    shift(49);                      // ':'
    lookahead1W(267);               // Wildcard | EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral |
    whitespace();
    parse_ExprSingle();
    eventHandler.endNonterminal("PairConstructor", e0);
  }

  function try_PairConstructor()
  {
    try_ExprSingle();
    shiftT(49);                     // ':'
    lookahead1W(267);               // Wildcard | EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral |
    try_ExprSingle();
  }

  function parse_ArrayConstructor()
  {
    eventHandler.startNonterminal("ArrayConstructor", e0);
    shift(68);                      // '['
    lookahead1W(272);               // Wildcard | EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral |
    if (l1 != 69)                   // ']'
    {
      whitespace();
      parse_Expr();
    }
    shift(69);                      // ']'
    eventHandler.endNonterminal("ArrayConstructor", e0);
  }

  function try_ArrayConstructor()
  {
    shiftT(68);                     // '['
    lookahead1W(272);               // Wildcard | EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral |
    if (l1 != 69)                   // ']'
    {
      try_Expr();
    }
    shiftT(69);                     // ']'
  }

  function parse_BlockExpr()
  {
    eventHandler.startNonterminal("BlockExpr", e0);
    shift(276);                     // '{'
    lookahead1W(277);               // Wildcard | EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral |
    whitespace();
    parse_StatementsAndOptionalExpr();
    shift(282);                     // '}'
    eventHandler.endNonterminal("BlockExpr", e0);
  }

  function try_BlockExpr()
  {
    shiftT(276);                    // '{'
    lookahead1W(277);               // Wildcard | EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral |
    try_StatementsAndOptionalExpr();
    shiftT(282);                    // '}'
  }

  function parse_FunctionDecl()
  {
    eventHandler.startNonterminal("FunctionDecl", e0);
    shift(145);                     // 'function'
    lookahead1W(255);               // EQName^Token | S^WS | '(:' | 'after' | 'allowing' | 'ancestor' |
    whitespace();
    parse_EQName();
    lookahead1W(22);                // S^WS | '(' | '(:'
    shift(34);                      // '('
    lookahead1W(94);                // S^WS | '$' | '(:' | ')'
    if (l1 == 31)                   // '$'
    {
      whitespace();
      parse_ParamList();
    }
    shift(37);                      // ')'
    lookahead1W(148);               // S^WS | '(:' | 'as' | 'external' | '{'
    if (l1 == 79)                   // 'as'
    {
      whitespace();
      parse_ReturnType();
    }
    lookahead1W(118);               // S^WS | '(:' | 'external' | '{'
    switch (l1)
    {
    case 276:                       // '{'
      shift(276);                   // '{'
      lookahead1W(277);             // Wildcard | EQName^Token | IntegerLiteral | DecimalLiteral | DoubleLiteral |
      whitespace();
      parse_StatementsAndOptionalExpr();
      shift(282);                   // '}'
      break;
    default:
      shift(133);                   // 'external'
    }
    eventHandler.endNonterminal("FunctionDecl", e0);
  }

  function parse_ReturnType()
  {
    eventHandler.startNonterminal("ReturnType", e0);
    shift(79);                      // 'as'
    lookahead1W(260);               // EQName^Token | S^WS | '%' | '(' | '(:' | 'after' | 'allowing' | 'ancestor' |
    whitespace();
    parse_SequenceType();
    eventHandler.endNonterminal("ReturnType", e0);
  }

  function shift(t)
  {
    if (l1 == t)
    {
      whitespace();
      eventHandler.terminal(XQueryParser.TOKEN[l1], b1, e1 > size ? size : e1);
      b0 = b1; e0 = e1; l1 = l2; if (l1 != 0) {
      b1 = b2; e1 = e2; l2 = 0; }
    }
    else
    {
      error(b1, e1, 0, l1, t);
    }
  }

  function shiftT(t)
  {
    if (l1 == t)
    {
      b0 = b1; e0 = e1; l1 = l2; if (l1 != 0) {
      b1 = b2; e1 = e2; l2 = 0; }
    }
    else
    {
      error(b1, e1, 0, l1, t);
    }
  }

  function skip(code)
  {
    var b0W = b0; var e0W = e0; var l1W = l1;
    var b1W = b1; var e1W = e1;

    l1 = code; b1 = begin; e1 = end;
    l2 = 0;

    try_Whitespace();

    b0 = b0W; e0 = e0W; l1 = l1W; if (l1 != 0) {
    b1 = b1W; e1 = e1W; }
  }

  function whitespace()
  {
    if (e0 != b1)
    {
      b0 = e0;
      e0 = b1;
      eventHandler.whitespace(b0, e0);
    }
  }

  function matchW(set)
  {
    var code;
    for (;;)
    {
      code = match(set);
      if (code != 22)               // S^WS
      {
        if (code != 36)             // '(:'
        {
          break;
        }
        skip(code);
      }
    }
    return code;
  }

  function lookahead1W(set)
  {
    if (l1 == 0)
    {
      l1 = matchW(set);
      b1 = begin;
      e1 = end;
    }
  }

  function lookahead2W(set)
  {
    if (l2 == 0)
    {
      l2 = matchW(set);
      b2 = begin;
      e2 = end;
    }
    lk = (l2 << 9) | l1;
  }

  function lookahead1(set)
  {
    if (l1 == 0)
    {
      l1 = match(set);
      b1 = begin;
      e1 = end;
    }
  }

  function lookahead2(set)
  {
    if (l2 == 0)
    {
      l2 = match(set);
      b2 = begin;
      e2 = end;
    }
    lk = (l2 << 9) | l1;
  }

  function error(b, e, s, l, t)
  {
    if (e > ex)
    {
      bx = b;
      ex = e;
      sx = s;
      lx = l;
      tx = t;
    }
    throw new self.ParseException(bx, ex, sx, lx, tx);
  }

  var lk, b0, e0;
  var l1, b1, e1;
  var l2, b2, e2;
  var bx, ex, sx, lx, tx;
  var eventHandler;
  var memo;

  function memoize(i, e, v)
  {
    memo[(e << 4) + i] = v;
  }

  function memoized(i, e)
  {
    var v = memo[(e << 4) + i];
    return typeof v != "undefined" ? v : 0;
  }

  var input;
  var size;
  var begin;
  var end;

  function match(tokenSetId)
  {
    var nonbmp = false;
    begin = end;
    var current = end;
    var result = XQueryParser.INITIAL[tokenSetId];
    var state = 0;

    for (var code = result & 4095; code != 0; )
    {
      var charclass;
      var c0 = current < size ? input.charCodeAt(current) : 0;
      ++current;
      if (c0 < 0x80)
      {
        charclass = XQueryParser.MAP0[c0];
      }
      else if (c0 < 0xd800)
      {
        var c1 = c0 >> 4;
        charclass = XQueryParser.MAP1[(c0 & 15) + XQueryParser.MAP1[(c1 & 31) + XQueryParser.MAP1[c1 >> 5]]];
      }
      else
      {
        if (c0 < 0xdc00)
        {
          var c1 = current < size ? input.charCodeAt(current) : 0;
          if (c1 >= 0xdc00 && c1 < 0xe000)
          {
            ++current;
            c0 = ((c0 & 0x3ff) << 10) + (c1 & 0x3ff) + 0x10000;
            nonbmp = true;
          }
        }
        var lo = 0, hi = 5;
        for (var m = 3; ; m = (hi + lo) >> 1)
        {
          if (XQueryParser.MAP2[m] > c0) hi = m - 1;
          else if (XQueryParser.MAP2[6 + m] < c0) lo = m + 1;
          else {charclass = XQueryParser.MAP2[12 + m]; break;}
          if (lo > hi) {charclass = 0; break;}
        }
      }

      state = code;
      var i0 = (charclass << 12) + code - 1;
      code = XQueryParser.TRANSITION[(i0 & 15) + XQueryParser.TRANSITION[i0 >> 4]];

      if (code > 4095)
      {
        result = code;
        code &= 4095;
        end = current;
      }
    }

    result >>= 12;
    if (result == 0)
    {
      end = current - 1;
      var c1 = end < size ? input.charCodeAt(end) : 0;
      if (c1 >= 0xdc00 && c1 < 0xe000) --end;
      return error(begin, end, state, -1, -1);
    }

    if (nonbmp)
    {
      for (var i = result >> 9; i > 0; --i)
      {
        --end;
        var c1 = end < size ? input.charCodeAt(end) : 0;
        if (c1 >= 0xdc00 && c1 < 0xe000) --end;
      }
    }
    else
    {
      end -= result >> 9;
    }

    return (result & 511) - 1;
  }
}

XQueryParser.getTokenSet = function(tokenSetId)
{
  var set = [];
  var s = tokenSetId < 0 ? - tokenSetId : INITIAL[tokenSetId] & 4095;
  for (var i = 0; i < 284; i += 32)
  {
    var j = i;
    var i0 = (i >> 5) * 3694 + s - 1;
    var i1 = i0 >> 1;
    var i2 = i1 >> 2;
    var f = XQueryParser.EXPECTED[(i0 & 1) + XQueryParser.EXPECTED[(i1 & 3) + XQueryParser.EXPECTED[(i2 & 3) + XQueryParser.EXPECTED[i2 >> 2]]]];
    for ( ; f != 0; f >>>= 1, ++j)
    {
      if ((f & 1) != 0)
      {
        set.push(XQueryParser.TOKEN[j]);
      }
    }
  }
  return set;
};

XQueryParser.MAP0 =
[ 70, 0, 0, 0, 0, 0, 0, 0, 0, 1, 1, 0, 0, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 17, 17, 17, 17, 17, 17, 17, 17, 17, 18, 19, 20, 21, 22, 23, 24, 25, 26, 27, 28, 29, 26, 30, 30, 30, 30, 30, 31, 32, 33, 30, 30, 34, 30, 30, 35, 30, 30, 30, 36, 30, 30, 37, 38, 39, 38, 30, 38, 40, 41, 42, 43, 44, 45, 46, 47, 48, 49, 50, 51, 52, 53, 54, 55, 56, 57, 58, 59, 60, 61, 62, 63, 64, 65, 66, 67, 68, 38, 38
];

XQueryParser.MAP1 =
[ 108, 124, 214, 214, 214, 214, 214, 214, 214, 214, 214, 214, 214, 214, 214, 214, 156, 181, 181, 181, 181, 181, 214, 215, 213, 214, 214, 214, 214, 214, 214, 214, 214, 214, 214, 214, 214, 214, 214, 214, 214, 214, 214, 214, 214, 214, 214, 214, 214, 214, 214, 214, 214, 214, 214, 214, 214, 214, 214, 214, 214, 214, 214, 214, 214, 214, 214, 214, 214, 214, 214, 214, 214, 214, 214, 214, 214, 214, 214, 214, 214, 214, 214, 214, 214, 214, 214, 214, 214, 214, 214, 214, 214, 214, 214, 214, 214, 214, 214, 214, 214, 214, 214, 214, 214, 214, 214, 214, 247, 261, 277, 293, 309, 355, 371, 387, 423, 423, 423, 415, 339, 331, 339, 331, 339, 339, 339, 339, 339, 339, 339, 339, 339, 339, 339, 339, 339, 339, 339, 339, 440, 440, 440, 440, 440, 440, 440, 324, 339, 339, 339, 339, 339, 339, 339, 339, 401, 423, 423, 424, 422, 423, 423, 339, 339, 339, 339, 339, 339, 339, 339, 339, 339, 339, 339, 339, 339, 339, 339, 339, 339, 423, 423, 423, 423, 423, 423, 423, 423, 423, 423, 423, 423, 423, 423, 423, 423, 423, 423, 423, 423, 423, 423, 423, 423, 423, 423, 423, 423, 423, 423, 423, 423, 338, 339, 339, 339, 339, 339, 339, 339, 339, 339, 339, 339, 339, 339, 339, 339, 339, 339, 339, 339, 339, 339, 339, 339, 339, 339, 339, 339, 339, 339, 339, 339, 339, 423, 70, 0, 0, 0, 0, 0, 0, 0, 0, 1, 1, 0, 0, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 17, 17, 17, 17, 17, 17, 17, 17, 17, 18, 19, 20, 21, 22, 23, 24, 25, 26, 27, 28, 29, 26, 30, 30, 30, 30, 30, 31, 32, 33, 30, 30, 30, 30, 30, 30, 30, 30, 30, 30, 30, 30, 30, 30, 38, 30, 30, 30, 30, 30, 30, 30, 30, 30, 30, 30, 30, 30, 30, 30, 30, 30, 34, 30, 30, 35, 30, 30, 30, 36, 30, 30, 37, 38, 39, 38, 30, 38, 40, 41, 42, 43, 44, 45, 46, 47, 48, 49, 50, 51, 52, 53, 54, 55, 56, 57, 58, 59, 60, 61, 62, 63, 64, 65, 66, 67, 68, 38, 38, 38, 38, 38, 38, 38, 38, 38, 38, 38, 38, 30, 30, 38, 38, 38, 38, 38, 38, 38, 69, 38, 38, 38, 38, 38, 38, 38, 38, 38, 38, 38, 38, 38, 38, 38, 38, 69, 69, 69, 69, 69, 69, 69, 69, 69, 69, 69, 69, 69, 69, 69, 69, 69
];

XQueryParser.MAP2 =
[ 57344, 63744, 64976, 65008, 65536, 983040, 63743, 64975, 65007, 65533, 983039, 1114111, 38, 30, 38, 30, 30, 38
];

XQueryParser.INITIAL =
[ 1, 12290, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20, 21, 22, 23, 24, 25, 26, 27, 28, 29, 30, 31, 32, 33, 34, 35, 36, 37, 38, 39, 40, 41, 42, 43, 44, 45, 46, 47, 48, 49, 50, 51, 52, 53, 54, 55, 56, 57, 58, 59, 60, 61, 62, 63, 64, 65, 66, 67, 68, 69, 70, 71, 72, 73, 74, 75, 76, 77, 78, 79, 80, 81, 82, 83, 84, 85, 86, 87, 88, 89, 90, 91, 92, 93, 94, 95, 96, 97, 98, 99, 100, 101, 102, 103, 104, 105, 106, 107, 108, 109, 110, 111, 112, 113, 114, 115, 116, 117, 118, 119, 120, 121, 122, 123, 124, 125, 126, 127, 128, 129, 130, 131, 132, 133, 134, 135, 136, 137, 138, 139, 140, 141, 142, 143, 144, 145, 146, 147, 148, 149, 150, 151, 152, 153, 154, 155, 156, 157, 158, 159, 160, 161, 162, 163, 164, 165, 166, 167, 168, 169, 170, 171, 172, 173, 174, 175, 176, 177, 178, 179, 180, 181, 182, 183, 184, 185, 186, 187, 188, 189, 190, 191, 192, 193, 194, 195, 196, 197, 198, 199, 200, 201, 202, 203, 204, 205, 206, 207, 208, 209, 210, 211, 212, 213, 214, 215, 216, 217, 218, 219, 220, 221, 222, 223, 224, 225, 226, 227, 228, 229, 230, 231, 232, 233, 234, 235, 236, 237, 238, 239, 240, 241, 242, 243, 244, 245, 246, 247, 248, 249, 250, 251, 252, 253, 254, 255, 256, 257, 258, 259, 260, 261, 262, 263, 264, 265, 266, 267, 268, 269, 270, 271, 272, 273, 274, 275, 276, 277, 278, 279, 280, 281, 282, 283, 284, 285, 286, 287
];

XQueryParser.TRANSITION =
[ 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 25651, 25666, 25670, 25670, 25670, 18189, 25670, 25670, 25670, 25670, 18201, 25670, 25670, 25670, 25670, 18176, 25670, 25671, 18217, 42516, 42516, 42516, 27632, 42516, 42516, 51474, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 32629, 20505, 21066, 18953, 21068, 19019, 18919, 18270, 40105, 18293, 18315, 42516, 42516, 18360, 42516, 49435, 42820, 18380, 18411, 18395, 18574, 19226, 18433, 20169, 18459, 25598, 18486, 18508, 51199, 18465, 25604, 18492, 18514, 20728, 18530, 42516, 30418, 42516, 42516, 42516, 50250, 25964, 50770, 42516, 18553, 39446, 18564, 20175, 20411, 20131, 20505, 23879, 18590, 26008, 18612, 18632, 18596, 18470, 18594, 26012, 18616, 18636, 20798, 20741, 20757, 42516, 18652, 18689, 18711, 18748, 18695, 18788, 42516, 41546, 48871, 18807, 18835, 19802, 18823, 19840, 25206, 18851, 18904, 18942, 18979, 19680, 19006, 18851, 18904, 18942, 18979, 19680, 19045, 20750, 46697, 19097, 19132, 20936, 31767, 19148, 19174, 19196, 20141, 20921, 19788, 19212, 19242, 19278, 25525, 20579, 19301, 19596, 19271, 25518, 20572, 19294, 19589, 19324, 19353, 35639, 19379, 39467, 19401, 41728, 19421, 19500, 24458, 19544, 19575, 19614, 19632, 19650, 19559, 19612, 19630, 19648, 19666, 19029, 19696, 19712, 19728, 19753, 19773, 19830, 19435, 19449, 19888, 19905, 19856, 19887, 19904, 19921, 19947, 19984, 20043, 20080, 20119, 20157, 20536, 20191, 20213, 20542, 20197, 20219, 19931, 20235, 20264, 20301, 20317, 20346, 20333, 20349, 20365, 20381, 20397, 20422, 20606, 20438, 20477, 20503, 21067, 21068, 20521, 20558, 20595, 20622, 20652, 23888, 18878, 20683, 20699, 20715, 20779, 20814, 20880, 20890, 20906, 19463, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 21141, 42516, 42516, 42516, 42516, 48069, 42516, 42516, 42516, 42516, 25917, 18730, 20976, 20988, 20999, 21015, 25420, 18732, 21040, 42516, 42516, 42516, 27632, 42516, 42516, 51474, 31122, 42516, 42516, 42516, 42516, 42516, 42516, 32629, 20505, 21066, 18953, 21068, 21056, 21084, 18270, 40105, 18293, 18315, 42516, 42516, 18360, 42516, 49435, 42820, 18380, 18411, 18395, 18574, 19226, 18433, 20169, 18459, 25598, 18486, 18508, 51199, 18465, 25604, 18492, 18514, 20728, 18530, 42516, 30418, 42516, 42516, 42516, 50250, 25964, 50770, 42516, 18553, 39446, 18564, 20175, 20411, 20131, 20505, 23879, 18590, 26008, 18612, 18632, 18596, 18470, 18594, 26012, 18616, 18636, 20798, 20741, 20757, 42516, 18652, 18689, 18711, 18748, 18695, 18788, 42516, 41546, 48871, 18807, 18835, 19802, 18823, 19840, 25206, 18851, 18904, 18942, 18979, 19680, 19006, 18851, 18904, 18942, 18979, 19680, 19045, 20750, 46697, 19097, 19132, 20936, 31767, 19148, 19174, 19196, 20141, 20921, 19788, 19212, 19242, 19278, 25525, 20579, 19301, 19596, 19271, 25518, 20572, 19294, 19589, 19324, 19353, 35639, 19379, 39467, 19401, 41728, 19421, 19500, 24458, 19544, 19575, 19614, 19632, 19650, 19559, 19612, 19630, 19648, 19666, 19029, 19696, 19712, 19728, 19753, 19773, 19830, 19435, 19449, 19888, 19905, 19856, 19887, 19904, 19921, 19947, 19984, 20043, 20080, 20119, 20157, 20536, 20191, 20213, 20542, 20197, 20219, 19931, 20235, 20264, 20301, 20317, 20346, 20333, 20349, 20365, 20381, 20397, 20422, 20606, 20438, 20477, 20503, 21067, 21068, 20521, 20558, 20595, 20622, 20652, 23888, 18878, 20683, 20699, 20715, 20779, 20814, 20880, 20890, 20906, 19463, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 23286, 21107, 42516, 42516, 42516, 39416, 42516, 42516, 43470, 47286, 25568, 42516, 42516, 42516, 42516, 23672, 41495, 21126, 21160, 42516, 42516, 42516, 27632, 42516, 42516, 35938, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 32629, 20505, 21066, 18953, 21068, 19019, 25556, 18270, 40105, 18293, 18315, 42516, 42516, 18360, 42516, 49435, 42820, 18380, 18411, 18395, 18574, 19226, 18433, 20169, 18459, 25598, 18486, 18508, 51199, 18465, 25604, 18492, 18514, 20728, 18530, 42516, 30418, 42516, 42516, 42516, 50250, 25964, 50770, 42516, 18553, 39446, 18564, 20175, 20411, 20131, 20505, 23879, 18590, 26008, 18612, 18632, 18596, 18470, 18594, 26012, 18616, 18636, 20798, 20741, 20757, 42516, 18652, 18689, 18711, 18748, 18695, 18788, 42516, 41546, 48871, 18807, 18835, 19802, 18823, 19840, 25206, 18851, 18904, 18942, 18979, 19680, 19006, 18851, 18904, 18942, 18979, 19680, 19045, 20750, 46697, 19097, 19132, 20936, 31767, 19148, 19174, 19196, 20141, 20921, 19788, 19212, 19242, 19278, 25525, 20579, 19301, 19596, 19271, 25518, 20572, 19294, 19589, 19324, 19353, 35639, 19379, 39467, 19401, 41728, 19421, 19500, 24458, 19544, 19575, 19614, 19632, 19650, 19559, 19612, 19630, 19648, 19666, 19029, 19696, 19712, 19728, 19753, 19773, 19830, 19435, 19449, 19888, 19905, 19856, 19887, 19904, 19921, 19947, 19984, 20043, 20080, 20119, 20157, 20536, 20191, 20213, 20542, 20197, 20219, 19931, 20235, 20264, 20301, 20317, 20346, 20333, 20349, 20365, 20381, 20397, 20422, 20606, 20438, 20477, 20503, 21067, 21068, 20521, 20558, 20595, 20622, 20652, 23888, 18878, 20683, 20699, 20715, 20779, 20814, 20880, 20890, 20906, 19463, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 21176, 21248, 42516, 42516, 42516, 50595, 42516, 42516, 42445, 42516, 25917, 23619, 21211, 21279, 21269, 21226, 21240, 44419, 21040, 42516, 42516, 42516, 27632, 21302, 42516, 32247, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 32629, 20505, 21066, 18953, 21068, 19019, 19871, 21321, 40105, 18293, 18315, 42516, 42516, 18360, 42516, 49435, 42820, 18380, 18411, 18395, 18574, 19226, 18433, 20169, 18459, 25598, 18486, 18508, 51199, 18465, 25604, 18492, 18514, 20728, 18530, 42516, 30418, 42516, 42516, 42516, 50250, 25964, 50770, 42516, 18553, 39446, 18564, 20175, 20411, 20131, 20505, 23879, 21344, 26008, 18612, 18632, 18596, 21349, 18594, 26012, 18616, 18636, 20798, 20741, 20757, 42516, 18652, 18689, 18711, 18748, 18695, 18788, 42516, 41546, 48871, 18807, 18835, 19802, 18823, 19840, 25206, 18851, 18904, 18942, 18979, 19680, 19006, 18851, 18904, 18942, 18979, 19680, 19045, 20750, 46697, 19097, 19132, 20936, 31767, 19148, 19174, 19196, 20141, 20921, 19788, 19212, 19242, 19278, 25525, 20579, 19301, 19596, 19271, 25518, 20572, 19294, 19589, 19324, 19353, 35639, 19379, 39467, 19401, 41728, 19421, 19500, 24458, 19544, 19575, 19614, 19632, 19650, 19559, 19612, 19630, 19648, 19666, 19029, 19696, 19712, 19728, 19753, 19773, 19830, 19435, 19449, 19888, 19905, 19856, 19887, 19904, 19921, 19947, 19984, 20043, 20080, 20119, 20157, 20536, 20191, 20213, 20542, 20197, 20219, 19931, 20235, 20264, 20301, 20317, 20346, 20333, 20349, 20365, 20381, 20397, 20422, 20606, 20438, 20477, 20503, 21067, 21068, 20521, 20558, 20595, 20622, 20652, 23888, 18878, 20683, 20699, 20715, 20779, 20814, 20880, 20890, 20906, 19463, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 21141, 42510, 42516, 42516, 42516, 48894, 42515, 42516, 51366, 42516, 21365, 42504, 21403, 42515, 21410, 42516, 27599, 27612, 21040, 42516, 42516, 42516, 27632, 42516, 42516, 51474, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 32629, 20505, 21066, 18953, 21068, 19019, 18919, 18270, 40105, 18293, 18315, 42516, 42516, 18360, 42516, 49435, 42820, 18380, 18411, 18395, 18574, 19226, 18433, 20169, 18459, 25598, 18486, 18508, 51199, 18465, 25604, 18492, 18514, 20728, 18530, 42516, 30418, 42516, 42516, 42516, 50250, 25964, 50770, 42516, 18553, 39446, 18564, 20175, 20411, 20131, 20505, 23879, 18590, 26008, 18612, 18632, 18596, 18470, 18594, 26012, 18616, 18636, 20798, 20741, 20757, 42516, 18652, 18689, 18711, 18748, 18695, 18788, 42516, 41546, 48871, 18807, 18835, 19802, 18823, 19840, 25206, 18851, 18904, 18942, 18979, 19680, 19006, 18851, 18904, 18942, 18979, 19680, 19045, 20750, 46697, 19097, 19132, 20936, 31767, 19148, 19174, 19196, 20141, 20921, 19788, 19212, 19242, 19278, 25525, 20579, 19301, 19596, 19271, 25518, 20572, 19294, 19589, 19324, 19353, 35639, 19379, 39467, 19401, 41728, 19421, 19500, 24458, 19544, 19575, 19614, 19632, 19650, 19559, 19612, 19630, 19648, 19666, 19029, 19696, 19712, 19728, 19753, 19773, 19830, 19435, 19449, 19888, 19905, 19856, 19887, 19904, 19921, 19947, 19984, 20043, 20080, 20119, 20157, 20536, 20191, 20213, 20542, 20197, 20219, 19931, 20235, 20264, 20301, 20317, 20346, 20333, 20349, 20365, 20381, 20397, 20422, 20606, 20438, 20477, 20503, 21067, 21068, 20521, 20558, 20595, 20622, 20652, 23888, 18878, 20683, 20699, 20715, 20779, 20814, 20880, 20890, 20906, 19463, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 21141, 42516, 42516, 42516, 42516, 48069, 21426, 42516, 42516, 42516, 22170, 42516, 42516, 21445, 42516, 42516, 21468, 21481, 21040, 42516, 42516, 42516, 27632, 42516, 42516, 51474, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 32629, 20505, 21066, 18953, 21068, 19019, 18919, 18270, 40105, 18293, 18315, 42516, 42516, 18360, 42516, 49435, 42820, 18380, 18411, 18395, 18574, 19226, 18433, 20169, 18459, 25598, 18486, 18508, 51199, 18465, 25604, 18492, 18514, 20728, 18530, 42516, 30418, 42516, 42516, 42516, 50250, 25964, 50770, 42516, 18553, 39446, 18564, 20175, 20411, 20131, 20505, 23879, 18590, 26008, 18612, 18632, 18596, 18470, 18594, 26012, 18616, 18636, 20798, 20741, 20757, 42516, 18652, 18689, 18711, 18748, 18695, 18788, 42516, 41546, 48871, 18807, 18835, 19802, 18823, 19840, 25206, 18851, 18904, 18942, 18979, 19680, 19006, 18851, 18904, 18942, 18979, 19680, 19045, 20750, 46697, 19097, 19132, 20936, 31767, 19148, 19174, 19196, 20141, 20921, 19788, 19212, 19242, 19278, 25525, 20579, 19301, 19596, 19271, 25518, 20572, 19294, 19589, 19324, 19353, 35639, 19379, 39467, 19401, 41728, 19421, 19500, 24458, 19544, 19575, 19614, 19632, 19650, 19559, 19612, 19630, 19648, 19666, 19029, 19696, 19712, 19728, 19753, 19773, 19830, 19435, 19449, 19888, 19905, 19856, 19887, 19904, 19921, 19947, 19984, 20043, 20080, 20119, 20157, 20536, 20191, 20213, 20542, 20197, 20219, 19931, 20235, 20264, 20301, 20317, 20346, 20333, 20349, 20365, 20381, 20397, 20422, 20606, 20438, 20477, 20503, 21067, 21068, 20521, 20558, 20595, 20622, 20652, 23888, 18878, 20683, 20699, 20715, 20779, 20814, 20880, 20890, 20906, 19463, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 21141, 42516, 42516, 42516, 42516, 48069, 42516, 42516, 42516, 42516, 24484, 42516, 42516, 42516, 42516, 42516, 42516, 42517, 21497, 42516, 42516, 42516, 27632, 42516, 42516, 51474, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 32629, 20505, 21066, 18953, 21068, 19019, 18919, 18270, 40105, 18293, 18315, 42516, 42516, 18360, 42516, 49435, 42820, 18380, 18411, 18395, 18574, 19226, 18433, 20169, 21513, 25598, 18486, 18508, 51408, 18465, 25604, 18492, 18514, 20728, 18530, 42516, 30418, 42516, 42516, 42516, 50250, 25964, 50770, 42516, 18553, 39446, 18564, 20175, 20411, 20131, 20505, 23879, 18590, 26008, 18612, 18632, 18596, 18470, 18594, 26012, 18616, 18636, 20798, 20741, 20757, 42516, 18652, 18689, 18711, 18748, 18695, 18788, 42516, 41546, 48871, 18807, 18835, 19802, 18823, 19840, 25206, 18851, 18904, 18942, 18979, 19680, 19006, 18851, 18904, 18942, 18979, 19680, 19045, 20750, 46697, 19097, 19132, 20936, 31767, 19148, 19174, 19196, 20141, 20921, 19788, 19212, 19242, 19278, 25525, 20579, 19301, 19596, 19271, 25518, 20572, 19294, 19589, 19324, 19353, 35639, 19379, 39467, 19401, 41728, 19421, 19500, 24458, 19544, 19575, 19614, 19632, 19650, 19559, 19612, 19630, 19648, 19666, 19029, 19696, 19712, 19728, 19753, 19773, 19830, 19435, 19449, 19888, 19905, 19856, 19887, 19904, 19921, 19947, 19984, 20043, 20080, 20119, 20157, 20536, 20191, 20213, 20542, 20197, 20219, 19931, 20235, 20264, 20301, 20317, 20346, 20333, 20349, 20365, 20381, 20397, 20422, 20606, 20438, 20477, 20503, 21067, 21068, 20521, 20558, 20595, 20622, 20652, 23888, 18878, 20683, 20699, 20715, 20779, 20814, 20880, 20890, 20906, 19463, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 25358, 21538, 42516, 42516, 42516, 29996, 42516, 42516, 26519, 46446, 25383, 42516, 42516, 42516, 42516, 25736, 28473, 18232, 21557, 42516, 42516, 42516, 27632, 42516, 42516, 51474, 21573, 42516, 42516, 42516, 42516, 42516, 42516, 32629, 20505, 21066, 18953, 21068, 19019, 19060, 21591, 40105, 18293, 18315, 42516, 42516, 18360, 42516, 49435, 42820, 18380, 18411, 18395, 18574, 19226, 18433, 20169, 18459, 25598, 18486, 18508, 51199, 18465, 25604, 18492, 18514, 20728, 18530, 42516, 30418, 42516, 42516, 42516, 50250, 25964, 50770, 42516, 18553, 39446, 18564, 20175, 20411, 20131, 20505, 23879, 18590, 26008, 18612, 18632, 18596, 18470, 18594, 26012, 18616, 18636, 20798, 20741, 20757, 42516, 18652, 18689, 18711, 18748, 18695, 18788, 42516, 41546, 48871, 18807, 18835, 19802, 18823, 19840, 25206, 18851, 18904, 18942, 18979, 19680, 19006, 18851, 18904, 18942, 18979, 19680, 19045, 20750, 46697, 19097, 19132, 20936, 31767, 19148, 19174, 19196, 20141, 20921, 19788, 19212, 19242, 19278, 25525, 20579, 19301, 19596, 19271, 25518, 20572, 19294, 19589, 19324, 19353, 35639, 19379, 39467, 19401, 41728, 19421, 19500, 24458, 19544, 19575, 19614, 19632, 19650, 19559, 19612, 19630, 19648, 19666, 19029, 19696, 19712, 19728, 19753, 19773, 19830, 19435, 19449, 19888, 19905, 19856, 19887, 19904, 19921, 19947, 19984, 20043, 20080, 20119, 20157, 20536, 20191, 20213, 20542, 20197, 20219, 19931, 20235, 20264, 20301, 20317, 20346, 20333, 20349, 20365, 20381, 20397, 20422, 20606, 20438, 20477, 20503, 21067, 21068, 20521, 20558, 20595, 20622, 20652, 23888, 18878, 20683, 20699, 20715, 20779, 20814, 20880, 20890, 20906, 19463, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 23074, 23089, 21619, 21619, 21619, 21672, 21614, 21619, 21624, 23096, 21640, 21745, 21688, 21665, 21655, 21732, 21703, 21716, 21040, 42516, 42516, 42516, 33326, 42516, 42516, 51474, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 32629, 20505, 21066, 18953, 21068, 19019, 18919, 18270, 40105, 18293, 21761, 42516, 42516, 18360, 42516, 49435, 42820, 18380, 18411, 18395, 18574, 19226, 18433, 20169, 18459, 25598, 18486, 18508, 51199, 18465, 25604, 18492, 18514, 20728, 18530, 42516, 30418, 42516, 42516, 42516, 50250, 25964, 50770, 42516, 18553, 39446, 18564, 20175, 20411, 20131, 20505, 23879, 18590, 26008, 18612, 18632, 18596, 18470, 18594, 26012, 18616, 18636, 20798, 20741, 20757, 42516, 18652, 18689, 18711, 18748, 18695, 18788, 42516, 41546, 48871, 18807, 18835, 19802, 18823, 19840, 25206, 18851, 18904, 18942, 18979, 19680, 19006, 18851, 18904, 18942, 18979, 19680, 19045, 20750, 46697, 19097, 19132, 20936, 31767, 19148, 19174, 19196, 20141, 20921, 19788, 19212, 19242, 19278, 25525, 20579, 19301, 19596, 19271, 25518, 20572, 19294, 19589, 19324, 19353, 35639, 19379, 39467, 19401, 41728, 19421, 19500, 24458, 19544, 19575, 19614, 19632, 19650, 19559, 19612, 19630, 19648, 19666, 19029, 19696, 19712, 19728, 19753, 19773, 19830, 19435, 19449, 19888, 19905, 19856, 19887, 19904, 19921, 19947, 19984, 20043, 20080, 20119, 20157, 20536, 20191, 20213, 20542, 20197, 20219, 19931, 20235, 20264, 20301, 20317, 20346, 20333, 20349, 20365, 20381, 20397, 20422, 20606, 20438, 20477, 20503, 21067, 21068, 20521, 20558, 20595, 20622, 20652, 23888, 18878, 20683, 20699, 20715, 20779, 20814, 20880, 20890, 20906, 19463, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 21141, 45317, 42516, 42516, 42516, 49458, 21798, 42516, 22640, 21804, 25917, 45316, 42516, 21821, 21844, 21828, 21860, 21869, 21885, 42516, 42516, 42516, 29550, 42516, 42516, 51474, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 32629, 20505, 21066, 18953, 21068, 19019, 18919, 18270, 40105, 18293, 21901, 42516, 42516, 18360, 42516, 49435, 42820, 18380, 18411, 18395, 18574, 19226, 18433, 20169, 18459, 25598, 18486, 18508, 51199, 18465, 25604, 18492, 18514, 20728, 18530, 42516, 30418, 42516, 42516, 42516, 50250, 25964, 50770, 42516, 18553, 39446, 18564, 20175, 20411, 20131, 20505, 23879, 18590, 26008, 18612, 18632, 18596, 18470, 18594, 26012, 18616, 18636, 20798, 20741, 20757, 42516, 18652, 18689, 18711, 18748, 18695, 18788, 42516, 41546, 48871, 18807, 18835, 19802, 18823, 19840, 25206, 18851, 18904, 18942, 18979, 19680, 19006, 18851, 18904, 18942, 18979, 19680, 19045, 20750, 46697, 19097, 19132, 20936, 31767, 19148, 19174, 19196, 20141, 20921, 19788, 19212, 19242, 19278, 25525, 20579, 19301, 19596, 19271, 25518, 20572, 19294, 19589, 19324, 19353, 35639, 19379, 39467, 19401, 41728, 19421, 19500, 24458, 19544, 19575, 19614, 19632, 19650, 19559, 19612, 19630, 19648, 19666, 19029, 19696, 19712, 19728, 19753, 19773, 19830, 19435, 19449, 19888, 19905, 19856, 19887, 19904, 19921, 19947, 19984, 20043, 20080, 20119, 20157, 20536, 20191, 20213, 20542, 20197, 20219, 19931, 20235, 20264, 20301, 20317, 20346, 20333, 20349, 20365, 20381, 20397, 20422, 20606, 20438, 20477, 20503, 21067, 21068, 20521, 20558, 20595, 20622, 20652, 23888, 18878, 20683, 20699, 20715, 20779, 20814, 20880, 20890, 20906, 19463, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 21141, 21944, 42516, 42516, 42516, 48069, 42516, 42516, 42516, 26308, 36543, 25445, 25454, 21937, 25455, 21960, 21985, 21998, 21040, 42516, 42516, 42516, 27632, 42516, 42516, 51474, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 32629, 20505, 21066, 18953, 21068, 19019, 18919, 18270, 40105, 18293, 18315, 42516, 42516, 18360, 42516, 49435, 42820, 18380, 18411, 18395, 18574, 19226, 18433, 20169, 18459, 25598, 18486, 18508, 18888, 18465, 25604, 18492, 18514, 20728, 18530, 42516, 30418, 42516, 42516, 42516, 50250, 25964, 50770, 42516, 18553, 39446, 18564, 20175, 20411, 20131, 20505, 23879, 18590, 26008, 18612, 18632, 18596, 18470, 18594, 26012, 18616, 18636, 20798, 20741, 20757, 42516, 18652, 18689, 18711, 18748, 18695, 18788, 42516, 41546, 48871, 18807, 18835, 19802, 18823, 19840, 25206, 18851, 18904, 18942, 18979, 19680, 19006, 18851, 18904, 18942, 18979, 19680, 19045, 20750, 46697, 19097, 19132, 20936, 31767, 19148, 19174, 19196, 20141, 20921, 19788, 19212, 19242, 19278, 25525, 20579, 19301, 19596, 19271, 25518, 20572, 19294, 19589, 19324, 19353, 35639, 19379, 39467, 19401, 41728, 19421, 19500, 24458, 19544, 19575, 19614, 19632, 19650, 19559, 19612, 19630, 19648, 19666, 19029, 19696, 19712, 19728, 19753, 19773, 19830, 19435, 19449, 19888, 19905, 19856, 19887, 19904, 19921, 19947, 19984, 20043, 20080, 20119, 20157, 20536, 20191, 20213, 20542, 20197, 20219, 19931, 20235, 20264, 20301, 20317, 20346, 20333, 20349, 20365, 20381, 20397, 20422, 20606, 20438, 20477, 20503, 21067, 21068, 20521, 20558, 20595, 20622, 20652, 23888, 18878, 20683, 20699, 20715, 20779, 20814, 20880, 20890, 20906, 19463, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 21141, 42516, 42516, 42516, 42516, 48069, 42516, 42516, 42516, 26329, 37275, 42655, 22014, 22031, 22014, 22046, 42653, 22015, 21040, 42516, 42516, 42516, 27632, 42516, 42516, 51474, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 32629, 20505, 21066, 18953, 21068, 19019, 18919, 18270, 40105, 18293, 18315, 42516, 42516, 22071, 42516, 49435, 42820, 18380, 18411, 18395, 18574, 19226, 18433, 20169, 18459, 25598, 18486, 18508, 51199, 18465, 25604, 18492, 18514, 20728, 18530, 42516, 30418, 42516, 42516, 42516, 50250, 25964, 50770, 42516, 18553, 39446, 18564, 20175, 20411, 20131, 20505, 23879, 18590, 26008, 18612, 18632, 18596, 18470, 18594, 26012, 18616, 18636, 20798, 20741, 20757, 42516, 18652, 18689, 18711, 18748, 18695, 18788, 42516, 41546, 48871, 18807, 18835, 19802, 18823, 19840, 25206, 18851, 18904, 18942, 18979, 19680, 19006, 18851, 18904, 18942, 18979, 19680, 19045, 20750, 46697, 19097, 19132, 20936, 31767, 19148, 19174, 19196, 20141, 20921, 19788, 19212, 19242, 19278, 25525, 20579, 19301, 19596, 19271, 25518, 20572, 19294, 19589, 19324, 19353, 35639, 19379, 39467, 19401, 41728, 19421, 19500, 24458, 19544, 19575, 19614, 19632, 19650, 19559, 19612, 19630, 19648, 19666, 19029, 19696, 19712, 19728, 19753, 19773, 19830, 19435, 19449, 19888, 19905, 19856, 19887, 19904, 19921, 19947, 19984, 20043, 20080, 20119, 20157, 20536, 20191, 20213, 20542, 20197, 20219, 19931, 20235, 20264, 20301, 20317, 20346, 20333, 20349, 20365, 20381, 20397, 20422, 20606, 20438, 20477, 20503, 21067, 21068, 20521, 20558, 20595, 20622, 20652, 23888, 18878, 20683, 20699, 20715, 20779, 20814, 20880, 20890, 20906, 19463, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 21141, 26891, 42516, 42516, 42516, 48069, 22092, 42516, 23653, 22130, 40293, 22116, 22186, 22191, 22191, 22207, 26888, 44587, 21040, 42516, 42516, 42516, 27632, 42516, 42516, 51474, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 32629, 20505, 21066, 18953, 21068, 19019, 18919, 18270, 40105, 18293, 18315, 42516, 42516, 18360, 42516, 49435, 42820, 18380, 18411, 18395, 18574, 19226, 18433, 20169, 18459, 25598, 18486, 18508, 51199, 18465, 25604, 18492, 18514, 20728, 18530, 42516, 30418, 42516, 42516, 42516, 50250, 25964, 50770, 42516, 18553, 39446, 18564, 20175, 20411, 20131, 20505, 23879, 18590, 26008, 18612, 18632, 18596, 18470, 18594, 26012, 18616, 18636, 20798, 20741, 20757, 42516, 18652, 18689, 18711, 18748, 18695, 18788, 42516, 41546, 48871, 18807, 18835, 19802, 18823, 19840, 25206, 18851, 18904, 18942, 18979, 19680, 19006, 18851, 18904, 18942, 18979, 19680, 19045, 20750, 46697, 19097, 19132, 20936, 31767, 19148, 19174, 19196, 20141, 20921, 19788, 19212, 19242, 19278, 25525, 20579, 19301, 19596, 19271, 25518, 20572, 19294, 19589, 19324, 19353, 35639, 19379, 39467, 19401, 41728, 19421, 19500, 24458, 19544, 19575, 19614, 19632, 19650, 19559, 19612, 19630, 19648, 19666, 19029, 19696, 19712, 19728, 19753, 19773, 19830, 19435, 19449, 19888, 19905, 19856, 19887, 19904, 19921, 19947, 19984, 20043, 20080, 20119, 20157, 20536, 20191, 20213, 20542, 20197, 20219, 19931, 20235, 20264, 20301, 20317, 20346, 20333, 20349, 20365, 20381, 20397, 20422, 20606, 20438, 20477, 20503, 21067, 21068, 20521, 20558, 20595, 20622, 20652, 23888, 18878, 20683, 20699, 20715, 20779, 20814, 20880, 20890, 20906, 19463, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 18247, 42516, 42516, 42516, 42516, 48069, 42516, 42516, 42516, 42516, 25917, 35843, 22232, 22249, 22232, 22264, 35841, 22233, 22289, 42516, 42516, 42516, 27632, 42516, 42516, 51474, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 28241, 22657, 46314, 35273, 38147, 34810, 46363, 42516, 33841, 42516, 34304, 43899, 22305, 22071, 36154, 42516, 42516, 42516, 42516, 28243, 22657, 22322, 22657, 22657, 22341, 33347, 46316, 46316, 39789, 38147, 35514, 38147, 38147, 22359, 22382, 42516, 42516, 42516, 42516, 42516, 28815, 42516, 22405, 22426, 42516, 42516, 38368, 22657, 22657, 22657, 22657, 38374, 22541, 46316, 46316, 46316, 46316, 44805, 38147, 38147, 38147, 38147, 38147, 32923, 22444, 47355, 26569, 42516, 42516, 42516, 42516, 42516, 45225, 42516, 42516, 32981, 50880, 22657, 22657, 22657, 30360, 22467, 39622, 46316, 22501, 46316, 22343, 36422, 42625, 38147, 22520, 38147, 27826, 41766, 42516, 42516, 42516, 42516, 42516, 49148, 42516, 23255, 22657, 44467, 22657, 22657, 30361, 22539, 42077, 46316, 46316, 46316, 30525, 38147, 40186, 38147, 38147, 34440, 41769, 42516, 42516, 20027, 37487, 42516, 28242, 22657, 22657, 36250, 37745, 46316, 46316, 44329, 37798, 38147, 38147, 43834, 27827, 30836, 42516, 42516, 19405, 22558, 38368, 22576, 22657, 40730, 22597, 46316, 36583, 22617, 38147, 40469, 22638, 42516, 25905, 42516, 34153, 22656, 33064, 46073, 46316, 22674, 22710, 27827, 22690, 51145, 42516, 32408, 22729, 37572, 22752, 32120, 22683, 29050, 30609, 22771, 38961, 22813, 36803, 22874, 22893, 22911, 28238, 43147, 27775, 22913, 22451, 22736, 46400, 22448, 28385, 36584, 34366, 22755, 31692, 28429, 22929, 23000, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 21141, 42516, 42516, 42516, 42516, 48069, 42516, 42516, 42516, 50215, 25917, 42516, 42516, 42516, 42516, 42516, 45715, 23059, 23112, 42516, 42516, 42516, 27632, 42516, 42516, 37325, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 28241, 22657, 46314, 35273, 38147, 23128, 46363, 42516, 42516, 42516, 34304, 42516, 42516, 18360, 42516, 42516, 42516, 42516, 42516, 28243, 22657, 22657, 22657, 22657, 22341, 46316, 46316, 46316, 39789, 38147, 38147, 38147, 38147, 40896, 22382, 42516, 42516, 42516, 42516, 42516, 28815, 42516, 42516, 42516, 42516, 42516, 38368, 22657, 22657, 22657, 22657, 38374, 22541, 46316, 46316, 46316, 46316, 44805, 38147, 38147, 38147, 38147, 38147, 32923, 22444, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 38370, 22657, 22657, 22657, 22657, 30360, 22540, 46316, 46316, 46316, 46316, 22343, 30533, 38147, 38147, 38147, 38147, 27826, 41766, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 23255, 22657, 22657, 22657, 22657, 30361, 22539, 46316, 46316, 46316, 46316, 30525, 38147, 38147, 38147, 38147, 34440, 41769, 42516, 42516, 42516, 42516, 42516, 28242, 22657, 22657, 22657, 37745, 46316, 46316, 46316, 37798, 38147, 38147, 38147, 27827, 22690, 42516, 42516, 42516, 42516, 38368, 22657, 22657, 40730, 46316, 46316, 36583, 38147, 38147, 40469, 42516, 42516, 42516, 42516, 38373, 22657, 28384, 46316, 46316, 36586, 38147, 27827, 22690, 34920, 42516, 38372, 43148, 46316, 28405, 38147, 22683, 36804, 38368, 28385, 28404, 30535, 36803, 38372, 46315, 38146, 28238, 43147, 27775, 22913, 28244, 40729, 36585, 22448, 28385, 36584, 34366, 22755, 31692, 28429, 22929, 23000, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 21141, 23154, 42516, 42516, 42516, 46371, 42516, 42516, 42516, 42516, 25917, 35960, 23175, 23206, 23195, 23229, 35959, 23179, 21040, 42516, 42516, 42516, 27632, 42516, 42516, 51474, 27227, 42516, 43023, 42516, 42516, 42516, 42516, 32629, 20505, 21066, 18953, 21068, 19019, 18919, 18270, 40105, 18293, 18315, 42516, 42516, 18360, 42516, 49435, 42820, 18380, 18411, 18395, 18574, 19226, 18433, 20169, 18459, 25598, 18486, 18508, 51199, 18465, 25604, 18492, 18514, 20728, 18530, 42516, 30418, 42516, 42516, 42516, 50250, 25964, 50770, 42516, 18553, 39446, 18564, 20175, 20411, 20131, 20505, 23879, 18590, 26008, 18612, 18632, 18596, 18470, 18594, 26012, 18616, 18636, 20798, 20741, 20757, 42516, 18652, 18689, 18711, 18748, 18695, 18788, 42516, 41546, 48871, 18807, 18835, 19802, 18823, 19840, 25206, 18851, 18904, 18942, 18979, 19680, 19006, 18851, 18904, 18942, 18979, 19680, 19045, 20750, 46697, 19097, 19132, 20936, 31767, 19148, 19174, 19196, 20141, 20921, 19788, 19212, 19242, 19278, 25525, 20579, 19301, 19596, 19271, 25518, 20572, 19294, 19589, 19324, 19353, 35639, 19379, 39467, 19401, 41728, 19421, 19500, 24458, 19544, 19575, 19614, 19632, 19650, 19559, 19612, 19630, 19648, 19666, 19029, 19696, 19712, 19728, 19753, 19773, 19830, 19435, 19449, 19888, 19905, 19856, 19887, 19904, 19921, 19947, 19984, 20043, 20080, 20119, 20157, 20536, 20191, 20213, 20542, 20197, 20219, 19931, 20235, 20264, 20301, 20317, 20346, 20333, 20349, 20365, 20381, 20397, 20422, 20606, 20438, 20477, 20503, 21067, 21068, 20521, 20558, 20595, 20622, 20652, 23888, 18878, 20683, 20699, 20715, 20779, 20814, 20880, 20890, 20906, 19463, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 21141, 23254, 42516, 42516, 42516, 48069, 42516, 42516, 42516, 19968, 25917, 42516, 42516, 42516, 42516, 42516, 42762, 23271, 23302, 42516, 42516, 42516, 27632, 42516, 42516, 21380, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 28241, 22657, 46314, 35273, 38147, 23318, 46363, 42516, 42516, 42516, 34304, 42516, 42516, 23344, 42516, 42516, 42516, 42516, 42516, 28243, 22657, 22657, 22657, 22657, 22341, 46316, 46316, 46316, 39789, 38147, 38147, 38147, 38147, 40896, 23370, 23409, 42516, 42516, 42516, 42516, 38331, 23429, 42516, 42516, 42516, 42516, 38368, 22657, 22657, 22657, 22657, 38374, 22541, 46316, 46316, 46316, 46316, 44805, 38147, 38147, 38147, 38147, 38147, 32923, 23450, 23383, 42516, 42516, 42516, 42516, 42516, 23487, 42516, 42516, 42516, 38370, 22657, 22657, 22657, 22657, 50700, 22540, 46316, 46316, 46316, 46316, 47852, 30533, 38147, 38147, 38147, 38147, 27826, 35762, 23510, 42516, 42516, 42516, 42516, 42516, 42516, 23255, 22657, 22657, 22657, 22657, 29796, 22539, 46316, 46316, 46316, 46316, 23533, 38147, 38147, 38147, 38147, 34440, 41769, 42516, 42516, 42516, 42516, 42516, 28242, 22657, 22657, 22657, 46191, 46316, 46316, 46316, 32038, 38147, 38147, 38147, 27827, 22690, 42516, 42516, 42516, 42516, 38368, 22657, 22657, 40730, 46316, 46316, 36583, 38147, 38147, 40469, 42516, 42516, 42516, 42516, 38373, 22657, 28384, 46316, 46316, 36586, 38147, 27827, 22690, 34920, 42516, 38372, 43148, 46316, 28405, 38147, 22683, 36804, 38368, 28385, 28404, 30535, 36803, 38372, 46315, 38146, 28238, 43147, 27775, 22913, 28244, 40729, 36585, 22448, 28385, 36584, 34366, 22755, 31692, 28429, 22929, 23000, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 21141, 48371, 42516, 42516, 42516, 31492, 32078, 42516, 42516, 23557, 25917, 18721, 49121, 23587, 23594, 23610, 42516, 43878, 23635, 23651, 42516, 42516, 33439, 23669, 42516, 51474, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 32629, 20505, 25038, 24355, 23991, 25028, 23804, 18270, 40105, 18293, 23688, 42516, 42516, 18360, 42516, 49435, 42820, 18380, 18411, 18395, 18574, 19226, 18433, 20169, 23724, 24084, 23749, 23789, 24317, 23827, 23849, 23904, 24015, 23917, 23946, 42516, 30418, 42516, 42516, 42516, 50250, 25964, 50770, 42516, 18553, 39446, 18564, 20175, 20411, 20131, 20505, 23879, 23969, 24689, 24310, 25086, 25092, 23974, 23990, 24416, 24845, 24007, 23833, 24977, 24993, 42516, 18652, 18689, 18711, 18748, 18695, 18788, 42516, 41546, 48871, 18807, 18835, 19802, 18823, 19840, 25206, 24031, 23930, 24046, 24073, 24057, 24100, 24137, 24664, 24240, 24198, 24251, 24225, 24986, 46697, 19097, 19132, 20936, 31767, 19148, 19174, 19196, 20141, 20921, 19788, 19212, 19242, 24267, 24283, 24299, 24348, 24333, 24371, 24167, 24428, 24402, 23773, 24444, 24474, 35639, 19379, 39467, 19401, 41728, 19421, 19500, 24458, 19544, 24500, 24558, 24572, 24588, 24604, 24182, 24745, 24620, 24636, 24767, 19696, 19712, 19728, 19753, 19773, 19830, 19435, 19514, 24680, 24514, 19528, 24705, 24152, 24757, 19947, 19984, 20043, 20080, 20119, 20157, 20667, 24542, 24528, 24731, 24830, 24209, 24939, 20235, 20264, 20301, 24783, 24799, 24815, 24715, 24861, 20381, 20397, 19814, 23733, 24877, 24893, 20503, 25039, 24968, 23864, 24927, 24955, 25015, 20652, 18443, 23763, 24386, 25055, 24651, 25071, 25108, 25159, 25175, 25191, 25243, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 21141, 27003, 42516, 42516, 42516, 48069, 25296, 42516, 42516, 25291, 40518, 25312, 25320, 25320, 25320, 25327, 27168, 25343, 21040, 42516, 42516, 42516, 27632, 42516, 42516, 51474, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 32629, 20505, 21066, 18953, 21068, 19019, 18919, 18270, 40105, 18293, 18315, 42516, 42516, 18360, 42516, 49435, 42820, 18380, 18411, 18395, 18574, 19226, 18433, 20169, 18459, 25598, 18486, 18508, 51199, 18465, 25604, 18492, 18514, 20728, 18530, 42516, 30418, 42516, 42516, 42516, 50250, 25399, 50770, 42516, 18553, 39446, 18564, 20175, 20411, 20131, 20505, 23879, 18590, 26008, 18612, 18632, 18596, 18470, 18594, 26012, 18616, 18636, 20798, 20741, 18963, 25373, 18652, 18689, 18711, 18748, 18695, 25415, 42516, 41546, 48871, 18807, 18835, 19802, 18823, 19840, 25206, 18851, 18904, 18942, 18979, 19680, 19006, 18851, 18904, 18942, 18979, 19680, 19045, 19337, 25436, 19097, 19132, 20936, 31767, 19148, 19174, 19196, 20141, 20921, 19788, 19212, 25471, 25509, 25525, 20579, 19301, 19596, 25541, 25518, 20572, 19294, 19589, 19324, 19353, 35639, 19379, 39467, 19401, 41728, 19421, 19500, 24458, 19544, 25584, 19614, 19632, 19650, 25620, 19612, 19630, 19648, 19666, 19029, 19696, 19712, 19728, 19753, 19773, 19830, 19435, 19449, 19888, 19905, 19856, 19887, 19904, 19921, 19947, 19984, 20043, 20080, 20119, 20157, 20536, 20191, 20213, 20542, 20197, 20219, 19931, 20235, 20264, 20301, 20317, 20346, 20333, 20349, 20365, 20381, 20397, 20422, 20606, 20438, 20477, 20503, 21067, 21068, 20521, 20558, 20595, 20622, 20652, 23888, 18878, 20683, 20699, 20715, 20779, 20814, 20880, 20890, 20906, 19463, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 21141, 42516, 42516, 42516, 42516, 48069, 42516, 42516, 42516, 42516, 21541, 28730, 28739, 28739, 28739, 28746, 47533, 25636, 21040, 42516, 42516, 42516, 27632, 42516, 42516, 51474, 42516, 42516, 27653, 42516, 42516, 42516, 42516, 32629, 20505, 21066, 18953, 21068, 19019, 25687, 18270, 40105, 18293, 18315, 42516, 42516, 18360, 42516, 49435, 42820, 18380, 18411, 18395, 18574, 19226, 18433, 20169, 18459, 25598, 18486, 18508, 51199, 18465, 25604, 18492, 18514, 20728, 18530, 42516, 30418, 42516, 42516, 42516, 50250, 25964, 50770, 42516, 18553, 39446, 18564, 20175, 20411, 20131, 20505, 23879, 18590, 26008, 18612, 18632, 18596, 18470, 18594, 26012, 18616, 18636, 20798, 20741, 20757, 42516, 18652, 18689, 18711, 18748, 18695, 18788, 42516, 41546, 48871, 18807, 18835, 19802, 18823, 19840, 25206, 18851, 18904, 18942, 18979, 19680, 19006, 18851, 18904, 18942, 18979, 19680, 19045, 20750, 46697, 19097, 19132, 20936, 31767, 19148, 19174, 19196, 20141, 20921, 19788, 19212, 19242, 19278, 25525, 20579, 19301, 19596, 19271, 25518, 20572, 19294, 19589, 19324, 19353, 35639, 19379, 39467, 19401, 41728, 19421, 19500, 24458, 19544, 19575, 19614, 19632, 19650, 19559, 19612, 19630, 19648, 19666, 19029, 19696, 19712, 19728, 19753, 19773, 19830, 19435, 19449, 19888, 19905, 19856, 19887, 19904, 19921, 19947, 19984, 20043, 20080, 20119, 20157, 20536, 20191, 20213, 20542, 20197, 20219, 19931, 20235, 20264, 20301, 20317, 20346, 20333, 20349, 20365, 20381, 20397, 20422, 20606, 20438, 20477, 20503, 21067, 21068, 20521, 20558, 20595, 20622, 20652, 23888, 18878, 20683, 20699, 20715, 20779, 20814, 20880, 20890, 20906, 19463, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 51268, 23158, 42516, 42516, 42516, 48069, 42516, 42516, 42516, 42516, 25917, 22795, 25710, 25710, 25710, 25717, 23159, 22797, 21040, 25733, 42516, 42516, 27632, 42516, 42516, 51474, 42516, 42516, 44769, 42516, 42516, 42516, 42516, 32629, 20505, 21066, 18953, 21068, 19019, 25752, 18270, 40105, 18293, 18315, 42516, 42516, 18360, 42516, 49435, 42820, 18380, 18411, 18395, 18574, 19226, 18433, 20169, 18459, 25598, 18486, 18508, 51199, 18465, 25604, 18492, 18514, 20728, 18530, 42516, 30418, 42516, 42516, 42516, 50250, 25964, 50770, 42516, 18553, 39446, 18564, 20175, 20411, 20131, 20505, 23879, 18590, 26008, 18612, 18632, 18596, 18470, 18594, 26012, 18616, 18636, 20798, 20741, 20757, 42516, 18652, 18689, 18711, 18748, 18695, 18788, 42516, 41546, 48871, 18807, 18835, 19802, 18823, 19840, 25206, 18851, 18904, 18942, 18979, 19680, 19006, 18851, 18904, 18942, 18979, 19680, 19045, 20750, 46697, 19097, 19132, 20936, 31767, 19148, 19174, 19196, 20141, 20921, 19788, 19212, 19242, 19278, 25525, 20579, 19301, 19596, 19271, 25518, 20572, 19294, 19589, 19324, 19353, 35639, 19379, 39467, 19401, 41728, 19421, 19500, 24458, 19544, 19575, 19614, 19632, 19650, 19559, 19612, 19630, 19648, 19666, 19029, 19696, 19712, 19728, 19753, 19773, 19830, 19435, 19449, 19888, 19905, 19856, 19887, 19904, 19921, 19947, 19984, 20043, 20080, 20119, 20157, 20536, 20191, 20213, 20542, 20197, 20219, 19931, 20235, 20264, 20301, 20317, 20346, 20333, 20349, 20365, 20381, 20397, 20422, 20606, 20438, 20477, 20503, 21067, 21068, 20521, 20558, 20595, 20622, 20652, 23888, 18878, 20683, 20699, 20715, 20779, 20814, 20880, 20890, 20906, 19463, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 26066, 26075, 42516, 42516, 42516, 42907, 42516, 42516, 42516, 42516, 25917, 26405, 25775, 25775, 25775, 25782, 28588, 26407, 25798, 25814, 42516, 42516, 27632, 42516, 42516, 51474, 42516, 42516, 23413, 42516, 42516, 42516, 42516, 32629, 20505, 21066, 18953, 21068, 19019, 25831, 18270, 40105, 18293, 18315, 42516, 42516, 18360, 42516, 49435, 42820, 18380, 18411, 18395, 18574, 19226, 18433, 20169, 18459, 25598, 18486, 18508, 51199, 18465, 25604, 18492, 18514, 20728, 18530, 42516, 30418, 42516, 42516, 42516, 50250, 25964, 50770, 42516, 18553, 39446, 18564, 20175, 20411, 20131, 20505, 23879, 18590, 26008, 18612, 18632, 18596, 18470, 18594, 26012, 18616, 18636, 20798, 20741, 20757, 42516, 18652, 18689, 18711, 18748, 18695, 18788, 42516, 41546, 48871, 18807, 18835, 19802, 18823, 19840, 25206, 18851, 18904, 18942, 18979, 19680, 19006, 18851, 18904, 18942, 18979, 19680, 19045, 20750, 46697, 19097, 19132, 20936, 31767, 19148, 19174, 19196, 20141, 20921, 19788, 19212, 19242, 19278, 25525, 20579, 19301, 19596, 19271, 25518, 20572, 19294, 19589, 19324, 19353, 35639, 19379, 39467, 19401, 41728, 19421, 19500, 24458, 19544, 19575, 19614, 19632, 19650, 19559, 19612, 19630, 19648, 19666, 19029, 19696, 19712, 19728, 19753, 19773, 19830, 19435, 19449, 19888, 19905, 19856, 19887, 19904, 19921, 19947, 19984, 20043, 20080, 20119, 20157, 20536, 20191, 20213, 20542, 20197, 20219, 19931, 20235, 20264, 20301, 20317, 20346, 20333, 20349, 20365, 20381, 20397, 20422, 20606, 20438, 25857, 20503, 21067, 21068, 20521, 20558, 20595, 20622, 20652, 23888, 18878, 20683, 20699, 20715, 20779, 20814, 20880, 20890, 20906, 19463, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 25891, 42516, 42516, 42516, 42516, 48069, 25942, 42516, 42516, 28970, 33768, 42516, 42516, 42516, 25933, 25960, 26180, 25944, 25980, 42516, 42516, 42516, 27632, 42516, 42516, 51474, 27927, 42516, 42516, 42516, 42516, 42516, 42516, 32629, 20505, 21066, 18953, 21068, 25996, 26028, 18270, 40105, 18293, 18315, 42516, 42516, 18360, 42516, 49435, 42820, 18380, 18411, 18395, 18574, 19226, 18433, 20169, 18459, 25598, 18486, 18508, 51199, 18465, 25604, 18492, 18514, 20728, 18530, 42516, 30418, 42516, 42516, 42516, 50250, 25964, 50770, 42516, 18553, 39446, 18564, 20175, 20411, 20131, 20505, 23879, 18590, 26008, 18612, 18632, 18596, 18470, 18594, 26012, 18616, 18636, 20798, 20741, 20757, 42516, 18652, 18689, 18711, 18748, 18695, 18788, 42516, 41546, 48871, 18807, 18835, 19802, 18823, 19840, 25206, 18851, 18904, 18942, 18979, 19680, 19006, 18851, 18904, 18942, 18979, 19680, 19045, 20750, 46697, 19097, 19132, 20936, 31767, 19148, 19174, 19196, 20141, 20921, 19788, 19212, 19242, 19278, 25525, 20579, 19301, 19596, 19271, 25518, 20572, 19294, 19589, 19324, 19353, 35639, 19379, 39467, 19401, 41728, 19421, 19500, 24458, 19544, 19575, 19614, 19632, 19650, 19559, 19612, 19630, 19648, 19666, 19029, 19696, 19712, 19728, 19753, 19773, 19830, 19435, 19449, 19888, 19905, 19856, 19887, 19904, 19921, 19947, 19984, 20043, 20080, 20119, 20157, 20536, 20191, 20213, 20542, 20197, 20219, 19931, 20235, 20264, 20301, 20317, 20346, 20333, 20349, 20365, 20381, 20397, 20422, 20606, 20438, 20477, 20503, 21067, 21068, 20521, 20558, 20595, 20622, 20652, 23888, 18878, 20683, 20699, 20715, 20779, 20814, 20880, 20890, 20906, 19463, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 21141, 42516, 42516, 42516, 42516, 48069, 42516, 42516, 42516, 42516, 25917, 42516, 42516, 42516, 42516, 42516, 41865, 26051, 21040, 42516, 42516, 42516, 27632, 42516, 42516, 51474, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 32629, 20505, 21066, 18953, 21068, 19019, 18919, 18270, 40105, 18293, 18315, 42516, 42516, 18360, 42516, 49435, 42820, 18380, 18411, 18395, 18574, 19226, 18433, 20169, 18459, 25598, 18486, 18508, 51199, 18465, 25604, 18492, 18514, 20728, 18530, 42516, 30418, 42516, 42516, 42516, 50250, 25964, 50770, 42516, 18553, 39446, 18564, 20175, 20411, 20131, 20505, 23879, 18590, 26008, 18612, 18632, 18596, 18470, 18594, 26012, 18616, 18636, 20798, 20741, 20757, 42516, 18652, 18689, 18711, 18748, 18695, 18788, 42516, 41546, 48871, 18807, 18835, 19802, 18823, 19840, 25206, 18851, 18904, 18942, 18979, 19680, 19006, 18851, 18904, 18942, 18979, 19680, 19045, 20750, 46697, 19097, 19132, 20936, 31767, 19148, 19174, 19196, 20141, 20921, 19788, 19212, 19242, 19278, 25525, 20579, 19301, 19596, 19271, 25518, 20572, 19294, 19589, 19324, 19353, 35639, 19379, 39467, 19401, 41728, 19421, 19500, 24458, 19544, 19575, 19614, 19632, 19650, 19559, 19612, 19630, 19648, 19666, 19029, 19696, 19712, 19728, 19753, 19773, 19830, 19435, 19449, 19888, 19905, 19856, 19887, 19904, 19921, 19947, 19984, 20043, 20080, 20119, 20157, 20536, 20191, 20213, 20542, 20197, 20219, 19931, 20235, 20264, 20301, 20317, 20346, 20333, 20349, 20365, 20381, 20397, 20422, 20606, 20438, 20477, 20503, 21067, 21068, 20521, 20558, 20595, 20622, 20652, 23888, 18878, 20683, 20699, 20715, 20779, 20814, 20880, 20890, 20906, 19463, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 51361, 42516, 42516, 42516, 42516, 51537, 42516, 42516, 42516, 42516, 25917, 42516, 42516, 42516, 42516, 46741, 49861, 51029, 26099, 42516, 42516, 42516, 27632, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 28241, 22657, 46314, 35273, 38147, 34810, 28721, 42516, 42516, 42516, 34304, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 28243, 22657, 22657, 22657, 22657, 27401, 46316, 46316, 46316, 48318, 38147, 38147, 38147, 38147, 40896, 22382, 42516, 42516, 42516, 42516, 42516, 42516, 23487, 42516, 42516, 42516, 42516, 38368, 22657, 22657, 22657, 22657, 38374, 22541, 46316, 46316, 46316, 46316, 44805, 38147, 38147, 38147, 38147, 38147, 32923, 26115, 26152, 42516, 42516, 42516, 42516, 42516, 23487, 42516, 42516, 42516, 38370, 22657, 22657, 22657, 22657, 30360, 22540, 46316, 46316, 46316, 46316, 22343, 30533, 38147, 38147, 38147, 38147, 27826, 35762, 23510, 42516, 42516, 42516, 42516, 26178, 42516, 23255, 22657, 22657, 22657, 22657, 22877, 22539, 46316, 46316, 46316, 46316, 32774, 38147, 38147, 38147, 38147, 34440, 41769, 42516, 42516, 42516, 42516, 42516, 28242, 22657, 22657, 22657, 46191, 46316, 46316, 46316, 32038, 38147, 38147, 38147, 27827, 22690, 42516, 42516, 23464, 42516, 38368, 22657, 22657, 40730, 46316, 46316, 36583, 38147, 38147, 40469, 42516, 42516, 42516, 42516, 38373, 22657, 28384, 46316, 46316, 36586, 38147, 27827, 22690, 34920, 42516, 38372, 43148, 46316, 28405, 38147, 22683, 36804, 38368, 28385, 28404, 30535, 36803, 38372, 46315, 38146, 28238, 43147, 27775, 22913, 28244, 40729, 36585, 22448, 28385, 36584, 34366, 22755, 31692, 28429, 22929, 23000, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 51361, 42516, 42516, 42516, 42516, 51537, 42516, 42516, 42516, 42516, 25917, 42516, 42516, 42516, 42516, 46741, 49861, 51029, 26099, 42516, 42516, 42516, 27632, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 28241, 22657, 46314, 35273, 38147, 34810, 28721, 42516, 42516, 42516, 34304, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 28243, 22657, 22657, 22657, 22657, 27401, 46316, 46316, 46316, 48318, 38147, 38147, 38147, 38147, 40896, 22382, 42516, 42516, 42516, 42516, 42516, 42516, 23487, 42516, 42516, 42516, 42516, 38368, 22657, 22657, 22657, 22657, 38374, 22541, 46316, 46316, 46316, 46316, 44805, 38147, 38147, 38147, 38147, 38147, 32923, 26115, 26152, 42516, 42516, 42516, 42516, 42516, 23487, 42516, 42516, 42516, 38370, 22657, 22657, 22657, 22657, 30360, 22540, 46316, 46316, 46316, 46316, 22343, 30533, 38147, 38147, 38147, 38147, 27826, 35762, 23510, 42516, 42516, 42516, 42516, 42516, 42516, 23255, 22657, 22657, 22657, 22657, 22877, 22539, 46316, 46316, 46316, 46316, 32774, 38147, 38147, 38147, 38147, 34440, 41769, 42516, 42516, 42516, 42516, 42516, 28242, 22657, 22657, 22657, 46191, 46316, 46316, 46316, 32038, 38147, 38147, 38147, 27827, 22690, 42516, 42516, 42516, 42516, 38368, 22657, 22657, 40730, 46316, 46316, 36583, 38147, 38147, 40469, 42516, 42516, 42516, 42516, 38373, 22657, 28384, 46316, 46316, 36586, 38147, 27827, 22690, 34920, 42516, 38372, 43148, 46316, 28405, 38147, 22683, 36804, 38368, 28385, 28404, 30535, 36803, 38372, 46315, 38146, 28238, 43147, 27775, 22913, 28244, 40729, 36585, 22448, 28385, 36584, 34366, 22755, 31692, 28429, 22929, 23000, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 51361, 42516, 42516, 42516, 42516, 51537, 42516, 42516, 42516, 42516, 25917, 42516, 42516, 42516, 42516, 46741, 49861, 51029, 26099, 42516, 42516, 42516, 27632, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 28241, 22657, 46314, 35273, 38147, 34810, 28721, 42516, 42516, 42516, 34304, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 28243, 22657, 22657, 22657, 22657, 27401, 46316, 46316, 46316, 48318, 38147, 38147, 38147, 38147, 40896, 22382, 42516, 42516, 42516, 42516, 42516, 42516, 23487, 26196, 42516, 42516, 42516, 38368, 22657, 22657, 22657, 22657, 38374, 22541, 46316, 46316, 46316, 46316, 44805, 38147, 38147, 38147, 38147, 38147, 32923, 26115, 26152, 42516, 42516, 42516, 42516, 42516, 23487, 42516, 42516, 42516, 38370, 22657, 22657, 22657, 22657, 30360, 22540, 46316, 46316, 46316, 46316, 22343, 30533, 38147, 38147, 38147, 38147, 27826, 35762, 23510, 42516, 42516, 42516, 42516, 42516, 42516, 23255, 22657, 22657, 22657, 22657, 22877, 22539, 46316, 46316, 46316, 46316, 32774, 38147, 38147, 38147, 38147, 34440, 41769, 42516, 42516, 42516, 42516, 42516, 28242, 22657, 22657, 22657, 46191, 46316, 46316, 46316, 32038, 38147, 38147, 38147, 27827, 22690, 42516, 42516, 42516, 42516, 38368, 22657, 22657, 40730, 46316, 46316, 36583, 38147, 38147, 40469, 42516, 42516, 42516, 42516, 38373, 22657, 28384, 46316, 46316, 36586, 38147, 27827, 22690, 34920, 42516, 38372, 43148, 46316, 28405, 38147, 22683, 36804, 38368, 28385, 28404, 30535, 36803, 38372, 46315, 38146, 28238, 43147, 27775, 22913, 28244, 40729, 36585, 22448, 28385, 36584, 34366, 22755, 31692, 28429, 22929, 23000, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 51361, 42516, 42516, 42516, 42516, 51537, 42516, 42516, 42516, 42516, 25917, 42516, 42516, 42516, 42516, 46741, 49861, 51029, 26099, 42516, 42516, 42516, 27632, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 28241, 22657, 46314, 35273, 38147, 34810, 28721, 42516, 42516, 42516, 34304, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 28243, 22657, 22657, 22657, 22657, 27401, 46316, 46316, 46316, 48318, 38147, 38147, 38147, 38147, 40896, 22382, 42516, 42516, 42516, 42516, 42516, 42516, 23487, 42516, 42516, 42516, 42516, 38368, 22657, 22657, 22657, 22657, 38374, 22541, 46316, 46316, 46316, 46316, 44805, 38147, 38147, 38147, 38147, 38147, 32923, 26115, 26152, 42516, 42516, 42516, 42516, 42516, 26216, 42516, 42516, 42516, 38370, 22657, 22657, 22657, 22657, 30360, 22540, 46316, 46316, 46316, 46316, 22343, 30533, 38147, 38147, 38147, 38147, 27826, 35762, 23510, 42516, 42516, 42516, 42516, 42516, 42516, 23255, 22657, 22657, 22657, 22657, 22877, 22539, 46316, 46316, 46316, 46316, 32774, 38147, 38147, 38147, 38147, 34440, 41769, 42516, 42516, 42516, 42516, 42516, 28242, 22657, 22657, 22657, 46191, 46316, 46316, 46316, 32038, 38147, 38147, 38147, 27827, 22690, 42516, 42516, 42516, 42516, 38368, 22657, 22657, 40730, 46316, 46316, 36583, 38147, 38147, 40469, 42516, 42516, 42516, 42516, 38373, 22657, 28384, 46316, 46316, 36586, 38147, 27827, 22690, 34920, 42516, 38372, 43148, 46316, 28405, 38147, 22683, 36804, 38368, 28385, 28404, 30535, 36803, 38372, 46315, 38146, 28238, 43147, 27775, 22913, 28244, 40729, 36585, 22448, 28385, 36584, 34366, 22755, 31692, 28429, 22929, 23000, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 51361, 42516, 42516, 42516, 42516, 51537, 42516, 42516, 42516, 42516, 25917, 42516, 42516, 42516, 42516, 46741, 49861, 51029, 26099, 42516, 42516, 42516, 27632, 42516, 42516, 31151, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 28241, 22657, 46314, 35273, 38147, 34810, 28721, 42516, 42516, 42516, 34304, 42516, 42516, 31154, 42516, 42516, 42516, 42516, 42516, 28243, 22657, 22657, 22657, 22657, 27401, 46316, 46316, 46316, 48318, 38147, 38147, 38147, 38147, 40896, 22382, 42516, 42516, 42516, 42516, 42516, 42516, 23487, 42516, 42516, 42516, 42516, 38368, 22657, 22657, 22657, 22657, 38374, 22541, 46316, 46316, 46316, 46316, 44805, 38147, 38147, 38147, 38147, 38147, 32923, 26115, 26152, 42516, 42516, 42516, 42516, 42516, 23487, 42516, 42516, 42516, 38370, 22657, 22657, 22657, 22657, 30360, 22540, 46316, 46316, 46316, 46316, 22343, 30533, 38147, 38147, 38147, 38147, 27826, 35762, 23510, 42516, 42516, 42516, 42516, 42516, 42516, 23255, 22657, 22657, 22657, 22657, 22877, 22539, 46316, 46316, 46316, 46316, 32774, 38147, 38147, 38147, 38147, 34440, 41769, 42516, 42516, 42516, 42516, 42516, 28242, 22657, 22657, 22657, 46191, 46316, 46316, 46316, 32038, 38147, 38147, 38147, 27827, 22690, 42516, 42516, 42516, 42516, 38368, 22657, 22657, 40730, 46316, 46316, 36583, 38147, 38147, 40469, 42516, 42516, 42516, 42516, 38373, 22657, 28384, 46316, 46316, 36586, 38147, 27827, 22690, 34920, 42516, 38372, 43148, 46316, 28405, 38147, 22683, 36804, 38368, 28385, 28404, 30535, 36803, 38372, 46315, 38146, 28238, 43147, 27775, 22913, 28244, 40729, 36585, 22448, 28385, 36584, 34366, 22755, 31692, 28429, 22929, 23000, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 51361, 42516, 42516, 42516, 42516, 51537, 42516, 42516, 42516, 42516, 25917, 42516, 42516, 42516, 42516, 46741, 49861, 51029, 26099, 42516, 42516, 42516, 27632, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 28241, 22657, 46314, 35273, 38147, 34810, 28721, 42516, 42516, 42516, 34304, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 28243, 22657, 22657, 22657, 22657, 27401, 46316, 46316, 46316, 48318, 38147, 38147, 38147, 38147, 40896, 22382, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 38368, 22657, 22657, 22657, 22657, 38374, 22541, 46316, 46316, 46316, 46316, 44805, 38147, 38147, 38147, 38147, 38147, 32923, 22444, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 38370, 22657, 22657, 22657, 22657, 30360, 22540, 46316, 46316, 46316, 46316, 22343, 30533, 38147, 38147, 38147, 38147, 27826, 41766, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 23255, 22657, 22657, 22657, 22657, 30361, 22539, 46316, 46316, 46316, 46316, 30525, 38147, 38147, 38147, 38147, 34440, 41769, 42516, 42516, 42516, 42516, 42516, 28242, 22657, 22657, 22657, 37745, 46316, 46316, 46316, 37798, 38147, 38147, 38147, 27827, 22690, 42516, 42516, 42516, 42516, 38368, 22657, 22657, 40730, 46316, 46316, 36583, 38147, 38147, 40469, 42516, 42516, 42516, 42516, 38373, 22657, 28384, 46316, 46316, 36586, 38147, 27827, 22690, 34920, 42516, 38372, 43148, 46316, 28405, 38147, 22683, 36804, 38368, 28385, 28404, 30535, 36803, 38372, 46315, 38146, 28238, 43147, 27775, 22913, 28244, 40729, 36585, 22448, 28385, 36584, 34366, 22755, 31692, 28429, 22929, 23000, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 51361, 42516, 42516, 42516, 42516, 51537, 42516, 42516, 42516, 42516, 25917, 42516, 42516, 42516, 42516, 46741, 49861, 51029, 26099, 42516, 42516, 42516, 27632, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 28241, 22657, 46314, 35273, 38147, 34810, 22786, 42516, 42516, 42516, 34304, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 28243, 22657, 22657, 22657, 22657, 27401, 46316, 46316, 46316, 48318, 38147, 38147, 38147, 38147, 40896, 22382, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 38368, 22657, 22657, 22657, 22657, 38374, 22541, 46316, 46316, 46316, 46316, 44805, 38147, 38147, 38147, 38147, 38147, 32923, 22444, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 38370, 22657, 22657, 22657, 22657, 30360, 22540, 46316, 46316, 46316, 46316, 22343, 30533, 38147, 38147, 38147, 38147, 27826, 41766, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 23255, 22657, 22657, 22657, 22657, 30361, 22539, 46316, 46316, 46316, 46316, 30525, 38147, 38147, 38147, 38147, 34440, 41769, 42516, 42516, 42516, 42516, 42516, 28242, 22657, 22657, 22657, 37745, 46316, 46316, 46316, 37798, 38147, 38147, 38147, 27827, 22690, 42516, 42516, 42516, 42516, 38368, 22657, 22657, 40730, 46316, 46316, 36583, 38147, 38147, 40469, 42516, 42516, 42516, 42516, 38373, 22657, 28384, 46316, 46316, 36586, 38147, 27827, 22690, 34920, 42516, 38372, 43148, 46316, 28405, 38147, 22683, 36804, 38368, 28385, 28404, 30535, 36803, 38372, 46315, 38146, 28238, 43147, 27775, 22913, 28244, 40729, 36585, 22448, 28385, 36584, 34366, 22755, 31692, 28429, 22929, 23000, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 51361, 42516, 42516, 42516, 42516, 51537, 42516, 42516, 42516, 42516, 25917, 42516, 42516, 42516, 42516, 46741, 49861, 51029, 26258, 42516, 42516, 42516, 27632, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 28241, 22657, 46314, 35273, 38147, 34810, 28721, 42516, 42516, 42516, 34304, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 28243, 22657, 22657, 22657, 22657, 27401, 46316, 46316, 46316, 48318, 38147, 38147, 38147, 38147, 40896, 22382, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 38368, 22657, 22657, 22657, 22657, 38374, 22541, 46316, 46316, 46316, 46316, 44805, 38147, 38147, 38147, 38147, 38147, 32923, 22444, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 38370, 22657, 22657, 22657, 22657, 30360, 22540, 46316, 46316, 46316, 46316, 22343, 30533, 38147, 38147, 38147, 38147, 27826, 41766, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 23255, 22657, 22657, 22657, 22657, 30361, 22539, 46316, 46316, 46316, 46316, 30525, 38147, 38147, 38147, 38147, 34440, 41769, 42516, 42516, 42516, 42516, 42516, 28242, 22657, 22657, 22657, 37745, 46316, 46316, 46316, 37798, 38147, 38147, 38147, 27827, 22690, 42516, 42516, 42516, 42516, 38368, 22657, 22657, 40730, 46316, 46316, 36583, 38147, 38147, 40469, 42516, 42516, 42516, 42516, 38373, 22657, 28384, 46316, 46316, 36586, 38147, 27827, 22690, 34920, 42516, 38372, 43148, 46316, 28405, 38147, 22683, 36804, 38368, 28385, 28404, 30535, 36803, 38372, 46315, 38146, 28238, 43147, 27775, 22913, 28244, 40729, 36585, 22448, 28385, 36584, 34366, 22755, 31692, 28429, 22929, 23000, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 51361, 42516, 42516, 42516, 42516, 51537, 42516, 42516, 42516, 42516, 25917, 48694, 42516, 42516, 42516, 46741, 49861, 51029, 26099, 42516, 42516, 42516, 27632, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 28241, 22657, 46314, 35273, 38147, 34810, 28721, 42516, 42516, 42516, 34304, 42516, 42516, 42516, 28605, 42516, 42516, 42516, 42516, 28243, 22657, 22657, 22657, 22657, 27401, 46316, 46316, 46316, 48318, 38147, 38147, 38147, 38147, 40896, 22382, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 38368, 22657, 22657, 22657, 22657, 38374, 22541, 46316, 46316, 46316, 46316, 44805, 38147, 38147, 38147, 38147, 38147, 32923, 22444, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 38370, 22657, 22657, 22657, 22657, 30360, 22540, 46316, 46316, 46316, 46316, 22343, 30533, 38147, 38147, 38147, 38147, 27826, 41766, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 23255, 22657, 22657, 22657, 22657, 30361, 22539, 46316, 46316, 46316, 46316, 30525, 38147, 38147, 38147, 38147, 34440, 41769, 42516, 42516, 42516, 42516, 42516, 28242, 22657, 22657, 22657, 37745, 46316, 46316, 46316, 37798, 38147, 38147, 38147, 27827, 22690, 42516, 42516, 42516, 42516, 38368, 22657, 22657, 40730, 46316, 46316, 36583, 38147, 38147, 40469, 42516, 42516, 42516, 42516, 38373, 22657, 28384, 46316, 46316, 36586, 38147, 27827, 22690, 34920, 42516, 38372, 43148, 46316, 28405, 38147, 22683, 36804, 38368, 28385, 28404, 30535, 36803, 38372, 46315, 38146, 28238, 43147, 27775, 22913, 28244, 40729, 36585, 22448, 28385, 36584, 34366, 22755, 31692, 28429, 22929, 23000, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 51361, 42516, 42516, 42516, 42516, 51537, 42516, 42516, 42516, 42516, 25917, 42516, 42516, 42516, 42516, 47769, 26274, 26287, 26099, 42516, 42516, 42516, 27632, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 28241, 22657, 46314, 35273, 38147, 34810, 28721, 42516, 42516, 42516, 34304, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 28243, 22657, 22657, 22657, 22657, 27401, 46316, 46316, 46316, 48318, 38147, 38147, 38147, 38147, 40896, 22382, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 38368, 22657, 22657, 22657, 22657, 38374, 22541, 46316, 46316, 46316, 46316, 44805, 38147, 38147, 38147, 38147, 38147, 32923, 22444, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 38370, 22657, 22657, 22657, 22657, 30360, 22540, 46316, 46316, 46316, 46316, 22343, 30533, 38147, 38147, 38147, 38147, 27826, 41766, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 23255, 22657, 22657, 22657, 22657, 30361, 22539, 46316, 46316, 46316, 46316, 30525, 38147, 38147, 38147, 38147, 34440, 41769, 42516, 42516, 42516, 42516, 42516, 28242, 22657, 22657, 22657, 37745, 46316, 46316, 46316, 37798, 38147, 38147, 38147, 27827, 22690, 42516, 42516, 42516, 42516, 38368, 22657, 22657, 40730, 46316, 46316, 36583, 38147, 38147, 40469, 42516, 42516, 42516, 42516, 38373, 22657, 28384, 46316, 46316, 36586, 38147, 27827, 22690, 34920, 42516, 38372, 43148, 46316, 28405, 38147, 22683, 36804, 38368, 28385, 28404, 30535, 36803, 38372, 46315, 38146, 28238, 43147, 27775, 22913, 28244, 40729, 36585, 22448, 28385, 36584, 34366, 22755, 31692, 28429, 22929, 23000, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 51361, 42516, 42516, 42516, 42516, 51537, 42516, 42516, 42516, 42516, 25917, 42516, 42516, 42516, 42516, 46741, 49861, 51029, 26099, 42516, 42516, 42516, 27632, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 28241, 22657, 46314, 35273, 38147, 34810, 28721, 42516, 42516, 42516, 34304, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 28243, 22657, 22657, 22657, 22657, 27401, 46316, 46316, 46316, 48318, 38147, 38147, 38147, 38147, 40896, 22382, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 38368, 22657, 22657, 22657, 22657, 38374, 22541, 46316, 46316, 46316, 46316, 44805, 38147, 38147, 38147, 38147, 38147, 32923, 22444, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 38370, 22657, 22657, 22657, 22657, 30360, 22540, 46316, 46316, 46316, 46316, 22343, 30533, 38147, 38147, 38147, 38147, 27826, 41766, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 23255, 22657, 22657, 22657, 22657, 30361, 22539, 46316, 46316, 46316, 46316, 30525, 38147, 38147, 38147, 38147, 34440, 41769, 42516, 42516, 42516, 26303, 42516, 28242, 22657, 22657, 22657, 37745, 46316, 46316, 46316, 37798, 38147, 38147, 38147, 27827, 22690, 42516, 42516, 42516, 42516, 38368, 22657, 22657, 40730, 46316, 46316, 36583, 38147, 38147, 40469, 42516, 42516, 42516, 42516, 38373, 22657, 28384, 46316, 46316, 36586, 38147, 27827, 22690, 34920, 42516, 38372, 43148, 46316, 28405, 38147, 22683, 36804, 38368, 28385, 28404, 30535, 36803, 38372, 46315, 38146, 28238, 43147, 27775, 22913, 28244, 40729, 36585, 22448, 28385, 36584, 34366, 22755, 31692, 28429, 22929, 23000, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 26324, 42516, 42516, 42516, 42516, 51537, 42516, 42516, 42516, 42516, 25917, 42516, 42516, 42516, 42516, 46741, 49861, 51029, 26099, 42516, 42516, 42516, 27632, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 28241, 22657, 46314, 35273, 38147, 34810, 28721, 42516, 42516, 42516, 34304, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 28243, 22657, 22657, 22657, 22657, 27401, 46316, 46316, 46316, 48318, 38147, 38147, 38147, 38147, 40896, 22382, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 38368, 22657, 22657, 22657, 22657, 38374, 22541, 46316, 46316, 46316, 46316, 44805, 38147, 38147, 38147, 38147, 38147, 32923, 22444, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 38370, 22657, 22657, 22657, 22657, 30360, 22540, 46316, 46316, 46316, 46316, 22343, 30533, 38147, 38147, 38147, 38147, 27826, 41766, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 23255, 22657, 22657, 22657, 22657, 30361, 22539, 46316, 46316, 46316, 46316, 30525, 38147, 38147, 38147, 38147, 34440, 41769, 42516, 42516, 42516, 42516, 42516, 28242, 22657, 22657, 22657, 37745, 46316, 46316, 46316, 37798, 38147, 38147, 38147, 27827, 22690, 42516, 42516, 42516, 42516, 38368, 22657, 22657, 40730, 46316, 46316, 36583, 38147, 38147, 40469, 42516, 42516, 42516, 42516, 38373, 22657, 28384, 46316, 46316, 36586, 38147, 27827, 22690, 34920, 42516, 38372, 43148, 46316, 28405, 38147, 22683, 36804, 38368, 28385, 28404, 30535, 36803, 38372, 46315, 38146, 28238, 43147, 27775, 22913, 28244, 40729, 36585, 22448, 28385, 36584, 34366, 22755, 31692, 28429, 22929, 23000, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 21141, 42516, 42516, 42516, 42516, 48069, 42516, 42516, 42516, 42516, 25917, 35004, 26345, 26373, 26362, 26396, 35005, 26346, 21040, 42516, 42516, 42516, 27632, 42516, 42516, 51474, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 32629, 20505, 21066, 18953, 21068, 19019, 18919, 18270, 40105, 18293, 18315, 42516, 42516, 18360, 37963, 49435, 42820, 18380, 18411, 18395, 18574, 19226, 18433, 20169, 18459, 25598, 18486, 18508, 51199, 18465, 25604, 18492, 18514, 20728, 18530, 42516, 30418, 42516, 42516, 42516, 50250, 25964, 50770, 42516, 18553, 39446, 18564, 20175, 20411, 20131, 20505, 23879, 18590, 26008, 18612, 18632, 18596, 18470, 18594, 26012, 18616, 18636, 20798, 20741, 20757, 42516, 18652, 18689, 18711, 18748, 18695, 18788, 42516, 41546, 48871, 18807, 18835, 19802, 18823, 19840, 25206, 18851, 18904, 18942, 18979, 19680, 19006, 18851, 18904, 18942, 18979, 19680, 19045, 20750, 46697, 19097, 19132, 20936, 31767, 19148, 19174, 19196, 20141, 20921, 19788, 19212, 19242, 19278, 25525, 20579, 19301, 19596, 19271, 25518, 20572, 19294, 19589, 19324, 19353, 35639, 19379, 39467, 19401, 41728, 19421, 19500, 24458, 19544, 19575, 19614, 19632, 19650, 19559, 19612, 19630, 19648, 19666, 19029, 19696, 19712, 19728, 19753, 19773, 19830, 19435, 19449, 19888, 19905, 19856, 19887, 19904, 19921, 19947, 19984, 26423, 20080, 20119, 20157, 20536, 20191, 20213, 20542, 20197, 20219, 19931, 20235, 20264, 20301, 20317, 20346, 20333, 20349, 20365, 20381, 20397, 20422, 20606, 20438, 20477, 20503, 21067, 21068, 20521, 20558, 20595, 20622, 20652, 23888, 18878, 20683, 20699, 20715, 20779, 20814, 20880, 20890, 20906, 19463, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 21141, 42516, 42516, 42516, 42516, 48069, 42516, 42516, 42516, 42516, 25917, 42516, 42516, 42516, 42516, 42516, 42516, 42517, 21040, 42516, 42516, 42516, 27632, 42516, 42516, 51474, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 32629, 20505, 21066, 18953, 21068, 19019, 18919, 18270, 40105, 18293, 18315, 42516, 42516, 18360, 42516, 49435, 42820, 18380, 18411, 18395, 18574, 19226, 18433, 20169, 18459, 25598, 18486, 18508, 51199, 18465, 25604, 18492, 18514, 20728, 18530, 42516, 30418, 42516, 42516, 42516, 50250, 25964, 50770, 42516, 18553, 39446, 18564, 20175, 20411, 20131, 20505, 23879, 18590, 26008, 18612, 18632, 18596, 18470, 18594, 26012, 18616, 18636, 20798, 20741, 20757, 42516, 18652, 18689, 18711, 18748, 18695, 18788, 42516, 41546, 48871, 18807, 18835, 19802, 18823, 19840, 25206, 18851, 18904, 18942, 18979, 19680, 19006, 18851, 18904, 18942, 18979, 19680, 19045, 20750, 46697, 19097, 19132, 20936, 31767, 19148, 19174, 19196, 20141, 20921, 19788, 19212, 19242, 19278, 25525, 20579, 19301, 19596, 19271, 25518, 20572, 19294, 19589, 19324, 19353, 35639, 19379, 39467, 19401, 41728, 19421, 19500, 24458, 19544, 19575, 19614, 19632, 19650, 19559, 19612, 19630, 19648, 19666, 19029, 19696, 19712, 19728, 19753, 19773, 19830, 19435, 19449, 19888, 19905, 19856, 19887, 19904, 19921, 19947, 19984, 20043, 20080, 20119, 20157, 20536, 20191, 20213, 20542, 20197, 20219, 19931, 20235, 20264, 20301, 20317, 20346, 20333, 20349, 20365, 20381, 20397, 20422, 20606, 20438, 20477, 20503, 21067, 21068, 20521, 20558, 20595, 20622, 20652, 23888, 18878, 20683, 20699, 20715, 20779, 20814, 20880, 20890, 20906, 19463, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 21141, 42516, 42516, 42516, 42516, 48069, 42516, 42516, 42516, 42516, 25917, 47727, 42516, 26460, 26483, 26467, 42516, 47735, 21040, 42516, 42516, 42516, 27632, 42516, 42516, 51474, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 32629, 20505, 21066, 18953, 21068, 19019, 18919, 18270, 40105, 18293, 18315, 42516, 42516, 18360, 42516, 49435, 42820, 18380, 18411, 18395, 18574, 19226, 18433, 20169, 18459, 25598, 18486, 18508, 51199, 18465, 25604, 18492, 18514, 20728, 18530, 42516, 30418, 42516, 42516, 42516, 50250, 25964, 50770, 42516, 18553, 39446, 18564, 20175, 20411, 20131, 20505, 23879, 18590, 26008, 18612, 18632, 18596, 18470, 18594, 26012, 18616, 18636, 20798, 20741, 20757, 42516, 18652, 18689, 18711, 18748, 18695, 18788, 42516, 41546, 48871, 18807, 18835, 19802, 18823, 19840, 25206, 18851, 18904, 18942, 18979, 19680, 19006, 18851, 18904, 18942, 18979, 19680, 19045, 20750, 46697, 19097, 19132, 20936, 31767, 19148, 19174, 19196, 20141, 20921, 19788, 19212, 19242, 19278, 25525, 20579, 19301, 19596, 19271, 25518, 20572, 19294, 19589, 19324, 19353, 35639, 19379, 39467, 19401, 41728, 19421, 19500, 24458, 19544, 19575, 19614, 19632, 19650, 19559, 19612, 19630, 19648, 19666, 19029, 19696, 19712, 19728, 19753, 19773, 19830, 19435, 19449, 19888, 19905, 19856, 19887, 19904, 19921, 19947, 19984, 20043, 20080, 20119, 20157, 20536, 20191, 20213, 20542, 20197, 20219, 19931, 26499, 20264, 20301, 20317, 20346, 20333, 20349, 20365, 26535, 20397, 20422, 20606, 20438, 26551, 20503, 21067, 21068, 20521, 20558, 20595, 20622, 20652, 23888, 18878, 20683, 20699, 20715, 20779, 20814, 20880, 20890, 20906, 19463, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 51361, 29068, 42516, 42516, 42516, 51537, 26585, 26616, 29663, 26601, 26633, 26690, 26699, 26715, 26730, 26742, 26758, 26771, 26099, 19999, 42516, 26787, 26162, 26806, 41856, 26832, 26883, 26907, 26946, 30733, 26962, 26998, 27019, 27054, 46783, 31896, 27070, 46885, 46348, 36728, 27120, 42516, 42516, 34304, 27137, 37896, 42516, 27163, 34995, 39190, 42516, 27184, 43650, 22657, 22657, 22657, 27203, 27401, 46316, 46316, 41985, 48318, 38147, 38147, 38147, 37213, 40896, 22382, 42516, 27222, 48046, 42516, 27243, 27263, 27280, 42516, 42516, 27038, 20763, 27308, 27328, 22657, 22657, 27363, 27386, 27419, 35266, 46316, 30482, 27448, 27495, 27524, 27566, 38147, 35889, 38740, 46095, 27585, 26152, 27628, 42516, 45564, 42516, 47310, 23487, 42516, 46936, 27648, 27669, 27685, 34339, 22657, 27729, 22984, 44673, 27748, 45967, 46316, 27773, 39768, 32782, 27791, 37607, 38147, 27824, 22485, 27843, 27859, 27882, 27899, 42516, 27922, 31758, 27943, 23255, 33056, 27962, 22657, 42185, 33093, 27982, 30091, 46316, 35423, 43042, 32774, 28017, 38147, 38147, 42363, 28033, 23138, 28062, 40096, 28115, 41486, 28137, 21969, 22956, 28160, 22657, 46191, 35558, 28176, 28197, 32038, 49740, 47969, 28214, 27827, 28233, 42899, 42516, 21253, 42516, 38368, 42934, 43766, 40730, 37393, 48025, 36583, 51021, 44713, 40469, 42516, 42516, 42516, 42516, 38373, 22657, 28384, 46316, 46316, 36586, 38147, 27827, 22690, 34920, 42516, 38372, 43148, 46316, 28405, 38147, 22683, 36804, 38368, 28385, 28404, 30535, 28260, 28298, 33550, 28320, 28341, 28379, 28401, 28421, 26921, 40729, 36585, 44924, 28385, 36584, 34366, 22755, 31692, 28429, 22929, 23000, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 51361, 42516, 28445, 42516, 42516, 51537, 42516, 42516, 42516, 42516, 31409, 28464, 42516, 28489, 28496, 28512, 28528, 28541, 26099, 42516, 42516, 42516, 27632, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 28241, 42942, 46314, 28557, 30860, 34810, 28721, 42516, 42516, 42516, 34304, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 28243, 22657, 22657, 22657, 22657, 27401, 46316, 46316, 46316, 48318, 38147, 38147, 38147, 38147, 40896, 22382, 42516, 42516, 42516, 42516, 42516, 28583, 23487, 28604, 45602, 42516, 42516, 38368, 22657, 22657, 22657, 44648, 28621, 22541, 46316, 46316, 46316, 40008, 44805, 38147, 38147, 38147, 38147, 41449, 32923, 26115, 26152, 42516, 42516, 42516, 42516, 42516, 23487, 42516, 42516, 42516, 38370, 22657, 22657, 22657, 22657, 30360, 22540, 46316, 46316, 46316, 46316, 22343, 30533, 38147, 38147, 38147, 38147, 27826, 35762, 23510, 42516, 32305, 42516, 42516, 28652, 43395, 23255, 28671, 22657, 22657, 22657, 28690, 28706, 46316, 46316, 46316, 37589, 32774, 28762, 38147, 38147, 38147, 44356, 41769, 42516, 42516, 42516, 42516, 42516, 28242, 22657, 22657, 22657, 46191, 46316, 46316, 46316, 32038, 38147, 38147, 38147, 27827, 22690, 42516, 42516, 42516, 42516, 38368, 22657, 22657, 40730, 46316, 46316, 36583, 38147, 38147, 40469, 42516, 42516, 42516, 42516, 38373, 22657, 28384, 46316, 46316, 36586, 38147, 27827, 22690, 34920, 42516, 38372, 43148, 46316, 28405, 38147, 22683, 36804, 38368, 28385, 28404, 30535, 36803, 38372, 46315, 38146, 23013, 26674, 44310, 41661, 28244, 40729, 36585, 22448, 28385, 36584, 34366, 22755, 31692, 28429, 22929, 23000, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 51361, 42516, 28784, 42516, 42516, 51537, 42516, 28813, 42516, 28831, 36313, 28851, 28860, 28876, 28891, 28903, 28919, 28932, 26099, 42516, 42516, 33807, 29427, 42516, 42516, 28948, 42516, 28965, 42516, 42516, 42516, 25275, 42516, 28986, 44283, 46314, 29021, 22622, 37863, 32069, 42516, 29066, 42516, 34304, 29084, 49128, 42516, 45291, 29103, 36853, 42516, 39408, 30216, 37513, 22657, 22657, 22657, 29138, 29162, 46316, 46316, 48917, 29187, 29203, 38147, 38147, 40896, 22382, 23471, 42516, 42516, 37970, 42516, 41320, 23487, 39926, 29222, 42516, 42516, 38368, 29257, 38293, 22657, 29296, 38374, 29315, 31567, 38618, 38779, 46316, 44805, 42631, 29346, 29367, 48610, 38147, 32923, 26115, 26152, 42516, 42516, 35359, 42516, 36267, 23487, 42516, 42516, 42516, 38370, 22657, 22657, 38875, 29587, 30360, 22540, 46316, 46316, 49716, 48753, 22343, 30533, 38147, 38147, 47896, 47206, 27826, 35762, 29388, 42516, 42516, 42516, 29423, 42516, 29443, 23255, 22657, 22657, 22657, 43545, 22877, 22539, 46316, 46316, 49625, 46316, 32774, 38147, 38147, 38147, 29464, 34440, 41769, 42516, 42516, 19074, 42516, 46902, 29483, 22657, 45738, 22657, 46191, 46316, 28001, 46316, 32038, 38147, 48630, 38147, 27827, 29518, 29546, 42516, 39530, 20864, 38368, 29566, 29583, 46250, 46316, 29603, 36583, 29623, 29640, 40469, 42516, 42516, 42516, 29661, 38373, 22657, 29679, 46316, 51108, 36586, 38147, 50541, 22690, 34920, 42516, 38372, 43148, 46316, 28405, 38147, 22683, 36804, 38368, 28385, 28404, 30535, 36803, 38372, 46315, 38146, 29702, 29722, 29738, 29754, 28244, 40729, 36585, 22448, 28385, 36584, 34366, 22755, 31692, 34720, 29778, 23000, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 51361, 42516, 26790, 42516, 42516, 51537, 42516, 29812, 25227, 42516, 31450, 29831, 29840, 29856, 29871, 29878, 29894, 29907, 26099, 42516, 42516, 42516, 27632, 42516, 42516, 42516, 42516, 42516, 42516, 31072, 31070, 42516, 32276, 32283, 40826, 29686, 35273, 29923, 34810, 28721, 42516, 35352, 29943, 34304, 42516, 29968, 36872, 42516, 42516, 29988, 30012, 34471, 30028, 22581, 34160, 30064, 24121, 30107, 48780, 30128, 30172, 30191, 29624, 22523, 47650, 28217, 40896, 22382, 42516, 42516, 42516, 42735, 42516, 30232, 23487, 42516, 42516, 42516, 27906, 38368, 22657, 22657, 22657, 26867, 38374, 22541, 46316, 46316, 46316, 37425, 44805, 38147, 38147, 38147, 38147, 30252, 50615, 30272, 30304, 42516, 42516, 42516, 30320, 30338, 23487, 42516, 42516, 27104, 21782, 30358, 30377, 22657, 22657, 30395, 30434, 30481, 30498, 46316, 46316, 30515, 51071, 30551, 30568, 38147, 38147, 30585, 35762, 23510, 42516, 42516, 42516, 42516, 42516, 30625, 23255, 30648, 22657, 22657, 30668, 22877, 30687, 46316, 46316, 34855, 46316, 32837, 38147, 38147, 43217, 38147, 34440, 41769, 45029, 42516, 30729, 42516, 21024, 28242, 29299, 22657, 30749, 46191, 50174, 46316, 30776, 32038, 38147, 30812, 48472, 27827, 22690, 23708, 42516, 42516, 42516, 38368, 22657, 22657, 40730, 46316, 46316, 36583, 38147, 38147, 40469, 42516, 37071, 42516, 42516, 38373, 22657, 31523, 46316, 43307, 36586, 38147, 39352, 22690, 34920, 42516, 38372, 43148, 46316, 38137, 38147, 30829, 36804, 38368, 28385, 28404, 30535, 27085, 28354, 31634, 30852, 30600, 30074, 38197, 42103, 28244, 40729, 36585, 22448, 28385, 36584, 34366, 22755, 31692, 28429, 22929, 23000, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 51361, 42516, 21805, 30876, 42516, 51537, 49762, 30895, 42516, 30918, 43094, 30942, 30951, 30967, 30982, 30994, 31010, 31023, 26099, 42516, 31039, 39573, 31055, 20844, 31088, 31151, 40795, 31438, 31118, 31138, 31170, 31208, 31224, 31259, 31325, 39316, 31341, 31357, 42305, 39859, 31243, 31397, 39944, 31425, 33906, 31466, 31484, 36903, 42516, 42516, 38844, 42516, 28243, 31508, 40831, 43064, 26242, 31546, 31583, 49551, 32501, 35578, 43001, 31607, 35023, 30465, 31623, 22382, 40777, 39000, 31658, 40711, 31716, 41838, 31747, 42230, 41234, 31783, 31813, 31829, 31873, 31912, 31936, 31952, 31988, 32019, 35868, 32054, 38034, 47598, 32094, 32110, 32136, 39813, 32152, 32186, 47170, 32233, 26152, 32263, 42516, 49503, 42516, 32299, 32321, 32337, 32365, 32386, 32406, 32424, 22657, 35078, 32463, 30760, 22540, 32487, 46316, 47681, 50971, 43200, 30533, 32517, 38147, 37625, 38481, 40265, 32560, 23510, 18666, 32576, 32596, 39698, 32624, 37664, 32645, 31965, 32706, 47478, 32693, 32731, 32762, 40433, 46316, 32798, 32825, 32774, 32853, 32872, 49013, 32893, 32919, 41769, 42516, 21429, 46706, 42516, 42516, 32939, 25143, 43551, 27695, 46191, 47568, 50517, 33128, 32038, 44391, 37914, 44027, 27827, 22690, 51501, 22158, 32960, 23571, 32997, 33045, 33080, 33109, 33144, 33160, 33196, 33225, 33259, 33289, 25257, 41794, 33305, 33321, 22943, 47433, 44826, 33342, 33363, 33392, 45913, 37247, 22690, 23238, 33434, 38372, 47456, 46316, 49977, 38147, 33455, 33478, 32944, 28385, 33515, 33536, 30206, 38372, 33571, 38971, 31372, 39304, 33587, 33611, 33627, 33643, 33674, 33720, 28385, 36584, 33273, 33742, 31692, 28429, 22929, 23000, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 51361, 42516, 42516, 33784, 42516, 51537, 33805, 33823, 29972, 43389, 33866, 33897, 42516, 33882, 33940, 33947, 33963, 33976, 26099, 42516, 42516, 20279, 27632, 42516, 42516, 42516, 42516, 42516, 41269, 42516, 41261, 42516, 41268, 25123, 41628, 38176, 37400, 33992, 34810, 28721, 42516, 34012, 42516, 34304, 34012, 42516, 42516, 42516, 34029, 34013, 37295, 42516, 28243, 34045, 22657, 22657, 22657, 34069, 34103, 46316, 46316, 48318, 47159, 38147, 38147, 38147, 40896, 22382, 42516, 42516, 42516, 42516, 42516, 42516, 23487, 42516, 42516, 45931, 42516, 38368, 22657, 22657, 22657, 44213, 38374, 22541, 46316, 46316, 46316, 42968, 44805, 38147, 38147, 38147, 38147, 34120, 32923, 26115, 26152, 42516, 42516, 42516, 42516, 42516, 23487, 42516, 46602, 42516, 38370, 22657, 22657, 30041, 22657, 30360, 22540, 46316, 46316, 42268, 46316, 22343, 30533, 38147, 38147, 44000, 38147, 27826, 35762, 23510, 42516, 42516, 42516, 42516, 42516, 39273, 23255, 22657, 22657, 22657, 37139, 22877, 22539, 46316, 46316, 49643, 46316, 32774, 38147, 38147, 38147, 48815, 34440, 41769, 42516, 42516, 42516, 42516, 42516, 28242, 22657, 22657, 22657, 46191, 46316, 46316, 46316, 32038, 38147, 38147, 38147, 27827, 22690, 42516, 42516, 42516, 42516, 38368, 22657, 22657, 40730, 46316, 46316, 36583, 38147, 38147, 40469, 46733, 42516, 42516, 42516, 36647, 22657, 28384, 27995, 46316, 49984, 38147, 27827, 22690, 34920, 42516, 38372, 43148, 46316, 28405, 38147, 22683, 36804, 38368, 28385, 28404, 30535, 36803, 38372, 46315, 38146, 28238, 43147, 27775, 22913, 28244, 40729, 36585, 22448, 28385, 36584, 44874, 43254, 31692, 29762, 34139, 23000, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 51361, 42516, 42516, 42516, 42516, 51537, 42516, 38852, 42516, 42516, 51280, 34176, 34185, 34201, 34208, 34215, 34231, 34244, 26099, 42516, 42516, 42516, 27632, 42516, 42516, 34401, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 28241, 22657, 46314, 35273, 38147, 34810, 37878, 42057, 42516, 42516, 34304, 42516, 42516, 42516, 42516, 34260, 42516, 42516, 42516, 28243, 22657, 22657, 22657, 22657, 27401, 46316, 46316, 46316, 48318, 38147, 38147, 38147, 38147, 40896, 22382, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 34278, 42516, 28081, 38368, 22657, 22657, 22657, 22657, 38374, 42076, 46316, 46316, 46316, 46316, 46212, 38147, 38147, 38147, 38147, 38147, 32923, 22444, 42516, 42516, 42516, 47761, 42516, 34299, 19108, 42516, 42516, 19113, 34320, 22657, 22657, 34338, 22657, 30360, 22540, 46316, 34104, 46316, 46316, 22343, 30533, 38147, 42837, 38147, 38147, 27826, 34355, 42516, 42516, 42516, 34399, 42516, 42516, 42516, 23255, 22657, 22657, 22657, 22657, 30361, 22539, 46316, 46316, 46316, 46316, 30525, 38147, 38147, 38147, 38147, 34440, 41769, 42516, 42516, 38663, 42516, 34417, 28242, 22657, 22657, 38760, 37745, 46316, 46316, 50996, 37798, 38147, 38147, 38147, 34438, 22690, 34456, 18364, 34487, 34538, 24114, 31309, 31274, 34559, 34565, 27432, 34581, 44559, 30458, 34604, 22076, 42516, 19757, 34657, 29792, 34679, 47804, 33555, 31559, 34736, 35451, 34758, 22690, 41150, 39051, 35676, 32439, 38610, 34776, 34795, 22683, 36804, 38368, 28385, 28404, 30535, 36803, 38372, 46315, 38146, 28238, 43147, 27775, 22913, 28244, 40729, 36585, 22448, 28385, 36584, 34366, 22755, 34833, 34871, 22929, 23000, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 51361, 42516, 42516, 42516, 42516, 51537, 42516, 42516, 42516, 42516, 25917, 42516, 42516, 42516, 42516, 46741, 49861, 51029, 26099, 18277, 42516, 42516, 34899, 42516, 42516, 42516, 36062, 42516, 36065, 42516, 42516, 34947, 34936, 36697, 22657, 34963, 27757, 38148, 48007, 34986, 42516, 42516, 29815, 34304, 21387, 19116, 42516, 38636, 42516, 42516, 42516, 42516, 28243, 22657, 22657, 43325, 22657, 27401, 46316, 46316, 30705, 48318, 38147, 38147, 38147, 35021, 40896, 22382, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 35376, 42516, 35039, 38368, 22657, 22657, 22657, 22657, 50945, 22541, 46316, 46316, 46316, 46316, 38717, 38147, 38147, 38147, 38147, 38147, 41757, 22444, 42516, 35059, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 39196, 38370, 35077, 22657, 22657, 22657, 30360, 46821, 46316, 46316, 46316, 46316, 22343, 23541, 38147, 38147, 38147, 38147, 27826, 41766, 42516, 42516, 42516, 50318, 42516, 42516, 42516, 23255, 22657, 22657, 22657, 46040, 30361, 22539, 46316, 46316, 46316, 34080, 30525, 38147, 38147, 38147, 42204, 34440, 41769, 42516, 42516, 46608, 42516, 42516, 28242, 22657, 22657, 22657, 37745, 46316, 46316, 46316, 37798, 38147, 38147, 38147, 27827, 22690, 42516, 42516, 42516, 42516, 38368, 22657, 22657, 40730, 46316, 46316, 36583, 38147, 38147, 40469, 42516, 23434, 42516, 22428, 38373, 22657, 28384, 46316, 46316, 36586, 38147, 27827, 35094, 34920, 22560, 38372, 50497, 46316, 40393, 38147, 43581, 36804, 38368, 28385, 28404, 30535, 36803, 38372, 46315, 38146, 28238, 43147, 27775, 22913, 28244, 40729, 36585, 22448, 28385, 36584, 34366, 22755, 31692, 28429, 22929, 23000, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 51361, 42516, 42516, 44978, 42516, 51537, 30322, 33499, 42482, 50012, 50022, 18763, 18772, 35115, 35131, 35143, 35159, 35172, 26099, 42516, 42516, 42516, 36498, 41847, 42516, 42516, 42516, 31671, 41803, 35188, 36161, 35205, 35221, 43491, 35237, 35256, 29171, 35289, 37464, 35305, 42516, 42516, 42516, 35339, 42516, 28090, 35375, 42516, 42516, 28949, 35314, 39266, 28243, 48989, 48108, 22657, 35392, 47509, 37433, 35422, 34970, 35439, 27808, 35467, 38147, 35501, 35536, 22382, 42516, 35594, 39908, 48509, 34515, 35614, 42516, 18299, 18417, 42516, 35655, 35672, 48116, 22657, 43129, 29280, 37184, 22541, 48019, 46316, 38117, 35692, 47637, 38147, 35719, 38147, 35737, 27472, 35753, 22444, 42516, 42516, 42516, 42516, 35778, 48535, 35820, 42516, 42516, 42516, 23024, 36244, 22657, 22657, 29502, 38418, 35859, 41980, 46316, 46316, 48733, 41934, 49389, 35884, 38147, 38147, 40673, 49832, 35905, 42516, 36909, 35924, 35976, 35997, 18673, 36033, 34522, 36081, 47410, 36098, 36117, 36141, 36177, 46316, 36198, 32809, 43953, 30446, 38147, 43626, 42012, 36214, 36230, 23328, 20636, 45205, 36266, 36283, 36301, 35981, 36329, 26662, 46771, 36350, 36390, 48253, 35703, 36411, 36456, 40231, 38454, 41461, 37653, 42516, 42516, 37028, 36479, 40577, 43121, 45398, 22366, 37772, 48725, 38205, 36432, 50457, 40469, 42516, 25485, 42516, 36514, 38373, 22657, 28384, 46316, 46316, 36586, 38147, 27827, 36626, 36531, 42516, 28996, 31885, 36559, 36579, 36602, 36619, 36804, 36642, 28385, 28404, 30535, 42423, 49897, 44263, 36663, 36694, 43147, 27775, 22913, 28244, 40729, 36585, 22448, 36713, 36744, 34366, 22755, 31692, 28429, 36774, 36790, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 51361, 42516, 42516, 39935, 42516, 51537, 42516, 42516, 42516, 42516, 25917, 42516, 29952, 39942, 19158, 38671, 36820, 36833, 26099, 42516, 42516, 42516, 27632, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 28241, 22657, 46314, 35273, 38147, 34810, 28721, 42516, 42516, 42516, 34304, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 28243, 22657, 22657, 44206, 22657, 27401, 46316, 46317, 46316, 48318, 38147, 38147, 34588, 38147, 40896, 22382, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 38368, 22657, 22657, 22657, 22657, 38374, 22541, 46316, 46316, 46316, 46316, 44805, 38147, 38147, 38147, 38147, 38147, 32923, 22444, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 38370, 22657, 22657, 22657, 22657, 30360, 22540, 46316, 46316, 46316, 46316, 22343, 30533, 38147, 38147, 38147, 38147, 27826, 41766, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 23255, 22657, 22657, 22657, 22657, 30361, 22539, 46316, 46316, 46316, 46316, 30525, 38147, 38147, 38147, 38147, 34440, 41769, 42516, 42516, 42516, 42516, 42516, 28242, 22657, 22657, 22657, 37745, 46316, 46316, 46316, 37798, 38147, 38147, 38147, 27827, 22690, 42516, 42516, 42516, 42516, 38368, 22657, 22657, 40730, 46316, 46316, 36583, 38147, 38147, 40469, 42516, 42516, 42516, 42516, 38373, 22657, 28384, 46316, 46316, 36586, 38147, 27827, 22690, 34920, 42516, 38372, 43148, 46316, 28405, 38147, 22683, 36804, 38368, 28385, 28404, 30535, 36803, 38372, 46315, 38146, 28238, 43147, 27775, 22913, 28244, 40729, 36585, 22448, 28385, 36584, 34366, 22755, 31692, 28429, 22929, 23000, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 51361, 42516, 42516, 19484, 42516, 51537, 42516, 42516, 42516, 42516, 25917, 42516, 42516, 42516, 42516, 46741, 49861, 51029, 26099, 42516, 42516, 42516, 27632, 42516, 42516, 36849, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 28241, 22657, 46314, 35273, 38147, 34810, 28721, 42516, 42516, 42516, 34304, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 28243, 22657, 22657, 22657, 22657, 27401, 46316, 46316, 46316, 48318, 38147, 38147, 38147, 38147, 40896, 22382, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 38368, 22657, 22657, 22657, 22657, 38374, 22541, 46316, 46316, 46316, 46316, 44805, 38147, 38147, 38147, 38147, 38147, 32923, 22444, 42516, 36869, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 46294, 22657, 22657, 22657, 22657, 30360, 49232, 46316, 46316, 46316, 46316, 22343, 30796, 38147, 38147, 38147, 38147, 27826, 41766, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 23255, 22657, 22657, 22657, 22657, 30361, 22539, 46316, 46316, 46316, 46316, 30525, 38147, 38147, 38147, 38147, 34440, 41769, 42516, 42516, 42516, 42516, 42516, 28242, 22657, 22657, 22657, 37745, 46316, 46316, 46316, 37798, 38147, 38147, 38147, 27827, 22690, 42516, 42516, 42516, 42516, 38368, 22657, 22657, 40730, 46316, 46316, 36583, 38147, 38147, 40469, 42516, 42516, 42516, 42516, 38373, 22657, 28384, 46316, 46316, 36586, 38147, 27827, 22690, 34920, 42516, 38372, 43148, 46316, 28405, 38147, 22683, 36804, 38368, 28385, 28404, 30535, 36803, 38372, 46315, 38146, 28238, 43147, 27775, 22913, 28244, 40729, 36585, 22448, 28385, 36584, 34366, 22755, 31692, 28429, 22929, 23000, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 51361, 42516, 42516, 46996, 42516, 51537, 42516, 23354, 42516, 42753, 36888, 36925, 36938, 36954, 36961, 36977, 36993, 37006, 26099, 42516, 37022, 42516, 27632, 42516, 41516, 26816, 42516, 42516, 37044, 42516, 37049, 18537, 37065, 37087, 37117, 44086, 37200, 37235, 39844, 33658, 37263, 42516, 42516, 35791, 42516, 42516, 34914, 28072, 42516, 37291, 22389, 37311, 18926, 37716, 37348, 43712, 48153, 37366, 37382, 46316, 37416, 40607, 40193, 49587, 35520, 37449, 27508, 37480, 37934, 42516, 42516, 42516, 42516, 41144, 42516, 42516, 42516, 42516, 42516, 38368, 37503, 36125, 22657, 37529, 39594, 37550, 37566, 48265, 46316, 37588, 30713, 29645, 37605, 37623, 38799, 38147, 37641, 22444, 42516, 42516, 37688, 42516, 42516, 49494, 24999, 42516, 42516, 42516, 38370, 22657, 22657, 29494, 37704, 37740, 22540, 46316, 46316, 40001, 37761, 37795, 30533, 38147, 38147, 37825, 37814, 37848, 41766, 42516, 37894, 42516, 42516, 51139, 42516, 42516, 23255, 22657, 41398, 22657, 22657, 30361, 22539, 42274, 46316, 46316, 46316, 30525, 38147, 37912, 38147, 38147, 34440, 41769, 37930, 36490, 42516, 34663, 37950, 28242, 37986, 22657, 22657, 46812, 38013, 46316, 46316, 45485, 49261, 38147, 38147, 48821, 22690, 49311, 42516, 42516, 38050, 38368, 41404, 22657, 40730, 47057, 46316, 36583, 48937, 38147, 40469, 42516, 21575, 42516, 42516, 38373, 22657, 38069, 46316, 45780, 36586, 38147, 40463, 22690, 34920, 42516, 38372, 43148, 46316, 28405, 38147, 22683, 36804, 38368, 28385, 28404, 30535, 36803, 38372, 46315, 38146, 28238, 43147, 27775, 22913, 28244, 38089, 46273, 22694, 23034, 40134, 34366, 22755, 38105, 38164, 22929, 23000, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 51361, 42516, 42516, 26200, 42516, 51537, 42516, 27946, 42516, 42516, 47834, 20951, 20960, 38221, 38236, 38243, 38259, 38272, 26258, 42516, 38551, 42516, 27632, 42516, 34262, 42516, 42516, 42516, 38553, 42516, 38546, 21091, 38552, 21774, 38288, 31700, 35273, 38309, 34810, 28721, 42516, 42516, 38329, 34304, 42516, 29241, 23811, 23953, 42516, 42516, 42516, 38347, 38364, 33022, 22657, 38390, 38410, 28636, 46316, 29607, 39628, 38434, 42370, 38147, 48603, 38470, 47663, 38504, 38541, 39253, 39681, 42516, 42516, 42516, 42516, 41197, 42516, 42516, 42516, 38368, 31857, 22657, 38569, 22657, 38374, 22541, 38589, 28198, 46316, 46316, 44805, 29206, 27800, 32877, 38147, 38147, 32923, 22444, 42516, 28144, 42516, 38634, 42516, 45188, 42516, 38652, 42516, 42516, 38370, 39723, 22657, 22657, 38687, 30379, 22540, 23043, 46316, 46316, 38710, 22895, 30533, 49398, 38147, 38147, 38733, 27826, 41766, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 23255, 22657, 22657, 22657, 22657, 30361, 22539, 46316, 46316, 46316, 46316, 30525, 38147, 38147, 38147, 38147, 34440, 41769, 42516, 42516, 42516, 42516, 42516, 28242, 22657, 22657, 22657, 37745, 46316, 46316, 46316, 37798, 38147, 38147, 38147, 27827, 33462, 42516, 42516, 42516, 42516, 38368, 38756, 22657, 40730, 38776, 46316, 36583, 38795, 38147, 40469, 42516, 42516, 42516, 42516, 38373, 22657, 28384, 46316, 38815, 36586, 49663, 27827, 22690, 34920, 42516, 38372, 43148, 46316, 28405, 38147, 22683, 49485, 38368, 44256, 40047, 45820, 38834, 38868, 32210, 38891, 28238, 43147, 27775, 22913, 28244, 40729, 36585, 22448, 28385, 38914, 38935, 22755, 31692, 28429, 22929, 23000, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 51361, 42516, 42516, 27187, 42516, 30926, 42516, 34543, 38987, 37672, 39016, 39067, 39107, 39092, 39111, 39076, 39127, 39140, 26099, 42516, 25841, 23701, 20829, 39222, 20009, 42516, 39156, 41252, 39175, 39212, 32746, 21195, 39238, 26847, 39289, 45668, 39332, 39368, 34810, 28721, 32390, 42516, 42516, 26444, 30879, 36010, 28099, 40787, 39398, 38525, 26035, 39432, 39483, 44460, 39503, 41633, 22657, 32003, 22601, 46316, 49709, 48318, 47976, 47889, 30552, 38147, 33209, 39523, 39917, 39159, 42516, 39546, 42516, 42516, 40976, 42516, 42516, 42516, 39568, 39487, 37157, 22657, 39589, 22657, 38374, 39610, 45436, 46316, 39644, 46316, 44805, 27542, 39661, 38313, 38147, 38147, 32923, 22444, 42516, 42516, 42516, 42516, 30632, 42516, 42516, 21187, 39678, 39697, 39714, 22657, 22657, 50559, 22657, 39739, 39760, 46316, 46316, 38818, 46316, 39784, 39805, 38147, 38147, 32856, 38147, 39829, 41766, 46006, 39893, 48391, 49682, 42516, 39960, 34641, 39976, 38394, 40024, 37997, 33010, 30361, 22539, 40063, 40121, 43817, 40157, 30525, 40868, 40175, 40209, 40247, 34760, 44916, 20285, 40281, 34623, 40309, 51090, 50324, 48429, 40325, 40344, 40364, 39645, 40409, 40428, 40449, 38147, 40485, 49825, 39382, 22690, 40506, 42516, 42516, 22216, 40534, 39507, 48098, 30083, 50926, 38024, 33595, 45640, 28567, 40469, 40559, 42516, 40087, 26129, 45371, 31289, 28384, 40593, 41927, 40637, 40665, 40689, 34614, 41203, 40707, 27312, 43148, 48295, 28405, 40861, 22683, 50287, 38368, 40727, 40746, 40649, 40767, 40811, 22827, 40847, 33407, 49334, 48207, 33243, 28244, 40729, 36585, 22448, 28385, 36584, 41673, 40884, 31692, 28429, 22929, 40922, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 51361, 42516, 42516, 42516, 40951, 51537, 42516, 42744, 40972, 42516, 27292, 40992, 41001, 41017, 41032, 41039, 41055, 41068, 26099, 20248, 41084, 41102, 41129, 21286, 41166, 42516, 41182, 41219, 41286, 31731, 41302, 41336, 41352, 41368, 41384, 34883, 41420, 41436, 48349, 41477, 42516, 42240, 27121, 34422, 42516, 41511, 25267, 41532, 41575, 42516, 42516, 41594, 28243, 22657, 31843, 41613, 34053, 27401, 40159, 35568, 34847, 41649, 38147, 27550, 41689, 41744, 40896, 22382, 42516, 26380, 42516, 41578, 42516, 44762, 41785, 42516, 41828, 27866, 41270, 19255, 22657, 41881, 41897, 22657, 35406, 41916, 43971, 44095, 46316, 46316, 41950, 22851, 37219, 42001, 38147, 29467, 32923, 42028, 42044, 42516, 46644, 19385, 42516, 42516, 42516, 42516, 42516, 34632, 38370, 22657, 27339, 43920, 22657, 30360, 22540, 46316, 44514, 36395, 46316, 22343, 30533, 38147, 35478, 49592, 38147, 27826, 41766, 42516, 42516, 21305, 42516, 31235, 42516, 42516, 23255, 22657, 22657, 22657, 22657, 20064, 42074, 46316, 46316, 46316, 46316, 42093, 38147, 38147, 38147, 38147, 50734, 41769, 50664, 35952, 42516, 42516, 42516, 41113, 22657, 42177, 22657, 45759, 46316, 43945, 46316, 45799, 38147, 45843, 38147, 27827, 29041, 42127, 42516, 42516, 42516, 48412, 30671, 22657, 38073, 44692, 46316, 33520, 45533, 38147, 40469, 42146, 42516, 42516, 42516, 38373, 22657, 28384, 46316, 46316, 36586, 38147, 27827, 43588, 34920, 42516, 42166, 40348, 33173, 48271, 42201, 22683, 42220, 41559, 42256, 42290, 42321, 33756, 44448, 35547, 42352, 28238, 43147, 27775, 22913, 28244, 40729, 36585, 22448, 28385, 36584, 32531, 22755, 42386, 28429, 22929, 42410, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 51361, 42516, 42516, 42516, 39045, 51537, 42516, 42516, 42439, 19363, 42461, 42498, 42516, 31468, 42533, 42547, 42563, 42576, 26099, 42516, 42516, 25867, 27632, 42516, 42516, 42516, 42592, 26617, 42516, 42516, 26436, 25867, 42516, 30236, 31920, 49620, 42612, 36440, 34810, 28721, 42516, 42647, 42516, 34304, 42516, 44608, 36046, 42516, 35598, 42516, 42516, 36057, 51481, 44194, 32471, 36334, 37148, 42671, 32033, 42687, 33121, 42706, 38488, 30256, 30569, 29372, 32199, 42722, 42778, 42516, 42799, 42516, 49524, 50043, 35835, 42516, 42516, 42516, 42516, 38368, 22657, 22657, 37175, 22657, 38374, 22541, 46316, 46316, 47563, 46316, 44805, 38147, 38147, 38147, 42836, 38147, 32923, 42853, 42888, 35656, 42516, 42516, 42516, 50763, 30902, 39460, 48700, 42516, 38370, 22657, 42923, 22657, 22657, 37350, 22540, 46316, 42958, 46316, 46316, 27403, 30533, 38147, 42991, 38147, 38147, 27826, 41766, 42516, 29233, 42516, 42516, 42516, 43017, 42516, 23255, 22657, 22657, 49093, 22657, 22325, 22539, 46316, 46316, 43039, 46316, 30788, 38147, 38147, 35485, 38147, 34440, 41769, 42516, 42516, 42516, 42516, 42516, 28242, 22657, 22657, 22657, 37745, 46316, 46316, 46316, 37798, 38147, 38147, 38147, 27827, 22690, 42516, 36017, 42516, 42516, 43058, 22657, 22657, 40730, 46316, 46316, 36583, 38147, 38147, 48843, 42516, 42516, 42516, 42516, 38373, 22657, 28384, 46316, 46316, 36586, 38147, 27827, 22690, 34920, 42516, 38372, 43148, 46316, 28405, 38147, 46409, 43080, 43110, 32715, 30112, 30535, 40935, 43145, 46315, 38146, 28238, 43147, 27775, 22913, 28244, 40729, 36585, 22448, 28385, 36584, 34366, 22755, 31692, 28429, 22929, 23000, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 51361, 42516, 42516, 42516, 42516, 51537, 42516, 42516, 42516, 42516, 25917, 42516, 42516, 42516, 42516, 46741, 49861, 51029, 26099, 42516, 42516, 42516, 27632, 42516, 42516, 35061, 42516, 42516, 42516, 28279, 28282, 18254, 28275, 31684, 43164, 43182, 29146, 43216, 43233, 29330, 43270, 42516, 42516, 34304, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 28243, 22657, 22657, 22657, 22657, 27401, 46316, 46316, 46316, 48318, 38147, 38147, 38147, 38147, 40896, 22382, 42516, 42516, 42516, 43287, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 38368, 22657, 22657, 22657, 22657, 38374, 43306, 46316, 46316, 46316, 46316, 50852, 38147, 38147, 38147, 38147, 38147, 32923, 22444, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 38370, 22657, 22657, 22657, 22657, 30360, 22540, 46316, 46316, 46316, 46316, 22343, 30533, 38147, 38147, 38147, 38147, 27826, 41766, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 23255, 22657, 22657, 22657, 22657, 30361, 22539, 46316, 46316, 46316, 46316, 30525, 38147, 38147, 38147, 38147, 34440, 41769, 42516, 42516, 42516, 42516, 42516, 28242, 22657, 22657, 22657, 37745, 46316, 46316, 46316, 37798, 38147, 38147, 38147, 27827, 22690, 42516, 42516, 42516, 42516, 38368, 22657, 22657, 40730, 46316, 46316, 36583, 38147, 38147, 40469, 42516, 42516, 42516, 42516, 38373, 43323, 28384, 43789, 46316, 36586, 43341, 27827, 22690, 34920, 42516, 38372, 43148, 46316, 28405, 38147, 22683, 36804, 38368, 28385, 28404, 30535, 36803, 38372, 46315, 38146, 28238, 43147, 27775, 22913, 28244, 40729, 36585, 22448, 28385, 36584, 34366, 22755, 31692, 28429, 22929, 23000, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 51361, 42516, 42516, 42516, 50278, 22100, 42516, 42516, 50243, 42516, 29530, 43375, 33789, 43361, 43411, 43418, 43434, 43447, 26099, 42516, 33915, 33924, 27632, 40956, 43463, 43486, 41243, 43507, 26512, 26972, 26982, 27147, 41812, 42783, 43531, 49966, 43567, 43604, 44058, 40078, 42516, 42516, 42516, 23213, 43642, 34502, 43666, 43684, 38517, 42516, 42516, 35323, 43702, 22657, 43728, 30048, 43746, 43782, 42975, 46316, 43805, 34708, 43833, 43850, 27569, 49274, 49874, 43871, 42516, 42516, 42516, 45270, 35189, 42516, 42516, 43894, 43668, 42516, 42814, 33726, 22657, 43915, 22657, 37724, 49041, 43936, 46316, 43969, 46316, 45680, 43987, 44022, 33996, 38147, 38147, 44043, 44074, 22444, 42516, 42516, 19081, 42516, 44111, 44127, 42516, 43290, 44154, 44166, 44182, 22657, 37101, 44229, 44279, 30652, 44299, 46316, 42336, 45461, 44326, 40412, 44345, 38147, 44372, 45987, 44388, 34123, 41766, 28121, 42516, 42516, 24911, 44407, 50085, 42516, 44435, 27732, 44483, 22975, 22657, 30361, 22539, 44504, 42690, 46316, 46316, 30525, 28768, 44537, 44554, 38147, 40691, 44575, 42516, 44603, 44624, 42516, 42516, 28242, 44645, 22657, 22657, 44664, 44689, 46316, 46316, 49378, 44708, 38147, 38147, 49798, 22690, 42516, 44729, 42516, 42516, 20056, 22657, 22657, 34376, 46316, 51047, 22477, 38147, 44538, 40469, 42516, 22273, 42516, 42516, 38373, 22657, 28384, 46316, 46316, 36586, 38147, 27827, 22690, 34920, 42516, 38372, 43148, 46316, 28405, 38147, 22683, 44749, 26230, 34694, 49922, 44785, 36803, 38372, 46315, 38146, 47702, 44821, 44842, 44862, 21913, 40729, 36585, 22448, 28385, 36584, 40621, 44890, 44940, 42111, 22929, 23000, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 51361, 42516, 42516, 42516, 46959, 51537, 45028, 47264, 44964, 45002, 45045, 45083, 45015, 45108, 45123, 45139, 45155, 45168, 26099, 45184, 28448, 42516, 28797, 42516, 42516, 30342, 42516, 42516, 45204, 45221, 31185, 28655, 31192, 26648, 48147, 38948, 50978, 45241, 34810, 28721, 45262, 45286, 25875, 34304, 35804, 42516, 42516, 42516, 42516, 45307, 45333, 42516, 45362, 45387, 27370, 22657, 22657, 45422, 45457, 45477, 46316, 48318, 45501, 22713, 45531, 38147, 40896, 22382, 41086, 42516, 42516, 45549, 45580, 42516, 42516, 42516, 42516, 42516, 41312, 38368, 32670, 37166, 22657, 22657, 33029, 22541, 45618, 47052, 46316, 46316, 45690, 35721, 38147, 45637, 38147, 38147, 45656, 22444, 42516, 42516, 45593, 44733, 42516, 45600, 35835, 45706, 27030, 38348, 45731, 22657, 22657, 40543, 26861, 45754, 45775, 46316, 46316, 43191, 44800, 45796, 45815, 38147, 38147, 40256, 45836, 45859, 28046, 42867, 49429, 42516, 25220, 42516, 19477, 43271, 20461, 22657, 22657, 27347, 45890, 30361, 22539, 46316, 48577, 44521, 46316, 30525, 38147, 38147, 46850, 45910, 34440, 41769, 42516, 50831, 45929, 19737, 42516, 28242, 22657, 45947, 43759, 50887, 46316, 45965, 46065, 28181, 38147, 45983, 46857, 27827, 22690, 42516, 42516, 46003, 44138, 38368, 22657, 22657, 40730, 46316, 46316, 36583, 38147, 38147, 29031, 51383, 46022, 26136, 47337, 46039, 22657, 46056, 46316, 34383, 36586, 38147, 46089, 46111, 34920, 46149, 46173, 43148, 46207, 44846, 46228, 32903, 36804, 38368, 46247, 46266, 45515, 36803, 38372, 46315, 38146, 35099, 35240, 22504, 22913, 46289, 46310, 46333, 36758, 46387, 47879, 34366, 22755, 31692, 28429, 22929, 23000, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 51361, 42516, 42516, 42516, 27247, 51537, 42516, 42516, 23393, 42516, 46425, 46462, 46474, 46490, 46505, 46521, 46537, 46550, 26099, 46566, 25493, 25815, 46587, 26561, 21328, 46624, 21598, 42516, 46636, 46660, 46669, 46685, 46722, 46757, 46799, 32544, 46837, 46873, 41704, 47524, 33832, 37332, 35627, 34304, 46901, 42516, 44629, 46918, 39031, 46952, 46975, 47012, 47022, 48184, 31972, 27713, 43730, 47038, 31642, 47073, 47098, 47607, 47121, 47145, 47186, 47202, 47222, 47250, 47280, 47302, 42058, 42516, 47326, 47353, 42516, 42516, 47371, 47388, 46928, 47404, 47426, 47449, 47472, 29567, 47494, 47549, 47584, 47623, 47679, 36182, 44805, 36463, 37832, 40141, 38147, 32161, 32170, 47697, 42516, 45060, 42596, 50682, 47718, 42130, 47751, 46440, 36515, 21144, 38370, 47785, 29270, 22657, 44488, 47820, 47850, 47868, 40376, 46316, 45441, 47912, 47938, 47958, 43617, 38147, 38919, 47992, 41766, 29407, 48041, 42516, 27096, 48062, 42516, 50062, 40568, 48085, 38573, 48132, 48169, 30361, 48200, 48223, 48241, 48287, 48311, 47922, 47129, 22858, 48334, 43855, 34440, 40906, 48365, 50193, 42516, 22144, 48387, 48407, 44243, 48428, 22657, 49223, 47234, 48445, 46316, 47082, 50795, 48488, 38147, 33704, 22690, 42516, 48504, 48525, 42516, 20103, 22966, 48559, 32447, 40385, 48575, 48593, 50156, 48626, 48646, 48680, 42516, 33850, 39552, 21921, 26930, 48716, 48749, 48769, 48802, 45246, 48837, 48960, 48859, 48887, 32658, 28363, 48910, 33376, 48933, 48953, 36804, 38368, 28385, 28404, 30535, 36803, 38372, 46315, 38146, 28238, 43147, 45621, 46231, 48976, 39991, 49005, 49029, 40039, 49057, 44903, 22755, 31692, 28429, 49073, 23000, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 51361, 42516, 42516, 42516, 42516, 49109, 42516, 42516, 49144, 42516, 32608, 18328, 18337, 49164, 18337, 18344, 49180, 49193, 26099, 42516, 42516, 42516, 27632, 20863, 28835, 42516, 42516, 42516, 42516, 28834, 42516, 20857, 42516, 28241, 49209, 31530, 22841, 49248, 36678, 30143, 33493, 42516, 42516, 34304, 42516, 42516, 35043, 49290, 42516, 42516, 42516, 38053, 28243, 49327, 22657, 22657, 22658, 49350, 46316, 46316, 46316, 31591, 33687, 38147, 38147, 38147, 49366, 49414, 49451, 42516, 49474, 49519, 42516, 42516, 43686, 45346, 42516, 46571, 46157, 38368, 45949, 40328, 45406, 25137, 29005, 22541, 48786, 34087, 38601, 49540, 48459, 38147, 33697, 39345, 49573, 33236, 49608, 22444, 42516, 19180, 42516, 50588, 42516, 42516, 42516, 18791, 42516, 42516, 38370, 38694, 22657, 22657, 22657, 30360, 49641, 32217, 46316, 46316, 46316, 22343, 49659, 38898, 38147, 38147, 38147, 27826, 41766, 46990, 42516, 49679, 42516, 42516, 42516, 42516, 20094, 31303, 22657, 22657, 22657, 30361, 49698, 46316, 46316, 46316, 46316, 30525, 49732, 38147, 38147, 38147, 34440, 35908, 42516, 42516, 46023, 49756, 42516, 29706, 37130, 22657, 22657, 39744, 38128, 46316, 46316, 49778, 49814, 38147, 38147, 49848, 22690, 22055, 42516, 42516, 42516, 38368, 22657, 22657, 40730, 46316, 46316, 36583, 38147, 38147, 40469, 42516, 42516, 42516, 42516, 38373, 22657, 28384, 46316, 46316, 36586, 38147, 27827, 22690, 34920, 42516, 34322, 43148, 47105, 28405, 38446, 22683, 36804, 38368, 28385, 28404, 30535, 36803, 38372, 46315, 38146, 28238, 43147, 27775, 22913, 49890, 49913, 49938, 22448, 28385, 36584, 34366, 34779, 33418, 49954, 22929, 23000, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 51361, 42516, 42516, 42516, 42516, 50000, 50082, 50038, 50059, 42516, 32349, 42516, 20487, 50078, 34283, 44986, 50101, 50114, 26099, 42516, 42516, 42516, 27632, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 28241, 50130, 42394, 35273, 50148, 34810, 28721, 42516, 36285, 32580, 34304, 42516, 42516, 42516, 42516, 42516, 25759, 42516, 42516, 28243, 27704, 22657, 37534, 22657, 27401, 50172, 46316, 38188, 48318, 29351, 38147, 30813, 38147, 40896, 22382, 42516, 42516, 42516, 42516, 42516, 41597, 42516, 42516, 42516, 42476, 42516, 38368, 22657, 50132, 22657, 22657, 38374, 22541, 46316, 49557, 46316, 46316, 44805, 38147, 38147, 50449, 38147, 38147, 32923, 22444, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 38370, 22657, 22657, 22657, 22657, 30360, 22540, 46316, 46316, 46316, 46316, 22343, 30533, 38147, 38147, 38147, 38147, 27826, 41766, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 23255, 22657, 22657, 22657, 22657, 30361, 22539, 46316, 46316, 46316, 46316, 30525, 38147, 38147, 38147, 38147, 34440, 41769, 42516, 42516, 50190, 42516, 42516, 28242, 22657, 22657, 22657, 37745, 46316, 46316, 46316, 37798, 38147, 38147, 38147, 27827, 22690, 42516, 50209, 42516, 42516, 38368, 22657, 22657, 40730, 46316, 46316, 36583, 38147, 38147, 40469, 42516, 42516, 42516, 42516, 38373, 22657, 28384, 46316, 46316, 36586, 38147, 27827, 22690, 50231, 42516, 38372, 43148, 46316, 28405, 38147, 22683, 36804, 38368, 28385, 28404, 30535, 36803, 38372, 46315, 38146, 28238, 43147, 27775, 22913, 28244, 40729, 36585, 22448, 28385, 36584, 34366, 22755, 31692, 28429, 22929, 23000, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 51361, 42516, 42516, 42516, 42516, 50266, 42516, 22306, 42516, 45067, 50303, 50340, 42516, 50365, 50380, 50394, 50410, 50423, 26099, 42516, 42516, 42516, 27632, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 28241, 45894, 46314, 50439, 28325, 41965, 36365, 42516, 42516, 42516, 34304, 42516, 42516, 42516, 42516, 25694, 42516, 42516, 42516, 28243, 22657, 22657, 22657, 22657, 27401, 46316, 46316, 46316, 48318, 38147, 38147, 38147, 38147, 40896, 22382, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 38368, 22657, 22657, 22657, 22657, 38374, 22541, 46316, 46316, 46316, 46316, 44805, 38147, 38147, 38147, 38147, 38147, 32923, 22444, 42516, 42516, 42516, 42516, 42516, 42516, 50473, 42516, 42516, 42516, 50491, 22657, 22657, 22657, 22657, 30360, 50513, 46316, 46316, 46316, 46316, 22343, 50533, 38147, 38147, 38147, 38147, 27826, 41766, 42516, 42516, 29399, 42516, 42516, 42516, 42516, 23255, 22657, 22657, 50557, 22657, 50575, 22539, 46316, 43245, 46316, 33180, 30525, 38147, 47942, 38147, 38147, 50611, 41769, 42516, 42516, 42516, 42516, 42516, 28242, 22657, 22657, 22657, 37745, 46316, 46316, 46316, 37798, 38147, 38147, 38147, 27827, 22690, 42516, 42516, 42516, 42516, 38368, 22657, 22657, 40730, 46316, 46316, 36583, 38147, 38147, 40469, 42516, 42516, 42516, 42516, 38373, 22657, 28384, 46316, 46316, 36586, 38147, 27827, 22690, 34920, 42516, 38372, 43148, 46316, 28405, 38147, 22683, 36804, 38368, 28385, 28404, 30535, 36803, 38372, 46315, 38146, 28238, 43147, 27775, 22913, 28244, 40729, 36585, 22448, 28385, 36584, 34366, 22755, 31692, 28429, 22929, 23000, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 26324, 42516, 42516, 42516, 42516, 51537, 42516, 42516, 42516, 42516, 25917, 42516, 42516, 42516, 42516, 48543, 50631, 50644, 26099, 42516, 42516, 42516, 27632, 50660, 42516, 50680, 42516, 42516, 42516, 29122, 42516, 21452, 29118, 32973, 50698, 44948, 35273, 50716, 34810, 28721, 42516, 42516, 42516, 50750, 27264, 27883, 43515, 42516, 42516, 42516, 42516, 42516, 28243, 22657, 43166, 22657, 49088, 27401, 46316, 30696, 30175, 48318, 38147, 38147, 50786, 27533, 40896, 50811, 50827, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 38368, 22657, 22657, 22657, 22657, 38374, 22541, 46316, 46316, 46316, 46316, 44805, 38147, 38147, 38147, 38147, 38147, 32923, 22444, 42516, 42516, 42516, 42516, 42516, 42516, 51530, 42516, 42516, 42516, 38370, 22657, 27966, 22657, 22657, 27206, 22540, 46316, 36563, 46316, 46316, 48225, 30533, 38147, 43345, 38147, 38147, 27826, 41766, 21110, 42516, 42516, 42516, 42516, 42516, 42516, 23255, 36082, 22657, 22657, 22657, 30361, 22539, 50847, 46316, 46316, 46316, 30525, 40490, 38147, 38147, 38147, 34440, 41769, 42516, 42516, 42516, 42516, 42516, 28242, 22657, 22657, 22657, 37745, 46316, 46316, 46316, 37798, 38147, 38147, 38147, 27827, 22690, 42516, 42516, 42516, 42516, 38368, 22657, 22657, 40730, 46316, 46316, 36583, 38147, 38147, 40469, 42516, 42516, 42516, 42516, 38373, 22657, 28384, 46316, 46316, 36586, 38147, 27827, 22690, 34920, 42516, 38372, 43148, 46316, 28405, 38147, 22683, 36804, 38368, 28385, 28404, 30535, 36803, 38372, 46315, 38146, 28238, 43147, 27775, 22913, 28244, 40729, 36585, 22448, 28385, 36584, 34366, 22755, 31692, 28429, 22929, 23000, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 51361, 42516, 42516, 42516, 42516, 51537, 42516, 42516, 42516, 42516, 25917, 42516, 42516, 42516, 42516, 46741, 49861, 51029, 26099, 24903, 42516, 42516, 27632, 42516, 42516, 20017, 42516, 42516, 42516, 42516, 20023, 42516, 42516, 50868, 36101, 46314, 27462, 29927, 45874, 41719, 42516, 29087, 42516, 34304, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42872, 28243, 22657, 22657, 22657, 47797, 27401, 46316, 46316, 22542, 48318, 38147, 38147, 38147, 27479, 40896, 22382, 42516, 50903, 42516, 42516, 42516, 42516, 42516, 42516, 42150, 42516, 42516, 38368, 46183, 22657, 41900, 22657, 38374, 22541, 50922, 46316, 37779, 46316, 44805, 36603, 38147, 38147, 40222, 38147, 32923, 22444, 42516, 50475, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 38370, 22657, 50942, 22657, 22657, 30360, 50961, 30499, 50994, 46316, 46316, 22343, 49789, 39662, 51012, 38147, 38147, 27826, 41766, 42516, 42516, 42516, 42516, 42516, 42516, 48664, 23255, 22657, 22657, 32677, 22657, 28304, 22539, 46316, 46316, 51045, 46316, 51063, 38147, 38147, 50725, 38147, 44006, 41769, 42516, 42516, 50906, 42516, 42516, 28242, 22657, 22657, 22657, 37745, 46316, 46316, 46316, 37798, 38147, 38147, 38147, 27827, 48656, 42516, 42516, 47372, 42516, 31381, 22657, 22657, 34817, 46316, 46316, 40751, 38147, 38147, 40469, 42516, 42516, 51087, 42516, 38373, 28681, 28384, 46316, 51106, 36586, 34742, 27827, 22690, 34920, 42516, 38372, 43148, 46316, 28405, 38147, 22683, 36804, 38368, 28385, 28404, 30535, 36803, 38372, 46315, 38146, 28238, 43147, 27775, 22913, 28244, 40729, 36585, 22448, 28385, 36584, 34366, 22755, 31692, 28429, 22929, 23000, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 51361, 42516, 42516, 42516, 42516, 51537, 42516, 42516, 42516, 42516, 25917, 19962, 42516, 42516, 42516, 46741, 49861, 51029, 26099, 42516, 42516, 42516, 27632, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 28241, 22657, 46314, 35273, 38147, 34810, 28721, 42516, 42516, 42516, 34304, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 28243, 22657, 22657, 22657, 22657, 27401, 46316, 46316, 46316, 48318, 38147, 38147, 38147, 38147, 40896, 22382, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 38368, 22657, 22657, 22657, 22657, 38374, 22541, 46316, 46316, 46316, 46316, 44805, 38147, 38147, 38147, 38147, 38147, 32923, 22444, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 38370, 22657, 22657, 22657, 22657, 30360, 22540, 46316, 46316, 46316, 46316, 22343, 30533, 38147, 38147, 38147, 38147, 27826, 41766, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 23255, 22657, 22657, 22657, 22657, 30361, 22539, 46316, 46316, 46316, 46316, 30525, 38147, 38147, 38147, 38147, 34440, 41769, 42516, 42516, 42516, 42516, 42516, 28242, 22657, 22657, 22657, 37745, 46316, 46316, 46316, 37798, 38147, 38147, 38147, 27827, 22690, 42516, 42516, 42516, 42516, 38368, 22657, 22657, 40730, 46316, 46316, 36583, 38147, 38147, 40469, 42516, 42516, 42516, 42516, 38373, 22657, 28384, 46316, 46316, 36586, 38147, 27827, 22690, 34920, 42516, 38372, 43148, 46316, 28405, 38147, 22683, 36804, 38368, 28385, 28404, 30535, 36803, 38372, 46315, 38146, 28238, 43147, 27775, 22913, 28244, 40729, 36585, 22448, 28385, 36584, 34366, 22755, 31692, 28429, 22929, 23000, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 21141, 42516, 42516, 42516, 42516, 46123, 30288, 30286, 49299, 30156, 51124, 22408, 22410, 49308, 30152, 46133, 51161, 51174, 21040, 42516, 42516, 42516, 27632, 42516, 42516, 31797, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 32629, 20505, 20792, 18990, 21068, 19019, 18919, 18270, 40105, 18293, 18315, 42516, 42516, 18360, 42516, 49435, 42820, 18380, 18411, 18395, 18574, 19226, 18433, 20169, 51190, 25598, 18486, 18508, 19308, 18465, 25604, 18492, 18514, 20728, 18530, 42516, 30418, 42516, 42516, 42516, 50250, 25964, 50770, 42516, 18553, 39446, 18564, 20175, 20411, 20131, 20505, 23879, 18590, 26008, 18612, 18632, 18596, 18470, 18594, 26012, 18616, 18636, 20798, 20741, 20757, 42516, 18652, 18689, 18711, 18748, 18695, 18788, 42516, 41546, 48871, 18807, 18835, 19802, 18823, 19840, 25206, 18851, 18904, 18942, 18979, 19680, 19006, 18851, 18904, 18942, 18979, 19680, 19045, 20750, 46697, 19097, 19132, 20936, 31767, 19148, 19174, 19196, 20141, 20921, 19788, 19212, 19242, 19278, 25525, 20579, 19301, 19596, 19271, 25518, 20572, 19294, 19589, 19324, 19353, 35639, 19379, 39467, 19401, 41728, 19421, 19500, 24458, 19544, 19575, 19614, 19632, 19650, 19559, 19612, 19630, 19648, 19666, 19029, 19696, 19712, 19728, 19753, 19773, 19830, 19435, 19449, 19888, 19905, 19856, 19887, 19904, 19921, 19947, 19984, 20043, 20080, 20119, 20157, 20536, 20191, 20213, 20542, 20197, 20219, 19931, 20235, 20264, 20301, 20317, 20346, 20333, 20349, 20365, 20381, 20397, 20422, 20606, 20438, 20477, 20503, 21067, 21068, 20521, 20558, 20595, 20622, 20652, 23888, 18878, 20683, 20699, 20715, 20779, 20814, 20880, 20890, 20906, 19463, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 21141, 42516, 42516, 42516, 42516, 48069, 42516, 42516, 30409, 42516, 25917, 39868, 39877, 51215, 51230, 51237, 29448, 51253, 21040, 42516, 42516, 42516, 27632, 42516, 42516, 51474, 42516, 42516, 42516, 23517, 42516, 42516, 23514, 32629, 20505, 21066, 18953, 21068, 18866, 18919, 18270, 40105, 18293, 18315, 42516, 42516, 18360, 42516, 49435, 42820, 18380, 18411, 18395, 18574, 19226, 18433, 20169, 18459, 25598, 18486, 18508, 51199, 18465, 25604, 18492, 18514, 20728, 18530, 42516, 30418, 42516, 42516, 42516, 50250, 25964, 50770, 42516, 18553, 39446, 18564, 20175, 20411, 20131, 20505, 23879, 18590, 26008, 18612, 18632, 18596, 18470, 18594, 26012, 18616, 18636, 20798, 20741, 20757, 42516, 18652, 18689, 18711, 18748, 18695, 18788, 42516, 41546, 48871, 18807, 18835, 19802, 18823, 19840, 25206, 18851, 18904, 18942, 18979, 19680, 19006, 18851, 18904, 18942, 18979, 19680, 19045, 20750, 46697, 19097, 19132, 20936, 31767, 19148, 19174, 19196, 20141, 20921, 19788, 19212, 19242, 19278, 25525, 20579, 19301, 19596, 19271, 25518, 20572, 19294, 19589, 19324, 19353, 35639, 19379, 39467, 19401, 41728, 19421, 19500, 24458, 19544, 19575, 19614, 19632, 19650, 19559, 19612, 19630, 19648, 19666, 19029, 19696, 19712, 19728, 19753, 19773, 19830, 19435, 19449, 19888, 19905, 19856, 19887, 19904, 19921, 19947, 19984, 20043, 20080, 20119, 20157, 20536, 20191, 20213, 20542, 20197, 20219, 19931, 20235, 20264, 20301, 20317, 20346, 20333, 20349, 20365, 20381, 20397, 20422, 20606, 20438, 20477, 20503, 21067, 21068, 20521, 20558, 20595, 20622, 20652, 23888, 18878, 20683, 20699, 20715, 20779, 20814, 20880, 20890, 20906, 19463, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 21141, 42516, 42516, 42516, 42516, 36374, 42516, 42516, 42516, 42516, 31102, 50349, 51296, 51323, 51307, 51330, 32370, 51346, 21040, 42516, 42516, 42516, 27632, 42516, 42516, 51474, 51382, 42516, 26083, 42516, 42516, 42516, 26079, 32629, 20505, 21066, 18953, 21068, 19019, 18919, 18270, 40105, 18293, 18315, 42516, 42516, 18360, 42516, 49435, 42820, 18380, 18411, 18395, 18574, 19226, 18433, 20169, 51399, 25598, 18486, 18508, 21522, 18465, 25604, 18492, 18514, 20728, 18530, 42516, 30418, 42516, 42516, 42516, 50250, 25964, 50770, 42516, 18553, 39446, 18564, 20175, 20411, 20131, 20505, 23879, 18590, 26008, 18612, 18632, 18596, 18470, 18594, 26012, 18616, 18636, 20798, 20741, 20757, 42516, 18652, 18689, 18711, 18748, 18695, 18788, 42516, 41546, 48871, 18807, 18835, 19802, 18823, 19840, 25206, 18851, 18904, 18942, 18979, 19680, 19006, 18851, 18904, 18942, 18979, 19680, 19045, 20750, 46697, 19097, 19132, 20936, 31767, 19148, 19174, 19196, 20141, 20921, 19788, 19212, 19242, 19278, 25525, 20579, 19301, 19596, 19271, 25518, 20572, 19294, 19589, 19324, 19353, 35639, 19379, 39467, 19401, 41728, 19421, 19500, 24458, 19544, 19575, 19614, 19632, 19650, 19559, 19612, 19630, 19648, 19666, 19029, 19696, 19712, 19728, 19753, 19773, 19830, 19435, 19449, 19888, 19905, 19856, 19887, 19904, 19921, 19947, 19984, 20043, 20080, 20119, 20157, 20536, 20191, 20213, 20542, 20197, 20219, 19931, 20235, 20264, 20301, 20317, 20346, 20333, 20349, 20365, 20381, 20397, 20422, 20606, 20438, 20477, 20503, 21067, 21068, 20521, 20558, 20595, 20622, 20652, 23888, 18878, 20683, 20699, 20715, 20779, 20814, 20880, 20890, 20906, 19463, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 21141, 42516, 42516, 42516, 42516, 48069, 42516, 42516, 42516, 42516, 25917, 42516, 42516, 42516, 42516, 42516, 42516, 42517, 26099, 42516, 42516, 42516, 27632, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 28241, 22657, 46314, 35273, 38147, 34810, 46363, 42516, 42516, 42516, 34304, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 28243, 22657, 22657, 22657, 22657, 22341, 46316, 46316, 46316, 39789, 38147, 38147, 38147, 38147, 40896, 22382, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 38368, 22657, 22657, 22657, 22657, 38374, 22541, 46316, 46316, 46316, 46316, 44805, 38147, 38147, 38147, 38147, 38147, 32923, 22444, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 38370, 22657, 22657, 22657, 22657, 30360, 22540, 46316, 46316, 46316, 46316, 22343, 30533, 38147, 38147, 38147, 38147, 27826, 41766, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 23255, 22657, 22657, 22657, 22657, 30361, 22539, 46316, 46316, 46316, 46316, 30525, 38147, 38147, 38147, 38147, 34440, 41769, 42516, 42516, 42516, 42516, 42516, 28242, 22657, 22657, 22657, 37745, 46316, 46316, 46316, 37798, 38147, 38147, 38147, 27827, 22690, 42516, 42516, 42516, 42516, 38368, 22657, 22657, 40730, 46316, 46316, 36583, 38147, 38147, 40469, 42516, 42516, 42516, 42516, 38373, 22657, 28384, 46316, 46316, 36586, 38147, 27827, 22690, 34920, 42516, 38372, 43148, 46316, 28405, 38147, 22683, 36804, 38368, 28385, 28404, 30535, 36803, 38372, 46315, 38146, 28238, 43147, 27775, 22913, 28244, 40729, 36585, 22448, 28385, 36584, 34366, 22755, 31692, 28429, 22929, 23000, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 23494, 42516, 42516, 45092, 51424, 51429, 51429, 51451, 23492, 51445, 51467, 42516, 42516, 42516, 42516, 42516, 42516, 51474, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 32629, 20505, 21066, 18953, 21068, 19019, 18919, 18270, 40105, 18293, 51497, 42516, 42516, 18360, 42516, 49435, 42820, 18380, 18411, 18395, 18574, 19226, 18433, 20169, 51190, 25598, 18486, 18508, 19308, 18465, 25604, 18492, 18514, 20728, 18530, 42516, 30418, 42516, 42516, 42516, 50250, 25964, 50770, 42516, 18553, 39446, 18564, 20175, 20411, 20131, 20505, 23879, 18590, 26008, 18612, 18632, 18596, 18470, 18594, 26012, 18616, 18636, 20798, 20741, 20757, 42516, 18652, 18689, 18711, 18748, 18695, 18788, 42516, 41546, 48871, 18807, 18835, 19802, 18823, 19840, 25206, 18851, 18904, 18942, 18979, 19680, 19006, 18851, 18904, 18942, 18979, 19680, 19045, 20750, 46697, 19097, 19132, 20936, 31767, 19148, 19174, 19196, 20141, 20921, 19788, 19212, 19242, 19278, 25525, 20579, 19301, 19596, 19271, 25518, 20572, 19294, 19589, 19324, 19353, 35639, 19379, 39467, 19401, 41728, 19421, 19500, 24458, 19544, 19575, 19614, 19632, 19650, 19559, 19612, 19630, 19648, 19666, 19029, 19696, 19712, 19728, 19753, 19773, 19830, 19435, 19449, 19888, 19905, 19856, 19887, 19904, 19921, 19947, 19984, 20043, 20080, 20119, 20157, 20536, 20191, 20213, 20542, 20197, 20219, 19931, 51517, 20264, 20301, 20317, 20346, 20333, 20349, 20365, 51553, 20397, 20422, 20606, 20438, 20453, 20503, 21067, 21068, 20521, 20558, 20595, 20622, 20652, 23888, 18878, 20683, 20699, 20715, 20779, 20814, 20880, 20890, 20906, 19463, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 42516, 94506, 94506, 94506, 94506, 94506, 94506, 94506, 94506, 94506, 94506, 0, 94506, 90409, 94506, 94506, 94506, 94506, 94506, 94506, 94506, 94506, 94506, 365, 94506, 90409, 94506, 94506, 94506, 94506, 94506, 94506, 94506, 69632, 73728, 94506, 94506, 94506, 94506, 94506, 65536, 94506, 12290, 3, 0, 0, 2183168, 0, 0, 0, 90409, 94506, 299, 300, 0, 2134016, 303, 304, 304, 304, 304, 304, 304, 304, 0, 0, 0, 0, 0, 304, 0, 304, 1, 289, 3, 0, 0, 0, 295, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 796, 0, 796, 0, 0, 0, 0, 0, 2424832, 2433024, 0, 0, 2457600, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 650, 0, 0, 0, 0, 0, 0, 0, 0, 0, 2904064, 2908160, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1685, 1686, 0, 1688, 0, 0, 0, 0, 3117056, 0, 0, 0, 0, 0, 0, 0, 365, 365, 0, 0, 0, 0, 0, 0, 448, 0, 0, 474, 474, 474, 474, 474, 474, 474, 474, 474, 474, 474, 474, 474, 474, 474, 474, 559, 582, 559, 582, 559, 559, 582, 0, 0, 0, 2138112, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 2991, 0, 0, 0, 2772992, 2805760, 2830336, 0, 2863104, 2920448, 0, 0, 0, 0, 0, 0, 0, 2805760, 2920448, 0, 0, 0, 0, 0, 0, 0, 2732032, 0, 2179072, 2179072, 2179072, 2179072, 2424832, 2433024, 0, 0, 0, 0, 0, 2920448, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1702, 0, 1704, 0, 0, 2179072, 2830336, 2179072, 2179072, 2863104, 2179072, 2179072, 2179072, 2179072, 2920448, 2179072, 2179072, 2179072, 2179072, 2179072, 2179072, 2126737, 2126737, 2126737, 2126737, 2126737, 2593681, 2126737, 2126737, 2126737, 2126737, 0, 914, 2125824, 2125824, 2125824, 2125824, 2125824, 2424832, 2433024, 2125824, 2125824, 2125824, 2125824, 2125824, 2125824, 2125824, 2125824, 2125824, 2125824, 2125824, 2125824, 2125824, 0, 0, 2125824, 2125824, 2125824, 2125824, 2723840, 2125824, 2732032, 2772992, 2125824, 2125824, 2125824, 2805760, 2125824, 2830336, 2125824, 2125824, 2863104, 2125824, 2125824, 2125824, 2125824, 2920448, 2125824, 2125824, 2125824, 2125824, 2125824, 2920448, 2125824, 2125824, 2125824, 2125824, 2125824, 2125824, 2125824, 2125824, 2125824, 2125824, 2125824, 3117056, 2125824, 2125824, 2125824, 2125824, 2125824, 2125824, 2125824, 2125824, 2457600, 2125824, 2125824, 2125824, 2125824, 2183168, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 661, 0, 661, 0, 0, 0, 2408448, 0, 0, 2584576, 0, 0, 0, 0, 2838528, 0, 0, 2838528, 0, 0, 0, 0, 0, 2179072, 2179072, 2179072, 2408448, 2179072, 2179072, 2179072, 2179072, 2179072, 2179072, 2179072, 2179072, 2179072, 2179072, 2179072, 2179072, 2179072, 2625536, 2179072, 2179072, 0, 2125824, 2125824, 2125824, 2408448, 2125824, 2125824, 2125824, 2125824, 2125824, 2125824, 2125824, 2125824, 2125824, 2125824, 2125824, 2125824, 2125824, 2125824, 2125824, 3125248, 2125824, 2125824, 2125824, 2125824, 2125824, 2125824, 2125824, 2662400, 2125824, 2125824, 2125824, 2125824, 2125824, 2125824, 2125824, 2125824, 2125824, 2125824, 2125824, 2801664, 2813952, 2125824, 2125824, 2801664, 2813952, 2125824, 2838528, 2125824, 2125824, 2125824, 2125824, 2125824, 2125824, 2125824, 2125824, 2125824, 2125824, 2125824, 2125824, 2125824, 2125824, 2461696, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 2600960, 0, 0, 0, 0, 0, 0, 2441, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 2493, 2494, 0, 0, 2497, 0, 2768896, 2777088, 2781184, 0, 2822144, 0, 0, 2883584, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 3055616, 0, 0, 0, 3080192, 3100672, 3104768, 0, 0, 0, 0, 3186688, 0, 0, 0, 0, 0, 0, 0, 307, 204800, 0, 0, 0, 0, 0, 0, 0, 0, 0, 111051, 111051, 111051, 111051, 111051, 111051, 111051, 111051, 1, 0, 0, 0, 0, 0, 2797568, 0, 0, 0, 0, 0, 0, 0, 2850816, 2867200, 0, 0, 0, 0, 0, 441, 0, 0, 332, 468, 468, 468, 468, 468, 468, 468, 468, 468, 468, 468, 468, 468, 468, 468, 468, 0, 0, 3133440, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 2131, 0, 0, 0, 0, 0, 2179072, 2179072, 2179072, 2179072, 2179072, 2179072, 2179072, 2179072, 2179072, 2179072, 2461696, 2465792, 2179072, 2768896, 2777088, 2781184, 2797568, 2822144, 2179072, 2179072, 2179072, 2883584, 2179072, 2912256, 2179072, 2179072, 2179072, 2179072, 2179072, 2179072, 2523136, 2179072, 2179072, 2179072, 2179072, 2179072, 2179072, 2179072, 2179072, 2600960, 0, 0, 2125824, 2125824, 2125824, 2125824, 2125824, 2125824, 2125824, 2125824, 2125824, 2125824, 2125824, 2461696, 2465792, 2125824, 0, 1142784, 0, 2179072, 2125824, 2125824, 2179072, 2179072, 2179072, 2179072, 2179072, 2125824, 2125824, 2125824, 2125824, 2125824, 2125824, 2125824, 2125824, 2125824, 2592768, 2125824, 2125824, 2125824, 2125824, 2125824, 2125824, 2125824, 2125824, 2125824, 0, 24576, 988, 2125824, 2125824, 2125824, 2125824, 2125824, 2125824, 2125824, 2125824, 2125824, 2523136, 2125824, 2125824, 2125824, 2125824, 2125824, 2125824, 2125824, 2125824, 2600960, 2125824, 0, 2125824, 2125824, 2125824, 2125824, 2125824, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 541, 1272, 541, 541, 541, 541, 2125824, 2125824, 2125824, 2641920, 2125824, 2125824, 2125824, 2125824, 2125824, 2125824, 2719744, 2125824, 2125824, 2125824, 2125824, 2125824, 2125824, 2125824, 2125824, 2125824, 0, 2125824, 2125824, 2125824, 2125824, 2125824, 2125824, 299, 0, 0, 0, 299, 0, 300, 0, 0, 0, 2768896, 2777088, 2781184, 2797568, 2822144, 2125824, 2125824, 2125824, 2883584, 2125824, 2912256, 2125824, 2125824, 2125824, 2125824, 2125824, 2125824, 2125824, 2125824, 2125824, 0, 2125824, 2126812, 2125824, 2125824, 2125824, 2125824, 2125824, 2125824, 3133440, 2125824, 2125824, 2125824, 2125824, 2125824, 2125824, 2125824, 2125824, 2125824, 3207168, 2125824, 0, 0, 0, 2179072, 2125824, 2125824, 2179072, 2179072, 2179072, 2179072, 2179072, 2125824, 2125824, 2125824, 2125824, 0, 0, 0, 0, 0, 0, 2510848, 2514944, 0, 0, 2125824, 2125824, 3133440, 2125824, 2125824, 2125824, 2125824, 2125824, 2125824, 2125824, 2125824, 2125824, 3207168, 2125824, 2179072, 2125824, 0, 2125824, 2125824, 2125824, 2125824, 2125824, 0, 0, 0, 0, 0, 0, 300, 0, 0, 0, 0, 0, 0, 2764, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 2059, 2060, 0, 2062, 2063, 0, 0, 0, 0, 2605056, 0, 0, 0, 0, 2887680, 0, 2924544, 0, 0, 0, 0, 0, 0, 0, 2108, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1177, 0, 0, 0, 0, 0, 0, 3162112, 3170304, 0, 0, 3219456, 3035136, 0, 0, 0, 0, 0, 3072000, 3190784, 0, 0, 0, 0, 0, 0, 0, 0, 2576384, 0, 0, 0, 0, 0, 0, 0, 334, 0, 0, 334, 0, 0, 334, 0, 0, 0, 0, 0, 0, 0, 2387968, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 2050, 0, 0, 0, 0, 3121152, 3141632, 0, 0, 0, 2924544, 0, 2682880, 0, 0, 0, 0, 0, 0, 3112960, 2387968, 2924544, 2179072, 2179072, 2179072, 2179072, 2179072, 2179072, 2179072, 2179072, 2179072, 3035136, 2179072, 2179072, 3072000, 2179072, 2179072, 2179072, 2179072, 2699264, 2179072, 2715648, 2179072, 2723840, 2179072, 2732032, 2772992, 2179072, 2179072, 2179072, 2805760, 3121152, 2179072, 2179072, 3141632, 2179072, 2179072, 2179072, 3170304, 2179072, 2179072, 3190784, 3194880, 2179072, 0, 0, 0, 0, 0, 0, 541, 1734, 541, 541, 541, 541, 541, 541, 1740, 541, 2125824, 3190784, 3194880, 2125824, 0, 0, 0, 0, 0, 0, 2387968, 2125824, 2125824, 2125824, 2420736, 2125824, 2125824, 2125824, 2125824, 2125824, 2453504, 2125824, 2473984, 2125824, 2736128, 2125824, 2125824, 2125824, 2125824, 2125824, 2125824, 2125824, 2125824, 2887680, 2125824, 2125824, 2924544, 2125824, 2125824, 2125824, 2125824, 2125824, 2125824, 2125824, 2125824, 2125824, 0, 0, 0, 2125824, 2125824, 2125824, 2125824, 2125824, 3141632, 2125824, 2125824, 2125824, 3170304, 2125824, 2125824, 3190784, 3194880, 2125824, 2179072, 2125824, 2125824, 2179072, 2179072, 2179072, 2179072, 2125824, 2125824, 2125824, 2125824, 2125824, 2125824, 2125824, 2125824, 299, 0, 300, 3112960, 3219456, 2125824, 2125824, 3112960, 3219456, 2125824, 2125824, 3112960, 3219456, 0, 0, 0, 0, 0, 0, 0, 347, 0, 405, 0, 0, 0, 0, 0, 405, 3022848, 0, 0, 3145728, 0, 3203072, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 2072, 0, 0, 0, 0, 0, 0, 0, 3067904, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 3003, 0, 0, 0, 0, 0, 2621440, 0, 3182592, 2899968, 0, 2961408, 0, 0, 2179072, 2179072, 2416640, 2179072, 2179072, 2179072, 2179072, 2928640, 2179072, 2179072, 2179072, 2998272, 2179072, 2179072, 2179072, 2179072, 3059712, 2179072, 2179072, 2179072, 2179072, 3178496, 2125824, 2125824, 2125824, 2125824, 2125824, 2125824, 2125824, 2125824, 2494464, 2125824, 2125824, 0, 2179072, 2125824, 2125824, 0, 2179072, 2125824, 2125824, 2985984, 2985984, 2985984, 2985984, 0, 0, 0, 0, 0, 0, 2490, 0, 0, 0, 0, 0, 0, 0, 0, 0, 335, 0, 0, 0, 0, 0, 0, 2179072, 2445312, 2179072, 2179072, 2179072, 2179072, 2179072, 2179072, 2179072, 2179072, 2179072, 2179072, 2179072, 2551808, 2179072, 2179072, 2179072, 2179072, 3178496, 2126737, 2126737, 2126737, 2126737, 2126737, 2126737, 2126737, 2126737, 2495377, 2126737, 2126737, 2126737, 2126737, 2126737, 3179409, 2126811, 2126811, 2126811, 2126811, 2126811, 2126811, 2126811, 2126811, 2495451, 2126811, 2895872, 2916352, 2179072, 2179072, 2945024, 2179072, 2179072, 2994176, 2179072, 3002368, 2179072, 2179072, 3022848, 2179072, 3067904, 3084288, 2125824, 3096576, 2125824, 2125824, 2125824, 2125824, 2125824, 2125824, 2125824, 3223552, 0, 0, 2125824, 2125824, 2416640, 3096576, 2179072, 2179072, 2179072, 2179072, 2179072, 2179072, 2179072, 3223552, 0, 0, 2125824, 2125824, 2416640, 2125824, 2125824, 2125824, 2125824, 2125824, 2125824, 2125824, 3035136, 2125824, 2125824, 3072000, 2125824, 2125824, 2125824, 3121152, 2125824, 2125824, 3141632, 2125824, 2125824, 2125824, 3170304, 2125824, 2125824, 2125824, 2125824, 2445312, 2125824, 2125824, 2125824, 2125824, 2125824, 2125824, 2125824, 2125824, 2125824, 2125824, 2125824, 2551808, 2125824, 2125824, 2125824, 2125824, 2125824, 2125824, 2125824, 2637824, 2125824, 2125824, 2125824, 2125824, 2727936, 2752512, 2125824, 2125824, 2125824, 2125824, 2842624, 2846720, 2842624, 2846720, 2125824, 2895872, 2916352, 2125824, 2125824, 2945024, 2125824, 2125824, 2994176, 2125824, 3002368, 2125824, 2125824, 3022848, 2125824, 3067904, 2125824, 3067904, 3084288, 2125824, 3096576, 2125824, 2125824, 2125824, 2125824, 2125824, 2125824, 2125824, 3223552, 2179072, 2125824, 2125824, 2125824, 2125824, 2125824, 2125824, 2125824, 3039232, 2125824, 3063808, 2125824, 2125824, 2125824, 2125824, 2125824, 3100672, 2547712, 2596864, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 3178496, 2670592, 0, 2744320, 0, 0, 0, 0, 0, 2928640, 0, 0, 0, 3059712, 0, 2543616, 2666496, 0, 2633728, 0, 0, 0, 0, 0, 0, 2494464, 0, 0, 0, 0, 0, 0, 0, 0, 0, 2780, 0, 0, 0, 0, 2785, 0, 0, 0, 0, 2957312, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 3188, 0, 0, 0, 0, 3211264, 0, 0, 0, 2179072, 2179072, 2179072, 2179072, 2179072, 2179072, 2179072, 2179072, 2494464, 2179072, 2179072, 2179072, 2707456, 2179072, 2736128, 2179072, 2179072, 2179072, 2179072, 2179072, 2179072, 2179072, 2887680, 2179072, 2179072, 2179072, 2179072, 2641920, 2179072, 2179072, 2179072, 2179072, 2179072, 2179072, 2719744, 2179072, 2179072, 2179072, 2179072, 2179072, 2179072, 2179072, 3137536, 2126737, 2126737, 2499473, 2126737, 2126737, 2126737, 2556817, 2565009, 2179072, 2514944, 2179072, 2179072, 2179072, 2543616, 2547712, 2179072, 2179072, 2596864, 2179072, 2179072, 2179072, 2179072, 2179072, 2179072, 2179072, 2179072, 3039232, 2179072, 3063808, 2179072, 2179072, 2179072, 2179072, 3100672, 2125824, 2125824, 2125824, 2125824, 2125824, 3178496, 2125824, 2125824, 2125824, 2125824, 2125824, 2125824, 2125824, 2125824, 2494464, 2125824, 0, 2125824, 2125824, 2125824, 2125824, 2125824, 0, 0, 0, 0, 1080, 1084, 0, 0, 1088, 2125824, 2514944, 2125824, 2125824, 2125824, 2543616, 2547712, 2125824, 2125824, 2596864, 2125824, 2125824, 2125824, 2125824, 2125824, 2125824, 2125824, 2125824, 2125824, 2125824, 2125824, 2125824, 2928640, 2125824, 2125824, 2125824, 2998272, 2125824, 2125824, 2125824, 2125824, 3059712, 2125824, 2125824, 2125824, 2125824, 2125824, 2125824, 2125824, 2125824, 3178496, 2179072, 2125824, 2125824, 2179072, 2179072, 2125824, 2125824, 2125824, 2125824, 0, 2486272, 0, 0, 0, 0, 0, 2678784, 2854912, 3006464, 2441216, 0, 0, 0, 0, 0, 0, 0, 0, 0, 2932736, 2965504, 0, 0, 3076096, 0, 0, 0, 0, 0, 444, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 407, 0, 0, 0, 0, 0, 0, 2695168, 3174400, 2646016, 2613248, 2703360, 0, 0, 0, 0, 2977792, 0, 0, 3047424, 3129344, 0, 0, 0, 0, 0, 645, 0, 0, 648, 649, 0, 0, 0, 0, 0, 0, 0, 725, 0, 0, 0, 0, 0, 0, 0, 0, 0, 743, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 2769, 0, 0, 0, 2981888, 2396160, 0, 3153920, 0, 0, 0, 0, 0, 0, 0, 0, 2740224, 0, 0, 0, 0, 0, 0, 541, 3027, 541, 541, 541, 541, 541, 541, 541, 541, 541, 2584, 541, 541, 541, 0, 0, 0, 0, 0, 2793472, 0, 0, 0, 0, 0, 2469888, 2506752, 2756608, 0, 0, 2580480, 0, 0, 0, 0, 0, 0, 2517, 0, 0, 0, 0, 0, 0, 0, 0, 541, 541, 541, 3029, 541, 541, 541, 541, 541, 541, 0, 2396160, 2400256, 2179072, 2179072, 2441216, 2179072, 2469888, 2179072, 2179072, 2179072, 2519040, 2179072, 2179072, 2179072, 2179072, 2179072, 2179072, 2801664, 2813952, 2179072, 2838528, 2179072, 2179072, 2179072, 2179072, 2179072, 2179072, 2179072, 2179072, 2453504, 2179072, 2473984, 2482176, 2179072, 2179072, 2179072, 2179072, 2588672, 2179072, 2613248, 2646016, 2179072, 2179072, 2695168, 2756608, 2179072, 2179072, 2179072, 2932736, 2179072, 2179072, 2179072, 2179072, 2179072, 3117056, 2179072, 2179072, 2179072, 2179072, 2179072, 2179072, 2179072, 2179072, 2179072, 2179072, 2179072, 2179072, 2179072, 2584576, 2179072, 2179072, 2125824, 2125824, 2125824, 2519040, 2125824, 2125824, 2125824, 2125824, 2588672, 2125824, 2613248, 2646016, 2125824, 2125824, 2695168, 2756608, 2125824, 2125824, 2125824, 2125824, 2932736, 2125824, 2125824, 2125824, 2125824, 2125824, 2932736, 2125824, 2125824, 2125824, 2125824, 2125824, 2125824, 2125824, 2125824, 3129344, 2125824, 2125824, 3153920, 3166208, 3174400, 2506752, 2506752, 2506752, 0, 3108864, 3198976, 0, 0, 3043328, 0, 3149824, 2936832, 0, 2760704, 3180, 2437120, 0, 0, 0, 0, 0, 0, 646, 0, 0, 0, 0, 651, 652, 653, 654, 655, 0, 0, 0, 0, 0, 2953216, 0, 0, 2826240, 3158016, 2428928, 0, 3018752, 2764800, 2572288, 0, 0, 0, 0, 0, 677, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 2741, 0, 0, 0, 0, 0, 3051520, 2179072, 2428928, 2437120, 2179072, 2486272, 2179072, 2179072, 2179072, 2179072, 2179072, 2179072, 2179072, 2179072, 2654208, 2678784, 2760704, 2764800, 2854912, 2969600, 2179072, 3006464, 2179072, 3018752, 2179072, 2179072, 2179072, 3149824, 2125824, 2428928, 2437120, 2969600, 2125824, 3006464, 2125824, 3018752, 2125824, 2125824, 2125824, 2125824, 3149824, 2125824, 2428928, 2437120, 2125824, 2486272, 2125824, 2125824, 2125824, 2125824, 2125824, 2125824, 2125824, 2125824, 2654208, 2678784, 2760704, 2764800, 2785280, 2854912, 2969600, 2125824, 3006464, 2125824, 3018752, 2125824, 2125824, 2125824, 2125824, 3149824, 2179072, 3051520, 2125824, 3051520, 2125824, 3051520, 0, 2490368, 2498560, 0, 0, 0, 0, 2875392, 0, 0, 0, 3180, 0, 0, 0, 0, 0, 0, 2834432, 0, 3227648, 2568192, 2564096, 0, 2940928, 2179072, 2179072, 2498560, 2179072, 2179072, 2179072, 2555904, 2564096, 2179072, 2179072, 2179072, 2617344, 2179072, 2179072, 2179072, 2179072, 2179072, 2179072, 2662400, 2179072, 2179072, 2179072, 2179072, 2179072, 2179072, 2179072, 3137536, 2125824, 2125824, 2498560, 2125824, 2125824, 2125824, 2555904, 2564096, 2125824, 2555904, 2564096, 2125824, 2125824, 2125824, 2125824, 2125824, 2125824, 2125824, 2125824, 2125824, 2125824, 3137536, 2940928, 2940928, 0, 0, 0, 0, 0, 2748416, 2879488, 0, 0, 0, 0, 0, 0, 0, 0, 0, 2519, 0, 0, 0, 0, 0, 541, 2940928, 0, 0, 0, 0, 0, 2748416, 2879488, 0, 3180, 0, 0, 0, 0, 0, 0, 0, 375, 0, 0, 0, 0, 0, 0, 0, 360, 0, 0, 2179072, 2179072, 2179072, 2179072, 2179072, 2179072, 2179072, 2179072, 2179072, 2179072, 2179072, 2179072, 2179072, 2179072, 2179072, 2179072, 2125824, 0, 2502656, 0, 0, 3010560, 0, 0, 0, 0, 0, 0, 0, 0, 2990080, 2179072, 2179072, 2179072, 3129344, 2179072, 2179072, 3153920, 3166208, 3174400, 2396160, 2400256, 2125824, 2125824, 2441216, 2125824, 2469888, 2125824, 2125824, 2125824, 2519040, 2125824, 2125824, 2179072, 2502656, 2179072, 2179072, 2179072, 2179072, 2179072, 2179072, 2179072, 2179072, 2179072, 3010560, 2179072, 2179072, 2125824, 2125824, 2125824, 2125824, 2125824, 2125824, 2605056, 2125824, 2629632, 2125824, 2125824, 2650112, 2125824, 2125824, 2125824, 2707456, 2125824, 2736128, 2125824, 2125824, 2125824, 2125824, 2125824, 2502656, 2125824, 2125824, 2125824, 2125824, 2125824, 2125824, 2125824, 2125824, 2125824, 3010560, 2125824, 2125824, 2125824, 2125824, 2125824, 2125824, 2125824, 2125824, 2125824, 2125824, 3137536, 2125824, 2125824, 2498560, 2125824, 2125824, 2502656, 2125824, 2125824, 2125824, 2125824, 2125824, 2125824, 2125824, 2125824, 2125824, 3010560, 2125824, 2125824, 2125824, 0, 0, 0, 0, 0, 0, 2739, 0, 0, 0, 0, 0, 0, 0, 2743, 0, 0, 0, 2592768, 0, 0, 0, 0, 0, 0, 2179072, 2179072, 2179072, 2179072, 2179072, 2592768, 2179072, 2179072, 2179072, 3129344, 2179072, 2179072, 3153920, 3166208, 3174400, 2397073, 2401169, 2126737, 2126737, 2442129, 2126737, 2470801, 2125824, 2125824, 2449408, 0, 2535424, 3031040, 2859008, 0, 0, 0, 0, 0, 2179072, 2449408, 2179072, 2535424, 2179072, 2609152, 2179072, 2859008, 2179072, 2179072, 2179072, 3031040, 2125824, 2449408, 2125824, 2535424, 2125824, 2609152, 2125824, 2859008, 2125824, 2125824, 2125824, 3031040, 2125824, 2125824, 2449408, 2125824, 2535424, 2125824, 2609152, 2125824, 2859008, 2125824, 2125824, 2125824, 0, 2179072, 2125824, 2125824, 2457600, 2179072, 2179072, 2179072, 2179072, 2457600, 2125824, 2125824, 2125824, 2125824, 2125824, 2125824, 2125824, 2179072, 2125824, 2125824, 2179072, 2179072, 2179072, 2179072, 2125824, 2125824, 2125824, 2125824, 2125824, 2125824, 2125824, 2125824, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1727, 0, 0, 1728, 0, 3031040, 2125824, 2527232, 0, 0, 0, 0, 0, 2179072, 2527232, 2179072, 2179072, 2179072, 2179072, 2179072, 2125824, 2126738, 2125824, 2125824, 2125824, 2125824, 2125824, 2125824, 2125824, 2125824, 2125824, 2125824, 2125824, 2125824, 3125248, 2125824, 2125824, 2125824, 2125824, 2125824, 2527232, 2125824, 2125824, 2125824, 2125824, 2125824, 3092480, 2125824, 2527232, 2125824, 2125824, 2125824, 2125824, 2125824, 3092480, 0, 0, 0, 0, 0, 693, 0, 0, 0, 0, 0, 0, 365, 365, 365, 0, 0, 0, 0, 0, 707, 708, 0, 0, 0, 0, 0, 714, 0, 0, 0, 0, 0, 0, 703, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 3020, 3026944, 0, 0, 0, 2404352, 2179072, 2179072, 2179072, 2179072, 3026944, 2404352, 2125824, 2125824, 2125824, 2125824, 3026944, 0, 2539520, 0, 2949120, 2179072, 2658304, 2973696, 2179072, 2125824, 2658304, 2973696, 2125824, 2125824, 2658304, 2973696, 2125824, 0, 2711552, 2560000, 2179072, 2560000, 2125824, 2560000, 2125824, 0, 2179072, 2179072, 2531328, 2179072, 2179072, 2179072, 2179072, 2179072, 2179072, 2179072, 2605056, 2179072, 2629632, 2179072, 2179072, 2650112, 0, 0, 2809856, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 3088384, 0, 0, 0, 0, 0, 442, 0, 0, 454, 470, 470, 470, 470, 470, 470, 470, 470, 470, 470, 480, 470, 470, 470, 470, 470, 111051, 111051, 111051, 111051, 111051, 111051, 111051, 111051, 111051, 111051, 111051, 111051, 111051, 111051, 111051, 111051, 459, 459, 111051, 459, 459, 459, 459, 459, 459, 459, 459, 459, 111051, 111051, 111051, 111051, 111051, 111051, 111051, 111051, 111051, 111051, 459, 111051, 111051, 111051, 459, 111051, 111051, 111051, 111051, 111051, 0, 0, 0, 0, 0, 0, 0, 0, 0, 2795, 0, 0, 0, 0, 0, 0, 12290, 3, 0, 0, 2183168, 0, 0, 0, 0, 0, 299, 300, 0, 2134016, 303, 304, 2125824, 1060, 0, 0, 2179072, 2125824, 2125824, 2179072, 2179072, 2179072, 2179072, 2179072, 2125824, 2125824, 2125824, 2125824, 2125824, 2125824, 2125824, 2125824, 2125824, 2125824, 2125824, 2125824, 2125824, 2125824, 2125824, 2125824, 2125824, 1060, 2125824, 2125824, 2125824, 2125824, 2125824, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 662, 0, 662, 0, 0, 0, 0, 303, 118784, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 2434, 0, 303, 303, 303, 303, 303, 303, 303, 0, 0, 0, 0, 0, 303, 0, 303, 1, 12290, 3, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 2155, 0, 12290, 3, 0, 0, 2183168, 0, 0, 0, 0, 0, 33403, 300, 0, 2134016, 49791, 304, 2105632, 12290, 3, 0, 0, 294, 0, 0, 0, 0, 294, 0, 0, 0, 0, 0, 0, 0, 2124, 0, 0, 0, 0, 0, 0, 0, 0, 0, 788, 814, 0, 814, 809, 0, 0, 0, 122880, 122880, 122880, 122880, 122880, 122880, 122880, 122880, 122880, 122880, 122880, 122880, 122880, 122880, 122880, 0, 0, 0, 122880, 0, 122880, 122880, 122880, 0, 0, 0, 0, 0, 122880, 0, 0, 122880, 0, 0, 0, 0, 0, 0, 0, 0, 122880, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 3002, 0, 0, 0, 0, 0, 0, 0, 0, 122880, 122880, 122880, 122880, 122880, 122880, 122880, 122880, 122880, 0, 0, 122880, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 712, 0, 0, 0, 716, 0, 0, 0, 147456, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 2459, 0, 1092, 0, 2424832, 2433024, 0, 0, 2457600, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 728, 0, 0, 0, 0, 0, 1823, 2125824, 2125824, 2125824, 2408448, 2125824, 2125824, 2125824, 2125824, 2125824, 2125824, 2125824, 2125824, 2125824, 2125824, 2125824, 0, 1919, 2125824, 2125824, 2125824, 0, 0, 0, 131072, 0, 0, 0, 69632, 73728, 0, 0, 0, 0, 0, 65536, 0, 0, 0, 0, 0, 740, 407, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1158, 0, 0, 0, 0, 0, 0, 131072, 0, 0, 131072, 131072, 0, 0, 0, 0, 0, 0, 0, 131072, 0, 0, 131072, 0, 0, 131072, 0, 0, 0, 0, 135168, 135168, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 2756, 0, 0, 0, 0, 135168, 0, 0, 135168, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 735, 0, 797, 0, 0, 0, 0, 0, 0, 0, 135168, 0, 135168, 135168, 135168, 135168, 135168, 135168, 0, 135168, 135168, 135168, 135168, 135168, 135168, 135168, 0, 0, 0, 0, 0, 135168, 0, 135168, 1, 12290, 3, 0, 0, 2183168, 0, 0, 0, 0, 0, 636, 637, 0, 2134016, 640, 641, 0, 1361, 2125824, 2125824, 2125824, 2125824, 2125824, 2424832, 2433024, 2125824, 2125824, 2125824, 2125824, 2125824, 2125824, 2125824, 2125824, 2125824, 0, 0, 987, 2125824, 2125824, 2125824, 2125824, 0, 304, 139264, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 225709, 0, 12290, 3, 0, 0, 2183168, 0, 0, 0, 0, 0, 299, 33406, 0, 2134016, 303, 49794, 0, 61440, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 3178, 0, 304, 2424832, 2433024, 0, 0, 2457600, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 757, 0, 0, 0, 0, 762, 301, 301, 301, 143661, 371, 301, 301, 301, 301, 301, 301, 301, 301, 301, 301, 301, 301, 301, 301, 301, 301, 143661, 301, 301, 143661, 301, 301, 301, 143731, 301, 301, 301, 143731, 69632, 73728, 301, 301, 143661, 301, 301, 65536, 301, 301, 301, 301, 301, 143661, 143661, 143661, 143661, 143661, 143661, 143661, 143661, 143661, 301, 301, 143661, 301, 301, 301, 301, 301, 301, 301, 301, 301, 366, 301, 0, 143661, 301, 301, 301, 143661, 143661, 143661, 143661, 143661, 143661, 143661, 143661, 143661, 143661, 143661, 143661, 143661, 143661, 143661, 143661, 301, 301, 301, 143661, 301, 143661, 143661, 143661, 143661, 143661, 143731, 143661, 143731, 143731, 143731, 143731, 143731, 143731, 143731, 143661, 143661, 143661, 143661, 143661, 143661, 143661, 143661, 1, 301, 143661, 301, 143661, 143661, 143661, 143661, 143661, 143661, 301, 0, 301, 0, 301, 301, 301, 301, 301, 301, 301, 301, 301, 143661, 301, 143661, 143661, 143661, 143661, 301, 0, 0, 0, 3117056, 0, 0, 0, 0, 0, 0, 0, 2200252, 2200252, 0, 0, 0, 0, 0, 0, 662, 0, 0, 0, 0, 0, 541, 541, 541, 541, 541, 541, 541, 541, 2168, 541, 541, 541, 0, 0, 0, 0, 0, 155648, 155648, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 325, 0, 0, 0, 0, 155648, 155648, 0, 155648, 155648, 155648, 155648, 155648, 155648, 155648, 155648, 155648, 0, 0, 0, 0, 155648, 0, 0, 155648, 155648, 155648, 155648, 155648, 155648, 155648, 155648, 155648, 155648, 155648, 155648, 155648, 155648, 155648, 155648, 0, 0, 0, 0, 0, 155648, 0, 155648, 0, 0, 0, 0, 0, 155648, 0, 0, 0, 0, 0, 155648, 155648, 0, 155648, 155648, 0, 12290, 3, 0, 0, 2183168, 126976, 0, 0, 0, 0, 299, 300, 0, 2134016, 303, 304, 0, 0, 0, 3117056, 0, 0, 0, 0, 0, 0, 0, 365, 0, 0, 0, 0, 0, 0, 0, 3563, 0, 541, 541, 541, 541, 541, 541, 541, 541, 541, 3209, 541, 541, 541, 541, 541, 159744, 159744, 159744, 159744, 0, 0, 159744, 0, 0, 0, 0, 0, 0, 0, 0, 159744, 0, 0, 0, 0, 0, 0, 0, 159744, 159744, 159744, 163840, 159744, 159744, 159744, 159744, 159744, 0, 0, 0, 0, 0, 0, 0, 0, 0, 2805, 0, 541, 541, 541, 541, 541, 0, 25162, 0, 0, 0, 159744, 0, 0, 0, 25162, 25162, 25162, 159744, 25162, 25162, 25162, 25162, 25162, 25162, 25162, 25162, 159744, 159744, 159744, 159744, 25162, 159744, 25162, 1, 167936, 167936, 167936, 167936, 167936, 167936, 167936, 167936, 167936, 167936, 167936, 167936, 167936, 167936, 167936, 167936, 1, 167936, 167936, 167936, 167936, 0, 0, 167936, 0, 0, 0, 0, 0, 0, 0, 0, 167936, 167936, 167936, 167936, 167936, 167936, 167936, 167936, 167936, 0, 0, 0, 0, 0, 0, 0, 0, 0, 2976, 0, 0, 0, 0, 0, 0, 0, 0, 0, 2138112, 1183, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 3168, 0, 0, 0, 0, 0, 0, 0, 0, 172032, 0, 172032, 0, 0, 0, 0, 0, 0, 0, 0, 0, 365, 0, 293, 0, 0, 0, 348, 172032, 172032, 0, 0, 172032, 0, 0, 172032, 172032, 0, 172032, 0, 0, 0, 0, 172032, 172032, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 172032, 0, 0, 0, 0, 0, 0, 2777, 0, 0, 0, 0, 0, 0, 2784, 0, 0, 0, 0, 0, 0, 2986, 2987, 0, 2988, 0, 2990, 0, 0, 0, 0, 0, 0, 0, 69632, 73728, 0, 135168, 135168, 0, 0, 65536, 135168, 172032, 0, 172032, 172032, 0, 172032, 172032, 172032, 172032, 172032, 172032, 172032, 172032, 172032, 172032, 172032, 172032, 172032, 172032, 172032, 172032, 172032, 172032, 172032, 172032, 172032, 172032, 172032, 172032, 172032, 0, 0, 0, 0, 0, 0, 0, 0, 0, 3014, 0, 0, 0, 0, 0, 0, 176128, 176128, 176128, 176128, 176128, 176128, 176128, 176128, 176128, 176128, 176128, 176128, 176128, 176128, 176128, 176128, 1, 176128, 176128, 176128, 176128, 0, 0, 176128, 0, 0, 0, 0, 0, 0, 0, 0, 176128, 176128, 176128, 176128, 176128, 176128, 176128, 176128, 176128, 0, 0, 0, 0, 0, 0, 0, 0, 0, 3175, 0, 0, 0, 0, 0, 0, 0, 3, 78115, 78115, 293, 0, 634, 0, 0, 0, 299, 300, 0, 2134016, 303, 304, 1164, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 361, 541, 541, 1308, 541, 541, 541, 541, 541, 541, 541, 541, 541, 541, 541, 541, 541, 0, 0, 2242, 0, 914, 563, 563, 563, 563, 563, 563, 563, 563, 563, 563, 563, 563, 563, 563, 0, 0, 587, 587, 587, 1203, 541, 587, 563, 541, 541, 541, 541, 541, 563, 563, 563, 563, 3070, 563, 563, 563, 563, 3074, 563, 563, 587, 587, 587, 587, 587, 1079, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1102, 0, 0, 0, 0, 0, 0, 0, 229376, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1134592, 0, 0, 0, 0, 1691, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 3201, 563, 563, 587, 587, 587, 587, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 541, 541, 541, 541, 3569, 541, 541, 0, 0, 563, 563, 563, 563, 563, 563, 563, 2251, 563, 563, 563, 563, 563, 563, 587, 3109, 587, 587, 587, 587, 587, 587, 587, 587, 2407, 587, 587, 587, 587, 587, 2412, 2413, 563, 563, 2288, 563, 563, 563, 563, 563, 563, 563, 563, 563, 563, 563, 563, 563, 563, 3541, 587, 587, 587, 2375, 587, 587, 587, 587, 587, 587, 587, 587, 587, 587, 587, 587, 587, 1497, 587, 587, 0, 0, 0, 563, 563, 563, 563, 563, 563, 563, 563, 563, 563, 563, 563, 563, 563, 563, 1439, 0, 3007, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 3317, 541, 541, 541, 541, 3038, 541, 541, 541, 541, 541, 541, 541, 541, 541, 541, 541, 541, 1300, 541, 541, 541, 563, 563, 563, 3078, 563, 563, 563, 563, 563, 563, 563, 563, 563, 563, 563, 563, 1401, 563, 563, 563, 587, 587, 587, 587, 3120, 587, 587, 587, 587, 587, 587, 587, 587, 587, 587, 587, 1042, 587, 587, 587, 587, 0, 3159, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 155648, 3214, 541, 541, 541, 541, 541, 541, 541, 541, 541, 541, 541, 541, 541, 541, 541, 541, 1359, 563, 563, 563, 587, 3259, 587, 587, 587, 3262, 587, 587, 587, 587, 587, 587, 587, 541, 541, 563, 563, 587, 587, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 3609, 541, 541, 541, 587, 587, 3270, 587, 587, 587, 587, 587, 587, 587, 587, 587, 587, 587, 587, 587, 1498, 587, 587, 541, 541, 541, 3330, 3331, 541, 541, 541, 541, 541, 541, 541, 541, 563, 563, 563, 563, 3581, 563, 563, 563, 563, 563, 3349, 563, 563, 563, 563, 563, 563, 563, 563, 563, 587, 587, 587, 587, 587, 587, 587, 587, 0, 541, 541, 541, 3406, 541, 541, 541, 541, 563, 563, 563, 563, 563, 563, 3415, 563, 0, 587, 587, 587, 587, 587, 291, 1079, 0, 0, 0, 0, 0, 0, 0, 0, 0, 249856, 249856, 249856, 249856, 249856, 249856, 249856, 249856, 1, 587, 3431, 587, 587, 587, 587, 587, 587, 3437, 587, 587, 587, 3441, 587, 541, 563, 563, 3473, 563, 563, 563, 563, 3478, 563, 563, 563, 563, 3483, 563, 563, 563, 563, 977, 563, 563, 563, 563, 0, 587, 587, 587, 587, 587, 587, 1931, 587, 587, 587, 587, 587, 587, 587, 587, 587, 2678, 587, 587, 587, 2681, 587, 587, 0, 0, 3456, 541, 541, 541, 541, 541, 541, 541, 541, 541, 541, 541, 541, 541, 0, 2586, 0, 541, 3471, 563, 563, 563, 563, 563, 563, 563, 563, 563, 563, 563, 563, 563, 563, 0, 2327, 563, 3487, 587, 587, 587, 587, 587, 587, 587, 587, 587, 587, 587, 587, 587, 587, 0, 0, 563, 563, 587, 587, 587, 587, 0, 0, 541, 541, 563, 563, 587, 587, 0, 541, 541, 541, 3205, 541, 541, 541, 541, 541, 541, 541, 3210, 541, 541, 541, 541, 541, 541, 2816, 541, 541, 2820, 541, 541, 541, 541, 541, 541, 541, 541, 3041, 541, 541, 541, 541, 541, 541, 541, 541, 2556, 541, 541, 541, 541, 541, 541, 541, 541, 2233, 541, 541, 541, 541, 541, 0, 0, 563, 587, 0, 541, 563, 587, 0, 541, 563, 587, 0, 541, 563, 587, 0, 0, 0, 0, 0, 0, 0, 0, 0, 3510, 0, 0, 0, 0, 541, 541, 541, 541, 541, 2165, 541, 541, 541, 541, 541, 541, 541, 541, 3616, 563, 563, 563, 563, 563, 563, 563, 2265, 563, 563, 563, 563, 2270, 563, 563, 563, 184932, 184932, 184932, 184932, 184932, 184932, 184932, 0, 0, 0, 0, 0, 184932, 0, 184932, 1, 12290, 3, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 301, 301, 0, 0, 301, 301, 143661, 301, 301, 301, 301, 301, 301, 301, 301, 301, 301, 301, 143731, 301, 301, 301, 301, 12290, 3, 78115, 78115, 293, 0, 0, 0, 0, 0, 299, 300, 0, 2134016, 303, 304, 587, 0, 0, 188416, 541, 587, 563, 541, 541, 541, 541, 541, 563, 563, 563, 563, 587, 587, 587, 587, 0, 0, 0, 0, 2731, 0, 0, 0, 0, 305, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 249856, 0, 0, 0, 192972, 192972, 192972, 192972, 192972, 192972, 192972, 192972, 192972, 192972, 192972, 192972, 192972, 192972, 192972, 192972, 0, 192972, 0, 1, 0, 0, 0, 0, 0, 192972, 192972, 192972, 192972, 192972, 192972, 192972, 192972, 192972, 192972, 0, 0, 192972, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1145, 365, 365, 0, 0, 1148, 192972, 192972, 192972, 0, 192972, 192972, 192972, 192972, 192972, 0, 0, 0, 0, 0, 0, 0, 0, 0, 3304, 0, 3180, 0, 0, 0, 0, 302, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 541, 407, 407, 407, 407, 407, 407, 407, 0, 0, 0, 0, 0, 407, 0, 407, 1, 12290, 3, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 118784, 299, 12290, 3, 78115, 78115, 293, 0, 0, 0, 0, 0, 299, 300, 0, 302, 303, 304, 587, 0, 0, 740, 541, 587, 563, 541, 541, 541, 541, 541, 563, 563, 563, 563, 587, 587, 587, 587, 0, 0, 2729, 0, 0, 0, 0, 0, 0, 740, 1184, 0, 0, 0, 0, 1188, 0, 0, 0, 0, 0, 0, 0, 378, 0, 381, 0, 0, 0, 381, 0, 0, 563, 587, 587, 587, 587, 587, 1079, 1559, 0, 0, 0, 1565, 0, 0, 0, 1571, 2034, 0, 0, 0, 0, 1577, 2036, 0, 0, 0, 0, 0, 0, 0, 397, 0, 0, 0, 0, 0, 397, 0, 0, 0, 0, 0, 1577, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 262144, 0, 0, 0, 0, 0, 1188, 1669, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 3177, 0, 0, 0, 563, 563, 587, 587, 587, 587, 1559, 2030, 0, 0, 0, 0, 1565, 2032, 0, 0, 0, 0, 0, 0, 2997, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1586, 1587, 0, 0, 0, 0, 0, 0, 0, 0, 1669, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 106496, 0, 0, 0, 0, 0, 0, 2034, 0, 2036, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1150976, 0, 0, 0, 0, 0, 563, 563, 563, 563, 2325, 2652, 0, 0, 0, 0, 587, 587, 587, 587, 587, 587, 587, 587, 587, 587, 587, 587, 587, 2343, 0, 307, 0, 307, 0, 0, 0, 0, 0, 0, 0, 0, 0, 307, 0, 0, 0, 0, 0, 0, 3011, 0, 0, 0, 0, 3016, 3017, 0, 3019, 0, 0, 0, 0, 0, 204800, 204800, 0, 204800, 204800, 204800, 204800, 204800, 204800, 204800, 204800, 204800, 204800, 204800, 204800, 204800, 204800, 204800, 205107, 204800, 204800, 205106, 205107, 204800, 205106, 205106, 204800, 204800, 0, 0, 0, 0, 0, 0, 0, 0, 0, 122880, 0, 122880, 122880, 122880, 122880, 122880, 12290, 3, 0, 0, 2183801, 0, 0, 0, 0, 0, 299, 300, 151552, 2134016, 303, 304, 0, 212992, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 172032, 0, 0, 151552, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 303, 0, 0, 0, 0, 0, 3117056, 0, 0, 0, 0, 0, 0, 0, 0, 2200253, 0, 0, 0, 0, 0, 0, 678, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 2977, 0, 0, 0, 0, 0, 0, 914, 2126737, 2126737, 2126737, 2126737, 2126737, 2425745, 2433937, 2126737, 2126737, 2126737, 2126737, 2126737, 2126737, 2126737, 2126737, 2126737, 2126737, 3138449, 2126811, 2126811, 2499547, 2126811, 2126811, 2126737, 2724753, 2126737, 2732945, 2773905, 2126737, 2126737, 2126737, 2806673, 2126737, 2831249, 2126737, 2126737, 2864017, 2126737, 2126737, 2126737, 2126737, 2126811, 2126811, 2126811, 2126811, 2126811, 2593755, 2126811, 2126811, 2126811, 2126811, 2126811, 2126811, 2126811, 3036123, 2126811, 2126811, 3072987, 2126811, 2126811, 2126811, 3122139, 2126811, 2126737, 2126737, 2921361, 2126737, 2126737, 2126737, 2126737, 2126737, 2126737, 2126737, 2126737, 2126737, 2126737, 2126737, 3117969, 2126737, 0, 2126811, 2126811, 2126811, 2126811, 2126811, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1190, 0, 0, 0, 0, 0, 2126811, 2425819, 2434011, 2126811, 2126811, 2126811, 2126811, 2126811, 2126811, 2126811, 2126811, 2126811, 2126811, 2126811, 2126811, 2126811, 3126235, 2126811, 2126811, 2126811, 2126811, 2126811, 2626523, 2126811, 2126811, 2126811, 2126811, 2126811, 2126811, 2700251, 2126811, 2716635, 2126811, 2724827, 2126811, 2733019, 2773979, 2126811, 0, 2502656, 0, 0, 3010560, 0, 0, 0, 0, 0, 0, 0, 0, 2990080, 2179072, 2179072, 3125248, 2179072, 2179072, 2179072, 2179072, 2179072, 2179072, 2179072, 2179072, 2179072, 2179072, 2179072, 2179072, 2125824, 2125824, 2125824, 2125824, 2125824, 2592768, 2125824, 2125824, 2125824, 2125824, 2126811, 2126811, 2806747, 2126811, 2831323, 2126811, 2126811, 2864091, 2126811, 2126811, 2126811, 2126811, 2921435, 2126811, 2126811, 2126811, 0, 2179072, 2126811, 2126737, 2457600, 2179072, 2179072, 2179072, 2179072, 2458513, 2126737, 2126737, 2126737, 2126737, 2126737, 2524049, 2126737, 2126737, 2126737, 2126737, 2126737, 2126737, 2126737, 2126737, 2601873, 2126737, 2126737, 2458587, 2126811, 2126811, 2126811, 2126811, 2183168, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1206, 0, 1208, 0, 0, 0, 0, 2126737, 2126737, 2126737, 2409361, 2126737, 2126737, 2126737, 2126737, 2126737, 2126737, 2126737, 2126737, 2126737, 2126737, 2126737, 0, 0, 2126811, 2126811, 2126811, 2409435, 2126811, 2126811, 2126811, 2126811, 2126811, 2126811, 2126811, 2126811, 2126811, 2126811, 2126811, 2126811, 2126811, 2126811, 2126811, 2126811, 2126811, 2839515, 2126811, 2126811, 2126811, 2126811, 2126811, 2126811, 2126811, 2126811, 2126811, 2126811, 2126811, 2126811, 2126811, 2126811, 3118043, 2126811, 2126811, 2126811, 2126811, 2126811, 2126811, 2126811, 0, 0, 2126737, 2126737, 2126737, 2126737, 2126737, 2126737, 2126737, 2126737, 2126737, 2126737, 2126737, 2462609, 2466705, 2126737, 2126737, 2126737, 2642833, 2126737, 2126737, 2126737, 2126737, 2126737, 2126737, 2720657, 2126737, 2126737, 2126737, 2126737, 2126737, 2126737, 2126737, 3040145, 2126737, 3064721, 2126737, 2126737, 2126737, 2126737, 2126737, 3101585, 2769809, 2778001, 2782097, 2798481, 2823057, 2126737, 2126737, 2126737, 2884497, 2126737, 2913169, 2126737, 2126737, 2126737, 2126737, 2126737, 2126737, 2626449, 2126737, 2126737, 2126737, 2126737, 2126737, 2126737, 2700177, 2126737, 2716561, 2126737, 2126737, 3134353, 2126737, 2126737, 2126737, 2126737, 2126737, 2126737, 2126737, 2126737, 2126737, 3208081, 2126737, 0, 0, 0, 0, 0, 0, 3026, 541, 541, 541, 541, 541, 541, 541, 541, 541, 541, 1352, 541, 541, 541, 541, 541, 0, 0, 2126811, 2126811, 2126811, 2126811, 2126811, 2126811, 2126811, 2126811, 2126811, 2126811, 2126811, 2462683, 2466779, 2126811, 2126811, 2126811, 2126811, 2126811, 2929627, 2126811, 2126811, 2126811, 2999259, 2126811, 2126811, 2126811, 2126811, 3060699, 2126811, 2126811, 2126811, 2126811, 2454491, 2126811, 2474971, 2483163, 2126811, 2126811, 2126811, 2126811, 2126811, 2126811, 2532315, 2126811, 2126811, 2126811, 2446299, 2126811, 2126811, 2126811, 2126811, 2126811, 2126811, 2126811, 2126811, 2126811, 2126811, 2126811, 2552795, 2769883, 2778075, 2782171, 2798555, 2823131, 2126811, 2126811, 2126811, 2884571, 2126811, 2913243, 2126811, 2126811, 2126811, 2126811, 2126811, 2126811, 2126811, 3130331, 2126811, 2126811, 3154907, 3167195, 3175387, 2506752, 2507739, 2507665, 2126811, 2126811, 3134427, 2126811, 2126811, 2126811, 2126811, 2126811, 2126811, 2126811, 2126811, 2126811, 3208155, 2126811, 2179072, 2126811, 2126811, 2126811, 2642907, 2126811, 2126811, 2126811, 2126811, 2126811, 2126811, 2720731, 2126811, 2126811, 2126811, 2126811, 2126811, 2126811, 2126811, 3040219, 2126811, 3064795, 2126811, 2126811, 2126811, 2126811, 2126811, 3101659, 0, 0, 0, 2388881, 2126737, 2126737, 2126737, 2421649, 2126737, 2126737, 2126737, 2126737, 2126737, 2454417, 2126737, 2474897, 2483089, 2126737, 2126737, 2126737, 2126737, 2126737, 2126737, 2532241, 2126737, 2126737, 2126737, 2126737, 2126737, 2126737, 2126737, 2605969, 2126737, 2630545, 2126737, 2126737, 2651025, 2126737, 2126737, 2126737, 2708369, 2126737, 2737041, 2126737, 2126737, 2126737, 2126737, 2126737, 2126737, 2663313, 2126737, 2126737, 2126737, 2126737, 2126737, 2126737, 2126737, 2126737, 2126737, 0, 0, 988, 2126811, 2126811, 2126811, 2126811, 3036049, 2126737, 2126737, 3072913, 2126737, 2126737, 2126737, 3122065, 2126737, 2126737, 3142545, 2126737, 2126737, 2126737, 3171217, 2126737, 2126737, 2126737, 2888593, 2126737, 2126737, 2925457, 2126737, 2126737, 2126737, 2126737, 2126737, 2126737, 2126737, 2126737, 2126737, 986, 2126811, 2126811, 2126811, 2126811, 2126811, 2126811, 2126737, 3191697, 3195793, 2126737, 0, 0, 0, 0, 0, 0, 2388955, 2126811, 2126811, 2126811, 2421723, 2126811, 2126811, 2449408, 0, 2535424, 3031040, 2859008, 0, 0, 0, 0, 0, 2179072, 2449408, 2179072, 2535424, 2126811, 2737115, 2126811, 2126811, 2126811, 2126811, 2126811, 2126811, 2126811, 2126811, 2888667, 2126811, 2126811, 2925531, 2126811, 2126811, 2126811, 2126811, 2126811, 2585563, 2126811, 2126811, 2126811, 2126811, 2126811, 2618331, 2126811, 2126811, 2126811, 2126811, 2126811, 2126811, 2606043, 2126811, 2630619, 2126811, 2126811, 2651099, 2126811, 2126811, 2126811, 2708443, 2126811, 3142619, 2126811, 2126811, 2126811, 3171291, 2126811, 2126811, 3191771, 3195867, 2126811, 2179072, 2126811, 2126737, 2179072, 2179072, 2179072, 2637824, 2179072, 2179072, 2179072, 2179072, 2727936, 2752512, 2179072, 2179072, 2179072, 2842624, 2846720, 2179072, 3112960, 3219456, 2126737, 2126737, 3113873, 3220369, 2126811, 2126811, 3113947, 3220443, 0, 0, 0, 0, 0, 0, 0, 415, 415, 0, 0, 0, 0, 0, 415, 0, 3096576, 2179072, 2179072, 2179072, 2179072, 2179072, 2179072, 2179072, 3223552, 0, 0, 2126737, 2126737, 2417553, 2126737, 2126737, 2126737, 2126737, 2929553, 2126737, 2126737, 2126737, 2999185, 2126737, 2126737, 2126737, 2126737, 3060625, 2126737, 2126737, 2126737, 2126737, 2933649, 2126737, 2126737, 2126737, 2126737, 2126737, 2126737, 2126737, 2126737, 3130257, 2126737, 2126737, 2126737, 2519953, 2126737, 2126737, 2126737, 2126737, 2589585, 2126737, 2614161, 2646929, 2126737, 2126737, 2696081, 2757521, 2126737, 2446225, 2126737, 2126737, 2126737, 2126737, 2126737, 2126737, 2126737, 2126737, 2126737, 2126737, 2126737, 2552721, 2126737, 2126737, 2126737, 2638737, 2126737, 2126737, 2126737, 2126737, 2728849, 2753425, 2126737, 2126737, 2126737, 2126737, 2843537, 2847633, 2126737, 2896785, 2917265, 2126737, 2126737, 2945937, 2126737, 2126737, 2995089, 2126737, 3003281, 2126737, 2126737, 3023761, 2126737, 3068817, 3085201, 2126737, 3097489, 2126737, 2126737, 2126737, 2126737, 2126737, 2126737, 2126737, 3224465, 0, 0, 2126811, 2126811, 2417627, 2843611, 2847707, 2126811, 2896859, 2917339, 2126811, 2126811, 2946011, 2126811, 2126811, 2995163, 2126811, 3003355, 2126811, 2126811, 3023835, 2126811, 3068891, 3085275, 2126811, 3097563, 2126811, 2126811, 2126811, 2126811, 2126811, 2126811, 2126811, 3224539, 2179072, 2126811, 2126737, 2126737, 2126737, 3031953, 2126737, 2126811, 2450395, 2126811, 2536411, 2126811, 2610139, 2126811, 2859995, 2126811, 2126811, 2126811, 2126811, 2126811, 2524123, 2126811, 2126811, 2126811, 2126811, 2126811, 2126811, 2126811, 2126811, 2601947, 2126811, 2515857, 2126737, 2126737, 2126737, 2544529, 2548625, 2126737, 2126737, 2597777, 2126737, 2126737, 2126737, 2126737, 2126737, 2126737, 2126737, 2126737, 2126737, 2585489, 2126737, 2126737, 2126737, 2126737, 2126737, 2618257, 2126811, 2515931, 2126811, 2126811, 2126811, 2544603, 2548699, 2126811, 2126811, 2597851, 2126811, 2126811, 2126811, 2126811, 2126811, 2126811, 2126811, 2655195, 2679771, 2761691, 2765787, 2786267, 2855899, 2970587, 2126811, 3007451, 3154833, 3167121, 3175313, 2397147, 2401243, 2126811, 2126811, 2442203, 2126811, 2470875, 2126811, 2126811, 2126811, 2520027, 2126811, 2126811, 2126811, 2126811, 2126811, 2638811, 2126811, 2126811, 2126811, 2126811, 2728923, 2753499, 2126811, 2126811, 2126811, 2126811, 2126811, 2126811, 3179483, 2179072, 2126811, 2126737, 2179072, 2179072, 2126737, 2126737, 2126811, 2126811, 0, 0, 0, 0, 0, 0, 2510848, 2514944, 0, 0, 2678784, 2760704, 2764800, 2854912, 2969600, 2179072, 3006464, 2179072, 3018752, 2179072, 2179072, 2179072, 3149824, 2126737, 2429841, 2438033, 2126737, 2487185, 2126737, 2126737, 2126737, 2126737, 2126737, 2126737, 2126737, 2126737, 2655121, 2679697, 2761617, 2765713, 2786193, 2855825, 2970513, 2126737, 3007377, 2126737, 3019665, 2126737, 2126737, 2126737, 2126737, 3150737, 2126811, 2429915, 2438107, 2126811, 2487259, 2126811, 2126811, 2589659, 2126811, 2614235, 2647003, 2126811, 2126811, 2696155, 2757595, 2126811, 2126811, 2126811, 2126811, 2933723, 2126811, 2126811, 2663387, 2126811, 2126811, 2126811, 2126811, 2126811, 2126811, 2126811, 2126811, 2126811, 2126811, 2126811, 2802651, 2814939, 2126811, 3019739, 2126811, 2126811, 2126811, 2126811, 3150811, 2179072, 3051520, 2126737, 3052433, 2126811, 3052507, 0, 2490368, 2498560, 2126811, 2556891, 2565083, 2126811, 2126811, 2126811, 2126811, 2126811, 2126811, 2126811, 2126811, 2126811, 2126811, 3138523, 2940928, 2941841, 2941915, 0, 0, 0, 0, 0, 2748416, 2879488, 0, 3180, 0, 0, 0, 0, 0, 0, 0, 647, 0, 0, 0, 0, 0, 0, 0, 0, 0, 2467, 0, 0, 2470, 0, 0, 0, 2179072, 2502656, 2179072, 2179072, 2179072, 2179072, 2179072, 2179072, 2179072, 2179072, 2179072, 3010560, 2179072, 2179072, 2126737, 2126737, 2126811, 2126811, 0, 2486272, 0, 0, 0, 0, 0, 2678784, 2854912, 3006464, 2503569, 2126737, 2126737, 2126737, 2126737, 2126737, 2126737, 2126737, 2126737, 2126737, 3011473, 2126737, 2126737, 2126737, 2126811, 2126811, 2126811, 2126811, 2126811, 2126811, 2126811, 2126811, 2126811, 2126811, 2126811, 2126811, 2126811, 2126811, 2126811, 2179072, 2126811, 2126737, 2179072, 2179072, 2179072, 2179072, 2126737, 2126737, 2126737, 2126737, 2126811, 2126811, 2126811, 2126811, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 2112, 0, 0, 0, 0, 2503643, 2126811, 2126811, 2126811, 2126811, 2126811, 2126811, 2126811, 2126811, 2126811, 3011547, 2126811, 2126811, 2126811, 0, 0, 0, 2179072, 2126811, 2126737, 2179072, 2179072, 2179072, 2179072, 2179072, 2126737, 2126737, 2126737, 2126737, 2126737, 2126737, 2126737, 2126737, 2126737, 2126737, 2126737, 2126737, 2126737, 2126737, 2126737, 2179072, 2609152, 2179072, 2859008, 2179072, 2179072, 2179072, 3031040, 2126737, 2450321, 2126737, 2536337, 2126737, 2610065, 2126737, 2859921, 3032027, 2126811, 2527232, 0, 0, 0, 0, 0, 2179072, 2527232, 2179072, 2179072, 2179072, 2179072, 2179072, 2126737, 2126737, 2802577, 2814865, 2126737, 2839441, 2126737, 2126737, 2126737, 2126737, 2126737, 2126737, 2126737, 2126737, 2126737, 2126737, 2126737, 2126737, 2126737, 2126737, 3126161, 2126737, 2528145, 2126737, 2126737, 2126737, 2126737, 2126737, 3093393, 2126811, 2528219, 2126811, 2126811, 2126811, 2126811, 2126811, 3093467, 0, 0, 0, 0, 0, 782, 0, 0, 0, 0, 0, 0, 541, 843, 541, 541, 541, 541, 541, 1796, 541, 541, 541, 541, 541, 541, 541, 541, 541, 541, 541, 2822, 541, 541, 541, 541, 3026944, 0, 0, 0, 2404352, 2179072, 2179072, 2179072, 2179072, 3026944, 2405265, 2126737, 2126737, 2126737, 2126737, 3027857, 2405339, 2126811, 2126811, 2126811, 2126811, 3027931, 0, 2539520, 0, 2949120, 2179072, 2658304, 2973696, 2179072, 2126737, 2659217, 2974609, 2126737, 2126811, 2659291, 2974683, 2126811, 0, 2711552, 2560000, 2179072, 2560913, 2126737, 2560987, 2126811, 0, 2179072, 2179072, 3133440, 2179072, 2179072, 2179072, 2179072, 2179072, 2179072, 2179072, 2179072, 2179072, 3207168, 2179072, 0, 0, 0, 0, 0, 0, 2464, 0, 0, 0, 0, 0, 0, 0, 0, 0, 324, 398, 0, 0, 0, 324, 0, 2126737, 2126811, 0, 2179072, 2126737, 2126811, 0, 2179072, 2126737, 2126811, 2985984, 2985984, 2986897, 2986971, 0, 0, 0, 0, 0, 0, 3164, 0, 0, 3167, 0, 0, 0, 0, 0, 0, 0, 1135, 0, 0, 0, 0, 0, 0, 0, 0, 0, 824, 0, 0, 0, 0, 0, 0, 0, 221184, 221184, 0, 0, 0, 0, 0, 0, 0, 0, 0, 221184, 221184, 0, 0, 221184, 221184, 221184, 0, 0, 0, 0, 0, 0, 0, 221184, 0, 0, 221184, 221184, 221184, 221184, 221184, 221184, 221184, 221184, 221184, 221184, 221184, 221184, 221184, 221184, 221184, 221184, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 221184, 0, 221184, 221184, 221184, 221184, 221184, 221184, 221184, 221184, 221184, 221184, 1, 12290, 3, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 139264, 300, 0, 303, 0, 0, 0, 303, 0, 304, 0, 0, 0, 304, 0, 0, 0, 304, 69632, 139682, 0, 0, 0, 0, 0, 65536, 0, 0, 0, 0, 98304, 0, 0, 0, 53248, 0, 0, 0, 0, 0, 2662400, 0, 2813952, 0, 0, 3133440, 0, 98304, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 111051, 0, 0, 0, 0, 303, 0, 304, 0, 0, 0, 2473984, 2478080, 0, 0, 0, 0, 0, 0, 0, 0, 0, 159744, 159744, 159744, 159744, 159744, 159744, 159744, 159744, 159744, 159744, 159744, 159744, 159744, 159744, 159744, 159744, 163840, 3121152, 2179072, 2179072, 3141632, 2179072, 2179072, 2179072, 3170304, 2179072, 2179072, 3190784, 3194880, 2179072, 914, 0, 0, 0, 0, 0, 0, 3172, 3173, 0, 0, 0, 0, 0, 0, 0, 0, 0, 665, 0, 0, 668, 0, 0, 0, 0, 914, 0, 2387968, 2125824, 2125824, 2125824, 2420736, 2125824, 2125824, 2125824, 2125824, 2125824, 2453504, 2125824, 2473984, 2482176, 2125824, 2125824, 2125824, 2125824, 2125824, 2125824, 2531328, 2125824, 2125824, 2125824, 2125824, 2125824, 2125824, 2125824, 2605056, 2125824, 3190784, 3194880, 2125824, 988, 0, 0, 0, 988, 0, 2387968, 2125824, 2125824, 2125824, 2420736, 2125824, 0, 2125824, 2125824, 2125824, 2125824, 2125824, 0, 0, 0, 299, 0, 0, 0, 303, 0, 0, 0, 303, 119198, 73728, 0, 0, 0, 0, 0, 65536, 0, 3096576, 2179072, 2179072, 2179072, 2179072, 2179072, 2179072, 2179072, 3223552, 914, 0, 2125824, 2125824, 2416640, 2125824, 2125824, 2125824, 2125824, 2125824, 2125824, 2625536, 2125824, 2125824, 2125824, 2125824, 2125824, 2125824, 2699264, 2125824, 2715648, 2125824, 2723840, 2125824, 2732032, 2772992, 2125824, 3084288, 2125824, 3096576, 2125824, 2125824, 2125824, 2125824, 2125824, 2125824, 2125824, 3223552, 988, 0, 2125824, 2125824, 2416640, 225890, 225890, 225890, 225890, 225890, 225890, 225890, 225741, 225741, 225741, 225741, 225741, 225906, 225741, 225906, 1, 12290, 3, 0, 0, 0, 0, 0, 0, 0, 90409, 90409, 90409, 90409, 0, 94506, 94506, 90409, 90409, 94506, 94506, 94506, 94506, 94506, 94506, 94506, 94506, 94506, 94506, 94506, 94506, 94506, 94506, 94506, 94506, 1, 2125824, 237568, 2125824, 2125824, 2125824, 2125824, 2125824, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1222, 0, 0, 0, 0, 0, 249856, 249856, 249856, 249856, 249856, 249856, 249856, 249856, 249856, 249856, 249856, 249856, 249856, 249856, 249856, 249856, 0, 0, 0, 0, 0, 0, 0, 0, 0, 217088, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 304, 0, 0, 2125824, 241664, 2125824, 2125824, 2125824, 2125824, 2125824, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1236, 0, 0, 0, 0, 0, 254414, 254414, 254414, 254414, 254414, 254414, 254414, 254414, 254414, 254414, 254414, 254414, 254414, 254414, 254414, 254414, 0, 0, 0, 0, 0, 0, 0, 12290, 0, 0, 0, 2183168, 0, 0, 270336, 0, 0, 299, 300, 0, 2134016, 303, 304, 200704, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 687, 2125824, 0, 2125824, 2125824, 2125824, 2125824, 2125824, 0, 0, 180224, 0, 0, 0, 0, 0, 0, 0, 663, 0, 0, 666, 667, 0, 0, 0, 0, 2940928, 0, 0, 0, 0, 0, 2748416, 2879488, 0, 20480, 0, 0, 0, 0, 0, 0, 0, 679, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1129, 0, 0, 0, 0, 0, 0, 1, 12290, 2113826, 0, 0, 0, 0, 0, 0, 296, 0, 0, 0, 296, 0, 0, 0, 0, 0, 0, 3182, 0, 0, 0, 0, 3187, 0, 0, 0, 0, 0, 0, 0, 69632, 73728, 0, 0, 0, 0, 0, 65536, 0, 0, 0, 0, 266240, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 266240, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 0, 0, 266240, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 2662400, 0, 2813952, 12290, 2113826, 0, 0, 2183168, 0, 0, 0, 0, 0, 299, 300, 0, 2134016, 303, 304, 2125824, 245760, 0, 0, 2179072, 2125824, 2125824, 2179072, 2179072, 2179072, 2179072, 2179072, 2125824, 2125824, 2125824, 2125824, 2125824, 2125824, 2125824, 2125824, 2125824, 2584576, 2125824, 2125824, 2125824, 2125824, 2125824, 2617344, 2125824, 2125824, 2125824, 2125824, 2125824, 245760, 2125824, 2125824, 2125824, 2125824, 2125824, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1245, 0, 0, 0, 0, 0, 274432, 274432, 274432, 274432, 274432, 274432, 274432, 0, 0, 0, 0, 0, 274432, 0, 274432, 1, 12290, 3, 0, 0, 0, 0, 0, 253952, 0, 0, 0, 253952, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1155072, 0, 0, 0, 0, 0, 0, 12290, 3, 78115, 78115, 293, 0, 0, 0, 0, 0, 299, 300, 0, 0, 303, 304, 563, 563, 587, 587, 587, 587, 0, 2030, 0, 0, 0, 0, 0, 2032, 0, 0, 0, 0, 0, 0, 3196, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 3186, 0, 0, 0, 3189, 0, 0, 0, 0, 2034, 0, 0, 0, 0, 0, 2036, 0, 0, 0, 0, 0, 0, 0, 695, 0, 0, 0, 0, 365, 365, 365, 0, 0, 2485, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 266240, 0, 0, 0, 1678, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 338, 0, 0, 0, 0, 0, 0, 0, 1669, 0, 0, 0, 0, 0, 0, 0, 0, 2114, 0, 0, 0, 0, 0, 0, 3395, 541, 541, 541, 541, 3399, 541, 541, 541, 541, 541, 541, 1346, 541, 541, 541, 541, 541, 541, 1356, 541, 541, 12290, 3, 78115, 78456, 293, 0, 0, 0, 0, 0, 299, 300, 0, 0, 303, 304, 541, 588, 564, 564, 564, 564, 564, 564, 564, 588, 588, 588, 541, 588, 588, 588, 588, 588, 588, 588, 588, 564, 564, 541, 564, 588, 564, 588, 1, 0, 0, 0, 0, 2775, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 163840, 0, 0, 0, 1, 12290, 3, 78116, 293, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 167936, 0, 0, 0, 282624, 282624, 282624, 282624, 282624, 282624, 282624, 282624, 282624, 282624, 282624, 282624, 282624, 282624, 282624, 282624, 1, 0, 0, 0, 0, 0, 282624, 282624, 282624, 282624, 282624, 282624, 282624, 282624, 282624, 282624, 0, 0, 282624, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1600, 1601, 0, 0, 0, 0, 282624, 282624, 282624, 0, 282624, 282624, 282624, 282624, 282624, 0, 0, 0, 0, 0, 0, 0, 0, 0, 254414, 254414, 254414, 254414, 254414, 254414, 254414, 254414, 1, 2981888, 2396160, 0, 3153920, 3180, 0, 0, 0, 0, 0, 0, 0, 2740224, 0, 0, 0, 0, 0, 0, 679, 751, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1144, 0, 365, 365, 0, 1147, 0, 0, 0, 0, 0, 286720, 286720, 0, 286720, 286720, 286720, 286720, 286720, 286720, 286720, 286720, 286720, 0, 0, 0, 0, 0, 0, 0, 286720, 286720, 286720, 286720, 286720, 286720, 286720, 286720, 286720, 286720, 286720, 286720, 286720, 286720, 286720, 286720, 0, 3108864, 3198976, 0, 0, 3043328, 0, 3149824, 2936832, 0, 2760704, 3305, 2437120, 0, 0, 0, 0, 0, 0, 680, 0, 0, 0, 0, 0, 0, 0, 0, 0, 300, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 2875392, 0, 0, 0, 3386, 0, 0, 0, 0, 0, 0, 2834432, 2940928, 0, 0, 0, 0, 0, 2748416, 2879488, 0, 3386, 0, 0, 0, 0, 0, 0, 0, 709, 0, 0, 0, 0, 0, 0, 0, 0, 0, 2048, 0, 0, 0, 0, 0, 0, 0, 0, 0, 309, 0, 0, 0, 0, 0, 308, 0, 308, 309, 0, 308, 308, 0, 0, 0, 308, 308, 309, 309, 0, 0, 0, 0, 0, 0, 308, 408, 309, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 781, 0, 0, 0, 309, 413, 0, 0, 69632, 73728, 0, 0, 0, 0, 0, 65536, 0, 0, 0, 0, 0, 783, 0, 802, 0, 0, 0, 0, 541, 847, 541, 541, 541, 541, 541, 2830, 2831, 541, 541, 541, 541, 2834, 541, 541, 541, 541, 541, 541, 541, 3521, 541, 3523, 541, 541, 3525, 541, 563, 563, 0, 0, 432, 0, 0, 0, 0, 308, 449, 463, 463, 463, 463, 463, 463, 463, 463, 463, 463, 463, 463, 463, 463, 463, 463, 463, 463, 463, 463, 489, 489, 463, 489, 489, 489, 489, 489, 489, 489, 514, 489, 489, 489, 489, 489, 489, 489, 489, 489, 489, 489, 489, 489, 489, 489, 534, 489, 489, 489, 489, 489, 542, 565, 542, 565, 542, 542, 565, 542, 589, 565, 565, 565, 565, 565, 565, 565, 589, 589, 589, 542, 589, 589, 589, 589, 589, 589, 589, 589, 565, 565, 542, 565, 589, 565, 589, 1, 0, 0, 674, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 323, 324, 0, 0, 704, 0, 0, 0, 0, 0, 0, 0, 711, 0, 0, 0, 0, 0, 0, 0, 742, 0, 0, 0, 0, 742, 0, 748, 0, 0, 0, 736, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 747, 0, 0, 0, 0, 0, 792, 663, 841, 0, 0, 0, 0, 541, 845, 541, 541, 541, 541, 541, 2217, 541, 541, 541, 541, 541, 541, 541, 541, 541, 541, 541, 1802, 541, 541, 541, 541, 0, 0, 0, 0, 645, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 172032, 0, 0, 0, 0, 0, 0, 0, 0, 765, 0, 0, 768, 0, 0, 0, 0, 774, 0, 0, 778, 0, 0, 0, 0, 0, 0, 3562, 0, 3564, 541, 541, 541, 541, 541, 541, 541, 541, 541, 3219, 541, 541, 541, 541, 541, 3224, 0, 0, 785, 0, 0, 0, 0, 789, 0, 0, 0, 0, 0, 0, 0, 793, 0, 0, 736, 0, 793, 0, 0, 0, 0, 648, 0, 0, 0, 0, 0, 0, 0, 807, 808, 0, 0, 0, 0, 808, 0, 0, 807, 0, 0, 0, 0, 0, 755, 0, 0, 816, 817, 0, 0, 0, 793, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 221184, 0, 0, 0, 0, 0, 0, 648, 0, 0, 0, 0, 0, 834, 789, 0, 0, 0, 0, 0, 0, 0, 2139, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1712, 0, 0, 0, 0, 0, 0, 838, 839, 789, 789, 0, 0, 0, 0, 789, 736, 789, 0, 541, 541, 849, 853, 563, 964, 563, 967, 563, 563, 981, 563, 563, 0, 587, 587, 587, 996, 1000, 587, 0, 0, 0, 0, 0, 0, 0, 0, 3180, 3448, 0, 0, 0, 0, 0, 0, 0, 2465, 0, 0, 0, 0, 0, 0, 0, 0, 0, 2152, 0, 0, 0, 0, 0, 0, 1093, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1135, 0, 0, 0, 0, 1152, 0, 0, 0, 0, 1157, 0, 0, 0, 0, 0, 0, 0, 822, 0, 816, 0, 664, 0, 0, 0, 0, 0, 0, 0, 0, 1201, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 221184, 0, 221184, 0, 0, 0, 1253, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 339, 340, 341, 541, 541, 1342, 541, 541, 541, 541, 541, 541, 541, 541, 541, 541, 541, 541, 541, 541, 2239, 0, 0, 0, 0, 0, 1596, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 233472, 0, 0, 0, 0, 0, 0, 1640, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 354, 355, 356, 357, 1653, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1163, 0, 0, 0, 0, 1669, 0, 0, 0, 0, 0, 0, 1673, 0, 0, 0, 0, 0, 0, 0, 69632, 73728, 0, 0, 0, 346, 345, 65536, 344, 0, 1729, 0, 0, 0, 0, 541, 541, 541, 541, 541, 541, 541, 541, 541, 541, 541, 3327, 541, 541, 1742, 541, 541, 541, 541, 541, 541, 541, 541, 541, 1755, 541, 541, 541, 541, 541, 541, 541, 2193, 541, 541, 541, 541, 541, 541, 541, 541, 541, 2557, 541, 541, 541, 541, 541, 541, 541, 541, 541, 1794, 541, 541, 1797, 541, 541, 541, 541, 541, 541, 541, 541, 541, 541, 1314, 541, 541, 541, 1318, 541, 541, 1809, 541, 541, 541, 541, 541, 1814, 541, 541, 541, 541, 541, 541, 541, 1360, 914, 563, 563, 563, 563, 563, 563, 563, 563, 563, 563, 563, 563, 563, 563, 0, 2328, 1824, 563, 563, 563, 563, 563, 1831, 563, 563, 563, 563, 563, 1837, 563, 563, 563, 563, 563, 3093, 563, 563, 563, 563, 563, 563, 563, 563, 563, 3102, 563, 563, 1892, 563, 563, 563, 563, 563, 563, 563, 563, 563, 563, 1905, 563, 563, 563, 563, 978, 563, 563, 563, 563, 0, 587, 587, 587, 587, 587, 587, 1997, 587, 587, 587, 587, 587, 587, 587, 587, 587, 1531, 587, 587, 587, 587, 587, 587, 563, 563, 563, 1910, 563, 563, 563, 563, 563, 563, 563, 26028, 1920, 587, 587, 587, 0, 1285, 1469, 1377, 541, 541, 1339, 541, 541, 563, 563, 1431, 563, 587, 587, 1927, 587, 587, 587, 587, 587, 1933, 587, 587, 587, 587, 587, 587, 587, 1529, 587, 587, 587, 587, 587, 587, 587, 587, 1932, 587, 587, 587, 587, 587, 587, 587, 587, 1494, 587, 587, 587, 587, 587, 587, 587, 587, 587, 1946, 587, 587, 587, 587, 587, 587, 587, 587, 587, 587, 587, 587, 587, 1513, 587, 587, 563, 563, 2026, 587, 587, 587, 0, 2030, 0, 0, 0, 0, 0, 2032, 0, 0, 0, 0, 0, 0, 131072, 0, 131072, 131072, 131072, 131072, 0, 131072, 131072, 131072, 131072, 131072, 131072, 131072, 0, 0, 0, 0, 0, 131072, 0, 131072, 1, 0, 0, 0, 2042, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 365, 365, 365, 0, 0, 2141, 0, 0, 2148, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 237568, 0, 0, 0, 0, 0, 0, 2140, 541, 541, 541, 541, 541, 541, 541, 541, 541, 541, 541, 2170, 541, 541, 2173, 541, 541, 541, 541, 541, 541, 2181, 541, 541, 541, 541, 541, 541, 541, 541, 2841, 541, 541, 541, 541, 541, 541, 541, 541, 1294, 541, 541, 541, 541, 541, 541, 541, 541, 1327, 541, 541, 541, 1334, 1336, 541, 541, 541, 541, 2214, 541, 541, 541, 541, 541, 541, 541, 541, 541, 541, 541, 541, 541, 541, 2535, 541, 563, 2258, 563, 563, 563, 563, 563, 563, 2266, 563, 563, 563, 563, 563, 563, 563, 983, 563, 0, 587, 587, 587, 587, 1001, 587, 563, 2300, 563, 563, 563, 563, 563, 563, 563, 563, 563, 563, 563, 563, 563, 563, 587, 587, 587, 2345, 587, 587, 587, 587, 587, 587, 2353, 587, 587, 587, 587, 587, 587, 587, 1950, 587, 587, 587, 587, 587, 587, 587, 587, 1468, 587, 587, 587, 587, 587, 587, 587, 587, 2387, 587, 587, 587, 587, 587, 587, 587, 587, 587, 587, 587, 587, 587, 587, 541, 587, 563, 2414, 541, 541, 541, 541, 563, 563, 563, 563, 587, 587, 587, 587, 2030, 0, 2032, 0, 2034, 0, 2036, 0, 0, 2428, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1713, 0, 0, 0, 0, 0, 2436, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1179, 2449, 0, 0, 0, 0, 0, 2453, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1726, 0, 0, 0, 0, 0, 0, 0, 0, 0, 2477, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 245760, 0, 0, 0, 0, 0, 2501, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 386, 338, 0, 541, 541, 541, 2539, 541, 541, 541, 541, 541, 541, 541, 541, 541, 541, 541, 541, 541, 2198, 541, 541, 0, 0, 0, 563, 563, 563, 563, 563, 563, 563, 563, 563, 2595, 563, 563, 563, 563, 563, 3238, 563, 563, 563, 563, 563, 563, 563, 563, 563, 563, 2880, 563, 563, 563, 563, 563, 587, 587, 587, 2661, 587, 587, 587, 587, 587, 587, 587, 587, 587, 587, 587, 2669, 587, 587, 587, 587, 2714, 587, 587, 587, 587, 587, 587, 541, 587, 563, 541, 541, 2417, 2418, 563, 563, 2421, 2422, 587, 587, 2425, 2426, 0, 1563, 0, 0, 0, 2735, 0, 0, 0, 0, 0, 0, 2740, 0, 0, 0, 0, 0, 0, 0, 1102, 1101, 0, 0, 0, 0, 0, 0, 0, 0, 1724, 0, 0, 0, 0, 0, 0, 0, 0, 1172, 0, 0, 0, 0, 0, 0, 0, 0, 1187, 0, 0, 0, 0, 0, 0, 1104, 0, 0, 0, 0, 0, 2763, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 2431, 2432, 0, 0, 0, 0, 0, 0, 0, 0, 2792, 2793, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 2049, 0, 0, 0, 0, 0, 541, 541, 541, 541, 2829, 541, 541, 541, 541, 541, 541, 541, 541, 541, 541, 2836, 563, 563, 563, 563, 2876, 563, 563, 563, 563, 563, 563, 563, 563, 563, 563, 563, 0, 2329, 587, 587, 587, 2884, 563, 563, 563, 563, 563, 563, 563, 563, 563, 563, 563, 563, 563, 563, 563, 1874, 587, 587, 2933, 587, 587, 587, 587, 587, 587, 587, 587, 587, 587, 587, 587, 587, 1536, 587, 587, 2955, 541, 2957, 563, 2959, 587, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 541, 541, 541, 541, 541, 541, 541, 587, 3442, 0, 3444, 0, 0, 0, 0, 0, 3180, 0, 0, 0, 0, 3452, 0, 0, 0, 0, 0, 796, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 796, 0, 0, 0, 0, 3454, 0, 541, 3457, 541, 3459, 541, 541, 541, 541, 541, 541, 541, 541, 541, 541, 541, 541, 2585, 0, 0, 0, 563, 587, 3488, 587, 3490, 587, 587, 587, 587, 587, 587, 587, 587, 587, 587, 587, 1047, 587, 587, 587, 587, 587, 0, 0, 0, 3505, 0, 0, 0, 0, 0, 0, 0, 3512, 0, 0, 541, 541, 541, 541, 541, 541, 3462, 541, 541, 541, 541, 541, 541, 541, 541, 541, 3334, 541, 541, 541, 563, 563, 563, 541, 541, 541, 3517, 541, 541, 541, 541, 541, 541, 541, 541, 541, 541, 563, 563, 563, 563, 563, 563, 563, 563, 563, 563, 3530, 563, 563, 563, 563, 563, 563, 563, 563, 563, 563, 563, 587, 587, 587, 587, 587, 587, 587, 587, 3544, 587, 587, 587, 587, 587, 587, 587, 587, 587, 587, 587, 0, 0, 0, 0, 541, 541, 541, 541, 563, 563, 310, 311, 312, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 669, 0, 0, 0, 0, 0, 421, 0, 0, 0, 0, 450, 0, 0, 0, 0, 0, 0, 0, 0, 304, 304, 304, 304, 0, 304, 304, 304, 0, 0, 0, 0, 450, 450, 421, 450, 450, 450, 450, 450, 450, 450, 450, 450, 450, 450, 450, 450, 450, 533, 450, 533, 533, 533, 450, 533, 533, 533, 533, 450, 543, 566, 543, 566, 543, 543, 566, 543, 590, 566, 566, 566, 566, 566, 566, 566, 590, 590, 590, 543, 590, 590, 590, 590, 590, 590, 590, 590, 566, 566, 616, 621, 590, 621, 627, 1, 960, 563, 563, 563, 563, 563, 563, 563, 563, 0, 587, 587, 587, 587, 587, 587, 587, 3136, 587, 587, 3138, 587, 587, 587, 587, 587, 0, 0, 0, 0, 1657, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 254414, 0, 0, 0, 1676, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1211, 541, 541, 541, 1810, 541, 541, 541, 541, 541, 541, 541, 541, 541, 541, 541, 1360, 914, 563, 563, 563, 563, 563, 563, 563, 563, 563, 563, 563, 563, 563, 1378, 0, 0, 2486, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 694, 0, 0, 541, 541, 541, 541, 2528, 541, 541, 541, 541, 2532, 541, 541, 541, 541, 541, 541, 541, 541, 3218, 541, 541, 541, 541, 541, 541, 541, 541, 2583, 541, 541, 541, 541, 0, 2586, 0, 0, 0, 0, 563, 563, 563, 563, 563, 563, 2592, 563, 563, 563, 563, 2596, 563, 0, 587, 587, 587, 587, 587, 78115, 1079, 0, 0, 0, 0, 0, 0, 0, 0, 0, 225741, 225741, 225741, 225741, 225741, 225741, 225741, 225741, 225741, 225741, 225741, 225741, 225741, 225741, 225741, 225741, 0, 0, 0, 0, 0, 0, 0, 2658, 587, 587, 587, 587, 2662, 587, 587, 587, 587, 587, 587, 587, 587, 587, 587, 2665, 587, 587, 587, 587, 587, 0, 0, 0, 313, 314, 315, 316, 317, 318, 319, 320, 321, 322, 0, 0, 0, 0, 0, 0, 694, 0, 0, 0, 0, 0, 365, 365, 365, 0, 0, 313, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 2142208, 0, 0, 316, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 730, 0, 0, 0, 423, 431, 433, 422, 431, 0, 313, 431, 451, 464, 464, 464, 464, 464, 464, 464, 464, 464, 464, 464, 464, 464, 464, 464, 464, 464, 464, 464, 485, 490, 490, 501, 490, 490, 490, 490, 490, 490, 490, 490, 516, 516, 529, 529, 530, 530, 530, 530, 530, 530, 530, 530, 530, 530, 530, 516, 530, 530, 530, 530, 530, 544, 567, 544, 567, 544, 544, 567, 544, 591, 567, 567, 567, 567, 567, 567, 567, 591, 591, 591, 613, 591, 591, 591, 591, 591, 591, 591, 614, 615, 615, 613, 615, 614, 615, 614, 1, 734, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1238, 0, 0, 0, 0, 767, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 266240, 0, 0, 0, 0, 0, 702, 0, 0, 0, 0, 0, 702, 0, 0, 0, 541, 541, 541, 541, 541, 541, 3322, 541, 541, 541, 541, 541, 541, 541, 541, 1816, 541, 541, 541, 541, 541, 541, 1360, 563, 563, 563, 968, 563, 563, 563, 563, 563, 0, 587, 587, 587, 587, 587, 587, 587, 3150, 3151, 3152, 541, 541, 563, 563, 587, 587, 0, 0, 2963, 0, 0, 0, 0, 0, 0, 0, 0, 3180, 0, 0, 3389, 0, 0, 0, 0, 0, 1108, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 308, 309, 0, 1150, 1108, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1119, 0, 0, 1212, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1225, 0, 0, 0, 0, 0, 797, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 797, 0, 1360, 914, 563, 563, 1364, 563, 563, 1368, 563, 563, 563, 563, 563, 563, 563, 563, 985, 0, 587, 587, 587, 587, 587, 587, 1382, 563, 1387, 563, 563, 1391, 563, 563, 1394, 563, 563, 563, 563, 563, 563, 563, 984, 563, 0, 587, 587, 587, 587, 587, 1004, 587, 1460, 587, 587, 587, 587, 587, 587, 587, 587, 1474, 587, 1479, 587, 587, 1483, 587, 587, 1486, 587, 587, 587, 587, 587, 587, 587, 587, 587, 587, 587, 587, 587, 1939, 587, 587, 0, 1692, 0, 1694, 0, 0, 0, 0, 0, 0, 1701, 0, 0, 0, 0, 0, 0, 0, 2442, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1173, 0, 0, 0, 0, 0, 0, 541, 541, 1745, 541, 541, 541, 541, 541, 541, 541, 541, 541, 1757, 541, 541, 541, 541, 541, 541, 2192, 541, 541, 2195, 541, 541, 541, 541, 541, 541, 541, 1798, 541, 541, 541, 541, 541, 541, 1806, 541, 541, 1792, 1793, 541, 541, 541, 541, 541, 541, 541, 541, 541, 541, 541, 541, 541, 541, 2824, 541, 0, 563, 563, 563, 563, 563, 563, 563, 563, 563, 563, 563, 563, 563, 1840, 563, 0, 587, 587, 587, 587, 587, 78115, 1079, 0, 0, 1083, 1087, 0, 0, 1091, 587, 587, 587, 587, 1948, 587, 587, 587, 587, 587, 587, 587, 587, 587, 587, 587, 1478, 587, 587, 587, 587, 587, 587, 587, 587, 1961, 587, 587, 587, 587, 587, 587, 587, 587, 587, 587, 587, 1533, 587, 587, 587, 587, 0, 2034, 0, 2036, 0, 0, 0, 0, 0, 0, 2430, 0, 0, 0, 0, 0, 0, 0, 2454, 0, 0, 0, 0, 0, 0, 0, 0, 0, 2429, 0, 0, 0, 0, 0, 0, 0, 0, 0, 2476, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 365, 365, 365, 702, 0, 0, 0, 2502, 2503, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1147355, 0, 0, 0, 587, 587, 2700, 587, 587, 587, 587, 587, 587, 587, 587, 587, 587, 587, 587, 587, 2004, 587, 587, 0, 0, 2802, 0, 0, 0, 0, 0, 0, 0, 2806, 541, 541, 541, 541, 541, 541, 541, 2204, 541, 541, 541, 541, 541, 541, 541, 541, 541, 2220, 541, 541, 541, 541, 541, 541, 541, 2956, 563, 2958, 587, 2960, 0, 0, 0, 0, 0, 2966, 0, 0, 0, 0, 0, 0, 0, 69632, 73728, 0, 0, 0, 350, 348, 65536, 0, 0, 0, 0, 2970, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 365, 365, 208896, 0, 3035, 541, 541, 541, 541, 541, 541, 541, 541, 541, 541, 541, 541, 541, 541, 541, 1807, 541, 541, 3050, 3051, 541, 541, 541, 541, 541, 541, 541, 541, 541, 541, 541, 541, 541, 2223, 2224, 541, 563, 3090, 563, 3092, 563, 563, 563, 563, 563, 563, 563, 563, 563, 563, 563, 563, 1415, 563, 563, 563, 3117, 587, 587, 587, 587, 587, 587, 587, 587, 587, 587, 587, 587, 587, 587, 587, 1484, 587, 587, 3132, 587, 3134, 587, 587, 587, 587, 587, 587, 587, 587, 587, 587, 587, 1937, 1938, 587, 587, 587, 0, 3192, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 309, 308, 541, 541, 541, 541, 3228, 541, 541, 541, 541, 563, 563, 563, 563, 563, 563, 563, 563, 563, 945, 563, 563, 563, 563, 587, 3503, 0, 3504, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 541, 541, 541, 541, 2810, 3515, 541, 3516, 541, 541, 541, 3520, 541, 541, 541, 541, 541, 541, 541, 563, 3528, 563, 3529, 563, 563, 563, 3533, 563, 563, 563, 563, 563, 563, 563, 563, 587, 3542, 587, 3543, 587, 587, 587, 3547, 587, 587, 587, 587, 587, 587, 587, 587, 0, 0, 0, 0, 3673, 541, 541, 541, 3675, 563, 563, 3676, 587, 587, 587, 3678, 0, 0, 541, 541, 563, 563, 587, 587, 0, 541, 541, 3204, 541, 541, 541, 541, 541, 541, 541, 541, 541, 541, 541, 541, 541, 2238, 2586, 0, 324, 324, 374, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1133, 0, 0, 0, 0, 434, 374, 0, 439, 445, 0, 452, 465, 465, 465, 465, 465, 465, 465, 465, 465, 465, 465, 465, 465, 465, 465, 465, 465, 465, 465, 465, 491, 491, 502, 491, 491, 491, 491, 491, 491, 491, 491, 517, 517, 517, 517, 517, 517, 517, 517, 517, 517, 517, 517, 517, 517, 517, 517, 545, 568, 545, 568, 545, 545, 568, 545, 592, 568, 568, 568, 568, 568, 568, 568, 592, 592, 592, 545, 592, 592, 592, 592, 592, 592, 592, 592, 568, 568, 545, 568, 592, 568, 592, 1, 587, 587, 587, 1019, 587, 587, 587, 587, 587, 587, 587, 587, 587, 587, 587, 587, 1052, 587, 587, 587, 1122, 0, 1124, 1125, 0, 0, 0, 1127, 1128, 0, 0, 0, 0, 0, 0, 0, 0, 334, 0, 0, 0, 0, 0, 334, 0, 0, 0, 1166, 1167, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 400, 0, 0, 0, 0, 0, 1228, 0, 0, 0, 0, 1233, 0, 0, 0, 0, 0, 0, 0, 0, 0, 365, 300, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1241, 0, 0, 0, 0, 1244, 0, 1194, 0, 1113, 0, 1250, 1127, 0, 0, 0, 0, 0, 0, 0, 0, 0, 541, 541, 1274, 541, 541, 541, 541, 541, 541, 2203, 541, 541, 541, 541, 541, 541, 541, 541, 541, 541, 1329, 541, 541, 541, 541, 541, 541, 541, 1322, 541, 541, 1324, 541, 541, 541, 1328, 541, 541, 541, 541, 541, 541, 541, 541, 3522, 541, 541, 541, 541, 541, 563, 563, 3068, 563, 563, 563, 563, 563, 563, 563, 563, 2603, 563, 563, 563, 563, 563, 563, 563, 1360, 914, 563, 563, 1365, 563, 563, 563, 563, 563, 563, 563, 563, 563, 563, 563, 587, 3427, 587, 587, 3429, 563, 563, 563, 1405, 563, 563, 563, 563, 563, 563, 563, 1414, 563, 563, 1416, 563, 0, 587, 587, 587, 1051, 587, 78115, 1079, 0, 0, 0, 0, 0, 0, 0, 0, 1134592, 0, 0, 0, 0, 0, 0, 1134592, 0, 0, 0, 0, 563, 563, 1420, 563, 563, 563, 563, 563, 563, 563, 563, 563, 563, 563, 563, 563, 1437, 563, 563, 563, 563, 563, 1444, 563, 563, 563, 563, 563, 26028, 1360, 988, 587, 587, 1457, 587, 0, 0, 0, 0, 0, 0, 0, 3447, 3180, 0, 0, 0, 0, 0, 0, 0, 1237, 0, 0, 541, 541, 1273, 541, 1276, 541, 0, 0, 0, 1656, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 541, 846, 541, 541, 587, 587, 587, 1993, 587, 587, 587, 587, 587, 587, 587, 587, 587, 587, 587, 587, 1496, 587, 587, 587, 563, 2025, 587, 587, 587, 2029, 0, 2030, 0, 0, 0, 0, 0, 2032, 0, 0, 0, 0, 0, 0, 1134592, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1134592, 0, 0, 0, 2034, 0, 0, 0, 0, 0, 2036, 0, 0, 0, 0, 0, 2039, 0, 2078, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 331, 0, 0, 0, 0, 2092, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 643, 0, 0, 0, 541, 2172, 541, 541, 541, 541, 541, 541, 541, 541, 541, 541, 541, 541, 541, 541, 0, 0, 0, 541, 2188, 541, 541, 541, 541, 541, 541, 541, 541, 541, 541, 541, 541, 541, 541, 0, 2240, 541, 541, 541, 541, 2229, 541, 2231, 541, 541, 541, 541, 541, 541, 541, 0, 0, 0, 0, 0, 0, 1146880, 0, 1146880, 0, 0, 0, 0, 0, 0, 0, 0, 2617344, 0, 0, 0, 0, 0, 2789376, 0, 0, 0, 563, 563, 563, 563, 563, 563, 563, 563, 563, 2253, 563, 563, 563, 563, 0, 0, 0, 0, 0, 0, 587, 2654, 587, 587, 587, 587, 587, 587, 3135, 587, 587, 587, 587, 587, 587, 587, 587, 587, 1530, 587, 587, 587, 587, 587, 587, 2257, 563, 563, 563, 563, 563, 563, 563, 563, 563, 563, 563, 563, 563, 563, 563, 1889, 2273, 563, 563, 563, 563, 563, 563, 563, 563, 563, 563, 563, 563, 563, 563, 563, 2272, 563, 563, 563, 563, 2316, 563, 2318, 563, 563, 563, 563, 563, 563, 563, 0, 0, 0, 0, 0, 0, 587, 587, 587, 587, 587, 587, 587, 587, 587, 587, 587, 587, 587, 587, 541, 563, 2344, 587, 587, 587, 587, 587, 587, 587, 587, 587, 587, 587, 587, 587, 587, 587, 1516, 2360, 587, 587, 587, 587, 587, 587, 587, 587, 587, 587, 587, 587, 587, 587, 587, 1517, 587, 587, 587, 587, 2403, 587, 2405, 587, 587, 587, 587, 587, 587, 587, 541, 587, 0, 0, 0, 0, 0, 0, 0, 3508, 0, 0, 0, 0, 0, 0, 541, 541, 541, 541, 541, 541, 3400, 541, 541, 541, 2499, 0, 0, 0, 0, 0, 2504, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 2086, 2087, 0, 0, 0, 0, 541, 541, 2526, 2527, 541, 541, 541, 541, 541, 541, 541, 541, 541, 541, 541, 541, 541, 2237, 0, 0, 541, 541, 2566, 541, 541, 541, 541, 541, 541, 541, 541, 541, 541, 541, 541, 541, 541, 3046, 541, 0, 0, 0, 563, 563, 563, 2590, 563, 2591, 563, 563, 563, 563, 563, 563, 563, 1411, 563, 563, 563, 563, 563, 563, 563, 563, 1429, 563, 563, 563, 563, 563, 563, 563, 563, 1916, 563, 563, 26028, 1921, 587, 587, 587, 0, 0, 0, 2761, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 674, 0, 0, 0, 541, 541, 2837, 541, 541, 541, 541, 541, 541, 541, 2843, 541, 541, 541, 541, 541, 541, 541, 2232, 541, 541, 2235, 2236, 541, 541, 0, 0, 563, 563, 563, 2885, 563, 563, 563, 563, 563, 563, 563, 2891, 563, 563, 563, 563, 0, 0, 2329, 0, 0, 0, 587, 587, 587, 587, 587, 587, 587, 587, 587, 587, 2341, 587, 587, 587, 2920, 587, 587, 587, 587, 587, 587, 587, 587, 587, 587, 587, 587, 587, 587, 587, 1519, 587, 587, 587, 587, 3373, 587, 587, 541, 541, 563, 563, 587, 587, 0, 0, 0, 0, 0, 0, 0, 0, 0, 2968, 563, 587, 587, 587, 587, 587, 587, 3493, 587, 587, 587, 587, 587, 587, 587, 587, 1034, 587, 587, 587, 587, 587, 587, 587, 326, 327, 328, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1161, 0, 0, 0, 0, 0, 325, 373, 328, 372, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 2111, 0, 0, 0, 0, 0, 325, 0, 0, 372, 372, 402, 0, 328, 0, 0, 0, 0, 0, 0, 0, 0, 0, 365, 339, 293, 0, 0, 0, 0, 0, 0, 325, 0, 327, 0, 0, 0, 453, 466, 466, 466, 466, 466, 466, 466, 479, 466, 466, 466, 466, 466, 466, 466, 466, 466, 466, 466, 466, 492, 492, 466, 492, 492, 507, 509, 492, 492, 507, 492, 518, 518, 518, 518, 518, 518, 518, 518, 518, 518, 518, 518, 518, 518, 518, 535, 518, 518, 518, 518, 518, 546, 569, 546, 569, 546, 546, 569, 546, 593, 569, 569, 569, 569, 569, 569, 569, 593, 593, 593, 546, 593, 593, 593, 593, 593, 593, 593, 593, 569, 569, 546, 569, 593, 569, 593, 1, 0, 0, 658, 659, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 670, 671, 0, 689, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 365, 365, 365, 0, 0, 0, 0, 0, 800, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 718, 0, 720, 0, 0, 0, 0, 0, 0, 727, 0, 0, 0, 731, 0, 0, 0, 0, 0, 0, 1159168, 417, 417, 0, 0, 0, 0, 0, 417, 0, 0, 784, 0, 786, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 759, 0, 0, 0, 0, 0, 798, 0, 803, 0, 806, 0, 0, 0, 0, 803, 806, 0, 0, 0, 0, 0, 0, 741, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 806, 806, 803, 0, 0, 0, 0, 0, 0, 0, 786, 0, 798, 0, 815, 0, 0, 0, 0, 0, 802, 0, 0, 783, 0, 0, 0, 0, 802, 0, 0, 0, 0, 0, 0, 0, 802, 0, 0, 0, 0, 806, 0, 707, 0, 0, 823, 0, 0, 0, 0, 0, 823, 823, 826, 0, 0, 0, 786, 0, 0, 0, 0, 0, 835, 0, 0, 0, 0, 0, 0, 0, 2480, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1100, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 784, 0, 0, 0, 835, 815, 835, 0, 541, 541, 850, 541, 541, 541, 541, 541, 3052, 541, 541, 541, 541, 541, 541, 541, 541, 3060, 541, 541, 541, 541, 541, 3217, 541, 541, 541, 541, 541, 541, 541, 3222, 541, 541, 541, 541, 541, 2529, 541, 541, 541, 541, 541, 541, 541, 541, 541, 541, 541, 3043, 541, 541, 541, 541, 856, 541, 541, 869, 541, 541, 880, 541, 885, 541, 541, 893, 896, 901, 541, 909, 563, 563, 966, 969, 974, 563, 982, 563, 563, 0, 587, 587, 587, 997, 587, 1003, 587, 587, 1016, 587, 587, 1027, 587, 1032, 587, 587, 1040, 1043, 1048, 587, 1056, 587, 0, 0, 0, 0, 0, 0, 3507, 0, 0, 0, 0, 0, 0, 0, 541, 541, 541, 541, 541, 3031, 541, 541, 541, 541, 0, 0, 0, 1110, 0, 0, 0, 0, 0, 0, 0, 1117, 0, 0, 0, 0, 0, 0, 0, 69632, 73728, 0, 0, 0, 421, 0, 65536, 0, 0, 1137, 1138, 0, 0, 0, 0, 1142, 0, 0, 0, 365, 365, 0, 0, 0, 0, 0, 0, 769, 0, 0, 0, 775, 776, 0, 0, 0, 0, 0, 0, 0, 69632, 73728, 0, 0, 0, 374, 0, 65536, 0, 0, 1165, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 515, 522, 0, 0, 1182, 741, 0, 0, 0, 1134, 0, 0, 0, 0, 0, 0, 0, 0, 0, 367, 0, 0, 0, 0, 0, 0, 541, 541, 541, 1281, 541, 541, 541, 541, 541, 541, 1296, 541, 541, 541, 1301, 541, 541, 541, 541, 541, 3229, 541, 541, 541, 563, 563, 563, 563, 563, 563, 563, 941, 563, 563, 563, 563, 563, 563, 1360, 914, 563, 563, 563, 563, 563, 563, 563, 563, 563, 563, 1373, 563, 563, 563, 563, 563, 3249, 563, 3251, 563, 563, 563, 563, 563, 563, 563, 563, 1852, 563, 563, 563, 563, 563, 563, 563, 563, 563, 563, 1388, 563, 563, 563, 1393, 563, 563, 563, 563, 563, 563, 563, 563, 1451, 26028, 1360, 988, 587, 587, 587, 587, 587, 1485, 587, 587, 587, 587, 587, 587, 587, 587, 587, 587, 587, 587, 587, 1500, 1540, 587, 587, 0, 541, 587, 563, 541, 541, 541, 541, 541, 563, 563, 563, 563, 563, 563, 3477, 563, 563, 563, 563, 563, 563, 563, 563, 1396, 563, 563, 563, 563, 563, 563, 563, 0, 0, 1607, 1608, 1609, 0, 1611, 1612, 0, 0, 0, 0, 1617, 0, 0, 0, 0, 0, 0, 770, 0, 0, 773, 0, 0, 777, 0, 0, 0, 0, 0, 0, 796, 0, 0, 0, 0, 0, 541, 541, 541, 541, 541, 541, 563, 563, 563, 563, 563, 563, 933, 563, 563, 947, 563, 563, 563, 563, 0, 0, 1639, 0, 0, 1642, 0, 1644, 0, 0, 0, 0, 0, 1650, 1651, 0, 0, 0, 0, 0, 805, 0, 0, 692, 0, 0, 672, 0, 692, 0, 810, 0, 1667, 0, 0, 1669, 0, 0, 0, 0, 1671, 1672, 0, 0, 0, 0, 0, 0, 0, 2491, 2492, 0, 0, 0, 0, 0, 0, 0, 0, 0, 2686976, 2736128, 0, 0, 2531328, 2707456, 0, 0, 0, 0, 0, 1708, 0, 0, 1711, 0, 0, 0, 0, 0, 1715, 0, 0, 0, 0, 0, 0, 2134016, 0, 0, 0, 0, 0, 0, 0, 0, 1138688, 0, 1719, 1720, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1711, 0, 0, 0, 1731, 1585, 1585, 1733, 541, 1735, 541, 1736, 1737, 541, 1739, 541, 541, 541, 541, 541, 1310, 541, 541, 541, 541, 541, 541, 541, 1317, 541, 541, 541, 541, 541, 1748, 541, 541, 541, 541, 541, 541, 541, 541, 541, 1759, 541, 1743, 541, 541, 541, 541, 541, 541, 541, 541, 541, 1756, 541, 541, 541, 541, 541, 541, 541, 3333, 541, 541, 541, 541, 541, 563, 563, 563, 922, 926, 563, 563, 563, 563, 563, 563, 952, 563, 957, 1760, 541, 541, 541, 541, 1764, 541, 1766, 541, 541, 541, 541, 541, 541, 541, 541, 541, 889, 541, 541, 541, 541, 907, 541, 1774, 1775, 1777, 541, 541, 541, 541, 541, 541, 541, 1785, 1786, 1787, 541, 541, 1790, 1791, 541, 541, 541, 541, 541, 541, 541, 1799, 541, 541, 541, 1803, 541, 541, 541, 541, 541, 541, 2530, 541, 541, 541, 541, 541, 541, 541, 541, 541, 541, 1315, 541, 541, 541, 541, 1320, 541, 541, 541, 541, 1811, 541, 541, 541, 541, 541, 541, 541, 541, 541, 1822, 1360, 914, 563, 563, 563, 563, 563, 563, 563, 563, 563, 563, 563, 563, 563, 1379, 0, 1827, 563, 1829, 563, 1830, 563, 1832, 563, 1834, 563, 563, 563, 1838, 563, 563, 563, 563, 1390, 563, 563, 563, 563, 563, 563, 563, 563, 563, 563, 563, 2652, 0, 587, 587, 587, 563, 1859, 563, 1861, 563, 563, 563, 563, 563, 563, 563, 563, 1869, 1870, 1872, 563, 0, 587, 587, 1042, 587, 587, 78115, 1079, 0, 0, 0, 0, 0, 0, 0, 0, 307, 307, 307, 0, 0, 0, 0, 0, 1907, 563, 563, 563, 563, 563, 563, 563, 563, 563, 1918, 26028, 0, 1923, 587, 1925, 587, 1926, 587, 1928, 587, 1930, 587, 587, 587, 1934, 587, 587, 587, 587, 587, 587, 587, 3366, 587, 587, 587, 587, 3367, 3368, 587, 587, 587, 587, 587, 1947, 587, 587, 587, 587, 1951, 587, 587, 587, 587, 1955, 587, 1957, 587, 587, 1976, 1977, 1978, 587, 587, 1981, 1982, 587, 587, 587, 587, 587, 587, 587, 1999, 2000, 587, 587, 587, 587, 587, 587, 587, 2015, 2016, 2017, 541, 2019, 541, 541, 563, 2023, 1990, 587, 587, 587, 1994, 587, 587, 587, 587, 587, 587, 587, 2003, 587, 587, 587, 0, 1288, 1472, 1380, 541, 541, 541, 541, 541, 563, 563, 563, 563, 3475, 563, 563, 563, 563, 563, 563, 563, 563, 563, 563, 2267, 563, 563, 563, 563, 563, 563, 2024, 563, 1982, 587, 2028, 587, 0, 2030, 0, 0, 0, 0, 0, 2032, 0, 0, 0, 0, 0, 0, 2134016, 0, 0, 0, 0, 0, 0, 0, 746, 0, 0, 0, 0, 0, 2043, 0, 0, 2046, 2047, 0, 0, 0, 2051, 0, 0, 0, 0, 0, 0, 800, 0, 0, 0, 0, 0, 0, 0, 800, 0, 0, 0, 0, 541, 541, 541, 541, 2090, 0, 0, 0, 0, 2094, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 2456, 0, 0, 0, 0, 0, 0, 0, 0, 1669, 0, 0, 0, 0, 0, 0, 0, 2113, 0, 0, 2116, 0, 0, 2119, 0, 0, 0, 0, 0, 0, 2126, 0, 2128, 0, 0, 0, 0, 0, 0, 0, 69632, 73728, 0, 370, 370, 0, 0, 65536, 370, 0, 0, 0, 0, 2137, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1159168, 0, 0, 0, 0, 2047, 0, 2147, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1103, 1104, 1105, 1106, 0, 2158, 0, 0, 541, 541, 541, 541, 541, 541, 541, 541, 541, 541, 541, 541, 541, 3329, 541, 541, 541, 2174, 541, 2176, 541, 541, 541, 541, 541, 541, 2184, 541, 2186, 541, 541, 541, 541, 541, 3332, 541, 541, 541, 541, 541, 541, 541, 563, 563, 563, 3069, 563, 563, 563, 563, 563, 563, 563, 2212, 541, 541, 541, 541, 541, 541, 2219, 541, 541, 541, 541, 541, 541, 541, 541, 541, 1312, 541, 541, 541, 541, 541, 541, 563, 563, 2259, 563, 2261, 563, 563, 563, 563, 563, 563, 2269, 563, 2271, 563, 563, 563, 563, 1422, 563, 563, 563, 563, 563, 563, 563, 563, 563, 563, 1438, 587, 587, 2346, 587, 2348, 587, 587, 587, 587, 587, 587, 2356, 587, 2358, 587, 587, 0, 0, 0, 3634, 0, 3636, 541, 541, 541, 541, 541, 541, 541, 563, 563, 921, 563, 563, 563, 563, 940, 944, 950, 563, 955, 563, 563, 2261, 541, 2416, 541, 541, 563, 2420, 563, 563, 587, 2424, 587, 587, 2030, 0, 2032, 0, 0, 0, 2452, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1132, 0, 0, 0, 0, 0, 2461, 0, 0, 0, 0, 0, 2466, 0, 2468, 2469, 0, 0, 0, 0, 0, 0, 0, 69632, 73728, 0, 420, 420, 0, 0, 65536, 420, 0, 0, 0, 0, 2488, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 2179072, 2179072, 2179072, 2179072, 0, 0, 0, 0, 2516, 2468, 0, 0, 0, 0, 0, 2521, 0, 0, 0, 541, 541, 541, 541, 541, 3321, 541, 541, 541, 3325, 541, 541, 541, 541, 541, 541, 1749, 541, 541, 541, 541, 541, 541, 541, 541, 541, 541, 2558, 541, 541, 541, 541, 541, 2563, 541, 541, 541, 2568, 541, 541, 541, 541, 541, 541, 541, 2575, 541, 541, 541, 541, 541, 541, 2542, 541, 2544, 541, 541, 541, 541, 541, 541, 541, 541, 563, 3411, 563, 563, 3413, 563, 563, 563, 541, 541, 541, 541, 2580, 541, 541, 541, 541, 541, 541, 541, 541, 0, 2586, 0, 0, 0, 0, 0, 812, 0, 809, 792, 0, 0, 814, 0, 667, 0, 788, 0, 0, 0, 563, 563, 563, 563, 563, 563, 563, 563, 2594, 563, 563, 563, 563, 0, 2652, 0, 0, 0, 0, 587, 587, 587, 587, 587, 587, 587, 587, 587, 587, 587, 587, 2342, 587, 563, 2624, 563, 563, 563, 563, 2628, 563, 563, 563, 2633, 563, 563, 563, 563, 563, 563, 2629, 563, 563, 563, 563, 563, 563, 2636, 563, 563, 563, 563, 2640, 563, 563, 563, 563, 563, 563, 563, 563, 2646, 563, 563, 563, 563, 0, 2652, 0, 0, 0, 0, 587, 587, 587, 2656, 587, 2657, 587, 587, 2660, 587, 587, 587, 587, 587, 587, 587, 587, 587, 587, 587, 587, 587, 2383, 587, 587, 587, 587, 2672, 587, 2674, 587, 587, 587, 587, 587, 587, 587, 587, 587, 587, 587, 1970, 587, 587, 587, 587, 587, 2699, 587, 587, 587, 587, 587, 587, 587, 2706, 587, 587, 587, 587, 587, 587, 587, 3375, 541, 3376, 563, 3377, 587, 0, 0, 0, 587, 587, 2712, 587, 587, 587, 587, 587, 587, 587, 587, 541, 587, 563, 541, 541, 541, 541, 563, 563, 2800, 2801, 0, 0, 0, 0, 0, 0, 0, 0, 0, 541, 541, 541, 541, 541, 541, 541, 541, 541, 3402, 2992, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 2992, 0, 0, 0, 0, 0, 0, 840, 0, 0, 0, 0, 0, 541, 541, 541, 541, 541, 541, 2166, 541, 541, 541, 541, 541, 0, 3022, 0, 0, 0, 3025, 541, 541, 541, 541, 541, 541, 3032, 541, 541, 541, 541, 541, 541, 2570, 541, 541, 541, 541, 2574, 541, 541, 541, 541, 541, 541, 1286, 541, 541, 541, 541, 541, 541, 541, 541, 541, 541, 1818, 541, 541, 541, 541, 1360, 541, 541, 3036, 541, 541, 541, 541, 541, 541, 541, 3042, 541, 541, 541, 541, 541, 541, 541, 2531, 541, 541, 541, 541, 541, 541, 541, 541, 541, 563, 3230, 563, 563, 563, 3233, 563, 3048, 541, 541, 541, 541, 541, 541, 541, 541, 541, 541, 3057, 3059, 541, 541, 541, 541, 541, 541, 2582, 541, 541, 541, 541, 541, 541, 0, 2586, 0, 3062, 541, 3064, 3065, 541, 563, 563, 563, 563, 563, 563, 3072, 563, 563, 563, 563, 563, 1425, 563, 563, 563, 563, 563, 563, 563, 563, 563, 563, 2889, 563, 563, 563, 563, 563, 563, 563, 3076, 563, 563, 563, 563, 563, 563, 563, 3082, 563, 563, 563, 563, 563, 3088, 563, 563, 3091, 563, 563, 563, 563, 563, 563, 563, 563, 3098, 3100, 563, 563, 563, 563, 563, 3342, 563, 563, 563, 563, 563, 563, 563, 563, 563, 563, 2645, 563, 563, 563, 563, 563, 563, 563, 3104, 563, 3106, 3107, 563, 587, 587, 587, 587, 587, 587, 3114, 587, 587, 587, 0, 1544, 1545, 1546, 541, 541, 1548, 541, 541, 563, 563, 1552, 563, 587, 587, 3118, 587, 587, 587, 587, 587, 587, 587, 3124, 587, 587, 587, 587, 587, 587, 1998, 587, 587, 587, 587, 587, 587, 587, 587, 587, 3551, 587, 587, 3553, 587, 0, 0, 3130, 587, 587, 3133, 587, 587, 587, 587, 587, 587, 587, 587, 3140, 3142, 587, 587, 0, 0, 3633, 0, 0, 0, 541, 541, 3638, 541, 541, 541, 3642, 563, 587, 587, 3146, 587, 3148, 3149, 587, 541, 587, 563, 541, 3154, 563, 3156, 587, 3158, 0, 0, 0, 0, 0, 3181, 0, 3183, 0, 0, 0, 0, 0, 0, 0, 3190, 0, 0, 0, 3193, 3194, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 2200252, 2200252, 2200252, 0, 563, 563, 563, 563, 3237, 563, 563, 563, 563, 563, 563, 563, 563, 563, 563, 563, 1400, 563, 563, 563, 563, 563, 563, 563, 3248, 563, 563, 563, 563, 563, 563, 563, 563, 3256, 563, 563, 563, 563, 563, 3352, 563, 563, 563, 563, 587, 587, 587, 587, 587, 3358, 563, 563, 563, 587, 587, 587, 3261, 587, 587, 587, 587, 587, 587, 587, 3266, 587, 0, 0, 0, 0, 0, 3506, 0, 0, 3509, 0, 0, 0, 0, 0, 541, 541, 541, 541, 3662, 541, 563, 563, 563, 563, 3666, 563, 0, 3310, 0, 0, 3313, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 2200253, 151552, 2200253, 0, 587, 587, 587, 3372, 587, 587, 587, 541, 541, 563, 563, 587, 587, 0, 0, 0, 0, 0, 0, 0, 0, 2967, 0, 0, 0, 0, 0, 0, 3383, 3384, 0, 3180, 0, 0, 0, 0, 0, 3392, 0, 0, 0, 0, 0, 1097, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 331, 382, 384, 0, 0, 0, 563, 3417, 563, 563, 563, 563, 563, 563, 563, 563, 563, 587, 587, 587, 587, 587, 587, 587, 3115, 587, 587, 587, 587, 587, 587, 3433, 587, 587, 587, 587, 587, 587, 587, 587, 587, 541, 563, 3472, 563, 3474, 563, 563, 563, 563, 563, 563, 563, 563, 563, 563, 563, 563, 3243, 563, 563, 563, 3470, 563, 563, 563, 563, 563, 563, 563, 563, 563, 563, 563, 563, 563, 563, 3485, 563, 563, 563, 563, 3532, 563, 563, 563, 563, 563, 563, 563, 563, 563, 587, 587, 3110, 587, 587, 587, 587, 587, 587, 587, 587, 587, 587, 587, 3546, 587, 587, 587, 587, 587, 587, 587, 587, 587, 3555, 3556, 0, 0, 0, 3559, 0, 0, 0, 0, 0, 3565, 3566, 3567, 541, 541, 541, 3570, 541, 3572, 541, 541, 541, 541, 3577, 3578, 3579, 563, 563, 563, 3582, 563, 3584, 563, 0, 587, 587, 1044, 587, 587, 291, 1079, 0, 0, 1082, 1086, 0, 0, 1090, 563, 563, 563, 3589, 3590, 3591, 3592, 587, 587, 587, 3595, 587, 3597, 587, 587, 587, 587, 587, 1464, 587, 587, 587, 1473, 587, 587, 587, 587, 587, 587, 1949, 587, 587, 587, 587, 587, 587, 587, 587, 587, 2949, 587, 587, 587, 541, 587, 563, 587, 3602, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 541, 541, 541, 541, 541, 541, 1738, 541, 541, 541, 563, 3644, 563, 563, 563, 3648, 563, 587, 587, 3650, 587, 587, 587, 3654, 587, 0, 0, 0, 0, 0, 0, 0, 0, 3180, 0, 3449, 0, 0, 0, 0, 0, 0, 0, 69632, 73728, 266240, 0, 0, 0, 0, 65536, 0, 0, 0, 0, 329, 330, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 349, 0, 0, 0, 0, 0, 369, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 686, 0, 0, 0, 0, 369, 0, 0, 0, 377, 379, 0, 0, 0, 0, 0, 0, 0, 0, 1099, 0, 0, 0, 0, 0, 0, 0, 0, 1115, 0, 0, 0, 0, 0, 0, 0, 0, 0, 3185, 0, 0, 0, 0, 0, 0, 0, 0, 412, 0, 0, 0, 412, 69632, 73728, 0, 369, 369, 0, 424, 65536, 369, 0, 0, 0, 369, 424, 499, 503, 499, 499, 508, 499, 499, 499, 508, 499, 424, 424, 0, 330, 424, 0, 0, 424, 424, 0, 0, 0, 0, 0, 0, 0, 0, 1156, 0, 0, 0, 0, 0, 0, 0, 0, 664, 0, 0, 0, 0, 0, 0, 0, 0, 680, 681, 0, 0, 0, 0, 0, 0, 424, 424, 424, 424, 424, 424, 424, 424, 424, 424, 424, 424, 424, 424, 424, 424, 547, 570, 547, 570, 547, 547, 570, 547, 594, 570, 570, 570, 570, 570, 570, 570, 594, 594, 594, 547, 594, 594, 594, 594, 594, 594, 594, 594, 570, 570, 547, 570, 594, 570, 594, 1, 587, 587, 587, 1020, 587, 587, 587, 587, 587, 587, 587, 587, 587, 587, 587, 587, 1954, 587, 587, 587, 0, 0, 1109, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1215, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1226, 541, 1279, 541, 541, 541, 541, 541, 1291, 541, 541, 541, 541, 541, 541, 541, 541, 541, 1351, 541, 541, 541, 541, 541, 541, 1360, 914, 563, 563, 563, 563, 563, 563, 563, 563, 1371, 563, 563, 563, 563, 563, 563, 2643, 563, 563, 563, 563, 563, 563, 563, 563, 563, 1866, 563, 563, 563, 563, 563, 563, 1383, 563, 563, 563, 563, 563, 563, 563, 563, 563, 563, 563, 563, 563, 563, 563, 2286, 587, 587, 1992, 587, 587, 587, 587, 587, 587, 587, 587, 587, 587, 587, 587, 587, 2411, 541, 587, 563, 563, 3677, 587, 587, 587, 0, 0, 541, 541, 563, 563, 587, 587, 0, 541, 3203, 541, 541, 541, 3206, 541, 541, 541, 541, 541, 541, 541, 541, 541, 541, 1313, 541, 541, 541, 541, 541, 425, 425, 0, 0, 425, 440, 0, 425, 425, 467, 467, 467, 467, 467, 467, 467, 467, 467, 467, 467, 467, 467, 467, 467, 467, 467, 467, 467, 467, 493, 493, 467, 493, 493, 493, 493, 493, 493, 493, 493, 493, 493, 493, 493, 493, 493, 493, 493, 548, 571, 548, 571, 548, 548, 571, 548, 595, 571, 571, 571, 571, 571, 571, 571, 595, 595, 595, 548, 595, 595, 595, 595, 595, 595, 595, 595, 571, 571, 548, 571, 595, 571, 595, 1, 0, 1213, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 732, 0, 0, 0, 0, 0, 1695, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 360, 0, 0, 0, 0, 0, 0, 0, 0, 2093, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 365, 365, 0, 0, 0, 2157, 0, 0, 0, 541, 541, 541, 541, 541, 541, 541, 541, 541, 541, 541, 541, 3328, 541, 2201, 541, 541, 541, 541, 541, 541, 541, 541, 541, 541, 541, 541, 541, 541, 541, 2200, 563, 2415, 541, 541, 541, 2419, 563, 563, 563, 2423, 587, 587, 587, 0, 0, 0, 0, 0, 0, 541, 541, 541, 541, 541, 541, 541, 563, 3067, 563, 563, 563, 563, 563, 563, 563, 563, 563, 3253, 563, 563, 563, 563, 563, 563, 0, 2460, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 748, 0, 0, 0, 0, 0, 2791, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 365, 365, 1146, 0, 0, 2943, 587, 587, 587, 587, 587, 587, 587, 587, 587, 587, 587, 587, 541, 587, 563, 541, 541, 0, 0, 0, 0, 2971, 0, 0, 0, 2975, 0, 0, 0, 0, 0, 2981, 0, 0, 0, 0, 0, 1128, 0, 0, 0, 0, 0, 1233, 0, 0, 0, 1265, 0, 0, 0, 2994, 0, 2996, 0, 0, 0, 0, 0, 0, 0, 0, 3004, 0, 0, 0, 0, 0, 1169, 0, 1171, 0, 0, 0, 0, 1176, 0, 0, 0, 0, 0, 0, 1643, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 2520, 0, 0, 0, 0, 541, 0, 0, 0, 0, 3009, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 383, 0, 0, 0, 387, 541, 3063, 541, 541, 541, 3066, 563, 563, 563, 563, 563, 563, 563, 563, 563, 563, 3083, 563, 563, 563, 563, 563, 563, 563, 3105, 563, 563, 563, 3108, 587, 587, 587, 587, 587, 587, 587, 587, 587, 1509, 587, 587, 587, 587, 587, 587, 3144, 587, 587, 3147, 587, 587, 587, 541, 587, 563, 541, 541, 563, 563, 587, 587, 0, 0, 3295, 0, 0, 0, 0, 0, 0, 0, 0, 2766, 0, 0, 0, 0, 0, 0, 0, 0, 2151, 0, 0, 0, 0, 0, 0, 0, 0, 2506, 0, 0, 0, 0, 0, 2512, 0, 0, 0, 0, 0, 0, 3195, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 2782, 0, 0, 0, 0, 541, 541, 541, 541, 3216, 541, 541, 541, 541, 541, 541, 541, 3221, 541, 3223, 541, 541, 541, 541, 541, 3408, 541, 541, 3410, 563, 563, 563, 563, 3414, 563, 563, 563, 563, 1446, 1447, 563, 563, 563, 26028, 1360, 988, 587, 587, 587, 587, 587, 587, 3671, 0, 0, 0, 541, 541, 541, 3674, 563, 563, 563, 563, 563, 587, 587, 3260, 587, 587, 587, 587, 587, 587, 587, 587, 587, 587, 3274, 587, 587, 587, 587, 587, 587, 3280, 587, 587, 587, 587, 587, 587, 587, 587, 587, 587, 587, 541, 587, 563, 541, 2722, 563, 3350, 563, 563, 563, 563, 563, 563, 563, 563, 587, 587, 587, 587, 587, 587, 587, 587, 3655, 587, 587, 587, 587, 3363, 587, 587, 587, 587, 587, 587, 587, 587, 587, 3369, 587, 0, 0, 0, 541, 587, 563, 541, 541, 541, 541, 541, 563, 563, 563, 563, 563, 3071, 563, 563, 563, 563, 563, 0, 0, 0, 0, 541, 541, 3660, 3661, 541, 541, 563, 563, 3664, 3665, 563, 563, 563, 563, 1423, 563, 563, 1430, 563, 563, 563, 563, 563, 563, 563, 563, 2631, 563, 563, 563, 563, 563, 563, 563, 587, 587, 3668, 3669, 587, 587, 0, 0, 0, 0, 541, 541, 541, 541, 563, 563, 563, 924, 928, 931, 563, 939, 563, 563, 563, 954, 956, 959, 0, 0, 690, 691, 0, 0, 0, 0, 696, 0, 0, 0, 365, 365, 365, 0, 0, 0, 0, 0, 1185, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 3180, 0, 0, 0, 0, 0, 0, 829, 0, 0, 0, 0, 0, 0, 0, 0, 0, 758, 0, 0, 0, 0, 0, 0, 758, 0, 0, 0, 0, 0, 758, 758, 910, 541, 563, 563, 563, 563, 927, 563, 563, 563, 563, 563, 563, 563, 563, 563, 1432, 563, 563, 1436, 563, 563, 563, 983, 0, 587, 587, 587, 587, 1057, 78115, 1079, 0, 0, 0, 0, 0, 0, 0, 0, 1220, 0, 0, 0, 0, 0, 0, 0, 0, 0, 282624, 282624, 282624, 282624, 282624, 282624, 282624, 282624, 587, 1521, 587, 587, 587, 587, 587, 587, 587, 587, 587, 587, 587, 587, 587, 587, 1514, 587, 0, 0, 0, 1721, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1193, 0, 0, 0, 0, 2040, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 749, 0, 2171, 541, 541, 541, 541, 541, 541, 541, 541, 541, 541, 541, 541, 541, 541, 541, 2211, 3288, 541, 3290, 563, 3292, 587, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 3514, 468, 468, 468, 486, 494, 494, 486, 494, 494, 494, 494, 494, 494, 494, 494, 519, 527, 527, 527, 527, 527, 527, 527, 527, 527, 527, 527, 527, 527, 527, 527, 536, 527, 527, 527, 527, 527, 549, 572, 549, 572, 549, 549, 572, 549, 596, 572, 572, 572, 572, 572, 572, 572, 596, 596, 596, 549, 596, 596, 596, 596, 596, 596, 596, 596, 572, 572, 549, 572, 596, 572, 596, 1, 795, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1652, 0, 818, 0, 0, 0, 795, 0, 0, 818, 0, 0, 0, 0, 0, 818, 818, 0, 0, 0, 0, 795, 0, 0, 0, 0, 0, 0, 836, 791, 0, 0, 836, 857, 541, 865, 541, 541, 541, 541, 541, 541, 541, 541, 541, 541, 541, 541, 541, 541, 3527, 563, 911, 541, 563, 563, 563, 563, 563, 930, 563, 938, 563, 563, 563, 563, 563, 563, 1850, 563, 563, 563, 563, 563, 563, 563, 563, 563, 0, 587, 587, 587, 587, 587, 587, 587, 1012, 587, 587, 587, 587, 587, 587, 587, 587, 587, 587, 587, 587, 587, 1058, 984, 0, 587, 587, 587, 1077, 1058, 78115, 1079, 0, 0, 0, 0, 0, 0, 0, 0, 1243, 0, 0, 0, 0, 0, 0, 0, 0, 1259, 0, 0, 0, 1263, 0, 0, 0, 0, 0, 0, 0, 1139, 1140, 0, 0, 0, 0, 0, 365, 365, 0, 0, 0, 0, 0, 0, 1113, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 2071, 0, 0, 0, 0, 0, 1180, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1705, 1340, 541, 541, 1344, 541, 541, 541, 541, 541, 1350, 541, 541, 541, 1357, 541, 541, 541, 541, 541, 1812, 541, 541, 541, 541, 541, 541, 541, 541, 541, 1360, 1403, 563, 563, 563, 563, 563, 563, 563, 563, 563, 563, 563, 563, 563, 563, 563, 2638, 563, 563, 1442, 563, 563, 563, 1449, 563, 563, 26028, 1360, 988, 587, 587, 587, 587, 587, 587, 3272, 587, 587, 587, 587, 587, 587, 587, 587, 3278, 587, 587, 1487, 587, 587, 587, 587, 587, 587, 587, 1495, 587, 587, 587, 587, 587, 587, 2365, 587, 587, 587, 587, 587, 587, 587, 587, 587, 2691, 587, 587, 587, 587, 587, 587, 587, 587, 587, 1524, 587, 587, 1528, 587, 587, 587, 587, 587, 1534, 587, 587, 587, 587, 587, 1492, 587, 587, 587, 587, 587, 587, 587, 587, 587, 587, 1510, 587, 587, 587, 587, 587, 1541, 587, 587, 0, 541, 587, 563, 541, 541, 541, 541, 541, 563, 563, 563, 563, 563, 3476, 563, 563, 563, 3480, 563, 563, 563, 563, 563, 563, 2863, 563, 563, 2867, 563, 563, 563, 563, 563, 563, 1409, 563, 563, 1413, 563, 563, 563, 563, 563, 563, 1448, 563, 563, 26028, 1360, 988, 587, 587, 587, 587, 0, 0, 0, 1595, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1223, 0, 0, 0, 0, 0, 1655, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1664, 0, 0, 0, 0, 0, 0, 1126, 0, 0, 0, 1130, 1131, 0, 0, 0, 0, 0, 0, 0, 2818048, 2846720, 0, 2916352, 0, 0, 3002368, 0, 0, 1718, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 2053, 1702, 0, 0, 0, 0, 0, 541, 541, 541, 541, 541, 541, 541, 541, 541, 541, 3326, 541, 541, 541, 563, 563, 563, 1893, 563, 563, 563, 563, 563, 563, 1901, 563, 563, 563, 563, 563, 563, 2887, 563, 563, 563, 2890, 563, 2892, 563, 563, 563, 587, 1944, 587, 587, 587, 587, 587, 587, 587, 587, 587, 587, 587, 587, 587, 587, 1940, 587, 587, 1974, 587, 587, 587, 587, 1980, 587, 587, 587, 587, 587, 587, 587, 587, 1989, 2007, 2009, 587, 587, 587, 587, 587, 541, 587, 563, 541, 541, 541, 541, 563, 563, 563, 563, 587, 587, 587, 587, 2030, 0, 2032, 0, 0, 2079, 0, 0, 0, 2082, 0, 0, 0, 0, 0, 2088, 0, 0, 0, 0, 0, 0, 1141, 0, 1143, 0, 0, 365, 365, 0, 0, 0, 0, 0, 0, 1154, 0, 0, 0, 0, 0, 1160, 0, 1162, 0, 2104, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 2115, 0, 0, 0, 0, 0, 1192, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 176128, 176128, 176128, 176128, 176128, 176128, 176128, 0, 0, 563, 563, 563, 563, 563, 563, 2250, 563, 563, 563, 563, 563, 563, 563, 1851, 563, 563, 563, 563, 1855, 563, 563, 563, 587, 587, 587, 587, 2349, 587, 587, 587, 587, 587, 587, 587, 587, 587, 587, 587, 1985, 587, 587, 1988, 587, 2262, 541, 541, 541, 541, 563, 563, 563, 563, 587, 587, 587, 587, 0, 0, 0, 0, 0, 2732, 0, 2450, 0, 0, 0, 0, 0, 0, 0, 0, 2455, 0, 0, 2458, 0, 0, 0, 0, 0, 0, 2134016, 0, 0, 0, 0, 0, 0, 57344, 0, 0, 0, 0, 0, 0, 2748, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 192972, 192972, 192972, 192972, 192972, 192972, 192972, 0, 0, 0, 0, 2462, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 541, 541, 541, 2809, 541, 2473, 0, 0, 0, 0, 0, 0, 0, 2481, 0, 0, 0, 2483, 0, 0, 0, 0, 0, 0, 1170, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 2989, 0, 0, 0, 0, 0, 0, 2500, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 2510, 0, 0, 0, 0, 0, 0, 1186, 0, 0, 0, 1191, 0, 0, 0, 0, 1107, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 758, 0, 0, 0, 0, 0, 0, 0, 2524, 541, 541, 541, 541, 541, 541, 541, 541, 541, 541, 541, 541, 541, 541, 541, 2536, 541, 541, 2552, 541, 541, 541, 541, 541, 541, 541, 541, 541, 541, 541, 541, 541, 905, 541, 541, 2564, 541, 541, 541, 541, 541, 541, 2571, 541, 541, 541, 541, 541, 541, 541, 541, 541, 1768, 541, 541, 541, 541, 541, 541, 541, 2578, 541, 541, 541, 2581, 541, 541, 541, 541, 541, 541, 541, 0, 0, 0, 0, 0, 0, 1203, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 813, 0, 0, 0, 0, 0, 0, 0, 0, 563, 2588, 563, 563, 563, 563, 563, 563, 563, 563, 563, 563, 563, 1903, 1904, 563, 563, 563, 2611, 563, 563, 563, 563, 563, 563, 2616, 563, 563, 563, 563, 563, 563, 563, 2622, 587, 587, 587, 587, 2702, 587, 587, 587, 587, 587, 587, 587, 587, 587, 587, 2710, 587, 587, 587, 2713, 587, 587, 587, 587, 587, 587, 587, 2718, 2719, 2720, 541, 541, 541, 541, 541, 2177, 541, 541, 541, 541, 541, 541, 541, 541, 541, 541, 541, 541, 2845, 541, 541, 541, 2758, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 2103, 0, 2773, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1120, 0, 0, 0, 2789, 0, 0, 0, 0, 2794, 0, 0, 0, 2796, 0, 0, 0, 0, 0, 0, 0, 69632, 73728, 316, 317, 317, 422, 423, 65536, 430, 541, 541, 541, 541, 2814, 541, 541, 541, 541, 541, 541, 541, 541, 541, 541, 541, 541, 1333, 541, 541, 541, 541, 541, 2848, 541, 541, 541, 541, 541, 541, 0, 0, 563, 563, 563, 2856, 563, 0, 587, 587, 1047, 587, 587, 78115, 1079, 0, 0, 0, 0, 0, 0, 0, 0, 1159168, 365, 0, 0, 0, 0, 0, 0, 563, 563, 563, 563, 2861, 563, 563, 563, 563, 563, 563, 563, 563, 563, 563, 563, 2294, 563, 563, 563, 563, 563, 563, 563, 563, 2897, 563, 563, 563, 563, 563, 563, 0, 0, 587, 587, 587, 587, 587, 587, 587, 2338, 587, 587, 587, 587, 587, 587, 587, 3122, 587, 587, 587, 587, 587, 587, 587, 587, 1036, 587, 587, 587, 587, 1054, 587, 587, 2905, 587, 587, 587, 587, 587, 2910, 587, 587, 587, 587, 587, 587, 587, 587, 587, 1935, 587, 587, 587, 587, 1941, 587, 3006, 0, 0, 0, 0, 3010, 0, 0, 0, 0, 3015, 0, 0, 0, 0, 0, 0, 0, 2749, 0, 0, 0, 0, 0, 0, 0, 0, 0, 697, 698, 0, 365, 365, 365, 0, 3191, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 2145, 3300, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 3180, 0, 0, 0, 0, 0, 0, 0, 69632, 73728, 163840, 0, 0, 0, 0, 65536, 0, 563, 563, 563, 3340, 563, 563, 563, 563, 563, 563, 563, 563, 563, 563, 563, 563, 2283, 563, 563, 563, 563, 563, 563, 3351, 563, 563, 563, 563, 563, 563, 587, 587, 587, 587, 587, 587, 587, 587, 587, 587, 587, 587, 587, 3359, 587, 587, 587, 587, 587, 587, 587, 587, 587, 587, 587, 587, 587, 587, 587, 1942, 3370, 587, 587, 587, 587, 587, 587, 541, 541, 563, 563, 587, 587, 0, 0, 0, 0, 3297, 0, 0, 0, 0, 0, 0, 0, 0, 0, 3394, 0, 541, 541, 541, 541, 541, 541, 541, 541, 541, 541, 541, 541, 3211, 541, 541, 3486, 587, 587, 587, 587, 587, 587, 587, 587, 3495, 587, 3497, 3498, 587, 3500, 587, 0, 0, 0, 541, 587, 563, 541, 541, 541, 904, 541, 563, 563, 563, 977, 3502, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 541, 541, 541, 854, 541, 541, 3612, 541, 3613, 541, 541, 541, 563, 563, 563, 563, 563, 563, 3619, 563, 0, 996, 1075, 1041, 587, 587, 78115, 1079, 0, 0, 1081, 1085, 0, 0, 1089, 3620, 563, 563, 563, 563, 587, 587, 587, 587, 587, 587, 3627, 587, 3628, 587, 587, 0, 3603, 0, 0, 0, 0, 0, 0, 0, 0, 541, 541, 3610, 541, 563, 563, 587, 587, 587, 587, 0, 0, 541, 541, 563, 563, 587, 587, 3683, 3684, 3685, 3686, 0, 541, 563, 587, 0, 541, 563, 587, 0, 541, 563, 587, 0, 0, 0, 0, 0, 0, 0, 0, 3180, 0, 0, 0, 0, 0, 0, 0, 550, 597, 573, 573, 573, 573, 573, 573, 573, 597, 597, 597, 550, 597, 597, 597, 597, 597, 597, 597, 597, 573, 573, 550, 573, 597, 573, 597, 1, 0, 0, 0, 737, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1237, 0, 0, 0, 0, 0, 2041, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1194, 1196, 0, 403, 0, 0, 0, 0, 381, 0, 69632, 73728, 0, 0, 0, 0, 426, 65536, 0, 0, 0, 0, 0, 1202, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 2444, 2445, 0, 0, 2448, 426, 426, 0, 0, 426, 0, 446, 426, 426, 469, 469, 469, 476, 469, 469, 469, 469, 469, 469, 469, 469, 469, 476, 469, 469, 469, 469, 469, 469, 469, 469, 483, 469, 495, 495, 469, 495, 495, 495, 495, 495, 495, 495, 495, 495, 495, 495, 495, 495, 495, 495, 495, 495, 495, 495, 495, 495, 495, 495, 495, 538, 551, 574, 551, 574, 551, 551, 574, 551, 598, 574, 574, 574, 574, 574, 574, 574, 598, 598, 598, 551, 598, 598, 598, 598, 598, 598, 598, 598, 574, 574, 551, 574, 598, 574, 598, 1, 0, 0, 0, 0, 660, 661, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 3001, 0, 0, 0, 0, 0, 0, 0, 0, 661, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 661, 0, 0, 0, 0, 0, 827, 0, 0, 0, 661, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 3176, 0, 0, 0, 0, 0, 0, 0, 729, 0, 742, 661, 0, 0, 0, 0, 0, 541, 844, 541, 541, 541, 541, 541, 2191, 541, 541, 541, 541, 541, 541, 2197, 541, 2199, 541, 541, 859, 541, 541, 541, 541, 541, 541, 541, 541, 541, 541, 897, 541, 541, 541, 541, 541, 541, 2817, 541, 2819, 541, 541, 541, 541, 541, 541, 541, 541, 2572, 541, 541, 541, 541, 541, 541, 541, 541, 1349, 541, 541, 541, 541, 541, 541, 541, 541, 1752, 541, 541, 541, 541, 541, 541, 541, 541, 1767, 541, 541, 541, 541, 541, 541, 541, 541, 1782, 541, 541, 541, 541, 541, 541, 541, 541, 1815, 1817, 541, 541, 541, 541, 541, 1360, 563, 563, 563, 970, 563, 563, 563, 563, 563, 0, 587, 587, 990, 587, 587, 587, 587, 587, 1526, 587, 587, 587, 587, 587, 587, 587, 587, 587, 587, 1953, 587, 587, 587, 587, 587, 1006, 587, 587, 587, 587, 587, 587, 587, 587, 587, 587, 1044, 587, 587, 587, 587, 587, 587, 3285, 587, 587, 587, 587, 587, 587, 541, 587, 563, 1094, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1101, 1102, 0, 0, 0, 0, 0, 0, 0, 69632, 73728, 167936, 0, 0, 0, 0, 65536, 0, 0, 0, 0, 1229, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1247, 0, 0, 0, 0, 0, 0, 0, 1102, 0, 0, 0, 0, 1260, 1261, 0, 0, 1101, 0, 0, 0, 0, 0, 0, 2134756, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1116, 0, 0, 0, 0, 0, 541, 1306, 541, 541, 541, 541, 541, 541, 541, 541, 541, 541, 541, 541, 541, 541, 0, 2241, 1360, 914, 563, 1363, 563, 563, 563, 563, 563, 563, 563, 563, 563, 563, 563, 1377, 1384, 563, 563, 563, 563, 563, 563, 563, 563, 563, 1398, 563, 563, 563, 563, 563, 563, 3079, 563, 563, 563, 563, 563, 563, 563, 563, 563, 0, 587, 587, 989, 587, 587, 587, 1418, 563, 563, 563, 563, 563, 563, 563, 1431, 563, 563, 563, 563, 563, 563, 563, 1897, 563, 563, 563, 563, 563, 563, 563, 563, 1395, 563, 563, 563, 563, 563, 563, 563, 587, 587, 1523, 587, 587, 587, 587, 587, 587, 587, 587, 587, 587, 587, 1537, 587, 0, 0, 0, 541, 587, 563, 541, 541, 541, 1067, 911, 563, 563, 563, 1072, 563, 587, 587, 1523, 587, 587, 1079, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 2781, 0, 0, 0, 0, 0, 541, 541, 541, 1746, 1747, 541, 541, 541, 541, 1754, 541, 541, 541, 541, 541, 541, 541, 1290, 541, 1295, 541, 541, 1299, 541, 541, 1302, 541, 541, 541, 541, 1795, 541, 541, 541, 541, 541, 541, 541, 541, 541, 541, 541, 541, 1335, 541, 541, 541, 1825, 563, 563, 563, 563, 563, 563, 563, 563, 563, 563, 563, 563, 563, 563, 1841, 1842, 563, 563, 563, 563, 1849, 563, 563, 563, 563, 563, 563, 563, 563, 563, 563, 3347, 563, 563, 563, 563, 3348, 1890, 563, 563, 563, 563, 563, 563, 563, 563, 563, 563, 563, 563, 563, 563, 563, 2649, 587, 1945, 587, 587, 587, 587, 587, 587, 587, 587, 587, 587, 587, 587, 587, 587, 2372, 587, 587, 1959, 587, 587, 587, 587, 587, 587, 587, 587, 587, 587, 587, 587, 587, 587, 2384, 2385, 587, 587, 587, 587, 2012, 587, 587, 541, 587, 563, 541, 541, 541, 541, 563, 563, 587, 587, 2961, 0, 0, 2964, 2965, 0, 0, 0, 0, 0, 0, 0, 2505, 0, 0, 0, 0, 0, 0, 0, 0, 0, 387, 0, 0, 0, 0, 0, 387, 0, 0, 0, 2054, 0, 2055, 0, 0, 0, 0, 0, 0, 0, 0, 0, 2064, 541, 541, 541, 541, 2216, 541, 541, 541, 541, 541, 541, 2221, 541, 541, 541, 541, 541, 541, 1285, 1292, 541, 541, 541, 541, 541, 541, 541, 541, 541, 1800, 541, 541, 541, 1805, 541, 541, 541, 2226, 541, 541, 541, 541, 541, 541, 541, 541, 541, 541, 541, 541, 0, 0, 563, 563, 563, 563, 563, 563, 563, 563, 2302, 563, 563, 563, 563, 563, 563, 2307, 563, 563, 563, 563, 563, 563, 3080, 563, 563, 563, 563, 563, 563, 563, 563, 563, 1883, 563, 563, 563, 563, 563, 563, 563, 2313, 563, 563, 563, 563, 563, 563, 563, 563, 563, 563, 563, 563, 0, 0, 587, 587, 587, 587, 587, 587, 2389, 587, 587, 587, 587, 587, 587, 2394, 587, 587, 587, 587, 587, 587, 2377, 587, 587, 587, 587, 587, 587, 587, 587, 587, 1952, 587, 587, 587, 587, 1956, 587, 587, 2400, 587, 587, 587, 587, 587, 587, 587, 587, 587, 587, 587, 587, 541, 587, 0, 0, 0, 541, 587, 563, 541, 541, 895, 541, 541, 563, 563, 968, 563, 0, 587, 587, 587, 587, 587, 78115, 1079, 0, 0, 1082, 1086, 0, 0, 1090, 0, 2437, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1178, 0, 587, 2671, 587, 587, 587, 587, 587, 587, 587, 587, 587, 587, 587, 587, 587, 587, 2931, 587, 0, 0, 0, 2736, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1588, 1589, 0, 0, 2787, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 2797, 0, 0, 0, 0, 0, 0, 1204, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1631, 0, 0, 0, 0, 0, 541, 541, 541, 2813, 541, 541, 541, 541, 541, 541, 2821, 541, 541, 541, 541, 541, 541, 541, 2555, 541, 541, 541, 541, 541, 541, 541, 2562, 563, 563, 563, 2860, 563, 563, 563, 563, 563, 563, 2868, 563, 563, 563, 563, 563, 563, 3094, 563, 563, 3096, 563, 563, 563, 563, 563, 563, 1880, 1881, 1882, 563, 563, 1885, 1886, 563, 563, 563, 0, 0, 3008, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1197, 0, 0, 541, 541, 3227, 541, 541, 541, 541, 541, 541, 563, 563, 563, 563, 563, 563, 563, 3073, 563, 563, 563, 3571, 541, 3573, 541, 541, 541, 563, 563, 563, 563, 563, 563, 563, 3583, 563, 3585, 0, 0, 0, 0, 541, 3659, 541, 541, 541, 541, 563, 3663, 563, 563, 563, 563, 563, 1878, 563, 563, 563, 563, 1884, 563, 563, 563, 563, 563, 563, 2864, 563, 2866, 563, 563, 563, 563, 563, 563, 563, 3354, 563, 563, 587, 587, 587, 587, 587, 587, 587, 587, 587, 587, 587, 587, 587, 587, 587, 587, 1057, 587, 3667, 587, 587, 587, 587, 0, 0, 0, 0, 541, 541, 541, 541, 563, 563, 915, 563, 563, 563, 563, 563, 563, 946, 563, 563, 563, 563, 563, 1427, 563, 563, 563, 563, 563, 563, 563, 563, 563, 563, 3535, 563, 563, 563, 563, 563, 563, 587, 587, 587, 587, 3112, 587, 587, 587, 587, 3116, 470, 470, 470, 470, 454, 454, 470, 454, 454, 454, 454, 454, 454, 454, 454, 520, 520, 520, 520, 520, 520, 520, 520, 520, 520, 520, 520, 520, 520, 520, 520, 552, 575, 552, 575, 552, 552, 575, 552, 599, 575, 575, 575, 575, 575, 575, 575, 599, 599, 599, 552, 599, 599, 599, 599, 599, 599, 599, 599, 575, 575, 552, 575, 599, 575, 599, 1, 541, 860, 541, 541, 874, 541, 541, 541, 541, 541, 541, 541, 541, 541, 541, 541, 541, 1770, 541, 541, 541, 1007, 587, 587, 1021, 587, 587, 587, 587, 587, 587, 587, 587, 587, 587, 587, 587, 1971, 587, 587, 587, 0, 1123, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1184, 1184, 1251, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 2156, 0, 0, 0, 1268, 0, 0, 0, 0, 0, 0, 541, 541, 541, 541, 541, 541, 541, 541, 541, 541, 541, 541, 541, 541, 541, 1360, 541, 541, 541, 1323, 541, 541, 541, 541, 541, 541, 541, 541, 541, 541, 541, 541, 541, 2534, 541, 541, 541, 1341, 541, 541, 541, 541, 541, 1348, 541, 541, 541, 541, 541, 541, 541, 541, 541, 2234, 541, 541, 541, 541, 0, 0, 1440, 563, 563, 563, 563, 563, 563, 563, 563, 26028, 1360, 988, 587, 587, 587, 587, 587, 587, 3365, 587, 587, 587, 587, 587, 587, 587, 587, 587, 2936, 587, 587, 587, 2939, 587, 2941, 587, 587, 587, 587, 587, 1525, 587, 587, 587, 587, 587, 1532, 587, 587, 587, 587, 587, 587, 2392, 587, 587, 587, 587, 587, 587, 587, 587, 587, 1472, 587, 587, 587, 587, 1482, 587, 563, 587, 587, 587, 1557, 587, 1079, 0, 1561, 0, 0, 0, 1567, 0, 0, 0, 0, 0, 0, 1218, 1219, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1235, 0, 0, 0, 0, 0, 0, 1573, 0, 0, 0, 1579, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 662, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 541, 541, 541, 1779, 541, 541, 541, 541, 541, 541, 541, 541, 541, 541, 541, 541, 541, 2548, 541, 541, 563, 1843, 563, 563, 563, 563, 563, 563, 563, 563, 563, 1854, 563, 563, 563, 563, 563, 1879, 563, 563, 563, 563, 563, 563, 563, 563, 563, 563, 3344, 563, 563, 563, 563, 563, 563, 563, 563, 1865, 563, 563, 563, 563, 563, 563, 563, 2065, 2066, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1210, 0, 2117, 0, 0, 2120, 2121, 0, 0, 0, 0, 0, 2127, 0, 0, 0, 0, 0, 0, 0, 2765, 0, 0, 0, 0, 0, 0, 0, 0, 0, 550, 573, 550, 573, 550, 550, 573, 541, 2213, 541, 541, 541, 541, 2218, 541, 541, 541, 541, 541, 541, 541, 541, 541, 541, 2182, 541, 541, 541, 541, 541, 2299, 563, 563, 563, 563, 2304, 563, 563, 563, 563, 563, 563, 563, 563, 563, 563, 1917, 563, 26028, 0, 587, 587, 587, 2386, 587, 587, 587, 587, 2391, 587, 587, 587, 587, 587, 587, 587, 587, 587, 587, 2001, 587, 587, 587, 587, 587, 2006, 541, 541, 541, 3037, 541, 541, 541, 541, 541, 541, 541, 541, 541, 541, 541, 541, 541, 2846, 541, 541, 563, 563, 3077, 563, 563, 563, 563, 563, 563, 563, 563, 563, 563, 563, 563, 563, 1887, 1888, 563, 587, 587, 587, 3119, 587, 587, 587, 587, 587, 587, 587, 587, 587, 587, 587, 587, 1986, 587, 587, 587, 563, 563, 3247, 563, 563, 563, 563, 563, 563, 563, 563, 563, 563, 563, 563, 563, 2296, 563, 563, 587, 0, 0, 0, 3445, 0, 0, 0, 0, 3180, 0, 0, 0, 0, 0, 0, 0, 1242, 0, 0, 0, 0, 0, 0, 0, 0, 0, 380, 0, 0, 0, 0, 0, 0, 0, 0, 541, 541, 541, 541, 3460, 541, 541, 541, 541, 541, 541, 541, 541, 541, 541, 2207, 541, 541, 541, 541, 541, 563, 587, 587, 587, 587, 3491, 587, 587, 587, 587, 587, 587, 587, 587, 587, 587, 2354, 587, 587, 587, 587, 587, 587, 563, 563, 563, 563, 3623, 587, 587, 587, 587, 587, 587, 587, 587, 587, 587, 587, 2395, 587, 587, 587, 587, 587, 3631, 0, 0, 0, 0, 0, 0, 541, 541, 541, 541, 541, 541, 541, 563, 563, 920, 563, 563, 563, 563, 563, 563, 949, 951, 563, 563, 563, 563, 563, 3421, 563, 563, 563, 3425, 563, 587, 587, 587, 587, 587, 587, 587, 587, 587, 587, 587, 587, 587, 587, 3501, 388, 390, 340, 0, 0, 0, 0, 0, 0, 339, 0, 0, 340, 0, 0, 0, 0, 0, 0, 1598, 0, 0, 0, 0, 0, 0, 0, 0, 1604, 0, 0, 0, 387, 0, 0, 0, 69632, 73728, 0, 0, 0, 0, 0, 65536, 0, 0, 0, 0, 0, 1217, 0, 0, 0, 0, 0, 0, 0, 1224, 0, 0, 0, 0, 0, 347, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 3316, 0, 0, 0, 0, 0, 0, 435, 339, 0, 0, 447, 0, 0, 471, 471, 471, 471, 471, 471, 471, 471, 471, 553, 576, 553, 576, 553, 553, 576, 471, 482, 471, 471, 471, 500, 477, 500, 500, 500, 500, 500, 500, 500, 500, 471, 471, 477, 471, 471, 471, 471, 471, 471, 471, 471, 471, 471, 471, 481, 481, 471, 482, 471, 471, 553, 600, 576, 576, 576, 576, 576, 576, 576, 600, 600, 600, 553, 600, 600, 600, 600, 600, 600, 600, 600, 576, 576, 553, 576, 600, 576, 600, 1, 0, 0, 750, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1603, 0, 0, 0, 0, 0, 0, 663, 0, 788, 0, 0, 0, 0, 0, 0, 0, 792, 0, 0, 0, 0, 0, 1231, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 2153, 0, 0, 0, 0, 0, 0, 0, 801, 0, 0, 0, 0, 0, 809, 0, 0, 0, 0, 706, 0, 0, 0, 0, 0, 0, 0, 0, 715, 0, 717, 0, 828, 0, 0, 0, 663, 831, 0, 788, 0, 0, 0, 0, 0, 837, 0, 0, 0, 0, 0, 1597, 0, 0, 0, 577536, 0, 0, 1602, 0, 0, 0, 0, 0, 0, 1257, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 2508, 0, 0, 0, 0, 0, 541, 861, 541, 541, 875, 541, 541, 541, 541, 888, 541, 541, 541, 541, 906, 541, 541, 541, 541, 541, 3519, 541, 541, 541, 541, 541, 541, 541, 541, 563, 563, 563, 923, 563, 929, 563, 563, 942, 563, 563, 953, 563, 958, 961, 563, 563, 563, 563, 979, 563, 563, 563, 0, 587, 587, 991, 587, 587, 587, 587, 587, 1962, 587, 587, 587, 587, 587, 587, 587, 587, 587, 587, 3287, 587, 587, 587, 541, 587, 563, 1008, 587, 587, 1022, 587, 587, 587, 587, 1035, 587, 587, 587, 587, 1053, 587, 587, 587, 587, 587, 2945, 587, 587, 2948, 587, 587, 2951, 587, 2952, 2953, 2954, 0, 0, 0, 0, 1216, 0, 0, 0, 0, 1221, 0, 0, 0, 0, 0, 0, 0, 1258, 0, 0, 0, 0, 0, 0, 0, 0, 0, 365, 299, 0, 0, 0, 0, 0, 0, 1144, 0, 0, 1256, 0, 0, 0, 0, 0, 0, 0, 0, 1235, 0, 0, 0, 0, 0, 0, 2891776, 0, 0, 0, 0, 0, 2392064, 2412544, 0, 0, 0, 0, 0, 0, 2123, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 2727936, 0, 0, 0, 3084288, 0, 0, 0, 1267, 0, 0, 0, 0, 0, 0, 0, 541, 541, 541, 541, 541, 541, 541, 541, 541, 1741, 541, 541, 541, 1309, 541, 541, 541, 541, 541, 541, 541, 541, 541, 541, 541, 541, 541, 3045, 541, 3047, 563, 587, 587, 1556, 587, 587, 1079, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 3000, 0, 0, 0, 0, 0, 1621, 0, 0, 1624, 0, 1626, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 3198, 3199, 0, 0, 0, 0, 0, 0, 0, 1722, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 683, 684, 685, 0, 0, 541, 541, 541, 541, 1780, 541, 541, 541, 541, 541, 541, 541, 541, 541, 541, 541, 541, 1820, 541, 541, 1360, 0, 563, 563, 563, 563, 563, 563, 563, 563, 563, 563, 1836, 563, 563, 563, 563, 563, 2263, 563, 563, 563, 563, 563, 563, 563, 563, 563, 563, 1433, 563, 563, 563, 563, 563, 1875, 563, 563, 563, 563, 563, 563, 563, 563, 563, 563, 563, 563, 563, 563, 563, 2872, 1943, 587, 587, 587, 587, 587, 587, 587, 587, 587, 587, 587, 587, 587, 587, 587, 2359, 0, 0, 2135, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1618, 0, 0, 2146, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 2484, 0, 0, 0, 0, 541, 541, 541, 2163, 2164, 541, 541, 541, 541, 541, 541, 541, 541, 2180, 541, 541, 541, 541, 2185, 541, 541, 541, 541, 541, 2228, 541, 541, 541, 541, 541, 541, 541, 541, 541, 541, 0, 0, 563, 563, 563, 563, 2857, 0, 0, 563, 563, 563, 2247, 563, 2249, 563, 563, 563, 563, 563, 563, 563, 563, 2320, 563, 563, 563, 563, 563, 0, 0, 563, 563, 563, 2315, 563, 563, 563, 563, 563, 563, 563, 563, 563, 563, 0, 0, 988, 587, 587, 587, 587, 0, 0, 587, 587, 587, 2334, 587, 2336, 587, 587, 587, 587, 587, 587, 587, 587, 1965, 1966, 1968, 587, 587, 587, 587, 587, 587, 587, 587, 2402, 587, 587, 587, 587, 587, 587, 587, 587, 587, 587, 541, 587, 0, 0, 0, 541, 587, 563, 541, 541, 897, 541, 541, 563, 563, 970, 563, 0, 997, 587, 1076, 1048, 587, 78115, 1079, 0, 0, 0, 0, 0, 0, 0, 0, 458, 1147355, 1147355, 1147355, 1147355, 1147355, 1147355, 1147355, 1147355, 1147355, 1147355, 1147355, 1147355, 1147355, 1147355, 1147355, 1147355, 0, 0, 2438, 0, 0, 0, 0, 0, 0, 2443, 0, 0, 0, 2446, 2447, 0, 0, 0, 0, 0, 1610, 0, 0, 1613, 0, 0, 0, 0, 0, 0, 0, 0, 1584, 0, 0, 0, 0, 0, 0, 0, 0, 1683, 0, 0, 0, 0, 0, 0, 0, 0, 334, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1134, 0, 0, 0, 0, 2487, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 2498, 0, 0, 0, 2515, 0, 0, 0, 0, 0, 0, 0, 0, 0, 2523, 0, 541, 541, 541, 541, 541, 3576, 563, 563, 563, 3580, 563, 563, 563, 563, 563, 563, 2290, 563, 563, 563, 563, 563, 563, 563, 563, 563, 1899, 563, 563, 563, 563, 563, 1906, 2537, 2538, 541, 541, 541, 541, 541, 2543, 541, 2545, 541, 541, 541, 541, 2549, 541, 541, 541, 541, 541, 3614, 3615, 541, 563, 563, 563, 563, 563, 563, 563, 563, 3424, 563, 563, 587, 587, 587, 587, 587, 563, 563, 2598, 563, 563, 2601, 2602, 563, 563, 563, 563, 563, 2607, 563, 2609, 563, 0, 999, 1039, 587, 1050, 587, 78115, 1079, 0, 0, 0, 0, 0, 0, 0, 0, 3184, 0, 0, 0, 0, 0, 0, 0, 0, 2750, 0, 0, 0, 0, 0, 0, 0, 0, 0, 2732032, 0, 0, 0, 0, 0, 0, 563, 563, 563, 2613, 563, 563, 563, 563, 563, 563, 563, 563, 2619, 563, 563, 563, 563, 563, 3624, 587, 587, 587, 587, 587, 587, 587, 587, 587, 587, 1967, 587, 587, 587, 1972, 587, 587, 563, 2639, 563, 563, 563, 563, 563, 563, 563, 563, 563, 563, 563, 563, 563, 563, 1402, 563, 587, 587, 587, 2673, 587, 2675, 587, 587, 587, 587, 2679, 587, 587, 587, 587, 587, 587, 2676, 587, 587, 587, 587, 587, 587, 587, 587, 587, 1469, 1476, 587, 587, 587, 587, 587, 587, 587, 587, 2685, 587, 587, 587, 587, 587, 587, 587, 587, 2693, 587, 587, 587, 587, 587, 1979, 587, 587, 587, 587, 587, 587, 587, 587, 587, 587, 2926, 2927, 587, 587, 587, 587, 2930, 587, 587, 587, 587, 587, 2701, 587, 587, 587, 587, 2705, 587, 587, 587, 587, 587, 587, 587, 2378, 2379, 587, 587, 587, 587, 587, 587, 587, 2406, 587, 587, 2409, 2410, 587, 587, 2176, 2348, 0, 2745, 0, 0, 0, 0, 0, 0, 0, 2751, 2752, 2753, 0, 0, 0, 0, 0, 0, 0, 69632, 73728, 172032, 0, 0, 0, 0, 65536, 0, 2772, 0, 2774, 0, 0, 0, 0, 2778, 0, 0, 0, 0, 2783, 0, 0, 2786, 541, 541, 2828, 541, 541, 541, 541, 541, 541, 541, 541, 541, 541, 541, 541, 541, 1771, 541, 541, 541, 541, 541, 2838, 541, 541, 541, 541, 541, 541, 541, 541, 541, 541, 541, 541, 541, 3337, 563, 563, 541, 2847, 541, 541, 2850, 541, 541, 2853, 541, 0, 0, 2854, 563, 563, 563, 563, 563, 2277, 563, 563, 2280, 563, 563, 563, 563, 563, 563, 563, 3081, 563, 563, 563, 563, 563, 563, 563, 563, 3355, 563, 587, 587, 587, 587, 587, 587, 563, 563, 2875, 563, 563, 563, 563, 563, 563, 563, 563, 563, 563, 563, 563, 563, 2324, 0, 0, 563, 563, 563, 563, 2886, 563, 563, 563, 563, 563, 563, 563, 563, 563, 563, 563, 2606, 563, 2608, 563, 563, 563, 563, 563, 2896, 563, 563, 2899, 563, 563, 2902, 563, 0, 0, 2903, 587, 587, 587, 587, 587, 3284, 587, 587, 587, 587, 587, 587, 587, 541, 587, 563, 541, 541, 563, 563, 587, 587, 587, 2921, 587, 587, 2924, 587, 587, 587, 587, 587, 587, 587, 587, 587, 587, 587, 2666, 587, 587, 587, 587, 0, 0, 0, 0, 0, 2972, 2973, 0, 0, 0, 0, 2978, 0, 0, 0, 0, 0, 0, 0, 69632, 73728, 221184, 0, 0, 0, 0, 65536, 0, 3021, 0, 0, 3023, 0, 0, 541, 541, 3028, 541, 541, 541, 541, 541, 541, 541, 541, 2205, 2206, 541, 541, 541, 541, 541, 541, 0, 0, 3160, 0, 3162, 0, 0, 0, 3166, 0, 0, 0, 0, 0, 0, 0, 0, 2518, 0, 0, 0, 0, 0, 0, 541, 541, 541, 541, 3030, 541, 541, 541, 541, 3034, 3234, 3235, 563, 563, 563, 563, 3239, 3240, 563, 563, 563, 563, 563, 3244, 563, 563, 563, 563, 1445, 563, 563, 563, 563, 26028, 1360, 988, 587, 1455, 587, 587, 0, 3632, 0, 0, 0, 0, 541, 541, 541, 541, 541, 3641, 541, 563, 563, 563, 563, 587, 587, 587, 587, 587, 587, 587, 3263, 3264, 587, 587, 587, 587, 587, 587, 3435, 587, 587, 587, 587, 587, 587, 587, 541, 563, 3268, 3269, 587, 587, 587, 587, 587, 3273, 587, 587, 587, 587, 587, 587, 587, 587, 2393, 587, 587, 587, 587, 587, 587, 587, 3279, 587, 587, 587, 587, 587, 587, 587, 587, 587, 587, 587, 587, 541, 587, 563, 2721, 541, 0, 0, 0, 3312, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1633, 0, 0, 0, 541, 3404, 541, 541, 541, 541, 541, 541, 563, 563, 563, 563, 563, 563, 563, 563, 563, 563, 563, 563, 563, 563, 3419, 563, 563, 563, 563, 563, 563, 563, 587, 587, 587, 587, 587, 3113, 587, 587, 587, 587, 587, 0, 3443, 0, 0, 3446, 0, 0, 0, 3180, 0, 0, 0, 0, 0, 0, 0, 1583, 0, 1585, 0, 0, 0, 0, 0, 0, 0, 1106, 0, 0, 0, 0, 0, 0, 0, 0, 0, 756, 0, 0, 0, 0, 0, 0, 0, 0, 541, 541, 3458, 541, 541, 541, 541, 3463, 541, 541, 541, 541, 3468, 541, 541, 541, 541, 872, 541, 541, 541, 541, 541, 541, 541, 541, 541, 541, 541, 541, 1316, 541, 541, 541, 563, 587, 587, 3489, 587, 587, 587, 587, 3494, 587, 587, 587, 587, 3499, 587, 587, 587, 587, 587, 3364, 587, 587, 587, 587, 587, 587, 587, 587, 587, 587, 2664, 587, 587, 2667, 2668, 587, 587, 563, 563, 3645, 3646, 563, 563, 563, 587, 587, 587, 3651, 3652, 587, 587, 587, 0, 541, 587, 563, 541, 541, 541, 541, 541, 563, 563, 563, 563, 587, 587, 587, 587, 0, 0, 0, 2730, 0, 0, 563, 587, 0, 541, 563, 587, 3691, 3692, 3693, 3694, 0, 541, 563, 587, 0, 0, 0, 0, 0, 0, 0, 0, 3180, 0, 0, 0, 0, 0, 3453, 342, 343, 344, 345, 346, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 713, 0, 0, 0, 0, 0, 0, 0, 391, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1674, 0, 0, 0, 345, 345, 0, 346, 345, 0, 344, 345, 455, 472, 472, 472, 472, 472, 472, 472, 472, 472, 472, 472, 472, 472, 472, 472, 472, 472, 472, 472, 487, 496, 496, 504, 496, 506, 496, 496, 506, 506, 496, 506, 521, 521, 521, 521, 521, 521, 521, 521, 521, 521, 521, 521, 521, 521, 521, 521, 554, 577, 554, 577, 554, 554, 577, 554, 601, 577, 577, 577, 577, 577, 577, 577, 601, 601, 601, 554, 601, 601, 601, 601, 601, 601, 601, 601, 577, 577, 554, 577, 601, 577, 601, 1, 656, 657, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1590, 1591, 672, 673, 0, 675, 676, 0, 0, 0, 0, 0, 682, 0, 0, 0, 0, 0, 0, 0, 2804, 0, 0, 0, 541, 2808, 541, 541, 541, 0, 0, 0, 0, 692, 0, 0, 0, 0, 0, 0, 699, 365, 365, 365, 0, 0, 0, 0, 0, 1658, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 3180, 0, 0, 0, 3308, 0, 719, 0, 721, 0, 723, 0, 0, 0, 0, 0, 0, 0, 0, 0, 733, 0, 0, 0, 0, 646, 752, 753, 754, 0, 0, 0, 0, 0, 760, 761, 0, 0, 0, 0, 0, 1680, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 3180, 0, 3306, 0, 0, 763, 764, 675, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 779, 780, 0, 0, 0, 0, 0, 1696, 1697, 1698, 1699, 0, 0, 0, 0, 0, 0, 0, 0, 755, 0, 0, 0, 0, 0, 0, 0, 0, 772, 0, 0, 0, 0, 0, 0, 0, 0, 782, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1601, 0, 0, 761, 0, 0, 787, 0, 0, 0, 0, 0, 0, 0, 0, 0, 794, 675, 723, 0, 692, 811, 0, 0, 0, 0, 761, 0, 0, 0, 0, 0, 0, 0, 1591, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1662, 0, 0, 0, 0, 0, 0, 0, 0, 754, 820, 821, 0, 0, 0, 0, 0, 0, 754, 0, 0, 825, 699, 0, 0, 0, 830, 0, 0, 0, 832, 0, 0, 0, 692, 699, 0, 0, 692, 830, 830, 0, 0, 0, 0, 0, 0, 0, 0, 0, 692, 541, 541, 851, 855, 858, 541, 866, 541, 541, 541, 881, 883, 886, 541, 541, 541, 898, 902, 541, 541, 541, 541, 541, 2541, 541, 541, 541, 541, 541, 541, 541, 541, 541, 541, 541, 541, 3044, 541, 541, 541, 563, 563, 563, 971, 975, 563, 563, 563, 563, 0, 587, 587, 587, 998, 1002, 1005, 587, 1013, 587, 587, 587, 1028, 1030, 1033, 587, 587, 587, 1045, 1049, 587, 587, 587, 587, 587, 1995, 587, 587, 587, 587, 587, 2002, 587, 587, 587, 587, 587, 587, 2946, 587, 587, 587, 587, 587, 587, 541, 587, 563, 1073, 0, 998, 587, 1045, 1049, 1078, 78115, 1079, 0, 0, 0, 0, 0, 0, 0, 0, 2779, 0, 0, 0, 0, 0, 0, 0, 0, 303, 303, 303, 303, 0, 303, 303, 303, 0, 0, 0, 0, 1168, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 729, 0, 0, 0, 0, 0, 0, 1199, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1209, 0, 0, 0, 0, 0, 0, 2912256, 0, 3207168, 2465792, 0, 0, 2719744, 0, 0, 0, 0, 0, 0, 541, 541, 541, 3397, 541, 541, 541, 541, 3401, 541, 0, 0, 1214, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1634, 0, 0, 0, 0, 1254, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1665, 0, 0, 1321, 541, 541, 541, 541, 541, 541, 541, 541, 541, 541, 1331, 541, 541, 1338, 541, 541, 541, 541, 873, 541, 541, 541, 541, 541, 541, 541, 541, 541, 541, 541, 541, 1332, 541, 541, 541, 563, 563, 1443, 563, 563, 563, 563, 563, 563, 26028, 1360, 988, 587, 587, 587, 587, 587, 587, 3548, 587, 3550, 587, 587, 3552, 587, 587, 0, 0, 0, 0, 0, 0, 541, 541, 541, 3639, 3640, 541, 541, 563, 1501, 587, 587, 1505, 587, 587, 587, 587, 587, 587, 587, 587, 587, 587, 1515, 587, 0, 0, 0, 541, 587, 563, 541, 541, 899, 541, 541, 563, 563, 972, 563, 0, 1074, 587, 587, 1052, 587, 78115, 1079, 0, 0, 0, 0, 0, 0, 0, 0, 2445312, 0, 2842624, 0, 0, 0, 2637824, 0, 587, 1522, 587, 587, 587, 587, 587, 587, 587, 587, 587, 587, 1535, 587, 587, 587, 587, 587, 2013, 587, 541, 587, 563, 541, 541, 541, 541, 563, 563, 563, 563, 587, 587, 587, 587, 0, 0, 0, 0, 0, 0, 1666, 0, 1668, 0, 0, 0, 1670, 0, 748, 0, 0, 0, 0, 0, 0, 0, 0, 3174, 0, 0, 0, 0, 0, 0, 0, 0, 791, 0, 0, 0, 0, 0, 0, 0, 0, 833, 0, 0, 0, 0, 0, 0, 808, 0, 0, 1693, 0, 0, 0, 0, 0, 0, 1700, 0, 0, 0, 0, 0, 0, 0, 1660, 1661, 0, 0, 0, 0, 0, 0, 0, 0, 710, 0, 0, 0, 0, 0, 0, 0, 0, 726, 0, 0, 0, 0, 0, 0, 0, 0, 0, 274432, 274432, 274432, 0, 274432, 274432, 274432, 541, 541, 1762, 541, 541, 541, 541, 541, 541, 541, 541, 1769, 541, 541, 541, 1773, 541, 541, 1778, 541, 541, 541, 541, 541, 541, 541, 541, 541, 541, 541, 541, 541, 1788, 541, 541, 0, 563, 1828, 563, 563, 563, 563, 563, 563, 563, 1835, 563, 563, 563, 563, 563, 563, 3250, 563, 563, 563, 563, 563, 563, 563, 563, 563, 2321, 563, 563, 563, 563, 0, 0, 563, 1908, 563, 563, 563, 563, 563, 563, 563, 563, 563, 26028, 0, 587, 1924, 587, 0, 0, 0, 541, 587, 563, 541, 541, 900, 541, 541, 563, 563, 973, 563, 563, 563, 563, 2262, 563, 563, 563, 563, 563, 563, 563, 563, 563, 563, 563, 1434, 563, 563, 563, 563, 587, 587, 587, 1960, 587, 587, 587, 1964, 587, 587, 1969, 587, 587, 587, 587, 587, 587, 2688, 587, 587, 587, 587, 587, 587, 2695, 587, 587, 563, 563, 587, 587, 587, 587, 0, 0, 0, 2031, 0, 1082, 0, 0, 0, 2033, 0, 1086, 0, 0, 0, 2035, 0, 1090, 0, 0, 0, 2037, 0, 1094, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1620, 2587, 0, 1825, 563, 563, 563, 563, 563, 563, 563, 563, 563, 563, 563, 563, 563, 563, 563, 2610, 2650, 563, 563, 563, 0, 0, 0, 2653, 0, 1921, 587, 587, 587, 587, 587, 587, 587, 3549, 587, 587, 587, 587, 587, 587, 0, 0, 3672, 0, 541, 541, 541, 541, 563, 563, 0, 0, 2969, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 2102, 0, 0, 0, 0, 0, 3161, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1703, 0, 0, 0, 0, 0, 3319, 541, 541, 541, 541, 541, 541, 541, 3324, 541, 541, 541, 541, 541, 541, 541, 2832, 541, 541, 541, 541, 541, 541, 541, 541, 541, 2573, 541, 541, 541, 541, 541, 541, 587, 587, 3361, 587, 587, 587, 587, 587, 587, 587, 587, 587, 587, 587, 587, 587, 2709, 587, 587, 3379, 0, 0, 3382, 0, 0, 0, 0, 3180, 3387, 0, 0, 0, 0, 0, 0, 0, 1682, 0, 1684, 0, 0, 0, 0, 0, 0, 0, 1114, 0, 0, 0, 0, 1118, 0, 0, 1121, 541, 541, 541, 541, 3407, 541, 541, 541, 563, 563, 563, 3412, 563, 563, 563, 563, 563, 2289, 563, 563, 563, 563, 563, 563, 563, 563, 563, 563, 2605, 563, 563, 563, 563, 563, 3416, 563, 563, 563, 563, 563, 3422, 563, 563, 563, 563, 587, 587, 587, 3428, 587, 0, 0, 0, 541, 587, 563, 850, 541, 1066, 901, 541, 923, 563, 1071, 974, 587, 587, 587, 3432, 587, 587, 587, 587, 587, 3438, 587, 587, 587, 587, 541, 563, 563, 563, 563, 2276, 563, 563, 563, 563, 563, 563, 2282, 563, 2284, 563, 2287, 563, 587, 587, 587, 587, 587, 3492, 587, 587, 587, 3496, 587, 587, 587, 587, 587, 587, 2704, 587, 587, 587, 587, 587, 587, 587, 587, 587, 1470, 587, 587, 587, 587, 587, 587, 0, 0, 3657, 0, 541, 541, 541, 541, 541, 541, 563, 563, 563, 563, 563, 563, 936, 563, 563, 563, 563, 563, 563, 563, 563, 587, 3687, 3688, 3689, 3690, 0, 541, 563, 587, 0, 541, 563, 587, 0, 0, 0, 0, 0, 0, 0, 0, 3180, 0, 0, 3450, 3451, 0, 0, 389, 0, 0, 0, 393, 389, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 122880, 0, 0, 0, 0, 0, 0, 0, 405, 0, 347, 0, 69632, 73728, 0, 0, 0, 0, 0, 65536, 0, 0, 0, 0, 0, 1709, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 399, 0, 0, 0, 0, 0, 0, 0, 436, 0, 0, 443, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 131072, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 522, 522, 522, 522, 522, 0, 0, 0, 0, 0, 0, 0, 0, 0, 522, 522, 522, 522, 522, 522, 522, 522, 522, 555, 578, 555, 578, 555, 555, 578, 555, 602, 578, 578, 578, 578, 578, 578, 578, 602, 602, 602, 555, 602, 602, 602, 602, 602, 602, 602, 602, 578, 578, 617, 622, 602, 622, 628, 1, 0, 0, 0, 751, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 2061, 0, 0, 0, 962, 563, 563, 563, 563, 980, 563, 563, 563, 0, 587, 587, 992, 587, 587, 587, 587, 587, 2350, 587, 587, 587, 587, 587, 587, 587, 587, 587, 587, 1936, 587, 587, 587, 587, 587, 1107, 0, 0, 0, 0, 1112, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 167936, 167936, 167936, 167936, 167936, 167936, 167936, 1360, 914, 563, 563, 563, 1366, 563, 563, 563, 563, 563, 563, 563, 563, 563, 1380, 563, 563, 1404, 563, 563, 563, 563, 563, 563, 563, 563, 563, 563, 563, 563, 563, 2620, 563, 563, 563, 1441, 563, 563, 563, 563, 563, 563, 563, 26028, 1360, 988, 587, 587, 587, 1458, 563, 587, 587, 587, 587, 587, 1079, 0, 1562, 0, 0, 0, 1568, 0, 0, 0, 0, 0, 0, 1627, 1628, 1629, 0, 0, 0, 0, 0, 0, 0, 0, 343, 0, 0, 0, 0, 0, 0, 0, 0, 403, 0, 0, 0, 0, 0, 0, 0, 0, 407, 407, 407, 407, 0, 407, 407, 407, 1574, 0, 0, 0, 1580, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 808, 541, 541, 852, 541, 1605, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1619, 0, 0, 0, 0, 0, 1723, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 2625536, 0, 2699264, 2715648, 0, 1973, 587, 587, 587, 587, 587, 587, 587, 587, 587, 587, 587, 587, 587, 587, 587, 2373, 563, 563, 587, 587, 587, 587, 0, 0, 1563, 0, 0, 0, 0, 0, 1569, 0, 1575, 0, 1581, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1262, 0, 0, 0, 0, 0, 0, 0, 0, 1575, 0, 0, 0, 0, 0, 1581, 0, 0, 0, 0, 0, 0, 0, 2974, 0, 0, 0, 0, 0, 0, 0, 0, 0, 365, 0, 253952, 0, 0, 0, 0, 541, 541, 541, 2190, 541, 541, 541, 541, 2194, 541, 2196, 541, 541, 541, 541, 541, 541, 541, 3039, 541, 541, 541, 541, 541, 541, 541, 541, 541, 887, 541, 541, 541, 541, 541, 541, 563, 563, 2275, 563, 563, 563, 563, 2279, 563, 2281, 563, 563, 563, 563, 563, 563, 1896, 563, 563, 563, 563, 563, 563, 563, 563, 563, 1397, 1399, 563, 563, 563, 563, 563, 587, 587, 2362, 587, 587, 587, 587, 2366, 587, 2368, 587, 587, 587, 587, 587, 587, 1465, 587, 587, 587, 587, 587, 587, 1480, 587, 587, 0, 0, 0, 0, 0, 2489, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 196608, 0, 0, 0, 0, 563, 563, 2625, 563, 563, 563, 563, 563, 563, 563, 563, 563, 563, 563, 563, 563, 2648, 563, 563, 0, 0, 0, 0, 3024, 0, 541, 541, 541, 541, 541, 541, 541, 541, 541, 541, 541, 1330, 541, 541, 541, 541, 0, 3380, 0, 0, 0, 0, 0, 0, 3180, 0, 0, 0, 0, 3391, 0, 0, 0, 0, 0, 328, 0, 69632, 73728, 0, 0, 0, 0, 0, 65536, 0, 3393, 0, 0, 0, 0, 0, 541, 3396, 541, 541, 3398, 541, 541, 541, 541, 541, 541, 541, 3040, 541, 541, 541, 541, 541, 541, 541, 541, 541, 1783, 541, 541, 541, 541, 1789, 541, 0, 3455, 541, 541, 541, 541, 541, 541, 541, 541, 541, 541, 541, 541, 541, 541, 563, 563, 563, 541, 862, 541, 541, 541, 541, 541, 541, 541, 541, 541, 541, 541, 541, 541, 541, 1319, 541, 541, 912, 563, 563, 563, 563, 563, 563, 935, 563, 563, 563, 563, 563, 563, 563, 2291, 2292, 563, 563, 563, 563, 563, 563, 563, 2319, 563, 563, 2322, 2323, 563, 563, 0, 0, 1009, 587, 587, 587, 587, 587, 587, 587, 587, 587, 587, 587, 587, 587, 587, 587, 2697, 1059, 0, 0, 0, 541, 587, 563, 541, 541, 541, 541, 541, 563, 563, 563, 563, 563, 2614, 563, 563, 563, 563, 563, 563, 563, 563, 563, 563, 3649, 587, 587, 587, 587, 587, 587, 587, 0, 1095, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 2513, 0, 0, 1623, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 2130, 0, 2132, 1826, 563, 563, 563, 563, 563, 563, 563, 563, 563, 563, 563, 563, 563, 563, 563, 3258, 541, 3215, 541, 541, 541, 541, 541, 541, 541, 541, 541, 541, 541, 541, 541, 541, 1337, 541, 587, 587, 587, 3271, 587, 587, 587, 587, 587, 587, 587, 587, 587, 587, 587, 587, 2370, 587, 587, 587, 0, 0, 0, 0, 348, 348, 350, 348, 348, 348, 348, 348, 348, 513, 348, 348, 0, 350, 348, 0, 0, 348, 348, 0, 0, 0, 0, 349, 0, 0, 0, 0, 0, 329, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 2509, 0, 0, 0, 0, 348, 348, 348, 348, 348, 348, 348, 348, 348, 348, 348, 348, 348, 348, 348, 348, 556, 579, 556, 579, 556, 556, 579, 556, 603, 579, 579, 579, 579, 579, 579, 579, 603, 603, 603, 556, 603, 603, 603, 603, 603, 603, 603, 603, 579, 579, 556, 579, 603, 579, 603, 1, 0, 0, 0, 0, 722, 0, 724, 0, 0, 0, 0, 0, 0, 0, 0, 0, 299, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 739, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 836, 541, 541, 541, 541, 0, 0, 0, 766, 0, 0, 0, 771, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1189, 0, 0, 0, 0, 0, 0, 541, 541, 541, 870, 541, 541, 541, 541, 541, 890, 892, 541, 541, 903, 541, 541, 541, 541, 541, 2569, 541, 541, 541, 541, 541, 541, 541, 541, 541, 541, 541, 541, 2835, 541, 541, 541, 963, 965, 563, 563, 976, 563, 563, 563, 563, 0, 587, 587, 993, 999, 587, 587, 587, 587, 587, 3374, 587, 541, 541, 563, 563, 587, 587, 0, 0, 0, 0, 0, 0, 3299, 0, 0, 0, 587, 587, 1017, 587, 587, 587, 587, 587, 1037, 1039, 587, 587, 1050, 587, 587, 587, 587, 587, 2364, 587, 587, 2367, 587, 587, 587, 587, 587, 587, 587, 2677, 587, 587, 587, 587, 587, 587, 2682, 587, 0, 0, 0, 1151, 0, 1153, 0, 1155, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1231, 541, 541, 541, 541, 541, 541, 0, 1181, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1636, 0, 0, 1198, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1675, 0, 0, 1266, 0, 0, 0, 0, 0, 0, 0, 1181, 541, 541, 541, 541, 541, 541, 541, 1326, 541, 541, 541, 541, 541, 541, 541, 1339, 1305, 1307, 541, 541, 541, 541, 541, 541, 541, 541, 541, 541, 541, 541, 541, 541, 1358, 541, 541, 541, 1343, 541, 541, 541, 541, 541, 541, 541, 541, 1354, 1355, 541, 541, 541, 541, 541, 541, 2840, 541, 541, 541, 541, 541, 541, 541, 541, 541, 541, 3056, 541, 3058, 541, 541, 541, 1360, 914, 563, 563, 563, 563, 1367, 563, 563, 563, 563, 563, 563, 563, 563, 563, 3242, 563, 563, 563, 563, 563, 563, 563, 563, 563, 1421, 563, 563, 563, 563, 563, 563, 563, 1435, 563, 563, 563, 563, 563, 2627, 563, 563, 563, 563, 563, 563, 2635, 563, 563, 563, 1459, 587, 587, 587, 587, 587, 587, 587, 587, 587, 587, 587, 587, 587, 587, 587, 2942, 587, 587, 587, 1489, 1491, 587, 587, 587, 587, 587, 587, 587, 587, 587, 587, 587, 2707, 2708, 587, 587, 587, 1554, 587, 1555, 587, 587, 1558, 1079, 0, 0, 0, 0, 0, 0, 0, 0, 0, 306, 204800, 204800, 0, 205106, 204800, 1, 0, 1677, 0, 0, 1679, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1159, 0, 0, 0, 0, 541, 541, 541, 541, 1763, 541, 541, 541, 541, 541, 541, 541, 541, 541, 541, 541, 541, 2208, 541, 541, 541, 0, 563, 563, 563, 563, 563, 563, 563, 1833, 563, 563, 563, 563, 563, 563, 563, 2879, 563, 563, 563, 563, 563, 563, 563, 563, 2644, 563, 563, 563, 2647, 563, 563, 563, 1858, 563, 563, 563, 563, 563, 563, 563, 563, 563, 563, 563, 563, 563, 563, 563, 1857, 563, 563, 563, 1909, 563, 563, 563, 563, 1915, 563, 563, 563, 26028, 0, 587, 587, 587, 587, 587, 2376, 587, 587, 587, 587, 587, 587, 587, 587, 587, 587, 2717, 541, 587, 563, 541, 541, 587, 587, 587, 587, 1929, 587, 587, 587, 587, 587, 587, 587, 587, 587, 587, 587, 2938, 587, 587, 587, 587, 587, 1991, 587, 587, 587, 1996, 587, 587, 587, 587, 587, 587, 587, 587, 2005, 587, 0, 0, 0, 541, 587, 563, 852, 892, 541, 903, 541, 925, 965, 563, 976, 587, 587, 587, 2011, 587, 587, 587, 541, 587, 563, 541, 541, 541, 541, 563, 563, 916, 563, 563, 563, 932, 563, 563, 563, 563, 563, 563, 563, 1864, 563, 563, 563, 1868, 563, 563, 1873, 563, 2077, 0, 0, 0, 0, 0, 0, 0, 0, 2085, 0, 0, 0, 0, 0, 2089, 0, 0, 2091, 0, 0, 0, 0, 0, 0, 0, 2099, 0, 0, 0, 0, 0, 0, 0, 3012, 3013, 0, 0, 0, 0, 3018, 0, 0, 2133, 2134, 0, 2136, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 2144, 0, 0, 2149, 2150, 0, 0, 0, 0, 0, 2154, 0, 0, 0, 0, 0, 0, 0, 2160, 541, 541, 541, 541, 541, 541, 2167, 541, 541, 541, 541, 541, 541, 1288, 541, 541, 541, 541, 1298, 541, 541, 541, 541, 541, 541, 1325, 541, 541, 541, 541, 541, 541, 541, 541, 541, 541, 1801, 541, 541, 541, 541, 541, 2202, 541, 541, 541, 541, 541, 541, 541, 541, 541, 541, 541, 541, 2209, 541, 541, 541, 541, 541, 2815, 541, 2818, 541, 541, 541, 541, 2823, 541, 541, 541, 541, 541, 541, 3409, 541, 563, 563, 563, 563, 563, 563, 563, 563, 3479, 563, 3481, 3482, 563, 3484, 563, 541, 541, 541, 2215, 541, 541, 541, 541, 541, 541, 541, 541, 541, 541, 541, 541, 895, 541, 541, 541, 0, 0, 2244, 563, 563, 563, 563, 563, 563, 563, 2252, 563, 563, 563, 563, 563, 563, 3534, 563, 3536, 563, 563, 3538, 563, 563, 587, 587, 563, 563, 2301, 563, 563, 563, 563, 563, 563, 563, 563, 563, 563, 563, 563, 563, 2893, 563, 563, 0, 0, 2331, 587, 587, 587, 587, 587, 587, 587, 2339, 587, 587, 587, 587, 587, 587, 2715, 587, 587, 587, 587, 541, 587, 563, 541, 541, 587, 587, 587, 587, 2363, 587, 587, 587, 587, 587, 587, 2369, 587, 2371, 587, 2374, 587, 587, 2388, 587, 587, 587, 587, 587, 587, 587, 587, 587, 587, 587, 587, 587, 2918, 587, 587, 0, 2474, 0, 0, 0, 0, 0, 0, 0, 0, 0, 2482, 0, 0, 0, 0, 0, 0, 0, 122880, 122880, 122880, 122880, 122880, 0, 122880, 0, 2105632, 0, 0, 2514, 0, 0, 0, 0, 0, 0, 0, 0, 0, 2522, 0, 0, 541, 541, 541, 541, 541, 3461, 541, 541, 541, 3465, 541, 541, 541, 541, 541, 541, 1287, 541, 541, 541, 541, 541, 541, 541, 541, 541, 541, 2546, 541, 541, 541, 541, 541, 541, 541, 541, 541, 2540, 541, 541, 541, 541, 541, 541, 541, 541, 541, 541, 541, 541, 2222, 541, 541, 541, 563, 563, 563, 2599, 563, 563, 563, 563, 563, 2604, 563, 563, 563, 563, 563, 563, 2278, 563, 563, 563, 563, 563, 563, 563, 563, 563, 2632, 563, 563, 563, 563, 563, 563, 2670, 587, 587, 587, 587, 587, 587, 587, 587, 587, 587, 587, 587, 587, 587, 587, 3143, 587, 587, 587, 587, 2686, 587, 587, 587, 587, 587, 587, 587, 587, 587, 587, 587, 3125, 587, 587, 587, 587, 541, 541, 2723, 563, 563, 563, 2725, 587, 587, 587, 2727, 2728, 0, 0, 0, 0, 0, 0, 0, 172032, 172032, 172032, 172032, 172032, 172032, 172032, 172032, 1, 0, 0, 2746, 0, 2747, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1175, 0, 0, 0, 0, 0, 0, 0, 0, 2762, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1192, 0, 1195, 0, 0, 2811, 541, 2812, 541, 541, 541, 541, 541, 541, 541, 541, 541, 541, 541, 541, 541, 1804, 541, 541, 541, 541, 541, 2849, 541, 541, 541, 541, 541, 0, 0, 563, 563, 563, 563, 563, 563, 563, 563, 563, 563, 563, 563, 2255, 563, 2858, 563, 2859, 563, 563, 563, 563, 563, 563, 563, 563, 563, 563, 563, 563, 563, 3086, 563, 563, 587, 587, 2907, 587, 2908, 587, 587, 587, 587, 587, 587, 587, 587, 587, 587, 587, 3139, 587, 3141, 587, 587, 0, 0, 0, 2983, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 2073, 0, 0, 0, 0, 0, 3381, 0, 0, 0, 0, 0, 3180, 0, 0, 0, 3390, 0, 0, 0, 0, 0, 0, 1659, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 114688, 0, 241664, 258048, 0, 0, 3430, 587, 587, 587, 587, 587, 587, 587, 587, 587, 3439, 587, 587, 587, 541, 563, 563, 563, 563, 2303, 563, 563, 563, 563, 563, 563, 563, 563, 563, 563, 563, 26028, 0, 587, 587, 587, 541, 541, 541, 541, 3518, 541, 541, 541, 541, 541, 541, 541, 541, 541, 563, 563, 563, 3232, 563, 563, 563, 563, 563, 563, 3531, 563, 563, 563, 563, 563, 563, 563, 563, 563, 563, 587, 587, 587, 3357, 587, 587, 587, 587, 587, 3545, 587, 587, 587, 587, 587, 587, 587, 587, 587, 587, 0, 0, 0, 0, 0, 0, 3637, 541, 541, 541, 541, 541, 541, 3643, 563, 563, 563, 563, 3647, 563, 563, 587, 587, 587, 587, 587, 3653, 587, 587, 0, 0, 0, 0, 3635, 0, 541, 541, 541, 541, 541, 541, 541, 563, 2724, 563, 563, 587, 2726, 587, 587, 0, 0, 0, 0, 0, 0, 0, 0, 3607, 0, 541, 541, 541, 541, 0, 3656, 0, 3658, 541, 541, 541, 541, 541, 541, 563, 563, 563, 563, 563, 563, 937, 563, 563, 563, 563, 563, 563, 563, 0, 0, 0, 0, 394, 395, 0, 396, 0, 0, 0, 0, 0, 396, 0, 0, 0, 0, 0, 331, 332, 333, 0, 0, 0, 0, 0, 0, 0, 0, 0, 560, 583, 560, 583, 560, 560, 583, 401, 0, 0, 0, 0, 0, 368, 376, 404, 0, 0, 0, 0, 0, 368, 0, 0, 396, 0, 0, 0, 0, 0, 351, 0, 0, 368, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 2744, 409, 411, 0, 0, 368, 376, 0, 69632, 73728, 0, 0, 0, 0, 427, 65536, 0, 0, 0, 0, 0, 2044, 2045, 0, 0, 0, 0, 0, 0, 0, 0, 0, 361, 0, 0, 0, 0, 0, 361, 427, 427, 437, 0, 427, 0, 411, 427, 456, 0, 0, 0, 0, 0, 0, 0, 0, 106496, 0, 106496, 0, 0, 0, 0, 106496, 396, 0, 409, 0, 497, 497, 0, 497, 497, 497, 497, 497, 497, 497, 497, 523, 523, 523, 523, 523, 456, 456, 456, 456, 531, 456, 456, 532, 456, 523, 537, 523, 523, 523, 537, 523, 523, 523, 523, 539, 557, 580, 557, 580, 557, 557, 580, 557, 604, 580, 580, 580, 580, 580, 580, 580, 604, 604, 604, 557, 604, 604, 604, 604, 604, 604, 604, 604, 580, 580, 618, 623, 604, 623, 629, 1, 0, 0, 0, 643, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 2101, 0, 0, 0, 783, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 2757, 0, 0, 0, 802, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 2129, 0, 0, 0, 587, 587, 587, 1023, 1025, 587, 587, 587, 587, 587, 587, 587, 587, 587, 587, 587, 3275, 587, 587, 587, 587, 0, 0, 0, 0, 1096, 0, 0, 1098, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1630, 0, 0, 0, 0, 0, 1636, 0, 0, 0, 0, 1111, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1207, 0, 0, 0, 0, 1098, 1227, 0, 0, 0, 0, 0, 0, 1234, 0, 0, 0, 0, 0, 0, 0, 0, 155648, 0, 0, 0, 0, 0, 0, 0, 0, 1239, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1239, 1248, 0, 0, 0, 0, 0, 0, 1681, 0, 0, 0, 0, 0, 0, 1687, 0, 1689, 0, 0, 0, 0, 1239, 1269, 1270, 0, 1239, 0, 541, 541, 541, 541, 541, 541, 541, 3207, 3208, 541, 541, 541, 541, 3212, 3213, 1278, 541, 541, 541, 1282, 541, 541, 1293, 541, 541, 1297, 541, 541, 541, 541, 541, 541, 541, 3054, 541, 541, 541, 541, 541, 541, 541, 541, 541, 1784, 541, 541, 541, 541, 541, 541, 1360, 914, 563, 563, 563, 563, 563, 563, 563, 1370, 563, 563, 563, 1374, 563, 563, 563, 563, 1847, 563, 563, 563, 563, 563, 563, 563, 563, 563, 563, 563, 2308, 563, 563, 563, 563, 1385, 563, 563, 1389, 563, 563, 563, 563, 563, 563, 563, 563, 563, 563, 563, 563, 2295, 563, 563, 563, 563, 563, 563, 1406, 563, 563, 563, 1410, 563, 563, 563, 563, 563, 563, 563, 563, 2901, 563, 563, 0, 0, 587, 587, 587, 587, 587, 587, 1462, 587, 587, 587, 1466, 587, 587, 1477, 587, 587, 1481, 587, 587, 587, 587, 587, 3434, 587, 3436, 587, 587, 587, 587, 587, 587, 541, 563, 587, 1502, 587, 587, 587, 587, 587, 587, 587, 587, 587, 587, 587, 587, 587, 587, 3128, 587, 0, 1622, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1635, 0, 0, 0, 0, 0, 2067, 0, 0, 0, 0, 0, 0, 0, 0, 0, 2076, 1637, 0, 0, 0, 1641, 0, 0, 0, 1645, 0, 0, 0, 1649, 0, 0, 0, 0, 0, 0, 2056, 0, 2057, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1676, 0, 563, 563, 1844, 563, 563, 563, 563, 563, 563, 563, 563, 563, 563, 563, 563, 563, 3540, 587, 587, 1958, 587, 587, 587, 587, 587, 587, 587, 587, 587, 587, 587, 587, 587, 587, 587, 3127, 587, 3129, 587, 587, 2010, 587, 587, 587, 587, 1749, 1940, 1844, 541, 541, 541, 541, 563, 563, 917, 563, 563, 563, 934, 563, 563, 948, 563, 563, 563, 563, 563, 1895, 563, 563, 563, 1900, 563, 563, 563, 563, 563, 563, 1914, 563, 563, 563, 563, 26028, 0, 587, 587, 587, 0, 2118, 0, 0, 0, 0, 0, 0, 2125, 0, 0, 0, 0, 0, 0, 0, 0, 184726, 184932, 184932, 184932, 0, 184932, 184932, 184932, 0, 0, 2159, 0, 541, 541, 2162, 541, 541, 541, 541, 541, 541, 541, 541, 541, 541, 2833, 541, 541, 541, 541, 541, 541, 541, 2227, 541, 541, 541, 541, 541, 541, 541, 541, 541, 541, 541, 0, 0, 563, 2855, 563, 563, 563, 0, 0, 563, 563, 2246, 563, 563, 563, 563, 563, 563, 563, 563, 563, 563, 563, 3255, 563, 563, 563, 563, 563, 563, 2314, 563, 563, 563, 563, 563, 563, 563, 563, 563, 563, 563, 0, 0, 587, 2904, 587, 0, 0, 587, 587, 2333, 587, 587, 587, 587, 587, 587, 587, 587, 587, 587, 587, 3440, 587, 587, 541, 563, 587, 587, 587, 587, 2390, 587, 587, 587, 587, 587, 587, 587, 587, 587, 587, 587, 2928, 587, 587, 587, 587, 587, 587, 587, 587, 2401, 587, 587, 587, 587, 587, 587, 587, 587, 587, 587, 587, 541, 587, 0, 0, 0, 541, 587, 563, 1064, 541, 541, 905, 541, 1069, 563, 563, 978, 541, 541, 541, 2567, 541, 541, 541, 541, 541, 541, 541, 541, 541, 541, 541, 541, 900, 541, 541, 541, 2698, 587, 587, 587, 587, 587, 587, 587, 587, 587, 587, 587, 587, 587, 587, 587, 3277, 587, 587, 0, 2759, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1716, 0, 541, 2827, 541, 541, 541, 541, 541, 541, 541, 541, 541, 541, 541, 541, 541, 541, 1758, 541, 563, 2874, 563, 563, 563, 563, 563, 563, 563, 563, 563, 563, 563, 563, 563, 563, 2285, 563, 587, 587, 587, 2923, 587, 587, 587, 587, 587, 587, 587, 587, 587, 587, 587, 587, 2382, 587, 587, 587, 0, 0, 2993, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 2433, 0, 2435, 3171, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 2771, 3202, 541, 541, 541, 541, 541, 541, 541, 541, 541, 541, 541, 541, 541, 541, 541, 2577, 3225, 541, 541, 541, 541, 541, 541, 541, 541, 563, 563, 563, 563, 563, 563, 563, 2888, 563, 563, 563, 563, 563, 563, 563, 563, 3241, 563, 563, 563, 563, 563, 563, 563, 587, 587, 587, 3282, 587, 587, 587, 587, 587, 587, 587, 587, 587, 541, 587, 563, 2018, 541, 541, 541, 2022, 563, 541, 3289, 563, 3291, 587, 3293, 3294, 0, 0, 0, 0, 3298, 0, 0, 0, 0, 0, 0, 0, 1134592, 0, 365, 0, 0, 0, 1134592, 0, 0, 0, 1134592, 1134592, 0, 0, 1134592, 0, 0, 1134592, 0, 0, 0, 3311, 0, 0, 0, 0, 3315, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1725, 0, 0, 0, 0, 0, 0, 3318, 0, 541, 541, 541, 3320, 541, 541, 541, 3323, 541, 541, 541, 541, 541, 541, 541, 1751, 541, 541, 541, 541, 541, 541, 541, 541, 541, 2586, 0, 563, 563, 563, 563, 563, 3338, 563, 563, 563, 3341, 563, 563, 563, 563, 563, 563, 563, 563, 563, 563, 563, 26028, 1921, 587, 587, 587, 587, 3360, 587, 587, 587, 587, 587, 587, 587, 587, 587, 587, 587, 587, 587, 587, 3554, 0, 0, 3403, 541, 3405, 541, 541, 541, 541, 541, 563, 563, 563, 563, 563, 563, 563, 563, 563, 563, 3075, 563, 563, 3418, 563, 3420, 563, 563, 563, 563, 563, 563, 587, 587, 587, 587, 587, 587, 587, 3596, 587, 3598, 587, 587, 0, 0, 0, 0, 3560, 0, 0, 0, 0, 541, 541, 541, 541, 541, 541, 541, 541, 541, 2169, 541, 541, 541, 541, 541, 3574, 541, 541, 563, 563, 563, 563, 563, 563, 563, 563, 563, 563, 563, 563, 563, 563, 563, 563, 1417, 3586, 563, 563, 563, 587, 587, 587, 587, 587, 587, 587, 587, 587, 587, 3599, 587, 0, 0, 0, 853, 1000, 926, 849, 1065, 894, 541, 541, 922, 1070, 967, 563, 0, 587, 587, 587, 587, 587, 78115, 0, 0, 0, 0, 0, 0, 0, 0, 0, 365, 0, 305, 0, 0, 0, 0, 3611, 541, 541, 541, 541, 541, 541, 541, 563, 563, 3617, 563, 3618, 563, 563, 563, 563, 587, 587, 587, 587, 3594, 587, 587, 587, 587, 587, 587, 587, 541, 541, 563, 563, 587, 587, 3378, 0, 0, 410, 356, 0, 0, 0, 0, 0, 69632, 73728, 0, 0, 0, 0, 0, 65536, 0, 0, 0, 0, 0, 2122, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 304, 304, 0, 0, 0, 0, 0, 0, 354, 0, 0, 0, 356, 0, 0, 473, 473, 473, 473, 473, 473, 473, 478, 473, 473, 473, 473, 473, 473, 473, 473, 473, 473, 473, 478, 473, 484, 473, 0, 0, 473, 0, 0, 0, 0, 0, 0, 0, 0, 524, 528, 528, 528, 528, 473, 473, 473, 473, 473, 473, 473, 478, 473, 528, 524, 528, 528, 528, 524, 528, 528, 528, 528, 540, 558, 581, 558, 581, 558, 558, 581, 558, 605, 581, 581, 581, 581, 581, 581, 581, 605, 605, 605, 558, 605, 605, 605, 605, 605, 605, 605, 605, 581, 581, 619, 624, 605, 624, 630, 1, 0, 0, 0, 0, 644, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1606, 0, 0, 0, 0, 688, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 365, 365, 365, 0, 0, 0, 0, 0, 2138, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 2768, 0, 0, 0, 0, 0, 0, 0, 738, 0, 0, 0, 644, 738, 0, 744, 745, 644, 0, 0, 0, 0, 0, 0, 790, 0, 0, 0, 0, 0, 0, 0, 0, 0, 2058, 0, 0, 0, 0, 0, 0, 0, 0, 799, 0, 804, 0, 0, 0, 0, 0, 0, 804, 0, 0, 0, 0, 0, 644, 0, 0, 0, 799, 0, 804, 0, 790, 0, 819, 0, 0, 0, 665, 0, 0, 0, 0, 819, 0, 0, 0, 0, 0, 0, 0, 2473984, 2478080, 0, 0, 0, 0, 0, 0, 0, 0, 0, 2767, 0, 0, 0, 0, 0, 0, 0, 644, 0, 0, 0, 0, 0, 0, 0, 0, 790, 0, 0, 0, 0, 0, 0, 0, 3165, 0, 0, 0, 0, 0, 0, 0, 0, 0, 541, 563, 541, 563, 541, 541, 563, 0, 0, 790, 790, 0, 644, 0, 0, 790, 804, 842, 0, 541, 848, 541, 541, 541, 541, 541, 2839, 541, 541, 541, 2842, 541, 2844, 541, 541, 541, 541, 541, 541, 879, 541, 884, 541, 891, 541, 894, 541, 541, 908, 541, 541, 867, 871, 877, 541, 882, 541, 541, 541, 541, 541, 899, 541, 541, 541, 541, 541, 541, 2852, 541, 541, 0, 0, 563, 563, 563, 563, 563, 563, 563, 563, 563, 563, 563, 563, 563, 2256, 563, 563, 563, 972, 563, 563, 563, 563, 563, 0, 587, 587, 995, 587, 587, 587, 587, 587, 2687, 587, 587, 587, 587, 587, 587, 587, 587, 587, 587, 2937, 587, 587, 587, 587, 587, 587, 587, 1014, 1018, 1024, 587, 1029, 587, 587, 587, 587, 587, 1046, 587, 587, 587, 587, 587, 1026, 587, 1031, 587, 1038, 587, 1041, 587, 587, 1055, 587, 1149, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 2799, 0, 0, 0, 1200, 0, 0, 0, 0, 0, 1205, 0, 0, 0, 0, 0, 0, 0, 1714, 0, 0, 0, 0, 0, 0, 0, 0, 0, 2140, 2141, 0, 0, 2143, 0, 0, 1099, 0, 0, 0, 1230, 0, 1232, 0, 0, 0, 0, 0, 0, 0, 0, 0, 351, 352, 353, 0, 0, 0, 0, 1240, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1246, 0, 1249, 1200, 0, 0, 0, 0, 0, 2427, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 336, 337, 0, 0, 0, 0, 1230, 1252, 0, 1255, 0, 0, 0, 0, 0, 1130, 0, 0, 0, 0, 1264, 0, 0, 1149, 1264, 0, 1271, 541, 541, 541, 541, 1277, 1360, 914, 1362, 563, 563, 563, 563, 563, 1369, 563, 563, 563, 563, 1375, 563, 563, 563, 563, 1862, 563, 563, 563, 563, 563, 563, 563, 563, 563, 563, 563, 3084, 563, 563, 563, 563, 563, 563, 563, 1407, 563, 563, 563, 563, 1412, 563, 563, 563, 563, 563, 563, 563, 2900, 563, 563, 563, 0, 0, 587, 587, 587, 563, 1419, 563, 563, 563, 1426, 1428, 563, 563, 563, 563, 563, 563, 563, 563, 563, 3346, 563, 563, 563, 563, 563, 563, 587, 587, 1461, 587, 587, 587, 587, 1467, 587, 587, 587, 587, 587, 587, 587, 587, 2663, 587, 587, 587, 587, 587, 587, 587, 587, 587, 1488, 587, 587, 587, 587, 587, 587, 587, 587, 587, 587, 1499, 587, 587, 587, 587, 1463, 587, 587, 587, 587, 587, 1475, 587, 587, 587, 587, 587, 587, 2014, 541, 587, 563, 1791, 541, 2020, 541, 1886, 563, 587, 587, 1504, 587, 587, 587, 587, 587, 587, 587, 587, 1511, 587, 587, 587, 1518, 1520, 587, 587, 587, 587, 587, 587, 587, 587, 587, 587, 587, 587, 587, 587, 587, 2396, 2397, 2398, 587, 587, 1542, 587, 0, 541, 587, 563, 541, 541, 541, 541, 541, 563, 563, 563, 563, 563, 2862, 563, 2865, 563, 563, 563, 563, 2870, 563, 563, 563, 563, 587, 587, 587, 587, 587, 1079, 0, 0, 1563, 0, 0, 0, 1569, 0, 0, 0, 0, 0, 376, 0, 0, 0, 0, 368, 0, 385, 0, 351, 0, 0, 1575, 0, 0, 0, 1581, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 303, 303, 0, 0, 0, 0, 1592, 1593, 0, 0, 0, 0, 0, 1599, 0, 0, 0, 0, 0, 0, 0, 0, 0, 2098, 0, 2100, 0, 0, 0, 0, 0, 1638, 0, 0, 0, 0, 0, 0, 0, 1646, 1647, 0, 0, 0, 0, 0, 0, 0, 3197, 0, 0, 0, 0, 0, 0, 3200, 0, 0, 1654, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 2038, 0, 1690, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 3005, 0, 1706, 1707, 0, 0, 0, 1710, 0, 0, 0, 0, 0, 1714, 0, 0, 1717, 0, 0, 1730, 0, 1732, 1706, 541, 541, 541, 541, 541, 541, 541, 541, 541, 541, 541, 2547, 541, 541, 541, 541, 541, 1744, 541, 541, 541, 541, 1750, 541, 541, 541, 541, 541, 541, 541, 541, 541, 541, 3220, 541, 541, 541, 541, 541, 541, 1761, 541, 541, 541, 541, 1765, 541, 541, 541, 541, 541, 541, 541, 541, 541, 541, 3335, 541, 541, 563, 563, 563, 541, 1776, 541, 541, 541, 1781, 541, 541, 541, 541, 541, 541, 541, 541, 541, 541, 541, 2559, 541, 541, 541, 541, 1808, 541, 541, 541, 541, 541, 541, 541, 541, 541, 541, 541, 541, 541, 541, 1360, 914, 563, 563, 563, 563, 563, 563, 563, 563, 563, 563, 563, 563, 1376, 563, 0, 587, 587, 1046, 587, 587, 78115, 1079, 0, 0, 0, 0, 0, 0, 0, 0, 225890, 225890, 225890, 225890, 225741, 225890, 225890, 225890, 0, 563, 563, 563, 563, 563, 563, 563, 563, 563, 563, 563, 563, 1839, 563, 563, 563, 563, 1877, 563, 563, 563, 563, 563, 563, 563, 563, 563, 563, 563, 2869, 563, 563, 563, 563, 563, 563, 1845, 563, 563, 563, 563, 563, 563, 563, 563, 563, 563, 1856, 563, 563, 563, 563, 1894, 563, 563, 563, 1898, 563, 563, 563, 563, 563, 563, 563, 1450, 563, 26028, 1360, 988, 1454, 587, 587, 587, 563, 563, 1860, 563, 563, 563, 563, 563, 563, 563, 563, 563, 563, 1871, 563, 563, 563, 563, 1911, 1913, 563, 563, 563, 563, 563, 26028, 0, 587, 587, 587, 587, 587, 1506, 587, 587, 1508, 587, 587, 587, 1512, 587, 587, 587, 0, 1286, 1470, 1378, 541, 541, 541, 1549, 541, 563, 563, 563, 1553, 563, 1876, 563, 563, 563, 563, 563, 563, 563, 563, 563, 563, 563, 563, 563, 563, 2297, 2298, 563, 563, 587, 2027, 587, 587, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 3511, 0, 3513, 0, 541, 0, 0, 0, 2080, 2081, 0, 0, 2083, 2084, 0, 0, 0, 0, 0, 0, 0, 0, 286720, 0, 0, 0, 0, 0, 0, 0, 0, 0, 286720, 286720, 0, 286720, 286720, 1, 0, 0, 0, 2106, 0, 0, 0, 0, 2109, 2110, 0, 0, 0, 0, 0, 0, 0, 2069, 0, 0, 0, 0, 0, 0, 0, 0, 0, 541, 564, 541, 564, 541, 541, 564, 541, 541, 541, 541, 2175, 541, 541, 2179, 541, 541, 541, 2183, 541, 541, 541, 541, 541, 541, 1347, 541, 541, 541, 541, 541, 541, 541, 541, 541, 563, 563, 3231, 563, 563, 563, 563, 2225, 541, 541, 541, 541, 2230, 541, 541, 541, 541, 541, 541, 541, 541, 0, 0, 0, 0, 0, 386, 0, 69632, 73728, 0, 0, 0, 0, 0, 65536, 0, 2242, 0, 563, 563, 563, 563, 563, 563, 563, 563, 563, 563, 563, 563, 563, 563, 2325, 0, 563, 563, 563, 2260, 563, 563, 2264, 563, 563, 563, 2268, 563, 563, 563, 563, 563, 587, 587, 3625, 587, 3626, 587, 587, 587, 587, 587, 587, 1493, 587, 587, 587, 587, 587, 587, 587, 587, 587, 2380, 587, 587, 587, 587, 587, 587, 2312, 563, 563, 563, 563, 2317, 563, 563, 563, 563, 563, 563, 563, 563, 0, 0, 0, 0, 0, 0, 587, 587, 2655, 587, 587, 587, 2329, 0, 587, 587, 587, 587, 587, 587, 587, 587, 587, 587, 587, 587, 587, 587, 2680, 587, 587, 587, 587, 587, 587, 2347, 587, 587, 2351, 587, 587, 587, 2355, 587, 587, 587, 587, 587, 587, 2925, 587, 587, 587, 587, 587, 587, 587, 587, 587, 1471, 587, 587, 587, 587, 587, 587, 2399, 587, 587, 587, 587, 2404, 587, 587, 587, 587, 587, 587, 587, 587, 541, 587, 0, 0, 0, 854, 1001, 927, 541, 541, 541, 541, 910, 563, 563, 563, 563, 563, 1848, 563, 563, 563, 563, 563, 563, 563, 563, 563, 563, 3097, 563, 3099, 563, 563, 563, 0, 0, 0, 0, 2439, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1616, 0, 0, 0, 0, 0, 0, 2475, 0, 0, 0, 2479, 0, 0, 0, 0, 0, 0, 0, 0, 0, 365, 0, 0, 0, 0, 0, 0, 541, 2525, 541, 541, 541, 541, 541, 541, 541, 541, 541, 541, 2533, 541, 541, 541, 541, 541, 541, 3053, 541, 541, 3055, 541, 541, 541, 541, 541, 541, 541, 1311, 541, 541, 541, 541, 541, 541, 541, 541, 541, 1753, 541, 541, 541, 541, 541, 541, 541, 2551, 541, 541, 2553, 541, 2554, 541, 541, 541, 541, 541, 541, 541, 2561, 541, 541, 541, 541, 876, 878, 541, 541, 541, 541, 541, 541, 541, 541, 541, 541, 541, 1353, 541, 541, 541, 541, 541, 2565, 541, 541, 541, 541, 541, 541, 541, 541, 541, 541, 541, 541, 2576, 541, 541, 541, 541, 1283, 541, 541, 541, 541, 541, 541, 541, 541, 541, 541, 1304, 0, 0, 0, 563, 563, 2589, 563, 563, 563, 563, 563, 563, 563, 563, 563, 563, 3537, 563, 563, 3539, 563, 587, 587, 563, 2597, 563, 563, 563, 563, 563, 563, 563, 563, 563, 563, 563, 563, 563, 563, 2326, 0, 563, 563, 2612, 563, 563, 563, 2615, 563, 563, 2617, 563, 2618, 563, 563, 563, 563, 563, 2877, 2878, 563, 563, 563, 563, 2881, 563, 563, 563, 563, 563, 1863, 563, 563, 563, 563, 563, 563, 563, 563, 563, 563, 3356, 587, 587, 587, 587, 587, 563, 563, 563, 563, 2626, 563, 563, 2630, 563, 563, 563, 563, 563, 563, 563, 563, 3345, 563, 563, 563, 563, 563, 563, 563, 563, 563, 563, 563, 2641, 2642, 563, 563, 563, 563, 563, 563, 563, 563, 563, 563, 26028, 1360, 988, 587, 587, 587, 587, 2683, 587, 2684, 587, 587, 587, 587, 587, 587, 587, 587, 2692, 587, 587, 2696, 587, 0, 0, 0, 1061, 1062, 1063, 851, 541, 898, 902, 1068, 924, 563, 971, 975, 2733, 2734, 0, 0, 2737, 2738, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 306, 307, 0, 0, 0, 0, 0, 2788, 0, 2790, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 2457, 0, 0, 0, 0, 0, 0, 0, 2803, 0, 0, 0, 0, 0, 0, 541, 541, 541, 541, 541, 541, 541, 3033, 541, 541, 2826, 541, 541, 541, 541, 541, 541, 541, 541, 541, 541, 541, 541, 541, 541, 541, 2825, 2873, 563, 563, 563, 563, 563, 563, 563, 563, 563, 563, 563, 563, 2883, 563, 563, 563, 563, 1912, 563, 563, 563, 563, 563, 563, 26028, 0, 587, 587, 587, 587, 587, 2934, 587, 587, 587, 587, 587, 587, 587, 2940, 587, 587, 587, 587, 2922, 587, 587, 587, 587, 587, 587, 587, 587, 587, 587, 587, 587, 2932, 0, 2982, 0, 0, 2984, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1632, 0, 0, 0, 0, 0, 0, 0, 0, 2995, 0, 0, 2998, 0, 2999, 0, 0, 0, 0, 0, 0, 0, 2096, 0, 0, 0, 0, 0, 0, 0, 0, 0, 562, 585, 562, 585, 562, 562, 585, 541, 3049, 541, 541, 541, 541, 541, 541, 541, 541, 541, 541, 541, 541, 541, 3061, 3089, 563, 563, 563, 563, 563, 563, 563, 563, 563, 563, 563, 563, 563, 563, 563, 2621, 563, 3103, 563, 563, 563, 563, 563, 587, 587, 587, 3111, 587, 587, 587, 587, 587, 587, 1507, 587, 587, 587, 587, 587, 587, 587, 587, 587, 1983, 1984, 587, 587, 587, 587, 587, 587, 3131, 587, 587, 587, 587, 587, 587, 587, 587, 587, 587, 587, 587, 587, 587, 2929, 587, 587, 587, 587, 3145, 587, 587, 587, 587, 587, 541, 587, 563, 541, 541, 563, 563, 587, 587, 0, 2962, 0, 0, 0, 0, 0, 0, 0, 0, 0, 2507, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 3163, 0, 0, 0, 0, 0, 0, 0, 3169, 0, 0, 0, 0, 0, 438, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 2142, 0, 0, 0, 0, 541, 3226, 541, 541, 541, 541, 541, 541, 541, 563, 563, 563, 563, 563, 563, 563, 3095, 563, 563, 563, 563, 563, 563, 563, 563, 2306, 563, 563, 563, 563, 563, 563, 563, 563, 563, 3236, 563, 563, 563, 563, 563, 563, 563, 563, 563, 563, 563, 563, 563, 2309, 2310, 2311, 563, 563, 3246, 563, 563, 563, 563, 563, 563, 3252, 563, 3254, 563, 563, 563, 563, 563, 1392, 563, 563, 563, 563, 563, 563, 563, 563, 563, 563, 1853, 563, 563, 563, 563, 563, 563, 563, 563, 587, 587, 587, 587, 587, 587, 587, 587, 587, 3265, 587, 587, 587, 587, 587, 2703, 587, 587, 587, 587, 587, 587, 587, 587, 587, 587, 2950, 587, 587, 541, 587, 563, 587, 587, 3281, 587, 3283, 587, 587, 587, 587, 587, 587, 587, 587, 541, 587, 563, 3153, 541, 3155, 563, 3157, 587, 0, 0, 0, 3301, 0, 0, 0, 0, 0, 0, 0, 3180, 0, 0, 0, 0, 0, 0, 0, 3014656, 3207168, 0, 2691072, 0, 0, 3215360, 0, 0, 3309, 0, 0, 0, 0, 0, 3314, 0, 0, 0, 0, 0, 0, 0, 0, 0, 365, 0, 0, 0, 0, 131072, 131072, 563, 563, 3339, 563, 563, 563, 3343, 563, 563, 563, 563, 563, 563, 563, 563, 563, 26028, 1360, 988, 587, 587, 1456, 587, 587, 587, 587, 3362, 587, 587, 587, 587, 587, 587, 587, 587, 587, 587, 587, 587, 3126, 587, 587, 587, 587, 587, 3371, 587, 587, 587, 587, 541, 541, 563, 563, 587, 587, 0, 0, 0, 3296, 0, 0, 0, 0, 0, 0, 3557, 3558, 0, 0, 0, 0, 0, 0, 0, 541, 541, 541, 3568, 541, 541, 541, 541, 541, 1284, 541, 541, 541, 541, 541, 541, 541, 541, 541, 1303, 563, 563, 3588, 563, 587, 587, 587, 3593, 587, 587, 587, 587, 587, 587, 587, 587, 2690, 587, 587, 587, 587, 2694, 587, 587, 3601, 587, 0, 0, 0, 0, 0, 3604, 3605, 3606, 0, 3608, 541, 541, 541, 541, 541, 541, 1813, 541, 541, 541, 541, 1819, 541, 541, 541, 1360, 563, 3621, 3622, 563, 563, 587, 587, 587, 587, 587, 587, 587, 587, 587, 3629, 3630, 563, 563, 587, 587, 587, 587, 3679, 0, 541, 3680, 563, 3681, 587, 3682, 0, 541, 541, 541, 541, 1345, 541, 541, 541, 541, 541, 541, 541, 541, 541, 541, 541, 541, 2560, 541, 541, 541, 358, 359, 0, 0, 0, 0, 0, 0, 0, 365, 0, 293, 0, 0, 0, 0, 0, 0, 306, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1174, 0, 0, 0, 0, 0, 0, 0, 0, 392, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 2495, 0, 0, 0, 474, 474, 474, 488, 0, 0, 488, 359, 359, 359, 510, 359, 359, 359, 359, 474, 559, 606, 582, 582, 582, 582, 582, 582, 582, 606, 606, 606, 559, 606, 606, 606, 606, 606, 606, 606, 606, 582, 582, 559, 582, 606, 582, 606, 1, 541, 541, 868, 541, 541, 541, 541, 541, 541, 541, 541, 541, 541, 904, 541, 541, 541, 541, 541, 2851, 541, 541, 541, 0, 0, 563, 563, 563, 563, 563, 563, 563, 563, 563, 563, 2254, 563, 563, 563, 587, 1015, 587, 587, 587, 587, 587, 587, 587, 587, 587, 587, 1051, 587, 587, 587, 587, 587, 2909, 587, 587, 587, 587, 587, 587, 2917, 587, 587, 587, 587, 587, 1527, 587, 587, 587, 587, 587, 587, 587, 587, 1538, 1539, 1197, 0, 0, 0, 0, 0, 0, 0, 1197, 0, 0, 0, 0, 0, 0, 0, 0, 1134592, 0, 0, 1134592, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 2980, 0, 0, 541, 541, 1280, 541, 541, 541, 1289, 541, 541, 541, 541, 541, 541, 541, 541, 541, 541, 3524, 541, 541, 3526, 563, 563, 1360, 914, 563, 563, 563, 563, 563, 563, 563, 563, 563, 1372, 563, 563, 563, 1381, 587, 587, 1543, 0, 1289, 1473, 1381, 541, 541, 541, 541, 541, 563, 563, 563, 563, 563, 2898, 563, 563, 563, 563, 563, 0, 0, 587, 587, 587, 587, 587, 587, 2337, 587, 587, 587, 587, 587, 587, 587, 2352, 587, 587, 587, 587, 2357, 587, 587, 587, 563, 587, 587, 587, 587, 587, 1079, 0, 0, 0, 1564, 0, 0, 0, 1570, 0, 0, 0, 0, 0, 2440, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 2871296, 0, 0, 2424832, 0, 0, 0, 1576, 0, 0, 0, 1582, 0, 0, 0, 0, 0, 0, 0, 0, 0, 365, 0, 0, 0, 0, 155648, 0, 0, 1606, 0, 0, 0, 0, 0, 0, 0, 1614, 1615, 0, 0, 0, 0, 0, 0, 0, 3385, 3180, 0, 0, 0, 0, 0, 0, 0, 0, 2097, 0, 0, 0, 0, 0, 0, 0, 0, 2070, 0, 0, 0, 0, 2074, 0, 0, 0, 0, 0, 0, 1625, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1648, 0, 0, 0, 0, 563, 1891, 563, 563, 563, 563, 563, 563, 563, 563, 1902, 563, 563, 563, 563, 563, 1408, 563, 563, 563, 563, 563, 563, 563, 563, 563, 563, 1867, 563, 563, 563, 563, 563, 587, 1975, 587, 587, 587, 587, 587, 587, 587, 587, 587, 587, 587, 1987, 587, 587, 587, 587, 1490, 587, 587, 587, 587, 587, 587, 587, 587, 587, 587, 587, 2381, 587, 587, 587, 587, 2008, 587, 587, 587, 587, 587, 587, 541, 587, 563, 541, 541, 541, 541, 563, 563, 918, 563, 563, 563, 563, 563, 563, 563, 563, 563, 563, 563, 2634, 563, 563, 563, 563, 0, 2243, 563, 563, 563, 563, 563, 563, 563, 563, 563, 563, 563, 563, 563, 563, 2637, 563, 0, 2330, 587, 587, 587, 587, 587, 587, 587, 587, 587, 587, 587, 587, 587, 587, 3276, 587, 587, 587, 0, 0, 2451, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 2471, 0, 2472, 0, 0, 0, 563, 563, 563, 563, 563, 563, 563, 2593, 563, 563, 563, 563, 563, 1424, 563, 563, 563, 563, 563, 563, 563, 563, 563, 563, 2293, 563, 563, 563, 563, 563, 563, 587, 2659, 587, 587, 587, 587, 587, 587, 587, 587, 587, 587, 587, 587, 587, 587, 2912, 587, 587, 2916, 587, 587, 587, 587, 0, 0, 0, 0, 0, 2776, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 372, 0, 0, 373, 0, 0, 563, 2895, 563, 563, 563, 563, 563, 563, 563, 563, 563, 0, 0, 587, 587, 587, 587, 2335, 587, 587, 587, 587, 587, 587, 587, 587, 587, 2947, 587, 587, 587, 587, 587, 541, 587, 563, 587, 2906, 587, 587, 587, 587, 587, 587, 2913, 587, 2915, 587, 587, 587, 587, 587, 587, 2935, 587, 587, 587, 587, 587, 587, 587, 587, 587, 2408, 587, 587, 587, 587, 2177, 2349, 587, 587, 587, 2944, 587, 587, 587, 587, 587, 587, 587, 587, 587, 541, 587, 563, 563, 563, 563, 563, 563, 563, 587, 587, 587, 541, 587, 587, 587, 0, 541, 587, 563, 541, 1547, 541, 541, 1550, 563, 1551, 563, 563, 0, 0, 0, 0, 0, 3561, 0, 0, 0, 541, 541, 541, 541, 541, 541, 541, 541, 3464, 541, 3466, 3467, 541, 3469, 541, 541, 541, 541, 3575, 541, 563, 563, 563, 563, 563, 563, 563, 563, 563, 563, 3423, 563, 563, 563, 3426, 587, 587, 587, 587, 563, 3587, 563, 563, 587, 587, 587, 587, 587, 587, 587, 587, 587, 587, 587, 3600, 587, 587, 587, 587, 3670, 587, 0, 0, 0, 0, 541, 541, 541, 541, 563, 563, 919, 925, 563, 563, 563, 563, 943, 563, 563, 563, 563, 563, 563, 3353, 563, 563, 563, 587, 587, 587, 587, 587, 587, 587, 587, 587, 587, 587, 587, 3267, 0, 0, 360, 0, 0, 0, 0, 0, 0, 365, 0, 293, 0, 0, 0, 0, 0, 0, 331, 0, 0, 0, 0, 0, 0, 0, 331, 0, 0, 69632, 73728, 0, 419, 419, 0, 0, 65536, 419, 0, 0, 0, 0, 375, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1663, 0, 0, 0, 0, 0, 0, 360, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 2511, 0, 0, 0, 0, 0, 370, 0, 0, 370, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 2496, 0, 0, 560, 607, 583, 583, 583, 583, 583, 583, 583, 607, 607, 607, 560, 607, 607, 607, 607, 607, 607, 607, 607, 583, 583, 560, 583, 607, 583, 607, 1, 541, 863, 541, 541, 541, 541, 541, 541, 541, 541, 541, 541, 541, 541, 541, 541, 1772, 541, 1010, 587, 587, 587, 587, 587, 587, 587, 587, 587, 587, 587, 587, 587, 587, 587, 3123, 587, 587, 587, 587, 587, 587, 587, 563, 1386, 563, 563, 563, 563, 563, 563, 563, 563, 563, 563, 563, 563, 563, 563, 2871, 563, 0, 0, 2760, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 2755, 0, 0, 0, 0, 0, 0, 0, 2985, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 406, 0, 0, 0, 0, 0, 0, 0, 0, 0, 3302, 0, 3303, 0, 0, 0, 0, 3180, 0, 0, 0, 0, 0, 0, 348, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 2801664, 0, 0, 0, 0, 2142208, 0, 0, 0, 361, 362, 363, 364, 0, 0, 365, 0, 293, 0, 0, 0, 0, 0, 0, 348, 349, 350, 0, 0, 0, 0, 0, 0, 0, 0, 3180, 0, 3388, 0, 0, 0, 0, 0, 0, 362, 0, 361, 0, 0, 0, 69632, 73728, 0, 0, 0, 0, 428, 65536, 0, 0, 0, 0, 0, 2463, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 2807, 541, 541, 541, 541, 428, 428, 0, 0, 428, 0, 362, 428, 457, 0, 0, 0, 0, 0, 0, 0, 0, 1159168, 0, 1159168, 0, 0, 0, 0, 1159168, 0, 0, 0, 0, 498, 498, 0, 505, 505, 505, 505, 511, 512, 505, 505, 525, 525, 525, 525, 525, 457, 457, 457, 457, 457, 457, 457, 457, 457, 525, 525, 525, 525, 525, 525, 525, 525, 525, 561, 584, 561, 584, 561, 561, 584, 561, 608, 584, 584, 584, 584, 584, 584, 584, 608, 608, 608, 561, 608, 608, 608, 608, 608, 608, 608, 608, 584, 584, 620, 625, 608, 625, 631, 1, 563, 563, 563, 973, 563, 563, 563, 563, 563, 0, 587, 587, 587, 587, 587, 587, 1963, 587, 587, 587, 587, 587, 587, 587, 587, 587, 3137, 587, 587, 587, 587, 587, 587, 587, 0, 2105, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 2052, 0, 0, 0, 0, 0, 541, 2161, 541, 541, 541, 541, 541, 541, 541, 541, 541, 541, 541, 3336, 541, 563, 563, 563, 0, 0, 563, 2245, 563, 563, 563, 563, 563, 563, 563, 563, 563, 563, 563, 563, 2882, 563, 563, 563, 0, 0, 587, 2332, 587, 587, 587, 587, 587, 587, 587, 587, 587, 587, 587, 587, 3286, 587, 587, 587, 587, 541, 587, 563, 2550, 541, 541, 541, 541, 541, 541, 541, 541, 541, 541, 541, 541, 541, 541, 541, 2210, 541, 541, 541, 2579, 541, 541, 541, 541, 541, 541, 541, 541, 541, 541, 0, 0, 0, 0, 0, 0, 2068, 0, 0, 0, 0, 0, 0, 0, 0, 0, 365, 0, 0, 122880, 122880, 0, 0, 2711, 587, 587, 587, 587, 587, 587, 587, 587, 587, 587, 541, 587, 563, 541, 541, 541, 2021, 563, 563, 562, 609, 585, 585, 585, 585, 585, 585, 585, 609, 609, 609, 562, 609, 609, 609, 609, 609, 609, 609, 609, 585, 585, 562, 585, 609, 585, 609, 1, 0, 0, 0, 705, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 2742, 0, 0, 0, 0, 735, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 2075, 0, 541, 864, 541, 541, 541, 541, 541, 541, 541, 541, 541, 541, 541, 541, 541, 541, 2238, 0, 1011, 587, 587, 587, 587, 587, 587, 587, 587, 587, 587, 587, 587, 587, 587, 587, 2689, 587, 587, 587, 587, 587, 587, 587, 587, 2716, 587, 587, 587, 541, 587, 563, 541, 541, 1136, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 365, 365, 0, 0, 0, 0, 0, 0, 2095, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 2375680, 0, 0, 0, 0, 0, 587, 1503, 587, 587, 587, 587, 587, 587, 587, 587, 587, 587, 587, 587, 587, 587, 2911, 587, 2914, 587, 587, 587, 587, 2919, 587, 563, 587, 587, 587, 587, 587, 1079, 1560, 0, 0, 0, 1566, 0, 0, 0, 1572, 0, 0, 0, 1578, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 2754, 0, 0, 0, 563, 563, 563, 563, 2600, 563, 563, 563, 563, 563, 563, 563, 563, 563, 563, 563, 26028, 1922, 587, 587, 587, 0, 0, 0, 0, 647, 0, 0, 0, 0, 0, 0, 743, 541, 541, 541, 541, 541, 541, 2178, 541, 541, 541, 541, 541, 541, 541, 541, 541, 0, 2242, 563, 563, 563, 563, 563, 0, 0, 1594, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 2770, 0, 0, 563, 563, 563, 1846, 563, 563, 563, 563, 563, 563, 563, 563, 563, 563, 563, 563, 3085, 563, 3087, 563, 2187, 541, 2189, 541, 541, 541, 541, 541, 541, 541, 541, 541, 541, 541, 541, 541, 1821, 541, 1360, 0, 0, 563, 563, 563, 563, 2248, 563, 563, 563, 563, 563, 563, 563, 563, 563, 2305, 563, 563, 563, 563, 563, 563, 563, 563, 563, 0, 587, 587, 994, 587, 587, 587, 563, 2274, 563, 563, 563, 563, 563, 563, 563, 563, 563, 563, 563, 563, 563, 563, 2894, 563, 587, 2361, 587, 587, 587, 587, 587, 587, 587, 587, 587, 587, 587, 587, 587, 587, 3121, 587, 587, 587, 587, 587, 587, 587, 587, 563, 563, 541, 563, 587, 563, 587, 1, 2623, 563, 563, 563, 563, 563, 563, 563, 563, 563, 563, 563, 563, 563, 563, 563, 3101, 563, 563, 563, 563, 2651, 0, 0, 0, 0, 0, 0, 587, 587, 587, 587, 587, 587, 587, 587, 587, 2340, 587, 587, 587, 587, 0, 0, 3179, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 2798, 0, 0, 3245, 563, 563, 563, 563, 563, 563, 563, 563, 563, 563, 563, 563, 563, 563, 563, 3257, 563, 1134592, 0, 1134592, 0, 0, 0, 1134592, 1135008, 1135008, 0, 0, 0, 0, 0, 1135008, 0, 0, 0, 0, 0, 2478, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 3180, 0, 0, 3307, 0, 1134592, 0, 1134592, 1134592, 0, 0, 0, 0, 1135203, 1135203, 1135203, 1135203, 1134592, 1135203, 1135203, 1135203, 1135203, 1135203, 1135203, 1135203, 0, 1134592, 1134592, 1134592, 1134592, 1135203, 1134592, 1135203, 1, 0, 0, 2125824, 2125824, 2125824, 2125824, 2125824, 2424832, 2433024, 2125824, 2125824, 2125824, 2125824, 2125824, 2125824, 2125824, 2125824, 2125824, 0, 0, 988, 2125824, 2125824, 2125824, 2125824, 1147355, 1147355, 1147355, 1147355, 458, 458, 1147355, 458, 458, 458, 458, 458, 458, 458, 458, 1147406, 1147406, 1147406, 1147406, 1147406, 1147406, 1147406, 1147406, 1147406, 1147406, 1147406, 1147406, 1147406, 1147406, 1147406, 1147406, 0, 0, 0, 0, 0, 0, 0, 0, 458, 0, 0, 0, 0, 0, 1147355, 1147355, 1147355, 1147406, 1147406, 1147355, 1147406, 1147406, 1, 12290, 3, 0, 0, 0, 0, 249856, 0, 0, 0, 249856, 0, 0, 0, 0, 0, 0, 0, 69632, 73728, 0, 0, 0, 0, 425, 65536, 0, 1159168, 0, 0, 1159168, 0, 1159168, 1159168, 0, 1159168, 1159168, 0, 1159168, 1159168, 1159168, 1159168, 1159168, 1159168, 1159168, 1159168, 1159168, 1159168, 1159168, 1159168, 1159168, 1159168, 1159168, 1159168, 1159168, 1159168, 1159168, 0, 1159168, 1159168, 0, 1159168, 1159168, 1159168, 1159168, 1159168, 1159168, 1159168, 1159168, 1159168, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1159168, 0, 0, 1159168, 1159168, 1159168, 1159168, 1159168, 1159168, 1159168, 1159168, 1159168, 1159168, 1, 12290, 3, 78115, 293, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 131072, 131072, 0, 0, 1163264, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 3170, 0, 913, 2125824, 2125824, 2125824, 2125824, 2125824, 2424832, 2433024, 2125824, 2125824, 2125824, 2125824, 2125824, 2125824, 2125824, 2125824, 2125824, 0, 0, 1453, 2125824, 2125824, 2125824, 2125824, 106496, 0, 106496, 106496, 0, 106496, 106496, 106496, 106496, 106496, 106496, 106496, 106496, 106496, 106496, 106496, 106496, 106496, 106496, 106496, 106496, 0, 0, 0, 106496, 0, 0, 106496, 106496, 106496, 106496, 106496, 106496, 106496, 106496, 106496, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 2183168, 0, 0, 0, 0, 0, 0, 0, 0, 2134016, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 541, 541, 541, 1275, 541, 541, 0, 0, 0, 3117056, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 2979, 0, 0, 0, 0, 3108864, 3198976, 0, 0, 3043328, 0, 3149824, 2936832, 0, 2760704, 0, 2437120, 0, 0, 0, 0, 0, 0, 2107, 0, 0, 0, 0, 0, 0, 0, 0, 0, 365, 0, 293, 0, 0, 0, 0, 0, 0, 0, 0, 2875392, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 2834432
];

XQueryParser.EXPECTED =
[ 1039, 1047, 1048, 1046, 1042, 1052, 1056, 1060, 1064, 1068, 1446, 1074, 2948, 2053, 1092, 1446, 2490, 1670, 1213, 1080, 1084, 1085, 1446, 1089, 1446, 1446, 2818, 1098, 1103, 1108, 1187, 1181, 1181, 1114, 1118, 1446, 1933, 1123, 1446, 1136, 1098, 1098, 1204, 1108, 1108, 1171, 1181, 1181, 1130, 1134, 1446, 1446, 1140, 1446, 1147, 1098, 1152, 1108, 1108, 1109, 1181, 1181, 1182, 1158, 1446, 1446, 1238, 1668, 1098, 1148, 1108, 1108, 1206, 1181, 1181, 1165, 1446, 3130, 2162, 1098, 1099, 1108, 1177, 1181, 1215, 1219, 2143, 1136, 1098, 1186, 1110, 1181, 1191, 3130, 2153, 1099, 1108, 1180, 1217, 2659, 2819, 1197, 1173, 1201, 1104, 1235, 1210, 1224, 1154, 1228, 1232, 1168, 1243, 1247, 1251, 1255, 1259, 1261, 1266, 1262, 1270, 1274, 1278, 1282, 1286, 1290, 1220, 1295, 1890, 1296, 1446, 1446, 1446, 1446, 1375, 2489, 1446, 1446, 1446, 2385, 1446, 1446, 1446, 1446, 1446, 1446, 1446, 1446, 1076, 1446, 1446, 1446, 1446, 2986, 1446, 1446, 1446, 1446, 1446, 1446, 1446, 1446, 1446, 1446, 1446, 1446, 1446, 1446, 1446, 1446, 1446, 1446, 1446, 1446, 1446, 1446, 1446, 1446, 1446, 1446, 1446, 1446, 1446, 1446, 1446, 1446, 1446, 1446, 1446, 1446, 1446, 1446, 1446, 1446, 1446, 1446, 1446, 1446, 1446, 1446, 1446, 1446, 1446, 1446, 1446, 1446, 1446, 1446, 1446, 1446, 1446, 1446, 1446, 1446, 1446, 1446, 1446, 1446, 1446, 1446, 1446, 1446, 1446, 1446, 1446, 1446, 1446, 1446, 1446, 1446, 1446, 1300, 1304, 1305, 1309, 1405, 1315, 1318, 1322, 1326, 1330, 1446, 1446, 1804, 1334, 2315, 1338, 2350, 1142, 1143, 1342, 1346, 1446, 1446, 2834, 1352, 1362, 1369, 1446, 1126, 1446, 2386, 1379, 1446, 1384, 1390, 1446, 2987, 2588, 1396, 2688, 1457, 1446, 1518, 1446, 1446, 2269, 1446, 1446, 1446, 1402, 1446, 1441, 1897, 1409, 1413, 1446, 1447, 1419, 1446, 1447, 1419, 1446, 1446, 1193, 1446, 2988, 3136, 2482, 1446, 2617, 1425, 1446, 1448, 1431, 1446, 1918, 1446, 2648, 1436, 1457, 1348, 1457, 1380, 1440, 1446, 1445, 3050, 1452, 1446, 1456, 1311, 1446, 1916, 2325, 1463, 2125, 1347, 1472, 2317, 1909, 1478, 3010, 2343, 2076, 2746, 2758, 1482, 2765, 2077, 1799, 1486, 2744, 1493, 1358, 1446, 1446, 2131, 1446, 2559, 1386, 1365, 1497, 1501, 1505, 1509, 1515, 2369, 1569, 2450, 1522, 2538, 1526, 1530, 1536, 2479, 2088, 1457, 1971, 1540, 1544, 1548, 1552, 1557, 1446, 2103, 1446, 1532, 1446, 2479, 2082, 1563, 1568, 2137, 1573, 1577, 3026, 1446, 1446, 2116, 1446, 1446, 3027, 1446, 1586, 1938, 1584, 2123, 1647, 1590, 1432, 1594, 1446, 2517, 1600, 1446, 2517, 1600, 1446, 2872, 1606, 1612, 1616, 1620, 1626, 1446, 1239, 1633, 1446, 1596, 1640, 3118, 2782, 1645, 1651, 1863, 1446, 1863, 1446, 1602, 1446, 2675, 2173, 1655, 2685, 1559, 1866, 2685, 3124, 1659, 1665, 2904, 2903, 3055, 2673, 1674, 1678, 3047, 2266, 2239, 1682, 1686, 1690, 1694, 1698, 2892, 1702, 1715, 1719, 1723, 1727, 1731, 1446, 2332, 1773, 1737, 1744, 1748, 1751, 1755, 1758, 1446, 1772, 2979, 1777, 1781, 2506, 1788, 1792, 1635, 1636, 1446, 2668, 2874, 1796, 1808, 2784, 1815, 1819, 1823, 2304, 1828, 1446, 2430, 1446, 1446, 2334, 1853, 2583, 1834, 1843, 1446, 2069, 1446, 1446, 1954, 1446, 1446, 2070, 1446, 1446, 1622, 1608, 1847, 1851, 1857, 1641, 1871, 1446, 2225, 1956, 1446, 2225, 1956, 1446, 2374, 1885, 1876, 1811, 1446, 1883, 1446, 1947, 1889, 1446, 1511, 1446, 1291, 2287, 1894, 1901, 1906, 1446, 1906, 1446, 1913, 1446, 1740, 1922, 1446, 1932, 1733, 1446, 1932, 1830, 1094, 1732, 1446, 1937, 2066, 2439, 2671, 1945, 1942, 2455, 1489, 2323, 1951, 1709, 1711, 1629, 1161, 1962, 1968, 1977, 1980, 1984, 1446, 1119, 1994, 1999, 2003, 2007, 2011, 2015, 2019, 2023, 1986, 1872, 2093, 1928, 2028, 2032, 2036, 1802, 2040, 1446, 1446, 2044, 1580, 2050, 2059, 2063, 2074, 2046, 1446, 2024, 2081, 1446, 2086, 1446, 1446, 1988, 2092, 2097, 2109, 2113, 1446, 2120, 1446, 1446, 2129, 1446, 1558, 2135, 1446, 1446, 2141, 2535, 2866, 2147, 1446, 1990, 1446, 1446, 3096, 1446, 1446, 3096, 1446, 1446, 1466, 2151, 2157, 2204, 1458, 2161, 1446, 2914, 1446, 1446, 1764, 1446, 3053, 2166, 2170, 1446, 2182, 1446, 2187, 1446, 2916, 1957, 2192, 2202, 2305, 1446, 2210, 2496, 1824, 3112, 2217, 1761, 1459, 1558, 2224, 2229, 2841, 2969, 2682, 2815, 2233, 2248, 2236, 2243, 1837, 1839, 2247, 2252, 2100, 1446, 1446, 1446, 1446, 2918, 1766, 2256, 2260, 2273, 2277, 2281, 2284, 1446, 2889, 1902, 2985, 2524, 2433, 2299, 2576, 2212, 2213, 2303, 1415, 1457, 2309, 1958, 2321, 2329, 1446, 2976, 1446, 1398, 1446, 1553, 2338, 2342, 2751, 1446, 2347, 2503, 2354, 2358, 1558, 2363, 1446, 1995, 2368, 1446, 1421, 2364, 2373, 2733, 2378, 2383, 2390, 3044, 1446, 2715, 1446, 1446, 1860, 1446, 1446, 1860, 2719, 2733, 2397, 2176, 2401, 1946, 2461, 1446, 1446, 2405, 1446, 1473, 2717, 2413, 2417, 2427, 2437, 3061, 1446, 2105, 1446, 1973, 2407, 2443, 2447, 2454, 2459, 2379, 1446, 2465, 2472, 2476, 2486, 2511, 2494, 2701, 2409, 2500, 2510, 2516, 2521, 2528, 2220, 2532, 2542, 2546, 2198, 2312, 2550, 2554, 2563, 2567, 2571, 2575, 1446, 2580, 2592, 2596, 2600, 2604, 2608, 2612, 2616, 1446, 2621, 2393, 1372, 2628, 2632, 2636, 2640, 2652, 3103, 1446, 2708, 2468, 2656, 2665, 2679, 2692, 1558, 2699, 1446, 2705, 1446, 2998, 2712, 1446, 2723, 2624, 1768, 1879, 3159, 1446, 2205, 2727, 1446, 1468, 2732, 1446, 2206, 2982, 1446, 3004, 2737, 2741, 3109, 2750, 1446, 2755, 1446, 1446, 2762, 1446, 1446, 2762, 2769, 3037, 2775, 2420, 2779, 1446, 2788, 1446, 1446, 2795, 1446, 1427, 2801, 3020, 2805, 2646, 1446, 2812, 1446, 2809, 1446, 3067, 2853, 2823, 2643, 2183, 2827, 2863, 2512, 2832, 2838, 1707, 1070, 2359, 2845, 2851, 2857, 1964, 1474, 3084, 2557, 2878, 2955, 3090, 2882, 2293, 2295, 2886, 2896, 2900, 2908, 2912, 1446, 1446, 1446, 2922, 2926, 2930, 2934, 2938, 2942, 2946, 1446, 2055, 2952, 1355, 2791, 2178, 2959, 1867, 2963, 1784, 2967, 1446, 2973, 2423, 2992, 2996, 3002, 1446, 2728, 1446, 1446, 3008, 1446, 3014, 3018, 1446, 3024, 2188, 3031, 3147, 1446, 1446, 2694, 1446, 1446, 2847, 1446, 1446, 2695, 3035, 1446, 3041, 3078, 3141, 3059, 1446, 1446, 3065, 1446, 1446, 3065, 1446, 1446, 3071, 1446, 3075, 1925, 3153, 1446, 1446, 3082, 1446, 1564, 3088, 1446, 1661, 3094, 3100, 2195, 3116, 1446, 3122, 1446, 3128, 1446, 3134, 1705, 3140, 2586, 2797, 1446, 3145, 2771, 1392, 3151, 1446, 3157, 2290, 2828, 2661, 2869, 2422, 3106, 2860, 2263, 1446, 1446, 1446, 1446, 1446, 1446, 1446, 1446, 1446, 1446, 1473, 3163, 3166, 3170, 3182, 3186, 3182, 3189, 3181, 3182, 3182, 3182, 3182, 3177, 3173, 3193, 3182, 3197, 3201, 3204, 3210, 3214, 3216, 3206, 3220, 3224, 3228, 3232, 3235, 3239, 3243, 3729, 3929, 3929, 3246, 3929, 3929, 5383, 3929, 3929, 3300, 3929, 3261, 3562, 3273, 4078, 3734, 3929, 3929, 3929, 3277, 3287, 3294, 4847, 3929, 3250, 3929, 3929, 3344, 3929, 4873, 4873, 4873, 4873, 3266, 4873, 4873, 3564, 3564, 3567, 3564, 3564, 3564, 3564, 3267, 3256, 3305, 3562, 3568, 3335, 4081, 3929, 3929, 3929, 3323, 4919, 3377, 4693, 3929, 3252, 3722, 3750, 3365, 3315, 3269, 3334, 4080, 3732, 3929, 3929, 3559, 4873, 5799, 4692, 3929, 3929, 3688, 3929, 3929, 3560, 4873, 4873, 4873, 4195, 4873, 4873, 4873, 4874, 3564, 3267, 3264, 3331, 3339, 3929, 3342, 3929, 3738, 3256, 3367, 3317, 3321, 3560, 4873, 3564, 3309, 3256, 3256, 3365, 3371, 3564, 3564, 3564, 3567, 3256, 3256, 3256, 3256, 3327, 4195, 3564, 3564, 3564, 3310, 3365, 3359, 3929, 3929, 3813, 3929, 3564, 3564, 3564, 3364, 3929, 5741, 3559, 4873, 4194, 3564, 3564, 3268, 3256, 4193, 4873, 3266, 3564, 3310, 3256, 3256, 3256, 3366, 3348, 3929, 3929, 3929, 3505, 3383, 3256, 3373, 3766, 3256, 3384, 3929, 4872, 4874, 3564, 3268, 3256, 3257, 3373, 4694, 3929, 3929, 3929, 3355, 3566, 3256, 3321, 4194, 3565, 3311, 3560, 3563, 3384, 3561, 3319, 3360, 3388, 3390, 4109, 3394, 3401, 3405, 3429, 3429, 3429, 3429, 3424, 3430, 3412, 3416, 3420, 3428, 3434, 3438, 3442, 3397, 3446, 3450, 3453, 3457, 3460, 3463, 3465, 3469, 3473, 3408, 3477, 3481, 3485, 3696, 3498, 4473, 3929, 3929, 3929, 3512, 3511, 3929, 3929, 3929, 3517, 4964, 4966, 3929, 3523, 3527, 3929, 3929, 3929, 3752, 3534, 3543, 3929, 3929, 3850, 3854, 3578, 3582, 3586, 3589, 3590, 3594, 3597, 3601, 3604, 3608, 3611, 3614, 3618, 3622, 3626, 4965, 3648, 3636, 5178, 3647, 5820, 3654, 4257, 3659, 3929, 3669, 4219, 3694, 3703, 3929, 5714, 3525, 3929, 3929, 3929, 3770, 3790, 3929, 5539, 3707, 3929, 3342, 5695, 3904, 3929, 3289, 3290, 3906, 3929, 4658, 4464, 5227, 4306, 3936, 3718, 3726, 3743, 3929, 3375, 3631, 3929, 3548, 5050, 5064, 3749, 3929, 3929, 3929, 3836, 3758, 3298, 3929, 3929, 3915, 3927, 4893, 3528, 3929, 3929, 3921, 4548, 5744, 4281, 3929, 3929, 3932, 5019, 3929, 5587, 5178, 3929, 3553, 3557, 3572, 5826, 3929, 3929, 4530, 3789, 3794, 3929, 3929, 3972, 4899, 3798, 3806, 3929, 3929, 3972, 5040, 3817, 3826, 3929, 3929, 3972, 5490, 3785, 3929, 3929, 3929, 3869, 4728, 4203, 3770, 3790, 4142, 3929, 3929, 3929, 3894, 4001, 3929, 3929, 3929, 3929, 3252, 3798, 4127, 4203, 3850, 3854, 4139, 3744, 3929, 3929, 3929, 3844, 5313, 5832, 4137, 4141, 3929, 3650, 3929, 3929, 4854, 4858, 4140, 3929, 3929, 3929, 3930, 5557, 3873, 3929, 3929, 3883, 3929, 5107, 3929, 5107, 4780, 3929, 4781, 5020, 3929, 3929, 3738, 5109, 3929, 3290, 3904, 3940, 3947, 3946, 3942, 3951, 3955, 3956, 3956, 3960, 3962, 3963, 3962, 3967, 3971, 3929, 3929, 4017, 4021, 4708, 3978, 3988, 3929, 3683, 3781, 3802, 3995, 4392, 3929, 5597, 5363, 3530, 3929, 4135, 4005, 4013, 3929, 3929, 4065, 4070, 4026, 3929, 3929, 4026, 3375, 3929, 3929, 4049, 4056, 4576, 3997, 5403, 5596, 3929, 3856, 4795, 3529, 3929, 3929, 3929, 3931, 4061, 3929, 3929, 3929, 3972, 4199, 3911, 3929, 3929, 3929, 3974, 5170, 3929, 3929, 4704, 3929, 4075, 5320, 4248, 5165, 3855, 4657, 4085, 3929, 3690, 4699, 5178, 3978, 3251, 3929, 3929, 4101, 3929, 5319, 4247, 5772, 5010, 4113, 3981, 3929, 3929, 4164, 4148, 4148, 4120, 3929, 3929, 4165, 4115, 4709, 4132, 3929, 3929, 4247, 4459, 5170, 3929, 3929, 4706, 3879, 4572, 4576, 5321, 4249, 4655, 3929, 3929, 4274, 3929, 3972, 4146, 4150, 3929, 3735, 3929, 3735, 4154, 4158, 3929, 3929, 4396, 3929, 3929, 3982, 3929, 3929, 3929, 4029, 4107, 3929, 4706, 3929, 4366, 4574, 4181, 5320, 5850, 4644, 4180, 5319, 4886, 4217, 4208, 4673, 3929, 3929, 4424, 5855, 3279, 5321, 4215, 3929, 3766, 4873, 4873, 3563, 3564, 4228, 3929, 4451, 4182, 4233, 4305, 4240, 4273, 4435, 4246, 4434, 4245, 4708, 4241, 3929, 3699, 4502, 3698, 4255, 5609, 4272, 5609, 4272, 3699, 5199, 5610, 4273, 4253, 3281, 4246, 4270, 3929, 3809, 3929, 4549, 3737, 3929, 3896, 3929, 5021, 3770, 4271, 3771, 4182, 5163, 5161, 3929, 3283, 5762, 3710, 4279, 4279, 4211, 4127, 4128, 4405, 3712, 4286, 3929, 3929, 3929, 4052, 3929, 5548, 3754, 4301, 3929, 3842, 3574, 4534, 4651, 5125, 3745, 4537, 4310, 4314, 4318, 4319, 4323, 4327, 4333, 4334, 4331, 4338, 4339, 4339, 4340, 3929, 3844, 4827, 3929, 3846, 3929, 3929, 4905, 3929, 3376, 5780, 4202, 3929, 3929, 3929, 4235, 4524, 5352, 5187, 4364, 4473, 3342, 4273, 3929, 3888, 5737, 4840, 4356, 4374, 4378, 4842, 5051, 4864, 3929, 4387, 4491, 3929, 4413, 3929, 3892, 5107, 3929, 3665, 3929, 3929, 4681, 3640, 4265, 4472, 3929, 4802, 3929, 4518, 3670, 4266, 4421, 3929, 4204, 4203, 3929, 4718, 4428, 4432, 3929, 3929, 3929, 4451, 4440, 4445, 3929, 3929, 4450, 4541, 4470, 3929, 3343, 3929, 3897, 3929, 4294, 3929, 4296, 3929, 5616, 3929, 5461, 4463, 3929, 5347, 4362, 5468, 4800, 3929, 3929, 4457, 4461, 5243, 3929, 4484, 3929, 3931, 5074, 3929, 3972, 4166, 4116, 3929, 3929, 3929, 4035, 4737, 3929, 3929, 3929, 4505, 3929, 5347, 4495, 3929, 3972, 5410, 5415, 4015, 4019, 3929, 3929, 4457, 4490, 4499, 3929, 3929, 3929, 4624, 5347, 3929, 4801, 3929, 3997, 3929, 5743, 4160, 3929, 3929, 3929, 4628, 3929, 5704, 4529, 3929, 3998, 5353, 5435, 3929, 5703, 4528, 3929, 4000, 3929, 3929, 3999, 3528, 4505, 5352, 4800, 3929, 4007, 4653, 3929, 4009, 3929, 4642, 5697, 3929, 3929, 3929, 4695, 4052, 3929, 3929, 3929, 4708, 4545, 3929, 3929, 3301, 4450, 3929, 3929, 3929, 4734, 3929, 5303, 3342, 3929, 4030, 4480, 3929, 3929, 3929, 4516, 4994, 3894, 3929, 3895, 3929, 3929, 3351, 3892, 3737, 3735, 3929, 4039, 3929, 3929, 4086, 5090, 3895, 5020, 3895, 3737, 5022, 3736, 3737, 4554, 4556, 3929, 3929, 4638, 4794, 3929, 3929, 4668, 4806, 4689, 3929, 3929, 3929, 4752, 4504, 3929, 3929, 5097, 4566, 4022, 5334, 4383, 5691, 4570, 4582, 4586, 4590, 4594, 4598, 4602, 4605, 4608, 4612, 4614, 4618, 4634, 4632, 4634, 4621, 3929, 3929, 3929, 4757, 4725, 4816, 3929, 4648, 4662, 4666, 5081, 4672, 3929, 5344, 4398, 4352, 4678, 3929, 3929, 4678, 4687, 5178, 3929, 3929, 4745, 4749, 4210, 3929, 4965, 4768, 5739, 3929, 3929, 3322, 5689, 3899, 5181, 4703, 5110, 5214, 4713, 4722, 3929, 4051, 3558, 3929, 4028, 4478, 4500, 3929, 5400, 4741, 3929, 3929, 4780, 3929, 4780, 4762, 3929, 3929, 3929, 4821, 4756, 4761, 3929, 3929, 4818, 4822, 4505, 3863, 3867, 3929, 3341, 3341, 3929, 4766, 4294, 3929, 3899, 3929, 4066, 3929, 3929, 4087, 5091, 3767, 5433, 4817, 4683, 4716, 3929, 5809, 3929, 4091, 4095, 3751, 3972, 4773, 4777, 3929, 4107, 3929, 3929, 4137, 4141, 4786, 4775, 3929, 3929, 4820, 3910, 4788, 4777, 3929, 3929, 4828, 3972, 3322, 4792, 3929, 3929, 4847, 3929, 5432, 4816, 4833, 4799, 3929, 5147, 3929, 3929, 4872, 4873, 4966, 4295, 4304, 4814, 4826, 3929, 3929, 3929, 4871, 4674, 3929, 4708, 3897, 5180, 5433, 4832, 3929, 4159, 3929, 3377, 3299, 3929, 4503, 4708, 3929, 4837, 3929, 3929, 3929, 4913, 5207, 3929, 3929, 3929, 4965, 3929, 5148, 3929, 4008, 3929, 4408, 3929, 4172, 3929, 4172, 3900, 5432, 3628, 3929, 3929, 3929, 4856, 4860, 3844, 5313, 3929, 3929, 4982, 3929, 5086, 3768, 5434, 3630, 3929, 4174, 3929, 4562, 5312, 3929, 3929, 3929, 5003, 4502, 5148, 4878, 4885, 4293, 3929, 3929, 4296, 3929, 4296, 3929, 4229, 5198, 5520, 3928, 3929, 3928, 3897, 4293, 3929, 3898, 3929, 5228, 3899, 3898, 3929, 4296, 3929, 5547, 3519, 4911, 3929, 5584, 4917, 3769, 3929, 3929, 3929, 4240, 4273, 3929, 3779, 3800, 3744, 4923, 4927, 4934, 4935, 4930, 4939, 4943, 4949, 4953, 4945, 4957, 4957, 4958, 4962, 3929, 4247, 4511, 3929, 3972, 5871, 3929, 3983, 3929, 5572, 5591, 3929, 4731, 4359, 5861, 5581, 4990, 3929, 3929, 3929, 5111, 4846, 5503, 4707, 5051, 3929, 4263, 4465, 3929, 4282, 3929, 3929, 3860, 3929, 5742, 3929, 3929, 3737, 3929, 3894, 5322, 3929, 5000, 5007, 3507, 3929, 4290, 3929, 3929, 4449, 3929, 5018, 3929, 3929, 3642, 5026, 3929, 3929, 3929, 5177, 4628, 3929, 3300, 3929, 4346, 3674, 3681, 3547, 3929, 3539, 5030, 5034, 3929, 3929, 3929, 5337, 5040, 3822, 3929, 3929, 3641, 5042, 3929, 3929, 3929, 5368, 3643, 3929, 3929, 3929, 5405, 5793, 3929, 3929, 3929, 5453, 4626, 3377, 5052, 3929, 3929, 3929, 3721, 4501, 4368, 3929, 3546, 3490, 5496, 5308, 5793, 3929, 3929, 5485, 4370, 3929, 3929, 5068, 3972, 5073, 4863, 3929, 3513, 4450, 5122, 3375, 5406, 3842, 3929, 3655, 5793, 3929, 3739, 4707, 5781, 3929, 3990, 3929, 3929, 4006, 3299, 4452, 4369, 3929, 4439, 4444, 3929, 3895, 4547, 3886, 3929, 5078, 3929, 3929, 5048, 5304, 3929, 5095, 3929, 5486, 5051, 4451, 4368, 3929, 4474, 4050, 4057, 4293, 4450, 3929, 3929, 5228, 3929, 5453, 3929, 3929, 5072, 4862, 3929, 5101, 3929, 3512, 5742, 3929, 5241, 5105, 4986, 3751, 5323, 3296, 4502, 4381, 3929, 4486, 4034, 3929, 3684, 3783, 3744, 5062, 3929, 3929, 4209, 3929, 3929, 3929, 3560, 3929, 5118, 3929, 3929, 5111, 4852, 4515, 4382, 5129, 3929, 4502, 4176, 3929, 4350, 5314, 3549, 3929, 5797, 3929, 3929, 3929, 5526, 5795, 3929, 3929, 3929, 5603, 5136, 5144, 5227, 4451, 4236, 3929, 4975, 3676, 5063, 4465, 3744, 4042, 4464, 5152, 3929, 4507, 3865, 3929, 4121, 5622, 4297, 4174, 3342, 5169, 3342, 5169, 4514, 3537, 4561, 4558, 3929, 4559, 4453, 3929, 5213, 4466, 3929, 4520, 3929, 3929, 5170, 4045, 5196, 5187, 5174, 4560, 5185, 4558, 5193, 5203, 5211, 5218, 5221, 5222, 5226, 3929, 3929, 3929, 5725, 3929, 5232, 4690, 3929, 4524, 5352, 3677, 3929, 3929, 3929, 5771, 3929, 5740, 3929, 4274, 4848, 5238, 5247, 5773, 5254, 5251, 4704, 3918, 5260, 5266, 5262, 5268, 5272, 5276, 5280, 5284, 5287, 5291, 5294, 5293, 5298, 5302, 3929, 3929, 3929, 5742, 5425, 3929, 4694, 5050, 3546, 3490, 3494, 3929, 5318, 3929, 5327, 3923, 4464, 4390, 5227, 3929, 5816, 5331, 5341, 3929, 5782, 3744, 3929, 4547, 3770, 3929, 4547, 3929, 3736, 5322, 3929, 5189, 3929, 3929, 5189, 5361, 5367, 3374, 3929, 4693, 3929, 3929, 3829, 3929, 5372, 3929, 5411, 3929, 4704, 4292, 3929, 3892, 3929, 3929, 4517, 5713, 4819, 4170, 3929, 5416, 3991, 3929, 4708, 4852, 3929, 4186, 4190, 3929, 3765, 3775, 3800, 5841, 5084, 3929, 3929, 5111, 5157, 3753, 5377, 5381, 3929, 3929, 5117, 3929, 3929, 5388, 3821, 3929, 4708, 5234, 4550, 3929, 5393, 5392, 3929, 4751, 5041, 3744, 3929, 3929, 5048, 5036, 3929, 5373, 5397, 4693, 3501, 3929, 3929, 3929, 5766, 4862, 3929, 3929, 3929, 5789, 5048, 4769, 3378, 3492, 5429, 4705, 5779, 3929, 4781, 4779, 3929, 4505, 5110, 3876, 3929, 3929, 3929, 5791, 3972, 5439, 3839, 3929, 4815, 3929, 4778, 5514, 5443, 3839, 3929, 4815, 3929, 5108, 5841, 5843, 3929, 3929, 5139, 3929, 5050, 5457, 3492, 4485, 5465, 3770, 3878, 3929, 4821, 3929, 3929, 4417, 3929, 3929, 5472, 5477, 3929, 4881, 3929, 5701, 5473, 5482, 3929, 3929, 5140, 3929, 5478, 3929, 5420, 3877, 3546, 5494, 3714, 3632, 3929, 5514, 5508, 3929, 4907, 5500, 3929, 4890, 4880, 3929, 4193, 4873, 4873, 4195, 5742, 3379, 5773, 3631, 5525, 3929, 3929, 3929, 5870, 5524, 3760, 3896, 3929, 5819, 4519, 5513, 3488, 5530, 5351, 3929, 4708, 4846, 4159, 3929, 3929, 3929, 5155, 5159, 5336, 5518, 3761, 3929, 3929, 5512, 5536, 4519, 4522, 3989, 3929, 3929, 3929, 4913, 5518, 3929, 4964, 4293, 5179, 5352, 3929, 3929, 4125, 3929, 3929, 4247, 4402, 4086, 3353, 3929, 5556, 5570, 3929, 5570, 5578, 3983, 3929, 5573, 3929, 4971, 3751, 3929, 4253, 3929, 4261, 5594, 4522, 3929, 5601, 3984, 3929, 5574, 3929, 4978, 4224, 3929, 3929, 5608, 5571, 5607, 5351, 5349, 3929, 3929, 3929, 5206, 4827, 3929, 3929, 4897, 4901, 5733, 5834, 3662, 4782, 5757, 3879, 5132, 5614, 5467, 5620, 5626, 5630, 5634, 5638, 5642, 5646, 5650, 5654, 5658, 5662, 5666, 5670, 5673, 5677, 5681, 5685, 3929, 3929, 5256, 4275, 5756, 3929, 3972, 3929, 4996, 3929, 5561, 5708, 5718, 3929, 5722, 5729, 3929, 3888, 3998, 5749, 3929, 3929, 3929, 5311, 3929, 5731, 5753, 5459, 3929, 5014, 4985, 3929, 4344, 4461, 3929, 3875, 5420, 3929, 3878, 3929, 3929, 3929, 3736, 4071, 4506, 3929, 3766, 4521, 4967, 3929, 3929, 3929, 5387, 3820, 5761, 4485, 3929, 3929, 5424, 4691, 5113, 3998, 3929, 3929, 5435, 3873, 3929, 5112, 5770, 4234, 5849, 3929, 3929, 3929, 5447, 3929, 5732, 5805, 3929, 3972, 4093, 4097, 3929, 3905, 3929, 4408, 3670, 5786, 3929, 3929, 3929, 5447, 4693, 4517, 5803, 5807, 3929, 5046, 5056, 3929, 4434, 4245, 3929, 3770, 5321, 3929, 3649, 3929, 3929, 4977, 4223, 3929, 5830, 3929, 3929, 5450, 5042, 4423, 5838, 3929, 3929, 5507, 4159, 4423, 5838, 4248, 5847, 4578, 5805, 3929, 4548, 3929, 3929, 5813, 4423, 5854, 3929, 3929, 5552, 5542, 5859, 3929, 3929, 3929, 5564, 3929, 5786, 4235, 3929, 3929, 5566, 4810, 4103, 5807, 3990, 3929, 5059, 5357, 3929, 4549, 3929, 3929, 4549, 5416, 3929, 4674, 3929, 4868, 5532, 3929, 3929, 3929, 5710, 3929, 3831, 3998, 3929, 3929, 5712, 4818, 3973, 5866, 3929, 3929, 5739, 3929, 3972, 5865, 3929, 3929, 5745, 3929, 4964, 3929, 5180, 4523, 5824, 3832, 3929, 3929, 3929, 5777, 5545, 4965, 3929, 4409, 3929, 3675, 3738, 3973, 3547, 3929, 3929, 5841, 3929, 6277, 6502, 5875, 6282, 5878, 6547, 6266, 5881, 6546, 5889, 6232, 5882, 6232, 5892, 6221, 6267, 5888, 5883, 5884, 6232, 6232, 6232, 6232, 6414, 5887, 5883, 6232, 5891, 6232, 6232, 5894, 6232, 6232, 5922, 5937, 5949, 5938, 5939, 5949, 5944, 5950, 5944, 5944, 5940, 5944, 5941, 5952, 5941, 5944, 5946, 5945, 5946, 5946, 5946, 5946, 5947, 5941, 5948, 5952, 5948, 5944, 5942, 5954, 5956, 5958, 5962, 5960, 5959, 5964, 5966, 5967, 5971, 5967, 5968, 5967, 5969, 5974, 5973, 5976, 5978, 6293, 5998, 5875, 6282, 5896, 6871, 6437, 6282, 6321, 6282, 6282, 6282, 5917, 6456, 6456, 6456, 6456, 5991, 5906, 6379, 6455, 5990, 6019, 5991, 5904, 5904, 5904, 6456, 6456, 5926, 5899, 6456, 6456, 5997, 6282, 6408, 6410, 6282, 5932, 6282, 5933, 6464, 6024, 6282, 6282, 6379, 5924, 6282, 6282, 5924, 6282, 6362, 6010, 5984, 6282, 6003, 6282, 6362, 6282, 6282, 6282, 6004, 6456, 5906, 6455, 5990, 5904, 5902, 6456, 6456, 6456, 5906, 6457, 6019, 6019, 5904, 5904, 6456, 6456, 6282, 6282, 6282, 5980, 5917, 6456, 6456, 6456, 6455, 5905, 6456, 6445, 5926, 5926, 5926, 5926, 6032, 6032, 6008, 5932, 6282, 6282, 6282, 6434, 6024, 6282, 6019, 5904, 6456, 6282, 5896, 6873, 6282, 6282, 6282, 6474, 5903, 6457, 6019, 5904, 6456, 6229, 5904, 6456, 6456, 6456, 6018, 6457, 6019, 5991, 5905, 5906, 6282, 6282, 6282, 5984, 6282, 6282, 6282, 5925, 6543, 5905, 6456, 6456, 6456, 6282, 5905, 6229, 5905, 6229, 5905, 6282, 6650, 6437, 6043, 6515, 6056, 6672, 6088, 6052, 6515, 6520, 6054, 6060, 6058, 6515, 6515, 6083, 6528, 6101, 6761, 6518, 6521, 6516, 6517, 6063, 6065, 6068, 6061, 6057, 6515, 6515, 6519, 6515, 6070, 6072, 6066, 6515, 6515, 6515, 6515, 6125, 6522, 6074, 6515, 6515, 6076, 6515, 6523, 6079, 6082, 6515, 6081, 6055, 6091, 6089, 6090, 6093, 6114, 6092, 6113, 6114, 6114, 6114, 6114, 6115, 6117, 6116, 6117, 6117, 6118, 6119, 6120, 6121, 6121, 6095, 6097, 6099, 6122, 6123, 6124, 6524, 5900, 6515, 6518, 6103, 6108, 6110, 6104, 6105, 6109, 6106, 6112, 6121, 6127, 6129, 6282, 5899, 6282, 5925, 6463, 6321, 6282, 6294, 6006, 6378, 6282, 6508, 6282, 6449, 6646, 6218, 6604, 6282, 6524, 6291, 6282, 6774, 6282, 6270, 6282, 6282, 6282, 6000, 6000, 6027, 6161, 6282, 6282, 5895, 6375, 6282, 6312, 6230, 6366, 6265, 5876, 6282, 6282, 6282, 6015, 6406, 6282, 6284, 6027, 6027, 6137, 6282, 6282, 6379, 6281, 6372, 6028, 6159, 6282, 5899, 6282, 6282, 6282, 6573, 6282, 6282, 6467, 6282, 6311, 6312, 6372, 6282, 6282, 6282, 6019, 6019, 5904, 5904, 5904, 5904, 5905, 6456, 6456, 6031, 6154, 6158, 6282, 6149, 6273, 6282, 6163, 6282, 6677, 6280, 6166, 6279, 6682, 6685, 6501, 6173, 6175, 6178, 6178, 6178, 6178, 6176, 6181, 6180, 6181, 6181, 6183, 6181, 6185, 6187, 6188, 6188, 6188, 6194, 6193, 6194, 6193, 6193, 6189, 6191, 6190, 6190, 6196, 6196, 6198, 6203, 6202, 6204, 6208, 6203, 6205, 6199, 6200, 6210, 6206, 6212, 6282, 6282, 5896, 6282, 6282, 6282, 6512, 6282, 6150, 6135, 6320, 6500, 6163, 6282, 6282, 6282, 6026, 6282, 6026, 6166, 6282, 6282, 6282, 6027, 6221, 6282, 6505, 6282, 6282, 6282, 6032, 6238, 6282, 6241, 6282, 5899, 6451, 6282, 5913, 6669, 6401, 6238, 6282, 6282, 6282, 6037, 6245, 6282, 6282, 6282, 6040, 6282, 6282, 6225, 6248, 6282, 6282, 5916, 6722, 6377, 6247, 6249, 6282, 6282, 5924, 6370, 6252, 6253, 6282, 6282, 5932, 6282, 6282, 6530, 5924, 6254, 6282, 6282, 6254, 6282, 6322, 6228, 6282, 5933, 6378, 6282, 6378, 6282, 6452, 6282, 6432, 6317, 6282, 5917, 6086, 6725, 6727, 6216, 6086, 6726, 6450, 6231, 5926, 5933, 6008, 6008, 6008, 6282, 6282, 6282, 5918, 6282, 6282, 6282, 5923, 6309, 6683, 6282, 6282, 6282, 6148, 6216, 6308, 6310, 6282, 6282, 6282, 6149, 6282, 5903, 6397, 6605, 6282, 6282, 5933, 5933, 5933, 6780, 6282, 6282, 6282, 6229, 6282, 6282, 6282, 5933, 5924, 6316, 5917, 6723, 6377, 5916, 6722, 6323, 6448, 6454, 6216, 6332, 6436, 6683, 6282, 5917, 6328, 6465, 6450, 6217, 6231, 6160, 6436, 6683, 6328, 6448, 6454, 6216, 6610, 6436, 6683, 6282, 6221, 6602, 6310, 6282, 5984, 5899, 5925, 6229, 6435, 6500, 6281, 6722, 6013, 6465, 6450, 6160, 6603, 6683, 6282, 6282, 6231, 6218, 6310, 6282, 5984, 6282, 6283, 6043, 5914, 6282, 5933, 6328, 6465, 6450, 6647, 6604, 6282, 6000, 6282, 6282, 6283, 6277, 6707, 6733, 6282, 6722, 6465, 6230, 6396, 6282, 6282, 6282, 6235, 6686, 6282, 6708, 6281, 6282, 6000, 6437, 6362, 6338, 6279, 6282, 6282, 6283, 6444, 6145, 6683, 6282, 6282, 5933, 6003, 6282, 6282, 6282, 6006, 6375, 6465, 6708, 6282, 6005, 6282, 6282, 6287, 6289, 6282, 6548, 6282, 6282, 5933, 6282, 6282, 6282, 6137, 6282, 6282, 6020, 6282, 6375, 6282, 6282, 6282, 6281, 6471, 6218, 6683, 6282, 6282, 6282, 6370, 5984, 6282, 6010, 6795, 5984, 5899, 6282, 6282, 6661, 6282, 6000, 6137, 6282, 6282, 6282, 6282, 5896, 5914, 6704, 6273, 5928, 6340, 6344, 6503, 6348, 6349, 6349, 6350, 6355, 6352, 6349, 6349, 6349, 6349, 6354, 6355, 6355, 6355, 6356, 6357, 6357, 6357, 6357, 6357, 6358, 6359, 6359, 6359, 6359, 6383, 6360, 6382, 6382, 6382, 6361, 6282, 6282, 6282, 6283, 6043, 5903, 5897, 6485, 6376, 6395, 6425, 6282, 6282, 6282, 6293, 6291, 6338, 6282, 6282, 6282, 6284, 6282, 6282, 6345, 6282, 6282, 6291, 6282, 6282, 6282, 6230, 5876, 6282, 6407, 6282, 6282, 6282, 6286, 6282, 6282, 6037, 6411, 6417, 6282, 6282, 5979, 5903, 6470, 6313, 6647, 6281, 6282, 6282, 6408, 6416, 6418, 6282, 6282, 5979, 6525, 6458, 6342, 6422, 6282, 6282, 6282, 6288, 6282, 6664, 6731, 6282, 6012, 6000, 6282, 6012, 6335, 6310, 5985, 6504, 6282, 6282, 6273, 6470, 6372, 6273, 6504, 6385, 6282, 6282, 6412, 6257, 6598, 6600, 6282, 6411, 6413, 6597, 6599, 6599, 6282, 6282, 6282, 6291, 6410, 6282, 6322, 6451, 5926, 5926, 6008, 6008, 6008, 5932, 6236, 6282, 6282, 6282, 6292, 5982, 6282, 6283, 6442, 6050, 6013, 6609, 6259, 6424, 6310, 6282, 5919, 5920, 6282, 6282, 6011, 5916, 6282, 6664, 6282, 6282, 6023, 6437, 6459, 5983, 6013, 6594, 6700, 6437, 6282, 6424, 6437, 6282, 6282, 6235, 6480, 6481, 6282, 6282, 6133, 6282, 6282, 6485, 6376, 6647, 6282, 6016, 6282, 6282, 5933, 6330, 6454, 6217, 6683, 6282, 6282, 6484, 5917, 5929, 6011, 6608, 6259, 6425, 6282, 6486, 5983, 6013, 6594, 6786, 6437, 6282, 6282, 6282, 6294, 6282, 6283, 6484, 5917, 5983, 6013, 6531, 6471, 6282, 6282, 6146, 6683, 6282, 6282, 6155, 6282, 6282, 6385, 6282, 6322, 6282, 6282, 6275, 6493, 5917, 5983, 6324, 6259, 6729, 6282, 6019, 6019, 6019, 5991, 5904, 6277, 5903, 6531, 6011, 6148, 6282, 6282, 6282, 6234, 6531, 6282, 6282, 6282, 6322, 6282, 6133, 6372, 6505, 6282, 6230, 6282, 6282, 6171, 6756, 6531, 6046, 6834, 6437, 6282, 6282, 6285, 6530, 5924, 6282, 6684, 6282, 6149, 6282, 6282, 6282, 6325, 6285, 6530, 6045, 6159, 6024, 6046, 6386, 6282, 6282, 6282, 6372, 6282, 6294, 6282, 6530, 6464, 6386, 6282, 6282, 6215, 6224, 5933, 5924, 6410, 6282, 6040, 6282, 6482, 6282, 6282, 6282, 6531, 6536, 6024, 6282, 6282, 6282, 6370, 5987, 6282, 6535, 6282, 6282, 6227, 6282, 6282, 6282, 6378, 6378, 6378, 6282, 6452, 5994, 6282, 6282, 6282, 6394, 6282, 6282, 6282, 6404, 6539, 6333, 6683, 6282, 6020, 6282, 6282, 6282, 6341, 6282, 6570, 6300, 6569, 6301, 6370, 6545, 6300, 6302, 6303, 6303, 6303, 6303, 6303, 6304, 6551, 6554, 6552, 6561, 6552, 6555, 6563, 6564, 6563, 6563, 6563, 6563, 6564, 6556, 6557, 6557, 6557, 6557, 6558, 6372, 6273, 6282, 6282, 6243, 6282, 6509, 6566, 6282, 6282, 6271, 6213, 6282, 6283, 6568, 6282, 6021, 6041, 6282, 6027, 6282, 6321, 6282, 6006, 6282, 6282, 6027, 6462, 6282, 6282, 6282, 6572, 6282, 6168, 6282, 6575, 6510, 6282, 6029, 6282, 6282, 6282, 6588, 5907, 5909, 5911, 6282, 6036, 6282, 6282, 6388, 6407, 5908, 5910, 6282, 6282, 6272, 6282, 6526, 6282, 5916, 6322, 6282, 6378, 6282, 6020, 6282, 6278, 6282, 6282, 6394, 5924, 6278, 6282, 6283, 6502, 6365, 6282, 6579, 6282, 6282, 6275, 6525, 5914, 5992, 6371, 6048, 6365, 6367, 6369, 6282, 6282, 6277, 6531, 6046, 6275, 6583, 5993, 6047, 6256, 6256, 6366, 6368, 6282, 6282, 6452, 6000, 6282, 6282, 6282, 6408, 6282, 6027, 6282, 6372, 6526, 6282, 5899, 6539, 6664, 6682, 6282, 6282, 6282, 6146, 6282, 6482, 6282, 6006, 6451, 6282, 6282, 6282, 5985, 6458, 6342, 6313, 6136, 6152, 6282, 6037, 6378, 6282, 6282, 6282, 6421, 6584, 6020, 6437, 6683, 6282, 6027, 6321, 6282, 6451, 6646, 6152, 6282, 6282, 6282, 6410, 6282, 6282, 6282, 6375, 6282, 6000, 6526, 6322, 6533, 6682, 6282, 6408, 6282, 6282, 6282, 6505, 6282, 6282, 6278, 6282, 6282, 6282, 6143, 6371, 6472, 6282, 6282, 6282, 6423, 6539, 6437, 6683, 6282, 6040, 6148, 6040, 6149, 6273, 6229, 6682, 6282, 6470, 6282, 6282, 6283, 6282, 6282, 6282, 6524, 6548, 5933, 6282, 5933, 6282, 6434, 6410, 6282, 6282, 6434, 6537, 6282, 6664, 6491, 6866, 5876, 6482, 6491, 6282, 6282, 6283, 6410, 6282, 6427, 6282, 6282, 5980, 6011, 6276, 6037, 6285, 6038, 6590, 6783, 6607, 6612, 6620, 6619, 6620, 6620, 6616, 6620, 6620, 6618, 6622, 6620, 6614, 6613, 6624, 6627, 6626, 6627, 6627, 6628, 6630, 6630, 6634, 6630, 6632, 6631, 6630, 6630, 6630, 6636, 6638, 6638, 6637, 6637, 6640, 6641, 6282, 6132, 6282, 6282, 5895, 5924, 5984, 6282, 6640, 6637, 6640, 6640, 6640, 6640, 6505, 5903, 6644, 6331, 6649, 6282, 6282, 6229, 5879, 6282, 6527, 6653, 6148, 6282, 6137, 5924, 6282, 6282, 6282, 6230, 6281, 6282, 6282, 6306, 6402, 6433, 6346, 6656, 6658, 6549, 6282, 6282, 6283, 6475, 6423, 6282, 6282, 6282, 6437, 6682, 5912, 6668, 6400, 6282, 6157, 6282, 6282, 5896, 6451, 5980, 5917, 6029, 6221, 6281, 6282, 6282, 6524, 6282, 6282, 6282, 5930, 6664, 6282, 6264, 6683, 6694, 6282, 6282, 6282, 6452, 6282, 6282, 6282, 6285, 5897, 6453, 6654, 6282, 6346, 6295, 6282, 6282, 6283, 6582, 6297, 6282, 6681, 6282, 6164, 6024, 6282, 6133, 6451, 6282, 5988, 6281, 6285, 5980, 6458, 6371, 6263, 6647, 6281, 6282, 6676, 6282, 6676, 6514, 5903, 6680, 6465, 6489, 5995, 6282, 6282, 6283, 6496, 6085, 6282, 6513, 6525, 6679, 6688, 6688, 6399, 6713, 6281, 6282, 6286, 6282, 6037, 6282, 6282, 6282, 5899, 6691, 6497, 6724, 6488, 6734, 6282, 6282, 6282, 6498, 6282, 6282, 6282, 6255, 6282, 6283, 6691, 6696, 6724, 6488, 6698, 6331, 6222, 6282, 6282, 6282, 6431, 6296, 6282, 6282, 6282, 6499, 6282, 6282, 5903, 6464, 6231, 6734, 6696, 6134, 6733, 6281, 6693, 6282, 6282, 6434, 6282, 6282, 6282, 6277, 6525, 6531, 6471, 6218, 6231, 6490, 6282, 6282, 6282, 6504, 6548, 6282, 6283, 6278, 6282, 6275, 6710, 6733, 6282, 6169, 6282, 6282, 6509, 6003, 6712, 6282, 6282, 6282, 6508, 5899, 6702, 6490, 6282, 6282, 6283, 6861, 6854, 6033, 6858, 6646, 6218, 6604, 6282, 6282, 6282, 6577, 6286, 6282, 6394, 6282, 6229, 6019, 6019, 6019, 6019, 5904, 6286, 6282, 6137, 6282, 6229, 6282, 6292, 6229, 6282, 6282, 6372, 6505, 6282, 6437, 6683, 6282, 6229, 6146, 6265, 6282, 6283, 6525, 5927, 6539, 6729, 6683, 6282, 6282, 6650, 6282, 6282, 6283, 6863, 6364, 6024, 6282, 6282, 6283, 6869, 6282, 6362, 6282, 6362, 6363, 6010, 6592, 5925, 6719, 6591, 6593, 6718, 6721, 5934, 5934, 5935, 6469, 6736, 5934, 5934, 5934, 6736, 6738, 6742, 6740, 6739, 6744, 6746, 6751, 6751, 6752, 6753, 6747, 6748, 6748, 6748, 6749, 6751, 6751, 6751, 6753, 6753, 6753, 6753, 6758, 6754, 6755, 6282, 6282, 6285, 6282, 6282, 6282, 6014, 6760, 5981, 6007, 6728, 6282, 6763, 6282, 6282, 6285, 6496, 6531, 6282, 6283, 6768, 6770, 6282, 6282, 6282, 6539, 6392, 6282, 6282, 6392, 6027, 6156, 6282, 6282, 6293, 6586, 6282, 6373, 6281, 6285, 6507, 5903, 6470, 6326, 6505, 6659, 6282, 6234, 6229, 5879, 6283, 6496, 6703, 6776, 6704, 6777, 6282, 6282, 6282, 6548, 6282, 5918, 6025, 6282, 6282, 6026, 6282, 6286, 6505, 6394, 6282, 6291, 6370, 6282, 6370, 6282, 6496, 6085, 6831, 6147, 6603, 6683, 6282, 6380, 6282, 6282, 6370, 6282, 6282, 6282, 6003, 6282, 6294, 6336, 6000, 6282, 6239, 6846, 6282, 6260, 6282, 6282, 6282, 6642, 6379, 6282, 6505, 6336, 6283, 6702, 6782, 6319, 6218, 6604, 6282, 6785, 5999, 6282, 6269, 6663, 6282, 6140, 6282, 6282, 6393, 6282, 5982, 6724, 6319, 6218, 6604, 6539, 6310, 6282, 6282, 6375, 6370, 6375, 6788, 6320, 6310, 6000, 6000, 6282, 6282, 6375, 6548, 6282, 6282, 6282, 6275, 6084, 6447, 6282, 6788, 6429, 6683, 6282, 6282, 6664, 6683, 6282, 6273, 6541, 6148, 6282, 6394, 6000, 6282, 6274, 6282, 6003, 6318, 6429, 6683, 6282, 6275, 5898, 5915, 6282, 6282, 6370, 6370, 6370, 6664, 6282, 6683, 6282, 6410, 6282, 6699, 6282, 6275, 5901, 5982, 6724, 6291, 6282, 6282, 6531, 6689, 6282, 6282, 6220, 6282, 6537, 6282, 6282, 6282, 6664, 6146, 6282, 6282, 6434, 6281, 6282, 6282, 6282, 6020, 6282, 6006, 6410, 6408, 6282, 6027, 6282, 6282, 6250, 6843, 6434, 6410, 6282, 6434, 6410, 6282, 6408, 6282, 6322, 6282, 6410, 6410, 6027, 6282, 6275, 6702, 6231, 6490, 6146, 6282, 6146, 6282, 6282, 6282, 6671, 6146, 6410, 6408, 6408, 6409, 6409, 6409, 6409, 6409, 6282, 6282, 6282, 6682, 6282, 6282, 6285, 6039, 6216, 6221, 6281, 6286, 6282, 6282, 6790, 6543, 6282, 6505, 6003, 6282, 6559, 6282, 6282, 6559, 6508, 6792, 6282, 6077, 6139, 6282, 6282, 6379, 6374, 6793, 6806, 6798, 6798, 6801, 6799, 6798, 6799, 6798, 6797, 6803, 6808, 6807, 6808, 6805, 6808, 6809, 6810, 6813, 6814, 6815, 6816, 6815, 6818, 6811, 6813, 6811, 6813, 6820, 6822, 6822, 6821, 6821, 6824, 6824, 6824, 6824, 6821, 6824, 6824, 6828, 6825, 6826, 6282, 6282, 6282, 6683, 6282, 6282, 6689, 6439, 6282, 6277, 6715, 6282, 6282, 6282, 6510, 6035, 6282, 6282, 6282, 6684, 6282, 6282, 6282, 6233, 6282, 6716, 6282, 6580, 6419, 6282, 6836, 6282, 6281, 6282, 6282, 6283, 6293, 6337, 6282, 6838, 6840, 6282, 6282, 6666, 6282, 6143, 6292, 6282, 6282, 6292, 6282, 6282, 6282, 6378, 6845, 6282, 6282, 6845, 6282, 6487, 6282, 6282, 6391, 6391, 6651, 6282, 6282, 6282, 6730, 6512, 6282, 6282, 6282, 6830, 6853, 5931, 6141, 6428, 6602, 6604, 6282, 6282, 6409, 6410, 6282, 6852, 6854, 6856, 6450, 6002, 6282, 6001, 6003, 6282, 6645, 6231, 6281, 6282, 6282, 6674, 6282, 6167, 6282, 6282, 6000, 6282, 6000, 6034, 6282, 6282, 6282, 6848, 6282, 6394, 6683, 6282, 6282, 5933, 6003, 5933, 6003, 6282, 6830, 6645, 6231, 6222, 6487, 6282, 6024, 6282, 6282, 6693, 6282, 6282, 6282, 6464, 6861, 6854, 6009, 6377, 6525, 5931, 6011, 6465, 6282, 6830, 5885, 6282, 6282, 6702, 5931, 6319, 6436, 6683, 5916, 6282, 6282, 5925, 6282, 6282, 6448, 6282, 6282, 6931, 6282, 6282, 6451, 6282, 6282, 6282, 6860, 6044, 6011, 6465, 6449, 6647, 6310, 6282, 6282, 6142, 6436, 6683, 6282, 6282, 6732, 5924, 5984, 6861, 5983, 6377, 6454, 5982, 6321, 6282, 6294, 6138, 6282, 6461, 6151, 6437, 6282, 6282, 6732, 6772, 6860, 5983, 6465, 6151, 6437, 6282, 6865, 6282, 6282, 6282, 6860, 6337, 6437, 6282, 6282, 6466, 6282, 6144, 6436, 6282, 6282, 6282, 6868, 6219, 6282, 6282, 6282, 6468, 6282, 6282, 6732, 6282, 5916, 6262, 6299, 6282, 5918, 5918, 5918, 6378, 6282, 6282, 6526, 6282, 6148, 6282, 6283, 6293, 6586, 6292, 6502, 6437, 6282, 6282, 6282, 6294, 6282, 6292, 6294, 6282, 6282, 6474, 5980, 6293, 6291, 6282, 6282, 6292, 6502, 6282, 6278, 6294, 6282, 6278, 6282, 6282, 6765, 6282, 6229, 6362, 6282, 6230, 6320, 6500, 6282, 6505, 6282, 6292, 6502, 6282, 6505, 6233, 6282, 6282, 6293, 6282, 6282, 6282, 6474, 6478, 6291, 6278, 6282, 6282, 6292, 6531, 6536, 6875, 6255, 6282, 6282, 6476, 6282, 6006, 6255, 6282, 6006, 6390, 6282, 6314, 6049, 6282, 6877, 6878, 6881, 6375, 6880, 5930, 6282, 6532, 6542, 6883, 6596, 6258, 6885, 6596, 6595, 6595, 6886, 6889, 6595, 6888, 6890, 6889, 6892, 6897, 6893, 6894, 6895, 6899, 6900, 6905, 6905, 6901, 6902, 6912, 6903, 6902, 6906, 6905, 6906, 6905, 6906, 6907, 6909, 6908, 6910, 6914, 6908, 6908, 6922, 6922, 6921, 6922, 6923, 6916, 6917, 6918, 6920, 6926, 6925, 6928, 6460, 6371, 6282, 6282, 6482, 5932, 6321, 6285, 6282, 6282, 6494, 6371, 6930, 6282, 6282, 6282, 6505, 6494, 6371, 6130, 6282, 6282, 6282, 6505, 6506, 6282, 6282, 6282, 6312, 6933, 6282, 6505, 6935, 6282, 6282, 6937, 6282, 6282, 6767, 6769, 6290, 6282, 6282, 6282, 6505, 6692, 6011, 6313, 6291, 6233, 6282, 6282, 6508, 6282, 6282, 6282, 5916, 6030, 6282, 6423, 6170, 6282, 6282, 6170, 6011, 6372, 6282, 5916, 6282, 6282, 5915, 6451, 6440, 6282, 6282, 5918, 6322, 6283, 6939, 6446, 6705, 6447, 6291, 6282, 6282, 6294, 6282, 6282, 6282, 6438, 6282, 6282, 6511, 6282, 6282, 6282, 6842, 6149, 6282, 6149, 6282, 6282, 6779, 6007, 6728, 6310, 6282, 6282, 6319, 6310, 6282, 6282, 6363, 6282, 6692, 6011, 6372, 5916, 6282, 5915, 6282, 6282, 6511, 6268, 6286, 6282, 5924, 6282, 6282, 6833, 6282, 6279, 6398, 6282, 6282, 6014, 6282, 6282, 6282, 6601, 6282, 6439, 5933, 6133, 6282, 6148, 6282, 6255, 6282, 5914, 5931, 6705, 6282, 6282, 6850, 6282, 6850, 6282, 6372, 6282, 6372, 6282, 6149, 6282, 6505, 5914, 5931, 6291, 6282, 6282, 5982, 6585, 6282, 6282, 6660, 6291, 6043, 5914, 6291, 6282, 6282, 6283, 6043, 5899, 6282, 6282, 1048576, 1073741824, 0x80000000, 1075838976, 2097152, 0x80000000, 4194560, 4196352, -2143289344, -2143289344, 4194304, 0x80000000, 37748736, 541065216, 541065216, -2143289344, 4198144, 4196352, 276901888, 8540160, 4194304, 1, 4, 16, 64, 0, 48, 64, 32, 64, 64, 96, 0, 59, 140224, 5505024, -1887436800, 0, 63, 64, 128, 0, 64, 256, 0, 110, 110, 8425488, 4194304, 1024, 0, 128, 128, 512, 512, 1024, 1024, 2048, 0, 256, 256, 257, 37748736, 742391808, 742391808, 775946240, -1371537408, 775946240, 4718592, 775946240, 775946240, 171966464, 171966464, 775946240, 239075328, -1405091840, -1371537408, 239075328, 171966464, 64, 4718592, 2097216, 4720640, 541589504, 4194368, 4194368, 541065280, 541589504, 4194400, -2143289280, 4194368, -2143285440, -2143285408, -2143285408, -2109730976, 775946336, 776470528, -2143285408, 776470528, 775946304, 775946304, -1908404384, 775946304, -1908404384, 2, 8, 32, 128, 1024, 4096, 0, 260, 8392704, 0, 1856, 64, 524288, 64, 896, 8192, 67108864, 0x80000000, 96, 262144, 262144, 8192, 0, 288, 8388608, 0, 384, 0, 512, 2048, 2048, 4096, 4096, 8192, 8192, 16384, 0, 520, 520, 96, 524288, 524288, 0, 2432, 2048, 268435456, 0, 24576, 0, 32768, 32768, 65536, 1048576, 128, 2048, 12288, 0, 12289, 0, 16384, 16384, 32768, 0, 1864, 2, 16, 1024, 98304, 131072, 262144, 1048576, 512, 5120, 2, 536936448, 80, 528, 528, 2097168, 2097168, 268435472, 524304, 1048592, 2097168, 24, 560, 48, 2097680, 1048592, 3145744, 1048592, 20, 560, 48, 3146256, 2097552, 3146256, 28, 16, 8192, 2, 2098064, 163577856, 17, 21, 112, 128, 3584, 8192, 2228784, -161430188, -161430188, -161429680, -161430188, -161429676, -161430188, -161298576, -160299088, -161298576, -161298572, -160774288, -160299084, 146804757, 146812949, 146862101, 146863389, 146863389, 148960541, -161429740, -161429676, 146863421, 148960541, 146863389, -161429676, -160905388, -161429676, -161429676, -161429675, -161349072, -161349072, -161347728, -161347728, -161298572, -161298572, -160774284, -161298572, 16, 262160, -18860267, -160774284, -18729163, 0, 58368, 159383552, 0, 65536, 2097152, 8388608, 33554432, 0, 66048, 0, 77824, 524288, 33554432, 1024, 262144, 2097152, 16777216, 67108864, 0, 131072, 524288, 134217728, 0x80000000, 1, 32768, 196608, 0, 131328, 131072, 16777216, 100663296, -1073741824, 164096, 0, 131584, 2621440, 0, 139264, 0, 150528, 0, 235712, 16777216, 1073774592, 1226014816, 100665360, -2046818288, 100665360, 100665360, -2044196848, 1091799136, 1091799136, 1091803360, 1091799136, 1158908000, 1158908001, 1192462432, 1192462448, 1192462448, 1870638912, 1870655296, 1870638912, 1200851056, 1200851056, 1091799393, 1870655296, 1870655296, 1870655312, 1870655316, 1870655312, 1870655312, 1870638928, 1870655316, 1870655316, 1870655317, 1870655316, 1879043952, 1870655348, 1870655316, 1879027568, 1879043952, 1879043956, 0, 284672, 229440, 1048576, 2097152, 67108864, 134217728, 8, 4194304, 16777216, 0x80000000, 1224736768, 0, 503616, 2048, 100663296, 0, 524288, 2097152, 4194304, 4194304, 0, 40, 0, 44, -2046820352, 0, 605503, 231488, 1090519040, 1157627904, 1191182336, 9437184, 231744, 520000, 7864320, 1862270976, 0, 867391, 1862270976, 1862270976, 16252928, 0, 1048576, 4194304, 25165824, 25165824, 33554432, 8192, 98304, 1048576, 8388608, 134217728, 268435456, 4194432, 3145728, 0, 24, 0, 29, 0, 32, 1, 2, 2, 4, 0, 0x80000000, 0x80000000, 0, 0, 1, 0, 2, 0, 3, 240, 19456, 262144, 0, 4, 8, 0, 6, 0, 7, 150994944, 0, 1049088, 1049088, 12845065, 12845065, 147193865, 128, 6144, 4194304, 251658240, 536870912, 1073741824, 32768, 131072, 1048576, 4096, 83886080, 117440512, 0, 3145728, 16777216, 134217728, 0, 2048, 8192, 229376, 0, 2304, 1536, 8192, 1536, 65536, 4194304, 67108864, 536870912, 9216, 33554432, 262144, 134217728, 1073741824, 50331649, 9476, 512, 8192, 134218240, 1050624, 0, 5242880, 1275208192, 4194312, 4194312, 4194344, 4194312, 541065224, 4203820, -869654016, -869654016, 1279402504, 1279402504, 2143549415, 2143549415, 2143549423, 0, 8388608, 4096, 4194304, 8388608, 16777216, 33554432, -1946157056, 0, 8192, 131072, 0, 1792, 0, 1024, 8192, 65536, 0, 1536, 0x80000000, 2143549423, 2143549423, 2143549415, 1, 16777216, 268435456, 512, 139264, 2760704, -872415232, 0, 19947520, 0, 33554432, 67108864, 1073741824, 1073741824, 262144, 7340032, -2030043136, 0, 331776, 300, 4203520, 4333568, 1275068416, 0, 16777216, 16777216, 0, 999, 259072, 4194304, 4194432, 999, 29619200, 2113929216, 0, 58720256, 1007, 1007, 0, 67108864, 402653184, 536870912, 2048, 1048576, 16777216, 536870912, 300, 0, 83886080, 0, 2097152, 134217728, 536870912, 0, 49152, 0, 57344, 102, 384, 6, 96, 128, 3072, 16384, 65536, 524288, 1048576, 0, 4096, 262144, 524288, 96, 96, 64, 384, 512, 4096, 65536, 131072, 1024, 65536, 262144, 131072, 32768, 256, 384, 8192, 33554432, 0x80000000, 1, 6, 8, 8388608, 96, 384, 104, 104, 0, 134217728, 6, 32, 256, 512, 65536, 7340032, 50331648, 0, 8396800, 4, 32, 384, 4, 64, 1024, 2097152, 268435456, 1073741824, 8, 262144, 512, 0, 8, 8, 16, 0, 9, 0, 12, 0, 15, 16, 16, 17, 20, 16, 20, 48, 16, 28, 0, 16, 32, 0, 21, 53, 4, 256, 1024, 524288, 536870912, 256, 65536, 16777216, 1073741824, 2048, 524288, 32, 4100, 1024, 134217728, 1049088, 270532608, 2097152, 2097152, 0, 23, 5505537, 5587457, 5591557, 147202057, 5587457, 13894153, -1881791493, -1881791493, 0, 134218752, 5587465, 5587457, 13894153, 13894153, 81003049, 4456448, 8388608, 5505024, 0, 134348800, 134348800, 82432, 0, 142606336, 5, 86528, 41, 75497472, 81920, 0, 184549376, 2, 56, 64, 2048, 262144, 536870912, 2048, 134217728, -2113929216, 16777216, 1073743872, 268435968, 229376, 25165824, 92274688, 25165824, 100663296, 402653184, 1610612736, 0, 100663296, 134217728, 805306368, 1073741824, 8388608, 268567040, 16384, 229376, 4194304, 117440512, 2113544, 68423701, -2079059883, -2079059947, 85200917, 68423701, 68423765, 68489237, 68423701, 68423701, 72618005, 68423701, 68425749, 68423703, 85200919, 69488664, 69488664, 70537244, 70537245, 70537245, -2076946339, -2076946403, 70537245, 70537309, 70539293, -2022351745, -2022351617, -2022351745, -2022351617, -2022351617, 0, 243269632, 256, 32768, 1048576, 33554432, 134217728, -2113929216, 0, 268435456, 49152, 266240, 1048576, 67108864, -2080374784, -2080374784, 268288, 0, 301989888, 0, 318767104, 282624, 0, 536870912, 28, 3145728, 192, 351232, 7340032, 5, 16, 1049104, 12, 3145728, 13, 0, 1073741825, 192, 3072, 20480, 0, 1073741824, 0, 262144, 2621440, -1073741824, 20480, 65536, 268435456, 14, 32, 512, 131072, 268435456, 192, 1024, 64, 32768, 33554432, 268435456, 4, 128, 3840, 16384, 262144, 128, 2097152, 1073741824, 4, 2097152, 4, 50331648, 67108864, 128, 50331648, 1073741824, 128, 268435968, 268435968, 268436032, 256, 1536, 2048, 16384, 98304, 393216, 524288, 268435456, 536870912, 9216, 0, 4194304, 50331648, 0x80000000, 256, 536871168, -1879046336, -1879046334, -1879046326, -1879046334, 1073744256, -1879046334, -1879046326, -1845491902, -1878784182, 268444480, 268436288, 268436288, 268436289, 268444480, 268444480, 2100318149, 2100318149, 2100326341, 0, 1090519040, 2100326341, 2100326341, 1, 16, 536936448, 576, 0, 832, 8192, 1, 4036, 19939328, 2080374784, 0, 1, 1024, 768, 8192, 16384, 19922944, 2080374784, 1, 128, 4096, 3584, 16384, 524288, 8, 33554432, 402653184, 2048, 3145728, 128, 131072, 268500992, 4243456, 4096, 1048588, 0, 1258292224, 1124073472, 1124073472, 1124073488, 1124073474, 1124073472, 1392574464, 1124073472, 1073754113, 12289, 1124073472, 12289, 12289, 1098920193, 1132474625, 1098920209, 1132474625, 1132474625, 1124085761, 1124085761, 1124085777, 1258304513, 1124085761, 1400975617, 2132360255, 2132622399, 2132360255, 2132622399, 2132622399, 2141011263, 0, 2140749119, 2141011263, 2, 16384, 3145728, 12545, 25165824, 268435456, 12305, 13313, 12561, 0, 78081, 327155712, 605247, 1058013184, 1073741824, 867647, 1066401792, 0, 1, 12288, 256, 8388608, 1, 30, 32, 1024, 2048, 339968, 327680, 524288, 1, 14, 16, 14, 1024, 16384, 4194304, 134217728, 1, 12, 1024, 8, 134217728, 8, 536870912, 9437184, 0, 68157440, 137363456, 0, 137363456, 66, 66, 100680704, 25165824, 26214400, 92274688, 25165952, 93323264, 92274688, 92274688, 92274720, 93323264, 25165890, 100721928, 100721928, 100787464, 100721664, 100721664, 100853000, 100721928, 125977600, 125846528, 125846528, 125846560, 125977600, 125977600, 127026176, 281843, 281843, 1330419, 281843, 126895104, 125846528, 1330419, 1330419, 72633587, 5524723, 72633587, 92556531, 93605107, 93605107, 5524723, 5524723, 39079155, 97799411, 127290611, 127290611, 131484915, 0, 17408, 33554432, 1073741824, 58624, 0, 124160, 189696, 148480, 50331648, 2, 112
];

XQueryParser.TOKEN =
[
  "(0)",
  "PragmaContents",
  "DirCommentContents",
  "DirPIContents",
  "CDataSection",
  "Wildcard",
  "EQName",
  "URILiteral",
  "IntegerLiteral",
  "DecimalLiteral",
  "DoubleLiteral",
  "StringLiteral",
  "PredefinedEntityRef",
  "'\"\"'",
  "EscapeApos",
  "ElementContentChar",
  "QuotAttrContentChar",
  "AposAttrContentChar",
  "PITarget",
  "NCName",
  "QName",
  "S",
  "S",
  "CharRef",
  "CommentContents",
  "EOF",
  "'!'",
  "'!='",
  "'\"'",
  "'#'",
  "'#)'",
  "'$'",
  "'%'",
  "''''",
  "'('",
  "'(#'",
  "'(:'",
  "')'",
  "'*'",
  "'*'",
  "'+'",
  "','",
  "'-'",
  "'-->'",
  "'.'",
  "'..'",
  "'/'",
  "'//'",
  "'/>'",
  "':'",
  "':)'",
  "'::'",
  "':='",
  "';'",
  "'<'",
  "'<!--'",
  "'</'",
  "'<<'",
  "'<='",
  "'<?'",
  "'='",
  "'>'",
  "'>='",
  "'>>'",
  "'?'",
  "'?>'",
  "'@'",
  "'NaN'",
  "'['",
  "']'",
  "'after'",
  "'all'",
  "'allowing'",
  "'ancestor'",
  "'ancestor-or-self'",
  "'and'",
  "'any'",
  "'append'",
  "'array'",
  "'as'",
  "'ascending'",
  "'at'",
  "'attribute'",
  "'base-uri'",
  "'before'",
  "'boundary-space'",
  "'break'",
  "'by'",
  "'case'",
  "'cast'",
  "'castable'",
  "'catch'",
  "'check'",
  "'child'",
  "'collation'",
  "'collection'",
  "'comment'",
  "'constraint'",
  "'construction'",
  "'contains'",
  "'content'",
  "'context'",
  "'continue'",
  "'copy'",
  "'copy-namespaces'",
  "'count'",
  "'decimal-format'",
  "'decimal-separator'",
  "'declare'",
  "'default'",
  "'delete'",
  "'descendant'",
  "'descendant-or-self'",
  "'descending'",
  "'diacritics'",
  "'different'",
  "'digit'",
  "'distance'",
  "'div'",
  "'document'",
  "'document-node'",
  "'element'",
  "'else'",
  "'empty'",
  "'empty-sequence'",
  "'encoding'",
  "'end'",
  "'entire'",
  "'eq'",
  "'every'",
  "'exactly'",
  "'except'",
  "'exit'",
  "'external'",
  "'first'",
  "'following'",
  "'following-sibling'",
  "'for'",
  "'foreach'",
  "'foreign'",
  "'from'",
  "'ft-option'",
  "'ftand'",
  "'ftnot'",
  "'ftor'",
  "'function'",
  "'ge'",
  "'greatest'",
  "'group'",
  "'grouping-separator'",
  "'gt'",
  "'idiv'",
  "'if'",
  "'import'",
  "'in'",
  "'index'",
  "'infinity'",
  "'inherit'",
  "'insensitive'",
  "'insert'",
  "'instance'",
  "'integrity'",
  "'intersect'",
  "'into'",
  "'is'",
  "'item'",
  "'json'",
  "'json-item'",
  "'key'",
  "'language'",
  "'last'",
  "'lax'",
  "'le'",
  "'least'",
  "'let'",
  "'levels'",
  "'loop'",
  "'lowercase'",
  "'lt'",
  "'minus-sign'",
  "'mod'",
  "'modify'",
  "'module'",
  "'most'",
  "'namespace'",
  "'namespace-node'",
  "'ne'",
  "'next'",
  "'no'",
  "'no-inherit'",
  "'no-preserve'",
  "'node'",
  "'nodes'",
  "'not'",
  "'object'",
  "'occurs'",
  "'of'",
  "'on'",
  "'only'",
  "'option'",
  "'or'",
  "'order'",
  "'ordered'",
  "'ordering'",
  "'paragraph'",
  "'paragraphs'",
  "'parent'",
  "'pattern-separator'",
  "'per-mille'",
  "'percent'",
  "'phrase'",
  "'position'",
  "'preceding'",
  "'preceding-sibling'",
  "'preserve'",
  "'previous'",
  "'processing-instruction'",
  "'relationship'",
  "'rename'",
  "'replace'",
  "'return'",
  "'returning'",
  "'revalidation'",
  "'same'",
  "'satisfies'",
  "'schema'",
  "'schema-attribute'",
  "'schema-element'",
  "'score'",
  "'self'",
  "'sensitive'",
  "'sentence'",
  "'sentences'",
  "'skip'",
  "'sliding'",
  "'some'",
  "'stable'",
  "'start'",
  "'stemming'",
  "'stop'",
  "'strict'",
  "'strip'",
  "'structured-item'",
  "'switch'",
  "'text'",
  "'then'",
  "'thesaurus'",
  "'times'",
  "'to'",
  "'treat'",
  "'try'",
  "'tumbling'",
  "'type'",
  "'typeswitch'",
  "'union'",
  "'unique'",
  "'unordered'",
  "'updating'",
  "'uppercase'",
  "'using'",
  "'validate'",
  "'value'",
  "'variable'",
  "'version'",
  "'weight'",
  "'when'",
  "'where'",
  "'while'",
  "'wildcards'",
  "'window'",
  "'with'",
  "'without'",
  "'word'",
  "'words'",
  "'xquery'",
  "'zero-digit'",
  "'{'",
  "'{{'",
  "'{|'",
  "'|'",
  "'||'",
  "'|}'",
  "'}'",
  "'}}'"
];

},
{}],
11:[function(_dereq_,module,exports){
'use strict';

exports.TreeOps = {
    flatten: function(node){
        var that = this;
        var value = '';
        if(!node) {
            throw new Error('Invalid node found');
        } else if (node.value === undefined) {
            node.children.forEach(function(child){
                value += that.flatten(child);
            });
        } else {
            value += node.value;
        }
        return value;
    },
    
    concat: function(obj1, obj2, copy){
        var result = copy ? {} : obj1;
        if(copy){
            Object.keys(obj1).forEach(function(key){
                result[key] = obj1[key];
            });
        }
        var keys = Object.keys(obj2);
        keys.forEach(function(key){
            result[key] = obj2[key];
        });
        return result;
    },
    
    removeParentPtr: function(ast){
        if(ast.getParent !== undefined) {
            delete ast.getParent;
        }
        for(var i in ast.children) {
            var child = ast.children[i];
            this.removeParentPtr(child);
        }
    },
    
    inRange: function(p, pos, exclusive){
        if(p && p.sl <= pos.line && pos.line <= p.el) {
            if(p.sl < pos.line && pos.line < p.el) {
                return true;
            } else if(p.sl === pos.line && pos.line < p.el) {
                return p.sc <= pos.col;
            } else if(p.sl === pos.line && p.el === pos.line) {
                return p.sc <= pos.col && pos.col <= p.ec + (exclusive ? 1 : 0);
            } else if(p.sl < pos.line && p.el === pos.line) {
                return pos.col <= p.ec + (exclusive ? 1 : 0);
            }
        }
    },
    
    findNode: function(ast, pos) {
        if(!ast) {
            return;
        }
        var p = ast.pos;
        if(this.inRange(p, pos) === true) {
            for(var i in ast.children) {
                var child = ast.children[i];
                var n = this.findNode(child, pos);
                if(n !== undefined) {
                    return n;
                }
            }
            return ast;
        } else {
            return;
        }
    },
    
    astAsXML: function(node, indent){
        var result =  '';
        indent = indent ? indent : '';
        if(node.value) {
            result += (indent + '<' + node.name + '>' + node.value + '</' + node.name + '>\n');
        }
        result += indent + '<' + node.name + '>\n';
        var that = this;
        node.children.forEach(function(child){
            result += that.astAsXML(child, indent + '  ');
        });
        result += indent + '</' + node.name + '>\n';
        return result;
    }
};
},
{}],
12:[function(_dereq_,module,exports){
'use strict';

var JSONiqParser = _dereq_('./parsers/JSONiqParser').JSONiqParser;
var XQueryParser = _dereq_('./parsers/XQueryParser').XQueryParser;
var JSONParseTreeHandler = _dereq_('./parsers/JSONParseTreeHandler').JSONParseTreeHandler;
var Translator = _dereq_('./compiler/translator').Translator;
var StyleChecker = _dereq_('./formatter/style_checker').StyleChecker;
var completer = _dereq_('../lib/completion/completer');
    
var createStaticContext = exports.createStaticContext = function(){
    var StaticContext = _dereq_('./compiler/static_context').StaticContext;
    return new StaticContext();
};

var defaults = function(current, def){
    if(!current) {
        return def;
    }
    Object.keys(def).forEach(function(key){
        if(current[key] === undefined){
            current[key] = def[key];
        }
    });
    return current;
};

var convertPosition = function (code, begin, end) {
    var before = code.substring(0, begin);
    var after = code.substring(0, end);
    var startline = before.split('\n').length;
    var startcolumn = begin - before.lastIndexOf('\n');
    var endline = after.split('\n').length;
    var endcolumn = end - after.lastIndexOf('\n');
    var pos = {
        sl: startline - 1,
        sc: startcolumn - 1,
        el: endline - 1,
        ec: endcolumn - 1
    };
    return pos;
};

exports.XQLint = function (source, opts) {

    opts = opts ? opts : {};
    opts = defaults(opts, { styleCheck: false });

    var ast;
    this.getAST = function () {
        return ast;
    };

    var markers = [];
    this.getMarkers = function () {
        return markers;
    };
    
    this.getMarkers = function(type){
        var m = [];
        markers.forEach(function(marker){
            if(marker.type === type || type === undefined){
                m.push(marker);
            }
        });
        return m;
    };

    this.getErrors = function(){
        return this.getMarkers('error');
    };

    this.getWarnings = function(){
        return this.getMarkers('warning');
    };
    
    this.getCompletions = function(pos){
        return completer.complete(source, ast, sctx, pos);
    };

    var syntaxError = false;
    this.hasSyntaxError = function () {
        return syntaxError;
    };

    var sctx = opts.staticContext ? opts.staticContext : createStaticContext();
    var file = opts.fileName ? opts.fileName : '';
    var isJSONiq = ((file.substring(file.length - '.jq'.length).indexOf('.jq') !== -1) && source.indexOf('xquery version') !== 0) || source.indexOf('jsoniq version') === 0;
    var h = new JSONParseTreeHandler(source);
    var parser = isJSONiq ? new JSONiqParser(source, h) : new XQueryParser(source, h);
    try {
        parser.parse_XQuery();
    } catch (e) {
        if (e instanceof parser.ParseException) {
            syntaxError = true;
            h.closeParseTree();
            var pos = convertPosition(source, e.getBegin(), e.getEnd());
            var message = parser.getErrorMessage(e);
            if (pos.sc === pos.ec) {
                pos.ec++;
            }
            markers.push({
                pos: pos,
                type: 'error',
                level: 'error',
                message: message
            });
        } else {
            throw e;
        }
    }
    ast = h.getParseTree();
    if(opts.styleCheck) {
        markers = markers.concat(new StyleChecker(ast, source).getMarkers());
    }
    var translator = new Translator(sctx, ast);
    markers = markers.concat(translator.getMarkers());
};

},
{"../lib/completion/completer":6,"./compiler/static_context":4,"./compiler/translator":5,"./formatter/style_checker":7,"./parsers/JSONParseTreeHandler":8,"./parsers/JSONiqParser":9,"./parsers/XQueryParser":10}]},{},[12])
(12)

});

define("ace/mode/xquery/modules",["require","exports","module"], function(require, exports, module) {
    exports.Modules = 
{ "http://xbrl.io/modules/bizql/components" : { "ns" : "http://xbrl.io/modules/bizql/components", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">This module provides functionality for retrieving components.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Components help organizing the facts contained in archives\n in smaller parts that \"make sense\" together. A component is identified\n with a CID (component ID).</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">A component is made of networks (see the networks module) and hypercubes\n (see the hypercubes module).</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">With this module, you can retrieve all components, all components belonging\n to one or several archives. You can retrieve the CID of a component or retrieve\n the components associated with a couple of CIDs.</p>\n", "sees" : [  ], "authors" : [ "<xqdoc:author xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Charles Hoffman</xqdoc:author>", "<xqdoc:author xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Matthias Brantner</xqdoc:author>", "<xqdoc:author xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Dennis Knochenwefel</xqdoc:author>", "<xqdoc:author xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Ghislain Fourny</xqdoc:author>" ], "version" : null, "encoding" : "utf-8", "namespaces" : [ { "uri" : "http://xbrl.io/modules/bizql/archives", "prefix" : "archives" }, { "uri" : "http://xbrl.io/modules/bizql/components", "prefix" : "components" }, { "uri" : "http://www.28msec.com/modules/credentials", "prefix" : "credentials" }, { "uri" : "http://xbrl.io/modules/bizql/facts", "prefix" : "facts" }, { "uri" : "http://www.28msec.com/modules/mongodb", "prefix" : "mongo" }, { "uri" : "http://zorba.io/options/versioning", "prefix" : "ver" } ], "functions" : [ { "isDocumented" : true, "arity" : 1, "name" : "cid", "qname" : "components:cid", "signature" : "($component-or-id as item()) as atomic", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Converts the input to a normalized component id (CID). The input\n can be either an CID, or an component object which contains an _id.</p>\n", "summary" : "<p>  Converts the input to a normalized component id (CID).</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "component-or-id", "type" : "item()", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> an component object or an CID.</div>" } ], "returns" : { "type" : "atomic", "description" : "the normalized CID." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">components:INVALID_PARAMETER if the CID or component is not valid.</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 1, "name" : "components-for-archives", "qname" : "components:components-for-archives", "signature" : "($archive-or-ids as item()*) as object()*", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Retrieves all components that belong to the supplied archives.</p>\n", "summary" : "<p>  Retrieves all components that belong to the supplied archives.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "archive-or-ids", "type" : "item()", "occurrence" : "*", "description" : "" } ], "returns" : { "type" : "object()*", "description" : "all components in the archive with this AID." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 0, "name" : "components", "qname" : "components:components", "signature" : "() as object()*", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Retrieves all components.</p>\n", "summary" : "<p>  Retrieves all components.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [  ], "returns" : { "type" : "object()*", "description" : "all components." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "components", "qname" : "components:components", "signature" : "($component-or-ids as item()*) as object()*", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Retrieves the components with the given CIDs.</p>\n", "summary" : "<p>  Retrieves the components with the given CIDs.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "component-or-ids", "type" : "item()", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the CIDs or the components themselves.</div>" } ], "returns" : { "type" : "object()*", "description" : "the components whose _id field matches one of these CIDs." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "num-abstract-primary-items-in-hypercubes", "qname" : "components:num-abstract-primary-items-in-hypercubes", "signature" : "($components-or-ids) as integer*", "description" : " Return the number of (distinct) abstract primary items being\n in a hypercube for each of the given components.\n", "summary" : "<p> Return the number of (distinct) abstract primary items being\n in a hypercube for each of the given components.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "components-or-ids", "type" : null, "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> list of archives or IDs</div>" } ], "returns" : { "type" : "integer*", "description" : "the said number of primary items" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "num-concrete-primary-items-in-hypercubes", "qname" : "components:num-concrete-primary-items-in-hypercubes", "signature" : "($components-or-ids) as integer*", "description" : " Return the number of (distinct) concrete primary items being\n in a hypercube for each of the given components.\n", "summary" : "<p> Return the number of (distinct) concrete primary items being\n in a hypercube for each of the given components.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "components-or-ids", "type" : null, "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> list of archives or IDs</div>" } ], "returns" : { "type" : "integer*", "description" : "the said number of primary items" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "num-concrete-primary-items-not-in-hypercubes", "qname" : "components:num-concrete-primary-items-not-in-hypercubes", "signature" : "($components-or-ids) as integer*", "description" : " Return the number of (distinct) concrete primary items not being\n in a hypercube for each of the given components.\n", "summary" : "<p> Return the number of (distinct) concrete primary items not being\n in a hypercube for each of the given components.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "components-or-ids", "type" : null, "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> list of archives or IDs</div>" } ], "returns" : { "type" : "integer*", "description" : "the said number of primary items" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "num-distinct-abstract-primary-items-not-in-hypercubes", "qname" : "components:num-distinct-abstract-primary-items-not-in-hypercubes", "signature" : "($components-or-ids) as integer*", "description" : " Return the number of (distinct) abstract primary items not being\n in a hypercube for each of the given components.\n", "summary" : "<p> Return the number of (distinct) abstract primary items not being\n in a hypercube for each of the given components.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "components-or-ids", "type" : null, "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> list of archives or IDs</div>" } ], "returns" : { "type" : "integer*", "description" : "the said number of primary items" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "num-domains", "qname" : "components:num-domains", "signature" : "($components-or-ids) as integer*", "description" : " Return the number of (distinct) domains in each of the given components.\n", "summary" : "<p> Return the number of (distinct) domains in each of the given components.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "components-or-ids", "type" : null, "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> list of archives or IDs</div>" } ], "returns" : { "type" : "integer*", "description" : "the said number of domains" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "num-explicit-dimensions", "qname" : "components:num-explicit-dimensions", "signature" : "($components-or-ids) as integer*", "description" : " Return the number of (distinct) explicit dimensions in each of the given components.\n", "summary" : "<p> Return the number of (distinct) explicit dimensions in each of the given components.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "components-or-ids", "type" : null, "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> list of archives or IDs</div>" } ], "returns" : { "type" : "integer*", "description" : "the said number of dimensions" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "num-hypercubes", "qname" : "components:num-hypercubes", "signature" : "($components-or-ids as item()*) as integer*", "description" : " Return the number of hypercubes in each of the given components.\n", "summary" : "<p> Return the number of hypercubes in each of the given components.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "components-or-ids", "type" : "item()", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> list of archives or IDs</div>" } ], "returns" : { "type" : "integer*", "description" : "the said number of hypercubes" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "num-members", "qname" : "components:num-members", "signature" : "($components-or-ids) as integer*", "description" : " Return the number of (distinct) members in each of the given components.\n", "summary" : "<p> Return the number of (distinct) members in each of the given components.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "components-or-ids", "type" : null, "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> list of archives or IDs</div>" } ], "returns" : { "type" : "integer*", "description" : "the said number of members" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "num-networks", "qname" : "components:num-networks", "signature" : "($components-or-ids) as integer*", "description" : " Return the number of networks in each of the given components.\n", "summary" : "<p> Return the number of networks in each of the given components.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "components-or-ids", "type" : null, "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> list of archives or IDs</div>" } ], "returns" : { "type" : "integer*", "description" : "the said number of networks" }, "errors" : [  ] } ], "variables" : [ { "name" : "components:col", "type" : "xs:string", "description" : " Name of the collection the components are stored in.\n" }, { "name" : "components:ARCHIVE", "type" : "xs:string", "description" : " Name of the field pointing to the archive.\n" } ] }, "http://jsound.io/modules/validate" : { "ns" : "http://jsound.io/modules/validate", "description" : " JSound simple validator.\n This is a JSONiq implemenation of the JSound (the schema for JSON) validator.\n", "sees" : [  ], "authors" : [ "<xqdoc:author xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Cezar Andrei</xqdoc:author>" ], "version" : null, "encoding" : "utf-8", "namespaces" : [ { "uri" : "http://zorba.io/annotations", "prefix" : "an" }, { "uri" : "http://zorba.io/modules/fetch", "prefix" : "fetch" }, { "uri" : "http://jsoniq.org/functions", "prefix" : "jn" }, { "uri" : "http://jsound.io/modules/validate", "prefix" : "jsv" }, { "uri" : "http://jsound.io/modules/validate/map", "prefix" : "map" }, { "uri" : "http://zorba.io/options/versioning", "prefix" : "ver" } ], "functions" : [ { "isDocumented" : true, "arity" : 3, "name" : "jsd-valid", "qname" : "jsv:jsd-valid", "signature" : "($ns as string, $name as string, $instance as json-item()) as boolean", "description" : " Validates the $instance JSON item against the JSound type with name $name\n and namespace $ns, from the JSound schema definition $jsd.\n", "summary" : "<p> Validates the $instance JSON item against the JSound type with name $name\n and namespace $ns, from the JSound schema definition $jsd.</p>", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "ns", "type" : "string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the namespace of the expected type</div>" }, { "name" : "name", "type" : "string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the name of the expected type</div>" }, { "name" : "instance", "type" : "json-item()", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the instance to be validated</div>" } ], "returns" : { "type" : "boolean", "description" : "true if the instance is valid, otherwise throws an error." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">jsv:BadJSoundFormat If the schema is not a valid JSound schema</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">jsv:Invalid If the instance does not conform to the JSound schema</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">zerr:ZXQP0025 If the schema namespace URI cannot be resolved.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">jn:JNDY0021 If the loaded schema is syntactically incorrect</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 4, "name" : "jsd-valid", "qname" : "jsv:jsd-valid", "signature" : "($jsd as object(), $name as string, $ns as string, $instance as item()) as boolean", "description" : " Validates the $instance JSON item against the JSound type with name $name\n and namespace $ns, from the JSound schema definition $jsd.\n", "summary" : "<p> Validates the $instance JSON item against the JSound type with name $name\n and namespace $ns, from the JSound schema definition $jsd.</p>", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "jsd", "type" : "object()", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the JSound schema as a JSON object to be validated against</div>" }, { "name" : "name", "type" : "string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the name of the expected type</div>" }, { "name" : "ns", "type" : "string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the namespace of the expected type</div>" }, { "name" : "instance", "type" : "item()", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the instance to be validated</div>" } ], "returns" : { "type" : "boolean", "description" : "true if the instance is valid, otherwise throws an error." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">jsv:BadJSoundFormat If the schema is not a valid JSound schema</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">jsv:Invalid If the instance does not conform to the JSound schema</xqdoc:error>" ] } ], "variables" : [  ] }, "http://xbrl.io/modules/bizql/archives" : { "ns" : "http://xbrl.io/modules/bizql/archives", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">This module provides functions for retrieving metadata about archives.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Archives represent the granularity of \"report shipping\", i.e., an entity\n reports one archive at a time. An archive is identified with an AID (archive ID).</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Archives are made of reported facts (see facts module),\n which are structured and organized in components (see components module).</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">With this module, you can retrieve all archives, or a certain number of archives\n with their AIDs, or obtain the AID of archives you already have. You can also retrieve\n all archives submitted by one or several entities.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Archive information is stored in a MongoDB datasource called <b>xbrl</b>.</p>\n", "sees" : [  ], "authors" : [ "<xqdoc:author xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Charles Hoffman</xqdoc:author>", "<xqdoc:author xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Matthias Brantner</xqdoc:author>", "<xqdoc:author xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Dennis Knochenwefel</xqdoc:author>", "<xqdoc:author xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Ghislain Fourny</xqdoc:author>" ], "version" : null, "encoding" : "utf-8", "namespaces" : [ { "uri" : "http://xbrl.io/modules/bizql/archives", "prefix" : "archives" }, { "uri" : "http://www.28msec.com/modules/credentials", "prefix" : "credentials" }, { "uri" : "http://xbrl.io/modules/bizql/entities", "prefix" : "entities" }, { "uri" : "http://www.w3.org/2005/xqt-errors", "prefix" : "err" }, { "uri" : "http://www.28msec.com/modules/mongodb", "prefix" : "mongo" }, { "uri" : "http://zorba.io/options/versioning", "prefix" : "ver" } ], "functions" : [ { "isDocumented" : true, "arity" : 1, "name" : "aid", "qname" : "archives:aid", "signature" : "($archives-or-ids as item()*) as atomic*", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Converts the input to a normalized archive identifier (AID). The input\n can be either a pure AID, or an archive object which contains an AID.</p>\n", "summary" : "<p>  Converts the input to a normalized archive identifier (AID).</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "archives-or-ids", "type" : "item()", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> a sequence of archive objects or identifiers (AID).</div>" } ], "returns" : { "type" : "atomic*", "description" : "the normalized AIDs." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">archives:INVALID_PARAMETER if the AID or archive is not valid</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 1, "name" : "archives-for-entities", "qname" : "archives:archives-for-entities", "signature" : "($entities-or-ids as item()*) as object()*", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Return all archives created by the supplied entities.</p>\n", "summary" : "<p>  Return all archives created by the supplied entities.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "entities-or-ids", "type" : "item()", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> arbitrary number of entity objects or EIDs.</div>" } ], "returns" : { "type" : "object()*", "description" : "all archives created by these entities." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 0, "name" : "archives", "qname" : "archives:archives", "signature" : "() as object()*", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Retrieves all archives.</p>\n", "summary" : "<p>  Retrieves all archives.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [  ], "returns" : { "type" : "object()*", "description" : "all archives." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "archives", "qname" : "archives:archives", "signature" : "($archive-or-ids as item()*) as object()*", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Retrieves the archives with the given AIDs.</p>\n", "summary" : "<p>  Retrieves the archives with the given AIDs.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "archive-or-ids", "type" : "item()", "occurrence" : "*", "description" : "" } ], "returns" : { "type" : "object()*", "description" : "the archives with the given AIDs the empty sequence if no archive was found or if the input is an empty sequence." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "entities", "qname" : "archives:entities", "signature" : "($archives-or-ids as item()*) as object()*", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Returns the entities that submitted the supplied archives.</p>\n", "summary" : "<p>  Returns the entities that submitted the supplied archives.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "archives-or-ids", "type" : "item()", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> a sequence of archives or their identifiers (AID).</div>" } ], "returns" : { "type" : "object()*", "description" : "the submitting entities." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "num-abstract-primary-items-in-hypercubes", "qname" : "archives:num-abstract-primary-items-in-hypercubes", "signature" : "($archives-or-ids) as integer*", "description" : " Return the number of (distinct) abstract primary items being\n in a hypercube for each of the given archives.\n", "summary" : "<p> Return the number of (distinct) abstract primary items being\n in a hypercube for each of the given archives.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "archives-or-ids", "type" : null, "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> list of archives or IDs</div>" } ], "returns" : { "type" : "integer*", "description" : "the said number of primary items" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "num-components", "qname" : "archives:num-components", "signature" : "($archives-or-ids) as integer*", "description" : " Return the number of components of each of the given archives.\n", "summary" : "<p> Return the number of components of each of the given archives.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "archives-or-ids", "type" : null, "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> list of archives or IDs</div>" } ], "returns" : { "type" : "integer*", "description" : "the said number of components" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "num-concrete-primary-items-in-hypercubes", "qname" : "archives:num-concrete-primary-items-in-hypercubes", "signature" : "($archives-or-ids) as integer*", "description" : " Return the number of (distinct) concrete primary items being\n in a hypercube for each of the given archives.\n", "summary" : "<p> Return the number of (distinct) concrete primary items being\n in a hypercube for each of the given archives.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "archives-or-ids", "type" : null, "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> list of archives or IDs</div>" } ], "returns" : { "type" : "integer*", "description" : "the said number of primary items" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "num-concrete-primary-items-not-in-hypercubes", "qname" : "archives:num-concrete-primary-items-not-in-hypercubes", "signature" : "($archives-or-ids) as integer*", "description" : " Return the number of (distinct) concrete primary items not being\n in a hypercube for each of the given archives.\n", "summary" : "<p> Return the number of (distinct) concrete primary items not being\n in a hypercube for each of the given archives.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "archives-or-ids", "type" : null, "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> list of archives or IDs</div>" } ], "returns" : { "type" : "integer*", "description" : "the said number of primary items" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "num-distinct-abstract-primary-items-not-in-hypercubes", "qname" : "archives:num-distinct-abstract-primary-items-not-in-hypercubes", "signature" : "($archives-or-ids) as integer*", "description" : " Return the number of (distinct) abstract primary items not being\n in a hypercube for each of the given archives.\n", "summary" : "<p> Return the number of (distinct) abstract primary items not being\n in a hypercube for each of the given archives.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "archives-or-ids", "type" : null, "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> list of archives or IDs</div>" } ], "returns" : { "type" : "integer*", "description" : "the said number of primary items" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "num-domains", "qname" : "archives:num-domains", "signature" : "($archives-or-ids) as integer*", "description" : " Return the number of (distinct) domains in each of the given archives.\n", "summary" : "<p> Return the number of (distinct) domains in each of the given archives.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "archives-or-ids", "type" : null, "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> list of archives or IDs</div>" } ], "returns" : { "type" : "integer*", "description" : "the said number of domains" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "num-explicit-dimensions", "qname" : "archives:num-explicit-dimensions", "signature" : "($archives-or-ids) as integer*", "description" : " Return the number of (distinct) explicit dimensions in each of the given archives.\n", "summary" : "<p> Return the number of (distinct) explicit dimensions in each of the given archives.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "archives-or-ids", "type" : null, "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> list of archives or IDs</div>" } ], "returns" : { "type" : "integer*", "description" : "the said number of dimensions" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "num-facts", "qname" : "archives:num-facts", "signature" : "($archives-or-ids) as integer*", "description" : " Return the number of facts of each of the given archives.\n", "summary" : "<p> Return the number of facts of each of the given archives.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "archives-or-ids", "type" : null, "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> list of archives or IDs</div>" } ], "returns" : { "type" : "integer*", "description" : "the said number of facts" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "num-footnotes", "qname" : "archives:num-footnotes", "signature" : "($archives-or-ids) as integer*", "description" : " Return the number of XBRL footnotes of each of the given archives.\n", "summary" : "<p> Return the number of XBRL footnotes of each of the given archives.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "archives-or-ids", "type" : null, "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> list of archives or IDs</div>" } ], "returns" : { "type" : "integer*", "description" : "the said number of footnotes" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "num-hypercubes", "qname" : "archives:num-hypercubes", "signature" : "($archives-or-ids as item()*) as integer*", "description" : " Return the number of hypercubes in each of the given archives.\n", "summary" : "<p> Return the number of hypercubes in each of the given archives.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "archives-or-ids", "type" : "item()", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> list of archives or IDs</div>" } ], "returns" : { "type" : "integer*", "description" : "the said number of hypercubes" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "num-members", "qname" : "archives:num-members", "signature" : "($archives-or-ids) as integer*", "description" : " Return the number of (distinct) members in each of the given archives.\n", "summary" : "<p> Return the number of (distinct) members in each of the given archives.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "archives-or-ids", "type" : null, "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> list of archives or IDs</div>" } ], "returns" : { "type" : "integer*", "description" : "the said number of members" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "num-networks", "qname" : "archives:num-networks", "signature" : "($archives-or-ids) as integer*", "description" : " Return the number of networks in each of the given archives.\n", "summary" : "<p> Return the number of networks in each of the given archives.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "archives-or-ids", "type" : null, "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> list of archives or IDs</div>" } ], "returns" : { "type" : "integer*", "description" : "the said number of networks" }, "errors" : [  ] } ], "variables" : [ { "name" : "archives:col", "type" : "string", "description" : " Name of the collection the archives are stored in.\n" }, { "name" : "archives:ENTITY", "type" : "string", "description" : " Name of the field which points to the reporting entity.\n" } ] }, "http://www.28msec.com/modules/xmlrpc" : { "ns" : "http://www.28msec.com/modules/xmlrpc", "description" : " XML RPC Client Module\n This module provides the functions necessary to execute remote call\n procedures using\n <a xmlns:xqdoc=\"http://www.xqdoc.org/1.0\" href=\"http://www.xmlrpc.com/spec\" target=\"_blank\">XML-RPC</a>.\n The application/mashup creator does not need to know the\n specifics of <a xmlns:xqdoc=\"http://www.xqdoc.org/1.0\" href=\"http://www.xmlrpc.com/spec\" target=\"_blank\">XML-RPC</a> to use this module.\n Usage:\n <pre xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">xmlrpc:invoke(\"http://www.advogato.org/XMLRPC\", \"test.sumprod\", (5, 7))</pre>\n <pre xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">xmlrpc:invoke(\"http://www.advogato.org/XMLRPC\", \"test.capitalize\", \"HelloWorld\")</pre>\n <pre xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">xmlrpc:invoice(\"http://www.example.com/XMLRPC\", \"test.foo\", (\"some-parameter\",\n                      &lt;struct&gt;\n                        &lt;member&gt;\n                          &lt;name&gt;Foo&lt;/name&gt;\n                          &lt;value&gt;&lt;string&gt;Bar&lt;/string&gt;&lt;/value&gt;\n                        &lt;/member&gt;\n                      &lt;/struct&gt;))</pre>\n", "sees" : [ "<xqdoc:see xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"><a href=\"http://www.xmlrpc.com/spec\" target=\"_blank\">XML-RPC Specification</a></xqdoc:see>", "<xqdoc:see xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"><a href=\"http://en.wikipedia.org/wiki/XML-RPC\" target=\"_blank\">XML-RPC Wikipedia article</a></xqdoc:see>" ], "authors" : [ "<xqdoc:author xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">William Candillon {william.candillon@28msec.com}</xqdoc:author>" ], "version" : null, "encoding" : "utf-8", "namespaces" : [ { "uri" : "http://zorba.io/annotations", "prefix" : "ann" }, { "uri" : "http://expath.org/ns/http-client", "prefix" : "http" }, { "uri" : "http://expath.org/ns/http-client", "prefix" : "http-client" }, { "uri" : "http://zorba.io/options/versioning", "prefix" : "ver" }, { "uri" : "http://www.28msec.com/modules/xmlrpc", "prefix" : "xmlrpc" } ], "functions" : [ { "isDocumented" : true, "arity" : 2, "name" : "invoke", "qname" : "xmlrpc:invoke", "signature" : "($endpoint-url as xs:string, $method as xs:string) as item()*", "description" : " Invoke a remote method without parameters.\n Calling this function is equivalent to xmlrpc:invoke($url, $method, ())\n", "summary" : "<p> Invoke a remote method without parameters.</p>", "annotation_str" : " %ann:sequential", "annotations" : [ { "prefix" : "ann", "ns" : "http://zorba.io/annotations", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "endpoint-url", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> URL of the XML-RPC server.</div>" }, { "name" : "method", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> Name of the method to invoke.</div>" } ], "returns" : { "type" : "item()*", "description" : "Result of the method invocation." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 3, "name" : "invoke", "qname" : "xmlrpc:invoke", "signature" : "($endpoint-url as xs:string, $method as xs:string, $parameters as item()*) as item()*", "description" : " Invoke a remote method with parameters.\n", "summary" : "<p> Invoke a remote method with parameters.</p>", "annotation_str" : " %ann:sequential", "annotations" : [ { "prefix" : "ann", "ns" : "http://zorba.io/annotations", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "endpoint-url", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> URL of the XML-RPC server.</div>" }, { "name" : "method", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> Name of the method to invoke.</div>" }, { "name" : "parameters", "type" : "item()", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> Method parameters. Struct and Arrays need to follow XML-RPC format. Atomic types will be automatically convert to the proper XML-RPC data format.</div>" } ], "returns" : { "type" : "item()*", "description" : "Result of the method invocation." }, "errors" : [  ] } ], "variables" : [ { "name" : "xmlrpc:ERR_001", "type" : "xs:QName", "description" : " XML-RPC serialization error.\n" } ] }, "http://zorba.io/modules/unordered-maps" : { "ns" : "http://zorba.io/modules/unordered-maps", "description" : " This module defines a set of functions for working with maps. A map\n is identified by a string and can be created using the map:create function\n and dropped using the map:drop function.\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n It is possible to create persistent and transient maps. The lifetime of a\n transient map is limited by the execution of the current query.\n A persistent map lives until it is explicitly dropped.\n Accordingly, it is also available to other requests.\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n For example,\n <pre xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">map:create(\"my-map\", [\"string\", \"integer\"], { \"persistent\" : false })</pre>\n will create a transient map named my-map having two keys.\n The types of the keys are string and integer.\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n The key of a particular entry in the map can consist of a tuple of\n atomic values (called key attributes). The actual type of each attribute\n is determined when the map is created. The value of each entry is a\n sequence of items. If an item in this sequence is a object or array,\n this item needs to belong to a collection, otherwise, an error is raised.\n", "sees" : [  ], "authors" : [ "<xqdoc:author xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Matthias Brantner</xqdoc:author>" ], "version" : null, "encoding" : "utf-8", "namespaces" : [ { "uri" : "http://zorba.io/annotations", "prefix" : "an" }, { "uri" : "http://zorba.io/modules/unordered-maps", "prefix" : "map" }, { "uri" : "http://zorba.io/options/versioning", "prefix" : "ver" }, { "uri" : "http://zorba.io/errors", "prefix" : "zerr" } ], "functions" : [ { "isDocumented" : true, "arity" : 0, "name" : "available-maps", "qname" : "map:available-maps", "signature" : "() as string* external", "description" : " The function returns a sequence of names of the maps that are\n available (persistent and non-persistent). The sequence will be\n empty if there are no maps.\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n", "summary" : "<p> The function returns a sequence of names of the maps that are\n available (persistent and non-persistent).</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [  ], "returns" : { "type" : "string*", "description" : "A sequence of string, one for each available map, or an empty sequence." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 2, "name" : "create", "qname" : "map:create", "signature" : "($name as string, $key-types as item()) as empty-sequence() external", "description" : " Create a persistent map with a given name and type identifiers for the key\n attributes.\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n If the map has only one key attribute, a single type identifier is given,\n for more than one key attribute an array of type identifiers is given.\n Calling this function is equivalent to calling create with the options\n <code xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">{ \"persistent\" : true }</code>\n Note that the function is sequential and immediately creates the map.\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n Each key-type should be specified as string (e.g. \"integer\",\n \"string\", \"boolean\", \"double\", or \"datetime\").\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n For example,\n <pre xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">map:create(\"my-map\", \"string\")</pre> or\n <pre xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">map:create(\"my-map\", [\"string\", \"integer\"])</pre>.\n", "summary" : "<p> Create a persistent map with a given name and type identifiers for the key\n attributes.</p>", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "name", "type" : "string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the name of the map (the restrictions on collection names apply)</div>" }, { "name" : "key-types", "type" : "item()", "occurrence" : null, "description" : "" } ], "returns" : { "type" : "empty-sequence()", "description" : "the function is sequential and immediately creates the corresponding map. It returns the empty-sequence." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">err:XPTY0004 if any of the key attribute types is not a subtype of anyAtomicType.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">zerr:ZSTR0001 if a map with the given name already exists.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">zerr:ZDDY1000 if the given name is not a legal collection name</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">zerr:ZDDY0043 if any of the given options has an invalid type</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 3, "name" : "create", "qname" : "map:create", "signature" : "($name as string, $key-types as item(), $options as object()) as empty-sequence() external", "description" : " Create a map with a given name, type identifiers for the key attributes, and\n options.\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n If the map has only one key attribute, a single type identifier is given,\n for more than one key attribute an array of type identifiers is given.\n Currently only one option is supported: To create a transient map the object\n <code xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">{ \"persistent\" : false }</code>\n has to be passed to the $options parameter.\n Note that the function is sequential and immediately creates the map in the\n store.\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n Note that a map cannot be created if it already exists in a parent context.\n For example, a map that was created in an outer query cannot be\n created again in an inner query executed using the\n <code xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">reflection:eval-s</code> function.\n", "summary" : "<p> Create a map with a given name, type identifiers for the key attributes, and\n options.</p>", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "name", "type" : "string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the name of the map (the restrictions on collection names apply)</div>" }, { "name" : "key-types", "type" : "item()", "occurrence" : null, "description" : "" }, { "name" : "options", "type" : "object()", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> an object describing options for the map</div>" } ], "returns" : { "type" : "empty-sequence()", "description" : "the function is sequential and immediately creates the corresponding map but returns the empty-sequence." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">err:XPTY0004 if any of the attribute types is not a subtype of anyAtomicType.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">zerr:ZSTR0001 if a map with the given name already exists.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">zerr:ZDDY1000 if the given name is not a legal collection name</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">zerr:ZDDY0043 if any of the given options has an invalid type</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 2, "name" : "delete", "qname" : "map:delete", "signature" : "($name as string, $key as item()) as empty-sequence() external", "description" : " Removes an entry identified by the given key from the map.\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n If the map has only one key attribute, a single key value is given, for\n more than one key attribute an array of key values is given.\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n Note that it is possible to insert entries with empty key attributes.\n However as the removing the entries is based on the \"eq\" comparison and\n as \"eq\" with an empty sequence always return false, it is not possible\n to delete these entries.\n", "summary" : "<p> Removes an entry identified by the given key from the map.</p>", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "name", "type" : "string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the name of the map</div>" }, { "name" : "key", "type" : "item()", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> either a single attribute key or an array of keys</div>" } ], "returns" : { "type" : "empty-sequence()", "description" : "the function is sequential and immediately deletes the entry into the map but returns the empty-sequence." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">zerr:ZDDY0023 if a map with the given name does not exist.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">zerr:ZDDY0025 if the given number of key attributes does not match the number of key attributes specified when creating the map (see the map:create function).</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">zerr:ZXQD0005 if any of the given key attributes can not be cast (or is not a subtype) of the corresponding key attribute specified when creating the map.</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 1, "name" : "drop", "qname" : "map:drop", "signature" : "($name as string) as empty-sequence() external", "description" : " Deletes the map with the given name.\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n Note that a map can only be dropped in the context it was created.\n For example, a map that was created in an outer query cannot be\n dropped in an inner query executed using the\n <code xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">reflection:eval-s</code> function.\n", "summary" : "<p> Deletes the map with the given name.</p>", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "name", "type" : "string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the name of the map to drop</div>" } ], "returns" : { "type" : "empty-sequence()", "description" : "the function is sequential and immediately drops the map. It returns the empty-sequence." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">zerr:ZDDY0023 if a map with the given name does not exist.</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 2, "name" : "get", "qname" : "map:get", "signature" : "($name as string, $key as item()) as item()* external", "description" : " Returns the value of the entry with the given key from the map.\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n If the map has only one key attribute, a single key value is given, for\n more than one key attribute an array of key values is given.\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n Note that it is possible to insert entries with empty key attributes.\n However as the getting the entries is based on the \"eq\" comparison and\n as \"eq\" with an empty sequence always return false, it is not possible\n to retrieve these entries.\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n For example,\n <pre xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">map:get(\"my-map\", \"key\")</pre> or\n <pre xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">map:get(\"my-map\", [ \"key1\", \"key2\" ])</pre>.\n", "summary" : "<p> Returns the value of the entry with the given key from the map.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "name", "type" : "string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the name of the map</div>" }, { "name" : "key", "type" : "item()", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> either a single attribute key or an array of keys</div>" } ], "returns" : { "type" : "item()*", "description" : "the value of the entry in the map identified by the given key. The empty-sequence will be returned if no entry with the given key is contained in the map." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">zerr:ZDDY0023 if a map with the given name does not exist.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">zerr:ZDDY0025 if the given number of key attributes does not match the number of key attributes specified when creating the map (see the map:create function).</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">zerr:ZXQD0005 if any of the given key attributes can not be cast (or is not a subtype) of the corresponding key attribute specified when creating the map.</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 3, "name" : "insert", "qname" : "map:insert", "signature" : "($name as string, $key as item(), $value as item()*) as empty-sequence() external", "description" : " Inserts a new entry into the map with the given name.\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n If the map has only one key attribute, a single key value is given, for\n more than one key attribute an array of key values is given.\n If an entry with the given key already exists in the map, the value\n sequences of the existing entry and the sequence passed using $value\n argument are concatenated.\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n If an item in the value sequence is an object or array, this\n item needs to belong to a collection, otherwise, an an error\n is raised.\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n Note that it is possible to insert entries with empty key attributes\n or key attributes having the value <code xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">null</code>. However, as\n the comparison with an empty sequence or null always returns false,\n it is not possible to retrieve these entries.\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n For example,\n <pre xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">map:insert(\"my-map\", \"key\", \"value\")</pre> or\n <pre xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">map:insert(\"my-map\", [ \"key1\", \"key2\" ] , (42, \"value\"))</pre>.\n", "summary" : "<p> Inserts a new entry into the map with the given name.</p>", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "name", "type" : "string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the name of the map</div>" }, { "name" : "key", "type" : "item()", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> either a single attribute key or an array of keys</div>" }, { "name" : "value", "type" : "item()", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the value of the entry to insert</div>" } ], "returns" : { "type" : "empty-sequence()", "description" : "the function is sequential and immediately inserts the entry into the map. It returns the empty-sequence." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">zerr:ZDDY0023 if a map with the given name does not exist.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">zerr:ZDDY0025 if the given number of key attributes does not match the number of key attributes specified when creating the map (see the map:create function).</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">zerr:ZXQD0005 if any of the given key attributes can not be cast (or is not a subtype) of the corresponding key attribute specified when creating the map.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">zerr:ZDDY1003 if the value to insert is an object or array it must belong to a collection.</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 1, "name" : "keys", "qname" : "map:keys", "signature" : "($name as string) as array()* external", "description" : " Returns the keys of all entries of a map. The keys\n are returned as sequence of arrays.\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n The following condition always holds:\n <tt xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">map:size($name) eq count(map:keys($name))</tt>\n", "summary" : "<p> Returns the keys of all entries of a map.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "name", "type" : "string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the name of the map</div>" } ], "returns" : { "type" : "array()*", "description" : "an sequence of arrays each array containing the values of all attributes of one key." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">zerr:ZDDY0023 if a map with the given name does not exist.</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 1, "name" : "options", "qname" : "map:options", "signature" : "($name as string) as object() external", "description" : " The function returns the options that were passed during creation or the\n default options if no options were passed.\n", "summary" : "<p> The function returns the options that were passed during creation or the\n default options if no options were passed.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "name", "type" : "string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the name of the map</div>" } ], "returns" : { "type" : "object()", "description" : "an options object" }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">zerr:ZDDY0023 if a map with the given name does not exist.</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 1, "name" : "size", "qname" : "map:size", "signature" : "($name as string) as integer external", "description" : " Returns the number of entries in a map.\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n The following condition always holds:\n <tt xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">map:size($name) eq count(map:keys($name))</tt>\n", "summary" : "<p> Returns the number of entries in a map.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "name", "type" : "string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the name of the map</div>" } ], "returns" : { "type" : "integer", "description" : "the number of entries in the map." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">zerr:ZDDY0023 if a map with the given name does not exist.</xqdoc:error>" ] } ], "variables" : [ { "name" : "map:PERSISTENT", "type" : "string", "description" : " Constant containing the field name of the options object\n indiciating whether a map is persistent or transient.\n" } ] }, "http://www.28msec.com/modules/assertion" : { "ns" : "http://www.28msec.com/modules/assertion", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">\n   This module provides a set of assertion functions.\n </p>\n", "sees" : [  ], "authors" : [ "<xqdoc:author xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">28msec</xqdoc:author>" ], "version" : null, "encoding" : "utf-8", "namespaces" : [ { "uri" : "http://zorba.io/annotations", "prefix" : "an" }, { "uri" : "http://www.28msec.com/modules/assertion", "prefix" : "assertion" }, { "uri" : "http://zorba.io/options/versioning", "prefix" : "ver" } ], "functions" : [ { "isDocumented" : true, "arity" : 2, "name" : "equals-deep", "qname" : "assertion:equals-deep", "signature" : "($expected as item()*, $actual as item()*) as item()*", "description" : " Asserts that two objects are deep-equal.\n If they are not, an error containing the diff is raised.\n", "summary" : "<p> Asserts that two objects are deep-equal.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "expected", "type" : "item()", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> expected value</div>" }, { "name" : "actual", "type" : "item()", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> actual value</div>" } ], "returns" : { "type" : "item()*", "description" : "actual value" }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">assertion:A003 expected and actual value are not deep-equal</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 2, "name" : "equals-general", "qname" : "assertion:equals-general", "signature" : "($expected as item()*, $actual as item()*) as item()*", "description" : " Asserts that two objects are equal (by general comarison).\n If they are not, an error containing the diff is raised.\n", "summary" : "<p> Asserts that two objects are equal (by general comarison).</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "expected", "type" : "item()", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> expected value</div>" }, { "name" : "actual", "type" : "item()", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> actual value</div>" } ], "returns" : { "type" : "item()*", "description" : "actual value" }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">assertion:A002 expected and actual value are not equal</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 2, "name" : "equals-value", "qname" : "assertion:equals-value", "signature" : "($expected as item()?, $actual as item()?) as item()?", "description" : " Asserts that two objects are equal (by value comparison).\n If they are not, an error containing the diff is raised.\n", "summary" : "<p> Asserts that two objects are equal (by value comparison).</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "expected", "type" : "item()", "occurrence" : "?", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> expected value</div>" }, { "name" : "actual", "type" : "item()", "occurrence" : "?", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> actual value</div>" } ], "returns" : { "type" : "item()?", "description" : "actual value" }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">assertion:A001 expected and actual value are not equal</xqdoc:error>" ] } ], "variables" : [ { "name" : "assertion:A001", "type" : "xs:QName", "description" : " xs:QName with namespace URI=\"http://www.28msec.com/modules/assertion\" and\n local name 'A001'. 'equals-value' assertion failed.\n" }, { "name" : "assertion:A002", "type" : "xs:QName", "description" : " xs:QName with namespace URI=\"http://www.28msec.com/modules/assertion\" and\n local name 'A002'. 'equals-general' assertion failed.\n" }, { "name" : "assertion:A003", "type" : "xs:QName", "description" : " xs:QName with namespace URI=\"http://www.28msec.com/modules/assertion\" and\n local name 'A003'. 'equals-general' assertion failed.\n" } ] }, "http://www.zorba-xquery.com/modules/image/basic" : { "ns" : "http://www.zorba-xquery.com/modules/image/basic", "description" : " This module provides function to do the following basic image operations:\n <ul xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">\n   <li>create empty images</li>\n   <li>compare images</li>\n   <li>compress image</li>\n   <li>convert an image one format to another</li>\n   <li>retrieve with, height, format, and exif information from an image</li>\n </ul>\n The following image formats are supported:\n <ul xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">\n   <li>GIF</li>\n   <li>JPEG</li>\n   <li>PNG</li>\n   <li>TIFF</li>\n   <li>BMP</li>\n </ul>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">The errors raised by functions of this module have the namespace\n <tt>http://www.zorba-xquery.com/modules/image/error</tt> (associated with prefix ierr).</p>\n", "sees" : [  ], "authors" : [ "<xqdoc:author xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Daniel Thomas</xqdoc:author>" ], "version" : null, "encoding" : "utf-8", "namespaces" : [ { "uri" : "http://www.zorba-xquery.com/modules/image/basic", "prefix" : "basic" }, { "uri" : "http://www.w3.org/2005/xqt-errors", "prefix" : "err" }, { "uri" : "http://www.zorba-xquery.com/modules/image/error", "prefix" : "ierr" }, { "uri" : "http://www.zorba-xquery.com/modules/image/image", "prefix" : "image" }, { "uri" : "http://www.w3.org/2000/svg", "prefix" : "svg" }, { "uri" : "http://zorba.io/options/versioning", "prefix" : "ver" } ], "functions" : [ { "isDocumented" : true, "arity" : 2, "name" : "compress", "qname" : "basic:compress", "signature" : "($image as xs:base64Binary, $quality as xs:unsignedInt) as xs:base64Binary external", "description" : " Compresses the passed image.\n Compressing means lowering the quality and reducing the size.\n", "summary" : "<p> Compresses the passed image.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "image", "type" : "xs:base64Binary", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the image</div>" }, { "name" : "quality", "type" : "xs:unsignedInt", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> compression level, 0 to 100</div>" } ], "returns" : { "type" : "xs:base64Binary", "description" : "the compressed image" }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">ierr:IM001 the passed image is invalid.</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 2, "name" : "convert-svg-string", "qname" : "basic:convert-svg-string", "signature" : "($svg as xs:string, $format as xs:string) as xs:base64Binary", "description" : " Converts an SVG image to a supported image format.\n", "summary" : "<p> Converts an SVG image to a supported image format.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "svg", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the image to convert as string</div>" }, { "name" : "format", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> target format</div>" } ], "returns" : { "type" : "xs:base64Binary", "description" : "the resulting image" }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">ierr:IM001 the passed SVG is invalid.</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 2, "name" : "convert-svg", "qname" : "basic:convert-svg", "signature" : "($svg as element(svg:svg), $format as xs:string) as xs:base64Binary", "description" : " Converts an SVG image to a supported image format.\n", "summary" : "<p> Converts an SVG image to a supported image format.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "svg", "type" : "element(svg:svg)", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the image to convert</div>" }, { "name" : "format", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> target format</div>" } ], "returns" : { "type" : "xs:base64Binary", "description" : "the resulting image" }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">ierr:IM001 the passed SVG is invalid.</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 2, "name" : "convert", "qname" : "basic:convert", "signature" : "($image as xs:base64Binary, $format as xs:string) as xs:base64Binary", "description" : " Converts an image to another format.\n", "summary" : "<p> Converts an image to another format.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "image", "type" : "xs:base64Binary", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the source image</div>" }, { "name" : "format", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the format (see supported formats above) of the resulting image.</div>" } ], "returns" : { "type" : "xs:base64Binary", "description" : "A new image with the same content as the passed image but with the specified file format." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">ierr:IM001 the passed image is invalid.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">err:FORG0001 unsupported image format</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 3, "name" : "create", "qname" : "basic:create", "signature" : "($width as xs:unsignedInt, $height as xs:unsignedInt, $format as xs:string) as xs:base64Binary", "description" : " Creates an empty image with background color white.\n", "summary" : "<p> Creates an empty image with background color white.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "width", "type" : "xs:unsignedInt", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the width of the new image</div>" }, { "name" : "height", "type" : "xs:unsignedInt", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the height of the new image</div>" }, { "name" : "format", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the format of the new image</div>" } ], "returns" : { "type" : "xs:base64Binary", "description" : "newly created image" }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">err:FORG0001 unsupported image format</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 2, "name" : "equals", "qname" : "basic:equals", "signature" : "($image1 as xs:base64Binary, $image2 as xs:base64Binary) as xs:boolean external", "description" : " Compares two images.\n", "summary" : "<p> Compares two images.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "image1", "type" : "xs:base64Binary", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> first image</div>" }, { "name" : "image2", "type" : "xs:base64Binary", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> second image</div>" } ], "returns" : { "type" : "xs:boolean", "description" : "True if the images are equal." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">ierr:IM001 one of the passed images is invalid.</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 2, "name" : "exif", "qname" : "basic:exif", "signature" : "($image as xs:base64Binary, $tag as xs:string) as xs:string? external", "description" : " Reads exif information from an image.\n This function works for JPEG and TIFF images only.\n It returns empty sequence if no exif information matching the passed tag is found.\n", "summary" : "<p> Reads exif information from an image.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "image", "type" : "xs:base64Binary", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the image</div>" }, { "name" : "tag", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the field name we want read (e.g. DateTime).</div>" } ], "returns" : { "type" : "xs:string?", "description" : "exif field content" }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">ierr:IM001 the passed image is invalid.</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 1, "name" : "format", "qname" : "basic:format", "signature" : "($image as xs:base64Binary) as xs:string external", "description" : " Returns the format of the passed image.\n", "summary" : "<p> Returns the format of the passed image.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "image", "type" : "xs:base64Binary", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the image</div>" } ], "returns" : { "type" : "xs:string", "description" : "the format" }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">ierr:IM001 the passed image is invalid.</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 1, "name" : "height", "qname" : "basic:height", "signature" : "($image as xs:base64Binary) as xs:unsignedInt external", "description" : " Returns the height of the passed image.\n", "summary" : "<p> Returns the height of the passed image.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "image", "type" : "xs:base64Binary", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the image</div>" } ], "returns" : { "type" : "xs:unsignedInt", "description" : "the height in pixels" }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">ierr:IM001 the passed image is invalid.</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 1, "name" : "width", "qname" : "basic:width", "signature" : "($image as xs:base64Binary) as xs:unsignedInt external", "description" : " Returns the width of the passed image.\n", "summary" : "<p> Returns the width of the passed image.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "image", "type" : "xs:base64Binary", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the image</div>" } ], "returns" : { "type" : "xs:unsignedInt", "description" : "the width in pixels" }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">ierr:IM001 the passed image is invalid.</xqdoc:error>" ] } ], "variables" : [  ] }, "http://api.28.io/indices" : { "ns" : "http://api.28.io/indices", "description" : "", "sees" : [  ], "authors" : [  ], "version" : null, "encoding" : null, "namespaces" : [ { "uri" : "", "prefix" : "an" }, { "uri" : "http://api.28.io/indices", "prefix" : "in" }, { "uri" : "http://www.28msec.com/modules/http/request", "prefix" : "req" }, { "uri" : "http://www.28msec.com/modules/http/response", "prefix" : "resp" }, { "uri" : "http://api.28.io/util", "prefix" : "util" }, { "uri" : "http://api.28.io/validation", "prefix" : "validate" } ], "functions" : [ { "isDocumented" : false, "arity" : 1, "name" : "create-index", "qname" : "in:create-index", "signature" : "($new-index as object()) as empty-sequence()", "description" : "", "summary" : "", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "new-index", "type" : "object()", "occurrence" : null, "description" : "" } ], "returns" : { "type" : "empty-sequence()", "description" : "" }, "errors" : [  ] }, { "isDocumented" : false, "arity" : 1, "name" : "delete-index", "qname" : "in:delete-index", "signature" : "($name) as empty-sequence()", "description" : "", "summary" : "", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "name", "type" : null, "occurrence" : null, "description" : "" } ], "returns" : { "type" : "empty-sequence()", "description" : "" }, "errors" : [  ] }, { "isDocumented" : false, "arity" : 0, "name" : "dispatch", "qname" : "in:dispatch", "signature" : "()", "description" : "", "summary" : "", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [  ], "returns" : { "type" : null, "description" : "" }, "errors" : [  ] }, { "isDocumented" : false, "arity" : 1, "name" : "get-index", "qname" : "in:get-index", "signature" : "($name as xs:string) as object()?", "description" : "", "summary" : "", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "name", "type" : "xs:string", "occurrence" : null, "description" : "" } ], "returns" : { "type" : "object()?", "description" : "" }, "errors" : [  ] }, { "isDocumented" : false, "arity" : 0, "name" : "list-indices", "qname" : "in:list-indices", "signature" : "() as array()", "description" : "", "summary" : "", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [  ], "returns" : { "type" : "array()", "description" : "" }, "errors" : [  ] }, { "isDocumented" : false, "arity" : 0, "name" : "metadata", "qname" : "in:metadata", "signature" : "()", "description" : "", "summary" : "", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [  ], "returns" : { "type" : null, "description" : "" }, "errors" : [  ] }, { "isDocumented" : false, "arity" : 2, "name" : "put-index", "qname" : "in:put-index", "signature" : "($name as xs:string, $new-index-obj as object()) as empty-sequence()", "description" : "", "summary" : "", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "name", "type" : "xs:string", "occurrence" : null, "description" : "" }, { "name" : "new-index-obj", "type" : "object()", "occurrence" : null, "description" : "" } ], "returns" : { "type" : "empty-sequence()", "description" : "" }, "errors" : [  ] }, { "isDocumented" : false, "arity" : 1, "name" : "refresh-index", "qname" : "in:refresh-index", "signature" : "($index as xs:string) as empty-sequence()", "description" : "", "summary" : "", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "index", "type" : "xs:string", "occurrence" : null, "description" : "" } ], "returns" : { "type" : "empty-sequence()", "description" : "" }, "errors" : [  ] }, { "isDocumented" : false, "arity" : 1, "name" : "validate-index", "qname" : "in:validate-index", "signature" : "($index as object()) as empty-sequence()", "description" : "", "summary" : "", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "index", "type" : "object()", "occurrence" : null, "description" : "" } ], "returns" : { "type" : "empty-sequence()", "description" : "" }, "errors" : [  ] } ], "variables" : [  ] }, "http://api.28.io/xdmview" : { "ns" : "http://api.28.io/xdmview", "description" : "", "sees" : [  ], "authors" : [  ], "version" : null, "encoding" : null, "namespaces" : [ { "uri" : "", "prefix" : "an" }, { "uri" : "http://api.28.io/model", "prefix" : "model" }, { "uri" : "http://zorba.io/modules/reference", "prefix" : "ref" }, { "uri" : "http://www.zorba-xquery.com/schemas/xdm", "prefix" : "xdm" }, { "uri" : "http://api.28.io/xdmview", "prefix" : "xdmview" } ], "functions" : [ { "isDocumented" : false, "arity" : 1, "name" : "show-namespaces", "qname" : "xdmview:show-namespaces", "signature" : "($namespaces)", "description" : "", "summary" : "", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "namespaces", "type" : null, "occurrence" : null, "description" : "" } ], "returns" : { "type" : null, "description" : "" }, "errors" : [  ] }, { "isDocumented" : false, "arity" : 3, "name" : "show-node", "qname" : "xdmview:show-node", "signature" : "($node, $namespaces, $include-noderef as xs:boolean)", "description" : "", "summary" : "", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "node", "type" : null, "occurrence" : null, "description" : "" }, { "name" : "namespaces", "type" : null, "occurrence" : null, "description" : "" }, { "name" : "include-noderef", "type" : "xs:boolean", "occurrence" : null, "description" : "" } ], "returns" : { "type" : null, "description" : "" }, "errors" : [  ] }, { "isDocumented" : false, "arity" : 2, "name" : "show-nodes", "qname" : "xdmview:show-nodes", "signature" : "($nodes, $include-noderef as xs:boolean)", "description" : "", "summary" : "", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "nodes", "type" : null, "occurrence" : null, "description" : "" }, { "name" : "include-noderef", "type" : "xs:boolean", "occurrence" : null, "description" : "" } ], "returns" : { "type" : null, "description" : "" }, "errors" : [  ] } ], "variables" : [  ] }, "http://www.zorba-xquery.com/modules/xqdoc/json" : { "ns" : "http://www.zorba-xquery.com/modules/xqdoc/json", "description" : " Convert an XQDoc document into an HTML document.\n This module contains a single <code xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">convert()</code> function\n that transform an XQDoc document into an HTML document.\n Usage:\n <pre xmlns:xqdoc=\"http://www.xqdoc.org/1.0\" class=\"ace-static\" ace-mode=\"xquery\">\n let $xqdoc := xqdoc:xqdoc(\"http://expath.org/ns/file\")\n return html:convert($xqdoc)\n </pre>\n", "sees" : [  ], "authors" : [ "<xqdoc:author xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">William Candillon <a href=\"?anchor=\">wcandillon at gmail dot com</a></xqdoc:author>" ], "version" : null, "encoding" : "utf-8", "namespaces" : [ { "uri" : "http://www.zorba-xquery.com/modules/xqdoc/json", "prefix" : "html" }, { "uri" : "http://www.w3.org/2010/xslt-xquery-serialization", "prefix" : "o" }, { "uri" : "http://www.xqdoc.org/1.0", "prefix" : "xq" } ], "functions" : [ { "isDocumented" : false, "arity" : 1, "name" : "convert", "qname" : "html:convert", "signature" : "($xqdoc as element(xq:xqdoc)) as object()", "description" : "", "summary" : "", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "xqdoc", "type" : "element(xq:xqdoc)", "occurrence" : null, "description" : "" } ], "returns" : { "type" : "object()", "description" : "" }, "errors" : [  ] }, { "isDocumented" : false, "arity" : 1, "name" : "normalize-anchors", "qname" : "html:normalize-anchors", "signature" : "($node)", "description" : "", "summary" : "", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "node", "type" : null, "occurrence" : null, "description" : "" } ], "returns" : { "type" : null, "description" : "" }, "errors" : [  ] } ], "variables" : [  ] }, "http://zorba.io/modules/math" : { "ns" : "http://zorba.io/modules/math", "description" : " Extensive math library.<p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n", "sees" : [  ], "authors" : [ "<xqdoc:author xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Daniel Turcanu, Dan Muresan</xqdoc:author>" ], "version" : null, "encoding" : "utf-8", "namespaces" : [ { "uri" : "http://www.w3.org/2005/xpath-functions/math", "prefix" : "W3Cmath" }, { "uri" : "http://zorba.io/modules/math", "prefix" : "math" }, { "uri" : "http://zorba.io/options/versioning", "prefix" : "ver" } ], "functions" : [ { "isDocumented" : true, "arity" : 1, "name" : "acosh", "qname" : "math:acosh", "signature" : "($arg as double) as double external", "description" : " Inverse hyperbolic cosine.\n", "summary" : "<p> Inverse hyperbolic cosine.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "arg", "type" : "double", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the arg</div>" } ], "returns" : { "type" : "double", "description" : "the result of acosh(arg)" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "asinh", "qname" : "math:asinh", "signature" : "($arg as double) as double external", "description" : " Calculate the inverse hyperbolic sine.\n", "summary" : "<p> Calculate the inverse hyperbolic sine.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "arg", "type" : "double", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the arg</div>" } ], "returns" : { "type" : "double", "description" : "the result of asinh(arg)" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "atanh", "qname" : "math:atanh", "signature" : "($arg as double) as double external", "description" : " Calculate the hyperbolic tangent.\n", "summary" : "<p> Calculate the hyperbolic tangent.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "arg", "type" : "double", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> must be in range -1 ... +1 (exclusive)</div>" } ], "returns" : { "type" : "double", "description" : "the result of atanh(arg)" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "avedev", "qname" : "math:avedev", "signature" : "($numbers as double+) as double", "description" : " Returns the average of the absolute deviations of data points from their mean.<p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n The formula is sum(abs(x - average_x))/n, where n is the count of x in the sequence.<p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n Borrowed from excel module.\n", "summary" : "<p> Returns the average of the absolute deviations of data points from their mean.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "numbers", "type" : "double", "occurrence" : "+", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequence of numbers. Sequence can be of any length from 1 up.</div>" } ], "returns" : { "type" : "double", "description" : "The formula result" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "cast-as-numeric", "qname" : "math:cast-as-numeric", "signature" : "($number as anyAtomicType) as anyAtomicType", "description" : " Cast the anyAtomicType to a numeric type.<p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n If the value is already of a numeric type then nothing is changed.\n Otherwise the value is casted to the numeric type that is most appropriate.\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n Borrowed from excel module.\n", "summary" : "<p> Cast the anyAtomicType to a numeric type.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "number", "type" : "anyAtomicType", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The parameter can be a number, string, boolean value.</div>" } ], "returns" : { "type" : "anyAtomicType", "description" : "The casted value." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">math:VALUE_NOT_NUMERIC if the value cannot be casted to numeric type.</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 2, "name" : "ceiling", "qname" : "math:ceiling", "signature" : "($number as double, $significance as double) as double", "description" : " Returns number rounded up, away from zero, to the nearest multiple of significance.<p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n Significance must have the same sign as number.\n Number and significance must be of a numeric type or castable to numeric.\n Significance must not be zero.<p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n Borrowed from excel module.\n", "summary" : "<p> Returns number rounded up, away from zero, to the nearest multiple of significance.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "number", "type" : "double", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The value you want to round.</div>" }, { "name" : "significance", "type" : "double", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The multiple to which you want to round.</div>" } ], "returns" : { "type" : "double", "description" : "The rounded value." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">math:INVALID_ARGUMENT if significance is zero or it doesn't have the same sign as number.</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 1, "name" : "cosh", "qname" : "math:cosh", "signature" : "($arg as double) as double external", "description" : " Returns the hyperbolic cosine of x.<p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n If the result it too large, INF is returned.\n", "summary" : "<p> Returns the hyperbolic cosine of x.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "arg", "type" : "double", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> must be smaller than 7.104760e+002</div>" } ], "returns" : { "type" : "double", "description" : "cosh(arg)" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "deg-to-rad", "qname" : "math:deg-to-rad", "signature" : "($deg as double) as double", "description" : " Convert angle from degrees to radians. <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n The parameter is first converted to value range of (-360, 360).\n", "summary" : "<p> Convert angle from degrees to radians.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "deg", "type" : "double", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> angle in degrees</div>" } ], "returns" : { "type" : "double", "description" : "value in radians (-2PI, 2PI)" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "even", "qname" : "math:even", "signature" : "($number as double) as integer", "description" : " Returns number rounded up to the nearest even integer.<p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n Regardless of the sign of number, a value is rounded up when adjusted away from zero.<p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n Borrowed from excel module.\n", "summary" : "<p> Returns number rounded up to the nearest even integer.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "number", "type" : "double", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The value to round.</div>" } ], "returns" : { "type" : "integer", "description" : "The rounded value casted as numeric type." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "fact", "qname" : "math:fact", "signature" : "($number as integer) as integer", "description" : " Returns the factorial of a number.<p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n Borrowed from excel module.\n", "summary" : "<p> Returns the factorial of a number.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "number", "type" : "integer", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The non-negative number you want the factorial of.</div>" } ], "returns" : { "type" : "integer", "description" : "Returns the factorial of a number. The factorial of a number is equal to 1*2*3*...* number." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">math:INVALID_ARGUMENT if the number is smaller than zero</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 1, "name" : "factdouble", "qname" : "math:factdouble", "signature" : "($number as integer) as integer", "description" : " Returns the double factorial of a number.<p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n Computes the double factorial of n as n(n-2)(n-4)...<p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n Borrowed from excel module.\n", "summary" : "<p> Returns the double factorial of a number.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "number", "type" : "integer", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The positive integer value.</div>" } ], "returns" : { "type" : "integer", "description" : "The result as integer." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">math:INVALID_ARGUMENT if the number is negative.</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 2, "name" : "floor", "qname" : "math:floor", "signature" : "($number as double, $significance as double) as double", "description" : " Rounds number down, toward zero, to the nearest multiple of significance.<p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n Significance must have the same sign as number.\n Borrowed from excel module.\n", "summary" : "<p> Rounds number down, toward zero, to the nearest multiple of significance.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "number", "type" : "double", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The value you want to round.</div>" }, { "name" : "significance", "type" : "double", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The multiple to which you want to round.</div>" } ], "returns" : { "type" : "double", "description" : "The rounded value as numeric type." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">math:INVALID_ARGUMENT if significance is zero or it doesn't have the same sign as number.</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 2, "name" : "fmod", "qname" : "math:fmod", "signature" : "($x as double, $y as double) as double external", "description" : " Function performing the modulo operation between the two arguments.\n", "summary" : "<p> Function performing the modulo operation between the two arguments.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "x", "type" : "double", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the x</div>" }, { "name" : "y", "type" : "double", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the y</div>" } ], "returns" : { "type" : "double", "description" : "The remainder of x/y." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "frexp", "qname" : "math:frexp", "signature" : "($arg as double) as double+ external", "description" : " Returns the argument split as mantissa and exponent.<p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n The recombining formula is (mantissa * 2^exponent).\n", "summary" : "<p> Returns the argument split as mantissa and exponent.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "arg", "type" : "double", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the double to be split.</div>" } ], "returns" : { "type" : "double+", "description" : "A sequence of two doubles (mantissa, exponent)" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "gcd", "qname" : "math:gcd", "signature" : "($numbers as integer+) as integer", "description" : " Returns the greatest common divisor GCD of a sequence of integers.<p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n The sequence can have one or more positive integers.<p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n Borrowed from excel module.\n", "summary" : "<p> Returns the greatest common divisor GCD of a sequence of integers.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "numbers", "type" : "integer", "occurrence" : "+", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The sequence of positive integers.</div>" } ], "returns" : { "type" : "integer", "description" : "The GCD as integer." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">math:INVALID_ARGUMENT if any number is smaller than zero.</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 1, "name" : "int", "qname" : "math:int", "signature" : "($number as double) as integer", "description" : " Rounds a number down to the nearest integer.<p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n Positive numbers are rounded toward zero, negative numbers are rounded away from zero.<p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n Borrowed from excel module.\n", "summary" : "<p> Rounds a number down to the nearest integer.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "number", "type" : "double", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The value to be rounded.</div>" } ], "returns" : { "type" : "integer", "description" : "The rounded integer." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "is-a-number", "qname" : "math:is-a-number", "signature" : "($value as anyAtomicType) as boolean", "description" : " Checks if the anyAtomicType argument is actually a numeric type\n or can be converted to numeric.<p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n Borrowed from excel module.\n", "summary" : "<p> Checks if the anyAtomicType argument is actually a numeric type\n or can be converted to numeric.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "value", "type" : "anyAtomicType", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> Parameter to be checked.</div>" } ], "returns" : { "type" : "boolean", "description" : "true if the value can be casted to numeric." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "is_inf", "qname" : "math:is_inf", "signature" : "($arg as double) as boolean external", "description" : " Checks if the double value is positive or negative infinite.\n", "summary" : "<p> Checks if the double value is positive or negative infinite.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "arg", "type" : "double", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the double to be checked</div>" } ], "returns" : { "type" : "boolean", "description" : "boolean true if argument is pos INF or neg INF" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "is_nan", "qname" : "math:is_nan", "signature" : "($arg as double) as boolean external", "description" : " Checks if the double value is Not a Number (NaN).\n", "summary" : "<p> Checks if the double value is Not a Number (NaN).</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "arg", "type" : "double", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the arg</div>" } ], "returns" : { "type" : "boolean", "description" : "boolean true if the double is NaN" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 2, "name" : "large", "qname" : "math:large", "signature" : "($numbers as double+, $k as integer) as double", "description" : " Returns the k-th largest value in a data set. <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n If n is the number of data points in a range,\n   then LARGE(array,1) returns the largest value,\n   and LARGE(array,n) returns the smallest value.<p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n Borrowed from excel module.<br xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n", "summary" : "<p> Returns the k-th largest value in a data set.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "numbers", "type" : "double", "occurrence" : "+", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequence of numbers The sequence can be of any length, from 1 up.</div>" }, { "name" : "k", "type" : "integer", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the position of largest value, with value from 1 to count of values</div>" } ], "returns" : { "type" : "double", "description" : "The k-th largest value as numeric type" }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">math:INVALID_ARGUMENT if the sequence is empty or k is not a value between 1 and the size of the sequence</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 1, "name" : "lcm", "qname" : "math:lcm", "signature" : "($numbers as integer+) as integer", "description" : " Returns the least common multiple of integers.<p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n LCM for two numbers is computed by multiplying them and dividing with GCD.\n The function is applied recursively replacing the first two numbers in the sequence with their LCM.<p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n Borrowed from excel module.\n", "summary" : "<p> Returns the least common multiple of integers.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "numbers", "type" : "integer", "occurrence" : "+", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The sequence of one or more positive integers.</div>" } ], "returns" : { "type" : "integer", "description" : "The LCM as integer." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">math:INVALID_ARGUMENT if any number is smaller than zero.</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 2, "name" : "ldexp", "qname" : "math:ldexp", "signature" : "($x as double, $i as integer) as double external", "description" : " Computes a real number from the mantissa and exponent.<p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n The formula is (x * 2^i).\n", "summary" : "<p> Computes a real number from the mantissa and exponent.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "x", "type" : "double", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the mantissa</div>" }, { "name" : "i", "type" : "integer", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the exponent</div>" } ], "returns" : { "type" : "double", "description" : "the computed real number" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "median", "qname" : "math:median", "signature" : "($numbers as double*) as double", "description" : " Returns the median of the given numbers. <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n The median is the number in the middle of a set of numbers.\n Half the numbers have values that are greater than the median,\n and half the numbers have values that are less than the median. <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n Borrowed from excel module.\n", "summary" : "<p> Returns the median of the given numbers.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "numbers", "type" : "double", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequence of numbers, of any length</div>" } ], "returns" : { "type" : "double", "description" : "for odd count of numbers return the number in the middle of the sorted sequence. For even count of numbers return the average of the two numbers in the middle." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 2, "name" : "mod", "qname" : "math:mod", "signature" : "($number as double, $divisor as double) as double", "description" : " Returns the remainder after number is divided by divisor.<p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n The result has the same sign as divisor.<p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n Borrowed from excel module.\n", "summary" : "<p> Returns the remainder after number is divided by divisor.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "number", "type" : "double", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The number for which you want to find the remainder.</div>" }, { "name" : "divisor", "type" : "double", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The number by which you want to divide number. This cannot be zero.</div>" } ], "returns" : { "type" : "double", "description" : "The remainder from division as numeric type." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">math:DIVIDE_BY_0 if divisor is zero after casting to numeric.</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 1, "name" : "mode", "qname" : "math:mode", "signature" : "($numbers as double*) as double", "description" : " Returns the most frequently occurring, or repetitive, value in a sequence.<p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n Borrowed from excel module.\n", "summary" : "<p> Returns the most frequently occurring, or repetitive, value in a sequence.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "numbers", "type" : "double", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequence of numbers, of any length</div>" } ], "returns" : { "type" : "double", "description" : "The most occuring number" }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">math:INVALID_INPUT if there are no duplicate numbers</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 1, "name" : "modf", "qname" : "math:modf", "signature" : "($arg as double) as double+ external", "description" : " Splits a floating-point value into fractional and integer parts.<p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n Both the fraction and integer keep the original sign of the value.\n", "summary" : "<p> Splits a floating-point value into fractional and integer parts.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "arg", "type" : "double", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the double to be split.</div>" } ], "returns" : { "type" : "double+", "description" : "A sequence of two doubles (fraction, integer)" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 2, "name" : "mround", "qname" : "math:mround", "signature" : "($number as decimal, $multiple as double) as double", "description" : " Returns a number rounded to the desired multiple.<p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n MROUND rounds up, away from zero, if the remainder of dividing number by multiple\n is greater than or equal to half the value of multiple.\n MROUND is computed through math:floor function.<p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n Borrowed from excel module.\n", "summary" : "<p> Returns a number rounded to the desired multiple.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "number", "type" : "decimal", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The value to round,</div>" }, { "name" : "multiple", "type" : "double", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The multiple to which you want to round number.</div>" } ], "returns" : { "type" : "double", "description" : "The rounded number up to the desired multiple." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "odd", "qname" : "math:odd", "signature" : "($number as double) as integer", "description" : " Returns number rounded up to the nearest odd integer, away from zero.<p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n Borrowed from excel module.\n", "summary" : "<p> Returns number rounded up to the nearest odd integer, away from zero.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "number", "type" : "double", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The value to round.</div>" } ], "returns" : { "type" : "integer", "description" : "The odd integer." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 2, "name" : "percentile", "qname" : "math:percentile", "signature" : "($numbers as double*, $k_at as double) as double", "description" : " Returns the k-th percentile of values in a sequence.<p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n If k is not a multiple of 1/(n - 1),\n   PERCENTILE interpolates to determine the value at the k-th percentile.\n The function is computed by (max-min)*k + min<p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n Borrowed from excel module.\n", "summary" : "<p> Returns the k-th percentile of values in a sequence.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "numbers", "type" : "double", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequence of numbers, of any length</div>" }, { "name" : "k_at", "type" : "double", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the percentile, with value between 0 .. 1 inclusive</div>" } ], "returns" : { "type" : "double", "description" : "The computed percentile" }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">math:INVALID_ARGUMENT if percentile is not between 0 .. 1</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 2, "name" : "percentrank", "qname" : "math:percentrank", "signature" : "($numbers as double*, $x as double) as double", "description" : " Returns the rank of a value in a data set as a percentage of the data set.<p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n If x does not match one of the values in array,\n   PERCENTRANK interpolates to return the correct percentage rank. <br xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n The formula is uses: (RANK - 1) / (size - 1) .<p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n Borrowed from excel module.\n", "summary" : "<p> Returns the rank of a value in a data set as a percentage of the data set.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "numbers", "type" : "double", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequence of numbers. The sequence can be of any length, from 1 up.</div>" }, { "name" : "x", "type" : "double", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> is the value for which you want to know the rank</div>" } ], "returns" : { "type" : "double", "description" : "The percentage of rank." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">math:INVALID_ARGUMENT if the sequence is zero length</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 3, "name" : "prob", "qname" : "math:prob", "signature" : "($x_range as double+, $prob_range as double+, $range_lower_limit as double) as double", "description" : " This is the same as math:prob#4, only that upper_limit is not specified.<p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n The probability is computed only for range_lower_limit.<p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n Borrowed from excel module.\n", "summary" : "<p> This is the same as math:prob#4, only that upper_limit is not specified.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "x_range", "type" : "double", "occurrence" : "+", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> is the range of numeric values of x with which there are associated probabilities. This does not need to be ordered.</div>" }, { "name" : "prob_range", "type" : "double", "occurrence" : "+", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> is a set of probabilities associated with values in x_range.</div>" }, { "name" : "range_lower_limit", "type" : "double", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> is the value for which you want a probability.</div>" } ], "returns" : { "type" : "double", "description" : "The probability of the range_lower_limit value" }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">math:INVALID_ARGUMENT if any probability is not between 0 and 1</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">math:INVALID_ARGUMENT if the sum of probabilities is not equal to 1</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">math:INVALID_ARGUMENT if x_range and prob_range do not have the same number of values</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 4, "name" : "prob", "qname" : "math:prob", "signature" : "($x_range as double+, $prob_range as double+, $range_lower_limit as double, $upper_limit as double) as double", "description" : " Returns the probability that values in a range are between two limits.<p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n Borrowed from excel module.\n", "summary" : "<p> Returns the probability that values in a range are between two limits.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "x_range", "type" : "double", "occurrence" : "+", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> is the range of numeric values of x with which there are associated probabilities. This does not need to be ordered.</div>" }, { "name" : "prob_range", "type" : "double", "occurrence" : "+", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> is a set of probabilities associated with values in x_range.</div>" }, { "name" : "range_lower_limit", "type" : "double", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> is the lower bound on the value for which you want a probability.</div>" }, { "name" : "upper_limit", "type" : "double", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> is the upper bound on the value for which you want a probability.</div>" } ], "returns" : { "type" : "double", "description" : "The probability of the entire range" }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">math:INVALID_ARGUMENT if any probability is not between 0 and 1</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">math:INVALID_ARGUMENT if the sum of probabilities is not equal to 1</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">math:INVALID_ARGUMENT if x_range and prob_range do not have the same number of values</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 1, "name" : "product", "qname" : "math:product", "signature" : "($numbers as double*) as double", "description" : " Multiplies all the numbers given as arguments and returns the product.<p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n Borrowed from excel module.\n", "summary" : "<p> Multiplies all the numbers given as arguments and returns the product.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "numbers", "type" : "double", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The sequence of arguments convertible to numeric types. The sequence can be of any length.</div>" } ], "returns" : { "type" : "double", "description" : "The multiplication result as numeric type." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 2, "name" : "quartile", "qname" : "math:quartile", "signature" : "($numbers as double*, $quart as integer) as double", "description" : " Returns the quartile of a data set. <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n Borrowed from excel module.\n", "summary" : "<p> Returns the quartile of a data set.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "numbers", "type" : "double", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> sequence of numbers. The sequence can be of any length, from 1 up.</div>" }, { "name" : "quart", "type" : "integer", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> <dl>one of the values 0, 1, 2, 3, 4 with meaning: <dt>0</dt> <dd> compute minimum value</dd> <dt>1</dt> <dd> compute first quartile (25th percentile)</dd> <dt>2</dt> <dd> compute median value (50th percentile)</dd> <dt>3</dt> <dd> compute third quartile (75th percentile)</dd> <dt>4</dt> <dd> compute maximum value</dd></dl></div>" } ], "returns" : { "type" : "double", "description" : "the computed quartile, as numeric type" }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">math:INVALID_ARGUMENT if the sequence is zero length or $quart is not one of the values 0,1,3,4</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 2, "name" : "quotient", "qname" : "math:quotient", "signature" : "($numerator as double, $denominator as double) as integer", "description" : " Returns the integer portion of a division.<p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n Borrowed from excel module.\n", "summary" : "<p> Returns the integer portion of a division.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "numerator", "type" : "double", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The divider.</div>" }, { "name" : "denominator", "type" : "double", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The divisor. It cannot be zero.</div>" } ], "returns" : { "type" : "integer", "description" : "The result value as numeric type." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">math:DIVIDE_BY_0 if denominator casted as numeric type has value zero.</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 1, "name" : "rad-to-deg", "qname" : "math:rad-to-deg", "signature" : "($rad as double) as double", "description" : " Convert angle from radians to degrees. <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n", "summary" : "<p> Convert angle from radians to degrees.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "rad", "type" : "double", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> value in radians</div>" } ], "returns" : { "type" : "double", "description" : "value in degrees (-360, 360)" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 2, "name" : "rank", "qname" : "math:rank", "signature" : "($x as double, $numbers as double*) as double", "description" : " This RANK function is same as the above, only that $order_ascending is set by default to false.<p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n Borrowed from excel module.\n", "summary" : "<p> This RANK function is same as the above, only that $order_ascending is set by default to false.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "x", "type" : "double", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The number whose rank you want to find.</div>" }, { "name" : "numbers", "type" : "double", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequence of numbers. The sequence can be of any length.</div>" } ], "returns" : { "type" : "double", "description" : "The rank of $x." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 3, "name" : "rank", "qname" : "math:rank", "signature" : "($x as double, $numbers as double*, $order_ascending as boolean) as double", "description" : " Returns the rank of a number in a list of numbers. <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n The rank of a number is its size relative to other values in a list.\n (If you were to sort the list, the rank of the number would be its position.)\n RANK gives duplicate numbers the same rank.<p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n Borrowed from excel module.\n", "summary" : "<p> Returns the rank of a number in a list of numbers.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "x", "type" : "double", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The number whose rank you want to find.</div>" }, { "name" : "numbers", "type" : "double", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The sequence of numbers. The sequence can be of any length.</div>" }, { "name" : "order_ascending", "type" : "boolean", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> <dl>A boolean having the meaning: <dt>false</dt><dd>then rank the number as if the sequence was sorted in descending order.</dd> <dt>true</dt> <dd>then rank the number as if the sequence was sorted in ascending order.</dd></dl></div>" } ], "returns" : { "type" : "double", "description" : "The rank of $x." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "roman", "qname" : "math:roman", "signature" : "($number as integer) as string", "description" : " Converts an Arabic numeral to roman, as text.<p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n Only the classic format is supported (out of all formats Excel requires).\n M is the largest digit, it represents 1000.\n Numbers bigger than 2000 will be represented by a sequence of \"M\".\n D = 500, C = 100, L = 50, X = 10, V = 5, I = 1.<p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n Borrowed from excel module.\n", "summary" : "<p> Converts an Arabic numeral to roman, as text.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "number", "type" : "integer", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> A positive integer.</div>" } ], "returns" : { "type" : "string", "description" : "The roman string representation." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">math:INVALID_ARGUMENT if the input integer is negative</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 2, "name" : "round", "qname" : "math:round", "signature" : "($number as double, $precision as integer) as double", "description" : " Rounds a number to a specified number of digits.<p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n If precision is greater than 0 (zero), then number is rounded\n to the specified number of decimal places.\n If num_digits is 0, then number is rounded to the nearest integer.\n If num_digits is less than 0, then number is rounded to the left of the decimal point.\n The 0.5 is rounded away from zero. <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n Borrowed from excel module.\n", "summary" : "<p> Rounds a number to a specified number of digits.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "number", "type" : "double", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The number to round.</div>" }, { "name" : "precision", "type" : "integer", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The number of decimal places to keep.</div>" } ], "returns" : { "type" : "double", "description" : "The rounded number as numeric type." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 2, "name" : "rounddown", "qname" : "math:rounddown", "signature" : "($number as double, $precision as integer) as double", "description" : " Rounds a number down, toward zero.<p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n If num_digits is greater than 0 (zero), then number is rounded down\n to the specified number of decimal places.\n If num_digits is 0, then number is rounded down to the nearest integer.\n If num_digits is less than 0, then number is rounded down to the left of the decimal point. <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n Borrowed from excel module.\n", "summary" : "<p> Rounds a number down, toward zero.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "number", "type" : "double", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The number to round</div>" }, { "name" : "precision", "type" : "integer", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The number of decimal places to keep.</div>" } ], "returns" : { "type" : "double", "description" : "the truncated number toward zero, as numeric type." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 2, "name" : "roundup", "qname" : "math:roundup", "signature" : "($number as double, $precision as integer) as double", "description" : " Rounds a number up, away from 0 (zero).<p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n If num_digits is greater than 0 (zero), then number is rounded down\n to the specified number of decimal places.\n If num_digits is 0, then number is rounded down to the nearest integer.\n If num_digits is less than 0, then number is rounded down to the left of the decimal point. <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n Borrowed from excel module.\n", "summary" : "<p> Rounds a number up, away from 0 (zero).</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "number", "type" : "double", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The number to round</div>" }, { "name" : "precision", "type" : "integer", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The number of decimal places to keep.</div>" } ], "returns" : { "type" : "double", "description" : "The truncated number away from zero, as numeric type." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "sign", "qname" : "math:sign", "signature" : "($number as double) as integer", "description" : " Determines the sign of a number. <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n Returns 1 if the number is positive, zero (0) if the number is 0,\n and -1 if the number is negative.<p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n Borrowed from excel module.\n", "summary" : "<p> Determines the sign of a number.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "number", "type" : "double", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The argument</div>" } ], "returns" : { "type" : "integer", "description" : "The sign as (-1, 0, 1)." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "sinh", "qname" : "math:sinh", "signature" : "($arg as double) as double external", "description" : " Calculate the hyperbolic sine.\n", "summary" : "<p> Calculate the hyperbolic sine.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "arg", "type" : "double", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the arg</div>" } ], "returns" : { "type" : "double", "description" : "the result of sinh(arg)" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 2, "name" : "slope", "qname" : "math:slope", "signature" : "($known_y as double+, $known_x as double+) as double", "description" : " Returns the slope of the linear regression line through data points in known_y's and known_x's.<p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n The slope is the vertical distance divided by the horizontal distance between\n   any two points on the line, which is the rate of change along the regression line.\n It computes the formula:<p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n sum((x - average_x)(y - average_y)) / sum((x - average_x)^2)<p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n where average_x and average_y are computed with AVERAGE function.<p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n Borrowed from excel module.\n", "summary" : "<p> Returns the slope of the linear regression line through data points in known_y's and known_x's.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "known_y", "type" : "double", "occurrence" : "+", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequence of y numbers. The sequence can be of any length, from 1 up.</div>" }, { "name" : "known_x", "type" : "double", "occurrence" : "+", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequence of x numbers. The sequence can be of any length, from 1 up.</div>" } ], "returns" : { "type" : "double", "description" : "The slope value, as numeric type" }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">math:INVALID_INPUT if there are different numbers of x's and y's or if the sequence is empty</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">math:DIVIDE_BY_0 if all x's are equal</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 2, "name" : "small", "qname" : "math:small", "signature" : "($numbers as double*, $k as integer) as double", "description" : " This function computes the k-th smallest value in a data set. <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n Use this function to return values with a particular relative standing in a data set.\n If n is the number of data points in array, SMALL(array,1) equals the smallest value,\n   and SMALL(array,n) equals the largest value.\n Borrowed from excel module.\n", "summary" : "<p> This function computes the k-th smallest value in a data set.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "numbers", "type" : "double", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> A sequence of numbers. The sequence can be of any length, from 1 up.</div>" }, { "name" : "k", "type" : "integer", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The position (from the smallest) in the sequence of data to return. Must have value between 1 and size of sequence.</div>" } ], "returns" : { "type" : "double", "description" : "The k-th smallest value of $numbers." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">math:INVALID_ARGUMENT if the sequence is zero length or $k is not a value between 1 and the size of sequence.</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 1, "name" : "sort-numbers", "qname" : "math:sort-numbers", "signature" : "($numbers as double*) as double*", "description" : " Sorts a sequence of numbers or arguments castable to numeric.<p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n It first casts all arguments to numeric and then sorts ascending.<p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n Helper function.<br xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n Borrowed from excel module.\n", "summary" : "<p> Sorts a sequence of numbers or arguments castable to numeric.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "numbers", "type" : "double", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The sequence of arguments castable to numeric.</div>" } ], "returns" : { "type" : "double*", "description" : "The sorted sequence as numeric types." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 3, "name" : "standardize", "qname" : "math:standardize", "signature" : "($x as double, $mean as double, $standard_dev as double) as double", "description" : " Returns a normalized value from a distribution characterized by mean and standard_dev.<p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n The formula is (x - mean) / standard_dev .<p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n Borrowed from excel module.\n", "summary" : "<p> Returns a normalized value from a distribution characterized by mean and standard_dev.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "x", "type" : "double", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> is the value you want to normalize</div>" }, { "name" : "mean", "type" : "double", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> is the arithmetic mean of the distribution.</div>" }, { "name" : "standard_dev", "type" : "double", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> is the standard deviation of the distribution.</div>" } ], "returns" : { "type" : "double", "description" : "The normalized x, as numeric type" }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">math:INVALID_ARGUMENT if standard_dev is a value smaller than zero or equal</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 1, "name" : "stdev", "qname" : "math:stdev", "signature" : "($numbers as double+) as double", "description" : " Estimates standard deviation based on a sample. <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n The standard deviation is a measure of how widely values are dispersed\n   from the average value (the mean).\n It is computed with formula:\n sqrt( sum((x-average_x)^2) / (n-1) )    = sqrt ( VAR(numbers) )<p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n Borrowed from excel module.\n", "summary" : "<p> Estimates standard deviation based on a sample.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "numbers", "type" : "double", "occurrence" : "+", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequence of numbers The sequence can be of any length, from 1 up.</div>" } ], "returns" : { "type" : "double", "description" : "the standard deviation, as numeric type" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "stdeva", "qname" : "math:stdeva", "signature" : "($numbers as double+) as double", "description" : " Estimates standard deviation based on a sample. <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n The standard deviation is a measure of how widely values are dispersed\n   from the average value (the mean).\n It is computed with formula:\n sqrt( sum((x-average_x)^2) / (n-1) )    = sqrt ( VARA(numbers) )<p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n Borrowed from excel module.\n", "summary" : "<p> Estimates standard deviation based on a sample.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "numbers", "type" : "double", "occurrence" : "+", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequence of numbers. The sequence can be of any length, from 1 up.</div>" } ], "returns" : { "type" : "double", "description" : "the standard deviation, as numeric type" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "stdevp", "qname" : "math:stdevp", "signature" : "($numbers as double+) as double", "description" : " Calculates standard deviation based on the entire population given as arguments. <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n The standard deviation is a measure of how widely values are dispersed from\n   the average value (the mean).\n It is computed with formula:\n sqrt( sum((x-average_x)^2) / n )    = sqrt ( VARP(numbers) )<p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n Borrowed from excel module.\n", "summary" : "<p> Calculates standard deviation based on the entire population given as arguments.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "numbers", "type" : "double", "occurrence" : "+", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequence of numbers or values castable to numeric The sequence can be of any length, from 1 up.</div>" } ], "returns" : { "type" : "double", "description" : "the standard deviation, as numeric type" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "stdevpa", "qname" : "math:stdevpa", "signature" : "($numbers as double+) as double", "description" : " Calculates standard deviation based on the entire population given as arguments. <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n The standard deviation is a measure of how widely values are dispersed from\n   the average value (the mean).\n It is computed with formula:\n sqrt( sum((x-average_x)^2) / n )    = sqrt ( VARPA(numbers) )<p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n Borrowed from excel module.\n", "summary" : "<p> Calculates standard deviation based on the entire population given as arguments.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "numbers", "type" : "double", "occurrence" : "+", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequence of numbers or values castable to numeric The sequence can be of any length, from 1 up.</div>" } ], "returns" : { "type" : "double", "description" : "the standard deviation, as numeric type" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 2, "name" : "subtotal", "qname" : "math:subtotal", "signature" : "($function_num as integer, $numbers as double*) as double", "description" : " Returns a subtotal in a sequence of numbers.<p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n The function applied is given by $function_num.<p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n Borrowed from excel module.\n", "summary" : "<p> Returns a subtotal in a sequence of numbers.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "function_num", "type" : "integer", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> <dl>defines the function to be applied on sequence values. The possible values are: <dt>1 or 101</dt> <dd> AVERAGE</dd> <dt>2 or 102</dt> <dd> COUNT</dd> <dt>3 or 103</dt> <dd> COUNTA</dd> <dt>4 or 104</dt> <dd> MAX</dd> <dt>5 or 105</dt> <dd> MIN</dd> <dt>6 or 106</dt> <dd> PRODUCT</dd> <dt>7 or 107</dt> <dd> STDEV</dd> <dt>8 or 108</dt> <dd> STDEVP</dd> <dt>9 or 109</dt> <dd> SUM</dd> <dt>10 or 110</dt> <dd> VAR</dd> <dt>11 or 111</dt> <dd> VARP</dd></dl> In this implementation there is no difference between x and 10x.<br/></div>" }, { "name" : "numbers", "type" : "double", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequence of numbers. The sequence can be of any length.</div>" } ], "returns" : { "type" : "double", "description" : "The function result, as numeric type" }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">* depends on the function called</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">math:INVALID_ARGUMENT if $function_num is not a value between 1 .. 11 or 101 .. 111</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 2, "name" : "sumproduct", "qname" : "math:sumproduct", "signature" : "($array1 as double*, $array2 as double*) as double", "description" : " Multiplies the elements on the same position in each sequence\n and sums up the results.<p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n Borrowed from excel module.\n", "summary" : "<p> Multiplies the elements on the same position in each sequence\n and sums up the results.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "array1", "type" : "double", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers</div>" }, { "name" : "array2", "type" : "double", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers</div>" } ], "returns" : { "type" : "double", "description" : "the sum of products" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "sumsq", "qname" : "math:sumsq", "signature" : "($numbers as double+) as double", "description" : " Returns the sum of the squares of the arguments.<p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n It uses the sumproduct function.<p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n Borrowed from excel module.\n", "summary" : "<p> Returns the sum of the squares of the arguments.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "numbers", "type" : "double", "occurrence" : "+", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequence of one or more numbers</div>" } ], "returns" : { "type" : "double", "description" : "the sum of squared values, as numeric type" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "tanh", "qname" : "math:tanh", "signature" : "($arg as double) as double external", "description" : " Calculate the hyperbolic tangent.\n", "summary" : "<p> Calculate the hyperbolic tangent.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "arg", "type" : "double", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the arg</div>" } ], "returns" : { "type" : "double", "description" : "the result of tanh(arg)" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "trunc", "qname" : "math:trunc", "signature" : "($number as double) as integer", "description" : " Truncates a number to an integer by removing the fractional part of the number.<p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n Borrowed from excel module.\n", "summary" : "<p> Truncates a number to an integer by removing the fractional part of the number.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "number", "type" : "double", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The argument .</div>" } ], "returns" : { "type" : "integer", "description" : "The integer value." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 2, "name" : "trunc", "qname" : "math:trunc", "signature" : "($number as double, $precision as integer) as double", "description" : " Truncates a number down to precision.<p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n This behaves exactly like rounddown.<p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n Borrowed from excel module.\n", "summary" : "<p> Truncates a number down to precision.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "number", "type" : "double", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The argument castable to numeric type.</div>" }, { "name" : "precision", "type" : "integer", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The number of decimal places to keep .</div>" } ], "returns" : { "type" : "double", "description" : "The integer value." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "var", "qname" : "math:var", "signature" : "($numbers as double+) as double", "description" : " Estimates variance based on a sample.<p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n The formula is sum(x - average_x)^2 / (n - 1).\n average_x is computed with AVERAGE function.\n n is the count of numbers from the sequence, excluding empty values.<p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n Borrowed from excel module.\n", "summary" : "<p> Estimates variance based on a sample.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "numbers", "type" : "double", "occurrence" : "+", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequence of numbers. The sequence can be of any length, from 1 up.</div>" } ], "returns" : { "type" : "double", "description" : "The variance, as numeric type" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "vara", "qname" : "math:vara", "signature" : "($numbers as double+) as double", "description" : " Estimates variance based on a sample.<p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n The formula is sum(x - average_x)^2 / (n - 1).\n average_x is computed with AVERAGE function.\n n is the size of sequence, including empty values.<p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n Borrowed from excel module.\n", "summary" : "<p> Estimates variance based on a sample.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "numbers", "type" : "double", "occurrence" : "+", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequence of numbers. The sequence can be of any length, from 1 up.</div>" } ], "returns" : { "type" : "double", "description" : "The variance, as numeric type" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "varp", "qname" : "math:varp", "signature" : "($numbers as double+) as double", "description" : " Calculates variance based on the entire population.<p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n The formula is sum(x - average_x)^2 / n.\n average_x is computed with AVERAGE function.\n n is the count of numbers from the sequence, excluding empty values.<p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n Borrowed from excel module.<br xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n", "summary" : "<p> Calculates variance based on the entire population.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "numbers", "type" : "double", "occurrence" : "+", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequence of numbers. The sequence can be of any length, from 1 up.</div>" } ], "returns" : { "type" : "double", "description" : "The variance, as numeric type" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "varpa", "qname" : "math:varpa", "signature" : "($numbers as double+) as double", "description" : " Calculates variance based on the entire population.<p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n The formula is sum(x - average_x)^2 / n.\n average_x is computed with AVERAGE function.\n n is the size of sequence, including empty values.<p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n Borrowed from excel module.\n", "summary" : "<p> Calculates variance based on the entire population.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "numbers", "type" : "double", "occurrence" : "+", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequence of numbers. The sequence can be of any length, from 1 up.</div>" } ], "returns" : { "type" : "double", "description" : "The variance, as numeric type" }, "errors" : [  ] } ], "variables" : [  ] }, "http://www.zorba-xquery.com/modules/oauth/error" : { "ns" : "http://www.zorba-xquery.com/modules/oauth/error", "description" : " Module that defines the errors raised in Oauth modules.\n", "sees" : [  ], "authors" : [ "<xqdoc:author xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Stephanie Russell</xqdoc:author>" ], "version" : null, "encoding" : "utf-8", "namespaces" : [ { "uri" : "http://www.zorba-xquery.com/modules/oauth/error", "prefix" : "oerr" }, { "uri" : "http://zorba.io/options/versioning", "prefix" : "ver" } ], "functions" : [  ], "variables" : [ { "name" : "oerr:errNS", "type" : "xs:string", "description" : " Errors namespace URI.\n" }, { "name" : "oerr:OC001", "type" : "xs:QName", "description" : " xs:QName with namespace URI=\"http://www.zorba-xquery.com/modules/oauth/errors\" and local name 'OC001'. This signing method is not implemented yet.\n" }, { "name" : "oerr:OC002", "type" : "xs:QName", "description" : " xs:QName with namespace URI=\"http://www.zorba-xquery.com/modules/oauth/errors\" and local name 'OC002'. This signing method is not supported.\n" }, { "name" : "oerr:OC003", "type" : "xs:QName", "description" : " xs:QName with namespace URI=\"http://www.zorba-xquery.com/modules/oauth/errors\" and local name 'OC003'. Http 401 error.\n" }, { "name" : "oerr:OC004", "type" : "xs:QName", "description" : " xs:QName with namespace URI=\"http://www.zorba-xquery.com/modules/oauth/errors\" and local name 'OC004'. Http 500 error.\n" }, { "name" : "oerr:OC005", "type" : "xs:QName", "description" : " xs:QName with namespace URI=\"http://www.zorba-xquery.com/modules/oauth/errors\" and local name 'OC004'. Http 500 error.\n" } ] }, "http://jsoniq.org/functions" : { "ns" : "http://jsoniq.org/functions", "description" : " This module contains all of the functions defined by the JSONiq\n specification (see http://jsoniq.org/).\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">The module is always imported so you do not need to import it explicitly.\n Also, you do not need to fully qualify a function to invoke it.</p>\n", "sees" : [  ], "authors" : [ "<xqdoc:author xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Markos Zaharioudakis, Matthias Brantner, Ghislain Fourny</xqdoc:author>" ], "version" : null, "encoding" : "utf-8", "namespaces" : [ { "uri" : "http://www.w3.org/2005/xqt-errors", "prefix" : "err" }, { "uri" : "http://jsoniq.org/errors", "prefix" : "jerr" }, { "uri" : "http://jsoniq.org/functions", "prefix" : "jn" }, { "uri" : "http://jsoniq.org/types", "prefix" : "js" }, { "uri" : "http://zorba.io/modules/schema", "prefix" : "schema" }, { "uri" : "http://zorba.io/options/versioning", "prefix" : "ver" } ], "functions" : [ { "isDocumented" : true, "arity" : 1, "name" : "decode-from-roundtrip", "qname" : "jn:decode-from-roundtrip", "signature" : "($items as item()*) as item()* external", "description" : " This function decodes non-JSON types previously encoded with\n jn:encode-for-roundtrip.\n Calling this version of the function is equivalent to calling the\n 2 argument version of the function with the second argument\n   { \"prefix\" : \"Q{http://jsoniq.org/roundtrip}\" }\n", "summary" : "<p> This function decodes non-JSON types previously encoded with\n jn:encode-for-roundtrip.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "items", "type" : "item()", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the items to be decoded.</div>" } ], "returns" : { "type" : "item()*", "description" : "the decoded items." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 2, "name" : "decode-from-roundtrip", "qname" : "jn:decode-from-roundtrip", "signature" : "($items as item()*, $options as object()) as item()* external", "description" : " This function decodes non-JSON types previously encoded with\n jn:encode-for-roundtrip.\n The $options parameter contains options for the decoding process.\n Currently the only supported option is \"prefix\". It specifies the prefix\n that determines if this function decodes an item.\n Example:\n   jn:decode-from-roundtrip(\n     { \"nan\" : { \"pre-type\" : \"double\", \"pre-value\" : \"NaN\" } },\n     { \"prefix\" : \"pre-\" }\n   )\n returns the same instance that would be constructed by\n   { \"nan\" : double(\"NaN\") }\n So\n   let $decoded := jn:decode-from-roundtrip(\n           { \"nan\" : { \"pre-type\" : \"double\", \"pre-value\" : \"NaN\" } },\n           { \"prefix\" : \"pre-\" }\n       )\n   let $nan := $decoded(\"nan\")\n   return\n       ($nan instance of double, $nan)\n returns\n   true NaN\n", "summary" : "<p> This function decodes non-JSON types previously encoded with\n jn:encode-for-roundtrip.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "items", "type" : "item()", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the items to be decoded.</div>" }, { "name" : "options", "type" : "object()", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the decoding options.</div>" } ], "returns" : { "type" : "item()*", "description" : "the decoded items." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">jerr:JNTY0023 if $options(\"prefix\") is not a string</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 1, "name" : "encode-for-roundtrip", "qname" : "jn:encode-for-roundtrip", "signature" : "($items as item()*) as item()* external", "description" : " This function recursively encodes non-JSON types in such a way that they\n can be serialized as JSON while keeping roundtrip capability.\n", "summary" : "<p> This function recursively encodes non-JSON types in such a way that they\n can be serialized as JSON while keeping roundtrip capability.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "items", "type" : "item()", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the items to be encoded.</div>" } ], "returns" : { "type" : "item()*", "description" : "the encoded items." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 2, "name" : "encode-for-roundtrip", "qname" : "jn:encode-for-roundtrip", "signature" : "($items as item()*, $options as object()) as item()* external", "description" : " This function recursively encodes non-JSON types in such a way that they\n can be serialized as JSON while keeping roundtrip capability.\n Example:\n   jn:encode-for-roundtrip(\n     { \"nan\" : double(\"NaN\") },\n     { \"prefix\" : \"pre-\" }\n   )\n returns\n   { \"nan\" : { \"pre-type\" : \"double\", \"pre-value\" : \"NaN\" } }\n", "summary" : "<p> This function recursively encodes non-JSON types in such a way that they\n can be serialized as JSON while keeping roundtrip capability.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "items", "type" : "item()", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the items to be encoded.</div>" }, { "name" : "options", "type" : "object()", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the encoding options.</div>" } ], "returns" : { "type" : "item()*", "description" : "the encoded items." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">jerr:JNTY0023 if $options(\"prefix\") is not a string</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 1, "name" : "flatten", "qname" : "jn:flatten", "signature" : "($items as item()*) as item()* external", "description" : " For each item in the given sequence, this function returns the item itself,\n if it is not an array, or a sequence of items \"flattened-out\" from the array.\n Flattening an array means replacing the array with its members, and recursively\n flattening any arrays in the members sequence.\n Note: The function is equivalent to\n   define function jn:flatten($args as item()*)\n   {\n     for $arg in args\n     return\n       if ($arg instance of array())\n       then\n         for $value in $arg[]\n         return\n           if ($value instance of array())\n           then jn:flatten($value[])\n           else $value\n       else\n         $arg\n   };\n", "summary" : "<p> For each item in the given sequence, this function returns the item itself,\n if it is not an array, or a sequence of items \"flattened-out\" from the array.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "items", "type" : "item()", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> A sequence of items</div>" } ], "returns" : { "type" : "item()*", "description" : "The flattened-out items of the arrays in $items." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "keys", "qname" : "jn:keys", "signature" : "($o as item()*) as string* external", "description" : " Returns the set of keys belonging to the objects found inside a given\n sequence of items. The keys are returned in an implementation-defined\n order. Duplicate keys are eliminated.\n", "summary" : "<p> Returns the set of keys belonging to the objects found inside a given\n sequence of items.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "o", "type" : "item()", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> A sequence of items. Only object items are actually processed; items of any other kind are simply skipped.</div>" } ], "returns" : { "type" : "string*", "description" : "The distinct keys of the objects in the input sequence." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "members", "qname" : "jn:members", "signature" : "($a as item()*) as item()* external", "description" : " Returns the items belonging to the arrays found inside a given sequence\n of items. The items are returned in an implementation-defined order.\n", "summary" : "<p> Returns the items belonging to the arrays found inside a given sequence\n of items.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "a", "type" : "item()", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> A sequence of items. Only array items are actually processed; items of any other kind are simply skipped.</div>" } ], "returns" : { "type" : "item()*", "description" : "The members of the arrays in the input sequence." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 0, "name" : "null", "qname" : "jn:null", "signature" : "() as js:null external", "description" : " Returns the JSON null.\n", "summary" : "<p> Returns the JSON null.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [  ], "returns" : { "type" : "js:null", "description" : "The JSON null." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "parse-json", "qname" : "jn:parse-json", "signature" : "($j as string?) as json-item()* external", "description" : " This function parses a given string as JSON and returns a sequence\n of Objects or Arrays.\n Please note that this function allows to parse sequences of whitespace\n separated objects and arrays.\n", "summary" : "<p> This function parses a given string as JSON and returns a sequence\n of Objects or Arrays.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "j", "type" : "string", "occurrence" : "?", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> A string containing a valid JSON text.</div>" } ], "returns" : { "type" : "json-item()*", "description" : "A sequence of JSON Object or Array item." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">jerr:JNDY0021 if the given string is not valid JSON.</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 2, "name" : "parse-json", "qname" : "jn:parse-json", "signature" : "($j as string?, $o as object()) as json-item()* external", "description" : " This function parses a given string as JSON and returns a sequence\n of Objects or Arrays.\n", "summary" : "<p> This function parses a given string as JSON and returns a sequence\n of Objects or Arrays.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "j", "type" : "string", "occurrence" : "?", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> A string containing a valid JSON text.</div>" }, { "name" : "o", "type" : "object()", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> A JSON object defining options to configure the parser. Allowed options are <ul> <li>jsoniq-multiple-top-level-items: allow parsing of sequences of JSON Objects and Arrays (boolean; default: true)</li> <li>jsoniq-strip-top-level-array: if the top-level JSON item is an array, strip it and return its elements as multiple top-level items (boolean; default: false)</li> </ul></div>" } ], "returns" : { "type" : "json-item()*", "description" : "a sequence of JSON Object or Array item." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">jerr:JNDY0021 if the given string is not valid JSON or if jsoniq-multiple-top-level-items is false and there is additional content after the first JSON Object or Array.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">jerr:JNTY0020 if the value for the option jsoniq-multiple-top-level-items is not of type boolean.</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 2, "name" : "project", "qname" : "jn:project", "signature" : "($items as item()*, $keys as string*) as item()* external", "description" : " For each item in the given sequence, this function returns the item itself,\n if it is not an object, or its \"projected\" copy if it is an object. Projecting\n an object by a set of keys means creating a new object from the specified pairs\n of the source object. Specifically, for each key in $keys, if the object has a\n pair with that key, then a copy of that pair is included in the new object.\n", "summary" : "<p> For each item in the given sequence, this function returns the item itself,\n if it is not an object, or its \"projected\" copy if it is an object.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "items", "type" : "item()", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> A sequence of items.</div>" }, { "name" : "keys", "type" : "string", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The keys of the pairs to include from each object in $items.</div>" } ], "returns" : { "type" : "item()*", "description" : "The projection of the original sequence." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "size", "qname" : "jn:size", "signature" : "($a as array()?) as integer? external", "description" : " Returns the size of a JSON array, or the empty sequence if no array is given.\n The size of an Array is the number of members contained within it.\n", "summary" : "<p> Returns the size of a JSON array, or the empty sequence if no array is given.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "a", "type" : "array()", "occurrence" : "?", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">rray A JSON array.</div>" } ], "returns" : { "type" : "integer?", "description" : "The number of items in $array, or the empty sequence if $array is empty." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 2, "name" : "trim", "qname" : "jn:trim", "signature" : "($items as item()*, $keys as string*) as item()* external", "description" : " For each item in the given sequence, this function returns the item itself,\n if it is not an object, or its \"trimmed\" copy, if it is an object. Trimming\n an object by a set of keys means creating a new object containing all the\n pairs of the source object except the ones whose key appears in the given\n set of keys.\n", "summary" : "<p> For each item in the given sequence, this function returns the item itself,\n if it is not an object, or its \"trimmed\" copy, if it is an object.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "items", "type" : "item()", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> A sequence of items.</div>" }, { "name" : "keys", "type" : "string", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The keys of the pairs to exclude from each object in $items.</div>" } ], "returns" : { "type" : "item()*", "description" : "The trimmed version of the input sequence." }, "errors" : [  ] } ], "variables" : [  ] }, "http://zorba.io/modules/xqdoc" : { "ns" : "http://zorba.io/modules/xqdoc", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">\n The goal of xqDoc is to provide a simple vendor neutral solution for\n documenting XQuery modules, as well as tools to generate a user friendly\n presentation of this documentation and cross referencing information.\n Therefore, xqDoc proposes a new commenting convention that extends the\n currently defined XQuery comment style. This convention is modeled\n after Java's Javadoc commenting style, and provides a simple, uniform\n way to document XQuery source code. You can find more information about\n xqDoc on the website of the <a href=\"http://xqdoc.org/\">xqDoc project</a>.\n This library module provides XQDoc utility functions.\n </p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">\n Generating a user friendly presentation of the documentation is\n accomplished in the following steps:\n <ol>\n  <li>Module, variable, function, collection, and index declarations need\n      to be commented using the xqDoc commenting conventions. For example,\n      this module contains xqDoc-style comments</li>\n  <li>A xqDoc-enabled processor can parse such documentation and generate\n      a vendor neutral XML document which stores all the information about\n      the code and the comments. Such a document adheres to the xqDoc\n      Schema.</li>\n  <li>The information of an XML document generated by the second step,\n      can be transformed into arbitrary presentation formats\n      (e.g. html).</li>\n </ol>\n </p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">\n This module implements the first and second step of this process.\n That is, Zorba can parse XQuery modules which are annotated with\n xqDoc-style documentation and generate the vendor neutral\n XML representation.\n </p>\n", "sees" : [ "<xqdoc:see xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"><a href=\"http://xqdoc.org/\" target=\"_blank\">xqDoc specification</a></xqdoc:see>", "<xqdoc:see xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"><a href=\"http://www.zorba-xquery.com/tutorials/xqdoc.html\" target=\"_blank\">xqDoc tutorial with Zorba</a></xqdoc:see>" ], "authors" : [ "<xqdoc:author xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Gabriel Petrovay</xqdoc:author>" ], "version" : null, "encoding" : "utf-8", "namespaces" : [ { "uri" : "http://zorba.io/annotations", "prefix" : "an" }, { "uri" : "http://www.w3.org/2005/xqt-errors", "prefix" : "err" }, { "uri" : "http://zorba.io/modules/fetch", "prefix" : "fetch" }, { "uri" : "http://zorba.io/modules/xqdoc-options", "prefix" : "opt" }, { "uri" : "http://zorba.io/modules/schema", "prefix" : "schema" }, { "uri" : "http://zorba.io/options/versioning", "prefix" : "ver" }, { "uri" : "http://zorba.io/modules/xqdoc", "prefix" : "xqd" }, { "uri" : "http://zorba.io/errors", "prefix" : "zerr" } ], "functions" : [ { "isDocumented" : true, "arity" : 1, "name" : "xqdoc-content", "qname" : "xqd:xqdoc-content", "signature" : "($module as xs:string) as element(*)", "description" : " Generated the an XQDoc XML document for the module provided\n as parameter to this function.\n", "summary" : "<p> Generated the an XQDoc XML document for the module provided\n as parameter to this function.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "module", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The module (as string) for which to generate the XQDoc documentation.</div>" } ], "returns" : { "type" : "element(*)", "description" : "An element according to the xqdoc schema (<tt xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">http://zorba.io/modules/xqdoc.xsd</tt>)." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">zerr::ZXQD0002 if the xqdoc comments in the module contain invalid XML</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 2, "name" : "xqdoc-content", "qname" : "xqd:xqdoc-content", "signature" : "($module as xs:string, $options as element(opt:enable)) as element(*)", "description" : " Generated the an XQDoc XML document for the module provided\n as parameter to this function.\n In comparison to the single parameter version, this function does not\n generate XQDoc for all language components. By default, the\n following components are deactivated: XQuery comments, import\n statements, variable declarations, function declarations, collection\n declarations,  and index declarations. The second parameter is used to\n enable the XQDoc generation of those components.\n", "summary" : "<p> Generated the an XQDoc XML document for the module provided\n as parameter to this function.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "module", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The module (as string) for which to generate the XQDoc documentation.</div>" }, { "name" : "options", "type" : "element(opt:enable)", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> XQDoc generation options, e.g.: <pre> &lt;enable xmlns=\"http://zorba.io/modules/xqdoc-options\" comments=\"true\" functions=\"true\" indexes=\"true\" &gt; </pre></div>" } ], "returns" : { "type" : "element(*)", "description" : "An element according to the xqdoc schema (<tt xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">http://zorba.io/modules/xqdoc.xsd</tt>)." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">zerr::ZXQD0002 if the xqdoc comments in the module contain invalid XML</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 1, "name" : "xqdoc", "qname" : "xqd:xqdoc", "signature" : "($module-uri as xs:string) as element(*)", "description" : " Generates an XQDoc XML document for the module located\n at the URI provided as parameter to this function.\n", "summary" : "<p> Generates an XQDoc XML document for the module located\n at the URI provided as parameter to this function.</p>", "annotation_str" : " %an:nondeterministic", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "nondeterministic", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "module-uri", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The URL of the module for which to generate XQDoc.</div>" } ], "returns" : { "type" : "element(*)", "description" : "An element according to the xqdoc schema (<tt xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">http://zorba.io/modules/xqdoc.xsd</tt>)." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">zerr::ZXQD0002 if the xqdoc comments in the module contain invalid XML</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 2, "name" : "xqdoc", "qname" : "xqd:xqdoc", "signature" : "($module-uri as xs:string, $options as element(opt:enable)) as element(*)", "description" : " Generates an XQDoc XML document for the module located\n at the URI provided as parameter to this function.\n In comparison to the single parameter version, this function does not\n generate XQDoc for all language components. By default, the\n following components are deactivated: XQuery comments, import\n statements, variable declarations, function declarations, collection\n declarations,  and index declarations. The second parameter is used to\n enable the XQDoc generation of those components.\n", "summary" : "<p> Generates an XQDoc XML document for the module located\n at the URI provided as parameter to this function.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "module-uri", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The URL of the module for which to generate XQDoc.</div>" }, { "name" : "options", "type" : "element(opt:enable)", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> XQDoc generation options, e.g.: <pre> &lt;enable xmlns=\"http://zorba.io/modules/xqdoc-options\" comments=\"true\" functions=\"true\" indexes=\"true\" /&gt; </pre></div>" } ], "returns" : { "type" : "element(*)", "description" : "An element according to the xqdoc schema (<tt xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">http://zorba.io/modules/xqdoc.xsd</tt>)." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">zerr::ZXQD0002 if the xqdoc comments in the module contain invalid XML</xqdoc:error>" ] } ], "variables" : [  ] }, "http://zorba.io/modules/data-cleaning/character-based-string-similarity" : { "ns" : "http://zorba.io/modules/data-cleaning/character-based-string-similarity", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">This library module provides character-based string similarity functions\n that view strings as sequences of characters, generally computing a similarity score\n that corresponds to the cost of transforming one string into another.\n These functions are particularly useful for matching near duplicate strings\n in the presence of typographical errors. </p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">The logic contained in this module is not specific to any particular XQuery implementation.</p>\n", "sees" : [  ], "authors" : [ "<xqdoc:author xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Bruno Martins and Diogo Simões</xqdoc:author>" ], "version" : null, "encoding" : "utf-8", "namespaces" : [ { "uri" : "http://zorba.io/modules/data-cleaning/character-based-string-similarity", "prefix" : "simc" }, { "uri" : "http://zorba.io/options/versioning", "prefix" : "ver" } ], "functions" : [ { "isDocumented" : true, "arity" : 2, "name" : "edit-distance", "qname" : "simc:edit-distance", "signature" : "($s1 as xs:string, $s2 as xs:string) as xs:integer", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Returns the edit distance between two strings.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">This distance, also refered to as the Levenshtein distance, is defined as the minimum number\n of edits needed to transform one string into the other, with the allowable edit operations\n being insertion, deletion, or substitution of a single character.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Example usage : <pre class=\"ace-static\" ace-mode=\"xquery\">edit-distance(\"FLWOR\", \"FLOWER\")</pre></p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">The function invocation in the example above returns : <pre class=\"ace-static\" ace-mode=\"xquery\">2</pre></p>\n", "summary" : "<p>  Returns the edit distance between two strings.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "s1", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The first string.</div>" }, { "name" : "s2", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The second string.</div>" } ], "returns" : { "type" : "xs:integer", "description" : "The edit distance between the two strings." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 4, "name" : "jaro-winkler", "qname" : "simc:jaro-winkler", "signature" : "($s1 as xs:string, $s2 as xs:string, $prefix as xs:integer, $fact as xs:double) as xs:double", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Returns the Jaro-Winkler similarity coefficient between two strings.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">This similarity coefficient corresponds to an extension of the Jaro similarity coefficient that weights or\n penalizes strings based on their similarity at the beginning of the string, up to a given prefix size.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Example usage : <pre class=\"ace-static\" ace-mode=\"xquery\">jaro-winkler(\"DWAYNE\", \"DUANE\", 4, 0.1 )</pre></p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">The function invocation in the example above returns : <pre class=\"ace-static\" ace-mode=\"xquery\">0.8577777777777778</pre></p>\n", "summary" : "<p>  Returns the Jaro-Winkler similarity coefficient between two strings.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "s1", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The first string.</div>" }, { "name" : "s2", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The second string.</div>" }, { "name" : "prefix", "type" : "xs:integer", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The number of characters to consider when testing for equal prefixes in the strings.</div>" }, { "name" : "fact", "type" : "xs:double", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The weighting factor to consider when the input strings have equal prefixes.</div>" } ], "returns" : { "type" : "xs:double", "description" : "The Jaro-Winkler similarity coefficient between the two strings." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 2, "name" : "jaro", "qname" : "simc:jaro", "signature" : "($s1 as xs:string, $s2 as xs:string) as xs:double", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Returns the Jaro similarity coefficient between two strings.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">This similarity coefficient is based on the number of transposed characters and on a\n weighted sum of the percentage of matched characters held within the strings. The higher\n the Jaro-Winkler value is, the more similar the strings are. The coefficient is\n normalized such that 0 equates to no similarity and 1 is an exact match.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Example usage : <pre class=\"ace-static\" ace-mode=\"xquery\">jaro(\"FLWOR Found.\", \"FLWOR Foundation\")</pre></p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">The function invocation in the example above returns : <pre class=\"ace-static\" ace-mode=\"xquery\">0.5853174603174603</pre></p>\n", "summary" : "<p>  Returns the Jaro similarity coefficient between two strings.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "s1", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The first string.</div>" }, { "name" : "s2", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The second string.</div>" } ], "returns" : { "type" : "xs:double", "description" : "The Jaro similarity coefficient between the two strings." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 4, "name" : "needleman-wunsch", "qname" : "simc:needleman-wunsch", "signature" : "($s1 as xs:string, $s2 as xs:string, $score as xs:integer, $penalty as xs:integer) as xs:double", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Returns the Needleman-Wunsch distance between two strings.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">The Needleman-Wunsch distance is similar to the basic edit distance metric, adding a\n variable cost adjustment to the cost of a gap (i.e., an insertion or deletion) in the\n distance metric.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Example usage : <pre class=\"ace-static\" ace-mode=\"xquery\">needleman-wunsch(\"KAK\", \"KQRK\", 1, 1)</pre></p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">The function invocation in the example above returns : <pre class=\"ace-static\" ace-mode=\"xquery\">0</pre></p>\n", "summary" : "<p>  Returns the Needleman-Wunsch distance between two strings.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "s1", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The first string.</div>" }, { "name" : "s2", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The second string.</div>" }, { "name" : "score", "type" : "xs:integer", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The score value.</div>" }, { "name" : "penalty", "type" : "xs:integer", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The penalty value.</div>" } ], "returns" : { "type" : "xs:double", "description" : "The Needleman-Wunsch distance between the two strings." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 4, "name" : "smith-waterman", "qname" : "simc:smith-waterman", "signature" : "($s1 as xs:string, $s2 as xs:string, $score as xs:integer, $penalty as xs:integer) as xs:double", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Returns the Smith-Waterman distance between two strings.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Example usage : <pre class=\"ace-static\" ace-mode=\"xquery\">smith-waterman(\"ACACACTA\", \"AGCACACA\", 2, 1)</pre></p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">The function invocation in the example above returns : <pre class=\"ace-static\" ace-mode=\"xquery\">12</pre></p>\n", "summary" : "<p>  Returns the Smith-Waterman distance between two strings.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "s1", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The first string.</div>" }, { "name" : "s2", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The second string.</div>" }, { "name" : "score", "type" : "xs:integer", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The score value.</div>" }, { "name" : "penalty", "type" : "xs:integer", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The penalty value.</div>" } ], "returns" : { "type" : "xs:double", "description" : "The Smith-Waterman distance between the two strings." }, "errors" : [  ] } ], "variables" : [  ] }, "http://expath.org/ns/error" : { "ns" : "http://expath.org/ns/error", "description" : " This module defines all errors for the http-client module.\n", "sees" : [  ], "authors" : [ "<xqdoc:author xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Markus Pilman</xqdoc:author>" ], "version" : null, "encoding" : "utf-8", "namespaces" : [ { "uri" : "http://expath.org/ns/error", "prefix" : "err" } ], "functions" : [  ], "variables" : [ { "name" : "err:errNS", "type" : "xs:string", "description" : " Error namespace URI.\n" }, { "name" : "err:HC001", "type" : "xs:QName", "description" : " An HTTP error occurred.\n" }, { "name" : "err:HC002", "type" : "xs:QName", "description" : " Error parsing the entity content as XML or HTML.\n" }, { "name" : "err:HC003", "type" : "xs:QName", "description" : " With a multipart response, the override-media-type must be either a multipart media type or application/octet-stream.\n" }, { "name" : "err:HC004", "type" : "xs:QName", "description" : " The src attribute on the body element is mutually exclusive with all other attribute (except the media-type).\n" }, { "name" : "err:HC005", "type" : "xs:QName", "description" : " The request element is not valid.\n" }, { "name" : "err:HC006", "type" : "xs:QName", "description" : " A timeout occurred waiting for the response.\n" }, { "name" : "err:HCV01", "type" : "xs:QName", "description" : " This error is not defined in the specification but used by the Zorba implementation.\n It gets thrown when the user gives wrong arguments which are statically correct,\n but make no sense (for example: http:send-request((), ())).\n" }, { "name" : "err:HCV02", "type" : "xs:QName", "description" : " Zorba specific error\n This error is raised if trying to follow a redirect for a POST, PUT, or\n DELETE request\n" } ] }, "http://www.28msec.com/modules/s3" : { "ns" : "http://www.28msec.com/modules/s3", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">This module provides functionality for managing S3 buckets and object.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">It is not possible to access buckets whose names are not DNS-compliant. For\n instance a bucket name which contains uppercase letters or is longer than 63\n characters is not DNS-compliant. For additional details on bucket naming\n conventions refer to\n <a href=\"http://docs.aws.amazon.com/AmazonS3/latest/dev/BucketRestrictions.html\">\n Bucket Restrictions and Limitations</a>.</p>\n <h2 xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Authentication</h2>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">For each functionality two methods are provided:\n <ul>\n   <li>one which allows to specify the credentials to use, by means of the\n   optional $credentials parameter (named). If the parameter is not specified,\n   the default credentials in the \"S3\" category will be used. If the parameter\n   is specified it must be either a string or an object. If it is a string it\n   will be interpreted as the name of a credentials in the \"S3\" category. If\n   it is an object it must have the following structure:\n   <ul>\n     <li>accessKey: the AWS access key to use (string, mandatory).</li>\n     <li>secretKey: the AWS secret key to use (string, mandatory).</li>\n     <li>useHttps: whether to use secure HTTPS connections or not.\n     (boolean, optional). The default is <code>false</code></li>\n     <li>defaultBucket: the default bucket name (string, optional).</li>\n   </ul>\n   </li>\n   <li>one which does not allow to specify which credentials to use.\n   In this case the default credentials in the \"S3\" category will be used.</li>\n </ul>\n </p>\n <h2 xmlns:xqdoc=\"http://www.xqdoc.org/1.0\" id=\"determinism\">Important Notice Regarding Function Determinism</h2>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">The non side-effecting functions:\n <ul>\n   <li><a href=\"?anchor=list-buckets-0\">list-buckets#0</a></li>\n   <li><a href=\"?anchor=list-buckets-1\">list-buckets#1</a></li>\n   <li><a href=\"?anchor=list-bucket-0\">list-bucket#0</a></li>\n   <li><a href=\"?anchor=list-bucket-1\">list-bucket#1</a></li>\n   <li><a href=\"?anchor=list-bucket-2\">list-bucket#2</a></li>\n   <li><a href=\"?anchor=list-bucket-3\">list-bucket#3</a></li>\n   <li><a href=\"?anchor=list-bucket-versions-0\">list-bucket-versions#0</a></li>\n   <li><a href=\"?anchor=list-bucket-versions-1\">list-bucket-versions#1</a></li>\n   <li><a href=\"?anchor=list-bucket-versions-2\">list-bucket-versions#2</a></li>\n   <li><a href=\"?anchor=list-bucket-versions-3\">list-bucket-versions#3</a></li>\n   <li><a href=\"?anchor=read-text-1\">read-text#1</a></li>\n   <li><a href=\"?anchor=read-text-2\">read-text#2</a></li>\n   <li><a href=\"?anchor=read-binary-1\">read-binary#1</a></li>\n   <li><a href=\"?anchor=read-binary-2\">read-binary#2</a></li>\n   <li><a href=\"?anchor=read-object-1\">read-object#1</a></li>\n   <li><a href=\"?anchor=read-object-2\">read-object#2</a></li>\n   <li><a href=\"?anchor=object-metadata-1\">object-metadata#1</a></li>\n   <li><a href=\"?anchor=object-metadata-2\">object-metadata#2</a></li>\n   <li><a href=\"?anchor=object-torrent-1\">object-torrent#1</a></li>\n   <li><a href=\"?anchor=object-torrent-2\">object-torrent#2</a></li>\n   <li><a href=\"?anchor=object-permissions-1\">object-permissions#1</a></li>\n   <li><a href=\"?anchor=object-permissions-2\">object-permissions#2</a></li>\n </ul>\n are declared deterministic, which means that their results could be cached\n when invoked multiple times with the same arguments in the same query execution.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">To not use cached results you can use the following alternative functions:\n <ul>\n   <li><a href=\"?anchor=list-buckets-nondeterministic-0\">list-buckets-nondeterministic#0</a></li>\n   <li><a href=\"?anchor=list-buckets-nondeterministic-1\">list-buckets-nondeterministic#1</a></li>\n   <li><a href=\"?anchor=list-bucket-nondeterministic-0\">list-bucket-nondeterministic#0</a></li>\n   <li><a href=\"?anchor=list-bucket-nondeterministic-1\">list-bucket-nondeterministic#1</a></li>\n   <li><a href=\"?anchor=list-bucket-nondeterministic-2\">list-bucket-nondeterministic#2</a></li>\n   <li><a href=\"?anchor=list-bucket-nondeterministic-3\">list-bucket-nondeterministic#3</a></li>\n   <li><a href=\"?anchor=list-bucket-versions-nondeterministic-0\">list-bucket-versions-nondeterministic#0</a></li>\n   <li><a href=\"?anchor=list-bucket-versions-nondeterministic-1\">list-bucket-versions-nondeterministic#1</a></li>\n   <li><a href=\"?anchor=list-bucket-versions-nondeterministic-2\">list-bucket-versions-nondeterministic#2</a></li>\n   <li><a href=\"?anchor=list-bucket-versions-nondeterministic-3\">list-bucket-versions-nondeterministic#3</a></li>\n   <li><a href=\"?anchor=read-text-nondeterministic-1\">read-text-nondeterministic#1</a></li>\n   <li><a href=\"?anchor=read-text-nondeterministic-2\">read-text-nondeterministic#2</a></li>\n   <li><a href=\"?anchor=read-binary-nondeterministic-1\">read-binary-nondeterministic#1</a></li>\n   <li><a href=\"?anchor=read-binary-nondeterministic-2\">read-binary-nondeterministic#2</a></li>\n   <li><a href=\"?anchor=read-object-nondeterministic-1\">read-object-nondeterministic#1</a></li>\n   <li><a href=\"?anchor=read-object-nondeterministic-2\">read-object-nondeterministic#2</a></li>\n   <li><a href=\"?anchor=object-metadata-nondeterministic-1\">object-metadata-nondeterministic#1</a></li>\n   <li><a href=\"?anchor=object-metadata-nondeterministic-2\">object-metadata-nondeterministic#2</a></li>\n   <li><a href=\"?anchor=object-torrent-nondeterministic-1\">object-torrent-nondeterministic#1</a></li>\n   <li><a href=\"?anchor=object-torrent-nondeterministic-2\">object-torrent-nondeterministic#2</a></li>\n   <li><a href=\"?anchor=object-permissions-nondeterministic-1\">object-permissions-nondeterministic#1</a></li>\n   <li><a href=\"?anchor=object-permissions-nondeterministic-2\">object-permissions-nondeterministic#2</a></li>\n </ul>\n which have been declared as being non deterministic.</p>\n", "sees" : [  ], "authors" : [ "<xqdoc:author xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Alexander Kreutz</xqdoc:author>", "<xqdoc:author xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Federico Cavalieri</xqdoc:author>" ], "version" : null, "encoding" : "utf-8", "namespaces" : [ { "uri" : "http://zorba.io/annotations", "prefix" : "an" }, { "uri" : "http://www.28msec.com/modules/credentials", "prefix" : "credentials" }, { "uri" : "http://www.w3.org/2005/xqt-errors", "prefix" : "err" }, { "uri" : "http://www.zorba-xquery.com/modules/cryptography/hmac", "prefix" : "hmac" }, { "uri" : "http://zorba.io/modules/http-client", "prefix" : "http" }, { "uri" : "http://jsoniq.org/errors", "prefix" : "jerr" }, { "uri" : "http://www.28msec.com/modules/s3", "prefix" : "s3" }, { "uri" : "http://s3.amazonaws.com/doc/2006-03-01/", "prefix" : "s3s" }, { "uri" : "http://www.28msec.com/modules/sleep", "prefix" : "sleep" }, { "uri" : "http://zorba.io/options/versioning", "prefix" : "ver" } ], "functions" : [ { "isDocumented" : true, "arity" : 2, "name" : "copy-object", "qname" : "s3:copy-object", "signature" : "($s3-object-source as item(), $s3-object-target as item()) as item()*", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Copies an object already stored on s3 into a target bucket using the\n default credentials. If the target object already exists it will be overwritten.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">The source object metadata is copied to the target object, unless new\n metadata is specified. In this case the target object will possess only the\n specified metadata.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Unless the permissions for the target are specified, the target object will\n have the \"private\" ACL, that is, the owner gets FULL_CONTROL, and no one else\n has access rights.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">The source S3 object is specified through the $s3-object-source parameter.\n Either a string or a JSON object can be used. If a string is specified, it is\n interpreted as key for an object in the default bucket of the default\n credentials. If no default bucket is present in the default credentials, the\n <code>s3:BUCKET</code> error is raised. If a JSON object is used, it must have\n the following structure:\n <ul>\n   <li>key: the object key (string, mandatory).</li>\n   <li>bucket: the bucket name (string, optional). If this field is not present\n   and the default credentials do not have a default bucket, the\n   <code>s3:BUCKET</code> error is raised.</li>\n   <li>version: the object version (string, optional). If versioning is enabled\n   for the specified object and no version is specified the latest version of\n   the object will be copied. If specified, only that particular version of the\n   object will be copied.</li>\n </ul>\n </p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">The target S3 object is specified through the $s3-object-target parameter.\n Either a string or a JSON object can be used. If a string is specified and the\n default credentials contain a default bucket, the object will be copied in\n the credentials default bucket. Otherwise, the object will be copied in the\n source object bucket. If a JSON object is used, it must have the following\n structure:\n <ul>\n   <li>key: the object key (string, mandatory).</li>\n   <li>bucket: the bucket name (string, optional). If this field is not present\n   and the default credentials contain a default bucket, the object will be\n   copied in the credentials default bucket. Otherwise, the object will be\n   copied in the source object bucket.</li>\n   <li>metadata: an object specifying additional metadata. Each metadata is a\n   name-value pair and is represented as different fields (object, optional).\n   If not specified, the source object metadata will be copied to the target\n   object. Otherwise the target object metadata will be the specified one.</li>\n   <li>reducedRedundancy: whether to use reduced-redundancy or not (bool,\n   optional). Default is false.</li>\n   <li>permission: the permission to set (string, optional). It must be one of\n   \"private\", \"public-read\", \"public-read-write\", \"authenticated-read\",\n   \"bucket-owner-read\", \"bucket-owner-full-control\". For your convenience the\n   following variables can be used: <code>$s3:ACL-GRANT-PRIVATE</code>,\n   <code>$s3:ACL-GRANT-PUBLIC-READ</code>, <code>$s3:ACL-GRANT-PUBLIC-READ-WRITE</code>,\n   <code>$s3:ACL-GRANT-AUTHENTICATED-READ</code>, <code>$s3:ACL-GRANT-BUCKET-OWNER-READ</code>,\n   <code>$s3:ACL-GRANT-BUCKET-OWNER-FULL-CONTROL</code>. If this field is not\n   specified, the \"private\" ACL is used, that is, the owner gets FULL_CONTROL,\n   and no one else has access rights.</li>\n </ul>\n </p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Example:\n <pre>\n let $s3-object-source :=\n {\n   \"key\" : \"test.xml\",\n   \"bucket\": \"28msec\"\n }\n let $s3-object-target :=\n {\n   \"key\" : \"test-copy.xml\",\n   \"bucket\": \"28msec\",\n   \"permisstion\": $s3:ACL-GRANT-PUBLIC-READ\n   \"metadata\":\n   {\n     \"author\": \"28msec\"\n   }\n }\n return s3:copy-object($s3-object-source, $s3-object-target)\n </pre>\n </p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Example return object:\n <pre>\n {\n   \"sourceVersion\": \"3/L4kqtJlcpXroDTDmJ+rmSpXd3dIbrHY+MTRCxf3vjVBH40Nr8X8gdRQBpUMLUo\",\n   \"version\": \"QUpfdndhfd8438MNFDN93jdnJFkdmqnh893\",\n   \"lastModified\": \"2009-10-28T22:32:00\",\n   \"eTag\": \"\\\"9b2cf535f27731c974343645a3985328\\\"\",\n   \"expiration\": \"expiry-date=\\\"Fri, 21 Dec 2012 00:00:00 GMT\\\", rule-id=\\\"Rule for testfile.txt\\\"\"\n }\n </pre>\n </p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">The fields in the returned object have the following meanings:\n <ul>\n   <li>sourceVersion: the version of the copied object. When you enable versioning,\n   Amazon S3 generates a random number for objects added to a bucket. The value\n   is UTF-8 encoded and URL ready. When the soruce object is copied in a bucket where\n   versioning has been suspended, the version ID is always null. (string, optional)</li>\n   <li>version: the version of the copied object. When you enable versioning,\n   Amazon S3 generates a random number for objects added to a bucket. The value\n   is UTF-8 encoded and URL ready. When the soruce object is copied in a bucket where\n   versioning has been suspended, the version ID is always null. (string, optional)</li>\n   <li>lastModified: the last modification date of the target object. (dateTime, optional)</li>\n   <li>eTag: the entity tag is a hash of the copeid object. The eTag only reflects\n   changes to the contents of an object, not its metadata. The eTag is determined\n   when an object is created. The eTag returned by this method is a quoted,\n   32-digit hexadecimal string representing the MD5 digest of the object data.\n   For other objects, the eTag may or may not be an MD5 digest of the object data.\n   (string, optional).</li>\n   <li>expiration: if the object expiration is configured (see\n   <a href=\"http://docs.aws.amazon.com/AmazonS3/latest/API/RESTBucketPUTlifecycle.html\">PUT Bucket lifecycle</a>),\n   this field is present. It includes the expiry-date and rule-id key value\n   pairs providing object expiration information. The value of rule-id is URL\n   encoded.(string, optional).</li>\n </ul>\n </p>\n", "summary" : "<p>  Copies an object already stored on s3 into a target bucket using the\n default credentials.</p>", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "s3-object-source", "type" : "item()", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The S3 object or object version to copy from.</div>" }, { "name" : "s3-object-target", "type" : "item()", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The S3 object to write to.</div>" } ], "returns" : { "type" : "item()*", "description" : "the S3 copy result." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">s3:CREDENTIALS if the default S3 credentials cannot be found or are invalid</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">s3:OBJECT if the object specification is invalid</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">s3:BUCKET if no bucket has been specified and no default bucket is present in the used credentials</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">s3:HTTP an HTTP error occurred sending the request to S3</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">s3:REQUEST S3 refused to execute the request</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">s3:RESPONSE the response received from S3 cannot be parsed</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 3, "name" : "copy-object", "qname" : "s3:copy-object", "signature" : "($credentials as item()?, $s3-object-source as item(), $s3-object-target as item()) as object()", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Copies an object already stored on S3 into a target bucket using the\n specified credentials. If the target object already exists it will be overwritten.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">The source object metadata is copied to the target object, unless new\n metadata is specified. In this case the target object will possess only the\n specified metadata.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Unless the permissions for the target are specified, the target object will\n have the \"private\" ACL, that is, the owner gets FULL_CONTROL, and no one else\n has access rights.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">The source S3 object is specified through the $s3-object-source parameter.\n Either a string or a JSON object can be used. If a string is specified, it is\n interpreted as key for an object in the default bucket of the specified\n credentials. If no default bucket is present in the specified credentials, the\n <code>s3:BUCKET</code> error is raised. If a JSON object is used, it must have\n the following structure:\n <ul>\n   <li>key: the object key (string, mandatory).</li>\n   <li>bucket: the bucket name (string, optional). If this field is not present\n   and the specified credentials do not have a default bucket, the\n   <code>s3:BUCKET</code> error is raised.</li>\n   <li>version: the object version (string, optional). If versioning is enabled\n   for the specified object and no version is specified the latest version of\n   the object will be copied. If specified, only that particular version of the\n   object will be copied.</li>\n </ul>\n </p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">The target S3 object is specified through the $s3-object-target parameter.\n Either a string or a JSON object can be used. If a string is specified and the\n specified credentials contain a default bucket, the object will be copied in\n the credentials default bucket. Otherwise, the object will be copied in the\n source object bucket. If a JSON object is used, it must have the following\n structure:\n <ul>\n   <li>key: the object key (string, mandatory).</li>\n   <li>bucket: the bucket name (string, optional). If this field is not present\n   and the specified credentials contain a default bucket, the object will be\n   copied in the credentials default bucket. Otherwise, the object will be\n   copied in the source object bucket.</li>\n   <li>metadata: an object specifying additional metadata. Each metadata is a\n   name-value pair and is represented as different fields (object, optional).\n   If not specified, the source object metadata will be copied to the target\n   object. Otherwise the target object metadata will be the specified one.</li>\n   <li>reducedRedundancy: whether to use reduced-redundancy or not (bool,\n   optional). Default is false.</li>\n   <li>permission: the permission to set (string, optional). It must be one of\n   \"private\", \"public-read\", \"public-read-write\", \"authenticated-read\",\n   \"bucket-owner-read\", \"bucket-owner-full-control\". For your convenience the\n   following variables can be used: <code>$s3:ACL-GRANT-PRIVATE</code>,\n   <code>$s3:ACL-GRANT-PUBLIC-READ</code>, <code>$s3:ACL-GRANT-PUBLIC-READ-WRITE</code>,\n   <code>$s3:ACL-GRANT-AUTHENTICATED-READ</code>, <code>$s3:ACL-GRANT-BUCKET-OWNER-READ</code>,\n   <code>$s3:ACL-GRANT-BUCKET-OWNER-FULL-CONTROL</code>. If this field is not\n   specified, the \"private\" ACL is used, that is, the owner gets FULL_CONTROL,\n   and no one else has access rights.</li>\n </ul>\n </p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Example:\n <pre>\n let $s3-object-source :=\n {\n   \"key\" : \"test.xml\",\n   \"bucket\": \"28msec\"\n }\n let $s3-object-target :=\n {\n   \"key\" : \"test-copy.xml\",\n   \"bucket\": \"28msec\",\n   \"permisstion\": $s3:ACL-GRANT-PUBLIC-READ\n   \"metadata\":\n   {\n     \"author\": \"28msec\"\n   }\n }\n return s3:copy-object(\"credentials\", $s3-object-source, $s3-object-target)\n </pre>\n </p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Example return object:\n <pre>\n {\n   \"sourceVersion\": \"3/L4kqtJlcpXroDTDmJ+rmSpXd3dIbrHY+MTRCxf3vjVBH40Nr8X8gdRQBpUMLUo\",\n   \"version\": \"QUpfdndhfd8438MNFDN93jdnJFkdmqnh893\",\n   \"lastModified\": \"2009-10-28T22:32:00\",\n   \"eTag\": \"\\\"9b2cf535f27731c974343645a3985328\\\"\",\n   \"expiration\": \"expiry-date=\\\"Fri, 21 Dec 2012 00:00:00 GMT\\\", rule-id=\\\"Rule for testfile.txt\\\"\"\n }\n </pre>\n </p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">The fields in the returned object have the following meanings:\n <ul>\n   <li>sourceVersion: the version of the copied object. When you enable versioning,\n   Amazon S3 generates a random number for objects added to a bucket. The value\n   is UTF-8 encoded and URL ready. When the soruce object is copied in a bucket where\n   versioning has been suspended, the version ID is always null. (string, optional)</li>\n   <li>version: the version of the copied object. When you enable versioning,\n   Amazon S3 generates a random number for objects added to a bucket. The value\n   is UTF-8 encoded and URL ready. When the soruce object is copied in a bucket where\n   versioning has been suspended, the version ID is always null. (string, optional)</li>\n   <li>lastModified: the last modification date of the target object. (dateTime, optional)</li>\n   <li>eTag: the entity tag is a hash of the copeid object. The eTag only reflects\n   changes to the contents of an object, not its metadata. The eTag is determined\n   when an object is created. The eTag returned by this method is a quoted,\n   32-digit hexadecimal string representing the MD5 digest of the object data.\n   For other objects, the eTag may or may not be an MD5 digest of the object data.\n   (string, optional).</li>\n </ul>\n </p>\n", "summary" : "<p>  Copies an object already stored on S3 into a target bucket using the\n specified credentials.</p>", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "credentials", "type" : "item()", "occurrence" : "?", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The credentials to use.</div>" }, { "name" : "s3-object-source", "type" : "item()", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The S3 object or object version to copy from.</div>" }, { "name" : "s3-object-target", "type" : "item()", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The S3 object to write to.</div>" } ], "returns" : { "type" : "object()", "description" : "the S3 copy result." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">s3:CREDENTIALS if the specified S3 credentials cannot be found or are invalid</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">s3:OBJECT if the object specification is invalid</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">s3:BUCKET if no bucket has been specified and no default bucket is present in the used credentials</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">s3:HTTP an HTTP error occurred sending the request to S3</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">s3:REQUEST S3 refused to execute the request</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">s3:RESPONSE the response received from S3 cannot be parsed</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 1, "name" : "create-bucket", "qname" : "s3:create-bucket", "signature" : "($bucket as string) as empty-sequence()", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Creates a bucket using the default S3 credentials. The bucket owner will\n be the account of the default S3 credentials.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">The bucket is created with the S3 default bucket settings. Specifically,\n it is created in the US Standard region and the default credentials\n account becomes its owner. The bucket ACL permissions are set to \"private\",\n that is, the owner gets FULL_CONTROL, whereas no one else has access rights.\n The bucket name must comply with the following rules:\n <ul>\n   <li>bucket names must be no more than 255 characters long.</li>\n   <li>bucket names must be a combination of uppercase letters, lowercase letters,\n   numbers, periods (.), dashes (-) and underscores (_).</li>\n </ul>\n For more details, refer to the\n <a href=\"http://docs.aws.amazon.com/AmazonS3/latest/dev/BucketRestrictions.html\">\n Amazon S3 Bucket Restrictions</a> page.\n </p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">If the specified bucket already exists in the US Standard region and has\n the same owner, its permissions settings are reset and its contents are\n preserved. Otherwise, if the specified bucket already exists an s3:REQUEST\n error is raised.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Example:\n <pre>\n s3:create-bucket(\"28msec\");\n </pre>\n </p>\n", "summary" : "<p>  Creates a bucket using the default S3 credentials.</p>", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "bucket", "type" : "string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The bucket name.</div>" } ], "returns" : { "type" : "empty-sequence()", "description" : "the empty sequence" }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">s3:CREDENTIALS if the default S3 credentials cannot be found or are invalid</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">s3:HTTP an HTTP error occurred sending the request to S3</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">s3:REQUEST S3 refused to execute the request</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 2, "name" : "create-bucket", "qname" : "s3:create-bucket", "signature" : "($credentials as item()?, $bucket as string) as empty-sequence()", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Creates a bucket using the specified S3 credentials. The bucket owner will\n be the account of the specified S3 credentials.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">The bucket is created with the S3 default bucket settings. Specifically,\n it is created in the US Standard region and the specified credentials\n account becomes its owner. The bucket ACL permissions are set to \"private\",\n that is, the owner gets FULL_CONTROL, whereas no one else has access rights.\n The bucket name must comply with the following rules:\n <ul>\n   <li>bucket names must be no more than 255 characters long.</li>\n   <li>bucket names must be a combination of uppercase letters, lowercase letters,\n   numbers, periods (.), dashes (-) and underscores (_).</li>\n </ul>\n For more details, refer to the\n <a href=\"http://docs.aws.amazon.com/AmazonS3/latest/dev/BucketRestrictions.html\">\n Amazon S3 Bucket Restrictions</a> page.\n </p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">If the specified bucket already exists in the US Standard region and has\n the same owner, its permissions settings are reset and its contents are\n preserved. Otherwise, if the specified bucket already exists an s3:REQUEST\n error is raised.</p>\n", "summary" : "<p>  Creates a bucket using the specified S3 credentials.</p>", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "credentials", "type" : "item()", "occurrence" : "?", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The credentials to use.</div>" }, { "name" : "bucket", "type" : "string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The bucket name.</div>" } ], "returns" : { "type" : "empty-sequence()", "description" : "the empty sequence" }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">s3:CREDENTIALS if the specified S3 credentials cannot be found or are invalid</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">s3:HTTP an HTTP error occurred sending the request to S3</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">s3:REQUEST S3 refused to execute the request</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 1, "name" : "delete-bucket", "qname" : "s3:delete-bucket", "signature" : "($bucket as string) as empty-sequence()", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Deletes a bucket using the default S3 credentials. The bucket must be owned\n by the account of the default S3 credentials.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">This operation will fail if the deleted bucket is not empty. All objects,\n object versions, and delete markers have to be deleted beforehand using this\n function.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Example:\n <pre>\n s3:delete-bucket(\"28msec\");\n </pre>\n </p>\n", "summary" : "<p>  Deletes a bucket using the default S3 credentials.</p>", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "bucket", "type" : "string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The bucket name.</div>" } ], "returns" : { "type" : "empty-sequence()", "description" : "the empty sequence" }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">s3:CREDENTIALS if the default S3 credentials cannot be found or are invalid</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">s3:HTTP an HTTP error occurred sending the request to S3</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">s3:REQUEST S3 refused to execute the request</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 2, "name" : "delete-bucket", "qname" : "s3:delete-bucket", "signature" : "($credentials as item()?, $bucket as string) as empty-sequence()", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Deletes a bucket using the specified S3 credentials. The bucket must be owned\n by the account of the specified S3 credentials.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">This operation will fail if the deleted bucket is not empty. All objects,\n object versions, and delete markers have to be deleted beforehand using this\n function.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Example:\n <pre>\n s3:delete-bucket(\"credentials\", \"28msec\");\n </pre>\n </p>\n", "summary" : "<p>  Deletes a bucket using the specified S3 credentials.</p>", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "credentials", "type" : "item()", "occurrence" : "?", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The credentials to use.</div>" }, { "name" : "bucket", "type" : "string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The bucket name.</div>" } ], "returns" : { "type" : "empty-sequence()", "description" : "the empty sequence" }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">s3:CREDENTIALS if the specified S3 credentials cannot be found or are invalid</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">s3:HTTP an HTTP error occurred sending the request to S3</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">s3:REQUEST S3 refused to execute the request</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 1, "name" : "delete-object", "qname" : "s3:delete-object", "signature" : "($s3-object as item()) as empty-sequence()", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Deletes an object or a specific object version from a bucket using the\n default S3 credentials.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">The S3 object is specified through the $s3-object parameter. Either a string\n or a JSON object can be used. If a string is specified, it is interpreted as\n key for an object in the default bucket of the default credentials. If no default\n bucket is present in the default credentials, the <code>s3:BUCKET</code> error\n is raised. If a JSON object is used, it must have the following structure:\n <ul>\n   <li>key: the object key (string, mandatory).</li>\n   <li>bucket: the bucket name (string, optional). If this field is not present\n   and the default credentials do not have a default bucket, the\n   <code>s3:BUCKET</code> error is raised.</li>\n   <li>version: the object version (string, optional). If not specified, the\n   null version of the object is removed, if there is one. To remove a specific\n   version, this field must be specified.</li>\n </ul>\n </p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">If the bucket is versioned a delete marker is inserted for the object. If\n mfa-deletion is enabled you will not be able to delete an object through\n this method. If the specified bucket exists, and the specified object does not\n exist no errors will be raised.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Example:\n <pre>\n s3:delete-object(\n {\n   \"key\": \"test.xml\",\n   \"bucket\": \"28msec\",\n   \"version\": \"3/L4kqtJlcpXroDTDmJ+rmSpXd3dIbrHY+MTRCxf3vjVBH40Nr8X8gdRQBpUMLUo\"\n });\n </pre>\n </p>\n", "summary" : "<p>  Deletes an object or a specific object version from a bucket using the\n default S3 credentials.</p>", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "s3-object", "type" : "item()", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The S3 object or object version to delete.</div>" } ], "returns" : { "type" : "empty-sequence()", "description" : "the empty sequence" }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">s3:CREDENTIALS if the default S3 credentials cannot be found or are invalid</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">s3:OBJECT if the object specification is invalid</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">s3:BUCKET if no bucket has been specified and no default bucket is present in the used credentials</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">s3:HTTP an HTTP error occurred sending the request to S3</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">s3:REQUEST S3 refused to execute the request</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">s3:RESPONSE the response received from S3 cannot be parsed</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 2, "name" : "delete-object", "qname" : "s3:delete-object", "signature" : "($credentials as item()?, $s3-object as item()) as empty-sequence()", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Deletes an object or a specific object version from a bucket using the\n specified S3 credentials.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">The S3 object is specified through the $s3-object parameter. Either a string\n or a JSON object can be used. If a string is specified, it is interpreted as\n key for an object in the default bucket of the specified credentials. If no default\n bucket is present in the specified credentials, the <code>s3:BUCKET</code> error\n is raised. If a JSON object is used, it must have the following structure:\n <ul>\n   <li>key: the object key (string, mandatory).</li>\n   <li>bucket: the bucket name (string, optional). If this field is not present\n   and the specified credentials do not have a default bucket, the\n   <code>s3:BUCKET</code> error is raised.</li>\n   <li>version: the object version (string, optional). If not specified, the\n   null version of the object is removed, if there is one. To remove a specific\n   version, this field must be specified.</li>\n </ul>\n </p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">If the bucket is versioned a delete marker is inserted for the object. If\n mfa-deletion is enabled you will not be able to delete an object through\n this method. If the specified bucket exists, and the specified object does not\n exist no errors will be raised.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Example:\n <pre>\n s3:delete-object(\"credentials\",\n {\n   \"key\": \"test.xml\",\n   \"bucket\": \"28msec\",\n   \"version\": \"3/L4kqtJlcpXroDTDmJ+rmSpXd3dIbrHY+MTRCxf3vjVBH40Nr8X8gdRQBpUMLUo\"\n });\n </pre>\n </p>\n", "summary" : "<p>  Deletes an object or a specific object version from a bucket using the\n specified S3 credentials.</p>", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "credentials", "type" : "item()", "occurrence" : "?", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The credentials to use.</div>" }, { "name" : "s3-object", "type" : "item()", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The S3 object or object version to delete.</div>" } ], "returns" : { "type" : "empty-sequence()", "description" : "the empty sequence" }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">s3:CREDENTIALS if the specified S3 credentials cannot be found or are invalid</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">s3:OBJECT if the object specification is invalid</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">s3:BUCKET if no bucket has been specified and no default bucket is present in the used credentials</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">s3:HTTP an HTTP error occurred sending the request to S3</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">s3:REQUEST S3 refused to execute the request</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">s3:RESPONSE the response received from S3 cannot be parsed</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 0, "name" : "list-bucket-nondeterministic", "qname" : "s3:list-bucket-nondeterministic", "signature" : "() as object()", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Lists the objects in the default bucket of the default S3 credentials.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">This function has the same semantics as\n <a href=\"#list-bucket-0\">list-bucket#0</a>,\n but is declared as being non deterministic and thus should only be used when\n result caching is not desired.</p>\n", "summary" : "<p>  Lists the objects in the default bucket of the default S3 credentials.</p>", "annotation_str" : " %an:nondeterministic", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "nondeterministic", "value" : "" } ], "updating" : false, "parameters" : [  ], "returns" : { "type" : "object()", "description" : "the list of objects in the default bucket of the default S3 credentials" }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">s3:CREDENTIALS if the specified S3 credentials cannot be found or are invalid</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">s3:BUCKET if no default bucket is present in the default S3 credentials</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">s3:OPTIONS if the specified options are invalid</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">s3:HTTP an HTTP error occurred sending the request to S3</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">s3:REQUEST S3 refused to execute the request</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">s3:RESPONSE the response received from S3 cannot be parsed</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 1, "name" : "list-bucket-nondeterministic", "qname" : "s3:list-bucket-nondeterministic", "signature" : "($bucket as string?) as object()", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Lists the objects in a bucket using the default S3 credentials.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">This function has the same semantics as\n <a href=\"#list-bucket-1\">list-bucket#1</a>,\n but is declared as being non deterministic and thus should only be used when\n result caching is not desired.</p>\n", "summary" : "<p>  Lists the objects in a bucket using the default S3 credentials.</p>", "annotation_str" : " %an:nondeterministic", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "nondeterministic", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "bucket", "type" : "string", "occurrence" : "?", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The bucket to list. If the empty sequence is given, the default bucket of the default S3 credentials will be listed.</div>" } ], "returns" : { "type" : "object()", "description" : "the list of the objects in the specified bucket." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">s3:CREDENTIALS if the specified S3 credentials cannot be found or are invalid</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">s3:BUCKET if no bucket has been specified and no default bucket is present in the used credentials</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">s3:OPTIONS if the specified options are invalid</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">s3:HTTP an HTTP error occurred sending the request to S3</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">s3:REQUEST S3 refused to execute the request</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">s3:RESPONSE the response received from S3 cannot be parsed</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 2, "name" : "list-bucket-nondeterministic", "qname" : "s3:list-bucket-nondeterministic", "signature" : "($bucket as string?, $options as object()?) as object()", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Lists the objects in a bucket using the default S3 credentials.\n This method allows the specification of additional listing options.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">This function has the same semantics as\n <a href=\"#list-bucket-2\">list-bucket#2</a>,\n but is declared as being non deterministic and thus should only be used when\n result caching is not desired.</p>\n", "summary" : "<p>  Lists the objects in a bucket using the default S3 credentials.</p>", "annotation_str" : " %an:nondeterministic", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "nondeterministic", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "bucket", "type" : "string", "occurrence" : "?", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The bucket to list. If the empty sequence is given, the default bucket of the default S3 credentials will be listed.</div>" }, { "name" : "options", "type" : "object()", "occurrence" : "?", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The listing options.</div>" } ], "returns" : { "type" : "object()", "description" : "the list of the object in the specified bucket" }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">s3:CREDENTIALS if the specified S3 credentials cannot be found or are invalid</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">s3:BUCKET if no bucket has been specified and no default bucket is present in the default credentials</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">s3:OPTIONS if the specified options are invalid</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">s3:HTTP an HTTP error occurred sending the request to S3</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">s3:REQUEST S3 refused to execute the request</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">s3:RESPONSE the response received from S3 cannot be parsed</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 3, "name" : "list-bucket-nondeterministic", "qname" : "s3:list-bucket-nondeterministic", "signature" : "($credentials as item()?, $bucket as string?, $options as object()?) as object()", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Lists the objects in a bucket using the specified credentials.\n This method allows the specification of additional listing options.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">This function has the same semantics as\n <a href=\"#list-bucket-3\">list-bucket#3</a>,\n but is declared as being non deterministic and thus should only be used when\n result caching is not desired.</p>\n", "summary" : "<p>  Lists the objects in a bucket using the specified credentials.</p>", "annotation_str" : " %an:nondeterministic", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "nondeterministic", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "credentials", "type" : "item()", "occurrence" : "?", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The credentials to use.</div>" }, { "name" : "bucket", "type" : "string", "occurrence" : "?", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The bucket to list. If the empty sequence is given, the default bucket of the specified credentials will be listed.</div>" }, { "name" : "options", "type" : "object()", "occurrence" : "?", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The listing options.</div>" } ], "returns" : { "type" : "object()", "description" : "the list of the object in the specified bucket" }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">s3:CREDENTIALS if the specified S3 credentials cannot be found or are invalid</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">s3:BUCKET if no bucket has been specified and no default bucket is present in the used credentials</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">s3:OPTIONS if the specified options are invalid</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">s3:HTTP an HTTP error occurred sending the request to S3</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">s3:REQUEST S3 refused to execute the request</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">s3:RESPONSE the response received from S3 cannot be parsed</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 0, "name" : "list-bucket-versions-nondeterministic", "qname" : "s3:list-bucket-versions-nondeterministic", "signature" : "() as object()", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Lists the objects versions in the default bucket of the default S3\n credentials.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">This function has the same semantics as\n <a href=\"#list-bucket-versions-0\">list-bucket-versions#0</a>,\n but is declared as being non deterministic and thus should only be used when\n result caching is not desired.</p>\n", "summary" : "<p>  Lists the objects versions in the default bucket of the default S3\n credentials.</p>", "annotation_str" : " %an:nondeterministic", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "nondeterministic", "value" : "" } ], "updating" : false, "parameters" : [  ], "returns" : { "type" : "object()", "description" : "the list of the object versions in the default bucket of the default S3 credentials" }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">s3:CREDENTIALS if the specified S3 credentials cannot be found or are invalid</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">s3:BUCKET if no default bucket is present in the default S3 credentials</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">s3:OPTIONS if the specified options are invalid</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">s3:HTTP an HTTP error occurred sending the request to S3</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">s3:REQUEST S3 refused to execute the request</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">s3:RESPONSE the response received from S3 cannot be parsed</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 1, "name" : "list-bucket-versions-nondeterministic", "qname" : "s3:list-bucket-versions-nondeterministic", "signature" : "($bucket as string?) as object()", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Lists the objects versions in a bucket using the default S3 credentials.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">This function has the same semantics as\n <a href=\"#list-bucket-versions-1\">list-bucket-versions#1</a>,\n but is declared as being non deterministic and thus should only be used when\n result caching is not desired.</p>\n", "summary" : "<p>  Lists the objects versions in a bucket using the default S3 credentials.</p>", "annotation_str" : " %an:nondeterministic", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "nondeterministic", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "bucket", "type" : "string", "occurrence" : "?", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The bucket to list. If the empty sequence is given, the default bucket of the default credentials will be listed.</div>" } ], "returns" : { "type" : "object()", "description" : "the list of the object versions in the specified bucket" }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">s3:CREDENTIALS if the specified S3 credentials cannot be found or are invalid</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">s3:BUCKET if no bucket has been specified and no default bucket is present in the used credentials</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">s3:OPTIONS if the specified options are invalid</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">s3:HTTP an HTTP error occurred sending the request to S3</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">s3:REQUEST S3 refused to execute the request</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">s3:RESPONSE the response received from S3 cannot be parsed</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 2, "name" : "list-bucket-versions-nondeterministic", "qname" : "s3:list-bucket-versions-nondeterministic", "signature" : "($bucket as string?, $options as object()?) as object()", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Lists the objects versions in a bucket using the default S3 credentials.\n This method allows the specification of additional listing options.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">This function has the same semantics as\n <a href=\"#list-bucket-versions-2\">list-bucket-versions#2</a>,\n but is declared as being non deterministic and thus should only be used when\n result caching is not desired.</p>\n", "summary" : "<p>  Lists the objects versions in a bucket using the default S3 credentials.</p>", "annotation_str" : " %an:nondeterministic", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "nondeterministic", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "bucket", "type" : "string", "occurrence" : "?", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The bucket to list. If the empty sequence is given, the default bucket of the default credentials will be listed.</div>" }, { "name" : "options", "type" : "object()", "occurrence" : "?", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The listing options.</div>" } ], "returns" : { "type" : "object()", "description" : "the list of the object versions in the specified bucket" }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">s3:CREDENTIALS if the specified S3 credentials cannot be found or are invalid</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">s3:BUCKET if no bucket has been specified and no default bucket is present in the default credentials</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">s3:OPTIONS if the specified options are invalid</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">s3:HTTP an HTTP error occurred sending the request to S3</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">s3:REQUEST S3 refused to execute the request</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">s3:RESPONSE the response received from S3 cannot be parsed</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 3, "name" : "list-bucket-versions-nondeterministic", "qname" : "s3:list-bucket-versions-nondeterministic", "signature" : "($credentials as item()?, $bucket as string?, $options as object()?) as object()", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Lists the objects versions in a bucket using the specified credentials.\n This method allows the specification of additional listing options.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">This function has the same semantics as\n <a href=\"#list-bucket-versions-3\">list-bucket-versions#3</a>,\n but is declared as being non deterministic and thus should only be used when\n result caching is not desired.</p>\n", "summary" : "<p>  Lists the objects versions in a bucket using the specified credentials.</p>", "annotation_str" : " %an:nondeterministic", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "nondeterministic", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "credentials", "type" : "item()", "occurrence" : "?", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The credentials to use.</div>" }, { "name" : "bucket", "type" : "string", "occurrence" : "?", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The bucket to list. If the empty sequence is given, the default bucket of the specified credentials will be listed.</div>" }, { "name" : "options", "type" : "object()", "occurrence" : "?", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The listing options.</div>" } ], "returns" : { "type" : "object()", "description" : "the list of the object in the specified bucket" }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">s3:CREDENTIALS if the specified S3 credentials cannot be found or are invalid</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">s3:BUCKET if no bucket has been specified and no default bucket is present in the used credentials</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">s3:OPTIONS if the specified options are invalid</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">s3:HTTP an HTTP error occurred sending the request to S3</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">s3:REQUEST S3 refused to execute the request</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">s3:RESPONSE the response received from S3 cannot be parsed</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 0, "name" : "list-bucket-versions", "qname" : "s3:list-bucket-versions", "signature" : "() as object()", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Lists the objects versions in the default bucket of the default S3\n credentials.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">This function is declared as deterministic and should be used whenever result\n caching is acceptable.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Example:\n <pre>\n s3:list-bucket-versions()\n </pre>\n </p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Example return object:\n <pre>\n {\n   \"name\": \"bucket\",\n   \"prefix\": \"\",\n   \"keyMarker\" : \"image.jpg\",\n   \"versionIdMarker\" : \"QUpfdndhfd8438MNFDN93jdnJFkdmqnh893\",\n   \"nextKeyMarker\": \"marker\",\n   \"nextVersionIdMarker\": \"UIORUnfndfhnw89493jJFJ\",\n   \"maxKeys\": 1,\n   \"delimiter\": \"/\",\n   \"isTruncated\": true,\n   \"revisions\":\n   [\n     {\n       \"kind\": \"Version\",\n       \"key\": \"image.jpg\",\n       \"versionId\": \"QUpfdndhfd8438MNFDN93jdnJFkdmqnh893\",\n       \"isLatest\": false,\n       \"lastModified\": \"2009-10-12T17:50:30.000Z\",\n       \"eTag\": \"\\\"fba9dede5f27731c9771645a39863328\\\"\",\n       \"size\": 434234,\n       \"storageClass\": \"STANDARD\",\n       \"owner\":\n       {\n         \"id\": \"75aa57f09aa0c8caeab4f8c24e99d10f8e7faeebf76c078efc7c6caea54ba06a\"\n         \"displayName\": \"mtd@amazon.com\"\n       }\n     },\n     {\n       \"kind\": \"DeleteMarker\",\n       \"key\": \"image.jpg\",\n       \"versionId\": \"03jpff543dhffds434rfdsFDN943fdsFkdmqnh892\",\n       \"isLatest\": true,\n       \"lastModified\": \"2009-10-12T17:50:30.000Z\",\n       \"owner\":\n       {\n         \"id\": \"75aa57f09aa0c8caeab4f8c24e99d10f8e7faeebf76c078efc7c6caea54ba06a\"\n         \"displayName\": \"mtd@amazon.com\"\n       }\n     }\n   ],\n   \"commonPrefixes\": [\"photos/\"]\n }\n </pre>\n </p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">The fields in the returned object have the following meanings:\n <ul>\n   <li>name: the bucket name (string, mandatory).</li>\n   <li>prefix: the used prefix (string, mandatory).</li>\n   <li>keyMarker: indicates from which key in the listing begins\n   (string, mandatory).</li>\n   <li>versionMarker: indicates from which version  the listing begins\n   (string, optional).</li>\n   <li>nextKeyMarker: when the number of responses exceeds the value of maxKeys,\n   nextKeyMarker specifies the first key not returned that satisfies the search\n   criteria. Use this value for the keyMarker request parameter in a subsequent\n   request. (string, optional)</li>\n   <li>nextVersionMarker: when the number of responses exceeds the value of maxKeys,\n   nextVersionIdMarker specifies the first object version not returned that\n   satisfies the search criteria. Use this value for the versionMarker request\n   parameter in a subsequent request. (string, optional)</li>\n   <li>maxKeys: the maximum number of keys returned in the response body\n   (integer, mandatory).</li>\n   <li>delimiter: the used delimiter (string, optional).</li>\n   <li>isTruncated: whether or not all of the results were returned. All of the\n   results may not be returned if the number of results exceeds that specified\n   by the maxKeys request parameter (boolean, mandatory).</li>\n   <li>revisions: an array of objects listing the bucket object versions\n   (array, mandatory). The fields of each contained object have the following\n   meaning:\n   <ul>\n     <li>kind: the revision kind. One of \"Version\" or \"DeleteMarker\" (string,\n     mandatory).</li>\n     <li>key: the object key (string, mandatory).</li>\n     <li>versionId: the object version id (string, mandatory).</li>\n     <li>lastModified: the object last modification date (dateTime, mandatory).</li>\n     <li>eTag: the entity tag is a hash of the object. The eTag only reflects\n     changes to the contents of an object, not its metadata. The eTag is determined\n     when an object is created. For objects created by the PUT Object operation\n     and the POST Object operation, the eTag is a quoted, 32-digit hexadecimal\n     string representing the MD5 digest of the object data. For other objects,\n     the eTag may or may not be an MD5 digest of the object data. If the eTag is\n     not an MD5 digest of the object data, it will contain one or more non-hexadecimal\n     characters and/or will consist of less than 32 or more than 32 hexadecimal\n     digits.(string, mandatory for \"Version\" objects, not present for\n     \"DeleteMarker\" objects).</li>\n     <li>size: the object size in bytes (integer, mandatory for \"Version\" objects,\n     not present for \"DeleteMarker\" objects).</li>\n     <li>storageClass: the object storage class. One of \"STANDARD\",\n     \"REDUCED_REDUNDANCY\" or \"GLACIER\" (string, mandatory for \"Version\" objects,\n     not present for \"DeleteMarker\" objects).</li>\n     <li>owner: an object which specifies the owner of the object (object,\n     optional). It has the following fields:\n     <ul>\n       <li>id: the user identifier of the object owner (string, mandatory).</li>\n       <li>displayName: the screen name of the object owner (string, mandatory).</li>\n     </ul>\n     </li>\n   </ul>\n   </li>\n   <li>commonPrefixes: an array of strings which contains the list of the common\n   prefixes. It can be present only when the delimiter request parameter is\n   specified (array, optional).</li>\n </ul>\n </p>\n", "summary" : "<p>  Lists the objects versions in the default bucket of the default S3\n credentials.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [  ], "returns" : { "type" : "object()", "description" : "the list of the object versions in the default bucket of the default S3 credentials" }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">s3:CREDENTIALS if the specified S3 credentials cannot be found or are invalid</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">s3:BUCKET if no default bucket is present in the default S3 credentials</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">s3:OPTIONS if the specified options are invalid</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">s3:HTTP an HTTP error occurred sending the request to S3</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">s3:REQUEST S3 refused to execute the request</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">s3:RESPONSE the response received from S3 cannot be parsed</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 1, "name" : "list-bucket-versions", "qname" : "s3:list-bucket-versions", "signature" : "($bucket as string?) as object()", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Lists the objects versions in a bucket using the default S3 credentials.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">This function is declared as deterministic and should be used whenever result\n caching is acceptable.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Example:\n <pre>\n s3:list-bucket-versions(\"28msec\")\n </pre>\n </p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Example return object:\n <pre>\n {\n   \"name\": \"bucket\",\n   \"prefix\": \"\",\n   \"keyMarker\" : \"image.jpg\",\n   \"versionIdMarker\" : \"QUpfdndhfd8438MNFDN93jdnJFkdmqnh893\",\n   \"nextKeyMarker\": \"marker\",\n   \"nextVersionIdMarker\": \"UIORUnfndfhnw89493jJFJ\",\n   \"maxKeys\": 1,\n   \"delimiter\": \"/\",\n   \"isTruncated\": true,\n   \"revisions\":\n   [\n     {\n       \"kind\": \"Version\",\n       \"key\": \"image.jpg\",\n       \"versionId\": \"QUpfdndhfd8438MNFDN93jdnJFkdmqnh893\",\n       \"isLatest\": false,\n       \"lastModified\": \"2009-10-12T17:50:30.000Z\",\n       \"eTag\": \"\\\"fba9dede5f27731c9771645a39863328\\\"\",\n       \"size\": 434234,\n       \"storageClass\": \"STANDARD\",\n       \"owner\":\n       {\n         \"id\": \"75aa57f09aa0c8caeab4f8c24e99d10f8e7faeebf76c078efc7c6caea54ba06a\"\n         \"displayName\": \"mtd@amazon.com\"\n       }\n     },\n     {\n       \"kind\": \"DeleteMarker\",\n       \"key\": \"image.jpg\",\n       \"versionId\": \"03jpff543dhffds434rfdsFDN943fdsFkdmqnh892\",\n       \"isLatest\": true,\n       \"lastModified\": \"2009-10-12T17:50:30.000Z\",\n       \"owner\":\n       {\n         \"id\": \"75aa57f09aa0c8caeab4f8c24e99d10f8e7faeebf76c078efc7c6caea54ba06a\"\n         \"displayName\": \"mtd@amazon.com\"\n       }\n     }\n   ],\n   \"commonPrefixes\": [\"photos/\"]\n }\n </pre>\n </p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">The fields in the returned object have the following meanings:\n <ul>\n   <li>name: the bucket name (string, mandatory).</li>\n   <li>prefix: the used prefix (string, mandatory).</li>\n   <li>keyMarker: indicates from which key in the listing begins\n   (string, mandatory).</li>\n   <li>versionMarker: indicates from which version  the listing begins\n   (string, optional).</li>\n   <li>nextKeyMarker: when the number of responses exceeds the value of maxKeys,\n   nextKeyMarker specifies the first key not returned that satisfies the search\n   criteria. Use this value for the keyMarker request parameter in a subsequent\n   request. (string, optional)</li>\n   <li>nextVersionMarker: when the number of responses exceeds the value of maxKeys,\n   nextVersionIdMarker specifies the first object version not returned that\n   satisfies the search criteria. Use this value for the versionMarker request\n   parameter in a subsequent request. (string, optional)</li>\n   <li>maxKeys: the maximum number of keys returned in the response body\n   (integer, mandatory).</li>\n   <li>delimiter: the used delimiter (string, optional).</li>\n   <li>isTruncated: whether or not all of the results were returned. All of the\n   results may not be returned if the number of results exceeds that specified\n   by the maxKeys request parameter (boolean, mandatory).</li>\n   <li>revisions: an array of objects listing the bucket object versions\n   (array, mandatory). The fields of each contained object have the following\n   meaning:\n   <ul>\n     <li>kind: the revision kind. One of \"Version\" or \"DeleteMarker\" (string,\n     mandatory).</li>\n     <li>key: the object key (string, mandatory).</li>\n     <li>versionId: the object version id (string, mandatory).</li>\n     <li>lastModified: the object last modification date (dateTime, mandatory).</li>\n     <li>eTag: the entity tag is a hash of the object. The eTag only reflects\n     changes to the contents of an object, not its metadata. The eTag is determined\n     when an object is created. For objects created by the PUT Object operation\n     and the POST Object operation, the eTag is a quoted, 32-digit hexadecimal\n     string representing the MD5 digest of the object data. For other objects,\n     the eTag may or may not be an MD5 digest of the object data. If the eTag is\n     not an MD5 digest of the object data, it will contain one or more non-hexadecimal\n     characters and/or will consist of less than 32 or more than 32 hexadecimal\n     digits.(string, mandatory for \"Version\" objects, not present for\n     \"DeleteMarker\" objects).</li>\n     <li>size: the object size in bytes (integer, mandatory for \"Version\" objects,\n     not present for \"DeleteMarker\" objects).</li>\n     <li>storageClass: the object storage class. One of \"STANDARD\",\n     \"REDUCED_REDUNDANCY\" or \"GLACIER\" (string, mandatory for \"Version\" objects,\n     not present for \"DeleteMarker\" objects).</li>\n     <li>owner: an object which specifies the owner of the object (object,\n     optional). It has the following fields:\n     <ul>\n       <li>id: the user identifier of the object owner (string, mandatory).</li>\n       <li>displayName: the screen name of the object owner (string, mandatory).</li>\n     </ul>\n     </li>\n   </ul>\n   </li>\n   <li>commonPrefixes: an array of strings which contains the list of the common\n   prefixes. It can be present only when the delimiter request parameter is\n   specified (array, optional).</li>\n </ul>\n </p>\n", "summary" : "<p>  Lists the objects versions in a bucket using the default S3 credentials.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "bucket", "type" : "string", "occurrence" : "?", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The bucket to list. If the empty sequence is given, the default bucket of the default credentials will be listed.</div>" } ], "returns" : { "type" : "object()", "description" : "the list of the object versions in the specified bucket" }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">s3:CREDENTIALS if the specified S3 credentials cannot be found or are invalid</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">s3:BUCKET if no bucket has been specified and no default bucket is present in the used credentials</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">s3:OPTIONS if the specified options are invalid</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">s3:HTTP an HTTP error occurred sending the request to S3</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">s3:REQUEST S3 refused to execute the request</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">s3:RESPONSE the response received from S3 cannot be parsed</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 2, "name" : "list-bucket-versions", "qname" : "s3:list-bucket-versions", "signature" : "($bucket as string?, $options as object()?) as object()", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Lists the objects versions in a bucket using the default S3 credentials.\n This method allows the specification of additional listing options.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">This function is declared as deterministic and should be used whenever result\n caching is acceptable.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Example:\n <pre>\n let $options :=\n {\n   \"delimiter\": \"/\",\n   \"maxKeys\": 2,\n   \"keyMarker\": \"image.jpg\",\n   \"versionIdMarker\" : \"QUpfdndhfd8438MNFDN93jdnJFkdmqnh893\"\n }\n return s3:list-bucket-versions(\"28msec\", $options)\n </pre>\n </p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">The options object must have the following structure:\n <ul>\n   <li>delimiter: the delimiter marks where the listed results stop. For example,\n       a delimiter / lists all objects starting with $prefix plus arbitrary\n       characters but not / (string, optional). Default is the empty string,\n       that is, all objects are listed.</li>\n   <li>keyMarker: specifies a key as starting point; following keys (lexicographically\n       greater than the marker) in alphabetical order are listed (string, optional).\n       Default is the empty string, that is, all objects are listed.</li>\n   <li>versionMarker: specifies the object version you want starting from;\n       following revisions (newer than the marker) are listed (string, optional).\n       By default all versions are listed. Empty string is not allowed.</li>\n   <li>maxKeys: the maximum number of keys returned. If more keys than maxKeys\n       can be fetched, the result contains <IsTruncated>true</IsTruncated>\n       (integer, optional). Default is 1000.</li>\n   <li>prefix: only keys starting with the prefix are returned (string, optional).\n       Default is the empty string, that is, all objects are listed.</li>\n </ul>\n </p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Example return object:\n <pre>\n {\n   \"name\": \"bucket\",\n   \"prefix\": \"\",\n   \"keyMarker\" : \"image.jpg\",\n   \"versionIdMarker\" : \"QUpfdndhfd8438MNFDN93jdnJFkdmqnh893\",\n   \"nextKeyMarker\": \"marker\",\n   \"nextVersionIdMarker\": \"UIORUnfndfhnw89493jJFJ\",\n   \"maxKeys\": 1,\n   \"delimiter\": \"/\",\n   \"isTruncated\": true,\n   \"revisions\":\n   [\n     {\n       \"kind\": \"Version\",\n       \"key\": \"image.jpg\",\n       \"versionId\": \"QUpfdndhfd8438MNFDN93jdnJFkdmqnh893\",\n       \"isLatest\": false,\n       \"lastModified\": \"2009-10-12T17:50:30.000Z\",\n       \"eTag\": \"\\\"fba9dede5f27731c9771645a39863328\\\"\",\n       \"size\": 434234,\n       \"storageClass\": \"STANDARD\",\n       \"owner\":\n       {\n         \"id\": \"75aa57f09aa0c8caeab4f8c24e99d10f8e7faeebf76c078efc7c6caea54ba06a\"\n         \"displayName\": \"mtd@amazon.com\"\n       }\n     },\n     {\n       \"kind\": \"DeleteMarker\",\n       \"key\": \"image.jpg\",\n       \"versionId\": \"03jpff543dhffds434rfdsFDN943fdsFkdmqnh892\",\n       \"isLatest\": true,\n       \"lastModified\": \"2009-10-12T17:50:30.000Z\",\n       \"owner\":\n       {\n         \"id\": \"75aa57f09aa0c8caeab4f8c24e99d10f8e7faeebf76c078efc7c6caea54ba06a\"\n         \"displayName\": \"mtd@amazon.com\"\n       }\n     }\n   ],\n   \"commonPrefixes\": [\"photos/\"]\n }\n </pre>\n </p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">The fields in the returned object have the following meanings:\n <ul>\n   <li>name: the bucket name (string, mandatory).</li>\n   <li>prefix: the used prefix (string, mandatory).</li>\n   <li>keyMarker: indicates from which key in the listing begins\n   (string, mandatory).</li>\n   <li>versionMarker: indicates from which version  the listing begins\n   (string, optional).</li>\n   <li>nextKeyMarker: when the number of responses exceeds the value of maxKeys,\n   nextKeyMarker specifies the first key not returned that satisfies the search\n   criteria. Use this value for the keyMarker request parameter in a subsequent\n   request. (string, optional)</li>\n   <li>nextVersionMarker: when the number of responses exceeds the value of maxKeys,\n   nextVersionIdMarker specifies the first object version not returned that\n   satisfies the search criteria. Use this value for the versionMarker request\n   parameter in a subsequent request. (string, optional)</li>\n   <li>maxKeys: the maximum number of keys returned in the response body\n   (integer, mandatory).</li>\n   <li>delimiter: the used delimiter (string, optional).</li>\n   <li>isTruncated: whether or not all of the results were returned. All of the\n   results may not be returned if the number of results exceeds that specified\n   by the maxKeys request parameter (boolean, mandatory).</li>\n   <li>revisions: an array of objects listing the bucket object versions\n   (array, mandatory). The fields of each contained object have the following\n   meaning:\n   <ul>\n     <li>kind: the revision kind. One of \"Version\" or \"DeleteMarker\" (string,\n     mandatory).</li>\n     <li>key: the object key (string, mandatory).</li>\n     <li>versionId: the object version id (string, mandatory).</li>\n     <li>lastModified: the object last modification date (dateTime, mandatory).</li>\n     <li>eTag: the entity tag is a hash of the object. The eTag only reflects\n     changes to the contents of an object, not its metadata. The eTag is determined\n     when an object is created. For objects created by the PUT Object operation\n     and the POST Object operation, the eTag is a quoted, 32-digit hexadecimal\n     string representing the MD5 digest of the object data. For other objects,\n     the eTag may or may not be an MD5 digest of the object data. If the eTag is\n     not an MD5 digest of the object data, it will contain one or more non-hexadecimal\n     characters and/or will consist of less than 32 or more than 32 hexadecimal\n     digits.(string, mandatory for \"Version\" objects, not present for\n     \"DeleteMarker\" objects).</li>\n     <li>size: the object size in bytes (integer, mandatory for \"Version\" objects,\n     not present for \"DeleteMarker\" objects).</li>\n     <li>storageClass: the object storage class. One of \"STANDARD\",\n     \"REDUCED_REDUNDANCY\" or \"GLACIER\" (string, mandatory for \"Version\" objects,\n     not present for \"DeleteMarker\" objects).</li>\n     <li>owner: an object which specifies the owner of the object (object,\n     optional). It has the following fields:\n     <ul>\n       <li>id: the user identifier of the object owner (string, mandatory).</li>\n       <li>displayName: the screen name of the object owner (string, mandatory).</li>\n     </ul>\n     </li>\n   </ul>\n   </li>\n   <li>commonPrefixes: an array of strings which contains the list of the common\n   prefixes. It can be present only when the delimiter request parameter is\n   specified (array, optional).</li>\n </ul>\n </p>\n", "summary" : "<p>  Lists the objects versions in a bucket using the default S3 credentials.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "bucket", "type" : "string", "occurrence" : "?", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The bucket to list. If the empty sequence is given, the default bucket of the default credentials will be listed.</div>" }, { "name" : "options", "type" : "object()", "occurrence" : "?", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The listing options.</div>" } ], "returns" : { "type" : "object()", "description" : "the list of the object versions in the specified bucket" }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">s3:CREDENTIALS if the specified S3 credentials cannot be found or are invalid</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">s3:BUCKET if no bucket has been specified and no default bucket is present in the default credentials</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">s3:OPTIONS if the specified options are invalid</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">s3:HTTP an HTTP error occurred sending the request to S3</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">s3:REQUEST S3 refused to execute the request</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">s3:RESPONSE the response received from S3 cannot be parsed</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 3, "name" : "list-bucket-versions", "qname" : "s3:list-bucket-versions", "signature" : "($credentials as item()?, $bucket as string?, $options as object()?) as object()", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Lists the objects versions in a bucket using the specified credentials.\n This method allows the specification of additional listing options.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">This function is declared as deterministic and should be used whenever result\n caching is acceptable.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Example:\n <pre>\n let $options :=\n {\n   \"delimiter\": \"/\",\n   \"maxKeys\": 2,\n   \"keyMarker\": \"image.jpg\",\n   \"versionIdMarker\": \"QUpfdndhfd8438MNFDN93jdnJFkdmqnh893\"\n }\n return s3:list-bucket-versions(\"credentials\", \"28msec\", $options)\n </pre>\n </p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">The options object must have the following structure:\n <ul>\n   <li>delimiter: the delimiter marks where the listed results stop. For example,\n       a delimiter / lists all objects starting with $prefix plus arbitrary\n       characters but not / (string, optional). Default is the empty string,\n       that is, all objects are listed.</li>\n   <li>keyMarker: specifies a key as starting point; following keys (lexicographically\n       greater than the marker) in alphabetical order are listed (string, optional).\n       Default is the empty string, that is, all objects are listed.</li>\n   <li>versionMarker: specifies the object version you want starting from;\n       following revisions (newer than the marker) are listed (string, optional).\n       By default all versions are listed. Empty string is not allowed.</li>\n   <li>maxKeys: the maximum number of keys returned. If more keys than maxKeys\n       can be fetched, the result contains <IsTruncated>true</IsTruncated>\n       (integer, optional). Default is 1000.</li>\n   <li>prefix: only keys starting with the prefix are returned (string, optional).\n       Default is the empty string, that is, all objects are listed.</li>\n </ul>\n </p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Example return object:\n <pre>\n {\n   \"name\": \"bucket\",\n   \"prefix\": \"\",\n   \"keyMarker\" : \"image.jpg\",\n   \"VersionIdMarker\" : \"QUpfdndhfd8438MNFDN93jdnJFkdmqnh893\",\n   \"nextKeyMarker\": \"marker\",\n   \"nextVersionIdMarker\": \"UIORUnfndfhnw89493jJFJ\",\n   \"maxKeys\": 1,\n   \"delimiter\": \"/\",\n   \"isTruncated\": true,\n   \"revisions\":\n   [\n     {\n       \"kind\": \"Version\",\n       \"key\": \"image.jpg\",\n       \"versionId\": \"QUpfdndhfd8438MNFDN93jdnJFkdmqnh893\",\n       \"isLatest\": false,\n       \"lastModified\": \"2009-10-12T17:50:30.000Z\",\n       \"eTag\": \"\\\"fba9dede5f27731c9771645a39863328\\\"\",\n       \"size\": 434234,\n       \"storageClass\": \"STANDARD\",\n       \"owner\":\n       {\n         \"id\": \"75aa57f09aa0c8caeab4f8c24e99d10f8e7faeebf76c078efc7c6caea54ba06a\"\n         \"displayName\": \"mtd@amazon.com\"\n       }\n     },\n     {\n       \"kind\": \"DeleteMarker\",\n       \"key\": \"image.jpg\",\n       \"versionId\": \"03jpff543dhffds434rfdsFDN943fdsFkdmqnh892\",\n       \"isLatest\": true,\n       \"lastModified\": \"2009-10-12T17:50:30.000Z\",\n       \"owner\":\n       {\n         \"id\": \"75aa57f09aa0c8caeab4f8c24e99d10f8e7faeebf76c078efc7c6caea54ba06a\"\n         \"displayName\": \"mtd@amazon.com\"\n       }\n     }\n   ],\n   \"commonPrefixes\": [\"photos/\"]\n }\n </pre>\n </p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">The fields in the returned object have the following meanings:\n <ul>\n   <li>name: the bucket name (string, mandatory).</li>\n   <li>prefix: the used prefix (string, mandatory).</li>\n   <li>keyMarker: indicates from which key in the listing begins\n   (string, mandatory).</li>\n   <li>versionMarker: indicates from which version  the listing begins\n   (string, optional).</li>\n   <li>nextKeyMarker: when the number of responses exceeds the value of maxKeys,\n   nextKeyMarker specifies the first key not returned that satisfies the search\n   criteria. Use this value for the keyMarker request parameter in a subsequent\n   request. (string, optional)</li>\n   <li>nextVersionMarker: when the number of responses exceeds the value of maxKeys,\n   nextVersionIdMarker specifies the first object version not returned that\n   satisfies the search criteria. Use this value for the versionMarker request\n   parameter in a subsequent request. (string, optional)</li>\n   <li>maxKeys: the maximum number of keys returned in the response body\n   (integer, mandatory).</li>\n   <li>delimiter: the used delimiter (string, optional).</li>\n   <li>isTruncated: whether or not all of the results were returned. All of the\n   results may not be returned if the number of results exceeds that specified\n   by the maxKeys request parameter (boolean, mandatory).</li>\n   <li>revisions: an array of objects listing the bucket object versions\n   (array, mandatory). The fields of each contained object have the following\n   meaning:\n   <ul>\n     <li>kind: the revision kind. One of \"Version\" or \"DeleteMarker\" (string,\n     mandatory).</li>\n     <li>key: the object key (string, mandatory).</li>\n     <li>versionId: the object version id (string, mandatory).</li>\n     <li>lastModified: the object last modification date (dateTime, mandatory).</li>\n     <li>eTag: the entity tag is a hash of the object. The eTag only reflects\n     changes to the contents of an object, not its metadata. The eTag is determined\n     when an object is created. For objects created by the PUT Object operation\n     and the POST Object operation, the eTag is a quoted, 32-digit hexadecimal\n     string representing the MD5 digest of the object data. For other objects,\n     the eTag may or may not be an MD5 digest of the object data. If the eTag is\n     not an MD5 digest of the object data, it will contain one or more non-hexadecimal\n     characters and/or will consist of less than 32 or more than 32 hexadecimal\n     digits.(string, mandatory for \"Version\" objects, not present for\n     \"DeleteMarker\" objects).</li>\n     <li>size: the object size in bytes (integer, mandatory for \"Version\" objects,\n     not present for \"DeleteMarker\" objects).</li>\n     <li>storageClass: the object storage class. One of \"STANDARD\",\n     \"REDUCED_REDUNDANCY\" or \"GLACIER\" (string, mandatory for \"Version\" objects,\n     not present for \"DeleteMarker\" objects).</li>\n     <li>owner: an object which specifies the owner of the object (object,\n     optional). It has the following fields:\n     <ul>\n       <li>id: the user identifier of the object owner (string, mandatory).</li>\n       <li>displayName: the screen name of the object owner (string, mandatory).</li>\n     </ul>\n     </li>\n   </ul>\n   </li>\n   <li>commonPrefixes: an array of strings which contains the list of the common\n   prefixes. It can be present only when the delimiter request parameter is\n   specified (array, optional).</li>\n </ul>\n </p>\n", "summary" : "<p>  Lists the objects versions in a bucket using the specified credentials.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "credentials", "type" : "item()", "occurrence" : "?", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The credentials to use.</div>" }, { "name" : "bucket", "type" : "string", "occurrence" : "?", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The bucket to list. If the empty sequence is given, the default bucket of the specified credentials will be listed.</div>" }, { "name" : "options", "type" : "object()", "occurrence" : "?", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The listing options.</div>" } ], "returns" : { "type" : "object()", "description" : "the list of the object in the specified bucket" }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">s3:CREDENTIALS if the specified S3 credentials cannot be found or are invalid</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">s3:BUCKET if no bucket has been specified and no default bucket is present in the used credentials</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">s3:OPTIONS if the specified options are invalid</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">s3:HTTP an HTTP error occurred sending the request to S3</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">s3:REQUEST S3 refused to execute the request</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">s3:RESPONSE the response received from S3 cannot be parsed</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 0, "name" : "list-bucket", "qname" : "s3:list-bucket", "signature" : "() as object()", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Lists the objects in the default bucket of the default S3 credentials.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">This function is declared as deterministic and should be used whenever result\n caching is acceptable.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Example:\n <pre>\n s3:list-bucket(\"28msec\")\n </pre>\n </p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Example return object:\n <pre>\n {\n   \"name\": \"28msec\",\n   \"prefix\": \"\",\n   \"marker\" : \"\",\n   \"nextMarker\": \"marker\",\n   \"maxKeys\": 1,\n   \"delimiter\": \"\",\n   \"isTruncated\": true,\n   \"contents\":\n   [\n     {\n       \"key\": \"image.jpg\",\n       \"lastModified\": \"2009-10-12T17:50:30.000Z\",\n       \"eTag\": \"\\\"fba9dede5f27731c9771645a39863328\\\"\",\n       \"size\": 434234,\n       \"storageClass\": \"STANDARD\"\n       \"owner\":\n       {\n         \"id\": \"75aa57f09aa0c8caeab4f8c24e99d10f8e7faeebf76c078efc7c6caea54ba06a\"\n         \"displayName\": \"mtd@amazon.com\"\n       }\n     }\n   ]\n }\n </pre>\n </p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">The fields in the returned object have the following meanings:\n <ul>\n   <li>name: the bucket name (string, mandatory).</li>\n   <li>prefix: the used prefix (string, mandatory).</li>\n   <li>marker: indicates where in the bucket listing begins (string, mandatory).</li>\n   <li>nextMarker: when the response is truncated, you can use the key name\n   in this field as marker in the subsequent request to get next set of objects.\n   Amazon S3 lists objects in alphabetical order. This field is returned only\n   if you have specified the delimiter request parameter. If the response does\n   not include the nextMaker field and it is truncated, you can use the value\n   of the last Key in the response as the marker in the subsequent request to\n   get the next set of object keys. (string, optional)</li>\n   <li>maxKeys: the maximum number of keys returned in the response body\n   (integer, mandatory).</li>\n   <li>delimiter: the used delimiter (string, optional).</li>\n   <li>isTruncated: whether or not all of the results were returned. All of the\n   results may not be returned if the number of results exceeds that specified\n   by the maxKeys request parameter (boolean, mandatory).</li>\n   <li>contents: an array of objects listing the bucket content  (array, mandatory).\n   The fields of each contained object have the following meaning:\n   <ul>\n     <li>key: the object key (string, mandatory).</li>\n     <li>lastModified: the object last modification date (dateTime, mandatory).</li>\n     <li>eTag: the entity tag is a hash of the object. The eTag only reflects\n     changes to the contents of an object, not its metadata. The eTag is determined\n     when an object is created. For objects created by the PUT Object operation\n     and the POST Object operation, the eTag is a quoted, 32-digit hexadecimal\n     string representing the MD5 digest of the object data. For other objects,\n     the eTag may or may not be an MD5 digest of the object data. If the eTag is\n     not an MD5 digest of the object data, it will contain one or more non-hexadecimal\n     characters and/or will consist of less than 32 or more than 32 hexadecimal\n     digits.(string, mandatory).</li>\n     <li>storageClass: the object storage class. One of \"STANDARD\",\n     \"REDUCED_REDUNDANCY\" or \"GLACIER\" (string, mandatory).</li>\n     <li>owner: an object which specifies the owner of the object (object,\n     optional). It has the following fields:\n     <ul>\n       <li>id: the user identifier of the object owner (string, mandatory).</li>\n       <li>displayName: the screen name of the object owner (string, mandatory).</li>\n     </ul>\n     </li>\n   </ul>\n   </li>\n </ul>\n </p>\n", "summary" : "<p>  Lists the objects in the default bucket of the default S3 credentials.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [  ], "returns" : { "type" : "object()", "description" : "the list of objects in the default bucket of the default S3 credentials" }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">s3:CREDENTIALS if the specified S3 credentials cannot be found or are invalid</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">s3:BUCKET if no default bucket is present in the default S3 credentials</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">s3:OPTIONS if the specified options are invalid</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">s3:HTTP an HTTP error occurred sending the request to S3</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">s3:REQUEST S3 refused to execute the request</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">s3:RESPONSE the response received from S3 cannot be parsed</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 1, "name" : "list-bucket", "qname" : "s3:list-bucket", "signature" : "($bucket as string?) as object()", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Lists the objects in a bucket using the default S3 credentials.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">This function is declared as deterministic and should be used whenever result\n caching is acceptable.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Example:\n <pre>\n s3:list-bucket(\"28msec\")\n </pre>\n </p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Example return object:\n <pre>\n {\n   \"name\": \"28msec\",\n   \"prefix\": \"\",\n   \"marker\" : \"\",\n   \"nextMarker\": \"marker\",\n   \"maxKeys\": 1,\n   \"delimiter\": \"\",\n   \"isTruncated\": true,\n   \"contents\":\n   [\n     {\n       \"key\": \"image.jpg\",\n       \"lastModified\": \"2009-10-12T17:50:30.000Z\",\n       \"eTag\": \"\\\"fba9dede5f27731c9771645a39863328\\\"\",\n       \"size\": 434234,\n       \"storageClass\": \"STANDARD\"\n       \"owner\":\n       {\n         \"id\": \"75aa57f09aa0c8caeab4f8c24e99d10f8e7faeebf76c078efc7c6caea54ba06a\"\n         \"displayName\": \"mtd@amazon.com\"\n       }\n     }\n   ]\n }\n </pre>\n The meanings of the fields in the return object are the following:\n <ul>\n   <li>name: the bucket name (string, mandatory).</li>\n   <li>prefix: the used prefix (string, mandatory).</li>\n   <li>marker: indicates where in the bucket listing begins (string, mandatory).</li>\n   <li>nextMarker: when the response is truncated, you can use the key name\n   in this field as marker in the subsequent request to get next set of objects.\n   Amazon S3 lists objects in alphabetical order. This field is returned only\n   if you have specified the delimiter request parameter. If the response does\n   not include the nextMaker field and it is truncated, you can use the value\n   of the last Key in the response as the marker in the subsequent request to\n   get the next set of object keys. (string, optional)</li>\n   <li>maxKeys: the maximum number of keys returned in the response body\n   (integer, mandatory).</li>\n   <li>delimiter: the used delimiter (string, optional).</li>\n   <li>isTruncated: whether or not all of the results were returned. All of the\n   results may not be returned if the number of results exceeds that specified\n   by the maxKeys request parameter (boolean, mandatory).</li>\n   <li>contents: an array of objects listing the bucket content  (array, mandatory).\n   The fields of each contained object have the following meaning:\n   <ul>\n     <li>key: the object key (string, mandatory).</li>\n     <li>lastModified: the object last modification date (dateTime, mandatory).</li>\n     <li>eTag: the entity tag is a hash of the object. The eTag only reflects\n     changes to the contents of an object, not its metadata. The eTag is determined\n     when an object is created. For objects created by the PUT Object operation\n     and the POST Object operation, the eTag is a quoted, 32-digit hexadecimal\n     string representing the MD5 digest of the object data. For other objects,\n     the eTag may or may not be an MD5 digest of the object data. If the eTag is\n     not an MD5 digest of the object data, it will contain one or more non-hexadecimal\n     characters and/or will consist of less than 32 or more than 32 hexadecimal\n     digits.(string, mandatory).</li>\n     <li>storageClass: the object storage class. One of \"STANDARD\",\n     \"REDUCED_REDUNDANCY\" or \"GLACIER\" (string, mandatory).</li>\n     <li>owner: an object which specifies the owner of the object (object,\n     optional). It has the following fields:\n     <ul>\n       <li>id: the user identifier of the object owner (string, mandatory).</li>\n       <li>displayName: the screen name of the object owner (string, mandatory).</li>\n     </ul>\n     </li>\n   </ul>\n   </li>\n </ul>\n </p>\n", "summary" : "<p>  Lists the objects in a bucket using the default S3 credentials.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "bucket", "type" : "string", "occurrence" : "?", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The bucket to list. If the empty sequence is given, the default bucket of the default S3 credentials will be listed.</div>" } ], "returns" : { "type" : "object()", "description" : "the list of the objects in the specified bucket." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">s3:CREDENTIALS if the specified S3 credentials cannot be found or are invalid</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">s3:BUCKET if no bucket has been specified and no default bucket is present in the used credentials</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">s3:OPTIONS if the specified options are invalid</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">s3:HTTP an HTTP error occurred sending the request to S3</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">s3:REQUEST S3 refused to execute the request</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">s3:RESPONSE the response received from S3 cannot be parsed</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 2, "name" : "list-bucket", "qname" : "s3:list-bucket", "signature" : "($bucket as string?, $options as object()?) as object()", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Lists the objects in a bucket using the default S3 credentials.\n This method allows the specification of additional listing options.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">This function is declared as deterministic and should be used whenever result\n caching is acceptable.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Example:\n <pre>\n let $options :=\n {\n   \"delimiter\": \"/\",\n   \"marker\": \"\",\n   \"maxKeys\": 1,\n   \"prefix\": \"\"\n }\n return s3:list-bucket(\"credentials\", \"28msec\", $options)\n </pre>\n </p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">The options object must have the following structure:\n <ul>\n   <li>delimiter: the delimiter marks where the listed results stop. For example,\n       a delimiter / lists all objects starting with $prefix plus arbitrary\n       characters but not / (string, optional). Default is the empty string,\n       that is, all objects are listed.</li>\n   <li>marker: specifies a key as starting point; following keys (lexicographically\n       greater than the marker) in alphabetical order are listed (string, optional).\n       Default is the empty string, that is, all objects are listed.</li>\n   <li>maxKeys: the maximum number of keys returned. If more keys than maxKeys\n       can be fetched, the result contains <IsTruncated>true</IsTruncated>\n       (integer, optional). Default is 1000.</li>\n   <li>prefix: only keys starting with the prefix are returned (string, optional).\n       Default is the empty string, that is, all objects are listed.</li>\n </ul>\n </p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Example return object:\n <pre>\n {\n   \"name\": \"bucket\",\n   \"prefix\": \"\",\n   \"marker\" : \"\",\n   \"nextMarker\": \"marker\",\n   \"maxKeys\": 1,\n   \"delimiter\": \"/\",\n   \"isTruncated\": true,\n   \"contents\":\n   [\n     {\n       \"key\": \"image.jpg\",\n       \"lastModified\": \"2009-10-12T17:50:30.000Z\",\n       \"eTag\": \"\\\"fba9dede5f27731c9771645a39863328\\\"\",\n       \"size\": 434234,\n       \"storageClass\": \"STANDARD\"\n       \"owner\":\n       {\n         \"id\": \"75aa57f09aa0c8caeab4f8c24e99d10f8e7faeebf76c078efc7c6caea54ba06a\"\n         \"displayName\": \"mtd@amazon.com\"\n       }\n     }\n   ],\n   \"commonPrefixes\": [\"photos/\"]\n }\n </pre>\n </p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">The fields in the returned object have the following meanings:\n <ul>\n   <li>name: the bucket name (string, mandatory).</li>\n   <li>prefix: the used prefix (string, mandatory).</li>\n   <li>marker: indicates where in the bucket listing begins (string, mandatory).</li>\n   <li>nextMarker: when the response is truncated, you can use the key name\n   in this field as marker in the subsequent request to get next set of objects.\n   Amazon S3 lists objects in alphabetical order. This field is returned only\n   if you have specified the delimiter request parameter. If the response does\n   not include the nextMaker field and it is truncated, you can use the value\n   of the last Key in the response as the marker in the subsequent request to\n   get the next set of object keys. (string, optional)</li>\n   <li>maxKeys: the maximum number of keys returned in the response body\n   (integer, mandatory).</li>\n   <li>delimiter: the used delimiter (string, optional).</li>\n   <li>isTruncated: whether or not all of the results were returned. All of the\n   results may not be returned if the number of results exceeds that specified\n   by the maxKeys request parameter (boolean, mandatory).</li>\n   <li>contents: an array of objects listing the bucket content  (array, mandatory).\n   The fields of each contained object have the following meaning:\n   <ul>\n     <li>key: the object key (string, mandatory).</li>\n     <li>lastModified: the object last modification date (dateTime, mandatory).</li>\n     <li>eTag: the entity tag is a hash of the object. The eTag only reflects\n     changes to the contents of an object, not its metadata. The eTag is determined\n     when an object is created. For objects created by the PUT Object operation\n     and the POST Object operation, the eTag is a quoted, 32-digit hexadecimal\n     string representing the MD5 digest of the object data. For other objects,\n     the eTag may or may not be an MD5 digest of the object data. If the eTag is\n     not an MD5 digest of the object data, it will contain one or more non-hexadecimal\n     characters and/or will consist of less than 32 or more than 32 hexadecimal\n     digits.(string, mandatory).</li>\n     <li>storageClass: the object storage class. One of \"STANDARD\",\n     \"REDUCED_REDUNDANCY\" or \"GLACIER\" (string, mandatory).</li>\n     <li>owner: an object which specifies the owner of the object (object,\n     optional). It has the following fields:\n     <ul>\n       <li>id: the user identifier of the object owner (string, mandatory).</li>\n       <li>displayName: the screen name of the object owner (string, mandatory).</li>\n     </ul>\n     </li>\n   </ul>\n   </li>\n   <li>commonPrefixes: an array of strings which contains the list of the common\n   prefixes. It can be present only when the delimiter request parameter is\n   specified (array, optional).</li>\n </ul>\n </p>\n", "summary" : "<p>  Lists the objects in a bucket using the default S3 credentials.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "bucket", "type" : "string", "occurrence" : "?", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The bucket to list. If the empty sequence is given, the default bucket of the default S3 credentials will be listed.</div>" }, { "name" : "options", "type" : "object()", "occurrence" : "?", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The listing options.</div>" } ], "returns" : { "type" : "object()", "description" : "the list of the object in the specified bucket" }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">s3:CREDENTIALS if the specified S3 credentials cannot be found or are invalid</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">s3:BUCKET if no bucket has been specified and no default bucket is present in the default credentials</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">s3:OPTIONS if the specified options are invalid</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">s3:HTTP an HTTP error occurred sending the request to S3</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">s3:REQUEST S3 refused to execute the request</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">s3:RESPONSE the response received from S3 cannot be parsed</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 3, "name" : "list-bucket", "qname" : "s3:list-bucket", "signature" : "($credentials as item()?, $bucket as string?, $options as object()?) as object()", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Lists the objects in a bucket using the specified credentials.\n This method allows the specification of additional listing options.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">This function is declared as deterministic and should be used whenever result\n caching is acceptable.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Example:\n <pre>\n let $options :=\n {\n   \"delimiter\": \"/\",\n   \"marker\": \"\",\n   \"maxKeys\": 1,\n   \"prefix\": \"\"\n }\n return s3:list-bucket(\"credentials\", \"28msec\", $options)\n </pre>\n </p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">The options object must have the following structure:\n <ul>\n   <li>delimiter: the delimiter marks where the listed results stop. For example,\n       a delimiter / lists all objects starting with $prefix plus arbitrary\n       characters but not / (string, optional). Default is the empty string,\n       that is, all objects are listed.</li>\n   <li>marker: specifies a key as starting point; following keys (lexicographically\n       greater than the marker) in alphabetical order are listed (string, optional).\n       Default is the empty string, that is, all objects are listed.</li>\n   <li>maxKeys: the maximum number of keys returned. If more keys than maxKeys\n       can be fetched, the result contains <IsTruncated>true</IsTruncated>\n       (integer, optional). Default is 1000.</li>\n   <li>prefix: only keys starting with the prefix are returned (string, optional).\n       Default is the empty string, that is, all objects are listed.</li>\n </ul>\n </p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Example result:\n <pre>\n {\n   \"name\": \"bucket\",\n   \"prefix\": \"\",\n   \"marker\" : \"\",\n   \"nextMarker\": \"marker\",\n   \"maxKeys\": 1,\n   \"delimiter\": \"/\",\n   \"isTruncated\": true,\n   \"contents\":\n   [\n     {\n       \"key\": \"image.jpg\",\n       \"lastModified\": \"2009-10-12T17:50:30.000Z\",\n       \"eTag\": \"\\\"fba9dede5f27731c9771645a39863328\\\"\",\n       \"size\": 434234,\n       \"storageClass\": \"STANDARD\"\n       \"owner\":\n       {\n         \"id\": \"75aa57f09aa0c8caeab4f8c24e99d10f8e7faeebf76c078efc7c6caea54ba06a\"\n         \"displayName\": \"mtd@amazon.com\"\n       }\n     }\n   ],\n   \"commonPrefixes\": [\"photos/\"]\n }\n </pre>\n </p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">The fields in the returned object have the following meanings:\n <ul>\n   <li>name: the bucket name (string, mandatory).</li>\n   <li>prefix: the used prefix (string, mandatory).</li>\n   <li>marker: indicates where in the bucket listing begins (string, mandatory).</li>\n   <li>nextMarker: when the response is truncated, you can use the key name\n   in this field as marker in the subsequent request to get next set of objects.\n   Amazon S3 lists objects in alphabetical order. This field is returned only\n   if you have specified the delimiter request parameter. If the response does\n   not include the nextMaker field and it is truncated, you can use the value\n   of the last Key in the response as the marker in the subsequent request to\n   get the next set of object keys. (string, optional)</li>\n   <li>maxKeys: the maximum number of keys returned in the response body\n   (integer, mandatory).</li>\n   <li>delimiter: the used delimiter (string, optional).</li>\n   <li>isTruncated: whether or not all of the results were returned. All of the\n   results may not be returned if the number of results exceeds that specified\n   by the maxKeys request parameter (boolean, mandatory).</li>\n   <li>contents: an array of objects listing the bucket content  (array, mandatory).\n   The fields of each contained object have the following meaning:\n   <ul>\n     <li>key: the object key (string, mandatory).</li>\n     <li>lastModified: the object last modification date (dateTime, mandatory).</li>\n     <li>eTag: the entity tag is a hash of the object. The eTag only reflects\n     changes to the contents of an object, not its metadata. The eTag is determined\n     when an object is created. For objects created by the PUT Object operation\n     and the POST Object operation, the eTag is a quoted, 32-digit hexadecimal\n     string representing the MD5 digest of the object data. For other objects,\n     the eTag may or may not be an MD5 digest of the object data. If the eTag is\n     not an MD5 digest of the object data, it will contain one or more non-hexadecimal\n     characters and/or will consist of less than 32 or more than 32 hexadecimal\n     digits.(string, mandatory).</li>\n     <li>storageClass: the object storage class. One of \"STANDARD\",\n     \"REDUCED_REDUNDANCY\" or \"GLACIER\" (string, mandatory).</li>\n     <li>owner: an object which specifies the owner of the object (object,\n     optional). It has the following fields:\n     <ul>\n       <li>id: the user identifier of the object owner (string, mandatory).</li>\n       <li>displayName: the screen name of the object owner (string, mandatory).</li>\n     </ul>\n     </li>\n   </ul>\n   </li>\n   <li>commonPrefixes: an array of strings which contains the list of the common\n   prefixes. It can be present only when the delimiter request parameter is\n   specified (array, optional).</li>\n </ul>\n </p>\n", "summary" : "<p>  Lists the objects in a bucket using the specified credentials.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "credentials", "type" : "item()", "occurrence" : "?", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The credentials to use.</div>" }, { "name" : "bucket", "type" : "string", "occurrence" : "?", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The bucket to list. If the empty sequence is given, the default bucket of the specified credentials will be listed.</div>" }, { "name" : "options", "type" : "object()", "occurrence" : "?", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The listing options.</div>" } ], "returns" : { "type" : "object()", "description" : "the list of the object in the specified bucket" }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">s3:CREDENTIALS if the specified S3 credentials cannot be found or are invalid</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">s3:BUCKET if no bucket has been specified and no default bucket is present in the used credentials</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">s3:OPTIONS if the specified options are invalid</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">s3:HTTP an HTTP error occurred sending the request to S3</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">s3:REQUEST S3 refused to execute the request</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">s3:RESPONSE the response received from S3 cannot be parsed</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 0, "name" : "list-buckets-nondeterministic", "qname" : "s3:list-buckets-nondeterministic", "signature" : "() as object()", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Lists all buckets owned by the account of the default S3 credentials.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">This function has the same semantics as\n <a href=\"#list-buckets-0\">list-buckets#0</a>,\n but is declared as being non deterministic and thus should only be used when\n result caching is not desired.</p>\n", "summary" : "<p>  Lists all buckets owned by the account of the default S3 credentials.</p>", "annotation_str" : " %an:nondeterministic", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "nondeterministic", "value" : "" } ], "updating" : false, "parameters" : [  ], "returns" : { "type" : "object()", "description" : "the owned buckets list" }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">s3:CREDENTIALS if the default S3 credentials cannot be found or are invalid</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">s3:HTTP an HTTP error occurred sending the request to S3</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">s3:REQUEST S3 refused to execute the request</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">s3:RESPONSE the response received from S3 cannot be parsed</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 1, "name" : "list-buckets-nondeterministic", "qname" : "s3:list-buckets-nondeterministic", "signature" : "($credentials as item()?) as object()", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Lists all buckets owned by the account of the specified S3 credentials.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">This function has the same semantics as\n <a href=\"#list-buckets-1\">list-buckets#1</a>,\n but is declared as being non deterministic and thus should only be used when\n result caching is not desired.</p>\n", "summary" : "<p>  Lists all buckets owned by the account of the specified S3 credentials.</p>", "annotation_str" : " %an:nondeterministic", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "nondeterministic", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "credentials", "type" : "item()", "occurrence" : "?", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The credentials to use.</div>" } ], "returns" : { "type" : "object()", "description" : "the owned buckets list" }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">s3:CREDENTIALS if the specified S3 credentials cannot be found or are invalid</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">s3:HTTP an HTTP error occurred sending the request to S3</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">s3:REQUEST S3 refused to execute the request</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">s3:RESPONSE the response received from S3 cannot be parsed</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 0, "name" : "list-buckets", "qname" : "s3:list-buckets", "signature" : "() as object()", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Lists all buckets owned by the account of the default S3 credentials.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">This function is declared as deterministic and should be used whenever result\n caching is acceptable.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Example:\n <pre>\n s3:list-buckets()\n </pre>\n </p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Example return object:\n <pre>\n {\n   \"owner\" :\n   {\n     \"id\": \"dfe08489302934392afe39239fe953039d9e2af0c94\",\n     \"displayName\": \"28msec\"\n   },\n   \"buckets\" :\n   [\n     {\n       \"name\": \"28msec\",\n       \"creationDate\": \"2010-11-03T17:42:45.000Z\",\n     }\n   ]\n }\n </pre>\n </p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">The fields in the returned object have the following meanings:\n <ul>\n   <li>owner: an object which specifies the owner of the bucket (object,\n   mandatory). It has the following fields:\n   <ul>\n     <li>id: the user identifier of the bucket owner (string, mandatory).</li>\n     <li>displayName: the screen name of the bucket owner (string, mandatory).</li>\n   </ul>\n   </li>\n   <li>buckets: an array which contains an object for each owned bucket\n   (array, mandatory). Each object contains the following fields:\n   <ul>\n     <li>name: the bucket name (string, mandatory).</li>\n     <li>creationDate: the bucket creation date. (dateTime, mandatory).</li>\n   </ul>\n   </li>\n </ul>\n </p>\n", "summary" : "<p>  Lists all buckets owned by the account of the default S3 credentials.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [  ], "returns" : { "type" : "object()", "description" : "the owned buckets list" }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">s3:CREDENTIALS if the default S3 credentials cannot be found or are invalid</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">s3:HTTP an HTTP error occurred sending the request to S3</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">s3:REQUEST S3 refused to execute the request</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">s3:RESPONSE the response received from S3 cannot be parsed</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 1, "name" : "list-buckets", "qname" : "s3:list-buckets", "signature" : "($credentials as item()?) as object()", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Lists all buckets owned by the account of the specified S3 credentials.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">This function is declared as deterministic and should be used whenever result\n caching is acceptable.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Example:\n <pre>\n s3:list-buckets(\"credentials\")\n </pre>\n </p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Example return object:\n <pre>\n {\n   \"owner\" :\n   {\n     \"id\": \"dfe08489302934392afe39239fe953039d9e2af0c94\",\n     \"displayName\": \"28msec\"\n   },\n   \"buckets\" :\n   [\n     {\n       \"name\": \"28msec\",\n       \"creationDate\": \"2010-11-03T17:42:45.000Z\",\n     }\n   ]\n }\n </pre>\n </p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">The fields in the returned object have the following meanings:\n <ul>\n   <li>owner: an object which specifies the owner of the bucket (object,\n   mandatory). It has the following fields:\n   <ul>\n     <li>id: the user identifier of the bucket owner (string, mandatory).</li>\n     <li>displayName: the screen name of the bucket owner (string, mandatory).</li>\n   </ul>\n   </li>\n   <li>buckets: an array which contains an object for each owned bucket\n   (array, mandatory). Each object contains the following fields:\n   <ul>\n     <li>name: the bucket name (string, mandatory).</li>\n     <li>creationDate: the bucket creation date. (dateTime, mandatory).</li>\n   </ul>\n   </li>\n </ul>\n </p>\n", "summary" : "<p>  Lists all buckets owned by the account of the specified S3 credentials.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "credentials", "type" : "item()", "occurrence" : "?", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The credentials to use.</div>" } ], "returns" : { "type" : "object()", "description" : "the owned buckets list" }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">s3:CREDENTIALS if the specified S3 credentials cannot be found or are invalid</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">s3:HTTP an HTTP error occurred sending the request to S3</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">s3:REQUEST S3 refused to execute the request</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">s3:RESPONSE the response received from S3 cannot be parsed</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 1, "name" : "object-metadata-nondeterministic", "qname" : "s3:object-metadata-nondeterministic", "signature" : "($s3-object as item()) as object()", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Retrieves an object metadata using the default S3 credentials.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">This function has the same semantics as\n <a href=\"#object-metadata-1\">object-metadata#1</a>,\n but is declared as being non deterministic and thus should only be used when\n result caching is not desired.</p>\n", "summary" : "<p>  Retrieves an object metadata using the default S3 credentials.</p>", "annotation_str" : " %an:nondeterministic", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "nondeterministic", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "s3-object", "type" : "item()", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The S3 object or object version to retrieve the metadata for.</div>" } ], "returns" : { "type" : "object()", "description" : "the specified object metadata." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">s3:CREDENTIALS if the default S3 credentials cannot be found or are invalid</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">s3:OBJECT if the object specification is invalid</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">s3:BUCKET if no bucket has been specified and no default bucket is present in the used credentials</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">s3:HTTP an HTTP error occurred sending the request to S3</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">s3:REQUEST S3 refused to execute the request</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">s3:RESPONSE the response received from S3 cannot be parsed</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 2, "name" : "object-metadata-nondeterministic", "qname" : "s3:object-metadata-nondeterministic", "signature" : "($credentials as item()?, $s3-object as item()) as object()", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Retrieves an object metadata using the specified S3 credentials.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">This function has the same semantics as\n <a href=\"#object-metadata-2\">object-metadata#2</a>,\n but is declared as being non deterministic and thus should only be used when\n result caching is not desired.</p>\n", "summary" : "<p>  Retrieves an object metadata using the specified S3 credentials.</p>", "annotation_str" : " %an:nondeterministic", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "nondeterministic", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "credentials", "type" : "item()", "occurrence" : "?", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The credentials to use.</div>" }, { "name" : "s3-object", "type" : "item()", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The S3 object or object version to retrieve the metadata for.</div>" } ], "returns" : { "type" : "object()", "description" : "the specified object metadata." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">s3:CREDENTIALS if the specified S3 credentials cannot be found or are invalid</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">s3:OBJECT if the object specification is invalid</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">s3:BUCKET if no bucket has been specified and no default bucket is present in the used credentials</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">s3:HTTP an HTTP error occurred sending the request to S3</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">s3:REQUEST S3 refused to execute the request</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">s3:RESPONSE the response received from S3 cannot be parsed</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 1, "name" : "object-metadata", "qname" : "s3:object-metadata", "signature" : "($s3-object as item()) as object()", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Retrieves an object metadata using the default S3 credentials.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">This function is declared as deterministic and should be used whenever result\n caching is acceptable.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">The S3 object is specified through the $s3-object parameter. Either a string\n or a JSON object can be used. If a string is specified, it is interpreted as\n key for an object in the default bucket of the default credentials. If no default\n bucket is present in the default credentials, the <code>s3:BUCKET</code> error\n is raised. If a JSON object is used, it must have the following structure:\n <ul>\n   <li>key: the object key (string, mandatory).</li>\n   <li>bucket: the bucket name (string, optional). If this field is not present\n   and the default credentials do not have a default bucket, the\n   <code>s3:BUCKET</code> error is raised.</li>\n   <li>version: the object version (string, optional). If versioning is enabled\n   for the specified object and no version is specified the metadata of the\n   latest version of the object will be returned. If specified, only the\n   metadata of that particular version of the object will be returned.</li>\n </ul>\n </p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Example:\n <pre>\n let $object :=\n {\n   \"key\": \"object\",\n   \"bucket\": \"28msec\",\n   \"version\": \"3/L4kqtJlcpXroDTDmJ+rmSpXd3dIbrHY+MTRCxf3vjVBH40Nr8X8gdRQBpUMLUo\"\n }\n return s3:metadata-object($object)\n </pre>\n </p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Example return object:\n <pre>\n {\n   \"key\": \"test.xml\",\n   \"bucket\": \"28msec\",\n   \"metadata\":\n   {\n     \"author\": \"28msec\"\n   },\n   \"missing-metadata\" : 1,\n   \"version\": \"3GL4kqtJlcpXroDTDm3vjVBH40Nr8X8g\",\n   \"expiration\": \"expiry-date=\\\"Fri, 21 Dec 2012 00:00:00 GMT\\\", rule-id=\\\"Rule for testfile.txt\\\"\"\n   \"restore\": \"x-amz-restore: ongoing-request=\\\"false\\\", expiry-date=\\\"Wed, 07 Nov 2012 00:00:00 GMT\\\"\"\n   \"serverSideEncryption\": \"AES256\",\n   \"lastModified\": \"Mon, 15 Oct 2012 21:58:07 GMT\",\n   \"eTag\": \"1accb31fcf202eba0c0f41fa2f09b4d7\",\n   \"mediaType\": \"text/plain\"\n   \"length\": \"28\"\n }\n </pre>\n </p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">The fields in the returned object have the following meanings:\n <ul>\n   <li>key: the object key (string, mandatory).</li>\n   <li>bucket: the bucket name (string, mandatory).</li>\n   <li>metadata: an object specifying additional metadata. Each metadata is a\n   name-value pair and is represented as different fields (object, optional).</li>\n   <li>missing-metadata: the number of metadata entries that were not returned\n   in the metadata object. This can happen if you create metadata using an API\n   like SOAP that supports more flexible metadata than the REST API. (integer,\n   optional).</li>\n   <li>version: the object version (string, optional). If versioning is enabled\n   for the specified object and no version is specified the permissions for the\n   latest version of the object will be returned. If specified, only the\n   permissions for that particular version of the object will be returned.</li>\n   <li>expiration: if the object expiration is configured (see\n   <a href=\"http://docs.aws.amazon.com/AmazonS3/latest/API/RESTBucketPUTlifecycle.html\">PUT Bucket lifecycle</a>),\n   this field is present. It includes the expiry-date and rule-id key value\n   pairs providing object expiration information. The value of rule-id is URL\n   encoded.(string, optional).</li>\n   <li>restore: if the object is an archived object (an object whose storage\n   class is Glacier), the response includes this header if either the archive\n   restoration is in progress (see\n   <a href=\"http://docs.aws.amazon.com/AmazonS3/latest/API/RESTObjectPOSTrestore.html\">POST Object restore</a>),\n   or an archive copy is already restored.(string, optional).</li>\n   <li>serverSideEncryption: If the object is stored by using server-side\n   encryption, the response includes this header with a value of the encryption\n   algorithm that was used.(string, optional).</li>\n   <li>lastModified: the date in which the object was last modified (dateTime,\n   optional).</li>\n   <li>eTag: the entity tag is a hash of the object. The eTag only reflects\n   changes to the contents of an object, not its metadata. The eTag is determined\n   when an object is created. For objects created by the PUT Object operation\n   and the POST Object operation, the eTag is a quoted, 32-digit hexadecimal\n   string representing the MD5 digest of the object data. For other objects,\n   the eTag may or may not be an MD5 digest of the object data. If the eTag is\n   not an MD5 digest of the object data, it will contain one or more non-hexadecimal\n   characters and/or will consist of less than 32 or more than 32 hexadecimal\n   digits.(string, optional).</li>\n   <li>mediaType: the object media-type (string, mandatory).</li>\n   <li>length: the object content length (integer, optional).</li>\n </ul>\n </p>\n", "summary" : "<p>  Retrieves an object metadata using the default S3 credentials.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "s3-object", "type" : "item()", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The S3 object or object version to retrieve the metadata for.</div>" } ], "returns" : { "type" : "object()", "description" : "the specified object metadata." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">s3:CREDENTIALS if the default S3 credentials cannot be found or are invalid</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">s3:OBJECT if the object specification is invalid</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">s3:BUCKET if no bucket has been specified and no default bucket is present in the used credentials</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">s3:HTTP an HTTP error occurred sending the request to S3</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">s3:REQUEST S3 refused to execute the request</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">s3:RESPONSE the response received from S3 cannot be parsed</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 2, "name" : "object-metadata", "qname" : "s3:object-metadata", "signature" : "($credentials as item()?, $s3-object as item()) as object()", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Retrieves an object metadata using the specified S3 credentials.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">This function is declared as deterministic and should be used whenever result\n caching is acceptable.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">The S3 object is specified through the $s3-object parameter. Either a string\n or a JSON object can be used. If a string is specified, it is interpreted as\n key for an object in the default bucket of the specified credentials. If no default\n bucket is present in the specified credentials, the <code>s3:BUCKET</code> error\n is raised. If a JSON object is used, it must have the following structure:\n <ul>\n   <li>key: the object key (string, mandatory).</li>\n   <li>bucket: the bucket name (string, optional). If this field is not present\n   and the specified credentials do not have a default bucket, the\n   <code>s3:BUCKET</code> error is raised.</li>\n   <li>version: the object version (string, optional). If versioning is enabled\n   for the specified object and no version is specified the metadata of the\n   latest version of the object will be returned. If specified, only the\n   metadata of that particular version of the object will be returned.</li>\n </ul>\n </p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Example:\n <pre>\n let $object :=\n {\n   \"key\": \"object\",\n   \"bucket\": \"28msec\",\n   \"version\": \"3/L4kqtJlcpXroDTDmJ+rmSpXd3dIbrHY+MTRCxf3vjVBH40Nr8X8gdRQBpUMLUo\"\n }\n return s3:metadata-object(\"credentials\", $object)\n </pre>\n </p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Example return object:\n <pre>\n {\n   \"key\": \"test.xml\",\n   \"bucket\": \"28msec\",\n   \"metadata\":\n   {\n     \"author\": \"28msec\"\n   },\n   \"missing-metadata\" : 1,\n   \"version\": \"3GL4kqtJlcpXroDTDm3vjVBH40Nr8X8g\",\n   \"expiration\": \"expiry-date=\\\"Fri, 21 Dec 2012 00:00:00 GMT\\\", rule-id=\\\"Rule for testfile.txt\\\"\"\n   \"restore\": \"x-amz-restore: ongoing-request=\\\"false\\\", expiry-date=\\\"Wed, 07 Nov 2012 00:00:00 GMT\\\"\"\n   \"serverSideEncryption\": \"AES256\",\n   \"lastModified\": \"Mon, 15 Oct 2012 21:58:07 GMT\",\n   \"eTag\": \"1accb31fcf202eba0c0f41fa2f09b4d7\",\n   \"mediaType\": \"text/plain\"\n   \"length\": \"28\"\n }\n </pre>\n </p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">The fields in the returned object have the following meanings:\n <ul>\n   <li>key: the object key (string, mandatory).</li>\n   <li>bucket: the bucket name (string, mandatory).</li>\n   <li>metadata: an object specifying additional metadata. Each metadata is a\n   name-value pair and is represented as different fields (object, optional).</li>\n   <li>missing-metadata: the number of metadata entries that were not returned\n   in the metadata object. This can happen if you create metadata using an API\n   like SOAP that supports more flexible metadata than the REST API. (integer,\n   optional).</li>\n   <li>version: the object version (string, optional). If versioning is enabled\n   for the specified object and no version is specified the permissions for the\n   latest version of the object will be returned. If specified, only the\n   permissions for that particular version of the object will be returned.</li>\n   <li>expiration: if the object expiration is configured (see\n   <a href=\"http://docs.aws.amazon.com/AmazonS3/latest/API/RESTBucketPUTlifecycle.html\">PUT Bucket lifecycle</a>),\n   this field is present. It includes the expiry-date and rule-id key value\n   pairs providing object expiration information. The value of rule-id is URL\n   encoded.(string, optional).</li>\n   <li>restore: if the object is an archived object (an object whose storage\n   class is Glacier), the response includes this header if either the archive\n   restoration is in progress (see\n   <a href=\"http://docs.aws.amazon.com/AmazonS3/latest/API/RESTObjectPOSTrestore.html\">POST Object restore</a>),\n   or an archive copy is already restored.(string, optional).</li>\n   <li>serverSideEncryption: If the object is stored by using server-side\n   encryption, the response includes this header with a value of the encryption\n   algorithm that was used.(string, optional).</li>\n   <li>lastModified: the date in which the object was last modified (dateTime,\n   optional).</li>\n   <li>eTag: the entity tag is a hash of the object. The eTag only reflects\n   changes to the contents of an object, not its metadata. The eTag is determined\n   when an object is created. For objects created by the PUT Object operation\n   and the POST Object operation, the eTag is a quoted, 32-digit hexadecimal\n   string representing the MD5 digest of the object data. For other objects,\n   the eTag may or may not be an MD5 digest of the object data. If the eTag is\n   not an MD5 digest of the object data, it will contain one or more non-hexadecimal\n   characters and/or will consist of less than 32 or more than 32 hexadecimal\n   digits.(string, optional).</li>\n   <li>mediaType: the object media-type (string, mandatory).</li>\n   <li>length: the object content length (integer, optional).</li>\n </ul>\n </p>\n", "summary" : "<p>  Retrieves an object metadata using the specified S3 credentials.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "credentials", "type" : "item()", "occurrence" : "?", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The credentials to use.</div>" }, { "name" : "s3-object", "type" : "item()", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The S3 object or object version to retrieve the metadata for.</div>" } ], "returns" : { "type" : "object()", "description" : "the specified object metadata." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">s3:CREDENTIALS if the specified S3 credentials cannot be found or are invalid</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">s3:OBJECT if the object specification is invalid</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">s3:BUCKET if no bucket has been specified and no default bucket is present in the used credentials</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">s3:HTTP an HTTP error occurred sending the request to S3</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">s3:REQUEST S3 refused to execute the request</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">s3:RESPONSE the response received from S3 cannot be parsed</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 1, "name" : "object-permissions-nondeterministic", "qname" : "s3:object-permissions-nondeterministic", "signature" : "($s3-object as item()) as object()", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Retrieves the access control list (ACL) of an S3 object using the default\n S3 credentials.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">This function has the same semantics as\n <a href=\"#object-permissions-1\">object-permissions#1</a>,\n but is declared as being non deterministic and thus should only be used when\n result caching is not desired.</p>\n", "summary" : "<p>  Retrieves the access control list (ACL) of an S3 object using the default\n S3 credentials.</p>", "annotation_str" : " %an:nondeterministic", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "nondeterministic", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "s3-object", "type" : "item()", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The S3 object or object version to retrieve the ACL for.</div>" } ], "returns" : { "type" : "object()", "description" : "the object ACL" }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">s3:CREDENTIALS if the default S3 credentials cannot be found or are invalid</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">s3:OBJECT if the object specification is invalid</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">s3:BUCKET if no bucket has been specified and no default bucket is present in the used credentials</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">s3:HTTP an HTTP error occurred sending the request to S3</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">s3:REQUEST S3 refused to execute the request</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">s3:RESPONSE the response received from S3 cannot be parsed</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 2, "name" : "object-permissions-nondeterministic", "qname" : "s3:object-permissions-nondeterministic", "signature" : "($credentials as item()?, $s3-object as item()) as object()", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Retrieves the access control list (ACL) of an S3 object using the specified\n credentials.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">This function has the same semantics as\n <a href=\"#object-permissions-2\">object-permissions#2</a>,\n but is declared as being non deterministic and thus should only be used when\n result caching is not desired.</p>\n", "summary" : "<p>  Retrieves the access control list (ACL) of an S3 object using the specified\n credentials.</p>", "annotation_str" : " %an:nondeterministic", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "nondeterministic", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "credentials", "type" : "item()", "occurrence" : "?", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The credentials to use.</div>" }, { "name" : "s3-object", "type" : "item()", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The S3 object or object version to retrieve the ACL for.</div>" } ], "returns" : { "type" : "object()", "description" : "the object ACL" }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">s3:CREDENTIALS if the specified S3 credentials cannot be found or are invalid</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">s3:OBJECT if the object specification is invalid</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">s3:BUCKET if no bucket has been specified and no default bucket is present in the used credentials</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">s3:HTTP an HTTP error occurred sending the request to S3</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">s3:REQUEST S3 refused to execute the request</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">s3:RESPONSE the response received from S3 cannot be parsed</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 1, "name" : "object-permissions", "qname" : "s3:object-permissions", "signature" : "($s3-object as item()) as object()", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Retrieves the access control list (ACL) of an S3 object using the default\n S3 credentials.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">This function is declared as deterministic and should be used whenever result\n caching is acceptable.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">The S3 object is specified through the $s3-object parameter. Either a string\n or a JSON object can be used. If a string is specified, it is interpreted as\n key for an object in the default bucket of the default credentials. If no default\n bucket is present in the default credentials, the <code>s3:BUCKET</code> error\n is raised. If a JSON object is used, it must have the following structure:\n <ul>\n   <li>key: the object key (string, mandatory).</li>\n   <li>bucket: the bucket name (string, optional). If this field is not present\n   and the default credentials do not have a default bucket, the\n   <code>s3:BUCKET</code> error is raised.</li>\n   <li>version: the object version (string, optional). If versioning is enabled\n   for the specified object and no version is specified the permissions for the\n   latest version of the object will be returned. If specified, only the\n   permissions for that particular version of the object will be returned.</li>\n </ul>\n </p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Example:\n <pre>\n let $object :=\n {\n   \"key\": \"object\",\n   \"bucket\": \"28msec\",\n   \"version\": \"3/L4kqtJlcpXroDTDmJ+rmSpXd3dIbrHY+MTRCxf3vjVBH40Nr8X8gdRQBpUMLUo\"\n }\n return s3:object-permissions($object)\n </pre>\n </p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Example return object:\n <pre>\n {\n   \"owner\" :\n   {\n     \"id\" : \"ea3617f5543e366a51bf51f440c221410b8001c34744e9d3a81acb79cff9d2ed\",\n     \"displayName\" : \"28msec\"\n   },\n   \"acl\" :\n   [\n     {\n       \"grantee\" :\n       {\n         \"type\" : \"CanonicalUser\",\n         \"id\" : \"ea3617f5543e366a51bf51f440c221410b8001c34744e9d3a81acb79cff9d2ed\",\n         \"displayName\" : \"28msec\"\n       },\n       \"permission\" : \"FULL_CONTROL\"\n     }\n   ]\n }\n </pre>\n </p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">The fields in the returned object have the following meanings:\n <ul>\n <li>owner: an object which specifies the owner of the object (object,\n mandatory). It has the following fields:\n   <ul>\n     <li>id: the user identifier of the object owner (string, mandatory).</li>\n     <li>displayName: the screen name of the object owner (string, mandatory).</li>\n   </ul>\n </li>\n <li>acl: an array which contains an object for each grantee with the corresponding\n ACL settings (array, mandatory). Each object contains the following fields:\n   <ul>\n     <li>grantee: specifies the permissions grantee (object, mandatory). It\n     contains the following fields:\n       <ul>\n         <li>type: one of \"AmazonCustomerByEmail\", \"CanonicalUser\" or \"Group\".</li>\n         <li>emailAddress: the customer email address (string, present in all\n         and only AmazonCustomerByEmail grantees).</li>\n         <li>id: the user identifier (string, present in all and only CanonicalUser\n         grantees).</li>\n         <li>displayName: the user screen name (string, present in all and only\n         CanonicalUser grantees).</li>\n         <li>URI: the group URI (string, present in all and only Group grantees).</li>\n       </ul>\n     </li>\n     <li>permission: the granted permissions. One of \"READ\", \"WRITE\", \"READ_ACP\",\n     \"WRITE_ACP\", \"FULL_CONTROL\". (string, mandatory).</li>\n   </ul>\n </li>\n </ul>\n </p>\n", "summary" : "<p>  Retrieves the access control list (ACL) of an S3 object using the default\n S3 credentials.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "s3-object", "type" : "item()", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The S3 object or object version to retrieve the ACL for.</div>" } ], "returns" : { "type" : "object()", "description" : "the object ACL" }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">s3:CREDENTIALS if the default S3 credentials cannot be found or are invalid</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">s3:OBJECT if the object specification is invalid</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">s3:BUCKET if no bucket has been specified and no default bucket is present in the used credentials</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">s3:HTTP an HTTP error occurred sending the request to S3</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">s3:REQUEST S3 refused to execute the request</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">s3:RESPONSE the response received from S3 cannot be parsed</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 2, "name" : "object-permissions", "qname" : "s3:object-permissions", "signature" : "($credentials as item()?, $s3-object as item()) as object()", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Retrieves the access control list (ACL) of an S3 object using the specified\n credentials.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">This function is declared as deterministic and should be used whenever result\n caching is acceptable.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">The S3 object is specified through the $s3-object parameter. Either a string\n or a JSON object can be used. If a string is specified, it is interpreted as\n key for an object in the default bucket of the specified credentials. If no default\n bucket is present in the specified credentials, the <code>s3:BUCKET</code> error\n is raised. If a JSON object is used, it must have the following structure:\n <ul>\n   <li>key: the object key (string, mandatory).</li>\n   <li>bucket: the bucket name (string, optional). If this field is not present\n   and the specified credentials does not have a default bucket, the\n   <code>s3:BUCKET</code> error is raised.</li>\n   <li>version: the object version (string, optional). If versioning is enabled\n   for the specified object and no version is specified the permissions for the\n   latest version of the object will be returned. If specified, only the\n   permissions for that particular version of the object will be returned.</li>\n </ul>\n </p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Example:\n <pre>\n let $object :=\n {\n   \"key\": \"object\",\n   \"bucket\": \"28msec\",\n   \"version\": \"3/L4kqtJlcpXroDTDmJ+rmSpXd3dIbrHY+MTRCxf3vjVBH40Nr8X8gdRQBpUMLUo\"\n }\n return s3:object-permissions(\"credentials\", $object)\n </pre>\n </p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Example return object:\n <pre>\n {\n   \"owner\" :\n   {\n     \"id\" : \"ea3617f5543e366a51bf51f440c221410b8001c34744e9d3a81acb79cff9d2ed\",\n     \"displayName\" : \"28msec\"\n   },\n   \"acl\" :\n   [\n     {\n       \"grantee\" :\n       {\n         \"type\" : \"CanonicalUser\",\n         \"id\" : \"ea3617f5543e366a51bf51f440c221410b8001c34744e9d3a81acb79cff9d2ed\",\n         \"displayName\" : \"28msec\"\n       },\n       \"permission\" : \"FULL_CONTROL\"\n     }\n   ]\n }\n </pre>\n </p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">The fields in the returned object have the following meanings:\n <ul>\n <li>owner: an object which specifies the owner of the object (object,\n mandatory). It has the following fields:\n   <ul>\n     <li>id: the user identifier of the object owner (string, mandatory).</li>\n     <li>displayName: the screen name of the object owner (string, mandatory).</li>\n   </ul>\n </li>\n <li>acl: an array which contains an object for each grantee with the corresponding\n ACL settings (array, mandatory). Each object contains the following fields:\n   <ul>\n     <li>grantee: specifies the permissions grantee (object, mandatory). It contains\n     the following fields:\n       <ul>\n         <li>type: one of \"AmazonCustomerByEmail\", \"CanonicalUser\" or \"Group\".</li>\n         <li>emailAddress: the customer email address (string, present in all\n         and only AmazonCustomerByEmail grantees).</li>\n         <li>id: the user identifier (string, present in all and only CanonicalUser\n         grantees).</li>\n         <li>displayName: the user screen name (string, present in all and only\n         CanonicalUser grantees).</li>\n         <li>URI: the group URI (string, present in all and only Group grantees).</li>\n       </ul>\n     </li>\n     <li>permission: the granted permissions. One of \"READ\", \"WRITE\", \"READ_ACP\",\n     \"WRITE_ACP\", \"FULL_CONTROL\". (string, mandatory).</li>\n   </ul>\n </li>\n </ul>\n </p>\n", "summary" : "<p>  Retrieves the access control list (ACL) of an S3 object using the specified\n credentials.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "credentials", "type" : "item()", "occurrence" : "?", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The credentials to use.</div>" }, { "name" : "s3-object", "type" : "item()", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The S3 object or object version to retrieve the ACL for.</div>" } ], "returns" : { "type" : "object()", "description" : "the object ACL" }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">s3:CREDENTIALS if the specified S3 credentials cannot be found or are invalid</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">s3:OBJECT if the object specification is invalid</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">s3:BUCKET if no bucket has been specified and no default bucket is present in the used credentials</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">s3:HTTP an HTTP error occurred sending the request to S3</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">s3:REQUEST S3 refused to execute the request</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">s3:RESPONSE the response received from S3 cannot be parsed</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 1, "name" : "object-torrent-nondeterministic", "qname" : "s3:object-torrent-nondeterministic", "signature" : "($s3-object as item()) as base64Binary", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Returns a torrent file for a given object using the default S3 credentials.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">This function has the same semantics as\n <a href=\"#object-torrent-1\">object-torrent#1</a>,\n but is declared as being non deterministic and thus should only be used when\n result caching is not desired.</p>\n", "summary" : "<p>  Returns a torrent file for a given object using the default S3 credentials.</p>", "annotation_str" : " %an:nondeterministic", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "nondeterministic", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "s3-object", "type" : "item()", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The S3 object to retrieve the torrent file for</div>" } ], "returns" : { "type" : "base64Binary", "description" : "the base64 encoded torrent file for the specified object." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">s3:CREDENTIALS if the default S3 credentials cannot be found or are invalid</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">s3:OBJECT if the object specification is invalid</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">s3:BUCKET if no bucket has been specified and no default bucket is present in the used credentials</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">s3:HTTP an HTTP error occurred sending the request to S3</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">s3:REQUEST S3 refused to execute the request</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">s3:RESPONSE the response received from S3 cannot be parsed</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 2, "name" : "object-torrent-nondeterministic", "qname" : "s3:object-torrent-nondeterministic", "signature" : "($credentials as item()?, $s3-object as item()) as base64Binary", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Returns a torrent file for a given object using the specified S3 credentials.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">This function has the same semantics as\n <a href=\"#object-torrent-2\">object-torrent#2</a>,\n but is declared as being non deterministic and thus should only be used when\n result caching is not desired.</p>\n", "summary" : "<p>  Returns a torrent file for a given object using the specified S3 credentials.</p>", "annotation_str" : " %an:nondeterministic", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "nondeterministic", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "credentials", "type" : "item()", "occurrence" : "?", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The credentials to use.</div>" }, { "name" : "s3-object", "type" : "item()", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The S3 object to retrieve the torrent file for</div>" } ], "returns" : { "type" : "base64Binary", "description" : "the base64 encoded torrent file of the specified object." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">s3:CREDENTIALS if the specified S3 credentials cannot be found or are invalid</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">s3:OBJECT if the object specification is invalid</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">s3:BUCKET if no bucket has been specified and no default bucket is present in the used credentials</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">s3:HTTP an HTTP error occurred sending the request to S3</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">s3:REQUEST S3 refused to execute the request</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">s3:RESPONSE the response received from S3 cannot be parsed</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 1, "name" : "object-torrent", "qname" : "s3:object-torrent", "signature" : "($s3-object as item()) as base64Binary", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Returns a torrent file for a given object using the default S3 credentials.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">This function is declared as deterministic and should be used whenever result\n caching is acceptable.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">The S3 object is specified through the $s3-object parameter. Either a string\n or a JSON object can be used. If a string is specified, it is interpreted as\n key for an object in the default bucket of the default credentials. If no default\n bucket is present in the default credentials, the <code>s3:BUCKET</code> error\n is raised. If a JSON object is used, it must have the following structure:\n <ul>\n   <li>key: the object key (string, mandatory).</li>\n   <li>bucket: the bucket name (string, optional). If this field is not present\n   and the default credentials do not have a default bucket, the\n   <code>s3:BUCKET</code> error is raised.</li>\n </ul>\n </p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Example:\n <pre>\n let $object :=\n {\n   \"key\": \"object\",\n   \"bucket\": \"28msec\"\n }\n return s3:object-torrent($object)\n </pre>\n </p>\n", "summary" : "<p>  Returns a torrent file for a given object using the default S3 credentials.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "s3-object", "type" : "item()", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The S3 object to retrieve the torrent file for</div>" } ], "returns" : { "type" : "base64Binary", "description" : "the base64 encoded torrent file for the specified object." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">s3:CREDENTIALS if the default S3 credentials cannot be found or are invalid</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">s3:OBJECT if the object specification is invalid</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">s3:BUCKET if no bucket has been specified and no default bucket is present in the used credentials</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">s3:HTTP an HTTP error occurred sending the request to S3</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">s3:REQUEST S3 refused to execute the request</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">s3:RESPONSE the response received from S3 cannot be parsed</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 2, "name" : "object-torrent", "qname" : "s3:object-torrent", "signature" : "($credentials as item()?, $s3-object as item()) as base64Binary", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Returns a torrent file for a given object using the specified S3 credentials.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">This function is declared as deterministic and should be used whenever result\n caching is acceptable.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">The S3 object is specified through the $s3-object parameter. Either a string\n or a JSON object can be used. If a string is specified, it is interpreted as\n key for an object in the default bucket of the specified credentials. If no default\n bucket is present in the specified credentials, the <code>s3:BUCKET</code> error\n is raised. If a JSON object is used, it must have the following structure:\n <ul>\n   <li>key: the object key (string, mandatory).</li>\n   <li>bucket: the bucket name (string, optional). If this field is not present\n   and the specified credentials do not have a default bucket, the\n   <code>s3:BUCKET</code> error is raised.</li>\n </ul>\n </p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Example:\n <pre>\n let $object :=\n {\n   \"key\": \"object\",\n   \"bucket\": \"28msec\"\n }\n return s3:object-torrent($object)\n </pre>\n </p>\n", "summary" : "<p>  Returns a torrent file for a given object using the specified S3 credentials.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "credentials", "type" : "item()", "occurrence" : "?", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The credentials to use.</div>" }, { "name" : "s3-object", "type" : "item()", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The S3 object to retrieve the torrent file for</div>" } ], "returns" : { "type" : "base64Binary", "description" : "the base64 encoded torrent file of the specified object." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">s3:CREDENTIALS if the specified S3 credentials cannot be found or are invalid</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">s3:OBJECT if the object specification is invalid</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">s3:BUCKET if no bucket has been specified and no default bucket is present in the used credentials</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">s3:HTTP an HTTP error occurred sending the request to S3</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">s3:REQUEST S3 refused to execute the request</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">s3:RESPONSE the response received from S3 cannot be parsed</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 1, "name" : "read-binary-nondeterministic", "qname" : "s3:read-binary-nondeterministic", "signature" : "($s3-object as item()) as base64Binary", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Retrieves the binary content of an S3 object using the default S3 credentials.\n The object content is forced to be interpreted as binaryand will be returned\n as a base64Binary item.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">This function has the same semantics as\n <a href=\"#read-binary-1\">read-binary#1</a>,\n but is declared as being non deterministic and thus should only be used when\n result caching is not desired.</p>\n", "summary" : "<p>  Retrieves the binary content of an S3 object using the default S3 credentials.</p>", "annotation_str" : " %an:nondeterministic", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "nondeterministic", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "s3-object", "type" : "item()", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The S3 object or object version to retrieve.</div>" } ], "returns" : { "type" : "base64Binary", "description" : "the object or object version binary content." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">s3:CREDENTIALS if the default S3 credentials cannot be found or are invalid</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">s3:OBJECT if the object specification is invalid</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">s3:BUCKET if no bucket has been specified and no default bucket is present in the used credentials</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">s3:HTTP an HTTP error occurred sending the request to S3</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">s3:REQUEST S3 refused to execute the request</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 2, "name" : "read-binary-nondeterministic", "qname" : "s3:read-binary-nondeterministic", "signature" : "($credentials as item()?, $s3-object as item()) as base64Binary", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Retrieves the binary content of an S3 object using the specified S3 credentials.\n The object content is forced to be interpreted as binaryand will be returned\n as a base64Binary item.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">This function has the same semantics as\n <a href=\"#read-binary-2\">read-binary#2</a>,\n but is declared as being non deterministic and thus should only be used when\n result caching is not desired.</p>\n", "summary" : "<p>  Retrieves the binary content of an S3 object using the specified S3 credentials.</p>", "annotation_str" : " %an:nondeterministic", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "nondeterministic", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "credentials", "type" : "item()", "occurrence" : "?", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The credentials to use.</div>" }, { "name" : "s3-object", "type" : "item()", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The S3 object or object version to retrieve.</div>" } ], "returns" : { "type" : "base64Binary", "description" : "the object or object version binary content." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">s3:CREDENTIALS if the specified S3 credentials cannot be found or are invalid</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">s3:OBJECT if the object specification is invalid</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">s3:BUCKET if no bucket has been specified and no default bucket is present in the used credentials</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">s3:HTTP an HTTP error occurred sending the request to S3</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">s3:REQUEST S3 refused to execute the request</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 1, "name" : "read-binary", "qname" : "s3:read-binary", "signature" : "($s3-object as item()) as base64Binary", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Retrieves the binary content of an S3 object using the default S3 credentials.\n The object content is forced to be interpreted as binaryand will be returned\n as a base64Binary item.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">This function is declared as deterministic and should be used whenever result\n caching is acceptable.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">The S3 object is specified through the $s3-object parameter. Either a string\n or a JSON object can be used. If a string is specified, it is interpreted as\n key for an object in the default bucket of the default credentials. If no default\n bucket is present in the default credentials, the <code>s3:BUCKET</code> error\n is raised. If a JSON object is used, it must have the following structure:\n <ul>\n   <li>key: the object key (string, mandatory).</li>\n   <li>bucket: the bucket name (string, optional). If this field is not present\n   and the default credentials do not have a default bucket, the\n   <code>s3:BUCKET</code> error is raised.</li>\n   <li>version: the object version (string, optional). If versioning is enabled\n   for the specified object and no version is specified the latest version of\n   the object will be returned. If specified, only that particular version of\n   the object will be returned.</li>\n </ul>\n </p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Example:\n <pre>\n let $object :=\n {\n   \"key\": \"object\",\n   \"bucket\": \"28msec\",\n   \"version\": \"3/L4kqtJlcpXroDTDmJ+rmSpXd3dIbrHY+MTRCxf3vjVBH40Nr8X8gdRQBpUMLUo\"\n }\n return s3:read-binary($object)\n </pre>\n </p>\n", "summary" : "<p>  Retrieves the binary content of an S3 object using the default S3 credentials.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "s3-object", "type" : "item()", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The S3 object or object version to retrieve.</div>" } ], "returns" : { "type" : "base64Binary", "description" : "the object or object version binary content." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">s3:CREDENTIALS if the default S3 credentials cannot be found or are invalid</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">s3:OBJECT if the object specification is invalid</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">s3:BUCKET if no bucket has been specified and no default bucket is present in the used credentials</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">s3:HTTP an HTTP error occurred sending the request to S3</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">s3:REQUEST S3 refused to execute the request</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 2, "name" : "read-binary", "qname" : "s3:read-binary", "signature" : "($credentials as item()?, $s3-object as item()) as base64Binary", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Retrieves the binary content of an S3 object using the specified S3 credentials.\n The object content is forced to be interpreted as binaryand will be returned\n as a base64Binary item.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">This function is declared as deterministic and should be used whenever result\n caching is acceptable.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">The S3 object is specified through the $s3-object parameter. Either a string\n or a JSON object can be used. If a string is specified, it is interpreted as\n key for an object in the default bucket of the specified credentials. If no default\n bucket is present in the specified credentials, the <code>s3:BUCKET</code> error\n is raised. If a JSON object is used, it must have the following structure:\n <ul>\n   <li>key: the object key (string, mandatory).</li>\n   <li>bucket: the bucket name (string, optional). If this field is not present\n   and the specified credentials do not have a default bucket, the\n   <code>s3:BUCKET</code> error is raised.</li>\n   <li>version: the object version (string, optional). If versioning is enabled\n   for the specified object and no version is specified the latest version of\n   the object will be returned. If specified, only that particular version of\n   the object will be returned.</li>\n </ul>\n </p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Example:\n <pre>\n let $object :=\n {\n   \"key\": \"object\",\n   \"bucket\": \"28msec\",\n   \"version\": \"3/L4kqtJlcpXroDTDmJ+rmSpXd3dIbrHY+MTRCxf3vjVBH40Nr8X8gdRQBpUMLUo\"\n }\n return s3:read-binary(\"credentials\", $object)\n </pre>\n </p>\n", "summary" : "<p>  Retrieves the binary content of an S3 object using the specified S3 credentials.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "credentials", "type" : "item()", "occurrence" : "?", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The credentials to use.</div>" }, { "name" : "s3-object", "type" : "item()", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The S3 object or object version to retrieve.</div>" } ], "returns" : { "type" : "base64Binary", "description" : "the object or object version binary content." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">s3:CREDENTIALS if the specified S3 credentials cannot be found or are invalid</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">s3:OBJECT if the object specification is invalid</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">s3:BUCKET if no bucket has been specified and no default bucket is present in the used credentials</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">s3:HTTP an HTTP error occurred sending the request to S3</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">s3:REQUEST S3 refused to execute the request</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 1, "name" : "read-object-nondeterministic", "qname" : "s3:read-object-nondeterministic", "signature" : "($s3-object as item()) as object()", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Retrieves an S3 object using the default S3 credentials.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">This function has the same semantics as\n <a href=\"#read-object-1\">read-object#1</a>,\n but is declared as being non deterministic and thus should only be used when\n result caching is not desired.</p>\n", "summary" : "<p>  Retrieves an S3 object using the default S3 credentials.</p>", "annotation_str" : " %an:nondeterministic", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "nondeterministic", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "s3-object", "type" : "item()", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The S3 object or object version to retrieve.</div>" } ], "returns" : { "type" : "object()", "description" : "the specified object." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">s3:CREDENTIALS if the default S3 credentials cannot be found or are invalid</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">s3:OBJECT if the object specification is invalid</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">s3:BUCKET if no bucket has been specified and no default bucket is present in the used credentials</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">s3:HTTP an HTTP error occurred sending the request to S3</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">s3:REQUEST S3 refused to execute the request</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">s3:RESPONSE the response received from S3 cannot be parsed</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 2, "name" : "read-object-nondeterministic", "qname" : "s3:read-object-nondeterministic", "signature" : "($credentials as item()?, $s3-object as item()) as object()", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Retrieves an S3 object using the specified S3 credentials.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">This function has the same semantics as\n <a href=\"#read-object-2\">read-object#2</a>,\n but is declared as being non deterministic and thus should only be used when\n result caching is not desired.</p>\n", "summary" : "<p>  Retrieves an S3 object using the specified S3 credentials.</p>", "annotation_str" : " %an:nondeterministic", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "nondeterministic", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "credentials", "type" : "item()", "occurrence" : "?", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The credentials to use.</div>" }, { "name" : "s3-object", "type" : "item()", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The S3 object or object version to retrieve.</div>" } ], "returns" : { "type" : "object()", "description" : "the specified object." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">s3:CREDENTIALS if the specified S3 credentials cannot be found or are invalid</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">s3:OBJECT if the object specification is invalid</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">s3:BUCKET if no bucket has been specified and no default bucket is present in the used credentials</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">s3:HTTP an HTTP error occurred sending the request to S3</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">s3:REQUEST S3 refused to execute the request</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">s3:RESPONSE the response received from S3 cannot be parsed</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 1, "name" : "read-object", "qname" : "s3:read-object", "signature" : "($s3-object as item()) as object()", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Retrieves an S3 object using the default S3 credentials.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">This function is declared as deterministic and should be used whenever result\n caching is acceptable.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">The S3 object is specified through the $s3-object parameter. Either a string\n or a JSON object can be used. If a string is specified, it is interpreted as\n key for an object in the default bucket of the default credentials. If no default\n bucket is present in the default credentials, the <code>s3:BUCKET</code> error\n is raised. If a JSON object is used, it must have the following structure:\n <ul>\n   <li>key: the object key (string, mandatory).</li>\n   <li>bucket: the bucket name (string, optional). If this field is not present\n   and the default credentials do not have a default bucket, the\n   <code>s3:BUCKET</code> error is raised.</li>\n   <li>version: the object version (string, optional). If versioning is enabled\n   for the specified object and no version is specified the latest version of\n   the object will be returned. If specified, only that particular version of\n   the object will be returned.</li>\n </ul>\n </p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Example:\n <pre>\n let $object :=\n {\n   \"key\": \"object\",\n   \"bucket\": \"28msec\",\n   \"version\": \"3/L4kqtJlcpXroDTDmJ+rmSpXd3dIbrHY+MTRCxf3vjVBH40Nr8X8gdRQBpUMLUo\"\n }\n return s3:read-object($object)\n </pre>\n </p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Example return object:\n <pre>\n {\n   \"key\": \"test.xml\",\n   \"bucket\": \"28msec\",\n   \"metadata\":\n   {\n     \"author\": \"28msec\"\n   },\n   \"missing-metadata\" : 1,\n   \"version\": \"3GL4kqtJlcpXroDTDm3vjVBH40Nr8X8g\",\n   \"expiration\": \"expiry-date=\\\"Fri, 21 Dec 2012 00:00:00 GMT\\\", rule-id=\\\"Rule for testfile.txt\\\"\"\n   \"restore\": \"x-amz-restore: ongoing-request=\\\"false\\\", expiry-date=\\\"Wed, 07 Nov 2012 00:00:00 GMT\\\"\"\n   \"serverSideEncryption\": \"AES256\",\n   \"lastModified\": \"Mon, 15 Oct 2012 21:58:07 GMT\",\n   \"eTag\": \"1accb31fcf202eba0c0f41fa2f09b4d7\",\n   \"mediaType\": \"text/plain\"\n   \"length\": \"28\",\n   \"content: \"1234567890123456789012345678\"\n }\n </pre>\n </p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">The fields in the returned object have the following meanings:\n <ul>\n   <li>key: the object key (string, mandatory).</li>\n   <li>bucket: the bucket name (string, mandatory).</li>\n   <li>metadata: an object specifying additional metadata. Each metadata is a\n   name-value pair and is represented as different fields (object, optional).</li>\n   <li>missing-metadata: the number of metadata entries that were not returned\n   in the metadata object. This can happen if you create metadata using an API\n   like SOAP that supports more flexible metadata than the REST API. (integer,\n   optional).</li>\n   <li>version: the object version (string, optional). If versioning is enabled\n   for the specified object and no version is specified the permissions for the\n   latest version of the object will be returned. If specified, only the\n   permissions for that particular version of the object will be returned.</li>\n   <li>expiration: if the object expiration is configured (see\n   <a href=\"http://docs.aws.amazon.com/AmazonS3/latest/API/RESTBucketPUTlifecycle.html\">PUT Bucket lifecycle</a>),\n   this field is present. It includes the expiry-date and rule-id key value\n   pairs providing object expiration information. The value of rule-id is URL\n   encoded.(string, optional).</li>\n   <li>restore: if the object is an archived object (an object whose storage\n   class is Glacier), the response includes this header if either the archive\n   restoration is in progress (see\n   <a href=\"http://docs.aws.amazon.com/AmazonS3/latest/API/RESTObjectPOSTrestore.html\">POST Object restore</a>),\n   or an archive copy is already restored.(string, optional).</li>\n   <li>serverSideEncryption: If the object is stored by using server-side\n   encryption, the response includes this header with a value of the encryption\n   algorithm that was used.(string, optional).</li>\n   <li>lastModified: the date in which the object was last modified (dateTime,\n   optional).</li>\n   <li>eTag: the entity tag is a hash of the object. The eTag only reflects\n   changes to the contents of an object, not its metadata. The eTag is determined\n   when an object is created. For objects created by the PUT Object operation\n   and the POST Object operation, the eTag is a quoted, 32-digit hexadecimal\n   string representing the MD5 digest of the object data. For other objects,\n   the eTag may or may not be an MD5 digest of the object data. If the eTag is\n   not an MD5 digest of the object data, it will contain one or more non-hexadecimal\n   characters and/or will consist of less than 32 or more than 32 hexadecimal\n   digits.(string, optional).</li>\n   <li>mediaType: the object media-type (string, mandatory).</li>\n   <li>length: the object content length (integer, optional).</li>\n   <li>content: the object content. (item, optional). The type of this field\n   is determined by the media-type returned by the server. If the media-type\n   indicates that the body content is textual, then the content has type string,\n   base64Binary otherwise. Specifically, the body content is considered textual\n   if and only if the MIME-type specified in the media-type is one of:\n   <ul>\n     <li>\"application/json\"</li>\n     <li>\"application/x-javascript\"</li>\n     <li>\"application/xml\"</li>\n     <li>\"application/xml-external-parsed-entity\"</li>\n   </ul>\n   or if the MIME-type starts with \"text/\" or ends with \"+xml\".</li>\n </ul>\n </p>\n", "summary" : "<p>  Retrieves an S3 object using the default S3 credentials.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "s3-object", "type" : "item()", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The S3 object or object version to retrieve.</div>" } ], "returns" : { "type" : "object()", "description" : "the specified object." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">s3:CREDENTIALS if the default S3 credentials cannot be found or are invalid</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">s3:OBJECT if the object specification is invalid</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">s3:BUCKET if no bucket has been specified and no default bucket is present in the used credentials</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">s3:HTTP an HTTP error occurred sending the request to S3</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">s3:REQUEST S3 refused to execute the request</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">s3:RESPONSE the response received from S3 cannot be parsed</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 2, "name" : "read-object", "qname" : "s3:read-object", "signature" : "($credentials as item()?, $s3-object as item()) as object()", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Retrieves an S3 object using the specified S3 credentials.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">This function is declared as deterministic and should be used whenever result\n caching is acceptable.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">The S3 object is specified through the $s3-object parameter. Either a string\n or a JSON object can be used. If a string is specified, it is interpreted as\n key for an object in the default bucket of the specified credentials. If no default\n bucket is present in the specified credentials, the <code>s3:BUCKET</code> error\n is raised. If a JSON object is used, it must have the following structure:\n <ul>\n   <li>key: the object key (string, mandatory).</li>\n   <li>bucket: the bucket name (string, optional). If this field is not present\n   and the specified credentials do not have a default bucket, the\n   <code>s3:BUCKET</code> error is raised.</li>\n   <li>version: the object version (string, optional). If versioning is enabled\n   for the specified object and no version is specified the latest version of\n   the object will be returned. If specified, only that particular version of\n   the object will be returned.</li>\n </ul>\n </p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Example:\n <pre>\n let $object :=\n {\n   \"key\": \"object\",\n   \"bucket\": \"28msec\",\n   \"version\": \"3/L4kqtJlcpXroDTDmJ+rmSpXd3dIbrHY+MTRCxf3vjVBH40Nr8X8gdRQBpUMLUo\"\n }\n return s3:read-object($object)\n </pre>\n </p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Example return object:\n <pre>\n {\n   \"key\": \"test.xml\",\n   \"bucket\": \"28msec\",\n   \"metadata\":\n   {\n     \"author\": \"28msec\"\n   },\n   \"missing-metadata\" : 1,\n   \"version\": \"3GL4kqtJlcpXroDTDm3vjVBH40Nr8X8g\",\n   \"expiration\": \"expiry-date=\\\"Fri, 21 Dec 2012 00:00:00 GMT\\\", rule-id=\\\"Rule for testfile.txt\\\"\"\n   \"restore\": \"x-amz-restore: ongoing-request=\\\"false\\\", expiry-date=\\\"Wed, 07 Nov 2012 00:00:00 GMT\\\"\"\n   \"serverSideEncryption\": \"AES256\",\n   \"lastModified\": \"Mon, 15 Oct 2012 21:58:07 GMT\",\n   \"eTag\": \"1accb31fcf202eba0c0f41fa2f09b4d7\",\n   \"mediaType\": \"text/plain\"\n   \"length\": \"28\",\n   \"content: \"1234567890123456789012345678\"\n }\n </pre>\n </p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">The fields in the returned object have the following meanings:\n <ul>\n   <li>key: the object key (string, mandatory).</li>\n   <li>bucket: the bucket name (string, mandatory).</li>\n   <li>metadata: an object specifying additional metadata. Each metadata is a\n   name-value pair and is represented as different fields (object, optional).</li>\n   <li>missing-metadata: the number of metadata entries that were not returned\n   in the metadata object. This can happen if you create metadata using an API\n   like SOAP that supports more flexible metadata than the REST API. (integer,\n   optional).</li>\n   <li>version: the object version (string, optional). If versioning is enabled\n   for the specified object and no version is specified the permissions for the\n   latest version of the object will be returned. If specified, only the\n   permissions for that particular version of the object will be returned.</li>\n   <li>expiration: if the object expiration is configured (see\n   <a href=\"http://docs.aws.amazon.com/AmazonS3/latest/API/RESTBucketPUTlifecycle.html\">PUT Bucket lifecycle</a>),\n   this field is present. It includes the expiry-date and rule-id key value\n   pairs providing object expiration information. The value of rule-id is URL\n   encoded.(string, optional).</li>\n   <li>restore: if the object is an archived object (an object whose storage\n   class is Glacier), the response includes this header if either the archive\n   restoration is in progress (see\n   <a href=\"http://docs.aws.amazon.com/AmazonS3/latest/API/RESTObjectPOSTrestore.html\">POST Object restore</a>),\n   or an archive copy is already restored.(string, optional).</li>\n   <li>serverSideEncryption: If the object is stored by using server-side\n   encryption, the response includes this header with a value of the encryption\n   algorithm that was used.(string, optional).</li>\n   <li>lastModified: the date in which the object was last modified (dateTime,\n   optional).</li>\n   <li>eTag: the entity tag is a hash of the object. The eTag only reflects\n   changes to the contents of an object, not its metadata. The eTag is determined\n   when an object is created. For objects created by the PUT Object operation\n   and the POST Object operation, the eTag is a quoted, 32-digit hexadecimal\n   string representing the MD5 digest of the object data. For other objects,\n   the eTag may or may not be an MD5 digest of the object data. If the eTag is\n   not an MD5 digest of the object data, it will contain one or more non-hexadecimal\n   characters and/or will consist of less than 32 or more than 32 hexadecimal\n   digits.(string, optional).</li>\n   <li>mediaType: the object media-type (string, mandatory).</li>\n   <li>length: the object content length (integer, optional).</li>\n   <li>content: the object content. (item, optional). The type of this field\n   is determined by the media-type returned by the server. If the media-type\n   indicates that the body content is textual,then the content has type string,\n   base64Binary otherwise. Specifically, the body content is considered textual\n   if and only if the MIME-type specified in the media-type is one of:\n   <ul>\n     <li>\"application/json\"</li>\n     <li>\"application/x-javascript\"</li>\n     <li>\"application/xml\"</li>\n     <li>\"application/xml-external-parsed-entity\"</li>\n   </ul>\n   or if the MIME-type starts with \"text/\" or ends with \"+xml\".</li>\n </ul>\n </p>\n", "summary" : "<p>  Retrieves an S3 object using the specified S3 credentials.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "credentials", "type" : "item()", "occurrence" : "?", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The credentials to use.</div>" }, { "name" : "s3-object", "type" : "item()", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The S3 object or object version to retrieve.</div>" } ], "returns" : { "type" : "object()", "description" : "the specified object." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">s3:CREDENTIALS if the specified S3 credentials cannot be found or are invalid</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">s3:OBJECT if the object specification is invalid</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">s3:BUCKET if no bucket has been specified and no default bucket is present in the used credentials</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">s3:HTTP an HTTP error occurred sending the request to S3</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">s3:REQUEST S3 refused to execute the request</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">s3:RESPONSE the response received from S3 cannot be parsed</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 1, "name" : "read-text-nondeterministic", "qname" : "s3:read-text-nondeterministic", "signature" : "($s3-object as item()) as string", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Retrieves the textual content of an S3 object using the default S3 credentials.\n The object content is forced to be interpreted as textual, with a UTF-8 charset\n and will be returned as string.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">This function has the same semantics as\n <a href=\"#read-text-1\">read-text#1</a>,\n but is declared as being non deterministic and thus should only be used when\n result caching is not desired.</p>\n", "summary" : "<p>  Retrieves the textual content of an S3 object using the default S3 credentials.</p>", "annotation_str" : " %an:nondeterministic", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "nondeterministic", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "s3-object", "type" : "item()", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The S3 object or object version to retrieve.</div>" } ], "returns" : { "type" : "string", "description" : "the object or object version textual content." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">s3:CREDENTIALS if the default S3 credentials cannot be found or are invalid</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">s3:OBJECT if the object specification is invalid</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">s3:BUCKET if no bucket has been specified and no default bucket is present in the used credentials</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">s3:HTTP an HTTP error occurred sending the request to S3</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">s3:REQUEST S3 refused to execute the request</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">s3:RESPONSE the response received from S3 cannot be parsed</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 2, "name" : "read-text-nondeterministic", "qname" : "s3:read-text-nondeterministic", "signature" : "($credentials as item()?, $s3-object as item()) as string", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Retrieves the textual content of an S3 object using the specified S3 credentials.\n The object content is forced to be interpreted as textual, with a UTF-8 charset\n and will be returned as string.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">This function has the same semantics as\n <a href=\"#read-text-2\">read-text#2</a>,\n but is declared as being non deterministic and thus should only be used when\n result caching is not desired.</p>\n", "summary" : "<p>  Retrieves the textual content of an S3 object using the specified S3 credentials.</p>", "annotation_str" : " %an:nondeterministic", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "nondeterministic", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "credentials", "type" : "item()", "occurrence" : "?", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The credentials to use.</div>" }, { "name" : "s3-object", "type" : "item()", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The S3 object or object version to retrieve.</div>" } ], "returns" : { "type" : "string", "description" : "the object or object version textual content." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">s3:CREDENTIALS if the specified S3 credentials cannot be found or are invalid</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">s3:OBJECT if the object specification is invalid</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">s3:BUCKET if no bucket has been specified and no default bucket is present in the used credentials</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">s3:HTTP an HTTP error occurred sending the request to S3</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">s3:REQUEST S3 refused to execute the request</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">s3:RESPONSE the response received from S3 cannot be parsed</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 1, "name" : "read-text", "qname" : "s3:read-text", "signature" : "($s3-object as item()) as string", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Retrieves the textual content of an S3 object using the default S3 credentials.\n The object content is forced to be interpreted as textual, with a UTF-8 charset\n and will be returned as string.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">This function is declared as deterministic and should be used whenever result\n caching is acceptable.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">The S3 object is specified through the $s3-object parameter. Either a string\n or a JSON object can be used. If a string is specified, it is interpreted as\n key for an object in the default bucket of the default credentials. If no default\n bucket is present in the default credentials, the <code>s3:BUCKET</code> error\n is raised. If a JSON object is used, it must have the following structure:\n <ul>\n   <li>key: the object key (string, mandatory).</li>\n   <li>bucket: the bucket name (string, optional). If this field is not present\n   and the default credentials do not have a default bucket, the\n   <code>s3:BUCKET</code> error is raised.</li>\n   <li>version: the object version (string, optional). If versioning is enabled\n   for the specified object and no version is specified the latest version of\n   the object will be returned. If specified, only that particular version of\n   the object will be returned.</li>\n </ul>\n </p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Example:\n <pre>\n let $object :=\n {\n   \"key\": \"object\",\n   \"bucket\": \"28msec\",\n   \"version\": \"3/L4kqtJlcpXroDTDmJ+rmSpXd3dIbrHY+MTRCxf3vjVBH40Nr8X8gdRQBpUMLUo\"\n }\n return s3:read-text(\"credentials\", $object)\n </pre>\n </p>\n", "summary" : "<p>  Retrieves the textual content of an S3 object using the default S3 credentials.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "s3-object", "type" : "item()", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The S3 object or object version to retrieve.</div>" } ], "returns" : { "type" : "string", "description" : "the object or object version textual content." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">s3:CREDENTIALS if the default S3 credentials cannot be found or are invalid</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">s3:OBJECT if the object specification is invalid</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">s3:BUCKET if no bucket has been specified and no default bucket is present in the used credentials</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">s3:HTTP an HTTP error occurred sending the request to S3</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">s3:REQUEST S3 refused to execute the request</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">s3:RESPONSE the response received from S3 cannot be parsed</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 2, "name" : "read-text", "qname" : "s3:read-text", "signature" : "($credentials as item()?, $s3-object as item()) as string", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Retrieves the textual content of an S3 object using the specified S3 credentials.\n The object content is forced to be interpreted as textual, with a UTF-8 charset\n and will be returned as string.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">This function is declared as deterministic and should be used whenever result\n caching is acceptable.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">The S3 object is specified through the $s3-object parameter. Either a string\n or a JSON object can be used. If a string is specified, it is interpreted as\n key for an object in the default bucket of the specified credentials. If no default\n bucket is present in the specified credentials, the <code>s3:BUCKET</code> error\n is raised. If a JSON object is used, it must have the following structure:\n <ul>\n   <li>key: the object key (string, mandatory).</li>\n   <li>bucket: the bucket name (string, optional). If this field is not present\n   and the specified credentials do not have a default bucket, the\n   <code>s3:BUCKET</code> error is raised.</li>\n   <li>version: the object version (string, optional). If versioning is enabled\n   for the specified object and no version is specified the latest version of\n   the object will be returned. If specified, only that particular version of\n   the object will be returned.</li>\n </ul>\n </p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Example:\n <pre>\n let $object :=\n {\n   \"key\": \"object\",\n   \"bucket\": \"28msec\",\n   \"version\": \"3/L4kqtJlcpXroDTDmJ+rmSpXd3dIbrHY+MTRCxf3vjVBH40Nr8X8gdRQBpUMLUo\"\n }\n return s3:read-text(\"credentials\", $object)\n </pre>\n </p>\n", "summary" : "<p>  Retrieves the textual content of an S3 object using the specified S3 credentials.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "credentials", "type" : "item()", "occurrence" : "?", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The credentials to use.</div>" }, { "name" : "s3-object", "type" : "item()", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The S3 object or object version to retrieve.</div>" } ], "returns" : { "type" : "string", "description" : "the object or object version textual content." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">s3:CREDENTIALS if the specified S3 credentials cannot be found or are invalid</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">s3:OBJECT if the object specification is invalid</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">s3:BUCKET if no bucket has been specified and no default bucket is present in the used credentials</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">s3:HTTP an HTTP error occurred sending the request to S3</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">s3:REQUEST S3 refused to execute the request</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">s3:RESPONSE the response received from S3 cannot be parsed</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 1, "name" : "set-object-permissions", "qname" : "s3:set-object-permissions", "signature" : "($s3-object as item()) as empty-sequence()", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Sets the access control list (ACL) of an S3 object using the default S3\n credentials.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">The S3 object is specified through the $s3-object parameter. Either a string\n or a JSON object can be used. If a string is specified, it is interpreted as\n key for an object in the default bucket of the default credentials. If no default\n bucket is present in the default credentials, the <code>s3:BUCKET</code> error\n is raised. If a JSON object is used, it must have the following structure:\n <ul>\n   <li>key: the object key (string, mandatory).</li>\n   <li>bucket: the bucket name (string, optional). If this field is not present\n   and the default credentials do not have a default bucket, the\n   <code>s3:BUCKET</code> error is raised.</li>\n   <li>version: the object version (string, optional). If versioning is enabled\n   for the specified object and no version is specified the permissions for the\n   latest version of the object will be set. If specified, only the\n   permissions for that particular version of the object will be set.</li>\n   <li>permission: the permission to set (string, optional). It must be one of\n   \"private\", \"public-read\", \"public-read-write\", \"authenticated-read\",\n   \"bucket-owner-read\", \"bucket-owner-full-control\". For your convenience the\n   following variables can be used: <code>$s3:ACL-GRANT-PRIVATE</code>,\n   <code>$s3:ACL-GRANT-PUBLIC-READ</code>, <code>$s3:ACL-GRANT-PUBLIC-READ-WRITE</code>,\n   <code>$s3:ACL-GRANT-AUTHENTICATED-READ</code>, <code>$s3:ACL-GRANT-BUCKET-OWNER-READ</code>,\n   <code>$s3:ACL-GRANT-BUCKET-OWNER-FULL-CONTROL</code>. If this field is not\n   specified, the \"private\" ACL is used, that is, the owner gets FULL_CONTROL,\n   and no one else has access rights.</li>\n </ul>\n </p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Example:\n <pre>\n let $object :=\n {\n   \"key\": \"object\",\n   \"bucket\": \"28msec\",\n   \"version\": \"3/L4kqtJlcpXroDTDmJ+rmSpXd3dIbrHY+MTRCxf3vjVBH40Nr8X8gdRQBpUMLUo\",\n   \"permission\": $s3:ACL-GRANT-PUBLIC-READ\n }\n return s3:set-object-permissions($object)\n </pre>\n </p>\n", "summary" : "<p>  Sets the access control list (ACL) of an S3 object using the default S3\n credentials.</p>", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "s3-object", "type" : "item()", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The S3 object or object version and ACL to set.</div>" } ], "returns" : { "type" : "empty-sequence()", "description" : "the empty sequence" }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">s3:CREDENTIALS if the default S3 credentials cannot be found or are invalid</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">s3:OBJECT if the object specification is invalid</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">s3:BUCKET if no bucket has been specified and no default bucket is present in the used credentials</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">s3:HTTP an HTTP error occurred sending the request to S3</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">s3:REQUEST S3 refused to execute the request</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 2, "name" : "set-object-permissions", "qname" : "s3:set-object-permissions", "signature" : "($credentials as item()?, $s3-object as item()) as empty-sequence()", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Sets the access control list (ACL) of an S3 object using the specified S3\n credentials.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">The S3 object is specified through the $s3-object parameter. Either a string\n or a JSON object can be used. If a string is specified, it is interpreted as\n key for an object in the default bucket of the specified credentials. If no default\n bucket is present in the specified credentials, the <code>s3:BUCKET</code> error\n is raised. If a JSON object is used, it must have the following structure:\n <ul>\n   <li>key: the object key (string, mandatory).</li>\n   <li>bucket: the bucket name (string, optional). If this field is not present\n   and the specified credentials do not have a default bucket, the\n   <code>s3:BUCKET</code> error is raised.</li>\n   <li>version: the object version (string, optional). If versioning is enabled\n   for the specified object and no version is specified the permissions for the\n   latest version of the object will be set. If specified, only the\n   permissions for that particular version of the object will be set.</li>\n   <li>permission: the permission to set (string, optional). It must be one of\n   \"private\", \"public-read\", \"public-read-write\", \"authenticated-read\",\n   \"bucket-owner-read\", \"bucket-owner-full-control\". For your convenience the\n   following variables can be used: <code>$s3:ACL-GRANT-PRIVATE</code>,\n   <code>$s3:ACL-GRANT-PUBLIC-READ</code>, <code>$s3:ACL-GRANT-PUBLIC-READ-WRITE</code>,\n   <code>$s3:ACL-GRANT-AUTHENTICATED-READ</code>, <code>$s3:ACL-GRANT-BUCKET-OWNER-READ</code>,\n   <code>$s3:ACL-GRANT-BUCKET-OWNER-FULL-CONTROL</code>. If this field is not\n   specified, the \"private\" ACL is used, that is, the owner gets FULL_CONTROL,\n   and no one else has access rights.</li>\n </ul>\n </p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Example:\n <pre>\n let $object :=\n {\n   \"key\": \"object\",\n   \"bucket\": \"28msec\",\n   \"version\": \"3/L4kqtJlcpXroDTDmJ+rmSpXd3dIbrHY+MTRCxf3vjVBH40Nr8X8gdRQBpUMLUo\",\n   \"permission\": $s3:ACL-GRANT-PUBLIC-READ\n }\n return s3:set-object-permissions($object)\n </pre>\n </p>\n", "summary" : "<p>  Sets the access control list (ACL) of an S3 object using the specified S3\n credentials.</p>", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "credentials", "type" : "item()", "occurrence" : "?", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The credentials to use.</div>" }, { "name" : "s3-object", "type" : "item()", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The S3 object or object version and ACL to set.</div>" } ], "returns" : { "type" : "empty-sequence()", "description" : "the empty sequence" }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">s3:CREDENTIALS if the specified S3 credentials cannot be found or are invalid</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">s3:OBJECT if the object specification is invalid</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">s3:BUCKET if no bucket has been specified and no default bucket is present in the used credentials</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">s3:HTTP an HTTP error occurred sending the request to S3</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">s3:REQUEST S3 refused to execute the request</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 2, "name" : "write-binary", "qname" : "s3:write-binary", "signature" : "($object as item(), $binary-content as base64Binary) as object()", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Writes a binary object in an S3 bucket using the default S3 credentials.\n If the object already exists it is overwritten.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">The S3 object is specified through the $s3-object parameter. Either a string\n or a JSON object can be used.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">If a string is specified, it is interpreted as\n key for an object in the default bucket of the default credentials. If no default\n bucket is present in the default credentials, the <code>s3:BUCKET</code> error\n is raised. The object media-type will be set to \"binary/octet-stream\", no metadata will\n be set for the object, the object will be stored on standard redundancy storage,\n and the object permission will be set to \"private\", that is, the owner gets\n FULL_CONTROL, and no one else has access rights.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">If a JSON object is used, it must have the following structure:\n <ul>\n   <li>key: the object key (string, mandatory).</li>\n   <li>bucket: the bucket name (string, optional). If this field is not present\n   and the default credentials do not have a default bucket, the\n   <code>s3:BUCKET</code> error is raised.</li>\n   <li>mediaType: the media-type of the object (string, optional). If not specified,\n   \"binary/octet-stream\" is used.</li>\n   <li>metadata: an object specifying additional metadata. Each metadata is a\n   name-value pair and is represented as different fields (object, optional).</li>\n   <li>permission: the permission to set (string, optional). It must be one of\n   \"private\", \"public-read\", \"public-read-write\", \"authenticated-read\",\n   \"bucket-owner-read\", \"bucket-owner-full-control\". For your convenience the\n   following variables can be used: <code>$s3:ACL-GRANT-PRIVATE</code>,\n   <code>$s3:ACL-GRANT-PUBLIC-READ</code>, <code>$s3:ACL-GRANT-PUBLIC-READ-WRITE</code>,\n   <code>$s3:ACL-GRANT-AUTHENTICATED-READ</code>, <code>$s3:ACL-GRANT-BUCKET-OWNER-READ</code>,\n   <code>$s3:ACL-GRANT-BUCKET-OWNER-FULL-CONTROL</code>. If this field is not\n   specified, the \"private\" ACL is used, that is, the owner gets FULL_CONTROL,\n   and no one else has access rights.</li>\n   <li>reducedRedundancy: whether to use reduced-redundancy or not (bool,\n   optional). Default is false.</li>\n </ul>\n </p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Example:\n <pre>\n variable $s3-object :=\n {\n   \"key\": \"test.xml\",\n   \"bucket\": \"28msec\",\n   \"permission\": $const:ACL-GRANT-PUBLIC-READ,\n   \"metadata\": { \"author\": \"28msec\" },\n   \"mediaType\": \"application/octet-stream\"\n }\n s3:write-binary($s3-object, base64Binary(\"Mjhtc2Vj\"))\n </pre>\n Example Result:\n <pre>\n {\n   \"expiration\": \"expiry-date=\\\"Fri, 23 Dec 2012 00:00:00 GMT\\\", rule-id=\\\"1\\\"\",\n   \"version\": \"3GL4kqtJlcpXroDTDm3vjVBH40Nr8X8g\",\n   \"eTag\": \"\\\"1b2cf535f27731c974343645a3985328\\\"\"\n }\n </pre>\n </p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">The meaning of the fields in the result object is the following:\n <ul>\n   <li>expiration: if the object expiration is configured, the response\n   includes this header. It includes the expiry-date and rule-id key-value\n   pairs providing object expiration information. The value of the rule-id is\n   URL encoded. (string, optional)</li>\n   <li>version: the version of the object. When you enable versioning,\n   Amazon S3 generates a random number for objects added to a bucket. The value\n   is UTF-8 encoded and URL ready. When you PUT an object in a bucket where\n   versioning has been suspended, the version ID is always null. (string, optional)</li>\n   <li>eTag: the entity tag is a hash of the object. The eTag only reflects\n   changes to the contents of an object, not its metadata. The eTag is determined\n   when an object is created. The eTag returned by this method is a quoted,\n   32-digit hexadecimal string representing the MD5 digest of the object data.\n   For other objects, the eTag may or may not be an MD5 digest of the object data.\n   (string, optional).</li>\n </ul>\n </p>\n", "summary" : "<p>  Writes a binary object in an S3 bucket using the default S3 credentials.</p>", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "object", "type" : "item()", "occurrence" : null, "description" : "" }, { "name" : "binary-content", "type" : "base64Binary", "occurrence" : null, "description" : "" } ], "returns" : { "type" : "object()", "description" : "the S3 operation result." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">s3:CREDENTIALS if the default S3 credentials cannot be found or are invalid</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">s3:OBJECT if the object specification is invalid</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">s3:BUCKET if no bucket has been specified and no default bucket is present in the used credentials</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">s3:HTTP an HTTP error occurred sending the request to S3</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">s3:REQUEST S3 refused to execute the request</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 3, "name" : "write-binary", "qname" : "s3:write-binary", "signature" : "($credentials as item()?, $object as item(), $binary-content as base64Binary) as object()", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Writes a binary object in an S3 bucket using the specified S3 credentials.\n If the object already exists it is overwritten.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">The S3 object is specified through the $s3-object parameter. Either a string\n or a JSON object can be used.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">If a string is specified, it is interpreted as\n key for an object in the default bucket of the specified credentials. If no default\n bucket is present in the specified credentials, the <code>s3:BUCKET</code> error\n is raised. The object media-type will be set to \"binary/octet-stream\", no metadata will\n be set for the object, the object will be stored on standard redundancy storage,\n and the object permission will be set to \"private\", that is, the owner gets\n FULL_CONTROL, and no one else has access rights.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">If a JSON object is used, it must have the following structure:\n <ul>\n   <li>key: the object key (string, mandatory).</li>\n   <li>bucket: the bucket name (string, optional). If this field is not present\n   and the specified credentials do not have a default bucket, the\n   <code>s3:BUCKET</code> error is raised.</li>\n   <li>mediaType: the media-type of the object (string, optional). If not specified,\n   \"binary/octet-stream\" is used.</li>\n   <li>metadata: an object specifying additional metadata. Each metadata is a\n   name-value pair and is represented as different fields (object, optional).</li>\n   <li>permission: the permission to set (string, optional). It must be one of\n   \"private\", \"public-read\", \"public-read-write\", \"authenticated-read\",\n   \"bucket-owner-read\", \"bucket-owner-full-control\". For your convenience the\n   following variables can be used: <code>$s3:ACL-GRANT-PRIVATE</code>,\n   <code>$s3:ACL-GRANT-PUBLIC-READ</code>, <code>$s3:ACL-GRANT-PUBLIC-READ-WRITE</code>,\n   <code>$s3:ACL-GRANT-AUTHENTICATED-READ</code>, <code>$s3:ACL-GRANT-BUCKET-OWNER-READ</code>,\n   <code>$s3:ACL-GRANT-BUCKET-OWNER-FULL-CONTROL</code>. If this field is not\n   specified, the \"private\" ACL is used, that is, the owner gets FULL_CONTROL,\n   and no one else has access rights.</li>\n   <li>reducedRedundancy: whether to use reduced-redundancy or not (bool,\n   optional). Default is false.</li>\n </ul>\n </p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Example:\n <pre>\n variable $s3-object :=\n {\n   \"key\": \"test.xml\",\n   \"bucket\": \"28msec\",\n   \"permission\": $const:ACL-GRANT-PUBLIC-READ,\n   \"metadata\": { \"author\": \"28msec\" },\n   \"mediaType\": \"application/octet-stream\"\n }\n s3:write-binary(\"credentials\", $s3-object, base64Binary(\"Mjhtc2Vj\"))\n </pre>\n </p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Example return object:\n <pre>\n {\n   \"expiration\": \"expiry-date=\\\"Fri, 23 Dec 2012 00:00:00 GMT\\\", rule-id=\\\"1\\\"\",\n   \"version\": \"3GL4kqtJlcpXroDTDm3vjVBH40Nr8X8g\",\n   \"eTag\": \"\\\"1b2cf535f27731c974343645a3985328\\\"\"\n }\n </pre>\n </p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">The fields in the returned object have the following meanings:\n <ul>\n   <li>expiration: if the object expiration is configured, the response\n   includes this header. It includes the expiry-date and rule-id key-value\n   pairs providing object expiration information. The value of the rule-id is\n   URL encoded. (string, optional)</li>\n   <li>version: the version of the object. When you enable versioning,\n   Amazon S3 generates a random number for objects added to a bucket. The value\n   is UTF-8 encoded and URL ready. When you PUT an object in a bucket where\n   versioning has been suspended, the version ID is always null. (string, optional)</li>\n   <li>eTag: the entity tag is a hash of the object. The eTag only reflects\n   changes to the contents of an object, not its metadata. The eTag is determined\n   when an object is created. The eTag returned by this method is a quoted,\n   32-digit hexadecimal string representing the MD5 digest of the object data.\n   For other objects, the eTag may or may not be an MD5 digest of the object data.\n   (string, optional).</li>\n </ul>\n </p>\n", "summary" : "<p>  Writes a binary object in an S3 bucket using the specified S3 credentials.</p>", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "credentials", "type" : "item()", "occurrence" : "?", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The credentials to use.</div>" }, { "name" : "object", "type" : "item()", "occurrence" : null, "description" : "" }, { "name" : "binary-content", "type" : "base64Binary", "occurrence" : null, "description" : "" } ], "returns" : { "type" : "object()", "description" : "the S3 operation result." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">s3:CREDENTIALS if the specified S3 credentials cannot be found or are invalid</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">s3:OBJECT if the object specification is invalid</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">s3:BUCKET if no bucket has been specified and no default bucket is present in the used credentials</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">s3:HTTP an HTTP error occurred sending the request to S3</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">s3:REQUEST S3 refused to execute the request</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 1, "name" : "write-object", "qname" : "s3:write-object", "signature" : "($s3-object as item()) as object()", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Writes an object in an S3 bucket using the default S3 credentials.\n If the object already exists it is overwritten.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">The object to write and its contents are specified through the $s3-object\n parameter, which must have the following structure:\n <ul>\n   <li>key: the object key (string, mandatory).</li>\n   <li>bucket: the bucket name (string, optional). If this field is not present\n   and the default credentials do not have a default bucket, the\n   <code>s3:BUCKET</code> error is raised.</li>\n   <li>content: the object content. It must either be a string or a base64Binary\n   atomic. (item, mandatory).</li>\n   <li>mediaType: the media-type of the object (string, optional). If not specified,\n   \"text/plain\" is used if the content field is of type string, \"binary/octet-stream\"\n   if it is of type base64Binary.</li>\n   <li>metadata: an object specifying additional metadata. Each metadata is a\n   name-value pair and is represented as different fields (object, optional).</li>\n   <li>permission: the permission to set (string, optional). It must be one of\n   \"private\", \"public-read\", \"public-read-write\", \"authenticated-read\",\n   \"bucket-owner-read\", \"bucket-owner-full-control\". For your convenience the\n   following variables can be used: <code>$s3:ACL-GRANT-PRIVATE</code>,\n   <code>$s3:ACL-GRANT-PUBLIC-READ</code>, <code>$s3:ACL-GRANT-PUBLIC-READ-WRITE</code>,\n   <code>$s3:ACL-GRANT-AUTHENTICATED-READ</code>, <code>$s3:ACL-GRANT-BUCKET-OWNER-READ</code>,\n   <code>$s3:ACL-GRANT-BUCKET-OWNER-FULL-CONTROL</code>. If this field is not\n   specified, the \"private\" ACL is used, that is, the owner gets FULL_CONTROL,\n   and no one else has access rights.</li>\n   <li>reducedRedundancy: whether to use reduced-redundancy or not (bool,\n   optional). Default is false.</li>\n </ul>\n </p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Example:\n <pre>\n variable $s3-object :=\n {\n   \"key\": \"test.xml\",\n   \"bucket\": \"28msec\",\n   \"permission\": $const:ACL-GRANT-PUBLIC-READ,\n   \"metadata\": { \"author\": \"28msec\" },\n   \"content\": serialize({ \"Hello\": \"World\" }),\n   \"mediaType\": \"application/xml\"\n }\n s3:write-object($s3-object)\n </pre>\n </p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Example return object:\n <pre>\n {\n   \"expiration\": \"expiry-date=\\\"Fri, 23 Dec 2012 00:00:00 GMT\\\", rule-id=\\\"1\\\"\",\n   \"version\": \"3GL4kqtJlcpXroDTDm3vjVBH40Nr8X8g\",\n   \"eTag\": \"\\\"1b2cf535f27731c974343645a3985328\\\"\"\n }\n </pre>\n </p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">The fields in the returned object have the following meanings:\n <ul>\n   <li>expiration: if the object expiration is configured, the response\n   includes this header. It includes the expiry-date and rule-id key-value\n   pairs providing object expiration information. The value of the rule-id is\n   URL encoded. (string, optional)</li>\n   <li>version: the version of the object. When you enable versioning,\n   Amazon S3 generates a random number for objects added to a bucket. The value\n   is UTF-8 encoded and URL ready. When you PUT an object in a bucket where\n   versioning has been suspended, the version ID is always null. (string, optional)</li>\n   <li>eTag: the entity tag is a hash of the object. The eTag only reflects\n   changes to the contents of an object, not its metadata. The eTag is determined\n   when an object is created. The eTag returned by this method is a quoted,\n   32-digit hexadecimal string representing the MD5 digest of the object data.\n   For other objects, the eTag may or may not be an MD5 digest of the object data.\n   (string, optional).</li>\n </ul>\n </p>\n", "summary" : "<p>  Writes an object in an S3 bucket using the default S3 credentials.</p>", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "s3-object", "type" : "item()", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The S3 object to write.</div>" } ], "returns" : { "type" : "object()", "description" : "the S3 operation result." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">s3:CREDENTIALS if the default S3 credentials cannot be found or are invalid</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">s3:OBJECT if the object specification is invalid</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">s3:BUCKET if no bucket has been specified and no default bucket is present in the used credentials</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">s3:HTTP an HTTP error occurred sending the request to S3</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">s3:REQUEST S3 refused to execute the request</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 2, "name" : "write-object", "qname" : "s3:write-object", "signature" : "($credentials as item()?, $s3-object as object()) as object()", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Writes an object in an S3 bucket using the specified credentials.\n If the object already exists it is overwritten.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">The object to write and its contents are specified through the $s3-object\n parameter, which must have the following structure:\n <ul>\n   <li>key: the object key (string, mandatory).</li>\n   <li>bucket: the bucket name (string, optional). If this field is not present\n   and the specified credentials do not have a default bucket, the\n   <code>s3:BUCKET</code> error is raised.</li>\n   <li>content: the object content. It must either be a string or a base64Binary\n   atomic. (item, mandatory).</li>\n   <li>mediaType: the media-type of the object (string, optional). If not specified,\n   \"text/plain\" is used if the content field is of type string, \"binary/octet-stream\"\n   if it is of type base64Binary.</li>\n   <li>metadata: an object specifying additional metadata. Each metadata is a\n   name-value pair and is represented as different fields (object, optional).</li>\n   <li>permission: the permission to set (string, optional). It must be one of\n   \"private\", \"public-read\", \"public-read-write\", \"authenticated-read\",\n   \"bucket-owner-read\", \"bucket-owner-full-control\". For your convenience the\n   following variables can be used: <code>$s3:ACL-GRANT-PRIVATE</code>,\n   <code>$s3:ACL-GRANT-PUBLIC-READ</code>, <code>$s3:ACL-GRANT-PUBLIC-READ-WRITE</code>,\n   <code>$s3:ACL-GRANT-AUTHENTICATED-READ</code>, <code>$s3:ACL-GRANT-BUCKET-OWNER-READ</code>,\n   <code>$s3:ACL-GRANT-BUCKET-OWNER-FULL-CONTROL</code>. If this field is not\n   specified, the \"private\" ACL is used, that is, the owner gets FULL_CONTROL,\n   and no one else has access rights.</li>\n   <li>reducedRedundancy: whether to use reduced-redundancy or not (bool,\n   optional). Default is false.</li>\n </ul>\n </p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Example:\n <pre>\n variable $s3-object :=\n {\n   \"key\": \"test.xml\",\n   \"bucket\": \"28msec\",\n   \"permission\": $const:ACL-GRANT-PUBLIC-READ,\n   \"metadata\": { \"author\": \"28msec\" },\n   \"content\": serialize({ \"Hello\": \"World\" }),\n   \"mediaType\": \"application/xml\"\n }\n s3:write-object(\"s3\", $s3-object)\n </pre>\n </p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Example return object:\n <pre>\n {\n   \"expiration\": \"expiry-date=\\\"Fri, 23 Dec 2012 00:00:00 GMT\\\", rule-id=\\\"1\\\"\",\n   \"version\": \"3GL4kqtJlcpXroDTDm3vjVBH40Nr8X8g\",\n   \"eTag\": \"\\\"1b2cf535f27731c974343645a3985328\\\"\"\n }\n </pre>\n </p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">The fields in the returned object have the following meanings:\n <ul>\n   <li>expiration: if the object expiration is configured, the response\n   includes this header. It includes the expiry-date and rule-id key-value\n   pairs providing object expiration information. The value of the rule-id is\n   URL encoded. (string, optional)</li>\n   <li>version: the version of the object. When you enable versioning,\n   Amazon S3 generates a random number for objects added to a bucket. The value\n   is UTF-8 encoded and URL ready. When you PUT an object in a bucket where\n   versioning has been suspended, the version ID is always null. (string, optional)</li>\n   <li>eTag: the entity tag is a hash of the object. The eTag only reflects\n   changes to the contents of an object, not its metadata. The eTag is determined\n   when an object is created. The eTag returned by this method is a quoted,\n   32-digit hexadecimal string representing the MD5 digest of the object data.\n   For other objects, the eTag may or may not be an MD5 digest of the object data.\n   (string, optional).</li>\n </ul>\n </p>\n", "summary" : "<p>  Writes an object in an S3 bucket using the specified credentials.</p>", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "credentials", "type" : "item()", "occurrence" : "?", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The credentials to use.</div>" }, { "name" : "s3-object", "type" : "object()", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The S3 object to write.</div>" } ], "returns" : { "type" : "object()", "description" : "the S3 operation result." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">s3:CREDENTIALS if the specified S3 credentials cannot be found or are invalid</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">s3:OBJECT if the object specification is invalid</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">s3:BUCKET if no bucket has been specified and no default bucket is present in the used credentials</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">s3:HTTP an HTTP error occurred sending the request to S3</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">s3:REQUEST S3 refused to execute the request</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 2, "name" : "write-text", "qname" : "s3:write-text", "signature" : "($object as item(), $text-content as string) as object()", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Writes a textual object in an S3 bucket using the default S3 credentials.\n If the object already exists it is overwritten.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">The S3 object is specified through the $s3-object parameter. Either a string\n or a JSON object can be used.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">If a string is specified, it is interpreted as\n key for an object in the default bucket of the default credentials. If no default\n bucket is present in the default credentials, the <code>s3:BUCKET</code> error\n is raised. The object media-type will be set to \"text/plain\", no metadata will\n be set for the object, the object will be stored on standard redundancy storage,\n and the object permission will be set to \"private\", that is, the owner gets\n FULL_CONTROL, and no one else has access rights.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">If a JSON object is used, it must have the following structure:\n <ul>\n   <li>key: the object key (string, mandatory).</li>\n   <li>bucket: the bucket name (string, optional). If this field is not present\n   and the default credentials do not have a default bucket, the\n   <code>s3:BUCKET</code> error is raised.</li>\n   <li>mediaType: the media-type of the object (string, optional). If not specified,\n   \"text/plain\" is used.</li>\n   <li>metadata: an object specifying additional metadata. Each metadata is a\n   name-value pair and is represented as different fields (object, optional).</li>\n   <li>permission: the permission to set (string, optional). It must be one of\n   \"private\", \"public-read\", \"public-read-write\", \"authenticated-read\",\n   \"bucket-owner-read\", \"bucket-owner-full-control\". For your convenience the\n   following variables can be used: <code>$s3:ACL-GRANT-PRIVATE</code>,\n   <code>$s3:ACL-GRANT-PUBLIC-READ</code>, <code>$s3:ACL-GRANT-PUBLIC-READ-WRITE</code>,\n   <code>$s3:ACL-GRANT-AUTHENTICATED-READ</code>, <code>$s3:ACL-GRANT-BUCKET-OWNER-READ</code>,\n   <code>$s3:ACL-GRANT-BUCKET-OWNER-FULL-CONTROL</code>. If this field is not\n   specified, the \"private\" ACL is used, that is, the owner gets FULL_CONTROL,\n   and no one else has access rights.</li>\n   <li>reducedRedundancy: whether to use reduced-redundancy or not (bool,\n   optional). Default is false.</li>\n </ul>\n </p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Example:\n <pre>\n variable $s3-object :=\n {\n   \"key\": \"test.xml\",\n   \"bucket\": \"28msec\",\n   \"permission\": $const:ACL-GRANT-PUBLIC-READ,\n   \"metadata\": { \"author\": \"28msec\" },\n   \"mediaType\": \"text/xml\"\n }\n s3:write-text($s3-object, serialize(&lt;a/&gt;))\n </pre>\n </p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Example return object:\n <pre>\n {\n   \"expiration\": \"expiry-date=\\\"Fri, 23 Dec 2012 00:00:00 GMT\\\", rule-id=\\\"1\\\"\",\n   \"version\": \"3GL4kqtJlcpXroDTDm3vjVBH40Nr8X8g\",\n   \"eTag\": \"\\\"1b2cf535f27731c974343645a3985328\\\"\"\n }\n </pre>\n </p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">The fields in the returned object have the following meanings:\n <ul>\n   <li>expiration: if the object expiration is configured, the response\n   includes this header. It includes the expiry-date and rule-id key-value\n   pairs providing object expiration information. The value of the rule-id is\n   URL encoded. (string, optional)</li>\n   <li>version: the version of the object. When you enable versioning,\n   Amazon S3 generates a random number for objects added to a bucket. The value\n   is UTF-8 encoded and URL ready. When you PUT an object in a bucket where\n   versioning has been suspended, the version ID is always null. (string, optional)</li>\n   <li>eTag: the entity tag is a hash of the object. The eTag only reflects\n   changes to the contents of an object, not its metadata. The eTag is determined\n   when an object is created. The eTag returned by this method is a quoted,\n   32-digit hexadecimal string representing the MD5 digest of the object data.\n   For other objects, the eTag may or may not be an MD5 digest of the object data.\n   (string, optional).</li>\n </ul>\n </p>\n", "summary" : "<p>  Writes a textual object in an S3 bucket using the default S3 credentials.</p>", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "object", "type" : "item()", "occurrence" : null, "description" : "" }, { "name" : "text-content", "type" : "string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The textual object content.</div>" } ], "returns" : { "type" : "object()", "description" : "the S3 operation result." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">s3:CREDENTIALS if the default S3 credentials cannot be found or are invalid</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">s3:OBJECT if the object specification is invalid</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">s3:BUCKET if no bucket has been specified and no default bucket is present in the used credentials</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">s3:HTTP an HTTP error occurred sending the request to S3</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">s3:REQUEST S3 refused to execute the request</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 3, "name" : "write-text", "qname" : "s3:write-text", "signature" : "($credentials as item()?, $object as item(), $text-content as string) as object()", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Writes a text object in an S3 bucket using the specified S3 credentials.\n If the object already exists it is overwritten.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">The S3 object is specified through the $s3-object parameter. Either a string\n or a JSON object can be used.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">If a string is specified, it is interpreted as\n key for an object in the default bucket of the specified credentials. If no default\n bucket is present in the specified credentials, the <code>s3:BUCKET</code> error\n is raised. The object media-type will be set to \"text/plain\", no metadata will\n be set for the object, the object will be stored on standard redundancy storage,\n and the object permission will be set to \"private\", that is, the owner gets\n FULL_CONTROL, and no one else has access rights.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">If a JSON object is used, it must have the following structure:\n <ul>\n   <li>key: the object key (string, mandatory).</li>\n   <li>bucket: the bucket name (string, optional). If this field is not present\n   and the specified credentials do not have a default bucket, the\n   <code>s3:BUCKET</code> error is raised.</li>\n   <li>mediaType: the media-type of the object (string, optional). If not specified,\n   \"text/plain\" is used.</li>\n   <li>metadata: an object specifying additional metadata. Each metadata is a\n   name-value pair and is represented as different fields (object, optional).</li>\n   <li>permission: the permission to set (string, optional). It must be one of\n   \"private\", \"public-read\", \"public-read-write\", \"authenticated-read\",\n   \"bucket-owner-read\", \"bucket-owner-full-control\". For your convenience the\n   following variables can be used: <code>$s3:ACL-GRANT-PRIVATE</code>,\n   <code>$s3:ACL-GRANT-PUBLIC-READ</code>, <code>$s3:ACL-GRANT-PUBLIC-READ-WRITE</code>,\n   <code>$s3:ACL-GRANT-AUTHENTICATED-READ</code>, <code>$s3:ACL-GRANT-BUCKET-OWNER-READ</code>,\n   <code>$s3:ACL-GRANT-BUCKET-OWNER-FULL-CONTROL</code>. If this field is not\n   specified, the \"private\" ACL is used, that is, the owner gets FULL_CONTROL,\n   and no one else has access rights.</li>\n   <li>reducedRedundancy: whether to use reduced-redundancy or not (bool,\n   optional). Default is false.</li>\n </ul>\n </p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Example:\n <pre>\n variable $s3-object :=\n {\n   \"key\": \"test.xml\",\n   \"bucket\": \"28msec\",\n   \"permission\": $const:ACL-GRANT-PUBLIC-READ,\n   \"metadata\": { \"author\": \"28msec\" },\n   \"mediaType\": \"text/xml\"\n }\n s3:write-text(\"credentials\", $s3-object, serialize(&lt;a/&gt;))\n </pre>\n </p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Example return object:\n <pre>\n {\n   \"expiration\": \"expiry-date=\\\"Fri, 23 Dec 2012 00:00:00 GMT\\\", rule-id=\\\"1\\\"\",\n   \"version\": \"3GL4kqtJlcpXroDTDm3vjVBH40Nr8X8g\",\n   \"eTag\": \"\\\"1b2cf535f27731c974343645a3985328\\\"\"\n }\n </pre>\n </p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">The fields in the returned object have the following meanings:\n <ul>\n   <li>expiration: if the object expiration is configured, the response\n   includes this header. It includes the expiry-date and rule-id key-value\n   pairs providing object expiration information. The value of the rule-id is\n   URL encoded. (string, optional)</li>\n   <li>version: the version of the object. When you enable versioning,\n   Amazon S3 generates a random number for objects added to a bucket. The value\n   is UTF-8 encoded and URL ready. When you PUT an object in a bucket where\n   versioning has been suspended, the version ID is always null. (string, optional)</li>\n   <li>eTag: the entity tag is a hash of the object. The eTag only reflects\n   changes to the contents of an object, not its metadata. The eTag is determined\n   when an object is created. The eTag returned by this method is a quoted,\n   32-digit hexadecimal string representing the MD5 digest of the object data.\n   For other objects, the eTag may or may not be an MD5 digest of the object data.\n   (string, optional).</li>\n </ul>\n </p>\n", "summary" : "<p>  Writes a text object in an S3 bucket using the specified S3 credentials.</p>", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "credentials", "type" : "item()", "occurrence" : "?", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The credentials to use.</div>" }, { "name" : "object", "type" : "item()", "occurrence" : null, "description" : "" }, { "name" : "text-content", "type" : "string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The textual object content.</div>" } ], "returns" : { "type" : "object()", "description" : "the S3 operation result." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">s3:CREDENTIALS if the specified S3 credentials cannot be found or are invalid</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">s3:OBJECT if the object specification is invalid</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">s3:BUCKET if no bucket has been specified and no default bucket is present in the used credentials</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">s3:HTTP an HTTP error occurred sending the request to S3</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">s3:REQUEST S3 refused to execute the request</xqdoc:error>" ] } ], "variables" : [ { "name" : "s3:ACL-GRANT-PRIVATE", "type" : "item()*", "description" : " The private canned ACL. It is applicable to buckets and objects.\n Owner gets FULL_CONTROL. No one else has access rights.\n" }, { "name" : "s3:ACL-GRANT-PUBLIC-READ", "type" : "item()*", "description" : " The public-read canned ACL. Applicable to buckets and objects.\n Owner gets FULL_CONTROL. The AllUsers group gets READ access.\n" }, { "name" : "s3:ACL-GRANT-PUBLIC-READ-WRITE", "type" : "item()*", "description" : " The public-read-write canned ACL. Applicable to buckets and objects.\n Owner gets FULL_CONTROL. The AllUsers group gets READ and WRITE access.\n Granting this on a bucket is generally not recommended.\n" }, { "name" : "s3:ACL-GRANT-AUTHENTICATED-READ", "type" : "item()*", "description" : " The autenticated-read canned ACL. Applicable to buckets and objects.\n Owner gets FULL_CONTROL. The AuthenticatedUsers group gets READ access.\n" }, { "name" : "s3:ACL-GRANT-BUCKET-OWNER-READ", "type" : "item()*", "description" : " The grant-bucket-owner-read canned ACL. Applicable to objects.\n Object owner gets FULL_CONTROL. Bucket owner gets READ access.\n If you specify this canned ACL when creating a bucket, Amazon S3 ignores it.\n" }, { "name" : "s3:ACL-GRANT-BUCKET-OWNER-FULL-CONTROL", "type" : "item()*", "description" : " The grant-bucket-owner-full-control canned ACL. Applicable to objects.\n Both the object owner and the bucket owner get FULL_CONTROL over the object.\n If you specify this canned ACL when creating a bucket, Amazon S3 ignores it.\n" } ] }, "http://api.28.io/authorization" : { "ns" : "http://api.28.io/authorization", "description" : "", "sees" : [  ], "authors" : [  ], "version" : null, "encoding" : null, "namespaces" : [ { "uri" : "", "prefix" : "an" }, { "uri" : "http://api.28.io/authorization", "prefix" : "authorization" }, { "uri" : "http://zorba.io/modules/base64", "prefix" : "base64" }, { "uri" : "http://zorba.io/modules/hmac", "prefix" : "hmac" }, { "uri" : "http://expath.org/ns/http-client", "prefix" : "http-client" }, { "uri" : "http://www.28msec.com/modules/project", "prefix" : "project" }, { "uri" : "http://www.28msec.com/modules/http/request", "prefix" : "req" }, { "uri" : "http://www.28msec.com/modules/http/response", "prefix" : "res" } ], "functions" : [ { "isDocumented" : false, "arity" : 0, "name" : "authorized", "qname" : "authorization:authorized", "signature" : "() as xs:boolean", "description" : "", "summary" : "", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [  ], "returns" : { "type" : "xs:boolean", "description" : "" }, "errors" : [  ] }, { "isDocumented" : false, "arity" : 0, "name" : "html-do-login", "qname" : "authorization:html-do-login", "signature" : "()", "description" : "", "summary" : "", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [  ], "returns" : { "type" : null, "description" : "" }, "errors" : [  ] }, { "isDocumented" : false, "arity" : 0, "name" : "html-show-login", "qname" : "authorization:html-show-login", "signature" : "()", "description" : "", "summary" : "", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [  ], "returns" : { "type" : null, "description" : "" }, "errors" : [  ] } ], "variables" : [  ] }, "http://zorba.io/modules/store/static/integrity-constraints/dml" : { "ns" : "http://zorba.io/modules/store/static/integrity-constraints/dml", "description" : " This module defines a function to check if an integrity constraint is\n satisfied.\n The integrity constraint needs to be declared in the prolog of a module.\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n This module is part of\n <a xmlns:xqdoc=\"http://www.xqdoc.org/1.0\" href=\"../zorba/xqddf.html\">Zorba's XQuery Data Definition Facility</a>.\n All the integrity constraints managed by this module have to be pre-declared\n in the prolog of a module.\n Please refer to the\n <a xmlns:xqdoc=\"http://www.xqdoc.org/1.0\" href=\"../zorba/data_lifecycle.html\">general documentation</a>\n for more information and examples.\n", "sees" : [ "<xqdoc:see xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"><a href=\"../zorba/data_lifecycle.html\">Data Lifecycle</a></xqdoc:see>", "<xqdoc:see xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"><a href=\"../zorba/xqddf.html\">XQuery Data Definition Facility</a></xqdoc:see>", "<xqdoc:see xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">http://zorba.io/modules/store/static/integrity-constraints/dml</xqdoc:see>", "<xqdoc:see xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">http://zorba.io/modules/store/static/collections/dml</xqdoc:see>", "<xqdoc:see xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">http://zorba.io/modules/store/static/collections/ddl</xqdoc:see>", "<xqdoc:see xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">http://zorba.io/modules/store/static/indexes/ddl</xqdoc:see>", "<xqdoc:see xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">http://zorba.io/modules/store/static/indexes/dml</xqdoc:see>", "<xqdoc:see xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">http://zorba.io/errors</xqdoc:see>" ], "authors" : [ "<xqdoc:author xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Nicolae Brinza, Matthias Brantner, David Graf, Till Westmann, Markos Zaharioudakis</xqdoc:author>" ], "version" : null, "encoding" : "utf-8", "namespaces" : [ { "uri" : "http://zorba.io/modules/store/static/integrity-constraints/dml", "prefix" : "icdml" }, { "uri" : "http://zorba.io/options/versioning", "prefix" : "ver" }, { "uri" : "http://zorba.io/errors", "prefix" : "zerr" } ], "functions" : [ { "isDocumented" : true, "arity" : 1, "name" : "check-integrity-constraint", "qname" : "icdml:check-integrity-constraint", "signature" : "($name as xs:QName) as xs:boolean external", "description" : " Checks if the specified constraints are valid in the database.\n", "summary" : "<p> Checks if the specified constraints are valid in the database.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "name", "type" : "xs:QName", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The name of the integrity constraint to check.</div>" } ], "returns" : { "type" : "xs:boolean", "description" : "true if the constraints are valid; false otherwise." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">zerr:ZDDY0031 if the integrity constraint is not declared.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">zerr:ZDDY0032 if the integrity constraint is not available.</xqdoc:error>" ] } ], "variables" : [  ] }, "http://zorba.io/modules/excel/statistical" : { "ns" : "http://zorba.io/modules/excel/statistical", "description" : " This is a library module offering a part of the set of statistical functions\n defined by Microsoft Excel 2003.\n", "sees" : [ "<xqdoc:see xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"><a href=\"http://office.microsoft.com/en-us/excel/CH062528311033.aspx\" target=\"_blank\">Excel 2003 Documentation: Statistical Functions</a></xqdoc:see>" ], "authors" : [ "<xqdoc:author xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Daniel Turcanu</xqdoc:author>" ], "version" : null, "encoding" : "utf-8", "namespaces" : [ { "uri" : "http://zorba.io/modules/excel/statistical", "prefix" : "excel" }, { "uri" : "http://zorba.io/modules/excel/errors", "prefix" : "excel-err" }, { "uri" : "http://zorba.io/modules/excel/math", "prefix" : "excel-math" }, { "uri" : "http://zorba.io/options/versioning", "prefix" : "ver" } ], "functions" : [ { "isDocumented" : true, "arity" : 1, "name" : "avedev", "qname" : "excel:avedev", "signature" : "($numbers as xs:anyAtomicType+) as xs:anyAtomicType", "description" : " Returns the average of the absolute deviations of data points from their mean.\n The formula is sum(abs(x - average_x))/n, where n is the count of x in the sequence.\n", "summary" : "<p> Returns the average of the absolute deviations of data points from their mean.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "numbers", "type" : "xs:anyAtomicType", "occurrence" : "+", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequence of numbers or values castable to numeric. Sequence can be of any length from 1 up.</div>" } ], "returns" : { "type" : "xs:anyAtomicType", "description" : "The formula result" }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">excel-err:Value if the parameters cannot be casted to numeric type</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 1, "name" : "average", "qname" : "excel:average", "signature" : "($numbers as xs:anyAtomicType*) as xs:anyAtomicType", "description" : " Returns the average (arithmetic mean) of the arguments.\n Arguments can be empty values, otherwise must be castable to numeric.\n If sequence is empty then zero is returned.\n The sequence can be of any length.\n", "summary" : "<p> Returns the average (arithmetic mean) of the arguments.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "numbers", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The sequence of numbers or empty values.</div>" } ], "returns" : { "type" : "xs:anyAtomicType", "description" : "The sum of all numbers divided by the number of non-empty values." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">excel-err:Value if the parameters cannot be casted to numeric type.</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 1, "name" : "averagea", "qname" : "excel:averagea", "signature" : "($numbers as xs:anyAtomicType+) as xs:anyAtomicType", "description" : " Calculates the average (arithmetic mean) of the values in the sequence of arguments.\n Arguments can be of any type.\n The numbers are added, and the sum is divided by the size of entire sequence.\n", "summary" : "<p> Calculates the average (arithmetic mean) of the values in the sequence of arguments.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "numbers", "type" : "xs:anyAtomicType", "occurrence" : "+", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequence of values of any type. The sequence can be of any length, from 1 up.</div>" } ], "returns" : { "type" : "xs:anyAtomicType", "description" : "The result" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "count", "qname" : "excel:count", "signature" : "($numbers as xs:anyAtomicType*) as xs:integer", "description" : " Counts the number of cells that contain numbers or values castable to numeric.\n", "summary" : "<p> Counts the number of cells that contain numbers or values castable to numeric.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "numbers", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The sequence of values, of any length.</div>" } ], "returns" : { "type" : "xs:integer", "description" : "The count of numbers." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "counta", "qname" : "excel:counta", "signature" : "($numbers as xs:anyAtomicType*) as xs:integer", "description" : " Counts the number of values that are not empty.\n Empty values are the one with string value \"\".\n", "summary" : "<p> Counts the number of values that are not empty.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "numbers", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequence of values of any type, any length</div>" } ], "returns" : { "type" : "xs:integer", "description" : "The count of non-empty values" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "countblank", "qname" : "excel:countblank", "signature" : "($cells as xs:anyAtomicType*) as xs:integer", "description" : " Counts the empty values in a sequence.\n The empty values are the ones with string value \"\".\n The value 0 is not counted.\n", "summary" : "<p> Counts the empty values in a sequence.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "cells", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequence of values, of any length</div>" } ], "returns" : { "type" : "xs:integer", "description" : "The count" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 2, "name" : "large", "qname" : "excel:large", "signature" : "($numbers as xs:anyAtomicType*, $k as xs:integer) as xs:anyAtomicType", "description" : " Returns the k-th largest value in a data set.\n If n is the number of data points in a range,\n   then LARGE(array,1) returns the largest value,\n   and LARGE(array,n) returns the smallest value.\n", "summary" : "<p> Returns the k-th largest value in a data set.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "numbers", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequence of numbers or values castable to numeric. The sequence can be of any length, from 1 up.</div>" }, { "name" : "k", "type" : "xs:integer", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the position of largest value, with value from 1 to count of values</div>" } ], "returns" : { "type" : "xs:anyAtomicType", "description" : "The k-th largest value as numeric type" }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">excel-err:Value if the parameters cannot be casted to numeric type</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">excel-err:Num if the sequence is empty</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">excel-err:Num if k is not a value between 1 and the sequence size</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 1, "name" : "max", "qname" : "excel:max", "signature" : "($numbers as xs:anyAtomicType*) as xs:anyAtomicType", "description" : " Returns the largest number in a sequence.\n", "summary" : "<p> Returns the largest number in a sequence.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "numbers", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequence of numbers or values castable to numeric. The sequence can be of any length.</div>" } ], "returns" : { "type" : "xs:anyAtomicType", "description" : "The max" }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">excel-err:Value if the parameters cannot be casted to numeric type</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 1, "name" : "maxa", "qname" : "excel:maxa", "signature" : "($numbers as xs:anyAtomicType*) as xs:anyAtomicType", "description" : " Returns the largest value in a list of arguments.\n In this implementation there is no difference between MAX and MAXA.\n", "summary" : "<p> Returns the largest value in a list of arguments.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "numbers", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequence of numbers or values castable to numeric The sequence can be of any length.</div>" } ], "returns" : { "type" : "xs:anyAtomicType", "description" : "The max" }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">excel-err:Value if the parameters cannot be casted to numeric type</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 1, "name" : "median", "qname" : "excel:median", "signature" : "($numbers as xs:anyAtomicType*) as xs:anyAtomicType", "description" : " Returns the median of the given numbers.\n The median is the number in the middle of a set of numbers.\n Half the numbers have values that are greater than the median,\n and half the numbers have values that are less than the median.\n", "summary" : "<p> Returns the median of the given numbers.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "numbers", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequence of numbers, of any length</div>" } ], "returns" : { "type" : "xs:anyAtomicType", "description" : "for odd count of numbers return the number in the middle of the sorted sequence. For even count of numbers return the average of the two numbers in the middle." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">excel-err:Value if the parameters cannot be casted to numeric type</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 1, "name" : "min", "qname" : "excel:min", "signature" : "($numbers as xs:anyAtomicType*) as xs:anyAtomicType", "description" : " Returns the smallest number in a sequence.\n", "summary" : "<p> Returns the smallest number in a sequence.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "numbers", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequence of numbers or values castable to numeric. The sequence can be of any length.</div>" } ], "returns" : { "type" : "xs:anyAtomicType", "description" : "The min" }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">excel-err:Value if the parameters cannot be casted to numeric type</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 1, "name" : "mina", "qname" : "excel:mina", "signature" : "($numbers as xs:anyAtomicType*) as xs:anyAtomicType", "description" : " Returns the smallest value in a list of arguments.\n In this implementation there is no difference between MAX and MAXA.\n", "summary" : "<p> Returns the smallest value in a list of arguments.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "numbers", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequence of numbers or values castable to numeric The sequence can be of any length.</div>" } ], "returns" : { "type" : "xs:anyAtomicType", "description" : "The min" }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">excel-err:Value if the parameters cannot be casted to numeric type</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 1, "name" : "mode", "qname" : "excel:mode", "signature" : "($numbers as xs:anyAtomicType*) as xs:anyAtomicType", "description" : " Returns the most frequently occurring, or repetitive, value in a sequence.\n Arguments must be castable to numeric.\n", "summary" : "<p> Returns the most frequently occurring, or repetitive, value in a sequence.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "numbers", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequence of numbers, of any length</div>" } ], "returns" : { "type" : "xs:anyAtomicType", "description" : "The most occuring number" }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">excel-err:Value if the parameters cannot be casted to numeric type</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">fn:QName(\"http://zorba.io/modules/excel/errors\", \"excel-err:NA\") if there are no duplicate numbers</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 2, "name" : "percentile", "qname" : "excel:percentile", "signature" : "($numbers as xs:anyAtomicType*, $k_at as xs:anyAtomicType) as xs:anyAtomicType", "description" : " Returns the k-th percentile of values in a sequence.\n If k is not a multiple of 1/(n - 1),\n   PERCENTILE interpolates to determine the value at the k-th percentile.\n The function is computed by (max-min)*k + min\n", "summary" : "<p> Returns the k-th percentile of values in a sequence.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "numbers", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequence of numbers, of any length</div>" }, { "name" : "k_at", "type" : "xs:anyAtomicType", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the percentile, with value between 0 .. 1 inclusive</div>" } ], "returns" : { "type" : "xs:anyAtomicType", "description" : "The computed percentile" }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">excel-err:Value if the parameters cannot be casted to numeric type</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">excel-err:Num if percentile is not between 0 .. 1</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 2, "name" : "percentrank", "qname" : "excel:percentrank", "signature" : "($numbers as xs:anyAtomicType*, $x as xs:anyAtomicType) as xs:decimal", "description" : " Returns the rank of a value in a data set as a percentage of the data set.\n If x does not match one of the values in array,\n   PERCENTRANK interpolates to return the correct percentage rank. <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n The formula is uses: (RANK - 1) / (size - 1) .\n", "summary" : "<p> Returns the rank of a value in a data set as a percentage of the data set.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "numbers", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequence of numbers or values castable to numbers. The sequence can be of any length, from 1 up.</div>" }, { "name" : "x", "type" : "xs:anyAtomicType", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> is the value for which you want to know the rank</div>" } ], "returns" : { "type" : "xs:decimal", "description" : "The percentage of rank." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">excel-err:Value if the parameters cannot be casted to numeric type</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">excel-err:Num if the sequence is zero length</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 3, "name" : "prob", "qname" : "excel:prob", "signature" : "($x_range as xs:anyAtomicType+, $prob_range as xs:anyAtomicType+, $range_lower_limit as xs:anyAtomicType) as xs:anyAtomicType", "description" : " This is the same as above, only that upper_limit is not specified.\n The probability is computed only for range_lower_limit.\n", "summary" : "<p> This is the same as above, only that upper_limit is not specified.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "x_range", "type" : "xs:anyAtomicType", "occurrence" : "+", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> is the range of numeric values of x with which there are associated probabilities. This does not need to be ordered.</div>" }, { "name" : "prob_range", "type" : "xs:anyAtomicType", "occurrence" : "+", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> is a set of probabilities associated with values in x_range.</div>" }, { "name" : "range_lower_limit", "type" : "xs:anyAtomicType", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> is the value for which you want a probability.</div>" } ], "returns" : { "type" : "xs:anyAtomicType", "description" : "The probability of the range_lower_limit value" }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">excel-err:Num if any probability is not between 0 and 1</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">excel-err:Num if the sum of probabilities is not equal to 1</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">excel-err:Value if any parameter is not castable to numeric</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">excel-err:Num if x_range and prob_range do not have the same number of values</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 4, "name" : "prob", "qname" : "excel:prob", "signature" : "($x_range as xs:anyAtomicType+, $prob_range as xs:anyAtomicType+, $range_lower_limit as xs:anyAtomicType, $upper_limit as xs:anyAtomicType) as xs:anyAtomicType", "description" : " Returns the probability that values in a range are between two limits.\n", "summary" : "<p> Returns the probability that values in a range are between two limits.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "x_range", "type" : "xs:anyAtomicType", "occurrence" : "+", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> is the range of numeric values of x with which there are associated probabilities. This does not need to be ordered.</div>" }, { "name" : "prob_range", "type" : "xs:anyAtomicType", "occurrence" : "+", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> is a set of probabilities associated with values in x_range.</div>" }, { "name" : "range_lower_limit", "type" : "xs:anyAtomicType", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> is the lower bound on the value for which you want a probability.</div>" }, { "name" : "upper_limit", "type" : "xs:anyAtomicType", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> is the upper bound on the value for which you want a probability.</div>" } ], "returns" : { "type" : "xs:anyAtomicType", "description" : "The probability of the entire range" }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">excel-err:Num if any probability is not between 0 and 1</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">excel-err:Num if the sum of probabilities is not equal to 1</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">excel-err:Value if any parameter is not castable to numeric</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">excel-err:Num if x_range and prob_range do not have the same number of values</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 2, "name" : "quartile", "qname" : "excel:quartile", "signature" : "($numbers as xs:anyAtomicType*, $quart as xs:integer) as xs:anyAtomicType", "description" : " Returns the quartile of a data set.\n", "summary" : "<p> Returns the quartile of a data set.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "numbers", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> sequence of numbers or values castable to numbers. The sequence can be of any length, from 1 up.</div>" }, { "name" : "quart", "type" : "xs:integer", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> <dl>one of the values 0, 1, 2, 3, 4 with meaning: <dt>0</dt> <dd> compute minimum value</dd> <dt>1</dt> <dd> compute first quartile (25th percentile)</dd> <dt>2</dt> <dd> compute median value (50th percentile)</dd> <dt>3</dt> <dd> compute third quartile (75th percentile)</dd> <dt>4</dt> <dd> compute maximum value</dd></dl></div>" } ], "returns" : { "type" : "xs:anyAtomicType", "description" : "the computed quartile, as numeric type" }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">excel-err:Value if the parameters cannot be casted to numeric type</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">excel-err:Num if the sequence is zero length</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">excel-err:Num if $quart is not one of the values 0, 1, 2, 3, 4</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 2, "name" : "rank", "qname" : "excel:rank", "signature" : "($x as xs:anyAtomicType, $numbers as xs:anyAtomicType*) as xs:decimal", "description" : " This RANK function is same as the above, only that $order_ascending is set by default to false.\n", "summary" : "<p> This RANK function is same as the above, only that $order_ascending is set by default to false.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "x", "type" : "xs:anyAtomicType", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The number whose rank you want to find.</div>" }, { "name" : "numbers", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequence of numbers or values castable to numbers. The sequence can be of any length.</div>" } ], "returns" : { "type" : "xs:decimal", "description" : "The rank of $x." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">excel-err:Value if the parameters cannot be casted to numeric type.</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 3, "name" : "rank", "qname" : "excel:rank", "signature" : "($x as xs:anyAtomicType, $numbers as xs:anyAtomicType*, $order_ascending as xs:boolean) as xs:decimal", "description" : " Returns the rank of a number in a list of numbers.\n The rank of a number is its size relative to other values in a list.\n (If you were to sort the list, the rank of the number would be its position.)\n RANK gives duplicate numbers the same rank.\n", "summary" : "<p> Returns the rank of a number in a list of numbers.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "x", "type" : "xs:anyAtomicType", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The number whose rank you want to find.</div>" }, { "name" : "numbers", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The sequence of numbers or values castable to numbers. The sequence can be of any length.</div>" }, { "name" : "order_ascending", "type" : "xs:boolean", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> <dl>A boolean having the meaning: <dt>false</dt><dd>then rank the number as if the sequence was sorted in descending order.</dd> <dt>true</dt> <dd>then rank the number as if the sequence was sorted in ascending order.</dd></dl></div>" } ], "returns" : { "type" : "xs:decimal", "description" : "The rank of $x." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">excel-err:Value if the parameters cannot be casted to numeric type.</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 2, "name" : "slope", "qname" : "excel:slope", "signature" : "($known_y as xs:anyAtomicType+, $known_x as xs:anyAtomicType+) as xs:anyAtomicType", "description" : " Returns the slope of the linear regression line through data points in known_y's and known_x's.\n The slope is the vertical distance divided by the horizontal distance between\n   any two points on the line, which is the rate of change along the regression line.\n It computes the formula:<p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n sum((x - average_x)(y - average_y)) / sum((x - average_x)^2)  <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n where average_x and average_y are computed with AVERAGE function.\n", "summary" : "<p> Returns the slope of the linear regression line through data points in known_y's and known_x's.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "known_y", "type" : "xs:anyAtomicType", "occurrence" : "+", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequence of y numbers. The sequence can be of any length, from 1 up.</div>" }, { "name" : "known_x", "type" : "xs:anyAtomicType", "occurrence" : "+", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequence of x numbers. The sequence can be of any length, from 1 up.</div>" } ], "returns" : { "type" : "xs:anyAtomicType", "description" : "The slope value, as numeric type" }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">excel-err:Value if any parameter cannot be casted to numeric</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">fn:QName(\"http://zorba.io/modules/excel/errors\", \"excel-err:NA\") if there are different numbers of x's and y's</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">fn:QName(\"http://zorba.io/modules/excel/errors\", \"excel-err:NA\") if any sequence is empty</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">excel-err:Div0 if all x's are equal</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 2, "name" : "small", "qname" : "excel:small", "signature" : "($numbers as xs:anyAtomicType*, $k as xs:integer) as xs:anyAtomicType", "description" : " This function computes the k-th smallest value in a data set.\n Use this function to return values with a particular relative standing in a data set.\n If n is the number of data points in array, SMALL(array,1) equals the smallest value,\n   and SMALL(array,n) equals the largest value.\n", "summary" : "<p> This function computes the k-th smallest value in a data set.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "numbers", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> A sequence of numbers or values castable to numeric. The sequence can be of any length, from 1 up.</div>" }, { "name" : "k", "type" : "xs:integer", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The position (from the smallest) in the sequence of data to return. Must have value between 1 and size of sequence.</div>" } ], "returns" : { "type" : "xs:anyAtomicType", "description" : "The k-th smallest value of $numbers." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">excel-err:Value if the parameters cannot be casted to numeric type.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">excel-err:Num if the sequence is zero length.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">excel-err:Num if $k is not a value between 1 and the size of sequence.</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 3, "name" : "standardize", "qname" : "excel:standardize", "signature" : "($x as xs:anyAtomicType, $mean as xs:anyAtomicType, $standard_dev as xs:anyAtomicType) as xs:double", "description" : " Returns a normalized value from a distribution characterized by mean and standard_dev.<p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n The formula is (x - mean) / standard_dev .\n", "summary" : "<p> Returns a normalized value from a distribution characterized by mean and standard_dev.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "x", "type" : "xs:anyAtomicType", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> is the value you want to normalize</div>" }, { "name" : "mean", "type" : "xs:anyAtomicType", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> is the arithmetic mean of the distribution.</div>" }, { "name" : "standard_dev", "type" : "xs:anyAtomicType", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> is the standard deviation of the distribution.</div>" } ], "returns" : { "type" : "xs:double", "description" : "The normalized x, as numeric type" }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">excel-err:Value if any parameter cannot be casted to numeric</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">excel-err:Num if standard_dev is a value smaller than zero or equal</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 1, "name" : "var", "qname" : "excel:var", "signature" : "($numbers as xs:anyAtomicType+) as xs:anyAtomicType", "description" : " Estimates variance based on a sample.<p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n The formula is sum(x - average_x)^2 / (n - 1).<p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n average_x is computed with AVERAGE function.<p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n n is the count of numbers from the sequence, excluding empty values.\n", "summary" : "<p> Estimates variance based on a sample.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "numbers", "type" : "xs:anyAtomicType", "occurrence" : "+", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequence of numbers or values castable to numeric. The sequence can be of any length, from 1 up.</div>" } ], "returns" : { "type" : "xs:anyAtomicType", "description" : "The variance, as numeric type" }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">excel-err:Value if the parameters cannot be casted to numeric type</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 1, "name" : "vara", "qname" : "excel:vara", "signature" : "($numbers as xs:anyAtomicType+) as xs:anyAtomicType", "description" : " Estimates variance based on a sample.<p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n The formula is sum(x - average_x)^2 / (n - 1).<p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n average_x is computed with AVERAGE function.<p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n n is the size of sequence, including empty values.<p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n", "summary" : "<p> Estimates variance based on a sample.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "numbers", "type" : "xs:anyAtomicType", "occurrence" : "+", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequence of numbers or values castable to numeric. The sequence can be of any length, from 1 up.</div>" } ], "returns" : { "type" : "xs:anyAtomicType", "description" : "The variance, as numeric type" }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">excel-err:Value if the parameters cannot be casted to numeric type</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 1, "name" : "varp", "qname" : "excel:varp", "signature" : "($numbers as xs:anyAtomicType+) as xs:anyAtomicType", "description" : " Calculates variance based on the entire population.<p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n The formula is sum(x - average_x)^2 / n.<p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n average_x is computed with AVERAGE function.<p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n n is the count of numbers from the sequence, excluding empty values.<p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n", "summary" : "<p> Calculates variance based on the entire population.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "numbers", "type" : "xs:anyAtomicType", "occurrence" : "+", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequence of numbers or values castable to numeric. The sequence can be of any length, from 1 up.</div>" } ], "returns" : { "type" : "xs:anyAtomicType", "description" : "The variance, as numeric type" }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">excel-err:Value if the parameters cannot be casted to numeric type</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 1, "name" : "varpa", "qname" : "excel:varpa", "signature" : "($numbers as xs:anyAtomicType+) as xs:anyAtomicType", "description" : " Calculates variance based on the entire population.<p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n The formula is sum(x - average_x)^2 / n.<p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n average_x is computed with AVERAGE function.<p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n n is the size of sequence, including empty values.<p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n", "summary" : "<p> Calculates variance based on the entire population.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "numbers", "type" : "xs:anyAtomicType", "occurrence" : "+", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequence of numbers or values castable to numeric. The sequence can be of any length, from 1 up.</div>" } ], "returns" : { "type" : "xs:anyAtomicType", "description" : "The variance, as numeric type" }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">excel-err:Value if the parameters cannot be casted to numeric type</xqdoc:error>" ] } ], "variables" : [  ] }, "http://www.zorba-xquery.com/modules/languages/xslt" : { "ns" : "http://www.zorba-xquery.com/modules/languages/xslt", "description" : " This module provides XSLT 1.0 transformation functionality.\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">For details on XSLT see\n <a href=\"http://www.w3.org/TR/xslt\">XSLT 1.0 specification</a>.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">This module implements the invoking of an XSLT transformation from XQuery\n described in <a href=\"http://lists.w3.org/Archives/Member/w3c-xsl-wg/2008Apr/0052.html\">\n    Michael Kay's proposal</a>.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Example:\n <pre class=\"ace-static\" ace-mode=\"xquery\">import module namespace\n        xslt = \"http://www.zorba-xquery.com/modules/languages/xslt\";\n let $source :=\n     &lt;catalog&gt;\n         &lt;cd&gt;\n           &lt;title&gt;Empire Burlesque&lt;/title&gt;\n           &lt;artist&gt;Bob Dylan&lt;/artist&gt;\n           &lt;country&gt;USA&lt;/country&gt;\n           &lt;company&gt;Columbia&lt;/company&gt;\n           &lt;price&gt;10.90&lt;/price&gt;\n           &lt;year&gt;1985&lt;/year&gt;\n         &lt;/cd&gt;\n         &lt;cd&gt;\n           &lt;title&gt;Hide your heart&lt;/title&gt;\n           &lt;artist&gt;Bonnie Tyler&lt;/artist&gt;\n           &lt;country&gt;UK&lt;/country&gt;\n           &lt;company&gt;CBS Records&lt;/company&gt;\n           &lt;price&gt;9.90&lt;/price&gt;\n           &lt;year&gt;1988&lt;/year&gt;\n         &lt;/cd&gt;\n     &lt;/catalog&gt;\n let $stylesheet :=\n   &lt;xsl:stylesheet version=\"1.0\"\n       xmlns:xsl=\"http://www.w3.org/1999/XSL/Transform\"&gt;\n     &lt;xsl:template match=\"/\"&gt;\n       &lt;html&gt;\n       &lt;body&gt;\n       &lt;h2&gt;Music Collection&amp;lt;/h2&gt;\n         &lt;table border=\"1\"&gt;\n           &lt;tr bgcolor=\"lightblue\"&gt;\n             &lt;th&gt;Title&amp;lt;/th&gt;\n             &lt;th&gt;Artist&amp;lt;/th&gt;\n           &lt;/tr&gt;\n           &lt;xsl:for-each select=\"catalog/cd\"&gt;\n            &lt;tr&gt;\n              &lt;td&gt;&amp;lt;xsl:value-of select=\"title\"/&gt;&lt;/td&gt;\n              &lt;td&gt;&amp;lt;xsl:value-of select=\"artist\"/&gt;&lt;/td&gt;\n           &lt;/tr&gt;\n           &lt;/xsl:for-each&gt;\n         &lt;/table&gt;\n       &lt;/body&gt;\n       &lt;/html&gt;\n     &lt;/xsl:template&gt;\n   &lt;/xsl:stylesheet&gt;\n return\n   xslt:transform( $source, $stylesheet)</pre></p>\n", "sees" : [  ], "authors" : [ "<xqdoc:author xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Cezar Andrei</xqdoc:author>" ], "version" : null, "encoding" : "utf-8", "namespaces" : [ { "uri" : "http://zorba.io/options/versioning", "prefix" : "ver" }, { "uri" : "http://www.zorba-xquery.com/modules/languages/xslt", "prefix" : "xslt" } ], "functions" : [ { "isDocumented" : true, "arity" : 2, "name" : "transform", "qname" : "xslt:transform", "signature" : "($source as node(), $stylesheet as node()) as node() external", "description" : "<p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Invokes an XSLT transformation.</p>\n", "summary" : "<p>Invokes an XSLT transformation.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "source", "type" : "node()", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the input document to the transformation</div>" }, { "name" : "stylesheet", "type" : "node()", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the XSLT stylesheet module</div>" } ], "returns" : { "type" : "node()", "description" : "the result tree produced by the transformation" }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">xslt:XSLT001 if $stylesheet is not a valid XSLT stylesheet</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">xslt:XSLT002 if result can not be imported</xqdoc:error>" ] } ], "variables" : [  ] }, "http://zorba.io/modules/data-cleaning/phonetic-string-similarity" : { "ns" : "http://zorba.io/modules/data-cleaning/phonetic-string-similarity", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">This library module provides phonetic string similarity functions, comparing strings with basis on how they sound.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">These metrics are particularly effective in matching names, since names are often spelled in different\n ways that sound the same.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">The logic contained in this module is not specific to any particular XQuery implementation.</p>\n", "sees" : [  ], "authors" : [ "<xqdoc:author xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Bruno Martins</xqdoc:author>" ], "version" : null, "encoding" : "utf-8", "namespaces" : [ { "uri" : "http://zorba.io/modules/data-cleaning/phonetic-string-similarity", "prefix" : "simp" }, { "uri" : "http://zorba.io/options/versioning", "prefix" : "ver" } ], "functions" : [ { "isDocumented" : true, "arity" : 1, "name" : "metaphone-key", "qname" : "simp:metaphone-key", "signature" : "($s1 as xs:string) as xs:string", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Returns the Metaphone key for a given string.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">The Metaphone algorithm produces variable length keys as its output, as opposed to Soundex's fixed-length keys.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Example usage : <pre class=\"ace-static\" ace-mode=\"xquery\">metaphone-key(\"ALEKSANDER\")</pre></p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">The function invocation in the example above returns : <pre class=\"ace-static\" ace-mode=\"xquery\">\"ALKSNTR\"</pre></p>\n", "summary" : "<p>  Returns the Metaphone key for a given string.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "s1", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The string.</div>" } ], "returns" : { "type" : "xs:string", "description" : "The Metaphone key for the given input string." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 2, "name" : "metaphone", "qname" : "simp:metaphone", "signature" : "($s1 as xs:string, $s2 as xs:string) as xs:boolean", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Checks if two strings have the same Metaphone key.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Example usage : <pre class=\"ace-static\" ace-mode=\"xquery\">metaphone(\"ALEKSANDER\", \"ALEXANDRE\")</pre></p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">The function invocation in the example above returns : <pre class=\"ace-static\" ace-mode=\"xquery\">true</pre></p>\n", "summary" : "<p>  Checks if two strings have the same Metaphone key.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "s1", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The first string.</div>" }, { "name" : "s2", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The second string.</div>" } ], "returns" : { "type" : "xs:boolean", "description" : "Returns true if both strings have the same Metaphone key and false otherwise." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "soundex-key", "qname" : "simp:soundex-key", "signature" : "($s1 as xs:string) as xs:string", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Returns the Soundex key for a given string.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Example usage : <pre class=\"ace-static\" ace-mode=\"xquery\">soundex-key(\"Robert\")</pre></p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">The function invocation in the example above returns : <pre class=\"ace-static\" ace-mode=\"xquery\">\"R163\"</pre></p>\n", "summary" : "<p>  Returns the Soundex key for a given string.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "s1", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The string.</div>" } ], "returns" : { "type" : "xs:string", "description" : "The Soundex key for the given input string." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 2, "name" : "soundex", "qname" : "simp:soundex", "signature" : "($s1 as xs:string, $s2 as xs:string) as xs:boolean", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Checks if two strings have the same Soundex key.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Example usage : <pre class=\"ace-static\" ace-mode=\"xquery\">soundex( \"Robert\" , \"Rupert\" )</pre></p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">The function invocation in the example above returns : <pre class=\"ace-static\" ace-mode=\"xquery\">true</pre></p>\n", "summary" : "<p>  Checks if two strings have the same Soundex key.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "s1", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The first string.</div>" }, { "name" : "s2", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The second string.</div>" } ], "returns" : { "type" : "xs:boolean", "description" : "Returns true if both strings have the same Soundex key and false otherwise." }, "errors" : [  ] } ], "variables" : [  ] }, "http://www.28msec.com/modules/collections" : { "ns" : "http://www.28msec.com/modules/collections", "description" : " This module provides functions to work with collections. For example,\n it contains functions to retrieve the content of a collection or the\n names of all collections.\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">The module is always imported so you don't need to import it explicitly.\n Also, you don't need to fully qualify a function to invoke it.</p>\n", "sees" : [  ], "authors" : [  ], "version" : null, "encoding" : "utf-8", "namespaces" : [ { "uri" : "http://zorba.io/annotations", "prefix" : "an" }, { "uri" : "http://zorba.io/options/versioning", "prefix" : "ver" }, { "uri" : "http://zorba.io/errors", "prefix" : "zerr" } ], "functions" : [ { "isDocumented" : true, "arity" : 2, "name" : "apply-insert", "qname" : "db:apply-insert", "signature" : "($name as string, $content as item()*) as item()* external", "description" : " This function does the same as the insert function and it immediately applies\n the resulting pending updates and returns the items that have been inserted.\n Note that each item in the content sequence is copied before insertion. This\n function provides an efficient way to retrieve the actual copies that have\n been inserted.\n", "summary" : "<p> This function does the same as the insert function and it immediately applies\n the resulting pending updates and returns the items that have been inserted.</p>", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "name", "type" : "string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The name of the collection to which the items should be added.</div>" }, { "name" : "content", "type" : "item()", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The sequences of items whose copies should be added to the collection.</div>" } ], "returns" : { "type" : "item()*", "description" : "The result of the function is the sequence of items that have been inserted into the collection." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">zerr:ZDDY0003 if the collection identified by <tt>$name</tt> does not exist.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">zerr:ZDTY0001 if any of the items in the $content do not match the expected type (as specified in the collection declaration) or are not XML documents, XML elements, JSON objects, or JSON arrays.</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 0, "name" : "available-collections", "qname" : "db:available-collections", "signature" : "() as string* external", "description" : " Return the names of all existing collections.\n", "summary" : "<p> Return the names of all existing collections.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [  ], "returns" : { "type" : "string*", "description" : "The names of all existing collections." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "collection-name", "qname" : "db:collection-name", "signature" : "($o as item()) as string external", "description" : " The collection-name function returns the name of the containing collection\n of the given item.\n", "summary" : "<p> The collection-name function returns the name of the containing collection\n of the given item.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "o", "type" : "item()", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The item for which to get the name of the collection.</div>" } ], "returns" : { "type" : "string", "description" : "The name of the containing collection." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">zerr:ZDDY0011 if the given item does not belong to a collection.</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 1, "name" : "collection", "qname" : "db:collection", "signature" : "($name as string) as item()* external", "description" : " The collection function returns the sequence of items that belong\n to the collection identified by the given name.\n Please note that the order of the items returned is not deterministic,\n i.e. it might change between invocations. You can use\n <tt xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">db:collection($name, 0)</tt> to get a deterministic order.\n", "summary" : "<p> The collection function returns the sequence of items that belong\n to the collection identified by the given name.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "name", "type" : "string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The name of the collection.</div>" } ], "returns" : { "type" : "item()*", "description" : "The content of the collection." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">zerr:ZDDY0003 if the collection identified by $name does not exist.</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 2, "name" : "collection", "qname" : "db:collection", "signature" : "($name as string, $skip as integer) as item()* external", "description" : " The collection function returns the sequence of items that belong to\n the collection identified\n by the given name. The skip parameter allows to (efficiently) skip\n a given number of items.\n Note that the collections are generally unordered. However, there\n is an implicit deterministic ordering (i.e. sorting by the _id field\n in MongoDB) that is used by this function. This ordering is not present\n for the <tt xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">db:collection#1</tt> function. In order to return items with\n a stable ordering, db:collection($name, 0) can be used.\n", "summary" : "<p> The collection function returns the sequence of items that belong to\n the collection identified\n by the given name.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "name", "type" : "string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The name of the collection.</div>" }, { "name" : "skip", "type" : "integer", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The number of items to skip starting from the beginning.</div>" } ], "returns" : { "type" : "item()*", "description" : "The content of the collection starting at the <tt xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">$skip</tt>+1 item." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">zerr:ZDDY0003 if the collection identified by $name does not exist</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 3, "name" : "collection", "qname" : "db:collection", "signature" : "($name as string, $start as string, $skip as integer) as item()* external", "description" : " The collection function returns the sequence of items that belong to\n the collection identified\n by the given name. The start parameter is a reference and determines\n the first item to return. The skip parameter allows to (efficiently) skip\n a given number of items starting at the item referenced by <tt xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">$start</tt>.\n The start parameter is useful for efficiently implementing pagination.\n Note that the collections are generally unordered. However, there\n is an implicit deterministic ordering (i.e. sorting by the _id field\n in MongoDB) that is used by this function. This ordering is not present\n for the <tt xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">db:collection#1</tt> function. In order to return items with\n a stable ordering, db:collection($name, $start, 0) can be used.\n Example:\n <pre xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">\n import module namespace ref = \"http://zorba.io/modules/reference\";\n let $ref := ref:reference(db:collection(\"test\", 1)[1])\n return\n   db:collection(\"test\", $ref, 1)\n </pre>\n", "summary" : "<p> The collection function returns the sequence of items that belong to\n the collection identified\n by the given name.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "name", "type" : "string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The name of the collection.</div>" }, { "name" : "start", "type" : "string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The reference to the first item to return.</div>" }, { "name" : "skip", "type" : "integer", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The number of collection items to skip.</div>" } ], "returns" : { "type" : "item()*", "description" : "The content of the collection starting at the item referenced by <tt xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">$start</tt> and skipping <tt xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">$skip</tt> items." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">zerr:ZDDY0003 if the collection identified by <tt>$name</tt> does not exist.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">zerr:ZAPI0028 If the given reference $start is not a valid reference.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">zerr:ZSTR0066 if the given reference $start does not reference an item of this collection.</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 1, "name" : "create", "qname" : "db:create", "signature" : "($name as string) external", "description" : " The create function is an updating function that creates\n a new collection with the given name.\n It is not possible to create collections that start with\n \"system.\" or \"_28\" (zerr:ZDDY1000).\n Collections are identified by a name (string). Names are not allowed to\n contain the <tt xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">$</tt> sign or exceed the length of 70 characters.\n This is consistent with the restrictions for names of collections in MongoDB.\n", "summary" : "<p> The create function is an updating function that creates\n a new collection with the given name.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : true, "parameters" : [ { "name" : "name", "type" : "string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The name of the collection to create.</div>" } ], "returns" : { "type" : null, "description" : "The result of the function is an empty XDM instance and a pending update list which, once applied, creates a collection with the given name." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">zerr:ZDDY0002 if a collection with the given name already exists.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">zerr:ZDDY1000 if the given name is not a legal collection name</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 2, "name" : "create", "qname" : "db:create", "signature" : "($name as string, $content as item()*) external", "description" : " The create function is an updating function which creates\n a new collection with the given name. Moreover, it adds copies\n of the sequence <tt xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">$content</tt> to the new collection.\n It is not possible to create collections that start with\n \"system.\" or \"_28\" (zerr:ZDDY1000).\n Collections are identified by a name (string). Names are not allowed to\n contain the <tt xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">$</tt> sign or exceed the length of 70 characters.\n This is consistent with the restrictions for names of collections in MongoDB.\n", "summary" : "<p> The create function is an updating function which creates\n a new collection with the given name.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : true, "parameters" : [ { "name" : "name", "type" : "string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The string of the collection to create.</div>" }, { "name" : "content", "type" : "item()", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The sequences of items that should be added to the new collection.</div>" } ], "returns" : { "type" : null, "description" : "The result of the function is an empty XDM instance and a pending update list which, once applied, creates a collection with the given name and inserts the given items into it." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">zerr:ZDDY0002 if a collection with the given name already exists.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">zerr:ZDTY0001 if any of the items in the $content do not match the expected type (as specified in the collection declaration) or are not XML documents, XML elements, JSON objects, or JSON arrays.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">zerr:ZDDY1000 if the given name is not a legal collection name</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 1, "name" : "delete", "qname" : "db:delete", "signature" : "($target as item()*) external", "description" : " The delete function is an updating function that deletes zero or more items\n (JSON objects, JSON arrays, or XML nodes) from a collection.\n Please note that the all of the items belong to the same collection\n (zerr:ZDDY0011).\n", "summary" : "<p> The delete function is an updating function that deletes zero or more items\n (JSON objects, JSON arrays, or XML nodes) from a collection.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : true, "parameters" : [ { "name" : "target", "type" : "item()", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the items that should be deleted from the containing collection.</div>" } ], "returns" : { "type" : null, "description" : "The result of this function is an empty XDM instance and a pending update list which, once applied, deletes the items from the collections." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">zerr:ZDDY0003 if the collection identified by <tt>$name</tt> does not exist.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">zerr:ZDDY0011 if any item in the <tt>$target</tt> sequence is not a member of a collection or not all items belong to the same collection.</xqdoc:error>" ] }, { "isDocumented" : false, "arity" : 1, "name" : "drop", "qname" : "db:drop", "signature" : "($name as string) external", "description" : "", "summary" : "", "annotation_str" : "", "annotations" : [  ], "updating" : true, "parameters" : [ { "name" : "name", "type" : "string", "occurrence" : null, "description" : "" } ], "returns" : { "type" : null, "description" : "" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 2, "name" : "edit", "qname" : "db:edit", "signature" : "($target as item(), $content as item()) external", "description" : " The edit function is an updating function that edits the first supplied\n item so as to make it look exactly like a copy of the second supplied item,\n while retaining its original identity.\n", "summary" : "<p> The edit function is an updating function that edits the first supplied\n item so as to make it look exactly like a copy of the second supplied item,\n while retaining its original identity.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : true, "parameters" : [ { "name" : "target", "type" : "item()", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The target item, that must be edited.</div>" }, { "name" : "content", "type" : "item()", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The content item, that serves as an edit goal.</div>" } ], "returns" : { "type" : null, "description" : "The result of the function is an empty XDM instance and a pending update list which, once applied, performs the edit." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">zerr:ZDDY0017 if the $target item is not a member of a collection.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">zerr:ZDDY0040 if the target cannot be updated to match the content (for example because the target is a node and the content is an object).</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">zerr:ZDTY0001 if $content does not match the expected type (as specified in the collection declaration) according to the rules for SequenceType Matching.</xqdoc:error>" ] }, { "isDocumented" : false, "arity" : 1, "name" : "index-keys", "qname" : "db:index-keys", "signature" : "($index-name as string) as array()* external", "description" : "", "summary" : "", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "index-name", "type" : "string", "occurrence" : null, "description" : "" } ], "returns" : { "type" : "array()*", "description" : "" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 2, "name" : "insert", "qname" : "db:insert", "signature" : "($name as string, $content as item()*) external", "description" : " The insert function is an updating function that inserts copies of the given\n items into a collection.\n", "summary" : "<p> The insert function is an updating function that inserts copies of the given\n items into a collection.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : true, "parameters" : [ { "name" : "name", "type" : "string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The name of the collection to which the items should be added.</div>" }, { "name" : "content", "type" : "item()", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The sequences of items whose copies should be added.</div>" } ], "returns" : { "type" : null, "description" : "The result of the function is an empty XDM instance and a pending update list which, once applied, inserts the items into the collection." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">zerr:ZDDY0003 if the collection identified by <tt>$name</tt> does not exist.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">zerr:ZDTY0001 if any of the items in the $content do not match the expected type (as specified in the collection declaration) or are not XML documents, XML elements, JSON objects, or JSON arrays.</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 1, "name" : "is-available-collection", "qname" : "db:is-available-collection", "signature" : "($name as string) as boolean external", "description" : " Determine if the collection with the given name exists.\n", "summary" : "<p> Determine if the collection with the given name exists.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "name", "type" : "string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The string of the collection in question.</div>" } ], "returns" : { "type" : "boolean", "description" : "true if collection with the given name exists, false otherwise." }, "errors" : [  ] }, { "isDocumented" : false, "arity" : 2, "name" : "lookup", "qname" : "db:lookup", "signature" : "($index-name as string, $key as item()) as item()* external", "description" : "", "summary" : "", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "index-name", "type" : "string", "occurrence" : null, "description" : "" }, { "name" : "key", "type" : "item()", "occurrence" : null, "description" : "" } ], "returns" : { "type" : "item()*", "description" : "" }, "errors" : [  ] }, { "isDocumented" : false, "arity" : 3, "name" : "lookup", "qname" : "db:lookup", "signature" : "($index-name as string, $keys as item(), $skip as integer) as item()* external", "description" : "", "summary" : "", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "index-name", "type" : "string", "occurrence" : null, "description" : "" }, { "name" : "keys", "type" : "item()", "occurrence" : null, "description" : "" }, { "name" : "skip", "type" : "integer", "occurrence" : null, "description" : "" } ], "returns" : { "type" : "item()*", "description" : "" }, "errors" : [  ] }, { "isDocumented" : false, "arity" : 1, "name" : "refresh", "qname" : "db:refresh", "signature" : "($index-name as string) external", "description" : "", "summary" : "", "annotation_str" : "", "annotations" : [  ], "updating" : true, "parameters" : [ { "name" : "index-name", "type" : "string", "occurrence" : null, "description" : "" } ], "returns" : { "type" : null, "description" : "" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "truncate", "qname" : "db:truncate", "signature" : "($name as string) external", "description" : " The truncate function is an updating function that deletes the\n entire content of a given collection.\n Please note that applying this function can not be undone in case\n an error happens during the application of the containing PUL.\n", "summary" : "<p> The truncate function is an updating function that deletes the\n entire content of a given collection.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : true, "parameters" : [ { "name" : "name", "type" : "string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The name of the collection whose content to delete.</div>" } ], "returns" : { "type" : null, "description" : "The result of this function is an empty XDM instance and a pending update list which, once applied, deletes the nodes." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">zerr:ZDDY0003 if the collection identified by <tt>$name</tt> does not exist.</xqdoc:error>" ] } ], "variables" : [  ] }, "http://expath.org/ns/http-client" : { "ns" : "http://expath.org/ns/http-client", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">\n This module provides an implementation of the\n <a href=\"http://expath.org/modules/http-client/\">EXPath Http Client</a>.\n It provides functions for making HTTP requests and is a superset of the\n module specified by EXPath.\n Specifically, it implements the <code>http:send-request()</code> functions\n as specified by EXPath. Moreover, it adds an additional function\n <code>http:read()</code> (with several arities for the sake of ease).\n </p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">\n In general, both functions take a description of the HTTP request to make\n as parameter, execute the request, and return a representation of the HTTP\n response. For instance, in the following code snippet, we fetch the Zorba\n home page:\n </p>\n <pre xmlns:xqdoc=\"http://www.xqdoc.org/1.0\" class=\"ace-static\" ace-mode=\"xquery\">import module namespace http = \"http://expath.org/ns/http-client\";\n http:send-request(\n  &lt;http:request href=\"http://zorba.io\" method=\"get\" /&gt;\n )\n </pre>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">\n The <code>http:send-request()</code> functions are declared as sequential.\n Sequential functions are allowed to have side effects. For example, most probably,\n an HTTP POST request is a request that has side effects because it adds/changes\n a remote resource. Sequential functions are specified in the\n <a href=\"http://zorba.io/documentation/2.9/zorba/scripting_tutorial.html\">XQuery Scripting Extension</a>.\n In contrast, the http:read() functions are not declared as sequential -\n they are declared as being non deterministic though, which\n means that several calls may return different results.\n HTTP requests performed using these functions are <b>not</b> allowed to have\n side effects.\n </p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">\n The response is returned as a sequence of one or more items. The first\n one is an <code>http:response</code> element with quite the same\n structure as an http:request, but without the content itself.\n The content is returned as the second item (or several items in case of\n a multipart response) as a string, a document node, or a binary item.\n This depends on the content-type returned.\n Specifically, the rules are as follows:\n <ul>\n  <li>A document node is returned if the media type has a MIME type of\n     text/xml, application/xml, text/xml-external-parsed-entity, or\n     application/xml-external-parsed-entity, as defined in [RFC 3023]\n     (except that application/xml-dtd is considered a text media type).\n     MIME types ending by +xml are also XML media types.</li>\n  <li>A document node is returned if the media type has a MIME type of\n      text/html. In order to be able to make HTML parseable, tidy is automatically\n      invoked. If you want to prevent that, you can also set your own content-type\n      by setting the override-media-type attribute in the request element.\n       For tidying, the following <a href=\"http://tidy.sourceforge.net/docs/quickref.html\">options</a>\n       will be used:\n       <ul>\n         <li>TidyXmlOut=yes</li>\n         <li>TidyDoctypeMode=TidyDoctypeOmit</li>\n         <li>TidyQuoteNbsp=yes</li>\n         <li>TidyCharEncoding=\"utf8\"</li>\n         <li>TidyNewline=\"LF\"</li>\n       </ul>\n  </li>\n  <li>An xs:string item is returned if the media type has a text MIME type,\n     i.e. beginning with text/.</li>\n  <li>An xs:base64Binary item is returned for all the other media types.</li>\n </ul>\n </p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">\n The structure of a request element is defined in the schema that is imported\n by this module. The details are described in the\n <a href=\"http://expath.org/spec/http-client#d2e183\">specification</a>.\n Analogously, the response element is also described in this\n <a href=\"http://expath.org/spec/http-client#d2e491\">specification</a>.\n </p>\n", "sees" : [ "<xqdoc:see xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"><a href=\"http://www.w3.org/TR/xquery-3/#FunctionDeclns\">XQuery 3.0: Function Declaration</a></xqdoc:see>" ], "authors" : [ "<xqdoc:author xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Federico Cavalieri, Markus Pilman</xqdoc:author>" ], "version" : null, "encoding" : "utf-8", "namespaces" : [ { "uri" : "http://zorba.io/annotations", "prefix" : "an" }, { "uri" : "http://expath.org/ns/error", "prefix" : "err" }, { "uri" : "http://expath.org/ns/http-client", "prefix" : "http" }, { "uri" : "http://zorba.io/modules/http-client-wrapper", "prefix" : "http-wrapper" }, { "uri" : "http://expath.org/ns/http-client", "prefix" : "https" }, { "uri" : "http://www.zorba-xquery.com/modules/converters/html", "prefix" : "tidy" }, { "uri" : "http://www.zorba-xquery.com/modules/converters/html-options", "prefix" : "tidy-options" }, { "uri" : "http://zorba.io/options/versioning", "prefix" : "ver" } ], "functions" : [ { "isDocumented" : true, "arity" : 1, "name" : "send-request", "qname" : "http:send-request", "signature" : "($request as element(*)) as item()+", "description" : " Function for convenience.\n Calling this function is equivalent to calling\n <code xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">\n http:send-request($request, (), ())\n </code>\n", "summary" : "<p> Function for convenience.</p>", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "request", "type" : "element(*)", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> see request parameter of the sequential <a href=\"#send-request-3\">send-request</a> function with three parameters.</div>" } ], "returns" : { "type" : "item()+", "description" : "see return value of the sequential <a xmlns:xqdoc=\"http://www.xqdoc.org/1.0\" href=\"#send-request-3\">send-request</a> function with three parameters." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 2, "name" : "send-request", "qname" : "http:send-request", "signature" : "($request as element(*)?, $href as xs:string?) as item()+", "description" : " Function for convenience.\n Calling this function is equivalent to calling\n <code xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">\n http:send-request($request, $href, ())\n </code>\n", "summary" : "<p> Function for convenience.</p>", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "request", "type" : "element(*)", "occurrence" : "?", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> see request parameter of the sequential <a href=\"#send-request-3\">send-request</a> function with three parameters.</div>" }, { "name" : "href", "type" : "xs:string", "occurrence" : "?", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> see href parameter of the sequential <a href=\"#send-request-3\">send-request</a> function with three parameters.</div>" } ], "returns" : { "type" : "item()+", "description" : "see return of <a xmlns:xqdoc=\"http://www.xqdoc.org/1.0\" href=\"#send-request-3\">send-request</a>" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 3, "name" : "send-request", "qname" : "http:send-request", "signature" : "($request as element(*)?, $href as xs:string?, $bodies as item()*) as item()+", "description" : " This function sends an HTTP request and returns the corresponding response.\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">\n This function is declared as sequential (see XQuery Scripting).\n Sequential functions are allowed to have side effects. For example, most probably,\n an HTTP POST request is a request that has side effects because it adds/changes\n a remote resource.\n </p>\n", "summary" : "<p> This function sends an HTTP request and returns the corresponding response.</p>", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "request", "type" : "element(*)", "occurrence" : "?", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> Contains the various parameters of the request. See the <a href=\"http://expath.org/spec/http-client#d2e183\">specification</a>. for a full description of the structure of this element.</div>" }, { "name" : "href", "type" : "xs:string", "occurrence" : "?", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> is the HTTP or HTTPS URI to send the request to. It must be a valid xs:anyURI, but is declared as a string to be able to pass literal strings (without requiring to explicitly cast it to an xs:anyURI.)</div>" }, { "name" : "bodies", "type" : "item()", "occurrence" : "*", "description" : "" } ], "returns" : { "type" : "item()+", "description" : "a sequence of items, where the first item is a element of type http:responseType. The response element is also described in the <a xmlns:xqdoc=\"http://www.xqdoc.org/1.0\" href=\"http://expath.org/spec/http-client#d2e483\">specification</a>. If there is one (or several, in case of multipart) response body, the response bodies are the next items in the sequence." }, "errors" : [  ] } ], "variables" : [  ] }, "http://zorba.io/modules/schema" : { "ns" : "http://zorba.io/modules/schema", "description" : " This module provides function that are related to XML Schema support\n in Zorba.\n", "sees" : [  ], "authors" : [ "<xqdoc:author xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Cezar Andrei, Matthias Brantner</xqdoc:author>" ], "version" : null, "encoding" : "utf-8", "namespaces" : [ { "uri" : "http://zorba.io/modules/schema", "prefix" : "schema" }, { "uri" : "http://zorba.io/options/versioning", "prefix" : "ver" } ], "functions" : [ { "isDocumented" : true, "arity" : 1, "name" : "is-validated", "qname" : "schema:is-validated", "signature" : "($node as node()) as xs:boolean external", "description" : " This function returns true if the given node has been validated,\n and false otherwise.\n", "summary" : "<p> This function returns true if the given node has been validated,\n and false otherwise.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "node", "type" : "node()", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the node item that should be checked for validation</div>" } ], "returns" : { "type" : "xs:boolean", "description" : "true if the given node has been validated, false otherwise." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "schema-type", "qname" : "schema:schema-type", "signature" : "($item as item()) as xs:QName? external", "description" : " This function returns the name of the type of the item passed\n as parameter.\n", "summary" : "<p> This function returns the name of the type of the item passed\n as parameter.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "item", "type" : "item()", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the item from which the name of the type should be returned.</div>" } ], "returns" : { "type" : "xs:QName?", "description" : "the name of the type (as QName) of the item passed as parameter." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "validate-in-place", "qname" : "schema:validate-in-place", "signature" : "($node as node()) external", "description" : " Updating function that validates the document in place. After the updating\n query is applied the $node will contain the validated content.\n", "summary" : "<p> Updating function that validates the document in place.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : true, "parameters" : [ { "name" : "node", "type" : "node()", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the document or element to be validated, otherwise error</div>" } ], "returns" : { "type" : null, "description" : "The result of the function is an empty XDM instance and a pending update list that consists the schema:validate-in-place($node)) primitive." }, "errors" : [  ] } ], "variables" : [  ] }, "http://www.28msec.com/modules/mongodb" : { "ns" : "http://www.28msec.com/modules/mongodb", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">This module provides a driver to access a\n <a href=\"http://www.mongodb.org/\">MongoDB</a> database - similar to\n drivers for other high-level languages like e.g.\n <a href=\"http://api.mongodb.org/python/current/\">PyMongo</a>.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Here is a simple example of how we can raise the salary of each\n developer by 10%.\n Starting with an <em>employees</em> collection in MongoDB that contains\n <pre>\n { \"name\" : \"Peter\", \"role\" : \"developer\" , \"salary\" : 80 }\n { \"name\" : \"Paul\",  \"role\" : \"developer\" , \"salary\" : 75 }\n { \"name\" : \"Mary\",  \"role\" : \"manager\"   , \"salary\" : 90 } </pre>\n we can get a connection\n <pre>\n variable $conn := mongo:connect(\"hostname\", 27017, \"db\", \"user\", \"password\");\n </pre>\n run this update\n <pre>\n for $emp in mongo:find($conn, \"employees\")\n where $emp(\"role\") = \"developer\"\n let $salary := $emp(\"salary\")\n return {\n   replace value of json $emp(\"salary\") with $salary * 1.1;\n   mongo:save($conn, \"employees\", $emp)\n };\n </pre>\n and get the names and the current salaries using\n <pre>\n mongo:find($conn, \"employees\", {}, { \"_id\" : false, \"role\" : false }, {}) </pre>\n In this query we have removed the <em>_id</em> and <em>role</em> fields from\n the results and get\n <pre>\n { \"name\" : \"Peter\", \"salary\" : 88   }\n { \"name\" : \"Paul\",  \"salary\" : 82.5 }\n { \"name\" : \"Mary\",  \"salary\" : 90   }\n </pre>\n </p>\n <h2 xmlns:xqdoc=\"http://www.xqdoc.org/1.0\" id=\"determinism\">Important Notice Regarding Function Determinism</h2>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">The non side-effecting functions:\n <ul>\n   <li><a href=\"?anchor=connect-0\">connect#0</a></li>\n   <li><a href=\"?anchor=connect-1\">connect#1</a></li>\n   <li><a href=\"?anchor=connect-2\">connect#2</a></li>\n   <li><a href=\"?anchor=connect-5\">connect#5</a></li>\n   <li><a href=\"?anchor=collection-names-1\">collection-names#1</a></li>\n   <li><a href=\"?anchor=count-2\">count#2</a></li>\n   <li><a href=\"?anchor=count-3\">count#3</a></li>\n   <li><a href=\"?anchor=find-2\">find#2</a></li>\n   <li><a href=\"?anchor=find-3\">find#3</a></li>\n   <li><a href=\"?anchor=find-4\">find#4</a></li>\n   <li><a href=\"?anchor=find-5\">find#5</a></li>\n </ul>\n are declared deterministic, which means that their results could be cached\n when invoked multiple times with the same arguments in the same query execution.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">To not use cached results you can use the following alternative functions:\n <ul>\n   <li><a href=\"?anchor=connect-nondeterministic-0\">connect-nondeterministic#0</a></li>\n   <li><a href=\"?anchor=connect-nondeterministic-1\">connect-nondeterministic#1</a></li>\n   <li><a href=\"?anchor=connect-nondeterministic-2\">connect-nondeterministic#2</a></li>\n   <li><a href=\"?anchor=connect-nondeterministic-5\">connect-nondeterministic#5</a></li>\n   <li><a href=\"?anchor=collection-names-nondeterministic-1\">collection-names-nondeterministic#1</a></li>\n   <li><a href=\"?anchor=count-nondeterministic-2\">count-nondeterministic#2</a></li>\n   <li><a href=\"?anchor=count-nondeterministic-3\">count-nondeterministic#3</a></li>\n   <li><a href=\"?anchor=find-nondeterministic-2\">find-nondeterministic#2</a></li>\n   <li><a href=\"?anchor=find-nondeterministic-3\">find-nondeterministic#3</a></li>\n   <li><a href=\"?anchor=find-nondeterministic-4\">find-nondeterministic#4</a></li>\n   <li><a href=\"?anchor=find-nondeterministic-5\">find-nondeterministic#5</a></li>\n </ul>\n which have been declared as being non deterministic.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Three different functions have been defined to run mongo commands:\n <ul>\n   <li><a href=\"?anchor=run-cmd-2\">run-cmd#2</a></li>\n   <li><a href=\"?anchor=run-cmd-nondeterministic-2\">run-cmd-nondeterministic#2</a></li>\n   <li><a href=\"?anchor=run-cmd-deterministic-2\">run-cmd-deterministic#2</a></li>\n </ul>\n If your application depends on the ordering of side-effects from commands issued\n through these functions you should use <a href=\"?anchor=run-cmd-2\">run-cmd#2</a>,\n which has been declared as sequential.\n For non-side-effecting commands you can also use\n <a href=\"?anchor=run-cmd-nondeterministic-2\">run-cmd-nondeterministic#2</a> and\n <a href=\"?anchor=run-cmd-deterministic-2\">run-cmd-deterministic#2</a>.\n The results of commands executed by means of\n <a href=\"?anchor=run-cmd-deterministic-2\">run-cmd-deterministic#2</a> (which is\n declared <i>deterministic</i>) could be cached, whereas the results of\n commands executed by means of\n <a href=\"?anchor=run-cmd-nondeterministic-2\">run-cmd-nondeterministic#2</a> (which is\n declared <i>non-deterministic</i>) are never cached.</p>\n", "sees" : [  ], "authors" : [ "<xqdoc:author xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">28msec</xqdoc:author>" ], "version" : null, "encoding" : "utf-8", "namespaces" : [ { "uri" : "http://zorba.io/annotations", "prefix" : "an" }, { "uri" : "http://www.28msec.com/modules/credentials", "prefix" : "credentials" }, { "uri" : "http://www.28msec.com/modules/mongodb/types", "prefix" : "m-schema" }, { "uri" : "http://www.28msec.com/modules/mongodb", "prefix" : "mongo" }, { "uri" : "http://zorba.io/options/versioning", "prefix" : "ver" } ], "functions" : [ { "isDocumented" : true, "arity" : 1, "name" : "collection-names-nondeterministic", "qname" : "mongo:collection-names-nondeterministic", "signature" : "($db as xs:anyURI) as xs:string* external", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Get a list of all the collection names in this database.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">This function has the same semantics as\n <a href=\"#collection-names-1\">collection-names#1</a>,\n but is declared as being non deterministic and thus should only be used when\n result caching is not desired.</p>\n", "summary" : "<p>  Get a list of all the collection names in this database.</p>", "annotation_str" : " %an:nondeterministic", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "nondeterministic", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "db", "type" : "xs:anyURI", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> a database connection identifier</div>" } ], "returns" : { "type" : "xs:string*", "description" : "a list of the names of all collection in the given database." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">mongo:MONGO004 invalid database identifier</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">mongo:MONGO005 if any mongodb error happens</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 1, "name" : "collection-names", "qname" : "mongo:collection-names", "signature" : "($db as xs:anyURI) as xs:string* external", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Get a list of all the collection names in this database.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">This function is declared as deterministic and should be used whenever result\n caching is acceptable.</p>\n", "summary" : "<p>  Get a list of all the collection names in this database.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "db", "type" : "xs:anyURI", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> a database connection identifier</div>" } ], "returns" : { "type" : "xs:string*", "description" : "a list of the names of all collection in the given database." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">mongo:MONGO004 invalid database identifier</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">mongo:MONGO005 if any mongodb error happens</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 0, "name" : "connect-nondeterministic", "qname" : "mongo:connect-nondeterministic", "signature" : "() as xs:anyURI", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Connect to this project's default MongoDB database.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">This function has the same semantics as <a href=\"#connect-0\">connect#0</a>,\n but is declared as being non deterministic and thus should only be used when\n result caching is not desired.</p>\n", "summary" : "<p>  Connect to this project's default MongoDB database.</p>", "annotation_str" : " %an:nondeterministic", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "nondeterministic", "value" : "" } ], "updating" : false, "parameters" : [  ], "returns" : { "type" : "xs:anyURI", "description" : "an identifier for a connection to the MongoDB database." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">mongo:MONGO002 connection to MongoDB failed</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">mongo:MONGO003 authentication to the MongoDB database failed</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 1, "name" : "connect-nondeterministic", "qname" : "mongo:connect-nondeterministic", "signature" : "($connection-config as item()) as xs:anyURI", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Connect to a MongoDB database.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">This function has the same semantics as <a href=\"#connect-1\">connect#1</a>,\n but is declared as being non deterministic and thus should only be used when\n result caching is not desired.</p>\n", "summary" : "<p>  Connect to a MongoDB database.</p>", "annotation_str" : " %an:nondeterministic", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "nondeterministic", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "connection-config", "type" : "item()", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the connection specification.</div>" } ], "returns" : { "type" : "xs:anyURI", "description" : "an identifier for a connection to the MongoDB database." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">mongo:MONGO001 if the connection specification is invalid</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">mongo:MONGO002 connection to MongoDB failed</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">mongo:MONGO003 authentication to the MongoDB database failed</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 2, "name" : "connect-nondeterministic", "qname" : "mongo:connect-nondeterministic", "signature" : "($credentials-name as xs:string?, $options as object()) as xs:anyURI", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Connect to a MongoDB database.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">This function has the same semantics as <a href=\"#connect-2\">connect#2</a>,\n but is declared as being non deterministic and thus should only be used when\n result caching is not desired.</p>\n", "summary" : "<p>  Connect to a MongoDB database.</p>", "annotation_str" : " %an:nondeterministic", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "nondeterministic", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "credentials-name", "type" : "xs:string", "occurrence" : "?", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the name of the credentials to use.</div>" }, { "name" : "options", "type" : "object()", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the name of the credentials to use.</div>" } ], "returns" : { "type" : "xs:anyURI", "description" : "an identifier for a connection to the MongoDB database." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">mongo:MONGO001 if the connection specification is invalid</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">mongo:MONGO002 connection to MongoDB failed</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">mongo:MONGO003 authentication to the MongoDB database failed</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 5, "name" : "connect-nondeterministic", "qname" : "mongo:connect-nondeterministic", "signature" : "($host as xs:string, $port as xs:integer?, $db as xs:string, $user as xs:string?, $pass as xs:string?) as xs:anyURI", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Connect to a MongoDB database.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">This function has the same semantics as <a href=\"#connect-5\">connect#5</a>,\n but is declared as being non deterministic and thus should only be used when\n result caching is not desired.</p>\n", "summary" : "<p>  Connect to a MongoDB database.</p>", "annotation_str" : " %an:nondeterministic", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "nondeterministic", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "host", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the host to connect to</div>" }, { "name" : "port", "type" : "xs:integer", "occurrence" : "?", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the port to connect to</div>" }, { "name" : "db", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the database to connect to</div>" }, { "name" : "user", "type" : "xs:string", "occurrence" : "?", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the user used to authorize access to the db</div>" }, { "name" : "pass", "type" : "xs:string", "occurrence" : "?", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the password used to authorize access to the db</div>" } ], "returns" : { "type" : "xs:anyURI", "description" : "an identifier for a connection to the MongoDB database." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">mongo:MONGO002 connection to MongoDB failed</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">mongo:MONGO003 authentication to the MongoDB database failed</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 0, "name" : "connect", "qname" : "mongo:connect", "signature" : "() as xs:anyURI", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Connect to this project's default MongoDB database.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">This function is declared as deterministic and should be used whenever result\n caching is acceptable.</p>\n", "summary" : "<p>  Connect to this project's default MongoDB database.</p>", "annotation_str" : " %an:strictlydeterministic", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "strictlydeterministic", "value" : "" } ], "updating" : false, "parameters" : [  ], "returns" : { "type" : "xs:anyURI", "description" : "an identifier for a connection to the MongoDB database." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">mongo:MONGO002 connection to MongoDB failed</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">mongo:MONGO003 authentication to the MongoDB database failed</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 1, "name" : "connect", "qname" : "mongo:connect", "signature" : "($connection-config as item()) as xs:anyURI", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Connect to a MongoDB database.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">This function is declared as deterministic and should be used whenever result\n caching is acceptable.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">The $connection-config parameter is used to specify the connection information.\n If a string is used, then the function will interpret it as the name of\n a credential in the MongoDB category.\n If an object is used, then the function will open a connection using it.\n The object structure is the following:\n <ul>\n   <li>host (string; mandatory)</li>\n   <li>port (integer; default: 27017)</li>\n   <li>db (string; mandatory)</li>\n   <li>user (string)</li>\n   <li>pass (string)</li>\n   <li>timeout (decimal; default: 0)</li>\n   <li>pre-digested (boolean; default: false)</li>\n </ul>\n </p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">For example, using stored credential:\n <pre>mongo:connect(\"credentials-name\")</pre>\n <p>For example, specifying the connection information:</p>\n <pre>mongo:connect(\n   {\n     \"host\": \"hostname\",\n     \"port\": 11011,\n     \"db\": \"mydb\",\n     \"user\": \"myuser\"\n     \"password: \"mypass\"\n   })</pre>.\n </p>\n", "summary" : "<p>  Connect to a MongoDB database.</p>", "annotation_str" : " %an:strictlydeterministic", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "strictlydeterministic", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "connection-config", "type" : "item()", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the connection specification.</div>" } ], "returns" : { "type" : "xs:anyURI", "description" : "an identifier for a connection to the MongoDB database." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">mongo:MONGO001 if the connection specification is invalid</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">mongo:MONGO002 connection to MongoDB failed</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">mongo:MONGO003 authentication to the MongoDB database failed</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 2, "name" : "connect", "qname" : "mongo:connect", "signature" : "($credentials-name as xs:string?, $options as object()) as xs:anyURI", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Connect to a MongoDB database.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">This function is declared as deterministic and should be used whenever result\n caching is acceptable.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">The $credentials-name parameter is used to specify the connection information.\n If empty a connection will be opened to the project default MongoDB database.\n Otherwise, the function will use it to identify a credential in the MongoDB category.\n </p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The $options object can be used to specify connection options. The following\n fields are supported:\n <ul>\n   <li>timeout (decimal; default: 0)</li>\n </ul>\n </p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">For example, connecting to the project default MongoDB database:\n <pre>mongo:connect((), {\"timeout\": 10})</pre>\n </p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">For example, specifying the connection information:\n <pre>mongo:connect(\"credentials-name\", {\"timeout\": 10})</pre>\n </p>\n", "summary" : "<p>  Connect to a MongoDB database.</p>", "annotation_str" : " %an:strictlydeterministic", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "strictlydeterministic", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "credentials-name", "type" : "xs:string", "occurrence" : "?", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the name of the credentials to use.</div>" }, { "name" : "options", "type" : "object()", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the name of the credentials to use.</div>" } ], "returns" : { "type" : "xs:anyURI", "description" : "an identifier for a connection to the MongoDB database." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">mongo:MONGO001 if the connection specification is invalid</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">mongo:MONGO002 connection to MongoDB failed</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">mongo:MONGO003 authentication to the MongoDB database failed</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 5, "name" : "connect", "qname" : "mongo:connect", "signature" : "($host as xs:string, $port as xs:integer?, $db as xs:string, $user as xs:string?, $pass as xs:string?) as xs:anyURI", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Connect to a MongoDB database.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">This function is declared as deterministic and should be used whenever result\n caching is acceptable.</p>\n", "summary" : "<p>  Connect to a MongoDB database.</p>", "annotation_str" : " %an:strictlydeterministic", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "strictlydeterministic", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "host", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the host to connect to</div>" }, { "name" : "port", "type" : "xs:integer", "occurrence" : "?", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the port to connect to</div>" }, { "name" : "db", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the database to connect to</div>" }, { "name" : "user", "type" : "xs:string", "occurrence" : "?", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the user used to authorize access to the db</div>" }, { "name" : "pass", "type" : "xs:string", "occurrence" : "?", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the password used to authorize access to the db</div>" } ], "returns" : { "type" : "xs:anyURI", "description" : "an identifier for a connection to the MongoDB database." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">mongo:MONGO002 connection to MongoDB failed</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">mongo:MONGO003 authentication to the MongoDB database failed</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 3, "name" : "copy", "qname" : "mongo:copy", "signature" : "($db as xs:anyURI, $from-db as xs:string, $to-db as xs:string) as empty-sequence()", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Copies a MongoDB database. Be aware, you must call\n this function on the admin database.</p>\n", "summary" : "<p>  Copies a MongoDB database.</p>", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "db", "type" : "xs:anyURI", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> a database connection identifier</div>" }, { "name" : "from-db", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the name of the source database</div>" }, { "name" : "to-db", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the name of the target database</div>" } ], "returns" : { "type" : "empty-sequence()", "description" : "the function has side-effects and returns the empty sequence." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">mongo:MONGO004 invalid database identifier</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">mongo:MONGO005 if copy operation fails</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">mongo:MONGO006 if any mongodb error happens</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 2, "name" : "count-nondeterministic", "qname" : "mongo:count-nondeterministic", "signature" : "($db as xs:anyURI, $coll as xs:string) as xs:integer external", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Counts the number of documents in the given collection.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">This function has the same semantics as <a href=\"#count-2\">count#2</a>,\n but is declared as being non deterministic and thus should only be used when\n result caching is not desired.</p>\n", "summary" : "<p>  Counts the number of documents in the given collection.</p>", "annotation_str" : " %an:nondeterministic", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "nondeterministic", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "db", "type" : "xs:anyURI", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> a database connection identifier</div>" }, { "name" : "coll", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the name of the collection</div>" } ], "returns" : { "type" : "xs:integer", "description" : "the said count" }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">mongo:MONGO005 if any mongodb error happens</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 3, "name" : "count-nondeterministic", "qname" : "mongo:count-nondeterministic", "signature" : "($db as xs:anyURI, $coll as xs:string, $query as object()) as xs:integer external", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Counts the number of documents satisfying the query in the given collection.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">This function has the same semantics as <a href=\"#count-3\">count#3</a>,\n but is declared as being non deterministic and thus should only be used when\n result caching is not desired.</p>\n", "summary" : "<p>  Counts the number of documents satisfying the query in the given collection.</p>", "annotation_str" : " %an:nondeterministic", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "nondeterministic", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "db", "type" : "xs:anyURI", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> a database connection identifier</div>" }, { "name" : "coll", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the name of the collection</div>" }, { "name" : "query", "type" : "object()", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the query specifying which objects to count</div>" } ], "returns" : { "type" : "xs:integer", "description" : "the said count" }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">mongo:MONGO005 if any mongodb error happens</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 2, "name" : "count", "qname" : "mongo:count", "signature" : "($db as xs:anyURI, $coll as xs:string) as xs:integer external", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Counts the number of documents in the given collection.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">This function is declared as deterministic and should be used whenever result\n caching is acceptable.</p>\n", "summary" : "<p>  Counts the number of documents in the given collection.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "db", "type" : "xs:anyURI", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> a database connection identifier</div>" }, { "name" : "coll", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the name of the collection</div>" } ], "returns" : { "type" : "xs:integer", "description" : "the said count" }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">mongo:MONGO005 if any mongodb error happens</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 3, "name" : "count", "qname" : "mongo:count", "signature" : "($db as xs:anyURI, $coll as xs:string, $query as object()) as xs:integer external", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Counts the number of documents satisfying the query in the given collection.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">This function is declared as deterministic and should be used whenever result\n caching is acceptable.</p>\n", "summary" : "<p>  Counts the number of documents satisfying the query in the given collection.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "db", "type" : "xs:anyURI", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> a database connection identifier</div>" }, { "name" : "coll", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the name of the collection</div>" }, { "name" : "query", "type" : "object()", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the query specifying which objects to count</div>" } ], "returns" : { "type" : "xs:integer", "description" : "the said count" }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">mongo:MONGO005 if any mongodb error happens</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 1, "name" : "disconnect", "qname" : "mongo:disconnect", "signature" : "($db as xs:anyURI) as empty-sequence() external", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Disconnect from a MongoDB database.</p>\n", "summary" : "<p>  Disconnect from a MongoDB database.</p>", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "db", "type" : "xs:anyURI", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> a database connection identifier</div>" } ], "returns" : { "type" : "empty-sequence()", "description" : "the function has side-effects and returns the empty sequence." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">mongo:MONGO004 invalid database identifier</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">mongo:MONGO005 if any mongodb error happens</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 2, "name" : "drop-collection", "qname" : "mongo:drop-collection", "signature" : "($db as xs:anyURI, $coll as xs:string) as empty-sequence() external", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Drop a collection.</p>\n", "summary" : "<p>  Drop a collection.</p>", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "db", "type" : "xs:anyURI", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> a database connection identifier</div>" }, { "name" : "coll", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the name of the collection</div>" } ], "returns" : { "type" : "empty-sequence()", "description" : "the function has side-effects and returns the empty sequence." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">mongo:MONGO004 invalid database identifier</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">mongo:MONGO005 if any mongodb error happens</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 2, "name" : "find-nondeterministic", "qname" : "mongo:find-nondeterministic", "signature" : "($db as xs:anyURI, $coll as xs:string) as object()*", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Returns all objects of the given collection.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">This function has the same semantics as <a href=\"#find-2\">find#2</a>,\n but is declared as being non deterministic and thus should only be used when\n result caching is not desired.</p>\n", "summary" : "<p>  Returns all objects of the given collection.</p>", "annotation_str" : " %an:nondeterministic", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "nondeterministic", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "db", "type" : "xs:anyURI", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> a database connection identifier</div>" }, { "name" : "coll", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the name of the collection</div>" } ], "returns" : { "type" : "object()*", "description" : "all objects of the given collection" }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">mongo:MONGO004 invalid database identifier</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">mongo:MONGO005 if any mongodb error happens</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 3, "name" : "find-nondeterministic", "qname" : "mongo:find-nondeterministic", "signature" : "($db as xs:anyURI, $coll as xs:string, $query as object()) as object()*", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Performs a query operation on the given collection and\n returns all matches.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">This function has the same semantics as <a href=\"#find-3\">find#3</a>,\n but is declared as being non deterministic and thus should only be used when\n result caching is not desired.</p>\n", "summary" : "<p>  Performs a query operation on the given collection and\n returns all matches.</p>", "annotation_str" : " %an:nondeterministic", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "nondeterministic", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "db", "type" : "xs:anyURI", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> a database connection identifier</div>" }, { "name" : "coll", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the name of the collection</div>" }, { "name" : "query", "type" : "object()", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the query operation to perform</div>" } ], "returns" : { "type" : "object()*", "description" : "all matches returned by the given query operation" }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">mongo:MONGO004 invalid database identifier</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">mongo:MONGO005 if any mongodb error happens</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">mongo:MONGO006 if the given query could not be converted to BSON</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 4, "name" : "find-nondeterministic", "qname" : "mongo:find-nondeterministic", "signature" : "($db as xs:anyURI, $coll as xs:string, $query as object(), $options as object()) as object()*", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Performs a query operation on the given collection and\n returns all matches.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">This function has the same semantics as <a href=\"#find-4\">find#4</a>,\n but is declared as being non deterministic and thus should only be used when\n result caching is not desired.</p>\n", "summary" : "<p>  Performs a query operation on the given collection and\n returns all matches.</p>", "annotation_str" : " %an:nondeterministic", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "nondeterministic", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "db", "type" : "xs:anyURI", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> a database connection identifier</div>" }, { "name" : "coll", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the name of the collection</div>" }, { "name" : "query", "type" : "object()", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the query operation to perform</div>" }, { "name" : "options", "type" : "object()", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the options for this operation (see find#5 for available options)</div>" } ], "returns" : { "type" : "object()*", "description" : "all matches returned by the given query operation" }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">mongo:MONGO004 invalid database identifier</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">mongo:MONGO005 if any mongodb error happens</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">mongo:MONGO006 if the given query could not be converted to BSON</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 5, "name" : "find-nondeterministic", "qname" : "mongo:find-nondeterministic", "signature" : "($db as xs:anyURI, $coll as xs:string, $query as object(), $projection as object(), $options as object()) as object()* external", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Performs a query operation on the given collection and\n returns all matches.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">This function has the same semantics as <a href=\"#find-5\">find#5</a>,\n but is declared as being non deterministic and thus should only be used when\n result caching is not desired.</p>\n", "summary" : "<p>  Performs a query operation on the given collection and\n returns all matches.</p>", "annotation_str" : " %an:nondeterministic", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "nondeterministic", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "db", "type" : "xs:anyURI", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> a database connection identifier</div>" }, { "name" : "coll", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the name of the collection</div>" }, { "name" : "query", "type" : "object()", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the query operation to perform</div>" }, { "name" : "projection", "type" : "object()", "occurrence" : null, "description" : "" }, { "name" : "options", "type" : "object()", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the options for this operation</div>" } ], "returns" : { "type" : "object()*", "description" : "all matches returned by the given query operation" }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">mongo:MONGO004 invalid database identifier</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">mongo:MONGO005 if any mongodb error happens</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">mongo:MONGO006 if the given query or projection could not be converted to BSON</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 2, "name" : "find", "qname" : "mongo:find", "signature" : "($db as xs:anyURI, $coll as xs:string) as object()*", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Returns all objects of the given collection.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">This function is declared as deterministic and should be used whenever result\n caching is acceptable.</p>\n", "summary" : "<p>  Returns all objects of the given collection.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "db", "type" : "xs:anyURI", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> a database connection identifier</div>" }, { "name" : "coll", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the name of the collection</div>" } ], "returns" : { "type" : "object()*", "description" : "all objects of the given collection" }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">mongo:MONGO004 invalid database identifier</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">mongo:MONGO005 if any mongodb error happens</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 3, "name" : "find", "qname" : "mongo:find", "signature" : "($db as xs:anyURI, $coll as xs:string, $query as object()) as object()*", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Performs a query operation on the given collection and\n returns all matches.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">This function is declared as deterministic and should be used whenever result\n caching is acceptable.</p>\n", "summary" : "<p>  Performs a query operation on the given collection and\n returns all matches.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "db", "type" : "xs:anyURI", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> a database connection identifier</div>" }, { "name" : "coll", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the name of the collection</div>" }, { "name" : "query", "type" : "object()", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the query operation to perform</div>" } ], "returns" : { "type" : "object()*", "description" : "all matches returned by the given query operation" }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">mongo:MONGO004 invalid database identifier</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">mongo:MONGO005 if any mongodb error happens</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">mongo:MONGO006 if the given query could not be converted to BSON</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 4, "name" : "find", "qname" : "mongo:find", "signature" : "($db as xs:anyURI, $coll as xs:string, $query as object(), $options as object()) as object()*", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Performs a query operation on the given collection and\n returns all matches.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">This function is declared as deterministic and should be used whenever result\n caching is acceptable.</p>\n", "summary" : "<p>  Performs a query operation on the given collection and\n returns all matches.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "db", "type" : "xs:anyURI", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> a database connection identifier</div>" }, { "name" : "coll", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the name of the collection</div>" }, { "name" : "query", "type" : "object()", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the query operation to perform</div>" }, { "name" : "options", "type" : "object()", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the options for this operation (see find#5 for available options)</div>" } ], "returns" : { "type" : "object()*", "description" : "all matches returned by the given query operation" }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">mongo:MONGO004 invalid database identifier</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">mongo:MONGO005 if any mongodb error happens</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">mongo:MONGO006 if the given query could not be converted to BSON</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 5, "name" : "find", "qname" : "mongo:find", "signature" : "($db as xs:anyURI, $coll as xs:string, $query as object(), $projection as object(), $options as object()) as object()* external", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Performs a query operation on the given collection and\n returns all matches.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">This function is declared as deterministic and should be used whenever result\n caching is acceptable.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Available options:\n <ul>\n   <li>to-return: the maximum number of objects to return (0 = unlimited)</li>\n   <li>to-skip: start with the n-th object</li>\n   <li>batch-size: the number of objects to return in one batch</li>\n   <li>slave-ok: allow this query to be run against a replica secondary</li>\n   <li>await-data: the server will block for some extra time before returning,\n   waiting for more data to return</li>\n   <li>partial-results: return partial results if some shards are down instead\n   of returning an error</li>\n </ul>\n </p>\n", "summary" : "<p>  Performs a query operation on the given collection and\n returns all matches.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "db", "type" : "xs:anyURI", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> a database connection identifier</div>" }, { "name" : "coll", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the name of the collection</div>" }, { "name" : "query", "type" : "object()", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the query operation to perform</div>" }, { "name" : "projection", "type" : "object()", "occurrence" : null, "description" : "" }, { "name" : "options", "type" : "object()", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the options for this operation</div>" } ], "returns" : { "type" : "object()*", "description" : "all matches returned by the given query operation" }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">mongo:MONGO004 invalid database identifier</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">mongo:MONGO005 if any mongodb error happens</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">mongo:MONGO006 if the given query or projection could not be converted to BSON</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 1, "name" : "is-connected", "qname" : "mongo:is-connected", "signature" : "($db as xs:anyURI) as xs:boolean external", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Checks if the given identifiers is valid and the corresponding\n connection is open.</p>\n", "summary" : "<p>  Checks if the given identifiers is valid and the corresponding\n connection is open.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "db", "type" : "xs:anyURI", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> a database connection identifier</div>" } ], "returns" : { "type" : "xs:boolean", "description" : "true if the given connection identifier is valid and the corresponding connection is open, false otherwise." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">mongo:MONGO004 invalid database identifier</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">mongo:MONGO005 if any mongodb error happens</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 3, "name" : "remove", "qname" : "mongo:remove", "signature" : "($db as xs:anyURI, $coll as xs:string, $remove as object()) as empty-sequence()", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Performs a remove operation on the given collection.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">The operation will be checked and an error is raised if\n one of them fails.</p>\n", "summary" : "<p>  Performs a remove operation on the given collection.</p>", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "db", "type" : "xs:anyURI", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> a database connection identifier</div>" }, { "name" : "coll", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the name of the collection</div>" }, { "name" : "remove", "type" : "object()", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the remove command to be performed</div>" } ], "returns" : { "type" : "empty-sequence()", "description" : "the function has side-effects and returns the empty sequence." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">mongo:MONGO004 invalid database identifier</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">mongo:MONGO005 if any mongodb error happens</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">mongo:MONGO006 if the given document could not be converted to BSON</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 4, "name" : "remove", "qname" : "mongo:remove", "signature" : "($db as xs:anyURI, $coll as xs:string, $remove as object(), $options as object()) as empty-sequence() external", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Performs a remove operation on the given collection.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">\n <ul>\n   <li>safe: If to true, the operation will wait for a response from\n    the database and an error is raised if the operation fails.\n    Otherwise, the operation will not wait for a response.</li>\n  <li>just-one: true if the operation should stop after a single match\n    has been found and deleted</li>\n </ul>\n </p>\n", "summary" : "<p>  Performs a remove operation on the given collection.</p>", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "db", "type" : "xs:anyURI", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> a database connection identifier</div>" }, { "name" : "coll", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the name of the collection</div>" }, { "name" : "remove", "type" : "object()", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the remove command to be performed</div>" }, { "name" : "options", "type" : "object()", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the options for this operation</div>" } ], "returns" : { "type" : "empty-sequence()", "description" : "the function has side-effects and returns the empty sequence." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">mongo:MONGO004 invalid database identifier</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">mongo:MONGO005 if any mongodb error happens</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">mongo:MONGO006 if the given document could not be converted to BSON</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 2, "name" : "run-cmd-deterministic", "qname" : "mongo:run-cmd-deterministic", "signature" : "($db as xs:anyURI, $cmd as object()) as object() external", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Executes a\n <a href=\"http://docs.mongodb.org/manual/reference/commands/\">database command</a>.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">This function is marked as deterministic and should be used whenever the\n specified command has no side-effects and result caching is desired.</p>\n", "summary" : "<p>  Executes a\n  database command .</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "db", "type" : "xs:anyURI", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> a database connection identifier</div>" }, { "name" : "cmd", "type" : "object()", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the database command to execute</div>" } ], "returns" : { "type" : "object()", "description" : "The result object. Typically has { ok : ..., errmsg : ... } fields set." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">mongo:MONGO004 invalid database identifier</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">mongo:MONGO006 if any mongodb error happens</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 2, "name" : "run-cmd-nondeterministic", "qname" : "mongo:run-cmd-nondeterministic", "signature" : "($db as xs:anyURI, $cmd as object()) as object() external", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Executes a\n <a href=\"http://docs.mongodb.org/manual/reference/commands/\">database command</a>.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">This function is marked as non-deterministic and should be used whenever the\n specified command has no side-effects and result caching is undesired.</p>\n", "summary" : "<p>  Executes a\n  database command .</p>", "annotation_str" : " %an:nondeterministic", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "nondeterministic", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "db", "type" : "xs:anyURI", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> a database connection identifier</div>" }, { "name" : "cmd", "type" : "object()", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the database command to execute</div>" } ], "returns" : { "type" : "object()", "description" : "The result object. Typically has { ok : ..., errmsg : ... } fields set." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">mongo:MONGO004 invalid database identifier</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">mongo:MONGO006 if any mongodb error happens</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 2, "name" : "run-cmd", "qname" : "mongo:run-cmd", "signature" : "($db as xs:anyURI, $cmd as object()) as object() external", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Executes a\n <a href=\"http://docs.mongodb.org/manual/reference/commands/\">database command</a>.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">This function is marked as sequential and should be used whenever the\n specified command has side-effects.</p>\n", "summary" : "<p>  Executes a\n  database command .</p>", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "db", "type" : "xs:anyURI", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> a database connection identifier</div>" }, { "name" : "cmd", "type" : "object()", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the database command to execute</div>" } ], "returns" : { "type" : "object()", "description" : "The result object. Typically has { ok : ..., errmsg : ... } fields set." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">mongo:MONGO004 invalid database identifier</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">mongo:MONGO006 if any mongodb error happens</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 3, "name" : "save", "qname" : "mongo:save", "signature" : "($db as xs:anyURI, $coll as xs:string, $doc as object()) as empty-sequence()", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Save a sequence of documents in the given collection.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">If a document to be save already has an \"_id\" field, then an\n upsert operation is performed an any existing document with that\n id will be overwritten. Otherwise, an insert operation is performed\n and the \"_id\" generated for each document will be returned.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Each safe operation will be checked and an error is raised if\n one of them fails.</p>\n", "summary" : "<p>  Save a sequence of documents in the given collection.</p>", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "db", "type" : "xs:anyURI", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> a database connection identifier</div>" }, { "name" : "coll", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the name of the collection</div>" }, { "name" : "doc", "type" : "object()", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the document to be saved or upserted</div>" } ], "returns" : { "type" : "empty-sequence()", "description" : "the documents that have been inserted with \"_id\" fields." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">mongo:MONGO004 invalid database identifier</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">mongo:MONGO005 if any mongodb error happens</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">mongo:MONGO006 if the given document could not be converted to BSON</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 4, "name" : "save", "qname" : "mongo:save", "signature" : "($db as xs:anyURI, $coll as xs:string, $doc as object(), $options as object()) as m-schema:oid? external", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Save a sequence of documents in the given collection.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">If a document to be saved already has an \"_id\" field, then an\n upsert operation is performed and any existing document with that\n id will be overwritten. Otherwise, an insert operation is performed.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">If the manipulate option is set to true, an \"_id\" field will be\n added to the document. The new id will be returned. Otherwise,\n the \"_id\" field will be added by the server.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">If the safe options is set to true, each operation will wait for a\n response from the database and an error is raised if the operation\n fails. Otherwise, the operation will not wait for a response.</p>\n", "summary" : "<p>  Save a sequence of documents in the given collection.</p>", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "db", "type" : "xs:anyURI", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> a database connection identifier</div>" }, { "name" : "coll", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the name of the collection</div>" }, { "name" : "doc", "type" : "object()", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the document to be saved or upserted</div>" }, { "name" : "options", "type" : "object()", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the options for this operation</div>" } ], "returns" : { "type" : "m-schema:oid?", "description" : "a generated OID if the manipulate option was set to true, the empty sequence otherwise." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">mongo:MONGO004 invalid database identifier</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">mongo:MONGO005 if any mongodb error happens</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">mongo:MONGO006 if the given document could not be converted to BSON</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 4, "name" : "update", "qname" : "mongo:update", "signature" : "($db as xs:anyURI, $coll as xs:string, $query as object(), $update as object()) as empty-sequence()", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Performs an update command on the given collection.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">The operation will be checked and an error is raised if\n one of them fails. Also, this function only modifies one\n document matching the query and does not do any upserts.</p>\n", "summary" : "<p>  Performs an update command on the given collection.</p>", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "db", "type" : "xs:anyURI", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> a database connection identifier</div>" }, { "name" : "coll", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the name of the collection</div>" }, { "name" : "query", "type" : "object()", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the query to select the objects that are updated</div>" }, { "name" : "update", "type" : "object()", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the update specification to be performed</div>" } ], "returns" : { "type" : "empty-sequence()", "description" : "the function has side-effects and returns the empty sequence." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">mongo:MONGO004 invalid database identifier</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">mongo:MONGO005 if any mongodb error happens</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">mongo:MONGO006 if the given objects could not be converted to BSON</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 5, "name" : "update", "qname" : "mongo:update", "signature" : "($db as xs:anyURI, $coll as xs:string, $query as object(), $update as object(), $options as object()) as empty-sequence() external", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Performs an update operation on the given collection.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">\n <ul>\n   <li>safe: If to true, the operation will wait for a response from\n    the database and an error is raised if the operation fails.\n    Otherwise, the operation will not wait for a response.</li>\n  <li>multi: indicates if all documents matching criteria should be updated\n  rather than just one.</li>\n  <li>upsert: if this should be an \"upsert\" operation; that is,\n  if the record(s) do not exist, insert one. Upsert only inserts a single document.</li>\n </ul>\n </p>\n", "summary" : "<p>  Performs an update operation on the given collection.</p>", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "db", "type" : "xs:anyURI", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> a database connection identifier</div>" }, { "name" : "coll", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the name of the collection</div>" }, { "name" : "query", "type" : "object()", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the query to select the objects that are updated</div>" }, { "name" : "update", "type" : "object()", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the update specification to be performed</div>" }, { "name" : "options", "type" : "object()", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the options for this operation</div>" } ], "returns" : { "type" : "empty-sequence()", "description" : "the function has side-effects and returns the empty sequence." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">mongo:MONGO004 invalid database identifier</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">mongo:MONGO005 if any mongodb error happens</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">mongo:MONGO006 if the given objects could not be converted to BSON</xqdoc:error>" ] } ], "variables" : [  ] }, "http://jsoniq.org/function-library" : { "ns" : "http://jsoniq.org/function-library", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">This module provides extensions to the JSONiq core function library.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">The module is always imported so you do not need to import it explicitly.\n Also, you do not need to fully qualify a function to invoke it.</p>\n", "sees" : [ "<xqdoc:see xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">http://jsoniq.org/</xqdoc:see>" ], "authors" : [ "<xqdoc:author xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Ghislain Fourny</xqdoc:author>" ], "version" : null, "encoding" : "utf-8", "namespaces" : [ { "uri" : "http://jsoniq.org/functions", "prefix" : "jn" }, { "uri" : "http://jsoniq.org/function-library", "prefix" : "libjn" }, { "uri" : "http://zorba.io/options/versioning", "prefix" : "ver" } ], "functions" : [ { "isDocumented" : true, "arity" : 1, "name" : "accumulate", "qname" : "libjn:accumulate", "signature" : "($items as item()*) as object()", "description" : " This function dynamically builds an object, like the {||} syntax, except that\n it does not throw an error upon pair collision. Instead, it accumulates them\n into an array, if more than one.\n", "summary" : "<p> This function dynamically builds an object, like the {||} syntax, except that\n it does not throw an error upon pair collision.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "items", "type" : "item()", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> A sequence of items, the objects of which are going to be accumulated into a single object.</div>" } ], "returns" : { "type" : "object()", "description" : "The accumulated object." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "descendant-arrays", "qname" : "libjn:descendant-arrays", "signature" : "($items as item()*) as array()*", "description" : " This function returns all arrays contained at any depth within a sequence of items.\n", "summary" : "<p> This function returns all arrays contained at any depth within a sequence of items.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "items", "type" : "item()", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> A sequence of items.</div>" } ], "returns" : { "type" : "array()*", "description" : "The descendant arrays of the input sequence." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "descendant-objects", "qname" : "libjn:descendant-objects", "signature" : "($items as item()*) as object()*", "description" : " This function returns all objects contained at any depth within a sequence of items.\n", "summary" : "<p> This function returns all objects contained at any depth within a sequence of items.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "items", "type" : "item()", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> A sequence of items.</div>" } ], "returns" : { "type" : "object()*", "description" : "The descendant objects of the input sequence." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "descendant-pairs-priv", "qname" : "libjn:descendant-pairs-priv", "signature" : "($i as item()) as object()*", "description" : " Helper function for libjn:descendant-pairs()\n", "summary" : "<p> Helper function for libjn:descendant-pairs()\n</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "i", "type" : "item()", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> An item</div>" } ], "returns" : { "type" : "object()*", "description" : "The descendant pairs of the item" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "descendant-pairs", "qname" : "libjn:descendant-pairs", "signature" : "($items as item()*) as object()*", "description" : " This function returns all pairs contained at any depth within an sequence of items.\n", "summary" : "<p> This function returns all pairs contained at any depth within an sequence of items.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "items", "type" : "item()", "occurrence" : "*", "description" : "" } ], "returns" : { "type" : "object()*", "description" : "All direct and indirect descendant pairs." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "intersect", "qname" : "libjn:intersect", "signature" : "($items as item()*) as object()", "description" : " This function returns the intersection of the objects contained in the\n given sequence of items, aggregating values corresponding to the same key\n into an array.\n", "summary" : "<p> This function returns the intersection of the objects contained in the\n given sequence of items, aggregating values corresponding to the same key\n into an array.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "items", "type" : "item()", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> A sequence of items.</div>" } ], "returns" : { "type" : "object()", "description" : "The insersection of the objects contained in $items." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "values", "qname" : "libjn:values", "signature" : "($items as item()*) as item()*", "description" : " This functions returns all values of all objects contained in a sequence of items.\n", "summary" : "<p> This functions returns all values of all objects contained in a sequence of items.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "items", "type" : "item()", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> A sequence of items.</div>" } ], "returns" : { "type" : "item()*", "description" : "The values inside the objects of the sequence." }, "errors" : [  ] } ], "variables" : [  ] }, "http://zorba.io/modules/node-position" : { "ns" : "http://zorba.io/modules/node-position", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">This module provides a function (np:node-position) that, given a node,\n returns positional information about the node in the form of an xs:anyURI\n item. The module also defines functions that use such positional information\n to determine: (1) positional relationships between two nodes (e.g. if one\n is the ancestor of another) and (2) positional properties of a single node\n (e.g. its level in the tree).</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Within this module, the term \"node position\" will be used to refer to an\n xs:anyURI item that is returned by the np:node-position function.</p>\n", "sees" : [  ], "authors" : [ "<xqdoc:author xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Federico Cavalieri, Markos Zaharioudakis</xqdoc:author>" ], "version" : null, "encoding" : "utf-8", "namespaces" : [ { "uri" : "http://zorba.io/modules/node-position", "prefix" : "np" }, { "uri" : "http://zorba.io/options/versioning", "prefix" : "ver" } ], "functions" : [ { "isDocumented" : true, "arity" : 2, "name" : "ancestor-of", "qname" : "np:ancestor-of", "signature" : "($pos1 as xs:anyURI, $pos2 as xs:anyURI) as xs:boolean external", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Determines whether the node position given as second argument is\n an ancestor of the node position given as first argument.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">If the two positions were obtained within the same snapshot S, then the\n result of the function applies to the corresponding nodes as well, that\n is, within snapshot S, the second node is an ancestor of the first.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Otherwise, the result of the function does not imply anything about the\n positional relationship of the two nodes.</p>\n", "summary" : "<p>  Determines whether the node position given as second argument is\n an ancestor of the node position given as first argument.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "pos1", "type" : "xs:anyURI", "occurrence" : null, "description" : "" }, { "name" : "pos2", "type" : "xs:anyURI", "occurrence" : null, "description" : "" } ], "returns" : { "type" : "xs:boolean", "description" : "true if the node position $n-pos2 is an ancestor of the node position $n-pos1; false otherwise." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">zerr:ZAPI0028 if one of the given URI is not a valid node position computed by the <tt>np:node-position</tt> function.</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 2, "name" : "attribute-of", "qname" : "np:attribute-of", "signature" : "($n-pos1 as xs:anyURI, $n-pos2 as xs:anyURI) as xs:boolean external", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Determines whether the node position given as second argument is\n an attribute of the node position given as first argument.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">If the two positions were obtained within the same snapshot S, then the\n result of the function applies to the corresponding nodes as well, that\n is, within snapshot S, the second node is an attribute of the first.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Otherwise, the result of the function does not imply anything about the\n positional relationship of the two nodes.</p>\n", "summary" : "<p>  Determines whether the node position given as second argument is\n an attribute of the node position given as first argument.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "n-pos1", "type" : "xs:anyURI", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the potential parent node position</div>" }, { "name" : "n-pos2", "type" : "xs:anyURI", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the potential attribute node position</div>" } ], "returns" : { "type" : "xs:boolean", "description" : "true if the node position $n-pos2 is an attribute of the node position $n-pos1; false otherwise." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">zerr:ZAPI0028 if one of the given URI is not a valid node position computed by the <tt>np:node-position</tt> function.</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 2, "name" : "child-of", "qname" : "np:child-of", "signature" : "($n-pos1 as xs:anyURI, $n-pos2 as xs:anyURI) as xs:boolean external", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Determines whether the node position given as second argument is\n a child of the node position given as first argument.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">If the two positions were obtained within the same snapshot S, then the\n result of the function applies to the corresponding nodes as well, that\n is, within snapshot S, the second node is a child of the first.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Otherwise, the result of the function does not imply anything about the\n positional relationship of the two nodes.</p>\n", "summary" : "<p>  Determines whether the node position given as second argument is\n a child of the node position given as first argument.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "n-pos1", "type" : "xs:anyURI", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the potential parent node position</div>" }, { "name" : "n-pos2", "type" : "xs:anyURI", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the potential child node position</div>" } ], "returns" : { "type" : "xs:boolean", "description" : "true if the node position $n-pos2 is a child of the node position $n-pos1; false otherwise." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">zerr:ZAPI0028 if one of the given URI is not a valid node position computed by the <tt>np:node-position</tt> function.</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 2, "name" : "descendant-of", "qname" : "np:descendant-of", "signature" : "($n-pos1 as xs:anyURI, $n-pos2 as xs:anyURI) as xs:boolean external", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Determines whether the node position given as second argument is\n a descendant of the node position given as first argument.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">If the two positions were obtained within the same snapshot S, then the\n result of the function applies to the corresponding nodes as well, that\n is, within snapshot S, the second node is a descendant of the first.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Otherwise, the result of the function does not imply anything about the\n positional relationship of the two nodes.</p>\n", "summary" : "<p>  Determines whether the node position given as second argument is\n a descendant of the node position given as first argument.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "n-pos1", "type" : "xs:anyURI", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the potential ancestor node position</div>" }, { "name" : "n-pos2", "type" : "xs:anyURI", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the potential descendant node position</div>" } ], "returns" : { "type" : "xs:boolean", "description" : "true if the node position $n-pos2 is a descendant of the node position $n-pos1; false otherwise." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">zerr:ZAPI0028 if one of the given URI is not a valid node position computed by the <tt>np:node-position</tt> function.</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 2, "name" : "following-in-document-order-of", "qname" : "np:following-in-document-order-of", "signature" : "($n-pos1 as xs:anyURI, $n-pos2 as xs:anyURI) as xs:boolean external", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Determines whether the node position given as second argument is\n following in document order the node position given as first argument.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">If the two positions were obtained within the same snapshot S, then the\n result of the function applies to the corresponding nodes as well, that\n is, within snapshot S, the second node is following in document order the\n first.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Otherwise, the result of the function does not imply anything about\n the positional relationship of the two nodes.</p>\n", "summary" : "<p>  Determines whether the node position given as second argument is\n following in document order the node position given as first argument.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "n-pos1", "type" : "xs:anyURI", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the potential preceding node position</div>" }, { "name" : "n-pos2", "type" : "xs:anyURI", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the potential following node position</div>" } ], "returns" : { "type" : "xs:boolean", "description" : "true if the node position $n-pos2 is following in document order the node position $n-pos1; false otherwise." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">zerr:ZAPI0028 if one of the given URI is not a valid node position computed by the <tt>np:node-position</tt> function.</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 2, "name" : "following-of", "qname" : "np:following-of", "signature" : "($n-pos1 as xs:anyURI, $n-pos2 as xs:anyURI) as xs:boolean external", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Determines whether the node position given as second argument is\n following the node position given as first argument.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">If the two positions were obtained within the same snapshot S, then the\n result of the function applies to the corresponding nodes as well, that\n is, within snapshot S, the second node is following the first.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Otherwise, the result of the function does not imply anything about the\n positional relationship of the two nodes.</p>\n", "summary" : "<p>  Determines whether the node position given as second argument is\n following the node position given as first argument.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "n-pos1", "type" : "xs:anyURI", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the potential preceding node position</div>" }, { "name" : "n-pos2", "type" : "xs:anyURI", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the potential following node position</div>" } ], "returns" : { "type" : "xs:boolean", "description" : "true if node positions $n-pos1 and $n-pos2 belong to the same XML tree and $n-pos2 is following the node position $n-pos1; false otherwise." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">zerr:ZAPI0028 if one of the given URI is not a valid node position computed by the <tt>np:node-position</tt> function.</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 2, "name" : "following-sibling-of", "qname" : "np:following-sibling-of", "signature" : "($n-pos1 as xs:anyURI, $n-pos2 as xs:anyURI) as xs:boolean external", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Determines whether the node position given as second argument is\n a following-sibling of the node position given as first argument.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">If the two positions were obtained within the same snapshot S, then the\n result of the function applies to the corresponding nodes as well, that\n is, within snapshot S, the second node is a following-sibling of the first.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Otherwise, the result of the function does not imply anything about the\n positional relationship of the two nodes.</p>\n", "summary" : "<p>  Determines whether the node position given as second argument is\n a following-sibling of the node position given as first argument.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "n-pos1", "type" : "xs:anyURI", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the potential preceding-sibling node position</div>" }, { "name" : "n-pos2", "type" : "xs:anyURI", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the potential following-sibling node position</div>" } ], "returns" : { "type" : "xs:boolean", "description" : "true if the node position $n-pos2 is a following-sibling of the node position $n-pos1; false otherwise." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">zerr:ZAPI0028 if one of the given URI is not a valid node position computed by the <tt>np:node-position</tt> function.</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 1, "name" : "in-collection", "qname" : "np:in-collection", "signature" : "($n-pos as xs:anyURI) as xs:boolean external", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Determines whether a node position belongs to a collection.</p>\n", "summary" : "<p>  Determines whether a node position belongs to a collection.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "n-pos", "type" : "xs:anyURI", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the node position</div>" } ], "returns" : { "type" : "xs:boolean", "description" : "true if the node position $n-pos belongs to a collection; false otherwise." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">zerr:ZAPI0028 if the given URI is not a valid node position computed by the <tt>np:node-position</tt> function.</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 2, "name" : "in-same-collection-of", "qname" : "np:in-same-collection-of", "signature" : "($n-pos1 as xs:anyURI, $n-pos2 as xs:anyURI) as xs:boolean external", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Determines whether two node positions belong to the same collection.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">If the two positions were obtained within the same snapshot S, then the\n result of the function applies to the corresponding nodes as well, that\n is, within snapshot S, the two nodes belong to the same collection.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Otherwise, the result of the function does not imply anything about\n the positional relationship of the two nodes.</p>\n", "summary" : "<p>  Determines whether two node positions belong to the same collection.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "n-pos1", "type" : "xs:anyURI", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> a node position</div>" }, { "name" : "n-pos2", "type" : "xs:anyURI", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> a node position</div>" } ], "returns" : { "type" : "xs:boolean", "description" : "true if the two nodes whose node positions are $n-pos1 and $n-pos2 belong to the same collection." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">zerr:ZAPI0028 if one of the given URI is not a valid node position computed by the <tt>np:node-position</tt> function.</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 2, "name" : "in-same-tree-of", "qname" : "np:in-same-tree-of", "signature" : "($n-pos1 as xs:anyURI, $n-pos2 as xs:anyURI) as xs:boolean external", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Determines whether two node positions belong to the same tree.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">If the two positions were obtained within the same snapshot S, then the\n result of the function applies to the corresponding nodes as well, that\n is, within snapshot S, the two nodes belong to the same tree.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Otherwise, the result of the function does not imply anything about\n the positional relationship of the two nodes.</p>\n", "summary" : "<p>  Determines whether two node positions belong to the same tree.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "n-pos1", "type" : "xs:anyURI", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> a node position</div>" }, { "name" : "n-pos2", "type" : "xs:anyURI", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> a node position</div>" } ], "returns" : { "type" : "xs:boolean", "description" : "true if the two nodes whose node positions are $n-pos1 and $n-pos2 belong to the same tree." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">zerr:ZAPI0028 if one of the given URI is not a valid node position computed by the <tt>np:node-position</tt> function.</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 2, "name" : "in-subtree-of", "qname" : "np:in-subtree-of", "signature" : "($n-pos1 as xs:anyURI, $n-pos2 as xs:anyURI) as xs:boolean external", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Determines whether the node position given as second argument belongs\n to the subtree rooted at the node position given as first argument.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">If the two positions were obtained within the same snapshot S, then the\n result of the function applies to the corresponding nodes as well, that\n is, within snapshot S, the second node belongs to the subtree rooted at the\n first. Otherwise, the result of the function does not imply anything about\n the positional relationship of the two nodes.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">This function differs from np:descendant-of in the way it treats attribute\n nodes. np:descendant-of follows the XQuery/XPath specification for the\n descendant axis, and as a result, it does not consider attributes as\n descendants of any nodes; it will always return false if $n-pos2 was\n obtained from an attribute node.In contrast, np:in-subtree-of will return\n true if $n-pos2 was obtained from an attribute node that appeared in the\n subtree of the node that $n-pos1 was obtained from.</p>\n", "summary" : "<p>  Determines whether the node position given as second argument belongs\n to the subtree rooted at the node position given as first argument.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "n-pos1", "type" : "xs:anyURI", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the potential subtree root node position</div>" }, { "name" : "n-pos2", "type" : "xs:anyURI", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the potential node in the subtree node position</div>" } ], "returns" : { "type" : "xs:boolean", "description" : "true if the node position $n-pos2 belongs to the subtree rooted at the node position $n-pos1; false otherwise." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">zerr:ZAPI0028 if one of the given URI is not a valid node position computed by the <tt>np:node-position</tt> function.</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 1, "name" : "is-attribute", "qname" : "np:is-attribute", "signature" : "($n-pos1 as xs:anyURI) as xs:boolean external", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Determines whether a node position corresponds to an attribute node.</p>\n", "summary" : "<p>  Determines whether a node position corresponds to an attribute node.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "n-pos1", "type" : "xs:anyURI", "occurrence" : null, "description" : "" } ], "returns" : { "type" : "xs:boolean", "description" : "true if the node position $n-pos corresponds to an attribute; false otherwise." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">zerr:ZAPI0028 if the given URI is not a valid node position computed by the <tt>np:node-position</tt> function.</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 1, "name" : "is-comment", "qname" : "np:is-comment", "signature" : "($n-pos1 as xs:anyURI) as xs:boolean external", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Determines whether a node position corresponds to a comment node.</p>\n", "summary" : "<p>  Determines whether a node position corresponds to a comment node.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "n-pos1", "type" : "xs:anyURI", "occurrence" : null, "description" : "" } ], "returns" : { "type" : "xs:boolean", "description" : "true if the node position $n-pos corresponds to an comment; false otherwise." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">zerr:ZAPI0028 if the given URI is not a valid node position computed by the <tt>np:node-position</tt> function.</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 1, "name" : "is-document", "qname" : "np:is-document", "signature" : "($n-pos1 as xs:anyURI) as xs:boolean external", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Determines whether a node position corresponds to a document node.</p>\n", "summary" : "<p>  Determines whether a node position corresponds to a document node.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "n-pos1", "type" : "xs:anyURI", "occurrence" : null, "description" : "" } ], "returns" : { "type" : "xs:boolean", "description" : "true if the node position $n-pos corresponds to a document; false otherwise." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">zerr:ZAPI0028 if the given URI is not a valid node position computed by the <tt>np:node-position</tt> function.</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 1, "name" : "is-element", "qname" : "np:is-element", "signature" : "($n-pos1 as xs:anyURI) as xs:boolean external", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Determines whether a node position corresponds to an element node.</p>\n", "summary" : "<p>  Determines whether a node position corresponds to an element node.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "n-pos1", "type" : "xs:anyURI", "occurrence" : null, "description" : "" } ], "returns" : { "type" : "xs:boolean", "description" : "true if the node position $n-pos corresponds to an element; false otherwise." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">zerr:ZAPI0028 if the given URI is not a valid node position computed by the <tt>np:node-position</tt> function.</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 1, "name" : "is-processing-instruction", "qname" : "np:is-processing-instruction", "signature" : "($n-pos1 as xs:anyURI) as xs:boolean external", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Determines whether a node position corresponds to an processing-instruction\n node.</p>\n", "summary" : "<p>  Determines whether a node position corresponds to an processing-instruction\n node.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "n-pos1", "type" : "xs:anyURI", "occurrence" : null, "description" : "" } ], "returns" : { "type" : "xs:boolean", "description" : "true if the node position $n-pos corresponds to a processing instruction; false otherwise." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">zerr:ZAPI0028 if the given URI is not a valid node position computed by the <tt>np:node-position</tt> function.</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 1, "name" : "is-text", "qname" : "np:is-text", "signature" : "($n-pos1 as xs:anyURI) as xs:boolean external", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Determines whether a node position corresponds to a text node.</p>\n", "summary" : "<p>  Determines whether a node position corresponds to a text node.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "n-pos1", "type" : "xs:anyURI", "occurrence" : null, "description" : "" } ], "returns" : { "type" : "xs:boolean", "description" : "true if the node position $n-pos corresponds to a text; false otherwise." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">zerr:ZAPI0028 if the given URI is not a valid node position computed by the <tt>np:node-position</tt> function.</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 1, "name" : "level", "qname" : "np:level", "signature" : "($n-pos as xs:anyURI) as xs:integer external", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Computes the level of a node position in its tree.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Note: The root node of a tree is at level one.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">The result of the function applies to the corresponding node as well,\n that is, within the snapshot in which the position was computed, the node\n level is the returned one.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">The result of the function does not imply anything about the\n node level in other snapshots.</p>\n", "summary" : "<p>  Computes the level of a node position in its tree.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "n-pos", "type" : "xs:anyURI", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the node position of the node whose level should be determined.</div>" } ], "returns" : { "type" : "xs:integer", "description" : "the level in the tree of the node position $n-pos as xs:integer." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">zerr:ZAPI0028 if the given URI is not a valid node position computed by the <tt>np:node-position</tt> function.</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 1, "name" : "node-position", "qname" : "np:node-position", "signature" : "($arg as node()) as xs:anyURI external", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Return a URI item containing positional information for a given node.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Within a snapshot, each has a different positional URI. However,\n different nodes in different snapshots might have the same URI.</p>\n", "summary" : "<p>  Return a URI item containing positional information for a given node.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "arg", "type" : "node()", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the node for which the positional information URI should be computed</div>" } ], "returns" : { "type" : "xs:anyURI", "description" : "the opaque positional information URI of the node." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 2, "name" : "parent-of", "qname" : "np:parent-of", "signature" : "($n-pos1 as xs:anyURI, $n-pos2 as xs:anyURI) as xs:boolean external", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Determines whether the node position given as second argument is\n the parent of the node position given as first argument.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">If the two positions were obtained within the same snapshot S, then the\n result of the function applies to the corresponding nodes as well, that\n is, within snapshot S, the second node is the parent of the first.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Otherwise, the result of the function does not imply anything about the\n positional relationship of the two nodes.</p>\n", "summary" : "<p>  Determines whether the node position given as second argument is\n the parent of the node position given as first argument.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "n-pos1", "type" : "xs:anyURI", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the potential child node position</div>" }, { "name" : "n-pos2", "type" : "xs:anyURI", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the potential parent node position</div>" } ], "returns" : { "type" : "xs:boolean", "description" : "true if the node position $n-pos2 is the parent of the node position $n-pos1; false otherwise." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">zerr:ZAPI0028 if one of the given URI is not a valid node position computed by the <tt>np:node-position</tt> function.</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 2, "name" : "preceding-in-document-order-of", "qname" : "np:preceding-in-document-order-of", "signature" : "($n-pos1 as xs:anyURI, $n-pos2 as xs:anyURI) as xs:boolean external", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Determines whether the node position given as second argument is\n preceding in document order the node position given as first argument.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">If the two positions were obtained within the same snapshot S, then the\n result of the function applies to the corresponding nodes as well, that\n is, within snapshot S, the second node is preceding in document order the\n first.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Otherwise, the result of the function does not imply anything about\n the positional relationship of the two nodes.</p>\n", "summary" : "<p>  Determines whether the node position given as second argument is\n preceding in document order the node position given as first argument.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "n-pos1", "type" : "xs:anyURI", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the potential following node position</div>" }, { "name" : "n-pos2", "type" : "xs:anyURI", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the potential preceding node position</div>" } ], "returns" : { "type" : "xs:boolean", "description" : "true if the node position $n-pos2 is preceding in document order the node position $n-pos1; false otherwise." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">zerr:ZAPI0028 if one of the given URI is not a valid node position computed by the <tt>np:node-position</tt> function.</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 2, "name" : "preceding-of", "qname" : "np:preceding-of", "signature" : "($n-pos1 as xs:anyURI, $n-pos2 as xs:anyURI) as xs:boolean external", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Determines whether the node position given as second argument is\n preceding the node position given as first argument.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">If the two positions were obtained within the same snapshot S, then the\n result of the function applies to the corresponding nodes as well, that\n is, within snapshot S, the second node is preceding the first.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Otherwise, the result of the function does not imply anything about the\n positional relationship of the two nodes.</p>\n", "summary" : "<p>  Determines whether the node position given as second argument is\n preceding the node position given as first argument.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "n-pos1", "type" : "xs:anyURI", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the potential following node position</div>" }, { "name" : "n-pos2", "type" : "xs:anyURI", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the potential preceding node position</div>" } ], "returns" : { "type" : "xs:boolean", "description" : "true if node positions $n-pos1 and $n-pos2 belong to the same XML tree and $n-pos2 is preceding the node position $n-pos1; false otherwise." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">zerr:ZAPI0028 if one of the given URI is not a valid node position computed by the <tt>np:node-position</tt> function.</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 2, "name" : "preceding-sibling-of", "qname" : "np:preceding-sibling-of", "signature" : "($n-pos1 as xs:anyURI, $n-pos2 as xs:anyURI) as xs:boolean external", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Determines whether the node position given as second argument is\n a preceding-sibling of the node position given as first argument.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">If the two positions were obtained within the same snapshot S, then the\n result of the function applies to the corresponding nodes as well, that\n is, within snapshot S, the second node is a preceding-sibling of the first.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Otherwise, the result of the function does not imply anything about the\n positional relationship of the two nodes.</p>\n", "summary" : "<p>  Determines whether the node position given as second argument is\n a preceding-sibling of the node position given as first argument.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "n-pos1", "type" : "xs:anyURI", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the potential following-sibling node position</div>" }, { "name" : "n-pos2", "type" : "xs:anyURI", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the potential preceding-sibling node position</div>" } ], "returns" : { "type" : "xs:boolean", "description" : "true if the node position $n-pos2 is a preceding-sibling of the node position $n-pos1; false otherwise." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">zerr:ZAPI0028 if one of the given URI is not a valid node position computed by the <tt>np:node-position</tt> function.</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 2, "name" : "sibling-of", "qname" : "np:sibling-of", "signature" : "($n-pos1 as xs:anyURI, $n-pos2 as xs:anyURI) as xs:boolean external", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Determines whether two node positions are siblings.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">If the two positions were obtained within the same snapshot S, then the\n result of the function applies to the corresponding nodes as well, that\n is, within snapshot S, the second node is a sibling of the first.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Otherwise, the result of the function does not imply anything about the\n positional relationship of the two nodes.</p>\n", "summary" : "<p>  Determines whether two node positions are siblings.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "n-pos1", "type" : "xs:anyURI", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> a node position</div>" }, { "name" : "n-pos2", "type" : "xs:anyURI", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> a node position</div>" } ], "returns" : { "type" : "xs:boolean", "description" : "true if the two node positions $n-pos1 and $n-pos2 are siblings; false otherwise." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">zerr:ZAPI0028 if one of the given URI is not a valid node position computed by the <tt>np:node-position</tt> function.</xqdoc:error>" ] } ], "variables" : [  ] }, "http://www.zorba-xquery.com/modules/cryptography/hash" : { "ns" : "http://www.zorba-xquery.com/modules/cryptography/hash", "description" : " This module provides functions that perform different hash operations.\n", "sees" : [  ], "authors" : [ "<xqdoc:author xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Gabriel Petrovay, Markus Pilman</xqdoc:author>" ], "version" : null, "encoding" : "utf-8", "namespaces" : [ { "uri" : "http://www.zorba-xquery.com/modules/cryptography/hash", "prefix" : "hash" }, { "uri" : "http://zorba.io/options/versioning", "prefix" : "ver" } ], "functions" : [ { "isDocumented" : true, "arity" : 2, "name" : "hash-impl", "qname" : "hash:hash-impl", "signature" : "($value as xs:string, $alg as xs:string) as xs:string external", "description" : " This function computes a hash value of the string provided as parameter.\n The function expects the hash algorithm to be used as parameter.\n", "summary" : "<p> This function computes a hash value of the string provided as parameter.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "value", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The string to be hashed.</div>" }, { "name" : "alg", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The algorithm to use for this hashing operation. Currently only \"md5\" and \"sha1\" algorithms are available. If no valid algorithm name is given, md5 will be used.</div>" } ], "returns" : { "type" : "xs:string", "description" : "The hash of the provided string. In case SHA1 is used, the resulting hash value is base64 encoded." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "md5", "qname" : "hash:md5", "signature" : "($value as xs:string) as xs:string", "description" : " Computes the MD5 hash of the string provided as parameter.\n", "summary" : "<p> Computes the MD5 hash of the string provided as parameter.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "value", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The string to hash.</div>" } ], "returns" : { "type" : "xs:string", "description" : "The MD5 hash of the provided string." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "sha1", "qname" : "hash:sha1", "signature" : "($value as xs:string) as xs:string", "description" : " Computes the SHA1 hash of the string provided as parameter.\n", "summary" : "<p> Computes the SHA1 hash of the string provided as parameter.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "value", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The string to hash.</div>" } ], "returns" : { "type" : "xs:string", "description" : "The base64 encoded SHA1 hash of the provided string." }, "errors" : [  ] } ], "variables" : [  ] }, "http://zorba.io/modules/excel/lookup" : { "ns" : "http://zorba.io/modules/excel/lookup", "description" : " This module implements some Excel 2003 lookup functions.\n", "sees" : [ "<xqdoc:see xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"><a href=\"http://office.microsoft.com/en-us/excel/CH062528281033.aspx\" target=\"_blank\">Excel 2003 Documentation: Lookup Functions</a></xqdoc:see>" ], "authors" : [ "<xqdoc:author xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Daniel Turcanu</xqdoc:author>" ], "version" : null, "encoding" : "utf-8", "namespaces" : [ { "uri" : "http://zorba.io/modules/excel/lookup", "prefix" : "excel" }, { "uri" : "http://zorba.io/modules/excel/errors", "prefix" : "excel-err" }, { "uri" : "http://zorba.io/modules/excel/math", "prefix" : "excel-math" }, { "uri" : "http://zorba.io/options/versioning", "prefix" : "ver" } ], "functions" : [ { "isDocumented" : true, "arity" : 2, "name" : "choose", "qname" : "excel:choose", "signature" : "($index_num as xs:integer, $values as xs:anyAtomicType*) as xs:anyAtomicType", "description" : " Uses index_num to return a value from the sequence of value arguments.\n", "summary" : "<p> Uses index_num to return a value from the sequence of value arguments.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "index_num", "type" : "xs:integer", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The position in the sequence, 1 based.</div>" }, { "name" : "values", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The sequence of values.</div>" } ], "returns" : { "type" : "xs:anyAtomicType", "description" : "The value at the index position." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">excel-err:Value if index is smaller than 1 or bigger than the size of sequence.</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 30, "name" : "choose", "qname" : "excel:choose", "signature" : "($index_num as xs:integer, $value_sequence1 as xs:anyAtomicType*, $value_sequence2 as xs:anyAtomicType*, $value_sequence3 as xs:anyAtomicType*, $value_sequence4 as xs:anyAtomicType*, $value_sequence5 as xs:anyAtomicType*, $value_sequence6 as xs:anyAtomicType*, $value_sequence7 as xs:anyAtomicType*, $value_sequence8 as xs:anyAtomicType*, $value_sequence9 as xs:anyAtomicType*, $value_sequence10 as xs:anyAtomicType*, $value_sequence11 as xs:anyAtomicType*, $value_sequence12 as xs:anyAtomicType*, $value_sequence13 as xs:anyAtomicType*, $value_sequence14 as xs:anyAtomicType*, $value_sequence15 as xs:anyAtomicType*, $value_sequence16 as xs:anyAtomicType*, $value_sequence17 as xs:anyAtomicType*, $value_sequence18 as xs:anyAtomicType*, $value_sequence19 as xs:anyAtomicType*, $value_sequence20 as xs:anyAtomicType*, $value_sequence21 as xs:anyAtomicType*, $value_sequence22 as xs:anyAtomicType*, $value_sequence23 as xs:anyAtomicType*, $value_sequence24 as xs:anyAtomicType*, $value_sequence25 as xs:anyAtomicType*, $value_sequence26 as xs:anyAtomicType*, $value_sequence27 as xs:anyAtomicType*, $value_sequence28 as xs:anyAtomicType*, $value_sequence29 as xs:anyAtomicType*) as xs:anyAtomicType*", "description" : " Uses index_num to return a sequence from the list of sequences.\n Use CHOOSE to select one of up to 29 sequences based on the index number.\n", "summary" : "<p> Uses index_num to return a sequence from the list of sequences.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "index_num", "type" : "xs:integer", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the position in the sequence, 1 based</div>" }, { "name" : "value_sequence1", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> a sequence of values. Specify the empty sequence () if you don't need it.</div>" }, { "name" : "value_sequence2", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> a sequence of values. Specify the empty sequence () if you don't need it.</div>" }, { "name" : "value_sequence3", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> a sequence of values. Specify the empty sequence () if you don't need it.</div>" }, { "name" : "value_sequence4", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> a sequence of values. Specify the empty sequence () if you don't need it.</div>" }, { "name" : "value_sequence5", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> a sequence of values. Specify the empty sequence () if you don't need it.</div>" }, { "name" : "value_sequence6", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> a sequence of values. Specify the empty sequence () if you don't need it.</div>" }, { "name" : "value_sequence7", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> a sequence of values. Specify the empty sequence () if you don't need it.</div>" }, { "name" : "value_sequence8", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> a sequence of values. Specify the empty sequence () if you don't need it.</div>" }, { "name" : "value_sequence9", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> a sequence of values. Specify the empty sequence () if you don't need it.</div>" }, { "name" : "value_sequence10", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> a sequence of values. Specify the empty sequence () if you don't need it.</div>" }, { "name" : "value_sequence11", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> a sequence of values. Specify the empty sequence () if you don't need it.</div>" }, { "name" : "value_sequence12", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> a sequence of values. Specify the empty sequence () if you don't need it.</div>" }, { "name" : "value_sequence13", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> a sequence of values. Specify the empty sequence () if you don't need it.</div>" }, { "name" : "value_sequence14", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> a sequence of values. Specify the empty sequence () if you don't need it.</div>" }, { "name" : "value_sequence15", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> a sequence of values. Specify the empty sequence () if you don't need it.</div>" }, { "name" : "value_sequence16", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> a sequence of values. Specify the empty sequence () if you don't need it.</div>" }, { "name" : "value_sequence17", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> a sequence of values. Specify the empty sequence () if you don't need it.</div>" }, { "name" : "value_sequence18", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> a sequence of values. Specify the empty sequence () if you don't need it.</div>" }, { "name" : "value_sequence19", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> a sequence of values. Specify the empty sequence () if you don't need it.</div>" }, { "name" : "value_sequence20", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> a sequence of values. Specify the empty sequence () if you don't need it.</div>" }, { "name" : "value_sequence21", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> a sequence of values. Specify the empty sequence () if you don't need it.</div>" }, { "name" : "value_sequence22", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> a sequence of values. Specify the empty sequence () if you don't need it.</div>" }, { "name" : "value_sequence23", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> a sequence of values. Specify the empty sequence () if you don't need it.</div>" }, { "name" : "value_sequence24", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> a sequence of values. Specify the empty sequence () if you don't need it.</div>" }, { "name" : "value_sequence25", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> a sequence of values. Specify the empty sequence () if you don't need it.</div>" }, { "name" : "value_sequence26", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> a sequence of values. Specify the empty sequence () if you don't need it.</div>" }, { "name" : "value_sequence27", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> a sequence of values. Specify the empty sequence () if you don't need it.</div>" }, { "name" : "value_sequence28", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> a sequence of values. Specify the empty sequence () if you don't need it.</div>" }, { "name" : "value_sequence29", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> a sequence of values. Specify the empty sequence () if you don't need it.</div>" } ], "returns" : { "type" : "xs:anyAtomicType*", "description" : "The value at the index position" }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">excel-err:Value if index is smaller than 1 or bigger than 29</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 5, "name" : "hlookup", "qname" : "excel:hlookup", "signature" : "($lookup_value as xs:anyAtomicType, $table_array as xs:anyAtomicType+, $table_width as xs:integer, $table_height as xs:integer, $row_index_num as xs:integer) as xs:anyAtomicType", "description" : " Same as above, only that range_lookup is defaulted to true.\n That is, this Hlookup looks for the approximate value\n   and the first row must be ordered ascending.\n", "summary" : "<p> Same as above, only that range_lookup is defaulted to true.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "lookup_value", "type" : "xs:anyAtomicType", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the value to be searched. Allowed types are numeric, string, boolean. <p/> Boolean values are compared only with booleans. Numbers are compared only with numbers, if range_lookup is not zero. The other types are converted to string and compared to string value of all values.</div>" }, { "name" : "table_array", "type" : "xs:anyAtomicType", "occurrence" : "+", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequence of values, row after row</div>" }, { "name" : "table_width", "type" : "xs:integer", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the number of values in a row</div>" }, { "name" : "table_height", "type" : "xs:integer", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the number of rows</div>" }, { "name" : "row_index_num", "type" : "xs:integer", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the row index, 1 based</div>" } ], "returns" : { "type" : "xs:anyAtomicType", "description" : "The value found, with original type" }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">excel-err:Value if the array contains less elements than specified by table_height and table_width</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">excel-err:Value if row_index_num is outside the range 1 .. table_height</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">excel-err:Value if range_lookup is true and the value searched is smaller than the first value in the header</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">excel-err:Value if range_lookup=false and the value cannot be found</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 6, "name" : "hlookup", "qname" : "excel:hlookup", "signature" : "($lookup_value as xs:anyAtomicType, $table_array as xs:anyAtomicType+, $table_width as xs:integer, $table_height as xs:integer, $row_index_num as xs:integer, $range_lookup as xs:boolean) as xs:anyAtomicType", "description" : " Searches for a value in the top row of an array of values,\n   and then returns a value in the same column from a row you specify in the array.\n <dl xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Array is specified with 3 parameters:\n <dt>table_array</dt> <dd>is a sequence of elements, first row first, then second row and so on</dd>\n <dt>table_width</dt> <dd>specifies the number of elements in a row</dd>\n <dt>table_height</dt> <dd>specifies the number of rows</dd></dl>\n The number of elements in table_array must be equal or more than table_width * table_height.<p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n For wildchar matching, the XQuery regex matcher is used.\n", "summary" : "<p> Searches for a value in the top row of an array of values,\n   and then returns a value in the same column from a row you specify in the array.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "lookup_value", "type" : "xs:anyAtomicType", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the value to be searched. Allowed types are numeric, string, boolean. <p/> Boolean values are compared only with booleans. Numbers are compared only with numbers, if range_lookup is not zero. The other types are converted to string and compared to string value of all values.</div>" }, { "name" : "table_array", "type" : "xs:anyAtomicType", "occurrence" : "+", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequence of values, row after row</div>" }, { "name" : "table_width", "type" : "xs:integer", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the number of values in a row</div>" }, { "name" : "table_height", "type" : "xs:integer", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the number of rows</div>" }, { "name" : "row_index_num", "type" : "xs:integer", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the row index, 1 based</div>" }, { "name" : "range_lookup", "type" : "xs:boolean", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> <dl>specifies the algorithm to use: <dt>true</dt> <dd>find approximative match. First row of array must be sorted in ascending order.</dd> <dt>false</dt> <dd>find exact match, using xquery regex First row of array can be in any order. </dd></dl></div>" } ], "returns" : { "type" : "xs:anyAtomicType", "description" : "The value found, with original type" }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">excel-err:Value if the array contains less elements than specified by table_height and table_width</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">excel-err:Value if row_index_num is outside the range 1 .. table_height</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">excel-err:Value if range_lookup is true and the value searched is smaller than the first value in the header</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">excel-err:Value if range_lookup=false and the value cannot be found</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 5, "name" : "index", "qname" : "excel:index", "signature" : "($array as xs:anyAtomicType+, $array_height as xs:integer, $array_width as xs:integer, $row_num as xs:integer, $column_num as xs:integer) as xs:anyAtomicType+", "description" : " Returns a value from within an array.<p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n This is the Array form of the Excel Index function.<p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n <dl xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Array is specified with 3 parameters:\n <dt>array</dt> <dd>is a sequence of elements, first row first, then second row and so on</dd>\n <dt>array_height</dt> <dd>specifies the number of rows</dd>\n <dt>array_width</dt> <dd>specifies the number of elements in a row</dd></dl>\n The number of elements in array must be equal or more than array_width * array_height.\n", "summary" : "<p> Returns a value from within an array.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "array", "type" : "xs:anyAtomicType", "occurrence" : "+", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequence of values, row after row</div>" }, { "name" : "array_height", "type" : "xs:integer", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the number of rows</div>" }, { "name" : "array_width", "type" : "xs:integer", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the number of values in a row</div>" }, { "name" : "row_num", "type" : "xs:integer", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the row position of the value, 1 based</div>" }, { "name" : "column_num", "type" : "xs:integer", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the column position of the value, 1 based</div>" } ], "returns" : { "type" : "xs:anyAtomicType+", "description" : "The value from x-y in the array" }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">excel-err:Value if the array contains less elements than specified by table_height and table_width</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">excel-err:Ref if row_num is outside the range</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 3, "name" : "lookup", "qname" : "excel:lookup", "signature" : "($lookup_value as xs:anyAtomicType, $lookup_vector as xs:anyAtomicType+, $result_vector as xs:anyAtomicType+) as xs:anyAtomicType", "description" : " The Vector form. <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n Looks in a sequence for a value\n   and return a value from the same position in a second sequence.\n If the value is not found, then it matches the largest value in lookup_vector\n   that is less than or equal to lookup_value.\n", "summary" : "<p> The Vector form.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "lookup_value", "type" : "xs:anyAtomicType", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the value to be searched</div>" }, { "name" : "lookup_vector", "type" : "xs:anyAtomicType", "occurrence" : "+", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequence to be searched, in ascending order.</div>" }, { "name" : "result_vector", "type" : "xs:anyAtomicType", "occurrence" : "+", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequence containing the result values</div>" } ], "returns" : { "type" : "xs:anyAtomicType", "description" : "a value from $result_vector" }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">excel-err:NA if lookup value is smaller than the first value in lookup_vector</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">excel-err:NA if position found is outside the result range</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 4, "name" : "lookup", "qname" : "excel:lookup", "signature" : "($lookup_value as xs:anyAtomicType, $array as xs:anyAtomicType+, $array_width as xs:integer, $array_height as xs:integer) as xs:anyAtomicType", "description" : " The Array form.<p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n It looks in the first row or column of an array for the specified value\n   and returns a value from the same position in the last row or column of the array.<p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n If array covers an area that is wider than it is tall (more columns than rows),\n   LOOKUP searches for lookup_value in the first row.<p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n If array is square or is taller than it is wide (more rows than columns),\n   LOOKUP searches in the first column.<p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n The values in the first row or first column must be in ascending order.\n", "summary" : "<p> The Array form.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "lookup_value", "type" : "xs:anyAtomicType", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the value to be searched. If the value is not found, then it matches the largest value in lookup_vector that is less than or equal to lookup_value.</div>" }, { "name" : "array", "type" : "xs:anyAtomicType", "occurrence" : "+", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the array sequence, row after row</div>" }, { "name" : "array_width", "type" : "xs:integer", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the number of values in a row</div>" }, { "name" : "array_height", "type" : "xs:integer", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the number of rows in the array</div>" } ], "returns" : { "type" : "xs:anyAtomicType", "description" : "The corresponding value in the last row or column" }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">excel-err:Value if array contains less values than specified by array_width and array_height or array_width = 0 or array_height = 0</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">excel-err:NA if the lookup_value is smaller than the first value in the row or column</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 2, "name" : "match", "qname" : "excel:match", "signature" : "($lookup_value as xs:anyAtomicType, $sequence as xs:anyAtomicType+) as xs:anyAtomicType", "description" : " Same as above, but match_type is defaulted to 1.\n It finds the largest value that is less than or equal to lookup_value.\n", "summary" : "<p> Same as above, but match_type is defaulted to 1.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "lookup_value", "type" : "xs:anyAtomicType", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> value to be searched.</div>" }, { "name" : "sequence", "type" : "xs:anyAtomicType", "occurrence" : "+", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the vector where to search the value</div>" } ], "returns" : { "type" : "xs:anyAtomicType", "description" : "The position of found value" }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">excel-err:NA for match_type 1 or -1, the lookup_value is smaller or larger than the first value in sequence</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">excel-err:Value if range_lookup=0 and the value cannot be found</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 3, "name" : "match", "qname" : "excel:match", "signature" : "($lookup_value as xs:anyAtomicType, $sequence as xs:anyAtomicType+, $match_type as xs:integer) as xs:anyAtomicType", "description" : " Returns the relative position of an item in a sequence that\n   matches a specified value in a specified order.\n Only for one dimensional vector.\n", "summary" : "<p> Returns the relative position of an item in a sequence that\n   matches a specified value in a specified order.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "lookup_value", "type" : "xs:anyAtomicType", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> value to be searched.</div>" }, { "name" : "sequence", "type" : "xs:anyAtomicType", "occurrence" : "+", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the vector where to search the value</div>" }, { "name" : "match_type", "type" : "xs:integer", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> <dl>specifies the algorithm used for searching. Possible values: <dt>1</dt> <dd> finds the largest value that is less than or equal to lookup_value. Sequence must be in ascending order.</dd> <dt>0</dt> <dd> finds the first value that is exactly equal to lookup_value. <p/> Sequence can be in any order.<p/> If lookup_value is boolean, then only booleans are compared.<p/> For other types, they are casted to string and then compared using xquery regular expressions. Lookup_value can be a xquery regular expression.</dd> <dt>-1</dt> <dd> finds the smallest value that is greater than or equal to lookup_value.<p/> Sequence must be in descending order.</dd></dl></div>" } ], "returns" : { "type" : "xs:anyAtomicType", "description" : "The position of found value" }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">excel-err:NA for match_type 1 or -1, the lookup_value is smaller or larger than the first value in sequence</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">excel-err:Value if range_lookup=0 and the value cannot be found</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 5, "name" : "offset", "qname" : "excel:offset", "signature" : "($reference as xs:anyAtomicType+, $reference_height as xs:integer, $reference_width as xs:integer, $rows as xs:integer, $cols as xs:integer) as xs:anyAtomicType*", "description" : " Same as above, only that the sub-array is specified only by rows and cols relative position.\n The sub-array height and width is computed to contain the remaining elements of the array.\n", "summary" : "<p> Same as above, only that the sub-array is specified only by rows and cols relative position.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "reference", "type" : "xs:anyAtomicType", "occurrence" : "+", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the reference array</div>" }, { "name" : "reference_height", "type" : "xs:integer", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the number of rows in the reference array</div>" }, { "name" : "reference_width", "type" : "xs:integer", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the number of elements in the reference array row</div>" }, { "name" : "rows", "type" : "xs:integer", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the relative row position where the sub-array starts. It must be a positive value, zero relative.</div>" }, { "name" : "cols", "type" : "xs:integer", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the relative column position where the sub-array starts. It must be a positive value, zero relative.</div>" } ], "returns" : { "type" : "xs:anyAtomicType*", "description" : "The sequence specifying the sub-array, row after row" }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">excel-err:NA rows or cols are negative</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">excel-err:NA height or width are smaller than 1</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">excel-err:Value reference array contains less elements than specified by reference_height and reference_width</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">excel-err:NA the resulted sub-array is not completely contained inside reference array</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 7, "name" : "offset", "qname" : "excel:offset", "signature" : "($reference as xs:anyAtomicType+, $reference_height as xs:integer, $reference_width as xs:integer, $rows as xs:integer, $cols as xs:integer, $height as xs:integer, $width as xs:integer) as xs:anyAtomicType*", "description" : " Returns a sub-array from an array.\n The inner array must be within the reference array\n", "summary" : "<p> Returns a sub-array from an array.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "reference", "type" : "xs:anyAtomicType", "occurrence" : "+", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the reference array</div>" }, { "name" : "reference_height", "type" : "xs:integer", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the number of rows in the reference array</div>" }, { "name" : "reference_width", "type" : "xs:integer", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the number of elements in the reference array row</div>" }, { "name" : "rows", "type" : "xs:integer", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the relative row position where the sub-array starts. It must be a positive value, zero relative.</div>" }, { "name" : "cols", "type" : "xs:integer", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the relative column position where the sub-array starts. It must be a positive value, zero relative.</div>" }, { "name" : "height", "type" : "xs:integer", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the desired height of sub-array. The sub-array must be inside the reference array.</div>" }, { "name" : "width", "type" : "xs:integer", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the desired width of sub-array. The sub-array must be inside the reference array.</div>" } ], "returns" : { "type" : "xs:anyAtomicType*", "description" : "The sequence specifying the sub-array, row after row" }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">excel-err:NA rows or cols are negative</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">excel-err:NA height or width are smaller than 1</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">excel-err:Value reference array contains less elements than specified by reference_height and reference_width</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">excel-err:NA the resulted sub-array is not completely contained inside reference array</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 3, "name" : "transpose", "qname" : "excel:transpose", "signature" : "($array as xs:anyAtomicType+, $array_width as xs:integer, $array_height as xs:integer) as xs:anyAtomicType+", "description" : " Transposes an array. The rows become columns and vice versa.\n", "summary" : "<p> Transposes an array.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "array", "type" : "xs:anyAtomicType", "occurrence" : "+", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequence specifying the array, row after row</div>" }, { "name" : "array_width", "type" : "xs:integer", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the number of elements in a row</div>" }, { "name" : "array_height", "type" : "xs:integer", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the number of rows in the array</div>" } ], "returns" : { "type" : "xs:anyAtomicType+", "description" : "The transposed array. It will be a sequence specifying an array, row after row. The result width is the input height. The result height is the input width." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">excel-err:Value the array contains less elements than specified by array_width and array_height</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 5, "name" : "vlookup", "qname" : "excel:vlookup", "signature" : "($lookup_value as xs:anyAtomicType, $table_array as xs:anyAtomicType+, $table_width as xs:integer, $table_height as xs:integer, $col_index_num as xs:integer) as xs:anyAtomicType", "description" : " Same as above, with range_lookup defaulted to true.\n It finds the largest value that is less than or equal to lookup_value.\n First column must be in ascending order.\n", "summary" : "<p> Same as above, with range_lookup defaulted to true.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "lookup_value", "type" : "xs:anyAtomicType", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the value to be searched. Allowed types are numeric, string, boolean. <p/> Boolean values are compared only with booleans. Numbers are compared only with numbers, if range_lookup is not zero. The other types are converted to string and compared to string value of all values.</div>" }, { "name" : "table_array", "type" : "xs:anyAtomicType", "occurrence" : "+", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequence of values, row after row</div>" }, { "name" : "table_width", "type" : "xs:integer", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the number of values in a row</div>" }, { "name" : "table_height", "type" : "xs:integer", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the number of rows</div>" }, { "name" : "col_index_num", "type" : "xs:integer", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the row index, 1 based</div>" } ], "returns" : { "type" : "xs:anyAtomicType", "description" : "The value found, with original type" }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">excel-err:Value if the array contains less elements than specified by table_height and table_width</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">excel-err:Value if col_index_num is outside the range 1 .. table_height</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">excel-err:Value if range_lookup is true and the value searched is smaller than the first value in the first column</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">excel-err:Value if range_lookup=false and the value cannot be found</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 6, "name" : "vlookup", "qname" : "excel:vlookup", "signature" : "($lookup_value as xs:anyAtomicType, $table_array as xs:anyAtomicType+, $table_width as xs:integer, $table_height as xs:integer, $col_index_num as xs:integer, $range_lookup as xs:boolean) as xs:anyAtomicType", "description" : " Searches for a value in the first column of a table array\n   and returns a value in the same row from another column in the table array.\n <dl xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Array is specified with 3 parameters:\n <dt>table_array</dt> <dd>is a sequence of elements, first row first, then second row and so on</dd>\n <dt>table_width</dt> <dd>specifies the number of elements in a row</dd>\n <dt>table_height</dt> <dd>specifies the number of rows</dd></dl>\n For wildchar matching, the XQuery regex matcher is used.\n", "summary" : "<p> Searches for a value in the first column of a table array\n   and returns a value in the same row from another column in the table array.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "lookup_value", "type" : "xs:anyAtomicType", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the value to be searched. Allowed types are numeric, string, boolean. <p/> Boolean values are compared only with booleans. Numbers are compared only with numbers, if range_lookup is not zero. The other types are converted to string and compared to string value of all values.</div>" }, { "name" : "table_array", "type" : "xs:anyAtomicType", "occurrence" : "+", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequence of values, row after row</div>" }, { "name" : "table_width", "type" : "xs:integer", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the number of values in a row</div>" }, { "name" : "table_height", "type" : "xs:integer", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the number of rows</div>" }, { "name" : "col_index_num", "type" : "xs:integer", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the row index, 1 based</div>" }, { "name" : "range_lookup", "type" : "xs:boolean", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> <dl>specified the algorithm to use: <dt>true</dt> <dd> find approximative match. First column of array must be sorted in ascending order.</dd> <dt>false</dt> <dd> find exact match, using xquery regex. First column of array can be in any order.</dd></dl></div>" } ], "returns" : { "type" : "xs:anyAtomicType", "description" : "The value found, with original type" }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">excel-err:Value if the array contains less elements than specified by table_height and table_width</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">excel-err:Value if col_index_num is outside the range 1 .. table_height</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">excel-err:Value if range_lookup is true and the value searched is smaller than the first value in the first column</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">excel-err:Value if range_lookup=false and the value cannot be found</xqdoc:error>" ] } ], "variables" : [  ] }, "http://zorba.io/modules/datetime" : { "ns" : "http://zorba.io/modules/datetime", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">This module provides functions to retrieve the current dateTime and to\n parse dates and times.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">In contrast to the current-dateTime functions specified in\n <a href=\"http://www.w3.org/TR/xpath-functions-30/\">XQuery Functions and\n Operators</a>, the functions in this module are nondeterministic, that is,\n they do not return the current dateTime from the dynamic context, but return\n the actual value.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Dates and times are parsed according to the format given by\n <a href=\"http://pubs.opengroup.org/onlinepubs/007904975/functions/strptime.html\">strptime</a>.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">However, date and time values must be \"complete.\"</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">For a date, the year and either month and day or day of the year must have\n been parsed.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">For a time, the hour must have been parsed.\n (If either the minute, second, or timezone has not been parsed, they default\n to 0.)</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">For a dateTime, the parsing requirements of both date and time must be met.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">When a locale is given,\n it must be of the form {lang}[{sep}{country}[{encoding}]] where\n {lang} is an ISO 639-1 2-letter or 639-2 3-letter language code,\n {sep} is either '-' or '_',\n {country} is an ISO 3166-1 2-letter country code,\n and {encoding} is any string that begins with a '.'.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">The {sep}, {country}, and {encoding} are optional;\n {encoding} is always ignored.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Examples include: de, en-US, fr_CA, ru_RU.UTF-8.</p>\n", "sees" : [ "<xqdoc:see xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">http://www.w3.org/TR/xpath-functions/#context</xqdoc:see>" ], "authors" : [ "<xqdoc:author xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Matthias Brantner</xqdoc:author>", "<xqdoc:author xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Paul J. Lucas</xqdoc:author>" ], "version" : null, "encoding" : "utf-8", "namespaces" : [ { "uri" : "http://zorba.io/annotations", "prefix" : "an" }, { "uri" : "http://zorba.io/modules/datetime", "prefix" : "datetime" }, { "uri" : "http://zorba.io/options/versioning", "prefix" : "ver" } ], "functions" : [ { "isDocumented" : true, "arity" : 0, "name" : "current-date", "qname" : "datetime:current-date", "signature" : "() as xs:date external", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Gets the current date value in Universal time.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Note that this function is not stable: it returns the value of the date when\n the function is invoked.</p>\n", "summary" : "<p>  Gets the current date value in Universal time.</p>", "annotation_str" : " %an:nondeterministic", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "nondeterministic", "value" : "" } ], "updating" : false, "parameters" : [  ], "returns" : { "type" : "xs:date", "description" : "the non-stable date value" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 0, "name" : "current-dateTime", "qname" : "datetime:current-dateTime", "signature" : "() as xs:dateTimeStamp external", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Gets the current dateTime value in Universal time.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Note that this function is not stable: it returns the value of the date and\n time when the function is invoked.</p>\n", "summary" : "<p>  Gets the current dateTime value in Universal time.</p>", "annotation_str" : " %an:nondeterministic", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "nondeterministic", "value" : "" } ], "updating" : false, "parameters" : [  ], "returns" : { "type" : "xs:dateTimeStamp", "description" : "the non-stable datetime value" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 0, "name" : "current-time", "qname" : "datetime:current-time", "signature" : "() as xs:time external", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Return the current time value in Universal time.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Note that this function is not stable: it returns the value of the time when\n the function is invoked.</p>\n", "summary" : "<p>  Return the current time value in Universal time.</p>", "annotation_str" : " %an:nondeterministic", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "nondeterministic", "value" : "" } ], "updating" : false, "parameters" : [  ], "returns" : { "type" : "xs:time", "description" : "the non-stable time value" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "millis-to-dateTime", "qname" : "datetime:millis-to-dateTime", "signature" : "($millis as xs:long) as xs:dateTime external", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Converts the given number of milliseconds since epoch into its corresponding\n xs:dateTime.</p>\n", "summary" : "<p>  Converts the given number of milliseconds since epoch into its corresponding\n xs:dateTime.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "millis", "type" : "xs:long", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The number of milliseconds since epoch.</div>" } ], "returns" : { "type" : "xs:dateTime", "description" : "Returns an xs:dateTime." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 2, "name" : "parse-date", "qname" : "datetime:parse-date", "signature" : "($input as xs:string, $format as xs:string) as xs:date external", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Parses a date from a string in the current locale.</p>\n", "summary" : "<p>  Parses a date from a string in the current locale.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "input", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The string to parse.</div>" }, { "name" : "format", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The format string containing zero or more conversion specifications and ordinary characters. All ordinary characters are matched exactly with the buffer; all whitespace characters match any amount of whitespace in the buffer.</div>" } ], "returns" : { "type" : "xs:date", "description" : "Returns an xs:date." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">datetime:INVALID_SPECIFICATION if $format contains an invalid conversion specification.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">datetime:INSUFFICIENT_BUFFER if $input is insufficient for $format.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">datetime:INVALID_VALUE if $input contains an invalid value for a conversion specification.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">datetime:LITERAL_MISMATCH if there is a literal characer mismatch between $input and $format.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">datetime:INCOMPLETE_DATE_OR_TIME if the date is incomplete.</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 3, "name" : "parse-date", "qname" : "datetime:parse-date", "signature" : "($input as xs:string, $format as xs:string, $locale as xs:string) as xs:date external", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Parses a date from a string in the given locale.</p>\n", "summary" : "<p>  Parses a date from a string in the given locale.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "input", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The string to parse.</div>" }, { "name" : "format", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The format string containing zero or more conversion specifications and ordinary characters. All ordinary characters are matched exactly with the buffer; all whitespace characters match any amount of whitespace in the buffer.</div>" }, { "name" : "locale", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The locale to use.</div>" } ], "returns" : { "type" : "xs:date", "description" : "Returns an xs:date." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">datetime:INVALID_SPECIFICATION if $format contains an invalid conversion specification.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">datetime:INSUFFICIENT_BUFFER if $input is insufficient for $format.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">datetime:INVALID_VALUE if $input contains an invalid value for a conversion specification.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">datetime:LITERAL_MISMATCH if there is a literal characer mismatch between $input and $format.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">datetime:INCOMPLETE_DATE_OR_TIME if the date is incomplete.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">datetime:INVALID_LOCALE if $locale is in an invalid format.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">datetime:UNKNOWN_LOCALE if $locale is unknown.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">datetime:UNSUPPORTED_LOCALE if $locale is unsupported by the operating system.</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 2, "name" : "parse-dateTime", "qname" : "datetime:parse-dateTime", "signature" : "($input as xs:string, $format as xs:string) as xs:dateTime external", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Parses a dateTime from a string in the current locale.</p>\n", "summary" : "<p>  Parses a dateTime from a string in the current locale.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "input", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The string to parse.</div>" }, { "name" : "format", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The format string containing zero or more conversion specifications and ordinary characters. All ordinary characters are matched exactly with the buffer; all whitespace characters match any amount of whitespace in the buffer.</div>" } ], "returns" : { "type" : "xs:dateTime", "description" : "Returns an xs:dateTime." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">datetime:INVALID_SPECIFICATION if $format contains an invalid conversion specification.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">datetime:INSUFFICIENT_BUFFER if $input is insufficient for $format.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">datetime:INVALID_VALUE if $input contains an invalid value for a conversion specification.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">datetime:LITERAL_MISMATCH if there is a literal characer mismatch between $input and $format.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">datetime:INCOMPLETE_DATE_OR_TIME if either the date or time is incomplete.</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 3, "name" : "parse-dateTime", "qname" : "datetime:parse-dateTime", "signature" : "($input as xs:string, $format as xs:string, $locale as xs:string) as xs:dateTime external", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Parses a dateTime from a string in the given locale.</p>\n", "summary" : "<p>  Parses a dateTime from a string in the given locale.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "input", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The string to parse.</div>" }, { "name" : "format", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The format string containing zero or more conversion specifications and ordinary characters. All ordinary characters are matched exactly with the buffer; all whitespace characters match any amount of whitespace in the buffer.</div>" }, { "name" : "locale", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The locale to use.</div>" } ], "returns" : { "type" : "xs:dateTime", "description" : "Returns an xs:dateTime." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">datetime:INVALID_SPECIFICATION if $format contains an invalid conversion specification.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">datetime:INSUFFICIENT_BUFFER if $input is insufficient for $format.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">datetime:INVALID_VALUE if $input contains an invalid value for a conversion specification.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">datetime:LITERAL_MISMATCH if there is a literal characer mismatch between $input and $format.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">datetime:INCOMPLETE_DATE_OR_TIME if either the date or time is incomplete.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">datetime:INVALID_LOCALE if $locale is in an invalid format.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">datetime:UNKNOWN_LOCALE if $locale is unknown.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">datetime:UNSUPPORTED_LOCALE if $locale is unsupported by the operating system.</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 2, "name" : "parse-time", "qname" : "datetime:parse-time", "signature" : "($input as xs:string, $format as xs:string) as xs:time external", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Parses a time from a string in the current locale.</p>\n", "summary" : "<p>  Parses a time from a string in the current locale.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "input", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The string to parse.</div>" }, { "name" : "format", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The format string containing zero or more conversion specifications and ordinary characters. All ordinary characters are matched exactly with the buffer; all whitespace characters match any amount of whitespace in the buffer.</div>" } ], "returns" : { "type" : "xs:time", "description" : "Returns an xs:time." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">datetime:INVALID_SPECIFICATION if $format contains an invalid conversion specification.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">datetime:INSUFFICIENT_BUFFER if $input is insufficient for $format.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">datetime:INVALID_VALUE if $input contains an invalid value for a conversion specification.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">datetime:LITERAL_MISMATCH if there is a literal characer mismatch between $input and $format.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">datetime:INCOMPLETE_DATE_OR_TIME if the hour has not been parsed.</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 3, "name" : "parse-time", "qname" : "datetime:parse-time", "signature" : "($input as xs:string, $format as xs:string, $locale as xs:string) as xs:time external", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Parses a time from a string in the given locale.</p>\n", "summary" : "<p>  Parses a time from a string in the given locale.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "input", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The string to parse.</div>" }, { "name" : "format", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The format string containing zero or more conversion specifications and ordinary characters. All ordinary characters are matched exactly with the buffer; all whitespace characters match any amount of whitespace in the buffer.</div>" }, { "name" : "locale", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The locale to use.</div>" } ], "returns" : { "type" : "xs:time", "description" : "Returns an xs:time." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">datetime:INVALID_SPECIFICATION if $format contains an invalid conversion specification.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">datetime:INSUFFICIENT_BUFFER if $input is insufficient for $format.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">datetime:INVALID_VALUE if $input contains an invalid value for a conversion specification.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">datetime:LITERAL_MISMATCH if there is a literal characer mismatch between $input and $format.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">datetime:INCOMPLETE_DATE_OR_TIME if the hour has not been parsed.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">datetime:INVALID_LOCALE if $locale is in an invalid format.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">datetime:UNKNOWN_LOCALE if $locale is unknown.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">datetime:UNSUPPORTED_LOCALE if $locale is unsupported by the operating system.</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 0, "name" : "timestamp", "qname" : "datetime:timestamp", "signature" : "() as xs:long external", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Gets the the number of milliseconds since epoch.</p>\n", "summary" : "<p>  Gets the the number of milliseconds since epoch.</p>", "annotation_str" : " %an:nondeterministic", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "nondeterministic", "value" : "" } ], "updating" : false, "parameters" : [  ], "returns" : { "type" : "xs:long", "description" : "the said number of milliseconds." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 0, "name" : "utc-offset", "qname" : "datetime:utc-offset", "signature" : "() as xs:long external", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Gets the offset of the current timezone from Universal time.</p>\n", "summary" : "<p>  Gets the offset of the current timezone from Universal time.</p>", "annotation_str" : " %an:nondeterministic", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "nondeterministic", "value" : "" } ], "updating" : false, "parameters" : [  ], "returns" : { "type" : "xs:long", "description" : "the offset in seconds with positive values being east of the prime meridian." }, "errors" : [  ] } ], "variables" : [  ] }, "http://www.28msec.com/modules/http/util/diagnostic" : { "ns" : "http://www.28msec.com/modules/http/util/diagnostic", "description" : " This module provides utility functions to help with diagnostic analysis\n / debugging of RESTful Apps or webapps.\n", "sees" : [  ], "authors" : [ "<xqdoc:author xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">28msec</xqdoc:author>" ], "version" : null, "encoding" : "utf-8", "namespaces" : [ { "uri" : "http://zorba.io/modules/base64", "prefix" : "base64" }, { "uri" : "http://www.28msec.com/modules/http/util/diagnostic", "prefix" : "diagnostic" }, { "uri" : "http://www.w3.org/2005/xqt-errors", "prefix" : "err" }, { "uri" : "http://www.28msec.com/modules/http/util/multipart", "prefix" : "multipart" }, { "uri" : "http://www.28msec.com/modules/http/request", "prefix" : "request" }, { "uri" : "http://zorba.io/options/versioning", "prefix" : "ver" } ], "functions" : [ { "isDocumented" : true, "arity" : 0, "name" : "serialize-request-as-html", "qname" : "diagnostic:serialize-request-as-html", "signature" : "() as element(*)*", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Returns a sequence of html elements listing all request\n characteristics.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">This function is helpful for debugging purposes. It can be used\n to output request information within an HTML page.</p>\n", "summary" : "<p>  Returns a sequence of html elements listing all request\n characteristics.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [  ], "returns" : { "type" : "element(*)*", "description" : "HTML elements describing the request" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 0, "name" : "serialize-request-as-txt", "qname" : "diagnostic:serialize-request-as-txt", "signature" : "() as xs:string*", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Returns a sequence of xs:string listing all request\n characteristics.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">This function is helpful for debugging purposes. It can be used\n to output request information as plan text.</p>\n", "summary" : "<p>  Returns a sequence of xs:string listing all request\n characteristics.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [  ], "returns" : { "type" : "xs:string*", "description" : "sequence of strings describing each characteristic of a request" }, "errors" : [  ] } ], "variables" : [  ] }, "http://www.28msec.com/modules/credentials" : { "ns" : "http://www.28msec.com/modules/credentials", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">This module provides functionality for storing credentials\n using an AES-encrypted file the project configuration folder.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Specifically, the credentials will be stored in the <tt>credentials</tt>\n file in the <tt>config</tt> folder in the project root folder.\n The credentials file is encrypted using 256bit AES encryption. The AES\n key is computed using the project seed. If the project seed is changed the\n credential store is re-encrypted transparently.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Each credential has an associated name and category.\n Credentials can be retrieved specifying their name and category.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">For each category, it is possible to specify the default credentials.\n A category default credentials can then be easily retrieved. </p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">The name of credentials and categories can contain lowercase or\n uppercase letters, digits, and the following special characters: \"_\" , \"-\",\n \".\". Additionally it must not be empty and must start with a lowercase\n or uppercase letter.</p>\n", "sees" : [  ], "authors" : [ "<xqdoc:author xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Federico Cavalieri</xqdoc:author>" ], "version" : null, "encoding" : "utf-8", "namespaces" : [ { "uri" : "http://zorba.io/annotations", "prefix" : "an" }, { "uri" : "http://www.28msec.com/modules/credentials", "prefix" : "credentials" }, { "uri" : "http://www.w3.org/2005/xqt-errors", "prefix" : "err" }, { "uri" : "http://jsoniq.org/errors", "prefix" : "jerr" }, { "uri" : "http://www.28msec.com/modules/mongodb", "prefix" : "mongo" }, { "uri" : "http://zorba.io/options/versioning", "prefix" : "ver" } ], "functions" : [ { "isDocumented" : true, "arity" : 3, "name" : "add-credentials", "qname" : "credentials:add-credentials", "signature" : "($category as string, $name as string, $credentials as object()) as object()", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Adds the given credentials to the credential store.\n The credentials are not set as default.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">If credentials  with the given name and category are already present an\n error is raised.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Example:\n <pre>\n import module namespace credentials = \"http://www.28msec.com/modules/credentials\";\n credentials:add-credentials(\"S3\", \"DataBucket\", { ... });\n </pre>\n </p>\n", "summary" : "<p>  Adds the given credentials to the credential store.</p>", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "category", "type" : "string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The credentials category name.</div>" }, { "name" : "name", "type" : "string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The credentials name.</div>" }, { "name" : "credentials", "type" : "object()", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> An object containing the credentials data.</div>" } ], "returns" : { "type" : "object()", "description" : "the credential object" }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">credentials:EXIST if credentials with the given name and category are already present.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">credentials:NAME if the credentials or category name is invalid</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">credentials:MONGO-DEFAULT if the specified credentials and category name correspond with the default MongoDB credentials</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">credentials:TEST if a connection cannot be established using the specified credentials</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">credentials:CREDENTIALS-STORE if an internal error arises accessing the crendentials store</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 4, "name" : "add-credentials", "qname" : "credentials:add-credentials", "signature" : "($category as string, $name as string, $credentials as object(), $default as boolean) as object()", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Adds the given credentials to the credential store,\n either as default or not.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">If credentials  with the given name and category are already present an\n error is raised.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Example:\n <pre>\n import module namespace credentials = \"http://www.28msec.com/modules/credentials\";\n credentials:add-credentials(\"S3\", \"DataBucket\", { ... }, true);\n </pre>\n </p>\n", "summary" : "<p>  Adds the given credentials to the credential store,\n either as default or not.</p>", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "category", "type" : "string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The credentials category name.</div>" }, { "name" : "name", "type" : "string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The credentials name.</div>" }, { "name" : "credentials", "type" : "object()", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> An object containing the credentials data.</div>" }, { "name" : "default", "type" : "boolean", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> Whether the credentials will be default or not.</div>" } ], "returns" : { "type" : "object()", "description" : "the credential object" }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">credentials:EXIST if credentials with the given name and category are already present.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">credentials:NAME if the credentials or category name is invalid</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">credentials:MONGO-DEFAULT if trying to add the default MongoDB credentials</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">credentials:TEST if a connection cannot be established using the specified credentials</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">credentials:CREDENTIALS-STORE if an internal error arises accessing the crendentials store</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 1, "name" : "credentials", "qname" : "credentials:credentials", "signature" : "($category as string) as object()?", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Returns the given category's default credentials, if any.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">If no default credentials are present for the given category\n the empty sequence is returned.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Example:\n <pre>\n import module namespace credentials = \"http://www.28msec.com/modules/credentials\";\n credentials:credentials(\"S3\")\n </pre>\n </p>\n", "summary" : "<p>  Returns the given category's default credentials, if any.</p>", "annotation_str" : " %an:strictlydeterministic", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "strictlydeterministic", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "category", "type" : "string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The category name.</div>" } ], "returns" : { "type" : "object()?", "description" : "The specified category default credentials, if any." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">credentials:NAME if the credentials or category name is invalid</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">credentials:MONGO-DEFAULT if the specified category is MongoDB</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">credentials:CREDENTIALS-STORE if an internal error arises accessing the crendentials store</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 2, "name" : "credentials", "qname" : "credentials:credentials", "signature" : "($category as string, $name as string) as object()?", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Returns the specified credentials, if present in the credential store.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">If the specified credentials are not present the empty sequence is returned.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Example:\n <pre>\n import module namespace credentials = \"http://www.28msec.com/modules/credentials\";\n credentials:credentials(\"S3\", \"DataBucket\")\n </pre>\n </p>\n", "summary" : "<p>  Returns the specified credentials, if present in the credential store.</p>", "annotation_str" : " %an:strictlydeterministic", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "strictlydeterministic", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "category", "type" : "string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The credentials category name.</div>" }, { "name" : "name", "type" : "string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The credentials name.</div>" } ], "returns" : { "type" : "object()?", "description" : "The specified credentials, if present." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">credentials:NAME if the credentials or category name is invalid</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">credentials:MONGO-DEFAULT if the credentials or category name is invalid</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">credentials:CREDENTIALS-STORE if an internal error arises accessing the crendentials store</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 0, "name" : "list-categories", "qname" : "credentials:list-categories", "signature" : "() as array()", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Lists all the credentials categories in the credentials store.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">If no credentials are stored the empty sequence is returned.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Example:\n <pre>\n import module namespace credentials = \"http://www.28msec.com/modules/credentials\";\n credentials:list-categories()\n </pre>\n </p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">The returned array contains the name of each distinct credential category.</p>\n", "summary" : "<p>  Lists all the credentials categories in the credentials store.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [  ], "returns" : { "type" : "array()", "description" : "An array of all the distinct credentials categories in the credentials store." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">credentials:CREDENTIALS-STORE if an internal error arises accessing the crendentials store</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 1, "name" : "list-category-credentials", "qname" : "credentials:list-category-credentials", "signature" : "($category-name as string) as array()", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Lists all the credentials of the specified category in the\n credentials store.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">If no credentials for the given category are stored the empty\n sequence is returned.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Example:\n <pre>\n import module namespace credentials = \"http://www.28msec.com/modules/credentials\";\n credentials:list-category-credentials(\"category-name\")\n </pre>\n </p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">The returned array contains one object for each credential. Each object has\n the following structure:\n <pre>\n {\n   \"category\": \"category-name\",\n   \"name\": \"credential-name\",\n   \"default\": true\n }\n </pre>\n Specifically the fields of each object have the following meaning:\n <ul>\n  <li>category: the name of the credentials category (string)</li>\n  <li>name: the name of the credentials (string)</li>\n  <li>default: whether the credentials are the default credentials in\n      their category (boolean)</li>\n </ul>\n </p>\n", "summary" : "<p>  Lists all the credentials of the specified category in the\n credentials store.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "category-name", "type" : "string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The credentials category name.</div>" } ], "returns" : { "type" : "array()", "description" : "An array of all the credentials of the specified category in the credentials store." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">credentials:NAME if the credentials or category name is invalid</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">credentials:CREDENTIALS-STORE if an internal error arises accessing the crendentials store</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 0, "name" : "list-credentials", "qname" : "credentials:list-credentials", "signature" : "() as array()", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Lists all the credentials in the credentials store.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">If no credentials are stored the empty sequence is returned.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Example:\n <pre>\n import module namespace credentials = \"http://www.28msec.com/modules/credentials\";\n credentials:list-credentials()\n </pre>\n </p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">The returned array contains one object for each credential. Each object has\n the following structure:\n <pre>\n {\n   \"category\": \"category-name\",\n   \"name\": \"credential-name\",\n   \"default\": true\n }\n </pre>\n Specifically the fields of each object have the following meaning:\n <ul>\n  <li>category: the name of the credentials category (string)</li>\n  <li>name: the name of the credentials (string)</li>\n  <li>default: whether the credentials are the default credentials in\n      their category (boolean)</li>\n </ul>\n </p>\n", "summary" : "<p>  Lists all the credentials in the credentials store.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [  ], "returns" : { "type" : "array()", "description" : "An array of all credentials in the credentials store." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">credentials:CREDENTIALS-STORE if an internal error arises accessing the crendentials store</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 2, "name" : "make-default", "qname" : "credentials:make-default", "signature" : "($category as string, $name as string) as empty-sequence()", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Makes the specified credentials the default for their category.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">It is not allowed to call this method on credentials in the \"MongoDB\" category.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">If the specified credentials are not present an error is raised.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Example:\n <pre>\n import module namespace credentials = \"http://www.28msec.com/modules/credentials\";\n credentials:make-default(\"S3\", \"DataBucket\");\n </pre>\n </p>\n", "summary" : "<p>  Makes the specified credentials the default for their category.</p>", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "category", "type" : "string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The credentials category name.</div>" }, { "name" : "name", "type" : "string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The credentials name.</div>" } ], "returns" : { "type" : "empty-sequence()", "description" : "The empty sequence." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">credentials:NOT-EXIST if no credentials with the given name and category are present.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">credentials:NAME if the credentials or category name is invalid</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">credentials:MONGO-DEFAULT if the specified category is MongoDB</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">credentials:CREDENTIALS-STORE if an internal error arises accessing the crendentials store</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 2, "name" : "remove-credentials", "qname" : "credentials:remove-credentials", "signature" : "($category as string, $name as string) as empty-sequence()", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Removes the specified credentials from the credential store.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">If no credentials with the given name and category are present\n in the category store an error is raised.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Example:\n <pre>\n import module namespace credentials = \"http://www.28msec.com/modules/credentials\";\n credentials:remove-credentials(\"S3\", \"DataBucket\");\n </pre>\n </p>\n", "summary" : "<p>  Removes the specified credentials from the credential store.</p>", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "category", "type" : "string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The credentials category name.</div>" }, { "name" : "name", "type" : "string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The credentials name.</div>" } ], "returns" : { "type" : "empty-sequence()", "description" : "The empty sequence." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">credentials:NOT-EXIST if no credentials with the given name and category exist.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">credentials:NAME if the credentials or category name is invalid</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">credentials:MONGO-DEFAULT if the specified credentials and category name correspond with the default MongoDB credentials</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">credentials:CREDENTIALS-STORE if an internal error arises accessing the crendentials store</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 5, "name" : "update-credentials", "qname" : "credentials:update-credentials", "signature" : "($category as string, $name as string, $new-default as boolean?, $new-name as string?, $new-credentials as object()?) as empty-sequence()", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Updates the specified credentials in the credential store.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">It is possible to specify whether the credentials should become or stop being the\n default credentials in their category, rename the credentials or change the credentials\n object. These three changes are specified by means of the $new-default, $new-name and\n $new-credentials parameter. If the corresponding parameter is the empty sequence the\n corresponding property will not be changed.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">For example the following query renames the \"DataBucket\" credentials in the \"S3\"\n category as \"Bucket\":\n <pre>\n import module namespace credentials = \"http://www.28msec.com/modules/credentials\";\n credentials:update-credentials(\"S3\", \"DataBucket\", (), \"Bucket\", () );\n </pre>\n </p>\n", "summary" : "<p>  Updates the specified credentials in the credential store.</p>", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "category", "type" : "string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The credentials category name.</div>" }, { "name" : "name", "type" : "string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The credentials name.</div>" }, { "name" : "new-default", "type" : "boolean", "occurrence" : "?", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> Whether the credentials should become/stop being the default credentials in their category. If it is an empty sequence the current default property will not be changed.</div>" }, { "name" : "new-name", "type" : "string", "occurrence" : "?", "description" : "" }, { "name" : "new-credentials", "type" : "object()", "occurrence" : "?", "description" : "" } ], "returns" : { "type" : "empty-sequence()", "description" : "the empty sequence" }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">credentials:NOT-EXIST if credentials with the given name and category do not exist.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">credentials:EXIST if credentials with the given new name and category are already present.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">credentials:NAME if the credentials or category name is invalid</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">credentials:MONGO-DEFAULT if the specified credentials or category name correspond with the default MongoDB credentials</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">credentials:TEST if a connection cannot be established using the specified credentials</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">credentials:CREDENTIALS-STORE if an internal error arises accessing the crendentials store</xqdoc:error>" ] } ], "variables" : [  ] }, "http://www.28msec.com/modules/parallelism" : { "ns" : "http://www.28msec.com/modules/parallelism", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">This module provides functionality for executing FLWOR queries in parallel.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">There are two kinds of parallel jobs: map jobs and shuffle jobs.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">A map job runs a mapping function on the items of an input collection. The input\n collection is chunked and the mapping function is called on each chunk in parallel.\n The results are either inserted in a single collection, or on one ouptut collection\n for each chunk.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">A shuffle job shuffles the items of an input collection to several output collections.\n The input collection is chunked and each chunk is processed in parallel.\n The selection of the output collection for each item is done with a shuffling function.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">It is also possible to run a map or shuffle job on the output of another map or shuffle\n job (piping).\n Keep in mind though that if an intermediate output is in a\n single collection, no chunking can be done: only the first job of the pipeline can chunk\n its input.\n Hence, for performance, it makes sense that all intermediate jobs\n in the pipeline (except the last one, although not compulsory) output to automatically\n generated temporary collections rather than a single collection. That way, the next job\n can be executed on these collections in parallel.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Map jobs allow the parallel execution of \"simple\" FLWORs: a for on a big collection, let clauses,\n small for clauses, where clauses.\n Map-shuffle-map piping allows the parallel execution of FLWORs containing a group by clause.\n Jobs can be composed in many ways, for example map-map-shuffle-map-shuffle-map. This way,\n FLWORs with several group by clauses can be parallelized.\n In some circumstances, FLWORs with order by clauses (using bucket sort) can also be\n parallelized, but one should keep in mind that collections are not ordered so that\n any ordering within a map job is susceptible to be lost in the output.</p>\n", "sees" : [  ], "authors" : [ "<xqdoc:author xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Ghislain Fourny</xqdoc:author>", "<xqdoc:author xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Julien Ribon</xqdoc:author>" ], "version" : null, "encoding" : "utf-8", "namespaces" : [ { "uri" : "", "prefix" : "an" }, { "uri" : "http://www.28msec.com/modules/asynchronous-jobs", "prefix" : "job" }, { "uri" : "http://www.28msec.com/modules/parallelism", "prefix" : "parallel" }, { "uri" : "http://zorba.io/modules/random", "prefix" : "rand" }, { "uri" : "http://www.28msec.com/modules/store", "prefix" : "store" } ], "functions" : [ { "isDocumented" : true, "arity" : 1, "name" : "job-statuses", "qname" : "parallel:job-statuses", "signature" : "($id as xs:string) as object()?", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Returns the statuses of the underlying asynchronous jobs, as an object\n indicating the number of chunks in each status.</p>\n", "summary" : "<p>  Returns the statuses of the underlying asynchronous jobs, as an object\n indicating the number of chunks in each status.</p>", "annotation_str" : " %an:nondeterministic", "annotations" : [ { "prefix" : "an", "ns" : "", "name" : "nondeterministic", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "id", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The id of the parallel job to query, as it was returned by map or shuffle.</div>" } ], "returns" : { "type" : "object()?", "description" : "The statuses of the underlying jobs. The empty sequence if the parallel job does not exist." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 2, "name" : "map", "qname" : "parallel:map", "signature" : "($input as xs:string, $function as function (item()*) as item()*) as xs:string", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Schedules a parallel map job on a collection or on the output of another\n parallel job.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">The name of the output collections are randomly generated (one for\n each chunk and start with <tt>_28.temporary</tt>.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Example of usage:</p>\n parallel:map(\"input\", my:function#1)\n", "summary" : "<p>  Schedules a parallel map job on a collection or on the output of another\n parallel job.</p>", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "input", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The name of the collection to process, or the id of a parallel job of which the output will be taken.</div>" }, { "name" : "function", "type" : "function (item()*) as item()*", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the function which is run in parallel on the collection's contents.</div>" } ], "returns" : { "type" : "xs:string", "description" : "A parallel job id that can be used to query the parallel job's properties." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">job:JBDY0005 if the supplied function does not have a name.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">job:JBDY0006 if the supplied function is in the local namespace.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">job:JPDY0001 if the input is not an available collection or a parallel job id.</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 3, "name" : "map", "qname" : "parallel:map", "signature" : "($input as xs:string, $function as function (item()*) as item()*, $options as object()?) as xs:string", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Schedules a parallel map job on a collection or on the output of another\n parallel job.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">The $options parameter allows for the specification of properties for\n the parallel job. Allowed options are:\n <ul>\n   <li><tt>output-collection as string</tt>: the name of the collection in\n     which the output of the executed query is stored. If not specified,\n     the result of the query will be stored in a collection withing the\n     MongoDB database associated with the project. The name of the\n     collection is randomly generated and starts with <tt>_28.temporary</tt>\n   </li>\n   <li><tt>chunk-size</tt>: the size of the chunks the collection must be split\n   into, if it is not already sharded (default: MongoDB sharding, or 1000).\n   </li>\n </ul>\n </p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Example of usage:</p>\n parallel:map(\"input\", my:function#1, { \"chunk-size\" : xs:int(1000) })\n parallel:map(\"input\",\n              my:function#1,\n              {\n                \"chunk-size\" : xs:int(1000),\n                \"output-collection\" : output\"\n              }\n )\n", "summary" : "<p>  Schedules a parallel map job on a collection or on the output of another\n parallel job.</p>", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "input", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The name of the collection to process, or the id of a parallel job of which the output will be taken.</div>" }, { "name" : "function", "type" : "function (item()*) as item()*", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The function which is run in parallel on the collection's contents.</div>" }, { "name" : "options", "type" : "object()", "occurrence" : "?", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> An object specifying above options for the parallel job.</div>" } ], "returns" : { "type" : "xs:string", "description" : "A parallel job id that can be used to query the parallel job's properties." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">err:XPTY0004 if the type of a supplied option is incorrect.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">job:JBDY0005 if the supplied function does not have a name.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">job:JBDY0006 if the supplied function is in the local namespace.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">job:JPDY0001 if the input is not an available collection or a parallel job id.</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 1, "name" : "output-collections", "qname" : "parallel:output-collections", "signature" : "($id as xs:string) as xs:string*", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Returns the output collections of the specified parallel job.</p>\n", "summary" : "<p>  Returns the output collections of the specified parallel job.</p>", "annotation_str" : " %an:nondeterministic", "annotations" : [ { "prefix" : "an", "ns" : "", "name" : "nondeterministic", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "id", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The id of the parallel job to query, as it was returned by map or shuffle.</div>" } ], "returns" : { "type" : "xs:string*", "description" : "The names of the output collections to which the corresponding parallel job writes. The empty sequence if the job does not exist." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "results", "qname" : "parallel:results", "signature" : "($id as xs:string) as item()*", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Returns the results of the specified parallel job.</p>\n", "summary" : "<p>  Returns the results of the specified parallel job.</p>", "annotation_str" : " %an:nondeterministic", "annotations" : [ { "prefix" : "an", "ns" : "", "name" : "nondeterministic", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "id", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The id of the parallel job to query, as it was returned by map or shuffle.</div>" } ], "returns" : { "type" : "item()*", "description" : "The structured items output by the corresponding parallel job. The empty sequence if the job does not exist." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 2, "name" : "shuffle", "qname" : "parallel:shuffle", "signature" : "($input as xs:string, $function as function (item()) as xs:integer) as xs:string", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Schedules a parallel shuffle job on a collection or on the output of another\n parallel job.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">The output collection for each item is determined by calling the shuffle function,\n which gives an integer i. The i-th output collection\n is then taken. If necessary, a modulo operation is done\n to make sure that the integer is comprised between 1 and the number of output collections.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">The name of the output collections are randomly generated (10 of them) and\n start with <tt>_28.temporary</tt>.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Example of usage:</p>\n parallel:shuffle(\"input\", my:function#1)\n", "summary" : "<p>  Schedules a parallel shuffle job on a collection or on the output of another\n parallel job.</p>", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "input", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The name of the collection to process, or the id of a parallel job of which the output will be taken.</div>" }, { "name" : "function", "type" : "function (item()) as xs:integer", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the function which is run on each item to determine where it is shuffled to.</div>" } ], "returns" : { "type" : "xs:string", "description" : "A parallel job id that can be used to query the parallel job's properties." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">job:JBDY0005 if the supplied function does not have a name.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">job:JBDY0006 if the supplied function is in the local namespace.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">job:JPDY0001 if the input is not an available collection or a parallel job id.</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 3, "name" : "shuffle", "qname" : "parallel:shuffle", "signature" : "($input as item(), $function as function (item()) as xs:integer, $options as object()?) as xs:string", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Schedules a parallel shuffle job on a collection or on the output of another\n parallel job.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">The output collection for each item is determined by calling the shuffle function,\n which gives an integer i. The i-th output collection\n is then taken. If necessary, a modulo operation is done\n to make sure that the integer is comprised between 1 and the number of output collections.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">The $options parameter allows for the specification of properties for\n the parallel shuffle job. Allowed options are:\n <ul>\n   <li><tt>number-of-output-collections as integer</tt>: the number of output collections\n     to automatically generate if none are provided. The default value is 10. An error is raised\n     if the provided value is not positive.\n   </li>\n   <li><tt>output-collections as array</tt>: an array with the names of the collections in\n     which the items of the input collections are shuffled. If not specified,\n     these collections will be automatically generated (as many as specified in the\n     <tt>number-of-output-collections</tt> option) in the\n     MongoDB database associated with the project. The name of the\n     collections are randomly generated and start with <tt>_28.temporary</tt>\n   </li>\n   <li><tt>chunk-size</tt>: the size of the chunks the collection must be split\n   into, if it is not already sharded (default: MongoDB sharding, or 1000).\n   </li>\n </ul>\n </p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Example of usage:</p>\n parallel:shuffle(\"input\", my:function#1, { \"chunk-size\" : xs:int(1000) })\n parallel:shuffle(\"input\",\n                  my:function#1,\n                  {\n                    \"chunk-size\" : xs:int(1000),\n                    \"output-collections\" : [ \"output1\", \"output2\" ]\n                  }\n )\n parallel:shuffle(\"input\",\n                  my:function#1,\n                  {\n                    \"number-of-output-collections\" : 20\n                  }\n )\n", "summary" : "<p>  Schedules a parallel shuffle job on a collection or on the output of another\n parallel job.</p>", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "input", "type" : "item()", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The name of the collection to process, or the id of a parallel job of which the output will be taken.</div>" }, { "name" : "function", "type" : "function (item()) as xs:integer", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the function which is run on each item to determine where it is shuffled to.</div>" }, { "name" : "options", "type" : "object()", "occurrence" : "?", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> An object specifying above options for the parallel shuffle job.</div>" } ], "returns" : { "type" : "xs:string", "description" : "A parallel job id that can be used to query the parallel job's properties." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">err:XPTY0004 if the type of a supplied option is incorrect.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">job:JBDY0005 if the supplied function does not have a name.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">job:JBDY0006 if the supplied function is in the local namespace.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">job:JPDY0001 if the input is not an available collection or a parallel job id.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">err:FOCA0002 if number-of-output-collections is not a positive integer.</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 1, "name" : "status", "qname" : "parallel:status", "signature" : "($id as xs:string) as xs:string?", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Returns the general status of the parallel job.</p>\n", "summary" : "<p>  Returns the general status of the parallel job.</p>", "annotation_str" : " %an:nondeterministic", "annotations" : [ { "prefix" : "an", "ns" : "", "name" : "nondeterministic", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "id", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> id of the parallel job to query, as it was returned by map or shuffle.</div>" } ], "returns" : { "type" : "xs:string?", "description" : "The status of the corresponding parallel job. The empty sequence if the job does not exist." }, "errors" : [  ] } ], "variables" : [  ] }, "http://www.zorba-xquery.com/modules/image/graphviz" : { "ns" : "http://www.zorba-xquery.com/modules/image/graphviz", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Module that provides functions for generating SVG graphs.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">The module provides two function for generating graphs given in the :\n <ul><li>DOT language (see <a href=\"http://www.graphviz.org/\">\n http://www.graphviz.org</a>)</li><li>or in the XML-based Graph eXchange\n Language (see <a href=\"http://www.gupro.de/GXL/\">http://www.gupro.de/GXL/</a>),\n respectively.</li></ul>\n Both functions use the Graphviz Visualization Library in order to\n layout and render the graphs. As a result, both return a sequence\n of (document)-nodes (one for each input graph).\n These nodes are instances of the Scalable Vector Graphics (SVG) format.\n SVG is a language for describing two-dimensional graphics and\n graphical applications in XML. More information about SVG can\n be found at <a href=\"http://www.w3.org/Graphics/SVG/\">http://www.w3.org/Graphics/SVG/</a>.\n As second parameters, both functions take a sequence of strings that\n are parameters for the graph generation and rendering algorithms.\n Currently, only the empty-sequence is allowed here.\n These parameters exist for future use.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Please note that this feature is only available on Unix-based\n platforms (i.e. not on Windows).</p>\n", "sees" : [  ], "authors" : [ "<xqdoc:author xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"><a href=\"http://www.28msec.com/home/index\">28msec</a></xqdoc:author>" ], "version" : null, "encoding" : "utf-8", "namespaces" : [ { "uri" : "http://www.zorba-xquery.com/modules/image/graphviz", "prefix" : "gr" }, { "uri" : "http://zorba.io/options/versioning", "prefix" : "ver" } ], "functions" : [ { "isDocumented" : true, "arity" : 2, "name" : "dot", "qname" : "gr:dot", "signature" : "($dot as xs:string*, $params as xs:string*) as node()* external", "description" : " Layout one ore more graphs given in the DOT language and render\n them as SVG. For example,\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"><code>\n dot(\"digraph mygraph { p -&gt; q }\", ())\n </code></p>\n", "summary" : "<p> Layout one ore more graphs given in the DOT language and render\n them as SVG.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "dot", "type" : "xs:string", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> A dot description of the graph to render.</div>" }, { "name" : "params", "type" : "xs:string", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> Parameters to configure the layout and rendering process. Currently, only the empty-sequence is allowed here.</div>" } ], "returns" : { "type" : "node()*", "description" : "A graph for each item in the sequence given using the first parameter. The result sequence consists of items which are instance of the SVG data model." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 2, "name" : "gxl", "qname" : "gr:gxl", "signature" : "($gxl as node()*, $params as xs:string*) as node()* external", "description" : " Layout one ore more graphs given in the GXL language and render\n them as SVG.\n", "summary" : "<p> Layout one ore more graphs given in the GXL language and render\n them as SVG.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "gxl", "type" : "node()", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> A GXL description of the graph to render.</div>" }, { "name" : "params", "type" : "xs:string", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> Parameters to configure the layout and rendering process. Currently, only the empty-sequence is allowed here.</div>" } ], "returns" : { "type" : "node()*", "description" : "A graph for each item in the sequence given using the first parameter. The result sequence consists of items which are instance of the SVG data model." }, "errors" : [  ] } ], "variables" : [  ] }, "http://zorba.io/modules/data-cleaning/token-based-string-similarity" : { "ns" : "http://zorba.io/modules/data-cleaning/token-based-string-similarity", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">This library module provides token-based string similarity functions that view strings\n as sets or multi-sets of tokens and use set-related properties to compute similarity scores.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">The tokens correspond to groups of characters extracted from the strings being compared, such as\n individual words or character n-grams.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">These functions are particularly useful for matching near duplicate strings in cases where\n typographical conventions often lead to rearrangement of words (e.g., \"John Smith\" versus \"Smith, John\").</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">The logic contained in this module is not specific to any particular XQuery implementation,\n although the module requires the trigonometic functions of XQuery 3.0 or a math extension\n function such as sqrt($x as numeric) for computing the square root.</p>\n", "sees" : [  ], "authors" : [ "<xqdoc:author xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Bruno Martins</xqdoc:author>" ], "version" : null, "encoding" : "utf-8", "namespaces" : [ { "uri" : "http://www.w3.org/2005/xpath-functions/math", "prefix" : "math" }, { "uri" : "http://zorba.io/modules/data-cleaning/set-similarity", "prefix" : "set" }, { "uri" : "http://zorba.io/modules/data-cleaning/token-based-string-similarity", "prefix" : "simt" }, { "uri" : "http://zorba.io/options/versioning", "prefix" : "ver" } ], "functions" : [ { "isDocumented" : true, "arity" : 3, "name" : "cosine-ngrams", "qname" : "simt:cosine-ngrams", "signature" : "($s1 as xs:string, $s2 as xs:string, $n as xs:integer) as xs:double", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Returns the cosine similarity coefficient between sets of character n-grams extracted from two strings.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">The n-grams from each string are weighted according to their occurence frequency (i.e., weighted according to\n the term-frequency heuristic from Information Retrieval).</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Example usage : <pre class=\"ace-static\" ace-mode=\"xquery\"> cosine-ngrams(\"DWAYNE\", \"DUANE\", 2 ) </pre></p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">The function invocation in the example above returns : <pre class=\"ace-static\" ace-mode=\"xquery\"> 0.2401922307076307 </pre></p>\n", "summary" : "<p>  Returns the cosine similarity coefficient between sets of character n-grams extracted from two strings.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "s1", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The first string.</div>" }, { "name" : "s2", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The second string.</div>" }, { "name" : "n", "type" : "xs:integer", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The number of characters to consider when extracting n-grams.</div>" } ], "returns" : { "type" : "xs:double", "description" : "The cosine similarity coefficient between the sets n-grams extracted from the two strings." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 3, "name" : "cosine-tokens", "qname" : "simt:cosine-tokens", "signature" : "($s1 as xs:string, $s2 as xs:string, $r as xs:string) as xs:double", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Returns the cosine similarity coefficient between sets of tokens extracted from two strings. The tokens\n from each string are weighted according to their occurence frequency (i.e., weighted according to the\n term-frequency heuristic from Information Retrieval).</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Example usage : <pre class=\"ace-static\" ace-mode=\"xquery\"> cosine-tokens(\"The FLWOR Foundation\", \"FLWOR Found.\", \" +\" ) </pre></p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">The function invocation in the example above returns : <pre class=\"ace-static\" ace-mode=\"xquery\"> 0.408248290463863 </pre></p>\n", "summary" : "<p>  Returns the cosine similarity coefficient between sets of tokens extracted from two strings.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "s1", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The first string.</div>" }, { "name" : "s2", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The second string.</div>" }, { "name" : "r", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> A regular expression forming the delimiter character(s) which mark the boundaries between adjacent tokens.</div>" } ], "returns" : { "type" : "xs:double", "description" : "The cosine similarity coefficient between the sets tokens extracted from the two strings." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 2, "name" : "cosine", "qname" : "simt:cosine", "signature" : "($desc1 as xs:string*, $desc2 as xs:string*) as xs:double", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Auxiliary function for computing the cosine similarity coefficient between strings,\n using stringdescriptors based on sets of character n-grams or sets of tokens extracted from two strings.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Example usage : <pre class=\"ace-static\" ace-mode=\"xquery\"> cosine( (\"aa\",\"bb\") , (\"bb\",\"aa\")) </pre></p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">The function invocation in the example above returns : <pre class=\"ace-static\" ace-mode=\"xquery\"> 1.0 </pre></p>\n", "summary" : "<p>  Auxiliary function for computing the cosine similarity coefficient between strings,\n using stringdescriptors based on sets of character n-grams or sets of tokens extracted from two strings.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "desc1", "type" : "xs:string", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The descriptor for the first string.</div>" }, { "name" : "desc2", "type" : "xs:string", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The descriptor for the second string.</div>" } ], "returns" : { "type" : "xs:double", "description" : "The cosine similarity coefficient between the descriptors for the two strings." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 3, "name" : "dice-ngrams", "qname" : "simt:dice-ngrams", "signature" : "($s1 as xs:string, $s2 as xs:string, $n as xs:integer) as xs:double", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Returns the Dice similarity coefficient between sets of character n-grams extracted from two strings.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Example usage : <pre class=\"ace-static\" ace-mode=\"xquery\"> dice-ngrams(\"DWAYNE\", \"DUANE\", 2 ) </pre></p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">The function invocation in the example above returns : <pre class=\"ace-static\" ace-mode=\"xquery\"> 0.4615384615384616 </pre></p>\n", "summary" : "<p>  Returns the Dice similarity coefficient between sets of character n-grams extracted from two strings.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "s1", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The first string.</div>" }, { "name" : "s2", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The second string.</div>" }, { "name" : "n", "type" : "xs:integer", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The number of characters to consider when extracting n-grams.</div>" } ], "returns" : { "type" : "xs:double", "description" : "The Dice similarity coefficient between the sets of character n-grams extracted from the two strings." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 3, "name" : "dice-tokens", "qname" : "simt:dice-tokens", "signature" : "($s1 as xs:string, $s2 as xs:string, $r as xs:string) as xs:double", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Returns the Dice similarity coefficient between sets of tokens extracted from two strings.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Example usage : <pre class=\"ace-static\" ace-mode=\"xquery\"> dice-tokens(\"The FLWOR Foundation\", \"FLWOR Found.\", \" +\" ) </pre></p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">The function invocation in the example above returns : <pre class=\"ace-static\" ace-mode=\"xquery\"> 0.4 </pre></p>\n", "summary" : "<p>  Returns the Dice similarity coefficient between sets of tokens extracted from two strings.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "s1", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The first string.</div>" }, { "name" : "s2", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The second string.</div>" }, { "name" : "r", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> A regular expression forming the delimiter character(s) which mark the boundaries between adjacent tokens.</div>" } ], "returns" : { "type" : "xs:double", "description" : "The Dice similarity coefficient between the sets tokens extracted from the two strings." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 3, "name" : "jaccard-ngrams", "qname" : "simt:jaccard-ngrams", "signature" : "($s1 as xs:string, $s2 as xs:string, $n as xs:integer) as xs:double", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Returns the Jaccard similarity coefficient between sets of character n-grams extracted from two strings.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Example usage : <pre class=\"ace-static\" ace-mode=\"xquery\"> jaccard-ngrams(\"DWAYNE\", \"DUANE\", 2 ) </pre></p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">The function invocation in the example above returns : <pre class=\"ace-static\" ace-mode=\"xquery\"> 0.3 </pre></p>\n", "summary" : "<p>  Returns the Jaccard similarity coefficient between sets of character n-grams extracted from two strings.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "s1", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The first string.</div>" }, { "name" : "s2", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The second string.</div>" }, { "name" : "n", "type" : "xs:integer", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The number of characters to consider when extracting n-grams.</div>" } ], "returns" : { "type" : "xs:double", "description" : "The Jaccard similarity coefficient between the sets of character n-grams extracted from the two strings." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 3, "name" : "jaccard-tokens", "qname" : "simt:jaccard-tokens", "signature" : "($s1 as xs:string, $s2 as xs:string, $r as xs:string) as xs:double", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Returns the Jaccard similarity coefficient between sets of tokens extracted from two strings.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Example usage : <pre class=\"ace-static\" ace-mode=\"xquery\"> jaccard-tokens(\"The FLWOR Foundation\", \"FLWOR Found.\", \" +\" ) </pre></p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">The function invocation in the example above returns : <pre class=\"ace-static\" ace-mode=\"xquery\"> 0.25 </pre></p>\n", "summary" : "<p>  Returns the Jaccard similarity coefficient between sets of tokens extracted from two strings.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "s1", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The first string.</div>" }, { "name" : "s2", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The second string.</div>" }, { "name" : "r", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> A regular expression forming the delimiter character(s) which mark the boundaries between adjacent tokens.</div>" } ], "returns" : { "type" : "xs:double", "description" : "The Jaccard similarity coefficient between the sets tokens extracted from the two strings." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 2, "name" : "ngrams", "qname" : "simt:ngrams", "signature" : "($s as xs:string, $n as xs:integer) as xs:string*", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Returns the individual character n-grams forming a string.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Example usage : <pre class=\"ace-static\" ace-mode=\"xquery\"> ngrams(\"FLWOR\", 2 ) </pre></p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">The function invocation in the example above returns : <pre class=\"ace-static\" ace-mode=\"xquery\"> (\"_F\" , \"FL\" , \"LW\" , \"WO\" , \"LW\" , \"WO\" , \"OR\" , \"R_\") </pre></p>\n", "summary" : "<p>  Returns the individual character n-grams forming a string.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "s", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The input string.</div>" }, { "name" : "n", "type" : "xs:integer", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The number of characters to consider when extracting n-grams.</div>" } ], "returns" : { "type" : "xs:string*", "description" : "The sequence of strings with the extracted n-grams." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 3, "name" : "overlap-ngrams", "qname" : "simt:overlap-ngrams", "signature" : "($s1 as xs:string, $s2 as xs:string, $n as xs:integer) as xs:double", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Returns the overlap similarity coefficient between sets of character n-grams extracted from two strings.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Example usage : <pre class=\"ace-static\" ace-mode=\"xquery\"> overlap-ngrams(\"DWAYNE\", \"DUANE\", 2 ) </pre></p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">The function invocation in the example above returns : <pre class=\"ace-static\" ace-mode=\"xquery\"> 0.5 </pre></p>\n", "summary" : "<p>  Returns the overlap similarity coefficient between sets of character n-grams extracted from two strings.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "s1", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The first string.</div>" }, { "name" : "s2", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The second string.</div>" }, { "name" : "n", "type" : "xs:integer", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The number of characters to consider when extracting n-grams.</div>" } ], "returns" : { "type" : "xs:double", "description" : "The overlap similarity coefficient between the sets of character n-grams extracted from the two strings." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 3, "name" : "overlap-tokens", "qname" : "simt:overlap-tokens", "signature" : "($s1 as xs:string, $s2 as xs:string, $r as xs:string) as xs:double", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Returns the overlap similarity coefficient between sets of tokens extracted from two strings.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Example usage : <pre class=\"ace-static\" ace-mode=\"xquery\"> overlap-tokens(\"The FLWOR Foundation\", \"FLWOR Found.\", \" +\" ) </pre></p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">The function invocation in the example above returns : <pre class=\"ace-static\" ace-mode=\"xquery\"> 0.5 </pre></p>\n", "summary" : "<p>  Returns the overlap similarity coefficient between sets of tokens extracted from two strings.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "s1", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The first string.</div>" }, { "name" : "s2", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The second string.</div>" }, { "name" : "r", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> A regular expression forming the delimiter character(s) which mark the boundaries between adjacent tokens.</div>" } ], "returns" : { "type" : "xs:double", "description" : "The overlap similarity coefficient between the sets tokens extracted from the two strings." }, "errors" : [  ] } ], "variables" : [  ] }, "http://www.zorba-xquery.com/modules/image/paint" : { "ns" : "http://www.zorba-xquery.com/modules/image/paint", "description" : " This module provides a function to extend an image with additional shapes.\n Fully supported image formats are:\n <ul xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">\n   <li>GIF</li>\n   <li>JPEG</li>\n   <li>PNG</li>\n   <li>TIFF</li>\n   <li>BMP</li>\n </ul>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">The errors raised by functions of this module have the namespace\n <tt>http://www.zorba-xquery.com/modules/image/error</tt> (associated with prefix ierr).</p>\n", "sees" : [  ], "authors" : [ "<xqdoc:author xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Daniel Thomas</xqdoc:author>" ], "version" : null, "encoding" : "utf-8", "namespaces" : [ { "uri" : "http://www.w3.org/2005/xqt-errors", "prefix" : "err" }, { "uri" : "http://www.zorba-xquery.com/modules/image/error", "prefix" : "ierr" }, { "uri" : "http://www.zorba-xquery.com/modules/image/image", "prefix" : "img" }, { "uri" : "http://www.zorba-xquery.com/modules/image/paint", "prefix" : "paint" }, { "uri" : "http://zorba.io/options/versioning", "prefix" : "ver" } ], "functions" : [ { "isDocumented" : true, "arity" : 2, "name" : "paint", "qname" : "paint:paint", "signature" : "($image as xs:base64Binary, $shapes as element(*)*) as xs:base64Binary", "description" : " Extends the passed image with a sequence of shapes.\n The shapes are passed as a sequence of elements.\n The possibilities for shape elements are:\n    <ul xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">\n      <li> line:\n        <pre class=\"brush: xml\">\n          &lt;img:line&gt;\n            &lt;img:start&gt;&lt;img:x&gt;-20&lt;/img:x&gt;&lt;img:y&gt;-20&lt;/img:y&gt;&lt;/img:start&gt;\n            &lt;img:end&gt;&lt;img:x&gt;80&lt;/img:x&gt;&lt;img:y&gt;80&lt;/img:y&gt;&lt;/img:end&gt;\n          &lt;/img:line&gt;</pre>\n      </li>\n       <li> polyline:\n         <pre class=\"brush: xml\">\n           &lt;img:polyLine&gt;\n             &lt;img:point&gt;&lt;img:x&gt;10&lt;/img:x&gt;&lt;img:y&gt;10&lt;/img:y&gt;&lt;/img:point&gt;\n             &lt;img:point&gt;&lt;img:x&gt;40&lt;/img:x&gt;&lt;img:y&gt;80&lt;/img:y&gt;&lt;/point&gt;\n             &lt;img:point&gt;&lt;img:x&gt;50&lt;/img:x&gt;&lt;img:y&gt;30&lt;/img:y&gt;&lt;/point&gt;\n             &lt;img:point&gt;&lt;img:x&gt;200&lt;/img:x&gt;&lt;img:y&gt;200&lt;/img:y&gt;&lt;/point&gt;\n           &lt;/img:polyLine&gt;\n         </pre>\n       </li>\n       <li> stroked polyline:\n         <pre class=\"brush: xml\">\n           &lt;img:strokedPolyLine&gt;\n             &lt;img:point&gt;&lt;img:x&gt;10&lt;/img:x&gt;&lt;img:y&gt;10&lt;/img:y&gt;&lt;/img:point&gt;\n             &lt;img:point&gt;&lt;img:x&gt;40&lt;/img:x&gt;&lt;img:y&gt;80&lt;/img:y&gt;&lt;/img:point&gt;\n             &lt;img:point&gt;&lt;img:x&gt;50&lt;/img:x&gt;&lt;img:y&gt;30&lt;/img:y&gt;&lt;/img:point&gt;\n             &lt;img:strokeLength&gt;5&lt;/img:strokeLength&gt;&lt;img:gapLength&gt;2&lt;/img:gapLength&gt;\n           &lt;/img:strokedPolyLine&gt;\n         </pre>\n       </li>\n       <li> rectangle:\n         <pre class=\"brush: xml\">\n           &lt;img:rectangle&gt;\n             &lt;img:upperLeft&gt;&lt;img:x&gt;20&lt;/img:x&gt;&lt;img:y&gt;20&lt;/img:y&gt;&lt;/img:upperLeft&gt;\n             &lt;img:lowerRight&gt;&lt;img:x&gt;50&lt;/img:x&gt;&lt;img:y&gt;50&lt;/img:y&gt;&lt;/img:lowerRight&gt;\n           &lt;/img:rectangle&gt;\n         </pre>\n       </li>\n       <li> rounded rectangle:\n         <pre class=\"brush: xml\">\n           &lt;img:roundedRectangle&gt;\n             &lt;img:upperLeft&gt;&lt;img:x&gt;20&lt;/img:x&gt;&lt;img:y&gt;20&lt;/img:y&gt;&lt;/img:upperLeft&gt;\n             &lt;img:lowerRight&gt;&lt;img:x&gt;50&lt;/img:x&gt;&lt;img:y&gt;50&lt;/img:y&gt;&lt;/img:lowerRight&gt;\n             &lt;img:cornerWidth&gt;10&lt;/img:cornerWidth&gt;&lt;img:cornerHeight&gt;10&lt;/img:cornerHeight&gt;\n           &lt;/img:roundedRectangle&gt;\n         </pre>\n       </li>\n       <li> circle:\n         <pre class=\"brush: xml\">\n           &lt;img:circle&gt;\n             &lt;img:origin&gt;&lt;img:x&gt;20&lt;/img:x&gt;&lt;img:y&gt;20&lt;/img:y&gt;&lt;/img:origin&gt;\n             &lt;img:perimeter&gt;5&lt;/img:perimeter&gt;\n           &lt;/img:circle&gt;\n         </pre>\n       </li>\n       <li> ellipse:\n         <pre class=\"brush: xml\">\n           &lt;img:ellipse&gt;\n             &lt;img:origin&gt;&lt;img:x&gt;50&lt;/img:x&gt;&lt;img:y&gt;50&lt;/img:y&gt;&lt;/img:origin&gt;\n             &lt;img:perimeterX&gt;30&lt;/img:perimeterX&gt;&lt;img:perimeterY&gt;20&lt;/img:perimeterY&gt;\n           &lt;/img:ellipse&gt;\n         </pre>\n       </li>\n       <li> arc:\n         <pre class=\"brush: xml\">\n           &lt;img:arc&gt;\n             &lt;img:origin&gt;&lt;img:x&gt;50&lt;/img:x&gt;&lt;img:y&gt;50&lt;/img:y&gt;&lt;/img:origin&gt;\n             &lt;img:perimeterX&gt;10&lt;/img:perimeterX&gt;&lt;img:perimeterY&gt;20&lt;/img:perimeterY&gt;\n             &lt;img:startDegrees&gt;180&lt;/img:startDegrees&gt;&lt;img:endDegrees&gt;270&lt;/img:endDegrees&gt;\n           &lt;/img:arc&gt;\n         </pre>\n       </li>\n       <li> polygon:\n         <pre class=\"brush: xml\">\n           &lt;img:polygon&gt;\n             &lt;img:point&gt;&lt;img:x&gt;10&lt;/img:x&gt;&lt;img:y&gt;10&lt;/img:y&gt;&lt;/img:point&gt;\n             &lt;img:point&gt;&lt;img:x&gt;40&lt;/img:x&gt;&lt;img:y&gt;80&lt;/img:y&gt;&lt;/img:point&gt;\n             &lt;img:point&gt;&lt;img:x&gt;50&lt;/img:x&gt;&lt;img:y&gt;30&lt;/img:y&gt;&lt;/img:point&gt;\n           &lt;/img:polygon&gt;\n         </pre>\n       </li>\n       <li> text:\n         <pre class=\"brush: xml\">\n           &lt;img:text&gt;\n             &lt;img:origin&gt;&lt;img:x&gt;20&lt;/img:x&gt;&lt;img:y&gt;20&lt;/img:y&gt;&lt;/img:origin&gt;\n             &lt;img:text&gt;Hello Zorba&lt;/img:text&gt;&lt;img:font&gt;&lt;/img:font&gt;&lt;img:font-size&gt;12&lt;/img:font-size&gt;\n           &lt;/img:text&gt;\n         </pre>\n       </li>\n     </ul>\n Optionally, each of the shape elements can contain elements to define the stroke with, stroke color, fill color, and anti-aliasing.\n E.g.:\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">\n   <pre class=\"brush: xml\">\n     &lt;img:rectangle&gt;\n       &lt;img:strokeWidth&gt;5&lt;/img:strokeWidth&gt;\n       &lt;img:strokeColor&gt;#00AF00&lt;/img:strokeColor&gt;\n       &lt;img:fillColor&gt;#A10000&lt;/img:fillColor&gt;\n       &lt;img:antiAliasing&gt;true&lt;/img:antiAliasing&gt;\n       &lt;img:upperLeft&gt;&lt;img:x&gt;20&lt;/img:x&gt;&lt;img:y&gt;20&lt;/img:y&gt;&lt;/img:upperLeft&gt;\n       &lt;img:lowerRight&gt;&lt;img:x&gt;50&lt;/img:x&gt;&lt;img:y&gt;50&lt;/img:y&gt;&lt;/img:lowerRight&gt;\n     &lt;/img:rectangle&gt;\n   </pre>\n  </p>\n", "summary" : "<p> Extends the passed image with a sequence of shapes.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "image", "type" : "xs:base64Binary", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the passed image</div>" }, { "name" : "shapes", "type" : "element(*)", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the shapes</div>" } ], "returns" : { "type" : "xs:base64Binary", "description" : "image with additional shapes" }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">ierr:IM001 the passed image is invalid.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">err:FORG0001 one of the passed shape elements is invalid.</xqdoc:error>" ] } ], "variables" : [  ] }, "http://zorba.io/modules/fetch" : { "ns" : "http://zorba.io/modules/fetch", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">This module provides functions to fetch the content of a resource identified\n by a URI. For example, it fetches the content of file or http resources.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">In order to retrieve such content, the functions use the\n URI resolution and URL resolver process as documented at\n <a href=\"../zorba/uriresolvers.html\">\n URI Resolvers</a>.</p>\n", "sees" : [  ], "authors" : [ "<xqdoc:author xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Matthias Brantner</xqdoc:author>" ], "version" : null, "encoding" : "utf-8", "namespaces" : [ { "uri" : "http://zorba.io/annotations", "prefix" : "an" }, { "uri" : "http://zorba.io/modules/fetch", "prefix" : "fetch" }, { "uri" : "http://zorba.io/options/versioning", "prefix" : "ver" }, { "uri" : "http://zorba.io/errors", "prefix" : "zerr" } ], "functions" : [ { "isDocumented" : true, "arity" : 1, "name" : "content-binary", "qname" : "fetch:content-binary", "signature" : "($uri as xs:string) as xs:base64Binary", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Tries to fetch the resource referred to by the given URI and\n returning it as base64Binary.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">It queries all URI mappers and resolvers with kind\n <tt>EntityData::SOME_CONTENT</tt>.</p>\n", "summary" : "<p>  Tries to fetch the resource referred to by the given URI and\n returning it as base64Binary.</p>", "annotation_str" : " %an:streamable", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "streamable", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "uri", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the resource to fetch.</div>" } ], "returns" : { "type" : "xs:base64Binary", "description" : "the resource referred to by the given URI as streamble base64Binary." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">uri:URI_UNRESOLVED_OR_NOSTREAM if the URI could not be resolved or did not resolve to a <tt>StreamResource</tt>.</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 2, "name" : "content-binary", "qname" : "fetch:content-binary", "signature" : "($uri as xs:string, $entity-kind as xs:string) as xs:base64Binary external", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Tries to fetch the resource referred to by the given URI and\n returning it as base64Binary.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">It queries all URI mappers and resolvers with kind the specified\n entity kind.</p>\n", "summary" : "<p>  Tries to fetch the resource referred to by the given URI and\n returning it as base64Binary.</p>", "annotation_str" : " %an:streamable", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "streamable", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "uri", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the resource to fetch.</div>" }, { "name" : "entity-kind", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the kind of resource to fetch.</div>" } ], "returns" : { "type" : "xs:base64Binary", "description" : "the resource referred to by the given URI as streamble base64Binary." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">uri:URI_UNRESOLVED_OR_NOSTREAM if the URI could not be resolved or did not resolve to a <tt>StreamResource</tt>.</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 1, "name" : "content", "qname" : "fetch:content", "signature" : "($uri as xs:string) as xs:string", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Tries to fetch the resource referred to by the given URI.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">It queries all URI mappers and resolvers with kind\n <tt>EntityData::SOME_CONTENT</tt>.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">The content is assumed to be UTF-8 encoded.</p>\n", "summary" : "<p>  Tries to fetch the resource referred to by the given URI.</p>", "annotation_str" : " %an:streamable", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "streamable", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "uri", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the resource to fetch.</div>" } ], "returns" : { "type" : "xs:string", "description" : "the resource referred to by the given URI as streamble string." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">uri:URI_UNRESOLVED_OR_NOSTREAM if the URI could not be resolved or did not resolve to a <tt>StreamResource</tt>.</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 2, "name" : "content", "qname" : "fetch:content", "signature" : "($uri as xs:string, $entity-kind as xs:string) as xs:string", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Tries to fetch the resource referred to by the given URI.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">It queries all URI mappers and resolvers with kind the specified\n entity kind.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">The content is assumed to be UTF-8 encoded.</p>\n", "summary" : "<p>  Tries to fetch the resource referred to by the given URI.</p>", "annotation_str" : " %an:streamable", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "streamable", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "uri", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the resource to fetch.</div>" }, { "name" : "entity-kind", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the kind of resource to fetch.</div>" } ], "returns" : { "type" : "xs:string", "description" : "the resource referred to by the given URI as streamble string." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">uri:URI_NOT_RESOLVED_OR_NOSTREAM if the URI could not be resolved or did not resolve to a <tt>StreamResource</tt>.</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 3, "name" : "content", "qname" : "fetch:content", "signature" : "($uri as xs:string, $entity-kind as xs:string, $encoding as xs:string) as xs:string external", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Tries to fetch the resource referred to by the given URI.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">It queries all URI mappers and resolvers with kind the specified\n entity kind.</p>\n", "summary" : "<p>  Tries to fetch the resource referred to by the given URI.</p>", "annotation_str" : " %an:streamable", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "streamable", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "uri", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the resource to fetch.</div>" }, { "name" : "entity-kind", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the kind of resource to fetch.</div>" }, { "name" : "encoding", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the encoding of the content</div>" } ], "returns" : { "type" : "xs:string", "description" : "the resource referred to by the given URI as streamble string." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">uri:URI_UNRESOLVED_OR_NOSTREAM if the URI could not be resolved or did not resolve to a <tt>StreamResource</tt>.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">uri:CHARSET_UNKNOWN if the given encoding is invalid or not supported.</xqdoc:error>" ] } ], "variables" : [  ] }, "http://xbrl.io/modules/bizql/concept-maps" : { "ns" : "http://xbrl.io/modules/bizql/concept-maps", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">This module provides functions for storing, retrieving, and modifying\n concept maps. Concept maps can be used in BizQL queries.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">A concept map is a network of concepts that can be contained in a report schema.\n It maps \"virtual\" concepts to reported concepts, which allows comparison across\n archives that use different vocabularies (so-called extension facts).</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">With this module, you can retrieve the concept map associated with each report\n schema. You can also query for facts by making implicitly use of the mapping.</p>\n", "sees" : [  ], "authors" : [ "<xqdoc:author xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Charles Hoffman</xqdoc:author>", "<xqdoc:author xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Matthias Brantner</xqdoc:author>", "<xqdoc:author xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Dennis Knochenwefel</xqdoc:author>", "<xqdoc:author xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Ghislain Fourny</xqdoc:author>" ], "version" : null, "encoding" : "utf-8", "namespaces" : [ { "uri" : "http://xbrl.io/modules/bizql/archives", "prefix" : "archives" }, { "uri" : "http://xbrl.io/modules/bizql/concept-maps", "prefix" : "concept-maps" }, { "uri" : "http://xbrl.io/modules/bizql/facts", "prefix" : "facts" }, { "uri" : "http://xbrl.io/modules/bizql/networks", "prefix" : "networks" }, { "uri" : "http://xbrl.io/modules/bizql/report-schemas", "prefix" : "report-schemas" }, { "uri" : "http://zorba.io/options/versioning", "prefix" : "ver" } ], "functions" : [ { "isDocumented" : true, "arity" : 0, "name" : "concept-maps", "qname" : "concept-maps:concept-maps", "signature" : "() as object()*", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Retrieves all concept maps.</p>\n", "summary" : "<p>  Retrieves all concept maps.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [  ], "returns" : { "type" : "object()*", "description" : "all concept maps." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "concept-maps", "qname" : "concept-maps:concept-maps", "signature" : "($report-schemas-or-ids as item()*) as object()*", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Return the concept maps from the given report schemas.</p>\n", "summary" : "<p>  Return the concept maps from the given report schemas.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "report-schemas-or-ids", "type" : "item()", "occurrence" : "*", "description" : "" } ], "returns" : { "type" : "object()*", "description" : "the concept maps from the report schemas." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 3, "name" : "facts-for-archives-and-concepts", "qname" : "concept-maps:facts-for-archives-and-concepts", "signature" : "($archive-or-ids as item()*, $concepts as string*, $concept-maps as object()*) as object()*", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Resolves the supplied concepts according to the supplied concept map.</p>\n", "summary" : "<p>  Resolves the supplied concepts according to the supplied concept map.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "archive-or-ids", "type" : "item()", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> a sequence of archives or AIDs (or $concept-maps:ALL_OF_THEM to do no filter on archives).</div>" }, { "name" : "concepts", "type" : "string", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> a sequence of concept names (or $concept-maps:ALL_OF_THEM to do no filter on concepts).</div>" }, { "name" : "concept-maps", "type" : "object()", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> a sequence of concept maps.</div>" } ], "returns" : { "type" : "object()*", "description" : "the resolved facts, with the concept name overriden with the new name." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 4, "name" : "facts-for-archives-and-concepts", "qname" : "concept-maps:facts-for-archives-and-concepts", "signature" : "($archive-or-ids as item()*, $concepts as string*, $concept-maps as object()+, $options as object()?) as object()*", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Resolves the supplied concepts according to the supplied concept map.</p>\n", "summary" : "<p>  Resolves the supplied concepts according to the supplied concept map.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "archive-or-ids", "type" : "item()", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> a sequence of archives or AIDs (or $concept-maps:ALL_OF_THEM to do no filter on archives).</div>" }, { "name" : "concepts", "type" : "string", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> a sequence of concept names (or $concept-maps:ALL_OF_THEM to do no filter on concepts).</div>" }, { "name" : "concept-maps", "type" : "object()", "occurrence" : "+", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> a sequence of concept maps.</div>" }, { "name" : "options", "type" : "object()", "occurrence" : "?", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> <a href=\"facts#standard_options\">standard fact retrieving options</a>.</div>" } ], "returns" : { "type" : "object()*", "description" : "the resolved facts, with the concept name overriden with the new name." }, "errors" : [  ] } ], "variables" : [ { "name" : "concept-maps:ALL_OF_THEM", "type" : "boolean", "description" : " Joker for all archives or all concepts.\n" } ] }, "http://www.28msec.com/modules/http/cookie" : { "ns" : "http://www.28msec.com/modules/http/cookie", "description" : " The Sausalito Cookie module provides function to manipulate\n HTTP-Cookies.\n", "sees" : [  ], "authors" : [ "<xqdoc:author xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">28msec</xqdoc:author>" ], "version" : null, "encoding" : "utf-8", "namespaces" : [ { "uri" : "http://zorba.io/annotations", "prefix" : "an" }, { "uri" : "http://www.28msec.com/modules/http/cookie", "prefix" : "cookie" }, { "uri" : "http://www.28msec.com/modules/http/cookie", "prefix" : "cookie-schema" }, { "uri" : "http://www.28msec.com/modules/http/request", "prefix" : "req" }, { "uri" : "http://www.28msec.com/modules/http/response", "prefix" : "resp" }, { "uri" : "http://zorba.io/modules/schema", "prefix" : "schema" }, { "uri" : "http://www.zorba-xquery.com/modules/cryptography/hmac", "prefix" : "sec" }, { "uri" : "http://zorba.io/modules/uri", "prefix" : "uri" }, { "uri" : "http://zorba.io/options/versioning", "prefix" : "ver" } ], "functions" : [ { "isDocumented" : true, "arity" : 1, "name" : "create-session", "qname" : "cookie:create-session", "signature" : "($sessionData as node()) as empty-sequence()", "description" : " Creates a client-side cookie named \"_session\", containing the\n <tt xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">$sessionData</tt> node value together with its signature.\n The signature is generated with the server's private key.\n", "summary" : "<p> Creates a client-side cookie named \"_session\", containing the\n  $sessionData  node value together with its signature.</p>", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "sessionData", "type" : "node()", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The data to be used for the session.</div>" } ], "returns" : { "type" : "empty-sequence()", "description" : "The empty-sequence is returned." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 2, "name" : "create-session", "qname" : "cookie:create-session", "signature" : "($sessionData as node(), $expires as xs:dateTime) as empty-sequence()", "description" : " Creates a client-side cookie named \"_session\", containing the\n <tt xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">$sessionData</tt> node value together with its signature.\n The signature is generated with the server's private key.\n The cookie is also set to expire on the date specified by\n <tt xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">$expires</tt>.\n", "summary" : "<p> Creates a client-side cookie named \"_session\", containing the\n  $sessionData  node value together with its signature.</p>", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "sessionData", "type" : "node()", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The data to be used for the session.</div>" }, { "name" : "expires", "type" : "xs:dateTime", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The expiration time of the cookie.</div>" } ], "returns" : { "type" : "empty-sequence()", "description" : "The empty-sequence is returned." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 3, "name" : "create-session", "qname" : "cookie:create-session", "signature" : "($sessionData as node(), $expires as xs:dateTime?, $path as xs:string?) as empty-sequence()", "description" : " Creates a client-side cookie named \"_session\", containing the\n <tt xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">$sessionData</tt> node value together with its signature.\n The signature is generated with the server's private key.\n The cookie is also set to expire on the date specified by\n <tt xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">$expires</tt>. The cookie is only valid for the specified path.\n", "summary" : "<p> Creates a client-side cookie named \"_session\", containing the\n  $sessionData  node value together with its signature.</p>", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "sessionData", "type" : "node()", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The data to be used for the session.</div>" }, { "name" : "expires", "type" : "xs:dateTime", "occurrence" : "?", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The expiration time of the cookie.</div>" }, { "name" : "path", "type" : "xs:string", "occurrence" : "?", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The path of the URL for which the cookie is valid.</div>" } ], "returns" : { "type" : "empty-sequence()", "description" : "The empty-sequence is returned." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 0, "name" : "delete-session", "qname" : "cookie:delete-session", "signature" : "() as empty-sequence()", "description" : " Delete the \"_session\" cookie from the client. The next\n user request will not receive a valid session cookie\n anymore\n", "summary" : "<p> Delete the \"_session\" cookie from the client.</p>", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [  ], "returns" : { "type" : "empty-sequence()", "description" : "The empty-sequence is returned." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "delete-session", "qname" : "cookie:delete-session", "signature" : "($path as xs:string?) as empty-sequence()", "description" : " Delete the \"_session\" cookie from the client. The next\n user request will not receive a valid session cookie\n anymore\n", "summary" : "<p> Delete the \"_session\" cookie from the client.</p>", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "path", "type" : "xs:string", "occurrence" : "?", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The path of the URL for which the cookie is valid.</div>" } ], "returns" : { "type" : "empty-sequence()", "description" : "The empty-sequence is returned." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "get", "qname" : "cookie:get", "signature" : "($name as xs:string?) as element(cookie:cookie)*", "description" : " Returns the cookie in the request having the given name or the empty sequence\n if no such cookie exists. The cookie format returned is:\n <tt xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">\n &lt;cookie:cookie\n   name=\"..\"&gt;...&lt;/cookie:cookie&gt;</tt>.\n The content of the cookie element is the value of the cookie.\n", "summary" : "<p> Returns the cookie in the request having the given name or the empty sequence\n if no such cookie exists.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "name", "type" : "xs:string", "occurrence" : "?", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The name of the cookie that should be retrieved.</div>" } ], "returns" : { "type" : "element(cookie:cookie)*", "description" : "The cookie with the given name or the empty sequence if no cookie with the given name exist" }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cookie:S003 if $decode evaluates to fn:true() and the value of the cookie cannot be parsed.</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 2, "name" : "get", "qname" : "cookie:get", "signature" : "($name as xs:string?, $decode as xs:boolean) as element(cookie:cookie)*", "description" : " Returns the cookies in the request having the given name. If no name is given,\n it returns all available cookies. If no cookie value decoding is wanted,\n the user must specify a second argument that evaluates to fn:false().\n", "summary" : "<p> Returns the cookies in the request having the given name.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "name", "type" : "xs:string", "occurrence" : "?", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The name of the cookie that should be retrieved</div>" }, { "name" : "decode", "type" : "xs:boolean", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> Boolean to specify if cookie value decoding id needed</div>" } ], "returns" : { "type" : "element(cookie:cookie)*", "description" : "The cookie with the given name or the empty sequence if no cookie with the given name exist" }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cookie:S003 if $decode evaluates to fn:true() and the value of the cookie cannot be parsed.</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 0, "name" : "session-data", "qname" : "cookie:session-data", "signature" : "() as node()", "description" : " Reads the \"_session\" cookie in the request and verifies if the signature\n matches the ad-hoc computed signature of the content of the session. If the\n verification is successful, the function returns the session data. Otherwise,\n an error is raised.\n", "summary" : "<p> Reads the \"_session\" cookie in the request and verifies if the signature\n matches the ad-hoc computed signature of the content of the session.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [  ], "returns" : { "type" : "node()", "description" : "The session data is returned if verification succesful." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "set", "qname" : "cookie:set", "signature" : "($cookie as element(cookie:cookie)) as empty-sequence()", "description" : " Adds a <tt xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Set-Cookie</tt> header to the response. This function is equivalent to\n <tt xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cookie:set($cookie, fn:true())</tt>.\n", "summary" : "<p> Adds a  Set-Cookie  header to the response.</p>", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "cookie", "type" : "element(cookie:cookie)", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The XML representation of the cookie to be sent to the client.</div>" } ], "returns" : { "type" : "empty-sequence()", "description" : "The empty-sequence is returned." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cookie:S002 if the cookie is bigger than 4096 bytes.</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 2, "name" : "set", "qname" : "cookie:set", "signature" : "($cookie as element(cookie:cookie), $encode as xs:boolean) as empty-sequence()", "description" : " Adds a <tt xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Set-Cookie</tt> header to the response.\n The cookie parameter needs to adhere to the cookie schema. For example,\n <tt xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">\n &lt;cookie:cookie\n   name=\"xs:string\"\n   expires=\"xs:datetime\"\n   domain=\"xs:string\"\n   path=\"xs:string\"\n   secure=\"xs:boolean\"&gt;value&lt;/cookie:cookie&gt;\n </tt>.\n If the cookie parameter is not valid according to the schema, err:XQDY0027 is raised.\n If $encode evaluates to fn:true(), then the child nodes of the cookie element will be\n serialized using XML serialization (omitting the XML declaration). In order not to\n break the cookie syntax, any occurence of whitespace or column in the value will be\n encoded on the client using the URL encoding format. For this reason also the '%'\n character will be encoded using the same format.\n If $encode evaluates to fn:false(), the children nodes of the cookie element will be\n serialized as text and no encoding will be performed whatsoever. It is the\n responsibility of the caller to make sure the text serialization will not produce an\n invalid cookie.\n", "summary" : "<p> Adds a  Set-Cookie  header to the response.</p>", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "cookie", "type" : "element(cookie:cookie)", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The cookie to send to the client</div>" }, { "name" : "encode", "type" : "xs:boolean", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> if encoding should be performed on the value or not</div>" } ], "returns" : { "type" : "empty-sequence()", "description" : "The empty-sequence is returned." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cookie:S002 if the cookie is bigger than 4096 bytes.</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 0, "name" : "validate-session", "qname" : "cookie:validate-session", "signature" : "() as xs:boolean", "description" : " Reads the \"_session\" cookie in the request and verifies if\n the signature matches the ad-hoc computed signature of the\n content of the session. The signature is generated with a\n server private key. If no cookie named \"_session\" exists,\n this function returns false.\n", "summary" : "<p> Reads the \"_session\" cookie in the request and verifies if\n the signature matches the ad-hoc computed signature of the\n content of the session.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [  ], "returns" : { "type" : "xs:boolean", "description" : "Returns <tt xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">true</tt> if the session is valid, and and false otherwise." }, "errors" : [  ] } ], "variables" : [  ] }, "http://www.28msec.com/modules/couchbase" : { "ns" : "http://www.28msec.com/modules/couchbase", "description" : " This module provides functionality to interact with the\n Couchbase NoSQL database.<p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n The module is built using the libcouchbase 2.0 C client library and\n exposes most of its functionality in JSONiq.\n Beyond just allowing for basic key-value store operations (e.g.\n put-/get-text or put-/get-binary, this module also allows to work\n with Couchbase views in order to allow for complex JSON query\n operations.\n", "sees" : [  ], "authors" : [ "<xqdoc:author xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Cristi Dumitru</xqdoc:author>" ], "version" : null, "encoding" : "utf-8", "namespaces" : [ { "uri" : "http://zorba.io/annotations", "prefix" : "an" }, { "uri" : "http://www.28msec.com/modules/couchbase", "prefix" : "cb" }, { "uri" : "http://zorba.io/options/versioning", "prefix" : "ver" }, { "uri" : "http://www.zorba-xquery.com/modules/couchbase", "prefix" : "zcb" } ], "functions" : [ { "isDocumented" : true, "arity" : 1, "name" : "connect", "qname" : "cb:connect", "signature" : "($options as object()) as anyURI", "description" : " Connect to the Couchbase server.<p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">The input to the function is an object that contains the connection\n information. Specifically, it allows for the following options:\n <ul>\n   <li><tt>host</tt>: the endpoint of the Couchbase server (mandatory)</li>\n   <li><tt>user</tt>: the user used for connecting (optional)</li>\n   <li><tt>password</tt>: the password used for connecting (optional)</li>\n   <li><tt>bucket</tt>: name of an existing bucket (mandatory)</li>\n </ul></p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Example:\n <ul><li><pre>\n $conn := cb:connect({\n   \"host\" : \"192.168.1.56:8091\",\n   \"username\" : (),\n   \"password\" : (),\n   \"bucket\" : \"default\"\n });\n </pre></li></ul></p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">The function returns an opaque URI that represents the connection.\n This URI has to be passed to other functions of this module that require\n the <tt>$conn</tt> parameter as a first argument.</p>\n", "summary" : "<p> Connect to the Couchbase server.</p>", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "options", "type" : "object()", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> a object that contains the host, bucket, and authentication information.</div>" } ], "returns" : { "type" : "anyURI", "description" : "an identifier for the established connection." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">LCB0001 if the connection to the given host/bucket could not be established.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">CB0001 if mandatory connection information is missing.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">CB0007 if a given option is not supported.</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 4, "name" : "connect", "qname" : "cb:connect", "signature" : "($host as string, $username as string?, $password as string?, $bucket as string) as anyURI", "description" : " Connect to the Couchbase server.<p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">The function returns an opaque URI that represents the connection.\n This URI has to be passed to other functions of this module that require\n the <tt>$conn</tt> parameter as a first argument.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Example:\n <ul><li><pre>\n $conn := cb:connect(\"192.168.1.56:8091\", (), (), \"default\");\n </pre></li></ul></p>\n", "summary" : "<p> Connect to the Couchbase server.</p>", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "host", "type" : "string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> address of the couchbase server (mandatory) <code>\"192.168.1.56:8091\"</code></div>" }, { "name" : "username", "type" : "string", "occurrence" : "?", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> username used for the connection</div>" }, { "name" : "password", "type" : "string", "occurrence" : "?", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> password used for the connection</div>" }, { "name" : "bucket", "type" : "string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> name of the bucket to use (mandatory) <code>\"default\"</code></div>" } ], "returns" : { "type" : "anyURI", "description" : "an identifier that represents the connection to the server." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">LCB0001 if the connection to the given host/bucket could not be established.</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 3, "name" : "create-view", "qname" : "cb:create-view", "signature" : "($conn as anyURI, $doc-name as string, $view-names as string*) as string*", "description" : " Create a document and views.<p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n If the document already exists, it is replaced.\n A document can hold several views that must be specified in the same call.\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Example:\n <ul><li><pre>\n $views := cb:create-view($conn, \"zip\", (\"bystate\", \"bycity\"));\n </pre></li></ul></p>\n", "summary" : "<p> Create a document and views.</p>", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "conn", "type" : "anyURI", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The identifier of the active connection</div>" }, { "name" : "doc-name", "type" : "string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The name of the document to create</div>" }, { "name" : "view-names", "type" : "string", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The names of the views to create in the document</div>" } ], "returns" : { "type" : "string*", "description" : "The paths for the views that have been created." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">LCB0002 if any error occurs in the communication with the server.</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 4, "name" : "create-view", "qname" : "cb:create-view", "signature" : "($conn as anyURI, $doc-name as string, $view-names as string*, $options as object()*) as string*", "description" : " Create a document and views.<p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n If the document already exists, it is replaced.\n A document can hold several views that must be specified in the same call.\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">In addition, the function allows to specify several options:\n <ul>\n   <li><tt>key</tt>: <tt>string</tt> name of the value that will be used\n        as key in the view</li>\n   <li><tt>values</tt>: <tt>string/array</tt> name of the value(s) that\n        will be used as values in the view</li>\n   <li><tt>function</tt>: <tt>javascript function</tt> the map function\n        that will create the connection between key and value.<br/>\n        If <tt>function</tt> is specified, the <tt>key</tt> and\n        <tt>values</tt> options are ignored.</li>\n </ul></p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Example:\n <ul><li><pre>\n $views := cb:create-view($conn, \"zip\", \"bystate\", { \"key\": \"doc.state\", \"values\" : [\"doc.pop\", \"doc.city\"] });\n </pre></li>\n <li><pre>\n $views := cb:create-view($conn, \"zip\", \"bycity\", { \"function\": \"function(doc, meta) {  emit(doc.state, null); }\"});\n </pre></li></ul></p>\n", "summary" : "<p> Create a document and views.</p>", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "conn", "type" : "anyURI", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The identifier of the active connection</div>" }, { "name" : "doc-name", "type" : "string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The name of the document to create</div>" }, { "name" : "view-names", "type" : "string", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The names of the views to create in the document</div>" }, { "name" : "options", "type" : "object()", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> An object with additional options</div>" } ], "returns" : { "type" : "string*", "description" : "The paths for the views that have been created." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">LCB0002 if any error occurs in the communication with the server.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">CB0005 if the number of options doesn't match the number of view-names.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">CB0007 if any of the options is not supported.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">CB0010 if any of the given options has an invalid type.</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 2, "name" : "delete-view", "qname" : "cb:delete-view", "signature" : "($conn as anyURI, $doc as string*) as string*", "description" : " Delete a document and its views.<p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n If the document doesn't exists, the function does nothing.\n All the views hold in the document are deleted.\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Example:\n <ul><li><pre>\n cb:delete-view($conn, \"zip\");\n </pre></li></ul></p>\n", "summary" : "<p> Delete a document and its views.</p>", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "conn", "type" : "anyURI", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The identifier of the active connection</div>" }, { "name" : "doc", "type" : "string", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">-name The name of the document to delete</div>" } ], "returns" : { "type" : "string*", "description" : "a empty sequence." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">LCB0002 if any error occurs in the communication with the server.</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 1, "name" : "flush", "qname" : "cb:flush", "signature" : "($conn as anyURI) as empty-sequence()", "description" : " Remove all values and their keys.<p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Example:\n <ul><li><pre>\n cb:flush($conn);\n </pre></li></ul></p>\n", "summary" : "<p> Remove all values and their keys.</p>", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "conn", "type" : "anyURI", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The identifier of the active connection</div>" } ], "returns" : { "type" : "empty-sequence()", "description" : "a empty sequence." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">LCB0002 if any error occurs in the communication with the server.</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 2, "name" : "get-binary", "qname" : "cb:get-binary", "signature" : "($conn as anyURI, $key as string*) as base64Binary*", "description" : " Retrieve the values matching the given string keys, as base64Binary.<p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Example:\n <ul><li><pre>\n $result := cb:get-binary($conn, \"image\");\n </pre></li></ul></p>\n", "summary" : "<p> Retrieve the values matching the given string keys, as base64Binary.</p>", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "conn", "type" : "anyURI", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The identifier of the active connection</div>" }, { "name" : "key", "type" : "string", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The requested keys</div>" } ], "returns" : { "type" : "base64Binary*", "description" : "A sequence of base64Binary values corresponding to the keys" }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">LCB0002 if any error occurs in the communication with the server.</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 3, "name" : "get-binary", "qname" : "cb:get-binary", "signature" : "($conn as anyURI, $key as string*, $options as object()) as base64Binary*", "description" : " Retrieve the values matching the given string keys, as base64Binary.<p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">In addition, the function allows to specify several options:\n <ul>\n   <li><tt>expiration-time</tt>: <tt>integer</tt> value for refreshing the\n        expiration time in seconds\n       (default 0, which means values are kept indefinitely)</li>\n </ul></p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Example:\n <ul><li><pre>\n $result := cb:get-binary($conn, \"image\",{ \"expiration-time\" : 60 * 60 });\n </pre></li></ul></p>\n", "summary" : "<p> Retrieve the values matching the given string keys, as base64Binary.</p>", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "conn", "type" : "anyURI", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The identifier of the active connection</div>" }, { "name" : "key", "type" : "string", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The requested keys</div>" }, { "name" : "options", "type" : "object()", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> An object with additional options</div>" } ], "returns" : { "type" : "base64Binary*", "description" : "A sequence of base64Binary values corresponding to the keys" }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">LCB0002 if any error occurs in the communication with the server.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">CB0007 if any of the options is not supported.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">CB0009 if the given expiration time is not an integer.</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 2, "name" : "get-text", "qname" : "cb:get-text", "signature" : "($conn as anyURI, $key as string*) as string*", "description" : " Retrieve the values matching the given string keys, as string.<p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Example:\n <ul><li><pre>\n $result := cb:get-text($conn, \"35040\");\n </pre></li></ul></p>\n", "summary" : "<p> Retrieve the values matching the given string keys, as string.</p>", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "conn", "type" : "anyURI", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The identifier of the active connection</div>" }, { "name" : "key", "type" : "string", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The requested keys</div>" } ], "returns" : { "type" : "string*", "description" : "A sequence of string values corresponding to the keys" }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">LCB0002 if any error occurs in the communication with the server.</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 3, "name" : "get-text", "qname" : "cb:get-text", "signature" : "($conn as anyURI, $key as string*, $options as object()) as string*", "description" : " Retrieve the values matching the given string keys, as string.<p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">In addition, the function allows to specify several options:\n <ul>\n   <li><tt>expiration-time</tt>: <tt>integer</tt> value for refreshing the\n        expiration time in seconds\n       (default 0, which means values are kept indefinitely)</li>\n   <li><tt>encoding</tt>: <tt>string</tt> name of the encoding of the\n        returned string (default UTF-8)</li>\n </ul></p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Example:\n <ul><li><pre>\n $result := cb:get-text($conn, \"35040\", { \"encoding\" : \"ASCII\" } );\n </pre></li></ul></p>\n", "summary" : "<p> Retrieve the values matching the given string keys, as string.</p>", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "conn", "type" : "anyURI", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The identifier of the active connection</div>" }, { "name" : "key", "type" : "string", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The requested key</div>" }, { "name" : "options", "type" : "object()", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> An object with additional options</div>" } ], "returns" : { "type" : "string*", "description" : "A sequence of string values corresponding to the keys" }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">LCB0002 if any error occurs in the communication with the server.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">CB0006 if the given encoding is not supported.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">CB0007 if any of the options is not supported.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">CB0009 if the given expiration time is not an integer.</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 3, "name" : "put-binary", "qname" : "cb:put-binary", "signature" : "($conn as anyURI, $key as string*, $value as base64Binary*) as empty-sequence()", "description" : " Store base64binary values along with their string keys.<p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Example:\n <ul><li><pre>\n cb:put-binary($conn, \"image\",\n   http:get-binary(\"http://localhost:8091/images/couchbase_logo.png\")[2]);\n </pre></li></ul></p>\n", "summary" : "<p> Store base64binary values along with their string keys.</p>", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "conn", "type" : "anyURI", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The identifier of the active connection</div>" }, { "name" : "key", "type" : "string", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The keys to store</div>" }, { "name" : "value", "type" : "base64Binary", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The base64 binary values to be stored</div>" } ], "returns" : { "type" : "empty-sequence()", "description" : "a empty sequence." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">LCB0002 if any error occurs in the communication with the server.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">CB0005 if the number of keys doesn't match the number of values.</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 4, "name" : "put-binary", "qname" : "cb:put-binary", "signature" : "($conn as anyURI, $key as string*, $value as base64Binary*, $options as object()) as empty-sequence()", "description" : " Store base64Binary values along with their string keys.<p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">In addition, the function allows to specify several options:\n <ul>\n   <li><tt>expiration-time</tt>: <tt>integer</tt> value for refreshing the\n        expiration time in seconds\n       (default 0, which means values are kept indefinitely)</li>\n   <li><tt>operation</tt>: <tt>add/replace/set/append/prepend</tt> type of\n       operation</li>\n   <li><tt>wait</tt>: <tt>persist/false</tt> if the system should wait for\n        persistence of the keys</li>\n </ul></p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Example:\n <ul><li><pre>\n cb:put-binary($conn, \"image\",\n   http:get-binary(\"http://localhost:8091/images/couchbase_logo.png\")[2],\n   { \"expiration-time\" : 60 * 60 * 24 });\n </pre></li></ul></p>\n", "summary" : "<p> Store base64Binary values along with their string keys.</p>", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "conn", "type" : "anyURI", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The identifier of the active connection</div>" }, { "name" : "key", "type" : "string", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The keys to store</div>" }, { "name" : "value", "type" : "base64Binary", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The base64Binary values to be stored</div>" }, { "name" : "options", "type" : "object()", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> An object with additional options</div>" } ], "returns" : { "type" : "empty-sequence()", "description" : "a empty sequence." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">LCB0002 if any error occurs in the communication with the server.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">CB0005 if the number of keys doesn't match the number of values.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">CB0007 if any of the options is not supported.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">CB0009 if the given expiration time is not an integer.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">CB0011 if the stored Variable was not stored</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 3, "name" : "put-text", "qname" : "cb:put-text", "signature" : "($conn as anyURI, $key as string*, $value as string*) as empty-sequence()", "description" : " Store string values along with their string keys.<p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n The values are stored with the UTF-8 encoding and a default\n expiration time 0.\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Example:\n <ul><li><pre>\n cb:put-text($conn, \"35040\", fn:serialize({\n                      \"city\" : \"CALERA\",\n                      \"loc\" : [ -86.755987, 33.1098 ],\n                      \"pop\" : 4675,\n                      \"state\" : \"AL\",\n                      \"_id\" : \"35040\"\n                    }));\n </pre></li></ul></p>\n", "summary" : "<p> Store string values along with their string keys.</p>", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "conn", "type" : "anyURI", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The identifier of the active connection</div>" }, { "name" : "key", "type" : "string", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The keys to store</div>" }, { "name" : "value", "type" : "string", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The string values to be stored.</div>" } ], "returns" : { "type" : "empty-sequence()", "description" : "a empty sequence." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">LCB0002 if any error occurs in the communication with the server.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">CB0005 if the number of keys doesn't match the number of values.</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 4, "name" : "put-text", "qname" : "cb:put-text", "signature" : "($conn as anyURI, $key as string*, $value as string*, $options as object()) as empty-sequence()", "description" : " Store string values along with their string keys.<p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">In addition, the function allows to specify several options:\n <ul>\n   <li><tt>expiration-time</tt>: <tt>integer</tt> value for refreshing the\n        expiration time in seconds\n       (default 0, which means values are kept indefinitely)</li>\n   <li><tt>encoding</tt>: <tt>string</tt> name of the encoding of the\n        returned string (default UTF-8)</li>\n   <li><tt>operation</tt>: <tt>add/replace/set/append/prepend</tt> type\n        of operation</li>\n   <li><tt>wait</tt>: <tt>persist/false</tt> if the system should wait for\n        persistence of the keys</li>\n </ul></p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Example:\n <ul><li><pre>\n cb:put-text($conn, \"35040\", fn:serialize({\n                      \"city\" : \"CALERA\",\n                      \"loc\" : [ -86.755987, 33.1098 ],\n                      \"pop\" : 4675,\n                      \"state\" : \"AL\",\n                      \"_id\" : \"35040\"\n                    }), { \"expiration-time\" : 60 * 60 * 24 });\n </pre></li></ul></p>\n", "summary" : "<p> Store string values along with their string keys.</p>", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "conn", "type" : "anyURI", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The identifier of the active connection</div>" }, { "name" : "key", "type" : "string", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The keys to store</div>" }, { "name" : "value", "type" : "string", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The string values to be stored</div>" }, { "name" : "options", "type" : "object()", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> An object with additional options</div>" } ], "returns" : { "type" : "empty-sequence()", "description" : "a empty sequence." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">LCB0002 if any error occurs in the communication with the server.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">CB0005 if the number of keys doesn't match the number of values.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">CB0006 if the given encoding is not supported.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">CB0007 if any of the options is not supported.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">CB0009 if the given expiration time is not an integer.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">CB0011 if the stored Variable was not stored</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 2, "name" : "remove", "qname" : "cb:remove", "signature" : "($conn as anyURI, $key as string*) as empty-sequence()", "description" : " Remove the values matching the given string keys.<p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Example:\n <ul><li><pre>\n cb:remove($conn, \"35040\");\n </pre></li></ul></p>\n", "summary" : "<p> Remove the values matching the given string keys.</p>", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "conn", "type" : "anyURI", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The identifier of the active connection</div>" }, { "name" : "key", "type" : "string", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The requested keys to have their values removed</div>" } ], "returns" : { "type" : "empty-sequence()", "description" : "a empty sequence." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">LCB0002 if any error occurs in the communication with the server.</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 3, "name" : "touch", "qname" : "cb:touch", "signature" : "($conn as anyURI, $key as string*, $exp-time as integer) as empty-sequence()", "description" : " Refresh the expiration time of the given string keys.<p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Example:\n <ul><li><pre>\n cb:touch($conn, \"35040\", 60 * 60 * 24);\n </pre></li></ul></p>\n", "summary" : "<p> Refresh the expiration time of the given string keys.</p>", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "conn", "type" : "anyURI", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The identifier of the active connection</div>" }, { "name" : "key", "type" : "string", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The keys to touch</div>" }, { "name" : "exp-time", "type" : "integer", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> New expiration time in seconds</div>" } ], "returns" : { "type" : "empty-sequence()", "description" : "a empty sequence." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">LCB0002 if any error occurs in the communication with the server.</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 2, "name" : "view", "qname" : "cb:view", "signature" : "($conn as anyURI, $path as string*) as object()*", "description" : " Retrieve the content of existing views. <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Example:\n <ul><li><pre>\n $results := cb:view($conn, \"_design/zip/_view/bystate\");\n </pre></li></ul></p>\n", "summary" : "<p> Retrieve the content of existing views.</p>", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "conn", "type" : "anyURI", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The identifier of the active connection</div>" }, { "name" : "path", "type" : "string", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The view path</div>" } ], "returns" : { "type" : "object()*", "description" : "information about the view" }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">LCB0002 if any error occurs in the communication with the server.</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 3, "name" : "view", "qname" : "cb:view", "signature" : "($conn as anyURI, $path as string*, $options as object()) as object()*", "description" : " Retrieve the content of existing views.<p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Example:\n <ul><li><pre>\n $results := cb:view($conn, \"_design/zip/_view/bystate\", { \"limit\": 100 });\n </pre></li></ul></p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">In addition, the function allows to specify several options:\n <ul>\n   <li><tt>encoding</tt>: <tt>string</tt> name of the encoding of the\n        returned string (default UTF-8)</li>\n   <li><tt>stale</tt>: if the system should wait for persistence of the keys\n    <ul>\n      <li><tt>ok</tt> the view is not updated</li>\n      <li><tt>false</tt> the view is updated before the function view is\n      executed, this options needs the key to be on disk before the call of\n      the function.</li>\n      <li><tt>update_after</tt> the view is updated after the call\n      of the function.</li>\n    </ul></li>\n   <li><tt>limit</tt>: <tt>integer</tt> how many rows qill be shown\n         (default all)</li>\n </ul></p>\n", "summary" : "<p> Retrieve the content of existing views.</p>", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "conn", "type" : "anyURI", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The identifier of the active connection</div>" }, { "name" : "path", "type" : "string", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The view path <code>\"_design/test/_view/vies\"</code></div>" }, { "name" : "options", "type" : "object()", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> An object with additional options</div>" } ], "returns" : { "type" : "object()*", "description" : "information about the view" }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">LCB0002 if any error occurs in the communication with the server.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">CB0007 if any of the options is not supported.</xqdoc:error>" ] } ], "variables" : [  ] }, "http://zorba.io/modules/random" : { "ns" : "http://zorba.io/modules/random", "description" : " This module provides several functions for generating (pseudo-)random\n numbers and strings.\n", "sees" : [  ], "authors" : [ "<xqdoc:author xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Matthias Brantner, Sorin Nasoi</xqdoc:author>" ], "version" : null, "encoding" : "utf-8", "namespaces" : [ { "uri" : "http://zorba.io/annotations", "prefix" : "an" }, { "uri" : "http://zorba.io/modules/random", "prefix" : "r" }, { "uri" : "http://zorba.io/options/versioning", "prefix" : "ver" }, { "uri" : "http://zorba.io/errors", "prefix" : "zerr" } ], "functions" : [ { "isDocumented" : true, "arity" : 2, "name" : "random-between", "qname" : "r:random-between", "signature" : "($lower as integer, $upper as integer) as integer", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">This function generates one random number within a given range.\n The function is nondeterministic.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">The function is based on <tt>r:random-between#3</tt>.\n Specifically, it returns the value of invoking\n <tt>r:random-betwen($lower, $upper, 1)</tt>.</p>\n", "summary" : "<p>  This function generates one random number within a given range.</p>", "annotation_str" : " %an:nondeterministic", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "nondeterministic", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "lower", "type" : "integer", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the lower bound for the random number</div>" }, { "name" : "upper", "type" : "integer", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the upper bound for the random number</div>" } ], "returns" : { "type" : "integer", "description" : "a random integer within the given range" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 3, "name" : "random-between", "qname" : "r:random-between", "signature" : "($lower as integer, $upper as integer, $num as integer) as integer*", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">This function generates an arbitrary number of random numbers\n within a given range. The function is nondeterministic because\n the sequence is <b>not</b> repeatable.</p>\n", "summary" : "<p>  This function generates an arbitrary number of random numbers\n within a given range.</p>", "annotation_str" : " %an:nondeterministic", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "nondeterministic", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "lower", "type" : "integer", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the lower bound for every value within the sequence</div>" }, { "name" : "upper", "type" : "integer", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the upper bound for every value within the sequence</div>" }, { "name" : "num", "type" : "integer", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the length of the sequence returned</div>" } ], "returns" : { "type" : "integer*", "description" : "<tt xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">$num</tt> pseudo-random integers within (and including) the range specified by <tt xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">$lower</tt> and <tt xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">$upper</tt>. It returns <tt xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">$num</tt>-times <tt xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">$lower</tt> if <tt xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">$lower</tt> is equal to <tt xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">$upper</tt> and the empty sequence if <tt xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">$num</tt> is negative." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">r:INVALID_ARGUMENT if <tt>$lower</tt> is greater than <tt>$upper</tt></xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 0, "name" : "random", "qname" : "r:random", "signature" : "() as integer", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">This function generates one random number.\n The function is nondeterministic.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">The function is based on <tt>r:random#1</tt>. Specifically, it\n returns the value of invoking <tt>r:random(1)</tt>.</p>\n", "summary" : "<p>  This function generates one random number.</p>", "annotation_str" : " %an:nondeterministic", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "nondeterministic", "value" : "" } ], "updating" : false, "parameters" : [  ], "returns" : { "type" : "integer", "description" : "a random integer" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "random", "qname" : "r:random", "signature" : "($num as integer) as integer* external", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">This function generates an arbitrary number of random numbers.\n The function is nondeterministic because the sequence is\n <b>not</b> repeatable.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">However, the function is based on posix function <tt>srand()</tt> and\n <tt>rand()</tt>. Specifically, it invokes <tt>srand()</tt>\n with some random number based on the current time\n and then returns the values returned by invoking\n <tt>rand()</tt> <tt>$num</tt>-times.</p>\n", "summary" : "<p>  This function generates an arbitrary number of random numbers.</p>", "annotation_str" : " %an:nondeterministic", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "nondeterministic", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "num", "type" : "integer", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the length of the sequence returned</div>" } ], "returns" : { "type" : "integer*", "description" : "<tt xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">$num</tt> random integers, or the empty sequence if <tt xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">$num</tt> is negative." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 4, "name" : "seeded-random-between", "qname" : "r:seeded-random-between", "signature" : "($seed as integer, $lower as integer, $upper as integer, $num as integer) as integer*", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">This function generates an arbitrary number of pseudo-random numbers\n within a given range. The sequence is repeatable by calling the\n function with the same seed and boundaries.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">The function is based on the function <tt>r:seeded-random#2</tt>.\n Specifically, its result is repeatable if called with the\n same arguments.</p>\n", "summary" : "<p>  This function generates an arbitrary number of pseudo-random numbers\n within a given range.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "seed", "type" : "integer", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the initial seed value for the sequence</div>" }, { "name" : "lower", "type" : "integer", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the lower bound for every value within the sequence</div>" }, { "name" : "upper", "type" : "integer", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the upper bound for every value within the sequence</div>" }, { "name" : "num", "type" : "integer", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the length of the sequence returned</div>" } ], "returns" : { "type" : "integer*", "description" : "<tt xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">$num</tt> pseudo-random integers within (and including) the range specified by <tt xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">$lower</tt> and <tt xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">$upper</tt>. It returns <tt xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">$num</tt>-times <tt xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">$lower</tt> if <tt xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">$lower</tt> is equal to <tt xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">$upper</tt> and the empty sequence if <tt xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">$num</tt> is negative." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">zerr:ZQXD0004 if the given seed is negative or great than the max value of <tt>unsigned int</tt> on the given platform.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">r:INVALID_ARGUMENT if <tt>$lower</tt> is greater than <tt>$upper</tt></xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 2, "name" : "seeded-random", "qname" : "r:seeded-random", "signature" : "($seed as integer, $num as integer) as integer* external", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">This function generates an arbitrary number of pseudo-random numbers.\n The sequence is repeatable by calling the function with the same\n seed.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">The function is based on posix function <tt>srand()</tt> and\n <tt>rand()</tt>. Specifically, it invokes <tt>srand($seed)</tt>\n and then returns the values returned by invoking <tt>rand()</tt>\n <tt>$num</tt>-times.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Sequences returned by this function are not thread-safe (i.e.\n if multiple XQuery programs invoking this function are executed\n concurrently in several threads). This is because the function is\n based on <tt>srand()</tt> and <tt>rand()</tt>.</p>\n", "summary" : "<p>  This function generates an arbitrary number of pseudo-random numbers.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "seed", "type" : "integer", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the initial seed value for the sequence</div>" }, { "name" : "num", "type" : "integer", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the length of the sequence returned</div>" } ], "returns" : { "type" : "integer*", "description" : "<tt xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">$num</tt> pseudo-random integers, or the empty sequence if <tt xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">$num</tt> is negative." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">zerr:ZQXD0004 if the given seed is negative or great than the max value of <tt>unsigned int</tt> on the given platform.</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 0, "name" : "uuid", "qname" : "r:uuid", "signature" : "() as string external", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">This function returns a uuid.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> Note, that the function is not stable,\n that is, it returns a different UUID everytime the function is invoked.</p>\n", "summary" : "<p>  This function returns a uuid.</p>", "annotation_str" : " %an:nondeterministic", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "nondeterministic", "value" : "" } ], "updating" : false, "parameters" : [  ], "returns" : { "type" : "string", "description" : "the generated UUID as string" }, "errors" : [  ] } ], "variables" : [  ] }, "http://www.zorba-xquery.com/modules/http-client" : { "ns" : "http://www.zorba-xquery.com/modules/http-client", "description" : " <h1 xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Introduction</h1>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">\n This module provides provides simple functions for performing HTTP requests\n (GET, POST, DELETE etc.), as well as a more flexible general\n purpose function (<a href=\"?anchor=send-request-3\">send-request()</a>).\n </p>\n <h1 xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Examples of how to use this module</h1>\n <h4 xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Simple GET Request</h4>\n <pre xmlns:xqdoc=\"http://www.xqdoc.org/1.0\" class=\"ace-static\" ace-mode=\"xquery\">\n import module namespace http=\"http://www.zorba-xquery.com/modules/http-client\";\n declare namespace svg=\"http://www.w3.org/2000/svg\";\n http:get(\"http://www.w3.org/Graphics/SVG/svglogo.svg\")[2]/svg:svg/svg:title\n </pre>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">\n This example downloads an XML resource from the web (in this case,\n an SVG file, which is an XML-based image format) and returns it as\n a document node. Since the XML is in a namespace, we declare that\n namespace; we can then perform a path expression directly on the\n return value of http:get().\n </p>\n <h4 xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Simple GET Request (retrieving XHTML)</h4>\n   <pre xmlns:xqdoc=\"http://www.xqdoc.org/1.0\" class=\"ace-static\" ace-mode=\"xquery\">\n   import module namespace http=\"http://www.zorba-xquery.com/modules/http-client\";\n   declare namespace xhtml=\"http://www.w3.org/1999/xhtml\";\n   http:get-node( \"http://www.w3.org\" )[2]//xhtml:body\n   </pre>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">\n This example shows how to retrieve an XHTML resource. XHTML is\n XML, so the http:get-node() function will return it as a document node\n and you can operate on it with the full power of XQuery. As above, since this\n XML is in a particular namespace, the above query defines that namespace\n with the prefix \"xhtml\" so it can easily perform path expressions, etc.\n </p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">\n Note: many webservers, include www.w3.org, return XHTML with the\n HTTP Content-Type \"text/html\". Zorba cannot assume that \"text/html\"\n is actually XHTML, and so http:get() would have returned raw text\n rather than a document node. That is why the example above uses\n http:get-node(), which overrides the server's Content-Type and tells\n Zorba to attempt to parse the result as XML.\n </p>\n <h4 xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Simple GET Request (retrieving HTML as text)</h4>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">\n Note that XQuery does <b>not</b> understand plain HTML, and so if the URL\n you retrieve contains plain HTML data (not XHTML), it will be treated as\n plain text as shown in the next example. If you want to operate on the HTML\n with XQuery, you should use the HTML language module which can transform\n HTML to XHTML. The HTML module is supported by the Zorba team, but it is\n not a \"core module\", meaning that it is not shipped with every Zorba\n installation and may not be available. See\n <a href=\"http://www.zorba-xquery.com/site2/html/downloads.html\">the Zorba downloads\n page</a> for information about obtaining this module if you do not\n have it.</p>\n <pre xmlns:xqdoc=\"http://www.xqdoc.org/1.0\" class=\"ace-static\" ace-mode=\"xquery\">\n import module namespace http=\"http://www.zorba-xquery.com/modules/http-client\";\n http:get(\"http://www.example.com\")[2]\n </pre>\n returns\n   <pre xmlns:xqdoc=\"http://www.xqdoc.org/1.0\" class=\"ace-static\" ace-mode=\"xml\">\n   &lt;!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.01 Transitional//EN\"&gt;\n   &lt;html&gt;\n     &lt;head&gt;\n       &lt;meta http-equiv=\"Content-Type\"\n       content=\"text/html; charset=utf-8\" /&gt;\n       &lt;title&gt;Example Web Page&lt;/title&gt;\n     &lt;/head&gt;\n     &lt;body&gt;\n       &lt;p&gt;You have reached this web page by typing \"example.com\",\n       \"example.net\", or \"example.org\" into your web browser.&lt;/p&gt;\n       &lt;p&gt;These domain names are reserved for use in documentation and are\n       Not available for registration. See\n       &lt;a href=\"http://www.rfc-editor.org/rfc/rfc2606.txt\"&gt;RFC 2606&lt;/a&gt;,\n       Section 3.&lt;/p&gt;\n     &lt;/body&gt;\n   &lt;/html&gt;\n   </pre>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Note that the response data above is a simple\n xs:string value containing the HTML data, not actual XML data. If you\n executed the above query using the Zorba command-line client, you would\n have actually seen data like the following:</p>\n <pre xmlns:xqdoc=\"http://www.xqdoc.org/1.0\" class=\"ace-static\" ace-mode=\"xml\">\n   &amp;lt;!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.01 Transitional//EN\"&amp;gt;\n   &amp;lt;html&amp;gt;\n      ...\n </pre>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">because Zorba would attempt to serialize it as XML data, and would\n escape all the raw angle brackets in the original xs:string.</p>\n <h4 xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Simple POST Request</h4>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">\n Here is a simple example which sends text content by making an HTTP POST\n request.\n </p>\n <pre xmlns:xqdoc=\"http://www.xqdoc.org/1.0\" class=\"ace-static\" ace-mode=\"xquery\">\n import module namespace http=\"http://www.zorba-xquery.com/modules/http-client\";\n http:post( \"...\", \"Hello World\" )\n </pre>\n <h1 xmlns:xqdoc=\"http://www.xqdoc.org/1.0\" id=\"standard_return\">Return Values</h1>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Most functions in this module (all except\n <a href=\"?anchor=options-1\">options()</a>) return one or more items.\n (<a href=\"?anchor=head-1\">head()</a> returns exactly one.) For all of these,\n the first item returned will be a &lt;http-schema:response&gt;\n element, as seen in the examples above. This element has \"status\" and\n \"message\" attributes, representing the result of the HTTP call. It\n also has any number of &lt;http-schema:header&gt; child elements that\n encode the HTTP headers returned by the HTTP server. Finally, it\n will generally contain a &lt;http-schema:body&gt; child element with\n a \"media-type\" attribute that identifies the content-type of the\n result data.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">The full schema of this &lt;http-schema:response&gt; element is\n part of the <a href=\"http://expath.org/modules/http-client/\">EXPath\n HTTP Client module</a>. You can see the schema\n <a href=\"schemas/expath.org_ns_http-client.html\">here</a>.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Any items in function return values after the initial\n &lt;http-schema:response&gt; element are the body/bodies of the HTTP\n response from the server. (MIME Multi-part responses will have\n more than one body.) The type of these items depends on the\n Content-Type for each body. Each item will be:</p>\n <ul xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">\n  <li>\n    an element node, if the returned content type is one of:\n    <ul>\n      <li>text/xml</li>\n      <li>application/xml</li>\n      <li>text/xml-external-parsed-entity</li>\n      <li>application/xml-external-parsed-entity</li>\n      <li>or if the Content-Type ends with \"+xml\".</li>\n    </ul>\n  </li>\n  <li>\n    an xs:string, if the returned content type starts with \"text/\"\n    and does not match the above XML content types strings, or if\n    it is one of:\n    <ul>\n      <li>\"application/json\"</li>\n      <li>\"application/x-javascript\"</li>\n    </ul>\n  </li>\n  <li>xs:base64Binary for all other content types.</li>\n </ul>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">This return value - a sequence of items comprising one\n &lt;http-schema:response&gt; element followed by zero or more\n response items - is referred to as the \"standard http-client\n return type\" in the function declarations below.</p>\n <h1 xmlns:xqdoc=\"http://www.xqdoc.org/1.0\" id=\"url_string\">$href Arguments to Functions</h1>\n All functions in this module accept a URL argument named $href. In\n all cases, the value passed to $href must be a valid xs:anyURI.\n However, all functions declare $href to be of type xs:string. This\n is for convenience, since you can pass a string literal value (that\n is, a URL in double-quotes spelled out explicitly in your query)\n to an xs:string parameter.\n <h1 xmlns:xqdoc=\"http://www.xqdoc.org/1.0\" id=\"get_warning\">Important Notice Regarding get() Functions</h1>\n All of the get() functions in this module -\n <a xmlns:xqdoc=\"http://www.xqdoc.org/1.0\" href=\"?anchor=get-1\">get()</a>, <a xmlns:xqdoc=\"http://www.xqdoc.org/1.0\" href=\"?anchor=get-node-1\">get-node()</a>,\n <a xmlns:xqdoc=\"http://www.xqdoc.org/1.0\" href=\"?anchor=get-text-1\">get-text()</a>, and\n <a xmlns:xqdoc=\"http://www.xqdoc.org/1.0\" href=\"?anchor=get-binary()\">get-binary()</a> - are declared to be\n <i xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">nondeterministic</i>, which means that Zorba will not cache\n their results. However, they are <b xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">not</b> declared to be\n <i xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">sequential</i>, which means that Zorba may re-order them\n as part of its query optimization. According to the HTTP RFC,\n GET requests should only return data, and should not have any\n side-effects. However, in practice it is not uncommon for GET\n requests to have side-effects. If your application depends on\n the ordering of side-effects from making GET requests, you should\n either use the more complex <a xmlns:xqdoc=\"http://www.xqdoc.org/1.0\" href=\"?anchor=send-request-3\">send-request()</a>\n function (which <b xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">is</b> declared <i xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">sequential</i>), or alterately\n wrap each call to get() in your own sequential function, to ensure\n that Zorba does not place the GET requests out of order.\n <h1 xmlns:xqdoc=\"http://www.xqdoc.org/1.0\" id=\"expath_relation\">Relation to the EXPath http-client module</h1>\n <a xmlns:xqdoc=\"http://www.xqdoc.org/1.0\" href=\"http://expath.org/\">EXPath</a> defines its own http-client\n module, which is available separately for Zorba as a non-core module.\n There are two primary differences between EXPath's http-client and\n Zorba's core http-client (this module):\n <ol xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">\n   <li>EXPath defines only the send-request() function, although it\n does include convenient 1- and 2-argument forms in addition to the\n full 3-argument form. EXPath does not include the simpler get(),\n post(), put(), delete(), head(), and options() functions defined by\n this module.</li>\n   <li>EXPath specifies that all HTML content returned from the\n HTTP server will be <i>tidied up</i> into valid XML, and then parsed\n into an element. As this required an additional third-party library\n dependency, Zorba's http-client module does not perform this tidying.\n Instead, HTML content is returned as a string (with special XML\n characters replaced with XML entity references, as shown in the\n above examples).</li>\n </ol>\n See <a xmlns:xqdoc=\"http://www.xqdoc.org/1.0\" href=\"http://www.expath.org/spec/http-client\">the full spec\n of the EXPath http-client module</a> for more information.\n", "sees" : [ "<xqdoc:see xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"><a href=\"http://www.w3.org/TR/xquery-11/#FunctionDeclns\">XQuery 1.1: Function Declaration</a></xqdoc:see>" ], "authors" : [ "<xqdoc:author xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Markus Pilman, Federico Cavalieri</xqdoc:author>" ], "version" : null, "encoding" : "utf-8", "namespaces" : [ { "uri" : "http://zorba.io/annotations", "prefix" : "an" }, { "uri" : "http://www.w3.org/2005/xqt-errors", "prefix" : "err" }, { "uri" : "http://expath.org/ns/error", "prefix" : "error" }, { "uri" : "http://www.zorba-xquery.com/modules/http-client", "prefix" : "http" }, { "uri" : "http://expath.org/ns/http-client", "prefix" : "http-schema" }, { "uri" : "http://zorba.io/modules/http-client-wrapper", "prefix" : "http-wrapper" }, { "uri" : "http://jsoniq.org/functions", "prefix" : "jn" }, { "uri" : "http://zorba.io/modules/http-client", "prefix" : "json-http" }, { "uri" : "http://jsoniq.org/function-library", "prefix" : "libjn" }, { "uri" : "http://www.w3.org/2010/xslt-xquery-serialization", "prefix" : "ser" }, { "uri" : "http://zorba.io/options/versioning", "prefix" : "ver" } ], "functions" : [ { "isDocumented" : true, "arity" : 1, "name" : "delete", "qname" : "http:delete", "signature" : "($href as xs:string) as item()+", "description" : " This function makes an HTTP DELETE request to a given URL.\n", "summary" : "<p> This function makes an HTTP DELETE request to a given URL.</p>", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "href", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The URL to which the request will be made (see <a href=\"#url_string\">note</a> above).</div>" } ], "returns" : { "type" : "item()+", "description" : "<a xmlns:xqdoc=\"http://www.xqdoc.org/1.0\" href=\"#standard_return\">standard http-client return type</a>." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">error:HC001 An HTTP error occurred.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">error:HC002 Error parsing the response content as XML.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">error:HC006 A timeout occurred waiting for the response.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">error:HCV02 Trying to follow a redirect of a DELETE request.</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 1, "name" : "get-binary", "qname" : "http:get-binary", "signature" : "($href as xs:string) as item()+", "description" : " This function makes a GET request on a given URL. All returned bodies\n are forced to be interpreted as binary data, and will be returned\n as xs:base64Binary items.\n", "summary" : "<p> This function makes a GET request on a given URL.</p>", "annotation_str" : " %an:nondeterministic", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "nondeterministic", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "href", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The URL to which the request will be made (see <a href=\"#url_string\">note</a> above).</div>" } ], "returns" : { "type" : "item()+", "description" : "<a xmlns:xqdoc=\"http://www.xqdoc.org/1.0\" href=\"#standard_return\">standard http-client return type</a>." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">error:HC001 An HTTP error occurred.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">error:HC002 Error parsing the response content as XML.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">error:HC006 A timeout occurred waiting for the response.</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 1, "name" : "get-node", "qname" : "http:get-node", "signature" : "($href as xs:string) as item()+", "description" : " This function makes a GET request to a given URL. All returned bodies\n are forced to be interpreted as XML and parsed into elements.\n", "summary" : "<p> This function makes a GET request to a given URL.</p>", "annotation_str" : " %an:nondeterministic", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "nondeterministic", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "href", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The URL to which the request will be made (see <a href=\"#url_string\">note</a> above).</div>" } ], "returns" : { "type" : "item()+", "description" : "<a xmlns:xqdoc=\"http://www.xqdoc.org/1.0\" href=\"#standard_return\">standard http-client return type</a>." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">error:HC001 An HTTP error occurred.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">error:HC002 Error parsing the response content as XML.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">error:HC006 A timeout occurred waiting for the response.</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 1, "name" : "get-text", "qname" : "http:get-text", "signature" : "($href as xs:string) as item()+", "description" : " This function makes a GET request to a given URL. All returned bodies\n are forced to be interpreted as plain strings, and will be returned\n as xs:string items.\n", "summary" : "<p> This function makes a GET request to a given URL.</p>", "annotation_str" : " %an:nondeterministic", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "nondeterministic", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "href", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The URL to which the request will be made (see <a href=\"#url_string\">note</a> above).</div>" } ], "returns" : { "type" : "item()+", "description" : "<a xmlns:xqdoc=\"http://www.xqdoc.org/1.0\" href=\"#standard_return\">standard http-client return type</a>." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">error:HC001 An HTTP error occurred.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">error:HC002 Error parsing the response content as XML.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">error:HC006 A timeout occurred waiting for the response.</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 1, "name" : "get", "qname" : "http:get", "signature" : "($href as xs:string) as item()+", "description" : " This function makes a GET request to a given URL.\n", "summary" : "<p> This function makes a GET request to a given URL.</p>", "annotation_str" : " %an:nondeterministic", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "nondeterministic", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "href", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The URL to which the request will be made (see <a href=\"#url_string\">note</a> above).</div>" } ], "returns" : { "type" : "item()+", "description" : "<a xmlns:xqdoc=\"http://www.xqdoc.org/1.0\" href=\"#standard_return\">standard http-client return type</a>." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">error:HC001 An HTTP error occurred.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">error:HC002 Error parsing the response content as XML.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">error:HC006 A timeout occurred waiting for the response.</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 1, "name" : "head", "qname" : "http:head", "signature" : "($href as xs:string) as item()", "description" : " This function makes an HTTP HEAD request on a given URL.\n", "summary" : "<p> This function makes an HTTP HEAD request on a given URL.</p>", "annotation_str" : " %an:nondeterministic", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "nondeterministic", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "href", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The URL to which the request will be made (see <a href=\"#url_string\">note</a> above).</div>" } ], "returns" : { "type" : "item()", "description" : "<a xmlns:xqdoc=\"http://www.xqdoc.org/1.0\" href=\"#standard_return\">standard http-client return type</a> (since HEAD never returns any body data, only the &lt;http-schema:response&gt; element will be returned)." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">error:HC001 An HTTP error occurred.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">error:HC006 A timeout occurred waiting for the response.</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 1, "name" : "options", "qname" : "http:options", "signature" : "($href as xs:string) as xs:string*", "description" : " This function makes an HTTP OPTIONS request, which asks the server\n which operations it supports.\n", "summary" : "<p> This function makes an HTTP OPTIONS request, which asks the server\n which operations it supports.</p>", "annotation_str" : " %an:nondeterministic", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "nondeterministic", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "href", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The URL to which the request will be made (see <a href=\"#url_string\">note</a> above).</div>" } ], "returns" : { "type" : "xs:string*", "description" : "A sequence of xs:string values of the allowed operations." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">error:HC001 An HTTP error occurred.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">error:HC006 A timeout occurred waiting for the response.</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 2, "name" : "post", "qname" : "http:post", "signature" : "($href as xs:string, $body as item()) as item()+", "description" : " This function makes an HTTP POST request to a given URL. If the body\n passed to this function is an element, it will be serialized to XML\n to be sent to the server, and the Content-Type sent to the server will\n be \"text/xml\". Otherwise, the body will be converted to\n a plain string, and the Content-Type will be \"text/plain\".\n", "summary" : "<p> This function makes an HTTP POST request to a given URL.</p>", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "href", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The URL to which the request will be made (see <a href=\"#url_string\">note</a> above).</div>" }, { "name" : "body", "type" : "item()", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The body which will be sent to the server.</div>" } ], "returns" : { "type" : "item()+", "description" : "<a xmlns:xqdoc=\"http://www.xqdoc.org/1.0\" href=\"#standard_return\">standard http-client return type</a>." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">error:HC001 An HTTP error occurred.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">error:HC002 Error parsing the response content as XML.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">error:HC006 A timeout occurred waiting for the response.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">error:HCV02 Trying to follow a redirect of a POST request.</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 3, "name" : "post", "qname" : "http:post", "signature" : "($href as xs:string, $body as item(), $content-type as xs:string) as item()+", "description" : " This function makes an HTTP POST request to a given URL. If the body\n passed to this function is an element, it will be serialized\n according to the $content-type parameter as follows:\n <ul xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">\n  <li>If $content-type is \"text/xml\", \"application/xml\",\n \"text/xml-external-parsed-entity\", or\n \"application/xml-external-parsed-entity\", or if it ends with \"+xml\",\n $body will be serialized to XML.</li>\n  <li>If $content-type starts with \"text/html\", $body will be\n serialized to HTML.</li>\n  <li>Otherwise, $body will be serialized to text.</li>\n </ul>\n If $body is not an element, $body will be serialized to text\n regardless of $content-type.\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">In any case, Content-Type of the request sent to the server will\n be $content-type.</p>\n", "summary" : "<p> This function makes an HTTP POST request to a given URL.</p>", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "href", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The URL to which the request will be made (see <a href=\"#url_string\">note</a> above).</div>" }, { "name" : "body", "type" : "item()", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The body which will be sent to the server</div>" }, { "name" : "content-type", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The content type of the body as described above.</div>" } ], "returns" : { "type" : "item()+", "description" : "The first element of the result is the metadata (like headers, status etc), the next elements are the response" }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">error:HC001 An HTTP error occurred.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">error:HC002 Error parsing the response content as XML.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">error:HC006 A timeout occurred waiting for the response.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">error:HCV02 Trying to follow a redirect of a POST request.</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 2, "name" : "put", "qname" : "http:put", "signature" : "($href as xs:string, $body as item()) as item()+", "description" : " This function makes an HTTP PUT request to a given URL. If the body\n passed to this function is an element, it will be serialized to XML\n to be sent to the server, and the Content-Type sent to the server will\n be \"text/xml\". Otherwise, the body will be converted to\n a plain string, and the Content-Type will be \"text/plain\".\n", "summary" : "<p> This function makes an HTTP PUT request to a given URL.</p>", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "href", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The URL to which the request will be made (see <a href=\"#url_string\">note</a> above).</div>" }, { "name" : "body", "type" : "item()", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The body which will be sent to the server.</div>" } ], "returns" : { "type" : "item()+", "description" : "<a xmlns:xqdoc=\"http://www.xqdoc.org/1.0\" href=\"#standard_return\">standard http-client return type</a>." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">error:HC001 An HTTP error occurred.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">error:HC002 Error parsing the response content as XML.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">error:HC006 A timeout occurred waiting for the response.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">error:HCV02 Trying to follow a redirect of a PUT request.</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 3, "name" : "put", "qname" : "http:put", "signature" : "($href as xs:string, $body as item(), $content-type as xs:string) as item()+", "description" : " This function makes an HTTP PUT request to a given URL. If the body\n passed to this function is an element, it will be serialized\n according to the $content-type parameter as follows:\n <ul xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">\n  <li>If $content-type is \"text/xml\", \"application/xml\",\n \"text/xml-external-parsed-entity\", or\n \"application/xml-external-parsed-entity\", or if it ends with \"+xml\",\n $body will be serialized to XML.</li>\n  <li>If $content-type starts with \"text/html\", $body will be\n serialized to HTML.</li>\n  <li>Otherwise, $body will be serialized to text.</li>\n </ul>\n If $body is not an element, $body will be serialized to text\n regardless of $content-type.\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">In any case, Content-Type of the request sent to the server will\n be $content-type.</p>\n", "summary" : "<p> This function makes an HTTP PUT request to a given URL.</p>", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "href", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The URL to which the request will be made (see <a href=\"#url_string\">note</a> above).</div>" }, { "name" : "body", "type" : "item()", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The body which will be sent to the server.</div>" }, { "name" : "content-type", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The content type of $body as described above.</div>" } ], "returns" : { "type" : "item()+", "description" : "<a xmlns:xqdoc=\"http://www.xqdoc.org/1.0\" href=\"#standard_return\">standard http-client return type</a>." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">error:HC001 An HTTP error occurred.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">error:HC002 Error parsing the response content as XML.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">error:HC006 A timeout occurred waiting for the response.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">error:HCV02 Trying to follow a redirect of a PUT request.</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 3, "name" : "send-request", "qname" : "http:send-request", "signature" : "($request as element(http-schema:request)?, $href as xs:string?, $bodies as item()*) as item()+", "description" : " This function sends an HTTP request and returns the corresponding response.\n Its inputs, outputs, and behavior are identical to the\n <a xmlns:xqdoc=\"http://www.xqdoc.org/1.0\" href=\"http://expath.org/spec/http-client\">EXPath http-client</a>'s\n send-request() function (except that HTML responses are not tidied\n into XML - see <a xmlns:xqdoc=\"http://www.xqdoc.org/1.0\" href=\"#expath_relation\">the note above</a>). It\n is provided here for use in Zorba installations that do not have\n the EXPath module available. If you have the option of using the\n EXPath module instead of this function, please do so, as it will\n allow your application to be more interoperable between different\n XQuery engines.\n Full documentation of the $request parameter can be found in\n <a xmlns:xqdoc=\"http://www.xqdoc.org/1.0\" href=\"http://expath.org/spec/http-client#d2e183\">the EXPath\n specification</a>.\n", "summary" : "<p> This function sends an HTTP request and returns the corresponding response.</p>", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "request", "type" : "element(http-schema:request)", "occurrence" : "?", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> Contains the various parameters of the request (see above).</div>" }, { "name" : "href", "type" : "xs:string", "occurrence" : "?", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The URL to which the request will be made (see <a href=\"#url_string\">note</a> above). If this parameter is specified, it will override the \"href\" attribute of $request.</div>" }, { "name" : "bodies", "type" : "item()", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> is the request body content, for HTTP methods that can contain a body in the request (i.e. POST and PUT). It is an error if this param is not the empty sequence for methods</div>" } ], "returns" : { "type" : "item()+", "description" : "<a xmlns:xqdoc=\"http://www.xqdoc.org/1.0\" href=\"#standard_return\">standard http-client return type</a>." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">error:HC001 An HTTP error occurred.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">error:HC002 Error parsing the response content as XML.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">error:HC003 With a multipart response, the override-media-type must be either a multipart media type or application/octet-stream.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">error:HC004 The src attribute on the body element is mutually exclusive with all other attribute (except the media-type).</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">error:HC005 The input request element is not valid.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">error:HC006 A timeout occurred waiting for the response.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">error:HCV02 Trying to follow a redirect of a POST, PUT, or DELETE request</xqdoc:error>" ] } ], "variables" : [  ] }, "http://xbrl.io/modules/bizql/profiles/sec/networks" : { "ns" : "http://xbrl.io/modules/bizql/profiles/sec/networks", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">This module provides functionality for querying SEC networks in SEC filings.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">SEC Networks are actually XBRL Components. In the SEC profiles, all XBRL networks\n in an XBRL component must be consistent to each other. A way to look at it is that\n the SEC Network can be identified with the XBRL presentation network it contains.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">For XBRL-generic operations on XBRL components, use the\n generic components module.</p>\n", "sees" : [  ], "authors" : [ "<xqdoc:author xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Charles Hoffman</xqdoc:author>", "<xqdoc:author xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Matthias Brantner</xqdoc:author>", "<xqdoc:author xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Dennis Knochenwefel</xqdoc:author>", "<xqdoc:author xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Ghislain Fourny</xqdoc:author>" ], "version" : null, "encoding" : "utf-8", "namespaces" : [ { "uri" : "http://xbrl.io/modules/bizql/archives", "prefix" : "archives" }, { "uri" : "http://xbrl.io/modules/bizql/components", "prefix" : "components" }, { "uri" : "http://www.28msec.com/modules/credentials", "prefix" : "credentials" }, { "uri" : "http://xbrl.io/modules/bizql/facts", "prefix" : "facts" }, { "uri" : "http://xbrl.io/modules/bizql/profiles/sec/filings", "prefix" : "filings" }, { "uri" : "http://xbrl.io/modules/bizql/hypercubes", "prefix" : "hypercubes" }, { "uri" : "http://www.28msec.com/modules/mongodb", "prefix" : "mongo" }, { "uri" : "http://xbrl.io/modules/bizql/networks", "prefix" : "networks" }, { "uri" : "http://xbrl.io/modules/bizql/profiles/sec/core", "prefix" : "sec" }, { "uri" : "http://xbrl.io/modules/bizql/profiles/sec/networks", "prefix" : "sec-networks" }, { "uri" : "http://zorba.io/options/versioning", "prefix" : "ver" } ], "functions" : [ { "isDocumented" : true, "arity" : 1, "name" : "abstracts", "qname" : "sec-networks:abstracts", "signature" : "($networks-or-ids as item()*) as object()*", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Returns all SEC Abstracts contained in the supplied SEC Networks.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">SEC Abstracts are XBRL abstract primary items that may or may not be associated\n with a hypercube -- except those that are SEC LineItems (i.e., source of an all relation).</p>\n", "summary" : "<p>  Returns all SEC Abstracts contained in the supplied SEC Networks.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "networks-or-ids", "type" : "item()", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> a sequence of SEC Network objects, or their XBRL Component IDs.</div>" } ], "returns" : { "type" : "object()*", "description" : "the SEC Abstracts." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "axes", "qname" : "sec-networks:axes", "signature" : "($networks-or-ids as item()*) as object()*", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Returns all SEC Axes contained in the supplied SEC Networks.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">SEC Axes are XBRL dimensions.</p>\n", "summary" : "<p>  Returns all SEC Axes contained in the supplied SEC Networks.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "networks-or-ids", "type" : "item()", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> a sequence of SEC Network objects, or their XBRL Component IDs.</div>" } ], "returns" : { "type" : "object()*", "description" : "the SEC Axes." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "categories", "qname" : "sec-networks:categories", "signature" : "($networks-or-ids) as string*", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Return the categories of the supplied SEC networks (Statement, Disclosure, Document or Schedule).</p>\n", "summary" : "<p>  Return the categories of the supplied SEC networks (Statement, Disclosure, Document or Schedule).</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "networks-or-ids", "type" : null, "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> a sequence of SEC Network objects, or their XBRL Component IDs.</div>" } ], "returns" : { "type" : "string*", "description" : "the category of each network." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "concepts", "qname" : "sec-networks:concepts", "signature" : "($networks-or-ids as item()*) as object()*", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Returns all SEC Concepts contained in the supplied SEC Networks.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">SEC Concepts are XBRL concrete primary items that may or may not be associated\n with a hypercube.</p>\n", "summary" : "<p>  Returns all SEC Concepts contained in the supplied SEC Networks.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "networks-or-ids", "type" : "item()", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> a sequence of SEC Network objects, or their XBRL Component IDs.</div>" } ], "returns" : { "type" : "object()*", "description" : "the SEC Concepts." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "disclosures", "qname" : "sec-networks:disclosures", "signature" : "($networks-or-ids as item()*) as string+", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Returns the disclosures of the suplied networks.</p>\n", "summary" : "<p>  Returns the disclosures of the suplied networks.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "networks-or-ids", "type" : "item()", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> a sequence of SEC Network objects, or their XBRL Component IDs.</div>" } ], "returns" : { "type" : "string+", "description" : "the disclosure names, or \"UncategorizedInformation\" if none." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "fact-tables", "qname" : "sec-networks:fact-tables", "signature" : "($networks-or-ids as item()*) as array()", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Retrieves all facts belonging to the SEC Network.</p>\n", "summary" : "<p>  Retrieves all facts belonging to the SEC Network.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "networks-or-ids", "type" : "item()", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> a sequence of SEC Network objects, or their XBRL Component IDs.</div>" } ], "returns" : { "type" : "array()", "description" : "a array populated with fact values." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 2, "name" : "fact-tables", "qname" : "sec-networks:fact-tables", "signature" : "($networks-or-ids as item()*, $options as object()?) as array()", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Retrieves all facts belonging to the SEC Network.</p>\n", "summary" : "<p>  Retrieves all facts belonging to the SEC Network.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "networks-or-ids", "type" : "item()", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> a sequence of SEC Network objects, or their XBRL Component IDs.</div>" }, { "name" : "options", "type" : "object()", "occurrence" : "?", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> <a href=\"core#standard_options\">standard SEC BizQL options</a>.</div>" } ], "returns" : { "type" : "array()", "description" : "a array of arrays filled with fact values." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "facts", "qname" : "sec-networks:facts", "signature" : "($networks-or-ids as item()*) as object()*", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Retrieves all facts belonging to the SEC Network.</p>\n", "summary" : "<p>  Retrieves all facts belonging to the SEC Network.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "networks-or-ids", "type" : "item()", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> a sequence of SEC Network objects, or their XBRL Component IDs.</div>" } ], "returns" : { "type" : "object()*", "description" : "a sequence of facts." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 2, "name" : "facts", "qname" : "sec-networks:facts", "signature" : "($networks-or-ids as item()*, $options as object()?) as object()*", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Retrieves all facts belonging to the SEC Network.</p>\n", "summary" : "<p>  Retrieves all facts belonging to the SEC Network.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "networks-or-ids", "type" : "item()", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> a sequence of SEC Network objects, or their XBRL Component IDs.</div>" }, { "name" : "options", "type" : "object()", "occurrence" : "?", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> <a href=\"core#standard_options\">standard SEC BizQL options</a>.</div>" } ], "returns" : { "type" : "object()*", "description" : "a sequence of facts." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "line-items-report-elements", "qname" : "sec-networks:line-items-report-elements", "signature" : "($networks-or-ids as item()*) as object()*", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Returns all SEC LineItems report elements contained in the supplied SEC Networks.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">This is not to be confused with the \"line items\" terminology, which includes all\n SEC Abstracts and SEC Concepts.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">SEC LineItems report elements are XBRL abstract primary items that are top-level in their\n association with an XBRL hypercube (source of an all relation).</p>\n", "summary" : "<p>  Returns all SEC LineItems report elements contained in the supplied SEC Networks.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "networks-or-ids", "type" : "item()", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> a sequence of SEC Network objects, or their XBRL Component IDs.</div>" } ], "returns" : { "type" : "object()*", "description" : "the SEC LineItems report elements." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "line-items", "qname" : "sec-networks:line-items", "signature" : "($networks-or-ids as item()*) as object()*", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Returns all SEC Line Items (Concepts and Abstracts) contained in the\n supplied SEC Networks. Not to be confused with the LineItems report element.</p>\n", "summary" : "<p>  Returns all SEC Line Items (Concepts and Abstracts) contained in the\n supplied SEC Networks.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "networks-or-ids", "type" : "item()", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> a sequence of SEC Network objects, or their XBRL Component IDs.</div>" } ], "returns" : { "type" : "object()*", "description" : "the SEC Line Items." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "members", "qname" : "sec-networks:members", "signature" : "($networks-or-ids as item()*) as object()*", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Returns all SEC Members contained in the supplied SEC Networks.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">SEC Members are XBRL members that are in the transitive closure of SEC Axes\n via the dimension-domain and domain-member relations.</p>\n", "summary" : "<p>  Returns all SEC Members contained in the supplied SEC Networks.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "networks-or-ids", "type" : "item()", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> a sequence of SEC Network objects, or their XBRL Component IDs.</div>" } ], "returns" : { "type" : "object()*", "description" : "the SEC Members." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "model-structures", "qname" : "sec-networks:model-structures", "signature" : "($networks-or-ids as item()*) as object()*", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Computes the model structure of the supplied SEC Network, which is a hierarchy\n of SEC Report Elements (Tables, Axes, Members, LineItems, Abstracts, Concepts).</p>\n", "summary" : "<p>  Computes the model structure of the supplied SEC Network, which is a hierarchy\n of SEC Report Elements (Tables, Axes, Members, LineItems, Abstracts, Concepts).</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "networks-or-ids", "type" : "item()", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> a sequence of SEC Network objects, or their XBRL Component IDs.</div>" } ], "returns" : { "type" : "object()*", "description" : "the model structures of these SEC Networks." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "networks-for-disclosures", "qname" : "sec-networks:networks-for-disclosures", "signature" : "($disclosures as string*) as object()*", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Retrieves all SEC Networks that bear the supplied disclosures.</p>\n", "summary" : "<p>  Retrieves all SEC Networks that bear the supplied disclosures.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "disclosures", "type" : "string", "occurrence" : "*", "description" : "" } ], "returns" : { "type" : "object()*", "description" : "all models that match one of the disclosures." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 2, "name" : "networks-for-filings-and-categories", "qname" : "sec-networks:networks-for-filings-and-categories", "signature" : "($archive-or-ids as item()*, $categories as string*) as object()*", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Retrieves all models that belong to the archives given as first\n parameter and that match the supplied category.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">A category can be any of \"Statement\", \"Disclosure\", \"Document\",\n \"Schedule\", or \"Unknown\".</p>\n", "summary" : "<p>  Retrieves all models that belong to the archives given as first\n parameter and that match the supplied category.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "archive-or-ids", "type" : "item()", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> an arbitrary number of archive identifiers (AIDs) or archive objects.</div>" }, { "name" : "categories", "type" : "string", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> a list of categories.</div>" } ], "returns" : { "type" : "object()*", "description" : "all said models" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 2, "name" : "networks-for-filings-and-disclosures", "qname" : "sec-networks:networks-for-filings-and-disclosures", "signature" : "($archive-or-ids as item()*, $disclosures as string*) as object()*", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Retrieves all models that belong to the archives given as first\n parameter and that match the supplied disclosures.</p>\n", "summary" : "<p>  Retrieves all models that belong to the archives given as first\n parameter and that match the supplied disclosures.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "archive-or-ids", "type" : "item()", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> an arbitrary number of archive identifiers (AIDs) or archive objects.</div>" }, { "name" : "disclosures", "type" : "string", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> a list of disclosures.</div>" } ], "returns" : { "type" : "object()*", "description" : "all models in the archives and that match one of the disclosures." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "networks-for-filings", "qname" : "sec-networks:networks-for-filings", "signature" : "($archives-or-ids as item()*) as object()*", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Retrieves all SEC Networks in a given filing.</p>\n", "summary" : "<p>  Retrieves all SEC Networks in a given filing.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "archives-or-ids", "type" : "item()", "occurrence" : "*", "description" : "" } ], "returns" : { "type" : "object()*", "description" : "all networks in the supplied filings." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "num-abstracts", "qname" : "sec-networks:num-abstracts", "signature" : "($networks-or-ids as item()*) as integer*", "description" : " Return the number of (distinct) SEC Abstracts in each of the given components.\n", "summary" : "<p> Return the number of (distinct) SEC Abstracts in each of the given components.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "networks-or-ids", "type" : "item()", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> list of components or IDs</div>" } ], "returns" : { "type" : "integer*", "description" : "the said number of abstracts" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "num-axes", "qname" : "sec-networks:num-axes", "signature" : "($networks-or-ids as item()*) as integer*", "description" : " Return the number of (distinct) SEC Axes in each of the given components.\n", "summary" : "<p> Return the number of (distinct) SEC Axes in each of the given components.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "networks-or-ids", "type" : "item()", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> list of components or IDs</div>" } ], "returns" : { "type" : "integer*", "description" : "the said number of axes" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "num-concepts", "qname" : "sec-networks:num-concepts", "signature" : "($networks-or-ids as item()*) as integer*", "description" : " Return the number of (distinct) SEC Concepts in each of the given components.\n", "summary" : "<p> Return the number of (distinct) SEC Concepts in each of the given components.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "networks-or-ids", "type" : "item()", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> list of components or IDs</div>" } ], "returns" : { "type" : "integer*", "description" : "the said number of concepts" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "num-line-items", "qname" : "sec-networks:num-line-items", "signature" : "($networks-or-ids as item()*) as integer*", "description" : " Return the number of (distinct) SEC LineItems report elements in each of the given components.\n", "summary" : "<p> Return the number of (distinct) SEC LineItems report elements in each of the given components.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "networks-or-ids", "type" : "item()", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> list of components or IDs</div>" } ], "returns" : { "type" : "integer*", "description" : "the said number of line items" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "num-members", "qname" : "sec-networks:num-members", "signature" : "($networks-or-ids as item()*) as integer*", "description" : " Return the number of (distinct) SEC Members in each of the given components.\n", "summary" : "<p> Return the number of (distinct) SEC Members in each of the given components.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "networks-or-ids", "type" : "item()", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> list of components or IDs</div>" } ], "returns" : { "type" : "integer*", "description" : "the said number of members" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "num-report-elements", "qname" : "sec-networks:num-report-elements", "signature" : "($networks-or-ids as item()*) as integer*", "description" : " Return the number of (distinct) SEC report elements in each of the given components.\n", "summary" : "<p> Return the number of (distinct) SEC report elements in each of the given components.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "networks-or-ids", "type" : "item()", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> list of components or IDs</div>" } ], "returns" : { "type" : "integer*", "description" : "the said number of report elements" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "num-tables", "qname" : "sec-networks:num-tables", "signature" : "($networks-or-ids as item()*) as integer*", "description" : " Return the number of SEC Tables in each of the given components.\n", "summary" : "<p> Return the number of SEC Tables in each of the given components.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "networks-or-ids", "type" : "item()", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> list of components or IDs</div>" } ], "returns" : { "type" : "integer*", "description" : "the said number of tables" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "populated-model-structures", "qname" : "sec-networks:populated-model-structures", "signature" : "($networks-or-ids as item()*) as object()*", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Retrieves all facts belonging to the SEC Networks and populates the model structures.</p>\n", "summary" : "<p>  Retrieves all facts belonging to the SEC Networks and populates the model structures.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "networks-or-ids", "type" : "item()", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> a sequence of SEC Network objects, or their XBRL Component IDs.</div>" } ], "returns" : { "type" : "object()*", "description" : "the populated model structures (Facts array fields)." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 2, "name" : "populated-model-structures", "qname" : "sec-networks:populated-model-structures", "signature" : "($networks-or-ids as item()*, $options as object()?) as object()*", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Retrieves all facts belonging to the SEC Networks and populates the model structures.</p>\n", "summary" : "<p>  Retrieves all facts belonging to the SEC Networks and populates the model structures.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "networks-or-ids", "type" : "item()", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> a sequence of SEC Network objects, or their XBRL Component IDs.</div>" }, { "name" : "options", "type" : "object()", "occurrence" : "?", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> <a href=\"core#standard_options\">standard SEC BizQL options</a>.</div>" } ], "returns" : { "type" : "object()*", "description" : "the populated model structures (Facts array fields)." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "sub-categories", "qname" : "sec-networks:sub-categories", "signature" : "($networks-or-ids) as string*", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Return the sub-categories of the supplied SEC networks (Detail, TextBlockLevel4, TextBLockLevel1to3).</p>\n", "summary" : "<p>  Return the sub-categories of the supplied SEC networks (Detail, TextBlockLevel4, TextBLockLevel1to3).</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "networks-or-ids", "type" : null, "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> a sequence of SEC Network objects, or their XBRL Component IDs.</div>" } ], "returns" : { "type" : "string*", "description" : "the sub-category of each network." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "summaries", "qname" : "sec-networks:summaries", "signature" : "($networks-or-ids) as object()*", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Return summary information for the supplied SEC networks.</p>\n", "summary" : "<p>  Return summary information for the supplied SEC networks.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "networks-or-ids", "type" : null, "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> a sequence of SEC Network objects, or their XBRL Component IDs.</div>" } ], "returns" : { "type" : "object()*", "description" : "one object per network, containing a summary." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "tables", "qname" : "sec-networks:tables", "signature" : "($networks-or-ids as item()*) as object()*", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Returns all SEC Tables contained in the supplied SEC Networks.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">SEC Tables are XBRL hypercubes.</p>\n", "summary" : "<p>  Returns all SEC Tables contained in the supplied SEC Networks.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "networks-or-ids", "type" : "item()", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> a sequence of SEC Network objects, or their XBRL Component IDs.</div>" } ], "returns" : { "type" : "object()*", "description" : "the SEC Tables." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 2, "name" : "tables", "qname" : "sec-networks:tables", "signature" : "($networks-or-ids as item()*, $options as object()?) as object()*", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Returns all SEC Tables contained in the supplied SEC Networks.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">SEC Tables are XBRL hypercubes.</p>\n", "summary" : "<p>  Returns all SEC Tables contained in the supplied SEC Networks.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "networks-or-ids", "type" : "item()", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> a sequence of SEC Network objects, or their XBRL Component IDs.</div>" }, { "name" : "options", "type" : "object()", "occurrence" : "?", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> <a href=\"core#standard_options\">standard SEC BizQL options</a>.</div>" } ], "returns" : { "type" : "object()*", "description" : "the SEC Tables." }, "errors" : [  ] } ], "variables" : [ { "name" : "sec-networks:BALANCE_SHEET", "type" : "xs:string", "description" : " Disclosure for the balance sheet.\n" }, { "name" : "sec-networks:BALANCE_SHEET_PARENTHETICAL", "type" : "xs:string", "description" : " Disclosure for the parenthetical balance sheet.\n" }, { "name" : "sec-networks:INCOME_STATEMENT", "type" : "xs:string", "description" : " Disclosure for the income statement.\n" }, { "name" : "sec-networks:INCOME_STATEMENT_PARENTHETICAL", "type" : "xs:string", "description" : " Disclosure for the parenthetical income statement.\n" }, { "name" : "sec-networks:STATEMENT_OF_COMPREHENSIVE_INCOME", "type" : "xs:string", "description" : " Disclosure for the statement of comprehensive income.\n" }, { "name" : "sec-networks:CASH_FLOW_STATEMENT", "type" : "xs:string", "description" : " Disclosure for the cash flow statement.\n" }, { "name" : "sec-networks:CASH_FLOW_STATEMENT_PARENTHETICAL", "type" : "xs:string", "description" : " Disclosure for the parenthetical cash flow statement.\n" }, { "name" : "sec-networks:DEFAULT_COMPONENT", "type" : "xs:string", "description" : " Disclosure for the default component.\n" }, { "name" : "sec-networks:DOCUMENT_AND_ENTITY_INFO", "type" : "xs:string", "description" : " Disclosure for the document and entity information\n" } ] }, "http://www.28msec.com/modules/http/request" : { "ns" : "http://www.28msec.com/modules/http/request", "description" : " The Sausalito request module provides functions for accessing\n information contained in the current HTTP request. For example,\n the <tt xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">parameter-names</tt> function can be used to retrieve all\n the names of the parameters contained in a request.\n", "sees" : [  ], "authors" : [ "<xqdoc:author xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">28msec</xqdoc:author>" ], "version" : null, "encoding" : "utf-8", "namespaces" : [ { "uri" : "http://www.28msec.com/modules/http/request", "prefix" : "request" }, { "uri" : "http://www.zorba-xquery.com/modules/cryptography/hmac", "prefix" : "sec" }, { "uri" : "http://zorba.io/options/versioning", "prefix" : "ver" } ], "functions" : [ { "isDocumented" : true, "arity" : 0, "name" : "binary-content", "qname" : "request:binary-content", "signature" : "() as xs:base64Binary external", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Returns the content of the request as base64Binary.</p>\n", "summary" : "<p>  Returns the content of the request as base64Binary.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [  ], "returns" : { "type" : "xs:base64Binary", "description" : "The content of the request as base64Binary." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">request:no-binary-content if the content contained in the body of the request cannot be treated as binary because it is a request with multipart or url-encoded content.</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 1, "name" : "binary-part", "qname" : "request:binary-part", "signature" : "($ref as xs:string) as xs:base64Binary external", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Returns the value of a part as base64Binary.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">A part is identified by a reference that is the value of a\n <tt>src</tt> attribute returned by the <tt>request:parts</tt> function.</p>\n", "summary" : "<p>  Returns the value of a part as base64Binary.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "ref", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the name of the part</div>" } ], "returns" : { "type" : "xs:base64Binary", "description" : "the value of the part as base64Binary" }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">request:invalid-part if the part with the given name ($ref) does not exist</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">request:non-multipart if the current request does not contain multipart content</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">request:invalid-multipart if the multipart content is invalid (e.g. the boundary is missing)</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 0, "name" : "content-length", "qname" : "request:content-length", "signature" : "() as xs:integer? external", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Returns the length of the content in bytes.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">The value returned corresponds to the value of the HTTP\n content-length header. The function returns an empty sequence\n if this header does not exist in the request or its value\n could not be converted to item of type xs:integer</p>.\n", "summary" : "<p>  Returns the length of the content in bytes.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [  ], "returns" : { "type" : "xs:integer?", "description" : "The content-length in bytes of the content sent with this request or the empty sequence if the content-length header does not exist in the request." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 0, "name" : "content-type", "qname" : "request:content-type", "signature" : "() as xs:string? external", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Returns the content-type of the data sent with this request.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Note that the content-type is only set for PUT and POST requests.</p>\n", "summary" : "<p>  Returns the content-type of the data sent with this request.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [  ], "returns" : { "type" : "xs:string?", "description" : "The content-type of the request if it is a PUT or POST request. Otherwise, it returns the empty sequence." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 0, "name" : "header-accept", "qname" : "request:header-accept", "signature" : "() as element(request:accept)*", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Returns the values of the HTTP ACCEPT header.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">The data is returned as a sequence of elements with name\n <tt>request:accept</tt> as shown in the following example.</p>\n <code xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">\n  &lt;request:accept xmlns:request=\"http://www.28msec.com/modules/http/request\"&gt;\n    &lt;request:type&gt;text&lt;/request:type&gt;\n    &lt;request:subtype&gt;html&lt;/request:subtype&gt;\n    &lt;request:quality&gt;1&lt;/request:quality&gt;\n  &lt;/request:accept&gt;\n  &lt;request:accept xmlns=\"http://www.28msec.com/modules/http/request\"&gt;\n    &lt;request:type&gt;application&lt;/request:type&gt;\n    &lt;request:quality&gt;0.9&lt;/request:quality&gt;\n  &lt;/request:accept&gt;\n </code>\n", "summary" : "<p>  Returns the values of the HTTP ACCEPT header.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [  ], "returns" : { "type" : "element(request:accept)*", "description" : "The header values of the header ACCEPT or the empty sequence if the header is not contained in the request. The order of the returned elements reflects the order of the components in the header." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 0, "name" : "header-names", "qname" : "request:header-names", "signature" : "() as xs:string* external", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Returns the names of all the HTTP headers in this request.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Header fields are colon-separated name-value pairs, terminated\n by a carriage return (CR) and line feed (LF) character sequence. The\n names and values of each header are allowed to consist of US-ASCII\n characters only.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">The names of the headers are returned using upper-case letters.\n If a header with the same name is contained multiple times in a request,\n its name is only returned once. The order of the names in the resulting\n sequence does not reflect the order of the headers in the request. If\n a header does not have a value, it is as if the header does not exist\n in the request.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Note that the header names user-agent and content-type are not\n returned by this function. They are returned by the corresponding\n functions of this module module\n (e.g. <a href=\"#user-agent-0\">user-agent</a>).</p>\n", "summary" : "<p>  Returns the names of all the HTTP headers in this request.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [  ], "returns" : { "type" : "xs:string*", "description" : "The names of the headers of this request or the empty sequence if no headers are contained in the request." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "header-value", "qname" : "request:header-value", "signature" : "($name as xs:string) as xs:string? external", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Returns the value of the HTTP header with the given name.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Header fields are colon-separated name-value pairs, terminated\n by a carriage return (CR) and line feed (LF) character sequence. The\n names and values of each header are allowed to consist of US-ASCII\n characters only.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Please note that header names are considered case-insensitive.\n Also note, that only one value is returned if multiple headers with the\n same names exist in the request. This value is a comma-separated list\n of the values of the headers in the order in which the headers appeared\n in the request.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">All headers having a name that starts with SAUSALITO_ are reserved\n and will not be returned by this function.</p>\n", "summary" : "<p>  Returns the value of the HTTP header with the given name.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "name", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The header name for which the value should be returned.</div>" } ], "returns" : { "type" : "xs:string?", "description" : "The header value of the header with the <tt xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">$name</tt> argument or the empty sequence if no header with that name is contained in the request." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 0, "name" : "method-delete", "qname" : "request:method-delete", "signature" : "() as xs:boolean external", "description" : " Returns true if the HTTP method of this request is DELETE.\n", "summary" : "<p> Returns true if the HTTP method of this request is DELETE.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [  ], "returns" : { "type" : "xs:boolean", "description" : "true if the HTTP method of this request is DELETE, false otherwise." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 0, "name" : "method-get", "qname" : "request:method-get", "signature" : "() as xs:boolean external", "description" : " Returns true if the HTTP method of this request is GET.\n", "summary" : "<p> Returns true if the HTTP method of this request is GET.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [  ], "returns" : { "type" : "xs:boolean", "description" : "true if the HTTP method of this request is GET, false otherwise." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 0, "name" : "method-head", "qname" : "request:method-head", "signature" : "() as xs:boolean external", "description" : " Returns true if the HTTP method of this request is HEAD.\n", "summary" : "<p> Returns true if the HTTP method of this request is HEAD.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [  ], "returns" : { "type" : "xs:boolean", "description" : "true if the HTTP method of this request is HEAD, false otherwise." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 0, "name" : "method-options", "qname" : "request:method-options", "signature" : "() as xs:boolean external", "description" : " Returns true if the HTTP method of this request is OPTION.\n", "summary" : "<p> Returns true if the HTTP method of this request is OPTION.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [  ], "returns" : { "type" : "xs:boolean", "description" : "true if the HTTP method of this request is OPTION, false otherwise." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 0, "name" : "method-patch", "qname" : "request:method-patch", "signature" : "() as xs:boolean external", "description" : " Returns true if the HTTP method of this request is PATCH.\n", "summary" : "<p> Returns true if the HTTP method of this request is PATCH.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [  ], "returns" : { "type" : "xs:boolean", "description" : "true if the HTTP method of this request is PATCH, false otherwise." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 0, "name" : "method-post", "qname" : "request:method-post", "signature" : "() as xs:boolean external", "description" : " Returns true if the HTTP method of this request is POST.\n", "summary" : "<p> Returns true if the HTTP method of this request is POST.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [  ], "returns" : { "type" : "xs:boolean", "description" : "true if the HTTP method of this request is POST, false otherwise." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 0, "name" : "method-put", "qname" : "request:method-put", "signature" : "() as xs:boolean external", "description" : " Returns true if the HTTP method of this request is PUT.\n", "summary" : "<p> Returns true if the HTTP method of this request is PUT.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [  ], "returns" : { "type" : "xs:boolean", "description" : "true if the HTTP method of this request is PUT, false otherwise." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 0, "name" : "method", "qname" : "request:method", "signature" : "() as xs:string external", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Returns the name of the HTTP method used to make this request.</p>\n", "summary" : "<p>  Returns the name of the HTTP method used to make this request.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [  ], "returns" : { "type" : "xs:string", "description" : "The request method used to make this request (i.e. GET, POST, PUT, DELETE or HEAD)." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 0, "name" : "parameter-names", "qname" : "request:parameter-names", "signature" : "() as xs:string*", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Returns the names of the parameters contained in the current request.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Parameters are name-value pairs contained in the query string of the URL\n used to make this request. As defined in RFC 1738, the query string of a\n URL starts with a \"?\" character and ends with the character (if any).\n Additionally, such name-value pairs may be part of the request's body if\n it is a PUT or POST request and the content-type of the request is\n \"application/x-www-form-urlencoded\". Name-value pairs are separated\n using either the \"&amp;\" or the \";\" character.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">In general, the names and the values  are precent-encoded. This function\n does the decoding of the parameters, i.e. it returns the values being\n not percent-encoded.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Also, the names of each parameter (after being precent-decoded) are\n treated as UTF-8. Please see the <tt>http:parameter-names#1</tt> function\n for retrieving parameter names submitted using a encoding other than UTF-8.\n </p>\n", "summary" : "<p>  Returns the names of the parameters contained in the current request.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [  ], "returns" : { "type" : "xs:string*", "description" : "The names of all parameters in this request. The empty sequence is returned if there are none." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "parameter-names", "qname" : "request:parameter-names", "signature" : "($encoding as xs:string) as xs:string* external", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Returns the names of the parameters contained in the current request.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">This function is similar to the <tt>request:parameter-names#0</tt> function.\n However, the names are treated (after precent-decoding) using the\n given encoding supplied as parameter. For example, parameters might\n be encoded using the ISO-8859-1 encoding.</p>\n", "summary" : "<p>  Returns the names of the parameters contained in the current request.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "encoding", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The encoding of the parameters in the request (e.g. ISO-8859-1).</div>" } ], "returns" : { "type" : "xs:string*", "description" : "The names of all parameters in this request. The empty sequence is returned if there are none." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">request:invalid-encoding if the given encoding is invalid or not supported.</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 1, "name" : "parameter-values", "qname" : "request:parameter-values", "signature" : "($name as xs:string) as xs:string*", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Returns a sequence of parameter values for the given parameter name\n which are contained in the URL's query string or the body of a POST or PUT\n request.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Parameters are name-value pairs contained in the query string of the URL\n used to make this request. As defined in RFC 1738, the query string of a\n URL starts with a \"?\" character and ends with the character (if any).\n Additionally, such name-value pairs may be part of the request's body if\n it is a PUT or POST request and the content-type of the request is\n \"application/x-www-form-urlencoded\". Name-value pairs are separated\n using either the \"&amp;\" or the \";\" character.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">In general, the names and the values  are precent-encoded. This function\n does the decoding of the parameters, i.e. it returns the values being\n not percent-encoded.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Also, the names and the values of each parameter (after being precent-decoded)\n are treated as UTF-8. Please see the <tt>http:parameter-values#3</tt> function\n for retrieving parameters submitted using a encoding other than UTF-8.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">This function returns the empty-sequence if no parameter with the\n given name exists in this request. If you want the function to return\n a default value other than the empty sequence, use the\n <tt>http:parameter-values#2</tt> function.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">A URL could contain the following query string:\n <tt>name1=value1&amp;name2=value2;name1=value3&amp;name3</tt>.\n <tt>name1=value1&amp;name2=value2&amp;name1=value3&amp;name3</tt>.\n The name value pairs in this query string are\n <ul>\n   <li> name: <tt>name1</tt>; values: <tt>value1</tt> and <tt>value3</tt></li>\n   <li> name: <tt>name2</tt>; value: <tt>value2</tt></li>\n   <li> name: <tt>name3</tt>; value: <tt/></li>\n </ul>\n </p>\n", "summary" : "<p>  Returns a sequence of parameter values for the given parameter name\n which are contained in the URL's query string or the body of a POST or PUT\n request.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "name", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The name of the parameter whose value(s) should be returned.</div>" } ], "returns" : { "type" : "xs:string*", "description" : "A sequence of values for the parameter with the given name. The empty sequence is returned if no parameter exists with the given name." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 2, "name" : "parameter-values", "qname" : "request:parameter-values", "signature" : "($name as xs:string, $default-values as xs:string*) as xs:string*", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Returns a sequence of parameter values for the given parameter name\n which are contained in the URL's query string or the body of a POST or PUT\n request.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">This function is similar to the <tt>request:parameter-values#1</tt> function.\n However, instead of returning the empty-sequence as a default value it returns\n the given default-values sequence if no parameter with the given name is found\n in this request.</p>\n", "summary" : "<p>  Returns a sequence of parameter values for the given parameter name\n which are contained in the URL's query string or the body of a POST or PUT\n request.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "name", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The name of the parameter whose value(s) should be returned.</div>" }, { "name" : "default-values", "type" : "xs:string", "occurrence" : "*", "description" : "" } ], "returns" : { "type" : "xs:string*", "description" : "A sequence of values for the parameter with the given name. The sequence given as $default-values parameter is returned if no parameter exists with the given name." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 3, "name" : "parameter-values", "qname" : "request:parameter-values", "signature" : "($name as xs:string, $default-values as xs:string*, $encoding as xs:string) as xs:string* external", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Returns a sequence of parameter values for the given parameter name\n which are contained in the URL's query string or the body of a POST or PUT\n request.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">This function is similar to the <tt>request:parameter-values#2</tt> function.\n However, the names and values are treated (after precent-decoding) using the\n given encoding supplied as third parameter. For example, parameters might\n be encoded using the ISO-8859-1 encoding.</p>\n", "summary" : "<p>  Returns a sequence of parameter values for the given parameter name\n which are contained in the URL's query string or the body of a POST or PUT\n request.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "name", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The name of the parameter whose value(s) should be returned.</div>" }, { "name" : "default-values", "type" : "xs:string", "occurrence" : "*", "description" : "" }, { "name" : "encoding", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The encoding of the parameters in the request (e.g. ISO-8859-1)</div>" } ], "returns" : { "type" : "xs:string*", "description" : "A sequence of values for the parameter with the given name. The sequence given as $default-values parameter is returned if no parameter exists with the given name." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">request:invalid-encoding if the given encoding is invalid or not supported.</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 0, "name" : "parts", "qname" : "request:parts", "signature" : "() as element(request:multipart) external", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Returns the metadata of all parts contained in a multipart request.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">The data is returned as an element with name <tt>request:mulitpart</tt>\n as shown in the following example.</p>\n <code xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">\n  &lt;multipart xmlns=\"http://www.28msec.com/modules/http/request\"\n    media-type=\"multipart/...; boundary=...\"&gt;\n    &lt;header name=\"Content-Disposition\" value='form-data; filename=\"...\"'/&gt;\n    &lt;header name=\"Content-Type\" value=\"application/octet-stream\"/&gt;\n    &lt;body filename=\"...\" src=\"...\"/&gt;\n    &lt;header name=\"Content-Disposition\" value='form-data; filename=\"...\"'/&gt;\n    &lt;header name=\"Content-Type\" value=\"application/octet-stream\"/&gt;\n    &lt;body filename=\"...\" src=\"...\"/&gt;\n  &lt;/multipart&gt;\n </code>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">The <tt>media-type</tt> is the type of the content as given in the\n request (i.e. it's value is equal to the value returned by\n <tt>request:header-values(\"Content-Type\")</tt>).</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Within the <tt>multipart</tt> element is a sequence of\n (<tt>header</tt>*,<tt>body</tt>) elements. Each such group corresponds to\n one part. Every <tt>header</tt> belongs to a header for this part and the\n <tt>body</tt> refers to the value of a part. The actual value of a part\n can be retrieved by passing the value of the <tt>src</tt> attribute of\n the <tt>body</tt> to the <tt>request:text-part</tt> or\n <tt>request:binary-part</tt> functions. The other attributes of the\n <tt>body</tt> element represent a parameter of the Content-Disposition\n header as described in RFC 2183 (e.g. filename, name, creation-date).</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Please note that recursive multipart content is not supported.</p>\n", "summary" : "<p>  Returns the metadata of all parts contained in a multipart request.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [  ], "returns" : { "type" : "element(request:multipart)", "description" : "a multipart element representing the meta data of the multipart content" }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">request:non-multipart if the current request does not contain multipart content</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">request:invalid-multipart if the multipart content is invalid (e.g. the boundary is missing)</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 0, "name" : "path", "qname" : "request:path", "signature" : "() as xs:string", "description" : " Return the path component of the request URI. The path starts after the\n host and ends before the query string starts.\n", "summary" : "<p> Return the path component of the request URI.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [  ], "returns" : { "type" : "xs:string", "description" : "The path component of the request URI" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 0, "name" : "query-string", "qname" : "request:query-string", "signature" : "() as xs:string external", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Returns the query string that was used to make this request.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">The query string contains the part of the request URL that\n starts with the '?' character to the end or the starting of the\n fragment (i.e. the '#' character).</p>\n", "summary" : "<p>  Returns the query string that was used to make this request.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [  ], "returns" : { "type" : "xs:string", "description" : "The query string part of the request's URL" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 0, "name" : "remote-addr", "qname" : "request:remote-addr", "signature" : "() as xs:string external", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Returns the IP address of the client to which this request\n is connected.</p>\n", "summary" : "<p>  Returns the IP address of the client to which this request\n is connected.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [  ], "returns" : { "type" : "xs:string", "description" : "The IP address on the client side to which this request is connected." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 0, "name" : "remote-port", "qname" : "request:remote-port", "signature" : "() as xs:int external", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Returns the port of the client to which this request is connected.</p>\n", "summary" : "<p>  Returns the port of the client to which this request is connected.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [  ], "returns" : { "type" : "xs:int", "description" : "The port on the client side to which this request is connected." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 0, "name" : "server-name", "qname" : "request:server-name", "signature" : "() as xs:string external", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Returns the server name of the server running the application.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">The web server's hostname or IP address.</p>\n", "summary" : "<p>  Returns the server name of the server running the application.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [  ], "returns" : { "type" : "xs:string", "description" : "The name of the server that runs the application accepting this request." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 0, "name" : "server-port", "qname" : "request:server-port", "signature" : "() as xs:int external", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Returns the sever port to which the client making the current request\n is connected.</p>\n", "summary" : "<p>  Returns the sever port to which the client making the current request\n is connected.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [  ], "returns" : { "type" : "xs:int", "description" : "The server port to which the client is connected." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 0, "name" : "text-content", "qname" : "request:text-content", "signature" : "() as xs:string external", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Returns the content of the request as string.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">The function returns the content of the request only\n if the content-type refers to a type that can be treated\n as text (e.g. text/* or application/xml). The function raises\n an error if the content cannot be treated as text.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">The text content is interpreted using the encoding/charset\n that is specified in the Content-Type header of the request. If\n no charset is specified, the default ISO-8859-1 is used. If a encoding\n other than the specified or default one should be used, the\n <tt>request:text-content#1</tt> function should be used.</p>\n", "summary" : "<p>  Returns the content of the request as string.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [  ], "returns" : { "type" : "xs:string", "description" : "The content of the request as a string." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">request:invalid-encoding if the encoding specified in the Content-Type header is invalid or not supported.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">request:no-text-content if the content contained in the body of the request cannot be treated as text.</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 1, "name" : "text-content", "qname" : "request:text-content", "signature" : "($overwrite-encoding as xs:string) as xs:string external", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Returns the content of the request as string interpreting\n it with the given encoding.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">The function returns the content of the request only\n if the content-type refers to a type that can be treated\n as text (e.g. text/* or application/xml). The function raises\n an error if the content cannot be treated as text.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">The text content is interpreted using the given encoding/charset.\n That is, the charset specified in the Content-Type header of the request\n is ignored. An error is raised if the given encoding is invalid\n or not supported.</p>\n", "summary" : "<p>  Returns the content of the request as string interpreting\n it with the given encoding.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "overwrite-encoding", "type" : "xs:string", "occurrence" : null, "description" : "" } ], "returns" : { "type" : "xs:string", "description" : "The content of the request as a string." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">request:invalid-encoding if the encoding specified in the Content-Type header or the $overwrite-encoding parameter is invalid or not supported.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">request:no-text-content if the content contained in the body of the request cannot be treated as text.</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 1, "name" : "text-part", "qname" : "request:text-part", "signature" : "($ref as xs:string) as xs:string external", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Returns the value of a part as string</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">A part is identified by a reference that is the value of a\n <tt>src</tt> attribute returned by the <tt>request:parts</tt> function.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">The value of the text part is interpreted using the encoding/charset\n given in the headers of the part. If no encoding is given, the default\n US-ASCII is assumed.</p>\n", "summary" : "<p>  Returns the value of a part as string \n  A part is identified by a reference that is the value of a\n  src  attribute returned by the  request:parts  function.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "ref", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the name of the part</div>" } ], "returns" : { "type" : "xs:string", "description" : "the value of the part as string" }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">request:invalid-encoding if the encoding given in the headers of the part is invalid or not supported.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">request:no-text-content if the value of the part cannot be treated as text</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">request:invalid-part if the part with the given name ($ref) does not exist</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">request:non-multipart if the current request does not contain multipart content</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">request:invalid-multipart if the multipart content is invalid (e.g. the boundary is missing)</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 2, "name" : "text-part", "qname" : "request:text-part", "signature" : "($ref as xs:string, $overwrite-encoding as xs:string) as xs:string external", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Returns the value of a part as string interpreting\n it with the given encoding.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">A part is identified by a reference that is the value of a\n <tt>src</tt> attribute returned by the <tt>request:parts</tt> function.</p>\n", "summary" : "<p>  Returns the value of a part as string interpreting\n it with the given encoding.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "ref", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the name of the part</div>" }, { "name" : "overwrite-encoding", "type" : "xs:string", "occurrence" : null, "description" : "" } ], "returns" : { "type" : "xs:string", "description" : "the value of the part as string" }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">request:invalid-encoding if the encoding given using the $overwrite-encoding parameter is invalid or not supported.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">request:no-text-content if the value of the part cannot be treated as text</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">request:invalid-part if the part with the given name ($ref) does not exist</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">request:non-multipart if the current request does not contain multipart content</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">request:invalid-multipart if the multipart content is invalid (e.g. the boundary is missing)</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 0, "name" : "uri", "qname" : "request:uri", "signature" : "() as xs:string external", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Returns the URI that was used to make this request.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">The value returned contains the part of the URL starting\n from the path to the end or the starting of the fragment (i.e.\n the '#' character).</p>\n", "summary" : "<p>  Returns the URI that was used to make this request.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [  ], "returns" : { "type" : "xs:string", "description" : "The path and query string part of the request's URL" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 0, "name" : "user-agent", "qname" : "request:user-agent", "signature" : "() as xs:string? external", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Returns the user agent that made to perform the current request.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">This function returns the value of the User-Agent header\n contained in the current request.</p>\n", "summary" : "<p>  Returns the user agent that made to perform the current request.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [  ], "returns" : { "type" : "xs:string?", "description" : "The user agent used to perform this request of the empty sequence if there was no User-Agent header in the request." }, "errors" : [  ] } ], "variables" : [  ] }, "http://www.w3.org/2005/xpath-functions" : { "ns" : "http://www.w3.org/2005/xpath-functions", "description" : " This module contains all the functions specified in the\n W3C XPath and XQuery Functions and Operators 3.0.\n", "sees" : [ "<xqdoc:see xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">http://www.w3.org/TR/xpath-functions-30/</xqdoc:see>" ], "authors" : [ "<xqdoc:author xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">www.w3c.org</xqdoc:author>" ], "version" : null, "encoding" : "utf-8", "namespaces" : [  ], "functions" : [ { "isDocumented" : true, "arity" : 2, "name" : "QName", "qname" : "fn:QName", "signature" : "($paramURI as xs:string?, $paramQName as xs:string) as xs:QName external", "description" : " <div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">Constructs an <code>xs:QName</code> value given a namespace URI and a lexical\n             QName.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\"><example role=\"signature\"><proto name=\"QName\" return-type=\"xs:QName\" isOp=\"no\" prefix=\"fn\" returnEmptyOk=\"no\" returnSeq=\"no\" returnVaries=\"no\" isSchema=\"no\" isDatatype=\"no\" isSpecial=\"no\"><arg name=\"paramURI\" type=\"xs:string?\"/><arg name=\"paramQName\" type=\"xs:string\"/></proto></example></p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">This function is <termref def=\"dt-deterministic\">deterministic</termref>, <termref def=\"dt-context-independent\">context-independent</termref>,  and <termref def=\"dt-focus-independent\">focus-independent</termref>. </p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The namespace URI in the returned QName is taken from <code>$paramURI</code>. If\n                <code>$paramURI</code> is the zero-length string or the empty sequence, it represents\n             \"no namespace\".</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The prefix (or absence of a prefix) in <code>$paramQName</code> is retained in the\n             returned <code>xs:QName</code> value.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The local name in the result is taken from the local part of\n             <code>$paramQName</code>.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">A <phrase diff=\"add\" at=\"L\">dynamic</phrase> error is raised <errorref class=\"CA\" code=\"0002\"/> if <code>$paramQName</code> does\n             not have the correct lexical form for an instance of <code>xs:QName</code>.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">A <phrase diff=\"add\" at=\"L\">dynamic</phrase> error is raised <errorref class=\"CA\" code=\"0002\"/> if <code>$paramURI</code> is the\n             zero-length string or the empty sequence, and the value of <code>$paramQName</code>\n             contains a colon (<code>:</code>).</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\" diff=\"add\" at=\"G\">A <phrase diff=\"add\" at=\"L\">dynamic</phrase> error <rfc2119>may</rfc2119> be raised <errorref class=\"CA\" code=\"0002\"/> if <code>$paramURI</code> is not a valid URI (XML Namespaces 1.0) or\n             IRI (XML Namespaces 1.1). </p></div>\n", "summary" : "<p>  Constructs an  xs:QName  value given a namespace URI and a lexical\n             QName.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "paramURI", "type" : "xs:string", "occurrence" : "?", "description" : "" }, { "name" : "paramQName", "type" : "xs:string", "occurrence" : null, "description" : "" } ], "returns" : { "type" : "xs:QName", "description" : "" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "abs", "qname" : "fn:abs", "signature" : "($arg as numeric?) as numeric? external", "description" : " <div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">Returns the absolute value of <code>$arg</code>.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\"><example role=\"signature\"><proto name=\"abs\" return-type=\"numeric?\" isOp=\"no\" prefix=\"fn\" returnEmptyOk=\"no\" returnSeq=\"no\" returnVaries=\"no\" isSchema=\"no\" isDatatype=\"no\" isSpecial=\"no\"><arg name=\"arg\" type=\"numeric?\"/></proto></example></p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">This function is <termref def=\"dt-deterministic\">deterministic</termref>, <termref def=\"dt-context-independent\">context-independent</termref>,  and <termref def=\"dt-focus-independent\">focus-independent</termref>. </p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">General rules: see <specref ref=\"numeric-value-functions\"/>.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">If <code>$arg</code> is negative the function returns <code>-$arg</code>, otherwise it\n             returns <code>$arg</code>.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">If the type of <code>$arg</code> is one of the four numeric types <code>xs:float</code>,\n                <code>xs:double</code>, <code>xs:decimal</code> or <code>xs:integer</code> the type\n             of the result is the same as the type of <code>$arg</code>. If the type of\n                <code>$arg</code> is a type derived from one of the numeric types, the result is an\n             instance of the base numeric type.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">For <code>xs:float</code> and <code>xs:double</code> arguments, if the argument is\n             positive zero or negative zero, then positive zero is returned. If the argument is\n             positive or negative infinity, positive infinity is returned.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\" diff=\"del\" at=\"F\">For detailed type semantics, see [Formal Semantics].</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The expression <code>fn:abs(10.5)</code> returns <code>10.5</code>.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The expression <code>fn:abs(-10.5)</code> returns <code>10.5</code>.</p></div>\n", "summary" : "<p>  Returns the absolute value of  $arg .</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "arg", "type" : "numeric", "occurrence" : "?", "description" : "" } ], "returns" : { "type" : "numeric?", "description" : "" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "adjust-date-to-timezone", "qname" : "fn:adjust-date-to-timezone", "signature" : "($arg as xs:date?) as xs:date? external", "description" : " <div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">Adjusts an <code>xs:date</code> value to a specific timezone, or to no timezone\n             at all; the result is the date in the target timezone that contains the starting instant\n             of the supplied date.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\"><example role=\"signature\"><proto name=\"adjust-date-to-timezone\" return-type=\"xs:date?\" isOp=\"no\" prefix=\"fn\" returnEmptyOk=\"no\" returnSeq=\"no\" returnVaries=\"no\" isSchema=\"no\" isDatatype=\"no\" isSpecial=\"no\"><arg name=\"arg\" type=\"xs:date?\"/></proto></example><example role=\"signature\"><proto name=\"adjust-date-to-timezone\" return-type=\"xs:date?\" isOp=\"no\" prefix=\"fn\" returnEmptyOk=\"no\" returnSeq=\"no\" returnVaries=\"no\" isSchema=\"no\" isDatatype=\"no\" isSpecial=\"no\"><arg name=\"arg\" type=\"xs:date?\"/><arg name=\"timezone\" type=\"xs:dayTimeDuration?\"/></proto></example></p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The one-argument form of this function is <termref def=\"dt-deterministic\">deterministic</termref>, <termref def=\"dt-context-dependent\">context-dependent</termref>,  and <termref def=\"dt-focus-independent\">focus-independent</termref>.  It depends on\n \t\timplicit timezone.\n \t</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The two-argument form of this function is <termref def=\"dt-deterministic\">deterministic</termref>, <termref def=\"dt-context-independent\">context-independent</termref>,  and <termref def=\"dt-focus-independent\">focus-independent</termref>. </p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\"> If <code>$timezone</code> is not specified, then the effective value of\n                <code>$timezone</code> is the value of the implicit timezone in the dynamic\n             context.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\"> If <code>$arg</code> is the empty sequence, then the function returns the empty\n             sequence.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\"> If <code>$arg</code> does not have a timezone component and <code>$timezone</code> is\n             the empty sequence, then the result is the value of <code>$arg</code>.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\"> If <code>$arg</code> does not have a timezone component and <code>$timezone</code> is\n             not the empty sequence, then the result is <code>$arg</code> with <code>$timezone</code>\n             as the timezone component.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\"> If <code>$arg</code> has a timezone component and <code>$timezone</code> is the empty\n             sequence, then the result is the local value of <code>$arg</code> without its timezone\n             component.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\"> If <code>$arg</code> has a timezone component and <code>$timezone</code> is not the\n             empty sequence, then the function returns the value of the expression:</p><ulist xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\"><item><p>Let <code>$dt</code> be the value of <code>fn:dateTime($arg,\n                      xs:time('00:00:00'))</code>.</p></item><item><p>Let <code>$adt</code> be the value of <code>fn:adjust-dateTime-to-timezone($dt,\n                      $timezone)</code></p></item><item><p>The function returns the value of <code>xs:date($adt)</code></p></item></ulist><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">A <phrase diff=\"add\" at=\"L\">dynamic</phrase> error is raised <errorref class=\"DT\" code=\"0003\"/> if <code>$timezone</code> is less\n             than <code>-PT14H</code> or greater than <code>PT14H</code> or is not an integral number\n             of minutes.</p></div>\n", "summary" : "<p>  Adjusts an  xs:date  value to a specific timezone, or to no timezone\n             at all; the result is the date in the target timezone that contains the starting instant\n             of the supplied date.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "arg", "type" : "xs:date", "occurrence" : "?", "description" : "" } ], "returns" : { "type" : "xs:date?", "description" : "" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 2, "name" : "adjust-date-to-timezone", "qname" : "fn:adjust-date-to-timezone", "signature" : "($arg as xs:date?, $timezone as xs:dayTimeDuration?) as xs:date? external", "description" : " <div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">Adjusts an <code>xs:date</code> value to a specific timezone, or to no timezone\n             at all; the result is the date in the target timezone that contains the starting instant\n             of the supplied date.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\"><example role=\"signature\"><proto name=\"adjust-date-to-timezone\" return-type=\"xs:date?\" isOp=\"no\" prefix=\"fn\" returnEmptyOk=\"no\" returnSeq=\"no\" returnVaries=\"no\" isSchema=\"no\" isDatatype=\"no\" isSpecial=\"no\"><arg name=\"arg\" type=\"xs:date?\"/></proto></example><example role=\"signature\"><proto name=\"adjust-date-to-timezone\" return-type=\"xs:date?\" isOp=\"no\" prefix=\"fn\" returnEmptyOk=\"no\" returnSeq=\"no\" returnVaries=\"no\" isSchema=\"no\" isDatatype=\"no\" isSpecial=\"no\"><arg name=\"arg\" type=\"xs:date?\"/><arg name=\"timezone\" type=\"xs:dayTimeDuration?\"/></proto></example></p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The one-argument form of this function is <termref def=\"dt-deterministic\">deterministic</termref>, <termref def=\"dt-context-dependent\">context-dependent</termref>,  and <termref def=\"dt-focus-independent\">focus-independent</termref>.  It depends on\n \t\timplicit timezone.\n \t</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The two-argument form of this function is <termref def=\"dt-deterministic\">deterministic</termref>, <termref def=\"dt-context-independent\">context-independent</termref>,  and <termref def=\"dt-focus-independent\">focus-independent</termref>. </p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\"> If <code>$timezone</code> is not specified, then the effective value of\n                <code>$timezone</code> is the value of the implicit timezone in the dynamic\n             context.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\"> If <code>$arg</code> is the empty sequence, then the function returns the empty\n             sequence.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\"> If <code>$arg</code> does not have a timezone component and <code>$timezone</code> is\n             the empty sequence, then the result is the value of <code>$arg</code>.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\"> If <code>$arg</code> does not have a timezone component and <code>$timezone</code> is\n             not the empty sequence, then the result is <code>$arg</code> with <code>$timezone</code>\n             as the timezone component.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\"> If <code>$arg</code> has a timezone component and <code>$timezone</code> is the empty\n             sequence, then the result is the local value of <code>$arg</code> without its timezone\n             component.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\"> If <code>$arg</code> has a timezone component and <code>$timezone</code> is not the\n             empty sequence, then the function returns the value of the expression:</p><ulist xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\"><item><p>Let <code>$dt</code> be the value of <code>fn:dateTime($arg,\n                      xs:time('00:00:00'))</code>.</p></item><item><p>Let <code>$adt</code> be the value of <code>fn:adjust-dateTime-to-timezone($dt,\n                      $timezone)</code></p></item><item><p>The function returns the value of <code>xs:date($adt)</code></p></item></ulist><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">A <phrase diff=\"add\" at=\"L\">dynamic</phrase> error is raised <errorref class=\"DT\" code=\"0003\"/> if <code>$timezone</code> is less\n             than <code>-PT14H</code> or greater than <code>PT14H</code> or is not an integral number\n             of minutes.</p></div>\n", "summary" : "<p>  Adjusts an  xs:date  value to a specific timezone, or to no timezone\n             at all; the result is the date in the target timezone that contains the starting instant\n             of the supplied date.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "arg", "type" : "xs:date", "occurrence" : "?", "description" : "" }, { "name" : "timezone", "type" : "xs:dayTimeDuration", "occurrence" : "?", "description" : "" } ], "returns" : { "type" : "xs:date?", "description" : "" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "adjust-dateTime-to-timezone", "qname" : "fn:adjust-dateTime-to-timezone", "signature" : "($arg as xs:dateTime?) as xs:dateTime external", "description" : " <div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">Adjusts an <code>xs:dateTime</code> value to a specific timezone, or to no\n             timezone at all.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\"><example role=\"signature\"><proto name=\"adjust-dateTime-to-timezone\" return-type=\"xs:dateTime?\" isOp=\"no\" prefix=\"fn\" returnEmptyOk=\"no\" returnSeq=\"no\" returnVaries=\"no\" isSchema=\"no\" isDatatype=\"no\" isSpecial=\"no\"><arg name=\"arg\" type=\"xs:dateTime?\"/></proto></example><example role=\"signature\"><proto name=\"adjust-dateTime-to-timezone\" return-type=\"xs:dateTime?\" isOp=\"no\" prefix=\"fn\" returnEmptyOk=\"no\" returnSeq=\"no\" returnVaries=\"no\" isSchema=\"no\" isDatatype=\"no\" isSpecial=\"no\"><arg name=\"arg\" type=\"xs:dateTime?\"/><arg name=\"timezone\" type=\"xs:dayTimeDuration?\"/></proto></example></p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The one-argument form of this function is <termref def=\"dt-deterministic\">deterministic</termref>, <termref def=\"dt-context-dependent\">context-dependent</termref>,  and <termref def=\"dt-focus-independent\">focus-independent</termref>.  It depends on\n \t\timplicit timezone.\n \t</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The two-argument form of this function is <termref def=\"dt-deterministic\">deterministic</termref>, <termref def=\"dt-context-independent\">context-independent</termref>,  and <termref def=\"dt-focus-independent\">focus-independent</termref>. </p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\"> If <code>$timezone</code> is not specified, then the effective value of\n                <code>$timezone</code> is the value of the implicit timezone in the dynamic\n             context.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\"> If <code>$arg</code> is the empty sequence, then the function returns the empty\n             sequence.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\"> If <code>$arg</code> does not have a timezone component and <code>$timezone</code> is\n             the empty sequence, then the result is <code>$arg</code>.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\"> If <code>$arg</code> does not have a timezone component and <code>$timezone</code> is\n             not the empty sequence, then the result is <code>$arg</code> with <code>$timezone</code>\n             as the timezone component.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\"> If <code>$arg</code> has a timezone component and <code>$timezone</code> is the empty\n             sequence, then the result is the local value of <code>$arg</code> without its timezone\n             component.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\"> If <code>$arg</code> has a timezone component and <code>$timezone</code> is not the\n             empty sequence, then the result is the <code>xs:dateTime</code> value that is equal to\n                <code>$arg</code> and that has a timezone component equal to\n             <code>$timezone</code>.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\"> A <phrase diff=\"add\" at=\"L\">dynamic</phrase> error is raised <errorref class=\"DT\" code=\"0003\"/> if <code>$timezone</code> is less\n             than <code>-PT14H</code> or greater than <code>PT14H</code> or is not an integral number\n             of minutes.</p></div>\n", "summary" : "<p>  Adjusts an  xs:dateTime  value to a specific timezone, or to no\n             timezone at all.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "arg", "type" : "xs:dateTime", "occurrence" : "?", "description" : "" } ], "returns" : { "type" : "xs:dateTime", "description" : "" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 2, "name" : "adjust-dateTime-to-timezone", "qname" : "fn:adjust-dateTime-to-timezone", "signature" : "($arg as xs:dateTime?, $timezone as xs:dayTimeDuration?) as xs:dateTime external", "description" : " <div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">Adjusts an <code>xs:dateTime</code> value to a specific timezone, or to no\n             timezone at all.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\"><example role=\"signature\"><proto name=\"adjust-dateTime-to-timezone\" return-type=\"xs:dateTime?\" isOp=\"no\" prefix=\"fn\" returnEmptyOk=\"no\" returnSeq=\"no\" returnVaries=\"no\" isSchema=\"no\" isDatatype=\"no\" isSpecial=\"no\"><arg name=\"arg\" type=\"xs:dateTime?\"/></proto></example><example role=\"signature\"><proto name=\"adjust-dateTime-to-timezone\" return-type=\"xs:dateTime?\" isOp=\"no\" prefix=\"fn\" returnEmptyOk=\"no\" returnSeq=\"no\" returnVaries=\"no\" isSchema=\"no\" isDatatype=\"no\" isSpecial=\"no\"><arg name=\"arg\" type=\"xs:dateTime?\"/><arg name=\"timezone\" type=\"xs:dayTimeDuration?\"/></proto></example></p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The one-argument form of this function is <termref def=\"dt-deterministic\">deterministic</termref>, <termref def=\"dt-context-dependent\">context-dependent</termref>,  and <termref def=\"dt-focus-independent\">focus-independent</termref>.  It depends on\n \t\timplicit timezone.\n \t</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The two-argument form of this function is <termref def=\"dt-deterministic\">deterministic</termref>, <termref def=\"dt-context-independent\">context-independent</termref>,  and <termref def=\"dt-focus-independent\">focus-independent</termref>. </p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\"> If <code>$timezone</code> is not specified, then the effective value of\n                <code>$timezone</code> is the value of the implicit timezone in the dynamic\n             context.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\"> If <code>$arg</code> is the empty sequence, then the function returns the empty\n             sequence.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\"> If <code>$arg</code> does not have a timezone component and <code>$timezone</code> is\n             the empty sequence, then the result is <code>$arg</code>.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\"> If <code>$arg</code> does not have a timezone component and <code>$timezone</code> is\n             not the empty sequence, then the result is <code>$arg</code> with <code>$timezone</code>\n             as the timezone component.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\"> If <code>$arg</code> has a timezone component and <code>$timezone</code> is the empty\n             sequence, then the result is the local value of <code>$arg</code> without its timezone\n             component.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\"> If <code>$arg</code> has a timezone component and <code>$timezone</code> is not the\n             empty sequence, then the result is the <code>xs:dateTime</code> value that is equal to\n                <code>$arg</code> and that has a timezone component equal to\n             <code>$timezone</code>.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\"> A <phrase diff=\"add\" at=\"L\">dynamic</phrase> error is raised <errorref class=\"DT\" code=\"0003\"/> if <code>$timezone</code> is less\n             than <code>-PT14H</code> or greater than <code>PT14H</code> or is not an integral number\n             of minutes.</p></div>\n", "summary" : "<p>  Adjusts an  xs:dateTime  value to a specific timezone, or to no\n             timezone at all.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "arg", "type" : "xs:dateTime", "occurrence" : "?", "description" : "" }, { "name" : "timezone", "type" : "xs:dayTimeDuration", "occurrence" : "?", "description" : "" } ], "returns" : { "type" : "xs:dateTime", "description" : "" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "adjust-time-to-timezone", "qname" : "fn:adjust-time-to-timezone", "signature" : "($arg as xs:time?) as xs:time? external", "description" : " <div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">Adjusts an <code>xs:time</code> value to a specific timezone, or to no timezone\n             at all.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\"><example role=\"signature\"><proto name=\"adjust-time-to-timezone\" return-type=\"xs:time?\" isOp=\"no\" prefix=\"fn\" returnEmptyOk=\"no\" returnSeq=\"no\" returnVaries=\"no\" isSchema=\"no\" isDatatype=\"no\" isSpecial=\"no\"><arg name=\"arg\" type=\"xs:time?\"/></proto></example><example role=\"signature\"><proto name=\"adjust-time-to-timezone\" return-type=\"xs:time?\" isOp=\"no\" prefix=\"fn\" returnEmptyOk=\"no\" returnSeq=\"no\" returnVaries=\"no\" isSchema=\"no\" isDatatype=\"no\" isSpecial=\"no\"><arg name=\"arg\" type=\"xs:time?\"/><arg name=\"timezone\" type=\"xs:dayTimeDuration?\"/></proto></example></p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The one-argument form of this function is <termref def=\"dt-deterministic\">deterministic</termref>, <termref def=\"dt-context-dependent\">context-dependent</termref>,  and <termref def=\"dt-focus-independent\">focus-independent</termref>.  It depends on\n \t\timplicit timezone.\n \t</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The two-argument form of this function is <termref def=\"dt-deterministic\">deterministic</termref>, <termref def=\"dt-context-independent\">context-independent</termref>,  and <termref def=\"dt-focus-independent\">focus-independent</termref>. </p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\"> If <code>$timezone</code> is not specified, then the effective value of\n                <code>$timezone</code> is the value of the implicit timezone in the dynamic\n             context.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\"> If <code>$arg</code> is the empty sequence, then the function returns the empty\n             sequence.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\"> If <code>$arg</code> does not have a timezone component and <code>$timezone</code> is\n             the empty sequence, then the result is <code>$arg</code>.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\"> If <code>$arg</code> does not have a timezone component and <code>$timezone</code> is\n             not the empty sequence, then the result is <code>$arg</code> with <code>$timezone</code>\n             as the timezone component.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\"> If <code>$arg</code> has a timezone component and <code>$timezone</code> is the empty\n             sequence, then the result is the localized value of <code>$arg</code> without its\n             timezone component.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\"> If <code>$arg</code> has a timezone component and <code>$timezone</code> is not the\n             empty sequence, then:</p><ulist xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\"><item><p>Let <code>$dt</code> be the <code>xs:dateTime</code> value\n                      <code>fn:dateTime(xs:date('1972-12-31'), $arg)</code>.</p></item><item><p>Let <code>$adt</code> be the value of <code>fn:adjust-dateTime-to-timezone($dt,\n                      $timezone)</code>\n                </p></item><item><p>The function returns the <code>xs:time</code> value\n                   <code>xs:time($adt)</code>.</p></item></ulist><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">A <phrase diff=\"add\" at=\"L\">dynamic</phrase> error is raised <errorref class=\"DT\" code=\"0003\"/> if <code>$timezone</code> is less\n             than <code>-PT14H</code> or greater than <code>PT14H</code> or if does not contain an\n             integral number of minutes.</p></div>\n", "summary" : "<p>  Adjusts an  xs:time  value to a specific timezone, or to no timezone\n             at all.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "arg", "type" : "xs:time", "occurrence" : "?", "description" : "" } ], "returns" : { "type" : "xs:time?", "description" : "" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 2, "name" : "adjust-time-to-timezone", "qname" : "fn:adjust-time-to-timezone", "signature" : "($arg as xs:time?, $timezone as xs:dayTimeDuration?) as xs:time? external", "description" : " <div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">Adjusts an <code>xs:time</code> value to a specific timezone, or to no timezone\n             at all.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\"><example role=\"signature\"><proto name=\"adjust-time-to-timezone\" return-type=\"xs:time?\" isOp=\"no\" prefix=\"fn\" returnEmptyOk=\"no\" returnSeq=\"no\" returnVaries=\"no\" isSchema=\"no\" isDatatype=\"no\" isSpecial=\"no\"><arg name=\"arg\" type=\"xs:time?\"/></proto></example><example role=\"signature\"><proto name=\"adjust-time-to-timezone\" return-type=\"xs:time?\" isOp=\"no\" prefix=\"fn\" returnEmptyOk=\"no\" returnSeq=\"no\" returnVaries=\"no\" isSchema=\"no\" isDatatype=\"no\" isSpecial=\"no\"><arg name=\"arg\" type=\"xs:time?\"/><arg name=\"timezone\" type=\"xs:dayTimeDuration?\"/></proto></example></p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The one-argument form of this function is <termref def=\"dt-deterministic\">deterministic</termref>, <termref def=\"dt-context-dependent\">context-dependent</termref>,  and <termref def=\"dt-focus-independent\">focus-independent</termref>.  It depends on\n \t\timplicit timezone.\n \t</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The two-argument form of this function is <termref def=\"dt-deterministic\">deterministic</termref>, <termref def=\"dt-context-independent\">context-independent</termref>,  and <termref def=\"dt-focus-independent\">focus-independent</termref>. </p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\"> If <code>$timezone</code> is not specified, then the effective value of\n                <code>$timezone</code> is the value of the implicit timezone in the dynamic\n             context.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\"> If <code>$arg</code> is the empty sequence, then the function returns the empty\n             sequence.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\"> If <code>$arg</code> does not have a timezone component and <code>$timezone</code> is\n             the empty sequence, then the result is <code>$arg</code>.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\"> If <code>$arg</code> does not have a timezone component and <code>$timezone</code> is\n             not the empty sequence, then the result is <code>$arg</code> with <code>$timezone</code>\n             as the timezone component.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\"> If <code>$arg</code> has a timezone component and <code>$timezone</code> is the empty\n             sequence, then the result is the localized value of <code>$arg</code> without its\n             timezone component.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\"> If <code>$arg</code> has a timezone component and <code>$timezone</code> is not the\n             empty sequence, then:</p><ulist xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\"><item><p>Let <code>$dt</code> be the <code>xs:dateTime</code> value\n                      <code>fn:dateTime(xs:date('1972-12-31'), $arg)</code>.</p></item><item><p>Let <code>$adt</code> be the value of <code>fn:adjust-dateTime-to-timezone($dt,\n                      $timezone)</code>\n                </p></item><item><p>The function returns the <code>xs:time</code> value\n                   <code>xs:time($adt)</code>.</p></item></ulist><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">A <phrase diff=\"add\" at=\"L\">dynamic</phrase> error is raised <errorref class=\"DT\" code=\"0003\"/> if <code>$timezone</code> is less\n             than <code>-PT14H</code> or greater than <code>PT14H</code> or if does not contain an\n             integral number of minutes.</p></div>\n", "summary" : "<p>  Adjusts an  xs:time  value to a specific timezone, or to no timezone\n             at all.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "arg", "type" : "xs:time", "occurrence" : "?", "description" : "" }, { "name" : "timezone", "type" : "xs:dayTimeDuration", "occurrence" : "?", "description" : "" } ], "returns" : { "type" : "xs:time?", "description" : "" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 2, "name" : "analyze-string", "qname" : "fn:analyze-string", "signature" : "($input as xs:string?, $pattern as xs:string) as element(fn:analyze-string-result) external", "description" : " <div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">Analyzes a string using a regular expression, returning an XML structure that\n             identifies which parts of the input string matched or failed to match the regular\n             expression, and in the case of matched substrings, which substrings matched each\n             capturing group in the regular expression.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\"><example role=\"signature\"><proto name=\"analyze-string\" return-type=\"element(fn:analyze-string-result)\" isOp=\"no\" prefix=\"fn\" returnEmptyOk=\"no\" returnSeq=\"no\" returnVaries=\"no\" isSchema=\"no\" isDatatype=\"no\" isSpecial=\"no\"><arg name=\"input\" type=\"xs:string?\"/><arg name=\"pattern\" type=\"xs:string\"/></proto></example><example role=\"signature\"><proto name=\"analyze-string\" return-type=\"element(fn:analyze-string-result)\" isOp=\"no\" prefix=\"fn\" returnEmptyOk=\"no\" returnSeq=\"no\" returnVaries=\"no\" isSchema=\"no\" isDatatype=\"no\" isSpecial=\"no\"><arg name=\"input\" type=\"xs:string?\"/><arg name=\"pattern\" type=\"xs:string\"/><arg name=\"flags\" type=\"xs:string\"/></proto></example></p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">This function is <termref def=\"dt-nondeterministic\">nondeterministic</termref>, <termref def=\"dt-context-independent\">context-independent</termref>,  and <termref def=\"dt-focus-independent\">focus-independent</termref>. </p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The effect of calling the first version of this function (omitting the argument\n                <code>$flags</code>) is the same as the effect of calling the second version with the\n                <code>$flags</code> argument set to a zero-length string. Flags are defined in\n                <specref ref=\"flags\"/>.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The <code>$flags</code> argument is interpreted in the same way as for the\n                <code>fn:matches</code> function.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">If <code>$input</code> is the empty sequence the function behaves as if\n                <code>$input</code> were the zero-length string. In this situation the result will be\n             an element node with no children.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The function returns an element node whose local name is\n                <code>analyze-string-result</code>. This element and all its descendant elements have\n             the namespace URI <code>http://www.w3.org/2005/xpath-functions</code>. The namespace\n             prefix is <termref def=\"implementation-dependent\"/>. The children of this element are a\n             sequence of <code>fn:match</code> and <code>fn:non-match</code> elements. This sequence\n             is formed by breaking the <code>$input</code> string into a sequence of strings,\n             returning any substring that matches <code>$pattern</code> as the content of a\n                <code>match</code> element, and any intervening substring as the content of a\n                <code>non-match</code> element.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">More specifically, the function starts at the beginning of the input string and attempts\n             to find the first substring that matches the regular expression. If there are several\n             matches, the first match is defined to be the one whose starting position comes first in\n             the string. If several alternatives within the regular expression both match at the same\n             position in the input string, then the match that is chosen is the first alternative\n             that matches. For example, if the input string is <code>The quick brown fox jumps</code>\n             and the regular expression is <code>jump|jumps</code>, then the match that is chosen is\n                <code>jump</code>.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">Having found the first match, the instruction proceeds to find the second and subsequent\n             matches by repeating the search, starting at the first <termref def=\"character\">character</termref> that was not included in the previous match.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The input string is thus partitioned into a sequence of substrings, some of which match\n             the regular expression, others which do not match it. Each substring will contain at\n             least one character. This sequence is represented in the result by the sequence of\n                <code>fn:match</code> and <code>fn:non-match</code> children of the returned element\n             node; the string value of the <code>fn:match</code> or <code>fn:non-match</code> element\n             will be the corresponding substring of <code>$input</code>, and the string value of the\n             returned element node will therefore be the same as <code>$input</code>.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The content of an <code>fn:non-match</code> element is always a single text node.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The content of a <code>fn:match</code> element, however, is in general a sequence of\n             text nodes and <code>fn:group</code> element children. An <code>fn:group</code> element\n             with a <code>nr</code> attribute having the integer value <var>N</var> identifies the\n             substring captured by the <var>Nth</var> parenthesized sub-expression in the regular\n             expression. For each capturing subexpression there will be at most one corresponding\n                <code>fn:group</code> element in each <code>fn:match</code> element in the\n             result.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">If the function is called twice with the same arguments, it is <termref def=\"implementation-dependent\"/> whether the two calls return the same element node\n             or distinct (but deep equal) element nodes. In this respect it is\n             <termref def=\"nondeterministic\">nondeterministic</termref>.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\" diff=\"add\" at=\"J\">The base URI of the element nodes in the result is\n          <termref def=\"implementation-dependent\"/></p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">A schema is defined for the structure of the returned element, containing the\n             definitions below. The returned element and its descendants will have type annotations\n             obtained by validating the returned element against this schema, unless the function is\n             used in an environment where type annotations are not supported (for example, a Basic\n             XSLT Processor), in which case the elements will all be annotated as\n                <code>xs:untyped</code> and the attributes as <code>xs:untypedAtomic</code>.</p><note xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\"><p diff=\"add\" at=\"M\">A free-standing copy of this schema can be found at <loc xmlns:xlink=\"http://www.w3.org/1999/xlink\" href=\"analyze-string.xsd\" xlink:type=\"simple\" xlink:show=\"replace\" xlink:actuate=\"onRequest\">analyze-string.xsd</loc></p></note><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">\n             <eg xml:space=\"preserve\">&lt;?xml version=\"1.0\" encoding=\"UTF-8\"?&gt;\n &lt;xs:schema xmlns:xs=\"http://www.w3.org/2001/XMLSchema\"\n     targetNamespace=\"http://www.w3.org/2005/xpath-functions\"\n     xmlns:fn=\"http://www.w3.org/2005/xpath-functions\"\n     elementFormDefault=\"qualified\"&gt;\n     &lt;xs:element name=\"analyze-string-result\" type=\"fn:analyze-string-result-type\"/&gt;\n     &lt;xs:element name=\"match\" type=\"fn:match-type\"/&gt;\n     &lt;xs:element name=\"non-match\" type=\"xs:string\"/&gt;\n     &lt;xs:element name=\"group\" type=\"fn:group-type\"/&gt;\n     &lt;xs:complexType name=\"analyze-string-result-type\" mixed=\"true\"&gt;\n         &lt;xs:choice minOccurs=\"0\" maxOccurs=\"unbounded\"&gt;\n             &lt;xs:element ref=\"fn:match\"/&gt;\n             &lt;xs:element ref=\"fn:non-match\"/&gt;\n         &lt;/xs:choice&gt;\n     &lt;/xs:complexType&gt;\n     &lt;xs:complexType name=\"match-type\" mixed=\"true\"&gt;\n         &lt;xs:sequence&gt;\n             &lt;xs:element ref=\"fn:group\" minOccurs=\"0\" maxOccurs=\"unbounded\"/&gt;\n         &lt;/xs:sequence&gt;\n     &lt;/xs:complexType&gt;\n     &lt;xs:complexType name=\"group-type\" mixed=\"true\"&gt;\n         &lt;xs:sequence&gt;\n             &lt;xs:element ref=\"fn:group\" minOccurs=\"0\" maxOccurs=\"unbounded\"/&gt;\n         &lt;/xs:sequence&gt;\n         &lt;xs:attribute name=\"nr\" type=\"xs:positiveInteger\"/&gt;\n     &lt;/xs:complexType&gt;\n &lt;/xs:schema&gt;\n </eg>\n          </p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">A <phrase diff=\"add\" at=\"L\">dynamic</phrase> error is raised <errorref class=\"RX\" code=\"0002\"/> if the value of\n                <code>$pattern</code> is invalid according to the rules described in section <specref ref=\"regex-syntax\"/>.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">A <phrase diff=\"add\" at=\"L\">dynamic</phrase> error is raised <errorref class=\"RX\" code=\"0001\"/> if the value of\n                <code>$flags</code> is invalid according to the rules described in section <specref ref=\"regex-syntax\"/>.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">A <phrase diff=\"add\" at=\"L\">dynamic</phrase> error is raised <errorref class=\"RX\" code=\"0003\"/> if the supplied\n                <code>$pattern</code> matches a zero-length string, that is, if <code>fn:matches(\"\",\n                $pattern, $flags)</code> returns <code>true</code>.</p></div>\n", "summary" : "<p>  Analyzes a string using a regular expression, returning an XML structure that\n             identifies which parts of the input string matched or failed to match the regular\n             expression, and in the case of matched substrings, which substrings matched each\n             capturing group in the regular expression.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "input", "type" : "xs:string", "occurrence" : "?", "description" : "" }, { "name" : "pattern", "type" : "xs:string", "occurrence" : null, "description" : "" } ], "returns" : { "type" : "element(fn:analyze-string-result)", "description" : "" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 3, "name" : "analyze-string", "qname" : "fn:analyze-string", "signature" : "($input as xs:string?, $pattern as xs:string, $flags as xs:string) as element(fn:analyze-string-result) external", "description" : " <div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">Analyzes a string using a regular expression, returning an XML structure that\n             identifies which parts of the input string matched or failed to match the regular\n             expression, and in the case of matched substrings, which substrings matched each\n             capturing group in the regular expression.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\"><example role=\"signature\"><proto name=\"analyze-string\" return-type=\"element(fn:analyze-string-result)\" isOp=\"no\" prefix=\"fn\" returnEmptyOk=\"no\" returnSeq=\"no\" returnVaries=\"no\" isSchema=\"no\" isDatatype=\"no\" isSpecial=\"no\"><arg name=\"input\" type=\"xs:string?\"/><arg name=\"pattern\" type=\"xs:string\"/></proto></example><example role=\"signature\"><proto name=\"analyze-string\" return-type=\"element(fn:analyze-string-result)\" isOp=\"no\" prefix=\"fn\" returnEmptyOk=\"no\" returnSeq=\"no\" returnVaries=\"no\" isSchema=\"no\" isDatatype=\"no\" isSpecial=\"no\"><arg name=\"input\" type=\"xs:string?\"/><arg name=\"pattern\" type=\"xs:string\"/><arg name=\"flags\" type=\"xs:string\"/></proto></example></p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">This function is <termref def=\"dt-nondeterministic\">nondeterministic</termref>, <termref def=\"dt-context-independent\">context-independent</termref>,  and <termref def=\"dt-focus-independent\">focus-independent</termref>. </p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The effect of calling the first version of this function (omitting the argument\n                <code>$flags</code>) is the same as the effect of calling the second version with the\n                <code>$flags</code> argument set to a zero-length string. Flags are defined in\n                <specref ref=\"flags\"/>.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The <code>$flags</code> argument is interpreted in the same way as for the\n                <code>fn:matches</code> function.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">If <code>$input</code> is the empty sequence the function behaves as if\n                <code>$input</code> were the zero-length string. In this situation the result will be\n             an element node with no children.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The function returns an element node whose local name is\n                <code>analyze-string-result</code>. This element and all its descendant elements have\n             the namespace URI <code>http://www.w3.org/2005/xpath-functions</code>. The namespace\n             prefix is <termref def=\"implementation-dependent\"/>. The children of this element are a\n             sequence of <code>fn:match</code> and <code>fn:non-match</code> elements. This sequence\n             is formed by breaking the <code>$input</code> string into a sequence of strings,\n             returning any substring that matches <code>$pattern</code> as the content of a\n                <code>match</code> element, and any intervening substring as the content of a\n                <code>non-match</code> element.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">More specifically, the function starts at the beginning of the input string and attempts\n             to find the first substring that matches the regular expression. If there are several\n             matches, the first match is defined to be the one whose starting position comes first in\n             the string. If several alternatives within the regular expression both match at the same\n             position in the input string, then the match that is chosen is the first alternative\n             that matches. For example, if the input string is <code>The quick brown fox jumps</code>\n             and the regular expression is <code>jump|jumps</code>, then the match that is chosen is\n                <code>jump</code>.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">Having found the first match, the instruction proceeds to find the second and subsequent\n             matches by repeating the search, starting at the first <termref def=\"character\">character</termref> that was not included in the previous match.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The input string is thus partitioned into a sequence of substrings, some of which match\n             the regular expression, others which do not match it. Each substring will contain at\n             least one character. This sequence is represented in the result by the sequence of\n                <code>fn:match</code> and <code>fn:non-match</code> children of the returned element\n             node; the string value of the <code>fn:match</code> or <code>fn:non-match</code> element\n             will be the corresponding substring of <code>$input</code>, and the string value of the\n             returned element node will therefore be the same as <code>$input</code>.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The content of an <code>fn:non-match</code> element is always a single text node.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The content of a <code>fn:match</code> element, however, is in general a sequence of\n             text nodes and <code>fn:group</code> element children. An <code>fn:group</code> element\n             with a <code>nr</code> attribute having the integer value <var>N</var> identifies the\n             substring captured by the <var>Nth</var> parenthesized sub-expression in the regular\n             expression. For each capturing subexpression there will be at most one corresponding\n                <code>fn:group</code> element in each <code>fn:match</code> element in the\n             result.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">If the function is called twice with the same arguments, it is <termref def=\"implementation-dependent\"/> whether the two calls return the same element node\n             or distinct (but deep equal) element nodes. In this respect it is\n             <termref def=\"nondeterministic\">nondeterministic</termref>.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\" diff=\"add\" at=\"J\">The base URI of the element nodes in the result is\n          <termref def=\"implementation-dependent\"/></p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">A schema is defined for the structure of the returned element, containing the\n             definitions below. The returned element and its descendants will have type annotations\n             obtained by validating the returned element against this schema, unless the function is\n             used in an environment where type annotations are not supported (for example, a Basic\n             XSLT Processor), in which case the elements will all be annotated as\n                <code>xs:untyped</code> and the attributes as <code>xs:untypedAtomic</code>.</p><note xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\"><p diff=\"add\" at=\"M\">A free-standing copy of this schema can be found at <loc xmlns:xlink=\"http://www.w3.org/1999/xlink\" href=\"analyze-string.xsd\" xlink:type=\"simple\" xlink:show=\"replace\" xlink:actuate=\"onRequest\">analyze-string.xsd</loc></p></note><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">\n             <eg xml:space=\"preserve\">&lt;?xml version=\"1.0\" encoding=\"UTF-8\"?&gt;\n &lt;xs:schema xmlns:xs=\"http://www.w3.org/2001/XMLSchema\"\n     targetNamespace=\"http://www.w3.org/2005/xpath-functions\"\n     xmlns:fn=\"http://www.w3.org/2005/xpath-functions\"\n     elementFormDefault=\"qualified\"&gt;\n     &lt;xs:element name=\"analyze-string-result\" type=\"fn:analyze-string-result-type\"/&gt;\n     &lt;xs:element name=\"match\" type=\"fn:match-type\"/&gt;\n     &lt;xs:element name=\"non-match\" type=\"xs:string\"/&gt;\n     &lt;xs:element name=\"group\" type=\"fn:group-type\"/&gt;\n     &lt;xs:complexType name=\"analyze-string-result-type\" mixed=\"true\"&gt;\n         &lt;xs:choice minOccurs=\"0\" maxOccurs=\"unbounded\"&gt;\n             &lt;xs:element ref=\"fn:match\"/&gt;\n             &lt;xs:element ref=\"fn:non-match\"/&gt;\n         &lt;/xs:choice&gt;\n     &lt;/xs:complexType&gt;\n     &lt;xs:complexType name=\"match-type\" mixed=\"true\"&gt;\n         &lt;xs:sequence&gt;\n             &lt;xs:element ref=\"fn:group\" minOccurs=\"0\" maxOccurs=\"unbounded\"/&gt;\n         &lt;/xs:sequence&gt;\n     &lt;/xs:complexType&gt;\n     &lt;xs:complexType name=\"group-type\" mixed=\"true\"&gt;\n         &lt;xs:sequence&gt;\n             &lt;xs:element ref=\"fn:group\" minOccurs=\"0\" maxOccurs=\"unbounded\"/&gt;\n         &lt;/xs:sequence&gt;\n         &lt;xs:attribute name=\"nr\" type=\"xs:positiveInteger\"/&gt;\n     &lt;/xs:complexType&gt;\n &lt;/xs:schema&gt;\n </eg>\n          </p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">A <phrase diff=\"add\" at=\"L\">dynamic</phrase> error is raised <errorref class=\"RX\" code=\"0002\"/> if the value of\n                <code>$pattern</code> is invalid according to the rules described in section <specref ref=\"regex-syntax\"/>.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">A <phrase diff=\"add\" at=\"L\">dynamic</phrase> error is raised <errorref class=\"RX\" code=\"0001\"/> if the value of\n                <code>$flags</code> is invalid according to the rules described in section <specref ref=\"regex-syntax\"/>.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">A <phrase diff=\"add\" at=\"L\">dynamic</phrase> error is raised <errorref class=\"RX\" code=\"0003\"/> if the supplied\n                <code>$pattern</code> matches a zero-length string, that is, if <code>fn:matches(\"\",\n                $pattern, $flags)</code> returns <code>true</code>.</p></div>\n", "summary" : "<p>  Analyzes a string using a regular expression, returning an XML structure that\n             identifies which parts of the input string matched or failed to match the regular\n             expression, and in the case of matched substrings, which substrings matched each\n             capturing group in the regular expression.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "input", "type" : "xs:string", "occurrence" : "?", "description" : "" }, { "name" : "pattern", "type" : "xs:string", "occurrence" : null, "description" : "" }, { "name" : "flags", "type" : "xs:string", "occurrence" : null, "description" : "" } ], "returns" : { "type" : "element(fn:analyze-string-result)", "description" : "" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 0, "name" : "available-environment-variables", "qname" : "fn:available-environment-variables", "signature" : "() as xs:string* external", "description" : " <div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">Returns a list of environment variable names that are suitable for passing to\n                <code>fn:environment-variable</code>, as a (possibly empty) sequence of strings.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\"><example role=\"signature\"><proto name=\"available-environment-variables\" return-type=\"xs:string*\" isOp=\"no\" prefix=\"fn\" returnEmptyOk=\"no\" returnSeq=\"no\" returnVaries=\"no\" isSchema=\"no\" isDatatype=\"no\" isSpecial=\"no\"/></example></p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">This function is <termref def=\"dt-deterministic\">deterministic</termref>, <termref def=\"dt-context-dependent\">context-dependent</termref>,  and <termref def=\"dt-focus-independent\">focus-independent</termref>.  It depends on\n \t\tenvironment variables.\n \t</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The function returns a sequence of strings, being the names of the environment variables\n             in the dynamic context in some <termref def=\"implementation-dependent\">implementation-dependent</termref> order.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The function is <termref def=\"dt-deterministic\">deterministic</termref>: that is, the\n             set of available environment variables does not vary during evaluation.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The function returns a list of strings, containing no duplicates.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">It is intended that the strings in this list should be suitable for passing to\n                <code>fn:environment-variable</code>.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">See also the note on security under the definition of the\n                <code>fn:environment-variable</code> function. If access to environment variables has\n             been disabled, <code>fn:available-environment-variables</code> always returns the empty\n             sequence.</p></div>\n", "summary" : "<p>  Returns a list of environment variable names that are suitable for passing to\n                 fn:environment-variable , as a (possibly empty) sequence of strings.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [  ], "returns" : { "type" : "xs:string*", "description" : "" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 0, "name" : "available-environment-variables", "qname" : "fn:available-environment-variables", "signature" : "() as xs:string* external", "description" : " <div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">Returns a list of environment variable names that are suitable for passing to\n                <code>fn:environment-variable</code>, as a (possibly empty) sequence of strings.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\"><example role=\"signature\"><proto name=\"available-environment-variables\" return-type=\"xs:string*\" isOp=\"no\" prefix=\"fn\" returnEmptyOk=\"no\" returnSeq=\"no\" returnVaries=\"no\" isSchema=\"no\" isDatatype=\"no\" isSpecial=\"no\"/></example></p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">This function is <termref def=\"dt-deterministic\">deterministic</termref>, <termref def=\"dt-context-dependent\">context-dependent</termref>,  and <termref def=\"dt-focus-independent\">focus-independent</termref>.  It depends on\n \t\tenvironment variables.\n \t</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The function returns a sequence of strings, being the names of the environment variables\n             in the dynamic context in some <termref def=\"implementation-dependent\">implementation-dependent</termref> order.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The function is <termref def=\"dt-deterministic\">deterministic</termref>: that is, the\n             set of available environment variables does not vary during evaluation.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The function returns a list of strings, containing no duplicates.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">It is intended that the strings in this list should be suitable for passing to\n                <code>fn:environment-variable</code>.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">See also the note on security under the definition of the\n                <code>fn:environment-variable</code> function. If access to environment variables has\n             been disabled, <code>fn:available-environment-variables</code> always returns the empty\n             sequence.</p></div>\n", "summary" : "<p>  Returns a list of environment variable names that are suitable for passing to\n                 fn:environment-variable , as a (possibly empty) sequence of strings.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [  ], "returns" : { "type" : "xs:string*", "description" : "" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "avg", "qname" : "fn:avg", "signature" : "($arg as xs:anyAtomicType*) as xs:anyAtomicType? external", "description" : " <div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">Returns the average of the values in the input sequence <code>$arg</code>, that\n             is, the sum of the values divided by the number of values.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\"><example role=\"signature\"><proto name=\"avg\" return-type=\"xs:anyAtomicType?\" isOp=\"no\" prefix=\"fn\" returnEmptyOk=\"no\" returnSeq=\"no\" returnVaries=\"no\" isSchema=\"no\" isDatatype=\"no\" isSpecial=\"no\"><arg name=\"arg\" type=\"xs:anyAtomicType*\"/></proto></example></p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">This function is <termref def=\"dt-deterministic\">deterministic</termref>, <termref def=\"dt-context-independent\">context-independent</termref>,  and <termref def=\"dt-focus-independent\">focus-independent</termref>. </p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">If <code>$arg</code> is the empty sequence, the empty sequence is returned.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">If <code>$arg</code> contains values of type <code>xs:untypedAtomic</code> they are cast\n             to <code>xs:double</code>. </p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">Duration values must either all be <code>xs:yearMonthDuration</code> values or must all\n             be <code>xs:dayTimeDuration</code> values. For numeric values, the numeric promotion\n             rules defined in <specref ref=\"op.numeric\"/> are used to promote all values to a single\n             common type. After these operations, <code>$arg</code> must contain items of a single\n             type, which must be one of the four numeric types, <code>xs:yearMonthDuration</code> or\n                <code>xs:dayTimeDuration</code> or one if its subtypes.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The function returns the average of the values as <code>sum($arg) div\n             count($arg)</code>; but the implementation may use an otherwise equivalent algorithm\n             that avoids arithmetic overflow.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\" diff=\"del\" at=\"F\">For detailed type semantics, see [Formal Semantics].</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">A type error is raised <errorref class=\"RG\" code=\"0006\"/> if the input sequence contains\n             items of incompatible types, as described above.</p></div>\n", "summary" : "<p>  Returns the average of the values in the input sequence  $arg , that\n             is, the sum of the values divided by the number of values.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "arg", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "" } ], "returns" : { "type" : "xs:anyAtomicType?", "description" : "" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 0, "name" : "base-uri", "qname" : "fn:base-uri", "signature" : "() as xs:anyURI? external", "description" : " <div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">Returns the base URI of a node.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\"><example role=\"signature\"><proto name=\"base-uri\" return-type=\"xs:anyURI?\" isOp=\"no\" prefix=\"fn\" returnEmptyOk=\"no\" returnSeq=\"no\" returnVaries=\"no\" isSchema=\"no\" isDatatype=\"no\" isSpecial=\"no\"/></example><example role=\"signature\"><proto name=\"base-uri\" return-type=\"xs:anyURI?\" isOp=\"no\" prefix=\"fn\" returnEmptyOk=\"no\" returnSeq=\"no\" returnVaries=\"no\" isSchema=\"no\" isDatatype=\"no\" isSpecial=\"no\"><arg name=\"arg\" type=\"node()?\"/></proto></example></p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The zero-argument form of this function is <termref def=\"dt-deterministic\">deterministic</termref>, <termref def=\"dt-context-dependent\">context-dependent</termref>,  and <termref def=\"dt-focus-dependent\">focus-dependent</termref>. </p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The one-argument form of this function is <termref def=\"dt-deterministic\">deterministic</termref>, <termref def=\"dt-context-independent\">context-independent</termref>,  and <termref def=\"dt-focus-independent\">focus-independent</termref>. </p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\" diff=\"add\" at=\"B\">The zero-argument version of the function returns the base URI of the\n             context node: it is equivalent to calling <code>fn:base-uri(.)</code>.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\" diff=\"add\" at=\"B\">The single-argument version of the function behaves as follows:</p><olist xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\"><item>If <code>$arg</code> is the empty sequence, the function returns the empty\n                sequence.</item><item>Otherwise, the function returns the value of the <code>dm:base-uri</code> accessor\n                applied to the node <code>$arg</code>. This accessor is defined, for each kind of\n                node, in the XDM specification (See <xspecref spec=\"DM30\" ref=\"dm-base-uri\"/>).</item></olist><note xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">As explained in XDM, document, element and processing-instruction nodes have a\n             base-uri property which may be empty. The base-uri property for all other node kinds is\n             the empty sequence. The dm:base-uri accessor returns the base-uri property of a node if\n             it exists and is non-empty; otherwise it returns the result of applying the dm:base-uri\n             accessor to its parent, recursively. If the node does not have a parent, or if the\n             recursive ascent up the ancestor chain encounters a parentless node whose base-uri\n             property is empty, the empty sequence is returned. In the case of namespace nodes,\n             however, the result is always an empty sequence -- it does not depend on the base URI of\n             the parent element.</note><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">See also <code>fn:static-base-uri</code>.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The following errors may be raised when <code>$arg</code> is omitted:</p><ul xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\"><li><p>If the context\n                item is <xtermref ref=\"dt-absent\" spec=\"DM30\">absent</xtermref>, <phrase diff=\"add\" at=\"M\">dynamic error</phrase>\n                <xerrorref spec=\"XP\" class=\"DY\" code=\"0002\" type=\"dynamic\"/></p></li><li><p>If the context item is not a\n                node, <phrase diff=\"add\" at=\"M\">type error</phrase> <xerrorref spec=\"XP\" class=\"TY\" code=\"0004\" type=\"type\"/>.</p></li></ul></div>\n", "summary" : "<p>  Returns the base URI of a node.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [  ], "returns" : { "type" : "xs:anyURI?", "description" : "" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "base-uri", "qname" : "fn:base-uri", "signature" : "($arg as node()?) as xs:anyURI? external", "description" : " <div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">Returns the base URI of a node.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\"><example role=\"signature\"><proto name=\"base-uri\" return-type=\"xs:anyURI?\" isOp=\"no\" prefix=\"fn\" returnEmptyOk=\"no\" returnSeq=\"no\" returnVaries=\"no\" isSchema=\"no\" isDatatype=\"no\" isSpecial=\"no\"/></example><example role=\"signature\"><proto name=\"base-uri\" return-type=\"xs:anyURI?\" isOp=\"no\" prefix=\"fn\" returnEmptyOk=\"no\" returnSeq=\"no\" returnVaries=\"no\" isSchema=\"no\" isDatatype=\"no\" isSpecial=\"no\"><arg name=\"arg\" type=\"node()?\"/></proto></example></p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The zero-argument form of this function is <termref def=\"dt-deterministic\">deterministic</termref>, <termref def=\"dt-context-dependent\">context-dependent</termref>,  and <termref def=\"dt-focus-dependent\">focus-dependent</termref>. </p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The one-argument form of this function is <termref def=\"dt-deterministic\">deterministic</termref>, <termref def=\"dt-context-independent\">context-independent</termref>,  and <termref def=\"dt-focus-independent\">focus-independent</termref>. </p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\" diff=\"add\" at=\"B\">The zero-argument version of the function returns the base URI of the\n             context node: it is equivalent to calling <code>fn:base-uri(.)</code>.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\" diff=\"add\" at=\"B\">The single-argument version of the function behaves as follows:</p><olist xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\"><item>If <code>$arg</code> is the empty sequence, the function returns the empty\n                sequence.</item><item>Otherwise, the function returns the value of the <code>dm:base-uri</code> accessor\n                applied to the node <code>$arg</code>. This accessor is defined, for each kind of\n                node, in the XDM specification (See <xspecref spec=\"DM30\" ref=\"dm-base-uri\"/>).</item></olist><note xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">As explained in XDM, document, element and processing-instruction nodes have a\n             base-uri property which may be empty. The base-uri property for all other node kinds is\n             the empty sequence. The dm:base-uri accessor returns the base-uri property of a node if\n             it exists and is non-empty; otherwise it returns the result of applying the dm:base-uri\n             accessor to its parent, recursively. If the node does not have a parent, or if the\n             recursive ascent up the ancestor chain encounters a parentless node whose base-uri\n             property is empty, the empty sequence is returned. In the case of namespace nodes,\n             however, the result is always an empty sequence -- it does not depend on the base URI of\n             the parent element.</note><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">See also <code>fn:static-base-uri</code>.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The following errors may be raised when <code>$arg</code> is omitted:</p><ul xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\"><li><p>If the context\n                item is <xtermref ref=\"dt-absent\" spec=\"DM30\">absent</xtermref>, <phrase diff=\"add\" at=\"M\">dynamic error</phrase>\n                <xerrorref spec=\"XP\" class=\"DY\" code=\"0002\" type=\"dynamic\"/></p></li><li><p>If the context item is not a\n                node, <phrase diff=\"add\" at=\"M\">type error</phrase> <xerrorref spec=\"XP\" class=\"TY\" code=\"0004\" type=\"type\"/>.</p></li></ul></div>\n", "summary" : "<p>  Returns the base URI of a node.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "arg", "type" : "node()", "occurrence" : "?", "description" : "" } ], "returns" : { "type" : "xs:anyURI?", "description" : "" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "boolean", "qname" : "fn:boolean", "signature" : "($arg as item()*) as xs:boolean external", "description" : " <div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">Computes the effective boolean value of the sequence <code>$arg</code>.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\"><example role=\"signature\"><proto name=\"boolean\" return-type=\"xs:boolean\" isOp=\"no\" prefix=\"fn\" returnEmptyOk=\"no\" returnSeq=\"no\" returnVaries=\"no\" isSchema=\"no\" isDatatype=\"no\" isSpecial=\"no\"><arg name=\"arg\" type=\"item()*\"/></proto></example></p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The function computes the effective boolean value of a sequence, defined according to\n             the following rules. See also <xspecref spec=\"XP30\" ref=\"id-ebv\"/>.</p><ulist xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\"><item><p>If <code>$arg</code> is the empty sequence, <code>fn:boolean</code> returns\n                      <code>false</code>.</p></item><item><p>If <code>$arg</code> is a sequence whose first item is a node,\n                      <code>fn:boolean</code> returns <code>true</code>.</p></item><item><p>If <code>$arg</code> is a singleton value of type <code>xs:boolean</code> or a\n                   derived from <code>xs:boolean</code>, <code>fn:boolean</code> returns\n                      <code>$arg</code>.</p></item><item><p>If <code>$arg</code> is a singleton value of type <code>xs:string</code> or a type\n                   derived from <code>xs:string</code>, <code>xs:anyURI</code> or a type derived from\n                      <code>xs:anyURI</code> or <code>xs:untypedAtomic</code>,\n                      <code>fn:boolean</code> returns <code>false</code> if the operand value has\n                   zero length; otherwise it returns <code>true</code>.</p></item><item><p>If <code>$arg</code> is a singleton value of any numeric type or a type derived\n                   from a numeric type, <code>fn:boolean</code> returns <code>false</code> if the\n                   operand value is <code>NaN</code> or is numerically equal to zero; otherwise it\n                   returns <code>true</code>.</p></item><item><p>In all other cases, <code>fn:boolean</code> raises a type error <errorref class=\"RG\" code=\"0006\"/>.</p></item></ulist><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\" diff=\"del\" at=\"F\">The static semantics of this function are described in [Formal\n             Semantics].</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The result of this function is not necessarily the same as <code>$arg cast as\n                xs:boolean</code>. For example, <code>fn:boolean(\"false\")</code> returns the value\n                <code>true</code> whereas <code>\"false\" cast as xs:boolean</code> (which can also be\n             written <code>xs:boolean(\"false\")</code>) returns <code>false</code>.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">let <code>$abc</code> := <code>(\"a\", \"b\", \"\")</code></p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\"><code>fn:boolean($abc)</code> raises a type error <errorref class=\"RG\" code=\"0006\"/>.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The expression <code>fn:boolean($abc[1])</code> returns <code>true()</code>.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The expression <code>fn:boolean($abc[0])</code> returns <code>false()</code>.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The expression <code>fn:boolean($abc[3])</code> returns <code>false()</code>.</p></div>\n", "summary" : "<p>  Computes the effective boolean value of the sequence  $arg .</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "arg", "type" : "item()", "occurrence" : "*", "description" : "" } ], "returns" : { "type" : "xs:boolean", "description" : "" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "ceiling", "qname" : "fn:ceiling", "signature" : "($arg as numeric?) as numeric? external", "description" : " <div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">Rounds <code>$arg</code> upwards to a whole number.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\"><example role=\"signature\"><proto name=\"ceiling\" return-type=\"numeric?\" isOp=\"no\" prefix=\"fn\" returnEmptyOk=\"no\" returnSeq=\"no\" returnVaries=\"no\" isSchema=\"no\" isDatatype=\"no\" isSpecial=\"no\"><arg name=\"arg\" type=\"numeric?\"/></proto></example></p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">This function is <termref def=\"dt-deterministic\">deterministic</termref>, <termref def=\"dt-context-independent\">context-independent</termref>,  and <termref def=\"dt-focus-independent\">focus-independent</termref>. </p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">General rules: see <specref ref=\"numeric-value-functions\"/>.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The function returns the smallest (closest to negative infinity) number with no\n             fractional part that is not less than the value of <code>$arg</code>.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">If the type of <code>$arg</code> is one of the four numeric types <code>xs:float</code>,\n                <code>xs:double</code>, <code>xs:decimal</code> or <code>xs:integer</code> the type\n             of the result is the same as the type of <code>$arg</code>. If the type of\n                <code>$arg</code> is a type derived from one of the numeric types, the result is an\n             instance of the base numeric type.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">For <code>xs:float</code> and <code>xs:double</code> arguments, if the argument is\n             positive zero, then positive zero is returned. If the argument is negative zero, then\n             negative zero is returned. If the argument is less than zero and greater than -1,\n             negative zero is returned.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\" diff=\"del\" at=\"F\">For detailed type semantics, see [Formal Semantics].</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The expression <code>fn:ceiling(10.5)</code> returns <code>11</code>.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The expression <code>fn:ceiling(-10.5)</code> returns <code>-10</code>.</p></div>\n", "summary" : "<p>  Rounds  $arg  upwards to a whole number.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "arg", "type" : "numeric", "occurrence" : "?", "description" : "" } ], "returns" : { "type" : "numeric?", "description" : "" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 2, "name" : "codepoint-equal", "qname" : "fn:codepoint-equal", "signature" : "($comparand1 as xs:string?, $comparand2 as xs:string?) as xs:boolean? external", "description" : " <div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">Returns true if two strings are equal, considered codepoint-by-codepoint.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\"><example role=\"signature\"><proto name=\"codepoint-equal\" return-type=\"xs:boolean?\" isOp=\"no\" prefix=\"fn\" returnEmptyOk=\"no\" returnSeq=\"no\" returnVaries=\"no\" isSchema=\"no\" isDatatype=\"no\" isSpecial=\"no\"><arg name=\"comparand1\" type=\"xs:string?\"/><arg name=\"comparand2\" type=\"xs:string?\"/></proto></example></p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">This function is <termref def=\"dt-deterministic\">deterministic</termref>, <termref def=\"dt-context-independent\">context-independent</termref>,  and <termref def=\"dt-focus-independent\">focus-independent</termref>. </p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">If either argument is the empty sequence, the function returns the empty sequence. </p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">Otherwise, the function returns <code>true</code> or <code>false</code> depending on\n             whether the value of <code>$comparand1</code> is equal to the value of\n                <code>$comparand2</code>, according to the Unicode codepoint collation\n                (<code>http://www.w3.org/2005/xpath-functions/collation/codepoint</code>).</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">This function allows <code>xs:anyURI</code> values to be compared without having to\n             specify the Unicode codepoint collation.</p></div>\n", "summary" : "<p>  Returns true if two strings are equal, considered codepoint-by-codepoint.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "comparand1", "type" : "xs:string", "occurrence" : "?", "description" : "" }, { "name" : "comparand2", "type" : "xs:string", "occurrence" : "?", "description" : "" } ], "returns" : { "type" : "xs:boolean?", "description" : "" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "codepoints-to-string", "qname" : "fn:codepoints-to-string", "signature" : "($arg as xs:integer*) as xs:string external", "description" : " <div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">Creates an <code>xs:string</code> from a sequence of <termref def=\"codepoint\">codepoints</termref>.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\"><example role=\"signature\"><proto name=\"codepoints-to-string\" return-type=\"xs:string\" isOp=\"no\" prefix=\"fn\" returnEmptyOk=\"no\" returnSeq=\"no\" returnVaries=\"no\" isSchema=\"no\" isDatatype=\"no\" isSpecial=\"no\"><arg name=\"arg\" type=\"xs:integer*\"/></proto></example></p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">This function is <termref def=\"dt-deterministic\">deterministic</termref>, <termref def=\"dt-context-independent\">context-independent</termref>,  and <termref def=\"dt-focus-independent\">focus-independent</termref>. </p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The function returns the string made up from the <termref def=\"character\">characters</termref> whose Unicode <termref def=\"codepoint\">codepoints</termref> are\n             supplied in <code>$arg</code>. This will be the zero-length string if <code>$arg</code>\n             is the empty sequence. </p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">A <phrase diff=\"add\" at=\"M\">dynamic</phrase> error is raised <errorref class=\"CH\" code=\"0001\"/> if any of the codepoints in\n                <code>$arg</code> is not a permitted XML character.</p></div>\n", "summary" : "<p>  Creates an  xs:string  from a sequence of  codepoints .</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "arg", "type" : "xs:integer", "occurrence" : "*", "description" : "" } ], "returns" : { "type" : "xs:string", "description" : "" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "codepoints-to-string", "qname" : "fn:codepoints-to-string", "signature" : "($arg as xs:integer*) as xs:string external", "description" : " <div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">Creates an <code>xs:string</code> from a sequence of <termref def=\"codepoint\">codepoints</termref>.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\"><example role=\"signature\"><proto name=\"codepoints-to-string\" return-type=\"xs:string\" isOp=\"no\" prefix=\"fn\" returnEmptyOk=\"no\" returnSeq=\"no\" returnVaries=\"no\" isSchema=\"no\" isDatatype=\"no\" isSpecial=\"no\"><arg name=\"arg\" type=\"xs:integer*\"/></proto></example></p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">This function is <termref def=\"dt-deterministic\">deterministic</termref>, <termref def=\"dt-context-independent\">context-independent</termref>,  and <termref def=\"dt-focus-independent\">focus-independent</termref>. </p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The function returns the string made up from the <termref def=\"character\">characters</termref> whose Unicode <termref def=\"codepoint\">codepoints</termref> are\n             supplied in <code>$arg</code>. This will be the zero-length string if <code>$arg</code>\n             is the empty sequence. </p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">A <phrase diff=\"add\" at=\"M\">dynamic</phrase> error is raised <errorref class=\"CH\" code=\"0001\"/> if any of the codepoints in\n                <code>$arg</code> is not a permitted XML character.</p></div>\n", "summary" : "<p>  Creates an  xs:string  from a sequence of  codepoints .</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "arg", "type" : "xs:integer", "occurrence" : "*", "description" : "" } ], "returns" : { "type" : "xs:string", "description" : "" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 0, "name" : "collection", "qname" : "fn:collection", "signature" : "() as node()* external", "description" : " <div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">Returns a sequence of nodes representing a collection of documents indentified\n             by a collection URI; or a default collection if no URI is supplied.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\"><example role=\"signature\"><proto name=\"collection\" return-type=\"node()*\" isOp=\"no\" prefix=\"fn\" returnEmptyOk=\"no\" returnSeq=\"no\" returnVaries=\"no\" isSchema=\"no\" isDatatype=\"no\" isSpecial=\"no\"/></example><example role=\"signature\"><proto name=\"collection\" return-type=\"node()*\" isOp=\"no\" prefix=\"fn\" returnEmptyOk=\"no\" returnSeq=\"no\" returnVaries=\"no\" isSchema=\"no\" isDatatype=\"no\" isSpecial=\"no\"><arg name=\"arg\" type=\"xs:string?\"/></proto></example></p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">This function is <termref def=\"dt-deterministic\">deterministic</termref>, <termref def=\"dt-context-dependent\">context-dependent</termref>,  and <termref def=\"dt-focus-independent\">focus-independent</termref>.  It depends on\n \t\tavailable node collections, and static base uri.\n \t</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">This function takes an <code>xs:string</code> as argument and returns a sequence of\n             nodes obtained by interpreting <code>$arg</code> as an <code>xs:anyURI</code> and\n             resolving it according to the mapping specified in <term>Available node collections</term>\n             described in <xspecref spec=\"XP30\" ref=\"id-xp-evaluation-context-components\"/>.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">If <term>Available node collections</term> provides a mapping from this string to a sequence\n             of nodes, the function returns that sequence. If <term>Available node collections</term> maps\n             the string to an empty sequence, then the function returns an empty sequence.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">If <code>$arg</code> is not specified, the function returns the sequence of the nodes in\n             the default node collection in the dynamic context. See <xspecref spec=\"XP30\" ref=\"id-xp-evaluation-context-components\"/>. </p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">If the value of <code>$arg</code> is a relative <code>xs:anyURI</code>, it is resolved\n             against the value of the base-URI property from the static context. </p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">If <code>$arg</code> is the empty sequence, the function behaves as if it had been\n             called without an argument. See above.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">By default, this function is <termref def=\"deterministic\">deterministic</termref>.\n             This means that repeated\n             calls on the function with the same argument will return the same result. However, for\n             performance reasons, implementations may provide a user option to evaluate the function\n             without a guarantee of determinism. The manner in which any such option is provided is\n                <termref def=\"implementation-defined\"/>. If the user has not selected such an option,\n             a call to this function must either return a deterministic result or must raise a <phrase diff=\"add\" at=\"L\">dynamic</phrase> error\n                <errorref class=\"DC\" code=\"0003\"/>.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\" diff=\"add\" at=\"G\">There is no requirement that the returned nodes should be in document\n             order, nor is there a requirement that the result should contain no duplicates.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\" diff=\"del\" at=\"F\">For detailed type semantics, see [Formal Semantics].</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">A <phrase diff=\"add\" at=\"L\">dynamic</phrase> error is raised <errorref class=\"DC\" code=\"0002\"/> if no URI is supplied and the\n             value of the default collection is <xtermref ref=\"dt-absent\" spec=\"DM30\">absent</xtermref>.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\" diff=\"chg\" at=\"J\">A <phrase diff=\"add\" at=\"M\">dynamic</phrase> error is raised <errorref class=\"DC\" code=\"0002\"/> if <term>available\n                node collections</term> provides no mapping for the absolutized URI.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">A <phrase diff=\"add\" at=\"L\">dynamic</phrase> error is raised <errorref class=\"DC\" code=\"0004\"/> if <code>$arg</code> is not a\n             valid <code>xs:anyURI</code>.</p></div>\n", "summary" : "<p>  Returns a sequence of nodes representing a collection of documents indentified\n             by a collection URI; or a default collection if no URI is supplied.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [  ], "returns" : { "type" : "node()*", "description" : "" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "collection", "qname" : "fn:collection", "signature" : "($arg as xs:string?) as node()* external", "description" : " <div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">Returns a sequence of nodes representing a collection of documents indentified\n             by a collection URI; or a default collection if no URI is supplied.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\"><example role=\"signature\"><proto name=\"collection\" return-type=\"node()*\" isOp=\"no\" prefix=\"fn\" returnEmptyOk=\"no\" returnSeq=\"no\" returnVaries=\"no\" isSchema=\"no\" isDatatype=\"no\" isSpecial=\"no\"/></example><example role=\"signature\"><proto name=\"collection\" return-type=\"node()*\" isOp=\"no\" prefix=\"fn\" returnEmptyOk=\"no\" returnSeq=\"no\" returnVaries=\"no\" isSchema=\"no\" isDatatype=\"no\" isSpecial=\"no\"><arg name=\"arg\" type=\"xs:string?\"/></proto></example></p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">This function is <termref def=\"dt-deterministic\">deterministic</termref>, <termref def=\"dt-context-dependent\">context-dependent</termref>,  and <termref def=\"dt-focus-independent\">focus-independent</termref>.  It depends on\n \t\tavailable node collections, and static base uri.\n \t</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">This function takes an <code>xs:string</code> as argument and returns a sequence of\n             nodes obtained by interpreting <code>$arg</code> as an <code>xs:anyURI</code> and\n             resolving it according to the mapping specified in <term>Available node collections</term>\n             described in <xspecref spec=\"XP30\" ref=\"id-xp-evaluation-context-components\"/>.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">If <term>Available node collections</term> provides a mapping from this string to a sequence\n             of nodes, the function returns that sequence. If <term>Available node collections</term> maps\n             the string to an empty sequence, then the function returns an empty sequence.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">If <code>$arg</code> is not specified, the function returns the sequence of the nodes in\n             the default node collection in the dynamic context. See <xspecref spec=\"XP30\" ref=\"id-xp-evaluation-context-components\"/>. </p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">If the value of <code>$arg</code> is a relative <code>xs:anyURI</code>, it is resolved\n             against the value of the base-URI property from the static context. </p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">If <code>$arg</code> is the empty sequence, the function behaves as if it had been\n             called without an argument. See above.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">By default, this function is <termref def=\"deterministic\">deterministic</termref>.\n             This means that repeated\n             calls on the function with the same argument will return the same result. However, for\n             performance reasons, implementations may provide a user option to evaluate the function\n             without a guarantee of determinism. The manner in which any such option is provided is\n                <termref def=\"implementation-defined\"/>. If the user has not selected such an option,\n             a call to this function must either return a deterministic result or must raise a <phrase diff=\"add\" at=\"L\">dynamic</phrase> error\n                <errorref class=\"DC\" code=\"0003\"/>.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\" diff=\"add\" at=\"G\">There is no requirement that the returned nodes should be in document\n             order, nor is there a requirement that the result should contain no duplicates.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\" diff=\"del\" at=\"F\">For detailed type semantics, see [Formal Semantics].</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">A <phrase diff=\"add\" at=\"L\">dynamic</phrase> error is raised <errorref class=\"DC\" code=\"0002\"/> if no URI is supplied and the\n             value of the default collection is <xtermref ref=\"dt-absent\" spec=\"DM30\">absent</xtermref>.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\" diff=\"chg\" at=\"J\">A <phrase diff=\"add\" at=\"M\">dynamic</phrase> error is raised <errorref class=\"DC\" code=\"0002\"/> if <term>available\n                node collections</term> provides no mapping for the absolutized URI.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">A <phrase diff=\"add\" at=\"L\">dynamic</phrase> error is raised <errorref class=\"DC\" code=\"0004\"/> if <code>$arg</code> is not a\n             valid <code>xs:anyURI</code>.</p></div>\n", "summary" : "<p>  Returns a sequence of nodes representing a collection of documents indentified\n             by a collection URI; or a default collection if no URI is supplied.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "arg", "type" : "xs:string", "occurrence" : "?", "description" : "" } ], "returns" : { "type" : "node()*", "description" : "" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 2, "name" : "compare", "qname" : "fn:compare", "signature" : "($comparand1 as xs:string?, $comparand2 as xs:string?) as xs:integer? external", "description" : " <div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">Returns -1, 0, or 1, depending on whether <code>$comparand1</code> collates\n             before, equal to, or after <code>$comparand2</code> according to the rules of a selected\n             collation.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\"><example role=\"signature\"><proto name=\"compare\" return-type=\"xs:integer?\" isOp=\"no\" prefix=\"fn\" returnEmptyOk=\"no\" returnSeq=\"no\" returnVaries=\"no\" isSchema=\"no\" isDatatype=\"no\" isSpecial=\"no\"><arg name=\"comparand1\" type=\"xs:string?\"/><arg name=\"comparand2\" type=\"xs:string?\"/></proto></example><example role=\"signature\"><proto name=\"compare\" return-type=\"xs:integer?\" isOp=\"no\" prefix=\"fn\" returnEmptyOk=\"no\" returnSeq=\"no\" returnVaries=\"no\" isSchema=\"no\" isDatatype=\"no\" isSpecial=\"no\"><arg name=\"comparand1\" type=\"xs:string?\"/><arg name=\"comparand2\" type=\"xs:string?\"/><arg name=\"collation\" type=\"xs:string\"/></proto></example></p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The two-argument form of this function is <termref def=\"dt-deterministic\">deterministic</termref>, <termref def=\"dt-context-dependent\">context-dependent</termref>,  and <termref def=\"dt-focus-independent\">focus-independent</termref>.  It depends on\n \t\tcollations.\n \t</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The three-argument form of this function is <termref def=\"dt-deterministic\">deterministic</termref>, <termref def=\"dt-context-dependent\">context-dependent</termref>,  and <termref def=\"dt-focus-independent\">focus-independent</termref>.  It depends on\n \t\tcollations, and static base uri.\n \t</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">Returns -1, 0, or 1, depending on whether the value of the <code>$comparand1</code> is\n             respectively less than, equal to, or greater than the value of <code>$comparand2</code>,\n             according to the rules of the collation that is used. </p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The collation used by this function is determined according to the rules in <specref ref=\"choosing-a-collation\"/>. </p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">If either <code>$comparand1</code> or <code>$comparand2</code> is the empty sequence,\n             the function returns the empty sequence.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">This function, called with the first signature, defines the semantics of the \"eq\", \"ne\",\n             \"gt\", \"lt\", \"le\" and \"ge\" operators on <code>xs:string</code> values.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The expression <code>fn:compare('abc', 'abc')</code> returns <code>0</code>.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The expression <code>fn:compare('Strasse', 'Straße')</code> returns <code>0</code>. <emph>(Assuming the default collation includes provisions that equate\n                      <quote>ss</quote> and the (German) character <quote>ß</quote>\n                      (<quote>sharp-s</quote>). Otherwise, the returned value depends on the\n                   semantics of the default collation.).</emph></p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The expression <code>fn:compare('Strasse', 'Straße',\n                   'http://example.com/deutsch')</code> returns <code>0</code>. <emph>(Assuming the collation identified by the URI\n                      <code>http://example.com/deutsch</code> includes provisions that equate\n                      <quote>ss</quote> and the (German) character <quote>ß</quote>\n                      (<quote>sharp-s</quote>). Otherwise, the returned value depends on the\n                   semantics of that collation.).</emph></p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The expression <code>fn:compare('Strassen', 'Straße')</code> returns <code>1</code>. <emph>(Assuming the default collation includes provisions that treat\n                   differences between <quote>ss</quote> and the (German) character <quote>ß</quote>\n                      (<quote>sharp-s</quote>) with less strength than the differences between the\n                   base characters, such as the final <quote>n</quote>. ).</emph></p></div>\n", "summary" : "<p>  Returns -1, 0, or 1, depending on whether  $comparand1  collates\n             before, equal to, or after  $comparand2  according to the rules of a selected\n             collation.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "comparand1", "type" : "xs:string", "occurrence" : "?", "description" : "" }, { "name" : "comparand2", "type" : "xs:string", "occurrence" : "?", "description" : "" } ], "returns" : { "type" : "xs:integer?", "description" : "" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 3, "name" : "compare", "qname" : "fn:compare", "signature" : "($comparand1 as xs:string?, $comparand2 as xs:string?, $collation as xs:string) as xs:integer? external", "description" : " <div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">Returns -1, 0, or 1, depending on whether <code>$comparand1</code> collates\n             before, equal to, or after <code>$comparand2</code> according to the rules of a selected\n             collation.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\"><example role=\"signature\"><proto name=\"compare\" return-type=\"xs:integer?\" isOp=\"no\" prefix=\"fn\" returnEmptyOk=\"no\" returnSeq=\"no\" returnVaries=\"no\" isSchema=\"no\" isDatatype=\"no\" isSpecial=\"no\"><arg name=\"comparand1\" type=\"xs:string?\"/><arg name=\"comparand2\" type=\"xs:string?\"/></proto></example><example role=\"signature\"><proto name=\"compare\" return-type=\"xs:integer?\" isOp=\"no\" prefix=\"fn\" returnEmptyOk=\"no\" returnSeq=\"no\" returnVaries=\"no\" isSchema=\"no\" isDatatype=\"no\" isSpecial=\"no\"><arg name=\"comparand1\" type=\"xs:string?\"/><arg name=\"comparand2\" type=\"xs:string?\"/><arg name=\"collation\" type=\"xs:string\"/></proto></example></p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The two-argument form of this function is <termref def=\"dt-deterministic\">deterministic</termref>, <termref def=\"dt-context-dependent\">context-dependent</termref>,  and <termref def=\"dt-focus-independent\">focus-independent</termref>.  It depends on\n \t\tcollations.\n \t</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The three-argument form of this function is <termref def=\"dt-deterministic\">deterministic</termref>, <termref def=\"dt-context-dependent\">context-dependent</termref>,  and <termref def=\"dt-focus-independent\">focus-independent</termref>.  It depends on\n \t\tcollations, and static base uri.\n \t</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">Returns -1, 0, or 1, depending on whether the value of the <code>$comparand1</code> is\n             respectively less than, equal to, or greater than the value of <code>$comparand2</code>,\n             according to the rules of the collation that is used. </p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The collation used by this function is determined according to the rules in <specref ref=\"choosing-a-collation\"/>. </p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">If either <code>$comparand1</code> or <code>$comparand2</code> is the empty sequence,\n             the function returns the empty sequence.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">This function, called with the first signature, defines the semantics of the \"eq\", \"ne\",\n             \"gt\", \"lt\", \"le\" and \"ge\" operators on <code>xs:string</code> values.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The expression <code>fn:compare('abc', 'abc')</code> returns <code>0</code>.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The expression <code>fn:compare('Strasse', 'Straße')</code> returns <code>0</code>. <emph>(Assuming the default collation includes provisions that equate\n                      <quote>ss</quote> and the (German) character <quote>ß</quote>\n                      (<quote>sharp-s</quote>). Otherwise, the returned value depends on the\n                   semantics of the default collation.).</emph></p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The expression <code>fn:compare('Strasse', 'Straße',\n                   'http://example.com/deutsch')</code> returns <code>0</code>. <emph>(Assuming the collation identified by the URI\n                      <code>http://example.com/deutsch</code> includes provisions that equate\n                      <quote>ss</quote> and the (German) character <quote>ß</quote>\n                      (<quote>sharp-s</quote>). Otherwise, the returned value depends on the\n                   semantics of that collation.).</emph></p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The expression <code>fn:compare('Strassen', 'Straße')</code> returns <code>1</code>. <emph>(Assuming the default collation includes provisions that treat\n                   differences between <quote>ss</quote> and the (German) character <quote>ß</quote>\n                      (<quote>sharp-s</quote>) with less strength than the differences between the\n                   base characters, such as the final <quote>n</quote>. ).</emph></p></div>\n", "summary" : "<p>  Returns -1, 0, or 1, depending on whether  $comparand1  collates\n             before, equal to, or after  $comparand2  according to the rules of a selected\n             collation.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "comparand1", "type" : "xs:string", "occurrence" : "?", "description" : "" }, { "name" : "comparand2", "type" : "xs:string", "occurrence" : "?", "description" : "" }, { "name" : "collation", "type" : "xs:string", "occurrence" : null, "description" : "" } ], "returns" : { "type" : "xs:integer?", "description" : "" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 2, "name" : "concat", "qname" : "fn:concat", "signature" : "($arg1 as xs:anyAtomicType?, $arg2 as xs:anyAtomicType?) as xs:string external", "description" : " <div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">Returns the concatenation of the string values of the arguments.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">\n          The two-argument form of this function defines the semantics of the \"||\" operator.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\"><example role=\"signature\"><proto name=\"concat\" return-type=\"xs:string\" isOp=\"yes\" prefix=\"fn\" returnEmptyOk=\"no\" returnSeq=\"no\" returnVaries=\"no\" isSchema=\"no\" isDatatype=\"no\" isSpecial=\"no\"><arg name=\"arg1\" type=\"xs:anyAtomicType?\"/><arg name=\"arg2\" type=\"xs:anyAtomicType?\"/><arg name=\"...\" type=\"xs:anyAtomicType?\"/></proto></example></p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">This function is <termref def=\"dt-deterministic\">deterministic</termref>, <termref def=\"dt-context-independent\">context-independent</termref>,  and <termref def=\"dt-focus-independent\">focus-independent</termref>. </p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">This function accepts two or more <code>xs:anyAtomicType</code> arguments and casts each\n             one to <code>xs:string</code>. The function returns the <code>xs:string</code> that is\n             the concatenation of the values of its arguments after conversion. If any argument is\n             the empty sequence, that argument is treated as the zero-length string.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\" diff=\"chg\" at=\"A-E19\">The <code>fn:concat</code> function is specified to allow two or\n             more arguments, which are concatenated together. This is the only function specified in\n             this document that allows a variable number of arguments. This capability is retained\n             for compatibility with <bibref ref=\"xpath\"/>. </p></div>\n", "summary" : "<p>  Returns the concatenation of the string values of the arguments.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "arg1", "type" : "xs:anyAtomicType", "occurrence" : "?", "description" : "" }, { "name" : "arg2", "type" : "xs:anyAtomicType", "occurrence" : "?", "description" : "" } ], "returns" : { "type" : "xs:string", "description" : "" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 2, "name" : "contains", "qname" : "fn:contains", "signature" : "($arg1 as xs:string?, $arg2 as xs:string?) as xs:boolean external", "description" : " <div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">Returns true if the string <code>$arg1</code> contains <code>$arg2</code> as a\n             substring, taking collations into account.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\"><example role=\"signature\"><proto name=\"contains\" return-type=\"xs:boolean\" isOp=\"no\" prefix=\"fn\" returnEmptyOk=\"no\" returnSeq=\"no\" returnVaries=\"no\" isSchema=\"no\" isDatatype=\"no\" isSpecial=\"no\"><arg name=\"arg1\" type=\"xs:string?\"/><arg name=\"arg2\" type=\"xs:string?\"/></proto></example><example role=\"signature\"><proto name=\"contains\" return-type=\"xs:boolean\" isOp=\"no\" prefix=\"fn\" returnEmptyOk=\"no\" returnSeq=\"no\" returnVaries=\"no\" isSchema=\"no\" isDatatype=\"no\" isSpecial=\"no\"><arg name=\"arg1\" type=\"xs:string?\"/><arg name=\"arg2\" type=\"xs:string?\"/><arg name=\"collation\" type=\"xs:string\"/></proto></example></p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The two-argument form of this function is <termref def=\"dt-deterministic\">deterministic</termref>, <termref def=\"dt-context-dependent\">context-dependent</termref>,  and <termref def=\"dt-focus-independent\">focus-independent</termref>.  It depends on\n \t\tcollations.\n \t</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The three-argument form of this function is <termref def=\"dt-deterministic\">deterministic</termref>, <termref def=\"dt-context-dependent\">context-dependent</termref>,  and <termref def=\"dt-focus-independent\">focus-independent</termref>.  It depends on\n \t\tcollations, and static base uri.\n \t</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">If the value of <code>$arg1</code> or <code>$arg2</code> is the empty sequence, or\n             contains only ignorable collation units, it is interpreted as the zero-length\n             string.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">If the value of <code>$arg2</code> is the zero-length string, then the function returns\n                <code>true</code>.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">If the value of <code>$arg1</code> is the zero-length string, the function returns\n                <code>false</code>.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The collation used by this function is determined according to the rules in <specref ref=\"choosing-a-collation\"/>.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The function returns an <code>xs:boolean</code> indicating whether or not the value of\n                <code>$arg1</code> contains (at the beginning, at the end, or anywhere within) at\n             least one sequence of collation units that provides a <term>minimal match</term> to the\n             collation units in the value of <code>$arg2</code>, according to the collation that is\n             used.</p><note xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\"><p><term>Minimal match</term> is defined in <bibref ref=\"Unicode-Collations\"/>. </p></note><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">A <phrase diff=\"add\" at=\"L\">dynamic</phrase> error <rfc2119>may</rfc2119> be raised <errorref class=\"CH\" code=\"0004\"/> if the\n             specified collation does not support collation units.</p></div>\n", "summary" : "<p>  Returns true if the string  $arg1  contains  $arg2  as a\n             substring, taking collations into account.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "arg1", "type" : "xs:string", "occurrence" : "?", "description" : "" }, { "name" : "arg2", "type" : "xs:string", "occurrence" : "?", "description" : "" } ], "returns" : { "type" : "xs:boolean", "description" : "" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 3, "name" : "contains", "qname" : "fn:contains", "signature" : "($arg1 as xs:string?, $arg2 as xs:string?, $collation as xs:string) as xs:boolean external", "description" : " <div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">Returns true if the string <code>$arg1</code> contains <code>$arg2</code> as a\n             substring, taking collations into account.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\"><example role=\"signature\"><proto name=\"contains\" return-type=\"xs:boolean\" isOp=\"no\" prefix=\"fn\" returnEmptyOk=\"no\" returnSeq=\"no\" returnVaries=\"no\" isSchema=\"no\" isDatatype=\"no\" isSpecial=\"no\"><arg name=\"arg1\" type=\"xs:string?\"/><arg name=\"arg2\" type=\"xs:string?\"/></proto></example><example role=\"signature\"><proto name=\"contains\" return-type=\"xs:boolean\" isOp=\"no\" prefix=\"fn\" returnEmptyOk=\"no\" returnSeq=\"no\" returnVaries=\"no\" isSchema=\"no\" isDatatype=\"no\" isSpecial=\"no\"><arg name=\"arg1\" type=\"xs:string?\"/><arg name=\"arg2\" type=\"xs:string?\"/><arg name=\"collation\" type=\"xs:string\"/></proto></example></p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The two-argument form of this function is <termref def=\"dt-deterministic\">deterministic</termref>, <termref def=\"dt-context-dependent\">context-dependent</termref>,  and <termref def=\"dt-focus-independent\">focus-independent</termref>.  It depends on\n \t\tcollations.\n \t</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The three-argument form of this function is <termref def=\"dt-deterministic\">deterministic</termref>, <termref def=\"dt-context-dependent\">context-dependent</termref>,  and <termref def=\"dt-focus-independent\">focus-independent</termref>.  It depends on\n \t\tcollations, and static base uri.\n \t</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">If the value of <code>$arg1</code> or <code>$arg2</code> is the empty sequence, or\n             contains only ignorable collation units, it is interpreted as the zero-length\n             string.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">If the value of <code>$arg2</code> is the zero-length string, then the function returns\n                <code>true</code>.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">If the value of <code>$arg1</code> is the zero-length string, the function returns\n                <code>false</code>.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The collation used by this function is determined according to the rules in <specref ref=\"choosing-a-collation\"/>.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The function returns an <code>xs:boolean</code> indicating whether or not the value of\n                <code>$arg1</code> contains (at the beginning, at the end, or anywhere within) at\n             least one sequence of collation units that provides a <term>minimal match</term> to the\n             collation units in the value of <code>$arg2</code>, according to the collation that is\n             used.</p><note xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\"><p><term>Minimal match</term> is defined in <bibref ref=\"Unicode-Collations\"/>. </p></note><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">A <phrase diff=\"add\" at=\"L\">dynamic</phrase> error <rfc2119>may</rfc2119> be raised <errorref class=\"CH\" code=\"0004\"/> if the\n             specified collation does not support collation units.</p></div>\n", "summary" : "<p>  Returns true if the string  $arg1  contains  $arg2  as a\n             substring, taking collations into account.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "arg1", "type" : "xs:string", "occurrence" : "?", "description" : "" }, { "name" : "arg2", "type" : "xs:string", "occurrence" : "?", "description" : "" }, { "name" : "collation", "type" : "xs:string", "occurrence" : null, "description" : "" } ], "returns" : { "type" : "xs:boolean", "description" : "" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "count", "qname" : "fn:count", "signature" : "($arg as item()*) as xs:integer external", "description" : " <div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">Returns the number of items in a sequence.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\"><example role=\"signature\"><proto name=\"count\" return-type=\"xs:integer\" isOp=\"no\" prefix=\"fn\" returnEmptyOk=\"no\" returnSeq=\"no\" returnVaries=\"no\" isSchema=\"no\" isDatatype=\"no\" isSpecial=\"no\"><arg name=\"arg\" type=\"item()*\"/></proto></example></p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">This function is <termref def=\"dt-deterministic\">deterministic</termref>, <termref def=\"dt-context-independent\">context-independent</termref>,  and <termref def=\"dt-focus-independent\">focus-independent</termref>. </p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The function returns the number of items in the value of <code>$arg</code>.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">Returns 0 if <code>$arg</code> is the empty sequence.</p></div>\n", "summary" : "<p>  Returns the number of items in a sequence.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "arg", "type" : "item()", "occurrence" : "*", "description" : "" } ], "returns" : { "type" : "xs:integer", "description" : "" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 0, "name" : "current-date", "qname" : "fn:current-date", "signature" : "() as xs:date external", "description" : " <div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">Returns the current date.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\"><example role=\"signature\"><proto name=\"current-date\" return-type=\"xs:date\" isOp=\"no\" prefix=\"fn\" returnEmptyOk=\"no\" returnSeq=\"no\" returnVaries=\"no\" isSchema=\"no\" isDatatype=\"no\" isSpecial=\"no\"/></example></p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">This function is <termref def=\"dt-deterministic\">deterministic</termref>, <termref def=\"dt-context-dependent\">context-dependent</termref>,  and <termref def=\"dt-focus-independent\">focus-independent</termref>.  It depends on\n \t\timplicit timezone.\n \t</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">Returns <code>xs:date(fn:current-dateTime())</code>. This is an <code>xs:date</code>\n             (with timezone) that is current at some time during the evaluation of a query or\n             transformation in which <code>fn:current-date</code> is executed.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">This function is <termref def=\"dt-deterministic\"/>. The precise instant during the query or\n             transformation represented by the value of <code>fn:current-date</code> is <termref def=\"implementation-dependent\"/>.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The returned date will always have an associated timezone, which will always be the same\n             as the implicit timezone in the dynamic context</p></div>\n", "summary" : "<p>  Returns the current date.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [  ], "returns" : { "type" : "xs:date", "description" : "" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 0, "name" : "current-dateTime", "qname" : "fn:current-dateTime", "signature" : "() as xs:dateTimeStamp external", "description" : " <div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">Returns the current date and time (with timezone).</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\"><example role=\"signature\"><proto name=\"current-dateTime\" return-type=\"xs:dateTimeStamp\" isOp=\"no\" prefix=\"fn\" returnEmptyOk=\"no\" returnSeq=\"no\" returnVaries=\"no\" isSchema=\"no\" isDatatype=\"no\" isSpecial=\"no\"/></example></p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">This function is <termref def=\"dt-deterministic\">deterministic</termref>, <termref def=\"dt-context-dependent\">context-dependent</termref>,  and <termref def=\"dt-focus-independent\">focus-independent</termref>.  It depends on\n \t\timplicit timezone.\n \t</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">Returns the current dateTime (with timezone) from the dynamic context. (See <xspecref spec=\"XP30\" ref=\"id-xp-evaluation-context-components\"/>.) This is an\n                <code>xs:dateTime</code> that is current at some time during the evaluation of a\n             query or transformation in which <code>fn:current-dateTime</code> is executed.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">This function is <termref def=\"dt-deterministic\"/>. The precise instant during the query or\n             transformation represented by the value of <code>fn:current-dateTime()</code> is\n                <termref def=\"implementation-dependent\"/>.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\" diff=\"chg\" at=\"A\">If the implementation supports data types from XSD 1.1 then the\n             returned value will be an instance of <code>xs:dateTimeStamp</code>. Otherwise, the only\n             guarantees are that it will be an instance of <code>xs:dateTime</code> and will have a\n             timezone component.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The returned <code>xs:dateTime</code> will always have an associated timezone, which\n             will always be the same as the implicit timezone in the dynamic context</p></div>\n", "summary" : "<p>  Returns the current date and time (with timezone).</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [  ], "returns" : { "type" : "xs:dateTimeStamp", "description" : "" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 0, "name" : "current-time", "qname" : "fn:current-time", "signature" : "() as xs:time external", "description" : " <div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">Returns the current time.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\"><example role=\"signature\"><proto name=\"current-time\" return-type=\"xs:time\" isOp=\"no\" prefix=\"fn\" returnEmptyOk=\"no\" returnSeq=\"no\" returnVaries=\"no\" isSchema=\"no\" isDatatype=\"no\" isSpecial=\"no\"/></example></p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">This function is <termref def=\"dt-deterministic\">deterministic</termref>, <termref def=\"dt-context-dependent\">context-dependent</termref>,  and <termref def=\"dt-focus-independent\">focus-independent</termref>.  It depends on\n \t\timplicit timezone.\n \t</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">Returns <code>xs:time(fn:current-dateTime())</code>. This is an <code>xs:time</code>\n             (with timezone) that is current at some time during the evaluation of a query or\n             transformation in which <code>fn:current-time</code> is executed.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">This function is <termref def=\"dt-deterministic\"/>. The precise instant during the query or\n             transformation represented by the value of <code>fn:current-time()</code> is <termref def=\"implementation-dependent\"/>.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The returned time will always have an associated timezone, which will always be the same\n             as the implicit timezone in the dynamic context</p></div>\n", "summary" : "<p>  Returns the current time.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [  ], "returns" : { "type" : "xs:time", "description" : "" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 0, "name" : "data", "qname" : "fn:data", "signature" : "() as xs:anyAtomicType* external", "description" : " <div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">Returns the result of atomizing a sequence, that is, replacing all nodes in the\n             sequence by their typed values.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\"><example role=\"signature\"><proto name=\"data\" return-type=\"xs:anyAtomicType*\" isOp=\"no\" prefix=\"fn\" returnEmptyOk=\"no\" returnSeq=\"no\" returnVaries=\"no\" isSchema=\"no\" isDatatype=\"no\" isSpecial=\"no\"/></example><example role=\"signature\"><proto name=\"data\" return-type=\"xs:anyAtomicType*\" isOp=\"no\" prefix=\"fn\" returnEmptyOk=\"no\" returnSeq=\"no\" returnVaries=\"no\" isSchema=\"no\" isDatatype=\"no\" isSpecial=\"no\"><arg name=\"arg\" type=\"item()*\"/></proto></example></p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The zero-argument form of this function is <termref def=\"dt-deterministic\">deterministic</termref>, <termref def=\"dt-context-dependent\">context-dependent</termref>,  and <termref def=\"dt-focus-dependent\">focus-dependent</termref>. </p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The one-argument form of this function is <termref def=\"dt-deterministic\">deterministic</termref>, <termref def=\"dt-context-independent\">context-independent</termref>,  and <termref def=\"dt-focus-independent\">focus-independent</termref>. </p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\" diff=\"add\" at=\"F\">If the argument is omitted, it defaults to the context item\n                (<code>.</code>). The behavior of the function if the argument is omitted is exactly\n             the same as if the context item had been passed as the argument.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\"> The result of <code>fn:data</code> is the sequence of atomic values produced by\n             applying the following rules to each item in <code>$arg</code>:</p><ulist xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\"><item><p>If the item is an atomic value, it is appended to the result sequence.</p></item><item><p> If the item is a node, the typed value of the node is appended to the result\n                   sequence. The typed value is a sequence of zero or more atomic values:\n                   specifically, the result of the <code>dm:typed-value</code> accessor as defined in\n                      <bibref ref=\"xpath-datamodel-30\"/> (See <xspecref spec=\"DM30\" ref=\"dm-typed-value\"/>).</p></item></ulist><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">A <phrase diff=\"add\" at=\"L\">type</phrase> error is raised <errorref class=\"TY\" code=\"0012\" type=\"type\"/> if an item in the\n             sequence <code>$arg</code> is a node that does not have a typed value. </p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\" diff=\"add\" at=\"B\">A <phrase diff=\"add\" at=\"L\">type</phrase> error is raised <errorref class=\"TY\" code=\"0013\" type=\"dynamic\"/>\n             if an item in the sequence <code>$arg</code> is a function item. </p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\" diff=\"add\" at=\"I\">A <phrase diff=\"add\" at=\"M\">dynamic</phrase> error is raised\n             if <code>$arg</code> is omitted\n             and the context item is <xtermref ref=\"dt-absent\" spec=\"DM30\">absent</xtermref>.</p></div>\n", "summary" : "<p>  Returns the result of atomizing a sequence, that is, replacing all nodes in the\n             sequence by their typed values.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [  ], "returns" : { "type" : "xs:anyAtomicType*", "description" : "" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "data", "qname" : "fn:data", "signature" : "($arg as item()*) as xs:anyAtomicType* external", "description" : " <div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">Returns the result of atomizing a sequence, that is, replacing all nodes in the\n             sequence by their typed values.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\"><example role=\"signature\"><proto name=\"data\" return-type=\"xs:anyAtomicType*\" isOp=\"no\" prefix=\"fn\" returnEmptyOk=\"no\" returnSeq=\"no\" returnVaries=\"no\" isSchema=\"no\" isDatatype=\"no\" isSpecial=\"no\"/></example><example role=\"signature\"><proto name=\"data\" return-type=\"xs:anyAtomicType*\" isOp=\"no\" prefix=\"fn\" returnEmptyOk=\"no\" returnSeq=\"no\" returnVaries=\"no\" isSchema=\"no\" isDatatype=\"no\" isSpecial=\"no\"><arg name=\"arg\" type=\"item()*\"/></proto></example></p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The zero-argument form of this function is <termref def=\"dt-deterministic\">deterministic</termref>, <termref def=\"dt-context-dependent\">context-dependent</termref>,  and <termref def=\"dt-focus-dependent\">focus-dependent</termref>. </p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The one-argument form of this function is <termref def=\"dt-deterministic\">deterministic</termref>, <termref def=\"dt-context-independent\">context-independent</termref>,  and <termref def=\"dt-focus-independent\">focus-independent</termref>. </p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\" diff=\"add\" at=\"F\">If the argument is omitted, it defaults to the context item\n                (<code>.</code>). The behavior of the function if the argument is omitted is exactly\n             the same as if the context item had been passed as the argument.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\"> The result of <code>fn:data</code> is the sequence of atomic values produced by\n             applying the following rules to each item in <code>$arg</code>:</p><ulist xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\"><item><p>If the item is an atomic value, it is appended to the result sequence.</p></item><item><p> If the item is a node, the typed value of the node is appended to the result\n                   sequence. The typed value is a sequence of zero or more atomic values:\n                   specifically, the result of the <code>dm:typed-value</code> accessor as defined in\n                      <bibref ref=\"xpath-datamodel-30\"/> (See <xspecref spec=\"DM30\" ref=\"dm-typed-value\"/>).</p></item></ulist><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">A <phrase diff=\"add\" at=\"L\">type</phrase> error is raised <errorref class=\"TY\" code=\"0012\" type=\"type\"/> if an item in the\n             sequence <code>$arg</code> is a node that does not have a typed value. </p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\" diff=\"add\" at=\"B\">A <phrase diff=\"add\" at=\"L\">type</phrase> error is raised <errorref class=\"TY\" code=\"0013\" type=\"dynamic\"/>\n             if an item in the sequence <code>$arg</code> is a function item. </p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\" diff=\"add\" at=\"I\">A <phrase diff=\"add\" at=\"M\">dynamic</phrase> error is raised\n             if <code>$arg</code> is omitted\n             and the context item is <xtermref ref=\"dt-absent\" spec=\"DM30\">absent</xtermref>.</p></div>\n", "summary" : "<p>  Returns the result of atomizing a sequence, that is, replacing all nodes in the\n             sequence by their typed values.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "arg", "type" : "item()", "occurrence" : "*", "description" : "" } ], "returns" : { "type" : "xs:anyAtomicType*", "description" : "" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 2, "name" : "dateTime", "qname" : "fn:dateTime", "signature" : "($arg1 as xs:date?, $arg2 as xs:time?) as xs:dateTime? external", "description" : " <div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">Returns an <code>xs:dateTime</code> value created by combining an\n                <code>xs:date</code> and an <code>xs:time</code>.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\"><example role=\"signature\"><proto name=\"dateTime\" return-type=\"xs:dateTime?\" isOp=\"no\" prefix=\"fn\" returnEmptyOk=\"no\" returnSeq=\"no\" returnVaries=\"no\" isSchema=\"no\" isDatatype=\"no\" isSpecial=\"no\"><arg name=\"arg1\" type=\"xs:date?\"/><arg name=\"arg2\" type=\"xs:time?\"/></proto></example></p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">This function is <termref def=\"dt-deterministic\">deterministic</termref>, <termref def=\"dt-context-independent\">context-independent</termref>,  and <termref def=\"dt-focus-independent\">focus-independent</termref>. </p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">If either <code>$arg1</code> or <code>$arg2</code> is the empty sequence the function\n             returns the empty sequence.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">Otherwise, the function returns an <code>xs:dateTime</code> whose date component is\n             equal to <code>$arg1</code> and whose time component is equal to <code>$arg2</code>.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The timezone of the result is computed as follows:</p><ulist xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\"><item><p>If neither argument has a timezone, the result has no timezone.</p></item><item><p>If exactly one of the arguments has a timezone, or if both arguments have the same\n                   timezone, the result has this timezone.</p></item></ulist><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">A <phrase diff=\"add\" at=\"L\">dynamic</phrase> error is raised <errorref class=\"RG\" code=\"0008\"/> if the two arguments both have\n             timezones and the timezones are different. </p></div>\n", "summary" : "<p>  Returns an  xs:dateTime  value created by combining an\n                 xs:date  and an  xs:time .</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "arg1", "type" : "xs:date", "occurrence" : "?", "description" : "" }, { "name" : "arg2", "type" : "xs:time", "occurrence" : "?", "description" : "" } ], "returns" : { "type" : "xs:dateTime?", "description" : "" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "day-from-date", "qname" : "fn:day-from-date", "signature" : "($arg as xs:date?) as xs:integer? external", "description" : " <div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">Returns the day component of an <code>xs:date</code>.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\"><example role=\"signature\"><proto name=\"day-from-date\" return-type=\"xs:integer?\" isOp=\"no\" prefix=\"fn\" returnEmptyOk=\"no\" returnSeq=\"no\" returnVaries=\"no\" isSchema=\"no\" isDatatype=\"no\" isSpecial=\"no\"><arg name=\"arg\" type=\"xs:date?\"/></proto></example></p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">This function is <termref def=\"dt-deterministic\">deterministic</termref>, <termref def=\"dt-context-independent\">context-independent</termref>,  and <termref def=\"dt-focus-independent\">focus-independent</termref>. </p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">If <code>$arg</code> is the empty sequence, the function returns the empty sequence.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">Otherwise, the function returns an <code>xs:integer</code> between 1 and 31, both\n             inclusive, representing the day component in the localized value of\n             <code>$arg</code>.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The expression <code>fn:day-from-date(xs:date(\"1999-05-31-05:00\"))</code> returns <code>31</code>.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The expression <code>fn:day-from-date(xs:date(\"2000-01-01+05:00\"))</code> returns <code>1</code>.</p></div>\n", "summary" : "<p>  Returns the day component of an  xs:date .</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "arg", "type" : "xs:date", "occurrence" : "?", "description" : "" } ], "returns" : { "type" : "xs:integer?", "description" : "" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "days-from-duration", "qname" : "fn:days-from-duration", "signature" : "($arg as xs:duration?) as xs:integer? external", "description" : " <div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">Returns the number of days in a duration.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\"><example role=\"signature\"><proto name=\"days-from-duration\" return-type=\"xs:integer?\" isOp=\"no\" prefix=\"fn\" returnEmptyOk=\"no\" returnSeq=\"no\" returnVaries=\"no\" isSchema=\"no\" isDatatype=\"no\" isSpecial=\"no\"><arg name=\"arg\" type=\"xs:duration?\"/></proto></example></p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">This function is <termref def=\"dt-deterministic\">deterministic</termref>, <termref def=\"dt-context-independent\">context-independent</termref>,  and <termref def=\"dt-focus-independent\">focus-independent</termref>. </p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">If <code>$arg</code> is the empty sequence, the function returns the empty sequence.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">Otherwise, the function returns an <code>xs:integer</code> representing the days\n             component in the value of <code>$arg</code>. The result is obtained by casting\n                <code>$arg</code> to an <code>xs:dayTimeDuration</code> (see <specref ref=\"casting-to-durations\"/>) and then computing the days component as described in\n                <specref ref=\"canonical-dayTimeDuration\"/>.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">If <code>$arg</code> is a negative duration then the result will be negative..</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">If <code>$arg</code> is an <code>xs:yearMonthDuration</code> the function returns 0.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The expression <code>fn:days-from-duration(xs:dayTimeDuration(\"P3DT10H\"))</code> returns <code>3</code>.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The expression <code>fn:days-from-duration(xs:dayTimeDuration(\"P3DT55H\"))</code> returns <code>5</code>.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The expression <code>fn:days-from-duration(xs:yearMonthDuration(\"P3Y5M\"))</code> returns <code>0</code>.</p></div>\n", "summary" : "<p>  Returns the number of days in a duration.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "arg", "type" : "xs:duration", "occurrence" : "?", "description" : "" } ], "returns" : { "type" : "xs:integer?", "description" : "" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 2, "name" : "deep-equal", "qname" : "fn:deep-equal", "signature" : "($parameter1 as item()*, $parameter2 as item()*) as xs:boolean external", "description" : " <div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\"> This function assesses whether two sequences are deep-equal to each other. To\n             be deep-equal, they must contain items that are pairwise deep-equal; and for two items\n             to be deep-equal, they must either be atomic values that compare equal, or nodes of the\n             same kind, with the same name, whose children are deep-equal.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\"><example role=\"signature\"><proto name=\"deep-equal\" return-type=\"xs:boolean\" isOp=\"no\" prefix=\"fn\" returnEmptyOk=\"no\" returnSeq=\"no\" returnVaries=\"no\" isSchema=\"no\" isDatatype=\"no\" isSpecial=\"no\"><arg name=\"parameter1\" type=\"item()*\"/><arg name=\"parameter2\" type=\"item()*\"/></proto></example><example role=\"signature\"><proto name=\"deep-equal\" return-type=\"xs:boolean\" isOp=\"no\" prefix=\"fn\" returnEmptyOk=\"no\" returnSeq=\"no\" returnVaries=\"no\" isSchema=\"no\" isDatatype=\"no\" isSpecial=\"no\"><arg name=\"parameter1\" type=\"item()*\"/><arg name=\"parameter2\" type=\"item()*\"/><arg name=\"collation\" type=\"xs:string\"/></proto></example></p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The two-argument form of this function is <termref def=\"dt-deterministic\">deterministic</termref>, <termref def=\"dt-context-dependent\">context-dependent</termref>,  and <termref def=\"dt-focus-independent\">focus-independent</termref>.  It depends on\n \t\tcollations, and implicit timezone.\n \t</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The three-argument form of this function is <termref def=\"dt-deterministic\">deterministic</termref>, <termref def=\"dt-context-dependent\">context-dependent</termref>,  and <termref def=\"dt-focus-independent\">focus-independent</termref>.  It depends on\n \t\tcollations, and static base uri, and implicit timezone.\n \t</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The <code>$collation</code> argument identifies a collation which is used at all levels\n             of recursion when strings are compared (but not when names are compared), according to\n             the rules in <specref ref=\"choosing-a-collation\"/>.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">If the two sequences are both empty, the function returns <code>true</code>.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">If the two sequences are of different lengths, the function returns\n             <code>false</code>.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">If the two sequences are of the same length, the function returns <code>true</code> if\n             and only if every item in the sequence <code>$parameter1</code> is deep-equal to the\n             item at the same position in the sequence <code>$parameter2</code>. The rules for\n             deciding whether two items are deep-equal follow.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">Call the two items <code>$i1</code> and <code>$i2</code> respectively.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">If <code>$i1</code> and <code>$i2</code> are both atomic values, they are deep-equal if\n             and only if <code>($i1 eq $i2)</code> is <code>true</code>, or if both values are\n                <code>NaN</code>. If the <code>eq</code> operator is not defined for <code>$i1</code>\n             and <code>$i2</code>, the function returns <code>false</code>.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\"> If one of the pair <code>$i1</code> or <code>$i2</code> is an atomic value and the\n             other is not,\n             <!--<phrase diff=\"add\" at=\"MAP\">or if one is a node and the other is not, </phrase>-->\n             the function returns <code>false</code>.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">If <code>$i1</code> and <code>$i2</code> are both nodes, they are compared as described\n             below:</p><olist xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\"><item><p>If the two nodes are of different kinds, the result is <code>false</code>.</p></item><item><p>If the two nodes are both document nodes then they are deep-equal if and only if\n                   the sequence <code>$i1/(*|text())</code> is deep-equal to the sequence\n                      <code>$i2/(*|text())</code>.</p></item><item><p> If the two nodes are both element nodes then they are deep-equal if and only if\n                   all of the following conditions are satisfied:</p><olist><item><p>The two nodes have the same name, that is <code>(node-name($i1) eq\n                            node-name($i2))</code>.</p></item><item><!-- bug 17252 --><p diff=\"chg\" at=\"L\">Either both nodes are both annotated as having simple content or both nodes are\n                         annotated as having complex content. For this purpose \"simple content\" means either a simple\n                      type or a complex type with simple content; \"complex content\" means a complex type whose variety\n                      is mixed, element-only, or empty.</p><note diff=\"add\" at=\"L\"><p>It is a consequence of this rule that validating a document\n                      <var>D</var> against a schema will usually (but not necessarily) result in a document that is not deep-equal\n                         to <var>D</var>. The exception is when the schema allows all elements to have mixed content.</p></note></item><item><p>The two nodes have the same number of attributes, and for every attribute\n                            <code>$a1</code> in <code>$i1/@*</code> there exists an attribute\n                            <code>$a2</code> in <code>$i2/@*</code> such that <code>$a1</code> and\n                            <code>$a2</code> are deep-equal.</p></item><item><p> One of the following conditions holds:</p><ulist><item><p>Both element nodes are annotated as having simple content\n                               <phrase diff=\"add\" at=\"L\">(as defined in 3(b) above)</phrase>, and\n                               the typed value of <code>$i1</code> is deep-equal to the typed value\n                               of <code>$i2</code>.</p></item><item><p>Both element nodes have a type annotation that is <phrase diff=\"chg\" at=\"L\">a complex type with\n                               variety element-only, and the sequence <code>$i1/*</code> is\n                               deep-equal to the sequence <code>$i2/*</code>.</phrase></p></item><item><p>Both element nodes have a type annotation that is <phrase diff=\"chg\" at=\"L\">a complex type with\n                               variety mixed</phrase>, and the sequence <code>$i1/(*|text())</code> is\n                               deep-equal to the sequence <code>$i2/(*|text())</code>.</p></item><item><p>Both element nodes have a type annotation that is <phrase diff=\"chg\" at=\"L\">a complex type with\n                               variety empty</phrase>.</p></item></ulist></item></olist></item><item><p>If the two nodes are both attribute nodes then they are deep-equal if and only if\n                   both the following conditions are satisfied:</p><olist><item><p>The two nodes have the same name, that is <code>(node-name($i1) eq\n                            node-name($i2))</code>.</p></item><item><p>The typed value of <code>$i1</code> is deep-equal to the typed value of\n                            <code>$i2</code>.</p></item></olist></item><item><p> If the two nodes are both processing instruction nodes<phrase diff=\"del\" at=\"A-E42\"> or namespace bindings</phrase>, then they are deep-equal if and\n                   only if both the following conditions are satisfied:</p><olist><item><p>The two nodes have the same name, that is <code>(node-name($i1) eq\n                            node-name($i2))</code>.</p></item><item><p>The string value of <code>$i1</code> is equal to the string value of\n                            <code>$i2</code>.</p></item></olist></item><item><p diff=\"add\" at=\"A-E42\"> If the two nodes are both namespace nodes, then they are deep-equal if and only\n                   if both the following conditions are satisfied:</p><olist><item><p diff=\"add\" at=\"A-E42\">The two nodes either have the same name or are both nameless, that is\n                            <code>fn:deep-equal(node-name($i1), node-name($i2))</code>.</p></item><item><p diff=\"add\" at=\"A-E42\">The string value of <code>$i1</code> is equal to the string value of\n                            <code>$i2</code> when compared using the Unicode codepoint collation.</p></item></olist></item><item><p>If the two nodes are both text nodes or comment nodes, then they are deep-equal if\n                   and only if their string-values are equal.</p></item></olist><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\" diff=\"add\" at=\"B\">A <phrase diff=\"add\" at=\"L\">type</phrase> error is raised <errorref class=\"TY\" code=\"0015\" type=\"type\"/>\n             if either input sequence contains a function item.\n             <!--<phrase diff=\"add\" at=\"MAP\">that is not a map</phrase>,-->\n          </p></div>\n", "summary" : "<p>   This function assesses whether two sequences are deep-equal to each other.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "parameter1", "type" : "item()", "occurrence" : "*", "description" : "" }, { "name" : "parameter2", "type" : "item()", "occurrence" : "*", "description" : "" } ], "returns" : { "type" : "xs:boolean", "description" : "" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 3, "name" : "deep-equal", "qname" : "fn:deep-equal", "signature" : "($parameter1 as item()*, $parameter2 as item()*, $collation as xs:string) as xs:boolean external", "description" : " <div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\"> This function assesses whether two sequences are deep-equal to each other. To\n             be deep-equal, they must contain items that are pairwise deep-equal; and for two items\n             to be deep-equal, they must either be atomic values that compare equal, or nodes of the\n             same kind, with the same name, whose children are deep-equal.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\"><example role=\"signature\"><proto name=\"deep-equal\" return-type=\"xs:boolean\" isOp=\"no\" prefix=\"fn\" returnEmptyOk=\"no\" returnSeq=\"no\" returnVaries=\"no\" isSchema=\"no\" isDatatype=\"no\" isSpecial=\"no\"><arg name=\"parameter1\" type=\"item()*\"/><arg name=\"parameter2\" type=\"item()*\"/></proto></example><example role=\"signature\"><proto name=\"deep-equal\" return-type=\"xs:boolean\" isOp=\"no\" prefix=\"fn\" returnEmptyOk=\"no\" returnSeq=\"no\" returnVaries=\"no\" isSchema=\"no\" isDatatype=\"no\" isSpecial=\"no\"><arg name=\"parameter1\" type=\"item()*\"/><arg name=\"parameter2\" type=\"item()*\"/><arg name=\"collation\" type=\"xs:string\"/></proto></example></p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The two-argument form of this function is <termref def=\"dt-deterministic\">deterministic</termref>, <termref def=\"dt-context-dependent\">context-dependent</termref>,  and <termref def=\"dt-focus-independent\">focus-independent</termref>.  It depends on\n \t\tcollations, and implicit timezone.\n \t</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The three-argument form of this function is <termref def=\"dt-deterministic\">deterministic</termref>, <termref def=\"dt-context-dependent\">context-dependent</termref>,  and <termref def=\"dt-focus-independent\">focus-independent</termref>.  It depends on\n \t\tcollations, and static base uri, and implicit timezone.\n \t</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The <code>$collation</code> argument identifies a collation which is used at all levels\n             of recursion when strings are compared (but not when names are compared), according to\n             the rules in <specref ref=\"choosing-a-collation\"/>.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">If the two sequences are both empty, the function returns <code>true</code>.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">If the two sequences are of different lengths, the function returns\n             <code>false</code>.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">If the two sequences are of the same length, the function returns <code>true</code> if\n             and only if every item in the sequence <code>$parameter1</code> is deep-equal to the\n             item at the same position in the sequence <code>$parameter2</code>. The rules for\n             deciding whether two items are deep-equal follow.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">Call the two items <code>$i1</code> and <code>$i2</code> respectively.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">If <code>$i1</code> and <code>$i2</code> are both atomic values, they are deep-equal if\n             and only if <code>($i1 eq $i2)</code> is <code>true</code>, or if both values are\n                <code>NaN</code>. If the <code>eq</code> operator is not defined for <code>$i1</code>\n             and <code>$i2</code>, the function returns <code>false</code>.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\"> If one of the pair <code>$i1</code> or <code>$i2</code> is an atomic value and the\n             other is not,\n             <!--<phrase diff=\"add\" at=\"MAP\">or if one is a node and the other is not, </phrase>-->\n             the function returns <code>false</code>.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">If <code>$i1</code> and <code>$i2</code> are both nodes, they are compared as described\n             below:</p><olist xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\"><item><p>If the two nodes are of different kinds, the result is <code>false</code>.</p></item><item><p>If the two nodes are both document nodes then they are deep-equal if and only if\n                   the sequence <code>$i1/(*|text())</code> is deep-equal to the sequence\n                      <code>$i2/(*|text())</code>.</p></item><item><p> If the two nodes are both element nodes then they are deep-equal if and only if\n                   all of the following conditions are satisfied:</p><olist><item><p>The two nodes have the same name, that is <code>(node-name($i1) eq\n                            node-name($i2))</code>.</p></item><item><!-- bug 17252 --><p diff=\"chg\" at=\"L\">Either both nodes are both annotated as having simple content or both nodes are\n                         annotated as having complex content. For this purpose \"simple content\" means either a simple\n                      type or a complex type with simple content; \"complex content\" means a complex type whose variety\n                      is mixed, element-only, or empty.</p><note diff=\"add\" at=\"L\"><p>It is a consequence of this rule that validating a document\n                      <var>D</var> against a schema will usually (but not necessarily) result in a document that is not deep-equal\n                         to <var>D</var>. The exception is when the schema allows all elements to have mixed content.</p></note></item><item><p>The two nodes have the same number of attributes, and for every attribute\n                            <code>$a1</code> in <code>$i1/@*</code> there exists an attribute\n                            <code>$a2</code> in <code>$i2/@*</code> such that <code>$a1</code> and\n                            <code>$a2</code> are deep-equal.</p></item><item><p> One of the following conditions holds:</p><ulist><item><p>Both element nodes are annotated as having simple content\n                               <phrase diff=\"add\" at=\"L\">(as defined in 3(b) above)</phrase>, and\n                               the typed value of <code>$i1</code> is deep-equal to the typed value\n                               of <code>$i2</code>.</p></item><item><p>Both element nodes have a type annotation that is <phrase diff=\"chg\" at=\"L\">a complex type with\n                               variety element-only, and the sequence <code>$i1/*</code> is\n                               deep-equal to the sequence <code>$i2/*</code>.</phrase></p></item><item><p>Both element nodes have a type annotation that is <phrase diff=\"chg\" at=\"L\">a complex type with\n                               variety mixed</phrase>, and the sequence <code>$i1/(*|text())</code> is\n                               deep-equal to the sequence <code>$i2/(*|text())</code>.</p></item><item><p>Both element nodes have a type annotation that is <phrase diff=\"chg\" at=\"L\">a complex type with\n                               variety empty</phrase>.</p></item></ulist></item></olist></item><item><p>If the two nodes are both attribute nodes then they are deep-equal if and only if\n                   both the following conditions are satisfied:</p><olist><item><p>The two nodes have the same name, that is <code>(node-name($i1) eq\n                            node-name($i2))</code>.</p></item><item><p>The typed value of <code>$i1</code> is deep-equal to the typed value of\n                            <code>$i2</code>.</p></item></olist></item><item><p> If the two nodes are both processing instruction nodes<phrase diff=\"del\" at=\"A-E42\"> or namespace bindings</phrase>, then they are deep-equal if and\n                   only if both the following conditions are satisfied:</p><olist><item><p>The two nodes have the same name, that is <code>(node-name($i1) eq\n                            node-name($i2))</code>.</p></item><item><p>The string value of <code>$i1</code> is equal to the string value of\n                            <code>$i2</code>.</p></item></olist></item><item><p diff=\"add\" at=\"A-E42\"> If the two nodes are both namespace nodes, then they are deep-equal if and only\n                   if both the following conditions are satisfied:</p><olist><item><p diff=\"add\" at=\"A-E42\">The two nodes either have the same name or are both nameless, that is\n                            <code>fn:deep-equal(node-name($i1), node-name($i2))</code>.</p></item><item><p diff=\"add\" at=\"A-E42\">The string value of <code>$i1</code> is equal to the string value of\n                            <code>$i2</code> when compared using the Unicode codepoint collation.</p></item></olist></item><item><p>If the two nodes are both text nodes or comment nodes, then they are deep-equal if\n                   and only if their string-values are equal.</p></item></olist><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\" diff=\"add\" at=\"B\">A <phrase diff=\"add\" at=\"L\">type</phrase> error is raised <errorref class=\"TY\" code=\"0015\" type=\"type\"/>\n             if either input sequence contains a function item.\n             <!--<phrase diff=\"add\" at=\"MAP\">that is not a map</phrase>,-->\n          </p></div>\n", "summary" : "<p>   This function assesses whether two sequences are deep-equal to each other.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "parameter1", "type" : "item()", "occurrence" : "*", "description" : "" }, { "name" : "parameter2", "type" : "item()", "occurrence" : "*", "description" : "" }, { "name" : "collation", "type" : "xs:string", "occurrence" : null, "description" : "" } ], "returns" : { "type" : "xs:boolean", "description" : "" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 0, "name" : "default-collation", "qname" : "fn:default-collation", "signature" : "() as xs:string external", "description" : " <div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">Returns the value of the default collation property from the static context.\n          </p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\"><example role=\"signature\"><proto name=\"default-collation\" return-type=\"xs:string\" isOp=\"no\" prefix=\"fn\" returnEmptyOk=\"no\" returnSeq=\"no\" returnVaries=\"no\" isSchema=\"no\" isDatatype=\"no\" isSpecial=\"no\"/></example></p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">This function is <termref def=\"dt-deterministic\">deterministic</termref>, <termref def=\"dt-context-dependent\">context-dependent</termref>,  and <termref def=\"dt-focus-independent\">focus-independent</termref>.  It depends on\n \t\tcollations.\n \t</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">Returns the value of the default collation property from the static context. Components\n             of the static context are discussed in <xspecref spec=\"XP30\" ref=\"id-xp-static-context-components\"/>.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The default collation property can never be absent. If it is not explicitly defined, a\n             system defined default can be invoked. If this is not provided, the Unicode codepoint\n             collation (<code>http://www.w3.org/2005/xpath-functions/collation/codepoint</code>) is\n             used. </p></div>\n", "summary" : "<p>  Returns the value of the default collation property from the static context.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [  ], "returns" : { "type" : "xs:string", "description" : "" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "distinct-values", "qname" : "fn:distinct-values", "signature" : "($arg as xs:anyAtomicType*) as xs:anyAtomicType* external", "description" : " <div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">Returns the values that appear in a sequence, with duplicates eliminated.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\"><example role=\"signature\"><proto name=\"distinct-values\" return-type=\"xs:anyAtomicType*\" isOp=\"no\" prefix=\"fn\" returnEmptyOk=\"no\" returnSeq=\"no\" returnVaries=\"no\" isSchema=\"no\" isDatatype=\"no\" isSpecial=\"no\"><arg name=\"arg\" type=\"xs:anyAtomicType*\"/></proto></example><example role=\"signature\"><proto name=\"distinct-values\" return-type=\"xs:anyAtomicType*\" isOp=\"no\" prefix=\"fn\" returnEmptyOk=\"no\" returnSeq=\"no\" returnVaries=\"no\" isSchema=\"no\" isDatatype=\"no\" isSpecial=\"no\"><arg name=\"arg\" type=\"xs:anyAtomicType*\"/><arg name=\"collation\" type=\"xs:string\"/></proto></example></p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The one-argument form of this function is <termref def=\"dt-deterministic\">deterministic</termref>, <termref def=\"dt-context-dependent\">context-dependent</termref>,  and <termref def=\"dt-focus-independent\">focus-independent</termref>.  It depends on\n \t\tcollations, and implicit timezone.\n \t</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The two-argument form of this function is <termref def=\"dt-deterministic\">deterministic</termref>, <termref def=\"dt-context-dependent\">context-dependent</termref>,  and <termref def=\"dt-focus-independent\">focus-independent</termref>.  It depends on\n \t\tcollations, and static base uri, and implicit timezone.\n \t</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The function returns the sequence that results from removing from <code>$arg</code> all\n             but one of a set of values that are equal to one another. Values are compared using the\n                <code>eq</code> operator, subject to the caveats defined below.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">Values of type <code>xs:untypedAtomic</code> are compared as if they were of type\n                <code>xs:string</code>.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">Values that cannot be compared, because the <code>eq</code> operator is not defined for\n             their types, are considered to be distinct.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The collation used by this function is determined according to the rules in <specref ref=\"choosing-a-collation\"/>. This collation is used when string comparison is\n             required.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">For <code>xs:float</code> and <code>xs:double</code> values, positive zero is equal to\n             negative zero and, although <code>NaN</code> does not equal itself, if <code>$arg</code>\n             contains multiple <code>NaN</code> values a single <code>NaN</code> is returned.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\"> If <code>xs:dateTime</code>, <code>xs:date</code> or <code>xs:time</code> values do not\n             have a timezone, they are considered to have the implicit timezone provided by the\n             dynamic context for the purpose of comparison. Note that <code>xs:dateTime</code>,\n                <code>xs:date</code> or <code>xs:time</code> values can compare equal even if their\n             timezones are different.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The order in which the sequence of values is returned is <termref def=\"implementation-dependent\"/>.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">Which value of a set of values that compare equal is returned is <termref def=\"implementation-dependent\"/>. </p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\" diff=\"del\" at=\"F\">The static type of the result is a sequence of prime types as defined\n             in [Formal Semantics].</p><change xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\" diff=\"add\" at=\"A-E44\">\n             <p>If the input sequence contains values of different numeric types that differ from\n                each other by small amounts, then the eq operator is not transitive, because of\n                rounding effects occurring during type promotion. In the situation where the input\n                contains three values <code>A</code>, <code>B</code>, and <code>C</code> such that\n                   <code>A eq B</code>, <code>B eq C</code>, but <code>A ne C</code>, then the number\n                of items in the result of the function (as well as the choice of which items are\n                returned) is <termref def=\"implementation-dependent\"/>, subject only to the\n                constraints that (a) no two items in the result sequence compare equal to each other,\n                and (b) every input item that does not appear in the result sequence compares equal\n                to some item that does appear in the result sequence.</p>\n             <p>For example, this arises when computing:</p>\n             <eg xml:space=\"preserve\">    distinct-values(\n             (xs:float('1.0'),\n             xs:decimal('1.0000000000100000000001',\n             xs:double( '1.00000000001'))</eg>\n             <p>because the values of type <code>xs:float</code> and <code>xs:double</code> both\n                compare equal to the value of type <code>xs:decimal</code> but not equal to each\n                other. </p>\n          </change><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">If <code>$arg</code> is the empty sequence, the function returns the empty sequence.</p></div>\n", "summary" : "<p>  Returns the values that appear in a sequence, with duplicates eliminated.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "arg", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "" } ], "returns" : { "type" : "xs:anyAtomicType*", "description" : "" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 2, "name" : "distinct-values", "qname" : "fn:distinct-values", "signature" : "($arg as xs:anyAtomicType*, $collation as xs:string) as xs:anyAtomicType* external", "description" : " <div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">Returns the values that appear in a sequence, with duplicates eliminated.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\"><example role=\"signature\"><proto name=\"distinct-values\" return-type=\"xs:anyAtomicType*\" isOp=\"no\" prefix=\"fn\" returnEmptyOk=\"no\" returnSeq=\"no\" returnVaries=\"no\" isSchema=\"no\" isDatatype=\"no\" isSpecial=\"no\"><arg name=\"arg\" type=\"xs:anyAtomicType*\"/></proto></example><example role=\"signature\"><proto name=\"distinct-values\" return-type=\"xs:anyAtomicType*\" isOp=\"no\" prefix=\"fn\" returnEmptyOk=\"no\" returnSeq=\"no\" returnVaries=\"no\" isSchema=\"no\" isDatatype=\"no\" isSpecial=\"no\"><arg name=\"arg\" type=\"xs:anyAtomicType*\"/><arg name=\"collation\" type=\"xs:string\"/></proto></example></p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The one-argument form of this function is <termref def=\"dt-deterministic\">deterministic</termref>, <termref def=\"dt-context-dependent\">context-dependent</termref>,  and <termref def=\"dt-focus-independent\">focus-independent</termref>.  It depends on\n \t\tcollations, and implicit timezone.\n \t</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The two-argument form of this function is <termref def=\"dt-deterministic\">deterministic</termref>, <termref def=\"dt-context-dependent\">context-dependent</termref>,  and <termref def=\"dt-focus-independent\">focus-independent</termref>.  It depends on\n \t\tcollations, and static base uri, and implicit timezone.\n \t</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The function returns the sequence that results from removing from <code>$arg</code> all\n             but one of a set of values that are equal to one another. Values are compared using the\n                <code>eq</code> operator, subject to the caveats defined below.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">Values of type <code>xs:untypedAtomic</code> are compared as if they were of type\n                <code>xs:string</code>.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">Values that cannot be compared, because the <code>eq</code> operator is not defined for\n             their types, are considered to be distinct.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The collation used by this function is determined according to the rules in <specref ref=\"choosing-a-collation\"/>. This collation is used when string comparison is\n             required.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">For <code>xs:float</code> and <code>xs:double</code> values, positive zero is equal to\n             negative zero and, although <code>NaN</code> does not equal itself, if <code>$arg</code>\n             contains multiple <code>NaN</code> values a single <code>NaN</code> is returned.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\"> If <code>xs:dateTime</code>, <code>xs:date</code> or <code>xs:time</code> values do not\n             have a timezone, they are considered to have the implicit timezone provided by the\n             dynamic context for the purpose of comparison. Note that <code>xs:dateTime</code>,\n                <code>xs:date</code> or <code>xs:time</code> values can compare equal even if their\n             timezones are different.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The order in which the sequence of values is returned is <termref def=\"implementation-dependent\"/>.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">Which value of a set of values that compare equal is returned is <termref def=\"implementation-dependent\"/>. </p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\" diff=\"del\" at=\"F\">The static type of the result is a sequence of prime types as defined\n             in [Formal Semantics].</p><change xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\" diff=\"add\" at=\"A-E44\">\n             <p>If the input sequence contains values of different numeric types that differ from\n                each other by small amounts, then the eq operator is not transitive, because of\n                rounding effects occurring during type promotion. In the situation where the input\n                contains three values <code>A</code>, <code>B</code>, and <code>C</code> such that\n                   <code>A eq B</code>, <code>B eq C</code>, but <code>A ne C</code>, then the number\n                of items in the result of the function (as well as the choice of which items are\n                returned) is <termref def=\"implementation-dependent\"/>, subject only to the\n                constraints that (a) no two items in the result sequence compare equal to each other,\n                and (b) every input item that does not appear in the result sequence compares equal\n                to some item that does appear in the result sequence.</p>\n             <p>For example, this arises when computing:</p>\n             <eg xml:space=\"preserve\">    distinct-values(\n             (xs:float('1.0'),\n             xs:decimal('1.0000000000100000000001',\n             xs:double( '1.00000000001'))</eg>\n             <p>because the values of type <code>xs:float</code> and <code>xs:double</code> both\n                compare equal to the value of type <code>xs:decimal</code> but not equal to each\n                other. </p>\n          </change><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">If <code>$arg</code> is the empty sequence, the function returns the empty sequence.</p></div>\n", "summary" : "<p>  Returns the values that appear in a sequence, with duplicates eliminated.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "arg", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "" }, { "name" : "collation", "type" : "xs:string", "occurrence" : null, "description" : "" } ], "returns" : { "type" : "xs:anyAtomicType*", "description" : "" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "doc-available", "qname" : "fn:doc-available", "signature" : "($uri as xs:string?) as xs:boolean external", "description" : " <div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\" diff=\"chg\" at=\"A-E26\">The function returns true if and only if the function\n             call <code>fn:doc($uri)</code> would return a document node.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\"><example role=\"signature\"><proto name=\"doc-available\" return-type=\"xs:boolean\" isOp=\"no\" prefix=\"fn\" returnEmptyOk=\"no\" returnSeq=\"no\" returnVaries=\"no\" isSchema=\"no\" isDatatype=\"no\" isSpecial=\"no\"><arg name=\"uri\" type=\"xs:string?\"/></proto></example></p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">This function is <termref def=\"dt-deterministic\">deterministic</termref>, <termref def=\"dt-context-dependent\">context-dependent</termref>,  and <termref def=\"dt-focus-independent\">focus-independent</termref>.  It depends on\n \t\tavailable documents, and static base uri.\n \t</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\" diff=\"chg\" at=\"A-E26\">If <code>$uri</code> is an empty sequence, this function returns\n                <code>false</code>.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\" diff=\"chg\" at=\"A-E26\">If a call on <code>fn:doc($uri)</code> would return a document\n             node, this function returns <code>true</code>.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\" diff=\"chg\" at=\"A-E26\">A <phrase diff=\"add\" at=\"L\">dynamic</phrase> error is raised <errorref class=\"DC\" code=\"0005\"/> if\n                <code>$uri</code> is not a valid URI according to the rules applied by the\n             implementation of <code>fn:doc</code>.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\" diff=\"chg\" at=\"A-E26\">Otherwise, this function returns <code>false</code>.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">If this function returns <code>true</code>, then calling <code>fn:doc($uri)</code>\n             within the same <termref def=\"execution-scope\"/> must return a document node. However,\n             if nondeterministic processing has been selected for the <code>fn:doc</code> function,\n             this guarantee is lost.</p></div>\n", "summary" : "<p>  The function returns true if and only if the function\n             call  fn:doc($uri)  would return a document node.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "uri", "type" : "xs:string", "occurrence" : "?", "description" : "" } ], "returns" : { "type" : "xs:boolean", "description" : "" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "doc", "qname" : "fn:doc", "signature" : "($uri as xs:string?) as document()? external", "description" : " <div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\" diff=\"chg\" at=\"A-E26\">Retrieves a document using a URI supplied as an\n                <code>xs:string</code>, and returns the corresponding document node.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\"><example role=\"signature\"><proto name=\"doc\" return-type=\"document-node()?\" isOp=\"no\" prefix=\"fn\" returnEmptyOk=\"no\" returnSeq=\"no\" returnVaries=\"no\" isSchema=\"no\" isDatatype=\"no\" isSpecial=\"no\"><arg name=\"uri\" type=\"xs:string?\"/></proto></example></p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">This function is <termref def=\"dt-deterministic\">deterministic</termref>, <termref def=\"dt-context-dependent\">context-dependent</termref>,  and <termref def=\"dt-focus-independent\">focus-independent</termref>.  It depends on\n \t\tavailable documents, and static base uri.\n \t</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">If <code>$uri</code> is the empty sequence, the result is an empty sequence.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\" diff=\"add\" at=\"A-E26\">If <code>$uri</code> is a relative URI reference, it is resolved\n             relative to the value of the <phrase diff=\"chg\" at=\"L\">Static Base URI property from the static context</phrase>. The resulting\n             absolute URI is promoted to an <code>xs:string</code>.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\" diff=\"add\" at=\"A-E26\">If the <term>Available documents</term> described in <xspecref spec=\"XP30\" ref=\"eval_context\"/> provides a mapping from this string to a document\n             node, the function returns that document node.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\" diff=\"add\" at=\"A-E26\">The URI may include a fragment identifier.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">By default, this function is <termref def=\"deterministic\">deterministic</termref>. Two calls on this function\n             return the same document node if the same URI Reference (after resolution to an absolute\n             URI Reference) is supplied to both calls. Thus, the following expression (if it does not\n             raise an error) will always be true:</p><eg xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\" xml:space=\"preserve\">doc(\"foo.xml\") is doc(\"foo.xml\")</eg><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">However, for performance reasons, implementations may provide a user option to evaluate\n             the function without a guarantee of determinism. The manner in which any such option is\n             provided is implementation-defined. If the user has not selected such an option, a call\n             of the function must either return a deterministic result or must raise a <phrase diff=\"add\" at=\"L\">dynamic</phrase> error\n                <errorref class=\"DC\" code=\"0003\"/>.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\" diff=\"del\" at=\"F\">For detailed type semantics, see [Formal Semantics].</p><note xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\"><p>If <code>$uri</code> is read from a source document, it is generally appropriate to\n                resolve it relative to the base URI property of the relevant node in the source\n                document. This can be achieved by calling the <code>fn:resolve-uri</code> function,\n                and passing the resulting absolute URI as an argument to the <code>fn:doc</code>\n                function.</p></note><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">If two calls to this function supply different absolute URI References as arguments, the\n             same document node may be returned if the implementation can determine that the two\n             arguments refer to the same resource.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\"> By defining the semantics of this function in terms of a string-to-document-node\n             mapping in the dynamic context, the specification is acknowledging that the results of\n             this function are outside the purview of the language specification itself, and depend\n             entirely on the run-time environment in which the expression is evaluated. This run-time\n             environment includes not only an unpredictable collection of resources (\"the web\"), but\n             configurable machinery for locating resources and turning their contents into document\n             nodes within the XPath data model. Both the set of resources that are reachable, and the\n             mechanisms by which those resources are parsed and validated, are <termref def=\"implementation-dependent\"/>.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\"> One possible processing model for this function is as follows. The resource identified\n             by the URI Reference is retrieved. If the resource cannot be retrieved, a <phrase diff=\"add\" at=\"L\">dynamic</phrase> error is\n             raised <errorref class=\"DC\" code=\"0002\"/>. The data resulting from the retrieval action\n             is then parsed as an XML document and a tree is constructed in accordance with the\n                <bibref ref=\"xpath-datamodel-30\"/>. If the top-level media type is known and is\n             \"text\", the content is parsed in the same way as if the media type were text/xml;\n             otherwise, it is parsed in the same way as if the media type were application/xml. If\n             the contents cannot be parsed successfully, a <phrase diff=\"add\" at=\"L\">dynamic</phrase> error is raised <errorref class=\"DC\" code=\"0002\"/>. Otherwise, the result of the function is the document node at the root\n             of the resulting tree. This tree is then optionally validated against a schema.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">Various aspects of this processing are <termref def=\"implementation-defined\"/>.\n             Implementations may provide external configuration options that allow any aspect of the\n             processing to be controlled by the user. In particular:</p><ulist xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\"><item><p>The set of URI schemes that the implementation recognizes is\n                   implementation-defined. Implementations may allow the mapping of URIs to resources\n                   to be configured by the user, using mechanisms such as catalogs or user-written\n                   URI handlers.</p></item><item><p>The handling of non-XML media types is implementation-defined. Implementations may\n                   allow instances of the data model to be constructed from non-XML resources, under\n                   user control.</p></item><item><p>It is <termref def=\"implementation-defined\"/> whether DTD validation and/or schema\n                   validation is applied to the source document.</p></item><item><p>Implementations may provide user-defined error handling options that allow\n                   processing to continue following an error in retrieving a resource, or in parsing\n                   and validating its content. When errors have been handled in this way, the\n                   function may return either an empty sequence, or a fallback document provided by\n                   the error handler.</p></item><item><p>Implementations may provide user options that relax the requirement for the\n                   function to return deterministic results.</p></item></ulist><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\" diff=\"add\" at=\"A-E26\">A <phrase diff=\"add\" at=\"L\">dynamic</phrase> error <rfc2119>may</rfc2119> be raised <errorref class=\"DC\" code=\"0005\"/> if <code>$uri</code> is not a valid URI.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\" diff=\"chg\" at=\"G\">A <phrase diff=\"add\" at=\"L\">dynamic</phrase> error is raised <errorref class=\"DC\" code=\"0002\"/> if the\n                <term>available documents</term> provides no mapping for the absolutized URI.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">A <phrase diff=\"add\" at=\"L\">dynamic</phrase> error is raised <errorref class=\"DC\" code=\"0002\"/> if the resource cannot be\n             retrieved or cannot be parsed successfully as XML.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">A <phrase diff=\"add\" at=\"L\">dynamic</phrase> error is raised <errorref class=\"DC\" code=\"0003\"/> if the implementation is not able\n             to guarantee that the result of the function will be deterministic, and the user has not\n             indicated that an unstable result is acceptable.</p></div>\n", "summary" : "<p>  Retrieves a document using a URI supplied as an\n                 xs:string , and returns the corresponding document node.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "uri", "type" : "xs:string", "occurrence" : "?", "description" : "" } ], "returns" : { "type" : "document()?", "description" : "" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 0, "name" : "document-uri", "qname" : "fn:document-uri", "signature" : "() as xs:anyURI? external", "description" : " <div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">Returns the URI of a resource where a document can be found, if available.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\"><example role=\"signature\"><proto name=\"document-uri\" return-type=\"xs:anyURI?\" isOp=\"no\" prefix=\"fn\" returnEmptyOk=\"no\" returnSeq=\"no\" returnVaries=\"no\" isSchema=\"no\" isDatatype=\"no\" isSpecial=\"no\"/></example><example role=\"signature\"><proto name=\"document-uri\" return-type=\"xs:anyURI?\" isOp=\"no\" prefix=\"fn\" returnEmptyOk=\"no\" returnSeq=\"no\" returnVaries=\"no\" isSchema=\"no\" isDatatype=\"no\" isSpecial=\"no\"><arg name=\"arg\" type=\"node()?\"/></proto></example></p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The zero-argument form of this function is <termref def=\"dt-deterministic\">deterministic</termref>, <termref def=\"dt-context-dependent\">context-dependent</termref>,  and <termref def=\"dt-focus-dependent\">focus-dependent</termref>. </p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The one-argument form of this function is <termref def=\"dt-deterministic\">deterministic</termref>, <termref def=\"dt-context-independent\">context-independent</termref>,  and <termref def=\"dt-focus-independent\">focus-independent</termref>. </p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\" diff=\"add\" at=\"F\">If the argument is omitted, it defaults to the context item\n                (<code>.</code>). The behavior of the function if the argument is omitted is exactly\n             the same as if the context item had been passed as the argument.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">If <code>$arg</code> is the empty sequence, the function returns the empty sequence.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">If <code>$arg</code> is not a document node, the function returns the empty\n             sequence.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">Otherwise, the function returns the value of the <code>document-uri</code> accessor\n             applied to <code>$arg</code>, as defined in <bibref ref=\"xpath-datamodel-30\"/> (See\n                <xspecref spec=\"DM30\" ref=\"DocumentNodeAccessors\"/>).</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The following errors may be raised when <code>$arg</code> is omitted:</p><ul xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\"><li><p>If the context\n                item is <xtermref ref=\"dt-absent\" spec=\"DM30\">absent</xtermref>, <phrase diff=\"add\" at=\"M\">dynamic error</phrase>\n                <xerrorref spec=\"XP\" class=\"DY\" code=\"0002\" type=\"dynamic\"/></p></li><li><p>If the context item is not a\n                node, <phrase diff=\"add\" at=\"M\">type error</phrase> <xerrorref spec=\"XP\" class=\"TY\" code=\"0004\" type=\"type\"/>.</p></li></ul></div>\n", "summary" : "<p>  Returns the URI of a resource where a document can be found, if available.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [  ], "returns" : { "type" : "xs:anyURI?", "description" : "" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "document-uri", "qname" : "fn:document-uri", "signature" : "($arg as node()?) as xs:anyURI? external", "description" : " <div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">Returns the URI of a resource where a document can be found, if available.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\"><example role=\"signature\"><proto name=\"document-uri\" return-type=\"xs:anyURI?\" isOp=\"no\" prefix=\"fn\" returnEmptyOk=\"no\" returnSeq=\"no\" returnVaries=\"no\" isSchema=\"no\" isDatatype=\"no\" isSpecial=\"no\"/></example><example role=\"signature\"><proto name=\"document-uri\" return-type=\"xs:anyURI?\" isOp=\"no\" prefix=\"fn\" returnEmptyOk=\"no\" returnSeq=\"no\" returnVaries=\"no\" isSchema=\"no\" isDatatype=\"no\" isSpecial=\"no\"><arg name=\"arg\" type=\"node()?\"/></proto></example></p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The zero-argument form of this function is <termref def=\"dt-deterministic\">deterministic</termref>, <termref def=\"dt-context-dependent\">context-dependent</termref>,  and <termref def=\"dt-focus-dependent\">focus-dependent</termref>. </p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The one-argument form of this function is <termref def=\"dt-deterministic\">deterministic</termref>, <termref def=\"dt-context-independent\">context-independent</termref>,  and <termref def=\"dt-focus-independent\">focus-independent</termref>. </p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\" diff=\"add\" at=\"F\">If the argument is omitted, it defaults to the context item\n                (<code>.</code>). The behavior of the function if the argument is omitted is exactly\n             the same as if the context item had been passed as the argument.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">If <code>$arg</code> is the empty sequence, the function returns the empty sequence.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">If <code>$arg</code> is not a document node, the function returns the empty\n             sequence.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">Otherwise, the function returns the value of the <code>document-uri</code> accessor\n             applied to <code>$arg</code>, as defined in <bibref ref=\"xpath-datamodel-30\"/> (See\n                <xspecref spec=\"DM30\" ref=\"DocumentNodeAccessors\"/>).</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The following errors may be raised when <code>$arg</code> is omitted:</p><ul xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\"><li><p>If the context\n                item is <xtermref ref=\"dt-absent\" spec=\"DM30\">absent</xtermref>, <phrase diff=\"add\" at=\"M\">dynamic error</phrase>\n                <xerrorref spec=\"XP\" class=\"DY\" code=\"0002\" type=\"dynamic\"/></p></li><li><p>If the context item is not a\n                node, <phrase diff=\"add\" at=\"M\">type error</phrase> <xerrorref spec=\"XP\" class=\"TY\" code=\"0004\" type=\"type\"/>.</p></li></ul></div>\n", "summary" : "<p>  Returns the URI of a resource where a document can be found, if available.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "arg", "type" : "node()", "occurrence" : "?", "description" : "" } ], "returns" : { "type" : "xs:anyURI?", "description" : "" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "element-with-id", "qname" : "fn:element-with-id", "signature" : "($arg as xs:string*) as element(*)* external", "description" : " <div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\" diff=\"add\" at=\"A-E31\"> Returns the sequence of element nodes that have an\n                <code>ID</code> value matching the value of one or more of the <code>IDREF</code>\n             values supplied in <code>$arg</code>.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\"><example role=\"signature\"><proto name=\"element-with-id\" return-type=\"element()*\" isOp=\"no\" prefix=\"fn\" returnEmptyOk=\"no\" returnSeq=\"no\" returnVaries=\"no\" isSchema=\"no\" isDatatype=\"no\" isSpecial=\"no\"><arg name=\"arg\" type=\"xs:string*\"/></proto></example><example role=\"signature\"><proto name=\"element-with-id\" return-type=\"element()*\" isOp=\"no\" prefix=\"fn\" returnEmptyOk=\"no\" returnSeq=\"no\" returnVaries=\"no\" isSchema=\"no\" isDatatype=\"no\" isSpecial=\"no\"><arg name=\"arg\" type=\"xs:string*\"/><arg name=\"node\" type=\"node()\"/></proto></example></p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The one-argument form of this function is <termref def=\"dt-deterministic\">deterministic</termref>, <termref def=\"dt-context-dependent\">context-dependent</termref>,  and <termref def=\"dt-focus-dependent\">focus-dependent</termref>. </p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The two-argument form of this function is <termref def=\"dt-deterministic\">deterministic</termref>, <termref def=\"dt-context-independent\">context-independent</termref>,  and <termref def=\"dt-focus-independent\">focus-independent</termref>. </p><change xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\" diff=\"add\" at=\"A-E31\">\n             <note><p>The effect of this function is identical to <function>fn:id</function> in respect\n                   of elements that have an attribute with the <code>is-id</code> property. However,\n                   it behaves differently in respect of element nodes with the <code>is-id</code>\n                   property. Whereas the <code>fn:id</code>, for legacy reasons, returns the element\n                   that has the <code>is-id</code> property, this parent returns the element\n                   identified by the ID, which is the parent of the element having the\n                      <code>is-id</code> property.</p></note>\n             <p>The function returns a sequence, in document order with duplicates eliminated,\n                containing every element node <code>E</code> that satisfies all the following\n                conditions:</p>\n             <olist><item><p>\n                      <code>E</code> is in the target document. The target document is the document\n                      containing <code>$node</code>, or the document containing the context item\n                         (<code>.</code>) if the second argument is omitted. The behavior of the\n                      function if <code>$node</code> is omitted is exactly the same as if the context\n                      item had been passed as <code>$node</code>.</p></item><item><p><code>E</code> has an <code>ID</code> value equal to one of the candidate\n                         <code>IDREF</code> values, where:</p><ulist><item><p> An element has an <code>ID</code> value equal to <code>V</code> if\n                            either or both of the following conditions are true:</p><ulist><item><p>The element has an child element node whose <code>is-id</code>\n                                  property (See <xspecref spec=\"DM30\" ref=\"dm-is-id\"/>.) is true and\n                                  whose typed value is equal to <code>V</code> under the rules of the\n                                     <code>eq</code> operator using the Unicode code point collation\n                                     (<code>http://www.w3.org/2005/xpath-functions/collation/codepoint</code>).</p></item><item><p>The element has an attribute node whose <code>is-id</code> property\n                                  (See <xspecref spec=\"DM30\" ref=\"dm-is-id\"/>.) is true and whose\n                                  typed value is equal to <code>V</code> under the rules of the\n                                     <code>eq</code> operator using the Unicode code point collation\n                                     (<code>http://www.w3.org/2005/xpath-functions/collation/codepoint</code>).</p></item></ulist></item><item><p>Each <code>xs:string</code> in <code>$arg</code> is parsed as if it were\n                            of type <code>IDREFS</code>, that is, each <code>xs:string</code> in\n                               <code>$arg</code> is treated as a whitespace-separated sequence of\n                            tokens, each acting as an <code>IDREF</code>. These tokens are then\n                            included in the list of candidate <code>IDREF</code>s. If any of the\n                            tokens is not a lexically valid <code>IDREF</code> (that is, if it is not\n                            lexically an <code>xs:NCName</code>), it is ignored. Formally, the\n                            candidate <code>IDREF</code> values are the strings in the sequence given\n                            by the expression:</p><eg xml:space=\"preserve\">for $s in $arg return\n    fn:tokenize(fn:normalize-space($s), ' ')[. castable as xs:IDREF]</eg></item></ulist></item><item><p> If several elements have the same <code>ID</code> value, then <code>E</code>\n                      is the one that is first in document order.</p></item></olist>\n          </change><change xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\" diff=\"add\" at=\"A-E31\">\n             <p>A <phrase diff=\"add\" at=\"L\">dynamic</phrase> error is raised <errorref class=\"DC\" code=\"0001\" type=\"dynamic\"/> if\n                   <code>$node</code>, or the context item if the second argument is omitted, is a\n                   node in a tree whose root is not a document node.</p>\n             <p>The following errors may be raised when <code>$node</code> is omitted:</p>\n             <ul><li><p>If the context\n                   item is <xtermref ref=\"dt-absent\" spec=\"DM30\">absent</xtermref>, <phrase diff=\"add\" at=\"M\">dynamic error</phrase>\n                   <xerrorref spec=\"XP\" class=\"DY\" code=\"0002\" type=\"dynamic\"/></p></li><li><p>If the context item is not a\n                   node, <phrase diff=\"add\" at=\"M\">type error</phrase> <xerrorref spec=\"XP\" class=\"TY\" code=\"0004\" type=\"type\"/>.</p></li></ul>\n          </change></div>\n", "summary" : "<p>   Returns the sequence of element nodes that have an\n                 ID  value matching the value of one or more of the  IDREF \n             values supplied in  $arg .</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "arg", "type" : "xs:string", "occurrence" : "*", "description" : "" } ], "returns" : { "type" : "element(*)*", "description" : "" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 2, "name" : "element-with-id", "qname" : "fn:element-with-id", "signature" : "($arg as xs:string*, $node as node()) as element(*)* external", "description" : " <div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\" diff=\"add\" at=\"A-E31\"> Returns the sequence of element nodes that have an\n                <code>ID</code> value matching the value of one or more of the <code>IDREF</code>\n             values supplied in <code>$arg</code>.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\"><example role=\"signature\"><proto name=\"element-with-id\" return-type=\"element()*\" isOp=\"no\" prefix=\"fn\" returnEmptyOk=\"no\" returnSeq=\"no\" returnVaries=\"no\" isSchema=\"no\" isDatatype=\"no\" isSpecial=\"no\"><arg name=\"arg\" type=\"xs:string*\"/></proto></example><example role=\"signature\"><proto name=\"element-with-id\" return-type=\"element()*\" isOp=\"no\" prefix=\"fn\" returnEmptyOk=\"no\" returnSeq=\"no\" returnVaries=\"no\" isSchema=\"no\" isDatatype=\"no\" isSpecial=\"no\"><arg name=\"arg\" type=\"xs:string*\"/><arg name=\"node\" type=\"node()\"/></proto></example></p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The one-argument form of this function is <termref def=\"dt-deterministic\">deterministic</termref>, <termref def=\"dt-context-dependent\">context-dependent</termref>,  and <termref def=\"dt-focus-dependent\">focus-dependent</termref>. </p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The two-argument form of this function is <termref def=\"dt-deterministic\">deterministic</termref>, <termref def=\"dt-context-independent\">context-independent</termref>,  and <termref def=\"dt-focus-independent\">focus-independent</termref>. </p><change xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\" diff=\"add\" at=\"A-E31\">\n             <note><p>The effect of this function is identical to <function>fn:id</function> in respect\n                   of elements that have an attribute with the <code>is-id</code> property. However,\n                   it behaves differently in respect of element nodes with the <code>is-id</code>\n                   property. Whereas the <code>fn:id</code>, for legacy reasons, returns the element\n                   that has the <code>is-id</code> property, this parent returns the element\n                   identified by the ID, which is the parent of the element having the\n                      <code>is-id</code> property.</p></note>\n             <p>The function returns a sequence, in document order with duplicates eliminated,\n                containing every element node <code>E</code> that satisfies all the following\n                conditions:</p>\n             <olist><item><p>\n                      <code>E</code> is in the target document. The target document is the document\n                      containing <code>$node</code>, or the document containing the context item\n                         (<code>.</code>) if the second argument is omitted. The behavior of the\n                      function if <code>$node</code> is omitted is exactly the same as if the context\n                      item had been passed as <code>$node</code>.</p></item><item><p><code>E</code> has an <code>ID</code> value equal to one of the candidate\n                         <code>IDREF</code> values, where:</p><ulist><item><p> An element has an <code>ID</code> value equal to <code>V</code> if\n                            either or both of the following conditions are true:</p><ulist><item><p>The element has an child element node whose <code>is-id</code>\n                                  property (See <xspecref spec=\"DM30\" ref=\"dm-is-id\"/>.) is true and\n                                  whose typed value is equal to <code>V</code> under the rules of the\n                                     <code>eq</code> operator using the Unicode code point collation\n                                     (<code>http://www.w3.org/2005/xpath-functions/collation/codepoint</code>).</p></item><item><p>The element has an attribute node whose <code>is-id</code> property\n                                  (See <xspecref spec=\"DM30\" ref=\"dm-is-id\"/>.) is true and whose\n                                  typed value is equal to <code>V</code> under the rules of the\n                                     <code>eq</code> operator using the Unicode code point collation\n                                     (<code>http://www.w3.org/2005/xpath-functions/collation/codepoint</code>).</p></item></ulist></item><item><p>Each <code>xs:string</code> in <code>$arg</code> is parsed as if it were\n                            of type <code>IDREFS</code>, that is, each <code>xs:string</code> in\n                               <code>$arg</code> is treated as a whitespace-separated sequence of\n                            tokens, each acting as an <code>IDREF</code>. These tokens are then\n                            included in the list of candidate <code>IDREF</code>s. If any of the\n                            tokens is not a lexically valid <code>IDREF</code> (that is, if it is not\n                            lexically an <code>xs:NCName</code>), it is ignored. Formally, the\n                            candidate <code>IDREF</code> values are the strings in the sequence given\n                            by the expression:</p><eg xml:space=\"preserve\">for $s in $arg return\n    fn:tokenize(fn:normalize-space($s), ' ')[. castable as xs:IDREF]</eg></item></ulist></item><item><p> If several elements have the same <code>ID</code> value, then <code>E</code>\n                      is the one that is first in document order.</p></item></olist>\n          </change><change xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\" diff=\"add\" at=\"A-E31\">\n             <p>A <phrase diff=\"add\" at=\"L\">dynamic</phrase> error is raised <errorref class=\"DC\" code=\"0001\" type=\"dynamic\"/> if\n                   <code>$node</code>, or the context item if the second argument is omitted, is a\n                   node in a tree whose root is not a document node.</p>\n             <p>The following errors may be raised when <code>$node</code> is omitted:</p>\n             <ul><li><p>If the context\n                   item is <xtermref ref=\"dt-absent\" spec=\"DM30\">absent</xtermref>, <phrase diff=\"add\" at=\"M\">dynamic error</phrase>\n                   <xerrorref spec=\"XP\" class=\"DY\" code=\"0002\" type=\"dynamic\"/></p></li><li><p>If the context item is not a\n                   node, <phrase diff=\"add\" at=\"M\">type error</phrase> <xerrorref spec=\"XP\" class=\"TY\" code=\"0004\" type=\"type\"/>.</p></li></ul>\n          </change></div>\n", "summary" : "<p>   Returns the sequence of element nodes that have an\n                 ID  value matching the value of one or more of the  IDREF \n             values supplied in  $arg .</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "arg", "type" : "xs:string", "occurrence" : "*", "description" : "" }, { "name" : "node", "type" : "node()", "occurrence" : null, "description" : "" } ], "returns" : { "type" : "element(*)*", "description" : "" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "empty", "qname" : "fn:empty", "signature" : "($arg as item()*) as xs:boolean external", "description" : " <div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">Returns true if the argument is the empty sequence.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\"><example role=\"signature\"><proto name=\"empty\" return-type=\"xs:boolean\" isOp=\"no\" prefix=\"fn\" returnEmptyOk=\"no\" returnSeq=\"no\" returnVaries=\"no\" isSchema=\"no\" isDatatype=\"no\" isSpecial=\"no\"><arg name=\"arg\" type=\"item()*\"/></proto></example></p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">This function is <termref def=\"dt-deterministic\">deterministic</termref>, <termref def=\"dt-context-independent\">context-independent</termref>,  and <termref def=\"dt-focus-independent\">focus-independent</termref>. </p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">If the value of <code>$arg</code> is the empty sequence, the function returns\n                <code>true</code>; otherwise, the function returns <code>false</code>. </p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The expression <code>fn:empty((1,2,3)[10])</code> returns <code>true()</code>.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The expression <code>fn:empty(fn:remove((\"hello\", \"world\"), 1))</code> returns <code>false()</code>.</p></div>\n", "summary" : "<p>  Returns true if the argument is the empty sequence.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "arg", "type" : "item()", "occurrence" : "*", "description" : "" } ], "returns" : { "type" : "xs:boolean", "description" : "" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "encode-for-uri", "qname" : "fn:encode-for-uri", "signature" : "($uri-part as xs:string?) as xs:string external", "description" : " <div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">Encodes reserved characters in a string that is intended to be used in the path\n             segment of a URI.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\"><example role=\"signature\"><proto name=\"encode-for-uri\" return-type=\"xs:string\" isOp=\"no\" prefix=\"fn\" returnEmptyOk=\"no\" returnSeq=\"no\" returnVaries=\"no\" isSchema=\"no\" isDatatype=\"no\" isSpecial=\"no\"><arg name=\"uri-part\" type=\"xs:string?\"/></proto></example></p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">This function is <termref def=\"dt-deterministic\">deterministic</termref>, <termref def=\"dt-context-independent\">context-independent</termref>,  and <termref def=\"dt-focus-independent\">focus-independent</termref>. </p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">If <code>$uri-part</code> is the empty sequence, the function returns the zero-length\n             string.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">This function applies the URI escaping rules defined in section 2 of <bibref ref=\"rfc3986\"/> to the <code>xs:string</code> supplied as <code>$uri-part</code>. The\n             effect of the function is to escape reserved characters. Each such character in the\n             string is replaced with its percent-encoded form as described in <bibref ref=\"rfc3986\"/>.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">Since <bibref ref=\"rfc3986\"/> recommends that, for consistency, URI producers and\n             normalizers should use uppercase hexadecimal digits for all percent-encodings, this\n             function must always generate hexadecimal values using the upper-case letters A-F.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">All characters are escaped except those identified as \"unreserved\" by <bibref ref=\"rfc3986\"/>, that is the upper- and lower-case letters A-Z, the digits 0-9,\n             HYPHEN-MINUS (\"-\"), LOW LINE (\"_\"), FULL STOP \".\", and TILDE \"~\".</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">This function escapes URI delimiters and therefore cannot be used indiscriminately to\n             encode \"invalid\" characters in a path segment.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">This function is invertible but not idempotent. This is because a string containing a\n             percent character will be modified by applying the function: for example\n                <code>100%</code> becomes <code>100%25</code>, while <code>100%25</code> becomes\n                <code>100%2525</code>.</p></div>\n", "summary" : "<p>  Encodes reserved characters in a string that is intended to be used in the path\n             segment of a URI.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "uri-part", "type" : "xs:string", "occurrence" : "?", "description" : "" } ], "returns" : { "type" : "xs:string", "description" : "" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 2, "name" : "ends-with", "qname" : "fn:ends-with", "signature" : "($arg1 as xs:string?, $arg2 as xs:string?) as xs:boolean external", "description" : " <div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">Returns true if the string <code>$arg1</code> contains <code>$arg2</code> as a\n             trailing substring, taking collations into account.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\"><example role=\"signature\"><proto name=\"ends-with\" return-type=\"xs:boolean\" isOp=\"no\" prefix=\"fn\" returnEmptyOk=\"no\" returnSeq=\"no\" returnVaries=\"no\" isSchema=\"no\" isDatatype=\"no\" isSpecial=\"no\"><arg name=\"arg1\" type=\"xs:string?\"/><arg name=\"arg2\" type=\"xs:string?\"/></proto></example><example role=\"signature\"><proto name=\"ends-with\" return-type=\"xs:boolean\" isOp=\"no\" prefix=\"fn\" returnEmptyOk=\"no\" returnSeq=\"no\" returnVaries=\"no\" isSchema=\"no\" isDatatype=\"no\" isSpecial=\"no\"><arg name=\"arg1\" type=\"xs:string?\"/><arg name=\"arg2\" type=\"xs:string?\"/><arg name=\"collation\" type=\"xs:string\"/></proto></example></p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The two-argument form of this function is <termref def=\"dt-deterministic\">deterministic</termref>, <termref def=\"dt-context-dependent\">context-dependent</termref>,  and <termref def=\"dt-focus-independent\">focus-independent</termref>.  It depends on\n \t\tcollations.\n \t</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The three-argument form of this function is <termref def=\"dt-deterministic\">deterministic</termref>, <termref def=\"dt-context-dependent\">context-dependent</termref>,  and <termref def=\"dt-focus-independent\">focus-independent</termref>.  It depends on\n \t\tcollations, and static base uri.\n \t</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">If the value of <code>$arg1</code> or <code>$arg2</code> is the empty sequence, or\n             contains only ignorable collation units, it is interpreted as the zero-length\n             string.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">If the value of <code>$arg2</code> is the zero-length string, then the function returns\n                <code>true</code>. If the value of <code>$arg1</code> is the zero-length string and\n             the value of <code>$arg2</code> is not the zero-length string, then the function returns\n                <code>false</code>.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The collation used by this function is determined according to the rules in <specref ref=\"choosing-a-collation\"/>.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The function returns an <code>xs:boolean</code> indicating whether or not the value of\n                <code>$arg1</code> starts with a sequence of collation units that provides a\n                <term>match</term> to the collation units of <code>$arg2</code> according to the\n             collation that is used.</p><note xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\"><p>\n                <term>Match</term> is defined in <bibref ref=\"Unicode-Collations\"/>. </p></note><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">A <phrase diff=\"add\" at=\"L\">dynamic</phrase> error <rfc2119>may</rfc2119> be raised <errorref class=\"CH\" code=\"0004\"/> if the\n             specified collation does not support collation units.</p></div>\n", "summary" : "<p>  Returns true if the string  $arg1  contains  $arg2  as a\n             trailing substring, taking collations into account.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "arg1", "type" : "xs:string", "occurrence" : "?", "description" : "" }, { "name" : "arg2", "type" : "xs:string", "occurrence" : "?", "description" : "" } ], "returns" : { "type" : "xs:boolean", "description" : "" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 3, "name" : "ends-with", "qname" : "fn:ends-with", "signature" : "($arg1 as xs:string?, $arg2 as xs:string?, $collation as xs:string) as xs:boolean external", "description" : " <div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">Returns true if the string <code>$arg1</code> contains <code>$arg2</code> as a\n             trailing substring, taking collations into account.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\"><example role=\"signature\"><proto name=\"ends-with\" return-type=\"xs:boolean\" isOp=\"no\" prefix=\"fn\" returnEmptyOk=\"no\" returnSeq=\"no\" returnVaries=\"no\" isSchema=\"no\" isDatatype=\"no\" isSpecial=\"no\"><arg name=\"arg1\" type=\"xs:string?\"/><arg name=\"arg2\" type=\"xs:string?\"/></proto></example><example role=\"signature\"><proto name=\"ends-with\" return-type=\"xs:boolean\" isOp=\"no\" prefix=\"fn\" returnEmptyOk=\"no\" returnSeq=\"no\" returnVaries=\"no\" isSchema=\"no\" isDatatype=\"no\" isSpecial=\"no\"><arg name=\"arg1\" type=\"xs:string?\"/><arg name=\"arg2\" type=\"xs:string?\"/><arg name=\"collation\" type=\"xs:string\"/></proto></example></p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The two-argument form of this function is <termref def=\"dt-deterministic\">deterministic</termref>, <termref def=\"dt-context-dependent\">context-dependent</termref>,  and <termref def=\"dt-focus-independent\">focus-independent</termref>.  It depends on\n \t\tcollations.\n \t</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The three-argument form of this function is <termref def=\"dt-deterministic\">deterministic</termref>, <termref def=\"dt-context-dependent\">context-dependent</termref>,  and <termref def=\"dt-focus-independent\">focus-independent</termref>.  It depends on\n \t\tcollations, and static base uri.\n \t</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">If the value of <code>$arg1</code> or <code>$arg2</code> is the empty sequence, or\n             contains only ignorable collation units, it is interpreted as the zero-length\n             string.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">If the value of <code>$arg2</code> is the zero-length string, then the function returns\n                <code>true</code>. If the value of <code>$arg1</code> is the zero-length string and\n             the value of <code>$arg2</code> is not the zero-length string, then the function returns\n                <code>false</code>.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The collation used by this function is determined according to the rules in <specref ref=\"choosing-a-collation\"/>.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The function returns an <code>xs:boolean</code> indicating whether or not the value of\n                <code>$arg1</code> starts with a sequence of collation units that provides a\n                <term>match</term> to the collation units of <code>$arg2</code> according to the\n             collation that is used.</p><note xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\"><p>\n                <term>Match</term> is defined in <bibref ref=\"Unicode-Collations\"/>. </p></note><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">A <phrase diff=\"add\" at=\"L\">dynamic</phrase> error <rfc2119>may</rfc2119> be raised <errorref class=\"CH\" code=\"0004\"/> if the\n             specified collation does not support collation units.</p></div>\n", "summary" : "<p>  Returns true if the string  $arg1  contains  $arg2  as a\n             trailing substring, taking collations into account.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "arg1", "type" : "xs:string", "occurrence" : "?", "description" : "" }, { "name" : "arg2", "type" : "xs:string", "occurrence" : "?", "description" : "" }, { "name" : "collation", "type" : "xs:string", "occurrence" : null, "description" : "" } ], "returns" : { "type" : "xs:boolean", "description" : "" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "environment-variable", "qname" : "fn:environment-variable", "signature" : "($arg as xs:string) as xs:string? external", "description" : " <div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">Returns the value of a system environment variable, if it exists.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\"><example role=\"signature\"><proto name=\"environment-variable\" return-type=\"xs:string?\" isOp=\"no\" prefix=\"fn\" returnEmptyOk=\"no\" returnSeq=\"no\" returnVaries=\"no\" isSchema=\"no\" isDatatype=\"no\" isSpecial=\"no\"><arg name=\"name\" type=\"xs:string\"/></proto></example></p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">This function is <termref def=\"dt-deterministic\">deterministic</termref>, <termref def=\"dt-context-dependent\">context-dependent</termref>,  and <termref def=\"dt-focus-independent\">focus-independent</termref>.  It depends on\n \t\tenvironment variables.\n \t</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The set of available <xtermref spec=\"XP30\" ref=\"dt-environment-variables\">environment variables</xtermref>\n             is a set of (name, value) pairs forming part\n             of the dynamic context, in which the name is unique within the set of pairs. The name\n             and value are arbitrary strings.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">If the <code>$name</code> argument matches the name of one of these pairs, the function\n             returns the corresponding value.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">If there is no environment variable with a matching name, the function returns the empty\n             sequence.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The collation used for matching names is <termref def=\"implementation-defined\"/>, but\n             must be the same as the collation used to ensure that the names of all environment\n             variables are unique.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\" diff=\"add\" at=\"I\">The function is <termref def=\"dt-deterministic\">deterministic</termref>,\n             which means that if it is called several times\n          within the same <termref def=\"dt-execution-scope\">execution scope</termref>, with the same arguments,\n             it must return the same result.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">On many platforms, the term \"environment variable\" has a natural meaning in terms of\n             facilities provided by the operating system. This interpretation of the concept does not\n             exclude other interpretations, such as a mapping to a set of configuration parameters in\n             a database system.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">Environment variable names are usually case sensitive. Names are usually of the form\n                <code>(letter|_) (letter|_|digit)*</code>, but this varies by platform.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">On some platforms, there may sometimes be multiple environment variables with the same name;\n             in this case, it is implementation-dependent as to which is returned; see for example\n                <bibref ref=\"POSIX.1-2008\"/> (Chapter 8, Environment Variables).\n             Implementations <rfc2119>may</rfc2119> use prefixes or other naming conventions\n             to disambiguate the names.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The requirement to ensure that the function is deterministic means in practice that\n          the implementation must make a snapshot of the environment variables at some time\n          during execution, and return values obtained from this snapshot, rather than using\n          live values that are subject to change at any time.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">Operating system environment variables may be associated with a particular process,\n          while queries and stylesheets may execute across multiple processes (or multiple machines).\n          In such circumstances implementations <rfc2119>may</rfc2119> choose to provide access\n          to the environment variables associated with the process in which the query or stylesheet\n          processing was initiated.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">Security advice: Queries from untrusted sources should not be permitted unrestricted\n             access to environment variables. For example, the name of the account under which the\n             query is running may be useful information to a would-be intruder. An implementation may\n             therefore choose to restrict access to the environment, or may provide a facility to\n             make <code>fn:environment-variable</code> always return the empty sequence.</p></div>\n", "summary" : "<p>  Returns the value of a system environment variable, if it exists.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "arg", "type" : "xs:string", "occurrence" : null, "description" : "" } ], "returns" : { "type" : "xs:string?", "description" : "" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "environment-variable", "qname" : "fn:environment-variable", "signature" : "($name as xs:string) as xs:string? external", "description" : " <div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">Returns the value of a system environment variable, if it exists.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\"><example role=\"signature\"><proto name=\"environment-variable\" return-type=\"xs:string?\" isOp=\"no\" prefix=\"fn\" returnEmptyOk=\"no\" returnSeq=\"no\" returnVaries=\"no\" isSchema=\"no\" isDatatype=\"no\" isSpecial=\"no\"><arg name=\"name\" type=\"xs:string\"/></proto></example></p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">This function is <termref def=\"dt-deterministic\">deterministic</termref>, <termref def=\"dt-context-dependent\">context-dependent</termref>,  and <termref def=\"dt-focus-independent\">focus-independent</termref>.  It depends on\n \t\tenvironment variables.\n \t</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The set of available <xtermref spec=\"XP30\" ref=\"dt-environment-variables\">environment variables</xtermref>\n             is a set of (name, value) pairs forming part\n             of the dynamic context, in which the name is unique within the set of pairs. The name\n             and value are arbitrary strings.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">If the <code>$name</code> argument matches the name of one of these pairs, the function\n             returns the corresponding value.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">If there is no environment variable with a matching name, the function returns the empty\n             sequence.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The collation used for matching names is <termref def=\"implementation-defined\"/>, but\n             must be the same as the collation used to ensure that the names of all environment\n             variables are unique.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\" diff=\"add\" at=\"I\">The function is <termref def=\"dt-deterministic\">deterministic</termref>,\n             which means that if it is called several times\n          within the same <termref def=\"dt-execution-scope\">execution scope</termref>, with the same arguments,\n             it must return the same result.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">On many platforms, the term \"environment variable\" has a natural meaning in terms of\n             facilities provided by the operating system. This interpretation of the concept does not\n             exclude other interpretations, such as a mapping to a set of configuration parameters in\n             a database system.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">Environment variable names are usually case sensitive. Names are usually of the form\n                <code>(letter|_) (letter|_|digit)*</code>, but this varies by platform.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">On some platforms, there may sometimes be multiple environment variables with the same name;\n             in this case, it is implementation-dependent as to which is returned; see for example\n                <bibref ref=\"POSIX.1-2008\"/> (Chapter 8, Environment Variables).\n             Implementations <rfc2119>may</rfc2119> use prefixes or other naming conventions\n             to disambiguate the names.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The requirement to ensure that the function is deterministic means in practice that\n          the implementation must make a snapshot of the environment variables at some time\n          during execution, and return values obtained from this snapshot, rather than using\n          live values that are subject to change at any time.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">Operating system environment variables may be associated with a particular process,\n          while queries and stylesheets may execute across multiple processes (or multiple machines).\n          In such circumstances implementations <rfc2119>may</rfc2119> choose to provide access\n          to the environment variables associated with the process in which the query or stylesheet\n          processing was initiated.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">Security advice: Queries from untrusted sources should not be permitted unrestricted\n             access to environment variables. For example, the name of the account under which the\n             query is running may be useful information to a would-be intruder. An implementation may\n             therefore choose to restrict access to the environment, or may provide a facility to\n             make <code>fn:environment-variable</code> always return the empty sequence.</p></div>\n", "summary" : "<p>  Returns the value of a system environment variable, if it exists.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "name", "type" : "xs:string", "occurrence" : null, "description" : "" } ], "returns" : { "type" : "xs:string?", "description" : "" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 0, "name" : "error", "qname" : "fn:error", "signature" : "() as none external", "description" : " <div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">Calling the <code>fn:error</code> function raises an application-defined\n             error.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\"><example role=\"signature\"><proto name=\"error\" return-type=\"none\" isOp=\"no\" prefix=\"fn\" returnEmptyOk=\"no\" returnSeq=\"no\" returnVaries=\"no\" isSchema=\"no\" isDatatype=\"no\" isSpecial=\"no\"/></example><example role=\"signature\"><proto name=\"error\" return-type=\"none\" isOp=\"no\" prefix=\"fn\" returnEmptyOk=\"no\" returnSeq=\"no\" returnVaries=\"no\" isSchema=\"no\" isDatatype=\"no\" isSpecial=\"no\"><arg name=\"code\" type=\"xs:QName\"/></proto></example><example role=\"signature\"><proto name=\"error\" return-type=\"none\" isOp=\"no\" prefix=\"fn\" returnEmptyOk=\"no\" returnSeq=\"no\" returnVaries=\"no\" isSchema=\"no\" isDatatype=\"no\" isSpecial=\"no\"><arg name=\"code\" type=\"xs:QName?\"/><arg name=\"description\" type=\"xs:string\"/></proto></example><example role=\"signature\"><proto name=\"error\" return-type=\"none\" isOp=\"no\" prefix=\"fn\" returnEmptyOk=\"no\" returnSeq=\"no\" returnVaries=\"no\" isSchema=\"no\" isDatatype=\"no\" isSpecial=\"no\"><arg name=\"code\" type=\"xs:QName?\"/><arg name=\"description\" type=\"xs:string\"/><arg name=\"error-object\" type=\"item()*\"/></proto></example></p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">This function is <termref def=\"dt-nondeterministic\">nondeterministic</termref>, <termref def=\"dt-context-independent\">context-independent</termref>,  and <termref def=\"dt-focus-independent\">focus-independent</termref>. </p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">This function never returns a value. Instead it always raises an error. The effect of\n             the error is identical to the effect of dynamic errors raised implicitly, for example\n             when an incorrect argument is supplied to a function.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The parameters to the <code>fn:error</code> function supply information that is\n             associated with the error condition and that is made available to a caller that asks for\n             information about the error. The error may be caught either by the host language (using\n             a try/catch construct in XSLT or XQuery, for example), or by the calling application or\n             external processing environment. The way in which error information is returned to the\n             external processing environment is <termref def=\"implementation-dependent\"/></p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">If <code>fn:error</code> is called with no arguments, then its behavior is the same as\n             the function call: </p><eg xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\" xml:space=\"preserve\"> fn:error(fn:QName('http://www.w3.org/2005/xqt-errors', 'err:FOER0000')) </eg><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">If <code>$code</code> is the empty sequence then the effective value is the\n                <code>xs:QName</code> constructed by:</p><eg xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\" xml:space=\"preserve\"> fn:QName('http://www.w3.org/2005/xqt-errors', 'err:FOER0000')</eg><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">There are three pieces of information that may be associated with an error:</p><ulist xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\"><item><p>The <code>$code</code> is an error code that distinguishes this error from others.\n                   It is an <code>xs:QName</code>; the namespace URI conventionally identifies the\n                   component, subsystem, or authority responsible for defining the meaning of the\n                   error code, while the local part identifies the specific error condition. The\n                   namespace URI <code>http://www.w3.org/2005/xqt-errors</code> is used for errors\n                   defined in this specification; other namespace URIs may be used for errors defined\n                   by the application.</p><p>If the external processing environment expects the error code to be returned as a\n                   URI or a string rather than as an <code>xs:QName</code>, then an error code with\n                   namespace URI <code>NS</code> and local part <code>LP</code> will be returned in\n                   the form <code>NS#LP</code>. The namespace URI part of the error code should\n                   therefore not include a fragment identifier.</p></item><item><p>The <code>$description</code> is a natural-language description of the error\n                   condition.</p></item><item><p>The <code>$error-object</code> is an arbitrary value used to convey additional\n                   information about the error, and may be used in any way the application\n                   chooses.</p></item></ulist><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">This function always raises a <phrase diff=\"add\" at=\"L\">dynamic</phrase> error. By default, it raises <errorref class=\"ER\" code=\"0000\"/></p></div>\n", "summary" : "<p>  Calling the  fn:error  function raises an application-defined\n             error.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [  ], "returns" : { "type" : "none", "description" : "" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "error", "qname" : "fn:error", "signature" : "($code as xs:QName) as none external", "description" : " <div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">Calling the <code>fn:error</code> function raises an application-defined\n             error.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\"><example role=\"signature\"><proto name=\"error\" return-type=\"none\" isOp=\"no\" prefix=\"fn\" returnEmptyOk=\"no\" returnSeq=\"no\" returnVaries=\"no\" isSchema=\"no\" isDatatype=\"no\" isSpecial=\"no\"/></example><example role=\"signature\"><proto name=\"error\" return-type=\"none\" isOp=\"no\" prefix=\"fn\" returnEmptyOk=\"no\" returnSeq=\"no\" returnVaries=\"no\" isSchema=\"no\" isDatatype=\"no\" isSpecial=\"no\"><arg name=\"code\" type=\"xs:QName\"/></proto></example><example role=\"signature\"><proto name=\"error\" return-type=\"none\" isOp=\"no\" prefix=\"fn\" returnEmptyOk=\"no\" returnSeq=\"no\" returnVaries=\"no\" isSchema=\"no\" isDatatype=\"no\" isSpecial=\"no\"><arg name=\"code\" type=\"xs:QName?\"/><arg name=\"description\" type=\"xs:string\"/></proto></example><example role=\"signature\"><proto name=\"error\" return-type=\"none\" isOp=\"no\" prefix=\"fn\" returnEmptyOk=\"no\" returnSeq=\"no\" returnVaries=\"no\" isSchema=\"no\" isDatatype=\"no\" isSpecial=\"no\"><arg name=\"code\" type=\"xs:QName?\"/><arg name=\"description\" type=\"xs:string\"/><arg name=\"error-object\" type=\"item()*\"/></proto></example></p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">This function is <termref def=\"dt-nondeterministic\">nondeterministic</termref>, <termref def=\"dt-context-independent\">context-independent</termref>,  and <termref def=\"dt-focus-independent\">focus-independent</termref>. </p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">This function never returns a value. Instead it always raises an error. The effect of\n             the error is identical to the effect of dynamic errors raised implicitly, for example\n             when an incorrect argument is supplied to a function.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The parameters to the <code>fn:error</code> function supply information that is\n             associated with the error condition and that is made available to a caller that asks for\n             information about the error. The error may be caught either by the host language (using\n             a try/catch construct in XSLT or XQuery, for example), or by the calling application or\n             external processing environment. The way in which error information is returned to the\n             external processing environment is <termref def=\"implementation-dependent\"/></p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">If <code>fn:error</code> is called with no arguments, then its behavior is the same as\n             the function call: </p><eg xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\" xml:space=\"preserve\"> fn:error(fn:QName('http://www.w3.org/2005/xqt-errors', 'err:FOER0000')) </eg><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">If <code>$code</code> is the empty sequence then the effective value is the\n                <code>xs:QName</code> constructed by:</p><eg xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\" xml:space=\"preserve\"> fn:QName('http://www.w3.org/2005/xqt-errors', 'err:FOER0000')</eg><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">There are three pieces of information that may be associated with an error:</p><ulist xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\"><item><p>The <code>$code</code> is an error code that distinguishes this error from others.\n                   It is an <code>xs:QName</code>; the namespace URI conventionally identifies the\n                   component, subsystem, or authority responsible for defining the meaning of the\n                   error code, while the local part identifies the specific error condition. The\n                   namespace URI <code>http://www.w3.org/2005/xqt-errors</code> is used for errors\n                   defined in this specification; other namespace URIs may be used for errors defined\n                   by the application.</p><p>If the external processing environment expects the error code to be returned as a\n                   URI or a string rather than as an <code>xs:QName</code>, then an error code with\n                   namespace URI <code>NS</code> and local part <code>LP</code> will be returned in\n                   the form <code>NS#LP</code>. The namespace URI part of the error code should\n                   therefore not include a fragment identifier.</p></item><item><p>The <code>$description</code> is a natural-language description of the error\n                   condition.</p></item><item><p>The <code>$error-object</code> is an arbitrary value used to convey additional\n                   information about the error, and may be used in any way the application\n                   chooses.</p></item></ulist><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">This function always raises a <phrase diff=\"add\" at=\"L\">dynamic</phrase> error. By default, it raises <errorref class=\"ER\" code=\"0000\"/></p></div>\n", "summary" : "<p>  Calling the  fn:error  function raises an application-defined\n             error.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "code", "type" : "xs:QName", "occurrence" : null, "description" : "" } ], "returns" : { "type" : "none", "description" : "" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 2, "name" : "error", "qname" : "fn:error", "signature" : "($code as xs:QName?, $description as xs:string) as none external", "description" : " <div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">Calling the <code>fn:error</code> function raises an application-defined\n             error.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\"><example role=\"signature\"><proto name=\"error\" return-type=\"none\" isOp=\"no\" prefix=\"fn\" returnEmptyOk=\"no\" returnSeq=\"no\" returnVaries=\"no\" isSchema=\"no\" isDatatype=\"no\" isSpecial=\"no\"/></example><example role=\"signature\"><proto name=\"error\" return-type=\"none\" isOp=\"no\" prefix=\"fn\" returnEmptyOk=\"no\" returnSeq=\"no\" returnVaries=\"no\" isSchema=\"no\" isDatatype=\"no\" isSpecial=\"no\"><arg name=\"code\" type=\"xs:QName\"/></proto></example><example role=\"signature\"><proto name=\"error\" return-type=\"none\" isOp=\"no\" prefix=\"fn\" returnEmptyOk=\"no\" returnSeq=\"no\" returnVaries=\"no\" isSchema=\"no\" isDatatype=\"no\" isSpecial=\"no\"><arg name=\"code\" type=\"xs:QName?\"/><arg name=\"description\" type=\"xs:string\"/></proto></example><example role=\"signature\"><proto name=\"error\" return-type=\"none\" isOp=\"no\" prefix=\"fn\" returnEmptyOk=\"no\" returnSeq=\"no\" returnVaries=\"no\" isSchema=\"no\" isDatatype=\"no\" isSpecial=\"no\"><arg name=\"code\" type=\"xs:QName?\"/><arg name=\"description\" type=\"xs:string\"/><arg name=\"error-object\" type=\"item()*\"/></proto></example></p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">This function is <termref def=\"dt-nondeterministic\">nondeterministic</termref>, <termref def=\"dt-context-independent\">context-independent</termref>,  and <termref def=\"dt-focus-independent\">focus-independent</termref>. </p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">This function never returns a value. Instead it always raises an error. The effect of\n             the error is identical to the effect of dynamic errors raised implicitly, for example\n             when an incorrect argument is supplied to a function.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The parameters to the <code>fn:error</code> function supply information that is\n             associated with the error condition and that is made available to a caller that asks for\n             information about the error. The error may be caught either by the host language (using\n             a try/catch construct in XSLT or XQuery, for example), or by the calling application or\n             external processing environment. The way in which error information is returned to the\n             external processing environment is <termref def=\"implementation-dependent\"/></p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">If <code>fn:error</code> is called with no arguments, then its behavior is the same as\n             the function call: </p><eg xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\" xml:space=\"preserve\"> fn:error(fn:QName('http://www.w3.org/2005/xqt-errors', 'err:FOER0000')) </eg><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">If <code>$code</code> is the empty sequence then the effective value is the\n                <code>xs:QName</code> constructed by:</p><eg xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\" xml:space=\"preserve\"> fn:QName('http://www.w3.org/2005/xqt-errors', 'err:FOER0000')</eg><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">There are three pieces of information that may be associated with an error:</p><ulist xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\"><item><p>The <code>$code</code> is an error code that distinguishes this error from others.\n                   It is an <code>xs:QName</code>; the namespace URI conventionally identifies the\n                   component, subsystem, or authority responsible for defining the meaning of the\n                   error code, while the local part identifies the specific error condition. The\n                   namespace URI <code>http://www.w3.org/2005/xqt-errors</code> is used for errors\n                   defined in this specification; other namespace URIs may be used for errors defined\n                   by the application.</p><p>If the external processing environment expects the error code to be returned as a\n                   URI or a string rather than as an <code>xs:QName</code>, then an error code with\n                   namespace URI <code>NS</code> and local part <code>LP</code> will be returned in\n                   the form <code>NS#LP</code>. The namespace URI part of the error code should\n                   therefore not include a fragment identifier.</p></item><item><p>The <code>$description</code> is a natural-language description of the error\n                   condition.</p></item><item><p>The <code>$error-object</code> is an arbitrary value used to convey additional\n                   information about the error, and may be used in any way the application\n                   chooses.</p></item></ulist><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">This function always raises a <phrase diff=\"add\" at=\"L\">dynamic</phrase> error. By default, it raises <errorref class=\"ER\" code=\"0000\"/></p></div>\n", "summary" : "<p>  Calling the  fn:error  function raises an application-defined\n             error.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "code", "type" : "xs:QName", "occurrence" : "?", "description" : "" }, { "name" : "description", "type" : "xs:string", "occurrence" : null, "description" : "" } ], "returns" : { "type" : "none", "description" : "" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 3, "name" : "error", "qname" : "fn:error", "signature" : "($code as xs:QName?, $description as xs:string, $error-object as item()*) as none external", "description" : " <div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">Calling the <code>fn:error</code> function raises an application-defined\n             error.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\"><example role=\"signature\"><proto name=\"error\" return-type=\"none\" isOp=\"no\" prefix=\"fn\" returnEmptyOk=\"no\" returnSeq=\"no\" returnVaries=\"no\" isSchema=\"no\" isDatatype=\"no\" isSpecial=\"no\"/></example><example role=\"signature\"><proto name=\"error\" return-type=\"none\" isOp=\"no\" prefix=\"fn\" returnEmptyOk=\"no\" returnSeq=\"no\" returnVaries=\"no\" isSchema=\"no\" isDatatype=\"no\" isSpecial=\"no\"><arg name=\"code\" type=\"xs:QName\"/></proto></example><example role=\"signature\"><proto name=\"error\" return-type=\"none\" isOp=\"no\" prefix=\"fn\" returnEmptyOk=\"no\" returnSeq=\"no\" returnVaries=\"no\" isSchema=\"no\" isDatatype=\"no\" isSpecial=\"no\"><arg name=\"code\" type=\"xs:QName?\"/><arg name=\"description\" type=\"xs:string\"/></proto></example><example role=\"signature\"><proto name=\"error\" return-type=\"none\" isOp=\"no\" prefix=\"fn\" returnEmptyOk=\"no\" returnSeq=\"no\" returnVaries=\"no\" isSchema=\"no\" isDatatype=\"no\" isSpecial=\"no\"><arg name=\"code\" type=\"xs:QName?\"/><arg name=\"description\" type=\"xs:string\"/><arg name=\"error-object\" type=\"item()*\"/></proto></example></p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">This function is <termref def=\"dt-nondeterministic\">nondeterministic</termref>, <termref def=\"dt-context-independent\">context-independent</termref>,  and <termref def=\"dt-focus-independent\">focus-independent</termref>. </p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">This function never returns a value. Instead it always raises an error. The effect of\n             the error is identical to the effect of dynamic errors raised implicitly, for example\n             when an incorrect argument is supplied to a function.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The parameters to the <code>fn:error</code> function supply information that is\n             associated with the error condition and that is made available to a caller that asks for\n             information about the error. The error may be caught either by the host language (using\n             a try/catch construct in XSLT or XQuery, for example), or by the calling application or\n             external processing environment. The way in which error information is returned to the\n             external processing environment is <termref def=\"implementation-dependent\"/></p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">If <code>fn:error</code> is called with no arguments, then its behavior is the same as\n             the function call: </p><eg xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\" xml:space=\"preserve\"> fn:error(fn:QName('http://www.w3.org/2005/xqt-errors', 'err:FOER0000')) </eg><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">If <code>$code</code> is the empty sequence then the effective value is the\n                <code>xs:QName</code> constructed by:</p><eg xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\" xml:space=\"preserve\"> fn:QName('http://www.w3.org/2005/xqt-errors', 'err:FOER0000')</eg><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">There are three pieces of information that may be associated with an error:</p><ulist xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\"><item><p>The <code>$code</code> is an error code that distinguishes this error from others.\n                   It is an <code>xs:QName</code>; the namespace URI conventionally identifies the\n                   component, subsystem, or authority responsible for defining the meaning of the\n                   error code, while the local part identifies the specific error condition. The\n                   namespace URI <code>http://www.w3.org/2005/xqt-errors</code> is used for errors\n                   defined in this specification; other namespace URIs may be used for errors defined\n                   by the application.</p><p>If the external processing environment expects the error code to be returned as a\n                   URI or a string rather than as an <code>xs:QName</code>, then an error code with\n                   namespace URI <code>NS</code> and local part <code>LP</code> will be returned in\n                   the form <code>NS#LP</code>. The namespace URI part of the error code should\n                   therefore not include a fragment identifier.</p></item><item><p>The <code>$description</code> is a natural-language description of the error\n                   condition.</p></item><item><p>The <code>$error-object</code> is an arbitrary value used to convey additional\n                   information about the error, and may be used in any way the application\n                   chooses.</p></item></ulist><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">This function always raises a <phrase diff=\"add\" at=\"L\">dynamic</phrase> error. By default, it raises <errorref class=\"ER\" code=\"0000\"/></p></div>\n", "summary" : "<p>  Calling the  fn:error  function raises an application-defined\n             error.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "code", "type" : "xs:QName", "occurrence" : "?", "description" : "" }, { "name" : "description", "type" : "xs:string", "occurrence" : null, "description" : "" }, { "name" : "error-object", "type" : "item()", "occurrence" : "*", "description" : "" } ], "returns" : { "type" : "none", "description" : "" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "escape-html-uri", "qname" : "fn:escape-html-uri", "signature" : "($uri as xs:string?) as xs:string external", "description" : " <div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">Escapes a URI in the same way that HTML user agents handle attribute values\n             expected to contain URIs.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\"><example role=\"signature\"><proto name=\"escape-html-uri\" return-type=\"xs:string\" isOp=\"no\" prefix=\"fn\" returnEmptyOk=\"no\" returnSeq=\"no\" returnVaries=\"no\" isSchema=\"no\" isDatatype=\"no\" isSpecial=\"no\"><arg name=\"uri\" type=\"xs:string?\"/></proto></example></p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">This function is <termref def=\"dt-deterministic\">deterministic</termref>, <termref def=\"dt-context-independent\">context-independent</termref>,  and <termref def=\"dt-focus-independent\">focus-independent</termref>. </p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">If <code>$uri</code> is the empty sequence, the function returns the zero-length\n             string.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">Otherwise, the function escapes all <termref def=\"character\">characters</termref> except\n             printable characters of the US-ASCII coded character set, specifically the <termref def=\"codepoint\">codepoints</termref> between 32 and 126 (decimal) inclusive. Each\n             character in <code>$uri</code> to be escaped is replaced by an escape sequence, which is\n             formed by encoding the character as a sequence of octets in UTF-8, and then representing\n             each of these octets in the form %HH, where HH is the hexadecimal representation of the\n             octet. This function must always generate hexadecimal values using the upper-case\n             letters A-F.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The behavior of this function corresponds to the recommended handling of non-ASCII\n             characters in URI attribute values as described in <bibref ref=\"HTML40\"/> Appendix\n             B.2.1.</p></div>\n", "summary" : "<p>  Escapes a URI in the same way that HTML user agents handle attribute values\n             expected to contain URIs.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "uri", "type" : "xs:string", "occurrence" : "?", "description" : "" } ], "returns" : { "type" : "xs:string", "description" : "" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "exactly-one", "qname" : "fn:exactly-one", "signature" : "($arg as item()*) as item() external", "description" : " <div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">Returns <code>$arg</code> if it contains exactly one item. Otherwise, raises an\n             error. </p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\"><example role=\"signature\"><proto name=\"exactly-one\" return-type=\"item()\" isOp=\"no\" prefix=\"fn\" returnEmptyOk=\"no\" returnSeq=\"no\" returnVaries=\"no\" isSchema=\"no\" isDatatype=\"no\" isSpecial=\"no\"><arg name=\"arg\" type=\"item()*\"/></proto></example></p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">This function is <termref def=\"dt-deterministic\">deterministic</termref>, <termref def=\"dt-context-independent\">context-independent</termref>,  and <termref def=\"dt-focus-independent\">focus-independent</termref>. </p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">Except in error cases, the function returns <code>$arg</code> unchanged.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\" diff=\"del\" at=\"F\">For detailed type semantics, see [Formal Semantics].</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">A <phrase diff=\"add\" at=\"L\">dynamic</phrase> error is raised <errorref class=\"RG\" code=\"0005\"/> if <code>$arg</code> is an empty\n             sequence or a sequence containing more than one item.</p></div>\n", "summary" : "<p>  Returns  $arg  if it contains exactly one item.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "arg", "type" : "item()", "occurrence" : "*", "description" : "" } ], "returns" : { "type" : "item()", "description" : "" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "exists", "qname" : "fn:exists", "signature" : "($arg as item()*) as xs:boolean external", "description" : " <div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">Returns true if the argument is a non-empty sequence.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\"><example role=\"signature\"><proto name=\"exists\" return-type=\"xs:boolean\" isOp=\"no\" prefix=\"fn\" returnEmptyOk=\"no\" returnSeq=\"no\" returnVaries=\"no\" isSchema=\"no\" isDatatype=\"no\" isSpecial=\"no\"><arg name=\"arg\" type=\"item()*\"/></proto></example></p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">This function is <termref def=\"dt-deterministic\">deterministic</termref>, <termref def=\"dt-context-independent\">context-independent</termref>,  and <termref def=\"dt-focus-independent\">focus-independent</termref>. </p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">If the value of <code>$arg</code> is a non-empty sequence, the function returns\n                <code>true</code>; otherwise, the function returns <code>false</code>. </p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The expression <code>fn:exists(fn:remove((\"hello\"), 1))</code> returns <code>false()</code>.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The expression <code>fn:exists(fn:remove((\"hello\", \"world\"), 1))</code> returns <code>true()</code>.</p></div>\n", "summary" : "<p>  Returns true if the argument is a non-empty sequence.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "arg", "type" : "item()", "occurrence" : "*", "description" : "" } ], "returns" : { "type" : "xs:boolean", "description" : "" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 0, "name" : "false", "qname" : "fn:false", "signature" : "() as xs:boolean external", "description" : " <div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">Returns the <code>xs:boolean</code> value <code>false</code>.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\"><example role=\"signature\"><proto name=\"false\" return-type=\"xs:boolean\" isOp=\"no\" prefix=\"fn\" returnEmptyOk=\"no\" returnSeq=\"no\" returnVaries=\"no\" isSchema=\"no\" isDatatype=\"no\" isSpecial=\"no\"/></example></p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">This function is <termref def=\"dt-deterministic\">deterministic</termref>, <termref def=\"dt-context-independent\">context-independent</termref>,  and <termref def=\"dt-focus-independent\">focus-independent</termref>. </p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The result is equivalent to <code>xs:boolean(\"0\")</code>.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The expression <code>fn:false()</code> returns <code>xs:boolean(0)</code>.</p></div>\n", "summary" : "<p>  Returns the  xs:boolean  value  false .</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [  ], "returns" : { "type" : "xs:boolean", "description" : "" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 2, "name" : "filter", "qname" : "fn:filter", "signature" : "($seq as item()*, $f as function (item()) as xs:boolean) as item()* external", "description" : " <div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">Returns those items from the sequence <var>$seq</var> for which the supplied\n             function <var>$f</var> returns true.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\"><example role=\"signature\"><proto name=\"filter\" return-type=\"item()*\" isOp=\"no\" prefix=\"fn\" returnEmptyOk=\"no\" returnSeq=\"no\" returnVaries=\"no\" isSchema=\"no\" isDatatype=\"no\" isSpecial=\"no\"><arg name=\"seq\" type=\"item()*\"/><arg name=\"f\" type=\"function(item()) as xs:boolean\"/></proto></example></p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">This function is <termref def=\"dt-deterministic\">deterministic</termref>, <termref def=\"dt-context-independent\">context-independent</termref>,  and <termref def=\"dt-focus-independent\">focus-independent</termref>. </p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The effect of the function is equivalent to the following implementation in XQuery:</p><eg xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\" xml:space=\"preserve\">\n declare function fn:filter(\n         $seq as item()*,\n         $f as function(item()) as xs:boolean)\n         as item()* {\n   if (fn:empty($seq))\n   then ()\n   else ( fn:head($seq)[$f(.) eq fn:true()],\n          fn:filter(fn:tail($seq), $f)\n        )\n };</eg><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">or its equivalent in XSLT:</p><eg xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\" xml:space=\"preserve\">\n &lt;xsl:function name=\"fn:filter\" as=\"item()*\"&gt;\n   &lt;xsl:param name=\"seq\" as=\"item()*\"/&gt;\n   &lt;xsl:param name=\"f\" as=\"function(item()) as xs:boolean\"/&gt;\n   &lt;xsl:if test=\"fn:exists($seq)\"&gt;\n     &lt;xsl:sequence select=\"fn:head($seq)[$f(.) eq fn:true()], fn:filter(fn:tail($seq), $f)\"/&gt;\n   &lt;/xsl:if&gt;\n &lt;/xsl:function&gt;\n          </eg><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\" diff=\"chg\" at=\"G\">As a consequence of the function signature and the function calling\n             rules, a type error occurs if the supplied function <var>$f</var> returns anything other\n             than a single <code>xs:boolean</code> item; there is no conversion to an effective\n             boolean value.</p></div>\n", "summary" : "<p>  Returns those items from the sequence  $seq  for which the supplied\n             function  $f  returns true.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "seq", "type" : "item()", "occurrence" : "*", "description" : "" }, { "name" : "f", "type" : "function (item()) as xs:boolean", "occurrence" : null, "description" : "" } ], "returns" : { "type" : "item()*", "description" : "" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "floor", "qname" : "fn:floor", "signature" : "($arg as numeric?) as numeric? external", "description" : " <div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">Rounds <code>$arg</code> downwards to a whole number.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\"><example role=\"signature\"><proto name=\"floor\" return-type=\"numeric?\" isOp=\"no\" prefix=\"fn\" returnEmptyOk=\"no\" returnSeq=\"no\" returnVaries=\"no\" isSchema=\"no\" isDatatype=\"no\" isSpecial=\"no\"><arg name=\"arg\" type=\"numeric?\"/></proto></example></p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">This function is <termref def=\"dt-deterministic\">deterministic</termref>, <termref def=\"dt-context-independent\">context-independent</termref>,  and <termref def=\"dt-focus-independent\">focus-independent</termref>. </p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">General rules: see <specref ref=\"numeric-value-functions\"/>.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The function returns the largest (closest to positive infinity) number with no\n             fractional part that is not greater than the value of <code>$arg</code>.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">If the type of <code>$arg</code> is one of the four numeric types <code>xs:float</code>,\n                <code>xs:double</code>, <code>xs:decimal</code> or <code>xs:integer</code> the type\n             of the result is the same as the type of <code>$arg</code>. If the type of\n                <code>$arg</code> is a type derived from one of the numeric types, the result is an\n             instance of the base numeric type.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">For <code>xs:float</code> and <code>xs:double</code> arguments, if the argument is\n             positive zero, then positive zero is returned. If the argument is negative zero, then\n             negative zero is returned.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\" diff=\"del\" at=\"F\">For detailed type semantics, see [Formal Semantics].</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The expression <code>fn:floor(10.5)</code> returns <code>10</code>.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The expression <code>fn:floor(-10.5)</code> returns <code>-11</code>.</p></div>\n", "summary" : "<p>  Rounds  $arg  downwards to a whole number.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "arg", "type" : "numeric", "occurrence" : "?", "description" : "" } ], "returns" : { "type" : "numeric?", "description" : "" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 3, "name" : "fold-left", "qname" : "fn:fold-left", "signature" : "($seq as item()*, $zero as item()*, $f as function (item()*, item()) as item()*) as item()* external", "description" : " <div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">Processes the supplied sequence from left to right, applying the supplied\n             function repeatedly to each item in turn, together with an accumulated result value.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\"><example role=\"signature\"><proto name=\"fold-left\" return-type=\"item()*\" isOp=\"no\" prefix=\"fn\" returnEmptyOk=\"no\" returnSeq=\"no\" returnVaries=\"no\" isSchema=\"no\" isDatatype=\"no\" isSpecial=\"no\"><arg name=\"seq\" type=\"item()*\"/><arg name=\"zero\" type=\"item()*\"/><arg name=\"f\" type=\"function(item()*, item()) as item()*\"/></proto></example></p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">This function is <termref def=\"dt-deterministic\">deterministic</termref>, <termref def=\"dt-context-independent\">context-independent</termref>,  and <termref def=\"dt-focus-independent\">focus-independent</termref>. </p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The effect of the function is equivalent to the following implementation in XQuery:</p><eg xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\" xml:space=\"preserve\">\n declare function fn:fold-left(\n         $seq as item()*\n         $zero as item()*,\n         $f as function(item()*, item()) as item()*)\n         as item()* {\n   if (fn:empty($seq))\n   then $zero\n   else fn:fold-left(fn:tail($seq), $f($zero, fn:head($seq)), $f)\n };</eg><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">or its equivalent in XSLT:</p><eg xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\" xml:space=\"preserve\">\n &lt;xsl:function name=\"fn:fold-left\" as=\"item()*\"&gt;\n   &lt;xsl:param name=\"seq\" as=\"item()*\"/&gt;\n   &lt;xsl:param name=\"zero\" as=\"item()*\"/&gt;\n   &lt;xsl:param name=\"f\" as=\"function(item()*, item()) as item()*\"/&gt;\n   &lt;xsl:choose&gt;\n     &lt;xsl:when test=\"fn:empty($seq)\"&gt;\n       &lt;xsl:sequence select=\"$zero\"/&gt;\n     &lt;/xsl:when&gt;\n     &lt;xsl:otherwise&gt;\n       &lt;xsl:sequence select=\"fn:fold-left(fn:tail($seq), $f($zero, fn:head($seq)), $f)\"/&gt;\n     &lt;/xsl:otherwise&gt;\n   &lt;/xsl:choose&gt;\n &lt;/xsl:function&gt;\n          </eg><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\" diff=\"chg\" at=\"G\">As a consequence of the function signature and the function calling\n             rules, a type error occurs if the supplied function <var>$f</var> cannot be applied to\n             two arguments, where the first argument is either the value of <var>$zero</var> or the\n             result of a previous application of <var>$f</var>, and the second is <var>$seq</var> or\n             any trailing subsequence of <var>$seq</var>.</p></div>\n", "summary" : "<p>  Processes the supplied sequence from left to right, applying the supplied\n             function repeatedly to each item in turn, together with an accumulated result value.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "seq", "type" : "item()", "occurrence" : "*", "description" : "" }, { "name" : "zero", "type" : "item()", "occurrence" : "*", "description" : "" }, { "name" : "f", "type" : "function (item()*, item()) as item()*", "occurrence" : null, "description" : "" } ], "returns" : { "type" : "item()*", "description" : "" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 3, "name" : "fold-right", "qname" : "fn:fold-right", "signature" : "($seq as item()*, $zero as item()*, $f as function (item()*, item()) as item()*) as item()* external", "description" : " <div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">Processes the supplied sequence from right to left, applying the supplied\n             function repeatedly to each item in turn, together with an accumulated result value.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\"><example role=\"signature\"><proto name=\"fold-right\" return-type=\"item()*\" isOp=\"no\" prefix=\"fn\" returnEmptyOk=\"no\" returnSeq=\"no\" returnVaries=\"no\" isSchema=\"no\" isDatatype=\"no\" isSpecial=\"no\"><arg name=\"seq\" type=\"item()*\"/><arg name=\"zero\" type=\"item()*\"/><arg name=\"f\" type=\"function(item()*, item()) as item()*\"/></proto></example></p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">This function is <termref def=\"dt-deterministic\">deterministic</termref>, <termref def=\"dt-context-independent\">context-independent</termref>,  and <termref def=\"dt-focus-independent\">focus-independent</termref>. </p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The effect of the function is equivalent to the following implementation in XQuery:</p><eg xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\" xml:space=\"preserve\">\n declare function fn:fold-right(\n         $seq as item()*,\n         $zero as item()*,\n         $f as function(item(), item()*) as item()*)\n         as item()* {\n   if (fn:empty($seq))\n   then $zero\n   else $f(fn:head($seq), fn:fold-right(fn:tail($seq), $zero, $f))\n };</eg><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">or its equivalent in XSLT:</p><eg xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\" xml:space=\"preserve\">\n &lt;xsl:function name=\"fn:fold-right\" as=\"item()*\"&gt;\n   &lt;xsl:param name=\"seq\" as=\"item()*\"/&gt;\n   &lt;xsl:param name=\"zero\" as=\"item()*\"/&gt;\n   &lt;xsl:param name=\"f\" as=\"function(item(), item()*) as item()*\"/&gt;\n   &lt;xsl:choose&gt;\n     &lt;xsl:when test=\"fn:empty($seq)\"&gt;\n       &lt;xsl:sequence select=\"$zero\"/&gt;\n     &lt;/xsl:when&gt;\n     &lt;xsl:otherwise&gt;\n       &lt;xsl:sequence select=\"$f(fn:head($seq), fn:fold-right(fn:tail($seq), $zero, $f))\"/&gt;\n     &lt;/xsl:otherwise&gt;\n   &lt;/xsl:choose&gt;\n &lt;/xsl:function&gt;\n          </eg><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\" diff=\"chg\" at=\"G\">As a consequence of the function signature and the function calling\n             rules, a type error occurs if the supplied function <var>$f</var> cannot be applied to\n             two arguments, where the first argument is any item in the sequence <var>$seq</var>, and\n             the second is either the value of <var>$zero</var> or the result of a previous\n             application of <var>$f</var>.</p></div>\n", "summary" : "<p>  Processes the supplied sequence from right to left, applying the supplied\n             function repeatedly to each item in turn, together with an accumulated result value.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "seq", "type" : "item()", "occurrence" : "*", "description" : "" }, { "name" : "zero", "type" : "item()", "occurrence" : "*", "description" : "" }, { "name" : "f", "type" : "function (item()*, item()) as item()*", "occurrence" : null, "description" : "" } ], "returns" : { "type" : "item()*", "description" : "" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 3, "name" : "for-each-pair", "qname" : "fn:for-each-pair", "signature" : "($seq1 as item()*, $seq2 as item()*, $f as function (item(), item()) as item()*) as item()* external", "description" : " <div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">Applies the function item <var>$f</var> to successive pairs of items taken one\n             from <var>$seq1</var> and one from <var>$seq2</var>, returning the concatenation of the\n             resulting sequences in order.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\"><example role=\"signature\"><proto name=\"for-each-pair\" return-type=\"item()*\" isOp=\"no\" prefix=\"fn\" returnEmptyOk=\"no\" returnSeq=\"no\" returnVaries=\"no\" isSchema=\"no\" isDatatype=\"no\" isSpecial=\"no\"><arg name=\"seq1\" type=\"item()*\"/><arg name=\"seq2\" type=\"item()*\"/><arg name=\"f\" type=\"function(item(), item()) as item()*\"/></proto></example></p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">This function is <termref def=\"dt-deterministic\">deterministic</termref>, <termref def=\"dt-context-independent\">context-independent</termref>,  and <termref def=\"dt-focus-independent\">focus-independent</termref>. </p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The effect of the function is equivalent to the following implementation in XQuery:</p><eg xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\" xml:space=\"preserve\">\n declare function fn:for-each-pair($seq1, $seq2, $f)\n {\n    if(fn:exists($seq1) and fn:exists($seq2))\n    then (\n      $f(fn:head($seq1), fn:head($seq2)),\n      fn:for-each-pair(fn:tail($seq1), fn:tail($seq2), $f)\n    )\n    else ()\n };</eg><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">or its equivalent in XSLT:</p><eg xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\" xml:space=\"preserve\">\n &lt;xsl:function name=\"fn:for-each-pair\"&gt;\n   &lt;xsl:param name=\"seq1/&gt;\n   &lt;xsl:param name=\"seq2/&gt;\n   &lt;xsl:param name=\"f\"/&gt;\n   &lt;xsl:if test=\"fn:exists($seq1) and fn:exists($seq2)\"&gt;\n     &lt;xsl:sequence select=\"$f(fn:head($seq1), fn:head($seq2))\"/&gt;\n     &lt;xsl:sequence select=\"fn:for-each-pair(fn:tail($seq1), fn:tail($seq2), $f)\"/&gt;\n   &lt;/xsl:if&gt;\n &lt;/xsl:function&gt;\n          </eg><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The expression <code>fn:for-each-pair((\"a\", \"b\", \"c\"), (\"x\", \"y\", \"z\"), concat#2)</code> returns <code>(\"ax\", \"by\", \"cz\")</code>.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The expression <code>fn:for-each-pair(1 to 5, 1 to 5, function($a, $b){10*$a + $b}</code> returns <code>(11, 22, 33, 44, 55)</code>.</p></div>\n", "summary" : "<p>  Applies the function item  $f  to successive pairs of items taken one\n             from  $seq1  and one from  $seq2 , returning the concatenation of the\n             resulting sequences in order.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "seq1", "type" : "item()", "occurrence" : "*", "description" : "" }, { "name" : "seq2", "type" : "item()", "occurrence" : "*", "description" : "" }, { "name" : "f", "type" : "function (item(), item()) as item()*", "occurrence" : null, "description" : "" } ], "returns" : { "type" : "item()*", "description" : "" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 2, "name" : "for-each", "qname" : "fn:for-each", "signature" : "($seq as item()*, $f as function (item()) as item()*) as item()* external", "description" : " <div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">Applies the function item <var>$f</var> to every item from the sequence\n                <var>$seq</var> in turn, returning the concatenation of the resulting sequences in\n             order.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\"><example role=\"signature\"><proto name=\"for-each\" return-type=\"item()*\" isOp=\"no\" prefix=\"fn\" returnEmptyOk=\"no\" returnSeq=\"no\" returnVaries=\"no\" isSchema=\"no\" isDatatype=\"no\" isSpecial=\"no\"><arg name=\"seq\" type=\"item()*\"/><arg name=\"f\" type=\"function(item()) as item()*\"/></proto></example></p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">This function is <termref def=\"dt-deterministic\">deterministic</termref>, <termref def=\"dt-context-independent\">context-independent</termref>,  and <termref def=\"dt-focus-independent\">focus-independent</termref>. </p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The effect of the function is equivalent to the following implementation in XQuery:</p><eg xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\" xml:space=\"preserve\">\n declare function fn:for-each($seq, $f) {\n   if (fn:empty($seq))\n   then ()\n   else ($f(fn:head($seq)), fn:for-each(fn:tail($seq), $f))\n };</eg><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">or its equivalent in XSLT:</p><eg xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\" xml:space=\"preserve\">\n &lt;xsl:function name=\"fn:for-each\"&gt;\n   &lt;xsl:param name=\"seq\"/&gt;\n   &lt;xsl:param name=\"f\"/&gt;\n   &lt;xsl:if test=\"fn:exists($seq)\"&gt;\n     &lt;xsl:sequence select=\"$f(fn:head($seq)), fn:for-each(fn:tail($seq), $f)\"/&gt;\n   &lt;/xsl:if&gt;\n &lt;/xsl:function&gt;\n          </eg><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The function call <code>fn:for-each($SEQ, $F)</code> is equivalent to the expression\n                <code>for $i in $SEQ return $F($i)</code><phrase diff=\"add\" at=\"G\">, assuming that\n                ordering mode is <code>ordered</code>.</phrase></p></div>\n", "summary" : "<p>  Applies the function item  $f  to every item from the sequence\n                 $seq  in turn, returning the concatenation of the resulting sequences in\n             order.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "seq", "type" : "item()", "occurrence" : "*", "description" : "" }, { "name" : "f", "type" : "function (item()) as item()*", "occurrence" : null, "description" : "" } ], "returns" : { "type" : "item()*", "description" : "" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 2, "name" : "format-date", "qname" : "fn:format-date", "signature" : "($value as xs:date?, $picture as xs:string) as xs:string? external", "description" : " <div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">Returns a string containing an <code>xs:date</code> value formatted for display.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\"><example role=\"signature\"><proto name=\"format-date\" return-type=\"xs:string?\" isOp=\"no\" prefix=\"fn\" returnEmptyOk=\"no\" returnSeq=\"no\" returnVaries=\"no\" isSchema=\"no\" isDatatype=\"no\" isSpecial=\"no\"><arg name=\"value\" type=\"xs:date?\"/><arg name=\"picture\" type=\"xs:string\"/></proto></example><example role=\"signature\"><proto name=\"format-date\" return-type=\"xs:string?\" isOp=\"no\" prefix=\"fn\" returnEmptyOk=\"no\" returnSeq=\"no\" returnVaries=\"no\" isSchema=\"no\" isDatatype=\"no\" isSpecial=\"no\"><arg name=\"value\" type=\"xs:date?\"/><arg name=\"picture\" type=\"xs:string\"/><arg name=\"language\" type=\"xs:string?\"/><arg name=\"calendar\" type=\"xs:string?\"/><arg name=\"place\" type=\"xs:string?\"/></proto></example></p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The two-argument form of this function is <termref def=\"dt-deterministic\">deterministic</termref>, <termref def=\"dt-context-dependent\">context-dependent</termref>,  and <termref def=\"dt-focus-independent\">focus-independent</termref>.  It depends on\n \t\tdefault calendar, and default language, and default place, and implicit timezone.\n \t</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The five-argument form of this function is <termref def=\"dt-deterministic\">deterministic</termref>, <termref def=\"dt-context-dependent\">context-dependent</termref>,  and <termref def=\"dt-focus-independent\">focus-independent</termref>.  It depends on\n \t\timplicit timezone, and namespaces.\n \t</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">See <specref ref=\"rules-for-datetime-formatting\"/>.</p></div>\n", "summary" : "<p>  Returns a string containing an  xs:date  value formatted for display.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "value", "type" : "xs:date", "occurrence" : "?", "description" : "" }, { "name" : "picture", "type" : "xs:string", "occurrence" : null, "description" : "" } ], "returns" : { "type" : "xs:string?", "description" : "" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 5, "name" : "format-date", "qname" : "fn:format-date", "signature" : "($value as xs:date?, $picture as xs:string, $language as xs:string?, $calendar as xs:string?, $place as xs:string?) as xs:string? external", "description" : " <div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">Returns a string containing an <code>xs:date</code> value formatted for display.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\"><example role=\"signature\"><proto name=\"format-date\" return-type=\"xs:string?\" isOp=\"no\" prefix=\"fn\" returnEmptyOk=\"no\" returnSeq=\"no\" returnVaries=\"no\" isSchema=\"no\" isDatatype=\"no\" isSpecial=\"no\"><arg name=\"value\" type=\"xs:date?\"/><arg name=\"picture\" type=\"xs:string\"/></proto></example><example role=\"signature\"><proto name=\"format-date\" return-type=\"xs:string?\" isOp=\"no\" prefix=\"fn\" returnEmptyOk=\"no\" returnSeq=\"no\" returnVaries=\"no\" isSchema=\"no\" isDatatype=\"no\" isSpecial=\"no\"><arg name=\"value\" type=\"xs:date?\"/><arg name=\"picture\" type=\"xs:string\"/><arg name=\"language\" type=\"xs:string?\"/><arg name=\"calendar\" type=\"xs:string?\"/><arg name=\"place\" type=\"xs:string?\"/></proto></example></p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The two-argument form of this function is <termref def=\"dt-deterministic\">deterministic</termref>, <termref def=\"dt-context-dependent\">context-dependent</termref>,  and <termref def=\"dt-focus-independent\">focus-independent</termref>.  It depends on\n \t\tdefault calendar, and default language, and default place, and implicit timezone.\n \t</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The five-argument form of this function is <termref def=\"dt-deterministic\">deterministic</termref>, <termref def=\"dt-context-dependent\">context-dependent</termref>,  and <termref def=\"dt-focus-independent\">focus-independent</termref>.  It depends on\n \t\timplicit timezone, and namespaces.\n \t</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">See <specref ref=\"rules-for-datetime-formatting\"/>.</p></div>\n", "summary" : "<p>  Returns a string containing an  xs:date  value formatted for display.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "value", "type" : "xs:date", "occurrence" : "?", "description" : "" }, { "name" : "picture", "type" : "xs:string", "occurrence" : null, "description" : "" }, { "name" : "language", "type" : "xs:string", "occurrence" : "?", "description" : "" }, { "name" : "calendar", "type" : "xs:string", "occurrence" : "?", "description" : "" }, { "name" : "place", "type" : "xs:string", "occurrence" : "?", "description" : "" } ], "returns" : { "type" : "xs:string?", "description" : "" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 2, "name" : "format-dateTime", "qname" : "fn:format-dateTime", "signature" : "($value as xs:dateTime?, $picture as xs:string) as xs:string? external", "description" : " <div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">Returns a string containing an <code>xs:dateTime</code> value formatted for display.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\"><example role=\"signature\"><proto name=\"format-dateTime\" return-type=\"xs:string?\" isOp=\"no\" prefix=\"fn\" returnEmptyOk=\"no\" returnSeq=\"no\" returnVaries=\"no\" isSchema=\"no\" isDatatype=\"no\" isSpecial=\"no\"><arg name=\"value\" type=\"xs:dateTime?\"/><arg name=\"picture\" type=\"xs:string\"/></proto></example><example role=\"signature\"><proto name=\"format-dateTime\" return-type=\"xs:string?\" isOp=\"no\" prefix=\"fn\" returnEmptyOk=\"no\" returnSeq=\"no\" returnVaries=\"no\" isSchema=\"no\" isDatatype=\"no\" isSpecial=\"no\"><arg name=\"value\" type=\"xs:dateTime?\"/><arg name=\"picture\" type=\"xs:string\"/><arg name=\"language\" type=\"xs:string?\"/><arg name=\"calendar\" type=\"xs:string?\"/><arg name=\"place\" type=\"xs:string?\"/></proto></example></p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The two-argument form of this function is <termref def=\"dt-deterministic\">deterministic</termref>, <termref def=\"dt-context-dependent\">context-dependent</termref>,  and <termref def=\"dt-focus-independent\">focus-independent</termref>.  It depends on\n \t\tdefault calendar, and default language, and default place, and implicit timezone.\n \t</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The five-argument form of this function is <termref def=\"dt-deterministic\">deterministic</termref>, <termref def=\"dt-context-dependent\">context-dependent</termref>,  and <termref def=\"dt-focus-independent\">focus-independent</termref>.  It depends on\n \t\timplicit timezone, and namespaces.\n \t</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">See <specref ref=\"rules-for-datetime-formatting\"/>.</p></div>\n", "summary" : "<p>  Returns a string containing an  xs:dateTime  value formatted for display.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "value", "type" : "xs:dateTime", "occurrence" : "?", "description" : "" }, { "name" : "picture", "type" : "xs:string", "occurrence" : null, "description" : "" } ], "returns" : { "type" : "xs:string?", "description" : "" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 5, "name" : "format-dateTime", "qname" : "fn:format-dateTime", "signature" : "($value as xs:dateTime?, $picture as xs:string, $language as xs:string?, $calendar as xs:string?, $place as xs:string?) as xs:string? external", "description" : " <div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">Returns a string containing an <code>xs:dateTime</code> value formatted for display.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\"><example role=\"signature\"><proto name=\"format-dateTime\" return-type=\"xs:string?\" isOp=\"no\" prefix=\"fn\" returnEmptyOk=\"no\" returnSeq=\"no\" returnVaries=\"no\" isSchema=\"no\" isDatatype=\"no\" isSpecial=\"no\"><arg name=\"value\" type=\"xs:dateTime?\"/><arg name=\"picture\" type=\"xs:string\"/></proto></example><example role=\"signature\"><proto name=\"format-dateTime\" return-type=\"xs:string?\" isOp=\"no\" prefix=\"fn\" returnEmptyOk=\"no\" returnSeq=\"no\" returnVaries=\"no\" isSchema=\"no\" isDatatype=\"no\" isSpecial=\"no\"><arg name=\"value\" type=\"xs:dateTime?\"/><arg name=\"picture\" type=\"xs:string\"/><arg name=\"language\" type=\"xs:string?\"/><arg name=\"calendar\" type=\"xs:string?\"/><arg name=\"place\" type=\"xs:string?\"/></proto></example></p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The two-argument form of this function is <termref def=\"dt-deterministic\">deterministic</termref>, <termref def=\"dt-context-dependent\">context-dependent</termref>,  and <termref def=\"dt-focus-independent\">focus-independent</termref>.  It depends on\n \t\tdefault calendar, and default language, and default place, and implicit timezone.\n \t</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The five-argument form of this function is <termref def=\"dt-deterministic\">deterministic</termref>, <termref def=\"dt-context-dependent\">context-dependent</termref>,  and <termref def=\"dt-focus-independent\">focus-independent</termref>.  It depends on\n \t\timplicit timezone, and namespaces.\n \t</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">See <specref ref=\"rules-for-datetime-formatting\"/>.</p></div>\n", "summary" : "<p>  Returns a string containing an  xs:dateTime  value formatted for display.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "value", "type" : "xs:dateTime", "occurrence" : "?", "description" : "" }, { "name" : "picture", "type" : "xs:string", "occurrence" : null, "description" : "" }, { "name" : "language", "type" : "xs:string", "occurrence" : "?", "description" : "" }, { "name" : "calendar", "type" : "xs:string", "occurrence" : "?", "description" : "" }, { "name" : "place", "type" : "xs:string", "occurrence" : "?", "description" : "" } ], "returns" : { "type" : "xs:string?", "description" : "" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 2, "name" : "format-integer", "qname" : "fn:format-integer", "signature" : "($value as xs:integer?, $picture as xs:string) as xs:string external", "description" : " <div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">Formats an integer according to a given picture string, using the conventions\n             of a given natural language if specified.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\"><example role=\"signature\"><proto name=\"format-integer\" return-type=\"xs:string\" isOp=\"no\" prefix=\"fn\" returnEmptyOk=\"no\" returnSeq=\"no\" returnVaries=\"no\" isSchema=\"no\" isDatatype=\"no\" isSpecial=\"no\"><arg name=\"value\" type=\"xs:integer?\"/><arg name=\"picture\" type=\"xs:string\"/></proto></example><example role=\"signature\"><proto name=\"format-integer\" return-type=\"xs:string\" isOp=\"no\" prefix=\"fn\" returnEmptyOk=\"no\" returnSeq=\"no\" returnVaries=\"no\" isSchema=\"no\" isDatatype=\"no\" isSpecial=\"no\"><arg name=\"value\" type=\"xs:integer?\"/><arg name=\"picture\" type=\"xs:string\"/><arg name=\"lang\" type=\"xs:string?\"/></proto></example></p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The two-argument form of this function is <termref def=\"dt-deterministic\">deterministic</termref>, <termref def=\"dt-context-dependent\">context-dependent</termref>,  and <termref def=\"dt-focus-independent\">focus-independent</termref>.  It depends on\n \t\tdefault language.\n \t</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The three-argument form of this function is <termref def=\"dt-deterministic\">deterministic</termref>, <termref def=\"dt-context-independent\">context-independent</termref>,  and <termref def=\"dt-focus-independent\">focus-independent</termref>. </p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">If <code>$value</code> is an empty sequence, the function returns a zero-length\n             string.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">In all other cases, the <code>$picture</code> argument describes the format in which\n                <code>$value</code> is output.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The rules that follow describe how non-negative numbers are output. If the value of\n                <code>$value</code> is negative, the rules below are applied to the absolute value of\n                <code>$value</code>, and a minus sign is prepended to the result.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\" diff=\"chg\" at=\"L\">The value of <code>$picture</code> consists of a primary format token,\n             optionally followed\n             by a format modifier. The primary format token is always present and <rfc2119>must not</rfc2119>\n             be zero-length. If the string contains one or more semicolons then everything that\n             precedes the last semicolon is taken as the primary format token and everything\n             that follows is taken as the format modifier; if the string contains no\n             semicolon then the entire picture is taken as the primary format token, and the\n             format modifier is taken to be absent (which is equivalent to supplying a\n             zero-length string).</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The primary format token is classified as one of the following:</p><ulist xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\"><item><p>A <var>decimal-digit-pattern</var> made up of <var>optional-digit-signs</var>,\n                      <var>mandatory-digit-signs</var>, and <var>grouping-separator-signs</var>.</p><ulist><item><p>The <var>optional-digit-sign</var> is the character \"#\".</p></item><item><p>A <var>mandatory-digit-sign</var> is a <termref def=\"character\">character</termref> in Unicode category Nd. All\n                            <var>mandatory-digit-signs</var> within the format token <rfc2119>must</rfc2119> be from the\n                         same digit family, where a digit family is a sequence of ten consecutive\n                         characters in Unicode category Nd, having digit values 0 through 9. Within\n                         the format token, these digits are interchangeable: a three-digit number may\n                         thus be indicated equivalently by <code>000</code>, <code>001</code>, or\n                            <code>999</code>.</p></item><item><p>a <var>grouping-separator-sign</var> is a non-alphanumeric character, that\n                         is a <termref def=\"character\">character</termref> whose Unicode category is\n                         other than Nd, Nl, No, Lu, Ll, Lt, Lm or Lo.</p></item></ulist><p diff=\"chg\" at=\"L\">If the primary format token contains at least one Unicode digit\n                   then it is taken as a decimal digit pattern, and in this case it <rfc2119>must</rfc2119>\n                   match the regular expression <code>^((\\p{Nd}|#|[^\\p{N}\\p{L}])+?)$</code>. If it contains\n                   a digit but does not match this pattern, a <phrase diff=\"add\" at=\"L\">dynamic</phrase> error\n                   is raised <errorref class=\"DF\" code=\"1310\"/>.</p><note><p diff=\"add\" at=\"L\">If a semicolon is to be used as a grouping separator, then the primary\n                format token as a whole must be followed by another semicolon, to ensure that\n                the grouping separator is not mistaken as a separator between the primary format\n                token and the format modifier.</p></note><p>There <rfc2119>must</rfc2119> be at least one <var>mandatory-digit-sign</var>. There may be zero or\n                   more <var>optional-digit-signs</var>, and (if present) these <rfc2119>must</rfc2119> precede all\n                      <var>mandatory-digit-signs</var>. There may be zero or more\n                      <var>grouping-separator-signs</var>. A <var>grouping-separator-sign</var> <rfc2119>must\n                   not</rfc2119> appear at the start or end of the <var>decimal-digit-pattern</var>, nor\n                   adjacent to another <var>grouping-separator-sign</var>.</p><p>The corresponding output format is a decimal number, using this digit family, with\n                   at least as many digits as there are <var>mandatory-digit-signs</var> in the\n                   format token. Thus, a format token <code>1</code> generates the sequence <code>0 1\n                      2 ... 10 11 12 ...</code>, and a format token <code>01</code> (or equivalently,\n                      <code>00</code> or <code>99</code>) generates the sequence <code>00 01 02 ...\n                      09 10 11 12 ... 99 100 101</code>. A format token of <code>&amp;#x661;</code>\n                   (Arabic-Indic digit one) generates the sequence <code>١</code> then <code>٢</code>\n                   then <code>٣</code> ...</p><p>The <var>grouping-separator-signs</var> are handled as follows. The position of\n                   grouping separators within the format token, counting backwards from the last\n                   digit, indicates the position of grouping separators to appear within the\n                   formatted number, and the character used as the <var>grouping-separator-sign</var>\n                   within the format token indicates the character to be used as the corresponding\n                   grouping separator in the formatted number. If <var>grouping-separator-signs</var>\n                   appear at regular intervals within the format token, that is if the same grouping\n                   separator appears at positions forming a sequence <var>N</var>, 2<var>N</var>,\n                      3<var>N</var>, ... for some integer value <var>N</var> (including the case\n                   where there is only one number in the list), then the sequence is extrapolated to\n                   the left, so grouping separators will be used in the formatted number at every\n                   multiple of <var>N</var>. For example, if the format token is <code>0'000</code>\n                   then the number one million will be formatted as <code>1'000'000</code>, while the\n                   number fifteen will be formatted as <code>0'015</code>.</p><p>The only purpose of <var>optional-digit-signs</var> is to mark the position of\n                      <var>grouping-separator-signs</var>. For example, if the format token is\n                      <code>#'##0</code> then the number one million will be formatted as\n                      <code>1'000'000</code>, while the number fifteen will be formatted as\n                      <code>15</code>. A grouping separator is included in the formatted number only\n                   if there is a digit to its left, which will only be the case if either (a) the\n                   number is large enough to require that digit, or (b) the number of\n                      <var>mandatory-digit-signs</var> in the format token requires insignificant\n                   leading zeros to be present.</p><note><p>Numbers will never be truncated. Given the <var>decimal-digit-pattern</var>\n                      <code>01</code>, the number three hundred will be output as <code>300</code>,\n                      despite the absence of any <var>optional-digit-sign</var>.</p></note></item><item><p>The format token <code>A</code>, which generates the sequence <code>A B C ... Z AA AB\n                      AC...</code>.</p></item><item><p>The format token <code>a</code>, which generates the sequence <code>a b c ... z aa ab\n                      ac...</code>.</p></item><item><p>The format token <code>i</code>, which generates the sequence <code>i ii iii iv v vi vii\n                      viii ix x ...</code>.</p></item><item><p>The format token <code>I</code>, which generates the sequence <code>I II III IV V VI VII\n                      VIII IX X ...</code>.</p></item><item><p>The format token <code>w</code>, which generates numbers written as lower-case words, for\n                   example in English, <code>one two three four ...</code>\n                </p></item><item><p>The format token <code>W</code>, which generates numbers written as upper-case words, for\n                   example in English, <code>ONE TWO THREE FOUR ...</code>\n                </p></item><item><p>The format token <code>Ww</code>, which generates numbers written as title-case words, for\n                   example in English, <code>One Two Three Four ...</code>\n                </p></item><item><p>Any other format token, which indicates a numbering sequence in which that token\n                   represents the number 1 (one) (but see the note below).\n                   <!-- Where possible (given the constraint that format tokens\n \t\t\t\t\t\tmust be alphanumeric, and that they must be distinct) the format token\n \t\t\t\t\t\tused to represent a numbering sequence should be the same as the representation\n \t\t\t\t\t\tof the number 1 (one) in that sequence.-->\n                   It is <termref def=\"implementation-defined\">implementation-defined</termref> which\n                   numbering sequences, additional to those listed above, are supported. If an\n                   implementation does not support a numbering sequence represented by the given\n                   token, it <rfc2119>must</rfc2119> use a format token of <code>1</code>.</p><note><p>In some traditional numbering sequences additional signs are added to denote\n                      that the letters should be interpreted as numbers; these are not included in\n                      the format token. An example (see also the example below) is classical Greek\n                      where a <emph>dexia keraia</emph> (x0374, ʹ) and sometimes an <emph>aristeri keraia</emph>\n                      (x0375, ͵) is added.</p></note></item></ulist><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">For all format tokens other than the first kind above (one that consists of decimal\n             digits), there <rfc2119>may</rfc2119> be <termref def=\"implementation-defined\">implementation-defined</termref> lower and upper bounds on the range of numbers that\n             can be formatted using this format token; indeed, for some numbering sequences there may\n             be intrinsic limits. For example, the format token <code>&amp;#x2460;</code> (circled\n             digit one, ①) has a range <phrase diff=\"chg\" at=\"M\">imposed by the Unicode character repertoire — 1 to 20 in\n             Unicode versions prior to 4.0, increased in subsequent versions</phrase>. For\n             the numbering sequences described above any upper bound imposed by the implementation\n                <rfc2119>must not</rfc2119> be less than 1000 (one thousand) and any lower bound must\n             not be greater than 1. Numbers that fall outside this range <rfc2119>must</rfc2119> be\n             formatted using the format token <code>1</code>.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The above expansions of numbering sequences for format tokens such as <code>a</code> and\n                <code>i</code> are indicative but not prescriptive. There are various conventions in\n             use for how alphabetic sequences continue when the alphabet is exhausted, and differing\n             conventions for how roman numerals are written (for example, <code>IV</code> versus\n                <code>IIII</code> as the representation of the number 4). Sometimes alphabetic\n             sequences are used that omit letters such as <code>i</code> and <code>o</code>. This\n             specification does not prescribe the detail of any sequence other than those sequences\n             consisting entirely of decimal digits.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">Many numbering sequences are language-sensitive. This applies especially to the sequence\n             selected by the tokens <code>w</code>, <code>W</code> and <code>Ww</code>. It also\n             applies to other sequences, for example different languages using the Cyrillic alphabet\n             use different sequences of characters, each starting with the letter #x410 (Cyrillic\n             capital letter A). In such cases, the <code>$lang</code> argument specifies which\n             language's conventions are to be used. <phrase diff=\"chg\" at=\"L\">If the argument\n             is specified, the value <rfc2119>should</rfc2119> be either an empty sequence\n             or a value that would be valid for the <code>xml:lang</code> attribute (see <bibref ref=\"REC-xml\"/>).\n             Note that this permits the identification of sublanguages based on country codes (from ISO 3166-1)\n             as well as identification of dialects and regions within a country.</phrase>.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The set of languages\n             for which numbering is supported is <termref def=\"implementation-defined\"/>.\n             If the <code>$lang</code> argument is absent,\n             or is set to an empty sequence, or is invalid, or is not a language supported by the\n             implementation, then the number is formatted using <phrase diff=\"chg\" at=\"K\">the\n             default language from the dynamic context</phrase>.\n             </p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The format modifier <phrase diff=\"chg\" at=\"L\"><rfc2119>must</rfc2119>\n             be a string that matches the regular expression <code diff=\"chg\" at=\"M\">^([co](\\(.+\\))?)?[at]?$</code>. </phrase>\n             That is, if it is present it must consist of one <phrase diff=\"add\" at=\"G\">or more</phrase> of\n             the following, in <phrase diff=\"del\" at=\"K\">any</phrase> order:</p><ulist xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\"><item><p diff=\"chg\" at=\"G\">either <code>c</code> or <code>o</code>, optionally followed by\n                   a sequence of characters enclosed between parentheses, to indicate cardinal or\n                   ordinal numbering respectively, the default being cardinal numbering</p></item><item><p diff=\"chg\" at=\"G\">either <code>a</code> or <code>t</code>, to indicate alphabetic\n                   or traditional numbering respectively, the default being <termref def=\"implementation-defined\">implementation-defined</termref>.</p></item></ulist><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">If the <code>o</code> modifier is present, this indicates a request to output ordinal\n             numbers rather than cardinal numbers. For example, in English, when used with the format\n             token <code>1</code>, this outputs the sequence <code>1st 2nd 3rd 4th ...</code>, and\n             when used with the format token <code>w</code> outputs the sequence <code>first second\n             third fourth ...</code>.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\" diff=\"add\" at=\"L\">The string of characters between the parentheses, if present, is used to\n          select between other possible variations of cardinal or ordinal numbering sequences.\n          The interpretation of this string is <termref def=\"implemementation-defined\">implementation-defined</termref>. No error occurs\n          if the implementation does not define any interpretation for the defined string.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">For example, in some languages, ordinal numbers vary depending on the grammatical context:\n             they may have different genders and may decline with the noun that they qualify.\n             In such cases the string appearing in parentheses after the letter <code>o</code> may be\n             used to indicate the variation of the ordinal number required. The way in which the\n             variation is indicated will depend on the conventions of the language. For inflected\n             languages that vary the ending of the word, the <rfc2119>recommended</rfc2119> approach is to indicate the\n             required ending, preceded by a hyphen: for example in German, appropriate values are\n                <code>o(-e)</code>, <code>o(-er)</code>, <code>o(-es)</code>, <code>o(-en)</code>. </p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">It is <termref def=\"implementation-defined\">implementation-defined</termref> what\n             combinations of values of the format token, the language, and the cardinal/ordinal\n             modifier are supported. If ordinal numbering is not supported for the combination of the\n             format token, the language, and the string appearing in parentheses, the request is\n             ignored and cardinal numbers are generated instead.</p><example xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\"><head>Ordinal Numbering in Italian</head><p>The specification <code>\"1;o(-º)\"</code> with <code>$lang</code> equal to\n                   <code>it</code>, if supported, should produce the sequence:</p><eg xml:space=\"preserve\">1º 2º 3º 4º ...</eg><p>The specification <code>\"Ww;o\"</code> with <code>$lang</code> equal to\n                   <code>it</code>, if supported, should produce the sequence:</p><eg xml:space=\"preserve\">Primo Secondo Terzo Quarto Quinto ...</eg></example><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The <phrase diff=\"chg\" at=\"G\">use of the <code>a</code> or <code>t</code>\n                modifier</phrase> disambiguates between numbering sequences that use letters. In many\n             languages there are two commonly used numbering sequences that use letters. One\n             numbering sequence assigns numeric values to letters in alphabetic sequence, and the\n             other assigns numeric values to each letter in some other manner traditional in that\n             language. In English, these would correspond to the numbering sequences specified by the\n             format tokens <code>a</code> and <code>i</code>. In some languages, the first member of\n             each sequence is the same, and so the format token alone would be ambiguous. <phrase diff=\"chg\" at=\"G\">In the absence of the <code>a</code> or <code>t</code> modifier,\n                the default is <termref def=\"implementation-defined\">implementation-defined</termref></phrase>.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\" diff=\"add\" at=\"L\">A <phrase diff=\"add\" at=\"L\">dynamic</phrase> error is raised <errorref class=\"DF\" code=\"1310\"/> if the format token is invalid,\n             that is, if it violates any mandatory rules (indicated by an emphasized <rfc2119>must</rfc2119>\n             or <rfc2119>required</rfc2119> keyword in the above rules). For example, the error is raised if\n             the primary format token contains a digit but does not match the required regular expression.</p></div>\n", "summary" : "<p>  Formats an integer according to a given picture string, using the conventions\n             of a given natural language if specified.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "value", "type" : "xs:integer", "occurrence" : "?", "description" : "" }, { "name" : "picture", "type" : "xs:string", "occurrence" : null, "description" : "" } ], "returns" : { "type" : "xs:string", "description" : "" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 3, "name" : "format-integer", "qname" : "fn:format-integer", "signature" : "($value as xs:integer?, $picture as xs:string, $language as xs:string) as xs:string external", "description" : " <div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">Formats an integer according to a given picture string, using the conventions\n             of a given natural language if specified.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\"><example role=\"signature\"><proto name=\"format-integer\" return-type=\"xs:string\" isOp=\"no\" prefix=\"fn\" returnEmptyOk=\"no\" returnSeq=\"no\" returnVaries=\"no\" isSchema=\"no\" isDatatype=\"no\" isSpecial=\"no\"><arg name=\"value\" type=\"xs:integer?\"/><arg name=\"picture\" type=\"xs:string\"/></proto></example><example role=\"signature\"><proto name=\"format-integer\" return-type=\"xs:string\" isOp=\"no\" prefix=\"fn\" returnEmptyOk=\"no\" returnSeq=\"no\" returnVaries=\"no\" isSchema=\"no\" isDatatype=\"no\" isSpecial=\"no\"><arg name=\"value\" type=\"xs:integer?\"/><arg name=\"picture\" type=\"xs:string\"/><arg name=\"lang\" type=\"xs:string?\"/></proto></example></p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The two-argument form of this function is <termref def=\"dt-deterministic\">deterministic</termref>, <termref def=\"dt-context-dependent\">context-dependent</termref>,  and <termref def=\"dt-focus-independent\">focus-independent</termref>.  It depends on\n \t\tdefault language.\n \t</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The three-argument form of this function is <termref def=\"dt-deterministic\">deterministic</termref>, <termref def=\"dt-context-independent\">context-independent</termref>,  and <termref def=\"dt-focus-independent\">focus-independent</termref>. </p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">If <code>$value</code> is an empty sequence, the function returns a zero-length\n             string.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">In all other cases, the <code>$picture</code> argument describes the format in which\n                <code>$value</code> is output.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The rules that follow describe how non-negative numbers are output. If the value of\n                <code>$value</code> is negative, the rules below are applied to the absolute value of\n                <code>$value</code>, and a minus sign is prepended to the result.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\" diff=\"chg\" at=\"L\">The value of <code>$picture</code> consists of a primary format token,\n             optionally followed\n             by a format modifier. The primary format token is always present and <rfc2119>must not</rfc2119>\n             be zero-length. If the string contains one or more semicolons then everything that\n             precedes the last semicolon is taken as the primary format token and everything\n             that follows is taken as the format modifier; if the string contains no\n             semicolon then the entire picture is taken as the primary format token, and the\n             format modifier is taken to be absent (which is equivalent to supplying a\n             zero-length string).</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The primary format token is classified as one of the following:</p><ulist xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\"><item><p>A <var>decimal-digit-pattern</var> made up of <var>optional-digit-signs</var>,\n                      <var>mandatory-digit-signs</var>, and <var>grouping-separator-signs</var>.</p><ulist><item><p>The <var>optional-digit-sign</var> is the character \"#\".</p></item><item><p>A <var>mandatory-digit-sign</var> is a <termref def=\"character\">character</termref> in Unicode category Nd. All\n                            <var>mandatory-digit-signs</var> within the format token <rfc2119>must</rfc2119> be from the\n                         same digit family, where a digit family is a sequence of ten consecutive\n                         characters in Unicode category Nd, having digit values 0 through 9. Within\n                         the format token, these digits are interchangeable: a three-digit number may\n                         thus be indicated equivalently by <code>000</code>, <code>001</code>, or\n                            <code>999</code>.</p></item><item><p>a <var>grouping-separator-sign</var> is a non-alphanumeric character, that\n                         is a <termref def=\"character\">character</termref> whose Unicode category is\n                         other than Nd, Nl, No, Lu, Ll, Lt, Lm or Lo.</p></item></ulist><p diff=\"chg\" at=\"L\">If the primary format token contains at least one Unicode digit\n                   then it is taken as a decimal digit pattern, and in this case it <rfc2119>must</rfc2119>\n                   match the regular expression <code>^((\\p{Nd}|#|[^\\p{N}\\p{L}])+?)$</code>. If it contains\n                   a digit but does not match this pattern, a <phrase diff=\"add\" at=\"L\">dynamic</phrase> error\n                   is raised <errorref class=\"DF\" code=\"1310\"/>.</p><note><p diff=\"add\" at=\"L\">If a semicolon is to be used as a grouping separator, then the primary\n                format token as a whole must be followed by another semicolon, to ensure that\n                the grouping separator is not mistaken as a separator between the primary format\n                token and the format modifier.</p></note><p>There <rfc2119>must</rfc2119> be at least one <var>mandatory-digit-sign</var>. There may be zero or\n                   more <var>optional-digit-signs</var>, and (if present) these <rfc2119>must</rfc2119> precede all\n                      <var>mandatory-digit-signs</var>. There may be zero or more\n                      <var>grouping-separator-signs</var>. A <var>grouping-separator-sign</var> <rfc2119>must\n                   not</rfc2119> appear at the start or end of the <var>decimal-digit-pattern</var>, nor\n                   adjacent to another <var>grouping-separator-sign</var>.</p><p>The corresponding output format is a decimal number, using this digit family, with\n                   at least as many digits as there are <var>mandatory-digit-signs</var> in the\n                   format token. Thus, a format token <code>1</code> generates the sequence <code>0 1\n                      2 ... 10 11 12 ...</code>, and a format token <code>01</code> (or equivalently,\n                      <code>00</code> or <code>99</code>) generates the sequence <code>00 01 02 ...\n                      09 10 11 12 ... 99 100 101</code>. A format token of <code>&amp;#x661;</code>\n                   (Arabic-Indic digit one) generates the sequence <code>١</code> then <code>٢</code>\n                   then <code>٣</code> ...</p><p>The <var>grouping-separator-signs</var> are handled as follows. The position of\n                   grouping separators within the format token, counting backwards from the last\n                   digit, indicates the position of grouping separators to appear within the\n                   formatted number, and the character used as the <var>grouping-separator-sign</var>\n                   within the format token indicates the character to be used as the corresponding\n                   grouping separator in the formatted number. If <var>grouping-separator-signs</var>\n                   appear at regular intervals within the format token, that is if the same grouping\n                   separator appears at positions forming a sequence <var>N</var>, 2<var>N</var>,\n                      3<var>N</var>, ... for some integer value <var>N</var> (including the case\n                   where there is only one number in the list), then the sequence is extrapolated to\n                   the left, so grouping separators will be used in the formatted number at every\n                   multiple of <var>N</var>. For example, if the format token is <code>0'000</code>\n                   then the number one million will be formatted as <code>1'000'000</code>, while the\n                   number fifteen will be formatted as <code>0'015</code>.</p><p>The only purpose of <var>optional-digit-signs</var> is to mark the position of\n                      <var>grouping-separator-signs</var>. For example, if the format token is\n                      <code>#'##0</code> then the number one million will be formatted as\n                      <code>1'000'000</code>, while the number fifteen will be formatted as\n                      <code>15</code>. A grouping separator is included in the formatted number only\n                   if there is a digit to its left, which will only be the case if either (a) the\n                   number is large enough to require that digit, or (b) the number of\n                      <var>mandatory-digit-signs</var> in the format token requires insignificant\n                   leading zeros to be present.</p><note><p>Numbers will never be truncated. Given the <var>decimal-digit-pattern</var>\n                      <code>01</code>, the number three hundred will be output as <code>300</code>,\n                      despite the absence of any <var>optional-digit-sign</var>.</p></note></item><item><p>The format token <code>A</code>, which generates the sequence <code>A B C ... Z AA AB\n                      AC...</code>.</p></item><item><p>The format token <code>a</code>, which generates the sequence <code>a b c ... z aa ab\n                      ac...</code>.</p></item><item><p>The format token <code>i</code>, which generates the sequence <code>i ii iii iv v vi vii\n                      viii ix x ...</code>.</p></item><item><p>The format token <code>I</code>, which generates the sequence <code>I II III IV V VI VII\n                      VIII IX X ...</code>.</p></item><item><p>The format token <code>w</code>, which generates numbers written as lower-case words, for\n                   example in English, <code>one two three four ...</code>\n                </p></item><item><p>The format token <code>W</code>, which generates numbers written as upper-case words, for\n                   example in English, <code>ONE TWO THREE FOUR ...</code>\n                </p></item><item><p>The format token <code>Ww</code>, which generates numbers written as title-case words, for\n                   example in English, <code>One Two Three Four ...</code>\n                </p></item><item><p>Any other format token, which indicates a numbering sequence in which that token\n                   represents the number 1 (one) (but see the note below).\n                   <!-- Where possible (given the constraint that format tokens\n \t\t\t\t\t\tmust be alphanumeric, and that they must be distinct) the format token\n \t\t\t\t\t\tused to represent a numbering sequence should be the same as the representation\n \t\t\t\t\t\tof the number 1 (one) in that sequence.-->\n                   It is <termref def=\"implementation-defined\">implementation-defined</termref> which\n                   numbering sequences, additional to those listed above, are supported. If an\n                   implementation does not support a numbering sequence represented by the given\n                   token, it <rfc2119>must</rfc2119> use a format token of <code>1</code>.</p><note><p>In some traditional numbering sequences additional signs are added to denote\n                      that the letters should be interpreted as numbers; these are not included in\n                      the format token. An example (see also the example below) is classical Greek\n                      where a <emph>dexia keraia</emph> (x0374, ʹ) and sometimes an <emph>aristeri keraia</emph>\n                      (x0375, ͵) is added.</p></note></item></ulist><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">For all format tokens other than the first kind above (one that consists of decimal\n             digits), there <rfc2119>may</rfc2119> be <termref def=\"implementation-defined\">implementation-defined</termref> lower and upper bounds on the range of numbers that\n             can be formatted using this format token; indeed, for some numbering sequences there may\n             be intrinsic limits. For example, the format token <code>&amp;#x2460;</code> (circled\n             digit one, ①) has a range <phrase diff=\"chg\" at=\"M\">imposed by the Unicode character repertoire — 1 to 20 in\n             Unicode versions prior to 4.0, increased in subsequent versions</phrase>. For\n             the numbering sequences described above any upper bound imposed by the implementation\n                <rfc2119>must not</rfc2119> be less than 1000 (one thousand) and any lower bound must\n             not be greater than 1. Numbers that fall outside this range <rfc2119>must</rfc2119> be\n             formatted using the format token <code>1</code>.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The above expansions of numbering sequences for format tokens such as <code>a</code> and\n                <code>i</code> are indicative but not prescriptive. There are various conventions in\n             use for how alphabetic sequences continue when the alphabet is exhausted, and differing\n             conventions for how roman numerals are written (for example, <code>IV</code> versus\n                <code>IIII</code> as the representation of the number 4). Sometimes alphabetic\n             sequences are used that omit letters such as <code>i</code> and <code>o</code>. This\n             specification does not prescribe the detail of any sequence other than those sequences\n             consisting entirely of decimal digits.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">Many numbering sequences are language-sensitive. This applies especially to the sequence\n             selected by the tokens <code>w</code>, <code>W</code> and <code>Ww</code>. It also\n             applies to other sequences, for example different languages using the Cyrillic alphabet\n             use different sequences of characters, each starting with the letter #x410 (Cyrillic\n             capital letter A). In such cases, the <code>$lang</code> argument specifies which\n             language's conventions are to be used. <phrase diff=\"chg\" at=\"L\">If the argument\n             is specified, the value <rfc2119>should</rfc2119> be either an empty sequence\n             or a value that would be valid for the <code>xml:lang</code> attribute (see <bibref ref=\"REC-xml\"/>).\n             Note that this permits the identification of sublanguages based on country codes (from ISO 3166-1)\n             as well as identification of dialects and regions within a country.</phrase>.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The set of languages\n             for which numbering is supported is <termref def=\"implementation-defined\"/>.\n             If the <code>$lang</code> argument is absent,\n             or is set to an empty sequence, or is invalid, or is not a language supported by the\n             implementation, then the number is formatted using <phrase diff=\"chg\" at=\"K\">the\n             default language from the dynamic context</phrase>.\n             </p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The format modifier <phrase diff=\"chg\" at=\"L\"><rfc2119>must</rfc2119>\n             be a string that matches the regular expression <code diff=\"chg\" at=\"M\">^([co](\\(.+\\))?)?[at]?$</code>. </phrase>\n             That is, if it is present it must consist of one <phrase diff=\"add\" at=\"G\">or more</phrase> of\n             the following, in <phrase diff=\"del\" at=\"K\">any</phrase> order:</p><ulist xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\"><item><p diff=\"chg\" at=\"G\">either <code>c</code> or <code>o</code>, optionally followed by\n                   a sequence of characters enclosed between parentheses, to indicate cardinal or\n                   ordinal numbering respectively, the default being cardinal numbering</p></item><item><p diff=\"chg\" at=\"G\">either <code>a</code> or <code>t</code>, to indicate alphabetic\n                   or traditional numbering respectively, the default being <termref def=\"implementation-defined\">implementation-defined</termref>.</p></item></ulist><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">If the <code>o</code> modifier is present, this indicates a request to output ordinal\n             numbers rather than cardinal numbers. For example, in English, when used with the format\n             token <code>1</code>, this outputs the sequence <code>1st 2nd 3rd 4th ...</code>, and\n             when used with the format token <code>w</code> outputs the sequence <code>first second\n             third fourth ...</code>.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\" diff=\"add\" at=\"L\">The string of characters between the parentheses, if present, is used to\n          select between other possible variations of cardinal or ordinal numbering sequences.\n          The interpretation of this string is <termref def=\"implemementation-defined\">implementation-defined</termref>. No error occurs\n          if the implementation does not define any interpretation for the defined string.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">For example, in some languages, ordinal numbers vary depending on the grammatical context:\n             they may have different genders and may decline with the noun that they qualify.\n             In such cases the string appearing in parentheses after the letter <code>o</code> may be\n             used to indicate the variation of the ordinal number required. The way in which the\n             variation is indicated will depend on the conventions of the language. For inflected\n             languages that vary the ending of the word, the <rfc2119>recommended</rfc2119> approach is to indicate the\n             required ending, preceded by a hyphen: for example in German, appropriate values are\n                <code>o(-e)</code>, <code>o(-er)</code>, <code>o(-es)</code>, <code>o(-en)</code>. </p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">It is <termref def=\"implementation-defined\">implementation-defined</termref> what\n             combinations of values of the format token, the language, and the cardinal/ordinal\n             modifier are supported. If ordinal numbering is not supported for the combination of the\n             format token, the language, and the string appearing in parentheses, the request is\n             ignored and cardinal numbers are generated instead.</p><example xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\"><head>Ordinal Numbering in Italian</head><p>The specification <code>\"1;o(-º)\"</code> with <code>$lang</code> equal to\n                   <code>it</code>, if supported, should produce the sequence:</p><eg xml:space=\"preserve\">1º 2º 3º 4º ...</eg><p>The specification <code>\"Ww;o\"</code> with <code>$lang</code> equal to\n                   <code>it</code>, if supported, should produce the sequence:</p><eg xml:space=\"preserve\">Primo Secondo Terzo Quarto Quinto ...</eg></example><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The <phrase diff=\"chg\" at=\"G\">use of the <code>a</code> or <code>t</code>\n                modifier</phrase> disambiguates between numbering sequences that use letters. In many\n             languages there are two commonly used numbering sequences that use letters. One\n             numbering sequence assigns numeric values to letters in alphabetic sequence, and the\n             other assigns numeric values to each letter in some other manner traditional in that\n             language. In English, these would correspond to the numbering sequences specified by the\n             format tokens <code>a</code> and <code>i</code>. In some languages, the first member of\n             each sequence is the same, and so the format token alone would be ambiguous. <phrase diff=\"chg\" at=\"G\">In the absence of the <code>a</code> or <code>t</code> modifier,\n                the default is <termref def=\"implementation-defined\">implementation-defined</termref></phrase>.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\" diff=\"add\" at=\"L\">A <phrase diff=\"add\" at=\"L\">dynamic</phrase> error is raised <errorref class=\"DF\" code=\"1310\"/> if the format token is invalid,\n             that is, if it violates any mandatory rules (indicated by an emphasized <rfc2119>must</rfc2119>\n             or <rfc2119>required</rfc2119> keyword in the above rules). For example, the error is raised if\n             the primary format token contains a digit but does not match the required regular expression.</p></div>\n", "summary" : "<p>  Formats an integer according to a given picture string, using the conventions\n             of a given natural language if specified.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "value", "type" : "xs:integer", "occurrence" : "?", "description" : "" }, { "name" : "picture", "type" : "xs:string", "occurrence" : null, "description" : "" }, { "name" : "language", "type" : "xs:string", "occurrence" : null, "description" : "" } ], "returns" : { "type" : "xs:string", "description" : "" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 2, "name" : "format-number", "qname" : "fn:format-number", "signature" : "($value as numeric?, $picture as xs:string) as xs:string external", "description" : " <div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">Returns a string containing a number formatted according to a given picture\n             string, taking account of decimal formats specified in the static context.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\"><example role=\"signature\"><proto name=\"format-number\" return-type=\"xs:string\" isOp=\"no\" prefix=\"fn\" returnEmptyOk=\"no\" returnSeq=\"no\" returnVaries=\"no\" isSchema=\"no\" isDatatype=\"no\" isSpecial=\"no\"><arg name=\"value\" type=\"numeric?\"/><arg name=\"picture\" type=\"xs:string\"/></proto></example><example role=\"signature\"><proto name=\"format-number\" return-type=\"xs:string\" isOp=\"no\" prefix=\"fn\" returnEmptyOk=\"no\" returnSeq=\"no\" returnVaries=\"no\" isSchema=\"no\" isDatatype=\"no\" isSpecial=\"no\"><arg name=\"value\" type=\"numeric?\"/><arg name=\"picture\" type=\"xs:string\"/><arg name=\"decimal-format-name\" type=\"xs:string?\"/></proto></example></p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The two-argument form of this function is <termref def=\"dt-deterministic\">deterministic</termref>, <termref def=\"dt-context-independent\">context-independent</termref>,  and <termref def=\"dt-focus-independent\">focus-independent</termref>. </p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The three-argument form of this function is <termref def=\"dt-deterministic\">deterministic</termref>, <termref def=\"dt-context-dependent\">context-dependent</termref>,  and <termref def=\"dt-focus-independent\">focus-independent</termref>.  It depends on\n \t\tdecimal formats, and namespaces.\n \t</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\" diff=\"add\" at=\"K\">The effect of the two-argument form of the function is equivalent to calling\n          the three-argument form with an empty sequence as the value of the third argument.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The function formats <code>$value</code> as a string using the <termref def=\"dt-picture-string\">picture string</termref> specified by the\n                <code>$picture</code> argument and the decimal-format named by the\n                <code>$decimal-format-name</code> argument, or the default decimal-format, if there\n             is no <code>$decimal-format-name</code> argument. The syntax of the picture string is\n             described in <specref ref=\"syntax-of-picture-string\"/>.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The <code>$value</code> argument may be of any numeric data type\n             (<code>xs:double</code>, <code>xs:float</code>, <code>xs:decimal</code>, or their\n             subtypes including <code>xs:integer</code>). Note that if an <code>xs:decimal</code> is\n             supplied, it is not automatically promoted to an <code>xs:double</code>, as such\n             promotion can involve a loss of precision.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">If the supplied value of the <code>$value</code> argument is an empty sequence, the\n             function behaves as if the supplied value were the <code>xs:double</code> value\n                <code>NaN</code>.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The value of <code>$decimal-format-name</code>,\n             <phrase diff=\"add\" at=\"K\">if present and non-empty, </phrase>\n             <rfc2119>must</rfc2119> <phrase diff=\"chg\" at=\"L\">be a string which after removal of leading and trailing whitespace is in the form of an\n             an <code>EQName</code> as defined in the XPath 3.0 grammar, that is one of the following</phrase>:</p><ulist xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\"><item><p>A lexical QName, which is expanded using the\n                <phrase diff=\"chg\" at=\"J\">statically known namespaces</phrase>.\n                The default namespace is not used (no prefix means no namespace).</p></item><item><p diff=\"add\" at=\"L\">A <code>URIQualifiedName</code> using the syntax <code>Q{uri}local</code>,\n             where the URI can be zero-length to indicate a name in no namespace.</p></item></ulist><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\" diff=\"add\" at=\"K\">The decimal format that is used is the decimal format\n          in the static context whose name matches <code>$decimal-format-name</code> if supplied,\n          or the default decimal format in the static context otherwise.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The evaluation of the <function>format-number</function> function takes place in two\n             phases, an analysis phase described in <specref ref=\"analysing-picture-string\"/> and a\n             formatting phase described in <specref ref=\"formatting-the-number\"/>.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The analysis phase takes as its inputs the <termref def=\"dt-picture-string\">picture\n                string</termref> and the variables derived from the relevant decimal format in the\n             static context, and produces as its output a number of variables with defined values.\n             The formatting phase takes as its inputs the number to be formatted and the variables\n             produced by the analysis phase, and produces as its output a string containing a\n             formatted representation of the number.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The result of the function is the formatted string representation of the supplied\n             number.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">A <phrase diff=\"add\" at=\"L\">dynamic</phrase> error is raised <errorref class=\"DF\" code=\"1280\"/> if the name specified as the\n                <code>$decimal-format-name</code> argument is\n             <phrase diff=\"chg\" at=\"L\">neither a valid lexical QName nor a valid <code>URIQualifiedName</code></phrase>, or if it\n             uses a prefix <phrase diff=\"add\" at=\"J\">that is not found in the statically known namespaces</phrase>, or if the static\n             context does not contain a declaration of a decimal-format with a matching expanded\n             QName. If the processor is able to detect the error statically (for example, when the\n             argument is supplied as a string literal), then the processor <rfc2119>may</rfc2119>\n             optionally signal this as a static error.</p></div>\n", "summary" : "<p>  Returns a string containing a number formatted according to a given picture\n             string, taking account of decimal formats specified in the static context.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "value", "type" : "numeric", "occurrence" : "?", "description" : "" }, { "name" : "picture", "type" : "xs:string", "occurrence" : null, "description" : "" } ], "returns" : { "type" : "xs:string", "description" : "" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 3, "name" : "format-number", "qname" : "fn:format-number", "signature" : "($value as numeric?, $picture as xs:string, $decimal-format-name as xs:string) as xs:string external", "description" : " <div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">Returns a string containing a number formatted according to a given picture\n             string, taking account of decimal formats specified in the static context.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\"><example role=\"signature\"><proto name=\"format-number\" return-type=\"xs:string\" isOp=\"no\" prefix=\"fn\" returnEmptyOk=\"no\" returnSeq=\"no\" returnVaries=\"no\" isSchema=\"no\" isDatatype=\"no\" isSpecial=\"no\"><arg name=\"value\" type=\"numeric?\"/><arg name=\"picture\" type=\"xs:string\"/></proto></example><example role=\"signature\"><proto name=\"format-number\" return-type=\"xs:string\" isOp=\"no\" prefix=\"fn\" returnEmptyOk=\"no\" returnSeq=\"no\" returnVaries=\"no\" isSchema=\"no\" isDatatype=\"no\" isSpecial=\"no\"><arg name=\"value\" type=\"numeric?\"/><arg name=\"picture\" type=\"xs:string\"/><arg name=\"decimal-format-name\" type=\"xs:string?\"/></proto></example></p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The two-argument form of this function is <termref def=\"dt-deterministic\">deterministic</termref>, <termref def=\"dt-context-independent\">context-independent</termref>,  and <termref def=\"dt-focus-independent\">focus-independent</termref>. </p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The three-argument form of this function is <termref def=\"dt-deterministic\">deterministic</termref>, <termref def=\"dt-context-dependent\">context-dependent</termref>,  and <termref def=\"dt-focus-independent\">focus-independent</termref>.  It depends on\n \t\tdecimal formats, and namespaces.\n \t</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\" diff=\"add\" at=\"K\">The effect of the two-argument form of the function is equivalent to calling\n          the three-argument form with an empty sequence as the value of the third argument.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The function formats <code>$value</code> as a string using the <termref def=\"dt-picture-string\">picture string</termref> specified by the\n                <code>$picture</code> argument and the decimal-format named by the\n                <code>$decimal-format-name</code> argument, or the default decimal-format, if there\n             is no <code>$decimal-format-name</code> argument. The syntax of the picture string is\n             described in <specref ref=\"syntax-of-picture-string\"/>.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The <code>$value</code> argument may be of any numeric data type\n             (<code>xs:double</code>, <code>xs:float</code>, <code>xs:decimal</code>, or their\n             subtypes including <code>xs:integer</code>). Note that if an <code>xs:decimal</code> is\n             supplied, it is not automatically promoted to an <code>xs:double</code>, as such\n             promotion can involve a loss of precision.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">If the supplied value of the <code>$value</code> argument is an empty sequence, the\n             function behaves as if the supplied value were the <code>xs:double</code> value\n                <code>NaN</code>.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The value of <code>$decimal-format-name</code>,\n             <phrase diff=\"add\" at=\"K\">if present and non-empty, </phrase>\n             <rfc2119>must</rfc2119> <phrase diff=\"chg\" at=\"L\">be a string which after removal of leading and trailing whitespace is in the form of an\n             an <code>EQName</code> as defined in the XPath 3.0 grammar, that is one of the following</phrase>:</p><ulist xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\"><item><p>A lexical QName, which is expanded using the\n                <phrase diff=\"chg\" at=\"J\">statically known namespaces</phrase>.\n                The default namespace is not used (no prefix means no namespace).</p></item><item><p diff=\"add\" at=\"L\">A <code>URIQualifiedName</code> using the syntax <code>Q{uri}local</code>,\n             where the URI can be zero-length to indicate a name in no namespace.</p></item></ulist><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\" diff=\"add\" at=\"K\">The decimal format that is used is the decimal format\n          in the static context whose name matches <code>$decimal-format-name</code> if supplied,\n          or the default decimal format in the static context otherwise.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The evaluation of the <function>format-number</function> function takes place in two\n             phases, an analysis phase described in <specref ref=\"analysing-picture-string\"/> and a\n             formatting phase described in <specref ref=\"formatting-the-number\"/>.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The analysis phase takes as its inputs the <termref def=\"dt-picture-string\">picture\n                string</termref> and the variables derived from the relevant decimal format in the\n             static context, and produces as its output a number of variables with defined values.\n             The formatting phase takes as its inputs the number to be formatted and the variables\n             produced by the analysis phase, and produces as its output a string containing a\n             formatted representation of the number.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The result of the function is the formatted string representation of the supplied\n             number.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">A <phrase diff=\"add\" at=\"L\">dynamic</phrase> error is raised <errorref class=\"DF\" code=\"1280\"/> if the name specified as the\n                <code>$decimal-format-name</code> argument is\n             <phrase diff=\"chg\" at=\"L\">neither a valid lexical QName nor a valid <code>URIQualifiedName</code></phrase>, or if it\n             uses a prefix <phrase diff=\"add\" at=\"J\">that is not found in the statically known namespaces</phrase>, or if the static\n             context does not contain a declaration of a decimal-format with a matching expanded\n             QName. If the processor is able to detect the error statically (for example, when the\n             argument is supplied as a string literal), then the processor <rfc2119>may</rfc2119>\n             optionally signal this as a static error.</p></div>\n", "summary" : "<p>  Returns a string containing a number formatted according to a given picture\n             string, taking account of decimal formats specified in the static context.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "value", "type" : "numeric", "occurrence" : "?", "description" : "" }, { "name" : "picture", "type" : "xs:string", "occurrence" : null, "description" : "" }, { "name" : "decimal-format-name", "type" : "xs:string", "occurrence" : null, "description" : "" } ], "returns" : { "type" : "xs:string", "description" : "" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 2, "name" : "format-time", "qname" : "fn:format-time", "signature" : "($value as xs:time?, $picture as xs:string) as xs:string? external", "description" : " <div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">Returns a string containing an <code>xs:time</code> value formatted for display.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\"><example role=\"signature\"><proto name=\"format-time\" return-type=\"xs:string?\" isOp=\"no\" prefix=\"fn\" returnEmptyOk=\"no\" returnSeq=\"no\" returnVaries=\"no\" isSchema=\"no\" isDatatype=\"no\" isSpecial=\"no\"><arg name=\"value\" type=\"xs:time?\"/><arg name=\"picture\" type=\"xs:string\"/></proto></example><example role=\"signature\"><proto name=\"format-time\" return-type=\"xs:string?\" isOp=\"no\" prefix=\"fn\" returnEmptyOk=\"no\" returnSeq=\"no\" returnVaries=\"no\" isSchema=\"no\" isDatatype=\"no\" isSpecial=\"no\"><arg name=\"value\" type=\"xs:time?\"/><arg name=\"picture\" type=\"xs:string\"/><arg name=\"language\" type=\"xs:string?\"/><arg name=\"calendar\" type=\"xs:string?\"/><arg name=\"place\" type=\"xs:string?\"/></proto></example></p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The two-argument form of this function is <termref def=\"dt-deterministic\">deterministic</termref>, <termref def=\"dt-context-dependent\">context-dependent</termref>,  and <termref def=\"dt-focus-independent\">focus-independent</termref>.  It depends on\n \t\tdefault calendar, and default language, and default place, and implicit timezone.\n \t</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The five-argument form of this function is <termref def=\"dt-deterministic\">deterministic</termref>, <termref def=\"dt-context-dependent\">context-dependent</termref>,  and <termref def=\"dt-focus-independent\">focus-independent</termref>.  It depends on\n \t\timplicit timezone, and namespaces.\n \t</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">See <specref ref=\"rules-for-datetime-formatting\"/>.</p></div>\n", "summary" : "<p>  Returns a string containing an  xs:time  value formatted for display.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "value", "type" : "xs:time", "occurrence" : "?", "description" : "" }, { "name" : "picture", "type" : "xs:string", "occurrence" : null, "description" : "" } ], "returns" : { "type" : "xs:string?", "description" : "" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 5, "name" : "format-time", "qname" : "fn:format-time", "signature" : "($value as xs:time?, $picture as xs:string, $language as xs:string?, $calendar as xs:string?, $place as xs:string?) as xs:string? external", "description" : " <div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">Returns a string containing an <code>xs:time</code> value formatted for display.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\"><example role=\"signature\"><proto name=\"format-time\" return-type=\"xs:string?\" isOp=\"no\" prefix=\"fn\" returnEmptyOk=\"no\" returnSeq=\"no\" returnVaries=\"no\" isSchema=\"no\" isDatatype=\"no\" isSpecial=\"no\"><arg name=\"value\" type=\"xs:time?\"/><arg name=\"picture\" type=\"xs:string\"/></proto></example><example role=\"signature\"><proto name=\"format-time\" return-type=\"xs:string?\" isOp=\"no\" prefix=\"fn\" returnEmptyOk=\"no\" returnSeq=\"no\" returnVaries=\"no\" isSchema=\"no\" isDatatype=\"no\" isSpecial=\"no\"><arg name=\"value\" type=\"xs:time?\"/><arg name=\"picture\" type=\"xs:string\"/><arg name=\"language\" type=\"xs:string?\"/><arg name=\"calendar\" type=\"xs:string?\"/><arg name=\"place\" type=\"xs:string?\"/></proto></example></p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The two-argument form of this function is <termref def=\"dt-deterministic\">deterministic</termref>, <termref def=\"dt-context-dependent\">context-dependent</termref>,  and <termref def=\"dt-focus-independent\">focus-independent</termref>.  It depends on\n \t\tdefault calendar, and default language, and default place, and implicit timezone.\n \t</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The five-argument form of this function is <termref def=\"dt-deterministic\">deterministic</termref>, <termref def=\"dt-context-dependent\">context-dependent</termref>,  and <termref def=\"dt-focus-independent\">focus-independent</termref>.  It depends on\n \t\timplicit timezone, and namespaces.\n \t</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">See <specref ref=\"rules-for-datetime-formatting\"/>.</p></div>\n", "summary" : "<p>  Returns a string containing an  xs:time  value formatted for display.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "value", "type" : "xs:time", "occurrence" : "?", "description" : "" }, { "name" : "picture", "type" : "xs:string", "occurrence" : null, "description" : "" }, { "name" : "language", "type" : "xs:string", "occurrence" : "?", "description" : "" }, { "name" : "calendar", "type" : "xs:string", "occurrence" : "?", "description" : "" }, { "name" : "place", "type" : "xs:string", "occurrence" : "?", "description" : "" } ], "returns" : { "type" : "xs:string?", "description" : "" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "function-arity", "qname" : "fn:function-arity", "signature" : "($func as function (*)) as xs:integer external", "description" : " <div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">Returns the arity of the function identified by a function item.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\"><example role=\"signature\"><proto name=\"function-arity\" return-type=\"xs:integer\" isOp=\"no\" prefix=\"fn\" returnEmptyOk=\"no\" returnSeq=\"no\" returnVaries=\"no\" isSchema=\"no\" isDatatype=\"no\" isSpecial=\"no\"><arg name=\"func\" type=\"function(*)\"/></proto></example></p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">This function is <termref def=\"dt-deterministic\">deterministic</termref>, <termref def=\"dt-context-independent\">context-independent</termref>,  and <termref def=\"dt-focus-independent\">focus-independent</termref>. </p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The <code>fn:function-arity</code> function returns the arity (number of arguments) of\n             the function identified by <code>$func</code>.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The expression <code>fn:function-arity(fn:substring#2)</code> returns <code>2</code>.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The expression <code>fn:function-arity(function($node){name($node)})</code> returns <code>1</code>.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The expression <code>let $initial := fn:substring(?, 1, 1) return fn:function-arity($initial)</code> returns <code>1</code>.</p></div>\n", "summary" : "<p>  Returns the arity of the function identified by a function item.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "func", "type" : "function (*)", "occurrence" : null, "description" : "" } ], "returns" : { "type" : "xs:integer", "description" : "" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 0, "name" : "generate-id", "qname" : "fn:generate-id", "signature" : "() as xs:string external", "description" : " <div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">This function returns a string that uniquely identifies a given node. </p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\"><example role=\"signature\"><proto name=\"generate-id\" return-type=\"xs:string\" isOp=\"no\" prefix=\"fn\" returnEmptyOk=\"no\" returnSeq=\"no\" returnVaries=\"no\" isSchema=\"no\" isDatatype=\"no\" isSpecial=\"no\"/></example><example role=\"signature\"><proto name=\"generate-id\" return-type=\"xs:string\" isOp=\"no\" prefix=\"fn\" returnEmptyOk=\"no\" returnSeq=\"no\" returnVaries=\"no\" isSchema=\"no\" isDatatype=\"no\" isSpecial=\"no\"><arg name=\"arg\" type=\"node()?\"/></proto></example></p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The zero-argument form of this function is <termref def=\"dt-deterministic\">deterministic</termref>, <termref def=\"dt-context-dependent\">context-dependent</termref>,  and <termref def=\"dt-focus-dependent\">focus-dependent</termref>. </p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The one-argument form of this function is <termref def=\"dt-deterministic\">deterministic</termref>, <termref def=\"dt-context-independent\">context-independent</termref>,  and <termref def=\"dt-focus-independent\">focus-independent</termref>. </p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">If the argument is omitted, it defaults to the context item (<code>.</code>). The\n             behavior of the function if the argument is omitted is exactly the same as if the\n             context item had been passed as the argument.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">If the argument is the empty sequence, the result is the zero-length string.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">In other cases, the function returns a string that uniquely identifies a given node.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The returned identifier <rfc2119>must</rfc2119> consist of ASCII alphanumeric characters\n             and <rfc2119>must</rfc2119> start with an alphabetic character. Thus, the string is\n             syntactically an XML name.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">An implementation is free to generate an identifier in any convenient way provided that\n             it always generates the same identifier for the same node and that different identifiers\n             are always generated from different nodes. An implementation is under no obligation to\n             generate the same identifiers each time a document is transformed or queried.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The following errors may be raised when <code>$arg</code> is omitted:</p><ul xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\"><li><p>If the context\n                item is <xtermref ref=\"dt-absent\" spec=\"DM30\">absent</xtermref>, <phrase diff=\"add\" at=\"M\">dynamic error</phrase>\n                <xerrorref spec=\"XP\" class=\"DY\" code=\"0002\" type=\"dynamic\"/></p></li><li><p>If the context item is not a\n                node, <phrase diff=\"add\" at=\"M\">type error</phrase> <xerrorref spec=\"XP\" class=\"TY\" code=\"0004\" type=\"type\"/>.</p></li></ul></div>\n", "summary" : "<p>  This function returns a string that uniquely identifies a given node.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [  ], "returns" : { "type" : "xs:string", "description" : "" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "generate-id", "qname" : "fn:generate-id", "signature" : "($arg as node()?) as xs:string external", "description" : " <div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">This function returns a string that uniquely identifies a given node. </p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\"><example role=\"signature\"><proto name=\"generate-id\" return-type=\"xs:string\" isOp=\"no\" prefix=\"fn\" returnEmptyOk=\"no\" returnSeq=\"no\" returnVaries=\"no\" isSchema=\"no\" isDatatype=\"no\" isSpecial=\"no\"/></example><example role=\"signature\"><proto name=\"generate-id\" return-type=\"xs:string\" isOp=\"no\" prefix=\"fn\" returnEmptyOk=\"no\" returnSeq=\"no\" returnVaries=\"no\" isSchema=\"no\" isDatatype=\"no\" isSpecial=\"no\"><arg name=\"arg\" type=\"node()?\"/></proto></example></p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The zero-argument form of this function is <termref def=\"dt-deterministic\">deterministic</termref>, <termref def=\"dt-context-dependent\">context-dependent</termref>,  and <termref def=\"dt-focus-dependent\">focus-dependent</termref>. </p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The one-argument form of this function is <termref def=\"dt-deterministic\">deterministic</termref>, <termref def=\"dt-context-independent\">context-independent</termref>,  and <termref def=\"dt-focus-independent\">focus-independent</termref>. </p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">If the argument is omitted, it defaults to the context item (<code>.</code>). The\n             behavior of the function if the argument is omitted is exactly the same as if the\n             context item had been passed as the argument.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">If the argument is the empty sequence, the result is the zero-length string.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">In other cases, the function returns a string that uniquely identifies a given node.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The returned identifier <rfc2119>must</rfc2119> consist of ASCII alphanumeric characters\n             and <rfc2119>must</rfc2119> start with an alphabetic character. Thus, the string is\n             syntactically an XML name.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">An implementation is free to generate an identifier in any convenient way provided that\n             it always generates the same identifier for the same node and that different identifiers\n             are always generated from different nodes. An implementation is under no obligation to\n             generate the same identifiers each time a document is transformed or queried.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The following errors may be raised when <code>$arg</code> is omitted:</p><ul xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\"><li><p>If the context\n                item is <xtermref ref=\"dt-absent\" spec=\"DM30\">absent</xtermref>, <phrase diff=\"add\" at=\"M\">dynamic error</phrase>\n                <xerrorref spec=\"XP\" class=\"DY\" code=\"0002\" type=\"dynamic\"/></p></li><li><p>If the context item is not a\n                node, <phrase diff=\"add\" at=\"M\">type error</phrase> <xerrorref spec=\"XP\" class=\"TY\" code=\"0004\" type=\"type\"/>.</p></li></ul></div>\n", "summary" : "<p>  This function returns a string that uniquely identifies a given node.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "arg", "type" : "node()", "occurrence" : "?", "description" : "" } ], "returns" : { "type" : "xs:string", "description" : "" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 0, "name" : "has-children", "qname" : "fn:has-children", "signature" : "() as xs:boolean external", "description" : " <div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">Returns true if the supplied node has one or more child nodes (of any kind).</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\"><example role=\"signature\"><proto name=\"has-children\" return-type=\"xs:boolean\" isOp=\"no\" prefix=\"fn\" returnEmptyOk=\"no\" returnSeq=\"no\" returnVaries=\"no\" isSchema=\"no\" isDatatype=\"no\" isSpecial=\"no\"/></example><example role=\"signature\"><proto name=\"has-children\" return-type=\"xs:boolean\" isOp=\"no\" prefix=\"fn\" returnEmptyOk=\"no\" returnSeq=\"no\" returnVaries=\"no\" isSchema=\"no\" isDatatype=\"no\" isSpecial=\"no\"><arg name=\"node\" type=\"node()?\"/></proto></example></p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The zero-argument form of this function is <termref def=\"dt-deterministic\">deterministic</termref>, <termref def=\"dt-context-dependent\">context-dependent</termref>,  and <termref def=\"dt-focus-dependent\">focus-dependent</termref>. </p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The one-argument form of this function is <termref def=\"dt-deterministic\">deterministic</termref>, <termref def=\"dt-context-independent\">context-independent</termref>,  and <termref def=\"dt-focus-independent\">focus-independent</termref>. </p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">If the argument is omitted, it defaults to the context item (<code>.</code>). The\n             behavior of the function if the argument is omitted is exactly the same as if the\n             context item had been passed as the argument.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\"><phrase diff=\"add\" at=\"L\">Provided that the supplied argument <code>$node</code> matches the expected\n          type <code>node()?</code>, </phrase>the result of the function call <code>fn:has-children($node)</code> is defined to be\n             the same as the result of the expression\n             <code>fn:exists($node/child::node())</code>.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The following errors may be raised when <code>$node</code> is omitted:</p><ul xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\"><li><p>If the context\n                item is <xtermref ref=\"dt-absent\" spec=\"DM30\">absent</xtermref>, <phrase diff=\"add\" at=\"M\">dynamic error</phrase>\n                <xerrorref spec=\"XP\" class=\"DY\" code=\"0002\" type=\"dynamic\"/></p></li><li><p>If the context item is not a\n                node, <phrase diff=\"add\" at=\"M\">type error</phrase> <xerrorref spec=\"XP\" class=\"TY\" code=\"0004\" type=\"type\"/>.</p></li></ul></div>\n", "summary" : "<p>  Returns true if the supplied node has one or more child nodes (of any kind).</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [  ], "returns" : { "type" : "xs:boolean", "description" : "" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "has-children", "qname" : "fn:has-children", "signature" : "($node as node()?) as xs:boolean external", "description" : " <div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">Returns true if the supplied node has one or more child nodes (of any kind).</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\"><example role=\"signature\"><proto name=\"has-children\" return-type=\"xs:boolean\" isOp=\"no\" prefix=\"fn\" returnEmptyOk=\"no\" returnSeq=\"no\" returnVaries=\"no\" isSchema=\"no\" isDatatype=\"no\" isSpecial=\"no\"/></example><example role=\"signature\"><proto name=\"has-children\" return-type=\"xs:boolean\" isOp=\"no\" prefix=\"fn\" returnEmptyOk=\"no\" returnSeq=\"no\" returnVaries=\"no\" isSchema=\"no\" isDatatype=\"no\" isSpecial=\"no\"><arg name=\"node\" type=\"node()?\"/></proto></example></p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The zero-argument form of this function is <termref def=\"dt-deterministic\">deterministic</termref>, <termref def=\"dt-context-dependent\">context-dependent</termref>,  and <termref def=\"dt-focus-dependent\">focus-dependent</termref>. </p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The one-argument form of this function is <termref def=\"dt-deterministic\">deterministic</termref>, <termref def=\"dt-context-independent\">context-independent</termref>,  and <termref def=\"dt-focus-independent\">focus-independent</termref>. </p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">If the argument is omitted, it defaults to the context item (<code>.</code>). The\n             behavior of the function if the argument is omitted is exactly the same as if the\n             context item had been passed as the argument.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\"><phrase diff=\"add\" at=\"L\">Provided that the supplied argument <code>$node</code> matches the expected\n          type <code>node()?</code>, </phrase>the result of the function call <code>fn:has-children($node)</code> is defined to be\n             the same as the result of the expression\n             <code>fn:exists($node/child::node())</code>.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The following errors may be raised when <code>$node</code> is omitted:</p><ul xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\"><li><p>If the context\n                item is <xtermref ref=\"dt-absent\" spec=\"DM30\">absent</xtermref>, <phrase diff=\"add\" at=\"M\">dynamic error</phrase>\n                <xerrorref spec=\"XP\" class=\"DY\" code=\"0002\" type=\"dynamic\"/></p></li><li><p>If the context item is not a\n                node, <phrase diff=\"add\" at=\"M\">type error</phrase> <xerrorref spec=\"XP\" class=\"TY\" code=\"0004\" type=\"type\"/>.</p></li></ul></div>\n", "summary" : "<p>  Returns true if the supplied node has one or more child nodes (of any kind).</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "node", "type" : "node()", "occurrence" : "?", "description" : "" } ], "returns" : { "type" : "xs:boolean", "description" : "" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "head", "qname" : "fn:head", "signature" : "($arg as item()*) as item()? external", "description" : " <div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">Returns the first item in a sequence. </p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\"><example role=\"signature\"><proto name=\"head\" return-type=\"item()?\" isOp=\"no\" prefix=\"fn\" returnEmptyOk=\"no\" returnSeq=\"no\" returnVaries=\"no\" isSchema=\"no\" isDatatype=\"no\" isSpecial=\"no\"><arg name=\"arg\" type=\"item()*\"/></proto></example></p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">This function is <termref def=\"dt-deterministic\">deterministic</termref>, <termref def=\"dt-context-independent\">context-independent</termref>,  and <termref def=\"dt-focus-independent\">focus-independent</termref>. </p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The function returns the value of the expression <code>$arg[1]</code></p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">If <code>$arg</code> is the empty sequence, the empty sequence is returned. Otherwise\n             the first item in the sequence is returned.</p></div>\n", "summary" : "<p>  Returns the first item in a sequence.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "arg", "type" : "item()", "occurrence" : "*", "description" : "" } ], "returns" : { "type" : "item()?", "description" : "" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "hours-from-duration", "qname" : "fn:hours-from-duration", "signature" : "($arg as xs:duration?) as xs:integer? external", "description" : " <div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">Returns the number of hours in a duration.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\"><example role=\"signature\"><proto name=\"hours-from-duration\" return-type=\"xs:integer?\" isOp=\"no\" prefix=\"fn\" returnEmptyOk=\"no\" returnSeq=\"no\" returnVaries=\"no\" isSchema=\"no\" isDatatype=\"no\" isSpecial=\"no\"><arg name=\"arg\" type=\"xs:duration?\"/></proto></example></p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">This function is <termref def=\"dt-deterministic\">deterministic</termref>, <termref def=\"dt-context-independent\">context-independent</termref>,  and <termref def=\"dt-focus-independent\">focus-independent</termref>. </p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">If <code>$arg</code> is the empty sequence, the function returns the empty sequence.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">Otherwise, the function returns an <code>xs:integer</code> representing the hours\n             component in the value of <code>$arg</code>. The result is obtained by casting\n                <code>$arg</code> to an <code>xs:dayTimeDuration</code> (see <specref ref=\"casting-to-durations\"/>) and then computing the hours component as described in\n                <specref ref=\"canonical-dayTimeDuration\"/>.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">If <code>$arg</code> is a negative duration then the result will be negative..</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">If <code>$arg</code> is an <code>xs:yearMonthDuration</code> the function returns 0.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The expression <code>fn:hours-from-duration(xs:dayTimeDuration(\"P3DT10H\"))</code> returns <code>10</code>.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The expression <code>fn:hours-from-duration(xs:dayTimeDuration(\"P3DT12H32M12S\"))</code> returns <code>12</code>.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The expression <code>fn:hours-from-duration(xs:dayTimeDuration(\"PT123H\"))</code> returns <code>3</code>.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The expression <code>fn:hours-from-duration(xs:dayTimeDuration(\"-P3DT10H\"))</code> returns <code>-10</code>.</p></div>\n", "summary" : "<p>  Returns the number of hours in a duration.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "arg", "type" : "xs:duration", "occurrence" : "?", "description" : "" } ], "returns" : { "type" : "xs:integer?", "description" : "" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "hours-from-time", "qname" : "fn:hours-from-time", "signature" : "($arg as xs:time?) as xs:integer? external", "description" : " <div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">Returns the hours component of an <code>xs:time</code>.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\"><example role=\"signature\"><proto name=\"hours-from-time\" return-type=\"xs:integer?\" isOp=\"no\" prefix=\"fn\" returnEmptyOk=\"no\" returnSeq=\"no\" returnVaries=\"no\" isSchema=\"no\" isDatatype=\"no\" isSpecial=\"no\"><arg name=\"arg\" type=\"xs:time?\"/></proto></example></p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">This function is <termref def=\"dt-deterministic\">deterministic</termref>, <termref def=\"dt-context-independent\">context-independent</termref>,  and <termref def=\"dt-focus-independent\">focus-independent</termref>. </p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">If <code>$arg</code> is the empty sequence, the function returns the empty sequence.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">Otherwise, the function returns an <code>xs:integer</code> between 0 and 23, both\n             inclusive, representing the value of the hours component in the local value of\n                <code>$arg</code>.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">Assume that the dynamic context provides an implicit timezone value of\n                   <code>-05:00</code>. </p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The expression <code>fn:hours-from-time(xs:time(\"11:23:00\"))</code> returns <code>11</code>.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The expression <code>fn:hours-from-time(xs:time(\"21:23:00\"))</code> returns <code>21</code>.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The expression <code>fn:hours-from-time(xs:time(\"01:23:00+05:00\"))</code> returns <code>1</code>.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The expression <code>fn:hours-from-time(fn:adjust-time-to-timezone(xs:time(\"01:23:00+05:00\"),\n                   xs:dayTimeDuration(\"PT0S\")))</code> returns <code>20</code>.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The expression <code>fn:hours-from-time(xs:time(\"24:00:00\"))</code> returns <code>0</code>.</p></div>\n", "summary" : "<p>  Returns the hours component of an  xs:time .</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "arg", "type" : "xs:time", "occurrence" : "?", "description" : "" } ], "returns" : { "type" : "xs:integer?", "description" : "" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "id", "qname" : "fn:id", "signature" : "($arg as xs:string*) as element(*)* external", "description" : " <div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">Returns the sequence of element nodes that have an <code>ID</code> value\n             matching the value of one or more of the <code>IDREF</code> values supplied in\n                <code>$arg</code>.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\"><example role=\"signature\"><proto name=\"id\" return-type=\"element()*\" isOp=\"no\" prefix=\"fn\" returnEmptyOk=\"no\" returnSeq=\"no\" returnVaries=\"no\" isSchema=\"no\" isDatatype=\"no\" isSpecial=\"no\"><arg name=\"arg\" type=\"xs:string*\"/></proto></example><example role=\"signature\"><proto name=\"id\" return-type=\"element()*\" isOp=\"no\" prefix=\"fn\" returnEmptyOk=\"no\" returnSeq=\"no\" returnVaries=\"no\" isSchema=\"no\" isDatatype=\"no\" isSpecial=\"no\"><arg name=\"arg\" type=\"xs:string*\"/><arg name=\"node\" type=\"node()\"/></proto></example></p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The one-argument form of this function is <termref def=\"dt-deterministic\">deterministic</termref>, <termref def=\"dt-context-dependent\">context-dependent</termref>,  and <termref def=\"dt-focus-dependent\">focus-dependent</termref>. </p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The two-argument form of this function is <termref def=\"dt-deterministic\">deterministic</termref>, <termref def=\"dt-context-independent\">context-independent</termref>,  and <termref def=\"dt-focus-independent\">focus-independent</termref>. </p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The function returns a sequence, in document order with duplicates eliminated,\n             containing every element node <code>E</code> that satisfies all the following\n             conditions:</p><olist xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\"><item><p>\n                   <code>E</code> is in the target document. The target document is the document\n                   containing <code>$node</code>, or the document containing the context item\n                      (<code>.</code>) if the second argument is omitted. The behavior of the\n                   function if <code>$node</code> is omitted is exactly the same as if the context\n                   item had been passed as <code>$node</code>.</p></item><item><p><code>E</code> has an <code>ID</code> value equal to one of the candidate\n                      <code>IDREF</code> values, where:</p><ulist><item><p> An element has an <code>ID</code> value equal to <code>V</code> if either\n                         or both of the following conditions are true:</p><ulist><item><p>The <code>is-id</code> property (See <xspecref spec=\"DM30\" ref=\"dm-is-id\"/>.) of the element node is true, and the typed value\n                               of the element node is equal to <code>V</code> under the rules of the\n                                  <code>eq</code> operator using the Unicode codepoint collation\n                                  (<code>http://www.w3.org/2005/xpath-functions/collation/codepoint</code>).</p></item><item><p>The element has an attribute node whose <code>is-id</code> property\n                               (See <xspecref spec=\"DM30\" ref=\"dm-is-id\"/>.) is true and whose typed\n                               value is equal to <code>V</code> under the rules of the\n                                  <code>eq</code> operator using the Unicode code point collation\n                                  (<code>http://www.w3.org/2005/xpath-functions/collation/codepoint</code>).</p></item></ulist></item><item><p> Each <code>xs:string</code> in <code>$arg</code> is parsed as if it were of\n                         type <code>IDREFS</code>, that is, each <code>xs:string</code> in\n                            <code>$arg</code> is treated as a whitespace-separated sequence of\n                         tokens, each acting as an <code>IDREF</code>. These tokens are then included\n                         in the list of candidate <code>IDREF</code>s. If any of the tokens is not a\n                         lexically valid <code>IDREF</code> (that is, if it is not lexically an\n                            <code>xs:NCName</code>), it is ignored. Formally, the candidate\n                            <code>IDREF</code> values are the strings in the sequence given by the\n                         expression:</p><eg xml:space=\"preserve\">for $s in $arg return\n     fn:tokenize(fn:normalize-space($s), ' ')[. castable as xs:IDREF]</eg></item></ulist></item><item><p>If several elements have the same <code>ID</code> value, then <code>E</code> is\n                   the one that is first in document order.</p></item></olist><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">A <phrase diff=\"add\" at=\"L\">dynamic</phrase> error is raised <errorref class=\"DC\" code=\"0001\" type=\"dynamic\"/> if\n                <code>$node</code>, or the context item if the second argument is absent, is a node\n                in a tree whose root is not a document node.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The following errors may be raised when <code>$node</code> is omitted:</p><ul xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\"><li><p>If the context\n                item is <xtermref ref=\"dt-absent\" spec=\"DM30\">absent</xtermref>, <phrase diff=\"add\" at=\"M\">dynamic error</phrase>\n                <xerrorref spec=\"XP\" class=\"DY\" code=\"0002\" type=\"dynamic\"/></p></li><li><p>If the context item is not a\n                node, <phrase diff=\"add\" at=\"M\">type error</phrase> <xerrorref spec=\"XP\" class=\"TY\" code=\"0004\" type=\"type\"/>.</p></li></ul></div>\n", "summary" : "<p>  Returns the sequence of element nodes that have an  ID  value\n             matching the value of one or more of the  IDREF  values supplied in\n                 $arg .</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "arg", "type" : "xs:string", "occurrence" : "*", "description" : "" } ], "returns" : { "type" : "element(*)*", "description" : "" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 2, "name" : "id", "qname" : "fn:id", "signature" : "($arg as xs:string*, $node as node()) as element(*)* external", "description" : " <div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">Returns the sequence of element nodes that have an <code>ID</code> value\n             matching the value of one or more of the <code>IDREF</code> values supplied in\n                <code>$arg</code>.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\"><example role=\"signature\"><proto name=\"id\" return-type=\"element()*\" isOp=\"no\" prefix=\"fn\" returnEmptyOk=\"no\" returnSeq=\"no\" returnVaries=\"no\" isSchema=\"no\" isDatatype=\"no\" isSpecial=\"no\"><arg name=\"arg\" type=\"xs:string*\"/></proto></example><example role=\"signature\"><proto name=\"id\" return-type=\"element()*\" isOp=\"no\" prefix=\"fn\" returnEmptyOk=\"no\" returnSeq=\"no\" returnVaries=\"no\" isSchema=\"no\" isDatatype=\"no\" isSpecial=\"no\"><arg name=\"arg\" type=\"xs:string*\"/><arg name=\"node\" type=\"node()\"/></proto></example></p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The one-argument form of this function is <termref def=\"dt-deterministic\">deterministic</termref>, <termref def=\"dt-context-dependent\">context-dependent</termref>,  and <termref def=\"dt-focus-dependent\">focus-dependent</termref>. </p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The two-argument form of this function is <termref def=\"dt-deterministic\">deterministic</termref>, <termref def=\"dt-context-independent\">context-independent</termref>,  and <termref def=\"dt-focus-independent\">focus-independent</termref>. </p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The function returns a sequence, in document order with duplicates eliminated,\n             containing every element node <code>E</code> that satisfies all the following\n             conditions:</p><olist xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\"><item><p>\n                   <code>E</code> is in the target document. The target document is the document\n                   containing <code>$node</code>, or the document containing the context item\n                      (<code>.</code>) if the second argument is omitted. The behavior of the\n                   function if <code>$node</code> is omitted is exactly the same as if the context\n                   item had been passed as <code>$node</code>.</p></item><item><p><code>E</code> has an <code>ID</code> value equal to one of the candidate\n                      <code>IDREF</code> values, where:</p><ulist><item><p> An element has an <code>ID</code> value equal to <code>V</code> if either\n                         or both of the following conditions are true:</p><ulist><item><p>The <code>is-id</code> property (See <xspecref spec=\"DM30\" ref=\"dm-is-id\"/>.) of the element node is true, and the typed value\n                               of the element node is equal to <code>V</code> under the rules of the\n                                  <code>eq</code> operator using the Unicode codepoint collation\n                                  (<code>http://www.w3.org/2005/xpath-functions/collation/codepoint</code>).</p></item><item><p>The element has an attribute node whose <code>is-id</code> property\n                               (See <xspecref spec=\"DM30\" ref=\"dm-is-id\"/>.) is true and whose typed\n                               value is equal to <code>V</code> under the rules of the\n                                  <code>eq</code> operator using the Unicode code point collation\n                                  (<code>http://www.w3.org/2005/xpath-functions/collation/codepoint</code>).</p></item></ulist></item><item><p> Each <code>xs:string</code> in <code>$arg</code> is parsed as if it were of\n                         type <code>IDREFS</code>, that is, each <code>xs:string</code> in\n                            <code>$arg</code> is treated as a whitespace-separated sequence of\n                         tokens, each acting as an <code>IDREF</code>. These tokens are then included\n                         in the list of candidate <code>IDREF</code>s. If any of the tokens is not a\n                         lexically valid <code>IDREF</code> (that is, if it is not lexically an\n                            <code>xs:NCName</code>), it is ignored. Formally, the candidate\n                            <code>IDREF</code> values are the strings in the sequence given by the\n                         expression:</p><eg xml:space=\"preserve\">for $s in $arg return\n     fn:tokenize(fn:normalize-space($s), ' ')[. castable as xs:IDREF]</eg></item></ulist></item><item><p>If several elements have the same <code>ID</code> value, then <code>E</code> is\n                   the one that is first in document order.</p></item></olist><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">A <phrase diff=\"add\" at=\"L\">dynamic</phrase> error is raised <errorref class=\"DC\" code=\"0001\" type=\"dynamic\"/> if\n                <code>$node</code>, or the context item if the second argument is absent, is a node\n                in a tree whose root is not a document node.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The following errors may be raised when <code>$node</code> is omitted:</p><ul xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\"><li><p>If the context\n                item is <xtermref ref=\"dt-absent\" spec=\"DM30\">absent</xtermref>, <phrase diff=\"add\" at=\"M\">dynamic error</phrase>\n                <xerrorref spec=\"XP\" class=\"DY\" code=\"0002\" type=\"dynamic\"/></p></li><li><p>If the context item is not a\n                node, <phrase diff=\"add\" at=\"M\">type error</phrase> <xerrorref spec=\"XP\" class=\"TY\" code=\"0004\" type=\"type\"/>.</p></li></ul></div>\n", "summary" : "<p>  Returns the sequence of element nodes that have an  ID  value\n             matching the value of one or more of the  IDREF  values supplied in\n                 $arg .</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "arg", "type" : "xs:string", "occurrence" : "*", "description" : "" }, { "name" : "node", "type" : "node()", "occurrence" : null, "description" : "" } ], "returns" : { "type" : "element(*)*", "description" : "" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "idref", "qname" : "fn:idref", "signature" : "($arg as xs:string*) as node()* external", "description" : " <div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">Returns the sequence of element or attribute nodes with an <code>IDREF</code>\n             value matching the value of one or more of the <code>ID</code> values supplied in\n                <code>$arg</code>.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\"><example role=\"signature\"><proto name=\"idref\" return-type=\"node()*\" isOp=\"no\" prefix=\"fn\" returnEmptyOk=\"no\" returnSeq=\"no\" returnVaries=\"no\" isSchema=\"no\" isDatatype=\"no\" isSpecial=\"no\"><arg name=\"arg\" type=\"xs:string*\"/></proto></example><example role=\"signature\"><proto name=\"idref\" return-type=\"node()*\" isOp=\"no\" prefix=\"fn\" returnEmptyOk=\"no\" returnSeq=\"no\" returnVaries=\"no\" isSchema=\"no\" isDatatype=\"no\" isSpecial=\"no\"><arg name=\"arg\" type=\"xs:string*\"/><arg name=\"node\" type=\"node()\"/></proto></example></p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The one-argument form of this function is <termref def=\"dt-deterministic\">deterministic</termref>, <termref def=\"dt-context-dependent\">context-dependent</termref>,  and <termref def=\"dt-focus-dependent\">focus-dependent</termref>. </p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The two-argument form of this function is <termref def=\"dt-deterministic\">deterministic</termref>, <termref def=\"dt-context-independent\">context-independent</termref>,  and <termref def=\"dt-focus-independent\">focus-independent</termref>. </p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\"> The function returns a sequence, in document order with duplicates eliminated,\n             containing every element or attribute node <code>$N</code> that satisfies all the\n             following conditions:</p><olist xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\"><item><p><code>$N</code> is in the target document. The target document is the document\n                   containing <code>$node</code> or the document containing the context item\n                      (<code>.</code>) if the second argument is omitted. The behavior of the\n                   function if <code>$node</code> is omitted is exactly the same as if the context\n                   item had been passed as <code>$node</code>.</p></item><item><p><code>$N</code> has an <code>IDREF</code> value equal to one of the candidate\n                      <code>ID</code> values, where:</p><ulist><item><p>A node <code>$N</code> has an <code>IDREF</code> value equal to\n                            <code>V</code> if both of the following conditions are true:</p><ulist><item><p>The <code>is-idrefs</code> property (see <xspecref spec=\"DM30\" ref=\"dm-is-idrefs\"/>) of <code>$N</code> is <code>true</code>.</p></item><item><p>The sequence <!--Text replaced by erratum E29 change 1\"--><eg diff=\"chg\" at=\"A-E29\" xml:space=\"preserve\">fn:tokenize(fn:normalize-space(fn:string($N)), ' ')</eg>\n                               <!--End of text replaced by erratum E29--> contains a string that is\n                               equal to <code>V</code> under the rules of the <code>eq</code>\n                               operator using the Unicode code point collation\n                                  (<code>http://www.w3.org/2005/xpath-functions/collation/codepoint</code>).</p></item></ulist></item><item><p>Each <code>xs:string</code> in <code>$arg</code> is parsed as if it were of\n                         lexically of type <code>xs:ID</code>. These <code>xs:string</code>s are then\n                         included in the list of candidate <code>xs:ID</code>s. If any of the strings\n                         in <code>$arg</code> is not a lexically valid <code>xs:ID</code> (that is,\n                         if it is not lexically an <code>xs:NCName</code>), it is ignored. More\n                         formally, the candidate <code>ID</code> values are the strings in the\n                         sequence:</p><eg xml:space=\"preserve\">$arg[. castable as xs:NCName]</eg></item></ulist></item></olist><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">A <phrase diff=\"add\" at=\"L\">dynamic</phrase> error is raised <errorref class=\"DC\" code=\"0001\" type=\"dynamic\"/> if\n                <code>$node</code>, or the context item if the second argument is omitted, is a node\n                in a tree whose root is not a document node. </p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The following errors may be raised when <code>$node</code> is omitted:</p><ul xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\"><li><p>If the context\n                item is <xtermref ref=\"dt-absent\" spec=\"DM30\">absent</xtermref>, <phrase diff=\"add\" at=\"M\">dynamic error</phrase>\n                <xerrorref spec=\"XP\" class=\"DY\" code=\"0002\" type=\"dynamic\"/></p></li><li><p>If the context item is not a\n                node, <phrase diff=\"add\" at=\"M\">type error</phrase> <xerrorref spec=\"XP\" class=\"TY\" code=\"0004\" type=\"type\"/>.</p></li></ul></div>\n", "summary" : "<p>  Returns the sequence of element or attribute nodes with an  IDREF \n             value matching the value of one or more of the  ID  values supplied in\n                 $arg .</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "arg", "type" : "xs:string", "occurrence" : "*", "description" : "" } ], "returns" : { "type" : "node()*", "description" : "" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 2, "name" : "idref", "qname" : "fn:idref", "signature" : "($arg as xs:string*, $node as node()) as node()* external", "description" : " <div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">Returns the sequence of element or attribute nodes with an <code>IDREF</code>\n             value matching the value of one or more of the <code>ID</code> values supplied in\n                <code>$arg</code>.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\"><example role=\"signature\"><proto name=\"idref\" return-type=\"node()*\" isOp=\"no\" prefix=\"fn\" returnEmptyOk=\"no\" returnSeq=\"no\" returnVaries=\"no\" isSchema=\"no\" isDatatype=\"no\" isSpecial=\"no\"><arg name=\"arg\" type=\"xs:string*\"/></proto></example><example role=\"signature\"><proto name=\"idref\" return-type=\"node()*\" isOp=\"no\" prefix=\"fn\" returnEmptyOk=\"no\" returnSeq=\"no\" returnVaries=\"no\" isSchema=\"no\" isDatatype=\"no\" isSpecial=\"no\"><arg name=\"arg\" type=\"xs:string*\"/><arg name=\"node\" type=\"node()\"/></proto></example></p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The one-argument form of this function is <termref def=\"dt-deterministic\">deterministic</termref>, <termref def=\"dt-context-dependent\">context-dependent</termref>,  and <termref def=\"dt-focus-dependent\">focus-dependent</termref>. </p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The two-argument form of this function is <termref def=\"dt-deterministic\">deterministic</termref>, <termref def=\"dt-context-independent\">context-independent</termref>,  and <termref def=\"dt-focus-independent\">focus-independent</termref>. </p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\"> The function returns a sequence, in document order with duplicates eliminated,\n             containing every element or attribute node <code>$N</code> that satisfies all the\n             following conditions:</p><olist xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\"><item><p><code>$N</code> is in the target document. The target document is the document\n                   containing <code>$node</code> or the document containing the context item\n                      (<code>.</code>) if the second argument is omitted. The behavior of the\n                   function if <code>$node</code> is omitted is exactly the same as if the context\n                   item had been passed as <code>$node</code>.</p></item><item><p><code>$N</code> has an <code>IDREF</code> value equal to one of the candidate\n                      <code>ID</code> values, where:</p><ulist><item><p>A node <code>$N</code> has an <code>IDREF</code> value equal to\n                            <code>V</code> if both of the following conditions are true:</p><ulist><item><p>The <code>is-idrefs</code> property (see <xspecref spec=\"DM30\" ref=\"dm-is-idrefs\"/>) of <code>$N</code> is <code>true</code>.</p></item><item><p>The sequence <!--Text replaced by erratum E29 change 1\"--><eg diff=\"chg\" at=\"A-E29\" xml:space=\"preserve\">fn:tokenize(fn:normalize-space(fn:string($N)), ' ')</eg>\n                               <!--End of text replaced by erratum E29--> contains a string that is\n                               equal to <code>V</code> under the rules of the <code>eq</code>\n                               operator using the Unicode code point collation\n                                  (<code>http://www.w3.org/2005/xpath-functions/collation/codepoint</code>).</p></item></ulist></item><item><p>Each <code>xs:string</code> in <code>$arg</code> is parsed as if it were of\n                         lexically of type <code>xs:ID</code>. These <code>xs:string</code>s are then\n                         included in the list of candidate <code>xs:ID</code>s. If any of the strings\n                         in <code>$arg</code> is not a lexically valid <code>xs:ID</code> (that is,\n                         if it is not lexically an <code>xs:NCName</code>), it is ignored. More\n                         formally, the candidate <code>ID</code> values are the strings in the\n                         sequence:</p><eg xml:space=\"preserve\">$arg[. castable as xs:NCName]</eg></item></ulist></item></olist><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">A <phrase diff=\"add\" at=\"L\">dynamic</phrase> error is raised <errorref class=\"DC\" code=\"0001\" type=\"dynamic\"/> if\n                <code>$node</code>, or the context item if the second argument is omitted, is a node\n                in a tree whose root is not a document node. </p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The following errors may be raised when <code>$node</code> is omitted:</p><ul xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\"><li><p>If the context\n                item is <xtermref ref=\"dt-absent\" spec=\"DM30\">absent</xtermref>, <phrase diff=\"add\" at=\"M\">dynamic error</phrase>\n                <xerrorref spec=\"XP\" class=\"DY\" code=\"0002\" type=\"dynamic\"/></p></li><li><p>If the context item is not a\n                node, <phrase diff=\"add\" at=\"M\">type error</phrase> <xerrorref spec=\"XP\" class=\"TY\" code=\"0004\" type=\"type\"/>.</p></li></ul></div>\n", "summary" : "<p>  Returns the sequence of element or attribute nodes with an  IDREF \n             value matching the value of one or more of the  ID  values supplied in\n                 $arg .</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "arg", "type" : "xs:string", "occurrence" : "*", "description" : "" }, { "name" : "node", "type" : "node()", "occurrence" : null, "description" : "" } ], "returns" : { "type" : "node()*", "description" : "" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 0, "name" : "implicit-timezone", "qname" : "fn:implicit-timezone", "signature" : "() as xs:dayTimeDuration external", "description" : " <div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">Returns the value of the implicit timezone property from the dynamic context.\n          </p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\"><example role=\"signature\"><proto name=\"implicit-timezone\" return-type=\"xs:dayTimeDuration\" isOp=\"no\" prefix=\"fn\" returnEmptyOk=\"no\" returnSeq=\"no\" returnVaries=\"no\" isSchema=\"no\" isDatatype=\"no\" isSpecial=\"no\"/></example></p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">This function is <termref def=\"dt-deterministic\">deterministic</termref>, <termref def=\"dt-context-dependent\">context-dependent</termref>,  and <termref def=\"dt-focus-independent\">focus-independent</termref>.  It depends on\n \t\timplicit timezone.\n \t</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">Returns the value of the implicit timezone property from the dynamic context. Components\n             of the dynamic context are discussed in <xspecref spec=\"XP30\" ref=\"id-xp-evaluation-context-components\"/>.</p></div>\n", "summary" : "<p>  Returns the value of the implicit timezone property from the dynamic context.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [  ], "returns" : { "type" : "xs:dayTimeDuration", "description" : "" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "in-scope-prefixes", "qname" : "fn:in-scope-prefixes", "signature" : "($element as element(*)) as xs:string* external", "description" : " <div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">Returns the prefixes of the in-scope namespaces for an element node.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\"><example role=\"signature\"><proto name=\"in-scope-prefixes\" return-type=\"xs:string*\" isOp=\"no\" prefix=\"fn\" returnEmptyOk=\"no\" returnSeq=\"no\" returnVaries=\"no\" isSchema=\"no\" isDatatype=\"no\" isSpecial=\"no\"><arg name=\"element\" type=\"element()\"/></proto></example></p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">This function is <termref def=\"dt-deterministic\">deterministic</termref>, <termref def=\"dt-context-independent\">context-independent</termref>,  and <termref def=\"dt-focus-independent\">focus-independent</termref>. </p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The function returns a sequence of strings representing the prefixes of the in-scope\n             namespaces for <code>$element</code>.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">For namespace bindings that have a prefix, the function returns the prefix as an\n                <code>xs:NCName</code>. For the default namespace, which has no prefix, it returns\n             the zero-length string. </p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The result sequence contains no duplicates.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The ordering of the result sequence is <termref def=\"implementation-dependent\">implementation-dependent</termref>.</p></div>\n", "summary" : "<p>  Returns the prefixes of the in-scope namespaces for an element node.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "element", "type" : "element(*)", "occurrence" : null, "description" : "" } ], "returns" : { "type" : "xs:string*", "description" : "" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 2, "name" : "index-of", "qname" : "fn:index-of", "signature" : "($seq as xs:anyAtomicType*, $search as xs:anyAtomicType) as xs:integer* external", "description" : " <div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">Returns a sequence of positive integers giving the positions within the\n             sequence <code>$seq</code> of items that are equal to <code>$search</code>.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\"><example role=\"signature\"><proto name=\"index-of\" return-type=\"xs:integer*\" isOp=\"no\" prefix=\"fn\" returnEmptyOk=\"no\" returnSeq=\"no\" returnVaries=\"no\" isSchema=\"no\" isDatatype=\"no\" isSpecial=\"no\"><arg name=\"seq\" type=\"xs:anyAtomicType*\"/><arg name=\"search\" type=\"xs:anyAtomicType\"/></proto></example><example role=\"signature\"><proto name=\"index-of\" return-type=\"xs:integer*\" isOp=\"no\" prefix=\"fn\" returnEmptyOk=\"no\" returnSeq=\"no\" returnVaries=\"no\" isSchema=\"no\" isDatatype=\"no\" isSpecial=\"no\"><arg name=\"seq\" type=\"xs:anyAtomicType*\"/><arg name=\"search\" type=\"xs:anyAtomicType\"/><arg name=\"collation\" type=\"xs:string\"/></proto></example></p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The two-argument form of this function is <termref def=\"dt-deterministic\">deterministic</termref>, <termref def=\"dt-context-dependent\">context-dependent</termref>,  and <termref def=\"dt-focus-independent\">focus-independent</termref>.  It depends on\n \t\tcollations, and implicit timezone.\n \t</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The three-argument form of this function is <termref def=\"dt-deterministic\">deterministic</termref>, <termref def=\"dt-context-dependent\">context-dependent</termref>,  and <termref def=\"dt-focus-independent\">focus-independent</termref>.  It depends on\n \t\tcollations, and static base uri, and implicit timezone.\n \t</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The function returns a sequence of positive integers giving the positions within the\n             sequence <code>$seq</code> of items that are equal to <code>$search</code>.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The collation used by this function is determined according to the rules in <specref ref=\"choosing-a-collation\"/>. This collation is used when string comparison is\n             required.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The items in the sequence <code>$seq</code> are compared with <code>$search</code> under\n             the rules for the <code>eq</code> operator. Values of type <code>xs:untypedAtomic</code>\n             are compared as if they were of type <code>xs:string</code>. Values that cannot be\n             compared, because the <code>eq</code> operator is not defined for their types, are\n             considered to be distinct. If an item compares equal, then the position of that item in\n             the sequence <code>$seq</code> is included in the result.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The first item in a sequence is at position 1, not position 0.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The result sequence is in ascending numeric order.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">If the value of <code>$seq</code> is the empty sequence, or if no item in\n                <code>$seq</code> matches <code>$search</code>, then the function returns the empty\n             sequence.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\" diff=\"add\" at=\"D\">No error occurs if non-comparable values are encountered. So when\n             comparing two atomic values, the effective boolean value of <code>fn:index-of($a,\n                $b)</code> is true if <code>$a</code> and <code>$b</code> are equal, false if they\n             are not equal or not comparable.</p></div>\n", "summary" : "<p>  Returns a sequence of positive integers giving the positions within the\n             sequence  $seq  of items that are equal to  $search .</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "seq", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "" }, { "name" : "search", "type" : "xs:anyAtomicType", "occurrence" : null, "description" : "" } ], "returns" : { "type" : "xs:integer*", "description" : "" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 3, "name" : "index-of", "qname" : "fn:index-of", "signature" : "($seq as xs:anyAtomicType*, $search as xs:anyAtomicType, $collation as xs:string) as xs:integer* external", "description" : " <div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">Returns a sequence of positive integers giving the positions within the\n             sequence <code>$seq</code> of items that are equal to <code>$search</code>.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\"><example role=\"signature\"><proto name=\"index-of\" return-type=\"xs:integer*\" isOp=\"no\" prefix=\"fn\" returnEmptyOk=\"no\" returnSeq=\"no\" returnVaries=\"no\" isSchema=\"no\" isDatatype=\"no\" isSpecial=\"no\"><arg name=\"seq\" type=\"xs:anyAtomicType*\"/><arg name=\"search\" type=\"xs:anyAtomicType\"/></proto></example><example role=\"signature\"><proto name=\"index-of\" return-type=\"xs:integer*\" isOp=\"no\" prefix=\"fn\" returnEmptyOk=\"no\" returnSeq=\"no\" returnVaries=\"no\" isSchema=\"no\" isDatatype=\"no\" isSpecial=\"no\"><arg name=\"seq\" type=\"xs:anyAtomicType*\"/><arg name=\"search\" type=\"xs:anyAtomicType\"/><arg name=\"collation\" type=\"xs:string\"/></proto></example></p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The two-argument form of this function is <termref def=\"dt-deterministic\">deterministic</termref>, <termref def=\"dt-context-dependent\">context-dependent</termref>,  and <termref def=\"dt-focus-independent\">focus-independent</termref>.  It depends on\n \t\tcollations, and implicit timezone.\n \t</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The three-argument form of this function is <termref def=\"dt-deterministic\">deterministic</termref>, <termref def=\"dt-context-dependent\">context-dependent</termref>,  and <termref def=\"dt-focus-independent\">focus-independent</termref>.  It depends on\n \t\tcollations, and static base uri, and implicit timezone.\n \t</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The function returns a sequence of positive integers giving the positions within the\n             sequence <code>$seq</code> of items that are equal to <code>$search</code>.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The collation used by this function is determined according to the rules in <specref ref=\"choosing-a-collation\"/>. This collation is used when string comparison is\n             required.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The items in the sequence <code>$seq</code> are compared with <code>$search</code> under\n             the rules for the <code>eq</code> operator. Values of type <code>xs:untypedAtomic</code>\n             are compared as if they were of type <code>xs:string</code>. Values that cannot be\n             compared, because the <code>eq</code> operator is not defined for their types, are\n             considered to be distinct. If an item compares equal, then the position of that item in\n             the sequence <code>$seq</code> is included in the result.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The first item in a sequence is at position 1, not position 0.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The result sequence is in ascending numeric order.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">If the value of <code>$seq</code> is the empty sequence, or if no item in\n                <code>$seq</code> matches <code>$search</code>, then the function returns the empty\n             sequence.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\" diff=\"add\" at=\"D\">No error occurs if non-comparable values are encountered. So when\n             comparing two atomic values, the effective boolean value of <code>fn:index-of($a,\n                $b)</code> is true if <code>$a</code> and <code>$b</code> are equal, false if they\n             are not equal or not comparable.</p></div>\n", "summary" : "<p>  Returns a sequence of positive integers giving the positions within the\n             sequence  $seq  of items that are equal to  $search .</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "seq", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "" }, { "name" : "search", "type" : "xs:anyAtomicType", "occurrence" : null, "description" : "" }, { "name" : "collation", "type" : "xs:string", "occurrence" : null, "description" : "" } ], "returns" : { "type" : "xs:integer*", "description" : "" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "innermost", "qname" : "fn:innermost", "signature" : "($nodes as node()*) as node()* external", "description" : " <div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">Returns every node within the input sequence that is not an ancestor of another member\n             of the input sequence; the nodes are returned in document order with duplicates\n             eliminated.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\"><example role=\"signature\"><proto name=\"innermost\" return-type=\"node()*\" isOp=\"no\" prefix=\"fn\" returnEmptyOk=\"no\" returnSeq=\"no\" returnVaries=\"no\" isSchema=\"no\" isDatatype=\"no\" isSpecial=\"no\"><arg name=\"nodes\" type=\"node()*\"/></proto></example></p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">This function is <termref def=\"dt-deterministic\">deterministic</termref>, <termref def=\"dt-context-independent\">context-independent</termref>,  and <termref def=\"dt-focus-independent\">focus-independent</termref>. </p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The effect of the function call <code>fn:innermost($nodes)</code> is defined to be\n             equivalent to the result of the expression <code>$nodes except\n                $nodes/ancestor::node()</code>.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">That is, the function takes as input a sequence of nodes, and returns every node within\n             the sequence that is not an ancestor of another node within the sequence; the nodes are\n             returned in document order with duplicates eliminated.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">If the source document contains nested sections represented by <code>div</code>\n                elements, the expression <code>innermost(//div)</code> returns those <code>div</code>\n                elements that do not contain further <code>div</code> elements.</p></div>\n", "summary" : "<p>  Returns every node within the input sequence that is not an ancestor of another member\n             of the input sequence; the nodes are returned in document order with duplicates\n             eliminated.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "nodes", "type" : "node()", "occurrence" : "*", "description" : "" } ], "returns" : { "type" : "node()*", "description" : "" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 3, "name" : "insert-before", "qname" : "fn:insert-before", "signature" : "($target as item()*, $position as xs:integer, $inserts as item()*) as item()* external", "description" : " <div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">Returns a sequence constructed by inserting an item or a sequence of items at a\n             given position within an existing sequence.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\"><example role=\"signature\"><proto name=\"insert-before\" return-type=\"item()*\" isOp=\"no\" prefix=\"fn\" returnEmptyOk=\"no\" returnSeq=\"no\" returnVaries=\"no\" isSchema=\"no\" isDatatype=\"no\" isSpecial=\"no\"><arg name=\"target\" type=\"item()*\"/><arg name=\"position\" type=\"xs:integer\"/><arg name=\"inserts\" type=\"item()*\"/></proto></example></p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">This function is <termref def=\"dt-deterministic\">deterministic</termref>, <termref def=\"dt-context-independent\">context-independent</termref>,  and <termref def=\"dt-focus-independent\">focus-independent</termref>. </p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The value returned by the function consists of all items of <code>$target</code> whose\n             index is less than <code>$position</code>, followed by all items of\n                <code>$inserts</code>, followed by the remaining elements of <code>$target</code>, in\n             that order. </p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\" diff=\"del\" at=\"F\">For detailed type semantics, see [Formal Semantics].</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">If <code>$target</code> is the empty sequence, <code>$inserts</code> is returned. If\n                <code>$inserts</code> is the empty sequence, <code>$target</code> is returned.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">If <code>$position</code> is less than one (1), the first position, the effective value\n             of <code>$position</code> is one (1). If <code>$position</code> is greater than the\n             number of items in <code>$target</code>, then the effective value of\n                <code>$position</code> is equal to the number of items in <code>$target</code> plus\n             1. </p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The value of <code>$target</code> is not affected by the sequence construction.</p></div>\n", "summary" : "<p>  Returns a sequence constructed by inserting an item or a sequence of items at a\n             given position within an existing sequence.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "target", "type" : "item()", "occurrence" : "*", "description" : "" }, { "name" : "position", "type" : "xs:integer", "occurrence" : null, "description" : "" }, { "name" : "inserts", "type" : "item()", "occurrence" : "*", "description" : "" } ], "returns" : { "type" : "item()*", "description" : "" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "iri-to-uri", "qname" : "fn:iri-to-uri", "signature" : "($iri as xs:string?) as xs:string external", "description" : " <div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">Converts a string containing an IRI into a URI according to the rules of\n                <bibref ref=\"rfc3987\"/>.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\"><example role=\"signature\"><proto name=\"iri-to-uri\" return-type=\"xs:string\" isOp=\"no\" prefix=\"fn\" returnEmptyOk=\"no\" returnSeq=\"no\" returnVaries=\"no\" isSchema=\"no\" isDatatype=\"no\" isSpecial=\"no\"><arg name=\"iri\" type=\"xs:string?\"/></proto></example></p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">This function is <termref def=\"dt-deterministic\">deterministic</termref>, <termref def=\"dt-context-independent\">context-independent</termref>,  and <termref def=\"dt-focus-independent\">focus-independent</termref>. </p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">If <code>$iri</code> is the empty sequence, the function returns the zero-length\n             string.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">Otherwise, the function converts the value of <code>$iri</code> into a URI according to\n             the rules given in Section 3.1 of <bibref ref=\"rfc3987\"/> by percent-encoding characters\n             that are allowed in an IRI but not in a URI. If <code>$iri</code> contains a character\n             that is invalid in an IRI, such as the space character (see note below), the invalid\n             character is replaced by its percent-encoded form as described in <bibref ref=\"rfc3986\"/> before the conversion is performed.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">Since <bibref ref=\"rfc3986\"/> recommends that, for consistency, URI producers and\n             normalizers should use uppercase hexadecimal digits for all percent-encodings, this\n             function must always generate hexadecimal values using the upper-case letters A-F.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The function is idempotent but not invertible. Both the inputs <code>My Documents</code>\n             and <code>My%20Documents</code> will be converted to the output\n                <code>My%20Documents</code>.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">This function does not check whether <code>$iri</code> is a valid IRI. It treats it as\n             an <termref def=\"string\">string</termref> and operates on the <termref def=\"character\">characters</termref> in the string.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\"> The following printable ASCII characters are invalid in an IRI: \"&lt;\", \"&gt;\", <quote>\n                \" </quote> (double quote), space, \"{\", \"}\", \"|\", \"\\\", \"^\", and \"`\". Since these\n             characters should not appear in an IRI, if they do appear in <code>$iri</code> they will\n             be percent-encoded. In addition, characters outside the range x20-<phrase diff=\"chg\" at=\"A-E8\">x7E</phrase> will be percent-encoded because they are invalid in a URI. </p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\"> Since this function does not escape the PERCENT SIGN \"%\" and this character is not\n             allowed in data within a URI, users wishing to convert character strings (such as file\n             names) that include \"%\" to a URI should manually escape \"%\" by replacing it with \"%25\".\n          </p></div>\n", "summary" : "<p>  Converts a string containing an IRI into a URI according to the rules of\n                  .</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "iri", "type" : "xs:string", "occurrence" : "?", "description" : "" } ], "returns" : { "type" : "xs:string", "description" : "" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "lang", "qname" : "fn:lang", "signature" : "($testlang as xs:string?) as xs:boolean external", "description" : " <div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">This function tests whether the language of <code>$node</code>, or the context\n             item if the second argument is omitted, as specified by <code>xml:lang</code> attributes\n             is the same as, or is a sublanguage of, the language specified by\n             <code>$testlang</code>.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\"><example role=\"signature\"><proto name=\"lang\" return-type=\"xs:boolean\" isOp=\"no\" prefix=\"fn\" returnEmptyOk=\"no\" returnSeq=\"no\" returnVaries=\"no\" isSchema=\"no\" isDatatype=\"no\" isSpecial=\"no\"><arg name=\"testlang\" type=\"xs:string?\"/></proto></example><example role=\"signature\"><proto name=\"lang\" return-type=\"xs:boolean\" isOp=\"no\" prefix=\"fn\" returnEmptyOk=\"no\" returnSeq=\"no\" returnVaries=\"no\" isSchema=\"no\" isDatatype=\"no\" isSpecial=\"no\"><arg name=\"testlang\" type=\"xs:string?\"/><arg name=\"node\" type=\"node()\"/></proto></example></p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The one-argument form of this function is <termref def=\"dt-deterministic\">deterministic</termref>, <termref def=\"dt-context-dependent\">context-dependent</termref>,  and <termref def=\"dt-focus-dependent\">focus-dependent</termref>. </p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The two-argument form of this function is <termref def=\"dt-deterministic\">deterministic</termref>, <termref def=\"dt-context-independent\">context-independent</termref>,  and <termref def=\"dt-focus-independent\">focus-independent</termref>. </p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The behavior of the function if the second argument is omitted is exactly the same as if\n             the context item (<code>.</code>) had been passed as the second argument.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The language of the argument <code>$node</code>, or the context item if the second\n             argument is omitted, is determined by the value of the <code>xml:lang</code> attribute\n             on the node, or, if the node has no such attribute, by the value of the\n                <code>xml:lang</code> attribute on the nearest ancestor of the node that has an\n                <code>xml:lang</code> attribute. If there is no such ancestor, then the function\n             returns <code>false</code>. </p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">If <code>$testlang</code> is the empty sequence it is interpreted as the zero-length\n             string.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The relevant <code>xml:lang</code> attribute is determined by the value of the XPath\n             expression:</p><eg xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\" xml:space=\"preserve\">(ancestor-or-self::*/@xml:lang)[last()]</eg><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">If this expression returns an empty sequence, the function returns <code>false</code>. </p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">Otherwise, the function returns <code>true</code> if and only if, based on a caseless\n             default match as specified in section 3.13 of <bibref ref=\"Unicode\"/>, either:</p><olist xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\"><item><p diff=\"chg\" at=\"A-E16\">\n                   <code>$testlang</code> is equal to the string-value of the relevant\n                      <code>xml:lang</code> attribute, or</p></item><item><p diff=\"chg\" at=\"A-E16\">\n                   <code>$testlang</code> is equal to some substring of the string-value of the\n                   relevant <code>xml:lang</code> attribute that starts at the start of the\n                   string-value and ends immediately before a hyphen, \"-\" (the character \"-\" is\n                   HYPHEN-MINUS, #x002D).</p></item></olist><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The following errors may be raised when <code>$arg</code> is omitted:</p><ul xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\"><li><p>If the context\n                item is <xtermref ref=\"dt-absent\" spec=\"DM30\">absent</xtermref>, <phrase diff=\"add\" at=\"M\">dynamic error</phrase>\n                <xerrorref spec=\"XP\" class=\"DY\" code=\"0002\" type=\"dynamic\"/></p></li><li><p>If the context item is not a\n                node, <phrase diff=\"add\" at=\"M\">type error</phrase> <xerrorref spec=\"XP\" class=\"TY\" code=\"0004\" type=\"type\"/>.</p></li></ul></div>\n", "summary" : "<p>  This function tests whether the language of  $node , or the context\n             item if the second argument is omitted, as specified by  xml:lang  attributes\n             is the same as, or is a sublanguage of, the language specified by\n              $testlang .</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "testlang", "type" : "xs:string", "occurrence" : "?", "description" : "" } ], "returns" : { "type" : "xs:boolean", "description" : "" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 2, "name" : "lang", "qname" : "fn:lang", "signature" : "($testlang as xs:string?, $node as node()) as xs:boolean external", "description" : " <div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">This function tests whether the language of <code>$node</code>, or the context\n             item if the second argument is omitted, as specified by <code>xml:lang</code> attributes\n             is the same as, or is a sublanguage of, the language specified by\n             <code>$testlang</code>.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\"><example role=\"signature\"><proto name=\"lang\" return-type=\"xs:boolean\" isOp=\"no\" prefix=\"fn\" returnEmptyOk=\"no\" returnSeq=\"no\" returnVaries=\"no\" isSchema=\"no\" isDatatype=\"no\" isSpecial=\"no\"><arg name=\"testlang\" type=\"xs:string?\"/></proto></example><example role=\"signature\"><proto name=\"lang\" return-type=\"xs:boolean\" isOp=\"no\" prefix=\"fn\" returnEmptyOk=\"no\" returnSeq=\"no\" returnVaries=\"no\" isSchema=\"no\" isDatatype=\"no\" isSpecial=\"no\"><arg name=\"testlang\" type=\"xs:string?\"/><arg name=\"node\" type=\"node()\"/></proto></example></p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The one-argument form of this function is <termref def=\"dt-deterministic\">deterministic</termref>, <termref def=\"dt-context-dependent\">context-dependent</termref>,  and <termref def=\"dt-focus-dependent\">focus-dependent</termref>. </p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The two-argument form of this function is <termref def=\"dt-deterministic\">deterministic</termref>, <termref def=\"dt-context-independent\">context-independent</termref>,  and <termref def=\"dt-focus-independent\">focus-independent</termref>. </p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The behavior of the function if the second argument is omitted is exactly the same as if\n             the context item (<code>.</code>) had been passed as the second argument.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The language of the argument <code>$node</code>, or the context item if the second\n             argument is omitted, is determined by the value of the <code>xml:lang</code> attribute\n             on the node, or, if the node has no such attribute, by the value of the\n                <code>xml:lang</code> attribute on the nearest ancestor of the node that has an\n                <code>xml:lang</code> attribute. If there is no such ancestor, then the function\n             returns <code>false</code>. </p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">If <code>$testlang</code> is the empty sequence it is interpreted as the zero-length\n             string.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The relevant <code>xml:lang</code> attribute is determined by the value of the XPath\n             expression:</p><eg xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\" xml:space=\"preserve\">(ancestor-or-self::*/@xml:lang)[last()]</eg><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">If this expression returns an empty sequence, the function returns <code>false</code>. </p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">Otherwise, the function returns <code>true</code> if and only if, based on a caseless\n             default match as specified in section 3.13 of <bibref ref=\"Unicode\"/>, either:</p><olist xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\"><item><p diff=\"chg\" at=\"A-E16\">\n                   <code>$testlang</code> is equal to the string-value of the relevant\n                      <code>xml:lang</code> attribute, or</p></item><item><p diff=\"chg\" at=\"A-E16\">\n                   <code>$testlang</code> is equal to some substring of the string-value of the\n                   relevant <code>xml:lang</code> attribute that starts at the start of the\n                   string-value and ends immediately before a hyphen, \"-\" (the character \"-\" is\n                   HYPHEN-MINUS, #x002D).</p></item></olist><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The following errors may be raised when <code>$arg</code> is omitted:</p><ul xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\"><li><p>If the context\n                item is <xtermref ref=\"dt-absent\" spec=\"DM30\">absent</xtermref>, <phrase diff=\"add\" at=\"M\">dynamic error</phrase>\n                <xerrorref spec=\"XP\" class=\"DY\" code=\"0002\" type=\"dynamic\"/></p></li><li><p>If the context item is not a\n                node, <phrase diff=\"add\" at=\"M\">type error</phrase> <xerrorref spec=\"XP\" class=\"TY\" code=\"0004\" type=\"type\"/>.</p></li></ul></div>\n", "summary" : "<p>  This function tests whether the language of  $node , or the context\n             item if the second argument is omitted, as specified by  xml:lang  attributes\n             is the same as, or is a sublanguage of, the language specified by\n              $testlang .</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "testlang", "type" : "xs:string", "occurrence" : "?", "description" : "" }, { "name" : "node", "type" : "node()", "occurrence" : null, "description" : "" } ], "returns" : { "type" : "xs:boolean", "description" : "" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 0, "name" : "last", "qname" : "fn:last", "signature" : "() as xs:integer external", "description" : " <div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">Returns the context size from the dynamic context.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\"><example role=\"signature\"><proto name=\"last\" return-type=\"xs:integer\" isOp=\"no\" prefix=\"fn\" returnEmptyOk=\"no\" returnSeq=\"no\" returnVaries=\"no\" isSchema=\"no\" isDatatype=\"no\" isSpecial=\"no\"/></example></p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">This function is <termref def=\"dt-deterministic\">deterministic</termref>, <termref def=\"dt-context-dependent\">context-dependent</termref>,  and <termref def=\"dt-focus-dependent\">focus-dependent</termref>. </p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">Returns the context size from the dynamic context. (See <xspecref spec=\"XP30\" ref=\"id-xp-evaluation-context-components\"/>.)</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">A <phrase diff=\"add\" at=\"M\">dynamic</phrase> error is raised <xerrorref spec=\"XP\" class=\"DY\" code=\"0002\" type=\"type\"/> if the\n             context item is <xtermref ref=\"dt-absent\" spec=\"DM30\">absent</xtermref>.</p></div>\n", "summary" : "<p>  Returns the context size from the dynamic context.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [  ], "returns" : { "type" : "xs:integer", "description" : "" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "local-name-from-QName", "qname" : "fn:local-name-from-QName", "signature" : "($arg as xs:QName?) as xs:NCName? external", "description" : " <div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">Returns the local part of the supplied QName.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\"><example role=\"signature\"><proto name=\"local-name-from-QName\" return-type=\"xs:NCName?\" isOp=\"no\" prefix=\"fn\" returnEmptyOk=\"no\" returnSeq=\"no\" returnVaries=\"no\" isSchema=\"no\" isDatatype=\"no\" isSpecial=\"no\"><arg name=\"arg\" type=\"xs:QName?\"/></proto></example></p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">This function is <termref def=\"dt-deterministic\">deterministic</termref>, <termref def=\"dt-context-independent\">context-independent</termref>,  and <termref def=\"dt-focus-independent\">focus-independent</termref>. </p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">If <code>$arg</code> is the empty sequence the function returns the empty sequence.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">Otherwise, the function returns an <code>xs:NCName</code> representing the local part of\n                <code>$arg</code>.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The expression <code>fn:local-name-from-QName(fn:QName(\"http://www.example.com/example\",\n                   \"person\"))</code> returns <code>\"person\"</code>.</p></div>\n", "summary" : "<p>  Returns the local part of the supplied QName.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "arg", "type" : "xs:QName", "occurrence" : "?", "description" : "" } ], "returns" : { "type" : "xs:NCName?", "description" : "" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 0, "name" : "local-name", "qname" : "fn:local-name", "signature" : "() as xs:string external", "description" : " <div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">Returns the local part of the name of <code>$arg</code> as an\n                <code>xs:string</code> that is either the zero-length string, or has the lexical form\n             of an <code>xs:NCName</code>.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\"><example role=\"signature\"><proto name=\"local-name\" return-type=\"xs:string\" isOp=\"no\" prefix=\"fn\" returnEmptyOk=\"no\" returnSeq=\"no\" returnVaries=\"no\" isSchema=\"no\" isDatatype=\"no\" isSpecial=\"no\"/></example><example role=\"signature\"><proto name=\"local-name\" return-type=\"xs:string\" isOp=\"no\" prefix=\"fn\" returnEmptyOk=\"no\" returnSeq=\"no\" returnVaries=\"no\" isSchema=\"no\" isDatatype=\"no\" isSpecial=\"no\"><arg name=\"arg\" type=\"node()?\"/></proto></example></p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The zero-argument form of this function is <termref def=\"dt-deterministic\">deterministic</termref>, <termref def=\"dt-context-dependent\">context-dependent</termref>,  and <termref def=\"dt-focus-dependent\">focus-dependent</termref>. </p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The one-argument form of this function is <termref def=\"dt-deterministic\">deterministic</termref>, <termref def=\"dt-context-independent\">context-independent</termref>,  and <termref def=\"dt-focus-independent\">focus-independent</termref>. </p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">If the argument is omitted, it defaults to the context item (<code>.</code>). The\n             behavior of the function if the argument is omitted is exactly the same as if the\n             context item had been passed as the argument.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">If the argument is supplied and is the empty sequence, the function returns the\n             zero-length string.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">If the node identified by <code>$arg</code> has no name (that is, if it is a document\n             node, a comment, a text node, or a namespace node having no name), the function returns\n             the zero-length string.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">Otherwise, the function returns the local part of the expanded-QName of the node\n             identified by <code>$arg</code>, as determined by the <code>dm:node-name</code> accessor\n             defined in <xspecref spec=\"DM30\" ref=\"dm-node-name\"/>). This will be an\n                <code>xs:string</code> whose lexical form is an <code>xs:NCName</code>.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The following errors may be raised when <code>$arg</code> is omitted:</p><ul xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\"><li><p>If the context\n                item is <xtermref ref=\"dt-absent\" spec=\"DM30\">absent</xtermref>, <phrase diff=\"add\" at=\"M\">dynamic error</phrase>\n                <xerrorref spec=\"XP\" class=\"DY\" code=\"0002\" type=\"dynamic\"/></p></li><li><p>If the context item is not a\n                node, <phrase diff=\"add\" at=\"M\">type error</phrase> <xerrorref spec=\"XP\" class=\"TY\" code=\"0004\" type=\"type\"/>.</p></li></ul></div>\n", "summary" : "<p>  Returns the local part of the name of  $arg  as an\n                 xs:string  that is either the zero-length string, or has the lexical form\n             of an  xs:NCName .</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [  ], "returns" : { "type" : "xs:string", "description" : "" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "local-name", "qname" : "fn:local-name", "signature" : "($arg as node()?) as xs:string external", "description" : " <div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">Returns the local part of the name of <code>$arg</code> as an\n                <code>xs:string</code> that is either the zero-length string, or has the lexical form\n             of an <code>xs:NCName</code>.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\"><example role=\"signature\"><proto name=\"local-name\" return-type=\"xs:string\" isOp=\"no\" prefix=\"fn\" returnEmptyOk=\"no\" returnSeq=\"no\" returnVaries=\"no\" isSchema=\"no\" isDatatype=\"no\" isSpecial=\"no\"/></example><example role=\"signature\"><proto name=\"local-name\" return-type=\"xs:string\" isOp=\"no\" prefix=\"fn\" returnEmptyOk=\"no\" returnSeq=\"no\" returnVaries=\"no\" isSchema=\"no\" isDatatype=\"no\" isSpecial=\"no\"><arg name=\"arg\" type=\"node()?\"/></proto></example></p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The zero-argument form of this function is <termref def=\"dt-deterministic\">deterministic</termref>, <termref def=\"dt-context-dependent\">context-dependent</termref>,  and <termref def=\"dt-focus-dependent\">focus-dependent</termref>. </p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The one-argument form of this function is <termref def=\"dt-deterministic\">deterministic</termref>, <termref def=\"dt-context-independent\">context-independent</termref>,  and <termref def=\"dt-focus-independent\">focus-independent</termref>. </p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">If the argument is omitted, it defaults to the context item (<code>.</code>). The\n             behavior of the function if the argument is omitted is exactly the same as if the\n             context item had been passed as the argument.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">If the argument is supplied and is the empty sequence, the function returns the\n             zero-length string.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">If the node identified by <code>$arg</code> has no name (that is, if it is a document\n             node, a comment, a text node, or a namespace node having no name), the function returns\n             the zero-length string.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">Otherwise, the function returns the local part of the expanded-QName of the node\n             identified by <code>$arg</code>, as determined by the <code>dm:node-name</code> accessor\n             defined in <xspecref spec=\"DM30\" ref=\"dm-node-name\"/>). This will be an\n                <code>xs:string</code> whose lexical form is an <code>xs:NCName</code>.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The following errors may be raised when <code>$arg</code> is omitted:</p><ul xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\"><li><p>If the context\n                item is <xtermref ref=\"dt-absent\" spec=\"DM30\">absent</xtermref>, <phrase diff=\"add\" at=\"M\">dynamic error</phrase>\n                <xerrorref spec=\"XP\" class=\"DY\" code=\"0002\" type=\"dynamic\"/></p></li><li><p>If the context item is not a\n                node, <phrase diff=\"add\" at=\"M\">type error</phrase> <xerrorref spec=\"XP\" class=\"TY\" code=\"0004\" type=\"type\"/>.</p></li></ul></div>\n", "summary" : "<p>  Returns the local part of the name of  $arg  as an\n                 xs:string  that is either the zero-length string, or has the lexical form\n             of an  xs:NCName .</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "arg", "type" : "node()", "occurrence" : "?", "description" : "" } ], "returns" : { "type" : "xs:string", "description" : "" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "lower-case", "qname" : "fn:lower-case", "signature" : "($arg as xs:string?) as xs:string external", "description" : " <div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">Converts a string to lower case.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\"><example role=\"signature\"><proto name=\"lower-case\" return-type=\"xs:string\" isOp=\"no\" prefix=\"fn\" returnEmptyOk=\"no\" returnSeq=\"no\" returnVaries=\"no\" isSchema=\"no\" isDatatype=\"no\" isSpecial=\"no\"><arg name=\"arg\" type=\"xs:string?\"/></proto></example></p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">This function is <termref def=\"dt-deterministic\">deterministic</termref>, <termref def=\"dt-context-independent\">context-independent</termref>,  and <termref def=\"dt-focus-independent\">focus-independent</termref>. </p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">If the value of <code>$arg</code> is the empty sequence, the zero-length string is\n             returned.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">Otherwise, the function returns the value of <code>$arg</code> after translating every\n                <termref def=\"character\">character</termref> to its lower-case correspondent as\n             defined in the appropriate case mappings section in the Unicode standard <bibref ref=\"Unicode\"/>. For versions of Unicode beginning with the 2.1.8 update, only\n             locale-insensitive case mappings should be applied. Beginning with version 3.2.0 (and\n             likely future versions) of Unicode, precise mappings are described in default case\n             operations, which are full case mappings in the absence of tailoring for particular\n             languages and environments. Every upper-case character that does not have a lower-case\n             correspondent, as well as every lower-case character, is included in the returned value\n             in its original form. </p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">Case mappings may change the length of a string. In general, the\n                <code>fn:upper-case</code> and <code>fn:lower-case</code> functions are not inverses\n             of each other: <code>fn:lower-case(fn:upper-case($arg))</code> is not guaranteed to\n             return <code>$arg</code>, nor is <code>fn:upper-case(fn:lower-case($arg))</code>. The\n             Latin small letter dotless i (as used in Turkish) is perhaps the most prominent\n             lower-case letter which will not round-trip. The Latin capital letter i with dot above\n             is the most prominent upper-case letter which will not round trip; there are others,\n             such as Latin capital letter Sharp S (#1E9E) which is introduced in Unicode 5.1.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\"> These functions may not always be linguistically appropriate (e.g. Turkish i without\n             dot) or appropriate for the application (e.g. titlecase). In cases such as Turkish, a\n             simple translation should be used first.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\"> Because the function is not sensitive to locale, results will not always match user\n             expectations. In Quebec, for example, the standard uppercase equivalent of \"è\" is \"È\",\n             while in metropolitan France it is more commonly \"E\"; only one of these is supported by\n             the functions as defined.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\"> Many characters of class Ll lack uppercase equivalents in the Unicode case mapping\n             tables; many characters of class Lu lack lowercase equivalents.</p></div>\n", "summary" : "<p>  Converts a string to lower case.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "arg", "type" : "xs:string", "occurrence" : "?", "description" : "" } ], "returns" : { "type" : "xs:string", "description" : "" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 2, "name" : "matches", "qname" : "fn:matches", "signature" : "($input as xs:string?, $pattern as xs:string) as xs:boolean external", "description" : " <div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">Returns true if the supplied string matches a given regular expression.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\"><example role=\"signature\"><proto name=\"matches\" return-type=\"xs:boolean\" isOp=\"no\" prefix=\"fn\" returnEmptyOk=\"no\" returnSeq=\"no\" returnVaries=\"no\" isSchema=\"no\" isDatatype=\"no\" isSpecial=\"no\"><arg name=\"input\" type=\"xs:string?\"/><arg name=\"pattern\" type=\"xs:string\"/></proto></example><example role=\"signature\"><proto name=\"matches\" return-type=\"xs:boolean\" isOp=\"no\" prefix=\"fn\" returnEmptyOk=\"no\" returnSeq=\"no\" returnVaries=\"no\" isSchema=\"no\" isDatatype=\"no\" isSpecial=\"no\"><arg name=\"input\" type=\"xs:string?\"/><arg name=\"pattern\" type=\"xs:string\"/><arg name=\"flags\" type=\"xs:string\"/></proto></example></p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">This function is <termref def=\"dt-deterministic\">deterministic</termref>, <termref def=\"dt-context-independent\">context-independent</termref>,  and <termref def=\"dt-focus-independent\">focus-independent</termref>. </p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The effect of calling the first version of this function (omitting the argument\n                <code>$flags</code>) is the same as the effect of calling the second version with the\n                <code>$flags</code> argument set to a zero-length string. Flags are defined in\n                <specref ref=\"flags\"/>.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">If <code>$input</code> is the empty sequence, it is interpreted as the zero-length\n             string.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The function returns <code>true</code> if <code>$input</code> or some substring of\n                <code>$input</code> matches the regular expression supplied as <code>$pattern</code>.\n             Otherwise, the function returns <code>false</code>. The matching rules are influenced by\n             the value of <code>$flags</code> if present. </p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">A <phrase diff=\"add\" at=\"L\">dynamic</phrase> error is raised <errorref class=\"RX\" code=\"0002\"/> if the value of\n                <code>$pattern</code> is invalid according to the rules described in <specref ref=\"regex-syntax\"/>. </p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">A <phrase diff=\"add\" at=\"L\">dynamic</phrase> error is raised <errorref class=\"RX\" code=\"0001\"/> if the value of\n                <code>$flags</code> is invalid according to the rules described in <specref ref=\"flags\"/>. </p></div>\n", "summary" : "<p>  Returns true if the supplied string matches a given regular expression.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "input", "type" : "xs:string", "occurrence" : "?", "description" : "" }, { "name" : "pattern", "type" : "xs:string", "occurrence" : null, "description" : "" } ], "returns" : { "type" : "xs:boolean", "description" : "" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 3, "name" : "matches", "qname" : "fn:matches", "signature" : "($input as xs:string?, $pattern as xs:string, $flags as xs:string) as xs:boolean external", "description" : " <div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">Returns true if the supplied string matches a given regular expression.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\"><example role=\"signature\"><proto name=\"matches\" return-type=\"xs:boolean\" isOp=\"no\" prefix=\"fn\" returnEmptyOk=\"no\" returnSeq=\"no\" returnVaries=\"no\" isSchema=\"no\" isDatatype=\"no\" isSpecial=\"no\"><arg name=\"input\" type=\"xs:string?\"/><arg name=\"pattern\" type=\"xs:string\"/></proto></example><example role=\"signature\"><proto name=\"matches\" return-type=\"xs:boolean\" isOp=\"no\" prefix=\"fn\" returnEmptyOk=\"no\" returnSeq=\"no\" returnVaries=\"no\" isSchema=\"no\" isDatatype=\"no\" isSpecial=\"no\"><arg name=\"input\" type=\"xs:string?\"/><arg name=\"pattern\" type=\"xs:string\"/><arg name=\"flags\" type=\"xs:string\"/></proto></example></p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">This function is <termref def=\"dt-deterministic\">deterministic</termref>, <termref def=\"dt-context-independent\">context-independent</termref>,  and <termref def=\"dt-focus-independent\">focus-independent</termref>. </p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The effect of calling the first version of this function (omitting the argument\n                <code>$flags</code>) is the same as the effect of calling the second version with the\n                <code>$flags</code> argument set to a zero-length string. Flags are defined in\n                <specref ref=\"flags\"/>.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">If <code>$input</code> is the empty sequence, it is interpreted as the zero-length\n             string.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The function returns <code>true</code> if <code>$input</code> or some substring of\n                <code>$input</code> matches the regular expression supplied as <code>$pattern</code>.\n             Otherwise, the function returns <code>false</code>. The matching rules are influenced by\n             the value of <code>$flags</code> if present. </p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">A <phrase diff=\"add\" at=\"L\">dynamic</phrase> error is raised <errorref class=\"RX\" code=\"0002\"/> if the value of\n                <code>$pattern</code> is invalid according to the rules described in <specref ref=\"regex-syntax\"/>. </p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">A <phrase diff=\"add\" at=\"L\">dynamic</phrase> error is raised <errorref class=\"RX\" code=\"0001\"/> if the value of\n                <code>$flags</code> is invalid according to the rules described in <specref ref=\"flags\"/>. </p></div>\n", "summary" : "<p>  Returns true if the supplied string matches a given regular expression.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "input", "type" : "xs:string", "occurrence" : "?", "description" : "" }, { "name" : "pattern", "type" : "xs:string", "occurrence" : null, "description" : "" }, { "name" : "flags", "type" : "xs:string", "occurrence" : null, "description" : "" } ], "returns" : { "type" : "xs:boolean", "description" : "" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "max", "qname" : "fn:max", "signature" : "($arg as xs:anyAtomicType*) as xs:anyAtomicType? external", "description" : " <div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">Returns a value that is equal to the highest value appearing in the input\n             sequence.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\"><example role=\"signature\"><proto name=\"max\" return-type=\"xs:anyAtomicType?\" isOp=\"no\" prefix=\"fn\" returnEmptyOk=\"no\" returnSeq=\"no\" returnVaries=\"no\" isSchema=\"no\" isDatatype=\"no\" isSpecial=\"no\"><arg name=\"arg\" type=\"xs:anyAtomicType*\"/></proto></example><example role=\"signature\"><proto name=\"max\" return-type=\"xs:anyAtomicType?\" isOp=\"no\" prefix=\"fn\" returnEmptyOk=\"no\" returnSeq=\"no\" returnVaries=\"no\" isSchema=\"no\" isDatatype=\"no\" isSpecial=\"no\"><arg name=\"arg\" type=\"xs:anyAtomicType*\"/><arg name=\"collation\" type=\"xs:string\"/></proto></example></p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The zero-argument form of this function is <termref def=\"dt-deterministic\">deterministic</termref>, <termref def=\"dt-context-dependent\">context-dependent</termref>,  and <termref def=\"dt-focus-independent\">focus-independent</termref>.  It depends on\n \t\tcollations, and implicit timezone.\n \t</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The one-argument form of this function is <termref def=\"dt-deterministic\">deterministic</termref>, <termref def=\"dt-context-dependent\">context-dependent</termref>,  and <termref def=\"dt-focus-independent\">focus-independent</termref>.  It depends on\n \t\tcollations, and static base uri, and implicit timezone.\n \t</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The following rules are applied to the input sequence <code>$arg</code>:</p><ulist xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\"><item><p>Values of type <code>xs:untypedAtomic</code> in <code>$arg</code> are cast to\n                      <code>xs:double</code>.</p></item><item><!--Text replaced by erratum E27 change 1\"--><p diff=\"chg\" at=\"A-E27\">Numeric and <code>xs:anyURI</code> values are converted to\n                   the least common type reachable by a combination of type promotion and subtype\n                   substitution. See <xspecref spec=\"XP30\" ref=\"promotion\"/> and <xspecref spec=\"XP30\" ref=\"mapping\"/>.</p><!--End of text replaced by erratum E27--></item></ulist><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The items in the resulting sequence may be reordered in an arbitrary order. The\n             resulting sequence is referred to below as the converted sequence. The function returns\n             an item from the converted sequence rather than the input sequence. </p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">If the converted sequence is empty, the function returns the empty sequence.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">All items in the <phrase diff=\"chg\" at=\"A-E47\">converted sequence must be\n                derived</phrase> from a single base type for which the <code>le</code> operator is\n             defined. In addition, the values in the sequence must have a total order. If date/time\n             values do not have a timezone, they are considered to have the implicit timezone\n             provided by the dynamic context for the purpose of comparison. Duration values must\n             either all be <code>xs:yearMonthDuration</code> values or must all be\n                <code>xs:dayTimeDuration</code> values.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">If the converted sequence contains the value <code>NaN</code>, the value\n                <code>NaN</code> is returned.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">If the items in the <phrase diff=\"chg\" at=\"A-E47\">converted sequence</phrase> are of\n             type <code>xs:string</code> or types derived by restriction from <code>xs:string</code>,\n             then the determination of the item with the smallest value is made according to the\n             collation that is used. If the type of the items in the <phrase diff=\"chg\" at=\"A-E47\">converted sequence</phrase> is not <code>xs:string</code> and\n                <code>$collation</code> is specified, the collation is ignored.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The collation used by this function is determined according to the rules in <specref ref=\"choosing-a-collation\"/>.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The function returns the result of the expression:</p><eg xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\" xml:space=\"preserve\">\n    if (every $v in $c satisfies $c[1] ge $v)\n    then $c[1]\n    else fn:max(fn:subsequence($c, 2))</eg><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">evaluated with <code>$collation</code> as the default collation if specified, and with\n                <code>$c</code> as the converted sequence.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\" diff=\"del\" at=\"F\">For detailed type semantics, see [Formal Semantics].</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">A type error is raised <errorref class=\"RG\" code=\"0006\"/> if the input sequence contains\n             items of incompatible types, as described above.</p></div>\n", "summary" : "<p>  Returns a value that is equal to the highest value appearing in the input\n             sequence.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "arg", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "" } ], "returns" : { "type" : "xs:anyAtomicType?", "description" : "" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 2, "name" : "max", "qname" : "fn:max", "signature" : "($arg as xs:anyAtomicType*, $collation as xs:string) as xs:anyAtomicType? external", "description" : " <div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">Returns a value that is equal to the highest value appearing in the input\n             sequence.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\"><example role=\"signature\"><proto name=\"max\" return-type=\"xs:anyAtomicType?\" isOp=\"no\" prefix=\"fn\" returnEmptyOk=\"no\" returnSeq=\"no\" returnVaries=\"no\" isSchema=\"no\" isDatatype=\"no\" isSpecial=\"no\"><arg name=\"arg\" type=\"xs:anyAtomicType*\"/></proto></example><example role=\"signature\"><proto name=\"max\" return-type=\"xs:anyAtomicType?\" isOp=\"no\" prefix=\"fn\" returnEmptyOk=\"no\" returnSeq=\"no\" returnVaries=\"no\" isSchema=\"no\" isDatatype=\"no\" isSpecial=\"no\"><arg name=\"arg\" type=\"xs:anyAtomicType*\"/><arg name=\"collation\" type=\"xs:string\"/></proto></example></p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The zero-argument form of this function is <termref def=\"dt-deterministic\">deterministic</termref>, <termref def=\"dt-context-dependent\">context-dependent</termref>,  and <termref def=\"dt-focus-independent\">focus-independent</termref>.  It depends on\n \t\tcollations, and implicit timezone.\n \t</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The one-argument form of this function is <termref def=\"dt-deterministic\">deterministic</termref>, <termref def=\"dt-context-dependent\">context-dependent</termref>,  and <termref def=\"dt-focus-independent\">focus-independent</termref>.  It depends on\n \t\tcollations, and static base uri, and implicit timezone.\n \t</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The following rules are applied to the input sequence <code>$arg</code>:</p><ulist xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\"><item><p>Values of type <code>xs:untypedAtomic</code> in <code>$arg</code> are cast to\n                      <code>xs:double</code>.</p></item><item><!--Text replaced by erratum E27 change 1\"--><p diff=\"chg\" at=\"A-E27\">Numeric and <code>xs:anyURI</code> values are converted to\n                   the least common type reachable by a combination of type promotion and subtype\n                   substitution. See <xspecref spec=\"XP30\" ref=\"promotion\"/> and <xspecref spec=\"XP30\" ref=\"mapping\"/>.</p><!--End of text replaced by erratum E27--></item></ulist><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The items in the resulting sequence may be reordered in an arbitrary order. The\n             resulting sequence is referred to below as the converted sequence. The function returns\n             an item from the converted sequence rather than the input sequence. </p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">If the converted sequence is empty, the function returns the empty sequence.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">All items in the <phrase diff=\"chg\" at=\"A-E47\">converted sequence must be\n                derived</phrase> from a single base type for which the <code>le</code> operator is\n             defined. In addition, the values in the sequence must have a total order. If date/time\n             values do not have a timezone, they are considered to have the implicit timezone\n             provided by the dynamic context for the purpose of comparison. Duration values must\n             either all be <code>xs:yearMonthDuration</code> values or must all be\n                <code>xs:dayTimeDuration</code> values.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">If the converted sequence contains the value <code>NaN</code>, the value\n                <code>NaN</code> is returned.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">If the items in the <phrase diff=\"chg\" at=\"A-E47\">converted sequence</phrase> are of\n             type <code>xs:string</code> or types derived by restriction from <code>xs:string</code>,\n             then the determination of the item with the smallest value is made according to the\n             collation that is used. If the type of the items in the <phrase diff=\"chg\" at=\"A-E47\">converted sequence</phrase> is not <code>xs:string</code> and\n                <code>$collation</code> is specified, the collation is ignored.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The collation used by this function is determined according to the rules in <specref ref=\"choosing-a-collation\"/>.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The function returns the result of the expression:</p><eg xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\" xml:space=\"preserve\">\n    if (every $v in $c satisfies $c[1] ge $v)\n    then $c[1]\n    else fn:max(fn:subsequence($c, 2))</eg><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">evaluated with <code>$collation</code> as the default collation if specified, and with\n                <code>$c</code> as the converted sequence.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\" diff=\"del\" at=\"F\">For detailed type semantics, see [Formal Semantics].</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">A type error is raised <errorref class=\"RG\" code=\"0006\"/> if the input sequence contains\n             items of incompatible types, as described above.</p></div>\n", "summary" : "<p>  Returns a value that is equal to the highest value appearing in the input\n             sequence.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "arg", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "" }, { "name" : "collation", "type" : "xs:string", "occurrence" : null, "description" : "" } ], "returns" : { "type" : "xs:anyAtomicType?", "description" : "" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "min", "qname" : "fn:min", "signature" : "($arg as xs:anyAtomicType*) as xs:anyAtomicType? external", "description" : " <div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">Returns a value that is equal to the lowest value appearing in the input\n             sequence.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\"><example role=\"signature\"><proto name=\"min\" return-type=\"xs:anyAtomicType?\" isOp=\"no\" prefix=\"fn\" returnEmptyOk=\"no\" returnSeq=\"no\" returnVaries=\"no\" isSchema=\"no\" isDatatype=\"no\" isSpecial=\"no\"><arg name=\"arg\" type=\"xs:anyAtomicType*\"/></proto></example><example role=\"signature\"><proto name=\"min\" return-type=\"xs:anyAtomicType?\" isOp=\"no\" prefix=\"fn\" returnEmptyOk=\"no\" returnSeq=\"no\" returnVaries=\"no\" isSchema=\"no\" isDatatype=\"no\" isSpecial=\"no\"><arg name=\"arg\" type=\"xs:anyAtomicType*\"/><arg name=\"collation\" type=\"xs:string\"/></proto></example></p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The zero-argument form of this function is <termref def=\"dt-deterministic\">deterministic</termref>, <termref def=\"dt-context-dependent\">context-dependent</termref>,  and <termref def=\"dt-focus-independent\">focus-independent</termref>.  It depends on\n \t\tcollations, and implicit timezone.\n \t</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The one-argument form of this function is <termref def=\"dt-deterministic\">deterministic</termref>, <termref def=\"dt-context-dependent\">context-dependent</termref>,  and <termref def=\"dt-focus-independent\">focus-independent</termref>.  It depends on\n \t\tcollations, and static base uri, and implicit timezone.\n \t</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The following rules are applied to the input sequence:</p><ulist xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\"><item><p>Values of type <code>xs:untypedAtomic</code> in <code>$arg</code> are cast to\n                      <code>xs:double</code>.</p></item><item><!--Text replaced by erratum E27 change 2\"--><p diff=\"chg\" at=\"A-E27\">Numeric and <code>xs:anyURI</code> values are converted to\n                   the least common type reachable by a combination of type promotion and subtype\n                   substitution. See <xspecref spec=\"XP30\" ref=\"promotion\"/> and <xspecref spec=\"XP30\" ref=\"mapping\"/>.</p><!--End of text replaced by erratum E27--></item></ulist><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The items in the resulting sequence may be reordered in an arbitrary order. The\n             resulting sequence is referred to below as the converted sequence. The function returns\n             an item from the converted sequence rather than the input sequence. </p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">If the converted sequence is empty, the empty sequence is returned.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">All items in the <phrase diff=\"chg\" at=\"A-E47\">converted sequence must be\n                derived</phrase> from a single base type for which the <code>le</code> operator is\n             defined. In addition, the values in the sequence must have a total order. If date/time\n             values do not have a timezone, they are considered to have the implicit timezone\n             provided by the dynamic context for the purpose of comparison. Duration values must\n             either all be <code>xs:yearMonthDuration</code> values or must all be\n                <code>xs:dayTimeDuration</code> values.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">If the converted sequence contains the value <code>NaN</code>, the value\n                <code>NaN</code> is returned.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">If the items in the <phrase diff=\"chg\" at=\"A-E47\">converted sequence</phrase> are of\n             type <code>xs:string</code> or types derived by restriction from <code>xs:string</code>,\n             then the determination of the item with the smallest value is made according to the\n             collation that is used. If the type of the items in the <phrase diff=\"chg\" at=\"A-E47\">converted sequence</phrase> is not <code>xs:string</code> and\n                <code>$collation</code> is specified, the collation is ignored.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The collation used by this function is determined according to the rules in <specref ref=\"choosing-a-collation\"/>.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The function returns the result of the expression:</p><eg xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\" xml:space=\"preserve\">\n    if (every $v in $c satisfies $c[1] le $v)\n    then $c[1]\n    else fn:min(fn:subsequence($c, 2))</eg><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">evaluated with <code>$collation</code> as the default collation if specified, and with\n                <code>$c</code> as the converted sequence.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\" diff=\"del\" at=\"F\">For detailed type semantics, see [Formal Semantics].</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">A type error is raised <errorref class=\"RG\" code=\"0006\"/> if the input sequence contains\n             items of incompatible types, as described above.</p></div>\n", "summary" : "<p>  Returns a value that is equal to the lowest value appearing in the input\n             sequence.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "arg", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "" } ], "returns" : { "type" : "xs:anyAtomicType?", "description" : "" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 2, "name" : "min", "qname" : "fn:min", "signature" : "($arg as xs:anyAtomicType*, $collation as xs:string) as xs:anyAtomicType? external", "description" : " <div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">Returns a value that is equal to the lowest value appearing in the input\n             sequence.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\"><example role=\"signature\"><proto name=\"min\" return-type=\"xs:anyAtomicType?\" isOp=\"no\" prefix=\"fn\" returnEmptyOk=\"no\" returnSeq=\"no\" returnVaries=\"no\" isSchema=\"no\" isDatatype=\"no\" isSpecial=\"no\"><arg name=\"arg\" type=\"xs:anyAtomicType*\"/></proto></example><example role=\"signature\"><proto name=\"min\" return-type=\"xs:anyAtomicType?\" isOp=\"no\" prefix=\"fn\" returnEmptyOk=\"no\" returnSeq=\"no\" returnVaries=\"no\" isSchema=\"no\" isDatatype=\"no\" isSpecial=\"no\"><arg name=\"arg\" type=\"xs:anyAtomicType*\"/><arg name=\"collation\" type=\"xs:string\"/></proto></example></p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The zero-argument form of this function is <termref def=\"dt-deterministic\">deterministic</termref>, <termref def=\"dt-context-dependent\">context-dependent</termref>,  and <termref def=\"dt-focus-independent\">focus-independent</termref>.  It depends on\n \t\tcollations, and implicit timezone.\n \t</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The one-argument form of this function is <termref def=\"dt-deterministic\">deterministic</termref>, <termref def=\"dt-context-dependent\">context-dependent</termref>,  and <termref def=\"dt-focus-independent\">focus-independent</termref>.  It depends on\n \t\tcollations, and static base uri, and implicit timezone.\n \t</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The following rules are applied to the input sequence:</p><ulist xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\"><item><p>Values of type <code>xs:untypedAtomic</code> in <code>$arg</code> are cast to\n                      <code>xs:double</code>.</p></item><item><!--Text replaced by erratum E27 change 2\"--><p diff=\"chg\" at=\"A-E27\">Numeric and <code>xs:anyURI</code> values are converted to\n                   the least common type reachable by a combination of type promotion and subtype\n                   substitution. See <xspecref spec=\"XP30\" ref=\"promotion\"/> and <xspecref spec=\"XP30\" ref=\"mapping\"/>.</p><!--End of text replaced by erratum E27--></item></ulist><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The items in the resulting sequence may be reordered in an arbitrary order. The\n             resulting sequence is referred to below as the converted sequence. The function returns\n             an item from the converted sequence rather than the input sequence. </p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">If the converted sequence is empty, the empty sequence is returned.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">All items in the <phrase diff=\"chg\" at=\"A-E47\">converted sequence must be\n                derived</phrase> from a single base type for which the <code>le</code> operator is\n             defined. In addition, the values in the sequence must have a total order. If date/time\n             values do not have a timezone, they are considered to have the implicit timezone\n             provided by the dynamic context for the purpose of comparison. Duration values must\n             either all be <code>xs:yearMonthDuration</code> values or must all be\n                <code>xs:dayTimeDuration</code> values.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">If the converted sequence contains the value <code>NaN</code>, the value\n                <code>NaN</code> is returned.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">If the items in the <phrase diff=\"chg\" at=\"A-E47\">converted sequence</phrase> are of\n             type <code>xs:string</code> or types derived by restriction from <code>xs:string</code>,\n             then the determination of the item with the smallest value is made according to the\n             collation that is used. If the type of the items in the <phrase diff=\"chg\" at=\"A-E47\">converted sequence</phrase> is not <code>xs:string</code> and\n                <code>$collation</code> is specified, the collation is ignored.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The collation used by this function is determined according to the rules in <specref ref=\"choosing-a-collation\"/>.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The function returns the result of the expression:</p><eg xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\" xml:space=\"preserve\">\n    if (every $v in $c satisfies $c[1] le $v)\n    then $c[1]\n    else fn:min(fn:subsequence($c, 2))</eg><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">evaluated with <code>$collation</code> as the default collation if specified, and with\n                <code>$c</code> as the converted sequence.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\" diff=\"del\" at=\"F\">For detailed type semantics, see [Formal Semantics].</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">A type error is raised <errorref class=\"RG\" code=\"0006\"/> if the input sequence contains\n             items of incompatible types, as described above.</p></div>\n", "summary" : "<p>  Returns a value that is equal to the lowest value appearing in the input\n             sequence.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "arg", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "" }, { "name" : "collation", "type" : "xs:string", "occurrence" : null, "description" : "" } ], "returns" : { "type" : "xs:anyAtomicType?", "description" : "" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "minutes-from-dateTime", "qname" : "fn:minutes-from-dateTime", "signature" : "($arg as xs:dateTime?) as xs:integer? external", "description" : " <div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">Returns the minute component of an <code>xs:dateTime</code>.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\"><example role=\"signature\"><proto name=\"minutes-from-dateTime\" return-type=\"xs:integer?\" isOp=\"no\" prefix=\"fn\" returnEmptyOk=\"no\" returnSeq=\"no\" returnVaries=\"no\" isSchema=\"no\" isDatatype=\"no\" isSpecial=\"no\"><arg name=\"arg\" type=\"xs:dateTime?\"/></proto></example></p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">This function is <termref def=\"dt-deterministic\">deterministic</termref>, <termref def=\"dt-context-independent\">context-independent</termref>,  and <termref def=\"dt-focus-independent\">focus-independent</termref>. </p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">If <code>$arg</code> is the empty sequence, the function returns the empty sequence.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">Otherwise, the function returns an <code>xs:integer</code> value between 0 and 59, both\n             inclusive, representing the minute component in the local value of\n             <code>$arg</code>.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The expression <code>fn:minutes-from-dateTime(xs:dateTime(\"1999-05-31T13:20:00-05:00\"))</code> returns <code>20</code>.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The expression <code>fn:minutes-from-dateTime(xs:dateTime(\"1999-05-31T13:30:00+05:30\"))</code> returns <code>30</code>.</p></div>\n", "summary" : "<p>  Returns the minute component of an  xs:dateTime .</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "arg", "type" : "xs:dateTime", "occurrence" : "?", "description" : "" } ], "returns" : { "type" : "xs:integer?", "description" : "" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "minutes-from-duration", "qname" : "fn:minutes-from-duration", "signature" : "($arg as xs:duration?) as xs:integer? external", "description" : " <div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">Returns the number of minutes in a duration.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\"><example role=\"signature\"><proto name=\"minutes-from-duration\" return-type=\"xs:integer?\" isOp=\"no\" prefix=\"fn\" returnEmptyOk=\"no\" returnSeq=\"no\" returnVaries=\"no\" isSchema=\"no\" isDatatype=\"no\" isSpecial=\"no\"><arg name=\"arg\" type=\"xs:duration?\"/></proto></example></p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">This function is <termref def=\"dt-deterministic\">deterministic</termref>, <termref def=\"dt-context-independent\">context-independent</termref>,  and <termref def=\"dt-focus-independent\">focus-independent</termref>. </p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">If <code>$arg</code> is the empty sequence, the function returns the empty sequence.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">Otherwise, the function returns an <code>xs:integer</code> representing the minutes\n             component in the value of <code>$arg</code>. The result is obtained by casting\n                <code>$arg</code> to an <code>xs:dayTimeDuration</code> (see <specref ref=\"casting-to-durations\"/>) and then computing the minutes component as described\n             in <specref ref=\"canonical-dayTimeDuration\"/>.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">If <code>$arg</code> is a negative duration then the result will be negative..</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">If <code>$arg</code> is an <code>xs:yearMonthDuration</code> the function returns 0.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The expression <code>fn:minutes-from-duration(xs:dayTimeDuration(\"P3DT10H\"))</code> returns <code>0</code>.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The expression <code>fn:minutes-from-duration(xs:dayTimeDuration(\"-P5DT12H30M\"))</code> returns <code>-30</code>.</p></div>\n", "summary" : "<p>  Returns the number of minutes in a duration.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "arg", "type" : "xs:duration", "occurrence" : "?", "description" : "" } ], "returns" : { "type" : "xs:integer?", "description" : "" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "minutes-from-time", "qname" : "fn:minutes-from-time", "signature" : "($arg as xs:time?) as xs:integer? external", "description" : " <div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">Returns the minutes component of an <code>xs:time</code>.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\"><example role=\"signature\"><proto name=\"minutes-from-time\" return-type=\"xs:integer?\" isOp=\"no\" prefix=\"fn\" returnEmptyOk=\"no\" returnSeq=\"no\" returnVaries=\"no\" isSchema=\"no\" isDatatype=\"no\" isSpecial=\"no\"><arg name=\"arg\" type=\"xs:time?\"/></proto></example></p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">This function is <termref def=\"dt-deterministic\">deterministic</termref>, <termref def=\"dt-context-independent\">context-independent</termref>,  and <termref def=\"dt-focus-independent\">focus-independent</termref>. </p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">If <code>$arg</code> is the empty sequence, the function returns the empty sequence.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">Otherwise, the function returns an <code>xs:integer</code> value between 0 and 59, both\n             inclusive, representing the value of the minutes component in the local value of\n                <code>$arg</code>.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The expression <code>fn:minutes-from-time(xs:time(\"13:00:00Z\"))</code> returns <code>0</code>.</p></div>\n", "summary" : "<p>  Returns the minutes component of an  xs:time .</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "arg", "type" : "xs:time", "occurrence" : "?", "description" : "" } ], "returns" : { "type" : "xs:integer?", "description" : "" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "month-from-date", "qname" : "fn:month-from-date", "signature" : "($arg as xs:date?) as xs:integer? external", "description" : " <div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">Returns the month component of an <code>xs:date</code>.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\"><example role=\"signature\"><proto name=\"month-from-date\" return-type=\"xs:integer?\" isOp=\"no\" prefix=\"fn\" returnEmptyOk=\"no\" returnSeq=\"no\" returnVaries=\"no\" isSchema=\"no\" isDatatype=\"no\" isSpecial=\"no\"><arg name=\"arg\" type=\"xs:date?\"/></proto></example></p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">This function is <termref def=\"dt-deterministic\">deterministic</termref>, <termref def=\"dt-context-independent\">context-independent</termref>,  and <termref def=\"dt-focus-independent\">focus-independent</termref>. </p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">If <code>$arg</code> is the empty sequence, the function returns the empty sequence.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">Otherwise, the function returns an <code>xs:integer</code> between 1 and 12, both\n             inclusive, representing the month component in the local value of <code>$arg</code>.\n          </p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The expression <code>fn:month-from-date(xs:date(\"1999-05-31-05:00\"))</code> returns <code>5</code>.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The expression <code>fn:month-from-date(xs:date(\"2000-01-01+05:00\"))</code> returns <code>1</code>.</p></div>\n", "summary" : "<p>  Returns the month component of an  xs:date .</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "arg", "type" : "xs:date", "occurrence" : "?", "description" : "" } ], "returns" : { "type" : "xs:integer?", "description" : "" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "months-from-duration", "qname" : "fn:months-from-duration", "signature" : "($arg as xs:duration?) as xs:integer? external", "description" : " <div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">Returns the number of months in a duration.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\"><example role=\"signature\"><proto name=\"months-from-duration\" return-type=\"xs:integer?\" isOp=\"no\" prefix=\"fn\" returnEmptyOk=\"no\" returnSeq=\"no\" returnVaries=\"no\" isSchema=\"no\" isDatatype=\"no\" isSpecial=\"no\"><arg name=\"arg\" type=\"xs:duration?\"/></proto></example></p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">This function is <termref def=\"dt-deterministic\">deterministic</termref>, <termref def=\"dt-context-independent\">context-independent</termref>,  and <termref def=\"dt-focus-independent\">focus-independent</termref>. </p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">If <code>$arg</code> is the empty sequence, the function returns the empty sequence.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">Otherwise, the function returns an <code>xs:integer</code> representing the months\n             component in the value of <code>$arg</code>. The result is obtained by casting\n                <code>$arg</code> to an <code>xs:yearMonthDuration</code> (see <specref ref=\"casting-to-durations\"/>) and then computing the months component as described in\n                <specref ref=\"canonical-yearMonthDuration\"/>.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">If <code>$arg</code> is a negative duration then the result will be negative..</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">If <code>$arg</code> is an <code>xs:dayTimeDuration</code> the function returns 0.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The expression <code>fn:months-from-duration(xs:yearMonthDuration(\"P20Y15M\"))</code> returns <code>3</code>.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The expression <code>fn:months-from-duration(xs:yearMonthDuration(\"-P20Y18M\"))</code> returns <code>-6</code>.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The expression <code>fn:months-from-duration(xs:dayTimeDuration(\"-P2DT15H0M0S\"))</code> returns <code>0</code>.</p></div>\n", "summary" : "<p>  Returns the number of months in a duration.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "arg", "type" : "xs:duration", "occurrence" : "?", "description" : "" } ], "returns" : { "type" : "xs:integer?", "description" : "" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 0, "name" : "name", "qname" : "fn:name", "signature" : "() as xs:string external", "description" : " <div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">Returns the name of a node, as an <code>xs:string</code> that is either the\n             zero-length string, or has the lexical form of an <code>xs:QName</code>. </p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\"><example role=\"signature\"><proto name=\"name\" return-type=\"xs:string\" isOp=\"no\" prefix=\"fn\" returnEmptyOk=\"no\" returnSeq=\"no\" returnVaries=\"no\" isSchema=\"no\" isDatatype=\"no\" isSpecial=\"no\"/></example><example role=\"signature\"><proto name=\"name\" return-type=\"xs:string\" isOp=\"no\" prefix=\"fn\" returnEmptyOk=\"no\" returnSeq=\"no\" returnVaries=\"no\" isSchema=\"no\" isDatatype=\"no\" isSpecial=\"no\"><arg name=\"arg\" type=\"node()?\"/></proto></example></p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The zero-argument form of this function is <termref def=\"dt-deterministic\">deterministic</termref>, <termref def=\"dt-context-dependent\">context-dependent</termref>,  and <termref def=\"dt-focus-dependent\">focus-dependent</termref>. </p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The one-argument form of this function is <termref def=\"dt-deterministic\">deterministic</termref>, <termref def=\"dt-context-independent\">context-independent</termref>,  and <termref def=\"dt-focus-independent\">focus-independent</termref>. </p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">If the argument is omitted, it defaults to the context item (<code>.</code>). The\n             behavior of the function if the argument is omitted is exactly the same as if the\n             context item had been passed as the argument.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">If the argument is supplied and is the empty sequence, the function returns the\n             zero-length string.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">If the node identified by <code>$arg</code> has no name (that is, if it is a document\n             node, a comment, a text node, or a namespace node having no name), the function returns\n             the zero-length string.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">Otherwise, the function returns the value of the expression\n                <code>fn:string(fn:node-name($arg))</code>.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The following errors may be raised when <code>$arg</code> is omitted:</p><ul xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\"><li><p>If the context\n                item is <xtermref ref=\"dt-absent\" spec=\"DM30\">absent</xtermref>, <phrase diff=\"add\" at=\"M\">dynamic error</phrase>\n                <xerrorref spec=\"XP\" class=\"DY\" code=\"0002\" type=\"dynamic\"/></p></li><li><p>If the context item is not a\n                node, <phrase diff=\"add\" at=\"M\">type error</phrase> <xerrorref spec=\"XP\" class=\"TY\" code=\"0004\" type=\"type\"/>.</p></li></ul></div>\n", "summary" : "<p>  Returns the name of a node, as an  xs:string  that is either the\n             zero-length string, or has the lexical form of an  xs:QName .</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [  ], "returns" : { "type" : "xs:string", "description" : "" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "name", "qname" : "fn:name", "signature" : "($arg as node()?) as xs:string external", "description" : " <div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">Returns the name of a node, as an <code>xs:string</code> that is either the\n             zero-length string, or has the lexical form of an <code>xs:QName</code>. </p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\"><example role=\"signature\"><proto name=\"name\" return-type=\"xs:string\" isOp=\"no\" prefix=\"fn\" returnEmptyOk=\"no\" returnSeq=\"no\" returnVaries=\"no\" isSchema=\"no\" isDatatype=\"no\" isSpecial=\"no\"/></example><example role=\"signature\"><proto name=\"name\" return-type=\"xs:string\" isOp=\"no\" prefix=\"fn\" returnEmptyOk=\"no\" returnSeq=\"no\" returnVaries=\"no\" isSchema=\"no\" isDatatype=\"no\" isSpecial=\"no\"><arg name=\"arg\" type=\"node()?\"/></proto></example></p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The zero-argument form of this function is <termref def=\"dt-deterministic\">deterministic</termref>, <termref def=\"dt-context-dependent\">context-dependent</termref>,  and <termref def=\"dt-focus-dependent\">focus-dependent</termref>. </p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The one-argument form of this function is <termref def=\"dt-deterministic\">deterministic</termref>, <termref def=\"dt-context-independent\">context-independent</termref>,  and <termref def=\"dt-focus-independent\">focus-independent</termref>. </p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">If the argument is omitted, it defaults to the context item (<code>.</code>). The\n             behavior of the function if the argument is omitted is exactly the same as if the\n             context item had been passed as the argument.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">If the argument is supplied and is the empty sequence, the function returns the\n             zero-length string.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">If the node identified by <code>$arg</code> has no name (that is, if it is a document\n             node, a comment, a text node, or a namespace node having no name), the function returns\n             the zero-length string.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">Otherwise, the function returns the value of the expression\n                <code>fn:string(fn:node-name($arg))</code>.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The following errors may be raised when <code>$arg</code> is omitted:</p><ul xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\"><li><p>If the context\n                item is <xtermref ref=\"dt-absent\" spec=\"DM30\">absent</xtermref>, <phrase diff=\"add\" at=\"M\">dynamic error</phrase>\n                <xerrorref spec=\"XP\" class=\"DY\" code=\"0002\" type=\"dynamic\"/></p></li><li><p>If the context item is not a\n                node, <phrase diff=\"add\" at=\"M\">type error</phrase> <xerrorref spec=\"XP\" class=\"TY\" code=\"0004\" type=\"type\"/>.</p></li></ul></div>\n", "summary" : "<p>  Returns the name of a node, as an  xs:string  that is either the\n             zero-length string, or has the lexical form of an  xs:QName .</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "arg", "type" : "node()", "occurrence" : "?", "description" : "" } ], "returns" : { "type" : "xs:string", "description" : "" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 2, "name" : "namespace-uri-for-prefix", "qname" : "fn:namespace-uri-for-prefix", "signature" : "($prefix as xs:string?, $element as element(*)) as xs:anyURI? external", "description" : " <div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">Returns the namespace URI of one of the in-scope namespaces for\n                <code>$element</code>, identified by its namespace prefix.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\"><example role=\"signature\"><proto name=\"namespace-uri-for-prefix\" return-type=\"xs:anyURI?\" isOp=\"no\" prefix=\"fn\" returnEmptyOk=\"no\" returnSeq=\"no\" returnVaries=\"no\" isSchema=\"no\" isDatatype=\"no\" isSpecial=\"no\"><arg name=\"prefix\" type=\"xs:string?\"/><arg name=\"element\" type=\"element()\"/></proto></example></p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">This function is <termref def=\"dt-deterministic\">deterministic</termref>, <termref def=\"dt-context-independent\">context-independent</termref>,  and <termref def=\"dt-focus-independent\">focus-independent</termref>. </p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">If <code>$element</code> has an in-scope namespace whose namespace prefix is equal to\n                <code>$prefix</code>, the function returns the namespace URI of that namespace.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">If <code>$element</code> has no in-scope namespace whose namespace prefix is equal to\n             <code>$prefix</code>, the function returns the empty sequence.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\" diff=\"chg\" at=\"I\">If <code>$prefix</code> is the zero-length string or the empty\n             sequence, then if <code>$element</code> has a default namespace (that is, a namespace\n             node with no name), the function returns the namespace URI of the default namespace. If\n                <code>$element</code> has no default namespace, the function returns the empty\n             sequence.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">Prefixes are equal only if their Unicode codepoints match exactly.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">let <code>$e</code> := <eg xml:space=\"preserve\">\n &lt;z:a xmlns=\"http://example.org/one\" xmlns:z=\"http://example.org/two\"&gt;\n   &lt;b xmlns=\"\"/&gt;\n &lt;/z:a&gt;</eg></p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The expression <code>fn:namespace-uri-for-prefix(\"z\", $e)</code> returns <code>\"http://example.org/two\"</code>.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The expression <code>fn:namespace-uri-for-prefix(\"\", $e)</code> returns <code>\"http://example.org/one\"</code>.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The expression <code>fn:namespace-uri-for-prefix((), $e)</code> returns <code>\"http://example.org/one\"</code>.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The expression <code>fn:namespace-uri-for-prefix(\"xml\", $e)</code> returns <code>\"http://www.w3.org/XML/1998/namespace\"</code>.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The expression <code>fn:namespace-uri-for-prefix(\"xml\", $e)</code> returns <code>\"http://www.w3.org/XML/1998/namespace\"</code>.</p></div>\n", "summary" : "<p>  Returns the namespace URI of one of the in-scope namespaces for\n                 $element , identified by its namespace prefix.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "prefix", "type" : "xs:string", "occurrence" : "?", "description" : "" }, { "name" : "element", "type" : "element(*)", "occurrence" : null, "description" : "" } ], "returns" : { "type" : "xs:anyURI?", "description" : "" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "namespace-uri-from-QName", "qname" : "fn:namespace-uri-from-QName", "signature" : "($arg as xs:QName?) as xs:anyURI? external", "description" : " <div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">Returns the namespace URI part of the supplied QName.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\"><example role=\"signature\"><proto name=\"namespace-uri-from-QName\" return-type=\"xs:anyURI?\" isOp=\"no\" prefix=\"fn\" returnEmptyOk=\"no\" returnSeq=\"no\" returnVaries=\"no\" isSchema=\"no\" isDatatype=\"no\" isSpecial=\"no\"><arg name=\"arg\" type=\"xs:QName?\"/></proto></example></p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">This function is <termref def=\"dt-deterministic\">deterministic</termref>, <termref def=\"dt-context-independent\">context-independent</termref>,  and <termref def=\"dt-focus-independent\">focus-independent</termref>. </p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">If <code>$arg</code> is the empty sequence the function returns the empty sequence.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">Otherwise, the function returns an <code>xs:anyURI</code> representing the namespace URI\n             part of <code>$arg</code>.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">If <code>$arg</code> is in no namespace, the function returns the zero-length\n                <code>xs:anyURI</code>.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The expression <code>fn:namespace-uri-from-QName(fn:QName(\"http://www.example.com/example\",\n                   \"person\"))</code> returns <code>xs:anyURI(\"http://www.example.com/example\")</code>.</p></div>\n", "summary" : "<p>  Returns the namespace URI part of the supplied QName.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "arg", "type" : "xs:QName", "occurrence" : "?", "description" : "" } ], "returns" : { "type" : "xs:anyURI?", "description" : "" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 0, "name" : "namespace-uri", "qname" : "fn:namespace-uri", "signature" : "() as xs:anyURI external", "description" : " <div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\" diff=\"chg\" at=\"A-E15\">Returns the namespace URI part of the name of\n                <code>$arg</code>, as an <code>xs:anyURI</code> value.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\"><example role=\"signature\"><proto name=\"namespace-uri\" return-type=\"xs:anyURI\" isOp=\"no\" prefix=\"fn\" returnEmptyOk=\"no\" returnSeq=\"no\" returnVaries=\"no\" isSchema=\"no\" isDatatype=\"no\" isSpecial=\"no\"/></example><example role=\"signature\"><proto name=\"namespace-uri\" return-type=\"xs:anyURI\" isOp=\"no\" prefix=\"fn\" returnEmptyOk=\"no\" returnSeq=\"no\" returnVaries=\"no\" isSchema=\"no\" isDatatype=\"no\" isSpecial=\"no\"><arg name=\"arg\" type=\"node()?\"/></proto></example></p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The zero-argument form of this function is <termref def=\"dt-deterministic\">deterministic</termref>, <termref def=\"dt-context-dependent\">context-dependent</termref>,  and <termref def=\"dt-focus-dependent\">focus-dependent</termref>. </p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The one-argument form of this function is <termref def=\"dt-deterministic\">deterministic</termref>, <termref def=\"dt-context-independent\">context-independent</termref>,  and <termref def=\"dt-focus-independent\">focus-independent</termref>. </p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">If the argument is omitted, it defaults to the context node (<code>.</code>). The\n             behavior of the function if the argument is omitted is exactly the same as if the\n             context item had been passed as the argument.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">If the node identified by <code>$arg</code> is neither an element nor an attribute node,\n             or if it is an element or attribute node whose expanded-QName (as determined by the\n                <code>dm:node-name</code> accessor in the <xspecref spec=\"DM30\" ref=\"dm-node-name\"/>)\n             is in no namespace, then the function returns the zero-length <code>xs:anyURI</code>\n             value.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">Otherwise, the result will be the namespace URI part of the expanded-QName of the node\n             identified by <code>$arg</code>, as determined by the <code>dm:node-name</code> accessor\n             defined in <xspecref spec=\"DM30\" ref=\"dm-node-name\"/>), returned as an\n                <code>xs:anyURI</code> value.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The following errors may be raised when <code>$arg</code> is omitted:</p><ul xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\"><li><p>If the context\n                item is <xtermref ref=\"dt-absent\" spec=\"DM30\">absent</xtermref>, <phrase diff=\"add\" at=\"M\">dynamic error</phrase>\n                <xerrorref spec=\"XP\" class=\"DY\" code=\"0002\" type=\"dynamic\"/></p></li><li><p>If the context item is not a\n                node, <phrase diff=\"add\" at=\"M\">type error</phrase> <xerrorref spec=\"XP\" class=\"TY\" code=\"0004\" type=\"type\"/>.</p></li></ul></div>\n", "summary" : "<p>  Returns the namespace URI part of the name of\n                 $arg , as an  xs:anyURI  value.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [  ], "returns" : { "type" : "xs:anyURI", "description" : "" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "namespace-uri", "qname" : "fn:namespace-uri", "signature" : "($arg as node()?) as xs:anyURI external", "description" : " <div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\" diff=\"chg\" at=\"A-E15\">Returns the namespace URI part of the name of\n                <code>$arg</code>, as an <code>xs:anyURI</code> value.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\"><example role=\"signature\"><proto name=\"namespace-uri\" return-type=\"xs:anyURI\" isOp=\"no\" prefix=\"fn\" returnEmptyOk=\"no\" returnSeq=\"no\" returnVaries=\"no\" isSchema=\"no\" isDatatype=\"no\" isSpecial=\"no\"/></example><example role=\"signature\"><proto name=\"namespace-uri\" return-type=\"xs:anyURI\" isOp=\"no\" prefix=\"fn\" returnEmptyOk=\"no\" returnSeq=\"no\" returnVaries=\"no\" isSchema=\"no\" isDatatype=\"no\" isSpecial=\"no\"><arg name=\"arg\" type=\"node()?\"/></proto></example></p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The zero-argument form of this function is <termref def=\"dt-deterministic\">deterministic</termref>, <termref def=\"dt-context-dependent\">context-dependent</termref>,  and <termref def=\"dt-focus-dependent\">focus-dependent</termref>. </p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The one-argument form of this function is <termref def=\"dt-deterministic\">deterministic</termref>, <termref def=\"dt-context-independent\">context-independent</termref>,  and <termref def=\"dt-focus-independent\">focus-independent</termref>. </p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">If the argument is omitted, it defaults to the context node (<code>.</code>). The\n             behavior of the function if the argument is omitted is exactly the same as if the\n             context item had been passed as the argument.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">If the node identified by <code>$arg</code> is neither an element nor an attribute node,\n             or if it is an element or attribute node whose expanded-QName (as determined by the\n                <code>dm:node-name</code> accessor in the <xspecref spec=\"DM30\" ref=\"dm-node-name\"/>)\n             is in no namespace, then the function returns the zero-length <code>xs:anyURI</code>\n             value.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">Otherwise, the result will be the namespace URI part of the expanded-QName of the node\n             identified by <code>$arg</code>, as determined by the <code>dm:node-name</code> accessor\n             defined in <xspecref spec=\"DM30\" ref=\"dm-node-name\"/>), returned as an\n                <code>xs:anyURI</code> value.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The following errors may be raised when <code>$arg</code> is omitted:</p><ul xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\"><li><p>If the context\n                item is <xtermref ref=\"dt-absent\" spec=\"DM30\">absent</xtermref>, <phrase diff=\"add\" at=\"M\">dynamic error</phrase>\n                <xerrorref spec=\"XP\" class=\"DY\" code=\"0002\" type=\"dynamic\"/></p></li><li><p>If the context item is not a\n                node, <phrase diff=\"add\" at=\"M\">type error</phrase> <xerrorref spec=\"XP\" class=\"TY\" code=\"0004\" type=\"type\"/>.</p></li></ul></div>\n", "summary" : "<p>  Returns the namespace URI part of the name of\n                 $arg , as an  xs:anyURI  value.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "arg", "type" : "node()", "occurrence" : "?", "description" : "" } ], "returns" : { "type" : "xs:anyURI", "description" : "" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 0, "name" : "nilled", "qname" : "fn:nilled", "signature" : "() as xs:boolean external", "description" : " <div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">Returns true for an element that is <term>nilled</term>.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\"><example role=\"signature\"><proto name=\"nilled\" return-type=\"xs:boolean\" isOp=\"no\" prefix=\"fn\" returnEmptyOk=\"no\" returnSeq=\"no\" returnVaries=\"no\" isSchema=\"no\" isDatatype=\"no\" isSpecial=\"no\"/></example><example role=\"signature\"><proto name=\"nilled\" return-type=\"xs:boolean?\" isOp=\"no\" prefix=\"fn\" returnEmptyOk=\"no\" returnSeq=\"no\" returnVaries=\"no\" isSchema=\"no\" isDatatype=\"no\" isSpecial=\"no\"><arg name=\"arg\" type=\"node()?\"/></proto></example></p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The zero-argument form of this function is <termref def=\"dt-deterministic\">deterministic</termref>, <termref def=\"dt-context-dependent\">context-dependent</termref>,  and <termref def=\"dt-focus-dependent\">focus-dependent</termref>. </p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The one-argument form of this function is <termref def=\"dt-deterministic\">deterministic</termref>, <termref def=\"dt-context-independent\">context-independent</termref>,  and <termref def=\"dt-focus-independent\">focus-independent</termref>. </p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">If the argument is omitted, it defaults to the context item (<code>.</code>). The\n             behavior of the function if the argument is omitted is exactly the same as if the\n             context item had been passed as the argument.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">If <code>$arg</code> is the empty sequence, the function returns the empty sequence.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">Otherwise the function returns the result of the <code>dm:nilled</code> accessor as\n             defined in <bibref ref=\"xpath-datamodel-30\"/> (see <xspecref spec=\"DM30\" ref=\"dm-nilled\"/>).</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\" diff=\"add\" at=\"I\">The following errors may be raised when <code>$arg</code> is omitted:</p><ul xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\"><li><p>If the context\n                item is <xtermref ref=\"dt-absent\" spec=\"DM30\">absent</xtermref>, <phrase diff=\"add\" at=\"M\">dynamic error</phrase>\n                <xerrorref spec=\"XP\" class=\"DY\" code=\"0002\" type=\"dynamic\"/></p></li><li><p>If the context item is not a\n                node, <phrase diff=\"add\" at=\"M\">type error</phrase> <xerrorref spec=\"XP\" class=\"TY\" code=\"0004\" type=\"type\"/>.</p></li></ul></div>\n", "summary" : "<p>  Returns true for an element that is  nilled .</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [  ], "returns" : { "type" : "xs:boolean", "description" : "" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "nilled", "qname" : "fn:nilled", "signature" : "($arg as node()?) as xs:boolean? external", "description" : " <div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">Returns true for an element that is <term>nilled</term>.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\"><example role=\"signature\"><proto name=\"nilled\" return-type=\"xs:boolean\" isOp=\"no\" prefix=\"fn\" returnEmptyOk=\"no\" returnSeq=\"no\" returnVaries=\"no\" isSchema=\"no\" isDatatype=\"no\" isSpecial=\"no\"/></example><example role=\"signature\"><proto name=\"nilled\" return-type=\"xs:boolean?\" isOp=\"no\" prefix=\"fn\" returnEmptyOk=\"no\" returnSeq=\"no\" returnVaries=\"no\" isSchema=\"no\" isDatatype=\"no\" isSpecial=\"no\"><arg name=\"arg\" type=\"node()?\"/></proto></example></p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The zero-argument form of this function is <termref def=\"dt-deterministic\">deterministic</termref>, <termref def=\"dt-context-dependent\">context-dependent</termref>,  and <termref def=\"dt-focus-dependent\">focus-dependent</termref>. </p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The one-argument form of this function is <termref def=\"dt-deterministic\">deterministic</termref>, <termref def=\"dt-context-independent\">context-independent</termref>,  and <termref def=\"dt-focus-independent\">focus-independent</termref>. </p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">If the argument is omitted, it defaults to the context item (<code>.</code>). The\n             behavior of the function if the argument is omitted is exactly the same as if the\n             context item had been passed as the argument.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">If <code>$arg</code> is the empty sequence, the function returns the empty sequence.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">Otherwise the function returns the result of the <code>dm:nilled</code> accessor as\n             defined in <bibref ref=\"xpath-datamodel-30\"/> (see <xspecref spec=\"DM30\" ref=\"dm-nilled\"/>).</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\" diff=\"add\" at=\"I\">The following errors may be raised when <code>$arg</code> is omitted:</p><ul xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\"><li><p>If the context\n                item is <xtermref ref=\"dt-absent\" spec=\"DM30\">absent</xtermref>, <phrase diff=\"add\" at=\"M\">dynamic error</phrase>\n                <xerrorref spec=\"XP\" class=\"DY\" code=\"0002\" type=\"dynamic\"/></p></li><li><p>If the context item is not a\n                node, <phrase diff=\"add\" at=\"M\">type error</phrase> <xerrorref spec=\"XP\" class=\"TY\" code=\"0004\" type=\"type\"/>.</p></li></ul></div>\n", "summary" : "<p>  Returns true for an element that is  nilled .</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "arg", "type" : "node()", "occurrence" : "?", "description" : "" } ], "returns" : { "type" : "xs:boolean?", "description" : "" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 0, "name" : "node-name", "qname" : "fn:node-name", "signature" : "() as xs:QName? external", "description" : " <div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">Returns the name of a node, as an <code>xs:QName</code>.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\"><example role=\"signature\"><proto name=\"node-name\" return-type=\"xs:QName?\" isOp=\"no\" prefix=\"fn\" returnEmptyOk=\"no\" returnSeq=\"no\" returnVaries=\"no\" isSchema=\"no\" isDatatype=\"no\" isSpecial=\"no\"/></example><example role=\"signature\"><proto name=\"node-name\" return-type=\"xs:QName?\" isOp=\"no\" prefix=\"fn\" returnEmptyOk=\"no\" returnSeq=\"no\" returnVaries=\"no\" isSchema=\"no\" isDatatype=\"no\" isSpecial=\"no\"><arg name=\"arg\" type=\"node()?\"/></proto></example></p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The zero-argument form of this function is <termref def=\"dt-deterministic\">deterministic</termref>, <termref def=\"dt-context-dependent\">context-dependent</termref>,  and <termref def=\"dt-focus-dependent\">focus-dependent</termref>. </p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The one-argument form of this function is <termref def=\"dt-deterministic\">deterministic</termref>, <termref def=\"dt-context-independent\">context-independent</termref>,  and <termref def=\"dt-focus-independent\">focus-independent</termref>. </p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\" diff=\"add\" at=\"F\">If the argument is omitted, it defaults to the context item\n                (<code>.</code>). The behavior of the function if the argument is omitted is exactly\n             the same as if the context item had been passed as the argument.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">If <code>$arg</code> is the empty sequence, the empty sequence is returned.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">Otherwise, the function returns the result of the <code>dm:node-name</code> accessor as\n             defined in <bibref ref=\"xpath-datamodel-30\"/> (see <xspecref spec=\"DM30\" ref=\"dm-node-name\"/>).</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\" diff=\"add\" at=\"I\">The following errors may be raised when <code>$arg</code> is omitted:</p><ul xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\"><li><p>If the context\n                item is <xtermref ref=\"dt-absent\" spec=\"DM30\">absent</xtermref>, <phrase diff=\"add\" at=\"M\">dynamic error</phrase>\n                <xerrorref spec=\"XP\" class=\"DY\" code=\"0002\" type=\"dynamic\"/></p></li><li><p>If the context item is not a\n                node, <phrase diff=\"add\" at=\"M\">type error</phrase> <xerrorref spec=\"XP\" class=\"TY\" code=\"0004\" type=\"type\"/>.</p></li></ul></div>\n", "summary" : "<p>  Returns the name of a node, as an  xs:QName .</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [  ], "returns" : { "type" : "xs:QName?", "description" : "" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "node-name", "qname" : "fn:node-name", "signature" : "($arg as node()?) as xs:QName? external", "description" : " <div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">Returns the name of a node, as an <code>xs:QName</code>.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\"><example role=\"signature\"><proto name=\"node-name\" return-type=\"xs:QName?\" isOp=\"no\" prefix=\"fn\" returnEmptyOk=\"no\" returnSeq=\"no\" returnVaries=\"no\" isSchema=\"no\" isDatatype=\"no\" isSpecial=\"no\"/></example><example role=\"signature\"><proto name=\"node-name\" return-type=\"xs:QName?\" isOp=\"no\" prefix=\"fn\" returnEmptyOk=\"no\" returnSeq=\"no\" returnVaries=\"no\" isSchema=\"no\" isDatatype=\"no\" isSpecial=\"no\"><arg name=\"arg\" type=\"node()?\"/></proto></example></p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The zero-argument form of this function is <termref def=\"dt-deterministic\">deterministic</termref>, <termref def=\"dt-context-dependent\">context-dependent</termref>,  and <termref def=\"dt-focus-dependent\">focus-dependent</termref>. </p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The one-argument form of this function is <termref def=\"dt-deterministic\">deterministic</termref>, <termref def=\"dt-context-independent\">context-independent</termref>,  and <termref def=\"dt-focus-independent\">focus-independent</termref>. </p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\" diff=\"add\" at=\"F\">If the argument is omitted, it defaults to the context item\n                (<code>.</code>). The behavior of the function if the argument is omitted is exactly\n             the same as if the context item had been passed as the argument.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">If <code>$arg</code> is the empty sequence, the empty sequence is returned.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">Otherwise, the function returns the result of the <code>dm:node-name</code> accessor as\n             defined in <bibref ref=\"xpath-datamodel-30\"/> (see <xspecref spec=\"DM30\" ref=\"dm-node-name\"/>).</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\" diff=\"add\" at=\"I\">The following errors may be raised when <code>$arg</code> is omitted:</p><ul xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\"><li><p>If the context\n                item is <xtermref ref=\"dt-absent\" spec=\"DM30\">absent</xtermref>, <phrase diff=\"add\" at=\"M\">dynamic error</phrase>\n                <xerrorref spec=\"XP\" class=\"DY\" code=\"0002\" type=\"dynamic\"/></p></li><li><p>If the context item is not a\n                node, <phrase diff=\"add\" at=\"M\">type error</phrase> <xerrorref spec=\"XP\" class=\"TY\" code=\"0004\" type=\"type\"/>.</p></li></ul></div>\n", "summary" : "<p>  Returns the name of a node, as an  xs:QName .</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "arg", "type" : "node()", "occurrence" : "?", "description" : "" } ], "returns" : { "type" : "xs:QName?", "description" : "" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 0, "name" : "normalize-space", "qname" : "fn:normalize-space", "signature" : "() as xs:string external", "description" : " <div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">Returns the value of <code>$arg</code> with leading and trailing whitespace\n             removed, and sequences of internal whitespace reduced to a single space character.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\"><example role=\"signature\"><proto name=\"normalize-space\" return-type=\"xs:string\" isOp=\"no\" prefix=\"fn\" returnEmptyOk=\"no\" returnSeq=\"no\" returnVaries=\"no\" isSchema=\"no\" isDatatype=\"no\" isSpecial=\"no\"/></example><example role=\"signature\"><proto name=\"normalize-space\" return-type=\"xs:string\" isOp=\"no\" prefix=\"fn\" returnEmptyOk=\"no\" returnSeq=\"no\" returnVaries=\"no\" isSchema=\"no\" isDatatype=\"no\" isSpecial=\"no\"><arg name=\"arg\" type=\"xs:string?\"/></proto></example></p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The zero-argument form of this function is <termref def=\"dt-deterministic\">deterministic</termref>, <termref def=\"dt-context-dependent\">context-dependent</termref>,  and <termref def=\"dt-focus-dependent\">focus-dependent</termref>. </p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The one-argument form of this function is <termref def=\"dt-deterministic\">deterministic</termref>, <termref def=\"dt-context-independent\">context-independent</termref>,  and <termref def=\"dt-focus-independent\">focus-independent</termref>. </p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">If the value of <code>$arg</code> is the empty sequence, the function returns the\n             zero-length string.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The function returns a string constructed by stripping leading and trailing whitespace\n             from the value of <code>$arg</code>, and replacing sequences of one or more adjacent\n             whitespace characters with a single space, <code>#x20</code>.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The whitespace characters are defined in the metasymbol S (Production 3) of <bibref ref=\"REC-xml\"/>.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\" diff=\"chg\" at=\"A-E14\">If no argument is supplied, then <code>$arg</code> defaults to the\n             string value (calculated using <code>fn:string</code>) of the context item\n                (<code>.</code>). </p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">If no argument is supplied and the context item is <xtermref ref=\"dt-absent\" spec=\"DM30\">absent</xtermref> then a <phrase diff=\"add\" at=\"dynamic\">dynamic</phrase> error is raised: <xerrorref spec=\"XP\" class=\"DY\" code=\"0002\" type=\"dynamic\"/>.</p></div>\n", "summary" : "<p>  Returns the value of  $arg  with leading and trailing whitespace\n             removed, and sequences of internal whitespace reduced to a single space character.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [  ], "returns" : { "type" : "xs:string", "description" : "" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "normalize-space", "qname" : "fn:normalize-space", "signature" : "($arg as xs:string?) as xs:string external", "description" : " <div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">Returns the value of <code>$arg</code> with leading and trailing whitespace\n             removed, and sequences of internal whitespace reduced to a single space character.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\"><example role=\"signature\"><proto name=\"normalize-space\" return-type=\"xs:string\" isOp=\"no\" prefix=\"fn\" returnEmptyOk=\"no\" returnSeq=\"no\" returnVaries=\"no\" isSchema=\"no\" isDatatype=\"no\" isSpecial=\"no\"/></example><example role=\"signature\"><proto name=\"normalize-space\" return-type=\"xs:string\" isOp=\"no\" prefix=\"fn\" returnEmptyOk=\"no\" returnSeq=\"no\" returnVaries=\"no\" isSchema=\"no\" isDatatype=\"no\" isSpecial=\"no\"><arg name=\"arg\" type=\"xs:string?\"/></proto></example></p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The zero-argument form of this function is <termref def=\"dt-deterministic\">deterministic</termref>, <termref def=\"dt-context-dependent\">context-dependent</termref>,  and <termref def=\"dt-focus-dependent\">focus-dependent</termref>. </p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The one-argument form of this function is <termref def=\"dt-deterministic\">deterministic</termref>, <termref def=\"dt-context-independent\">context-independent</termref>,  and <termref def=\"dt-focus-independent\">focus-independent</termref>. </p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">If the value of <code>$arg</code> is the empty sequence, the function returns the\n             zero-length string.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The function returns a string constructed by stripping leading and trailing whitespace\n             from the value of <code>$arg</code>, and replacing sequences of one or more adjacent\n             whitespace characters with a single space, <code>#x20</code>.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The whitespace characters are defined in the metasymbol S (Production 3) of <bibref ref=\"REC-xml\"/>.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\" diff=\"chg\" at=\"A-E14\">If no argument is supplied, then <code>$arg</code> defaults to the\n             string value (calculated using <code>fn:string</code>) of the context item\n                (<code>.</code>). </p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">If no argument is supplied and the context item is <xtermref ref=\"dt-absent\" spec=\"DM30\">absent</xtermref> then a <phrase diff=\"add\" at=\"dynamic\">dynamic</phrase> error is raised: <xerrorref spec=\"XP\" class=\"DY\" code=\"0002\" type=\"dynamic\"/>.</p></div>\n", "summary" : "<p>  Returns the value of  $arg  with leading and trailing whitespace\n             removed, and sequences of internal whitespace reduced to a single space character.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "arg", "type" : "xs:string", "occurrence" : "?", "description" : "" } ], "returns" : { "type" : "xs:string", "description" : "" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "normalize-unicode", "qname" : "fn:normalize-unicode", "signature" : "($arg as xs:string?) as xs:string external", "description" : " <div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">Returns the value of <code>$arg</code> after applying Unicode\n             normalization.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\"><example role=\"signature\"><proto name=\"normalize-unicode\" return-type=\"xs:string\" isOp=\"no\" prefix=\"fn\" returnEmptyOk=\"no\" returnSeq=\"no\" returnVaries=\"no\" isSchema=\"no\" isDatatype=\"no\" isSpecial=\"no\"><arg name=\"arg\" type=\"xs:string?\"/></proto></example><example role=\"signature\"><proto name=\"normalize-unicode\" return-type=\"xs:string\" isOp=\"no\" prefix=\"fn\" returnEmptyOk=\"no\" returnSeq=\"no\" returnVaries=\"no\" isSchema=\"no\" isDatatype=\"no\" isSpecial=\"no\"><arg name=\"arg\" type=\"xs:string?\"/><arg name=\"normalizationForm\" type=\"xs:string\"/></proto></example></p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">This function is <termref def=\"dt-deterministic\">deterministic</termref>, <termref def=\"dt-context-independent\">context-independent</termref>,  and <termref def=\"dt-focus-independent\">focus-independent</termref>. </p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">If the value of <code>$arg</code> is the empty sequence, the function returns the\n             zero-length string.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">If the single-argument version of the function is used, the result is the same as\n             calling the two-argument version with <code>$normalizationForm</code> set to the string\n             \"NFC\".</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">Otherwise, the function returns the value of <code>$arg</code> normalized according to\n             the rules of the normalization form identified by the value of\n                <code>$normalizationForm</code>.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The effective value of <code>$normalizationForm</code> is the value of the expression\n                <code>fn:upper-case(fn:normalize-space($normalizationForm))</code>.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\" diff=\"del\" at=\"D\">See <bibref ref=\"charmod-normalization\"/> for a description of the\n             normalization forms.</p><ulist xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\"><item><p>If the effective value of <code>$normalizationForm</code> is <quote>NFC</quote>,\n                   then the function returns the value of <code>$arg</code> converted to Unicode\n                   Normalization Form C (NFC).</p></item><item><p>If the effective value of <code>$normalizationForm</code> is <quote>NFD</quote>,\n                   then the function returns the value of <code>$arg</code> converted to Unicode\n                   Normalization Form D (NFD).</p></item><item><p>If the effective value of <code>$normalizationForm</code> is <quote>NFKC</quote>,\n                   then the function returns the value of <code>$arg</code> in Unicode Normalization\n                   Form KC (NFKC).</p></item><item><p>If the effective value of <code>$normalizationForm</code> is <quote>NFKD</quote>,\n                   then the function returns the value of <code>$arg</code> converted to Unicode\n                   Normalization Form KD (NFKD).</p></item><item><p>If the effective value of <code>$normalizationForm</code> is\n                      <quote>FULLY-NORMALIZED</quote>, then the function returns the value of\n                      <code>$arg</code> converted to fully normalized form. </p></item><item><p>If the effective value of <code>$normalizationForm</code> is the zero-length\n                   string, no normalization is performed and <code>$arg</code> is returned.</p></item></ulist><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\" diff=\"add\" at=\"D\">Normalization forms NFC, NFD, NFKC, and NFKD, and the algorithms to be\n             used for converting a string to each of these forms, are defined in <bibref ref=\"Unicode-Normalization\"/>.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\" diff=\"add\" at=\"D\">The motivation for normalization form FULLY-NORMALIZED is explained in\n                <bibref ref=\"charmod-normalization\"/>. However, as that specification did not\n             progress beyond working draft status, the normative specification is as follows:</p><ulist xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\" diff=\"add\" at=\"D\"><item><p>A string is <term>fully-normalized</term> if (a) it is in normalization form NFC\n                   as defined in <bibref ref=\"Unicode-Normalization\"/>, and (b) it does not start\n                   with a composing character.</p></item><item><p>A composing character is a character that is one or both of the following:</p><ulist><item><p>the second character in the canonical decomposition mapping of some\n                         character that is not listed in the Composition Exclusion Table defined in\n                            <bibref ref=\"Unicode-Normalization\"/>;</p></item><item><p>of non-zero canonical combining class (as defined in <bibref ref=\"Unicode\"/>).</p></item></ulist></item><item><p>A string is converted to FULLY-NORMALIZED form as follows:</p><ulist><item><p>if the first character in the string is a composing character, prepend a\n                         single space (x20);</p></item><item><p>convert the resulting string to normalization form NFC.</p></item></ulist></item></ulist><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\" diff=\"add\" At=\"L\">Conforming implementations <rfc2119>must</rfc2119> support normalization form \"NFC\" and\n                <rfc2119>may</rfc2119> support normalization forms \"NFD\", \"NFKC\", \"NFKD\", and\n             \"FULLY-NORMALIZED\". They <rfc2119>may</rfc2119> also support other normalization forms\n             with <termref def=\"implementation-defined\">implementation-defined</termref> semantics. </p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\" diff=\"add\" at=\"L\">It is <termref def=\"dt-implementation-defined\">implementation-defined</termref> which version of\n             Unicode (and therefore, of the normalization algorithms and their underlying\n             data) is supported by the implementation. See <bibref ref=\"Unicode-Normalization\"/> for details of the\n             stability policy regarding changes to the normalization rules in future\n             versions of Unicode. If the input string contains codepoints that are\n             unassigned in the relevant version of Unicode, or for which no normalization\n             rules are defined, the <code>fn:normalize-unicode</code> function leaves such codepoints\n             unchanged. If the implementation supports the requested normalization form then\n             it <rfc2119>must</rfc2119> be able to handle every input string without raising an error.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">A <phrase diff=\"add\" at=\"L\">dynamic</phrase> error is raised <errorref class=\"CH\" code=\"0003\"/> if the effective value of the\n                <code>$normalizationForm</code> argument is not one of the values supported by the\n             implementation.</p></div>\n", "summary" : "<p>  Returns the value of  $arg  after applying Unicode\n             normalization.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "arg", "type" : "xs:string", "occurrence" : "?", "description" : "" } ], "returns" : { "type" : "xs:string", "description" : "" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 2, "name" : "normalize-unicode", "qname" : "fn:normalize-unicode", "signature" : "($arg as xs:string?, $normalizationForm as xs:string) as xs:string external", "description" : " <div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">Returns the value of <code>$arg</code> after applying Unicode\n             normalization.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\"><example role=\"signature\"><proto name=\"normalize-unicode\" return-type=\"xs:string\" isOp=\"no\" prefix=\"fn\" returnEmptyOk=\"no\" returnSeq=\"no\" returnVaries=\"no\" isSchema=\"no\" isDatatype=\"no\" isSpecial=\"no\"><arg name=\"arg\" type=\"xs:string?\"/></proto></example><example role=\"signature\"><proto name=\"normalize-unicode\" return-type=\"xs:string\" isOp=\"no\" prefix=\"fn\" returnEmptyOk=\"no\" returnSeq=\"no\" returnVaries=\"no\" isSchema=\"no\" isDatatype=\"no\" isSpecial=\"no\"><arg name=\"arg\" type=\"xs:string?\"/><arg name=\"normalizationForm\" type=\"xs:string\"/></proto></example></p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">This function is <termref def=\"dt-deterministic\">deterministic</termref>, <termref def=\"dt-context-independent\">context-independent</termref>,  and <termref def=\"dt-focus-independent\">focus-independent</termref>. </p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">If the value of <code>$arg</code> is the empty sequence, the function returns the\n             zero-length string.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">If the single-argument version of the function is used, the result is the same as\n             calling the two-argument version with <code>$normalizationForm</code> set to the string\n             \"NFC\".</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">Otherwise, the function returns the value of <code>$arg</code> normalized according to\n             the rules of the normalization form identified by the value of\n                <code>$normalizationForm</code>.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The effective value of <code>$normalizationForm</code> is the value of the expression\n                <code>fn:upper-case(fn:normalize-space($normalizationForm))</code>.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\" diff=\"del\" at=\"D\">See <bibref ref=\"charmod-normalization\"/> for a description of the\n             normalization forms.</p><ulist xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\"><item><p>If the effective value of <code>$normalizationForm</code> is <quote>NFC</quote>,\n                   then the function returns the value of <code>$arg</code> converted to Unicode\n                   Normalization Form C (NFC).</p></item><item><p>If the effective value of <code>$normalizationForm</code> is <quote>NFD</quote>,\n                   then the function returns the value of <code>$arg</code> converted to Unicode\n                   Normalization Form D (NFD).</p></item><item><p>If the effective value of <code>$normalizationForm</code> is <quote>NFKC</quote>,\n                   then the function returns the value of <code>$arg</code> in Unicode Normalization\n                   Form KC (NFKC).</p></item><item><p>If the effective value of <code>$normalizationForm</code> is <quote>NFKD</quote>,\n                   then the function returns the value of <code>$arg</code> converted to Unicode\n                   Normalization Form KD (NFKD).</p></item><item><p>If the effective value of <code>$normalizationForm</code> is\n                      <quote>FULLY-NORMALIZED</quote>, then the function returns the value of\n                      <code>$arg</code> converted to fully normalized form. </p></item><item><p>If the effective value of <code>$normalizationForm</code> is the zero-length\n                   string, no normalization is performed and <code>$arg</code> is returned.</p></item></ulist><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\" diff=\"add\" at=\"D\">Normalization forms NFC, NFD, NFKC, and NFKD, and the algorithms to be\n             used for converting a string to each of these forms, are defined in <bibref ref=\"Unicode-Normalization\"/>.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\" diff=\"add\" at=\"D\">The motivation for normalization form FULLY-NORMALIZED is explained in\n                <bibref ref=\"charmod-normalization\"/>. However, as that specification did not\n             progress beyond working draft status, the normative specification is as follows:</p><ulist xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\" diff=\"add\" at=\"D\"><item><p>A string is <term>fully-normalized</term> if (a) it is in normalization form NFC\n                   as defined in <bibref ref=\"Unicode-Normalization\"/>, and (b) it does not start\n                   with a composing character.</p></item><item><p>A composing character is a character that is one or both of the following:</p><ulist><item><p>the second character in the canonical decomposition mapping of some\n                         character that is not listed in the Composition Exclusion Table defined in\n                            <bibref ref=\"Unicode-Normalization\"/>;</p></item><item><p>of non-zero canonical combining class (as defined in <bibref ref=\"Unicode\"/>).</p></item></ulist></item><item><p>A string is converted to FULLY-NORMALIZED form as follows:</p><ulist><item><p>if the first character in the string is a composing character, prepend a\n                         single space (x20);</p></item><item><p>convert the resulting string to normalization form NFC.</p></item></ulist></item></ulist><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\" diff=\"add\" At=\"L\">Conforming implementations <rfc2119>must</rfc2119> support normalization form \"NFC\" and\n                <rfc2119>may</rfc2119> support normalization forms \"NFD\", \"NFKC\", \"NFKD\", and\n             \"FULLY-NORMALIZED\". They <rfc2119>may</rfc2119> also support other normalization forms\n             with <termref def=\"implementation-defined\">implementation-defined</termref> semantics. </p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\" diff=\"add\" at=\"L\">It is <termref def=\"dt-implementation-defined\">implementation-defined</termref> which version of\n             Unicode (and therefore, of the normalization algorithms and their underlying\n             data) is supported by the implementation. See <bibref ref=\"Unicode-Normalization\"/> for details of the\n             stability policy regarding changes to the normalization rules in future\n             versions of Unicode. If the input string contains codepoints that are\n             unassigned in the relevant version of Unicode, or for which no normalization\n             rules are defined, the <code>fn:normalize-unicode</code> function leaves such codepoints\n             unchanged. If the implementation supports the requested normalization form then\n             it <rfc2119>must</rfc2119> be able to handle every input string without raising an error.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">A <phrase diff=\"add\" at=\"L\">dynamic</phrase> error is raised <errorref class=\"CH\" code=\"0003\"/> if the effective value of the\n                <code>$normalizationForm</code> argument is not one of the values supported by the\n             implementation.</p></div>\n", "summary" : "<p>  Returns the value of  $arg  after applying Unicode\n             normalization.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "arg", "type" : "xs:string", "occurrence" : "?", "description" : "" }, { "name" : "normalizationForm", "type" : "xs:string", "occurrence" : null, "description" : "" } ], "returns" : { "type" : "xs:string", "description" : "" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "not", "qname" : "fn:not", "signature" : "($arg as item()*) as xs:boolean external", "description" : " <div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">Returns <code>true</code> if the effective boolean value of <code>$arg</code>\n             is <code>false</code>, or <code>false</code> if it is <code>true</code>.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\"><example role=\"signature\"><proto name=\"not\" return-type=\"xs:boolean\" isOp=\"no\" prefix=\"fn\" returnEmptyOk=\"no\" returnSeq=\"no\" returnVaries=\"no\" isSchema=\"no\" isDatatype=\"no\" isSpecial=\"no\"><arg name=\"arg\" type=\"item()*\"/></proto></example></p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">This function is <termref def=\"dt-deterministic\">deterministic</termref>, <termref def=\"dt-context-independent\">context-independent</termref>,  and <termref def=\"dt-focus-independent\">focus-independent</termref>. </p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The value of <code>$arg</code> is first reduced to an effective boolean value by\n             applying the <code>fn:boolean()</code> function. The function returns <code>true</code>\n             if the effective boolean value is <code>false</code>, or <code>false</code> if the\n             effective boolean value is <code>true</code>. </p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The expression <code>fn:not(fn:true())</code> returns <code>false()</code>.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The expression <code>fn:not(\"false\")</code> returns <code>false()</code>.</p></div>\n", "summary" : "<p>  Returns  true  if the effective boolean value of  $arg \n             is  false , or  false  if it is  true .</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "arg", "type" : "item()", "occurrence" : "*", "description" : "" } ], "returns" : { "type" : "xs:boolean", "description" : "" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 0, "name" : "number", "qname" : "fn:number", "signature" : "() as xs:double external", "description" : " <div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">Returns the value indicated by <code>$arg</code> or, if <code>$arg</code> is\n             not specified, the context item after atomization, converted to an\n                <code>xs:double</code>. </p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\"><example role=\"signature\"><proto name=\"number\" return-type=\"xs:double\" isOp=\"no\" prefix=\"fn\" returnEmptyOk=\"no\" returnSeq=\"no\" returnVaries=\"no\" isSchema=\"no\" isDatatype=\"no\" isSpecial=\"no\"/></example><example role=\"signature\"><proto name=\"number\" return-type=\"xs:double\" isOp=\"no\" prefix=\"fn\" returnEmptyOk=\"no\" returnSeq=\"no\" returnVaries=\"no\" isSchema=\"no\" isDatatype=\"no\" isSpecial=\"no\"><arg name=\"arg\" type=\"xs:anyAtomicType?\"/></proto></example></p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The zero-argument form of this function is <termref def=\"dt-deterministic\">deterministic</termref>, <termref def=\"dt-context-dependent\">context-dependent</termref>,  and <termref def=\"dt-focus-dependent\">focus-dependent</termref>. </p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The one-argument form of this function is <termref def=\"dt-deterministic\">deterministic</termref>, <termref def=\"dt-context-independent\">context-independent</termref>,  and <termref def=\"dt-focus-independent\">focus-independent</termref>. </p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">Calling the zero-argument version of the function is defined to give the same result as\n             calling the single-argument version with the context item (<code>.</code>). That is,\n                <code>fn:number()</code> is equivalent to <code>fn:number(.)</code>, as\n          defined by the rules that follow.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">If <code>$arg</code> is the empty sequence or if <code>$arg</code> <phrase diff=\"del\" at=\"L\">or the context item</phrase>\n             cannot be converted to an <code>xs:double</code>, the <code>xs:double</code> value\n                <code>NaN</code> is returned. </p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">Otherwise, <code>$arg</code> <phrase diff=\"del\" at=\"L\">, or the context item after atomization,</phrase> is converted to an\n                <code>xs:double</code> following the rules of <specref ref=\"casting-to-double\"/>. If\n             the conversion to <code>xs:double</code> fails, the <code>xs:double</code> value\n                <code>NaN</code> is returned.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">A <phrase diff=\"add\" at=\"M\">dynamic</phrase> error is raised <xerrorref spec=\"XP\" class=\"DY\" code=\"0002\" type=\"dynamic\"/> if\n                <code>$arg</code> is omitted and the context item is <xtermref ref=\"dt-absent\" spec=\"DM30\">absent</xtermref>.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\" diff=\"add\" at=\"L\">As a consequence of the rules given above, a type error occurs if the context\n          item cannot be atomized, or if the result of atomizing the context item is a sequence containing\n          more than one atomic value.</p></div>\n", "summary" : "<p>  Returns the value indicated by  $arg  or, if  $arg  is\n             not specified, the context item after atomization, converted to an\n                 xs:double .</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [  ], "returns" : { "type" : "xs:double", "description" : "" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "number", "qname" : "fn:number", "signature" : "($arg as xs:anyAtomicType?) as xs:double external", "description" : " <div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">Returns the value indicated by <code>$arg</code> or, if <code>$arg</code> is\n             not specified, the context item after atomization, converted to an\n                <code>xs:double</code>. </p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\"><example role=\"signature\"><proto name=\"number\" return-type=\"xs:double\" isOp=\"no\" prefix=\"fn\" returnEmptyOk=\"no\" returnSeq=\"no\" returnVaries=\"no\" isSchema=\"no\" isDatatype=\"no\" isSpecial=\"no\"/></example><example role=\"signature\"><proto name=\"number\" return-type=\"xs:double\" isOp=\"no\" prefix=\"fn\" returnEmptyOk=\"no\" returnSeq=\"no\" returnVaries=\"no\" isSchema=\"no\" isDatatype=\"no\" isSpecial=\"no\"><arg name=\"arg\" type=\"xs:anyAtomicType?\"/></proto></example></p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The zero-argument form of this function is <termref def=\"dt-deterministic\">deterministic</termref>, <termref def=\"dt-context-dependent\">context-dependent</termref>,  and <termref def=\"dt-focus-dependent\">focus-dependent</termref>. </p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The one-argument form of this function is <termref def=\"dt-deterministic\">deterministic</termref>, <termref def=\"dt-context-independent\">context-independent</termref>,  and <termref def=\"dt-focus-independent\">focus-independent</termref>. </p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">Calling the zero-argument version of the function is defined to give the same result as\n             calling the single-argument version with the context item (<code>.</code>). That is,\n                <code>fn:number()</code> is equivalent to <code>fn:number(.)</code>, as\n          defined by the rules that follow.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">If <code>$arg</code> is the empty sequence or if <code>$arg</code> <phrase diff=\"del\" at=\"L\">or the context item</phrase>\n             cannot be converted to an <code>xs:double</code>, the <code>xs:double</code> value\n                <code>NaN</code> is returned. </p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">Otherwise, <code>$arg</code> <phrase diff=\"del\" at=\"L\">, or the context item after atomization,</phrase> is converted to an\n                <code>xs:double</code> following the rules of <specref ref=\"casting-to-double\"/>. If\n             the conversion to <code>xs:double</code> fails, the <code>xs:double</code> value\n                <code>NaN</code> is returned.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">A <phrase diff=\"add\" at=\"M\">dynamic</phrase> error is raised <xerrorref spec=\"XP\" class=\"DY\" code=\"0002\" type=\"dynamic\"/> if\n                <code>$arg</code> is omitted and the context item is <xtermref ref=\"dt-absent\" spec=\"DM30\">absent</xtermref>.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\" diff=\"add\" at=\"L\">As a consequence of the rules given above, a type error occurs if the context\n          item cannot be atomized, or if the result of atomizing the context item is a sequence containing\n          more than one atomic value.</p></div>\n", "summary" : "<p>  Returns the value indicated by  $arg  or, if  $arg  is\n             not specified, the context item after atomization, converted to an\n                 xs:double .</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "arg", "type" : "xs:anyAtomicType", "occurrence" : "?", "description" : "" } ], "returns" : { "type" : "xs:double", "description" : "" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "one-or-more", "qname" : "fn:one-or-more", "signature" : "($arg as item()*) as item()+ external", "description" : " <div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">Returns <code>$arg</code> if it contains one or more items. Otherwise, raises\n             an error. </p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\"><example role=\"signature\"><proto name=\"one-or-more\" return-type=\"item()+\" isOp=\"no\" prefix=\"fn\" returnEmptyOk=\"no\" returnSeq=\"no\" returnVaries=\"no\" isSchema=\"no\" isDatatype=\"no\" isSpecial=\"no\"><arg name=\"arg\" type=\"item()*\"/></proto></example></p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">This function is <termref def=\"dt-deterministic\">deterministic</termref>, <termref def=\"dt-context-independent\">context-independent</termref>,  and <termref def=\"dt-focus-independent\">focus-independent</termref>. </p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">Except in error cases, the function returns <code>$arg</code> unchanged.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\" diff=\"del\" at=\"F\">For detailed type semantics, see [Formal Semantics].</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">A <phrase diff=\"add\" at=\"L\">dynamic</phrase> error is raised <errorref class=\"RG\" code=\"0004\"/> if <code>$arg</code> is an empty\n             sequence.</p></div>\n", "summary" : "<p>  Returns  $arg  if it contains one or more items.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "arg", "type" : "item()", "occurrence" : "*", "description" : "" } ], "returns" : { "type" : "item()+", "description" : "" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "outermost", "qname" : "fn:outermost", "signature" : "($nodes as node()*) as node()* external", "description" : " <div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">Returns every node within the input sequence that has no ancestor that is itself a\n             member of the input sequence; the nodes are returned in document order with duplicates\n             eliminated.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\"><example role=\"signature\"><proto name=\"outermost\" return-type=\"node()*\" isOp=\"no\" prefix=\"fn\" returnEmptyOk=\"no\" returnSeq=\"no\" returnVaries=\"no\" isSchema=\"no\" isDatatype=\"no\" isSpecial=\"no\"><arg name=\"nodes\" type=\"node()*\"/></proto></example></p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">This function is <termref def=\"dt-deterministic\">deterministic</termref>, <termref def=\"dt-context-independent\">context-independent</termref>,  and <termref def=\"dt-focus-independent\">focus-independent</termref>. </p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The effect of the function call <code>fn:outermost($nodes)</code> is defined to be\n             equivalent to the result of the expression <code diff=\"chg\" at=\"L\">$nodes[not(ancestor::node() intersect\n                $nodes)]/.</code>.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">That is, the function takes as input a sequence of nodes, and returns every node within\n             the sequence that <phrase diff=\"chg\" at=\"J\">does not have another node within the sequence as an ancestor</phrase>; the nodes are\n             returned in document order with duplicates eliminated.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The formulation <code>$nodes except $nodes/descendant::node()</code> might appear to be\n             simpler, but does not correctly account for attribute nodes, as these are not\n             descendants of their parent element.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The motivation for the function was based on XSLT streaming use cases. There are cases\n             where the <bibref ref=\"xslt-30\"/> streaming rules allow the construct\n                <code>outermost(//section)</code> but do not allow <code>//section</code>; the\n             function can therefore be useful in cases where it is known that sections will not be\n             nested, as well as cases where the application actually wishes to process all sections\n             except those that are nested within another.</p></div>\n", "summary" : "<p>  Returns every node within the input sequence that has no ancestor that is itself a\n             member of the input sequence; the nodes are returned in document order with duplicates\n             eliminated.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "nodes", "type" : "node()", "occurrence" : "*", "description" : "" } ], "returns" : { "type" : "node()*", "description" : "" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "parse-xml-fragment", "qname" : "fn:parse-xml-fragment", "signature" : "($arg as xs:string?) as document(element(*,xs:untyped)) external", "description" : " <div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">This function takes as input an XML external entity represented as a string,\n             and returns the document node at the root of an XDM tree representing the parsed\n             document fragment.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\"><example role=\"signature\"><proto name=\"parse-xml-fragment\" return-type=\"document-node()?\" isOp=\"no\" prefix=\"fn\" returnEmptyOk=\"no\" returnSeq=\"no\" returnVaries=\"no\" isSchema=\"no\" isDatatype=\"no\" isSpecial=\"no\"><arg name=\"arg\" type=\"xs:string?\"/></proto></example></p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">This function is <termref def=\"dt-nondeterministic\">nondeterministic</termref>, <termref def=\"dt-context-dependent\">context-dependent</termref>,  and <termref def=\"dt-focus-independent\">focus-independent</termref>.  It depends on\n \t\tstatic base uri.\n \t</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">If <code>$arg</code> is the empty sequence, the function returns the empty sequence.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The input must be a namespace-well-formed external general parsed entity. More specifically,\n             it must be a string conforming to the production rule <xnt xmlns:xlink=\"http://www.w3.org/1999/xlink\" spec=\"xml\" ref=\"NT-extParsedEnt\" xlink:type=\"simple\">extParsedEnt</xnt> in <bibref ref=\"REC-xml\"/>, it must contain no entity references other\n             than references to predefined entities,\n             and it must satisfy all the rules\n             of <bibref ref=\"REC-xml-names\"/> for namespace-well-formed documents with the exception\n             that the rule requiring it to be a well-formed document is replaced by the rule requiring\n             it to be a well-formed external general parsed entity.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The string is parsed to form a sequence of nodes which\n             become children of the new document node, in the same way as the content of any element\n             is converted into a sequence of children for the resulting element node.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">Schema validation is <emph>not</emph> invoked, which means that the nodes in the\n             returned document will all be untyped.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The precise process used to construct the XDM instance is <termref def=\"implementation-defined\"/>. In\n             particular, it is implementation-defined whether an XML 1.0 or XML 1.1 parser is\n             used.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\" diff=\"chg\" at=\"L\">The Static Base URI from the static context of the <code>fn:parse-xml-fragment</code> function call\n             is used as the base URI of the document node\n             that is returned.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The document URI of the returned node is <termref def=\"absent\">absent</termref>.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The function is <emph>not</emph>\n             <termref def=\"deterministic\">deterministic</termref>: that is, if the function is called\n             twice with the same arguments, it is <termref def=\"implementation-dependent\">implementation-dependent</termref> whether the same node is returned on both\n                occasions.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">A <phrase diff=\"add\" at=\"L\">dynamic</phrase> error is raised <errorref class=\"DC\" code=\"0006\"/> if the content of\n                <code>$arg</code> is not a well-formed external general parsed entity,\n             if it contains entity references other than references to predefined entities, or if a document that\n             incorporates this well-formed parsed entity would not be namespace-well-formed.</p></div>\n", "summary" : "<p>  This function takes as input an XML external entity represented as a string,\n             and returns the document node at the root of an XDM tree representing the parsed\n             document fragment.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "arg", "type" : "xs:string", "occurrence" : "?", "description" : "" } ], "returns" : { "type" : "document(element(*,xs:untyped))", "description" : "" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "parse-xml", "qname" : "fn:parse-xml", "signature" : "($arg as xs:string?) as document(element(*,xs:untyped)) external", "description" : " <div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">This function takes as input an XML document represented as a string, and\n             returns the document node at the root of an XDM tree representing the parsed\n             document.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\"><example role=\"signature\"><proto name=\"parse-xml\" return-type=\"document-node(element(*))?\" isOp=\"no\" prefix=\"fn\" returnEmptyOk=\"no\" returnSeq=\"no\" returnVaries=\"no\" isSchema=\"no\" isDatatype=\"no\" isSpecial=\"no\"><arg name=\"arg\" type=\"xs:string?\"/></proto></example></p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">This function is <termref def=\"dt-nondeterministic\">nondeterministic</termref>, <termref def=\"dt-context-dependent\">context-dependent</termref>,  and <termref def=\"dt-focus-independent\">focus-independent</termref>.  It depends on\n \t\tstatic base uri.\n \t</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\" diff=\"add\" at=\"G\">If <code>$arg</code> is the empty sequence, the function returns the\n             empty sequence.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The precise process used to construct the XDM instance is <termref def=\"implementation-defined\"/>. In\n             particular, it is implementation-defined whether DTD and/or schema validation is invoked, and it is\n             implementation-defined whether an XML 1.0 or XML 1.1 parser is used.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The Static Base URI property from the static context of the\n             <code>fn:parse-xml</code> function call is used both as the base URI used by the XML parser to resolve\n             relative entity references within the document, and as the base URI of the document node\n             that is returned.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The document URI of the returned node is <termref def=\"absent\">absent</termref>.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The function is <emph>not</emph>\n             <termref def=\"deterministic\">deterministic</termref>: that is, if the function is called\n             twice with the same arguments, it is <termref def=\"implementation-dependent\">implementation-dependent</termref> whether the same node is returned on both\n             occasions.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">A <phrase diff=\"add\" at=\"L\">dynamic</phrase> error is raised <errorref class=\"DC\" code=\"0006\"/> if the content of\n                <code>$arg</code> is not a well-formed and namespace-well-formed XML document.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\" diff=\"add\" at=\"G\">A <phrase diff=\"add\" at=\"L\">dynamic</phrase> error is raised <errorref class=\"DC\" code=\"0006\"/> if DTD-based\n             validation is carried out and the content of <code>$arg</code> is not valid against its\n             DTD.</p></div>\n", "summary" : "<p>  This function takes as input an XML document represented as a string, and\n             returns the document node at the root of an XDM tree representing the parsed\n             document.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "arg", "type" : "xs:string", "occurrence" : "?", "description" : "" } ], "returns" : { "type" : "document(element(*,xs:untyped))", "description" : "" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 2, "name" : "parse-xml", "qname" : "fn:parse-xml", "signature" : "($arg as xs:string?, $baseURI as xs:string) as document(element(*,xs:untyped)) external", "description" : " <div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">This function takes as input an XML document represented as a string, and\n             returns the document node at the root of an XDM tree representing the parsed\n             document.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\"><example role=\"signature\"><proto name=\"parse-xml\" return-type=\"document-node(element(*))?\" isOp=\"no\" prefix=\"fn\" returnEmptyOk=\"no\" returnSeq=\"no\" returnVaries=\"no\" isSchema=\"no\" isDatatype=\"no\" isSpecial=\"no\"><arg name=\"arg\" type=\"xs:string?\"/></proto></example></p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">This function is <termref def=\"dt-nondeterministic\">nondeterministic</termref>, <termref def=\"dt-context-dependent\">context-dependent</termref>,  and <termref def=\"dt-focus-independent\">focus-independent</termref>.  It depends on\n \t\tstatic base uri.\n \t</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\" diff=\"add\" at=\"G\">If <code>$arg</code> is the empty sequence, the function returns the\n             empty sequence.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The precise process used to construct the XDM instance is <termref def=\"implementation-defined\"/>. In\n             particular, it is implementation-defined whether DTD and/or schema validation is invoked, and it is\n             implementation-defined whether an XML 1.0 or XML 1.1 parser is used.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The Static Base URI property from the static context of the\n             <code>fn:parse-xml</code> function call is used both as the base URI used by the XML parser to resolve\n             relative entity references within the document, and as the base URI of the document node\n             that is returned.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The document URI of the returned node is <termref def=\"absent\">absent</termref>.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The function is <emph>not</emph>\n             <termref def=\"deterministic\">deterministic</termref>: that is, if the function is called\n             twice with the same arguments, it is <termref def=\"implementation-dependent\">implementation-dependent</termref> whether the same node is returned on both\n             occasions.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">A <phrase diff=\"add\" at=\"L\">dynamic</phrase> error is raised <errorref class=\"DC\" code=\"0006\"/> if the content of\n                <code>$arg</code> is not a well-formed and namespace-well-formed XML document.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\" diff=\"add\" at=\"G\">A <phrase diff=\"add\" at=\"L\">dynamic</phrase> error is raised <errorref class=\"DC\" code=\"0006\"/> if DTD-based\n             validation is carried out and the content of <code>$arg</code> is not valid against its\n             DTD.</p></div>\n", "summary" : "<p>  This function takes as input an XML document represented as a string, and\n             returns the document node at the root of an XDM tree representing the parsed\n             document.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "arg", "type" : "xs:string", "occurrence" : "?", "description" : "" }, { "name" : "baseURI", "type" : "xs:string", "occurrence" : null, "description" : "" } ], "returns" : { "type" : "document(element(*,xs:untyped))", "description" : "" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 0, "name" : "position", "qname" : "fn:position", "signature" : "() as xs:integer external", "description" : " <div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">Returns the context position from the dynamic context.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\"><example role=\"signature\"><proto name=\"position\" return-type=\"xs:integer\" isOp=\"no\" prefix=\"fn\" returnEmptyOk=\"no\" returnSeq=\"no\" returnVaries=\"no\" isSchema=\"no\" isDatatype=\"no\" isSpecial=\"no\"/></example></p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">This function is <termref def=\"dt-deterministic\">deterministic</termref>, <termref def=\"dt-context-dependent\">context-dependent</termref>,  and <termref def=\"dt-focus-dependent\">focus-dependent</termref>. </p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">Returns the context position from the dynamic context. (See <xspecref spec=\"XP30\" ref=\"id-xp-evaluation-context-components\"/>.)</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">A <phrase diff=\"add\" at=\"M\">dynamic</phrase> error is raised <xerrorref spec=\"XP\" class=\"DY\" code=\"0002\" type=\"type\"/> if the\n             context item is <xtermref ref=\"dt-absent\" spec=\"DM30\">absent</xtermref>.</p></div>\n", "summary" : "<p>  Returns the context position from the dynamic context.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [  ], "returns" : { "type" : "xs:integer", "description" : "" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "prefix-from-QName", "qname" : "fn:prefix-from-QName", "signature" : "($arg as xs:QName?) as xs:NCName? external", "description" : " <div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">Returns the prefix component of the supplied QName.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\"><example role=\"signature\"><proto name=\"prefix-from-QName\" return-type=\"xs:NCName?\" isOp=\"no\" prefix=\"fn\" returnEmptyOk=\"no\" returnSeq=\"no\" returnVaries=\"no\" isSchema=\"no\" isDatatype=\"no\" isSpecial=\"no\"><arg name=\"arg\" type=\"xs:QName?\"/></proto></example></p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">This function is <termref def=\"dt-deterministic\">deterministic</termref>, <termref def=\"dt-context-independent\">context-independent</termref>,  and <termref def=\"dt-focus-independent\">focus-independent</termref>. </p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">If <code>$arg</code> is the empty sequence the function returns the empty sequence.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">If <code>$arg</code> has no prefix component the function returns the empty\n             sequence.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">Otherwise, the function returns an <code>xs:NCName</code> representing the prefix\n             component of <code>$arg</code>.</p></div>\n", "summary" : "<p>  Returns the prefix component of the supplied QName.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "arg", "type" : "xs:QName", "occurrence" : "?", "description" : "" } ], "returns" : { "type" : "xs:NCName?", "description" : "" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 2, "name" : "remove", "qname" : "fn:remove", "signature" : "($target as item()*, $position as xs:integer) as item()* external", "description" : " <div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">Returns a new sequence containing all the items of <code>$target</code> except\n             the item at position <code>$position</code>.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\"><example role=\"signature\"><proto name=\"remove\" return-type=\"item()*\" isOp=\"no\" prefix=\"fn\" returnEmptyOk=\"no\" returnSeq=\"no\" returnVaries=\"no\" isSchema=\"no\" isDatatype=\"no\" isSpecial=\"no\"><arg name=\"target\" type=\"item()*\"/><arg name=\"position\" type=\"xs:integer\"/></proto></example></p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">This function is <termref def=\"dt-deterministic\">deterministic</termref>, <termref def=\"dt-context-independent\">context-independent</termref>,  and <termref def=\"dt-focus-independent\">focus-independent</termref>. </p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The function returns a sequence consisting of all items of <code>$target</code> whose\n             index is less than <code>$position</code>, followed by all items of <code>$target</code>\n             whose index is greater than <code>$position</code>. </p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\" diff=\"del\" at=\"F\">For detailed type semantics, see [Formal Semantics].</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">If <code>$position</code> is less than 1 or greater than the number of items in\n                <code>$target</code>, <code>$target</code> is returned.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">If <code>$target</code> is the empty sequence, the empty sequence is returned.</p></div>\n", "summary" : "<p>  Returns a new sequence containing all the items of  $target  except\n             the item at position  $position .</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "target", "type" : "item()", "occurrence" : "*", "description" : "" }, { "name" : "position", "type" : "xs:integer", "occurrence" : null, "description" : "" } ], "returns" : { "type" : "item()*", "description" : "" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 3, "name" : "replace", "qname" : "fn:replace", "signature" : "($input as xs:string?, $pattern as xs:string, $replacement as xs:string) as xs:string external", "description" : " <div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">Returns a string produced from the input string by replacing any substrings\n             that match a given regular expression with a supplied replacement string.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\"><example role=\"signature\"><proto name=\"replace\" return-type=\"xs:string\" isOp=\"no\" prefix=\"fn\" returnEmptyOk=\"no\" returnSeq=\"no\" returnVaries=\"no\" isSchema=\"no\" isDatatype=\"no\" isSpecial=\"no\"><arg name=\"input\" type=\"xs:string?\"/><arg name=\"pattern\" type=\"xs:string\"/><arg name=\"replacement\" type=\"xs:string\"/></proto></example><example role=\"signature\"><proto name=\"replace\" return-type=\"xs:string\" isOp=\"no\" prefix=\"fn\" returnEmptyOk=\"no\" returnSeq=\"no\" returnVaries=\"no\" isSchema=\"no\" isDatatype=\"no\" isSpecial=\"no\"><arg name=\"input\" type=\"xs:string?\"/><arg name=\"pattern\" type=\"xs:string\"/><arg name=\"replacement\" type=\"xs:string\"/><arg name=\"flags\" type=\"xs:string\"/></proto></example></p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">This function is <termref def=\"dt-deterministic\">deterministic</termref>, <termref def=\"dt-context-independent\">context-independent</termref>,  and <termref def=\"dt-focus-independent\">focus-independent</termref>. </p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The effect of calling the first version of this function (omitting the argument\n                <code>$flags</code>) is the same as the effect of calling the second version with the\n                <code>$flags</code> argument set to a zero-length string. Flags are defined in\n                <specref ref=\"flags\"/>.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The <code>$flags</code> argument is interpreted in the same manner as for the\n                <code>fn:matches</code> function. </p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">If <code>$input</code> is the empty sequence, it is interpreted as the zero-length\n             string.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The function returns the <code>xs:string</code> that is obtained by replacing each\n             non-overlapping substring of <code>$input</code> that matches the given\n                <code>$pattern</code> with an occurrence of the <code>$replacement</code> string.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">If two overlapping substrings of <code>$input</code> both match the\n                <code>$pattern</code>, then only the first one (that is, the one whose first <termref def=\"character\">character</termref> comes first in the <code>$input</code> string) is\n             replaced.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\" diff=\"add\" at=\"B\">If the <code>q</code> flag is present, the replacement string is used\n                <emph>as is</emph>.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\"><phrase diff=\"add\" at=\"B\">Otherwise,</phrase> within the <code>$replacement</code>\n             string, a variable <code>$N</code> may be used to refer to the substring captured by the\n             Nth parenthesized sub-expression in the regular expression. For each match of the\n             pattern, these variables are assigned the value of the content matched by the relevant\n             sub-expression, and the modified replacement string is then substituted for the <termref def=\"character\">characters</termref> in <code>$input</code> that matched the pattern.\n                <code>$0</code> refers to the substring captured by the regular expression as a\n             whole.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">More specifically, the rules are as follows, where <code>S</code> is the number of\n             parenthesized sub-expressions in the regular expression, and <code>N</code> is the\n             decimal number formed by taking all the digits that consecutively follow the\n                <code>$</code> character:</p><olist xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\"><item><p>If <code>N</code>=<code>0</code>, then the variable is replaced by the substring\n                   matched by the regular expression as a whole.</p></item><item><p>If <code>1</code>&lt;=<code>N</code>&lt;=<code>S</code>, then the variable is\n                   replaced by the substring captured by the Nth parenthesized sub-expression. If the\n                      <code>Nth</code> parenthesized sub-expression was not matched, then the\n                   variable is replaced by the zero-length string.</p></item><item><p>If <code>S</code>&lt;<code>N</code>&lt;=<code>9</code>, then the variable is\n                   replaced by the zero-length string.</p></item><item><p>Otherwise (if <code>N</code>&gt;<code>S</code> and\n                      <code>N</code>&gt;<code>9</code>), the last digit of <code>N</code> is taken to\n                   be a literal character to be included \"as is\" in the replacement string, and the\n                   rules are reapplied using the number <code>N</code> formed by stripping off this\n                   last digit.</p></item></olist><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">For example, if the replacement string is <quote>\n                <code>$23</code>\n             </quote> and there are 5 substrings, the result contains the value of the substring that\n             matches the second sub-expression, followed by the digit <quote>\n                <code>3</code>\n             </quote>.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\" diff=\"chg\" at=\"B\">Unless the <code>q</code> flag is used, a literal <code>$</code>\n             character within the replacement string must be written as <code>\\$</code>, and a\n             literal <code>\\</code> character must be written as <code>\\\\</code>.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\"> If two alternatives within the pattern both match at the same position in the\n                <code>$input</code>, then the match that is chosen is the one matched by the first\n             alternative. For example:</p><eg xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\" xml:space=\"preserve\"> fn:replace(\"abcd\", \"(ab)|(a)\", \"[1=$1][2=$2]\") returns \"[1=ab][2=]cd\"</eg><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">A <phrase diff=\"add\" at=\"L\">dynamic</phrase> error is raised <errorref class=\"RX\" code=\"0002\"/> if the value of\n                <code>$pattern</code> is invalid according to the rules described in section <specref ref=\"regex-syntax\"/>. </p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">A <phrase diff=\"add\" at=\"L\">dynamic</phrase> error is raised <errorref class=\"RX\" code=\"0001\"/> if the value of\n                <code>$flags</code> is invalid according to the rules described in section <specref ref=\"regex-syntax\"/>. </p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">A <phrase diff=\"add\" at=\"L\">dynamic</phrase> error is raised <errorref class=\"RX\" code=\"0003\"/> if the pattern matches a\n             zero-length string, that is, if the expression <code>fn:matches(\"\", $pattern,\n                $flags)</code> returns <code>true</code>. It is not an error, however, if a captured\n             substring is zero-length.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">A <phrase diff=\"add\" at=\"L\">dynamic</phrase> error is raised <errorref class=\"RX\" code=\"0004\"/> if the value of\n                <code>$replacement</code> contains a \"<code>$</code>\" character that is not\n             immediately followed by a digit <code>0-9</code> and not immediately preceded by a\n             \"\\\".</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">A <phrase diff=\"add\" at=\"L\">dynamic</phrase> error is raised <errorref class=\"RX\" code=\"0004\"/> if the value of\n                <code>$replacement</code> contains a \"<code>\\</code>\" character that is not part of a\n                \"<code>\\\\</code>\" pair, unless it is immediately followed by a \"<code>$</code>\"\n             character.</p></div>\n", "summary" : "<p>  Returns a string produced from the input string by replacing any substrings\n             that match a given regular expression with a supplied replacement string.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "input", "type" : "xs:string", "occurrence" : "?", "description" : "" }, { "name" : "pattern", "type" : "xs:string", "occurrence" : null, "description" : "" }, { "name" : "replacement", "type" : "xs:string", "occurrence" : null, "description" : "" } ], "returns" : { "type" : "xs:string", "description" : "" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 4, "name" : "replace", "qname" : "fn:replace", "signature" : "($input as xs:string?, $pattern as xs:string, $replacement as xs:string, $flags as xs:string) as xs:string external", "description" : " <div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">Returns a string produced from the input string by replacing any substrings\n             that match a given regular expression with a supplied replacement string.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\"><example role=\"signature\"><proto name=\"replace\" return-type=\"xs:string\" isOp=\"no\" prefix=\"fn\" returnEmptyOk=\"no\" returnSeq=\"no\" returnVaries=\"no\" isSchema=\"no\" isDatatype=\"no\" isSpecial=\"no\"><arg name=\"input\" type=\"xs:string?\"/><arg name=\"pattern\" type=\"xs:string\"/><arg name=\"replacement\" type=\"xs:string\"/></proto></example><example role=\"signature\"><proto name=\"replace\" return-type=\"xs:string\" isOp=\"no\" prefix=\"fn\" returnEmptyOk=\"no\" returnSeq=\"no\" returnVaries=\"no\" isSchema=\"no\" isDatatype=\"no\" isSpecial=\"no\"><arg name=\"input\" type=\"xs:string?\"/><arg name=\"pattern\" type=\"xs:string\"/><arg name=\"replacement\" type=\"xs:string\"/><arg name=\"flags\" type=\"xs:string\"/></proto></example></p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">This function is <termref def=\"dt-deterministic\">deterministic</termref>, <termref def=\"dt-context-independent\">context-independent</termref>,  and <termref def=\"dt-focus-independent\">focus-independent</termref>. </p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The effect of calling the first version of this function (omitting the argument\n                <code>$flags</code>) is the same as the effect of calling the second version with the\n                <code>$flags</code> argument set to a zero-length string. Flags are defined in\n                <specref ref=\"flags\"/>.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The <code>$flags</code> argument is interpreted in the same manner as for the\n                <code>fn:matches</code> function. </p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">If <code>$input</code> is the empty sequence, it is interpreted as the zero-length\n             string.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The function returns the <code>xs:string</code> that is obtained by replacing each\n             non-overlapping substring of <code>$input</code> that matches the given\n                <code>$pattern</code> with an occurrence of the <code>$replacement</code> string.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">If two overlapping substrings of <code>$input</code> both match the\n                <code>$pattern</code>, then only the first one (that is, the one whose first <termref def=\"character\">character</termref> comes first in the <code>$input</code> string) is\n             replaced.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\" diff=\"add\" at=\"B\">If the <code>q</code> flag is present, the replacement string is used\n                <emph>as is</emph>.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\"><phrase diff=\"add\" at=\"B\">Otherwise,</phrase> within the <code>$replacement</code>\n             string, a variable <code>$N</code> may be used to refer to the substring captured by the\n             Nth parenthesized sub-expression in the regular expression. For each match of the\n             pattern, these variables are assigned the value of the content matched by the relevant\n             sub-expression, and the modified replacement string is then substituted for the <termref def=\"character\">characters</termref> in <code>$input</code> that matched the pattern.\n                <code>$0</code> refers to the substring captured by the regular expression as a\n             whole.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">More specifically, the rules are as follows, where <code>S</code> is the number of\n             parenthesized sub-expressions in the regular expression, and <code>N</code> is the\n             decimal number formed by taking all the digits that consecutively follow the\n                <code>$</code> character:</p><olist xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\"><item><p>If <code>N</code>=<code>0</code>, then the variable is replaced by the substring\n                   matched by the regular expression as a whole.</p></item><item><p>If <code>1</code>&lt;=<code>N</code>&lt;=<code>S</code>, then the variable is\n                   replaced by the substring captured by the Nth parenthesized sub-expression. If the\n                      <code>Nth</code> parenthesized sub-expression was not matched, then the\n                   variable is replaced by the zero-length string.</p></item><item><p>If <code>S</code>&lt;<code>N</code>&lt;=<code>9</code>, then the variable is\n                   replaced by the zero-length string.</p></item><item><p>Otherwise (if <code>N</code>&gt;<code>S</code> and\n                      <code>N</code>&gt;<code>9</code>), the last digit of <code>N</code> is taken to\n                   be a literal character to be included \"as is\" in the replacement string, and the\n                   rules are reapplied using the number <code>N</code> formed by stripping off this\n                   last digit.</p></item></olist><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">For example, if the replacement string is <quote>\n                <code>$23</code>\n             </quote> and there are 5 substrings, the result contains the value of the substring that\n             matches the second sub-expression, followed by the digit <quote>\n                <code>3</code>\n             </quote>.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\" diff=\"chg\" at=\"B\">Unless the <code>q</code> flag is used, a literal <code>$</code>\n             character within the replacement string must be written as <code>\\$</code>, and a\n             literal <code>\\</code> character must be written as <code>\\\\</code>.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\"> If two alternatives within the pattern both match at the same position in the\n                <code>$input</code>, then the match that is chosen is the one matched by the first\n             alternative. For example:</p><eg xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\" xml:space=\"preserve\"> fn:replace(\"abcd\", \"(ab)|(a)\", \"[1=$1][2=$2]\") returns \"[1=ab][2=]cd\"</eg><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">A <phrase diff=\"add\" at=\"L\">dynamic</phrase> error is raised <errorref class=\"RX\" code=\"0002\"/> if the value of\n                <code>$pattern</code> is invalid according to the rules described in section <specref ref=\"regex-syntax\"/>. </p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">A <phrase diff=\"add\" at=\"L\">dynamic</phrase> error is raised <errorref class=\"RX\" code=\"0001\"/> if the value of\n                <code>$flags</code> is invalid according to the rules described in section <specref ref=\"regex-syntax\"/>. </p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">A <phrase diff=\"add\" at=\"L\">dynamic</phrase> error is raised <errorref class=\"RX\" code=\"0003\"/> if the pattern matches a\n             zero-length string, that is, if the expression <code>fn:matches(\"\", $pattern,\n                $flags)</code> returns <code>true</code>. It is not an error, however, if a captured\n             substring is zero-length.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">A <phrase diff=\"add\" at=\"L\">dynamic</phrase> error is raised <errorref class=\"RX\" code=\"0004\"/> if the value of\n                <code>$replacement</code> contains a \"<code>$</code>\" character that is not\n             immediately followed by a digit <code>0-9</code> and not immediately preceded by a\n             \"\\\".</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">A <phrase diff=\"add\" at=\"L\">dynamic</phrase> error is raised <errorref class=\"RX\" code=\"0004\"/> if the value of\n                <code>$replacement</code> contains a \"<code>\\</code>\" character that is not part of a\n                \"<code>\\\\</code>\" pair, unless it is immediately followed by a \"<code>$</code>\"\n             character.</p></div>\n", "summary" : "<p>  Returns a string produced from the input string by replacing any substrings\n             that match a given regular expression with a supplied replacement string.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "input", "type" : "xs:string", "occurrence" : "?", "description" : "" }, { "name" : "pattern", "type" : "xs:string", "occurrence" : null, "description" : "" }, { "name" : "replacement", "type" : "xs:string", "occurrence" : null, "description" : "" }, { "name" : "flags", "type" : "xs:string", "occurrence" : null, "description" : "" } ], "returns" : { "type" : "xs:string", "description" : "" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 2, "name" : "resolve-QName", "qname" : "fn:resolve-QName", "signature" : "($qname as xs:string?, $element as element(*)) as xs:QName? external", "description" : " <div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">Returns an <code>xs:QName</code> value (that is, an expanded-QName) by taking\n             an <code>xs:string</code> that has the lexical form of an <code>xs:QName</code> (a\n             string in the form \"prefix:local-name\" or \"local-name\") and resolving it using the\n             in-scope namespaces for a given element.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\"><example role=\"signature\"><proto name=\"resolve-QName\" return-type=\"xs:QName?\" isOp=\"no\" prefix=\"fn\" returnEmptyOk=\"no\" returnSeq=\"no\" returnVaries=\"no\" isSchema=\"no\" isDatatype=\"no\" isSpecial=\"no\"><arg name=\"qname\" type=\"xs:string?\"/><arg name=\"element\" type=\"element()\"/></proto></example></p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">This function is <termref def=\"dt-deterministic\">deterministic</termref>, <termref def=\"dt-context-independent\">context-independent</termref>,  and <termref def=\"dt-focus-independent\">focus-independent</termref>. </p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">If <code>$qname</code> is the empty sequence, returns the empty sequence.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">More specifically, the function searches the namespace bindings of <code>$element</code>\n             for a binding whose name matches the prefix of <code>$qname</code>, or the zero-length\n             string if it has no prefix, and constructs an expanded-QName whose local name is taken\n             from the supplied <code>$qname</code>, and whose namespace URI is taken from the string\n             value of the namespace binding.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">If the <code>$qname</code> has no prefix, and there is no namespace binding for\n                <code>$element</code> corresponding to the default (unnamed) namespace, then the\n             resulting expanded-QName has no namespace part.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The prefix (or absence of a prefix) in the supplied <code>$qname</code> argument is\n             retained in the returned expanded-QName, as discussed in <xspecref spec=\"DM30\" ref=\"terminology\"/>.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">A <phrase diff=\"add\" at=\"L\">dynamic</phrase> error is raised <errorref class=\"CA\" code=\"0002\"/> if <code>$qname</code> does not\n             have the correct lexical form for an instance of <code>xs:QName</code>.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">A <phrase diff=\"add\" at=\"L\">dynamic</phrase> error is raised <errorref class=\"NS\" code=\"0004\"/> if <code>$qname</code> has a\n             prefix and there is no namespace binding for <code>$element</code> that matches this\n             prefix.</p></div>\n", "summary" : "<p>  Returns an  xs:QName  value (that is, an expanded-QName) by taking\n             an  xs:string  that has the lexical form of an  xs:QName  (a\n             string in the form \"prefix:local-name\" or \"local-name\") and resolving it using the\n             in-scope namespaces for a given element.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "qname", "type" : "xs:string", "occurrence" : "?", "description" : "" }, { "name" : "element", "type" : "element(*)", "occurrence" : null, "description" : "" } ], "returns" : { "type" : "xs:QName?", "description" : "" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "resolve-uri", "qname" : "fn:resolve-uri", "signature" : "($relative as xs:string?) as xs:anyURI? external", "description" : " <div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">Resolves a relative IRI reference against an absolute IRI.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\"><example role=\"signature\"><proto name=\"resolve-uri\" return-type=\"xs:anyURI?\" isOp=\"no\" prefix=\"fn\" returnEmptyOk=\"no\" returnSeq=\"no\" returnVaries=\"no\" isSchema=\"no\" isDatatype=\"no\" isSpecial=\"no\"><arg name=\"relative\" type=\"xs:string?\"/></proto></example><example role=\"signature\"><proto name=\"resolve-uri\" return-type=\"xs:anyURI?\" isOp=\"no\" prefix=\"fn\" returnEmptyOk=\"no\" returnSeq=\"no\" returnVaries=\"no\" isSchema=\"no\" isDatatype=\"no\" isSpecial=\"no\"><arg name=\"relative\" type=\"xs:string?\"/><arg name=\"base\" type=\"xs:string\"/></proto></example></p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The one-argument form of this function is <termref def=\"dt-deterministic\">deterministic</termref>, <termref def=\"dt-context-dependent\">context-dependent</termref>,  and <termref def=\"dt-focus-independent\">focus-independent</termref>.  It depends on\n \t\tstatic base uri.\n \t</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The two-argument form of this function is <termref def=\"dt-deterministic\">deterministic</termref>, <termref def=\"dt-context-independent\">context-independent</termref>,  and <termref def=\"dt-focus-independent\">focus-independent</termref>. </p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">If the second argument is absent, the effect is the same as calling the two-argument\n             function with the value of <code>fn:static-base-uri()</code> as the second argument.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The function is defined to operate on IRI references as defined in <bibref ref=\"rfc3987\"/>, and the implementation <rfc2119>must</rfc2119> permit all arguments that are valid\n             according to that specification. In addition, the implementation <rfc2119>may</rfc2119>\n             accept some or all strings that conform to the rules for (absolute or relative) Legacy\n             Extended IRI references as defined in <bibref ref=\"LEIRI\"/>. For the purposes of this\n             section, the terms IRI and IRI reference include these extensions, insofar as the\n             implementation chooses to support them.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">If <code>$relative</code> is the empty sequence, the function returns the empty\n             sequence.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">If <code>$relative</code> is an absolute IRI (as defined above), then it is returned\n             unchanged.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">Otherwise, the function resolves the relative IRI reference <code>$relative</code>\n             against the base IRI <code>$base</code> using the algorithm defined in <bibref ref=\"rfc3986\"/>, adapted by treating any <termref def=\"character\">character</termref>\n             that would not be valid in an RFC3986 URI or relative reference in the same way that\n             RFC3986 treats unreserved characters. No percent-encoding takes place.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The first form of this function resolves <code>$relative</code> against the value of the\n             base-uri property from the static context. A <phrase diff=\"add\" at=\"L\">dynamic</phrase> error is raised <errorref class=\"NS\" code=\"0005\"/> if the base-uri property is not initialized in the static context. </p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">A <phrase diff=\"add\" at=\"L\">dynamic</phrase> error is raised <errorref class=\"RG\" code=\"0002\"/> if <code>$relative</code> is not a\n             valid IRI according to the rules of RFC3987, extended with an implementation-defined\n             subset of the extensions permitted in LEIRI, or if it is not a suitable relative\n             reference to use as input to the RFC3986 resolution algorithm extended to handle\n             additional unreserved characters. </p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">A <phrase diff=\"add\" at=\"L\">dynamic</phrase> error is raised <errorref class=\"RG\" code=\"0002\"/> if <code>$base</code> is not a\n             valid IRI according to the rules of RFC3987, extended with an implementation-defined\n             subset of the extensions permitted in LEIRI, or if it is not a suitable IRI to use as\n             input to the chosen resolution algorithm (for example, if it is a relative IRI\n             reference, if it is a non-hierarchic URI, or if it contains a fragment identifier). </p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">A <phrase diff=\"add\" at=\"L\">dynamic</phrase> error is raised <errorref class=\"RG\" code=\"0009\"/> if the chosen resolution algorithm\n             fails for any other reason. </p></div>\n", "summary" : "<p>  Resolves a relative IRI reference against an absolute IRI.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "relative", "type" : "xs:string", "occurrence" : "?", "description" : "" } ], "returns" : { "type" : "xs:anyURI?", "description" : "" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 2, "name" : "resolve-uri", "qname" : "fn:resolve-uri", "signature" : "($relative as xs:string?, $base as xs:string) as xs:anyURI? external", "description" : " <div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">Resolves a relative IRI reference against an absolute IRI.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\"><example role=\"signature\"><proto name=\"resolve-uri\" return-type=\"xs:anyURI?\" isOp=\"no\" prefix=\"fn\" returnEmptyOk=\"no\" returnSeq=\"no\" returnVaries=\"no\" isSchema=\"no\" isDatatype=\"no\" isSpecial=\"no\"><arg name=\"relative\" type=\"xs:string?\"/></proto></example><example role=\"signature\"><proto name=\"resolve-uri\" return-type=\"xs:anyURI?\" isOp=\"no\" prefix=\"fn\" returnEmptyOk=\"no\" returnSeq=\"no\" returnVaries=\"no\" isSchema=\"no\" isDatatype=\"no\" isSpecial=\"no\"><arg name=\"relative\" type=\"xs:string?\"/><arg name=\"base\" type=\"xs:string\"/></proto></example></p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The one-argument form of this function is <termref def=\"dt-deterministic\">deterministic</termref>, <termref def=\"dt-context-dependent\">context-dependent</termref>,  and <termref def=\"dt-focus-independent\">focus-independent</termref>.  It depends on\n \t\tstatic base uri.\n \t</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The two-argument form of this function is <termref def=\"dt-deterministic\">deterministic</termref>, <termref def=\"dt-context-independent\">context-independent</termref>,  and <termref def=\"dt-focus-independent\">focus-independent</termref>. </p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">If the second argument is absent, the effect is the same as calling the two-argument\n             function with the value of <code>fn:static-base-uri()</code> as the second argument.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The function is defined to operate on IRI references as defined in <bibref ref=\"rfc3987\"/>, and the implementation <rfc2119>must</rfc2119> permit all arguments that are valid\n             according to that specification. In addition, the implementation <rfc2119>may</rfc2119>\n             accept some or all strings that conform to the rules for (absolute or relative) Legacy\n             Extended IRI references as defined in <bibref ref=\"LEIRI\"/>. For the purposes of this\n             section, the terms IRI and IRI reference include these extensions, insofar as the\n             implementation chooses to support them.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">If <code>$relative</code> is the empty sequence, the function returns the empty\n             sequence.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">If <code>$relative</code> is an absolute IRI (as defined above), then it is returned\n             unchanged.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">Otherwise, the function resolves the relative IRI reference <code>$relative</code>\n             against the base IRI <code>$base</code> using the algorithm defined in <bibref ref=\"rfc3986\"/>, adapted by treating any <termref def=\"character\">character</termref>\n             that would not be valid in an RFC3986 URI or relative reference in the same way that\n             RFC3986 treats unreserved characters. No percent-encoding takes place.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The first form of this function resolves <code>$relative</code> against the value of the\n             base-uri property from the static context. A <phrase diff=\"add\" at=\"L\">dynamic</phrase> error is raised <errorref class=\"NS\" code=\"0005\"/> if the base-uri property is not initialized in the static context. </p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">A <phrase diff=\"add\" at=\"L\">dynamic</phrase> error is raised <errorref class=\"RG\" code=\"0002\"/> if <code>$relative</code> is not a\n             valid IRI according to the rules of RFC3987, extended with an implementation-defined\n             subset of the extensions permitted in LEIRI, or if it is not a suitable relative\n             reference to use as input to the RFC3986 resolution algorithm extended to handle\n             additional unreserved characters. </p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">A <phrase diff=\"add\" at=\"L\">dynamic</phrase> error is raised <errorref class=\"RG\" code=\"0002\"/> if <code>$base</code> is not a\n             valid IRI according to the rules of RFC3987, extended with an implementation-defined\n             subset of the extensions permitted in LEIRI, or if it is not a suitable IRI to use as\n             input to the chosen resolution algorithm (for example, if it is a relative IRI\n             reference, if it is a non-hierarchic URI, or if it contains a fragment identifier). </p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">A <phrase diff=\"add\" at=\"L\">dynamic</phrase> error is raised <errorref class=\"RG\" code=\"0009\"/> if the chosen resolution algorithm\n             fails for any other reason. </p></div>\n", "summary" : "<p>  Resolves a relative IRI reference against an absolute IRI.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "relative", "type" : "xs:string", "occurrence" : "?", "description" : "" }, { "name" : "base", "type" : "xs:string", "occurrence" : null, "description" : "" } ], "returns" : { "type" : "xs:anyURI?", "description" : "" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "reverse", "qname" : "fn:reverse", "signature" : "($arg as item()*) as item()* external", "description" : " <div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">Reverses the order of items in a sequence. </p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\"><example role=\"signature\"><proto name=\"reverse\" return-type=\"item()*\" isOp=\"no\" prefix=\"fn\" returnEmptyOk=\"no\" returnSeq=\"no\" returnVaries=\"no\" isSchema=\"no\" isDatatype=\"no\" isSpecial=\"no\"><arg name=\"arg\" type=\"item()*\"/></proto></example></p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">This function is <termref def=\"dt-deterministic\">deterministic</termref>, <termref def=\"dt-context-independent\">context-independent</termref>,  and <termref def=\"dt-focus-independent\">focus-independent</termref>. </p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The function returns a sequence containing the items in <code>$arg</code> in reverse\n             order.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\" diff=\"del\" at=\"F\">For detailed type semantics, see [Formal Semantics].</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">If <code>$arg</code> is the empty sequence, the empty sequence is returned. </p></div>\n", "summary" : "<p>  Reverses the order of items in a sequence.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "arg", "type" : "item()", "occurrence" : "*", "description" : "" } ], "returns" : { "type" : "item()*", "description" : "" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 0, "name" : "root", "qname" : "fn:root", "signature" : "() as node() external", "description" : " <div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">Returns the root of the tree to which <code>$arg</code> belongs. This will\n             usually, but not necessarily, be a document node.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\"><example role=\"signature\"><proto name=\"root\" return-type=\"node()\" isOp=\"no\" prefix=\"fn\" returnEmptyOk=\"no\" returnSeq=\"no\" returnVaries=\"no\" isSchema=\"no\" isDatatype=\"no\" isSpecial=\"no\"/></example><example role=\"signature\"><proto name=\"root\" return-type=\"node()?\" isOp=\"no\" prefix=\"fn\" returnEmptyOk=\"no\" returnSeq=\"no\" returnVaries=\"no\" isSchema=\"no\" isDatatype=\"no\" isSpecial=\"no\"><arg name=\"arg\" type=\"node()?\"/></proto></example></p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The zero-argument form of this function is <termref def=\"dt-deterministic\">deterministic</termref>, <termref def=\"dt-context-dependent\">context-dependent</termref>,  and <termref def=\"dt-focus-dependent\">focus-dependent</termref>. </p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The one-argument form of this function is <termref def=\"dt-deterministic\">deterministic</termref>, <termref def=\"dt-context-independent\">context-independent</termref>,  and <termref def=\"dt-focus-independent\">focus-independent</termref>. </p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">If the function is called without an argument, the context item (<code>.</code>) is used\n             as the default argument. The behavior of the function if the argument is omitted is\n             exactly the same as if the context item had been passed as the argument.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The function returns the value of the expression\n                <code>($arg/ancestor-or-self::node())[1]</code>.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The following errors may be raised when <code>$arg</code> is omitted:</p><ul xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\"><li><p>If the context\n                item is <xtermref ref=\"dt-absent\" spec=\"DM30\">absent</xtermref>, <phrase diff=\"add\" at=\"M\">dynamic error</phrase>\n                <xerrorref spec=\"XP\" class=\"DY\" code=\"0002\" type=\"dynamic\"/></p></li><li><p>If the context item is not a\n                node, <phrase diff=\"add\" at=\"M\">type error</phrase> <xerrorref spec=\"XP\" class=\"TY\" code=\"0004\" type=\"type\"/>.</p></li></ul></div>\n", "summary" : "<p>  Returns the root of the tree to which  $arg  belongs.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [  ], "returns" : { "type" : "node()", "description" : "" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "root", "qname" : "fn:root", "signature" : "($arg as node()?) as node()? external", "description" : " <div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">Returns the root of the tree to which <code>$arg</code> belongs. This will\n             usually, but not necessarily, be a document node.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\"><example role=\"signature\"><proto name=\"root\" return-type=\"node()\" isOp=\"no\" prefix=\"fn\" returnEmptyOk=\"no\" returnSeq=\"no\" returnVaries=\"no\" isSchema=\"no\" isDatatype=\"no\" isSpecial=\"no\"/></example><example role=\"signature\"><proto name=\"root\" return-type=\"node()?\" isOp=\"no\" prefix=\"fn\" returnEmptyOk=\"no\" returnSeq=\"no\" returnVaries=\"no\" isSchema=\"no\" isDatatype=\"no\" isSpecial=\"no\"><arg name=\"arg\" type=\"node()?\"/></proto></example></p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The zero-argument form of this function is <termref def=\"dt-deterministic\">deterministic</termref>, <termref def=\"dt-context-dependent\">context-dependent</termref>,  and <termref def=\"dt-focus-dependent\">focus-dependent</termref>. </p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The one-argument form of this function is <termref def=\"dt-deterministic\">deterministic</termref>, <termref def=\"dt-context-independent\">context-independent</termref>,  and <termref def=\"dt-focus-independent\">focus-independent</termref>. </p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">If the function is called without an argument, the context item (<code>.</code>) is used\n             as the default argument. The behavior of the function if the argument is omitted is\n             exactly the same as if the context item had been passed as the argument.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The function returns the value of the expression\n                <code>($arg/ancestor-or-self::node())[1]</code>.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The following errors may be raised when <code>$arg</code> is omitted:</p><ul xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\"><li><p>If the context\n                item is <xtermref ref=\"dt-absent\" spec=\"DM30\">absent</xtermref>, <phrase diff=\"add\" at=\"M\">dynamic error</phrase>\n                <xerrorref spec=\"XP\" class=\"DY\" code=\"0002\" type=\"dynamic\"/></p></li><li><p>If the context item is not a\n                node, <phrase diff=\"add\" at=\"M\">type error</phrase> <xerrorref spec=\"XP\" class=\"TY\" code=\"0004\" type=\"type\"/>.</p></li></ul></div>\n", "summary" : "<p>  Returns the root of the tree to which  $arg  belongs.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "arg", "type" : "node()", "occurrence" : "?", "description" : "" } ], "returns" : { "type" : "node()?", "description" : "" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "round-half-to-even", "qname" : "fn:round-half-to-even", "signature" : "($arg as numeric?) as numeric? external", "description" : " <div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">Rounds a value to a specified number of decimal places, rounding to make the\n             last digit even if two such values are equally near.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\"><example role=\"signature\"><proto name=\"round-half-to-even\" return-type=\"numeric?\" isOp=\"no\" prefix=\"fn\" returnEmptyOk=\"no\" returnSeq=\"no\" returnVaries=\"no\" isSchema=\"no\" isDatatype=\"no\" isSpecial=\"no\"><arg name=\"arg\" type=\"numeric?\"/></proto></example><example role=\"signature\"><proto name=\"round-half-to-even\" return-type=\"numeric?\" isOp=\"no\" prefix=\"fn\" returnEmptyOk=\"no\" returnSeq=\"no\" returnVaries=\"no\" isSchema=\"no\" isDatatype=\"no\" isSpecial=\"no\"><arg name=\"arg\" type=\"numeric?\"/><arg name=\"precision\" type=\"xs:integer\"/></proto></example></p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">This function is <termref def=\"dt-deterministic\">deterministic</termref>, <termref def=\"dt-context-independent\">context-independent</termref>,  and <termref def=\"dt-focus-independent\">focus-independent</termref>. </p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">General rules: see <specref ref=\"numeric-value-functions\"/>.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The function returns the nearest (that is, numerically closest) value to\n                <code>$arg</code> that is a multiple of ten to the power of minus\n                <code>$precision</code>. If two such values are equally near (e.g. if the fractional\n             part in <code>$arg</code> is exactly .500...), the function returns the one whose least\n             significant digit is even.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">If the type of <code>$arg</code> is one of the four numeric types <code>xs:float</code>,\n                <code>xs:double</code>, <code>xs:decimal</code> or <code>xs:integer</code> the type\n             of the result is the same as the type of <code>$arg</code>. If the type of\n                <code>$arg</code> is a type derived from one of the numeric types, the result is an\n             instance of the base numeric type.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\"> The first signature of this function produces the same result as the second signature\n             with <code>$precision=0</code>.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">For arguments of type <code>xs:float</code> and <code>xs:double</code>:</p><olist xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\"><item><p>If the argument is <code>NaN</code>, positive or negative zero, or positive or\n                   negative infinity, then the result is the same as the argument.</p></item><item><p>In all other cases, the argument is cast to <code>xs:decimal</code>\n                   <phrase diff=\"add\" at=\"A\">using an implementation of xs:decimal that imposes no\n                      limits on the number of digits that can be represented.</phrase> The function\n                   is applied to this <code>xs:decimal</code> value, and the resulting\n                      <code>xs:decimal</code> is cast back to <code>xs:float</code> or\n                      <code>xs:double</code> as appropriate to form the function result. If the\n                   resulting <code>xs:decimal</code> value is zero, then positive or negative zero is\n                   returned according to the sign of the original argument.</p></item></olist><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\" diff=\"del\" at=\"F\">For detailed type semantics, see [Formal Semantics].</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\" diff=\"chg\" at=\"A\">This function is typically used in financial applications where the\n             argument is of type <code>xs:decimal</code>. For arguments of type <code>xs:float</code>\n             and <code>xs:double</code> the results may be counter-intuitive. For example, consider\n                <code>round-half-to-even(xs:float(150.015), 2)</code>. The result is not 150.02 as\n             might be expected, but 150.01. This is because the conversion of the\n                <code>xs:float</code> value represented by the literal 150.015 to an\n                <code>xs:decimal</code> produces the <code>xs:decimal</code> value 150.014999389...,\n             which is closer to 150.01 than to 150.02.</p></div>\n", "summary" : "<p>  Rounds a value to a specified number of decimal places, rounding to make the\n             last digit even if two such values are equally near.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "arg", "type" : "numeric", "occurrence" : "?", "description" : "" } ], "returns" : { "type" : "numeric?", "description" : "" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 2, "name" : "round-half-to-even", "qname" : "fn:round-half-to-even", "signature" : "($arg as numeric?, $precision as xs:integer) as numeric? external", "description" : " <div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">Rounds a value to a specified number of decimal places, rounding to make the\n             last digit even if two such values are equally near.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\"><example role=\"signature\"><proto name=\"round-half-to-even\" return-type=\"numeric?\" isOp=\"no\" prefix=\"fn\" returnEmptyOk=\"no\" returnSeq=\"no\" returnVaries=\"no\" isSchema=\"no\" isDatatype=\"no\" isSpecial=\"no\"><arg name=\"arg\" type=\"numeric?\"/></proto></example><example role=\"signature\"><proto name=\"round-half-to-even\" return-type=\"numeric?\" isOp=\"no\" prefix=\"fn\" returnEmptyOk=\"no\" returnSeq=\"no\" returnVaries=\"no\" isSchema=\"no\" isDatatype=\"no\" isSpecial=\"no\"><arg name=\"arg\" type=\"numeric?\"/><arg name=\"precision\" type=\"xs:integer\"/></proto></example></p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">This function is <termref def=\"dt-deterministic\">deterministic</termref>, <termref def=\"dt-context-independent\">context-independent</termref>,  and <termref def=\"dt-focus-independent\">focus-independent</termref>. </p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">General rules: see <specref ref=\"numeric-value-functions\"/>.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The function returns the nearest (that is, numerically closest) value to\n                <code>$arg</code> that is a multiple of ten to the power of minus\n                <code>$precision</code>. If two such values are equally near (e.g. if the fractional\n             part in <code>$arg</code> is exactly .500...), the function returns the one whose least\n             significant digit is even.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">If the type of <code>$arg</code> is one of the four numeric types <code>xs:float</code>,\n                <code>xs:double</code>, <code>xs:decimal</code> or <code>xs:integer</code> the type\n             of the result is the same as the type of <code>$arg</code>. If the type of\n                <code>$arg</code> is a type derived from one of the numeric types, the result is an\n             instance of the base numeric type.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\"> The first signature of this function produces the same result as the second signature\n             with <code>$precision=0</code>.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">For arguments of type <code>xs:float</code> and <code>xs:double</code>:</p><olist xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\"><item><p>If the argument is <code>NaN</code>, positive or negative zero, or positive or\n                   negative infinity, then the result is the same as the argument.</p></item><item><p>In all other cases, the argument is cast to <code>xs:decimal</code>\n                   <phrase diff=\"add\" at=\"A\">using an implementation of xs:decimal that imposes no\n                      limits on the number of digits that can be represented.</phrase> The function\n                   is applied to this <code>xs:decimal</code> value, and the resulting\n                      <code>xs:decimal</code> is cast back to <code>xs:float</code> or\n                      <code>xs:double</code> as appropriate to form the function result. If the\n                   resulting <code>xs:decimal</code> value is zero, then positive or negative zero is\n                   returned according to the sign of the original argument.</p></item></olist><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\" diff=\"del\" at=\"F\">For detailed type semantics, see [Formal Semantics].</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\" diff=\"chg\" at=\"A\">This function is typically used in financial applications where the\n             argument is of type <code>xs:decimal</code>. For arguments of type <code>xs:float</code>\n             and <code>xs:double</code> the results may be counter-intuitive. For example, consider\n                <code>round-half-to-even(xs:float(150.015), 2)</code>. The result is not 150.02 as\n             might be expected, but 150.01. This is because the conversion of the\n                <code>xs:float</code> value represented by the literal 150.015 to an\n                <code>xs:decimal</code> produces the <code>xs:decimal</code> value 150.014999389...,\n             which is closer to 150.01 than to 150.02.</p></div>\n", "summary" : "<p>  Rounds a value to a specified number of decimal places, rounding to make the\n             last digit even if two such values are equally near.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "arg", "type" : "numeric", "occurrence" : "?", "description" : "" }, { "name" : "precision", "type" : "xs:integer", "occurrence" : null, "description" : "" } ], "returns" : { "type" : "numeric?", "description" : "" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "round", "qname" : "fn:round", "signature" : "($arg as numeric?) as numeric? external", "description" : " <div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">Rounds a value to a specified number of decimal places, rounding upwards if two\n             such values are equally near.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\"><example role=\"signature\"><proto name=\"round\" return-type=\"numeric?\" isOp=\"no\" prefix=\"fn\" returnEmptyOk=\"no\" returnSeq=\"no\" returnVaries=\"no\" isSchema=\"no\" isDatatype=\"no\" isSpecial=\"no\"><arg name=\"arg\" type=\"numeric?\"/></proto></example><example role=\"signature\"><proto name=\"round\" return-type=\"numeric?\" isOp=\"no\" prefix=\"fn\" returnEmptyOk=\"no\" returnSeq=\"no\" returnVaries=\"no\" isSchema=\"no\" isDatatype=\"no\" isSpecial=\"no\"><arg name=\"arg\" type=\"numeric?\"/><arg name=\"precision\" type=\"xs:integer\"/></proto></example></p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">This function is <termref def=\"dt-deterministic\">deterministic</termref>, <termref def=\"dt-context-independent\">context-independent</termref>,  and <termref def=\"dt-focus-independent\">focus-independent</termref>. </p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">General rules: see <specref ref=\"numeric-value-functions\"/>.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The function returns the nearest (that is, numerically closest) value to\n                <code>$arg</code> that is a multiple of ten to the power of minus\n                <code>$precision</code>. If two such values are equally near (for example, if the\n             fractional part in <code>$arg</code> is exactly .5), the function returns the one that\n             is closest to positive infinity.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">If the type of <code>$arg</code> is one of the four numeric types <code>xs:float</code>,\n                <code>xs:double</code>, <code>xs:decimal</code> or <code>xs:integer</code> the type\n             of the result is the same as the type of <code>$arg</code>. If the type of\n                <code>$arg</code> is a type derived from one of the numeric types, the result is an\n             instance of the base numeric type.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The single-argument version of this function produces the same result as the\n             two-argument version with <code>$precision=0</code> (that is, it rounds to a whole\n             number).</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">When <code>$arg</code> is of type <code>xs:float</code> and <code>xs:double</code>:</p><olist xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\"><item><p>If <code>$arg</code> is NaN, positive or negative zero, or positive or negative\n                   infinity, then the result is the same as the argument.</p></item><item><p>For other values, the argument is cast to <code>xs:decimal</code> using an\n                   implementation of <code>xs:decimal</code> that imposes no limits on the number of\n                   digits that can be represented. The function is applied to this\n                      <code>xs:decimal</code> value, and the resulting <code>xs:decimal</code> is\n                   cast back to <code>xs:float</code> or <code>xs:double</code> as appropriate to\n                   form the function result. If the resulting <code>xs:decimal</code> value is zero,\n                   then positive or negative zero is returned according to the sign of\n                      <code>$arg</code>.</p></item></olist><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\" diff=\"del\" at=\"F\">For detailed type semantics, see [Formal Semantics].</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">This function is typically used with a non-zero <code>$precision</code> in financial\n             applications where the argument is of type <code>xs:decimal</code>. For arguments of\n             type <code>xs:float</code> and <code>xs:double</code> the results may be\n             counter-intuitive. For example, consider <code>round(35.425e0, 2)</code>. The result is\n             not 35.43, as might be expected, but 35.42. This is because the <code>xs:double</code> written as 35.425e0\n             has an exact value equal to 35.42499999999..., which is closer\n             to 35.42 than to 35.43.</p></div>\n", "summary" : "<p>  Rounds a value to a specified number of decimal places, rounding upwards if two\n             such values are equally near.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "arg", "type" : "numeric", "occurrence" : "?", "description" : "" } ], "returns" : { "type" : "numeric?", "description" : "" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 2, "name" : "round", "qname" : "fn:round", "signature" : "($arg as numeric?, $precision as xs:integer) as numeric? external", "description" : " <div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">Rounds a value to a specified number of decimal places, rounding upwards if two\n             such values are equally near.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\"><example role=\"signature\"><proto name=\"round\" return-type=\"numeric?\" isOp=\"no\" prefix=\"fn\" returnEmptyOk=\"no\" returnSeq=\"no\" returnVaries=\"no\" isSchema=\"no\" isDatatype=\"no\" isSpecial=\"no\"><arg name=\"arg\" type=\"numeric?\"/></proto></example><example role=\"signature\"><proto name=\"round\" return-type=\"numeric?\" isOp=\"no\" prefix=\"fn\" returnEmptyOk=\"no\" returnSeq=\"no\" returnVaries=\"no\" isSchema=\"no\" isDatatype=\"no\" isSpecial=\"no\"><arg name=\"arg\" type=\"numeric?\"/><arg name=\"precision\" type=\"xs:integer\"/></proto></example></p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">This function is <termref def=\"dt-deterministic\">deterministic</termref>, <termref def=\"dt-context-independent\">context-independent</termref>,  and <termref def=\"dt-focus-independent\">focus-independent</termref>. </p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">General rules: see <specref ref=\"numeric-value-functions\"/>.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The function returns the nearest (that is, numerically closest) value to\n                <code>$arg</code> that is a multiple of ten to the power of minus\n                <code>$precision</code>. If two such values are equally near (for example, if the\n             fractional part in <code>$arg</code> is exactly .5), the function returns the one that\n             is closest to positive infinity.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">If the type of <code>$arg</code> is one of the four numeric types <code>xs:float</code>,\n                <code>xs:double</code>, <code>xs:decimal</code> or <code>xs:integer</code> the type\n             of the result is the same as the type of <code>$arg</code>. If the type of\n                <code>$arg</code> is a type derived from one of the numeric types, the result is an\n             instance of the base numeric type.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The single-argument version of this function produces the same result as the\n             two-argument version with <code>$precision=0</code> (that is, it rounds to a whole\n             number).</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">When <code>$arg</code> is of type <code>xs:float</code> and <code>xs:double</code>:</p><olist xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\"><item><p>If <code>$arg</code> is NaN, positive or negative zero, or positive or negative\n                   infinity, then the result is the same as the argument.</p></item><item><p>For other values, the argument is cast to <code>xs:decimal</code> using an\n                   implementation of <code>xs:decimal</code> that imposes no limits on the number of\n                   digits that can be represented. The function is applied to this\n                      <code>xs:decimal</code> value, and the resulting <code>xs:decimal</code> is\n                   cast back to <code>xs:float</code> or <code>xs:double</code> as appropriate to\n                   form the function result. If the resulting <code>xs:decimal</code> value is zero,\n                   then positive or negative zero is returned according to the sign of\n                      <code>$arg</code>.</p></item></olist><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\" diff=\"del\" at=\"F\">For detailed type semantics, see [Formal Semantics].</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">This function is typically used with a non-zero <code>$precision</code> in financial\n             applications where the argument is of type <code>xs:decimal</code>. For arguments of\n             type <code>xs:float</code> and <code>xs:double</code> the results may be\n             counter-intuitive. For example, consider <code>round(35.425e0, 2)</code>. The result is\n             not 35.43, as might be expected, but 35.42. This is because the <code>xs:double</code> written as 35.425e0\n             has an exact value equal to 35.42499999999..., which is closer\n             to 35.42 than to 35.43.</p></div>\n", "summary" : "<p>  Rounds a value to a specified number of decimal places, rounding upwards if two\n             such values are equally near.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "arg", "type" : "numeric", "occurrence" : "?", "description" : "" }, { "name" : "precision", "type" : "xs:integer", "occurrence" : null, "description" : "" } ], "returns" : { "type" : "numeric?", "description" : "" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "seconds-from-dateTime", "qname" : "fn:seconds-from-dateTime", "signature" : "($arg as xs:dateTime?) as xs:decimal? external", "description" : " <div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">Returns the seconds component of an <code>xs:dateTime</code>.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\"><example role=\"signature\"><proto name=\"seconds-from-dateTime\" return-type=\"xs:decimal?\" isOp=\"no\" prefix=\"fn\" returnEmptyOk=\"no\" returnSeq=\"no\" returnVaries=\"no\" isSchema=\"no\" isDatatype=\"no\" isSpecial=\"no\"><arg name=\"arg\" type=\"xs:dateTime?\"/></proto></example></p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">This function is <termref def=\"dt-deterministic\">deterministic</termref>, <termref def=\"dt-context-independent\">context-independent</termref>,  and <termref def=\"dt-focus-independent\">focus-independent</termref>. </p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">If <code>$arg</code> is the empty sequence, the function returns the empty sequence.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">Otherwise, the function returns an <code>xs:decimal</code> value greater than or equal\n             to zero and less than 60, representing the seconds and fractional seconds in the local\n             value of <code>$arg</code>.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The expression <code>fn:seconds-from-dateTime(xs:dateTime(\"1999-05-31T13:20:00-05:00\"))</code> returns <code>0</code>.</p></div>\n", "summary" : "<p>  Returns the seconds component of an  xs:dateTime .</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "arg", "type" : "xs:dateTime", "occurrence" : "?", "description" : "" } ], "returns" : { "type" : "xs:decimal?", "description" : "" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "seconds-from-duration", "qname" : "fn:seconds-from-duration", "signature" : "($arg as xs:duration?) as xs:decimal? external", "description" : " <div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">Returns the number of seconds in a duration.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\"><example role=\"signature\"><proto name=\"seconds-from-duration\" return-type=\"xs:decimal?\" isOp=\"no\" prefix=\"fn\" returnEmptyOk=\"no\" returnSeq=\"no\" returnVaries=\"no\" isSchema=\"no\" isDatatype=\"no\" isSpecial=\"no\"><arg name=\"arg\" type=\"xs:duration?\"/></proto></example></p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">This function is <termref def=\"dt-deterministic\">deterministic</termref>, <termref def=\"dt-context-independent\">context-independent</termref>,  and <termref def=\"dt-focus-independent\">focus-independent</termref>. </p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">If <code>$arg</code> is the empty sequence, the function returns the empty sequence.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">Otherwise, the function returns an <code>xs:decimal</code> representing the seconds\n             component in the value of <code>$arg</code>. The result is obtained by casting\n                <code>$arg</code> to an <code>xs:dayTimeDuration</code> (see <specref ref=\"casting-to-durations\"/>) and then computing the seconds component as described\n             in <specref ref=\"canonical-dayTimeDuration\"/>.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">If <code>$arg</code> is a negative duration then the result will be negative..</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">If <code>$arg</code> is an <code>xs:yearMonthDuration</code> the function returns 0.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The expression <code>fn:seconds-from-duration(xs:dayTimeDuration(\"P3DT10H12.5S\"))</code> returns <code>12.5</code>.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The expression <code>fn:seconds-from-duration(xs:dayTimeDuration(\"-PT256S\"))</code> returns <code>-16.0</code>.</p></div>\n", "summary" : "<p>  Returns the number of seconds in a duration.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "arg", "type" : "xs:duration", "occurrence" : "?", "description" : "" } ], "returns" : { "type" : "xs:decimal?", "description" : "" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "seconds-from-time", "qname" : "fn:seconds-from-time", "signature" : "($arg as xs:time?) as xs:decimal? external", "description" : " <div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">Returns the seconds component of an <code>xs:time</code>.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\"><example role=\"signature\"><proto name=\"seconds-from-time\" return-type=\"xs:decimal?\" isOp=\"no\" prefix=\"fn\" returnEmptyOk=\"no\" returnSeq=\"no\" returnVaries=\"no\" isSchema=\"no\" isDatatype=\"no\" isSpecial=\"no\"><arg name=\"arg\" type=\"xs:time?\"/></proto></example></p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">This function is <termref def=\"dt-deterministic\">deterministic</termref>, <termref def=\"dt-context-independent\">context-independent</termref>,  and <termref def=\"dt-focus-independent\">focus-independent</termref>. </p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">If <code>$arg</code> is the empty sequence, the function returns the empty sequence.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">Otherwise, the function returns an <code>xs:decimal</code> value greater than or equal\n             to zero and less than 60, representing the seconds and fractional seconds in the local\n             value of <code>$arg</code>.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The expression <code>fn:seconds-from-time(xs:time(\"13:20:10.5\"))</code> returns <code>10.5</code>.</p></div>\n", "summary" : "<p>  Returns the seconds component of an  xs:time .</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "arg", "type" : "xs:time", "occurrence" : "?", "description" : "" } ], "returns" : { "type" : "xs:decimal?", "description" : "" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "serialize", "qname" : "fn:serialize", "signature" : "($arg as item()*) as xs:string external", "description" : " <div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">This function serializes the supplied <phrase diff=\"chg\" at=\"G\">input\n                sequence</phrase>\n             <code>$arg</code> as described in <bibref ref=\"xslt-xquery-serialization-30\"/>,\n             returning the serialized <phrase diff=\"chg\" at=\"G\">representation of the\n                sequence</phrase> as a string.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\"><example role=\"signature\"><proto name=\"serialize\" return-type=\"xs:string\" isOp=\"no\" prefix=\"fn\" returnEmptyOk=\"no\" returnSeq=\"no\" returnVaries=\"no\" isSchema=\"no\" isDatatype=\"no\" isSpecial=\"no\"><arg name=\"arg\" type=\"item()*\"/></proto></example><example role=\"signature\"><proto name=\"serialize\" return-type=\"xs:string\" isOp=\"no\" prefix=\"fn\" returnEmptyOk=\"no\" returnSeq=\"no\" returnVaries=\"no\" isSchema=\"no\" isDatatype=\"no\" isSpecial=\"no\"><arg name=\"arg\" type=\"item()*\"/><arg name=\"params\" type=\"element(output:serialization-parameters)?\"/></proto></example></p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">This function is <termref def=\"dt-deterministic\">deterministic</termref>, <termref def=\"dt-context-independent\">context-independent</termref>,  and <termref def=\"dt-focus-independent\">focus-independent</termref>. </p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The value of <code>$arg</code> acts as the input sequence to the serialization process,\n             which starts with sequence normalization.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\" diff=\"chg\" at=\"G\">The single-argument version of this function has the same effect as\n             the two-argument version called with <code>$params</code> set to an empty sequence. This\n             in turn is the same as the effect of passing an\n                <code>output:serialization-parameters</code> element with no child elements.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\" diff=\"chg\" at=\"G\">The <code>$params</code> argument is used to identify a set of\n             serialization parameters. These are supplied in the form of an\n                <code>output:serialization-parameters</code> element, having the format described in\n                <xspecref spec=\"SER30\" ref=\"serparams-in-xdm-instance\"/>.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The final stage of serialization, that is, encoding, is skipped. If the serializer does\n             not allow this phase to be skipped, then the sequence of octets returned by the\n             serializer is decoded into a string by reversing the character encoding performed in the\n             final stage.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\" diff=\"add\" at=\"L\">If the host language makes serialization an optional feature and\n          the implementation does not support serialization, then a dynamic error\n             <errorref class=\"DC\" code=\"0010\"/> is raised.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The serialization process will raise an error if <code>$arg</code> is an attribute or\n             namespace node.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">If any serialization error occurs, including the detection of an invalid value for a\n             serialization parameter, this results in the <code>fn:serialize</code> call failing with\n             a dynamic error.</p></div>\n", "summary" : "<p>  This function serializes the supplied  input\n                sequence \n              $arg  as described in   ,\n             returning the serialized  representation of the\n                sequence  as a string.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "arg", "type" : "item()", "occurrence" : "*", "description" : "" } ], "returns" : { "type" : "xs:string", "description" : "" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 2, "name" : "serialize", "qname" : "fn:serialize", "signature" : "($arg as item()*, $params as element(output:serialization-parameters)?) as xs:string external", "description" : " <div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">This function serializes the supplied <phrase diff=\"chg\" at=\"G\">input\n                sequence</phrase>\n             <code>$arg</code> as described in <bibref ref=\"xslt-xquery-serialization-30\"/>,\n             returning the serialized <phrase diff=\"chg\" at=\"G\">representation of the\n                sequence</phrase> as a string.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\"><example role=\"signature\"><proto name=\"serialize\" return-type=\"xs:string\" isOp=\"no\" prefix=\"fn\" returnEmptyOk=\"no\" returnSeq=\"no\" returnVaries=\"no\" isSchema=\"no\" isDatatype=\"no\" isSpecial=\"no\"><arg name=\"arg\" type=\"item()*\"/></proto></example><example role=\"signature\"><proto name=\"serialize\" return-type=\"xs:string\" isOp=\"no\" prefix=\"fn\" returnEmptyOk=\"no\" returnSeq=\"no\" returnVaries=\"no\" isSchema=\"no\" isDatatype=\"no\" isSpecial=\"no\"><arg name=\"arg\" type=\"item()*\"/><arg name=\"params\" type=\"element(output:serialization-parameters)?\"/></proto></example></p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">This function is <termref def=\"dt-deterministic\">deterministic</termref>, <termref def=\"dt-context-independent\">context-independent</termref>,  and <termref def=\"dt-focus-independent\">focus-independent</termref>. </p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The value of <code>$arg</code> acts as the input sequence to the serialization process,\n             which starts with sequence normalization.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\" diff=\"chg\" at=\"G\">The single-argument version of this function has the same effect as\n             the two-argument version called with <code>$params</code> set to an empty sequence. This\n             in turn is the same as the effect of passing an\n                <code>output:serialization-parameters</code> element with no child elements.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\" diff=\"chg\" at=\"G\">The <code>$params</code> argument is used to identify a set of\n             serialization parameters. These are supplied in the form of an\n                <code>output:serialization-parameters</code> element, having the format described in\n                <xspecref spec=\"SER30\" ref=\"serparams-in-xdm-instance\"/>.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The final stage of serialization, that is, encoding, is skipped. If the serializer does\n             not allow this phase to be skipped, then the sequence of octets returned by the\n             serializer is decoded into a string by reversing the character encoding performed in the\n             final stage.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\" diff=\"add\" at=\"L\">If the host language makes serialization an optional feature and\n          the implementation does not support serialization, then a dynamic error\n             <errorref class=\"DC\" code=\"0010\"/> is raised.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The serialization process will raise an error if <code>$arg</code> is an attribute or\n             namespace node.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">If any serialization error occurs, including the detection of an invalid value for a\n             serialization parameter, this results in the <code>fn:serialize</code> call failing with\n             a dynamic error.</p></div>\n", "summary" : "<p>  This function serializes the supplied  input\n                sequence \n              $arg  as described in   ,\n             returning the serialized  representation of the\n                sequence  as a string.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "arg", "type" : "item()", "occurrence" : "*", "description" : "" }, { "name" : "params", "type" : "element(output:serialization-parameters)", "occurrence" : "?", "description" : "" } ], "returns" : { "type" : "xs:string", "description" : "" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 2, "name" : "starts-with", "qname" : "fn:starts-with", "signature" : "($arg1 as xs:string?, $arg2 as xs:string?) as xs:boolean external", "description" : " <div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">Returns true if the string <code>$arg1</code> contains <code>$arg2</code> as a\n             leading substring, taking collations into account.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\"><example role=\"signature\"><proto name=\"starts-with\" return-type=\"xs:boolean\" isOp=\"no\" prefix=\"fn\" returnEmptyOk=\"no\" returnSeq=\"no\" returnVaries=\"no\" isSchema=\"no\" isDatatype=\"no\" isSpecial=\"no\"><arg name=\"arg1\" type=\"xs:string?\"/><arg name=\"arg2\" type=\"xs:string?\"/></proto></example><example role=\"signature\"><proto name=\"starts-with\" return-type=\"xs:boolean\" isOp=\"no\" prefix=\"fn\" returnEmptyOk=\"no\" returnSeq=\"no\" returnVaries=\"no\" isSchema=\"no\" isDatatype=\"no\" isSpecial=\"no\"><arg name=\"arg1\" type=\"xs:string?\"/><arg name=\"arg2\" type=\"xs:string?\"/><arg name=\"collation\" type=\"xs:string\"/></proto></example></p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The two-argument form of this function is <termref def=\"dt-deterministic\">deterministic</termref>, <termref def=\"dt-context-dependent\">context-dependent</termref>,  and <termref def=\"dt-focus-independent\">focus-independent</termref>.  It depends on\n \t\tcollations.\n \t</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The three-argument form of this function is <termref def=\"dt-deterministic\">deterministic</termref>, <termref def=\"dt-context-dependent\">context-dependent</termref>,  and <termref def=\"dt-focus-independent\">focus-independent</termref>.  It depends on\n \t\tcollations, and static base uri.\n \t</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">If the value of <code>$arg1</code> or <code>$arg2</code> is the empty sequence, or\n             contains only ignorable collation units, it is interpreted as the zero-length\n             string.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">If the value of <code>$arg2</code> is the zero-length string, then the function returns\n                <code>true</code>. If the value of <code>$arg1</code> is the zero-length string and\n             the value of <code>$arg2</code> is not the zero-length string, then the function returns\n                <code>false</code>.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The collation used by this function is determined according to the rules in <specref ref=\"choosing-a-collation\"/>.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The function returns an <code>xs:boolean</code> indicating whether or not the value of\n                <code>$arg1</code> starts with a sequence of collation units that provides a\n                <term>match</term> to the collation units of <code>$arg2</code> according to the\n             collation that is used.</p><note xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\"><p><term>Match</term> is defined in <bibref ref=\"Unicode-Collations\"/>.</p></note><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">A <phrase diff=\"add\" at=\"L\">dynamic</phrase> error <rfc2119>may</rfc2119> be raised <errorref class=\"CH\" code=\"0004\"/> if the\n             specified collation does not support collation units.</p></div>\n", "summary" : "<p>  Returns true if the string  $arg1  contains  $arg2  as a\n             leading substring, taking collations into account.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "arg1", "type" : "xs:string", "occurrence" : "?", "description" : "" }, { "name" : "arg2", "type" : "xs:string", "occurrence" : "?", "description" : "" } ], "returns" : { "type" : "xs:boolean", "description" : "" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 3, "name" : "starts-with", "qname" : "fn:starts-with", "signature" : "($arg1 as xs:string?, $arg2 as xs:string?, $collation as xs:string) as xs:boolean external", "description" : " <div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">Returns true if the string <code>$arg1</code> contains <code>$arg2</code> as a\n             leading substring, taking collations into account.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\"><example role=\"signature\"><proto name=\"starts-with\" return-type=\"xs:boolean\" isOp=\"no\" prefix=\"fn\" returnEmptyOk=\"no\" returnSeq=\"no\" returnVaries=\"no\" isSchema=\"no\" isDatatype=\"no\" isSpecial=\"no\"><arg name=\"arg1\" type=\"xs:string?\"/><arg name=\"arg2\" type=\"xs:string?\"/></proto></example><example role=\"signature\"><proto name=\"starts-with\" return-type=\"xs:boolean\" isOp=\"no\" prefix=\"fn\" returnEmptyOk=\"no\" returnSeq=\"no\" returnVaries=\"no\" isSchema=\"no\" isDatatype=\"no\" isSpecial=\"no\"><arg name=\"arg1\" type=\"xs:string?\"/><arg name=\"arg2\" type=\"xs:string?\"/><arg name=\"collation\" type=\"xs:string\"/></proto></example></p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The two-argument form of this function is <termref def=\"dt-deterministic\">deterministic</termref>, <termref def=\"dt-context-dependent\">context-dependent</termref>,  and <termref def=\"dt-focus-independent\">focus-independent</termref>.  It depends on\n \t\tcollations.\n \t</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The three-argument form of this function is <termref def=\"dt-deterministic\">deterministic</termref>, <termref def=\"dt-context-dependent\">context-dependent</termref>,  and <termref def=\"dt-focus-independent\">focus-independent</termref>.  It depends on\n \t\tcollations, and static base uri.\n \t</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">If the value of <code>$arg1</code> or <code>$arg2</code> is the empty sequence, or\n             contains only ignorable collation units, it is interpreted as the zero-length\n             string.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">If the value of <code>$arg2</code> is the zero-length string, then the function returns\n                <code>true</code>. If the value of <code>$arg1</code> is the zero-length string and\n             the value of <code>$arg2</code> is not the zero-length string, then the function returns\n                <code>false</code>.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The collation used by this function is determined according to the rules in <specref ref=\"choosing-a-collation\"/>.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The function returns an <code>xs:boolean</code> indicating whether or not the value of\n                <code>$arg1</code> starts with a sequence of collation units that provides a\n                <term>match</term> to the collation units of <code>$arg2</code> according to the\n             collation that is used.</p><note xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\"><p><term>Match</term> is defined in <bibref ref=\"Unicode-Collations\"/>.</p></note><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">A <phrase diff=\"add\" at=\"L\">dynamic</phrase> error <rfc2119>may</rfc2119> be raised <errorref class=\"CH\" code=\"0004\"/> if the\n             specified collation does not support collation units.</p></div>\n", "summary" : "<p>  Returns true if the string  $arg1  contains  $arg2  as a\n             leading substring, taking collations into account.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "arg1", "type" : "xs:string", "occurrence" : "?", "description" : "" }, { "name" : "arg2", "type" : "xs:string", "occurrence" : "?", "description" : "" }, { "name" : "collation", "type" : "xs:string", "occurrence" : null, "description" : "" } ], "returns" : { "type" : "xs:boolean", "description" : "" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 0, "name" : "static-base-uri", "qname" : "fn:static-base-uri", "signature" : "() as xs:anyURI? external", "description" : " <div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">This function returns the value of the Static Base URI property from the static context.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\"><example role=\"signature\"><proto name=\"static-base-uri\" return-type=\"xs:anyURI?\" isOp=\"no\" prefix=\"fn\" returnEmptyOk=\"no\" returnSeq=\"no\" returnVaries=\"no\" isSchema=\"no\" isDatatype=\"no\" isSpecial=\"no\"/></example></p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">This function is <termref def=\"dt-deterministic\">deterministic</termref>, <termref def=\"dt-context-dependent\">context-dependent</termref>,  and <termref def=\"dt-focus-independent\">focus-independent</termref>.  It depends on\n \t\tstatic base uri.\n \t</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The function returns the value of the Static Base URI property from the static context. If the\n             property is absent, the empty sequence is returned.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">Components of the static context are discussed in <xspecref spec=\"XP30\" ref=\"static_context\"/> .</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\" diff=\"add\" at=\"L\">XQuery 3.0 and XSLT 3.0 give an implementation freedom to use different base URIs during the\n          static analysis phase and the dynamic evaluation phase, that is, for compile-time and run-time resources respectively.\n          In this situation, the <code>fn:static-base-uri</code> function should return a URI suitable for locating resources needed\n          during dynamic evaluation.</p></div>\n", "summary" : "<p>  This function returns the value of the Static Base URI property from the static context.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [  ], "returns" : { "type" : "xs:anyURI?", "description" : "" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "string-join", "qname" : "fn:string-join", "signature" : "($arg1 as xs:string*) as xs:string external", "description" : " <div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">Returns a string created by concatenating the items in a sequence, with a\n             defined separator between adjacent items.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\"><example role=\"signature\"><proto name=\"string-join\" return-type=\"xs:string\" isOp=\"no\" prefix=\"fn\" returnEmptyOk=\"no\" returnSeq=\"no\" returnVaries=\"no\" isSchema=\"no\" isDatatype=\"no\" isSpecial=\"no\"><arg name=\"arg1\" type=\"xs:string*\"/></proto></example><example role=\"signature\"><proto name=\"string-join\" return-type=\"xs:string\" isOp=\"no\" prefix=\"fn\" returnEmptyOk=\"no\" returnSeq=\"no\" returnVaries=\"no\" isSchema=\"no\" isDatatype=\"no\" isSpecial=\"no\"><arg name=\"arg1\" type=\"xs:string*\"/><arg name=\"arg2\" type=\"xs:string\"/></proto></example></p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">This function is <termref def=\"dt-deterministic\">deterministic</termref>, <termref def=\"dt-context-independent\">context-independent</termref>,  and <termref def=\"dt-focus-independent\">focus-independent</termref>. </p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\" diff=\"add\" at=\"B\">The effect of calling the single-argument version of this function is\n             the same as calling the two-argument version with <code>$arg2</code> set to a\n             zero-length string.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The function returns an <code>xs:string</code> created by concatenating the items in the\n             sequence <code>$arg1</code>, in order, using the value of <code>$arg2</code> as a\n             separator between adjacent items. If the value of <code>$arg2</code> is the zero-length\n             string, then the members of <code>$arg1</code> are concatenated without a separator.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">If the value of <code>$arg1</code> is the empty sequence, the function returns the\n             zero-length string.</p></div>\n", "summary" : "<p>  Returns a string created by concatenating the items in a sequence, with a\n             defined separator between adjacent items.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "arg1", "type" : "xs:string", "occurrence" : "*", "description" : "" } ], "returns" : { "type" : "xs:string", "description" : "" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 2, "name" : "string-join", "qname" : "fn:string-join", "signature" : "($arg1 as xs:string*, $arg2 as xs:string) as xs:string external", "description" : " <div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">Returns a string created by concatenating the items in a sequence, with a\n             defined separator between adjacent items.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\"><example role=\"signature\"><proto name=\"string-join\" return-type=\"xs:string\" isOp=\"no\" prefix=\"fn\" returnEmptyOk=\"no\" returnSeq=\"no\" returnVaries=\"no\" isSchema=\"no\" isDatatype=\"no\" isSpecial=\"no\"><arg name=\"arg1\" type=\"xs:string*\"/></proto></example><example role=\"signature\"><proto name=\"string-join\" return-type=\"xs:string\" isOp=\"no\" prefix=\"fn\" returnEmptyOk=\"no\" returnSeq=\"no\" returnVaries=\"no\" isSchema=\"no\" isDatatype=\"no\" isSpecial=\"no\"><arg name=\"arg1\" type=\"xs:string*\"/><arg name=\"arg2\" type=\"xs:string\"/></proto></example></p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">This function is <termref def=\"dt-deterministic\">deterministic</termref>, <termref def=\"dt-context-independent\">context-independent</termref>,  and <termref def=\"dt-focus-independent\">focus-independent</termref>. </p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\" diff=\"add\" at=\"B\">The effect of calling the single-argument version of this function is\n             the same as calling the two-argument version with <code>$arg2</code> set to a\n             zero-length string.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The function returns an <code>xs:string</code> created by concatenating the items in the\n             sequence <code>$arg1</code>, in order, using the value of <code>$arg2</code> as a\n             separator between adjacent items. If the value of <code>$arg2</code> is the zero-length\n             string, then the members of <code>$arg1</code> are concatenated without a separator.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">If the value of <code>$arg1</code> is the empty sequence, the function returns the\n             zero-length string.</p></div>\n", "summary" : "<p>  Returns a string created by concatenating the items in a sequence, with a\n             defined separator between adjacent items.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "arg1", "type" : "xs:string", "occurrence" : "*", "description" : "" }, { "name" : "arg2", "type" : "xs:string", "occurrence" : null, "description" : "" } ], "returns" : { "type" : "xs:string", "description" : "" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 0, "name" : "string-length", "qname" : "fn:string-length", "signature" : "() as xs:integer external", "description" : " <div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">Returns the number of <termref def=\"character\">characters</termref> in a\n             string.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\"><example role=\"signature\"><proto name=\"string-length\" return-type=\"xs:integer\" isOp=\"no\" prefix=\"fn\" returnEmptyOk=\"no\" returnSeq=\"no\" returnVaries=\"no\" isSchema=\"no\" isDatatype=\"no\" isSpecial=\"no\"/></example><example role=\"signature\"><proto name=\"string-length\" return-type=\"xs:integer\" isOp=\"no\" prefix=\"fn\" returnEmptyOk=\"no\" returnSeq=\"no\" returnVaries=\"no\" isSchema=\"no\" isDatatype=\"no\" isSpecial=\"no\"><arg name=\"arg\" type=\"xs:string?\"/></proto></example></p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The zero-argument form of this function is <termref def=\"dt-deterministic\">deterministic</termref>, <termref def=\"dt-context-dependent\">context-dependent</termref>,  and <termref def=\"dt-focus-dependent\">focus-dependent</termref>. </p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The one-argument form of this function is <termref def=\"dt-deterministic\">deterministic</termref>, <termref def=\"dt-context-independent\">context-independent</termref>,  and <termref def=\"dt-focus-independent\">focus-independent</termref>. </p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The function returns an <code>xs:integer</code> equal to the length in <termref def=\"character\">characters</termref> of the value of <code>$arg</code>.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">Calling the zero-argument version of the function is equivalent to calling\n                <code>fn:string-length(fn:string(.))</code>.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">If the value of <code>$arg</code> is the empty sequence, the function returns the\n                <code>xs:integer</code> value zero (0).</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">If <code>$arg</code> is not specified and the context item is <xtermref ref=\"dt-absent\" spec=\"DM30\">absent</xtermref>, a <phrase diff=\"add\" at=\"M\">dynamic</phrase> error is raised: <xerrorref spec=\"XP\" class=\"DY\" code=\"0002\" type=\"dynamic\"/>.</p></div>\n", "summary" : "<p>  Returns the number of  characters  in a\n             string.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [  ], "returns" : { "type" : "xs:integer", "description" : "" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "string-length", "qname" : "fn:string-length", "signature" : "($arg as xs:string?) as xs:integer external", "description" : " <div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">Returns the number of <termref def=\"character\">characters</termref> in a\n             string.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\"><example role=\"signature\"><proto name=\"string-length\" return-type=\"xs:integer\" isOp=\"no\" prefix=\"fn\" returnEmptyOk=\"no\" returnSeq=\"no\" returnVaries=\"no\" isSchema=\"no\" isDatatype=\"no\" isSpecial=\"no\"/></example><example role=\"signature\"><proto name=\"string-length\" return-type=\"xs:integer\" isOp=\"no\" prefix=\"fn\" returnEmptyOk=\"no\" returnSeq=\"no\" returnVaries=\"no\" isSchema=\"no\" isDatatype=\"no\" isSpecial=\"no\"><arg name=\"arg\" type=\"xs:string?\"/></proto></example></p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The zero-argument form of this function is <termref def=\"dt-deterministic\">deterministic</termref>, <termref def=\"dt-context-dependent\">context-dependent</termref>,  and <termref def=\"dt-focus-dependent\">focus-dependent</termref>. </p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The one-argument form of this function is <termref def=\"dt-deterministic\">deterministic</termref>, <termref def=\"dt-context-independent\">context-independent</termref>,  and <termref def=\"dt-focus-independent\">focus-independent</termref>. </p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The function returns an <code>xs:integer</code> equal to the length in <termref def=\"character\">characters</termref> of the value of <code>$arg</code>.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">Calling the zero-argument version of the function is equivalent to calling\n                <code>fn:string-length(fn:string(.))</code>.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">If the value of <code>$arg</code> is the empty sequence, the function returns the\n                <code>xs:integer</code> value zero (0).</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">If <code>$arg</code> is not specified and the context item is <xtermref ref=\"dt-absent\" spec=\"DM30\">absent</xtermref>, a <phrase diff=\"add\" at=\"M\">dynamic</phrase> error is raised: <xerrorref spec=\"XP\" class=\"DY\" code=\"0002\" type=\"dynamic\"/>.</p></div>\n", "summary" : "<p>  Returns the number of  characters  in a\n             string.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "arg", "type" : "xs:string", "occurrence" : "?", "description" : "" } ], "returns" : { "type" : "xs:integer", "description" : "" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "string-to-codepoints", "qname" : "fn:string-to-codepoints", "signature" : "($arg as xs:string?) as xs:integer* external", "description" : " <div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">Returns the sequence of <termref def=\"codepoint\">codepoints</termref> that\n             constitute an <code>xs:string</code> value. </p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\"><example role=\"signature\"><proto name=\"string-to-codepoints\" return-type=\"xs:integer*\" isOp=\"no\" prefix=\"fn\" returnEmptyOk=\"no\" returnSeq=\"no\" returnVaries=\"no\" isSchema=\"no\" isDatatype=\"no\" isSpecial=\"no\"><arg name=\"arg\" type=\"xs:string?\"/></proto></example></p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">This function is <termref def=\"dt-deterministic\">deterministic</termref>, <termref def=\"dt-context-independent\">context-independent</termref>,  and <termref def=\"dt-focus-independent\">focus-independent</termref>. </p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The function returns a sequence of integers, each integer being the Unicode <termref def=\"codepoint\">codepoints</termref> of the corresponding <termref def=\"character\">character</termref> in <code>$arg</code>.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">If <code>$arg</code> is a zero-length string or the empty sequence, the function returns\n             the empty sequence.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The expression <code>fn:string-to-codepoints(\"Thérèse\")</code> returns <code>(84, 104, 233, 114, 232, 115, 101)</code>.</p></div>\n", "summary" : "<p>  Returns the sequence of  codepoints  that\n             constitute an  xs:string  value.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "arg", "type" : "xs:string", "occurrence" : "?", "description" : "" } ], "returns" : { "type" : "xs:integer*", "description" : "" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "string-to-codepoints", "qname" : "fn:string-to-codepoints", "signature" : "($arg as xs:string?) as xs:integer* external", "description" : " <div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">Returns the sequence of <termref def=\"codepoint\">codepoints</termref> that\n             constitute an <code>xs:string</code> value. </p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\"><example role=\"signature\"><proto name=\"string-to-codepoints\" return-type=\"xs:integer*\" isOp=\"no\" prefix=\"fn\" returnEmptyOk=\"no\" returnSeq=\"no\" returnVaries=\"no\" isSchema=\"no\" isDatatype=\"no\" isSpecial=\"no\"><arg name=\"arg\" type=\"xs:string?\"/></proto></example></p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">This function is <termref def=\"dt-deterministic\">deterministic</termref>, <termref def=\"dt-context-independent\">context-independent</termref>,  and <termref def=\"dt-focus-independent\">focus-independent</termref>. </p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The function returns a sequence of integers, each integer being the Unicode <termref def=\"codepoint\">codepoints</termref> of the corresponding <termref def=\"character\">character</termref> in <code>$arg</code>.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">If <code>$arg</code> is a zero-length string or the empty sequence, the function returns\n             the empty sequence.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The expression <code>fn:string-to-codepoints(\"Thérèse\")</code> returns <code>(84, 104, 233, 114, 232, 115, 101)</code>.</p></div>\n", "summary" : "<p>  Returns the sequence of  codepoints  that\n             constitute an  xs:string  value.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "arg", "type" : "xs:string", "occurrence" : "?", "description" : "" } ], "returns" : { "type" : "xs:integer*", "description" : "" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 0, "name" : "string", "qname" : "fn:string", "signature" : "() as xs:string external", "description" : " <div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">Returns the value of <code>$arg</code> represented as an\n             <code>xs:string</code>.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\"><example role=\"signature\"><proto name=\"string\" return-type=\"xs:string\" isOp=\"no\" prefix=\"fn\" returnEmptyOk=\"no\" returnSeq=\"no\" returnVaries=\"no\" isSchema=\"no\" isDatatype=\"no\" isSpecial=\"no\"/></example><example role=\"signature\"><proto name=\"string\" return-type=\"xs:string\" isOp=\"no\" prefix=\"fn\" returnEmptyOk=\"no\" returnSeq=\"no\" returnVaries=\"no\" isSchema=\"no\" isDatatype=\"no\" isSpecial=\"no\"><arg name=\"arg\" type=\"item()?\"/></proto></example></p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The zero-argument form of this function is <termref def=\"dt-deterministic\">deterministic</termref>, <termref def=\"dt-context-dependent\">context-dependent</termref>,  and <termref def=\"dt-focus-dependent\">focus-dependent</termref>. </p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The one-argument form of this function is <termref def=\"dt-deterministic\">deterministic</termref>, <termref def=\"dt-context-independent\">context-independent</termref>,  and <termref def=\"dt-focus-independent\">focus-independent</termref>. </p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">In the zero-argument version of the function, <code>$arg</code> defaults to the context\n             item. That is, calling <code>fn:string()</code> is equivalent to calling\n                <code>fn:string(.)</code>.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">If <code>$arg</code> is the empty sequence, the function returns the zero-length\n             string.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">If <code>$arg</code> is a node, the function returns the string-value of the node, as\n             obtained using the <code>dm:string-value</code> accessor defined in <bibref ref=\"xpath-datamodel-30\"/> (see <xspecref spec=\"DM30\" ref=\"dm-string-value\"/>).</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">If <code>$arg</code> is an atomic value, the function returns the result of the\n             expression <code>$arg cast as xs:string</code> (see <specref ref=\"casting\"/>).</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">A <phrase diff=\"add\" at=\"M\">dynamic</phrase> error is raised <xerrorref spec=\"XP\" class=\"DY\" code=\"0002\" type=\"type\"/> by the\n             zero-argument version of the function if the context item is <xtermref ref=\"dt-absent\" spec=\"DM30\">absent</xtermref>. </p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">A <phrase diff=\"add\" at=\"L\">type</phrase> error is raised <errorref class=\"TY\" code=\"0014\" type=\"type\"/> if\n                <code>$arg</code> is a function item. </p></div>\n", "summary" : "<p>  Returns the value of  $arg  represented as an\n              xs:string .</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [  ], "returns" : { "type" : "xs:string", "description" : "" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "string", "qname" : "fn:string", "signature" : "($arg as item()?) as xs:string external", "description" : " <div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">Returns the value of <code>$arg</code> represented as an\n             <code>xs:string</code>.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\"><example role=\"signature\"><proto name=\"string\" return-type=\"xs:string\" isOp=\"no\" prefix=\"fn\" returnEmptyOk=\"no\" returnSeq=\"no\" returnVaries=\"no\" isSchema=\"no\" isDatatype=\"no\" isSpecial=\"no\"/></example><example role=\"signature\"><proto name=\"string\" return-type=\"xs:string\" isOp=\"no\" prefix=\"fn\" returnEmptyOk=\"no\" returnSeq=\"no\" returnVaries=\"no\" isSchema=\"no\" isDatatype=\"no\" isSpecial=\"no\"><arg name=\"arg\" type=\"item()?\"/></proto></example></p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The zero-argument form of this function is <termref def=\"dt-deterministic\">deterministic</termref>, <termref def=\"dt-context-dependent\">context-dependent</termref>,  and <termref def=\"dt-focus-dependent\">focus-dependent</termref>. </p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The one-argument form of this function is <termref def=\"dt-deterministic\">deterministic</termref>, <termref def=\"dt-context-independent\">context-independent</termref>,  and <termref def=\"dt-focus-independent\">focus-independent</termref>. </p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">In the zero-argument version of the function, <code>$arg</code> defaults to the context\n             item. That is, calling <code>fn:string()</code> is equivalent to calling\n                <code>fn:string(.)</code>.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">If <code>$arg</code> is the empty sequence, the function returns the zero-length\n             string.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">If <code>$arg</code> is a node, the function returns the string-value of the node, as\n             obtained using the <code>dm:string-value</code> accessor defined in <bibref ref=\"xpath-datamodel-30\"/> (see <xspecref spec=\"DM30\" ref=\"dm-string-value\"/>).</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">If <code>$arg</code> is an atomic value, the function returns the result of the\n             expression <code>$arg cast as xs:string</code> (see <specref ref=\"casting\"/>).</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">A <phrase diff=\"add\" at=\"M\">dynamic</phrase> error is raised <xerrorref spec=\"XP\" class=\"DY\" code=\"0002\" type=\"type\"/> by the\n             zero-argument version of the function if the context item is <xtermref ref=\"dt-absent\" spec=\"DM30\">absent</xtermref>. </p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">A <phrase diff=\"add\" at=\"L\">type</phrase> error is raised <errorref class=\"TY\" code=\"0014\" type=\"type\"/> if\n                <code>$arg</code> is a function item. </p></div>\n", "summary" : "<p>  Returns the value of  $arg  represented as an\n              xs:string .</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "arg", "type" : "item()", "occurrence" : "?", "description" : "" } ], "returns" : { "type" : "xs:string", "description" : "" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 2, "name" : "subsequence", "qname" : "fn:subsequence", "signature" : "($sourceSeq as item()*, $startingLoc as xs:double) as item()* external", "description" : " <div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">Returns the contiguous sequence of items in the value of\n                <code>$sourceSeq</code> beginning at the position indicated by the value of\n                <code>$startingLoc</code> and continuing for the number of items indicated by the\n             value of <code>$length</code>. </p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\"><example role=\"signature\"><proto name=\"subsequence\" return-type=\"item()*\" isOp=\"no\" prefix=\"fn\" returnEmptyOk=\"no\" returnSeq=\"no\" returnVaries=\"no\" isSchema=\"no\" isDatatype=\"no\" isSpecial=\"no\"><arg name=\"sourceSeq\" type=\"item()*\"/><arg name=\"startingLoc\" type=\"xs:double\"/></proto></example><example role=\"signature\"><proto name=\"subsequence\" return-type=\"item()*\" isOp=\"no\" prefix=\"fn\" returnEmptyOk=\"no\" returnSeq=\"no\" returnVaries=\"no\" isSchema=\"no\" isDatatype=\"no\" isSpecial=\"no\"><arg name=\"sourceSeq\" type=\"item()*\"/><arg name=\"startingLoc\" type=\"xs:double\"/><arg name=\"length\" type=\"xs:double\"/></proto></example></p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">This function is <termref def=\"dt-deterministic\">deterministic</termref>, <termref def=\"dt-context-independent\">context-independent</termref>,  and <termref def=\"dt-focus-independent\">focus-independent</termref>. </p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">In the two-argument case, returns:</p><eg xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\" diff=\"chg\" at=\"A-E2\" xml:space=\"preserve\">$sourceSeq[fn:round($startingLoc) le position()]</eg><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">In the three-argument case, returns:</p><eg xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\" diff=\"chg\" at=\"A-E2\" xml:space=\"preserve\">$sourceSeq[fn:round($startingLoc) le position()\n          and position() lt fn:round($startingLoc) + fn:round($length)]</eg><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\" diff=\"del\" at=\"F\">For detailed type semantics, see [Formal Semantics].</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The first item of a sequence is located at position 1, not position 0.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">If <code>$sourceSeq</code> is the empty sequence, the empty sequence is returned.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\"> If <code>$startingLoc</code> is zero or negative, the subsequence includes items from\n             the beginning of the <code>$sourceSeq</code>.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\"> If <code>$length</code> is not specified, the subsequence includes items to the end of\n                <code>$sourceSeq</code>.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\"> If <code>$length</code> is greater than the number of items in the value of\n                <code>$sourceSeq</code> following <code>$startingLoc</code>, the subsequence includes\n             items to the end of <code>$sourceSeq</code>.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\" diff=\"add\" at=\"F\">As an exception to the previous two notes, if\n                <code>$startingLoc</code> is <code>-INF</code> and <code>$length</code> is\n                <code>+INF</code>, then <code>fn:round($startingLoc) + fn:round($length)</code> is\n                <code>NaN</code>; since <code>position() lt NaN</code> is always false, the result is\n             an empty sequence.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The reason the function accepts arguments of type <code>xs:double</code> is that many\n             computations on untyped data return an <code>xs:double</code> result; and the reason for\n             the rounding rules is to compensate for any imprecision in these floating-point\n             computations.</p></div>\n", "summary" : "<p>  Returns the contiguous sequence of items in the value of\n                 $sourceSeq  beginning at the position indicated by the value of\n                 $startingLoc  and continuing for the number of items indicated by the\n             value of  $length .</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "sourceSeq", "type" : "item()", "occurrence" : "*", "description" : "" }, { "name" : "startingLoc", "type" : "xs:double", "occurrence" : null, "description" : "" } ], "returns" : { "type" : "item()*", "description" : "" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 3, "name" : "subsequence", "qname" : "fn:subsequence", "signature" : "($sourceSeq as item()*, $startingLoc as xs:double, $length as xs:double) as item()* external", "description" : " <div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">Returns the contiguous sequence of items in the value of\n                <code>$sourceSeq</code> beginning at the position indicated by the value of\n                <code>$startingLoc</code> and continuing for the number of items indicated by the\n             value of <code>$length</code>. </p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\"><example role=\"signature\"><proto name=\"subsequence\" return-type=\"item()*\" isOp=\"no\" prefix=\"fn\" returnEmptyOk=\"no\" returnSeq=\"no\" returnVaries=\"no\" isSchema=\"no\" isDatatype=\"no\" isSpecial=\"no\"><arg name=\"sourceSeq\" type=\"item()*\"/><arg name=\"startingLoc\" type=\"xs:double\"/></proto></example><example role=\"signature\"><proto name=\"subsequence\" return-type=\"item()*\" isOp=\"no\" prefix=\"fn\" returnEmptyOk=\"no\" returnSeq=\"no\" returnVaries=\"no\" isSchema=\"no\" isDatatype=\"no\" isSpecial=\"no\"><arg name=\"sourceSeq\" type=\"item()*\"/><arg name=\"startingLoc\" type=\"xs:double\"/><arg name=\"length\" type=\"xs:double\"/></proto></example></p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">This function is <termref def=\"dt-deterministic\">deterministic</termref>, <termref def=\"dt-context-independent\">context-independent</termref>,  and <termref def=\"dt-focus-independent\">focus-independent</termref>. </p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">In the two-argument case, returns:</p><eg xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\" diff=\"chg\" at=\"A-E2\" xml:space=\"preserve\">$sourceSeq[fn:round($startingLoc) le position()]</eg><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">In the three-argument case, returns:</p><eg xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\" diff=\"chg\" at=\"A-E2\" xml:space=\"preserve\">$sourceSeq[fn:round($startingLoc) le position()\n          and position() lt fn:round($startingLoc) + fn:round($length)]</eg><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\" diff=\"del\" at=\"F\">For detailed type semantics, see [Formal Semantics].</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The first item of a sequence is located at position 1, not position 0.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">If <code>$sourceSeq</code> is the empty sequence, the empty sequence is returned.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\"> If <code>$startingLoc</code> is zero or negative, the subsequence includes items from\n             the beginning of the <code>$sourceSeq</code>.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\"> If <code>$length</code> is not specified, the subsequence includes items to the end of\n                <code>$sourceSeq</code>.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\"> If <code>$length</code> is greater than the number of items in the value of\n                <code>$sourceSeq</code> following <code>$startingLoc</code>, the subsequence includes\n             items to the end of <code>$sourceSeq</code>.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\" diff=\"add\" at=\"F\">As an exception to the previous two notes, if\n                <code>$startingLoc</code> is <code>-INF</code> and <code>$length</code> is\n                <code>+INF</code>, then <code>fn:round($startingLoc) + fn:round($length)</code> is\n                <code>NaN</code>; since <code>position() lt NaN</code> is always false, the result is\n             an empty sequence.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The reason the function accepts arguments of type <code>xs:double</code> is that many\n             computations on untyped data return an <code>xs:double</code> result; and the reason for\n             the rounding rules is to compensate for any imprecision in these floating-point\n             computations.</p></div>\n", "summary" : "<p>  Returns the contiguous sequence of items in the value of\n                 $sourceSeq  beginning at the position indicated by the value of\n                 $startingLoc  and continuing for the number of items indicated by the\n             value of  $length .</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "sourceSeq", "type" : "item()", "occurrence" : "*", "description" : "" }, { "name" : "startingLoc", "type" : "xs:double", "occurrence" : null, "description" : "" }, { "name" : "length", "type" : "xs:double", "occurrence" : null, "description" : "" } ], "returns" : { "type" : "item()*", "description" : "" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 2, "name" : "substring-after", "qname" : "fn:substring-after", "signature" : "($arg1 as xs:string?, $arg2 as xs:string?) as xs:string external", "description" : " <div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">Returns the part of <code>$arg1</code> that follows the first occurrence of\n                <code>$arg2</code>, taking collations into account.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\"><example role=\"signature\"><proto name=\"substring-after\" return-type=\"xs:string\" isOp=\"no\" prefix=\"fn\" returnEmptyOk=\"no\" returnSeq=\"no\" returnVaries=\"no\" isSchema=\"no\" isDatatype=\"no\" isSpecial=\"no\"><arg name=\"arg1\" type=\"xs:string?\"/><arg name=\"arg2\" type=\"xs:string?\"/></proto></example><example role=\"signature\"><proto name=\"substring-after\" return-type=\"xs:string\" isOp=\"no\" prefix=\"fn\" returnEmptyOk=\"no\" returnSeq=\"no\" returnVaries=\"no\" isSchema=\"no\" isDatatype=\"no\" isSpecial=\"no\"><arg name=\"arg1\" type=\"xs:string?\"/><arg name=\"arg2\" type=\"xs:string?\"/><arg name=\"collation\" type=\"xs:string\"/></proto></example></p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The two-argument form of this function is <termref def=\"dt-deterministic\">deterministic</termref>, <termref def=\"dt-context-dependent\">context-dependent</termref>,  and <termref def=\"dt-focus-independent\">focus-independent</termref>.  It depends on\n \t\tcollations.\n \t</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The three-argument form of this function is <termref def=\"dt-deterministic\">deterministic</termref>, <termref def=\"dt-context-dependent\">context-dependent</termref>,  and <termref def=\"dt-focus-independent\">focus-independent</termref>.  It depends on\n \t\tcollations, and static base uri.\n \t</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">If the value of <code>$arg1</code> or <code>$arg2</code> is the empty sequence, or\n             contains only ignorable collation units, it is interpreted as the zero-length\n             string.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">If the value of <code>$arg2</code> is the zero-length string, then the function returns\n             the value of <code>$arg1</code>.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">If the value of <code>$arg1</code> does not contain a string that is equal to the value\n             of <code>$arg2</code>, then the function returns the zero-length string. </p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The collation used by this function is determined according to the rules in <specref ref=\"choosing-a-collation\"/>.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The function returns the substring of the value of <code>$arg1</code> that follows in\n             the value of <code>$arg1</code> the first occurrence of a sequence of collation units\n             that provides a <term>minimal match</term> to the collation units of <code>$arg2</code>\n             according to the collation that is used. </p><note xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\"><p><term>Minimal match</term> is defined in <bibref ref=\"Unicode-Collations\"/>. </p></note><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">A <phrase diff=\"add\" at=\"L\">dynamic</phrase> error <rfc2119>may</rfc2119> be raised <errorref class=\"CH\" code=\"0004\"/> if the\n             specified collation does not support collation units.</p></div>\n", "summary" : "<p>  Returns the part of  $arg1  that follows the first occurrence of\n                 $arg2 , taking collations into account.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "arg1", "type" : "xs:string", "occurrence" : "?", "description" : "" }, { "name" : "arg2", "type" : "xs:string", "occurrence" : "?", "description" : "" } ], "returns" : { "type" : "xs:string", "description" : "" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 3, "name" : "substring-after", "qname" : "fn:substring-after", "signature" : "($arg1 as xs:string?, $arg2 as xs:string?, $collation as xs:string) as xs:string external", "description" : " <div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">Returns the part of <code>$arg1</code> that follows the first occurrence of\n                <code>$arg2</code>, taking collations into account.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\"><example role=\"signature\"><proto name=\"substring-after\" return-type=\"xs:string\" isOp=\"no\" prefix=\"fn\" returnEmptyOk=\"no\" returnSeq=\"no\" returnVaries=\"no\" isSchema=\"no\" isDatatype=\"no\" isSpecial=\"no\"><arg name=\"arg1\" type=\"xs:string?\"/><arg name=\"arg2\" type=\"xs:string?\"/></proto></example><example role=\"signature\"><proto name=\"substring-after\" return-type=\"xs:string\" isOp=\"no\" prefix=\"fn\" returnEmptyOk=\"no\" returnSeq=\"no\" returnVaries=\"no\" isSchema=\"no\" isDatatype=\"no\" isSpecial=\"no\"><arg name=\"arg1\" type=\"xs:string?\"/><arg name=\"arg2\" type=\"xs:string?\"/><arg name=\"collation\" type=\"xs:string\"/></proto></example></p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The two-argument form of this function is <termref def=\"dt-deterministic\">deterministic</termref>, <termref def=\"dt-context-dependent\">context-dependent</termref>,  and <termref def=\"dt-focus-independent\">focus-independent</termref>.  It depends on\n \t\tcollations.\n \t</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The three-argument form of this function is <termref def=\"dt-deterministic\">deterministic</termref>, <termref def=\"dt-context-dependent\">context-dependent</termref>,  and <termref def=\"dt-focus-independent\">focus-independent</termref>.  It depends on\n \t\tcollations, and static base uri.\n \t</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">If the value of <code>$arg1</code> or <code>$arg2</code> is the empty sequence, or\n             contains only ignorable collation units, it is interpreted as the zero-length\n             string.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">If the value of <code>$arg2</code> is the zero-length string, then the function returns\n             the value of <code>$arg1</code>.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">If the value of <code>$arg1</code> does not contain a string that is equal to the value\n             of <code>$arg2</code>, then the function returns the zero-length string. </p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The collation used by this function is determined according to the rules in <specref ref=\"choosing-a-collation\"/>.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The function returns the substring of the value of <code>$arg1</code> that follows in\n             the value of <code>$arg1</code> the first occurrence of a sequence of collation units\n             that provides a <term>minimal match</term> to the collation units of <code>$arg2</code>\n             according to the collation that is used. </p><note xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\"><p><term>Minimal match</term> is defined in <bibref ref=\"Unicode-Collations\"/>. </p></note><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">A <phrase diff=\"add\" at=\"L\">dynamic</phrase> error <rfc2119>may</rfc2119> be raised <errorref class=\"CH\" code=\"0004\"/> if the\n             specified collation does not support collation units.</p></div>\n", "summary" : "<p>  Returns the part of  $arg1  that follows the first occurrence of\n                 $arg2 , taking collations into account.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "arg1", "type" : "xs:string", "occurrence" : "?", "description" : "" }, { "name" : "arg2", "type" : "xs:string", "occurrence" : "?", "description" : "" }, { "name" : "collation", "type" : "xs:string", "occurrence" : null, "description" : "" } ], "returns" : { "type" : "xs:string", "description" : "" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 2, "name" : "substring-before", "qname" : "fn:substring-before", "signature" : "($arg1 as xs:string?, $arg2 as xs:string?) as xs:string external", "description" : " <div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">Returns the part of <code>$arg1</code> that precedes the first occurrence of\n                <code>$arg2</code>, taking collations into account.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\"><example role=\"signature\"><proto name=\"substring-before\" return-type=\"xs:string\" isOp=\"no\" prefix=\"fn\" returnEmptyOk=\"no\" returnSeq=\"no\" returnVaries=\"no\" isSchema=\"no\" isDatatype=\"no\" isSpecial=\"no\"><arg name=\"arg1\" type=\"xs:string?\"/><arg name=\"arg2\" type=\"xs:string?\"/></proto></example><example role=\"signature\"><proto name=\"substring-before\" return-type=\"xs:string\" isOp=\"no\" prefix=\"fn\" returnEmptyOk=\"no\" returnSeq=\"no\" returnVaries=\"no\" isSchema=\"no\" isDatatype=\"no\" isSpecial=\"no\"><arg name=\"arg1\" type=\"xs:string?\"/><arg name=\"arg2\" type=\"xs:string?\"/><arg name=\"collation\" type=\"xs:string\"/></proto></example></p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The two-argument form of this function is <termref def=\"dt-deterministic\">deterministic</termref>, <termref def=\"dt-context-dependent\">context-dependent</termref>,  and <termref def=\"dt-focus-independent\">focus-independent</termref>.  It depends on\n \t\tcollations.\n \t</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The three-argument form of this function is <termref def=\"dt-deterministic\">deterministic</termref>, <termref def=\"dt-context-dependent\">context-dependent</termref>,  and <termref def=\"dt-focus-independent\">focus-independent</termref>.  It depends on\n \t\tcollations, and static base uri.\n \t</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\"> If the value of <code>$arg1</code> or <code>$arg2</code> is the empty sequence, or\n             contains only ignorable collation units, it is interpreted as the zero-length\n             string.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">If the value of <code>$arg2</code> is the zero-length string, then the function returns\n             the zero-length string. </p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">If the value of <code>$arg1</code> does not contain a string that is equal to the value\n             of <code>$arg2</code>, then the function returns the zero-length string. </p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The collation used by this function is determined according to the rules in <specref ref=\"choosing-a-collation\"/>.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The function returns the substring of the value of <code>$arg1</code> that precedes in\n             the value of <code>$arg1</code> the first occurrence of a sequence of collation units\n             that provides a <term>minimal match</term> to the collation units of <code>$arg2</code>\n             according to the collation that is used.</p><note xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\"><p><term>Minimal match</term> is defined in <bibref ref=\"Unicode-Collations\"/>. </p></note><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">A <phrase diff=\"add\" at=\"L\">dynamic</phrase> error <rfc2119>may</rfc2119> be raised <errorref class=\"CH\" code=\"0004\"/> if the\n             specified collation does not support collation units.</p></div>\n", "summary" : "<p>  Returns the part of  $arg1  that precedes the first occurrence of\n                 $arg2 , taking collations into account.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "arg1", "type" : "xs:string", "occurrence" : "?", "description" : "" }, { "name" : "arg2", "type" : "xs:string", "occurrence" : "?", "description" : "" } ], "returns" : { "type" : "xs:string", "description" : "" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 3, "name" : "substring-before", "qname" : "fn:substring-before", "signature" : "($arg1 as xs:string?, $arg2 as xs:string?, $collation as xs:string) as xs:string external", "description" : " <div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">Returns the part of <code>$arg1</code> that precedes the first occurrence of\n                <code>$arg2</code>, taking collations into account.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\"><example role=\"signature\"><proto name=\"substring-before\" return-type=\"xs:string\" isOp=\"no\" prefix=\"fn\" returnEmptyOk=\"no\" returnSeq=\"no\" returnVaries=\"no\" isSchema=\"no\" isDatatype=\"no\" isSpecial=\"no\"><arg name=\"arg1\" type=\"xs:string?\"/><arg name=\"arg2\" type=\"xs:string?\"/></proto></example><example role=\"signature\"><proto name=\"substring-before\" return-type=\"xs:string\" isOp=\"no\" prefix=\"fn\" returnEmptyOk=\"no\" returnSeq=\"no\" returnVaries=\"no\" isSchema=\"no\" isDatatype=\"no\" isSpecial=\"no\"><arg name=\"arg1\" type=\"xs:string?\"/><arg name=\"arg2\" type=\"xs:string?\"/><arg name=\"collation\" type=\"xs:string\"/></proto></example></p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The two-argument form of this function is <termref def=\"dt-deterministic\">deterministic</termref>, <termref def=\"dt-context-dependent\">context-dependent</termref>,  and <termref def=\"dt-focus-independent\">focus-independent</termref>.  It depends on\n \t\tcollations.\n \t</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The three-argument form of this function is <termref def=\"dt-deterministic\">deterministic</termref>, <termref def=\"dt-context-dependent\">context-dependent</termref>,  and <termref def=\"dt-focus-independent\">focus-independent</termref>.  It depends on\n \t\tcollations, and static base uri.\n \t</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\"> If the value of <code>$arg1</code> or <code>$arg2</code> is the empty sequence, or\n             contains only ignorable collation units, it is interpreted as the zero-length\n             string.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">If the value of <code>$arg2</code> is the zero-length string, then the function returns\n             the zero-length string. </p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">If the value of <code>$arg1</code> does not contain a string that is equal to the value\n             of <code>$arg2</code>, then the function returns the zero-length string. </p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The collation used by this function is determined according to the rules in <specref ref=\"choosing-a-collation\"/>.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The function returns the substring of the value of <code>$arg1</code> that precedes in\n             the value of <code>$arg1</code> the first occurrence of a sequence of collation units\n             that provides a <term>minimal match</term> to the collation units of <code>$arg2</code>\n             according to the collation that is used.</p><note xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\"><p><term>Minimal match</term> is defined in <bibref ref=\"Unicode-Collations\"/>. </p></note><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">A <phrase diff=\"add\" at=\"L\">dynamic</phrase> error <rfc2119>may</rfc2119> be raised <errorref class=\"CH\" code=\"0004\"/> if the\n             specified collation does not support collation units.</p></div>\n", "summary" : "<p>  Returns the part of  $arg1  that precedes the first occurrence of\n                 $arg2 , taking collations into account.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "arg1", "type" : "xs:string", "occurrence" : "?", "description" : "" }, { "name" : "arg2", "type" : "xs:string", "occurrence" : "?", "description" : "" }, { "name" : "collation", "type" : "xs:string", "occurrence" : null, "description" : "" } ], "returns" : { "type" : "xs:string", "description" : "" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 2, "name" : "substring", "qname" : "fn:substring", "signature" : "($sourceString as xs:string?, $start as xs:double) as xs:string external", "description" : " <div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">Returns the portion of the value of <code>$sourceString</code> beginning at the\n             position indicated by the value of <code>$start</code> and continuing for the number of\n                <termref def=\"character\">characters</termref> indicated by the value of\n                <code>$length</code>.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\"><example role=\"signature\"><proto name=\"substring\" return-type=\"xs:string\" isOp=\"no\" prefix=\"fn\" returnEmptyOk=\"no\" returnSeq=\"no\" returnVaries=\"no\" isSchema=\"no\" isDatatype=\"no\" isSpecial=\"no\"><arg name=\"sourceString\" type=\"xs:string?\"/><arg name=\"start\" type=\"xs:double\"/></proto></example><example role=\"signature\"><proto name=\"substring\" return-type=\"xs:string\" isOp=\"no\" prefix=\"fn\" returnEmptyOk=\"no\" returnSeq=\"no\" returnVaries=\"no\" isSchema=\"no\" isDatatype=\"no\" isSpecial=\"no\"><arg name=\"sourceString\" type=\"xs:string?\"/><arg name=\"start\" type=\"xs:double\"/><arg name=\"length\" type=\"xs:double\"/></proto></example></p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">This function is <termref def=\"dt-deterministic\">deterministic</termref>, <termref def=\"dt-context-independent\">context-independent</termref>,  and <termref def=\"dt-focus-independent\">focus-independent</termref>. </p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">If the value of <code>$sourceString</code> is the empty sequence, the function returns\n             the zero-length string. </p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">Otherwise, the function returns a string comprising those <termref def=\"character\">characters</termref> of <code>$sourceString</code> whose index position (counting\n             from one) is greater than or equal to the value of <code>$start</code> (rounded to an\n             integer), and (if <code>$length</code> is specified) less than the sum of\n                <code>$start</code> and <code>$length</code> (both rounded to integers).</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The characters returned do not extend beyond <code>$sourceString</code>. If\n                <code>$start</code> is zero or negative, only those characters in positions greater\n             than zero are returned.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">More specifically, the three argument version of the function returns the characters in\n                <code>$sourceString</code> whose position <code>$p</code> satisfies:</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">\n             <code>fn:round($start) &lt;= $p &lt; fn:round($start) + fn:round($length)</code>\n          </p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The two argument version of the function assumes that <code>$length</code> is infinite\n             and thus returns the <termref def=\"character\">characters</termref> in\n                <code>$sourceString</code> whose position <code>$p</code> satisfies:</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">\n             <code>fn:round($start) &lt;= $p</code>\n          </p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">In the above computations, the rules for <code>op:numeric-less-than</code> and\n                <code>op:numeric-greater-than</code> apply.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The first character of a string is located at position 1, not position 0.</p></div>\n", "summary" : "<p>  Returns the portion of the value of  $sourceString  beginning at the\n             position indicated by the value of  $start  and continuing for the number of\n                 characters  indicated by the value of\n                 $length .</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "sourceString", "type" : "xs:string", "occurrence" : "?", "description" : "" }, { "name" : "start", "type" : "xs:double", "occurrence" : null, "description" : "" } ], "returns" : { "type" : "xs:string", "description" : "" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 3, "name" : "substring", "qname" : "fn:substring", "signature" : "($sourceString as xs:string?, $start as xs:double, $length as xs:double) as xs:string external", "description" : " <div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">Returns the portion of the value of <code>$sourceString</code> beginning at the\n             position indicated by the value of <code>$start</code> and continuing for the number of\n                <termref def=\"character\">characters</termref> indicated by the value of\n                <code>$length</code>.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\"><example role=\"signature\"><proto name=\"substring\" return-type=\"xs:string\" isOp=\"no\" prefix=\"fn\" returnEmptyOk=\"no\" returnSeq=\"no\" returnVaries=\"no\" isSchema=\"no\" isDatatype=\"no\" isSpecial=\"no\"><arg name=\"sourceString\" type=\"xs:string?\"/><arg name=\"start\" type=\"xs:double\"/></proto></example><example role=\"signature\"><proto name=\"substring\" return-type=\"xs:string\" isOp=\"no\" prefix=\"fn\" returnEmptyOk=\"no\" returnSeq=\"no\" returnVaries=\"no\" isSchema=\"no\" isDatatype=\"no\" isSpecial=\"no\"><arg name=\"sourceString\" type=\"xs:string?\"/><arg name=\"start\" type=\"xs:double\"/><arg name=\"length\" type=\"xs:double\"/></proto></example></p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">This function is <termref def=\"dt-deterministic\">deterministic</termref>, <termref def=\"dt-context-independent\">context-independent</termref>,  and <termref def=\"dt-focus-independent\">focus-independent</termref>. </p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">If the value of <code>$sourceString</code> is the empty sequence, the function returns\n             the zero-length string. </p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">Otherwise, the function returns a string comprising those <termref def=\"character\">characters</termref> of <code>$sourceString</code> whose index position (counting\n             from one) is greater than or equal to the value of <code>$start</code> (rounded to an\n             integer), and (if <code>$length</code> is specified) less than the sum of\n                <code>$start</code> and <code>$length</code> (both rounded to integers).</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The characters returned do not extend beyond <code>$sourceString</code>. If\n                <code>$start</code> is zero or negative, only those characters in positions greater\n             than zero are returned.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">More specifically, the three argument version of the function returns the characters in\n                <code>$sourceString</code> whose position <code>$p</code> satisfies:</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">\n             <code>fn:round($start) &lt;= $p &lt; fn:round($start) + fn:round($length)</code>\n          </p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The two argument version of the function assumes that <code>$length</code> is infinite\n             and thus returns the <termref def=\"character\">characters</termref> in\n                <code>$sourceString</code> whose position <code>$p</code> satisfies:</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">\n             <code>fn:round($start) &lt;= $p</code>\n          </p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">In the above computations, the rules for <code>op:numeric-less-than</code> and\n                <code>op:numeric-greater-than</code> apply.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The first character of a string is located at position 1, not position 0.</p></div>\n", "summary" : "<p>  Returns the portion of the value of  $sourceString  beginning at the\n             position indicated by the value of  $start  and continuing for the number of\n                 characters  indicated by the value of\n                 $length .</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "sourceString", "type" : "xs:string", "occurrence" : "?", "description" : "" }, { "name" : "start", "type" : "xs:double", "occurrence" : null, "description" : "" }, { "name" : "length", "type" : "xs:double", "occurrence" : null, "description" : "" } ], "returns" : { "type" : "xs:string", "description" : "" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "sum", "qname" : "fn:sum", "signature" : "($arg as xs:anyAtomicType*) as xs:anyAtomicType external", "description" : " <div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">Returns a value obtained by adding together the values in\n             <code>$arg</code>.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\"><example role=\"signature\"><proto name=\"sum\" return-type=\"xs:anyAtomicType\" isOp=\"no\" prefix=\"fn\" returnEmptyOk=\"no\" returnSeq=\"no\" returnVaries=\"no\" isSchema=\"no\" isDatatype=\"no\" isSpecial=\"no\"><arg name=\"arg\" type=\"xs:anyAtomicType*\"/></proto></example><example role=\"signature\"><proto name=\"sum\" return-type=\"xs:anyAtomicType?\" isOp=\"no\" prefix=\"fn\" returnEmptyOk=\"no\" returnSeq=\"no\" returnVaries=\"no\" isSchema=\"no\" isDatatype=\"no\" isSpecial=\"no\"><arg name=\"arg\" type=\"xs:anyAtomicType*\"/><arg name=\"zero\" type=\"xs:anyAtomicType?\"/></proto></example></p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">This function is <termref def=\"dt-deterministic\">deterministic</termref>, <termref def=\"dt-context-independent\">context-independent</termref>,  and <termref def=\"dt-focus-independent\">focus-independent</termref>. </p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">Any values of type <code>xs:untypedAtomic</code> in <code>$arg</code> are cast to\n                <code>xs:double</code>. The items in the resulting sequence may be reordered in an\n             arbitrary order. The resulting sequence is referred to below as the converted\n             sequence.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">If the converted sequence is empty, then the single-argument form of the function\n             returns the <code>xs:integer</code> value <code>0</code>; the two-argument form returns\n             the value of the argument <code>$zero</code>.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">If the converted sequence contains the value <code>NaN</code>, <code>NaN</code> is\n             returned.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">All items in <code>$arg</code> must be numeric or derived from a single base type. In\n             addition, the type must support addition. Duration values must either all be\n                <code>xs:yearMonthDuration</code> values or must all be\n                <code>xs:dayTimeDuration</code> values. For numeric values, the numeric promotion\n             rules defined in <specref ref=\"op.numeric\"/> are used to promote all values to a single\n             common type. The sum of a sequence of integers will therefore be an integer, while the\n             sum of a numeric sequence that includes at least one <code>xs:double</code> will be an\n                <code>xs:double</code>. </p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The result of the function, using the second signature, is the result of the\n             expression:</p><eg xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\" xml:space=\"preserve\">\n if (fn:count($c) eq 0) then\n     $zero\n else if (fn:count($c) eq 1) then\n     $c[1]\n else\n     $c[1] + fn:sum(subsequence($c, 2))</eg><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">where <code>$c</code> is the converted sequence.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The result of the function, using the first signature, is the result of the expression:\n                <code>fn:sum($arg, 0)</code>.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\" diff=\"del\" at=\"F\">For detailed type semantics, see [Formal Semantics].</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">A type error is raised <errorref class=\"RG\" code=\"0006\"/> if the input sequence contains\n             items of incompatible types, as described above.</p></div>\n", "summary" : "<p>  Returns a value obtained by adding together the values in\n              $arg .</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "arg", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "" } ], "returns" : { "type" : "xs:anyAtomicType", "description" : "" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 2, "name" : "sum", "qname" : "fn:sum", "signature" : "($arg as xs:anyAtomicType*, $zero as xs:anyAtomicType?) as xs:anyAtomicType? external", "description" : " <div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">Returns a value obtained by adding together the values in\n             <code>$arg</code>.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\"><example role=\"signature\"><proto name=\"sum\" return-type=\"xs:anyAtomicType\" isOp=\"no\" prefix=\"fn\" returnEmptyOk=\"no\" returnSeq=\"no\" returnVaries=\"no\" isSchema=\"no\" isDatatype=\"no\" isSpecial=\"no\"><arg name=\"arg\" type=\"xs:anyAtomicType*\"/></proto></example><example role=\"signature\"><proto name=\"sum\" return-type=\"xs:anyAtomicType?\" isOp=\"no\" prefix=\"fn\" returnEmptyOk=\"no\" returnSeq=\"no\" returnVaries=\"no\" isSchema=\"no\" isDatatype=\"no\" isSpecial=\"no\"><arg name=\"arg\" type=\"xs:anyAtomicType*\"/><arg name=\"zero\" type=\"xs:anyAtomicType?\"/></proto></example></p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">This function is <termref def=\"dt-deterministic\">deterministic</termref>, <termref def=\"dt-context-independent\">context-independent</termref>,  and <termref def=\"dt-focus-independent\">focus-independent</termref>. </p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">Any values of type <code>xs:untypedAtomic</code> in <code>$arg</code> are cast to\n                <code>xs:double</code>. The items in the resulting sequence may be reordered in an\n             arbitrary order. The resulting sequence is referred to below as the converted\n             sequence.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">If the converted sequence is empty, then the single-argument form of the function\n             returns the <code>xs:integer</code> value <code>0</code>; the two-argument form returns\n             the value of the argument <code>$zero</code>.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">If the converted sequence contains the value <code>NaN</code>, <code>NaN</code> is\n             returned.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">All items in <code>$arg</code> must be numeric or derived from a single base type. In\n             addition, the type must support addition. Duration values must either all be\n                <code>xs:yearMonthDuration</code> values or must all be\n                <code>xs:dayTimeDuration</code> values. For numeric values, the numeric promotion\n             rules defined in <specref ref=\"op.numeric\"/> are used to promote all values to a single\n             common type. The sum of a sequence of integers will therefore be an integer, while the\n             sum of a numeric sequence that includes at least one <code>xs:double</code> will be an\n                <code>xs:double</code>. </p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The result of the function, using the second signature, is the result of the\n             expression:</p><eg xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\" xml:space=\"preserve\">\n if (fn:count($c) eq 0) then\n     $zero\n else if (fn:count($c) eq 1) then\n     $c[1]\n else\n     $c[1] + fn:sum(subsequence($c, 2))</eg><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">where <code>$c</code> is the converted sequence.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The result of the function, using the first signature, is the result of the expression:\n                <code>fn:sum($arg, 0)</code>.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\" diff=\"del\" at=\"F\">For detailed type semantics, see [Formal Semantics].</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">A type error is raised <errorref class=\"RG\" code=\"0006\"/> if the input sequence contains\n             items of incompatible types, as described above.</p></div>\n", "summary" : "<p>  Returns a value obtained by adding together the values in\n              $arg .</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "arg", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "" }, { "name" : "zero", "type" : "xs:anyAtomicType", "occurrence" : "?", "description" : "" } ], "returns" : { "type" : "xs:anyAtomicType?", "description" : "" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "tail", "qname" : "fn:tail", "signature" : "($arg as item()*) as item()* external", "description" : " <div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">Returns all but the first item in a sequence. </p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\"><example role=\"signature\"><proto name=\"tail\" return-type=\"item()*\" isOp=\"no\" prefix=\"fn\" returnEmptyOk=\"no\" returnSeq=\"no\" returnVaries=\"no\" isSchema=\"no\" isDatatype=\"no\" isSpecial=\"no\"><arg name=\"arg\" type=\"item()*\"/></proto></example></p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">This function is <termref def=\"dt-deterministic\">deterministic</termref>, <termref def=\"dt-context-independent\">context-independent</termref>,  and <termref def=\"dt-focus-independent\">focus-independent</termref>. </p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The function returns the value of the expression <code>subsequence($arg, 2)</code></p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">If <code>$arg</code> is the empty sequence, or a sequence containing a single item, then\n             the empty sequence is returned. </p></div>\n", "summary" : "<p>  Returns all but the first item in a sequence.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "arg", "type" : "item()", "occurrence" : "*", "description" : "" } ], "returns" : { "type" : "item()*", "description" : "" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "timezone-from-date", "qname" : "fn:timezone-from-date", "signature" : "($arg as xs:date?) as xs:dayTimeDuration? external", "description" : " <div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">Returns the timezone component of an <code>xs:date</code>.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\"><example role=\"signature\"><proto name=\"timezone-from-date\" return-type=\"xs:dayTimeDuration?\" isOp=\"no\" prefix=\"fn\" returnEmptyOk=\"no\" returnSeq=\"no\" returnVaries=\"no\" isSchema=\"no\" isDatatype=\"no\" isSpecial=\"no\"><arg name=\"arg\" type=\"xs:date?\"/></proto></example></p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">This function is <termref def=\"dt-deterministic\">deterministic</termref>, <termref def=\"dt-context-independent\">context-independent</termref>,  and <termref def=\"dt-focus-independent\">focus-independent</termref>. </p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">If <code>$arg</code> is the empty sequence, the function returns the empty sequence.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">Otherwise, the function returns the timezone component of <code>$arg</code>, if any. If\n                <code>$arg</code> has a timezone component, then the result is an\n                <code>xs:dayTimeDuration</code> that indicates deviation from UTC; its value may\n             range from +14:00 to -14:00 hours, both inclusive. If <code>$arg</code> has no timezone\n             component, the result is the empty sequence.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The expression <code>fn:timezone-from-date(xs:date(\"1999-05-31-05:00\"))</code> returns <code>xs:dayTimeDuration(\"-PT5H\")</code>.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The expression <code>fn:timezone-from-date(xs:date(\"2000-06-12Z\"))</code> returns <code>xs:dayTimeDuration(\"PT0S\")</code>.</p></div>\n", "summary" : "<p>  Returns the timezone component of an  xs:date .</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "arg", "type" : "xs:date", "occurrence" : "?", "description" : "" } ], "returns" : { "type" : "xs:dayTimeDuration?", "description" : "" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "timezone-from-dateTime", "qname" : "fn:timezone-from-dateTime", "signature" : "($arg as xs:dateTime?) as xs:dayTimeDuration? external", "description" : " <div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">Returns the timezone component of an <code>xs:dateTime</code>.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\"><example role=\"signature\"><proto name=\"timezone-from-dateTime\" return-type=\"xs:dayTimeDuration?\" isOp=\"no\" prefix=\"fn\" returnEmptyOk=\"no\" returnSeq=\"no\" returnVaries=\"no\" isSchema=\"no\" isDatatype=\"no\" isSpecial=\"no\"><arg name=\"arg\" type=\"xs:dateTime?\"/></proto></example></p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">This function is <termref def=\"dt-deterministic\">deterministic</termref>, <termref def=\"dt-context-independent\">context-independent</termref>,  and <termref def=\"dt-focus-independent\">focus-independent</termref>. </p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">If <code>$arg</code> is the empty sequence, the function returns the empty sequence.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">Otherwise, the function returns the timezone component of <code>$arg</code>, if any. If\n                <code>$arg</code> has a timezone component, then the result is an\n                <code>xs:dayTimeDuration</code> that indicates deviation from UTC; its value may\n             range from +14:00 to -14:00 hours, both inclusive. If <code>$arg</code> has no timezone\n             component, the result is the empty sequence.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The expression <code>fn:timezone-from-dateTime(xs:dateTime(\"1999-05-31T13:20:00-05:00\"))</code> returns <code>xs:dayTimeDuration(\"-PT5H\")</code>.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The expression <code>fn:timezone-from-dateTime(xs:dateTime(\"2000-06-12T13:20:00Z\"))</code> returns <code>xs:dayTimeDuration(\"PT0S\")</code>.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The expression <code>fn:timezone-from-dateTime(xs:dateTime(\"2004-08-27T00:00:00\"))</code> returns <code>()</code>.</p></div>\n", "summary" : "<p>  Returns the timezone component of an  xs:dateTime .</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "arg", "type" : "xs:dateTime", "occurrence" : "?", "description" : "" } ], "returns" : { "type" : "xs:dayTimeDuration?", "description" : "" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "timezone-from-time", "qname" : "fn:timezone-from-time", "signature" : "($arg as xs:time?) as xs:dayTimeDuration? external", "description" : " <div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">Returns the timezone component of an <code>xs:time</code>.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\"><example role=\"signature\"><proto name=\"timezone-from-time\" return-type=\"xs:dayTimeDuration?\" isOp=\"no\" prefix=\"fn\" returnEmptyOk=\"no\" returnSeq=\"no\" returnVaries=\"no\" isSchema=\"no\" isDatatype=\"no\" isSpecial=\"no\"><arg name=\"arg\" type=\"xs:time?\"/></proto></example></p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">This function is <termref def=\"dt-deterministic\">deterministic</termref>, <termref def=\"dt-context-independent\">context-independent</termref>,  and <termref def=\"dt-focus-independent\">focus-independent</termref>. </p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">If <code>$arg</code> is the empty sequence, the function returns the empty sequence.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">Otherwise, the function returns the timezone component of <code>$arg</code>, if any. If\n                <code>$arg</code> has a timezone component, then the result is an\n                <code>xs:dayTimeDuration</code> that indicates deviation from UTC; its value may\n             range from +14:00 to -14:00 hours, both inclusive. If <code>$arg</code> has no timezone\n             component, the result is the empty sequence.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The expression <code>fn:timezone-from-time(xs:time(\"13:20:00-05:00\"))</code> returns <code>xs:dayTimeDuration(\"-PT5H\")</code>.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The expression <code>fn:timezone-from-time(xs:time(\"13:20:00\"))</code> returns <code>()</code>.</p></div>\n", "summary" : "<p>  Returns the timezone component of an  xs:time .</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "arg", "type" : "xs:time", "occurrence" : "?", "description" : "" } ], "returns" : { "type" : "xs:dayTimeDuration?", "description" : "" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 2, "name" : "tokenize", "qname" : "fn:tokenize", "signature" : "($input as xs:string?, $pattern as xs:string) as xs:string* external", "description" : " <div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">Returns a sequence of strings constructed by splitting the input wherever a\n             separator is found; the separator is any substring that matches a given regular\n             expression.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\"><example role=\"signature\"><proto name=\"tokenize\" return-type=\"xs:string*\" isOp=\"no\" prefix=\"fn\" returnEmptyOk=\"no\" returnSeq=\"no\" returnVaries=\"no\" isSchema=\"no\" isDatatype=\"no\" isSpecial=\"no\"><arg name=\"input\" type=\"xs:string?\"/><arg name=\"pattern\" type=\"xs:string\"/></proto></example><example role=\"signature\"><proto name=\"tokenize\" return-type=\"xs:string*\" isOp=\"no\" prefix=\"fn\" returnEmptyOk=\"no\" returnSeq=\"no\" returnVaries=\"no\" isSchema=\"no\" isDatatype=\"no\" isSpecial=\"no\"><arg name=\"input\" type=\"xs:string?\"/><arg name=\"pattern\" type=\"xs:string\"/><arg name=\"flags\" type=\"xs:string\"/></proto></example></p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">This function is <termref def=\"dt-deterministic\">deterministic</termref>, <termref def=\"dt-context-independent\">context-independent</termref>,  and <termref def=\"dt-focus-independent\">focus-independent</termref>. </p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The effect of calling the first version of this function (omitting the argument\n                <code>$flags</code>) is the same as the effect of calling the second version with the\n                <code>$flags</code> argument set to a zero-length string. Flags are defined in\n                <specref ref=\"flags\"/>.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The <code>$flags</code> argument is interpreted in the same way as for the\n                <code>fn:matches</code> function.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">If <code>$input</code> is the empty sequence, or if <code>$input</code> is the\n             zero-length string, the function returns the empty sequence.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The function returns a sequence of strings formed by breaking the <code>$input</code>\n             string into a sequence of strings, treating any substring that matches\n                <code>$pattern</code> as a separator. The separators themselves are not returned.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">If a separator occurs at the start of the <code>$input</code> string, the result\n             sequence will start with a zero-length string. Zero-length strings will also occur in\n             the result sequence if a separator occurs at the end of the <code>$input</code> string,\n             or if two adjacent substrings match the supplied <code>$pattern</code>.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\"> If two alternatives within the supplied <code>$pattern</code> both match at the same\n             position in the <code>$input</code> string, then the match that is chosen is the first.\n             For example:</p><eg xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\" xml:space=\"preserve\"> fn:tokenize(\"abracadabra\", \"(ab)|(a)\") returns (\"\", \"r\", \"c\", \"d\", \"r\", \"\")</eg><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">A <phrase diff=\"add\" at=\"L\">dynamic</phrase> error is raised <errorref class=\"RX\" code=\"0002\"/> if the value of\n                <code>$pattern</code> is invalid according to the rules described in section <specref ref=\"regex-syntax\"/>.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">A <phrase diff=\"add\" at=\"L\">dynamic</phrase> error is raised <errorref class=\"RX\" code=\"0001\"/> if the value of\n                <code>$flags</code> is invalid according to the rules described in section <specref ref=\"regex-syntax\"/>.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">A <phrase diff=\"add\" at=\"L\">dynamic</phrase> error is raised <errorref class=\"RX\" code=\"0003\"/> if the supplied\n                <code>$pattern</code> matches a zero-length string, that is, if <code>fn:matches(\"\",\n                $pattern, $flags)</code> returns <code>true</code>. </p></div>\n", "summary" : "<p>  Returns a sequence of strings constructed by splitting the input wherever a\n             separator is found; the separator is any substring that matches a given regular\n             expression.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "input", "type" : "xs:string", "occurrence" : "?", "description" : "" }, { "name" : "pattern", "type" : "xs:string", "occurrence" : null, "description" : "" } ], "returns" : { "type" : "xs:string*", "description" : "" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 3, "name" : "tokenize", "qname" : "fn:tokenize", "signature" : "($input as xs:string?, $pattern as xs:string, $flags as xs:string) as xs:string* external", "description" : " <div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">Returns a sequence of strings constructed by splitting the input wherever a\n             separator is found; the separator is any substring that matches a given regular\n             expression.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\"><example role=\"signature\"><proto name=\"tokenize\" return-type=\"xs:string*\" isOp=\"no\" prefix=\"fn\" returnEmptyOk=\"no\" returnSeq=\"no\" returnVaries=\"no\" isSchema=\"no\" isDatatype=\"no\" isSpecial=\"no\"><arg name=\"input\" type=\"xs:string?\"/><arg name=\"pattern\" type=\"xs:string\"/></proto></example><example role=\"signature\"><proto name=\"tokenize\" return-type=\"xs:string*\" isOp=\"no\" prefix=\"fn\" returnEmptyOk=\"no\" returnSeq=\"no\" returnVaries=\"no\" isSchema=\"no\" isDatatype=\"no\" isSpecial=\"no\"><arg name=\"input\" type=\"xs:string?\"/><arg name=\"pattern\" type=\"xs:string\"/><arg name=\"flags\" type=\"xs:string\"/></proto></example></p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">This function is <termref def=\"dt-deterministic\">deterministic</termref>, <termref def=\"dt-context-independent\">context-independent</termref>,  and <termref def=\"dt-focus-independent\">focus-independent</termref>. </p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The effect of calling the first version of this function (omitting the argument\n                <code>$flags</code>) is the same as the effect of calling the second version with the\n                <code>$flags</code> argument set to a zero-length string. Flags are defined in\n                <specref ref=\"flags\"/>.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The <code>$flags</code> argument is interpreted in the same way as for the\n                <code>fn:matches</code> function.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">If <code>$input</code> is the empty sequence, or if <code>$input</code> is the\n             zero-length string, the function returns the empty sequence.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The function returns a sequence of strings formed by breaking the <code>$input</code>\n             string into a sequence of strings, treating any substring that matches\n                <code>$pattern</code> as a separator. The separators themselves are not returned.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">If a separator occurs at the start of the <code>$input</code> string, the result\n             sequence will start with a zero-length string. Zero-length strings will also occur in\n             the result sequence if a separator occurs at the end of the <code>$input</code> string,\n             or if two adjacent substrings match the supplied <code>$pattern</code>.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\"> If two alternatives within the supplied <code>$pattern</code> both match at the same\n             position in the <code>$input</code> string, then the match that is chosen is the first.\n             For example:</p><eg xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\" xml:space=\"preserve\"> fn:tokenize(\"abracadabra\", \"(ab)|(a)\") returns (\"\", \"r\", \"c\", \"d\", \"r\", \"\")</eg><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">A <phrase diff=\"add\" at=\"L\">dynamic</phrase> error is raised <errorref class=\"RX\" code=\"0002\"/> if the value of\n                <code>$pattern</code> is invalid according to the rules described in section <specref ref=\"regex-syntax\"/>.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">A <phrase diff=\"add\" at=\"L\">dynamic</phrase> error is raised <errorref class=\"RX\" code=\"0001\"/> if the value of\n                <code>$flags</code> is invalid according to the rules described in section <specref ref=\"regex-syntax\"/>.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">A <phrase diff=\"add\" at=\"L\">dynamic</phrase> error is raised <errorref class=\"RX\" code=\"0003\"/> if the supplied\n                <code>$pattern</code> matches a zero-length string, that is, if <code>fn:matches(\"\",\n                $pattern, $flags)</code> returns <code>true</code>. </p></div>\n", "summary" : "<p>  Returns a sequence of strings constructed by splitting the input wherever a\n             separator is found; the separator is any substring that matches a given regular\n             expression.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "input", "type" : "xs:string", "occurrence" : "?", "description" : "" }, { "name" : "pattern", "type" : "xs:string", "occurrence" : null, "description" : "" }, { "name" : "flags", "type" : "xs:string", "occurrence" : null, "description" : "" } ], "returns" : { "type" : "xs:string*", "description" : "" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 2, "name" : "trace", "qname" : "fn:trace", "signature" : "($value as item()*, $label as xs:string) as item()* external", "description" : " <div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">Provides an execution trace intended to be used in debugging queries.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\"><example role=\"signature\"><proto name=\"trace\" return-type=\"item()*\" isOp=\"no\" prefix=\"fn\" returnEmptyOk=\"no\" returnSeq=\"no\" returnVaries=\"no\" isSchema=\"no\" isDatatype=\"no\" isSpecial=\"no\"><arg name=\"value\" type=\"item()*\"/><arg name=\"label\" type=\"xs:string\"/></proto></example></p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">This function is <termref def=\"dt-deterministic\">deterministic</termref>, <termref def=\"dt-context-independent\">context-independent</termref>,  and <termref def=\"dt-focus-independent\">focus-independent</termref>. </p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The function returns the value of <code>$value</code>, unchanged.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">In addition, the values of <code>$value</code>, converted to an <code>xs:string</code>,\n             and <code>$label</code>\n             <rfc2119>may</rfc2119> be directed to a trace data set. The destination of the trace\n             output is <termref def=\"implementation-defined\"/>. The format of the trace output is\n                <termref def=\"implementation-dependent\"/>. The ordering of output from calls of the\n                <code>fn:trace</code> function is <termref def=\"implementation-dependent\"/>.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">Consider a situation in which a user wants to investigate the actual value passed to\n                a function. Assume that in a particular execution, <code>$v</code> is an\n                   <code>xs:decimal</code> with value <code>124.84</code>. Writing <code>fn:trace($v,\n                   'the value of $v is:')</code> will put the strings <code>\"124.84\"</code> and\n                   <code>\"the value of $v is:\"</code> in the trace data set in implementation\n                dependent order.</p></div>\n", "summary" : "<p>  Provides an execution trace intended to be used in debugging queries.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "value", "type" : "item()", "occurrence" : "*", "description" : "" }, { "name" : "label", "type" : "xs:string", "occurrence" : null, "description" : "" } ], "returns" : { "type" : "item()*", "description" : "" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 3, "name" : "translate", "qname" : "fn:translate", "signature" : "($arg as xs:string?, $mapString as xs:string, $transString as xs:string) as xs:string external", "description" : " <div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">Returns the value of <code>$arg</code> modified by replacing or removing\n             individual characters. </p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\"><example role=\"signature\"><proto name=\"translate\" return-type=\"xs:string\" isOp=\"no\" prefix=\"fn\" returnEmptyOk=\"no\" returnSeq=\"no\" returnVaries=\"no\" isSchema=\"no\" isDatatype=\"no\" isSpecial=\"no\"><arg name=\"arg\" type=\"xs:string?\"/><arg name=\"mapString\" type=\"xs:string\"/><arg name=\"transString\" type=\"xs:string\"/></proto></example></p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">This function is <termref def=\"dt-deterministic\">deterministic</termref>, <termref def=\"dt-context-independent\">context-independent</termref>,  and <termref def=\"dt-focus-independent\">focus-independent</termref>. </p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">If the value of <code>$arg</code> is the empty sequence, the function returns the\n             zero-length string.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">Otherwise, the function returns a result string constructed by processing each <termref def=\"character\">character</termref> in the value of <code>$arg</code>, in order,\n             according to the following rules:</p><olist xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\"><item><p>If the character does not appear in the value of <code>$mapString</code> then it\n                   is added to the result string unchanged.</p></item><item><p>If the character first appears in the value of <code>$mapString</code> at some\n                   position <emph>M</emph>, where the value of <code>$transString</code> is\n                      <emph>M</emph> or more characters in length, then the character at position\n                      <emph>M</emph> in <code>$transString</code> is added to the result string.</p></item><item><p>If the character first appears in the value of <code>$mapString</code> at some\n                   position <emph>M</emph>, where the value of <code>$transString</code> is less than\n                      <emph>M</emph> characters in length, then the character is omitted from the\n                   result string.</p></item></olist><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">If <code>$mapString</code> is the zero-length string then the function returns\n                <code>$arg</code> unchanged.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">If a character occurs more than once in <code>$mapString</code>, then the first\n             occurrence determines the action taken.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">If <code>$transString</code> is longer than <code>$mapString</code>, the excess\n             characters are ignored.</p></div>\n", "summary" : "<p>  Returns the value of  $arg  modified by replacing or removing\n             individual characters.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "arg", "type" : "xs:string", "occurrence" : "?", "description" : "" }, { "name" : "mapString", "type" : "xs:string", "occurrence" : null, "description" : "" }, { "name" : "transString", "type" : "xs:string", "occurrence" : null, "description" : "" } ], "returns" : { "type" : "xs:string", "description" : "" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 0, "name" : "true", "qname" : "fn:true", "signature" : "() as xs:boolean external", "description" : " <div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">Returns the <code>xs:boolean</code> value <code>true</code>.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\"><example role=\"signature\"><proto name=\"true\" return-type=\"xs:boolean\" isOp=\"no\" prefix=\"fn\" returnEmptyOk=\"no\" returnSeq=\"no\" returnVaries=\"no\" isSchema=\"no\" isDatatype=\"no\" isSpecial=\"no\"/></example></p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">This function is <termref def=\"dt-deterministic\">deterministic</termref>, <termref def=\"dt-context-independent\">context-independent</termref>,  and <termref def=\"dt-focus-independent\">focus-independent</termref>. </p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The result is equivalent to <code>xs:boolean(\"1\")</code>.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The expression <code>fn:true()</code> returns <code>xs:boolean(1)</code>.</p></div>\n", "summary" : "<p>  Returns the  xs:boolean  value  true .</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [  ], "returns" : { "type" : "xs:boolean", "description" : "" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "unordered", "qname" : "fn:unordered", "signature" : "($sourceSeq as item()*) as item()* external", "description" : " <div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">Returns the items of <code>$sourceSeq</code> in an <termref def=\"implementation-dependent\"/> order.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\"><example role=\"signature\"><proto name=\"unordered\" return-type=\"item()*\" isOp=\"no\" prefix=\"fn\" returnEmptyOk=\"no\" returnSeq=\"no\" returnVaries=\"no\" isSchema=\"no\" isDatatype=\"no\" isSpecial=\"no\"><arg name=\"sourceSeq\" type=\"item()*\"/></proto></example></p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">This function is <termref def=\"dt-deterministic\">deterministic</termref>, <termref def=\"dt-context-independent\">context-independent</termref>,  and <termref def=\"dt-focus-independent\">focus-independent</termref>. </p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The function returns the items of <code>$sourceSeq</code> in an <termref def=\"implementation-dependent\"/> order.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">Query optimizers may be able to do a better job if the order of the output sequence is\n             not specified. For example, when retrieving prices from a purchase order, if an index\n             exists on prices, it may be more efficient to return the prices in index order rather\n             than in document order.</p></div>\n", "summary" : "<p>  Returns the items of  $sourceSeq  in an    order.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "sourceSeq", "type" : "item()", "occurrence" : "*", "description" : "" } ], "returns" : { "type" : "item()*", "description" : "" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "unparsed-text-available", "qname" : "fn:unparsed-text-available", "signature" : "($href as xs:string?) as xs:boolean external", "description" : " <div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">Because errors in evaluating the <function>fn:unparsed-text</function> function are\n             non-recoverable, these two functions are provided to allow an application to determine\n             whether a call with particular arguments would succeed.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\"><example role=\"signature\"><proto name=\"unparsed-text-available\" return-type=\"xs:boolean\" isOp=\"no\" prefix=\"fn\" returnEmptyOk=\"no\" returnSeq=\"no\" returnVaries=\"no\" isSchema=\"no\" isDatatype=\"no\" isSpecial=\"no\"><arg name=\"href\" type=\"xs:string?\"/></proto></example><example role=\"signature\"><proto name=\"unparsed-text-available\" return-type=\"xs:boolean\" isOp=\"no\" prefix=\"fn\" returnEmptyOk=\"no\" returnSeq=\"no\" returnVaries=\"no\" isSchema=\"no\" isDatatype=\"no\" isSpecial=\"no\"><arg name=\"href\" type=\"xs:string?\"/><arg name=\"encoding\" type=\"xs:string\"/></proto></example></p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">This function is <termref def=\"dt-deterministic\">deterministic</termref>, <termref def=\"dt-context-dependent\">context-dependent</termref>,  and <termref def=\"dt-focus-independent\">focus-independent</termref>.  It depends on\n \t\tstatic base uri.\n \t</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The <function>fn:unparsed-text-available</function> function determines whether a call\n             on the <function>fn:unparsed-text</function> function with identical arguments would\n             return a string.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">If the first argument is an empty sequence, the function returns false. <phrase diff=\"del\" at=\"L\">If the second\n             argument is an empty sequence, the function behaves as if the second argument were\n             omitted.</phrase></p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">In other cases, the function returns true if a call on\n                <function>fn:unparsed-text</function> with the same arguments would succeed, and\n             false if a call on <function>fn:unparsed-text</function> with the same arguments would\n             fail with a non-recoverable dynamic error.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The functions <function>fn:unparsed-text</function> and\n                <function>fn:unparsed-text-available</function> have the same requirement for\n                <termref def=\"deterministic\">determinism</termref> as the functions\n                <code>fn:doc</code> and <code>fn:doc-available</code>. This means that unless the\n             user has explicitly stated a requirement for a reduced level of determinism, either of\n             these functions if called twice with the same arguments during the course of a\n             transformation <rfc2119>must</rfc2119> return the same results each time; moreover, the\n             results of a call on <function>fn:unparsed-text-available</function>\n             <rfc2119>must</rfc2119> be consistent with the results of a subsequent call on\n                <function>unparsed-text</function> with the same arguments.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">This requires that the <function>unparsed-text-available</function> function should\n             actually attempt to read the resource identified by the URI, and check that it is\n             correctly encoded and contains no characters that are invalid in XML. Implementations\n             may avoid the cost of repeating these checks for example by caching the validated\n             contents of the resource, to anticipate a subsequent call on the\n                <function>unparsed-text</function>\n             <phrase diff=\"add\" at=\"A\">or <function>unparsed-text-lines</function>\n             </phrase> function. Alternatively, implementations may be able to rewrite an expression\n             such as <code>if (unparsed-text-available(A)) then unparsed-text(A) else ...</code> to\n             generate a single call internally.</p></div>\n", "summary" : "<p>  Because errors in evaluating the  fn:unparsed-text  function are\n             non-recoverable, these two functions are provided to allow an application to determine\n             whether a call with particular arguments would succeed.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "href", "type" : "xs:string", "occurrence" : "?", "description" : "" } ], "returns" : { "type" : "xs:boolean", "description" : "" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "unparsed-text-available", "qname" : "fn:unparsed-text-available", "signature" : "($href as xs:string?) as xs:boolean external", "description" : " <div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">Because errors in evaluating the <function>fn:unparsed-text</function> function are\n             non-recoverable, these two functions are provided to allow an application to determine\n             whether a call with particular arguments would succeed.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\"><example role=\"signature\"><proto name=\"unparsed-text-available\" return-type=\"xs:boolean\" isOp=\"no\" prefix=\"fn\" returnEmptyOk=\"no\" returnSeq=\"no\" returnVaries=\"no\" isSchema=\"no\" isDatatype=\"no\" isSpecial=\"no\"><arg name=\"href\" type=\"xs:string?\"/></proto></example><example role=\"signature\"><proto name=\"unparsed-text-available\" return-type=\"xs:boolean\" isOp=\"no\" prefix=\"fn\" returnEmptyOk=\"no\" returnSeq=\"no\" returnVaries=\"no\" isSchema=\"no\" isDatatype=\"no\" isSpecial=\"no\"><arg name=\"href\" type=\"xs:string?\"/><arg name=\"encoding\" type=\"xs:string\"/></proto></example></p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">This function is <termref def=\"dt-deterministic\">deterministic</termref>, <termref def=\"dt-context-dependent\">context-dependent</termref>,  and <termref def=\"dt-focus-independent\">focus-independent</termref>.  It depends on\n \t\tstatic base uri.\n \t</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The <function>fn:unparsed-text-available</function> function determines whether a call\n             on the <function>fn:unparsed-text</function> function with identical arguments would\n             return a string.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">If the first argument is an empty sequence, the function returns false. <phrase diff=\"del\" at=\"L\">If the second\n             argument is an empty sequence, the function behaves as if the second argument were\n             omitted.</phrase></p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">In other cases, the function returns true if a call on\n                <function>fn:unparsed-text</function> with the same arguments would succeed, and\n             false if a call on <function>fn:unparsed-text</function> with the same arguments would\n             fail with a non-recoverable dynamic error.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The functions <function>fn:unparsed-text</function> and\n                <function>fn:unparsed-text-available</function> have the same requirement for\n                <termref def=\"deterministic\">determinism</termref> as the functions\n                <code>fn:doc</code> and <code>fn:doc-available</code>. This means that unless the\n             user has explicitly stated a requirement for a reduced level of determinism, either of\n             these functions if called twice with the same arguments during the course of a\n             transformation <rfc2119>must</rfc2119> return the same results each time; moreover, the\n             results of a call on <function>fn:unparsed-text-available</function>\n             <rfc2119>must</rfc2119> be consistent with the results of a subsequent call on\n                <function>unparsed-text</function> with the same arguments.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">This requires that the <function>unparsed-text-available</function> function should\n             actually attempt to read the resource identified by the URI, and check that it is\n             correctly encoded and contains no characters that are invalid in XML. Implementations\n             may avoid the cost of repeating these checks for example by caching the validated\n             contents of the resource, to anticipate a subsequent call on the\n                <function>unparsed-text</function>\n             <phrase diff=\"add\" at=\"A\">or <function>unparsed-text-lines</function>\n             </phrase> function. Alternatively, implementations may be able to rewrite an expression\n             such as <code>if (unparsed-text-available(A)) then unparsed-text(A) else ...</code> to\n             generate a single call internally.</p></div>\n", "summary" : "<p>  Because errors in evaluating the  fn:unparsed-text  function are\n             non-recoverable, these two functions are provided to allow an application to determine\n             whether a call with particular arguments would succeed.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "href", "type" : "xs:string", "occurrence" : "?", "description" : "" } ], "returns" : { "type" : "xs:boolean", "description" : "" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 2, "name" : "unparsed-text-available", "qname" : "fn:unparsed-text-available", "signature" : "($href as xs:string?, $encoding as xs:string) as xs:boolean external", "description" : " <div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">Because errors in evaluating the <function>fn:unparsed-text</function> function are\n             non-recoverable, these two functions are provided to allow an application to determine\n             whether a call with particular arguments would succeed.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\"><example role=\"signature\"><proto name=\"unparsed-text-available\" return-type=\"xs:boolean\" isOp=\"no\" prefix=\"fn\" returnEmptyOk=\"no\" returnSeq=\"no\" returnVaries=\"no\" isSchema=\"no\" isDatatype=\"no\" isSpecial=\"no\"><arg name=\"href\" type=\"xs:string?\"/></proto></example><example role=\"signature\"><proto name=\"unparsed-text-available\" return-type=\"xs:boolean\" isOp=\"no\" prefix=\"fn\" returnEmptyOk=\"no\" returnSeq=\"no\" returnVaries=\"no\" isSchema=\"no\" isDatatype=\"no\" isSpecial=\"no\"><arg name=\"href\" type=\"xs:string?\"/><arg name=\"encoding\" type=\"xs:string\"/></proto></example></p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">This function is <termref def=\"dt-deterministic\">deterministic</termref>, <termref def=\"dt-context-dependent\">context-dependent</termref>,  and <termref def=\"dt-focus-independent\">focus-independent</termref>.  It depends on\n \t\tstatic base uri.\n \t</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The <function>fn:unparsed-text-available</function> function determines whether a call\n             on the <function>fn:unparsed-text</function> function with identical arguments would\n             return a string.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">If the first argument is an empty sequence, the function returns false. <phrase diff=\"del\" at=\"L\">If the second\n             argument is an empty sequence, the function behaves as if the second argument were\n             omitted.</phrase></p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">In other cases, the function returns true if a call on\n                <function>fn:unparsed-text</function> with the same arguments would succeed, and\n             false if a call on <function>fn:unparsed-text</function> with the same arguments would\n             fail with a non-recoverable dynamic error.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The functions <function>fn:unparsed-text</function> and\n                <function>fn:unparsed-text-available</function> have the same requirement for\n                <termref def=\"deterministic\">determinism</termref> as the functions\n                <code>fn:doc</code> and <code>fn:doc-available</code>. This means that unless the\n             user has explicitly stated a requirement for a reduced level of determinism, either of\n             these functions if called twice with the same arguments during the course of a\n             transformation <rfc2119>must</rfc2119> return the same results each time; moreover, the\n             results of a call on <function>fn:unparsed-text-available</function>\n             <rfc2119>must</rfc2119> be consistent with the results of a subsequent call on\n                <function>unparsed-text</function> with the same arguments.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">This requires that the <function>unparsed-text-available</function> function should\n             actually attempt to read the resource identified by the URI, and check that it is\n             correctly encoded and contains no characters that are invalid in XML. Implementations\n             may avoid the cost of repeating these checks for example by caching the validated\n             contents of the resource, to anticipate a subsequent call on the\n                <function>unparsed-text</function>\n             <phrase diff=\"add\" at=\"A\">or <function>unparsed-text-lines</function>\n             </phrase> function. Alternatively, implementations may be able to rewrite an expression\n             such as <code>if (unparsed-text-available(A)) then unparsed-text(A) else ...</code> to\n             generate a single call internally.</p></div>\n", "summary" : "<p>  Because errors in evaluating the  fn:unparsed-text  function are\n             non-recoverable, these two functions are provided to allow an application to determine\n             whether a call with particular arguments would succeed.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "href", "type" : "xs:string", "occurrence" : "?", "description" : "" }, { "name" : "encoding", "type" : "xs:string", "occurrence" : null, "description" : "" } ], "returns" : { "type" : "xs:boolean", "description" : "" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 2, "name" : "unparsed-text-available", "qname" : "fn:unparsed-text-available", "signature" : "($href as xs:string?, $encoding as xs:string) as xs:boolean external", "description" : " <div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">Because errors in evaluating the <function>fn:unparsed-text</function> function are\n             non-recoverable, these two functions are provided to allow an application to determine\n             whether a call with particular arguments would succeed.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\"><example role=\"signature\"><proto name=\"unparsed-text-available\" return-type=\"xs:boolean\" isOp=\"no\" prefix=\"fn\" returnEmptyOk=\"no\" returnSeq=\"no\" returnVaries=\"no\" isSchema=\"no\" isDatatype=\"no\" isSpecial=\"no\"><arg name=\"href\" type=\"xs:string?\"/></proto></example><example role=\"signature\"><proto name=\"unparsed-text-available\" return-type=\"xs:boolean\" isOp=\"no\" prefix=\"fn\" returnEmptyOk=\"no\" returnSeq=\"no\" returnVaries=\"no\" isSchema=\"no\" isDatatype=\"no\" isSpecial=\"no\"><arg name=\"href\" type=\"xs:string?\"/><arg name=\"encoding\" type=\"xs:string\"/></proto></example></p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">This function is <termref def=\"dt-deterministic\">deterministic</termref>, <termref def=\"dt-context-dependent\">context-dependent</termref>,  and <termref def=\"dt-focus-independent\">focus-independent</termref>.  It depends on\n \t\tstatic base uri.\n \t</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The <function>fn:unparsed-text-available</function> function determines whether a call\n             on the <function>fn:unparsed-text</function> function with identical arguments would\n             return a string.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">If the first argument is an empty sequence, the function returns false. <phrase diff=\"del\" at=\"L\">If the second\n             argument is an empty sequence, the function behaves as if the second argument were\n             omitted.</phrase></p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">In other cases, the function returns true if a call on\n                <function>fn:unparsed-text</function> with the same arguments would succeed, and\n             false if a call on <function>fn:unparsed-text</function> with the same arguments would\n             fail with a non-recoverable dynamic error.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The functions <function>fn:unparsed-text</function> and\n                <function>fn:unparsed-text-available</function> have the same requirement for\n                <termref def=\"deterministic\">determinism</termref> as the functions\n                <code>fn:doc</code> and <code>fn:doc-available</code>. This means that unless the\n             user has explicitly stated a requirement for a reduced level of determinism, either of\n             these functions if called twice with the same arguments during the course of a\n             transformation <rfc2119>must</rfc2119> return the same results each time; moreover, the\n             results of a call on <function>fn:unparsed-text-available</function>\n             <rfc2119>must</rfc2119> be consistent with the results of a subsequent call on\n                <function>unparsed-text</function> with the same arguments.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">This requires that the <function>unparsed-text-available</function> function should\n             actually attempt to read the resource identified by the URI, and check that it is\n             correctly encoded and contains no characters that are invalid in XML. Implementations\n             may avoid the cost of repeating these checks for example by caching the validated\n             contents of the resource, to anticipate a subsequent call on the\n                <function>unparsed-text</function>\n             <phrase diff=\"add\" at=\"A\">or <function>unparsed-text-lines</function>\n             </phrase> function. Alternatively, implementations may be able to rewrite an expression\n             such as <code>if (unparsed-text-available(A)) then unparsed-text(A) else ...</code> to\n             generate a single call internally.</p></div>\n", "summary" : "<p>  Because errors in evaluating the  fn:unparsed-text  function are\n             non-recoverable, these two functions are provided to allow an application to determine\n             whether a call with particular arguments would succeed.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "href", "type" : "xs:string", "occurrence" : "?", "description" : "" }, { "name" : "encoding", "type" : "xs:string", "occurrence" : null, "description" : "" } ], "returns" : { "type" : "xs:boolean", "description" : "" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "unparsed-text-lines", "qname" : "fn:unparsed-text-lines", "signature" : "($href as xs:string?) as xs:string* external", "description" : " <div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The <code>fn:unparsed-text-lines</code> function reads an external resource (for\n             example, a file) and returns its contents as a sequence of strings, one for each line of\n             text in the  <phrase diff=\"chg\" at=\"L\">string representation</phrase> of the resource.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\"><example role=\"signature\"><proto name=\"unparsed-text-lines\" return-type=\"xs:string*\" isOp=\"no\" prefix=\"fn\" returnEmptyOk=\"no\" returnSeq=\"no\" returnVaries=\"no\" isSchema=\"no\" isDatatype=\"no\" isSpecial=\"no\"><arg name=\"href\" type=\"xs:string?\"/></proto></example><example role=\"signature\"><proto name=\"unparsed-text-lines\" return-type=\"xs:string*\" isOp=\"no\" prefix=\"fn\" returnEmptyOk=\"no\" returnSeq=\"no\" returnVaries=\"no\" isSchema=\"no\" isDatatype=\"no\" isSpecial=\"no\"><arg name=\"href\" type=\"xs:string?\"/><arg name=\"encoding\" type=\"xs:string\"/></proto></example></p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">This function is <termref def=\"dt-deterministic\">deterministic</termref>, <termref def=\"dt-context-dependent\">context-dependent</termref>,  and <termref def=\"dt-focus-independent\">focus-independent</termref>.  It depends on\n \t\tstatic base uri.\n \t</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The <code>unparsed-text-lines</code> function reads an external resource (for example, a\n             file) and returns its <phrase diff=\"chg\" at=\"L\">string representation</phrase> as a sequence of strings, separated at newline\n             boundaries. </p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The result of the single-argument function is the same as the result of the expression\n                <code>fn:tokenize(fn:unparsed-text($href), '\\r\\n|\\r|\\n')[not(position()=last() and\n                .='')]</code>. The result of the two-argument function is the same as the result of\n             the expression <code>fn:tokenize(fn:unparsed-text($href, $encoding),\n                '\\r\\n|\\r|\\n'))[not(position()=last() and .='')]</code>. </p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The result is a thus a sequence of strings containing the text of the resource retrieved\n             using the URI, each string representing one line of text. Lines are separated by one of\n             the sequences x0A, x0D, or x0Dx0A. The characters representing the newline are not\n             included in the returned strings. If there are two adjacent newline sequences, a\n             zero-length string will be returned to represent the empty line; but if the external\n             resource ends with a newline sequence, no zero-length string will be returned as the\n             last item in the result.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">Error conditions are the same as for the <code>fn:unparsed-text</code> function.</p></div>\n", "summary" : "<p>  The  fn:unparsed-text-lines  function reads an external resource (for\n             example, a file) and returns its contents as a sequence of strings, one for each line of\n             text in the   string representation  of the resource.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "href", "type" : "xs:string", "occurrence" : "?", "description" : "" } ], "returns" : { "type" : "xs:string*", "description" : "" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 2, "name" : "unparsed-text-lines", "qname" : "fn:unparsed-text-lines", "signature" : "($href as xs:string?, $encoding as xs:string) as xs:string* external", "description" : " <div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The <code>fn:unparsed-text-lines</code> function reads an external resource (for\n             example, a file) and returns its contents as a sequence of strings, one for each line of\n             text in the  <phrase diff=\"chg\" at=\"L\">string representation</phrase> of the resource.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\"><example role=\"signature\"><proto name=\"unparsed-text-lines\" return-type=\"xs:string*\" isOp=\"no\" prefix=\"fn\" returnEmptyOk=\"no\" returnSeq=\"no\" returnVaries=\"no\" isSchema=\"no\" isDatatype=\"no\" isSpecial=\"no\"><arg name=\"href\" type=\"xs:string?\"/></proto></example><example role=\"signature\"><proto name=\"unparsed-text-lines\" return-type=\"xs:string*\" isOp=\"no\" prefix=\"fn\" returnEmptyOk=\"no\" returnSeq=\"no\" returnVaries=\"no\" isSchema=\"no\" isDatatype=\"no\" isSpecial=\"no\"><arg name=\"href\" type=\"xs:string?\"/><arg name=\"encoding\" type=\"xs:string\"/></proto></example></p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">This function is <termref def=\"dt-deterministic\">deterministic</termref>, <termref def=\"dt-context-dependent\">context-dependent</termref>,  and <termref def=\"dt-focus-independent\">focus-independent</termref>.  It depends on\n \t\tstatic base uri.\n \t</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The <code>unparsed-text-lines</code> function reads an external resource (for example, a\n             file) and returns its <phrase diff=\"chg\" at=\"L\">string representation</phrase> as a sequence of strings, separated at newline\n             boundaries. </p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The result of the single-argument function is the same as the result of the expression\n                <code>fn:tokenize(fn:unparsed-text($href), '\\r\\n|\\r|\\n')[not(position()=last() and\n                .='')]</code>. The result of the two-argument function is the same as the result of\n             the expression <code>fn:tokenize(fn:unparsed-text($href, $encoding),\n                '\\r\\n|\\r|\\n'))[not(position()=last() and .='')]</code>. </p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The result is a thus a sequence of strings containing the text of the resource retrieved\n             using the URI, each string representing one line of text. Lines are separated by one of\n             the sequences x0A, x0D, or x0Dx0A. The characters representing the newline are not\n             included in the returned strings. If there are two adjacent newline sequences, a\n             zero-length string will be returned to represent the empty line; but if the external\n             resource ends with a newline sequence, no zero-length string will be returned as the\n             last item in the result.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">Error conditions are the same as for the <code>fn:unparsed-text</code> function.</p></div>\n", "summary" : "<p>  The  fn:unparsed-text-lines  function reads an external resource (for\n             example, a file) and returns its contents as a sequence of strings, one for each line of\n             text in the   string representation  of the resource.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "href", "type" : "xs:string", "occurrence" : "?", "description" : "" }, { "name" : "encoding", "type" : "xs:string", "occurrence" : null, "description" : "" } ], "returns" : { "type" : "xs:string*", "description" : "" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 2, "name" : "unparsed-text-lines", "qname" : "fn:unparsed-text-lines", "signature" : "($href as xs:string?, $encoding as xs:string) as xs:string* external", "description" : " <div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The <code>fn:unparsed-text-lines</code> function reads an external resource (for\n             example, a file) and returns its contents as a sequence of strings, one for each line of\n             text in the  <phrase diff=\"chg\" at=\"L\">string representation</phrase> of the resource.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\"><example role=\"signature\"><proto name=\"unparsed-text-lines\" return-type=\"xs:string*\" isOp=\"no\" prefix=\"fn\" returnEmptyOk=\"no\" returnSeq=\"no\" returnVaries=\"no\" isSchema=\"no\" isDatatype=\"no\" isSpecial=\"no\"><arg name=\"href\" type=\"xs:string?\"/></proto></example><example role=\"signature\"><proto name=\"unparsed-text-lines\" return-type=\"xs:string*\" isOp=\"no\" prefix=\"fn\" returnEmptyOk=\"no\" returnSeq=\"no\" returnVaries=\"no\" isSchema=\"no\" isDatatype=\"no\" isSpecial=\"no\"><arg name=\"href\" type=\"xs:string?\"/><arg name=\"encoding\" type=\"xs:string\"/></proto></example></p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">This function is <termref def=\"dt-deterministic\">deterministic</termref>, <termref def=\"dt-context-dependent\">context-dependent</termref>,  and <termref def=\"dt-focus-independent\">focus-independent</termref>.  It depends on\n \t\tstatic base uri.\n \t</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The <code>unparsed-text-lines</code> function reads an external resource (for example, a\n             file) and returns its <phrase diff=\"chg\" at=\"L\">string representation</phrase> as a sequence of strings, separated at newline\n             boundaries. </p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The result of the single-argument function is the same as the result of the expression\n                <code>fn:tokenize(fn:unparsed-text($href), '\\r\\n|\\r|\\n')[not(position()=last() and\n                .='')]</code>. The result of the two-argument function is the same as the result of\n             the expression <code>fn:tokenize(fn:unparsed-text($href, $encoding),\n                '\\r\\n|\\r|\\n'))[not(position()=last() and .='')]</code>. </p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The result is a thus a sequence of strings containing the text of the resource retrieved\n             using the URI, each string representing one line of text. Lines are separated by one of\n             the sequences x0A, x0D, or x0Dx0A. The characters representing the newline are not\n             included in the returned strings. If there are two adjacent newline sequences, a\n             zero-length string will be returned to represent the empty line; but if the external\n             resource ends with a newline sequence, no zero-length string will be returned as the\n             last item in the result.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">Error conditions are the same as for the <code>fn:unparsed-text</code> function.</p></div>\n", "summary" : "<p>  The  fn:unparsed-text-lines  function reads an external resource (for\n             example, a file) and returns its contents as a sequence of strings, one for each line of\n             text in the   string representation  of the resource.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "href", "type" : "xs:string", "occurrence" : "?", "description" : "" }, { "name" : "encoding", "type" : "xs:string", "occurrence" : null, "description" : "" } ], "returns" : { "type" : "xs:string*", "description" : "" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "unparsed-text", "qname" : "fn:unparsed-text", "signature" : "($href as xs:string?) as xs:string? external", "description" : " <div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The <code>fn:unparsed-text</code> function reads an external resource (for example, a\n             file) and returns <phrase diff=\"chg\" at=\"L\">a string representation of the resource</phrase>.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\"><example role=\"signature\"><proto name=\"unparsed-text\" return-type=\"xs:string?\" isOp=\"no\" prefix=\"fn\" returnEmptyOk=\"no\" returnSeq=\"no\" returnVaries=\"no\" isSchema=\"no\" isDatatype=\"no\" isSpecial=\"no\"><arg name=\"href\" type=\"xs:string?\"/></proto></example><example role=\"signature\"><proto name=\"unparsed-text\" return-type=\"xs:string?\" isOp=\"no\" prefix=\"fn\" returnEmptyOk=\"no\" returnSeq=\"no\" returnVaries=\"no\" isSchema=\"no\" isDatatype=\"no\" isSpecial=\"no\"><arg name=\"href\" type=\"xs:string?\"/><arg name=\"encoding\" type=\"xs:string\"/></proto></example></p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">This function is <termref def=\"dt-deterministic\">deterministic</termref>, <termref def=\"dt-context-dependent\">context-dependent</termref>,  and <termref def=\"dt-focus-independent\">focus-independent</termref>.  It depends on\n \t\tstatic base uri.\n \t</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The <code>$href</code> argument <rfc2119>must</rfc2119> be a string in the form of a URI\n             reference, which <rfc2119>must</rfc2119> contain no fragment identifier, and\n                <rfc2119>must</rfc2119> identify a resource\n             <phrase diff=\"chg\" at=\"L\">for which a string representation is available</phrase>. If the URI is a\n             relative URI reference, then it is resolved relative to the\n             <phrase diff=\"chg\" at=\"L\">Static Base URI property from the static context</phrase>.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\" diff=\"add\" at=\"L\">The mapping of URIs to the string representation of a resource is the mapping defined\n             in the <xtermref spec=\"XP30\" ref=\"dt-available-text-resources\">available text resources</xtermref>\n          component of the dynamic context.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">If the value of the <code>$href</code> argument is an empty sequence, the function\n             returns an empty sequence.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The <code>$encoding</code> argument, if present, is the name of an encoding. The values\n             for this attribute follow the same rules as for the <code>encoding</code> attribute in\n             an XML declaration. The only values which every <termref def=\"implementation\">implementation</termref> is <rfc2119>required</rfc2119> to recognize are\n                <code>utf-8</code> and <code>utf-16</code>.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The encoding of the external resource is determined as follows:</p><olist xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\"><item><p>external encoding information is used if available, otherwise</p></item><item><p>if the media type of the resource is <code>text/xml</code> or\n                      <code>application/xml</code> (see <bibref ref=\"rfc2376\"/>), or if it matches\n                   the conventions <code>text/*+xml</code> or <code>application/*+xml</code> (see\n                      <bibref ref=\"rfc3023\"/> and/or its successors), then the encoding is recognized\n                   as specified in <bibref ref=\"REC-xml\"/>, otherwise</p></item><item><p>the value of the <code>$encoding</code> argument is used if present, otherwise</p></item><item><p>the processor <rfc2119>may</rfc2119> use <termref def=\"implementation-defined\">implementation-defined</termref> heuristics to determine the likely encoding,\n                   otherwise</p></item><item><p>UTF-8 is assumed.</p></item></olist><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The result of the function is a string containing the\n             <phrase diff=\"chg\" at=\"L\">string representation</phrase> of the resource retrieved\n             using the URI.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">A <phrase diff=\"add\" at=\"L\">dynamic</phrase> error is raised <errorref class=\"UT\" code=\"1170\"/> if <code>$href</code>\n             contains a fragment identifier, or if it cannot be used to retrieve the\n             <phrase diff=\"chg\" at=\"L\">string representation</phrase> of a resource.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">A <phrase diff=\"add\" at=\"L\">dynamic</phrase> error is raised <errorref class=\"UT\" code=\"1190\"/> <phrase diff=\"add\" at=\"L\">if the value\n             of the <code>$encoding</code> argument is not a valid encoding name, </phrase>if the <termref def=\"dt-processor\">processor</termref> does not support the specified encoding,\n             if the <phrase diff=\"chg\" at=\"L\">string representation</phrase> of the retrieved\n             resource contains octets that cannot be decoded into Unicode <termref def=\"character\">characters</termref> using the specified encoding, or if the resulting characters\n             are not permitted XML characters.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">A <phrase diff=\"add\" at=\"L\">dynamic</phrase> error is raised <errorref class=\"UT\" code=\"1200\"/> if\n                <code>$encoding</code> is absent and the <termref def=\"dt-processor\">processor</termref> cannot infer the encoding using external information and the\n             encoding is not UTF-8.</p></div>\n", "summary" : "<p>  The  fn:unparsed-text  function reads an external resource (for example, a\n             file) and returns  a string representation of the resource .</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "href", "type" : "xs:string", "occurrence" : "?", "description" : "" } ], "returns" : { "type" : "xs:string?", "description" : "" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "unparsed-text", "qname" : "fn:unparsed-text", "signature" : "($href as xs:string?) as xs:string? external", "description" : " <div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The <code>fn:unparsed-text</code> function reads an external resource (for example, a\n             file) and returns <phrase diff=\"chg\" at=\"L\">a string representation of the resource</phrase>.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\"><example role=\"signature\"><proto name=\"unparsed-text\" return-type=\"xs:string?\" isOp=\"no\" prefix=\"fn\" returnEmptyOk=\"no\" returnSeq=\"no\" returnVaries=\"no\" isSchema=\"no\" isDatatype=\"no\" isSpecial=\"no\"><arg name=\"href\" type=\"xs:string?\"/></proto></example><example role=\"signature\"><proto name=\"unparsed-text\" return-type=\"xs:string?\" isOp=\"no\" prefix=\"fn\" returnEmptyOk=\"no\" returnSeq=\"no\" returnVaries=\"no\" isSchema=\"no\" isDatatype=\"no\" isSpecial=\"no\"><arg name=\"href\" type=\"xs:string?\"/><arg name=\"encoding\" type=\"xs:string\"/></proto></example></p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">This function is <termref def=\"dt-deterministic\">deterministic</termref>, <termref def=\"dt-context-dependent\">context-dependent</termref>,  and <termref def=\"dt-focus-independent\">focus-independent</termref>.  It depends on\n \t\tstatic base uri.\n \t</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The <code>$href</code> argument <rfc2119>must</rfc2119> be a string in the form of a URI\n             reference, which <rfc2119>must</rfc2119> contain no fragment identifier, and\n                <rfc2119>must</rfc2119> identify a resource\n             <phrase diff=\"chg\" at=\"L\">for which a string representation is available</phrase>. If the URI is a\n             relative URI reference, then it is resolved relative to the\n             <phrase diff=\"chg\" at=\"L\">Static Base URI property from the static context</phrase>.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\" diff=\"add\" at=\"L\">The mapping of URIs to the string representation of a resource is the mapping defined\n             in the <xtermref spec=\"XP30\" ref=\"dt-available-text-resources\">available text resources</xtermref>\n          component of the dynamic context.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">If the value of the <code>$href</code> argument is an empty sequence, the function\n             returns an empty sequence.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The <code>$encoding</code> argument, if present, is the name of an encoding. The values\n             for this attribute follow the same rules as for the <code>encoding</code> attribute in\n             an XML declaration. The only values which every <termref def=\"implementation\">implementation</termref> is <rfc2119>required</rfc2119> to recognize are\n                <code>utf-8</code> and <code>utf-16</code>.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The encoding of the external resource is determined as follows:</p><olist xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\"><item><p>external encoding information is used if available, otherwise</p></item><item><p>if the media type of the resource is <code>text/xml</code> or\n                      <code>application/xml</code> (see <bibref ref=\"rfc2376\"/>), or if it matches\n                   the conventions <code>text/*+xml</code> or <code>application/*+xml</code> (see\n                      <bibref ref=\"rfc3023\"/> and/or its successors), then the encoding is recognized\n                   as specified in <bibref ref=\"REC-xml\"/>, otherwise</p></item><item><p>the value of the <code>$encoding</code> argument is used if present, otherwise</p></item><item><p>the processor <rfc2119>may</rfc2119> use <termref def=\"implementation-defined\">implementation-defined</termref> heuristics to determine the likely encoding,\n                   otherwise</p></item><item><p>UTF-8 is assumed.</p></item></olist><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The result of the function is a string containing the\n             <phrase diff=\"chg\" at=\"L\">string representation</phrase> of the resource retrieved\n             using the URI.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">A <phrase diff=\"add\" at=\"L\">dynamic</phrase> error is raised <errorref class=\"UT\" code=\"1170\"/> if <code>$href</code>\n             contains a fragment identifier, or if it cannot be used to retrieve the\n             <phrase diff=\"chg\" at=\"L\">string representation</phrase> of a resource.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">A <phrase diff=\"add\" at=\"L\">dynamic</phrase> error is raised <errorref class=\"UT\" code=\"1190\"/> <phrase diff=\"add\" at=\"L\">if the value\n             of the <code>$encoding</code> argument is not a valid encoding name, </phrase>if the <termref def=\"dt-processor\">processor</termref> does not support the specified encoding,\n             if the <phrase diff=\"chg\" at=\"L\">string representation</phrase> of the retrieved\n             resource contains octets that cannot be decoded into Unicode <termref def=\"character\">characters</termref> using the specified encoding, or if the resulting characters\n             are not permitted XML characters.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">A <phrase diff=\"add\" at=\"L\">dynamic</phrase> error is raised <errorref class=\"UT\" code=\"1200\"/> if\n                <code>$encoding</code> is absent and the <termref def=\"dt-processor\">processor</termref> cannot infer the encoding using external information and the\n             encoding is not UTF-8.</p></div>\n", "summary" : "<p>  The  fn:unparsed-text  function reads an external resource (for example, a\n             file) and returns  a string representation of the resource .</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "href", "type" : "xs:string", "occurrence" : "?", "description" : "" } ], "returns" : { "type" : "xs:string?", "description" : "" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 2, "name" : "unparsed-text", "qname" : "fn:unparsed-text", "signature" : "($href as xs:string?, $encoding as xs:string) as xs:string? external", "description" : " <div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The <code>fn:unparsed-text</code> function reads an external resource (for example, a\n             file) and returns <phrase diff=\"chg\" at=\"L\">a string representation of the resource</phrase>.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\"><example role=\"signature\"><proto name=\"unparsed-text\" return-type=\"xs:string?\" isOp=\"no\" prefix=\"fn\" returnEmptyOk=\"no\" returnSeq=\"no\" returnVaries=\"no\" isSchema=\"no\" isDatatype=\"no\" isSpecial=\"no\"><arg name=\"href\" type=\"xs:string?\"/></proto></example><example role=\"signature\"><proto name=\"unparsed-text\" return-type=\"xs:string?\" isOp=\"no\" prefix=\"fn\" returnEmptyOk=\"no\" returnSeq=\"no\" returnVaries=\"no\" isSchema=\"no\" isDatatype=\"no\" isSpecial=\"no\"><arg name=\"href\" type=\"xs:string?\"/><arg name=\"encoding\" type=\"xs:string\"/></proto></example></p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">This function is <termref def=\"dt-deterministic\">deterministic</termref>, <termref def=\"dt-context-dependent\">context-dependent</termref>,  and <termref def=\"dt-focus-independent\">focus-independent</termref>.  It depends on\n \t\tstatic base uri.\n \t</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The <code>$href</code> argument <rfc2119>must</rfc2119> be a string in the form of a URI\n             reference, which <rfc2119>must</rfc2119> contain no fragment identifier, and\n                <rfc2119>must</rfc2119> identify a resource\n             <phrase diff=\"chg\" at=\"L\">for which a string representation is available</phrase>. If the URI is a\n             relative URI reference, then it is resolved relative to the\n             <phrase diff=\"chg\" at=\"L\">Static Base URI property from the static context</phrase>.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\" diff=\"add\" at=\"L\">The mapping of URIs to the string representation of a resource is the mapping defined\n             in the <xtermref spec=\"XP30\" ref=\"dt-available-text-resources\">available text resources</xtermref>\n          component of the dynamic context.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">If the value of the <code>$href</code> argument is an empty sequence, the function\n             returns an empty sequence.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The <code>$encoding</code> argument, if present, is the name of an encoding. The values\n             for this attribute follow the same rules as for the <code>encoding</code> attribute in\n             an XML declaration. The only values which every <termref def=\"implementation\">implementation</termref> is <rfc2119>required</rfc2119> to recognize are\n                <code>utf-8</code> and <code>utf-16</code>.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The encoding of the external resource is determined as follows:</p><olist xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\"><item><p>external encoding information is used if available, otherwise</p></item><item><p>if the media type of the resource is <code>text/xml</code> or\n                      <code>application/xml</code> (see <bibref ref=\"rfc2376\"/>), or if it matches\n                   the conventions <code>text/*+xml</code> or <code>application/*+xml</code> (see\n                      <bibref ref=\"rfc3023\"/> and/or its successors), then the encoding is recognized\n                   as specified in <bibref ref=\"REC-xml\"/>, otherwise</p></item><item><p>the value of the <code>$encoding</code> argument is used if present, otherwise</p></item><item><p>the processor <rfc2119>may</rfc2119> use <termref def=\"implementation-defined\">implementation-defined</termref> heuristics to determine the likely encoding,\n                   otherwise</p></item><item><p>UTF-8 is assumed.</p></item></olist><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The result of the function is a string containing the\n             <phrase diff=\"chg\" at=\"L\">string representation</phrase> of the resource retrieved\n             using the URI.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">A <phrase diff=\"add\" at=\"L\">dynamic</phrase> error is raised <errorref class=\"UT\" code=\"1170\"/> if <code>$href</code>\n             contains a fragment identifier, or if it cannot be used to retrieve the\n             <phrase diff=\"chg\" at=\"L\">string representation</phrase> of a resource.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">A <phrase diff=\"add\" at=\"L\">dynamic</phrase> error is raised <errorref class=\"UT\" code=\"1190\"/> <phrase diff=\"add\" at=\"L\">if the value\n             of the <code>$encoding</code> argument is not a valid encoding name, </phrase>if the <termref def=\"dt-processor\">processor</termref> does not support the specified encoding,\n             if the <phrase diff=\"chg\" at=\"L\">string representation</phrase> of the retrieved\n             resource contains octets that cannot be decoded into Unicode <termref def=\"character\">characters</termref> using the specified encoding, or if the resulting characters\n             are not permitted XML characters.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">A <phrase diff=\"add\" at=\"L\">dynamic</phrase> error is raised <errorref class=\"UT\" code=\"1200\"/> if\n                <code>$encoding</code> is absent and the <termref def=\"dt-processor\">processor</termref> cannot infer the encoding using external information and the\n             encoding is not UTF-8.</p></div>\n", "summary" : "<p>  The  fn:unparsed-text  function reads an external resource (for example, a\n             file) and returns  a string representation of the resource .</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "href", "type" : "xs:string", "occurrence" : "?", "description" : "" }, { "name" : "encoding", "type" : "xs:string", "occurrence" : null, "description" : "" } ], "returns" : { "type" : "xs:string?", "description" : "" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 2, "name" : "unparsed-text", "qname" : "fn:unparsed-text", "signature" : "($href as xs:string?, $encoding as xs:string) as xs:string? external", "description" : " <div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The <code>fn:unparsed-text</code> function reads an external resource (for example, a\n             file) and returns <phrase diff=\"chg\" at=\"L\">a string representation of the resource</phrase>.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\"><example role=\"signature\"><proto name=\"unparsed-text\" return-type=\"xs:string?\" isOp=\"no\" prefix=\"fn\" returnEmptyOk=\"no\" returnSeq=\"no\" returnVaries=\"no\" isSchema=\"no\" isDatatype=\"no\" isSpecial=\"no\"><arg name=\"href\" type=\"xs:string?\"/></proto></example><example role=\"signature\"><proto name=\"unparsed-text\" return-type=\"xs:string?\" isOp=\"no\" prefix=\"fn\" returnEmptyOk=\"no\" returnSeq=\"no\" returnVaries=\"no\" isSchema=\"no\" isDatatype=\"no\" isSpecial=\"no\"><arg name=\"href\" type=\"xs:string?\"/><arg name=\"encoding\" type=\"xs:string\"/></proto></example></p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">This function is <termref def=\"dt-deterministic\">deterministic</termref>, <termref def=\"dt-context-dependent\">context-dependent</termref>,  and <termref def=\"dt-focus-independent\">focus-independent</termref>.  It depends on\n \t\tstatic base uri.\n \t</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The <code>$href</code> argument <rfc2119>must</rfc2119> be a string in the form of a URI\n             reference, which <rfc2119>must</rfc2119> contain no fragment identifier, and\n                <rfc2119>must</rfc2119> identify a resource\n             <phrase diff=\"chg\" at=\"L\">for which a string representation is available</phrase>. If the URI is a\n             relative URI reference, then it is resolved relative to the\n             <phrase diff=\"chg\" at=\"L\">Static Base URI property from the static context</phrase>.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\" diff=\"add\" at=\"L\">The mapping of URIs to the string representation of a resource is the mapping defined\n             in the <xtermref spec=\"XP30\" ref=\"dt-available-text-resources\">available text resources</xtermref>\n          component of the dynamic context.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">If the value of the <code>$href</code> argument is an empty sequence, the function\n             returns an empty sequence.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The <code>$encoding</code> argument, if present, is the name of an encoding. The values\n             for this attribute follow the same rules as for the <code>encoding</code> attribute in\n             an XML declaration. The only values which every <termref def=\"implementation\">implementation</termref> is <rfc2119>required</rfc2119> to recognize are\n                <code>utf-8</code> and <code>utf-16</code>.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The encoding of the external resource is determined as follows:</p><olist xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\"><item><p>external encoding information is used if available, otherwise</p></item><item><p>if the media type of the resource is <code>text/xml</code> or\n                      <code>application/xml</code> (see <bibref ref=\"rfc2376\"/>), or if it matches\n                   the conventions <code>text/*+xml</code> or <code>application/*+xml</code> (see\n                      <bibref ref=\"rfc3023\"/> and/or its successors), then the encoding is recognized\n                   as specified in <bibref ref=\"REC-xml\"/>, otherwise</p></item><item><p>the value of the <code>$encoding</code> argument is used if present, otherwise</p></item><item><p>the processor <rfc2119>may</rfc2119> use <termref def=\"implementation-defined\">implementation-defined</termref> heuristics to determine the likely encoding,\n                   otherwise</p></item><item><p>UTF-8 is assumed.</p></item></olist><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The result of the function is a string containing the\n             <phrase diff=\"chg\" at=\"L\">string representation</phrase> of the resource retrieved\n             using the URI.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">A <phrase diff=\"add\" at=\"L\">dynamic</phrase> error is raised <errorref class=\"UT\" code=\"1170\"/> if <code>$href</code>\n             contains a fragment identifier, or if it cannot be used to retrieve the\n             <phrase diff=\"chg\" at=\"L\">string representation</phrase> of a resource.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">A <phrase diff=\"add\" at=\"L\">dynamic</phrase> error is raised <errorref class=\"UT\" code=\"1190\"/> <phrase diff=\"add\" at=\"L\">if the value\n             of the <code>$encoding</code> argument is not a valid encoding name, </phrase>if the <termref def=\"dt-processor\">processor</termref> does not support the specified encoding,\n             if the <phrase diff=\"chg\" at=\"L\">string representation</phrase> of the retrieved\n             resource contains octets that cannot be decoded into Unicode <termref def=\"character\">characters</termref> using the specified encoding, or if the resulting characters\n             are not permitted XML characters.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">A <phrase diff=\"add\" at=\"L\">dynamic</phrase> error is raised <errorref class=\"UT\" code=\"1200\"/> if\n                <code>$encoding</code> is absent and the <termref def=\"dt-processor\">processor</termref> cannot infer the encoding using external information and the\n             encoding is not UTF-8.</p></div>\n", "summary" : "<p>  The  fn:unparsed-text  function reads an external resource (for example, a\n             file) and returns  a string representation of the resource .</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "href", "type" : "xs:string", "occurrence" : "?", "description" : "" }, { "name" : "encoding", "type" : "xs:string", "occurrence" : null, "description" : "" } ], "returns" : { "type" : "xs:string?", "description" : "" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "upper-case", "qname" : "fn:upper-case", "signature" : "($arg as xs:string?) as xs:string external", "description" : " <div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">Converts a string to upper case.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\"><example role=\"signature\"><proto name=\"upper-case\" return-type=\"xs:string\" isOp=\"no\" prefix=\"fn\" returnEmptyOk=\"no\" returnSeq=\"no\" returnVaries=\"no\" isSchema=\"no\" isDatatype=\"no\" isSpecial=\"no\"><arg name=\"arg\" type=\"xs:string?\"/></proto></example></p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">This function is <termref def=\"dt-deterministic\">deterministic</termref>, <termref def=\"dt-context-independent\">context-independent</termref>,  and <termref def=\"dt-focus-independent\">focus-independent</termref>. </p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">If the value of <code>$arg</code> is the empty sequence, the zero-length string is\n             returned.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">Otherwise, the function returns the value of <code>$arg</code> after translating every\n                <termref def=\"character\">character</termref> to its upper-case correspondent as\n             defined in the appropriate case mappings section in the Unicode standard <bibref ref=\"Unicode\"/>. For versions of Unicode beginning with the 2.1.8 update, only\n             locale-insensitive case mappings should be applied. Beginning with version 3.2.0 (and\n             likely future versions) of Unicode, precise mappings are described in default case\n             operations, which are full case mappings in the absence of tailoring for particular\n             languages and environments. Every lower-case character that does not have an upper-case\n             correspondent, as well as every upper-case character, is included in the returned value\n             in its original form. </p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">Case mappings may change the length of a string. In general, the\n                <code>fn:upper-case</code> and <code>fn:lower-case</code> functions are not inverses\n             of each other: <code>fn:lower-case(fn:upper-case($arg))</code> is not guaranteed to\n             return <code>$arg</code>, nor is <code>fn:upper-case(fn:lower-case($arg))</code>. The\n             Latin small letter dotless i (as used in Turkish) is perhaps the most prominent\n             lower-case letter which will not round-trip. The Latin capital letter i with dot above\n             is the most prominent upper-case letter which will not round trip; there are others,\n             such as Latin capital letter Sharp S (#1E9E) which is introduced in Unicode 5.1.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\"> These functions may not always be linguistically appropriate (e.g. Turkish i without\n             dot) or appropriate for the application (e.g. titlecase). In cases such as Turkish, a\n             simple translation should be used first.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\"> Because the function is not sensitive to locale, results will not always match user\n             expectations. In Quebec, for example, the standard uppercase equivalent of \"è\" is \"È\",\n             while in metropolitan France it is more commonly \"E\"; only one of these is supported by\n             the functions as defined.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\"> Many characters of class Ll lack uppercase equivalents in the Unicode case mapping\n             tables; many characters of class Lu lack lowercase equivalents.</p></div>\n", "summary" : "<p>  Converts a string to upper case.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "arg", "type" : "xs:string", "occurrence" : "?", "description" : "" } ], "returns" : { "type" : "xs:string", "description" : "" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 0, "name" : "uri-collection", "qname" : "fn:uri-collection", "signature" : "() as xs:anyURI* external", "description" : " <div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">Returns a sequence of <code>xs:anyURI</code> values representing the URIs in a resource collection.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\"><example role=\"signature\"><proto name=\"uri-collection\" return-type=\"xs:anyURI*\" isOp=\"no\" prefix=\"fn\" returnEmptyOk=\"no\" returnSeq=\"no\" returnVaries=\"no\" isSchema=\"no\" isDatatype=\"no\" isSpecial=\"no\"/></example><example role=\"signature\"><proto name=\"uri-collection\" return-type=\"xs:anyURI*\" isOp=\"no\" prefix=\"fn\" returnEmptyOk=\"no\" returnSeq=\"no\" returnVaries=\"no\" isSchema=\"no\" isDatatype=\"no\" isSpecial=\"no\"><arg name=\"arg\" type=\"xs:string?\"/></proto></example></p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">This function is <termref def=\"dt-deterministic\">deterministic</termref>, <termref def=\"dt-context-dependent\">context-dependent</termref>,  and <termref def=\"dt-focus-independent\">focus-independent</termref>.  It depends on\n \t\tavailable resource collections, and static base uri.\n \t</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The zero-argument form of the function returns the URIs in the <term>Default resource collection</term>\n             described in <xspecref spec=\"XP30\" ref=\"id-xp-evaluation-context-components\"/>.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">If the value of <code>$arg</code> is a relative <code>xs:anyURI</code>, it is resolved\n             against the value of the base-URI property from the static context. </p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">If <code>$arg</code> is the empty sequence, the function behaves as if it had been\n             called without an argument. See above.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The single-argument form of the function returns the sequence of URIs corresponding to the\n             supplied URI in the <term>Available resource collections</term> described in\n             <xspecref spec=\"XP30\" ref=\"id-xp-evaluation-context-components\"/>.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">A <phrase diff=\"add\" at=\"L\">dynamic</phrase> error is raised <errorref class=\"DC\" code=\"0002\"/> if no URI is supplied (that is, if the\n             the function is called with no arguments, or with a single argument that evaluates to an empty sequence), and the\n             value of the default resource collection is <xtermref ref=\"dt-absent\" spec=\"DM30\">absent</xtermref>.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\" diff=\"chg\" at=\"J\">A <phrase diff=\"add\" at=\"L\">dynamic</phrase> error is raised <errorref class=\"DC\" code=\"0002\"/>\n             if <term>available resource collections</term> provides no mapping for the absolutized URI.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">A <phrase diff=\"add\" at=\"L\">dynamic</phrase> error is raised <errorref class=\"DC\" code=\"0004\"/> if <code>$arg</code> is not a\n             valid <code>xs:anyURI</code>.</p></div>\n", "summary" : "<p>  Returns a sequence of  xs:anyURI  values representing the URIs in a resource collection.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [  ], "returns" : { "type" : "xs:anyURI*", "description" : "" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 0, "name" : "uri-collection", "qname" : "fn:uri-collection", "signature" : "() as xs:anyURI* external", "description" : " <div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">Returns a sequence of <code>xs:anyURI</code> values representing the URIs in a resource collection.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\"><example role=\"signature\"><proto name=\"uri-collection\" return-type=\"xs:anyURI*\" isOp=\"no\" prefix=\"fn\" returnEmptyOk=\"no\" returnSeq=\"no\" returnVaries=\"no\" isSchema=\"no\" isDatatype=\"no\" isSpecial=\"no\"/></example><example role=\"signature\"><proto name=\"uri-collection\" return-type=\"xs:anyURI*\" isOp=\"no\" prefix=\"fn\" returnEmptyOk=\"no\" returnSeq=\"no\" returnVaries=\"no\" isSchema=\"no\" isDatatype=\"no\" isSpecial=\"no\"><arg name=\"arg\" type=\"xs:string?\"/></proto></example></p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">This function is <termref def=\"dt-deterministic\">deterministic</termref>, <termref def=\"dt-context-dependent\">context-dependent</termref>,  and <termref def=\"dt-focus-independent\">focus-independent</termref>.  It depends on\n \t\tavailable resource collections, and static base uri.\n \t</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The zero-argument form of the function returns the URIs in the <term>Default resource collection</term>\n             described in <xspecref spec=\"XP30\" ref=\"id-xp-evaluation-context-components\"/>.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">If the value of <code>$arg</code> is a relative <code>xs:anyURI</code>, it is resolved\n             against the value of the base-URI property from the static context. </p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">If <code>$arg</code> is the empty sequence, the function behaves as if it had been\n             called without an argument. See above.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The single-argument form of the function returns the sequence of URIs corresponding to the\n             supplied URI in the <term>Available resource collections</term> described in\n             <xspecref spec=\"XP30\" ref=\"id-xp-evaluation-context-components\"/>.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">A <phrase diff=\"add\" at=\"L\">dynamic</phrase> error is raised <errorref class=\"DC\" code=\"0002\"/> if no URI is supplied (that is, if the\n             the function is called with no arguments, or with a single argument that evaluates to an empty sequence), and the\n             value of the default resource collection is <xtermref ref=\"dt-absent\" spec=\"DM30\">absent</xtermref>.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\" diff=\"chg\" at=\"J\">A <phrase diff=\"add\" at=\"L\">dynamic</phrase> error is raised <errorref class=\"DC\" code=\"0002\"/>\n             if <term>available resource collections</term> provides no mapping for the absolutized URI.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">A <phrase diff=\"add\" at=\"L\">dynamic</phrase> error is raised <errorref class=\"DC\" code=\"0004\"/> if <code>$arg</code> is not a\n             valid <code>xs:anyURI</code>.</p></div>\n", "summary" : "<p>  Returns a sequence of  xs:anyURI  values representing the URIs in a resource collection.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [  ], "returns" : { "type" : "xs:anyURI*", "description" : "" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "uri-collection", "qname" : "fn:uri-collection", "signature" : "($arg as xs:string?) as xs:anyURI* external", "description" : " <div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">Returns a sequence of <code>xs:anyURI</code> values representing the URIs in a resource collection.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\"><example role=\"signature\"><proto name=\"uri-collection\" return-type=\"xs:anyURI*\" isOp=\"no\" prefix=\"fn\" returnEmptyOk=\"no\" returnSeq=\"no\" returnVaries=\"no\" isSchema=\"no\" isDatatype=\"no\" isSpecial=\"no\"/></example><example role=\"signature\"><proto name=\"uri-collection\" return-type=\"xs:anyURI*\" isOp=\"no\" prefix=\"fn\" returnEmptyOk=\"no\" returnSeq=\"no\" returnVaries=\"no\" isSchema=\"no\" isDatatype=\"no\" isSpecial=\"no\"><arg name=\"arg\" type=\"xs:string?\"/></proto></example></p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">This function is <termref def=\"dt-deterministic\">deterministic</termref>, <termref def=\"dt-context-dependent\">context-dependent</termref>,  and <termref def=\"dt-focus-independent\">focus-independent</termref>.  It depends on\n \t\tavailable resource collections, and static base uri.\n \t</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The zero-argument form of the function returns the URIs in the <term>Default resource collection</term>\n             described in <xspecref spec=\"XP30\" ref=\"id-xp-evaluation-context-components\"/>.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">If the value of <code>$arg</code> is a relative <code>xs:anyURI</code>, it is resolved\n             against the value of the base-URI property from the static context. </p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">If <code>$arg</code> is the empty sequence, the function behaves as if it had been\n             called without an argument. See above.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The single-argument form of the function returns the sequence of URIs corresponding to the\n             supplied URI in the <term>Available resource collections</term> described in\n             <xspecref spec=\"XP30\" ref=\"id-xp-evaluation-context-components\"/>.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">A <phrase diff=\"add\" at=\"L\">dynamic</phrase> error is raised <errorref class=\"DC\" code=\"0002\"/> if no URI is supplied (that is, if the\n             the function is called with no arguments, or with a single argument that evaluates to an empty sequence), and the\n             value of the default resource collection is <xtermref ref=\"dt-absent\" spec=\"DM30\">absent</xtermref>.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\" diff=\"chg\" at=\"J\">A <phrase diff=\"add\" at=\"L\">dynamic</phrase> error is raised <errorref class=\"DC\" code=\"0002\"/>\n             if <term>available resource collections</term> provides no mapping for the absolutized URI.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">A <phrase diff=\"add\" at=\"L\">dynamic</phrase> error is raised <errorref class=\"DC\" code=\"0004\"/> if <code>$arg</code> is not a\n             valid <code>xs:anyURI</code>.</p></div>\n", "summary" : "<p>  Returns a sequence of  xs:anyURI  values representing the URIs in a resource collection.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "arg", "type" : "xs:string", "occurrence" : "?", "description" : "" } ], "returns" : { "type" : "xs:anyURI*", "description" : "" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "uri-collection", "qname" : "fn:uri-collection", "signature" : "($arg as xs:string?) as xs:anyURI* external", "description" : " <div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">Returns a sequence of <code>xs:anyURI</code> values representing the URIs in a resource collection.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\"><example role=\"signature\"><proto name=\"uri-collection\" return-type=\"xs:anyURI*\" isOp=\"no\" prefix=\"fn\" returnEmptyOk=\"no\" returnSeq=\"no\" returnVaries=\"no\" isSchema=\"no\" isDatatype=\"no\" isSpecial=\"no\"/></example><example role=\"signature\"><proto name=\"uri-collection\" return-type=\"xs:anyURI*\" isOp=\"no\" prefix=\"fn\" returnEmptyOk=\"no\" returnSeq=\"no\" returnVaries=\"no\" isSchema=\"no\" isDatatype=\"no\" isSpecial=\"no\"><arg name=\"arg\" type=\"xs:string?\"/></proto></example></p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">This function is <termref def=\"dt-deterministic\">deterministic</termref>, <termref def=\"dt-context-dependent\">context-dependent</termref>,  and <termref def=\"dt-focus-independent\">focus-independent</termref>.  It depends on\n \t\tavailable resource collections, and static base uri.\n \t</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The zero-argument form of the function returns the URIs in the <term>Default resource collection</term>\n             described in <xspecref spec=\"XP30\" ref=\"id-xp-evaluation-context-components\"/>.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">If the value of <code>$arg</code> is a relative <code>xs:anyURI</code>, it is resolved\n             against the value of the base-URI property from the static context. </p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">If <code>$arg</code> is the empty sequence, the function behaves as if it had been\n             called without an argument. See above.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The single-argument form of the function returns the sequence of URIs corresponding to the\n             supplied URI in the <term>Available resource collections</term> described in\n             <xspecref spec=\"XP30\" ref=\"id-xp-evaluation-context-components\"/>.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">A <phrase diff=\"add\" at=\"L\">dynamic</phrase> error is raised <errorref class=\"DC\" code=\"0002\"/> if no URI is supplied (that is, if the\n             the function is called with no arguments, or with a single argument that evaluates to an empty sequence), and the\n             value of the default resource collection is <xtermref ref=\"dt-absent\" spec=\"DM30\">absent</xtermref>.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\" diff=\"chg\" at=\"J\">A <phrase diff=\"add\" at=\"L\">dynamic</phrase> error is raised <errorref class=\"DC\" code=\"0002\"/>\n             if <term>available resource collections</term> provides no mapping for the absolutized URI.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">A <phrase diff=\"add\" at=\"L\">dynamic</phrase> error is raised <errorref class=\"DC\" code=\"0004\"/> if <code>$arg</code> is not a\n             valid <code>xs:anyURI</code>.</p></div>\n", "summary" : "<p>  Returns a sequence of  xs:anyURI  values representing the URIs in a resource collection.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "arg", "type" : "xs:string", "occurrence" : "?", "description" : "" } ], "returns" : { "type" : "xs:anyURI*", "description" : "" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "year-from-date", "qname" : "fn:year-from-date", "signature" : "($arg as xs:date?) as xs:integer? external", "description" : " <div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">Returns the year component of an <code>xs:date</code>.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\"><example role=\"signature\"><proto name=\"year-from-date\" return-type=\"xs:integer?\" isOp=\"no\" prefix=\"fn\" returnEmptyOk=\"no\" returnSeq=\"no\" returnVaries=\"no\" isSchema=\"no\" isDatatype=\"no\" isSpecial=\"no\"><arg name=\"arg\" type=\"xs:date?\"/></proto></example></p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">This function is <termref def=\"dt-deterministic\">deterministic</termref>, <termref def=\"dt-context-independent\">context-independent</termref>,  and <termref def=\"dt-focus-independent\">focus-independent</termref>. </p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">If <code>$arg</code> is the empty sequence, the function returns the empty sequence.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">Otherwise, the function returns an <code>xs:integer</code> representing the year in the\n             local value of <code>$arg</code>. The value may be negative. </p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The expression <code>fn:year-from-date(xs:date(\"1999-05-31\"))</code> returns <code>1999</code>.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The expression <code>fn:year-from-date(xs:date(\"2000-01-01+05:00\"))</code> returns <code>2000</code>.</p></div>\n", "summary" : "<p>  Returns the year component of an  xs:date .</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "arg", "type" : "xs:date", "occurrence" : "?", "description" : "" } ], "returns" : { "type" : "xs:integer?", "description" : "" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "years-from-duration", "qname" : "fn:years-from-duration", "signature" : "($arg as xs:duration?) as xs:integer? external", "description" : " <div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">Returns the number of years in a duration.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\"><example role=\"signature\"><proto name=\"years-from-duration\" return-type=\"xs:integer?\" isOp=\"no\" prefix=\"fn\" returnEmptyOk=\"no\" returnSeq=\"no\" returnVaries=\"no\" isSchema=\"no\" isDatatype=\"no\" isSpecial=\"no\"><arg name=\"arg\" type=\"xs:duration?\"/></proto></example></p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">This function is <termref def=\"dt-deterministic\">deterministic</termref>, <termref def=\"dt-context-independent\">context-independent</termref>,  and <termref def=\"dt-focus-independent\">focus-independent</termref>. </p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">If <code>$arg</code> is the empty sequence, the function returns the empty sequence.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">Otherwise, the function returns an <code>xs:integer</code> representing the years\n             component in the value of <code>$arg</code>. The result is obtained by casting\n                <code>$arg</code> to an <code>xs:yearMonthDuration</code> (see <specref ref=\"casting-to-durations\"/>) and then computing the years component as described in\n                <specref ref=\"canonical-yearMonthDuration\"/>.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">If <code>$arg</code> is a negative duration then the result will be negative..</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">If <code>$arg</code> is an <code>xs:dayTimeDuration</code> the function returns 0.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The expression <code>fn:years-from-duration(xs:yearMonthDuration(\"P20Y15M\"))</code> returns <code>21</code>.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The expression <code>fn:years-from-duration(xs:yearMonthDuration(\"-P15M\"))</code> returns <code>-1</code>.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The expression <code>fn:years-from-duration(xs:dayTimeDuration(\"-P2DT15H\"))</code> returns <code>0</code>.</p></div>\n", "summary" : "<p>  Returns the number of years in a duration.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "arg", "type" : "xs:duration", "occurrence" : "?", "description" : "" } ], "returns" : { "type" : "xs:integer?", "description" : "" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "zero-or-one", "qname" : "fn:zero-or-one", "signature" : "($arg as item()*) as item()? external", "description" : " <div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">Returns <code>$arg</code> if it contains zero or one items. Otherwise, raises\n             an error.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\"><example role=\"signature\"><proto name=\"zero-or-one\" return-type=\"item()?\" isOp=\"no\" prefix=\"fn\" returnEmptyOk=\"no\" returnSeq=\"no\" returnVaries=\"no\" isSchema=\"no\" isDatatype=\"no\" isSpecial=\"no\"><arg name=\"arg\" type=\"item()*\"/></proto></example></p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">This function is <termref def=\"dt-deterministic\">deterministic</termref>, <termref def=\"dt-context-independent\">context-independent</termref>,  and <termref def=\"dt-focus-independent\">focus-independent</termref>. </p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">Except in error cases, the function returns <code>$arg</code> unchanged.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\" diff=\"del\" at=\"F\">For detailed type semantics, see [Formal Semantics].</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">A <phrase diff=\"add\" at=\"L\">dynamic</phrase> error is raised <errorref class=\"RG\" code=\"0003\"/> if <code>$arg</code> contains more\n             than one item.</p></div>\n", "summary" : "<p>  Returns  $arg  if it contains zero or one items.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "arg", "type" : "item()", "occurrence" : "*", "description" : "" } ], "returns" : { "type" : "item()?", "description" : "" }, "errors" : [  ] } ], "variables" : [  ] }, "http://zorba.io/modules/csv" : { "ns" : "http://zorba.io/modules/csv", "description" : " Function library providing converters from CSV/TXT to XML and back.\n The functions are optimized to work with large amounts of data, in a streaming way.\n", "sees" : [  ], "authors" : [ "<xqdoc:author xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Daniel Turcanu</xqdoc:author>" ], "version" : null, "encoding" : "utf-8", "namespaces" : [ { "uri" : "http://zorba.io/modules/csv", "prefix" : "csv" }, { "uri" : "http://zorba.io/modules/csv-options", "prefix" : "csv-options" }, { "uri" : "http://zorba.io/modules/schema", "prefix" : "schemaOptions" }, { "uri" : "http://zorba.io/options/versioning", "prefix" : "ver" } ], "functions" : [ { "isDocumented" : true, "arity" : 2, "name" : "parse", "qname" : "csv:parse", "signature" : "($csv as xs:string, $options as element(csv-options:options)?) as element(*)*", "description" : " Parse a CSV or fixed size text and convert to XML.<br xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n By default each line is converted to a &lt;row&gt; element, and each field to a &lt;column&gt; element inside &lt;row&gt;.<br xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n The format of the param $options is:<br xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n  <pre xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">\n    &lt;csv-options:options&gt;\n        &lt;csv  [separator=\"default comma ,\"] ?\n          [quote-char=\"default double quotes &amp;quote;\"]?\n          [quote-escape=\"default double double quotes &amp;quote;&amp;quote;\"]? /&gt;\n        or\n        &lt;column-widths&gt;\n          &lt;column-width&gt;<i>[column fixed width, unsigned int]</i>&lt;column-width&gt;*\n        &lt;/column-widths&gt;\n        or\n        &lt;column-positions&gt;\n          &lt;column-position&gt;<i>[column position on line, unsigned int]</i>&lt;column-position&gt;*\n        &lt;/column-positions&gt;\n        &lt;first-row-is-header [line=\"<i>first_line[-last_line]?</i>\"]?/&gt;?\n        &lt;start-from-row line=\"<i>first_line[-last_line]?</i>\"/&gt;?\n        &lt;add-last-void-columns/&gt;?\n        &lt;xml-nodes&gt;\n          [&lt;<i>row-name</i>&gt;\n            [&lt;<i>column-name/</i>&gt;]?\n          &lt;/<i>row-name</i>&gt;]?\n        &lt;/xml-nodes&gt;?\n    &lt;/csv-options:options&gt;\n  </pre>\n    All the parameters are optional and can appear in any order.<br xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n    All the parameters are case sensitive. The namespace used is \"http://zorba.io/modules/csv-options\".<br xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n    All strings must have UTF-8 encoding.<br xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n    Parameters csv, column-widths, column-positions are mutually exclusive. If none is specified,\n    the input string is assumed to be csv.<br xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n    Description of parameters:\n    <dl xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">\n     <dt><b>csv</b></dt>\n     <dd> Specifies the parameters for parsing a csv string.<br/>\n       <dl>\n        <dt><b>separator</b></dt>\n        <dd>The character or group of characters used to separating fields in a row.\n            If it is not specified, it defaults to comma ','.\n        </dd>\n        <dt><b>quote-char</b></dt>\n        <dd>The character or group of characters used for quoting the fields that may contain special characters,\n             like separator, new line or this quote char. The default value is double quote \".<br/>\n        </dd>\n        <dt><b>quote-escape</b></dt>\n        <dd>The group of characters used for escaping the quote char inside a field. The whole quote escape group\n           is translated to a quote char during parsing. The default value is double double quotes \"\".<br/>\n        </dd>\n       </dl>\n     </dd>\n     <br/>\n     <dt><b>column-widths</b></dt>\n     <dd>Specifies the column widths for fixed size text. It contains multiple column-width child elements\n        specifying the fixed width of each column, from left to right.<br/>\n        If the line has more fields than specified, they are ignored.\n     </dd>\n     <dt><b>column-positions</b></dt>\n     <dd>This is an alternative to column-widths, and specifies instead the starting position of each column.\n        Column positions are 1 based, and are specified in order from left to right.\n        The last column is read until end of line. The first column position can be greater than 1, if you want\n        to parse only a part of the input text.\n     </dd>\n     <dt><b>first-row-is-header</b></dt>\n     <dd>The presence of this element indicates that the first row is to be treated as the name of the columns.\n        If it is not present, then each field is enclosed in a &lt;column&gt; element,\n        or how it is specified in &lt;xml-nodes&gt; parameter.<br/>\n        If the first row is the header, then each field is enclosed in an element with the corresponding name from the header.<br/>\n        For example, the csv:\n        <pre>\n        <i>ID,Name,Occupation\n        1,John,student</i>\n        </pre>\n        is parsed into:\n        <pre>\n        <i>&lt;row&gt;\n        &lt;ID&gt;1&lt;/ID&gt;\n        &lt;Name&gt;John&lt;/Name&gt;\n        &lt;Occupation&gt;student&lt;/Occupation&gt;\n        &lt;/row&gt;</i>\n        </pre>\n        If the header names contain characters that cannot be used in a QName, they are replaced with underscore '_'.<br/>\n        The namespace for the header QNames is taken from the column name specified in xml-nodes parameter, or from\n        the row name, or if that doesn't exist either then empty namespace is used. <br/>\n        If the header is not the first line in the input string, the starting line can be specified in the <b>line</b> attribute.<br/>\n        If a column does not have a name, a new name is constructed in the form <i>columnN</i> where N is the position of the column,\n        starting from 1.<br/>\n        <b>Subheaders</b><br/>\n        If the header consists of more than one line, this can be specified in the <b>line</b> attribute in the form\n        \"<i>first_line - last_line</i>\". Having more lines as the header translates into a hierarchy of elements in the xml.<br/>\n        For example, the csv:\n        <pre>\n        <i>ID,Name,,Occupation\n        ,First Name,Last Name,\n        1,John,Howard,student</i>\n        </pre>\n        is parsed into:\n        <pre>\n        <i>&lt;row&gt;\n        &lt;ID&gt;1&lt;/ID&gt;\n        &lt;Name&gt;\n          &lt;First_Name&gt;John&lt;/First_Name&gt;\n          &lt;Last_Name&gt;Howard&lt;/Last_Name&gt;\n        &lt;/Name&gt;\n        &lt;Occupation&gt;student&lt;/Occupation&gt;\n        &lt;/row&gt;</i>\n        </pre>\n        This element can have an attribute \"accept-all-lines\" with values \"false\" or \"true\" (default \"false\").\n        When set to true it tells the parser to not report lines that do not have the same number of items as\n        the header. If set to false, the parser will raise a csv:WrongInput error for these lines.<br/>\n     </dd>\n     <dt><b>start-from-row</b></dt>\n     <dd>If the data does not start from line 1 or immediately after the header,\n        you can specify the starting line in the <b>line</b> attribute.<br/>\n        Also you can use this attribute in the form \"<i>first_line - last_line</i>\" to specify also the last line\n        if you don't want the whole csv to be parsed.\n     </dd>\n     <dt><b>add-last-void-columns</b></dt>\n     <dd>In the case when using headers and some data lines are shorter than the header, by default the excess columns are ignored\n          for those lines. You can set the add-last-void-columns parameter to make all the columns appear in xml even if they are void.\n     </dd>\n     <dt><b>xml-nodes</b></dt>\n     <dd>With this parameter you can specify the names for the row element and for the column element if there is no header.<br/>\n        The first element child of this element specifies the desired QName of the row element in the output xml.\n        The name of this element will be used as the name of the row element.<br/>\n        The element child of this row element is the column element, and its name will be used as the name of the column elements\n        that enclose the fields in the output xml if there is no header. <br/>\n        If the csv has a header, only the namespace is used from the column element.<br/>\n        For example, with parameter:\n        <pre>\n        <i>&lt;xml-nodes&gt;\n        &lt;r&gt;\n          &lt;c/&gt;\n        &lt;/r&gt;\n        &lt;/xml-nodes&gt;</i>\n        </pre>\n        the output for each line will look like:\n        <pre>\n        <i>&lt;r&gt;\n          &lt;c&gt;field1&lt;/c&gt;\n          &lt;c&gt;field2&lt;/c&gt;\n          .......\n        &lt;/r&gt;</i>\n        </pre>\n     </dd>\n    </dl>\n", "summary" : "<p> Parse a CSV or fixed size text and convert to XML.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "csv", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the string containing the csv or fixed size text.</div>" }, { "name" : "options", "type" : "element(csv-options:options)", "occurrence" : "?", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> this parameter is validated against \"http://zorba.io/modules/csv-options\" schema. If this parameter is not specified, the row name is by default \"row\" and the column name is by default \"column\".</div>" } ], "returns" : { "type" : "element(*)*", "description" : "a sequence of row elements, one for each line in csv" }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">csv:CSV001 if the input string is streamable string and cannot be rewinded</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">csv:WrongInput if the input string has lines with variable number of items, and the csv has headers and the options do not specify the ignore-foreign-input attribute</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">err:XQDY0027 if $options can not be validated against the csv-options schema</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">err:XQDY0084 if the options parameter doesn't have the name \"csv-options:options\".</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 2, "name" : "serialize", "qname" : "csv:serialize", "signature" : "($xml as element(*)*, $options as element(csv-options:options)?) as xs:string", "description" : " Convert XML into CSV or fixed size text.\n Note: if you want to serialize out the result, make sure that the serializer method is set to \"text\".\n For example, in zorba command line, you have to set the param --serialize-text.\n When using the <pre xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">file:write(...)</pre> function, you have to set the\n method serialization parameter to \"text\":\n <pre xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">\n &lt;output:serialization-parameters&lt;\n   &lt;output:method value=\"text\"/&lt;\n &lt;/output:serialization-parameters&lt;\n </pre>\n The <pre xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">$options</pre> parameter must have the following format:\n <pre xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">\n    &lt;csv-options:options&gt;<br/>\n        &lt;csv  [separator=\"default comma ,\"] ? <br/>\n          [quote-char=\"default double quotes &amp;quote;\"]? <br/>\n          [quote-escape=\"default double double quotes &amp;quote;&amp;quote;\"]? /&gt; <br/>\n        <br/>\n        or<br/>\n        &lt;column-widths [align=\"left|right\"]?&gt;<br/>\n          &lt;column-width [align=\"left|right\"]?&gt;<i>[column fixed width, unsigned int]</i>&lt;column-width&gt;*<br/>\n        &lt;/column-widths&gt;<br/>\n        <br/>\n        or<br/>\n        &lt;column-positions [align=\"left|right\"]?&gt;<br/>\n          &lt;column-position [align=\"left|right\"]?&gt;<i>[column position on line, unsigned int]</i>&lt;column-position&gt;*<br/>\n        &lt;/column-positions&gt;<br/>\n        <br/>\n        &lt;first-row-is-header/&gt;?<br/>\n    &lt;/csv-options:options&gt;\n </pre>\n All the parameters are optional and can appear in any order.<br xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n All the parameters are case sensitive. The namespace used is \"http://zorba.io/modules/csv-options\".<br xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n All strings must have UTF-8 encoding.<br xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n Parameters csv, column-widths, column-positions are mutually exclusive.\n If none is specified, the xml is converted to csv.\n Description of parameters:\n    <dl xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">\n     <dt><b>csv</b></dt>\n     <dd> Specifies the parameters for converting to csv.<br/>\n       <dl>\n        <dt><b>separator</b></dt>\n        <dd>The character or group of characters used to separating fields in a row.\n            If it is not specified, it defaults to comma ','.\n        </dd>\n        <dt><b>quote-char</b></dt>\n        <dd>The character or group of characters used for quoting the fields that may contain special characters,\n             like separator, new line or this quote char. The default value is double quote \".<br/>\n        </dd>\n        <dt><b>quote-escape</b></dt>\n        <dd>The group of characters used for escaping the quote char inside a field. The whole quote escape group\n           is translated to a quote char during parsing. The default value is double double quotes \"\".<br/>\n        </dd>\n       </dl>\n     </dd>\n     <br/>\n     <dt><b>column-widths</b></dt>\n     <dd>Specifies the column widths for fixed size text. It contains multiple column-width child elements\n        specifying the fixed width of each column, from left to right.<br/>\n        With the attribute <b>align</b> you can specify how to align fields that are smaller than the column width.\n        The default alignment is left.<br/>\n     </dd>\n     <dt><b>column-positions</b></dt>\n     <dd>This is an alternative to column-widths, and specifies instead the starting position of each column.\n        Column positions are 1 based, and are specified in order from left to right.\n        The last column has a variable length.<br/>\n        With the attribute <b>align</b> you can specify how to align fields that are smaller than the column width.\n        The default alignment is left. The last column does not need alignment.<br/>\n     </dd>\n     <dt><b>first-row-is-header</b></dt>\n     <dd>The presence of this element indicates that the first row will contain the header, that is, the names of\n        the column elements. Only the column names from the first row element are taken into account.<br/>\n        For example, the row xml:<br/>\n        <i>&lt;row&gt;<br/>\n        &lt;ID&gt;1&lt;/ID&gt;<br/>\n        &lt;Name&gt;John&lt;/Name&gt;<br/>\n        &lt;Occupation&gt;student&lt;/Occupation&gt;<br/>\n        &lt;/row&gt;</i><br/>\n        <br/>\n        is converted to<br/>\n        <i>ID,Name,Occupation<br/>\n        1,John,student</i><br/>\n        <br/>\n        The header names are the localnames of the column elements, and the namespace is ignored.<br/>\n        <b>Subheaders</b><br/>\n        If the row-column hierarchy is more complex, then subheaders are also generated on subsequent lines.\n        The number of subheaders depends on the depth of the column hierarchy.<br/>\n        When generating the subheaders, the non-whitespace text nodes are also taken into account,\n        and a separate column is generated for them too.<br/>\n        For example, the xml row element:<br/>\n        <i>&lt;row&gt;<br/>\n        &lt;ID&gt;1&lt;/ID&gt;<br/>\n        &lt;Name&gt;<br/>\n          Mr.<br/>\n          &lt;First_Name&gt;John&lt;/First_Name&gt;<br/>\n          &lt;Last_Name&gt;Howard&lt;/Last_Name&gt;<br/>\n        &lt;/Name&gt;<br/>\n        &lt;Occupation&gt;student&lt;/Occupation&gt;<br/>\n        &lt;/row&gt;</i><br/>\n        is converted to<br/>\n        <i>ID,Name,,Occupation<br/>\n        ,,First Name,Last Name,<br/>\n        1,Mr.,John,Howard,student</i><br/>\n        <br/>\n        If first-row-is-header is not specified and the columns have a deeper hierarchy,\n          only the first layer of columns is processed, and the fields are the string values of each column.<br/>\n        This element can have an attribute \"ignore-foreign-input\" with values \"false\" or \"true\" (default \"false\").\n        When set to true it tells the serializer to ignore elements that to not match the header names.\n        If set to false, the serializer will raise a csv:ForeignInput error for these elements.<br/>\n     </dd>\n    </dl>\n", "summary" : "<p> Convert XML into CSV or fixed size text.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "xml", "type" : "element(*)", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> a sequence of elements, each element representing a row. The name of each row element is ignored. The childs of each row are the column fields.</div>" }, { "name" : "options", "type" : "element(csv-options:options)", "occurrence" : "?", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The options parameter. See the function description for details. This parameter is validated against \"http://zorba.io/modules/csv-options\" schema.</div>" } ], "returns" : { "type" : "xs:string", "description" : "the csv or fixed size text as string containing all the lines" }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">csv:CSV003 if the serialize output is streamable string and cannot be reset</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">csv:ForeignInput if there are input elements in subsequent rows that do not match the headers, and the options specify first-row-is-header and do not specify the ignore-foreign-input attribute</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">err:XQDY0027 if $options can not be validated against csv-options schema</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">err:XQDY0084 if the options parameter doesn't have the name \"csv-options:options\".</xqdoc:error>" ] } ], "variables" : [  ] }, "http://zorba.io/modules/reference" : { "ns" : "http://zorba.io/modules/reference", "description" : " The module provides functions to compute an immutable and opaque reference\n for nodes, objects, or arrays and to retrieve such items given their\n identifier, respectively.\n The identifiers are immutable, i.e. a identifier does not change\n during the items lifetime and cannot be reused for another item after the\n original item gets deleted.\n Identifiers are unique, in that, two different items will never have the same\n identifier. A item, at any time during its lifetime, can be retrieved by its\n identifier.\n", "sees" : [  ], "authors" : [ "<xqdoc:author xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Matthias Brantner</xqdoc:author>" ], "version" : null, "encoding" : "utf-8", "namespaces" : [ { "uri" : "http://zorba.io/modules/reference", "prefix" : "ref" }, { "uri" : "http://zorba.io/options/versioning", "prefix" : "ver" }, { "uri" : "http://zorba.io/errors", "prefix" : "zerr" } ], "functions" : [ { "isDocumented" : true, "arity" : 1, "name" : "dereference", "qname" : "ref:dereference", "signature" : "($arg as xs:string) as item()? external", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Returns the node, object, or array identified by the given reference.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">The function returns the empty sequence if the item\n that is referenced does not exist.</p>\n", "summary" : "<p>  Returns the node, object, or array identified by the given reference.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "arg", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the URI of the item to retrieve.</div>" } ], "returns" : { "type" : "item()?", "description" : "the item identified by the URI passed as parameter or the empty-sequence if no item with that URI is found." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "reference", "qname" : "ref:reference", "signature" : "($arg as item()) as xs:string external", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Returns an immutable and opaque reference (with type xs:anyURI) for\n a given node, object, or array.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">The generated identifier is immutable, i.e. a identifier does not\n change during the item's lifetime and cannot be reused for another node after\n the original item gets deleted.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Identifiers are also unique, in that, two different items will never\n have the same identifier.</p>\n A item, at any time during its lifetime, can be retrieved by its\n identifier, using the <tt xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">ref:dereference</tt> function.\n Please note that a reference can only be retrieved for a JSON object or JSON\n array if the item is a member of a collection.\n", "summary" : "<p>  Returns an immutable and opaque reference (with type xs:anyURI) for\n a given node, object, or array.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "arg", "type" : "item()", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the node, object, or array for which the URI should be computed</div>" } ], "returns" : { "type" : "xs:string", "description" : "the opaque URI of the item." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">zerr::ZAPI0080 is raised if the object or array passed as argument is not a member of a collection.</xqdoc:error>" ] } ], "variables" : [  ] }, "http://zorba.io/modules/dctx" : { "ns" : "http://zorba.io/modules/dctx", "description" : " This module provides functions that gets components of the dynamic context.\n", "sees" : [  ], "authors" : [ "<xqdoc:author xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Federico Cavalieri</xqdoc:author>" ], "version" : null, "encoding" : "utf-8", "namespaces" : [ { "uri" : "http://zorba.io/modules/dctx", "prefix" : "dctx" }, { "uri" : "http://zorba.io/options/versioning", "prefix" : "ver" } ], "functions" : [ { "isDocumented" : true, "arity" : 0, "name" : "snapshot-id", "qname" : "dctx:snapshot-id", "signature" : "() as xs:unsignedLong external", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Retrieves the current snapshot id.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">The returned id is opaque and should not be used for reasoning about time.\n The only guarantee is that the value returned by this function increases each\n time a snapshot finishes.</p>\n", "summary" : "<p>  Retrieves the current snapshot id.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [  ], "returns" : { "type" : "xs:unsignedLong", "description" : "the current snapshot id." }, "errors" : [  ] } ], "variables" : [  ] }, "http://xbrl.io/modules/bizql/facts" : { "ns" : "http://xbrl.io/modules/bizql/facts", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">This module provides functions for retrieving facts.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Facts are the smallest reportable piece of information.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Facts have a certain number of characteristics: the archive in\n which they were reported, a number of XBRL aspects (concept, entity, period,\n unit, further XBRL dimensions), as well as profile-specific information.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">With this module, you can retrieve facts by picking the characteristics\n you would like your results to have. You can retrieve a fact with its FID\n (Fact ID). You can extract information about facts (period, entity, etc).\n You can perform a full-text search on fact values, and obtain footnotes.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">If you are interested in the structures in which facts can be organized (such\n as hypercubes), look at the components module.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Facts are stored in a MongoDB datasource called <b>xbrl</b>.</p>\n <h2 xmlns:xqdoc=\"http://www.xqdoc.org/1.0\" id=\"standard_options\">Standard <code>$options</code> Parameter</h2>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Most functions in the BizQL package allow an additional <code>$options</code>\n    parameter. The options parameter is a JSON object allowing the following\n    fields:</p>\n <ul xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">\n <li><b>Hypercube</b>: a hypercube object can be passed with the options to apply\n     implicit filtering for it. Only facts belonging to this hypercube will be\n     returned. Hypercube semantics (such as default dimension values) apply.\n     By default, the dimensionless hypercube is used (no dimensions allowed, no filtering).\n     You can override Hypercube with null to bypass hypercube semantics.</li>\n <li><b>Filter</b>: an object specifying the fields to filter for. Filtering fields\n     can be any field contained in facts, including profile specific fields, e.g.:\n     <pre class=\"ace-static\" ace-mode=\"java\">\n   {\n     Filter:\n       {\n         Archive: \"0000034088-13-000011\",\n         Aspects:\n         {\n           \"us-gaap:DefinedBenefitPlansDisclosuresDefinedBenefitPlansAxis\" :\n             \"us-gaap:ForeignPensionPlansDefinedBenefitMember\"\n         },\n         Profiles: {\n           SEC: {\n             Fiscal: {\n               Year: [2011, 2012]\n             }\n           }\n         }\n       }\n   }\n   </pre>\n   A filter must contain at least on of the fields Archive, Aspects.xbrl:Concept,\n   Aspects.xbrl:Period, or Aspects.xbrl:Entity.</li>\n <li><b>concept-maps</b>:\n   <ol><li>a string which is a name of a report schema that is stored in the\n       reportschemas collection and from which to load a ConceptMap</li>\n       <li>an object which is a ConceptMap network object</li>\n       <li>an array of ConceptMap network objects (to learn more about concept-maps\n       refer to the concept-maps module documentation)</li>\n   </ol></li>\n <li><b>Rules</b>:\n   <ol><li>a string which is a name of a report schema that is stored in the\n       reportschemas collection and from which to load Rules</li>\n       <li>an object which is a Rule object</li>\n       <li>an array of Rule objects</li>\n   </ol></li>\n <li><b>include-footnotes</b>: include XBRL Footnotes in each fact (true | false)</li>\n <li><b>Lang</b>: language identifier according to http://www.ietf.org/rfc/rfc3066.txt,\n     i.e. only return footnotes etc. for this specific language</li>\n <li><b>audit-trail</b>: if set to \"debug\" the audit trails will be more verbose</li>\n <li><b>facts-for-archives-and-concept</b>: to override how underlying facts are\n     resolved, for example with finer-grained, profile-specific filtering (option value\n     must be a function item). facts:facts-for-archives-and-concepts#3 is used by\n     default, but it is possible to supply another function that, for examples, filters\n     irrelevant facts out.</li>\n </ul>\n", "sees" : [  ], "authors" : [ "<xqdoc:author xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Charles Hoffman</xqdoc:author>", "<xqdoc:author xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Matthias Brantner</xqdoc:author>", "<xqdoc:author xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Dennis Knochenwefel</xqdoc:author>", "<xqdoc:author xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Ghislain Fourny</xqdoc:author>" ], "version" : null, "encoding" : "utf-8", "namespaces" : [ { "uri" : "http://xbrl.io/modules/bizql/archives", "prefix" : "archives" }, { "uri" : "http://xbrl.io/modules/bizql/concept-maps", "prefix" : "concept-maps" }, { "uri" : "http://www.28msec.com/modules/credentials", "prefix" : "credentials" }, { "uri" : "http://xbrl.io/modules/bizql/entities", "prefix" : "entities" }, { "uri" : "http://xbrl.io/modules/bizql/facts", "prefix" : "facts" }, { "uri" : "http://xbrl.io/modules/bizql/footnotes", "prefix" : "footnotes" }, { "uri" : "http://xbrl.io/modules/bizql/hypercubes", "prefix" : "hypercubes" }, { "uri" : "http://jsoniq.org/function-library", "prefix" : "j" }, { "uri" : "http://www.28msec.com/modules/mongodb", "prefix" : "mongo" }, { "uri" : "http://zorba.io/modules/reflection", "prefix" : "reflection" }, { "uri" : "http://xbrl.io/modules/bizql/rules", "prefix" : "rules" }, { "uri" : "http://zorba.io/modules/string", "prefix" : "string" }, { "uri" : "http://zorba.io/options/versioning", "prefix" : "ver" }, { "uri" : "http://zorba.io/modules/zorba-query", "prefix" : "zq" } ], "functions" : [ { "isDocumented" : true, "arity" : 1, "name" : "concept-for-fact", "qname" : "facts:concept-for-fact", "signature" : "($fact-or-id as item()) as string", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Retrieves the concept against which a fact is reported.</p>\n", "summary" : "<p>  Retrieves the concept against which a fact is reported.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "fact-or-id", "type" : "item()", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> a fact or its FID.</div>" } ], "returns" : { "type" : "string", "description" : "the concept name." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "decimal-value", "qname" : "facts:decimal-value", "signature" : "($facts as object()*) as decimal", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Returns the value of the given facts in case that it\n    is castable to decimal. If no facts are given or a fact value\n    is not castable to decimal 0 is returned instead.</p>\n", "summary" : "<p>  Returns the value of the given facts in case that it\n    is castable to decimal.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "facts", "type" : "object()", "occurrence" : "*", "description" : "" } ], "returns" : { "type" : "decimal", "description" : "the decimal value of the facts or 0." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "duration-for-fact", "qname" : "facts:duration-for-fact", "signature" : "($fact-or-id as item()) as object()?", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Retrieves the duration period for which a fact was reported.</p>\n", "summary" : "<p>  Retrieves the duration period for which a fact was reported.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "fact-or-id", "type" : "item()", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> a fact or its FID.</div>" } ], "returns" : { "type" : "object()?", "description" : "the duration period as an object with Start and End, or the empty sequence if it is not instant." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "entity-for-fact", "qname" : "facts:entity-for-fact", "signature" : "($fact-or-id as item()) as string", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Retrieves the eid of the entity who reported a fact.</p>\n", "summary" : "<p>  Retrieves the eid of the entity who reported a fact.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "fact-or-id", "type" : "item()", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> a fact or its FID.</div>" } ], "returns" : { "type" : "string", "description" : "the eid." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 2, "name" : "facts-for-archives-and-aspects", "qname" : "facts:facts-for-archives-and-aspects", "signature" : "($archives-or-ids as item()*, $aspects as object()) as object()*", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Return all facts reported in a given archive, and associated with a\n given entity, concept, period and/or other aspects.</p>\n", "summary" : "<p>  Return all facts reported in a given archive, and associated with a\n given entity, concept, period and/or other aspects.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "archives-or-ids", "type" : "item()", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> a sequence of archive or archive IDs to filter (or $facts:ALL_OF_THEM to do not filter on archives).</div>" }, { "name" : "aspects", "type" : "object()", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> an object containing aspects to filter, among which xbrl:Concept, xbrl:Entity and xbrl:Period (at least one of them is mandatory).</div>" } ], "returns" : { "type" : "object()*", "description" : "all facts satisfying all supplied conditions." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 3, "name" : "facts-for-archives-and-aspects", "qname" : "facts:facts-for-archives-and-aspects", "signature" : "($archives-or-ids as item()*, $aspects as object(), $options as object()?) as object()*", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Return all facts associated with a given entity, concept, period\n and/or other aspects.</p>\n", "summary" : "<p>  Return all facts associated with a given entity, concept, period\n and/or other aspects.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "archives-or-ids", "type" : "item()", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> a sequence of archive or archive IDs to filter (or $facts:ALL_OF_THEM to do not filter on archives).</div>" }, { "name" : "aspects", "type" : "object()", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> an object containing aspects to filter, among which xbrl:Concept, xbrl:Entity and xbrl:Period (all optional).</div>" }, { "name" : "options", "type" : "object()", "occurrence" : "?", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> <a href=\"#standard_options\">standard fact retrieving options</a>.</div>" } ], "returns" : { "type" : "object()*", "description" : "all facts satisfying all supplied conditions." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 2, "name" : "facts-for-archives-and-concepts", "qname" : "facts:facts-for-archives-and-concepts", "signature" : "($archives-or-ids as item()*, $concepts as item()*) as object()*", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Return facts associated with given concepts and archives.</p>\n", "summary" : "<p>  Return facts associated with given concepts and archives.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "archives-or-ids", "type" : "item()", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> a sequence of archive or archive IDs to filter (or $facts:ALL_OF_THEM to do not filter on archives).</div>" }, { "name" : "concepts", "type" : "item()", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the concepts (or $facts:ALL_OF_THEM to do no filter on concepts).</div>" } ], "returns" : { "type" : "object()*", "description" : "facts associated with these concepts and archives." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 3, "name" : "facts-for-archives-and-concepts", "qname" : "facts:facts-for-archives-and-concepts", "signature" : "($archives-or-ids as item()*, $concepts as item()*, $options as object()?) as object()*", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Return facts associated with given concepts and archives.</p>\n", "summary" : "<p>  Return facts associated with given concepts and archives.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "archives-or-ids", "type" : "item()", "occurrence" : "*", "description" : "" }, { "name" : "concepts", "type" : "item()", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the concepts (or $facts:ALL_OF_THEM to do no filter on concepts).</div>" }, { "name" : "options", "type" : "object()", "occurrence" : "?", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> <a href=\"#standard_options\">standard fact retrieving options</a>.</div>" } ], "returns" : { "type" : "object()*", "description" : "facts associated with these concepts." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "facts-for-archives", "qname" : "facts:facts-for-archives", "signature" : "($archives-or-ids as item()*) as object()*", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Return all facts reported within a given archive.</p>\n", "summary" : "<p>  Return all facts reported within a given archive.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "archives-or-ids", "type" : "item()", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> a sequence of archives or AIDs to filter.</div>" } ], "returns" : { "type" : "object()*", "description" : "all facts reported in these archives." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "facts-for-aspects", "qname" : "facts:facts-for-aspects", "signature" : "($aspects as object()) as object()*", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Return all facts associated with the given aspects.</p>\n", "summary" : "<p>  Return all facts associated with the given aspects.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "aspects", "type" : "object()", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> an object containing aspects to filter, among which xbrl:Concept, xbrl:Entity and xbrl:Period (at least one of them is mandatory).</div>" } ], "returns" : { "type" : "object()*", "description" : "all facts associated with these aspects." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 2, "name" : "facts-for-aspects", "qname" : "facts:facts-for-aspects", "signature" : "($aspects as object(), $options as object()?) as object()*", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Return all facts associated with the given aspects.</p>\n", "summary" : "<p>  Return all facts associated with the given aspects.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "aspects", "type" : "object()", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> an object containing aspects to filter, among which xbrl:Concept, xbrl:Entity and xbrl:Period (at least one of them is mandatory).</div>" }, { "name" : "options", "type" : "object()", "occurrence" : "?", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> <a href=\"#standard_options\">standard fact retrieving options</a>.</div>" } ], "returns" : { "type" : "object()*", "description" : "all facts associated with these aspects." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "facts-for-concepts", "qname" : "facts:facts-for-concepts", "signature" : "($concepts as string*) as object()*", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Return facts associated with given concepts.</p>\n", "summary" : "<p>  Return facts associated with given concepts.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "concepts", "type" : "string", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the concepts.</div>" } ], "returns" : { "type" : "object()*", "description" : "facts associated with these concepts." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 2, "name" : "facts-for-concepts", "qname" : "facts:facts-for-concepts", "signature" : "($concepts as string*, $options as object()?) as object()*", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Return facts associated with given concepts.</p>\n", "summary" : "<p>  Return facts associated with given concepts.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "concepts", "type" : "string", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the concepts.</div>" }, { "name" : "options", "type" : "object()", "occurrence" : "?", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> <a href=\"#standard_options\">standard fact retrieving options</a>.</div>" } ], "returns" : { "type" : "object()*", "description" : "facts associated with these concepts." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "facts-for-entities", "qname" : "facts:facts-for-entities", "signature" : "($entities-or-ids as item()*) as object()*", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Return facts reported by the given entities.</p>\n", "summary" : "<p>  Return facts reported by the given entities.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "entities-or-ids", "type" : "item()", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the entities or EIDs.</div>" } ], "returns" : { "type" : "object()*", "description" : "facts reported by the given entities." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "facts-for", "qname" : "facts:facts-for", "signature" : "($options as object()?) as object()*", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Return all facts that match a given filter object optionally interpreted\n    in the context of an optionally given hypercube.</p>\n", "summary" : "<p>  Return all facts that match a given filter object optionally interpreted\n    in the context of an optionally given hypercube.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "options", "type" : "object()", "occurrence" : "?", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> <a href=\"#standard_options\">standard fact retrieving options</a>.</div>" } ], "returns" : { "type" : "object()*", "description" : "all facts satisfying the filter and options." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">facts:FILTER-TOO-GENERIC The filter object must have at least one of the fields Archive, Aspects.xbrl:Concept, Aspects.xbrl:Period, or Aspects.xbrl:Entity.</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 1, "name" : "facts-search", "qname" : "facts:facts-search", "signature" : "($search as string) as object()*", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Return all facts that match the given search term.</p>\n", "summary" : "<p>  Return all facts that match the given search term.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "search", "type" : "string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the search query</div>" } ], "returns" : { "type" : "object()*", "description" : "all facts matching the given search query" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "facts", "qname" : "facts:facts", "signature" : "($fact-or-ids as item()*) as object()*", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Return the fact with the given FIDs.</p>\n", "summary" : "<p>  Return the fact with the given FIDs.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "fact-or-ids", "type" : "item()", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the FIDs or the facts themselves.</div>" } ], "returns" : { "type" : "object()*", "description" : "the facts with the given FIDs the empty sequence if no fact was found or if the input is an empty sequence." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "fid", "qname" : "facts:fid", "signature" : "($facts-or-ids as item()*) as atomic*", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Converts the input to a normalized fact id (FID). The input\n can be either an FID, or a fact object which contains an _id.</p>\n", "summary" : "<p>  Converts the input to a normalized fact id (FID).</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "facts-or-ids", "type" : "item()", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> a sequence of fact objects or FIDs.</div>" } ], "returns" : { "type" : "atomic*", "description" : "the normalized FIDs." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">facts:INVALID-PARAMETER if the FID or fact is not valid.</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 1, "name" : "instant-for-fact", "qname" : "facts:instant-for-fact", "signature" : "($fact-or-id as item()) as atomic?", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Retrieves the instant period for which a fact was reported.</p>\n", "summary" : "<p>  Retrieves the instant period for which a fact was reported.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "fact-or-id", "type" : "item()", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> a fact or its FID.</div>" } ], "returns" : { "type" : "atomic?", "description" : "the instance period, or the empty sequence if it is not instant." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "is-fact-forever", "qname" : "facts:is-fact-forever", "signature" : "($fact-or-id as item()) as boolean", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Tests whether a fact is reported forever.</p>\n", "summary" : "<p>  Tests whether a fact is reported forever.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "fact-or-id", "type" : "item()", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> a fact or its FID.</div>" } ], "returns" : { "type" : "boolean", "description" : "true if its period is forever, false otherwise." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 3, "name" : "merge-objects", "qname" : "facts:merge-objects", "signature" : "($o1 as object()?, $o2 as object()?, $prioritize-first-object as boolean) as object()?", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Helper function to deep-merge two objects. If the two given objects have\n    fields with the same name they are merged, which means:\n      1. if the values of the fields are objects then these are merged\n      2. in any other case the fields are accumulated into an array.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">The third parameter can be used to priotitize the first object. If the first\n    object is prioritized and both objects contain fields with the same name,\n    the fields are either merged (in case of two object values) or the value of\n    the first object is taken.</p>\n", "summary" : "<p>  Helper function to deep-merge two objects.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "o1", "type" : "object()", "occurrence" : "?", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> first object</div>" }, { "name" : "o2", "type" : "object()", "occurrence" : "?", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> second object</div>" }, { "name" : "prioritize-first-object", "type" : "boolean", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> boolean flag to give the first object higher priority in the merge</div>" } ], "returns" : { "type" : "object()?", "description" : "one merge object or an empty-sequence (in case both input objects are empty)." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "populate-with-footnotes", "qname" : "facts:populate-with-footnotes", "signature" : "($fact-or-ids as item()*) as object()*", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Populates a sequence of facts with their associated footnotes.\n More in detail, in each returned fact object an additional field\n Footnotes is added which contains all connected footnotes in an\n array.</p>\n", "summary" : "<p>  Populates a sequence of facts with their associated footnotes.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "fact-or-ids", "type" : "item()", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the FIDs or the facts themselves.</div>" } ], "returns" : { "type" : "object()*", "description" : "a sequence of facts with populated Footnotes field." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 2, "name" : "populate-with-footnotes", "qname" : "facts:populate-with-footnotes", "signature" : "($fact-or-ids as item()*, $options as object()?) as object()*", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Populates a sequence of facts with their associated footnotes.\n More in detail, in each returned fact object an additional field\n Footnotes is added which contains all connected footnotes in an\n array.</p>\n", "summary" : "<p>  Populates a sequence of facts with their associated footnotes.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "fact-or-ids", "type" : "item()", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the FIDs or the facts themselves.</div>" }, { "name" : "options", "type" : "object()", "occurrence" : "?", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> <a href=\"#standard_options\">standard fact retrieving options</a>.</div>" } ], "returns" : { "type" : "object()*", "description" : "a sequence of facts with populated Footnotes field." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "prefix-from-fact-concept", "qname" : "facts:prefix-from-fact-concept", "signature" : "($fact as object()) as string?", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Helper function to get the prefix of a given fact`s xbrl:Concept aspect.</p>\n", "summary" : "<p>  Helper function to get the prefix of a given fact`s xbrl:Concept aspect.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "fact", "type" : "object()", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> a fact object.</div>" } ], "returns" : { "type" : "string?", "description" : "the prefix of the fact's xbrl:Concept aspect or empty sequence if the concept doesn't have a prefix." }, "errors" : [  ] } ], "variables" : [ { "name" : "facts:col", "type" : "string", "description" : " Name of the collection the facts are stored in.\n" }, { "name" : "facts:ID", "type" : "string", "description" : " Name of the field that points to the facts FID.\n" }, { "name" : "facts:ARCHIVE", "type" : "string", "description" : " Name of the field that points to the archive.\n" }, { "name" : "facts:ASPECTS", "type" : "string", "description" : " Name of the field that stores the aspects.\n" }, { "name" : "facts:CONCEPT", "type" : "string", "description" : " Name of the concept aspect.\n" }, { "name" : "facts:PERIOD", "type" : "string", "description" : " Name of the period aspect.\n" }, { "name" : "facts:ENTITY", "type" : "string", "description" : " Name of the entity aspect.\n" }, { "name" : "facts:UNIT", "type" : "string", "description" : " Name of the unit aspect.\n" }, { "name" : "facts:FOOTNOTES", "type" : "string", "description" : " Name of the field that stores the Footnotes (if populated).\n" }, { "name" : "facts:ALL_OF_THEM", "type" : "boolean", "description" : " Joker for all archives or all concepts.\n" } ] }, "http://www.w3.org/2005/xqt-errors" : { "ns" : "http://www.w3.org/2005/xqt-errors", "description" : " This module contains one variable declaration for each diagnostic of the\n http://www.w3.org/2005/xqt-errors namespace.\n The variables serves as documentation for the errors but can also\n be used in the code. For example, one useful scenario is to compare\n an error caught in the catch clause of a try-catch expression with one of\n the variables.\n", "sees" : [  ], "authors" : [ "<xqdoc:author xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Carlos Lopez</xqdoc:author>" ], "version" : null, "encoding" : "utf-8", "namespaces" : [ { "uri" : "http://www.w3.org/2005/xqt-errors", "prefix" : "err" } ], "functions" : [  ], "variables" : [ { "name" : "err:FORG0006", "type" : "xs:QName", "description" : " Invalid argument type.\n" }, { "name" : "err:FODF1280", "type" : "xs:QName", "description" : " Invalid decimal format name supplied to \\c fn:format-number().\n" }, { "name" : "err:FODF1310", "type" : "xs:QName", "description" : " Invalid decimal/integer format picture string.\n" }, { "name" : "err:FODT0001", "type" : "xs:QName", "description" : " Overflow/underflow in date/time operation.\n" }, { "name" : "err:FODT0002", "type" : "xs:QName", "description" : " Overflow/underflow in duration operation.\n" }, { "name" : "err:FODT0003", "type" : "xs:QName", "description" : " Invalid timezone value.\n" }, { "name" : "err:FONS0004", "type" : "xs:QName", "description" : " No namespace found for prefix.\n" }, { "name" : "err:FONS0005", "type" : "xs:QName", "description" : " Base-URI not defined in static context.\n" }, { "name" : "err:FORG0001", "type" : "xs:QName", "description" : " Invalid value for cast/constructor.\n" }, { "name" : "err:FORG0002", "type" : "xs:QName", "description" : " Invalid argument to \\c fn:resolve-uri().\n" }, { "name" : "err:FORG0003", "type" : "xs:QName", "description" : " \\c fn:zero-or-one() called with a sequence containing more than one\n item.\n" }, { "name" : "err:FORG0004", "type" : "xs:QName", "description" : " \\c fn:one-or-more() called with a sequence containing no items.\n" }, { "name" : "err:FORG0005", "type" : "xs:QName", "description" : " \\c fn:exactly-one() called with a sequence containing zero or more\n than one item.\n" }, { "name" : "err:FODC0007", "type" : "xs:QName", "description" : " Base URI passed to \\c fn:parse() is not a valid absolute URI.\n" }, { "name" : "err:FORG0008", "type" : "xs:QName", "description" : " The two arguments to fn:dateTime() have inconsistent timezones.\n" }, { "name" : "err:FORG0009", "type" : "xs:QName", "description" : " Error in resolving a relative URI against a base URI in\n \\c fn:resolve-uri().\n" }, { "name" : "err:FORX0001", "type" : "xs:QName", "description" : " Invalid regular expression flags.\n" }, { "name" : "err:FORX0002", "type" : "xs:QName", "description" : " Invalid regular expression.\n" }, { "name" : "err:FORX0003", "type" : "xs:QName", "description" : " Regular expression matches zero-length string.\n" }, { "name" : "err:FORX0004", "type" : "xs:QName", "description" : " Invalid replacement string.\n" }, { "name" : "err:FOTY0012", "type" : "xs:QName", "description" : " Argument node does not have a typed value.\n" }, { "name" : "err:FOTY0013", "type" : "xs:QName", "description" : " An argument to \\c fn:data() contains a node that does not have a typed\n value.\n" }, { "name" : "err:FOTY0014", "type" : "xs:QName", "description" : " The argument to \\c fn:string() is a function item.\n" }, { "name" : "err:FOTY0015", "type" : "xs:QName", "description" : " An argument to \\c fn:deep-equal() contains a function item.\n" }, { "name" : "err:FOUT1170", "type" : "xs:QName", "description" : " Identifier cannot be used to retrive a resource containing text\n" }, { "name" : "err:FOCA0005", "type" : "xs:QName", "description" : " NaN supplied as float/double value.\n" }, { "name" : "err:FTDY0016", "type" : "xs:QName", "description" : " It is a dynamic error if a weight value is not within the required range\n of values; it is also a dynamic error if an implementation that does not\n support negative weights encounters a negative weight value.\n" }, { "name" : "err:FTDY0017", "type" : "xs:QName", "description" : " It is a dynamic error if an implementation encounters a mild not\n selection, one of whose operands evaluates to an AllMatches that\n contains a StringExclude.\n" }, { "name" : "err:FTST0018", "type" : "xs:QName", "description" : " It is a static error if, during the static analysis phase, the query is\n found to contain a thesaurus option that refers to a thesaurus that is\n not found in the statically known thesauri.\n" }, { "name" : "err:FTST0019", "type" : "xs:QName", "description" : " It is a static error if, within a single FTMatchOptions, there is more\n than one match option of any given match option group.\n" }, { "name" : "err:FTDY0020", "type" : "xs:QName", "description" : " It is a dynamic error if, when \"wildcards\" is in effect, a query string\n violates wildcard syntax.\n" }, { "name" : "err:FOER0000", "type" : "xs:QName", "description" : " Unidentified error.\n" }, { "name" : "err:FOAR0001", "type" : "xs:QName", "description" : " Division by zero.\n" }, { "name" : "err:FOAR0002", "type" : "xs:QName", "description" : " Numeric operation overflow/underflow.\n" }, { "name" : "err:FOCA0001", "type" : "xs:QName", "description" : " Input value too large for decimal.\n" }, { "name" : "err:FOCA0002", "type" : "xs:QName", "description" : " Invalid lexical value.\n" }, { "name" : "err:FOCA0003", "type" : "xs:QName", "description" : " Input value too large for integer.\n" }, { "name" : "err:FOUT1190", "type" : "xs:QName", "description" : " Retrieved resource contains octets that cannot be decoded into Unicode\n using the specified encoding, the resulting characters are not\n permitted XML characters or requested encoding not supported\n" }, { "name" : "err:FOCA0006", "type" : "xs:QName", "description" : " Raised when casting a string to xs:decimal if the string has more\n digits of precision than the implementation can represent (the\n implementation also has the option of rounding).\n" }, { "name" : "err:FOCH0001", "type" : "xs:QName", "description" : " Code point not valid.\n" }, { "name" : "err:FOCH0002", "type" : "xs:QName", "description" : " Unsupported collation.\n" }, { "name" : "err:FOCH0003", "type" : "xs:QName", "description" : " Unsupported normalization form.\n" }, { "name" : "err:FOCH0004", "type" : "xs:QName", "description" : " Collation does not support collation units.\n" }, { "name" : "err:FODC0001", "type" : "xs:QName", "description" : " No context document.\n" }, { "name" : "err:FODC0002", "type" : "xs:QName", "description" : " Error retrieving resource.\n" }, { "name" : "err:FODC0003", "type" : "xs:QName", "description" : " Raised by fn:doc, fn:collection to indicate that it is not possible to\n return a result that is guaranteed deterministic.\n" }, { "name" : "err:FODC0004", "type" : "xs:QName", "description" : " Invalid argument to \\c fn:collection().\n" }, { "name" : "err:FODC0005", "type" : "xs:QName", "description" : " Invalid argument to \\c fn:doc() or \\c fn:doc-available().\n" }, { "name" : "err:FODC0006", "type" : "xs:QName", "description" : " Invalid content passed to \\c fn:parse().\n" }, { "name" : "err:SERE0005", "type" : "xs:QName", "description" : " It is an error if the serialized result would contain an NCName Names\n that contains a character that is not permitted by the version of\n Namespaces in XML specified by the version parameter.\n" }, { "name" : "err:XUDY0024", "type" : "xs:QName", "description" : " It is a dynamic error if the effect of a set of updating expressions is\n to introduce conflicting namespace bindings into an element node.\n" }, { "name" : "err:XUDY0027", "type" : "xs:QName", "description" : " It is a dynamic error if the target expression of an insert, replace, or\n rename expression evaluates to an empty sequence.\n" }, { "name" : "err:XUST0028", "type" : "xs:QName", "description" : " It is a static error if a function declaration specifies both \\c updating\n and a return type.\n" }, { "name" : "err:XUDY0029", "type" : "xs:QName", "description" : " In an insert expression where \\c before or \\c after is specified, it is\n a dynamic error if the node returned by the target expression does not\n have a parent.\n" }, { "name" : "err:XUDY0030", "type" : "xs:QName", "description" : " It is a dynamic error if an insert expression specifies the insertion of\n an attribute node before or after a child of a document node.\n" }, { "name" : "err:XUDY0031", "type" : "xs:QName", "description" : " It is a dynamic error if multiple calls to \\c fn:put() in the same\n snapshot specify the same URI (after resolution of relative URIs).\n" }, { "name" : "err:FOUP0001", "type" : "xs:QName", "description" : " It is a dynamic error if the first operand of \\c fn:put() is not a node\n of a supported kind.\n" }, { "name" : "err:FOUP0002", "type" : "xs:QName", "description" : " It is a dynamic error if the second operand of \\c fn:put() is not a valid\n lexical representation of the \\c xs:anyURI type.\n" }, { "name" : "err:SENR0001", "type" : "xs:QName", "description" : " It is an error if an item in S6 in sequence normalization is an attribute\n node or a namespace node.\n" }, { "name" : "err:SERE0003", "type" : "xs:QName", "description" : " It is an error if the serializer is unable to satisfy the rules for\n either a well-formed XML document entity or a well-formed XML external\n general parsed entity, or both, except for content modified by the\n character expansion phase of serialization.\n" }, { "name" : "err:SEPM0004", "type" : "xs:QName", "description" : " It is an error to specify the doctype-system parameter, or to specify\n the standalone parameter with a value other than omit, if the instance\n of the data model contains text nodes or multiple element nodes as\n children of the root node.\n" }, { "name" : "err:XUDY0023", "type" : "xs:QName", "description" : " It is a dynamic error if an insert, replace, or rename expression\n affects an element node by introducing a new namespace binding that\n conflicts with one of its existing namespace bindings.\n" }, { "name" : "err:SERE0006", "type" : "xs:QName", "description" : " It is an error if the serialized result would contain a character that is\n not permitted by the version of XML specified by the version parameter.\n" }, { "name" : "err:SESU0007", "type" : "xs:QName", "description" : " It is an error if an output encoding other than UTF-8 or UTF-16 is\n requested and the serializer does not support that encoding.\n" }, { "name" : "err:SERE0008", "type" : "xs:QName", "description" : " It is an error if a character that cannot be represented in the encoding\n that the serializer is using for output appears in a context where\n character references are not allowed (for example if the character\n occurs in the name of an element).\n" }, { "name" : "err:SEPM0009", "type" : "xs:QName", "description" : " It is an error if the omit-xml-declaration parameter has the value yes,\n and the standalone attribute has a value other than omit; or the version\n parameter has a value other than 1.0 and the doctype-system parameter is\n specified.\n" }, { "name" : "err:SEPM0010", "type" : "xs:QName", "description" : " It is an error if the output method is xml, the value of the\n undeclare-prefixes parameter is yes, and the value of the version\n parameter is 1.0.\n" }, { "name" : "err:SESU0011", "type" : "xs:QName", "description" : " It is an error if the value of the normalization-form parameter\n specifies a normalization form that is not supported by the serializer.\n" }, { "name" : "err:SERE0012", "type" : "xs:QName", "description" : " It is an error if the value of the normalization-form parameter is\n fully-normalized and any relevant construct of the result begins with a\n combining character.\n" }, { "name" : "err:SESU0013", "type" : "xs:QName", "description" : " It is an error if the serializer does not support the version of XML or\n HTML specified by the version parameter.\n" }, { "name" : "err:SERE0014", "type" : "xs:QName", "description" : " It is an error to use the HTML output method when characters which are\n legal in XML but not in HTML, specifically the control characters\n #x7F-#x9F, appear in the instance of the data model.\n" }, { "name" : "err:SERE0015", "type" : "xs:QName", "description" : " It is an error to use the HTML output method when \\c &gt; appears within a\n processing instruction in the data model instance being serialized.\n" }, { "name" : "err:SEPM0016", "type" : "xs:QName", "description" : " It is a an error if a parameter value is invalid for the defined domain.\n" }, { "name" : "err:XUTY0010", "type" : "xs:QName", "description" : " In a replace expression where value of is not specified and\n the target is an element, text, comment, or processing instruction node,\n it is a type error if the replacement sequence does not consist of zero\n or more element, text, comment, or processing instruction nodes.\n" }, { "name" : "err:FOFL0001", "type" : "xs:QName", "description" : " This error is raised if the fn:function-lookup returns a context-dependent function and the context-dependent function is then called.\n" }, { "name" : "err:FOCZ0001", "type" : "xs:QName", "description" : " Invalid content passed to \\c x:canonicalize().\n" }, { "name" : "err:XUST0001", "type" : "xs:QName", "description" : " It is a static error if an updating expression is used in any position\n other than one of the following:\n - The topmost expression in the body of a query.\n - The \\c modify clause of a transform expression.\n - The \\c return clause of a FLWOR expression.\n - The \\c return clauses of a typeswitch expression in which every \\c\n return clause contains an updating expression or a vacuous expression.\n - The \\c then and \\c else clauses of a conditional statement in which\n both the \\c then and \\c else clauses contain either an updating\n expression or a vacuous expression.\n - An operand of a comma expression in which each operand is either an\n updating expression or a vacuous expression.\n - The content of a parenthesized expression.\n - The body of a function declaration in which the keyword \\c updating is\n specified.\n" }, { "name" : "err:XUST0002", "type" : "xs:QName", "description" : " It is a static error if a simple expression that is not a vacuous\n expression is used in one of the following positions:\n - The \\c modify clause of a transform expression.\n - The top-level expression in the body of a function declaration in\n which the keyword \\c updating is specified.\n" }, { "name" : "err:XUST0003", "type" : "xs:QName", "description" : " It is a static error if a Prolog contains more than one revalidation\n declaration.\n" }, { "name" : "err:XUTY0004", "type" : "xs:QName", "description" : " It is a type error if the insertion sequence of an insert expression\n contains an attribute node following a node that is not an attribute\n node.\n" }, { "name" : "err:XUTY0005", "type" : "xs:QName", "description" : " In an insert expression where into, as first\n into, or as last into is specified, it is a type\n error if the target expression returns a non-empty result that does not\n consist of a single element or document node.\n" }, { "name" : "err:XUTY0006", "type" : "xs:QName", "description" : " In an insert expression where \\c before or \\c after is specified, it is\n a type error if the target expression returns a non-empty result that\n does not consist of a single element, text, comment, or processing\n instruction node.\n" }, { "name" : "err:XUTY0007", "type" : "xs:QName", "description" : " It is a type error if the target expression of a delete expression does\n not return a sequence of zero or more nodes.\n" }, { "name" : "err:XUTY0008", "type" : "xs:QName", "description" : " In a replace expression, it is a type error if the target expression\n returns a non-empty result that does not consist of a single element,\n attribute, text, comment, or processing instruction node.\n" }, { "name" : "err:XUDY0009", "type" : "xs:QName", "description" : " In a replace expression where value of is not specified, it\n is a dynamic error if the node returned by the target expression does\n not have a parent.\n" }, { "name" : "err:FTST0009", "type" : "xs:QName", "description" : " It may be a static error if, during the static analysis phase, the query\n is found to contain a language identifier in a language option that the\n implementation does not support. The implementation may choose not to\n raise this error and instead provide some other implementation-defined\n behavior.\n" }, { "name" : "err:XUTY0011", "type" : "xs:QName", "description" : " In a replace expression where value of is not specified and\n the target is an attribute node, it is a type error if the replacement\n sequence does not consist of zero or more attribute nodes.\n" }, { "name" : "err:XUTY0012", "type" : "xs:QName", "description" : " In a rename expression, it is a type error if the target expression\n returns a non-empty result that does not consist of a single element,\n attribute, or processing instruction node.\n" }, { "name" : "err:XUTY0013", "type" : "xs:QName", "description" : " In a transform expression, it is a type error if a source expression in\n the \\c copy clause does not return a single node.\n" }, { "name" : "err:XUDY0014", "type" : "xs:QName", "description" : " In a transform expression, it is a dynamic error if the \\c modify clause\n modifies any node that was not created by the \\c copy clause.\n" }, { "name" : "err:XUDY0015", "type" : "xs:QName", "description" : " It is a dynamic error if any node is the target of more than one \\c\n rename expression within the same query.\n" }, { "name" : "err:XUDY0016", "type" : "xs:QName", "description" : " It is a dynamic error if any node is the target of more than one \\c\n replace expression (without value of being specified)\n within the same query.\n" }, { "name" : "err:XUDY0017", "type" : "xs:QName", "description" : " It is a dynamic error if any node is the target of more than one\n replace value of expression within the same query.\n" }, { "name" : "err:XUDY0018", "type" : "xs:QName", "description" : " It is a dynamic error if a function that was declared to be \\c external\n but not \\c updating returns a non-empty pending update list.\n" }, { "name" : "err:XUDY0019", "type" : "xs:QName", "description" : " It is a dynamic error if a function that was declared to be both \\c\n external and \\c updating returns a non-empty data model instance.\n" }, { "name" : "err:XUDY0021", "type" : "xs:QName", "description" : " It is a dynamic error if the XDM instance that would result from\n applying all the updates in a query violates any constraint specified in\n [XQuery 1.0 and XPath 2.0 Data Model]. In this case, none of the updates\n in the query are made effective.\n" }, { "name" : "err:XUTY0022", "type" : "xs:QName", "description" : " It is a type error if an insert expression specifies the insertion of an\n attribute node into a document node.\n" }, { "name" : "err:XQST0052", "type" : "xs:QName", "description" : " The type must be the name of a type defined in the in-scope schema types,\n and the {variety} of the type must be simple.\n" }, { "name" : "err:XQST0033", "type" : "xs:QName", "description" : " It is a static error if a module contains multiple bindings for the same\n namespace prefix.\n" }, { "name" : "err:XQST0034", "type" : "xs:QName", "description" : " It is a static error if multiple functions declared or imported by a\n module have the same number of arguments and their expanded QNames are\n equal (as defined by the eq operator).\n" }, { "name" : "err:XQST0035", "type" : "xs:QName", "description" : " It is a static error to import two schema components that both define the\n same name in the same symbol space and in the same scope.\n" }, { "name" : "err:XQST0036", "type" : "xs:QName", "description" : " It is a static error to import a module if the in-scope schema\n definitions of the importing module do not include all of the following:\n -# An in-scope schema type for each type-name that appears:\n - in the type of a variable that is declared in the imported module\n and referenced in the importing module, OR\n - in a parameter-type or result-type of a function that is declared in\n the imported module and referenced in the importing module.\n -# An in-scope element declaration for each element-name \\c EN such that:\n - \\c schema-element(EN) appears in the declared type of a variable in\n the imported module, and that variable is referenced in the\n importing module, OR\n - \\c schema-element(EN) appears in a parameter-type or result-type\n of a function declared in the imported module, and that function is\n referenced in the importing module.\n -# An in-scope attribute declaration for each attribute-name \\c AN such\n that:\n - \\c schema-attribute(AN) appears in the declared type of a variable\n in the imported module, and that variable is referenced in the\n importing module, OR\n - \\c schema-attribute(AN) appears in a parameter-type or result-type\n of a function declared in the imported module, and that function is\n referenced in the importing module.\n" }, { "name" : "err:XQST0038", "type" : "xs:QName", "description" : " It is a static error if a Prolog contains more than one default collation\n declaration, or the value specified by a default collation declaration is\n not present in statically known collations.\n" }, { "name" : "err:XQST0039", "type" : "xs:QName", "description" : " It is a static error for a function declaration to have more than one\n parameter with the same name.\n" }, { "name" : "err:XQST0040", "type" : "xs:QName", "description" : " It is a static error if the attributes specified by a direct element\n constructor do not have distinct expanded QNames.\n" }, { "name" : "err:XQST0045", "type" : "xs:QName", "description" : " It is a static error if the function name in a function declaration is in\n one of the following namespaces:\n http://www.w3.org/XML/1998/namespace,\n http://www.w3.org/2001/XMLSchema,\n http://www.w3.org/2001/XMLSchema-instance,\n http://www.w3.org/2005/xpath-functions.\n" }, { "name" : "err:XQST0046", "type" : "xs:QName", "description" : " An implementation MAY raise a static error if the value of a URILiteral\n is of nonzero length and is not in the lexical space of \\c xs:anyURI.\n" }, { "name" : "err:XQST0047", "type" : "xs:QName", "description" : " It is a static error if multiple module imports in the same Prolog\n specify the same target namespace.\n" }, { "name" : "err:XQST0048", "type" : "xs:QName", "description" : " It is a static error if a function or variable declared in a library\n module is not in the target namespace of the library module.\n" }, { "name" : "err:XQST0049", "type" : "xs:QName", "description" : " It is a static error if two or more variables declared or imported by a\n module have equal expanded QNames (as defined by the eq operator.)\n" }, { "name" : "err:XQST0032", "type" : "xs:QName", "description" : " A static error is raised if a Prolog contains more than one base URI\n declaration.\n" }, { "name" : "err:XQST0054", "type" : "xs:QName", "description" : " It is a static error if a variable depends on itself.\n" }, { "name" : "err:XQST0055", "type" : "xs:QName", "description" : " It is a static error if a Prolog contains more than one copy-namespaces\n declaration.\n" }, { "name" : "err:XQST0057", "type" : "xs:QName", "description" : " It is a static error if a schema import binds a namespace prefix but\n does not specify a target namespace other than a zero-length string.\n" }, { "name" : "err:XQST0058", "type" : "xs:QName", "description" : " It is a static error if multiple schema imports specify the same target\n namespace.\n" }, { "name" : "err:XQST0059", "type" : "xs:QName", "description" : " It is a static error if an implementation is unable to process a schema\n or module import by finding a schema or module with the specified\n target namespace.\n" }, { "name" : "err:XQST0060", "type" : "xs:QName", "description" : " It is a static error if the name of a function in a function declaration\n is not in a namespace (expanded QName has a null namespace URI).\n" }, { "name" : "err:XQST0065", "type" : "xs:QName", "description" : " A static error is raised if a Prolog contains more than one ordering mode\n declaration.\n" }, { "name" : "err:XQST0066", "type" : "xs:QName", "description" : " A static error is raised if a Prolog contains more than one default\n element/type namespace declaration, or more than one default function\n namespace declaration.\n" }, { "name" : "err:XQST0067", "type" : "xs:QName", "description" : " A static error is raised if a Prolog contains more than one construction\n declaration.\n" }, { "name" : "err:XQST0068", "type" : "xs:QName", "description" : " A static error is raised if a Prolog contains more than one\n boundary-space declaration.\n" }, { "name" : "err:XQST0069", "type" : "xs:QName", "description" : " A static error is raised if a Prolog contains more than one empty order\n declaration.\n" }, { "name" : "err:XPTY0019", "type" : "xs:QName", "description" : " It is a type error if the result of a step (other than the last step) in a\n path expression contains an atomic value.\n" }, { "name" : "err:XPST0001", "type" : "xs:QName", "description" : " It is a static error if analysis of an expression relies on some\n component of the static context that has not been assigned a value.\n" }, { "name" : "err:XPST0003", "type" : "xs:QName", "description" : " It is a static error if an expression is not a valid instance of the\n grammar.\n" }, { "name" : "err:XPST0005", "type" : "xs:QName", "description" : " During the analysis phase, it is a static error if the static type\n assigned to an expression other than the expression \\c () or \\c data(())\n is \\c empty-sequence().\n" }, { "name" : "err:XPST0008", "type" : "xs:QName", "description" : " It is a static error if an expression refers to an element name,\n attribute name, schema type name, namespace prefix, or variable name\n that is not defined in the static context, except for an ElementName in\n an ElementTest or an AttributeName in an AttributeTest.\n" }, { "name" : "err:XPST0017", "type" : "xs:QName", "description" : " It is a static error if the expanded QName and number of arguments in a\n function call do not match the name and arity of a function signature in\n the static context.\n" }, { "name" : "err:XPST0051", "type" : "xs:QName", "description" : " It is a static error if a QName that is used as an AtomicType in a\n SequenceType is not defined in the in-scope schema types as an atomic\n type.\n" }, { "name" : "err:XPST0080", "type" : "xs:QName", "description" : " It is a static error if the target type of a \\c cast or \\c castable\n expression is \\c xs:NOTATION or \\c xs:anyAtomicType.\n" }, { "name" : "err:XPST0081", "type" : "xs:QName", "description" : " It is a static error if a QName used in a query contains a namespace\n prefix that cannot be expanded into a namespace URI by using the\n statically known namespaces.\n" }, { "name" : "err:XPST0083", "type" : "xs:QName", "description" : " It is a static error if the target type of a \\c cast expression or\n constructor function is \\c xs:QName or a type derived from \\c xs:QName\n or \\c xs:NOTATION, and the argument of the cast expression or\n constructor function is not a string literal.\n" }, { "name" : "err:XPTY0004", "type" : "xs:QName", "description" : " It is a type error if, during the static analysis phase, an expression\n is found to have a static type that is not appropriate for the context\n in which the expression occurs, or during the dynamic evaluation phase,\n the dynamic type of a value does not match a required type as specified\n by the matching rules in 2.5.4 SequenceType Matching.\n" }, { "name" : "err:XPTY0018", "type" : "xs:QName", "description" : " It is a type error if the result of the last step in a path expression\n contains both nodes and non-nodes.\n" }, { "name" : "err:XQST0070", "type" : "xs:QName", "description" : " A static error is raised if one of the predefined prefixes \\c xml or \\c\n xmlns appears in a namespace declaration, or if any of the following\n conditions is statically detected in any expression or declaration:\n - The prefix \\c xml is bound to some namespace URI other than\n http://www.w3.org/XML/1998/namespace.\n - A prefix other than \\c xml is bound to the namespace URI\n http://www.w3.org/XML/1998/namespace.\n - The prefix \\c xmlns is bound to any namespace URI.\n - A prefix other than \\c xmlns is bound to the namespace URI\n http://www.w3.org/2000/xmlns/.\n" }, { "name" : "err:XPTY0020", "type" : "xs:QName", "description" : " It is a type error if, in an axis step, the context item is not a node.\n" }, { "name" : "err:XPTY0117", "type" : "xs:QName", "description" : " Attempt to cast to a namespace-sensitive type failed because the namespace\n bindings for the result can not be determined.\n" }, { "name" : "err:XQTY0024", "type" : "xs:QName", "description" : " It is a type error if the content sequence in an element constructor\n contains an attribute node following a node that is not an attribute node.\n" }, { "name" : "err:XQTY0030", "type" : "xs:QName", "description" : " It is a type error if the argument of a validate expression does not\n evaluate to exactly one document or element node.\n" }, { "name" : "err:XQTY0086", "type" : "xs:QName", "description" : " It is a type error if the typed value of a copied element or attribute\n node is namespace-sensitive when construction mode is \\c preserve and\n copy-namespaces mode is \\c no-preserve.\n" }, { "name" : "err:XQTY0105", "type" : "xs:QName", "description" : " It is a type error if the content sequence in an element constructor contains a function item.\n" }, { "name" : "err:XQST0009", "type" : "xs:QName", "description" : " An implementation that does not support the Schema Import Feature must\n raise a static error if a Prolog contains a schema import.\n" }, { "name" : "err:XQST0012", "type" : "xs:QName", "description" : " It is a static error if the set of definitions contained in all schemas\n imported by a Prolog do not satisfy the conditions for schema validity\n specified in Sections 3 and 5 of [XML Schema] Part 1--i.e., each\n definition must be valid, complete, and unique.\n" }, { "name" : "err:XQST0013", "type" : "xs:QName", "description" : " It is a static error if an implementation recognizes a pragma but\n determines that its content is invalid.\n" }, { "name" : "err:XQST0022", "type" : "xs:QName", "description" : " It is a static error if the value of a namespace declaration attribute is\n not a URILiteral.\n" }, { "name" : "err:XQST0031", "type" : "xs:QName", "description" : " It is a static error if the version number specified in a version\n declaration is not supported by the implementation.\n" }, { "name" : "err:XQDY0072", "type" : "xs:QName", "description" : " It is a dynamic error if the result of the content expression of a\n computed comment constructor contains two adjacent hyphens or ends with\n a hyphen.\n" }, { "name" : "err:XQST0128", "type" : "xs:QName", "description" : " It is a static error if a feature name that an implementation supports appears\n in a prohibit-feature option declaration, and the implementation is unable to\n disable the feature.\n" }, { "name" : "err:XPDY0002", "type" : "xs:QName", "description" : " It is a dynamic error if evaluation of an expression relies on some part\n of the dynamic context that has not been assigned a value.\n" }, { "name" : "err:XPDY0050", "type" : "xs:QName", "description" : " It is a dynamic error if the dynamic type of the operand of a treat\n expression does not match the sequence type specified by the treat\n expression. This error might also be raised by a path expression\n beginning with \"/\" or \"//\" if the context node\n is not in a tree that is rooted at a document node. This is because a\n leading \"/\" or \"//\" in a path expression is an\n abbreviation for an initial step that includes the clause \\c treat as \\c\n document-node().\n" }, { "name" : "err:XQDY0025", "type" : "xs:QName", "description" : " It is a dynamic error if any attribute of a constructed element does not\n have a name that is distinct from the names of all other attributes of\n the constructed element.\n" }, { "name" : "err:XQDY0026", "type" : "xs:QName", "description" : " It is a dynamic error if the result of the content expression of a\n computed processing instruction constructor contains the string \"?&gt;\".\n" }, { "name" : "err:XQDY0027", "type" : "xs:QName", "description" : " In a validate expression, it is a dynamic error if the root element\n information item in the PSVI resulting from validation does not have the\n expected validity property: \\c valid if validation mode is \\c strict, or\n either \\c valid or \\c notKnown if validation mode is \\c lax.\n" }, { "name" : "err:XQDY0041", "type" : "xs:QName", "description" : " It is a dynamic error if the value of the name expression in a computed\n processing instruction constructor cannot be cast to the type\n \\c xs:NCName.\n" }, { "name" : "err:XQDY0044", "type" : "xs:QName", "description" : " It is a static error the node-name of a node constructed by a computed\n attribute constructor has any of the following properties:\n - Its namespace prefix is \\c xmlns.\n - It has no namespace prefix and its local name is \\c xmlns.\n - Its namespace URI is http://www.w3.org/2000/xmlns/.\n - Its namespace prefix is \\c xml and its namespace URI is not\n http://www.w3.org/XML/1998/namespace.\n - Its namespace prefix is other than \\c xml and its namespace URI is\n http://www.w3.org/XML/1998/namespace.\n" }, { "name" : "err:XQDY0054", "type" : "xs:QName", "description" : " It is a dynamic error if a cycle is encountered in the definition of a\n module's dynamic context components, for example because of a cycle in\n variable declarations.\n" }, { "name" : "err:XQDY0061", "type" : "xs:QName", "description" : " It is a dynamic error if the operand of a validate expression is a\n document node whose children do not consist of exactly one element node\n and zero or more comment and processing instruction nodes, in any order.\n" }, { "name" : "err:XQDY0064", "type" : "xs:QName", "description" : " It is a dynamic error if the value of the name expression in a computed\n processing instruction constructor is equal to \"XML\" (in any combination\n of upper and lower case).\n" }, { "name" : "err:XQST0127", "type" : "xs:QName", "description" : " It is a static error if a given feature is both required and prohibited, directly or indirectly, in a module.\n" }, { "name" : "err:XQDY0074", "type" : "xs:QName", "description" : " It is a dynamic error if the value of the name expression in a computed\n element or attribute constructor cannot be converted to an expanded\n QName (for example, because it contains a namespace prefix not found in\n statically known namespaces).\n" }, { "name" : "err:XQDY0084", "type" : "xs:QName", "description" : " It is a dynamic error if the element validated by a \\c validate statement\n does not have a top-level element declaration in the in-scope element\n declarations, if validation mode is \\c strict.\n" }, { "name" : "err:XQDY0091", "type" : "xs:QName", "description" : " An implementation MAY raise a dynamic error if an \\c xml:id error, as\n defined in [XML ID], is encountered during construction of an attribute\n named \\c xml:id.\n" }, { "name" : "err:XQDY0092", "type" : "xs:QName", "description" : " An implementation MAY raise a dynamic error if a constructed attribute\n named \\c xml:space has a value other than \\c preserve or \\c default.\n" }, { "name" : "err:XQDY0096", "type" : "xs:QName", "description" : " It is a dynamic error the node-name of a node constructed by a computed\n element constructor has any of the following properties:\n - Its namespace prefix is \\c xmlns.\n - Its namespace URI is http://www.w3.org/2000/xmlns/.\n - Its namespace prefix is \\c xml and its namespace URI is not\n http://www.w3.org/XML/1998/namespace.\n - Its namespace prefix is other than \\c xml and its namespace URI is\n http://www.w3.org/XML/1998/namespace.\n" }, { "name" : "err:XQDY0101", "type" : "xs:QName", "description" : " Invalid prefix and/or uri in computed namespace constructor\n" }, { "name" : "err:XQDY0102", "type" : "xs:QName", "description" : " In an element constructor, if two or more namespace bindings in the in-scope bindings would have the same prefix, then an error is raised if they have different URIs; if they would have the same prefix and URI, duplicate bindings are ignored.\n" }, { "name" : "err:XTDE1310", "type" : "xs:QName", "description" : " It is a non-recoverable dynamic error if the picture string does not\n satisfy the format-number function rules.\n" }, { "name" : "err:FOFD1340", "type" : "xs:QName", "description" : " It is a non-recoverable dynamic error if the $picture, $language,\n $calendar, or $place argument for fn:format-date, fn:format-time, or\n fn:format-dateTime is invalid.\n" }, { "name" : "err:FOFD1350", "type" : "xs:QName", "description" : " It is a non-recoverable dynamic error if a component specifier within\n the picture refers to components that are not available in the given\n type of $value.\n" }, { "name" : "err:FTST0008", "type" : "xs:QName", "description" : " It is a static error if, during the static analysis phase, the query is\n found to contain a stop word option that refers to a stop word list that\n is not found in the statically known stop word lists.\n" }, { "name" : "err:XQST0098", "type" : "xs:QName", "description" : " It is a static error if, for any named or unnamed decimal format, the\n properties representing characters used in a picture string do not each\n have distinct values. These properties are decimal-separator-sign,\n grouping-separator, percent-sign, per-mille-sign, zero-digit,\n digit-sign, and pattern-separator-sign.\n" }, { "name" : "err:XQST0071", "type" : "xs:QName", "description" : " A static error is raised if the namespace declaration attributes of a\n direct element constructor do not have distinct names.\n" }, { "name" : "err:XQST0076", "type" : "xs:QName", "description" : " It is a static error if a \\c collation subclause in an order by clause\n of a FLWOR expression does not identify a collation that is present in\n statically known collations.\n" }, { "name" : "err:XQST0079", "type" : "xs:QName", "description" : " It is a static error if an extension expression contains neither a\n pragma that is recognized by the implementation nor an expression\n enclosed in curly braces.\n" }, { "name" : "err:XQST0085", "type" : "xs:QName", "description" : " It is a static error if the namespace URI in a namespace declaration\n attribute is a zero-length string, and the implementation does not\n support [XML Names 1.1].\n" }, { "name" : "err:XQST0087", "type" : "xs:QName", "description" : " It is a static error if the encoding specified in a Version Declaration\n does not conform to the definition of \\c EncName specified in [XML 1.0]\n" }, { "name" : "err:XQST0088", "type" : "xs:QName", "description" : " It is a static error if the literal that specifies the target namespace\n in a module import or a module declaration is of zero length.\n" }, { "name" : "err:XQST0089", "type" : "xs:QName", "description" : " It is a static error if a variable bound in a \\c for or \\c window clause\n of a FLWOR expression, and its associated positional variable, do not\n have distinct names (expanded QNames).\n" }, { "name" : "err:XQST0090", "type" : "xs:QName", "description" : " It is a static error if a character reference does not identify a valid\n character in the version of XML that is in use.\n" }, { "name" : "err:XQST0093", "type" : "xs:QName", "description" : " It is a static error to import a module M1 if there exists a sequence of\n modules M1 ... Mi ... M1 such that each module directly depends on the\n next module in the sequence (informally, if M1 depends on itself through\n some chain of module dependencies.)\n" }, { "name" : "err:XQST0094", "type" : "xs:QName", "description" : " In the group by clause of a FLWOR expression, it is a static error if the\n name of a grouping variable is not equal (by the eq operator on expanded\n QNames) to the name of a variable that is bound by a for or let clause\n that precedes the group by clause.\n" }, { "name" : "err:XQST0097", "type" : "xs:QName", "description" : " It is a static error for a decimal-format to specify a value that is\n not valid for a given property.\n" }, { "name" : "err:NS", "type" : "item()*", "description" : "" }, { "name" : "err:XQST0099", "type" : "xs:QName", "description" : " If a module contains more than one context item declaration, a static error is raised [err:XQST0099].\n" }, { "name" : "err:XQST0103", "type" : "xs:QName", "description" : " All variables in a window clause must have distinct names.\n" }, { "name" : "err:XQST0106", "type" : "xs:QName", "description" : " It is a static error if a function's annotations contain more than one\n annotation named \\c private or \\c public.  It is a static error if a\n function's annotations contain more than one annotation named \\c\n deterministic or \\c nondeterministic.\n" }, { "name" : "err:XQST0111", "type" : "xs:QName", "description" : " It is a static error for a query prolog to contain two decimal formats\n with the same name, or to contain two default decimal formats.\n" }, { "name" : "err:XQST0113", "type" : "xs:QName", "description" : " Specifying a VarValue or VarDefaultValue for a context item declaration\n in a library module is a static error.\n" }, { "name" : "err:XQST0114", "type" : "xs:QName", "description" : " It is a static error for a decimal format declaration to define the\n same property more than once.\n" }, { "name" : "err:XQST0116", "type" : "xs:QName", "description" : " It is a static error if a variable declaration contains both a %private\n and a %public annotation, more than one %private annotation, or more\n than one %public annotation.\n" }, { "name" : "err:XQST0120", "type" : "xs:QName", "description" : " It is a static error if a feature required by require-feature is not\n supported by the implementation.\n" }, { "name" : "err:XQST0122", "type" : "xs:QName", "description" : " It is a static error if the name of a feature in require-feature or\n prohibit-feature is not in the lexical space of QName.\n" }, { "name" : "err:XQST0123", "type" : "xs:QName", "description" : " It is a static error if the name of a feature in require-feature is not\n recognized by the implementation.\n" }, { "name" : "err:XQST0126", "type" : "xs:QName", "description" : " It is a static error if all-extensions appears in a require-feature option declaration.\n" } ] }, "http://zorba.io/modules/excel/math" : { "ns" : "http://zorba.io/modules/excel/math", "description" : " This is a library module offering a part of the set of functions\n defined by Microsoft Excel 2003.\n", "sees" : [ "<xqdoc:see xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"><a href=\"http://office.microsoft.com/en-us/excel/CH062528291033.aspx\" target=\"_blank\">Excel 2003 Documentation: Math Functions</a></xqdoc:see>" ], "authors" : [ "<xqdoc:author xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Daniel Turcanu</xqdoc:author>" ], "version" : null, "encoding" : "utf-8", "namespaces" : [ { "uri" : "http://zorba.io/modules/excel/math", "prefix" : "excel" }, { "uri" : "http://zorba.io/modules/excel/errors", "prefix" : "excel-err" }, { "uri" : "http://zorba.io/options/versioning", "prefix" : "ver" } ], "functions" : [ { "isDocumented" : true, "arity" : 1, "name" : "abs", "qname" : "excel:abs", "signature" : "($arg as xs:anyAtomicType) as xs:anyAtomicType", "description" : " Compute the abs of a numeric value.\n The value can also be a string and it will be casted to the appropriate numeric first.\n", "summary" : "<p> Compute the abs of a numeric value.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "arg", "type" : "xs:anyAtomicType", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The parameter can be a number, string, boolean value.</div>" } ], "returns" : { "type" : "xs:anyAtomicType", "description" : "The abs value as a numeric type." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">excel-err:Value if arg cannot be casted to numeric type.</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 1, "name" : "cast-as-numeric", "qname" : "excel:cast-as-numeric", "signature" : "($number as xs:anyAtomicType) as xs:anyAtomicType", "description" : " Cast the xs:anyAtomicType to a numeric type.\n If the value is already of a numeric type then nothing is changed.\n Otherwise the value is casted to the numeric type that is most appropriate.\n", "summary" : "<p> Cast the xs:anyAtomicType to a numeric type.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "number", "type" : "xs:anyAtomicType", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The parameter can be a number, string, boolean value.</div>" } ], "returns" : { "type" : "xs:anyAtomicType", "description" : "The casted value." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">excel-err:Value if the value cannot be casted to numeric type.</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 2, "name" : "ceiling", "qname" : "excel:ceiling", "signature" : "($number as xs:anyAtomicType, $significance as xs:anyAtomicType) as xs:anyAtomicType", "description" : " Returns number rounded up, away from zero, to the nearest multiple of significance.\n Significance must have the same sign as number.\n Number and significance must be of a numeric type or castable to numeric.\n Significance must not be zero.\n", "summary" : "<p> Returns number rounded up, away from zero, to the nearest multiple of significance.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "number", "type" : "xs:anyAtomicType", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The value you want to round.</div>" }, { "name" : "significance", "type" : "xs:anyAtomicType", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The multiple to which you want to round.</div>" } ], "returns" : { "type" : "xs:anyAtomicType", "description" : "The rounded value." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">excel-err:Value if parameters cannot be casted to numeric type.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">excel-err:Num if significance is zero or it doesn't have the same sign as number.</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 1, "name" : "degrees", "qname" : "excel:degrees", "signature" : "($radian as xs:double) as xs:integer", "description" : " Converts radians into degrees.\n", "summary" : "<p> Converts radians into degrees.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "radian", "type" : "xs:double", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The value in radians.</div>" } ], "returns" : { "type" : "xs:integer", "description" : "The value in degrees 0 .. 360 or 0 .. -360." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "even", "qname" : "excel:even", "signature" : "($number as xs:anyAtomicType) as xs:anyAtomicType", "description" : " Returns number rounded up to the nearest even integer.\n Regardless of the sign of number, a value is rounded up when adjusted away from zero.\n", "summary" : "<p> Returns number rounded up to the nearest even integer.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "number", "type" : "xs:anyAtomicType", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The value to round.</div>" } ], "returns" : { "type" : "xs:anyAtomicType", "description" : "The rounded value casted as numeric type." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">excel-err:Value if parameters cannot be casted to numeric type.</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 1, "name" : "fact", "qname" : "excel:fact", "signature" : "($number as xs:anyAtomicType) as xs:integer", "description" : " Returns the factorial of a number.\n", "summary" : "<p> Returns the factorial of a number.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "number", "type" : "xs:anyAtomicType", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The nonnegative number you want the factorial of. If number is not an integer, it is truncated.</div>" } ], "returns" : { "type" : "xs:integer", "description" : "Returns the factorial of a number. The factorial of a number is equal to 1*2*3*...* number." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">excel-err:Num if the number is smaller than zero</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 1, "name" : "factdouble", "qname" : "excel:factdouble", "signature" : "($number as xs:integer) as xs:integer", "description" : " Returns the double factorial of a number.\n Computes the double factorial of n as n(n-2)(n-4)...\n", "summary" : "<p> Returns the double factorial of a number.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "number", "type" : "xs:integer", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The positive integer value.</div>" } ], "returns" : { "type" : "xs:integer", "description" : "The result as integer." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">excel-err:Num if the number is negative.</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 2, "name" : "floor", "qname" : "excel:floor", "signature" : "($number as xs:anyAtomicType, $significance as xs:anyAtomicType) as xs:anyAtomicType", "description" : " Rounds number down, toward zero, to the nearest multiple of significance.\n Significance must have the same sign as number.\n", "summary" : "<p> Rounds number down, toward zero, to the nearest multiple of significance.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "number", "type" : "xs:anyAtomicType", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The value you want to round. The value is casted to numeric.</div>" }, { "name" : "significance", "type" : "xs:anyAtomicType", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The multiple to which you want to round.</div>" } ], "returns" : { "type" : "xs:anyAtomicType", "description" : "The rounded value as numeric type." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">excel-err:Value if parameters cannot be casted to numeric type.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">excel-err:Num if significance is zero or it doesn't have the same sign as number.</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 1, "name" : "gcd", "qname" : "excel:gcd", "signature" : "($numbers as xs:integer+) as xs:integer", "description" : " Returns the greatest common divisor GCD of a sequence of integers.\n The sequence can have one or more positive integers.\n", "summary" : "<p> Returns the greatest common divisor GCD of a sequence of integers.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "numbers", "type" : "xs:integer", "occurrence" : "+", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The sequence of positive integers.</div>" } ], "returns" : { "type" : "xs:integer", "description" : "The GCD as integer." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">excel-err:Num if any number is smaller than zero.</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 1, "name" : "int", "qname" : "excel:int", "signature" : "($number as xs:anyAtomicType) as xs:integer", "description" : " Rounds a number down to the nearest integer.\n Positive numbers are rounded toward zero, negative numbers are rounded away from zero.\n", "summary" : "<p> Rounds a number down to the nearest integer.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "number", "type" : "xs:anyAtomicType", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The value to be rounded.</div>" } ], "returns" : { "type" : "xs:integer", "description" : "The rounded integer." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">excel-err:Value if parameter cannot be casted to numeric type</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 1, "name" : "is-a-number", "qname" : "excel:is-a-number", "signature" : "($value as xs:anyAtomicType) as xs:boolean", "description" : " Checks if the xs:anyAtomicType argument is actually a numeric type\n or can be converted to numeric.\n", "summary" : "<p> Checks if the xs:anyAtomicType argument is actually a numeric type\n or can be converted to numeric.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "value", "type" : "xs:anyAtomicType", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> Parameter to be checked.</div>" } ], "returns" : { "type" : "xs:boolean", "description" : "true if the value can be casted to numeric." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "lcm", "qname" : "excel:lcm", "signature" : "($numbers as xs:integer+) as xs:integer", "description" : " Returns the least common multiple of integers.<p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n LCM for two numbers is computed by multiplying them and dividing with GCD. <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n The function is applied recursively replacing the first two numbers in the sequence with their LCM.\n", "summary" : "<p> Returns the least common multiple of integers.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "numbers", "type" : "xs:integer", "occurrence" : "+", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The sequence of one or more positive integers.</div>" } ], "returns" : { "type" : "xs:integer", "description" : "The LCM as integer." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">excel-err:Num if any number is smaller than zero.</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 2, "name" : "mod", "qname" : "excel:mod", "signature" : "($number as xs:anyAtomicType, $divisor as xs:anyAtomicType) as xs:anyAtomicType", "description" : " Returns the remainder after number is divided by divisor.\n The result has the same sign as divisor.\n", "summary" : "<p> Returns the remainder after number is divided by divisor.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "number", "type" : "xs:anyAtomicType", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The number for which you want to find the remainder.</div>" }, { "name" : "divisor", "type" : "xs:anyAtomicType", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The number by which you want to divide number. This cannot be zero.</div>" } ], "returns" : { "type" : "xs:anyAtomicType", "description" : "The remainder from division as numeric type." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">excel-err:Value if parameters cannot be casted to numeric type.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">excel-err:Div0 if divisor is zero after casting to numeric.</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 2, "name" : "mround", "qname" : "excel:mround", "signature" : "($number as xs:anyAtomicType, $multiple as xs:anyAtomicType) as xs:anyAtomicType", "description" : " Returns a number rounded to the desired multiple.\n MROUND rounds up, away from zero, if the remainder of dividing number by multiple\n is greater than or equal to half the value of multiple.\n MROUND is computed through floor function.\n", "summary" : "<p> Returns a number rounded to the desired multiple.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "number", "type" : "xs:anyAtomicType", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The value to round, castable to numeric type.</div>" }, { "name" : "multiple", "type" : "xs:anyAtomicType", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The multiple to which you want to round number.</div>" } ], "returns" : { "type" : "xs:anyAtomicType", "description" : "The rounded number up to the desired multiple." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">excel-err:Value if parameters cannot be casted to numeric type.</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 1, "name" : "odd", "qname" : "excel:odd", "signature" : "($number as xs:anyAtomicType) as xs:integer", "description" : " Returns number rounded up to the nearest odd integer, away from zero.\n", "summary" : "<p> Returns number rounded up to the nearest odd integer, away from zero.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "number", "type" : "xs:anyAtomicType", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The value to round.</div>" } ], "returns" : { "type" : "xs:integer", "description" : "The odd integer." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">excel-err:Value if parameter cannot be casted to numeric type.</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 0, "name" : "pi", "qname" : "excel:pi", "signature" : "() as xs:decimal", "description" : " Return the value of PI as decimal with 15 digits.\n", "summary" : "<p> Return the value of PI as decimal with 15 digits.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [  ], "returns" : { "type" : "xs:decimal", "description" : "The value of PI with 15 digits." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 2, "name" : "power", "qname" : "excel:power", "signature" : "($number as xs:anyAtomicType, $power as xs:integer) as xs:anyAtomicType", "description" : " Returns the result of a number raised to a power.\n The result is computed through successive multiplications.\n", "summary" : "<p> Returns the result of a number raised to a power.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "number", "type" : "xs:anyAtomicType", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The base number.</div>" }, { "name" : "power", "type" : "xs:integer", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The exponent as integer (cannot be floating point like in Excel).</div>" } ], "returns" : { "type" : "xs:anyAtomicType", "description" : "The result as numeric type." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">excel-err:Value if parameter cannot be casted to numeric type.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">excel-err:Value if power is smaller than zero.</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 1, "name" : "product", "qname" : "excel:product", "signature" : "($numbers as xs:anyAtomicType*) as xs:anyAtomicType", "description" : " Multiplies all the numbers given as arguments and returns the product.\n", "summary" : "<p> Multiplies all the numbers given as arguments and returns the product.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "numbers", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The sequence of arguments convertable to numeric types. The sequence can be of any length.</div>" } ], "returns" : { "type" : "xs:anyAtomicType", "description" : "The multiplication result as numeric type." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">excel-err:Value if parameters cannot be casted to numeric type.</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 2, "name" : "quotient", "qname" : "excel:quotient", "signature" : "($numerator as xs:anyAtomicType, $denominator as xs:anyAtomicType) as xs:integer", "description" : " Returns the integer portion of a division.\n", "summary" : "<p> Returns the integer portion of a division.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "numerator", "type" : "xs:anyAtomicType", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The divident.</div>" }, { "name" : "denominator", "type" : "xs:anyAtomicType", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The divisor. It cannot be zero.</div>" } ], "returns" : { "type" : "xs:integer", "description" : "The result value as numeric type." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">excel-err:Value if parameters cannot be casted to numeric type.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">excel-err:Div0 if denominator casted as numeric type has value zero.</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 1, "name" : "radians", "qname" : "excel:radians", "signature" : "($degree as xs:integer) as xs:decimal", "description" : " Converts degrees to radians.\n", "summary" : "<p> Converts degrees to radians.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "degree", "type" : "xs:integer", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> An angle in degrees that you want to convert.</div>" } ], "returns" : { "type" : "xs:decimal", "description" : "The value in radians." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "roman", "qname" : "excel:roman", "signature" : "($number as xs:integer) as xs:string", "description" : " Converts an arabic numeral to roman, as text.\n Only the clasic format is supported (out of all formats Excel requires).<p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n M is the largest digit, it represents 1000.\n Numbers bigger than 2000 will be represented by a sequence of \"M\".<p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n D = 500, C = 100, L = 50, X = 10, V = 5, I = 1.\n", "summary" : "<p> Converts an arabic numeral to roman, as text.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "number", "type" : "xs:integer", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> A positive integer.</div>" } ], "returns" : { "type" : "xs:string", "description" : "The roman string representation." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">excel-err:Num if the input integer is negative</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 2, "name" : "round", "qname" : "excel:round", "signature" : "($number as xs:anyAtomicType, $precision as xs:integer) as xs:anyAtomicType", "description" : " Rounds a number to a specified number of digits.\n If precision is greater than 0 (zero), then number is rounded\n to the specified number of decimal places.\n If num_digits is 0, then number is rounded to the nearest integer.\n If num_digits is less than 0, then number is rounded to the left of the decimal point.\n The 0.5 is rounded away from zero.\n", "summary" : "<p> Rounds a number to a specified number of digits.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "number", "type" : "xs:anyAtomicType", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The number to round, castable to a numeric type.</div>" }, { "name" : "precision", "type" : "xs:integer", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The number of decimal places to keep.</div>" } ], "returns" : { "type" : "xs:anyAtomicType", "description" : "The rounded number as numeric type." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">excel-err:Value if parameters cannot be casted to numeric type.</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 2, "name" : "rounddown", "qname" : "excel:rounddown", "signature" : "($number as xs:anyAtomicType, $precision as xs:integer) as xs:anyAtomicType", "description" : " Rounds a number down, toward zero.\n If num_digits is greater than 0 (zero), then number is rounded down\n to the specified number of decimal places.\n If num_digits is 0, then number is rounded down to the nearest integer.\n If num_digits is less than 0, then number is rounded down to the left of the decimal point.\n", "summary" : "<p> Rounds a number down, toward zero.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "number", "type" : "xs:anyAtomicType", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The number to round, castable to numeric type.</div>" }, { "name" : "precision", "type" : "xs:integer", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The number of decimal places to keep.</div>" } ], "returns" : { "type" : "xs:anyAtomicType", "description" : "the truncated number toward zero, as numeric type." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">excel-err:Value if parameters cannot be casted to numeric type.</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 2, "name" : "roundup", "qname" : "excel:roundup", "signature" : "($number as xs:anyAtomicType, $precision as xs:integer) as xs:anyAtomicType", "description" : " Rounds a number up, away from 0 (zero).\n If num_digits is greater than 0 (zero), then number is rounded down\n to the specified number of decimal places.\n If num_digits is 0, then number is rounded down to the nearest integer.\n If num_digits is less than 0, then number is rounded down to the left of the decimal point.\n", "summary" : "<p> Rounds a number up, away from 0 (zero).</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "number", "type" : "xs:anyAtomicType", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The number to round, castable to numeric type.</div>" }, { "name" : "precision", "type" : "xs:integer", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The number of decimal places to keep.</div>" } ], "returns" : { "type" : "xs:anyAtomicType", "description" : "The truncated number away from zero, as numeric type." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">excel-err:Value if parameters cannot be casted to numeric type.</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 1, "name" : "sign", "qname" : "excel:sign", "signature" : "($number as xs:anyAtomicType) as xs:integer", "description" : " Determines the sign of a number.\n Returns 1 if the number is positive, zero (0) if the number is 0,\n and -1 if the number is negative.\n", "summary" : "<p> Determines the sign of a number.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "number", "type" : "xs:anyAtomicType", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The argument castable to numeric type.</div>" } ], "returns" : { "type" : "xs:integer", "description" : "The sign as (-1, 0, 1)." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">excel-err:Value if parameters cannot be casted to numeric type.</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 1, "name" : "sort-numbers", "qname" : "excel:sort-numbers", "signature" : "($numbers as xs:anyAtomicType*) as xs:anyAtomicType*", "description" : " Helper function.<p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n Sorts a sequence of numbers or arguments castable to numeric.\n It first casts all arguments to numeric and then sorts ascending.\n", "summary" : "<p> Helper function.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "numbers", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The sequence of arguments castable to numeric.</div>" } ], "returns" : { "type" : "xs:anyAtomicType*", "description" : "The sorted sequence as numeric types." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">excel-err:Value if parameters cannot be casted to numeric type.</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 1, "name" : "sum", "qname" : "excel:sum", "signature" : "($numbers as xs:anyAtomicType*) as xs:anyAtomicType", "description" : " Adds all the numbers in the sequence.\n", "summary" : "<p> Adds all the numbers in the sequence.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "numbers", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The sequence of arguments castable to numeric types. The sequence can be of any length.</div>" } ], "returns" : { "type" : "xs:anyAtomicType", "description" : "The sum as numeric type." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">excel-err:Value if parameters cannot be casted to numeric type.</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 1, "name" : "trunc", "qname" : "excel:trunc", "signature" : "($number as xs:anyAtomicType) as xs:integer", "description" : " Truncates a number to an integer by removing the fractional part of the number.\n", "summary" : "<p> Truncates a number to an integer by removing the fractional part of the number.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "number", "type" : "xs:anyAtomicType", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The argument castable to numeric type.</div>" } ], "returns" : { "type" : "xs:integer", "description" : "The integer value." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">excel-err:Value if parameter cannot be casted to numeric type.</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 2, "name" : "trunc", "qname" : "excel:trunc", "signature" : "($number as xs:anyAtomicType, $precision as xs:integer) as xs:anyAtomicType", "description" : " Truncates a number down to precision.\n This behaves exactly like rounddown.\n", "summary" : "<p> Truncates a number down to precision.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "number", "type" : "xs:anyAtomicType", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The argument castable to numeric type.</div>" }, { "name" : "precision", "type" : "xs:integer", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The number of decimal places to keep .</div>" } ], "returns" : { "type" : "xs:anyAtomicType", "description" : "The integer value." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">excel-err:Value if parameter cannot be casted to numeric type.</xqdoc:error>" ] } ], "variables" : [  ] }, "http://api.28.io/browserview" : { "ns" : "http://api.28.io/browserview", "description" : "", "sees" : [  ], "authors" : [  ], "version" : null, "encoding" : null, "namespaces" : [ { "uri" : "", "prefix" : "an" }, { "uri" : "http://api.28.io/browserview", "prefix" : "browserview" }, { "uri" : "http://api.28.io/model", "prefix" : "model" }, { "uri" : "http://zorba.io/modules/reference", "prefix" : "ref" } ], "functions" : [ { "isDocumented" : false, "arity" : 1, "name" : "node-path", "qname" : "browserview:node-path", "signature" : "($node)", "description" : "", "summary" : "", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "node", "type" : null, "occurrence" : null, "description" : "" } ], "returns" : { "type" : null, "description" : "" }, "errors" : [  ] }, { "isDocumented" : false, "arity" : 2, "name" : "node-reference", "qname" : "browserview:node-reference", "signature" : "($node, $short as xs:boolean)", "description" : "", "summary" : "", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "node", "type" : null, "occurrence" : null, "description" : "" }, { "name" : "short", "type" : "xs:boolean", "occurrence" : null, "description" : "" } ], "returns" : { "type" : null, "description" : "" }, "errors" : [  ] }, { "isDocumented" : false, "arity" : 1, "name" : "serialize", "qname" : "browserview:serialize", "signature" : "($json)", "description" : "", "summary" : "", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "json", "type" : null, "occurrence" : null, "description" : "" } ], "returns" : { "type" : null, "description" : "" }, "errors" : [  ] }, { "isDocumented" : false, "arity" : 1, "name" : "serializeMulti", "qname" : "browserview:serializeMulti", "signature" : "($json)", "description" : "", "summary" : "", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "json", "type" : null, "occurrence" : null, "description" : "" } ], "returns" : { "type" : null, "description" : "" }, "errors" : [  ] }, { "isDocumented" : false, "arity" : 1, "name" : "show-namespaces", "qname" : "browserview:show-namespaces", "signature" : "($namespaces)", "description" : "", "summary" : "", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "namespaces", "type" : null, "occurrence" : null, "description" : "" } ], "returns" : { "type" : null, "description" : "" }, "errors" : [  ] }, { "isDocumented" : false, "arity" : 3, "name" : "show-node", "qname" : "browserview:show-node", "signature" : "($node, $namespaces, $short as xs:boolean)", "description" : "", "summary" : "", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "node", "type" : null, "occurrence" : null, "description" : "" }, { "name" : "namespaces", "type" : null, "occurrence" : null, "description" : "" }, { "name" : "short", "type" : "xs:boolean", "occurrence" : null, "description" : "" } ], "returns" : { "type" : null, "description" : "" }, "errors" : [  ] }, { "isDocumented" : false, "arity" : 1, "name" : "show-nodes", "qname" : "browserview:show-nodes", "signature" : "($nodes)", "description" : "", "summary" : "", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "nodes", "type" : null, "occurrence" : null, "description" : "" } ], "returns" : { "type" : null, "description" : "" }, "errors" : [  ] } ], "variables" : [  ] }, "http://zorba.io/modules/json-csv" : { "ns" : "http://zorba.io/modules/json-csv", "description" : " This module provides an API for parsing and serializing CSV (comma-separated\n values) files.\n See RFC 4180,\n \"Common Format and MIME Type for Comma-Separated Values (CSV) Files.\"\n", "sees" : [  ], "authors" : [ "<xqdoc:author xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Paul J. Lucas</xqdoc:author>" ], "version" : null, "encoding" : "utf-8", "namespaces" : [ { "uri" : "http://zorba.io/modules/json-csv", "prefix" : "csv" }, { "uri" : "http://zorba.io/options/versioning", "prefix" : "ver" } ], "functions" : [ { "isDocumented" : true, "arity" : 1, "name" : "parse", "qname" : "csv:parse", "signature" : "($csv as string) as object()*", "description" : " Parses a CSV (comma-separated values) string using the default options.\n A newline (U+000A), optionally preceeded by a carriage-return (U+000D),\n terminates lines, aka, \"records.\"\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n Quoted values are always considered strings;\n unquoted values are attempted to be cast to other types, e.g., integer\n (unless the <code xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cast-unquoted-values</code> option is <code xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">false</code>).\n Casting is attempted in the following order:\n integer, decimal, double, and boolean.\n If casting fails, the value is considered a string.\n Header field names are always considered strings even if unquoted.\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n In addition to the \"normal\" values of\n <code xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">true</code> and <code xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">false</code> for boolean,\n <code xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">T</code> and <code xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Y</code> are also considered \"true\"\n and <code xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">F</code> and <code xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">N</code> are also considered \"false.\"\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n The default options are:\n  <dl xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">\n    <dt><code>cast-unquoted-values</code></dt>\n      <dd>\n        Whether to attempt to cast unquoted values to\n        integer, decimal, double, or boolean;\n        default: <code>true</code>.\n      </dd>\n    <dt><code>extra-name</code></dt>\n      <dd>\n        The field name for extra values, if any;\n        default: none (error <code>csv:EXTRA_VALUE</code> is raised).\n      </dd>\n    <dt><code>field-names</code></dt>\n      <dd>\n        A JSON array of strings denoting field names;\n        default: none.\n        The first CSV line is assumed to be a header line\n        and the field names are taken from this line.\n      </dd>\n    <dt><code>missing-value</code></dt>\n      <dd>\n        What should happen when a missing value is detected;\n        default: <code>\"null\"</code>.\n        A \"missing\" value is one of:\n        <ul>\n          <li>Two consecutive <code>quote-char</code> characters.</li>\n          <li>A <code>quote-char</code> character as either the first\n              or last character on a line.</li>\n          <li>Fewer values than the number of field names.</li>\n        </ul>\n        When a missing value is detected,\n        the value is set to <code>null</code>.\n      </dd>\n    <dt><code>quote-char</code></dt>\n      <dd>\n        The single ASCII character that may be used to quote values;\n        default: <code>\"</code> (U+0022).\n      </dd>\n    <dt><code>quote-escape</code></dt>\n      <dd>\n        The single ASCII character used to escape <code>quote-char</code>;\n        default: same as <code>quote-char</code>.\n        This means that an escaped quote is doubled as <code>\"\"</code>.\n      </dd>\n    <dt><code>separator</code></dt>\n      <dd>\n        The single ASCII character used to separate values;\n        default: <code>,</code> (U+002C).\n      </dd>\n  </dl>\n", "summary" : "<p> Parses a CSV (comma-separated values) string using the default options.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "csv", "type" : "string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The CSV string to parse.</div>" } ], "returns" : { "type" : "object()*", "description" : "a sequence of zero or more JSON objects where each key is a field name and each value is a parsed value." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">csv:EXTRA_VALUE if an extra value is detected.</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 2, "name" : "parse", "qname" : "csv:parse", "signature" : "($csv as string, $options as object()) as object()* external", "description" : " Parses a CSV (comma-separated values) string using the given options.\n A newline (U+000A), optionally preceeded by a carriage-return (U+000D),\n terminates lines, aka, \"records.\"\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n Quoted values are always considered strings;\n unquoted values are attempted to be cast to other types, e.g., integer\n (unless the <code xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cast-unquoted-values</code> option is <code xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">false</code>).\n Casting is attempted in the following order:\n integer, decimal, double, and boolean.\n If casting fails, the value is considered a string.\n Header field names are always considered strings even if unquoted.\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n In addition to the \"normal\" values of\n <code xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">true</code> and <code xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">false</code> for boolean,\n <code xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">T</code> and <code xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Y</code> are also considered \"true\"\n and <code xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">F</code> and <code xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">N</code> are also considered \"false.\"\n", "summary" : "<p> Parses a CSV (comma-separated values) string using the given options.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "csv", "type" : "string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The CSV string to parse.</div>" }, { "name" : "options", "type" : "object()", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The options to use: <dl> <dt><code>cast-unquoted-values</code></dt> <dd> Whether to attempt to cast unquoted values to integer, decimal, double, or boolean; default: <code>true</code>. </dd> <dt><code>extra-name</code></dt> <dd> The field name for extra values, if any; default: none (error <code>csv:EXTRA_VALUE</code> is raised). If this option is given and a line contains one or more extra values (that is, values that have no corresponding field names), then the extra values are assigned as the values for fields having <code>extra-name</code> as their names. <p/> If <code>extra-name</code> contains a <code>#</code> (U+0023), then the <code>#</code> is substituted with the field number (where field numbers start at 1). If <code>extra-name</code> does not contains a <code>#</code>, then the field number is appended. </dd> <dt><code>field-names</code></dt> <dd> A JSON array of strings denoting field names; default: none. If this option is given, then the first CSV line is assumed not to be a header line; if omitted, then the first CSV line is assumed to be a header line and the field names are taken from this line. </dd> <dt><code>missing-value</code></dt> <dd> What should happen when a missing value is detected; default: <code>\"null\"</code>. A \"missing\" value is one of: <ul> <li>Two consecutive <code>separator</code> characters.</li> <li>A <code>separator</code> character as either the first or last character on a line.</li> <li>Fewer values than the number of field names.</li> </ul> When a missing value is detected, the value of this option determines what happens: <dl> <dt><code>\"error\"</code></dt> <dd>Error <code>csv:MISSING_VALUE</code> is raised.</dd> <dt><code>\"omit\"</code></dt> <dd>Both the value and its key are omitted from the result object.</dd> <dt><code>\"null\"</code></dt> <dd>The value is set to <code>null</code>.</dd> </dl> </dd> <dt><code>quote-char</code></dt> <dd> The single ASCII character that may be used to quote values; default: <code>\"</code> (U+0022). </dd> <dt><code>quote-escape</code></dt> <dd> The single ASCII character used to escape <code>quote-char</code>; default: same as <code>quote-char</code>. If <code>quote-escape</code> equals <code>quote-char</code>, it means that <code>quote-char</code> must be doubled to escape it. If <code>quote-escape</code> does not equal <code>quote-char</code>, it means that <code>quote-escape</code> is used to escape <code>quote-char</code>. For example, a <code>quote-char</code> of <code>\"</code> (U+0022) and a <code>quote-escape</code> of <code>\\</code> (U+005C) means that quotes will be escaped by <code>\\\"</code>. </dd> <dt><code>separator</code></dt> <dd> The single ASCII character used to separate values; default: <code>,</code> (U+002C). </dd> </dl></div>" } ], "returns" : { "type" : "object()*", "description" : "a sequence of zero or more JSON objects where each key is a field name and each value is a parsed value." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">csv:INVALID_OPTION if the <code>quote-char</code>, <code>quote-escape</code>, or <code>separator</code> option is given and it's not a single ASCII character.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">csv:MISSING_VALUE if a missing value is detected and the <code>missing-value</code> option is \"<code>error</code>\".</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">csv:EXTRA_VALUE if an extra value is detected and the <code>extra-name</code> option is not set.</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 1, "name" : "serialize", "qname" : "csv:serialize", "signature" : "($obj as object()*) as string*", "description" : " Serializes a sequence of JSON objects as CSV (comma-separated values) using\n the default options.\n The default options are:\n  <dl xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">\n    <dt><code>field-names</code></dt>\n      <dd>\n        A JSON array of strings denoting field names;\n        default: none.\n        The field names are taken from the first JSON object\n        and the order of the fields is implementation dependent.\n      </dd>\n    <dt><code>serialize-boolean-as</code></dt>\n      <dd>\n        What strings to serialize <code>true</code> and <code>false</code> as;\n        default: <code>true</code> and <code>false</code>.\n      </dd>\n    <dt><code>serialize-header</code></dt>\n      <dd>\n        Whether a header line is included;\n        default: <code>true</code>.\n        The first string result is the header line\n        comprised of all the objects' keys' names.\n      </dd>\n    <dt><code>serialize-null-as</code></dt>\n      <dd>\n        What string to serialize JSON <code>null</code> values as;\n        default: <code>null</code>.\n      </dd>\n    <dt><code>quote-char</code></dt>\n      <dd>\n        The single ASCII character that may be used to quote values;\n        default: <code>\"</code> (U+0022).\n      </dd>\n    <dt><code>quote-escape</code></dt>\n      <dd>\n        The single ASCII character used to escape <code>quote-char</code>;\n        default: same as <code>quote-char</code>.\n        This means that <code>quote-char</code> is doubled to escape it.\n      </dd>\n    <dt><code>separator</code></dt>\n      <dd>\n        The single ASCII character used to separate values;\n        default: <code>,</code> (U+002C).\n      </dd>\n  </dl>\n", "summary" : "<p> Serializes a sequence of JSON objects as CSV (comma-separated values) using\n the default options.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "obj", "type" : "object()", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The sequence of JSON objects to serialize.</div>" } ], "returns" : { "type" : "string*", "description" : "a sequence of strings where each string corresponds to a JSON object, aka, \"record.\"" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 2, "name" : "serialize", "qname" : "csv:serialize", "signature" : "($obj as object()*, $options as object()) as string* external", "description" : " Serializes a sequence of JSON objects as CSV (comma-separated values) using\n the given options.\n", "summary" : "<p> Serializes a sequence of JSON objects as CSV (comma-separated values) using\n the given options.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "obj", "type" : "object()", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The sequence of JSON objects to serialize.</div>" }, { "name" : "options", "type" : "object()", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The options to use: <dl> <dt><code>field-names</code></dt> <dd> A JSON array of strings denoting field names; default: none. If this option is not set, the field names are taken from the first JSON object and the order of the fields is implementation dependent. If this option is set, the fields are serielized in the order they are in the array. In either case, every JSON object must have the same keys as the first object. </dd> <dt><code>serialize-boolean-as</code></dt> <dd> What strings to serialize <code>true</code> and <code>false</code> as; default: <code>true</code> and <code>false</code>. This must be a sub-object with the two keys <code>\"true\"</code> and <code>\"false\"</code>, e.g.: <code>{ \"true\" : \"Y\", \"false\" : \"N\" }</code>. </dd> <dt><code>serialize-header</code></dt> <dd> Whether a header line is included; default: <code>true</code>. If <code>true</code>, the first string result is the header line comprised of all the objects' keys' names; if <code>false</code>, the heder line is not returned. </dd> <dt><code>serialize-null-as</code></dt> <dd> What string to serialize JSON <code>null</code> values as; default: <code>null</code>. </dd> <dt><code>quote-char</code></dt> <dd> The single ASCII character that may be used to quote values; default: <code>\"</code> (U+0022). </dd> <dt><code>quote-escape</code></dt> <dd> The single ASCII character used to escape <code>quote-char</code>; default: same as <code>quote-char</code>. If <code>quote-escape</code> equals <code>quote-char</code>, it means that <code>quote-char</code> must be doubled to escape it. If <code>quote-escape</code> does not equal <code>quote-char</code>, it means that <code>quote-escape</code> is used to escape <code>quote-char</code>. For example, a <code>quote-char</code> of <code>\"</code> (U+0022) and a <code>quote-escape</code> of <code>\\</code> (U+005C) means that quotes will be escaped by <code>\\\"</code>. </dd> <dt><code>separator</code></dt> <dd> The single ASCII character used to separate values; default: <code>,</code> (U+002C). </dd> </dl></div>" } ], "returns" : { "type" : "string*", "description" : "a sequence of strings where each string corresponds to a JSON object, aka, \"record.\"" }, "errors" : [  ] } ], "variables" : [  ] }, "http://www.zorba-xquery.com/modules/email/imap" : { "ns" : "http://www.zorba-xquery.com/modules/email/imap", "description" : " This module provides functions for accessing and manipulating emails on mail\n servers through the IMAP protocol.\n All functions in this module receive as the first argument the IMAP host and user\n information. This is an element with the type <code xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">hostInfoType</code> as defined\n in the email schema: <code xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">http://www.zorba-xquery.com/modules/email</code>.\n For example:\n <pre xmlns:xqdoc=\"http://www.xqdoc.org/1.0\" class=\"ace-static\" ace-mode=\"xquery\">\n &lt;email:hostInfo&gt;\n   &lt;email:hostName&gt;imap.example.com&lt;/email:hostName&gt;\n   &lt;email:userName&gt;myuser&lt;/email:userName&gt;\n   &lt;email:password&gt;mypassword&lt;/email:password&gt;\n &lt;/email:hostInfo&gt;\n </pre>\n The <code xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">hostInfoType</code> only needs to be in the email schema namespace\n (<code xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">http://www.zorba-xquery.com/modules/email</code>). It does not need\n to be validated since it's validated by the module.\n", "sees" : [ "<xqdoc:see xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"><a href=\"http://www.washington.edu/imap/\">c-client library part of UW IMAP toolkit</a></xqdoc:see>" ], "authors" : [ "<xqdoc:author xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Daniel Thomas, Gabriel Petrovay</xqdoc:author>" ], "version" : null, "encoding" : "utf-8", "namespaces" : [ { "uri" : "http://zorba.io/annotations", "prefix" : "an" }, { "uri" : "http://www.zorba-xquery.com/modules/email", "prefix" : "email" }, { "uri" : "http://www.zorba-xquery.com/modules/email/imap", "prefix" : "imap" }, { "uri" : "http://zorba.io/options/versioning", "prefix" : "ver" } ], "functions" : [ { "isDocumented" : true, "arity" : 5, "name" : "copy", "qname" : "imap:copy", "signature" : "($host-info as element(email:hostInfo), $mailbox-from as xs:string, $mailbox-to as xs:string, $messages as xs:long+, $uid as xs:boolean?) as empty-sequence()", "description" : " Copies messages between mailboxes.\n Depending on the value of <code xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">$uid</code>, the messages are either specified\n through their sequence number or through their unique id. Both mailboxes must exist.\n", "summary" : "<p> Copies messages between mailboxes.</p>", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "host-info", "type" : "element(email:hostInfo)", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The IMAP host, user name, and password.</div>" }, { "name" : "mailbox-from", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The mailbox in which the messages reside.</div>" }, { "name" : "mailbox-to", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The mailbox in to which the messages are copied.</div>" }, { "name" : "messages", "type" : "xs:long", "occurrence" : "+", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The messages to be copied, specified either by their sequence number or their unique id.</div>" }, { "name" : "uid", "type" : "xs:boolean", "occurrence" : "?", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> If true, <code>$messages</code> are treated as sequence numbers. Else as unique identifiers.</div>" } ], "returns" : { "type" : "empty-sequence()", "description" : "The function is declared as sequential and has side-effects. It returns the empty sequence." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">imap:IMAP0001 If the IMAP operation failed.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">imap:IMAP0002 If the connection to the IMAP server is refused.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">imap:IMAP0003 If no message is found with the provided sequence number/unique identifier.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">err:XQDY0027 If the value of <code>$host-info</code> is not valid according to the email schema: <code>http://www.zorba-xquery.com/modules/email</code></xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 2, "name" : "create", "qname" : "imap:create", "signature" : "($host-info as element(email:hostInfo), $mailbox-name as xs:string) as empty-sequence()", "description" : " Creates a new mailbox for the given user.\n", "summary" : "<p> Creates a new mailbox for the given user.</p>", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "host-info", "type" : "element(email:hostInfo)", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The IMAP host, user name, and password.</div>" }, { "name" : "mailbox-name", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The name for the new mailbox.</div>" } ], "returns" : { "type" : "empty-sequence()", "description" : "The function is declared as sequential and has side-effects. It returns the empty sequence." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">imap:IMAP0001 If the IMAP operation failed.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">imap:IMAP0002 If the connection to the IMAP server is refused.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">err:XQDY0027 If the value of <code>$host-info</code> is not valid according to the email schema: <code>http://www.zorba-xquery.com/modules/email</code></xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 2, "name" : "delete", "qname" : "imap:delete", "signature" : "($host-info as element(email:hostInfo), $mailbox-name as xs:string) as empty-sequence()", "description" : " Deletes a mailbox for the given user.\n", "summary" : "<p> Deletes a mailbox for the given user.</p>", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "host-info", "type" : "element(email:hostInfo)", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The IMAP host, user name, and password.</div>" }, { "name" : "mailbox-name", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The name of the mailbox to delete.</div>" } ], "returns" : { "type" : "empty-sequence()", "description" : "The function is declared as sequential and has side-effects. It returns the empty sequence." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">imap:IMAP0001 If the IMAP operation failed.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">imap:IMAP0002 If the connection to the IMAP server is refused.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">err:XQDY0027 If the value of <code>$host-info</code> is not valid according to the email schema: <code>http://www.zorba-xquery.com/modules/email</code></xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 2, "name" : "expunge", "qname" : "imap:expunge", "signature" : "($host-info as element(email:hostInfo), $mailbox as xs:string) as empty-sequence()", "description" : " Permanently deletes all messages of the given mailbox that have the \"deleted\" flag set.\n", "summary" : "<p> Permanently deletes all messages of the given mailbox that have the \"deleted\" flag set.</p>", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "host-info", "type" : "element(email:hostInfo)", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The IMAP host, user name, and password.</div>" }, { "name" : "mailbox", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The mailbox for which all messages that have the \\Deleted flag set should be permanently deleted.</div>" } ], "returns" : { "type" : "empty-sequence()", "description" : "The function is declared as sequential and has side-effects. It returns the empty sequence." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">imap:IMAP0001 If the IMAP operation failed.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">imap:IMAP0002 If the connection to the IMAP server is refused.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">err:XQDY0027 If the value of <code>$host-info</code> is not valid according to the email schema: <code>http://www.zorba-xquery.com/modules/email</code></xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 4, "name" : "fetch-envelope", "qname" : "imap:fetch-envelope", "signature" : "($host-info as element(email:hostInfo), $mailbox as xs:string, $message-number as xs:long, $uid as xs:boolean?) as element(email:envelope)", "description" : " Fetches the envelope of a message.\n", "summary" : "<p> Fetches the envelope of a message.</p>", "annotation_str" : " %an:nondeterministic", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "nondeterministic", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "host-info", "type" : "element(email:hostInfo)", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The IMAP host, user name, and password.</div>" }, { "name" : "mailbox", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The mailbox in which to search for the message.</div>" }, { "name" : "message-number", "type" : "xs:long", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The message for which to fetch the envelope (depending on <code>$uid</code> either as message sequence number or unique identifier).</div>" }, { "name" : "uid", "type" : "xs:boolean", "occurrence" : "?", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> If true, <code>$message-number</code> is treated as sequence number. Else as unique identifier.</div>" } ], "returns" : { "type" : "element(email:envelope)", "description" : "The <code xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">envelope</code> of the requested message. The result is validated against the schema: <code xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">http://www.zorba-xquery.com/modules/email</code>." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">imap:IMAP0001 If the IMAP operation failed.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">imap:IMAP0002 If the connection to the IMAP server is refused.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">imap:IMAP0003 If no message is found with the provided sequence number/unique identifier.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">err:XQDY0027 If the value of <code>$host-info</code> is not valid according to the email schema: <code>http://www.zorba-xquery.com/modules/email</code></xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 4, "name" : "fetch-flags", "qname" : "imap:fetch-flags", "signature" : "($host-info as element(email:hostInfo), $mailbox as xs:string, $message-number as xs:long, $uid as xs:boolean?) as element(email:flags)", "description" : " Fetches the flags of a message.\n", "summary" : "<p> Fetches the flags of a message.</p>", "annotation_str" : " %an:nondeterministic", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "nondeterministic", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "host-info", "type" : "element(email:hostInfo)", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The IMAP host, user name, and password.</div>" }, { "name" : "mailbox", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The mailbox containing the specified message.</div>" }, { "name" : "message-number", "type" : "xs:long", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> Either the message sequence number or the unique identifier of the message.</div>" }, { "name" : "uid", "type" : "xs:boolean", "occurrence" : "?", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> If true, <code>$message-number</code> is treated as sequence number. Else as unique identifier.</div>" } ], "returns" : { "type" : "element(email:flags)", "description" : "The <code xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">flags</code> of the specified message. The result is validated against the schema: <code xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">http://www.zorba-xquery.com/modules/email</code>." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">imap:IMAP0001 If the IMAP operation failed.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">imap:IMAP0002 If the connection to the IMAP server is refused.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">imap:IMAP0003 If no message is found with the provided sequence number/unique identifier.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">err:XQDY0027 If the value of <code>$host-info</code> is not valid according to the email schema: <code>http://www.zorba-xquery.com/modules/email</code></xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 3, "name" : "fetch-from", "qname" : "imap:fetch-from", "signature" : "($host-info as element(email:hostInfo), $mailbox as xs:string, $message-number as xs:long) as xs:string", "description" : " Fetches the 'from' string of a message.\n Please note that this function only words with message sequence numbers,\n not with unique identifiers. Only the first 255 characters of a 'from'\n string are fetched.\n", "summary" : "<p> Fetches the 'from' string of a message.</p>", "annotation_str" : " %an:nondeterministic", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "nondeterministic", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "host-info", "type" : "element(email:hostInfo)", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The IMAP host, user name, and password.</div>" }, { "name" : "mailbox", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The mailbox for which we want to get the 'from' string of a message.</div>" }, { "name" : "message-number", "type" : "xs:long", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> Denotes the message for which we want the 'from' string.</div>" } ], "returns" : { "type" : "xs:string", "description" : "The 'from' string of the specified message." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">imap:IMAP0001 If the IMAP operation failed.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">imap:IMAP0002 If the connection to the IMAP server is refused.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">imap:IMAP0003 If no message is found with the provided sequence number/unique identifier.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">err:XQDY0027 If the value of <code>$host-info</code> is not valid according to the email schema: <code>http://www.zorba-xquery.com/modules/email</code></xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 3, "name" : "fetch-message-sequence-number", "qname" : "imap:fetch-message-sequence-number", "signature" : "($host-info as element(email:hostInfo), $mailbox as xs:string, $message-number as xs:long) as xs:long", "description" : " Fetches the message sequence number for a given unique identifier.\n", "summary" : "<p> Fetches the message sequence number for a given unique identifier.</p>", "annotation_str" : " %an:nondeterministic", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "nondeterministic", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "host-info", "type" : "element(email:hostInfo)", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The IMAP host, user name, and password.</div>" }, { "name" : "mailbox", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The mailbox for which we want to get the message sequence number of an unique identifier.</div>" }, { "name" : "message-number", "type" : "xs:long", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The unique identifier for which we want the message sequence number.</div>" } ], "returns" : { "type" : "xs:long", "description" : "The message sequence number of the of the given unique identifier." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">imap:IMAP0001 If the IMAP operation failed.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">imap:IMAP0002 If the connection to the IMAP server is refused.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">imap:IMAP0003 If no message is found with the provided sequence number/unique identifier.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">err:XQDY0027 If the value of <code>$host-info</code> is not valid according to the email schema: <code>http://www.zorba-xquery.com/modules/email</code></xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 4, "name" : "fetch-message", "qname" : "imap:fetch-message", "signature" : "($host-info as element(email:hostInfo), $mailbox as xs:string, $message-number as xs:long, $uid as xs:boolean) as element(email:message)", "description" : " Fetches a whole message.\n", "summary" : "<p> Fetches a whole message.</p>", "annotation_str" : " %an:nondeterministic", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "nondeterministic", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "host-info", "type" : "element(email:hostInfo)", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The IMAP host, user name, and password.</div>" }, { "name" : "mailbox", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The mailbox in which to search for the message.</div>" }, { "name" : "message-number", "type" : "xs:long", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The message to fetch, denoted either by its sequence number or unique identifier.</div>" }, { "name" : "uid", "type" : "xs:boolean", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> If true, <code>$message-number</code> is treated as sequence number. Else as unique identifier.</div>" } ], "returns" : { "type" : "element(email:message)", "description" : "the <code xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">message</code> with the given <code xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">$message-number</code>. The result is validated against the schema: <code xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">http://www.zorba-xquery.com/modules/email</code>." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">imap:IMAP0001 If the IMAP operation failed.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">imap:IMAP0002 If the connection to the IMAP server is refused.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">imap:IMAP0003 If no message is found with the provided sequence number/unique identifier.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">err:XQDY0027 If the value of <code>$host-info</code> is not valid according to the email schema: <code>http://www.zorba-xquery.com/modules/email</code></xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 3, "name" : "fetch-subject", "qname" : "imap:fetch-subject", "signature" : "($host-info as element(email:hostInfo), $mailbox as xs:string, $message-number as xs:long) as xs:string", "description" : " Fetches the subject for a message.\n Please note that this function only works with message sequence numbers,\n not with unique identifiers. Only the first 255 characters of a subject\n are fetched.\n", "summary" : "<p> Fetches the subject for a message.</p>", "annotation_str" : " %an:nondeterministic", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "nondeterministic", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "host-info", "type" : "element(email:hostInfo)", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The IMAP host, user name, and password.</div>" }, { "name" : "mailbox", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The mailbox for which we want to get the subject of a message.</div>" }, { "name" : "message-number", "type" : "xs:long", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> Denotes the message for which we want the subject.</div>" } ], "returns" : { "type" : "xs:string", "description" : "The subject of the specified message." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">imap:IMAP0001 If the IMAP operation failed.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">imap:IMAP0002 If the connection to the IMAP server is refused.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">imap:IMAP0003 If no message is found with the provided sequence number/unique identifier.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">err:XQDY0027 If the value of <code>$host-info</code> is not valid according to the email schema: <code>http://www.zorba-xquery.com/modules/email</code></xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 3, "name" : "fetch-uid", "qname" : "imap:fetch-uid", "signature" : "($host-info as element(email:hostInfo), $mailbox as xs:string, $message-number as xs:long) as xs:long", "description" : " Fetches the unique identifier for a given message sequence number.\n", "summary" : "<p> Fetches the unique identifier for a given message sequence number.</p>", "annotation_str" : " %an:nondeterministic", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "nondeterministic", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "host-info", "type" : "element(email:hostInfo)", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The IMAP host, user name, and password.</div>" }, { "name" : "mailbox", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The mailbox for which we want to get the unique identifier of a message sequence number.</div>" }, { "name" : "message-number", "type" : "xs:long", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The message sequence number for which we want the unique identifier.</div>" } ], "returns" : { "type" : "xs:long", "description" : "The unique identifier of the given message sequence number." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">imap:IMAP0001 If the IMAP operation failed.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">imap:IMAP0002 If the connection to the IMAP server is refused.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">imap:IMAP0003 If no message is found with the provided sequence number/unique identifier.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">err:XQDY0027 If the value of <code>$host-info</code> is not valid according to the email schema: <code>http://www.zorba-xquery.com/modules/email</code></xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 4, "name" : "list", "qname" : "imap:list", "signature" : "($host-info as element(email:hostInfo), $mailbox-ref as xs:string, $pattern as xs:string, $only-subscribed as xs:boolean) as element(email:mailbox)*", "description" : " Lists IMAP folders for the specified user on the host that match the pattern.\n", "summary" : "<p> Lists IMAP folders for the specified user on the host that match the pattern.</p>", "annotation_str" : " %an:nondeterministic", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "nondeterministic", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "host-info", "type" : "element(email:hostInfo)", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The IMAP host, user name, and password.</div>" }, { "name" : "mailbox-ref", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> is applied to pattern in an implementation dependent fashion to search for matching mailbox names.</div>" }, { "name" : "pattern", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The pattern for mailboxes to look for (can include wildcards '*' and '%').</div>" }, { "name" : "only-subscribed", "type" : "xs:boolean", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> If set true, only mailboxes are listed to which the user is subscribed.</div>" } ], "returns" : { "type" : "element(email:mailbox)*", "description" : "A sequence of <code xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">mailbox</code> elements. The result elements are validated against the schema: <code xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">http://www.zorba-xquery.com/modules/email</code>." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">imap:IMAP0001 If the IMAP operation failed.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">imap:IMAP0002 If the connection to the IMAP server is refused.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">err:XQDY0027 If the value of <code>$host-info</code> is not valid according to the email schema: <code>http://www.zorba-xquery.com/modules/email</code></xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 5, "name" : "move", "qname" : "imap:move", "signature" : "($host-info as element(email:hostInfo), $mailbox-from as xs:string, $mailbox-to as xs:string, $messages as xs:long+, $uid as xs:boolean?) as empty-sequence()", "description" : " Moves messages between mailboxes.\n Depending on the value of <code xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">$uid</code>, the messages are either specified through\n their sequence number or through their unique id. Both mailboxes must exist.\n", "summary" : "<p> Moves messages between mailboxes.</p>", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "host-info", "type" : "element(email:hostInfo)", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The IMAP host, user name, and password.</div>" }, { "name" : "mailbox-from", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The mailbox in which the messages reside.</div>" }, { "name" : "mailbox-to", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The mailbox in to which the messages should be moved.</div>" }, { "name" : "messages", "type" : "xs:long", "occurrence" : "+", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The messages to be copied, specified either by their sequence number or their unique id.</div>" }, { "name" : "uid", "type" : "xs:boolean", "occurrence" : "?", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> If true, <code>$messages</code> are treated as sequence numbers. Else as unique identifiers.</div>" } ], "returns" : { "type" : "empty-sequence()", "description" : "The function is declared as sequential and has side-effects. It returns the empty sequence." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">imap:IMAP0001 If the IMAP operation failed.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">imap:IMAP0002 If the connection to the IMAP server is refused.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">imap:IMAP0003 If no message is found with the provided sequence number/unique identifier.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">err:XQDY0027 If the value of <code>$host-info</code> is not valid according to the email schema: <code>http://www.zorba-xquery.com/modules/email</code></xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 3, "name" : "rename", "qname" : "imap:rename", "signature" : "($host-info as element(email:hostInfo), $mailbox-old as xs:string, $mailbox-new as xs:string) as empty-sequence()", "description" : " Renames a mailbox.\n", "summary" : "<p> Renames a mailbox.</p>", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "host-info", "type" : "element(email:hostInfo)", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The IMAP host, user name, and password.</div>" }, { "name" : "mailbox-old", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The name of the mailbox we want to rename.</div>" }, { "name" : "mailbox-new", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The new name for the mailbox.</div>" } ], "returns" : { "type" : "empty-sequence()", "description" : "The function is declared as sequential and has side-effects. It returns the empty sequence." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">imap:IMAP0001 If the IMAP operation failed.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">imap:IMAP0002 If the connection to the IMAP server is refused.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">err:XQDY0027 If the value of <code>$host-info</code> is not valid according to the email schema: <code>http://www.zorba-xquery.com/modules/email</code></xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 4, "name" : "search", "qname" : "imap:search", "signature" : "($host-info as element(email:hostInfo), $mailbox as xs:string, $criteria as xs:string, $uid as xs:boolean?) as xs:long*", "description" : " Searches a mailbox for messages that match the given criteria.\n The criteria should be a string as defined in the RFC3501 (IMAP4rev1).\n A valid example would be: 'FROM zorba@gmail.com OR NOT SUBJECT Bug'.\n Depending on the value of <code xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">$uid</code>, the function will either\n return matching sequence numbers or unique identifiers.\n", "summary" : "<p> Searches a mailbox for messages that match the given criteria.</p>", "annotation_str" : " %an:nondeterministic", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "nondeterministic", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "host-info", "type" : "element(email:hostInfo)", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The IMAP host, user name, and password.</div>" }, { "name" : "mailbox", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The mailbox to search.</div>" }, { "name" : "criteria", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The searching criteria.</div>" }, { "name" : "uid", "type" : "xs:boolean", "occurrence" : "?", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> If true, the function returns the sequence of unique identifiers corresponding to the matching mails, else the corresponding sequence numbers are returned.</div>" } ], "returns" : { "type" : "xs:long*", "description" : "Either the sequence of matching sequence numbers or the sequence of matching unique identifiers." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">imap:IMAP0001 If the IMAP operation failed.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">imap:IMAP0002 If the connection to the IMAP server is refused.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">err:XQDY0027 If the value of <code>$host-info</code> is not valid according to the email schema: <code>http://www.zorba-xquery.com/modules/email</code></xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 5, "name" : "set-flags", "qname" : "imap:set-flags", "signature" : "($host-info as element(email:hostInfo), $mailbox as xs:string, $message-number as xs:long, $flags as element(email:flags), $uid as xs:boolean?) as empty-sequence()", "description" : " Sets the flags for a given message.\n The flags are set and unset according to the passed <code xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">$flags</code>.\n", "summary" : "<p> Sets the flags for a given message.</p>", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "host-info", "type" : "element(email:hostInfo)", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The IMAP host, user name, and password.</div>" }, { "name" : "mailbox", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The mailbox containing the specified message.</div>" }, { "name" : "message-number", "type" : "xs:long", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> Either the message sequence number or the unique identifier of the message (depending on the value of <code>$uid</code>).</div>" }, { "name" : "flags", "type" : "element(email:flags)", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> Defines which flags should be set for this message. The possibilities are \"seen\", \"deleted\", \"flagged\", \"answered\", and \"draft\". Setting all flags at once is done by passing the element: <code> &lt;email:flags&gt; &lt;email:seen/&gt; &lt;email:deleted/&gt; &lt;email:flagged/&gt; &lt;email:answered/&gt; &lt;email:draft/&gt; &lt;/email:flags&gt; </code>. Setting \"flagged\" only and unsetting all other at once can be done by passing: <code> &lt;email:flags&gt; &lt;email:flagged/&gt; &lt;/email:flags&gt; </code>.</div>" }, { "name" : "uid", "type" : "xs:boolean", "occurrence" : "?", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> If true, <code>$message-number</code> is treated as sequence number. Else as unique identifier.</div>" } ], "returns" : { "type" : "empty-sequence()", "description" : "The function is declared as sequential and has side-effects. It returns the empty sequence." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">imap:IMAP0001 If the IMAP operation failed.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">imap:IMAP0002 If the connection to the IMAP server is refused.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">imap:IMAP0003 If no message is found with the provided sequence number/unique identifier.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">err:XQDY0027 If the value of <code>$host-info</code> is not valid according to the email schema: <code>http://www.zorba-xquery.com/modules/email</code></xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 2, "name" : "status", "qname" : "imap:status", "signature" : "($host-info as element(email:hostInfo), $mailbox as xs:string) as element(email:status)", "description" : " Returns the status of the given mailbox.\n The status of a mailbox contains:\n <ul xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">\n  <li><code>messages</code>: the number of messages in the mailbox</li>\n  <li><code>recent</code>: the number of messages flagged as recent</li>\n  <li><code>unseen</code>: the number of messages flagged as unseen</li>\n  <li><code>uidnext</code>: the next unique identifier that will be assigned to a message</li>\n  <li><code>uidvalidity</code>: a value that, together with the <code>uidnext</code> value\n    forms a 64 bit number that must be unique for the server</li>\n </ul>\n", "summary" : "<p> Returns the status of the given mailbox.</p>", "annotation_str" : " %an:nondeterministic", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "nondeterministic", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "host-info", "type" : "element(email:hostInfo)", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The IMAP host, user name, and password.</div>" }, { "name" : "mailbox", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The mailbox for which we want to have the status.</div>" } ], "returns" : { "type" : "element(email:status)", "description" : "The <code xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">status</code> of the specified <code xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">$mailbox</code>. The result is validated against the schema: <code xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">http://www.zorba-xquery.com/modules/email</code>." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">imap:IMAP0001 If the IMAP operation failed.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">imap:IMAP0002 If the connection to the IMAP server is refused.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">err:XQDY0027 If the value of <code>$host-info</code> is not valid according to the email schema: <code>http://www.zorba-xquery.com/modules/email</code></xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 2, "name" : "subscribe", "qname" : "imap:subscribe", "signature" : "($host-info as element(email:hostInfo), $mailbox as xs:string) as empty-sequence()", "description" : " Subscribes the user to the specified mailbox.\n", "summary" : "<p> Subscribes the user to the specified mailbox.</p>", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "host-info", "type" : "element(email:hostInfo)", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The IMAP host, user name, and password.</div>" }, { "name" : "mailbox", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The mailbox the user wants to subscribe to.</div>" } ], "returns" : { "type" : "empty-sequence()", "description" : "The function is declared as sequential and has side-effects. It returns the empty sequence." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">imap:IMAP0001 If the IMAP operation failed.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">imap:IMAP0002 If the connection to the IMAP server is refused.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">err:XQDY0027 If the value of <code>$host-info</code> is not valid according to the email schema: <code>http://www.zorba-xquery.com/modules/email</code></xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 2, "name" : "unsubscribe", "qname" : "imap:unsubscribe", "signature" : "($host-info as element(email:hostInfo), $mailbox as xs:string) as empty-sequence()", "description" : " Unsubscribes the user from the specified mailbox.\n", "summary" : "<p> Unsubscribes the user from the specified mailbox.</p>", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "host-info", "type" : "element(email:hostInfo)", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The IMAP host, user name, and password.</div>" }, { "name" : "mailbox", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The mailbox the user wants to unsubscribe from.</div>" } ], "returns" : { "type" : "empty-sequence()", "description" : "The function is declared as sequential and has side-effects. It returns the empty sequence." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">imap:IMAP0001 If the IMAP operation failed.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">imap:IMAP0002 If the connection to the IMAP server is refused.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">err:XQDY0027 If the value of <code>$host-info</code> is not valid according to the email schema: <code>http://www.zorba-xquery.com/modules/email</code></xqdoc:error>" ] } ], "variables" : [  ] }, "http://zorba.io/errors" : { "ns" : "http://zorba.io/errors", "description" : " This module contains one variable declaration for each diagnostic of the\n http://zorba.io/errors namespace.\n The variables serves as documentation for the errors but can also\n be used in the code. For example, one useful scenario is to compare\n an error caught in the catch clause of a try-catch expression with one of\n the variables.\n", "sees" : [  ], "authors" : [ "<xqdoc:author xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Carlos Lopez</xqdoc:author>" ], "version" : null, "encoding" : "utf-8", "namespaces" : [ { "uri" : "http://zorba.io/errors", "prefix" : "zerr" } ], "functions" : [  ], "variables" : [ { "name" : "zerr:ZDST0032", "type" : "item()*", "description" : "" }, { "name" : "zerr:ZDST0004", "type" : "item()*", "description" : "" }, { "name" : "zerr:ZDST0006", "type" : "item()*", "description" : "" }, { "name" : "zerr:ZDST0007", "type" : "item()*", "description" : "" }, { "name" : "zerr:ZDST0021", "type" : "item()*", "description" : "" }, { "name" : "zerr:ZDST0022", "type" : "item()*", "description" : "" }, { "name" : "zerr:ZDST0023", "type" : "item()*", "description" : "" }, { "name" : "zerr:ZDST0024", "type" : "item()*", "description" : "" }, { "name" : "zerr:ZDST0025", "type" : "item()*", "description" : "" }, { "name" : "zerr:ZDST0026", "type" : "item()*", "description" : "" }, { "name" : "zerr:ZDST0027", "type" : "item()*", "description" : "" }, { "name" : "zerr:ZDST0028", "type" : "item()*", "description" : "" }, { "name" : "zerr:ZDST0029", "type" : "item()*", "description" : "" }, { "name" : "zerr:ZDST0030", "type" : "item()*", "description" : "" }, { "name" : "zerr:ZDST0031", "type" : "item()*", "description" : "" }, { "name" : "zerr:ZDST0003", "type" : "item()*", "description" : "" }, { "name" : "zerr:ZDST0033", "type" : "item()*", "description" : "" }, { "name" : "zerr:ZDST0034", "type" : "item()*", "description" : "" }, { "name" : "zerr:ZDST0035", "type" : "item()*", "description" : "" }, { "name" : "zerr:ZDST0036", "type" : "item()*", "description" : "" }, { "name" : "zerr:ZDST0041", "type" : "item()*", "description" : "" }, { "name" : "zerr:ZDST0044", "type" : "item()*", "description" : "" }, { "name" : "zerr:ZDST0048", "type" : "item()*", "description" : "" }, { "name" : "zerr:ZDST0060", "type" : "item()*", "description" : "" }, { "name" : "zerr:ZDTY0001", "type" : "item()*", "description" : "" }, { "name" : "zerr:ZDTY0010", "type" : "item()*", "description" : "" }, { "name" : "zerr:ZDTY0011", "type" : "item()*", "description" : "" }, { "name" : "zerr:ZDTY0012", "type" : "item()*", "description" : "" }, { "name" : "zerr:ZGDB0001", "type" : "item()*", "description" : "" }, { "name" : "zerr:ZOSE0001", "type" : "item()*", "description" : "" }, { "name" : "zerr:ZDDY0038", "type" : "item()*", "description" : "" }, { "name" : "zerr:ZDDY0024", "type" : "item()*", "description" : "" }, { "name" : "zerr:ZDDY0025", "type" : "item()*", "description" : "" }, { "name" : "zerr:ZDDY0026", "type" : "item()*", "description" : "" }, { "name" : "zerr:ZDDY0027", "type" : "item()*", "description" : "" }, { "name" : "zerr:ZDDY0028", "type" : "item()*", "description" : "" }, { "name" : "zerr:ZDDY0029", "type" : "item()*", "description" : "" }, { "name" : "zerr:ZDDY0030", "type" : "item()*", "description" : "" }, { "name" : "zerr:ZDDY0031", "type" : "item()*", "description" : "" }, { "name" : "zerr:ZDDY0032", "type" : "item()*", "description" : "" }, { "name" : "zerr:ZDDY0033", "type" : "item()*", "description" : "" }, { "name" : "zerr:ZDDY0034", "type" : "item()*", "description" : "" }, { "name" : "zerr:ZDDY0035", "type" : "item()*", "description" : "" }, { "name" : "zerr:ZDDY0036", "type" : "item()*", "description" : "" }, { "name" : "zerr:ZDDY0037", "type" : "item()*", "description" : "" }, { "name" : "zerr:ZOSE0002", "type" : "item()*", "description" : "" }, { "name" : "zerr:ZDDY0039", "type" : "item()*", "description" : "" }, { "name" : "zerr:ZDDY0040", "type" : "item()*", "description" : "" }, { "name" : "zerr:ZDDY0041", "type" : "item()*", "description" : "" }, { "name" : "zerr:ZDDY0042", "type" : "item()*", "description" : "" }, { "name" : "zerr:ZDDY0043", "type" : "item()*", "description" : "" }, { "name" : "zerr:ZDDY1000", "type" : "item()*", "description" : "" }, { "name" : "zerr:ZDDY1001", "type" : "item()*", "description" : "" }, { "name" : "zerr:ZDDY1003", "type" : "item()*", "description" : "" }, { "name" : "zerr:ZDDY1004", "type" : "item()*", "description" : "" }, { "name" : "zerr:ZDDY1005", "type" : "item()*", "description" : "" }, { "name" : "zerr:ZDDY1006", "type" : "item()*", "description" : "" }, { "name" : "zerr:ZDST0001", "type" : "item()*", "description" : "" }, { "name" : "zerr:ZDST0002", "type" : "item()*", "description" : "" }, { "name" : "zerr:ZJPE0005", "type" : "item()*", "description" : "" }, { "name" : "zerr:XSST0001", "type" : "item()*", "description" : "" }, { "name" : "zerr:XSST0002", "type" : "item()*", "description" : "" }, { "name" : "zerr:XSST0003", "type" : "item()*", "description" : "" }, { "name" : "zerr:XSST0004", "type" : "item()*", "description" : "" }, { "name" : "zerr:XSST0005", "type" : "item()*", "description" : "" }, { "name" : "zerr:XSST0006", "type" : "item()*", "description" : "" }, { "name" : "zerr:XSST0007", "type" : "item()*", "description" : "" }, { "name" : "zerr:XSST0008", "type" : "item()*", "description" : "" }, { "name" : "zerr:XSST0009", "type" : "item()*", "description" : "" }, { "name" : "zerr:XSST0010", "type" : "item()*", "description" : "" }, { "name" : "zerr:ZJPE0001", "type" : "item()*", "description" : "" }, { "name" : "zerr:ZJPE0002", "type" : "item()*", "description" : "" }, { "name" : "zerr:ZJPE0003", "type" : "item()*", "description" : "" }, { "name" : "zerr:ZJPE0004", "type" : "item()*", "description" : "" }, { "name" : "zerr:ZSTR0066", "type" : "item()*", "description" : "" }, { "name" : "zerr:ZJPE0006", "type" : "item()*", "description" : "" }, { "name" : "zerr:ZJPE0007", "type" : "item()*", "description" : "" }, { "name" : "zerr:ZJPE0008", "type" : "item()*", "description" : "" }, { "name" : "zerr:ZJPE0009", "type" : "item()*", "description" : "" }, { "name" : "zerr:ZJPE0010", "type" : "item()*", "description" : "" }, { "name" : "zerr:ZJSE0001", "type" : "item()*", "description" : "" }, { "name" : "zerr:ZJSE0002", "type" : "item()*", "description" : "" }, { "name" : "zerr:ZJSE0003", "type" : "item()*", "description" : "" }, { "name" : "zerr:ZJSE0004", "type" : "item()*", "description" : "" }, { "name" : "zerr:ZJSE0007", "type" : "item()*", "description" : "" }, { "name" : "zerr:ZJSE0008", "type" : "item()*", "description" : "" }, { "name" : "zerr:ZJSE0009", "type" : "item()*", "description" : "" }, { "name" : "zerr:ZJ2X0001", "type" : "item()*", "description" : "" }, { "name" : "zerr:ZSTR0012", "type" : "item()*", "description" : "" }, { "name" : "zerr:ZOSE0003", "type" : "item()*", "description" : "" }, { "name" : "zerr:ZOSE0004", "type" : "item()*", "description" : "" }, { "name" : "zerr:ZOSE0005", "type" : "item()*", "description" : "" }, { "name" : "zerr:ZOSE0006", "type" : "item()*", "description" : "" }, { "name" : "zerr:ZOSE0007", "type" : "item()*", "description" : "" }, { "name" : "zerr:ZSTR0001", "type" : "item()*", "description" : "" }, { "name" : "zerr:ZSTR0002", "type" : "item()*", "description" : "" }, { "name" : "zerr:ZSTR0003", "type" : "item()*", "description" : "" }, { "name" : "zerr:ZSTR0004", "type" : "item()*", "description" : "" }, { "name" : "zerr:ZSTR0007", "type" : "item()*", "description" : "" }, { "name" : "zerr:ZSTR0008", "type" : "item()*", "description" : "" }, { "name" : "zerr:ZSTR0009", "type" : "item()*", "description" : "" }, { "name" : "zerr:ZSTR0010", "type" : "item()*", "description" : "" }, { "name" : "zerr:ZSTR0011", "type" : "item()*", "description" : "" }, { "name" : "zerr:ZDDY0023", "type" : "item()*", "description" : "" }, { "name" : "zerr:ZSTR0013", "type" : "item()*", "description" : "" }, { "name" : "zerr:ZSTR0015", "type" : "item()*", "description" : "" }, { "name" : "zerr:ZSTR0016", "type" : "item()*", "description" : "" }, { "name" : "zerr:ZSTR0020", "type" : "item()*", "description" : "" }, { "name" : "zerr:ZSTR0021", "type" : "item()*", "description" : "" }, { "name" : "zerr:ZSTR0030", "type" : "item()*", "description" : "" }, { "name" : "zerr:ZSTR0040", "type" : "item()*", "description" : "" }, { "name" : "zerr:ZSTR0041", "type" : "item()*", "description" : "" }, { "name" : "zerr:ZSTR0045", "type" : "item()*", "description" : "" }, { "name" : "zerr:ZSTR0050", "type" : "item()*", "description" : "" }, { "name" : "zerr:ZSTR0055", "type" : "item()*", "description" : "" }, { "name" : "zerr:ZSTR0060", "type" : "item()*", "description" : "" }, { "name" : "zerr:ZSTR0065", "type" : "item()*", "description" : "" }, { "name" : "zerr:ZXQD0004", "type" : "item()*", "description" : "" }, { "name" : "zerr:ZXQP0036", "type" : "item()*", "description" : "" }, { "name" : "zerr:ZXQP0037", "type" : "item()*", "description" : "" }, { "name" : "zerr:ZXQP0038", "type" : "item()*", "description" : "" }, { "name" : "zerr:ZXQP0039", "type" : "item()*", "description" : "" }, { "name" : "zerr:ZXQP0040", "type" : "item()*", "description" : "" }, { "name" : "zerr:ZXQP0050", "type" : "item()*", "description" : "" }, { "name" : "zerr:ZXQP0060", "type" : "item()*", "description" : "" }, { "name" : "zerr:ZXQP0061", "type" : "item()*", "description" : "" }, { "name" : "zerr:ZXQP8401", "type" : "xs:QName", "description" : " The version of the thesaurus is not the expected version.\n" }, { "name" : "zerr:ZXQP8402", "type" : "xs:QName", "description" : " The thesaurus data file's endianness does not match that of the CPU.\n" }, { "name" : "zerr:ZXQP8403", "type" : "xs:QName", "description" : " The thesaurus data contains an unexpected value.\n" }, { "name" : "zerr:ZXQD0001", "type" : "item()*", "description" : "" }, { "name" : "zerr:ZXQD0002", "type" : "item()*", "description" : "" }, { "name" : "zerr:ZXQD0003", "type" : "item()*", "description" : "" }, { "name" : "zerr:ZXQP0035", "type" : "item()*", "description" : "" }, { "name" : "zerr:ZXQD0005", "type" : "item()*", "description" : "" }, { "name" : "zerr:ZXQD0006", "type" : "item()*", "description" : "" }, { "name" : "zerr:ZAPI0002", "type" : "item()*", "description" : "" }, { "name" : "zerr:ZAPI0003", "type" : "item()*", "description" : "" }, { "name" : "zerr:ZAPI0004", "type" : "item()*", "description" : "" }, { "name" : "zerr:ZAPI0005", "type" : "item()*", "description" : "" }, { "name" : "zerr:ZAPI0006", "type" : "item()*", "description" : "" }, { "name" : "zerr:ZAPI0007", "type" : "item()*", "description" : "" }, { "name" : "zerr:ZAPI0008", "type" : "item()*", "description" : "" }, { "name" : "zerr:ZAPI0009", "type" : "item()*", "description" : "" }, { "name" : "zerr:ZAPI0011", "type" : "item()*", "description" : "" }, { "name" : "zerr:ZAPI0014", "type" : "item()*", "description" : "" }, { "name" : "zerr:ZAPI0015", "type" : "item()*", "description" : "" }, { "name" : "zerr:ZAPI0019", "type" : "item()*", "description" : "" }, { "name" : "zerr:ZXQP0014", "type" : "item()*", "description" : "" }, { "name" : "zerr:ZXQP0000", "type" : "xs:QName", "description" : " An \"error\" constant for \"no error.\"\n" }, { "name" : "zerr:ZXQP0001", "type" : "item()*", "description" : "" }, { "name" : "zerr:ZXQP0002", "type" : "xs:QName", "description" : " A Zorba programming assertion failed.  If this error occurs, it is a bug\n and should be reported.\n" }, { "name" : "zerr:ZXQP0003", "type" : "xs:QName", "description" : " Something unexpected occurred in Zorba.  If this error occurs, it is a\n bug and should be reported.\n" }, { "name" : "zerr:ZXQP0004", "type" : "xs:QName", "description" : " A particular XQuery feature has not been implemented by Zorba.\n" }, { "name" : "zerr:ZXQP0005", "type" : "xs:QName", "description" : " A particular XQuery feature has been implemented by Zorba, but the\n feature has not been enabled in the current build.\n" }, { "name" : "zerr:ZXQP0006", "type" : "item()*", "description" : "" }, { "name" : "zerr:ZXQP0007", "type" : "item()*", "description" : "" }, { "name" : "zerr:ZXQP0008", "type" : "item()*", "description" : "" }, { "name" : "zerr:ZXQP0009", "type" : "item()*", "description" : "" }, { "name" : "zerr:ZXQP0010", "type" : "item()*", "description" : "" }, { "name" : "zerr:ZXQP0011", "type" : "item()*", "description" : "" }, { "name" : "zerr:ZXQP0012", "type" : "item()*", "description" : "" }, { "name" : "zerr:ZXQP0013", "type" : "item()*", "description" : "" }, { "name" : "zerr:ZAPI0020", "type" : "item()*", "description" : "" }, { "name" : "zerr:ZXQP0016", "type" : "item()*", "description" : "" }, { "name" : "zerr:ZXQP0017", "type" : "item()*", "description" : "" }, { "name" : "zerr:ZXQP0020", "type" : "item()*", "description" : "" }, { "name" : "zerr:ZXQP0021", "type" : "item()*", "description" : "" }, { "name" : "zerr:ZXQP0024", "type" : "item()*", "description" : "" }, { "name" : "zerr:ZXQP0025", "type" : "item()*", "description" : "" }, { "name" : "zerr:ZXQP0026", "type" : "item()*", "description" : "" }, { "name" : "zerr:ZXQP0028", "type" : "item()*", "description" : "" }, { "name" : "zerr:ZXQP0029", "type" : "item()*", "description" : "" }, { "name" : "zerr:ZXQP0030", "type" : "item()*", "description" : "" }, { "name" : "zerr:ZXQP0031", "type" : "item()*", "description" : "" }, { "name" : "zerr:ZXQP0032", "type" : "item()*", "description" : "" }, { "name" : "zerr:ZXQP0033", "type" : "item()*", "description" : "" }, { "name" : "zerr:ZDDY0009", "type" : "item()*", "description" : "" }, { "name" : "zerr:ZCSE0012", "type" : "item()*", "description" : "" }, { "name" : "zerr:ZCSE0013", "type" : "item()*", "description" : "" }, { "name" : "zerr:ZCSE0014", "type" : "item()*", "description" : "" }, { "name" : "zerr:ZCSE0015", "type" : "item()*", "description" : "" }, { "name" : "zerr:ZCSE0016", "type" : "item()*", "description" : "" }, { "name" : "zerr:ZCSE0017", "type" : "item()*", "description" : "" }, { "name" : "zerr:ZDDY0001", "type" : "item()*", "description" : "" }, { "name" : "zerr:ZDDY0002", "type" : "item()*", "description" : "" }, { "name" : "zerr:ZDDY0003", "type" : "item()*", "description" : "" }, { "name" : "zerr:ZDDY0004", "type" : "item()*", "description" : "" }, { "name" : "zerr:ZDDY0005", "type" : "item()*", "description" : "" }, { "name" : "zerr:ZDDY0006", "type" : "item()*", "description" : "" }, { "name" : "zerr:ZDDY0007", "type" : "item()*", "description" : "" }, { "name" : "zerr:ZDDY0008", "type" : "item()*", "description" : "" }, { "name" : "zerr:ZCSE0011", "type" : "item()*", "description" : "" }, { "name" : "zerr:ZDDY0010", "type" : "item()*", "description" : "" }, { "name" : "zerr:ZDDY0011", "type" : "item()*", "description" : "" }, { "name" : "zerr:ZDDY0012", "type" : "item()*", "description" : "" }, { "name" : "zerr:ZDDY0013", "type" : "item()*", "description" : "" }, { "name" : "zerr:ZDDY0014", "type" : "item()*", "description" : "" }, { "name" : "zerr:ZDDY0015", "type" : "item()*", "description" : "" }, { "name" : "zerr:ZDDY0016", "type" : "item()*", "description" : "" }, { "name" : "zerr:ZDDY0017", "type" : "item()*", "description" : "" }, { "name" : "zerr:ZDDY0018", "type" : "item()*", "description" : "" }, { "name" : "zerr:ZDDY0020", "type" : "item()*", "description" : "" }, { "name" : "zerr:ZDDY0019", "type" : "item()*", "description" : "" }, { "name" : "zerr:ZDDY0021", "type" : "item()*", "description" : "" }, { "name" : "zerr:ZDDY0022", "type" : "item()*", "description" : "" }, { "name" : "zerr:ZAPI0045", "type" : "item()*", "description" : "" }, { "name" : "zerr:ZAPI0021", "type" : "item()*", "description" : "" }, { "name" : "zerr:ZAPI0023", "type" : "item()*", "description" : "" }, { "name" : "zerr:ZAPI0024", "type" : "item()*", "description" : "" }, { "name" : "zerr:ZAPI0025", "type" : "item()*", "description" : "" }, { "name" : "zerr:ZAPI0026", "type" : "item()*", "description" : "" }, { "name" : "zerr:ZAPI0027", "type" : "item()*", "description" : "" }, { "name" : "zerr:ZAPI0028", "type" : "item()*", "description" : "" }, { "name" : "zerr:ZAPI0029", "type" : "item()*", "description" : "" }, { "name" : "zerr:ZAPI0030", "type" : "item()*", "description" : "" }, { "name" : "zerr:ZAPI0039", "type" : "item()*", "description" : "" }, { "name" : "zerr:ZAPI0040", "type" : "item()*", "description" : "" }, { "name" : "zerr:ZAPI0041", "type" : "item()*", "description" : "" }, { "name" : "zerr:ZAPI0042", "type" : "item()*", "description" : "" }, { "name" : "zerr:ZAPI0043", "type" : "item()*", "description" : "" }, { "name" : "zerr:NS", "type" : "item()*", "description" : "" }, { "name" : "zerr:ZAPI0070", "type" : "item()*", "description" : "" }, { "name" : "zerr:ZAPI0080", "type" : "item()*", "description" : "" }, { "name" : "zerr:ZAPI0090", "type" : "item()*", "description" : "" }, { "name" : "zerr:ZCSE0001", "type" : "item()*", "description" : "" }, { "name" : "zerr:ZCSE0002", "type" : "item()*", "description" : "" }, { "name" : "zerr:ZCSE0003", "type" : "item()*", "description" : "" }, { "name" : "zerr:ZCSE0004", "type" : "item()*", "description" : "" }, { "name" : "zerr:ZCSE0005", "type" : "item()*", "description" : "" }, { "name" : "zerr:ZCSE0006", "type" : "item()*", "description" : "" }, { "name" : "zerr:ZCSE0007", "type" : "item()*", "description" : "" }, { "name" : "zerr:ZCSE0008", "type" : "item()*", "description" : "" }, { "name" : "zerr:ZCSE0009", "type" : "item()*", "description" : "" }, { "name" : "zerr:ZCSE0010", "type" : "item()*", "description" : "" } ] }, "http://zorba.io/modules/sequence" : { "ns" : "http://zorba.io/modules/sequence", "description" : " This module provides an XQuery API to perform set operations on sequences.\n", "sees" : [  ], "authors" : [ "<xqdoc:author xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Paul J. Lucas</xqdoc:author>" ], "version" : null, "encoding" : "utf-8", "namespaces" : [ { "uri" : "http://zorba.io/modules/sequence", "prefix" : "seq" }, { "uri" : "http://zorba.io/options/versioning", "prefix" : "ver" } ], "functions" : [ { "isDocumented" : true, "arity" : 2, "name" : "value-except", "qname" : "seq:value-except", "signature" : "($seq1 as xs:anyAtomicType*, $seq2 as xs:anyAtomicType*) as xs:anyAtomicType* external", "description" : " Filters the first sequence of atomic items such that they are not in the\n second sequence based on their values.\n", "summary" : "<p> Filters the first sequence of atomic items such that they are not in the\n second sequence based on their values.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "seq1", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The first sequence.</div>" }, { "name" : "seq2", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The second sequence.</div>" } ], "returns" : { "type" : "xs:anyAtomicType*", "description" : "a sequence only containing items from <code xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">$seq1</code> that are not in <code xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">$seq2</code>." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 2, "name" : "value-intersect", "qname" : "seq:value-intersect", "signature" : "($seq1 as xs:anyAtomicType*, $seq2 as xs:anyAtomicType*) as xs:anyAtomicType* external", "description" : " Performs a set intersection of two sequences of atomic items based on their\n values.\n", "summary" : "<p> Performs a set intersection of two sequences of atomic items based on their\n values.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "seq1", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The first sequence.</div>" }, { "name" : "seq2", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The second sequence.</div>" } ], "returns" : { "type" : "xs:anyAtomicType*", "description" : "a sequence containing only items from <code xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">$seq1</code> that are also in <code xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">$seq2</code>." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 2, "name" : "value-union", "qname" : "seq:value-union", "signature" : "($seq1 as xs:anyAtomicType*, $seq2 as xs:anyAtomicType*) as xs:anyAtomicType* external", "description" : " Performs a set union of two sequences of atomic items based on their values.\n", "summary" : "<p> Performs a set union of two sequences of atomic items based on their values.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "seq1", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The first sequence.</div>" }, { "name" : "seq2", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The second sequence.</div>" } ], "returns" : { "type" : "xs:anyAtomicType*", "description" : "a sequence containing all items from <code xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">$seq1</code> and <code xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">seq2$</code> but without duplicates." }, "errors" : [  ] } ], "variables" : [  ] }, "http://api.28.io/functions" : { "ns" : "http://api.28.io/functions", "description" : "", "sees" : [  ], "authors" : [  ], "version" : null, "encoding" : null, "namespaces" : [ { "uri" : "", "prefix" : "an" }, { "uri" : "http://api.28.io/functions", "prefix" : "functions" }, { "uri" : "http://api.28.io/model", "prefix" : "model" }, { "uri" : "http://www.zorba-xquery.com/schemas/pul", "prefix" : "pul" }, { "uri" : "http://zorba.io/modules/reference", "prefix" : "ref" }, { "uri" : "http://www.28msec.com/modules/http/request", "prefix" : "req" }, { "uri" : "http://www.28msec.com/modules/http/response", "prefix" : "response" }, { "uri" : "http://api.28.io/sandbox", "prefix" : "sandbox" }, { "uri" : "http://www.zorba-xquery.com/schemas/xdm", "prefix" : "xdm" } ], "functions" : [ { "isDocumented" : false, "arity" : 1, "name" : "apply-pul", "qname" : "functions:apply-pul", "signature" : "($pul)", "description" : "", "summary" : "", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "pul", "type" : null, "occurrence" : null, "description" : "" } ], "returns" : { "type" : null, "description" : "" }, "errors" : [  ] }, { "isDocumented" : false, "arity" : 0, "name" : "collections", "qname" : "functions:collections", "signature" : "()", "description" : "", "summary" : "", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [  ], "returns" : { "type" : null, "description" : "" }, "errors" : [  ] }, { "isDocumented" : false, "arity" : 1, "name" : "decodeURI", "qname" : "functions:decodeURI", "signature" : "($str as xs:string) as xs:anyURI", "description" : "", "summary" : "", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "str", "type" : "xs:string", "occurrence" : null, "description" : "" } ], "returns" : { "type" : "xs:anyURI", "description" : "" }, "errors" : [  ] }, { "isDocumented" : false, "arity" : 1, "name" : "encode-for-js", "qname" : "functions:encode-for-js", "signature" : "($str as xs:string) as xs:string", "description" : "", "summary" : "", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "str", "type" : "xs:string", "occurrence" : null, "description" : "" } ], "returns" : { "type" : "xs:string", "description" : "" }, "errors" : [  ] }, { "isDocumented" : false, "arity" : 5, "name" : "entries", "qname" : "functions:entries", "signature" : "($collection as xs:string?, $node as xs:anyURI?, $index as xs:string?, $value as xs:string?, $format as xs:string)", "description" : "", "summary" : "", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "collection", "type" : "xs:string", "occurrence" : "?", "description" : "" }, { "name" : "node", "type" : "xs:anyURI", "occurrence" : "?", "description" : "" }, { "name" : "index", "type" : "xs:string", "occurrence" : "?", "description" : "" }, { "name" : "value", "type" : "xs:string", "occurrence" : "?", "description" : "" }, { "name" : "format", "type" : "xs:string", "occurrence" : null, "description" : "" } ], "returns" : { "type" : null, "description" : "" }, "errors" : [  ] }, { "isDocumented" : false, "arity" : 0, "name" : "indexes", "qname" : "functions:indexes", "signature" : "()", "description" : "", "summary" : "", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [  ], "returns" : { "type" : null, "description" : "" }, "errors" : [  ] }, { "isDocumented" : false, "arity" : 3, "name" : "key", "qname" : "functions:key", "signature" : "($collection as xs:string, $condition as xs:string, $namespaces)", "description" : "", "summary" : "", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "collection", "type" : "xs:string", "occurrence" : null, "description" : "" }, { "name" : "condition", "type" : "xs:string", "occurrence" : null, "description" : "" }, { "name" : "namespaces", "type" : null, "occurrence" : null, "description" : "" } ], "returns" : { "type" : null, "description" : "" }, "errors" : [  ] }, { "isDocumented" : false, "arity" : 3, "name" : "namesearch", "qname" : "functions:namesearch", "signature" : "($collection as xs:string, $path as xs:string, $namespaces)", "description" : "", "summary" : "", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "collection", "type" : "xs:string", "occurrence" : null, "description" : "" }, { "name" : "path", "type" : "xs:string", "occurrence" : null, "description" : "" }, { "name" : "namespaces", "type" : null, "occurrence" : null, "description" : "" } ], "returns" : { "type" : null, "description" : "" }, "errors" : [  ] }, { "isDocumented" : false, "arity" : 1, "name" : "node-to-js", "qname" : "functions:node-to-js", "signature" : "($node) as xs:string*", "description" : "", "summary" : "", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "node", "type" : null, "occurrence" : null, "description" : "" } ], "returns" : { "type" : "xs:string*", "description" : "" }, "errors" : [  ] }, { "isDocumented" : false, "arity" : 1, "name" : "parse-namespaces", "qname" : "functions:parse-namespaces", "signature" : "($namespaces as xs:string)", "description" : "", "summary" : "", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "namespaces", "type" : "xs:string", "occurrence" : null, "description" : "" } ], "returns" : { "type" : null, "description" : "" }, "errors" : [  ] }, { "isDocumented" : false, "arity" : 1, "name" : "pksearch", "qname" : "functions:pksearch", "signature" : "($collection as xs:string)", "description" : "", "summary" : "", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "collection", "type" : "xs:string", "occurrence" : null, "description" : "" } ], "returns" : { "type" : null, "description" : "" }, "errors" : [  ] }, { "isDocumented" : false, "arity" : 0, "name" : "startpage", "qname" : "functions:startpage", "signature" : "()", "description" : "", "summary" : "", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [  ], "returns" : { "type" : null, "description" : "" }, "errors" : [  ] }, { "isDocumented" : false, "arity" : 1, "name" : "strip-whitespaces", "qname" : "functions:strip-whitespaces", "signature" : "($xml)", "description" : "", "summary" : "", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "xml", "type" : null, "occurrence" : null, "description" : "" } ], "returns" : { "type" : null, "description" : "" }, "errors" : [  ] }, { "isDocumented" : false, "arity" : 3, "name" : "tuple", "qname" : "functions:tuple", "signature" : "($collection as xs:string, $condition as xs:string, $namespaces)", "description" : "", "summary" : "", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "collection", "type" : "xs:string", "occurrence" : null, "description" : "" }, { "name" : "condition", "type" : "xs:string", "occurrence" : null, "description" : "" }, { "name" : "namespaces", "type" : null, "occurrence" : null, "description" : "" } ], "returns" : { "type" : null, "description" : "" }, "errors" : [  ] }, { "isDocumented" : false, "arity" : 3, "name" : "valuesearch", "qname" : "functions:valuesearch", "signature" : "($collection as xs:string, $path as xs:string, $namespaces)", "description" : "", "summary" : "", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "collection", "type" : "xs:string", "occurrence" : null, "description" : "" }, { "name" : "path", "type" : "xs:string", "occurrence" : null, "description" : "" }, { "name" : "namespaces", "type" : null, "occurrence" : null, "description" : "" } ], "returns" : { "type" : null, "description" : "" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "xmltojs", "qname" : "functions:xmltojs", "signature" : "($content)", "description" : " Internal function. Converts the XML contents of a POST BODY to JavaScript calls that rebuild the given XML for the collection browser frontend\n", "summary" : "<p> Internal function.</p>", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "content", "type" : null, "occurrence" : null, "description" : "" } ], "returns" : { "type" : null, "description" : "JavaScript calls for collection browser frontend" }, "errors" : [  ] } ], "variables" : [  ] }, "http://xbrl.io/modules/bizql/profiles/sec/companies" : { "ns" : "http://xbrl.io/modules/bizql/profiles/sec/companies", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">This module provides functionality for querying companies (XBRL entities)\n  submitting to the SEC.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">SEC Companies are nothing else than XBRL entities. For XBRL-generic requests on\n entities, use the generic entities module.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">With this module, you can retrieve a company with its CIK (without converting\n it to an EID). You can also retrieve companies by sector, by SIC code, by types,\n by tags, by tickers.</p>\n", "sees" : [  ], "authors" : [ "<xqdoc:author xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Charles Hoffman</xqdoc:author>", "<xqdoc:author xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Matthias Brantner</xqdoc:author>", "<xqdoc:author xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Dennis Knochenwefel</xqdoc:author>", "<xqdoc:author xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Ghislain Fourny</xqdoc:author>" ], "version" : null, "encoding" : "utf-8", "namespaces" : [ { "uri" : "http://xbrl.io/modules/bizql/archives", "prefix" : "archives" }, { "uri" : "http://xbrl.io/modules/bizql/profiles/sec/companies", "prefix" : "companies" }, { "uri" : "http://www.28msec.com/modules/credentials", "prefix" : "credentials" }, { "uri" : "http://xbrl.io/modules/bizql/entities", "prefix" : "entities" }, { "uri" : "http://www.28msec.com/modules/mongodb", "prefix" : "mongo" }, { "uri" : "http://xbrl.io/modules/bizql/profiles/sec/core", "prefix" : "sec" }, { "uri" : "http://zorba.io/options/versioning", "prefix" : "ver" } ], "functions" : [ { "isDocumented" : true, "arity" : 1, "name" : "companies-by-types", "qname" : "companies:companies-by-types", "signature" : "($company-types as string*) as object()*", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Retrieves all companies whose company type matches the passed string(s).</p>\n", "summary" : "<p>  Retrieves all companies whose company type matches the passed string(s).</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "company-types", "type" : "string", "occurrence" : "*", "description" : "" } ], "returns" : { "type" : "object()*", "description" : "all companies with matching company type." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "companies-for-SIC", "qname" : "companies:companies-for-SIC", "signature" : "($sic-codes as string*) as object()*", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Retrieves all companies whose type of business\n matches the SIC (Standard Industrial Classification) code.</p>\n", "summary" : "<p>  Retrieves all companies whose type of business\n matches the SIC (Standard Industrial Classification) code.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "sic-codes", "type" : "string", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> a sequence of SIC codes.</div>" } ], "returns" : { "type" : "object()*", "description" : "all companies with one of these SIC codes." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "companies-for-sector", "qname" : "companies:companies-for-sector", "signature" : "($sectors as string*) as object()*", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Retrieves all companies in the given sectors.</p>\n", "summary" : "<p>  Retrieves all companies in the given sectors.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "sectors", "type" : "string", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> a sequence of sectors as strings.</div>" } ], "returns" : { "type" : "object()*", "description" : "all companies in these sectors." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "companies-for-tags", "qname" : "companies:companies-for-tags", "signature" : "($tags as string*) as object()*", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Return all companies with any of the given tags.</p>\n", "summary" : "<p>  Return all companies with any of the given tags.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "tags", "type" : "string", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the tags to filter.</div>" } ], "returns" : { "type" : "object()*", "description" : "all companies with the given tags." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "companies-for-tickers", "qname" : "companies:companies-for-tickers", "signature" : "($tickers as string*) as object()*", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Return all companies with any of the given ticker symbols.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Tickers are case insensitive</p>\n", "summary" : "<p>  Return all companies with any of the given ticker symbols.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "tickers", "type" : "string", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the tickers to filter.</div>" } ], "returns" : { "type" : "object()*", "description" : "all companies with the given tickers." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 0, "name" : "companies", "qname" : "companies:companies", "signature" : "() as object()*", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Return all companies</p>\n", "summary" : "<p>  Return all companies \n</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [  ], "returns" : { "type" : "object()*", "description" : "all companies." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "companies", "qname" : "companies:companies", "signature" : "($companies-or-ids as item()*) as object()*", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Return the companies with the given identifiers.</p>\n", "summary" : "<p>  Return the companies with the given identifiers.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "companies-or-ids", "type" : "item()", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the ids of the companies or the companies themselves.</div>" } ], "returns" : { "type" : "object()*", "description" : "the companies with the given identifiers the empty sequence if no company was found or if the input is an empty sequence." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "company-type", "qname" : "companies:company-type", "signature" : "($company-name as string) as string", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Return company type for a given company name. Company type can be one of:</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">\n   <ul>\n     <li>Corporation</li>\n     <li>Partnership</li>\n     <li>unknown</li>\n   </ul>\n </p>\n", "summary" : "<p>  Return company type for a given company name.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "company-name", "type" : "string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the name of a company</div>" } ], "returns" : { "type" : "string", "description" : "the company type string or \"unknown\" if the type can not be inferred" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "eid", "qname" : "companies:eid", "signature" : "($companies-or-eids-or-ciks as item()*) as string*", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Converts the input to a normalized CIK. The input\n can be either a pure CIK without scheme, or an already\n normalized CIK, or an entity object which contains a CIK\n in its id field.</p>\n", "summary" : "<p>  Converts the input to a normalized CIK.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "companies-or-eids-or-ciks", "type" : "item()", "occurrence" : "*", "description" : "" } ], "returns" : { "type" : "string*", "description" : "the normalized CIK." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">sec:INVALID_PARAMETER if the CIK or entity is not valid</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 1, "name" : "types", "qname" : "companies:types", "signature" : "($companies-or-ciks as item()*) as string*", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Retrieves the type of a company.</p>\n", "summary" : "<p>  Retrieves the type of a company.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "companies-or-ciks", "type" : "item()", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> a sequence of companies or their identifiers (CIKs).</div>" } ], "returns" : { "type" : "string*", "description" : "all company types." }, "errors" : [  ] } ], "variables" : [  ] }, "http://api.28.io/collections" : { "ns" : "http://api.28.io/collections", "description" : "", "sees" : [  ], "authors" : [  ], "version" : null, "encoding" : null, "namespaces" : [ { "uri" : "", "prefix" : "an" }, { "uri" : "http://api.28.io/collections", "prefix" : "cm" }, { "uri" : "http://www.28msec.com/modules/http/request", "prefix" : "req" }, { "uri" : "http://www.28msec.com/modules/http/response", "prefix" : "resp" }, { "uri" : "http://api.28.io/util", "prefix" : "util" }, { "uri" : "http://api.28.io/validation", "prefix" : "validate" } ], "functions" : [ { "isDocumented" : false, "arity" : 1, "name" : "definition-for-dynamic-collection", "qname" : "cm:definition-for-dynamic-collection", "signature" : "($name as xs:string) as object()", "description" : "", "summary" : "", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "name", "type" : "xs:string", "occurrence" : null, "description" : "" } ], "returns" : { "type" : "object()", "description" : "" }, "errors" : [  ] }, { "isDocumented" : false, "arity" : 2, "name" : "delete-collection-property", "qname" : "cm:delete-collection-property", "signature" : "($name as xs:string, $property as xs:string) as empty-sequence()", "description" : "", "summary" : "", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "name", "type" : "xs:string", "occurrence" : null, "description" : "" }, { "name" : "property", "type" : "xs:string", "occurrence" : null, "description" : "" } ], "returns" : { "type" : "empty-sequence()", "description" : "" }, "errors" : [  ] }, { "isDocumented" : false, "arity" : 1, "name" : "delete-collection", "qname" : "cm:delete-collection", "signature" : "($name) as empty-sequence()", "description" : "", "summary" : "", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "name", "type" : null, "occurrence" : null, "description" : "" } ], "returns" : { "type" : "empty-sequence()", "description" : "" }, "errors" : [  ] }, { "isDocumented" : false, "arity" : 0, "name" : "dispatch", "qname" : "cm:dispatch", "signature" : "()", "description" : "", "summary" : "", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [  ], "returns" : { "type" : null, "description" : "" }, "errors" : [  ] }, { "isDocumented" : false, "arity" : 2, "name" : "get-collection-property", "qname" : "cm:get-collection-property", "signature" : "($name as xs:string, $property as xs:string)", "description" : "", "summary" : "", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "name", "type" : "xs:string", "occurrence" : null, "description" : "" }, { "name" : "property", "type" : "xs:string", "occurrence" : null, "description" : "" } ], "returns" : { "type" : null, "description" : "" }, "errors" : [  ] }, { "isDocumented" : false, "arity" : 1, "name" : "get-collection", "qname" : "cm:get-collection", "signature" : "($name as xs:string) as object()", "description" : "", "summary" : "", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "name", "type" : "xs:string", "occurrence" : null, "description" : "" } ], "returns" : { "type" : "object()", "description" : "" }, "errors" : [  ] }, { "isDocumented" : false, "arity" : 1, "name" : "get-or-create-collection-dynamic", "qname" : "cm:get-or-create-collection-dynamic", "signature" : "($name as xs:string) as object()", "description" : "", "summary" : "", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "name", "type" : "xs:string", "occurrence" : null, "description" : "" } ], "returns" : { "type" : "object()", "description" : "" }, "errors" : [  ] }, { "isDocumented" : false, "arity" : 0, "name" : "list-collections", "qname" : "cm:list-collections", "signature" : "() as array()", "description" : "", "summary" : "", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [  ], "returns" : { "type" : "array()", "description" : "" }, "errors" : [  ] }, { "isDocumented" : false, "arity" : 0, "name" : "metadata", "qname" : "cm:metadata", "signature" : "()", "description" : "", "summary" : "", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [  ], "returns" : { "type" : null, "description" : "" }, "errors" : [  ] }, { "isDocumented" : false, "arity" : 3, "name" : "put-collection-property", "qname" : "cm:put-collection-property", "signature" : "($name as xs:string, $property as xs:string, $value) as empty-sequence()", "description" : "", "summary" : "", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "name", "type" : "xs:string", "occurrence" : null, "description" : "" }, { "name" : "property", "type" : "xs:string", "occurrence" : null, "description" : "" }, { "name" : "value", "type" : null, "occurrence" : null, "description" : "" } ], "returns" : { "type" : "empty-sequence()", "description" : "" }, "errors" : [  ] }, { "isDocumented" : false, "arity" : 2, "name" : "put-collection", "qname" : "cm:put-collection", "signature" : "($name as xs:string, $collection as object()) as empty-sequence()", "description" : "", "summary" : "", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "name", "type" : "xs:string", "occurrence" : null, "description" : "" }, { "name" : "collection", "type" : "object()", "occurrence" : null, "description" : "" } ], "returns" : { "type" : "empty-sequence()", "description" : "" }, "errors" : [  ] }, { "isDocumented" : false, "arity" : 1, "name" : "validate-collection", "qname" : "cm:validate-collection", "signature" : "($collection as object()) as empty-sequence()", "description" : "", "summary" : "", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "collection", "type" : "object()", "occurrence" : null, "description" : "" } ], "returns" : { "type" : "empty-sequence()", "description" : "" }, "errors" : [  ] } ], "variables" : [  ] }, "http://zorba.io/modules/excel/math-sumproduct" : { "ns" : "http://zorba.io/modules/excel/math-sumproduct", "description" : " Module implementing the sumproduct functions from Excel 2003 math library.\n There are 30 functions defined, implementing the same function\n but with 1 to 30 parameters.\n Each parameter can be a sequence of infinite length.\n", "sees" : [ "<xqdoc:see xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"><a href=\"http://office.microsoft.com/en-us/excel/HP052092931033.aspx\" target=\"_blank\">Excel 2003 Documentation: Math-sumproduct Functions</a></xqdoc:see>" ], "authors" : [ "<xqdoc:author xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Daniel Turcanu</xqdoc:author>" ], "version" : null, "encoding" : "utf-8", "namespaces" : [ { "uri" : "http://zorba.io/modules/excel/math-sumproduct", "prefix" : "excel" }, { "uri" : "http://zorba.io/modules/excel/errors", "prefix" : "excel-err" }, { "uri" : "http://zorba.io/modules/excel/math", "prefix" : "excel-math" }, { "uri" : "http://zorba.io/options/versioning", "prefix" : "ver" } ], "functions" : [ { "isDocumented" : true, "arity" : 1, "name" : "sumproduct", "qname" : "excel:sumproduct", "signature" : "($array1 as xs:anyAtomicType*) as xs:anyAtomicType", "description" : " Sums the values in the sequence.\n The sequence can be of any length.\n", "summary" : "<p> Sums the values in the sequence.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "array1", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequence of numbers or arguments castable to numeric</div>" } ], "returns" : { "type" : "xs:anyAtomicType", "description" : "the sum" }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">excel-err:Value if the parameters cannot be casted to numeric type</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 10, "name" : "sumproduct", "qname" : "excel:sumproduct", "signature" : "($array1 as xs:anyAtomicType*, $array2 as xs:anyAtomicType*, $array3 as xs:anyAtomicType*, $array4 as xs:anyAtomicType*, $array5 as xs:anyAtomicType*, $array6 as xs:anyAtomicType*, $array7 as xs:anyAtomicType*, $array8 as xs:anyAtomicType*, $array9 as xs:anyAtomicType*, $array10 as xs:anyAtomicType*) as xs:anyAtomicType", "description" : " Multiplies the elements on the same position in each sequence\n and sums up the results.\n", "summary" : "<p> Multiplies the elements on the same position in each sequence\n and sums up the results.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "array1", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" }, { "name" : "array2", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" }, { "name" : "array3", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" }, { "name" : "array4", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" }, { "name" : "array5", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" }, { "name" : "array6", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" }, { "name" : "array7", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" }, { "name" : "array8", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" }, { "name" : "array9", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" }, { "name" : "array10", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" } ], "returns" : { "type" : "xs:anyAtomicType", "description" : "the sum of products" }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">excel-err:Value if the parameters cannot be casted to numeric type</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 11, "name" : "sumproduct", "qname" : "excel:sumproduct", "signature" : "($array1 as xs:anyAtomicType*, $array2 as xs:anyAtomicType*, $array3 as xs:anyAtomicType*, $array4 as xs:anyAtomicType*, $array5 as xs:anyAtomicType*, $array6 as xs:anyAtomicType*, $array7 as xs:anyAtomicType*, $array8 as xs:anyAtomicType*, $array9 as xs:anyAtomicType*, $array10 as xs:anyAtomicType*, $array11 as xs:anyAtomicType*) as xs:anyAtomicType", "description" : " Multiplies the elements on the same position in each sequence\n and sums up the results.\n", "summary" : "<p> Multiplies the elements on the same position in each sequence\n and sums up the results.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "array1", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" }, { "name" : "array2", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" }, { "name" : "array3", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" }, { "name" : "array4", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" }, { "name" : "array5", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" }, { "name" : "array6", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" }, { "name" : "array7", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" }, { "name" : "array8", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" }, { "name" : "array9", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" }, { "name" : "array10", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" }, { "name" : "array11", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" } ], "returns" : { "type" : "xs:anyAtomicType", "description" : "the sum of products" }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">excel-err:Value if the parameters cannot be casted to numeric type</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 12, "name" : "sumproduct", "qname" : "excel:sumproduct", "signature" : "($array1 as xs:anyAtomicType*, $array2 as xs:anyAtomicType*, $array3 as xs:anyAtomicType*, $array4 as xs:anyAtomicType*, $array5 as xs:anyAtomicType*, $array6 as xs:anyAtomicType*, $array7 as xs:anyAtomicType*, $array8 as xs:anyAtomicType*, $array9 as xs:anyAtomicType*, $array10 as xs:anyAtomicType*, $array11 as xs:anyAtomicType*, $array12 as xs:anyAtomicType*) as xs:anyAtomicType", "description" : " Multiplies the elements on the same position in each sequence\n and sums up the results.\n", "summary" : "<p> Multiplies the elements on the same position in each sequence\n and sums up the results.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "array1", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" }, { "name" : "array2", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" }, { "name" : "array3", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" }, { "name" : "array4", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" }, { "name" : "array5", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" }, { "name" : "array6", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" }, { "name" : "array7", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" }, { "name" : "array8", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" }, { "name" : "array9", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" }, { "name" : "array10", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" }, { "name" : "array11", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" }, { "name" : "array12", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" } ], "returns" : { "type" : "xs:anyAtomicType", "description" : "the sum of products" }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">excel-err:Value if the parameters cannot be casted to numeric type</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 13, "name" : "sumproduct", "qname" : "excel:sumproduct", "signature" : "($array1 as xs:anyAtomicType*, $array2 as xs:anyAtomicType*, $array3 as xs:anyAtomicType*, $array4 as xs:anyAtomicType*, $array5 as xs:anyAtomicType*, $array6 as xs:anyAtomicType*, $array7 as xs:anyAtomicType*, $array8 as xs:anyAtomicType*, $array9 as xs:anyAtomicType*, $array10 as xs:anyAtomicType*, $array11 as xs:anyAtomicType*, $array12 as xs:anyAtomicType*, $array13 as xs:anyAtomicType*) as xs:anyAtomicType", "description" : " Multiplies the elements on the same position in each sequence\n and sums up the results.\n", "summary" : "<p> Multiplies the elements on the same position in each sequence\n and sums up the results.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "array1", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" }, { "name" : "array2", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" }, { "name" : "array3", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" }, { "name" : "array4", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" }, { "name" : "array5", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" }, { "name" : "array6", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" }, { "name" : "array7", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" }, { "name" : "array8", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" }, { "name" : "array9", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" }, { "name" : "array10", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" }, { "name" : "array11", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" }, { "name" : "array12", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" }, { "name" : "array13", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" } ], "returns" : { "type" : "xs:anyAtomicType", "description" : "the sum of products" }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">excel-err:Value if the parameters cannot be casted to numeric type</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 14, "name" : "sumproduct", "qname" : "excel:sumproduct", "signature" : "($array1 as xs:anyAtomicType*, $array2 as xs:anyAtomicType*, $array3 as xs:anyAtomicType*, $array4 as xs:anyAtomicType*, $array5 as xs:anyAtomicType*, $array6 as xs:anyAtomicType*, $array7 as xs:anyAtomicType*, $array8 as xs:anyAtomicType*, $array9 as xs:anyAtomicType*, $array10 as xs:anyAtomicType*, $array11 as xs:anyAtomicType*, $array12 as xs:anyAtomicType*, $array13 as xs:anyAtomicType*, $array14 as xs:anyAtomicType*) as xs:anyAtomicType", "description" : " Multiplies the elements on the same position in each sequence\n and sums up the results.\n", "summary" : "<p> Multiplies the elements on the same position in each sequence\n and sums up the results.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "array1", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" }, { "name" : "array2", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" }, { "name" : "array3", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" }, { "name" : "array4", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" }, { "name" : "array5", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" }, { "name" : "array6", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" }, { "name" : "array7", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" }, { "name" : "array8", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" }, { "name" : "array9", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" }, { "name" : "array10", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" }, { "name" : "array11", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" }, { "name" : "array12", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" }, { "name" : "array13", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" }, { "name" : "array14", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" } ], "returns" : { "type" : "xs:anyAtomicType", "description" : "the sum of products" }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">excel-err:Value if the parameters cannot be casted to numeric type</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 15, "name" : "sumproduct", "qname" : "excel:sumproduct", "signature" : "($array1 as xs:anyAtomicType*, $array2 as xs:anyAtomicType*, $array3 as xs:anyAtomicType*, $array4 as xs:anyAtomicType*, $array5 as xs:anyAtomicType*, $array6 as xs:anyAtomicType*, $array7 as xs:anyAtomicType*, $array8 as xs:anyAtomicType*, $array9 as xs:anyAtomicType*, $array10 as xs:anyAtomicType*, $array11 as xs:anyAtomicType*, $array12 as xs:anyAtomicType*, $array13 as xs:anyAtomicType*, $array14 as xs:anyAtomicType*, $array15 as xs:anyAtomicType*) as xs:anyAtomicType", "description" : " Multiplies the elements on the same position in each sequence\n and sums up the results.\n", "summary" : "<p> Multiplies the elements on the same position in each sequence\n and sums up the results.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "array1", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" }, { "name" : "array2", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" }, { "name" : "array3", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" }, { "name" : "array4", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" }, { "name" : "array5", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" }, { "name" : "array6", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" }, { "name" : "array7", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" }, { "name" : "array8", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" }, { "name" : "array9", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" }, { "name" : "array10", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" }, { "name" : "array11", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" }, { "name" : "array12", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" }, { "name" : "array13", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" }, { "name" : "array14", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" }, { "name" : "array15", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" } ], "returns" : { "type" : "xs:anyAtomicType", "description" : "the sum of products" }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">excel-err:Value if the parameters cannot be casted to numeric type</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 16, "name" : "sumproduct", "qname" : "excel:sumproduct", "signature" : "($array1 as xs:anyAtomicType*, $array2 as xs:anyAtomicType*, $array3 as xs:anyAtomicType*, $array4 as xs:anyAtomicType*, $array5 as xs:anyAtomicType*, $array6 as xs:anyAtomicType*, $array7 as xs:anyAtomicType*, $array8 as xs:anyAtomicType*, $array9 as xs:anyAtomicType*, $array10 as xs:anyAtomicType*, $array11 as xs:anyAtomicType*, $array12 as xs:anyAtomicType*, $array13 as xs:anyAtomicType*, $array14 as xs:anyAtomicType*, $array15 as xs:anyAtomicType*, $array16 as xs:anyAtomicType*) as xs:anyAtomicType", "description" : " Multiplies the elements on the same position in each sequence\n and sums up the results.\n", "summary" : "<p> Multiplies the elements on the same position in each sequence\n and sums up the results.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "array1", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" }, { "name" : "array2", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" }, { "name" : "array3", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" }, { "name" : "array4", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" }, { "name" : "array5", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" }, { "name" : "array6", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" }, { "name" : "array7", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" }, { "name" : "array8", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" }, { "name" : "array9", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" }, { "name" : "array10", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" }, { "name" : "array11", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" }, { "name" : "array12", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" }, { "name" : "array13", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" }, { "name" : "array14", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" }, { "name" : "array15", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" }, { "name" : "array16", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" } ], "returns" : { "type" : "xs:anyAtomicType", "description" : "the sum of products" }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">excel-err:Value if the parameters cannot be casted to numeric type</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 17, "name" : "sumproduct", "qname" : "excel:sumproduct", "signature" : "($array1 as xs:anyAtomicType*, $array2 as xs:anyAtomicType*, $array3 as xs:anyAtomicType*, $array4 as xs:anyAtomicType*, $array5 as xs:anyAtomicType*, $array6 as xs:anyAtomicType*, $array7 as xs:anyAtomicType*, $array8 as xs:anyAtomicType*, $array9 as xs:anyAtomicType*, $array10 as xs:anyAtomicType*, $array11 as xs:anyAtomicType*, $array12 as xs:anyAtomicType*, $array13 as xs:anyAtomicType*, $array14 as xs:anyAtomicType*, $array15 as xs:anyAtomicType*, $array16 as xs:anyAtomicType*, $array17 as xs:anyAtomicType*) as xs:anyAtomicType", "description" : " Multiplies the elements on the same position in each sequence\n and sums up the results.\n", "summary" : "<p> Multiplies the elements on the same position in each sequence\n and sums up the results.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "array1", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" }, { "name" : "array2", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" }, { "name" : "array3", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" }, { "name" : "array4", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" }, { "name" : "array5", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" }, { "name" : "array6", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" }, { "name" : "array7", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" }, { "name" : "array8", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" }, { "name" : "array9", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" }, { "name" : "array10", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" }, { "name" : "array11", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" }, { "name" : "array12", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" }, { "name" : "array13", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" }, { "name" : "array14", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" }, { "name" : "array15", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" }, { "name" : "array16", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" }, { "name" : "array17", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" } ], "returns" : { "type" : "xs:anyAtomicType", "description" : "the sum of products" }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">excel-err:Value if the parameters cannot be casted to numeric type</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 18, "name" : "sumproduct", "qname" : "excel:sumproduct", "signature" : "($array1 as xs:anyAtomicType*, $array2 as xs:anyAtomicType*, $array3 as xs:anyAtomicType*, $array4 as xs:anyAtomicType*, $array5 as xs:anyAtomicType*, $array6 as xs:anyAtomicType*, $array7 as xs:anyAtomicType*, $array8 as xs:anyAtomicType*, $array9 as xs:anyAtomicType*, $array10 as xs:anyAtomicType*, $array11 as xs:anyAtomicType*, $array12 as xs:anyAtomicType*, $array13 as xs:anyAtomicType*, $array14 as xs:anyAtomicType*, $array15 as xs:anyAtomicType*, $array16 as xs:anyAtomicType*, $array17 as xs:anyAtomicType*, $array18 as xs:anyAtomicType*) as xs:anyAtomicType", "description" : " Multiplies the elements on the same position in each sequence\n and sums up the results.\n", "summary" : "<p> Multiplies the elements on the same position in each sequence\n and sums up the results.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "array1", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" }, { "name" : "array2", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" }, { "name" : "array3", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" }, { "name" : "array4", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" }, { "name" : "array5", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" }, { "name" : "array6", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" }, { "name" : "array7", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" }, { "name" : "array8", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" }, { "name" : "array9", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" }, { "name" : "array10", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" }, { "name" : "array11", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" }, { "name" : "array12", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" }, { "name" : "array13", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" }, { "name" : "array14", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" }, { "name" : "array15", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" }, { "name" : "array16", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" }, { "name" : "array17", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" }, { "name" : "array18", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" } ], "returns" : { "type" : "xs:anyAtomicType", "description" : "the sum of products" }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">excel-err:Value if the parameters cannot be casted to numeric type</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 19, "name" : "sumproduct", "qname" : "excel:sumproduct", "signature" : "($array1 as xs:anyAtomicType*, $array2 as xs:anyAtomicType*, $array3 as xs:anyAtomicType*, $array4 as xs:anyAtomicType*, $array5 as xs:anyAtomicType*, $array6 as xs:anyAtomicType*, $array7 as xs:anyAtomicType*, $array8 as xs:anyAtomicType*, $array9 as xs:anyAtomicType*, $array10 as xs:anyAtomicType*, $array11 as xs:anyAtomicType*, $array12 as xs:anyAtomicType*, $array13 as xs:anyAtomicType*, $array14 as xs:anyAtomicType*, $array15 as xs:anyAtomicType*, $array16 as xs:anyAtomicType*, $array17 as xs:anyAtomicType*, $array18 as xs:anyAtomicType*, $array19 as xs:anyAtomicType*) as xs:anyAtomicType", "description" : " Multiplies the elements on the same position in each sequence\n and sums up the results.\n", "summary" : "<p> Multiplies the elements on the same position in each sequence\n and sums up the results.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "array1", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" }, { "name" : "array2", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" }, { "name" : "array3", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" }, { "name" : "array4", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" }, { "name" : "array5", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" }, { "name" : "array6", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" }, { "name" : "array7", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" }, { "name" : "array8", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" }, { "name" : "array9", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" }, { "name" : "array10", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" }, { "name" : "array11", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" }, { "name" : "array12", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" }, { "name" : "array13", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" }, { "name" : "array14", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" }, { "name" : "array15", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" }, { "name" : "array16", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" }, { "name" : "array17", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" }, { "name" : "array18", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" }, { "name" : "array19", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" } ], "returns" : { "type" : "xs:anyAtomicType", "description" : "the sum of products" }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">excel-err:Value if the parameters cannot be casted to numeric type</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 2, "name" : "sumproduct", "qname" : "excel:sumproduct", "signature" : "($array1 as xs:anyAtomicType*, $array2 as xs:anyAtomicType*) as xs:anyAtomicType", "description" : " Multiplies the elements on the same position in each sequence\n and sums up the results.\n", "summary" : "<p> Multiplies the elements on the same position in each sequence\n and sums up the results.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "array1", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" }, { "name" : "array2", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" } ], "returns" : { "type" : "xs:anyAtomicType", "description" : "the sum of products" }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">excel-err:Value if the parameters cannot be casted to numeric type</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 20, "name" : "sumproduct", "qname" : "excel:sumproduct", "signature" : "($array1 as xs:anyAtomicType*, $array2 as xs:anyAtomicType*, $array3 as xs:anyAtomicType*, $array4 as xs:anyAtomicType*, $array5 as xs:anyAtomicType*, $array6 as xs:anyAtomicType*, $array7 as xs:anyAtomicType*, $array8 as xs:anyAtomicType*, $array9 as xs:anyAtomicType*, $array10 as xs:anyAtomicType*, $array11 as xs:anyAtomicType*, $array12 as xs:anyAtomicType*, $array13 as xs:anyAtomicType*, $array14 as xs:anyAtomicType*, $array15 as xs:anyAtomicType*, $array16 as xs:anyAtomicType*, $array17 as xs:anyAtomicType*, $array18 as xs:anyAtomicType*, $array19 as xs:anyAtomicType*, $array20 as xs:anyAtomicType*) as xs:anyAtomicType", "description" : " Multiplies the elements on the same position in each sequence\n and sums up the results.\n", "summary" : "<p> Multiplies the elements on the same position in each sequence\n and sums up the results.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "array1", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" }, { "name" : "array2", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" }, { "name" : "array3", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" }, { "name" : "array4", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" }, { "name" : "array5", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" }, { "name" : "array6", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" }, { "name" : "array7", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" }, { "name" : "array8", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" }, { "name" : "array9", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" }, { "name" : "array10", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" }, { "name" : "array11", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" }, { "name" : "array12", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" }, { "name" : "array13", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" }, { "name" : "array14", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" }, { "name" : "array15", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" }, { "name" : "array16", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" }, { "name" : "array17", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" }, { "name" : "array18", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" }, { "name" : "array19", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" }, { "name" : "array20", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" } ], "returns" : { "type" : "xs:anyAtomicType", "description" : "the sum of products" }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">excel-err:Value if the parameters cannot be casted to numeric type</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 21, "name" : "sumproduct", "qname" : "excel:sumproduct", "signature" : "($array1 as xs:anyAtomicType*, $array2 as xs:anyAtomicType*, $array3 as xs:anyAtomicType*, $array4 as xs:anyAtomicType*, $array5 as xs:anyAtomicType*, $array6 as xs:anyAtomicType*, $array7 as xs:anyAtomicType*, $array8 as xs:anyAtomicType*, $array9 as xs:anyAtomicType*, $array10 as xs:anyAtomicType*, $array11 as xs:anyAtomicType*, $array12 as xs:anyAtomicType*, $array13 as xs:anyAtomicType*, $array14 as xs:anyAtomicType*, $array15 as xs:anyAtomicType*, $array16 as xs:anyAtomicType*, $array17 as xs:anyAtomicType*, $array18 as xs:anyAtomicType*, $array19 as xs:anyAtomicType*, $array20 as xs:anyAtomicType*, $array21 as xs:anyAtomicType*) as xs:anyAtomicType", "description" : " Multiplies the elements on the same position in each sequence\n and sums up the results.\n", "summary" : "<p> Multiplies the elements on the same position in each sequence\n and sums up the results.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "array1", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" }, { "name" : "array2", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" }, { "name" : "array3", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" }, { "name" : "array4", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" }, { "name" : "array5", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" }, { "name" : "array6", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" }, { "name" : "array7", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" }, { "name" : "array8", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" }, { "name" : "array9", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" }, { "name" : "array10", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" }, { "name" : "array11", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" }, { "name" : "array12", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" }, { "name" : "array13", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" }, { "name" : "array14", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" }, { "name" : "array15", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" }, { "name" : "array16", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" }, { "name" : "array17", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" }, { "name" : "array18", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" }, { "name" : "array19", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" }, { "name" : "array20", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" }, { "name" : "array21", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" } ], "returns" : { "type" : "xs:anyAtomicType", "description" : "the sum of products" }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">excel-err:Value if the parameters cannot be casted to numeric type</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 22, "name" : "sumproduct", "qname" : "excel:sumproduct", "signature" : "($array1 as xs:anyAtomicType*, $array2 as xs:anyAtomicType*, $array3 as xs:anyAtomicType*, $array4 as xs:anyAtomicType*, $array5 as xs:anyAtomicType*, $array6 as xs:anyAtomicType*, $array7 as xs:anyAtomicType*, $array8 as xs:anyAtomicType*, $array9 as xs:anyAtomicType*, $array10 as xs:anyAtomicType*, $array11 as xs:anyAtomicType*, $array12 as xs:anyAtomicType*, $array13 as xs:anyAtomicType*, $array14 as xs:anyAtomicType*, $array15 as xs:anyAtomicType*, $array16 as xs:anyAtomicType*, $array17 as xs:anyAtomicType*, $array18 as xs:anyAtomicType*, $array19 as xs:anyAtomicType*, $array20 as xs:anyAtomicType*, $array21 as xs:anyAtomicType*, $array22 as xs:anyAtomicType*) as xs:anyAtomicType", "description" : " Multiplies the elements on the same position in each sequence\n and sums up the results.\n", "summary" : "<p> Multiplies the elements on the same position in each sequence\n and sums up the results.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "array1", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" }, { "name" : "array2", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" }, { "name" : "array3", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" }, { "name" : "array4", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" }, { "name" : "array5", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" }, { "name" : "array6", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" }, { "name" : "array7", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" }, { "name" : "array8", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" }, { "name" : "array9", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" }, { "name" : "array10", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" }, { "name" : "array11", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" }, { "name" : "array12", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" }, { "name" : "array13", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" }, { "name" : "array14", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" }, { "name" : "array15", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" }, { "name" : "array16", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" }, { "name" : "array17", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" }, { "name" : "array18", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" }, { "name" : "array19", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" }, { "name" : "array20", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" }, { "name" : "array21", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" }, { "name" : "array22", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" } ], "returns" : { "type" : "xs:anyAtomicType", "description" : "the sum of products" }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">excel-err:Value if the parameters cannot be casted to numeric type</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 23, "name" : "sumproduct", "qname" : "excel:sumproduct", "signature" : "($array1 as xs:anyAtomicType*, $array2 as xs:anyAtomicType*, $array3 as xs:anyAtomicType*, $array4 as xs:anyAtomicType*, $array5 as xs:anyAtomicType*, $array6 as xs:anyAtomicType*, $array7 as xs:anyAtomicType*, $array8 as xs:anyAtomicType*, $array9 as xs:anyAtomicType*, $array10 as xs:anyAtomicType*, $array11 as xs:anyAtomicType*, $array12 as xs:anyAtomicType*, $array13 as xs:anyAtomicType*, $array14 as xs:anyAtomicType*, $array15 as xs:anyAtomicType*, $array16 as xs:anyAtomicType*, $array17 as xs:anyAtomicType*, $array18 as xs:anyAtomicType*, $array19 as xs:anyAtomicType*, $array20 as xs:anyAtomicType*, $array21 as xs:anyAtomicType*, $array22 as xs:anyAtomicType*, $array23 as xs:anyAtomicType*) as xs:anyAtomicType", "description" : " Multiplies the elements on the same position in each sequence\n and sums up the results.\n", "summary" : "<p> Multiplies the elements on the same position in each sequence\n and sums up the results.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "array1", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" }, { "name" : "array2", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" }, { "name" : "array3", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" }, { "name" : "array4", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" }, { "name" : "array5", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" }, { "name" : "array6", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" }, { "name" : "array7", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" }, { "name" : "array8", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" }, { "name" : "array9", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" }, { "name" : "array10", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" }, { "name" : "array11", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" }, { "name" : "array12", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" }, { "name" : "array13", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" }, { "name" : "array14", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" }, { "name" : "array15", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" }, { "name" : "array16", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" }, { "name" : "array17", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" }, { "name" : "array18", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" }, { "name" : "array19", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" }, { "name" : "array20", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" }, { "name" : "array21", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" }, { "name" : "array22", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" }, { "name" : "array23", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" } ], "returns" : { "type" : "xs:anyAtomicType", "description" : "the sum of products" }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">excel-err:Value if the parameters cannot be casted to numeric type</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 24, "name" : "sumproduct", "qname" : "excel:sumproduct", "signature" : "($array1 as xs:anyAtomicType*, $array2 as xs:anyAtomicType*, $array3 as xs:anyAtomicType*, $array4 as xs:anyAtomicType*, $array5 as xs:anyAtomicType*, $array6 as xs:anyAtomicType*, $array7 as xs:anyAtomicType*, $array8 as xs:anyAtomicType*, $array9 as xs:anyAtomicType*, $array10 as xs:anyAtomicType*, $array11 as xs:anyAtomicType*, $array12 as xs:anyAtomicType*, $array13 as xs:anyAtomicType*, $array14 as xs:anyAtomicType*, $array15 as xs:anyAtomicType*, $array16 as xs:anyAtomicType*, $array17 as xs:anyAtomicType*, $array18 as xs:anyAtomicType*, $array19 as xs:anyAtomicType*, $array20 as xs:anyAtomicType*, $array21 as xs:anyAtomicType*, $array22 as xs:anyAtomicType*, $array23 as xs:anyAtomicType*, $array24 as xs:anyAtomicType*) as xs:anyAtomicType", "description" : " Multiplies the elements on the same position in each sequence\n and sums up the results.\n", "summary" : "<p> Multiplies the elements on the same position in each sequence\n and sums up the results.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "array1", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" }, { "name" : "array2", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" }, { "name" : "array3", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" }, { "name" : "array4", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" }, { "name" : "array5", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" }, { "name" : "array6", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" }, { "name" : "array7", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" }, { "name" : "array8", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" }, { "name" : "array9", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" }, { "name" : "array10", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" }, { "name" : "array11", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" }, { "name" : "array12", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" }, { "name" : "array13", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" }, { "name" : "array14", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" }, { "name" : "array15", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" }, { "name" : "array16", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" }, { "name" : "array17", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" }, { "name" : "array18", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" }, { "name" : "array19", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" }, { "name" : "array20", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" }, { "name" : "array21", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" }, { "name" : "array22", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" }, { "name" : "array23", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" }, { "name" : "array24", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" } ], "returns" : { "type" : "xs:anyAtomicType", "description" : "the sum of products" }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">excel-err:Value if the parameters cannot be casted to numeric type</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 25, "name" : "sumproduct", "qname" : "excel:sumproduct", "signature" : "($array1 as xs:anyAtomicType*, $array2 as xs:anyAtomicType*, $array3 as xs:anyAtomicType*, $array4 as xs:anyAtomicType*, $array5 as xs:anyAtomicType*, $array6 as xs:anyAtomicType*, $array7 as xs:anyAtomicType*, $array8 as xs:anyAtomicType*, $array9 as xs:anyAtomicType*, $array10 as xs:anyAtomicType*, $array11 as xs:anyAtomicType*, $array12 as xs:anyAtomicType*, $array13 as xs:anyAtomicType*, $array14 as xs:anyAtomicType*, $array15 as xs:anyAtomicType*, $array16 as xs:anyAtomicType*, $array17 as xs:anyAtomicType*, $array18 as xs:anyAtomicType*, $array19 as xs:anyAtomicType*, $array20 as xs:anyAtomicType*, $array21 as xs:anyAtomicType*, $array22 as xs:anyAtomicType*, $array23 as xs:anyAtomicType*, $array24 as xs:anyAtomicType*, $array25 as xs:anyAtomicType*) as xs:anyAtomicType", "description" : " Multiplies the elements on the same position in each sequence\n and sums up the results.\n", "summary" : "<p> Multiplies the elements on the same position in each sequence\n and sums up the results.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "array1", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" }, { "name" : "array2", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" }, { "name" : "array3", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" }, { "name" : "array4", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" }, { "name" : "array5", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" }, { "name" : "array6", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" }, { "name" : "array7", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" }, { "name" : "array8", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" }, { "name" : "array9", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" }, { "name" : "array10", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" }, { "name" : "array11", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" }, { "name" : "array12", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" }, { "name" : "array13", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" }, { "name" : "array14", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" }, { "name" : "array15", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" }, { "name" : "array16", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" }, { "name" : "array17", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" }, { "name" : "array18", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" }, { "name" : "array19", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" }, { "name" : "array20", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" }, { "name" : "array21", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" }, { "name" : "array22", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" }, { "name" : "array23", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" }, { "name" : "array24", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" }, { "name" : "array25", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" } ], "returns" : { "type" : "xs:anyAtomicType", "description" : "the sum of products" }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">excel-err:Value if the parameters cannot be casted to numeric type</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 26, "name" : "sumproduct", "qname" : "excel:sumproduct", "signature" : "($array1 as xs:anyAtomicType*, $array2 as xs:anyAtomicType*, $array3 as xs:anyAtomicType*, $array4 as xs:anyAtomicType*, $array5 as xs:anyAtomicType*, $array6 as xs:anyAtomicType*, $array7 as xs:anyAtomicType*, $array8 as xs:anyAtomicType*, $array9 as xs:anyAtomicType*, $array10 as xs:anyAtomicType*, $array11 as xs:anyAtomicType*, $array12 as xs:anyAtomicType*, $array13 as xs:anyAtomicType*, $array14 as xs:anyAtomicType*, $array15 as xs:anyAtomicType*, $array16 as xs:anyAtomicType*, $array17 as xs:anyAtomicType*, $array18 as xs:anyAtomicType*, $array19 as xs:anyAtomicType*, $array20 as xs:anyAtomicType*, $array21 as xs:anyAtomicType*, $array22 as xs:anyAtomicType*, $array23 as xs:anyAtomicType*, $array24 as xs:anyAtomicType*, $array25 as xs:anyAtomicType*, $array26 as xs:anyAtomicType*) as xs:anyAtomicType", "description" : " Multiplies the elements on the same position in each sequence\n and sums up the results.\n", "summary" : "<p> Multiplies the elements on the same position in each sequence\n and sums up the results.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "array1", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" }, { "name" : "array2", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" }, { "name" : "array3", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" }, { "name" : "array4", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" }, { "name" : "array5", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" }, { "name" : "array6", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" }, { "name" : "array7", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" }, { "name" : "array8", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" }, { "name" : "array9", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" }, { "name" : "array10", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" }, { "name" : "array11", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" }, { "name" : "array12", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" }, { "name" : "array13", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" }, { "name" : "array14", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" }, { "name" : "array15", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" }, { "name" : "array16", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" }, { "name" : "array17", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" }, { "name" : "array18", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" }, { "name" : "array19", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" }, { "name" : "array20", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" }, { "name" : "array21", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" }, { "name" : "array22", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" }, { "name" : "array23", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" }, { "name" : "array24", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" }, { "name" : "array25", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" }, { "name" : "array26", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" } ], "returns" : { "type" : "xs:anyAtomicType", "description" : "the sum of products" }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">excel-err:Value if the parameters cannot be casted to numeric type</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 27, "name" : "sumproduct", "qname" : "excel:sumproduct", "signature" : "($array1 as xs:anyAtomicType*, $array2 as xs:anyAtomicType*, $array3 as xs:anyAtomicType*, $array4 as xs:anyAtomicType*, $array5 as xs:anyAtomicType*, $array6 as xs:anyAtomicType*, $array7 as xs:anyAtomicType*, $array8 as xs:anyAtomicType*, $array9 as xs:anyAtomicType*, $array10 as xs:anyAtomicType*, $array11 as xs:anyAtomicType*, $array12 as xs:anyAtomicType*, $array13 as xs:anyAtomicType*, $array14 as xs:anyAtomicType*, $array15 as xs:anyAtomicType*, $array16 as xs:anyAtomicType*, $array17 as xs:anyAtomicType*, $array18 as xs:anyAtomicType*, $array19 as xs:anyAtomicType*, $array20 as xs:anyAtomicType*, $array21 as xs:anyAtomicType*, $array22 as xs:anyAtomicType*, $array23 as xs:anyAtomicType*, $array24 as xs:anyAtomicType*, $array25 as xs:anyAtomicType*, $array26 as xs:anyAtomicType*, $array27 as xs:anyAtomicType*) as xs:anyAtomicType", "description" : " Multiplies the elements on the same position in each sequence\n and sums up the results.\n", "summary" : "<p> Multiplies the elements on the same position in each sequence\n and sums up the results.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "array1", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" }, { "name" : "array2", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" }, { "name" : "array3", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" }, { "name" : "array4", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" }, { "name" : "array5", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" }, { "name" : "array6", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" }, { "name" : "array7", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" }, { "name" : "array8", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" }, { "name" : "array9", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" }, { "name" : "array10", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" }, { "name" : "array11", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" }, { "name" : "array12", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" }, { "name" : "array13", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" }, { "name" : "array14", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" }, { "name" : "array15", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" }, { "name" : "array16", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" }, { "name" : "array17", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" }, { "name" : "array18", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" }, { "name" : "array19", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" }, { "name" : "array20", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" }, { "name" : "array21", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" }, { "name" : "array22", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" }, { "name" : "array23", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" }, { "name" : "array24", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" }, { "name" : "array25", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" }, { "name" : "array26", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" }, { "name" : "array27", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" } ], "returns" : { "type" : "xs:anyAtomicType", "description" : "the sum of products" }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">excel-err:Value if the parameters cannot be casted to numeric type</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 28, "name" : "sumproduct", "qname" : "excel:sumproduct", "signature" : "($array1 as xs:anyAtomicType*, $array2 as xs:anyAtomicType*, $array3 as xs:anyAtomicType*, $array4 as xs:anyAtomicType*, $array5 as xs:anyAtomicType*, $array6 as xs:anyAtomicType*, $array7 as xs:anyAtomicType*, $array8 as xs:anyAtomicType*, $array9 as xs:anyAtomicType*, $array10 as xs:anyAtomicType*, $array11 as xs:anyAtomicType*, $array12 as xs:anyAtomicType*, $array13 as xs:anyAtomicType*, $array14 as xs:anyAtomicType*, $array15 as xs:anyAtomicType*, $array16 as xs:anyAtomicType*, $array17 as xs:anyAtomicType*, $array18 as xs:anyAtomicType*, $array19 as xs:anyAtomicType*, $array20 as xs:anyAtomicType*, $array21 as xs:anyAtomicType*, $array22 as xs:anyAtomicType*, $array23 as xs:anyAtomicType*, $array24 as xs:anyAtomicType*, $array25 as xs:anyAtomicType*, $array26 as xs:anyAtomicType*, $array27 as xs:anyAtomicType*, $array28 as xs:anyAtomicType*) as xs:anyAtomicType", "description" : " Multiplies the elements on the same position in each sequence\n and sums up the results.\n", "summary" : "<p> Multiplies the elements on the same position in each sequence\n and sums up the results.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "array1", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" }, { "name" : "array2", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" }, { "name" : "array3", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" }, { "name" : "array4", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" }, { "name" : "array5", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" }, { "name" : "array6", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" }, { "name" : "array7", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" }, { "name" : "array8", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" }, { "name" : "array9", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" }, { "name" : "array10", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" }, { "name" : "array11", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" }, { "name" : "array12", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" }, { "name" : "array13", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" }, { "name" : "array14", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" }, { "name" : "array15", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" }, { "name" : "array16", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" }, { "name" : "array17", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" }, { "name" : "array18", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" }, { "name" : "array19", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" }, { "name" : "array20", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" }, { "name" : "array21", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" }, { "name" : "array22", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" }, { "name" : "array23", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" }, { "name" : "array24", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" }, { "name" : "array25", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" }, { "name" : "array26", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" }, { "name" : "array27", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" }, { "name" : "array28", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" } ], "returns" : { "type" : "xs:anyAtomicType", "description" : "the sum of products" }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">excel-err:Value if the parameters cannot be casted to numeric type</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 29, "name" : "sumproduct", "qname" : "excel:sumproduct", "signature" : "($array1 as xs:anyAtomicType*, $array2 as xs:anyAtomicType*, $array3 as xs:anyAtomicType*, $array4 as xs:anyAtomicType*, $array5 as xs:anyAtomicType*, $array6 as xs:anyAtomicType*, $array7 as xs:anyAtomicType*, $array8 as xs:anyAtomicType*, $array9 as xs:anyAtomicType*, $array10 as xs:anyAtomicType*, $array11 as xs:anyAtomicType*, $array12 as xs:anyAtomicType*, $array13 as xs:anyAtomicType*, $array14 as xs:anyAtomicType*, $array15 as xs:anyAtomicType*, $array16 as xs:anyAtomicType*, $array17 as xs:anyAtomicType*, $array18 as xs:anyAtomicType*, $array19 as xs:anyAtomicType*, $array20 as xs:anyAtomicType*, $array21 as xs:anyAtomicType*, $array22 as xs:anyAtomicType*, $array23 as xs:anyAtomicType*, $array24 as xs:anyAtomicType*, $array25 as xs:anyAtomicType*, $array26 as xs:anyAtomicType*, $array27 as xs:anyAtomicType*, $array28 as xs:anyAtomicType*, $array29 as xs:anyAtomicType*) as xs:anyAtomicType", "description" : " Multiplies the elements on the same position in each sequence\n and sums up the results.\n", "summary" : "<p> Multiplies the elements on the same position in each sequence\n and sums up the results.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "array1", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" }, { "name" : "array2", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" }, { "name" : "array3", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" }, { "name" : "array4", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" }, { "name" : "array5", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" }, { "name" : "array6", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" }, { "name" : "array7", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" }, { "name" : "array8", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" }, { "name" : "array9", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" }, { "name" : "array10", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" }, { "name" : "array11", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" }, { "name" : "array12", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" }, { "name" : "array13", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" }, { "name" : "array14", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" }, { "name" : "array15", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" }, { "name" : "array16", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" }, { "name" : "array17", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" }, { "name" : "array18", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" }, { "name" : "array19", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" }, { "name" : "array20", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" }, { "name" : "array21", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" }, { "name" : "array22", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" }, { "name" : "array23", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" }, { "name" : "array24", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" }, { "name" : "array25", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" }, { "name" : "array26", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" }, { "name" : "array27", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" }, { "name" : "array28", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" }, { "name" : "array29", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" } ], "returns" : { "type" : "xs:anyAtomicType", "description" : "the sum of products" }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">excel-err:Value if the parameters cannot be casted to numeric type</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 3, "name" : "sumproduct", "qname" : "excel:sumproduct", "signature" : "($array1 as xs:anyAtomicType*, $array2 as xs:anyAtomicType*, $array3 as xs:anyAtomicType*) as xs:anyAtomicType", "description" : " Multiplies the elements on the same position in each sequence\n and sums up the results.\n", "summary" : "<p> Multiplies the elements on the same position in each sequence\n and sums up the results.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "array1", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" }, { "name" : "array2", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" }, { "name" : "array3", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" } ], "returns" : { "type" : "xs:anyAtomicType", "description" : "the sum of products" }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">excel-err:Value if the parameters cannot be casted to numeric type</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 30, "name" : "sumproduct", "qname" : "excel:sumproduct", "signature" : "($array1 as xs:anyAtomicType*, $array2 as xs:anyAtomicType*, $array3 as xs:anyAtomicType*, $array4 as xs:anyAtomicType*, $array5 as xs:anyAtomicType*, $array6 as xs:anyAtomicType*, $array7 as xs:anyAtomicType*, $array8 as xs:anyAtomicType*, $array9 as xs:anyAtomicType*, $array10 as xs:anyAtomicType*, $array11 as xs:anyAtomicType*, $array12 as xs:anyAtomicType*, $array13 as xs:anyAtomicType*, $array14 as xs:anyAtomicType*, $array15 as xs:anyAtomicType*, $array16 as xs:anyAtomicType*, $array17 as xs:anyAtomicType*, $array18 as xs:anyAtomicType*, $array19 as xs:anyAtomicType*, $array20 as xs:anyAtomicType*, $array21 as xs:anyAtomicType*, $array22 as xs:anyAtomicType*, $array23 as xs:anyAtomicType*, $array24 as xs:anyAtomicType*, $array25 as xs:anyAtomicType*, $array26 as xs:anyAtomicType*, $array27 as xs:anyAtomicType*, $array28 as xs:anyAtomicType*, $array29 as xs:anyAtomicType*, $array30 as xs:anyAtomicType*) as xs:anyAtomicType", "description" : " Multiplies the elements on the same position in each sequence\n and sums up the results.\n", "summary" : "<p> Multiplies the elements on the same position in each sequence\n and sums up the results.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "array1", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" }, { "name" : "array2", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" }, { "name" : "array3", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" }, { "name" : "array4", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" }, { "name" : "array5", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" }, { "name" : "array6", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" }, { "name" : "array7", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" }, { "name" : "array8", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" }, { "name" : "array9", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" }, { "name" : "array10", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" }, { "name" : "array11", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" }, { "name" : "array12", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" }, { "name" : "array13", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" }, { "name" : "array14", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" }, { "name" : "array15", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" }, { "name" : "array16", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" }, { "name" : "array17", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" }, { "name" : "array18", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" }, { "name" : "array19", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" }, { "name" : "array20", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" }, { "name" : "array21", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" }, { "name" : "array22", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" }, { "name" : "array23", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" }, { "name" : "array24", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" }, { "name" : "array25", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" }, { "name" : "array26", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" }, { "name" : "array27", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" }, { "name" : "array28", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" }, { "name" : "array29", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" }, { "name" : "array30", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" } ], "returns" : { "type" : "xs:anyAtomicType", "description" : "the sum of products" }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">excel-err:Value if the parameters cannot be casted to numeric type</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 4, "name" : "sumproduct", "qname" : "excel:sumproduct", "signature" : "($array1 as xs:anyAtomicType*, $array2 as xs:anyAtomicType*, $array3 as xs:anyAtomicType*, $array4 as xs:anyAtomicType*) as xs:anyAtomicType", "description" : " Multiplies the elements on the same position in each sequence\n and sums up the results.\n", "summary" : "<p> Multiplies the elements on the same position in each sequence\n and sums up the results.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "array1", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" }, { "name" : "array2", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" }, { "name" : "array3", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" }, { "name" : "array4", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" } ], "returns" : { "type" : "xs:anyAtomicType", "description" : "the sum of products" }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">excel-err:Value if the parameters cannot be casted to numeric type</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 5, "name" : "sumproduct", "qname" : "excel:sumproduct", "signature" : "($array1 as xs:anyAtomicType*, $array2 as xs:anyAtomicType*, $array3 as xs:anyAtomicType*, $array4 as xs:anyAtomicType*, $array5 as xs:anyAtomicType*) as xs:anyAtomicType", "description" : " Multiplies the elements on the same position in each sequence\n and sums up the results.\n", "summary" : "<p> Multiplies the elements on the same position in each sequence\n and sums up the results.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "array1", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" }, { "name" : "array2", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" }, { "name" : "array3", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" }, { "name" : "array4", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" }, { "name" : "array5", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" } ], "returns" : { "type" : "xs:anyAtomicType", "description" : "the sum of products" }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">excel-err:Value if the parameters cannot be casted to numeric type</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 6, "name" : "sumproduct", "qname" : "excel:sumproduct", "signature" : "($array1 as xs:anyAtomicType*, $array2 as xs:anyAtomicType*, $array3 as xs:anyAtomicType*, $array4 as xs:anyAtomicType*, $array5 as xs:anyAtomicType*, $array6 as xs:anyAtomicType*) as xs:anyAtomicType", "description" : " Multiplies the elements on the same position in each sequence\n and sums up the results.\n", "summary" : "<p> Multiplies the elements on the same position in each sequence\n and sums up the results.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "array1", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" }, { "name" : "array2", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" }, { "name" : "array3", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" }, { "name" : "array4", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" }, { "name" : "array5", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" }, { "name" : "array6", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" } ], "returns" : { "type" : "xs:anyAtomicType", "description" : "the sum of products" }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">excel-err:Value if the parameters cannot be casted to numeric type</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 7, "name" : "sumproduct", "qname" : "excel:sumproduct", "signature" : "($array1 as xs:anyAtomicType*, $array2 as xs:anyAtomicType*, $array3 as xs:anyAtomicType*, $array4 as xs:anyAtomicType*, $array5 as xs:anyAtomicType*, $array6 as xs:anyAtomicType*, $array7 as xs:anyAtomicType*) as xs:anyAtomicType", "description" : " Multiplies the elements on the same position in each sequence\n and sums up the results.\n", "summary" : "<p> Multiplies the elements on the same position in each sequence\n and sums up the results.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "array1", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" }, { "name" : "array2", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" }, { "name" : "array3", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" }, { "name" : "array4", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" }, { "name" : "array5", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" }, { "name" : "array6", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" }, { "name" : "array7", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" } ], "returns" : { "type" : "xs:anyAtomicType", "description" : "the sum of products" }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">excel-err:Value if the parameters cannot be casted to numeric type</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 8, "name" : "sumproduct", "qname" : "excel:sumproduct", "signature" : "($array1 as xs:anyAtomicType*, $array2 as xs:anyAtomicType*, $array3 as xs:anyAtomicType*, $array4 as xs:anyAtomicType*, $array5 as xs:anyAtomicType*, $array6 as xs:anyAtomicType*, $array7 as xs:anyAtomicType*, $array8 as xs:anyAtomicType*) as xs:anyAtomicType", "description" : " Multiplies the elements on the same position in each sequence\n and sums up the results.\n", "summary" : "<p> Multiplies the elements on the same position in each sequence\n and sums up the results.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "array1", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" }, { "name" : "array2", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" }, { "name" : "array3", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" }, { "name" : "array4", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" }, { "name" : "array5", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" }, { "name" : "array6", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" }, { "name" : "array7", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" }, { "name" : "array8", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" } ], "returns" : { "type" : "xs:anyAtomicType", "description" : "the sum of products" }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">excel-err:Value if the parameters cannot be casted to numeric type</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 9, "name" : "sumproduct", "qname" : "excel:sumproduct", "signature" : "($array1 as xs:anyAtomicType*, $array2 as xs:anyAtomicType*, $array3 as xs:anyAtomicType*, $array4 as xs:anyAtomicType*, $array5 as xs:anyAtomicType*, $array6 as xs:anyAtomicType*, $array7 as xs:anyAtomicType*, $array8 as xs:anyAtomicType*, $array9 as xs:anyAtomicType*) as xs:anyAtomicType", "description" : " Multiplies the elements on the same position in each sequence\n and sums up the results.\n", "summary" : "<p> Multiplies the elements on the same position in each sequence\n and sums up the results.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "array1", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" }, { "name" : "array2", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" }, { "name" : "array3", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" }, { "name" : "array4", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" }, { "name" : "array5", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" }, { "name" : "array6", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" }, { "name" : "array7", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" }, { "name" : "array8", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" }, { "name" : "array9", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequences of numbers or arguments castable to numeric</div>" } ], "returns" : { "type" : "xs:anyAtomicType", "description" : "the sum of products" }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">excel-err:Value if the parameters cannot be casted to numeric type</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 1, "name" : "sumsq", "qname" : "excel:sumsq", "signature" : "($numbers as xs:anyAtomicType+) as xs:anyAtomicType", "description" : " Returns the sum of the squares of the arguments.\n It used the sumproduct function.\n", "summary" : "<p> Returns the sum of the squares of the arguments.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "numbers", "type" : "xs:anyAtomicType", "occurrence" : "+", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequence of one or more numbers or arguments castable to numeric</div>" } ], "returns" : { "type" : "xs:anyAtomicType", "description" : "the sum of squared values, as numeric type" }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">excel-err:Value if the parameters cannot be casted to numeric type</xqdoc:error>" ] } ], "variables" : [  ] }, "http://zorba.io/modules/item" : { "ns" : "http://zorba.io/modules/item", "description" : " This module provides utility functions on items. For example,\n it provides a function that allows estimating the size in bytes\n that a given item allocates in memory.\n", "sees" : [  ], "authors" : [ "<xqdoc:author xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Matthias Brantner</xqdoc:author>" ], "version" : null, "encoding" : "utf-8", "namespaces" : [ { "uri" : "http://zorba.io/modules/item", "prefix" : "item" }, { "uri" : "http://zorba.io/options/versioning", "prefix" : "ver" } ], "functions" : [ { "isDocumented" : true, "arity" : 1, "name" : "size", "qname" : "item:size", "signature" : "($item as item()) as xs:integer external", "description" : " Computes the size in bytes of the given item in main memory.<p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n", "summary" : "<p> Computes the size in bytes of the given item in main memory.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "item", "type" : "item()", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the item whose size to compute.</div>" } ], "returns" : { "type" : "xs:integer", "description" : "the size allocated by the item in bytes." }, "errors" : [  ] } ], "variables" : [  ] }, "http://www.zorba-xquery.com/modules/xqdoc/menu" : { "ns" : "http://www.zorba-xquery.com/modules/xqdoc/menu", "description" : " Generate navigation for XQDoc batches.\n", "sees" : [  ], "authors" : [ "<xqdoc:author xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">William Candillon</xqdoc:author>" ], "version" : null, "encoding" : "utf-8", "namespaces" : [ { "uri" : "http://www.zorba-xquery.com/modules/xqdoc/menu", "prefix" : "menu" }, { "uri" : "http://zorba.io/options/versioning", "prefix" : "ver" } ], "functions" : [ { "isDocumented" : false, "arity" : 2, "name" : "categories-as-js", "qname" : "menu:categories-as-js", "signature" : "($url-prefix as xs:string, $cats as element(*)) as xs:string", "description" : "", "summary" : "", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "url-prefix", "type" : "xs:string", "occurrence" : null, "description" : "" }, { "name" : "cats", "type" : "element(*)", "occurrence" : null, "description" : "" } ], "returns" : { "type" : "xs:string", "description" : "" }, "errors" : [  ] }, { "isDocumented" : false, "arity" : 1, "name" : "categories", "qname" : "menu:categories", "signature" : "($top as element(*)) as xs:string*", "description" : "", "summary" : "", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "top", "type" : "element(*)", "occurrence" : null, "description" : "" } ], "returns" : { "type" : "xs:string*", "description" : "" }, "errors" : [  ] }, { "isDocumented" : false, "arity" : 2, "name" : "closed-tabs", "qname" : "menu:closed-tabs", "signature" : "($item as element(section), $url-prefix as xs:string) as element(li)*", "description" : "", "summary" : "", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "item", "type" : "element(section)", "occurrence" : null, "description" : "" }, { "name" : "url-prefix", "type" : "xs:string", "occurrence" : null, "description" : "" } ], "returns" : { "type" : "element(li)*", "description" : "" }, "errors" : [  ] }, { "isDocumented" : false, "arity" : 3, "name" : "closed-tabs", "qname" : "menu:closed-tabs", "signature" : "($item as element(*)?, $result as element(*)*, $url-prefix as xs:string) as element(li)*", "description" : "", "summary" : "", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "item", "type" : "element(*)", "occurrence" : "?", "description" : "" }, { "name" : "result", "type" : "element(*)", "occurrence" : "*", "description" : "" }, { "name" : "url-prefix", "type" : "xs:string", "occurrence" : null, "description" : "" } ], "returns" : { "type" : "element(li)*", "description" : "" }, "errors" : [  ] }, { "isDocumented" : false, "arity" : 1, "name" : "item-uri", "qname" : "menu:item-uri", "signature" : "($item as element(*)) as xs:string", "description" : "", "summary" : "", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "item", "type" : "element(*)", "occurrence" : null, "description" : "" } ], "returns" : { "type" : "xs:string", "description" : "" }, "errors" : [  ] }, { "isDocumented" : false, "arity" : 2, "name" : "item-uri", "qname" : "menu:item-uri", "signature" : "($item as element(*)?, $result as xs:string*) as xs:string+", "description" : "", "summary" : "", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "item", "type" : "element(*)", "occurrence" : "?", "description" : "" }, { "name" : "result", "type" : "xs:string", "occurrence" : "*", "description" : "" } ], "returns" : { "type" : "xs:string+", "description" : "" }, "errors" : [  ] }, { "isDocumented" : false, "arity" : 2, "name" : "item", "qname" : "menu:item", "signature" : "($ctx as element(*)?, $path as xs:string*) as element(*)?", "description" : "", "summary" : "", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "ctx", "type" : "element(*)", "occurrence" : "?", "description" : "" }, { "name" : "path", "type" : "xs:string", "occurrence" : "*", "description" : "" } ], "returns" : { "type" : "element(*)?", "description" : "" }, "errors" : [  ] }, { "isDocumented" : false, "arity" : 2, "name" : "menu", "qname" : "menu:menu", "signature" : "($item as element(*), $url-prefix as xs:string)", "description" : "", "summary" : "", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "item", "type" : "element(*)", "occurrence" : null, "description" : "" }, { "name" : "url-prefix", "type" : "xs:string", "occurrence" : null, "description" : "" } ], "returns" : { "type" : null, "description" : "" }, "errors" : [  ] } ], "variables" : [ { "name" : "menu:not-found", "type" : "item()*", "description" : "" } ] }, "http://zorba.io/modules/dbgp-message-handler" : { "ns" : "http://zorba.io/modules/dbgp-message-handler", "description" : " Zorba debugger module.\n", "sees" : [  ], "authors" : [ "<xqdoc:author xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Gabriel Petrovay</xqdoc:author>" ], "version" : null, "encoding" : "utf-8", "namespaces" : [ { "uri" : "http://zorba.io/modules/base64", "prefix" : "base64" }, { "uri" : "http://zorba.io/modules/dbgp-message-handler", "prefix" : "dmh" }, { "uri" : "http://zorba.io/options/versioning", "prefix" : "ver" } ], "functions" : [ { "isDocumented" : true, "arity" : 1, "name" : "process", "qname" : "dmh:process", "signature" : "($message as element(*)) as xs:anyAtomicType*", "description" : " Process one message received from the Zorba debugger server.\n", "summary" : "<p> Process one message received from the Zorba debugger server.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "message", "type" : "element(*)", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the message.</div>" } ], "returns" : { "type" : "xs:anyAtomicType*", "description" : "()." }, "errors" : [  ] } ], "variables" : [ { "name" : "dmh:debug", "type" : "xs:boolean", "description" : " Set this variale to true if you want to have mode debug information when\n an error occurs.\n" } ] }, "http://www.28msec.com/modules/ws/mailchimp" : { "ns" : "http://www.28msec.com/modules/ws/mailchimp", "description" : " Mailchimp Client Module.\n", "sees" : [ "<xqdoc:see xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"><a href=\"http://apidocs.mailchimp.com\" target=\"_blank\">http://apidocs.mailchimp.com</a></xqdoc:see>" ], "authors" : [ "<xqdoc:author xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">William Candillon {william.candillon@28msec.com}</xqdoc:author>" ], "version" : null, "encoding" : "utf-8", "namespaces" : [ { "uri" : "http://zorba.io/annotations", "prefix" : "an" }, { "uri" : "http://expath.org/ns/http-client", "prefix" : "http-client" }, { "uri" : "http://www.28msec.com/modules/ws/mailchimp", "prefix" : "mailchimp" }, { "uri" : "http://zorba.io/options/versioning", "prefix" : "ver" }, { "uri" : "http://www.28msec.com/modules/xmlrpc", "prefix" : "xmlrpc" } ], "functions" : [ { "isDocumented" : true, "arity" : 3, "name" : "apikey-add", "qname" : "mailchimp:apikey-add", "signature" : "($username as xs:string, $password as xs:string, $apikey as xs:string) as item()*", "description" : " Convenience function for <a xmlns:xqdoc=\"http://www.xqdoc.org/1.0\" href=\"#apikey-add-4\">apikey-add#4</a>.\n", "summary" : "<p> Convenience function for  apikey-add#4 .</p>", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "username", "type" : "xs:string", "occurrence" : null, "description" : "" }, { "name" : "password", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> Your MailChimp password</div>" }, { "name" : "apikey", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> Any valid API Key</div>" } ], "returns" : { "type" : "item()*", "description" : "A new API Key that can be immediately used." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 4, "name" : "apikey-add", "qname" : "mailchimp:apikey-add", "signature" : "($endpoint-url as xs:string, $username as xs:string, $password as xs:string, $apikey as xs:string) as item()*", "description" : " Add an API Key to your account. We will generate a new key for you and return it.\n", "summary" : "<p> Add an API Key to your account.</p>", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "endpoint-url", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> Mailchimp endpoint URL, e.g. http://us1.api.mailchimp.com/1.3/</div>" }, { "name" : "username", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> Your MailChimp user name</div>" }, { "name" : "password", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> Your MailChimp password</div>" }, { "name" : "apikey", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> Any valid API Key</div>" } ], "returns" : { "type" : "item()*", "description" : "A new API Key that can be immediately used." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 3, "name" : "apikey-expire", "qname" : "mailchimp:apikey-expire", "signature" : "($username as xs:string, $password as xs:string, $apikey as xs:string) as item()*", "description" : " Convenience function for <a xmlns:xqdoc=\"http://www.xqdoc.org/1.0\" href=\"#apikey-expire-4\">apikey-expire#4</a>.\n", "summary" : "<p> Convenience function for  apikey-expire#4 .</p>", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "username", "type" : "xs:string", "occurrence" : null, "description" : "" }, { "name" : "password", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> Your MailChimp password</div>" }, { "name" : "apikey", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> Any valid API Key that you wish to expire</div>" } ], "returns" : { "type" : "item()*", "description" : "True if it worked, otherwise an error is thrown." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 4, "name" : "apikey-expire", "qname" : "mailchimp:apikey-expire", "signature" : "($endpoint-url as xs:string, $username as xs:string, $password as xs:string, $apikey as xs:string) as item()*", "description" : " Expire a Specific API Key. Note that if you expire all of your keys, just visit <a xmlns:xqdoc=\"http://www.xqdoc.org/1.0\" href=\"http://admin.mailchimp.com/account/api\" target=\"_blank\">your API dashboard</a> to create a new one. If you are trying to shut off access to your account for an old developer, change your MailChimp password, then expire all of the keys they had access to. Note that this takes effect immediately, so make sure you replace the keys in any working application before expiring them! Consider yourself warned...\n", "summary" : "<p> Expire a Specific API Key.</p>", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "endpoint-url", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> Mailchimp endpoint URL, e.g. http://us1.api.mailchimp.com/1.3/</div>" }, { "name" : "username", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> Your MailChimp user name</div>" }, { "name" : "password", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> Your MailChimp password</div>" }, { "name" : "apikey", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> Any valid API Key that you wish to expire</div>" } ], "returns" : { "type" : "item()*", "description" : "True if it worked, otherwise an error is thrown." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 4, "name" : "apikeys", "qname" : "mailchimp:apikeys", "signature" : "($username as xs:string, $password as xs:string, $apikey as xs:string, $expired as xs:boolean) as item()*", "description" : " Convenience function for <a xmlns:xqdoc=\"http://www.xqdoc.org/1.0\" href=\"#apikeys-5\">apikeys#5</a>.\n", "summary" : "<p> Convenience function for  apikeys#5 .</p>", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "username", "type" : "xs:string", "occurrence" : null, "description" : "" }, { "name" : "password", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> Your MailChimp password</div>" }, { "name" : "apikey", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> Any valid API Key for your account</div>" }, { "name" : "expired", "type" : "xs:boolean", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> optional - whether or not to include expired keys, defaults to false</div>" } ], "returns" : { "type" : "item()*", "description" : "An array of API keys." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 5, "name" : "apikeys", "qname" : "mailchimp:apikeys", "signature" : "($endpoint-url as xs:string, $username as xs:string, $password as xs:string, $apikey as xs:string, $expired as xs:boolean) as item()*", "description" : " Retrieve a list of all MailChimp API Keys for this User.\n", "summary" : "<p> Retrieve a list of all MailChimp API Keys for this User.</p>", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "endpoint-url", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> Mailchimp endpoint URL, e.g. http://us1.api.mailchimp.com/1.3/</div>" }, { "name" : "username", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> Your MailChimp user name</div>" }, { "name" : "password", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> Your MailChimp password</div>" }, { "name" : "apikey", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> Any valid API Key for your account</div>" }, { "name" : "expired", "type" : "xs:boolean", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> optional - whether or not to include expired keys, defaults to false</div>" } ], "returns" : { "type" : "item()*", "description" : "An array of API keys." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 5, "name" : "campaign-abuse-reports", "qname" : "mailchimp:campaign-abuse-reports", "signature" : "($apikey as xs:string, $cid as xs:string, $since as xs:integer, $start as xs:integer, $limit as xs:string) as item()*", "description" : " Convenience function for <a xmlns:xqdoc=\"http://www.xqdoc.org/1.0\" href=\"#campaign-abuse-reports-6\">campaign-abuse-reports#6</a>.\n", "summary" : "<p> Convenience function for  campaign-abuse-reports#6 .</p>", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "apikey", "type" : "xs:string", "occurrence" : null, "description" : "" }, { "name" : "cid", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the campaign id to pull abuse reports for (can be gathered using <a href=\"#campaigns-4\">campaigns()</a>)</div>" }, { "name" : "since", "type" : "xs:integer", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> optional for large data sets, the page number to start at - defaults to 1st page of data (page 0)</div>" }, { "name" : "start", "type" : "xs:integer", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> optional for large data sets, the number of results to return - defaults to 500, upper limit set at 1000</div>" }, { "name" : "limit", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> optional pull only messages since this time - use YYYY-MM-DD HH:II:SS format in <strong>GMT</strong></div>" } ], "returns" : { "type" : "item()*", "description" : "Reports the abuse reports for this campaign" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 6, "name" : "campaign-abuse-reports", "qname" : "mailchimp:campaign-abuse-reports", "signature" : "($endpoint-url as xs:string, $apikey as xs:string, $cid as xs:string, $since as xs:integer, $start as xs:integer, $limit as xs:string) as item()*", "description" : " Get all email addresses that complained about a given campaign.\n", "summary" : "<p> Get all email addresses that complained about a given campaign.</p>", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "endpoint-url", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> Mailchimp endpoint URL, e.g. http://us1.api.mailchimp.com/1.3/</div>" }, { "name" : "apikey", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> a valid API Key for your user account. Get by visiting <a href=\"http://admin.mailchimp.com/account/api\" target=\"_blank\">your API dashboard</a></div>" }, { "name" : "cid", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the campaign id to pull abuse reports for (can be gathered using <a href=\"#campaigns-4\">campaigns()</a>)</div>" }, { "name" : "since", "type" : "xs:integer", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> optional for large data sets, the page number to start at - defaults to 1st page of data (page 0)</div>" }, { "name" : "start", "type" : "xs:integer", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> optional for large data sets, the number of results to return - defaults to 500, upper limit set at 1000</div>" }, { "name" : "limit", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> optional pull only messages since this time - use YYYY-MM-DD HH:II:SS format in <strong>GMT</strong></div>" } ], "returns" : { "type" : "item()*", "description" : "Reports the abuse reports for this campaign" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 2, "name" : "campaign-advice", "qname" : "mailchimp:campaign-advice", "signature" : "($apikey as xs:string, $cid as xs:string) as item()*", "description" : " Convenience function for <a xmlns:xqdoc=\"http://www.xqdoc.org/1.0\" href=\"#campaign-advice-3\">campaign-advice#3</a>.\n", "summary" : "<p> Convenience function for  campaign-advice#3 .</p>", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "apikey", "type" : "xs:string", "occurrence" : null, "description" : "" }, { "name" : "cid", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the campaign id to pull advice text for (can be gathered using <a href=\"#campaigns-4\">campaigns()</a>)</div>" } ], "returns" : { "type" : "item()*", "description" : "Advice on the campaign's performance" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 3, "name" : "campaign-advice", "qname" : "mailchimp:campaign-advice", "signature" : "($endpoint-url as xs:string, $apikey as xs:string, $cid as xs:string) as item()*", "description" : " Retrieve the text presented in our app for how a campaign performed and any advice we may have for you - best suited for display in customized reports pages.\n Note: some messages will contain HTML - clean tags as necessary.\n", "summary" : "<p> Retrieve the text presented in our app for how a campaign performed and any advice we may have for you - best suited for display in customized reports pages.</p>", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "endpoint-url", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> Mailchimp endpoint URL, e.g. http://us1.api.mailchimp.com/1.3/</div>" }, { "name" : "apikey", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> a valid API Key for your user account. Get by visiting <a href=\"http://admin.mailchimp.com/account/api\" target=\"_blank\">your API dashboard</a></div>" }, { "name" : "cid", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the campaign id to pull advice text for (can be gathered using <a href=\"#campaigns-4\">campaigns()</a>)</div>" } ], "returns" : { "type" : "item()*", "description" : "Advice on the campaign's performance" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 2, "name" : "campaign-analytics", "qname" : "mailchimp:campaign-analytics", "signature" : "($apikey as xs:string, $cid as xs:string) as item()*", "description" : " Convenience function for <a xmlns:xqdoc=\"http://www.xqdoc.org/1.0\" href=\"#campaign-analytics-3\">campaign-analytics#3</a>.\n", "summary" : "<p> Convenience function for  campaign-analytics#3 .</p>", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "apikey", "type" : "xs:string", "occurrence" : null, "description" : "" }, { "name" : "cid", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the campaign id to pull bounces for (can be gathered using <a href=\"#campaigns-4\">campaigns()</a>)</div>" } ], "returns" : { "type" : "item()*", "description" : "Analytics we've collected for the passed campaign." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 3, "name" : "campaign-analytics", "qname" : "mailchimp:campaign-analytics", "signature" : "($endpoint-url as xs:string, $apikey as xs:string, $cid as xs:string) as item()*", "description" : " Retrieve the Google Analytics data we've collected for this campaign. Note, requires Google Analytics Add-on to be installed and configured.\n", "summary" : "<p> Retrieve the Google Analytics data we've collected for this campaign.</p>", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "endpoint-url", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> Mailchimp endpoint URL, e.g. http://us1.api.mailchimp.com/1.3/</div>" }, { "name" : "apikey", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> a valid API Key for your user account. Get by visiting <a href=\"http://admin.mailchimp.com/account/api\" target=\"_blank\">your API dashboard</a></div>" }, { "name" : "cid", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the campaign id to pull bounces for (can be gathered using <a href=\"#campaigns-4\">campaigns()</a>)</div>" } ], "returns" : { "type" : "item()*", "description" : "Analytics we've collected for the passed campaign." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 3, "name" : "campaign-bounce-message", "qname" : "mailchimp:campaign-bounce-message", "signature" : "($apikey as xs:string, $cid as xs:string, $email as xs:string) as item()*", "description" : " Convenience function for <a xmlns:xqdoc=\"http://www.xqdoc.org/1.0\" href=\"#campaign-bounce-message-4\">campaign-bounce-message#4</a>.\n", "summary" : "<p> Convenience function for  campaign-bounce-message#4 .</p>", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "apikey", "type" : "xs:string", "occurrence" : null, "description" : "" }, { "name" : "cid", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the campaign id to pull bounces for (can be gathered using <a href=\"#campaigns-4\">campaigns()</a>)</div>" }, { "name" : "email", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the email address or unique id of the member to pull a bounce message for.</div>" } ], "returns" : { "type" : "item()*", "description" : "The full bounce message for this email+campaign along with some extra data." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 4, "name" : "campaign-bounce-message", "qname" : "mailchimp:campaign-bounce-message", "signature" : "($endpoint-url as xs:string, $apikey as xs:string, $cid as xs:string, $email as xs:string) as item()*", "description" : " Retrieve the most recent full bounce message for a specific email address on the given campaign. Messages over 30 days old are subject to being removed.\n", "summary" : "<p> Retrieve the most recent full bounce message for a specific email address on the given campaign.</p>", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "endpoint-url", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> Mailchimp endpoint URL, e.g. http://us1.api.mailchimp.com/1.3/</div>" }, { "name" : "apikey", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> a valid API Key for your user account. Get by visiting <a href=\"http://admin.mailchimp.com/account/api\" target=\"_blank\">your API dashboard</a></div>" }, { "name" : "cid", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the campaign id to pull bounces for (can be gathered using <a href=\"#campaigns-4\">campaigns()</a>)</div>" }, { "name" : "email", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the email address or unique id of the member to pull a bounce message for.</div>" } ], "returns" : { "type" : "item()*", "description" : "The full bounce message for this email+campaign along with some extra data." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 5, "name" : "campaign-bounce-messages", "qname" : "mailchimp:campaign-bounce-messages", "signature" : "($apikey as xs:string, $cid as xs:string, $start as xs:integer, $limit as xs:integer, $since as xs:string) as item()*", "description" : " Convenience function for <a xmlns:xqdoc=\"http://www.xqdoc.org/1.0\" href=\"#campaign-bounce-messages-6\">campaign-bounce-messages#6</a>.\n", "summary" : "<p> Convenience function for  campaign-bounce-messages#6 .</p>", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "apikey", "type" : "xs:string", "occurrence" : null, "description" : "" }, { "name" : "cid", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the campaign id to pull bounces for (can be gathered using <a href=\"#campaigns-4\">campaigns()</a>)</div>" }, { "name" : "start", "type" : "xs:integer", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> optional for large data sets, the page number to start at - defaults to 1st page of data (page 0)</div>" }, { "name" : "limit", "type" : "xs:integer", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> optional for large data sets, the number of results to return - defaults to 25, upper limit set at 50</div>" }, { "name" : "since", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> optional pull only messages since this time - use YYYY-MM-DD format in <strong>GMT</strong> (we only store the date, not the time)</div>" } ], "returns" : { "type" : "item()*", "description" : "Bounces the full bounce messages for this campaign" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 6, "name" : "campaign-bounce-messages", "qname" : "mailchimp:campaign-bounce-messages", "signature" : "($endpoint-url as xs:string, $apikey as xs:string, $cid as xs:string, $start as xs:integer, $limit as xs:integer, $since as xs:string) as item()*", "description" : " Retrieve the full bounce messages for the given campaign. Note that this can return very large amounts of data depending on how large the campaign was and how much cruft the bounce provider returned.\n", "summary" : "<p> Retrieve the full bounce messages for the given campaign.</p>", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "endpoint-url", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> Mailchimp endpoint URL, e.g. http://us1.api.mailchimp.com/1.3/</div>" }, { "name" : "apikey", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> a valid API Key for your user account. Get by visiting <a href=\"http://admin.mailchimp.com/account/api\" target=\"_blank\">your API dashboard</a></div>" }, { "name" : "cid", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the campaign id to pull bounces for (can be gathered using <a href=\"#campaigns-4\">campaigns()</a>)</div>" }, { "name" : "start", "type" : "xs:integer", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> optional for large data sets, the page number to start at - defaults to 1st page of data (page 0)</div>" }, { "name" : "limit", "type" : "xs:integer", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> optional for large data sets, the number of results to return - defaults to 25, upper limit set at 50</div>" }, { "name" : "since", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> optional pull only messages since this time - use YYYY-MM-DD format in <strong>GMT</strong> (we only store the date, not the time)</div>" } ], "returns" : { "type" : "item()*", "description" : "Bounces the full bounce messages for this campaign" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 5, "name" : "campaign-click-detail-AIM", "qname" : "mailchimp:campaign-click-detail-AIM", "signature" : "($apikey as xs:string, $cid as xs:string, $url as xs:string, $start as xs:integer, $limit as xs:integer) as item()*", "description" : " Convenience function for <a xmlns:xqdoc=\"http://www.xqdoc.org/1.0\" href=\"#campaign-click-detail-AIM-6\">campaign-click-detail-AIM#6</a>.\n", "summary" : "<p> Convenience function for  campaign-click-detail-AIM#6 .</p>", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "apikey", "type" : "xs:string", "occurrence" : null, "description" : "" }, { "name" : "cid", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the campaign id to get click stats for (can be gathered using <a href=\"#campaigns-4\">campaigns()</a>)</div>" }, { "name" : "url", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the URL of the link that was clicked on</div>" }, { "name" : "start", "type" : "xs:integer", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> art optional for large data sets, the page number to start at - defaults to 1st page of data (page 0)</div>" }, { "name" : "limit", "type" : "xs:integer", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> mit optional for large data sets, the number of results to return - defaults to 1000, upper limit set at 15000</div>" } ], "returns" : { "type" : "item()*", "description" : "Array containing the total records matched and the specific records for this page" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 6, "name" : "campaign-click-detail-AIM", "qname" : "mailchimp:campaign-click-detail-AIM", "signature" : "($endpoint-url as xs:string, $apikey as xs:string, $cid as xs:string, $url as xs:string, $start as xs:integer, $limit as xs:integer) as item()*", "description" : " Return the list of email addresses that clicked on a given url, and how many times they clicked.\n", "summary" : "<p> Return the list of email addresses that clicked on a given url, and how many times they clicked.</p>", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "endpoint-url", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> Mailchimp endpoint URL, e.g. http://us1.api.mailchimp.com/1.3/</div>" }, { "name" : "apikey", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> a valid API Key for your user account. Get by visiting <a href=\"http://admin.mailchimp.com/account/api\" target=\"_blank\">your API dashboard</a></div>" }, { "name" : "cid", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the campaign id to get click stats for (can be gathered using <a href=\"#campaigns-4\">campaigns()</a>)</div>" }, { "name" : "url", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the URL of the link that was clicked on</div>" }, { "name" : "start", "type" : "xs:integer", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> art optional for large data sets, the page number to start at - defaults to 1st page of data (page 0)</div>" }, { "name" : "limit", "type" : "xs:integer", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> mit optional for large data sets, the number of results to return - defaults to 1000, upper limit set at 15000</div>" } ], "returns" : { "type" : "item()*", "description" : "Array containing the total records matched and the specific records for this page" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 2, "name" : "campaign-click-stats", "qname" : "mailchimp:campaign-click-stats", "signature" : "($apikey as xs:string, $cid as xs:string) as item()*", "description" : " Convenience function for <a xmlns:xqdoc=\"http://www.xqdoc.org/1.0\" href=\"#campaign-click-stats-3\">campaign-click-stats#3</a>.\n", "summary" : "<p> Convenience function for  campaign-click-stats#3 .</p>", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "apikey", "type" : "xs:string", "occurrence" : null, "description" : "" }, { "name" : "cid", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the campaign id to pull stats for (can be gathered using <a href=\"#campaigns-4\">campaigns()</a>)</div>" } ], "returns" : { "type" : "item()*", "description" : "URLs will be keys and contain their associated statistics: clicks (number of times a specific link was clicked) and unique (number of unique people who clicked on the specific link)." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 3, "name" : "campaign-click-stats", "qname" : "mailchimp:campaign-click-stats", "signature" : "($endpoint-url as xs:string, $apikey as xs:string, $cid as xs:string) as item()*", "description" : " Get an array of the urls being tracked, and their click counts for a given campaign.\n", "summary" : "<p> Get an array of the urls being tracked, and their click counts for a given campaign.</p>", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "endpoint-url", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> Mailchimp endpoint URL, e.g. http://us1.api.mailchimp.com/1.3/</div>" }, { "name" : "apikey", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> a valid API Key for your user account. Get by visiting <a href=\"http://admin.mailchimp.com/account/api\" target=\"_blank\">your API dashboard</a></div>" }, { "name" : "cid", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the campaign id to pull stats for (can be gathered using <a href=\"#campaigns-4\">campaigns()</a>)</div>" } ], "returns" : { "type" : "item()*", "description" : "URLs will be keys and contain their associated statistics: clicks (number of times a specific link was clicked) and unique (number of unique people who clicked on the specific link)." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 3, "name" : "campaign-content", "qname" : "mailchimp:campaign-content", "signature" : "($apikey as xs:string, $cid as xs:string, $for_archive as xs:boolean) as item()*", "description" : " Convenience function for <a xmlns:xqdoc=\"http://www.xqdoc.org/1.0\" href=\"#campaign-content-4\">campaign-content#4</a>.\n", "summary" : "<p> Convenience function for  campaign-content#4 .</p>", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "apikey", "type" : "xs:string", "occurrence" : null, "description" : "" }, { "name" : "cid", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the campaign id to get content for (can be gathered using <a href=\"#campaigns-4\">campaigns()</a>)</div>" }, { "name" : "for_archive", "type" : "xs:boolean", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> optional controls whether we return the Archive version (true) or the Raw version (false), defaults to true</div>" } ], "returns" : { "type" : "item()*", "description" : "Struct containing all content for the campaign" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 4, "name" : "campaign-content", "qname" : "mailchimp:campaign-content", "signature" : "($endpoint-url as xs:string, $apikey as xs:string, $cid as xs:string, $for_archive as xs:boolean) as item()*", "description" : " Get the content (both html and text) for a campaign either as it would appear in the campaign archive or as the raw, original content.\n", "summary" : "<p> Get the content (both html and text) for a campaign either as it would appear in the campaign archive or as the raw, original content.</p>", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "endpoint-url", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> Mailchimp endpoint URL, e.g. http://us1.api.mailchimp.com/1.3/</div>" }, { "name" : "apikey", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> a valid API Key for your user account. Get by visiting <a href=\"http://admin.mailchimp.com/account/api\" target=\"_blank\">your API dashboard</a></div>" }, { "name" : "cid", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the campaign id to get content for (can be gathered using <a href=\"#campaigns-4\">campaigns()</a>)</div>" }, { "name" : "for_archive", "type" : "xs:boolean", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> optional controls whether we return the Archive version (true) or the Raw version (false), defaults to true</div>" } ], "returns" : { "type" : "item()*", "description" : "Struct containing all content for the campaign" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 6, "name" : "campaign-create", "qname" : "mailchimp:campaign-create", "signature" : "($apikey as xs:string, $type as xs:string, $options as element(array), $content as element(array), $segment_opts as element(array), $type_opts as element(array)) as item()*", "description" : " Convenience function for <a xmlns:xqdoc=\"http://www.xqdoc.org/1.0\" href=\"#campaign-create-7\">campaign-create#7</a>.\n", "summary" : "<p> Convenience function for  campaign-create#7 .</p>", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "apikey", "type" : "xs:string", "occurrence" : null, "description" : "" }, { "name" : "type", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the Campaign Type to create - one of \"regular\", \"plaintext\", \"absplit\", \"rss\", \"trans\", \"auto\"</div>" }, { "name" : "options", "type" : "element(array)", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> a hash of the standard options for this campaign. See <a href=\"http://apidocs.mailchimp.com/rtfm/campaigncreate.func.php\">Mailchimp API documentation</a> for more information about the parameter structure.</div>" }, { "name" : "content", "type" : "element(array)", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the content for this campaign - use a struct with the following keys. See <a href=\"http://apidocs.mailchimp.com/rtfm/campaigncreate.func.php\">Mailchimp API documentation</a> for more information about the parameter structure.</div>" }, { "name" : "segment_opts", "type" : "element(array)", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> optional - if you wish to do Segmentation with this campaign this array should contain: see <a href=\"#campaign-segment-test-3\">campaign-segment-test()</a>. It's suggested that you test your options against <a href=\"#campaign-segment-test-3\">campaign-segment-test()</a>. Also, \"trans\" campaigns <strong>do not</strong> support segmentation.</div>" }, { "name" : "type_opts", "type" : "element(array)", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> optional -</div>" } ], "returns" : { "type" : "item()*", "description" : "The ID for the created campaign" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 7, "name" : "campaign-create", "qname" : "mailchimp:campaign-create", "signature" : "($endpoint-url as xs:string, $apikey as xs:string, $type as xs:string, $options as element(array), $content as element(array), $segment_opts as element(array), $type_opts as element(array)) as item()*", "description" : " Create a new draft campaign to send. You can not have more than 32,000 campaigns in your account.\n", "summary" : "<p> Create a new draft campaign to send.</p>", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "endpoint-url", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> Mailchimp endpoint URL, e.g. http://us1.api.mailchimp.com/1.3/</div>" }, { "name" : "apikey", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> a valid API Key for your user account. Get by visiting <a href=\"http://admin.mailchimp.com/account/api\" target=\"_blank\">your API dashboard</a></div>" }, { "name" : "type", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the Campaign Type to create - one of \"regular\", \"plaintext\", \"absplit\", \"rss\", \"trans\", \"auto\"</div>" }, { "name" : "options", "type" : "element(array)", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> a hash of the standard options for this campaign. See <a href=\"http://apidocs.mailchimp.com/rtfm/campaigncreate.func.php\">Mailchimp API documentation</a> for more information about the parameter structure.</div>" }, { "name" : "content", "type" : "element(array)", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the content for this campaign - use a struct with the following keys. See <a href=\"http://apidocs.mailchimp.com/rtfm/campaigncreate.func.php\">Mailchimp API documentation</a> for more information about the parameter structure.</div>" }, { "name" : "segment_opts", "type" : "element(array)", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> optional - if you wish to do Segmentation with this campaign this array should contain: see <a href=\"#campaign-segment-test-3\"><a href=\"#campaign-segment-test-3\">campaign-segment-test()</a></a>. It's suggested that you test your options against <a href=\"#campaign-segment-test-3\">campaign-segment-test()</a>. Also, \"trans\" campaigns <strong>do not</strong> support segmentation.</div>" }, { "name" : "type_opts", "type" : "element(array)", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> optional</div>" } ], "returns" : { "type" : "item()*", "description" : "The ID for the created campaign" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 2, "name" : "campaign-delete", "qname" : "mailchimp:campaign-delete", "signature" : "($apikey as xs:string, $cid as xs:string) as item()*", "description" : " Convenience function for <a xmlns:xqdoc=\"http://www.xqdoc.org/1.0\" href=\"#campaign-delete-3\">campaign-delete#3</a>.\n", "summary" : "<p> Convenience function for  campaign-delete#3 .</p>", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "apikey", "type" : "xs:string", "occurrence" : null, "description" : "" }, { "name" : "cid", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the Campaign Id to delete</div>" } ], "returns" : { "type" : "item()*", "description" : "True if the delete succeeds, otherwise an error will be thrown" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 3, "name" : "campaign-delete", "qname" : "mailchimp:campaign-delete", "signature" : "($endpoint-url as xs:string, $apikey as xs:string, $cid as xs:string) as item()*", "description" : " Delete a campaign. Seriously, \"poof, gone!\" - be careful!.\n", "summary" : "<p> Delete a campaign.</p>", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "endpoint-url", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> Mailchimp endpoint URL, e.g. http://us1.api.mailchimp.com/1.3/</div>" }, { "name" : "apikey", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> a valid API Key for your user account. Get by visiting <a href=\"http://admin.mailchimp.com/account/api\" target=\"_blank\">your API dashboard</a></div>" }, { "name" : "cid", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the Campaign Id to delete</div>" } ], "returns" : { "type" : "item()*", "description" : "True if the delete succeeds, otherwise an error will be thrown" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 2, "name" : "campaign-ecomm-order-add", "qname" : "mailchimp:campaign-ecomm-order-add", "signature" : "($apikey as xs:string, $order as element(array)) as item()*", "description" : " Convenience function for <a xmlns:xqdoc=\"http://www.xqdoc.org/1.0\" href=\"#campaign-ecomm-order-add-3\">campaign-ecomm-order-add#3</a>.\n", "summary" : "<p> Convenience function for  campaign-ecomm-order-add#3 .</p>", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "apikey", "type" : "xs:string", "occurrence" : null, "description" : "" }, { "name" : "order", "type" : "element(array)", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> an array of information pertaining to the order that has completed. Use the following keys. See <a href=\"http://apidocs.mailchimp.com/rtfm/campaignecommorderadd.func.php\">Mailchimp API documentation</a> for more information about the parameter structure.</div>" } ], "returns" : { "type" : "item()*", "description" : "True if the data is saved, otherwise an error is thrown." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 3, "name" : "campaign-ecomm-order-add", "qname" : "mailchimp:campaign-ecomm-order-add", "signature" : "($endpoint-url as xs:string, $apikey as xs:string, $order as element(array)) as item()*", "description" : " Attach Ecommerce Order Information to a Campaign.\n", "summary" : "<p> Attach Ecommerce Order Information to a Campaign.</p>", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "endpoint-url", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> Mailchimp endpoint URL, e.g. http://us1.api.mailchimp.com/1.3/</div>" }, { "name" : "apikey", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> a valid API Key for your user account. Get by visiting <a href=\"http://admin.mailchimp.com/account/api\" target=\"_blank\">your API dashboard</a></div>" }, { "name" : "order", "type" : "element(array)", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> an array of information pertaining to the order that has completed. Use the following keys. See <a href=\"http://apidocs.mailchimp.com/rtfm/campaignecommorderadd.func.php\">Mailchimp API documentation</a> for more information about the parameter structure.</div>" } ], "returns" : { "type" : "item()*", "description" : "True if the data is saved, otherwise an error is thrown." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 5, "name" : "campaign-ecomm-orders", "qname" : "mailchimp:campaign-ecomm-orders", "signature" : "($apikey as xs:string, $cid as xs:string, $start as xs:integer, $limit as xs:integer, $since as xs:string) as item()*", "description" : " Convenience function for <a xmlns:xqdoc=\"http://www.xqdoc.org/1.0\" href=\"#campaign-ecomm-orders-6\">campaign-ecomm-orders#6</a>.\n", "summary" : "<p> Convenience function for  campaign-ecomm-orders#6 .</p>", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "apikey", "type" : "xs:string", "occurrence" : null, "description" : "" }, { "name" : "cid", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the campaign id to pull bounces for (can be gathered using <a href=\"#campaigns-4\">campaigns()</a>)</div>" }, { "name" : "start", "type" : "xs:integer", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> optional for large data sets, the page number to start at - defaults to 1st page of data (page 0)</div>" }, { "name" : "limit", "type" : "xs:integer", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> optional for large data sets, the number of results to return - defaults to 100, upper limit set at 500</div>" }, { "name" : "since", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> optional pull only messages since this time - use YYYY-MM-DD HH:II:SS format in <strong>GMT</strong></div>" } ], "returns" : { "type" : "item()*", "description" : "The total matching orders and the specific orders for the requested page" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 6, "name" : "campaign-ecomm-orders", "qname" : "mailchimp:campaign-ecomm-orders", "signature" : "($endpoint-url as xs:string, $apikey as xs:string, $cid as xs:string, $start as xs:integer, $limit as xs:integer, $since as xs:string) as item()*", "description" : " Retrieve the Ecommerce Orders tracked by <a xmlns:xqdoc=\"http://www.xqdoc.org/1.0\" href=\"#campaign-ecomm-order-add-2\">campaign-ecomm-order-add()</a>.\n", "summary" : "<p> Retrieve the Ecommerce Orders tracked by  campaign-ecomm-order-add() .</p>", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "endpoint-url", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> Mailchimp endpoint URL, e.g. http://us1.api.mailchimp.com/1.3/</div>" }, { "name" : "apikey", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> a valid API Key for your user account. Get by visiting <a href=\"http://admin.mailchimp.com/account/api\" target=\"_blank\">your API dashboard</a></div>" }, { "name" : "cid", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the campaign id to pull bounces for (can be gathered using <a href=\"#campaigns-4\">campaigns()</a>)</div>" }, { "name" : "start", "type" : "xs:integer", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> optional for large data sets, the page number to start at - defaults to 1st page of data (page 0)</div>" }, { "name" : "limit", "type" : "xs:integer", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> optional for large data sets, the number of results to return - defaults to 100, upper limit set at 500</div>" }, { "name" : "since", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> optional pull only messages since this time - use YYYY-MM-DD HH:II:SS format in <strong>GMT</strong></div>" } ], "returns" : { "type" : "item()*", "description" : "The total matching orders and the specific orders for the requested page" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 2, "name" : "campaign-eep-url-stats", "qname" : "mailchimp:campaign-eep-url-stats", "signature" : "($apikey as xs:string, $cid as xs:string) as item()*", "description" : " Convenience function for <a xmlns:xqdoc=\"http://www.xqdoc.org/1.0\" href=\"#campaign-eep-url-stats-3\">campaign-eep-url-stats#3</a>.\n", "summary" : "<p> Convenience function for  campaign-eep-url-stats#3 .</p>", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "apikey", "type" : "xs:string", "occurrence" : null, "description" : "" }, { "name" : "cid", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the campaign id to pull bounces for (can be gathered using <a href=\"#campaigns-4\">campaigns()</a>)</div>" } ], "returns" : { "type" : "item()*", "description" : "Stats an array containing tweets, retweets, clicks, and referrer related to using the campaign's eepurl" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 3, "name" : "campaign-eep-url-stats", "qname" : "mailchimp:campaign-eep-url-stats", "signature" : "($endpoint-url as xs:string, $apikey as xs:string, $cid as xs:string) as item()*", "description" : " Retrieve the tracked eepurl mentions on Twitter.\n", "summary" : "<p> Retrieve the tracked eepurl mentions on Twitter.</p>", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "endpoint-url", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> Mailchimp endpoint URL, e.g. http://us1.api.mailchimp.com/1.3/</div>" }, { "name" : "apikey", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> a valid API Key for your user account. Get by visiting <a href=\"http://admin.mailchimp.com/account/api\" target=\"_blank\">your API dashboard</a></div>" }, { "name" : "cid", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the campaign id to pull bounces for (can be gathered using <a href=\"#campaigns-4\">campaigns()</a>)</div>" } ], "returns" : { "type" : "item()*", "description" : "Stats an array containing tweets, retweets, clicks, and referrer related to using the campaign's eepurl" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 2, "name" : "campaign-email-domain-performance", "qname" : "mailchimp:campaign-email-domain-performance", "signature" : "($apikey as xs:string, $cid as xs:string) as item()*", "description" : " Convenience function for <a xmlns:xqdoc=\"http://www.xqdoc.org/1.0\" href=\"#campaign-email-domain-performance-3\">campaign-email-domain-performance#3</a>.\n", "summary" : "<p> Convenience function for  campaign-email-domain-performance#3 .</p>", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "apikey", "type" : "xs:string", "occurrence" : null, "description" : "" }, { "name" : "cid", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the campaign id to pull email domain performance for (can be gathered using <a href=\"#campaigns-4\">campaigns()</a>)</div>" } ], "returns" : { "type" : "item()*", "description" : "Domains email domains and their associated stats" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 3, "name" : "campaign-email-domain-performance", "qname" : "mailchimp:campaign-email-domain-performance", "signature" : "($endpoint-url as xs:string, $apikey as xs:string, $cid as xs:string) as item()*", "description" : " Get the top 5 performing email domains for this campaign. Users want more than 5 should use campaign <a xmlns:xqdoc=\"http://www.xqdoc.org/1.0\" href=\"#campaign-email-stats-AIM-3\">campaign-email-stats-AIM()</a> or <a xmlns:xqdoc=\"http://www.xqdoc.org/1.0\" href=\"#campaign-email-stats-AIM-all-4\">campaign-email-stats-AIM-all()</a> and generate any additional stats they require.\n", "summary" : "<p> Get the top 5 performing email domains for this campaign.</p>", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "endpoint-url", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> Mailchimp endpoint URL, e.g. http://us1.api.mailchimp.com/1.3/</div>" }, { "name" : "apikey", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> a valid API Key for your user account. Get by visiting <a href=\"http://admin.mailchimp.com/account/api\" target=\"_blank\">your API dashboard</a></div>" }, { "name" : "cid", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the campaign id to pull email domain performance for (can be gathered using <a href=\"#campaigns-4\">campaigns()</a>)</div>" } ], "returns" : { "type" : "item()*", "description" : "Domains email domains and their associated stats" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 4, "name" : "campaign-email-stats-AIM-all", "qname" : "mailchimp:campaign-email-stats-AIM-all", "signature" : "($apikey as xs:string, $cid as xs:string, $start as xs:integer, $limit as xs:integer) as item()*", "description" : " Convenience function for <a xmlns:xqdoc=\"http://www.xqdoc.org/1.0\" href=\"#campaign-email-stats-AIM-all-5\">campaign-email-stats-AIM-all#5</a>.\n", "summary" : "<p> Convenience function for  campaign-email-stats-AIM-all#5 .</p>", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "apikey", "type" : "xs:string", "occurrence" : null, "description" : "" }, { "name" : "cid", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the campaign id to get stats for (can be gathered using <a href=\"#campaigns-4\">campaigns()</a>)</div>" }, { "name" : "start", "type" : "xs:integer", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> optional for large data sets, the page number to start at - defaults to 1st page of data (page 0)</div>" }, { "name" : "limit", "type" : "xs:integer", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> optional for large data sets, the number of results to return - defaults to 100, upper limit set at 1000</div>" } ], "returns" : { "type" : "item()*", "description" : "Array containing a total record count and data including the actions (opens and clicks) for each email, with timestamps" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 5, "name" : "campaign-email-stats-AIM-all", "qname" : "mailchimp:campaign-email-stats-AIM-all", "signature" : "($endpoint-url as xs:string, $apikey as xs:string, $cid as xs:string, $start as xs:integer, $limit as xs:integer) as item()*", "description" : " Given a campaign and correct paging limits, return the entire click and open history with timestamps, ordered by time, for every user a campaign was delivered to.\n", "summary" : "<p> Given a campaign and correct paging limits, return the entire click and open history with timestamps, ordered by time, for every user a campaign was delivered to.</p>", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "endpoint-url", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> Mailchimp endpoint URL, e.g. http://us1.api.mailchimp.com/1.3/</div>" }, { "name" : "apikey", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> a valid API Key for your user account. Get by visiting <a href=\"http://admin.mailchimp.com/account/api\" target=\"_blank\">your API dashboard</a></div>" }, { "name" : "cid", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the campaign id to get stats for (can be gathered using <a href=\"#campaigns-4\">campaigns()</a>)</div>" }, { "name" : "start", "type" : "xs:integer", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> optional for large data sets, the page number to start at - defaults to 1st page of data (page 0)</div>" }, { "name" : "limit", "type" : "xs:integer", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> optional for large data sets, the number of results to return - defaults to 100, upper limit set at 1000</div>" } ], "returns" : { "type" : "item()*", "description" : "Array containing a total record count and data including the actions (opens and clicks) for each email, with timestamps" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 3, "name" : "campaign-email-stats-AIM", "qname" : "mailchimp:campaign-email-stats-AIM", "signature" : "($apikey as xs:string, $cid as xs:string, $email_address as element(array)) as item()*", "description" : " Convenience function for <a xmlns:xqdoc=\"http://www.xqdoc.org/1.0\" href=\"#campaign-email-stats-AIM-4\">campaign-email-stats-AIM#4</a>.\n", "summary" : "<p> Convenience function for  campaign-email-stats-AIM#4 .</p>", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "apikey", "type" : "xs:string", "occurrence" : null, "description" : "" }, { "name" : "cid", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the campaign id to get stats for (can be gathered using <a href=\"#campaigns-4\">campaigns()</a>)</div>" }, { "name" : "email_address", "type" : "element(array)", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> an array of up to 50 email addresses to check OR the email \"id\" returned from listMemberInfo, Webhooks, and Campaigns. For backwards compatibility, if a string is passed, it will be treated as an array with a single element (will not work with XML-RPC).</div>" } ], "returns" : { "type" : "item()*", "description" : "Array an array with the keys listed in Returned Fields below" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 4, "name" : "campaign-email-stats-AIM", "qname" : "mailchimp:campaign-email-stats-AIM", "signature" : "($endpoint-url as xs:string, $apikey as xs:string, $cid as xs:string, $email_address as element(array)) as item()*", "description" : " Given a campaign and email address, return the entire click and open history with timestamps, ordered by time.\n", "summary" : "<p> Given a campaign and email address, return the entire click and open history with timestamps, ordered by time.</p>", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "endpoint-url", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> Mailchimp endpoint URL, e.g. http://us1.api.mailchimp.com/1.3/</div>" }, { "name" : "apikey", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> a valid API Key for your user account. Get by visiting <a href=\"http://admin.mailchimp.com/account/api\" target=\"_blank\">your API dashboard</a></div>" }, { "name" : "cid", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the campaign id to get stats for (can be gathered using <a href=\"#campaigns-4\">campaigns()</a>)</div>" }, { "name" : "email_address", "type" : "element(array)", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> an array of up to 50 email addresses to check OR the email \"id\" returned from listMemberInfo, Webhooks, and Campaigns. For backwards compatibility, if a string is passed, it will be treated as an array with a single element (will not work with XML-RPC).</div>" } ], "returns" : { "type" : "item()*", "description" : "Array an array with the keys listed in Returned Fields below" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 3, "name" : "campaign-geo-opens-for-country", "qname" : "mailchimp:campaign-geo-opens-for-country", "signature" : "($apikey as xs:string, $cid as xs:string, $code as xs:string) as item()*", "description" : " Convenience function for <a xmlns:xqdoc=\"http://www.xqdoc.org/1.0\" href=\"#campaign-geo-opens-for-country-4\">campaign-geo-opens-for-country#4</a>.\n", "summary" : "<p> Convenience function for  campaign-geo-opens-for-country#4 .</p>", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "apikey", "type" : "xs:string", "occurrence" : null, "description" : "" }, { "name" : "cid", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the campaign id to pull bounces for (can be gathered using <a href=\"#campaigns-4\">campaigns()</a>)</div>" }, { "name" : "code", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> An ISO3166 2 digit country code</div>" } ], "returns" : { "type" : "item()*", "description" : "Regions an array of regions within the provided country where opens occurred." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 4, "name" : "campaign-geo-opens-for-country", "qname" : "mailchimp:campaign-geo-opens-for-country", "signature" : "($endpoint-url as xs:string, $apikey as xs:string, $cid as xs:string, $code as xs:string) as item()*", "description" : " Retrieve the regions and number of opens tracked for a certain country. Email address are not returned.\n", "summary" : "<p> Retrieve the regions and number of opens tracked for a certain country.</p>", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "endpoint-url", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> Mailchimp endpoint URL, e.g. http://us1.api.mailchimp.com/1.3/</div>" }, { "name" : "apikey", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> a valid API Key for your user account. Get by visiting <a href=\"http://admin.mailchimp.com/account/api\" target=\"_blank\">your API dashboard</a></div>" }, { "name" : "cid", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the campaign id to pull bounces for (can be gathered using <a href=\"#campaigns-4\">campaigns()</a>)</div>" }, { "name" : "code", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> An ISO3166 2 digit country code</div>" } ], "returns" : { "type" : "item()*", "description" : "Regions an array of regions within the provided country where opens occurred." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 2, "name" : "campaign-geo-opens", "qname" : "mailchimp:campaign-geo-opens", "signature" : "($apikey as xs:string, $cid as xs:string) as item()*", "description" : " Convenience function for <a xmlns:xqdoc=\"http://www.xqdoc.org/1.0\" href=\"#campaign-geo-opens-3\">campaign-geo-opens#3</a>.\n", "summary" : "<p> Convenience function for  campaign-geo-opens#3 .</p>", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "apikey", "type" : "xs:string", "occurrence" : null, "description" : "" }, { "name" : "cid", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the campaign id to pull bounces for (can be gathered using <a href=\"#campaigns-4\">campaigns()</a>)</div>" } ], "returns" : { "type" : "item()*", "description" : "Countries an array of countries where opens occurred" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 3, "name" : "campaign-geo-opens", "qname" : "mailchimp:campaign-geo-opens", "signature" : "($endpoint-url as xs:string, $apikey as xs:string, $cid as xs:string) as item()*", "description" : " Retrieve the countries and number of opens tracked for each. Email address are not returned.\n", "summary" : "<p> Retrieve the countries and number of opens tracked for each.</p>", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "endpoint-url", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> Mailchimp endpoint URL, e.g. http://us1.api.mailchimp.com/1.3/</div>" }, { "name" : "apikey", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> a valid API Key for your user account. Get by visiting <a href=\"http://admin.mailchimp.com/account/api\" target=\"_blank\">your API dashboard</a></div>" }, { "name" : "cid", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the campaign id to pull bounces for (can be gathered using <a href=\"#campaigns-4\">campaigns()</a>)</div>" } ], "returns" : { "type" : "item()*", "description" : "Countries an array of countries where opens occurred" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 4, "name" : "campaign-hard-bounces", "qname" : "mailchimp:campaign-hard-bounces", "signature" : "($apikey as xs:string, $cid as xs:string, $start as xs:integer, $limit as xs:integer) as item()*", "description" : " Convenience function for <a xmlns:xqdoc=\"http://www.xqdoc.org/1.0\" href=\"#campaign-hard-bounces-5\">campaign-hard-bounces#5</a>.\n", "summary" : "<p> Convenience function for  campaign-hard-bounces#5 .</p>", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "apikey", "type" : "xs:string", "occurrence" : null, "description" : "" }, { "name" : "cid", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the campaign id to pull bounces for (can be gathered using <a href=\"#campaigns-4\">campaigns()</a>)</div>" }, { "name" : "start", "type" : "xs:integer", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> art optional for large data sets, the page number to start at - defaults to 1st page of data (page 0)</div>" }, { "name" : "limit", "type" : "xs:integer", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> mit optional for large data sets, the number of results to return - defaults to 1000, upper limit set at 15000</div>" } ], "returns" : { "type" : "item()*", "description" : "array a total of all hard bounced emails and the specific emails for this page" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 5, "name" : "campaign-hard-bounces", "qname" : "mailchimp:campaign-hard-bounces", "signature" : "($endpoint-url as xs:string, $apikey as xs:string, $cid as xs:string, $start as xs:integer, $limit as xs:integer) as item()*", "description" : " <strong xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">DEPRECATED</strong> Get all email addresses with Hard Bounces for a given campaign the email address that bounced.\n", "summary" : "<p>  DEPRECATED  Get all email addresses with Hard Bounces for a given campaign the email address that bounced.</p>", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "endpoint-url", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> Mailchimp endpoint URL, e.g. http://us1.api.mailchimp.com/1.3/</div>" }, { "name" : "apikey", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> a valid API Key for your user account. Get by visiting <a href=\"http://admin.mailchimp.com/account/api\" target=\"_blank\">your API dashboard</a></div>" }, { "name" : "cid", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the campaign id to pull bounces for (can be gathered using <a href=\"#campaigns-4\">campaigns()</a>)</div>" }, { "name" : "start", "type" : "xs:integer", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> art optional for large data sets, the page number to start at - defaults to 1st page of data (page 0)</div>" }, { "name" : "limit", "type" : "xs:integer", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> mit optional for large data sets, the number of results to return - defaults to 1000, upper limit set at 15000</div>" } ], "returns" : { "type" : "item()*", "description" : "array a total of all hard bounced emails and the specific emails for this page" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 5, "name" : "campaign-members", "qname" : "mailchimp:campaign-members", "signature" : "($apikey as xs:string, $cid as xs:string, $status as xs:string, $start as xs:integer, $limit as xs:integer) as item()*", "description" : " Convenience function for <a xmlns:xqdoc=\"http://www.xqdoc.org/1.0\" href=\"#campaign-members-6\">campaign-members#6</a>.\n", "summary" : "<p> Convenience function for  campaign-members#6 .</p>", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "apikey", "type" : "xs:string", "occurrence" : null, "description" : "" }, { "name" : "cid", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the campaign id to pull members for (can be gathered using <a href=\"#campaigns-4\">campaigns()</a>)</div>" }, { "name" : "status", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> optional the status to pull - one of 'sent', 'hard' (bounce), or 'soft' (bounce). By default, all records are returned</div>" }, { "name" : "start", "type" : "xs:integer", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> art optional for large data sets, the page number to start at - defaults to 1st page of data (page 0)</div>" }, { "name" : "limit", "type" : "xs:integer", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> mit optional for large data sets, the number of results to return - defaults to 1000, upper limit set at 15000</div>" } ], "returns" : { "type" : "item()*", "description" : "A total of all matching emails and the specific emails for this page" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 6, "name" : "campaign-members", "qname" : "mailchimp:campaign-members", "signature" : "($endpoint-url as xs:string, $apikey as xs:string, $cid as xs:string, $status as xs:string, $start as xs:integer, $limit as xs:integer) as item()*", "description" : " Get all email addresses the campaign was successfully sent to (ie, no bounces).\n", "summary" : "<p> Get all email addresses the campaign was successfully sent to (ie, no bounces).</p>", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "endpoint-url", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> Mailchimp endpoint URL, e.g. http://us1.api.mailchimp.com/1.3/</div>" }, { "name" : "apikey", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> a valid API Key for your user account. Get by visiting <a href=\"http://admin.mailchimp.com/account/api\" target=\"_blank\">your API dashboard</a></div>" }, { "name" : "cid", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the campaign id to pull members for (can be gathered using <a href=\"#campaigns-4\">campaigns()</a>)</div>" }, { "name" : "status", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> optional the status to pull - one of 'sent', 'hard' (bounce), or 'soft' (bounce). By default, all records are returned</div>" }, { "name" : "start", "type" : "xs:integer", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> art optional for large data sets, the page number to start at - defaults to 1st page of data (page 0)</div>" }, { "name" : "limit", "type" : "xs:integer", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> mit optional for large data sets, the number of results to return - defaults to 1000, upper limit set at 15000</div>" } ], "returns" : { "type" : "item()*", "description" : "A total of all matching emails and the specific emails for this page" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 4, "name" : "campaign-not-opened-AIM", "qname" : "mailchimp:campaign-not-opened-AIM", "signature" : "($apikey as xs:string, $cid as xs:string, $start as xs:integer, $limit as xs:integer) as item()*", "description" : " Convenience function for <a xmlns:xqdoc=\"http://www.xqdoc.org/1.0\" href=\"#campaign-not-opened-AIM-5\">campaign-not-opened-AIM#5</a>.\n", "summary" : "<p> Convenience function for  campaign-not-opened-AIM#5 .</p>", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "apikey", "type" : "xs:string", "occurrence" : null, "description" : "" }, { "name" : "cid", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the campaign id to get no opens for (can be gathered using <a href=\"#campaigns-4\">campaigns()</a>)</div>" }, { "name" : "start", "type" : "xs:integer", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> art optional for large data sets, the page number to start at - defaults to 1st page of data (page 0)</div>" }, { "name" : "limit", "type" : "xs:integer", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> mit optional for large data sets, the number of results to return - defaults to 1000, upper limit set at 15000</div>" } ], "returns" : { "type" : "item()*", "description" : "Array containing the total records matched and the specific records for this page" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 5, "name" : "campaign-not-opened-AIM", "qname" : "mailchimp:campaign-not-opened-AIM", "signature" : "($endpoint-url as xs:string, $apikey as xs:string, $cid as xs:string, $start as xs:integer, $limit as xs:integer) as item()*", "description" : " Retrieve the list of email addresses that did not open a given campaign string email Email address that opened the campaign.\n", "summary" : "<p> Retrieve the list of email addresses that did not open a given campaign string email Email address that opened the campaign.</p>", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "endpoint-url", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> Mailchimp endpoint URL, e.g. http://us1.api.mailchimp.com/1.3/</div>" }, { "name" : "apikey", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> a valid API Key for your user account. Get by visiting <a href=\"http://admin.mailchimp.com/account/api\" target=\"_blank\">your API dashboard</a></div>" }, { "name" : "cid", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the campaign id to get no opens for (can be gathered using <a href=\"#campaigns-4\">campaigns()</a>)</div>" }, { "name" : "start", "type" : "xs:integer", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> art optional for large data sets, the page number to start at - defaults to 1st page of data (page 0)</div>" }, { "name" : "limit", "type" : "xs:integer", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> mit optional for large data sets, the number of results to return - defaults to 1000, upper limit set at 15000</div>" } ], "returns" : { "type" : "item()*", "description" : "array array containing the total records matched and the specific records for this page" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 4, "name" : "campaign-opened-AIM", "qname" : "mailchimp:campaign-opened-AIM", "signature" : "($apikey as xs:string, $cid as xs:string, $start as xs:integer, $limit as xs:integer) as item()*", "description" : " Convenience function for <a xmlns:xqdoc=\"http://www.xqdoc.org/1.0\" href=\"#campaign-opened-AIM-5\">campaign-openedAIM#5</a>.\n", "summary" : "<p> Convenience function for  campaign-openedAIM#5 .</p>", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "apikey", "type" : "xs:string", "occurrence" : null, "description" : "" }, { "name" : "cid", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the campaign id to get opens for (can be gathered using <a href=\"#campaigns-4\">campaigns()</a>)</div>" }, { "name" : "start", "type" : "xs:integer", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> art optional for large data sets, the page number to start at - defaults to 1st page of data (page 0)</div>" }, { "name" : "limit", "type" : "xs:integer", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> mit optional for large data sets, the number of results to return - defaults to 1000, upper limit set at 15000</div>" } ], "returns" : { "type" : "item()*", "description" : "Array containing the total records matched and the specific records for this page" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 5, "name" : "campaign-opened-AIM", "qname" : "mailchimp:campaign-opened-AIM", "signature" : "($endpoint-url as xs:string, $apikey as xs:string, $cid as xs:string, $start as xs:integer, $limit as xs:integer) as item()*", "description" : " Retrieve the list of email addresses that opened a given campaign with how many times they opened.\n", "summary" : "<p> Retrieve the list of email addresses that opened a given campaign with how many times they opened.</p>", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "endpoint-url", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> Mailchimp endpoint URL, e.g. http://us1.api.mailchimp.com/1.3/</div>" }, { "name" : "apikey", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> a valid API Key for your user account. Get by visiting <a href=\"http://admin.mailchimp.com/account/api\" target=\"_blank\">your API dashboard</a></div>" }, { "name" : "cid", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the campaign id to get opens for (can be gathered using <a href=\"#campaigns-4\">campaigns()</a>)</div>" }, { "name" : "start", "type" : "xs:integer", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> art optional for large data sets, the page number to start at - defaults to 1st page of data (page 0)</div>" }, { "name" : "limit", "type" : "xs:integer", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> mit optional for large data sets, the number of results to return - defaults to 1000, upper limit set at 15000</div>" } ], "returns" : { "type" : "item()*", "description" : "Array containing the total records matched and the specific records for this page" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 2, "name" : "campaign-pause", "qname" : "mailchimp:campaign-pause", "signature" : "($apikey as xs:string, $cid as xs:string) as item()*", "description" : " Convenience function for <a xmlns:xqdoc=\"http://www.xqdoc.org/1.0\" href=\"#campaign-pause-3\">campaign-pause#3</a>.\n", "summary" : "<p> Convenience function for  campaign-pause#3 .</p>", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "apikey", "type" : "xs:string", "occurrence" : null, "description" : "" }, { "name" : "cid", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the id of the campaign to pause</div>" } ], "returns" : { "type" : "item()*", "description" : "True on success" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 3, "name" : "campaign-pause", "qname" : "mailchimp:campaign-pause", "signature" : "($endpoint-url as xs:string, $apikey as xs:string, $cid as xs:string) as item()*", "description" : " Pause an AutoResponder or RSS campaign from sending.\n", "summary" : "<p> Pause an AutoResponder or RSS campaign from sending.</p>", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "endpoint-url", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> Mailchimp endpoint URL, e.g. http://us1.api.mailchimp.com/1.3/</div>" }, { "name" : "apikey", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> a valid API Key for your user account. Get by visiting <a href=\"http://admin.mailchimp.com/account/api\" target=\"_blank\">your API dashboard</a></div>" }, { "name" : "cid", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the id of the campaign to pause</div>" } ], "returns" : { "type" : "item()*", "description" : "True on success" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 2, "name" : "campaign-replicate", "qname" : "mailchimp:campaign-replicate", "signature" : "($apikey as xs:string, $cid as xs:string) as item()*", "description" : " Convenience function for <a xmlns:xqdoc=\"http://www.xqdoc.org/1.0\" href=\"#campaign-replicate-3\">campaign-replicate#3</a>.\n", "summary" : "<p> Convenience function for  campaign-replicate#3 .</p>", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "apikey", "type" : "xs:string", "occurrence" : null, "description" : "" }, { "name" : "cid", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the Campaign Id to replicate</div>" } ], "returns" : { "type" : "item()*", "description" : "The id of the replicated Campaign created, otherwise an error will be thrown" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 3, "name" : "campaign-replicate", "qname" : "mailchimp:campaign-replicate", "signature" : "($endpoint-url as xs:string, $apikey as xs:string, $cid as xs:string) as item()*", "description" : " Replicate a campaign.\n", "summary" : "<p> Replicate a campaign.</p>", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "endpoint-url", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> Mailchimp endpoint URL, e.g. http://us1.api.mailchimp.com/1.3/</div>" }, { "name" : "apikey", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> a valid API Key for your user account. Get by visiting <a href=\"http://admin.mailchimp.com/account/api\" target=\"_blank\">your API dashboard</a></div>" }, { "name" : "cid", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the Campaign Id to replicate</div>" } ], "returns" : { "type" : "item()*", "description" : "The id of the replicated Campaign created, otherwise an error will be thrown" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 2, "name" : "campaign-resume", "qname" : "mailchimp:campaign-resume", "signature" : "($apikey as xs:string, $cid as xs:string) as item()*", "description" : " Convenience function for <a xmlns:xqdoc=\"http://www.xqdoc.org/1.0\" href=\"#campaign-resume-3\">campaign-resume#3</a>.\n", "summary" : "<p> Convenience function for  campaign-resume#3 .</p>", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "apikey", "type" : "xs:string", "occurrence" : null, "description" : "" }, { "name" : "cid", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the id of the campaign to pause</div>" } ], "returns" : { "type" : "item()*", "description" : "True on success" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 3, "name" : "campaign-resume", "qname" : "mailchimp:campaign-resume", "signature" : "($endpoint-url as xs:string, $apikey as xs:string, $cid as xs:string) as item()*", "description" : " Resume sending an AutoResponder or RSS campaign.\n", "summary" : "<p> Resume sending an AutoResponder or RSS campaign.</p>", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "endpoint-url", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> Mailchimp endpoint URL, e.g. http://us1.api.mailchimp.com/1.3/</div>" }, { "name" : "apikey", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> a valid API Key for your user account. Get by visiting <a href=\"http://admin.mailchimp.com/account/api\" target=\"_blank\">your API dashboard</a></div>" }, { "name" : "cid", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the id of the campaign to pause</div>" } ], "returns" : { "type" : "item()*", "description" : "True on success" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 4, "name" : "campaign-schedule", "qname" : "mailchimp:campaign-schedule", "signature" : "($apikey as xs:string, $cid as xs:string, $schedule_time as xs:string, $schedule_time_b as xs:string) as item()*", "description" : " Convenience function for <a xmlns:xqdoc=\"http://www.xqdoc.org/1.0\" href=\"#campaign-schedule-5\">campaign-schedule#5</a>.\n", "summary" : "<p> Convenience function for  campaign-schedule#5 .</p>", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "apikey", "type" : "xs:string", "occurrence" : null, "description" : "" }, { "name" : "cid", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the id of the campaign to schedule</div>" }, { "name" : "schedule_time", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the time to schedule the campaign. For A/B Split \"schedule\" campaigns, the time for Group A - in YYYY-MM-DD HH:II:SS format in <strong>GMT</strong></div>" }, { "name" : "schedule_time_b", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> optional -the time to schedule Group B of an A/B Split \"schedule\" campaign - in YYYY-MM-DD HH:II:SS format in <strong>GMT</strong></div>" } ], "returns" : { "type" : "item()*", "description" : "boolean True on success" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 5, "name" : "campaign-schedule", "qname" : "mailchimp:campaign-schedule", "signature" : "($endpoint-url as xs:string, $apikey as xs:string, $cid as xs:string, $schedule_time as xs:string, $schedule_time_b as xs:string) as item()*", "description" : " Schedule a campaign to be sent in the future.\n", "summary" : "<p> Schedule a campaign to be sent in the future.</p>", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "endpoint-url", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> Mailchimp endpoint URL, e.g. http://us1.api.mailchimp.com/1.3/</div>" }, { "name" : "apikey", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> a valid API Key for your user account. Get by visiting <a href=\"http://admin.mailchimp.com/account/api\" target=\"_blank\">your API dashboard</a></div>" }, { "name" : "cid", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the id of the campaign to schedule</div>" }, { "name" : "schedule_time", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the time to schedule the campaign. For A/B Split \"schedule\" campaigns, the time for Group A - in YYYY-MM-DD HH:II:SS format in <strong>GMT</strong></div>" }, { "name" : "schedule_time_b", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> optional -the time to schedule Group B of an A/B Split \"schedule\" campaign - in YYYY-MM-DD HH:II:SS format in <strong>GMT</strong></div>" } ], "returns" : { "type" : "item()*", "description" : "boolean True on success" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 3, "name" : "campaign-segment-test", "qname" : "mailchimp:campaign-segment-test", "signature" : "($apikey as xs:string, $list_id as xs:string, $options as element(array)) as item()*", "description" : " Convenience function for <a xmlns:xqdoc=\"http://www.xqdoc.org/1.0\" href=\"#campaign-segment-test-4\">campaign-segment-test#4</a>.\n", "summary" : "<p> Convenience function for  campaign-segment-test#4 .</p>", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "apikey", "type" : "xs:string", "occurrence" : null, "description" : "" }, { "name" : "list_id", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the list to test segmentation on - get lists using lists()</div>" }, { "name" : "options", "type" : "element(array)", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> with 2 keys. See <a href=\"http://apidocs.mailchimp.com/rtfm/campaignsegmenttest.func.php\">Mailchimp API documentation</a> for more information about the parameter structure.</div>" } ], "returns" : { "type" : "item()*", "description" : "The total number of subscribers matching your segmentation options" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 4, "name" : "campaign-segment-test", "qname" : "mailchimp:campaign-segment-test", "signature" : "($endpoint-url as xs:string, $apikey as xs:string, $list_id as xs:string, $options as element(array)) as item()*", "description" : " Allows one to test their segmentation rules before creating a campaign using them.\n", "summary" : "<p> Allows one to test their segmentation rules before creating a campaign using them.</p>", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "endpoint-url", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> Mailchimp endpoint URL, e.g. http://us1.api.mailchimp.com/1.3/</div>" }, { "name" : "apikey", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> a valid API Key for your user account. Get by visiting <a href=\"http://admin.mailchimp.com/account/api\" target=\"_blank\">your API dashboard</a></div>" }, { "name" : "list_id", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the list to test segmentation on - get lists using <a href=\"#lists-4\">lists()</a></div>" }, { "name" : "options", "type" : "element(array)", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> with 2 keys. See <a href=\"http://apidocs.mailchimp.com/rtfm/campaignsegmenttest.func.php\">Mailchimp API documentation</a> for more information about the parameter structure.</div>" } ], "returns" : { "type" : "item()*", "description" : "The total number of subscribers matching your segmentation options" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 2, "name" : "campaign-send-now", "qname" : "mailchimp:campaign-send-now", "signature" : "($apikey as xs:string, $cid as xs:string) as item()*", "description" : " Convenience function for <a xmlns:xqdoc=\"http://www.xqdoc.org/1.0\" href=\"#campaign-send-now-3\">campaign-send-now#3</a>.\n", "summary" : "<p> Convenience function for  campaign-send-now#3 .</p>", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "apikey", "type" : "xs:string", "occurrence" : null, "description" : "" }, { "name" : "cid", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the id of the campaign to send</div>" } ], "returns" : { "type" : "item()*", "description" : "True on success" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 3, "name" : "campaign-send-now", "qname" : "mailchimp:campaign-send-now", "signature" : "($endpoint-url as xs:string, $apikey as xs:string, $cid as xs:string) as item()*", "description" : " Send a given campaign immediately. For RSS campaigns, this will \"start\" them.\n", "summary" : "<p> Send a given campaign immediately.</p>", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "endpoint-url", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> Mailchimp endpoint URL, e.g. http://us1.api.mailchimp.com/1.3/</div>" }, { "name" : "apikey", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> a valid API Key for your user account. Get by visiting <a href=\"http://admin.mailchimp.com/account/api\" target=\"_blank\">your API dashboard</a></div>" }, { "name" : "cid", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the id of the campaign to send</div>" } ], "returns" : { "type" : "item()*", "description" : "True on success" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 4, "name" : "campaign-send-test", "qname" : "mailchimp:campaign-send-test", "signature" : "($apikey as xs:string, $cid as xs:string, $test_emails as element(array), $send_type as xs:string) as item()*", "description" : " Convenience function for <a xmlns:xqdoc=\"http://www.xqdoc.org/1.0\" href=\"#campaign-send-test-5\">campaign-send-test#5</a>.\n", "summary" : "<p> Convenience function for  campaign-send-test#5 .</p>", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "apikey", "type" : "xs:string", "occurrence" : null, "description" : "" }, { "name" : "cid", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the id of the campaign to test</div>" }, { "name" : "test_emails", "type" : "element(array)", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> an array of email address to receive the test message</div>" }, { "name" : "send_type", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> optional by default (null) both formats are sent - \"html\" or \"text\" send just that format</div>" } ], "returns" : { "type" : "item()*", "description" : "True on success" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 5, "name" : "campaign-send-test", "qname" : "mailchimp:campaign-send-test", "signature" : "($endpoint-url as xs:string, $apikey as xs:string, $cid as xs:string, $test_emails as element(array), $send_type as xs:string) as item()*", "description" : " Send a test of this campaign to the provided email address.\n", "summary" : "<p> Send a test of this campaign to the provided email address.</p>", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "endpoint-url", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> Mailchimp endpoint URL, e.g. http://us1.api.mailchimp.com/1.3/</div>" }, { "name" : "apikey", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> a valid API Key for your user account. Get by visiting <a href=\"http://admin.mailchimp.com/account/api\" target=\"_blank\">your API dashboard</a></div>" }, { "name" : "cid", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the id of the campaign to test</div>" }, { "name" : "test_emails", "type" : "element(array)", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> an array of email address to receive the test message</div>" }, { "name" : "send_type", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> optional by default (null) both formats are sent - \"html\" or \"text\" send just that format</div>" } ], "returns" : { "type" : "item()*", "description" : "True on success" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 3, "name" : "campaign-share-report", "qname" : "mailchimp:campaign-share-report", "signature" : "($apikey as xs:string, $cid as xs:string, $opts as element(array)) as item()*", "description" : " Convenience function for <a xmlns:xqdoc=\"http://www.xqdoc.org/1.0\" href=\"#campaign-share-report-4\">campaign-share-report#4</a>.\n", "summary" : "<p> Convenience function for  campaign-share-report#4 .</p>", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "apikey", "type" : "xs:string", "occurrence" : null, "description" : "" }, { "name" : "cid", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the campaign id to share a report for (can be gathered using <a href=\"#campaigns-4\">campaigns()</a>)</div>" }, { "name" : "opts", "type" : "element(array)", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> s optional various parameters which can be used to configure the shared report</div>" } ], "returns" : { "type" : "item()*", "description" : "Struct containing details for the shared report" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 4, "name" : "campaign-share-report", "qname" : "mailchimp:campaign-share-report", "signature" : "($endpoint-url as xs:string, $apikey as xs:string, $cid as xs:string, $opts as element(array)) as item()*", "description" : " Get the URL to a customized <a xmlns:xqdoc=\"http://www.xqdoc.org/1.0\" href=\"http://eepurl.com/gKmL\" target=\"_blank\">VIP Report</a> for the specified campaign and optionally send an email to someone with links to it. Note subsequent calls will overwrite anything already set for the same campign (eg, the password).\n", "summary" : "<p> Get the URL to a customized  VIP Report  for the specified campaign and optionally send an email to someone with links to it.</p>", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "endpoint-url", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> Mailchimp endpoint URL, e.g. http://us1.api.mailchimp.com/1.3/</div>" }, { "name" : "apikey", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> a valid API Key for your user account. Get by visiting <a href=\"http://admin.mailchimp.com/account/api\" target=\"_blank\">your API dashboard</a></div>" }, { "name" : "cid", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the campaign id to share a report for (can be gathered using <a href=\"#campaigns-4\">campaigns()</a>)</div>" }, { "name" : "opts", "type" : "element(array)", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> s optional various parameters which can be used to configure the shared report</div>" } ], "returns" : { "type" : "item()*", "description" : "Struct containing details for the shared report" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 4, "name" : "campaign-soft-bounces", "qname" : "mailchimp:campaign-soft-bounces", "signature" : "($apikey as xs:string, $cid as xs:string, $start as xs:integer, $limit as xs:integer) as item()*", "description" : " Convenience function for <a xmlns:xqdoc=\"http://www.xqdoc.org/1.0\" href=\"#campaign-soft-bounces-5\">campaign-soft-bounces#5</a>.\n", "summary" : "<p> Convenience function for  campaign-soft-bounces#5 .</p>", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "apikey", "type" : "xs:string", "occurrence" : null, "description" : "" }, { "name" : "cid", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the campaign id to pull bounces for (can be gathered using <a href=\"#campaigns-4\">campaigns()</a>)</div>" }, { "name" : "start", "type" : "xs:integer", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> art optional for large data sets, the page number to start at - defaults to 1st page of data (page 0)</div>" }, { "name" : "limit", "type" : "xs:integer", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> mit optional for large data sets, the number of results to return - defaults to 1000, upper limit set at 15000</div>" } ], "returns" : { "type" : "item()*", "description" : "A total of all soft bounced emails and the specific emails for this page" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 5, "name" : "campaign-soft-bounces", "qname" : "mailchimp:campaign-soft-bounces", "signature" : "($endpoint-url as xs:string, $apikey as xs:string, $cid as xs:string, $start as xs:integer, $limit as xs:integer) as item()*", "description" : " <strong xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">DEPRECATED</strong> Get all email addresses with Soft Bounces for a given campaign the email address that bounced.\n", "summary" : "<p>  DEPRECATED  Get all email addresses with Soft Bounces for a given campaign the email address that bounced.</p>", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "endpoint-url", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> Mailchimp endpoint URL, e.g. http://us1.api.mailchimp.com/1.3/</div>" }, { "name" : "apikey", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> a valid API Key for your user account. Get by visiting <a href=\"http://admin.mailchimp.com/account/api\" target=\"_blank\">your API dashboard</a></div>" }, { "name" : "cid", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the campaign id to pull bounces for (can be gathered using <a href=\"#campaigns-4\">campaigns()</a>)</div>" }, { "name" : "start", "type" : "xs:integer", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> art optional for large data sets, the page number to start at - defaults to 1st page of data (page 0)</div>" }, { "name" : "limit", "type" : "xs:integer", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> mit optional for large data sets, the number of results to return - defaults to 1000, upper limit set at 15000</div>" } ], "returns" : { "type" : "item()*", "description" : "A total of all soft bounced emails and the specific emails for this page" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 2, "name" : "campaign-stats", "qname" : "mailchimp:campaign-stats", "signature" : "($apikey as xs:string, $cid as xs:string) as item()*", "description" : " Convenience function for <a xmlns:xqdoc=\"http://www.xqdoc.org/1.0\" href=\"#campaign-stats-3\">campaign-stats#3</a>.\n", "summary" : "<p> Convenience function for  campaign-stats#3 .</p>", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "apikey", "type" : "xs:string", "occurrence" : null, "description" : "" }, { "name" : "cid", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the campaign id to pull stats for (can be gathered using <a href=\"#campaigns-4\">campaigns()</a>)</div>" } ], "returns" : { "type" : "item()*", "description" : "array struct of the statistics for this campaign" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 3, "name" : "campaign-stats", "qname" : "mailchimp:campaign-stats", "signature" : "($endpoint-url as xs:string, $apikey as xs:string, $cid as xs:string) as item()*", "description" : " Given a list and a campaign, get all the relevant campaign statistics (opens, bounces, clicks, etc.)\n", "summary" : "<p> Given a list and a campaign, get all the relevant campaign statistics (opens, bounces, clicks, etc.</p>", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "endpoint-url", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> Mailchimp endpoint URL, e.g. http://us1.api.mailchimp.com/1.3/</div>" }, { "name" : "apikey", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> a valid API Key for your user account. Get by visiting <a href=\"http://admin.mailchimp.com/account/api\" target=\"_blank\">your API dashboard</a></div>" }, { "name" : "cid", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the campaign id to pull stats for (can be gathered using <a href=\"#campaigns-4\">campaigns()</a>)</div>" } ], "returns" : { "type" : "item()*", "description" : "array struct of the statistics for this campaign" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 2, "name" : "campaign-template-content", "qname" : "mailchimp:campaign-template-content", "signature" : "($apikey as xs:string, $cid as xs:string) as item()*", "description" : " Convenience function for <a xmlns:xqdoc=\"http://www.xqdoc.org/1.0\" href=\"#campaign-template-content-3\">campaign-template-content#3</a>.\n", "summary" : "<p> Convenience function for  campaign-template-content#3 .</p>", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "apikey", "type" : "xs:string", "occurrence" : null, "description" : "" }, { "name" : "cid", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the campaign id to get content for (can be gathered using <a href=\"#campaigns-4\">campaigns()</a>)</div>" } ], "returns" : { "type" : "item()*", "description" : "Array containing all content section for the campaign -" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 3, "name" : "campaign-template-content", "qname" : "mailchimp:campaign-template-content", "signature" : "($endpoint-url as xs:string, $apikey as xs:string, $cid as xs:string) as item()*", "description" : " Get the HTML template content sections for a campaign. Note that this <strong xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">will</strong> return very jagged, non-standard results based on the template a campaign is using. You only want to use this if you want to allow editing template sections in your applicaton.\n", "summary" : "<p> Get the HTML template content sections for a campaign.</p>", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "endpoint-url", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> Mailchimp endpoint URL, e.g. http://us1.api.mailchimp.com/1.3/</div>" }, { "name" : "apikey", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> a valid API Key for your user account. Get by visiting <a href=\"http://admin.mailchimp.com/account/api\" target=\"_blank\">your API dashboard</a></div>" }, { "name" : "cid", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the campaign id to get content for (can be gathered using <a href=\"#campaigns-4\">campaigns()</a>)</div>" } ], "returns" : { "type" : "item()*", "description" : "Array containing all content section for the campaign -" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 2, "name" : "campaign-unschedule", "qname" : "mailchimp:campaign-unschedule", "signature" : "($apikey as xs:string, $cid as xs:string) as item()*", "description" : " Convenience function for <a xmlns:xqdoc=\"http://www.xqdoc.org/1.0\" href=\"#campaign-unschedule-3\">campaign-unschedule#3</a>.\n", "summary" : "<p> Convenience function for  campaign-unschedule#3 .</p>", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "apikey", "type" : "xs:string", "occurrence" : null, "description" : "" }, { "name" : "cid", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the id of the campaign to unschedule</div>" } ], "returns" : { "type" : "item()*", "description" : "True on success" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 3, "name" : "campaign-unschedule", "qname" : "mailchimp:campaign-unschedule", "signature" : "($endpoint-url as xs:string, $apikey as xs:string, $cid as xs:string) as item()*", "description" : " Unschedule a campaign that is scheduled to be sent in the future.\n", "summary" : "<p> Unschedule a campaign that is scheduled to be sent in the future.</p>", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "endpoint-url", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> Mailchimp endpoint URL, e.g. http://us1.api.mailchimp.com/1.3/</div>" }, { "name" : "apikey", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> a valid API Key for your user account. Get by visiting <a href=\"http://admin.mailchimp.com/account/api\" target=\"_blank\">your API dashboard</a></div>" }, { "name" : "cid", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the id of the campaign to unschedule</div>" } ], "returns" : { "type" : "item()*", "description" : "True on success" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 4, "name" : "campaign-unsubscribes", "qname" : "mailchimp:campaign-unsubscribes", "signature" : "($apikey as xs:string, $cid as xs:string, $start as xs:integer, $limit as xs:integer) as item()*", "description" : " Convenience function for <a xmlns:xqdoc=\"http://www.xqdoc.org/1.0\" href=\"#campaign-unsubscribes-5\">campaign-unsubscribes#5</a>.\n", "summary" : "<p> Convenience function for  campaign-unsubscribes#5 .</p>", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "apikey", "type" : "xs:string", "occurrence" : null, "description" : "" }, { "name" : "cid", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the campaign id to pull bounces for (can be gathered using <a href=\"#campaigns-4\">campaigns()</a>)</div>" }, { "name" : "start", "type" : "xs:integer", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> art optional for large data sets, the page number to start at - defaults to 1st page of data (page 0)</div>" }, { "name" : "limit", "type" : "xs:integer", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> mit optional for large data sets, the number of results to return - defaults to 1000, upper limit set at 15000</div>" } ], "returns" : { "type" : "item()*", "description" : "array email addresses that unsubscribed from this campaign along with reasons, if given array a total of all unsubscribed emails and the specific emails for this page" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 5, "name" : "campaign-unsubscribes", "qname" : "mailchimp:campaign-unsubscribes", "signature" : "($endpoint-url as xs:string, $apikey as xs:string, $cid as xs:string, $start as xs:integer, $limit as xs:integer) as item()*", "description" : " Get all unsubscribed email addresses for a given campaign.\n", "summary" : "<p> Get all unsubscribed email addresses for a given campaign.</p>", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "endpoint-url", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> Mailchimp endpoint URL, e.g. http://us1.api.mailchimp.com/1.3/</div>" }, { "name" : "apikey", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> a valid API Key for your user account. Get by visiting <a href=\"http://admin.mailchimp.com/account/api\" target=\"_blank\">your API dashboard</a></div>" }, { "name" : "cid", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the campaign id to pull bounces for (can be gathered using <a href=\"#campaigns-4\">campaigns()</a>)</div>" }, { "name" : "start", "type" : "xs:integer", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> art optional for large data sets, the page number to start at - defaults to 1st page of data (page 0)</div>" }, { "name" : "limit", "type" : "xs:integer", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> mit optional for large data sets, the number of results to return - defaults to 1000, upper limit set at 15000</div>" } ], "returns" : { "type" : "item()*", "description" : "array email addresses that unsubscribed from this campaign along with reasons, if given array a total of all unsubscribed emails and the specific emails for this page" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 4, "name" : "campaign-update", "qname" : "mailchimp:campaign-update", "signature" : "($apikey as xs:string, $cid as xs:string, $name as xs:string, $value as item()) as item()*", "description" : " Convenience function for <a xmlns:xqdoc=\"http://www.xqdoc.org/1.0\" href=\"#campaign-update-5\">campaign-update#5</a>.\n", "summary" : "<p> Convenience function for  campaign-update#5 .</p>", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "apikey", "type" : "xs:string", "occurrence" : null, "description" : "" }, { "name" : "cid", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the Campaign Id to update</div>" }, { "name" : "name", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the parameter name (see <a href=\"#campaign-create-6\">campaigncreate()</a>). For items in the <strong>options</strong> array, this will be that parameter's name (subject, from_email, etc.). Additional parameters will be that option name (content, segment_opts). \"type_opts\" will be the name of the type - rss, auto, trans, etc.</div>" }, { "name" : "value", "type" : "item()", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> e an appropriate value for the parameter ( see <a href=\"#campaign-create-6\">campaigncreate()</a>). For items in the <strong>options</strong> array, this will be that parameter's value. For additional parameters, this is the same value passed to them.</div>" } ], "returns" : { "type" : "item()*", "description" : "boolean true if the update succeeds, otherwise an error will be thrown" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 5, "name" : "campaign-update", "qname" : "mailchimp:campaign-update", "signature" : "($endpoint-url as xs:string, $apikey as xs:string, $cid as xs:string, $name as xs:string, $value as item()) as item()*", "description" : " Update just about any setting for a campaign that has <em xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">not</em> been sent. See <a xmlns:xqdoc=\"http://www.xqdoc.org/1.0\" href=\"#campaign-create-6\">campaigncreate()</a> for details.\n Caveats: <br xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n  <ul xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">\n    <li>If you set list_id, all segmentation options will be deleted and must be re-added.</li>\n    <li>If you set template_id, you need to follow that up by setting it's 'content'</li>\n    <li>If you set segment_opts, you should have tested your options against <a href=\"#campaign-segment-test-3\">campaign-segment-test()</a> as <a href=\"#campaign-update-4\">campaign-update()</a> will not allow you to set a segment that includes no members.</li>\n  </ul>.\n", "summary" : "<p> Update just about any setting for a campaign that has  not  been sent.</p>", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "endpoint-url", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> Mailchimp endpoint URL, e.g. http://us1.api.mailchimp.com/1.3/</div>" }, { "name" : "apikey", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> a valid API Key for your user account. Get by visiting <a href=\"http://admin.mailchimp.com/account/api\" target=\"_blank\">your API dashboard</a></div>" }, { "name" : "cid", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the Campaign Id to update</div>" }, { "name" : "name", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the parameter name ( see <a href=\"#campaign-create-6\">campaigncreate()</a>). For items in the <strong>options</strong> array, this will be that parameter's name (subject, from_email, etc.). Additional parameters will be that option name (content, segment_opts). \"type_opts\" will be the name of the type - rss, auto, trans, etc.</div>" }, { "name" : "value", "type" : "item()", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> An appropriate value for the parameter ( see <a href=\"#campaign-create-6\">campaigncreate()</a>). For items in the <strong>options</strong> array, this will be that parameter's value. For additional parameters, this is the same value passed to them.</div>" } ], "returns" : { "type" : "item()*", "description" : "True if the update succeeds, otherwise an error will be thrown" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 2, "name" : "campaigns-for-email", "qname" : "mailchimp:campaigns-for-email", "signature" : "($apikey as xs:string, $email_address as xs:string) as item()*", "description" : " Convenience function for <a xmlns:xqdoc=\"http://www.xqdoc.org/1.0\" href=\"#campaigns-for-email-3\">campaigns-for-email#3</a>.\n", "summary" : "<p> Convenience function for  campaigns-for-email#3 .</p>", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "apikey", "type" : "xs:string", "occurrence" : null, "description" : "" }, { "name" : "email_address", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the email address to unsubscribe OR the email \"id\" returned from listMemberInfo, Webhooks, and Campaigns</div>" } ], "returns" : { "type" : "item()*", "description" : "An array of campaign_ids the member received" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 3, "name" : "campaigns-for-email", "qname" : "mailchimp:campaigns-for-email", "signature" : "($endpoint-url as xs:string, $apikey as xs:string, $email_address as xs:string) as item()*", "description" : " Retrieve all Campaigns Ids a member was sent.\n", "summary" : "<p> Retrieve all Campaigns Ids a member was sent.</p>", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "endpoint-url", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> Mailchimp endpoint URL, e.g. http://us1.api.mailchimp.com/1.3/</div>" }, { "name" : "apikey", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> a valid API Key for your user account. Get by visiting <a href=\"http://admin.mailchimp.com/account/api\" target=\"_blank\">your API dashboard</a></div>" }, { "name" : "email_address", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the email address to unsubscribe OR the email \"id\" returned from listMemberInfo, Webhooks, and Campaigns</div>" } ], "returns" : { "type" : "item()*", "description" : "An array of campaign_ids the member received" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 4, "name" : "campaigns", "qname" : "mailchimp:campaigns", "signature" : "($apikey as xs:string, $filters as element(array), $start as xs:integer, $limit as xs:integer) as item()*", "description" : " Convenience function for <a xmlns:xqdoc=\"http://www.xqdoc.org/1.0\" href=\"#campaigns-5\">campaigns#5</a>.\n", "summary" : "<p> Convenience function for  campaigns#5 .</p>", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "apikey", "type" : "xs:string", "occurrence" : null, "description" : "" }, { "name" : "filters", "type" : "element(array)", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> a hash of filters to apply to this query - all are optional. See <a href=\"http://apidocs.mailchimp.com/rtfm/campaigns.func.php\">Mailchimp API documentation</a> for more information about the parameter structure.</div>" }, { "name" : "start", "type" : "xs:integer", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> optional - control paging of campaigns, start results at this campaign #, defaults to 1st page of data (page 0)</div>" }, { "name" : "limit", "type" : "xs:integer", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> optional - control paging of campaigns, number of campaigns to return with each call, defaults to 25 (max=1000)</div>" } ], "returns" : { "type" : "item()*", "description" : "An array containing a count of all matching campaigns and the specific ones for the current page (see <a xmlns:xqdoc=\"http://www.xqdoc.org/1.0\" href=\"http://apidocs.mailchimp.com/rtfm/campaigns.func.php\">Mailchimp API documentation</a> for more information about the return type)" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 5, "name" : "campaigns", "qname" : "mailchimp:campaigns", "signature" : "($endpoint-url as xs:string, $apikey as xs:string, $filters as element(array), $start as xs:integer, $limit as xs:integer) as item()*", "description" : " Get the list of campaigns and their details matching the specified filters.\n", "summary" : "<p> Get the list of campaigns and their details matching the specified filters.</p>", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "endpoint-url", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> Mailchimp endpoint URL, e.g. http://us1.api.mailchimp.com/1.3/</div>" }, { "name" : "apikey", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> a valid API Key for your user account. Get by visiting <a href=\"http://admin.mailchimp.com/account/api\" target=\"_blank\">your API dashboard</a></div>" }, { "name" : "filters", "type" : "element(array)", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> a hash of filters to apply to this query - all are optional. See <a href=\"http://apidocs.mailchimp.com/rtfm/campaigns.func.php\">Mailchimp API documentation</a> for more information about the parameter structure.</div>" }, { "name" : "start", "type" : "xs:integer", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> optional - control paging of campaigns, start results at this campaign #, defaults to 1st page of data (page 0)</div>" }, { "name" : "limit", "type" : "xs:integer", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> optional - control paging of campaigns, number of campaigns to return with each call, defaults to 25 (max=1000)</div>" } ], "returns" : { "type" : "item()*", "description" : "An array containing a count of all matching campaigns and the specific ones for the current page (see <a xmlns:xqdoc=\"http://www.xqdoc.org/1.0\" href=\"http://apidocs.mailchimp.com/rtfm/campaigns.func.php\">Mailchimp API documentation</a> for more information about the return type)" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "chimp-chatter", "qname" : "mailchimp:chimp-chatter", "signature" : "($apikey as xs:string) as item()*", "description" : " Convenience function for <a xmlns:xqdoc=\"http://www.xqdoc.org/1.0\" href=\"#chimp-chatter-2\">chimp-chatter#2</a>.\n", "summary" : "<p> Convenience function for  chimp-chatter#2 .</p>", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "apikey", "type" : "xs:string", "occurrence" : null, "description" : "" } ], "returns" : { "type" : "item()*", "description" : "An array of chatter messages and properties" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 2, "name" : "chimp-chatter", "qname" : "mailchimp:chimp-chatter", "signature" : "($endpoint-url as xs:string, $apikey as xs:string) as item()*", "description" : " Return the current Chimp Chatter messages for an account.\n", "summary" : "<p> Return the current Chimp Chatter messages for an account.</p>", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "endpoint-url", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> Mailchimp endpoint URL, e.g. http://us1.api.mailchimp.com/1.3/</div>" }, { "name" : "apikey", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> a valid API Key for your user account. Get by visiting <a href=\"http://admin.mailchimp.com/account/api\" target=\"_blank\">your API dashboard</a></div>" } ], "returns" : { "type" : "item()*", "description" : "An array of chatter messages and properties" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 2, "name" : "ecomm-order-add", "qname" : "mailchimp:ecomm-order-add", "signature" : "($apikey as xs:string, $order as element(array)) as item()*", "description" : " Convenience function for <a xmlns:xqdoc=\"http://www.xqdoc.org/1.0\" href=\"#ecomm-order-add-3\">ecomm-order-add#3</a>.\n", "summary" : "<p> Convenience function for  ecomm-order-add#3 .</p>", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "apikey", "type" : "xs:string", "occurrence" : null, "description" : "" }, { "name" : "order", "type" : "element(array)", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> an array of information pertaining to the order that has completed. Use the following keys. See <a href=\"http://apidocs.mailchimp.com/rtfm/ecommorderadd.func.php\">Mailchimp API documentation</a> for more information about the parameter structure.</div>" } ], "returns" : { "type" : "item()*", "description" : "True if the data is saved, otherwise an error is thrown." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 3, "name" : "ecomm-order-add", "qname" : "mailchimp:ecomm-order-add", "signature" : "($endpoint-url as xs:string, $apikey as xs:string, $order as element(array)) as item()*", "description" : " Import Ecommerce Order Information to be used for Segmentation.\n", "summary" : "<p> Import Ecommerce Order Information to be used for Segmentation.</p>", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "endpoint-url", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> Mailchimp endpoint URL, e.g. http://us1.api.mailchimp.com/1.3/</div>" }, { "name" : "apikey", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> a valid API Key for your user account. Get by visiting <a href=\"http://admin.mailchimp.com/account/api\" target=\"_blank\">your API dashboard</a></div>" }, { "name" : "order", "type" : "element(array)", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> an array of information pertaining to the order that has completed. Use the following keys. See <a href=\"http://apidocs.mailchimp.com/rtfm/ecommorderadd.func.php\">Mailchimp API documentation</a> for more information about the parameter structure.</div>" } ], "returns" : { "type" : "item()*", "description" : "True if the data is saved, otherwise an error is thrown." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 3, "name" : "ecomm-order-del", "qname" : "mailchimp:ecomm-order-del", "signature" : "($apikey as xs:string, $store_id as xs:string, $order_id as xs:string) as item()*", "description" : " Convenience function for <a xmlns:xqdoc=\"http://www.xqdoc.org/1.0\" href=\"#ecomm-order-del-4\">ecomm-order-del#4</a>.\n", "summary" : "<p> Convenience function for  ecomm-order-del#4 .</p>", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "apikey", "type" : "xs:string", "occurrence" : null, "description" : "" }, { "name" : "store_id", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the store id the order belongs to</div>" }, { "name" : "order_id", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the order id (generated by the store) to delete</div>" } ], "returns" : { "type" : "item()*", "description" : "True if an order is deleted, otherwise an error is thrown." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 4, "name" : "ecomm-order-del", "qname" : "mailchimp:ecomm-order-del", "signature" : "($endpoint-url as xs:string, $apikey as xs:string, $store_id as xs:string, $order_id as xs:string) as item()*", "description" : " Delete Ecommerce Order Information used for segmentation. This will generally be used by ecommerce package plugins <a xmlns:xqdoc=\"http://www.xqdoc.org/1.0\" href=\"/plugins/ecomm360.phtml\">that we provide</a> or by 3rd part system developers.\n", "summary" : "<p> Delete Ecommerce Order Information used for segmentation.</p>", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "endpoint-url", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> Mailchimp endpoint URL, e.g. http://us1.api.mailchimp.com/1.3/</div>" }, { "name" : "apikey", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> a valid API Key for your user account. Get by visiting <a href=\"http://admin.mailchimp.com/account/api\" target=\"_blank\">your API dashboard</a></div>" }, { "name" : "store_id", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the store id the order belongs to</div>" }, { "name" : "order_id", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the order id (generated by the store) to delete</div>" } ], "returns" : { "type" : "item()*", "description" : "True if an order is deleted, otherwise an error is thrown." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 4, "name" : "ecomm-orders", "qname" : "mailchimp:ecomm-orders", "signature" : "($apikey as xs:string, $start as xs:integer, $limit as xs:integer, $since as xs:string) as item()*", "description" : " Convenience function for <a xmlns:xqdoc=\"http://www.xqdoc.org/1.0\" href=\"#ecomm-orders-5\">ecomm-orders#5</a>.\n", "summary" : "<p> Convenience function for  ecomm-orders#5 .</p>", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "apikey", "type" : "xs:string", "occurrence" : null, "description" : "" }, { "name" : "start", "type" : "xs:integer", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> optional for large data sets, the page number to start at - defaults to 1st page of data (page 0)</div>" }, { "name" : "limit", "type" : "xs:integer", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> optional for large data sets, the number of results to return - defaults to 100, upper limit set at 500</div>" }, { "name" : "since", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> optional pull only messages since this time - use YYYY-MM-DD HH:II:SS format in <strong>GMT</strong></div>" } ], "returns" : { "type" : "item()*", "description" : "Array the total matching orders and the specific orders for the requested page" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 5, "name" : "ecomm-orders", "qname" : "mailchimp:ecomm-orders", "signature" : "($endpoint-url as xs:string, $apikey as xs:string, $start as xs:integer, $limit as xs:integer, $since as xs:string) as item()*", "description" : " Retrieve the Ecommerce Orders for an account.\n", "summary" : "<p> Retrieve the Ecommerce Orders for an account.</p>", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "endpoint-url", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> Mailchimp endpoint URL, e.g. http://us1.api.mailchimp.com/1.3/</div>" }, { "name" : "apikey", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> a valid API Key for your user account. Get by visiting <a href=\"http://admin.mailchimp.com/account/api\" target=\"_blank\">your API dashboard</a></div>" }, { "name" : "start", "type" : "xs:integer", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> optional for large data sets, the page number to start at - defaults to 1st page of data (page 0)</div>" }, { "name" : "limit", "type" : "xs:integer", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> optional for large data sets, the number of results to return - defaults to 100, upper limit set at 500</div>" }, { "name" : "since", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> optional pull only messages since this time - use YYYY-MM-DD HH:II:SS format in <strong>GMT</strong></div>" } ], "returns" : { "type" : "item()*", "description" : "Array the total matching orders and the specific orders for the requested page" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 3, "name" : "folder-add", "qname" : "mailchimp:folder-add", "signature" : "($apikey as xs:string, $name as xs:string, $type as xs:string) as item()*", "description" : " Convenience function for <a xmlns:xqdoc=\"http://www.xqdoc.org/1.0\" href=\"#folder-add-4\">folder-add#4</a>.\n", "summary" : "<p> Convenience function for  folder-add#4 .</p>", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "apikey", "type" : "xs:string", "occurrence" : null, "description" : "" }, { "name" : "name", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> a unique name for a folder (max 100 bytes)</div>" }, { "name" : "type", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> optional the type of folder to create - either \"campaign\" or \"autoresponder\". Defaults to \"campaign\"</div>" } ], "returns" : { "type" : "item()*", "description" : "The folder_id of the newly created folder." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 4, "name" : "folder-add", "qname" : "mailchimp:folder-add", "signature" : "($endpoint-url as xs:string, $apikey as xs:string, $name as xs:string, $type as xs:string) as item()*", "description" : " Add a new folder to file campaigns or autoresponders in.\n", "summary" : "<p> Add a new folder to file campaigns or autoresponders in.</p>", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "endpoint-url", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> Mailchimp endpoint URL, e.g. http://us1.api.mailchimp.com/1.3/</div>" }, { "name" : "apikey", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> a valid API Key for your user account. Get by visiting <a href=\"http://admin.mailchimp.com/account/api\" target=\"_blank\">your API dashboard</a></div>" }, { "name" : "name", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> a unique name for a folder (max 100 bytes)</div>" }, { "name" : "type", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> optional the type of folder to create - either \"campaign\" or \"autoresponder\". Defaults to \"campaign\"</div>" } ], "returns" : { "type" : "item()*", "description" : "The folder_id of the newly created folder." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 3, "name" : "folder-del", "qname" : "mailchimp:folder-del", "signature" : "($apikey as xs:string, $fid as xs:integer, $type as xs:string) as item()*", "description" : " Convenience function for <a xmlns:xqdoc=\"http://www.xqdoc.org/1.0\" href=\"#folder-del-4\">folder-del#4</a>.\n", "summary" : "<p> Convenience function for  folder-del#4 .</p>", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "apikey", "type" : "xs:string", "occurrence" : null, "description" : "" }, { "name" : "fid", "type" : "xs:integer", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the folder id to update - retrieve from folders()</div>" }, { "name" : "type", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> optional the type of folder to create - either \"campaign\" or \"autoresponder\". Defaults to \"campaign\"</div>" } ], "returns" : { "type" : "item()*", "description" : "True if the delete worked, otherwise an exception is thrown" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 4, "name" : "folder-del", "qname" : "mailchimp:folder-del", "signature" : "($endpoint-url as xs:string, $apikey as xs:string, $fid as xs:integer, $type as xs:string) as item()*", "description" : " Delete a campaign or autoresponder folder. Note that this will simply make campaigns in the folder appear unfiled, they are not removed.\n", "summary" : "<p> Delete a campaign or autoresponder folder.</p>", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "endpoint-url", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> Mailchimp endpoint URL, e.g. http://us1.api.mailchimp.com/1.3/</div>" }, { "name" : "apikey", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> a valid API Key for your user account. Get by visiting <a href=\"http://admin.mailchimp.com/account/api\" target=\"_blank\">your API dashboard</a></div>" }, { "name" : "fid", "type" : "xs:integer", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the folder id to update - retrieve from folders()</div>" }, { "name" : "type", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> optional the type of folder to create - either \"campaign\" or \"autoresponder\". Defaults to \"campaign\"</div>" } ], "returns" : { "type" : "item()*", "description" : "True if the delete worked, otherwise an exception is thrown" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 4, "name" : "folder-update", "qname" : "mailchimp:folder-update", "signature" : "($apikey as xs:string, $fid as xs:integer, $name as xs:string, $type as xs:string) as item()*", "description" : " Convenience function for <a xmlns:xqdoc=\"http://www.xqdoc.org/1.0\" href=\"#folder-update-5\">folder-update#5</a>.\n", "summary" : "<p> Convenience function for  folder-update#5 .</p>", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "apikey", "type" : "xs:string", "occurrence" : null, "description" : "" }, { "name" : "fid", "type" : "xs:integer", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the folder id to update - retrieve from folders()</div>" }, { "name" : "name", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> a new, unique name for the folder (max 100 bytes)</div>" }, { "name" : "type", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> optional the type of folder to create - either \"campaign\" or \"autoresponder\". Defaults to \"campaign\"</div>" } ], "returns" : { "type" : "item()*", "description" : "True if the update worked, otherwise an exception is thrown" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 5, "name" : "folder-update", "qname" : "mailchimp:folder-update", "signature" : "($endpoint-url as xs:string, $apikey as xs:string, $fid as xs:integer, $name as xs:string, $type as xs:string) as item()*", "description" : " Update the name of a folder for campaigns or autoresponders.\n", "summary" : "<p> Update the name of a folder for campaigns or autoresponders.</p>", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "endpoint-url", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> Mailchimp endpoint URL, e.g. http://us1.api.mailchimp.com/1.3/</div>" }, { "name" : "apikey", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> a valid API Key for your user account. Get by visiting <a href=\"http://admin.mailchimp.com/account/api\" target=\"_blank\">your API dashboard</a></div>" }, { "name" : "fid", "type" : "xs:integer", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the folder id to update - retrieve from folders()</div>" }, { "name" : "name", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> a new, unique name for the folder (max 100 bytes)</div>" }, { "name" : "type", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> optional the type of folder to create - either \"campaign\" or \"autoresponder\". Defaults to \"campaign\"</div>" } ], "returns" : { "type" : "item()*", "description" : "True if the update worked, otherwise an exception is thrown" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 2, "name" : "folders", "qname" : "mailchimp:folders", "signature" : "($apikey as xs:string, $type as xs:string) as item()*", "description" : " Convenience function for <a xmlns:xqdoc=\"http://www.xqdoc.org/1.0\" href=\"#folders-3\">folders#3</a>.\n", "summary" : "<p> Convenience function for  folders#3 .</p>", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "apikey", "type" : "xs:string", "occurrence" : null, "description" : "" }, { "name" : "type", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> optional the type of folders to return - either \"campaign\" or \"autoresponder\". Defaults to \"campaign\"</div>" } ], "returns" : { "type" : "item()*", "description" : "Array of folder structs (see Returned Fields for details)" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 3, "name" : "folders", "qname" : "mailchimp:folders", "signature" : "($endpoint-url as xs:string, $apikey as xs:string, $type as xs:string) as item()*", "description" : " List all the folders for a user account.\n", "summary" : "<p> List all the folders for a user account.</p>", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "endpoint-url", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> Mailchimp endpoint URL, e.g. http://us1.api.mailchimp.com/1.3/</div>" }, { "name" : "apikey", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> a valid API Key for your user account. Get by visiting <a href=\"http://admin.mailchimp.com/account/api\" target=\"_blank\">your API dashboard</a></div>" }, { "name" : "type", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> optional the type of folders to return - either \"campaign\" or \"autoresponder\". Defaults to \"campaign\"</div>" } ], "returns" : { "type" : "item()*", "description" : "Array of folder structs (see Returned Fields for details)" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 3, "name" : "generate-text", "qname" : "mailchimp:generate-text", "signature" : "($apikey as xs:string, $type as xs:string, $content as item()) as item()*", "description" : " Convenience function for <a xmlns:xqdoc=\"http://www.xqdoc.org/1.0\" href=\"#generate-text-4\">generate-text#4</a>.\n", "summary" : "<p> Convenience function for  generate-text#4 .</p>", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "apikey", "type" : "xs:string", "occurrence" : null, "description" : "" }, { "name" : "type", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The type of content to parse. Must be one of: \"html\", \"template\", \"url\", \"cid\" (Campaign Id), or \"tid\" (Template Id)</div>" }, { "name" : "content", "type" : "item()", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The content to use. For \"html\" expects a single string value, \"template\" expects an array like you send to campaignCreate, \"url\" expects a valid &amp; public URL to pull from, \"cid\" expects a valid Campaign Id, and \"tid\" expects a valid Template Id on your account.</div>" } ], "returns" : { "type" : "item()*", "description" : "The content pass in converted to text." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 4, "name" : "generate-text", "qname" : "mailchimp:generate-text", "signature" : "($endpoint-url as xs:string, $apikey as xs:string, $type as xs:string, $content as item()) as item()*", "description" : " Have HTML content auto-converted to a text-only format. You can send: plain HTML, an array of Template content, an existing Campaign Id, or an existing Template Id. Note that this will <b xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">not</b> save anything to or update any of your lists, campaigns, or templates.\n", "summary" : "<p> Have HTML content auto-converted to a text-only format.</p>", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "endpoint-url", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> Mailchimp endpoint URL, e.g. http://us1.api.mailchimp.com/1.3/</div>" }, { "name" : "apikey", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> a valid API Key for your user account. Get by visiting <a href=\"http://admin.mailchimp.com/account/api\" target=\"_blank\">your API dashboard</a></div>" }, { "name" : "type", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The type of content to parse. Must be one of: \"html\", \"template\", \"url\", \"cid\" (Campaign Id), or \"tid\" (Template Id)</div>" }, { "name" : "content", "type" : "item()", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The content to use. For \"html\" expects a single string value, \"template\" expects an array like you send to campaignCreate, \"url\" expects a valid &amp; public URL to pull from, \"cid\" expects a valid Campaign Id, and \"tid\" expects a valid Template Id on your account.</div>" } ], "returns" : { "type" : "item()*", "description" : "The content pass in converted to text." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "get-account-details", "qname" : "mailchimp:get-account-details", "signature" : "($apikey as xs:string) as item()*", "description" : " Convenience function for <a xmlns:xqdoc=\"http://www.xqdoc.org/1.0\" href=\"#get-account-details-2\">get-account-details#2</a>.\n", "summary" : "<p> Convenience function for  get-account-details#2 .</p>", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "apikey", "type" : "xs:string", "occurrence" : null, "description" : "" } ], "returns" : { "type" : "item()*", "description" : "array containing the details for the account tied to this API Key" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 2, "name" : "get-account-details", "qname" : "mailchimp:get-account-details", "signature" : "($endpoint-url as xs:string, $apikey as xs:string) as item()*", "description" : " Retrieve lots of account information including payments made, plan info, some account stats, installed modules, contact info, and more.\n", "summary" : "<p> Retrieve lots of account information including payments made, plan info, some account stats, installed modules, contact info, and more.</p>", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "endpoint-url", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> Mailchimp endpoint URL, e.g. http://us1.api.mailchimp.com/1.3/</div>" }, { "name" : "apikey", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> a valid API Key for your user account. Get by visiting <a href=\"http://admin.mailchimp.com/account/api\" target=\"_blank\">your API dashboard</a></div>" } ], "returns" : { "type" : "item()*", "description" : "Array containing the details for the account tied to this API Key" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 3, "name" : "inline-css", "qname" : "mailchimp:inline-css", "signature" : "($apikey as xs:string, $html as xs:string, $strip_css as xs:boolean) as item()*", "description" : " Convenience function for <a xmlns:xqdoc=\"http://www.xqdoc.org/1.0\" href=\"#inline-css-4\">inline-css#4</a>.\n", "summary" : "<p> Convenience function for  inline-css#4 .</p>", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "apikey", "type" : "xs:string", "occurrence" : null, "description" : "" }, { "name" : "html", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> Your HTML content</div>" }, { "name" : "strip_css", "type" : "xs:boolean", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> optional Whether you want the CSS &amp;lt;style&amp;gt; tags stripped from the returned document. Defaults to false.</div>" } ], "returns" : { "type" : "item()*", "description" : "Your HTML content with all CSS inlined, just like if we sent it." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 4, "name" : "inline-css", "qname" : "mailchimp:inline-css", "signature" : "($endpoint-url as xs:string, $apikey as xs:string, $html as xs:string, $strip_css as xs:boolean) as item()*", "description" : " Send your HTML content to have the CSS inlined and optionally remove the original styles.\n", "summary" : "<p> Send your HTML content to have the CSS inlined and optionally remove the original styles.</p>", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "endpoint-url", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> Mailchimp endpoint URL, e.g. http://us1.api.mailchimp.com/1.3/</div>" }, { "name" : "apikey", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> a valid API Key for your user account. Get by visiting <a href=\"http://admin.mailchimp.com/account/api\" target=\"_blank\">your API dashboard</a></div>" }, { "name" : "html", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> Your HTML content</div>" }, { "name" : "strip_css", "type" : "xs:boolean", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> optional Whether you want the CSS &amp;lt;style&amp;gt; tags stripped from the returned document. Defaults to false.</div>" } ], "returns" : { "type" : "item()*", "description" : "Your HTML content with all CSS inlined, just like if we sent it." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 5, "name" : "list-abuse-reports", "qname" : "mailchimp:list-abuse-reports", "signature" : "($apikey as xs:string, $id as xs:string, $start as xs:integer, $limit as xs:integer, $since as xs:string) as item()*", "description" : " Convenience function for <a xmlns:xqdoc=\"http://www.xqdoc.org/1.0\" href=\"#list-abuse-reports-6\">list-abuse-reports#6</a>.\n", "summary" : "<p> Convenience function for  list-abuse-reports#6 .</p>", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "apikey", "type" : "xs:string", "occurrence" : null, "description" : "" }, { "name" : "id", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the list id to pull abuse reports for (can be gathered using lists())</div>" }, { "name" : "start", "type" : "xs:integer", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> optional for large data sets, the page number to start at - defaults to 1st page of data (page 0)</div>" }, { "name" : "limit", "type" : "xs:integer", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> optional for large data sets, the number of results to return - defaults to 500, upper limit set at 1000</div>" }, { "name" : "since", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> optional pull only messages since this time - use YYYY-MM-DD HH:II:SS format in <strong>GMT</strong></div>" } ], "returns" : { "type" : "item()*", "description" : "The total of all reports and the specific reports reports this page" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 6, "name" : "list-abuse-reports", "qname" : "mailchimp:list-abuse-reports", "signature" : "($endpoint-url as xs:string, $apikey as xs:string, $id as xs:string, $start as xs:integer, $limit as xs:integer, $since as xs:string) as item()*", "description" : " Get all email addresses that complained about a given campaign.\n", "summary" : "<p> Get all email addresses that complained about a given campaign.</p>", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "endpoint-url", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> Mailchimp endpoint URL, e.g. http://us1.api.mailchimp.com/1.3/</div>" }, { "name" : "apikey", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> a valid API Key for your user account. Get by visiting <a href=\"http://admin.mailchimp.com/account/api\" target=\"_blank\">your API dashboard</a></div>" }, { "name" : "id", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the list id to pull abuse reports for (can be gathered using lists())</div>" }, { "name" : "start", "type" : "xs:integer", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> optional for large data sets, the page number to start at - defaults to 1st page of data (page 0)</div>" }, { "name" : "limit", "type" : "xs:integer", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> optional for large data sets, the number of results to return - defaults to 500, upper limit set at 1000</div>" }, { "name" : "since", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> optional pull only messages since this time - use YYYY-MM-DD HH:II:SS format in <strong>GMT</strong></div>" } ], "returns" : { "type" : "item()*", "description" : "The total of all reports and the specific reports reports this page" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 2, "name" : "list-activity", "qname" : "mailchimp:list-activity", "signature" : "($apikey as xs:string, $id as xs:string) as item()*", "description" : " Convenience function for <a xmlns:xqdoc=\"http://www.xqdoc.org/1.0\" href=\"#list-activity-3\">list-activity#3</a>.\n", "summary" : "<p> Convenience function for  list-activity#3 .</p>", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "apikey", "type" : "xs:string", "occurrence" : null, "description" : "" }, { "name" : "id", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the list id to connect to. Get by calling <a href=\"#lists-4\">lists()</a></div>" } ], "returns" : { "type" : "item()*", "description" : "Array of array of daily values." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 3, "name" : "list-activity", "qname" : "mailchimp:list-activity", "signature" : "($endpoint-url as xs:string, $apikey as xs:string, $id as xs:string) as item()*", "description" : " Access up to the previous 180 days of daily detailed aggregated activity stats for a given list.\n", "summary" : "<p> Access up to the previous 180 days of daily detailed aggregated activity stats for a given list.</p>", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "endpoint-url", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> Mailchimp endpoint URL, e.g. http://us1.api.mailchimp.com/1.3/</div>" }, { "name" : "apikey", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> a valid API Key for your user account. Get by visiting <a href=\"http://admin.mailchimp.com/account/api\" target=\"_blank\">your API dashboard</a></div>" }, { "name" : "id", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the list id to connect to. Get by calling <a href=\"#lists-4\">lists()</a></div>" } ], "returns" : { "type" : "item()*", "description" : "array array of array of daily values." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 6, "name" : "list-batch-subscribe", "qname" : "mailchimp:list-batch-subscribe", "signature" : "($apikey as xs:string, $id as xs:string, $batch as element(array), $double_optin as xs:boolean, $update_existing as xs:boolean, $replace_interests as xs:boolean) as item()*", "description" : " Convenience function for <a xmlns:xqdoc=\"http://www.xqdoc.org/1.0\" href=\"#list-batch-subscribe-7\">list-batch-subscribe#7</a>.\n", "summary" : "<p> Convenience function for  list-batch-subscribe#7 .</p>", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "apikey", "type" : "xs:string", "occurrence" : null, "description" : "" }, { "name" : "id", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the list id to connect to. Get by calling <a href=\"#lists-4\">lists()</a></div>" }, { "name" : "batch", "type" : "element(array)", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> an array of structs for each address to import with two special keys: \"EMAIL\" for the email address, and \"EMAIL_TYPE\" for the email type option (html, text, or mobile)</div>" }, { "name" : "double_optin", "type" : "xs:boolean", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> flag to control whether to send an opt-in confirmation email - defaults to true</div>" }, { "name" : "update_existing", "type" : "xs:boolean", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> flag to control whether to update members that are already subscribed to the list or to return an error, defaults to false (return error)</div>" }, { "name" : "replace_interests", "type" : "xs:boolean", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> flag to determine whether we replace the interest groups with the updated groups provided, or we add the provided groups to the member's interest groups (optional, defaults to true)</div>" } ], "returns" : { "type" : "item()*", "description" : "Array of result counts and any errors that occurred" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 7, "name" : "list-batch-subscribe", "qname" : "mailchimp:list-batch-subscribe", "signature" : "($endpoint-url as xs:string, $apikey as xs:string, $id as xs:string, $batch as element(array), $double_optin as xs:boolean, $update_existing as xs:boolean, $replace_interests as xs:boolean) as item()*", "description" : " Subscribe a batch of email addresses to a list at once. If you are using a serialized version of the API, we strongly suggest that you only run this method as a POST request, and <em xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">not</em> a GET request. Maximum batch sizes vary based on the amount of data in each record, though you should cap them at 5k - 10k records, depending on your experience. These calls are also long, so be sure you increase your timeout values.\n", "summary" : "<p> Subscribe a batch of email addresses to a list at once.</p>", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "endpoint-url", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> Mailchimp endpoint URL, e.g. http://us1.api.mailchimp.com/1.3/</div>" }, { "name" : "apikey", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> a valid API Key for your user account. Get by visiting <a href=\"http://admin.mailchimp.com/account/api\" target=\"_blank\">your API dashboard</a></div>" }, { "name" : "id", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the list id to connect to. Get by calling <a href=\"#lists-4\">lists()</a></div>" }, { "name" : "batch", "type" : "element(array)", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> an array of structs for each address to import with two special keys: \"EMAIL\" for the email address, and \"EMAIL_TYPE\" for the email type option (html, text, or mobile)</div>" }, { "name" : "double_optin", "type" : "xs:boolean", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> flag to control whether to send an opt-in confirmation email - defaults to true</div>" }, { "name" : "update_existing", "type" : "xs:boolean", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> flag to control whether to update members that are already subscribed to the list or to return an error, defaults to false (return error)</div>" }, { "name" : "replace_interests", "type" : "xs:boolean", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> flag to determine whether we replace the interest groups with the updated groups provided, or we add the provided groups to the member's interest groups (optional, defaults to true)</div>" } ], "returns" : { "type" : "item()*", "description" : "Array of result counts and any errors that occurred" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 6, "name" : "list-batch-unsubscribe", "qname" : "mailchimp:list-batch-unsubscribe", "signature" : "($apikey as xs:string, $id as xs:string, $emails as element(array), $delete_member as xs:boolean, $send_goodbye as xs:boolean, $send_notify as xs:boolean) as item()*", "description" : " Convenience function for <a xmlns:xqdoc=\"http://www.xqdoc.org/1.0\" href=\"#list-batch-unsubscribe-7\">list-batch-unsubscribe#7</a>.\n", "summary" : "<p> Convenience function for  list-batch-unsubscribe#7 .</p>", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "apikey", "type" : "xs:string", "occurrence" : null, "description" : "" }, { "name" : "id", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the list id to connect to. Get by calling <a href=\"#lists-4\">lists()</a></div>" }, { "name" : "emails", "type" : "element(array)", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> array of email addresses to unsubscribe</div>" }, { "name" : "delete_member", "type" : "xs:boolean", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> flag to completely delete the member from your list instead of just unsubscribing, default to false</div>" }, { "name" : "send_goodbye", "type" : "xs:boolean", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> flag to send the goodbye email to the email addresses, defaults to true</div>" }, { "name" : "send_notify", "type" : "xs:boolean", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> flag to send the unsubscribe notification email to the address defined in the list email notification settings, defaults to false</div>" } ], "returns" : { "type" : "item()*", "description" : "Array of result counts and any errors that occurred" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 7, "name" : "list-batch-unsubscribe", "qname" : "mailchimp:list-batch-unsubscribe", "signature" : "($endpoint-url as xs:string, $apikey as xs:string, $id as xs:string, $emails as element(array), $delete_member as xs:boolean, $send_goodbye as xs:boolean, $send_notify as xs:boolean) as item()*", "description" : " Unsubscribe a batch of email addresses to a list.\n", "summary" : "<p> Unsubscribe a batch of email addresses to a list.</p>", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "endpoint-url", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> Mailchimp endpoint URL, e.g. http://us1.api.mailchimp.com/1.3/</div>" }, { "name" : "apikey", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> a valid API Key for your user account. Get by visiting <a href=\"http://admin.mailchimp.com/account/api\" target=\"_blank\">your API dashboard</a></div>" }, { "name" : "id", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the list id to connect to. Get by calling <a href=\"#lists-4\">lists()</a></div>" }, { "name" : "emails", "type" : "element(array)", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> array of email addresses to unsubscribe</div>" }, { "name" : "delete_member", "type" : "xs:boolean", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> flag to completely delete the member from your list instead of just unsubscribing, default to false</div>" }, { "name" : "send_goodbye", "type" : "xs:boolean", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> flag to send the goodbye email to the email addresses, defaults to true</div>" }, { "name" : "send_notify", "type" : "xs:boolean", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> flag to send the unsubscribe notification email to the address defined in the list email notification settings, defaults to false</div>" } ], "returns" : { "type" : "item()*", "description" : "Array of result counts and any errors that occurred" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 2, "name" : "list-clients", "qname" : "mailchimp:list-clients", "signature" : "($apikey as xs:string, $id as xs:string) as item()*", "description" : " Convenience function for <a xmlns:xqdoc=\"http://www.xqdoc.org/1.0\" href=\"#list-clients-3\">list-clients#3</a>.\n", "summary" : "<p> Convenience function for  list-clients#3 .</p>", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "apikey", "type" : "xs:string", "occurrence" : null, "description" : "" }, { "name" : "id", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the list id to connect to. Get by calling <a href=\"#lists-4\">lists()</a></div>" } ], "returns" : { "type" : "item()*", "description" : "array the desktop and mobile user agents in use on the list" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 3, "name" : "list-clients", "qname" : "mailchimp:list-clients", "signature" : "($endpoint-url as xs:string, $apikey as xs:string, $id as xs:string) as item()*", "description" : " Retrieve the clients that the list's subscribers have been tagged as being used based on user agents seen. Made possible by <a xmlns:xqdoc=\"http://www.xqdoc.org/1.0\" href=\"http://user-agent-string.info\" target=\"_blank\">user-agent-string.info</a> double penetration the percent of desktop clients in use array clients a record containing the 'client', an 'icon' image url, the 'percent' using the client, and the total 'members' represented double penetration the percent of mobile clients in use array clients a record containing the 'client', an 'icon' image url, the 'percent' using the client, and the total 'members' represented.\n", "summary" : "<p> Retrieve the clients that the list's subscribers have been tagged as being used based on user agents seen.</p>", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "endpoint-url", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> Mailchimp endpoint URL, e.g. http://us1.api.mailchimp.com/1.3/</div>" }, { "name" : "apikey", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> a valid API Key for your user account. Get by visiting <a href=\"http://admin.mailchimp.com/account/api\" target=\"_blank\">your API dashboard</a></div>" }, { "name" : "id", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the list id to connect to. Get by calling <a href=\"#lists-4\">lists()</a></div>" } ], "returns" : { "type" : "item()*", "description" : "Array the desktop and mobile user agents in use on the list" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 2, "name" : "list-growth-history", "qname" : "mailchimp:list-growth-history", "signature" : "($apikey as xs:string, $id as xs:string) as item()*", "description" : " Convenience function for <a xmlns:xqdoc=\"http://www.xqdoc.org/1.0\" href=\"#list-growth-history-3\">list-growth-history#3</a>.\n", "summary" : "<p> Convenience function for  list-growth-history#3 .</p>", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "apikey", "type" : "xs:string", "occurrence" : null, "description" : "" }, { "name" : "id", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the list id to connect to. Get by calling <a href=\"#lists-4\">lists()</a></div>" } ], "returns" : { "type" : "item()*", "description" : "Array of months and growth" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 3, "name" : "list-growth-history", "qname" : "mailchimp:list-growth-history", "signature" : "($endpoint-url as xs:string, $apikey as xs:string, $id as xs:string) as item()*", "description" : " Access the Growth History by Month for a given list.\n", "summary" : "<p> Access the Growth History by Month for a given list.</p>", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "endpoint-url", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> Mailchimp endpoint URL, e.g. http://us1.api.mailchimp.com/1.3/</div>" }, { "name" : "apikey", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> a valid API Key for your user account. Get by visiting <a href=\"http://admin.mailchimp.com/account/api\" target=\"_blank\">your API dashboard</a></div>" }, { "name" : "id", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the list id to connect to. Get by calling <a href=\"#lists-4\">lists()</a></div>" } ], "returns" : { "type" : "item()*", "description" : "Array of months and growth" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 4, "name" : "list-interest-group-add", "qname" : "mailchimp:list-interest-group-add", "signature" : "($apikey as xs:string, $id as xs:string, $group_name as xs:string, $grouping_id as xs:integer) as item()*", "description" : " Convenience function for <a xmlns:xqdoc=\"http://www.xqdoc.org/1.0\" href=\"#list-interest-group-add-5\">list-interest-group-add#5</a>.\n", "summary" : "<p> Convenience function for  list-interest-group-add#5 .</p>", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "apikey", "type" : "xs:string", "occurrence" : null, "description" : "" }, { "name" : "id", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the list id to connect to. Get by calling <a href=\"#lists-4\">lists()</a></div>" }, { "name" : "group_name", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the interest group to add - group names must be unique within a grouping</div>" }, { "name" : "grouping_id", "type" : "xs:integer", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> ouping_id The grouping to add the new group to - get using <a href=\"#list-interest-groupings-2\">list-interest-groupings()</a>. If not supplied, the first grouping on the list is used.</div>" } ], "returns" : { "type" : "item()*", "description" : "True if the request succeeds, otherwise an error will be thrown" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 5, "name" : "list-interest-group-add", "qname" : "mailchimp:list-interest-group-add", "signature" : "($endpoint-url as xs:string, $apikey as xs:string, $id as xs:string, $group_name as xs:string, $grouping_id as xs:integer) as item()*", "description" : " Add a single Interest Group - if interest groups for the List are not yet enabled, adding the first group will automatically turn them on.\n", "summary" : "<p> Add a single Interest Group - if interest groups for the List are not yet enabled, adding the first group will automatically turn them on.</p>", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "endpoint-url", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> Mailchimp endpoint URL, e.g. http://us1.api.mailchimp.com/1.3/</div>" }, { "name" : "apikey", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> a valid API Key for your user account. Get by visiting <a href=\"http://admin.mailchimp.com/account/api\" target=\"_blank\">your API dashboard</a></div>" }, { "name" : "id", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the list id to connect to. Get by calling <a href=\"#lists-4\">lists()</a></div>" }, { "name" : "group_name", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the interest group to add - group names must be unique within a grouping</div>" }, { "name" : "grouping_id", "type" : "xs:integer", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> ouping_id The grouping to add the new group to - get using <a href=\"#list-interest-groupings-2\">list-interest-groupings()</a> . If not supplied, the first grouping on the list is used.</div>" } ], "returns" : { "type" : "item()*", "description" : "bool true if the request succeeds, otherwise an error will be thrown" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 4, "name" : "list-interest-group-del", "qname" : "mailchimp:list-interest-group-del", "signature" : "($apikey as xs:string, $id as xs:string, $group_name as xs:string, $grouping_id as xs:integer) as item()*", "description" : " Convenience function for <a xmlns:xqdoc=\"http://www.xqdoc.org/1.0\" href=\"#list-interest-group-del-5\">list-interest-group-del#5</a>.\n", "summary" : "<p> Convenience function for  list-interest-group-del#5 .</p>", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "apikey", "type" : "xs:string", "occurrence" : null, "description" : "" }, { "name" : "id", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the list id to connect to. Get by calling <a href=\"#lists-4\">lists()</a></div>" }, { "name" : "group_name", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the interest group to delete</div>" }, { "name" : "grouping_id", "type" : "xs:integer", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The grouping to delete the group from - get using <a href=\"#list-interest-groupings-2\">list-interest-groupings()</a> . If not supplied, the first grouping on the list is used.</div>" } ], "returns" : { "type" : "item()*", "description" : "True if the request succeeds, otherwise an error will be thrown" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 5, "name" : "list-interest-group-del", "qname" : "mailchimp:list-interest-group-del", "signature" : "($endpoint-url as xs:string, $apikey as xs:string, $id as xs:string, $group_name as xs:string, $grouping_id as xs:integer) as item()*", "description" : " Delete a single Interest Group - if the last group for a list is deleted, this will also turn groups for the list off.\n", "summary" : "<p> Delete a single Interest Group - if the last group for a list is deleted, this will also turn groups for the list off.</p>", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "endpoint-url", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> Mailchimp endpoint URL, e.g. http://us1.api.mailchimp.com/1.3/</div>" }, { "name" : "apikey", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> a valid API Key for your user account. Get by visiting <a href=\"http://admin.mailchimp.com/account/api\" target=\"_blank\">your API dashboard</a></div>" }, { "name" : "id", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the list id to connect to. Get by calling <a href=\"#lists-4\">lists()</a></div>" }, { "name" : "group_name", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the interest group to delete</div>" }, { "name" : "grouping_id", "type" : "xs:integer", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The grouping to delete the group from - get using <a href=\"#list-interest-groupings-2\">list-interest-groupings()</a> . If not supplied, the first grouping on the list is used.</div>" } ], "returns" : { "type" : "item()*", "description" : "True if the request succeeds, otherwise an error will be thrown" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 5, "name" : "list-interest-group-update", "qname" : "mailchimp:list-interest-group-update", "signature" : "($apikey as xs:string, $id as xs:string, $old_name as xs:string, $new_name as xs:string, $grouping_id as xs:integer) as item()*", "description" : " Convenience function for <a xmlns:xqdoc=\"http://www.xqdoc.org/1.0\" href=\"#list-interest-group-update-6\">list-interest-group-update#6</a>.\n", "summary" : "<p> Convenience function for  list-interest-group-update#6 .</p>", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "apikey", "type" : "xs:string", "occurrence" : null, "description" : "" }, { "name" : "id", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the list id to connect to. Get by calling <a href=\"#lists-4\">lists()</a></div>" }, { "name" : "old_name", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the interest group name to be changed</div>" }, { "name" : "new_name", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the new interest group name to be set</div>" }, { "name" : "grouping_id", "type" : "xs:integer", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> ouping_id The grouping to delete the group from - get using <a href=\"#list-interest-groupings-2\">list-interest-groupings()</a> . If not supplied, the first grouping on the list is used.</div>" } ], "returns" : { "type" : "item()*", "description" : "True if the request succeeds, otherwise an error will be thrown" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 6, "name" : "list-interest-group-update", "qname" : "mailchimp:list-interest-group-update", "signature" : "($endpoint-url as xs:string, $apikey as xs:string, $id as xs:string, $old_name as xs:string, $new_name as xs:string, $grouping_id as xs:integer) as item()*", "description" : " Change the name of an Interest Group.\n", "summary" : "<p> Change the name of an Interest Group.</p>", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "endpoint-url", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> Mailchimp endpoint URL, e.g. http://us1.api.mailchimp.com/1.3/</div>" }, { "name" : "apikey", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> a valid API Key for your user account. Get by visiting <a href=\"http://admin.mailchimp.com/account/api\" target=\"_blank\">your API dashboard</a></div>" }, { "name" : "id", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the list id to connect to. Get by calling <a href=\"#lists-4\">lists()</a></div>" }, { "name" : "old_name", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the interest group name to be changed</div>" }, { "name" : "new_name", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the new interest group name to be set</div>" }, { "name" : "grouping_id", "type" : "xs:integer", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> ouping_id The grouping to delete the group from - get using <a href=\"#list-interest-groupings-2\">list-interest-groupings()</a> . If not supplied, the first grouping on the list is used.</div>" } ], "returns" : { "type" : "item()*", "description" : "True if the request succeeds, otherwise an error will be thrown" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 5, "name" : "list-interest-grouping-add", "qname" : "mailchimp:list-interest-grouping-add", "signature" : "($apikey as xs:string, $id as xs:string, $name as xs:string, $type as xs:string, $groups as element(array)) as item()*", "description" : " Convenience function for <a xmlns:xqdoc=\"http://www.xqdoc.org/1.0\" href=\"#list-interest-grouping-add-6\">list-interest-grouping-add#6</a>.\n", "summary" : "<p> Convenience function for  list-interest-grouping-add#6 .</p>", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "apikey", "type" : "xs:string", "occurrence" : null, "description" : "" }, { "name" : "id", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the list id to connect to. Get by calling <a href=\"#lists-4\">lists()</a></div>" }, { "name" : "name", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the interest grouping to add - grouping names must be unique</div>" }, { "name" : "type", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The type of the grouping to add - one of \"checkboxes\", \"hidden\", \"dropdown\", \"radio\"</div>" }, { "name" : "groups", "type" : "element(array)", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The lists of initial group names to be added - at least 1 is required and the names must be unique within a grouping. If the number takes you over the 60 group limit, an error will be thrown.</div>" } ], "returns" : { "type" : "item()*", "description" : "The new grouping id if the request succeeds, otherwise an error will be thrown" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 6, "name" : "list-interest-grouping-add", "qname" : "mailchimp:list-interest-grouping-add", "signature" : "($endpoint-url as xs:string, $apikey as xs:string, $id as xs:string, $name as xs:string, $type as xs:string, $groups as element(array)) as item()*", "description" : " Add a new Interest Grouping - if interest groups for the List are not yet enabled, adding the first grouping will automatically turn them on.\n", "summary" : "<p> Add a new Interest Grouping - if interest groups for the List are not yet enabled, adding the first grouping will automatically turn them on.</p>", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "endpoint-url", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> Mailchimp endpoint URL, e.g. http://us1.api.mailchimp.com/1.3/</div>" }, { "name" : "apikey", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> a valid API Key for your user account. Get by visiting <a href=\"http://admin.mailchimp.com/account/api\" target=\"_blank\">your API dashboard</a></div>" }, { "name" : "id", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the list id to connect to. Get by calling <a href=\"#lists-4\">lists()</a></div>" }, { "name" : "name", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the interest grouping to add - grouping names must be unique</div>" }, { "name" : "type", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The type of the grouping to add - one of \"checkboxes\", \"hidden\", \"dropdown\", \"radio\"</div>" }, { "name" : "groups", "type" : "element(array)", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The lists of initial group names to be added - at least 1 is required and the names must be unique within a grouping. If the number takes you over the 60 group limit, an error will be thrown.</div>" } ], "returns" : { "type" : "item()*", "description" : "The new grouping id if the request succeeds, otherwise an error will be thrown" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 2, "name" : "list-interest-grouping-del", "qname" : "mailchimp:list-interest-grouping-del", "signature" : "($apikey as xs:string, $grouping_id as xs:integer) as item()*", "description" : " Convenience function for <a xmlns:xqdoc=\"http://www.xqdoc.org/1.0\" href=\"#list-interest-grouping-del-3\">list-interest-grouping-del#3</a>.\n", "summary" : "<p> Convenience function for  list-interest-grouping-del#3 .</p>", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "apikey", "type" : "xs:string", "occurrence" : null, "description" : "" }, { "name" : "grouping_id", "type" : "xs:integer", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the interest grouping id - get from <a href=\"#list-interest-groupings-2\">list-interest-groupings()</a></div>" } ], "returns" : { "type" : "item()*", "description" : "True if the request succeeds, otherwise an error will be thrown" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 3, "name" : "list-interest-grouping-del", "qname" : "mailchimp:list-interest-grouping-del", "signature" : "($endpoint-url as xs:string, $apikey as xs:string, $grouping_id as xs:integer) as item()*", "description" : " Delete an existing Interest Grouping - this will permanently delete all contained interest groups and will remove those selections from all list members.\n", "summary" : "<p> Delete an existing Interest Grouping - this will permanently delete all contained interest groups and will remove those selections from all list members.</p>", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "endpoint-url", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> Mailchimp endpoint URL, e.g. http://us1.api.mailchimp.com/1.3/</div>" }, { "name" : "apikey", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> a valid API Key for your user account. Get by visiting <a href=\"http://admin.mailchimp.com/account/api\" target=\"_blank\">your API dashboard</a></div>" }, { "name" : "grouping_id", "type" : "xs:integer", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the interest grouping id - get from <a href=\"#list-interest-groupings-2\">list-interest-groupings()</a></div>" } ], "returns" : { "type" : "item()*", "description" : "True if the request succeeds, otherwise an error will be thrown" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 4, "name" : "list-interest-grouping-update", "qname" : "mailchimp:list-interest-grouping-update", "signature" : "($apikey as xs:string, $grouping_id as xs:integer, $name as xs:string, $value as xs:string) as item()*", "description" : " Convenience function for <a xmlns:xqdoc=\"http://www.xqdoc.org/1.0\" href=\"#list-interest-grouping-update-5\">list-interest-grouping-update#5</a>.\n", "summary" : "<p> Convenience function for  list-interest-grouping-update#5 .</p>", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "apikey", "type" : "xs:string", "occurrence" : null, "description" : "" }, { "name" : "grouping_id", "type" : "xs:integer", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the interest grouping id - get from <a href=\"#list-interest-groupings-2\">list-interest-groupings()</a></div>" }, { "name" : "name", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The name of the field to update - either \"name\" or \"type\". Groups with in the grouping should be manipulated using the standard listInterestGroup* methods</div>" }, { "name" : "value", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The new value of the field. Grouping names must be unique - only \"hidden\" and \"checkboxes\" grouping types can be converted between each other.</div>" } ], "returns" : { "type" : "item()*", "description" : "True if the request succeeds, otherwise an error will be thrown" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 5, "name" : "list-interest-grouping-update", "qname" : "mailchimp:list-interest-grouping-update", "signature" : "($endpoint-url as xs:string, $apikey as xs:string, $grouping_id as xs:integer, $name as xs:string, $value as xs:string) as item()*", "description" : " Update an existing Interest Grouping.\n", "summary" : "<p> Update an existing Interest Grouping.</p>", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "endpoint-url", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> Mailchimp endpoint URL, e.g. http://us1.api.mailchimp.com/1.3/</div>" }, { "name" : "apikey", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> a valid API Key for your user account. Get by visiting <a href=\"http://admin.mailchimp.com/account/api\" target=\"_blank\">your API dashboard</a></div>" }, { "name" : "grouping_id", "type" : "xs:integer", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the interest grouping id - get from <a href=\"#list-interest-groupings-2\">list-interest-groupings()</a></div>" }, { "name" : "name", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The name of the field to update - either \"name\" or \"type\". Groups with in the grouping should be manipulated using the standard listInterestGroup* methods</div>" }, { "name" : "value", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The new value of the field. Grouping names must be unique - only \"hidden\" and \"checkboxes\" grouping types can be converted between each other.</div>" } ], "returns" : { "type" : "item()*", "description" : "True if the request succeeds, otherwise an error will be thrown" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 2, "name" : "list-interest-groupings", "qname" : "mailchimp:list-interest-groupings", "signature" : "($apikey as xs:string, $id as xs:string) as item()*", "description" : " Convenience function for <a xmlns:xqdoc=\"http://www.xqdoc.org/1.0\" href=\"#list-interest-groupings-3\">list-interest-groupings#3</a>.\n", "summary" : "<p> Convenience function for  list-interest-groupings#3 .</p>", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "apikey", "type" : "xs:string", "occurrence" : null, "description" : "" }, { "name" : "id", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the list id to connect to. Get by calling <a href=\"#lists-4\">lists()</a></div>" } ], "returns" : { "type" : "item()*", "description" : "List of interest groups for the list" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 3, "name" : "list-interest-groupings", "qname" : "mailchimp:list-interest-groupings", "signature" : "($endpoint-url as xs:string, $apikey as xs:string, $id as xs:string) as item()*", "description" : " Get the list of interest groupings for a given list, including the label, form information, and included groups for each.\n", "summary" : "<p> Get the list of interest groupings for a given list, including the label, form information, and included groups for each.</p>", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "endpoint-url", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> Mailchimp endpoint URL, e.g. http://us1.api.mailchimp.com/1.3/</div>" }, { "name" : "apikey", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> a valid API Key for your user account. Get by visiting <a href=\"http://admin.mailchimp.com/account/api\" target=\"_blank\">your API dashboard</a></div>" }, { "name" : "id", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the list id to connect to. Get by calling <a href=\"#lists-4\">lists()</a></div>" } ], "returns" : { "type" : "item()*", "description" : "List of interest groups for the list" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 2, "name" : "list-locations", "qname" : "mailchimp:list-locations", "signature" : "($apikey as xs:string, $id as xs:string) as item()*", "description" : " Convenience function for <a xmlns:xqdoc=\"http://www.xqdoc.org/1.0\" href=\"#list-locations-3\">list-locations#3</a>.\n", "summary" : "<p> Convenience function for  list-locations#3 .</p>", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "apikey", "type" : "xs:string", "occurrence" : null, "description" : "" }, { "name" : "id", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the list id to connect to. Get by calling <a href=\"#lists-4\">lists()</a></div>" } ], "returns" : { "type" : "item()*", "description" : "Array of locations" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 3, "name" : "list-locations", "qname" : "mailchimp:list-locations", "signature" : "($endpoint-url as xs:string, $apikey as xs:string, $id as xs:string) as item()*", "description" : " Retrieve the locations (countries) that the list's subscribers have been tagged to based on geocoding their IP address.\n", "summary" : "<p> Retrieve the locations (countries) that the list's subscribers have been tagged to based on geocoding their IP address.</p>", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "endpoint-url", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> Mailchimp endpoint URL, e.g. http://us1.api.mailchimp.com/1.3/</div>" }, { "name" : "apikey", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> a valid API Key for your user account. Get by visiting <a href=\"http://admin.mailchimp.com/account/api\" target=\"_blank\">your API dashboard</a></div>" }, { "name" : "id", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the list id to connect to. Get by calling <a href=\"#lists-4\">lists()</a></div>" } ], "returns" : { "type" : "item()*", "description" : "Array of locations" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 3, "name" : "list-member-activity", "qname" : "mailchimp:list-member-activity", "signature" : "($apikey as xs:string, $id as xs:string, $email_address as element(array)) as item()*", "description" : " Convenience function for <a xmlns:xqdoc=\"http://www.xqdoc.org/1.0\" href=\"#list-member-activity-4\">list-member-activity#4</a>.\n", "summary" : "<p> Convenience function for  list-member-activity#4 .</p>", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "apikey", "type" : "xs:string", "occurrence" : null, "description" : "" }, { "name" : "id", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the list id to connect to. Get by calling <a href=\"#lists-4\">lists()</a></div>" }, { "name" : "email_address", "type" : "element(array)", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> an array of up to 50 email addresses to get information for OR the \"id\"(s) for the member returned from listMembers, Webhooks, and Campaigns.</div>" } ], "returns" : { "type" : "item()*", "description" : "Array of data and success/error counts" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 4, "name" : "list-member-activity", "qname" : "mailchimp:list-member-activity", "signature" : "($endpoint-url as xs:string, $apikey as xs:string, $id as xs:string, $email_address as element(array)) as item()*", "description" : " Get the most recent 100 activities for particular list members (open, click, bounce, unsub, abuse, sent to).\n", "summary" : "<p> Get the most recent 100 activities for particular list members (open, click, bounce, unsub, abuse, sent to).</p>", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "endpoint-url", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> Mailchimp endpoint URL, e.g. http://us1.api.mailchimp.com/1.3/</div>" }, { "name" : "apikey", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> a valid API Key for your user account. Get by visiting <a href=\"http://admin.mailchimp.com/account/api\" target=\"_blank\">your API dashboard</a></div>" }, { "name" : "id", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the list id to connect to. Get by calling <a href=\"#lists-4\">lists()</a></div>" }, { "name" : "email_address", "type" : "element(array)", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> an array of up to 50 email addresses to get information for OR the \"id\"(s) for the member returned from listMembers, Webhooks, and Campaigns.</div>" } ], "returns" : { "type" : "item()*", "description" : "Array of data and success/error counts" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 3, "name" : "list-member-info", "qname" : "mailchimp:list-member-info", "signature" : "($apikey as xs:string, $id as xs:string, $email_address as element(array)) as item()*", "description" : " Convenience function for <a xmlns:xqdoc=\"http://www.xqdoc.org/1.0\" href=\"#list-member-info-4\">list-member-info#4</a>.\n", "summary" : "<p> Convenience function for  list-member-info#4 .</p>", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "apikey", "type" : "xs:string", "occurrence" : null, "description" : "" }, { "name" : "id", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the list id to connect to. Get by calling <a href=\"#lists-4\">lists()</a></div>" }, { "name" : "email_address", "type" : "element(array)", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> an array of up to 50 email addresses to get information for OR the \"id\"(s) for the member returned from listMembers, Webhooks, and Campaigns. For backwards compatibility, if a string is passed, it will be treated as an array with a single element (will not work with XML-RPC).</div>" } ], "returns" : { "type" : "item()*", "description" : "Array of list members with their info in an array (see Returned Fields for details)" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 4, "name" : "list-member-info", "qname" : "mailchimp:list-member-info", "signature" : "($endpoint-url as xs:string, $apikey as xs:string, $id as xs:string, $email_address as element(array)) as item()*", "description" : " Get all the information for particular members of a list.\n", "summary" : "<p> Get all the information for particular members of a list.</p>", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "endpoint-url", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> Mailchimp endpoint URL, e.g. http://us1.api.mailchimp.com/1.3/</div>" }, { "name" : "apikey", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> a valid API Key for your user account. Get by visiting <a href=\"http://admin.mailchimp.com/account/api\" target=\"_blank\">your API dashboard</a></div>" }, { "name" : "id", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the list id to connect to. Get by calling <a href=\"#lists-4\">lists()</a></div>" }, { "name" : "email_address", "type" : "element(array)", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> an array of up to 50 email addresses to get information for OR the \"id\"(s) for the member returned from listMembers, Webhooks, and Campaigns. For backwards compatibility, if a string is passed, it will be treated as an array with a single element (will not work with XML-RPC).</div>" } ], "returns" : { "type" : "item()*", "description" : "Array of list members with their info in an array (see Returned Fields for details)" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 6, "name" : "list-members", "qname" : "mailchimp:list-members", "signature" : "($apikey as xs:string, $id as xs:string, $status as xs:string, $since as xs:string, $start as xs:integer, $limit as xs:integer) as item()*", "description" : " Convenience function for <a xmlns:xqdoc=\"http://www.xqdoc.org/1.0\" href=\"#list-members-7\">list-members#7</a>.\n", "summary" : "<p> Convenience function for  list-members#7 .</p>", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "apikey", "type" : "xs:string", "occurrence" : null, "description" : "" }, { "name" : "id", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the list id to connect to. Get by calling <a href=\"#lists-4\">lists()</a></div>" }, { "name" : "status", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the status to get members for - one of(subscribed, unsubscribed, <a target=\"_blank\" href=\"http://eepurl.com/dwk1\">cleaned</a>, updated), defaults to subscribed</div>" }, { "name" : "since", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> optional pull all members whose status (subscribed/unsubscribed/cleaned) has changed or whose profile (updated) has changed since this date/time (in GMT) - format is YYYY-MM-DD HH:mm:ss (24hr)</div>" }, { "name" : "start", "type" : "xs:integer", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> optional for large data sets, the page number to start at - defaults to 1st page of data (page 0)</div>" }, { "name" : "limit", "type" : "xs:integer", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> optional for large data sets, the number of results to return - defaults to 100, upper limit set at 15000</div>" } ], "returns" : { "type" : "item()*", "description" : "Array of a the total records match and matching list member data for this page (see Returned Fields for details)" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 7, "name" : "list-members", "qname" : "mailchimp:list-members", "signature" : "($endpoint-url as xs:string, $apikey as xs:string, $id as xs:string, $status as xs:string, $since as xs:string, $start as xs:integer, $limit as xs:integer) as item()*", "description" : " Get all of the list members for a list that are of a particular status.\n", "summary" : "<p> Get all of the list members for a list that are of a particular status.</p>", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "endpoint-url", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> Mailchimp endpoint URL, e.g. http://us1.api.mailchimp.com/1.3/</div>" }, { "name" : "apikey", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> a valid API Key for your user account. Get by visiting <a href=\"http://admin.mailchimp.com/account/api\" target=\"_blank\">your API dashboard</a></div>" }, { "name" : "id", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the list id to connect to. Get by calling <a href=\"#lists-4\">lists()</a></div>" }, { "name" : "status", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the status to get members for - one of(subscribed, unsubscribed, <a target=\"_blank\" href=\"http://eepurl.com/dwk1\">cleaned</a>, updated), defaults to subscribed</div>" }, { "name" : "since", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> optional pull all members whose status (subscribed/unsubscribed/cleaned) has changed or whose profile (updated) has changed since this date/time (in GMT) - format is YYYY-MM-DD HH:mm:ss (24hr)</div>" }, { "name" : "start", "type" : "xs:integer", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> optional for large data sets, the page number to start at - defaults to 1st page of data (page 0)</div>" }, { "name" : "limit", "type" : "xs:integer", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> optional for large data sets, the number of results to return - defaults to 100, upper limit set at 15000</div>" } ], "returns" : { "type" : "item()*", "description" : "Array of a the total records match and matching list member data for this page (see Returned Fields for details)" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 5, "name" : "list-merge-var-add", "qname" : "mailchimp:list-merge-var-add", "signature" : "($apikey as xs:string, $id as xs:string, $tag as xs:string, $name as xs:string, $options as element(array)) as item()*", "description" : " Convenience function for <a xmlns:xqdoc=\"http://www.xqdoc.org/1.0\" href=\"#list-merge-var-add-6\">list-merge-var-add#6</a>.\n", "summary" : "<p> Convenience function for  list-merge-var-add#6 .</p>", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "apikey", "type" : "xs:string", "occurrence" : null, "description" : "" }, { "name" : "id", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the list id to connect to. Get by calling <a href=\"#lists-4\">lists()</a></div>" }, { "name" : "tag", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The merge tag to add, e.g. FNAME</div>" }, { "name" : "name", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The long description of the tag being added, used for user displays</div>" }, { "name" : "options", "type" : "element(array)", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> optional Various options for this merge var. <em>note:</em> for historical purposes this can also take a \"boolean\"</div>" } ], "returns" : { "type" : "item()*", "description" : "True if the request succeeds, otherwise an error will be thrown" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 6, "name" : "list-merge-var-add", "qname" : "mailchimp:list-merge-var-add", "signature" : "($endpoint-url as xs:string, $apikey as xs:string, $id as xs:string, $tag as xs:string, $name as xs:string, $options as element(array)) as item()*", "description" : " Add a new merge tag to a given list.\n", "summary" : "<p> Add a new merge tag to a given list.</p>", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "endpoint-url", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> Mailchimp endpoint URL, e.g. http://us1.api.mailchimp.com/1.3/</div>" }, { "name" : "apikey", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> a valid API Key for your user account. Get by visiting <a href=\"http://admin.mailchimp.com/account/api\" target=\"_blank\">your API dashboard</a></div>" }, { "name" : "id", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the list id to connect to. Get by calling <a href=\"#lists-4\">lists()</a></div>" }, { "name" : "tag", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The merge tag to add, e.g. FNAME</div>" }, { "name" : "name", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The long description of the tag being added, used for user displays</div>" }, { "name" : "options", "type" : "element(array)", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> optional Various options for this merge var. <em>note:</em> for historical purposes this can also take a \"boolean\"</div>" } ], "returns" : { "type" : "item()*", "description" : "True if the request succeeds, otherwise an error will be thrown" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 3, "name" : "list-merge-var-del", "qname" : "mailchimp:list-merge-var-del", "signature" : "($apikey as xs:string, $id as xs:string, $tag as xs:string) as item()*", "description" : " Convenience function for <a xmlns:xqdoc=\"http://www.xqdoc.org/1.0\" href=\"#list-merge-var-del-4\">list-merge-var-del#4</a>.\n", "summary" : "<p> Convenience function for  list-merge-var-del#4 .</p>", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "apikey", "type" : "xs:string", "occurrence" : null, "description" : "" }, { "name" : "id", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the list id to connect to. Get by calling <a href=\"#lists-4\">lists()</a></div>" }, { "name" : "tag", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The merge tag to delete</div>" } ], "returns" : { "type" : "item()*", "description" : "bool true if the request succeeds, otherwise an error will be thrown" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 4, "name" : "list-merge-var-del", "qname" : "mailchimp:list-merge-var-del", "signature" : "($endpoint-url as xs:string, $apikey as xs:string, $id as xs:string, $tag as xs:string) as item()*", "description" : " Delete a merge tag from a given list and all its members. Seriously - the data is removed from all members as well! Note that on large lists this method may seem a bit slower than calls you typically make.\n", "summary" : "<p> Delete a merge tag from a given list and all its members.</p>", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "endpoint-url", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> Mailchimp endpoint URL, e.g. http://us1.api.mailchimp.com/1.3/</div>" }, { "name" : "apikey", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> a valid API Key for your user account. Get by visiting <a href=\"http://admin.mailchimp.com/account/api\" target=\"_blank\">your API dashboard</a></div>" }, { "name" : "id", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the list id to connect to. Get by calling <a href=\"#lists-4\">lists()</a></div>" }, { "name" : "tag", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The merge tag to delete</div>" } ], "returns" : { "type" : "item()*", "description" : "True if the request succeeds, otherwise an error will be thrown" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 4, "name" : "list-merge-var-update", "qname" : "mailchimp:list-merge-var-update", "signature" : "($apikey as xs:string, $id as xs:string, $tag as xs:string, $options as element(array)) as item()*", "description" : " Convenience function for <a xmlns:xqdoc=\"http://www.xqdoc.org/1.0\" href=\"#list-merge-var-update-5\">list-merge-var-update#5</a>.\n", "summary" : "<p> Convenience function for  list-merge-var-update#5 .</p>", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "apikey", "type" : "xs:string", "occurrence" : null, "description" : "" }, { "name" : "id", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the list id to connect to. Get by calling <a href=\"#lists-4\">lists()</a></div>" }, { "name" : "tag", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The merge tag to update</div>" }, { "name" : "options", "type" : "element(array)", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The options to change for a merge var. See <a href=\"#list-merge-var-add-5\">list-merge-var-add()</a> for valid options</div>" } ], "returns" : { "type" : "item()*", "description" : "True if the request succeeds, otherwise an error will be thrown" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 5, "name" : "list-merge-var-update", "qname" : "mailchimp:list-merge-var-update", "signature" : "($endpoint-url as xs:string, $apikey as xs:string, $id as xs:string, $tag as xs:string, $options as element(array)) as item()*", "description" : " Update most parameters for a merge tag on a given list. You cannot currently change the merge type.\n", "summary" : "<p> Update most parameters for a merge tag on a given list.</p>", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "endpoint-url", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> Mailchimp endpoint URL, e.g. http://us1.api.mailchimp.com/1.3/</div>" }, { "name" : "apikey", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> a valid API Key for your user account. Get by visiting <a href=\"http://admin.mailchimp.com/account/api\" target=\"_blank\">your API dashboard</a></div>" }, { "name" : "id", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the list id to connect to. Get by calling <a href=\"#lists-4\">lists()</a></div>" }, { "name" : "tag", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The merge tag to update</div>" }, { "name" : "options", "type" : "element(array)", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The options to change for a merge var. See <a href=\"#list-merge-var-add-5\">list-merge-var-add()</a> for valid options</div>" } ], "returns" : { "type" : "item()*", "description" : "True if the request succeeds, otherwise an error will be thrown" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 2, "name" : "list-merge-vars", "qname" : "mailchimp:list-merge-vars", "signature" : "($apikey as xs:string, $id as xs:string) as item()*", "description" : " Convenience function for <a xmlns:xqdoc=\"http://www.xqdoc.org/1.0\" href=\"#list-merge-vars-3\">list-merge-vars#3</a>.\n", "summary" : "<p> Convenience function for  list-merge-vars#3 .</p>", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "apikey", "type" : "xs:string", "occurrence" : null, "description" : "" }, { "name" : "id", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the list id to connect to. Get by calling <a href=\"#lists-4\">lists()</a></div>" } ], "returns" : { "type" : "item()*", "description" : "List of merge tags for the list" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 3, "name" : "list-merge-vars", "qname" : "mailchimp:list-merge-vars", "signature" : "($endpoint-url as xs:string, $apikey as xs:string, $id as xs:string) as item()*", "description" : " Get the list of merge tags for a given list, including their name, tag, and required setting.\n", "summary" : "<p> Get the list of merge tags for a given list, including their name, tag, and required setting.</p>", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "endpoint-url", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> Mailchimp endpoint URL, e.g. http://us1.api.mailchimp.com/1.3/</div>" }, { "name" : "apikey", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> a valid API Key for your user account. Get by visiting <a href=\"http://admin.mailchimp.com/account/api\" target=\"_blank\">your API dashboard</a></div>" }, { "name" : "id", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the list id to connect to. Get by calling <a href=\"#lists-4\">lists()</a></div>" } ], "returns" : { "type" : "item()*", "description" : "List of merge tags for the list" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 3, "name" : "list-static-segment-add", "qname" : "mailchimp:list-static-segment-add", "signature" : "($apikey as xs:string, $id as xs:string, $name as xs:string) as item()*", "description" : " Convenience function for <a xmlns:xqdoc=\"http://www.xqdoc.org/1.0\" href=\"#list-static-segment-add-4\">list-static-segment-add#4</a>.\n", "summary" : "<p> Convenience function for  list-static-segment-add#4 .</p>", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "apikey", "type" : "xs:string", "occurrence" : null, "description" : "" }, { "name" : "id", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the list id to connect to. Get by calling <a href=\"#lists-4\">lists()</a></div>" }, { "name" : "name", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> a unique name per list for the segment - 50 byte maximum length, anything longer will throw an error</div>" } ], "returns" : { "type" : "item()*", "description" : "The id of the new segment, otherwise an error will be thrown." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 4, "name" : "list-static-segment-add", "qname" : "mailchimp:list-static-segment-add", "signature" : "($endpoint-url as xs:string, $apikey as xs:string, $id as xs:string, $name as xs:string) as item()*", "description" : " Save a segment against a list for later use. There is no limit to the number of segments which can be saved. Static Segments <strong xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">are not</strong> tied to any merge data, interest groups, etc. They essentially allow you to configure an unlimited number of custom segments which will have standard performance. When using proper segments, Static Segments are one of the available options for segmentation just as if you used a merge var (and they can be used with other segmentation options), though performance may degrade at that point.\n", "summary" : "<p> Save a segment against a list for later use.</p>", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "endpoint-url", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> Mailchimp endpoint URL, e.g. http://us1.api.mailchimp.com/1.3/</div>" }, { "name" : "apikey", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> a valid API Key for your user account. Get by visiting <a href=\"http://admin.mailchimp.com/account/api\" target=\"_blank\">your API dashboard</a></div>" }, { "name" : "id", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the list id to connect to. Get by calling <a href=\"#lists-4\">lists()</a></div>" }, { "name" : "name", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> a unique name per list for the segment - 50 byte maximum length, anything longer will throw an error</div>" } ], "returns" : { "type" : "item()*", "description" : "The id of the new segment, otherwise an error will be thrown." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 3, "name" : "list-static-segment-del", "qname" : "mailchimp:list-static-segment-del", "signature" : "($apikey as xs:string, $id as xs:string, $seg_id as xs:integer) as item()*", "description" : " Convenience function for <a xmlns:xqdoc=\"http://www.xqdoc.org/1.0\" href=\"#listStaticSegmentDel-4\">list-static-segment-del#4</a>.\n", "summary" : "<p> Convenience function for  list-static-segment-del#4 .</p>", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "apikey", "type" : "xs:string", "occurrence" : null, "description" : "" }, { "name" : "id", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the list id to connect to. Get by calling <a href=\"#lists-4\">lists()</a></div>" }, { "name" : "seg_id", "type" : "xs:integer", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the id of the static segment to delete - get from <a href=\"#list-static-segments-2\">list-static-segments()</a></div>" } ], "returns" : { "type" : "item()*", "description" : "True if it worked, otherwise an error is thrown." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 4, "name" : "list-static-segment-del", "qname" : "mailchimp:list-static-segment-del", "signature" : "($endpoint-url as xs:string, $apikey as xs:string, $id as xs:string, $seg_id as xs:integer) as item()*", "description" : " Delete a static segment. Note that this will, of course, remove any member affiliations with the segment.\n", "summary" : "<p> Delete a static segment.</p>", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "endpoint-url", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> Mailchimp endpoint URL, e.g. http://us1.api.mailchimp.com/1.3/</div>" }, { "name" : "apikey", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> a valid API Key for your user account. Get by visiting <a href=\"http://admin.mailchimp.com/account/api\" target=\"_blank\">your API dashboard</a></div>" }, { "name" : "id", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the list id to connect to. Get by calling <a href=\"#lists-4\">lists()</a></div>" }, { "name" : "seg_id", "type" : "xs:integer", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the id of the static segment to delete - get from <a href=\"#list-static-segments-2\">list-static-segments()</a></div>" } ], "returns" : { "type" : "item()*", "description" : "True if it worked, otherwise an error is thrown." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 4, "name" : "list-static-segment-members-add", "qname" : "mailchimp:list-static-segment-members-add", "signature" : "($apikey as xs:string, $id as xs:string, $seg_id as xs:integer, $batch as element(array)) as item()*", "description" : " Convenience function for <a xmlns:xqdoc=\"http://www.xqdoc.org/1.0\" href=\"#list-static-segment-members-add-5\">list-static-segment-members-add#5</a>.\n", "summary" : "<p> Convenience function for  list-static-segment-members-add#5 .</p>", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "apikey", "type" : "xs:string", "occurrence" : null, "description" : "" }, { "name" : "id", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the list id to connect to. Get by calling <a href=\"#lists-4\">lists()</a></div>" }, { "name" : "seg_id", "type" : "xs:integer", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the id of the static segment to modify - get from <a href=\"#list-static-segments-2\">list-static-segments()</a></div>" }, { "name" : "batch", "type" : "element(array)", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> an array of email addresses and/or unique_ids to add to the segment</div>" } ], "returns" : { "type" : "item()*", "description" : "An array with the results of the operation" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 5, "name" : "list-static-segment-members-add", "qname" : "mailchimp:list-static-segment-members-add", "signature" : "($endpoint-url as xs:string, $apikey as xs:string, $id as xs:string, $seg_id as xs:integer, $batch as element(array)) as item()*", "description" : " Add list members to a static segment. It is suggested that you limit batch size to no more than 10,000 addresses per call. Email addresses must exist on the list in order to be included - this <strong xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">will not</strong> subscribe them to the list!.\n", "summary" : "<p> Add list members to a static segment.</p>", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "endpoint-url", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> Mailchimp endpoint URL, e.g. http://us1.api.mailchimp.com/1.3/</div>" }, { "name" : "apikey", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> a valid API Key for your user account. Get by visiting <a href=\"http://admin.mailchimp.com/account/api\" target=\"_blank\">your API dashboard</a></div>" }, { "name" : "id", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the list id to connect to. Get by calling <a href=\"#lists-4\">lists()</a></div>" }, { "name" : "seg_id", "type" : "xs:integer", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the id of the static segment to modify - get from <a href=\"#list-static-segments-2\">list-static-segments()</a></div>" }, { "name" : "batch", "type" : "element(array)", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> an array of email addresses and/or unique_ids to add to the segment</div>" } ], "returns" : { "type" : "item()*", "description" : "array an array with the results of the operation" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 4, "name" : "list-static-segment-members-del", "qname" : "mailchimp:list-static-segment-members-del", "signature" : "($apikey as xs:string, $id as xs:string, $seg_id as xs:integer, $batch as element(array)) as item()*", "description" : " Convenience function for <a xmlns:xqdoc=\"http://www.xqdoc.org/1.0\" href=\"#list-static-segment-members-del-5\">list-static-segment-members-del#5</a>.\n", "summary" : "<p> Convenience function for  list-static-segment-members-del#5 .</p>", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "apikey", "type" : "xs:string", "occurrence" : null, "description" : "" }, { "name" : "id", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the list id to connect to. Get by calling <a href=\"#lists-4\">lists()</a></div>" }, { "name" : "seg_id", "type" : "xs:integer", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the id of the static segment to delete - get from <a href=\"#list-static-segments-2\">list-static-segments()</a></div>" }, { "name" : "batch", "type" : "element(array)", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> an array of email addresses and/or unique_ids to remove from the segment</div>" } ], "returns" : { "type" : "item()*", "description" : "An array with the results of the operation" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 5, "name" : "list-static-segment-members-del", "qname" : "mailchimp:list-static-segment-members-del", "signature" : "($endpoint-url as xs:string, $apikey as xs:string, $id as xs:string, $seg_id as xs:integer, $batch as element(array)) as item()*", "description" : " Remove list members from a static segment. It is suggested that you limit batch size to no more than 10,000 addresses per call. Email addresses must exist on the list in order to be removed - this <strong xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">will not</strong> unsubscribe them from the list!.\n", "summary" : "<p> Remove list members from a static segment.</p>", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "endpoint-url", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> Mailchimp endpoint URL, e.g. http://us1.api.mailchimp.com/1.3/</div>" }, { "name" : "apikey", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> a valid API Key for your user account. Get by visiting <a href=\"http://admin.mailchimp.com/account/api\" target=\"_blank\">your API dashboard</a></div>" }, { "name" : "id", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the list id to connect to. Get by calling <a href=\"#lists-4\">lists()</a></div>" }, { "name" : "seg_id", "type" : "xs:integer", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the id of the static segment to delete - get from <a href=\"#list-static-segments-2\">list-static-segments()</a></div>" }, { "name" : "batch", "type" : "element(array)", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> an array of email addresses and/or unique_ids to remove from the segment</div>" } ], "returns" : { "type" : "item()*", "description" : "An array with the results of the operation" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 3, "name" : "list-static-segment-reset", "qname" : "mailchimp:list-static-segment-reset", "signature" : "($apikey as xs:string, $id as xs:string, $seg_id as xs:integer) as item()*", "description" : " Convenience function for <a xmlns:xqdoc=\"http://www.xqdoc.org/1.0\" href=\"#list-static-segment-reset-4\">list-static-segment-reset#4</a>.\n", "summary" : "<p> Convenience function for  list-static-segment-reset#4 .</p>", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "apikey", "type" : "xs:string", "occurrence" : null, "description" : "" }, { "name" : "id", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the list id to connect to. Get by calling <a href=\"#lists-4\">lists()</a></div>" }, { "name" : "seg_id", "type" : "xs:integer", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the id of the static segment to reset - get from <a href=\"#list-static-segments-2\">list-static-segments()</a></div>" } ], "returns" : { "type" : "item()*", "description" : "True if it worked, otherwise an error is thrown." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 4, "name" : "list-static-segment-reset", "qname" : "mailchimp:list-static-segment-reset", "signature" : "($endpoint-url as xs:string, $apikey as xs:string, $id as xs:string, $seg_id as xs:integer) as item()*", "description" : " Resets a static segment - removes <strong xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">all</strong> members from the static segment. Note: does not actually affect list member data.\n", "summary" : "<p> Resets a static segment - removes  all  members from the static segment.</p>", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "endpoint-url", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> Mailchimp endpoint URL, e.g. http://us1.api.mailchimp.com/1.3/</div>" }, { "name" : "apikey", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> a valid API Key for your user account. Get by visiting <a href=\"http://admin.mailchimp.com/account/api\" target=\"_blank\">your API dashboard</a></div>" }, { "name" : "id", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the list id to connect to. Get by calling <a href=\"#lists-4\">lists()</a></div>" }, { "name" : "seg_id", "type" : "xs:integer", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the id of the static segment to reset - get from <a href=\"#list-static-segments-2\">list-static-segments()</a></div>" } ], "returns" : { "type" : "item()*", "description" : "True if it worked, otherwise an error is thrown." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 2, "name" : "list-static-segments", "qname" : "mailchimp:list-static-segments", "signature" : "($apikey as xs:string, $id as xs:string) as item()*", "description" : " Convenience function for <a xmlns:xqdoc=\"http://www.xqdoc.org/1.0\" href=\"#list-static-segments-3\">list-static-segments#3</a>.\n", "summary" : "<p> Convenience function for  list-static-segments#3 .</p>", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "apikey", "type" : "xs:string", "occurrence" : null, "description" : "" }, { "name" : "id", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the list id to connect to. Get by calling <a href=\"#lists-4\">lists()</a></div>" } ], "returns" : { "type" : "item()*", "description" : "An array of parameters for each static segment" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 3, "name" : "list-static-segments", "qname" : "mailchimp:list-static-segments", "signature" : "($endpoint-url as xs:string, $apikey as xs:string, $id as xs:string) as item()*", "description" : " Retrieve all of the Static Segments for a list.\n", "summary" : "<p> Retrieve all of the Static Segments for a list.</p>", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "endpoint-url", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> Mailchimp endpoint URL, e.g. http://us1.api.mailchimp.com/1.3/</div>" }, { "name" : "apikey", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> a valid API Key for your user account. Get by visiting <a href=\"http://admin.mailchimp.com/account/api\" target=\"_blank\">your API dashboard</a></div>" }, { "name" : "id", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the list id to connect to. Get by calling <a href=\"#lists-4\">lists()</a></div>" } ], "returns" : { "type" : "item()*", "description" : "An array of parameters for each static segment" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 10, "name" : "list-subscribe", "qname" : "mailchimp:list-subscribe", "signature" : "($endpoint-url as xs:string, $apikey as xs:string, $id as xs:string, $email_address as xs:string, $merge_vars as element(array), $email_type as xs:string, $double_optin as xs:boolean, $update_existing as xs:boolean, $replace_interests as xs:boolean, $send_welcome as xs:boolean) as item()*", "description" : " Subscribe the provided email to a list.\n", "summary" : "<p> Subscribe the provided email to a list.</p>", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "endpoint-url", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> Mailchimp endpoint URL, e.g. http://us1.api.mailchimp.com/1.3/</div>" }, { "name" : "apikey", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> a valid API Key for your user account. Get by visiting <a href=\"http://admin.mailchimp.com/account/api\" target=\"_blank\">your API dashboard</a></div>" }, { "name" : "id", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the list id to connect to. Get by calling <a href=\"#lists-4\">lists()</a></div>" }, { "name" : "email_address", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the email address to subscribe</div>" }, { "name" : "merge_vars", "type" : "element(array)", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> optional merges for the email (FNAME, LNAME, etc.) (see examples below for handling \"blank\" arrays). Note that a merge field can only hold up to 255 bytes. Also, there are a few \"special\" keys. See <a href=\"http://apidocs.mailchimp.com/rtfm/listsubscribe.func.php\">Mailchimp API documentation</a> for more information about the parameter structure.</div>" }, { "name" : "email_type", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> optional email type preference for the email (html, text, or mobile defaults to html)</div>" }, { "name" : "double_optin", "type" : "xs:boolean", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> optional flag to control whether a double opt-in confirmation message is sent, defaults to true. <em>Abusing this may cause your account to be suspended.</em></div>" }, { "name" : "update_existing", "type" : "xs:boolean", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> optional flag to control whether a existing subscribers should be updated instead of throwing and error, defaults to false</div>" }, { "name" : "replace_interests", "type" : "xs:boolean", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> optional flag to determine whether we replace the interest groups with the groups provided, or we add the provided groups to the member's interest groups (optional, defaults to true)</div>" }, { "name" : "send_welcome", "type" : "xs:boolean", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> optional if your double_optin is false and this is true, we will send your lists Welcome Email if this subscribe succeeds - this will *not* fire if we end up updating an existing subscriber. If double_optin is true, this has no effect. defaults to false.</div>" } ], "returns" : { "type" : "item()*", "description" : "True on success, false on failure." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 9, "name" : "list-subscribe", "qname" : "mailchimp:list-subscribe", "signature" : "($apikey as xs:string, $id as xs:string, $email_address as xs:string, $merge_vars as element(array), $email_type as xs:string, $double_optin as xs:boolean, $update_existing as xs:boolean, $replace_interests as xs:boolean, $send_welcome as xs:boolean) as item()*", "description" : " Convenience function for <a xmlns:xqdoc=\"http://www.xqdoc.org/1.0\" href=\"#list-subscribe-10\">list-subscribe#10</a>.\n", "summary" : "<p> Convenience function for  list-subscribe#10 .</p>", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "apikey", "type" : "xs:string", "occurrence" : null, "description" : "" }, { "name" : "id", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the list id to connect to. Get by calling <a href=\"#lists-4\">lists()</a></div>" }, { "name" : "email_address", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the email address to subscribe</div>" }, { "name" : "merge_vars", "type" : "element(array)", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> optional merges for the email (FNAME, LNAME, etc.) (see examples below for handling \"blank\" arrays). Note that a merge field can only hold up to 255 bytes. Also, there are a few \"special\" keys. See <a href=\"http://apidocs.mailchimp.com/rtfm/listsubscribe.func.php\">Mailchimp API documentation</a> for more information about the parameter structure.</div>" }, { "name" : "email_type", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> optional email type preference for the email (html, text, or mobile defaults to html)</div>" }, { "name" : "double_optin", "type" : "xs:boolean", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> optional flag to control whether a double opt-in confirmation message is sent, defaults to true. <em>Abusing this may cause your account to be suspended.</em></div>" }, { "name" : "update_existing", "type" : "xs:boolean", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> optional flag to control whether a existing subscribers should be updated instead of throwing and error, defaults to false</div>" }, { "name" : "replace_interests", "type" : "xs:boolean", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> optional flag to determine whether we replace the interest groups with the groups provided, or we add the provided groups to the member's interest groups (optional, defaults to true)</div>" }, { "name" : "send_welcome", "type" : "xs:boolean", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> optional if your double_optin is false and this is true, we will send your lists Welcome Email if this subscribe succeeds - this will *not* fire if we end up updating an existing subscriber. If double_optin is true, this has no effect. defaults to false.</div>" } ], "returns" : { "type" : "item()*", "description" : "True on success, false on failure." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 6, "name" : "list-unsubscribe", "qname" : "mailchimp:list-unsubscribe", "signature" : "($apikey as xs:string, $id as xs:string, $email_address as xs:string, $delete_member as xs:boolean, $send_goodbye as xs:boolean, $send_notify as xs:boolean) as item()*", "description" : " Convenience function for <a xmlns:xqdoc=\"http://www.xqdoc.org/1.0\" href=\"#list-unsubscribe-7\">list-unsubscribe#7</a>.\n", "summary" : "<p> Convenience function for  list-unsubscribe#7 .</p>", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "apikey", "type" : "xs:string", "occurrence" : null, "description" : "" }, { "name" : "id", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the list id to connect to. Get by calling <a href=\"#lists-4\">lists()</a></div>" }, { "name" : "email_address", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the email address to unsubscribe OR the email \"id\" returned from listMemberInfo, Webhooks, and Campaigns</div>" }, { "name" : "delete_member", "type" : "xs:boolean", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> flag to completely delete the member from your list instead of just unsubscribing, default to false</div>" }, { "name" : "send_goodbye", "type" : "xs:boolean", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> flag to send the goodbye email to the email address, defaults to true</div>" }, { "name" : "send_notify", "type" : "xs:boolean", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> flag to send the unsubscribe notification email to the address defined in the list email notification settings, defaults to true</div>" } ], "returns" : { "type" : "item()*", "description" : "True on success, false on failure." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 7, "name" : "list-unsubscribe", "qname" : "mailchimp:list-unsubscribe", "signature" : "($endpoint-url as xs:string, $apikey as xs:string, $id as xs:string, $email_address as xs:string, $delete_member as xs:boolean, $send_goodbye as xs:boolean, $send_notify as xs:boolean) as item()*", "description" : " Unsubscribe the given email address from the list.\n", "summary" : "<p> Unsubscribe the given email address from the list.</p>", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "endpoint-url", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> Mailchimp endpoint URL, e.g. http://us1.api.mailchimp.com/1.3/</div>" }, { "name" : "apikey", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> a valid API Key for your user account. Get by visiting <a href=\"http://admin.mailchimp.com/account/api\" target=\"_blank\">your API dashboard</a></div>" }, { "name" : "id", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the list id to connect to. Get by calling <a href=\"#lists-4\">lists()</a></div>" }, { "name" : "email_address", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the email address to unsubscribe OR the email \"id\" returned from listMemberInfo, Webhooks, and Campaigns</div>" }, { "name" : "delete_member", "type" : "xs:boolean", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> flag to completely delete the member from your list instead of just unsubscribing, default to false</div>" }, { "name" : "send_goodbye", "type" : "xs:boolean", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> flag to send the goodbye email to the email address, defaults to true</div>" }, { "name" : "send_notify", "type" : "xs:boolean", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> flag to send the unsubscribe notification email to the address defined in the list email notification settings, defaults to true</div>" } ], "returns" : { "type" : "item()*", "description" : "True on success, false on failure." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 6, "name" : "list-update-member", "qname" : "mailchimp:list-update-member", "signature" : "($apikey as xs:string, $id as xs:string, $email_address as xs:string, $merge_vars as element(array), $email_type as xs:string, $replace_interests as xs:boolean) as item()*", "description" : " Convenience function for <a xmlns:xqdoc=\"http://www.xqdoc.org/1.0\" href=\"#list-update-member-7\">list-update-member#7</a>.\n", "summary" : "<p> Convenience function for  list-update-member#7 .</p>", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "apikey", "type" : "xs:string", "occurrence" : null, "description" : "" }, { "name" : "id", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the list id to connect to. Get by calling <a href=\"#lists-4\">lists()</a></div>" }, { "name" : "email_address", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the current email address of the member to update OR the \"id\" for the member returned from listMemberInfo, Webhooks, and Campaigns</div>" }, { "name" : "merge_vars", "type" : "element(array)", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> array of new field values to update the member with. See merge_vars in <a href=\"#list-subscribe-10\">list-subscribe()</a> for details.</div>" }, { "name" : "email_type", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> change the email type preference for the member (\"html\", \"text\", or \"mobile\"). Leave blank to keep the existing preference (optional)</div>" }, { "name" : "replace_interests", "type" : "xs:boolean", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> flag to determine whether we replace the interest groups with the updated groups provided, or we add the provided groups to the member's interest groups (optional, defaults to true)</div>" } ], "returns" : { "type" : "item()*", "description" : "True on success, false on failure." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 7, "name" : "list-update-member", "qname" : "mailchimp:list-update-member", "signature" : "($endpoint-url as xs:string, $apikey as xs:string, $id as xs:string, $email_address as xs:string, $merge_vars as element(array), $email_type as xs:string, $replace_interests as xs:boolean) as item()*", "description" : " Edit the email address, merge fields, and interest groups for a list member. If you are doing a batch update on lots of users, consider using <a xmlns:xqdoc=\"http://www.xqdoc.org/1.0\" href=\"#list-batch-subscribe-6\">#list-batch-subscribe()</a> with the update_existing and possible replace_interests parameter.\n", "summary" : "<p> Edit the email address, merge fields, and interest groups for a list member.</p>", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "endpoint-url", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> Mailchimp endpoint URL, e.g. http://us1.api.mailchimp.com/1.3/</div>" }, { "name" : "apikey", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> a valid API Key for your user account. Get by visiting <a href=\"http://admin.mailchimp.com/account/api\" target=\"_blank\">your API dashboard</a></div>" }, { "name" : "id", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the list id to connect to. Get by calling <a href=\"#lists-4\">lists()</a></div>" }, { "name" : "email_address", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the current email address of the member to update OR the \"id\" for the member returned from listMemberInfo, Webhooks, and Campaigns</div>" }, { "name" : "merge_vars", "type" : "element(array)", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> array of new field values to update the member with. See merge_vars in <a href=\"#list-subscribe-10\">list-subscribe()</a> for details.</div>" }, { "name" : "email_type", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> change the email type preference for the member (\"html\", \"text\", or \"mobile\"). Leave blank to keep the existing preference (optional)</div>" }, { "name" : "replace_interests", "type" : "xs:boolean", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> flag to determine whether we replace the interest groups with the updated groups provided, or we add the provided groups to the member's interest groups (optional, defaults to true)</div>" } ], "returns" : { "type" : "item()*", "description" : "True on success, false on failure." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 5, "name" : "list-webhook-add", "qname" : "mailchimp:list-webhook-add", "signature" : "($apikey as xs:string, $id as xs:string, $url as xs:string, $actions as element(array), $sources as element(array)) as item()*", "description" : " Convenience function for <a xmlns:xqdoc=\"http://www.xqdoc.org/1.0\" href=\"#list-webhook-add-6\">list-webhook-add#6</a>.\n", "summary" : "<p> Convenience function for  list-webhook-add#6 .</p>", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "apikey", "type" : "xs:string", "occurrence" : null, "description" : "" }, { "name" : "id", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the list id to connect to. Get by calling <a href=\"#lists-4\">lists()</a></div>" }, { "name" : "url", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> a valid URL for the Webhook - it will be validated. note that a url may only exist on a list once.</div>" }, { "name" : "actions", "type" : "element(array)", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> optional a hash of actions to fire this Webhook for</div>" }, { "name" : "sources", "type" : "element(array)", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> optional a hash of sources to fire this Webhook for</div>" } ], "returns" : { "type" : "item()*", "description" : "True if the call succeeds, otherwise an exception will be thrown" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 6, "name" : "list-webhook-add", "qname" : "mailchimp:list-webhook-add", "signature" : "($endpoint-url as xs:string, $apikey as xs:string, $id as xs:string, $url as xs:string, $actions as element(array), $sources as element(array)) as item()*", "description" : " Add a new Webhook URL for the given list.\n", "summary" : "<p> Add a new Webhook URL for the given list.</p>", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "endpoint-url", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> Mailchimp endpoint URL, e.g. http://us1.api.mailchimp.com/1.3/</div>" }, { "name" : "apikey", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> a valid API Key for your user account. Get by visiting <a href=\"http://admin.mailchimp.com/account/api\" target=\"_blank\">your API dashboard</a></div>" }, { "name" : "id", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the list id to connect to. Get by calling <a href=\"#lists-4\">lists()</a></div>" }, { "name" : "url", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> a valid URL for the Webhook - it will be validated. note that a url may only exist on a list once.</div>" }, { "name" : "actions", "type" : "element(array)", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> optional a hash of actions to fire this Webhook for</div>" }, { "name" : "sources", "type" : "element(array)", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> optional a hash of sources to fire this Webhook for</div>" } ], "returns" : { "type" : "item()*", "description" : "True if the call succeeds, otherwise an exception will be thrown" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 3, "name" : "list-webhook-del", "qname" : "mailchimp:list-webhook-del", "signature" : "($apikey as xs:string, $id as xs:string, $url as xs:string) as item()*", "description" : " Convenience function for <a xmlns:xqdoc=\"http://www.xqdoc.org/1.0\" href=\"#list-webhook-del-4\">list-webhook-del#4</a>.\n", "summary" : "<p> Convenience function for  list-webhook-del#4 .</p>", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "apikey", "type" : "xs:string", "occurrence" : null, "description" : "" }, { "name" : "id", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the list id to connect to. Get by calling <a href=\"#lists-4\">lists()</a></div>" }, { "name" : "url", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the URL of a Webhook on this list</div>" } ], "returns" : { "type" : "item()*", "description" : "True if the call succeeds, otherwise an exception will be thrown" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 4, "name" : "list-webhook-del", "qname" : "mailchimp:list-webhook-del", "signature" : "($endpoint-url as xs:string, $apikey as xs:string, $id as xs:string, $url as xs:string) as item()*", "description" : " Delete an existing Webhook URL from a given list.\n", "summary" : "<p> Delete an existing Webhook URL from a given list.</p>", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "endpoint-url", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> Mailchimp endpoint URL, e.g. http://us1.api.mailchimp.com/1.3/</div>" }, { "name" : "apikey", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> a valid API Key for your user account. Get by visiting <a href=\"http://admin.mailchimp.com/account/api\" target=\"_blank\">your API dashboard</a></div>" }, { "name" : "id", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the list id to connect to. Get by calling <a href=\"#lists-4\">lists()</a></div>" }, { "name" : "url", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the URL of a Webhook on this list</div>" } ], "returns" : { "type" : "item()*", "description" : "True if the call succeeds, otherwise an exception will be thrown" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 2, "name" : "list-webhooks", "qname" : "mailchimp:list-webhooks", "signature" : "($apikey as xs:string, $id as xs:string) as item()*", "description" : " Convenience function for <a xmlns:xqdoc=\"http://www.xqdoc.org/1.0\" href=\"#list-webhooks-3\">list-webhooks#3</a>.\n", "summary" : "<p> Convenience function for  list-webhooks#3 .</p>", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "apikey", "type" : "xs:string", "occurrence" : null, "description" : "" }, { "name" : "id", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the list id to connect to. Get by calling <a href=\"#lists-4\">lists()</a></div>" } ], "returns" : { "type" : "item()*", "description" : "List of webhooks" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 3, "name" : "list-webhooks", "qname" : "mailchimp:list-webhooks", "signature" : "($endpoint-url as xs:string, $apikey as xs:string, $id as xs:string) as item()*", "description" : " Return the Webhooks configured for the given list.\n", "summary" : "<p> Return the Webhooks configured for the given list.</p>", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "endpoint-url", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> Mailchimp endpoint URL, e.g. http://us1.api.mailchimp.com/1.3/</div>" }, { "name" : "apikey", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> a valid API Key for your user account. Get by visiting <a href=\"http://admin.mailchimp.com/account/api\" target=\"_blank\">your API dashboard</a></div>" }, { "name" : "id", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the list id to connect to. Get by calling <a href=\"#lists-4\">lists()</a></div>" } ], "returns" : { "type" : "item()*", "description" : "List of webhooks" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 2, "name" : "lists-for-email", "qname" : "mailchimp:lists-for-email", "signature" : "($apikey as xs:string, $email_address as xs:string) as item()*", "description" : " Convenience function for <a xmlns:xqdoc=\"http://www.xqdoc.org/1.0\" href=\"#lists-for-email-3\">lists-for-email#3</a>.\n", "summary" : "<p> Convenience function for  lists-for-email#3 .</p>", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "apikey", "type" : "xs:string", "occurrence" : null, "description" : "" }, { "name" : "email_address", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the email address to check OR the email \"id\" returned from listMemberInfo, Webhooks, and Campaigns</div>" } ], "returns" : { "type" : "item()*", "description" : "An array of list_ids the member is subscribed to." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 3, "name" : "lists-for-email", "qname" : "mailchimp:lists-for-email", "signature" : "($endpoint-url as xs:string, $apikey as xs:string, $email_address as xs:string) as item()*", "description" : " Retrieve all List Ids a member is subscribed to.\n", "summary" : "<p> Retrieve all List Ids a member is subscribed to.</p>", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "endpoint-url", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> Mailchimp endpoint URL, e.g. http://us1.api.mailchimp.com/1.3/</div>" }, { "name" : "apikey", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> a valid API Key for your user account. Get by visiting <a href=\"http://admin.mailchimp.com/account/api\" target=\"_blank\">your API dashboard</a></div>" }, { "name" : "email_address", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the email address to check OR the email \"id\" returned from listMemberInfo, Webhooks, and Campaigns</div>" } ], "returns" : { "type" : "item()*", "description" : "An array of list_ids the member is subscribed to." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 4, "name" : "lists", "qname" : "mailchimp:lists", "signature" : "($apikey as xs:string, $filters as element(array), $start as xs:integer, $limit as xs:integer) as item()*", "description" : " Convenience function for <a xmlns:xqdoc=\"http://www.xqdoc.org/1.0\" href=\"#lists-5\">lists#5</a>.\n", "summary" : "<p> Convenience function for  lists#5 .</p>", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "apikey", "type" : "xs:string", "occurrence" : null, "description" : "" }, { "name" : "filters", "type" : "element(array)", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> a hash of filters to apply to this query - all are optional. See <a href=\"http://apidocs.mailchimp.com/rtfm/lists.func.php\">Mailchimp API documentation</a> for more information about the parameter structure.</div>" }, { "name" : "start", "type" : "xs:integer", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> optional - control paging of lists, start results at this list #, defaults to 1st page of data (page 0)</div>" }, { "name" : "limit", "type" : "xs:integer", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> optional - control paging of lists, number of lists to return with each call, defaults to 25 (max=100)</div>" } ], "returns" : { "type" : "item()*", "description" : "Array with keys listed in Returned Fields below" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 5, "name" : "lists", "qname" : "mailchimp:lists", "signature" : "($endpoint-url as xs:string, $apikey as xs:string, $filters as element(array), $start as xs:integer, $limit as xs:integer) as item()*", "description" : " Retrieve all of the lists defined for your user account.\n", "summary" : "<p> Retrieve all of the lists defined for your user account.</p>", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "endpoint-url", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> Mailchimp endpoint URL, e.g. http://us1.api.mailchimp.com/1.3/</div>" }, { "name" : "apikey", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> a valid API Key for your user account. Get by visiting <a href=\"http://admin.mailchimp.com/account/api\" target=\"_blank\">your API dashboard</a></div>" }, { "name" : "filters", "type" : "element(array)", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> a hash of filters to apply to this query - all are optional. See <a href=\"http://apidocs.mailchimp.com/rtfm/lists.func.php\">Mailchimp API documentation</a> for more information about the parameter structure.</div>" }, { "name" : "start", "type" : "xs:integer", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> optional - control paging of lists, start results at this list #, defaults to 1st page of data (page 0)</div>" }, { "name" : "limit", "type" : "xs:integer", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> optional - control paging of lists, number of lists to return with each call, defaults to 25 (max=100)</div>" } ], "returns" : { "type" : "item()*", "description" : "Array with keys listed in Returned Fields below" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "ping", "qname" : "mailchimp:ping", "signature" : "($apikey as xs:string) as item()*", "description" : " Convenience function for <a xmlns:xqdoc=\"http://www.xqdoc.org/1.0\" href=\"#ping-2\">ping#2</a>.\n", "summary" : "<p> Convenience function for  ping#2 .</p>", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "apikey", "type" : "xs:string", "occurrence" : null, "description" : "" } ], "returns" : { "type" : "item()*", "description" : "Returns \"Everything's Chimpy!\" if everything is chimpy, otherwise returns an error message" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 2, "name" : "ping", "qname" : "mailchimp:ping", "signature" : "($endpoint-url as xs:string, $apikey as xs:string) as item()*", "description" : " \"Ping\" the MailChimp API - a simple method you can call that will return a constant value as long as everything is good. Note than unlike most all of our methods, we don't throw an Exception if we are having issues. You will simply receive a different string back that will explain our view on what is going on.\n", "summary" : "<p> \"Ping\" the MailChimp API - a simple method you can call that will return a constant value as long as everything is good.</p>", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "endpoint-url", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> Mailchimp endpoint URL, e.g. http://us1.api.mailchimp.com/1.3/</div>" }, { "name" : "apikey", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> a valid API Key for your user account. Get by visiting <a href=\"http://admin.mailchimp.com/account/api\" target=\"_blank\">your API dashboard</a></div>" } ], "returns" : { "type" : "item()*", "description" : "Returns \"Everything's Chimpy!\" if everything is chimpy, otherwise returns an error message" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 3, "name" : "template-add", "qname" : "mailchimp:template-add", "signature" : "($apikey as xs:string, $name as xs:string, $html as xs:string) as item()*", "description" : " Convenience function for <a xmlns:xqdoc=\"http://www.xqdoc.org/1.0\" href=\"#template-add-4\">template-add#4</a>.\n", "summary" : "<p> Convenience function for  template-add#4 .</p>", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "apikey", "type" : "xs:string", "occurrence" : null, "description" : "" }, { "name" : "name", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the name for the template - names must be unique and a max of 50 bytes</div>" }, { "name" : "html", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> a string specifying the entire template to be created. This is <strong>NOT</strong> campaign content. They are intended to utilize our <a href=\"http://www.mailchimp.com/resources/email-template-language/\" target=\"_blank\">template language</a>.</div>" } ], "returns" : { "type" : "item()*", "description" : "The new template id, otherwise an error is thrown." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 4, "name" : "template-add", "qname" : "mailchimp:template-add", "signature" : "($endpoint-url as xs:string, $apikey as xs:string, $name as xs:string, $html as xs:string) as item()*", "description" : " Create a new user template, <strong xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">NOT</strong> campaign content. These templates can then be applied while creating campaigns.\n", "summary" : "<p> Create a new user template,  NOT  campaign content.</p>", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "endpoint-url", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> Mailchimp endpoint URL, e.g. http://us1.api.mailchimp.com/1.3/</div>" }, { "name" : "apikey", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> a valid API Key for your user account. Get by visiting <a href=\"http://admin.mailchimp.com/account/api\" target=\"_blank\">your API dashboard</a></div>" }, { "name" : "name", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the name for the template - names must be unique and a max of 50 bytes</div>" }, { "name" : "html", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> a string specifying the entire template to be created. This is <strong>NOT</strong> campaign content. They are intended to utilize our <a href=\"http://www.mailchimp.com/resources/email-template-language/\" target=\"_blank\">template language</a>.</div>" } ], "returns" : { "type" : "item()*", "description" : "The new template id, otherwise an error is thrown." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 2, "name" : "template-del", "qname" : "mailchimp:template-del", "signature" : "($apikey as xs:string, $id as xs:integer) as item()*", "description" : " Convenience function for <a xmlns:xqdoc=\"http://www.xqdoc.org/1.0\" href=\"#template-del-3\">template-del#3</a>.\n", "summary" : "<p> Convenience function for  template-del#3 .</p>", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "apikey", "type" : "xs:string", "occurrence" : null, "description" : "" }, { "name" : "id", "type" : "xs:integer", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the id of the user template to delete</div>" } ], "returns" : { "type" : "item()*", "description" : "True if the template was deleted, otherwise an error will be thrown" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 3, "name" : "template-del", "qname" : "mailchimp:template-del", "signature" : "($endpoint-url as xs:string, $apikey as xs:string, $id as xs:integer) as item()*", "description" : " Delete (deactivate) a user template.\n", "summary" : "<p> Delete (deactivate) a user template.</p>", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "endpoint-url", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> Mailchimp endpoint URL, e.g. http://us1.api.mailchimp.com/1.3/</div>" }, { "name" : "apikey", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> a valid API Key for your user account. Get by visiting <a href=\"http://admin.mailchimp.com/account/api\" target=\"_blank\">your API dashboard</a></div>" }, { "name" : "id", "type" : "xs:integer", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the id of the user template to delete</div>" } ], "returns" : { "type" : "item()*", "description" : "True if the template was deleted, otherwise an error will be thrown" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 3, "name" : "template-info", "qname" : "mailchimp:template-info", "signature" : "($apikey as xs:string, $tid as xs:integer, $type as xs:string) as item()*", "description" : " Convenience function for <a xmlns:xqdoc=\"http://www.xqdoc.org/1.0\" href=\"#template-info-4\">template-info#4</a>.\n", "summary" : "<p> Convenience function for  template-info#4 .</p>", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "apikey", "type" : "xs:string", "occurrence" : null, "description" : "" }, { "name" : "tid", "type" : "xs:integer", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the template id - get from templates()</div>" }, { "name" : "type", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the template type to load - one of 'user', 'gallery', 'base'</div>" } ], "returns" : { "type" : "item()*", "description" : "An array of info to be used when editing" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 4, "name" : "template-info", "qname" : "mailchimp:template-info", "signature" : "($endpoint-url as xs:string, $apikey as xs:string, $tid as xs:integer, $type as xs:string) as item()*", "description" : " Pull details for a specific template to help support editing.\n", "summary" : "<p> Pull details for a specific template to help support editing.</p>", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "endpoint-url", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> Mailchimp endpoint URL, e.g. http://us1.api.mailchimp.com/1.3/</div>" }, { "name" : "apikey", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> a valid API Key for your user account. Get by visiting <a href=\"http://admin.mailchimp.com/account/api\" target=\"_blank\">your API dashboard</a></div>" }, { "name" : "tid", "type" : "xs:integer", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the template id - get from templates()</div>" }, { "name" : "type", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the template type to load - one of 'user', 'gallery', 'base'</div>" } ], "returns" : { "type" : "item()*", "description" : "An array of info to be used when editing" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 2, "name" : "template-undel", "qname" : "mailchimp:template-undel", "signature" : "($apikey as xs:string, $id as xs:integer) as item()*", "description" : " Convenience function for <a xmlns:xqdoc=\"http://www.xqdoc.org/1.0\" href=\"#template-undel-3\">template-undel#3</a>.\n", "summary" : "<p> Convenience function for  template-undel#3 .</p>", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "apikey", "type" : "xs:string", "occurrence" : null, "description" : "" }, { "name" : "id", "type" : "xs:integer", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the id of the user template to reactivate</div>" } ], "returns" : { "type" : "item()*", "description" : "boolean true if the template was deleted, otherwise an error will be thrown" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 3, "name" : "template-undel", "qname" : "mailchimp:template-undel", "signature" : "($endpoint-url as xs:string, $apikey as xs:string, $id as xs:integer) as item()*", "description" : " Undelete (reactivate) a user template.\n", "summary" : "<p> Undelete (reactivate) a user template.</p>", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "endpoint-url", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> Mailchimp endpoint URL, e.g. http://us1.api.mailchimp.com/1.3/</div>" }, { "name" : "apikey", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> a valid API Key for your user account. Get by visiting <a href=\"http://admin.mailchimp.com/account/api\" target=\"_blank\">your API dashboard</a></div>" }, { "name" : "id", "type" : "xs:integer", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the id of the user template to reactivate</div>" } ], "returns" : { "type" : "item()*", "description" : "boolean true if the template was deleted, otherwise an error will be thrown" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 3, "name" : "template-update", "qname" : "mailchimp:template-update", "signature" : "($apikey as xs:string, $id as xs:integer, $values as element(array)) as item()*", "description" : " Convenience function for <a xmlns:xqdoc=\"http://www.xqdoc.org/1.0\" href=\"#template-update-4\">template-update#4</a>.\n", "summary" : "<p> Convenience function for  template-update#4 .</p>", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "apikey", "type" : "xs:string", "occurrence" : null, "description" : "" }, { "name" : "id", "type" : "xs:integer", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the id of the user template to update</div>" }, { "name" : "values", "type" : "element(array)", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> s the values to updates - while both are optional, at least one should be provided. Both can be updated at the same time.</div>" } ], "returns" : { "type" : "item()*", "description" : "True if the template was updated, otherwise an error will be thrown" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 4, "name" : "template-update", "qname" : "mailchimp:template-update", "signature" : "($endpoint-url as xs:string, $apikey as xs:string, $id as xs:integer, $values as element(array)) as item()*", "description" : " Replace the content of a user template, <strong xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">NOT</strong> campaign content.\n", "summary" : "<p> Replace the content of a user template,  NOT  campaign content.</p>", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "endpoint-url", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> Mailchimp endpoint URL, e.g. http://us1.api.mailchimp.com/1.3/</div>" }, { "name" : "apikey", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> a valid API Key for your user account. Get by visiting <a href=\"http://admin.mailchimp.com/account/api\" target=\"_blank\">your API dashboard</a></div>" }, { "name" : "id", "type" : "xs:integer", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the id of the user template to update</div>" }, { "name" : "values", "type" : "element(array)", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> s the values to updates - while both are optional, at least one should be provided. Both can be updated at the same time.</div>" } ], "returns" : { "type" : "item()*", "description" : "True if the template was updated, otherwise an error will be thrown" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 4, "name" : "templates", "qname" : "mailchimp:templates", "signature" : "($apikey as xs:string, $types as element(array), $category as xs:string, $inactives as element(array)) as item()*", "description" : " Convenience function for <a xmlns:xqdoc=\"http://www.xqdoc.org/1.0\" href=\"#templates-5\">templates#5</a>.\n", "summary" : "<p> Convenience function for  templates#5 .</p>", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "apikey", "type" : "xs:string", "occurrence" : null, "description" : "" }, { "name" : "types", "type" : "element(array)", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> optional the types of templates to return</div>" }, { "name" : "category", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> optional for Gallery templates only, limit to a specific template category</div>" }, { "name" : "inactives", "type" : "element(array)", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> optional options to control how inactive templates are returned, if at all</div>" } ], "returns" : { "type" : "item()*", "description" : "An array of structs, one for each template (see Returned Fields for details)" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 5, "name" : "templates", "qname" : "mailchimp:templates", "signature" : "($endpoint-url as xs:string, $apikey as xs:string, $types as element(array), $category as xs:string, $inactives as element(array)) as item()*", "description" : " Retrieve various templates available in the system, allowing some thing similar to our template gallery to be created. boolean user Customer template for this user account. Defaults to true. boolean gallery Templates from our Gallery. Note that some templates that require extra configuration are withheld. (eg, the Etsy template). Defaults to false. boolean base Our \"start from scratch\" extremely basic templates boolean include user templates are not deleted, only set inactive. defaults to false. boolean only only include inactive templates. defaults to false.\n", "summary" : "<p> Retrieve various templates available in the system, allowing some thing similar to our template gallery to be created.</p>", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "endpoint-url", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> Mailchimp endpoint URL, e.g. http://us1.api.mailchimp.com/1.3/</div>" }, { "name" : "apikey", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> a valid API Key for your user account. Get by visiting <a href=\"http://admin.mailchimp.com/account/api\" target=\"_blank\">your API dashboard</a></div>" }, { "name" : "types", "type" : "element(array)", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> optional the types of templates to return</div>" }, { "name" : "category", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> optional for Gallery templates only, limit to a specific template category</div>" }, { "name" : "inactives", "type" : "element(array)", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> optional options to control how inactive templates are returned, if at all</div>" } ], "returns" : { "type" : "item()*", "description" : "An array of structs, one for each template (see Returned Fields for details)" }, "errors" : [  ] } ], "variables" : [ { "name" : "mailchimp:gateway", "type" : "item()*", "description" : " Mailchimp gateway where the client data is stored.\n" }, { "name" : "mailchimp:version", "type" : "item()*", "description" : " Mailchimp API version.\n" }, { "name" : "mailchimp:endpoint", "type" : "item()*", "description" : " Endpoint URL for the XML-RPC requests.\n" } ] }, "http://www.zorba-xquery.com/modules/couchbase" : { "ns" : "http://www.zorba-xquery.com/modules/couchbase", "description" : " This module provides minimal functionality to interact with the\n Couchbase NoSQL database.\n The module is built using the libcouchbase C client library and\n exposes most of its functionality in XQuery with JSONiq extensions.\n Beyond just allowing for basic key-value store operations (e.g.\n put-/get-text or put-/get-binary, this module also allows to work\n with Couchbase views in order to allow for complex JSON query\n operations.\n", "sees" : [  ], "authors" : [ "<xqdoc:author xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Juan Zacarias</xqdoc:author>" ], "version" : null, "encoding" : "utf-8", "namespaces" : [ { "uri" : "http://zorba.io/annotations", "prefix" : "an" }, { "uri" : "http://www.zorba-xquery.com/modules/couchbase", "prefix" : "cb" }, { "uri" : "http://jsoniq.org/functions", "prefix" : "jn" }, { "uri" : "http://zorba.io/options/versioning", "prefix" : "ver" } ], "functions" : [ { "isDocumented" : true, "arity" : 1, "name" : "connect", "qname" : "cb:connect", "signature" : "($options as object()) as xs:anyURI external", "description" : " Connect to the Couchbase server and return an opaque identifier\n representing the established connection.\n", "summary" : "<p> Connect to the Couchbase server and return an opaque identifier\n representing the established connection.</p>", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "options", "type" : "object()", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> a JSONiq object that contains the host, bucket, and user information.</div>" } ], "returns" : { "type" : "xs:anyURI", "description" : "an identifier for the established connection. Example: <code xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> { \"host\": \"localhost:8091\", \"username\" : null, \"password\" : null, \"bucket\" : \"default\" } </code>" }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cb:LCB0001 if the connection to the given host/bucket could not be established.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cb:CB0001 if mandatory connection information is missing.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cb:CB0007 if a given option is not supported.</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 4, "name" : "connect", "qname" : "cb:connect", "signature" : "($host as xs:string, $username as xs:string?, $password as xs:string?, $bucket as xs:string) as xs:anyURI", "description" : " Connect to the Couchbase server and return an opaque identifier\n representing the established connection.\n", "summary" : "<p> Connect to the Couchbase server and return an opaque identifier\n representing the established connection.</p>", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "host", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> address of the couchbase server (e.g. \"localhost:8091\")</div>" }, { "name" : "username", "type" : "xs:string", "occurrence" : "?", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> username used for the connection</div>" }, { "name" : "password", "type" : "xs:string", "occurrence" : "?", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> password used for the connection</div>" }, { "name" : "bucket", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> name of the bucket to use (e.g. \"default\")</div>" } ], "returns" : { "type" : "xs:anyURI", "description" : "an identifier for the established connection." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cb:LCB0001 if the connection to the given host/bucket could not be established.</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 3, "name" : "create-view", "qname" : "cb:create-view", "signature" : "($db as xs:anyURI, $doc-name as xs:string, $view-names as xs:string*) as xs:string* external", "description" : " Create a document/view.\n If the document already exists, it is replaced. A document can hold several\n views that must be specified in the same call of cb:create-view.\n", "summary" : "<p> Create a document/view.</p>", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "db", "type" : "xs:anyURI", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> connection reference</div>" }, { "name" : "doc-name", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> name of the document to create.</div>" }, { "name" : "view-names", "type" : "xs:string", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> names of the views to create in the document.</div>" } ], "returns" : { "type" : "xs:string*", "description" : "the names of the paths for the views that have been created." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cb:LCB0002 if any error occurs in the communication with the server.</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 4, "name" : "create-view", "qname" : "cb:create-view", "signature" : "($db as xs:anyURI, $doc-name as xs:string, $view-names as xs:string*, $options as object()*) as xs:string* external", "description" : " Create a document/view.\n If the document already exists, it is replaced. A document can hold several\n views that must be specified in the same call of cb:create-view.\n", "summary" : "<p> Create a document/view.</p>", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "db", "type" : "xs:anyURI", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> connection reference</div>" }, { "name" : "doc-name", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> name of the document to create.</div>" }, { "name" : "view-names", "type" : "xs:string", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> names of the views to create in the document.</div>" }, { "name" : "options", "type" : "object()", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> options describing how to create the view.</div>" } ], "returns" : { "type" : "xs:string*", "description" : "the names of the paths for the views that have been created." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cb:LCB0002 if any error occurs in the communication with the server.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cb:CB0005 if the number of options doesn't match the number of view-names.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cb:CB0007 if any of the options is not supported.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cb:CB0010 if any of the given options has an invalid type.</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 2, "name" : "delete-view", "qname" : "cb:delete-view", "signature" : "($db as xs:anyURI, $doc as xs:string*) as xs:string* external", "description" : " Delete a document/view.\n If the document doesn't exists, function does nothing. All the views hold in the\n Document are deleted, this function can't delete single views.\n", "summary" : "<p> Delete a document/view.</p>", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "db", "type" : "xs:anyURI", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> connection reference</div>" }, { "name" : "doc", "type" : "xs:string", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">-name name of the document to create.</div>" } ], "returns" : { "type" : "xs:string*", "description" : "empty sequence." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cb:LCB0002 if any error occurs in the communication with the server.</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 1, "name" : "flush", "qname" : "cb:flush", "signature" : "($db as xs:anyURI) as empty-sequence() external", "description" : " Remove all key/value pairs from the cluster\n", "summary" : "<p> Remove all key/value pairs from the cluster\n</p>", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "db", "type" : "xs:anyURI", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> connection reference</div>" } ], "returns" : { "type" : "empty-sequence()", "description" : "a empty sequence." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cb:LCB0002 if any error occurs in the communication with the server.</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 2, "name" : "get-binary", "qname" : "cb:get-binary", "signature" : "($db as xs:anyURI, $key as xs:string*) as xs:base64Binary* external", "description" : " Return the values of the given keys (type xs:string) as base64Binary.\n", "summary" : "<p> Return the values of the given keys (type xs:string) as base64Binary.</p>", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "db", "type" : "xs:anyURI", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> connection reference</div>" }, { "name" : "key", "type" : "xs:string", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the requested keys</div>" } ], "returns" : { "type" : "xs:base64Binary*", "description" : "a sequence of xs:base64Binary items for the given keys." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cb:LCB0002 if any error occurs in the communication with the server.</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 3, "name" : "get-binary", "qname" : "cb:get-binary", "signature" : "($db as xs:anyURI, $key as xs:string*, $options as object()) as xs:base64Binary* external", "description" : " Return the values of the given keys (type xs:string) as base64Binary.\n", "summary" : "<p> Return the values of the given keys (type xs:string) as base64Binary.</p>", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "db", "type" : "xs:anyURI", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> connection reference</div>" }, { "name" : "key", "type" : "xs:string", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the requested keys</div>" }, { "name" : "options", "type" : "object()", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> JSONiq object with additional options</div>" } ], "returns" : { "type" : "xs:base64Binary*", "description" : "a sequence of xs:base64Binary items for the given keys." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cb:LCB0002 if any error occurs in the communication with the server.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cb:CB0007 if any of the options is not supported.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cb:CB0009 if the given expiration time is not an xs:integer.</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 2, "name" : "get-text", "qname" : "cb:get-text", "signature" : "($db as xs:anyURI, $key as xs:string*) as xs:string* external", "description" : " Return the values of the given keys (type xs:string) as string.\n", "summary" : "<p> Return the values of the given keys (type xs:string) as string.</p>", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "db", "type" : "xs:anyURI", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> connection reference</div>" }, { "name" : "key", "type" : "xs:string", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the requested keys</div>" } ], "returns" : { "type" : "xs:string*", "description" : "A sequence of string Items corresponding to the key" }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cb:LCB0002 if any error occurs in the communication with the server.</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 3, "name" : "get-text", "qname" : "cb:get-text", "signature" : "($db as xs:anyURI, $key as xs:string*, $options as object()) as xs:string* external", "description" : " Return the values of the given keys (type xs:string) as string.\n", "summary" : "<p> Return the values of the given keys (type xs:string) as string.</p>", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "db", "type" : "xs:anyURI", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> connection reference</div>" }, { "name" : "key", "type" : "xs:string", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the requested keys</div>" }, { "name" : "options", "type" : "object()", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> JSONiq object with additional options</div>" } ], "returns" : { "type" : "xs:string*", "description" : "a sequence of strings for the given keys." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cb:LCB0002 if any error occurs in the communication with the server.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cb:CB0006 if the given encoding is not supported.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cb:CB0007 if any of the options is not supported.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cb:CB0009 if the given expiration time is not an xs:integer.</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 3, "name" : "put-binary", "qname" : "cb:put-binary", "signature" : "($db as xs:anyURI, $key as xs:string*, $value as xs:base64Binary*) as empty-sequence()", "description" : " Store the given key-value bindings.\n", "summary" : "<p> Store the given key-value bindings.</p>", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "db", "type" : "xs:anyURI", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> connection reference</div>" }, { "name" : "key", "type" : "xs:string", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the keys to store</div>" }, { "name" : "value", "type" : "xs:base64Binary", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the values (as xs:base64binary) to be stored.</div>" } ], "returns" : { "type" : "empty-sequence()", "description" : "a empty sequence." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cb:LCB0002 if any error occurs in the communication with the server.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cb:CB0005 if the number of keys doesn't match the number of values.</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 4, "name" : "put-binary", "qname" : "cb:put-binary", "signature" : "($db as xs:anyURI, $key as xs:string*, $value as xs:base64Binary*, $options as object()) as empty-sequence() external", "description" : " Store the given key-value bindings.\n", "summary" : "<p> Store the given key-value bindings.</p>", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "db", "type" : "xs:anyURI", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> connection reference</div>" }, { "name" : "key", "type" : "xs:string", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the keys to store</div>" }, { "name" : "value", "type" : "xs:base64Binary", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the values (as xs:base64binary) to be stored.</div>" }, { "name" : "options", "type" : "object()", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> JSONiq object with additional options</div>" } ], "returns" : { "type" : "empty-sequence()", "description" : "a empty sequence." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cb:LCB0002 if any error occurs in the communication with the server.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cb:CB0005 if the number of keys doesn't match the number of values.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cb:CB0007 if any of the options is not supported.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cb:CB0009 if the given expiration time is not an xs:integer.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cb:CB0011 if the stored Variable was not stored</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 3, "name" : "put-text", "qname" : "cb:put-text", "signature" : "($db as xs:anyURI, $key as xs:string*, $value as xs:string*) as empty-sequence()", "description" : " Store the given key-value bindings.\n The values are stored with the UTF-8 encoding and a default\n expiration time of 60 seconds.\n", "summary" : "<p> Store the given key-value bindings.</p>", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "db", "type" : "xs:anyURI", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> connection reference</div>" }, { "name" : "key", "type" : "xs:string", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the keys to store</div>" }, { "name" : "value", "type" : "xs:string", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the values (as xs:string) to be stored.</div>" } ], "returns" : { "type" : "empty-sequence()", "description" : "a empty sequence." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cb:LCB0002 if any error occurs in the communication with the server.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cb:CB0005 if the number of keys doesn't match the number of values.</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 4, "name" : "put-text", "qname" : "cb:put-text", "signature" : "($db as xs:anyURI, $key as xs:string*, $value as xs:string*, $options as object()) as empty-sequence() external", "description" : " Store the given key-value bindings.\n", "summary" : "<p> Store the given key-value bindings.</p>", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "db", "type" : "xs:anyURI", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> connection reference</div>" }, { "name" : "key", "type" : "xs:string", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the keys to store</div>" }, { "name" : "value", "type" : "xs:string", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the values (as xs:string) to be stored.</div>" }, { "name" : "options", "type" : "object()", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> JSONiq object with additional options</div>" } ], "returns" : { "type" : "empty-sequence()", "description" : "a empty sequence." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cb:LCB0002 if any error occurs in the communication with the server.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cb:CB0005 if the number of keys doesn't match the number of values.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cb:CB0006 if the given encoding is not supported.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cb:CB0007 if any of the options is not supported.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cb:CB0009 if the given expiration time is not an xs:integer.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cb:CB0011 if the stored Variable was not stored</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 2, "name" : "remove", "qname" : "cb:remove", "signature" : "($db as xs:anyURI, $key as xs:string*) as empty-sequence() external", "description" : " Remove the values matching the given keys (xs:string) from the server.\n", "summary" : "<p> Remove the values matching the given keys (xs:string) from the server.</p>", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "db", "type" : "xs:anyURI", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> connection reference</div>" }, { "name" : "key", "type" : "xs:string", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the keys of the values that should be removed.</div>" } ], "returns" : { "type" : "empty-sequence()", "description" : "a empty sequence." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cb:LCB0002 if any error occurs in the communication with the server.</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 3, "name" : "touch", "qname" : "cb:touch", "signature" : "($db as xs:anyURI, $key as xs:string*, $exp-time as xs:integer) as empty-sequence() external", "description" : " Refresh the expiration time of the given keys.\n", "summary" : "<p> Refresh the expiration time of the given keys.</p>", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "db", "type" : "xs:anyURI", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> connection reference</div>" }, { "name" : "key", "type" : "xs:string", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the keys to touch</div>" }, { "name" : "exp-time", "type" : "xs:integer", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> new expieration time in seconds</div>" } ], "returns" : { "type" : "empty-sequence()", "description" : "a empty sequence." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cb:LCB0002 if any error occurs in the communication with the server.</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 2, "name" : "view", "qname" : "cb:view", "signature" : "($db as xs:anyURI, $path as xs:string*) as object()*", "description" : " Retrieve the content of existing views.\n", "summary" : "<p> Retrieve the content of existing views.</p>", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "db", "type" : "xs:anyURI", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> connection reference</div>" }, { "name" : "path", "type" : "xs:string", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> contains the string of a view path (e.g. \"_design/test/_view/vies\").</div>" } ], "returns" : { "type" : "object()*", "description" : "a sequence of strings (as JSON) containing information of the views." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cb:LCB0002 if any error occurs in the communication with the server.</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 3, "name" : "view", "qname" : "cb:view", "signature" : "($db as xs:anyURI, $path as xs:string*, $options as object()) as object()*", "description" : " Retrieve the content of existing views.\n", "summary" : "<p> Retrieve the content of existing views.</p>", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "db", "type" : "xs:anyURI", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> connection reference</div>" }, { "name" : "path", "type" : "xs:string", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> contains the string of a view path (e.g. \"_design/test/_view/vies\").</div>" }, { "name" : "options", "type" : "object()", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> JSONiq object with additional options</div>" } ], "returns" : { "type" : "object()*", "description" : "a sequence of strings (as JSON) containing information of the views." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cb:LCB0002 if any error occurs in the communication with the server.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cb:CB0007 if any of the options is not supported.</xqdoc:error>" ] } ], "variables" : [  ] }, "http://zorba.io/modules/excel/text" : { "ns" : "http://zorba.io/modules/excel/text", "description" : " This is a library module offering the same set of functions\n defined by Microsoft Excel, under Text and Data Functions.\n", "sees" : [ "<xqdoc:see xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"><a href=\"http://office.microsoft.com/en-us/excel/CH062528321033.aspx\" target=\"_blank\">Excel Documentation: Text Functions</a></xqdoc:see>" ], "authors" : [ "<xqdoc:author xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Sorin Nasoi</xqdoc:author>" ], "version" : null, "encoding" : "utf-8", "namespaces" : [ { "uri" : "http://zorba.io/modules/excel/errors", "prefix" : "excel-err" }, { "uri" : "http://zorba.io/modules/excel/math", "prefix" : "excel-math" }, { "uri" : "http://zorba.io/modules/excel/text", "prefix" : "excel-text" }, { "uri" : "http://zorba.io/options/versioning", "prefix" : "ver" } ], "functions" : [ { "isDocumented" : true, "arity" : 1, "name" : "asc", "qname" : "excel-text:asc", "signature" : "($text as xs:string) as xs:string", "description" : " Returns the given $text unchanged.\n", "summary" : "<p> Returns the given $text unchanged.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "text", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the time</div>" } ], "returns" : { "type" : "xs:string", "description" : "The given $text unchanged." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "char", "qname" : "excel-text:char", "signature" : "($number as xs:integer) as xs:string", "description" : " Returns the character specified by a certain codepoint.\n", "summary" : "<p> Returns the character specified by a certain codepoint.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "number", "type" : "xs:integer", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the codepoint.</div>" } ], "returns" : { "type" : "xs:string", "description" : "the character specified by a certain codepoint." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">excel-err:Value provided $number must be in range [1,255].</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 1, "name" : "clean", "qname" : "excel-text:clean", "signature" : "($arg as xs:string?) as xs:string?", "description" : " Removes all nonprintable characters from text.\n", "summary" : "<p> Removes all nonprintable characters from text.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "arg", "type" : "xs:string", "occurrence" : "?", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the string.</div>" } ], "returns" : { "type" : "xs:string?", "description" : "Removes all nonprintable characters from text. The CLEAN function was designed. to remove the first 32 nonprinting characters in the 7-bit ASCII code (values 0 through 31) from text. <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/> In the Unicode character set, there are additional nonprinting characters (values 127, 129, 141, 143, 144, and 157). <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/> By itself, the CLEAN function does not remove these additional nonprinting characters." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "code", "qname" : "excel-text:code", "signature" : "($arg as xs:string) as xs:integer", "description" : " Returns a codepoint for the first character in a text string.\n", "summary" : "<p> Returns a codepoint for the first character in a text string.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "arg", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the string.</div>" } ], "returns" : { "type" : "xs:integer", "description" : "A codepoint for the first character in a text string." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">excel-err:Value Provided $arg was empty.</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 1, "name" : "concatenate", "qname" : "excel-text:concatenate", "signature" : "($args as xs:anyAtomicType*) as xs:string", "description" : " Joins several text strings into one text string.\n", "summary" : "<p> Joins several text strings into one text string.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "args", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> a sequence of strings.</div>" } ], "returns" : { "type" : "xs:string", "description" : "Joins several text strings into one text string." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 2, "name" : "concatenate", "qname" : "excel-text:concatenate", "signature" : "($arg1 as xs:anyAtomicType?, $arg2 as xs:anyAtomicType?) as xs:string", "description" : " Joins two text strings into one text string.\n", "summary" : "<p> Joins two text strings into one text string.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "arg1", "type" : "xs:anyAtomicType", "occurrence" : "?", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the first string.</div>" }, { "name" : "arg2", "type" : "xs:anyAtomicType", "occurrence" : "?", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the second string.</div>" } ], "returns" : { "type" : "xs:string", "description" : "Joins two text strings into one text string." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "dollar", "qname" : "excel-text:dollar", "signature" : "($number as xs:decimal) as xs:string", "description" : " Converts a number to text format and applies a currency symbol. The number of\ndigits to the right of the decimal point is 2.\n", "summary" : "<p> Converts a number to text format and applies a currency symbol.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "number", "type" : "xs:decimal", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> is the number.</div>" } ], "returns" : { "type" : "xs:string", "description" : "Converts a number to text format and applies a currency symbol. The number of digits to the right of the decimal point is 2." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 2, "name" : "dollar", "qname" : "excel-text:dollar", "signature" : "($number as xs:decimal, $decimals as xs:decimal) as xs:string", "description" : " Converts a number to text format and applies a currency symbol.\n", "summary" : "<p> Converts a number to text format and applies a currency symbol.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "number", "type" : "xs:decimal", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> is the number.</div>" }, { "name" : "decimals", "type" : "xs:decimal", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> is the number of digits to the right of the decimal point. <p/> If decimals is negative, number is rounded to the left of the decimal point.</div>" } ], "returns" : { "type" : "xs:string", "description" : "Converts a number to text format and applies a currency symbol." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 2, "name" : "exact", "qname" : "excel-text:exact", "signature" : "($arg1 as xs:string, $arg2 as xs:string) as xs:boolean", "description" : " Compares two text strings and returns TRUE if they are exactly the same,\nFALSE otherwise. EXACT is case-sensitive but ignores formatting differences.\n", "summary" : "<p> Compares two text strings and returns TRUE if they are exactly the same,\nFALSE otherwise.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "arg1", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the first string.</div>" }, { "name" : "arg2", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the second string.</div>" } ], "returns" : { "type" : "xs:boolean", "description" : "Compares two text strings and returns TRUE if they are exactly the same, FALSE otherwise. EXACT is case-sensitive but ignores formatting differences." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 2, "name" : "find", "qname" : "excel-text:find", "signature" : "($find_text as xs:string, $within_text as xs:string) as xs:integer?", "description" : " Locate one text string within a second text string, and return the number of the\n starting position of the first text string from the first character of the second text string. <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n The search is case sensitive.\n", "summary" : "<p> Locate one text string within a second text string, and return the number of the\n starting position of the first text string from the first character of the second text string.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "find_text", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> text you want to find.</div>" }, { "name" : "within_text", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> text in which you want to search for $find_text.</div>" } ], "returns" : { "type" : "xs:integer?", "description" : "Locate one text string within a second text string, and return the number of the starting position of the first text string from the first character of the second text string. <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/> The search is case sensitive." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">excel-err:Value the value is not greater than zero or is greater than the length of within_text.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">excel-err:Value value was not found.</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 3, "name" : "find", "qname" : "excel-text:find", "signature" : "($find_text as xs:string, $within_text as xs:string, $start_num as xs:integer) as xs:integer?", "description" : " Locate one text string within a second text string, and return the number of the\n starting position of the first text string from the first character of the second text string.<p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n The search is case sensitive.\n", "summary" : "<p> Locate one text string within a second text string, and return the number of the\n starting position of the first text string from the first character of the second text string.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "find_text", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> text you want to find.</div>" }, { "name" : "within_text", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> text in which you want to search for $find_text.</div>" }, { "name" : "start_num", "type" : "xs:integer", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> specifies the character at which to start the search.</div>" } ], "returns" : { "type" : "xs:integer?", "description" : "Locate one text string within a second text string, and return the number of the starting position of the first text string from the first character of the second text string.<p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/> The search is case sensitive." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">excel-err:Value the value is not greater than zero or is greater than the length of within_text.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">excel-err:Value value was not found.</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 2, "name" : "fixed", "qname" : "excel-text:fixed", "signature" : "($number as xs:decimal, $decimals as xs:decimal) as xs:string", "description" : " Rounds a number to the specified number of decimals, formats the number in\n decimal format using a period and commas, and returns the result as text.\n", "summary" : "<p> Rounds a number to the specified number of decimals, formats the number in\n decimal format using a period and commas, and returns the result as text.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "number", "type" : "xs:decimal", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> is the number you want to round and convert to text.</div>" }, { "name" : "decimals", "type" : "xs:decimal", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> is the number of digits to the right of the decimal point.</div>" } ], "returns" : { "type" : "xs:string", "description" : "Rounds a number to the specified number of decimals, formats the number in decimal format using a period and commas, and returns the result as text." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 3, "name" : "fixed", "qname" : "excel-text:fixed", "signature" : "($number as xs:decimal, $decimals as xs:decimal, $no_commas as xs:boolean) as xs:string", "description" : " Rounds a number to the specified number of decimals, formats the number in\ndecimal format using a period and commas, and returns the result as text.\n", "summary" : "<p> Rounds a number to the specified number of decimals, formats the number in\ndecimal format using a period and commas, and returns the result as text.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "number", "type" : "xs:decimal", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> is the number you want to round and convert to text.</div>" }, { "name" : "decimals", "type" : "xs:decimal", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> is the number of digits to the right of the decimal point.</div>" }, { "name" : "no_commas", "type" : "xs:boolean", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> is a logical value that, if TRUE, prevents FIXED from including commas in the returned text.</div>" } ], "returns" : { "type" : "xs:string", "description" : "Rounds a number to the specified number of decimals, formats the number in decimal format using a period and commas, and returns the result as text." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "left", "qname" : "excel-text:left", "signature" : "($arg as xs:string) as xs:string", "description" : " Returns the first character in a text string.\n", "summary" : "<p> Returns the first character in a text string.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "arg", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> is the text string that contains the characters you want to extract.</div>" } ], "returns" : { "type" : "xs:string", "description" : "The first character in a text string." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 2, "name" : "left", "qname" : "excel-text:left", "signature" : "($text as xs:string, $num_chars as xs:integer) as xs:string", "description" : " Returns the first character or characters in $text, based on the number of $num_chars you specify.\n", "summary" : "<p> Returns the first character or characters in $text, based on the number of $num_chars you specify.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "text", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> is the text string that contains the characters you want to extract.</div>" }, { "name" : "num_chars", "type" : "xs:integer", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> specifies the number of characters you want to extract.</div>" } ], "returns" : { "type" : "xs:string", "description" : "The first character or characters in $text, based on the number of $num_chars you specify." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "len", "qname" : "excel-text:len", "signature" : "($arg as xs:string?) as xs:integer", "description" : " Returns the number of characters in a text string.\n", "summary" : "<p> Returns the number of characters in a text string.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "arg", "type" : "xs:string", "occurrence" : "?", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the string.</div>" } ], "returns" : { "type" : "xs:integer", "description" : "The number of characters in a text string." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "lower", "qname" : "excel-text:lower", "signature" : "($arg as xs:string?) as xs:string?", "description" : " Converts all uppercase letters in a text string to lowercase.\n", "summary" : "<p> Converts all uppercase letters in a text string to lowercase.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "arg", "type" : "xs:string", "occurrence" : "?", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the string.</div>" } ], "returns" : { "type" : "xs:string?", "description" : "Converts all uppercase letters in a text string to lowercase." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 3, "name" : "mid", "qname" : "excel-text:mid", "signature" : "($text as xs:string?, $start_num as xs:integer, $num_chars as xs:integer) as xs:string?", "description" : " Returns a specific number of characters from a text string, starting at\nthe position you specify, based on the number of characters you specify.\n", "summary" : "<p> Returns a specific number of characters from a text string, starting at\nthe position you specify, based on the number of characters you specify.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "text", "type" : "xs:string", "occurrence" : "?", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the text string containing the characters you want to extract.</div>" }, { "name" : "start_num", "type" : "xs:integer", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the position of the first character you want to extract in text. The first character in text has start_num 1, and so on.</div>" }, { "name" : "num_chars", "type" : "xs:integer", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the number of characters you want to return from text.</div>" } ], "returns" : { "type" : "xs:string?", "description" : "A specific number of characters from a text string, starting at the position you specify, based on the number of characters you specify." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 3, "name" : "pad-integer-to-length", "qname" : "excel-text:pad-integer-to-length", "signature" : "($toPad as xs:anyAtomicType?, $padChar as xs:string, $length as xs:integer) as xs:string", "description" : " Returns $toPad appended with enough repetitions of $padChar to make its length $length, the characters are added before the string.\n", "summary" : "<p> Returns $toPad appended with enough repetitions of $padChar to make its length $length, the characters are added before the string.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "toPad", "type" : "xs:anyAtomicType", "occurrence" : "?", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the value to be padded.</div>" }, { "name" : "padChar", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the character used for padding.</div>" }, { "name" : "length", "type" : "xs:integer", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the desired length.</div>" } ], "returns" : { "type" : "xs:string", "description" : "$toPad appended with enough repetitions of $padChar to make its length $length, the characters are added before the string." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">excel-err:Value if the length of the $toPad is greater than the desired length.</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 4, "name" : "replace", "qname" : "excel-text:replace", "signature" : "($old_text as xs:string?, $start_num as xs:integer, $num_chars as xs:integer, $new_text as xs:string) as xs:string", "description" : " Replaces part of a text string, based on the number of characters you specify, with a different text string.\n", "summary" : "<p> Replaces part of a text string, based on the number of characters you specify, with a different text string.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "old_text", "type" : "xs:string", "occurrence" : "?", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> is text in which you want to replace some characters.</div>" }, { "name" : "start_num", "type" : "xs:integer", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the position of the character in old_text that you want to replace with new_text.</div>" }, { "name" : "num_chars", "type" : "xs:integer", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the number of characters in old_text that you want REPLACE to replace with new_text.</div>" }, { "name" : "new_text", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the text that will replace characters in old_text.</div>" } ], "returns" : { "type" : "xs:string", "description" : "Replaces part of a text string, based on the number of characters you specify, with a different text string." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "right", "qname" : "excel-text:right", "signature" : "($arg as xs:string) as xs:string", "description" : " Returns the last character in a text string.\n", "summary" : "<p> Returns the last character in a text string.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "arg", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the text string containing the characters you want to extract.</div>" } ], "returns" : { "type" : "xs:string", "description" : "The last character in a text string." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 2, "name" : "right", "qname" : "excel-text:right", "signature" : "($text as xs:string, $num_chars as xs:integer) as xs:string", "description" : " Returns the last character or characters in a text string, based on the number of characters you specify.\n", "summary" : "<p> Returns the last character or characters in a text string, based on the number of characters you specify.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "text", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the text string containing the characters you want to extract.</div>" }, { "name" : "num_chars", "type" : "xs:integer", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> specifies the number of characters you want RIGHT to extract.</div>" } ], "returns" : { "type" : "xs:string", "description" : "The last character or characters in a text string, based on the number of characters you specify." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 2, "name" : "search", "qname" : "excel-text:search", "signature" : "($find_text as xs:string, $within_text as xs:string) as xs:integer?", "description" : " Locate one text string within a second text string, and return the number of\n the starting position of the first text string from the first character of the\n second text string. <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/> The search starts at position 1, and it is not case sensitive.\n", "summary" : "<p> Locate one text string within a second text string, and return the number of\n the starting position of the first text string from the first character of the\n second text string.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "find_text", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> text you want to find.</div>" }, { "name" : "within_text", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> text in which you want to search for $find_text.</div>" } ], "returns" : { "type" : "xs:integer?", "description" : "Locate one text string within a second text string, and return the number of the starting position of the first text string from the first character of the second text string. <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/> The search starts at position 1, and it is not case sensitive." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">excel-err:Value the value is not greater than zero or is greater than the length of within_text.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">excel-err:Value value was not found.</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 3, "name" : "search", "qname" : "excel-text:search", "signature" : "($find_text as xs:string, $within_text as xs:string, $start_num as xs:integer) as xs:integer?", "description" : " Locate one text string within a second text string, and return the number of\n the starting position of the first text string from the first character of the\n second text string.<p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/> The search starts at $start_num, and it is not case sensitive.\n", "summary" : "<p> Locate one text string within a second text string, and return the number of\n the starting position of the first text string from the first character of the\n second text string.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "find_text", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> text you want to find.</div>" }, { "name" : "within_text", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> text in which you want to search for $find_text.</div>" }, { "name" : "start_num", "type" : "xs:integer", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the character number in within_text at which you want to start searching.</div>" } ], "returns" : { "type" : "xs:integer?", "description" : "Locate one text string within a second text string, and return the number of the starting position of the first text string from the first character of the second text string.<p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/> The search starts at $start_num, and it is not case sensitive." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">excel-err:Value the value is not greater than zero or is greater than the length of within_text.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">excel-err:Value value was not found.</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 3, "name" : "substitute", "qname" : "excel-text:substitute", "signature" : "($text as xs:string, $old_text as xs:string, $new_text as xs:string) as xs:string?", "description" : " Substitutes new_text for old_text in a text string. Every occurrence of old_text in text is changed to new_text.\n", "summary" : "<p> Substitutes new_text for old_text in a text string.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "text", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the text or the reference to a cell containing text for which you want to substitute characters.</div>" }, { "name" : "old_text", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> text you want to replace.</div>" }, { "name" : "new_text", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> text you want to replace old_text with.</div>" } ], "returns" : { "type" : "xs:string?", "description" : "Substitutes new_text for old_text in a text string. Every occurrence of old_text in text is changed to new_text." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 4, "name" : "substitute", "qname" : "excel-text:substitute", "signature" : "($text as xs:string, $old_text as xs:string, $new_text as xs:string, $instance_num as xs:integer?) as xs:string", "description" : " Substitutes new_text for old_text in a text string.\n", "summary" : "<p> Substitutes new_text for old_text in a text string.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "text", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the text or the reference to a cell containing text for which you want to substitute characters.</div>" }, { "name" : "old_text", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> text you want to replace.</div>" }, { "name" : "new_text", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> text you want to replace old_text with.</div>" }, { "name" : "instance_num", "type" : "xs:integer", "occurrence" : "?", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> specifies which occurrence of old_text you want to replace with new_text. <p/> Only that instance of old_text is replaced.</div>" } ], "returns" : { "type" : "xs:string", "description" : "Substitutes new_text for old_text in a text string. <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/> Use SUBSTITUTE when you want to replace specific text in a text string; use REPLACE when you want to replace any text that occurs in a specific location in a text string." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "t", "qname" : "excel-text:t", "signature" : "($value as xs:anyAtomicType?) as xs:string", "description" : " Converts the $value to string.\n", "summary" : "<p> Converts the $value to string.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "value", "type" : "xs:anyAtomicType", "occurrence" : "?", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the value</div>" } ], "returns" : { "type" : "xs:string", "description" : "Converts the $value to string." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "trim", "qname" : "excel-text:trim", "signature" : "($text as xs:string?) as xs:string?", "description" : " Removes all spaces from text except for single spaces between words.\n", "summary" : "<p> Removes all spaces from text except for single spaces between words.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "text", "type" : "xs:string", "occurrence" : "?", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> from which you want spaces removed.</div>" } ], "returns" : { "type" : "xs:string?", "description" : "Removes all spaces from text except for single spaces between words." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "upper", "qname" : "excel-text:upper", "signature" : "($text as xs:string?) as xs:string?", "description" : " Converts text to uppercase.\n", "summary" : "<p> Converts text to uppercase.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "text", "type" : "xs:string", "occurrence" : "?", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> text you want converted to uppercase.</div>" } ], "returns" : { "type" : "xs:string?", "description" : "Converts text to uppercase." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 2, "name" : "value-except", "qname" : "excel-text:value-except", "signature" : "($arg1 as xs:anyAtomicType*, $arg2 as xs:anyAtomicType*) as xs:anyAtomicType*", "description" : " Returns the values in one sequence that do not appear in the second sequence in an implementation-defined order.\n", "summary" : "<p> Returns the values in one sequence that do not appear in the second sequence in an implementation-defined order.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "arg1", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the first sequence.</div>" }, { "name" : "arg2", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the second sequence.</div>" } ], "returns" : { "type" : "xs:anyAtomicType*", "description" : "The values in one sequence that do not appear in the second sequence in an implementation-defined order." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "value", "qname" : "excel-text:value", "signature" : "($arg as xs:anyAtomicType?) as xs:anyAtomicType?", "description" : " Converts a text string that represents a number to a number.\n", "summary" : "<p> Converts a text string that represents a number to a number.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "arg", "type" : "xs:anyAtomicType", "occurrence" : "?", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the value.</div>" } ], "returns" : { "type" : "xs:anyAtomicType?", "description" : "Converts a text string that represents a number to a number." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">excel-err:Value provided value is not a number.</xqdoc:error>" ] } ], "variables" : [  ] }, "http://www.28msec.com/modules/http/response" : { "ns" : "http://www.28msec.com/modules/http/response", "description" : " The Sausalito response module can be used to modify the HTTP response\n that will be send as a result of this request. For example, the\n <tt xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">set-content-type</tt> function can be used to set the Content-Type\n header and determine the encoding of data in the response.\n", "sees" : [  ], "authors" : [ "<xqdoc:author xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">28msec</xqdoc:author>" ], "version" : null, "encoding" : "utf-8", "namespaces" : [ { "uri" : "http://zorba.io/annotations", "prefix" : "an" }, { "uri" : "http://www.28msec.com/modules/http/response", "prefix" : "http" }, { "uri" : "http://www.w3.org/2010/xslt-xquery-serialization", "prefix" : "output" }, { "uri" : "http://www.28msec.com/modules/http/response", "prefix" : "resp" }, { "uri" : "http://zorba.io/modules/schema", "prefix" : "s" }, { "uri" : "http://zorba.io/options/versioning", "prefix" : "ver" } ], "functions" : [ { "isDocumented" : true, "arity" : 1, "name" : "code-for-status", "qname" : "resp:code-for-status", "signature" : "($status as xs:QName) as xs:int", "description" : " Convert between the HTTP status code as QName and\n the integer value of that status code.\n", "summary" : "<p> Convert between the HTTP status code as QName and\n the integer value of that status code.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "status", "type" : "xs:QName", "occurrence" : null, "description" : "" } ], "returns" : { "type" : "xs:int", "description" : "the integer value of the given QName as xs:int" }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">http:invalid-status if the given QName does not represent a valid http status code</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 1, "name" : "content-type-binary", "qname" : "resp:content-type-binary", "signature" : "($type as xs:string) as xs:boolean external", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Test if a given content-type is a binary content-type. A\n content-type is considered to be binary if it's not a text\n content-type.</p>\n", "summary" : "<p>  Test if a given content-type is a binary content-type.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "type", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the content-type to test.</div>" } ], "returns" : { "type" : "xs:boolean", "description" : "true if the given content-type is a binary content-type, false otherwise." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "content-type-html", "qname" : "resp:content-type-html", "signature" : "($type as xs:string) as xs:boolean external", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Test if a given content-type is a HTML content-type, i.e.\n the content-type is the string \"text/html\".</p>\n", "summary" : "<p>  Test if a given content-type is a HTML content-type, i.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "type", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the content-type to test.</div>" } ], "returns" : { "type" : "xs:boolean", "description" : "true if the given content-type is the HTML content-type, false otherwise." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "content-type-json", "qname" : "resp:content-type-json", "signature" : "($type as xs:string) as xs:boolean external", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Test if a given content-type is a JSON content-type, i.e.\n the content-type is the string \"application/json\".</p>\n", "summary" : "<p>  Test if a given content-type is a JSON content-type, i.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "type", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the content-type to test.</div>" } ], "returns" : { "type" : "xs:boolean", "description" : "true if the given content-type is the JSON content-type, false otherwise." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "content-type-mixed-json-xml", "qname" : "resp:content-type-mixed-json-xml", "signature" : "($type as xs:string) as xs:boolean external", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Test if a given content-type is a mixed JSON-XML content-type, i.e.\n the content-type is the string \"application/mixed-json-xml\".</p>\n", "summary" : "<p>  Test if a given content-type is a mixed JSON-XML content-type, i.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "type", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the content-type to test.</div>" } ], "returns" : { "type" : "xs:boolean", "description" : "true if the given content-type is a mixed JSON-XML content-type, false otherwise." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "content-type-text", "qname" : "resp:content-type-text", "signature" : "($type as xs:string) as xs:boolean external", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Test if a given content-type is a text content-type.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">A text content-type starts with \"text/\" or contains either of\n the strings \"xml\" or \"json\".</p>\n", "summary" : "<p>  Test if a given content-type is a text content-type.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "type", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the content-type to test.</div>" } ], "returns" : { "type" : "xs:boolean", "description" : "true if the given content-type is a text content-type, false otherwise." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "content-type-xhtml", "qname" : "resp:content-type-xhtml", "signature" : "($type as xs:string) as xs:boolean external", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Test if a given content-type is a XHTML content-type, i.e.\n the content-type is the string \"application/xhtml+xml\".</p>\n", "summary" : "<p>  Test if a given content-type is a XHTML content-type, i.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "type", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the content-type to test.</div>" } ], "returns" : { "type" : "xs:boolean", "description" : "true if the given content-type is the XHTML content-type, false otherwise." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "content-type-xml", "qname" : "resp:content-type-xml", "signature" : "($type as xs:string) as xs:boolean external", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Test if a given content-type is a XML content-type</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">A XML content-type is a content-type that is \"application/xml\"\n or ends : with the string \"+xml\".</p>\n", "summary" : "<p>  Test if a given content-type is a XML content-type \n  A XML content-type is a content-type that is \"application/xml\"\n or ends : with the string \"+xml\".</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "type", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the content-type to test.</div>" } ], "returns" : { "type" : "xs:boolean", "description" : "true if the given content-type is a XML content-type, false otherwise." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 0, "name" : "decode-binary", "qname" : "resp:decode-binary", "signature" : "() as xs:boolean external", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Returns the boolean indiciating whether base64Binaries returned by\n this request will be decoded.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">The default if not modified using resp:set-decode-binary is false.</p>\n", "summary" : "<p>  Returns the boolean indiciating whether base64Binaries returned by\n this request will be decoded.</p>", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [  ], "returns" : { "type" : "xs:boolean", "description" : "the said boolean value" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "message-for-status", "qname" : "resp:message-for-status", "signature" : "($status as xs:QName) as xs:string", "description" : " Convert between the HTTP status code as QName and\n the name/message of that status code.\n", "summary" : "<p> Convert between the HTTP status code as QName and\n the name/message of that status code.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "status", "type" : "xs:QName", "occurrence" : null, "description" : "" } ], "returns" : { "type" : "xs:string", "description" : "the message of the given QName as xs:string" }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">http:invalid-status if the given QName does not represent a valid http status code</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 0, "name" : "serialization-parameters", "qname" : "resp:serialization-parameters", "signature" : "() as element(output:serialization-parameters)", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Returns the serialization parameters that are currently\n active, i.e. the ones that will be used to serialize the result\n of this request.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">This will either be the default, the defaults set when\n calling resp:set-content-type(), or the ones set by\n resp:set-serialization-parameters().</p>\n", "summary" : "<p>  Returns the serialization parameters that are currently\n active, i.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [  ], "returns" : { "type" : "element(output:serialization-parameters)", "description" : "the said serialization parameters" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 0, "name" : "serializer-defaults-html", "qname" : "resp:serializer-defaults-html", "signature" : "() as element(*)*", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Returns an element that can be used to specify the\n settings of serialization parameters for the HTML serialization\n method.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">The defaults are\n <ul>\n  <li>encoding: UTF-8</li>\n  <li>indent: yes</li>\n  <li>version: 4.01</li>\n  <li>doctype-system: http://www.w3.org/TR/html4/loose.dtd</li>\n  <li>doctype-public: -//W3C//DTD HTML 4.01 Transitional//EN</li>\n  <li>include-content-type: yes</li>\n </ul>\n </p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">For example, those defaults are used when setting\n the content-type to some HTML content-type (i.e. text/html).</p>\n", "summary" : "<p>  Returns an element that can be used to specify the\n settings of serialization parameters for the HTML serialization\n method.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [  ], "returns" : { "type" : "element(*)*", "description" : "an element that can be used to specify the above serialization parameters." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 0, "name" : "serializer-defaults-json-xml-hybrid", "qname" : "resp:serializer-defaults-json-xml-hybrid", "signature" : "() as element(output:serialization-parameters)", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Returns an element that can be used to specify the\n settings of serialization parameters for the json-xml-hybrid\n serialization method.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">The defaults are\n <ul>\n  <li>encoding: UTF-8</li>\n  <li>indent: yes</li>\n  <li>method: json-xml-hybrid</li>\n  <li>omit-xml-declaration: yes</li>\n  <li>version: 1.0</li>\n </ul>\n </p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">For example, those defaults are used when setting\n the content-type to application/mixed-json-xml.</p>\n", "summary" : "<p>  Returns an element that can be used to specify the\n settings of serialization parameters for the json-xml-hybrid\n serialization method.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [  ], "returns" : { "type" : "element(output:serialization-parameters)", "description" : "an element that can be used to specify the above serialization parameters." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 0, "name" : "serializer-defaults-json", "qname" : "resp:serializer-defaults-json", "signature" : "() as element(output:serialization-parameters)", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Returns an element that can be used to specify the\n settings of serialization parameters for the json\n serialization method.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">The defaults are\n <ul>\n  <li>encoding: UTF-8</li>\n  <li>indent: yes</li>\n  <li>method: json</li>\n </ul>\n </p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">For example, those defaults are used when setting\n the content-type to some JSON content-type (e.g. application/json).</p>\n", "summary" : "<p>  Returns an element that can be used to specify the\n settings of serialization parameters for the json\n serialization method.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [  ], "returns" : { "type" : "element(output:serialization-parameters)", "description" : "an element that can be used to specify the above serialization parameters." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 0, "name" : "serializer-defaults-text", "qname" : "resp:serializer-defaults-text", "signature" : "() as element(*)*", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Returns an element that can be used to specify the\n settings of serialization parameters for the text serialization\n method.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">The only default used is\n <ul>\n  <li>encoding: UTF-8</li>\n </ul>\n </p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">For example, this default is used when setting\n the content-type to some text content-type (e.g. text/plain).</p>\n", "summary" : "<p>  Returns an element that can be used to specify the\n settings of serialization parameters for the text serialization\n method.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [  ], "returns" : { "type" : "element(*)*", "description" : "an element that can be used to specify the above serialization parameters." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 0, "name" : "serializer-defaults-xhtml", "qname" : "resp:serializer-defaults-xhtml", "signature" : "() as element(*)*", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Returns an element that can be used to specify the\n settings of serialization parameters for the XHTML serialization\n method.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">The defaults are\n <ul>\n  <li>encoding: UTF-8</li>\n  <li>indent: yes</li>\n  <li>omit-xml-declaration: yes</li>\n  <li>version: 1.0</li>\n  <li>doctype-system: http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd</li>\n  <li>doctype-public: -//W3C//DTD XHTML 1.0 Transitional//EN</li>\n </ul>\n </p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">For example, those defaults are used when setting\n the content-type to some XHTML content-type (i.e. application/xhtml+xml).</p>\n", "summary" : "<p>  Returns an element that can be used to specify the\n settings of serialization parameters for the XHTML serialization\n method.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [  ], "returns" : { "type" : "element(*)*", "description" : "an element that can be used to specify the above serialization parameters." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 0, "name" : "serializer-defaults-xml", "qname" : "resp:serializer-defaults-xml", "signature" : "() as element(output:serialization-parameters)", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Returns an element that can be used to specify the\n settings of serialization parameters for the XML serialization\n method.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">The defaults are\n <ul>\n  <li>encoding: UTF-8</li>\n  <li>indent: yes</li>\n  <li>omit-xml-declaration: yes</li>\n  <li>version: 1.0</li>\n </ul>\n </p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">For example, those defaults are used when setting\n the content-type to some XML content-type (e.g. application/atom+xml).</p>\n", "summary" : "<p>  Returns an element that can be used to specify the\n settings of serialization parameters for the XML serialization\n method.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [  ], "returns" : { "type" : "element(output:serialization-parameters)", "description" : "an element that can be used to specify the above serialization parameters." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "set-content-type", "qname" : "resp:set-content-type", "signature" : "($type as xs:string) as empty-sequence()", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Sets the Content-Type header for the response.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">\n For example,\n <tt>resp:set-content-type(\"text/plain\")</tt> will cause the\n header\n <tt>Content-Type: text/plain</tt> to be added for the response.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">In addtion to setting the Content-Type header, the function also\n sets the serialization parameters to the default values for serialization\n method for the given content-type. For example, if the content-type is set to\n text/plain, the default serialization parameters for the text\n serialization method will be used. For application/atom+xml, the default\n serialization parameters for the XML serialization method will be used.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">The functions resp:content-type-text/xml/html/xhtml/binary may be used\n to figure out which serialization method will be used for a specific\n content-type.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">If the given content-type contains a charset declaration (e.g.\n <tt>resp:set-content-type(\"text/plain;charset=ISO-8859-1\")</tt>), the\n content of the response will be transcoded to the given encoding.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">The default serialization parameters set by this function can\n be overwritten using the resp:set-serialization-parameters() function.</p>\n", "summary" : "<p>  Sets the Content-Type header for the response.</p>", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "type", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the content-type to be set</div>" } ], "returns" : { "type" : "empty-sequence()", "description" : "the function is sequential and returns the empty sequence" }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">http:invalid-encoding if the given encoding is invalid or not supported.</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 2, "name" : "set-content-type", "qname" : "resp:set-content-type", "signature" : "($type as xs:string, $params as element(output:serialization-parameters)) as empty-sequence()", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Sets the Content-Type header for the response.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">For example,\n <tt>resp:set-content-type(\"text/plain\")</tt> will cause the\n header\n <tt>Content-Type: text/plain</tt> to be added for the response.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">In addition to the content-type, the function also allows to\n specify the serialization method and parameters that will be used\n for serializing the result. For details about this parameter, please\n refer to resp:set-serialization-parameters().</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">If the given content-type contains a charset declaration (e.g.\n <tt>resp:set-content-type(\"text/plain;charset=ISO-8859-1\")</tt>), the\n content of the response will be transcoded to the given encoding.\n If the serialization parameters also contain a charset declaration,\n the charset contained in the content-type will be used. to transcode\n the result.</p>\n", "summary" : "<p>  Sets the Content-Type header for the response.</p>", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "type", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the content-type to be set</div>" }, { "name" : "params", "type" : "element(output:serialization-parameters)", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the serialization parameters that will be used for serialization the result.</div>" } ], "returns" : { "type" : "empty-sequence()", "description" : "the function is sequential and returns the empty sequence" }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">http:invalid-encoding if the given encoding is invalid or not supported.</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 1, "name" : "set-decode-binary", "qname" : "resp:set-decode-binary", "signature" : "($decode as xs:boolean) as empty-sequence() external", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">If this function is invoked with true, the result of the request\n will not be serialized. Instead, the result will be the binary values\n of any item that is of type xs:base64Binary. The values of all other\n items will not be part of the result.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Please note that all serialization parameters which have been set using\n resp:set-serialization-parameters will be ignored. Also, the output encoding\n is ignored for binaries.</p>\n", "summary" : "<p>  If this function is invoked with true, the result of the request\n will not be serialized.</p>", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "decode", "type" : "xs:boolean", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> boolean value indicating whether base64Binaries should be decoded.</div>" } ], "returns" : { "type" : "empty-sequence()", "description" : "the function is sequential and returns the empty sequence." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "set-encoding", "qname" : "resp:set-encoding", "signature" : "($encoding as xs:string) as empty-sequence() external", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">The function sets the output encoding that will be\n used for the payload of the response.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">For example,\n <tt>resp:set-encoding(\"ISO-8859-1\")</tt> will cause the\n content in the response to be encoded using ISO-8859-1.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Please note that the encoding only applies to textual data.\n It is not used if the function <tt>set-decode-binary()</tt>\n was invoked passing true as parameter.</p>\n", "summary" : "<p>  The function sets the output encoding that will be\n used for the payload of the response.</p>", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "encoding", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the encoding to be used for the payload of the response.</div>" } ], "returns" : { "type" : "empty-sequence()", "description" : "the function is sequential and returns the empty sequence" }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">http:invalid-encoding if the given encoding is invalid or not supported.</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 2, "name" : "set-header-impl", "qname" : "resp:set-header-impl", "signature" : "($name as xs:string, $value as xs:string) as xs:string? external", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Set an HTTP header in the response.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">This function is only used for internal purposes and should\n not be called by the user.</p>\n", "summary" : "<p>  Set an HTTP header in the response.</p>", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "name", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the name for the header to set</div>" }, { "name" : "value", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the value for the header to set</div>" } ], "returns" : { "type" : "xs:string?", "description" : "the value of the header previously set or the empty sequence if no header has been set with the same name." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 2, "name" : "set-header", "qname" : "resp:set-header", "signature" : "($name as xs:string, $value as xs:string) as xs:string?", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Set a HTTP header in the response.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">If a header with the same name was already set, the\n value is overwritten and the function returns the old value. As defined\n in the HTTP specification, multiple headers with the same name\n can be combined into one header whose value is a comma-separated\n list of the values.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">The following headers must not be set using this function.\n Instead, other functions of this module should be used in order\n to implemented the required semantics:\n <ul>\n   <li>Status: use set-status() instead</li>\n   <li>Content-Type: use set-content-type() instead</li>\n </ul>\n </p>\n", "summary" : "<p>  Set a HTTP header in the response.</p>", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "name", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the name of the header to set</div>" }, { "name" : "value", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the value of the header to set `</div>" } ], "returns" : { "type" : "xs:string?", "description" : "the value of the header previously set or the empty sequence if no header has been set with the same name." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">resp:invalid-header-name if an invalid name is used for the header.</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 1, "name" : "set-redirect", "qname" : "resp:set-redirect", "signature" : "($url as xs:string) as empty-sequence()", "description" : " This function sets the HTTP 302 redirect status code in the response. As\n a result, a redirect to the URL given as parameter will be made.\n", "summary" : "<p> This function sets the HTTP 302 redirect status code in the response.</p>", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "url", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The URL to which the redirect will be made.</div>" } ], "returns" : { "type" : "empty-sequence()", "description" : "The empty-sequence is returned." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "set-serialization-parameters", "qname" : "resp:set-serialization-parameters", "signature" : "($params as element(output:serialization-parameters)) as empty-sequence()", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Set the serialization parameters used for serializing the result\n of the request.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">The following example shows how to set several options\n for the HTML serialization method. Specifically, it sets the\n HTML method to HTML 4.01, set the doctype-system and\n doctype-public makes sure that the output is indented. In addition,\n the output is UTF-8 encoded and\n <pre>\n &lt;output:serialization-parameters&gt;\n   &lt;output:encoding value=\"UTF-8\"/&gt;\n   &lt;output:doctype-system value=\"http://www.w3.org/TR/html4/loose.dtd\"/&gt;\n   &lt;output:doctype-public value=\"-//W3C//DTD HTML 4.01 Transitional//EN\"/&gt;\n &lt;/output:serialization-parameters&gt;\n </pre>\n </p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">The element passed as parameter need to be valid according\n to the schema http://www.w3.org/2010/xslt-xquery-serialization. This\n function validates its input (if it has not already been validated)\n and may raise an error if the input is not valid.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Please note that serialization options set by this function can\n be overwritten by a subsequent call to resp:set-content-type. In this\n case, the options will be reset to the default for the given\n content-type. Also note, that the output encoding can be overwritten\n by subsequently calling the resp:set-encoding function.</p>\n", "summary" : "<p>  Set the serialization parameters used for serializing the result\n of the request.</p>", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "params", "type" : "element(output:serialization-parameters)", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the serialization parameters that will be used to serialize the result of the request</div>" } ], "returns" : { "type" : "empty-sequence()", "description" : "the function is sequential and, on success, returns the empty sequence." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">err:XQDY0027 if the input is not valid according to the schema http://www.w3.org/2010/xslt-xquery-serialization.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">http:invalid-encoding if the encoding specified in the serialization options is invalid or not supported.</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 1, "name" : "set-status-code", "qname" : "resp:set-status-code", "signature" : "($status as xs:integer) as empty-sequence() external", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Sets the status code of the HTTP response to the integer given as\n parameter.</p>\n For example,\n <tt xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">resp:set-status(204)</tt>\n will result in \"HTTP/1.1 204 No Content\".\n", "summary" : "<p>  Sets the status code of the HTTP response to the integer given as\n parameter.</p>", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "status", "type" : "xs:integer", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The status code of the HTTP response as integer.</div>" } ], "returns" : { "type" : "empty-sequence()", "description" : "On success, the empty-sequence is returned" }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">http:invalid-status-code if the given integer does not reflect a valid HTTP status code</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 1, "name" : "set-status", "qname" : "resp:set-status", "signature" : "($status as xs:QName) as empty-sequence()", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Sets the status code of the HTTP response to the QName given as\n parameter.</p>\n For example,\n <tt xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">resp:set-status($http:no-content)</tt>\n will result in \"HTTP/1.1 204 No Content\".\n", "summary" : "<p>  Sets the status code of the HTTP response to the QName given as\n parameter.</p>", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "status", "type" : "xs:QName", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The status code of the HTTP response as a QName (e.g. $http:no-content)</div>" } ], "returns" : { "type" : "empty-sequence()", "description" : "On success, the empty-sequence is returned" }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">http:invalid-status if the given QName does not represent a valid HTTP status code;</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 1, "name" : "valid-status", "qname" : "resp:valid-status", "signature" : "($status as xs:QName) as xs:boolean", "description" : " Test whether the given QName refers to a valid HTTP status code.\n", "summary" : "<p> Test whether the given QName refers to a valid HTTP status code.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "status", "type" : "xs:QName", "occurrence" : null, "description" : "" } ], "returns" : { "type" : "xs:boolean", "description" : "true if the given QName refers to a valid HTTP status code, false otherwise." }, "errors" : [  ] } ], "variables" : [ { "name" : "http:precondition-required", "type" : "xs:QName", "description" : " QName representing the HTTP Status Code 428 Precondition Required (RFC 6585)\n" }, { "name" : "http:gone", "type" : "xs:QName", "description" : " QName representing the HTTP Status Code 410 GONE\n" }, { "name" : "http:length-required", "type" : "xs:QName", "description" : " QName representing the HTTP Status Code 411 Length Required\n" }, { "name" : "http:precondition-failed", "type" : "xs:QName", "description" : " QName representing the HTTP Status Code 412 Precondition Failed\n" }, { "name" : "http:request-entity-too-large", "type" : "xs:QName", "description" : " QName representing the HTTP Status Code 413 Request Entity Too Large\n" }, { "name" : "http:request-entity-too-long", "type" : "xs:QName", "description" : " QName representing the HTTP Status Code 414 Request Entity Too Long\n" }, { "name" : "http:unsupported-media-type", "type" : "xs:QName", "description" : " QName representing the HTTP Status Code 415 Unsupported Media Type\n" }, { "name" : "http:request-range-not-satisfiable", "type" : "xs:QName", "description" : " QName representing the HTTP Status Code 416 Request Range Not Satisfiable\n" }, { "name" : "http:expectation-failed", "type" : "xs:QName", "description" : " QName representing the HTTP Status Code 417 Expectation Failed\n" }, { "name" : "http:im-a-teapot", "type" : "xs:QName", "description" : " QName representing the HTTP Status Code 418 I'm a teapot (RFC 2324)\n" }, { "name" : "http:enhance-your-calm", "type" : "xs:QName", "description" : " QName representing the HTTP Status Code 420 Enhance Your Calm\n" }, { "name" : "http:unprocessable-entity", "type" : "xs:QName", "description" : " QName representing the HTTP Status Code 422 Unprocessable Entity (WebDAV; RFC 4918)\n" }, { "name" : "http:locked", "type" : "xs:QName", "description" : " QName representing the HTTP Status Code 423 Locked (WebDAV; RFC 4918)\n" }, { "name" : "http:failed-dependency", "type" : "xs:QName", "description" : " QName representing the HTTP Status Code 424 Failed Dependency (WebDAV; RFC 4918)\n" }, { "name" : "http:unordered-collection", "type" : "xs:QName", "description" : " QName representing the HTTP Status Code 425 Unordered Collection\n" }, { "name" : "http:upgrade-required", "type" : "xs:QName", "description" : " QName representing the HTTP Status Code 426 Upgrade Required\n" }, { "name" : "http:conflict", "type" : "xs:QName", "description" : " QName representing the HTTP Status Code 409 Conflict\n" }, { "name" : "http:too-many-requests", "type" : "xs:QName", "description" : " QName representing the HTTP Status Code 429 Too Many Requests (RFC 6585)\n" }, { "name" : "http:request-header-fields-too-large", "type" : "xs:QName", "description" : " QName representing the HTTP Status Code 431 Request Header Fields Too Large (RFC 6585)\n" }, { "name" : "http:internal-server-error", "type" : "xs:QName", "description" : " QName representing the HTTP Status Code 500 Internal Server Error\n" }, { "name" : "http:not-implemented", "type" : "xs:QName", "description" : " QName representing the HTTP Status Code 501 Not Implemented\n" }, { "name" : "http:bad-gateway", "type" : "xs:QName", "description" : " QName representing the HTTP Status Code 502 Bad Gateway\n" }, { "name" : "http:service-unavailable", "type" : "xs:QName", "description" : " QName representing the HTTP Status Code 503 Service Unavailable\n" }, { "name" : "http:gateway-timeout", "type" : "xs:QName", "description" : " QName representing the HTTP Status Code 504 Gateway Timeout\n" }, { "name" : "http:http-version-not-supported", "type" : "xs:QName", "description" : " QName representing the HTTP Status Code 505 HTTP Version Not Supported\n" }, { "name" : "http:variant-also-negotiates", "type" : "xs:QName", "description" : " QName representing the HTTP Status Code 506 Variant Also Negotiates (RFC 2295)\n" }, { "name" : "http:insufficient-storage", "type" : "xs:QName", "description" : " QName representing the HTTP Status Code 507 Insufficient Storage (WebDAV; RFC 4918)\n" }, { "name" : "http:loop-detected", "type" : "xs:QName", "description" : " QName representing the HTTP status code 508 Loop Detected (webdav; rfc 5842)\n" }, { "name" : "http:not-extended", "type" : "xs:QName", "description" : " QName representing the HTTP status code 510 Not Extended (RFC 2774)\n" }, { "name" : "http:network-authentication-required", "type" : "xs:QName", "description" : " QName representing the HTTP status code 511 Network Authentication Required (RFC 6585)\n" }, { "name" : "resp:status-info", "type" : "item()*", "description" : " Variable containing a description for all the HTTP Status Codes\n" }, { "name" : "http:see-other", "type" : "xs:QName", "description" : " QName representing the HTTP Status Code 303 See Other\n" }, { "name" : "http:switching-protocols", "type" : "xs:QName", "description" : " QName representing the HTTP Status Code 101 Switching Protocols\n" }, { "name" : "http:processing", "type" : "xs:QName", "description" : " QName representing the HTTP Status Code 102 Processing (Web; RFC 2518)\n" }, { "name" : "http:ok", "type" : "xs:QName", "description" : " QName representing the HTTP Status Code 200 OK\n" }, { "name" : "http:created", "type" : "xs:QName", "description" : " QName representing the HTTP Status Code 201 Created\n" }, { "name" : "http:accepted", "type" : "xs:QName", "description" : " QName representing the HTTP Status Code 202 Accepted\n" }, { "name" : "http:non-authoritative", "type" : "xs:QName", "description" : " QName representing the HTTP Status Code 203 Non-Authoritative\n" }, { "name" : "http:no-content", "type" : "xs:QName", "description" : " QName representing the HTTP Status Code 204 No Content\n" }, { "name" : "http:reset-content", "type" : "xs:QName", "description" : " QName representing the HTTP Status Code 205 Reset Content\n" }, { "name" : "http:partial-content", "type" : "xs:QName", "description" : " QName representing the HTTP Status Code 206 Partial Content\n" }, { "name" : "http:multi-status", "type" : "xs:QName", "description" : " QName representing the HTTP Status Code 207 Multi-Status (WebDAV; RFC 5842)\n" }, { "name" : "http:already-reported", "type" : "xs:QName", "description" : " QName representing the HTTP Status Code 208 Already Reported (WebDAV; RFC 5842)\n" }, { "name" : "http:im-used", "type" : "xs:QName", "description" : " QName representing the HTTP Status Code 226 IM Used (RFC 3229)\n" }, { "name" : "http:multiple-choices", "type" : "xs:QName", "description" : " QName representing the HTTP Status Code 300 Multiple Choices\n" }, { "name" : "http:moved-permanently", "type" : "xs:QName", "description" : " QName representing the HTTP Status Code 301 Moved Permantently\n" }, { "name" : "http:found", "type" : "xs:QName", "description" : " QName representing the HTTP Status Code 302 Found\n" }, { "name" : "http:continue", "type" : "xs:QName", "description" : " QName representing the HTTP Status Code 100 Continue\n" }, { "name" : "http:not-modified", "type" : "xs:QName", "description" : " QName representing the HTTP Status Code 304 Not Modified\n" }, { "name" : "http:use-proxy", "type" : "xs:QName", "description" : " QName representing the HTTP Status Code 305 Use Proxy\n" }, { "name" : "http:switch-proxy", "type" : "xs:QName", "description" : " QName representing the HTTP Status Code 306 Switch Proxy\n" }, { "name" : "http:temporary-redirect", "type" : "xs:QName", "description" : " QName representing the HTTP Status Code 307 Temporary Redirect\n" }, { "name" : "http:permanent-redirect", "type" : "xs:QName", "description" : " QName representing the HTTP Status Code 308 Permanent Redirect (http://tools.ietf.org/html/draft-reschke-http-status-308-07)\n" }, { "name" : "http:bad-request", "type" : "xs:QName", "description" : " QName representing the HTTP Status Code 400 Bad Request\n" }, { "name" : "http:unauthorized", "type" : "xs:QName", "description" : " QName representing the HTTP Status Code 401 Unauthorized\n" }, { "name" : "http:payment-required", "type" : "xs:QName", "description" : " QName representing the HTTP Status Code 402 Payment Required\n" }, { "name" : "http:forbidden", "type" : "xs:QName", "description" : " QName representing the HTTP Status Code 403 Forbidden\n" }, { "name" : "http:not-found", "type" : "xs:QName", "description" : " QName representing the HTTP Status Code 404 Not Found\n" }, { "name" : "http:not-allowed", "type" : "xs:QName", "description" : " QName representing the HTTP Status Code 405 Not Allowed\n" }, { "name" : "http:not-acceptable", "type" : "xs:QName", "description" : " QName representing the HTTP Status Code 406 Not Acceptable\n" }, { "name" : "http:proxy-authentication-required", "type" : "xs:QName", "description" : " QName representing the HTTP Status Code 407 Proxy Authentication Required\n" }, { "name" : "http:request-timeout", "type" : "xs:QName", "description" : " QName representing the HTTP Status Code 408 Request Timeout\n" } ] }, "http://zorba.io/modules/uri" : { "ns" : "http://zorba.io/modules/uri", "description" : " This module provides functions for processing URIs and URLs.\n", "sees" : [  ], "authors" : [ "<xqdoc:author xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Matthias Brantner, Luis Rodriguez Gonzalez</xqdoc:author>" ], "version" : null, "encoding" : "utf-8", "namespaces" : [ { "uri" : "http://zorba.io/modules/uri", "prefix" : "uri" }, { "uri" : "http://zorba.io/options/versioning", "prefix" : "ver" }, { "uri" : "http://zorba.io/errors", "prefix" : "zerr" } ], "functions" : [ { "isDocumented" : true, "arity" : 1, "name" : "decode", "qname" : "uri:decode", "signature" : "($u as xs:string) as xs:string", "description" : " Percent-decodes (aka URL decoding) the given string.\n All percent encoded octets will be translated into their\n decoded UTF-8 representation.\n Please note that the percent encoding guarantees that a string\n consists of ASCII characters only. Passing a string that contains\n non-ASCII characters results in undefined behavior.\n", "summary" : "<p> Percent-decodes (aka URL decoding) the given string.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "u", "type" : "xs:string", "occurrence" : null, "description" : "" } ], "returns" : { "type" : "xs:string", "description" : "the percent decoded string" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 2, "name" : "decode", "qname" : "uri:decode", "signature" : "($u as xs:string, $decode-plus as xs:boolean) as xs:string", "description" : " Percent-decodes (aka URL decoding) the given string.\n All percent encoded octets will be translated into their\n decoded UTF-8 representation.\n If $decode-plus is specified all occurrences of the char '+'\n will be replaced with a space ' ' before the percent decoding\n happens.\n Please note that the percent encoding guarantees that a string\n consists of ASCII characters only. Passing a string that contains\n non-ASCII characters results in undefined behavior.\n", "summary" : "<p> Percent-decodes (aka URL decoding) the given string.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "u", "type" : "xs:string", "occurrence" : null, "description" : "" }, { "name" : "decode-plus", "type" : "xs:boolean", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> whether '+' chars will be replaced with spaces</div>" } ], "returns" : { "type" : "xs:string", "description" : "the percent decoded string" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 3, "name" : "decode", "qname" : "uri:decode", "signature" : "($s as xs:string, $decode-plus as xs:boolean, $charset as xs:string) as xs:string external", "description" : " Percent-decodes (aka URL decoding) the given string.\n All percent encoded octets will be translated into their\n decoded UTF-8 representation.\n If $decode-plus is specified all occurrences of the char '+'\n will be replaced with a space ' ' before the percent decoding\n happens.\n The $charset parameter specifies the source charset after precent\n decoding. It is used to convert the decoded string into UTF-8.\n Please note that the percent encoding guarantees that a string\n consists of ASCII characters only. Passing a string that contains\n non-ASCII characters results in undefined behavior.\n", "summary" : "<p> Percent-decodes (aka URL decoding) the given string.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "s", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the string to decode</div>" }, { "name" : "decode-plus", "type" : "xs:boolean", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> whether '+' chars will be replaced with spaces</div>" }, { "name" : "charset", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the source charset of the string after percent decoding</div>" } ], "returns" : { "type" : "xs:string", "description" : "the percent decoded string" }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">uri:CHARSET_UNKNOWN if the given charset is unknown or not supported</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">zerr:ZOSE0006 if there is an error transcoding the string</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 1, "name" : "parse", "qname" : "uri:parse", "signature" : "($uri as xs:string) as object() external", "description" : " Parses the URI passed as string. The returned object\n contains only members with field names declared as constants in\n this module.\n For example,\n <pre xmlns:xqdoc=\"http://www.xqdoc.org/1.0\" class=\"ace-static\" ace-mode=\"xquery\">\n let my-uri := \"http://www.my.project.com/a/b/c?user=john;pass=1234#profile\"\n return uri:parse(my-uri)\n </pre>\n returns\n <pre xmlns:xqdoc=\"http://www.xqdoc.org/1.0\" class=\"ace-static\" ace-mode=\"java\">\n { \"squeme\" : \"http\", \"host\" : \"www.my.project.com\", \"path\" : \"/a/b/c\",\n   \"query\" : \"user=john;pass=123\", \"fragment\" : \"profile\" }\n </pre>\n", "summary" : "<p> Parses the URI passed as string.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "uri", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the URI to parse</div>" } ], "returns" : { "type" : "object()", "description" : "the JSON object" }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">err:XQST0046 if the URI is textually invalid</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 1, "name" : "serialize", "qname" : "uri:serialize", "signature" : "($uri as object()) as xs:string external", "description" : " Serialize the URI passed as object into a string.\n", "summary" : "<p> Serialize the URI passed as object into a string.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "uri", "type" : "object()", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the object representing the URI</div>" } ], "returns" : { "type" : "xs:string", "description" : "the URI as string" }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">uri:OPAQUE_COMB_NOT_VALID if opaque part is specified in conjunction with host/port/path/user-info/query.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">uri:OPAQUE_WITHOUT_SCHEME if opaque part is present but no scheme is present.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">uri:INVALID_ABSOLUTE_PATH if a path component for an absolute URI doesn't start with \"/\".</xqdoc:error>" ] } ], "variables" : [ { "name" : "uri:SCHEME", "type" : "xs:string", "description" : " Constant for the \"scheme\" part of a URI object.\n" }, { "name" : "uri:AUTHORITY", "type" : "xs:string", "description" : " Constant for the \"authority\" part of a URI object.\n" }, { "name" : "uri:USER-INFO", "type" : "xs:string", "description" : " Constant for the \"user-info\" part of a URI object.\n" }, { "name" : "uri:HOST", "type" : "xs:string", "description" : " Constant for the \"host\" part of a URI object.\n" }, { "name" : "uri:PORT", "type" : "xs:string", "description" : " Constant for the \"port\" part of a URI object.\n" }, { "name" : "uri:PATH", "type" : "xs:string", "description" : " Constant for the \"path\" part of a URI object.\n" }, { "name" : "uri:QUERY", "type" : "xs:string", "description" : " Constant for the \"query\" part of a URI object.\n" }, { "name" : "uri:FRAGMENT", "type" : "xs:string", "description" : " Constant for the \"fragment\" part of a URI object.\n" }, { "name" : "uri:OPAQUE-PART", "type" : "xs:string", "description" : " Constant for the \"opaque-part\" part of a URI object.\n If this is set in a URI object, then none of $uri:PATH, $uri:HOST,\n $uri:PORT, $uri:USER-INFO, or : $uri:QUERY may be specified.\n If this is set in a URI object, $uri:SCHEME must also be specified\n (ie, it must be an absolute URI).\n" } ] }, "http://xbrl.io/modules/bizql/profiles/sec/fiscal/core" : { "ns" : "http://xbrl.io/modules/bizql/profiles/sec/fiscal/core", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">This module provides functionality for querying fiscal reports (10-K, 10-Q)\n submitted to the SEC.</p>\n", "sees" : [  ], "authors" : [ "<xqdoc:author xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Charles Hoffman</xqdoc:author>", "<xqdoc:author xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Matthias Brantner</xqdoc:author>", "<xqdoc:author xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Dennis Knochenwefel</xqdoc:author>", "<xqdoc:author xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Ghislain Fourny</xqdoc:author>" ], "version" : null, "encoding" : "utf-8", "namespaces" : [ { "uri" : "http://xbrl.io/modules/bizql/archives", "prefix" : "archives" }, { "uri" : "http://xbrl.io/modules/bizql/profiles/sec/companies", "prefix" : "companies" }, { "uri" : "http://xbrl.io/modules/bizql/components", "prefix" : "components" }, { "uri" : "http://www.28msec.com/modules/credentials", "prefix" : "credentials" }, { "uri" : "http://xbrl.io/modules/bizql/facts", "prefix" : "facts" }, { "uri" : "http://xbrl.io/modules/bizql/profiles/sec/filings", "prefix" : "filings" }, { "uri" : "http://xbrl.io/modules/bizql/profiles/sec/fiscal/core", "prefix" : "fiscal-core" }, { "uri" : "http://jsoniq.org/function-library", "prefix" : "j" }, { "uri" : "http://www.28msec.com/modules/mongodb", "prefix" : "mongo" }, { "uri" : "http://xbrl.io/modules/bizql/profiles/sec/core", "prefix" : "sec" }, { "uri" : "http://xbrl.io/modules/bizql/profiles/sec/networks", "prefix" : "sec-networks" }, { "uri" : "http://zorba.io/options/versioning", "prefix" : "ver" } ], "functions" : [ { "isDocumented" : true, "arity" : 3, "name" : "balance-sheets-for-fiscal-periods-and-years", "qname" : "fiscal-core:balance-sheets-for-fiscal-periods-and-years", "signature" : "($entity-or-ids as item()*, $fiscal-period-focus as string*, $fiscal-year-focus as integer*) as object()*", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Retrieves the balance sheet for the given entities and fiscal periods and years.</p>\n", "summary" : "<p>  Retrieves the balance sheet for the given entities and fiscal periods and years.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "entity-or-ids", "type" : "item()", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> a list of entities or their EIDs.</div>" }, { "name" : "fiscal-period-focus", "type" : "string", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the periods (YTD1, YTD2, YTD3, Q1, Q2, Q3 or FY) or $fiscal-core:ALL_FISCAL_PERIODS to accept all.</div>" }, { "name" : "fiscal-year-focus", "type" : "integer", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the years (2012) or $fiscal-core:ALL_FISCAL_YEARS to accept all.</div>" } ], "returns" : { "type" : "object()*", "description" : "the balance sheet at that date and for that formType." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">entities:INVALID_PARAMETER if the CIK or entity is not valid.</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 3, "name" : "facts-for-aspects-and-fiscal-periods-and-years", "qname" : "fiscal-core:facts-for-aspects-and-fiscal-periods-and-years", "signature" : "($aspects as object()?, $fiscal-period-focus as string*, $fiscal-year-focus as integer*) as object()*", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Retrieves facts for the given aspects, fiscal periods and years.</p>\n", "summary" : "<p>  Retrieves facts for the given aspects, fiscal periods and years.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "aspects", "type" : "object()", "occurrence" : "?", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> an object containing aspects to filter, among which xbrl:Concept, xbrl:Entity and xbrl:Period (all optional).</div>" }, { "name" : "fiscal-period-focus", "type" : "string", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the periods (YTD1, YTD2, YTD3, Q1, Q2, Q3 or FY).</div>" }, { "name" : "fiscal-year-focus", "type" : "integer", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the years (2012)</div>" } ], "returns" : { "type" : "object()*", "description" : "all facts matching the given aspects, fiscal period and year." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 4, "name" : "facts-for-aspects-and-fiscal-periods-and-years", "qname" : "fiscal-core:facts-for-aspects-and-fiscal-periods-and-years", "signature" : "($aspects as object()?, $fiscal-period-focus as string*, $fiscal-year-focus as integer*, $options as object()?) as object()*", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Retrieves facts for the given aspects, fiscal periods and years.</p>\n", "summary" : "<p>  Retrieves facts for the given aspects, fiscal periods and years.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "aspects", "type" : "object()", "occurrence" : "?", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> an object containing aspects to filter, among which xbrl:Concept, xbrl:Entity and xbrl:Period (all optional).</div>" }, { "name" : "fiscal-period-focus", "type" : "string", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the periods (YTD1, YTD2, YTD3, Q1, Q2, Q3 or FY) or $fiscal-core:ALL_FISCAL_PERIODS to accept all.</div>" }, { "name" : "fiscal-year-focus", "type" : "integer", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the years (2012) or $fiscal-core:ALL_FISCAL_YEARS to accept all.</div>" }, { "name" : "options", "type" : "object()", "occurrence" : "?", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> <a href=\"../core#standard_options\">standard SEC BizQL options</a>.</div>" } ], "returns" : { "type" : "object()*", "description" : "all facts matching the given aspects, fiscal period and year." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 4, "name" : "facts-for-entities-and-concepts-and-fiscal-periods-and-years", "qname" : "fiscal-core:facts-for-entities-and-concepts-and-fiscal-periods-and-years", "signature" : "($entities-or-ids as item()*, $concepts as string*, $fiscal-period-focus as string*, $fiscal-year-focus as integer*) as object()*", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Retrieves facts for the given entities, concepts, fiscal periods and years.</p>\n", "summary" : "<p>  Retrieves facts for the given entities, concepts, fiscal periods and years.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "entities-or-ids", "type" : "item()", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> a list of entities or their EIDs.</div>" }, { "name" : "concepts", "type" : "string", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> a list of concept names (us-gaap:Assets)</div>" }, { "name" : "fiscal-period-focus", "type" : "string", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the periods (YTD1, YTD2, YTD3, Q1, Q2, Q3 or FY) or $fiscal-core:ALL_FISCAL_PERIODS to accept all.</div>" }, { "name" : "fiscal-year-focus", "type" : "integer", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the years (2012) or $fiscal-core:ALL_FISCAL_YEARS to accept all.</div>" } ], "returns" : { "type" : "object()*", "description" : "all facts matching entities, concepts, fiscal periods and years" }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">entities:INVALID_PARAMETER if the CIK or entity is not valid.</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 5, "name" : "facts-for-entities-and-concepts-and-fiscal-periods-and-years", "qname" : "fiscal-core:facts-for-entities-and-concepts-and-fiscal-periods-and-years", "signature" : "($entities-or-ids as item()*, $concepts as string*, $fiscal-period-focus as string*, $fiscal-year-focus as integer*, $options as object()?) as object()*", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Retrieves facts for the given entities, concepts, fiscal periods and years.</p>\n", "summary" : "<p>  Retrieves facts for the given entities, concepts, fiscal periods and years.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "entities-or-ids", "type" : "item()", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> a list of entities or their EIDs.</div>" }, { "name" : "concepts", "type" : "string", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> a list of concept names (us-gaap:Assets)</div>" }, { "name" : "fiscal-period-focus", "type" : "string", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the periods (YTD1, YTD2, YTD3, Q1, Q2, Q3 or FY) or $fiscal-core:ALL_FISCAL_PERIODS to accept all.</div>" }, { "name" : "fiscal-year-focus", "type" : "integer", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the years (2012) or $fiscal-core:ALL_FISCAL_YEARS to accept all.</div>" }, { "name" : "options", "type" : "object()", "occurrence" : "?", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> <a href=\"../core#standard_options\">standard SEC BizQL options</a>.</div>" } ], "returns" : { "type" : "object()*", "description" : "all facts matching entities, concepts, fiscal periods and years" }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">entities:INVALID_PARAMETER if the CIK or entity is not valid.</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 3, "name" : "filings-for-entities-and-fiscal-periods-and-years", "qname" : "fiscal-core:filings-for-entities-and-fiscal-periods-and-years", "signature" : "($entities-or-ids as item()*, $fiscal-period-focus as string*, $fiscal-year-focus as integer*) as object()*", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Retrieves all filings by entities and for given fiscal periods and years.</p>\n", "summary" : "<p>  Retrieves all filings by entities and for given fiscal periods and years.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "entities-or-ids", "type" : "item()", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> entities or their ids.</div>" }, { "name" : "fiscal-period-focus", "type" : "string", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> fiscal periods (FY, YTD1, YTD2, YTD3, Q1, Q2, Q3, Q4) or $fiscal-core:ALL_FISCAL_PERIODS to accept all..</div>" }, { "name" : "fiscal-year-focus", "type" : "integer", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> fiscal years or $fiscal-core:ALL_FISCAL_YEARS to accept all..</div>" } ], "returns" : { "type" : "object()*", "description" : "all filings that match the criteria." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "fiscal-period", "qname" : "fiscal-core:fiscal-period", "signature" : "($filing-fact-or-id as item()?) as string?", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Retrieves the fiscal period of a filing or a fact.</p>\n", "summary" : "<p>  Retrieves the fiscal period of a filing or a fact.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "filing-fact-or-id", "type" : "item()", "occurrence" : "?", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> a filing, a fact, or its id.</div>" } ], "returns" : { "type" : "string?", "description" : "the fiscal period (FY, YTD2, YTD3, Q1, Q2 or Q3)." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "fiscal-year", "qname" : "fiscal-core:fiscal-year", "signature" : "($filing-fact-or-id as item()?) as integer?", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Retrieves the fiscal year of a filing or a fact.</p>\n", "summary" : "<p>  Retrieves the fiscal year of a filing or a fact.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "filing-fact-or-id", "type" : "item()", "occurrence" : "?", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> a filing, a fact, or its id.</div>" } ], "returns" : { "type" : "integer?", "description" : "the fiscal year (e.g., 2012)." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "latest-reported-fiscal-period", "qname" : "fiscal-core:latest-reported-fiscal-period", "signature" : "($entity-or-id as item()) as object()*", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Retrieves the latest reported fiscal period and year for the\n supplied entity.</p>\n", "summary" : "<p>  Retrieves the latest reported fiscal period and year for the\n supplied entity.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "entity-or-id", "type" : "item()", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> an entity or its EID.</div>" } ], "returns" : { "type" : "object()*", "description" : "an object with two fields: period and year." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 2, "name" : "latest-reported-fiscal-period", "qname" : "fiscal-core:latest-reported-fiscal-period", "signature" : "($entity-or-id as item(), $type as string) as object()*", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Retrieves the latest fiscal period and year for the\n supplied entity.</p>\n", "summary" : "<p>  Retrieves the latest fiscal period and year for the\n supplied entity.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "entity-or-id", "type" : "item()", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> an entity or its EID.</div>" }, { "name" : "type", "type" : "string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> 10-K or 10-Q or FY, YTD1, YTD2, YTD3, Q1, Q2, or Q3.</div>" } ], "returns" : { "type" : "object()*", "description" : "an object with two fields: period and year." }, "errors" : [  ] } ], "variables" : [ { "name" : "fiscal-core:ALL_FISCAL_PERIODS", "type" : "string", "description" : " Joker for all fiscal periods.\n" }, { "name" : "fiscal-core:ALL_FISCAL_YEARS", "type" : "integer", "description" : " Joker for all fiscal years.\n" } ] }, "http://zorba.io/modules/base64" : { "ns" : "http://zorba.io/modules/base64", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Base64 encoding and decoding.</p>\n", "sees" : [  ], "authors" : [ "<xqdoc:author xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Matthias Brantner</xqdoc:author>" ], "version" : null, "encoding" : "utf-8", "namespaces" : [ { "uri" : "http://zorba.io/modules/base64", "prefix" : "base64" }, { "uri" : "http://zorba.io/options/versioning", "prefix" : "ver" }, { "uri" : "http://zorba.io/errors", "prefix" : "zerr" } ], "functions" : [ { "isDocumented" : true, "arity" : 1, "name" : "decode", "qname" : "base64:decode", "signature" : "($base64 as base64Binary) as string external", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Decode a base64Binary.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">The function assumes that the content after decoding is valid\n UTF-8.</p>\n", "summary" : "<p>  Decode a base64Binary.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "base64", "type" : "base64Binary", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The base64Binary item to decode</div>" } ], "returns" : { "type" : "string", "description" : "the base64-decoded value as string" }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">zerr:ZOSE0006 if $base64 contains invalid base-64 data.</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 2, "name" : "decode", "qname" : "base64:decode", "signature" : "($base64 as base64Binary, $encoding as string) as string external", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Decode a base64Binary.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">The function assumes that the content after decoding has\n the given encoding.</p>\n", "summary" : "<p>  Decode a base64Binary.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "base64", "type" : "base64Binary", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The base64Binary item to decode</div>" }, { "name" : "encoding", "type" : "string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The encoding of the string after base64-decoding it. The encoding parameter is case insensitive.</div>" } ], "returns" : { "type" : "string", "description" : "the base64-decoded value as a string" }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">zerr:ZXQP0006 if the given encoding is invalid or not supported. \"ASCII\" and \"UTF-8\" are guaranteed to be supported; other encodings may be supported depending on the installation.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">zerr:ZOSE0006 if $base64 contains invalid base-64 data.</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 1, "name" : "encode", "qname" : "base64:encode", "signature" : "($string as string) as base64Binary external", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Encode a string as base64Binary.</p>\n", "summary" : "<p>  Encode a string as base64Binary.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "string", "type" : "string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The item whose string-value should be encoded</div>" } ], "returns" : { "type" : "base64Binary", "description" : "the base64-encoded string-value of the item parameter" }, "errors" : [  ] } ], "variables" : [  ] }, "http://xbrl.io/modules/bizql/networks" : { "ns" : "http://xbrl.io/modules/bizql/networks", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">This module provides functionality for handling networks.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">A network is a graph of concepts and resources, of which the edges\n share the same semantics.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">There are several kinds of networks.\n A network can be a tree or a DAG of concepts. It can be a bipartite\n graph mapping concepts to resources.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">XBRL defines a number of standard networks: presentation, calculation,\n definition (essence-alias, general-special, requires-element, similar-tuples, domain-member),\n and label networks.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">With this module, you can retrieve the networks contained in a component, and you\n can retrieve a standard XBRL network with the provided short names.</p>\n", "sees" : [  ], "authors" : [ "<xqdoc:author xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Charles Hoffman</xqdoc:author>", "<xqdoc:author xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Matthias Brantner</xqdoc:author>", "<xqdoc:author xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Dennis Knochenwefel</xqdoc:author>", "<xqdoc:author xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Ghislain Fourny</xqdoc:author>" ], "version" : null, "encoding" : "utf-8", "namespaces" : [ { "uri" : "http://xbrl.io/modules/bizql/components", "prefix" : "components" }, { "uri" : "http://xbrl.io/modules/bizql/networks", "prefix" : "networks" } ], "functions" : [ { "isDocumented" : true, "arity" : 2, "name" : "networks-for-components-and-short-names", "qname" : "networks:networks-for-components-and-short-names", "signature" : "($components as item()*, $short-name as string*) as object()*", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Retrieves the standardized networks in the supplied components with the given short names.</p>\n", "summary" : "<p>  Retrieves the standardized networks in the supplied components with the given short names.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "components", "type" : "item()", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> a sequence of components.</div>" }, { "name" : "short-name", "type" : "string", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the short names of the networks.</div>" } ], "returns" : { "type" : "object()*", "description" : "the standardized networks with these short names." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "networks-for-components", "qname" : "networks:networks-for-components", "signature" : "($components as item()*) as object()*", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Retrieves all networks in the supplied components.</p>\n", "summary" : "<p>  Retrieves all networks in the supplied components.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "components", "type" : "item()", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> a sequence of components.</div>" } ], "returns" : { "type" : "object()*", "description" : "all networks." }, "errors" : [  ] } ], "variables" : [ { "name" : "networks:PRESENTATION_NETWORK", "type" : "xs:string", "description" : " Short name of the presentation network.\n" }, { "name" : "networks:CALCULATION_NETWORK", "type" : "xs:string", "description" : " Short name of the calculation network.\n" }, { "name" : "networks:ESSENCE_ALIAS_NETWORK", "type" : "xs:string", "description" : " Short name of the essence-alias definition network.\n" }, { "name" : "networks:GENERAL_SPECIAL_NETWORK", "type" : "xs:string", "description" : " Short name of the general-special definition network.\n" }, { "name" : "networks:REQUIRES_ELEMENT_NETWORK", "type" : "xs:string", "description" : " Short name of the requires-element definition network.\n" }, { "name" : "networks:SIMILAR_TUPLES_NETWORK", "type" : "xs:string", "description" : " Short name of the similar-tuples definition network.\n" }, { "name" : "networks:DOMAIN_MEMBER_NETWORK", "type" : "xs:string", "description" : " Short name of the domain-member definition network.\n" } ] }, "http://jsound.io/modules/validate/map" : { "ns" : "http://jsound.io/modules/validate/map", "description" : " Map utility\n", "sees" : [  ], "authors" : [ "<xqdoc:author xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Chris Hillery, Cezar Andrei</xqdoc:author>" ], "version" : null, "encoding" : "utf-8", "namespaces" : [ { "uri" : "http://zorba.io/annotations", "prefix" : "an" }, { "uri" : "http://jsound.io/modules/validate/map", "prefix" : "map" }, { "uri" : "http://zorba.io/options/versioning", "prefix" : "ver" } ], "functions" : [ { "isDocumented" : true, "arity" : 2, "name" : "get", "qname" : "map:get", "signature" : "($map as object(), $key as string) as item()", "description" : " Returns the value under the key.\n", "summary" : "<p> Returns the value under the key.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "map", "type" : "object()", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the map JSON object</div>" }, { "name" : "key", "type" : "string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the key</div>" } ], "returns" : { "type" : "item()", "description" : "the value under the $key, () if empty Example:" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 2, "name" : "has-key", "qname" : "map:has-key", "signature" : "($map as object(), $key as string) as boolean", "description" : " Returns true if $key exists in the map.\n", "summary" : "<p> Returns true if $key exists in the map.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "map", "type" : "object()", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the map JSON object</div>" }, { "name" : "key", "type" : "string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the key</div>" } ], "returns" : { "type" : "boolean", "description" : "true if $key exists, false otherwise Example:" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 3, "name" : "set-if-empty", "qname" : "map:set-if-empty", "signature" : "($map as object(), $key as xs:string, $value as item()) as boolean", "description" : " Only if key doesn't exist, inserts new key and value into the map and\n returns true. Otherwise returns false.\n", "summary" : "<p> Only if key doesn't exist, inserts new key and value into the map and\n returns true.</p>", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "map", "type" : "object()", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the map JSON object</div>" }, { "name" : "key", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the key</div>" }, { "name" : "value", "type" : "item()", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the value</div>" } ], "returns" : { "type" : "boolean", "description" : "true if $key is empty, false otherwise Example:" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 3, "name" : "set", "qname" : "map:set", "signature" : "($map as object(), $key as xs:string, $value as item()) as boolean", "description" : " Inserts new key and value into the map or replaces value under the exiting key.\n Returns true all the time.\n", "summary" : "<p> Inserts new key and value into the map or replaces value under the exiting key.</p>", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "map", "type" : "object()", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the map JSON object</div>" }, { "name" : "key", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the key</div>" }, { "name" : "value", "type" : "item()", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the value</div>" } ], "returns" : { "type" : "boolean", "description" : "true Example:" }, "errors" : [  ] } ], "variables" : [  ] }, "http://zorba.io/modules/xml" : { "ns" : "http://zorba.io/modules/xml", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">\n This module provides functions for reading XML files from string inputs.\n It allows reading of well-formed XML documents as well as well-formed\n external parsed entities, described by\n <a href=\"http://www.w3.org/TR/xml/#wf-entities\">XML 1.0 Well-Formed\n Parsed Entities</a>. The functions can also perform Schema and DTD\n validation of the input documents.\n </p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">The following example parses a sequence of XML elements and returns\n them in a streaming fashion - each at a time:</p>\n <pre xmlns:xqdoc=\"http://www.xqdoc.org/1.0\" class=\"ace-static\" ace-mode=\"xquery\">\n import module namespace x = \"http://zorba.io/modules/xml\";\n import schema namespace opt = \"http://zorba.io/modules/xml-options\";\n x:parse(\n   \"&lt;from1&gt;Jani&lt;/from1&gt;&lt;from2&gt;Jani&lt;/from2&gt;&lt;from3&gt;Jani&lt;/from3&gt;\",\n   &lt;opt:options&gt;\n     &lt;opt:parse-external-parsed-entity/&gt;\n   &lt;/opt:options&gt;\n )\n </pre>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Another useful option allows to skip an arbitrary number of levels\n before returning a sequence of nodes as shown in the following example:</p>\n <pre xmlns:xqdoc=\"http://www.xqdoc.org/1.0\" class=\"ace-static\" ace-mode=\"xquery\">\n import module namespace x = \"http://zorba.io/modules/xml\";\n import schema namespace opt = \"http://zorba.io/modules/xml-options\";\n x:parse(\n   \"&lt;root&gt;\n     &lt;from1&gt;Jani1&lt;/from1&gt;\n     &lt;from2&gt;Jani2&lt;/from2&gt;\n     &lt;from3&gt;Jani3&lt;/from3&gt;\n   &lt;/root&gt;\",\n   &lt;opt:options&gt;\n     &lt;opt:parse-external-parsed-entity opt:skip-root-nodes=\"1\"/&gt;\n   &lt;/opt:options&gt;\n )\n </pre>\n", "sees" : [ "<xqdoc:see xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"><a href=\"http://www.w3.org/TR/xml/#wf-entities\">XML 1.0 Well-Formed Parsed Entities</a></xqdoc:see>", "<xqdoc:see xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"><a href=\"http://www.w3.org/TR/xpath-functions-30/#func-parse-xml\"> fn:parse-xml() function in XPath and XQuery Functions and Operators 3.0</a></xqdoc:see>", "<xqdoc:see xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"><a href=\"http://xmlsoft.org/html/libxml-parser.html\">LibXml2 parser</a></xqdoc:see>" ], "authors" : [ "<xqdoc:author xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Nicolae Brinza, Juan Zacarias</xqdoc:author>" ], "version" : null, "encoding" : "utf-8", "namespaces" : [ { "uri" : "http://www.w3.org/xqt-errors", "prefix" : "err" }, { "uri" : "http://zorba.io/modules/xml-options", "prefix" : "opt" }, { "uri" : "http://zorba.io/modules/schema", "prefix" : "schema" }, { "uri" : "http://zorba.io/options/versioning", "prefix" : "ver" }, { "uri" : "http://zorba.io/modules/xml", "prefix" : "x" }, { "uri" : "http://zorba.io/errors", "prefix" : "zerr" } ], "functions" : [ { "isDocumented" : true, "arity" : 1, "name" : "canonicalize", "qname" : "x:canonicalize", "signature" : "($xml-string as xs:string) as xs:string", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">A function to canonicalize the given XML string, that is, transform\n it into Canonical XML as defined by <a href=\"http://www.w3.org/TR/xml-c14n\">Canonical XML</a>.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Note: This function is not streamable. If a streamable string is used\n as input for the function it will be materialized.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Note: This function sets the\n <a href=\"http://xmlsoft.org/html/libxml-parser.html#xmlParserOption\">XML_PARSE_NOERROR</a>\n option when parsing the XML input.</p>\n", "summary" : "<p>  A function to canonicalize the given XML string, that is, transform\n it into Canonical XML as defined by  Canonical XML .</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "xml-string", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> a string representation of a well formed XML to canonicalize. XML fragments are not allowed.</div>" } ], "returns" : { "type" : "xs:string", "description" : "the canonicalized XML string." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">err:CANO0001 invalid input.</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 2, "name" : "canonicalize", "qname" : "x:canonicalize", "signature" : "($xml-string as xs:string, $options as element(opt:options)) as xs:string", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">A function to canonicalize the given XML string, that is, transform\n it into Canonical XML as defined by <a href=\"http://www.w3.org/TR/xml-c14n\">Canonical XML</a>.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">This version of the function allows specifying certain options to be\n used when initially parsing the XML string. These are of the same form\n as the options to x:parse#2(), although the following options are\n currently ignored for this function:\n <ul>\n <li>&lt;opt:no-error/&gt;</li>\n <li>&lt;opt:base-uri/&gt;</li>\n <li>&lt;opt:schema-validate/&gt;</li>\n <li>&lt;opt:parse-external-parsed-entity/&gt;</li>\n </ul>\n </p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Note: This function is not streamable, if a streamable string is used\n as input for the function it will be materialized.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Note: This function sets the\n <a href=\"http://xmlsoft.org/html/libxml-parser.html#xmlParserOption\">XML_PARSE_NOERROR</a>\n option when parsing the XML input.</p>\n", "summary" : "<p>  A function to canonicalize the given XML string, that is, transform\n it into Canonical XML as defined by  Canonical XML .</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "xml-string", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> a string representation of a well formed XML to canonicalize. XML fragments are not allowed.</div>" }, { "name" : "options", "type" : "element(opt:options)", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> an XML containg options for the canonicalize function.</div>" } ], "returns" : { "type" : "xs:string", "description" : "the canonicalized XML string." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">err:CANO0001 invalid input.</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 2, "name" : "parse", "qname" : "x:parse", "signature" : "($xml-string as xs:string?, $options as element(opt:options)?) as node()* external", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">A function to parse XML files and fragments (i.e.\n <a href=\"http://www.w3.org/TR/xml/#wf-entities\">external general parsed\n entities</a>).</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">The functions takes two arguments: the first one is the\n string to be parsed and the second argument is an &lt;options/&gt; element that\n passes a list of options to the parsing function. They are described below.\n The options element must conform to the xml-options:options element type\n from the xml-options.xsd schema. Some of these\n will be passed to the underlying library (LibXml2) and further documentation\n for them can be found at <a href=\"http://xmlsoft.org/html/libxml-parser.html\">\n LibXml2 parser</a>.</p>\n The list of available options:\n <ul xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">\n <li>\n &lt;base-uri/&gt; - the element must have a \"value\" attribute, which will provide\n the baseURI that will be used as the baseURI for every node returned by this\n function.\n </li>\n <li>\n &lt;no-error/&gt; - if present, the option will disable fatal error processing. Any\n failure to parse or validate the input in the requested manner will result\n in the function returning an empty sequence and no error will raised.\n </li>\n <li>\n &lt;schema-validate/&gt; - if present, it will request that the input string be Schema\n validated. The element accepts an attribute named \"mode\" which can have two\n values: \"strict and \"lax\". Enabling the option will produce a result that is\n equivalent to processing the input with the option disabled, and then copying\n the result using the XQuery \"validate strict|lax\" expression. This option can not\n be used together with either the &lt;DTD-validate/&gt; or the &lt;parse-external-parsed-entity/&gt;\n option. Doing so will raise a zerr:ZXQD0003 error.\n </li>\n <li>\n &lt;DTD-validate/&gt; - the option will enable the DTD-based validation. If this\n option is enabled and the input references a DTD, then the input must be a\n well-formed and DTD-valid XML document. The &lt;DTD-load/&gt; option must be used for\n external DTD files to be loaded. If the option is enabled and the input does\n not reference a DTD then the option is ignored. If the option is disabled, the\n input is not required to reference a DTD and if it does reference a DTD then\n the DTD is ignored for validation purposes. This option can not\n be used together with either the &lt;schema-validate/&gt; or the &lt;parse-external-parsed-entity&gt;\n option. Doing so will raise a zerr:ZXQD0003 error.\n </li>\n <li>\n &lt;DTD-load/&gt; - if present, it will enable loading of external DTD files.\n </li>\n <li>\n &lt;default-DTD-attributes/&gt; - if present, it will enable the default DTD attributes.\n </li>\n <li>\n &lt;parse-external-parsed-entity/&gt; - if present, it will enable the processing of XML\n external entities. If the option\n is enabled, the input must conform to the syntax extParsedEnt (production\n [78] in XML 1.0, see <a href=\"http://www.w3.org/TR/xml/#wf-entities\">\n Well-Formed Parsed Entities</a>). In addition, by default a DOCTYPE declaration is allowed,\n as described by the [28] doctypedecl production, see <a href=\"http://www.w3.org/TR/xml/#NT-doctypedecl\">\n Document Type Definition</a>. A parameter is available to forbid the appearance of the DOCTYPE.\n The result of the function call is a list\n of nodes corresponding to the top-level components of the content of the\n external entity: that is, elements, processing instructions, comments, and\n text nodes. CDATA sections and character references are expanded, and\n adjacent characters are merged so the result contains no adjacent text\n nodes. If the option is disabled, the input must be a well-formed XML\n document conforming to the Document production\n (<a href=\"http://www.w3.org/TR/xml/#sec-well-formed\">production [1] in XML 1.0</a>).\n This option can not be used together with either the &lt;schema-validate/&gt; or the &lt;DTD-validate/&gt;\n option. Doing so will raise a zerr:ZXQD0003 error.\n The &lt;parse-external-parsed-entity/&gt; option has three parameters, given by attributes. The first\n attribute is \"skip-root-nodes\" and it can have a non-negative value. Specifying the paramter\n tells the parser to skip the given number of root nodes and return only their children. E.g.\n skip-root-nodes=\"1\" is equivalent to parse-xml($xml-string)/node()/node() . skip-root-nodes=\"2\" is equivalent\n to parse-xml($xml-string)/node()/node()/node() , etc. The second attribute is \"skip-top-level-text-nodes\" with a\n boolean value. Specifying \"true\" will tell the parser to skip top level text nodes, returning\n only the top level elements, comments, PIs, etc. This parameter works in combination with\n the \"skip-root-nodes\" paramter, thus top level text nodes are skipped after \"skip-root-nodes\" has\n been applied. The third paramter is \"error-on-doctype\" and will generate an error if a DOCTYPE\n declaration appears in the input, which by default is allowed.\n </li>\n <li>\n &lt;substitute-entities/&gt; - if present, it will enable the XML entities substitutions.\n </li>\n <li>\n &lt;remove-redundant-ns/&gt; - if present, the parser will remove redundant namespaces declarations.\n </li>\n <li>\n &lt;no-CDATA/&gt; - if present, the parser will merge CDATA nodes as text nodes.\n </li>\n <li>\n &lt;xinclude-substitutions/&gt; - if present, it will enable the XInclude substitutions.\n </li>\n <li>\n &lt;no-xinclude-nodes/&gt; - if present, the parser will not generate XInclude START/END nodes.\n </li>\n </ul>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">\n An example that sets the base-uri of the parsed external entities:\n </p>\n <pre xmlns:xqdoc=\"http://www.xqdoc.org/1.0\" class=\"ace-static\" ace-mode=\"xquery\">\n   import module namespace x = \"http://zorba.io/modules/xml\";\n   import schema namespace opt = \"http://zorba.io/modules/xml-options\";\n   x:parse(\"&lt;from1&gt;Jani&lt;/from1&gt;&lt;from2&gt;Jani&lt;/from2&gt;&lt;from3&gt;Jani&lt;/from3&gt;\",\n     &lt;opt:options&gt;\n       &lt;opt:base-uri opt:value=\"urn:test\"/&gt;\n       &lt;opt:parse-external-parsed-entity/&gt;\n     &lt;/opt:options&gt;\n   )\n </pre>\n", "summary" : "<p>  A function to parse XML files and fragments (i.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "xml-string", "type" : "xs:string", "occurrence" : "?", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The string that holds the XML to be parsed. If empty, the function will return an empty sequence</div>" }, { "name" : "options", "type" : "element(opt:options)", "occurrence" : "?", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The options for the parsing</div>" } ], "returns" : { "type" : "node()*", "description" : "The parsed XML as a document node or a list of nodes, or an empty sequence." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">zerr:ZXQD0003 The error will be raised if the options to the function are inconsistent.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">err:FODC0006 The error will be raised if the input string is not a valid XML document or fragment (external general parsed entity) or if DTD validation was enabled and the document has not passed it.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">err:XQDY0027 The error will be raised if schema validation was enabled and the input document has not passed it or if the parsing options are not conformant to the xml-options.xsd schema.</xqdoc:error>" ] } ], "variables" : [  ] }, "http://www.zorba-xquery.com/modules/process" : { "ns" : "http://www.zorba-xquery.com/modules/process", "description" : " This module provides functions to create a native process and return the result\n (i.e. exit code, result on standard out and error).\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">\n Example:\n<pre class=\"ace-static\" ace-mode=\"xquery\">\n  import module namespace proc = \"http://www.zorba-xquery.com/modules/process\";\n  proc:exec(\"ls\")\n</pre>\n </p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">\n Potential result:\n <pre class=\"ace-static\" ace-mode=\"xquery\">\n &lt;result xmlns=\"http://www.zorba-xquery.com/modules/process\"&gt;\n   &lt;stdout&gt;myfile.txt&lt;/stout&gt;\n   &lt;stderr/&gt;\n   &lt;exit-code&gt;0&lt;/exit-code&gt;\n &lt;/result&gt;\n </pre>\n </p>\n", "sees" : [  ], "authors" : [ "<xqdoc:author xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Cezar Andrei</xqdoc:author>" ], "version" : null, "encoding" : "utf-8", "namespaces" : [ { "uri" : "http://zorba.io/annotations", "prefix" : "an" }, { "uri" : "http://www.zorba-xquery.com/modules/process", "prefix" : "process" }, { "uri" : "http://zorba.io/options/versioning", "prefix" : "ver" } ], "functions" : [ { "isDocumented" : true, "arity" : 1, "name" : "exec", "qname" : "process:exec", "signature" : "($cmd as xs:string) as element(process:result) external", "description" : " Executes the specified string command in a separate process.\n This function does not allow arguments to be passed to\n the command.\n", "summary" : "<p> Executes the specified string command in a separate process.</p>", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "cmd", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> command to be executed (without arguments)</div>" } ], "returns" : { "type" : "element(process:result)", "description" : "the result of the execution as an element as shown in the documentation of this module. The exit-code element returns the exit code of the child process. For POSIX compliant platforms: returns the process exit code. If process is terminated or stopped: 128 + termination signal code. For Windows platforms: returns the return value of the process or the exit or terminate process specified value." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">process:PROC01 if an error occurred while communicating with the executed process.</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 2, "name" : "exec", "qname" : "process:exec", "signature" : "($cmd as xs:string, $args as xs:string*) as element(process:result) external", "description" : " Executes the specified string command in a separate process.\n Each of the strings in the sequence passed in as the second\n argument is passed as an argument to the executed command.\n", "summary" : "<p> Executes the specified string command in a separate process.</p>", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "cmd", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> command to be executed (without arguments)</div>" }, { "name" : "args", "type" : "xs:string", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the arguments passed to the executed command (e.g. \"-la\")</div>" } ], "returns" : { "type" : "element(process:result)", "description" : "the result of the execution as an element as shown in the documentation of this module. The exit-code element returns the exit code of the child process. For POSIX compliant platforms: returns the process exit code. If process is terminated or stopped: 128 + termination signal code. For Windows platforms: returns the return value of the process or the exit or terminate process specified value." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">process:PROC01 if an error occurred while communicating with the executed process.</xqdoc:error>" ] } ], "variables" : [  ] }, "http://www.28msec.com/modules/maps" : { "ns" : "http://www.28msec.com/modules/maps", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">This module defines a set of functions for working with maps. A map\n is identified by a string and can be created using the map:create function\n and dropped using the map:drop function.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">It is possible to create persistent and transient maps. The lifetime of a\n transient map is limited by the execution of the current query.\n A persistent map lives until it is explicitly dropped.\n Accordingly, it is also available to other requests.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">For example,\n <pre class=\"ace-static\" ace-mode=\"xquery\">map:create(\"my-map\", [\"string\", \"integer\"], { \"persistent\" : false })</pre>\n will create a transient map named my-map having two keys.\n The types of the keys are string and integer.\n </p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">The key of a particular entry in the map can consist of a tuple of\n atomic values (called key attributes). The actual type of each attribute\n is determined when the map is created. The value of each entry is a\n sequence of items. If an item in this sequence is a object or array,\n this item needs to belong to a collection, otherwise, an error is raised.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">For every persistent map, 28.io will create a new collection\n in your MongoDB database. The collection will have the prefix <em>_28.map</em>.\n For example, if your map is called foo, the according collection will\n be named <em>_28.map.foo</em>.</p>\n", "sees" : [  ], "authors" : [ "<xqdoc:author xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">28msec</xqdoc:author>" ], "version" : null, "encoding" : "utf-8", "namespaces" : [ { "uri" : "http://zorba.io/annotations", "prefix" : "an" }, { "uri" : "http://www.28msec.com/modules/maps", "prefix" : "map" }, { "uri" : "http://zorba.io/options/versioning", "prefix" : "ver" }, { "uri" : "http://zorba.io/errors", "prefix" : "zerr" } ], "functions" : [ { "isDocumented" : true, "arity" : 0, "name" : "available-maps", "qname" : "map:available-maps", "signature" : "() as string* external", "description" : " The function returns a sequence of names of the maps that are\n available (persistent and non-persistent). The sequence will be\n empty if there are no maps.\n", "summary" : "<p> The function returns a sequence of names of the maps that are\n available (persistent and non-persistent).</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [  ], "returns" : { "type" : "string*", "description" : "A sequence of string, one for each available map, or an emtpy sequence." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 2, "name" : "create", "qname" : "map:create", "signature" : "($name as string, $key-types as item()) as empty-sequence() external", "description" : " Create a persistent map with a given name and type identifiers for the key\n attributes.\n If the map has only one key attribute, a single type identifier is given,\n for more than one key attribute an array of type identifiers is given.\n Calling this function is equivalent to calling create with the options\n <pre xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">{ \"persistent\" : true }</pre>\n Note that the function is sequential and immediately creates the map.\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Each key-type should be specified as string (e.g. \"integer\",\n \"string\", \"boolean\", \"double\", or \"datetime\").</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">For example,\n <pre>map:create(\"my-map\", \"string\")</pre> or\n <pre>map:create(\"my-map\", [\"string\", \"integer\"])</pre>.</p>\n", "summary" : "<p> Create a persistent map with a given name and type identifiers for the key\n attributes.</p>", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "name", "type" : "string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the name of the map (the restrictons on collection names apply)</div>" }, { "name" : "key-types", "type" : "item()", "occurrence" : null, "description" : "" } ], "returns" : { "type" : "empty-sequence()", "description" : "the function is sequential and immediately creates the corresponding map. It returns the empty-sequence." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">err:XPTY0004 if any of the key attribute types is not a subtype of anyAtomicType.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">zerr:ZSTR0001 if a map with the given name already exists.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">zerr:ZDDY1000 if the given name is not a legal collection name</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">zerr:ZDDY0043 if any of the given options has an invalid type</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 3, "name" : "create", "qname" : "map:create", "signature" : "($name as string, $key-types as item(), $options as object()) as empty-sequence() external", "description" : " Create a map with a given name, type identifiers for the key attributes, and\n options.\n If the map has only one key attribute, a single type identifier is given,\n for more than one key attribute an array of type identifiers is given.\n Currently only one option is supported: To create a transient map the object\n <code xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">{ \"persistent\" : false }</code>\n has to be passed to the $options parameter.\n Note that the function is sequential and immediately creates the map in the\n store.\n", "summary" : "<p> Create a map with a given name, type identifiers for the key attributes, and\n options.</p>", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "name", "type" : "string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the name of the map (the restrictons on collection names apply)</div>" }, { "name" : "key-types", "type" : "item()", "occurrence" : null, "description" : "" }, { "name" : "options", "type" : "object()", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> an object describing options for the map</div>" } ], "returns" : { "type" : "empty-sequence()", "description" : "the function is sequential and immediately creates the corresponding map but returns the empty-sequence." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">err:XPTY0004 if any of the attribute types is not a subtype of anyAtomicType.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">zerr:ZSTR0001 if a map with the given name already exists.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">zerr:ZDDY1000 if the given name is not a legal collection name</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">zerr:ZDDY0043 if any of the given options has an invalid type</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 2, "name" : "delete", "qname" : "map:delete", "signature" : "($name as string, $key as item()) as empty-sequence() external", "description" : " Removes an entry identified by the given key from the map.\n If the map has only one key attribute, a single key value is given, for\n more than one key attribute an array of key values is given.\n Note that it is possible to insert entries with empty key attributes.\n However as the removing the entries is based on the \"eq\" comparison and\n as \"eq\" with an empty sequence always return false, it is not possible\n to delete these entries.\n", "summary" : "<p> Removes an entry identified by the given key from the map.</p>", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "name", "type" : "string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the name of the map</div>" }, { "name" : "key", "type" : "item()", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> either a single attribute key or an array of keys</div>" } ], "returns" : { "type" : "empty-sequence()", "description" : "the function is sequential and immediately deletes the entry into the map but returns the empty-sequence." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">zerr:ZDDY0023 if a map with the given name does not exist.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">zerr:ZDDY0025 if the given number of key attributes does not match the number of key attributes specified when creating the map (see the map:create function).</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">zerr:ZXQD0005 if any of the given key attributes can not be cast (or is not a subtype) of the corresponding key attribute specified when creating the map.</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 1, "name" : "drop", "qname" : "map:drop", "signature" : "($name as string) as empty-sequence() external", "description" : " Deletes the map with the given name.\n", "summary" : "<p> Deletes the map with the given name.</p>", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "name", "type" : "string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the name of the map to drop</div>" } ], "returns" : { "type" : "empty-sequence()", "description" : "the function is sequential and immediately drops the map. It returns the empty-sequence." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">zerr:ZDDY0023 if a map with the given name does not exist.</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 2, "name" : "get", "qname" : "map:get", "signature" : "($name as string, $key as item()) as item()* external", "description" : " Returns the value of the entry with the given key from the map.\n If the map has only one key attribute, a single key value is given, for\n more than one key attribute an array of key values is given.\n Note that it is possible to insert entries with empty key attributes.\n However as the getting the entries is based on the \"eq\" comparison and\n as \"eq\" with an empty sequence always return false, it is not possible\n to retrieve these entries.\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">For example,\n <pre>map:get(\"my-map\", \"key\")</pre> or\n <pre>map:get(\"my-map\", [ \"key1\", \"key2\" ])</pre>.</p>\n", "summary" : "<p> Returns the value of the entry with the given key from the map.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "name", "type" : "string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the name of the map</div>" }, { "name" : "key", "type" : "item()", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> either a single attribute key or an array of keys</div>" } ], "returns" : { "type" : "item()*", "description" : "the value of the entry in the map identified by the given key. The empty-sequence will be returned if no entry with the given key is contained in the map." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">zerr:ZDDY0023 if a map with the given name does not exist.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">zerr:ZDDY0025 if the given number of key attributes does not match the number of key attributes specified when creating the map (see the map:create function).</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">zerr:ZXQD0005 if any of the given key attributes can not be cast (or is not a subtype) of the corresponding key attribute specified when creating the map.</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 3, "name" : "insert", "qname" : "map:insert", "signature" : "($name as string, $key as item(), $value as item()*) as empty-sequence() external", "description" : " Inserts a new entry into the map with the given name.\n If the map has only one key attribute, a single key value is given, for\n more than one key attribute an array of key values is given.\n If an entry with the given key already exists in the map, the value\n sequences of the existing entry and the sequence passed using $value\n argument are concatenated.\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">If a map is persistent and if an item in the value sequence\n is an object or array, this item needs to belong to a collection,\n otherwise, an an error is raised.</p>\n Note that it is possible to insert entries with empty key attributes\n or key attributes having the value <code xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">null</code>. However, as\n the comparison with an empty sequence or null always returns false,\n it is not possible to retrieve these entries.\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">For example,\n <pre>map:insert(\"my-map\", \"key\", \"value\")</pre> or\n <pre>map:insert(\"my-map\", [ \"key1\", \"key2\" ] , (42, \"value\"))</pre>.</p>\n", "summary" : "<p> Inserts a new entry into the map with the given name.</p>", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "name", "type" : "string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the name of the map</div>" }, { "name" : "key", "type" : "item()", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> either a single attribute key or an array of keys</div>" }, { "name" : "value", "type" : "item()", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the value of the entry to insert</div>" } ], "returns" : { "type" : "empty-sequence()", "description" : "the function is sequential and immediately inserts the entry into the map. It returns the empty-sequence." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">zerr:ZDDY0023 if a map with the given name does not exist.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">zerr:ZDDY0025 if the given number of key attributes does not match the number of key attributes specified when creating the map (see the map:create function).</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">zerr:ZXQD0005 if any of the given key attributes can not be cast (or is not a subtype) of the corresponding key attribute specified when creating the map.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">zerr:ZDDY1003 if the value to insert is an object or array it must belong to a collection.</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 1, "name" : "keys", "qname" : "map:keys", "signature" : "($name as string) as array()* external", "description" : " Returns the keys of all entries of a map. The keys\n are returned as sequence of arrays.\n The following condition always holds:\n <tt xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">map:size($name) eq count(map:keys($name))</tt>\n", "summary" : "<p> Returns the keys of all entries of a map.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "name", "type" : "string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the name of the map</div>" } ], "returns" : { "type" : "array()*", "description" : "an sequence of arrays each array containing the values of all attributes of one key." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">zerr:ZDDY0023 if a map with the given name does not exist.</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 1, "name" : "options", "qname" : "map:options", "signature" : "($name as string) as object() external", "description" : " The function returns the options that were passed during creation or the\n default options if no options were passed.\n", "summary" : "<p> The function returns the options that were passed during creation or the\n default options if no options were passed.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "name", "type" : "string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the name of the map</div>" } ], "returns" : { "type" : "object()", "description" : "an options object" }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">zerr:ZDDY0023 if a map with the given name does not exist.</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 1, "name" : "size", "qname" : "map:size", "signature" : "($name as string) as integer external", "description" : " The number of entries in a map.\n The following condition always holds:\n <tt xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">map:size($name) eq count(map:keys($name))</tt>\n", "summary" : "<p> The number of entries in a map.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "name", "type" : "string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the name of the map</div>" } ], "returns" : { "type" : "integer", "description" : "the number of entries in the map." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">zerr:ZDDY0023 if a map with the given name does not exist.</xqdoc:error>" ] } ], "variables" : [  ] }, "http://www.w3.org/2005/xpath-functions/math" : { "ns" : "http://www.w3.org/2005/xpath-functions/math", "description" : " This module contains all the functions part of the\n W3C XPath and XQuery Functions and Operators 3.0\n section \"4.7 Trigonometric and exponential functions\".\n", "sees" : [ "<xqdoc:see xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">http://www.w3.org/TR/xpath-functions-30/#trigonometry</xqdoc:see>" ], "authors" : [ "<xqdoc:author xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">www.w3c.org</xqdoc:author>" ], "version" : null, "encoding" : "utf-8", "namespaces" : [ { "uri" : "http://www.w3.org/2005/xpath-functions/math", "prefix" : "math" } ], "functions" : [ { "isDocumented" : true, "arity" : 1, "name" : "acos", "qname" : "math:acos", "signature" : "($arg as xs:double?) as xs:double? external", "description" : " <div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">Returns the arc cosine of the argument, the result being in the range zero to\n                +<var>π</var> radians.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\"><example role=\"signature\"><proto name=\"acos\" return-type=\"xs:double?\" isOp=\"no\" prefix=\"math\" returnEmptyOk=\"no\" returnSeq=\"no\" returnVaries=\"no\" isSchema=\"no\" isDatatype=\"no\" isSpecial=\"no\"><arg name=\"arg\" type=\"xs:double?\"/></proto></example></p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">This function is <termref def=\"dt-deterministic\">deterministic</termref>, <termref def=\"dt-context-independent\">context-independent</termref>,  and <termref def=\"dt-focus-independent\">focus-independent</termref>. </p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">If <code>$arg</code> is the empty sequence, the function returns the empty sequence.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\" diff=\"chg\" at=\"G\">Otherwise the result is the arc cosine of <code>$</code><var>θ</var>,\n             treated as an angle in radians, as defined in the <bibref ref=\"ieee754-2008\"/>\n             specification of the <code>acos</code> function applied to 64-bit binary floating point\n             values.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The treatment of the <code>invalidOperation</code> exception is defined in <specref ref=\"op.numeric\"/>. </p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">If <code>$arg</code> is <code>NaN</code>, or if its absolute value is greater than one,\n             then the result is <code>NaN</code>.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">In other cases the result is an <code>xs:double</code> value representing an angle\n                <var>θ</var> in radians in the range <code>0 &lt;= $</code><var>θ</var><code> &lt;=\n                +</code><var>π</var>. </p></div>\n", "summary" : "<p>  Returns the arc cosine of the argument, the result being in the range zero to\n                + π  radians.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "arg", "type" : "xs:double", "occurrence" : "?", "description" : "" } ], "returns" : { "type" : "xs:double?", "description" : "" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "asin", "qname" : "math:asin", "signature" : "($arg as xs:double?) as xs:double? external", "description" : " <div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">Returns the arc sine of the argument, the result being in the range\n                -<var>π</var>/2 to +<var>π</var>/2 radians.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\"><example role=\"signature\"><proto name=\"asin\" return-type=\"xs:double?\" isOp=\"no\" prefix=\"math\" returnEmptyOk=\"no\" returnSeq=\"no\" returnVaries=\"no\" isSchema=\"no\" isDatatype=\"no\" isSpecial=\"no\"><arg name=\"arg\" type=\"xs:double?\"/></proto></example></p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">This function is <termref def=\"dt-deterministic\">deterministic</termref>, <termref def=\"dt-context-independent\">context-independent</termref>,  and <termref def=\"dt-focus-independent\">focus-independent</termref>. </p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">If <code>$arg</code> is the empty sequence, the function returns the empty sequence.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\" diff=\"chg\" at=\"G\">Otherwise the result is the arc sine of <code>$</code><var>θ</var>,\n             treated as an angle in radians, as defined in the <bibref ref=\"ieee754-2008\"/>\n             specification of the <code>asin</code> function applied to 64-bit binary floating point\n             values.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The treatment of the <code>invalidOperation</code> and <code>underflow</code> exceptions\n             is defined in <specref ref=\"op.numeric\"/>. </p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">If <code>$arg</code> is positive or negative zero, the result is <code>$arg</code>.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">If <code>$arg</code> is <code>NaN</code>, or if its absolute value is greater than one,\n             then the result is <code>NaN</code>.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">In other cases the result is an <code>xs:double</code> value representing an angle\n                <var>θ</var> in radians in the range -<var>π</var><code>/2 &lt;=\n                $</code><var>θ</var><code> &lt;= +</code><var>π</var><code>/2</code>. </p></div>\n", "summary" : "<p>  Returns the arc sine of the argument, the result being in the range\n                - π /2 to + π /2 radians.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "arg", "type" : "xs:double", "occurrence" : "?", "description" : "" } ], "returns" : { "type" : "xs:double?", "description" : "" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "atan", "qname" : "math:atan", "signature" : "($arg as xs:double?) as xs:double? external", "description" : " <div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">Returns the arc tangent of the argument, the result being in the range\n                -<var>π</var>/2 to +<var>π</var>/2 radians.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\"><example role=\"signature\"><proto name=\"atan\" return-type=\"xs:double?\" isOp=\"no\" prefix=\"math\" returnEmptyOk=\"no\" returnSeq=\"no\" returnVaries=\"no\" isSchema=\"no\" isDatatype=\"no\" isSpecial=\"no\"><arg name=\"arg\" type=\"xs:double?\"/></proto></example></p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">This function is <termref def=\"dt-deterministic\">deterministic</termref>, <termref def=\"dt-context-independent\">context-independent</termref>,  and <termref def=\"dt-focus-independent\">focus-independent</termref>. </p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">If <code>$arg</code> is the empty sequence, the function returns the empty sequence.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\" diff=\"chg\" at=\"G\">Otherwise the result is the arc tangent of <code>$</code><var>θ</var>,\n             treated as an angle in radians, as defined in the <bibref ref=\"ieee754-2008\"/>\n             specification of the <code>atan</code> function applied to 64-bit binary floating point\n             values.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The treatment of the <code>underflow</code> exception is defined in <specref ref=\"op.numeric\"/>. </p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">If <code>$arg</code> is positive or negative zero, the result is <code>$arg</code>.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">If <code>$arg</code> is <code>NaN</code> then the result is <code>NaN</code>.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">In other cases the result is an <code>xs:double</code> value representing an angle\n                <var>θ</var> in radians in the range -<var>π</var><code>/2 &lt;=\n                $</code><var>θ</var><code> &lt;= +</code><var>π</var><code>/2</code>. </p></div>\n", "summary" : "<p>  Returns the arc tangent of the argument, the result being in the range\n                - π /2 to + π /2 radians.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "arg", "type" : "xs:double", "occurrence" : "?", "description" : "" } ], "returns" : { "type" : "xs:double?", "description" : "" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 2, "name" : "atan2", "qname" : "math:atan2", "signature" : "($y as xs:double, $x as xs:double) as xs:double external", "description" : " <div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">Returns the angle in radians subtended at the origin by the point on a plane\n             with coordinates (x, y) and the positive x-axis, the result being in the range\n                -<var>π</var> to +<var>π</var>.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\"><example role=\"signature\"><proto name=\"atan2\" return-type=\"xs:double\" isOp=\"no\" prefix=\"math\" returnEmptyOk=\"no\" returnSeq=\"no\" returnVaries=\"no\" isSchema=\"no\" isDatatype=\"no\" isSpecial=\"no\"><arg name=\"y\" type=\"xs:double\"/><arg name=\"x\" type=\"xs:double\"/></proto></example></p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">This function is <termref def=\"dt-deterministic\">deterministic</termref>, <termref def=\"dt-context-independent\">context-independent</termref>,  and <termref def=\"dt-focus-independent\">focus-independent</termref>. </p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\" diff=\"chg\" at=\"G\">The result is the value of <code>atan2(y, x)</code> as defined in the\n                <bibref ref=\"ieee754-2008\"/> specification of the <code>atan2</code> function applied\n             to 64-bit binary floating point values.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The treatment of the <code>underflow</code> exception is defined in <specref ref=\"op.numeric\"/>. </p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">If <phrase diff=\"chg\" at=\"L\">either argument</phrase> is <code>NaN</code>\n             then the result is <code>NaN</code>.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">If <code>$y</code> is positive and <code>$x</code> is positive and finite, then\n             (subject to rules for overflow, underflow and approximation)\n          the value of <code>atan2($y, $x)</code> is <code>atan($y div $x)</code>.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">If <code>$y</code> is positive and <code>$x</code> is negative and finite, then\n             (subject to the same caveats)\n             the value of <code>atan2($y, $x)</code> is <var>π</var> <code>- atan($y div $x)</code>.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">Some results for special values of the arguments are shown in the examples below.</p></div>\n", "summary" : "<p>  Returns the angle in radians subtended at the origin by the point on a plane\n             with coordinates (x, y) and the positive x-axis, the result being in the range\n                - π  to + π .</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "y", "type" : "xs:double", "occurrence" : null, "description" : "" }, { "name" : "x", "type" : "xs:double", "occurrence" : null, "description" : "" } ], "returns" : { "type" : "xs:double", "description" : "" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "cos", "qname" : "math:cos", "signature" : "($theta as xs:double?) as xs:double? external", "description" : " <div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">Returns the cosine of the argument, expressed in radians.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\"><example role=\"signature\"><proto name=\"cos\" return-type=\"xs:double?\" isOp=\"no\" prefix=\"math\" returnEmptyOk=\"no\" returnSeq=\"no\" returnVaries=\"no\" isSchema=\"no\" isDatatype=\"no\" isSpecial=\"no\"><arg name=\"θ\" type=\"xs:double?\"/></proto></example></p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">This function is <termref def=\"dt-deterministic\">deterministic</termref>, <termref def=\"dt-context-independent\">context-independent</termref>,  and <termref def=\"dt-focus-independent\">focus-independent</termref>. </p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">If <code>$</code><var>θ</var> is the empty sequence, the function returns the empty\n             sequence.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">If <code>$</code><var>θ</var> is positive or negative infinity, or <code>NaN</code>,\n             then the result is <code>NaN</code>.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\" diff=\"chg\" at=\"G\">Otherwise the result is the cosine of <code>$</code><var>θ</var>,\n             treated as an angle in radians, as defined in the <bibref ref=\"ieee754-2008\"/>\n             specification of the <code>cos</code> function applied to 64-bit binary floating point\n             values.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The treatment of the <code>invalidOperation</code> exception is defined in <specref ref=\"op.numeric\"/>. </p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">If <code>$</code><var>θ</var> is positive or negative zero, the result is\n                <code>$</code><var>θ</var>.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">If <code>$</code><var>θ</var> is positive or negative infinity, or <code>NaN</code>,\n             then the result is <code>NaN</code>.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">Otherwise the result is always in the range -1.0e0 to +1.0e0</p></div>\n", "summary" : "<p>  Returns the cosine of the argument, expressed in radians.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "theta", "type" : "xs:double", "occurrence" : "?", "description" : "" } ], "returns" : { "type" : "xs:double?", "description" : "" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "exp", "qname" : "math:exp", "signature" : "($arg as xs:double?) as xs:double? external", "description" : " <div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">Returns the value of <var>e</var><sup><var>x</var></sup>.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\"><example role=\"signature\"><proto name=\"exp\" return-type=\"xs:double?\" isOp=\"no\" prefix=\"math\" returnEmptyOk=\"no\" returnSeq=\"no\" returnVaries=\"no\" isSchema=\"no\" isDatatype=\"no\" isSpecial=\"no\"><arg name=\"arg\" type=\"xs:double?\"/></proto></example></p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">This function is <termref def=\"dt-deterministic\">deterministic</termref>, <termref def=\"dt-context-independent\">context-independent</termref>,  and <termref def=\"dt-focus-independent\">focus-independent</termref>. </p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">If <code>$arg</code> is the empty sequence, the function returns the empty sequence.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\" diff=\"chg\" at=\"G\">Otherwise the result is the mathematical constant <var>e</var>\n             raised to the power of <code>$arg</code>, as defined in the <bibref ref=\"ieee754-2008\"/>\n             specification of the <code>exp</code> function applied to 64-bit binary floating point\n             values.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The treatment of overflow and underflow is defined in <specref ref=\"op.numeric\"/>.</p></div>\n", "summary" : "<p>  Returns the value of  e x .</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "arg", "type" : "xs:double", "occurrence" : "?", "description" : "" } ], "returns" : { "type" : "xs:double?", "description" : "" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "exp10", "qname" : "math:exp10", "signature" : "($arg as xs:double?) as xs:double? external", "description" : " <div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">Returns the value of <code>10</code><sup><var>x</var></sup>.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\"><example role=\"signature\"><proto name=\"exp10\" return-type=\"xs:double?\" isOp=\"no\" prefix=\"math\" returnEmptyOk=\"no\" returnSeq=\"no\" returnVaries=\"no\" isSchema=\"no\" isDatatype=\"no\" isSpecial=\"no\"><arg name=\"arg\" type=\"xs:double?\"/></proto></example></p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">This function is <termref def=\"dt-deterministic\">deterministic</termref>, <termref def=\"dt-context-independent\">context-independent</termref>,  and <termref def=\"dt-focus-independent\">focus-independent</termref>. </p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">If <code>$arg</code> is the empty sequence, the function returns the empty sequence.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\" diff=\"chg\" at=\"G\">Otherwise the result is ten raised to the power of <code>$arg</code>,\n             as defined in the <bibref ref=\"ieee754-2008\"/> specification of the <code>exp10</code>\n             function applied to 64-bit binary floating point values.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The treatment of overflow and underflow is defined in <specref ref=\"op.numeric\"/>.</p></div>\n", "summary" : "<p>  Returns the value of  10 x .</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "arg", "type" : "xs:double", "occurrence" : "?", "description" : "" } ], "returns" : { "type" : "xs:double?", "description" : "" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "log", "qname" : "math:log", "signature" : "($arg as xs:double?) as xs:double? external", "description" : " <div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">Returns the natural logarithm of the argument.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\"><example role=\"signature\"><proto name=\"log\" return-type=\"xs:double?\" isOp=\"no\" prefix=\"math\" returnEmptyOk=\"no\" returnSeq=\"no\" returnVaries=\"no\" isSchema=\"no\" isDatatype=\"no\" isSpecial=\"no\"><arg name=\"arg\" type=\"xs:double?\"/></proto></example></p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">This function is <termref def=\"dt-deterministic\">deterministic</termref>, <termref def=\"dt-context-independent\">context-independent</termref>,  and <termref def=\"dt-focus-independent\">focus-independent</termref>. </p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">If <code>$arg</code> is the empty sequence, the function returns the empty sequence.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\" diff=\"chg\" at=\"G\">Otherwise the result is the natural logarithm of <code>$arg</code>, as\n             defined in the <bibref ref=\"ieee754-2008\"/> specification of the <code>log</code>\n             function applied to 64-bit binary floating point values.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The treatment of <code>divideByZero</code> and <code>invalidOperation</code> exceptions\n             is defined in <specref ref=\"op.numeric\"/>. The effect is that if the argument is less\n             than or equal to zero, the result is <code>NaN</code>.</p></div>\n", "summary" : "<p>  Returns the natural logarithm of the argument.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "arg", "type" : "xs:double", "occurrence" : "?", "description" : "" } ], "returns" : { "type" : "xs:double?", "description" : "" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "log10", "qname" : "math:log10", "signature" : "($arg as xs:double?) as xs:double? external", "description" : " <div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">Returns the base-ten logarithm of the argument.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\"><example role=\"signature\"><proto name=\"log10\" return-type=\"xs:double?\" isOp=\"no\" prefix=\"math\" returnEmptyOk=\"no\" returnSeq=\"no\" returnVaries=\"no\" isSchema=\"no\" isDatatype=\"no\" isSpecial=\"no\"><arg name=\"arg\" type=\"xs:double?\"/></proto></example></p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">This function is <termref def=\"dt-deterministic\">deterministic</termref>, <termref def=\"dt-context-independent\">context-independent</termref>,  and <termref def=\"dt-focus-independent\">focus-independent</termref>. </p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">If <code>$arg</code> is the empty sequence, the function returns the empty sequence.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\" diff=\"chg\" at=\"G\">Otherwise the result is the base-10 logarithm of <code>$arg</code>, as\n             defined in the <bibref ref=\"ieee754-2008\"/> specification of the <code>log10</code>\n             function applied to 64-bit binary floating point values.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The treatment of <code>divideByZero</code> and <code>invalidOperation</code> exceptions\n             is defined in <specref ref=\"op.numeric\"/>. The effect is that if the argument is less\n             than or equal to zero, the result is <code>NaN</code>.</p></div>\n", "summary" : "<p>  Returns the base-ten logarithm of the argument.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "arg", "type" : "xs:double", "occurrence" : "?", "description" : "" } ], "returns" : { "type" : "xs:double?", "description" : "" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 0, "name" : "pi", "qname" : "math:pi", "signature" : "() as xs:double external", "description" : " <div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">Returns an approximation to the mathematical constant <var>π</var>.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\"><example role=\"signature\"><proto name=\"pi\" return-type=\"xs:double\" isOp=\"no\" prefix=\"math\" returnEmptyOk=\"no\" returnSeq=\"no\" returnVaries=\"no\" isSchema=\"no\" isDatatype=\"no\" isSpecial=\"no\"/></example></p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">This function is <termref def=\"dt-deterministic\">deterministic</termref>, <termref def=\"dt-context-independent\">context-independent</termref>,  and <termref def=\"dt-focus-independent\">focus-independent</termref>. </p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">This function returns the <code>xs:double</code> value whose lexical representation is\n             3.141592653589793e0</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The expression <code>2*math:pi()</code> returns <code>6.283185307179586e0</code>.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The expression <code>60 * (math:pi() div 180)</code> converts an angle of 60 degrees\n                to radians. </p></div>\n", "summary" : "<p>  Returns an approximation to the mathematical constant  π .</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [  ], "returns" : { "type" : "xs:double", "description" : "" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 2, "name" : "pow", "qname" : "math:pow", "signature" : "($x as xs:double?, $y as numeric) as xs:double? external", "description" : " <div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">Returns the result of raising the first argument to the power of the\n             second.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\"><example role=\"signature\"><proto name=\"pow\" return-type=\"xs:double?\" isOp=\"no\" prefix=\"math\" returnEmptyOk=\"no\" returnSeq=\"no\" returnVaries=\"no\" isSchema=\"no\" isDatatype=\"no\" isSpecial=\"no\"><arg name=\"x\" type=\"xs:double?\"/><arg name=\"y\" type=\"numeric\"/></proto></example></p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">This function is <termref def=\"dt-deterministic\">deterministic</termref>, <termref def=\"dt-context-independent\">context-independent</termref>,  and <termref def=\"dt-focus-independent\">focus-independent</termref>. </p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">If <code>$x</code> is the empty sequence, the function returns the empty sequence.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">If <code>$y</code> is an instance of <code>xs:integer</code>, the result is\n                <code>$x</code> raised to the power of <code>$y</code> as defined in the <bibref ref=\"ieee754-2008\"/> specification of the <code>pown</code> function applied to a\n             64-bit binary floating point value and an integer.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">Otherwise <code>$y</code> is converted to an <code>xs:double</code> by numeric\n             promotion, and the result is the value of <code>$x</code> raised to the power of\n                <code>$y</code> as defined in the <bibref ref=\"ieee754-2008\"/> specification of the\n                <code>pow</code> function applied to two 64-bit binary floating point values.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The treatment of the <code>divideByZero</code> and <code>invalidOperation</code>\n             exceptions is defined in <specref ref=\"op.numeric\"/>. Some of the consequences are\n             illustrated in the examples below.</p></div>\n", "summary" : "<p>  Returns the result of raising the first argument to the power of the\n             second.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "x", "type" : "xs:double", "occurrence" : "?", "description" : "" }, { "name" : "y", "type" : "numeric", "occurrence" : null, "description" : "" } ], "returns" : { "type" : "xs:double?", "description" : "" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "sin", "qname" : "math:sin", "signature" : "($theta as xs:double?) as xs:double? external", "description" : " <div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">Returns the sine of the argument, expressed in radians.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\"><example role=\"signature\"><proto name=\"sin\" return-type=\"xs:double?\" isOp=\"no\" prefix=\"math\" returnEmptyOk=\"no\" returnSeq=\"no\" returnVaries=\"no\" isSchema=\"no\" isDatatype=\"no\" isSpecial=\"no\"><arg name=\"θ\" type=\"xs:double?\"/></proto></example></p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">This function is <termref def=\"dt-deterministic\">deterministic</termref>, <termref def=\"dt-context-independent\">context-independent</termref>,  and <termref def=\"dt-focus-independent\">focus-independent</termref>. </p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">If <code>$</code><var>θ</var> is the empty sequence, the function returns the empty\n             sequence.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\" diff=\"chg\" at=\"G\">Otherwise the result is the sine of <code>$</code><var>θ</var>,\n             treated as an angle in radians, as defined in the <bibref ref=\"ieee754-2008\"/>\n             specification of the <code>sin</code> function applied to 64-bit binary floating point\n             values.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The treatment of the <code>invalidOperation</code> and <code>underflow</code> exceptions\n             is defined in <specref ref=\"op.numeric\"/>. </p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">If <code>$</code><var>θ</var> is positive or negative zero, the result is\n                <code>$</code><var>θ</var>.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">If <code>$</code><var>θ</var> is positive or negative infinity, or <code>NaN</code>,\n             then the result is <code>NaN</code>.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">Otherwise the result is always in the range -1.0e0 to +1.0e0</p></div>\n", "summary" : "<p>  Returns the sine of the argument, expressed in radians.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "theta", "type" : "xs:double", "occurrence" : "?", "description" : "" } ], "returns" : { "type" : "xs:double?", "description" : "" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "sqrt", "qname" : "math:sqrt", "signature" : "($arg as xs:double?) as xs:double? external", "description" : " <div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">Returns the non-negative square root of the argument.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\"><example role=\"signature\"><proto name=\"sqrt\" return-type=\"xs:double?\" isOp=\"no\" prefix=\"math\" returnEmptyOk=\"no\" returnSeq=\"no\" returnVaries=\"no\" isSchema=\"no\" isDatatype=\"no\" isSpecial=\"no\"><arg name=\"arg\" type=\"xs:double?\"/></proto></example></p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">This function is <termref def=\"dt-deterministic\">deterministic</termref>, <termref def=\"dt-context-independent\">context-independent</termref>,  and <termref def=\"dt-focus-independent\">focus-independent</termref>. </p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">If <code>$arg</code> is the empty sequence, the function returns the empty sequence.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\" diff=\"chg\" at=\"G\">Otherwise the result is the mathematical non-negative square root of\n                <code>$arg</code> as defined in the <bibref ref=\"ieee754-2008\"/> specification of the\n                <code>squareRoot</code> function applied to 64-bit binary floating point values.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The treatment of the <code>invalidOperation</code> exception is defined in <specref ref=\"op.numeric\"/>. The effect is that if the argument is less than zero, the result\n             is <code>NaN</code>.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">If <code>$arg</code> is positive or negative zero, positive infinity, or\n                <code>NaN</code>, then the result is <code>$arg</code>. (Negative zero is the only\n             case where the result can have negative sign)</p></div>\n", "summary" : "<p>  Returns the non-negative square root of the argument.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "arg", "type" : "xs:double", "occurrence" : "?", "description" : "" } ], "returns" : { "type" : "xs:double?", "description" : "" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "tan", "qname" : "math:tan", "signature" : "($theta as xs:double?) as xs:double? external", "description" : " <div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">Returns the tangent of the argument, expressed in radians.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\"><example role=\"signature\"><proto name=\"tan\" return-type=\"xs:double?\" isOp=\"no\" prefix=\"math\" returnEmptyOk=\"no\" returnSeq=\"no\" returnVaries=\"no\" isSchema=\"no\" isDatatype=\"no\" isSpecial=\"no\"><arg name=\"θ\" type=\"xs:double?\"/></proto></example></p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">This function is <termref def=\"dt-deterministic\">deterministic</termref>, <termref def=\"dt-context-independent\">context-independent</termref>,  and <termref def=\"dt-focus-independent\">focus-independent</termref>. </p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">If <code>$</code><var>θ</var> is the empty sequence, the function returns the empty\n             sequence.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\" diff=\"chg\" at=\"G\">Otherwise the result is the tangent of <code>$</code><var>θ</var>,\n             treated as an angle in radians, as defined in the <bibref ref=\"ieee754-2008\"/>\n             specification of the <code>tan</code> function applied to 64-bit binary floating point\n             values.</p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">The treatment of the <code>invalidOperation</code> and <code>underflow</code> exceptions\n             is defined in <specref ref=\"op.numeric\"/>. </p><p xmlns:e=\"http://www.w3.org/1999/XSL/Spec/ElementSyntax\">If <code>$</code><var>θ</var> is positive or negative infinity, or <code>NaN</code>,\n             then the result is <code>NaN</code>.</p></div>\n", "summary" : "<p>  Returns the tangent of the argument, expressed in radians.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "theta", "type" : "xs:double", "occurrence" : "?", "description" : "" } ], "returns" : { "type" : "xs:double?", "description" : "" }, "errors" : [  ] } ], "variables" : [  ] }, "http://www.zorba-xquery.com/modules/oauth/client" : { "ns" : "http://www.zorba-xquery.com/modules/oauth/client", "description" : " This module provides the functions necessary to acquire access to the personal\n resources of a user through the open standard called\n <a xmlns:xqdoc=\"http://www.xqdoc.org/1.0\" href=\"http://oauth.net/\" target=\"_blank\">OAuth</a>.\n The application/mashup creator does not need to know the\n specifics of <a xmlns:xqdoc=\"http://www.xqdoc.org/1.0\" href=\"http://oauth.net/\" target=\"_blank\">OAuth</a> to use this module.\n", "sees" : [ "<xqdoc:see xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"><a href=\"http://oauth.net/\" target=\"_blank\">OAuth Website</a></xqdoc:see>" ], "authors" : [ "<xqdoc:author xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Stephanie Russell</xqdoc:author>", "<xqdoc:author xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"><a href=\"mailto:william.candillon@28msec.com\">William Candillon</a></xqdoc:author>" ], "version" : null, "encoding" : "utf-8", "namespaces" : [ { "uri" : "http://zorba.io/annotations", "prefix" : "an" }, { "uri" : "http://zorba.io/modules/base64", "prefix" : "base64" }, { "uri" : "http://zorba.io/features", "prefix" : "f" }, { "uri" : "http://zorba.io/modules/hmac", "prefix" : "hmac" }, { "uri" : "http://expath.org/ns/http-client", "prefix" : "http" }, { "uri" : "http://expath.org/ns/http-client", "prefix" : "http-client" }, { "uri" : "http://www.zorba-xquery.com/modules/oauth/client", "prefix" : "oauth" }, { "uri" : "http://www.zorba-xquery.com/modules/oauth/error", "prefix" : "oerr" }, { "uri" : "http://zorba.io/options/features", "prefix" : "op" }, { "uri" : "http://www.zorba-xquery.com/schemas/oauth/parameters", "prefix" : "p" }, { "uri" : "http://zorba.io/modules/random", "prefix" : "ra" }, { "uri" : "http://www.zorba-xquery.com/schemas/oauth/service-provider", "prefix" : "sp" }, { "uri" : "http://zorba.io/options/versioning", "prefix" : "ver" } ], "functions" : [ { "isDocumented" : true, "arity" : 2, "name" : "access-token", "qname" : "oauth:access-token", "signature" : "($service-provider as schema-element(sp:service-provider), $parameters as schema-element(p:parameters)) as schema-element(p:parameters)", "description" : " This function allows the client to obtain a set of token credentials from the service provider by making an authenticated HTTP request to the Token Request endpoint.\n This function is provided for convenience.\n", "summary" : "<p> This function allows the client to obtain a set of token credentials from the service provider by making an authenticated HTTP request to the Token Request endpoint.</p>", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "service-provider", "type" : "schema-element(sp:service-provider)", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> Contains service provider information</div>" }, { "name" : "parameters", "type" : "schema-element(p:parameters)", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> parameters</div>" } ], "returns" : { "type" : "schema-element(p:parameters)", "description" : "token credentials correctly parsed as parameter elements, or an error if http response status is not 200 OK <pre xmlns:xqdoc=\"http://www.xqdoc.org/1.0\" class=\"ace-static\" ace-mode=\"xquery\"> let $service-provider := oauth:service-provider(...) let $parameters := oauth:parameters(\"oauth_token\", \"#\") let $parameters := oauth:add-parameter($parameters, \"oauth_token_secret\", \"#\") let $tokens := oauth:access-token($service-provider, $parameters) ... </pre>" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 3, "name" : "add-parameter", "qname" : "oauth:add-parameter", "signature" : "($parameters as schema-element(p:parameters)?, $name as xs:string, $value as xs:string) as schema-element(p:parameters)", "description" : " Adds an OAuth parameter to an OAuth Parameters instance.\n Instances of OAuth parameters are used to\n contain value/pair data such as <em xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">oauth_token</em>\n and <em xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">oauth_token_secret</em>.\n For instance the following code snippet:\n <pre xmlns:xqdoc=\"http://www.xqdoc.org/1.0\" class=\"ace-static\" ace-mode=\"xquery\">\n  let $params := oauth:parameters(\"oauth_token\", \"#\")\n  let $params := oauth:add-parameter($params, \"oauth_token_secret\", \"#\")\n  return $params\n </pre>\n Returns the following XML schema instance:\n <pre xmlns:xqdoc=\"http://www.xqdoc.org/1.0\" class=\"ace-static\" ace-mode=\"xquery\">\n <p:parameters xmlns:p=\"http://www.zorba-xquery.com/schemas/oauth/parameters\">\n   <p:parameter name=\"oauth_token\" value=\"#\"/>\n   <p:parameter name=\"oauth_token_secret\" value=\"#\"/>\n </p:parameters>\n </pre>\n", "summary" : "<p> Adds an OAuth parameter to an OAuth Parameters instance.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "parameters", "type" : "schema-element(p:parameters)", "occurrence" : "?", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> parameters</div>" }, { "name" : "name", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> parameter name</div>" }, { "name" : "value", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> parameter value</div>" } ], "returns" : { "type" : "schema-element(p:parameters)", "description" : "instance of the OAuth parameters XML schema." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 2, "name" : "parameter", "qname" : "oauth:parameter", "signature" : "($params as schema-element(p:parameters), $string as xs:string) as xs:string", "description" : " This function returns the string value of the parameters whose key matches a $string input.\n", "summary" : "<p> This function returns the string value of the parameters whose key matches a $string input.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "params", "type" : "schema-element(p:parameters)", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> element parameters</div>" }, { "name" : "string", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> string as the \"key\" name</div>" } ], "returns" : { "type" : "xs:string", "description" : "string value of the parameter with key $string <pre xmlns:xqdoc=\"http://www.xqdoc.org/1.0\" class=\"ace-static\" ace-mode=\"xquery\"> let $params := oauth:parameters(\"oauth_token\", \"token\") let $params := oauth:add-parameter($params, \"oauth_token_secret\", \"secret\") let $token-secret := oauth:parameter($params, \"oauth_token_secret\") return $token-secret </pre>" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 2, "name" : "parameters", "qname" : "oauth:parameters", "signature" : "($name as xs:string, $value as xs:string) as element(p:parameters)", "description" : " Create an OAuth Parameters instance.\n Instances of OAuth parameters are used to\n contain value/pair data such as <em xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">oauth_token</em>\n and <em xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">oauth_token_secret</em>.\n For instance the following code snippet:\n <pre xmlns:xqdoc=\"http://www.xqdoc.org/1.0\" class=\"ace-static\" ace-mode=\"xquery\">\n  oauth:parameters(\"oauth_token\", \"#\")\n </pre>\n Returns the following XML schema instance:\n <pre xmlns:xqdoc=\"http://www.xqdoc.org/1.0\" class=\"ace-static\" ace-mode=\"xml\">\n <p:parameters xmlns:p=\"http://www.zorba-xquery.com/schemas/oauth/parameters\">\n   <p:parameter name=\"oauth_token\" value=\"#\"/>\n </p:parameters>\n </pre>\n", "summary" : "<p> Create an OAuth Parameters instance.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "name", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> parameter name</div>" }, { "name" : "value", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> parameter value</div>" } ], "returns" : { "type" : "element(p:parameters)", "description" : "instance of the OAuth parameters XML schema." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 3, "name" : "protected-resource", "qname" : "oauth:protected-resource", "signature" : "($protected-resource as schema-element(http:request), $service-provider as schema-element(sp:service-provider), $parameters as schema-element(p:parameters)) as item()*", "description" : " This function allows the client access to the protected resources of the user.\n This function is provided for convenience.\n", "summary" : "<p> This function allows the client access to the protected resources of the user.</p>", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "protected-resource", "type" : "schema-element(http:request)", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> (Not schema-validated) http:request element with http method and href.</div>" }, { "name" : "service-provider", "type" : "schema-element(sp:service-provider)", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> Information about the service provider</div>" }, { "name" : "parameters", "type" : "schema-element(p:parameters)", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> parameters</div>" } ], "returns" : { "type" : "item()*", "description" : "protected resources parsed as parameter elements, or an error if http response status is not 200 OK <pre xmlns:xqdoc=\"http://www.xqdoc.org/1.0\" class=\"ace-static\" ace-mode=\"xquery\"> let $tokens := oauth:parameters(\"oauth_token\", \"#\") let $tokens := oauth:add-parameter($tokens, \"oauth_token_secret\", \"#\") let $service-provider := oauth:service-provider(...) let $request := validate { <request xmlns=\"http://expath.org/ns/http-client\" href=\"http://twitter.com/account/verify_credentials.xml\" method=\"GET\"/> } return oauth:protected-resource($request, $service-provider, $tokens) </pre>" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "request-token", "qname" : "oauth:request-token", "signature" : "($service-provider as schema-element(sp:service-provider)) as schema-element(p:parameters)", "description" : " This function allows the client to obtain a set of temporary credentials from the service provider by making an authenticated HTTP request to the Temporary Credential Request endpoint.\n This function is provided for convenience for <a xmlns:xqdoc=\"http://www.xqdoc.org/1.0\" href=\"#request-token-2\">request-token#2</a>.\n Invoking this function is equivalent to:\n <pre xmlns:xqdoc=\"http://www.xqdoc.org/1.0\" class=\"ace-static\" ace-mode=\"xquery\">\n oauth:request-token($service-provider, ())\n </pre>\n", "summary" : "<p> This function allows the client to obtain a set of temporary credentials from the service provider by making an authenticated HTTP request to the Temporary Credential Request endpoint.</p>", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "service-provider", "type" : "schema-element(sp:service-provider)", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> Information about the service provider</div>" } ], "returns" : { "type" : "schema-element(p:parameters)", "description" : "temporary credentials correctly parsed as parameter elements, or an error if http response status is not 200 OK" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 2, "name" : "request-token", "qname" : "oauth:request-token", "signature" : "($service-provider as schema-element(sp:service-provider), $parameters as schema-element(p:parameters)?) as schema-element(p:parameters)", "description" : " This function allows the client to obtain a set of temporary credentials from the service provider by making an authenticated HTTP request to the Temporary Credential Request endpoint.\n This function is provided for convenience.\n", "summary" : "<p> This function allows the client to obtain a set of temporary credentials from the service provider by making an authenticated HTTP request to the Temporary Credential Request endpoint.</p>", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "service-provider", "type" : "schema-element(sp:service-provider)", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> Information about the service provider</div>" }, { "name" : "parameters", "type" : "schema-element(p:parameters)", "occurrence" : "?", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> Additionnal parameters to the request</div>" } ], "returns" : { "type" : "schema-element(p:parameters)", "description" : "temporary credentials correctly parsed as parameter elements, or an error if http response status is not 200 OK <pre xmlns:xqdoc=\"http://www.xqdoc.org/1.0\" class=\"ace-static\" ace-mode=\"xquery\"> let $twitter-config := oauth:service-provider(...) let $additional-parameter := oauth:parameters(\"foo\", \"bar\") let $tokens := oauth:request-token($twitter-config, $additional-parameter) let $token := oauth:parameter($tokens, \"oauth_token\") let $token-secret := oauth:parameter($tokens, \"oauth_token_secret\") ... </pre>" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 10, "name" : "service-provider", "qname" : "oauth:service-provider", "signature" : "($consumer-key as xs:string, $consumer-secret as xs:string, $signature-method as xs:string, $realm as xs:string, $authorize-url as xs:string, $request-token-method as xs:string, $request-token-url as xs:string, $request-token-callback-url as xs:string, $access-token-method as xs:string, $access-token-url as xs:string) as schema-element(sp:service-provider)", "description" : " Utility function to build a service provider object.\n This object contains the information required by the\n OAuth client to interact with an OAuth service provider.\n For instance the following expression:\n <pre xmlns:xqdoc=\"http://www.xqdoc.org/1.0\" class=\"ace-static\" ace-mode=\"xquery\">\n let $consumer-key     := \"#\"\n let $consumer-secret  := \"#\"\n let $signature-method := \"HMAC-SHA1\"\n let $realm            := \"twitter.com\"\n let $authorize-url    := \"http://api.twitter.com/oauth/authorize\"\n let $request-token-method := \"POST\"\n let $request-token-url := \"https://twitter.com/oauth/request_token\"\n let $request-token-callback-url := \"https://twitter.com/oauth/request_token\"\n let $access-token-method := \"POST\"\n let $access-token-url := \"https://api.twitter.com/oauth/access_token\"\n return oauth:service-provider(\n   $consumer-key, $consumer-secret, $signature-method,\n   $realm, $authorize-url, $request-token-method,\n   $request-token-url, $request-token-callback-url,\n   $access-token-method, $access-token-url\n )\n </pre>\n Will return the following XML schema instance:\n <pre xmlns:xqdoc=\"http://www.xqdoc.org/1.0\" class=\"ace-static\" ace-mode=\"xml\">\n <sp:service-provider xmlns:sp=\"http://www.zorba-xquery.com/schemas/oauth/service-provider\" consumer-key=\"#\" consumer-secret=\"#\" signature-method=\"HMAC-SHA1\" realm=\"twitter.com\" authorize-url=\"http://api.twitter.com/oauth/authorize\">\n   <sp:request-token method=\"POST\" href=\"https://twitter.com/oauth/request_token\" callback-url=\"https://twitter.com/oauth/request_token\"/>\n   <sp:access-token method=\"POST\" href=\"https://api.twitter.com/oauth/access_token\"/>\n </sp:service-provider>\n </pre>\n", "summary" : "<p> Utility function to build a service provider object.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "consumer-key", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> Client Identifier, also known as the consumer-key</div>" }, { "name" : "consumer-secret", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> Client Shared-Secret, also known as the consumer-secret</div>" }, { "name" : "signature-method", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> Method with which the signing key is signed (typically HMAC-SHA1)</div>" }, { "name" : "realm", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> Realm that defines the protection space</div>" }, { "name" : "authorize-url", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> authorize-url</div>" }, { "name" : "request-token-method", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> request-token-method</div>" }, { "name" : "request-token-url", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> request-token-url</div>" }, { "name" : "request-token-callback-url", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> request-token-callback-url</div>" }, { "name" : "access-token-method", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> access-token-method</div>" }, { "name" : "access-token-url", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> access-token-url</div>" } ], "returns" : { "type" : "schema-element(sp:service-provider)", "description" : "instance of the OAuth service provider XML schema." }, "errors" : [  ] } ], "variables" : [  ] }, "http://www.28msec.com/modules/http/util/multipart" : { "ns" : "http://www.28msec.com/modules/http/util/multipart", "description" : " This module provides utility functions to help dealing with the\n content of multipart requests.\n In their implementation, all the functions use the multipart\n related functions of the request module (e.g. parts() or text-part()).\n", "sees" : [  ], "authors" : [ "<xqdoc:author xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">28msec</xqdoc:author>" ], "version" : null, "encoding" : "utf-8", "namespaces" : [ { "uri" : "http://www.28msec.com/modules/http/util/multipart", "prefix" : "multipart" }, { "uri" : "http://www.28msec.com/modules/http/request", "prefix" : "req" }, { "uri" : "http://zorba.io/options/versioning", "prefix" : "ver" } ], "functions" : [ { "isDocumented" : true, "arity" : 1, "name" : "binary-content", "qname" : "multipart:binary-content", "signature" : "($name as xs:string) as xs:base64Binary*", "description" : " Returns the contents of the parts/files in the multipart requests with\n the given part name as xs:base64Binary.\n", "summary" : "<p> Returns the contents of the parts/files in the multipart requests with\n the given part name as xs:base64Binary.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "name", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the name of the requested part.</div>" } ], "returns" : { "type" : "xs:base64Binary*", "description" : "the said content or the empty sequence if no part with the given name exists." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">request:non-multipart if the current request does not contain multipart content</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">request:invalid-multipart if the multipart content is invalid (e.g. the boundary is missing)</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 1, "name" : "binary-file", "qname" : "multipart:binary-file", "signature" : "($filename as xs:string) as xs:base64Binary*", "description" : " Returns the contents of the part/file in the multipart requests with\n the given filename as xs:base64Binary.\n Please note that the file is identified by the filename (and not the\n name of the part) that is contained in the Content-Disposition header\n of the part.\n", "summary" : "<p> Returns the contents of the part/file in the multipart requests with\n the given filename as xs:base64Binary.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "filename", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the name of the requested file.</div>" } ], "returns" : { "type" : "xs:base64Binary*", "description" : "the said content or the empty sequence if no part with the given filename exists." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">request:non-multipart if the current request does not contain multipart content</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">request:invalid-multipart if the multipart content is invalid (e.g. the boundary is missing)</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 1, "name" : "content-length", "qname" : "multipart:content-length", "signature" : "($name as xs:string) as xs:integer*", "description" : " Returns the content length of the parts/files in the multipart requests with\n the given part name as xs:integer.\n", "summary" : "<p> Returns the content length of the parts/files in the multipart requests with\n the given part name as xs:integer.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "name", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the name of the requested part.</div>" } ], "returns" : { "type" : "xs:integer*", "description" : "The content-length in bytes of the parts/files or the empty sequence if the part does not exist" }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">request:non-multipart if the current request does not contain multipart content</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">request:invalid-multipart if the multipart content is invalid (e.g. the boundary is missing)</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 1, "name" : "content-type", "qname" : "multipart:content-type", "signature" : "($name as xs:string) as xs:string*", "description" : " Returns the content type of the parts/files in the multipart requests with\n the given part name as xs:string.\n", "summary" : "<p> Returns the content type of the parts/files in the multipart requests with\n the given part name as xs:string.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "name", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the name of the requested part.</div>" } ], "returns" : { "type" : "xs:string*", "description" : "The content-types of the parts/files or the empty sequence if the part does not exist" }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">request:non-multipart if the current request does not contain multipart content</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">request:invalid-multipart if the multipart content is invalid (e.g. the boundary is missing)</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 1, "name" : "filename", "qname" : "multipart:filename", "signature" : "($name as xs:string) as xs:string*", "description" : " Returns the filenames of the parts/files in the multipart requests with\n the given part name as xs:string.\n", "summary" : "<p> Returns the filenames of the parts/files in the multipart requests with\n the given part name as xs:string.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "name", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the name of the requested part.</div>" } ], "returns" : { "type" : "xs:string*", "description" : "The filename of the parts/files or the empty sequence if the part does not exist" }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">request:non-multipart if the current request does not contain multipart content</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">request:invalid-multipart if the multipart content is invalid (e.g. the boundary is missing)</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 0, "name" : "filenames", "qname" : "multipart:filenames", "signature" : "() as xs:string*", "description" : " Returns the filenames of all the parts contained in the multipart\n content of the given request.\n", "summary" : "<p> Returns the filenames of all the parts contained in the multipart\n content of the given request.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [  ], "returns" : { "type" : "xs:string*", "description" : "the said filenames" }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">request:non-multipart if the current request does not contain multipart content</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">request:invalid-multipart if the multipart content is invalid (e.g. the boundary is missing)</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 0, "name" : "names", "qname" : "multipart:names", "signature" : "() as xs:string*", "description" : " Returns the names of all the parts contained in the multipart\n content of the given request.\n", "summary" : "<p> Returns the names of all the parts contained in the multipart\n content of the given request.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [  ], "returns" : { "type" : "xs:string*", "description" : "the said names" }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">request:non-multipart if the current request does not contain multipart content</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">request:invalid-multipart if the multipart content is invalid (e.g. the boundary is missing)</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 1, "name" : "part", "qname" : "multipart:part", "signature" : "($name as xs:string) as element(req:part)*", "description" : " Returns the <tt xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">req:part</tt> element among all the parts contained\n in the multipart requests with the given name.\n", "summary" : "<p> Returns the  req:part  element among all the parts contained\n in the multipart requests with the given name.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "name", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the name of the requested part.</div>" } ], "returns" : { "type" : "element(req:part)*", "description" : "the said part elements or the empty sequence if no part with the given name exists." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">request:non-multipart if the current request does not contain multipart content</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">request:invalid-multipart if the multipart content is invalid (e.g. the boundary is missing)</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 1, "name" : "text-content", "qname" : "multipart:text-content", "signature" : "($name as xs:string) as xs:string*", "description" : " Returns the contents of the parts/files in the multipart requests with\n the given part name as a string.\n It is assumed that the character set of the file is UTF-8.\n", "summary" : "<p> Returns the contents of the parts/files in the multipart requests with\n the given part name as a string.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "name", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the name of the requested part.</div>" } ], "returns" : { "type" : "xs:string*", "description" : "the said content or the empty sequence if no part with the given filename exists." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">request:no-text-content if the value of the part cannot be treated as text</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">request:non-multipart if the current request does not contain multipart content</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">request:invalid-multipart if the multipart content is invalid (e.g. the boundary is missing)</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 1, "name" : "text-file", "qname" : "multipart:text-file", "signature" : "($filename as xs:string) as xs:string*", "description" : " Returns the contents of the part/file in the multipart requests with\n the given filename as a string.\n It is assumed that the character set of the file is UTF-8.\n Please note that the file is identified by the filename (and not the\n name of the part) that is contained in the Content-Disposition header\n of the part.\n", "summary" : "<p> Returns the contents of the part/file in the multipart requests with\n the given filename as a string.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "filename", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the name of the requested file.</div>" } ], "returns" : { "type" : "xs:string*", "description" : "the said content or the empty sequence if no part with the given filename exists." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">request:no-text-content if the value of the part cannot be treated as text</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">request:non-multipart if the current request does not contain multipart content</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">request:invalid-multipart if the multipart content is invalid (e.g. the boundary is missing)</xqdoc:error>" ] } ], "variables" : [  ] }, "http://zorba.io/modules/excel/statistical-zorba" : { "ns" : "http://zorba.io/modules/excel/statistical-zorba", "description" : " This module implements some Excel 2003 statistical functions\n that cannot be implemented with standard XQuery functions.\n It uses Zorba specific functions.\n", "sees" : [ "<xqdoc:see xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">http://office.microsoft.com/en-us/excel/CH062528311033.aspx</xqdoc:see>" ], "authors" : [ "<xqdoc:author xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Daniel Turcanu</xqdoc:author>" ], "version" : null, "encoding" : "utf-8", "namespaces" : [ { "uri" : "http://zorba.io/modules/excel/statistical-zorba", "prefix" : "excel" }, { "uri" : "http://zorba.io/modules/excel/errors", "prefix" : "excel-err" }, { "uri" : "http://zorba.io/modules/excel/math", "prefix" : "excel-math" }, { "uri" : "http://zorba.io/modules/excel/statistical", "prefix" : "excel-statistical" }, { "uri" : "http://www.w3.org/2005/xpath-functions/math", "prefix" : "math" }, { "uri" : "http://zorba.io/options/versioning", "prefix" : "ver" } ], "functions" : [ { "isDocumented" : true, "arity" : 1, "name" : "stdev", "qname" : "excel:stdev", "signature" : "($numbers as xs:anyAtomicType+) as xs:anyAtomicType", "description" : " Estimates standard deviation based on a sample.\n The standard deviation is a measure of how widely values are dispersed\n   from the average value (the mean).\n It is computed with formula:\n sqrt( sum((x-average_x)^2) / (n-1) )    = sqrt ( VAR(numbers) )\n", "summary" : "<p> Estimates standard deviation based on a sample.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "numbers", "type" : "xs:anyAtomicType", "occurrence" : "+", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequence of numbers or values castable to numeric The sequence can be of any length, from 1 up.</div>" } ], "returns" : { "type" : "xs:anyAtomicType", "description" : "the standard deviation, as numeric type" }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">excel-err:Value if the parameters cannot be casted to numeric type</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 1, "name" : "stdeva", "qname" : "excel:stdeva", "signature" : "($numbers as xs:anyAtomicType+) as xs:anyAtomicType", "description" : " Estimates standard deviation based on a sample.\n The standard deviation is a measure of how widely values are dispersed\n   from the average value (the mean).\n It is computed with formula:\n sqrt( sum((x-average_x)^2) / (n-1) )    = sqrt ( VARA(numbers) )\n", "summary" : "<p> Estimates standard deviation based on a sample.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "numbers", "type" : "xs:anyAtomicType", "occurrence" : "+", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequence of numbers or values castable to numeric The sequence can be of any length, from 1 up.</div>" } ], "returns" : { "type" : "xs:anyAtomicType", "description" : "the standard deviation, as numeric type" }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">excel-err:Value if the parameters cannot be casted to numeric type</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 1, "name" : "stdevp", "qname" : "excel:stdevp", "signature" : "($numbers as xs:anyAtomicType+) as xs:anyAtomicType", "description" : " Calculates standard deviation based on the entire population given as arguments.\n The standard deviation is a measure of how widely values are dispersed from\n   the average value (the mean).\n It is computed with formula:\n sqrt( sum((x-average_x)^2) / n )    = sqrt ( VARP(numbers) )\n", "summary" : "<p> Calculates standard deviation based on the entire population given as arguments.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "numbers", "type" : "xs:anyAtomicType", "occurrence" : "+", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequence of numbers or values castable to numeric The sequence can be of any length, from 1 up.</div>" } ], "returns" : { "type" : "xs:anyAtomicType", "description" : "the standard deviation, as numeric type" }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">excel-err:Value if the parameters cannot be casted to numeric type</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 1, "name" : "stdevpa", "qname" : "excel:stdevpa", "signature" : "($numbers as xs:anyAtomicType+) as xs:anyAtomicType", "description" : " Calculates standard deviation based on the entire population given as arguments.\n The standard deviation is a measure of how widely values are dispersed from\n   the average value (the mean).\n It is computed with formula:\n sqrt( sum((x-average_x)^2) / n )    = sqrt ( VARPA(numbers) )\n", "summary" : "<p> Calculates standard deviation based on the entire population given as arguments.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "numbers", "type" : "xs:anyAtomicType", "occurrence" : "+", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequence of numbers or values castable to numeric The sequence can be of any length, from 1 up.</div>" } ], "returns" : { "type" : "xs:anyAtomicType", "description" : "the standard deviation, as numeric type" }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">excel-err:Value if the parameters cannot be casted to numeric type</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 2, "name" : "subtotal", "qname" : "excel:subtotal", "signature" : "($function_num as xs:integer, $numbers as xs:anyAtomicType*) as xs:anyAtomicType", "description" : " Moved from math module.\n Returns a subtotal in a sequence of numbers.\n The function applied is given by $function_num.\n", "summary" : "<p> Moved from math module.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "function_num", "type" : "xs:integer", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> <dl>defines the function to be applied on sequence values. The possible values are: <dt>1 or 101</dt> <dd> AVERAGE</dd> <dt>2 or 102</dt> <dd> COUNT</dd> <dt>3 or 103</dt> <dd> COUNTA</dd> <dt>4 or 104</dt> <dd> MAX</dd> <dt>5 or 105</dt> <dd> MIN</dd> <dt>6 or 106</dt> <dd> PRODUCT</dd> <dt>7 or 107</dt> <dd> STDEV</dd> <dt>8 or 108</dt> <dd> STDEVP</dd> <dt>9 or 109</dt> <dd> SUM</dd> <dt>10 or 110</dt> <dd> VAR</dd> <dt>11 or 111</dt> <dd> VARP</dd></dl> In this implementation there is no difference between x and 10x.<p/></div>" }, { "name" : "numbers", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequence of numbers or values castable to numeric. The sequence can be of any length.</div>" } ], "returns" : { "type" : "xs:anyAtomicType", "description" : "The function result, as numeric type" }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">depends on the function called</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">excel-err:Num if $function_num is not a value between 1 .. 11 or 101 .. 111</xqdoc:error>" ] } ], "variables" : [  ] }, "http://www.28msec.com/modules/debug" : { "ns" : "http://www.28msec.com/modules/debug", "description" : " This module contains internal debug functions.\n", "sees" : [  ], "authors" : [ "<xqdoc:author xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Federico Cavalieri</xqdoc:author>" ], "version" : null, "encoding" : "utf-8", "namespaces" : [ { "uri" : "http://zorba.io/annotations", "prefix" : "an" }, { "uri" : "http://www.28msec.com/modules/debug", "prefix" : "debug" }, { "uri" : "http://zorba.io/options/versioning", "prefix" : "ver" } ], "functions" : [ { "isDocumented" : true, "arity" : 0, "name" : "non-seeakable-streamable-string", "qname" : "debug:non-seeakable-streamable-string", "signature" : "() as string external", "description" : " Returns a non seekable streamable string\n", "summary" : "<p> Returns a non seekable streamable string\n</p>", "annotation_str" : " %an:deterministic", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "deterministic", "value" : "" } ], "updating" : false, "parameters" : [  ], "returns" : { "type" : "string", "description" : "a non seekable streamable string" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "test-01-cache", "qname" : "debug:test-01-cache", "signature" : "($arg as xs:integer) as item()* external", "description" : " Equivalent to:\n declare %an:cache function local:test-01-cache($x as xs:integer)\n {\n  if ($x &lt; 10)\n  then $x * $x\n  else if ($x &lt; 20)\n  then ()\n  else ($x, \" == \", $x * $x)\n };\n", "summary" : "<p> Equivalent to:\n declare %an:cache function local:test-01-cache($x as xs:integer)\n {\n  if ($x &lt; 10)\n  then $x * $x\n  else if ($x &lt; 20)\n  then ()\n  else ($x, \" == \", $x * $x)\n };\n</p>", "annotation_str" : " %an:cache", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "cache", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "arg", "type" : "xs:integer", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> as described above</div>" } ], "returns" : { "type" : "item()*", "description" : "as described above" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "test-01-sd", "qname" : "debug:test-01-sd", "signature" : "($arg as xs:integer) as item()* external", "description" : " Equivalent to:\n declare %an:strictlydeterministic function local:test-01-sd($x as xs:integer)\n {\n  if ($x &lt; 10)\n  then $x * $x\n  else if ($x &lt; 20)\n  then ()\n  else ($x, \" == \", $x * $x)\n };\n", "summary" : "<p> Equivalent to:\n declare %an:strictlydeterministic function local:test-01-sd($x as xs:integer)\n {\n  if ($x &lt; 10)\n  then $x * $x\n  else if ($x &lt; 20)\n  then ()\n  else ($x, \" == \", $x * $x)\n };\n</p>", "annotation_str" : " %an:strictlydeterministic", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "strictlydeterministic", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "arg", "type" : "xs:integer", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> as described above</div>" } ], "returns" : { "type" : "item()*", "description" : "as described above" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "test-02-cache", "qname" : "debug:test-02-cache", "signature" : "($arg) as item()* external", "description" : " Equivalent to:\n declare %an:cache function local:test-02-cache($arg)\n {\n   serialize($arg),\n   uuid:uuid()\n };\n", "summary" : "<p> Equivalent to:\n declare %an:cache function local:test-02-cache($arg)\n {\n   serialize($arg),\n   uuid:uuid()\n };\n</p>", "annotation_str" : " %an:cache", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "cache", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "arg", "type" : null, "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> as described above</div>" } ], "returns" : { "type" : "item()*", "description" : "as described above" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "test-02-sd", "qname" : "debug:test-02-sd", "signature" : "($arg) as item()* external", "description" : " Equivalent to:\n declare %an:strictlydeterministic function local:test-02-sd($arg)\n {\n   serialize($arg),\n   uuid:uuid()\n };\n", "summary" : "<p> Equivalent to:\n declare %an:strictlydeterministic function local:test-02-sd($arg)\n {\n   serialize($arg),\n   uuid:uuid()\n };\n</p>", "annotation_str" : " %an:strictlydeterministic", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "strictlydeterministic", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "arg", "type" : null, "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> as described above</div>" } ], "returns" : { "type" : "item()*", "description" : "as described above" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 0, "name" : "test-03-cache", "qname" : "debug:test-03-cache", "signature" : "() as item()* external", "description" : " Equivalent to:\n declare %an:cache function local:test-03-cache()\n {\n   { \"a\" : 1 },\n   [1],\n   &lt;a/&gt;,\n   1\n };\n", "summary" : "<p> Equivalent to:\n declare %an:cache function local:test-03-cache()\n {\n   { \"a\" : 1 },\n   [1],\n   &lt;a/&gt;,\n   1\n };\n</p>", "annotation_str" : " %an:cache", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "cache", "value" : "" } ], "updating" : false, "parameters" : [  ], "returns" : { "type" : "item()*", "description" : "as described above" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 0, "name" : "test-03-sd", "qname" : "debug:test-03-sd", "signature" : "() as item()* external", "description" : " Equivalent to:\n declare %an:strictlydeterministic function local:test-03-sd()\n {\n   { \"a\" : 1 },\n   [1],\n   &lt;a/&gt;,\n   1\n };\n", "summary" : "<p> Equivalent to:\n declare %an:strictlydeterministic function local:test-03-sd()\n {\n   { \"a\" : 1 },\n   [1],\n   &lt;a/&gt;,\n   1\n };\n</p>", "annotation_str" : " %an:strictlydeterministic", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "strictlydeterministic", "value" : "" } ], "updating" : false, "parameters" : [  ], "returns" : { "type" : "item()*", "description" : "as described above" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 2, "name" : "test-04-cache", "qname" : "debug:test-04-cache", "signature" : "($x, $y) as item()* external", "description" : " Equivalent to:\n declare %an:cache function local:test-04-cache($x, $y)\n {\n   debug:non-seeakable-streamable-string(),\n   {\"key\" : debug:non-seeakable-streamable-string()},\n   [debug:non-seeakable-streamable-string()],\n   &lt;a&gt;{debug:non-seeakable-streamable-string()}&lt;/a&gt;\n   serialize($x),\n   serialize($y)\n };\n", "summary" : "<p> Equivalent to:\n declare %an:cache function local:test-04-cache($x, $y)\n {\n   debug:non-seeakable-streamable-string(),\n   {\"key\" : debug:non-seeakable-streamable-string()},\n   [debug:non-seeakable-streamable-string()],\n   &lt;a&gt;{debug:non-seeakable-streamable-string()}&lt;/a&gt;\n   serialize($x),\n   serialize($y)\n };\n</p>", "annotation_str" : " %an:cache", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "cache", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "x", "type" : null, "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> as described above</div>" }, { "name" : "y", "type" : null, "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> as described above</div>" } ], "returns" : { "type" : "item()*", "description" : "as described above" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 2, "name" : "test-04-sd", "qname" : "debug:test-04-sd", "signature" : "($x, $y) as item()* external", "description" : " Equivalent to:\n declare %an:strictlydeterministic function local:test-04-sd($x, $y)\n {\n   debug:non-seeakable-streamable-string(),\n   {\"key\" : debug:non-seeakable-streamable-string()},\n   [debug:non-seeakable-streamable-string()],\n   &lt;a&gt;{debug:non-seeakable-streamable-string()}&lt;/a&gt;\n   serialize($x),\n   serialize($y)\n };\n", "summary" : "<p> Equivalent to:\n declare %an:strictlydeterministic function local:test-04-sd($x, $y)\n {\n   debug:non-seeakable-streamable-string(),\n   {\"key\" : debug:non-seeakable-streamable-string()},\n   [debug:non-seeakable-streamable-string()],\n   &lt;a&gt;{debug:non-seeakable-streamable-string()}&lt;/a&gt;\n   serialize($x),\n   serialize($y)\n };\n</p>", "annotation_str" : " %an:strictlydeterministic", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "strictlydeterministic", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "x", "type" : null, "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> as described above</div>" }, { "name" : "y", "type" : null, "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> as described above</div>" } ], "returns" : { "type" : "item()*", "description" : "as described above" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 2, "name" : "test-05-cache", "qname" : "debug:test-05-cache", "signature" : "($x as xs:boolean, $y) as item()* external", "description" : " Equivalent to:\n declare %an:cache function local:test-05-cache($x as xs:boolean, $y)\n {\n   if ($x)\n   then $y\n   else $x\n };\n", "summary" : "<p> Equivalent to:\n declare %an:cache function local:test-05-cache($x as xs:boolean, $y)\n {\n   if ($x)\n   then $y\n   else $x\n };\n</p>", "annotation_str" : " %an:cache", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "cache", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "x", "type" : "xs:boolean", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> as described above</div>" }, { "name" : "y", "type" : null, "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> as described above</div>" } ], "returns" : { "type" : "item()*", "description" : "as described above" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 2, "name" : "test-05-sd", "qname" : "debug:test-05-sd", "signature" : "($x as xs:boolean, $y) as item()* external", "description" : " Equivalent to:\n declare %an:strictlydeterministic function local:test-05-sd($x as xs:boolean, $y)\n {\n   if ($x)\n   then $y\n   else $x\n };\n", "summary" : "<p> Equivalent to:\n declare %an:strictlydeterministic function local:test-05-sd($x as xs:boolean, $y)\n {\n   if ($x)\n   then $y\n   else $x\n };\n</p>", "annotation_str" : " %an:strictlydeterministic", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "strictlydeterministic", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "x", "type" : "xs:boolean", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> as described above</div>" }, { "name" : "y", "type" : null, "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> as described above</div>" } ], "returns" : { "type" : "item()*", "description" : "as described above" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 0, "name" : "uuid", "qname" : "debug:uuid", "signature" : "() as string external", "description" : " This function is not deterministic despite the annotation\n", "summary" : "<p> This function is not deterministic despite the annotation\n</p>", "annotation_str" : " %an:deterministic", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "deterministic", "value" : "" } ], "updating" : false, "parameters" : [  ], "returns" : { "type" : "string", "description" : "a uuid" }, "errors" : [  ] } ], "variables" : [  ] }, "http://zorba.io/modules/json-xml" : { "ns" : "http://zorba.io/modules/json-xml", "description" : " Using this module, you can parse JSON data into XML, manipulate it like any\n other XML data using XQuery, and serialize the result back as JSON.<p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n There are many ways to represent JSON data in XML, some loss-less (\"round\n tripable\") and some lossy (\"one way\").  Loss-less representations preserve\n the JSON data types <i xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">boolean</i>, <i xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">number</i>, and <i xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">null</i>; lossy\n representations convert all data to strings.\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n For a loss-less representation, this module implements that proposed by\n <a xmlns:xqdoc=\"http://www.xqdoc.org/1.0\" href=\"http://john.snelson.org.uk/parsing-json-into-xquery\">John Snelson</a>.\n For example:\n <pre xmlns:xqdoc=\"http://www.xqdoc.org/1.0\" class=\"ace-static\" ace-mode=\"java\">\n   {\n     \"firstName\" : \"John\",\n     \"lastName\" : \"Smith\",\n     \"address\" : {\n       \"streetAddress\" : \"21 2nd Street\",\n       \"city\" : \"New York\",\n       \"state\" : \"NY\",\n       \"postalCode\" : 10021\n     },\n     \"phoneNumbers\" : [ \"212 732-1234\", \"646 123-4567\" ]\n   }\n </pre>\n would be represented as:\n <pre xmlns:xqdoc=\"http://www.xqdoc.org/1.0\" class=\"ace-static\" ace-mode=\"xquery\">\n   &lt;json type=\"object\"&gt;\n     &lt;pair name=\"firstName\" type=\"string\"&gt;John&lt;/pair&gt;\n     &lt;pair name=\"lastName\" type=\"string\"&gt;Smith&lt;/pair&gt;\n     &lt;pair name=\"address\" type=\"object\"&gt;\n       &lt;pair name=\"streetAddress\" type=\"string\"&gt;21 2nd Street&lt;/pair&gt;\n       &lt;pair name=\"city\" type=\"string\"&gt;New York&lt;/pair&gt;\n       &lt;pair name=\"state\" type=\"string\"&gt;NY&lt;/pair&gt;\n       &lt;pair name=\"postalCode\" type=\"number\"&gt;10021&lt;/pair&gt;\n     &lt;/pair&gt;\n     &lt;pair name=\"phoneNumbers\" type=\"array\"&gt;\n       &lt;item type=\"string\"&gt;212 732-1234&lt;/item&gt;\n       &lt;item type=\"string\"&gt;646 123-4567&lt;/item&gt;\n     &lt;/pair&gt;\n   &lt;/json&gt;\n </pre>\n For a lossy representation, this module implements\n <a xmlns:xqdoc=\"http://www.xqdoc.org/1.0\" href=\"http://jsonml.org/\">JsonML</a> (the array form).\n For example:\n <pre xmlns:xqdoc=\"http://www.xqdoc.org/1.0\" class=\"ace-static\" ace-mode=\"java\">\n   [ \"person\",\n     { \"created\" : \"2006-11-11T19:23\",\n       \"modified\" : \"2006-12-31T23:59\" },\n     [ \"firstName\", \"Robert\" ],\n     [ \"lastName\", \"Smith\" ],\n     [ \"address\",\n       { \"type\" : \"home\" },\n       [ \"street\", \"12345 Sixth Ave\" ],\n       [ \"city\", \"Anytown\" ],\n       [ \"state\", \"CA\" ],\n       [ \"postalCode\", \"98765-4321\" ]\n     ]\n   ]\n </pre>\n would be represented as:\n <pre xmlns:xqdoc=\"http://www.xqdoc.org/1.0\" class=\"ace-static\" ace-mode=\"xquery\">\n   &lt;person created=\"2006-11-11T19:23\" modified=\"2006-12-31T23:59\"&gt;\n     &lt;firstName&gt;Robert&lt;/firstName&gt;\n     &lt;lastName&gt;Smith&lt;/lastName&gt;\n     &lt;address type=\"home\"&gt;\n       &lt;street&gt;12345 Sixth Ave&lt;/street&gt;\n       &lt;city&gt;Anytown&lt;/city&gt;\n       &lt;state&gt;CA&lt;/state&gt;\n       &lt;postalCode&gt;98765-4321&lt;/postalCode&gt;\n     &lt;/address&gt;\n   &lt;/person&gt;\n </pre>\n", "sees" : [  ], "authors" : [ "<xqdoc:author xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Paul J. Lucas</xqdoc:author>" ], "version" : null, "encoding" : "utf-8", "namespaces" : [ { "uri" : "http://www.w3.org/2005/xqt-errors", "prefix" : "err" }, { "uri" : "http://zorba.io/modules/json-xml", "prefix" : "jx" }, { "uri" : "http://zorba.io/options/versioning", "prefix" : "ver" }, { "uri" : "http://zorba.io/errors", "prefix" : "zerr" } ], "functions" : [ { "isDocumented" : true, "arity" : 1, "name" : "json-to-xml", "qname" : "jx:json-to-xml", "signature" : "($json as json-item()?) as element(*,xs:untyped)?", "description" : " Converts JSON data into an XDM instance using the Snelson representation\n described above.<p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n", "summary" : "<p> Converts JSON data into an XDM instance using the Snelson representation\n described above.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "json", "type" : "json-item()", "occurrence" : "?", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The JSON data.</div>" } ], "returns" : { "type" : "element(*,xs:untyped)?", "description" : "said XDM instance." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 2, "name" : "json-to-xml", "qname" : "jx:json-to-xml", "signature" : "($json as json-item()?, $options as object()) as element(*,xs:untyped)?", "description" : " Converts JSON data into an XDM instance using one of the representations\n described above.<p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n", "summary" : "<p> Converts JSON data into an XDM instance using one of the representations\n described above.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "json", "type" : "json-item()", "occurrence" : "?", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The JSON data.</div>" }, { "name" : "options", "type" : "object()", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The JSON conversion options, for example: <pre> { \"json-format\" : \"JsonML-array\" } </pre></div>" } ], "returns" : { "type" : "element(*,xs:untyped)?", "description" : "said XDM instance." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "xml-to-json", "qname" : "jx:xml-to-json", "signature" : "($xml as item()*) as json-item()*", "description" : " Converts XML data into a JSON item using the Snelson representation\n described above.<p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n", "summary" : "<p> Converts XML data into a JSON item using the Snelson representation\n described above.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "xml", "type" : "item()", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The XML data to convert.</div>" } ], "returns" : { "type" : "json-item()*", "description" : "said JSON items." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">zerr:ZJSE0001 if $xml is not a document or element node.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">zerr:ZJSE0002 if $xml contains an element that is missing a required attribute.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">zerr:ZJSE0003 if $xml contains an attribute having an illegal value.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">zerr:ZJSE0004 if $xml contains an illegal element. type.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">zerr:ZJSE0007 if $xml contains an element that is missing a required value.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">zerr:ZJSE0008 if $xml contains an illegal value for a JSON type.</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 2, "name" : "xml-to-json", "qname" : "jx:xml-to-json", "signature" : "($xml as item()*, $options as object()) as json-item()*", "description" : " Converts XML data into a JSON item using one of the respresentations\n described above.<p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n", "summary" : "<p> Converts XML data into a JSON item using one of the respresentations\n described above.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "xml", "type" : "item()", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The XML data to convert.</div>" }, { "name" : "options", "type" : "object()", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The conversion options, for example: <pre> { \"json-format\" : \"JsonML-array\" } </pre></div>" } ], "returns" : { "type" : "json-item()*", "description" : "said JSON items." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">zerr:ZJSE0001 if $xml is not a document or element node.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">zerr:ZJSE0002 if $xml contains an element that is missing a required attribute.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">zerr:ZJSE0003 if $xml contains an attribute having an illegal value.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">zerr:ZJSE0004 if $xml contains an illegal element. type.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">zerr:ZJSE0007 if $xml contains an element that is missing a required value.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">zerr:ZJSE0008 if $xml contains an illegal value for a JSON type.</xqdoc:error>" ] } ], "variables" : [  ] }, "http://www.zorba-xquery.com/modules/image/manipulation" : { "ns" : "http://www.zorba-xquery.com/modules/image/manipulation", "description" : " This module provides functions to handle image manipulations like resizing, zooming,\n special effects etc.\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">The errors raised by functions of this module have the namespace\n <tt>http://www.zorba-xquery.com/modules/image/error</tt> (associated with prefix ierr).</p>\n", "sees" : [  ], "authors" : [ "<xqdoc:author xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Daniel Thomas</xqdoc:author>" ], "version" : null, "encoding" : "utf-8", "namespaces" : [ { "uri" : "http://www.w3.org/2005/xqt-errors", "prefix" : "err" }, { "uri" : "http://www.zorba-xquery.com/modules/image/error", "prefix" : "ierr" }, { "uri" : "http://www.zorba-xquery.com/modules/image/image", "prefix" : "image" }, { "uri" : "http://www.zorba-xquery.com/modules/image/manipulation", "prefix" : "man" }, { "uri" : "http://zorba.io/options/versioning", "prefix" : "ver" } ], "functions" : [ { "isDocumented" : true, "arity" : 2, "name" : "add-noise", "qname" : "man:add-noise", "signature" : "($image as xs:base64Binary, $noise-type as xs:string) as xs:base64Binary", "description" : " Add noise to an image.\n Allowed noise types are:\n <ul xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">\n  <li>UniformNoise</li>\n  <li>GaussianNoise</li>\n  <li>MultiplicativeGaussianNoise</li>\n  <li>ImpulseNoise</li>\n  <li>LaplaceianNoise</li>\n  <li>PoissonNoise</li>\n </ul>\n", "summary" : "<p> Add noise to an image.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "image", "type" : "xs:base64Binary", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the source image</div>" }, { "name" : "noise-type", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> specifies the type of noise to add</div>" } ], "returns" : { "type" : "xs:base64Binary", "description" : "A copy of $image with added noise" }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">ierr:IM001 passed image is invalid</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">err:FORG0001 unsupported noise type</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 3, "name" : "blur", "qname" : "man:blur", "signature" : "($image as xs:base64Binary, $radius as xs:int, $sigma as xs:int) as xs:base64Binary external", "description" : " Blur an image.\n", "summary" : "<p> Blur an image.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "image", "type" : "xs:base64Binary", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the source image</div>" }, { "name" : "radius", "type" : "xs:int", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> is the radius of the Gaussian in pixels.</div>" }, { "name" : "sigma", "type" : "xs:int", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> is the standard deviation of the Laplacian in pixels.</div>" } ], "returns" : { "type" : "xs:base64Binary", "description" : "A blured copy of $image" }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">ierr:IM001 passed image is invalid</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 3, "name" : "charcoal", "qname" : "man:charcoal", "signature" : "($image as xs:base64Binary, $radius as xs:double, $sigma as xs:double) as xs:base64Binary external", "description" : " Apply a charcoal effect to the image (looks like a charcoal sketch).\n", "summary" : "<p> Apply a charcoal effect to the image (looks like a charcoal sketch).</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "image", "type" : "xs:base64Binary", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the source image</div>" }, { "name" : "radius", "type" : "xs:double", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> radius of the Gaussian in pixels</div>" }, { "name" : "sigma", "type" : "xs:double", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> standard deviation of the Laplacian in pixels</div>" } ], "returns" : { "type" : "xs:base64Binary", "description" : "A charcoaled copy of $image" }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">ierr:IM001 passed image is invalid</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 3, "name" : "chop", "qname" : "man:chop", "signature" : "($image as xs:base64Binary, $upper-left-x as xs:unsignedInt, $upper-left-y as xs:unsignedInt) as xs:base64Binary external", "description" : " Copy a part of a source image as new image.\n The copied part is all right of $upper-left-x and below $upper-left-y.\n", "summary" : "<p> Copy a part of a source image as new image.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "image", "type" : "xs:base64Binary", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> source image</div>" }, { "name" : "upper-left-x", "type" : "xs:unsignedInt", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> x position of the upper left corner of the part to copy</div>" }, { "name" : "upper-left-y", "type" : "xs:unsignedInt", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> y position of the upper left corner of the part to copy</div>" } ], "returns" : { "type" : "xs:base64Binary", "description" : "A new image copied from a part of source image" }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">ierr:IM001 passed image is invalid</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 2, "name" : "contrast", "qname" : "man:contrast", "signature" : "($image as xs:base64Binary, $sharpen as xs:double) as xs:base64Binary external", "description" : " Contrast an image (enhances image intensity differences) by a given value.\n", "summary" : "<p> Contrast an image (enhances image intensity differences) by a given value.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "image", "type" : "xs:base64Binary", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the source image</div>" }, { "name" : "sharpen", "type" : "xs:double", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> defines how much the image is contrasted.</div>" } ], "returns" : { "type" : "xs:base64Binary", "description" : "A contrasted copy of $image" }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">ierr:IM001 passed image is invalid</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 3, "name" : "crop", "qname" : "man:crop", "signature" : "($image as xs:base64Binary, $lower-right-x as xs:unsignedInt, $lower-right-y as xs:unsignedInt) as xs:base64Binary external", "description" : " Copy a part of a source image as new image.\n The copied part is all left of $lower-right-x and above $lower-right-y.\n", "summary" : "<p> Copy a part of a source image as new image.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "image", "type" : "xs:base64Binary", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> source image</div>" }, { "name" : "lower-right-x", "type" : "xs:unsignedInt", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> x position of the lower right corner of the part to copy</div>" }, { "name" : "lower-right-y", "type" : "xs:unsignedInt", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> y position of the lower right corner of the part to copy</div>" } ], "returns" : { "type" : "xs:base64Binary", "description" : "A new image copied from a part of source image" }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">ierr:IM001 passed image is invalid</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 1, "name" : "despeckle", "qname" : "man:despeckle", "signature" : "($image as xs:base64Binary) as xs:base64Binary external", "description" : " Despeckle an image.\n", "summary" : "<p> Despeckle an image.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "image", "type" : "xs:base64Binary", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the source image</div>" } ], "returns" : { "type" : "xs:base64Binary", "description" : "A despeckled copy of $image" }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">ierr:IM001 passed image is invalid</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 2, "name" : "edge", "qname" : "man:edge", "signature" : "($image as xs:base64Binary, $radius as xs:unsignedInt) as xs:base64Binary external", "description" : " Highlight edges in an image.\n", "summary" : "<p> Highlight edges in an image.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "image", "type" : "xs:base64Binary", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the source image</div>" }, { "name" : "radius", "type" : "xs:unsignedInt", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> radius of the pixel neighborhood (0 for automatic selection)</div>" } ], "returns" : { "type" : "xs:base64Binary", "description" : "An edged copy of $image" }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">ierr:IM001 passed image is invalid</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 3, "name" : "emboss", "qname" : "man:emboss", "signature" : "($image as xs:base64Binary, $radius as xs:double, $sigma as xs:double) as xs:base64Binary external", "description" : " Emboss an images (highlights edges with 3D effect).\n", "summary" : "<p> Emboss an images (highlights edges with 3D effect).</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "image", "type" : "xs:base64Binary", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the source image</div>" }, { "name" : "radius", "type" : "xs:double", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> radius of the Gaussian in pixels</div>" }, { "name" : "sigma", "type" : "xs:double", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> standard deviation of the Laplacian in pixels</div>" } ], "returns" : { "type" : "xs:base64Binary", "description" : "An embossed copy of $image" }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">ierr:IM001 passed image is invalid</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 1, "name" : "enhance", "qname" : "man:enhance", "signature" : "($image as xs:base64Binary) as xs:base64Binary external", "description" : " Enhance an images (minimizes noise).\n", "summary" : "<p> Enhance an images (minimizes noise).</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "image", "type" : "xs:base64Binary", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the source image</div>" } ], "returns" : { "type" : "xs:base64Binary", "description" : "An enhanced copy of $image." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">ierr:IM001 passed image is invalid</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 1, "name" : "equalize", "qname" : "man:equalize", "signature" : "($image as xs:base64Binary) as xs:base64Binary external", "description" : " Equalize an images (histogramm equalization).\n", "summary" : "<p> Equalize an images (histogramm equalization).</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "image", "type" : "xs:base64Binary", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the source image</div>" } ], "returns" : { "type" : "xs:base64Binary", "description" : "An equalized copy of $image." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">ierr:IM001 passed image is invalid</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 1, "name" : "erase", "qname" : "man:erase", "signature" : "($image as xs:base64Binary) as xs:base64Binary external", "description" : " Set all pixels of the image to the current backround color.\n In most cases, this will result in all pixels to be set to white.\n", "summary" : "<p> Set all pixels of the image to the current backround color.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "image", "type" : "xs:base64Binary", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> image to erase</div>" } ], "returns" : { "type" : "xs:base64Binary", "description" : "A copy of image with all pixels set to the current background color" }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">ierr:IM001 passed image is invalid</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 1, "name" : "flip", "qname" : "man:flip", "signature" : "($image as xs:base64Binary) as xs:base64Binary external", "description" : " Flip an image (vertical rotation).\n", "summary" : "<p> Flip an image (vertical rotation).</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "image", "type" : "xs:base64Binary", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> source image</div>" } ], "returns" : { "type" : "xs:base64Binary", "description" : "A vertically rotated copy of $image" }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">ierr:IM001 passed image is invalid</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 1, "name" : "flop", "qname" : "man:flop", "signature" : "($image as xs:base64Binary) as xs:base64Binary external", "description" : " Flop an image (horizontal rotation).\n", "summary" : "<p> Flop an image (horizontal rotation).</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "image", "type" : "xs:base64Binary", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> source image</div>" } ], "returns" : { "type" : "xs:base64Binary", "description" : "A horizontally rotated copy of $image" }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">ierr:IM001 passed image is invalid</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 2, "name" : "gamma", "qname" : "man:gamma", "signature" : "($image as xs:base64Binary, $gamma-value as xs:double) as xs:base64Binary external", "description" : " Gamma correct an image.\n Gamma values less than zero will erase the image.\n", "summary" : "<p> Gamma correct an image.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "image", "type" : "xs:base64Binary", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the source image</div>" }, { "name" : "gamma-value", "type" : "xs:double", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> value for which to gamma correct the image</div>" } ], "returns" : { "type" : "xs:base64Binary", "description" : "A gamma corrected copy of $image" }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">ierr:IM001 passed image is invalid</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 4, "name" : "gamma", "qname" : "man:gamma", "signature" : "($image as xs:base64Binary, $gamma-red as xs:double, $gamma-green as xs:double, $gamma-blue as xs:double) as xs:base64Binary external", "description" : " Gamma correct an image for every color channel seperately.\n Gamma values less than zero for any color will erase the corresponding color.\n", "summary" : "<p> Gamma correct an image for every color channel seperately.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "image", "type" : "xs:base64Binary", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the source image</div>" }, { "name" : "gamma-red", "type" : "xs:double", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> value to gamma correct the red channel of the image</div>" }, { "name" : "gamma-green", "type" : "xs:double", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> value to gamma correct the green channel of the image</div>" }, { "name" : "gamma-blue", "type" : "xs:double", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> value to gamma correct the blue channel of the image</div>" } ], "returns" : { "type" : "xs:base64Binary", "description" : "A gamma corrected copy of $image" }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">ierr:IM001 passed image is invalid</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 2, "name" : "implode", "qname" : "man:implode", "signature" : "($image as xs:base64Binary, $factor as xs:double) as xs:base64Binary external", "description" : " Apply an implode effect to an image (a sort of special effect).\n", "summary" : "<p> Apply an implode effect to an image (a sort of special effect).</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "image", "type" : "xs:base64Binary", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the source image</div>" }, { "name" : "factor", "type" : "xs:double", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> factor to implode to</div>" } ], "returns" : { "type" : "xs:base64Binary", "description" : "An imploded copy of $image." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">ierr:IM001 passed image is invalid</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 2, "name" : "oil-paint", "qname" : "man:oil-paint", "signature" : "($image as xs:base64Binary, $radius as xs:double) as xs:base64Binary external", "description" : " Apply an oil paint effect to an image (makes the image look as if it was\n an oil paint).\n", "summary" : "<p> Apply an oil paint effect to an image (makes the image look as if it was\n an oil paint).</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "image", "type" : "xs:base64Binary", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the source image</div>" }, { "name" : "radius", "type" : "xs:double", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> radius with which to oil paint</div>" } ], "returns" : { "type" : "xs:base64Binary", "description" : "A oil-painted copy of $image" }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">ierr:IM001 passed image is invalid</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 5, "name" : "overlay", "qname" : "man:overlay", "signature" : "($image as xs:base64Binary, $overlay-image as xs:base64Binary, $overlay-upper-left-x as xs:unsignedInt, $overlay-upper-left-y as xs:unsignedInt, $operator as xs:string) as xs:base64Binary", "description" : " Overlay $image with $overlay-image at the specfied position.\n The $operator defines the details of the overlay and can have one of the\n following values:\n <ul xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">\n   <li>OverCompositeOp: The result is the union of the two image shapes\n       with the overlay image obscuring image in the region of overlap.</li>\n   <li>InCompositeOp: The result is a simple overlay image cut by the shape\n       of image. None of the image data of image is included in the result.</li>\n   <li>OutCompositeOp: The resulting image is the overlay image with the shape\n       of image cut out.</li>\n   <li>AtopCompositeOp: The result is the same shape as image, with overlay\n       image obscuring image there the image shapes overlap. Note that this\n       differs from OverCompositeOp because the portion of composite image\n       outside of image's shape does not appear in the result.</li>\n   <li>XorCompositeOp: The result is the image data from both overlay image\n       and image that is outside the overlap region. The overlap region will\n       be blank.</li>\n   <li>PlusCompositeOp: The result is just the sum of the image data of both\n       images. Output values are cropped to 255 (no overflow). This operation\n       is independent of the matte channels.</li>\n   <li>MinusCompositeOp: The result of overlay image - image, with overflow\n       cropped to zero. The matte chanel is ignored (set to 255, full\n       coverage).</li>\n   <li>AddCompositeOp: The result of overlay image + image, with overflow\n       wrapping around (mod 256).</li>\n   <li>SubtractCompositeOp: The result of overlay image - image, with underflow\n       wrapping around (mod 256). The add and subtract operators can be used to\n       perform reverible transformations.</li>\n   <li>DifferenceCompositeOp: The result of abs(overlay image - image). This is\n       useful for comparing two very similar images.</li>\n   <li>BumpmapCompositeOp: The result image shaded by overlay image.</li>\n </ul>\n", "summary" : "<p> Overlay $image with $overlay-image at the specfied position.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "image", "type" : "xs:base64Binary", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> base image</div>" }, { "name" : "overlay-image", "type" : "xs:base64Binary", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> image to overlay.</div>" }, { "name" : "overlay-upper-left-x", "type" : "xs:unsignedInt", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> horizontal position within $image where the left upper edge of the $overlay-image is placed</div>" }, { "name" : "overlay-upper-left-y", "type" : "xs:unsignedInt", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> vertical position within $image where the left upper edge of the $overlay-image is placed</div>" }, { "name" : "operator", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> defines how the overlay image should be overlayed (see details in operator listing above)</div>" } ], "returns" : { "type" : "xs:base64Binary", "description" : "A new image consisting of $image overlayed with $overlay-image." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">ierr:IM001 passed image is invalid</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">err:FORG0001 unsupported operator</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 2, "name" : "reduce-noise", "qname" : "man:reduce-noise", "signature" : "($image as xs:base64Binary, $order as xs:double) as xs:base64Binary external", "description" : " Reduce noise of an image using a noise peak elemination filter.\n", "summary" : "<p> Reduce noise of an image using a noise peak elemination filter.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "image", "type" : "xs:base64Binary", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the source image</div>" }, { "name" : "order", "type" : "xs:double", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> defines how much the noise is reduced</div>" } ], "returns" : { "type" : "xs:base64Binary", "description" : "A copy of $image with reduced noise" }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">ierr:IM001 passed image is invalid</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 3, "name" : "resize", "qname" : "man:resize", "signature" : "($image as xs:base64Binary, $width as xs:unsignedInt, $height as xs:unsignedInt) as xs:base64Binary external", "description" : " Get a copy of the passed image with changed width and height (without\n zooming the image's content).\n To change the size of the actual contents of an image, use the zoom function.\n More in detail: If the new dimensions are greater than the current dimensions\n the new image will have the passed image in the upper left corner and the rest\n will be filled with the current background color.\n If the passed dimensions are less than the current dimensions, the new image\n will contain the specified rectangle of the passed image beginning at the upper\n left corner.\n", "summary" : "<p> Get a copy of the passed image with changed width and height (without\n zooming the image's content).</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "image", "type" : "xs:base64Binary", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> image to resize</div>" }, { "name" : "width", "type" : "xs:unsignedInt", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> new width</div>" }, { "name" : "height", "type" : "xs:unsignedInt", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> new height</div>" } ], "returns" : { "type" : "xs:base64Binary", "description" : "resized copy of the source image" }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">ierr:IM001 passed image is invalid.</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 2, "name" : "rotate", "qname" : "man:rotate", "signature" : "($image as xs:base64Binary, $angle as xs:int) as xs:base64Binary external", "description" : " Get a new image as rotated copy of a passed source image (rotated by -360 to\n 360 degrees).\n The image is enlarged if this is required for containing the rotated image,\n but never shrunk even if the rotation would make a smaller image possible.\n", "summary" : "<p> Get a new image as rotated copy of a passed source image (rotated by -360 to\n 360 degrees).</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "image", "type" : "xs:base64Binary", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> source image.</div>" }, { "name" : "angle", "type" : "xs:int", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> between -360 to 360 degrees. Other values will be adjusted by modulo 360</div>" } ], "returns" : { "type" : "xs:base64Binary", "description" : "A rotated copy of $image" }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">ierr:IM001 passed image is invalid</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 2, "name" : "solarize", "qname" : "man:solarize", "signature" : "($image as xs:base64Binary, $factor as xs:double) as xs:base64Binary external", "description" : " Apply a solarize effect to the image (similar to the effect seen when\n exposing a photographic film to light during the development process).\n", "summary" : "<p> Apply a solarize effect to the image (similar to the effect seen when\n exposing a photographic film to light during the development process).</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "image", "type" : "xs:base64Binary", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the source image</div>" }, { "name" : "factor", "type" : "xs:double", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> strength of the solarization (0 to 65535; 65535=100%)</div>" } ], "returns" : { "type" : "xs:base64Binary", "description" : "A solarized copy of $image." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">ierr:IM001 passed image is invalid</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 2, "name" : "stereo", "qname" : "man:stereo", "signature" : "($left-image as xs:base64Binary, $right-image as xs:base64Binary) as xs:base64Binary external", "description" : " Make two passed images appear as stereo image when viewed with red-blue glasses.\n Both images should be same but from a slightly different angle for this to work.\n Both images should have the same size, if not, the size of the left image will\n be taken.\n", "summary" : "<p> Make two passed images appear as stereo image when viewed with red-blue glasses.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "left-image", "type" : "xs:base64Binary", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> left image for the stereo image.</div>" }, { "name" : "right-image", "type" : "xs:base64Binary", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> right image for the stereo image.</div>" } ], "returns" : { "type" : "xs:base64Binary", "description" : "A new image as combined stereo image of both source images" }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">ierr:IM001 one of the passed images is invalid</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 5, "name" : "sub-image", "qname" : "man:sub-image", "signature" : "($image as xs:base64Binary, $left-upper-x as xs:unsignedInt, $left-upper-y as xs:unsignedInt, $width as xs:unsignedInt, $height as xs:unsignedInt) as xs:base64Binary external", "description" : " Copy a part of the source image specified by a rectangle.\n If the passed parameters for the sub-image specify a rectangle that isn't\n entirely within the source image only the area that lies within the image\n boundaries will be returned.\n", "summary" : "<p> Copy a part of the source image specified by a rectangle.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "image", "type" : "xs:base64Binary", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the image from which to extract a sub-image</div>" }, { "name" : "left-upper-x", "type" : "xs:unsignedInt", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> is the x value of the upper left corner of the rectangle to cut out</div>" }, { "name" : "left-upper-y", "type" : "xs:unsignedInt", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> is the y value of the upper left corner of the rectangle to cut out.</div>" }, { "name" : "width", "type" : "xs:unsignedInt", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> width of the rectangle to cut out</div>" }, { "name" : "height", "type" : "xs:unsignedInt", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> height of the rectangle to cut out</div>" } ], "returns" : { "type" : "xs:base64Binary", "description" : "A new image containing parts of the source image" }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">ierr:IM001 passed image is invalid</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 2, "name" : "swirl", "qname" : "man:swirl", "signature" : "($image as xs:base64Binary, $degree as xs:double) as xs:base64Binary external", "description" : " Swirl an image (image pixels are rotated by degree).\n", "summary" : "<p> Swirl an image (image pixels are rotated by degree).</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "image", "type" : "xs:base64Binary", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the source image</div>" }, { "name" : "degree", "type" : "xs:double", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> degree to swirl image pixels</div>" } ], "returns" : { "type" : "xs:base64Binary", "description" : "A swirled copy of $image" }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">ierr:IM001 passed image is invalid</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 2, "name" : "transparent", "qname" : "man:transparent", "signature" : "($image as xs:base64Binary, $color as xs:string) as xs:base64Binary", "description" : " Make all pixels of the specfied color transparent.\n This works correctly only with image types supporting transparency\n (e.g GIF or PNG).\n", "summary" : "<p> Make all pixels of the specfied color transparent.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "image", "type" : "xs:base64Binary", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the source image</div>" }, { "name" : "color", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> color to make transparent (e.g. '#FFFFFF')</div>" } ], "returns" : { "type" : "xs:base64Binary", "description" : "A copy of $image with the specified color made transparent." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">ierr:IM001 passed image is invalid</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">err:FORG0001 unsupported color</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 1, "name" : "trim", "qname" : "man:trim", "signature" : "($image as xs:base64Binary) as xs:base64Binary external", "description" : " Trim edges of the image's background color from the image.\n", "summary" : "<p> Trim edges of the image's background color from the image.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "image", "type" : "xs:base64Binary", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the source image</div>" } ], "returns" : { "type" : "xs:base64Binary", "description" : "A trimmed copy of $image" }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">ierr:IM001 passed image is invalid</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 2, "name" : "watermark", "qname" : "man:watermark", "signature" : "($image as xs:base64Binary, $watermark as xs:base64Binary) as xs:base64Binary external", "description" : " Add a $watermark image to $image.\n", "summary" : "<p> Add a $watermark image to $image.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "image", "type" : "xs:base64Binary", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the source image</div>" }, { "name" : "watermark", "type" : "xs:base64Binary", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the watermark image</div>" } ], "returns" : { "type" : "xs:base64Binary", "description" : "A watermarked copy of $image" }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">ierr:IM001 one of the passed images is invalid</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 2, "name" : "zoom-by-height", "qname" : "man:zoom-by-height", "signature" : "($image as xs:base64Binary, $height as xs:unsignedInt) as xs:base64Binary external", "description" : " Zoom the passed image to a given new height while keeping the ratio between\n width and height.\n So, the width is scaled accordingly.\n Important note: this function does not change the size information stored\n in the image (e.g. basic:width will not show a different value).\n", "summary" : "<p> Zoom the passed image to a given new height while keeping the ratio between\n width and height.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "image", "type" : "xs:base64Binary", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> image to resize</div>" }, { "name" : "height", "type" : "xs:unsignedInt", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> new height for the image in pixels</div>" } ], "returns" : { "type" : "xs:base64Binary", "description" : "A copy of $image with given $height and width adjusted accordingly" }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">ierr:IM001 passed image is invalid</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 2, "name" : "zoom-by-width", "qname" : "man:zoom-by-width", "signature" : "($image as xs:base64Binary, $width as xs:unsignedInt) as xs:base64Binary external", "description" : " Zoom the passed image to a given new width while keeping the ratio between\n width and height.\n So, the height is scaled accordingly.\n Important note: this function does not change the size information stored\n in the image (e.g. basic:width will not show a different value).\n", "summary" : "<p> Zoom the passed image to a given new width while keeping the ratio between\n width and height.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "image", "type" : "xs:base64Binary", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> image to resize</div>" }, { "name" : "width", "type" : "xs:unsignedInt", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> new width for the image in pixels</div>" } ], "returns" : { "type" : "xs:base64Binary", "description" : "A copy of $image with given $width and height changed accordingly" }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">ierr:IM001 passed image is invalid</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 2, "name" : "zoom", "qname" : "man:zoom", "signature" : "($image as xs:base64Binary, $ratio as xs:double) as xs:base64Binary external", "description" : " Zoom the passed image by the specified factor while keeping the ratio between\n width and height.\n A ratio of less than 1 will make the image smaller.\n A ratio of less or equal than 0 will not effect the image.\n Important note: this function does not change the size information stored in the\n image (e.g. basic:width will not show a different value).\n", "summary" : "<p> Zoom the passed image by the specified factor while keeping the ratio between\n width and height.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "image", "type" : "xs:base64Binary", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> image to resize</div>" }, { "name" : "ratio", "type" : "xs:double", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> ratio to zoom width and height by</div>" } ], "returns" : { "type" : "xs:base64Binary", "description" : "A copy of $image with resized content" }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">ierr:IM001 passed image is invalid</xqdoc:error>" ] } ], "variables" : [  ] }, "http://zorba.io/modules/http-client-wrapper" : { "ns" : "http://zorba.io/modules/http-client-wrapper", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">This module provides conversion functions between the\n XML Expath http-client request and response formats and the\n JSON http-client request and response formats used by the\n <code>http://zorba.io/modules/http-client</code> module.\n </p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">This module is reserved for internal use by the\n <code>http://www.zorba-xquery.com/modules/http-client</code> and the\n <code>http://expath.org/ns/http-client</code> modules.\n This module may be removed at any time. Method signature and\n semantics may change.\n </p>\n", "sees" : [  ], "authors" : [ "<xqdoc:author xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Federico Cavalieri</xqdoc:author>" ], "version" : null, "encoding" : "utf-8", "namespaces" : [ { "uri" : "http://zorba.io/annotations", "prefix" : "an" }, { "uri" : "http://www.w3.org/2005/xqt-errors", "prefix" : "err" }, { "uri" : "http://expath.org/ns/error", "prefix" : "error" }, { "uri" : "http://expath.org/ns/http-client", "prefix" : "http-schema" }, { "uri" : "http://zorba.io/modules/http-client-wrapper", "prefix" : "http-wrapper" }, { "uri" : "http://jsoniq.org/functions", "prefix" : "jn" }, { "uri" : "http://zorba.io/modules/http-client", "prefix" : "json-http" }, { "uri" : "http://jsoniq.org/function-library", "prefix" : "libjn" }, { "uri" : "http://www.w3.org/2010/xslt-xquery-serialization", "prefix" : "ser" }, { "uri" : "http://zorba.io/options/versioning", "prefix" : "ver" } ], "functions" : [ { "isDocumented" : true, "arity" : 3, "name" : "http-nondeterministic-request", "qname" : "http-wrapper:http-nondeterministic-request", "signature" : "($request as element(*)?, $href as xs:string?, $bodies as item()*) as item()+", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">This function sends an HTTP request and returns the corresponding response.\n This function is declared non-deterministic and should only be used to issue\n requests which do not change the state of the server.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Its inputs, outputs, and behavior are identical to the\n <a href=\"http://expath.org/spec/http-client\">EXPath http-client</a>'s\n send-request() function (except that HTML responses are not tidied\n into XML - see <a href=\"#expath_relation\">the note above</a>). It\n is provided here for use in Zorba installations that do not have\n the EXPath module available. If you have the option of using the\n EXPath module instead of this function, please do so, as it will\n allow your application to be more interoperable between different\n XQuery engines.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Full documentation of the $request parameter can be found in\n <a href=\"http://expath.org/spec/http-client#d2e183\">the EXPath\n specification</a>.</p>\n", "summary" : "<p>  This function sends an HTTP request and returns the corresponding response.</p>", "annotation_str" : " %an:nondeterministic", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "nondeterministic", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "request", "type" : "element(*)", "occurrence" : "?", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> Contains the various parameters of the request (see above).</div>" }, { "name" : "href", "type" : "xs:string", "occurrence" : "?", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The URL to which the request will be made (see <a href=\"#url_string\">note</a> above). If this parameter is specified, it will override the \"href\" attribute of $request.</div>" }, { "name" : "bodies", "type" : "item()", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> is the request body content, for HTTP methods that can contain a body in the request (i.e. POST and PUT). It is an error if this param is not the empty sequence for methods</div>" } ], "returns" : { "type" : "item()+", "description" : "<a xmlns:xqdoc=\"http://www.xqdoc.org/1.0\" href=\"#standard_return\">standard http-client return type</a>." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">error:HC001 An HTTP error occurred.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">error:HC002 Error parsing the response content as XML.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">error:HC003 With a multipart response, the override-media-type must be either a multipart media type or application/octet-stream.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">error:HC004 The src attribute on the body element is mutually exclusive with all other attribute (except the media-type).</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">error:HC005 The input request element is not valid.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">error:HC006 A timeout occurred waiting for the response.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">error:HCV02 Trying to follow a redirect of a POST, PUT, or DELETE request</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 3, "name" : "http-sequential-request", "qname" : "http-wrapper:http-sequential-request", "signature" : "($request as element(*)?, $href as xs:string?, $bodies as item()*) as item()+", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">This function sends an HTTP request and returns the corresponding response.\n This function is declared sequential and can be used to issue\n requests which change the state of the server.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Its inputs, outputs, and behavior are identical to the\n <a href=\"http://expath.org/spec/http-client\">EXPath http-client</a>'s\n send-request() function (except that HTML responses are not tidied\n into XML - see <a href=\"#expath_relation\">the note above</a>). It\n is provided here for use in Zorba installations that do not have\n the EXPath module available. If you have the option of using the\n EXPath module instead of this function, please do so, as it will\n allow your application to be more interoperable between different\n XQuery engines.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Full documentation of the $request parameter can be found in\n <a href=\"http://expath.org/spec/http-client#d2e183\">the EXPath\n specification</a>.</p>\n", "summary" : "<p>  This function sends an HTTP request and returns the corresponding response.</p>", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "request", "type" : "element(*)", "occurrence" : "?", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> Contains the various parameters of the request (see above).</div>" }, { "name" : "href", "type" : "xs:string", "occurrence" : "?", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The URL to which the request will be made (see <a href=\"#url_string\">note</a> above). If this parameter is specified, it will override the \"href\" attribute of $request.</div>" }, { "name" : "bodies", "type" : "item()", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> is the request body content, for HTTP methods that can contain a body in the request (i.e. POST and PUT). It is an error if this param is not the empty sequence for methods</div>" } ], "returns" : { "type" : "item()+", "description" : "<a xmlns:xqdoc=\"http://www.xqdoc.org/1.0\" href=\"#standard_return\">standard http-client return type</a>." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">error:HC001 An HTTP error occurred.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">error:HC002 Error parsing the response content as XML.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">error:HC003 With a multipart response, the override-media-type must be either a multipart media type or application/octet-stream.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">error:HC004 The src attribute on the body element is mutually exclusive with all other attribute (except the media-type).</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">error:HC005 The input request element is not valid.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">error:HC006 A timeout occurred waiting for the response.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">error:HCV02 Trying to follow a redirect of a POST, PUT, or DELETE request</xqdoc:error>" ] } ], "variables" : [  ] }, "http://www.28msec.com/modules/cloudsearch" : { "ns" : "http://www.28msec.com/modules/cloudsearch", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">This module offers functionality to search, add, update and remove documents in\n an Amazon CloudSearch domain.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">For each functionality two methods are provided\n <ul>\n  <li>one which accepts a single $options parameter. In this case the default\n      credentials for the \"CloudSearch\" category will be used to determine the\n      CloudSearch endpoint to use.</li>\n  <li>one which accepts both an $endpoint and an $options parameter.\n      In this case, if the $endpoint parameter is of type string, it will be\n      interpreted as the name of a credentials in the \"CloudSearch\" category.\n      Otherwise, if the $endpoint parameter is of type anyURI, it will be\n      interpreted as the endpoint URI.</li>\n </ul>\n </p>\n <h2 xmlns:xqdoc=\"http://www.xqdoc.org/1.0\" id=\"determinism\">Important Notice Regarding Function Determinism</h2>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">The search functions (<a href=\"?anchor=search-1\">search#1</a> and\n <a href=\"?anchor=search-2\">search#2</a>) are declared deterministic, which means that\n their results could be cached when invoked multiple times with the same arguments\n in the same query execution.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">To not use cached results you can use two alternative search functions\n (<a href=\"?anchor=search-nondeterministic-1\">search-nondeterministic#1</a> and\n <a href=\"?anchor=search-nondeterministic-2\">search-nondeterministic#2</a>), which have\n been declared as being non deterministic.</p>\n <h2 xmlns:xqdoc=\"http://www.xqdoc.org/1.0\" id=\"search-options\">Search options</h2>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">The search settings are specified by means of an object whose form\n is described in the following. The two most important fields are the\n \"q\" and \"bq\" fields. At least one of them must be specified. If the \"bq\"\n field is specified in conjunction with the \"q\" parameter, the values\n are joined with a top-level AND.\n <ul>\n   <li>bq: one or more match expressions that define a Boolean search. Multiple\n       expressions are joined with a top-level AND.\n       <p>Type: string</p>\n   </li>\n   <li>q: the string to search for. You use the q parameter to perform simple text\n       searches. This searches the default search field for the specified text.\n       <p>Type: string</p>\n   </li>\n </ul>\n </p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Additional documentation on the format of the query and boolean query strings can\n be found at:\n <a href=\"http://docs.aws.amazon.com/cloudsearch/latest/developerguide/Search.Requests.html#Search.MatchSetExpression\">\n Expression Syntax for Boolean Queries</a>.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">The following additional optional parameters can be specified.\n <ul>\n   <li>facet: a comma-separated list of the fields for which you want to compute facets.\n       The specified fields must be numeric fields or defined as facet enabled in the\n       domain configuration.\n       <p>Type: string</p>\n   </li>\n   <li>facet-<i>FIELD</i>-constraints: the field values (facet constraints) that\n       you want to count for a particular field. FIELD is the name of the field.\n       Constraints are specified as a comma-separated list of ranges or\n       single-quoted strings. If you don't specify facet constraints, counts\n       are computed for all field values.\n       <p>Type: string</p>\n   </li>\n   <li>facet-<i>FIELD</i>-sort: how you want to sort facet values for a\n       particular field. FIELD is the name of the field. There are four sorting options:\n       <ul>\n         <li>alpha: Sort the facet values alphabetically (in ascending order).</li>\n         <li>count: Sort the facet values by their counts (in descending order).</li>\n         <li>max: Sort the facet values according to the maximum values in the specified\n             field. This option is specified as <code>max(FIELD)</code>. By default, the\n             facet values are sorted in ascending order. To sort in descending order,\n             prefix the sort option with - (minus): <code>-max(FIELD)</code>.\n         </li>\n         <li>sum: Sort the facet values according to the sum of the values in the\n             specified field (in ascending order). This option is specified as\n             <code>sum(FIELD)</code>.\n         </li>\n       </ul>\n   </li>\n   <li>facet-<i>FIELD</i>-top-n: set the maximum number of facet constraints to\n       be included for the specified field in the search results. By default,\n       the results include counts for the top 40 constraints.\n        <p>Type: integer</p>\n   </li>\n   <li>no-cache: if false, a cached result can be returned. If true, a revalidation of the\n        results is forced. Default is false.\n        <p>Type: boolean</p>\n   </li>\n   <li>rank: a comma-separated list of fields or rank expressions to use for ranking. A\n        maximum of 10 fields and rank expressions can be specified. You can use any uint\n        field to rank results numerically. Any result-enabled text or literal field can be\n        used to rank results alphabetically. To rank results by relevance, you can specify\n        the name of a custom rank expression or text_relevance. Hits are ordered according\n        to the specified rank field(s). By default, hits are ranked in ascending order.\n        <p>You can prefix a field name with a minus (-) to rank in descending order. If no\n        rank parameter is specified, it defaults to <code>rank=-text_relevance</code>, which\n        lists results according to their text_relevance scores with the highest-scoring\n        documents first.</p>\n       <p>Type: string</p>\n   </li>\n   <li>rank-<i>RANKNAME</i>: define a rank expression that can be used with the\n       rank parameter. You can also specify the new rank expression as a return\n       field and use it to set thresholds for the search results with the\n       t-<i>FIELD</i> parameter. For more information about constructing\n       rank expressions, see\n       <a href=\"http://docs.aws.amazon.com/cloudsearch/latest/developerguide/tuneranking.html\">\n       Customizing Result Ranking with Amazon CloudSearch</a>.\n       <p>Type: string</p>\n   </li>\n   <li>return-fields: the document fields to include in the response. Up to 2 KB\n       of data can be returned from a text field. If the field contents exceed\n       2 KB, only the first 2 KB is included in the results. Specified as a\n       comma-separated list of field names. If no return-fields are specified,\n       only the document ids of the hits are returned.\n       <p>Type: string</p>\n   </li>\n   <li>size: the maximum number of search hits to return. The default is 10.\n       <p>Type: integer</p>\n   </li>\n   <li>start: the offset of the first search hit you want to return. The default is 0\n       (the first hit).\n       <p>Type: integer</p>\n   </li>\n   <li>t-<i>FIELD</i>: restrict the match set used in subsequent post-processing\n       steps according to the specified rank expression. Only hits that have a\n       score within the specified range are included. Ranges are specified as\n       described in\n       <a href=\"http://docs.aws.amazon.com/cloudsearch/latest/developerguide/Search.Requests.html#Search.MatchSetExpression\">\n       Expression Syntax for Boolean Queries</a>.\n       <p>Type: integer</p>\n   </li>\n </ul>\n For additional details on the search parameters you can refer to the\n <a href=\"http://docs.aws.amazon.com/cloudsearch/latest/developerguide/Search.Requests.html\">\n Amazon CloudSearch search requests documentation</a>.</p>\n <h2 xmlns:xqdoc=\"http://www.xqdoc.org/1.0\" id=\"search-results\">Search Results</h2>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">The format of the returned object is the following:\n <pre>\n {\n   \"rank\" : \"-text_relevance\",\n   \"match-expr\" : \"(label 'star wars')\",\n   \"hits\" :\n   {\n     \"found\" : 2,\n     \"start\" : 0,\n     \"hit\" :\n     [\n       {\n         \"id\" : \"tt1185834\",\n         \"data\" :\n         {\n           \"actor\" : [\"Abercrombie, Ian\",\"Baker, Dee\",\"Burton, Corey\"],\n           \"title\" : [\"Star Wars: The Clone Wars\"]\n         }\n       },\n       {\n         \"id\" :\"tt0121766\",\n         \"data\" :\n         {\n           \"actor\" : [\"Bai, Ling\",\"Bryant, Gene\",\"Castle-Hughes, Keisha\"],\n           \"title\" : [\"Star Wars: Episode III - Revenge of the Sith\"]\n         }\n       }\n     ]\n   },\n   \"info\" :\n   {\n     \"rid\" : \"b7c167f6c2da6d93531b9a7b314ad030b3a74803b4b7797edb905ba5a6a08\",\n     \"time-ms\" : 2,\n     \"cpu-time-ms\" : 0\n   }\n }\n </pre>\n </p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">The returned object fields have the following meaning:\n <ul>\n   <li>match-expr: Shows the match expression constructed from the search\n       parameters.</li>\n   <li>hits: Contains hit statistics (found, start) and a hit array that lists\n       the document ids and data for each hit.</li>\n   <li>found: The total number of hits that match the search request after\n       Amazon CloudSearch finished processing the match set.</li>\n   <li>start: The index of the first hit returned in this response.</li>\n   <li>hit: An array that lists the document ids and data for each hit.</li>\n   <li>id: The unique identifier for a document.</li>\n   <li>data: A list of returned fields.</li>\n   <li>facets: Contains facet information and facet counts.</li>\n   <li><i>FacetFieldName</i>: A field for which facets were calculated.</li>\n   <li>constraints: An array of the facet values and counts.</li>\n   <li>value: The facet value being counted.</li>\n   <li>count: The number of hits that contain the facet value in <i>FacetFieldName</i>.\n       </li>\n   <li>info: Contains information about the request processing.</li>\n   <li>rank: Lists the fields that were used to rank the search hits.</li>\n   <li>rid: The encrypted Resource ID.</li>\n   <li>time-ms: How long it took to process the search request in milliseconds.\n       </li>\n   <li>cpu-time-ms: The CPU time required to process the search request in\n       milliseconds.</li>\n   <li>messages: Contains any error messages returned by the search service.\n       The severity, code, and message properties are included for each item.</li>\n   <li>severity: The severity of the message. It is always warning, which indicates\n       a problem with the query string that did not prevent the request from being\n       processed.</li>\n   <li>code: The error code. The search service returns the following error codes:\n     <ul>\n       <li>CS-InvalidFieldOrRankAliasInRankParameter: the specified ranking field\n           could not be found.</li>\n       <li>CS-RankExpressionParseError: one of the specified rank expressions could\n           not be parsed. No query-time rank expressions will be used. </li>\n       <li>CS-RankExpressionValidationError: one of the specified rank expressions\n           could not be validated. No query-time rank expressions will be used.</li>\n       <li>CS-UndefinedField: an unknown field was specified in the match\n           expression.</li>\n       <li>CS-UnknownFieldInMatchExpression: a field specified in the bq parameter\n           could not be found.</li>\n       <li>CS-WildcardTermLimit: more than 2000 terms matched the wildcard in the\n           search request. The number of terms matched was limited to 2000.</li>\n     </ul>\n   </li>\n   <li>message: A description of the error that was returned by the search service.</li>\n </ul>\n </p>\n", "sees" : [  ], "authors" : [ "<xqdoc:author xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Federico Cavalieri</xqdoc:author>" ], "version" : null, "encoding" : "utf-8", "namespaces" : [ { "uri" : "http://zorba.io/annotations", "prefix" : "an" }, { "uri" : "http://www.28msec.com/modules/cloudsearch", "prefix" : "cloudsearch" }, { "uri" : "http://www.28msec.com/modules/credentials", "prefix" : "credentials" }, { "uri" : "http://zorba.io/modules/http-client", "prefix" : "http" } ], "functions" : [ { "isDocumented" : true, "arity" : 1, "name" : "add-document", "qname" : "cloudsearch:add-document", "signature" : "($options as object()) as object()", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Adds or replaces a document in the default Amazon CloudSearch domain.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">An add operation is applied only if a document with the same id is not\n present or if the version number specified in the operation is greater than\n the existing document's version number.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">The document to add or replace is specified through an object with\n the following form:\n <pre>\n {\n   \"id\" : \"tt0484562\",\n   \"version\" : 1,\n   \"lang\" : \"en\",\n   \"fields\" :\n   {\n     \"title\" : \"The Seeker: The Dark Is Rising\",\n     \"genre\" : [\"Adventure\",\"Drama\",\"Fantasy\",\"Thriller\"]\n   }\n }\n </pre>\n </p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Specifically, the fields in the request object have the following meaning:\n <ul>\n   <li>id: An alphanumeric string. Allowed characters are: a-z, 0-9, and _.\n       Document IDs cannot begin with an underscore. The max length is 128\n       characters.</li>\n   <li>version: Any non-negative number less than 2^32.</li>\n   <li>lang: An ISO-639-1 two-letter language code.</li>\n   <li>fields: An object containing one or more fields. Each field specifies\n       a field within the document being added. Field names must begin with a\n       lowercase letter and can contain the following characters: a-z, 0-9,\n       and _. Field names must be at least 3 and no more than 64 characters.\n       The names \"body\", \"docid\", and \"text_relevance\" are reserved names and\n       cannot be used as field names. To specify multiple values for a field,\n       you can specify an array of values instead of a single value.</li>\n </ul>\n </p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">The returned object lists all warnings that were generated.\n <pre>\n {\n   \"status\" : \"success\",\n   \"warning\" :\n   [\n     {\"message\" : \"Warning message.\"}\n   ]\n }\n </pre>\n </p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Specifically, the fields in the response object have the following meaning:\n <ul>\n   <li>status: the result status, always \"success\". In case an error occurred\n       processing the request, an error is raised.</li>\n   <li>warning: provides information about warnings generated during parsing\n       or validation. The field is not present if no warning were generated.</li>\n </ul>\n </p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Example, specifying endpoint URI:\n <pre>\n cloudsearch:add-document(\n anyURI(\"http://doc-movies-h2pc7ftfnsdlqh6pqqawbftrhu.us-east-1.cloudsearch.amazonaws.com\"),\n {\n   \"id\" : \"tt0484562\",\n   \"version\" : 1,\n   \"lang\" : \"en\",\n   \"fields\" :\n   {\n     \"title\" : \"The Seeker: The Dark Is Rising\",\n     \"genre\" : [\"Adventure\",\"Drama\",\"Fantasy\",\"Thriller\"]\n   }\n });\n </pre>\n </p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Example, using stored credentials:\n <pre>\n cloudsearch:add-document(\"movies\",\n {\n   \"id\" : \"tt0484562\",\n   \"version\" : 1,\n   \"lang\" : \"en\",\n   \"fields\" :\n   {\n     \"title\" : \"The Seeker: The Dark Is Rising\",\n     \"genre\" : [\"Adventure\",\"Drama\",\"Fantasy\",\"Thriller\"]\n   }\n });\n </pre>\n </p>\n", "summary" : "<p>  Adds or replaces a document in the default Amazon CloudSearch domain.</p>", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "options", "type" : "object()", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> An object specifying the request options</div>" } ], "returns" : { "type" : "object()", "description" : "The request response" }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cloudsearch:ENDPOINT if the default credentials endpoint is invalid or not present</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cloudsearch:AUTHORIZATION if the request was unauthorized</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cloudsearch:HTTP if an HTTP error has occurred</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cloudsearch:REQUEST if the request is malformed</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cloudsearch:RESOURCES if the server storage or bandwidth resources are insufficient to execute the request</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cloudsearch:INTERNAL if an internal server error occurred</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cloudsearch:RESPONSE if the CloudSearch response cannot be parsed</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 2, "name" : "add-document", "qname" : "cloudsearch:add-document", "signature" : "($endpoint as atomic, $options as object()) as object()", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Adds or replaces a document in the specified Amazon CloudSearch domain.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">An add operation is applied only if a document with the same id is not\n present or if the version number specified in the operation is greater than\n the existing document's version number.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">The document to add or replace is specified through an object with\n the following form:\n <pre>\n {\n   \"id\" : \"tt0484562\",\n   \"version\" : 1,\n   \"lang\" : \"en\",\n   \"fields\" :\n   {\n     \"title\" : \"The Seeker: The Dark Is Rising\",\n     \"genre\" : [\"Adventure\",\"Drama\",\"Fantasy\",\"Thriller\"]\n   }\n }\n </pre>\n </p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Specifically, the fields in the request object have the following meaning:\n <ul>\n   <li>id: An alphanumeric string. Allowed characters are: a-z, 0-9, and _.\n       Document IDs cannot begin with an underscore. The max length is 128\n       characters.</li>\n   <li>version: Any non-negative number less than 2^32.</li>\n   <li>lang: An ISO-639-1 two-letter language code.</li>\n   <li>fields: An object containing one or more fields. Each field specifies\n       a field within the document being added. Field names must begin with a\n       lowercase letter and can contain the following characters: a-z, 0-9,\n       and _. Field names must be at least 3 and no more than 64 characters.\n       The names \"body\", \"docid\", and \"text_relevance\" are reserved names and\n       cannot be used as field names. To specify multiple values for a field,\n       you can specify an array of values instead of a single value.</li>\n </ul>\n </p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">The returned object lists all warnings that were generated.\n <pre>\n {\n   \"status\" : \"success\",\n   \"warning\" :\n   [\n     {\"message\" : \"Warning message.\"}\n   ]\n }\n </pre>\n </p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Specifically, the fields in the response object have the following meaning:\n <ul>\n   <li>status: the result status, always \"success\". In case an error occurred\n       processing the request, an error is raised.</li>\n   <li>warning: provides information about warnings generated during parsing\n       or validation. The field is not present if no warning were generated.</li>\n </ul>\n </p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Example, specifying endpoint URI:\n <pre>\n cloudsearch:add-document(\n anyURI(\"http://doc-movies-h2pc7ftfnsdlqh6pqqawbftrhu.us-east-1.cloudsearch.amazonaws.com\"),\n {\n   \"id\" : \"tt0484562\",\n   \"version\" : 1,\n   \"lang\" : \"en\",\n   \"fields\" :\n   {\n     \"title\" : \"The Seeker: The Dark Is Rising\",\n     \"genre\" : [\"Adventure\",\"Drama\",\"Fantasy\",\"Thriller\"]\n   }\n });\n </pre>\n </p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Example, using stored credentials:\n <pre>\n cloudsearch:add-document(\"movies\",\n {\n   \"id\" : \"tt0484562\",\n   \"version\" : 1,\n   \"lang\" : \"en\",\n   \"fields\" :\n   {\n     \"title\" : \"The Seeker: The Dark Is Rising\",\n     \"genre\" : [\"Adventure\",\"Drama\",\"Fantasy\",\"Thriller\"]\n   }\n });\n </pre>\n </p>\n", "summary" : "<p>  Adds or replaces a document in the specified Amazon CloudSearch domain.</p>", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "endpoint", "type" : "atomic", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The endpoint URI or the name of a stored CloudSearch credentials</div>" }, { "name" : "options", "type" : "object()", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> An object specifying the request options</div>" } ], "returns" : { "type" : "object()", "description" : "The request response" }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cloudsearch:ENDPOINT if the specified endpoint is invalid</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cloudsearch:AUTHORIZATION if the request was unauthorized</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cloudsearch:HTTP if an HTTP error has occurred</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cloudsearch:REQUEST if the request is malformed</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cloudsearch:RESOURCES if the server storage or bandwidth resources are insufficient to execute the request</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cloudsearch:INTERNAL if an internal server error occurred</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cloudsearch:RESPONSE if the CloudSearch response cannot be parsed</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 1, "name" : "batch-document", "qname" : "cloudsearch:batch-document", "signature" : "($options as array()) as object()", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Adds, replaces or removes one or more documents from the default Amazon\n CloudSearch domain.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">An add or delete operation is only applied to an existing document if the\n version number specified in the operation is greater than the existing document\n version number.\n If multiple add or delete operations for the same document are specified, the\n operation with the highest version number is applied. If multiple operations in\n a batch specify the same document and version number, the document service\n arbitrarily picks which one to apply.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">The documents to add, replace or delete can be specified through an array with\n the following form:\n <pre>\n [\n   {\n     \"type\" : \"add\",\n     \"id\" :   \"tt0484562\",\n     \"version\" : 1,\n     \"lang\" : \"en\",\n     \"fields\" :\n     {\n       \"title\" : \"The Seeker: The Dark Is Rising\",\n       \"genre\" : [\"Adventure\",\"Drama\",\"Fantasy\",\"Thriller\"],\n     }\n   },\n   {\n     \"type\" : \"delete\",\n     \"id\" :   \"tt0484575\",\n     \"version\" : 2\n   }\n ]\n </pre>\n </p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Specifically, the fields in the request array have the following meaning:\n <ul>\n   <li>type: The operation type, \"add\" or \"delete\".</li>\n   <li>id: An alphanumeric string. Allowed characters are: a-z, 0-9, and _.\n       Document IDs cannot begin with an underscore. The max length is 128\n       characters.</li>\n   <li>version: Any non-negative number less than 2^32.</li>\n   <li>lang: An ISO-639-1 two-letter language code.</li>\n   <li>fields: An object containing one or more fields. Each field specifies\n       a field within the document being added. Field names must begin with a\n       lowercase letter and can contain the following characters: a-z, 0-9,\n       and _. Field names must be at least 3 and no more than 64 characters.\n       The names \"body\", \"docid\", and \"text_relevance\" are reserved names and\n       cannot be used as field names. To specify multiple values for a field,\n       you can specify an array of values instead of a single value.</li>\n </ul>\n </p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">The response body lists any warning that was generated.\n <pre>\n {\n   \"status\" : \"success\",\n   \"adds\" : 1,\n   \"deletes: 1,\n   \"warning\" :\n   [\n     {\"message\" : \"Warning message.\"}\n   ]\n }\n </pre>\n </p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Specifically, the fields in the response object have the following meaning:\n <ul>\n   <li>status: the result status, which is either success or error.</li>\n   <li>adds: the number of add document operations that were performed.</li>\n   <li>deletes: the number of delete document operations that were performed.</li>\n   <li>warning: provides information about a warning generated during parsing\n       or validation. The field is not present if no warning was generated.</li>\n </ul>\n </p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Example, using specified endpoint URI:\n <pre>\n cloudsearch:batch-document(\n anyURI(\"http://doc-movies-h2pc7ftfnsdlqh6pqqawbftrhu.us-east-1.cloudsearch.amazonaws.com\"),\n [\n   {\n     \"type\" : \"add\",\n     \"id\" :   \"tt0484562\",\n     \"version\" : 1,\n     \"lang\" : \"en\",\n     \"fields\" :\n     {\n       \"title\" : \"The Seeker: The Dark Is Rising\",\n       \"genre\" : [\"Adventure\",\"Drama\",\"Fantasy\",\"Thriller\"],\n     }\n   },\n   {\n     \"type\" : \"delete\",\n     \"id\" :   \"tt0484575\",\n     \"version\" : 2\n   }\n ]);\n </pre>\n </p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Example, using specified endpoint URI:\n <pre>\n cloudsearch:batch-document(\"movies\",\n [\n   {\n     \"type\" : \"add\",\n     \"id\" :   \"tt0484562\",\n     \"version\" : 1,\n     \"lang\" : \"en\",\n     \"fields\" :\n     {\n       \"title\" : \"The Seeker: The Dark Is Rising\",\n       \"genre\" : [\"Adventure\",\"Drama\",\"Fantasy\",\"Thriller\"],\n     }\n   },\n   {\n     \"type\" : \"delete\",\n     \"id\" :   \"tt0484575\",\n     \"version\" : 2\n   }\n ]);\n </pre>\n </p>\n", "summary" : "<p>  Adds, replaces or removes one or more documents from the default Amazon\n CloudSearch domain.</p>", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "options", "type" : "array()", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> An object specifying the request options</div>" } ], "returns" : { "type" : "object()", "description" : "The request response" }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cloudsearch:ENDPOINT if the default credentials endpoint is invalid or not present</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cloudsearch:AUTHORIZATION if the request was unauthorized</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cloudsearch:HTTP if an HTTP error has occurred</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cloudsearch:REQUEST if the request is malformed</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cloudsearch:RESOURCES if the server storage or bandwidth resources are insufficient to execute the request</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cloudsearch:INTERNAL if an internal server error occurred</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cloudsearch:RESPONSE if the CloudSearch response cannot be parsed</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 2, "name" : "batch-document", "qname" : "cloudsearch:batch-document", "signature" : "($endpoint as atomic, $options as array()) as object()", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Adds, replaces or removes one or more documents from the specified Amazon\n CloudSearch domain.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">An add or delete operation is only applied to an existing document if the\n version number specified in the operation is greater than the existing document\n version number.\n If multiple add or delete operations for the same document are specified, the\n operation with the highest version number is applied. If multiple operations in\n a batch specify the same document and version number, the document service\n arbitrarily picks which one to apply.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">The documents to add, replace or delete can be specified through an array with\n the following form:\n <pre>\n [\n   {\n     \"type\" : \"add\",\n     \"id\" :   \"tt0484562\",\n     \"version\" : 1,\n     \"lang\" : \"en\",\n     \"fields\" :\n     {\n       \"title\" : \"The Seeker: The Dark Is Rising\",\n       \"genre\" : [\"Adventure\",\"Drama\",\"Fantasy\",\"Thriller\"],\n     }\n   },\n   {\n     \"type\" : \"delete\",\n     \"id\" :   \"tt0484575\",\n     \"version\" : 2\n   }\n ]\n </pre>\n </p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Specifically, the fields in the request array have the following meaning:\n <ul>\n   <li>type: The operation type, \"add\" or \"delete\".</li>\n   <li>id: An alphanumeric string. Allowed characters are: a-z, 0-9, and _.\n       Document IDs cannot begin with an underscore. The max length is 128\n       characters.</li>\n   <li>version: Any non-negative number less than 2^32.</li>\n   <li>lang: An ISO-639-1 two-letter language code.</li>\n   <li>fields: An object containing one or more fields. Each field specifies\n       a field within the document being added. Field names must begin with a\n       lowercase letter and can contain the following characters: a-z, 0-9,\n       and _. Field names must be at least 3 and no more than 64 characters.\n       The names \"body\", \"docid\", and \"text_relevance\" are reserved names and\n       cannot be used as field names. To specify multiple values for a field,\n       you can specify an array of values instead of a single value.</li>\n </ul>\n </p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">The response body lists any warning that was generated.\n <pre>\n {\n   \"status\" : \"success\",\n   \"adds\" : 1,\n   \"deletes: 1,\n   \"warning\" :\n   [\n     {\"message\" : \"Warning message.\"}\n   ]\n }\n </pre>\n </p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Specifically, the fields in the response object have the following meaning:\n <ul>\n   <li>status: the result status, which is either success or error.</li>\n   <li>adds: the number of add document operations that were performed.</li>\n   <li>deletes: the number of delete document operations that were performed.</li>\n   <li>warning: provides information about a warning generated during parsing\n       or validation. The field is not present if no warning was generated.</li>\n </ul>\n </p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Example, using specified endpoint URI:\n <pre>\n cloudsearch:batch-document(\n anyURI(\"http://doc-movies-h2pc7ftfnsdlqh6pqqawbftrhu.us-east-1.cloudsearch.amazonaws.com\"),\n [\n   {\n     \"type\" : \"add\",\n     \"id\" :   \"tt0484562\",\n     \"version\" : 1,\n     \"lang\" : \"en\",\n     \"fields\" :\n     {\n       \"title\" : \"The Seeker: The Dark Is Rising\",\n       \"genre\" : [\"Adventure\",\"Drama\",\"Fantasy\",\"Thriller\"],\n     }\n   },\n   {\n     \"type\" : \"delete\",\n     \"id\" :   \"tt0484575\",\n     \"version\" : 2\n   }\n ]);\n </pre>\n </p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Example, using specified endpoint URI:\n <pre>\n cloudsearch:batch-document(\"movies\",\n [\n   {\n     \"type\" : \"add\",\n     \"id\" :   \"tt0484562\",\n     \"version\" : 1,\n     \"lang\" : \"en\",\n     \"fields\" :\n     {\n       \"title\" : \"The Seeker: The Dark Is Rising\",\n       \"genre\" : [\"Adventure\",\"Drama\",\"Fantasy\",\"Thriller\"],\n     }\n   },\n   {\n     \"type\" : \"delete\",\n     \"id\" :   \"tt0484575\",\n     \"version\" : 2\n   }\n ]);\n </pre>\n </p>\n", "summary" : "<p>  Adds, replaces or removes one or more documents from the specified Amazon\n CloudSearch domain.</p>", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "endpoint", "type" : "atomic", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The endpoint URI or the name of a stored CloudSearch credentials</div>" }, { "name" : "options", "type" : "array()", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> An object specifying the request options</div>" } ], "returns" : { "type" : "object()", "description" : "The request response" }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cloudsearch:ENDPOINT if the specified endpoint is invalid</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cloudsearch:AUTHORIZATION if the request was unauthorized</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cloudsearch:HTTP if an HTTP error has occurred</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cloudsearch:REQUEST if the request is malformed</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cloudsearch:RESOURCES if the server storage or bandwidth resources are insufficient to execute the request</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cloudsearch:INTERNAL if an internal server error occurred</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cloudsearch:RESPONSE if the CloudSearch response cannot be parsed</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 1, "name" : "delete-document", "qname" : "cloudsearch:delete-document", "signature" : "($options as object()) as object()", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Deletes a document from the default Amazon CloudSearch domain.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">A delete operation is only applied to an existing document if the\n version number specified in the operation is greater than the existing\n document's version number.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">The document to remove is specified through an object with the\n following form:\n <pre>\n {\n   \"id\" : \"tt0484562\",\n   \"version\" : 2\n }\n </pre>\n </p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Specifically, the fields in the request object have the following meaning:\n <ul>\n   <li>id: An alphanumeric string. Allowed characters are: a-z, 0-9, and _.\n       Document IDs cannot begin with an underscore. The max length is 128\n       characters.</li>\n   <li>version: Any non-negative number less than 2^32.</li>\n </ul>\n </p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">The response body lists any warning that was generated.\n <pre>\n {\n   \"status\" : \"success\",\n   \"warning\" :\n   [\n     {\"message\" : \"Warning message.\"}\n   ]\n }\n </pre>\n </p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Specifically, the fields in the response object have the following meaning:\n <ul>\n   <li>status: the result status, always \"success\". In case of errors an error is\n       raised.</li>\n   <li>warning: provides information about a warning generated during parsing or\n       validation. The field is not present if no warning was generated.</li>\n </ul>\n </p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Example, using specified endpoint URI:\n <pre>\n cloudsearch:delete-document(\n anyURI(\"http://doc-movies-h2pc7ftfnsdlqh6pqqawbftrhu.us-east-1.cloudsearch.amazonaws.com\"),\n {\n   \"id\" : \"tt0484562\",\n   \"version\" : 2\n });\n </pre>\n </p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Example, using specified stored endpoing:\n <pre>\n cloudsearch:delete-document(\"movies\",\n {\n   \"id\" : \"tt0484562\",\n   \"version\" : 2\n });\n </pre>\n </p>\n", "summary" : "<p>  Deletes a document from the default Amazon CloudSearch domain.</p>", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "options", "type" : "object()", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> An object specifying the request options</div>" } ], "returns" : { "type" : "object()", "description" : "The request response" }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cloudsearch:ENDPOINT if the default credentials endpoint is invalid or not present</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cloudsearch:AUTHORIZATION if the request was unauthorized</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cloudsearch:HTTP if an HTTP error has occurred</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cloudsearch:REQUEST if the request is malformed</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cloudsearch:RESOURCES if the server storage or bandwidth resources are insufficient to execute the request</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cloudsearch:INTERNAL if an internal server error occurred</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cloudsearch:RESPONSE if the CloudSearch response cannot be parsed</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 2, "name" : "delete-document", "qname" : "cloudsearch:delete-document", "signature" : "($endpoint as atomic, $options as object()) as object()", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Deletes a document from the specified Amazon CloudSearch domain.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">A delete operation is only applied to an existing document if the\n version number specified in the operation is greater than the existing\n document's version number.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">The document to remove is specified through an object with the\n following form:\n <pre>\n {\n   \"id\" : \"tt0484562\",\n   \"version\" : 2\n }\n </pre>\n </p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Specifically, the fields in the request object have the following meaning:\n <ul>\n   <li>id: An alphanumeric string. Allowed characters are: a-z, 0-9, and _.\n       Document IDs cannot begin with an underscore. The max length is 128\n       characters.</li>\n   <li>version: Any non-negative number less than 2^32.</li>\n </ul>\n </p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">The response body lists any warning that was generated.\n <pre>\n {\n   \"status\" : \"success\",\n   \"warning\" :\n   [\n     {\"message\" : \"Warning message.\"}\n   ]\n }\n </pre>\n </p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Specifically, the fields in the response object have the following meaning:\n <ul>\n   <li>status: the result status, always \"success\". In case of errors an error is\n       raised.</li>\n   <li>warning: provides information about a warning generated during parsing or\n       validation. The field is not present if no warning was generated.</li>\n </ul>\n </p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Example, using specified endpoint URI:\n <pre>\n cloudsearch:delete-document(\n anyURI(\"http://doc-movies-h2pc7ftfnsdlqh6pqqawbftrhu.us-east-1.cloudsearch.amazonaws.com\"),\n {\n   \"id\" : \"tt0484562\",\n   \"version\" : 2\n });\n </pre>\n </p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Example, using specified stored endpoing:\n <pre>\n cloudsearch:delete-document(\"movies\",\n {\n   \"id\" : \"tt0484562\",\n   \"version\" : 2\n });\n </pre>\n </p>\n", "summary" : "<p>  Deletes a document from the specified Amazon CloudSearch domain.</p>", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "endpoint", "type" : "atomic", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The endpoint URI or the name of a stored CloudSearch credentials</div>" }, { "name" : "options", "type" : "object()", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> An object specifying the request options</div>" } ], "returns" : { "type" : "object()", "description" : "The request response" }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cloudsearch:ENDPOINT if the specified endpoint is invalid</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cloudsearch:AUTHORIZATION if the request was unauthorized</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cloudsearch:HTTP if an HTTP error has occurred</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cloudsearch:REQUEST if the request is malformed</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cloudsearch:RESOURCES if the server storage or bandwidth resources are insufficient to execute the request</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cloudsearch:INTERNAL if an internal server error occurred</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cloudsearch:RESPONSE if the CloudSearch response cannot be parsed</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 1, "name" : "search-nondeterministic", "qname" : "cloudsearch:search-nondeterministic", "signature" : "($options as object()) as object()", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Searches the documents in the default Amazon CloudSearch domain.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">This function has the same semantics as <a href=\"#search-1\">search#1</a>,\n but is declared as being non deterministic and thus should only be used when\n result caching is not desired.</p>\n", "summary" : "<p>  Searches the documents in the default Amazon CloudSearch domain.</p>", "annotation_str" : " %an:nondeterministic", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "nondeterministic", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "options", "type" : "object()", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The <a href=\"#search-options\">search options</a> object</div>" } ], "returns" : { "type" : "object()", "description" : "The <a xmlns:xqdoc=\"http://www.xqdoc.org/1.0\" href=\"#search-results\">search results</a> object" }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cloudsearch:ENDPOINT if the default credentials endpoint is invalid or not present</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cloudsearch:AUTHORIZATION if the request was unauthorized</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cloudsearch:HTTP if an HTTP error has occurred</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cloudsearch:REQUEST if the request is malformed</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cloudsearch:INTERNAL if an internal server error occurred</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cloudsearch:RESPONSE if the CloudSearch response cannot be parsed</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 2, "name" : "search-nondeterministic", "qname" : "cloudsearch:search-nondeterministic", "signature" : "($endpoint as atomic, $options as object()) as object()", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Searches the documents in the specified Amazon CloudSearch domain.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">This function has the same semantics as <a href=\"#search-2\">search#2</a>,\n but is declared as being non deterministic and thus should only be used when\n result caching is not desired.</p>\n", "summary" : "<p>  Searches the documents in the specified Amazon CloudSearch domain.</p>", "annotation_str" : " %an:nondeterministic", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "nondeterministic", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "endpoint", "type" : "atomic", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The endpoint URI or the name of a stored CloudSearch credentials</div>" }, { "name" : "options", "type" : "object()", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The <a href=\"search-options\">search options</a> object</div>" } ], "returns" : { "type" : "object()", "description" : "The <a xmlns:xqdoc=\"http://www.xqdoc.org/1.0\" href=\"search-results\">search results</a> object" }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cloudsearch:ENDPOINT if the specified endpoint is invalid</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cloudsearch:AUTHORIZATION if the request was unauthorized</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cloudsearch:HTTP if an HTTP error has occurred</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cloudsearch:REQUEST if the request is malformed</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cloudsearch:INTERNAL if an internal server error occurred</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cloudsearch:RESPONSE if the CloudSearch response cannot be parsed</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 1, "name" : "search", "qname" : "cloudsearch:search", "signature" : "($options as object()) as object()", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Searches the documents in the default Amazon CloudSearch domain.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">This function is declared as deterministic and should be used whenever result\n caching is acceptable.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">The format of the <a href=\"search-options\">search options</a> object and\n of the <a href=\"search-results\">search results</a> object is documented in\n the module introduction.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Example:\n <pre>\n cloudsearch:search({\"q\" : \"star+wars\", \"return-fields\" : \"title\"})\n </pre>\n </p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Example returned object:\n <pre>\n {\n   \"rank\" : \"-text_relevance\",\n   \"match-expr\" : \"(label 'star wars')\",\n   \"hits\" :\n   {\n     \"found\" : 2,\n     \"start\" : 0,\n     \"hit\" :\n     [\n       {\n         \"id\" : \"tt1185834\",\n         \"data\" :\n         {\n           \"actor\" : [\"Abercrombie, Ian\",\"Baker, Dee\",\"Burton, Corey\"],\n           \"title\" : [\"Star Wars: The Clone Wars\"]\n         }\n       },\n       {\n         \"id\" :\"tt0121766\",\n         \"data\" :\n         {\n           \"actor\" : [\"Bai, Ling\",\"Bryant, Gene\",\"Castle-Hughes, Keisha\"],\n           \"title\" : [\"Star Wars: Episode III - Revenge of the Sith\"]\n         }\n       }\n     ]\n   },\n   \"info\" :\n   {\n     \"rid\" : \"b7c167f6c2da6d93531b9a7b314ad030b3a74803b4b7797edb905ba5a6a08\",\n     \"time-ms\" : 2,\n     \"cpu-time-ms\" : 0\n   }\n }\n </pre>\n </p>\n", "summary" : "<p>  Searches the documents in the default Amazon CloudSearch domain.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "options", "type" : "object()", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The <a href=\"search-options\">search options</a> object</div>" } ], "returns" : { "type" : "object()", "description" : "The <a xmlns:xqdoc=\"http://www.xqdoc.org/1.0\" href=\"search-results\">search results</a> object" }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cloudsearch:ENDPOINT if the default credentials endpoint is invalid or not present</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cloudsearch:AUTHORIZATION if the request was unauthorized</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cloudsearch:HTTP if an HTTP error has occurred</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cloudsearch:REQUEST if the request is malformed</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cloudsearch:INTERNAL if an internal server error occurred</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cloudsearch:RESPONSE if the CloudSearch response cannot be parsed</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 2, "name" : "search", "qname" : "cloudsearch:search", "signature" : "($endpoint as atomic, $options as object()) as object()", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Searches the documents in the specified Amazon CloudSearch domain.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">This function is declared as deterministic and should be used whenever result\n caching is acceptable.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">The format of the <a href=\"search-options\">search options</a> object and\n of the <a href=\"search-results\">search results</a> object is documented in\n the module introduction.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">The following examples performs a simple text search for all documents\n containing both the word \"star\" and \"trek\" and returns the matching document\n title field.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Example, using specified endpoint URI:\n <pre>\n cloudsearch:search(\n anyURI(\"http://doc-movies-h2pc7ftfnsdlqh6pqqawbftrhu.us-east-1.cloudsearch.amazonaws.com\"),\n {\"q\" : \"star+wars\", \"return-fields\" : \"title\"})\n </pre>\n </p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Example, using a stored endpoint:\n <pre>\n cloudsearch:search(\"movies\",\n {\"q\" : \"star+wars\", \"return-fields\" : \"title\"})\n </pre>\n </p>\n", "summary" : "<p>  Searches the documents in the specified Amazon CloudSearch domain.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "endpoint", "type" : "atomic", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The endpoint URI or the name of a stored CloudSearch credentials</div>" }, { "name" : "options", "type" : "object()", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The <a href=\"search-options\">search options</a> object</div>" } ], "returns" : { "type" : "object()", "description" : "The <a xmlns:xqdoc=\"http://www.xqdoc.org/1.0\" href=\"search-results\">search results</a> object" }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cloudsearch:ENDPOINT if the specified endpoint is invalid</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cloudsearch:AUTHORIZATION if the request was unauthorized</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cloudsearch:HTTP if an HTTP error has occurred</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cloudsearch:REQUEST if the request is malformed</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cloudsearch:INTERNAL if an internal server error occurred</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cloudsearch:RESPONSE if the CloudSearch response cannot be parsed</xqdoc:error>" ] } ], "variables" : [  ] }, "http://zorba.io/modules/store/static/indexes/ddl" : { "ns" : "http://zorba.io/modules/store/static/indexes/ddl", "description" : " This modules defines a set of functions for managing indexes that are\n declared in the prolog of a module.\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n This module is part of\n <a xmlns:xqdoc=\"http://www.xqdoc.org/1.0\" href=\"../zorba/xqddf.html\">Zorba's XQuery Data Definition Facility</a>.\n All the indexes managed by this module have to be pre-declared in the prolog\n of a module.\n Please refer to the\n <a xmlns:xqdoc=\"http://www.xqdoc.org/1.0\" href=\"../zorba/data_lifecycle.html\">general documentation</a>\n for more information and examples.\n", "sees" : [ "<xqdoc:see xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"><a href=\"../zorba/data_lifecycle.html\">Data Lifecycle</a></xqdoc:see>", "<xqdoc:see xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"><a href=\"../zorba/xqddf.html\">XQuery Data Definition Facility</a></xqdoc:see>", "<xqdoc:see xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">http://zorba.io/modules/store/static/indexes/dml</xqdoc:see>", "<xqdoc:see xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">http://zorba.io/modules/store/static/collections/dml</xqdoc:see>", "<xqdoc:see xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">http://zorba.io/modules/store/static/collections/ddl</xqdoc:see>", "<xqdoc:see xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">http://zorba.io/modules/store/static/integrity-constraints/ddl</xqdoc:see>", "<xqdoc:see xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">http://zorba.io/modules/store/static/integrity-constraints/dml</xqdoc:see>", "<xqdoc:see xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">http://zorba.io/errors</xqdoc:see>" ], "authors" : [ "<xqdoc:author xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Nicolae Brinza, Matthias Brantner, David Graf, Till Westmann, Markos Zaharioudakis</xqdoc:author>" ], "version" : null, "encoding" : "utf-8", "namespaces" : [ { "uri" : "http://zorba.io/modules/store/static/indexes/ddl", "prefix" : "iddl" }, { "uri" : "http://zorba.io/options/versioning", "prefix" : "ver" }, { "uri" : "http://zorba.io/errors", "prefix" : "zerr" } ], "functions" : [ { "isDocumented" : true, "arity" : 0, "name" : "available-indexes", "qname" : "iddl:available-indexes", "signature" : "() as xs:QName* external", "description" : " Gets the available indexes.\n", "summary" : "<p> Gets the available indexes.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [  ], "returns" : { "type" : "xs:QName*", "description" : "A sequence of QNames, one for each available index, or an empty sequence if none are." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "create", "qname" : "iddl:create", "signature" : "($name as xs:QName) external", "description" : " Creates an index.\n", "summary" : "<p> Creates an index.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : true, "parameters" : [ { "name" : "name", "type" : "xs:QName", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The name of the index to create.</div>" } ], "returns" : { "type" : null, "description" : "An empty XDM instance and a pending update list that, once applied, creates the index with the given name." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">zerr:ZDDY0021 if <code>$name</code> is not equal to the name of any resource in the statically known indexes.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">zerr:ZDDY0022 if an index with <code>$name</code> already exists.</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 0, "name" : "declared-indexes", "qname" : "iddl:declared-indexes", "signature" : "() as xs:QName* external", "description" : " Gets a sequence of QNames representing the indexes that have been declared\n in the prolog of the static context.\n", "summary" : "<p> Gets a sequence of QNames representing the indexes that have been declared\n in the prolog of the static context.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [  ], "returns" : { "type" : "xs:QName*", "description" : "A sequence of QNames, one for each created collection, or an emtpy sequence." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "delete", "qname" : "iddl:delete", "signature" : "($name as xs:QName) external", "description" : " Deletes an index.\n", "summary" : "<p> Deletes an index.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : true, "parameters" : [ { "name" : "name", "type" : "xs:QName", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The name of the index to delete.</div>" } ], "returns" : { "type" : null, "description" : "An empty XDM instance and a pending update list that, once applied, deletes the index with the given name." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">zerr:ZDDY0009 if the index does not exist.</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 1, "name" : "is-available-index", "qname" : "iddl:is-available-index", "signature" : "($name as xs:QName) as xs:boolean external", "description" : " Gets whether an index exists.\n", "summary" : "<p> Gets whether an index exists.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "name", "type" : "xs:QName", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The name of the index that is being checked.</div>" } ], "returns" : { "type" : "xs:boolean", "description" : "true if the index is available; false otherwise." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "is-declared-index", "qname" : "iddl:is-declared-index", "signature" : "($name as xs:QName) as xs:boolean external", "description" : " Gets whether an index has been declared in the prolog of the static context.\n", "summary" : "<p> Gets whether an index has been declared in the prolog of the static context.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "name", "type" : "xs:QName", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The name of the index that is being checked.</div>" } ], "returns" : { "type" : "xs:boolean", "description" : "true if the index was declared; false otherwise." }, "errors" : [  ] } ], "variables" : [  ] }, "http://zorba.io/modules/excel/datetime" : { "ns" : "http://zorba.io/modules/excel/datetime", "description" : " This is a library module offering the same set of functions\n defined by Microsoft Excel.\n", "sees" : [ "<xqdoc:see xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"><a href=\"http://office.microsoft.com/en-us/excel/CH062528231033.aspx\" target=\"_blank\">Excel Documentation: Datetime Functions</a></xqdoc:see>" ], "authors" : [ "<xqdoc:author xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Sorin Nasoi</xqdoc:author>" ], "version" : null, "encoding" : "utf-8", "namespaces" : [ { "uri" : "http://zorba.io/modules/excel/datetime", "prefix" : "excel-datetime" }, { "uri" : "http://zorba.io/modules/excel/errors", "prefix" : "excel-err" }, { "uri" : "http://zorba.io/modules/excel/text", "prefix" : "excel-text" }, { "uri" : "http://zorba.io/options/versioning", "prefix" : "ver" } ], "functions" : [ { "isDocumented" : true, "arity" : 3, "name" : "date", "qname" : "excel-datetime:date", "signature" : "($year as xs:integer, $month as xs:integer, $day as xs:integer) as xs:date?", "description" : " Constructs a date given the hours, months and days.\n", "summary" : "<p> Constructs a date given the hours, months and days.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "year", "type" : "xs:integer", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the year</div>" }, { "name" : "month", "type" : "xs:integer", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the month</div>" }, { "name" : "day", "type" : "xs:integer", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the day</div>" } ], "returns" : { "type" : "xs:date?", "description" : "A date given the hours, months and days" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "day", "qname" : "excel-datetime:day", "signature" : "($date as xs:date) as xs:integer?", "description" : " Returns the day of a $date, represented by a serial number.\n", "summary" : "<p> Returns the day of a $date, represented by a serial number.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "date", "type" : "xs:date", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the date.</div>" } ], "returns" : { "type" : "xs:integer?", "description" : "The day of a $date, represented by a serial number. The day is given as an integer ranging from 1 to 31." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 2, "name" : "days360", "qname" : "excel-datetime:days360", "signature" : "($start_date as xs:date, $end_date as xs:date) as xs:integer", "description" : " Returns the number of days between two dates based on a 360-day year.\n", "summary" : "<p> Returns the number of days between two dates based on a 360-day year.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "start_date", "type" : "xs:date", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the start date.</div>" }, { "name" : "end_date", "type" : "xs:date", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the end date.</div>" } ], "returns" : { "type" : "xs:integer", "description" : "The number of days between two dates based on a 360-day year (twelve 30-day months), which is used in some accounting calculations.<p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/> Use this function to help compute payments if your accounting system is based on twelve 30-day months.<p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/> The metod used is U.S. (NASD). If the starting date is the last day of a month, it becomes equal to the 30th of the same month. <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/> If the ending date is the last day of a month and the starting date is earlier than the 30th of a month, the ending date becomes equal to the 1st of the next month; otherwise the ending date becomes equal to the 30th of the same month." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 3, "name" : "days360", "qname" : "excel-datetime:days360", "signature" : "($start_date as xs:date, $end_date as xs:date, $method as xs:boolean) as xs:integer", "description" : " Returns the number of days between two dates based on a 360-day year.\n", "summary" : "<p> Returns the number of days between two dates based on a 360-day year.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "start_date", "type" : "xs:date", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the start date.</div>" }, { "name" : "end_date", "type" : "xs:date", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the end date.</div>" }, { "name" : "method", "type" : "xs:boolean", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> if false then US/NASD Method is used, otherwise the European Method is used.</div>" } ], "returns" : { "type" : "xs:integer", "description" : "The number of days between two dates based on a 360-day year (twelve 30-day months), which is used in some accounting calculations. Use this function to help compute payments if your accounting system is based on twelve 30-day months. <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/> The European Method (30E/360)<p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/> - If either date A or B falls on the 31st of the month, that date will be changed to the 30th;<p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/> - Where date B falls on the last day of February, the actual date B will be used.<p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/> The US/NASD Method (30US/360)<p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/> - If both date A and B fall on the last day of February, then date B will be changed to the 30th.<p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/> - If date A falls on the 31st of a month or last day of February, then date A will be changed to the 30th.<p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/> - If date A falls on the 30th of a month after applying (2) above and date B falls on the 31st of a month, then date B will be changed to the 30th." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "hour", "qname" : "excel-datetime:hour", "signature" : "($time as xs:time) as xs:integer?", "description" : " Returns the hour of a time value.\n", "summary" : "<p> Returns the hour of a time value.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "time", "type" : "xs:time", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the time.</div>" } ], "returns" : { "type" : "xs:integer?", "description" : "The hour of a time value. The hour is as an integer, ranging from 0 (12:00 A.M.) to 23 (11:00 P.M.)." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "minute", "qname" : "excel-datetime:minute", "signature" : "($time as xs:time) as xs:integer?", "description" : " Returns the minutes of a time value.\n", "summary" : "<p> Returns the minutes of a time value.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "time", "type" : "xs:time", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the time.</div>" } ], "returns" : { "type" : "xs:integer?", "description" : "The minutes of a time value. The minute is given as an integer, ranging from 0 to 59." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "month", "qname" : "excel-datetime:month", "signature" : "($date as xs:date) as xs:integer?", "description" : " Returns the month of a $date.\n", "summary" : "<p> Returns the month of a $date.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "date", "type" : "xs:date", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the date.</div>" } ], "returns" : { "type" : "xs:integer?", "description" : "the month of a $date. The month is given as an integer, ranging from 1 (January) to 12 (December)." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 2, "name" : "networkdays", "qname" : "excel-datetime:networkdays", "signature" : "($start_date as xs:date, $end_date as xs:date) as xs:integer*", "description" : " Returns the number of whole working days between $start_date and $end_date.\n", "summary" : "<p> Returns the number of whole working days between $start_date and $end_date.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "start_date", "type" : "xs:date", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the start date.</div>" }, { "name" : "end_date", "type" : "xs:date", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the end date.</div>" } ], "returns" : { "type" : "xs:integer*", "description" : "The number of whole working days between start_date and end_date.<p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/> Working days exclude weekends and any dates identified in holidays.<p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/> Use NETWORKDAYS to calculate employee benefits that accrue based on the number of days worked during a specific term." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 3, "name" : "networkdays", "qname" : "excel-datetime:networkdays", "signature" : "($start_date as xs:date, $end_date as xs:date, $holidays as xs:date*) as xs:integer*", "description" : " Returns the number of whole working days between $start_date and $end_date.\n", "summary" : "<p> Returns the number of whole working days between $start_date and $end_date.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "start_date", "type" : "xs:date", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the start date.</div>" }, { "name" : "end_date", "type" : "xs:date", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the end date.</div>" }, { "name" : "holidays", "type" : "xs:date", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> one or more dates to exclude from the working calendar, such as state and federal holidays and floating holidays.</div>" } ], "returns" : { "type" : "xs:integer*", "description" : "The number of whole working days between start_date and end_date.<p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/> Working days exclude weekends and any dates identified in holidays.<p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/> Use NETWORKDAYS to calculate employee benefits that accrue based on the number of days worked during a specific term." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 0, "name" : "now", "qname" : "excel-datetime:now", "signature" : "() as xs:dateTime?", "description" : " Returns the current date and time.\n", "summary" : "<p> Returns the current date and time.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [  ], "returns" : { "type" : "xs:dateTime?", "description" : "The current date and time." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "second", "qname" : "excel-datetime:second", "signature" : "($time as xs:time) as xs:decimal?", "description" : " Returns the seconds of a $time value.\n", "summary" : "<p> Returns the seconds of a $time value.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "time", "type" : "xs:time", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the time.</div>" } ], "returns" : { "type" : "xs:decimal?", "description" : "The seconds of a $time value. The second is given as an integer in the range 0 (zero) to 59." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 3, "name" : "time", "qname" : "excel-datetime:time", "signature" : "($hour as xs:integer, $minute as xs:integer, $second as xs:integer) as xs:time?", "description" : " Constructs a time given the hours, minutes and seconds.\n", "summary" : "<p> Constructs a time given the hours, minutes and seconds.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "hour", "type" : "xs:integer", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the hour.</div>" }, { "name" : "minute", "type" : "xs:integer", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the minute.</div>" }, { "name" : "second", "type" : "xs:integer", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the second.</div>" } ], "returns" : { "type" : "xs:time?", "description" : "A time given the hours, minutes and seconds." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 0, "name" : "today", "qname" : "excel-datetime:today", "signature" : "() as xs:date?", "description" : " Returns the current date.\n", "summary" : "<p> Returns the current date.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [  ], "returns" : { "type" : "xs:date?", "description" : "The current date." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "weekday", "qname" : "excel-datetime:weekday", "signature" : "($date as xs:date) as xs:integer?", "description" : " Returns the day of the week corresponding to a $date.\n", "summary" : "<p> Returns the day of the week corresponding to a $date.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "date", "type" : "xs:date", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the date.</div>" } ], "returns" : { "type" : "xs:integer?", "description" : "The day of the week corresponding to a $date. The day is given as an integer, ranging from 1 (Sunday) to 7 (Saturday)." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 2, "name" : "weekday", "qname" : "excel-datetime:weekday", "signature" : "($date as xs:date, $return_type as xs:integer) as xs:integer?", "description" : " Returns the day of the week corresponding to a $date depending on $return_type.\n", "summary" : "<p> Returns the day of the week corresponding to a $date depending on $return_type.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "date", "type" : "xs:date", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the date.</div>" }, { "name" : "return_type", "type" : "xs:integer", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> 1 for Numbers 1 (Sunday) through 7 (Saturday). 2 for Numbers 1 (Monday) through 7 (Sunday). 3 for Numbers 0 (Monday) through 6 (Sunday).</div>" } ], "returns" : { "type" : "xs:integer?", "description" : "The day of the week corresponding to a $date depending on $return_type." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">excel-err:Value if $return_type is outside the range [1,3].</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 1, "name" : "year", "qname" : "excel-datetime:year", "signature" : "($date as xs:date) as xs:integer?", "description" : " Returns the year corresponding to a date.\n", "summary" : "<p> Returns the year corresponding to a date.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "date", "type" : "xs:date", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the date.</div>" } ], "returns" : { "type" : "xs:integer?", "description" : "The year corresponding to a date." }, "errors" : [  ] } ], "variables" : [  ] }, "http://xbrl.io/modules/bizql/profiles/sec/filings" : { "ns" : "http://xbrl.io/modules/bizql/profiles/sec/filings", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">This module provides functionality for querying financial reports (filings)\n submitted to the SEC.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">SEC Filings are nothing else than XBRL archives. For XBRL-generic requests on archives,\n use the generic archives module.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">With this module, you can access a number of properties of an SEC filing, such as\n its document type. You can also retrieve an SEC filing given an SEC company (or its CIK).</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">You can also access SEC filings statistics (the number of SEC Networks, of SEC Axes, etc).</p>\n", "sees" : [  ], "authors" : [ "<xqdoc:author xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Charles Hoffman</xqdoc:author>", "<xqdoc:author xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Matthias Brantner</xqdoc:author>", "<xqdoc:author xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Dennis Knochenwefel</xqdoc:author>", "<xqdoc:author xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Ghislain Fourny</xqdoc:author>" ], "version" : null, "encoding" : "utf-8", "namespaces" : [ { "uri" : "http://xbrl.io/modules/bizql/archives", "prefix" : "archives" }, { "uri" : "http://xbrl.io/modules/bizql/profiles/sec/companies", "prefix" : "companies" }, { "uri" : "http://www.28msec.com/modules/credentials", "prefix" : "credentials" }, { "uri" : "http://zorba.io/modules/datetime", "prefix" : "datetime" }, { "uri" : "http://xbrl.io/modules/bizql/facts", "prefix" : "facts" }, { "uri" : "http://xbrl.io/modules/bizql/profiles/sec/filings", "prefix" : "filings" }, { "uri" : "http://www.28msec.com/modules/mongodb", "prefix" : "mongo" }, { "uri" : "http://xbrl.io/modules/bizql/profiles/sec/core", "prefix" : "sec" }, { "uri" : "http://zorba.io/options/versioning", "prefix" : "ver" } ], "functions" : [ { "isDocumented" : true, "arity" : 1, "name" : "acceptance-dateTimes", "qname" : "filings:acceptance-dateTimes", "signature" : "($filings-or-ids as item()*) as dateTime*", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Retrieves the acceptance date of filings.</p>\n", "summary" : "<p>  Retrieves the acceptance date of filings.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "filings-or-ids", "type" : "item()", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> a sequence of filings or their IDs.</div>" } ], "returns" : { "type" : "dateTime*", "description" : "the acceptance dateTimes." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "document-types", "qname" : "filings:document-types", "signature" : "($filings-or-ids as item()*) as string?", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Retrieves the document type of the filings.</p>\n", "summary" : "<p>  Retrieves the document type of the filings.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "filings-or-ids", "type" : "item()", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> a sequence of filings or their ids.</div>" } ], "returns" : { "type" : "string?", "description" : "the document types (10-K, 10-Q)." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "filing-dates", "qname" : "filings:filing-dates", "signature" : "($filings-or-ids as item()*) as date*", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Retrieves the submission date of filings.</p>\n", "summary" : "<p>  Retrieves the submission date of filings.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "filings-or-ids", "type" : "item()", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> a sequence of filings or their IDs.</div>" } ], "returns" : { "type" : "date*", "description" : "the submission dates." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "filings-for-companies", "qname" : "filings:filings-for-companies", "signature" : "($companies-or-ciks as item()*) as object()*", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Return all filings submitted by the supplied companies.</p>\n", "summary" : "<p>  Return all filings submitted by the supplied companies.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "companies-or-ciks", "type" : "item()", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> arbitrary number of company objects or CIKs.</div>" } ], "returns" : { "type" : "object()*", "description" : "all filings submitted by these companies." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "generators", "qname" : "filings:generators", "signature" : "($filings-or-ids as item()*) as string*", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Retrieves the generators of filings.</p>\n", "summary" : "<p>  Retrieves the generators of filings.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "filings-or-ids", "type" : "item()", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> a sequence of filings or their IDs.</div>" } ], "returns" : { "type" : "string*", "description" : "the generators used." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "num-abstracts", "qname" : "filings:num-abstracts", "signature" : "($filings-or-ids) as integer*", "description" : " Return the number of (distinct) abstracts in each of the given filings.\n", "summary" : "<p> Return the number of (distinct) abstracts in each of the given filings.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "filings-or-ids", "type" : null, "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> list of filings or IDs</div>" } ], "returns" : { "type" : "integer*", "description" : "the said number of abstracts" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "num-axes", "qname" : "filings:num-axes", "signature" : "($filings-or-ids) as integer*", "description" : " Return the number of (distinct) axes in each of the given filings.\n", "summary" : "<p> Return the number of (distinct) axes in each of the given filings.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "filings-or-ids", "type" : null, "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> list of filings or IDs</div>" } ], "returns" : { "type" : "integer*", "description" : "the said number of axes" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "num-concepts", "qname" : "filings:num-concepts", "signature" : "($filings-or-ids) as integer*", "description" : " Return the number of (distinct) concepts in each of the given filings.\n", "summary" : "<p> Return the number of (distinct) concepts in each of the given filings.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "filings-or-ids", "type" : null, "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> list of filings or IDs</div>" } ], "returns" : { "type" : "integer*", "description" : "the said number of concepts" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "num-extension-abstracts", "qname" : "filings:num-extension-abstracts", "signature" : "($filings-or-ids) as integer*", "description" : " Return the number of extension abstracts in each of the given filings.\n", "summary" : "<p> Return the number of extension abstracts in each of the given filings.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "filings-or-ids", "type" : null, "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> list of filings or IDs</div>" } ], "returns" : { "type" : "integer*", "description" : "the said number of abstracts" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "num-extension-concepts", "qname" : "filings:num-extension-concepts", "signature" : "($filings-or-ids) as integer*", "description" : " Return the number of extension concepts in each of the given filings.\n", "summary" : "<p> Return the number of extension concepts in each of the given filings.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "filings-or-ids", "type" : null, "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> list of filings or IDs</div>" } ], "returns" : { "type" : "integer*", "description" : "the said number of concepts" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "num-extension-facts", "qname" : "filings:num-extension-facts", "signature" : "($filings-or-ids) as integer*", "description" : " Return the number of extension facts in each of the given filings.\n", "summary" : "<p> Return the number of extension facts in each of the given filings.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "filings-or-ids", "type" : null, "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> list of filings or IDs</div>" } ], "returns" : { "type" : "integer*", "description" : "the said number of facts" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "num-facts", "qname" : "filings:num-facts", "signature" : "($filings-or-ids) as integer*", "description" : " Return the number of facts in each of the given filings.\n", "summary" : "<p> Return the number of facts in each of the given filings.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "filings-or-ids", "type" : null, "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> list of filings or IDs</div>" } ], "returns" : { "type" : "integer*", "description" : "the said number of facts" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "num-footnotes", "qname" : "filings:num-footnotes", "signature" : "($filings-or-ids) as integer*", "description" : " Return the number of footnotes in each of the given filings.\n", "summary" : "<p> Return the number of footnotes in each of the given filings.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "filings-or-ids", "type" : null, "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> list of filings or IDs</div>" } ], "returns" : { "type" : "integer*", "description" : "the said number of facts" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "num-line-items", "qname" : "filings:num-line-items", "signature" : "($filings-or-ids) as integer*", "description" : " Return the number of (distinct) line items in each of the given filings.\n", "summary" : "<p> Return the number of (distinct) line items in each of the given filings.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "filings-or-ids", "type" : null, "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> list of filings or IDs</div>" } ], "returns" : { "type" : "integer*", "description" : "the said number of line items" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "num-members", "qname" : "filings:num-members", "signature" : "($filings-or-ids) as integer*", "description" : " Return the number of (distinct) members in each of the given filings.\n", "summary" : "<p> Return the number of (distinct) members in each of the given filings.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "filings-or-ids", "type" : null, "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> list of filings or IDs</div>" } ], "returns" : { "type" : "integer*", "description" : "the said number of members" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "num-networks", "qname" : "filings:num-networks", "signature" : "($filings-or-ids) as integer*", "description" : " Return the number of networks in each of the given filings.\n", "summary" : "<p> Return the number of networks in each of the given filings.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "filings-or-ids", "type" : null, "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> list of filings or IDs</div>" } ], "returns" : { "type" : "integer*", "description" : "the said number of networks" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "num-report-elements", "qname" : "filings:num-report-elements", "signature" : "($filings-or-ids) as integer*", "description" : " Return the number of (distinct) report elements in each of the given filings.\n", "summary" : "<p> Return the number of (distinct) report elements in each of the given filings.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "filings-or-ids", "type" : null, "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> list of filings or IDs</div>" } ], "returns" : { "type" : "integer*", "description" : "the said number of report elements" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "num-tables", "qname" : "filings:num-tables", "signature" : "($filings-or-ids) as integer*", "description" : " Return the number of tables in each of the given filings.\n", "summary" : "<p> Return the number of tables in each of the given filings.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "filings-or-ids", "type" : null, "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> list of filings or IDs</div>" } ], "returns" : { "type" : "integer*", "description" : "the said number of tables" }, "errors" : [  ] } ], "variables" : [  ] }, "http://zorba.io/modules/string" : { "ns" : "http://zorba.io/modules/string", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">This module provides string related functions.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Mostly, the functions in this module provide primitives\n to work with streamable strings. For example, it allows to\n check whether a given string is streamable or seekable.</p>\n", "sees" : [  ], "authors" : [ "<xqdoc:author xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Matthias Brantner</xqdoc:author>" ], "version" : null, "encoding" : "utf-8", "namespaces" : [ { "uri" : "http://zorba.io/modules/string", "prefix" : "string" }, { "uri" : "http://zorba.io/options/versioning", "prefix" : "ver" } ], "functions" : [ { "isDocumented" : true, "arity" : 2, "name" : "analyze-string", "qname" : "string:analyze-string", "signature" : "($input as xs:string?, $pattern as xs:string) as array()", "description" : " Analyzes a string using a regular expression, returning sequence of JSON\n objects that identify which parts of the input string matched or failed to\n match the regular expression; and in the case of matched substrings, which\n substrings matched each capturing group in the regular expression.\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n This function behaves like\n <a xmlns:xqdoc=\"http://www.xqdoc.org/1.0\" href=\"http://www.w3.org/TR/xpath-functions-30/#func-analyze-string\"><code>fn:analyze-string</code></a>\n but returns a JSON array rather than an XML element.\n", "summary" : "<p> Analyzes a string using a regular expression, returning sequence of JSON\n objects that identify which parts of the input string matched or failed to\n match the regular expression; and in the case of matched substrings, which\n substrings matched each capturing group in the regular expression.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "input", "type" : "xs:string", "occurrence" : "?", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The string to analyze. If the empty sequence, the function behaves as if <code>$input</code> were a zero-length string.</div>" }, { "name" : "pattern", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The regular expression.</div>" } ], "returns" : { "type" : "array()", "description" : "a JSON array of objects where each object contains a single key/value pair. Each key is either <code xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">match</code> or <code xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">non-match</code>. For <code xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">non-match</code>, the value is a string that is the part of <code xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">$input</code> that did not match; for <code xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">match</code>, the value is either a string that is the part of <code xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">$input</code> that matched (when <code xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">$pattern</code> contains no capturing groups) or an array containing values for both capturing groups and other matches. <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/> Capturing group matches are themselves arrays where the first element is the group number (1-based) and subsequent elements are either a string that is the part of <code xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">$input</code> that matched or sub-arrays for nested capturing groups." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 3, "name" : "analyze-string", "qname" : "string:analyze-string", "signature" : "($input as xs:string?, $pattern as xs:string, $flags as xs:string) as array() external", "description" : " Analyzes a string using a regular expression, returning sequence of JSON\n objects that identify which parts of the input string matched or failed to\n match the regular expression; and in the case of matched substrings, which\n substrings matched each capturing group in the regular expression.\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n This function behaves like\n <a xmlns:xqdoc=\"http://www.xqdoc.org/1.0\" href=\"http://www.w3.org/TR/xpath-functions-30/#func-analyze-string\"><code>fn:analyze-string</code></a>\n but returns a JSON array rather than an XML element.\n", "summary" : "<p> Analyzes a string using a regular expression, returning sequence of JSON\n objects that identify which parts of the input string matched or failed to\n match the regular expression; and in the case of matched substrings, which\n substrings matched each capturing group in the regular expression.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "input", "type" : "xs:string", "occurrence" : "?", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The string to analyze. If the empty sequence, the function behaves as if <code>$input</code> were a zero-length string.</div>" }, { "name" : "pattern", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The regular expression.</div>" }, { "name" : "flags", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The  argument is interpreted in the same way as for the <a href=\"http://www.w3.org/TR/xpath-functions-30/#func-matches\"><code>fn:matches</code></a> function.</div>" } ], "returns" : { "type" : "array()", "description" : "a JSON array of objects where each object contains a single key/value pair. Each key is either <code xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">match</code> or <code xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">non-match</code>. For <code xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">non-match</code>, the value is a string that is the part of <code xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">$input</code> that did not match; for <code xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">match</code>, the value is either a string that is the part of <code xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">$input</code> that matched (when <code xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">$pattern</code> contains no capturing groups) or an array containing values for both capturing groups and other matches. <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/> Capturing group matches are themselves arrays where the first element is the group number (1-based) and subsequent elements are either a string that is the part of <code xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">$input</code> that matched or sub-arrays for nested capturing groups." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "is-seekable", "qname" : "string:is-seekable", "signature" : "($s as string) as boolean external", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">This function checks whether a given string item is a\n seekable stream string.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">For example, a seekable streamable string is returned by the\n file module.</p>\n", "summary" : "<p>  This function checks whether a given string item is a\n seekable stream string.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "s", "type" : "string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the string to check</div>" } ], "returns" : { "type" : "boolean", "description" : "true if the given item is a seekable stream string or false otherwise." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "is-streamable", "qname" : "string:is-streamable", "signature" : "($s as string) as boolean external", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">This function checks whether a given string item is implemented by a\n streamable string.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">A streamable string is produced by some functions of a module.\n It's an optimized implementation of an string to handle arbitrary\n sized data. The drawback is that its value can only be consumed once.\n That is, only one function can access the value of a streamable string\n item.</p>\n", "summary" : "<p>  This function checks whether a given string item is implemented by a\n streamable string.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "s", "type" : "string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the string to check</div>" } ], "returns" : { "type" : "boolean", "description" : "true if the given item is implemented using a streamable string or false otherwise." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "materialize", "qname" : "string:materialize", "signature" : "($s as string) as string external", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">This function materializes a streamable string.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">The drawback of a streamable (non-seekable) string is that\n its value can only be consumed once. That is, only one function\n can access the value of a streamable string item.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">In order to remedy this situation, this function can be used to\n convert a streamable string into its non-streamable counterpart. As a\n result, the string returned by this function has the same value as its\n input but is materialized and, hence, can be consumed multiple times.</p>\n", "summary" : "<p>  This function materializes a streamable string.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "s", "type" : "string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the streamable string item to materialize</div>" } ], "returns" : { "type" : "string", "description" : "a materialized string of its input or the input if the input item was not a streamable string." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 2, "name" : "split", "qname" : "string:split", "signature" : "($s as string, $separator as string) as string* external", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Returns a sequence of strings constructed by splitting the input wherever\n the given separator is found.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">The function is different from tokenize. It doesn't allow\n the separator to be a regular expression. This restriction allows for more\n performant implementation. Specifically, the function processes\n streamable strings as input in a streamable way which is particularly useful\n to tokenize huge strings.</p>\n", "summary" : "<p>  Returns a sequence of strings constructed by splitting the input wherever\n the given separator is found.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "s", "type" : "string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the input string to split</div>" }, { "name" : "separator", "type" : "string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the separator used for splitting the input string $s</div>" } ], "returns" : { "type" : "string*", "description" : "a sequence of strings constructed by splitting the input" }, "errors" : [  ] } ], "variables" : [  ] }, "http://www.28msec.com/modules/store" : { "ns" : "http://www.28msec.com/modules/store", "description" : " This module provides functions to perform MongoDB query operations\n (i.e. find and aggregate) on the database associated with the\n project.\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">The module is always imported so you don't need to import it explicitly.\n Also, you don't need to fully qualify the function if you want to invoke it.</p>\n", "sees" : [  ], "authors" : [ "<xqdoc:author xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">28msec</xqdoc:author>" ], "version" : null, "encoding" : "utf-8", "namespaces" : [ { "uri" : "http://zorba.io/annotations", "prefix" : "an" }, { "uri" : "http://www.28msec.com/modules/mongodb/types", "prefix" : "mongo" }, { "uri" : "http://www.28msec.com/modules/store", "prefix" : "store" }, { "uri" : "http://zorba.io/options/versioning", "prefix" : "ver" } ], "functions" : [ { "isDocumented" : true, "arity" : 2, "name" : "aggregate", "qname" : "store:aggregate", "signature" : "($collection as string, $pipeline as array()) as object() external", "description" : " Performs a MongoDB aggregation framework job on the given collection.\n The <em xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">$pipeline</em> parameter needs to specify a valid aggregation\n framework pipeline. For example,\n <pre xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">\n  [\n    { $project : {\n       author : 1,\n       tags : 1,\n    } },\n    { $unwind : \"$tags\" },\n    { $group : {\n       _id : { tags : \"$tags\" },\n       authors : { $addToSet : \"$author\" }\n    } }\n ]\n </pre>\n The function returns the result as one object. The object contains\n the field named \"ok\" with value 0 if the execution failed or 1 if\n it succeeded. If it succeeded, the result is contained as an array\n in the result field. Otherwise, the errmsg field contains the description\n of the error. For exampl,e\n <pre xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">\n {\n   \"result\" : [ ... ],\n   \"ok\" : 1\n }\n </pre>\n", "summary" : "<p> Performs a MongoDB aggregation framework job on the given collection.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "collection", "type" : "string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the name of the collection to execute the job on</div>" }, { "name" : "pipeline", "type" : "array()", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the specification of the pipeline to execute</div>" } ], "returns" : { "type" : "object()", "description" : "an object with the result or the error message." }, "errors" : [  ] }, { "isDocumented" : false, "arity" : 1, "name" : "clear-if", "qname" : "store:clear-if", "signature" : "($cond as boolean) as integer", "description" : "", "summary" : "", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "cond", "type" : "boolean", "occurrence" : null, "description" : "" } ], "returns" : { "type" : "integer", "description" : "" }, "errors" : [  ] }, { "isDocumented" : false, "arity" : 0, "name" : "clear", "qname" : "store:clear", "signature" : "() as integer external", "description" : "", "summary" : "", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [  ], "returns" : { "type" : "integer", "description" : "" }, "errors" : [  ] }, { "isDocumented" : false, "arity" : 1, "name" : "collection-chunk-specs", "qname" : "store:collection-chunk-specs", "signature" : "($name as string) as object()*", "description" : "", "summary" : "", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "name", "type" : "string", "occurrence" : null, "description" : "" } ], "returns" : { "type" : "object()*", "description" : "" }, "errors" : [  ] }, { "isDocumented" : false, "arity" : 2, "name" : "collection-chunk-specs", "qname" : "store:collection-chunk-specs", "signature" : "($name as string, $chunk-size as integer) as object()* external", "description" : "", "summary" : "", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "name", "type" : "string", "occurrence" : null, "description" : "" }, { "name" : "chunk-size", "type" : "integer", "occurrence" : null, "description" : "" } ], "returns" : { "type" : "object()*", "description" : "" }, "errors" : [  ] }, { "isDocumented" : false, "arity" : 2, "name" : "collection-chunk", "qname" : "store:collection-chunk", "signature" : "($name as string, $chunk as object()) as item()* external", "description" : "", "summary" : "", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "name", "type" : "string", "occurrence" : null, "description" : "" }, { "name" : "chunk", "type" : "object()", "occurrence" : null, "description" : "" } ], "returns" : { "type" : "item()*", "description" : "" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 2, "name" : "find", "qname" : "store:find", "signature" : "($coll as string, $query as object()) as object()*", "description" : " Performs a MongoDB query operation on the given collection and\n returns all matches.\n", "summary" : "<p> Performs a MongoDB query operation on the given collection and\n returns all matches.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "coll", "type" : "string", "occurrence" : null, "description" : "" }, { "name" : "query", "type" : "object()", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the query operation to perform</div>" } ], "returns" : { "type" : "object()*", "description" : "all matches returned by the given query operation" }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">zerr:ZSTR0009 if the given collection does not exist</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 3, "name" : "find", "qname" : "store:find", "signature" : "($coll as string, $query as object(), $options as object()) as object()* external", "description" : " Performs a MongoDB query operation on the given collection and\n returns all matches.\n Available options:\n <ul xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">\n   <li>to-return: the maximum number of objects to return (0 = unlimited)</li>\n   <li>to-skip: start with the n-th object</li>\n   <li>batch-size: the number of objects to return in one batch</li>\n   <li>slave-ok: allow this query to be run against a replica secondary</li>\n   <li>await-data: the server will block for some extra time before returning,\n   waiting for more data to return</li>\n   <li>partial-results: return partial results if some shards are down instead\n   of returning an error</li>\n </ul>\n", "summary" : "<p> Performs a MongoDB query operation on the given collection and\n returns all matches.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "coll", "type" : "string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the name of the collection</div>" }, { "name" : "query", "type" : "object()", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the query operation to perform</div>" }, { "name" : "options", "type" : "object()", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the options for this operation</div>" } ], "returns" : { "type" : "object()*", "description" : "all matches returned by the given query operation" }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">zerr:ZSTR0009 if the given collection does not exist</xqdoc:error>" ] }, { "isDocumented" : false, "arity" : 1, "name" : "flush-if", "qname" : "store:flush-if", "signature" : "($cond as boolean) as empty-sequence()", "description" : "", "summary" : "", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "cond", "type" : "boolean", "occurrence" : null, "description" : "" } ], "returns" : { "type" : "empty-sequence()", "description" : "" }, "errors" : [  ] }, { "isDocumented" : false, "arity" : 0, "name" : "flush", "qname" : "store:flush", "signature" : "() as empty-sequence() external", "description" : "", "summary" : "", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [  ], "returns" : { "type" : "empty-sequence()", "description" : "" }, "errors" : [  ] } ], "variables" : [  ] }, "http://zorba.io/modules/hash" : { "ns" : "http://zorba.io/modules/hash", "description" : " This module provides functions that perform different hash operations.\n For example, they compute MD5 and various SHA functions on either\n strings or binary. The result is the base64 encoded value of the hash.\n", "sees" : [  ], "authors" : [ "<xqdoc:author xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Gabriel Petrovay, Markus Pilman, Matthias Brantner</xqdoc:author>" ], "version" : null, "encoding" : "utf-8", "namespaces" : [ { "uri" : "http://zorba.io/modules/hash", "prefix" : "hash" }, { "uri" : "http://zorba.io/options/versioning", "prefix" : "ver" } ], "functions" : [ { "isDocumented" : true, "arity" : 2, "name" : "hash-binary", "qname" : "hash:hash-binary", "signature" : "($value as xs:base64Binary, $alg as xs:string) as xs:base64Binary external", "description" : " This function computes a hash value of the binary form of the given\n base64Binary item, i.e. the item is base64-decoded before hashing.\n", "summary" : "<p> This function computes a hash value of the binary form of the given\n base64Binary item, i.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "value", "type" : "xs:base64Binary", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The binary item to be hashed.</div>" }, { "name" : "alg", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The algorithm to use for this hashing operation. Supported algorithms are \"md5\", \"sha1\", and \"sha256\".</div>" } ], "returns" : { "type" : "xs:base64Binary", "description" : "The hash as xs:base64Binary of the provided binary" }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">hash:UNSUPPORTED-ALGORITHM if the given hash algorithm is not supported</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 2, "name" : "hash", "qname" : "hash:hash", "signature" : "($value as xs:string, $alg as xs:string) as xs:base64Binary external", "description" : " This function computes a hash value of the string provided as parameter.\n The function expects the hash algorithm to be used as parameter.\n", "summary" : "<p> This function computes a hash value of the string provided as parameter.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "value", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The string to be hashed.</div>" }, { "name" : "alg", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The algorithm to use for this hashing operation. Supported algorithms are \"md5\", \"sha1\", and \"sha256\".</div>" } ], "returns" : { "type" : "xs:base64Binary", "description" : "The hash as xs:base64binary of the provided string" }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">hash:UNSUPPORTED-ALGORITHM if the given hash algorithm is not supported</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 1, "name" : "md5-binary", "qname" : "hash:md5-binary", "signature" : "($value as xs:base64Binary) as xs:base64Binary", "description" : " This function computes the MD5 hash value of the binary form of the given\n base64Binary item, i.e. the item is base64-decoded before hashing.\n", "summary" : "<p> This function computes the MD5 hash value of the binary form of the given\n base64Binary item, i.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "value", "type" : "xs:base64Binary", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The binary item to hash.</div>" } ], "returns" : { "type" : "xs:base64Binary", "description" : "The MD5 hash of the provided binary." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "md5", "qname" : "hash:md5", "signature" : "($value as xs:string) as xs:base64Binary", "description" : " Computes the MD5 hash of the string provided as parameter.\n", "summary" : "<p> Computes the MD5 hash of the string provided as parameter.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "value", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The string to hash</div>" } ], "returns" : { "type" : "xs:base64Binary", "description" : "The MD5 hash as xs:base64Binary" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "sha1-binary", "qname" : "hash:sha1-binary", "signature" : "($value as xs:base64Binary) as xs:base64Binary", "description" : " This function computes the SHA1 hash value of the binary form of the given\n base64Binary item, i.e. the item is base64-decoded before hashing.\n", "summary" : "<p> This function computes the SHA1 hash value of the binary form of the given\n base64Binary item, i.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "value", "type" : "xs:base64Binary", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The binary item to hash.</div>" } ], "returns" : { "type" : "xs:base64Binary", "description" : "The base64 encoded SHA1 hash of the provided binary." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "sha1", "qname" : "hash:sha1", "signature" : "($value as xs:string) as xs:base64Binary", "description" : " Computes the SHA1 hash of the string provided as parameter.\n", "summary" : "<p> Computes the SHA1 hash of the string provided as parameter.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "value", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The string to hash.</div>" } ], "returns" : { "type" : "xs:base64Binary", "description" : "The SHA1 hash as xs:base64Binary" }, "errors" : [  ] } ], "variables" : [  ] }, "http://zorba.io/modules/xsl-fo" : { "ns" : "http://zorba.io/modules/xsl-fo", "description" : " This module converts <a xmlns:xqdoc=\"http://www.xqdoc.org/1.0\" href=\"http://www.w3schools.com/xslfo/default.asp\">XSL-FO</a> documents\n to various formats such as PDF, EPS, PCL, AFP, Text, PNG, Postscript, RTF, and TIFF.\n For instance, the following example converts a simple XSL-FO document to PDF:\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n <pre xmlns:xqdoc=\"http://www.xqdoc.org/1.0\" class=\"ace-static\" ace-mode=\"xquery\">import module namespace fop = \"http://zorba.io/modules/xsl-fo\";\n import module namespace file = \"http://expath.org/ns/file\";\n declare namespace fo = \"http://www.w3.org/1999/XSL/Format\";\n let $xsl-fo := &lt;fo:root xmlns:fo=\"http://www.w3.org/1999/XSL/Format\"&gt;\n   &lt;fo:layout-master-set&gt;\n     &lt;fo:simple-page-master master-name=\"my-page\"&gt;\n       &lt;fo:region-body margin=\"1in\"/&gt;\n     &lt;/fo:simple-page-master&gt;\n   &lt;/fo:layout-master-set&gt;\n   &lt;fo:page-sequence master-reference=\"my-page\"&gt;\n     &lt;fo:flow flow-name=\"xsl-region-body\"&gt;\n       &lt;fo:block&gt;Hello, world!&lt;/fo:block&gt;\n     &lt;/fo:flow&gt;\n   &lt;/fo:page-sequence&gt;\n  &lt;/fo:root&gt;\n let $pdf := fop:generator($fop:PDF, $xsl-fo)\n return file:write-binary(\"simple.pdf\", $pdf)</pre>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n This module uses Apache-FOP to generate content from an XSL-FO document.\n See <a xmlns:xqdoc=\"http://www.xqdoc.org/1.0\" href=\"http://xmlgraphics.apache.org/fop/\">the Apache FOP documentation</a> for further information.\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n <b xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Note for Windows users</b>: On Windows, this module won't work out of the box, since\n this module uses Java. But the Java VM dll is not in the system path by default. To make\n this module work, you need to add the directory where the jvm.dll is located to the\n system path. This dll is located at JRE_DIR\\bin\\client. On a standard installation, this would\n be something a path like \"C:\\Program Files\\Java\\jre6\\bin\\client\".\n", "sees" : [ "<xqdoc:see xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">http://xmlgraphics.apache.org/fop/</xqdoc:see>" ], "authors" : [ "<xqdoc:author xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Markus Pilman</xqdoc:author>" ], "version" : null, "encoding" : "utf-8", "namespaces" : [ { "uri" : "http://www.w3.org/2005/xqt-errors", "prefix" : "err" }, { "uri" : "http://www.zorba-xquery.com/modules/util-jvm", "prefix" : "util-jvm" }, { "uri" : "http://zorba.io/options/versioning", "prefix" : "ver" }, { "uri" : "http://zorba.io/modules/xsl-fo", "prefix" : "xsl-fo" } ], "functions" : [ { "isDocumented" : true, "arity" : 2, "name" : "generator", "qname" : "xsl-fo:generator", "signature" : "($output-format as xs:string, $xsl-fo-document as node()) as xs:base64Binary", "description" : " The generator function takes an XSL-FO document as input and generates output\n in the format given as input.\n The output format can be given as a MIME type - for example \"application/pdf\"\n - or one of the predefined variables can be used - like $xsl-fo:PDF. Please\n refer to the Apache FOP documentation for\n <a xmlns:xqdoc=\"http://www.xqdoc.org/1.0\" href=\"http://xmlgraphics.apache.org/fop/0.95/output.html\">supported output formats</a>.\n Apache FOP does not support 100% of the XSL-FO standard.\n Please consult the <a xmlns:xqdoc=\"http://www.xqdoc.org/1.0\" href=\"http://xmlgraphics.apache.org/fop/\">official\n documentation for further information</a>.\n This function tries to find the needed Java libraries itself.<p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n On a Mac OS X computer, it should be sufficient to install Apache FOP via Mac\n Ports.<p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n On Ubuntu it should be sufficient to install the fop packages via apt-get.<p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n On Windows, the classpath needs to be set manually using\n <a xmlns:xqdoc=\"http://www.xqdoc.org/1.0\" href=\"#generator#3\">generator#3</a>.<p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n This function tries to find the jar files via environment variables. The user can set the\n variable FOP_HOME to the root directory of an Apache FOP distribution. If you have all\n JAR files in the same directory, you can set the environment variable FOP_LIB_DIR to this\n directory.\n", "summary" : "<p> The generator function takes an XSL-FO document as input and generates output\n in the format given as input.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "output-format", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The mime of the output format, to tell Apache FOP which kind of document it should create.</div>" }, { "name" : "xsl-fo-document", "type" : "node()", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The XSL-FO document from which the output should be generated.</div>" } ], "returns" : { "type" : "xs:base64Binary", "description" : "The generated output document." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">xsl-fo:JVM-NOT-STARTED If zorba was unable to start the JVM.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">xsl-fo:JAVA-EXCEPTION If Apache FOP throws an exception - i.e. if the input format is not correct/supported.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">xsl-fo:JAR-NOT-FOUND If a needed Java library could not be found.</xqdoc:error>" ] } ], "variables" : [ { "name" : "xsl-fo:AFP", "type" : "xs:string", "description" : " The mime type of IBMs AFP format (application/x-afp).\n" }, { "name" : "xsl-fo:EPS", "type" : "xs:string", "description" : " The mime type of the EPS format (application/postscript).\n" }, { "name" : "xsl-fo:PCL", "type" : "xs:string", "description" : " The mime type of the PCL format (application/x-pcl).\n" }, { "name" : "xsl-fo:PDF", "type" : "xs:string", "description" : " The mime type of the PDF format (application/pdf).\n" }, { "name" : "xsl-fo:PLAIN_TEXT", "type" : "xs:string", "description" : " The mime type for plain text files (text/plain).\n" }, { "name" : "xsl-fo:PNG", "type" : "xs:string", "description" : " The mime type of the PNG format (image/png).\n" }, { "name" : "xsl-fo:POSTSCRIPT", "type" : "xs:string", "description" : " The mime type of the postscript format (application/postscript).\n" }, { "name" : "xsl-fo:RTF", "type" : "xs:string", "description" : " The mime type of the RTF format (application/rtf).\n" }, { "name" : "xsl-fo:TIFF", "type" : "xs:string", "description" : " The mime type of TIFF format (application/tiff).\n" } ] }, "http://zorba.io/warnings" : { "ns" : "http://zorba.io/warnings", "description" : " This module contains one variable declaration for each diagnostic of the\n http://zorba.io/warnings namespace.\n The variables serves as documentation for the errors but can also\n be used in the code. For example, one useful scenario is to compare\n an error caught in the catch clause of a try-catch expression with one of\n the variables.\n", "sees" : [  ], "authors" : [ "<xqdoc:author xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Carlos Lopez</xqdoc:author>" ], "version" : null, "encoding" : "utf-8", "namespaces" : [ { "uri" : "http://zorba.io/warnings", "prefix" : "zwarn" } ], "functions" : [  ], "variables" : [ { "name" : "zwarn:NS", "type" : "item()*", "description" : "" }, { "name" : "zwarn:ZWST0002", "type" : "xs:QName", "description" : " This warning is reported if the declaration of a function, variable,\n collection, or index contains an annotation that is not in the\n http://zorba.io/annotations namespace and Zorba doesn't know how to\n handle.\n" }, { "name" : "zwarn:ZWST0003", "type" : "item()*", "description" : "" }, { "name" : "zwarn:ZWST0004", "type" : "item()*", "description" : "" }, { "name" : "zwarn:ZWST0005", "type" : "xs:QName", "description" : " This warning is raised if the user explicitly enables caching\n of function results (using the %an:cache or %an:strictlydeterministic\n annotation) but the function cannot be cached.\n For %an:cache this happens if the function is updating or variadic.\n For %an:strictlydeterministic this happens if the function is updating,\n variadic or sequential.\n" }, { "name" : "zwarn:ZWST0006", "type" : "xs:QName", "description" : " This warning is raised if the user explicitly enables caching\n of function results (using the %an:cache annotation) and the function\n is annotated as sequential or nondeterministic.\n" }, { "name" : "zwarn:ZWST0007", "type" : "item()*", "description" : "" }, { "name" : "zwarn:ZWST0008", "type" : "item()*", "description" : "" }, { "name" : "zwarn:ZWST0009", "type" : "xs:QName", "description" : " This warning is enabled when the \"common-language\" option is employed. It will raise warnings\n for language features that are not supported by both XQuery and JSONiq grammars.\n" } ] }, "http://www.28msec.com/modules/sparql" : { "ns" : "http://www.28msec.com/modules/sparql", "description" : " This module contains functions to interact with a SPARQL endpoint.\n Requests to an endpoint are made using HTTP.\n Specifically, this module allows SELECT, ASK, CONSTRUCT and DESCRIBE queries. Additionally, it also allows for UPDATE statements.\n Various result formats are supported (e.g. XML, JSON, CSV)\n <h2 xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Configuration</h2>\n For this module you may use a preconfigured default or named datasource of the SPARQL category.\n You can also provide a configuration object directly.\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">As config parameter you can either pass a string that will be interpreted as a SPARQL endpoint datasource name or\n  an object with the following properties:\n <ul>\n  <li><tt>href</tt>: The URL of the SPARQL endpoint to use. Required.</li>\n  <li><tt>auth_method</tt>: The HTTP authentication method to use.</li>\n  <li><tt>username</tt>: The username to use for HTTP authentication.</li>\n  <li><tt>password</tt>: The password to use for HTTP authentication.</li>\n </ul>\n </p>\n <h2 xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Options</h2>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">The options object that can be passed to plenty of the functions in this module. It may have the following properties:\n <ul>\n   <li><tt>default-graph-uri</tt>: A string or an array of strings with the graph URIs to be used by the query.</li>\n   <li><tt>named-graph-uri</tt>: A string or an array of strings with the named graph URIs to be used by the query.</li>\n   <li><tt>using-graph-uri</tt>: A string or an array of strings with the graph URIs to be used by an update statement.</li>\n   <li><tt>using-named-graph-uri</tt>: A string or an array of strings with the named graph URIs to be used by an update statement.</li>\n   <li><tt>method</tt>: The request method to be used. Available method values are $sparql:METHOD-GET, $sparql:METHOD-POST-URLENCODED or $sparql:METHOD-POST-SPARQL.</li>\n   <li><tt>format</tt> : The result format to request. Possible format values are $sparql:FORMAT-XML, $sparql:FORMAT-JSON, $sparql:FORMAT-CSV, $sparql:FORMAT-TSV, $sparql:FORMAT-ANY. For CONSTRUCT or DESCRIBE queries $sparql:FORMAT-RDF is available.</li>\n   <li><tt>parameters</tt>: Many SPARQL endpoints support additional parameters which are not part of the SPARQL endpoint specification. They can be provided here.</li>\n   <li><tt>http-headers</tt>: object that will be used as the headers property for the requests made by the http-client.</li>\n   <li><tt>http-options</tt>: object that will be used as the options property for the requests made by the http-client.</li>\n  </ul>\n <p>All those properties are optional.</p>\n </p>\n", "sees" : [  ], "authors" : [ "<xqdoc:author xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Alexander.Kreutz@28msec.com</xqdoc:author>" ], "version" : null, "encoding" : "utf-8", "namespaces" : [ { "uri" : "", "prefix" : "an" }, { "uri" : "http://www.28msec.com/modules/credentials", "prefix" : "credentials" }, { "uri" : "http://zorba.io/modules/http-client", "prefix" : "http-client" }, { "uri" : "http://www.w3.org/1999/02/22-rdf-syntax-ns#", "prefix" : "rdf" }, { "uri" : "http://www.w3.org/2005/sparql-results#", "prefix" : "s" }, { "uri" : "http://www.28msec.com/modules/sparql", "prefix" : "sparql" }, { "uri" : "http://zorba.io/options/versioning", "prefix" : "ver" } ], "functions" : [ { "isDocumented" : true, "arity" : 1, "name" : "ask", "qname" : "sparql:ask", "signature" : "($query as string) as xs:boolean", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">\n This function sends a SPARQL ASK query to an endpoint and returns the result as boolean value.\n The default datasource for SPARQL will be used.\n </p>\n", "summary" : "<p>  \n This function sends a SPARQL ASK query to an endpoint and returns the result as boolean value.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "query", "type" : "string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> A SPARQL ASK query as string.</div>" } ], "returns" : { "type" : "xs:boolean", "description" : "The boolean ASK query result." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 2, "name" : "ask", "qname" : "sparql:ask", "signature" : "($query as string, $options as object()?) as xs:boolean", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">\n This function sends a SPARQL ASK query to an endpoint and returns the result as boolean value.\n The default datasource for SPARQL will be used.\n </p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">\n   <ul>\n    <li>The <tt>default-graph-uri</tt> and <tt>named-graph-uri</tt> properties of the options object may be used to pass default graph URIs or named graph URIs to the endpoint.</li>\n    <li>The <tt>method</tt> property of the options object may be used to set the request method to be used. GET will be used as default.</li>\n    <li>The <tt>format</tt> property of the options object may be used to request a specific result format. However this function will try to extract only the boolean answer from the result.</li>\n   </ul>\n </p>\n", "summary" : "<p>  \n This function sends a SPARQL ASK query to an endpoint and returns the result as boolean value.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "query", "type" : "string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> A SPARQL ASK query as string.</div>" }, { "name" : "options", "type" : "object()", "occurrence" : "?", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> An optional options object as described in the module description.</div>" } ], "returns" : { "type" : "xs:boolean", "description" : "The boolean ASK query result." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 3, "name" : "ask", "qname" : "sparql:ask", "signature" : "($config as item()?, $query as string, $options as object()?) as xs:boolean", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">\n This function sends a SPARQL ASK query to an endpoint and returns the result as boolean value.\n </p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">\n   <ul>\n    <li>The <tt>default-graph-uri</tt> and <tt>named-graph-uri</tt> properties of the options object may be used to pass default graph URIs or named graph URIs to the endpoint.</li>\n    <li>The <tt>method</tt> property of the options object may be used to set the request method to be used. GET will be used as default.</li>\n    <li>The <tt>format</tt> property of the options object may be used to request a specific result format. However this function will try to extract only the boolean answer from the result.</li>\n   </ul>\n </p>\n", "summary" : "<p>  \n This function sends a SPARQL ASK query to an endpoint and returns the result as boolean value.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "config", "type" : "item()", "occurrence" : "?", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The name of the credentials to use as string or a config object as described in the module description.</div>" }, { "name" : "query", "type" : "string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> A SPARQL ASK query as string.</div>" }, { "name" : "options", "type" : "object()", "occurrence" : "?", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> An optional options object as described in the module description.</div>" } ], "returns" : { "type" : "xs:boolean", "description" : "The boolean ASK query result." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">sparql:INVALID-RESPONSE-TYPE if the response format cannot be handled by this function</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">sparql:ERROR if any http or endpoint error occurs</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">sparql:INVALID-PARAMETER invalid option specification</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">sparql:CREDENTIALS-NOT-FOUND credential information not found</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">sparql:CREDENTIALS-INVALID credential information not valid</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 1, "name" : "query-json", "qname" : "sparql:query-json", "signature" : "($query as string) as object()", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">\n This function sends a SPARQL SELECT or ASK query to an endpoint and returns the query response as SPARQL JSON format object.\n If the endpoint fails to deliver SPARQL JSON result format this function will try to convert the results to JSON if possible.\n The default datasource for SPARQL will be used.\n </p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Link to the specification of the result format at W3C: <a href=\"http://www.w3.org/TR/sparql11-results-json\" target=\"_blank\">SPARQL 1.1 Query Results JSON Format</a></p>\n", "summary" : "<p>  \n This function sends a SPARQL SELECT or ASK query to an endpoint and returns the query response as SPARQL JSON format object.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "query", "type" : "string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> A SPARQL SELECT or ASK query as string.</div>" } ], "returns" : { "type" : "object()", "description" : "The query result in SPARQL JSON format." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 2, "name" : "query-json", "qname" : "sparql:query-json", "signature" : "($query as string, $options as object()?) as object()", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">\n This function sends a SPARQL SELECT or ASK query to an endpoint and returns the query response as SPARQL JSON format object.\n If the endpoint fails to deliver SPARQL JSON result format this function will try to convert the results to JSON if possible.\n The default datasource for SPARQL will be used.\n </p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Link to the specification of the result format at W3C: <a href=\"http://www.w3.org/TR/sparql11-results-json\" target=\"_blank\">SPARQL 1.1 Query Results JSON Format</a></p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">\n   <ul>\n    <li>The <tt>default-graph-uri</tt> and <tt>named-graph-uri</tt> properties of the options object may be used to pass default graph URIs or named graph URIs to the endpoint.</li>\n    <li>The <tt>method</tt> property of the options object may be used to set the request method to be used. GET will be used as default.</li>\n    <li>The <tt>format</tt> property of the options object will be ignored. \"application/sparql-results+json\" will be requested as result format. </li>\n   </ul>\n </p>\n", "summary" : "<p>  \n This function sends a SPARQL SELECT or ASK query to an endpoint and returns the query response as SPARQL JSON format object.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "query", "type" : "string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> A SPARQL SELECT or ASK query as string.</div>" }, { "name" : "options", "type" : "object()", "occurrence" : "?", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> An optional options object as described in the module description.</div>" } ], "returns" : { "type" : "object()", "description" : "The query result in SPARQL JSON format." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 3, "name" : "query-json", "qname" : "sparql:query-json", "signature" : "($config as item()?, $query as string, $options as object()?) as object()", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">\n This function sends a SPARQL SELECT or ASK query to an endpoint and returns the query response as SPARQL JSON format object.\n If the endpoint fails to deliver SPARQL JSON result format this function will try to convert the results to JSON if possible.\n </p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Link to the specification of the result format at W3C: <a href=\"http://www.w3.org/TR/sparql11-results-json\" target=\"_blank\">SPARQL 1.1 Query Results JSON Format</a></p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">\n   <ul>\n    <li>The <tt>default-graph-uri</tt> and <tt>named-graph-uri</tt> properties of the options object may be used to pass default graph URIs or named graph URIs to the endpoint.</li>\n    <li>The <tt>method</tt> property of the options object may be used to set the request method to be used. GET will be used as default.</li>\n    <li>The <tt>format</tt> property of the options object will be ignored. \"application/sparql-results+json\" will be requested as result format. </li>\n   </ul>\n </p>\n", "summary" : "<p>  \n This function sends a SPARQL SELECT or ASK query to an endpoint and returns the query response as SPARQL JSON format object.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "config", "type" : "item()", "occurrence" : "?", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The name of the credentials to use as string or a config object as described in the module description.</div>" }, { "name" : "query", "type" : "string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> A SPARQL SELECT or ASK query as string.</div>" }, { "name" : "options", "type" : "object()", "occurrence" : "?", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> An optional options object as described in the module description.</div>" } ], "returns" : { "type" : "object()", "description" : "The query result in SPARQL JSON format." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">sparql:INVALID-RESPONSE-TYPE if the response format cannot be handled by this function</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">sparql:ERROR if any http or endpoint error occurs</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">sparql:INVALID-PARAMETER invalid option specification</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">sparql:CREDENTIALS-NOT-FOUND credential information not found</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">sparql:CREDENTIALS-INVALID credential information not valid</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 1, "name" : "query-rdf", "qname" : "sparql:query-rdf", "signature" : "($query as string) as element(rdf:RDF)", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">\n This function sends a SPARQL CONSTRUCT or DESCRIBE query to an endpoint and returns the query response as RDF.\n The default datasource for SPARQL will be used.\n </p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Link to the specification of the result format at W3C: <a href=\"http://www.w3.org/TR/rdf-syntax-grammar\" target=\"_blank\">RDF/XML Syntax Specification</a></p>\n", "summary" : "<p>  \n This function sends a SPARQL CONSTRUCT or DESCRIBE query to an endpoint and returns the query response as RDF.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "query", "type" : "string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> A SPARQL CONSTRUCT or DESCRIBE query as string.</div>" } ], "returns" : { "type" : "element(rdf:RDF)", "description" : "The query result as RDF XML element." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 2, "name" : "query-rdf", "qname" : "sparql:query-rdf", "signature" : "($query as string, $options as object()?) as element(rdf:RDF)", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">\n This function sends a SPARQL CONSTRUCT or DESCRIBE query to an endpoint and returns the query response as RDF.\n The default datasource for SPARQL will be used.\n </p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Link to the specification of the result format at W3C: <a href=\"http://www.w3.org/TR/rdf-syntax-grammar\" target=\"_blank\">RDF/XML Syntax Specification</a></p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">\n   <ul>\n    <li>The <tt>default-graph-uri</tt> and <tt>named-graph-uri</tt> properties of the options object may be used to pass default graph URIs or named graph URIs to the endpoint.</li>\n    <li>The <tt>method</tt> property of the options object may be used to set the request method to be used. GET will be used as default.</li>\n    <li>The <tt>format</tt> property of the options object will be ignored. \"application/rdf+xml\" will be requested as result format. </li>\n   </ul>\n </p>\n", "summary" : "<p>  \n This function sends a SPARQL CONSTRUCT or DESCRIBE query to an endpoint and returns the query response as RDF.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "query", "type" : "string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> A SPARQL CONSTRUCT or DESCRIBE query as string.</div>" }, { "name" : "options", "type" : "object()", "occurrence" : "?", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> An optional options object as described in the module description.</div>" } ], "returns" : { "type" : "element(rdf:RDF)", "description" : "The query result as RDF XML element." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 3, "name" : "query-rdf", "qname" : "sparql:query-rdf", "signature" : "($config as item()?, $query as string, $options as object()?) as element(rdf:RDF)", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">\n This function sends a SPARQL CONSTRUCT or DESCRIBE query to an endpoint and returns the query response as RDF.\n </p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Link to the specification of the result format at W3C: <a href=\"http://www.w3.org/TR/rdf-syntax-grammar\" target=\"_blank\">RDF/XML Syntax Specification</a></p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">\n   <ul>\n    <li>The <tt>default-graph-uri</tt> and <tt>named-graph-uri</tt> properties of the options object may be used to pass default graph URIs or named graph URIs to the endpoint.</li>\n    <li>The <tt>method</tt> property of the options object may be used to set the request method to be used. GET will be used as default.</li>\n    <li>The <tt>format</tt> property of the options object will be ignored. \"application/rdf+xml\" will be requested as result format. </li>\n   </ul>\n </p>\n", "summary" : "<p>  \n This function sends a SPARQL CONSTRUCT or DESCRIBE query to an endpoint and returns the query response as RDF.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "config", "type" : "item()", "occurrence" : "?", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The name of the credentials to use as string or a config object as described in the module description.</div>" }, { "name" : "query", "type" : "string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> A SPARQL CONSTRUCT or DESCRIBE query as string.</div>" }, { "name" : "options", "type" : "object()", "occurrence" : "?", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> An optional options object as described in the module description.</div>" } ], "returns" : { "type" : "element(rdf:RDF)", "description" : "The query result as RDF XML element." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">sparql:INVALID-RESPONSE-TYPE if the response format cannot be handled by this function</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">sparql:ERROR if any http or endpoint error occurs</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">sparql:INVALID-PARAMETER invalid option specification</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">sparql:CREDENTIALS-NOT-FOUND credential information not found</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">sparql:CREDENTIALS-INVALID credential information not valid</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 1, "name" : "query-xml", "qname" : "sparql:query-xml", "signature" : "($query as string) as element(s:sparql)", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">\n This function sends a SPARQL SELECT or ASK query to an endpoint and returns the query response as SPARQL XML format element.\n If the endpoint fails to deliver SPARQL XML result format this function will try to convert the results to XML if possible.\n The default datasource for SPARQL will be used.\n </p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Link to the specification of the result format at W3C: <a href=\"http://www.w3.org/TR/rdf-sparql-XMLres/\" target=\"_blank\">SPARQL Query Results XML Format</a></p>\n", "summary" : "<p>  \n This function sends a SPARQL SELECT or ASK query to an endpoint and returns the query response as SPARQL XML format element.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "query", "type" : "string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> A SPARQL SELECT or ASK query as string.</div>" } ], "returns" : { "type" : "element(s:sparql)", "description" : "The query result in SPARQL XML format." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 2, "name" : "query-xml", "qname" : "sparql:query-xml", "signature" : "($query as string, $options as object()?) as element(s:sparql)", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">\n This function sends a SPARQL SELECT or ASK query to an endpoint and returns the query response as SPARQL XML format element.\n If the endpoint fails to deliver SPARQL XML result format this function will try to convert the results to XML if possible.\n The default datasource for SPARQL will be used.\n </p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Link to the specification of the result format at W3C: <a href=\"http://www.w3.org/TR/rdf-sparql-XMLres/\" target=\"_blank\">SPARQL Query Results XML Format</a></p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">\n   <ul>\n    <li>The <tt>default-graph-uri</tt> and <tt>named-graph-uri</tt> properties of the options object may be used to pass default graph URIs or named graph URIs to the endpoint.</li>\n    <li>The <tt>method</tt> property of the options object may be used to set the request method to be used. GET will be used as default.</li>\n    <li>The <tt>format</tt> property of the options object will be ignored. \"application/sparql-results+xml\" will be requested as result format. </li>\n   </ul>\n </p>\n", "summary" : "<p>  \n This function sends a SPARQL SELECT or ASK query to an endpoint and returns the query response as SPARQL XML format element.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "query", "type" : "string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> A SPARQL SELECT or ASK query as string.</div>" }, { "name" : "options", "type" : "object()", "occurrence" : "?", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> An optional options object as described in the module description.</div>" } ], "returns" : { "type" : "element(s:sparql)", "description" : "The query result in SPARQL XML format." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 3, "name" : "query-xml", "qname" : "sparql:query-xml", "signature" : "($config as item()?, $query as string, $options as object()?) as element(s:sparql)", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">\n This function sends a SPARQL SELECT or ASK query to an endpoint and returns the query response as SPARQL XML format element.\n If the endpoint fails to deliver SPARQL XML result format this function will try to convert the results to XML if possible.\n </p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Link to the specification of the result format at W3C: <a href=\"http://www.w3.org/TR/rdf-sparql-XMLres/\" target=\"_blank\">SPARQL Query Results XML Format</a></p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">\n   <ul>\n    <li>The <tt>default-graph-uri</tt> and <tt>named-graph-uri</tt> properties of the options object may be used to pass default graph URIs or named graph URIs to the endpoint.</li>\n    <li>The <tt>method</tt> property of the options object may be used to set the request method to be used. GET will be used as default.</li>\n    <li>The <tt>format</tt> property of the options object will be ignored. \"application/sparql-results+xml\" will be requested as result format. </li>\n   </ul>\n </p>\n", "summary" : "<p>  \n This function sends a SPARQL SELECT or ASK query to an endpoint and returns the query response as SPARQL XML format element.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "config", "type" : "item()", "occurrence" : "?", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The name of the credentials to use as string or a config object as described in the module description.</div>" }, { "name" : "query", "type" : "string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> A SPARQL SELECT or ASK query as string.</div>" }, { "name" : "options", "type" : "object()", "occurrence" : "?", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> An optional options object as described in the module description.</div>" } ], "returns" : { "type" : "element(s:sparql)", "description" : "The query result in SPARQL XML format." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">sparql:INVALID-RESPONSE-TYPE if the response format cannot be handled by this function</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">sparql:ERROR if any http or endpoint error occurs</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">sparql:INVALID-PARAMETER invalid option specification</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">sparql:CREDENTIALS-NOT-FOUND credential information not found</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">sparql:CREDENTIALS-INVALID credential information not valid</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 1, "name" : "query", "qname" : "sparql:query", "signature" : "($query as string) as object()", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">\n This function sends a SPARQL query to an endpoint and directly returns the corresponding HTTP response.\n The default datasource for SPARQL will be used.\n </p>\n", "summary" : "<p>  \n This function sends a SPARQL query to an endpoint and directly returns the corresponding HTTP response.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "query", "type" : "string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> A SPARQL query as string. May be any kind of query but not an update.</div>" } ], "returns" : { "type" : "object()", "description" : "The endpoints response. This function returns the plain response object from the http-client." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 2, "name" : "query", "qname" : "sparql:query", "signature" : "($query as string, $options as object()?) as object()", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">\n This function sends a SPARQL query to an endpoint and directly returns the corresponding HTTP response.\n The default datasource for SPARQL will be used.\n </p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">\n   <ul>\n    <li>The <tt>default-graph-uri</tt> and <tt>named-graph-uri</tt> properties of the options object may be used to pass default graph URIs or named graph URIs to the endpoint.</li>\n    <li>The <tt>method</tt> property of the options object may be used to set the request method to be used. GET will be used as default.</li>\n    <li>The <tt>format</tt> property of the options object may be used to request a specific result format. XML will be used as default. However, note that this function does not interpret the result.</li>\n   </ul>\n </p>\n", "summary" : "<p>  \n This function sends a SPARQL query to an endpoint and directly returns the corresponding HTTP response.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "query", "type" : "string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> A SPARQL query as string. May be any kind of query but not an update.</div>" }, { "name" : "options", "type" : "object()", "occurrence" : "?", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> An optional options object as described in the module description.</div>" } ], "returns" : { "type" : "object()", "description" : "The endpoints response. This function returns the plain response object from the http-client." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 3, "name" : "query", "qname" : "sparql:query", "signature" : "($config as item()?, $query as string, $options as object()?) as object()", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">\n This function sends a SPARQL query to an endpoint and directly returns the corresponding HTTP response.\n </p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">\n   <ul>\n    <li>The <tt>default-graph-uri</tt> and <tt>named-graph-uri</tt> properties of the options object may be used to pass default graph URIs or named graph URIs to the endpoint.</li>\n    <li>The <tt>method</tt> property of the options object may be used to set the request method to be used. GET will be used as default.</li>\n    <li>The <tt>format</tt> property of the options object may be used to request a specific result format. XML will be used as default. However, note that this function does not interpret the result.</li>\n   </ul>\n </p>\n", "summary" : "<p>  \n This function sends a SPARQL query to an endpoint and directly returns the corresponding HTTP response.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "config", "type" : "item()", "occurrence" : "?", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The name of the credentials to use as string or a config object as described in the module description.</div>" }, { "name" : "query", "type" : "string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> A SPARQL query as string. May be any kind of query but not an update.</div>" }, { "name" : "options", "type" : "object()", "occurrence" : "?", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> An optional options object as described in the module description.</div>" } ], "returns" : { "type" : "object()", "description" : "The endpoints response. This function returns the plain response object from the http-client." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">sparql:INVALID-PARAMETER invalid option specification</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">sparql:CREDENTIALS-NOT-FOUND credential information not found</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">sparql:CREDENTIALS-INVALID credential information not valid</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 1, "name" : "update", "qname" : "sparql:update", "signature" : "($update-statement as string) as object()", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">\n This function sends a SPARQL update statement to an endpoint and directly returns the corresponding HTTP response.\n The default datasource for SPARQL will be used.\n </p>\n", "summary" : "<p>  \n This function sends a SPARQL update statement to an endpoint and directly returns the corresponding HTTP response.</p>", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "update-statement", "type" : "string", "occurrence" : null, "description" : "" } ], "returns" : { "type" : "object()", "description" : "The endpoints response. This function returns the plain response object from the http-client." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 2, "name" : "update", "qname" : "sparql:update", "signature" : "($update-statement as string, $options as object()?) as object()", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">\n This function sends a SPARQL update statement to an endpoint and directly returns the corresponding HTTP response.\n The default datasource for SPARQL will be used.\n </p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">\n   <ul>\n    <li>The <tt>using-graph-uri</tt> and <tt>using-named-graph-uri</tt> properties of the options object may be used to pass graph URIs or named graph URIs to the endpoint.</li>\n    <li>The <tt>method</tt> property of the options object may be used to set the request method to be used. POST-URLENCODED will be used as default.</li>\n    <li>The <tt>format</tt> property of the options object may be used to request a specific result format. XML will be used as default. However, note that this function does not interpret the result.</li>\n   </ul>\n </p>\n", "summary" : "<p>  \n This function sends a SPARQL update statement to an endpoint and directly returns the corresponding HTTP response.</p>", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "update-statement", "type" : "string", "occurrence" : null, "description" : "" }, { "name" : "options", "type" : "object()", "occurrence" : "?", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> An optional options object as described in the module description.</div>" } ], "returns" : { "type" : "object()", "description" : "The endpoints response. This function returns the plain response object from the http-client." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 3, "name" : "update", "qname" : "sparql:update", "signature" : "($config as item()?, $update-statement as string, $options as object()?) as object()", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">\n This function sends a SPARQL update statement to an endpoint and directly returns the corresponding HTTP response.\n </p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">\n   <ul>\n    <li>The <tt>using-graph-uri</tt> and <tt>using-named-graph-uri</tt> properties of the options object may be used to pass graph URIs or named graph URIs to the endpoint.</li>\n    <li>The <tt>method</tt> property of the options object may be used to set the request method to be used. POST-URLENCODED will be used as default.</li>\n    <li>The <tt>format</tt> property of the options object may be used to request a specific result format. XML will be used as default. However, note that this function does not interpret the result.</li>\n   </ul>\n </p>\n", "summary" : "<p>  \n This function sends a SPARQL update statement to an endpoint and directly returns the corresponding HTTP response.</p>", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "config", "type" : "item()", "occurrence" : "?", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The name of the credentials to use as string or a config object as described in the module description.</div>" }, { "name" : "update-statement", "type" : "string", "occurrence" : null, "description" : "" }, { "name" : "options", "type" : "object()", "occurrence" : "?", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> An optional options object as described in the module description.</div>" } ], "returns" : { "type" : "object()", "description" : "The endpoints response. This function returns the plain response object from the http-client." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">sparql:INVALID-PARAMETER invalid option specification</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">sparql:CREDENTIALS-NOT-FOUND credential information not found</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">sparql:CREDENTIALS-INVALID credential information not valid</xqdoc:error>" ] } ], "variables" : [ { "name" : "sparql:CREDENTIALS-CATEGORY", "type" : "string", "description" : " This variable represents the category of the datasource.\n" }, { "name" : "sparql:METHOD-GET", "type" : "string", "description" : " This variable represents the request method using HTTP GET requests. May be used for the <tt xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">method</tt> property of the options object.\n" }, { "name" : "sparql:METHOD-POST-URLENCODED", "type" : "string", "description" : " This variable represents the request method using HTTP POST with URL-encoded parameters. May be used for the <tt xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">method</tt> property of the options object.\n" }, { "name" : "sparql:METHOD-POST-SPARQL", "type" : "string", "description" : " This variable represents the request method using HTTP POST with the application/sparql-query content type. May be used for the <tt xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">method</tt> property of the options object.\n" }, { "name" : "sparql:FORMAT-XML", "type" : "string", "description" : " This variable represents the SPARQL XML result format.May be used for the <tt xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">format</tt> property of the options object.\n" }, { "name" : "sparql:FORMAT-JSON", "type" : "string", "description" : " This variable represents the SPARQL JSON result format.May be used for the <tt xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">format</tt> property of the options object.\n" }, { "name" : "sparql:FORMAT-RDF", "type" : "string", "description" : " This variable represents the RDF+XML result format.May be used for the <tt xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">format</tt> property of the options object.\n" }, { "name" : "sparql:FORMAT-CSV", "type" : "string", "description" : " This variable represents the CSV result format.May be used for the <tt xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">format</tt> property of the options object.\n" }, { "name" : "sparql:FORMAT-TSV", "type" : "string", "description" : " This variable represents the TSV result format.May be used for the <tt xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">format</tt> property of the options object.\n" }, { "name" : "sparql:FORMAT-ANY", "type" : "string", "description" : " This variable represents any result format. May be used for the <tt xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">format</tt> property of the options object to not specify a requested format.\n" } ] }, "http://api.28.io/dispatcher" : { "ns" : "http://api.28.io/dispatcher", "description" : "", "sees" : [  ], "authors" : [  ], "version" : null, "encoding" : null, "namespaces" : [ { "uri" : "", "prefix" : "an" }, { "uri" : "http://api.28.io/browserview", "prefix" : "browserview" }, { "uri" : "http://api.28.io/csvview", "prefix" : "csvview" }, { "uri" : "http://api.28.io/dispatcher", "prefix" : "dispatcher" }, { "uri" : "http://api.28.io/functions", "prefix" : "functions" }, { "uri" : "http://api.28.io/model", "prefix" : "model" }, { "uri" : "http://zorba.io/modules/xml", "prefix" : "parse-xml" }, { "uri" : "http://www.zorba-xquery.com/schemas/pul", "prefix" : "pul" }, { "uri" : "http://www.28msec.com/modules/http/request", "prefix" : "req" }, { "uri" : "http://www.28msec.com/modules/http/response", "prefix" : "response" }, { "uri" : "http://www.zorba-xquery.com/schemas/xdm", "prefix" : "xdm" }, { "uri" : "http://api.28.io/xdmview", "prefix" : "xdmview" } ], "functions" : [ { "isDocumented" : true, "arity" : 0, "name" : "restapi", "qname" : "dispatcher:restapi", "signature" : "()", "description" : " Main function for the collection browser. May be called from a handler function.\n Uses the call URL to determine which page needs to be returned.\n The URL for the initial page needs to end with /index\n", "summary" : "<p> Main function for the collection browser.</p>", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [  ], "returns" : { "type" : null, "description" : "the requested page of the collection browser" }, "errors" : [  ] } ], "variables" : [  ] }, "http://zorba.io/modules/store/static/collections/ddl" : { "ns" : "http://zorba.io/modules/store/static/collections/ddl", "description" : " This modules defines a set of functions for managing collections that are\n declared in the prolog of a module.\n For example, it provides functions to create, delete, or introspect\n collections.\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n This module is part of <a xmlns:xqdoc=\"http://www.xqdoc.org/1.0\" href=\"../zorba/xqddf.html\">Zorba's XQuery Data\n Definition Facility</a>. All the collections managed by this module\n have to be pre-declared in the prolog of a module. Please refer to the\n <a xmlns:xqdoc=\"http://www.xqdoc.org/1.0\" href=\"../zorba/data_lifecycle.html\">general documentation</a>\n for more information and examples.\n", "sees" : [ "<xqdoc:see xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"><a href=\"../zorba/data_lifecycle.html\">Data Lifecycle</a></xqdoc:see>", "<xqdoc:see xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"><a href=\"../zorba/xqddf.html\">XQuery Data Definition Facility</a></xqdoc:see>", "<xqdoc:see xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">http://zorba.io/modules/store/static/collections/dml</xqdoc:see>", "<xqdoc:see xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">http://zorba.io/modules/store/static/indexes/ddl</xqdoc:see>", "<xqdoc:see xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">http://zorba.io/modules/store/static/indexes/dml</xqdoc:see>", "<xqdoc:see xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">http://zorba.io/modules/store/static/integrity-constraints/ddl</xqdoc:see>", "<xqdoc:see xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">http://zorba.io/modules/store/static/integrity-constraints/dml</xqdoc:see>", "<xqdoc:see xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">http://zorba.io/errors</xqdoc:see>" ], "authors" : [ "<xqdoc:author xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Nicolae Brinza, Matthias Brantner, David Graf, Till Westmann, Markos Zaharioudakis</xqdoc:author>" ], "version" : null, "encoding" : "utf-8", "namespaces" : [ { "uri" : "http://zorba.io/modules/store/static/collections/ddl", "prefix" : "cddl" }, { "uri" : "http://zorba.io/options/versioning", "prefix" : "ver" }, { "uri" : "http://zorba.io/errors", "prefix" : "zerr" } ], "functions" : [ { "isDocumented" : true, "arity" : 0, "name" : "available-collections", "qname" : "cddl:available-collections", "signature" : "() as xs:QName* external", "description" : " Gets the QNames of the collections that have been statically declared and\n are available, if any.\n", "summary" : "<p> Gets the QNames of the collections that have been statically declared and\n are available, if any.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [  ], "returns" : { "type" : "xs:QName*", "description" : "A sequence comprising one QName for each statically declared and available collection or an emtpy sequence if no such collections are available." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "create", "qname" : "cddl:create", "signature" : "($name as xs:QName) external", "description" : " Creates a collection.\n", "summary" : "<p> Creates a collection.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : true, "parameters" : [ { "name" : "name", "type" : "xs:QName", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The of the collection to create.</div>" } ], "returns" : { "type" : null, "description" : "An empty XDM instance and a pending update list that, once applied, creates a collection with the given name." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">zerr:ZDDY0001 if <code>$name</code> is not equal to any of the declared collections in the static context.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">zerr:ZDDY0002 if a collection with <code>$name</code> already exists.</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 2, "name" : "create", "qname" : "cddl:create", "signature" : "($name as xs:QName, $content as item()*) external", "description" : " Creates a collection and adds the given sequence as content to the new\n collection.\n", "summary" : "<p> Creates a collection and adds the given sequence as content to the new\n collection.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : true, "parameters" : [ { "name" : "name", "type" : "xs:QName", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The name of the collection to create.</div>" }, { "name" : "content", "type" : "item()", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The sequences of items (nodes or JSON items) to be added to the new collection.</div>" } ], "returns" : { "type" : null, "description" : "An empty XDM instance and a pending update list that, once applied, creates a collection with the given name and inserts the given items into it." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">zerr:ZDDY0001 if <code>$name</code> is not equal to any of the declared collections in the static context.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">zerr:ZDDY0002 if a collection with <code>$name</code> already exists.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">zerr:ZDTY0001 if <code>$content</code> does not match the expected type declared by the collection according to the rules for SequenceType Matching.</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 0, "name" : "declared-collections", "qname" : "cddl:declared-collections", "signature" : "() as xs:QName* external", "description" : " Gets the collections that have been declared in the prolog of the static\n context.\n", "summary" : "<p> Gets the collections that have been declared in the prolog of the static\n context.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [  ], "returns" : { "type" : "xs:QName*", "description" : "A sequence of QNames, one for each collection created in the static context, or an emtpy sequence." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "delete", "qname" : "cddl:delete", "signature" : "($name as xs:QName) external", "description" : " Deletes a collection.\n", "summary" : "<p> Deletes a collection.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : true, "parameters" : [ { "name" : "name", "type" : "xs:QName", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The collection to delete.</div>" } ], "returns" : { "type" : null, "description" : "An empty XDM instance and a pending update list that, once applied, deletes the collection with the given name." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">zerr:ZDDY0001 if <code>$name</code> is not equal to any of the declared collections in the static context.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">zerr:ZDDY0003 if the collection identified by <code>$name</code> is not available.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">zerr:ZDDY0013 if the domain or key expression of any of the available indexes access the collection having <code>$name</code>.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">zerr:ZDDY0015 if any of the in-scope variables references an item that belongs to the collection having <code>$name</code></xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 1, "name" : "is-available-collection", "qname" : "cddl:is-available-collection", "signature" : "($name as xs:QName) as xs:boolean external", "description" : " Gets whether a collection is statically declared and available.\n", "summary" : "<p> Gets whether a collection is statically declared and available.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "name", "type" : "xs:QName", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The name of the collection to check.</div>" } ], "returns" : { "type" : "xs:boolean", "description" : "true if the collection was statically declared and is available; false otherwise." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "is-declared-collection", "qname" : "cddl:is-declared-collection", "signature" : "($name as xs:QName) as xs:boolean external", "description" : " Gers whether a collection was declared in the prolog of the static context.\n", "summary" : "<p> Gers whether a collection was declared in the prolog of the static context.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "name", "type" : "xs:QName", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The name of the collection to check.</div>" } ], "returns" : { "type" : "xs:boolean", "description" : "true if the collection was declared; false otherwise." }, "errors" : [  ] } ], "variables" : [  ] }, "http://www.28msec.com/modules/math" : { "ns" : "http://www.28msec.com/modules/math", "description" : " The functions in this module perform trigonometric and other mathematical\n calculations on double values.\n For a more detailed description of the semantics of each function, please\n refer to <a xmlns:xqdoc=\"http://www.xqdoc.org/1.0\" href=\"http://www.w3.org/TR/xpath-functions-30/#trigonometry\">\n Trigonometric and exponential functions</a>.\n", "sees" : [  ], "authors" : [  ], "version" : null, "encoding" : "utf-8", "namespaces" : [ { "uri" : "http://www.w3.org/2005/xpath-functions/math", "prefix" : "fnmath" }, { "uri" : "http://www.28msec.com/modules/math", "prefix" : "math" }, { "uri" : "http://zorba.io/options/versioning", "prefix" : "ver" } ], "functions" : [ { "isDocumented" : true, "arity" : 1, "name" : "acos", "qname" : "math:acos", "signature" : "($arg as double?) as double?", "description" : " Returns the arc cosine of the argument, the result being in the range zero to +π radians.\n", "summary" : "<p> Returns the arc cosine of the argument, the result being in the range zero to +π radians.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "arg", "type" : "double", "occurrence" : "?", "description" : "" } ], "returns" : { "type" : "double?", "description" : "" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "asin", "qname" : "math:asin", "signature" : "($arg as double?) as double?", "description" : " Returns the arc sine of the argument, the result being in the range -π/2 to +π/2 radians.\n", "summary" : "<p> Returns the arc sine of the argument, the result being in the range -π/2 to +π/2 radians.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "arg", "type" : "double", "occurrence" : "?", "description" : "" } ], "returns" : { "type" : "double?", "description" : "" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "atan", "qname" : "math:atan", "signature" : "($arg as double?) as double?", "description" : " Returns the arc tangent of the argument, the result being in the range -π/2 to +π/2 radians.\n", "summary" : "<p> Returns the arc tangent of the argument, the result being in the range -π/2 to +π/2 radians.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "arg", "type" : "double", "occurrence" : "?", "description" : "" } ], "returns" : { "type" : "double?", "description" : "" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 2, "name" : "atan2", "qname" : "math:atan2", "signature" : "($y as double, $x as double) as double", "description" : " Returns the angle in radians subtended at the origin by the point on a plane with\n coordinates (x, y) and the positive x-axis, the result being in the range -π to +π.\n", "summary" : "<p> Returns the angle in radians subtended at the origin by the point on a plane with\n coordinates (x, y) and the positive x-axis, the result being in the range -π to +π.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "y", "type" : "double", "occurrence" : null, "description" : "" }, { "name" : "x", "type" : "double", "occurrence" : null, "description" : "" } ], "returns" : { "type" : "double", "description" : "" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "cos", "qname" : "math:cos", "signature" : "($theta as double?) as double?", "description" : " Returns the cosine of the argument, expressed in radians.\n", "summary" : "<p> Returns the cosine of the argument, expressed in radians.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "theta", "type" : "double", "occurrence" : "?", "description" : "" } ], "returns" : { "type" : "double?", "description" : "" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "exp", "qname" : "math:exp", "signature" : "($arg as double?) as double?", "description" : " Returns the value of ex.\n", "summary" : "<p> Returns the value of ex.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "arg", "type" : "double", "occurrence" : "?", "description" : "" } ], "returns" : { "type" : "double?", "description" : "" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "exp10", "qname" : "math:exp10", "signature" : "($arg as double?) as double?", "description" : " Returns the value of 10x.\n", "summary" : "<p> Returns the value of 10x.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "arg", "type" : "double", "occurrence" : "?", "description" : "" } ], "returns" : { "type" : "double?", "description" : "" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "log", "qname" : "math:log", "signature" : "($arg as double?) as double?", "description" : " Returns the natural logarithm of the argument.\n", "summary" : "<p> Returns the natural logarithm of the argument.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "arg", "type" : "double", "occurrence" : "?", "description" : "" } ], "returns" : { "type" : "double?", "description" : "" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "log10", "qname" : "math:log10", "signature" : "($arg as double?) as double?", "description" : " Returns the base-ten logarithm of the argument.\n", "summary" : "<p> Returns the base-ten logarithm of the argument.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "arg", "type" : "double", "occurrence" : "?", "description" : "" } ], "returns" : { "type" : "double?", "description" : "" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 0, "name" : "pi", "qname" : "math:pi", "signature" : "() as double", "description" : " Returns an approximation to the mathematical constant π.\n", "summary" : "<p> Returns an approximation to the mathematical constant π.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [  ], "returns" : { "type" : "double", "description" : "" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 2, "name" : "pow", "qname" : "math:pow", "signature" : "($x as double?, $y) as double?", "description" : " Returns the result of raising the first argument to the power of the second.\n", "summary" : "<p> Returns the result of raising the first argument to the power of the second.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "x", "type" : "double", "occurrence" : "?", "description" : "" }, { "name" : "y", "type" : null, "occurrence" : null, "description" : "" } ], "returns" : { "type" : "double?", "description" : "" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "sin", "qname" : "math:sin", "signature" : "($theta as double?) as double?", "description" : " Returns the sine of the argument, expressed in radians.\n", "summary" : "<p> Returns the sine of the argument, expressed in radians.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "theta", "type" : "double", "occurrence" : "?", "description" : "" } ], "returns" : { "type" : "double?", "description" : "" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "sqrt", "qname" : "math:sqrt", "signature" : "($arg as double?) as double?", "description" : " Returns the non-negative square root of the argument.\n", "summary" : "<p> Returns the non-negative square root of the argument.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "arg", "type" : "double", "occurrence" : "?", "description" : "" } ], "returns" : { "type" : "double?", "description" : "" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "tan", "qname" : "math:tan", "signature" : "($theta as double?) as double?", "description" : " Returns the tangent of the argument, expressed in radians.\n", "summary" : "<p> Returns the tangent of the argument, expressed in radians.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "theta", "type" : "double", "occurrence" : "?", "description" : "" } ], "returns" : { "type" : "double?", "description" : "" }, "errors" : [  ] } ], "variables" : [  ] }, "http://zorba.io/modules/data-cleaning/normalization" : { "ns" : "http://zorba.io/modules/data-cleaning/normalization", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">This library module provides data normalization functions for processing calendar dates,\n temporal values, currency values, units of measurement, location names and postal addresses.\n These functions are particularly useful for converting different data representations into cannonical formats.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">The logic contained in this module is not specific to any particular XQuery implementation.</p>\n", "sees" : [  ], "authors" : [ "<xqdoc:author xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Bruno Martins and Diogo Simões</xqdoc:author>" ], "version" : null, "encoding" : "utf-8", "namespaces" : [ { "uri" : "http://zorba.io/annotations", "prefix" : "an" }, { "uri" : "http://www.zorba-xquery.com/modules/http-client", "prefix" : "http" }, { "uri" : "http://zorba.io/modules/data-cleaning/normalization", "prefix" : "normalization" }, { "uri" : "http://zorba.io/options/versioning", "prefix" : "ver" } ], "functions" : [ { "isDocumented" : true, "arity" : 1, "name" : "normalize-address", "qname" : "normalization:normalize-address", "signature" : "($addr as xs:string*) as xs:string*", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Uses an address normalization Web service to convert a postal address given as input into a\n cannonical representation format.</p>\n", "summary" : "<p>  Uses an address normalization Web service to convert a postal address given as input into a\n cannonical representation format.</p>", "annotation_str" : " %an:nondeterministic", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "nondeterministic", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "addr", "type" : "xs:string", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> A sequence of strings encoding an address, where each string in the sequence corresponds to a different component (e.g., street, city, country, etc.) of the address.</div>" } ], "returns" : { "type" : "xs:string*", "description" : "A sequence of strings with the address encoded in a cannonical format, where each string in the sequence corresponds to a different component (e.g., street, city, country, etc.) of the address." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "normalize-phone", "qname" : "normalization:normalize-phone", "signature" : "($addr as xs:string*) as xs:string*", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Uses an phone number normalization Web service to convert a phone number given as input into a\n cannonical representation.</p>\n", "summary" : "<p>  Uses an phone number normalization Web service to convert a phone number given as input into a\n cannonical representation.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "addr", "type" : "xs:string", "occurrence" : "*", "description" : "" } ], "returns" : { "type" : "xs:string*", "description" : "A strings with the phone number encoded in a cannonical format. <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/> <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"><b> Attention : This function is still not implemented. </b></p>" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 2, "name" : "to-date", "qname" : "normalization:to-date", "signature" : "($sd as xs:string, $format as xs:string?) as xs:string", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Converts a given string representation of a date value into a date representation valid according\n to the corresponding XML Schema type.</p>\n", "summary" : "<p>  Converts a given string representation of a date value into a date representation valid according\n to the corresponding XML Schema type.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "sd", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The string representation for the date</div>" }, { "name" : "format", "type" : "xs:string", "occurrence" : "?", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> An optional parameter denoting the format used to represent the date in the string, according to a sequence of conversion specifications. In the format string, a conversion specification is introduced by '%', usually followed by a single letter or 'O' or 'E' and then a single letter. Any character in the format string that is not part of a conversion specification is interpreted literally, and the string '%%' gives '%'. The supported conversion specifications are as follows: <pre> '%b' Abbreviated month name in the current locale. '%B' Full month name in the current locale. '%d' Day of the month as decimal number (01-31). '%m' Month as decimal number (01-12). '%x' Date, locale-specific. '%y' Year without century (00-99). '%Y' Year with century. '%C' Century (00-99): the integer part of the year divided by 100. '%D' Locale-specific date format such as '%m/%d/%y'. '%e' Day of the month as decimal number (1-31), with a leading pace for a single-digit number. '%F' Equivalent to %Y-%m-%d (the ISO 8601 date format). '%h' Equivalent to '%b'. </pre></div>" } ], "returns" : { "type" : "xs:string", "description" : "The date value resulting from the conversion." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 2, "name" : "to-dateTime", "qname" : "normalization:to-dateTime", "signature" : "($sd as xs:string, $format as xs:string?) as xs:string", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Converts a given string representation of a dateTime value into a dateTime representation\n valid according to the corresponding XML Schema type.</p>\n", "summary" : "<p>  Converts a given string representation of a dateTime value into a dateTime representation\n valid according to the corresponding XML Schema type.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "sd", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The string representation for the dateTime.</div>" }, { "name" : "format", "type" : "xs:string", "occurrence" : "?", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> An optional parameter denoting the format used to represent the dateTime in the string, according to a sequence of conversion specifications. In the format string, a conversion specification is introduced by '%', usually followed by a single letter or 'O' or 'E' and then a single letter. Any character in the format string that is not part of a conversion specification is interpreted literally, and the string '%%' gives '%'. The supported conversion specifications are as follows: <p/> <pre class=\"ace-static\"> '%b' Abbreviated month name in the current locale. '%B' Full month name in the current locale. '%c' Date and time, locale-specific. '%C' Century (00-99): the integer part of the year divided by 100. '%d' Day of the month as decimal number (01-31). '%H' Hours as decimal number (00-23). '%I' Hours as decimal number (01-12). '%j' Day of year as decimal number (001-366). '%m' Month as decimal number (01-12). '%M' Minute as decimal number (00-59). '%p' AM/PM indicator in the locale. Used in conjunction with '%I' and *not* with '%H'. '%S' Second as decimal number (00-61), allowing for up to two leap-seconds. '%x' Date, locale-specific. '%X' Time, locale-specific. '%y' Year without century (00-99). '%Y' Year with century. '%z' Offset from Greenwich, so '-0900' is 9 hours west of Greenwich. '%Z' Time zone as a character string. '%D' Locale-specific date format such as '%m/%d/%y': ISO C99 says it should be that exact format. '%e' Day of the month as decimal number (1-31), with a leading pace for a single-digit number. '%F' Equivalent to %Y-%m-%d (the ISO 8601 date format). '%g' The last two digits of the week-based year (see '%V'). '%G' The week-based year (see '%V') as a decimal number. '%h' Equivalent to '%b'. '%k' The 24-hour clock time with single digits preceded by a blank. '%l' The 12-hour clock time with single digits preceded by a blank. '%r' The 12-hour clock time (using the locale's AM or PM). '%R' Equivalent to '%H:%M'. '%T' Equivalent to '%H:%M:%S'. </pre></div>" } ], "returns" : { "type" : "xs:string", "description" : "The dateTime value resulting from the conversion." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">normalization:NOTSUPPORTED if the dateTime type is not known to the service.</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 2, "name" : "to-time", "qname" : "normalization:to-time", "signature" : "($sd as xs:string, $format as xs:string?) as xs:string?", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Converts a given string representation of a time value into a time representation valid according to\n the corresponding XML Schema type.</p>\n", "summary" : "<p>  Converts a given string representation of a time value into a time representation valid according to\n the corresponding XML Schema type.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "sd", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The string representation for the time.</div>" }, { "name" : "format", "type" : "xs:string", "occurrence" : "?", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> An optional parameter denoting the format used to represent the time in the string, according to a sequence of conversion specifications. In the format string, a conversion specification is introduced by '%', usually followed by a single letter or 'O' or 'E' and then a single letter. Any character in the format string that is not part of a conversion specification is interpreted literally, and the string '%%' gives '%'. The supported conversion specifications are as follows: <p/> <pre class=\"ace-static\"> '%H' Hours as decimal number (00-23). '%I' Hours as decimal number (01-12). '%M' Minute as decimal number (00-59). '%p' AM/PM indicator in the locale. Used in conjunction with '%I' and *not* with '%H'. '%S' Second as decimal number (00-61), allowing for up to two leap-seconds. '%X' Time, locale-specific. '%z' Offset from Greenwich, so '-0900' is 9 hours west of Greenwich. '%Z' Time zone as a character string. '%k' The 24-hour clock time with single digits preceded by a blank. '%l' The 12-hour clock time with single digits preceded by a blank. '%r' The 12-hour clock time (using the locale's AM or PM). '%R' Equivalent to '%H:%M'. '%T' Equivalent to '%H:%M:%S'. </pre></div>" } ], "returns" : { "type" : "xs:string?", "description" : "The time value resulting from the conversion." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">normalization:NOTSUPPORTED if the date type is not known to the service.</xqdoc:error>" ] } ], "variables" : [  ] }, "http://zorba.io/modules/excel/engineering" : { "ns" : "http://zorba.io/modules/excel/engineering", "description" : "  This is a library module offering the same set of functions\n defined by Microsoft Excel, under Engineering Functions.\n", "sees" : [ "<xqdoc:see xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"><a href=\"http://office.microsoft.com/en-us/excel/CH062528241033.aspx\" target=\"_blank\">Excel Documentation: Engineering Functions</a></xqdoc:see>" ], "authors" : [ "<xqdoc:author xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Sorin Nasoi</xqdoc:author>" ], "version" : null, "encoding" : "utf-8", "namespaces" : [ { "uri" : "http://zorba.io/modules/excel/engineering", "prefix" : "excel-engineering" }, { "uri" : "http://zorba.io/modules/excel/errors", "prefix" : "excel-err" }, { "uri" : "http://zorba.io/modules/excel/math", "prefix" : "excel-math" }, { "uri" : "http://zorba.io/modules/excel/text", "prefix" : "excel-text" }, { "uri" : "http://zorba.io/options/versioning", "prefix" : "ver" } ], "functions" : [ { "isDocumented" : true, "arity" : 1, "name" : "bin2dec", "qname" : "excel-engineering:bin2dec", "signature" : "($arg as xs:anyAtomicType) as xs:integer", "description" : " Converts a binary number to decimal.\n", "summary" : "<p> Converts a binary number to decimal.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "arg", "type" : "xs:anyAtomicType", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the number.</div>" } ], "returns" : { "type" : "xs:integer", "description" : "A decimal representation of a number given it's binary representation." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">excel-err:Value if provided value for $arg is not a binary representation of a number.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">excel-err:Value if provided value for $arg contains more than 10 characters.</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 1, "name" : "bin2hex", "qname" : "excel-engineering:bin2hex", "signature" : "($arg as xs:anyAtomicType) as xs:string", "description" : " Converts a binary number to hexadecimal.\n", "summary" : "<p> Converts a binary number to hexadecimal.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "arg", "type" : "xs:anyAtomicType", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the number.</div>" } ], "returns" : { "type" : "xs:string", "description" : "A hexadecimal representation of a number given it's binary representation." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">excel-err:Value if provided value for $arg is not a binary representation of a number.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">excel-err:Value if provided value for $arg contains more than 10 characters.</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 2, "name" : "bin2hex", "qname" : "excel-engineering:bin2hex", "signature" : "($arg as xs:anyAtomicType, $places as xs:anyAtomicType) as xs:string", "description" : " Converts a binary number to hexadecimal.\n", "summary" : "<p> Converts a binary number to hexadecimal.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "arg", "type" : "xs:anyAtomicType", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the number.</div>" }, { "name" : "places", "type" : "xs:anyAtomicType", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> is the number of characters to use. Places is useful for padding the return value with leading 0s (zeros).</div>" } ], "returns" : { "type" : "xs:string", "description" : "A hexadecimal representation of a number given it's binary representation." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">excel-err:Value if provided value for $arg is not a binary representation of a number.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">excel-err:Value if provided value for $arg contains more than 10 characters.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">excel-err:Value if provided value for $places is not numeric.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">excel-err:Num if provided value for $places is zero or negative.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">excel-err:Num if provided value for $places is too small.</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 1, "name" : "bin2oct", "qname" : "excel-engineering:bin2oct", "signature" : "($arg as xs:anyAtomicType) as xs:string", "description" : " Converts a binary number to octal.\n", "summary" : "<p> Converts a binary number to octal.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "arg", "type" : "xs:anyAtomicType", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the number.</div>" } ], "returns" : { "type" : "xs:string", "description" : "A octal representation of a number given it's binary representation." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">excel-err:Value if provided value for $arg is not a binary representation of a number.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">excel-err:Value if provided value for $arg contains more than 10 characters.</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 2, "name" : "bin2oct", "qname" : "excel-engineering:bin2oct", "signature" : "($arg as xs:anyAtomicType, $places as xs:anyAtomicType) as xs:string", "description" : " Converts a binary number to octal.\n", "summary" : "<p> Converts a binary number to octal.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "arg", "type" : "xs:anyAtomicType", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the number.</div>" }, { "name" : "places", "type" : "xs:anyAtomicType", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> is the number of characters to use. Places is useful for padding the return value with leading 0s (zeros).</div>" } ], "returns" : { "type" : "xs:string", "description" : "A octal representation of a number given it's binary representation." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">excel-err:Value if provided value for $arg is not a binary representation of a number.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">excel-err:Value if provided value for $arg contains more than 10 characters.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">excel-err:Value if provided value for $places is not numeric.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">excel-err:Num if provided value for $places is zero or negative.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">excel-err:Num if provided value for $places is too small.</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 1, "name" : "dec2bin", "qname" : "excel-engineering:dec2bin", "signature" : "($arg as xs:anyAtomicType) as xs:string", "description" : " Converts a decimal number to binary.\n", "summary" : "<p> Converts a decimal number to binary.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "arg", "type" : "xs:anyAtomicType", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the number.</div>" } ], "returns" : { "type" : "xs:string", "description" : "A binary representation of a number given it's decimal representation." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">excel-err:Value if provided value for $arg is not numeric.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">excel-err:Num if provided value for $arg is smaller than -512 or bigger than 511.</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 2, "name" : "dec2bin", "qname" : "excel-engineering:dec2bin", "signature" : "($arg as xs:anyAtomicType, $places as xs:anyAtomicType) as xs:string", "description" : " Converts a decimal number to binary.\n", "summary" : "<p> Converts a decimal number to binary.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "arg", "type" : "xs:anyAtomicType", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the number.</div>" }, { "name" : "places", "type" : "xs:anyAtomicType", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> is the number of characters to use. Places is useful for padding the return value with leading 0s (zeros).</div>" } ], "returns" : { "type" : "xs:string", "description" : "A binary representation of a number given it's decimal representation." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">excel-err:Value if provided value for $arg is not numeric.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">excel-err:Value if provided value for $places is not numeric.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">excel-err:Num if provided value for $places is zero or negative.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">excel-err:Num if provided value for $places is too small.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">excel-err:Num if provided value for $arg is smaller than -512 or bigger than 511.</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 1, "name" : "dec2hex", "qname" : "excel-engineering:dec2hex", "signature" : "($arg as xs:anyAtomicType) as xs:string", "description" : " Converts a decimal number to hexadecimal.\n", "summary" : "<p> Converts a decimal number to hexadecimal.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "arg", "type" : "xs:anyAtomicType", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the number.</div>" } ], "returns" : { "type" : "xs:string", "description" : "A hexadecimal representation of a number given it's decimal representation." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">excel-err:Value if provided value for $arg is not numeric.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">excel-err:Num if provided value for $arg is smaller than -549755813888 or bigger than 549755813887</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 2, "name" : "dec2hex", "qname" : "excel-engineering:dec2hex", "signature" : "($arg as xs:anyAtomicType, $places as xs:anyAtomicType) as xs:string", "description" : " Converts a decimal number to hexadecimal.\n", "summary" : "<p> Converts a decimal number to hexadecimal.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "arg", "type" : "xs:anyAtomicType", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the number.</div>" }, { "name" : "places", "type" : "xs:anyAtomicType", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> is the number of characters to use. Places is useful for padding the return value with leading 0s (zeros).</div>" } ], "returns" : { "type" : "xs:string", "description" : "A hexadecimal representation of a number given it's decimal representation." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">excel-err:Value if provided value for $arg is not numeric.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">excel-err:Value if provided value for $places is not numeric.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">excel-err:Num if provided value for $places is zero or negative.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">excel-err:Num if provided value for $places is too small.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">excel-err:Num if provided value for $arg is smaller than -549755813888 or bigger than 549755813887.</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 1, "name" : "dec2oct", "qname" : "excel-engineering:dec2oct", "signature" : "($arg as xs:anyAtomicType) as xs:string", "description" : " Converts a decimal number to octal.\n", "summary" : "<p> Converts a decimal number to octal.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "arg", "type" : "xs:anyAtomicType", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the number.</div>" } ], "returns" : { "type" : "xs:string", "description" : "An octal representation of a number given it's decimal representation." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">excel-err:Value if provided value for $arg is not numeric.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">excel-err:Num if provided value for $arg is smaller than -536870912 or bigger than 536870911.</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 2, "name" : "dec2oct", "qname" : "excel-engineering:dec2oct", "signature" : "($arg as xs:anyAtomicType, $places as xs:anyAtomicType) as xs:string", "description" : " Converts a decimal number to octal.\n", "summary" : "<p> Converts a decimal number to octal.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "arg", "type" : "xs:anyAtomicType", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the number</div>" }, { "name" : "places", "type" : "xs:anyAtomicType", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> is the number of characters to use. Places is useful for padding the return value with leading 0s (zeros).</div>" } ], "returns" : { "type" : "xs:string", "description" : "An octal representation of a number given it's decimal representation." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">excel-err:Value if provided value for $arg is not numeric</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">excel-err:Value if provided value for $places is not numeric</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">excel-err:Num if provided value for $places is zero or negative</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">excel-err:Num if provided value for $places is too small</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">excel-err:Num if provided value for $arg is smaller than -536870912 or bigger than 536870911</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 1, "name" : "hex2bin", "qname" : "excel-engineering:hex2bin", "signature" : "($arg as xs:anyAtomicType) as xs:string", "description" : " Converts a hexadecimal number to binary.\n", "summary" : "<p> Converts a hexadecimal number to binary.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "arg", "type" : "xs:anyAtomicType", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the number.</div>" } ], "returns" : { "type" : "xs:string", "description" : "A binary representation of a number given it's hexadecimal representation." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">excel-err:Value if provided value for $arg is not a hexadecimal representation of a number.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">excel-err:Value if provided value for $arg contains more than 10 characters.</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 2, "name" : "hex2bin", "qname" : "excel-engineering:hex2bin", "signature" : "($arg as xs:anyAtomicType, $places as xs:anyAtomicType) as xs:string", "description" : " Converts a hexadecimal number to binary.\n", "summary" : "<p> Converts a hexadecimal number to binary.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "arg", "type" : "xs:anyAtomicType", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the number.</div>" }, { "name" : "places", "type" : "xs:anyAtomicType", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> is the number of characters to use. Places is useful for padding the return value with leading 0s (zeros).</div>" } ], "returns" : { "type" : "xs:string", "description" : "A binary representation of a number given it's hexadecimal representation." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">excel-err:Value if provided value for $arg is not a hexadecimal representation of a number.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">excel-err:Value if provided value for $arg contains more than 10 characters.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">excel-err:Value if provided value for $places is not numeric.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">excel-err:Num if provided value for $places is zero or negative.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">excel-err:Num if provided value for $places is too small.</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 1, "name" : "hex2dec", "qname" : "excel-engineering:hex2dec", "signature" : "($arg as xs:string) as xs:integer", "description" : " Converts a hexadecimal number to decimal.\n", "summary" : "<p> Converts a hexadecimal number to decimal.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "arg", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the number.</div>" } ], "returns" : { "type" : "xs:integer", "description" : "A decimal representation of a number given it's hexadecimal representation." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">excel-err:Value if provided value for $arg is not a hexadecimal representation of a number.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">excel-err:Value if provided value for $arg contains more than 10 characters.</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 1, "name" : "hex2oct", "qname" : "excel-engineering:hex2oct", "signature" : "($arg as xs:anyAtomicType) as xs:string", "description" : " Converts a hexadecimal number to octal.\n", "summary" : "<p> Converts a hexadecimal number to octal.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "arg", "type" : "xs:anyAtomicType", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the number.</div>" } ], "returns" : { "type" : "xs:string", "description" : "A octal representation of a number given it's hexadecimal representation." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">excel-err:Value if provided value for $arg is not a hexadecimal representation of a number.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">excel-err:Value if provided value for $arg contains more than 10 characters.</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 2, "name" : "hex2oct", "qname" : "excel-engineering:hex2oct", "signature" : "($arg as xs:anyAtomicType, $places as xs:anyAtomicType) as xs:string", "description" : " Converts a hexadecimal number to octal.\n", "summary" : "<p> Converts a hexadecimal number to octal.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "arg", "type" : "xs:anyAtomicType", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the number.</div>" }, { "name" : "places", "type" : "xs:anyAtomicType", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> is the number of characters to use. Places is useful for padding the return value with leading 0s (zeros).</div>" } ], "returns" : { "type" : "xs:string", "description" : "A octal representation of a number given it's hexadecimal representation." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">excel-err:Value if provided value for $arg is not a hexadecimal representation of a number.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">excel-err:Value if provided value for $arg contains more than 10 characters.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">excel-err:Value if provided value for $places is not numeric.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">excel-err:Num if provided value for $places is zero or negative.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">excel-err:Num if provided value for $places is too small.</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 1, "name" : "oct2bin", "qname" : "excel-engineering:oct2bin", "signature" : "($arg as xs:anyAtomicType) as xs:string", "description" : " Converts an octal number to binary.\n", "summary" : "<p> Converts an octal number to binary.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "arg", "type" : "xs:anyAtomicType", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the number.</div>" } ], "returns" : { "type" : "xs:string", "description" : "A binary representation of a number given it's octal representation." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">excel-err:Value if provided value for $arg is not an octal representation of a number.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">excel-err:Value if provided value for $arg contains more than 10 characters.</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 2, "name" : "oct2bin", "qname" : "excel-engineering:oct2bin", "signature" : "($arg as xs:anyAtomicType, $places as xs:anyAtomicType) as xs:string", "description" : " Converts an octal number to binary.\n", "summary" : "<p> Converts an octal number to binary.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "arg", "type" : "xs:anyAtomicType", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the number.</div>" }, { "name" : "places", "type" : "xs:anyAtomicType", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> is the number of characters to use. Places is useful for padding the return value with leading 0s (zeros).</div>" } ], "returns" : { "type" : "xs:string", "description" : "A binary representation of a number given it's octal representation." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">excel-err:Value if provided value for $arg is not an octal representation of a number.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">excel-err:Value if provided value for $arg contains more than 10 characters.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">excel-err:Value if provided value for $places is not numeric.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">excel-err:Num if provided value for $places is zero or negative.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">excel-err:Num if provided value for $places is too small.</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 1, "name" : "oct2dec", "qname" : "excel-engineering:oct2dec", "signature" : "($arg as xs:anyAtomicType) as xs:integer", "description" : " Converts an octal number to decimal.\n", "summary" : "<p> Converts an octal number to decimal.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "arg", "type" : "xs:anyAtomicType", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the number.</div>" } ], "returns" : { "type" : "xs:integer", "description" : "A decimal representation of a number given it's octal representation." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">excel-err:Value if provided value for $arg is not an octal representation of a number.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">excel-err:Value if provided value for $arg contains more than 10 characters.</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 1, "name" : "oct2hex", "qname" : "excel-engineering:oct2hex", "signature" : "($arg as xs:anyAtomicType) as xs:string", "description" : " Converts an octal number to hexadecimal.\n", "summary" : "<p> Converts an octal number to hexadecimal.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "arg", "type" : "xs:anyAtomicType", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the number.</div>" } ], "returns" : { "type" : "xs:string", "description" : "A hexadecimal representation of a number given it's octal representation." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">excel-err:Value if provided value for $arg is not an octal representation of a number.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">excel-err:Value if provided value for $arg contains more than 10 characters.</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 2, "name" : "oct2hex", "qname" : "excel-engineering:oct2hex", "signature" : "($arg as xs:anyAtomicType, $places as xs:anyAtomicType) as xs:string", "description" : " Converts an octal number to hexadecimal.\n", "summary" : "<p> Converts an octal number to hexadecimal.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "arg", "type" : "xs:anyAtomicType", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the number.</div>" }, { "name" : "places", "type" : "xs:anyAtomicType", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> is the number of characters to use. Places is useful for padding the return value with leading 0s (zeros).</div>" } ], "returns" : { "type" : "xs:string", "description" : "A hexadecimal representation of a number given it's octal representation." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">excel-err:Value if provided value for $arg is not an octal representation of a number.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">excel-err:Value if provided value for $arg contains more than 10 characters.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">excel-err:Value if provided value for $places is not numeric.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">excel-err:Num if provided value for $places is zero or negative.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">excel-err:Num if provided value for $places is too small.</xqdoc:error>" ] } ], "variables" : [  ] }, "http://www.zorba-xquery.com/modules/util-jvm" : { "ns" : "http://www.zorba-xquery.com/modules/util-jvm", "description" : " This module provides common functionality for modules that use java\n implementations.\n <br xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n <br xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n Modules using java implementations must import this module\n to specify the dependency.\n <br xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n <br xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n <b xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Note:</b> Since this module has a Java library dependency a JVM is required\n to be installed on the system. For Windows: jvm.dll is required on the system\n path (usually located in \"C:\\Program Files\\Java\\jre\\bin\\client\").\n", "sees" : [  ], "authors" : [ "<xqdoc:author xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Cezar Andrei</xqdoc:author>" ], "version" : null, "encoding" : "utf-8", "namespaces" : [ { "uri" : "http://www.zorba-xquery.com/modules/util-jvm", "prefix" : "util-jvm" }, { "uri" : "http://zorba.io/options/versioning", "prefix" : "ver" } ], "functions" : [  ], "variables" : [  ] }, "http://www.28msec.com/modules/asynchronous-jobs" : { "ns" : "http://www.28msec.com/modules/asynchronous-jobs", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">This module provides functionality for executing asynchronous jobs.\n There are three kinds of jobs: (1) a job for executing a (public or private)\n query of your project, (2) a job for mapping an input\n collection into an output collection, and (3) a job for shuffling the items\n of an input collection to several output collections.\n Map and Shuffle jobs are mostly used within\n the parallelism framework and are not meant to be created directly.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Jobs that have been scheduled for execution can be referred to by\n an opaque identifier being returned by the corresponding scheduling\n function (e.g. job:execute or job:map). Specifically,\n the job's identifier can be used to retrieve the status of a job\n (e.g. job:status) or the names of its input and output collections.</p>\n", "sees" : [  ], "authors" : [ "<xqdoc:author xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Ghislain Fourny</xqdoc:author>", "<xqdoc:author xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Julien Ribon</xqdoc:author>", "<xqdoc:author xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Matthias Brantner</xqdoc:author>" ], "version" : null, "encoding" : "utf-8", "namespaces" : [ { "uri" : "", "prefix" : "an" }, { "uri" : "http://zorba.io/modules/base64", "prefix" : "base64" }, { "uri" : "http://zorba.io/modules/fetch", "prefix" : "fetch" }, { "uri" : "http://www.zorba-xquery.com/modules/http-client", "prefix" : "http-client" }, { "uri" : "http://www.28msec.com/modules/asynchronous-jobs", "prefix" : "job" }, { "uri" : "http://www.28msec.com/modules/project", "prefix" : "project" }, { "uri" : "http://zorba.io/modules/random", "prefix" : "rand" }, { "uri" : "http://www.28msec.com/modules/http/request", "prefix" : "request" }, { "uri" : "http://www.28msec.com/modules/store", "prefix" : "store" } ], "functions" : [ { "isDocumented" : true, "arity" : 2, "name" : "build-map-shuffle", "qname" : "job:build-map-shuffle", "signature" : "($input-collection as xs:string, $options as object()?) as object()", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Builds an object describing an asynchronous job (map or shuffle).</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">This object can then be sent to the scheduler.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">The $options parameter allows for the specification of properties for\n the shuffle job. Allowed options are:\n <ul>\n   <li><tt>kind</tt>: the kind of job (\"map\" or \"shuffle\").</li>\n   <li><tt>map-function</tt> (map jobs only): a function with any signature\n   (item()* to item()*),\n   and that is used to map the input collection to the output collection.</li>\n   <li><tt>shuffle-function</tt> (shuffle jobs only): a function taking an item\n   and returning an integer.\n   It is used to select the output collection to which each item is sent.</li>\n   <li><tt>output-collection as string</tt> (map jobs only): the name of the collection in\n     which the output of the executed query is stored. If not specified,\n     the result of the query will be stored in a collection withing the\n     MongoDB database associated with the project. The name of the\n     collection is randomly generated and starts with <tt>_28.temporary</tt>\n   </li>\n   <li><tt>output-collections as array</tt> (shuffle jobs only): an array with the names of the collections in\n     which the items of the input collections are shuffled. Compulsory.\n   </li>\n   <li><tt>chunk-info as object</tt>: an object containing the specification of a chunk\n     of the input collection. The shuffling will only occur on this chunk.\n     If not specified, the entire input collection will be mapped.\n   </li>\n   <li><tt>allow-streaming as boolean</tt> (map jobs only): a boolean indicating if calling the map function can\n     be distributed over partitions of the input collection. If false, the map function will\n     be called only once on the entire input collection. This might result\n     in a cache overflow if the number of items is too big. If true,\n     the input will be arbitrarily partitioned and the map function will be called for each\n     partition. The size of a partition is chosen in order to be smaller than the cache size.\n     The default is false.\n   </li>\n   <li><tt>dependencies as array</tt>: an array containing the ids of all asynchronous jobs that must\n     be completed before this job starts. This job will be hold pending until all these dependencies\n     are completed.\n   </li>\n </ul>\n </p>\n", "summary" : "<p>  Builds an object describing an asynchronous job (map or shuffle).</p>", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "input-collection", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The name of the collection that the job processes.</div>" }, { "name" : "options", "type" : "object()", "occurrence" : "?", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> An object specifying above options for the asynchronous job.</div>" } ], "returns" : { "type" : "object()", "description" : "the id of the scheduled job, which can be used to pull for the status and results." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">err:XPTY0004 if the type of a supplied option is incorrect.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">job:JBDY0005 if the supplied function does not have a name.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">job:JBDY0006 if the supplied function is in the local namespace.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">job:JBDY0008 if the supplied input collection is not available.</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 1, "name" : "error", "qname" : "job:error", "signature" : "($id as xs:string) as object()?", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Returns the error object output by an asynchronous job, in case of a failed job.</p>\n", "summary" : "<p>  Returns the error object output by an asynchronous job, in case of a failed job.</p>", "annotation_str" : " %an:nondeterministic", "annotations" : [ { "prefix" : "an", "ns" : "", "name" : "nondeterministic", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "id", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the id of the asynchronous job to query.</div>" } ], "returns" : { "type" : "object()?", "description" : "the error object output by the corresponding asynchronous job." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "execute", "qname" : "job:execute", "signature" : "($query as xs:string) as xs:string", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Schedules an asynchronous job executing the given (public or private)\n query from your project.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">The result of the query will be stored in a collection withing the\n MongoDB database associated with the project. The name of the collection\n is randomly generated and starts with <tt>_28.temporary</tt>.</p>\n", "summary" : "<p>  Schedules an asynchronous job executing the given (public or private)\n query from your project.</p>", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "query", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The path of the query to execute, which must begin with /public or /private. (examples: /public/query.xq, /private/query.jq).</div>" } ], "returns" : { "type" : "xs:string", "description" : "The id of the scheduled job." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">job:JBDY0003 if the supplied query path does not begin with /public or /private.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">job:JBDY0004 if the supplied query path cannot be resolved.</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 2, "name" : "execute", "qname" : "job:execute", "signature" : "($query as xs:string, $options as object()?) as xs:string", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Schedules an asynchronous job executing the given (public or private)\n query from your project.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">The $options parameter allows for the specification of properties for\n the asynchronously executed job. Allowed options are:\n <ul>\n   <li><tt>output-collection as string</tt>: the name of the collection in\n     which the output of the executed query is stored. If not specified,\n     the result of the query will be stored in a collection withing the\n     MongoDB database associated with the project. The name of the\n     collection is randomly generated and starts with <tt>_28.temporary</tt>\n   </li>\n </ul>\n </p>\n", "summary" : "<p>  Schedules an asynchronous job executing the given (public or private)\n query from your project.</p>", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "query", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The path of the query to execute, which must begin with /public or /private. (examples: /public/query.xq, /private/query.jq).</div>" }, { "name" : "options", "type" : "object()", "occurrence" : "?", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> An object specifying above options for the asynchronous job.</div>" } ], "returns" : { "type" : "xs:string", "description" : "The id of the scheduled job." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">err:XPTY0004 if the type of a supplied option is incorrect.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">job:JBDY0003 if the supplied query path does not begin with /public or /private.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">job:JBDY0004 if the supplied query path cannot be resolved.</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 1, "name" : "function", "qname" : "job:function", "signature" : "($id as xs:string) as xs:string?", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Returns the name of the map or shuffle function used by an asynchronous job.</p>\n", "summary" : "<p>  Returns the name of the map or shuffle function used by an asynchronous job.</p>", "annotation_str" : " %an:nondeterministic", "annotations" : [ { "prefix" : "an", "ns" : "", "name" : "nondeterministic", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "id", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the id of the asynchronous job to query.</div>" } ], "returns" : { "type" : "xs:string?", "description" : "the name of the map or shuffle function of the corresponding asynchronous job." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "get", "qname" : "job:get", "signature" : "($id as xs:string) as object()*", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Returns the asynchronous job associated with the supplied asynchronous job id,\n or the asynchronous jobs associated with the supplied parallel job id.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">This function is internal and should only be used by the parallelism module.</p>\n", "summary" : "<p>  Returns the asynchronous job associated with the supplied asynchronous job id,\n or the asynchronous jobs associated with the supplied parallel job id.</p>", "annotation_str" : " %an:nondeterministic", "annotations" : [ { "prefix" : "an", "ns" : "", "name" : "nondeterministic", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "id", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the id of the asynchronous or parallel job to query.</div>" } ], "returns" : { "type" : "object()*", "description" : "the objects describing the asynchronous jobs." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "input-collection", "qname" : "job:input-collection", "signature" : "($id as xs:string) as xs:string?", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Returns the name of the input collection of a mapping asynchronous job.</p>\n", "summary" : "<p>  Returns the name of the input collection of a mapping asynchronous job.</p>", "annotation_str" : " %an:nondeterministic", "annotations" : [ { "prefix" : "an", "ns" : "", "name" : "nondeterministic", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "id", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the id of the asynchronous job to query.</div>" } ], "returns" : { "type" : "xs:string?", "description" : "the name of the input collection of the corresponding asynchronous job." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 0, "name" : "jobs", "qname" : "job:jobs", "signature" : "() as object()*", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Returns the ids of all jobs created by this project.</p>\n", "summary" : "<p>  Returns the ids of all jobs created by this project.</p>", "annotation_str" : " %an:nondeterministic", "annotations" : [ { "prefix" : "an", "ns" : "", "name" : "nondeterministic", "value" : "" } ], "updating" : false, "parameters" : [  ], "returns" : { "type" : "object()*", "description" : "the sequence of the ids of all jobs created so far." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "kind", "qname" : "job:kind", "signature" : "($id as xs:string) as xs:string?", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Returns the kind of the asynchronous job.</p>\n", "summary" : "<p>  Returns the kind of the asynchronous job.</p>", "annotation_str" : " %an:nondeterministic", "annotations" : [ { "prefix" : "an", "ns" : "", "name" : "nondeterministic", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "id", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the id of the asynchronous job to query.</div>" } ], "returns" : { "type" : "xs:string?", "description" : "the kind of the asynchronous job." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 2, "name" : "map", "qname" : "job:map", "signature" : "($input-collection as xs:string, $map-function as function (item()*) as item()*) as xs:string", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Schedules an asynchronous job mapping the input collection\n to an output collection with the specified function.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">The name of the output collection is randomly generated and\n starts with <tt>_28.temporary</tt>.</p>\n", "summary" : "<p>  Schedules an asynchronous job mapping the input collection\n to an output collection with the specified function.</p>", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "input-collection", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The name of the collection that the job processes.</div>" }, { "name" : "map-function", "type" : "function (item()*) as item()*", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The function which is run on the input collection's contents.</div>" } ], "returns" : { "type" : "xs:string", "description" : "The id of the scheduled job, which can be used to pull for the status and results." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">job:JBDY0005 if the supplied function does not have a name.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">job:JBDY0006 if the supplied function is in the local namespace.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">job:JBDY0008 if the supplied input collection does not exist.</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 3, "name" : "map", "qname" : "job:map", "signature" : "($input-collection as xs:string, $map-function as function (item()*) as item()*, $options as object()?) as xs:string", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Schedules an asynchronous job mapping the input collection\n to an output collection with the specified function.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">The $options parameter allows for the specification of properties for\n the map job. Allowed options are:\n <ul>\n   <li><tt>output-collection as string</tt>: the name of the collection in\n     which the output of the executed query is stored. If not specified,\n     the result of the query will be stored in a collection withing the\n     MongoDB database associated with the project. The name of the\n     collection is randomly generated and starts with <tt>_28.temporary</tt>\n   </li>\n   <li><tt>chunk-info as object</tt>: an object containing the specification of a chunk\n     of the input collection. The mapping will only occur on this chunk.\n     If not specified, the entire input collection will be mapped.\n   </li>\n   <li><tt>allow-streaming as boolean</tt>: a boolean indicating if calling the map function can\n     be distributed over partitions of the input collection. If false, the map function will\n     be called only once on the entire input collection. This might result\n     in a cache overflow if the number of items is too big. If true,\n     the input will be arbitrarily partitioned and the map function will be called for each\n     partition. The size of a partition is chosen in order to be smaller than the cache size.\n     The default is false.\n   </li>\n   <li><tt>dependencies as array</tt>: an array containing the ids of all asynchronous jobs that must\n     be completed before this job starts. This job will be hold pending until all these dependencies\n     are completed.\n   </li>\n </ul>\n </p>\n", "summary" : "<p>  Schedules an asynchronous job mapping the input collection\n to an output collection with the specified function.</p>", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "input-collection", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The name of the collection that the job processes.</div>" }, { "name" : "map-function", "type" : "function (item()*) as item()*", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The function which is run on the collection's contents.</div>" }, { "name" : "options", "type" : "object()", "occurrence" : "?", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> An object specifying above options for the asynchronous job.</div>" } ], "returns" : { "type" : "xs:string", "description" : "the id of the scheduled job, which can be used to pull for the status and results." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">err:XPTY0004 if the type of a supplied option is incorrect.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">job:JBDY0005 if the supplied function does not have a name.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">job:JBDY0006 if the supplied function is in the local namespace.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">job:JBDY0008 if the supplied input collection does not exist.</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 1, "name" : "output-collection", "qname" : "job:output-collection", "signature" : "($id as xs:string) as xs:string?", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Returns the name of the output collection of a mapping or executing asynchronous job.</p>\n", "summary" : "<p>  Returns the name of the output collection of a mapping or executing asynchronous job.</p>", "annotation_str" : " %an:nondeterministic", "annotations" : [ { "prefix" : "an", "ns" : "", "name" : "nondeterministic", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "id", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the id of the asynchronous job to query.</div>" } ], "returns" : { "type" : "xs:string?", "description" : "the name of the output collection of the corresponding asynchronous job." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "output-collections", "qname" : "job:output-collections", "signature" : "($id as xs:string) as xs:string*", "description" : " Returns the name of the output collections of a shuffle asynchronous job.\n", "summary" : "<p> Returns the name of the output collections of a shuffle asynchronous job.</p>", "annotation_str" : " %an:nondeterministic", "annotations" : [ { "prefix" : "an", "ns" : "", "name" : "nondeterministic", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "id", "type" : "xs:string", "occurrence" : null, "description" : "" } ], "returns" : { "type" : "xs:string*", "description" : "the name of the output collections of the corresponding asynchronous job." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "properties", "qname" : "job:properties", "signature" : "($id as xs:string) as object()", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Returns the properties of an asynchronous job as an object with the following\n fields:</p>\n <ul xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">\n  <li><tt>accessed</tt>: the time at which the job was last accessed.</li>\n  <li><tt>completed</tt>: the time at which the job was completed.</li>\n  <li><tt>created</tt>: the time at which the job was created.</li>\n  <li><tt>lock-acquired</tt>: the time at which a processing lock was taken on the job.</li>\n  <li><tt>status</tt>: the status of the job. One of <tt>pending</tt>, <tt>in progress</tt>,\n  <tt>completed</tt>, <tt>failed</tt>, <tt>timed out</tt>.</li>\n  <li><tt>error</tt>: An object containing error information in case of failure.</li>\n </ul>\n", "summary" : "<p>  Returns the properties of an asynchronous job as an object with the following\n fields: \n  \n   accessed : the time at which the job was last accessed.</p>", "annotation_str" : " %an:nondeterministic", "annotations" : [ { "prefix" : "an", "ns" : "", "name" : "nondeterministic", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "id", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the id of the asynchronous job to query.</div>" } ], "returns" : { "type" : "object()", "description" : "the time at which the corresponding asynchronous job was last accessed." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "query-name", "qname" : "job:query-name", "signature" : "($id as xs:string) as xs:string?", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Returns the name of the query executed by an executing asynchronous job.</p>\n", "summary" : "<p>  Returns the name of the query executed by an executing asynchronous job.</p>", "annotation_str" : " %an:nondeterministic", "annotations" : [ { "prefix" : "an", "ns" : "", "name" : "nondeterministic", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "id", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the id of the asynchronous job to query.</div>" } ], "returns" : { "type" : "xs:string?", "description" : "the name of the query executed by the corresponding asynchronous job." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "schedule", "qname" : "job:schedule", "signature" : "($jobs as object()*) as object()*", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Sends jobs to the job queue.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">This function is internal and should only be used by this and the parallelism module.</p>\n", "summary" : "<p>  Sends jobs to the job queue.</p>", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "jobs", "type" : "object()", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The job to send, as generated by job:build-map-shuffle().</div>" } ], "returns" : { "type" : "object()*", "description" : "Objects containing the new ids." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 3, "name" : "shuffle", "qname" : "job:shuffle", "signature" : "($input-collection as xs:string, $shuffle-function as function (item()) as xs:integer, $options as object()?) as xs:string", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Schedules an asynchronous job shuffling the items of the input collections to output collections\n using the specified shuffle function.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">The $options parameter allows for the specification of properties for\n the shuffle job. Allowed options are:\n <ul>\n   <li><tt>output-collections as array</tt>: an array with the names of the collections in\n     which the items of the input collections are shuffled. Compulsory.\n   </li>\n   <li><tt>chunk-info as object</tt>: an object containing the specification of a chunk\n     of the input collection. The shuffling will only occur on this chunk.\n     If not specified, the entire input collection will be mapped.\n   </li>\n   <li><tt>dependencies as array</tt>: an array containing the ids of all asynchronous jobs that must\n     be completed before this job starts. This job will be hold pending until all these dependencies\n     are completed.\n   </li>\n </ul>\n </p>\n", "summary" : "<p>  Schedules an asynchronous job shuffling the items of the input collections to output collections\n using the specified shuffle function.</p>", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "input-collection", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The name of the collection that the job processes.</div>" }, { "name" : "shuffle-function", "type" : "function (item()) as xs:integer", "occurrence" : null, "description" : "" }, { "name" : "options", "type" : "object()", "occurrence" : "?", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> An object specifying above options for the asynchronous job.</div>" } ], "returns" : { "type" : "xs:string", "description" : "the id of the scheduled job, which can be used to pull for the status and results." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">err:XPTY0004 if the type of a supplied option is incorrect.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">job:JBDY0005 if the supplied function does not have a name.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">job:JBDY0006 if the supplied function is in the local namespace.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">job:JBDY0008 if the supplied input collection does not exist.</xqdoc:error>" ] } ], "variables" : [  ] }, "http://www.28msec.com/modules/datetime" : { "ns" : "http://www.28msec.com/modules/datetime", "description" : " This modules contains function that operate on items of type date, time,\n dateTime, and duration.\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">The module is always imported so you don't need to import it explicitly.\n Also, you don't need to fully qualify a function to invoke it.</p>\n", "sees" : [  ], "authors" : [  ], "version" : null, "encoding" : "utf-8", "namespaces" : [ { "uri" : "http://www.28msec.com/modules/datetime", "prefix" : "datetime" }, { "uri" : "http://zorba.io/options/versioning", "prefix" : "ver" } ], "functions" : [ { "isDocumented" : true, "arity" : 1, "name" : "adjust-date-to-timezone", "qname" : "datetime:adjust-date-to-timezone", "signature" : "($arg as date?) as date?", "description" : " Adjusts a date value to the implicit timezone.\n", "summary" : "<p> Adjusts a date value to the implicit timezone.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "arg", "type" : "date", "occurrence" : "?", "description" : "" } ], "returns" : { "type" : "date?", "description" : "the date in the implicit timezone that contains the starting instant of the supplied date. For a detailed description of the semantics of this function, please see <a xmlns:xqdoc=\"http://www.xqdoc.org/1.0\" href=\"http://www.w3.org/TR/xpath-functions-30/#func-adjust-date-to-timezone\">adjust-date-to-timezone</a>." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 2, "name" : "adjust-date-to-timezone", "qname" : "datetime:adjust-date-to-timezone", "signature" : "($arg as date?, $timezone as dayTimeDuration?) as date?", "description" : " Adjusts a date value to a specific timezone, or to no timezone at all.\n", "summary" : "<p> Adjusts a date value to a specific timezone, or to no timezone at all.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "arg", "type" : "date", "occurrence" : "?", "description" : "" }, { "name" : "timezone", "type" : "dayTimeDuration", "occurrence" : "?", "description" : "" } ], "returns" : { "type" : "date?", "description" : "the date in the target timezone that contains the starting instant of the supplied date. <ul xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> <li><code>adjust-date-to-timezone(date(\"2002-03-07\"), dayTimeDuration(\"-PT10H\"))</code> returns <code>date(\"2002-03-06-10:00\")</code>.</li> <li><code>adjust-date-to-timezone(date(\"2002-03-07-07:00\"), ())</code> returns <code>date(\"2002-03-07\")</code>.</li> </ul> For a detailed description of the semantics of this function, please see <a xmlns:xqdoc=\"http://www.xqdoc.org/1.0\" href=\"http://www.w3.org/TR/xpath-functions-30/#func-adjust-date-to-timezone\">adjust-date-to-timezone</a>" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "adjust-dateTime-to-timezone", "qname" : "datetime:adjust-dateTime-to-timezone", "signature" : "($arg as dateTime?) as dateTime", "description" : " Adjusts a dateTime value to the implicit timezone.\n", "summary" : "<p> Adjusts a dateTime value to the implicit timezone.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "arg", "type" : "dateTime", "occurrence" : "?", "description" : "" } ], "returns" : { "type" : "dateTime", "description" : "the dateTime in the implicit timezone. For a detailed description of the semantics of this function, please see <a xmlns:xqdoc=\"http://www.xqdoc.org/1.0\" href=\"http://www.w3.org/TR/xpath-functions-30/#func-adjust-dateTime-to-timezone\">adjust-dateTime-to-timezone</a>" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 2, "name" : "adjust-dateTime-to-timezone", "qname" : "datetime:adjust-dateTime-to-timezone", "signature" : "($arg as dateTime?, $timezone as dayTimeDuration?) as dateTime", "description" : " Adjusts a dateTime value to the specified timezone, or to no timezone at all.\n", "summary" : "<p> Adjusts a dateTime value to the specified timezone, or to no timezone at all.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "arg", "type" : "dateTime", "occurrence" : "?", "description" : "" }, { "name" : "timezone", "type" : "dayTimeDuration", "occurrence" : "?", "description" : "" } ], "returns" : { "type" : "dateTime", "description" : "the dateTime in the specified timezone. <ul xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> <li><code>adjust-dateTime-to-timezone(dateTime('2002-03-07T10:00:00-07:00'), dayTimeDuration(\"-PT10H\"))</code> returns <code>dateTime('2002-03-07T07:00:00-10:00')</code>.</li> <li><code>adjust-dateTime-to-timezone(dateTime('2002-03-07T10:00:00'), ())</code> returns <code>dateTime('2002-03-07T10:00:00')</code>.</li> </ul> For a detailed description of the semantics of this function, please see <a xmlns:xqdoc=\"http://www.xqdoc.org/1.0\" href=\"http://www.w3.org/TR/xpath-functions-30/#func-adjust-dateTime-to-timezone\">adjust-dateTime-to-timezone</a>" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "adjust-time-to-timezone", "qname" : "datetime:adjust-time-to-timezone", "signature" : "($arg as time?) as time?", "description" : " Adjusts a time value to an implicit timezone.\n", "summary" : "<p> Adjusts a time value to an implicit timezone.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "arg", "type" : "time", "occurrence" : "?", "description" : "" } ], "returns" : { "type" : "time?", "description" : "the time in the implicit timezone. For a detailed description of the semantics of this function, please see <a xmlns:xqdoc=\"http://www.xqdoc.org/1.0\" href=\"http://www.w3.org/TR/xpath-functions-30/#func-adjust-time-to-timezone\">adjust-time-to-timezone</a>" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 2, "name" : "adjust-time-to-timezone", "qname" : "datetime:adjust-time-to-timezone", "signature" : "($arg as time?, $timezone as dayTimeDuration?) as time?", "description" : " Adjusts a time value to a specific timezone, or to no timezone at all.\n", "summary" : "<p> Adjusts a time value to a specific timezone, or to no timezone at all.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "arg", "type" : "time", "occurrence" : "?", "description" : "" }, { "name" : "timezone", "type" : "dayTimeDuration", "occurrence" : "?", "description" : "" } ], "returns" : { "type" : "time?", "description" : "the time in the specified timezone. <ul xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> <li><code>adjust-time-to-timezone(time(\"10:00:00\"), dayTimeDuration(\"-PT10H\"))</code> returns <code>time(\"07:00:00-10:00\")</code>.</li> <li><code>adjust-time-to-timezone(time(\"10:00:00-07:00\"), ())</code> returns <code>time(\"10:00:00\")</code>.</li> </ul> For a detailed description of the semantics of this function, please see <a xmlns:xqdoc=\"http://www.xqdoc.org/1.0\" href=\"http://www.w3.org/TR/xpath-functions-30/#func-adjust-time-to-timezone\">adjust-time-to-timezone</a>" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 0, "name" : "current-date", "qname" : "datetime:current-date", "signature" : "() as date external", "description" : " Returns the current date.\n For a detailed description of the semantics of this function, please see\n <a xmlns:xqdoc=\"http://www.xqdoc.org/1.0\" href=\"http://www.w3.org/TR/xpath-functions-30/#func-current-date\">current-date</a>\n", "summary" : "<p> Returns the current date.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [  ], "returns" : { "type" : "date", "description" : "" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 0, "name" : "current-dateTime", "qname" : "datetime:current-dateTime", "signature" : "() as dateTimeStamp external", "description" : " Returns the current dateTime.\n For a detailed description of the semantics of this function, please see\n <a xmlns:xqdoc=\"http://www.xqdoc.org/1.0\" href=\"http://www.w3.org/TR/xpath-functions-30/#func-current-dateTime\">current-dateTime</a>\n", "summary" : "<p> Returns the current dateTime.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [  ], "returns" : { "type" : "dateTimeStamp", "description" : "" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 0, "name" : "current-time", "qname" : "datetime:current-time", "signature" : "() as time external", "description" : " Returns the current time.\n For a detailed description of the semantics of this function, please see\n <a xmlns:xqdoc=\"http://www.xqdoc.org/1.0\" href=\"http://www.w3.org/TR/xpath-functions-30/#func-current-time\">current-time</a>\n", "summary" : "<p> Returns the current time.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [  ], "returns" : { "type" : "time", "description" : "" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 2, "name" : "dateTime", "qname" : "datetime:dateTime", "signature" : "($arg1 as date?, $arg2 as time?) as dateTime?", "description" : " Returns a dateTime value created by combining a date and a time.\n", "summary" : "<p> Returns a dateTime value created by combining a date and a time.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "arg1", "type" : "date", "occurrence" : "?", "description" : "" }, { "name" : "arg2", "type" : "time", "occurrence" : "?", "description" : "" } ], "returns" : { "type" : "dateTime?", "description" : "the dateTime value created by combinding the given date and time. <ul xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> <li><code>dateTime(date(\"1999-12-31\"), time(\"12:00:00\"))</code> returns <code>dateTime(\"1999-12-31T12:00:00\")</code>.</li> </ul> For a detailed description of the semantics of this function, please see <a xmlns:xqdoc=\"http://www.xqdoc.org/1.0\" href=\"http://www.w3.org/TR/xpath-functions-30/#func-dateTime\">dateTime</a>" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "day-from-date", "qname" : "datetime:day-from-date", "signature" : "($arg as date?) as integer?", "description" : " Returns the day component of a date.\n", "summary" : "<p> Returns the day component of a date.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "arg", "type" : "date", "occurrence" : "?", "description" : "" } ], "returns" : { "type" : "integer?", "description" : "the day component of the given date. <ul xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> <li><code>day-from-date(date(\"1999-05-31-05:00\"))</code> returns <code>31</code>.</li> </ul> For a detailed description of the semantics of this function, please see <a xmlns:xqdoc=\"http://www.xqdoc.org/1.0\" href=\"http://www.w3.org/TR/xpath-functions-30/#func-day-from-date\">day-from-date</a>" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "day-from-dateTime", "qname" : "datetime:day-from-dateTime", "signature" : "($arg as dateTime?) as integer?", "description" : " Returns the day component of a dateTime.\n", "summary" : "<p> Returns the day component of a dateTime.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "arg", "type" : "dateTime", "occurrence" : "?", "description" : "" } ], "returns" : { "type" : "integer?", "description" : "the day component of the given dateTime. <ul xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> <li><code>day-from-dateTime(dateTime(\"1999-12-31T20:00:00-05:00\"))</code> returns <code>31</code>.</li> </ul> For a detailed description of the semantics of this function, please see <a xmlns:xqdoc=\"http://www.xqdoc.org/1.0\" href=\"http://www.w3.org/TR/xpath-functions-30/#func-day-from-dateTime\">day-from-dateTime</a>" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "days-from-duration", "qname" : "datetime:days-from-duration", "signature" : "($arg as duration?) as integer?", "description" : " Returns the number of days in a duration.\n", "summary" : "<p> Returns the number of days in a duration.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "arg", "type" : "duration", "occurrence" : "?", "description" : "" } ], "returns" : { "type" : "integer?", "description" : "the number of days in the given duration. <ul xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> <li><code>days-from-duration(dayTimeDuration(\"P3DT55H\"))</code> returns <code>5</code>.</li> </ul> For a detailed description of the semantics of this function, please see <a xmlns:xqdoc=\"http://www.xqdoc.org/1.0\" href=\"http://www.w3.org/TR/xpath-functions-30/#func-days-from-duration\">days-from-duration</a>" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 2, "name" : "format-date", "qname" : "datetime:format-date", "signature" : "($value as date?, $picture as string) as string?", "description" : " Returns a string containing a date value formatted for display.\n", "summary" : "<p> Returns a string containing a date value formatted for display.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "value", "type" : "date", "occurrence" : "?", "description" : "" }, { "name" : "picture", "type" : "string", "occurrence" : null, "description" : "" } ], "returns" : { "type" : "string?", "description" : "the string formatted according to the given picture. The following examples assume <code xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">let $d := date(\"2002-12-31\")</code> and the Gregorian calendar as the default calendar. <ul xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> <li><code>format-date($d, \"[Y0001]-[M01]-[D01]\")</code> returns <code>2002-12-31</code>.</li> <li><code>format-date($d, \"[D1] [MI] [Y]\")</code> returns <code>31 XII 2002</code>.</li> </ul> For a detailed description of the semantics of this function, please see <a xmlns:xqdoc=\"http://www.xqdoc.org/1.0\" href=\"http://www.w3.org/TR/xpath-functions-30/#func-format-date\">format-date</a>" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 5, "name" : "format-date", "qname" : "datetime:format-date", "signature" : "($value as date?, $picture as string, $language as string?, $calendar as string?, $place as string?) as string?", "description" : " Returns a string containing a date value formatted for display.\n", "summary" : "<p> Returns a string containing a date value formatted for display.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "value", "type" : "date", "occurrence" : "?", "description" : "" }, { "name" : "picture", "type" : "string", "occurrence" : null, "description" : "" }, { "name" : "language", "type" : "string", "occurrence" : "?", "description" : "" }, { "name" : "calendar", "type" : "string", "occurrence" : "?", "description" : "" }, { "name" : "place", "type" : "string", "occurrence" : "?", "description" : "" } ], "returns" : { "type" : "string?", "description" : "the string formatted according to the given picture. The following examples assume <code xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">let $d := date(\"2002-12-31\")</code> and the Gregorian calendar as the default calendar. <ul xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> <li><code>format-date($d, \"[YWw]\", \"en\", (), ())</code> returns <code>Two Thousand and Three</code>.</li> <li><code>format-date($d, \"[D] [MNn], [Y]\", \"de\", (), ())</code> returns <code>31 Dezember, 2002</code>.</li> <li><code>format-date($d, \"[D\\u0E51] [Mn] [Y\\u0E51]\", \"th\", \"BE\", ())</code> returns <code>๓๑ ธันวาคม ๒๐๐๒</code>.</li> </ul> For a detailed description of the semantics of this function, please see <a xmlns:xqdoc=\"http://www.xqdoc.org/1.0\" href=\"http://www.w3.org/TR/xpath-functions-30/#func-format-date\">format-date</a>" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 2, "name" : "format-dateTime", "qname" : "datetime:format-dateTime", "signature" : "($value as dateTime?, $picture as string) as string?", "description" : " Returns a string containing a dateTime value formatted for display.\n", "summary" : "<p> Returns a string containing a dateTime value formatted for display.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "value", "type" : "dateTime", "occurrence" : "?", "description" : "" }, { "name" : "picture", "type" : "string", "occurrence" : null, "description" : "" } ], "returns" : { "type" : "string?", "description" : "the string formatted according to the given picture. The following examples assume <code xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">let $dt := dateTime(\"2002-12-31T15:58:45\")</code> and the Gregorian calendar as the default calendar. <ul xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> <li><code>format-dateTime($dt, \"[h].[m01][Pn] on [FNn], [D1o] [MNn]\")</code> returns <code>3.58pm on Tuesday, 31st December</code>.</li> <li><code>format-dateTime($dt, \"[M01]/[D01]/[Y0001] at [H01]:[m01]:[s01]\")</code> returns <code>12/31/2002 at 15:58:45</code>.</li> </ul> For a detailed description of the semantics of this function, please see <a xmlns:xqdoc=\"http://www.xqdoc.org/1.0\" href=\"http://www.w3.org/TR/xpath-functions-30/#func-format-dateTime\">format-dateTime</a>" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 5, "name" : "format-dateTime", "qname" : "datetime:format-dateTime", "signature" : "($value as dateTime?, $picture as string, $language as string?, $calendar as string?, $place as string?) as string?", "description" : " Returns a string containing a dateTime value formatted for display.\n", "summary" : "<p> Returns a string containing a dateTime value formatted for display.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "value", "type" : "dateTime", "occurrence" : "?", "description" : "" }, { "name" : "picture", "type" : "string", "occurrence" : null, "description" : "" }, { "name" : "language", "type" : "string", "occurrence" : "?", "description" : "" }, { "name" : "calendar", "type" : "string", "occurrence" : "?", "description" : "" }, { "name" : "place", "type" : "string", "occurrence" : "?", "description" : "" } ], "returns" : { "type" : "string?", "description" : "the string formatted according to the given picture. For a detailed description of the semantics of this function, please see <a xmlns:xqdoc=\"http://www.xqdoc.org/1.0\" href=\"http://www.w3.org/TR/xpath-functions-30/#func-format-dateTime\">format-dateTime</a>" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "hours-from-dateTime", "qname" : "datetime:hours-from-dateTime", "signature" : "($arg as dateTime?) as integer?", "description" : " Returns the hours component of a dateTime.\n", "summary" : "<p> Returns the hours component of a dateTime.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "arg", "type" : "dateTime", "occurrence" : "?", "description" : "" } ], "returns" : { "type" : "integer?", "description" : "the hours component of the given dateTime. <ul xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> <li><code>hours-from-dateTime(dateTime(\"1999-12-31T21:20:00-05:00\"))</code> returns <code>21</code>.</li> <li><code>hours-from-dateTime(adjust-dateTime-to-timezone(dateTime(\"1999-12-31T21:20:00-05:00\"), dayTimeDuration(\"PT0S\")))</code> returns <code>2</code>.</li> </ul> For a detailed description of the semantics of this function, please see <a xmlns:xqdoc=\"http://www.xqdoc.org/1.0\" href=\"http://www.w3.org/TR/xpath-functions-30/#func-hours-from-time\">hours-from-dateTime</a>" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "hours-from-duration", "qname" : "datetime:hours-from-duration", "signature" : "($arg as duration?) as integer?", "description" : " Returns the number of hours in a duration.\n", "summary" : "<p> Returns the number of hours in a duration.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "arg", "type" : "duration", "occurrence" : "?", "description" : "" } ], "returns" : { "type" : "integer?", "description" : "the number of hours in the given duration. <ul xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> <li><code>hours-from-duration(dayTimeDuration(\"P3DT10H\"))</code> returns <code>10</code>.</li> <li><code>hours-from-duration(dayTimeDuration(\"-P3DT10H\"))</code> returns <code>-10</code>.</li> </ul> For a detailed description of the semantics of this function, please see <a xmlns:xqdoc=\"http://www.xqdoc.org/1.0\" href=\"http://www.w3.org/TR/xpath-functions-30/#func-hours-from-duration\">hours-from-duration</a>" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "hours-from-time", "qname" : "datetime:hours-from-time", "signature" : "($arg as time?) as integer?", "description" : " Returns the hours component of a time.\n", "summary" : "<p> Returns the hours component of a time.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "arg", "type" : "time", "occurrence" : "?", "description" : "" } ], "returns" : { "type" : "integer?", "description" : "the hours component of the given time. <ul xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> <li><code>hours-from-time(time(\"11:23:00\"))</code> returns <code>11</code>.</li> <li><code>hours-from-time(time(\"24:00:00\"))</code> returns <code>0</code>.</li> <li><code>hours-from-time(adjust-time-to-timezone(time(\"01:23:00+05:00\"), dayTimeDuration(\"PT0S\")))</code> returns <code>20</code>.</li> </ul> For a detailed description of the semantics of this function, please see <a xmlns:xqdoc=\"http://www.xqdoc.org/1.0\" href=\"http://www.w3.org/TR/xpath-functions-30/#func-hours-from-time\">hours-from-time</a>" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 0, "name" : "implicit-timezone", "qname" : "datetime:implicit-timezone", "signature" : "() as dayTimeDuration", "description" : " Returns the value of the implicit timezone property set in the platform.\n", "summary" : "<p> Returns the value of the implicit timezone property set in the platform.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [  ], "returns" : { "type" : "dayTimeDuration", "description" : "the implicit timezone property. For a detailed description of the semantics of this function, please see <a xmlns:xqdoc=\"http://www.xqdoc.org/1.0\" href=\"http://www.w3.org/TR/xpath-functions-30/#func-implicit-timezone\">implicit-timezone</a>" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "minutes-from-dateTime", "qname" : "datetime:minutes-from-dateTime", "signature" : "($arg as dateTime?) as integer?", "description" : " Returns the minute component of a dateTime.\n", "summary" : "<p> Returns the minute component of a dateTime.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "arg", "type" : "dateTime", "occurrence" : "?", "description" : "" } ], "returns" : { "type" : "integer?", "description" : "the minute component of the given dateTime. <ul xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> <li><code>minutes-from-dateTime(dateTime(\"1999-05-31T13:20:00-05:00\"))</code> returns <code>20</code>.</li> </ul> For a detailed description of the semantics of this function, please see <a xmlns:xqdoc=\"http://www.xqdoc.org/1.0\" href=\"http://www.w3.org/TR/xpath-functions-30/#func-minutes-from-dateTime\">minutes-from-dateTime</a>" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "minutes-from-duration", "qname" : "datetime:minutes-from-duration", "signature" : "($arg as duration?) as integer?", "description" : " Returns the number of minutes in a duration.\n", "summary" : "<p> Returns the number of minutes in a duration.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "arg", "type" : "duration", "occurrence" : "?", "description" : "" } ], "returns" : { "type" : "integer?", "description" : "the number of minutes of the given duration. <ul xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> <li><code>minutes-from-duration(dayTimeDuration(\"P3DT10H\"))</code> returns <code>0</code>.</li> <li><code>minutes-from-duration(dayTimeDuration(\"-P5DT12H30M\"))</code> returns <code>-30</code>.</li> </ul> For a detailed description of the semantics of this function, please see <a xmlns:xqdoc=\"http://www.xqdoc.org/1.0\" href=\"http://www.w3.org/TR/xpath-functions-30/#func-minutes-from-duration\">minutes-from-duration</a>" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "minutes-from-time", "qname" : "datetime:minutes-from-time", "signature" : "($arg as time?) as integer?", "description" : " Returns the minutes component of a time.\n", "summary" : "<p> Returns the minutes component of a time.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "arg", "type" : "time", "occurrence" : "?", "description" : "" } ], "returns" : { "type" : "integer?", "description" : "the minutes component of the given time. <ul xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> <li><code>minutes-from-time(time(\"13:00:00Z\"))</code> returns <code>0</code>.</li> </ul> For a detailed description of the semantics of this function, please see <a xmlns:xqdoc=\"http://www.xqdoc.org/1.0\" href=\"http://www.w3.org/TR/xpath-functions-30/#func-minutes-from-time\">minutes-from-time</a>" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "month-from-date", "qname" : "datetime:month-from-date", "signature" : "($arg as date?) as integer?", "description" : " Returns the month component of a date.\n", "summary" : "<p> Returns the month component of a date.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "arg", "type" : "date", "occurrence" : "?", "description" : "" } ], "returns" : { "type" : "integer?", "description" : "the month component of the given date. <ul xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> <li><code>month-from-date(date(\"1999-05-31-05:00\"))</code> returns <code>5</code>.</li> <li><code>month-from-date(date(\"2000-01-01+05:00\"))</code> returns <code>1</code>.</li> </ul> For a detailed description of the semantics of this function, please see <a xmlns:xqdoc=\"http://www.xqdoc.org/1.0\" href=\"http://www.w3.org/TR/xpath-functions-30/#func-month-from-date\">month-from-date</a>" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "month-from-dateTime", "qname" : "datetime:month-from-dateTime", "signature" : "($arg as dateTime?) as integer?", "description" : " Returns the month component of a dateTime.\n", "summary" : "<p> Returns the month component of a dateTime.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "arg", "type" : "dateTime", "occurrence" : "?", "description" : "" } ], "returns" : { "type" : "integer?", "description" : "the month component of the given dateTime. <ul xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> <li><code>month-from-dateTime(dateTime(\"1999-05-31T21:20:00-05:00\"))</code> returns <code>5</code>.</li> <li><code>month-from-dateTime(dateTime(\"2000-01-01T11:05:00+05:00\"))</code> returns <code>1</code>.</li> </ul> For a detailed description of the semantics of this function, please see <a xmlns:xqdoc=\"http://www.xqdoc.org/1.0\" href=\"http://www.w3.org/TR/xpath-functions-30/#func-month-from-dateTime\">month-from-dateTime</a>" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "months-from-duration", "qname" : "datetime:months-from-duration", "signature" : "($arg as duration?) as integer?", "description" : " Returns the number of months in a duration.\n", "summary" : "<p> Returns the number of months in a duration.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "arg", "type" : "duration", "occurrence" : "?", "description" : "" } ], "returns" : { "type" : "integer?", "description" : "the number of months in the given duration. <ul xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> <li><code>months-from-duration(yearMonthDuration(\"P20Y15M\"))</code> returns <code>3</code>.</li> <li><code>months-from-duration(yearMonthDuration(\"-P20Y18M\"))</code> returns <code>6</code>.</li> </ul> For a detailed description of the semantics of this function, please see <a xmlns:xqdoc=\"http://www.xqdoc.org/1.0\" href=\"http://www.w3.org/TR/xpath-functions-30/#func-months-from-duration\">months-from-duration</a>" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "seconds-from-dateTime", "qname" : "datetime:seconds-from-dateTime", "signature" : "($arg as dateTime?) as decimal?", "description" : " Returns the seconds component of a dateTime.\n", "summary" : "<p> Returns the seconds component of a dateTime.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "arg", "type" : "dateTime", "occurrence" : "?", "description" : "" } ], "returns" : { "type" : "decimal?", "description" : "the seconds component of the given dateTime. <ul xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> <li><code>seconds-from-dateTime(dateTime(\"1999-05-31T13:20:00-05:00\"))</code> returns <code>0</code>.</li> </ul> For a detailed description of the semantics of this function, please see <a xmlns:xqdoc=\"http://www.xqdoc.org/1.0\" href=\"http://www.w3.org/TR/xpath-functions-30/#func-seconds-from-dateTime\">seconds-from-dateTime</a>" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "seconds-from-duration", "qname" : "datetime:seconds-from-duration", "signature" : "($arg as duration?) as decimal?", "description" : " Returns the number of seconds in a duration.\n", "summary" : "<p> Returns the number of seconds in a duration.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "arg", "type" : "duration", "occurrence" : "?", "description" : "" } ], "returns" : { "type" : "decimal?", "description" : "the number of seconds in the given duration. <ul xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> <li><code>seconds-from-duration(dayTimeDuration(\"P3DT10H12.5S\"))</code> returns <code>12.5</code>.</li> <li><code>seconds-from-duration(dayTimeDuration(\"-PT256S\"))</code> returns <code>-16.0</code>.</li> </ul> For a detailed description of the semantics of this function, please see <a xmlns:xqdoc=\"http://www.xqdoc.org/1.0\" href=\"http://www.w3.org/TR/xpath-functions-30/#func-seconds-from-duration\">seconds-from-duration</a>" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "seconds-from-time", "qname" : "datetime:seconds-from-time", "signature" : "($arg as time?) as decimal?", "description" : " Returns the seconds component of a time.\n", "summary" : "<p> Returns the seconds component of a time.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "arg", "type" : "time", "occurrence" : "?", "description" : "" } ], "returns" : { "type" : "decimal?", "description" : "the seconds component of the given time. <ul xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> <li><code>seconds-from-time(time(\"13:20:10.5\"))</code> returns <code>10.5</code>.</li> </ul> For a detailed description of the semantics of this function, please see <a xmlns:xqdoc=\"http://www.xqdoc.org/1.0\" href=\"http://www.w3.org/TR/xpath-functions-30/#func-seconds-from-time\">seconds-from-time</a>" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "timezone-from-date", "qname" : "datetime:timezone-from-date", "signature" : "($arg as date?) as dayTimeDuration?", "description" : " Returns the timezone component of a date.\n", "summary" : "<p> Returns the timezone component of a date.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "arg", "type" : "date", "occurrence" : "?", "description" : "" } ], "returns" : { "type" : "dayTimeDuration?", "description" : "the timezone component of the given date. <ul xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> <li><code>timezone-from-date(date(\"1999-05-31-05:00\"))</code> returns <code>dayTimeDuration(\"-PT5H\")</code>.</li> <li><code>timezone-from-date(date(\"2000-06-12Z\"))</code> returns <code>dayTimeDuration(\"PT0S\")</code>.</li> </ul> For a detailed description of the semantics of this function, please see <a xmlns:xqdoc=\"http://www.xqdoc.org/1.0\" href=\"http://www.w3.org/TR/xpath-functions-30/#func-timezone-from-date\">timezone-from-date</a>" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "timezone-from-dateTime", "qname" : "datetime:timezone-from-dateTime", "signature" : "($arg as dateTime?) as dayTimeDuration?", "description" : " Returns the timezone component of a dateTime.\n", "summary" : "<p> Returns the timezone component of a dateTime.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "arg", "type" : "dateTime", "occurrence" : "?", "description" : "" } ], "returns" : { "type" : "dayTimeDuration?", "description" : "the timezone component of the given dateTime. <ul xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> <li><code>timezone-from-dateTime(dateTime(\"1999-05-31T13:20:00-05:00\"))</code> returns <code>dayTimeDuration(\"-PT5H\")</code>.</li> <li><code>timezone-from-dateTime(dateTime(\"2000-06-12T13:20:00Z\"))</code> returns <code>dayTimeDuration(\"PT0S\")</code>.</li> </ul> For a detailed description of the semantics of this function, please see <a xmlns:xqdoc=\"http://www.xqdoc.org/1.0\" href=\"http://www.w3.org/TR/xpath-functions-30/#func-timezone-from-dateTime\">timezone-from-dateTime</a>" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "timezone-from-time", "qname" : "datetime:timezone-from-time", "signature" : "($arg as time?) as dayTimeDuration?", "description" : " Returns the timezone component of a time.\n", "summary" : "<p> Returns the timezone component of a time.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "arg", "type" : "time", "occurrence" : "?", "description" : "" } ], "returns" : { "type" : "dayTimeDuration?", "description" : "the timezone component of the given time. <ul xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> <li><code>timezone-from-time(time(\"13:20:00-05:00\"))</code> returns <code>dayTimeDuration(\"-PT5H\")</code>.</li> <li><code>timezone-from-time(time(\"13:20:00\"))</code> returns <code>()</code>.</li> </ul> For a detailed description of the semantics of this function, please see <a xmlns:xqdoc=\"http://www.xqdoc.org/1.0\" href=\"http://www.w3.org/TR/xpath-functions-30/#func-timezone-from-time\">timezone-from-time</a>" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "year-from-date", "qname" : "datetime:year-from-date", "signature" : "($arg as date?) as integer?", "description" : " Returns the year component of a date.\n", "summary" : "<p> Returns the year component of a date.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "arg", "type" : "date", "occurrence" : "?", "description" : "" } ], "returns" : { "type" : "integer?", "description" : "the year component of the given date. <ul xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> <li><code>year-from-date(date(\"2000-01-01+05:00\"))</code> returns <code>2000</code>.</li> </ul> For a detailed description of the semantics of this function, please see <a xmlns:xqdoc=\"http://www.xqdoc.org/1.0\" href=\"http://www.w3.org/TR/xpath-functions-30/#func-year-from-date\">year-from-date</a>" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "year-from-dateTime", "qname" : "datetime:year-from-dateTime", "signature" : "($arg as dateTime?) as integer?", "description" : " Returns the year component of a dateTime.\n", "summary" : "<p> Returns the year component of a dateTime.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "arg", "type" : "dateTime", "occurrence" : "?", "description" : "" } ], "returns" : { "type" : "integer?", "description" : "the year component of the given dateTime. <ul xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> <li><code>year-from-dateTime(dateTime(\"1999-05-31T13:20:00-05:00\"))</code> returns <code>1999</code>.</li> <li><code>year-from-dateTime(dateTime(\"1999-12-31T24:00:00\"))</code> returns <code>2000</code>.</li> </ul> For a detailed description of the semantics of this function, please see <a xmlns:xqdoc=\"http://www.xqdoc.org/1.0\" href=\"http://www.w3.org/TR/xpath-functions-30/#func-year-from-dateTime\">year-from-dateTime</a>" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "years-from-duration", "qname" : "datetime:years-from-duration", "signature" : "($arg as duration?) as integer?", "description" : " Returns the number of years in a duration.\n", "summary" : "<p> Returns the number of years in a duration.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "arg", "type" : "duration", "occurrence" : "?", "description" : "" } ], "returns" : { "type" : "integer?", "description" : "the number of years in the given duration. <ul xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> <li><code>years-from-duration(yearMonthDuration(\"P20Y15M\"))</code> returns <code>21</code>.</li> <li><code>years-from-duration(yearMonthDuration(\"-P15M\"))</code> returns <code>-1</code>.</li> </ul> For a detailed description of the semantics of this function, please see <a xmlns:xqdoc=\"http://www.xqdoc.org/1.0\" href=\"http://www.w3.org/TR/xpath-functions-30/#func-years-from-duration\">years-from-duration</a>" }, "errors" : [  ] } ], "variables" : [  ] }, "http://api.28.io/jdbc" : { "ns" : "http://api.28.io/jdbc", "description" : "", "sees" : [  ], "authors" : [  ], "version" : null, "encoding" : null, "namespaces" : [ { "uri" : "", "prefix" : "an" }, { "uri" : "http://api.28.io/jdbc", "prefix" : "api" }, { "uri" : "http://www.28msec.com/modules/jdbc", "prefix" : "jdbc" }, { "uri" : "http://www.28msec.com/modules/http/request", "prefix" : "req" }, { "uri" : "http://www.28msec.com/modules/http/response", "prefix" : "res" }, { "uri" : "http://www.28msec.com/modules/store", "prefix" : "store" }, { "uri" : "http://zorba.io/errors", "prefix" : "zerr" } ], "functions" : [ { "isDocumented" : false, "arity" : 0, "name" : "dispatch", "qname" : "api:dispatch", "signature" : "()", "description" : "", "summary" : "", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [  ], "returns" : { "type" : null, "description" : "" }, "errors" : [  ] }, { "isDocumented" : false, "arity" : 0, "name" : "import", "qname" : "api:import", "signature" : "()", "description" : "", "summary" : "", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [  ], "returns" : { "type" : null, "description" : "" }, "errors" : [  ] }, { "isDocumented" : false, "arity" : 0, "name" : "tables", "qname" : "api:tables", "signature" : "()", "description" : "", "summary" : "", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [  ], "returns" : { "type" : null, "description" : "" }, "errors" : [  ] } ], "variables" : [  ] }, "http://zorba.io/modules/excel/information" : { "ns" : "http://zorba.io/modules/excel/information", "description" : " This is a library module offering the same set of functions\n defined by Microsoft Excel, under Information Functions.\n", "sees" : [ "<xqdoc:see xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"><a href=\"http://office.microsoft.com/en-us/excel/CH062528261033.aspx\" target=\"_blank\">Excel Documentation: Information Functions</a></xqdoc:see>" ], "authors" : [ "<xqdoc:author xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Sorin Nasoi</xqdoc:author>" ], "version" : null, "encoding" : "utf-8", "namespaces" : [ { "uri" : "http://zorba.io/modules/excel/errors", "prefix" : "excel-err" }, { "uri" : "http://zorba.io/modules/excel/information", "prefix" : "excel-information" }, { "uri" : "http://zorba.io/modules/excel/math", "prefix" : "excel-math" }, { "uri" : "http://zorba.io/options/versioning", "prefix" : "ver" } ], "functions" : [ { "isDocumented" : true, "arity" : 1, "name" : "is-blank", "qname" : "excel-information:is-blank", "signature" : "($value as xs:anyAtomicType?) as xs:boolean", "description" : " Test if the passed argument is empty of not.\n", "summary" : "<p> Test if the passed argument is empty of not.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "value", "type" : "xs:anyAtomicType", "occurrence" : "?", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the value.</div>" } ], "returns" : { "type" : "xs:boolean", "description" : "If the value of $arg is the empty sequence, the function returns true, otherwise the function returns false." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "is-even", "qname" : "excel-information:is-even", "signature" : "($value as xs:anyAtomicType?) as xs:boolean", "description" : " Test is a number is even.\n", "summary" : "<p> Test is a number is even.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "value", "type" : "xs:anyAtomicType", "occurrence" : "?", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the value.</div>" } ], "returns" : { "type" : "xs:boolean", "description" : "TRUE if number is even, FALSE if number is odd." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">excel-err:Value if provided value is not a number.</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 1, "name" : "is-odd", "qname" : "excel-information:is-odd", "signature" : "($value as xs:anyAtomicType?) as xs:boolean", "description" : " Test is a number is odd.\n", "summary" : "<p> Test is a number is odd.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "value", "type" : "xs:anyAtomicType", "occurrence" : "?", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the value.</div>" } ], "returns" : { "type" : "xs:boolean", "description" : "TRUE if number is odd, FALSE if number is even." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">excel-err:Value if provided value is not a number.</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 1, "name" : "islogical", "qname" : "excel-information:islogical", "signature" : "($value as xs:anyAtomicType?) as xs:boolean", "description" : " Tests if the passed $value is a logical value.\n", "summary" : "<p> Tests if the passed $value is a logical value.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "value", "type" : "xs:anyAtomicType", "occurrence" : "?", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the value.</div>" } ], "returns" : { "type" : "xs:boolean", "description" : "TRUE if $value refers to a logical value." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "isnumber", "qname" : "excel-information:isnumber", "signature" : "($value as xs:anyAtomicType?) as xs:boolean", "description" : " Tests if the passed $value is a number.\n", "summary" : "<p> Tests if the passed $value is a number.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "value", "type" : "xs:anyAtomicType", "occurrence" : "?", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the value.</div>" } ], "returns" : { "type" : "xs:boolean", "description" : "TRUE if $value refers to a number." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "istext", "qname" : "excel-information:istext", "signature" : "($value as xs:anyAtomicType?) as xs:boolean", "description" : " Tests if the passed $value is a string.\n", "summary" : "<p> Tests if the passed $value is a string.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "value", "type" : "xs:anyAtomicType", "occurrence" : "?", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the value.</div>" } ], "returns" : { "type" : "xs:boolean", "description" : "TRUE if $value refers to text." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "n", "qname" : "excel-information:n", "signature" : "($value as xs:anyAtomicType?) as xs:anyAtomicType", "description" : " Converts a $value to a number.\n", "summary" : "<p> Converts a $value to a number.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "value", "type" : "xs:anyAtomicType", "occurrence" : "?", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the value.</div>" } ], "returns" : { "type" : "xs:anyAtomicType", "description" : "A $value converted to a number." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 0, "name" : "na", "qname" : "excel-information:na", "signature" : "() as xs:anyAtomicType", "description" : " Raises the error value #N/A.\n", "summary" : "<p> Raises the error value #N/A.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [  ], "returns" : { "type" : "xs:anyAtomicType", "description" : "The error value #N/A. #N/A is the error value that means \"no value is available.\"" }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">excel-err:NA the purpose of this function is to raise this error</xqdoc:error>" ] } ], "variables" : [  ] }, "http://www.zorba-xquery.com/modules/image/animation" : { "ns" : "http://www.zorba-xquery.com/modules/image/animation", "description" : " This module provides functions to create animated GIF images.\n", "sees" : [  ], "authors" : [ "<xqdoc:author xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Daniel Thomas</xqdoc:author>" ], "version" : null, "encoding" : "utf-8", "namespaces" : [ { "uri" : "http://www.zorba-xquery.com/modules/image/animation", "prefix" : "anim" }, { "uri" : "http://www.zorba-xquery.com/modules/image/error", "prefix" : "ierr" }, { "uri" : "http://zorba.io/options/versioning", "prefix" : "ver" } ], "functions" : [ { "isDocumented" : true, "arity" : 3, "name" : "create-animated-gif", "qname" : "anim:create-animated-gif", "signature" : "($images as xs:base64Binary+, $delay as xs:unsignedInt, $iterations as xs:unsignedInt) as xs:base64Binary external", "description" : " Creates an animated GIF image.\n The resulting animated GIF shows the passed images consecutively.\n It has the same width and height as the first passed image.\n", "summary" : "<p> Creates an animated GIF image.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "images", "type" : "xs:base64Binary", "occurrence" : "+", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the image sequence</div>" }, { "name" : "delay", "type" : "xs:unsignedInt", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the hundredths of seconds an image is shown</div>" }, { "name" : "iterations", "type" : "xs:unsignedInt", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the amount of times all images are shown. 0 for infinite.</div>" } ], "returns" : { "type" : "xs:base64Binary", "description" : "the animated GIF" }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">ierr:IM001 one of the passed images is invalid.</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 4, "name" : "create-morphed-gif", "qname" : "anim:create-morphed-gif", "signature" : "($images as xs:base64Binary+, $delay as xs:unsignedInt, $iterations as xs:unsignedInt, $nr-of-morph-images as xs:unsignedInt) as xs:base64Binary external", "description" : " Creates an animated GIF image with morph effect.\n The resulting animated GIF shows the passed images consecutively with morph effect between the changes.\n It has the same width and height as the first passed image.\n", "summary" : "<p> Creates an animated GIF image with morph effect.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "images", "type" : "xs:base64Binary", "occurrence" : "+", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the image sequence</div>" }, { "name" : "delay", "type" : "xs:unsignedInt", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the hundredths of seconds an image is shown</div>" }, { "name" : "iterations", "type" : "xs:unsignedInt", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the amount of times all images are shown. 0 for infinite.</div>" }, { "name" : "nr-of-morph-images", "type" : "xs:unsignedInt", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the number of additionally added images to create the morph effect between two passed images.</div>" } ], "returns" : { "type" : "xs:base64Binary", "description" : "the animated GIF" }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">ierr:IM001 one of the passed images is invalid.</xqdoc:error>" ] } ], "variables" : [  ] }, "http://28.io/modules/error" : { "ns" : "http://28.io/modules/error", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Applications written with Sausalito use this default error module\n if an error happens inside a Sausalito project.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">An error can occur during the run time of a Sausalito project on one\n of the following cases:</p>\n <ol xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">\n   <li>If the XQuery code raises a dynamic error that is not caught in a\n     try-catch block.</li>\n   <li>If the given XQuery program is syntactically incorrect or contains\n     other static errors.</li>\n   <li>If an explicit call to fn:error() was made and the error is not\n     caught in a try-catch block.</li>\n   <li>If a request is made to a module or a function which does\n     not exist.</li>\n   <li>If any other unexpected error happens during the processing of\n     the request.</li>\n </ol>\n", "sees" : [  ], "authors" : [ "<xqdoc:author xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">28msec</xqdoc:author>" ], "version" : null, "encoding" : "utf-8", "namespaces" : [ { "uri" : "http://zorba.io/annotations", "prefix" : "an" }, { "uri" : "http://28.io/modules/error", "prefix" : "err" }, { "uri" : "http://www.28msec.com/modules/http/request", "prefix" : "request" }, { "uri" : "http://www.28msec.com/modules/http/response", "prefix" : "resp" }, { "uri" : "http://zorba.io/options/versioning", "prefix" : "ver" } ], "functions" : [ { "isDocumented" : true, "arity" : 4, "name" : "handle", "qname" : "err:handle", "signature" : "($code as xs:QName, $description as xs:string?, $value as item()*, $stack) as item()*", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">This is the default function that is called if an error happens.\n The default can be overridden by adding an error.xq library module\n into the Sausalito project. This module needs to declare a function\n with the same signature as the err:handle function in this module.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">The value returned by this function is sent to the client. If the\n default is overridden, the implementation is free to define all\n parameters of the response (e.g. set the corresponding HTTP status\n code or an arbitrary header).</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">If the default is used, the status code is 500 if the $code QName\n is not equal to one of the status code QNames declared in HTTP module.\n Otherwise, the corresponding status code is set.</p>\n", "summary" : "<p>  This is the default function that is called if an error happens.</p>", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "code", "type" : "xs:QName", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the error code that triggered the problem as a QName</div>" }, { "name" : "description", "type" : "xs:string", "occurrence" : "?", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> a description of the error</div>" }, { "name" : "value", "type" : "item()", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> a potentially empty list of items that were involved in causing the error</div>" }, { "name" : "stack", "type" : null, "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the stacktrace leading to the error as an element. For example, <tt> &lt;stack&gt; &lt;call ns=\"http://www.example.com/\" localName=\"my-function\" arity=\"3\"/&gt; &lt;/stack&gt; </tt></div>" } ], "returns" : { "type" : "item()*", "description" : "content of the response message" }, "errors" : [  ] } ], "variables" : [  ] }, "http://www.zorba-xquery.com/modules/internal-debug" : { "ns" : "http://www.zorba-xquery.com/modules/internal-debug", "description" : " This internal module provides functions for testing error handling and\n internal function caching.\n", "sees" : [  ], "authors" : [ "<xqdoc:author xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Federico Cavalieri</xqdoc:author>" ], "version" : null, "encoding" : "utf-8", "namespaces" : [ { "uri" : "http://www.zorba-xquery.com/modules/internal-debug", "prefix" : "debug" }, { "uri" : "http://zorba.io/options/versioning", "prefix" : "ver" } ], "functions" : [ { "isDocumented" : true, "arity" : 0, "name" : "cpp-error", "qname" : "debug:cpp-error", "signature" : "() as empty-sequence() external", "description" : " Raises a c++ exception.\n", "summary" : "<p> Raises a c++ exception.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [  ], "returns" : { "type" : "empty-sequence()", "description" : "empty sequence" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 0, "name" : "cpp-exit", "qname" : "debug:cpp-exit", "signature" : "() as empty-sequence() external", "description" : " C++ exit is invoked.\n", "summary" : "<p> C++ exit is invoked.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [  ], "returns" : { "type" : "empty-sequence()", "description" : "empty sequence" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 0, "name" : "user-error-no-location", "qname" : "debug:user-error-no-location", "signature" : "() as empty-sequence() external", "description" : " Raises a user error without query location.\n", "summary" : "<p> Raises a user error without query location.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [  ], "returns" : { "type" : "empty-sequence()", "description" : "empty sequence" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 0, "name" : "user-error", "qname" : "debug:user-error", "signature" : "() as empty-sequence() external", "description" : " Raises a user error.\n", "summary" : "<p> Raises a user error.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [  ], "returns" : { "type" : "empty-sequence()", "description" : "empty sequence" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 0, "name" : "zorba-error", "qname" : "debug:zorba-error", "signature" : "() as empty-sequence() external", "description" : " Raises a zorba error.\n", "summary" : "<p> Raises a zorba error.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [  ], "returns" : { "type" : "empty-sequence()", "description" : "empty sequence" }, "errors" : [  ] } ], "variables" : [  ] }, "http://zorba.io/modules/reflection" : { "ns" : "http://zorba.io/modules/reflection", "description" : " This module provides functions to dynamically invoke functions or main modules,\n respectively. Each of the functions (invoke or eval) come in four variants depending\n whether the expression being invoked is simple, nondeterministic, updating,\n or sequential.\n", "sees" : [  ], "authors" : [ "<xqdoc:author xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Nicolae Brinza</xqdoc:author>" ], "version" : null, "encoding" : "utf-8", "namespaces" : [ { "uri" : "http://zorba.io/annotations", "prefix" : "an" }, { "uri" : "http://www.w3.org/2005/xqt-errors", "prefix" : "err" }, { "uri" : "http://zorba.io/modules/reflection", "prefix" : "reflection" }, { "uri" : "http://zorba.io/options/versioning", "prefix" : "ver" } ], "functions" : [ { "isDocumented" : true, "arity" : 1, "name" : "eval-n", "qname" : "reflection:eval-n", "signature" : "($query as xs:string) as item()* external", "description" : " See documentation of reflection:eval() except the main module that is to\n be executed may be nondeterministc.\n", "summary" : "<p> See documentation of reflection:eval() except the main module that is to\n be executed may be nondeterministc.</p>", "annotation_str" : " %an:nondeterministic", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "nondeterministic", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "query", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the query string to be evaluated</div>" } ], "returns" : { "type" : "item()*", "description" : "the result of evaluating the query" }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">whatever error the evaluated XQuery may return</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">err:XQST0031 If the XQuery version of the inner program is greater than the version of the outer program.</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 1, "name" : "eval-s", "qname" : "reflection:eval-s", "signature" : "($query as xs:string) as item()* external", "description" : " See documentation of reflection:eval() except the main module that is to\n be executed may be sequential, i.e. may have side-effects.\n", "summary" : "<p> See documentation of reflection:eval() except the main module that is to\n be executed may be sequential, i.</p>", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "query", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the query string to be evaluated</div>" } ], "returns" : { "type" : "item()*", "description" : "the result of evaluating the query (the result is not supposed to contain any PUL)." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">whatever error the evaluated XQuery may return</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">err:XQST0031 If the XQuery version of the inner program is greater than the version of the outer program.</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 1, "name" : "eval-u", "qname" : "reflection:eval-u", "signature" : "($query as xs:string) external", "description" : " See documentation of reflection:eval() except the main module that is to\n be executed may be updating, i.e. return a pending update list.\n", "summary" : "<p> See documentation of reflection:eval() except the main module that is to\n be executed may be updating, i.</p>", "annotation_str" : " %an:nondeterministic", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "nondeterministic", "value" : "" } ], "updating" : true, "parameters" : [ { "name" : "query", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the query string to be evaluated</div>" } ], "returns" : { "type" : null, "description" : "the PUL resulting from evaluating the query" }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">whatever error the evaluated XQuery may return</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">err:XQST0031 If the XQuery version of the inner program is greater than the version of the outer program.</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 1, "name" : "eval", "qname" : "reflection:eval", "signature" : "($query as xs:string) as item()* external", "description" : " The purpose of this function is to (dynamically) execute an XQuery program\n from inside another XQuery program.<p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n The XQuery program that invokes the\n eval function will be referred to as the \"outer\" program and the XQuery\n program that is executed by the eval invocation will be referred to as\n the \"inner\" program. The function is given as a string argument.\n Typically, the outer program constructs this string dynamically,\n e.g., based on data extracted from documents and/or the values of\n external variables. The eval function treats this string as\n an XQuery main module. That is, it parses the string, compiles the\n resulting parse tree, executes the resulting execution plan, and finally\n returns the result or error (if any) to the outer program.<p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n The given XQuery program needs to be a valid according to XQuery's\n MainModule production (see <a xmlns:xqdoc=\"http://www.xqdoc.org/1.0\" href=\"http://www.w3.org/TR/xquery/#doc-xquery-MainModule\">\n http://www.w3.org/TR/xquery/#doc-xquery-MainModule</a>. Please note\n that the inner pogram must at least have the XQuery version of the\n outer program [err:XQST0031].<p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n The inner program \"inherits\" the static and dynamic context of the outer\n program. Specifically, evaluation of the inner program is done in static and\n dynamic contextes that are initialized as copies of the static and dynamic\n contextes of the outer program at the place where the eval invocation appears\n at. This means that, for example, all variables that are in-scope at the place\n where the eval function is invoked from, are also in-scope inside the inner\n program and can be referenced there without having to be re-declared. On the other\n hand, declarations that appear in the prolog of the inner main module or are\n imported by the inner main module from library modules, hide their corresponding\n inherited declarations. For example, if the inner main module declares\n a variable or function with the same name as an inherited variable or function,\n the inner variable/function hides the inherited one.<p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n If the inner program declares an external variable with the same name as an\n inherited variable, the value of the inherited variable is used to initialize\n the inner external variable. If, however, an inner external variable has no\n default initializer and no corresponding inherited variable, it will remain\n uninitialized, causing the inner program to raise an error when executed.\n", "summary" : "<p> The purpose of this function is to (dynamically) execute an XQuery program\n from inside another XQuery program.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "query", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the query string to be evaluated</div>" } ], "returns" : { "type" : "item()*", "description" : "the result of evaluating the query" }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">whatever error the evaluated XQuery may return.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">err:XQST0031 If the XQuery version of the inner program is greater than the version of the outer program.</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 1, "name" : "invoke-n", "qname" : "reflection:invoke-n", "signature" : "($name as xs:QName) as item()* external", "description" : " See documentation for reflection:invoke except the function that\n is to be invoked may be nondeterministic.\n", "summary" : "<p> See documentation for reflection:invoke except the function that\n is to be invoked may be nondeterministic.</p>", "annotation_str" : " %an:nondeterministic %an:variadic", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "nondeterministic", "value" : "" }, { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "variadic", "value" : "" } ], "updating" : true, "parameters" : [ { "name" : "name", "type" : "xs:QName", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the QName of the function that is to be invoked</div>" } ], "returns" : { "type" : "item()*", "description" : "the result that is returned by the invoked function" }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">whatever error the invoked function may return</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 1, "name" : "invoke-s", "qname" : "reflection:invoke-s", "signature" : "($name as xs:QName) as item()* external", "description" : " See documentation for reflection:invoke except the function that\n is to be invoked may be sequential, i.e. may have side-effects.\n", "summary" : "<p> See documentation for reflection:invoke except the function that\n is to be invoked may be sequential, i.</p>", "annotation_str" : " %an:variadic %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "variadic", "value" : "" }, { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "name", "type" : "xs:QName", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the QName of the function that is to be invoked</div>" } ], "returns" : { "type" : "item()*", "description" : "the result that is returned by the invoked function" }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">whatever error the invoked function may return</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 1, "name" : "invoke-u", "qname" : "reflection:invoke-u", "signature" : "($name as xs:QName) external", "description" : " See documentation for reflection:invoke-n except the function that\n is to be invoked may be updating, i.e. return a pending update list.\n", "summary" : "<p> See documentation for reflection:invoke-n except the function that\n is to be invoked may be updating, i.</p>", "annotation_str" : " %an:nondeterministic %an:variadic", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "nondeterministic", "value" : "" }, { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "variadic", "value" : "" } ], "updating" : true, "parameters" : [ { "name" : "name", "type" : "xs:QName", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the QName of the function that is to be invoked</div>" } ], "returns" : { "type" : null, "description" : "the result that is returned by the invoked function" }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">whatever error the invoked function may return</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 1, "name" : "invoke", "qname" : "reflection:invoke", "signature" : "($name as xs:QName) as item()* external", "description" : " The invoke function allows to dynamically call a function given its QName\n and parameters.<p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n It is possible to invoke a function whose name is not known\n at compilation time -- it can be computed, passed through an external\n variable, taken from a file, etc. The first parameter must always be a\n QName identifying a known function.<p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n The function is declared with the %an:variadic annotation. Hence, it allows\n for an arbitrary number of parameters. All of these parameters (except the\n first one) will be passed to the function that is called.\n <br xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n Example usage : <pre xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> reflection:invoke ( xs:QName(\"fn:max\"), (1,2,3) ) </pre>\n <br xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n Returns <pre xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> 3 </pre>.\n", "summary" : "<p> The invoke function allows to dynamically call a function given its QName\n and parameters.</p>", "annotation_str" : " %an:variadic", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "variadic", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "name", "type" : "xs:QName", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the QName of the function that is to be invoked</div>" } ], "returns" : { "type" : "item()*", "description" : "the result that is returned by the invoked function" }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">whatever error the invoked function may return</xqdoc:error>" ] } ], "variables" : [  ] }, "http://zorba.io/modules/hmac" : { "ns" : "http://zorba.io/modules/hmac", "description" : " This module provides functions that perform HMAC\n (hash-based message authentication code) operations.\n For example, they calculate message codes involving hash functions such\n as MD5 and various SHA variants. The result is the base64 encoded value\n of the hash. A hash may be used to verify the data integrity and\n the authenticity of a message.\n", "sees" : [  ], "authors" : [ "<xqdoc:author xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">William Candillon, Matthias Brantner</xqdoc:author>" ], "version" : null, "encoding" : "utf-8", "namespaces" : [ { "uri" : "http://zorba.io/modules/hmac", "prefix" : "hmac" }, { "uri" : "http://zorba.io/options/versioning", "prefix" : "ver" } ], "functions" : [ { "isDocumented" : true, "arity" : 3, "name" : "compute-binary", "qname" : "hmac:compute-binary", "signature" : "($message as xs:base64Binary, $secret-key as xs:string, $hash-algo as xs:string) as xs:base64Binary external", "description" : " Calculate the HMAC for the given message and secret-key involving\n an custom hash function. Before calculating the code, the given\n base64-encoded message is base64-decoded.\n", "summary" : "<p> Calculate the HMAC for the given message and secret-key involving\n an custom hash function.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "message", "type" : "xs:base64Binary", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the message to be authenticated</div>" }, { "name" : "secret-key", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the secret key used for calculating the authentication</div>" }, { "name" : "hash-algo", "type" : "xs:string", "occurrence" : null, "description" : "" } ], "returns" : { "type" : "xs:base64Binary", "description" : "the base64 encoded message authentication code" }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">hash:UNSUPPORTED-ALGORITHM if the given hash algorithm is not supported</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 3, "name" : "compute", "qname" : "hmac:compute", "signature" : "($message as xs:string, $secret-key as xs:string, $alg as xs:string) as xs:base64Binary external", "description" : " Calculate the HMAC for the given message and secret-key involving\n an custom hash function.\n", "summary" : "<p> Calculate the HMAC for the given message and secret-key involving\n an custom hash function.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "message", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the message to be authenticated</div>" }, { "name" : "secret-key", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the secret key used for calculating the authentication</div>" }, { "name" : "alg", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The algorithm to use for the hashing operation. Supported algorithms are \"md5\", \"sha1\", and \"sha256\".</div>" } ], "returns" : { "type" : "xs:base64Binary", "description" : "the base64 encoded message authentication code" }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">hash:UNSUPPORTED-ALGORITHM if the given hash algorithm is not supported</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 2, "name" : "md5-binary", "qname" : "hmac:md5-binary", "signature" : "($message as xs:base64Binary, $secret-key as xs:string) as xs:base64Binary", "description" : " Calculate the HMAC for the given message and secret-key involving\n the MD5 hash function. Before calculating the code, the given\n base64-encoded message is base64-decoded.\n", "summary" : "<p> Calculate the HMAC for the given message and secret-key involving\n the MD5 hash function.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "message", "type" : "xs:base64Binary", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the message to be authenticated</div>" }, { "name" : "secret-key", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the secret key used for calculating the authentication</div>" } ], "returns" : { "type" : "xs:base64Binary", "description" : "the base64 encoded message authentication code" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 2, "name" : "md5", "qname" : "hmac:md5", "signature" : "($message as xs:string, $secret-key as xs:string) as xs:base64Binary", "description" : " Calculate the HMAC for the given message and secret-key involving\n the MD5 hash function.\n", "summary" : "<p> Calculate the HMAC for the given message and secret-key involving\n the MD5 hash function.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "message", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the message to be authenticated</div>" }, { "name" : "secret-key", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the secret key used for calculating the authentication</div>" } ], "returns" : { "type" : "xs:base64Binary", "description" : "the base64 encoded message authentication code" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 2, "name" : "sha1-binary", "qname" : "hmac:sha1-binary", "signature" : "($message as xs:base64Binary, $secret-key as xs:string) as xs:base64Binary", "description" : " Calculate the HMAC for the given message and secret-key involving\n the SHA1 hash function. Before calculating the code, the given\n base64-encoded message is base64-decoded.\n", "summary" : "<p> Calculate the HMAC for the given message and secret-key involving\n the SHA1 hash function.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "message", "type" : "xs:base64Binary", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the message to be authenticated</div>" }, { "name" : "secret-key", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the secret key used for calculating the authentication</div>" } ], "returns" : { "type" : "xs:base64Binary", "description" : "the base64 encoded message authentication code" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 2, "name" : "sha1", "qname" : "hmac:sha1", "signature" : "($message as xs:string, $secret-key as xs:string) as xs:base64Binary", "description" : " Calculate the HMAC for the given message and secret-key involving\n the SHA1 hash function.\n", "summary" : "<p> Calculate the HMAC for the given message and secret-key involving\n the SHA1 hash function.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "message", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the message to be authenticated</div>" }, { "name" : "secret-key", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the secret key used for calculating the authentication</div>" } ], "returns" : { "type" : "xs:base64Binary", "description" : "the base64 encoded message authentication code" }, "errors" : [  ] } ], "variables" : [  ] }, "http://www.28msec.com/modules/sleep" : { "ns" : "http://www.28msec.com/modules/sleep", "description" : " This module provides a function to put the currently executing request\n to sleep.\n", "sees" : [  ], "authors" : [ "<xqdoc:author xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">28msec</xqdoc:author>" ], "version" : null, "encoding" : "utf-8", "namespaces" : [ { "uri" : "http://zorba.io/annotations", "prefix" : "an" }, { "uri" : "http://www.28msec.com/modules/sleep", "prefix" : "sleep" }, { "uri" : "http://zorba.io/options/versioning", "prefix" : "ver" } ], "functions" : [ { "isDocumented" : true, "arity" : 1, "name" : "millis", "qname" : "sleep:millis", "signature" : "($millis as xs:integer) as empty-sequence() external", "description" : " Puts the currently executing request to sleep.\n This function is mainly useful in development e.g. to simulate the\n effects of long-running tasks wrt. the concurrent execution of\n requests.\n Here the function is used to simulate a request that runs for 1 second\n <pre xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">\n declare $acquired := lock:try-acquire(\"my-lock\");\n if ($acquired)\n   sleep:millis(1000);\n else\n   fn:error(xs:QName(\"...\"), \"failed to acquire lock\");\n </pre>\n", "summary" : "<p> Puts the currently executing request to sleep.</p>", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "millis", "type" : "xs:integer", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the number of milliseconds to sleep</div>" } ], "returns" : { "type" : "empty-sequence()", "description" : "the function is sequential and returns the empty sequence" }, "errors" : [  ] } ], "variables" : [  ] }, "http://www.28msec.com/modules/cloudant" : { "ns" : "http://www.28msec.com/modules/cloudant", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">This module provides functionality for creating, reading, updating,\n deleting and searching data in <a href=\"https://cloudant.com/\">Cloudant</a>\n databases.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Before issuing a request to Cloudant it is mandatory to create\n a new connection through one of the <code>connect</code> functions.\n These functions return a connection identifier which needs to be used\n to access data in Cloudant through the other functions.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">In these functions Cloudant databases will be identified through\n their names. By default, the functions will assume that the database is owned by\n the user for which the given connection has been created.\n To specify a different database owner, most functions accept an\n <code>$options</code> parameter that allows a <code>database-owner</code>\n option to specify a different database owner for a single request.\n Moreover, when connecting, it is possible to specify the default\n database owner using the <code>connect#3</code> function.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Additional information on the Cloudant APIs can be found\n <a href=\"https://cloudant.com/for-developers/\">on the Cloudant website</a>.</p>\n <h2 xmlns:xqdoc=\"http://www.xqdoc.org/1.0\" id=\"determinism\">Important Notice Regarding Function Determinism</h2>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">The non side-effecting functions:\n <ul>\n   <li><a href=\"?anchor=list-databases-1\">list-databases#1</a></li>\n   <li><a href=\"?anchor=list-databases-2\">list-databases#2</a></li>\n   <li><a href=\"?anchor=database-info-2\">database-info#2</a></li>\n   <li><a href=\"?anchor=database-info-3\">database-info#3</a></li>\n   <li><a href=\"?anchor=all-documents-2\">all-documents#2</a></li>\n   <li><a href=\"?anchor=all-documents-3\">all-documents#3</a></li>\n   <li><a href=\"?anchor=multiple-documents-3\">multiple-documents#3</a></li>\n   <li><a href=\"?anchor=multiple-documents-4\">multiple-documents#4</a></li>\n   <li><a href=\"?anchor=document-3\">document#3</a></li>\n   <li><a href=\"?anchor=document-4\">document#4</a></li>\n   <li><a href=\"?anchor=document-info-3\">document-info#3</a></li>\n   <li><a href=\"?anchor=document-info-4\">document-info#4</a></li>\n   <li><a href=\"?anchor=attachment-4\">attachment#4</a></li>\n   <li><a href=\"?anchor=attachment-5\">attachment#5</a></li>\n   <li><a href=\"?anchor=all-view-documents-4\">all-view-documents#4</a></li>\n   <li><a href=\"?anchor=all-view-documents-5\">all-view-documents#5</a></li>\n   <li><a href=\"?anchor=multiple-view-documents-5\">multiple-view-documents#5</a></li>\n   <li><a href=\"?anchor=multiple-view-documents-6\">multiple-view-documents#6</a></li>\n   <li><a href=\"?anchor=lucene-query-5\">lucene-query#5</a></li>\n   <li><a href=\"?anchor=lucene-query-6\">lucene-query#6</a></li>\n </ul>\n are declared deterministic, which means that their results could be cached\n when invoked multiple times with the same arguments in the same query execution.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">To not use cached results you can use the following alternative functions:\n <ul>\n   <li><a href=\"?anchor=list-databases-nondeterministic-1\">list-databases-nondeterministic#1</a></li>\n   <li><a href=\"?anchor=list-databases-nondeterministic-2\">list-databases-nondeterministic#2</a></li>\n   <li><a href=\"?anchor=database-info-nondeterministic-2\">database-info-nondeterministic#2</a></li>\n   <li><a href=\"?anchor=database-info-nondeterministic-3\">database-info-nondeterministic#3</a></li>\n   <li><a href=\"?anchor=all-documents-nondeterministic-2\">all-documents-nondeterministic#2</a></li>\n   <li><a href=\"?anchor=all-documents-nondeterministic-3\">all-documents-nondeterministic#3</a></li>\n   <li><a href=\"?anchor=multiple-documents-nondeterministic-3\">multiple-documents-nondeterministic#3</a></li>\n   <li><a href=\"?anchor=multiple-documents-nondeterministic-4\">multiple-documents-nondeterministic#4</a></li>\n   <li><a href=\"?anchor=document-nondeterministic-3\">document-nondeterministic#3</a></li>\n   <li><a href=\"?anchor=document-nondeterministic-4\">document-nondeterministic#4</a></li>\n   <li><a href=\"?anchor=document-info-nondeterministic-3\">document-info-nondeterministic#3</a></li>\n   <li><a href=\"?anchor=document-info-nondeterministic-4\">document-info-nondeterministic#4</a></li>\n   <li><a href=\"?anchor=attachment-nondeterministic-4\">attachment-nondeterministic#4</a></li>\n   <li><a href=\"?anchor=attachment-nondeterministic-5\">attachment-nondeterministic#5</a></li>\n   <li><a href=\"?anchor=all-view-documents-nondeterministic-4\">all-view-documents-nondeterministic#4</a></li>\n   <li><a href=\"?anchor=all-view-documents-nondeterministic-5\">all-view-documents-nondeterministic#5</a></li>\n   <li><a href=\"?anchor=multiple-view-documents-nondeterministic-5\">multiple-view-documents-nondeterministic#5</a></li>\n   <li><a href=\"?anchor=multiple-view-documents-nondeterministic-6\">multiple-view-documents-nondeterministic#6</a></li>\n   <li><a href=\"?anchor=lucene-query-nondeterministic-5\">lucene-query-nondeterministic#5</a></li>\n   <li><a href=\"?anchor=lucene-query-nondeterministic-6\">lucene-query-nondeterministic#6</a></li>\n </ul>\n which have been declared as being non deterministic.</p>\n", "sees" : [  ], "authors" : [ "<xqdoc:author xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Federico Cavalieri</xqdoc:author>" ], "version" : null, "encoding" : "utf-8", "namespaces" : [ { "uri" : "http://zorba.io/annotations", "prefix" : "an" }, { "uri" : "http://www.28msec.com/modules/cloudant", "prefix" : "cloudant" }, { "uri" : "http://www.28msec.com/modules/credentials", "prefix" : "credentials" }, { "uri" : "http://www.w3.org/2005/xqt-errors", "prefix" : "err" }, { "uri" : "http://zorba.io/modules/http-client", "prefix" : "http" }, { "uri" : "http://www.28msec.com/modules/maps", "prefix" : "map" }, { "uri" : "http://zorba.io/modules/random", "prefix" : "random" }, { "uri" : "http://www.zorba-xquery.com/options/versioning", "prefix" : "ver" } ], "functions" : [ { "isDocumented" : true, "arity" : 2, "name" : "all-documents-nondeterministic", "qname" : "cloudant:all-documents-nondeterministic", "signature" : "($connection as anyURI, $database as string) as object()", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Lists all the documents in a given database.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">This function has the same semantics as\n <a href=\"#all-documents-2\">all-documents#2</a>,\n but is declared as being non deterministic and thus should only be used when\n result caching is not desired.</p>\n", "summary" : "<p>  Lists all the documents in a given database.</p>", "annotation_str" : " %an:nondeterministic", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "nondeterministic", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "connection", "type" : "anyURI", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> A connection identifier</div>" }, { "name" : "database", "type" : "string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> A database name</div>" } ], "returns" : { "type" : "object()", "description" : "An object listing the documents in the specified database" }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cloudant:AUTHORIZATION Authorization error</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cloudant:HTTP An HTTP error occurred when issuing the request</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cloudant:RESPONSE An error occurred parsing the server response</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cloudant:CONNECTION The specified connection does not exist</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cloudant:INTERNAL Cloudant internal error</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 3, "name" : "all-documents-nondeterministic", "qname" : "cloudant:all-documents-nondeterministic", "signature" : "($connection as anyURI, $database as string, $options as object()) as object()", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Lists all the documents in a given database.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">This function has the same semantics as\n <a href=\"#all-documents-3\">all-documents#3</a>,\n but is declared as being non deterministic and thus should only be used when\n result caching is not desired.</p>\n", "summary" : "<p>  Lists all the documents in a given database.</p>", "annotation_str" : " %an:nondeterministic", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "nondeterministic", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "connection", "type" : "anyURI", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> A connection identifier</div>" }, { "name" : "database", "type" : "string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> A database name</div>" }, { "name" : "options", "type" : "object()", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> An object specifying additional request options</div>" } ], "returns" : { "type" : "object()", "description" : "An object listing the documents in the specified database" }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cloudant:AUTHORIZATION Authorization error</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cloudant:HTTP An HTTP error occurred when issuing the request</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cloudant:RESPONSE An error occurred parsing the server response</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cloudant:CONNECTION The specified connection does not exist</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cloudant:INTERNAL Cloudant internal error</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cloudant:OPTIONS Malformed options object</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 2, "name" : "all-documents", "qname" : "cloudant:all-documents", "signature" : "($connection as anyURI, $database as string) as object()", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Lists all the documents in a given database.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">This function is declared as deterministic and should be used whenever result\n caching is acceptable.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">The information is returned as a JSON structure containing meta information\n about the return structure, and the list of documents each with its ID, revision\n and key. The key is generated from the document ID.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">For example,\n <pre>cloudant:all-documents($connection, \"db\")</pre>.\n </p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">An object with the following format is returned:\n <pre>\n {\n   \"total_rows\":3,\n   \"offset\":0,\n   \"rows\":[\n   {\n     \"id\":\"5a049246-179f-42ad-87ac-8f080426c17c\",\n     \"key\":\"5a049246-179f-42ad-87ac-8f080426c17c\",\n     \"value\":\n     {\n       \"rev\":\"2-9d5401898196997853b5ac4163857a29\"\n     }\n   },\n   {\n     \"id\":\"96f898f0-f6ff-4a9b-aac4-503992f31b01\",\n     \"key\":\"96f898f0-f6ff-4a9b-aac4-503992f31b01\",\n     \"value\":\n     {\n       \"rev\":\"2-ff7b85665c4c297838963c80ecf481a3\"\n      }\n   },\n   {\n     \"id\":\"d1f61e66-7708-4da6-aa05-7cbc33b44b7e\",\n     \"key\":\"d1f61e66-7708-4da6-aa05-7cbc33b44b7e\",\n     \"value\":\n     {\n       \"rev\":\"2-cbdef49ef3ddc127eff86350844a6108\"\n     }\n   }]\n }\n </pre>\n </p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">The fields have the following meaning:\n <ul>\n  <li>offset: offset where the document list started.</li>\n  <li>rows: array of document objects, each containing id, key and revision number.</li>\n  <li>total_rows: number of documents in the database.</li>\n  <li>update_seq: current update sequence database.</li>\n </ul>\n </p>\n", "summary" : "<p>  Lists all the documents in a given database.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "connection", "type" : "anyURI", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> A connection identifier</div>" }, { "name" : "database", "type" : "string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> A database name</div>" } ], "returns" : { "type" : "object()", "description" : "An object listing the documents in the specified database" }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cloudant:AUTHORIZATION Authorization error</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cloudant:HTTP An HTTP error occurred when issuing the request</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cloudant:RESPONSE An error occurred parsing the server response</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cloudant:CONNECTION The specified connection does not exist</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cloudant:INTERNAL Cloudant internal error</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 3, "name" : "all-documents", "qname" : "cloudant:all-documents", "signature" : "($connection as anyURI, $database as string, $options as object()) as object()", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Lists all the documents in a given database.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">This function is declared as deterministic and should be used whenever result\n caching is acceptable.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">The information is returned as a JSON structure containing meta information\n about the return structure, and the list of documents each with its ID, revision\n and key. The key is generated from the document ID.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">This function allows to specify an additional options object.\n The following options are supported:\n <ul>\n   <li>descending: return the documents in descending by key order (boolean,\n   default: false).</li>\n   <li>endkey: stop returning records when the specified key is reached (string).</li>\n   <li>endkey_docid: stop returning records when the specified document ID is\n   reached (string).</li>\n   <li>group: group the results using the reduce function to a group or single\n   row (boolean, default: false).</li>\n   <li>group_level: specify the group level to be used (numeric).</li>\n   <li>include_docs: include the full content of the documents in the return\n   (boolean, default: false).</li>\n   <li>inclusive_end: specifies whether the specified end key should be included\n   in the result (boolean, default: true).</li>\n   <li>key: return only documents that match the specified key (string).</li>\n   <li>limit: limit the number of the returned documents to the specified number\n   (numeric).</li>\n   <li>reduce: use the reduction function (boolean, default: true).</li>\n   <li>skip: skip this number of records before starting to return the results\n   (numeric, default: 0).</li>\n   <li>stale: allow the results from a stale view to be used (string, allowed\n   value: \"ok\").</li>\n   <li>startkey: start returning records when the specified key is reached (string).</li>\n   <li>startkey_docid: start returning records when the specified document ID\n   is reached (string).</li>\n   <li>database-owner: specifies the database owner (string, default: connection\n   user).</li>\n </ul>\n </p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">For example,\n <pre>cloudant:all-documents($connection, \"db\", {\"database-owner\": \"username\"})</pre>.\n </p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">An object with the following format is returned:\n <pre>\n {\n   \"total_rows\":3,\n   \"offset\":0,\n   \"rows\":[\n   {\n     \"id\":\"5a049246-179f-42ad-87ac-8f080426c17c\",\n     \"key\":\"5a049246-179f-42ad-87ac-8f080426c17c\",\n     \"value\":\n     {\n       \"rev\":\"2-9d5401898196997853b5ac4163857a29\"\n     }\n   },\n   {\n     \"id\":\"96f898f0-f6ff-4a9b-aac4-503992f31b01\",\n     \"key\":\"96f898f0-f6ff-4a9b-aac4-503992f31b01\",\n     \"value\":\n     {\n       \"rev\":\"2-ff7b85665c4c297838963c80ecf481a3\"\n      }\n   },\n   {\n     \"id\":\"d1f61e66-7708-4da6-aa05-7cbc33b44b7e\",\n     \"key\":\"d1f61e66-7708-4da6-aa05-7cbc33b44b7e\",\n     \"value\":\n     {\n       \"rev\":\"2-cbdef49ef3ddc127eff86350844a6108\"\n     }\n   }]\n }\n </pre>\n </p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">The fields have the following meaning:\n <ul>\n  <li>offset: offset where the document list started.</li>\n  <li>rows: array of document objects, each containing id, key and revision number.</li>\n  <li>total_rows: number of documents in the database.</li>\n  <li>update_seq: current update sequence database.</li>\n </ul>\n </p>\n", "summary" : "<p>  Lists all the documents in a given database.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "connection", "type" : "anyURI", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> A connection identifier</div>" }, { "name" : "database", "type" : "string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> A database name</div>" }, { "name" : "options", "type" : "object()", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> An object specifying additional request options</div>" } ], "returns" : { "type" : "object()", "description" : "An object listing the documents in the specified database" }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cloudant:AUTHORIZATION Authorization error</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cloudant:HTTP An HTTP error occurred when issuing the request</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cloudant:RESPONSE An error occurred parsing the server response</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cloudant:CONNECTION The specified connection does not exist</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cloudant:INTERNAL Cloudant internal error</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cloudant:OPTIONS Malformed options object</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 4, "name" : "all-view-documents-nondeterministic", "qname" : "cloudant:all-view-documents-nondeterministic", "signature" : "($connection as anyURI, $database as string, $design-document as string, $view as string) as object()", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Returns a JSON object describing all the documents in a given view.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">This function has the same semantics as\n <a href=\"#all-view-documents-4\">all-view-documents#4</a>,\n but is declared as being non deterministic and thus should only be used when\n result caching is not desired.</p>\n", "summary" : "<p>  Returns a JSON object describing all the documents in a given view.</p>", "annotation_str" : " %an:nondeterministic", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "nondeterministic", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "connection", "type" : "anyURI", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> A connection identifier</div>" }, { "name" : "database", "type" : "string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> A database name</div>" }, { "name" : "design-document", "type" : "string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> A design document name</div>" }, { "name" : "view", "type" : "string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> A view name</div>" } ], "returns" : { "type" : "object()", "description" : "An object listing all documents in the specified view" }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cloudant:AUTHORIZATION Authorization error</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cloudant:HTTP An HTTP error occurred when issuing the request</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cloudant:RESPONSE An error occurred parsing the server response</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cloudant:CONNECTION The specified connection does not exist</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cloudant:INTERNAL Cloudant internal error</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 5, "name" : "all-view-documents-nondeterministic", "qname" : "cloudant:all-view-documents-nondeterministic", "signature" : "($connection as anyURI, $database as string, $design-document as string, $view as string, $options as object()) as object()", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Returns a JSON object describing all the documents in a given view.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">This function has the same semantics as\n <a href=\"#all-view-documents-5\">all-view-documents#5</a>,\n but is declared as being non deterministic and thus should only be used when\n result caching is not desired.</p>\n", "summary" : "<p>  Returns a JSON object describing all the documents in a given view.</p>", "annotation_str" : " %an:nondeterministic", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "nondeterministic", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "connection", "type" : "anyURI", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> A connection identifier</div>" }, { "name" : "database", "type" : "string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> A database name</div>" }, { "name" : "design-document", "type" : "string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> A design document name</div>" }, { "name" : "view", "type" : "string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> A view name</div>" }, { "name" : "options", "type" : "object()", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> An object specifying additional request options</div>" } ], "returns" : { "type" : "object()", "description" : "An object listing all documents in the specified view" }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cloudant:AUTHORIZATION Authorization error</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cloudant:HTTP An HTTP error occurred when issuing the request</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cloudant:RESPONSE An error occurred parsing the server response</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cloudant:CONNECTION The specified connection does not exist</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cloudant:INTERNAL Cloudant internal error</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cloudant:OPTIONS Malformed options object</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 4, "name" : "all-view-documents", "qname" : "cloudant:all-view-documents", "signature" : "($connection as anyURI, $database as string, $design-document as string, $view as string) as object()", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Returns a JSON object describing all the documents in a given view.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">This function is declared as deterministic and should be used whenever result\n caching is acceptable.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">The information is returned as a JSON structure containing meta information\n about the return structure, and the list of documents each with its ID, revision\n and key. The key is generated from the document ID.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">For example,\n <pre>cloudant:all-view-documents($connection, \"db\", \"recipes\", \"by_title\")</pre>.\n </p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">An object with the following format is returned:\n <pre>\n {\n   \"offset\" : 0,\n   \"rows\" :\n   [\n     {\n       \"id\" : \"3-tiersalmonspinachandavocadoterrine\",\n       \"key\" : \"3-tier salmon, spinach and avocado terrine\",\n       \"value\" : [\"3-tier salmon, spinach and avocado terrine\"]\n     },\n     {\n       \"id\" : \"Aberffrawcake\",\n       \"key\" : \"Aberffraw cake\",\n       \"value\" : [\"Aberffraw cake\"]\n     },\n     {\n       \"id\" : \"Adukiandorangecasserole-microwave\",\n       \"key\" : \"Aduki and orange casserole - microwave\",\n       \"value\" : [\"Aduki and orange casserole - microwave\"]\n     }\n   ],\n   \"total_rows\" : 3\n }\n </pre>\n </p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">The fields have the following meaning:\n <ul>\n  <li>offset: offset where the document list started.</li>\n  <li>rows: array of document objects, each containing id, key and revision number.</li>\n  <li>total_rows: number of documents in the database.</li>\n </ul>\n </p>\n", "summary" : "<p>  Returns a JSON object describing all the documents in a given view.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "connection", "type" : "anyURI", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> A connection identifier</div>" }, { "name" : "database", "type" : "string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> A database name</div>" }, { "name" : "design-document", "type" : "string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> A design document name</div>" }, { "name" : "view", "type" : "string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> A view name</div>" } ], "returns" : { "type" : "object()", "description" : "An object listing all documents in the specified view" }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cloudant:AUTHORIZATION Authorization error</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cloudant:HTTP An HTTP error occurred when issuing the request</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cloudant:RESPONSE An error occurred parsing the server response</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cloudant:CONNECTION The specified connection does not exist</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cloudant:INTERNAL Cloudant internal error</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 5, "name" : "all-view-documents", "qname" : "cloudant:all-view-documents", "signature" : "($connection as anyURI, $database as string, $design-document as string, $view as string, $options as object()) as object()", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Returns a JSON object describing all the documents in a given view.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">This function is declared as deterministic and should be used whenever result\n caching is acceptable.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">The information is returned as a JSON structure containing meta information\n about the return structure, and the list of documents each with its ID, revision\n and key. The key is generated from the document ID.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">This function allows to specify an additional options object.\n The following options are supported:\n <ul>\n   <li>descending: return the documents in descending by key order (boolean,\n   default: false)</li>\n   <li>endkey: stop returning records when the specified key is reached (string)</li>\n   <li>endkey_docid: stop returning records when the specified document ID is\n   reached (string)</li>\n   <li>group: group the results using the reduce function to a group or single\n   row (boolean, default: false)</li>\n   <li>group_level: specify the group level to be used (numeric)</li>\n   <li>include_docs: include the full content of the documents in the return\n   (boolean, default: false)</li>\n   <li>inclusive_end: specifies whether the specified end key should be included\n   in the result (boolean, default: true)</li>\n   <li>key: return only documents that match the specified key (string)</li>\n   <li>limit: limit the number of the returned documents to the specified number\n   (numeric)</li>\n   <li>reduce: use the reduction function (boolean, default: true)</li>\n   <li>skip: skip this number of records before starting to return the results\n   (numeric, default: 0)</li>\n   <li>stale: allow the results from a stale view to be used (string, allowed\n   value: \"ok\")</li>\n   <li>startkey: start returning records when the specified key is reached\n   (string)</li>\n   <li>startkey_docid: start returning records when the specified document ID\n   is reached (string)</li>\n   <li>database-owner: specifies the database owner (string, default:\n   connection user).</li>\n </ul>\n </p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">For example,\n <pre>cloudant:all-view-documents($connection, \"db\", \"recipes\", \"by_title\",\n   {\"database-owner\" : \"username\"})</pre>.\n </p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">An object with the following format is returned:\n <pre>\n {\n   \"offset\" : 0,\n   \"rows\" :\n   [\n     {\n       \"id\" : \"3-tiersalmonspinachandavocadoterrine\",\n       \"key\" : \"3-tier salmon, spinach and avocado terrine\",\n       \"value\" : [\"3-tier salmon, spinach and avocado terrine\"]\n     },\n     {\n       \"id\" : \"Aberffrawcake\",\n       \"key\" : \"Aberffraw cake\",\n       \"value\" : [\"Aberffraw cake\"]\n     },\n     {\n       \"id\" : \"Adukiandorangecasserole-microwave\",\n       \"key\" : \"Aduki and orange casserole - microwave\",\n       \"value\" : [\"Aduki and orange casserole - microwave\"]\n     }\n   ],\n   \"total_rows\" : 3\n }\n </pre>\n </p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">The fields have the following meaning:\n <ul>\n  <li>offset: offset where the document list started.</li>\n  <li>rows: array of document objects, each containing id, key and revision\n  number.</li>\n  <li>total_rows: number of documents in the database.</li>\n </ul>\n </p>\n", "summary" : "<p>  Returns a JSON object describing all the documents in a given view.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "connection", "type" : "anyURI", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> A connection identifier</div>" }, { "name" : "database", "type" : "string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> A database name</div>" }, { "name" : "design-document", "type" : "string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> A design document name</div>" }, { "name" : "view", "type" : "string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> A view name</div>" }, { "name" : "options", "type" : "object()", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> An object specifying additional request options</div>" } ], "returns" : { "type" : "object()", "description" : "An object listing all documents in the specified view" }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cloudant:AUTHORIZATION Authorization error</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cloudant:HTTP An HTTP error occurred when issuing the request</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cloudant:RESPONSE An error occurred parsing the server response</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cloudant:CONNECTION The specified connection does not exist</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cloudant:INTERNAL Cloudant internal error</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cloudant:OPTIONS Malformed options object</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 4, "name" : "attachment-nondeterministic", "qname" : "cloudant:attachment-nondeterministic", "signature" : "($connection as anyURI, $database as string, $document-id as string, $attachment-name as string) as object()", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Retrieves the specified document attachment.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">This function has the same semantics as\n <a href=\"#attachment-4\">attachment#4</a>,\n but is declared as being non deterministic and thus should only be used when\n result caching is not desired.</p>\n", "summary" : "<p>  Retrieves the specified document attachment.</p>", "annotation_str" : " %an:nondeterministic", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "nondeterministic", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "connection", "type" : "anyURI", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> A connection identifier</div>" }, { "name" : "database", "type" : "string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> A database name</div>" }, { "name" : "document-id", "type" : "string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> A document identifier</div>" }, { "name" : "attachment-name", "type" : "string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> An attchment name</div>" } ], "returns" : { "type" : "object()", "description" : "The specified document attachment" }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cloudant:AUTHORIZATION Authorization error</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cloudant:DATABASE-NAME Invalid database name</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cloudant:HTTP An HTTP error occurred when issuing the request</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cloudant:RESPONSE An error occurred parsing the server response</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cloudant:CONNECTION The specified connection does not exist</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cloudant:INTERNAL Cloudant internal error</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 5, "name" : "attachment-nondeterministic", "qname" : "cloudant:attachment-nondeterministic", "signature" : "($connection as anyURI, $database as string, $document-id as string, $attachment-name as string, $options as object()) as object()", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Retrieves the specified document attachment.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">This function has the same semantics as <a href=\"#attachment-5\">attachment#5</a>,\n but is declared as being non deterministic and thus should only be used when\n result caching is not desired.</p>\n", "summary" : "<p>  Retrieves the specified document attachment.</p>", "annotation_str" : " %an:nondeterministic", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "nondeterministic", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "connection", "type" : "anyURI", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> A connection identifier</div>" }, { "name" : "database", "type" : "string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> A database name</div>" }, { "name" : "document-id", "type" : "string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> A document identifier</div>" }, { "name" : "attachment-name", "type" : "string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> An attchment name</div>" }, { "name" : "options", "type" : "object()", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> An object specifying additional request options</div>" } ], "returns" : { "type" : "object()", "description" : "The specified document attachment" }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cloudant:AUTHORIZATION Authorization error</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cloudant:DATABASE-NAME Invalid database name</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cloudant:ATTACHMENT-NOT-EXISTS Attachment does not exists</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cloudant:HTTP An HTTP error occurred when issuing the request</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cloudant:RESPONSE An error occurred parsing the server response</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cloudant:CONNECTION The specified connection does not exist</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cloudant:INTERNAL Cloudant internal error</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cloudant:OPTIONS Malformed options object</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 4, "name" : "attachment", "qname" : "cloudant:attachment", "signature" : "($connection as anyURI, $database as string, $document-id as string, $attachment-name as string) as object()", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Retrieves the specified document attachment.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">This function is declared as deterministic and should be used whenever result\n caching is acceptable.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">For example,\n <pre>cloudant:attachment($connection, \"db\", \"DocID\", \"Attachment\")</pre>.\n </p>\n <h4 xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Retrieving a design document attachment.</h4>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">To retrieve a design document attachment the document id must\n start with <code>_design/</code>.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">For example,\n <pre>cloudant:attachment($connection, \"db\", \"_design/DocID\", \"Attachment\")</pre>.\n </p>\n <h4 xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Return value</h4>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">The returned object reports the media-type of the attachment as\n it was specified when the attachment was submitted to the database and\n its raw content.\n The format of the returned object is the following:\n <pre>\n {\n   \"media-type\": \"text/plain\",\n   \"content\" : \"Hello World\"\n }\n </pre>\n </p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">The type of the content field is determined by the media-type returned by the\n server. If the media-type indicates that the body content is textual,\n then the content has type string, base64Binary otherwise.\n Specifically, the body content is considered textual only if the MIME-type specified in\n the media-type is one of:\n <ul>\n   <li>\"application/json\"</li>\n   <li>\"application/x-javascript\"</li>\n   <li>\"application/xml\"</li>\n   <li>\"application/xml-external-parsed-entity\"</li>\n </ul>\n or if the MIME-type starts with \"text/\" or ends with \"+xml\".</p>\n", "summary" : "<p>  Retrieves the specified document attachment.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "connection", "type" : "anyURI", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> A connection identifier</div>" }, { "name" : "database", "type" : "string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> A database name</div>" }, { "name" : "document-id", "type" : "string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> A document identifier</div>" }, { "name" : "attachment-name", "type" : "string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> An attchment name</div>" } ], "returns" : { "type" : "object()", "description" : "The specified document attachment" }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cloudant:AUTHORIZATION Authorization error</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cloudant:DATABASE-NAME Invalid database name</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cloudant:HTTP An HTTP error occurred when issuing the request</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cloudant:RESPONSE An error occurred parsing the server response</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cloudant:CONNECTION The specified connection does not exist</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cloudant:INTERNAL Cloudant internal error</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 5, "name" : "attachment", "qname" : "cloudant:attachment", "signature" : "($connection as anyURI, $database as string, $document-id as string, $attachment-name as string, $options as object()) as object()", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Retrieves the specified document attachment.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">This function is declared as deterministic and should be used whenever result\n caching is acceptable.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">This function allows to specify an additional options object.\n The following options are supported:\n <ul>\n   <li>database-owner: specifies the database owner (string, default: connection user).</li>\n </ul>\n </p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">For example,\n <pre>cloudant:attachment($connection, \"db\", \"DocID\", \"Attachment\",\n   {\"database-owner\": \"username\"})</pre>.\n </p>\n <h4 xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Retrieving a design document attachment.</h4>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">To retrieve a design document attachment the document id must\n start with <code>_design/</code>.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">For example,\n <pre>cloudant:attachment($connection, \"db\", \"_design/DocID\", \"Attachment\",\n   {\"database-owner\": \"username\"})</pre>.\n </p>\n <h4 xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Return value</h4>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">The returned object reports the media-type of the attachment as\n it was specified when the attachment was submitted to the database and\n its raw content.\n The format of the returned object is the following:\n <pre>\n {\n   \"media-type\": \"text/plain\",\n   \"content\" : \"Hello World\"\n }\n </pre>\n </p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">The type of the content field is determined by the media-type returned by the\n server. If the media-type indicates that the body content is textual,\n then the content has type string, base64Binary otherwise.\n Specifically, the body content is considered textual only if the MIME-type specified in\n the media-type is one of:\n <ul>\n   <li>\"application/json\"</li>\n   <li>\"application/x-javascript\"</li>\n   <li>\"application/xml\"</li>\n   <li>\"application/xml-external-parsed-entity\"</li>\n </ul>\n or if the MIME-type starts with \"text/\" or ends with \"+xml\".</p>\n", "summary" : "<p>  Retrieves the specified document attachment.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "connection", "type" : "anyURI", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> A connection identifier</div>" }, { "name" : "database", "type" : "string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> A database name</div>" }, { "name" : "document-id", "type" : "string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> A document identifier</div>" }, { "name" : "attachment-name", "type" : "string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> An attchment name</div>" }, { "name" : "options", "type" : "object()", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> An object specifying additional request options</div>" } ], "returns" : { "type" : "object()", "description" : "The specified document attachment" }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cloudant:AUTHORIZATION Authorization error</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cloudant:DATABASE-NAME Invalid database name</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cloudant:ATTACHMENT-NOT-EXISTS Attachment does not exists</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cloudant:HTTP An HTTP error occurred when issuing the request</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cloudant:RESPONSE An error occurred parsing the server response</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cloudant:CONNECTION The specified connection does not exist</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cloudant:INTERNAL Cloudant internal error</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cloudant:OPTIONS Malformed options object</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 3, "name" : "bulk-crud", "qname" : "cloudant:bulk-crud", "signature" : "($connection as anyURI, $database as string, $documents as object()*) as array()", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Creates, updates or deletes multiple documents with a single request.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">When creating new documents the document ID is optional. For updating\n existing documents, you must provide the document ID, revision information,\n and new document values. To delete existing documents, you must provide the\n document ID, revision information and add a field <code>_deleted</code> having\n value <code>true</code>.\n </p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">For example,\n <pre>cloudant:bulk-crud($connection, \"db\",\n (\n   {\n     \"name\":\"Nicholas\",\n     \"age\":45,\n     \"gender\":\"male\",\n   },\n   {\n     \"name\":\"Taylor\",\n     \"age\":50,\n     \"gender\":\"male\",\n     \"_id\":\"5a049246-179f-42ad-87ac-8f080426c17c\",\n   },\n   {\n     \"name\":\"Owen\",\n     \"age\":51,\n     \"gender\":\"male\",\n     \"_id\":\"d1f61e66-7708-4da6-aa05-7cbc33b44b7e\",\n     \"_rev\":\"2-f29c836d0bedc4b4b95cfaa6d99e95df\",\n   },\n   {\n     \"_id\":\"b675e932-9bb6-4fc9-b889-50238ac3512b\",\n     \"_rev\":\"2-abd3942fdab3515bfed224abed2451feb\",\n     \"_deleted\": true\n   }\n ))\n </pre>\n requires to insert the first document with a system-generated identifier, to\n insert the second one with a user-specified identifier, to update the third\n one and, finally,to delete the last one.\n </p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">\n The JSON returned by the_bulk_docs operation consists of an array\n of JSON structures, one for each submitted document.\n The returned JSON structure should be examined to ensure that all of\n the documents submitted in the original request were successfully added\n to the database. When no errors are raised, the revision of the new\n document is reported for all documents.\n </p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">\n <pre>\n [{\n    \"id\":\"96f898f0-f6ff-4a9b-aac4-503992f31b01\",\n    \"rev\":\"2-ff7b85665c4c297838963c80ecf481a3\"\n  },\n  {\n    \"id\":\"5a049246-179f-42ad-87ac-8f080426c17c\",\n    \"rev\":\"2-9d5401898196997853b5ac4163857a29\"\n  },\n  {\n    \"id\":\"d1f61e66-7708-4da6-aa05-7cbc33b44b7e\",\n    \"rev\":\"2-cbdef49ef3ddc127eff86350844a6108\"\n  },\n  {\n    \"id\": \"b675e932-9bb6-4fc9-b889-50238ac3512b\",\n    \"rev\":\"2-12356bafb1232167befabb32127823943d\"\n  }]\n </pre>\n </p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Cloudant will only guarantee that some of the documents will be\n saved when you send the request. The response will contain the list of\n documents successfully inserted or updated during the process.\n In the event of a crash, some of the documents may have been successfully\n saved, and some will have been lost.\n The response structure will indicate whether the document was updated by\n supplying the <code>rev</code> parameter indicating a new document revision\n was created. If the update failed, then you will get an error of type\n <code>conflict</code>.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">For example:\n <pre>\n [\n   {\n     \"id\":\"FishStew\",\n     \"error\":\"conflict\",\n     \"reason\":\"Document update conflict.\"\n   },\n   {\n     \"id\":\"LambStew\",\n     \"error\":\"conflict\",\n     \"reason\":\"Document update conflict.\"\n   },\n   {\n     \"id\":\"7f7638c86173eb440b8890839ff35433\",\n     \"error\":\"conflict\",\n     \"reason\":\"Document update conflict.\"\n   }\n ]\n </pre>\n In this case no new revision has been created and you will need to submit\n the document update with the correct revision tag, to update the document.\n </p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">The exact structure of each document in the returned array is:\n <ul>\n   <li>id: the document ID</li>\n   <li>rev: the new document revision, if a new revision was created</li>\n   <li>error: the error type, if an error was raised</li>\n   <li>reason: a description of the raised error, if an error was raised</li>\n </ul>\n </p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">The error type can either be conflict or forbidden. The first type means that\n the document as submitted is in conflict. The new revision has not been created\n and you will need to re-submit the document to the database. Entries with forbidden\n error type indicate that the validation routine applied to the document during submission\n has returned an error.</p>\n", "summary" : "<p>  Creates, updates or deletes multiple documents with a single request.</p>", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "connection", "type" : "anyURI", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> A connection identifier</div>" }, { "name" : "database", "type" : "string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> A database name</div>" }, { "name" : "documents", "type" : "object()", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> An sequence of documents</div>" } ], "returns" : { "type" : "array()", "description" : "An object describing the performed operations" }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cloudant:AUTHORIZATION Authorization error</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cloudant:HTTP An HTTP error occurred when issuing the request</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cloudant:RESPONSE An error occurred parsing the server response</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cloudant:CONNECTION The specified connection does not exist</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cloudant:INTERNAL Cloudant internal error</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 4, "name" : "bulk-crud", "qname" : "cloudant:bulk-crud", "signature" : "($connection as anyURI, $database as string, $documents as object()*, $options as object()) as array()", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Creates, updates or deletes multiple documents with a single request.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">When creating new documents the document ID is optional. For updating\n existing documents, you must provide the document ID, revision information,\n and new document values. To delete existing documents, you must provide the\n document ID, revision information and add a field <code>_deleted</code> having\n value <code>true</code>.\n </p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">This function allows to specify an additional options object.\n The following options are supported:\n <ul>\n   <li>all_or_nothing: sets the database commit mode to use all-or-nothing\n   semantics (boolean, default: false).</li>\n   <li>database-owner: specifies the database owner (string, default: connection\n   user).</li>\n </ul>\n </p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">For example,\n <pre>cloudant:bulk-crud($connection, \"db\",\n (\n   {\n     \"name\":\"Nicholas\",\n     \"age\":45,\n     \"gender\":\"male\",\n   },\n   {\n     \"name\":\"Taylor\",\n     \"age\":50,\n     \"gender\":\"male\",\n     \"_id\":\"5a049246-179f-42ad-87ac-8f080426c17c\",\n   },\n   {\n     \"name\":\"Owen\",\n     \"age\":51,\n     \"gender\":\"male\",\n     \"_id\":\"d1f61e66-7708-4da6-aa05-7cbc33b44b7e\",\n     \"_rev\":\"2-f29c836d0bedc4b4b95cfaa6d99e95df\",\n   },\n   {\n     \"_id\":\"b675e932-9bb6-4fc9-b889-50238ac3512b\",\n     \"_rev\":\"2-abd3942fdab3515bfed224abed2451feb\",\n     \"_deleted\": true\n   }\n ),{\"database-owner\": \"username\"})\n </pre>\n requires to insert the first document with a system-generated identifier, to\n insert the second one with a user-specified identifier, to update the third\n one and, finally, to delete the last one.\n </p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">\n The JSON returned by the_bulk_docs operation consists of an array\n of JSON structures, one for each document in the original submission.\n The returned JSON structure should be examined to ensure that all of\n the documents submitted in the original request were successfully added\n to the database. When no errors are raised, the revision of the new\n document is reported for all documents.\n </p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">\n <pre>\n [{\n    \"id\":\"96f898f0-f6ff-4a9b-aac4-503992f31b01\",\n    \"rev\":\"2-ff7b85665c4c297838963c80ecf481a3\"\n  },\n  {\n    \"id\":\"5a049246-179f-42ad-87ac-8f080426c17c\",\n    \"rev\":\"2-9d5401898196997853b5ac4163857a29\"\n  },\n  {\n    \"id\":\"d1f61e66-7708-4da6-aa05-7cbc33b44b7e\",\n    \"rev\":\"2-cbdef49ef3ddc127eff86350844a6108\"\n  },\n  {\n    \"id\": \"b675e932-9bb6-4fc9-b889-50238ac3512b\",\n    \"rev\":\"2-12356bafb1232167befabb32127823943d\"\n  }]\n </pre>\n </p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Cloudant will only guarantee that some of the documents will be\n saved when you send the request. The response will contain the list of\n documents successfully inserted or updated during the process.\n In the event of a crash, some of the documents may have been successfully\n saved, and some will have been lost.\n The response structure will indicate whether the document was updated by\n supplying the <code>rev</code> parameter indicating a new document revision\n was created. If the update failed, then you will get an error of type\n <code>conflict</code>.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">For example:\n <pre>\n [\n   {\n     \"id\":\"FishStew\",\n     \"error\":\"conflict\",\n     \"reason\":\"Document update conflict.\"\n   },\n   {\n     \"id\":\"LambStew\",\n     \"error\":\"conflict\",\n     \"reason\":\"Document update conflict.\"\n   },\n   {\n     \"id\":\"7f7638c86173eb440b8890839ff35433\",\n     \"error\":\"conflict\",\n     \"reason\":\"Document update conflict.\"\n   }\n ]\n </pre>\n In this case no new revision has been created and you will need to submit\n the document update with the correct revision tag, to update the document.\n </p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">The exact structure of each document in the returned array is:\n <ul>\n   <li>id: the document ID</li>\n   <li>rev: the new document revision, if a new revision was created</li>\n   <li>error: the error type, if an error was raised</li>\n   <li>reason: a description of the raised error, if an error was raised</li>\n </ul>\n </p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">The error type can either be conflict or forbidden. The first type means that\n the document as submitted is in conflict. The new revision has not been created\n and you will need to re-submit the document to the database. Entries with forbidden\n error type indicate that the validation routine applied to the document during submission\n has returned an error.</p>\n", "summary" : "<p>  Creates, updates or deletes multiple documents with a single request.</p>", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "connection", "type" : "anyURI", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> A connection identifier</div>" }, { "name" : "database", "type" : "string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> A database name</div>" }, { "name" : "documents", "type" : "object()", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> An sequence of documents</div>" }, { "name" : "options", "type" : "object()", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> An object specifying additional request options</div>" } ], "returns" : { "type" : "array()", "description" : "An object describing the performed operations" }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cloudant:AUTHORIZATION Authorization error</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cloudant:HTTP An HTTP error occurred when issuing the request</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cloudant:RESPONSE An error occurred parsing the server response</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cloudant:CONNECTION The specified connection does not exist</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cloudant:INTERNAL Cloudant internal error</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cloudant:OPTIONS Malformed options object</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 2, "name" : "commit-changes", "qname" : "cloudant:commit-changes", "signature" : "($connection as anyURI, $database as string) as object()", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Commits any recent changes to the specified database to disk.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">For example,\n <pre>cloudant:commit-changes($connection, \"db\")</pre>.\n </p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">An object with the following format is returned:\n <pre>\n {\n   \"ok\" : true,\n   \"instance_start_time\" : \"0\"\n }\n </pre>\n </p>\n", "summary" : "<p>  Commits any recent changes to the specified database to disk.</p>", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "connection", "type" : "anyURI", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> A connection identifier</div>" }, { "name" : "database", "type" : "string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> A database name</div>" } ], "returns" : { "type" : "object()", "description" : "An object reporting the instance start time" }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cloudant:AUTHORIZATION Authorization error</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cloudant:DATABASE-NOT-EXISTS The specified database does not exists</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cloudant:HTTP An HTTP error occurred when issuing the request</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cloudant:RESPONSE An error occurred parsing the server response</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cloudant:CONNECTION The specified connection does not exist</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cloudant:INTERNAL Cloudant internal error</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 3, "name" : "commit-changes", "qname" : "cloudant:commit-changes", "signature" : "($connection as anyURI, $database as string, $options as object()) as object()", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Commits any recent changes to the specified database to disk.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">This function allows to specify an additional options object.\n The following options are supported:\n <ul>\n   <li>database-owner: specifies the database owner (string, default: connection\n   user).</li>\n </ul>\n </p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">For example,\n <pre>\n cloudant:commit-changes($connection, \"db\", {\"database-owner\": \"username\"})\n </pre>.\n </p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">An object with the following format is returned:\n <pre>\n {\n   \"ok\" : true,\n   \"instance_start_time\" : \"0\"\n }\n </pre>\n </p>\n", "summary" : "<p>  Commits any recent changes to the specified database to disk.</p>", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "connection", "type" : "anyURI", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> A connection identifier</div>" }, { "name" : "database", "type" : "string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> A database name</div>" }, { "name" : "options", "type" : "object()", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> An object specifying additional request options</div>" } ], "returns" : { "type" : "object()", "description" : "An object reporting the instance start time" }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cloudant:AUTHORIZATION Authorization error</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cloudant:DATABASE-NOT-EXISTS The specified database does not exists</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cloudant:HTTP An HTTP error occurred when issuing the request</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cloudant:RESPONSE An error occurred parsing the server response</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cloudant:CONNECTION The specified connection does not exist</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cloudant:INTERNAL Cloudant internal error</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cloudant:OPTIONS Malformed options object</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 0, "name" : "connect", "qname" : "cloudant:connect", "signature" : "() as anyURI", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Opens a connection to Cloudant using the default credentials.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">The function returns an opaque URI that can represents the connection.\n This URI has to be passed to other functions of this module that require\n a <code>$connection</code> parameter as a first argument.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">This function requires to specify an options object.\n The following options are supported:\n <ul>\n   <li>username: the username used for connecting (string, mandatory).</li>\n   <li>password: the password used for connecting (string, mandatory).</li>\n   <li>default-database-owner: the default database owner which will be\n       used when calling the other module functions (string).</li>\n </ul>\n </p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">For example,\n <pre>cloudant:connect()</pre>.\n </p>\n", "summary" : "<p>  Opens a connection to Cloudant using the default credentials.</p>", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [  ], "returns" : { "type" : "anyURI", "description" : "An identifier that represents the connection to the server" }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cloudant:CREDENTIALS Missing or invalid credentials</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cloudant:AUTHORIZATION Authentication error</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cloudant:HTTP An HTTP error occurred when issuing the request</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cloudant:INTERNAL Cloudant internal error</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 1, "name" : "connect", "qname" : "cloudant:connect", "signature" : "($credentials as item()) as anyURI", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Opens a connection to Cloudant using the specified credentials.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">The function returns an opaque URI that can represents the connection.\n This URI has to be passed to other functions of this module that require\n a <code>$connection</code> parameter as a first argument.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">The $credentials parameter is used to specify the connection information.\n If a string is used, then the function will interpret it as the name of\n a credential in the Cloudant category.\n If an object is used, then the function will open a connection using it.\n The object structure is the following:\n <ul>\n   <li>username: the username used for connecting (string, mandatory).</li>\n   <li>password: the password used for connecting (string, mandatory).</li>\n   <li>default-database-owner: the default database owner which will be\n       used when calling the other module functions (string, optional).</li>\n </ul>\n </p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">For example, using stored credential:\n <pre>cloudant:connect(\"credentials-name\")</pre>\n For example, specifying the connection information:\n <pre>cloudant:connect(\n   {\n     \"username\": \"user\",\n     \"password: \"pass\",\n     \"owner\": \"another-user\"\n   })</pre>.\n </p>\n", "summary" : "<p>  Opens a connection to Cloudant using the specified credentials.</p>", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "credentials", "type" : "item()", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The connection specification</div>" } ], "returns" : { "type" : "anyURI", "description" : "An identifier that represents the connection to the server" }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cloudant:AUTHORIZATION Authentication error</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cloudant:HTTP An HTTP error occurred when issuing the request</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cloudant:INTERNAL Cloudant internal error</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cloudant:CREDENTIALS Missing or malformed credentials</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 2, "name" : "create-database", "qname" : "cloudant:create-database", "signature" : "($connection as anyURI, $database as string) as empty-sequence()", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Creates a new database.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">The database name must be composed of one or more of the following\n characters:\n <ul>\n  <li>Lowercase characters (a-z).</li>\n  <li>Digits (0-9).</li>\n  <li>Any of the characters: _,$,(,),+,- and /.</li>\n </ul>\n </p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">For example,\n <pre>cloudant:create-database($connection, \"db\")</pre>.\n </p>\n", "summary" : "<p>  Creates a new database.</p>", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "connection", "type" : "anyURI", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> A connection identifier</div>" }, { "name" : "database", "type" : "string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> A database name</div>" } ], "returns" : { "type" : "empty-sequence()", "description" : "Empty sequence" }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cloudant:AUTHORIZATION Authorization error</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cloudant:DATABASE-NAME Invalid database name</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cloudant:DATABASE-EXISTS Database already exists</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cloudant:HTTP An HTTP error occurred when issuing the request</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cloudant:CONNECTION The specified connection does not exist</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cloudant:INTERNAL Cloudant internal error</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 3, "name" : "create-document", "qname" : "cloudant:create-document", "signature" : "($connection as anyURI, $database as string, $document as object()) as object()", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Creates a new document in the specified database.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Creates a new document in the specified database,\n using the supplied JSON document structure. If the JSON\n structure includes the _id field, then the document will\n be created with the specified document ID. If the _id\n field is not specified, a new unique ID will be generated.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">You can include one or more attachments with a given document\n by incorporating the attachment information within the JSON of\n the document. This provides a simpler alternative to loading\n documents with attachments than making a separate call.\n To do so add you can add the <code>_attachments</code> object\n field to the document. It can have one ore more fields, each\n defining a different attachment.\n The name of the field is the name of the attachment.\n Its value is an object containing the following fields:\n <ul>\n  <li>content_type: MIME Content type string</li>\n  <li>data: File attachment content, Base64 encoded</li>\n </ul>\n </p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">For example,\n <pre>cloudant:create-document($connection, \"db\",\n   {\n     \"_id\" : \"FishStew\",\n     \"servings\" : 4,\n     \"subtitle\" : \"Delicious with fresh bread\",\n     \"title\" : \"Fish Stew\"\n     \"_attachments\" : {\n       \"styling.css\" : {\n       \"content-type\" : \"text/css\",\n       \"data\" : \"cCB7IGZvbnQtc2l6ZTogMTJwdDsgfQo=\"\n       }\n     }\n   })\n </pre>\n creates a document with an attachment named <code>styling.css</code>.\n </p>\n <h4 xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Creating a design document</h4>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">To create a design document the <code>_id</code> field must be present and must\n start with <code>_design/</code>.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">For example, the following expression creates a design document\n <pre>cloudant:create-document($connection, \"db\",\n   {\n     \"_id\": \"_design/DesDocID\",\n     \"views\": {\n       \"view1\": {\n       \"map\":\"function(doc){emit(doc.field, 1)}\",\n       \"reduce\": \"function(key, value, rereduce){return sum(values)}\"\n       }\n     },\n     \"indexes\": {\n       \"mysearch\" : {\n         \"analyzer\": {\"name\": \"portuguese\", \"stopwords\":[\"foo\", \"bar, \"baz\"]},\n         \"index\": \"function(doc){ ... }\"\n       },\n     }\n   }, {\"database-owner\": \"username\"})\n </pre>.\n </p>\n <h4 xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Return value</h4>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">The returned JSON object will report the new document revision\n and has the following form:\n <pre>\n {\n   \"id\":\"64575eef70ab90a2b8d55fc09e00440d\",\n   \"ok\":true,\n   \"rev\":\"1-9c65296036141e575d32ba9c034dd3ee\"\n }\n </pre>\n </p>\n", "summary" : "<p>  Creates a new document in the specified database.</p>", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "connection", "type" : "anyURI", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> A connection identifier</div>" }, { "name" : "database", "type" : "string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> A database name</div>" }, { "name" : "document", "type" : "object()", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> A document</div>" } ], "returns" : { "type" : "object()", "description" : "An object which contains the document id and revision" }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cloudant:AUTHORIZATION Authorization error</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cloudant:DATABASE-NAME Invalid database name</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cloudant:DOCUMENT Document creation conflict</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cloudant:HTTP An HTTP error occurred when issuing the request</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cloudant:RESPONSE An error occurred parsing the server response</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cloudant:CONNECTION The specified connection does not exist</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cloudant:INTERNAL Cloudant internal error</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 4, "name" : "create-document", "qname" : "cloudant:create-document", "signature" : "($connection as anyURI, $database as string, $document as object(), $options as object()) as object()", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Creates a new document in the specified database</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Creates a new document in the specified database,\n using the supplied JSON document structure. If the JSON\n structure includes the _id field, then the document will\n be created with the specified document ID. If the _id\n field is not specified, a new unique ID will be generated.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">You can include one or more attachments with a given document\n by incorporating the attachment information within the JSON of\n the document. This provides a simpler alternative to loading\n documents with attachments than making a separate call.\n To do so add you can add the <code>_attachments</code> object\n field to the document. It can have one ore more fields, each\n defining a different attachment.\n The name of the field is the name of the attachment.\n Its value is an object containing the following fields:\n <ul>\n  <li>content_type: MIME Content type string</li>\n  <li>data: File attachment content, Base64 encoded</li>\n </ul>\n </p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">For example,\n <pre>cloudant:create-document($connection, \"db\",\n   {\n     \"_id\" : \"FishStew\",\n     \"servings\" : 4,\n     \"subtitle\" : \"Delicious with fresh bread\",\n     \"title\" : \"Fish Stew\"\n     \"_attachments\" : {\n       \"styling.css\" : {\n       \"content-type\" : \"text/css\",\n       \"data\" : \"cCB7IGZvbnQtc2l6ZTogMTJwdDsgfQo=\"\n       }\n     }\n   }, {\"database-owner\": \"username\"})\n </pre>\n creates a document with an attachment named <code>styling.css</code>.\n </p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">This function allows to specify an additional options object.\n The following options are supported:\n <ul>\n   <li>batch: requires batch mode for insertions (string, allowed value: \"ok\")</li>\n   <li>database-owner: specifies the database owner (string, default: connection user).</li>\n </ul>\n </p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">You can write documents to the database at a higher rate\n by using the batch option. This collects document writes\n together in memory (on a user-by-user basis) before they are\n committed to disk. This increases the risk of the documents\n not being stored in the event of a failure, since the documents\n are not written to disk immediately.</p>\n <h4 xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Creating a design document</h4>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">To create a design document the <code>_id</code> field must be present and must\n start with <code>_design/</code>.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">For example, the following expression creates a design document\n <pre>cloudant:create-document($connection, \"db\",\n   {\n     \"_id\": \"_design/DesDocID\",\n     \"views\": {\n       \"view1\": {\n       \"map\":\"function(doc){emit(doc.field, 1)}\",\n       \"reduce\": \"function(key, value, rereduce){return sum(values)}\"\n       }\n     },\n     \"indexes\": {\n       \"mysearch\" : {\n         \"analyzer\": {\"name\": \"portuguese\", \"stopwords\":[\"foo\", \"bar, \"baz\"]},\n         \"index\": \"function(doc){ ... }\"\n       },\n     }\n   }, {\"database-owner\": \"username\"})\n </pre>.\n </p>\n <h4 xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Return value</h4>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">The returned JSON object will report the new document revision\n and has the following form:\n <pre>\n {\n   \"id\":\"64575eef70ab90a2b8d55fc09e00440d\",\n   \"ok\":true,\n   \"rev\":\"1-9c65296036141e575d32ba9c034dd3ee\"\n }\n </pre>\n </p>\n", "summary" : "<p>  Creates a new document in the specified database \n  Creates a new document in the specified database,\n using the supplied JSON document structure.</p>", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "connection", "type" : "anyURI", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> A connection identifier</div>" }, { "name" : "database", "type" : "string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> A database name</div>" }, { "name" : "document", "type" : "object()", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> A document</div>" }, { "name" : "options", "type" : "object()", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> An object specifying additional request options</div>" } ], "returns" : { "type" : "object()", "description" : "An object which contains the document id and revision" }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cloudant:AUTHORIZATION Authorization error</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cloudant:DATABASE-NAME Invalid database name</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cloudant:DOCUMENT Document creation conflict</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cloudant:HTTP An HTTP error occurred when issuing the request</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cloudant:RESPONSE An error occurred parsing the server response</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cloudant:CONNECTION The specified connection does not exist</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cloudant:INTERNAL Cloudant internal error</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cloudant:OPTIONS Malformed options object</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 7, "name" : "create-or-update-attachment", "qname" : "cloudant:create-or-update-attachment", "signature" : "($connection as anyURI, $database as string, $document-id as string, $document-rev as string, $attachment-name as string, $media-type as string, $attachment as atomic) as object()", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Creates or updates a document attachment.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">The latest document revision must be specified.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">For example,\n <pre>cloudant:create-or-update-attachment($connection, \"db\", \"FishStew\",\n   \"8-7c4740b4dcf26683e941d6641c00c39d\", \"AttachmentName\", \"text/plain\",\n   \"AttachmentContent\")</pre>.\n </p>\n <h4 xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Creating or updating a design document attachment.</h4>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">To create or update a design document attachment the document id must\n start with <code>_design/</code>.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">For example,\n <pre>cloudant:create-or-update-attachment($connection, \"db\", \"_design/DesDocId\",\n   \"8-7c4740b4dcf26683e941d6641c00c39d\", \"AttachmentName\", \"text/plain\",\n   \"AttachmentContent\")</pre>.\n </p>\n <h4 xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Return value</h4>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">The returned object reports the document id and its new\n revision, as follows:\n <pre>\n {\n   \"id\":\"FishStew\",\n   \"ok\":true,\n   \"rev\":\"9-247bb19a41bfd9bfdaf5ee6e2e05be74\"\n }\n </pre>\n </p>\n", "summary" : "<p>  Creates or updates a document attachment.</p>", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "connection", "type" : "anyURI", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> A connection identifier</div>" }, { "name" : "database", "type" : "string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> A database name</div>" }, { "name" : "document-id", "type" : "string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> A document identifier</div>" }, { "name" : "document-rev", "type" : "string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> A document revision</div>" }, { "name" : "attachment-name", "type" : "string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The name of the attachment</div>" }, { "name" : "media-type", "type" : "string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The media-type of the attachment</div>" }, { "name" : "attachment", "type" : "atomic", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">-name The name of the attachment</div>" } ], "returns" : { "type" : "object()", "description" : "An object containing the document id and its new revision" }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cloudant:AUTHORIZATION Authorization error</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cloudant:DATABASE-NAME Invalid database name</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cloudant:HTTP An HTTP error occurred when issuing the request</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cloudant:RESPONSE An error occurred parsing the server response</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cloudant:CONNECTION The specified connection does not exist</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cloudant:ATTACHMENT-TYPE Attachment type must be string, base64Binary or hexBinary</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cloudant:INTERNAL Cloudant internal error</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 8, "name" : "create-or-update-attachment", "qname" : "cloudant:create-or-update-attachment", "signature" : "($connection as anyURI, $database as string, $document-id as string, $document-rev as string, $attachment-name as string, $media-type as string, $attachment as atomic, $options as object()) as object()", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Creates or updates a document attachment.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">The latest document revision must be specified.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">This function allows to specify an additional options object.\n The following options are supported:\n <ul>\n   <li>database-owner: specifies the database owner (string, default: connection user).</li>\n </ul>\n </p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">For example,\n <pre>cloudant:create-or-update-attachment($connection, \"db\", \"FishStew\",\n   \"8-7c4740b4dcf26683e941d6641c00c39d\", \"AttachmentName\", \"text/plain\",\n   \"AttachmentContent\", {\"database-owner\": \"username\"})</pre>.\n </p>\n <h4 xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Creating or updating a design document attachment.</h4>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">To create or update a design document attachment the document id must\n start with <code>_design/</code>.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">For example,\n <pre>cloudant:create-or-update-attachment($connection, \"db\", \"_design/DesDocId\",\n   \"8-7c4740b4dcf26683e941d6641c00c39d\", \"AttachmentName\", \"text/plain\",\n   \"AttachmentContent\", {\"database-owner\": \"username\"})</pre>.\n </p>\n <h4 xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Return value</h4>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">The returned object reports the document id and its new\n revision, as follows:\n <pre>\n {\n   \"id\":\"FishStew\",\n   \"ok\":true,\n   \"rev\":\"9-247bb19a41bfd9bfdaf5ee6e2e05be74\"\n }\n </pre>\n </p>\n", "summary" : "<p>  Creates or updates a document attachment.</p>", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "connection", "type" : "anyURI", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> A connection identifier</div>" }, { "name" : "database", "type" : "string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> A database name</div>" }, { "name" : "document-id", "type" : "string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> A document identifier</div>" }, { "name" : "document-rev", "type" : "string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> A document revision</div>" }, { "name" : "attachment-name", "type" : "string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The name of the attachment</div>" }, { "name" : "media-type", "type" : "string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The media-type of the attachment</div>" }, { "name" : "attachment", "type" : "atomic", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">-name The name of the attachment</div>" }, { "name" : "options", "type" : "object()", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> An object specifying additional request options</div>" } ], "returns" : { "type" : "object()", "description" : "An object containing the document id and its new revision" }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cloudant:AUTHORIZATION Authorization error</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cloudant:DATABASE-NAME Invalid database name</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cloudant:HTTP An HTTP error occurred when issuing the request</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cloudant:RESPONSE An error occurred parsing the server response</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cloudant:CONNECTION The specified connection does not exist</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cloudant:ATTACHMENT-TYPE Attachment type must be string, base64Binary or hexBinary</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cloudant:INTERNAL Cloudant internal error</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cloudant:OPTIONS Malformed options object</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 3, "name" : "create-or-update-document", "qname" : "cloudant:create-or-update-document", "signature" : "($connection as anyURI, $database as string, $document as object()) as object()", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Creates or updates a document.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">When creating a new document, the <code>_id</code> field must\n be specified.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">For example,\n <pre>cloudant:create-or-update-document($connection, \"db\",\n   {\n     \"name\":\"Hannah\",\n     \"age\":120,\n     \"gender\":\"female\",\n     \"_id\":\"DocID\"\n   })</pre>.\n </p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">When updating a document, the <code>_id</code> and the\n <code>_rev</code> field, which contains the last document revision,\n must be specified.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">For example,\n <pre>cloudant:create-or-update-document($connection, \"db\",\n   {\n     \"name\":\"Hannah\",\n     \"age\":40,\n     \"gender\":\"female\",\n     \"_id\":\"DocID\",\n     \"_rev\":\"1-764b9b11845fd0b73cfa0e61acc74ecf\"\n   })</pre>.\n </p>\n <h4 xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Creating or updating a design document</h4>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">To create or update a design document the document id must\n start with <code>_design/</code>.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">For example, the following expression creates a design document\n <pre>cloudant:create-or-update-document($connection, \"db\",\n   {\n     \"_id\": \"_design/DesDocID\",\n     \"views\": {\n       \"view1\": {\n       \"map\":\"function(doc){emit(doc.field, 1)}\",\n       \"reduce\": \"function(key, value, rereduce){return sum(values)}\"\n       }\n     },\n     \"indexes\": {\n       \"mysearch\" : {\n         \"analyzer\": {\"name\": \"portuguese\", \"stopwords\":[\"foo\", \"bar, \"baz\"]},\n         \"index\": \"function(doc){ ... }\"\n       },\n     }\n   })\n </pre>.\n </p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">For example, the following expression updates a design document\n <pre>cloudant:create-or-update-document($connection, \"db\",\n   {\n     \"_id\": \"_design/DesDocID\",\n     \"_rev\": \"2-f29c836d0bedc4b4b95cfaa6d99e95df\",\n     \"views\": {\n       \"view1\": {\n       \"map\":\"function(doc){emit(doc.field, 1)}\",\n       \"reduce\": \"function(key, value, rereduce){return sum(values)}\"\n       }\n     },\n     \"indexes\": {\n       \"mysearch\" : {\n         \"analyzer\": {\"name\": \"portuguese\", \"stopwords\":[\"foo\", \"bar, \"baz\"]},\n         \"index\": \"function(doc){ ... }\"\n       },\n     }\n   })\n </pre>.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">\n The meaning of the fields in the design document is the following:\n <ul>\n   <li>_id: document ID</li>\n   <li>_rev: document revision</li>\n   <li>views: object defining the views. Each field defines a different view.\n       The field name is the name of the view and has the following content:\n     <ul>\n       <li>map: view map function</li>\n       <li>reduce: view reduce function (optional)</li>\n     </ul>\n   </li>\n   <li>indexes: object defining the view indexes. Each field defines a different index.\n       The field name is the name of the index and has the following content:\n     <ul>\n       <li>analyzer: Name of the analyzer to be used or an object with the\n       following fields:\n         <ul>\n           <li>name: Name of the analyzer</li>\n           <li>stopwords: An array of stop words. Stop words are words that\n           should not be indexed.</li>\n         </ul>\n       </li>\n       <li>index: Function that handles the indexing</li>\n     </ul>\n   </li>\n </ul>\n </p>\n <h4 xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Return value</h4>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">The format of the returned object is the following and contains\n the document id and revision.\n <pre>\n {\n   \"ok\":true,\n   \"id\":\"DocID\",\n   \"rev\":\"1-764b9b11845fd0b73cfa0e61acc74ecf\"\n }\n </pre>\n </p>\n", "summary" : "<p>  Creates or updates a document.</p>", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "connection", "type" : "anyURI", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> A connection identifier</div>" }, { "name" : "database", "type" : "string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> A database name</div>" }, { "name" : "document", "type" : "object()", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> A document</div>" } ], "returns" : { "type" : "object()", "description" : "An object which specifies the document id and its new revision" }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cloudant:AUTHORIZATION Authorization error</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cloudant:DATABASE-NAME Invalid database name</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cloudant:HTTP An HTTP error occurred when issuing the request</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cloudant:RESPONSE An error occurred parsing the server response</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cloudant:CONNECTION The specified connection does not exist</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cloudant:DOCUMENT-MALFORMED Malformed document object</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cloudant:INTERNAL Cloudant internal error</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 4, "name" : "create-or-update-document", "qname" : "cloudant:create-or-update-document", "signature" : "($connection as anyURI, $database as string, $document as object(), $options as object()) as object()", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Creates or updates a document.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">When creating a new document, the <code>_id</code> field must\n be specified.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">For example,\n <pre>cloudant:create-or-update-document($connection, \"db\", \"DocID\",\n   {\n     \"name\":\"Hannah\",\n     \"age\":120,\n     \"gender\":\"female\",\n     \"_id\":\"DocID\"\n   }, {\"database-owner\": \"username\"})</pre>.\n </p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">When updating a document, the <code>_id</code> and the\n <code>_rev</code> field, which contains the last document revision,\n must be specified.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">For example,\n <pre>cloudant:create-or-update-document($connection, \"db\", \"DocID\",\n   {\n     \"name\":\"Hannah\",\n     \"age\":40,\n     \"gender\":\"female\",\n     \"_id\":\"DocID\",\n     \"_rev\":\"1-764b9b11845fd0b73cfa0e61acc74ecf\"\n   }, {\"database-owner\": \"username\"})</pre>.\n </p>\n <h4 xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Creating or updating a design document</h4>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">To create or update a design document the document id must\n start with <code>_design/</code>.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">For example, the following expression creates a design document\n <pre>cloudant:create-or-update-document($connection, \"db\",\n   {\n     \"_id\": \"_design/DesDocID\",\n     \"views\": {\n       \"view1\": {\n       \"map\":\"function(doc){emit(doc.field, 1)}\",\n       \"reduce\": \"function(key, value, rereduce){return sum(values)}\"\n       }\n     },\n     \"indexes\": {\n       \"mysearch\" : {\n         \"analyzer\": {\"name\": \"portuguese\", \"stopwords\":[\"foo\", \"bar, \"baz\"]},\n         \"index\": \"function(doc){ ... }\"\n       },\n     }\n   }, {\"database-owner\": \"username\"})\n </pre>.\n </p>\n <pre xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cloudant:create-or-update-document($connection, \"db\",\n   {\n     \"_id\": \"_design/DesDocID\",\n     \"_rev\": \"2-f29c836d0bedc4b4b95cfaa6d99e95df\",\n     \"views\": {\n       \"view1\": {\n       \"map\":\"function(doc){emit(doc.field, 1)}\",\n       \"reduce\": \"function(key, value, rereduce){return sum(values)}\"\n       }\n     },\n     \"indexes\": {\n       \"mysearch\" : {\n         \"analyzer\": {\"name\": \"portuguese\", \"stopwords\":[\"foo\", \"bar, \"baz\"]},\n         \"index\": \"function(doc){ ... }\"\n       },\n     }\n   }, {\"database-owner\": \"username\"})\n </pre>.\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">\n The meaning of the fields in the design document is the following:\n <ul>\n   <li>_id: document ID</li>\n   <li>_rev: document revision</li>\n   <li>views: object defining the views. Each field defines a different view.\n       The field name is the name of the view and has the following content:\n     <ul>\n       <li>map: view map function</li>\n       <li>reduce: view reduce function (optional)</li>\n     </ul>\n   </li>\n   <li>indexes: object defining the view indexes. Each field defines a different index.\n       The field name is the name of the index and has the following content:\n     <ul>\n       <li>analyzer: Name of the analyzer to be used or an object with the\n       following fields:\n         <ul>\n           <li>name: Name of the analyzer</li>\n           <li>stopwords: An array of stop words. Stop words are words that\n           should not be indexed.</li>\n         </ul>\n       </li>\n       <li>index: Function that handles the indexing</li>\n     </ul>\n   </li>\n </ul>\n </p>\n <h4 xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Return value</h4>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">The format of the returned object is the following and contains\n the document id and revision.\n <pre>\n {\n   \"ok\":true,\n   \"id\":\"DocID\",\n   \"rev\":\"1-764b9b11845fd0b73cfa0e61acc74ecf\"\n }\n </pre>\n </p>\n", "summary" : "<p>  Creates or updates a document.</p>", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "connection", "type" : "anyURI", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> A connection identifier</div>" }, { "name" : "database", "type" : "string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> A database name</div>" }, { "name" : "document", "type" : "object()", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> A document</div>" }, { "name" : "options", "type" : "object()", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> An object specifying additional request options</div>" } ], "returns" : { "type" : "object()", "description" : "An object which specifies the document id and its new revision" }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cloudant:AUTHORIZATION Authorization error</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cloudant:DATABASE-NAME Invalid database name</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cloudant:HTTP An HTTP error occurred when issuing the request</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cloudant:RESPONSE An error occurred parsing the server response</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cloudant:CONNECTION The specified connection does not exist</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cloudant:DOCUMENT-MALFORMED Malformed document object</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cloudant:OPTIONS Malformed options object</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cloudant:INTERNAL Cloudant internal error</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 2, "name" : "database-info-nondeterministic", "qname" : "cloudant:database-info-nondeterministic", "signature" : "($connection as anyURI, $database as string) as object()", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Returns information about a database.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">This function has the same semantics as <a href=\"#database-info-2\">database-info#2</a>,\n but is declared as being non deterministic and thus should only be used when\n result caching is not desired.</p>\n", "summary" : "<p>  Returns information about a database.</p>", "annotation_str" : " %an:nondeterministic", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "nondeterministic", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "connection", "type" : "anyURI", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> A connection identifier</div>" }, { "name" : "database", "type" : "string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> A database name</div>" } ], "returns" : { "type" : "object()", "description" : "An object describing the specified database" }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cloudant:AUTHORIZATION Authorization error</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cloudant:HTTP An HTTP error occurred when issuing the request</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cloudant:RESPONSE An error occurred parsing the server response</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cloudant:CONNECTION The specified connection does not exist</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cloudant:INTERNAL Cloudant internal error</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 3, "name" : "database-info-nondeterministic", "qname" : "cloudant:database-info-nondeterministic", "signature" : "($connection as anyURI, $database as string, $options as object()) as object()", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Returns information about a database.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">This function has the same semantics as\n <a href=\"#database-info-3\">database-info#3</a>,\n but is declared as being non deterministic and thus should only be used when\n result caching is not desired.</p>\n", "summary" : "<p>  Returns information about a database.</p>", "annotation_str" : " %an:nondeterministic", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "nondeterministic", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "connection", "type" : "anyURI", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> A connection identifier</div>" }, { "name" : "database", "type" : "string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> a database name</div>" }, { "name" : "options", "type" : "object()", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> An object specifying additional request options</div>" } ], "returns" : { "type" : "object()", "description" : "An object describing the specified database" }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cloudant:AUTHORIZATION Authorization error</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cloudant:HTTP An HTTP error occurred when issuing the request</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cloudant:RESPONSE An error occurred parsing the server response</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cloudant:CONNECTION The specified connection does not exist</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cloudant:INTERNAL Cloudant internal error</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cloudant:OPTIONS Malformed options object</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 2, "name" : "database-info", "qname" : "cloudant:database-info", "signature" : "($connection as anyURI, $database as string) as object()", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Returns information about a database.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">This function is declared as deterministic and should be used whenever result\n caching is acceptable.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">For example,\n <pre>cloudant:database-info($connection, \"db\")</pre>.\n </p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">An object with the following format is returned:\n <pre>\n {\n   \"update_seq\": \"0-g1AAAADneJzLYWBg...\",\n   \"db_name\": \"db\",\n   \"purge_seq\": 0,\n   \"other\": {\n     \"data_size\": 0\n   },\n   \"doc_del_count\": 0,\n   \"doc_count\": 0,\n   \"disk_size\": 316,\n   \"disk_format_version\": 5,\n   \"compact_running\": false,\n   \"instance_start_time\": \"0\"\n }\n </pre>\n </p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">The fields have the following meaning:\n <ul>\n  <li>compact_running: set to true if the database compaction routine is\n  operating on this database.</li>\n  <li>db_name: the name of the database.</li>\n  <li>disk_format_version: the version of the physical format used for the data\n  when it is stored on disk.</li>\n  <li>disk_size: size in bytes of the data as stored on the disk. Views indexes\n  are not included in the calculation.</li>\n  <li>doc_count: a count of the documents in the specified database.</li>\n  <li>doc_del_count: number of deleted documents.</li>\n  <li>instance_start_time: always 0.</li>\n  <li>purge_seq: the number of purge operations on the database.</li>\n  <li>update_seq: the current number of updates to the database.</li>\n  <li>other: JSON object containing a data_size field.</li>\n </ul>\n </p>\n", "summary" : "<p>  Returns information about a database.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "connection", "type" : "anyURI", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> A connection identifier</div>" }, { "name" : "database", "type" : "string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> A database name</div>" } ], "returns" : { "type" : "object()", "description" : "An object describing the specified database" }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cloudant:AUTHORIZATION Authorization error</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cloudant:HTTP An HTTP error occurred when issuing the request</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cloudant:RESPONSE An error occurred parsing the server response</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cloudant:CONNECTION The specified connection does not exist</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cloudant:INTERNAL Cloudant internal error</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 3, "name" : "database-info", "qname" : "cloudant:database-info", "signature" : "($connection as anyURI, $database as string, $options as object()) as object()", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Returns information about a database.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">This function is declared as deterministic and should be used whenever result\n caching is acceptable.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">This function allows to specify an additional options object.\n The following options are supported:\n <ul>\n   <li>database-owner: specifies the database owner (string, default: connection user).</li>\n </ul>\n </p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">For example,\n <pre>cloudant:database-info($connection, \"db\", {\"database-owner\": \"username\"})</pre>.\n </p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">An object with the following format is returned:\n <pre>\n {\n   \"update_seq\": \"0-g1AAAADneJzLYWBg...\",\n   \"db_name\": \"db\",\n   \"purge_seq\": 0,\n   \"other\": {\n     \"data_size\": 0\n   },\n   \"doc_del_count\": 0,\n   \"doc_count\": 0,\n   \"disk_size\": 316,\n   \"disk_format_version\": 5,\n   \"compact_running\": false,\n   \"instance_start_time\": \"0\"\n }\n </pre>\n </p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">The fields have the following meaning:\n <ul>\n  <li>compact_running: set to true if the database compaction routine is\n  operating on this database.</li>\n  <li>db_name: the name of the database.</li>\n  <li>disk_format_version: the version of the physical format used for the data\n  when it is stored on disk.</li>\n  <li>disk_size: size in bytes of the data as stored on the disk. Views indexes\n  are not included in the calculation.</li>\n  <li>doc_count: a count of the documents in the specified database.</li>\n  <li>doc_del_count: number of deleted documents.</li>\n  <li>instance_start_time: always 0.</li>\n  <li>purge_seq: the number of purge operations on the database.</li>\n  <li>update_seq: the current number of updates to the database.</li>\n  <li>other: JSON object containing a data_size field.</li>\n </ul>\n </p>\n", "summary" : "<p>  Returns information about a database.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "connection", "type" : "anyURI", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> A connection identifier</div>" }, { "name" : "database", "type" : "string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> a database name</div>" }, { "name" : "options", "type" : "object()", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> An object specifying additional request options</div>" } ], "returns" : { "type" : "object()", "description" : "An object describing the specified database" }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cloudant:AUTHORIZATION Authorization error</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cloudant:HTTP An HTTP error occurred when issuing the request</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cloudant:RESPONSE An error occurred parsing the server response</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cloudant:CONNECTION The specified connection does not exist</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cloudant:INTERNAL Cloudant internal error</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cloudant:OPTIONS Malformed options object</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 5, "name" : "delete-attachment", "qname" : "cloudant:delete-attachment", "signature" : "($connection as anyURI, $database as string, $document-id as string, $document-rev as string, $attachment-name as string) as object()", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Deletes the specified attachment.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">The latest document revision must be specified.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">For example,\n <pre>cloudant:delete-attachment($connection, \"db\", \"FishStew\",\n   \"8-7c4740b4dcf26683e941d6641c00c39d\", \"AttachmentName\")</pre>.\n </p>\n <h4 xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Deleting a design document attachment.</h4>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">To delete a design document attachment the document id must\n start with <code>_design/</code>.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">For example,\n <pre>cloudant:delete-attachment($connection, \"db\", \"_design/DesDocId\",\n   \"8-7c4740b4dcf26683e941d6641c00c39d\",\n   \"AttachmentName\", \"text/plain\", \"AttachmentContent\")</pre>.\n </p>\n <h4 xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Return value</h4>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">The returned object reports the document id and its new\n revision, as follows:\n <pre>\n {\n   \"id\":\"FishStew\",\n   \"ok\":true,\n   \"rev\":\"9-247bb19a41bfd9bfdaf5ee6e2e05be74\"\n }\n </pre>\n </p>\n", "summary" : "<p>  Deletes the specified attachment.</p>", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "connection", "type" : "anyURI", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> A connection identifier</div>" }, { "name" : "database", "type" : "string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> A database name</div>" }, { "name" : "document-id", "type" : "string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> A document identifier</div>" }, { "name" : "document-rev", "type" : "string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> A document revision</div>" }, { "name" : "attachment-name", "type" : "string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The name of the attachment</div>" } ], "returns" : { "type" : "object()", "description" : "An object containing the document id and its new revision" }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cloudant:AUTHORIZATION Authorization error</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cloudant:DATABASE-NAME Invalid database name</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cloudant:ATTACHMENT-NOT-EXISTS Attachment does not exists</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cloudant:HTTP An HTTP error occurred when issuing the request</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cloudant:RESPONSE An error occurred parsing the server response</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cloudant:CONNECTION The specified connection does not exist</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cloudant:ATTACHMENT-TYPE Attachment type must be string, base64Binary or hexBinary</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cloudant:INTERNAL Cloudant internal error</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 6, "name" : "delete-attachment", "qname" : "cloudant:delete-attachment", "signature" : "($connection as anyURI, $database as string, $document-id as string, $document-rev as string, $attachment-name as string, $options as object()) as object()", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Deletes the specified attachment.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">The latest document revision must be specified.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">This function allows to specify an additional options object.\n The following options are supported:\n <ul>\n   <li>database-owner: specifies the database owner (string, default: connection user).</li>\n </ul>\n </p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">For example,\n <pre>cloudant:delete-attachment($connection, \"db\", \"FishStew\",\n \"8-7c4740b4dcf26683e941d6641c00c39d\", \"AttachmentName\",\n {\"database-owner\": \"username\"})</pre>.\n </p>\n <h4 xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Deleting a design document attachment.</h4>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">To delete a design document attachment the document id must\n start with <code>_design/</code>.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">For example,\n <pre>cloudant:delete-attachment($connection, \"db\", \"_design/DesDocId\",\n   \"8-7c4740b4dcf26683e941d6641c00c39d\", \"AttachmentName\", \"text/plain\",\n   \"AttachmentContent\", {\"database-owner\": \"username\"})</pre>.\n </p>\n <h4 xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Return value</h4>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">The returned object reports the document id and its new\n revision, as follows:\n <pre>\n {\n   \"id\":\"FishStew\",\n   \"ok\":true,\n   \"rev\":\"9-247bb19a41bfd9bfdaf5ee6e2e05be74\"\n }\n </pre>\n </p>\n", "summary" : "<p>  Deletes the specified attachment.</p>", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "connection", "type" : "anyURI", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> A connection identifier</div>" }, { "name" : "database", "type" : "string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> A database name</div>" }, { "name" : "document-id", "type" : "string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> A document identifier</div>" }, { "name" : "document-rev", "type" : "string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> A document revision</div>" }, { "name" : "attachment-name", "type" : "string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The name of the attachment</div>" }, { "name" : "options", "type" : "object()", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> An object specifying additional request options</div>" } ], "returns" : { "type" : "object()", "description" : "An object containing the document id and its new revision" }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cloudant:AUTHORIZATION Authorization error</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cloudant:DATABASE-NAME Invalid database name</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cloudant:ATTACHMENT-NOT-EXISTS Attachment does not exists</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cloudant:HTTP An HTTP error occurred when issuing the request</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cloudant:RESPONSE An error occurred parsing the server response</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cloudant:CONNECTION The specified connection does not exist</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cloudant:ATTACHMENT-TYPE Attachment type must be string, base64Binary or hexBinary</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cloudant:INTERNAL Cloudant internal error</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cloudant:OPTIONS Malformed options object</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 2, "name" : "delete-database", "qname" : "cloudant:delete-database", "signature" : "($connection as anyURI, $database as string) as empty-sequence()", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Deletes a database and all the documents and attachments contained in it.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">The database name must be composed of one or more of the following characters:\n <ul>\n  <li>Lowercase characters (a-z).</li>\n  <li>Digits (0-9).</li>\n  <li>Any of the characters: _,$,(,),+,- and /.</li>\n </ul>\n </p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">For example,\n <pre>cloudant:delete-database($connection, \"db\")</pre>.\n </p>\n", "summary" : "<p>  Deletes a database and all the documents and attachments contained in it.</p>", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "connection", "type" : "anyURI", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> A connection identifier</div>" }, { "name" : "database", "type" : "string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> A database name</div>" } ], "returns" : { "type" : "empty-sequence()", "description" : "Empty sequence" }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cloudant:AUTHORIZATION Authorization error</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cloudant:DATABASE-NAME Invalid database name</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cloudant:DATABASE-NOT-EXISTS Database does not exist</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cloudant:HTTP An HTTP error occurred when issuing the request</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cloudant:CONNECTION The specified connection does not exist</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cloudant:INTERNAL Cloudant internal error</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 4, "name" : "delete-document", "qname" : "cloudant:delete-document", "signature" : "($connection as anyURI, $database as string, $document-id as string, $document-rev as string) as object()", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Deletes the specified document from a database.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">The latest document revision must be specified.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">For example,\n <pre>\n cloudant:delete-document($connection, \"db\", \"DocID\", \"3-7c4740b4dcf26683e941d6641c00c39d\")\n </pre>.\n </p>\n <h4 xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Deleting a design document</h4>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">To delete a design document the document id must\n start with <code>_design/</code>.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">For example,\n <pre>\n cloudant:delete-document($connection, \"db\", \"_design/DesDocID\", \"3-7c4740b4dcf26683e941d6641c00c39d\")\n </pre>.\n </p>\n <h4 xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Return value</h4>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">The returned object reports the document id and its\n new revision, as follows:\n <pre>\n {\n   \"id\":\"DocID\",\n   \"ok\":true,\n   \"rev\":\"4-2719fd41187c60762ff584761b714cfb\"\n }\n </pre>\n </p>\n", "summary" : "<p>  Deletes the specified document from a database.</p>", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "connection", "type" : "anyURI", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> A connection identifier</div>" }, { "name" : "database", "type" : "string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> A database name</div>" }, { "name" : "document-id", "type" : "string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> A document identifier</div>" }, { "name" : "document-rev", "type" : "string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> A document revision</div>" } ], "returns" : { "type" : "object()", "description" : "An object which specifies the document id and its new revision" }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cloudant:AUTHORIZATION Authorization error</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cloudant:DATABASE-NAME Invalid database name</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cloudant:REVISION Specified revision is missing, invalid or not the latest.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cloudant:HTTP An HTTP error occurred when issuing the request</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cloudant:RESPONSE An error occurred parsing the server response</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cloudant:CONNECTION The specified connection does not exist</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cloudant:INTERNAL Cloudant internal error</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 5, "name" : "delete-document", "qname" : "cloudant:delete-document", "signature" : "($connection as anyURI, $database as string, $document-id as string, $document-rev as string, $options as object()) as object()", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Deletes the specified document from a database.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">The latest document revision must be specified.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">This function allows to specify an additional options object.\n The following options are supported:\n <ul>\n   <li>database-owner: specifies the database owner (string, default: connection user).</li>\n </ul>\n </p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">For example,\n <pre>\n cloudant:delete-document($connection, \"db\", \"DocID\", \"3-7c4740b4dcf26683e941d6641c00c39d\",\n   {\"database-owner\": \"username\"})\n </pre>.\n </p>\n <h4 xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Deleting a design document</h4>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">To delete a design document the document id must\n start with <code>_design/</code>.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">For example,\n <pre>\n cloudant:delete-document($connection, \"db\", \"_design/DesDocID\", \"3-7c4740b4dcf26683e941d6641c00c39d\",\n   {\"database-owner\": \"username\"})\n </pre>.\n </p>\n <h4 xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Return value</h4>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">The returned object reports the document id and its\n new revision, as follows:\n <pre>\n {\n   \"id\":\"DocID\",\n   \"ok\":true,\n   \"rev\":\"4-2719fd41187c60762ff584761b714cfb\"\n }\n </pre>\n </p>\n", "summary" : "<p>  Deletes the specified document from a database.</p>", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "connection", "type" : "anyURI", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> A connection identifier</div>" }, { "name" : "database", "type" : "string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> A database name</div>" }, { "name" : "document-id", "type" : "string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> A document identifier</div>" }, { "name" : "document-rev", "type" : "string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> A document revision</div>" }, { "name" : "options", "type" : "object()", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> An object specifying additional request options</div>" } ], "returns" : { "type" : "object()", "description" : "An object which specifies the document id and its new revision" }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cloudant:AUTHORIZATION Authorization error</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cloudant:DATABASE-NAME Invalid database name</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cloudant:REVISION Specified revision is missing, invalid or not the latest.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cloudant:HTTP An HTTP error occurred when issuing the request</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cloudant:RESPONSE An error occurred parsing the server response</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cloudant:CONNECTION The specified connection does not exist</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cloudant:INTERNAL Cloudant internal error</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cloudant:OPTIONS Malformed options object</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 3, "name" : "document-info-nondeterministic", "qname" : "cloudant:document-info-nondeterministic", "signature" : "($connection as anyURI, $database as string, $document-id as string) as object()", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Returns the latest revision and size for a given document.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">This function has the same semantics as\n <a href=\"#document-info-3\">document-info#3</a>,\n but is declared as being non deterministic and thus should only be used when\n result caching is not desired.</p>\n", "summary" : "<p>  Returns the latest revision and size for a given document.</p>", "annotation_str" : " %an:nondeterministic", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "nondeterministic", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "connection", "type" : "anyURI", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> A connection identifier</div>" }, { "name" : "database", "type" : "string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> A database name</div>" }, { "name" : "document-id", "type" : "string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> A document identifier</div>" } ], "returns" : { "type" : "object()", "description" : "An object specifying the document id, revision and size" }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cloudant:AUTHORIZATION Authorization error</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cloudant:DOCUMENT The specified document or revision cannot be found</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cloudant:HTTP An HTTP error occurred when issuing the request</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cloudant:RESPONSE An error occurred parsing the server response</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cloudant:CONNECTION The specified connection does not exist</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cloudant:INTERNAL Cloudant internal error</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 4, "name" : "document-info-nondeterministic", "qname" : "cloudant:document-info-nondeterministic", "signature" : "($connection as anyURI, $database as string, $document-id as string, $options as object()) as object()", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Returns the latest revision and size for a given document.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">This function has the same semantics as\n <a href=\"#document-info-4\">document-info#4</a>,\n but is declared as being non deterministic and thus should only be used when\n result caching is not desired.</p>\n", "summary" : "<p>  Returns the latest revision and size for a given document.</p>", "annotation_str" : " %an:nondeterministic", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "nondeterministic", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "connection", "type" : "anyURI", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> A connection identifier</div>" }, { "name" : "database", "type" : "string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> A database name</div>" }, { "name" : "document-id", "type" : "string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> A document identifier</div>" }, { "name" : "options", "type" : "object()", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> An object specifying additional request options</div>" } ], "returns" : { "type" : "object()", "description" : "An object specifying the document id, revision and size" }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cloudant:AUTHORIZATION Authorization error</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cloudant:DOCUMENT The specified document or revision cannot be found</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cloudant:HTTP An HTTP error occurred when issuing the request</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cloudant:RESPONSE An error occurred parsing the server response</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cloudant:CONNECTION The specified connection does not exist</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cloudant:INTERNAL Cloudant internal error</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cloudant:OPTIONS Malformed options object</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 3, "name" : "document-info", "qname" : "cloudant:document-info", "signature" : "($connection as anyURI, $database as string, $document-id as string) as object()", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Returns the latest revision and size for a given document.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">This function is declared as deterministic and should be used whenever result\n caching is acceptable.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">For example,\n <pre>cloudant:document-info($connection, \"db\", \"DocID\")</pre>.\n </p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">The format of the returned object is the following and contains\n the document id, latest revision and size.\n <pre>\n {\n   \"ok\": true,\n   \"id\": \"DocID\",\n   \"rev\": \"1-764b9b11845fd0b73cfa0e61acc74ecf\",\n   \"size\": 500\n }\n </pre>\n </p>\n <h4 xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Returning the latest revision and size for a given design document</h4>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">To return the latest revision and size of a design document the document\n id must start with\n <code>_design/</code>.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">For example,\n <pre>cloudant:document-info($connection, \"db\", \"_design/DesDocID\")</pre>.\n </p>\n", "summary" : "<p>  Returns the latest revision and size for a given document.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "connection", "type" : "anyURI", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> A connection identifier</div>" }, { "name" : "database", "type" : "string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> A database name</div>" }, { "name" : "document-id", "type" : "string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> A document identifier</div>" } ], "returns" : { "type" : "object()", "description" : "An object specifying the document id, revision and size" }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cloudant:AUTHORIZATION Authorization error</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cloudant:DOCUMENT The specified document or revision cannot be found</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cloudant:HTTP An HTTP error occurred when issuing the request</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cloudant:RESPONSE An error occurred parsing the server response</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cloudant:CONNECTION The specified connection does not exist</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cloudant:INTERNAL Cloudant internal error</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 4, "name" : "document-info", "qname" : "cloudant:document-info", "signature" : "($connection as anyURI, $database as string, $document-id as string, $options as object()) as object()", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Returns the latest revision and size for a given document.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">This function is declared as deterministic and should be used whenever result\n caching is acceptable.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">This function allows to specify an additional options object.\n The following options are supported:\n <ul>\n   <li>database-owner: specifies the database owner (string, default: connection user).</li>\n </ul>\n </p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">For example,\n <pre>\n cloudant:document-info($connection, \"db\", \"DocID\", {\"database-owner\": \"username\"})\n </pre>.\n </p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">The format of the returned object is the following and contains\n the document id, latest revision and size.\n <pre>\n {\n   \"ok\": true,\n   \"id\": \"DocID\",\n   \"rev\": \"1-764b9b11845fd0b73cfa0e61acc74ecf\",\n   \"size\": 500\n }\n </pre>\n </p>\n <h4 xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Returning the latest revision and size for a given design document</h4>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">To return the latest revision and size of a design document the document\n id must start with <code>_design/</code>.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">For example,\n <pre>\n cloudant:document-info($connection, \"db\", \"_design/DesDocID\", {\"database-owner\": \"username\"})\n </pre>.\n </p>\n", "summary" : "<p>  Returns the latest revision and size for a given document.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "connection", "type" : "anyURI", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> A connection identifier</div>" }, { "name" : "database", "type" : "string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> A database name</div>" }, { "name" : "document-id", "type" : "string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> A document identifier</div>" }, { "name" : "options", "type" : "object()", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> An object specifying additional request options</div>" } ], "returns" : { "type" : "object()", "description" : "An object specifying the document id, revision and size" }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cloudant:AUTHORIZATION Authorization error</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cloudant:DOCUMENT The specified document or revision cannot be found</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cloudant:HTTP An HTTP error occurred when issuing the request</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cloudant:RESPONSE An error occurred parsing the server response</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cloudant:CONNECTION The specified connection does not exist</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cloudant:INTERNAL Cloudant internal error</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cloudant:OPTIONS Malformed options object</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 3, "name" : "document-nondeterministic", "qname" : "cloudant:document-nondeterministic", "signature" : "($connection as anyURI, $database as string, $document-id as string) as object()", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Retrieves a document from the specified database.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">This function has the same semantics as <a href=\"#document-3\">document#3</a>,\n but is declared as being non deterministic and thus should only be used when\n result caching is not desired.</p>\n", "summary" : "<p>  Retrieves a document from the specified database.</p>", "annotation_str" : " %an:nondeterministic", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "nondeterministic", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "connection", "type" : "anyURI", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> A connection identifier</div>" }, { "name" : "database", "type" : "string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> A database name</div>" }, { "name" : "document-id", "type" : "string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> A document identifier</div>" } ], "returns" : { "type" : "object()", "description" : "The specified document" }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cloudant:AUTHORIZATION Authorization error</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cloudant:DATABASE-NAME Invalid database name</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cloudant:DOCUMENT The specified document or revision cannot be found</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cloudant:HTTP An HTTP error occurred when issuing the request</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cloudant:RESPONSE An error occurred parsing the server response</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cloudant:CONNECTION The specified connection does not exist</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cloudant:INTERNAL Cloudant internal error</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 4, "name" : "document-nondeterministic", "qname" : "cloudant:document-nondeterministic", "signature" : "($connection as anyURI, $database as string, $document-id as string, $options as object()) as object()", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Retrieves a document from the specified database</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">This function has the same semantics as <a href=\"#document-4\">document#4</a>,\n but is declared as being non deterministic and thus should only be used when\n result caching is not desired.</p>\n", "summary" : "<p>  Retrieves a document from the specified database \n  This function has the same semantics as  document#4 ,\n but is declared as being non deterministic and thus should only be used when\n result caching is not desired.</p>", "annotation_str" : " %an:nondeterministic", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "nondeterministic", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "connection", "type" : "anyURI", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> A connection identifier</div>" }, { "name" : "database", "type" : "string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> A database name</div>" }, { "name" : "document-id", "type" : "string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> A document identifier</div>" }, { "name" : "options", "type" : "object()", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> An object specifying additional request options</div>" } ], "returns" : { "type" : "object()", "description" : "The specified document" }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cloudant:AUTHORIZATION Authorization error</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cloudant:DATABASE-NAME Invalid database name</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cloudant:DOCUMENT The specified document or revision cannot be found</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cloudant:HTTP An HTTP error occurred when issuing the request</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cloudant:RESPONSE An error occurred parsing the server response</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cloudant:CONNECTION The specified connection does not exist</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cloudant:INTERNAL Cloudant internal error</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cloudant:OPTIONS Malformed options object</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 3, "name" : "document", "qname" : "cloudant:document", "signature" : "($connection as anyURI, $database as string, $document-id as string) as object()", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Retrieves a document from the specified database.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">This function is declared as deterministic and should be used whenever result\n caching is acceptable.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">The latest revision of the document will be returned.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">If the document includes attachments, then the\n returned structure will contain a summary of the\n attachments associated with the document, but not the\n attachment data itself.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">For example,\n <pre>cloudant:document($connection, \"db\", \"DocID\")</pre>.\n </p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">The returned JSON object will contain the document\n and has the following format:\n <pre>\n {\n   \"_id\": \"DocID\",\n   \"_rev\": \"2-f29c836d0bedc4b4b95cfaa6d99e95df\",\n   \"name\": \"Anna\",\n   \"age\": 89,\n   \"gender\": \"female\",\n   \"_attachments\": {\n     \"my attachment\": {\n       \"content_type\": \"application/json; charset=UTF-8\",\n       \"revpos\": 2,\n       \"digest\": \"md5-37IZysiyWLRWx31J/1WQHw==\",\n       \"length\": 12,\n       \"stub\": true\n       }\n     }\n   }\n </pre>\n </p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">\n The meaning of the fields in the returned object is the following:\n <ul>\n   <li>_id: document ID</li>\n   <li>_rev: document revision</li>\n   <li>attachments: document attachments (optional), each field denotes the name\n       of a different attachment\n     <ul>\n       <li>content_type: attachment MIME Content type string</li>\n       <li>length: attachment length in bytes</li>\n       <li>revpos: revision where this attachment exists</li>\n       <li>digest: MD5 checksum of the attachment</li>\n       <li>stub: indicates whether the attachment is a stub</li>\n    </ul>\n  </li>\n </ul>\n </p>\n <h4 xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Retrieving a design document from the specified database</h4>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">To retrieve a design document the document id must start with\n <code>_design/</code>.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">For example,\n <pre>cloudant:document($connection, \"db\", \"_design/DesDocID\")</pre>.\n </p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">A JSON object with the following format is returned:\n <pre>\n {\n  \"_id\": \"_design/DesDocID\",\n   \"views\": {\n     \"view1\": {\n       \"map\":\"function(doc){emit(doc.field, 1)}\",\n       \"reduce\": \"function(key, value, rereduce){return sum(values)}\"\n     }\n   },\n   \"indexes\": {\n     \"mysearch\" : {\n       \"analyzer\": {\"name\": \"portuguese\", \"stopwords\":[\"foo\", \"bar, \"baz\"]},\n       \"index\": \"function(doc){ ... }\"\n     },\n   }\n }\n </pre>\n </p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">\n The meaning of the fields in the returned object is the following:\n <ul>\n   <li>_id: document ID</li>\n   <li>_rev: document revision</li>\n   <li>views: object defining the views. Each field defines a different view.\n       The field name is the name of the view and has the following content:\n     <ul>\n       <li>map: view map function</li>\n       <li>reduce: view reduce function (optional)</li>\n     </ul>\n   </li>\n   <li>indexes: object defining the view indexes. Each field defines a different index.\n       The field name is the name of the index and has the following content:\n     <ul>\n       <li>analyzer: Name of the analyzer to be used or an object with the\n       following fields:\n         <ul>\n           <li>name: Name of the analyzer</li>\n           <li>stopwords: An array of stop words. Stop words are words that\n           should not be indexed.</li>\n         </ul>\n       </li>\n       <li>index: Function that handles the indexing</li>\n     </ul>\n   </li>\n </ul>\n </p>\n", "summary" : "<p>  Retrieves a document from the specified database.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "connection", "type" : "anyURI", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> A connection identifier</div>" }, { "name" : "database", "type" : "string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> A database name</div>" }, { "name" : "document-id", "type" : "string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> A document identifier</div>" } ], "returns" : { "type" : "object()", "description" : "The specified document" }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cloudant:AUTHORIZATION Authorization error</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cloudant:DATABASE-NAME Invalid database name</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cloudant:DOCUMENT The specified document or revision cannot be found</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cloudant:HTTP An HTTP error occurred when issuing the request</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cloudant:RESPONSE An error occurred parsing the server response</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cloudant:CONNECTION The specified connection does not exist</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cloudant:INTERNAL Cloudant internal error</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 4, "name" : "document", "qname" : "cloudant:document", "signature" : "($connection as anyURI, $database as string, $document-id as string, $options as object()) as object()", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Retrieves a document from the specified database</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">This function is declared as deterministic and should be used whenever result\n caching is acceptable.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Unless you request a specific revision, the latest\n revision of the document will always be returned.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">This function allows to specify an additional options object.\n The following options are supported:\n <ul>\n   <li>conflict: returns the conflict tree for the document (boolean)</li>\n   <li>rev: specifies the revision to return (string)</li>\n   <li>revs: return a list of the revisions for the document (boolean)</li>\n   <li>revs_info: return a list of detailed revision information for the document\n   (boolean, allowed value: true)</li>\n   <li>database-owner: specifies the database owner (string, default: connection user).</li>\n </ul>\n </p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">If the document includes attachments, then the\n returned structure will contain a summary of the\n attachments associated with the document, but not the\n attachment data itself.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">For example,\n <pre>\n cloudant:document($connection, \"db\", \"DocID\", {\"database-owner\": \"username\"})\n </pre>.\n </p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">The returned JSON object will contain the document\n and has the following format:\n <pre>\n {\n   \"_id\": \"DocID\",\n   \"_rev\": \"2-f29c836d0bedc4b4b95cfaa6d99e95df\",\n   \"name\": \"Anna\",\n   \"age\": 89,\n   \"gender\": \"female\",\n   \"_attachments\": {\n     \"my attachment\": {\n       \"content_type\": \"application/json; charset=UTF-8\",\n       \"revpos\": 2,\n       \"digest\": \"md5-37IZysiyWLRWx31J/1WQHw==\",\n       \"length\": 12,\n       \"stub\": true\n       }\n     }\n   }\n </pre>\n </p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">\n The meaning of the fields in the returned object is the following:\n <ul>\n   <li>_id: document ID</li>\n   <li>_rev: document revision</li>\n   <li>attachments: document attachments (optional), each field denotes the name\n       of a different attachment:\n   <ul>\n     <li>content_type: attachment MIME Content type string</li>\n     <li>length: attachment length in bytes</li>\n     <li>revpos: revision where this attachment exists</li>\n     <li>digest: MD5 checksum of the attachment</li>\n     <li>stub: indicates whether the attachment is a stub</li>\n  </ul>\n  </li>\n </ul>\n </p>\n <h4 xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Retrieving a design document from the specified database</h4>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">To retrieve a design document the document id must start with\n <code>_design/</code>.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">For example,\n <pre>cloudant:document($connection, \"db\", \"_design/DesDocID\", {\"database-owner\": \"username\"})</pre>.\n </p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">A JSON object with the following format is returned:\n <pre>\n {\n  \"_id\": \"_design/DesDocID\",\n  \"_rev\": \"2-f29c836d0bedc4b4b95cfaa6d99e95df\",\n   \"views\": {\n     \"view1\": {\n       \"map\":\"function(doc){emit(doc.field, 1)}\",\n       \"reduce\": \"function(key, value, rereduce){return sum(values)}\"\n     }\n   },\n   \"indexes\": {\n     \"mysearch\" : {\n       \"analyzer\": {\"name\": \"portuguese\", \"stopwords\":[\"foo\", \"bar, \"baz\"]},\n       \"index\": \"function(doc){ ... }\"\n     },\n   }\n }\n </pre>\n </p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">\n The meaning of the fields in the returned object is the following:\n <ul>\n   <li>_id: document ID</li>\n   <li>_rev: document revision</li>\n   <li>views: object defining the views. Each field defines a different view.\n       The field name is the name of the view and has the following content:\n     <ul>\n       <li>map: view map function</li>\n       <li>reduce: view reduce function (optional)</li>\n     </ul>\n   </li>\n   <li>indexes: object defining the view indexes. Each field defines a different index.\n       The field name is the name of the index and has the following content:\n     <ul>\n       <li>analyzer: Name of the analyzer to be used or an object with the\n       following fields:\n         <ul>\n           <li>name: Name of the analyzer</li>\n           <li>stopwords: An array of stop words. Stop words are words that\n           should not be indexed.</li>\n         </ul>\n       </li>\n       <li>index: Function that handles the indexing</li>\n     </ul>\n   </li>\n </ul>\n </p>\n <h4 xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Getting a List of Revisions</h4>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">You can obtain a list of the revisions for a given document\n by the revs option.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">For example,\n <pre>cloudant:document($connection, \"db\", \"DocID\", {\"revs\": true})</pre>.\n </p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">The returned JSON object includes the original document and\n a <code>_revisions</code> structure that includes the revision\n information:\n <pre>\n {\n   \"servings\":4,\n   \"subtitle\":\"Delicious with a green salad\",\n   \"_id\":\"FishStew\",\n   \"title\":\"Irish Fish Stew\",\n   \"_revisions\":\n   {\n     \"ids\": [\n       \"a1a9b39ee3cc39181b796a69cb48521c\",\n       \"7c4740b4dcf26683e941d6641c00c39d\",\n       \"9c65296036141e575d32ba9c034dd3ee\"\n      ],\n     \"start\":3\n  },\n  \"_rev\":\"3-a1a9b39ee3cc39181b796a69cb48521c\"\n }\n </pre>\n </p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">The meaning of the additional fields is the following:\n <ul>\n   <li>_revisions: document revisions</li>\n   <li>_ids: array of valid revision IDs, in reverse order (latest first)</li>\n   <li>start: prefix number for the latest revision</li>\n </ul>\n </p>\n <h4 xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Obtaining an Extended Revision History</h4>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">You can get additional information about the revisions for a given document\n with the <code>revs_info</code> option.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">For example,\n <pre>cloudant:document($connection, \"db\", \"DocID\", {\"revs_info\": true})</pre>.\n </p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">This returns extended revision information, including the availability and status\n of each revision:\n <pre>\n {\n   \"servings\":4,\n   \"subtitle\":\"Delicious with a green salad\",\n   \"_id\":\"FishStew\",\n   \"_revs_info\":[\n     {\n       \"status\":\"available\",\n       \"rev\":\"3-a1a9b39ee3cc39181b796a69cb48521c\"\n     },\n     {\n       \"status\":\"available\",\n       \"rev\":\"2-7c4740b4dcf26683e941d6641c00c39d\"\n     },\n     {\n       \"status\":\"available\",\n       \"rev\":\"1-9c65296036141e575d32ba9c034dd3ee\"\n     }\n   ],\n   \"title\":\"Irish Fish Stew\",\n   \"_rev\":\"3-a1a9b39ee3cc39181b796a69cb48521c\"\n }\n </pre>\n </p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">The meaning of the additional fields is the following:\n <ul>\n   <li>_revs_info: document extended revision info</li>\n   <li>rev: revision ID</li>\n   <li>status: revision status</li>\n </ul>\n </p>\n", "summary" : "<p>  Retrieves a document from the specified database \n  This function is declared as deterministic and should be used whenever result\n caching is acceptable.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "connection", "type" : "anyURI", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> A connection identifier</div>" }, { "name" : "database", "type" : "string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> A database name</div>" }, { "name" : "document-id", "type" : "string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> A document identifier</div>" }, { "name" : "options", "type" : "object()", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> An object specifying additional request options</div>" } ], "returns" : { "type" : "object()", "description" : "The specified document" }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cloudant:AUTHORIZATION Authorization error</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cloudant:DATABASE-NAME Invalid database name</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cloudant:DOCUMENT The specified document or revision cannot be found</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cloudant:HTTP An HTTP error occurred when issuing the request</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cloudant:RESPONSE An error occurred parsing the server response</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cloudant:CONNECTION The specified connection does not exist</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cloudant:INTERNAL Cloudant internal error</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cloudant:OPTIONS Malformed options object</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 1, "name" : "list-databases-nondeterministic", "qname" : "cloudant:list-databases-nondeterministic", "signature" : "($connection as anyURI) as array()", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Returns an array containing the names of all the user's databases.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">This function has the same semantics as\n <a href=\"#list-databases-1\">list-databases#1</a>,\n but is declared as being non deterministic and thus should only be used when\n result caching is not desired.</p>\n", "summary" : "<p>  Returns an array containing the names of all the user's databases.</p>", "annotation_str" : " %an:nondeterministic", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "nondeterministic", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "connection", "type" : "anyURI", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> A connection identifier</div>" } ], "returns" : { "type" : "array()", "description" : "An array of all the database names" }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cloudant:AUTHORIZATION Authorization error</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cloudant:HTTP An HTTP error occurred when issuing the request</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cloudant:RESPONSE An error occurred parsing the server response</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cloudant:CONNECTION The specified connection does not exist</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cloudant:INTERNAL Cloudant internal error</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 2, "name" : "list-databases-nondeterministic", "qname" : "cloudant:list-databases-nondeterministic", "signature" : "($connection as anyURI, $options as object()) as array()", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Returns an array containing the names of all the user's databases.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">This function has the same semantics as\n <a href=\"#list-databases-2\">list-databases#2</a>,\n but is declared as being non deterministic and thus should only be used when\n result caching is not desired.</p>\n", "summary" : "<p>  Returns an array containing the names of all the user's databases.</p>", "annotation_str" : " %an:nondeterministic", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "nondeterministic", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "connection", "type" : "anyURI", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> A connection identifier</div>" }, { "name" : "options", "type" : "object()", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> An object specifying additional request options</div>" } ], "returns" : { "type" : "array()", "description" : "An array of all the databases names" }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cloudant:AUTHORIZATION Authorization error</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cloudant:HTTP An HTTP error occurred when issuing the request</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cloudant:RESPONSE An error occurred parsing the server response</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cloudant:CONNECTION The specified connection does not exist</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cloudant:INTERNAL Cloudant internal error</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cloudant:OPTIONS Malformed options object</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 1, "name" : "list-databases", "qname" : "cloudant:list-databases", "signature" : "($connection as anyURI) as array()", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Returns an array containing the names of all the user's databases.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">This function is declared as deterministic and should be used whenever result\n caching is acceptable.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">For example,\n <pre>cloudant:list-databases($connection)</pre>.\n </p>\n", "summary" : "<p>  Returns an array containing the names of all the user's databases.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "connection", "type" : "anyURI", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> A connection identifier</div>" } ], "returns" : { "type" : "array()", "description" : "An array of all the database names" }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cloudant:AUTHORIZATION Authorization error</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cloudant:HTTP An HTTP error occurred when issuing the request</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cloudant:RESPONSE An error occurred parsing the server response</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cloudant:CONNECTION The specified connection does not exist</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cloudant:INTERNAL Cloudant internal error</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 2, "name" : "list-databases", "qname" : "cloudant:list-databases", "signature" : "($connection as anyURI, $options as object()) as array()", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Returns an array containing the names of all the user's databases.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">This function is declared as deterministic and should be used whenever result\n caching is acceptable.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">This function allows to specify an additional options object.\n The following options are supported:\n <ul>\n   <li>database-owner: specifies the database owner (string, default: connection user).</li>\n </ul>\n </p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">For example,\n <pre>cloudant:list-databases($connection, {\"database-owner\" : \"username\"})</pre>.\n </p>\n", "summary" : "<p>  Returns an array containing the names of all the user's databases.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "connection", "type" : "anyURI", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> A connection identifier</div>" }, { "name" : "options", "type" : "object()", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> An object specifying additional request options</div>" } ], "returns" : { "type" : "array()", "description" : "An array of all the databases names" }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cloudant:AUTHORIZATION Authorization error</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cloudant:HTTP An HTTP error occurred when issuing the request</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cloudant:RESPONSE An error occurred parsing the server response</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cloudant:CONNECTION The specified connection does not exist</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cloudant:INTERNAL Cloudant internal error</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cloudant:OPTIONS Malformed options object</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 5, "name" : "lucene-query-nondeterministic", "qname" : "cloudant:lucene-query-nondeterministic", "signature" : "($connection as anyURI, $database as string, $design-document as string, $index as string, $query as string) as object()", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Executes a Lucene query against a view and returns the query result.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">This function has the same semantics as\n <a href=\"#lucene-query-5\">lucene-query#5</a>,\n but is declared as being non deterministic and thus should only be used when\n result caching is not desired.</p>\n", "summary" : "<p>  Executes a Lucene query against a view and returns the query result.</p>", "annotation_str" : " %an:nondeterministic", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "nondeterministic", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "connection", "type" : "anyURI", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> A connection identifier</div>" }, { "name" : "database", "type" : "string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> A database name</div>" }, { "name" : "design-document", "type" : "string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> A design document name</div>" }, { "name" : "index", "type" : "string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> An index name</div>" }, { "name" : "query", "type" : "string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> A Lucene query</div>" } ], "returns" : { "type" : "object()", "description" : "The Lucene query results" }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cloudant:AUTHORIZATION Authorization error</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cloudant:HTTP An HTTP error occurred when issuing the request</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cloudant:RESPONSE An error occurred parsing the server response</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cloudant:CONNECTION The specified connection does not exist</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cloudant:INTERNAL Cloudant internal error</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 6, "name" : "lucene-query-nondeterministic", "qname" : "cloudant:lucene-query-nondeterministic", "signature" : "($connection as anyURI, $database as string, $design-document as string, $index as string, $query as string, $options as object()) as object()", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Executes a Lucene query against a view and returns the query result.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">This function has the same semantics as <a href=\"#lucene-query-6\">lucene-query#6</a>,\n but is declared as being non deterministic and thus should only be used when\n result caching is not desired.</p>\n", "summary" : "<p>  Executes a Lucene query against a view and returns the query result.</p>", "annotation_str" : " %an:nondeterministic", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "nondeterministic", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "connection", "type" : "anyURI", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> A connection identifier</div>" }, { "name" : "database", "type" : "string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> A database name</div>" }, { "name" : "design-document", "type" : "string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> A design document name</div>" }, { "name" : "index", "type" : "string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> An index name</div>" }, { "name" : "query", "type" : "string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> A Lucene query</div>" }, { "name" : "options", "type" : "object()", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> An object specifying additional request options</div>" } ], "returns" : { "type" : "object()", "description" : "The Lucene query results" }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cloudant:AUTHORIZATION Authorization error</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cloudant:HTTP An HTTP error occurred when issuing the request</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cloudant:RESPONSE An error occurred parsing the server response</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cloudant:CONNECTION The specified connection does not exist</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cloudant:INTERNAL Cloudant internal error</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cloudant:OPTIONS Malformed options object</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 5, "name" : "lucene-query", "qname" : "cloudant:lucene-query", "signature" : "($connection as anyURI, $database as string, $design-document as string, $index as string, $query as string) as object()", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Executes a Lucene query against a view and returns the query result.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">This function is declared as deterministic and should be used whenever result\n caching is acceptable.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">This method searches for documents whose index fields match the Lucene query.\n Which fields of a document are indexed and how is determined by the index functions\n in the design document.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">The information is returned as a JSON structure containing meta information\n about the return structure, and the list of documents each with its ID, revision\n and key. The key is generated from the document ID.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">For example,\n <pre>cloudant:lucene-query($connection, \"db\", \"designdoc\", \"view\", \"a*\")</pre>.\n </p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">An object with the following format is returned:\n <pre>\n {\n   \"total_rows\": 3,\n   \"bookmark\": \"g1AAAACWeJzLYWBgYMpgTmFQSElKzi9KdUhJMtbLTS3KLElMT9VLzskvTUnMK9HLSy3JAalMcgCSSfX____PAvPdQHwQSGTIIt6UPBaQlgNA6j_CJPsPcJOyANNEKzY\",\n   \"rows\":\n   [\n     {\n       \"id\": \"dd828eb4-c3f1-470f-aeff-c375ef70e4ad\",\n       \"order\": [0.0, 1],\n       \"fields\":\n       {\n         \"default\": \"aa\",\n         \"foo\": 0.0\n       }\n     },\n     {\n       \"id\": \"ea522cf1-eb8e-4477-aa92-d1fa459bb216\",\n       \"order\": [1.0, 0],\n       \"fields\":\n       {\n         \"default\": \"ab\",\n         \"foo\": 1.0\n       }\n     },\n     {\n       \"id\": \"c838baed-d573-43ea-9c34-621cf0f13301\",\n       \"order\": [2.0, 0],\n       \"fields\":\n       {\n         \"default\": \"ac\",\n         \"foo\": 2.0\n       }\n     }\n   ]\n }\n </pre>\n </p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">The fields have the following meaning:\n <ul>\n  <li>total_rows: number of results returned.</li>\n  <li>bookmark: string to be submitted in the next query to page through results.\n      If this response contained no results, the bookmark will be the same as the\n      one used to obtain this response.</li>\n  <li>rows: array of document objects, each document contains the following fields:\n    <ul>\n      <li>order: specifies the order with regard to the indexed fields.</li>\n      <li>fields: Object containing other search indexes.</li>\n    </ul>\n  </li>\n </ul>\n </p>\n", "summary" : "<p>  Executes a Lucene query against a view and returns the query result.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "connection", "type" : "anyURI", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> A connection identifier</div>" }, { "name" : "database", "type" : "string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> A database name</div>" }, { "name" : "design-document", "type" : "string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> A design document name</div>" }, { "name" : "index", "type" : "string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> An index name</div>" }, { "name" : "query", "type" : "string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> A Lucene query</div>" } ], "returns" : { "type" : "object()", "description" : "The Lucene query results" }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cloudant:AUTHORIZATION Authorization error</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cloudant:HTTP An HTTP error occurred when issuing the request</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cloudant:RESPONSE An error occurred parsing the server response</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cloudant:CONNECTION The specified connection does not exist</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cloudant:INTERNAL Cloudant internal error</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 6, "name" : "lucene-query", "qname" : "cloudant:lucene-query", "signature" : "($connection as anyURI, $database as string, $design-document as string, $index as string, $query as string, $options as object()) as object()", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Executes a Lucene query against a view and returns the query result.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">This function is declared as deterministic and should be used whenever result\n caching is acceptable.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">This method searches for documents whose index fields match the Lucene query.\n Which fields of a document are indexed and how is determined by the index functions\n in the design document.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">This function allows to specify an additional options object.\n The following options are supported:\n <ul>\n   <li>bookmark: A bookmark that was received from a previous search. This\n   allows you to page through the results. If there are no more results after\n   the bookmark, you will get a response with an empty rows array and the\n   same bookmark. That way you can determine that you have reached the end\n   of the result list (string).</li>\n   <li>stale: allow the results from a stale view to be used (string, allowed\n   value: \"ok\").</li>\n   <li>limit: limit the number of the returned documents to the specified\n   number (numeric).</li>\n   <li>include_docs: include the full content of the documents in the return\n   (boolean, default: false).</li>\n   <li>sort: specifies the sort order of the results. A JSON string of the\n   form <code>\"fieldname&lt;type&gt;\"</code> or <code>-fieldname&lt;type&gt;</code>\n   for descending order, where fieldname is the name of a string or number field\n   and type is either number or string. The type part is optional and defaults\n   to number. Some examples are <code>\"foo\"</code>, <code>\"-foo\"</code>,\n   <code>\"bar&lt;string&gt;\"</code>, <code>\"-foo&lt;number&gt;\"</code>. String\n   fields used for sorting must not be analyzed fields. The field(s) used for\n   sorting must be indexed by the same indexer used for the search query.\n   Alternatively, a JSON array of such strings is allowed.</li>\n </ul>\n </p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">The information is returned as a JSON structure containing meta information\n about the return structure, and the list of documents each with its ID, revision\n and key. The key is generated from the document ID.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">For example,\n <pre>cloudant:lucene-query($connection, \"db\", \"designdoc\", \"view\", \"a*\",\n {\"database-owner\": \"username\"})</pre>.\n </p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">An object with the following format is returned:\n <pre>\n {\n   \"total_rows\": 3,\n   \"bookmark\": \"g1AAAACWeJzLYWBgYMpgTmFQSElKzi9KdUhJMtbLTS3KLElMT9VLzskvTUnMK9HLSy3JAalMcgCSSfX____PAvPdQHwQSGTIIt6UPBaQlgNA6j_CJPsPcJOyANNEKzY\",\n   \"rows\":\n   [\n     {\n       \"id\": \"dd828eb4-c3f1-470f-aeff-c375ef70e4ad\",\n       \"order\": [0.0, 1],\n       \"fields\":\n       {\n         \"default\": \"aa\",\n         \"foo\": 0.0\n       }\n     },\n     {\n       \"id\": \"ea522cf1-eb8e-4477-aa92-d1fa459bb216\",\n       \"order\": [1.0, 0],\n       \"fields\":\n       {\n         \"default\": \"ab\",\n         \"foo\": 1.0\n       }\n     },\n     {\n       \"id\": \"c838baed-d573-43ea-9c34-621cf0f13301\",\n       \"order\": [2.0, 0],\n       \"fields\":\n       {\n         \"default\": \"ac\",\n         \"foo\": 2.0\n       }\n     }\n   ]\n }\n </pre>\n </p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">The fields have the following meaning:\n <ul>\n  <li>total_rows: number of results returned.</li>\n  <li>bookmark: string to be submitted in the next query to page through results.\n      If this response contained no results, the bookmark will be the same as the\n      one used to obtain this response.</li>\n  <li>rows: array of document objects, each document contains the following fields:\n    <ul>\n      <li>order: specifies the order with regard to the indexed fields.</li>\n      <li>fields: Object containing other search indexes.</li>\n    </ul>\n  </li>\n </ul>\n </p>\n", "summary" : "<p>  Executes a Lucene query against a view and returns the query result.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "connection", "type" : "anyURI", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> A connection identifier</div>" }, { "name" : "database", "type" : "string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> A database name</div>" }, { "name" : "design-document", "type" : "string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> A design document name</div>" }, { "name" : "index", "type" : "string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> An index name</div>" }, { "name" : "query", "type" : "string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> A Lucene query</div>" }, { "name" : "options", "type" : "object()", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> An object specifying additional request options</div>" } ], "returns" : { "type" : "object()", "description" : "The Lucene query results" }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cloudant:AUTHORIZATION Authorization error</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cloudant:HTTP An HTTP error occurred when issuing the request</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cloudant:RESPONSE An error occurred parsing the server response</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cloudant:CONNECTION The specified connection does not exist</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cloudant:INTERNAL Cloudant internal error</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cloudant:OPTIONS Malformed options object</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 3, "name" : "multiple-documents-nondeterministic", "qname" : "cloudant:multiple-documents-nondeterministic", "signature" : "($connection as anyURI, $database as string, $keys as string*) as object()", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">List the specified documents in a given database.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">This function has the same semantics as\n <a href=\"#multiple-documents-3\">multiple-documents#3</a>,\n but is declared as being non deterministic and thus should only be used when\n result caching is not desired.</p>\n", "summary" : "<p>  List the specified documents in a given database.</p>", "annotation_str" : " %an:nondeterministic", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "nondeterministic", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "connection", "type" : "anyURI", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> A connection identifier</div>" }, { "name" : "database", "type" : "string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> A database name</div>" }, { "name" : "keys", "type" : "string", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The keys of the documents which must be retrieved</div>" } ], "returns" : { "type" : "object()", "description" : "An object listing the specified documents in the specified database" }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cloudant:AUTHORIZATION Authorization error</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cloudant:HTTP An HTTP error occurred when issuing the request</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cloudant:RESPONSE An error occurred parsing the server response</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cloudant:CONNECTION The specified connection does not exist</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cloudant:INTERNAL Cloudant internal error</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 4, "name" : "multiple-documents-nondeterministic", "qname" : "cloudant:multiple-documents-nondeterministic", "signature" : "($connection as anyURI, $database as string, $keys as string*, $options as object()) as object()", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Lists the specified documents in a given database.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">This function has the same semantics as\n <a href=\"#multiple-documents-4\">multiple-documents#4</a>,\n but is declared as being non deterministic and thus should only be used when\n result caching is not desired.</p>\n", "summary" : "<p>  Lists the specified documents in a given database.</p>", "annotation_str" : " %an:nondeterministic", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "nondeterministic", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "connection", "type" : "anyURI", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> A connection identifier</div>" }, { "name" : "database", "type" : "string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> A database name</div>" }, { "name" : "keys", "type" : "string", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The keys of the documents which must be retrieved</div>" }, { "name" : "options", "type" : "object()", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> An object specifying additional request options</div>" } ], "returns" : { "type" : "object()", "description" : "An object listing the specified documents in the specified database" }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cloudant:AUTHORIZATION Authorization error</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cloudant:HTTP An HTTP error occurred when issuing the request</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cloudant:RESPONSE An error occurred parsing the server response</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cloudant:CONNECTION The specified connection does not exist</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cloudant:INTERNAL Cloudant internal error</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cloudant:OPTIONS Malformed options object</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 3, "name" : "multiple-documents", "qname" : "cloudant:multiple-documents", "signature" : "($connection as anyURI, $database as string, $keys as string*) as object()", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">List the specified documents in a given database.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">This function is declared as deterministic and should be used whenever result\n caching is acceptable.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">The information is returned as a JSON structure containing meta information\n about the return structure, and the list of documents each with its ID, revision\n and key. The key is generated from the document ID.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">For example,\n <pre>cloudant:multiple-documents($connection, \"db\",\n   (\"5a049246-179f-42ad-87ac-8f080426c17c\",\n    \"d1f61e66-7708-4da6-aa05-7cbc33b44b7e\",\n    \"96f898f0-f6ff-4a9b-aac4-503992f31b01\"))</pre>.\n </p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">An object with the following format is returned:\n <pre>\n {\n   \"total_rows\":3,\n   \"offset\":0,\n   \"rows\":[\n   {\n     \"id\":\"5a049246-179f-42ad-87ac-8f080426c17c\",\n     \"key\":\"5a049246-179f-42ad-87ac-8f080426c17c\",\n     \"value\":\n     {\n       \"rev\":\"2-9d5401898196997853b5ac4163857a29\"\n     }\n   },\n   {\n     \"id\":\"96f898f0-f6ff-4a9b-aac4-503992f31b01\",\n     \"key\":\"96f898f0-f6ff-4a9b-aac4-503992f31b01\",\n     \"value\":\n     {\n       \"rev\":\"2-ff7b85665c4c297838963c80ecf481a3\"\n      }\n   },\n   {\n     \"id\":\"d1f61e66-7708-4da6-aa05-7cbc33b44b7e\",\n     \"key\":\"d1f61e66-7708-4da6-aa05-7cbc33b44b7e\",\n     \"value\":\n     {\n       \"rev\":\"2-cbdef49ef3ddc127eff86350844a6108\"\n     }\n   }]\n }\n </pre>\n </p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">The fields have the following meaning:\n <ul>\n  <li>offset: offset where the document list started.</li>\n  <li>rows: array of document objects, each containing id, key and revision number.</li>\n  <li>total_rows: number of documents in the database.</li>\n  <li>update_seq: current update sequence database.</li>\n </ul>\n </p>\n", "summary" : "<p>  List the specified documents in a given database.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "connection", "type" : "anyURI", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> A connection identifier</div>" }, { "name" : "database", "type" : "string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> A database name</div>" }, { "name" : "keys", "type" : "string", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The keys of the documents which must be retrieved</div>" } ], "returns" : { "type" : "object()", "description" : "An object listing the specified documents in the specified database" }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cloudant:AUTHORIZATION Authorization error</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cloudant:HTTP An HTTP error occurred when issuing the request</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cloudant:RESPONSE An error occurred parsing the server response</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cloudant:CONNECTION The specified connection does not exist</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cloudant:INTERNAL Cloudant internal error</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 4, "name" : "multiple-documents", "qname" : "cloudant:multiple-documents", "signature" : "($connection as anyURI, $database as string, $keys as string*, $options as object()) as object()", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Lists the specified documents in a given database.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">This function is declared as deterministic and should be used whenever result\n caching is acceptable.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">The information is returned as a JSON structure containing meta information\n about the return structure, and the list of documents each with its ID, revision\n and key. The key is generated from the document ID.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">This function allows to specify an additional options object.\n The following options are supported:\n <ul>\n   <li>descending: return the documents in descending by key order (boolean,\n   default: false).</li>\n   <li>endkey: stop returning records when the specified key is reached (string).</li>\n   <li>endkey_docid: stop returning records when the specified document ID is\n   reached (string).</li>\n   <li>group: group the results using the reduce function to a group or single\n   row (boolean, default: false).</li>\n   <li>group_level: specify the group level to be used (numeric).</li>\n   <li>include_docs: include the full content of the documents in the return\n   (boolean, default: false).</li>\n   <li>inclusive_end: specifies whether the specified end key should be included\n   in the result (boolean, default: true).</li>\n   <li>key: return only documents that match the specified key (string).</li>\n   <li>limit: limit the number of the returned documents to the specified number\n   (numeric).</li>\n   <li>reduce: use the reduction function (boolean, default: true).</li>\n   <li>skip: skip this number of records before starting to return the results\n   (numeric, default: 0).</li>\n   <li>stale: allow the results from a stale view to be used (string, allowed\n   value: \"ok\").</li>\n   <li>startkey: start returning records when the specified key is reached\n   (string).</li>\n   <li>startkey_docid: start returning records when the specified document ID\n   is reached (string).</li>\n   <li>database-owner: specifies the database owner (string, default: connection\n   user).</li>\n </ul>\n </p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">For example,\n <pre>cloudant:multiple-documents($connection, \"db\",\n   (\"5a049246-179f-42ad-87ac-8f080426c17c\",\n    \"d1f61e66-7708-4da6-aa05-7cbc33b44b7e\",\n    \"96f898f0-f6ff-4a9b-aac4-503992f31b01\"),\n   {\"database-owner\": \"username\"})</pre>.\n </p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">An object with the following format is returned:\n <pre>\n {\n   \"total_rows\":3,\n   \"offset\":0,\n   \"rows\":[\n   {\n     \"id\":\"5a049246-179f-42ad-87ac-8f080426c17c\",\n     \"key\":\"5a049246-179f-42ad-87ac-8f080426c17c\",\n     \"value\":\n     {\n       \"rev\":\"2-9d5401898196997853b5ac4163857a29\"\n     }\n   },\n   {\n     \"id\":\"96f898f0-f6ff-4a9b-aac4-503992f31b01\",\n     \"key\":\"96f898f0-f6ff-4a9b-aac4-503992f31b01\",\n     \"value\":\n     {\n       \"rev\":\"2-ff7b85665c4c297838963c80ecf481a3\"\n      }\n   },\n   {\n     \"id\":\"d1f61e66-7708-4da6-aa05-7cbc33b44b7e\",\n     \"key\":\"d1f61e66-7708-4da6-aa05-7cbc33b44b7e\",\n     \"value\":\n     {\n       \"rev\":\"2-cbdef49ef3ddc127eff86350844a6108\"\n     }\n   }]\n }\n </pre>\n </p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">The fields have the following meaning:\n <ul>\n  <li>offset: offset where the document list started.</li>\n  <li>rows: array of document objects, each containing id, key and revision number.</li>\n  <li>total_rows: number of documents in the database.</li>\n  <li>update_seq: current update sequence database.</li>\n </ul>\n </p>\n", "summary" : "<p>  Lists the specified documents in a given database.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "connection", "type" : "anyURI", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> A connection identifier</div>" }, { "name" : "database", "type" : "string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> A database name</div>" }, { "name" : "keys", "type" : "string", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The keys of the documents which must be retrieved</div>" }, { "name" : "options", "type" : "object()", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> An object specifying additional request options</div>" } ], "returns" : { "type" : "object()", "description" : "An object listing the specified documents in the specified database" }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cloudant:AUTHORIZATION Authorization error</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cloudant:HTTP An HTTP error occurred when issuing the request</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cloudant:RESPONSE An error occurred parsing the server response</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cloudant:CONNECTION The specified connection does not exist</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cloudant:INTERNAL Cloudant internal error</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cloudant:OPTIONS Malformed options object</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 5, "name" : "multiple-view-documents-nondeterministic", "qname" : "cloudant:multiple-view-documents-nondeterministic", "signature" : "($connection as anyURI, $database as string, $design-document as string, $view as string, $keys as string*) as object()", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">List the specified documents in a given view.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">This function has the same semantics as\n <a href=\"#multiple-view-documents-5\">multiple-view-documents#5</a>,\n but is declared as being non deterministic and thus should only be used when\n result caching is not desired.</p>\n", "summary" : "<p>  List the specified documents in a given view.</p>", "annotation_str" : " %an:nondeterministic", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "nondeterministic", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "connection", "type" : "anyURI", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> A connection identifier</div>" }, { "name" : "database", "type" : "string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> A database name</div>" }, { "name" : "design-document", "type" : "string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> A design document name</div>" }, { "name" : "view", "type" : "string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> A view name</div>" }, { "name" : "keys", "type" : "string", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The keys of the documents which must be retrieved</div>" } ], "returns" : { "type" : "object()", "description" : "An object listing all the specified documents in the specified view" }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cloudant:AUTHORIZATION Authorization error</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cloudant:HTTP An HTTP error occurred when issuing the request</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cloudant:RESPONSE An error occurred parsing the server response</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cloudant:CONNECTION The specified connection does not exist</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cloudant:INTERNAL Cloudant internal error</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 6, "name" : "multiple-view-documents-nondeterministic", "qname" : "cloudant:multiple-view-documents-nondeterministic", "signature" : "($connection as anyURI, $database as string, $design-document as string, $view as string, $keys as string*, $options as object()) as object()", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Lists the specified documents in a given view.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">This function has the same semantics as\n <a href=\"#multiple-view-documents-6\">multiple-view-documents#6</a>,\n but is declared as being non deterministic and thus should only be used when\n result caching is not desired.</p>\n", "summary" : "<p>  Lists the specified documents in a given view.</p>", "annotation_str" : " %an:nondeterministic", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "nondeterministic", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "connection", "type" : "anyURI", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> A connection identifier</div>" }, { "name" : "database", "type" : "string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> A database name</div>" }, { "name" : "design-document", "type" : "string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> A design document name</div>" }, { "name" : "view", "type" : "string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> A view name</div>" }, { "name" : "keys", "type" : "string", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The keys of the documents which must be retrieved</div>" }, { "name" : "options", "type" : "object()", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> An object specifying additional request options</div>" } ], "returns" : { "type" : "object()", "description" : "An object listing all the specified documents in the specified view" }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cloudant:AUTHORIZATION Authorization error</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cloudant:HTTP An HTTP error occurred when issuing the request</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cloudant:RESPONSE An error occurred parsing the server response</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cloudant:CONNECTION The specified connection does not exist</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cloudant:INTERNAL Cloudant internal error</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cloudant:OPTIONS Malformed options object</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 5, "name" : "multiple-view-documents", "qname" : "cloudant:multiple-view-documents", "signature" : "($connection as anyURI, $database as string, $design-document as string, $view as string, $keys as string*) as object()", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">List the specified documents in a given view.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">This function is declared as deterministic and should be used whenever result\n caching is acceptable.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">The information is returned as a JSON structure containing meta information\n about the return structure, and the list of documents each with its ID, revision\n and key. The key is generated from the document ID.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">For example,\n <pre>cloudant:multiple-view-documents($connection, \"db\", \"recipes\", \"by_ingredient\",\n (\"claret\", \"clear apple juice\"))\n </pre>.\n </p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">An object with the following format is returned:\n <pre>\n {\n   \"total_rows\" : 26484,\n   \"rows\" : [\n     {\n       \"value\" : [\"Scotch collops\"]],\n       \"id\" : \"Scotchcollops\",\n       \"key\" : \"claret\"\n     },\n     {\n       \"value\" : [\"Stand pie\"],\n       \"id\" : \"Standpie\",\n       \"key\" : \"clear apple juice\"\n     }\n   ],\n   \"offset\" : 6324\n }\n </pre>\n </p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">The fields have the following meaning:\n <ul>\n  <li>offset: offset where the document list started.</li>\n  <li>rows: array of document objects, each containing id, key and revision\n  number.</li>\n  <li>total_rows: number of documents in the database.</li>\n  <li>update_seq: current update sequence database.</li>\n </ul>\n </p>\n", "summary" : "<p>  List the specified documents in a given view.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "connection", "type" : "anyURI", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> A connection identifier</div>" }, { "name" : "database", "type" : "string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> A database name</div>" }, { "name" : "design-document", "type" : "string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> A design document name</div>" }, { "name" : "view", "type" : "string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> A view name</div>" }, { "name" : "keys", "type" : "string", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The keys of the documents which must be retrieved</div>" } ], "returns" : { "type" : "object()", "description" : "An object listing all the specified documents in the specified view" }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cloudant:AUTHORIZATION Authorization error</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cloudant:HTTP An HTTP error occurred when issuing the request</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cloudant:RESPONSE An error occurred parsing the server response</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cloudant:CONNECTION The specified connection does not exist</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cloudant:INTERNAL Cloudant internal error</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 6, "name" : "multiple-view-documents", "qname" : "cloudant:multiple-view-documents", "signature" : "($connection as anyURI, $database as string, $design-document as string, $view as string, $keys as string*, $options as object()) as object()", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Lists the specified documents in a given view.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">This function is declared as deterministic and should be used whenever result\n caching is acceptable.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">The information is returned as a JSON structure containing meta information\n about the return structure, and the list of documents each with its ID, revision\n and key. The key is generated from the document ID.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">This function allows to specify an additional options object.\n The following options are supported:\n <ul>\n   <li>descending: return the documents in descending by key order (boolean,\n   default: false).</li>\n   <li>endkey: stop returning records when the specified key is reached (string).</li>\n   <li>endkey_docid: stop returning records when the specified document ID is\n   reached (string).</li>\n   <li>group: group the results using the reduce function to a group or single\n   row (boolean, default: false).</li>\n   <li>group_level: specify the group level to be used (numeric).</li>\n   <li>include_docs: include the full content of the documents in the return\n   (boolean, default: false).</li>\n   <li>inclusive_end: specifies whether the specified end key should be included\n   in the result (boolean, default: true).</li>\n   <li>key: return only documents that match the specified key (string).</li>\n   <li>limit: limit the number of the returned documents to the specified number\n   (numeric).</li>\n   <li>reduce: use the reduction function (boolean, default: true).</li>\n   <li>skip: skip this number of records before starting to return the results\n   (numeric, default: 0).</li>\n   <li>stale: allow the results from a stale view to be used (string, allowed\n   value: \"ok\").</li>\n   <li>startkey: start returning records when the specified key is reached\n   (string).</li>\n   <li>startkey_docid: start returning records when the specified document ID\n   is reached (string).</li>\n   <li>database-owner: specifies the database owner (string, default: connection\n   user).</li>\n </ul>\n </p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">For example,\n <pre>cloudant:multiple-view-documents($connection, \"db\", \"recipes\",\n   \"by_ingredient\", (\"claret\", \"clear apple juice\"),\n   {\"database-owner\": \"username\"})\n </pre>.\n </p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">An object with the following format is returned:\n <pre>\n {\n   \"total_rows\" : 26484,\n   \"rows\" : [\n     {\n       \"value\" : [\"Scotch collops\"]],\n       \"id\" : \"Scotchcollops\",\n       \"key\" : \"claret\"\n     },\n     {\n       \"value\" : [\"Stand pie\"],\n       \"id\" : \"Standpie\",\n       \"key\" : \"clear apple juice\"\n     }\n   ],\n   \"offset\" : 6324\n }\n </pre>\n </p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">The fields have the following meaning:\n <ul>\n  <li>offset: offset where the document list started.</li>\n  <li>rows: array of document objects, each containing id, key and revision number.</li>\n  <li>total_rows: number of documents in the database.</li>\n  <li>update_seq: current update sequence database.</li>\n </ul>\n </p>\n", "summary" : "<p>  Lists the specified documents in a given view.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "connection", "type" : "anyURI", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> A connection identifier</div>" }, { "name" : "database", "type" : "string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> A database name</div>" }, { "name" : "design-document", "type" : "string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> A design document name</div>" }, { "name" : "view", "type" : "string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> A view name</div>" }, { "name" : "keys", "type" : "string", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The keys of the documents which must be retrieved</div>" }, { "name" : "options", "type" : "object()", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> An object specifying additional request options</div>" } ], "returns" : { "type" : "object()", "description" : "An object listing all the specified documents in the specified view" }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cloudant:AUTHORIZATION Authorization error</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cloudant:HTTP An HTTP error occurred when issuing the request</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cloudant:RESPONSE An error occurred parsing the server response</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cloudant:CONNECTION The specified connection does not exist</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cloudant:INTERNAL Cloudant internal error</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">cloudant:OPTIONS Malformed options object</xqdoc:error>" ] } ], "variables" : [  ] }, "http://api.28.io/exportimport" : { "ns" : "http://api.28.io/exportimport", "description" : "", "sees" : [  ], "authors" : [  ], "version" : null, "encoding" : null, "namespaces" : [ { "uri" : "", "prefix" : "an" }, { "uri" : "http://api.28.io/collections", "prefix" : "cm" }, { "uri" : "http://api.28.io/exportimport", "prefix" : "exportimport" }, { "uri" : "http://zorba.io/modules/fetch", "prefix" : "fetch" }, { "uri" : "http://api.28.io/indices", "prefix" : "in" }, { "uri" : "http://www.28msec.com/modules/maps", "prefix" : "map" }, { "uri" : "http://www.28msec.com/modules/http/request", "prefix" : "req" }, { "uri" : "http://www.28msec.com/modules/http/response", "prefix" : "resp" }, { "uri" : "http://www.28msec.com/modules/store", "prefix" : "store" }, { "uri" : "http://api.28.io/validation", "prefix" : "validate" } ], "functions" : [ { "isDocumented" : false, "arity" : 1, "name" : "create-map", "qname" : "exportimport:create-map", "signature" : "($map as object())", "description" : "", "summary" : "", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "map", "type" : "object()", "occurrence" : null, "description" : "" } ], "returns" : { "type" : null, "description" : "" }, "errors" : [  ] }, { "isDocumented" : false, "arity" : 0, "name" : "export", "qname" : "exportimport:export", "signature" : "()", "description" : "", "summary" : "", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [  ], "returns" : { "type" : null, "description" : "" }, "errors" : [  ] }, { "isDocumented" : false, "arity" : 0, "name" : "import", "qname" : "exportimport:import", "signature" : "()", "description" : "", "summary" : "", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [  ], "returns" : { "type" : null, "description" : "" }, "errors" : [  ] }, { "isDocumented" : false, "arity" : 2, "name" : "import", "qname" : "exportimport:import", "signature" : "($import as object(), $overwrite as xs:boolean)", "description" : "", "summary" : "", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "import", "type" : "object()", "occurrence" : null, "description" : "" }, { "name" : "overwrite", "type" : "xs:boolean", "occurrence" : null, "description" : "" } ], "returns" : { "type" : null, "description" : "" }, "errors" : [  ] }, { "isDocumented" : false, "arity" : 1, "name" : "is-system-collection", "qname" : "exportimport:is-system-collection", "signature" : "($name as xs:string) as xs:boolean", "description" : "", "summary" : "", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "name", "type" : "xs:string", "occurrence" : null, "description" : "" } ], "returns" : { "type" : "xs:boolean", "description" : "" }, "errors" : [  ] }, { "isDocumented" : false, "arity" : 1, "name" : "validate-map", "qname" : "exportimport:validate-map", "signature" : "($map as object()) as empty-sequence()", "description" : "", "summary" : "", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "map", "type" : "object()", "occurrence" : null, "description" : "" } ], "returns" : { "type" : "empty-sequence()", "description" : "" }, "errors" : [  ] } ], "variables" : [  ] }, "http://jsoniq.org/errors" : { "ns" : "http://jsoniq.org/errors", "description" : " This module contains one variable declaration for each diagnostic of the\n http://jsoniq.org/errors namespace.\n The variables serves as documentation for the errors but can also\n be used in the code. For example, one useful scenario is to compare\n an error caught in the catch clause of a try-catch expression with one of\n the variables.\n", "sees" : [  ], "authors" : [ "<xqdoc:author xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Carlos Lopez</xqdoc:author>" ], "version" : null, "encoding" : "utf-8", "namespaces" : [ { "uri" : "http://jsoniq.org/errors", "prefix" : "jerr" } ], "functions" : [  ], "variables" : [ { "name" : "jerr:JNSE0013", "type" : "xs:QName", "description" : "It is a dynamic error to serialize an atomic value not\n supported by JSON or a node with the JSON output method and with\n the jsoniq-serialization-extensions serialization parameter\n set to false.\n" }, { "name" : "jerr:JNDY0021", "type" : "xs:QName", "description" : "parser error raised by jn:parse-json\n" }, { "name" : "jerr:JNTY0021", "type" : "xs:QName", "description" : "array or object selector on heterogeneous sequence\n" }, { "name" : "jerr:JNTY0020", "type" : "xs:QName", "description" : "parser error for invalid option type\n" }, { "name" : "jerr:JNTY0024", "type" : "xs:QName", "description" : "objects or arrays don't have a string value\n" }, { "name" : "jerr:JNTY0023", "type" : "xs:QName", "description" : "It is a type error if the prefix is not a string or if the\n serialization parameters are not an element.\n" }, { "name" : "jerr:JNUP0019", "type" : "xs:QName", "description" : "It is a dynamic error if the content expression, in an object insert expression, does not evaluate to a sequence of objects.\n" }, { "name" : "jerr:JNTY0018", "type" : "xs:QName", "description" : "It is a dynamic error if there is not exactly one supplied parameter for an object or array selector.\n" }, { "name" : "jerr:JNUP0017", "type" : "xs:QName", "description" : " It is a dynamic error if the value in a replace expression is not exactly a single item.\n" }, { "name" : "jerr:JNUP0016", "type" : "xs:QName", "description" : "It is a dynamic error if it is attempted to create a replace, delete or rename update primitive with a selector that cannot be resolved against the target array or object.\n" }, { "name" : "jerr:JNSE0022", "type" : "xs:QName", "description" : "It is a dynamic error to serialize a sequence that does\n not exist of exactly one document node with XML, HTML, XHTML, Text.\n" }, { "name" : "jerr:JNSE0014", "type" : "xs:QName", "description" : "It is a dynamic error to serialize a function or a node with the\n JSON output method.\n" }, { "name" : "jerr:NS", "type" : "item()*", "description" : "" }, { "name" : "jerr:JNSE0012", "type" : "xs:QName", "description" : "It is a dynamic error to serialize a sequence of less\n or more than one item with the JSON output method if the\n jsoniq-serialization-multiple-items is set to no.\n" }, { "name" : "jerr:JNTY0011", "type" : "xs:QName", "description" : "It is a type error if the content sequence in a node constructor or in an XQUF insert or replace update expression contains an object or an array.\n" }, { "name" : "jerr:JNUP0010", "type" : "xs:QName", "description" : "It is a dynamic error if a pending update list contains two renaming update primitives on the same object and with the same selector.\n" }, { "name" : "jerr:JNUP0009", "type" : "xs:QName", "description" : "It is a dynamic error if a pending update list contains two replacing update primitives on the same object or array, and with the same selector.\n" }, { "name" : "jerr:JNUP0008", "type" : "xs:QName", "description" : "It is a dynamic error if the target of a deleting or replacing expression is not an array or an object.\n It is a dynamic error if the target of a renaming expression is not an object.\n It is a dynamic error if the target of an appending expression is not an array.\n It is a dynamic error if the target of a position-inserting expression is not an array.\n It is a dynamic error if the target of a non-position-inserting expression is not an object.\n" }, { "name" : "jerr:JNUP0007", "type" : "xs:QName", "description" : "It is a type error if, in an updating expression, an array selector cannot be cast to xs:integer or if an object selector cannot be cast to xs:string.\n" }, { "name" : "jerr:JNUP0006", "type" : "xs:QName", "description" : "It is a dynamic error if upd:applyUpdates causes an object to contain two pairs with the same name.\n" }, { "name" : "jerr:JNUP0005", "type" : "xs:QName", "description" : "It is a dynamic error if a pending update list contains two inserting update primitives on the same object and pair name.\n" }, { "name" : "jerr:JNTY0004", "type" : "xs:QName", "description" : "It is a type error to call fn:data on a sequence containing an array or an object.\n" }, { "name" : "jerr:JNDY0003", "type" : "xs:QName", "description" : "It is a dynamic error if two pairs in an object constructor or in a simple object union have the same name.\n" }, { "name" : "jerr:JNTY0002", "type" : "xs:QName", "description" : "It is a type error if the right-hand-side expression of a pair constructor does not return exactly one item.\n" } ] }, "http://zorba.io/modules/sctx" : { "ns" : "http://zorba.io/modules/sctx", "description" : " This module provides functions that gets components of the static context.\n", "sees" : [ "<xqdoc:see xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">http://www.w3.org/TR/xquery/#id-xq-static-context-components</xqdoc:see>" ], "authors" : [ "<xqdoc:author xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Nicolae Brinza</xqdoc:author>" ], "version" : null, "encoding" : "utf-8", "namespaces" : [ { "uri" : "http://zorba.io/modules/sctx", "prefix" : "sctx" }, { "uri" : "http://zorba.io/options/versioning", "prefix" : "ver" } ], "functions" : [ { "isDocumented" : true, "arity" : 0, "name" : "base-uri", "qname" : "sctx:base-uri", "signature" : "() as xs:string? external", "description" : " Gets the base URI.\n", "summary" : "<p> Gets the base URI.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [  ], "returns" : { "type" : "xs:string?", "description" : "The base URI." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 0, "name" : "boundary-space-policy", "qname" : "sctx:boundary-space-policy", "signature" : "() as xs:string external", "description" : " Gets the boundary whitespace policy used by direct element constructors.\n", "summary" : "<p> Gets the boundary whitespace policy used by direct element constructors.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [  ], "returns" : { "type" : "xs:string", "description" : "Either <code xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">preserve</code> or <code xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">strip</code>." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 0, "name" : "construction-mode", "qname" : "sctx:construction-mode", "signature" : "() as xs:string external", "description" : " Gets the static context construction mode of element and document nodes.\n", "summary" : "<p> Gets the static context construction mode of element and document nodes.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [  ], "returns" : { "type" : "xs:string", "description" : "Either <code xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">preserve</code> or <code xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">strip</code>." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 0, "name" : "copy-namespaces-mode", "qname" : "sctx:copy-namespaces-mode", "signature" : "() as xs:string+ external", "description" : " Gets the static context components that control the the namespace bindings\n that are assigned when an existing element node is copied\n by an element constructor.\n", "summary" : "<p> Gets the static context components that control the the namespace bindings\n that are assigned when an existing element node is copied\n by an element constructor.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [  ], "returns" : { "type" : "xs:string+", "description" : "A sequence of two strings: the first is either <code xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">preserve</code> or <code xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">no-preserve</code> and the second is either <code xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">inherit</code> or <code xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">no-inherit</code>." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 0, "name" : "default-collation", "qname" : "sctx:default-collation", "signature" : "() as xs:string external", "description" : " Gets one of statically known collations used by functions and operators\n for comparing and ordering values of type <code xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">xs:string</code>\n or <code xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">xs:anyURI</code> when no explicit collation is specified.\n", "summary" : "<p> Gets one of statically known collations used by functions and operators\n for comparing and ordering values of type  xs:string \n or  xs:anyURI  when no explicit collation is specified.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [  ], "returns" : { "type" : "xs:string", "description" : "The collations that is used by default." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 0, "name" : "default-collection-type", "qname" : "sctx:default-collection-type", "signature" : "() as xs:string external", "description" : " Gets the statically known default collection type.\n This is the type of the sequence of nodes that would result from calling the\n <code xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">fn:collection</code> function with no arguments.\n", "summary" : "<p> Gets the statically known default collection type.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [  ], "returns" : { "type" : "xs:string", "description" : "The type of the default collection." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 0, "name" : "default-function-namespace", "qname" : "sctx:default-function-namespace", "signature" : "() as xs:string external", "description" : " Gets the URI of the default function namespace.\n", "summary" : "<p> Gets the URI of the default function namespace.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [  ], "returns" : { "type" : "xs:string", "description" : "The URI of the of the default function namespace." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 0, "name" : "default-order", "qname" : "sctx:default-order", "signature" : "() as xs:string external", "description" : " Gets the component that controls the processing of empty sequences and NaN\n values as ordering keys in an <code xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">order by</code> clause\n in a FLWOR expression.\n", "summary" : "<p> Gets the component that controls the processing of empty sequences and NaN\n values as ordering keys in an  order by  clause\n in a FLWOR expression.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [  ], "returns" : { "type" : "xs:string", "description" : "Either <code xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">greatest</code> or <code xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">least</code>." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 2, "name" : "function-annotations", "qname" : "sctx:function-annotations", "signature" : "($name as xs:QName, $arity as xs:integer) as xs:QName* external", "description" : " Gets the list of annotations declared for the given function.\n", "summary" : "<p> Gets the list of annotations declared for the given function.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "name", "type" : "xs:QName", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The name of the function.</div>" }, { "name" : "arity", "type" : "xs:integer", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The number of arguments the function takes.</div>" } ], "returns" : { "type" : "xs:QName*", "description" : "the list of annotations" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "function-arguments-count", "qname" : "sctx:function-arguments-count", "signature" : "($function as xs:QName) as xs:int* external", "description" : " Gets the number of arguments the given XQuery function takes.\n", "summary" : "<p> Gets the number of arguments the given XQuery function takes.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "function", "type" : "xs:QName", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> A QName identifying a function.</div>" } ], "returns" : { "type" : "xs:int*", "description" : "Either a sequence of zero or more integers (one for each overloaded version of the given function) or an empty sequence if the function is not defined." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 0, "name" : "function-names", "qname" : "sctx:function-names", "signature" : "() as xs:QName* external", "description" : " Gets a sequence containing the QNames of all defined functions\n that are available to be called from within an expression.\n", "summary" : "<p> Gets a sequence containing the QNames of all defined functions\n that are available to be called from within an expression.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [  ], "returns" : { "type" : "xs:QName*", "description" : "A sequence for QNames identifying all functions." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 0, "name" : "functions", "qname" : "sctx:functions", "signature" : "() as object()* external", "description" : " Gets a sequence of JSON objects containing the name, arity, and annotations\n of all defined functions that are available to be called form within an\n expression.\n", "summary" : "<p> Gets a sequence of JSON objects containing the name, arity, and annotations\n of all defined functions that are available to be called form within an\n expression.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [  ], "returns" : { "type" : "object()*", "description" : "A sequence of objects." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 0, "name" : "in-scope-attribute-declarations", "qname" : "sctx:in-scope-attribute-declarations", "signature" : "() as xs:QName* external", "description" : " Gets a sequence of QNames identifying declared attributes\n in the imported schemas.\n", "summary" : "<p> Gets a sequence of QNames identifying declared attributes\n in the imported schemas.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [  ], "returns" : { "type" : "xs:QName*", "description" : "A sequence of QNames, one for each attribute." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 0, "name" : "in-scope-attribute-groups", "qname" : "sctx:in-scope-attribute-groups", "signature" : "() as xs:QName* external", "description" : " Gets a sequence of QNames identifying declared in-scope\n schema attribute groups.\n", "summary" : "<p> Gets a sequence of QNames identifying declared in-scope\n schema attribute groups.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [  ], "returns" : { "type" : "xs:QName*", "description" : "A sequence of QNames, one for each attribute group." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 0, "name" : "in-scope-element-declarations", "qname" : "sctx:in-scope-element-declarations", "signature" : "() as xs:QName* external", "description" : " Gets a sequence of QNames identifying declared elements\n in the imported schemas.\n", "summary" : "<p> Gets a sequence of QNames identifying declared elements\n in the imported schemas.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [  ], "returns" : { "type" : "xs:QName*", "description" : "A sequence of QNames, one for each declared element." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 0, "name" : "in-scope-element-groups", "qname" : "sctx:in-scope-element-groups", "signature" : "() as xs:QName* external", "description" : " Gets a sequence of QNames identifying declared in-scope\n schema element groups.\n", "summary" : "<p> Gets a sequence of QNames identifying declared in-scope\n schema element groups.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [  ], "returns" : { "type" : "xs:QName*", "description" : "A sequence of QNames, one for each element group." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 0, "name" : "in-scope-schema-types", "qname" : "sctx:in-scope-schema-types", "signature" : "() as xs:QName* external", "description" : " Gets a sequence of QNames identifying in-scope schema types\n including all the predefined schema types and all definitions\n found in imported schemas.\n", "summary" : "<p> Gets a sequence of QNames identifying in-scope schema types\n including all the predefined schema types and all definitions\n found in imported schemas.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [  ], "returns" : { "type" : "xs:QName*", "description" : "A sequence of QNames, one for each defined type." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 0, "name" : "in-scope-variables", "qname" : "sctx:in-scope-variables", "signature" : "() as xs:QName* external", "description" : " Gets a sequence of QNames identifying declared variables\n from the static context.\n", "summary" : "<p> Gets a sequence of QNames identifying declared variables\n from the static context.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [  ], "returns" : { "type" : "xs:QName*", "description" : "A sequence of QNames, one for each variable." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "option", "qname" : "sctx:option", "signature" : "($name as xs:QName) as xs:string? external", "description" : " Gets the value of an option that is declared in the prolog of the module.\n", "summary" : "<p> Gets the value of an option that is declared in the prolog of the module.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "name", "type" : "xs:QName", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the name of the option value to retrieve</div>" } ], "returns" : { "type" : "xs:string?", "description" : "the value of the option if contained in the static context or the empty sequence otherwise." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 0, "name" : "ordering-mode", "qname" : "sctx:ordering-mode", "signature" : "() as xs:string external", "description" : " Gets the ordering mode that affects the ordering of the result sequences\n returned by certain expressions.\n", "summary" : "<p> Gets the ordering mode that affects the ordering of the result sequences\n returned by certain expressions.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [  ], "returns" : { "type" : "xs:string", "description" : "Either <code xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">ordered</code> or <code xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">unordered</code>." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 0, "name" : "statically-known-collations", "qname" : "sctx:statically-known-collations", "signature" : "() as xs:anyURI* external", "description" : " Gets a sequence that contains the statically known collations.\n", "summary" : "<p> Gets a sequence that contains the statically known collations.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [  ], "returns" : { "type" : "xs:anyURI*", "description" : "The sequence of collations." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "statically-known-document-type", "qname" : "sctx:statically-known-document-type", "signature" : "($document as xs:string) as xs:QName external", "description" : " For the given document, returns the static type of its root node.\n", "summary" : "<p> For the given document, returns the static type of its root node.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "document", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> A URI of a document.</div>" } ], "returns" : { "type" : "xs:QName", "description" : "The static type of the given document." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 0, "name" : "statically-known-documents", "qname" : "sctx:statically-known-documents", "signature" : "() as xs:anyURI* external", "description" : " Gets a sequence that contains the URIs of all statically known documents.\n", "summary" : "<p> Gets a sequence that contains the URIs of all statically known documents.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [  ], "returns" : { "type" : "xs:anyURI*", "description" : "The sequence of document URIs." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "statically-known-namespace-binding", "qname" : "sctx:statically-known-namespace-binding", "signature" : "($prefix as xs:string) as xs:string? external", "description" : " Gets the bound URI for the given prefix.\n", "summary" : "<p> Gets the bound URI for the given prefix.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "prefix", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The prefix of the known namespace for which the bound URI is being requested.</div>" } ], "returns" : { "type" : "xs:string?", "description" : "The URI bound to the prefix is returned, or an empty sequence if the prefix is not known." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 0, "name" : "statically-known-namespaces", "qname" : "sctx:statically-known-namespaces", "signature" : "() as xs:string* external", "description" : " Gets a list of known statically known namespaces as prefixes\n (fn, xml, xs, xsi, etc).\n", "summary" : "<p> Gets a list of known statically known namespaces as prefixes\n (fn, xml, xs, xsi, etc).</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [  ], "returns" : { "type" : "xs:string*", "description" : "A sequence of xs:strings, one for each known namespace prefix." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 0, "name" : "xpath10-compatibility-mode", "qname" : "sctx:xpath10-compatibility-mode", "signature" : "() as xs:boolean external", "description" : " Gets true or false depending on whether rules for compatibility\n with XPath 1.0 are in effect.\n XQuery set the value of this component to false.\n", "summary" : "<p> Gets true or false depending on whether rules for compatibility\n with XPath 1.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [  ], "returns" : { "type" : "xs:boolean", "description" : "The function should return <code xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">false</code>." }, "errors" : [  ] } ], "variables" : [  ] }, "http://xbrl.io/modules/bizql/profiles/sec/import" : { "ns" : "http://xbrl.io/modules/bizql/profiles/sec/import", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">This module provides functionality for querying XBRL Infosets of financial reports submitted to the SEC.\n </p>\n", "sees" : [  ], "authors" : [ "<xqdoc:author xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Charles Hoffman</xqdoc:author>", "<xqdoc:author xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Matthias Brantner</xqdoc:author>", "<xqdoc:author xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Dennis Knochenwefel</xqdoc:author>", "<xqdoc:author xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Ghislain Fourny</xqdoc:author>" ], "version" : null, "encoding" : "utf-8", "namespaces" : [ { "uri" : "http://xbrl.io/modules/bizql/archives", "prefix" : "archives" }, { "uri" : "http://www.28msec.com/modules/credentials", "prefix" : "credentials" }, { "uri" : "http://xbrl.io/modules/bizql/entities", "prefix" : "entities" }, { "uri" : "http://xbrl.io/modules/bizql/profiles/sec/import", "prefix" : "imp" }, { "uri" : "http://www.28msec.com/modules/mongodb", "prefix" : "m" }, { "uri" : "http://zorba.io/modules/string", "prefix" : "string" }, { "uri" : "http://zorba.io/options/versioning", "prefix" : "ver" } ], "functions" : [ { "isDocumented" : true, "arity" : 1, "name" : "disclosure-for-network-label", "qname" : "imp:disclosure-for-network-label", "signature" : "($component as object()) as object()", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">\n Compute the disclore for a component's network label.\n </p>\n", "summary" : "<p>  \n Compute the disclore for a component's network label.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "component", "type" : "object()", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the component</div>" } ], "returns" : { "type" : "object()", "description" : "returns the given component ammended with the disclosure" }, "errors" : [  ] } ], "variables" : [ { "name" : "imp:disclosure-matching", "type" : "object()", "description" : " Declarative description of the disclosure matching heuristics.\n" } ] }, "http://xbrl.io/modules/bizql/profiles/sec/core" : { "ns" : "http://xbrl.io/modules/bizql/profiles/sec/core", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">This module provides functionality for querying XBRL Models of financial reports submitted to the SEC.\n </p>\n <h2 xmlns:xqdoc=\"http://www.xqdoc.org/1.0\" id=\"standard_options\">Standard <code>$options</code> Parameter</h2>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Most functions in the BizQL package allow an additional <code>$options</code>\n    parameter. The options parameter is a JSON object which is defined in the documentation\n    of the <a href=\"../../facts#standard_options\">facts module</a>.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">In addition to the option fields defined in the\n    <a href=\"../../facts#standard_options\">facts module</a> the following\n    fields can be used in the options of a function in the SEC profile:</p>\n <ul xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">\n <li><b>HideAmendedFacts</b>: if set to true (default behavior) all facts that have been\n     amended are not included in the result.</li>\n <li><b>IncludeImpliedTable</b>: set to true in order to output the implied table if\n     there is no Table (default is false).</li>\n </ul>\n", "sees" : [  ], "authors" : [ "<xqdoc:author xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Charles Hoffman</xqdoc:author>", "<xqdoc:author xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Matthias Brantner</xqdoc:author>", "<xqdoc:author xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Dennis Knochenwefel</xqdoc:author>", "<xqdoc:author xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Ghislain Fourny</xqdoc:author>" ], "version" : null, "encoding" : "utf-8", "namespaces" : [ { "uri" : "http://xbrl.io/modules/bizql/archives", "prefix" : "archives" }, { "uri" : "http://xbrl.io/modules/bizql/concept-maps", "prefix" : "concept-maps" }, { "uri" : "http://www.28msec.com/modules/credentials", "prefix" : "credentials" }, { "uri" : "http://xbrl.io/modules/bizql/entities", "prefix" : "entities" }, { "uri" : "http://xbrl.io/modules/bizql/facts", "prefix" : "facts" }, { "uri" : "http://xbrl.io/modules/bizql/hypercubes", "prefix" : "hypercubes" }, { "uri" : "http://www.28msec.com/modules/mongodb", "prefix" : "m" }, { "uri" : "http://xbrl.io/modules/bizql/report-schemas", "prefix" : "report-schemas" }, { "uri" : "http://xbrl.io/modules/bizql/profiles/sec/core", "prefix" : "sec" }, { "uri" : "http://zorba.io/modules/string", "prefix" : "string" }, { "uri" : "http://zorba.io/options/versioning", "prefix" : "ver" } ], "functions" : [ { "isDocumented" : true, "arity" : 1, "name" : "edgar-facts-for-archives", "qname" : "sec:edgar-facts-for-archives", "signature" : "($archive_or_ids as item()*) as object()*", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Return all facts for concepts that are defined by one of the common\n edgar taxonomies. In particular, this function returns facts for concepts\n having one of the following prefixes:</p>\n <ul xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">\n   <li>xbrli</li>\n   <li>link</li>\n   <li>xl</li>\n   <li>xlink</li>\n   <li>xbrldt</li>\n   <li>xbrldi</li>\n   <li>nonnum</li>\n   <li>num</li>\n   <li>ref</li>\n   <li>us-gaap</li>\n   <li>dei</li>\n   <li>us-types</li>\n   <li>invest</li>\n   <li>country</li>\n   <li>currency</li>\n   <li>exch</li>\n   <li>naics</li>\n   <li>sic</li>\n   <li>stpr</li>\n   <li>rr</li>\n   <li>rr-ent</li>\n   <li>rr-cal</li>\n   <li>rr-def</li>\n   <li>rr-pre</li>\n </ul>\n", "summary" : "<p>  Return all facts for concepts that are defined by one of the common\n edgar taxonomies.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "archive_or_ids", "type" : "item()", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> a sequence of archive or archive IDs to filter.</div>" } ], "returns" : { "type" : "object()*", "description" : "all facts having one of the above prefixes." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "end-date", "qname" : "sec:end-date", "signature" : "($filing-fact-or-id as item()?) as xs:date?", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Retrieves the document end date of a filing or a fact.</p>\n", "summary" : "<p>  Retrieves the document end date of a filing or a fact.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "filing-fact-or-id", "type" : "item()", "occurrence" : "?", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> a filing, a fact, or its id.</div>" } ], "returns" : { "type" : "xs:date?", "description" : "the document end date (e.g., xs:date(\"2011-04-30\"))." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "extension-facts-for-archives", "qname" : "sec:extension-facts-for-archives", "signature" : "($archive_or_ids as item()*) as object()*", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Return all facts for concepts that are NOT defined by one of the common\n edgar taxonomies. In particular, this function returns facts for concepts\n having none of the prefixes listed under function\n sec:edgar-facts-for-archives.</p>\n", "summary" : "<p>  Return all facts for concepts that are NOT defined by one of the common\n edgar taxonomies.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "archive_or_ids", "type" : "item()", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> a sequence of archive or archive IDs to filter.</div>" } ], "returns" : { "type" : "object()*", "description" : "all facts having none of the common edgar prefixes." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 2, "name" : "fact-table-for-schema", "qname" : "sec:fact-table-for-schema", "signature" : "($schema as item(), $archives as item()*) as array()", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Retrieves all facts from the supplied archives, that are relevant to the\n supplied schema, and populates them with the default dimension values\n when missing.</p>\n", "summary" : "<p>  Retrieves all facts from the supplied archives, that are relevant to the\n supplied schema, and populates them with the default dimension values\n when missing.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "schema", "type" : "item()", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> a report schema or its RID.</div>" }, { "name" : "archives", "type" : "item()", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> a sequence of archives or their AIDs or $sec:ALL_OF_THEM for no filtering.</div>" } ], "returns" : { "type" : "array()", "description" : "a sequence of facts with populated dimension values." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 2, "name" : "facts-for-archives-and-concepts", "qname" : "sec:facts-for-archives-and-concepts", "signature" : "($archive_or_ids as item()*, $concepts as item()*) as object()*", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Retrieves the facts associated with the supplied concepts\n from the supplied archive, that match the fiscal focus of the archive.</p>\n", "summary" : "<p>  Retrieves the facts associated with the supplied concepts\n from the supplied archive, that match the fiscal focus of the archive.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "archive_or_ids", "type" : "item()", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> a sequence of archives or their ids or $sec:ALL_OF_THEM for no filtering.</div>" }, { "name" : "concepts", "type" : "item()", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the concepts or $sec:ALL_OF_THEM for no filtering.</div>" } ], "returns" : { "type" : "object()*", "description" : "the latest facts associated with the supplied concept." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 3, "name" : "facts-for-archives-and-concepts", "qname" : "sec:facts-for-archives-and-concepts", "signature" : "($archive_or_ids as item()*, $concepts as item()*, $options as object()?) as object()*", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Retrieves the facts associated with the supplied concepts\n from the supplied archive, that match the fiscal focus of the archive.</p>\n", "summary" : "<p>  Retrieves the facts associated with the supplied concepts\n from the supplied archive, that match the fiscal focus of the archive.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "archive_or_ids", "type" : "item()", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> a sequence of archives or their ids or $sec:ALL_OF_THEM for no filtering.</div>" }, { "name" : "concepts", "type" : "item()", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the concepts or $sec:ALL_OF_THEM for no filtering.</div>" }, { "name" : "options", "type" : "object()", "occurrence" : "?", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> <a href=\"#standard_options\">standard SEC BizQL options</a>.</div>" } ], "returns" : { "type" : "object()*", "description" : "the latest facts associated with the supplied concept." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 2, "name" : "facts-for-schema", "qname" : "sec:facts-for-schema", "signature" : "($schema as item(), $archives as item()*) as item()*", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Retrieves all facts from the supplied archives, that are relevant to the\n supplied schema and to the fiscal focus, and populates\n them with the default dimension values when missing.</p>\n", "summary" : "<p>  Retrieves all facts from the supplied archives, that are relevant to the\n supplied schema and to the fiscal focus, and populates\n them with the default dimension values when missing.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "schema", "type" : "item()", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> a schema.</div>" }, { "name" : "archives", "type" : "item()", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> a sequence of archives or their AIDs or $sec:ALL_OF_THEM for no filtering.</div>" } ], "returns" : { "type" : "item()*", "description" : "a sequence of facts with populated dimension values." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "hide-amended-facts", "qname" : "sec:hide-amended-facts", "signature" : "($facts as object()*) as object()*", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Filters a list of facts and returns only the latest amending facts.\n    Accordingly, all amended facts are filtered out.</p>\n", "summary" : "<p>  Filters a list of facts and returns only the latest amending facts.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "facts", "type" : "object()", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> a list of facts.</div>" } ], "returns" : { "type" : "object()*", "description" : "a sequence of facts without amended facts." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 2, "name" : "populate-schema-with-facts", "qname" : "sec:populate-schema-with-facts", "signature" : "($schema as item(), $archives as item()*) as object()*", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Populates a concept-tree network with all facts from the supplied archives,\n that are relevant to the\n supplied schema. Default dimension values are added to the facts\n when missing.</p>\n", "summary" : "<p>  Populates a concept-tree network with all facts from the supplied archives,\n that are relevant to the\n supplied schema.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "schema", "type" : "item()", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> a report schema or its RID.</div>" }, { "name" : "archives", "type" : "item()", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> a sequence of archives or their AIDs.</div>" } ], "returns" : { "type" : "object()*", "description" : "a sequence of facts with populated dimension values." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 3, "name" : "populate-schema-with-facts", "qname" : "sec:populate-schema-with-facts", "signature" : "($schema as item(), $archives as item()*, $options as object()?) as object()*", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Populates a concept-tree network with all facts from the supplied archives,\n that are relevant to the\n supplied schema. Default dimension values are added to the facts\n when missing.</p>\n", "summary" : "<p>  Populates a concept-tree network with all facts from the supplied archives,\n that are relevant to the\n supplied schema.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "schema", "type" : "item()", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> a report schema or its RID.</div>" }, { "name" : "archives", "type" : "item()", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> a sequence of archives or their AIDs or $sec:ALL_OF_THEM for no filtering.</div>" }, { "name" : "options", "type" : "object()", "occurrence" : "?", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> <a href=\"#standard_options\">standard SEC BizQL options</a>.</div>" } ], "returns" : { "type" : "object()*", "description" : "a sequence of facts with populated dimension values." }, "errors" : [  ] } ], "variables" : [ { "name" : "sec:CIK", "type" : "xs:string", "description" : " Scheme for the SEC entity identifiers (i.e. http://www.sec.gov/CIK)\n" }, { "name" : "sec:ALL_OF_THEM", "type" : "boolean", "description" : " Joker value for all archives or all concepts.\n" }, { "name" : "sec:EDGAR_PREFIXES", "type" : "item()*", "description" : " All prefixes pre-declared in SEC's EDGAR.\n" } ] }, "http://zorba.io/modules/excel/logical" : { "ns" : "http://zorba.io/modules/excel/logical", "description" : " This is a library module offering the same set of functions\n defined by Microsoft Excel, under Logical Functions.\n", "sees" : [ "<xqdoc:see xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"><a href=\"http://office.microsoft.com/en-us/excel/CH062528271033.aspx\" target=\"_blank\">Excel Documentation: Logical Functions</a></xqdoc:see>" ], "authors" : [ "<xqdoc:author xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Sorin Nasoi</xqdoc:author>" ], "version" : null, "encoding" : "utf-8", "namespaces" : [ { "uri" : "http://zorba.io/modules/excel/errors", "prefix" : "excel-err" }, { "uri" : "http://zorba.io/modules/excel/logical", "prefix" : "excel-logical" }, { "uri" : "http://zorba.io/options/versioning", "prefix" : "ver" } ], "functions" : [ { "isDocumented" : true, "arity" : 1, "name" : "and", "qname" : "excel-logical:and", "signature" : "($values as xs:anyAtomicType*) as xs:boolean", "description" : " Returns TRUE if all its arguments are TRUE; FALSE if one or more arguments are FALSE.\n", "summary" : "<p> Returns TRUE if all its arguments are TRUE; FALSE if one or more arguments are FALSE.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "values", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequence of arguments.</div>" } ], "returns" : { "type" : "xs:boolean", "description" : "TRUE if all its arguments are TRUE; FALSE if one or more arguments are FALSE." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">excel-err:Value provided sequence is empty.</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 2, "name" : "and", "qname" : "excel-logical:and", "signature" : "($arg1 as xs:anyAtomicType, $arg2 as xs:anyAtomicType) as xs:boolean", "description" : " Returns TRUE if all its arguments are TRUE; FALSE if one or more arguments are FALSE.\n", "summary" : "<p> Returns TRUE if all its arguments are TRUE; FALSE if one or more arguments are FALSE.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "arg1", "type" : "xs:anyAtomicType", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the first argument.</div>" }, { "name" : "arg2", "type" : "xs:anyAtomicType", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the second argument.</div>" } ], "returns" : { "type" : "xs:boolean", "description" : "TRUE if all its arguments are TRUE; FALSE if one or more arguments are FALSE." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 3, "name" : "if", "qname" : "excel-logical:if", "signature" : "($logical_test as xs:boolean, $value_if_true as item()*, $value_if_false as item()*) as item()*", "description" : " Returns one value if a condition you specify evaluates to TRUE and another value if it evaluates to FALSE.\n", "summary" : "<p> Returns one value if a condition you specify evaluates to TRUE and another value if it evaluates to FALSE.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "logical_test", "type" : "xs:boolean", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> is any value or expression that can be evaluated to TRUE or FALSE.</div>" }, { "name" : "value_if_true", "type" : "item()", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the value that is returned if logical_test is TRUE.</div>" }, { "name" : "value_if_false", "type" : "item()", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the value that is returned if logical_test is FALSE.</div>" } ], "returns" : { "type" : "item()*", "description" : "One value if a condition you specify evaluates to TRUE and another value if it evaluates to FALSE." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "or", "qname" : "excel-logical:or", "signature" : "($values as xs:anyAtomicType*) as xs:boolean", "description" : " Returns TRUE if any argument is TRUE; FALSE if all arguments are FALSE.\n", "summary" : "<p> Returns TRUE if any argument is TRUE; FALSE if all arguments are FALSE.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "values", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequence of arguments.</div>" } ], "returns" : { "type" : "xs:boolean", "description" : "TRUE if any argument is TRUE; FALSE if all arguments are FALSE." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">excel-err:Value provided sequence is empty.</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 2, "name" : "or", "qname" : "excel-logical:or", "signature" : "($arg1 as xs:anyAtomicType, $arg2 as xs:anyAtomicType) as xs:boolean", "description" : " Returns TRUE if any argument is TRUE; FALSE if all arguments are FALSE.\n", "summary" : "<p> Returns TRUE if any argument is TRUE; FALSE if all arguments are FALSE.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "arg1", "type" : "xs:anyAtomicType", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the first argument.</div>" }, { "name" : "arg2", "type" : "xs:anyAtomicType", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the second argument.</div>" } ], "returns" : { "type" : "xs:boolean", "description" : "TRUE if any argument is TRUE; FALSE if all arguments are FALSE." }, "errors" : [  ] } ], "variables" : [  ] }, "http://www.28msec.com/modules/sequence" : { "ns" : "http://www.28msec.com/modules/sequence", "description" : " The sequence module allows you to generate consecutive\n application-unique xs:integer ids. This is required e.g. in accounting\n applications as some legislations require consecutive invoice numbers.\n", "sees" : [  ], "authors" : [ "<xqdoc:author xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">28msec</xqdoc:author>" ], "version" : null, "encoding" : "utf-8", "namespaces" : [ { "uri" : "http://zorba.io/annotations", "prefix" : "an" }, { "uri" : "http://zorba.io/modules/store/static/collections/dml", "prefix" : "cdml" }, { "uri" : "http://zorba.io/modules/store/static/indexes/dml", "prefix" : "idml" }, { "uri" : "http://www.28msec.com/modules/lock", "prefix" : "lock" }, { "uri" : "http://www.28msec.com/modules/sequence", "prefix" : "seq" }, { "uri" : "http://www.28msec.com/modules/sleep", "prefix" : "sleep" }, { "uri" : "http://zorba.io/options/versioning", "prefix" : "ver" } ], "functions" : [ { "isDocumented" : true, "arity" : 1, "name" : "id", "qname" : "seq:id", "signature" : "($uri as xs:string) as xs:integer", "description" : " Returns a unique (sequentially incremented) id for the\n uri specified as parameter.\n Here the function is used to create consecutive invoice numbers:\n <pre xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">\n if (order:fulfilled($order-id))\n then\n   {\n     variable $invoice-id := seq:id(\"invoice\");\n     invoice:create($invoice-id, $order-id);\n   }\n else\n   ...\n </pre>\n", "summary" : "<p> Returns a unique (sequentially incremented) id for the\n uri specified as parameter.</p>", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "uri", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The name of the sequence counter as uri.</div>" } ], "returns" : { "type" : "xs:integer", "description" : "A sequentially incremented id." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "reset", "qname" : "seq:reset", "signature" : "($uri as xs:string) as empty-sequence()", "description" : " Resets the unique (sequentially incremented) id generation\n for the uri specified as parameter.\n", "summary" : "<p> Resets the unique (sequentially incremented) id generation\n for the uri specified as parameter.</p>", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "uri", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The name of the sequence counter as uri.</div>" } ], "returns" : { "type" : "empty-sequence()", "description" : "empty-sequence()" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "value", "qname" : "seq:value", "signature" : "($uri as xs:string) as xs:integer", "description" : " Returns the id for the uri specified as parameter without incrementing it.\n", "summary" : "<p> Returns the id for the uri specified as parameter without incrementing it.</p>", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "uri", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The name of the sequence counter as uri.</div>" } ], "returns" : { "type" : "xs:integer", "description" : "The current value" }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">seq:not-found if no counter with the given $uri was found</xqdoc:error>" ] } ], "variables" : [ { "name" : "seq:counters", "type" : "item()*", "description" : " The QName for the counters collection.\n" }, { "name" : "seq:counters-by-uri", "type" : "item()*", "description" : " The QName for the counters-by-uri collection.\n" } ] }, "http://api.28.io/databrowser" : { "ns" : "http://api.28.io/databrowser", "description" : "", "sees" : [  ], "authors" : [  ], "version" : null, "encoding" : null, "namespaces" : [ { "uri" : "", "prefix" : "an" }, { "uri" : "http://api.28.io/databrowser", "prefix" : "api" }, { "uri" : "http://zorba.io/modules/base64", "prefix" : "base64" }, { "uri" : "http://zorba.io/modules/store/static/collections/dml", "prefix" : "dml" }, { "uri" : "http://www.zorba-xquery.com/extensions", "prefix" : "ext" }, { "uri" : "http://www.functx.com", "prefix" : "functx" }, { "uri" : "http://expath.org/ns/http-client", "prefix" : "http" }, { "uri" : "http://www.zorba-xquery.com/modules/http-client", "prefix" : "http-client" }, { "uri" : "http://zorba.io/modules/xml-options", "prefix" : "opt" }, { "uri" : "http://www.28msec.com/modules/project", "prefix" : "project" }, { "uri" : "http://zorba.io/modules/reflection", "prefix" : "r" }, { "uri" : "http://zorba.io/modules/reference", "prefix" : "ref" }, { "uri" : "http://www.28msec.com/modules/http/request", "prefix" : "req" }, { "uri" : "http://www.28msec.com/modules/http/response", "prefix" : "res" }, { "uri" : "http://zorba.io/modules/xml", "prefix" : "xmlmod" }, { "uri" : "http://zorba.io/errors", "prefix" : "zerr" } ], "functions" : [ { "isDocumented" : false, "arity" : 1, "name" : "collection", "qname" : "api:collection", "signature" : "($name as xs:string)", "description" : "", "summary" : "", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "name", "type" : "xs:string", "occurrence" : null, "description" : "" } ], "returns" : { "type" : null, "description" : "" }, "errors" : [  ] }, { "isDocumented" : false, "arity" : 0, "name" : "delete", "qname" : "api:delete", "signature" : "()", "description" : "", "summary" : "", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [  ], "returns" : { "type" : null, "description" : "" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 0, "name" : "dispatch", "qname" : "api:dispatch", "signature" : "()", "description" : " Data Browser API dispatched.\n Handles API authorization to the portal and dispatching.\n", "summary" : "<p> Data Browser API dispatched.</p>", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [  ], "returns" : { "type" : null, "description" : "" }, "errors" : [  ] }, { "isDocumented" : false, "arity" : 0, "name" : "edit", "qname" : "api:edit", "signature" : "()", "description" : "", "summary" : "", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [  ], "returns" : { "type" : null, "description" : "" }, "errors" : [  ] }, { "isDocumented" : false, "arity" : 0, "name" : "options", "qname" : "api:options", "signature" : "()", "description" : "", "summary" : "", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [  ], "returns" : { "type" : null, "description" : "" }, "errors" : [  ] }, { "isDocumented" : false, "arity" : 1, "name" : "run", "qname" : "api:run", "signature" : "($sequential as xs:boolean) as object()", "description" : "", "summary" : "", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "sequential", "type" : "xs:boolean", "occurrence" : null, "description" : "" } ], "returns" : { "type" : "object()", "description" : "" }, "errors" : [  ] }, { "isDocumented" : false, "arity" : 1, "name" : "serialize", "qname" : "api:serialize", "signature" : "($items as item()*) as object()", "description" : "", "summary" : "", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "items", "type" : "item()", "occurrence" : "*", "description" : "" } ], "returns" : { "type" : "object()", "description" : "" }, "errors" : [  ] }, { "isDocumented" : false, "arity" : 0, "name" : "upload", "qname" : "api:upload", "signature" : "()", "description" : "", "summary" : "", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [  ], "returns" : { "type" : null, "description" : "" }, "errors" : [  ] } ], "variables" : [  ] }, "http://xbrl.io/modules/bizql/footnotes" : { "ns" : "http://xbrl.io/modules/bizql/footnotes", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">This module provides functions for retrieving XBRL footnotes\n (see section 4.11 Footnotes of the XBRL 2.1 specification).</p>\n", "sees" : [  ], "authors" : [ "<xqdoc:author xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Charles Hoffman</xqdoc:author>", "<xqdoc:author xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Matthias Brantner</xqdoc:author>", "<xqdoc:author xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Dennis Knochenwefel</xqdoc:author>", "<xqdoc:author xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Ghislain Fourny</xqdoc:author>" ], "version" : null, "encoding" : "utf-8", "namespaces" : [ { "uri" : "http://xbrl.io/modules/bizql/archives", "prefix" : "archives" }, { "uri" : "http://www.28msec.com/modules/credentials", "prefix" : "credentials" }, { "uri" : "http://xbrl.io/modules/bizql/entities", "prefix" : "entities" }, { "uri" : "http://xbrl.io/modules/bizql/facts", "prefix" : "facts" }, { "uri" : "http://xbrl.io/modules/bizql/footnotes", "prefix" : "footnotes" }, { "uri" : "http://www.28msec.com/modules/mongodb/types", "prefix" : "m" }, { "uri" : "http://www.28msec.com/modules/mongodb", "prefix" : "mongo" }, { "uri" : "http://zorba.io/modules/string", "prefix" : "string" }, { "uri" : "http://zorba.io/options/versioning", "prefix" : "ver" } ], "functions" : [ { "isDocumented" : true, "arity" : 1, "name" : "fnid", "qname" : "footnotes:fnid", "signature" : "($footnote-or-id as item()) as atomic", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Converts the input to a normalized footnote id (FNID). The input\n can be either an FNID, or a footnote object which contains an _id.</p>\n", "summary" : "<p>  Converts the input to a normalized footnote id (FNID).</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "footnote-or-id", "type" : "item()", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> an footnote object or FNID.</div>" } ], "returns" : { "type" : "atomic", "description" : "the normalized FNID." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">footnotes:INVALID-PARAMETER if the FNID or footnote is not valid.</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 1, "name" : "footnotes-for-archives", "qname" : "footnotes:footnotes-for-archives", "signature" : "($archive-or-ids as item()*) as object()*", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Return all footnotes reported within a given archive.</p>\n", "summary" : "<p>  Return all footnotes reported within a given archive.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "archive-or-ids", "type" : "item()", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> a sequence of archives or AIDs to filter.</div>" } ], "returns" : { "type" : "object()*", "description" : "all footnotes reported in these archives." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "footnotes-for-facts", "qname" : "footnotes:footnotes-for-facts", "signature" : "($facts-or-ids as item()*) as object()*", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Return all footnotes associated with the given facts.</p>\n", "summary" : "<p>  Return all footnotes associated with the given facts.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "facts-or-ids", "type" : "item()", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> a sequence of facts or fact IDs to filter</div>" } ], "returns" : { "type" : "object()*", "description" : "all footnotes associated with these facts." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 2, "name" : "footnotes-for-facts", "qname" : "footnotes:footnotes-for-facts", "signature" : "($facts-or-ids as item()*, $options as object()?) as object()*", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Return all footnotes associated with the given facts.</p>\n", "summary" : "<p>  Return all footnotes associated with the given facts.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "facts-or-ids", "type" : "item()", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> a sequence of facts or fact IDs to filter</div>" }, { "name" : "options", "type" : "object()", "occurrence" : "?", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> <a href=\"facts#standard_options\">standard fact retrieving options</a>.</div>" } ], "returns" : { "type" : "object()*", "description" : "all footnotes associated with these facts." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "footnotes-search", "qname" : "footnotes:footnotes-search", "signature" : "($search as string) as object()*", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Return all footnotes that match the given search term.</p>\n", "summary" : "<p>  Return all footnotes that match the given search term.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "search", "type" : "string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the search query</div>" } ], "returns" : { "type" : "object()*", "description" : "all footnotes matching the given search query" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 0, "name" : "footnotes", "qname" : "footnotes:footnotes", "signature" : "() as object()*", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Return all footnotes.</p>\n", "summary" : "<p>  Return all footnotes.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [  ], "returns" : { "type" : "object()*", "description" : "all footnotes." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "footnotes", "qname" : "footnotes:footnotes", "signature" : "($footnote-or-ids as item()*) as object()?", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Return the footnote with the given FNIDs.</p>\n", "summary" : "<p>  Return the footnote with the given FNIDs.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "footnote-or-ids", "type" : "item()", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the FNIDs or the footnotes themselves.</div>" } ], "returns" : { "type" : "object()?", "description" : "the footnotes with the given FNIDs the empty sequence if no footnote was found or if the input is an empty sequence." }, "errors" : [  ] } ], "variables" : [ { "name" : "footnotes:col", "type" : "string", "description" : " Name of the collection the footnotes are stored in.\n" }, { "name" : "footnotes:ARCHIVE", "type" : "string", "description" : " Name of the field that points to the archive.\n" }, { "name" : "footnotes:FACTS", "type" : "string", "description" : " Name of the field that points to the facts linked to this footnote.\n" }, { "name" : "footnotes:LANG", "type" : "string", "description" : " Name of the field that holds the language attribute of the footnote.\n" } ] }, "http://zorba.io/modules/store/static/indexes/dml" : { "ns" : "http://zorba.io/modules/store/static/indexes/dml", "description" : " This module defines a set of functions to probe and refresh indexes which are\n declared in the prolog of a module.\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n This module is part of\n <a xmlns:xqdoc=\"http://www.xqdoc.org/1.0\" href=\"../zorba/xqddf.html\">Zorba's XQuery Data Definition Facility</a>.\n All the indexes managed by this module have to be pre-declared in the prolog\n of a library module.\n Please refer to the\n <a xmlns:xqdoc=\"http://www.xqdoc.org/1.0\" href=\"../zorba/data_lifecycle.html\">general documentation</a>\n for more information and examples.\n", "sees" : [ "<xqdoc:see xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"><a href=\"../zorba/data_lifecycle.html\">Data Lifecycle</a></xqdoc:see>", "<xqdoc:see xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"><a href=\"../zorba/xqddf.html\">XQuery Data Definition Facility</a></xqdoc:see>", "<xqdoc:see xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">http://zorba.io/modules/store/static/indexes/ddl</xqdoc:see>", "<xqdoc:see xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">http://zorba.io/modules/store/static/collections/dml</xqdoc:see>", "<xqdoc:see xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">http://zorba.io/modules/store/static/collections/ddl</xqdoc:see>", "<xqdoc:see xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">http://zorba.io/modules/store/static/integrity-constraints/ddl</xqdoc:see>", "<xqdoc:see xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">http://zorba.io/modules/store/static/integrity-constraints/dml</xqdoc:see>", "<xqdoc:see xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">http://zorba.io/errors</xqdoc:see>" ], "authors" : [ "<xqdoc:author xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Zorba Team</xqdoc:author>" ], "version" : null, "encoding" : "utf-8", "namespaces" : [ { "uri" : "http://zorba.io/annotations", "prefix" : "an" }, { "uri" : "http://zorba.io/modules/store/static/indexes/dml", "prefix" : "idml" }, { "uri" : "http://zorba.io/options/versioning", "prefix" : "ver" }, { "uri" : "http://zorba.io/errors", "prefix" : "zerr" } ], "functions" : [ { "isDocumented" : true, "arity" : 1, "name" : "keys", "qname" : "idml:keys", "signature" : "($name as xs:QName) as node()* external", "description" : " Gets a sequence of all keys contained in the index with the given name.\n Each element has the following structure:\n  <pre xmlns:xqdoc=\"http://www.xqdoc.org/1.0\" class=\"brush: xml;\">\n   &lt;key xmlns=\"http://zorba.io/modules/store/static/indexes/dml\"&gt;\n     &lt;attribute value=\"key1_value\"/&gt;\n     &lt;attribute value=\"key2_value\"/&gt;\n     &lt;attribute value=\"key3_value\"/&gt;\n   &lt;/key&gt;\n  </pre>\n Note that the order of the attribute elements reflects the order of\n the keys in the index specification. Also note that the values in\n these attributes have the type that is declared in the corresponding\n index specification.\n", "summary" : "<p> Gets a sequence of all keys contained in the index with the given name.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "name", "type" : "xs:QName", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The name of the index to get the keys for.</div>" } ], "returns" : { "type" : "node()*", "description" : "A sequence of elements comprising the keys in the index." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">zerr:ZDDY0021 if the index was not declared.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">zerr:ZDDY0023 if the index does not exist.</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 2, "name" : "probe-index-point-general", "qname" : "idml:probe-index-point-general", "signature" : "($name as xs:QName, $key as xs:anyAtomicType*) as node()* external", "description" : " Gets from an index the domain nodes associated by general equality with a\n given <em xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">search sequence </em>.\n The search sequence consists of an arbitrary number of <em xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">search keys</em>\n where each search key is an atomic item.\n The function is supported by general indexes only.\n", "summary" : "<p> Gets from an index the domain nodes associated by general equality with a\n given  search sequence  .</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "name", "type" : "xs:QName", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The name of the index to probe.</div>" }, { "name" : "key", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The search sequence.</div>" } ], "returns" : { "type" : "node()*", "description" : "The set of domain nodes for which the following XQuery expression returns true: <pre xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> $keys = $node/keyExpr </pre> where keyExpr is the expression specified in the keyspec of the index (remember that for general indexes, there can be only one keyspec)." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">err:XPTY0004 if the search sequence contains a search key whose type does not match the sequence type specified in the keyspec of the index.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">zerr:ZDDY0021 if the index with name $name is not declared.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">zerr:ZDDY0023 if the index with name $name does not exist.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">zerr:ZDDY0029 if the index is not general.</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 3, "name" : "probe-index-point-value-skip", "qname" : "idml:probe-index-point-value-skip", "signature" : "($name as xs:QName, $skip as xs:integer, $key_i as xs:anyAtomicType?) as node()* external", "description" : " This is an extension of the <code xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">probe-index-point-value()</code> function\n in that it allows index items to be skipped.\n", "summary" : "<p> This is an extension of the  probe-index-point-value()  function\n in that it allows index items to be skipped.</p>", "annotation_str" : " %an:variadic", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "variadic", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "name", "type" : "xs:QName", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The name of the index to probe.</div>" }, { "name" : "skip", "type" : "xs:integer", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The number of index items to skip.</div>" }, { "name" : "key_i", "type" : "xs:anyAtomicType", "occurrence" : "?", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The search keys used to probe the index with. The i-th search key corresponds to the i-th key expression in the index declaration.</div>" } ], "returns" : { "type" : "node()*", "description" : "The set of domain nodes that satisfy the search condition." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">zerr:ZDDY0021 if the index is not declared.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">zerr:ZDDY0023 if the index does not exist.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">zerr:ZDDY0025 if the number of search keys passed as arguments is not the same as the number of keys declared for the index.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">err:XPTY0004 if a non-empty seach key is given whose type does not match the sequence type specified in the corresponding keyspec.</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 2, "name" : "probe-index-point-value", "qname" : "idml:probe-index-point-value", "signature" : "($name as xs:QName, $key_i as xs:anyAtomicType?) as node()* external", "description" : " Gets the domain nodes from an index associated by value equality with a\n given <em xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">search tuple</em>.\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n The search tuple consists of a number of <em xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">search keys</em> where each\n search key is either an atomic item or the empty sequence.  The number of\n search keys given must be equal to the number of keys declared for the\n index. Since the number of keys differs from one index to another, this\n function is variadic.\n", "summary" : "<p> Gets the domain nodes from an index associated by value equality with a\n given  search tuple .</p>", "annotation_str" : " %an:variadic", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "variadic", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "name", "type" : "xs:QName", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The of the index to probe.</div>" }, { "name" : "key_i", "type" : "xs:anyAtomicType", "occurrence" : "?", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> A search key used to probe the index with. The i-th search key corresponds to the i-th key expression in the index declaration.</div>" } ], "returns" : { "type" : "node()*", "description" : "The set of domain nodes for which the following XQuery expression returns true: <pre xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> $key1 eq $node/keyExpr1 and ... and $keyM eq $node/keyExprM </pre> where <i xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">keyExpr<sub>i</sub></i> is the expression specified in the i-th keyspec of the index." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">zerr:ZDDY0021 if the index is not declared.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">zerr:ZDDY0023 if the index does not exist.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">zerr:ZDDY0025 if the number of search keys passed as arguments is not the same as the number of keys declared for the index.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">err:XPTY0004 if a non-empty seach key is given whose type does not match the sequence type specified in the corresponding keyspec.</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 7, "name" : "probe-index-range-general", "qname" : "idml:probe-index-range-general", "signature" : "($name as xs:QName, $lowerBound as xs:anyAtomicType*, $upperBound as xs:anyAtomicType*, $haveLowerBound as xs:boolean, $haveUpperBound as xs:boolean, $lowerBoundIncluded as xs:boolean, $upperBoundIncluded as xs:boolean) as node()* external", "description" : " Gets the domain nodes associated by general order-comparison (operators\n <code xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">&lt;=</code>, <code xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">&lt;</code>, <code xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">&gt;=</code>,\n <code xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">&gt;</code>) with one or two <em xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">search sequences</em>.\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n Each search sequence consists of an arbitrary number of\n <em xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">search keys</em> where each search key is an atomic item.\n This method is supported by general range indexes only.\n Its result is either an error or the set of domain nodes for which the\n following XQuery expression returns true:\n  <pre xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">\n    if ( $haveLowerBound and $haveUpperBound ) then\n      $lowerBoundKeys lop $node/keyExpr and $node/keyExpr uop $upperBoundKeys\n    else if ( $haveLowerBound ) then\n      $lowerBoundKeys lop $node/keyExpr\n    else if ( $haveUpperBound ) then\n      $node/keyExpr uop $upperBoundKeys\n    else\n      fn:true()\n  </pre>\n where <i xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">keyExpr</i> is the expression specified in the keyspec of the\n index, <i xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">lop</i> is either the <code xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">&lt;=</code> or the <code xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">&lt;</code>\n operator depending on whether <code xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">$lowerBoundsIncluded</code> is true or\n false, and <i xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">uop</i> is either the <code xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">&lt;=</code> or the\n <code xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">&lt;</code> operator depending on whether\n <code xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">$upperBoundsIncluded</code> is true or false.\n", "summary" : "<p> Gets the domain nodes associated by general order-comparison (operators\n  &lt;= ,  &lt; ,  &gt;= ,\n  &gt; ) with one or two  search sequences .</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "name", "type" : "xs:QName", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The of the index to probe.</div>" }, { "name" : "lowerBound", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The lower bound search sequence.</div>" }, { "name" : "upperBound", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The upper bound search sequence.</div>" }, { "name" : "haveLowerBound", "type" : "xs:boolean", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> Whether a lower bound search sequence exists or not.</div>" }, { "name" : "haveUpperBound", "type" : "xs:boolean", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> Whether an upper bound search sequence exists or not.</div>" }, { "name" : "lowerBoundIncluded", "type" : "xs:boolean", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> Whether to use the <code>&lt;=</code> or the <code>&lt;</code> operator when comparing a search key from <code>$lowerBound</code> with an index key.</div>" }, { "name" : "upperBoundIncluded", "type" : "xs:boolean", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> Whether to use the <code>&lt;=</code> or the <code>&lt;</code> operator when comparing an index key with a search key from <code>$upperBound</code>.</div>" } ], "returns" : { "type" : "node()*", "description" : "The set of domain nodes that satisfy the search condition." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">zerr:ZDDY0021 if the index was not declared.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">zerr:ZDDY0023 if the index does not exist.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">zerr:ZDDY0030 if the index is not a general range index.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">err:XPTY0004 if <code>$haveLowerBound</code> is true and <code>$lowerBoundKeys</code> contains an atomic item whose type does not match the sequence type specified by the index keyspec, or <code>$haveUpperBound</code> is true and <code>$upperBoundKeys</code> contains an atomic item whose type does not match the sequence type specified by the index keyspec.</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 8, "name" : "probe-index-range-value-skip", "qname" : "idml:probe-index-range-value-skip", "signature" : "($name as xs:QName, $skip as xs:integer, $lowerBound-i as xs:anyAtomicType?, $upperBound-i as xs:anyAtomicType?, $haveLowerBound-i as xs:boolean, $haveUpperBound-i as xs:boolean, $lowerBoundIncluded-i as xs:boolean, $upperBoundIncluded-i as xs:boolean) as node()* external", "description" : " This function is an extension of the <code xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">probe-index-range-value()</code>\n function that index items to be skipped.\n", "summary" : "<p> This function is an extension of the  probe-index-range-value() \n function that index items to be skipped.</p>", "annotation_str" : " %an:variadic", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "variadic", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "name", "type" : "xs:QName", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The QName of the index to probe</div>" }, { "name" : "skip", "type" : "xs:integer", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The number of index items to skip.</div>" }, { "name" : "lowerBound-i", "type" : "xs:anyAtomicType", "occurrence" : "?", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The lower bound in a range of key values.</div>" }, { "name" : "upperBound-i", "type" : "xs:anyAtomicType", "occurrence" : "?", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The upper bound in a range of key values.</div>" }, { "name" : "haveLowerBound-i", "type" : "xs:boolean", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> If false, then there is no lower bound, or equivalently, the lower bound is -INFINITY. Otherwise, the lower bound is the one given by the <code>$lowerBound-i</code> value.</div>" }, { "name" : "haveUpperBound-i", "type" : "xs:boolean", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> If false, then there is no upper bound, or equivalently, the upper bound is +INFINITY. Otherwise, the upper bound is the one given by the <code>$upperBound-i</code> value.</div>" }, { "name" : "lowerBoundIncluded-i", "type" : "xs:boolean", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> If false, then the range is open from below, i.e., the lowerBound-i value is not considered part of the range. Otherwise, the range is closed from below, i.e., the <code>$lowerBound-i</code> value is part of the range.</div>" }, { "name" : "upperBoundIncluded-i", "type" : "xs:boolean", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> If false, then the range is open from above, i.e., the upperBound-i value is not considered part of the range. Otherwise, the range is closed from above, i.e., the <code>$upperBound-i</code> value is part of the range.</div>" } ], "returns" : { "type" : "node()*", "description" : "The set of domain nodes that satisfy the search condition." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">zerr:ZDDY0021 if the index with name $name is not declared.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">zerr:ZDDY0023 if the index with name $name does not exist.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">zerr:ZDDY0025 if the number of rangespecs passed as arguments is zero or greater than the number of keys declared for the index.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">zerr:ZDDY0026 if the index is not a range index.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">err:XPTY0004 if <code>$haveLowerBound-i</code> is true and <code>$lowerBound-i</code> is an atomic item whose type does not match the sequence type specified by the i<sup>th</sup> keyspec, or <code>$haveUpperBound-i</code> is true and <code>$upperBound-i</code> is an atomic item whose type does not match the sequence type specified by the i<sup>th</sup> keyspec.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">zerr:ZDDY0034 if (a) the index is general (in which case there is only one rangespac), (b) the index is untyped, (c) there is both a lower and an upper bound, and (d) if T1 and T2 are the types of the lower and upper bound, neither T1 is a subtype of T2 nor T2 is a subtype of T1.</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 7, "name" : "probe-index-range-value", "qname" : "idml:probe-index-range-value", "signature" : "($name as xs:QName, $lowerBound-i as xs:anyAtomicType?, $upperBound-i as xs:anyAtomicType?, $haveLowerBound-i as xs:boolean, $haveUpperBound-i as xs:boolean, $lowerBoundIncluded-i as xs:boolean, $upperBoundIncluded-i as xs:boolean) as node()* external", "description" : " Gets the domain nodes associated by value order-comparison (operators\n <code xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">le</code>, <code xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">lt</code>, <code xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">ge</code>, <code xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">gt</code>) with a\n given <em xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">search box</em>.\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n The search box is specified as a number <i xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">M</i> of <em xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">rangespecs</em>\n where each rangespec consists of six values.\n The number <i xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">M</i> must be greater than 0 and less than or equal to the\n number <i xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">N</i> of keyspecs found in the index declaration.\n If <i xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">M</i> &lt; <i xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">N</i>, then the \"missing\" rangespecs are assumed to have\n the following value: [(), (), false, false, false, false].\n As a result, we can assume that <i xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">M</i> = <i xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">N</i>.\n Remember that for general indexes, there can be only one IndexKeySpec and,\n as a result for general indexes, <i xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">M</i> = <i xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">N</i> = 1.\n Since the number of keys differs from one index to another,\n this function is variadic.\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n The i<sup xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">th</sup> rangespec corresponds to the i<sup xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">th</sup> keyspec, and\n specifies a search condition on the key values that are produced by\n evaluating that keyspec for every domain node.\n Specifically, we define the i<sup xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">th</sup> <em xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">rangespec result</em> as the\n set of domain nodes for which the following XQuery expression returns\n true:\n  <pre xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">\n    if ( $haveLowerBound-i and $haveUpperBound-i ) then\n      $lowerBound-i lop $node/keyExpr-i and $node/keyExpr-i uop $upperBound-i\n    else if ( $haveLowerBound-i ) then\n      $lowerBound-i lop $node/keyExpr-i\n    else if ( $haveUpperBound-i ) then\n      $node/keyExpr-i uop $upperBound-i\n    else\n      fn:true()\n  </pre>\n where <i xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">keyExpr-i</i> is the expression specified by the i<sup xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">th</sup>\n keyspec of the index, <i xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">lop</i> is either the <code xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">le</code> or the\n <code xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">lt</code> operator depending on whether\n <code xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">$lowerBoundsIncluded-i</code> is true or false, and <i xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">uop</i> is\n either the <i xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">le</i> or the <i xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">lt</i> operator depending on whether\n <code xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">$upperBoundsIncluded-i</code> is true or false.\n", "summary" : "<p> Gets the domain nodes associated by value order-comparison (operators\n  le ,  lt ,  ge ,  gt ) with a\n given  search box .</p>", "annotation_str" : " %an:variadic", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "variadic", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "name", "type" : "xs:QName", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The name of the index to probe.</div>" }, { "name" : "lowerBound-i", "type" : "xs:anyAtomicType", "occurrence" : "?", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The lower bound in a range of key values.</div>" }, { "name" : "upperBound-i", "type" : "xs:anyAtomicType", "occurrence" : "?", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The upper bound in a range of key values.</div>" }, { "name" : "haveLowerBound-i", "type" : "xs:boolean", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> If false, then there is no lower bound, or equivalently, the lower bound is -INFINITY. Otherwise, the lower bound is the one given by the <code>$lowerBound-i</code> value.</div>" }, { "name" : "haveUpperBound-i", "type" : "xs:boolean", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> If false, then there is no upper bound, or equivalently, the upper bound is +INFINITY. Otherwise, the upper bound is the one given by the <code>$upperBound-i</code> value.</div>" }, { "name" : "lowerBoundIncluded-i", "type" : "xs:boolean", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> If false, then the range is open from below, i.e., the <code>$lowerBound-i</code> value is not considered part of the range. Otherwise, the range is closed from below, i.e., the <code>$lowerBound-i</code> value is part of the range.</div>" }, { "name" : "upperBoundIncluded-i", "type" : "xs:boolean", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> If false, then the range is open from above, i.e., the <code>$upperBound-i</code> value is not considered part of the range. Otherwise, the range is closed from above, i.e., the <code>$upperBound-i</code> value is part of the range.</div>" } ], "returns" : { "type" : "node()*", "description" : "The intersection of all the rangespec results." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">zerr:ZDDY0021 if the index is not declared.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">zerr:ZDDY0023 if the index does not exist.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">zerr:ZDDY0025 if the number of rangespecs passed as arguments is zero or greater than the number of keys declared for the index.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">zerr:ZDDY0026 if the index is not a range index.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">err:XPTY0004 if <code>$haveLowerBound-i</code> is true and <code>$lowerBound-i</code> is an atomic item whose type does not match the sequence type specified by the i<sup>th</sup> keyspec, or <code>$haveUpperBound-i</code> is true and <code>$upperBound-i</code> is an atomic item whose type does not match the sequence type specified by the i<sup>th</sup> keyspec.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">zerr:ZDDY0034 if (a) the index is general (in which case there is only one rangespac), (b) the index is untyped, (c) there is both a lower and an upper bound, and (d) if T1 and T2 are the types of the lower and upper bound, neither T1 is a subtype of T2 nor T2 is a subtype of T1.</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 1, "name" : "refresh-index", "qname" : "idml:refresh-index", "signature" : "($name as xs:QName) external", "description" : " Updates the index with the given name.  Note that if the maintenance\n property of the index is automatic, this function does nothing.\n", "summary" : "<p> Updates the index with the given name.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : true, "parameters" : [ { "name" : "name", "type" : "xs:QName", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The name of the index to refresh.</div>" } ], "returns" : { "type" : null, "description" : "An empty XDM instance and a pending update list that, when applied, refreshes the contents of the index." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">zerr:ZDDY0021 if the index is not declared.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">zerr:ZDDY0023 if the index does not exist.</xqdoc:error>" ] } ], "variables" : [  ] }, "http://www.28msec.com/modules/full-text" : { "ns" : "http://www.28msec.com/modules/full-text", "description" : " This module provides an API to full-text functions such as tokenization,\n stemming, or stop word detection. The default language is set to english\n for all functions that don't take an explicit language parameter.\n <h2 xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Notes on stemming</h2>\n The <code xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">stem()</code> functions return the\n <a xmlns:xqdoc=\"http://www.xqdoc.org/1.0\" href=\"http://en.wikipedia.org/wiki/Word_stem\">stem</a>\n of a word.\n On 28.io,\n the stem of a word itself, however, is not guaranteed to be a word.\n It is best to consider a stem as an opaque byte sequence.\n All that is guaranteed about a stem is that,\n for a given word,\n the stem of that word will always be the same byte sequence.\n Hence,\n you sould never compare the result of one of the <code xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">stem()</code>\n functions against a non-stemmed string,\n for example:\n <pre xmlns:xqdoc=\"http://www.xqdoc.org/1.0\" class=\"ace-static\">\n  if ( ft:stem( \"apples\" ) eq \"apple\" )             ** WRONG **\n </pre>\n Instead do:\n <pre xmlns:xqdoc=\"http://www.xqdoc.org/1.0\" class=\"ace-static\">\n  if ( ft:stem( \"apples\" ) eq ft:stem( \"apple\" ) )  ** CORRECT **\n </pre>\n <h2 xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Notes on the thesaurus</h2>\n 28msec uses the\n <a xmlns:xqdoc=\"http://www.xqdoc.org/1.0\" href=\"http://wordnet.princeton.edu/\">WordNet lexical database</a> version 3.0,\n In WordNet, the number of \"levels\" that two phrases are apart\n are how many hierarchical meanings apart they are.\n For example,\n \"canary\" is 5 levels away from \"vertebrate\"\n (carary &gt; finch &gt; oscine &gt; passerine &gt; bird &gt; vertebrate).\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n When using the WordNet implementation,\n 28msec supports all of the relationships (and their abbreviations)\n specified by\n <a xmlns:xqdoc=\"http://www.xqdoc.org/1.0\" href=\"http://www.iso.org/iso/iso_catalogue/catalogue_tc/catalogue_detail.htm?csnumber=7776\">ISO 2788</a>\n and\n <a xmlns:xqdoc=\"http://www.xqdoc.org/1.0\" href=\"http://www.niso.org/kst/reports/standards?step=2&amp;gid=&amp;project_key=7cc9b583cb5a62e8c15d3099e0bb46bbae9cf38a\">ANSI/NISO Z39.19-2005</a>\n with the exceptions of \"HN\" (history note)\n and \"X SN\" (see scope note for).\n These relationships are:\n  <table xmlns:xqdoc=\"http://www.xqdoc.org/1.0\" class=\"table table-bordered\">\n    <tr>\n      <th>Rel.</th>\n      <th>Meaning</th>\n      <th>WordNet Rel.</th>\n    </tr>\n    <tr>\n      <td>BT</td>\n      <td>broader term</td>\n      <td>hypernym</td>\n    </tr>\n    <tr>\n      <td>BTG</td>\n      <td>broader term generic</td>\n      <td>hypernym</td>\n    </tr>\n    <tr>\n      <td>BTI</td>\n      <td>broader term instance</td>\n      <td>instance hypernym</td>\n    </tr>\n    <tr>\n      <td>BTP</td>\n      <td>broader term partitive</td>\n      <td>part meronym</td>\n    </tr>\n    <tr>\n      <td>NT</td>\n      <td>narrower term</td>\n      <td>hyponym</td>\n    </tr>\n    <tr>\n      <td>NTG</td>\n      <td>narrower term generic</td>\n      <td>hyponym</td>\n    </tr>\n    <tr>\n      <td>NTI</td>\n      <td>narrower term instance</td>\n      <td>instance hyponym</td>\n    </tr>\n    <tr>\n      <td>NTP</td>\n      <td>narrower term partitive</td>\n      <td>part holonym</td>\n    </tr>\n    <tr>\n      <td>RT</td>\n      <td>related term</td>\n      <td>also see</td>\n    </tr>\n    <tr>\n      <td>SN</td>\n      <td>scope note</td>\n      <td>n/a</td>\n    </tr>\n    <tr>\n      <td>TT</td>\n      <td>top term</td>\n      <td>hypernym</td>\n    </tr>\n    <tr>\n      <td>UF</td>\n      <td>non-preferred term</td>\n      <td>n/a</td>\n    </tr>\n    <tr>\n      <td>USE</td>\n      <td>preferred term</td>\n      <td>n/a</td>\n    </tr>\n  </table>\n Note that you can specify relationships\n either by their abbreviation\n or their meaning.\n Relationships are case-insensitive.\n In addition to the\n <a xmlns:xqdoc=\"http://www.xqdoc.org/1.0\" href=\"http://www.iso.org/iso/iso_catalogue/catalogue_tc/catalogue_detail.htm?csnumber=7776\">ISO 2788</a>\n and\n <a xmlns:xqdoc=\"http://www.xqdoc.org/1.0\" href=\"http://www.niso.org/kst/reports/standards?step=2&amp;gid=&amp;project_key=7cc9b583cb5a62e8c15d3099e0bb46bbae9cf38a\">ANSI/NISO Z39.19-2005</a>\n relationships,\n 28msec also supports all of the relationships offered by WordNet.\n These relationships are:\n  <table xmlns:xqdoc=\"http://www.xqdoc.org/1.0\" class=\"ft_rels table table-bordered\">\n    <tr>\n      <th>Relationship</th>\n      <th>Meaning</th>\n    </tr>\n    <tr>\n      <td nowrap=\"nowrap\">also see</td>\n      <td>\n        A word that is related to another,\n        e.g., for \"varnished\" (furniture)\n        one should <em>also see</em> \"finished.\"\n      </td>\n    </tr>\n    <tr>\n      <td>antonym</td>\n      <td>\n        A word opposite in meaning to another,\n        e.g., \"light\" is an <em>antonym</em> for \"heavy.\"\n      </td>\n    </tr>\n    <tr>\n      <td>attribute</td>\n      <td>\n        A noun for which adjectives express values,\n        e.g., \"weight\" is an <em>attribute</em>\n        for which the adjectives \"light\" and \"heavy\"\n        express values.\n      </td>\n    </tr>\n    <tr>\n      <td>cause</td>\n      <td>\n        A verb that causes another,\n        e.g., \"show\" is a <em>cause</em> of \"see.\"\n      </td>\n    </tr>\n    <tr>\n      <td nowrap=\"nowrap\">derivationally related form</td>\n      <td>\n        A word that is derived from a root word,\n        e.g., \"metric\" is a <em>derivationally related form</em> of \"meter.\"\n      </td>\n    </tr>\n    <tr>\n      <td nowrap=\"nowrap\">derived from adjective</td>\n      <td>\n        An adverb that is derived from an adjective,\n        e.g., \"correctly\" is <em>derived from the adjective</em> \"correct.\"\n      </td>\n    </tr>\n    <tr>\n      <td>entailment</td>\n      <td>\n        A verb that presupposes another,\n        e.g., \"snoring\" <em>entails</em> \"sleeping.\"\n      </td>\n    </tr>\n    <tr>\n      <td>hypernym</td>\n      <td>\n        A word with a broad meaning that more specific words fall under,\n        e.g., \"meal\" is a <em>hypernym</em> of \"breakfast.\"\n      </td>\n    </tr>\n    <tr>\n      <td>hyponym</td>\n      <td>\n        A word of more specific meaning than a general term applicable to it,\n        e.g., \"breakfast\" is a <em>hyponym</em> of \"meal.\"\n      </td>\n    </tr>\n    <tr>\n      <td nowrap=\"nowrap\">instance hypernym</td>\n      <td>\n        A word that denotes a category of some specific instance,\n        e.g., \"author\" is an <em>instance hypernym</em> of \"Asimov.\"\n      </td>\n    </tr>\n    <tr>\n      <td nowrap=\"nowrap\">instance hyponym</td>\n      <td>\n        A term that donotes a specific instance of some general category,\n        e.g., \"Asimov\" is an <em>instance hyponym</em> of \"author.\"\n      </td>\n    </tr>\n    <tr>\n      <td nowrap=\"nowrap\">member holonym</td>\n      <td>\n        A word that denotes a collection of individuals,\n        e.g., \"faculty\" is a <em>member holonym</em> of \"professor.\"\n      </td>\n    </tr>\n    <tr>\n      <td nowrap=\"nowrap\">member meronym</td>\n      <td>\n        A word that denotes a member of a larger group,\n        e.g., a \"person\" is a <em>member meronym</em> of a \"crowd.\"\n      </td>\n    </tr>\n    <tr>\n      <td nowrap=\"nowrap\">part holonym</td>\n      <td>\n        A word that denotes a larger whole comprised of some part,\n        e.g., \"car\" is a <em>part holonym</em> of \"engine.\"\n      </td>\n    </tr>\n    <tr>\n      <td nowrap=\"nowrap\">part meronym</td>\n      <td>\n        A word that denotes a part of a larger whole,\n        e.g., an \"engine\" is <em>part meronym</em> of a \"car.\"\n      </td>\n    </tr>\n    <tr>\n      <td nowrap=\"nowrap\">participle of verb</td>\n      <td>\n        An adjective that is the participle of some verb,\n        e.g., \"breaking\" is the <em>participle of the verb</em> \"break.\"\n      </td>\n    </tr>\n    <tr>\n      <td>pertainym</td>\n      <td>\n        An adjective that classifies its noun,\n        e.g., \"musical\" is a <em>pertainym</em> in \"musical instrument.\"\n      </td>\n    </tr>\n    <tr>\n      <td nowrap=\"nowrap\">similar to</td>\n      <td>\n        Similar, though not necessarily interchangeable, adjectives.\n        For example, \"shiny\" is <em>similar to</em> \"bright\",\n        but they have subtle differences.\n      </td>\n    </tr>\n    <tr>\n      <td nowrap=\"nowrap\">substance holonym</td>\n      <td>\n        A word that denotes a larger whole containing some constituent\n        substance, e.g., \"bread\" is a <em>substance holonym</em> of \"flour.\"\n      </td>\n    </tr>\n    <tr>\n      <td nowrap=\"nowrap\">substance meronym</td>\n      <td>\n        A word that denotes a constituant substance of some larger whole,\n        e.g., \"flour\" is a <em>substance meronym</em> of \"bread.\"\n      </td>\n    </tr>\n    <tr>\n      <td nowrap=\"nowrap\">verb group</td>\n      <td>\n        A verb that is a member of a group of similar verbs,\n        e.g., \"live\" is in the <em>verb group</em>\n        of \"dwell\", \"live\", \"inhabit\", etc.\n      </td>\n    </tr>\n  </table>\n", "sees" : [  ], "authors" : [ "<xqdoc:author xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Paul J. Lucas</xqdoc:author>" ], "version" : null, "encoding" : "utf-8", "namespaces" : [ { "uri" : "http://www.w3.org/2005/xqt-errors", "prefix" : "err" }, { "uri" : "http://www.28msec.com/modules/full-text", "prefix" : "ft" }, { "uri" : "http://zorba.io/options/versioning", "prefix" : "ver" }, { "uri" : "http://zorba.io/errors", "prefix" : "zerr" }, { "uri" : "http://zorba.io/modules/full-text", "prefix" : "zft" } ], "functions" : [ { "isDocumented" : true, "arity" : 1, "name" : "is-stem-lang-supported", "qname" : "ft:is-stem-lang-supported", "signature" : "($lang as language) as boolean", "description" : " Checks whether the given language is supported for stemming.\n", "summary" : "<p> Checks whether the given language is supported for stemming.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "lang", "type" : "language", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The language to check.</div>" } ], "returns" : { "type" : "boolean", "description" : "<code xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">true</code> only if the language is supported." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "is-stop-word-lang-supported", "qname" : "ft:is-stop-word-lang-supported", "signature" : "($lang as language) as boolean", "description" : " Checks whether the given language\n is supported for stop words.\n", "summary" : "<p> Checks whether the given language\n is supported for stop words.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "lang", "type" : "language", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The language to check.</div>" } ], "returns" : { "type" : "boolean", "description" : "<code xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">true</code> only if the language is supported." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "is-stop-word", "qname" : "ft:is-stop-word", "signature" : "($word as string) as boolean", "description" : " Checks whether the given word is a stop-word.\n", "summary" : "<p> Checks whether the given word is a stop-word.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "word", "type" : "string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The word to check. The word's language is assumed to be the one returned by <code>ft:current-lang()</code>.</div>" } ], "returns" : { "type" : "boolean", "description" : "<code xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">true</code> only if <code xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">$word</code> is a stop-word." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">err:FTST0009 if <code>ft:current-lang()</code> is not supported.</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 2, "name" : "is-stop-word", "qname" : "ft:is-stop-word", "signature" : "($word as string, $lang as language) as boolean", "description" : " Checks whether the given word is a stop-word.\n", "summary" : "<p> Checks whether the given word is a stop-word.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "word", "type" : "string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The word to check.</div>" }, { "name" : "lang", "type" : "language", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The language of <code>$word</code>.</div>" } ], "returns" : { "type" : "boolean", "description" : "<code xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">true</code> only if <code xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">$word</code> is a stop-word." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">err:FTST0009 if <code>$lang</code> is not supported.</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 1, "name" : "is-thesaurus-lang-supported", "qname" : "ft:is-thesaurus-lang-supported", "signature" : "($lang as language) as boolean", "description" : " Checks whether the given language\n is supported for look-up using the Wordnet thesaurus.\n", "summary" : "<p> Checks whether the given language\n is supported for look-up using the Wordnet thesaurus.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "lang", "type" : "language", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The language to check.</div>" } ], "returns" : { "type" : "boolean", "description" : "<code xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">true</code> only if the language is supported." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "is-tokenizer-lang-supported", "qname" : "ft:is-tokenizer-lang-supported", "signature" : "($lang as language) as boolean", "description" : " Checks whether the given language\n is supported for tokenization.\n", "summary" : "<p> Checks whether the given language\n is supported for tokenization.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "lang", "type" : "language", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The language to check.</div>" } ], "returns" : { "type" : "boolean", "description" : "<code xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">true</code> only if the language is supported." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "stem", "qname" : "ft:stem", "signature" : "($word as string) as string", "description" : " Stems the given word.\n", "summary" : "<p> Stems the given word.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "word", "type" : "string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The word to stem. The word's language is assumed to be the one returned by <code>ft:current-lang()</code>.</div>" } ], "returns" : { "type" : "string", "description" : "the stem of <code xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">$word</code>." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">err:FTST0009 if <code>ft:current-lang()</code> is not supported.</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 2, "name" : "stem", "qname" : "ft:stem", "signature" : "($word as string, $lang as language) as string", "description" : " Stems the given word.\n", "summary" : "<p> Stems the given word.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "word", "type" : "string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The word to stem.</div>" }, { "name" : "lang", "type" : "language", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The language of <code>$word</code>.</div>" } ], "returns" : { "type" : "string", "description" : "the stem of <code xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">$word</code>." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">err:FTST0009 if <code>$lang</code> is not supported.</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 1, "name" : "strip-diacritics", "qname" : "ft:strip-diacritics", "signature" : "($string as string) as string", "description" : " Strips all diacritical marks from all characters.\n", "summary" : "<p> Strips all diacritical marks from all characters.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "string", "type" : "string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The string to strip diacritical marks from.</div>" } ], "returns" : { "type" : "string", "description" : "<code xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">$string</code> with diacritical marks stripped." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "thesaurus-lookup", "qname" : "ft:thesaurus-lookup", "signature" : "($phrase as string) as string*", "description" : " Looks-up the given phrase in the Wordnet thesaurus.\n", "summary" : "<p> Looks-up the given phrase in the Wordnet thesaurus.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "phrase", "type" : "string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The phrase to look up.</div>" } ], "returns" : { "type" : "string*", "description" : "the related phrases if <code xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">$phrase</code> is found in the thesaurus or the empty sequence if not." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">err:FTST0009 if <code>ft:current-lang()</code> is not supported.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">zerr:ZXQP8401 if the thesaurus data file's version is not supported by the currently running version of Zorba.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">zerr:ZXQP8402 if the thesaurus data file's endianness does not match that of the CPU on which Zorba is currently running.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">zerr:ZXQP8403 if there was an error reading the thesaurus data.</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 2, "name" : "thesaurus-lookup", "qname" : "ft:thesaurus-lookup", "signature" : "($phrase as string, $relationship as string) as string*", "description" : " Looks-up the given phrase in a thesaurus.\n", "summary" : "<p> Looks-up the given phrase in a thesaurus.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "phrase", "type" : "string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The phrase to look up.</div>" }, { "name" : "relationship", "type" : "string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The relationship the results are to have to <code>$phrase</code>.</div>" } ], "returns" : { "type" : "string*", "description" : "the related phrases if <code xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">$phrase</code> is found in the thesaurus or the empty sequence if not." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">err:FTST0018 if <code>$uri</code> refers to a thesaurus that is not found in the statically known thesauri.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">err:FTST0009 if <code>$lang</code> is not supported.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">zerr:ZOSE0001 if the thesaurus data file could not be found.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">zerr:ZOSE0002 if the thesaurus data file is not a plain file.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">zerr:ZXQP8401 if the thesaurus data file's version is not supported by the currently running version of Zorba.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">zerr:ZXQP8402 if the thesaurus data file's endianness does not match that of the CPU on which Zorba is currently running.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">zerr:ZXQP8403 if there was an error reading the thesaurus data file.</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 4, "name" : "thesaurus-lookup", "qname" : "ft:thesaurus-lookup", "signature" : "($phrase as string, $relationship as string, $level-least as integer, $level-most as integer) as string*", "description" : " Looks-up the given phrase in a thesaurus.\n", "summary" : "<p> Looks-up the given phrase in a thesaurus.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "phrase", "type" : "string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The phrase to look up.</div>" }, { "name" : "relationship", "type" : "string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The relationship the results are to have to <code>$phrase</code>.</div>" }, { "name" : "level-least", "type" : "integer", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The minimum number of levels within the thesaurus to be traversed.</div>" }, { "name" : "level-most", "type" : "integer", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The maximum number of levels within the thesaurus to be traversed.</div>" } ], "returns" : { "type" : "string*", "description" : "the related phrases if <code xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">$phrase</code> is found in the thesaurus or the empty sequence if not." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">err:FOCA0003 if either <code>$level-least</code> or <code>$level-most</code> is either negative or too large.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">err:FTST0018 if <code>$uri</code> refers to a thesaurus that is not found in the statically known thesauri.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">err:FTST0009 if <code>$lang</code> is not supported.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">zerr:ZOSE0001 if the thesaurus data file could not be found.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">zerr:ZOSE0002 if the thesaurus data file is not a plain file.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">zerr:ZXQP8401 if the thesaurus data file's version is not supported by the currently running version of Zorba.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">zerr:ZXQP8402 if the thesaurus data file's endianness does not match that of the CPU on which Zorba is currently running.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">zerr:ZXQP8403 if there was an error reading the thesaurus data file.</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 1, "name" : "tokenize", "qname" : "ft:tokenize", "signature" : "($string as string) as string*", "description" : " Tokenizes the given string.\n", "summary" : "<p> Tokenizes the given string.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "string", "type" : "string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The string to tokenize. The string's language is assumed to be the one returned by <code>ft:current-lang()</code>.</div>" } ], "returns" : { "type" : "string*", "description" : "a (possibly empty) sequence of tokens." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">err:FTST0009 if <code>ft:current-lang()</code> is not supported.</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 2, "name" : "tokenize", "qname" : "ft:tokenize", "signature" : "($string as string, $lang as language) as string*", "description" : " Tokenizes the given string.\n", "summary" : "<p> Tokenizes the given string.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "string", "type" : "string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The string to tokenize.</div>" }, { "name" : "lang", "type" : "language", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The language of <code>$string</code>.</div>" } ], "returns" : { "type" : "string*", "description" : "a (possibly empty) sequence of tokens." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">err:FTST0009 if <code>$lang</code> is not supported.</xqdoc:error>" ] } ], "variables" : [ { "name" : "ft:lang-da", "type" : "language", "description" : " Predeclared constant for the Danish language.\n" }, { "name" : "ft:lang-de", "type" : "language", "description" : " Predeclared constant for the German language.\n" }, { "name" : "ft:lang-en", "type" : "language", "description" : " Predeclared constant for the English language.\n" }, { "name" : "ft:lang-es", "type" : "language", "description" : " Predeclared constant for the Spanish language.\n" }, { "name" : "ft:lang-fi", "type" : "language", "description" : " Predeclared constant for the Finnish language.\n" }, { "name" : "ft:lang-fr", "type" : "language", "description" : " Predeclared constant for the French language.\n" }, { "name" : "ft:lang-hu", "type" : "language", "description" : " Predeclared constant for the Hungarian language.\n" }, { "name" : "ft:lang-it", "type" : "language", "description" : " Predeclared constant for the Italian language.\n" }, { "name" : "ft:lang-nl", "type" : "language", "description" : " Predeclared constant for the Dutch language.\n" }, { "name" : "ft:lang-no", "type" : "language", "description" : " Predeclared constant for the Norwegian language.\n" }, { "name" : "ft:lang-pt", "type" : "language", "description" : " Predeclared constant for the Portuguese language.\n" }, { "name" : "ft:lang-ro", "type" : "language", "description" : " Predeclared constant for the Romanian language.\n" }, { "name" : "ft:lang-ru", "type" : "language", "description" : " Predeclared constant for the Russian language.\n" }, { "name" : "ft:lang-sv", "type" : "language", "description" : " Predeclared constant for the Swedish language.\n" }, { "name" : "ft:lang-tr", "type" : "language", "description" : " Predeclared constant for the Turkish language.\n" } ] }, "http://xbrl.io/modules/bizql/report-schemas" : { "ns" : "http://xbrl.io/modules/bizql/report-schemas", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">This module provides functions for storing, retrieving, and modifying\n report schemas. Report schemas can be used in BizQL queries. They can be\n instantiated generating a business report.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Report schemas are nothing else than a user-defined component. However,\n one of their hypercubes (the default hypercube), as well as two of their\n networks (presentation, concept-map) are special. A report schema is identified\n with an RID (Report schema ID).</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">With this module, you can retrieve facts from an archive according to a report\n schema. You can retrieve them as a (2D) fact table, or populate the presentation\n network of the report schema with them. The concept map network will be used\n by default to map the report schema concepts to reported concepts.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Report schemas are stored in the collection reportschemas in\n the project's MongoDB database.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">A report schema needs to be a syntactically valid JSound document.</p>\n", "sees" : [  ], "authors" : [ "<xqdoc:author xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Charles Hoffman</xqdoc:author>", "<xqdoc:author xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Matthias Brantner</xqdoc:author>", "<xqdoc:author xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Dennis Knochenwefel</xqdoc:author>", "<xqdoc:author xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Ghislain Fourny</xqdoc:author>" ], "version" : null, "encoding" : "utf-8", "namespaces" : [ { "uri" : "", "prefix" : "an" }, { "uri" : "http://xbrl.io/modules/bizql/hypercubes", "prefix" : "hypercubes" }, { "uri" : "http://xbrl.io/modules/bizql/networks", "prefix" : "networks" }, { "uri" : "http://xbrl.io/modules/bizql/report-schemas", "prefix" : "report-schemas" }, { "uri" : "http://zorba.io/options/versioning", "prefix" : "ver" } ], "functions" : [ { "isDocumented" : true, "arity" : 1, "name" : "add", "qname" : "report-schemas:add", "signature" : "($report-schema as object()) as empty-sequence()", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Adds the given report schema to the database.</p>\n", "summary" : "<p>  Adds the given report schema to the database.</p>", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "report-schema", "type" : "object()", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the report schema to add.</div>" } ], "returns" : { "type" : "empty-sequence()", "description" : "the empty sequence." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">report-schemas:INVALID-SCHEMA if the given report schema object does not contain a name field</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">report-schemas:EXISTS if a report schema with the given name already exists</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 1, "name" : "delete", "qname" : "report-schemas:delete", "signature" : "($report-schema-or-id as item()) as empty-sequence()", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Deletes a report schema from the database.</p>\n", "summary" : "<p>  Deletes a report schema from the database.</p>", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "report-schema-or-id", "type" : "item()", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the report schema to delete or its RID.</div>" } ], "returns" : { "type" : "empty-sequence()", "description" : "the empty sequence." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">report-schemas:DOES-NOT-EXIST if no report schema with the given RID exists.</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 2, "name" : "fact-table", "qname" : "report-schemas:fact-table", "signature" : "($schema as item(), $archives as item()*) as array()", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Retrieves all facts from the supplied archives, that are relevant to the\n supplied schema, and populates them with the default dimension values\n when missing.</p>\n", "summary" : "<p>  Retrieves all facts from the supplied archives, that are relevant to the\n supplied schema, and populates them with the default dimension values\n when missing.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "schema", "type" : "item()", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> a schema or its RID,</div>" }, { "name" : "archives", "type" : "item()", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> a sequence of archives or their AIDs.</div>" } ], "returns" : { "type" : "array()", "description" : "a sequence of facts with populated dimension values." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 3, "name" : "fact-table", "qname" : "report-schemas:fact-table", "signature" : "($schema as item(), $archives as item()*, $options as object()?) as array()", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Retrieves all facts from the supplied archives, that are relevant to the\n supplied schema, and populates them with the default dimension values\n when missing.</p>\n", "summary" : "<p>  Retrieves all facts from the supplied archives, that are relevant to the\n supplied schema, and populates them with the default dimension values\n when missing.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "schema", "type" : "item()", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> a schema or its RID,</div>" }, { "name" : "archives", "type" : "item()", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> a sequence of archives or their AIDs or $report-schemas:ALL_ARCHIVES for no filtering.</div>" }, { "name" : "options", "type" : "object()", "occurrence" : "?", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> <a href=\"facts#standard_options\">standard fact retrieving options</a>.</div>" } ], "returns" : { "type" : "array()", "description" : "a sequence of facts with populated dimension values." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 2, "name" : "facts", "qname" : "report-schemas:facts", "signature" : "($schema as item(), $archives as item()*) as object()*", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Retrieves all facts from the supplied archives, that are relevant to the\n supplied schema, and populates them with the default dimension values\n when missing.</p>\n", "summary" : "<p>  Retrieves all facts from the supplied archives, that are relevant to the\n supplied schema, and populates them with the default dimension values\n when missing.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "schema", "type" : "item()", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> a report schema or its RID,</div>" }, { "name" : "archives", "type" : "item()", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> a sequence of archives or their AIDs or $report-schemas:ALL_ARCHIVES for no filtering.</div>" } ], "returns" : { "type" : "object()*", "description" : "a sequence of facts with populated dimension values." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 3, "name" : "facts", "qname" : "report-schemas:facts", "signature" : "($schema as item(), $archives as item()*, $options as object()?) as object()*", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Retrieves all facts from the supplied archives, that are relevant to the\n supplied schema, and populates them with the default dimension values\n when missing.</p>\n", "summary" : "<p>  Retrieves all facts from the supplied archives, that are relevant to the\n supplied schema, and populates them with the default dimension values\n when missing.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "schema", "type" : "item()", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> a report schema or its RID,</div>" }, { "name" : "archives", "type" : "item()", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> a sequence of archives or their AIDs or $report-schemas:ALL_ARCHIVES for no filtering.</div>" }, { "name" : "options", "type" : "object()", "occurrence" : "?", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> <a href=\"facts#standard_options\">standard fact retrieving options</a>.</div>" } ], "returns" : { "type" : "object()*", "description" : "a sequence of facts with populated dimension values." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 2, "name" : "populate-with-facts", "qname" : "report-schemas:populate-with-facts", "signature" : "($schema as item(), $archives as item()*) as object()*", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Populates a concept-tree network with all facts from the supplied archives,\n that are relevant to the\n supplied schema. Default dimension values are added to the facts\n when missing.</p>\n", "summary" : "<p>  Populates a concept-tree network with all facts from the supplied archives,\n that are relevant to the\n supplied schema.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "schema", "type" : "item()", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">.</div>" }, { "name" : "archives", "type" : "item()", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> a sequence of archives or their AIDs or $report-schemas:ALL_ARCHIVES for no filtering.</div>" } ], "returns" : { "type" : "object()*", "description" : "a sequence of facts with populated dimension values." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 3, "name" : "populate-with-facts", "qname" : "report-schemas:populate-with-facts", "signature" : "($schema as item(), $archives as item()*, $options as object()?) as object()*", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Populates a concept-tree network with all facts from the supplied archives,\n that are relevant to the\n supplied schema. Default dimension values are added to the facts\n when missing.</p>\n", "summary" : "<p>  Populates a concept-tree network with all facts from the supplied archives,\n that are relevant to the\n supplied schema.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "schema", "type" : "item()", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">.</div>" }, { "name" : "archives", "type" : "item()", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> a sequence of archives or their AIDs or $report-schemas:ALL_ARCHIVES for no filtering.</div>" }, { "name" : "options", "type" : "object()", "occurrence" : "?", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> <a href=\"facts#standard_options\">standard fact retrieving options</a>.</div>" } ], "returns" : { "type" : "object()*", "description" : "a sequence of facts with populated dimension values." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 0, "name" : "report-schemas", "qname" : "report-schemas:report-schemas", "signature" : "() as object()*", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Retrieves all report schemas.</p>\n", "summary" : "<p>  Retrieves all report schemas.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [  ], "returns" : { "type" : "object()*", "description" : "all report schemas." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "report-schemas", "qname" : "report-schemas:report-schemas", "signature" : "($report-schemas-or-ids as item()*) as object()*", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Return the report schemas with the given names (RIDs).</p>\n", "summary" : "<p>  Return the report schemas with the given names (RIDs).</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "report-schemas-or-ids", "type" : "item()", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the ids of the report schemas (RIDs) or the report schemas themselves.</div>" } ], "returns" : { "type" : "object()*", "description" : "the report schemas with the given RIDs or the empty sequence if no report schema was found or the input is an empty sequence." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "rid", "qname" : "report-schemas:rid", "signature" : "($report-schema-or-id as item()) as atomic", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Converts the input to a normalized report schema identifier (RID). The input\n can be either a pure RID, or a report schema object which contains an RID.</p>\n", "summary" : "<p>  Converts the input to a normalized report schema identifier (RID).</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "report-schema-or-id", "type" : "item()", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> a report schema identifier (RID)) or a report schema object.</div>" } ], "returns" : { "type" : "atomic", "description" : "the normalized RID." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">report-schemas:INVALID_PARAMETER if the RID or report schema is not valid</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 1, "name" : "update", "qname" : "report-schemas:update", "signature" : "($report-schema as object()) as empty-sequence()", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Updates a report schema.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Replaces a report schema in the database with the given schema.\n The schema to be replaced is identified by the value of the _id\n field of the given schema (RID).</p>\n", "summary" : "<p>  Updates a report schema.</p>", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "report-schema", "type" : "object()", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the new report schema</div>" } ], "returns" : { "type" : "empty-sequence()", "description" : "the empty sequence" }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">report-schemas:DOES-NOT-EXIST if a report schema with the given name does not exist.</xqdoc:error>" ] } ], "variables" : [ { "name" : "report-schemas:col", "type" : "string", "description" : " Name of the collection the report schemas are stored in.\n" }, { "name" : "report-schemas:ALL_ARCHIVES", "type" : "boolean", "description" : " Joker for all archives.\n" } ] }, "http://zorba.io/modules/store/static/integrity-constraints/ddl" : { "ns" : "http://zorba.io/modules/store/static/integrity-constraints/ddl", "description" : " This module defines a set of functions to manage integrity constraints\n that are declared in the prolog of a module.\n For example, it provides functions to activate or deactivate integrity\n constraints.\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n This module is part of\n <a xmlns:xqdoc=\"http://www.xqdoc.org/1.0\" href=\"../zorba/xqddf.html\">Zorba's XQuery Data Definition Facility</a>.\n All the integrity constraints managed by this module have to be pre-declared\n in the prolog of a module.\n Please refer to the\n <a xmlns:xqdoc=\"http://www.xqdoc.org/1.0\" href=\"../zorba/data_lifecycle.html\">general documentation</a>\n for more information and examples.\n", "sees" : [ "<xqdoc:see xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"><a href=\"../zorba/data_lifecycle.html\">Data Lifecycle</a></xqdoc:see>", "<xqdoc:see xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"><a href=\"../zorba/xqddf.html\">XQuery Data Definition Facility</a></xqdoc:see>", "<xqdoc:see xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">http://zorba.io/modules/store/static/collections/dml</xqdoc:see>", "<xqdoc:see xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">http://zorba.io/modules/store/static/collections/ddl</xqdoc:see>", "<xqdoc:see xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">http://zorba.io/modules/store/static/indexes/ddl</xqdoc:see>", "<xqdoc:see xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">http://zorba.io/modules/store/static/indexes/dml</xqdoc:see>", "<xqdoc:see xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">http://zorba.io/modules/store/static/integrity-constraints/dml</xqdoc:see>", "<xqdoc:see xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">http://zorba.io/errors</xqdoc:see>" ], "authors" : [ "<xqdoc:author xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Nicolae Brinza, Matthias Brantner, David Graf, Till Westmann, Markos Zaharioudakis</xqdoc:author>" ], "version" : null, "encoding" : "utf-8", "namespaces" : [ { "uri" : "http://zorba.io/modules/store/static/integrity-constraints/ddl", "prefix" : "icddl" }, { "uri" : "http://zorba.io/options/versioning", "prefix" : "ver" }, { "uri" : "http://zorba.io/errors", "prefix" : "zerr" } ], "functions" : [ { "isDocumented" : true, "arity" : 1, "name" : "activate", "qname" : "icddl:activate", "signature" : "($name as xs:QName) external", "description" : " Activates an integrity constraint in the dynamic context.\n", "summary" : "<p> Activates an integrity constraint in the dynamic context.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : true, "parameters" : [ { "name" : "name", "type" : "xs:QName", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The name of the integrity constraint to activate.</div>" } ], "returns" : { "type" : null, "description" : "An empty XDM instance and a pending update list that consists of a <code xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">upd:activateIntegrityConstraint($name)</code> update primitive." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">zerr:ZDDY0031 if the integrity constraint does not exist in the static context.</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 0, "name" : "activated-integrity-constraints", "qname" : "icddl:activated-integrity-constraints", "signature" : "() as xs:QName* external", "description" : " Gets the integrity constraints that are activated, if any.\n", "summary" : "<p> Gets the integrity constraints that are activated, if any.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [  ], "returns" : { "type" : "xs:QName*", "description" : "A sequence of QNames, one for each activated integrity constraint, or an empty sequence if none." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "deactivate", "qname" : "icddl:deactivate", "signature" : "($name as xs:QName) external", "description" : " Deactivates the integrity constraint.\n", "summary" : "<p> Deactivates the integrity constraint.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : true, "parameters" : [ { "name" : "name", "type" : "xs:QName", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The name of the integrity constraint to deactivate.</div>" } ], "returns" : { "type" : null, "description" : "An empty XDM instance and a pending update list that, once applied, deactivates the integrity constraint." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">zerr:ZDDY0032 if the integrity constraint was not declared in the the static context.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">zerr:ZDDY0032 if the integrity constraints is not activated.</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 0, "name" : "declared-integrity-constraints", "qname" : "icddl:declared-integrity-constraints", "signature" : "() as xs:QName* external", "description" : " Gets the sequence of QNames representing the integrity constraints that have\n been declared in the prolog of the static context.\n", "summary" : "<p> Gets the sequence of QNames representing the integrity constraints that have\n been declared in the prolog of the static context.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [  ], "returns" : { "type" : "xs:QName*", "description" : "A sequence of QNames, one for each created integrity constraints, or an emtpy sequence if none." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "is-activated-integrity-constraint", "qname" : "icddl:is-activated-integrity-constraint", "signature" : "($name as xs:QName) as xs:boolean external", "description" : " Gets whether an integrity constraints is activated.\n", "summary" : "<p> Gets whether an integrity constraints is activated.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "name", "type" : "xs:QName", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The name of the constraint to check.</div>" } ], "returns" : { "type" : "xs:boolean", "description" : "true if the integrity constraint is activated; false otherwise." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "is-declared-integrity-constraint", "qname" : "icddl:is-declared-integrity-constraint", "signature" : "($name as xs:QName) as xs:boolean external", "description" : " Gets whether an integrity constraint is declared in the prolog of the static\n context.\n", "summary" : "<p> Gets whether an integrity constraint is declared in the prolog of the static\n context.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "name", "type" : "xs:QName", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The name of the constraint to check.</div>" } ], "returns" : { "type" : "xs:boolean", "description" : "true if the constraint is declared; false otherwise." }, "errors" : [  ] } ], "variables" : [  ] }, "http://www.functx.com" : { "ns" : "http://www.functx.com", "description" : " --------------------------------\n The FunctX XQuery Function Library\n --------------------------------\n Copyright (C) 2007 Datypic\n This library is free software; you can redistribute it and/or\n modify it under the terms of the GNU Lesser General Public\n License as published by the Free Software Foundation; either\n version 2.1 of the License.\n This library is distributed in the hope that it will be useful,\n but WITHOUT ANY WARRANTY; without even the implied warranty of\n MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU\n Lesser General Public License for more details.\n You should have received a copy of the GNU Lesser General Public\n License along with this library; if not, write to the Free Software\n Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA\n For more information on the FunctX XQuery library, contact contrib@functx.com.\n", "sees" : [ "<xqdoc:see xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">http://www.xqueryfunctions.com</xqdoc:see>" ], "authors" : [  ], "version" : null, "encoding" : "utf-8", "namespaces" : [ { "uri" : "http://www.functx.com", "prefix" : "functx" } ], "functions" : [ { "isDocumented" : true, "arity" : 3, "name" : "add-attributes", "qname" : "functx:add-attributes", "signature" : "($elements as element(*)*, $attrNames as xs:QName*, $attrValues as xs:anyAtomicType*) as element(*)?", "description" : " Adds attributes to XML elements\n", "summary" : "<p> Adds attributes to XML elements\n</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "elements", "type" : "element(*)", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the element(s) to which you wish to add the attribute</div>" }, { "name" : "attrNames", "type" : "xs:QName", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the name(s) of the attribute(s) to add</div>" }, { "name" : "attrValues", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the value(s) of the attribute(s) to add</div>" } ], "returns" : { "type" : "element(*)?", "description" : "" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 2, "name" : "add-months", "qname" : "functx:add-months", "signature" : "($date as xs:anyAtomicType?, $months as xs:integer) as xs:date?", "description" : " Adds months to a date\n", "summary" : "<p> Adds months to a date\n</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "date", "type" : "xs:anyAtomicType", "occurrence" : "?", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the date</div>" }, { "name" : "months", "type" : "xs:integer", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the number of months to add</div>" } ], "returns" : { "type" : "xs:date?", "description" : "" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 3, "name" : "add-or-update-attributes", "qname" : "functx:add-or-update-attributes", "signature" : "($elements as element(*)*, $attrNames as xs:QName*, $attrValues as xs:anyAtomicType*) as element(*)?", "description" : " Adds attributes to XML elements\n", "summary" : "<p> Adds attributes to XML elements\n</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "elements", "type" : "element(*)", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the element(s) to which you wish to add the attribute</div>" }, { "name" : "attrNames", "type" : "xs:QName", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the name(s) of the attribute(s) to add</div>" }, { "name" : "attrValues", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the value(s) of the attribute(s) to add</div>" } ], "returns" : { "type" : "element(*)?", "description" : "" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "all-whitespace", "qname" : "functx:all-whitespace", "signature" : "($arg as xs:string?) as xs:boolean", "description" : " Whether a value is all whitespace or a zero-length string\n", "summary" : "<p> Whether a value is all whitespace or a zero-length string\n</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "arg", "type" : "xs:string", "occurrence" : "?", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the string (or node) to test</div>" } ], "returns" : { "type" : "xs:boolean", "description" : "" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "are-distinct-values", "qname" : "functx:are-distinct-values", "signature" : "($seq as xs:anyAtomicType*) as xs:boolean", "description" : " Whether all the values in a sequence are distinct\n", "summary" : "<p> Whether all the values in a sequence are distinct\n</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "seq", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequence of values</div>" } ], "returns" : { "type" : "xs:boolean", "description" : "" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "atomic-type", "qname" : "functx:atomic-type", "signature" : "($values as xs:anyAtomicType*) as xs:string*", "description" : " The built-in type of an atomic value\n", "summary" : "<p> The built-in type of an atomic value\n</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "values", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the value(s) whose type you want to determine</div>" } ], "returns" : { "type" : "xs:string*", "description" : "" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 2, "name" : "avg-empty-is-zero", "qname" : "functx:avg-empty-is-zero", "signature" : "($values as xs:anyAtomicType*, $allNodes as node()*) as xs:double", "description" : " The average, counting \"empty\" values as zero\n", "summary" : "<p> The average, counting \"empty\" values as zero\n</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "values", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the values to be averaged</div>" }, { "name" : "allNodes", "type" : "node()", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequence of all nodes to find the average over</div>" } ], "returns" : { "type" : "xs:double", "description" : "" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 3, "name" : "between-exclusive", "qname" : "functx:between-exclusive", "signature" : "($value as xs:anyAtomicType?, $minValue as xs:anyAtomicType, $maxValue as xs:anyAtomicType) as xs:boolean", "description" : " Whether a value is between two provided values\n", "summary" : "<p> Whether a value is between two provided values\n</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "value", "type" : "xs:anyAtomicType", "occurrence" : "?", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the value to be tested</div>" }, { "name" : "minValue", "type" : "xs:anyAtomicType", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the minimum value</div>" }, { "name" : "maxValue", "type" : "xs:anyAtomicType", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the maximum value</div>" } ], "returns" : { "type" : "xs:boolean", "description" : "" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 3, "name" : "between-inclusive", "qname" : "functx:between-inclusive", "signature" : "($value as xs:anyAtomicType?, $minValue as xs:anyAtomicType, $maxValue as xs:anyAtomicType) as xs:boolean", "description" : " Whether a value is between two provided values, or equal to one of them\n", "summary" : "<p> Whether a value is between two provided values, or equal to one of them\n</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "value", "type" : "xs:anyAtomicType", "occurrence" : "?", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the value to be tested</div>" }, { "name" : "minValue", "type" : "xs:anyAtomicType", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the minimum value</div>" }, { "name" : "maxValue", "type" : "xs:anyAtomicType", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the maximum value</div>" } ], "returns" : { "type" : "xs:boolean", "description" : "" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 2, "name" : "camel-case-to-words", "qname" : "functx:camel-case-to-words", "signature" : "($arg as xs:string?, $delim as xs:string) as xs:string", "description" : " Turns a camelCase string into space-separated words\n", "summary" : "<p> Turns a camelCase string into space-separated words\n</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "arg", "type" : "xs:string", "occurrence" : "?", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the string to modify</div>" }, { "name" : "delim", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the delimiter for the words (e.g. a space)</div>" } ], "returns" : { "type" : "xs:string", "description" : "" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "capitalize-first", "qname" : "functx:capitalize-first", "signature" : "($arg as xs:string?) as xs:string?", "description" : " Capitalizes the first character of a string\n", "summary" : "<p> Capitalizes the first character of a string\n</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "arg", "type" : "xs:string", "occurrence" : "?", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the word or phrase to capitalize</div>" } ], "returns" : { "type" : "xs:string?", "description" : "" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 3, "name" : "change-element-names-deep", "qname" : "functx:change-element-names-deep", "signature" : "($nodes as node()*, $oldNames as xs:QName*, $newNames as xs:QName*) as node()*", "description" : " Changes the names of elements in an XML fragment\n", "summary" : "<p> Changes the names of elements in an XML fragment\n</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "nodes", "type" : "node()", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the element(s) to change</div>" }, { "name" : "oldNames", "type" : "xs:QName", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequence of names to change from</div>" }, { "name" : "newNames", "type" : "xs:QName", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequence of names to change to</div>" } ], "returns" : { "type" : "node()*", "description" : "" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 3, "name" : "change-element-ns-deep", "qname" : "functx:change-element-ns-deep", "signature" : "($nodes as node()*, $newns as xs:string, $prefix as xs:string) as node()*", "description" : " Changes the namespace of XML elements and its descendants\n", "summary" : "<p> Changes the namespace of XML elements and its descendants\n</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "nodes", "type" : "node()", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the nodes to change</div>" }, { "name" : "newns", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the new namespace</div>" }, { "name" : "prefix", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the prefix to use for the new namespace</div>" } ], "returns" : { "type" : "node()*", "description" : "" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 3, "name" : "change-element-ns", "qname" : "functx:change-element-ns", "signature" : "($elements as element(*)*, $newns as xs:string, $prefix as xs:string) as element(*)?", "description" : " Changes the namespace of XML elements\n", "summary" : "<p> Changes the namespace of XML elements\n</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "elements", "type" : "element(*)", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the elements to change</div>" }, { "name" : "newns", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the new namespace</div>" }, { "name" : "prefix", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the prefix to use for the new namespace</div>" } ], "returns" : { "type" : "element(*)?", "description" : "" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "chars", "qname" : "functx:chars", "signature" : "($arg as xs:string?) as xs:string*", "description" : " Converts a string to a sequence of characters\n", "summary" : "<p> Converts a string to a sequence of characters\n</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "arg", "type" : "xs:string", "occurrence" : "?", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the string to split</div>" } ], "returns" : { "type" : "xs:string*", "description" : "" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 2, "name" : "contains-any-of", "qname" : "functx:contains-any-of", "signature" : "($arg as xs:string?, $searchStrings as xs:string*) as xs:boolean", "description" : " Whether a string contains any of a sequence of strings\n", "summary" : "<p> Whether a string contains any of a sequence of strings\n</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "arg", "type" : "xs:string", "occurrence" : "?", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the string to test</div>" }, { "name" : "searchStrings", "type" : "xs:string", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the strings to look for</div>" } ], "returns" : { "type" : "xs:boolean", "description" : "" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 2, "name" : "contains-case-insensitive", "qname" : "functx:contains-case-insensitive", "signature" : "($arg as xs:string?, $substring as xs:string) as xs:boolean?", "description" : " Whether one string contains another, without regard to case\n", "summary" : "<p> Whether one string contains another, without regard to case\n</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "arg", "type" : "xs:string", "occurrence" : "?", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the string to search</div>" }, { "name" : "substring", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the substring to find</div>" } ], "returns" : { "type" : "xs:boolean?", "description" : "" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 2, "name" : "contains-word", "qname" : "functx:contains-word", "signature" : "($arg as xs:string?, $word as xs:string) as xs:boolean", "description" : " Whether one string contains another, as a separate word\n", "summary" : "<p> Whether one string contains another, as a separate word\n</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "arg", "type" : "xs:string", "occurrence" : "?", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the string to search</div>" }, { "name" : "word", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the word to find</div>" } ], "returns" : { "type" : "xs:boolean", "description" : "" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 2, "name" : "copy-attributes", "qname" : "functx:copy-attributes", "signature" : "($copyTo as element(*), $copyFrom as element(*)) as element(*)", "description" : " Copies attributes from one element to another\n", "summary" : "<p> Copies attributes from one element to another\n</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "copyTo", "type" : "element(*)", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the element to copy attributes to</div>" }, { "name" : "copyFrom", "type" : "element(*)", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the element to copy attributes from</div>" } ], "returns" : { "type" : "element(*)", "description" : "" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 3, "name" : "date", "qname" : "functx:date", "signature" : "($year as xs:anyAtomicType, $month as xs:anyAtomicType, $day as xs:anyAtomicType) as xs:date", "description" : " Construct a date from a year, month and day\n", "summary" : "<p> Construct a date from a year, month and day\n</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "year", "type" : "xs:anyAtomicType", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the year</div>" }, { "name" : "month", "type" : "xs:anyAtomicType", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the month</div>" }, { "name" : "day", "type" : "xs:anyAtomicType", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the day</div>" } ], "returns" : { "type" : "xs:date", "description" : "" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 6, "name" : "dateTime", "qname" : "functx:dateTime", "signature" : "($year as xs:anyAtomicType, $month as xs:anyAtomicType, $day as xs:anyAtomicType, $hour as xs:anyAtomicType, $minute as xs:anyAtomicType, $second as xs:anyAtomicType) as xs:dateTime", "description" : " Construct a date/time from individual components\n", "summary" : "<p> Construct a date/time from individual components\n</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "year", "type" : "xs:anyAtomicType", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the year</div>" }, { "name" : "month", "type" : "xs:anyAtomicType", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the month</div>" }, { "name" : "day", "type" : "xs:anyAtomicType", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the day</div>" }, { "name" : "hour", "type" : "xs:anyAtomicType", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the hour</div>" }, { "name" : "minute", "type" : "xs:anyAtomicType", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the minute</div>" }, { "name" : "second", "type" : "xs:anyAtomicType", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the second</div>" } ], "returns" : { "type" : "xs:dateTime", "description" : "" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "day-in-year", "qname" : "functx:day-in-year", "signature" : "($date as xs:anyAtomicType?) as xs:integer?", "description" : " The day of the year (a number between 1 and 366)\n", "summary" : "<p> The day of the year (a number between 1 and 366)\n</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "date", "type" : "xs:anyAtomicType", "occurrence" : "?", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the date</div>" } ], "returns" : { "type" : "xs:integer?", "description" : "" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "day-of-week-abbrev-en", "qname" : "functx:day-of-week-abbrev-en", "signature" : "($date as xs:anyAtomicType?) as xs:string?", "description" : " The abbreviated day of the week, from a date, in English\n", "summary" : "<p> The abbreviated day of the week, from a date, in English\n</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "date", "type" : "xs:anyAtomicType", "occurrence" : "?", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the date</div>" } ], "returns" : { "type" : "xs:string?", "description" : "" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "day-of-week-name-en", "qname" : "functx:day-of-week-name-en", "signature" : "($date as xs:anyAtomicType?) as xs:string?", "description" : " The name of the day of the week, from a date, in English\n", "summary" : "<p> The name of the day of the week, from a date, in English\n</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "date", "type" : "xs:anyAtomicType", "occurrence" : "?", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the date</div>" } ], "returns" : { "type" : "xs:string?", "description" : "" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "day-of-week", "qname" : "functx:day-of-week", "signature" : "($date as xs:anyAtomicType?) as xs:integer?", "description" : " The day of the week, from a date\n", "summary" : "<p> The day of the week, from a date\n</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "date", "type" : "xs:anyAtomicType", "occurrence" : "?", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the date</div>" } ], "returns" : { "type" : "xs:integer?", "description" : "" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 4, "name" : "dayTimeDuration", "qname" : "functx:dayTimeDuration", "signature" : "($days as xs:decimal?, $hours as xs:decimal?, $minutes as xs:decimal?, $seconds as xs:decimal?) as xs:dayTimeDuration", "description" : " Construct a dayTimeDuration from a number of days, hours, etc.\n", "summary" : "<p> Construct a dayTimeDuration from a number of days, hours, etc.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "days", "type" : "xs:decimal", "occurrence" : "?", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the number of days</div>" }, { "name" : "hours", "type" : "xs:decimal", "occurrence" : "?", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the number of hours</div>" }, { "name" : "minutes", "type" : "xs:decimal", "occurrence" : "?", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the number of minutes</div>" }, { "name" : "seconds", "type" : "xs:decimal", "occurrence" : "?", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the number of seconds</div>" } ], "returns" : { "type" : "xs:dayTimeDuration", "description" : "" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "days-in-month", "qname" : "functx:days-in-month", "signature" : "($date as xs:anyAtomicType?) as xs:integer?", "description" : " Number of days in the month\n", "summary" : "<p> Number of days in the month\n</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "date", "type" : "xs:anyAtomicType", "occurrence" : "?", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the date</div>" } ], "returns" : { "type" : "xs:integer?", "description" : "" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "depth-of-node", "qname" : "functx:depth-of-node", "signature" : "($node as node()?) as xs:integer", "description" : " The depth (level) of a node in an XML tree\n", "summary" : "<p> The depth (level) of a node in an XML tree\n</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "node", "type" : "node()", "occurrence" : "?", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the node to check</div>" } ], "returns" : { "type" : "xs:integer", "description" : "" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "distinct-attribute-names", "qname" : "functx:distinct-attribute-names", "signature" : "($nodes as node()*) as xs:string*", "description" : " The distinct names of all attributes in an XML fragment\n", "summary" : "<p> The distinct names of all attributes in an XML fragment\n</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "nodes", "type" : "node()", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the root to start from</div>" } ], "returns" : { "type" : "xs:string*", "description" : "" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "distinct-deep", "qname" : "functx:distinct-deep", "signature" : "($nodes as node()*) as node()*", "description" : " The XML nodes with distinct values, taking into account attributes and descendants\n", "summary" : "<p> The XML nodes with distinct values, taking into account attributes and descendants\n</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "nodes", "type" : "node()", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequence of nodes to test</div>" } ], "returns" : { "type" : "node()*", "description" : "" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "distinct-element-names", "qname" : "functx:distinct-element-names", "signature" : "($nodes as node()*) as xs:string*", "description" : " The distinct names of all elements in an XML fragment\n", "summary" : "<p> The distinct names of all elements in an XML fragment\n</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "nodes", "type" : "node()", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the root(s) to start from</div>" } ], "returns" : { "type" : "xs:string*", "description" : "" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "distinct-element-paths", "qname" : "functx:distinct-element-paths", "signature" : "($nodes as node()*) as xs:string*", "description" : " The distinct paths of all descendant elements in an XML fragment\n", "summary" : "<p> The distinct paths of all descendant elements in an XML fragment\n</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "nodes", "type" : "node()", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the root(s) to start from</div>" } ], "returns" : { "type" : "xs:string*", "description" : "" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "distinct-nodes", "qname" : "functx:distinct-nodes", "signature" : "($nodes as node()*) as node()*", "description" : " The distinct XML nodes in a sequence (by node identity)\n", "summary" : "<p> The distinct XML nodes in a sequence (by node identity)\n</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "nodes", "type" : "node()", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the node sequence</div>" } ], "returns" : { "type" : "node()*", "description" : "" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "duration-from-timezone", "qname" : "functx:duration-from-timezone", "signature" : "($timezone as xs:string) as xs:dayTimeDuration", "description" : " Converts a timezone like \"-05:00\" or \"Z\" into xs:dayTimeDuration\n", "summary" : "<p> Converts a timezone like \"-05:00\" or \"Z\" into xs:dayTimeDuration\n</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "timezone", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the time zone, in (+|-)HH:MM format</div>" } ], "returns" : { "type" : "xs:dayTimeDuration", "description" : "" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 2, "name" : "dynamic-path", "qname" : "functx:dynamic-path", "signature" : "($parent as node(), $path as xs:string) as item()*", "description" : " Dynamically evaluates a simple XPath path\n", "summary" : "<p> Dynamically evaluates a simple XPath path\n</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "parent", "type" : "node()", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the root to start from</div>" }, { "name" : "path", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the path expression</div>" } ], "returns" : { "type" : "item()*", "description" : "" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "escape-for-regex", "qname" : "functx:escape-for-regex", "signature" : "($arg as xs:string?) as xs:string", "description" : " Escapes regex special characters\n", "summary" : "<p> Escapes regex special characters\n</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "arg", "type" : "xs:string", "occurrence" : "?", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the string to escape</div>" } ], "returns" : { "type" : "xs:string", "description" : "" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 2, "name" : "exclusive-or", "qname" : "functx:exclusive-or", "signature" : "($arg1 as xs:boolean?, $arg2 as xs:boolean?) as xs:boolean?", "description" : " Whether one (and only one) of two boolean values is true\n", "summary" : "<p> Whether one (and only one) of two boolean values is true\n</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "arg1", "type" : "xs:boolean", "occurrence" : "?", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the first boolean value</div>" }, { "name" : "arg2", "type" : "xs:boolean", "occurrence" : "?", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the second boolean value</div>" } ], "returns" : { "type" : "xs:boolean?", "description" : "" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "first-day-of-month", "qname" : "functx:first-day-of-month", "signature" : "($date as xs:anyAtomicType?) as xs:date?", "description" : " The first day of the month of a date\n", "summary" : "<p> The first day of the month of a date\n</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "date", "type" : "xs:anyAtomicType", "occurrence" : "?", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the date</div>" } ], "returns" : { "type" : "xs:date?", "description" : "" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "first-day-of-year", "qname" : "functx:first-day-of-year", "signature" : "($date as xs:anyAtomicType?) as xs:date?", "description" : " The first day of the year of a date\n", "summary" : "<p> The first day of the year of a date\n</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "date", "type" : "xs:anyAtomicType", "occurrence" : "?", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the date</div>" } ], "returns" : { "type" : "xs:date?", "description" : "" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "first-node", "qname" : "functx:first-node", "signature" : "($nodes as node()*) as node()?", "description" : " The XML node in a sequence that appears first in document order\n", "summary" : "<p> The XML node in a sequence that appears first in document order\n</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "nodes", "type" : "node()", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequence of nodes</div>" } ], "returns" : { "type" : "node()?", "description" : "" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 2, "name" : "follows-not-descendant", "qname" : "functx:follows-not-descendant", "signature" : "($a as node()?, $b as node()?) as xs:boolean", "description" : " Whether an XML node follows another without being its descendant\n", "summary" : "<p> Whether an XML node follows another without being its descendant\n</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "a", "type" : "node()", "occurrence" : "?", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the first node</div>" }, { "name" : "b", "type" : "node()", "occurrence" : "?", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the second node</div>" } ], "returns" : { "type" : "xs:boolean", "description" : "" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "format-as-title-en", "qname" : "functx:format-as-title-en", "signature" : "($titles as xs:string*) as xs:string*", "description" : " Moves title words like \"the\" and \"a\" to the end of strings\n", "summary" : "<p> Moves title words like \"the\" and \"a\" to the end of strings\n</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "titles", "type" : "xs:string", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the titles to format</div>" } ], "returns" : { "type" : "xs:string*", "description" : "" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "fragment-from-uri", "qname" : "functx:fragment-from-uri", "signature" : "($uri as xs:string?) as xs:string?", "description" : " Returns the fragment from a URI\n", "summary" : "<p> Returns the fragment from a URI\n</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "uri", "type" : "xs:string", "occurrence" : "?", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the URI</div>" } ], "returns" : { "type" : "xs:string?", "description" : "" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 2, "name" : "get-matches-and-non-matches", "qname" : "functx:get-matches-and-non-matches", "signature" : "($string as xs:string?, $regex as xs:string) as element(*)*", "description" : " Splits a string into matching and non-matching regions\n", "summary" : "<p> Splits a string into matching and non-matching regions\n</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "string", "type" : "xs:string", "occurrence" : "?", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the string to split</div>" }, { "name" : "regex", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the pattern</div>" } ], "returns" : { "type" : "element(*)*", "description" : "" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 2, "name" : "get-matches", "qname" : "functx:get-matches", "signature" : "($string as xs:string?, $regex as xs:string) as xs:string*", "description" : " Return the matching regions of a string\n", "summary" : "<p> Return the matching regions of a string\n</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "string", "type" : "xs:string", "occurrence" : "?", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the string to split</div>" }, { "name" : "regex", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the pattern</div>" } ], "returns" : { "type" : "xs:string*", "description" : "" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "has-element-only-content", "qname" : "functx:has-element-only-content", "signature" : "($element as element(*)) as xs:boolean", "description" : " Whether an element has element-only content\n", "summary" : "<p> Whether an element has element-only content\n</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "element", "type" : "element(*)", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the XML element to test</div>" } ], "returns" : { "type" : "xs:boolean", "description" : "" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "has-empty-content", "qname" : "functx:has-empty-content", "signature" : "($element as element(*)) as xs:boolean", "description" : " Whether an element has empty content\n", "summary" : "<p> Whether an element has empty content\n</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "element", "type" : "element(*)", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the XML element to test</div>" } ], "returns" : { "type" : "xs:boolean", "description" : "" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "has-mixed-content", "qname" : "functx:has-mixed-content", "signature" : "($element as element(*)) as xs:boolean", "description" : " Whether an element has mixed content\n", "summary" : "<p> Whether an element has mixed content\n</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "element", "type" : "element(*)", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the XML element to test</div>" } ], "returns" : { "type" : "xs:boolean", "description" : "" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "has-simple-content", "qname" : "functx:has-simple-content", "signature" : "($element as element(*)) as xs:boolean", "description" : " Whether an element has simple content\n", "summary" : "<p> Whether an element has simple content\n</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "element", "type" : "element(*)", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the XML element to test</div>" } ], "returns" : { "type" : "xs:boolean", "description" : "" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "id-from-element", "qname" : "functx:id-from-element", "signature" : "($element as element(*)?) as xs:string?", "description" : " Gets the ID of an XML element\n", "summary" : "<p> Gets the ID of an XML element\n</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "element", "type" : "element(*)", "occurrence" : "?", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the element</div>" } ], "returns" : { "type" : "xs:string?", "description" : "" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 2, "name" : "id-untyped", "qname" : "functx:id-untyped", "signature" : "($node as node()*, $id as xs:anyAtomicType) as element(*)*", "description" : " Gets XML element(s) that have an attribute with a particular value\n", "summary" : "<p> Gets XML element(s) that have an attribute with a particular value\n</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "node", "type" : "node()", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the root node(s) to start from</div>" }, { "name" : "id", "type" : "xs:anyAtomicType", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the \"id\" to find</div>" } ], "returns" : { "type" : "element(*)*", "description" : "" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 2, "name" : "if-absent", "qname" : "functx:if-absent", "signature" : "($arg as item()*, $value as item()*) as item()*", "description" : " The first argument if it is not empty, otherwise the second argument\n", "summary" : "<p> The first argument if it is not empty, otherwise the second argument\n</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "arg", "type" : "item()", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the item(s) that may be absent</div>" }, { "name" : "value", "type" : "item()", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the item(s) to use if the item is absent</div>" } ], "returns" : { "type" : "item()*", "description" : "" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 2, "name" : "if-empty", "qname" : "functx:if-empty", "signature" : "($arg as item()?, $value as item()*) as item()*", "description" : " The first argument if it is not blank, otherwise the second argument\n", "summary" : "<p> The first argument if it is not blank, otherwise the second argument\n</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "arg", "type" : "item()", "occurrence" : "?", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the node that may be empty</div>" }, { "name" : "value", "type" : "item()", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the item(s) to use if the node is empty</div>" } ], "returns" : { "type" : "item()*", "description" : "" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 2, "name" : "index-of-deep-equal-node", "qname" : "functx:index-of-deep-equal-node", "signature" : "($nodes as node()*, $nodeToFind as node()) as xs:integer*", "description" : " The position of a node in a sequence, based on contents and attributes\n", "summary" : "<p> The position of a node in a sequence, based on contents and attributes\n</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "nodes", "type" : "node()", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the node sequence</div>" }, { "name" : "nodeToFind", "type" : "node()", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the node to find in the sequence</div>" } ], "returns" : { "type" : "xs:integer*", "description" : "" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 2, "name" : "index-of-match-first", "qname" : "functx:index-of-match-first", "signature" : "($arg as xs:string?, $pattern as xs:string) as xs:integer?", "description" : " The first position of a matching substring\n", "summary" : "<p> The first position of a matching substring\n</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "arg", "type" : "xs:string", "occurrence" : "?", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the string</div>" }, { "name" : "pattern", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the pattern to match</div>" } ], "returns" : { "type" : "xs:integer?", "description" : "" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 2, "name" : "index-of-node", "qname" : "functx:index-of-node", "signature" : "($nodes as node()*, $nodeToFind as node()) as xs:integer*", "description" : " The position of a node in a sequence, based on node identity\n", "summary" : "<p> The position of a node in a sequence, based on node identity\n</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "nodes", "type" : "node()", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the node sequence</div>" }, { "name" : "nodeToFind", "type" : "node()", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the node to find in the sequence</div>" } ], "returns" : { "type" : "xs:integer*", "description" : "" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 2, "name" : "index-of-string-first", "qname" : "functx:index-of-string-first", "signature" : "($arg as xs:string?, $substring as xs:string) as xs:integer?", "description" : " The first position of a substring\n", "summary" : "<p> The first position of a substring\n</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "arg", "type" : "xs:string", "occurrence" : "?", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the string</div>" }, { "name" : "substring", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the substring to find</div>" } ], "returns" : { "type" : "xs:integer?", "description" : "" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 2, "name" : "index-of-string-last", "qname" : "functx:index-of-string-last", "signature" : "($arg as xs:string?, $substring as xs:string) as xs:integer?", "description" : " The last position of a substring\n", "summary" : "<p> The last position of a substring\n</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "arg", "type" : "xs:string", "occurrence" : "?", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the string</div>" }, { "name" : "substring", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the substring to find</div>" } ], "returns" : { "type" : "xs:integer?", "description" : "" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 2, "name" : "index-of-string", "qname" : "functx:index-of-string", "signature" : "($arg as xs:string?, $substring as xs:string) as xs:integer*", "description" : " The position(s) of a substring\n", "summary" : "<p> The position(s) of a substring\n</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "arg", "type" : "xs:string", "occurrence" : "?", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the string</div>" }, { "name" : "substring", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the substring to find</div>" } ], "returns" : { "type" : "xs:integer*", "description" : "" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 3, "name" : "insert-string", "qname" : "functx:insert-string", "signature" : "($originalString as xs:string?, $stringToInsert as xs:string?, $pos as xs:integer) as xs:string", "description" : " Inserts a string at a specified position\n", "summary" : "<p> Inserts a string at a specified position\n</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "originalString", "type" : "xs:string", "occurrence" : "?", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the original string to insert into</div>" }, { "name" : "stringToInsert", "type" : "xs:string", "occurrence" : "?", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the string to insert</div>" }, { "name" : "pos", "type" : "xs:integer", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the position</div>" } ], "returns" : { "type" : "xs:string", "description" : "" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "is-a-number", "qname" : "functx:is-a-number", "signature" : "($value as xs:anyAtomicType?) as xs:boolean", "description" : " Whether a value is numeric\n", "summary" : "<p> Whether a value is numeric\n</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "value", "type" : "xs:anyAtomicType", "occurrence" : "?", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the value to test</div>" } ], "returns" : { "type" : "xs:boolean", "description" : "" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "is-absolute-uri", "qname" : "functx:is-absolute-uri", "signature" : "($uri as xs:string?) as xs:boolean", "description" : " Whether a URI is absolute\n", "summary" : "<p> Whether a URI is absolute\n</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "uri", "type" : "xs:string", "occurrence" : "?", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the URI to test</div>" } ], "returns" : { "type" : "xs:boolean", "description" : "" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 2, "name" : "is-ancestor", "qname" : "functx:is-ancestor", "signature" : "($node1 as node(), $node2 as node()) as xs:boolean", "description" : " Whether an XML node is an ancestor of another node\n", "summary" : "<p> Whether an XML node is an ancestor of another node\n</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "node1", "type" : "node()", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the first node</div>" }, { "name" : "node2", "type" : "node()", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the second node</div>" } ], "returns" : { "type" : "xs:boolean", "description" : "" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 2, "name" : "is-descendant", "qname" : "functx:is-descendant", "signature" : "($node1 as node(), $node2 as node()) as xs:boolean", "description" : " Whether an XML node is a descendant of another node\n", "summary" : "<p> Whether an XML node is a descendant of another node\n</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "node1", "type" : "node()", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the first node</div>" }, { "name" : "node2", "type" : "node()", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the second node</div>" } ], "returns" : { "type" : "xs:boolean", "description" : "" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "is-leap-year", "qname" : "functx:is-leap-year", "signature" : "($date as xs:anyAtomicType?) as xs:boolean", "description" : " Whether a date falls in a leap year\n", "summary" : "<p> Whether a date falls in a leap year\n</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "date", "type" : "xs:anyAtomicType", "occurrence" : "?", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the date or year</div>" } ], "returns" : { "type" : "xs:boolean", "description" : "" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 2, "name" : "is-node-among-descendants-deep-equal", "qname" : "functx:is-node-among-descendants-deep-equal", "signature" : "($node as node()?, $seq as node()*) as xs:boolean", "description" : " Whether an XML node is among the descendants of a sequence, based on contents and attributes\n", "summary" : "<p> Whether an XML node is among the descendants of a sequence, based on contents and attributes\n</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "node", "type" : "node()", "occurrence" : "?", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the node to test</div>" }, { "name" : "seq", "type" : "node()", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequence of nodes to search</div>" } ], "returns" : { "type" : "xs:boolean", "description" : "" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 2, "name" : "is-node-among-descendants", "qname" : "functx:is-node-among-descendants", "signature" : "($node as node()?, $seq as node()*) as xs:boolean", "description" : " Whether an XML node is among the descendants of a sequence, based on node identity\n", "summary" : "<p> Whether an XML node is among the descendants of a sequence, based on node identity\n</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "node", "type" : "node()", "occurrence" : "?", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the node to test</div>" }, { "name" : "seq", "type" : "node()", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequence of nodes to search</div>" } ], "returns" : { "type" : "xs:boolean", "description" : "" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 2, "name" : "is-node-in-sequence-deep-equal", "qname" : "functx:is-node-in-sequence-deep-equal", "signature" : "($node as node()?, $seq as node()*) as xs:boolean", "description" : " Whether an XML node is in a sequence, based on contents and attributes\n", "summary" : "<p> Whether an XML node is in a sequence, based on contents and attributes\n</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "node", "type" : "node()", "occurrence" : "?", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the node to test</div>" }, { "name" : "seq", "type" : "node()", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequence of nodes to search</div>" } ], "returns" : { "type" : "xs:boolean", "description" : "" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 2, "name" : "is-node-in-sequence", "qname" : "functx:is-node-in-sequence", "signature" : "($node as node()?, $seq as node()*) as xs:boolean", "description" : " Whether an XML node is in a sequence, based on node identity\n", "summary" : "<p> Whether an XML node is in a sequence, based on node identity\n</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "node", "type" : "node()", "occurrence" : "?", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the node to test</div>" }, { "name" : "seq", "type" : "node()", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequence of nodes to search</div>" } ], "returns" : { "type" : "xs:boolean", "description" : "" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 2, "name" : "is-value-in-sequence", "qname" : "functx:is-value-in-sequence", "signature" : "($value as xs:anyAtomicType?, $seq as xs:anyAtomicType*) as xs:boolean", "description" : " Whether an atomic value appears in a sequence\n", "summary" : "<p> Whether an atomic value appears in a sequence\n</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "value", "type" : "xs:anyAtomicType", "occurrence" : "?", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the atomic value to test</div>" }, { "name" : "seq", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequence of values to search</div>" } ], "returns" : { "type" : "xs:boolean", "description" : "" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "last-day-of-month", "qname" : "functx:last-day-of-month", "signature" : "($date as xs:anyAtomicType?) as xs:date?", "description" : " The last day of the month of a date\n", "summary" : "<p> The last day of the month of a date\n</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "date", "type" : "xs:anyAtomicType", "occurrence" : "?", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the date</div>" } ], "returns" : { "type" : "xs:date?", "description" : "" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "last-day-of-year", "qname" : "functx:last-day-of-year", "signature" : "($date as xs:anyAtomicType?) as xs:date?", "description" : " The last day of the month of a date\n", "summary" : "<p> The last day of the month of a date\n</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "date", "type" : "xs:anyAtomicType", "occurrence" : "?", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the date</div>" } ], "returns" : { "type" : "xs:date?", "description" : "" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "last-node", "qname" : "functx:last-node", "signature" : "($nodes as node()*) as node()?", "description" : " The XML node in a sequence that is last in document order\n", "summary" : "<p> The XML node in a sequence that is last in document order\n</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "nodes", "type" : "node()", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequence of nodes</div>" } ], "returns" : { "type" : "node()?", "description" : "" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "leaf-elements", "qname" : "functx:leaf-elements", "signature" : "($root as node()?) as element(*)*", "description" : " All XML elements that don't have any child elements\n", "summary" : "<p> All XML elements that don't have any child elements\n</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "root", "type" : "node()", "occurrence" : "?", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the root</div>" } ], "returns" : { "type" : "element(*)*", "description" : "" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "left-trim", "qname" : "functx:left-trim", "signature" : "($arg as xs:string?) as xs:string", "description" : " Trims leading whitespace\n", "summary" : "<p> Trims leading whitespace\n</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "arg", "type" : "xs:string", "occurrence" : "?", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the string to trim</div>" } ], "returns" : { "type" : "xs:string", "description" : "" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "line-count", "qname" : "functx:line-count", "signature" : "($arg as xs:string?) as xs:integer", "description" : " The number of lines\n", "summary" : "<p> The number of lines\n</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "arg", "type" : "xs:string", "occurrence" : "?", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the string to test</div>" } ], "returns" : { "type" : "xs:integer", "description" : "" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "lines", "qname" : "functx:lines", "signature" : "($arg as xs:string?) as xs:string*", "description" : " Split a string into separate lines\n", "summary" : "<p> Split a string into separate lines\n</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "arg", "type" : "xs:string", "occurrence" : "?", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the string to split</div>" } ], "returns" : { "type" : "xs:string*", "description" : "" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "max-depth", "qname" : "functx:max-depth", "signature" : "($root as node()?) as xs:integer?", "description" : " The maximum depth of elements in an XML tree\n", "summary" : "<p> The maximum depth of elements in an XML tree\n</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "root", "type" : "node()", "occurrence" : "?", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the root to start from</div>" } ], "returns" : { "type" : "xs:integer?", "description" : "" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "max-determine-type", "qname" : "functx:max-determine-type", "signature" : "($seq as xs:anyAtomicType*) as xs:anyAtomicType?", "description" : " The maximum value in a sequence, figuring out its type (numeric or string)\n", "summary" : "<p> The maximum value in a sequence, figuring out its type (numeric or string)\n</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "seq", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequence of values to test</div>" } ], "returns" : { "type" : "xs:anyAtomicType?", "description" : "" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "max-line-length", "qname" : "functx:max-line-length", "signature" : "($arg as xs:string?) as xs:integer", "description" : " The maximum line length\n", "summary" : "<p> The maximum line length\n</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "arg", "type" : "xs:string", "occurrence" : "?", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the string to test</div>" } ], "returns" : { "type" : "xs:integer", "description" : "" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "max-node", "qname" : "functx:max-node", "signature" : "($nodes as node()*) as node()*", "description" : " The XML node whose typed value is the maximum\n", "summary" : "<p> The XML node whose typed value is the maximum\n</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "nodes", "type" : "node()", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequence of nodes to test</div>" } ], "returns" : { "type" : "node()*", "description" : "" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "max-string", "qname" : "functx:max-string", "signature" : "($strings as xs:anyAtomicType*) as xs:string?", "description" : " The maximum of a sequence of values, treating them like strings\n", "summary" : "<p> The maximum of a sequence of values, treating them like strings\n</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "strings", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequence of values</div>" } ], "returns" : { "type" : "xs:string?", "description" : "" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "min-determine-type", "qname" : "functx:min-determine-type", "signature" : "($seq as xs:anyAtomicType*) as xs:anyAtomicType?", "description" : " The minimum value in a sequence, figuring out its type (numeric or string)\n", "summary" : "<p> The minimum value in a sequence, figuring out its type (numeric or string)\n</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "seq", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequence of values to test</div>" } ], "returns" : { "type" : "xs:anyAtomicType?", "description" : "" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "min-node", "qname" : "functx:min-node", "signature" : "($nodes as node()*) as node()*", "description" : " The XML node whose typed value is the minimum\n", "summary" : "<p> The XML node whose typed value is the minimum\n</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "nodes", "type" : "node()", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequence of nodes to test</div>" } ], "returns" : { "type" : "node()*", "description" : "" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "min-non-empty-string", "qname" : "functx:min-non-empty-string", "signature" : "($strings as xs:string*) as xs:string?", "description" : " The minimum of a sequence of strings, ignoring \"empty\" values\n", "summary" : "<p> The minimum of a sequence of strings, ignoring \"empty\" values\n</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "strings", "type" : "xs:string", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequence of strings to search</div>" } ], "returns" : { "type" : "xs:string?", "description" : "" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "min-string", "qname" : "functx:min-string", "signature" : "($strings as xs:anyAtomicType*) as xs:string?", "description" : " The minimum of a sequence of values, treating them like strings\n", "summary" : "<p> The minimum of a sequence of values, treating them like strings\n</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "strings", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequence of strings</div>" } ], "returns" : { "type" : "xs:string?", "description" : "" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "mmddyyyy-to-date", "qname" : "functx:mmddyyyy-to-date", "signature" : "($dateString as xs:string?) as xs:date?", "description" : " Converts a string with format MMDDYYYY (with any delimiters) to a date\n", "summary" : "<p> Converts a string with format MMDDYYYY (with any delimiters) to a date\n</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "dateString", "type" : "xs:string", "occurrence" : "?", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the MMDDYYYY string</div>" } ], "returns" : { "type" : "xs:date?", "description" : "" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "month-abbrev-en", "qname" : "functx:month-abbrev-en", "signature" : "($date as xs:anyAtomicType?) as xs:string?", "description" : " The month of a date as an abbreviated word (Jan, Feb, etc.)\n", "summary" : "<p> The month of a date as an abbreviated word (Jan, Feb, etc.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "date", "type" : "xs:anyAtomicType", "occurrence" : "?", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the date</div>" } ], "returns" : { "type" : "xs:string?", "description" : "" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "month-name-en", "qname" : "functx:month-name-en", "signature" : "($date as xs:anyAtomicType?) as xs:string?", "description" : " The month of a date as a word (January, February, etc.)\n", "summary" : "<p> The month of a date as a word (January, February, etc.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "date", "type" : "xs:anyAtomicType", "occurrence" : "?", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the date</div>" } ], "returns" : { "type" : "xs:string?", "description" : "" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 2, "name" : "name-test", "qname" : "functx:name-test", "signature" : "($testname as xs:string?, $names as xs:string*) as xs:boolean", "description" : " Whether a name matches a list of names or name wildcards\n", "summary" : "<p> Whether a name matches a list of names or name wildcards\n</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "testname", "type" : "xs:string", "occurrence" : "?", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the name to test</div>" }, { "name" : "names", "type" : "xs:string", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the list of names or name wildcards</div>" } ], "returns" : { "type" : "xs:boolean", "description" : "" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "namespaces-in-use", "qname" : "functx:namespaces-in-use", "signature" : "($root as node()?) as xs:anyURI*", "description" : " A list of namespaces used in element/attribute names in an XML fragment\n", "summary" : "<p> A list of namespaces used in element/attribute names in an XML fragment\n</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "root", "type" : "node()", "occurrence" : "?", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the root node to start from</div>" } ], "returns" : { "type" : "xs:anyURI*", "description" : "" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "next-day", "qname" : "functx:next-day", "signature" : "($date as xs:anyAtomicType?) as xs:date?", "description" : " The next day\n", "summary" : "<p> The next day\n</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "date", "type" : "xs:anyAtomicType", "occurrence" : "?", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the date</div>" } ], "returns" : { "type" : "xs:date?", "description" : "" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "node-kind", "qname" : "functx:node-kind", "signature" : "($nodes as node()*) as xs:string*", "description" : " The XML node kind (element, attribute, text, etc.)\n", "summary" : "<p> The XML node kind (element, attribute, text, etc.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "nodes", "type" : "node()", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the node(s) whose kind you want to determine</div>" } ], "returns" : { "type" : "xs:string*", "description" : "" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "non-distinct-values", "qname" : "functx:non-distinct-values", "signature" : "($seq as xs:anyAtomicType*) as xs:anyAtomicType*", "description" : " Returns any values that appear more than once in a sequence\n", "summary" : "<p> Returns any values that appear more than once in a sequence\n</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "seq", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequence of values</div>" } ], "returns" : { "type" : "xs:anyAtomicType*", "description" : "" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 2, "name" : "number-of-matches", "qname" : "functx:number-of-matches", "signature" : "($arg as xs:string?, $pattern as xs:string) as xs:integer", "description" : " The number of regions that match a pattern\n", "summary" : "<p> The number of regions that match a pattern\n</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "arg", "type" : "xs:string", "occurrence" : "?", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the string to test</div>" }, { "name" : "pattern", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the regular expression</div>" } ], "returns" : { "type" : "xs:integer", "description" : "" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "ordinal-number-en", "qname" : "functx:ordinal-number-en", "signature" : "($num as xs:integer?) as xs:string", "description" : " Reformats a number as an ordinal number, e.g. 1st, 2nd, 3rd.\n", "summary" : "<p> Reformats a number as an ordinal number, e.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "num", "type" : "xs:integer", "occurrence" : "?", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the number</div>" } ], "returns" : { "type" : "xs:string", "description" : "" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 2, "name" : "pad-integer-to-length", "qname" : "functx:pad-integer-to-length", "signature" : "($integerToPad as xs:anyAtomicType?, $length as xs:integer) as xs:string", "description" : " Pads an integer to a desired length by adding leading zeros\n", "summary" : "<p> Pads an integer to a desired length by adding leading zeros\n</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "integerToPad", "type" : "xs:anyAtomicType", "occurrence" : "?", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the integer to pad</div>" }, { "name" : "length", "type" : "xs:integer", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the desired length</div>" } ], "returns" : { "type" : "xs:string", "description" : "" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 3, "name" : "pad-string-to-length", "qname" : "functx:pad-string-to-length", "signature" : "($stringToPad as xs:string?, $padChar as xs:string, $length as xs:integer) as xs:string", "description" : " Pads a string to a desired length\n", "summary" : "<p> Pads a string to a desired length\n</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "stringToPad", "type" : "xs:string", "occurrence" : "?", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the string to pad</div>" }, { "name" : "padChar", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the character(s) to use as padding</div>" }, { "name" : "length", "type" : "xs:integer", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the desired length</div>" } ], "returns" : { "type" : "xs:string", "description" : "" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "path-to-node-with-pos", "qname" : "functx:path-to-node-with-pos", "signature" : "($node as node()?) as xs:string", "description" : " A unique path to an XML node (or sequence of nodes)\n", "summary" : "<p> A unique path to an XML node (or sequence of nodes)\n</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "node", "type" : "node()", "occurrence" : "?", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the node sequence</div>" } ], "returns" : { "type" : "xs:string", "description" : "" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "path-to-node", "qname" : "functx:path-to-node", "signature" : "($nodes as node()*) as xs:string*", "description" : " A path to an XML node (or sequence of nodes)\n", "summary" : "<p> A path to an XML node (or sequence of nodes)\n</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "nodes", "type" : "node()", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the node sequence</div>" } ], "returns" : { "type" : "xs:string*", "description" : "" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 2, "name" : "precedes-not-ancestor", "qname" : "functx:precedes-not-ancestor", "signature" : "($a as node()?, $b as node()?) as xs:boolean", "description" : " Whether an XML node precedes another without being its ancestor\n", "summary" : "<p> Whether an XML node precedes another without being its ancestor\n</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "a", "type" : "node()", "occurrence" : "?", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the first node</div>" }, { "name" : "b", "type" : "node()", "occurrence" : "?", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the second node</div>" } ], "returns" : { "type" : "xs:boolean", "description" : "" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "previous-day", "qname" : "functx:previous-day", "signature" : "($date as xs:anyAtomicType?) as xs:date?", "description" : " The previous day\n", "summary" : "<p> The previous day\n</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "date", "type" : "xs:anyAtomicType", "occurrence" : "?", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the date</div>" } ], "returns" : { "type" : "xs:date?", "description" : "" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 2, "name" : "remove-attributes-deep", "qname" : "functx:remove-attributes-deep", "signature" : "($nodes as node()*, $names as xs:string*) as node()*", "description" : " Removes attributes from an XML fragment, based on name\n", "summary" : "<p> Removes attributes from an XML fragment, based on name\n</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "nodes", "type" : "node()", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the root(s) to start from</div>" }, { "name" : "names", "type" : "xs:string", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the names of the attributes to remove, or * for all attributes</div>" } ], "returns" : { "type" : "node()*", "description" : "" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 2, "name" : "remove-attributes", "qname" : "functx:remove-attributes", "signature" : "($elements as element(*)*, $names as xs:string*) as element(*)", "description" : " Removes attributes from an XML element, based on name\n", "summary" : "<p> Removes attributes from an XML element, based on name\n</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "elements", "type" : "element(*)", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the element(s) from which to remove the attributes</div>" }, { "name" : "names", "type" : "xs:string", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the names of the attributes to remove, or * for all attributes</div>" } ], "returns" : { "type" : "element(*)", "description" : "" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 2, "name" : "remove-elements-deep", "qname" : "functx:remove-elements-deep", "signature" : "($nodes as node()*, $names as xs:string*) as node()*", "description" : " Removes descendant elements from an XML node, based on name\n", "summary" : "<p> Removes descendant elements from an XML node, based on name\n</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "nodes", "type" : "node()", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> root(s) to start from</div>" }, { "name" : "names", "type" : "xs:string", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the names of the elements to remove</div>" } ], "returns" : { "type" : "node()*", "description" : "" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 2, "name" : "remove-elements-not-contents", "qname" : "functx:remove-elements-not-contents", "signature" : "($nodes as node()*, $names as xs:string*) as node()*", "description" : " Removes descendant XML elements but keeps their content\n", "summary" : "<p> Removes descendant XML elements but keeps their content\n</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "nodes", "type" : "node()", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the root(s) to start from</div>" }, { "name" : "names", "type" : "xs:string", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the names of the elements to remove</div>" } ], "returns" : { "type" : "node()*", "description" : "" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 2, "name" : "remove-elements", "qname" : "functx:remove-elements", "signature" : "($elements as element(*)*, $names as xs:string*) as element(*)*", "description" : " Removes child elements from an XML node, based on name\n", "summary" : "<p> Removes child elements from an XML node, based on name\n</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "elements", "type" : "element(*)", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the element(s) from which you wish to remove the children</div>" }, { "name" : "names", "type" : "xs:string", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the names of the child elements to remove</div>" } ], "returns" : { "type" : "element(*)*", "description" : "" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 2, "name" : "repeat-string", "qname" : "functx:repeat-string", "signature" : "($stringToRepeat as xs:string?, $count as xs:integer) as xs:string", "description" : " Repeats a string a given number of times\n", "summary" : "<p> Repeats a string a given number of times\n</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "stringToRepeat", "type" : "xs:string", "occurrence" : "?", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the string to repeat</div>" }, { "name" : "count", "type" : "xs:integer", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the desired number of copies</div>" } ], "returns" : { "type" : "xs:string", "description" : "" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 3, "name" : "replace-beginning", "qname" : "functx:replace-beginning", "signature" : "($arg as xs:string?, $pattern as xs:string, $replacement as xs:string) as xs:string", "description" : " Replaces the beginning of a string, up to a matched pattern\n", "summary" : "<p> Replaces the beginning of a string, up to a matched pattern\n</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "arg", "type" : "xs:string", "occurrence" : "?", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the entire string to change</div>" }, { "name" : "pattern", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the pattern of characters to replace up to</div>" }, { "name" : "replacement", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the replacement string</div>" } ], "returns" : { "type" : "xs:string", "description" : "" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 2, "name" : "replace-element-values", "qname" : "functx:replace-element-values", "signature" : "($elements as element(*)*, $values as xs:anyAtomicType*) as element(*)*", "description" : " Updates the content of one or more elements\n", "summary" : "<p> Updates the content of one or more elements\n</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "elements", "type" : "element(*)", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the elements whose content you wish to replace</div>" }, { "name" : "values", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the replacement values</div>" } ], "returns" : { "type" : "element(*)*", "description" : "" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 3, "name" : "replace-first", "qname" : "functx:replace-first", "signature" : "($arg as xs:string?, $pattern as xs:string, $replacement as xs:string) as xs:string", "description" : " Replaces the first match of a pattern\n", "summary" : "<p> Replaces the first match of a pattern\n</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "arg", "type" : "xs:string", "occurrence" : "?", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the entire string to change</div>" }, { "name" : "pattern", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the pattern of characters to replace</div>" }, { "name" : "replacement", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the replacement string</div>" } ], "returns" : { "type" : "xs:string", "description" : "" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 3, "name" : "replace-multi", "qname" : "functx:replace-multi", "signature" : "($arg as xs:string?, $changeFrom as xs:string*, $changeTo as xs:string*) as xs:string?", "description" : " Performs multiple replacements, using pairs of replace parameters\n", "summary" : "<p> Performs multiple replacements, using pairs of replace parameters\n</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "arg", "type" : "xs:string", "occurrence" : "?", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the string to manipulate</div>" }, { "name" : "changeFrom", "type" : "xs:string", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequence of strings or patterns to change from</div>" }, { "name" : "changeTo", "type" : "xs:string", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequence of strings to change to</div>" } ], "returns" : { "type" : "xs:string?", "description" : "" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "reverse-string", "qname" : "functx:reverse-string", "signature" : "($arg as xs:string?) as xs:string", "description" : " Reverses the order of characters\n", "summary" : "<p> Reverses the order of characters\n</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "arg", "type" : "xs:string", "occurrence" : "?", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the string to reverse</div>" } ], "returns" : { "type" : "xs:string", "description" : "" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "right-trim", "qname" : "functx:right-trim", "signature" : "($arg as xs:string?) as xs:string", "description" : " Trims trailing whitespace\n", "summary" : "<p> Trims trailing whitespace\n</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "arg", "type" : "xs:string", "occurrence" : "?", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the string to trim</div>" } ], "returns" : { "type" : "xs:string", "description" : "" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "scheme-from-uri", "qname" : "functx:scheme-from-uri", "signature" : "($uri as xs:string?) as xs:string?", "description" : " Returns the scheme from a URI\n", "summary" : "<p> Returns the scheme from a URI\n</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "uri", "type" : "xs:string", "occurrence" : "?", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the URI</div>" } ], "returns" : { "type" : "xs:string?", "description" : "" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 2, "name" : "sequence-deep-equal", "qname" : "functx:sequence-deep-equal", "signature" : "($seq1 as item()*, $seq2 as item()*) as xs:boolean", "description" : " Whether two sequences have the same XML node content and/or values\n", "summary" : "<p> Whether two sequences have the same XML node content and/or values\n</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "seq1", "type" : "item()", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the first sequence</div>" }, { "name" : "seq2", "type" : "item()", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the second sequence</div>" } ], "returns" : { "type" : "xs:boolean", "description" : "" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 2, "name" : "sequence-node-equal-any-order", "qname" : "functx:sequence-node-equal-any-order", "signature" : "($seq1 as node()*, $seq2 as node()*) as xs:boolean", "description" : " Whether two sequences contain the same XML nodes, regardless of order\n", "summary" : "<p> Whether two sequences contain the same XML nodes, regardless of order\n</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "seq1", "type" : "node()", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the first sequence of nodes</div>" }, { "name" : "seq2", "type" : "node()", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the second sequence of nodes</div>" } ], "returns" : { "type" : "xs:boolean", "description" : "" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 2, "name" : "sequence-node-equal", "qname" : "functx:sequence-node-equal", "signature" : "($seq1 as node()*, $seq2 as node()*) as xs:boolean", "description" : " Whether two sequences contain the same XML nodes, in the same order\n", "summary" : "<p> Whether two sequences contain the same XML nodes, in the same order\n</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "seq1", "type" : "node()", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the first sequence of nodes</div>" }, { "name" : "seq2", "type" : "node()", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the second sequence of nodes</div>" } ], "returns" : { "type" : "xs:boolean", "description" : "" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "sequence-type", "qname" : "functx:sequence-type", "signature" : "($items as item()*) as xs:string", "description" : " The sequence type that represents a sequence of nodes or values\n", "summary" : "<p> The sequence type that represents a sequence of nodes or values\n</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "items", "type" : "item()", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the items whose sequence type you want to determine</div>" } ], "returns" : { "type" : "xs:string", "description" : "" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "siblings-same-name", "qname" : "functx:siblings-same-name", "signature" : "($element as element(*)?) as element(*)*", "description" : " The siblings of an XML element that have the same name\n", "summary" : "<p> The siblings of an XML element that have the same name\n</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "element", "type" : "element(*)", "occurrence" : "?", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the node</div>" } ], "returns" : { "type" : "element(*)*", "description" : "" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "siblings", "qname" : "functx:siblings", "signature" : "($node as node()?) as node()*", "description" : " The siblings of an XML node\n", "summary" : "<p> The siblings of an XML node\n</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "node", "type" : "node()", "occurrence" : "?", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the node</div>" } ], "returns" : { "type" : "node()*", "description" : "" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "sort-as-numeric", "qname" : "functx:sort-as-numeric", "signature" : "($seq as item()*) as item()*", "description" : " Sorts a sequence of numeric values or nodes\n", "summary" : "<p> Sorts a sequence of numeric values or nodes\n</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "seq", "type" : "item()", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequence to sort</div>" } ], "returns" : { "type" : "item()*", "description" : "" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "sort-case-insensitive", "qname" : "functx:sort-case-insensitive", "signature" : "($seq as item()*) as item()*", "description" : " Sorts a sequence of values or nodes regardless of capitalization\n", "summary" : "<p> Sorts a sequence of values or nodes regardless of capitalization\n</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "seq", "type" : "item()", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequence to sort</div>" } ], "returns" : { "type" : "item()*", "description" : "" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "sort-document-order", "qname" : "functx:sort-document-order", "signature" : "($seq as node()*) as node()*", "description" : " Sorts a sequence of nodes in document order\n", "summary" : "<p> Sorts a sequence of nodes in document order\n</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "seq", "type" : "node()", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequence to sort</div>" } ], "returns" : { "type" : "node()*", "description" : "" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "sort", "qname" : "functx:sort", "signature" : "($seq as item()*) as item()*", "description" : " Sorts a sequence of values or nodes\n", "summary" : "<p> Sorts a sequence of values or nodes\n</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "seq", "type" : "item()", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequence to sort</div>" } ], "returns" : { "type" : "item()*", "description" : "" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 2, "name" : "substring-after-if-contains", "qname" : "functx:substring-after-if-contains", "signature" : "($arg as xs:string?, $delim as xs:string) as xs:string?", "description" : " Performs substring-after, returning the entire string if it does not contain the delimiter\n", "summary" : "<p> Performs substring-after, returning the entire string if it does not contain the delimiter\n</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "arg", "type" : "xs:string", "occurrence" : "?", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the string to substring</div>" }, { "name" : "delim", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the delimiter</div>" } ], "returns" : { "type" : "xs:string?", "description" : "" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 2, "name" : "substring-after-last-match", "qname" : "functx:substring-after-last-match", "signature" : "($arg as xs:string?, $regex as xs:string) as xs:string", "description" : " The substring after the last text that matches a regex\n", "summary" : "<p> The substring after the last text that matches a regex\n</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "arg", "type" : "xs:string", "occurrence" : "?", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the string to substring</div>" }, { "name" : "regex", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the regular expression</div>" } ], "returns" : { "type" : "xs:string", "description" : "" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 2, "name" : "substring-after-last", "qname" : "functx:substring-after-last", "signature" : "($arg as xs:string?, $delim as xs:string) as xs:string", "description" : " The substring after the last occurrence of a delimiter\n", "summary" : "<p> The substring after the last occurrence of a delimiter\n</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "arg", "type" : "xs:string", "occurrence" : "?", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the string to substring</div>" }, { "name" : "delim", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the delimiter</div>" } ], "returns" : { "type" : "xs:string", "description" : "" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 2, "name" : "substring-after-match", "qname" : "functx:substring-after-match", "signature" : "($arg as xs:string?, $regex as xs:string) as xs:string?", "description" : " The substring after the first text that matches a regex\n", "summary" : "<p> The substring after the first text that matches a regex\n</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "arg", "type" : "xs:string", "occurrence" : "?", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the string to substring</div>" }, { "name" : "regex", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the regular expression</div>" } ], "returns" : { "type" : "xs:string?", "description" : "" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 2, "name" : "substring-before-if-contains", "qname" : "functx:substring-before-if-contains", "signature" : "($arg as xs:string?, $delim as xs:string) as xs:string?", "description" : " Performs substring-before, returning the entire string if it does not contain the delimiter\n", "summary" : "<p> Performs substring-before, returning the entire string if it does not contain the delimiter\n</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "arg", "type" : "xs:string", "occurrence" : "?", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the string to substring</div>" }, { "name" : "delim", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the delimiter</div>" } ], "returns" : { "type" : "xs:string?", "description" : "" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 2, "name" : "substring-before-last-match", "qname" : "functx:substring-before-last-match", "signature" : "($arg as xs:string?, $regex as xs:string) as xs:string?", "description" : " The substring after the first text that matches a regex\n", "summary" : "<p> The substring after the first text that matches a regex\n</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "arg", "type" : "xs:string", "occurrence" : "?", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the string to substring</div>" }, { "name" : "regex", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the regular expression</div>" } ], "returns" : { "type" : "xs:string?", "description" : "" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 2, "name" : "substring-before-last", "qname" : "functx:substring-before-last", "signature" : "($arg as xs:string?, $delim as xs:string) as xs:string", "description" : " The substring before the last occurrence of a delimiter\n", "summary" : "<p> The substring before the last occurrence of a delimiter\n</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "arg", "type" : "xs:string", "occurrence" : "?", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the string to substring</div>" }, { "name" : "delim", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the delimiter</div>" } ], "returns" : { "type" : "xs:string", "description" : "" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 2, "name" : "substring-before-match", "qname" : "functx:substring-before-match", "signature" : "($arg as xs:string?, $regex as xs:string) as xs:string", "description" : " The substring before the last text that matches a regex\n", "summary" : "<p> The substring before the last text that matches a regex\n</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "arg", "type" : "xs:string", "occurrence" : "?", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the string to substring</div>" }, { "name" : "regex", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the regular expression</div>" } ], "returns" : { "type" : "xs:string", "description" : "" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 3, "name" : "time", "qname" : "functx:time", "signature" : "($hour as xs:anyAtomicType, $minute as xs:anyAtomicType, $second as xs:anyAtomicType) as xs:time", "description" : " Construct a time from an hour, minute and second\n", "summary" : "<p> Construct a time from an hour, minute and second\n</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "hour", "type" : "xs:anyAtomicType", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the hour</div>" }, { "name" : "minute", "type" : "xs:anyAtomicType", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the minute</div>" }, { "name" : "second", "type" : "xs:anyAtomicType", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the second</div>" } ], "returns" : { "type" : "xs:time", "description" : "" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "timezone-from-duration", "qname" : "functx:timezone-from-duration", "signature" : "($duration as xs:dayTimeDuration) as xs:string", "description" : " Converts an xs:dayTimeDuration into a timezone like \"-05:00\" or \"Z\"\n", "summary" : "<p> Converts an xs:dayTimeDuration into a timezone like \"-05:00\" or \"Z\"\n</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "duration", "type" : "xs:dayTimeDuration", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the duration</div>" } ], "returns" : { "type" : "xs:string", "description" : "" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "total-days-from-duration", "qname" : "functx:total-days-from-duration", "signature" : "($duration as xs:dayTimeDuration?) as xs:decimal?", "description" : " The total number of days in a dayTimeDuration\n", "summary" : "<p> The total number of days in a dayTimeDuration\n</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "duration", "type" : "xs:dayTimeDuration", "occurrence" : "?", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the duration</div>" } ], "returns" : { "type" : "xs:decimal?", "description" : "" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "total-hours-from-duration", "qname" : "functx:total-hours-from-duration", "signature" : "($duration as xs:dayTimeDuration?) as xs:decimal?", "description" : " The total number of hours in a dayTimeDuration\n", "summary" : "<p> The total number of hours in a dayTimeDuration\n</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "duration", "type" : "xs:dayTimeDuration", "occurrence" : "?", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the duration</div>" } ], "returns" : { "type" : "xs:decimal?", "description" : "" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "total-minutes-from-duration", "qname" : "functx:total-minutes-from-duration", "signature" : "($duration as xs:dayTimeDuration?) as xs:decimal?", "description" : " The total number of minutes in a dayTimeDuration\n", "summary" : "<p> The total number of minutes in a dayTimeDuration\n</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "duration", "type" : "xs:dayTimeDuration", "occurrence" : "?", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the duration</div>" } ], "returns" : { "type" : "xs:decimal?", "description" : "" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "total-months-from-duration", "qname" : "functx:total-months-from-duration", "signature" : "($duration as xs:yearMonthDuration?) as xs:decimal?", "description" : " The total number of months in a yearMonthDuration\n", "summary" : "<p> The total number of months in a yearMonthDuration\n</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "duration", "type" : "xs:yearMonthDuration", "occurrence" : "?", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the duration</div>" } ], "returns" : { "type" : "xs:decimal?", "description" : "" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "total-seconds-from-duration", "qname" : "functx:total-seconds-from-duration", "signature" : "($duration as xs:dayTimeDuration?) as xs:decimal?", "description" : " The total number of seconds in a dayTimeDuration\n", "summary" : "<p> The total number of seconds in a dayTimeDuration\n</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "duration", "type" : "xs:dayTimeDuration", "occurrence" : "?", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the duration</div>" } ], "returns" : { "type" : "xs:decimal?", "description" : "" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "total-years-from-duration", "qname" : "functx:total-years-from-duration", "signature" : "($duration as xs:yearMonthDuration?) as xs:decimal?", "description" : " The total number of years in a yearMonthDuration\n", "summary" : "<p> The total number of years in a yearMonthDuration\n</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "duration", "type" : "xs:yearMonthDuration", "occurrence" : "?", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the duration</div>" } ], "returns" : { "type" : "xs:decimal?", "description" : "" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "trim", "qname" : "functx:trim", "signature" : "($arg as xs:string?) as xs:string", "description" : " Trims leading and trailing whitespace\n", "summary" : "<p> Trims leading and trailing whitespace\n</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "arg", "type" : "xs:string", "occurrence" : "?", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the string to trim</div>" } ], "returns" : { "type" : "xs:string", "description" : "" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 3, "name" : "update-attributes", "qname" : "functx:update-attributes", "signature" : "($elements as element(*)*, $attrNames as xs:QName*, $attrValues as xs:anyAtomicType*) as element(*)?", "description" : " Updates the attribute value of an XML element\n", "summary" : "<p> Updates the attribute value of an XML element\n</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "elements", "type" : "element(*)", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the element(s) for which you wish to update the attribute</div>" }, { "name" : "attrNames", "type" : "xs:QName", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the name(s) of the attribute(s) to add</div>" }, { "name" : "attrValues", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the value(s) of the attribute(s) to add</div>" } ], "returns" : { "type" : "element(*)?", "description" : "" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 2, "name" : "value-except", "qname" : "functx:value-except", "signature" : "($arg1 as xs:anyAtomicType*, $arg2 as xs:anyAtomicType*) as xs:anyAtomicType*", "description" : " The values in one sequence that aren't in another sequence\n", "summary" : "<p> The values in one sequence that aren't in another sequence\n</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "arg1", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the first sequence</div>" }, { "name" : "arg2", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the second sequence</div>" } ], "returns" : { "type" : "xs:anyAtomicType*", "description" : "" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 2, "name" : "value-intersect", "qname" : "functx:value-intersect", "signature" : "($arg1 as xs:anyAtomicType*, $arg2 as xs:anyAtomicType*) as xs:anyAtomicType*", "description" : " The intersection of two sequences of values\n", "summary" : "<p> The intersection of two sequences of values\n</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "arg1", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the first sequence</div>" }, { "name" : "arg2", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the second sequence</div>" } ], "returns" : { "type" : "xs:anyAtomicType*", "description" : "" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 2, "name" : "value-union", "qname" : "functx:value-union", "signature" : "($arg1 as xs:anyAtomicType*, $arg2 as xs:anyAtomicType*) as xs:anyAtomicType*", "description" : " The union of two sequences of values\n", "summary" : "<p> The union of two sequences of values\n</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "arg1", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the first sequence</div>" }, { "name" : "arg2", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the second sequence</div>" } ], "returns" : { "type" : "xs:anyAtomicType*", "description" : "" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "word-count", "qname" : "functx:word-count", "signature" : "($arg as xs:string?) as xs:integer", "description" : " The number of words\n", "summary" : "<p> The number of words\n</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "arg", "type" : "xs:string", "occurrence" : "?", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the string to measure</div>" } ], "returns" : { "type" : "xs:integer", "description" : "" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "words-to-camel-case", "qname" : "functx:words-to-camel-case", "signature" : "($arg as xs:string?) as xs:string", "description" : " Turns a string of words into camelCase\n", "summary" : "<p> Turns a string of words into camelCase\n</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "arg", "type" : "xs:string", "occurrence" : "?", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the string to modify</div>" } ], "returns" : { "type" : "xs:string", "description" : "" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 2, "name" : "wrap-values-in-elements", "qname" : "functx:wrap-values-in-elements", "signature" : "($values as xs:anyAtomicType*, $elementName as xs:QName) as element(*)*", "description" : " Wraps a sequence of atomic values in XML elements\n", "summary" : "<p> Wraps a sequence of atomic values in XML elements\n</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "values", "type" : "xs:anyAtomicType", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the values to wrap in elements</div>" }, { "name" : "elementName", "type" : "xs:QName", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the name of the elements to construct</div>" } ], "returns" : { "type" : "element(*)*", "description" : "" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 2, "name" : "yearMonthDuration", "qname" : "functx:yearMonthDuration", "signature" : "($years as xs:decimal?, $months as xs:integer?) as xs:yearMonthDuration", "description" : " Construct a yearMonthDuration from a number of years and months\n", "summary" : "<p> Construct a yearMonthDuration from a number of years and months\n</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "years", "type" : "xs:decimal", "occurrence" : "?", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the number of years</div>" }, { "name" : "months", "type" : "xs:integer", "occurrence" : "?", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the number of months</div>" } ], "returns" : { "type" : "xs:yearMonthDuration", "description" : "" }, "errors" : [  ] } ], "variables" : [  ] }, "http://zorba.io/modules/zorba-query" : { "ns" : "http://zorba.io/modules/zorba-query", "description" : " This module contains functions to compile and evaluate queries\n written in either JSONiq or XQuery. Also, it contains function that\n allow to parameterize the static or dynamic evaluation phase.\n", "sees" : [  ], "authors" : [ "<xqdoc:author xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Juan Zacarias</xqdoc:author>" ], "version" : null, "encoding" : "utf-8", "namespaces" : [ { "uri" : "http://zorba.io/annotations", "prefix" : "an" }, { "uri" : "http://zorba.io/features", "prefix" : "f" }, { "uri" : "http://zorba.io/options/features", "prefix" : "op" }, { "uri" : "http://zorba.io/options/versioning", "prefix" : "ver" }, { "uri" : "http://zorba.io/modules/zorba-query", "prefix" : "zq" } ], "functions" : [ { "isDocumented" : true, "arity" : 2, "name" : "bind-context-item", "qname" : "zq:bind-context-item", "signature" : "($query-key as xs:anyURI, $dot as item()) as empty-sequence() external", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">This function binds the context-item of the prepared query\n identified by the given key to the $dot argument.</p>\n", "summary" : "<p>  This function binds the context-item of the prepared query\n identified by the given key to the $dot argument.</p>", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "query-key", "type" : "xs:anyURI", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the identifier for a compiled query</div>" }, { "name" : "dot", "type" : "item()", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the context item to bind</div>" } ], "returns" : { "type" : "empty-sequence()", "description" : "the function has side effects and returns the empty sequence." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">zq:NO_QUERY_MATCH if no query with the given identifier was prepared.</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 3, "name" : "bind-variable", "qname" : "zq:bind-variable", "signature" : "($query-key as xs:anyURI, $var as xs:QName, $value as item()*) as empty-sequence() external", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">This function binds the variable with name $name of\n the prepared query identified by $query-key to the given sequence.</p>\n", "summary" : "<p>  This function binds the variable with name $name of\n the prepared query identified by $query-key to the given sequence.</p>", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "query-key", "type" : "xs:anyURI", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the identifier for a compiled query</div>" }, { "name" : "var", "type" : "xs:QName", "occurrence" : null, "description" : "" }, { "name" : "value", "type" : "item()", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the sequence to which the external variable $name should be bound</div>" } ], "returns" : { "type" : "empty-sequence()", "description" : "the function has side effects and returns the empty sequence." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">zq:NO_QUERY_MATCH if no query with the given identifier was prepared.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">zq:UNDECLARED_VARIABLE if the given variable is not declared in the query.</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 1, "name" : "delete-query", "qname" : "zq:delete-query", "signature" : "($query-key as xs:anyURI) as empty-sequence() external", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Deletes the prepared query associated with the given identifier.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">After the query is deleted, the corresponding identifier should\n not be used as argument to any of the functions of this module.</p>\n", "summary" : "<p>  Deletes the prepared query associated with the given identifier.</p>", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "query-key", "type" : "xs:anyURI", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the identifier for a compiled query</div>" } ], "returns" : { "type" : "empty-sequence()", "description" : "the function has side effects and returns the empty sequence." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">zq:NO_QUERY_MATCH if no query with the given identifier was prepared.</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 1, "name" : "evaluate-sequential", "qname" : "zq:evaluate-sequential", "signature" : "($query-key as xs:string) as item()* external", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Evaluates the given prepared query and returns the result\n of the evaluation. The query must be sequential.</p>\n", "summary" : "<p>  Evaluates the given prepared query and returns the result\n of the evaluation.</p>", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "query-key", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the identifier for a compiled query</div>" } ], "returns" : { "type" : "item()*", "description" : "the result of evaluating the query." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">zq:NO_QUERY_MATCH if no query with the given identifier was prepared.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">zq:QUERY_NOT_SEQUENTIAL if the query is not sequential.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">zq:QUERY_IS_UPDATING if the query is an updating query.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">any dynamic error that is raised by evaluating the given query.</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 1, "name" : "evaluate-updating", "qname" : "zq:evaluate-updating", "signature" : "($query-key as xs:anyURI) external", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Evaluates the given prepared query and applies the updates\n computed by this query. The query must be an updating query.</p>\n", "summary" : "<p>  Evaluates the given prepared query and applies the updates\n computed by this query.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : true, "parameters" : [ { "name" : "query-key", "type" : "xs:anyURI", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the identifier for a compiled query</div>" } ], "returns" : { "type" : null, "description" : "the function has side effects because it applies the updates of the query. It returns the empty sequence." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">zq:NO_QUERY_MATCH if no query with the given identifier was prepared.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">zq:QUERY_NOT_UPDATING if the query is not an updating query.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">zq:QUERY_IS_SEQUENTIAL if the query is sequential.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">any dynamic error that is raised by evaluating the given query or applying its updates.</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 1, "name" : "evaluate", "qname" : "zq:evaluate", "signature" : "($query-key as xs:anyURI) as item()* external", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Evaluates the given prepared query and returns the result\n of the evaluation. The query must not be sequential or\n updating.</p>\n", "summary" : "<p>  Evaluates the given prepared query and returns the result\n of the evaluation.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "query-key", "type" : "xs:anyURI", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the identifier for a compiled query</div>" } ], "returns" : { "type" : "item()*", "description" : "the result of evaluating the given query" }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">zq:NO_QUERY_MATCH if no query with the given identifier was prepared.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">zq:QUERY_IS_UPDATING if the query is an updating query.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">zq:QUERY_IS_SEQUENTIAL if the query is sequential.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">any dynamic error that is raised by evaluating the given query.</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 1, "name" : "external-variables", "qname" : "zq:external-variables", "signature" : "($query-key as xs:anyURI) as xs:QName* external", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">The function returns the names of the external variables that\n are declared in the given query (either in the main module or\n in any of the imported library modules).</p>\n", "summary" : "<p>  The function returns the names of the external variables that\n are declared in the given query (either in the main module or\n in any of the imported library modules).</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "query-key", "type" : "xs:anyURI", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the identifier for a compiled query</div>" } ], "returns" : { "type" : "xs:QName*", "description" : "the sequence of names of the said external variables." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">zq:NO_QUERY_MATCH if no query with the given identifier was prepared.</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 1, "name" : "is-bound-context-item", "qname" : "zq:is-bound-context-item", "signature" : "($query-key as xs:anyURI) as xs:boolean external", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">The function tests if the context-item is bound for the\n execution of the query referred to by the given query identifier.</p>\n", "summary" : "<p>  The function tests if the context-item is bound for the\n execution of the query referred to by the given query identifier.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "query-key", "type" : "xs:anyURI", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the identifier for a compiled query</div>" } ], "returns" : { "type" : "xs:boolean", "description" : "true if the context-item is bound, false otherwise." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">zq:NO_QUERY_MATCH if no query with the given identifier was prepared.</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 2, "name" : "is-bound-variable", "qname" : "zq:is-bound-variable", "signature" : "($query-key as xs:anyURI, $var-name as xs:QName) as xs:boolean external", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">The function tests if the given variable is bound for the\n execution of the query referred to by the given query identifier.</p>\n", "summary" : "<p>  The function tests if the given variable is bound for the\n execution of the query referred to by the given query identifier.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "query-key", "type" : "xs:anyURI", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the identifier for a compiled query</div>" }, { "name" : "var-name", "type" : "xs:QName", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the name of the variable</div>" } ], "returns" : { "type" : "xs:boolean", "description" : "true if the variable is bound, false otherwise." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">zq:NO_QUERY_MATCH if no query with the given identifier was prepared.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">zq:UNDECLARED_VARIABLE if the given variable is not declared in the query.</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 1, "name" : "is-sequential", "qname" : "zq:is-sequential", "signature" : "($query-key as xs:anyURI) as xs:boolean external", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">The function tests if the query identified by the given key\n is sequential query.</p>\n", "summary" : "<p>  The function tests if the query identified by the given key\n is sequential query.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "query-key", "type" : "xs:anyURI", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the identifier for a compiled query</div>" } ], "returns" : { "type" : "xs:boolean", "description" : "true if the query is a sequential, false otherwise." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">zq:NO_QUERY_MATCH if no query with the given identifier was prepared.</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 1, "name" : "is-updating", "qname" : "zq:is-updating", "signature" : "($query-key as xs:anyURI) as xs:boolean external", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">The function tests if the query identified by the given key\n is an updating query.</p>\n", "summary" : "<p>  The function tests if the query identified by the given key\n is an updating query.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "query-key", "type" : "xs:anyURI", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the identifier for a compiled query</div>" } ], "returns" : { "type" : "xs:boolean", "description" : "true if the query is an updating query, false otherwise." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">zq:NO_QUERY_MATCH if no query with the given identifier was prepared.</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 1, "name" : "load-from-query-plan", "qname" : "zq:load-from-query-plan", "signature" : "($plan as xs:base64Binary) as xs:anyURI external", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">The function loads a given query for execution from a\n xs:base64Binary query plan, obtained through the zq:query-plan function.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">If the query was successfully loaded, the function returns an\n identifier as xs:anyURI. This URI can be passed to other functions\n of this module (e.g. to actually evaluate the query). The URI\n is opaque and its lifetime is bound by the lifetime of the query\n that invoked this function. Further reference or uses\n of the identifier lead to unexpected results.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Successfully prepared queries need to be deleted by passing the resulting\n identifier to the zq:delete-query function of this module.</p>\n", "summary" : "<p>  The function loads a given query for execution from a\n xs:base64Binary query plan, obtained through the zq:query-plan function.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "plan", "type" : "xs:base64Binary", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the binary query plan.</div>" } ], "returns" : { "type" : "xs:anyURI", "description" : "an identifier for the compiled query that can be passed as arguments to other functions of this module." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">any (static or type) error that may be raised during the compilation of the query. For example, err:XPST0003 if the given query could not be parsed.</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 3, "name" : "load-from-query-plan", "qname" : "zq:load-from-query-plan", "signature" : "($plan as xs:base64Binary, $resolver as item()?, $mapper as item()?) as xs:anyURI external", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">The function loads a given query for execution from a\n xs:base64Binary query plan, obtained through the zq:query-plan function.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">If the query was successfully loaded, the function returns an\n identifier as xs:anyURI. This URI can be passed to other functions\n of this module (e.g. to actually evaluate the query). The URI\n is opaque and its lilfetime is bound by the lifetime of the query\n that invoked this function. Further reference or uses\n of the identifier lead to unexpected results.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">For important notes regarding the second and third parameters of the\n function, review the comments in zq:prepare-main-module#3.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Successfully prepared queries need to be deleted by passing the resulting\n identifier to the zq:delete-query function of this module.</p>\n", "summary" : "<p>  The function loads a given query for execution from a\n xs:base64Binary query plan, obtained through the zq:query-plan function.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "plan", "type" : "xs:base64Binary", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the binary query plan.</div>" }, { "name" : "resolver", "type" : "item()", "occurrence" : "?", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the URL resolver function.</div>" }, { "name" : "mapper", "type" : "item()", "occurrence" : "?", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the URI mapper function.</div>" } ], "returns" : { "type" : "xs:anyURI", "description" : "an identifier for the compiled query that can be passed as arguments to other functions of this module." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">any (static or type) error that may be raised during the compilation of the query. For example, err:XPST0003 if the given query could not be parsed.</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 1, "name" : "prepare-library-module", "qname" : "zq:prepare-library-module", "signature" : "($library-module-text as xs:string) as empty-sequence() external", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">This function compiles a given XQuery or JSONiq library module.\n It can be used to compile-check a module.</p>\n", "summary" : "<p>  This function compiles a given XQuery or JSONiq library module.</p>", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "library-module-text", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the library module that should be prepared.</div>" } ], "returns" : { "type" : "empty-sequence()", "description" : "the empty-sequence." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">any (static or type) error that may be raised during the compilation of the library module. For example, err:XPST0003 if the given library module could not be parsed.</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 1, "name" : "prepare-main-module", "qname" : "zq:prepare-main-module", "signature" : "($main-module-text as xs:string) as xs:anyURI external", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">The function prepares a given a query for execution.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">If the query was successfully compiled, the function returns an\n identifier as xs:anyURI. This URI can be passed to other functions\n of this module (e.g. to actually evaluate the query). The URI\n is opaque and its lifetime is bound by the lifetime of the query\n that invoked this function. Further reference or uses\n of the identifier lead to unexpected results.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Successfully prepared queries need to be deleted by passing the resulting\n identifier to the zq:delete-query function of this module.</p>\n", "summary" : "<p>  The function prepares a given a query for execution.</p>", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "main-module-text", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the query that should be prepared. The query needs to be a XQuery or JSONiq main module.</div>" } ], "returns" : { "type" : "xs:anyURI", "description" : "an identifier for the compiled query that can be passed as arguments to other functions of this module." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">any (static or type) error that may be raised during the compilation of the query. For example, err:XPST0003 if the given query could not be parsed.</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 3, "name" : "prepare-main-module", "qname" : "zq:prepare-main-module", "signature" : "($main-module-text as xs:string, $resolver as function (xs:string, xs:string) as item()??, $mapper as function (xs:string, xs:string) as xs:string*?) as xs:anyURI external", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">The function prepares a given query for execution.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">If the query was successfully compiled, the function returns an\n identifier as xs:anyURI. This URI can be passed to other functions\n of this module (e.g. to actually evaluate the query). The URI\n is opaque and its lifetime is bound by the lifetime of the query\n that invoked this function. Further reference or uses\n of the identifier lead to unexpected results.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Important notes regarding the second and third parameters of the function:</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">--------------------------------------------------------------------------</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">These parameters allow you to specify a URL resolver and a URI mapper\n for Zorba to use when executing this query. See\n <a href=\"http://www.zorba-xquery.com/html/documentation/2.7.0/zorba/uriresolvers\">here</a></p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n <ul xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">The second parameter is a function item for a URL\n resolver. The URL resolver function must recive 2 parameters:\n <li>A $namespace as xs:string that will contain the url to be resolved.</li>\n <li>A $entity as xs:string that will contain the type of resolving needed.\n   This can be one of two values: \"module\" or \"schema\".</li>\n </ul>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">The function must return the empty sequence when the specified $namespace\n or $entity are not the ones to be resolved.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Example:</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n <pre xmlns:xqdoc=\"http://www.xqdoc.org/1.0\" class=\"ace-static\" ace-mode=\"xquery\">declare function mymod:url-resolver($namespace as xs:string, $entity as xs:string) as item()?\n {\n  if($namespace = 'http://test.xq')\n  then \"module namespace test = 'http://test'; declare function test:foo(){'foo'};\"\n  else ()\n };</pre>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">The URL resolver function's namespace, name, and parameter naming are\n not restricted by ZQ.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">The URL resolver function's return type is not restricted, it could be a string, a sequence,\n a node, etc. All the outputs types are to be serialized as a string.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">The third parameter is a function item for a URI mapper.</p>\n <ul xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">The URI mapper function, just like the URL resolver, receives 2 parameters:\n <li>A $namespace as xs:string that will contain the URI to be mapped.</li>\n <li>A $entity as xs:string that will contain the type of resolving needed.\n   This can be one of two values: \"module\" or \"schema\".</li>\n </ul>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">The URI mapper must return an empty sequence when the specified $namesapce or $entity\n are not to be mapped. Unlike the URL resolver this function must return a sequence of strings.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Example:</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n <pre xmlns:xqdoc=\"http://www.xqdoc.org/1.0\" class=\"ace-static\" ace-mode=\"xquery\">declare function mymod:uri-mapper($namespace as xs:string, $entity as xs:string)\n {\n  if($namespace = 'http://test')\n  then (\"http://zorba.io/test\", \"http://foo.com/schema/test\")\n  else ()\n };</pre>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">The URI mapper function's namespace, name, and parameter naming are\n not restricted by ZQ.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">In order to pass the above URL resolver and URI mapper to this function,\n use the following syntax:</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n   <pre xmlns:xqdoc=\"http://www.xqdoc.org/1.0\" class=\"ace-static\" ace-mode=\"xquery\">variable $queryID := zq:prepare-main-module(\"..query text..\",\n      mymod:url-resolver#2, mymod:uri-mapper#2);</pre>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">That is, the QName of the function followed by \"#2\". This is XQuery\n \"higher-order function\" syntax, meaning the function with the specified\n QName which takes two arguments. Since URL resolvers and URI mappers\n must take two arguments, both will always be specified with \"#2\".</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Both the URL resolver and URI mapper functions are optional, meaning you\n may pass the empty-sequence () for either.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Successfully prepared queries need to be deleted by passing the resulting\n identifier to the zq:delete-query function of this module.</p>\n", "summary" : "<p>  The function prepares a given query for execution.</p>", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "main-module-text", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the query that should be prepared. The query needs to be a XQuery or JSONiq main module.</div>" }, { "name" : "resolver", "type" : "function (xs:string, xs:string) as item()?", "occurrence" : "?", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the URL resolver function.</div>" }, { "name" : "mapper", "type" : "function (xs:string, xs:string) as xs:string*", "occurrence" : "?", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the URI mapper function.</div>" } ], "returns" : { "type" : "xs:anyURI", "description" : "an identifier for the compiled query that can be passed as arguments to other functions of this module." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">any (static or type) error that may be raised during the compilation of the query. For example, err:XPST0003 if the given query could not be parsed.</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 1, "name" : "query-plan", "qname" : "zq:query-plan", "signature" : "($query-key as xs:anyURI) as xs:base64Binary external", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Returns the compiled query identified by the given query-key\n as binary data.</p>\n", "summary" : "<p>  Returns the compiled query identified by the given query-key\n as binary data.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "query-key", "type" : "xs:anyURI", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the identifier of a compiled query.</div>" } ], "returns" : { "type" : "xs:base64Binary", "description" : "the query as xs:base64Binary." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">zq:NO_QUERY_MATCH if no query with the given identifier was prepared.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">zq:NO_QUERY_PLAN if there is an error serializing the query.</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 2, "name" : "variable-value", "qname" : "zq:variable-value", "signature" : "($query-key as xs:anyURI, $var-name as xs:QName) as item()* external", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">This function returns the value of a variable that is bound in the\n given query.</p>\n", "summary" : "<p>  This function returns the value of a variable that is bound in the\n given query.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "query-key", "type" : "xs:anyURI", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the identifier of a compiled query.</div>" }, { "name" : "var-name", "type" : "xs:QName", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the name of the variable whose value should be returned.</div>" } ], "returns" : { "type" : "item()*", "description" : "the value bound to the given variable." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">zq:NO_QUERY_MATCH if no query with the given identifier was prepared.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">zq:UNDECLARED_VARIABLE if the given variable is not declared in the query.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">zq:UNBOUND_VARIABLE if the given variable doesn't have a value.</xqdoc:error>" ] } ], "variables" : [  ] }, "http://xbrl.io/modules/bizql/hypercubes" : { "ns" : "http://xbrl.io/modules/bizql/hypercubes", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">This module provides functionality for manipulating hypercubes.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">A hypercube provides a multi-dimensional structure to organize facts.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">The dimensions of a hypercube correspond to XBRL aspects (concepts, entities, periods,\n units, further XBRL dimensions). An XBRL hypercube only is made of XBRL dimensions, however\n including the other aspects as well is very useful in the context of NOLAP, so that xbrl.io\n does so.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">xbrl.io introduces the notion of default hypercube. A default hypercube is implicitly added\n to each component, and only contains the concept, entity, period and unit aspects.\n In other words, the default hypercube\n of a component contains all these facts that do not have any XBRL dimensions.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">With this module, you can retrieve all hypercubes contained in a component. You can\n retrieve all facts contained in a hypercube (default dimension values are processed automatically),\n either in raw form, or organized as a (2D) fact table. You can also populate a network (for example,\n a presentation network) with the facts contained in a hypercube.</p>\n", "sees" : [  ], "authors" : [ "<xqdoc:author xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Charles Hoffman</xqdoc:author>", "<xqdoc:author xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Matthias Brantner</xqdoc:author>", "<xqdoc:author xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Dennis Knochenwefel</xqdoc:author>", "<xqdoc:author xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Ghislain Fourny</xqdoc:author>" ], "version" : null, "encoding" : "utf-8", "namespaces" : [ { "uri" : "http://xbrl.io/modules/bizql/archives", "prefix" : "archives" }, { "uri" : "http://xbrl.io/modules/bizql/concept-maps", "prefix" : "concept-maps" }, { "uri" : "http://xbrl.io/modules/bizql/entities", "prefix" : "entities" }, { "uri" : "http://xbrl.io/modules/bizql/facts", "prefix" : "facts" }, { "uri" : "http://xbrl.io/modules/bizql/hypercubes", "prefix" : "hypercubes" }, { "uri" : "http://zorba.io/options/versioning", "prefix" : "ver" } ], "functions" : [ { "isDocumented" : true, "arity" : 0, "name" : "dimensionless-hypercube", "qname" : "hypercubes:dimensionless-hypercube", "signature" : "() as object()", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Returns an instantiation of a dimensionless Hypercube containing only the basic\n    characteristics (xbrl:Concept, xbrl:Period, xbrl:Entity, and xbrl:Unit).\n    For each of those included aspects the value space is not limited.</p>\n", "summary" : "<p>  Returns an instantiation of a dimensionless Hypercube containing only the basic\n    characteristics (xbrl:Concept, xbrl:Period, xbrl:Entity, and xbrl:Unit).</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [  ], "returns" : { "type" : "object()", "description" : "dimensionless hypercube instantiation." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "dimensionless-hypercube", "qname" : "hypercubes:dimensionless-hypercube", "signature" : "($options as object()?) as object()", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Returns an instantiation of a dimensionless Hypercube containing only the basic\n    characteristics (xbrl:Concept, xbrl:Period, xbrl:Entity, and xbrl:Unit).\n    For each of those included aspects the value space is not limited.</p>\n", "summary" : "<p>  Returns an instantiation of a dimensionless Hypercube containing only the basic\n    characteristics (xbrl:Concept, xbrl:Period, xbrl:Entity, and xbrl:Unit).</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "options", "type" : "object()", "occurrence" : "?", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">: additional options among which: - Concepts: an array of concept names to include in the hypercube. - Periods: an array of periods to include in the hypercube. - Entities: an array of EIDs to include in the hypercube. - Units: an array of units to include in the hypercube.</div>" } ], "returns" : { "type" : "object()", "description" : "dimensionless hypercube instantiation." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 2, "name" : "fact-table-for-hypercube", "qname" : "hypercubes:fact-table-for-hypercube", "signature" : "($hypercube as object(), $archives as item()*) as array()", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Retrieves all facts from the supplied archives, that are relevant to the\n supplied hypercube, and populates them with the default dimension values\n when missing.</p>\n", "summary" : "<p>  Retrieves all facts from the supplied archives, that are relevant to the\n supplied hypercube, and populates them with the default dimension values\n when missing.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "hypercube", "type" : "object()", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> a hypercube.</div>" }, { "name" : "archives", "type" : "item()", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> a sequence of archives or their AIDs or $hypercubes:ALL_ARCHIVES for no filtering.</div>" } ], "returns" : { "type" : "array()", "description" : "a sequence of facts with populated dimension values." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 3, "name" : "fact-table-for-hypercube", "qname" : "hypercubes:fact-table-for-hypercube", "signature" : "($hypercube as object(), $archives as item()*, $options as object()?) as array()", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Retrieves all facts from the supplied archives, that are relevant to the\n supplied hypercube, and populates them with the default dimension values\n when missing.</p>\n", "summary" : "<p>  Retrieves all facts from the supplied archives, that are relevant to the\n supplied hypercube, and populates them with the default dimension values\n when missing.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "hypercube", "type" : "object()", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> a hypercube.</div>" }, { "name" : "archives", "type" : "item()", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> a sequence of archives or their AIDs</div>" }, { "name" : "options", "type" : "object()", "occurrence" : "?", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> <a href=\"facts#standard_options\">standard fact retrieving options</a>.</div>" } ], "returns" : { "type" : "array()", "description" : "a sequence of facts with populated dimension values." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 2, "name" : "facts-for-hypercube", "qname" : "hypercubes:facts-for-hypercube", "signature" : "($hypercube as object(), $archives as item()*) as item()*", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Retrieves all facts from the supplied archives, that are relevant to the\n supplied hypercube, and populates them with the default dimension values\n when missing.</p>\n", "summary" : "<p>  Retrieves all facts from the supplied archives, that are relevant to the\n supplied hypercube, and populates them with the default dimension values\n when missing.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "hypercube", "type" : "object()", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> a hypercube.</div>" }, { "name" : "archives", "type" : "item()", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> a sequence of archives or their AIDs or $hypercubes:ALL_ARCHIVES for no filtering.</div>" } ], "returns" : { "type" : "item()*", "description" : "a sequence of facts with populated dimension values." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 3, "name" : "facts-for-hypercube", "qname" : "hypercubes:facts-for-hypercube", "signature" : "($hypercube as object(), $archives as item()*, $options as object()?) as item()*", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Retrieves all facts from the supplied archives, that are relevant to the\n supplied hypercube, and populates them with the default dimension values\n when missing.</p>\n", "summary" : "<p>  Retrieves all facts from the supplied archives, that are relevant to the\n supplied hypercube, and populates them with the default dimension values\n when missing.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "hypercube", "type" : "object()", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> a hypercube.</div>" }, { "name" : "archives", "type" : "item()", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> a sequence of archives or their AIDs or $hypercubes:ALL_ARCHIVES for no filtering.</div>" }, { "name" : "options", "type" : "object()", "occurrence" : "?", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> <a href=\"facts#standard_options\">standard fact retrieving options</a>.</div>" } ], "returns" : { "type" : "item()*", "description" : "a sequence of facts with populated dimension values." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "hypercubes-for-components", "qname" : "hypercubes:hypercubes-for-components", "signature" : "($components as object()*) as object()*", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Retrieves all hypercubes in the supplied components.</p>\n", "summary" : "<p>  Retrieves all hypercubes in the supplied components.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "components", "type" : "object()", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> a sequence of components.</div>" } ], "returns" : { "type" : "object()*", "description" : "all hypercubes." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 2, "name" : "hypercubes-for-components", "qname" : "hypercubes:hypercubes-for-components", "signature" : "($components as object()*, $names as string*) as object()*", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Retrieves all hypercubes in the supplied components and\n with the given names.</p>\n", "summary" : "<p>  Retrieves all hypercubes in the supplied components and\n with the given names.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "components", "type" : "object()", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> a sequence of components.</div>" }, { "name" : "names", "type" : "string", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> a sequence of names.</div>" } ], "returns" : { "type" : "object()*", "description" : "all hypercubes." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 3, "name" : "populate-networks-with-facts", "qname" : "hypercubes:populate-networks-with-facts", "signature" : "($networks as object()*, $hypercube as object(), $archives as item()*) as object()*", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Populates a concept-tree network with all facts from the supplied archives,\n that are relevant to the\n supplied hypercube. Default dimension values are added to the facts\n when missing.</p>\n", "summary" : "<p>  Populates a concept-tree network with all facts from the supplied archives,\n that are relevant to the\n supplied hypercube.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "networks", "type" : "object()", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> networks.</div>" }, { "name" : "hypercube", "type" : "object()", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> a hypercube.</div>" }, { "name" : "archives", "type" : "item()", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> a sequence of archives or their AIDs or $hypercubes:ALL_ARCHIVES for no filtering.</div>" } ], "returns" : { "type" : "object()*", "description" : "a sequence of facts with populated dimension values." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 4, "name" : "populate-networks-with-facts", "qname" : "hypercubes:populate-networks-with-facts", "signature" : "($networks as object()*, $hypercube as object(), $archives as item()*, $options as object()?) as object()*", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Populates a concept-tree network with all facts from the supplied archives,\n that are relevant to the\n supplied hypercube. Default dimension values are added to the facts\n when missing.</p>\n", "summary" : "<p>  Populates a concept-tree network with all facts from the supplied archives,\n that are relevant to the\n supplied hypercube.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "networks", "type" : "object()", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> networks.</div>" }, { "name" : "hypercube", "type" : "object()", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> a hypercube.</div>" }, { "name" : "archives", "type" : "item()", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> a sequence of archives or their AIDs.</div>" }, { "name" : "options", "type" : "object()", "occurrence" : "?", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> <a href=\"facts#standard_options\">standard fact retrieving options</a>.</div>" } ], "returns" : { "type" : "object()*", "description" : "a sequence of facts with populated dimension values." }, "errors" : [  ] } ], "variables" : [ { "name" : "hypercubes:ALL_ARCHIVES", "type" : "boolean", "description" : " Joker for all archives.\n" } ] }, "http://www.zorba-xquery.com/modules/xqdoc/batch" : { "ns" : "http://www.zorba-xquery.com/modules/xqdoc/batch", "description" : " Process XQDoc batches.\n This module generates XQDoc HTML documentation from multiple\n XQuery modules.\n", "sees" : [  ], "authors" : [ "<xqdoc:author xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">William Candillon <a href=\"?anchor=\">wcandillon at gmail dot com</a></xqdoc:author>" ], "version" : null, "encoding" : "utf-8", "namespaces" : [ { "uri" : "http://zorba.io/annotations", "prefix" : "an" }, { "uri" : "http://www.zorba-xquery.com/modules/xqdoc/batch", "prefix" : "batch" }, { "uri" : "http://www.w3.org/2005/xqt-errors", "prefix" : "err" }, { "uri" : "http://expath.org/ns/file", "prefix" : "file" }, { "uri" : "http://www.functx.com", "prefix" : "functx" }, { "uri" : "http://www.w3.org/1999/xhtml", "prefix" : "h" }, { "uri" : "http://www.zorba-xquery.com/modules/xqdoc/html", "prefix" : "html" }, { "uri" : "http://www.zorba-xquery.com/modules/xqdoc/menu", "prefix" : "menu" }, { "uri" : "http://www.w3.org/2010/xslt-xquery-serialization", "prefix" : "out" }, { "uri" : "http://zorba.io/options/versioning", "prefix" : "ver" }, { "uri" : "http://www.xqdoc.org/1.0", "prefix" : "xq" }, { "uri" : "http://zorba.io/modules/xqdoc", "prefix" : "xqdoc" } ], "functions" : [ { "isDocumented" : false, "arity" : 2, "name" : "add-predeclared-namespaces", "qname" : "batch:add-predeclared-namespaces", "signature" : "($xqdoc as element(xq:xqdoc), $namespaces as element(namespace)*) as element(xq:xqdoc)", "description" : "", "summary" : "", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "xqdoc", "type" : "element(xq:xqdoc)", "occurrence" : null, "description" : "" }, { "name" : "namespaces", "type" : "element(namespace)", "occurrence" : "*", "description" : "" } ], "returns" : { "type" : "element(xq:xqdoc)", "description" : "" }, "errors" : [  ] }, { "isDocumented" : false, "arity" : 1, "name" : "add-trailing-slash", "qname" : "batch:add-trailing-slash", "signature" : "($path as xs:string) as xs:string", "description" : "", "summary" : "", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "path", "type" : "xs:string", "occurrence" : null, "description" : "" } ], "returns" : { "type" : "xs:string", "description" : "" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 4, "name" : "build-xqdoc", "qname" : "batch:build-xqdoc", "signature" : "($output-folder as xs:string, $static-folders as xs:string*, $template as element(*), $modules as element(modules))", "description" : " Run an XQDoc batch.\n", "summary" : "<p> Run an XQDoc batch.</p>", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "output-folder", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> Where to write the generated files.</div>" }, { "name" : "static-folders", "type" : "xs:string", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> Where to copy the static files from.</div>" }, { "name" : "template", "type" : "element(*)", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> HTML layout of the generated files. The layount can contains different variables. For instance: <pre class=\"ace-static\" ace-mode=\"xml\"><body><var name=\"page\"/></body></pre> is a valid example. Three variable names are available: <code>page</code>, <code>title</code>, and <code>menu</code>.</div>" }, { "name" : "modules", "type" : "element(modules)", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> Document describing the documentation project. For instance: <pre class=\"ace-static\" ace-mode=\"xml\"> <modules> <namespace prefix=\"ann\" uri=\"http://zorba.io/annotations\"/> <section id=\"modules\" label=\"My Modules\"> <!-- Location hints are optionnals --> <module ns=\"http://example.com/mymodule\" label=\"My Module\" id=\"module\" file=\"module.xq\"/> </section> </modules> </pre></div>" } ], "returns" : { "type" : null, "description" : "" }, "errors" : [  ] }, { "isDocumented" : false, "arity" : 2, "name" : "copy-static-folders", "qname" : "batch:copy-static-folders", "signature" : "($output-folder as xs:string, $static-folders as xs:string*)", "description" : "", "summary" : "", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "output-folder", "type" : "xs:string", "occurrence" : null, "description" : "" }, { "name" : "static-folders", "type" : "xs:string", "occurrence" : "*", "description" : "" } ], "returns" : { "type" : null, "description" : "" }, "errors" : [  ] }, { "isDocumented" : false, "arity" : 3, "name" : "create-page", "qname" : "batch:create-page", "signature" : "($output-folder as xs:string, $page-name as xs:string, $page as element(h:html))", "description" : "", "summary" : "", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "output-folder", "type" : "xs:string", "occurrence" : null, "description" : "" }, { "name" : "page-name", "type" : "xs:string", "occurrence" : null, "description" : "" }, { "name" : "page", "type" : "element(h:html)", "occurrence" : null, "description" : "" } ], "returns" : { "type" : null, "description" : "" }, "errors" : [  ] }, { "isDocumented" : false, "arity" : 1, "name" : "create-xml-folder", "qname" : "batch:create-xml-folder", "signature" : "($folder as xs:string)", "description" : "", "summary" : "", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "folder", "type" : "xs:string", "occurrence" : null, "description" : "" } ], "returns" : { "type" : null, "description" : "" }, "errors" : [  ] }, { "isDocumented" : false, "arity" : 3, "name" : "page", "qname" : "batch:page", "signature" : "($template as element(*), $menu as element(ul), $section as element(*)) as element(h:html)", "description" : "", "summary" : "", "annotation_str" : " %an:nondeterministic", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "nondeterministic", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "template", "type" : "element(*)", "occurrence" : null, "description" : "" }, { "name" : "menu", "type" : "element(ul)", "occurrence" : null, "description" : "" }, { "name" : "section", "type" : "element(*)", "occurrence" : null, "description" : "" } ], "returns" : { "type" : "element(h:html)", "description" : "" }, "errors" : [  ] }, { "isDocumented" : false, "arity" : 2, "name" : "save-xml", "qname" : "batch:save-xml", "signature" : "($output-file, $page)", "description" : "", "summary" : "", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "output-file", "type" : null, "occurrence" : null, "description" : "" }, { "name" : "page", "type" : null, "occurrence" : null, "description" : "" } ], "returns" : { "type" : null, "description" : "" }, "errors" : [  ] }, { "isDocumented" : false, "arity" : 1, "name" : "section", "qname" : "batch:section", "signature" : "($sections as element(section)+) as element(section)+", "description" : "", "summary" : "", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "sections", "type" : "element(section)", "occurrence" : "+", "description" : "" } ], "returns" : { "type" : "element(section)+", "description" : "" }, "errors" : [  ] }, { "isDocumented" : false, "arity" : 2, "name" : "section", "qname" : "batch:section", "signature" : "($sections as element(section)+, $level as xs:integer) as element(section)+", "description" : "", "summary" : "", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "sections", "type" : "element(section)", "occurrence" : "+", "description" : "" }, { "name" : "level", "type" : "xs:integer", "occurrence" : null, "description" : "" } ], "returns" : { "type" : "element(section)+", "description" : "" }, "errors" : [  ] }, { "isDocumented" : false, "arity" : 1, "name" : "xqdoc", "qname" : "batch:xqdoc", "signature" : "($module as element(module)) as element(xq:xqdoc)", "description" : "", "summary" : "", "annotation_str" : " %an:nondeterministic", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "nondeterministic", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "module", "type" : "element(module)", "occurrence" : null, "description" : "" } ], "returns" : { "type" : "element(xq:xqdoc)", "description" : "" }, "errors" : [  ] } ], "variables" : [  ] }, "http://zorba.io/modules/data-cleaning/hybrid-string-similarity" : { "ns" : "http://zorba.io/modules/data-cleaning/hybrid-string-similarity", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">This library module provides hybrid string similarity functions, combining the properties of\n character-based string similarity functions and token-based string similarity functions.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">The logic contained in this module is not specific to any particular XQuery implementation,\n although the module requires the trigonometic functions of XQuery 3.0 or a math extension\n function such as sqrt($x as numeric) for computing the square root.</p>\n", "sees" : [  ], "authors" : [ "<xqdoc:author xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Bruno Martins and Diogo Simões</xqdoc:author>" ], "version" : null, "encoding" : "utf-8", "namespaces" : [ { "uri" : "http://www.w3.org/2005/xpath-functions/math", "prefix" : "math" }, { "uri" : "http://zorba.io/modules/data-cleaning/set-similarity", "prefix" : "set" }, { "uri" : "http://zorba.io/modules/data-cleaning/character-based-string-similarity", "prefix" : "simc" }, { "uri" : "http://zorba.io/modules/data-cleaning/hybrid-string-similarity", "prefix" : "simh" }, { "uri" : "http://zorba.io/modules/data-cleaning/phonetic-string-similarity", "prefix" : "simp" }, { "uri" : "http://zorba.io/modules/data-cleaning/token-based-string-similarity", "prefix" : "simt" }, { "uri" : "http://zorba.io/options/versioning", "prefix" : "ver" } ], "functions" : [ { "isDocumented" : true, "arity" : 4, "name" : "monge-elkan-jaro-winkler", "qname" : "simh:monge-elkan-jaro-winkler", "signature" : "($s1 as xs:string, $s2 as xs:string, $prefix as xs:integer, $fact as xs:double) as xs:double", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Returns the Monge-Elkan similarity coefficient between two strings, using the Jaro-Winkler</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">similarity function to discover token identity.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Example usage : <pre class=\"ace-static\" ace-mode=\"xquery\"> monge-elkan-jaro-winkler(\"Comput. Sci. and Eng. Dept., University of California, San Diego\", \"Department of Computer Scinece, Univ. Calif., San Diego\", 4, 0.1) </pre></p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">The function invocation in the example above returns : <pre class=\"ace-static\" ace-mode=\"xquery\"> 0.992 </pre></p>\n", "summary" : "<p>  Returns the Monge-Elkan similarity coefficient between two strings, using the Jaro-Winkler \n  similarity function to discover token identity.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "s1", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The first string.</div>" }, { "name" : "s2", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The second string.</div>" }, { "name" : "prefix", "type" : "xs:integer", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The number of characters to consider when testing for equal prefixes with the Jaro-Winkler metric.</div>" }, { "name" : "fact", "type" : "xs:double", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The weighting factor to consider when the input strings have equal prefixes with the Jaro-Winkler metric.</div>" } ], "returns" : { "type" : "xs:double", "description" : "The Monge-Elkan similarity coefficient between the two strings." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 4, "name" : "soft-cosine-tokens-edit-distance", "qname" : "simh:soft-cosine-tokens-edit-distance", "signature" : "($s1 as xs:string, $s2 as xs:string, $r as xs:string, $t as xs:integer) as xs:double", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Returns the cosine similarity coefficient between sets of tokens extracted from two strings.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">The tokens from each string are weighted according to their occurence frequency (i.e., weighted according to the\n term-frequency heuristic from Information Retrieval).</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">The Edit Distance similarity function is used to discover token identity, and tokens having an edit distance\n bellow a given threshold are considered as matching tokens.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Example usage : <pre class=\"ace-static\" ace-mode=\"xquery\"> soft-cosine-tokens-edit-distance(\"The FLWOR Foundation\", \"FLWOR Found.\", \" +\", 0 ) </pre></p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">The function invocation in the example above returns : <pre class=\"ace-static\" ace-mode=\"xquery\"> 0.408248290463863 </pre></p>\n", "summary" : "<p>  Returns the cosine similarity coefficient between sets of tokens extracted from two strings.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "s1", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The first string.</div>" }, { "name" : "s2", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The second string.</div>" }, { "name" : "r", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> A regular expression forming the delimiter character(s) which mark the boundaries between adjacent tokens.</div>" }, { "name" : "t", "type" : "xs:integer", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> A threshold for the similarity function used to discover token identity.</div>" } ], "returns" : { "type" : "xs:double", "description" : "The cosine similarity coefficient between the sets tokens extracted from the two strings." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 6, "name" : "soft-cosine-tokens-jaro-winkler", "qname" : "simh:soft-cosine-tokens-jaro-winkler", "signature" : "($s1 as xs:string, $s2 as xs:string, $r as xs:string, $t as xs:double, $prefix as xs:integer?, $fact as xs:double?) as xs:double", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Returns the cosine similarity coefficient between sets of tokens extracted from two strings.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">The tokens from each string are weighted according to their occurence frequency (i.e., weighted according to the\n term-frequency heuristic from Information Retrieval).</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">The Jaro-Winkler similarity function is used to discover token identity, and tokens having a Jaro-Winkler\n similarity above a given threshold are considered as matching tokens.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Example usage : <pre class=\"ace-static\" ace-mode=\"xquery\"> soft-cosine-tokens-jaro-winkler(\"The FLWOR Foundation\", \"FLWOR Found.\", \" +\", 1, 4, 0.1 ) </pre></p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">The function invocation in the example above returns : <pre class=\"ace-static\" ace-mode=\"xquery\"> 0.45 </pre></p>\n", "summary" : "<p>  Returns the cosine similarity coefficient between sets of tokens extracted from two strings.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "s1", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The first string.</div>" }, { "name" : "s2", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The second string.</div>" }, { "name" : "r", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> A regular expression forming the delimiter character(s) which mark the boundaries between adjacent tokens.</div>" }, { "name" : "t", "type" : "xs:double", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> A threshold for the similarity function used to discover token identity.</div>" }, { "name" : "prefix", "type" : "xs:integer", "occurrence" : "?", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The number of characters to consider when testing for equal prefixes with the Jaro-Winkler metric.</div>" }, { "name" : "fact", "type" : "xs:double", "occurrence" : "?", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The weighting factor to consider when the input strings have equal prefixes with the Jaro-Winkler metric.</div>" } ], "returns" : { "type" : "xs:double", "description" : "The cosine similarity coefficient between the sets tokens extracted from the two strings." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 4, "name" : "soft-cosine-tokens-jaro", "qname" : "simh:soft-cosine-tokens-jaro", "signature" : "($s1 as xs:string, $s2 as xs:string, $r as xs:string, $t as xs:double) as xs:double", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Returns the cosine similarity coefficient between sets of tokens extracted from two strings.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">The tokens from each string are weighted according to their occurence frequency (i.e., weighted according to the\n term-frequency heuristic from Information Retrieval).</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">The Jaro similarity function is used to discover token identity, and tokens having a Jaro similarity above\n a given threshold are considered as matching tokens.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Example usage : <pre class=\"ace-static\" ace-mode=\"xquery\"> soft-cosine-tokens-jaro(\"The FLWOR Foundation\", \"FLWOR Found.\", \" +\", 1 ) </pre></p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">The function invocation in the example above returns : <pre class=\"ace-static\" ace-mode=\"xquery\"> 0.5 </pre></p>\n", "summary" : "<p>  Returns the cosine similarity coefficient between sets of tokens extracted from two strings.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "s1", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The first string.</div>" }, { "name" : "s2", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The second string.</div>" }, { "name" : "r", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> A regular expression forming the delimiter character(s) which mark the boundaries between adjacent tokens.</div>" }, { "name" : "t", "type" : "xs:double", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> A threshold for the similarity function used to discover token identity.</div>" } ], "returns" : { "type" : "xs:double", "description" : "The cosine similarity coefficient between the sets tokens extracted from the two strings." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 3, "name" : "soft-cosine-tokens-metaphone", "qname" : "simh:soft-cosine-tokens-metaphone", "signature" : "($s1 as xs:string, $s2 as xs:string, $r as xs:string) as xs:double", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Returns the cosine similarity coefficient between sets of tokens extracted from two strings.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">The tokens from each string are weighted according to their occurence frequency (i.e., weighted according to the\n term-frequency heuristic from Information Retrieval).</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">The Metaphone phonetic similarity function is used to discover token identity, which is equivalent to saying that\n this function returns the cosine similarity coefficient between sets of Metaphone keys.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Example usage : <pre class=\"ace-static\" ace-mode=\"xquery\"> soft-cosine-tokens-metaphone(\"ALEKSANDER SMITH\", \"ALEXANDER SMYTH\", \" +\" ) </pre></p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">The function invocation in the example above returns : <pre class=\"ace-static\" ace-mode=\"xquery\"> 1.0 </pre></p>\n", "summary" : "<p>  Returns the cosine similarity coefficient between sets of tokens extracted from two strings.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "s1", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The first string.</div>" }, { "name" : "s2", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The second string.</div>" }, { "name" : "r", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> A regular expression forming the delimiter character(s) which mark the boundaries between adjacent tokens.</div>" } ], "returns" : { "type" : "xs:double", "description" : "The cosine similarity coefficient between the sets Metaphone keys extracted from the two strings." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 3, "name" : "soft-cosine-tokens-soundex", "qname" : "simh:soft-cosine-tokens-soundex", "signature" : "($s1 as xs:string, $s2 as xs:string, $r as xs:string) as xs:double", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Returns the cosine similarity coefficient between sets of tokens extracted from two strings.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">The tokens from each string are weighted according to their occurence frequency (i.e., weighted according to the\n term-frequency heuristic from Information Retrieval).</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">The Soundex phonetic similarity function is used to discover token identity, which is equivalent to saying that\n this function returns the cosine similarity coefficient between sets of Soundex keys.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Example usage : <pre class=\"ace-static\" ace-mode=\"xquery\"> soft-cosine-tokens-soundex(\"ALEKSANDER SMITH\", \"ALEXANDER SMYTH\", \" +\") </pre></p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">The function invocation in the example above returns : <pre class=\"ace-static\" ace-mode=\"xquery\"> 1.0 </pre></p>\n", "summary" : "<p>  Returns the cosine similarity coefficient between sets of tokens extracted from two strings.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "s1", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The first string.</div>" }, { "name" : "s2", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The second string.</div>" }, { "name" : "r", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> A regular expression forming the delimiter character(s) which mark the boundaries between adjacent tokens.</div>" } ], "returns" : { "type" : "xs:double", "description" : "The cosine similarity coefficient between the sets of Soundex keys extracted from the two strings." }, "errors" : [  ] } ], "variables" : [  ] }, "http://xbrl.io/modules/bizql/rules" : { "ns" : "http://xbrl.io/modules/bizql/rules", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">This module provides functions for storing, retrieving, and modifying\n rules. Rules can be used in BizQL queries to:</p>\n <ul xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">\n   <li>Impute Facts that were not reported within an instance, yet can be\n       derived from reported facts.</li>\n   <li>Compute arbitrary new Facts (Ratios, Calculated Facts, Facts from different\n       Datasources/Instances, etc.).</li>\n   <li>Run validation and/or verification rules</li>\n </ul>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">A Rule is an object containing a BizQL fomula to execute if a condition is met.\n For example if a user queries for a certain concept and a rule exists for this\n concept then the rule is applied to get the fact.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">With this module, you can retrieve the rules associated with each report\n schema. You can also query for facts by making implicitly use of the mapping.</p>\n", "sees" : [  ], "authors" : [ "<xqdoc:author xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Charles Hoffman</xqdoc:author>", "<xqdoc:author xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Matthias Brantner</xqdoc:author>", "<xqdoc:author xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Dennis Knochenwefel</xqdoc:author>", "<xqdoc:author xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Ghislain Fourny</xqdoc:author>" ], "version" : null, "encoding" : "utf-8", "namespaces" : [ { "uri" : "http://xbrl.io/modules/bizql/archives", "prefix" : "archives" }, { "uri" : "http://xbrl.io/modules/bizql/facts", "prefix" : "facts" }, { "uri" : "http://xbrl.io/modules/bizql/networks", "prefix" : "networks" }, { "uri" : "http://xbrl.io/modules/bizql/report-schemas", "prefix" : "report-schemas" }, { "uri" : "http://xbrl.io/modules/bizql/rules", "prefix" : "rules" }, { "uri" : "http://zorba.io/options/versioning", "prefix" : "ver" } ], "functions" : [ { "isDocumented" : true, "arity" : 7, "name" : "create-computed-fact", "qname" : "rules:create-computed-fact", "signature" : "($template-fact as object(), $concept-name-or-aspects as item(), $value as item(), $rule as object(), $audit-trail-message as string, $source-facts as object()*, $options as object()?) as object()", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Helper function to create a new fact within a rule.</p>\n", "summary" : "<p>  Helper function to create a new fact within a rule.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "template-fact", "type" : "object()", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> a fact object that will be used as a template for the newly created fact</div>" }, { "name" : "concept-name-or-aspects", "type" : "item()", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> either a name of the concept for the newly created fact or a complete aspects object to be used in the new fact.</div>" }, { "name" : "value", "type" : "item()", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> a value for the newly created fact</div>" }, { "name" : "rule", "type" : "object()", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the rule in which this fact has been created</div>" }, { "name" : "audit-trail-message", "type" : "string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> a verbose string message explaining how and why this fact has been created</div>" }, { "name" : "source-facts", "type" : "object()", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> sequence of facts that have been used to compute the new fact (this will only be added to the AuditTrails if the audit-trail option is set to \"debug\")</div>" }, { "name" : "options", "type" : "object()", "occurrence" : "?", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> <a href=\"facts#standard_options\">standard fact retrieving options</a>.</div>" } ], "returns" : { "type" : "object()", "description" : "the decimal value of the fact or 0." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "fact-trail", "qname" : "rules:fact-trail", "signature" : "($fact as object()?) as string", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Serializes a fact to a simple string format that can be used in\n    audit trail messages to trail the value of the fact.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">The returned string follows the pattern: fact-concept-name[fact-value]</p>\n", "summary" : "<p>  Serializes a fact to a simple string format that can be used in\n    audit trail messages to trail the value of the fact.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "fact", "type" : "object()", "occurrence" : "?", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> a single fact to serialize to a simple informative string</div>" } ], "returns" : { "type" : "string", "description" : "the string with key information about the fact." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 2, "name" : "fact-trail", "qname" : "rules:fact-trail", "signature" : "($fact as object()?, $name as string) as string", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Serializes a fact to a simple string format that can be used in\n    audit trail messages to trail the value of the fact.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">The returned string follows the pattern: fact-concept-name[fact-value]</p>\n", "summary" : "<p>  Serializes a fact to a simple string format that can be used in\n    audit trail messages to trail the value of the fact.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "fact", "type" : "object()", "occurrence" : "?", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> an empty-sequence or a single fact to serialize to a simple informative string</div>" }, { "name" : "name", "type" : "string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> an alternative name to use if the $fact is an empty-sequence otherwise the name will be taken from the fact</div>" } ], "returns" : { "type" : "string", "description" : "the string with key information about the fact." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 0, "name" : "rules", "qname" : "rules:rules", "signature" : "() as object()*", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Retrieves all rules from all report schemas.</p>\n", "summary" : "<p>  Retrieves all rules from all report schemas.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [  ], "returns" : { "type" : "object()*", "description" : "all rules." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "rules", "qname" : "rules:rules", "signature" : "($report-schemas-or-ids as item()*) as object()*", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Return the rules from the given report schemas.</p>\n", "summary" : "<p>  Return the rules from the given report schemas.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "report-schemas-or-ids", "type" : "item()", "occurrence" : "*", "description" : "" } ], "returns" : { "type" : "object()*", "description" : "the rules from the report schemas." }, "errors" : [  ] } ], "variables" : [  ] }, "http://www.28msec.com/modules/lock" : { "ns" : "http://www.28msec.com/modules/lock", "description" : " Sausalito provides a an application level locking mechanism that can be\n used e.g. to ensure exclusive access to data.\n As the store ensures atomicity of of single document updates, atomic\n updates to multiple documents are implemented using this mechanism.\n Locks can be acquired and re-acquired at any time during a request and\n are released at the end of the request.\n", "sees" : [  ], "authors" : [ "<xqdoc:author xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">28msec</xqdoc:author>" ], "version" : null, "encoding" : "utf-8", "namespaces" : [ { "uri" : "http://zorba.io/annotations", "prefix" : "an" }, { "uri" : "http://www.28msec.com/modules/lock", "prefix" : "lock" }, { "uri" : "http://zorba.io/options/versioning", "prefix" : "ver" } ], "functions" : [ { "isDocumented" : true, "arity" : 1, "name" : "try-acquire", "qname" : "lock:try-acquire", "signature" : "($lock-name as xs:string) as xs:boolean external", "description" : " Tries to acquire a lock.\n This operation is non-blocking if the lock cannot be acquired immediately.\n To check and modify the stock of a product wihtout interference from\n concurrent requests a lock could be used like this:\n <pre xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">\n if (lock:try-acquire(\"stock-update\"))\n then\n   if (stock:check($product-id))\n   then\n     {\n       order:finalize($order-id);\n       stock:decrement($product-id);\n     }\n   else\n     order:hold($order-id);\n else\n   ...\n </pre>\n", "summary" : "<p> Tries to acquire a lock.</p>", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "lock-name", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The name of the lock.</div>" } ], "returns" : { "type" : "xs:boolean", "description" : "true if the lock could be acquired, false otherwise." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 2, "name" : "try-acquire", "qname" : "lock:try-acquire", "signature" : "($lock-name as xs:string, $reason as xs:string) as xs:boolean external", "description" : " tries to acquire a lock\n", "summary" : "<p> tries to acquire a lock\n</p>", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "lock-name", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The name of the lock.</div>" }, { "name" : "reason", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> a reason for the acquisition of the lock that can be used for</div>" } ], "returns" : { "type" : "xs:boolean", "description" : "true if the lock could be acquired, false otherwise." }, "errors" : [  ] } ], "variables" : [  ] }, "http://www.zorba-xquery.com/modules/xqdoc/html" : { "ns" : "http://www.zorba-xquery.com/modules/xqdoc/html", "description" : " Convert an XQDoc document into an HTML document.\n This module contains a single <code xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">convert()</code> function\n that transform an XQDoc document into an HTML document.\n Usage:\n <pre xmlns:xqdoc=\"http://www.xqdoc.org/1.0\" class=\"ace-static\" ace-mode=\"xquery\">\n let $xqdoc := xqdoc:xqdoc(\"http://expath.org/ns/file\")\n return html:convert($xqdoc)\n </pre>\n", "sees" : [  ], "authors" : [ "<xqdoc:author xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">William Candillon <a href=\"?anchor=\">wcandillon at gmail dot com</a></xqdoc:author>" ], "version" : null, "encoding" : "utf-8", "namespaces" : [ { "uri" : "http://www.zorba-xquery.com/modules/xqdoc/html", "prefix" : "html" }, { "uri" : "http://www.w3.org/2010/xslt-xquery-serialization", "prefix" : "o" }, { "uri" : "http://zorba.io/options/versioning", "prefix" : "ver" }, { "uri" : "http://www.xqdoc.org/1.0", "prefix" : "xq" } ], "functions" : [ { "isDocumented" : false, "arity" : 1, "name" : "convert", "qname" : "html:convert", "signature" : "($xqdoc as element(xq:xqdoc)) as element(div)", "description" : "", "summary" : "", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "xqdoc", "type" : "element(xq:xqdoc)", "occurrence" : null, "description" : "" } ], "returns" : { "type" : "element(div)", "description" : "" }, "errors" : [  ] } ], "variables" : [  ] }, "http://api.28.io/model" : { "ns" : "http://api.28.io/model", "description" : "", "sees" : [  ], "authors" : [  ], "version" : null, "encoding" : null, "namespaces" : [ { "uri" : "", "prefix" : "an" }, { "uri" : "http://zorba.io/modules/store/static/collections/ddl", "prefix" : "cddl" }, { "uri" : "http://zorba.io/modules/store/static/collections/dml", "prefix" : "cdml" }, { "uri" : "http://zorba.io/modules/store/static/indexes/ddl", "prefix" : "iddl" }, { "uri" : "http://zorba.io/modules/store/static/indexes/dml", "prefix" : "idml" }, { "uri" : "http://api.28.io/model", "prefix" : "model" }, { "uri" : "http://www.zorba-xquery.com/schemas/pul", "prefix" : "pul" }, { "uri" : "http://zorba.io/modules/reference", "prefix" : "ref" }, { "uri" : "http://www.zorba-xquery.com/schemas/xdm", "prefix" : "xdm" } ], "functions" : [ { "isDocumented" : false, "arity" : 1, "name" : "applyPUL", "qname" : "model:applyPUL", "signature" : "($pul as schema-element(pul:pending-update-list))", "description" : "", "summary" : "", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "pul", "type" : "schema-element(pul:pending-update-list)", "occurrence" : null, "description" : "" } ], "returns" : { "type" : null, "description" : "" }, "errors" : [  ] }, { "isDocumented" : false, "arity" : 1, "name" : "applyPULOp", "qname" : "model:applyPULOp", "signature" : "($pul)", "description" : "", "summary" : "", "annotation_str" : "", "annotations" : [  ], "updating" : true, "parameters" : [ { "name" : "pul", "type" : null, "occurrence" : null, "description" : "" } ], "returns" : { "type" : null, "description" : "" }, "errors" : [  ] }, { "isDocumented" : false, "arity" : 1, "name" : "build-nodes", "qname" : "model:build-nodes", "signature" : "($definition)", "description" : "", "summary" : "", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "definition", "type" : null, "occurrence" : null, "description" : "" } ], "returns" : { "type" : null, "description" : "" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "collection-qname", "qname" : "model:collection-qname", "signature" : "($name as xs:string) as xs:QName", "description" : " Returns the QName for a collection. Returns the empty sequence if no collection with the given name exists.\n", "summary" : "<p> Returns the QName for a collection.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "name", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the name of the collection as string</div>" } ], "returns" : { "type" : "xs:QName", "description" : "the QName for the collection or the empty sequence" }, "errors" : [  ] }, { "isDocumented" : false, "arity" : 2, "name" : "delete-node-collection", "qname" : "model:delete-node-collection", "signature" : "($collection as xs:QName, $noderef as xs:anyURI?)", "description" : "", "summary" : "", "annotation_str" : "", "annotations" : [  ], "updating" : true, "parameters" : [ { "name" : "collection", "type" : "xs:QName", "occurrence" : null, "description" : "" }, { "name" : "noderef", "type" : "xs:anyURI", "occurrence" : "?", "description" : "" } ], "returns" : { "type" : null, "description" : "" }, "errors" : [  ] }, { "isDocumented" : false, "arity" : 1, "name" : "delete-node", "qname" : "model:delete-node", "signature" : "($noderef as xs:anyURI)", "description" : "", "summary" : "", "annotation_str" : "", "annotations" : [  ], "updating" : true, "parameters" : [ { "name" : "noderef", "type" : "xs:anyURI", "occurrence" : null, "description" : "" } ], "returns" : { "type" : null, "description" : "" }, "errors" : [  ] }, { "isDocumented" : false, "arity" : 1, "name" : "element-node", "qname" : "model:element-node", "signature" : "($definition as schema-element(xdm:element))", "description" : "", "summary" : "", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "definition", "type" : "schema-element(xdm:element)", "occurrence" : null, "description" : "" } ], "returns" : { "type" : null, "description" : "" }, "errors" : [  ] }, { "isDocumented" : false, "arity" : 1, "name" : "find-keys", "qname" : "model:find-keys", "signature" : "($collection)", "description" : "", "summary" : "", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "collection", "type" : null, "occurrence" : null, "description" : "" } ], "returns" : { "type" : null, "description" : "" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 0, "name" : "get-collection-names", "qname" : "model:get-collection-names", "signature" : "() as xs:string*", "description" : " Returns all collection names\n", "summary" : "<p> Returns all collection names\n</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [  ], "returns" : { "type" : "xs:string*", "description" : "collection names as a sequence of strings" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "get-collection-size", "qname" : "model:get-collection-size", "signature" : "($name as xs:string) as xs:integer", "description" : " Returns the size of a collection\n", "summary" : "<p> Returns the size of a collection\n</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "name", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the name of the collection as string</div>" } ], "returns" : { "type" : "xs:integer", "description" : "the size of the collection" }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">if the collection chosen by $name does not exist</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 1, "name" : "get-collection", "qname" : "model:get-collection", "signature" : "($name as xs:string) as node()*", "description" : " Returns the whole collection with a given name\n", "summary" : "<p> Returns the whole collection with a given name\n</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "name", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> name of collection to return</div>" } ], "returns" : { "type" : "node()*", "description" : "sequence of nodes of the collection" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 4, "name" : "get-collection", "qname" : "model:get-collection", "signature" : "($name as xs:string, $first-noderef as xs:anyURI?, $offset as xs:integer?, $limit as xs:integer)", "description" : " Returns \"a page\" of the collection (a part)\n", "summary" : "<p> Returns \"a page\" of the collection (a part)\n</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "name", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the name of the collection as string</div>" }, { "name" : "first-noderef", "type" : "xs:anyURI", "occurrence" : "?", "description" : "" }, { "name" : "offset", "type" : "xs:integer", "occurrence" : "?", "description" : "" }, { "name" : "limit", "type" : "xs:integer", "occurrence" : null, "description" : "" } ], "returns" : { "type" : null, "description" : "a sequence of nodes from the collection" }, "errors" : [  ] }, { "isDocumented" : false, "arity" : 4, "name" : "get-column-id", "qname" : "model:get-column-id", "signature" : "($basepath as xs:string?, $name as xs:string, $isattribute as xs:boolean, $columns) as xs:integer", "description" : "", "summary" : "", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "basepath", "type" : "xs:string", "occurrence" : "?", "description" : "" }, { "name" : "name", "type" : "xs:string", "occurrence" : null, "description" : "" }, { "name" : "isattribute", "type" : "xs:boolean", "occurrence" : null, "description" : "" }, { "name" : "columns", "type" : null, "occurrence" : null, "description" : "" } ], "returns" : { "type" : "xs:integer", "description" : "" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 0, "name" : "get-index-names", "qname" : "model:get-index-names", "signature" : "() as xs:string*", "description" : " Returns all index names\n", "summary" : "<p> Returns all index names\n</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [  ], "returns" : { "type" : "xs:string*", "description" : "index names as a sequence of strings" }, "errors" : [  ] }, { "isDocumented" : false, "arity" : 2, "name" : "get-index-point", "qname" : "model:get-index-point", "signature" : "($index-name as xs:string, $index-key)", "description" : "", "summary" : "", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "index-name", "type" : "xs:string", "occurrence" : null, "description" : "" }, { "name" : "index-key", "type" : null, "occurrence" : null, "description" : "" } ], "returns" : { "type" : null, "description" : "" }, "errors" : [  ] }, { "isDocumented" : false, "arity" : 1, "name" : "get-node-group", "qname" : "model:get-node-group", "signature" : "($noderef as xs:anyURI)", "description" : "", "summary" : "", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "noderef", "type" : "xs:anyURI", "occurrence" : null, "description" : "" } ], "returns" : { "type" : null, "description" : "" }, "errors" : [  ] }, { "isDocumented" : false, "arity" : 1, "name" : "get-node-path", "qname" : "model:get-node-path", "signature" : "($node)", "description" : "", "summary" : "", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "node", "type" : null, "occurrence" : null, "description" : "" } ], "returns" : { "type" : null, "description" : "" }, "errors" : [  ] }, { "isDocumented" : false, "arity" : 1, "name" : "get-node", "qname" : "model:get-node", "signature" : "($noderef as xs:anyURI)", "description" : "", "summary" : "", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "noderef", "type" : "xs:anyURI", "occurrence" : null, "description" : "" } ], "returns" : { "type" : null, "description" : "" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "index-qname", "qname" : "model:index-qname", "signature" : "($name as xs:string) as xs:QName", "description" : " Returns the QName for an index. Returns the empty sequence if no index with the given name exists.\n", "summary" : "<p> Returns the QName for an index.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "name", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the name of the index as string</div>" } ], "returns" : { "type" : "xs:QName", "description" : "the QName for the index or the empty sequence" }, "errors" : [  ] }, { "isDocumented" : false, "arity" : 3, "name" : "insert-attribute", "qname" : "model:insert-attribute", "signature" : "($noderef as xs:anyURI, $attribute as xs:string, $value)", "description" : "", "summary" : "", "annotation_str" : "", "annotations" : [  ], "updating" : true, "parameters" : [ { "name" : "noderef", "type" : "xs:anyURI", "occurrence" : null, "description" : "" }, { "name" : "attribute", "type" : "xs:string", "occurrence" : null, "description" : "" }, { "name" : "value", "type" : null, "occurrence" : null, "description" : "" } ], "returns" : { "type" : null, "description" : "" }, "errors" : [  ] }, { "isDocumented" : false, "arity" : 4, "name" : "insert-node-collection-ordered", "qname" : "model:insert-node-collection-ordered", "signature" : "($collection as xs:QName, $noderef as xs:anyURI?, $position as xs:string, $nodes)", "description" : "", "summary" : "", "annotation_str" : "", "annotations" : [  ], "updating" : true, "parameters" : [ { "name" : "collection", "type" : "xs:QName", "occurrence" : null, "description" : "" }, { "name" : "noderef", "type" : "xs:anyURI", "occurrence" : "?", "description" : "" }, { "name" : "position", "type" : "xs:string", "occurrence" : null, "description" : "" }, { "name" : "nodes", "type" : null, "occurrence" : null, "description" : "" } ], "returns" : { "type" : null, "description" : "" }, "errors" : [  ] }, { "isDocumented" : false, "arity" : 4, "name" : "insert-node-collection", "qname" : "model:insert-node-collection", "signature" : "($collection as xs:QName, $noderef as xs:anyURI?, $position as xs:string, $nodes)", "description" : "", "summary" : "", "annotation_str" : "", "annotations" : [  ], "updating" : true, "parameters" : [ { "name" : "collection", "type" : "xs:QName", "occurrence" : null, "description" : "" }, { "name" : "noderef", "type" : "xs:anyURI", "occurrence" : "?", "description" : "" }, { "name" : "position", "type" : "xs:string", "occurrence" : null, "description" : "" }, { "name" : "nodes", "type" : null, "occurrence" : null, "description" : "" } ], "returns" : { "type" : null, "description" : "" }, "errors" : [  ] }, { "isDocumented" : false, "arity" : 5, "name" : "insert-node-collection", "qname" : "model:insert-node-collection", "signature" : "($collection as xs:QName, $noderef as xs:anyURI?, $position as xs:string, $nodes, $validate as xs:boolean)", "description" : "", "summary" : "", "annotation_str" : "", "annotations" : [  ], "updating" : true, "parameters" : [ { "name" : "collection", "type" : "xs:QName", "occurrence" : null, "description" : "" }, { "name" : "noderef", "type" : "xs:anyURI", "occurrence" : "?", "description" : "" }, { "name" : "position", "type" : "xs:string", "occurrence" : null, "description" : "" }, { "name" : "nodes", "type" : null, "occurrence" : null, "description" : "" }, { "name" : "validate", "type" : "xs:boolean", "occurrence" : null, "description" : "" } ], "returns" : { "type" : null, "description" : "" }, "errors" : [  ] }, { "isDocumented" : false, "arity" : 3, "name" : "insert-node", "qname" : "model:insert-node", "signature" : "($noderef as xs:anyURI, $position as xs:string, $newnodes)", "description" : "", "summary" : "", "annotation_str" : "", "annotations" : [  ], "updating" : true, "parameters" : [ { "name" : "noderef", "type" : "xs:anyURI", "occurrence" : null, "description" : "" }, { "name" : "position", "type" : "xs:string", "occurrence" : null, "description" : "" }, { "name" : "newnodes", "type" : null, "occurrence" : null, "description" : "" } ], "returns" : { "type" : null, "description" : "" }, "errors" : [  ] }, { "isDocumented" : false, "arity" : 4, "name" : "make-table-row", "qname" : "model:make-table-row", "signature" : "($idx, $node, $columns, $namespaces)", "description" : "", "summary" : "", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "idx", "type" : null, "occurrence" : null, "description" : "" }, { "name" : "node", "type" : null, "occurrence" : null, "description" : "" }, { "name" : "columns", "type" : null, "occurrence" : null, "description" : "" }, { "name" : "namespaces", "type" : null, "occurrence" : null, "description" : "" } ], "returns" : { "type" : null, "description" : "" }, "errors" : [  ] }, { "isDocumented" : false, "arity" : 5, "name" : "make-table-row", "qname" : "model:make-table-row", "signature" : "($node, $path as xs:string?, $row, $columns, $namespaces)", "description" : "", "summary" : "", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "node", "type" : null, "occurrence" : null, "description" : "" }, { "name" : "path", "type" : "xs:string", "occurrence" : "?", "description" : "" }, { "name" : "row", "type" : null, "occurrence" : null, "description" : "" }, { "name" : "columns", "type" : null, "occurrence" : null, "description" : "" }, { "name" : "namespaces", "type" : null, "occurrence" : null, "description" : "" } ], "returns" : { "type" : null, "description" : "" }, "errors" : [  ] }, { "isDocumented" : false, "arity" : 2, "name" : "make-table", "qname" : "model:make-table", "signature" : "($nodes, $namespaces)", "description" : "", "summary" : "", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "nodes", "type" : null, "occurrence" : null, "description" : "" }, { "name" : "namespaces", "type" : null, "occurrence" : null, "description" : "" } ], "returns" : { "type" : null, "description" : "" }, "errors" : [  ] }, { "isDocumented" : false, "arity" : 2, "name" : "node-name", "qname" : "model:node-name", "signature" : "($node as element(*), $namespaces) as xs:string", "description" : "", "summary" : "", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "node", "type" : "element(*)", "occurrence" : null, "description" : "" }, { "name" : "namespaces", "type" : null, "occurrence" : null, "description" : "" } ], "returns" : { "type" : "xs:string", "description" : "" }, "errors" : [  ] }, { "isDocumented" : false, "arity" : 2, "name" : "node-path-index", "qname" : "model:node-path-index", "signature" : "($node as node(), $parent as node()) as xs:integer?", "description" : "", "summary" : "", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "node", "type" : "node()", "occurrence" : null, "description" : "" }, { "name" : "parent", "type" : "node()", "occurrence" : null, "description" : "" } ], "returns" : { "type" : "xs:integer?", "description" : "" }, "errors" : [  ] }, { "isDocumented" : false, "arity" : 2, "name" : "node-prefix", "qname" : "model:node-prefix", "signature" : "($node as element(*), $namespaces) as xs:string?", "description" : "", "summary" : "", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "node", "type" : "element(*)", "occurrence" : null, "description" : "" }, { "name" : "namespaces", "type" : null, "occurrence" : null, "description" : "" } ], "returns" : { "type" : "xs:string?", "description" : "" }, "errors" : [  ] }, { "isDocumented" : false, "arity" : 1, "name" : "node-reference", "qname" : "model:node-reference", "signature" : "($node) as xs:string", "description" : "", "summary" : "", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "node", "type" : null, "occurrence" : null, "description" : "" } ], "returns" : { "type" : "xs:string", "description" : "" }, "errors" : [  ] }, { "isDocumented" : false, "arity" : 1, "name" : "node", "qname" : "model:node", "signature" : "($definition)", "description" : "", "summary" : "", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "definition", "type" : null, "occurrence" : null, "description" : "" } ], "returns" : { "type" : null, "description" : "" }, "errors" : [  ] }, { "isDocumented" : false, "arity" : 2, "name" : "rename-node", "qname" : "model:rename-node", "signature" : "($noderef as xs:anyURI, $name as xs:QName)", "description" : "", "summary" : "", "annotation_str" : "", "annotations" : [  ], "updating" : true, "parameters" : [ { "name" : "noderef", "type" : "xs:anyURI", "occurrence" : null, "description" : "" }, { "name" : "name", "type" : "xs:QName", "occurrence" : null, "description" : "" } ], "returns" : { "type" : null, "description" : "" }, "errors" : [  ] }, { "isDocumented" : false, "arity" : 2, "name" : "replace-node", "qname" : "model:replace-node", "signature" : "($noderef as xs:anyURI, $nodes)", "description" : "", "summary" : "", "annotation_str" : "", "annotations" : [  ], "updating" : true, "parameters" : [ { "name" : "noderef", "type" : "xs:anyURI", "occurrence" : null, "description" : "" }, { "name" : "nodes", "type" : null, "occurrence" : null, "description" : "" } ], "returns" : { "type" : null, "description" : "" }, "errors" : [  ] }, { "isDocumented" : false, "arity" : 2, "name" : "replace-value", "qname" : "model:replace-value", "signature" : "($noderef as xs:anyURI, $value)", "description" : "", "summary" : "", "annotation_str" : "", "annotations" : [  ], "updating" : true, "parameters" : [ { "name" : "noderef", "type" : "xs:anyURI", "occurrence" : null, "description" : "" }, { "name" : "value", "type" : null, "occurrence" : null, "description" : "" } ], "returns" : { "type" : null, "description" : "" }, "errors" : [  ] }, { "isDocumented" : false, "arity" : 2, "name" : "resolve-qname", "qname" : "model:resolve-qname", "signature" : "($name as xs:QName, $namespaces) as xs:string", "description" : "", "summary" : "", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "name", "type" : "xs:QName", "occurrence" : null, "description" : "" }, { "name" : "namespaces", "type" : null, "occurrence" : null, "description" : "" } ], "returns" : { "type" : "xs:string", "description" : "" }, "errors" : [  ] }, { "isDocumented" : false, "arity" : 1, "name" : "type-save", "qname" : "model:type-save", "signature" : "($val) as xs:string", "description" : "", "summary" : "", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "val", "type" : null, "occurrence" : null, "description" : "" } ], "returns" : { "type" : "xs:string", "description" : "" }, "errors" : [  ] }, { "isDocumented" : false, "arity" : 1, "name" : "type", "qname" : "model:type", "signature" : "($val as xs:anyAtomicType?) as xs:string", "description" : "", "summary" : "", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "val", "type" : "xs:anyAtomicType", "occurrence" : "?", "description" : "" } ], "returns" : { "type" : "xs:string", "description" : "" }, "errors" : [  ] }, { "isDocumented" : false, "arity" : 1, "name" : "unused-prefix", "qname" : "model:unused-prefix", "signature" : "($namespaces) as xs:string", "description" : "", "summary" : "", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "namespaces", "type" : null, "occurrence" : null, "description" : "" } ], "returns" : { "type" : "xs:string", "description" : "" }, "errors" : [  ] } ], "variables" : [  ] }, "http://api.28.io/validation" : { "ns" : "http://api.28.io/validation", "description" : "", "sees" : [  ], "authors" : [  ], "version" : null, "encoding" : null, "namespaces" : [ { "uri" : "http://zorba.io/modules/reflection", "prefix" : "reflection" }, { "uri" : "http://www.28msec.com/modules/http/response", "prefix" : "resp" }, { "uri" : "http://api.28.io/validation", "prefix" : "validate" } ], "functions" : [ { "isDocumented" : false, "arity" : 3, "name" : "by-schema-array", "qname" : "validate:by-schema-array", "signature" : "($obj, $schema as array(), $path as xs:string)", "description" : "", "summary" : "", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "obj", "type" : null, "occurrence" : null, "description" : "" }, { "name" : "schema", "type" : "array()", "occurrence" : null, "description" : "" }, { "name" : "path", "type" : "xs:string", "occurrence" : null, "description" : "" } ], "returns" : { "type" : null, "description" : "" }, "errors" : [  ] }, { "isDocumented" : false, "arity" : 3, "name" : "by-schema-obj", "qname" : "validate:by-schema-obj", "signature" : "($obj as object(), $schema as object(), $path as xs:string)", "description" : "", "summary" : "", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "obj", "type" : "object()", "occurrence" : null, "description" : "" }, { "name" : "schema", "type" : "object()", "occurrence" : null, "description" : "" }, { "name" : "path", "type" : "xs:string", "occurrence" : null, "description" : "" } ], "returns" : { "type" : null, "description" : "" }, "errors" : [  ] }, { "isDocumented" : false, "arity" : 3, "name" : "by-schema-primitive", "qname" : "validate:by-schema-primitive", "signature" : "($obj, $schema-type as xs:string, $path)", "description" : "", "summary" : "", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "obj", "type" : null, "occurrence" : null, "description" : "" }, { "name" : "schema-type", "type" : "xs:string", "occurrence" : null, "description" : "" }, { "name" : "path", "type" : null, "occurrence" : null, "description" : "" } ], "returns" : { "type" : null, "description" : "" }, "errors" : [  ] }, { "isDocumented" : false, "arity" : 2, "name" : "by-schema", "qname" : "validate:by-schema", "signature" : "($obj as object(), $schema as object())", "description" : "", "summary" : "", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "obj", "type" : "object()", "occurrence" : null, "description" : "" }, { "name" : "schema", "type" : "object()", "occurrence" : null, "description" : "" } ], "returns" : { "type" : null, "description" : "" }, "errors" : [  ] }, { "isDocumented" : false, "arity" : 2, "name" : "expression", "qname" : "validate:expression", "signature" : "($expression as xs:string, $is-domain-expr as xs:boolean) as xs:boolean", "description" : "", "summary" : "", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "expression", "type" : "xs:string", "occurrence" : null, "description" : "" }, { "name" : "is-domain-expr", "type" : "xs:boolean", "occurrence" : null, "description" : "" } ], "returns" : { "type" : "xs:boolean", "description" : "" }, "errors" : [  ] }, { "isDocumented" : false, "arity" : 1, "name" : "type", "qname" : "validate:type", "signature" : "($type as xs:string) as xs:boolean", "description" : "", "summary" : "", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "type", "type" : "xs:string", "occurrence" : null, "description" : "" } ], "returns" : { "type" : "xs:boolean", "description" : "" }, "errors" : [  ] } ], "variables" : [  ] }, "http://api.28.io/sandbox" : { "ns" : "http://api.28.io/sandbox", "description" : "", "sees" : [  ], "authors" : [  ], "version" : null, "encoding" : null, "namespaces" : [ { "uri" : "http://zorba.io/modules/reference", "prefix" : "ref" }, { "uri" : "http://zorba.io/modules/reflection", "prefix" : "reflection" }, { "uri" : "http://api.28.io/sandbox", "prefix" : "sandbox" } ], "functions" : [ { "isDocumented" : false, "arity" : 3, "name" : "filter", "qname" : "sandbox:filter", "signature" : "($nodes, $xpath as xs:string, $namespaces)", "description" : "", "summary" : "", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "nodes", "type" : null, "occurrence" : null, "description" : "" }, { "name" : "xpath", "type" : "xs:string", "occurrence" : null, "description" : "" }, { "name" : "namespaces", "type" : null, "occurrence" : null, "description" : "" } ], "returns" : { "type" : null, "description" : "" }, "errors" : [  ] }, { "isDocumented" : false, "arity" : 3, "name" : "get-names", "qname" : "sandbox:get-names", "signature" : "($nodes, $path as xs:string, $namespaces)", "description" : "", "summary" : "", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "nodes", "type" : null, "occurrence" : null, "description" : "" }, { "name" : "path", "type" : "xs:string", "occurrence" : null, "description" : "" }, { "name" : "namespaces", "type" : null, "occurrence" : null, "description" : "" } ], "returns" : { "type" : null, "description" : "" }, "errors" : [  ] }, { "isDocumented" : false, "arity" : 3, "name" : "get-values", "qname" : "sandbox:get-values", "signature" : "($nodes, $path as xs:string, $namespaces)", "description" : "", "summary" : "", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "nodes", "type" : null, "occurrence" : null, "description" : "" }, { "name" : "path", "type" : "xs:string", "occurrence" : null, "description" : "" }, { "name" : "namespaces", "type" : null, "occurrence" : null, "description" : "" } ], "returns" : { "type" : null, "description" : "" }, "errors" : [  ] }, { "isDocumented" : false, "arity" : 3, "name" : "key-constraint", "qname" : "sandbox:key-constraint", "signature" : "($nodes, $condition as xs:string, $namespaces)", "description" : "", "summary" : "", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "nodes", "type" : null, "occurrence" : null, "description" : "" }, { "name" : "condition", "type" : "xs:string", "occurrence" : null, "description" : "" }, { "name" : "namespaces", "type" : null, "occurrence" : null, "description" : "" } ], "returns" : { "type" : null, "description" : "" }, "errors" : [  ] }, { "isDocumented" : false, "arity" : 1, "name" : "ns-declarations", "qname" : "sandbox:ns-declarations", "signature" : "($namespaces) as xs:string", "description" : "", "summary" : "", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "namespaces", "type" : null, "occurrence" : null, "description" : "" } ], "returns" : { "type" : "xs:string", "description" : "" }, "errors" : [  ] }, { "isDocumented" : false, "arity" : 4, "name" : "order", "qname" : "sandbox:order", "signature" : "($nodes, $path as xs:string, $descending as xs:boolean, $namespaces)", "description" : "", "summary" : "", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "nodes", "type" : null, "occurrence" : null, "description" : "" }, { "name" : "path", "type" : "xs:string", "occurrence" : null, "description" : "" }, { "name" : "descending", "type" : "xs:boolean", "occurrence" : null, "description" : "" }, { "name" : "namespaces", "type" : null, "occurrence" : null, "description" : "" } ], "returns" : { "type" : null, "description" : "" }, "errors" : [  ] }, { "isDocumented" : false, "arity" : 3, "name" : "tuple-constraint", "qname" : "sandbox:tuple-constraint", "signature" : "($nodes, $condition as xs:string, $namespaces)", "description" : "", "summary" : "", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "nodes", "type" : null, "occurrence" : null, "description" : "" }, { "name" : "condition", "type" : "xs:string", "occurrence" : null, "description" : "" }, { "name" : "namespaces", "type" : null, "occurrence" : null, "description" : "" } ], "returns" : { "type" : null, "description" : "" }, "errors" : [  ] } ], "variables" : [  ] }, "http://zorba.io/modules/store/static/collections/dml" : { "ns" : "http://zorba.io/modules/store/static/collections/dml", "description" : " This modules provides a set of functions to modify a collection\n and retrieve the items contained in a particular collection.\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n This module is part of\n <a xmlns:xqdoc=\"http://www.xqdoc.org/1.0\" href=\"../zorba/xqddf.html\">Zorba's XQuery Data Definition Facility</a>.\n All the collections managed by this module have to be pre-declared in the\n prolog of a module.\n Please refer to the\n <a xmlns:xqdoc=\"http://www.xqdoc.org/1.0\" href=\"../zorba/data_lifecycle.html\">general documentation</a>\n for more information and examples.\n", "sees" : [ "<xqdoc:see xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"><a href=\"../zorba/data_lifecycle.html\">Data Lifecycle</a></xqdoc:see>", "<xqdoc:see xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"><a href=\"../zorba/xqddf.html\">XQuery Data Definition Facility</a></xqdoc:see>", "<xqdoc:see xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">http://zorba.io/modules/store/static/collections/ddl</xqdoc:see>", "<xqdoc:see xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">http://zorba.io/modules/store/static/indexes/ddl</xqdoc:see>", "<xqdoc:see xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">http://zorba.io/modules/store/static/indexes/dml</xqdoc:see>", "<xqdoc:see xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">http://zorba.io/modules/store/static/integrity-constraints/ddl</xqdoc:see>", "<xqdoc:see xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">http://zorba.io/modules/store/static/integrity-constraints/dml</xqdoc:see>", "<xqdoc:see xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">http://zorba.io/errors</xqdoc:see>" ], "authors" : [ "<xqdoc:author xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Nicolae Brinza, Matthias Brantner, David Graf, Till Westmann, Markos Zaharioudakis</xqdoc:author>" ], "version" : null, "encoding" : "utf-8", "namespaces" : [ { "uri" : "http://zorba.io/annotations", "prefix" : "an" }, { "uri" : "http://zorba.io/modules/store/static/collections/dml", "prefix" : "cdml" }, { "uri" : "http://zorba.io/options/versioning", "prefix" : "ver" }, { "uri" : "http://zorba.io/errors", "prefix" : "zerr" } ], "functions" : [ { "isDocumented" : true, "arity" : 3, "name" : "apply-insert-after", "qname" : "cdml:apply-insert-after", "signature" : "($name as xs:QName, $pos as item(), $content as item()*) as item()* external", "description" : " This function does the same thing as the <code xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">insert-after()</code>\n function except it immediately applies the resulting pending updates and\n returns the items that have been inserted.\n", "summary" : "<p> This function does the same thing as the  insert-after() \n function except it immediately applies the resulting pending updates and\n returns the items that have been inserted.</p>", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "name", "type" : "xs:QName", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The name of the collection to insert into.</div>" }, { "name" : "pos", "type" : "item()", "occurrence" : null, "description" : "" }, { "name" : "content", "type" : "item()", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The sequence of items whose copies to insert.</div>" } ], "returns" : { "type" : "item()*", "description" : "The sequence of items that have been inserted." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">zerr:ZDDY0001 if the collection is not declared.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">zerr:ZDDY0003 if the collection is not available.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">zerr:ZDDY0006 if the collection is const, append-only, or queue.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">zerr:ZDDY0012 if the collection is unordered.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">zerr:ZDTY0001 if <code>$content</code> does not match the expected type as specified in the collection declaration according to the rules for SequenceType Matching.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">zerr:ZDDY0011 if <code>$target</code> is not in the collection.</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 3, "name" : "apply-insert-before", "qname" : "cdml:apply-insert-before", "signature" : "($name as xs:QName, $target as item(), $content as item()*) as item()* external", "description" : " This function does the same thing as <code xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">insert-before()</code> except it\n immediately applies the resulting pending updates and returns the items that\n have been inserted.\n", "summary" : "<p> This function does the same thing as  insert-before()  except it\n immediately applies the resulting pending updates and returns the items that\n have been inserted.</p>", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "name", "type" : "xs:QName", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The name of the collection to insert into.</div>" }, { "name" : "target", "type" : "item()", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The item in the collection before which <code>$content</code> will be inserted.</div>" }, { "name" : "content", "type" : "item()", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The sequence of items whose copies to insert.</div>" } ], "returns" : { "type" : "item()*", "description" : "The sequence of items that have been inserted." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">zerr:ZDDY0001 if the collection is not declared.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">zerr:ZDDY0003 if the collection is not available.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">zerr:ZDDY0006 if the collection $name is const, append-only, or queue.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">zerr:ZDDY0012 if the collection is unordered.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">zerr:ZDTY0001 if <code>$content</code> does not match the expected type as specified in the collection declaration according to the rules for SequenceType Matching.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">zerr:ZDDY0011 if <code>$target</code> is not an item that is contained in the collection.</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 2, "name" : "apply-insert-first", "qname" : "cdml:apply-insert-first", "signature" : "($name as xs:QName, $content as item()*) as item()* external", "description" : " This function does the same thing as <code xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">insert-first()</code> except it\n immediately applies the resulting pending updates and returns the items that\n have been inserted.\n", "summary" : "<p> This function does the same thing as  insert-first()  except it\n immediately applies the resulting pending updates and returns the items that\n have been inserted.</p>", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "name", "type" : "xs:QName", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The name of the collection to insert into.</div>" }, { "name" : "content", "type" : "item()", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The sequence of items whose copies to insert.</div>" } ], "returns" : { "type" : "item()*", "description" : "The Sequence of items that have been inserted." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">zerr:ZDDY0001 if the collection is not declared.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">zerr:ZDDY0003 if the collection is not available.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">zerr:ZDDY0006 if the collection $name is append-only, const, or queue.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">zerr:ZDDY0012 if the collection is unordered.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">zerr:ZDTY0001 if <code>$content</code> does not match the expected type as specified in the collection declaration according to the rules for SequenceType Matching.</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 2, "name" : "apply-insert-last", "qname" : "cdml:apply-insert-last", "signature" : "($name as xs:QName, $content as item()*) as item()* external", "description" : " This function does the same thing as <code xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">insert-last()</code> except it\n immediately applies the resulting pending updates and returns the items that\n have been inserted.\n", "summary" : "<p> This function does the same thing as  insert-last()  except it\n immediately applies the resulting pending updates and returns the items that\n have been inserted.</p>", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "name", "type" : "xs:QName", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The name of the collection to insert into.</div>" }, { "name" : "content", "type" : "item()", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The sequence of items whose copies to insert.</div>" } ], "returns" : { "type" : "item()*", "description" : "The sequence of items that have been inserted." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">zerr:ZDDY0001 if the collection is not declared.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">zerr:ZDDY0003 if the collection is not available.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">zerr:ZDDY0006 if the collection is const.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">zerr:ZDDY0012 if the collection is unordered.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">zerr:ZDTY0001 if <code>$content</code> does not match the expected type as specified in the collection declaration according to the rules for SequenceType Matching.</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 2, "name" : "apply-insert", "qname" : "cdml:apply-insert", "signature" : "($name as xs:QName, $content as item()*) as item()* external", "description" : " This function does the same thing as <code xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">insert()</code> except it\n immediately applies the resulting pending updates and returns the items that\n have been inserted.\n", "summary" : "<p> This function does the same thing as  insert()  except it\n immediately applies the resulting pending updates and returns the items that\n have been inserted.</p>", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "name", "type" : "xs:QName", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The name of the collection to insert into.</div>" }, { "name" : "content", "type" : "item()", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The sequence of items whose copies to insert.</div>" } ], "returns" : { "type" : "item()*", "description" : "The sequence of items that have been inserted." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">zerr:ZDDY0001 if the collection is not declared.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">zerr:ZDDY0003 if the collection is not available.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">zerr:ZDDY0006 if the collection is append-only, const, or queue.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">zerr:ZDTY0001 if <code>$content</code> does not match the expected type as specified in the collection declaration according to the rules for SequenceType Matching.</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 1, "name" : "collection-name", "qname" : "cdml:collection-name", "signature" : "($item as item()) as xs:QName external", "description" : " Gets the name of the collection the given item (node or JSON item) belongs\n to.\n", "summary" : "<p> Gets the name of the collection the given item (node or JSON item) belongs\n to.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "item", "type" : "item()", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The item for which to get the name of its collection.</div>" } ], "returns" : { "type" : "xs:QName", "description" : "The name of the collection to which the given item belongs." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">zerr:ZDDY0011 if <code>$item</code> does not belong to a collection.</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 1, "name" : "collection", "qname" : "cdml:collection", "signature" : "($name as xs:QName) as item()* external", "description" : " Gets the sequence of nodes or JSON items from a collection.\n", "summary" : "<p> Gets the sequence of nodes or JSON items from a collection.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "name", "type" : "xs:QName", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The name of the collection.</div>" } ], "returns" : { "type" : "item()*", "description" : "The seqnence of items from the collection." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">zerr:ZDDY0001 if the collection is not declared.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">zerr:ZDDY0003 if the collection is not available.</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 2, "name" : "collection", "qname" : "cdml:collection", "signature" : "($name as xs:QName, $skip as xs:integer) as item()* external", "description" : " Gets the sequence of nodes or JSON items from a collection.\n", "summary" : "<p> Gets the sequence of nodes or JSON items from a collection.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "name", "type" : "xs:QName", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The name of the collection.</div>" }, { "name" : "skip", "type" : "xs:integer", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The initial number of items to skip.</div>" } ], "returns" : { "type" : "item()*", "description" : "The (sub)sequence of items from the collection." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">zerr:ZDDY0001 if the collection is not declared.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">zerr:ZDDY0003 if the collection is not available.</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 3, "name" : "collection", "qname" : "cdml:collection", "signature" : "($name as xs:QName, $start as xs:anyURI, $skip as xs:integer) as item()* external", "description" : " Gets the sequence of items (nodes or JSON items) from a collection.\n The parameters <code xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">$start</code> and <code xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">$skip</code> can be used to\n skip over some items at the beginning of the collection.\n If both are given, both are applied:\n first <code xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">$start</code> to skip to the referenced item\n and then <code xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">$skip</code> to skip that additional number of items.\n", "summary" : "<p> Gets the sequence of items (nodes or JSON items) from a collection.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "name", "type" : "xs:QName", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The name of the collection.</div>" }, { "name" : "start", "type" : "xs:anyURI", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The reference to the first item to return.</div>" }, { "name" : "skip", "type" : "xs:integer", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The number of additional items to skip.</div>" } ], "returns" : { "type" : "item()*", "description" : "The sub-sequence from the collection." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">zerr:ZAPI0028 If the given URI is not a valid node position computed by the <code>np:node-position</code> function.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">zerr:ZDDY0001 if the collection is not declared.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">zerr:ZDDY0003 if the collection is not available.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">zerr:ZSTR0066 if <code>$start</code> does not reference a node from the collection.</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 1, "name" : "delete-first", "qname" : "cdml:delete-first", "signature" : "($name as xs:QName) external", "description" : " Deletes the first item from a collection.\n", "summary" : "<p> Deletes the first item from a collection.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : true, "parameters" : [ { "name" : "name", "type" : "xs:QName", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The name of the collection to delete from.</div>" } ], "returns" : { "type" : null, "description" : "An empty XDM instance and a pending update list that, once applied, deletes the first item from the collection." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">zerr:ZDDY0001 if the collection is not declared.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">zerr:ZDDY0003 if the collection is not available.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">zerr:ZDDY0006 if the collection is const or append-only.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">zerr:ZDDY0012 if the collection is unordered.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">zerr:ZDDY0011 if the collection is empty.</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 2, "name" : "delete-first", "qname" : "cdml:delete-first", "signature" : "($name as xs:QName, $number as xs:integer) external", "description" : " Deletes the first <i xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">N</i> items from a collection.\n", "summary" : "<p> Deletes the first  N  items from a collection.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : true, "parameters" : [ { "name" : "name", "type" : "xs:QName", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The name of the collection to delete from.</div>" }, { "name" : "number", "type" : "xs:integer", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The number of items to delete.</div>" } ], "returns" : { "type" : null, "description" : "An empty XDM instance and a pending update list that, once applied, deletes the items from the collection." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">zerr:ZDDY0001 if the collection is not declared.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">zerr:ZDDY0003 if the collection is not available.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">zerr:ZDDY0006 if the collection is const or append-only.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">zerr:ZDDY0012 if the collection is unordered.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">zerr:ZDDY0011 if the collection contains less than <code>$number</code> items.</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 1, "name" : "delete-last", "qname" : "cdml:delete-last", "signature" : "($name as xs:QName) external", "description" : " Deletes the last item from a collection.\n", "summary" : "<p> Deletes the last item from a collection.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : true, "parameters" : [ { "name" : "name", "type" : "xs:QName", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The name of the collection to delete from.</div>" } ], "returns" : { "type" : null, "description" : "An empty XDM instance and a pending update list that, once applied, deletes the last item from the collection." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">zerr:ZDDY0001 if the collection is not declared.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">zerr:ZDDY0003 if the collection is not available.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">zerr:ZDDY0006 if the collection is const, append-only, or queue.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">zerr:ZDDY0012 if the collection is unordered.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">zerr:ZDDY0011 if the collection is empty.</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 2, "name" : "delete-last", "qname" : "cdml:delete-last", "signature" : "($name as xs:QName, $number as xs:integer) external", "description" : " Deletes the last <i xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">N</i> items from a collection.\n", "summary" : "<p> Deletes the last  N  items from a collection.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : true, "parameters" : [ { "name" : "name", "type" : "xs:QName", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The name of the collection to delete from.</div>" }, { "name" : "number", "type" : "xs:integer", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The number of items to delete.</div>" } ], "returns" : { "type" : null, "description" : "An empty XDM instance and a pending update list that, once applied, deletes the items." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">zerr:ZDDY0001 if the collection is not declared.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">zerr:ZDDY0003 if the collection is not available.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">zerr:ZDDY0012 if the collection is unordered.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">zerr:ZDDY0011 if the collection contains less than <code>$number</code> items.</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 1, "name" : "delete", "qname" : "cdml:delete", "signature" : "($items as item()*) external", "description" : " Deletes items (nodes or JSON items) from a collection.\n", "summary" : "<p> Deletes items (nodes or JSON items) from a collection.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : true, "parameters" : [ { "name" : "items", "type" : "item()", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The items in the collection to delete.</div>" } ], "returns" : { "type" : null, "description" : "An empty XDM instance and a pending update list that, once applied, deletes the items from their collections." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">zerr:ZDDY0001 if the collection is not declared.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">zerr:ZDDY0003 if the collection is not available.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">zerr:ZDDY0006 if the collection is const, append-only, or queue.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">zerr:ZDDY0011 if any item in <code>$items</code> is not a member of a collection or not all items belong to the same collection.</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 2, "name" : "edit", "qname" : "cdml:edit", "signature" : "($target as item(), $content as item()) external", "description" : " Edits the first supplied item so as to make it look exactly like a copy of\n the second supplied item while retaining its original identity.\n", "summary" : "<p> Edits the first supplied item so as to make it look exactly like a copy of\n the second supplied item while retaining its original identity.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : true, "parameters" : [ { "name" : "target", "type" : "item()", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The target item to be edited.</div>" }, { "name" : "content", "type" : "item()", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The item that serves as an edit goal.</div>" } ], "returns" : { "type" : null, "description" : "An empty XDM instance and a pending update list that, once applied, performs the edit." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">zerr:ZDDY0001 if the collection to which <code>$target</code> belongs is not declared.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">zerr:ZDDY0003 if the collection to which <code>$target</code> belongs is not available.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">zerr:ZDDY0006 if the collection to which <code>$target</code> belongs is append-only, const, or queue.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">zerr:ZDDY0017 if <code>$target</code> is not a member of a collection.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">zerr:ZDDY0037 if the collection is append-only.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">zerr:ZDDY0038 if the collection is a queue.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">zerr:ZDDY0039 if <code>$target</code> is not a root.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">zerr:ZDDY0040 if <code>$target</code> cannot be updated to match the content (for example, because the target is a node and the content is an object).</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">zerr:ZDTY0001 if <code>$content</code> does not match the expected type (as specified in the collection declaration) according to the rules for SequenceType Matching.</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 1, "name" : "index-of", "qname" : "cdml:index-of", "signature" : "($item as item()) as xs:integer external", "description" : " Gets the position of the given item (node or JSON item) within its\n collection.\n", "summary" : "<p> Gets the position of the given item (node or JSON item) within its\n collection.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "item", "type" : "item()", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The item to get the index of.</div>" } ], "returns" : { "type" : "xs:integer", "description" : "The position of <code xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">$item</code> in its collection." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">zerr:ZDDY0011 if <code>$item</code> does not belong to a collection.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">zerr:ZDDY0012 if the collection is unordered.</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 3, "name" : "insert-after", "qname" : "cdml:insert-after", "signature" : "($name as xs:QName, $target as item(), $content as item()*) external", "description" : " The insert-after function is an updating function that inserts copies of the\n given items (nodes or JSON items) into a collection at the position\n directly following the given target item.\n", "summary" : "<p> The insert-after function is an updating function that inserts copies of the\n given items (nodes or JSON items) into a collection at the position\n directly following the given target item.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : true, "parameters" : [ { "name" : "name", "type" : "xs:QName", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The name of the collection to insert into.</div>" }, { "name" : "target", "type" : "item()", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The item in the collection after which <code>$content</code> will be inserted.</div>" }, { "name" : "content", "type" : "item()", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The sequence of items whose copies to insert.</div>" } ], "returns" : { "type" : null, "description" : "An empty XDM instance and a pending update list that, once applied, inserts the items into the collection." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">zerr:ZDDY0001 if the collection is not declared.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">zerr:ZDDY0003 if the collection is not available.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">zerr:ZDDY0006 if the collection is const, append-only, or queue.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">zerr:ZDDY0012 if the collection is unordered.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">zerr:ZDTY0001 if <code>$content</code> does not match the expected type as specified in the collection declaration according to the rules for SequenceType Matching.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">zerr:ZDDY0011 if <code>$target</code> is not a node that is contained in the collection.</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 3, "name" : "insert-before", "qname" : "cdml:insert-before", "signature" : "($name as xs:QName, $target as item(), $content as item()*) external", "description" : " Inserts copies of the given items (nodes or JSON items) into a collection at\n the position directly preceding the given target item.\n", "summary" : "<p> Inserts copies of the given items (nodes or JSON items) into a collection at\n the position directly preceding the given target item.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : true, "parameters" : [ { "name" : "name", "type" : "xs:QName", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The name of the collection to insert into.</div>" }, { "name" : "target", "type" : "item()", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The item in the collection before which <code>$content</code> will be inserted.</div>" }, { "name" : "content", "type" : "item()", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The sequence of items whose copies to insert.</div>" } ], "returns" : { "type" : null, "description" : "An empty XDM instance and a pending update list that, once applied, inserts the items into the collection." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">zerr:ZDDY0001 if the collection is not declared.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">zerr:ZDDY0003 if the collection is not available.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">zerr:ZDDY0006 if the collection is const, append-only, or queue.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">zerr:ZDDY0012 if the collection is unordered.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">zerr:ZDTY0001 if <code>$content</code> does not match the expected type as specified in the collection declaration according to the rules for SequenceType Matching.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">zerr:ZDDY0011 if <code>$target</code> is not an item that is contained in the collection.</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 2, "name" : "insert-first", "qname" : "cdml:insert-first", "signature" : "($name as xs:QName, $content as item()*) external", "description" : " Inserts copies of the given items (nodes or JSON items) at the beginning of\n a collection.\n", "summary" : "<p> Inserts copies of the given items (nodes or JSON items) at the beginning of\n a collection.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : true, "parameters" : [ { "name" : "name", "type" : "xs:QName", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The name of the collection to insert into.</div>" }, { "name" : "content", "type" : "item()", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The sequence of items whose copies to insert.</div>" } ], "returns" : { "type" : null, "description" : "An empty XDM instance and a pending update list that, once applied, inserts the items into the collection." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">zerr:ZDDY0001 if the collection is not declared.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">zerr:ZDDY0003 if the collection is not available.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">zerr:ZDDY0006 if the collection is append-only, const, or queue.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">zerr:ZDDY0012 if the collection is unordered.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">zerr:ZDTY0001 if <code>$content</code> does not match the expected type as specified in the collection declaration according to the rules for SequenceType Matching.</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 2, "name" : "insert-last", "qname" : "cdml:insert-last", "signature" : "($name as xs:QName, $content as item()*) external", "description" : " Inserts copies of the given items (nodes or JSON items) at the end of a\n collection.\n", "summary" : "<p> Inserts copies of the given items (nodes or JSON items) at the end of a\n collection.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : true, "parameters" : [ { "name" : "name", "type" : "xs:QName", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The name of the collection to insert into.</div>" }, { "name" : "content", "type" : "item()", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The sequence of itemss whose copies to insert.</div>" } ], "returns" : { "type" : null, "description" : "An empty XDM instance and a pending update list that, once applied, inserts the items into the collection." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">zerr:ZDDY0001 if the collection is not declared.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">zerr:ZDDY0003 if the collection is not available.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">zerr:ZDDY0006 if the collection is const.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">zerr:ZDDY0012 if the collection is unordered.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">zerr:ZDTY0001 if <code>$content</code> does not match the expected type as specified in the collection declaration according to the rules for SequenceType Matching.</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 2, "name" : "insert", "qname" : "cdml:insert", "signature" : "($name as xs:QName, $content as item()*) external", "description" : " Inserts copies of the given items (nodes or JSON items) into a collection.\n Note that the insertion position of the items in the collection is not\n defined.\n", "summary" : "<p> Inserts copies of the given items (nodes or JSON items) into a collection.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : true, "parameters" : [ { "name" : "name", "type" : "xs:QName", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The name of the collection to insert into.</div>" }, { "name" : "content", "type" : "item()", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The sequence of items whose copies to insert.</div>" } ], "returns" : { "type" : null, "description" : "An empty XDM instance and a pending update list that, once applied, inserts the items into the collection." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">zerr:ZDDY0001 if the collection is not declared.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">zerr:ZDDY0003 if the collection is not available.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">zerr:ZDDY0006 if the collection is append-only, const, or queue.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">zerr:ZDTY0001 if <code>$content</code> does not match the expected type as specified in the collection declaration according to the rules for SequenceType Matching.</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 1, "name" : "truncate", "qname" : "cdml:truncate", "signature" : "($name as xs:QName) external", "description" : " Deletes the entire contents of collection.\n Please note that applying this function can not be undone in case\n an error happens during the application of the containing PUL.\n", "summary" : "<p> Deletes the entire contents of collection.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : true, "parameters" : [ { "name" : "name", "type" : "xs:QName", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The name of the collection whose content to delete.</div>" } ], "returns" : { "type" : null, "description" : "An empty XDM instance and a pending update list that, once applied, deletes the nodes." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">zerr:ZDDY0001 if the collection is not declared.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">zerr:ZDDY0003 if the collection is not available.</xqdoc:error>" ] } ], "variables" : [  ] }, "http://www.28msec.com/modules/http-request" : { "ns" : "http://www.28msec.com/modules/http-request", "description" : " The request module provides functions for accessing\n information contained in the HTTP request used to evaluate the current\n query. For example, the <tt xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">param-names</tt> function can be used to\n retrieve all the names of the parameters contained in a request.\n", "sees" : [  ], "authors" : [ "<xqdoc:author xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Matthias Brantner</xqdoc:author>" ], "version" : null, "encoding" : "utf-8", "namespaces" : [ { "uri" : "http://www.w3.org/2005/xqt-errors", "prefix" : "err" }, { "uri" : "http://www.28msec.com/modules/http/request#2.0", "prefix" : "req" }, { "uri" : "http://www.28msec.com/modules/http-request", "prefix" : "request" }, { "uri" : "http://zorba.io/options/versioning", "prefix" : "ver" } ], "functions" : [ { "isDocumented" : true, "arity" : 0, "name" : "binary-content", "qname" : "request:binary-content", "signature" : "() as xs:base64Binary", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Returns the content of the request as base64Binary.</p>\n", "summary" : "<p>  Returns the content of the request as base64Binary.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [  ], "returns" : { "type" : "xs:base64Binary", "description" : "The content of the request as base64Binary." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">request:no-binary-content if the content contained in the body of the request cannot be treated as binary because it is a request with multipart or url-encoded content.</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 1, "name" : "binary-part", "qname" : "request:binary-part", "signature" : "($ref as xs:string) as xs:base64Binary", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Returns the value of a part as base64Binary.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">A part is identified by a reference that is the value of a\n <tt>src</tt> field returned by the <tt>request:parts</tt> function.</p>\n", "summary" : "<p>  Returns the value of a part as base64Binary.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "ref", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the name of the part</div>" } ], "returns" : { "type" : "xs:base64Binary", "description" : "the value of the part as base64Binary" }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">request:invalid-part if the part with the given name ($ref) does not exist</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">request:non-multipart if the current request does not contain multipart content</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">request:invalid-multipart if the multipart content is invalid (e.g. the boundary is missing)</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 0, "name" : "content-length", "qname" : "request:content-length", "signature" : "() as xs:integer?", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Returns the length of the content in bytes.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">The value returned corresponds to the value of the HTTP\n content-length header. The function returns an empty sequence\n if this header does not exist in the request or its value\n could not be converted to item of type xs:integer</p>.\n", "summary" : "<p>  Returns the length of the content in bytes.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [  ], "returns" : { "type" : "xs:integer?", "description" : "The content-length in bytes of the content sent with this request or the empty sequence if the content-length header does not exist in the request." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 0, "name" : "content-type", "qname" : "request:content-type", "signature" : "() as xs:string?", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Returns the content-type of the data sent with this request.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Note that the content-type is only set for PUT and POST requests.</p>\n", "summary" : "<p>  Returns the content-type of the data sent with this request.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [  ], "returns" : { "type" : "xs:string?", "description" : "The content-type of the request if it is a PUT or POST request. Otherwise, it returns the empty sequence." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 0, "name" : "header-accept", "qname" : "request:header-accept", "signature" : "() as object()*", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Returns the values of the HTTP ACCEPT header.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">The data is returned as a sequence of objects\n as shown in the following example.</p>\n <pre xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">\n {\n   \"type\" : \"text\",\n   \"subtype\" : \"html\",\n   \"quality\" : 1\n }\n </pre>\n", "summary" : "<p>  Returns the values of the HTTP ACCEPT header.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [  ], "returns" : { "type" : "object()*", "description" : "The header values of the header ACCEPT or the empty sequence if the header is not contained in the request. The order of the returned objects reflects the order of the components in the header." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">request:invalid-header if the accept header cannot be parsed</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 0, "name" : "header-names", "qname" : "request:header-names", "signature" : "() as xs:string*", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Returns the names of all the HTTP headers in this request.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Header fields are colon-separated name-value pairs, terminated\n by a carriage return (CR) and line feed (LF) character sequence. The\n names and values of each header are allowed to consist of US-ASCII\n characters only.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">The names of the headers are returned using upper-case letters.\n If a header with the same name is contained multiple times in a request,\n its name is only returned once. The order of the names in the resulting\n sequence does not reflect the order of the headers in the request. If\n a header does not have a value, it is as if the header does not exist\n in the request.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Note that the header names user-agent and content-type are not\n returned by this function. They are returned by the corresponding\n functions of this module module\n (e.g. <a href=\"#user-agent-0\">user-agent</a>).</p>\n", "summary" : "<p>  Returns the names of all the HTTP headers in this request.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [  ], "returns" : { "type" : "xs:string*", "description" : "The names of the headers of this request or the empty sequence if no headers are contained in the request." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "header-value", "qname" : "request:header-value", "signature" : "($name as xs:string) as xs:string?", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Returns the value of the HTTP header with the given name.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Header fields are colon-separated name-value pairs, terminated\n by a carriage return (CR) and line feed (LF) character sequence. The\n names and values of each header are allowed to consist of US-ASCII\n characters only.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Please note that header names are considered case-insensitive.\n Also note, that only one value is returned if multiple headers with the\n same names exist in the request. This value is a comma-separated list\n of the values of the headers in the order in which the headers appeared\n in the request.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">All headers having a name that starts with SAUSALITO_ are reserved\n and will not be returned by this function.</p>\n", "summary" : "<p>  Returns the value of the HTTP header with the given name.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "name", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The header name for which the value should be returned.</div>" } ], "returns" : { "type" : "xs:string?", "description" : "The header value of the header with the <tt xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">$name</tt> argument or the empty sequence if no header with that name is contained in the request." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 0, "name" : "headers", "qname" : "request:headers", "signature" : "() as object()", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Returns an object containing the request's HTTP header names and\n values.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Header fields are colon-separated name-value pairs, terminated\n by a carriage return (CR) and line feed (LF) character sequence. The\n names and values of each header are allowed to consist of US-ASCII\n characters only.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">The structure of the object is as shown in the following example:\n <pre>\n </pre>\n </p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">All headers having a name that starts with SAUSALITO_ are reserved\n and will not be returned by this function.</p>\n", "summary" : "<p>  Returns an object containing the request's HTTP header names and\n values.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [  ], "returns" : { "type" : "object()", "description" : "The header value of the header with the <tt xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">$name</tt> argument or the empty sequence if no header with that name is contained in the request." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 0, "name" : "method-delete", "qname" : "request:method-delete", "signature" : "() as xs:boolean", "description" : " Returns true if the HTTP method of this request is DELETE.\n", "summary" : "<p> Returns true if the HTTP method of this request is DELETE.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [  ], "returns" : { "type" : "xs:boolean", "description" : "true if the HTTP method of this request is DELETE, false otherwise." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 0, "name" : "method-get", "qname" : "request:method-get", "signature" : "() as xs:boolean", "description" : " Returns true if the HTTP method of this request is GET.\n", "summary" : "<p> Returns true if the HTTP method of this request is GET.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [  ], "returns" : { "type" : "xs:boolean", "description" : "true if the HTTP method of this request is GET, false otherwise." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 0, "name" : "method-head", "qname" : "request:method-head", "signature" : "() as xs:boolean", "description" : " Returns true if the HTTP method of this request is HEAD.\n", "summary" : "<p> Returns true if the HTTP method of this request is HEAD.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [  ], "returns" : { "type" : "xs:boolean", "description" : "true if the HTTP method of this request is HEAD, false otherwise." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 0, "name" : "method-options", "qname" : "request:method-options", "signature" : "() as xs:boolean", "description" : " Returns true if the HTTP method of this request is OPTION.\n", "summary" : "<p> Returns true if the HTTP method of this request is OPTION.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [  ], "returns" : { "type" : "xs:boolean", "description" : "true if the HTTP method of this request is OPTION, false otherwise." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 0, "name" : "method-post", "qname" : "request:method-post", "signature" : "() as xs:boolean", "description" : " Returns true if the HTTP method of this request is POST.\n", "summary" : "<p> Returns true if the HTTP method of this request is POST.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [  ], "returns" : { "type" : "xs:boolean", "description" : "true if the HTTP method of this request is POST, false otherwise." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 0, "name" : "method-put", "qname" : "request:method-put", "signature" : "() as xs:boolean", "description" : " Returns true if the HTTP method of this request is PUT.\n", "summary" : "<p> Returns true if the HTTP method of this request is PUT.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [  ], "returns" : { "type" : "xs:boolean", "description" : "true if the HTTP method of this request is PUT, false otherwise." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 0, "name" : "method", "qname" : "request:method", "signature" : "() as xs:string", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Returns the name of the HTTP method used to make this request.</p>\n", "summary" : "<p>  Returns the name of the HTTP method used to make this request.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [  ], "returns" : { "type" : "xs:string", "description" : "The request method used to make this request (i.e. GET, POST, PUT, DELETE or HEAD)." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 0, "name" : "param-names", "qname" : "request:param-names", "signature" : "() as xs:string*", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Returns the names of the parameters contained in the current request.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Parameters are name-value pairs contained in the query string of the URL\n used to make this request. As defined in RFC 1738, the query string of a\n URL starts with a \"?\" character and ends with the character (if any).\n Additionally, such name-value pairs may be part of the request's body if\n it is a PUT or POST request and the content-type of the request is\n \"application/x-www-form-urlencoded\". Name-value pairs are separated\n using either the \"&amp;\" or the \";\" character.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">In general, the names and the values  are precent-encoded. This function\n does the decoding of the parameters, i.e. it returns the values being\n not percent-encoded.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Also, the names of each parameter (after being precent-decoded) are\n treated as UTF-8. Please see the <tt>http:param-names#1</tt> function\n for retrieving parameter names submitted using a encoding other than UTF-8.\n </p>\n", "summary" : "<p>  Returns the names of the parameters contained in the current request.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [  ], "returns" : { "type" : "xs:string*", "description" : "The names of all parameters in this request. The empty sequence is returned if there are none." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "param-names", "qname" : "request:param-names", "signature" : "($encoding as xs:string) as xs:string*", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Returns the names of the parameters contained in the current request.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">This function is similar to the <tt>request:param-names#0</tt> function.\n However, the names are treated (after precent-decoding) using the\n given encoding supplied as parameter. For example, parameters might\n be encoded using the ISO-8859-1 encoding.</p>\n", "summary" : "<p>  Returns the names of the parameters contained in the current request.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "encoding", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The encoding of the parameters in the request (e.g. ISO-8859-1).</div>" } ], "returns" : { "type" : "xs:string*", "description" : "The names of all parameters in this request. The empty sequence is returned if there are none." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">request:invalid-encoding if the given encoding is invalid or not supported.</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 1, "name" : "param-values", "qname" : "request:param-values", "signature" : "($name as xs:string) as xs:string*", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Returns a sequence of parameter values for the given parameter name\n which are contained in the URL's query string or the body of a POST or PUT\n request.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Parameters are name-value pairs contained in the query string of the URL\n used to make this request. As defined in RFC 1738, the query string of a\n URL starts with a \"?\" character and ends with the character (if any).\n Additionally, such name-value pairs may be part of the request's body if\n it is a PUT or POST request and the content-type of the request is\n \"application/x-www-form-urlencoded\". Name-value pairs are separated\n using either the \"&amp;\" or the \";\" character.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">In general, the names and the values  are precent-encoded. This function\n does the decoding of the parameters, i.e. it returns the values being\n not percent-encoded.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Also, the names and the values of each parameter (after being precent-decoded)\n are treated as UTF-8. Please see the <tt>http:param-values#3</tt> function\n for retrieving parameters submitted using a encoding other than UTF-8.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">This function returns the empty-sequence if no parameter with the\n given name exists in this request. If you want the function to return\n a default value other than the empty sequence, use the\n <tt>http:param-values#2</tt> function.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">A URL could contain the following query string:\n <tt>name1=value1&amp;name2=value2;name1=value3&amp;name3</tt>.\n <tt>name1=value1&amp;name2=value2&amp;name1=value3&amp;name3</tt>.\n The name value pairs in this query string are\n <ul>\n   <li> name: <tt>name1</tt>; values: <tt>value1</tt> and <tt>value3</tt></li>\n   <li> name: <tt>name2</tt>; value: <tt>value2</tt></li>\n   <li> name: <tt>name3</tt>; value: <tt/></li>\n </ul>\n </p>\n", "summary" : "<p>  Returns a sequence of parameter values for the given parameter name\n which are contained in the URL's query string or the body of a POST or PUT\n request.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "name", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The name of the parameter whose value(s) should be returned.</div>" } ], "returns" : { "type" : "xs:string*", "description" : "A sequence of values for the parameter with the given name. The empty sequence is returned if no parameter exists with the given name." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 2, "name" : "param-values", "qname" : "request:param-values", "signature" : "($name as xs:string, $default-values as xs:string*) as xs:string*", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Returns a sequence of parameter values for the given parameter name\n which are contained in the URL's query string or the body of a POST or PUT\n request.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">This function is similar to the <tt>request:param-values#1</tt> function.\n However, instead of returning the empty-sequence as a default value it returns\n the given default-values sequence if no parameter with the given name is found\n in this request.</p>\n", "summary" : "<p>  Returns a sequence of parameter values for the given parameter name\n which are contained in the URL's query string or the body of a POST or PUT\n request.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "name", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The name of the parameter whose value(s) should be returned.</div>" }, { "name" : "default-values", "type" : "xs:string", "occurrence" : "*", "description" : "" } ], "returns" : { "type" : "xs:string*", "description" : "A sequence of values for the parameter with the given name. The sequence given as $default-values parameter is returned if no parameter exists with the given name." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 3, "name" : "param-values", "qname" : "request:param-values", "signature" : "($name as xs:string, $default-values as xs:string*, $encoding as xs:string) as xs:string*", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Returns a sequence of parameter values for the given parameter name\n which are contained in the URL's query string or the body of a POST or PUT\n request.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">This function is similar to the <tt>request:param-values#2</tt> function.\n However, the names and values are treated (after precent-decoding) using the\n given encoding supplied as third parameter. For example, parameters might\n be encoded using the ISO-8859-1 encoding.</p>\n", "summary" : "<p>  Returns a sequence of parameter values for the given parameter name\n which are contained in the URL's query string or the body of a POST or PUT\n request.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "name", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The name of the parameter whose value(s) should be returned.</div>" }, { "name" : "default-values", "type" : "xs:string", "occurrence" : "*", "description" : "" }, { "name" : "encoding", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The encoding of the parameters in the request (e.g. ISO-8859-1)</div>" } ], "returns" : { "type" : "xs:string*", "description" : "A sequence of values for the parameter with the given name. The sequence given as $default-values parameter is returned if no parameter exists with the given name." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">request:invalid-encoding if the given encoding is invalid or not supported.</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 0, "name" : "params", "qname" : "request:params", "signature" : "() as object()", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Returns an object containg the parameter names and values contained\n in the URL's query string or the body of a POST or PUT\n request.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">The structure of the object for the query string\n \"param1=value1;param2&amp;param2=value2\"\n <pre>\n {\n   \"param1\" : \"value1\",\n   \"param2\" : [ \"\", \"value2\" ]\n }\n </pre>\n </p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Parameters are name-value pairs contained in the query string of the URL\n used to make this request. As defined in RFC 1738, the query string of a\n URL starts with a \"?\" character and ends with the character (if any).\n Additionally, such name-value pairs may be part of the request's body if\n it is a PUT or POST request and the content-type of the request is\n \"application/x-www-form-urlencoded\". Name-value pairs are separated\n using either the \"&amp;\" or the \";\" character.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">In general, the names and the values  are precent-encoded. This function\n does the decoding of the parameters, i.e. it returns the values being\n not percent-encoded.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Also, the names and the values of each parameter (after being precent-decoded)\n are treated as UTF-8. Please see the <tt>http:params#1</tt> function\n for retrieving parameters submitted using a encoding other than UTF-8.</p>\n", "summary" : "<p>  Returns an object containg the parameter names and values contained\n in the URL's query string or the body of a POST or PUT\n request.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [  ], "returns" : { "type" : "object()", "description" : "An object containing all the parameter names and values. An empty object is returned if the request doesn't contain any parameters." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "params", "qname" : "request:params", "signature" : "($encoding as xs:string) as object()", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Returns an object containg the parameter names and values contained\n in the URL's query string or the body of a POST or PUT\n request.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">The structure of the object for the query string\n \"param1=value1;param2&amp;param2=value2\"\n <pre>\n {\n   \"param1\" : \"value1\",\n   \"param2\" : [ \"\", \"value2\" ]\n }\n </pre>\n </p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Parameters are name-value pairs contained in the query string of the URL\n used to make this request. As defined in RFC 1738, the query string of a\n URL starts with a \"?\" character and ends with the character (if any).\n Additionally, such name-value pairs may be part of the request's body if\n it is a PUT or POST request and the content-type of the request is\n \"application/x-www-form-urlencoded\". Name-value pairs are separated\n using either the \"&amp;\" or the \";\" character.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">In general, the names and the values  are precent-encoded. This function\n does the decoding of the parameters, i.e. it returns the values being\n not percent-encoded.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">The name and value of each parameter (after being percent-decoded) are treated\n in the encoding given by the <tt>$encoding</tt> paramter.</p>\n", "summary" : "<p>  Returns an object containg the parameter names and values contained\n in the URL's query string or the body of a POST or PUT\n request.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "encoding", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The encoding of the parameters in the request (e.g. ISO-8859-1)</div>" } ], "returns" : { "type" : "object()", "description" : "An object containing all the parameter names and values. An empty object is returned if the request doesn't contain any parameters." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">request:invalid-encoding if the given encoding is invalid or not supported.</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 0, "name" : "parts", "qname" : "request:parts", "signature" : "() as object()?", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Returns the metadata of all parts contained in a multipart request.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">The metadata is returned as an object\n as shown in the following example.</p>\n <pre xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">\n {\n   \"media-type\" : \"multipart/form-data; boundary=----------------------------93298e7a66a4\",\n   \"parts\" : [ {\n     \"headers\" : {\n       \"Content-Disposition\" : \"form-data; name=\\\"upload\\\"; filename=\\\"tmp.txt\\\"\",\n       \"Content-Type\" : \"text/plain\"\n     },\n     \"filename\" : \"tmp.txt\",\n     \"name\" : \"upload\",\n     \"src\" : \"urn:uuid:09be48d1-da0e-42c1-a115-a697e1779c45\",\n     \"size\" : \"153\"\n   }, {\n     \"headers\" : {\n       \"Content-Disposition\" : \"form-data; name=\\\"press\\\"\"\n     },\n     \"name\" : \"press\",\n     \"src\" : \"urn:uuid:c80e0609-b703-4d47-9171-441eb397a562\",\n     \"size\" : \"2\"\n   } ]\n }\n </pre>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">The <tt>media-type</tt> field describes the content-type as given in the\n request. Specifically, its value is equal to the value returned by\n <tt>request:header-values(\"Content-Type\")</tt>).</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Each of the array members of the <tt>parts</tt> field represents one\n part of the multipart request. Each such part contains a header field\n containing one field for each headers belonging to that part.\n The value of the <tt>src</tt> field can be used to retrieve the actual value\n of the part by passing it to the <tt>http:text-part</tt> or\n <tt>http:binary-part</tt> functions. The remaining fields represent a\n parameter of the Content-Disposition header as described in RFC 2183\n (e.g. filename, name, creation-date).</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Please note that recursive multipart content is not supported.</p>\n", "summary" : "<p>  Returns the metadata of all parts contained in a multipart request.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [  ], "returns" : { "type" : "object()?", "description" : "an object representing the metadata of the multipart content or an empty sequence if there is no content." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">request:non-multipart if the current request does not contain multipart content</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">request:invalid-multipart if the multipart content is invalid (e.g. the boundary is missing)</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 0, "name" : "path", "qname" : "request:path", "signature" : "() as xs:string", "description" : " Return the path component of the request URI. The path starts after the\n host and ends before the query string starts.\n", "summary" : "<p> Return the path component of the request URI.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [  ], "returns" : { "type" : "xs:string", "description" : "The path component of the request URI" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 0, "name" : "query", "qname" : "request:query", "signature" : "() as xs:string", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Returns the query string that was used to make this request.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">The query string contains the part of the request URL that\n starts with the '?' character to the end or the starting of the\n fragment (i.e. the '#' character).</p>\n", "summary" : "<p>  Returns the query string that was used to make this request.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [  ], "returns" : { "type" : "xs:string", "description" : "The query string part of the request's URL" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 0, "name" : "remote-addr", "qname" : "request:remote-addr", "signature" : "() as xs:string", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Returns the IP address of the client to which this request\n is connected.</p>\n", "summary" : "<p>  Returns the IP address of the client to which this request\n is connected.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [  ], "returns" : { "type" : "xs:string", "description" : "The IP address on the client side to which this request is connected." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 0, "name" : "remote-port", "qname" : "request:remote-port", "signature" : "() as xs:int", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Returns the port of the client to which this request is connected.</p>\n", "summary" : "<p>  Returns the port of the client to which this request is connected.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [  ], "returns" : { "type" : "xs:int", "description" : "The port on the client side to which this request is connected." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 0, "name" : "server-name", "qname" : "request:server-name", "signature" : "() as xs:string", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Returns the server name of the server running the application.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">The web server's hostname or IP address.</p>\n", "summary" : "<p>  Returns the server name of the server running the application.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [  ], "returns" : { "type" : "xs:string", "description" : "The name of the server that runs the application accepting this request." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 0, "name" : "server-port", "qname" : "request:server-port", "signature" : "() as xs:int", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Returns the sever port to which the client making the current request\n is connected.</p>\n", "summary" : "<p>  Returns the sever port to which the client making the current request\n is connected.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [  ], "returns" : { "type" : "xs:int", "description" : "The server port to which the client is connected." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 0, "name" : "text-content", "qname" : "request:text-content", "signature" : "() as xs:string", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Returns the content of the request as string.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">The function returns the content of the request only\n if the content-type refers to a type that can be treated\n as text (e.g. text/* or application/xml). The function raises\n an error if the content cannot be treated as text.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">The text content is interpreted using the encoding/charset\n that is specified in the Content-Type header of the request. If\n no charset is specified, the default ISO-8859-1 is used. If a encoding\n other than the specified or default one should be used, the\n <tt>request:text-content#1</tt> function should be used.</p>\n", "summary" : "<p>  Returns the content of the request as string.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [  ], "returns" : { "type" : "xs:string", "description" : "The content of the request as a string." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">request:invalid-encoding if the encoding specified in the Content-Type header is invalid or not supported.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">request:no-text-content if the content contained in the body of the request cannot be treated as text.</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 1, "name" : "text-content", "qname" : "request:text-content", "signature" : "($overwrite-encoding as xs:string) as xs:string", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Returns the content of the request as string interpreting\n it with the given encoding.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">The function returns the content of the request only\n if the content-type refers to a type that can be treated\n as text (e.g. text/* or application/xml). The function raises\n an error if the content cannot be treated as text.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">The text content is interpreted using the given encoding/charset.\n That is, the charset specified in the Content-Type header of the request\n is ignored. An error is raised if the given encoding is invalid\n or not supported.</p>\n", "summary" : "<p>  Returns the content of the request as string interpreting\n it with the given encoding.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "overwrite-encoding", "type" : "xs:string", "occurrence" : null, "description" : "" } ], "returns" : { "type" : "xs:string", "description" : "The content of the request as a string." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">request:invalid-encoding if the encoding specified in the Content-Type header or the $overwrite-encoding parameter is invalid or not supported.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">request:no-text-content if the content contained in the body of the request cannot be treated as text.</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 1, "name" : "text-part", "qname" : "request:text-part", "signature" : "($ref as xs:string) as xs:string", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Returns the value of a part as string</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">A part is identified by a reference that is the value of a\n <tt>src</tt> field returned by the <tt>request:parts</tt> function.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">The value of the text part is interpreted using the encoding/charset\n given in the headers of the part. If no encoding is given, the default\n US-ASCII is assumed.</p>\n", "summary" : "<p>  Returns the value of a part as string \n  A part is identified by a reference that is the value of a\n  src  field returned by the  request:parts  function.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "ref", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the name of the part</div>" } ], "returns" : { "type" : "xs:string", "description" : "the value of the part as string" }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">request:invalid-encoding if the encoding given in the headers of the part is invalid or not supported.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">request:no-text-content if the value of the part cannot be treated as text</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">request:invalid-part if the part with the given name ($ref) does not exist</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">request:non-multipart if the current request does not contain multipart content</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">request:invalid-multipart if the multipart content is invalid (e.g. the boundary is missing)</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 2, "name" : "text-part", "qname" : "request:text-part", "signature" : "($ref as xs:string, $overwrite-encoding as xs:string) as xs:string", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Returns the value of a part as string interpreting\n it with the given encoding.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">A part is identified by a reference that is the value of a\n <tt>src</tt> field returned by the <tt>request:parts</tt> function.</p>\n", "summary" : "<p>  Returns the value of a part as string interpreting\n it with the given encoding.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "ref", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the name of the part</div>" }, { "name" : "overwrite-encoding", "type" : "xs:string", "occurrence" : null, "description" : "" } ], "returns" : { "type" : "xs:string", "description" : "the value of the part as string" }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">request:invalid-encoding if the encoding given using the $overwrite-encoding parameter is invalid or not supported.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">request:no-text-content if the value of the part cannot be treated as text</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">request:invalid-part if the part with the given name ($ref) does not exist</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">request:non-multipart if the current request does not contain multipart content</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">request:invalid-multipart if the multipart content is invalid (e.g. the boundary is missing)</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 0, "name" : "uri", "qname" : "request:uri", "signature" : "() as xs:string", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Returns the URI that was used to make this request.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">The value returned contains the part of the URL starting\n from the path to the end or the starting of the fragment (i.e.\n the '#' character).</p>\n", "summary" : "<p>  Returns the URI that was used to make this request.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [  ], "returns" : { "type" : "xs:string", "description" : "The path and query string part of the request's URL" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 0, "name" : "user-agent", "qname" : "request:user-agent", "signature" : "() as xs:string?", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Returns the user agent that made to perform the current request.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">This function returns the value of the User-Agent header\n contained in the current request.</p>\n", "summary" : "<p>  Returns the user agent that made to perform the current request.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [  ], "returns" : { "type" : "xs:string?", "description" : "The user agent used to perform this request of the empty sequence if there was no User-Agent header in the request." }, "errors" : [  ] } ], "variables" : [  ] }, "http://www.28msec.com/modules/physical-reference" : { "ns" : "http://www.28msec.com/modules/physical-reference", "description" : " The module provides functions to compute an immutable and opaque reference\n for nodes, objects, or arrays and to retrieve such items given their\n identifier, respectively.\n The identifiers are immutable, i.e. a identifier does not change\n during the items lifetime and cannot be reused for another item after the\n original item gets deleted.\n Identifiers are unique, in that, two different items will never have the same\n identifier. A item, at any time during its lifetime, can be retrieved by its\n identifier. Identifiers can only be computed for items stored in a MongoDB\n collection.\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Please see the <a href=\"../../html/data_lifecycle.html\">data lifecycle\n documentation</a> about details on storing items.</p>\n", "sees" : [ "<xqdoc:see xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"><a href=\"../../html/data_lifecycle.html\">Data Lifecycle</a></xqdoc:see>" ], "authors" : [ "<xqdoc:author xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Federico Cavalieri</xqdoc:author>" ], "version" : null, "encoding" : "utf-8", "namespaces" : [ { "uri" : "http://www.28msec.com/modules/physical-reference", "prefix" : "ref" }, { "uri" : "http://zorba.io/options/versioning", "prefix" : "ver" } ], "functions" : [ { "isDocumented" : true, "arity" : 1, "name" : "dereference", "qname" : "ref:dereference", "signature" : "($arg as object()) as item()? external", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Returns the node, object, or array identified by the given reference.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">The function returns the empty sequence if the item\n that is referenced does not exist.</p>\n", "summary" : "<p>  Returns the node, object, or array identified by the given reference.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "arg", "type" : "object()", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the identifier of the item to retrieve.</div>" } ], "returns" : { "type" : "item()?", "description" : "the item identified by the identifier passed as parameter or the empty-sequence if no item with that URI is found." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">ref::REFERENCE is if the given identifier is invalid.</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 1, "name" : "reference", "qname" : "ref:reference", "signature" : "($arg as item()) as object() external", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Returns an immutable and opaque reference (with type xs:anyURI) for\n a given node, object, or array.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">The generated identifier is immutable, i.e. a identifier does not\n change during the item's lifetime and cannot be reused for another node after\n the original item gets deleted.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Identifiers are also unique, in that, two different items will never\n have the same identifier.</p>\n A item, at any time during its lifetime, can be retrieved by its\n identifier, using the <tt xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">ref:dereference</tt> function.\n Please note that a reference can only be retrieved for a JSON object or JSON\n array if the item is a member of a collection.\n", "summary" : "<p>  Returns an immutable and opaque reference (with type xs:anyURI) for\n a given node, object, or array.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "arg", "type" : "item()", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the node, object, or array for which the URI should be computed</div>" } ], "returns" : { "type" : "object()", "description" : "the opaque URI of the item." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">ref::COLLECTION is raised if the object or array passed as argument is not a member of a collection.</xqdoc:error>" ] } ], "variables" : [  ] }, "http://zorba.io/modules/data-cleaning/conversion" : { "ns" : "http://zorba.io/modules/data-cleaning/conversion", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">This library module provides data conversion functions for processing calendar dates,\n temporal values, currency values, units of measurement, location names and postal addresses.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">The logic contained in this module is not specific to any particular XQuery implementation.</p>\n", "sees" : [  ], "authors" : [ "<xqdoc:author xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Bruno Martins and Diogo Simões</xqdoc:author>" ], "version" : null, "encoding" : "utf-8", "namespaces" : [ { "uri" : "http://zorba.io/annotations", "prefix" : "an" }, { "uri" : "http://zorba.io/modules/data-cleaning/conversion", "prefix" : "conversion" }, { "uri" : "http://www.ecb.int/vocabulary/2002-08-01/eurofxref", "prefix" : "exref" }, { "uri" : "http://www.zorba-xquery.com/modules/http-client", "prefix" : "http" }, { "uri" : "http://zorba.io/modules/reflection", "prefix" : "reflection" }, { "uri" : "http://zorba.io/options/versioning", "prefix" : "ver" }, { "uri" : "http://api.whitepages.com/schema/", "prefix" : "wp" } ], "functions" : [ { "isDocumented" : true, "arity" : 1, "name" : "address-from-domain", "qname" : "conversion:address-from-domain", "signature" : "($domain as xs:string) as xs:string*", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Uses a whois service to discover information about a given domain name, returning a sequence of strings\n for the addresses associated to the name.</p>\n", "summary" : "<p>  Uses a whois service to discover information about a given domain name, returning a sequence of strings\n for the addresses associated to the name.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "domain", "type" : "xs:string", "occurrence" : null, "description" : "" } ], "returns" : { "type" : "xs:string*", "description" : "A sequence of strings for the addresses associated to the domain name. <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"><b> Attention : This function is still not implemented. </b></p>" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 2, "name" : "address-from-geocode", "qname" : "conversion:address-from-geocode", "signature" : "($lat as xs:double, $lon as xs:double) as xs:string*", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Geospatial coordinates to placename converter, acting as a wrapper over the Yahoo! reverse geocoder service.</p>\n", "summary" : "<p>  Geospatial coordinates to placename converter, acting as a wrapper over the Yahoo! reverse geocoder service.</p>", "annotation_str" : " %an:nondeterministic", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "nondeterministic", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "lat", "type" : "xs:double", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> Geospatial latitude.</div>" }, { "name" : "lon", "type" : "xs:double", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> Geospatial longitude.</div>" } ], "returns" : { "type" : "xs:string*", "description" : "The sequence of strings corresponding to the different components (e.g., street, city, country, etc.) of the place name that corresponds to the input geospatial coordinates." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "address-from-phone", "qname" : "conversion:address-from-phone", "signature" : "($phone-number as xs:string) as xs:string*", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Uses a White-pages Web service to discover information about a given phone number,\n returning a string for the address associated to the phone number.</p>\n", "summary" : "<p>  Uses a White-pages Web service to discover information about a given phone number,\n returning a string for the address associated to the phone number.</p>", "annotation_str" : " %an:nondeterministic", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "nondeterministic", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "phone-number", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> A string with 10 digits corresponding to the phone number.</div>" } ], "returns" : { "type" : "xs:string*", "description" : "A string for the addresses associated to the phone number." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "address-from-user", "qname" : "conversion:address-from-user", "signature" : "($name as xs:string) as xs:string*", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Uses a White-pages Web service to discover information about a given name,\n returning a sequence of strings for the addresses associated to the name.</p>\n", "summary" : "<p>  Uses a White-pages Web service to discover information about a given name,\n returning a sequence of strings for the addresses associated to the name.</p>", "annotation_str" : " %an:nondeterministic", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "nondeterministic", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "name", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The name of person or organization.</div>" } ], "returns" : { "type" : "xs:string*", "description" : "A sequence of strings for the addresses associated to the name." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 4, "name" : "currency-convert", "qname" : "conversion:currency-convert", "signature" : "($v as xs:double, $m1 as xs:string, $m2 as xs:string, $date as xs:string) as xs:double", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Currency conversion function, acting as a wrapper over the WebService from the European Central Bank.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">WebService documentation at <a src=\"http://www.ecb.int/stats/exchange/eurofxref/html/index.en.html\">http://www.ecb.int/stats/exchange/eurofxref/html/index.en.html</a></p>\n", "summary" : "<p>  Currency conversion function, acting as a wrapper over the WebService from the European Central Bank.</p>", "annotation_str" : " %an:nondeterministic", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "nondeterministic", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "v", "type" : "xs:double", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The amount we wish to convert.</div>" }, { "name" : "m1", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The source currency (e.g., \"EUR\").</div>" }, { "name" : "m2", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The target currency (e.g., \"USD\").</div>" }, { "name" : "date", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The reference date.</div>" } ], "returns" : { "type" : "xs:double", "description" : "The value resulting from the conversion." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">conversion:NOTSUPPORTED if the date, the source currency type or the target currency type are not known to the service.</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 1, "name" : "geocode-from-address", "qname" : "conversion:geocode-from-address", "signature" : "($q as xs:string*) as xs:double*", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Placename to geospatial coordinates converter, acting as a wrapper over the Yahoo! geocoder service.</p>\n", "summary" : "<p>  Placename to geospatial coordinates converter, acting as a wrapper over the Yahoo! geocoder service.</p>", "annotation_str" : " %an:nondeterministic", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "nondeterministic", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "q", "type" : "xs:string", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> A sequence of strings corresponding to the different components (e.g., street, city, country, etc.) of the place name.</div>" } ], "returns" : { "type" : "xs:double*", "description" : "The pair of latitude and longitude coordinates associated with the input address." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "name-from-domain", "qname" : "conversion:name-from-domain", "signature" : "($domain as xs:string) as xs:string*", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Uses a whois service to discover information about a given domain name, returning a sequence of strings\n for the person or organization names associated to the name.</p>\n", "summary" : "<p>  Uses a whois service to discover information about a given domain name, returning a sequence of strings\n for the person or organization names associated to the name.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "domain", "type" : "xs:string", "occurrence" : null, "description" : "" } ], "returns" : { "type" : "xs:string*", "description" : "A sequence of strings for the person or organization names associated to the domain name. <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"><b> Attention : This function is still not implemented. </b></p>" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "phone-from-address", "qname" : "conversion:phone-from-address", "signature" : "($address as xs:string) as xs:string*", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Uses a White-pages Web service to discover information about a given address,\n returning a sequence of strings for the phone number associated to the address.</p>\n", "summary" : "<p>  Uses a White-pages Web service to discover information about a given address,\n returning a sequence of strings for the phone number associated to the address.</p>", "annotation_str" : " %an:nondeterministic", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "nondeterministic", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "address", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> A string corresponding to the address (ex: 5655 E Gaskill Rd, Willcox, AZ, US).</div>" } ], "returns" : { "type" : "xs:string*", "description" : "A sequence of strings for the phone number or organization's names associated to the address." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "phone-from-domain", "qname" : "conversion:phone-from-domain", "signature" : "($domain as xs:string) as xs:string*", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Uses a whois service to discover information about a given domain name, returning a sequence of strings\n for the phone numbers associated to the name.</p>\n", "summary" : "<p>  Uses a whois service to discover information about a given domain name, returning a sequence of strings\n for the phone numbers associated to the name.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "domain", "type" : "xs:string", "occurrence" : null, "description" : "" } ], "returns" : { "type" : "xs:string*", "description" : "A sequence of strings for the phone numbers associated to the domain name. <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"><b> Attention : This function is still not implemented. </b></p>" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "phone-from-user", "qname" : "conversion:phone-from-user", "signature" : "($name as xs:string) as xs:string*", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Uses a White-pages Web service to discover information about a given name,\n returning a sequence of strings for the phone numbers associated to the name.</p>\n", "summary" : "<p>  Uses a White-pages Web service to discover information about a given name,\n returning a sequence of strings for the phone numbers associated to the name.</p>", "annotation_str" : " %an:nondeterministic", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "nondeterministic", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "name", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The name of person or organization.</div>" } ], "returns" : { "type" : "xs:string*", "description" : "A sequence of strings for the phone numbers associated to the name." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 4, "name" : "unit-convert", "qname" : "conversion:unit-convert", "signature" : "($v as xs:double, $t as xs:string, $m1 as xs:string, $m2 as xs:string) as xs:double", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Conversion function for units of measurement, acting as a wrapper over the CuppaIT WebService.</p>\n", "summary" : "<p>  Conversion function for units of measurement, acting as a wrapper over the CuppaIT WebService.</p>", "annotation_str" : " %an:nondeterministic", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "nondeterministic", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "v", "type" : "xs:double", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The amount we wish to convert.</div>" }, { "name" : "t", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The type of metric (e.g., \"Distance\")</div>" }, { "name" : "m1", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The source measurement unit metric (e.g., \"meter\")</div>" }, { "name" : "m2", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The target measurement unit metric (e.g., \"mile\")</div>" } ], "returns" : { "type" : "xs:double", "description" : "The value resulting from the conversion" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "user-from-address", "qname" : "conversion:user-from-address", "signature" : "($address as xs:string) as xs:string*", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Uses a White-pages Web service to discover information about a given address,\n returning a sequence of strings for the names associated to the address.</p>\n", "summary" : "<p>  Uses a White-pages Web service to discover information about a given address,\n returning a sequence of strings for the names associated to the address.</p>", "annotation_str" : " %an:nondeterministic", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "nondeterministic", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "address", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> A string corresponding to the address (ex: 5655 E Gaskill Rd, Willcox, AZ, US).</div>" } ], "returns" : { "type" : "xs:string*", "description" : "A sequence of strings for the person or organization's names associated to the address." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "user-from-phone", "qname" : "conversion:user-from-phone", "signature" : "($phone-number as xs:string) as xs:string*", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Uses a White-pages Web service to discover information about a given phone number,\n returning a sequence of strings for the name associated to the phone number.</p>\n", "summary" : "<p>  Uses a White-pages Web service to discover information about a given phone number,\n returning a sequence of strings for the name associated to the phone number.</p>", "annotation_str" : " %an:nondeterministic", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "nondeterministic", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "phone-number", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> A string with 10 digits corresponding to the phone number.</div>" } ], "returns" : { "type" : "xs:string*", "description" : "A sequence of strings for the person or organization's name associated to the phone number." }, "errors" : [  ] } ], "variables" : [ { "name" : "conversion:key", "type" : "item()*", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">The key to be used when accessing the White Pages Web service</p>\n" } ] }, "http://www.28msec.com/modules/jdbc" : { "ns" : "http://www.28msec.com/modules/jdbc", "description" : " This module allows connecting, querying, and updating JDBC datasources.\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">The results of a query are returned as a sequence of objects:\n <code> { column: value } </code>.\n The type of the value depends on the type of the column in the database:\n <table class=\"table table-bordered\">\n   <tr><th>JDBC Types</th><th>JSONiq Type</th></tr>\n   <tr><td>integer, bigint, smallint, tinyint</td><td>integer</td></tr>\n   <tr><td>decimal, double, float, numeric, real</td><td>double</td></tr>\n   <tr><td>boolean, bit</td><td>boolean</td></tr>\n   <tr><td>char, blob, longvarchar, longnvarchar, nchar, nclob, nvarchar,\n  varchar, sqlxml, date, time, timestamp</td><td>string</td></tr>\n   <tr><td>binary, blob, longvarbinary, varbinary, array, datalink,\n  java_object, other, ref</td><td>base64Binary</td></tr>\n </table>\n </p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">\n If the value of a column is null, it is mapped to the JSONiq null value.\n </p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Connecting to a JDBC source requires the following options:\n <ul>\n   <li><tt>url</tt>: the JDBC connection URI (mandatory)</li>\n   <li><tt>user</tt>: the user used for connecting (optional)</li>\n   <li><tt>password</tt>: the password used for connecting (optional)</li>\n </ul></p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Examples:\n <ul>\n  <li><pre>\n {\n   \"url\" : \"jdbc:mysql://localhost/testdb\",\n   \"user\" : \"root\",\n   \"password\" : \"\"\n }</pre></li>\n  <li><pre>\n {\n   \"url\" : \"jdbc:sqlserver://192.168.1.1;databaseName=testdb\",\n   \"user\" : \"sa\",\n   \"password\" : \"\"\n }</pre></li>\n  <li><pre>\n {\n   \"url\" : \"jdbc:postgresql://localhost/testdb\",\n   \"user\" : \"root\",\n   \"password\" : \"\"\n }</pre></li>\n </ul></p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Currently, the 28.io platform supports connections to\n <ul>\n   <li>MySQL using the mariadb-java-client.jar version 1.1.2,</li>\n   <li>PostgreSQL using postgresql.jar version 9.2, and </li>\n   <li>Microsoft SQL Server using sqljdbc4.jar version 4.0.</li>\n </ul>\n </p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">If you are interested in connecting to other JDBC datasources,\n please contact us at support@28.io.</p>\n <h2 xmlns:xqdoc=\"http://www.xqdoc.org/1.0\" id=\"determinism\">Important Notice Regarding Function Determinism</h2>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">The non side-effecting functions:\n <ul>\n   <li><a href=\"?anchor=connect-0\">connect#0</a></li>\n   <li><a href=\"?anchor=connect-1\">connect#1</a></li>\n   <li><a href=\"?anchor=connect-2\">connect#2</a></li>\n   <li><a href=\"?anchor=execute-query-2\">execute-query#2</a></li>\n   <li><a href=\"?anchor=execute-query-prepared-1\">execute-query-prepared#1</a></li>\n   <li><a href=\"?anchor=tables-1\">tables#1</a></li>\n   <li><a href=\"?anchor=tables-4\">tables#4</a></li>\n </ul>\n are declared deterministic, which means that their results could be cached\n when invoked multiple times with the same arguments in the same query execution.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">To not use cached results you can use the following alternative functions:\n <ul>\n   <li><a href=\"?anchor=connect-notdeterministic-0\">connect-notdeterministic#0</a></li>\n   <li><a href=\"?anchor=connect-notdeterministic-1\">connect-notdeterministic#1</a></li>\n   <li><a href=\"?anchor=connect-notdeterministic-2\">connect-notdeterministic#2</a></li>\n   <li><a href=\"?anchor=execute-query-notdeterministic-2\">execute-query-notdeterministic#2</a></li>\n   <li><a href=\"?anchor=execute-query-prepared-notdeterministic-1\">execute-query-prepared-notdeterministic#1</a></li>\n   <li><a href=\"?anchor=tables-notdeterministic-1\">tables-notdeterministic#1</a></li>\n   <li><a href=\"?anchor=tables-notdeterministic-4\">tables-notdeterministic#4</a></li>\n </ul>\n which have been declared as being non deterministic.</p>\n", "sees" : [  ], "authors" : [ "<xqdoc:author xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Cristi Dumitru</xqdoc:author>", "<xqdoc:author xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Federico Cavalieri</xqdoc:author>" ], "version" : null, "encoding" : "utf-8", "namespaces" : [ { "uri" : "http://zorba.io/annotations", "prefix" : "an" }, { "uri" : "http://www.28msec.com/modules/credentials", "prefix" : "credentials" }, { "uri" : "http://www.28msec.com/modules/jdbc", "prefix" : "jdbc" }, { "uri" : "http://zorba.io/options/versioning", "prefix" : "ver" }, { "uri" : "http://www.zorba-xquery.com/modules/jdbc", "prefix" : "zjdbc" } ], "functions" : [ { "isDocumented" : true, "arity" : 1, "name" : "affected-rows", "qname" : "jdbc:affected-rows", "signature" : "($dataset as anyURI) as integer", "description" : " Return the number of affected rows of an updating DataSet.\n", "summary" : "<p> Return the number of affected rows of an updating DataSet.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "dataset", "type" : "anyURI", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The identifier of the DataSet.</div>" } ], "returns" : { "type" : "integer", "description" : "the number of affected rows." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">SQL008 DataSet doesn't exist.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">SQL08000 Connection is closed.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">SQL001 Descriptive error, see error in attached message.</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 1, "name" : "clear-params", "qname" : "jdbc:clear-params", "signature" : "($prepared-statement as anyURI) as empty-sequence()", "description" : " Clear all the parameters of the statement.\n", "summary" : "<p> Clear all the parameters of the statement.</p>", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "prepared-statement", "type" : "anyURI", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The identifier of the prepared statement.</div>" } ], "returns" : { "type" : "empty-sequence()", "description" : "an empty-sequence()" }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">SQL003 Prepared statement doesn't exist.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">SQL08000 Connection is closed.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">SQL001 Descriptive error, see error in attached message.</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 1, "name" : "commit", "qname" : "jdbc:commit", "signature" : "($conn as anyURI) as empty-sequence()", "description" : " Commit current transaction from an active connection.\n", "summary" : "<p> Commit current transaction from an active connection.</p>", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "conn", "type" : "anyURI", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The identifier of the connection with a transaction to be commited.</div>" } ], "returns" : { "type" : "empty-sequence()", "description" : "an empty-sequence()" }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">SQL08003 Connection doesn't exist</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">SQL08000 Connection is closed</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">SQL001 Descriptive error, see error in attached message</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 0, "name" : "connect-nondeterministic", "qname" : "jdbc:connect-nondeterministic", "signature" : "() as anyURI", "description" : " Open a connection to a database using the default credentials.\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">This function has the same semantics as\n <a href=\"#connect-0\">connect#0</a>,\n but is declared as being non deterministic and thus should only be used when\n result caching is not desired.</p>\n", "summary" : "<p> Open a connection to a database using the default credentials.</p>", "annotation_str" : " %an:nondeterministic", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "nondeterministic", "value" : "" } ], "updating" : false, "parameters" : [  ], "returns" : { "type" : "anyURI", "description" : "an identifier that represents the connection to the server." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">NOT-FOUND The default credentials not found.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">SQL28000 Authentication failed.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">SQL08001 Connection error.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">SQL40003 Isolation level not supported.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">SQL001 Descriptive error, see attached message.</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 1, "name" : "connect-nondeterministic", "qname" : "jdbc:connect-nondeterministic", "signature" : "($connection-config as item()) as anyURI", "description" : " Open a connection to a database.\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">This function has the same semantics as\n <a href=\"#connect-1\">connect#1</a>,\n but is declared as being non deterministic and thus should only be used when\n result caching is not desired.</p>\n", "summary" : "<p> Open a connection to a database.</p>", "annotation_str" : " %an:nondeterministic", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "nondeterministic", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "connection-config", "type" : "item()", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> object containing the configuration information.</div>" } ], "returns" : { "type" : "anyURI", "description" : "an identifier that represents the connection to the server." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">NOT-FOUND The specified credentials have not been found.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">NOT-VALID Invalid connection information.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">SQL28000 Authentication failed.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">SQL08001 Connection error.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">SQL40003 Isolation level not supported.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">SQL001 Descriptive error, see attached message.</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 2, "name" : "connect-nondeterministic", "qname" : "jdbc:connect-nondeterministic", "signature" : "($connection-config as item(), $options as object()?) as anyURI", "description" : " Open a connection to a database.\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">This function has the same semantics as\n <a href=\"#connect-2\">connect#2</a>,\n but is declared as being non deterministic and thus should only be used when\n result caching is not desired.</p>\n", "summary" : "<p> Open a connection to a database.</p>", "annotation_str" : " %an:nondeterministic", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "nondeterministic", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "connection-config", "type" : "item()", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> object containing the configuration information.</div>" }, { "name" : "options", "type" : "object()", "occurrence" : "?", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> object to specify additional connection options.</div>" } ], "returns" : { "type" : "anyURI", "description" : "an identifier that represents the connection to the server." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">NOT-FOUND The specified credentials have not been found.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">NOT-VALID Invalid connection information.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">SQL28000 Authentication failed.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">SQL08001 Connection error.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">SQL40003 Isolation level not supported.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">SQL001 Descriptive error, see error in attached message.</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 0, "name" : "connect", "qname" : "jdbc:connect", "signature" : "() as anyURI", "description" : " Open a connection to a database using the default credentials.\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">This function is declared as deterministic and should be used whenever result\n caching is acceptable.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">The function returns an opaque URI that can represents the connection.\n This URI has to be passed to other functions of this module that require\n the <tt>$conn</tt> parameter as a first argument.</p>\n", "summary" : "<p> Open a connection to a database using the default credentials.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [  ], "returns" : { "type" : "anyURI", "description" : "an identifier that represents the connection to the server." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">NOT-FOUND The default credentials not found.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">SQL28000 Authentication failed.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">SQL08001 Connection error.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">SQL40003 Isolation level not supported.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">SQL001 Descriptive error, see attached message.</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 1, "name" : "connect", "qname" : "jdbc:connect", "signature" : "($connection-config as item()) as anyURI", "description" : " Open a connection to a database.\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">This function is declared as deterministic and should be used whenever result\n caching is acceptable.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">The input to the function contains the connection information.\n If a string is used, then the function will interpret it as credential name\n and will connect using the JDBC credentials with the specified name.\n If an object is used, then the function will open a connection using it.\n The object's required structure is described in the module's description.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">The function returns an opaque URI that can represents the connection.\n This URI has to be passed to other functions of this module that require\n the <tt>$conn</tt> parameter as a first argument.</p>\n", "summary" : "<p> Open a connection to a database.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "connection-config", "type" : "item()", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> object containing the configuration information.</div>" } ], "returns" : { "type" : "anyURI", "description" : "an identifier that represents the connection to the server." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">NOT-FOUND The specified credentials have not been found.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">NOT-VALID Invalid connection information.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">SQL28000 Authentication failed.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">SQL08001 Connection error.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">SQL40003 Isolation level not supported.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">SQL001 Descriptive error, see attached message.</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 2, "name" : "connect", "qname" : "jdbc:connect", "signature" : "($connection-config as item(), $options as object()?) as anyURI", "description" : " Open a connection to a database.\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">This function is declared as deterministic and should be used whenever result\n caching is acceptable.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">The input to the function contains the connection information.\n If a string is used, then the function will interpret it as credential name\n and will connect using the JDBC credentials with the specified name.\n If an object is used, then the function will open a connection using it.\n The object's required structure is described in the module's description.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">In addition to the connection configuration, the function allows to\n specify several options:\n <ul>\n   <li><tt>autocommit</tt>: <tt>true/false</tt> turn on/off auto commit\n (default: true)</li>\n   <li><tt>readonly</tt>: <tt>true/false</tt> configure readonly/write on this\n connection (default: false)</li>\n   <li><tt>isolation-level</tt>: configure the isolation level for this connection\n     <ul>\n       <li><tt>READ-COMMITTED</tt>: set the isolation level read-committed</li>\n       <li><tt>READ-UNCOMMITTED</tt>: set the isolation level read-uncommitted</li>\n       <li><tt>READ-REPEATABLE</tt>: set the isolation level repeatable-read</li>\n       <li><tt>SERIALIZABLE</tt>: set the isolation level serializable</li>\n     </ul>\n   If no isolation level is provided by the user the connection will be created\n   with the default isolation level of the database.</li>\n </ul></p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">The function returns an opaque URI that represents the connection.\n This URI has to be passed to other functions of this module that require\n the <tt>$conn</tt> parameter as a first argument.</p>\n", "summary" : "<p> Open a connection to a database.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "connection-config", "type" : "item()", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> object containing the configuration information.</div>" }, { "name" : "options", "type" : "object()", "occurrence" : "?", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> object to specify additional connection options.</div>" } ], "returns" : { "type" : "anyURI", "description" : "an identifier that represents the connection to the server." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">NOT-FOUND The specified credentials have not been found.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">NOT-VALID Invalid connection information.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">SQL28000 Authentication failed.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">SQL08001 Connection error.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">SQL40003 Isolation level not supported.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">SQL001 Descriptive error, see error in attached message.</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 1, "name" : "connection-options", "qname" : "jdbc:connection-options", "signature" : "($conn as anyURI) as object()", "description" : " Return a set with options for a specified connection.\n The returned options are equal to the options specified in function jdbc:connect.\n Consequently, the options are specified as follows:\n <ul xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">\n   <li><tt>autocommit</tt>: <tt>true/false</tt> shows auto commit</li>\n   <li><tt>readonly</tt>: <tt>true/false</tt> shows if the connection is readonly/write</li>\n   <li><tt>isolation-level</tt>: shows the isolation level for this connection\n     <ul>\n       <li><tt>READ-COMMITTED</tt>: isolation level is read-committed</li>\n       <li><tt>READ-UNCOMMITTED</tt>: isolation level is read-uncommitted</li>\n       <li><tt>READ-REPEATABLE</tt>: isolation level is repeatable-read</li>\n       <li><tt>SERIALIZABLE</tt>: isolation level is serializable</li>\n     </ul></li>\n </ul>\n", "summary" : "<p> Return a set with options for a specified connection.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "conn", "type" : "anyURI", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The identifier of the connection.</div>" } ], "returns" : { "type" : "object()", "description" : "an object with the connection options." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">SQL08003 Connection doesn't exist</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">SQL08000 Connection is closed</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">SQL001 Descriptive error, see error in attached message</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 1, "name" : "execute-prepared", "qname" : "jdbc:execute-prepared", "signature" : "($prepared-statement as anyURI) as anyURI", "description" : " Execute SQL statements prepared with jdbc:prepare-statement,\n after setting the parameters values accordingly and returns an identifier to\n a Dataset.\n", "summary" : "<p> Execute SQL statements prepared with jdbc:prepare-statement,\n after setting the parameters values accordingly and returns an identifier to\n a Dataset.</p>", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "prepared-statement", "type" : "anyURI", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The identifier of the prepared statement.</div>" } ], "returns" : { "type" : "anyURI", "description" : "the identifier of a DataSet." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">SQL003 Prepared statement doesn't exist.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">SQL08000 Connection is closed.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">SQL001 Descriptive error, see error in attached message.</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 2, "name" : "execute-query-nondeterministic", "qname" : "jdbc:execute-query-nondeterministic", "signature" : "($conn as anyURI, $sql as string) as object()*", "description" : " Execute non-updating SQL statements.\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">This function has the same semantics as\n <a href=\"#execute-query-2\">execute-query#2</a>,\n but is declared as being non deterministic and thus should only be used when\n result caching is not desired.</p>\n", "summary" : "<p> Execute non-updating SQL statements.</p>", "annotation_str" : " %an:nondeterministic", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "nondeterministic", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "conn", "type" : "anyURI", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The identifier of the active connection.</div>" }, { "name" : "sql", "type" : "string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The query string to be executed.</div>" } ], "returns" : { "type" : "object()*", "description" : "an object with the result data rows from the query provided." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">SQL08003 Connection doesn't exist.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">SQL08000 Connection is closed.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">SQL005 The statement is Updating type.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">SQL001 Descriptive error, see error in attached message.</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 1, "name" : "execute-query-prepared-nondeterministic", "qname" : "jdbc:execute-query-prepared-nondeterministic", "signature" : "($prepared-statement as anyURI) as object()*", "description" : " Execute non-updating SQL statements prepared with jdbc:prepare-statement,\n after setting the parameters values accordingly.\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">This function has the same semantics as\n <a href=\"#execute-query-prepared-1\">execute-query-prepared#1</a>,\n but is declared as being non deterministic and thus should only be used when\n result caching is not desired.</p>\n", "summary" : "<p> Execute non-updating SQL statements prepared with jdbc:prepare-statement,\n after setting the parameters values accordingly.</p>", "annotation_str" : " %an:nondeterministic", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "nondeterministic", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "prepared-statement", "type" : "anyURI", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The identifier of the prepared statement.</div>" } ], "returns" : { "type" : "object()*", "description" : "an object with the result data rows from the query processed with the parameters provided." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">SQL003 Prepared statement doesn't exist.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">SQL005 The prepared statement is an updating query.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">SQL08000 Connection is closed.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">SQL001 Descriptive error, see error in attached message.</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 1, "name" : "execute-query-prepared", "qname" : "jdbc:execute-query-prepared", "signature" : "($prepared-statement as anyURI) as object()*", "description" : " Execute non-updating SQL statements prepared with jdbc:prepare-statement,\n after setting the parameters values accordingly.\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">This function is declared as deterministic and should be used whenever result\n caching is acceptable.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Every row is represented by an object of column-value representation, as\n in the module description.</p>\n", "summary" : "<p> Execute non-updating SQL statements prepared with jdbc:prepare-statement,\n after setting the parameters values accordingly.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "prepared-statement", "type" : "anyURI", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The identifier of the prepared statement.</div>" } ], "returns" : { "type" : "object()*", "description" : "an object with the result data rows from the query processed with the parameters provided." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">SQL003 Prepared statement doesn't exist.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">SQL005 The prepared statement is an updating query.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">SQL08000 Connection is closed.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">SQL001 Descriptive error, see error in attached message.</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 2, "name" : "execute-query", "qname" : "jdbc:execute-query", "signature" : "($conn as anyURI, $sql as string) as object()*", "description" : " Execute non-updating SQL statements.\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">This function is declared as deterministic and should be used whenever result\n caching is acceptable.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Every row is represented by an object of column-value representation, as in\n the module description.</p>\n", "summary" : "<p> Execute non-updating SQL statements.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "conn", "type" : "anyURI", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The identifier of the active connection.</div>" }, { "name" : "sql", "type" : "string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The query string to be executed.</div>" } ], "returns" : { "type" : "object()*", "description" : "an object with the result data rows from the query provided." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">SQL08003 Connection doesn't exist.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">SQL08000 Connection is closed.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">SQL005 The statement is Updating type.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">SQL001 Descriptive error, see error in attached message.</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 1, "name" : "execute-update-prepared", "qname" : "jdbc:execute-update-prepared", "signature" : "($prepared-statement as anyURI) as integer", "description" : " Execute updating SQL statements prepared with jdbc:prepare-statement.\n", "summary" : "<p> Execute updating SQL statements prepared with jdbc:prepare-statement.</p>", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "prepared-statement", "type" : "anyURI", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The identifier of the prepared statement.</div>" } ], "returns" : { "type" : "integer", "description" : "the number of affected rows." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">SQL003 Prepared statement doesn't exist.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">SQL006 The prepared statement is a non-updating query.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">SQL08000 Connection is closed.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">SQL001 Descriptive error, see error in attached message.</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 2, "name" : "execute-update", "qname" : "jdbc:execute-update", "signature" : "($conn as anyURI, $sql as string) as integer", "description" : " Execute updating SQL statements.\n", "summary" : "<p> Execute updating SQL statements.</p>", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "conn", "type" : "anyURI", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The identifier of the active connection.</div>" }, { "name" : "sql", "type" : "string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The query string to be executed.</div>" } ], "returns" : { "type" : "integer", "description" : "the number of affected rows." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">SQL08003 Connection doesn't exist.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">SQL08000 Connection is closed.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">SQL005 The statement is Read-only type.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">SQL001 Descriptive error, see error in attached message.</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 2, "name" : "execute", "qname" : "jdbc:execute", "signature" : "($conn as anyURI, $sql as string) as anyURI", "description" : " Execute any kind of SQL statement.\n", "summary" : "<p> Execute any kind of SQL statement.</p>", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "conn", "type" : "anyURI", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The identifier of the active connection.</div>" }, { "name" : "sql", "type" : "string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The query string to be executed.</div>" } ], "returns" : { "type" : "anyURI", "description" : "an identifier of a DataSet." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">SQL08003 Connection doesn't exist.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">SQL08000 Connection is closed.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">SQL001 Descriptive error, see error in attached message.</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 1, "name" : "is-connected", "qname" : "jdbc:is-connected", "signature" : "($conn as anyURI) as boolean", "description" : " Verify if a connection is still active.\n", "summary" : "<p> Verify if a connection is still active.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "conn", "type" : "anyURI", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The identifier of the connection to be verified.</div>" } ], "returns" : { "type" : "boolean", "description" : "true if connected, false otherwise" }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">SQL08003 Connection doesn't exist</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">SQL001 Descriptive error, see error in attached message</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 1, "name" : "metadata", "qname" : "jdbc:metadata", "signature" : "($dataset as anyURI) as object()", "description" : " Return the metadata of the result of a DataSet.\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">\n More in detail, it returns information about column names, types, and whether\n a column can contain a null value.\n The metadata information can only be returned for DataSets that have been\n executed explicitly using the jdbc:execute function.\n The metadata node returned has the following structure:\n <pre>{\n   \"columns\": [ {\n       \"name\": string,\n       \"type\": string,\n       \"autoincrement\"? = boolean,\n       \"nillable\"? = boolean } * ]\n }</pre>\n </p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">\n If the query is an updating query, then the result object will return the\n number of affected rows like:\n <pre>{ \"affectedrows\": integer }</pre>\n </p>\n", "summary" : "<p> Return the metadata of the result of a DataSet.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "dataset", "type" : "anyURI", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The identifier of the DataSet.</div>" } ], "returns" : { "type" : "object()", "description" : "the metadata associated with an executed DataSet." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">SQL008 DataSet doesn't exist.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">SQL08000 Connection is closed.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">SQL001 Descriptive error, see error in attached message.</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 1, "name" : "parameter-metadata", "qname" : "jdbc:parameter-metadata", "signature" : "($prepared-statement as anyURI) as object()", "description" : " Retrieve the names and types of the prepared statement parameters.\n The metadata returned has the following structure:\n <pre xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">  {\n     columns: [{\n       \"name\": string,\n       \"type\": string\n       }]\n   } </pre>\n", "summary" : "<p> Retrieve the names and types of the prepared statement parameters.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "prepared-statement", "type" : "anyURI", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The identifier of the prepared statement.</div>" } ], "returns" : { "type" : "object()", "description" : "parameters metadata (name and type) associated with a prepared statement." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">SQL003 Prepared statement doesn't exist.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">SQL08000 Connection is closed.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">SQL001 Descriptive error, see error in attached message.</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 2, "name" : "prepare-statement", "qname" : "jdbc:prepare-statement", "signature" : "($conn as anyURI, $sql as string) as anyURI", "description" : " Create a prepared statement for multiple executions with diferent parameters.\n Example:\n <pre xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">\n jdbc:prepare-statement($connection, \"SELECT * FROM users WHERE id=? AND age&gt;?\")\n </pre>\n", "summary" : "<p> Create a prepared statement for multiple executions with diferent parameters.</p>", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "conn", "type" : "anyURI", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The identifier of the active connection.</div>" }, { "name" : "sql", "type" : "string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The query string to be executed.</div>" } ], "returns" : { "type" : "anyURI", "description" : "the identifier of the prepared statement." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">SQL08003 Connection doesn't exist.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">SQL08000 Connection is closed.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">SQL001 Descriptive error, see error in attached message.</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 1, "name" : "result-set", "qname" : "jdbc:result-set", "signature" : "($dataset as anyURI) as object()*", "description" : " Extract the data rows from a non-updating DataSet.\n Every row is represented by an object of column-value representation, as in\n the module description.\n", "summary" : "<p> Extract the data rows from a non-updating DataSet.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "dataset", "type" : "anyURI", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The identifier of the DataSet.</div>" } ], "returns" : { "type" : "object()*", "description" : "an object with the result data rows from the DataSet provided." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">SQL008 DataSet doesn't exist.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">SQL08000 Connection is closed.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">SQL001 Descriptive error, see error in attached message.</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 1, "name" : "rollback", "qname" : "jdbc:rollback", "signature" : "($conn as anyURI) as empty-sequence()", "description" : " Rollback the current transaction of a connection.\n", "summary" : "<p> Rollback the current transaction of a connection.</p>", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "conn", "type" : "anyURI", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The identifier of the connection with a transaction to be rollbacked.</div>" } ], "returns" : { "type" : "empty-sequence()", "description" : "an empty-sequence()" }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">SQL08003 Connection doesn't exist</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">SQL08000 Connection is closed</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">SQL001 Descriptive error, see error in attached message</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 3, "name" : "set-boolean", "qname" : "jdbc:set-boolean", "signature" : "($prepared-statement as anyURI, $parameter-index as integer, $value as boolean) as empty-sequence()", "description" : " Set the value of the designated parameter with a <tt xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">boolean</tt> value.\n", "summary" : "<p> Set the value of the designated parameter with a  boolean  value.</p>", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "prepared-statement", "type" : "anyURI", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The identifier of the prepared statement.</div>" }, { "name" : "parameter-index", "type" : "integer", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The index from the parameter to be set.</div>" }, { "name" : "value", "type" : "boolean", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The value to be set.</div>" } ], "returns" : { "type" : "empty-sequence()", "description" : "an empty-sequence()" }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">SQL003 Prepared statement doesn't exist.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">SQL007 Parameter casting error.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">SQL08000 Connection is closed.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">SQL001 Descriptive error, see error in attached message.</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 2, "name" : "set-null", "qname" : "jdbc:set-null", "signature" : "($prepared-statement as anyURI, $parameter-index as integer) as empty-sequence()", "description" : " Set the value of the designated parameter with a <tt xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">NULL</tt> value.\n", "summary" : "<p> Set the value of the designated parameter with a  NULL  value.</p>", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "prepared-statement", "type" : "anyURI", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The identifier of the prepared statement.</div>" }, { "name" : "parameter-index", "type" : "integer", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The index from the parameter to be set.</div>" } ], "returns" : { "type" : "empty-sequence()", "description" : "an empty-sequence()" }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">SQL003 Prepared statement doesn't exist.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">SQL007 Parameter casting error.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">SQL08000 Connection is closed.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">SQL001 Descriptive error, see error in attached message.</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 3, "name" : "set-numeric", "qname" : "jdbc:set-numeric", "signature" : "($prepared-statement as anyURI, $parameter-index as decimal, $value as anyAtomicType) as empty-sequence()", "description" : " Set the value of the designated parameter with a <tt xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">numeric</tt> value.\n", "summary" : "<p> Set the value of the designated parameter with a  numeric  value.</p>", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "prepared-statement", "type" : "anyURI", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The identifier of the prepared statement.</div>" }, { "name" : "parameter-index", "type" : "decimal", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The index from the parameter to be set.</div>" }, { "name" : "value", "type" : "anyAtomicType", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The value to be set.</div>" } ], "returns" : { "type" : "empty-sequence()", "description" : "an empty-sequence()" }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">SQL003 Prepared statement doesn't exist.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">SQL007 Parameter casting error.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">SQL08000 Connection is closed.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">SQL001 Descriptive error, see error in attached message.</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 3, "name" : "set-string", "qname" : "jdbc:set-string", "signature" : "($prepared-statement as anyURI, $parameter-index as integer, $value as string) as empty-sequence()", "description" : " Set the value of the designated parameter with a <tt xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">string</tt> value.\n", "summary" : "<p> Set the value of the designated parameter with a  string  value.</p>", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "prepared-statement", "type" : "anyURI", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The identifier of the prepared statement.</div>" }, { "name" : "parameter-index", "type" : "integer", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The index from the parameter to be set.</div>" }, { "name" : "value", "type" : "string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The value to be set.</div>" } ], "returns" : { "type" : "empty-sequence()", "description" : "an empty-sequence()" }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">SQL003 Prepared statement doesn't exist.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">SQL007 Parameter casting error.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">SQL08000 Connection is closed.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">SQL001 Descriptive error, see error in attached message.</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 3, "name" : "set-value", "qname" : "jdbc:set-value", "signature" : "($prepared-statement as anyURI, $parameter-index as decimal, $value as anyAtomicType) as empty-sequence()", "description" : " Set the value of the designated parameter with the given value.\n The function will try to cast the value to the correct data type and assign\n it to the parameter.\n", "summary" : "<p> Set the value of the designated parameter with the given value.</p>", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "prepared-statement", "type" : "anyURI", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The identifier of the prepared statement.</div>" }, { "name" : "parameter-index", "type" : "decimal", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The index from the parameter to be set.</div>" }, { "name" : "value", "type" : "anyAtomicType", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The value to be set.</div>" } ], "returns" : { "type" : "empty-sequence()", "description" : "an empty-sequence()" }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">SQL003 Prepared statement doesn't exist.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">SQL007 Parameter casting error.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">SQL08000 Connection is closed.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">SQL001 Descriptive error, see error in attached message.</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 1, "name" : "tables-nondeterministic", "qname" : "jdbc:tables-nondeterministic", "signature" : "($conn as anyURI) as object()*", "description" : " Return the list of tables from a connection.\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">This function has the same semantics as\n <a href=\"#tables-1\">tables#1</a>,\n but is declared as being non deterministic and thus should only be used when\n result caching is not desired.</p>\n", "summary" : "<p> Return the list of tables from a connection.</p>", "annotation_str" : " %an:nondeterministic", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "nondeterministic", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "conn", "type" : "anyURI", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The identifier to a connection.</div>" } ], "returns" : { "type" : "object()*", "description" : "Return an object with the result data rows from the query provided, the data rows are defined as follows: { column:value* }* Every row is represented by an object of column-value representation of the returned SQL result." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">SQL08000 Connection is closed.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">SQL001 Descriptive error, see error in attached message.</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 4, "name" : "tables-nondeterministic", "qname" : "jdbc:tables-nondeterministic", "signature" : "($conn as anyURI, $catalog as string?, $schema as string?, $table as string?) as object()*", "description" : " Return the list of tables from a connection.\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">This function has the same semantics as <a href=\"#tables-4\">tables#4</a>,\n but is declared as being non deterministic and thus should only be used when\n result caching is not desired.</p>\n", "summary" : "<p> Return the list of tables from a connection.</p>", "annotation_str" : " %an:nondeterministic", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "nondeterministic", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "conn", "type" : "anyURI", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The identifier to a connection.</div>" }, { "name" : "catalog", "type" : "string", "occurrence" : "?", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> A filter of the catalog name of the tables. Send empty-sequence for all tables.</div>" }, { "name" : "schema", "type" : "string", "occurrence" : "?", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> A filter of the schema name of the tables. Send empty-sequence for all tables.</div>" }, { "name" : "table", "type" : "string", "occurrence" : "?", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> A filter of the name of the tables. Send empty-sequence for all tables.</div>" } ], "returns" : { "type" : "object()*", "description" : "Return an object with the result data rows from the query provided, the data rows are defined as follows: { column:value* }* Every row is represented by an object of column-value representation of the returned SQL result." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">SQL08000 Connection is closed.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">SQL001 Descriptive error, see error in attached message.</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 1, "name" : "tables", "qname" : "jdbc:tables", "signature" : "($conn as anyURI) as object()*", "description" : " Return the list of tables from a connection.\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">This function is declared as deterministic and should be used whenever result\n caching is acceptable.</p>\n", "summary" : "<p> Return the list of tables from a connection.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "conn", "type" : "anyURI", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The identifier to a connection.</div>" } ], "returns" : { "type" : "object()*", "description" : "Return an object with the result data rows from the query provided, the data rows are defined as follows: { column:value* }* Every row is represented by an object of column-value representation of the returned SQL result." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">SQL08000 Connection is closed.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">SQL001 Descriptive error, see error in attached message.</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 4, "name" : "tables", "qname" : "jdbc:tables", "signature" : "($conn as anyURI, $catalog as string?, $schema as string?, $table as string?) as object()*", "description" : " Return the list of tables from a connection.\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">This function is declared as deterministic and should be used whenever result\n caching is acceptable.</p>\n", "summary" : "<p> Return the list of tables from a connection.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "conn", "type" : "anyURI", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The identifier to a connection.</div>" }, { "name" : "catalog", "type" : "string", "occurrence" : "?", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> A filter of the catalog name of the tables. Send empty-sequence for all tables.</div>" }, { "name" : "schema", "type" : "string", "occurrence" : "?", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> A filter of the schema name of the tables. Send empty-sequence for all tables.</div>" }, { "name" : "table", "type" : "string", "occurrence" : "?", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> A filter of the name of the tables. Send empty-sequence for all tables.</div>" } ], "returns" : { "type" : "object()*", "description" : "Return an object with the result data rows from the query provided, the data rows are defined as follows: { column:value* }* Every row is represented by an object of column-value representation of the returned SQL result." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">SQL08000 Connection is closed.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">SQL001 Descriptive error, see error in attached message.</xqdoc:error>" ] } ], "variables" : [ { "name" : "jdbc:READ-COMMITTED", "type" : "item()*", "description" : " This variable represents the READ-COMMITTED Isolation Level.\n" }, { "name" : "jdbc:READ-UNCOMMITTED", "type" : "item()*", "description" : " This variable represents the READ-UNCOMMITTED Isolation Level.\n" }, { "name" : "jdbc:REPEATABLE-READ", "type" : "item()*", "description" : " This variable represents the REPEATABLE-READ Isolation Levels.\n" }, { "name" : "jdbc:SERIALIZABLE", "type" : "item()*", "description" : " This variable represents the SERIALIZABLE Isolation Levels.\n" }, { "name" : "jdbc:CREDENTIALS-CATEGORY", "type" : "item()*", "description" : " This variable represents the category for the credentials used.\n" } ] }, "http://zorba.io/modules/full-text" : { "ns" : "http://zorba.io/modules/full-text", "description" : " This module provides an XQuery API to full-text functions.\n For general information about this implementation of the\n <a xmlns:xqdoc=\"http://www.xqdoc.org/1.0\" href=\"http://www.w3.org/TR/xpath-full-text-10/\">XQuery and XPath Full Text 1.0 specification</a>\n as well as instructions for building an installing a thesaurus,\n see the <a xmlns:xqdoc=\"http://www.xqdoc.org/1.0\" href=\"../zorba/ft_thesaurus.html\">Full Text Thesaurus documentation</a>.\n <h2 xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Notes on languages</h2>\n To refer to particular human languages,\n uses either the\n <a xmlns:xqdoc=\"http://www.xqdoc.org/1.0\" href=\"http://en.wikipedia.org/wiki/ISO_639-1\">ISO 639-1</a>\n or\n <a xmlns:xqdoc=\"http://www.xqdoc.org/1.0\" href=\"http://en.wikipedia.org/wiki/ISO_639-2\">ISO 639-2</a>\n languages codes.\n Note that only a subset of the\n <a xmlns:xqdoc=\"http://www.xqdoc.org/1.0\" href=\"http://en.wikipedia.org/wiki/List_of_ISO_639-1_codes\">complete list of language codes</a>\n are supported and not every function supports the same subset.\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n Most functions in this module take a language as a parameter\n using the\n <a xmlns:xqdoc=\"http://www.xqdoc.org/1.0\" href=\"http://www.w3.org/TR/xmlschema-2/#language\"><code>xs:language</code></a>\n XML schema data type.\n <h2 xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Notes on stemming</h2>\n The <code xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">stem()</code> functions return the\n <a xmlns:xqdoc=\"http://www.xqdoc.org/1.0\" href=\"http://en.wikipedia.org/wiki/Word_stem\">stem</a>\n of a word.\n The stem of a word itself, however, is not guaranteed to be a word.\n It is best to consider a stem as an opaque byte sequence.\n All that is guaranteed about a stem is that,\n for a given word,\n the stem of that word will always be the same byte sequence.\n Hence,\n you should never compare the result of one of the <code xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">stem()</code>\n functions against a non-stemmed string,\n for example:\n <pre xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">\n  if ( ft:stem( \"apples\" ) eq \"apple\" )             ** WRONG **\n </pre>\n Instead do:\n <pre xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">\n  if ( ft:stem( \"apples\" ) eq ft:stem( \"apple\" ) )  ** CORRECT **\n </pre>\n <h2 xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Notes on the thesaurus</h2>\n The <code xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">thesaurus-lookup()</code> functions have \"levels\"\n and \"relationship\" parameters.\n The values for these are implementation-defined.\n The default implementation uses the\n <a xmlns:xqdoc=\"http://www.xqdoc.org/1.0\" href=\"http://wordnet.princeton.edu/\">WordNet lexical database</a>,\n version 3.0.\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n In WordNet,\n the number of \"levels\" that two phrases are apart\n are how many hierarchical meanings apart they are.\n For example,\n \"canary\" is 5 levels away from \"vertebrate\"\n (carary &gt; finch &gt; oscine &gt; passerine &gt; bird &gt; vertebrate).\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n When using the WordNet implementation,\n all of the relationships (and their abbreviations)\n specified by\n <a xmlns:xqdoc=\"http://www.xqdoc.org/1.0\" href=\"http://www.iso.org/iso/iso_catalogue/catalogue_tc/catalogue_detail.htm?csnumber=7776\">ISO 2788</a>\n and\n <a xmlns:xqdoc=\"http://www.xqdoc.org/1.0\" href=\"http://www.niso.org/kst/reports/standards?step=2&amp;gid=&amp;project_key=7cc9b583cb5a62e8c15d3099e0bb46bbae9cf38a\">ANSI/NISO Z39.19-2005</a>\n with the exceptions of \"HN\" (history note)\n and \"X SN\" (see scope note for) are supported.\n These relationships are:\n  <table xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">\n    <tr>\n      <th>Rel.</th>\n      <th>Meaning</th>\n      <th>WordNet Rel.</th>\n    </tr>\n    <tr>\n      <td>BT</td>\n      <td>broader term</td>\n      <td>hypernym</td>\n    </tr>\n    <tr>\n      <td>BTG</td>\n      <td>broader term generic</td>\n      <td>hypernym</td>\n    </tr>\n    <tr>\n      <td>BTI</td>\n      <td>broader term instance</td>\n      <td>instance hypernym</td>\n    </tr>\n    <tr>\n      <td>BTP</td>\n      <td>broader term partitive</td>\n      <td>part meronym</td>\n    </tr>\n    <tr>\n      <td>NT</td>\n      <td>narrower term</td>\n      <td>hyponym</td>\n    </tr>\n    <tr>\n      <td>NTG</td>\n      <td>narrower term generic</td>\n      <td>hyponym</td>\n    </tr>\n    <tr>\n      <td>NTI</td>\n      <td>narrower term instance</td>\n      <td>instance hyponym</td>\n    </tr>\n    <tr>\n      <td>NTP</td>\n      <td>narrower term partitive</td>\n      <td>part holonym</td>\n    </tr>\n    <tr>\n      <td>RT</td>\n      <td>related term</td>\n      <td>also see</td>\n    </tr>\n    <tr>\n      <td>SN</td>\n      <td>scope note</td>\n      <td>n/a</td>\n    </tr>\n    <tr>\n      <td>TT</td>\n      <td>top term</td>\n      <td>hypernym</td>\n    </tr>\n    <tr>\n      <td>UF</td>\n      <td>non-preferred term</td>\n      <td>n/a</td>\n    </tr>\n    <tr>\n      <td>USE</td>\n      <td>preferred term</td>\n      <td>n/a</td>\n    </tr>\n  </table>\n Note that you can specify relationships\n either by their abbreviation\n or their meaning.\n Relationships are case-insensitive.\n In addition to the\n <a xmlns:xqdoc=\"http://www.xqdoc.org/1.0\" href=\"http://www.iso.org/iso/iso_catalogue/catalogue_tc/catalogue_detail.htm?csnumber=7776\">ISO 2788</a>\n and\n <a xmlns:xqdoc=\"http://www.xqdoc.org/1.0\" href=\"http://www.niso.org/kst/reports/standards?step=2&amp;gid=&amp;project_key=7cc9b583cb5a62e8c15d3099e0bb46bbae9cf38a\">ANSI/NISO Z39.19-2005</a>\n relationships,\n All of the relationships offered by WordNet are also supported.\n These relationships are:\n  <table xmlns:xqdoc=\"http://www.xqdoc.org/1.0\" class=\"ft_rels\">\n    <tr>\n      <th>Relationship</th>\n      <th>Meaning</th>\n    </tr>\n    <tr>\n      <td nowrap=\"nowrap\">also see</td>\n      <td>\n        A word that is related to another,\n        e.g., for \"varnished\" (furniture)\n        one should <em>also see</em> \"finished.\"\n      </td>\n    </tr>\n    <tr>\n      <td>antonym</td>\n      <td>\n        A word opposite in meaning to another,\n        e.g., \"light\" is an <em>antonym</em> for \"heavy.\"\n      </td>\n    </tr>\n    <tr>\n      <td>attribute</td>\n      <td>\n        A noun for which adjectives express values,\n        e.g., \"weight\" is an <em>attribute</em>\n        for which the adjectives \"light\" and \"heavy\"\n        express values.\n      </td>\n    </tr>\n    <tr>\n      <td>cause</td>\n      <td>\n        A verb that causes another,\n        e.g., \"show\" is a <em>cause</em> of \"see.\"\n      </td>\n    </tr>\n    <tr>\n      <td nowrap=\"nowrap\">derivationally related form</td>\n      <td>\n        A word that is derived from a root word,\n        e.g., \"metric\" is a <em>derivationally related form</em> of \"meter.\"\n      </td>\n    </tr>\n    <tr>\n      <td nowrap=\"nowrap\">derived from adjective</td>\n      <td>\n        An adverb that is derived from an adjective,\n        e.g., \"correctly\" is <em>derived from the adjective</em> \"correct.\"\n      </td>\n    </tr>\n    <tr>\n      <td>entailment</td>\n      <td>\n        A verb that presupposes another,\n        e.g., \"snoring\" <em>entails</em> \"sleeping.\"\n      </td>\n    </tr>\n    <tr>\n      <td>hypernym</td>\n      <td>\n        A word with a broad meaning that more specific words fall under,\n        e.g., \"meal\" is a <em>hypernym</em> of \"breakfast.\"\n      </td>\n    </tr>\n    <tr>\n      <td>hyponym</td>\n      <td>\n        A word of more specific meaning than a general term applicable to it,\n        e.g., \"breakfast\" is a <em>hyponym</em> of \"meal.\"\n      </td>\n    </tr>\n    <tr>\n      <td nowrap=\"nowrap\">instance hypernym</td>\n      <td>\n        A word that denotes a category of some specific instance,\n        e.g., \"author\" is an <em>instance hypernym</em> of \"Asimov.\"\n      </td>\n    </tr>\n    <tr>\n      <td nowrap=\"nowrap\">instance hyponym</td>\n      <td>\n        A term that donotes a specific instance of some general category,\n        e.g., \"Asimov\" is an <em>instance hyponym</em> of \"author.\"\n      </td>\n    </tr>\n    <tr>\n      <td nowrap=\"nowrap\">member holonym</td>\n      <td>\n        A word that denotes a collection of individuals,\n        e.g., \"faculty\" is a <em>member holonym</em> of \"professor.\"\n      </td>\n    </tr>\n    <tr>\n      <td nowrap=\"nowrap\">member meronym</td>\n      <td>\n        A word that denotes a member of a larger group,\n        e.g., a \"person\" is a <em>member meronym</em> of a \"crowd.\"\n      </td>\n    </tr>\n    <tr>\n      <td nowrap=\"nowrap\">part holonym</td>\n      <td>\n        A word that denotes a larger whole comprised of some part,\n        e.g., \"car\" is a <em>part holonym</em> of \"engine.\"\n      </td>\n    </tr>\n    <tr>\n      <td nowrap=\"nowrap\">part meronym</td>\n      <td>\n        A word that denotes a part of a larger whole,\n        e.g., an \"engine\" is <em>part meronym</em> of a \"car.\"\n      </td>\n    </tr>\n    <tr>\n      <td nowrap=\"nowrap\">participle of verb</td>\n      <td>\n        An adjective that is the participle of some verb,\n        e.g., \"breaking\" is the <em>participle of the verb</em> \"break.\"\n      </td>\n    </tr>\n    <tr>\n      <td>pertainym</td>\n      <td>\n        An adjective that classifies its noun,\n        e.g., \"musical\" is a <em>pertainym</em> in \"musical instrument.\"\n      </td>\n    </tr>\n    <tr>\n      <td nowrap=\"nowrap\">similar to</td>\n      <td>\n        Similar, though not necessarily interchangeable, adjectives.\n        For example, \"shiny\" is <em>similar to</em> \"bright\",\n        but they have subtle differences.\n      </td>\n    </tr>\n    <tr>\n      <td nowrap=\"nowrap\">substance holonym</td>\n      <td>\n        A word that denotes a larger whole containing some constituent\n        substance, e.g., \"bread\" is a <em>substance holonym</em> of \"flour.\"\n      </td>\n    </tr>\n    <tr>\n      <td nowrap=\"nowrap\">substance meronym</td>\n      <td>\n        A word that denotes a constituant substance of some larger whole,\n        e.g., \"flour\" is a <em>substance meronym</em> of \"bread.\"\n      </td>\n    </tr>\n    <tr>\n      <td nowrap=\"nowrap\">verb group</td>\n      <td>\n        A verb that is a member of a group of similar verbs,\n        e.g., \"live\" is in the <em>verb group</em>\n        of \"dwell\", \"live\", \"inhabit\", etc.\n      </td>\n    </tr>\n  </table>\n <h2 xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Notes on tokenization</h2>\n For general information about the implementation of tokenization,\n including what constitutes a token,\n see the <a xmlns:xqdoc=\"http://www.xqdoc.org/1.0\" href=\"../zorba/ft_tokenizer.html\">Full Text Tokenizer</a> documentation.\n", "sees" : [  ], "authors" : [ "<xqdoc:author xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Paul J. Lucas</xqdoc:author>" ], "version" : null, "encoding" : "utf-8", "namespaces" : [ { "uri" : "http://www.w3.org/2005/xqt-errors", "prefix" : "err" }, { "uri" : "http://zorba.io/modules/full-text", "prefix" : "ft" }, { "uri" : "http://zorba.io/options/versioning", "prefix" : "ver" }, { "uri" : "http://zorba.io/errors", "prefix" : "zerr" } ], "functions" : [ { "isDocumented" : true, "arity" : 0, "name" : "current-compare-options", "qname" : "ft:current-compare-options", "signature" : "() as object() external", "description" : " Gets the current compare options.\n", "summary" : "<p> Gets the current compare options.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [  ], "returns" : { "type" : "object()", "description" : "said compare options." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 0, "name" : "current-lang", "qname" : "ft:current-lang", "signature" : "() as xs:language external", "description" : " Gets the current\n <a xmlns:xqdoc=\"http://www.xqdoc.org/1.0\" href=\"http://www.w3.org/TR/xmlschema-2/#language\">language</a>:\n either the language specified by the\n <code xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"><a href=\"http://www.w3.org/TR/xpath-full-text-10/#doc-xquery10-FTOptionDecl\">declare ft-option using</a>\n <a href=\"http://www.w3.org/TR/xpath-full-text-10/#ftlanguageoption\">language</a></code>\n statement (if any)\n or the one returned by <code xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">ft:host-lang()</code> (if none).\n", "summary" : "<p> Gets the current\n  language :\n either the language specified by the\n  declare ft-option using \n  language \n statement (if any)\n or the one returned by  ft:host-lang()  (if none).</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [  ], "returns" : { "type" : "xs:language", "description" : "said language." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 0, "name" : "host-lang", "qname" : "ft:host-lang", "signature" : "() as xs:language external", "description" : " Gets the host's current\n <a xmlns:xqdoc=\"http://www.xqdoc.org/1.0\" href=\"http://www.w3.org/TR/xmlschema-2/#language\">language</a>.\n The \"host\" is the computer on which the software is running.\n The host's current language is obtained as follows:\n  <ul xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">\n    <li>\n      For *nix systems:\n      <ol>\n        <li>\n          If <a ref=\"http://www.cplusplus.com/reference/clibrary/clocale/setlocale/\"><code>setlocale</code>(3)</a> returns non-null,\n          the language corresponding to that locale is used.\n        </li>\n        <li>\n          Else, if the <code>LANG</code> environment variable is set,\n          that language is ued.\n        </li>\n        <li>\n          Otherwise, there is no default language.\n        </li>\n      </ol>\n    </li>\n    <li>\n      For Windows systems,\n      the language corresponding to the locale returned by the\n      <a href=\"http://msdn.microsoft.com/en-us/library/windows/desktop/dd318101(v=vs.85).aspx\"><code>GetLocaleInfo()</code></a>\n      function is used.\n    </li>\n  </ul>\n", "summary" : "<p> Gets the host's current\n  language .</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [  ], "returns" : { "type" : "xs:language", "description" : "said language." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "is-stem-lang-supported", "qname" : "ft:is-stem-lang-supported", "signature" : "($lang as xs:language) as xs:boolean external", "description" : " Checks whether the given\n <a xmlns:xqdoc=\"http://www.xqdoc.org/1.0\" href=\"http://www.w3.org/TR/xmlschema-2/#language\">language</a>\n is supported for stemming.\n", "summary" : "<p> Checks whether the given\n  language \n is supported for stemming.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "lang", "type" : "xs:language", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The language to check.</div>" } ], "returns" : { "type" : "xs:boolean", "description" : "<code xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">true</code> only if the language is supported." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "is-stop-word-lang-supported", "qname" : "ft:is-stop-word-lang-supported", "signature" : "($lang as xs:language) as xs:boolean external", "description" : " Checks whether the given\n <a xmlns:xqdoc=\"http://www.xqdoc.org/1.0\" href=\"http://www.w3.org/TR/xmlschema-2/#language\">language</a>\n is supported for stop words.\n", "summary" : "<p> Checks whether the given\n  language \n is supported for stop words.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "lang", "type" : "xs:language", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The language to check.</div>" } ], "returns" : { "type" : "xs:boolean", "description" : "<code xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">true</code> only if the language is supported." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "is-stop-word", "qname" : "ft:is-stop-word", "signature" : "($word as xs:string) as xs:boolean external", "description" : " Checks whether the given word is a stop-word.\n", "summary" : "<p> Checks whether the given word is a stop-word.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "word", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The word to check. The word's <a href=\"http://www.w3.org/TR/xmlschema-2/#language\">language</a> is assumed to be the one returned by <code>ft:current-lang()</code>.</div>" } ], "returns" : { "type" : "xs:boolean", "description" : "<code xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">true</code> only if <code xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">$word</code> is a stop-word." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">err:FTST0009 if <code>ft:current-lang()</code> is not supported.</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 2, "name" : "is-stop-word", "qname" : "ft:is-stop-word", "signature" : "($word as xs:string, $lang as xs:language) as xs:boolean external", "description" : " Checks whether the given word is a stop-word.\n", "summary" : "<p> Checks whether the given word is a stop-word.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "word", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The word to check.</div>" }, { "name" : "lang", "type" : "xs:language", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The <a href=\"http://www.w3.org/TR/xmlschema-2/#language\">language</a> of <code>$word</code>.</div>" } ], "returns" : { "type" : "xs:boolean", "description" : "<code xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">true</code> only if <code xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">$word</code> is a stop-word." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">err:FTST0009 if <code>$lang</code> is not supported.</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 1, "name" : "is-thesaurus-lang-supported", "qname" : "ft:is-thesaurus-lang-supported", "signature" : "($lang as xs:language) as xs:boolean external", "description" : " Checks whether the given\n <a xmlns:xqdoc=\"http://www.xqdoc.org/1.0\" href=\"http://www.w3.org/TR/xmlschema-2/#language\">language</a>\n is supported for look-up using the default thesaurus.\n", "summary" : "<p> Checks whether the given\n  language \n is supported for look-up using the default thesaurus.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "lang", "type" : "xs:language", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The language to check.</div>" } ], "returns" : { "type" : "xs:boolean", "description" : "<code xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">true</code> only if the language is supported." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 2, "name" : "is-thesaurus-lang-supported", "qname" : "ft:is-thesaurus-lang-supported", "signature" : "($uri as xs:string, $lang as xs:language) as xs:boolean external", "description" : " Checks whether the given\n <a xmlns:xqdoc=\"http://www.xqdoc.org/1.0\" href=\"http://www.w3.org/TR/xmlschema-2/#language\">language</a>\n is supported for look-up using the thesaurus specified by the given URI.\n", "summary" : "<p> Checks whether the given\n  language \n is supported for look-up using the thesaurus specified by the given URI.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "uri", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The URI specifying the thesaurus to use.</div>" }, { "name" : "lang", "type" : "xs:language", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The language to check.</div>" } ], "returns" : { "type" : "xs:boolean", "description" : "<code xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">true</code> only if the language is supported." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">err:FTST0018 if <code>$uri</code> refers to a thesaurus that is not found in the statically known thesauri.</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 1, "name" : "is-tokenizer-lang-supported", "qname" : "ft:is-tokenizer-lang-supported", "signature" : "($lang as xs:language) as xs:boolean external", "description" : " Checks whether the given\n <a xmlns:xqdoc=\"http://www.xqdoc.org/1.0\" href=\"http://www.w3.org/TR/xmlschema-2/#language\">language</a>\n is supported for tokenization.\n", "summary" : "<p> Checks whether the given\n  language \n is supported for tokenization.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "lang", "type" : "xs:language", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The language to check.</div>" } ], "returns" : { "type" : "xs:boolean", "description" : "<code xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">true</code> only if the language is supported." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "stem", "qname" : "ft:stem", "signature" : "($word as xs:string) as xs:string external", "description" : " Stems the given word.\n", "summary" : "<p> Stems the given word.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "word", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The word to stem. The word's <a href=\"http://www.w3.org/TR/xmlschema-2/#language\">language</a> is assumed to be the one returned by <code>ft:current-lang()</code>.</div>" } ], "returns" : { "type" : "xs:string", "description" : "the stem of <code xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">$word</code>." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">err:FTST0009 if <code>ft:current-lang()</code> is not supported.</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 2, "name" : "stem", "qname" : "ft:stem", "signature" : "($word as xs:string, $lang as xs:language) as xs:string external", "description" : " Stems the given word.\n", "summary" : "<p> Stems the given word.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "word", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The word to stem.</div>" }, { "name" : "lang", "type" : "xs:language", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The <a href=\"http://www.w3.org/TR/xmlschema-2/#language\">language</a> of <code>$word</code>.</div>" } ], "returns" : { "type" : "xs:string", "description" : "the stem of <code xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">$word</code>." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">err:FTST0009 if <code>$lang</code> is not supported.</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 1, "name" : "strip-diacritics", "qname" : "ft:strip-diacritics", "signature" : "($string as xs:string) as xs:string external", "description" : " Strips all diacritical marks from all characters.\n", "summary" : "<p> Strips all diacritical marks from all characters.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "string", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The string to strip diacritical marks from.</div>" } ], "returns" : { "type" : "xs:string", "description" : "<code xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">$string</code> with diacritical marks stripped." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "thesaurus-lookup", "qname" : "ft:thesaurus-lookup", "signature" : "($phrase as xs:string) as xs:string* external", "description" : " Looks-up the given phrase in the default thesaurus.\n", "summary" : "<p> Looks-up the given phrase in the default thesaurus.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "phrase", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The phrase to look up. The phrase's <a href=\"http://www.w3.org/TR/xmlschema-2/#language\">language</a> is assumed to be the one returned by <code>ft:current-lang()</code>.</div>" } ], "returns" : { "type" : "xs:string*", "description" : "the related phrases if <code xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">$phrase</code> is found in the thesaurus or the empty sequence if not." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">err:FTST0009 if <code>ft:current-lang()</code> is not supported.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">zerr:ZXQP8401 if the thesaurus data file's version is not supported by the currently running version of the software.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">zerr:ZXQP8402 if the thesaurus data file's endianness does not match that of the CPU on which the software is currently running.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">zerr:ZXQP8403 if there was an error reading the thesaurus data.</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 2, "name" : "thesaurus-lookup", "qname" : "ft:thesaurus-lookup", "signature" : "($uri as xs:string, $phrase as xs:string) as xs:string* external", "description" : " Looks-up the given phrase in a thesaurus.\n", "summary" : "<p> Looks-up the given phrase in a thesaurus.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "uri", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The URI specifying the thesaurus to use.</div>" }, { "name" : "phrase", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The phrase to look up. The phrase's <a href=\"http://www.w3.org/TR/xmlschema-2/#language\">language</a> is assumed to be the one the one returned by <code>ft:current-lang()</code>.</div>" } ], "returns" : { "type" : "xs:string*", "description" : "the related phrases if <code xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">$phrase</code> is found in the thesaurus or the empty sequence if not." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">err:FTST0009 if <code>ft:current-lang()</code> is unsupported.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">err:FTST0018 if <code>$uri</code> refers to a thesaurus that is not found in the statically known thesauri.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">zerr:ZOSE0001 if the thesaurus data file could not be found.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">zerr:ZOSE0002 if the thesaurus data file is not a plain file.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">zerr:ZXQP8401 if the thesaurus data file's version is not supported by the currently running version of the software.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">zerr:ZXQP8402 if the thesaurus data file's endianness does not match that of the CPU on which the software is currently running.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">zerr:ZXQP8403 if there was an error reading the thesaurus data file.</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 3, "name" : "thesaurus-lookup", "qname" : "ft:thesaurus-lookup", "signature" : "($uri as xs:string, $phrase as xs:string, $lang as xs:language) as xs:string* external", "description" : " Looks-up the given phrase in the thesaurus specified by the given URI.\n", "summary" : "<p> Looks-up the given phrase in the thesaurus specified by the given URI.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "uri", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The URI specifying the thesaurus to use.</div>" }, { "name" : "phrase", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The phrase to look up.</div>" }, { "name" : "lang", "type" : "xs:language", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The <a href=\"http://www.w3.org/TR/xmlschema-2/#language\">language</a> of <code>$phrase</code>.</div>" } ], "returns" : { "type" : "xs:string*", "description" : "the related phrases if <code xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">$phrase</code> is found in the thesaurus or the empty sequence if not." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">err:FTST0009 if <code>$lang</code> is not supported.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">err:FTST0018 if <code>$uri</code> refers to a thesaurus that is not found in the statically known thesauri.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">zerr:ZOSE0001 if the thesaurus data file could not be found.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">zerr:ZOSE0002 if the thesaurus data file is not a plain file.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">zerr:ZXQP8401 if the thesaurus data file's version is not supported by the currently running version of the software.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">zerr:ZXQP8402 if the thesaurus data file's endianness does not match that of the CPU on which the software is currently running.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">zerr:ZXQP8403 if there was an error reading the thesaurus data file.</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 4, "name" : "thesaurus-lookup", "qname" : "ft:thesaurus-lookup", "signature" : "($uri as xs:string, $phrase as xs:string, $lang as xs:language, $relationship as xs:string) as xs:string* external", "description" : " Looks-up the given phrase in a thesaurus.\n", "summary" : "<p> Looks-up the given phrase in a thesaurus.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "uri", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The URI specifying the thesaurus to use.</div>" }, { "name" : "phrase", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The phrase to look up.</div>" }, { "name" : "lang", "type" : "xs:language", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The <a href=\"http://www.w3.org/TR/xmlschema-2/#language\">language</a> of <code>$phrase</code>.</div>" }, { "name" : "relationship", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The relationship the results are to have to <code>$phrase</code>.</div>" } ], "returns" : { "type" : "xs:string*", "description" : "the related phrases if <code xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">$phrase</code> is found in the thesaurus or the empty sequence if not." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">err:FTST0018 if <code>$uri</code> refers to a thesaurus that is not found in the statically known thesauri.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">err:FTST0009 if <code>$lang</code> is not supported.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">zerr:ZOSE0001 if the thesaurus data file could not be found.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">zerr:ZOSE0002 if the thesaurus data file is not a plain file.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">zerr:ZXQP8401 if the thesaurus data file's version is not supported by the currently running version of the software.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">zerr:ZXQP8402 if the thesaurus data file's endianness does not match that of the CPU on which the software is currently running.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">zerr:ZXQP8403 if there was an error reading the thesaurus data file.</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 6, "name" : "thesaurus-lookup", "qname" : "ft:thesaurus-lookup", "signature" : "($uri as xs:string, $phrase as xs:string, $lang as xs:language, $relationship as xs:string, $level-least as xs:integer, $level-most as xs:integer) as xs:string* external", "description" : " Looks-up the given phrase in a thesaurus.\n", "summary" : "<p> Looks-up the given phrase in a thesaurus.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "uri", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The URI specifying the thesaurus to use.</div>" }, { "name" : "phrase", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The phrase to look up.</div>" }, { "name" : "lang", "type" : "xs:language", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The <a href=\"http://www.w3.org/TR/xmlschema-2/#language\">language</a> of <code>$phrase</code>.</div>" }, { "name" : "relationship", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The relationship the results are to have to <code>$phrase</code>.</div>" }, { "name" : "level-least", "type" : "xs:integer", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The minimum number of levels within the thesaurus to be traversed.</div>" }, { "name" : "level-most", "type" : "xs:integer", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The maximum number of levels within the thesaurus to be traversed.</div>" } ], "returns" : { "type" : "xs:string*", "description" : "the related phrases if <code xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">$phrase</code> is found in the thesaurus or the empty sequence if not." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">err:FOCA0003 if either <code>$level-least</code> or <code>$level-most</code> is either negative or too large.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">err:FTST0018 if <code>$uri</code> refers to a thesaurus that is not found in the statically known thesauri.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">err:FTST0009 if <code>$lang</code> is not supported.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">zerr:ZOSE0001 if the thesaurus data file could not be found.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">zerr:ZOSE0002 if the thesaurus data file is not a plain file.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">zerr:ZXQP8401 if the thesaurus data file's version is not supported by the currently running version of the software.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">zerr:ZXQP8402 if the thesaurus data file's endianness does not match that of the CPU on which the software is currently running.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">zerr:ZXQP8403 if there was an error reading the thesaurus data file.</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 1, "name" : "tokenize-node", "qname" : "ft:tokenize-node", "signature" : "($node as node()) as object()* external", "description" : " Tokenizes the given node and all of its descendants.\n", "summary" : "<p> Tokenizes the given node and all of its descendants.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "node", "type" : "node()", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The node to tokenize. The node's default <a href=\"http://www.w3.org/TR/xmlschema-2/#language\">language</a> is assumed to be the one returned by <code>ft:current-lang()</code>.</div>" } ], "returns" : { "type" : "object()*", "description" : "a (possibly empty) sequence of tokens." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">err:FTST0009 if <code>ft:current-lang()</code> is not supported.</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 2, "name" : "tokenize-node", "qname" : "ft:tokenize-node", "signature" : "($node as node(), $lang as xs:language) as object()* external", "description" : " Tokenizes the given node and all of its decendants.\n", "summary" : "<p> Tokenizes the given node and all of its decendants.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "node", "type" : "node()", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The node to tokenize.</div>" }, { "name" : "lang", "type" : "xs:language", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The default <a href=\"http://www.w3.org/TR/xmlschema-2/#language\">language</a> of <code>$node</code>.</div>" } ], "returns" : { "type" : "object()*", "description" : "a (possibly empty) sequence of tokens." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">err:FTST0009 if <code>$lang</code> is not supported.</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 2, "name" : "tokenize-nodes", "qname" : "ft:tokenize-nodes", "signature" : "($includes as node()+, $excludes as node()*) as object()* external", "description" : " Tokenizes the set of nodes comprising <code xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">$includes</code> (and all of its\n descendants) but excluding <code xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">$excludes</code> (and all of its\n descendants), if any.\n", "summary" : "<p> Tokenizes the set of nodes comprising  $includes  (and all of its\n descendants) but excluding  $excludes  (and all of its\n descendants), if any.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "includes", "type" : "node()", "occurrence" : "+", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The set of nodes (and its descendants) to include. The default <a href=\"http://www.w3.org/TR/xmlschema-2/#language\">language</a> is assumed to be the one returned by <code>ft:current-lang()</code>.</div>" }, { "name" : "excludes", "type" : "node()", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The set of nodes (and its descendants) to exclude.</div>" } ], "returns" : { "type" : "object()*", "description" : "a (possibly empty) sequence of tokens." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">err:FTST0009 if <code>ft:current-lang()</code> is not supported.</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 3, "name" : "tokenize-nodes", "qname" : "ft:tokenize-nodes", "signature" : "($includes as node()+, $excludes as node()*, $lang as xs:language) as object()* external", "description" : " Tokenizes the set of nodes comprising <code xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">$includes</code> (and all of its\n descendants) but excluding <code xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">$excludes</code> (and all of its\n descendants), if any.\n", "summary" : "<p> Tokenizes the set of nodes comprising  $includes  (and all of its\n descendants) but excluding  $excludes  (and all of its\n descendants), if any.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "includes", "type" : "node()", "occurrence" : "+", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The set of nodes (and its descendants) to include.</div>" }, { "name" : "excludes", "type" : "node()", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The set of nodes (and its descendants) to exclude.</div>" }, { "name" : "lang", "type" : "xs:language", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The default <a href=\"http://www.w3.org/TR/xmlschema-2/#language\">language</a> for nodes.</div>" } ], "returns" : { "type" : "object()*", "description" : "a (possibly empty) sequence of tokens." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">err:FTST0009 if <code>$lang</code> is not supported.</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 1, "name" : "tokenize-string", "qname" : "ft:tokenize-string", "signature" : "($string as xs:string) as xs:string* external", "description" : " Tokenizes the given string.\n", "summary" : "<p> Tokenizes the given string.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "string", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The string to tokenize. The string's <a href=\"http://www.w3.org/TR/xmlschema-2/#language\">language</a> is assumed to be the one returned by <code>ft:current-lang()</code>.</div>" } ], "returns" : { "type" : "xs:string*", "description" : "a (possibly empty) sequence of tokens." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">err:FTST0009 if <code>ft:current-lang()</code> is not supported.</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 2, "name" : "tokenize-string", "qname" : "ft:tokenize-string", "signature" : "($string as xs:string, $lang as xs:language) as xs:string* external", "description" : " Tokenizes the given string.\n", "summary" : "<p> Tokenizes the given string.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "string", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The string to tokenize.</div>" }, { "name" : "lang", "type" : "xs:language", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The <a href=\"http://www.w3.org/TR/xmlschema-2/#language\">language</a> of <code>$string</code>.</div>" } ], "returns" : { "type" : "xs:string*", "description" : "a (possibly empty) sequence of tokens." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">err:FTST0009 if <code>$lang</code> is not supported.</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 0, "name" : "tokenizer-properties", "qname" : "ft:tokenizer-properties", "signature" : "() as object() external", "description" : " Gets properties of the tokenizer for the\n <a xmlns:xqdoc=\"http://www.xqdoc.org/1.0\" href=\"http://www.w3.org/TR/xmlschema-2/#language\">language</a>\n returned by <code xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">ft:current-lang()</code>.\n", "summary" : "<p> Gets properties of the tokenizer for the\n  language \n returned by  ft:current-lang() .</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [  ], "returns" : { "type" : "object()", "description" : "said properties." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">err:FTST0009 if <code>ft:current-lang()</code> is not supported.</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 1, "name" : "tokenizer-properties", "qname" : "ft:tokenizer-properties", "signature" : "($lang as xs:language) as object() external", "description" : " Gets properties of the tokenizer for the given\n <a xmlns:xqdoc=\"http://www.xqdoc.org/1.0\" href=\"http://www.w3.org/TR/xmlschema-2/#language\">language</a>.\n", "summary" : "<p> Gets properties of the tokenizer for the given\n  language .</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "lang", "type" : "xs:language", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The language of the tokenizer to get the properties of.</div>" } ], "returns" : { "type" : "object()", "description" : "said properties." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">err:FTST0009 if <code>$lang</code> is not supported. tokenization specifically.</xqdoc:error>" ] } ], "variables" : [ { "name" : "ft:LANG-DA", "type" : "xs:language", "description" : " Predeclared constant for the Danish\n <a xmlns:xqdoc=\"http://www.xqdoc.org/1.0\" href=\"http://www.w3.org/TR/xmlschema-2/#language\"><code>xs:language</code></a>.\n" }, { "name" : "ft:LANG-DE", "type" : "xs:language", "description" : " Predeclared constant for the German\n <a xmlns:xqdoc=\"http://www.xqdoc.org/1.0\" href=\"http://www.w3.org/TR/xmlschema-2/#language\"><code>xs:language</code></a>.\n" }, { "name" : "ft:LANG-EN", "type" : "xs:language", "description" : " Predeclared constant for the English\n <a xmlns:xqdoc=\"http://www.xqdoc.org/1.0\" href=\"http://www.w3.org/TR/xmlschema-2/#language\"><code>xs:language</code></a>.\n" }, { "name" : "ft:LANG-ES", "type" : "xs:language", "description" : " Predeclared constant for the Spanish\n <a xmlns:xqdoc=\"http://www.xqdoc.org/1.0\" href=\"http://www.w3.org/TR/xmlschema-2/#language\"><code>xs:language</code></a>.\n" }, { "name" : "ft:LANG-FI", "type" : "xs:language", "description" : " Predeclared constant for the Finnish\n <a xmlns:xqdoc=\"http://www.xqdoc.org/1.0\" href=\"http://www.w3.org/TR/xmlschema-2/#language\"><code>xs:language</code></a>.\n" }, { "name" : "ft:LANG-FR", "type" : "xs:language", "description" : " Predeclared constant for the French\n <a xmlns:xqdoc=\"http://www.xqdoc.org/1.0\" href=\"http://www.w3.org/TR/xmlschema-2/#language\"><code>xs:language</code></a>.\n" }, { "name" : "ft:LANG-HU", "type" : "xs:language", "description" : " Predeclared constant for the Hungarian\n <a xmlns:xqdoc=\"http://www.xqdoc.org/1.0\" href=\"http://www.w3.org/TR/xmlschema-2/#language\"><code>xs:language</code></a>.\n" }, { "name" : "ft:LANG-IT", "type" : "xs:language", "description" : " Predeclared constant for the Italian\n <a xmlns:xqdoc=\"http://www.xqdoc.org/1.0\" href=\"http://www.w3.org/TR/xmlschema-2/#language\"><code>xs:language</code></a>.\n" }, { "name" : "ft:LANG-NL", "type" : "xs:language", "description" : " Predeclared constant for the Dutch\n <a xmlns:xqdoc=\"http://www.xqdoc.org/1.0\" href=\"http://www.w3.org/TR/xmlschema-2/#language\"><code>xs:language</code></a>.\n" }, { "name" : "ft:LANG-NO", "type" : "xs:language", "description" : " Predeclared constant for the Norwegian\n <a xmlns:xqdoc=\"http://www.xqdoc.org/1.0\" href=\"http://www.w3.org/TR/xmlschema-2/#language\"><code>xs:language</code></a>.\n" }, { "name" : "ft:LANG-PT", "type" : "xs:language", "description" : " Predeclared constant for the Portuguese\n <a xmlns:xqdoc=\"http://www.xqdoc.org/1.0\" href=\"http://www.w3.org/TR/xmlschema-2/#language\"><code>xs:language</code></a>.\n" }, { "name" : "ft:LANG-RO", "type" : "xs:language", "description" : " Predeclared constant for the Romanian\n <a xmlns:xqdoc=\"http://www.xqdoc.org/1.0\" href=\"http://www.w3.org/TR/xmlschema-2/#language\"><code>xs:language</code></a>.\n" }, { "name" : "ft:LANG-RU", "type" : "xs:language", "description" : " Predeclared constant for the Russian\n <a xmlns:xqdoc=\"http://www.xqdoc.org/1.0\" href=\"http://www.w3.org/TR/xmlschema-2/#language\"><code>xs:language</code></a>.\n" }, { "name" : "ft:LANG-SV", "type" : "xs:language", "description" : " Predeclared constant for the Swedish\n <a xmlns:xqdoc=\"http://www.xqdoc.org/1.0\" href=\"http://www.w3.org/TR/xmlschema-2/#language\"><code>xs:language</code></a>.\n" }, { "name" : "ft:LANG-TR", "type" : "xs:language", "description" : " Predeclared constant for the Turkish\n <a xmlns:xqdoc=\"http://www.xqdoc.org/1.0\" href=\"http://www.w3.org/TR/xmlschema-2/#language\"><code>xs:language</code></a>.\n" } ] }, "http://www.zorba-xquery.com/modules/jdbc" : { "ns" : "http://www.zorba-xquery.com/modules/jdbc", "description" : " This module contains functions to connect to any JDBC datasource\n using jvm-util module to handle Java interaction.\n <h2 xmlns:xqdoc=\"http://www.xqdoc.org/1.0\" id=\"determinism\">Important Notice Regarding Function Determinism</h2>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">The non side-effecting functions:\n <ul>\n   <li><a href=\"?anchor=connect-1\">connect#1</a></li>\n   <li><a href=\"?anchor=connect-2\">connect#2</a></li>\n   <li><a href=\"?anchor=execute-query-2\">execute-query#2</a></li>\n   <li><a href=\"?anchor=execute-query-prepared-1\">execute-query-prepared#1</a></li>\n   <li><a href=\"?anchor=tables-1\">tables#1</a></li>\n   <li><a href=\"?anchor=tables-4\">tables#4</a></li>\n </ul>\n are declared deterministic, which means that their results could be cached\n when invoked multiple times with the same arguments in the same query execution.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">To not use cached results you can use the following alternative functions:\n <ul>\n   <li><a href=\"?anchor=connect-notdeterministic-0\">connect-notdeterministic#0</a></li>\n   <li><a href=\"?anchor=connect-notdeterministic-1\">connect-notdeterministic#1</a></li>\n   <li><a href=\"?anchor=connect-notdeterministic-2\">connect-notdeterministic#2</a></li>\n   <li><a href=\"?anchor=execute-query-notdeterministic-2\">execute-query-notdeterministic#2</a></li>\n   <li><a href=\"?anchor=execute-query-prepared-notdeterministic-1\">execute-query-prepared-notdeterministic#1</a></li>\n   <li><a href=\"?anchor=tables-notdeterministic-1\">tables-notdeterministic#1</a></li>\n   <li><a href=\"?anchor=tables-notdeterministic-4\">tables-notdeterministic#4</a></li>\n </ul>\n which have been declared as being non deterministic.</p>\n", "sees" : [  ], "authors" : [ "<xqdoc:author xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Rodolfo Ochoa</xqdoc:author>", "<xqdoc:author xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Federico Cavalieri</xqdoc:author>" ], "version" : null, "encoding" : "utf-8", "namespaces" : [ { "uri" : "http://zorba.io/annotations", "prefix" : "an" }, { "uri" : "http://www.w3.org/2005/xqt-errors", "prefix" : "err" }, { "uri" : "http://www.zorba-xquery.com/modules/jdbc", "prefix" : "jdbc" }, { "uri" : "http://zorba.io/options/versioning", "prefix" : "ver" } ], "functions" : [ { "isDocumented" : true, "arity" : 1, "name" : "affected-rows", "qname" : "jdbc:affected-rows", "signature" : "($dataset-id as xs:anyURI) as xs:integer external", "description" : " Return the number of affected rows of a particular DataSet.\n", "summary" : "<p> Return the number of affected rows of a particular DataSet.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "dataset-id", "type" : "xs:anyURI", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The identifier to a DataSet.</div>" } ], "returns" : { "type" : "xs:integer", "description" : "Returns an xs:integer with the number of affected rows." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">SQL008 DataSet doesn't exist.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">SQL08000 Connection is closed.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">SQL001 Descriptive error, see error in attached message.</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 1, "name" : "clear-params", "qname" : "jdbc:clear-params", "signature" : "($prepared-statement as xs:anyURI) as empty-sequence() external", "description" : " Clears the current parameter values immediately.\n", "summary" : "<p> Clears the current parameter values immediately.</p>", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "prepared-statement", "type" : "xs:anyURI", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The identifier to a prepared statement.</div>" } ], "returns" : { "type" : "empty-sequence()", "description" : "This function returns an empty-sequence()" }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">SQL003 Prepared statement doesn't exist.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">SQL08000 Connection is closed.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">SQL001 Descriptive error, see error in attached message.</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 1, "name" : "close-dataset", "qname" : "jdbc:close-dataset", "signature" : "($dataset-id as xs:anyURI) as empty-sequence() external", "description" : " Closes and free resources from a particular DataSet.\n", "summary" : "<p> Closes and free resources from a particular DataSet.</p>", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "dataset-id", "type" : "xs:anyURI", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The identifier to a DataSet.</div>" } ], "returns" : { "type" : "empty-sequence()", "description" : "This function returns an empty-sequence()" }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">SQL008 DataSet doesn't exist.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">SQL08000 Connection is closed.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">SQL001 Descriptive error, see error in attached message.</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 1, "name" : "close-prepared", "qname" : "jdbc:close-prepared", "signature" : "($prepared-statement as xs:anyURI) as empty-sequence() external", "description" : " Closes and frees from memory any prepared SQL statement created with jdbc:prepare-statement\n", "summary" : "<p> Closes and frees from memory any prepared SQL statement created with jdbc:prepare-statement\n</p>", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "prepared-statement", "type" : "xs:anyURI", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The identifier to a prepared statement.</div>" } ], "returns" : { "type" : "empty-sequence()", "description" : "This function returns an empty-sequence()" }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">SQL003 Prepared statement doesn't exist.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">SQL08000 Connection is closed.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">SQL001 Descriptive error, see error in attached message.</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 1, "name" : "commit", "qname" : "jdbc:commit", "signature" : "($connection-id as xs:anyURI) as empty-sequence() external", "description" : " Commit current transaction from an active connection.\n", "summary" : "<p> Commit current transaction from an active connection.</p>", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "connection-id", "type" : "xs:anyURI", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The identifier to the connection to be commited.</div>" } ], "returns" : { "type" : "empty-sequence()", "description" : "This function returns an empty-sequence()" }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">SQL08003 Connection doesn't exist</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">SQL08000 Connection is closed</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">SQL001 Descriptive error, see error in attached message</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 1, "name" : "connect-nondeterministic", "qname" : "jdbc:connect-nondeterministic", "signature" : "($connection-config as object()) as xs:anyURI external", "description" : " Opens a connection to a database.\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">This function has the same semantics as <a href=\"#connect-1\">connect#1</a>,\n but is declared as being non deterministic and thus should only be used when\n result caching is not desired.</p>\n", "summary" : "<p> Opens a connection to a database.</p>", "annotation_str" : " %an:nondeterministic", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "nondeterministic", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "connection-config", "type" : "object()", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> json object that has the host and user informations.</div>" } ], "returns" : { "type" : "xs:anyURI", "description" : "Return an identifier that represents the connection to the server." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">SQL28000 Authentication failed.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">SQL08001 Connection error.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">SQL40003 Isolation level not supported.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">SQL001 Descriptive error, see attached message.</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 2, "name" : "connect-nondeterministic", "qname" : "jdbc:connect-nondeterministic", "signature" : "($connection-config as object(), $options as object()?) as xs:anyURI external", "description" : " Opens a connection to a database with specified options.\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">This function has the same semantics as <a href=\"#connect-2\">connect#2</a>,\n but is declared as being non deterministic and thus should only be used when\n result caching is not desired.</p>\n", "summary" : "<p> Opens a connection to a database with specified options.</p>", "annotation_str" : " %an:nondeterministic", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "nondeterministic", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "connection-config", "type" : "object()", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> json object that has the host and user informations.</div>" }, { "name" : "options", "type" : "object()", "occurrence" : "?", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> json object that specifies the connection options.</div>" } ], "returns" : { "type" : "xs:anyURI", "description" : "Return an identifier that represents the connection to the server." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">SQL28000 Authentication failed.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">SQL08001 Connection error.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">SQL40003 Isolation level not supported.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">SQL001 Descriptive error, see error in attached message.</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 1, "name" : "connect", "qname" : "jdbc:connect", "signature" : "($connection-config as object()) as xs:anyURI external", "description" : " Opens a connection to a database.\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">This function is declared as deterministic and should be used whenever result\n caching is acceptable.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Returns a URI identifying the connection that has been opened. The\n implementing code determines from the $connection-config either explicitly\n (interpreting the driver attribute) or implicitly (using the type attribute)\n which driver it has to load.</p>\n", "summary" : "<p> Opens a connection to a database.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "connection-config", "type" : "object()", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> json object that has the host and user informations.</div>" } ], "returns" : { "type" : "xs:anyURI", "description" : "Return an identifier that represents the connection to the server. Connection coonfiguration example: { \"url\" : \"jdbc:mysql://localhost/\", \"user\" : \"root\", \"password\" : \"\" }" }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">SQL28000 Authentication failed.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">SQL08001 Connection error.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">SQL40003 Isolation level not supported.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">SQL001 Descriptive error, see attached message.</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 2, "name" : "connect", "qname" : "jdbc:connect", "signature" : "($connection-config as object(), $options as object()?) as xs:anyURI external", "description" : " Opens a connection to a database with specified options.\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">This function is declared as deterministic and should be used whenever result\n caching is acceptable.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Returns a URI identifying the connection that has been opened. The\n implementing code determines from the $connection-config either explicitly\n (interpreting the driver attribute) or implicitly (using the type attribute)\n which driver it has to load.</p>\n", "summary" : "<p> Opens a connection to a database with specified options.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "connection-config", "type" : "object()", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> json object that has the host and user informations.</div>" }, { "name" : "options", "type" : "object()", "occurrence" : "?", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> json object that specifies the connection options.</div>" } ], "returns" : { "type" : "xs:anyURI", "description" : "Return an identifier that represents the connection to the server. Connection options example: { \"autocommit\" : false, \"readonly\"? : true, \"isolation-level\"? : $jdbc:READ-COMMITTED }" }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">SQL28000 Authentication failed.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">SQL08001 Connection error.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">SQL40003 Isolation level not supported.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">SQL001 Descriptive error, see error in attached message.</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 1, "name" : "connection-options", "qname" : "jdbc:connection-options", "signature" : "($connection-id as xs:anyURI) as object() external", "description" : " Returns a set with options for a specified connection.\n", "summary" : "<p> Returns a set with options for a specified connection.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "connection-id", "type" : "xs:anyURI", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The identifier to the connection to be verify.</div>" } ], "returns" : { "type" : "object()", "description" : "Returns and object with the connection options. The returned options are equal to the options specified in function jdbc:connect. Consequently, the options are specified as follows: { \"autocommit\" : xs:boolean, \"readonly\" : xs:boolean, \"isolation-level\" : xs:string }" }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">SQL08003 Connection doesn't exist</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">SQL08000 Connection is closed</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">SQL001 Descriptive error, see error in attached message</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 1, "name" : "execute-prepared", "qname" : "jdbc:execute-prepared", "signature" : "($prepared-statement as xs:anyURI) as xs:anyURI external", "description" : " Executes SQL statements prepared with 5.1 jsql:prepare-statement with values set\n and returns an identifier to a Dataset.\n", "summary" : "<p> Executes SQL statements prepared with 5.</p>", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "prepared-statement", "type" : "xs:anyURI", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The identifier to a prepared statement.</div>" } ], "returns" : { "type" : "xs:anyURI", "description" : "Return an identifier that represents a DataSet." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">SQL003 Prepared statement doesn't exist.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">SQL08000 Connection is closed.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">SQL001 Descriptive error, see error in attached message.</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 2, "name" : "execute-query-nondeterministic", "qname" : "jdbc:execute-query-nondeterministic", "signature" : "($connection-id as xs:anyURI, $sql as xs:string) as object()* external", "description" : " Executes non-updating SQL statements.\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">This function has the same semantics as\n <a href=\"#execute-query-2\">execute-query#2</a>,\n but is declared as being non deterministic and thus should only be used when\n result caching is not desired.</p>\n", "summary" : "<p> Executes non-updating SQL statements.</p>", "annotation_str" : " %an:nondeterministic", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "nondeterministic", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "connection-id", "type" : "xs:anyURI", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The identifier to an active connection.</div>" }, { "name" : "sql", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The query string to be executed.</div>" } ], "returns" : { "type" : "object()*", "description" : "Return an object with the result data rows from the query provided, the data rows are defined as follows: { column:value* }* Every row is represented by an object of column-value representation of the returned SQL result." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">SQL08003 Connection doesn't exist.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">SQL08000 Connection is closed.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">SQL005 The statement is Updating type.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">SQL001 Descriptive error, see error in attached message.</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 1, "name" : "execute-query-prepared-nondeterministic", "qname" : "jdbc:execute-query-prepared-nondeterministic", "signature" : "($prepared-statement as xs:anyURI) as object()* external", "description" : " Executes a non-updating SQL statement prepared with 5.1 jsql:prepare-statement.\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">This function has the same semantics as\n <a href=\"#execute-query-prepared-1\">execute-query-prepared#1</a>,\n but is declared as being non deterministic and thus should only be used when\n result caching is not desired.</p>\n", "summary" : "<p> Executes a non-updating SQL statement prepared with 5.</p>", "annotation_str" : " %an:nondeterministic", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "nondeterministic", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "prepared-statement", "type" : "xs:anyURI", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The identifier to a prepared statement.</div>" } ], "returns" : { "type" : "object()*", "description" : "Return an object with the result data rows from the query processed with the parameter values provided, the data rows are defined as follows: { column:value* }* Every row is represented by an object of column-value representation of the returned SQL result." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">SQL003 Prepared statement doesn't exist.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">SQL005 The prepared statement is an updating query.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">SQL08000 Connection is closed.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">SQL001 Descriptive error, see error in attached message.</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 1, "name" : "execute-query-prepared", "qname" : "jdbc:execute-query-prepared", "signature" : "($prepared-statement as xs:anyURI) as object()* external", "description" : " Executes a non-updating SQL statement prepared with 5.1 jsql:prepare-statement.\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">This function is declared as deterministic and should be used whenever result\n caching is acceptable.</p>\n", "summary" : "<p> Executes a non-updating SQL statement prepared with 5.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "prepared-statement", "type" : "xs:anyURI", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The identifier to a prepared statement.</div>" } ], "returns" : { "type" : "object()*", "description" : "Return an object with the result data rows from the query processed with the parameter values provided, the data rows are defined as follows: { column:value* }* Every row is represented by an object of column-value representation of the returned SQL result." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">SQL003 Prepared statement doesn't exist.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">SQL005 The prepared statement is an updating query.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">SQL08000 Connection is closed.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">SQL001 Descriptive error, see error in attached message.</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 2, "name" : "execute-query", "qname" : "jdbc:execute-query", "signature" : "($connection-id as xs:anyURI, $sql as xs:string) as object()* external", "description" : " Executes non-updating SQL statements.\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">This function is declared as deterministic and should be used whenever result\n caching is acceptable.</p>\n", "summary" : "<p> Executes non-updating SQL statements.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "connection-id", "type" : "xs:anyURI", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The identifier to an active connection.</div>" }, { "name" : "sql", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The query string to be executed.</div>" } ], "returns" : { "type" : "object()*", "description" : "Return an object with the result data rows from the query provided, the data rows are defined as follows: { column:value* }* Every row is represented by an object of column-value representation of the returned SQL result." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">SQL08003 Connection doesn't exist.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">SQL08000 Connection is closed.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">SQL005 The statement is Updating type.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">SQL001 Descriptive error, see error in attached message.</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 1, "name" : "execute-update-prepared", "qname" : "jdbc:execute-update-prepared", "signature" : "($prepared-statement as xs:anyURI) as xs:integer external", "description" : " Executes an updating SQL statement prepared with 5.1 jsql:prepare-statement.\n", "summary" : "<p> Executes an updating SQL statement prepared with 5.</p>", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "prepared-statement", "type" : "xs:anyURI", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The identifier to a prepared statement.</div>" } ], "returns" : { "type" : "xs:integer", "description" : "Returns an xs:integer with the number of affected rows." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">SQL003 Prepared statement doesn't exist.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">SQL006 The prepared statement is a non-updating query.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">SQL08000 Connection is closed.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">SQL001 Descriptive error, see error in attached message.</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 2, "name" : "execute-update", "qname" : "jdbc:execute-update", "signature" : "($connection-id as xs:anyURI, $sql as xs:string) as xs:integer external", "description" : " Executes updating SQL statements.\n", "summary" : "<p> Executes updating SQL statements.</p>", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "connection-id", "type" : "xs:anyURI", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The identifier to an active connection.</div>" }, { "name" : "sql", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The query string to be executed.</div>" } ], "returns" : { "type" : "xs:integer", "description" : "Returns an xs:integer with the number of affected rows." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">SQL08003 Connection doesn't exist.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">SQL08000 Connection is closed.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">SQL005 The statement is Read-only type.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">SQL001 Descriptive error, see error in attached message.</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 2, "name" : "execute", "qname" : "jdbc:execute", "signature" : "($connection-id as xs:anyURI, $sql as xs:string) as xs:anyURI external", "description" : " Executes any kind of SQL statement.\n", "summary" : "<p> Executes any kind of SQL statement.</p>", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "connection-id", "type" : "xs:anyURI", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The identifier to an active connection.</div>" }, { "name" : "sql", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The query string to be executed.</div>" } ], "returns" : { "type" : "xs:anyURI", "description" : "Return an identifier that represents a DataSet." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">SQL08003 Connection doesn't exist.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">SQL08000 Connection is closed.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">SQL001 Descriptive error, see error in attached message.</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 1, "name" : "is-connected", "qname" : "jdbc:is-connected", "signature" : "($connection-id as xs:anyURI) as xs:boolean external", "description" : " Verify if a connection is still active.\n", "summary" : "<p> Verify if a connection is still active.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "connection-id", "type" : "xs:anyURI", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The identifier to the connection to be verify.</div>" } ], "returns" : { "type" : "xs:boolean", "description" : "Returns true if connected." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">SQL08003 Connection doesn't exist</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">SQL001 Descriptive error, see error in attached message</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 1, "name" : "metadata", "qname" : "jdbc:metadata", "signature" : "($dataset-id as xs:anyURI) as object() external", "description" : " Return the metadata of the result of a particular DataSet.\n", "summary" : "<p> Return the metadata of the result of a particular DataSet.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "dataset-id", "type" : "xs:anyURI", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The identifier to a DataSet.</div>" } ], "returns" : { "type" : "object()", "description" : "This function returns the metadata associated with an executed DataSet. More in detail, it returns information about column names, types, and whether a column can contain a null value. The metadata information can only be returned for DataSets that have been executed explicitly using the jsql:execute function. The metadata node returned by this function is defined as follows: { \"columns\": [ { \"name\": xs:string, \"type\": xs:string, \"autoincrement\"? = xs:boolean, \"nillable\"? = xs:boolean } * ] }" }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">SQL008 DataSet doesn't exist.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">SQL08000 Connection is closed.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">SQL001 Descriptive error, see error in attached message.</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 1, "name" : "parameter-metadata", "qname" : "jdbc:parameter-metadata", "signature" : "($prepared-statement as xs:anyURI) as object() external", "description" : " Retrieves the number, types and properties of the prepared statement parameters.\n", "summary" : "<p> Retrieves the number, types and properties of the prepared statement parameters.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "prepared-statement", "type" : "xs:anyURI", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The identifier to a prepared statement.</div>" } ], "returns" : { "type" : "object()", "description" : "This function returns the parameters metadata associated with a prepared statement. In other words, it returns information about the column name associated with the parameter, the type, etc. The metadata node returned by this function is defined as follows: { columns: [{ \"name\": xs:string, \"type\": xs:string }] }" }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">SQL003 Prepared statement doesn't exist.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">SQL08000 Connection is closed.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">SQL001 Descriptive error, see error in attached message.</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 2, "name" : "prepare-statement", "qname" : "jdbc:prepare-statement", "signature" : "($connection-id as xs:anyURI, $sql as xs:string) as xs:anyURI external", "description" : "  Creates a prepared statement for multiple executions with diferent values.\n", "summary" : "<p>  Creates a prepared statement for multiple executions with diferent values.</p>", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "connection-id", "type" : "xs:anyURI", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The identifier to an active connection.</div>" }, { "name" : "sql", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The query string to be executed.</div>" } ], "returns" : { "type" : "xs:anyURI", "description" : "Return an identifier that represents the prepared statement. Example: jsql:prepare-statement($connection, \"SELECT * FROM users WHERE id=? AND age&gt;?\")" }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">SQL08003 Connection doesn't exist.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">SQL08000 Connection is closed.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">SQL001 Descriptive error, see error in attached message.</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 1, "name" : "result-set", "qname" : "jdbc:result-set", "signature" : "($dataset-id as xs:anyURI) as object()* external", "description" : " This function returns a sequence of objects representing the rows of data from a non-updating query.\n", "summary" : "<p> This function returns a sequence of objects representing the rows of data from a non-updating query.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "dataset-id", "type" : "xs:anyURI", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The identifier to a DataSet.</div>" } ], "returns" : { "type" : "object()*", "description" : "Return an object with the result data rows from the DataSet provided, the data rows are defined as follows: { column:value* }* Every row is represented by an object of column-value representation of the returned SQL result." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">SQL008 DataSet doesn't exist.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">SQL08000 Connection is closed.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">SQL001 Descriptive error, see error in attached message.</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 1, "name" : "rollback", "qname" : "jdbc:rollback", "signature" : "($connection-id as xs:anyURI) as empty-sequence() external", "description" : " Rollback the current transaction of a connection.\n", "summary" : "<p> Rollback the current transaction of a connection.</p>", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "connection-id", "type" : "xs:anyURI", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The identifier to the connection to be rollbacked.</div>" } ], "returns" : { "type" : "empty-sequence()", "description" : "This function returns an empty-sequence()" }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">SQL08003 Connection doesn't exist</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">SQL08000 Connection is closed</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">SQL001 Descriptive error, see error in attached message</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 3, "name" : "set-boolean", "qname" : "jdbc:set-boolean", "signature" : "($prepared-statement as xs:anyURI, $parameter-index as xs:integer, $value as xs:boolean) as empty-sequence() external", "description" : " Set the value of the designated parameter with the given value, this function will assign only boolean values.\n", "summary" : "<p> Set the value of the designated parameter with the given value, this function will assign only boolean values.</p>", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "prepared-statement", "type" : "xs:anyURI", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The identifier to a prepared statement.</div>" }, { "name" : "parameter-index", "type" : "xs:integer", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The index from the parameter to be set.</div>" }, { "name" : "value", "type" : "xs:boolean", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The value to be set.</div>" } ], "returns" : { "type" : "empty-sequence()", "description" : "This function returns an empty-sequence()" }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">SQL003 Prepared statement doesn't exist.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">SQL007 Parameter casting error.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">SQL08000 Connection is closed.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">SQL001 Descriptive error, see error in attached message.</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 2, "name" : "set-null", "qname" : "jdbc:set-null", "signature" : "($prepared-statement as xs:anyURI, $parameter-index as xs:integer) as empty-sequence() external", "description" : " Set the value of the designated parameter with the given value, this function\n will assign only null values if possible.\n", "summary" : "<p> Set the value of the designated parameter with the given value, this function\n will assign only null values if possible.</p>", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "prepared-statement", "type" : "xs:anyURI", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The identifier to a prepared statement.</div>" }, { "name" : "parameter-index", "type" : "xs:integer", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The index from the parameter to be set.</div>" } ], "returns" : { "type" : "empty-sequence()", "description" : "This function returns an empty-sequence()" }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">SQL003 Prepared statement doesn't exist.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">SQL007 Parameter casting error.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">SQL08000 Connection is closed.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">SQL001 Descriptive error, see error in attached message.</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 3, "name" : "set-numeric", "qname" : "jdbc:set-numeric", "signature" : "($prepared-statement as xs:anyURI, $parameter-index as xs:decimal, $value as xs:anyAtomicType) as empty-sequence() external", "description" : " Set the value of the designated parameter with the given value, this function will assign only numeric values.\n", "summary" : "<p> Set the value of the designated parameter with the given value, this function will assign only numeric values.</p>", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "prepared-statement", "type" : "xs:anyURI", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The identifier to a prepared statement.</div>" }, { "name" : "parameter-index", "type" : "xs:decimal", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The index from the parameter to be set.</div>" }, { "name" : "value", "type" : "xs:anyAtomicType", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The value to be set.</div>" } ], "returns" : { "type" : "empty-sequence()", "description" : "This function returns an empty-sequence()" }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">SQL003 Prepared statement doesn't exist.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">SQL007 Parameter casting error.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">SQL08000 Connection is closed.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">SQL001 Descriptive error, see error in attached message.</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 3, "name" : "set-string", "qname" : "jdbc:set-string", "signature" : "($prepared-statement as xs:anyURI, $parameter-index as xs:integer, $value as xs:string) as empty-sequence() external", "description" : " Set the value of the designated parameter with the given value, this function will assign only string values.\n", "summary" : "<p> Set the value of the designated parameter with the given value, this function will assign only string values.</p>", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "prepared-statement", "type" : "xs:anyURI", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The identifier to a prepared statement.</div>" }, { "name" : "parameter-index", "type" : "xs:integer", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The index from the parameter to be set.</div>" }, { "name" : "value", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The value to be set.</div>" } ], "returns" : { "type" : "empty-sequence()", "description" : "This function returns an empty-sequence()" }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">SQL003 Prepared statement doesn't exist.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">SQL007 Parameter casting error.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">SQL08000 Connection is closed.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">SQL001 Descriptive error, see error in attached message.</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 3, "name" : "set-value", "qname" : "jdbc:set-value", "signature" : "($prepared-statement as xs:anyURI, $parameter-index as xs:decimal, $value as xs:anyAtomicType) as empty-sequence() external", "description" : " Set the value of the designated parameter with the given value,\n this function will assign any value you send\n and it will try to cast to the correct type.\n", "summary" : "<p> Set the value of the designated parameter with the given value,\n this function will assign any value you send\n and it will try to cast to the correct type.</p>", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "prepared-statement", "type" : "xs:anyURI", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The identifier to a prepared statement.</div>" }, { "name" : "parameter-index", "type" : "xs:decimal", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The index from the parameter to be set.</div>" }, { "name" : "value", "type" : "xs:anyAtomicType", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The value to be set.</div>" } ], "returns" : { "type" : "empty-sequence()", "description" : "This function returns an empty-sequence()" }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">SQL003 Prepared statement doesn't exist.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">SQL007 Parameter casting error.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">SQL08000 Connection is closed.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">SQL001 Descriptive error, see error in attached message.</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 1, "name" : "tables-nondeterministic", "qname" : "jdbc:tables-nondeterministic", "signature" : "($connection-id as xs:anyURI) as object()*", "description" : " Return the list of tables from a connection\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">This function has the same semantics as <a href=\"#tables-1\">tables#1</a>,\n but is declared as being non deterministic and thus should only be used when\n result caching is not desired.</p>\n", "summary" : "<p> Return the list of tables from a connection\n  This function has the same semantics as  tables#1 ,\n but is declared as being non deterministic and thus should only be used when\n result caching is not desired.</p>", "annotation_str" : " %an:nondeterministic", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "nondeterministic", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "connection-id", "type" : "xs:anyURI", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The identifier to a connection.</div>" } ], "returns" : { "type" : "object()*", "description" : "Return an object with the result data rows from the query provided, the data rows are defined as follows: { column:value* }* Every row is represented by an object of column-value representation of the returned SQL result." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">SQL08000 Connection is closed.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">SQL001 Descriptive error, see error in attached message.</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 4, "name" : "tables-nondeterministic", "qname" : "jdbc:tables-nondeterministic", "signature" : "($connection-id as xs:anyURI, $catalog as xs:string?, $schema as xs:string?, $table as xs:string?) as object()* external", "description" : " Return the list of tables from a connection.\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">This function has the same semantics as <a href=\"#tables-4\">tables#4</a>,\n but is declared as being non deterministic and thus should only be used when\n result caching is not desired.</p>\n", "summary" : "<p> Return the list of tables from a connection.</p>", "annotation_str" : " %an:nondeterministic", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "nondeterministic", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "connection-id", "type" : "xs:anyURI", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The identifier to a connection.</div>" }, { "name" : "catalog", "type" : "xs:string", "occurrence" : "?", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> A filter of the catalog name of the tables. Send empty-sequence for all tables.</div>" }, { "name" : "schema", "type" : "xs:string", "occurrence" : "?", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> A filter of the schema name of the tables. Send empty-sequence for all tables.</div>" }, { "name" : "table", "type" : "xs:string", "occurrence" : "?", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> A filter of the name of the tables. Send empty-sequence for all tables.</div>" } ], "returns" : { "type" : "object()*", "description" : "Return an object with the result data rows from the query provided, the data rows are defined as follows: { column:value* }* Every row is represented by an object of column-value representation of the returned SQL result." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">SQL08000 Connection is closed.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">SQL001 Descriptive error, see error in attached message.</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 1, "name" : "tables", "qname" : "jdbc:tables", "signature" : "($connection-id as xs:anyURI) as object()*", "description" : " Return the list of tables from a connection\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">This function is declared as deterministic and should be used whenever result\n caching is acceptable.</p>\n", "summary" : "<p> Return the list of tables from a connection\n  This function is declared as deterministic and should be used whenever result\n caching is acceptable.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "connection-id", "type" : "xs:anyURI", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The identifier to a connection.</div>" } ], "returns" : { "type" : "object()*", "description" : "Return an object with the result data rows from the query provided, the data rows are defined as follows: { column:value* }* Every row is represented by an object of column-value representation of the returned SQL result." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">SQL08000 Connection is closed.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">SQL001 Descriptive error, see error in attached message.</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 4, "name" : "tables", "qname" : "jdbc:tables", "signature" : "($connection-id as xs:anyURI, $catalog as xs:string?, $schema as xs:string?, $table as xs:string?) as object()* external", "description" : " Return the list of tables from a connection\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">This function is declared as deterministic and should be used whenever result\n caching is acceptable.</p>\n", "summary" : "<p> Return the list of tables from a connection\n  This function is declared as deterministic and should be used whenever result\n caching is acceptable.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "connection-id", "type" : "xs:anyURI", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The identifier to a connection.</div>" }, { "name" : "catalog", "type" : "xs:string", "occurrence" : "?", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> A filter of the catalog name of the tables. Send empty-sequence for all tables.</div>" }, { "name" : "schema", "type" : "xs:string", "occurrence" : "?", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> A filter of the schema name of the tables. Send empty-sequence for all tables.</div>" }, { "name" : "table", "type" : "xs:string", "occurrence" : "?", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> A filter of the name of the tables. Send empty-sequence for all tables.</div>" } ], "returns" : { "type" : "object()*", "description" : "Return an object with the result data rows from the query provided, the data rows are defined as follows: { column:value* }* Every row is represented by an object of column-value representation of the returned SQL result." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">SQL08000 Connection is closed.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">SQL001 Descriptive error, see error in attached message.</xqdoc:error>" ] } ], "variables" : [ { "name" : "jdbc:NOT-SUPPORTED", "type" : "item()*", "description" : " This variable represents the NOT-SUPPORTED level for Isolation Levels in $options for 2.2 connect function.\n" }, { "name" : "jdbc:READ-COMMITTED", "type" : "item()*", "description" : " This variable represents the READ-COMMITTED level for Isolation Levels in $options for 2.2 connect function.\n" }, { "name" : "jdbc:READ-UNCOMMITTED", "type" : "item()*", "description" : " This variable represents the READ-UNCOMMITTED level for Isolation Levels in $options for 2.2 connect function.\n" }, { "name" : "jdbc:REPEATABLE-READ", "type" : "item()*", "description" : " This variable represents the REPEATABLE-READ level for Isolation Levels in $options for 2.2 connect function.\n" }, { "name" : "jdbc:SERIALIZABLE", "type" : "item()*", "description" : " This variable represents the SERIALIZABLE level for Isolation Levels in $options for 2.2 connect function.\n" } ] }, "http://www.28msec.com/modules/http-response" : { "ns" : "http://www.28msec.com/modules/http-response", "description" : " The response module can be used to modify the HTTP response\n that will be send as a result. For example, the\n <tt xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">content-type</tt> function can be used to set the\n Content-Type header and determine the encoding of data in the\n response.\n", "sees" : [  ], "authors" : [ "<xqdoc:author xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Matthias Brantner</xqdoc:author>" ], "version" : null, "encoding" : "utf-8", "namespaces" : [ { "uri" : "http://zorba.io/annotations", "prefix" : "an" }, { "uri" : "http://www.28msec.com/modules/http-response", "prefix" : "http" }, { "uri" : "http://www.w3.org/2010/xslt-xquery-serialization", "prefix" : "output" }, { "uri" : "http://www.28msec.com/modules/http/response#2.0", "prefix" : "res" }, { "uri" : "http://www.28msec.com/modules/http-response", "prefix" : "resp" }, { "uri" : "http://zorba.io/options/versioning", "prefix" : "ver" } ], "functions" : [ { "isDocumented" : true, "arity" : 1, "name" : "code-for-status", "qname" : "resp:code-for-status", "signature" : "($status as xs:string) as xs:integer", "description" : " Convert between the name of the HTTP status code and\n the integer value of that status code.\n", "summary" : "<p> Convert between the name of the HTTP status code and\n the integer value of that status code.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "status", "type" : "xs:string", "occurrence" : null, "description" : "" } ], "returns" : { "type" : "xs:integer", "description" : "the integer value of the given status code." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">http:invalid-status if the given name does not represent a valid http status code</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 1, "name" : "content-type-binary", "qname" : "resp:content-type-binary", "signature" : "($type as xs:string) as xs:boolean", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Test if a given content-type is a binary content-type. A\n content-type is considered to be binary if it's not a text\n content-type.</p>\n", "summary" : "<p>  Test if a given content-type is a binary content-type.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "type", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the content-type to test.</div>" } ], "returns" : { "type" : "xs:boolean", "description" : "true if the given content-type is a binary content-type, false otherwise." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "content-type-html", "qname" : "resp:content-type-html", "signature" : "($type as xs:string) as xs:boolean", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Test if a given content-type is a HTML content-type, i.e.\n the content-type is the string \"text/html\".</p>\n", "summary" : "<p>  Test if a given content-type is a HTML content-type, i.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "type", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the content-type to test.</div>" } ], "returns" : { "type" : "xs:boolean", "description" : "true if the given content-type is the HTML content-type, false otherwise." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "content-type-json", "qname" : "resp:content-type-json", "signature" : "($type as xs:string) as xs:boolean", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Test if a given content-type is a JSON content-type, i.e.\n the content-type is the string \"application/json\".</p>\n", "summary" : "<p>  Test if a given content-type is a JSON content-type, i.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "type", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the content-type to test.</div>" } ], "returns" : { "type" : "xs:boolean", "description" : "true if the given content-type is the JSON content-type, false otherwise." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "content-type-mixed-json-xml", "qname" : "resp:content-type-mixed-json-xml", "signature" : "($type as xs:string) as xs:boolean", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Test if a given content-type is a mixed JSON-XML content-type, i.e.\n the content-type is the string \"application/mixed-json-xml\".</p>\n", "summary" : "<p>  Test if a given content-type is a mixed JSON-XML content-type, i.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "type", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the content-type to test.</div>" } ], "returns" : { "type" : "xs:boolean", "description" : "true if the given content-type is a mixed JSON-XML content-type, false otherwise." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "content-type-text", "qname" : "resp:content-type-text", "signature" : "($type as xs:string) as xs:boolean", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Test if a given content-type is a text content-type.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">A text content-type starts with \"text/\" or contains either of\n the strings \"xml\" or \"json\".</p>\n", "summary" : "<p>  Test if a given content-type is a text content-type.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "type", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the content-type to test.</div>" } ], "returns" : { "type" : "xs:boolean", "description" : "true if the given content-type is a text content-type, false otherwise." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "content-type-xhtml", "qname" : "resp:content-type-xhtml", "signature" : "($type as xs:string) as xs:boolean", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Test if a given content-type is a XHTML content-type, i.e.\n the content-type is the string \"application/xhtml+xml\".</p>\n", "summary" : "<p>  Test if a given content-type is a XHTML content-type, i.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "type", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the content-type to test.</div>" } ], "returns" : { "type" : "xs:boolean", "description" : "true if the given content-type is the XHTML content-type, false otherwise." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "content-type-xml", "qname" : "resp:content-type-xml", "signature" : "($type as xs:string) as xs:boolean", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Test if a given content-type is a XML content-type</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">A XML content-type is a content-type that is \"application/xml\"\n or ends : with the string \"+xml\".</p>\n", "summary" : "<p>  Test if a given content-type is a XML content-type \n  A XML content-type is a content-type that is \"application/xml\"\n or ends : with the string \"+xml\".</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "type", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the content-type to test.</div>" } ], "returns" : { "type" : "xs:boolean", "description" : "true if the given content-type is a XML content-type, false otherwise." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "content-type", "qname" : "resp:content-type", "signature" : "($type as xs:string) as empty-sequence()", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Sets the Content-Type header for the response.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">\n For example,\n <tt>resp:content-type(\"text/plain\")</tt> will cause the\n header\n <tt>Content-Type: text/plain</tt> to be added for the response.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">In addtion to setting the Content-Type header, the function also\n sets the serialization parameters to the default values for serialization\n method for the given content-type. For example, if the content-type is set to\n text/plain, the default serialization parameters for the text\n serialization method will be used.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">The functions resp:content-type-text/xml/html/xhtml/binary may be used\n to figure out which serialization method will be used for a specific\n content-type.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">If the given content-type contains a charset declaration (e.g.\n <tt>resp:content-type(\"text/plain;charset=ISO-8859-1\")</tt>), the\n content of the response will be transcoded to the given encoding.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">The default serialization parameters set by this function can\n be overwritten using the resp:serialization-parameters() function.</p>\n", "summary" : "<p>  Sets the Content-Type header for the response.</p>", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "type", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the content-type to be set</div>" } ], "returns" : { "type" : "empty-sequence()", "description" : "the function is sequential and returns the empty sequence" }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">http:invalid-encoding if the given encoding is invalid or not supported.</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 2, "name" : "content-type", "qname" : "resp:content-type", "signature" : "($type as xs:string, $params as object()) as empty-sequence()", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Sets the Content-Type header for the response.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">For example,\n <tt>resp:content-type(\"text/plain\")</tt> will cause the\n header\n <tt>Content-Type: text/plain</tt> to be added for the response.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">In addition to the content-type, the function also allows to\n specify the serialization method and parameters that will be used\n for serializing the result. For details about this parameter, please\n refer to resp:serialization-parameters#1.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">If the content-type suggest using a binary serialization,\n resp:decode-binary#1 will be called and set to true except if\n the serialization method is specified at the same time. In this\n case, the serialization method will be used as specified.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">If the given content-type contains a charset declaration (e.g.\n <tt>resp:content-type(\"text/plain;charset=ISO-8859-1\")</tt>), the\n content of the response will be transcoded to the given encoding.\n If the serialization parameters also contain a charset declaration,\n the charset contained in the content-type will be used. to transcode\n the result.</p>\n", "summary" : "<p>  Sets the Content-Type header for the response.</p>", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "type", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the content-type to be set</div>" }, { "name" : "params", "type" : "object()", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the serialization parameters that will be used for serialization the result.</div>" } ], "returns" : { "type" : "empty-sequence()", "description" : "the function is sequential and returns the empty sequence" }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">http:invalid-encoding if the given encoding is invalid or not supported.</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 0, "name" : "decode-binary", "qname" : "resp:decode-binary", "signature" : "() as xs:boolean", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Returns the boolean indiciating whether base64Binary returned by\n this request will be decoded.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">The default if not modified using resp:decode-binary is false.</p>\n", "summary" : "<p>  Returns the boolean indiciating whether base64Binary returned by\n this request will be decoded.</p>", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [  ], "returns" : { "type" : "xs:boolean", "description" : "the said boolean value" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "decode-binary", "qname" : "resp:decode-binary", "signature" : "($decode as xs:boolean) as empty-sequence()", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">If this function is invoked with true, the result of the request\n will not be serialized. Instead, the result will be the binary values\n of any item that is of type base64Binary. The values of all other\n items will not be part of the result.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Please note that all serialization parameters which have been set using\n resp:serialization-parameters will be ignored. Also, the output encoding\n is ignored for binaries.</p>\n", "summary" : "<p>  If this function is invoked with true, the result of the request\n will not be serialized.</p>", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "decode", "type" : "xs:boolean", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> boolean value indicating whether base64Binary should be decoded.</div>" } ], "returns" : { "type" : "empty-sequence()", "description" : "the function is sequential and returns the empty sequence." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "encoding", "qname" : "resp:encoding", "signature" : "($encoding as xs:string) as empty-sequence()", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">The function sets the output encoding that will be\n used for the payload of the response.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">For example,\n <tt>resp:encoding(\"ISO-8859-1\")</tt> will cause the\n content in the response to be encoded using ISO-8859-1.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Please note that the encoding only applies to textual data.\n It is not used if the function <tt>decode-binary()</tt>\n was invoked passing true as parameter.</p>\n", "summary" : "<p>  The function sets the output encoding that will be\n used for the payload of the response.</p>", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "encoding", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the encoding to be used for the payload of the response.</div>" } ], "returns" : { "type" : "empty-sequence()", "description" : "the function is sequential and returns the empty sequence" }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">http:invalid-encoding if the given encoding is invalid or not supported.</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 2, "name" : "header", "qname" : "resp:header", "signature" : "($name as xs:string, $value as xs:string) as xs:string?", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Set a HTTP header in the response.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">If a header with the same name was already set, the\n value is overwritten and the function returns the old value. As defined\n in the HTTP specification, multiple headers with the same name\n can be combined into one header whose value is a comma-separated\n list of the values.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">The following headers must not be set using this function.\n Instead, other functions of this module should be used in order\n to implemented the required semantics:\n <ul>\n   <li>Status: use status() instead</li>\n   <li>Content-Type: use content-type() instead</li>\n </ul>\n </p>\n", "summary" : "<p>  Set a HTTP header in the response.</p>", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "name", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the name of the header to set</div>" }, { "name" : "value", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the value of the header to set `</div>" } ], "returns" : { "type" : "xs:string?", "description" : "the value of the header previously set or the empty sequence if no header has been set with the same name." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">resp:invalid-header-name if an invalid name is used for the header.</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 1, "name" : "message-for-status", "qname" : "resp:message-for-status", "signature" : "($status as xs:string) as xs:string", "description" : " Convert between the HTTP status code as string and\n the name/message of that status code.\n", "summary" : "<p> Convert between the HTTP status code as string and\n the name/message of that status code.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "status", "type" : "xs:string", "occurrence" : null, "description" : "" } ], "returns" : { "type" : "xs:string", "description" : "the message for the given name" }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">http:invalid-status if the given name does not represent a valid http status code</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 1, "name" : "redirect", "qname" : "resp:redirect", "signature" : "($url as xs:string) as empty-sequence()", "description" : " This function sets the HTTP 302 redirect status code in the response. As\n a result, a redirect to the URL given as parameter will be made.\n", "summary" : "<p> This function sets the HTTP 302 redirect status code in the response.</p>", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "url", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The URL to which the redirect will be made.</div>" } ], "returns" : { "type" : "empty-sequence()", "description" : "The empty-sequence is returned." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 0, "name" : "serialization-parameters", "qname" : "resp:serialization-parameters", "signature" : "() as object()", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Returns the serialization parameters that are currently\n active, i.e. the ones that will be used to serialize the result\n of this request.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">This will either be the default, the defaults set when\n calling resp:content-type(), or the ones set by\n resp:serialization-parameters().</p>\n", "summary" : "<p>  Returns the serialization parameters that are currently\n active, i.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [  ], "returns" : { "type" : "object()", "description" : "the said serialization parameters" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "serialization-parameters", "qname" : "resp:serialization-parameters", "signature" : "($params as object()) as empty-sequence()", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Set the serialization parameters used for serializing the result\n of the request.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">The following example shows how to set several options\n for the JSON serialization method.\n <pre>\n {\n   \"method\" : \"json\",\n   \"indent\" : false,\n   \"encoding\" : \"UTF-16\"\n }\n </pre>\n </p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Please note that serialization options set by this function can\n be overwritten by a subsequent call to resp:content-type. In this\n case, the options will be reset to the default for the given\n content-type. Also note, that the output encoding can be overwritten\n by subsequently calling the resp:encoding function.</p>\n", "summary" : "<p>  Set the serialization parameters used for serializing the result\n of the request.</p>", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "params", "type" : "object()", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the serialization parameters that will be used to serialize the result of the request</div>" } ], "returns" : { "type" : "empty-sequence()", "description" : "the function is sequential and, on success, returns the empty sequence." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">err:XQDY0027 if the input is not valid</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">http:invalid-encoding if the encoding specified in the serialization options is invalid or not supported.</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 0, "name" : "serializer-defaults-html", "qname" : "resp:serializer-defaults-html", "signature" : "() as object()", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Returns an object that can be used to specify the\n settings of serialization parameters for the HTML serialization\n method.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">The defaults are\n <pre>\n {\n   \"encoding\" : \"UTF-8\",\n   \"indent\" : true,\n   \"method\" : \"html\",\n   \"version\" : \"4.01\",\n   \"doctype-system\" : \"http://www.w3.org/TR/html4/loose.dtd\",\n   \"doctype-public\" : \"-//W3C//DTD HTML 4.01 Transitional//EN\"\n }\n </pre>\n </p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">For example, those defaults are used when setting\n the content-type to some HTML content-type (i.e. text/html).</p>\n", "summary" : "<p>  Returns an object that can be used to specify the\n settings of serialization parameters for the HTML serialization\n method.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [  ], "returns" : { "type" : "object()", "description" : "an object that can be used to specify the above serialization parameters." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 0, "name" : "serializer-defaults-json-xml-hybrid", "qname" : "resp:serializer-defaults-json-xml-hybrid", "signature" : "() as object()", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Returns an object that can be used to specify the\n settings of serialization parameters for the json\n serialization method.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">The defaults are\n <pre>\n {\n   \"encoding\" : \"UTF-8\",\n   \"indent\" : \"yes\",\n   \"method\" : \"json\"\n }\n </pre>\n </p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">For example, those defaults are used when setting\n the content-type to some XML content-type (e.g. application/atom+xml).</p>\n", "summary" : "<p>  Returns an object that can be used to specify the\n settings of serialization parameters for the json\n serialization method.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [  ], "returns" : { "type" : "object()", "description" : "an object that can be used to specify the above serialization parameters." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 0, "name" : "serializer-defaults-json", "qname" : "resp:serializer-defaults-json", "signature" : "() as object()", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Returns an object that can be used to specify the\n settings of serialization parameters for the json-xml hybrid\n serialization method.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">The defaults are\n <pre>\n {\n   \"encoding\" : \"UTF-8\",\n   \"indent\" : \"yes\",\n   \"method\" : \"json\"\n }\n </pre>\n </p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">For example, those defaults are used when setting\n the content-type to application/mixed-json-xml.</p>\n", "summary" : "<p>  Returns an object that can be used to specify the\n settings of serialization parameters for the json-xml hybrid\n serialization method.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [  ], "returns" : { "type" : "object()", "description" : "an object that can be used to specify the above serialization parameters." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 0, "name" : "serializer-defaults-text", "qname" : "resp:serializer-defaults-text", "signature" : "() as object()", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Returns an object that can be used to specify the\n settings of serialization parameters for the text serialization\n method.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">The only default used is\n <pre>\n {\n  \"method\" : \"text\",\n  \"encoding\" : \"UTF-8\",\n }\n </pre>\n </p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">For example, this default is used when setting\n the content-type to some text content-type (e.g. text/plain).</p>\n", "summary" : "<p>  Returns an object that can be used to specify the\n settings of serialization parameters for the text serialization\n method.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [  ], "returns" : { "type" : "object()", "description" : "an object that can be used to specify the above serialization parameters." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 0, "name" : "serializer-defaults-xhtml", "qname" : "resp:serializer-defaults-xhtml", "signature" : "() as object()", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Returns an object that can be used to specify the\n settings of serialization parameters for the XHTML serialization\n method.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">The defaults are\n <pre>\n {\n   \"encoding\" : \"UTF-8\",\n   \"indent\" : true,\n   \"method\" : \"xhtml\",\n   \"omit-xml-declaration\" : true,\n   \"version\" : \"1.0\",\n   \"doctype-system\" : \"http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd\",\n   \"doctype-public\" : \"-//W3C//DTD XHTML 1.0 Transitional//EN\"\n }\n </pre>\n </p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">For example, those defaults are used when setting\n the content-type to some XHTML content-type (i.e. application/xhtml+xml).</p>\n", "summary" : "<p>  Returns an object that can be used to specify the\n settings of serialization parameters for the XHTML serialization\n method.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [  ], "returns" : { "type" : "object()", "description" : "an object that can be used to specify the above serialization parameters." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 0, "name" : "serializer-defaults-xml", "qname" : "resp:serializer-defaults-xml", "signature" : "() as object()", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Returns an object that can be used to specify the\n settings of serialization parameters for the XML serialization\n method.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">The defaults are\n <pre>\n {\n  \"method\" : \"xml\",\n  \"encoding\" : \"UTF-8\",\n  \"indent\" : true,\n  \"omit-xml-declaration\" : true,\n  \"version\" : \"1.0\"\n }\n </pre>\n </p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">For example, those defaults are used when setting\n the content-type to some XML content-type (e.g. application/atom+xml).</p>\n", "summary" : "<p>  Returns an object that can be used to specify the\n settings of serialization parameters for the XML serialization\n method.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [  ], "returns" : { "type" : "object()", "description" : "an object that can be used to specify the above serialization parameters." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "status-code", "qname" : "resp:status-code", "signature" : "($status as xs:integer) as empty-sequence()", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Sets the status code of the HTTP response to the integer given as\n parameter.</p>\n For example,\n <tt xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">resp:status(204)</tt>\n will result in \"HTTP/1.1 204 No Content\".\n", "summary" : "<p>  Sets the status code of the HTTP response to the integer given as\n parameter.</p>", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "status", "type" : "xs:integer", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The status code of the HTTP response as integer.</div>" } ], "returns" : { "type" : "empty-sequence()", "description" : "On success, the empty-sequence is returned" }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">http:invalid-status-code if the given integer does not reflect a valid HTTP status code</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 1, "name" : "status", "qname" : "resp:status", "signature" : "($status as xs:string) as empty-sequence()", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Sets the status code of the HTTP response to code associated\n with the given string.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">For example,\n <tt>resp:status($http:no-content)</tt>\n will result in \"HTTP/1.1 204 No Content\".</p>\n", "summary" : "<p>  Sets the status code of the HTTP response to code associated\n with the given string.</p>", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "status", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The status code of the HTTP response as a string (e.g. $http:no-content)</div>" } ], "returns" : { "type" : "empty-sequence()", "description" : "On success, the empty-sequence is returned" }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">http:invalid-status if the given string does not represent a valid HTTP status code;</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 1, "name" : "valid-status", "qname" : "resp:valid-status", "signature" : "($status as xs:string) as xs:boolean", "description" : " Test whether the given name refers to a valid HTTP status code.\n", "summary" : "<p> Test whether the given name refers to a valid HTTP status code.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "status", "type" : "xs:string", "occurrence" : null, "description" : "" } ], "returns" : { "type" : "xs:boolean", "description" : "true if the given name refers to a valid HTTP status code, false otherwise." }, "errors" : [  ] } ], "variables" : [ { "name" : "http:precondition-required", "type" : "item()*", "description" : " A string representing the HTTP Status Code 428 Precondition Required (RFC 6585)\n" }, { "name" : "http:gone", "type" : "item()*", "description" : " A string representing the HTTP Status Code 410 GONE\n" }, { "name" : "http:length-required", "type" : "item()*", "description" : " A string representing the HTTP Status Code 411 Length Required\n" }, { "name" : "http:precondition-failed", "type" : "item()*", "description" : " A string representing the HTTP Status Code 412 Precondition Failed\n" }, { "name" : "http:request-entity-too-large", "type" : "item()*", "description" : " A string representing the HTTP Status Code 413 Request Entity Too Large\n" }, { "name" : "http:request-entity-too-long", "type" : "item()*", "description" : " A string representing the HTTP Status Code 414 Request Entity Too Long\n" }, { "name" : "http:unsupported-media-type", "type" : "item()*", "description" : " A string representing the HTTP Status Code 415 Unsupported Media Type\n" }, { "name" : "http:request-range-not-satisfiable", "type" : "item()*", "description" : " A string representing the HTTP Status Code 416 Request Range Not Satisfiable\n" }, { "name" : "http:expectation-failed", "type" : "item()*", "description" : " A string representing the HTTP Status Code 417 Expectation Failed\n" }, { "name" : "http:im-a-teapot", "type" : "item()*", "description" : " A string representing the HTTP Status Code 418 I'm a teapot (RFC 2324)\n" }, { "name" : "http:enhance-your-calm", "type" : "item()*", "description" : " A string representing the HTTP Status Code 420 Enhance Your Calm\n" }, { "name" : "http:unprocessable-entity", "type" : "item()*", "description" : " A string representing the HTTP Status Code 422 Unprocessable Entity (WebDAV; RFC 4918)\n" }, { "name" : "http:locked", "type" : "item()*", "description" : " A string representing the HTTP Status Code 423 Locked (WebDAV; RFC 4918)\n" }, { "name" : "http:failed-dependency", "type" : "item()*", "description" : " A string representing the HTTP Status Code 424 Failed Dependency (WebDAV; RFC 4918)\n" }, { "name" : "http:unordered-collection", "type" : "item()*", "description" : " A string representing the HTTP Status Code 425 Unordered Collection\n" }, { "name" : "http:upgrade-required", "type" : "item()*", "description" : " A string representing the HTTP Status Code 426 Upgrade Required\n" }, { "name" : "http:conflict", "type" : "item()*", "description" : " A string representing the HTTP Status Code 409 Conflict\n" }, { "name" : "http:too-many-requests", "type" : "item()*", "description" : " A string representing the HTTP Status Code 429 Too Many Requests (RFC 6585)\n" }, { "name" : "http:request-header-fields-too-large", "type" : "item()*", "description" : " A string representing the HTTP Status Code 431 Request Header Fields Too Large (RFC 6585)\n" }, { "name" : "http:internal-server-error", "type" : "item()*", "description" : " A string representing the HTTP Status Code 500 Internal Server Error\n" }, { "name" : "http:not-implemented", "type" : "item()*", "description" : " A string representing the HTTP Status Code 501 Not Implemented\n" }, { "name" : "http:bad-gateway", "type" : "item()*", "description" : " A string representing the HTTP Status Code 502 Bad Gateway\n" }, { "name" : "http:service-unavailable", "type" : "item()*", "description" : " A string representing the HTTP Status Code 503 Service Unavailable\n" }, { "name" : "http:gateway-timeout", "type" : "item()*", "description" : " A string representing the HTTP Status Code 504 Gateway Timeout\n" }, { "name" : "http:http-version-not-supported", "type" : "item()*", "description" : " A string representing the HTTP Status Code 505 HTTP Version Not Supported\n" }, { "name" : "http:variant-also-negotiates", "type" : "item()*", "description" : " A string representing the HTTP Status Code 506 Variant Also Negotiates (RFC 2295)\n" }, { "name" : "http:insufficient-storage", "type" : "item()*", "description" : " A string representing the HTTP Status Code 507 Insufficient Storage (WebDAV; RFC 4918)\n" }, { "name" : "http:loop-detected", "type" : "item()*", "description" : " A string representing the HTTP status code 508 Loop Detected (webdav; rfc 5842)\n" }, { "name" : "http:not-extended", "type" : "item()*", "description" : " A string representing the HTTP status code 510 Not Extended (RFC 2774)\n" }, { "name" : "http:network-authentication-required", "type" : "item()*", "description" : " A string representing the HTTP status code 511 Network Authentication Required (RFC 6585)\n" }, { "name" : "resp:status-info", "type" : "object()", "description" : " Variable containing a description for all the HTTP Status Codes\n" }, { "name" : "http:see-other", "type" : "item()*", "description" : " A string representing the HTTP Status Code 303 See Other\n" }, { "name" : "http:switching-protocols", "type" : "item()*", "description" : " A string representing the HTTP Status Code 101 Switching Protocols\n" }, { "name" : "http:processing", "type" : "item()*", "description" : " A string representing the HTTP Status Code 102 Processing (Web; RFC 2518)\n" }, { "name" : "http:ok", "type" : "item()*", "description" : " A string representing the HTTP Status Code 200 OK\n" }, { "name" : "http:created", "type" : "item()*", "description" : " A string representing the HTTP Status Code 201 Created\n" }, { "name" : "http:accepted", "type" : "item()*", "description" : " A string representing the HTTP Status Code 202 Accepted\n" }, { "name" : "http:non-authoritative", "type" : "item()*", "description" : " A string representing the HTTP Status Code 203 Non-Authoritative\n" }, { "name" : "http:no-content", "type" : "item()*", "description" : " A string representing the HTTP Status Code 204 No Content\n" }, { "name" : "http:reset-content", "type" : "item()*", "description" : " A string representing the HTTP Status Code 205 Reset Content\n" }, { "name" : "http:partial-content", "type" : "item()*", "description" : " A string representing the HTTP Status Code 206 Partial Content\n" }, { "name" : "http:multi-status", "type" : "item()*", "description" : " A string representing the HTTP Status Code 207 Multi-Status (WebDAV; RFC 5842)\n" }, { "name" : "http:already-reported", "type" : "item()*", "description" : " A string representing the HTTP Status Code 208 Already Reported (WebDAV; RFC 5842)\n" }, { "name" : "http:im-used", "type" : "item()*", "description" : " A string representing the HTTP Status Code 226 IM Used (RFC 3229)\n" }, { "name" : "http:multiple-choices", "type" : "item()*", "description" : " A string representing the HTTP Status Code 300 Multiple Choices\n" }, { "name" : "http:moved-permanently", "type" : "item()*", "description" : " A string representing the HTTP Status Code 301 Moved Permantently\n" }, { "name" : "http:found", "type" : "item()*", "description" : " A string representing the HTTP Status Code 302 Found\n" }, { "name" : "http:continue", "type" : "item()*", "description" : " A string representing the HTTP Status Code 100 Continue\n" }, { "name" : "http:not-modified", "type" : "item()*", "description" : " A string representing the HTTP Status Code 304 Not Modified\n" }, { "name" : "http:use-proxy", "type" : "item()*", "description" : " A string representing the HTTP Status Code 305 Use Proxy\n" }, { "name" : "http:switch-proxy", "type" : "item()*", "description" : " A string representing the HTTP Status Code 306 Switch Proxy\n" }, { "name" : "http:temporary-redirect", "type" : "item()*", "description" : " A string representing the HTTP Status Code 307 Temporary Redirect\n" }, { "name" : "http:permanent-redirect", "type" : "item()*", "description" : " A string representing the HTTP Status Code 308 Permanent Redirect (http://tools.ietf.org/html/draft-reschke-http-status-308-07)\n" }, { "name" : "http:bad-request", "type" : "item()*", "description" : " A string representing the HTTP Status Code 400 Bad Request\n" }, { "name" : "http:unauthorized", "type" : "item()*", "description" : " A string representing the HTTP Status Code 401 Unauthorized\n" }, { "name" : "http:payment-required", "type" : "item()*", "description" : " A string representing the HTTP Status Code 402 Payment Required\n" }, { "name" : "http:forbidden", "type" : "item()*", "description" : " A string representing the HTTP Status Code 403 Forbidden\n" }, { "name" : "http:not-found", "type" : "item()*", "description" : " A string representing the HTTP Status Code 404 Not Found\n" }, { "name" : "http:not-allowed", "type" : "item()*", "description" : " A string representing the HTTP Status Code 405 Not Allowed\n" }, { "name" : "http:not-acceptable", "type" : "item()*", "description" : " A string representing the HTTP Status Code 406 Not Acceptable\n" }, { "name" : "http:proxy-authentication-required", "type" : "item()*", "description" : " A string representing the HTTP Status Code 407 Proxy Authentication Required\n" }, { "name" : "http:request-timeout", "type" : "item()*", "description" : " A string representing the HTTP Status Code 408 Request Timeout\n" } ] }, "http://www.zorba-xquery.com/modules/email/smtp" : { "ns" : "http://www.zorba-xquery.com/modules/email/smtp", "description" : " This module can be used for sending emails.\n The SMTP module contains only one public function that receives two parameters.\n The SMTP server access information passed as an <code xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">hostInfo</code> element\n and the email message representation as a <code xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">message</code> element.\n For a quick start see the examples associates with the <code xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">send(...)</code>\n function. For a complete specification read, the description and the\n documentation associated with this function.\n", "sees" : [ "<xqdoc:see xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"><a href=\"http://www.washington.edu/imap/\">c-client Library part of UW IMAP toolkit</a></xqdoc:see>" ], "authors" : [ "<xqdoc:author xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Sorin Nasoi, Daniel Thomas</xqdoc:author>" ], "version" : null, "encoding" : "utf-8", "namespaces" : [ { "uri" : "http://zorba.io/annotations", "prefix" : "an" }, { "uri" : "http://www.zorba-xquery.com/modules/email", "prefix" : "email" }, { "uri" : "http://www.zorba-xquery.com/modules/email/smtp", "prefix" : "smtp" }, { "uri" : "http://zorba.io/options/versioning", "prefix" : "ver" } ], "functions" : [ { "isDocumented" : true, "arity" : 2, "name" : "send", "qname" : "smtp:send", "signature" : "($host-info as element(email:hostInfo), $message as element(email:message)) as empty-sequence()", "description" : " This function sends an email message from the specified account.\n The <code xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">hostName</code> child element of <code xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">$host-info</code> must have the form:\n <code xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"><b>remote_system_name</b> [\":\" <b>port</b>] [<b>flags</b>]</code>. This syntax is part of the\n <a xmlns:xqdoc=\"http://www.xqdoc.org/1.0\" href=\"http://www.washington.edu/imap/documentation/naming.txt.html\" target=\"_blank\">Remote names</a>\n syntax defined in the UW IMAP toolkit. The <code xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"><b>remote_system_name</b></code> and\n <code xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"><b>flags</b></code> fragments are explained in the section <code xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">III</code> of this document.\n For example the hostName could look like:\n <ul xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">\n   <li><code>&lt;hostName&gt;smtp.gmail.com:587/tls/novalidate-cert&lt;hostName&gt;</code></li>\n   <li><code>&lt;hostName&gt;[209.85.129.111]:587/tls/novalidate-cert&lt;hostName&gt;</code></li>\n </ul>\n The <code xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">$host-info</code> parameter could then look like this:\n <pre xmlns:xqdoc=\"http://www.xqdoc.org/1.0\" class=\"ace-static\" ace-mode=\"xquery\">\n &lt;hostInfo&gt;\n   &lt;hostName&gt;smtp.gmail.com:587/tls/novalidate-cert&lt;/hostName&gt;\n   &lt;userName&gt;username&lt;/userName&gt;\n   &lt;password&gt;password&lt;/password&gt;\n &lt;/hostInfo&gt;\n </pre>\n For a complete of the structure of an email message, see the imported email\n schema: <code xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">http://www.zorba-xquery.com/modules/email</code>\n All the data passed to this function does not need to be validated.\n The only requirement is that they have a valid format and are in the\n correct namespace according to the schema:\n <code xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">http://www.zorba-xquery.com/modules/email</code>.\n", "summary" : "<p> This function sends an email message from the specified account.</p>", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "host-info", "type" : "element(email:hostInfo)", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The SMTP host, user name, and password.</div>" }, { "name" : "message", "type" : "element(email:message)", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The message to send as defined in the email XML schema.</div>" } ], "returns" : { "type" : "empty-sequence()", "description" : "The function is declared as sequential and has side-effects. It returns the empty sequence." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">smtp:SMTP0001 The message format is invalid.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">smtp:SMTP0002 The message has no recipient.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">smtp:SMTP0003 The message could not be sent.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">smtp:SMTP9999 If any other error occurs.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">err:XQDY0027 If the values of the arguments are not not valid according to the email schema: <code>http://www.zorba-xquery.com/modules/email</code></xqdoc:error>" ] } ], "variables" : [  ] }, "http://www.28msec.com/modules/project" : { "ns" : "http://www.28msec.com/modules/project", "description" : " This module contains functions to introspect a project.\n", "sees" : [  ], "authors" : [ "<xqdoc:author xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">28msec</xqdoc:author>" ], "version" : null, "encoding" : "utf-8", "namespaces" : [ { "uri" : "http://zorba.io/annotations", "prefix" : "an" }, { "uri" : "http://zorba.io/modules/base64", "prefix" : "base64" }, { "uri" : "http://zorba.io/modules/hmac", "prefix" : "hmac" }, { "uri" : "http://www.28msec.com/modules/project", "prefix" : "project" }, { "uri" : "http://zorba.io/modules/random", "prefix" : "random" }, { "uri" : "http://zorba.io/options/versioning", "prefix" : "ver" } ], "functions" : [ { "isDocumented" : true, "arity" : 0, "name" : "create-seed", "qname" : "project:create-seed", "signature" : "() as string", "description" : " Creates and return new random project seed.\n", "summary" : "<p> Creates and return new random project seed.</p>", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [  ], "returns" : { "type" : "string", "description" : "the project seed" }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">project:SEED-EXISTS the project already has a seed file</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">project:SEED-FILE error writing the seed file</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">project:SEED the specified seed is not at least 8 characters long</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 1, "name" : "create-seed", "qname" : "project:create-seed", "signature" : "($seed as string) as empty-sequence()", "description" : " Creates a new project seed.\n The specified seed must be at least 8 characters long.\n", "summary" : "<p> Creates a new project seed.</p>", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "seed", "type" : "string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The new project seed.</div>" } ], "returns" : { "type" : "empty-sequence()", "description" : "the empty sequence" }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">project:SEED-EXISTS the project already has a seed file</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">project:SEED-FILE error writing the seed file</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 0, "name" : "has-seed", "qname" : "project:has-seed", "signature" : "() as boolean external", "description" : " Checks if the project has a seed file.\n", "summary" : "<p> Checks if the project has a seed file.</p>", "annotation_str" : " %an:nondeterministic", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "nondeterministic", "value" : "" } ], "updating" : false, "parameters" : [  ], "returns" : { "type" : "boolean", "description" : "whether the project has a seed file or not" }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">project:SEED-FILE error accessing the seed file</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 0, "name" : "name", "qname" : "project:name", "signature" : "() as string external", "description" : " Returns the name of the project.\n", "summary" : "<p> Returns the name of the project.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [  ], "returns" : { "type" : "string", "description" : "the said name" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 0, "name" : "read-or-create-seed", "qname" : "project:read-or-create-seed", "signature" : "() as string", "description" : " Returns the project seed. If the project has no seed a new random one is created.\n", "summary" : "<p> Returns the project seed.</p>", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [  ], "returns" : { "type" : "string", "description" : "the project seed" }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">project:SEED-FILE error reading or writing the seed file</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 0, "name" : "scheduler-host", "qname" : "project:scheduler-host", "signature" : "() as string external", "description" : " Returns the host of the scheduler.\n", "summary" : "<p> Returns the host of the scheduler.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [  ], "returns" : { "type" : "string", "description" : "the scheduler host." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 0, "name" : "seed", "qname" : "project:seed", "signature" : "() as string external", "description" : " Returns the project seed.\n", "summary" : "<p> Returns the project seed.</p>", "annotation_str" : " %an:nondeterministic", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "nondeterministic", "value" : "" } ], "updating" : false, "parameters" : [  ], "returns" : { "type" : "string", "description" : "the project seed" }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">project:NO-SEED the project seed file cannot be found</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">project:SEED-FILE error reading the seed file</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 0, "name" : "set-seed", "qname" : "project:set-seed", "signature" : "() as string", "description" : " Sets a new random project seed.\n", "summary" : "<p> Sets a new random project seed.</p>", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [  ], "returns" : { "type" : "string", "description" : "the project seed" }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">project:SEED-FILE error reading or writing the seed file</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 1, "name" : "set-seed", "qname" : "project:set-seed", "signature" : "($seed as string) as empty-sequence()", "description" : " Sets the project seed.\n The specified seed must be at least 8 characters long.\n", "summary" : "<p> Sets the project seed.</p>", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "seed", "type" : "string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The new project seed.</div>" } ], "returns" : { "type" : "empty-sequence()", "description" : "the project seed" }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">project:SEED-FILE error reading or writing the seed file</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">project:SEED the specified seed is not at least 8 characters long</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 0, "name" : "token", "qname" : "project:token", "signature" : "() as xs:base64Binary", "description" : " Creates a new project token, that expires after 12 hours.\n", "summary" : "<p> Creates a new project token, that expires after 12 hours.</p>", "annotation_str" : " %an:nondeterministic", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "nondeterministic", "value" : "" } ], "updating" : false, "parameters" : [  ], "returns" : { "type" : "xs:base64Binary", "description" : "the project token" }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "token", "qname" : "project:token", "signature" : "($expiration as xs:anyAtomicType) as xs:base64Binary", "description" : " Creates a new project token, that expires at the desired date.\n The input to the function contains the expiration information.\n If a duration is used, then the function will create a token expiring after the specified duration.\n If a dateTime is used, then the function will create a token expiring that date.\n If an integer is used, then the function will create a token expiring after the specified number of seconds.\n", "summary" : "<p> Creates a new project token, that expires at the desired date.</p>", "annotation_str" : " %an:nondeterministic", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "nondeterministic", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "expiration", "type" : "xs:anyAtomicType", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> describes the expiration date of the token.</div>" } ], "returns" : { "type" : "xs:base64Binary", "description" : "the new project token" }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">project:INVALID-PARAMETER expiration information is invalid (not a duration, date or number of seconds)</xqdoc:error>" ] } ], "variables" : [  ] }, "http://zorba.io/modules/http-client" : { "ns" : "http://zorba.io/modules/http-client", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">\n This module provides functions for performing HTTP requests.\n </p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">A simple GET request using the get#1 convenience function</p>\n <pre xmlns:xqdoc=\"http://www.xqdoc.org/1.0\" class=\"ace-static\" ace-mode=\"xquery\">\n import module namespace http=\"http://zorba.io/modules/http-client\";\n http:get(\"http://www.example.com\")\n </pre>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">\n This example makes a GET request to example.com and returns the server's response\n as a JSON object.\n </p>\n <pre xmlns:xqdoc=\"http://www.xqdoc.org/1.0\" class=\"ace-static\" ace-mode=\"java\">\n {\n   \"status\" : 200,\n   \"message\" : \"OK\",\n   \"headers\" : {\n     \"Content-Length\" : \"1270\",\n     \"Date\" : \"Tue, 11 Jun 2013 22:27:10 GMT\",\n     ...\n   },\n   \"body\" : {\n     \"media-type\" : \"text/html\",\n     \"content\" : \"...\"\n   }\n }\n </pre>\n <h2 xmlns:xqdoc=\"http://www.xqdoc.org/1.0\" id=\"standard_return\">Response format</h2>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Most functions in this module (all except <a href=\"?anchor=options-1\">options#1</a>)\n return a single JSON item, describing the server's response, as in the previous\n example.\n The server status (integer) and message (string) fields are always present.\n If the server replied sending one or more headers, they are reported\n in an optional headers object. Each header is represented as a single (string)\n field.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">For non-multipart responses, as in the previous example, the response body,\n if any, is reported as a body object. This object contains both the (string)\n media-type returned by the server and its content.\n The type of the content field is determined by the media-type returned by the\n server. If the media-type indicates that the body content is textual,\n then the content has type string, base64Binary otherwise.\n Specifically, the body content is considered textual only if the MIME-type specified in\n the media-type is one of:\n <ul>\n   <li>\"application/json\"</li>\n   <li>\"application/x-javascript\"</li>\n   <li>\"application/xml\"</li>\n   <li>\"application/xml-external-parsed-entity\"</li>\n </ul>\n or if the MIME-type starts with \"text/\" or ends with \"+xml\".</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">For multipart responses, multiple bodies are returned, as in the following example: </p>\n <pre xmlns:xqdoc=\"http://www.xqdoc.org/1.0\" class=\"ace-static\" ace-mode=\"java\">\n {\n   \"status\" : 200,\n   \"message\" : \"OK\",\n   \"headers\" : {\n     \"Date\" : \"Tue, 11 Jun 2013 22:34:13 GMT\",\n     ...\n   },\n   \"multipart\" : {\n     \"boundary\": \"--AaB03x\",\n     \"parts\": [\n       {\n         \"headers\" : {\n            \"Content Disposition: file\",\n            ...\n         },\n         \"body\": {\n           \"media-type\" : \"image/gif\",\n           \"content\" : \"...\"\n         }\n       },\n       {\n         \"body\" : {\n           \"media-type\" : \"text/html\",\n           \"content\" : \"...\"\n         }\n       }\n    ]\n }\n </pre>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">The multipart field contains both the boundary used to separate parts\n and an array containing all parts. Each part contains its specific headers,\n if any, and the corresponding body.</p>\n <h2 xmlns:xqdoc=\"http://www.xqdoc.org/1.0\" id=\"nondeterministic_warning\">Important Notice Regarding Nondeterministic Functions</h2>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">\n The following functions in this module -\n <a href=\"?anchor=get-1\">get#1</a>,\n <a href=\"?anchor=get-text-1\">get-text#1</a>,\n <a href=\"?anchor=get-binary-1\">get-binary#1</a>,\n <a href=\"?anchor=send-nondeterministic-request-1\">send-nondeterministic-request-1</a>,\n <a href=\"?anchor=head-1\">head#1</a>, and\n <a href=\"?anchor=options-1\">options#1</a>\n are declared to be <i>nondeterministic</i>, which means that their results\n will not be cached.\n However, they are <b>not</b> declared to be\n <i>sequential</i>, which means that they may be re-ordered during query optimization.\n According to the HTTP RFC, GET, HEAD an OPTIONS requests should not have any side-effects.\n However, in practice it is not uncommon, especially for GET requests, to have side-effects.\n If your application depends on the ordering of side-effects from requests issued through\n these functions, you should either use the <a href=\"?anchor=send-request-1\">send-request()</a>\n function (which is declared <i>sequential</i>), or alternatively\n wrap each call to get() in your own sequential function, to ensure\n that the requests are not reordered.\n Conversely, if you want their results to be cached you can use the\n <a href=\"?anchor=send-deterministic-request-3\">send-deterministic-request()</a>\n function (which is declared <i>deterministic</i>).\n </p>\n <h2 xmlns:xqdoc=\"http://www.xqdoc.org/1.0\" id=\"url_string\">$href Arguments to Functions</h2>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Several functions in this module accept a URL argument named $href. In\n all cases, the value passed to $href must be a valid anyURI.\n However, all functions declare $href to be of type string. This\n is for convenience, since you can pass a string literal value (that\n is, a URL in double-quotes spelled out explicitly in your query)\n to an string parameter.</p>\n <h2 xmlns:xqdoc=\"http://www.xqdoc.org/1.0\" id=\"expath_relation\">Relation to the EXPath http-client module</h2>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"><a href=\"http://expath.org/\">EXPath</a> defines its own http-client\n module, which is available separately.\n There are two primary differences between EXPath's http-client and\n this module:\n <ol>\n   <li>EXPath does not include the simpler get(), post(), put(), delete(),\n head(), options() and patch() functions defined by this module.</li>\n <li>EXPath uses XML to represent request for its send-request() function,\n whereas this module uses JSON.</li>\n <li>EXPath specifies that all XML content returned by an HTTP server\n will be parsed and returned as an XML document, whereas all HTML content\n will be <i>tidied up</i> into valid XML, and then parsed into an element.\n This module returns any textual content as string and any binary content\n as base6Binary.</li>\n <li>EXPath accepts XML nodes as body in the send-request() function and\n automatically serializes them into a string. The send-request() function\n defined in this module only allows string, base64Binary, and hexBinary\n as body types.\n </li>\n </ol>\n </p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">\n See <a href=\"http://www.expath.org/spec/http-client\">the full spec\n of the EXPath http-client module</a> for more information.\n </p>\n", "sees" : [ "<xqdoc:see xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"><a href=\"http://www.w3.org/TR/xquery-11/#FunctionDeclns\">XQuery 1.1: Function Declaration</a></xqdoc:see>" ], "authors" : [ "<xqdoc:author xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Federico Cavalieri</xqdoc:author>" ], "version" : null, "encoding" : "utf-8", "namespaces" : [ { "uri" : "http://zorba.io/annotations", "prefix" : "an" }, { "uri" : "http://www.w3.org/2005/xqt-errors", "prefix" : "err" }, { "uri" : "http://zorba.io/modules/http-client", "prefix" : "http" }, { "uri" : "http://jsoniq.org/function-library", "prefix" : "libjn" }, { "uri" : "http://zorba.io/options/versioning", "prefix" : "ver" } ], "functions" : [ { "isDocumented" : true, "arity" : 1, "name" : "delete", "qname" : "http:delete", "signature" : "($href as string) as object()", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">\n This function makes an HTTP DELETE request to a given URL.\n </p>\n", "summary" : "<p>  \n This function makes an HTTP DELETE request to a given URL.</p>", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "href", "type" : "string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The URL to which the request will be made (see <a href=\"#url_string\">note</a> above).</div>" } ], "returns" : { "type" : "object()", "description" : "<a xmlns:xqdoc=\"http://www.xqdoc.org/1.0\" href=\"#standard_return\">standard http-client return type</a>." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">http:HTTP An HTTP error occurred.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">http:REQUEST The specified request is not valid.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">http:TIMEOUT A timeout occurred waiting for the response.</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 1, "name" : "get-binary", "qname" : "http:get-binary", "signature" : "($href as string) as object()", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">\n This function makes a GET request on a given URL. All returned bodies\n are forced to be interpreted as binary data, and will be returned\n as base64Binary items.\n </p>\n", "summary" : "<p>  \n This function makes a GET request on a given URL.</p>", "annotation_str" : " %an:nondeterministic", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "nondeterministic", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "href", "type" : "string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The URL to which the request will be made (see <a href=\"#url_string\">note</a> above).</div>" } ], "returns" : { "type" : "object()", "description" : "<a xmlns:xqdoc=\"http://www.xqdoc.org/1.0\" href=\"#standard_return\">standard http-client return type</a>." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">http:HTTP An HTTP error occurred.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">http:REQUEST The specified href is not valid.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">http:TIMEOUT A timeout occurred waiting for the response.</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 1, "name" : "get-text", "qname" : "http:get-text", "signature" : "($href as string) as object()", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">\n This function makes a GET request to a given URL. All returned bodies\n are forced to be interpreted as textual, with a UTF-8 charset and will\n be returned as string items.\n </p>\n", "summary" : "<p>  \n This function makes a GET request to a given URL.</p>", "annotation_str" : " %an:nondeterministic", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "nondeterministic", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "href", "type" : "string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The URL to which the request will be made (see <a href=\"#url_string\">note</a> above).</div>" } ], "returns" : { "type" : "object()", "description" : "<a xmlns:xqdoc=\"http://www.xqdoc.org/1.0\" href=\"#standard_return\">standard http-client return type</a>." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">http:HTTP An HTTP error occurred.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">http:REQUEST The specified href is not valid.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">http:TIMEOUT A timeout occurred waiting for the response.</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 1, "name" : "get", "qname" : "http:get", "signature" : "($href as string) as object()", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">\n This function makes a GET request to a given URL.\n </p>\n", "summary" : "<p>  \n This function makes a GET request to a given URL.</p>", "annotation_str" : " %an:nondeterministic", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "nondeterministic", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "href", "type" : "string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The URL to which the request will be made (see <a href=\"#url_string\">note</a> above).</div>" } ], "returns" : { "type" : "object()", "description" : "<a xmlns:xqdoc=\"http://www.xqdoc.org/1.0\" href=\"#standard_return\">standard http-client return type</a>." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">http:HTTP An HTTP error occurred.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">http:REQUEST The specified href is not valid.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">http:TIMEOUT A timeout occurred waiting for the response.</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 1, "name" : "head", "qname" : "http:head", "signature" : "($href as string) as object()", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">\n This function makes an HTTP HEAD request on a given URL.\n </p>\n", "summary" : "<p>  \n This function makes an HTTP HEAD request on a given URL.</p>", "annotation_str" : " %an:nondeterministic", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "nondeterministic", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "href", "type" : "string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The URL to which the request will be made (see <a href=\"#url_string\">note</a> above).</div>" } ], "returns" : { "type" : "object()", "description" : "<a xmlns:xqdoc=\"http://www.xqdoc.org/1.0\" href=\"#standard_return\">standard http-client return type</a>." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">http:HTTP An HTTP error occurred.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">http:REQUEST The specified href is not valid.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">http:TIMEOUT A timeout occurred waiting for the response.</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 1, "name" : "options", "qname" : "http:options", "signature" : "($href as string) as string*", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">\n This function makes an HTTP OPTIONS request, which asks the server\n which operations it supports.\n </p>\n", "summary" : "<p>  \n This function makes an HTTP OPTIONS request, which asks the server\n which operations it supports.</p>", "annotation_str" : " %an:nondeterministic", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "nondeterministic", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "href", "type" : "string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The URL to which the request will be made (see <a href=\"#url_string\">note</a> above).</div>" } ], "returns" : { "type" : "string*", "description" : "A sequence of string values of the allowed operations." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">http:HTTP An HTTP error occurred.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">http:REQUEST The specified href is not valid.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">http:TIMEOUT A timeout occurred waiting for the response.</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 2, "name" : "patch", "qname" : "http:patch", "signature" : "($href as string, $body as atomic) as object()", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">\n This function makes an HTTP PATCH request to a given URL.\n </p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">\n The body passed to this function must be either a string, a base64Binary or\n an hexBinary.\n If it is a string, the Content-Type sent to the server will be \"text/plain\",\n \"application/octet-stream\" otherwise.\n </p>\n", "summary" : "<p>  \n This function makes an HTTP PATCH request to a given URL.</p>", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "href", "type" : "string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The URL to which the request will be made (see <a href=\"#url_string\">note</a> above).</div>" }, { "name" : "body", "type" : "atomic", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The body which will be sent to the server.</div>" } ], "returns" : { "type" : "object()", "description" : "<a xmlns:xqdoc=\"http://www.xqdoc.org/1.0\" href=\"#standard_return\">standard http-client return type</a>." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">http:HTTP An HTTP error occurred.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">http:REQUEST The specified request is not valid.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">http:TIMEOUT A timeout occurred waiting for the response.</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 3, "name" : "patch", "qname" : "http:patch", "signature" : "($href as string, $body as atomic, $content-type as string) as object()", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">\n This function makes an HTTP PATCH request to a given URL.\n </p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">\n The body passed to this function must be either a string, a base64Binary, or\n an hexBinary.\n In any case, Content-Type of the request sent to the server will\n be $content-type.\n </p>\n", "summary" : "<p>  \n This function makes an HTTP PATCH request to a given URL.</p>", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "href", "type" : "string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The URL to which the request will be made (see <a href=\"#url_string\">note</a> above).</div>" }, { "name" : "body", "type" : "atomic", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The body which will be sent to the server.</div>" }, { "name" : "content-type", "type" : "string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The content type of $body to send to the server.</div>" } ], "returns" : { "type" : "object()", "description" : "<a xmlns:xqdoc=\"http://www.xqdoc.org/1.0\" href=\"#standard_return\">standard http-client return type</a>." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">http:HTTP An HTTP error occurred.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">http:REQUEST The specified request is not valid.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">http:TIMEOUT A timeout occurred waiting for the response.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">http:CHARSET The specified charset is unsupported.</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 2, "name" : "post", "qname" : "http:post", "signature" : "($href as string, $body as atomic) as object()", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">\n This function makes an HTTP POST request to a given URL.\n </p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">\n The body passed to this function must be either a string, a base64Binary, or an\n hexBinary.\n If it is a string, the Content-Type sent to the server will be \"text/plain\",\n \"application/octet-stream\" otherwise.\n </p>\n", "summary" : "<p>  \n This function makes an HTTP POST request to a given URL.</p>", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "href", "type" : "string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The URL to which the request will be made (see <a href=\"#url_string\">note</a> above).</div>" }, { "name" : "body", "type" : "atomic", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The body which will be sent to the server.</div>" } ], "returns" : { "type" : "object()", "description" : "<a xmlns:xqdoc=\"http://www.xqdoc.org/1.0\" href=\"#standard_return\">standard http-client return type</a>." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">http:HTTP An HTTP error occurred.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">http:REQUEST The specified request is not valid.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">http:TIMEOUT A timeout occurred waiting for the response.</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 3, "name" : "post", "qname" : "http:post", "signature" : "($href as string, $body as atomic, $content-type as string) as object()", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">\n This function makes an HTTP POST request to a given URL.\n </p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">\n The body passed to this function must be either a string, a base64Binary,\n or an hexBinary.\n In any case, Content-Type of the request sent to the server will\n be $content-type.\n </p>\n", "summary" : "<p>  \n This function makes an HTTP POST request to a given URL.</p>", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "href", "type" : "string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The URL to which the request will be made (see <a href=\"#url_string\">note</a> above).</div>" }, { "name" : "body", "type" : "atomic", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The body which will be sent to the server</div>" }, { "name" : "content-type", "type" : "string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The content type of the body as described above.</div>" } ], "returns" : { "type" : "object()", "description" : "<a xmlns:xqdoc=\"http://www.xqdoc.org/1.0\" href=\"#standard_return\">standard http-client return type</a>." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">http:HTTP An HTTP error occurred.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">http:REQUEST The specified request is not valid.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">http:TIMEOUT A timeout occurred waiting for the response.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">http:CHARSET The specified charset is unsupported.</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 2, "name" : "put", "qname" : "http:put", "signature" : "($href as string, $body as atomic) as object()", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">\n This function makes an HTTP PUT request to a given URL.\n </p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">\n The body passed to this function must be either a string, a base64Binary or\n an hexBinary.\n If it is a string, the Content-Type sent to the server will be \"text/plain\",\n \"application/octet-stream\" otherwise.\n </p>\n", "summary" : "<p>  \n This function makes an HTTP PUT request to a given URL.</p>", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "href", "type" : "string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The URL to which the request will be made (see <a href=\"#url_string\">note</a> above).</div>" }, { "name" : "body", "type" : "atomic", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The body which will be sent to the server.</div>" } ], "returns" : { "type" : "object()", "description" : "<a xmlns:xqdoc=\"http://www.xqdoc.org/1.0\" href=\"#standard_return\">standard http-client return type</a>." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">http:HTTP An HTTP error occurred.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">http:REQUEST The specified request is not valid.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">http:TIMEOUT A timeout occurred waiting for the response.</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 3, "name" : "put", "qname" : "http:put", "signature" : "($href as string, $body as atomic, $content-type as string) as object()", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">\n This function makes an HTTP PUT request to a given URL.\n </p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">\n The body passed to this function must be either a string, a base64Binary, or\n an hexBinary.\n In any case, Content-Type of the request sent to the server will\n be $content-type.\n </p>\n", "summary" : "<p>  \n This function makes an HTTP PUT request to a given URL.</p>", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "href", "type" : "string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The URL to which the request will be made (see <a href=\"#url_string\">note</a> above).</div>" }, { "name" : "body", "type" : "atomic", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The body which will be sent to the server.</div>" }, { "name" : "content-type", "type" : "string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The content type of $body to send to the server.</div>" } ], "returns" : { "type" : "object()", "description" : "<a xmlns:xqdoc=\"http://www.xqdoc.org/1.0\" href=\"#standard_return\">standard http-client return type</a>." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">http:HTTP An HTTP error occurred.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">http:REQUEST The specified request is not valid.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">http:TIMEOUT A timeout occurred waiting for the response.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">http:CHARSET The specified charset is unsupported.</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 1, "name" : "send-deterministic-request", "qname" : "http:send-deterministic-request", "signature" : "($request as object()) as object()", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">\n This function sends an HTTP request and returns the corresponding response.\n </p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">\n This function has the same semantics as <a href=\"#send-request-1\">send-request-1</a>,\n but is declared as deterministic and thus should only be used when\n the request has no side-effects and behaves deterministic as required by the\n application.\n </p>\n", "summary" : "<p>  \n This function sends an HTTP request and returns the corresponding response.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "request", "type" : "object()", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> see request parameter of <a href=\"#send-request-1\">send-request#1</a></div>" } ], "returns" : { "type" : "object()", "description" : "<a xmlns:xqdoc=\"http://www.xqdoc.org/1.0\" href=\"#standard_return\">standard http-client return type</a>." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">http:HTTP An HTTP error occurred.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">http:REQUEST The specified request is not valid.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">http:TIMEOUT A timeout occurred waiting for the response.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">http:FOLLOW Cannot follow a redirect of a POST, PUT, or DELETE request.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">http:CHARSET The specified charset is unsupported.</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 1, "name" : "send-nondeterministic-request", "qname" : "http:send-nondeterministic-request", "signature" : "($request as object()) as object()", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">\n This function sends an HTTP request and returns the corresponding response.\n </p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">\n This function has the same semantics as <a href=\"#send-request-1\">send-request-1</a>,\n but is declared as being non deterministic and thus should only be used when\n the request has no side-effects.\n </p>\n", "summary" : "<p>  \n This function sends an HTTP request and returns the corresponding response.</p>", "annotation_str" : " %an:nondeterministic", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "nondeterministic", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "request", "type" : "object()", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> see request parameter of <a href=\"#send-request-1\">send-request#1</a></div>" } ], "returns" : { "type" : "object()", "description" : "<a xmlns:xqdoc=\"http://www.xqdoc.org/1.0\" href=\"#standard_return\">standard http-client return type</a>." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">http:HTTP An HTTP error occurred.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">http:REQUEST The specified request is not valid.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">http:TIMEOUT A timeout occurred waiting for the response.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">http:FOLLOW Cannot follow a redirect of a POST, PUT, or DELETE request.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">http:CHARSET The specified charset is unsupported.</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 1, "name" : "send-request", "qname" : "http:send-request", "signature" : "($request as object()) as object()", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">\n This function sends an HTTP request and returns the corresponding response.\n </p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">\n This function is declared as sequential and should be used whenever the\n request may have side-effects.\n </p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">\n The request parameters are specified in the $request JSON object, which\n has the following minimal structure:\n <pre class=\"ace-static\" ace-mode=\"java\">\n   {\n     \"href\": \"http://www.example.com\"\n   }\n </pre>\n </p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">This object specifies a GET request of the URI \"http://www.example.com\"</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Additional optional parameters can be specified when issuing a request,\n using the following structure:</p>\n <pre xmlns:xqdoc=\"http://www.xqdoc.org/1.0\" class=\"ace-static\" ace-mode=\"java\">\n  {\n    \"method\": \"POST\",\n    \"href\": \"http://www.example.com\",\n    \"authentication\":\n    {\n      \"username\" : \"user\",\n      \"password\" : \"pass\",\n      \"auth-method\" : \"Basic\"\n    },\n    \"options\":\n    {\n      \"status-only\": true,\n      \"override-media-type\": \"text/plain\",\n      \"follow-redirect\": false,\n      \"timeout\": 30,\n      \"user-agent\": \"Mozilla/5.0\",\n      \"retry\":\n      {\n        \"delay\": [1000, 2000, 5000],\n        \"on-connection-error\": false,\n        \"on-statuses\": [500, 501]\n      }\n    },\n    \"headers\":\n    {\n      \"name\": \"value\",\n      ...\n    },\n    \"body\":\n    {\n      \"media-type\": \"text/plain\",\n      \"content\": \"...\"\n    }\n  }\n</pre>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">\n The method field (string) defines the HTTP verb to use in the HTTP request (i.e., GET, HEAD, OPTIONS,\n POST, PUT, DELETE). If not specified GET will be used.\n The authentication field can be used to specify the credentials and authentication method\n used when issuing a request (e.g, Basic). If the authentication field is specified, all its (string)\n subfields must be specified. If an authentication object is provided, it overrides any\n Authorization header specified in the request.\n Additionally, the following options can be specified:\n <ul>\n    <li>status-only. If true, the response body contents are omitted from the response object.</li>\n    <li>override-media-type. Is a MIME type that will override the Content-Type header returned\n        by the server. It affects the type of the result body content.</li>\n    <li>follow-redirect. Control whether an http redirect is automatically followed or not. If\n        it is false, the http redirect is returned as the response. If it is\n        true (the default) the function tries to follow the redirect, by\n        sending the same request to the new address (including body, headers,\n         and authentication credentials.) Maximum one redirect is followed\n        (there is no attempt to follow a redirect in response to following a\n          first redirect).</li>\n    <li>timeout. Is the maximum number of seconds to wait for the server to respond.\n         If no response is received withing this time duration, an error is thrown.</li>\n    <li>user-agent. The user agent sent to the server when issuing the request.\n        If not specified libcurl-agent/1.0 is used.</li>\n    <li>retry. The request will be automatically retried in case of connection error\n        following mandatory suboptions:\n        <ul>\n            <li>delay: an array specifying the milliseconds of wait before each retry.</li>\n            <li>on-connection-error: whether to retry the request if a connection to the\n                server cannot be estabilished</li>\n            <li>on-statuses: an array containing the statuses which will trigger a retry</li>\n        </ul></li>\n </ul>\n </p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">One or more headers can be sent to the server, specifying them in an optional headers object.\n Each header is represented as a single (string) field. These headers are overridden if the corresponding\n option/authentication has been specified in the request.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">For non-multipart request a body object can be specified.\n This object must contain both the desired (string) media-type and its content.\n The type of the content field must be either string, base64Binary, or hexBinary. </p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">For multipart requests, multipart object can be specified in place of the body object.\n The multipart object has the following structure: </p>\n <pre xmlns:xqdoc=\"http://www.xqdoc.org/1.0\" class=\"ace-static\" ace-mode=\"java\">\n  \"multipart\" : {\n    \"boundary\": \"--AaB03x\",\n    \"parts\": [\n      {\n        \"headers\" : {\n           \"Content Disposition: file\",\n           ...\n        },\n        \"body\": {\n          \"media-type\" : \"image/gif\",\n          \"content\" : \"...\"\n        }\n      },\n      {\n        \"body\" : {\n          \"media-type\" : \"text/html\",\n          \"content\" : \"...\"\n        }\n      }\n   ]\n }\n </pre>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">The multipart field contains an optional (string) field which specifies\n the boundary used to separate each part and an array containing all parts.\n Each part contains its specific headers, if any, and the corresponding body.\n </p>\n", "summary" : "<p>  \n This function sends an HTTP request and returns the corresponding response.</p>", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "request", "type" : "object()", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> a JSON http-client request object</div>" } ], "returns" : { "type" : "object()", "description" : "<a xmlns:xqdoc=\"http://www.xqdoc.org/1.0\" href=\"#standard_return\">standard http-client return type</a>." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">http:HTTP An HTTP error occurred.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">http:REQUEST The specified request is not valid.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">http:TIMEOUT A timeout occurred waiting for the response.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">http:FOLLOW Cannot follow a redirect of a POST, PUT, or DELETE request.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">http:CHARSET The specified charset is unsupported.</xqdoc:error>" ] } ], "variables" : [  ] }, "http://zorba.io/modules/data-cleaning/consolidation" : { "ns" : "http://zorba.io/modules/data-cleaning/consolidation", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">This library module provides data consolidation functions that generally take as input a sequence of XML nodes\n and apply some rule in order do decide which node is better suited to represent the entire sequence.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">The logic contained in this module is not specific to any particular XQuery implementation,\n although the consolidation functions based on matching sequences against XPath expressions require\n some form of dynamic evaluation for XPath expressions.</p>\n", "sees" : [  ], "authors" : [ "<xqdoc:author xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Bruno Martins</xqdoc:author>" ], "version" : null, "encoding" : "utf-8", "namespaces" : [ { "uri" : "http://zorba.io/modules/data-cleaning/consolidation", "prefix" : "con" }, { "uri" : "http://zorba.io/modules/data-cleaning/set-similarity", "prefix" : "set" }, { "uri" : "http://zorba.io/modules/data-cleaning/character-based-string-similarity", "prefix" : "simc" }, { "uri" : "http://zorba.io/options/versioning", "prefix" : "ver" } ], "functions" : [ { "isDocumented" : true, "arity" : 2, "name" : "all-xpaths", "qname" : "con:all-xpaths", "signature" : "($s as element(*)*, $paths as xs:string*) as element(*)*", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Returns the elements from an input sequence of elements that, when matched to a given set of XPath expressions,\n produce a non-empty set of nodes in all the cases.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Example usage : <pre class=\"ace-static\" ace-mode=\"xquery\"> all-xpaths( ( &lt;a&gt;&lt;b/&gt;&lt;/a&gt;, &lt;c&gt;&lt;d/&gt;&lt;/c&gt;, &lt;d/&gt;), (\".//b\") ) </pre></p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">The function invocation in the example above returns : <pre class=\"ace-static\" ace-mode=\"xquery\"> (&lt;a&gt;&lt;b/&gt;&lt;/a&gt;) </pre></p>\n", "summary" : "<p>  Returns the elements from an input sequence of elements that, when matched to a given set of XPath expressions,\n produce a non-empty set of nodes in all the cases.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "s", "type" : "element(*)", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> A sequence of elements.</div>" }, { "name" : "paths", "type" : "xs:string", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> A sequence of strings denoting XPath expressions.</div>" } ], "returns" : { "type" : "element(*)*", "description" : "The elements that, when matched to the given set of XPath expressions, always return a non-empty set of nodes." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "least-attributes", "qname" : "con:least-attributes", "signature" : "($s) as element(*)", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Returns the single node having the smallest number of descending attributes (attributes at any given depth)\n in a sequence of nodes provided as input.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">If more then one answer is possible, return the first node according to the order of the input sequence.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Example usage : <pre class=\"ace-static\" ace-mode=\"xquery\">least-attributes( ( &lt;a att1=\"a1\" att2=\"a2\"/&gt;, &lt;b att1=\"a1\" /&gt;, &lt;c/&gt; ) )</pre></p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">The function invocation in the example above returns : <pre class=\"ace-static\" ace-mode=\"xquery\">(&lt;c/&gt;)</pre></p>\n", "summary" : "<p>  Returns the single node having the smallest number of descending attributes (attributes at any given depth)\n in a sequence of nodes provided as input.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "s", "type" : null, "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> A sequence of nodes.</div>" } ], "returns" : { "type" : "element(*)", "description" : "The node having the smallest number of descending attributes in the input sequence." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "least-distinct-attributes", "qname" : "con:least-distinct-attributes", "signature" : "($s) as element(*)", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Returns the single node having the smallest number of distinct descending attributes (attributes at any\n given depth) in a sequence of nodes provided as input.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">If more then one answer is possible, return the first node according to the order of the input sequence.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Example usage : <pre class=\"ace-static\" ace-mode=\"xquery\"> least-distinct-attributes( ( &lt;a att1=\"a1\" att2=\"a2\"/&gt;, &lt;b att1=\"a1\" /&gt;, &lt;c/&gt; ) ) </pre></p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">The function invocation in the example above returns : <pre class=\"ace-static\" ace-mode=\"xquery\"> (&lt;c/&gt;) </pre></p>\n", "summary" : "<p>  Returns the single node having the smallest number of distinct descending attributes (attributes at any\n given depth) in a sequence of nodes provided as input.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "s", "type" : null, "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> A sequence of nodes.</div>" } ], "returns" : { "type" : "element(*)", "description" : "The node having the smallest number of distinct descending attributes in the input sequence." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "least-distinct-elements", "qname" : "con:least-distinct-elements", "signature" : "($s) as element(*)", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Returns the single node having the smallest number of distinct descending elements (sub-elements at any\n given depth) in a sequence of nodes provided as input.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">If more then one answer is possible, return the first node according to the order of the input sequence.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Example usage : <pre class=\"ace-static\" ace-mode=\"xquery\"> least-distinct-elements( ( &lt;a&gt;&lt;b/&gt;&lt;/a&gt;, &lt;b&gt;&lt;c/&gt;&lt;/b&gt;, &lt;d/&gt;) ) </pre></p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">The function invocation in the example above returns : <pre class=\"ace-static\" ace-mode=\"xquery\"> (&lt;d/&gt;) </pre></p>\n", "summary" : "<p>  Returns the single node having the smallest number of distinct descending elements (sub-elements at any\n given depth) in a sequence of nodes provided as input.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "s", "type" : null, "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> A sequence of nodes.</div>" } ], "returns" : { "type" : "element(*)", "description" : "The node having the smallest number of distinct descending elements in the input sequence." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "least-distinct-nodes", "qname" : "con:least-distinct-nodes", "signature" : "($s) as element(*)", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Returns the single node having the smallest number of distinct descending nodes (sub-nodes at any given depth)\n in a sequence of nodes provided as input.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">If more then one answer is possible, return the first node according to the order of the input sequence.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Example usage : <pre class=\"ace-static\" ace-mode=\"xquery\"> least-distinct-nodes( ( &lt;a&gt;&lt;b/&gt;&lt;/a&gt;, &lt;b&gt;&lt;c/&gt;&lt;/b&gt;, &lt;d/&gt;) ) </pre></p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">The function invocation in the example above returns : <pre class=\"ace-static\" ace-mode=\"xquery\"> (&lt;d/&gt;) </pre></p>\n", "summary" : "<p>  Returns the single node having the smallest number of distinct descending nodes (sub-nodes at any given depth)\n in a sequence of nodes provided as input.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "s", "type" : null, "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> A sequence of nodes.</div>" } ], "returns" : { "type" : "element(*)", "description" : "The node having the smallest number of distinct descending nodes in the input sequence." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "least-elements", "qname" : "con:least-elements", "signature" : "($s) as element(*)", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Returns the single node having the smallest number of descending elements (sub-elements at any given depth)\n in a sequence of nodes provided as input.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">If more then one answer is possible, return the first node according to the order of the input sequence.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Example usage : <pre class=\"ace-static\" ace-mode=\"xquery\">least-elements( ( &lt;a&gt;&lt;b/&gt;&lt;/a&gt;, &lt;b&gt;&lt;c/&gt;&lt;/b&gt;, &lt;d/&gt;) )</pre></p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">The function invocation in the example above returns : <pre class=\"ace-static\" ace-mode=\"xquery\">(&lt;d/&gt;)</pre></p>\n", "summary" : "<p>  Returns the single node having the smallest number of descending elements (sub-elements at any given depth)\n in a sequence of nodes provided as input.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "s", "type" : null, "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> A sequence of nodes.</div>" } ], "returns" : { "type" : "element(*)", "description" : "The node having the smallest number of descending elements in the input sequence." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "least-frequent", "qname" : "con:least-frequent", "signature" : "($s) as item()", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Returns the single less frequent node in a sequence of nodes provided as input.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">If more then one answer is possible, return the first node according to the order of the input sequence.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Example usage : <pre class=\"ace-static\" ace-mode=\"xquery\">least-frequent( ( \"a\", \"a\", \"b\") )</pre></p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">The function invocation in the example above returns : <pre class=\"ace-static\" ace-mode=\"xquery\">(\"b\")</pre></p>\n", "summary" : "<p>  Returns the single less frequent node in a sequence of nodes provided as input.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "s", "type" : null, "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> A sequence of nodes.</div>" } ], "returns" : { "type" : "item()", "description" : "The least frequent node in the input sequence." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "least-nodes", "qname" : "con:least-nodes", "signature" : "($s) as element(*)", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Returns the single node having the smallest number of descending nodes (sub-nodes at any given depth)\n in a sequence of nodes provided as input.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">If more then one answer is possible, return the first node according to the order of the input sequence.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Example usage : <pre class=\"ace-static\" ace-mode=\"xquery\">least-nodes( ( &lt;a&gt;&lt;b/&gt;&lt;/a&gt;, &lt;b&gt;&lt;c/&gt;&lt;/b&gt;, &lt;d/&gt;) )</pre></p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">The function invocation in the example above returns : <pre class=\"ace-static\" ace-mode=\"xquery\">(&lt;d/&gt;)</pre></p>\n", "summary" : "<p>  Returns the single node having the smallest number of descending nodes (sub-nodes at any given depth)\n in a sequence of nodes provided as input.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "s", "type" : null, "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> A sequence of nodes.</div>" } ], "returns" : { "type" : "element(*)", "description" : "The node having the smallest number of descending nodes in the input sequence." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 2, "name" : "least-similar-edit-distance", "qname" : "con:least-similar-edit-distance", "signature" : "($s as xs:string*, $m as xs:string) as xs:string?", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Returns the single least similar string, in terms of the edit distance metric towards an input string,\n in a sequence of strings provided as input. If more than one string has a minimum similarity (a maximum\n value for the edit distance metric), return the first string according to the order of the input sequence.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Example usage : <pre class=\"ace-static\" ace-mode=\"xquery\">least-similar-edit-distance( ( \"aaabbbccc\", \"aaabbb\", \"eeefff\" ), \"aaab\" )</pre></p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">The function invocation in the example above returns : <pre class=\"ace-static\" ace-mode=\"xquery\">( \"eeefff\" )</pre></p>\n", "summary" : "<p>  Returns the single least similar string, in terms of the edit distance metric towards an input string,\n in a sequence of strings provided as input.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "s", "type" : "xs:string", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> A sequence of strings.</div>" }, { "name" : "m", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The string towards which we want to measure the edit distance.</div>" } ], "returns" : { "type" : "xs:string?", "description" : "The least similar string in the input sequence." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 2, "name" : "least-tokens", "qname" : "con:least-tokens", "signature" : "($s as xs:string*, $r as xs:string) as xs:string?", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Returns the single shortest string, in terms of the number of tokens, in a sequence of strings provided as input.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">If more then one answer is possible, return the first string according to the order of the input sequence.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Example usage : <pre class=\"ace-static\" ace-mode=\"xquery\">least-tokens( ( \"a b c\", \"a b\", \"a\"), \" +\" )</pre></p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">The function invocation in the example above returns : <pre class=\"ace-static\" ace-mode=\"xquery\">(\"a\")</pre></p>\n", "summary" : "<p>  Returns the single shortest string, in terms of the number of tokens, in a sequence of strings provided as input.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "s", "type" : "xs:string", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> A sequence of strings.</div>" }, { "name" : "r", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> A regular expression forming the delimiter character(s) which mark the boundaries between adjacent tokens.</div>" } ], "returns" : { "type" : "xs:string?", "description" : "The shortest string in the input sequence, in terms of the number of tokens." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 2, "name" : "least-xpaths", "qname" : "con:least-xpaths", "signature" : "($s as element(*)*, $paths as xs:string*) as element(*)*", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Returns the single element from an input sequence of elements that matches the smallest number of\n XPath expressions from a given set, producing a non-empty set of nodes.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">If more then one answer is possible, return the first element according to the order of the input sequence.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Example usage : <pre class=\"ace-static\" ace-mode=\"xquery\"> least-xpaths( ( &lt;a&gt;&lt;b/&gt;&lt;/a&gt;, &lt;d&gt;&lt;c/&gt;&lt;b/&gt;&lt;/d&gt;, &lt;d/&gt;) , (\".//b\", \".//c\") ) </pre></p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">The function invocation in the example above returns : <pre class=\"ace-static\" ace-mode=\"xquery\"> ( $lt;d/&gt; ) </pre></p>\n", "summary" : "<p>  Returns the single element from an input sequence of elements that matches the smallest number of\n XPath expressions from a given set, producing a non-empty set of nodes.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "s", "type" : "element(*)", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> A sequence of elements.</div>" }, { "name" : "paths", "type" : "xs:string", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> A sequence of strings denoting XPath expressions.</div>" } ], "returns" : { "type" : "element(*)*", "description" : "The element that matches the smallest number of XPath expressions producing a non-empty set of nodes." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "longest", "qname" : "con:longest", "signature" : "($s as xs:string*) as xs:string?", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Returns the single longest string, in terms of the number of characters, in a sequence of strings provided as input.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">If more then one answer is possible, return the first string according to the order of the input sequence.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Example usage : <pre class=\"ace-static\" ace-mode=\"xquery\">con:longest( ( \"a\", \"aa\", \"aaa\") )</pre></p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">The function invocation in the example above returns : <pre class=\"ace-static\" ace-mode=\"xquery\">(\"aaa\")</pre></p>\n", "summary" : "<p>  Returns the single longest string, in terms of the number of characters, in a sequence of strings provided as input.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "s", "type" : "xs:string", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> A sequence of strings.</div>" } ], "returns" : { "type" : "xs:string?", "description" : "The longest string in the input sequence." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 2, "name" : "matching", "qname" : "con:matching", "signature" : "($s as xs:string*, $r as xs:string) as xs:string*", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Returns the strings from an input sequence of strings that match a particular regular expression.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Example usage : <pre class=\"ace-static\" ace-mode=\"xquery\">matching( ( \"a A b\", \"c AAA d\", \"e BB f\"), \"A+\" )</pre></p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">The function invocation in the example above returns : <pre class=\"ace-static\" ace-mode=\"xquery\">( \"a A b\", \"c AAA d\")</pre></p>\n", "summary" : "<p>  Returns the strings from an input sequence of strings that match a particular regular expression.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "s", "type" : "xs:string", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> A sequence of strings.</div>" }, { "name" : "r", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The regular expression to be used in the matching.</div>" } ], "returns" : { "type" : "xs:string*", "description" : "The strings in the input sequence that match the input regular expression." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "most-attributes", "qname" : "con:most-attributes", "signature" : "($s) as element(*)", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Returns the single node having the largest number of descending attributes (attributes at any given depth)\n in a sequence of nodes provided as input.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">If more then one answer is possible, return the first node according to the order of the input sequence.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Example usage : <pre class=\"ace-static\" ace-mode=\"xquery\">most-attributes( ( &lt;a att1=\"a1\" att2=\"a2\"/&gt;, &lt;b att1=\"a1\" /&gt;, &lt;c/&gt; ) )</pre></p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">The function invocation in the example above returns : <pre class=\"ace-static\" ace-mode=\"xquery\">(&lt;a att1=\"a1\" att2=\"a2\"/&gt;)</pre></p>\n", "summary" : "<p>  Returns the single node having the largest number of descending attributes (attributes at any given depth)\n in a sequence of nodes provided as input.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "s", "type" : null, "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> A sequence of nodes.</div>" } ], "returns" : { "type" : "element(*)", "description" : "The node having the largest number of descending attributes in the input sequence." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "most-distinct-attributes", "qname" : "con:most-distinct-attributes", "signature" : "($s) as element(*)", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Returns the single node having the largest number of distinct descending attributes (attributes at any\n given depth) in a sequence of nodes provided as input.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">If more then one answer is possible, return the first node according to the order of the input sequence.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Example usage : <pre class=\"ace-static\" ace-mode=\"xquery\">most-distinct-attributes( ( &lt;a att1=\"a1\" att2=\"a2\" att3=\"a3\"/&gt;, &lt;a att1=\"a1\" att2=\"a2\"&gt;&lt;b att2=\"a2\" /&gt;&lt;/a&gt;, &lt;c/&gt; ) )</pre></p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">The function invocation in the example above returns : <pre class=\"ace-static\" ace-mode=\"xquery\">(&lt;a att1=\"a1\" att2=\"a2\" att3=\"a3\"/&gt;)</pre></p>\n", "summary" : "<p>  Returns the single node having the largest number of distinct descending attributes (attributes at any\n given depth) in a sequence of nodes provided as input.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "s", "type" : null, "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> A sequence of nodes.</div>" } ], "returns" : { "type" : "element(*)", "description" : "The node having the largest number of distinct descending attributes in the input sequence." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "most-distinct-elements", "qname" : "con:most-distinct-elements", "signature" : "($s) as element(*)", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Returns the single node having the largest number of distinct descending elements (sub-elements at any\n given depth) in a sequence of nodes provided as input.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">If more then one answer is possible, return the first node according to the order of the input sequence.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Example usage : <pre class=\"ace-static\" ace-mode=\"xquery\">most-distinct-elements( ( &lt;a&gt;&lt;b/&gt;&lt;c/&gt;&lt;d/&gt;&lt;/a&gt;, &lt;a&gt;&lt;b/&gt;&lt;b/&gt;&lt;c/&gt;&lt;/a&gt;, &lt;a/&gt; ) )</pre></p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">The function invocation in the example above returns : <pre class=\"ace-static\" ace-mode=\"xquery\">(&lt;a&gt;&lt;b/&gt;&lt;c/&gt;&lt;d/&gt;&lt;/a&gt;)</pre></p>\n", "summary" : "<p>  Returns the single node having the largest number of distinct descending elements (sub-elements at any\n given depth) in a sequence of nodes provided as input.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "s", "type" : null, "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> A sequence of nodes.</div>" } ], "returns" : { "type" : "element(*)", "description" : "The node having the largest number of distinct descending elements in the input sequence." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "most-distinct-nodes", "qname" : "con:most-distinct-nodes", "signature" : "($s) as element(*)", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Returns the single node having the largest number of distinct descending nodes (sub-nodes at any given depth) in\n a sequence of nodes provided as input.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">If more then one answer is possible, return the first node according to the order of the input sequence.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Example usage : <pre class=\"ace-static\" ace-mode=\"xquery\">most-distinct-nodes( ( &lt;a&gt;&lt;b/&gt;&lt;/a&gt;, &lt;a&gt;&lt;a/&gt;&lt;/a&gt;, &lt;b/&gt;) )</pre></p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">The function invocation in the example above returns : <pre class=\"ace-static\" ace-mode=\"xquery\">(&lt;a&gt;&lt;b/&gt;&lt;/a&gt;)</pre></p>\n", "summary" : "<p>  Returns the single node having the largest number of distinct descending nodes (sub-nodes at any given depth) in\n a sequence of nodes provided as input.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "s", "type" : null, "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> A sequence of nodes.</div>" } ], "returns" : { "type" : "element(*)", "description" : "The node having the largest number of distinct descending nodes in the input sequence." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "most-elements", "qname" : "con:most-elements", "signature" : "($s) as element(*)", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Returns the single node having the largest number of descending elements (sub-elements at any given depth)\n in a sequence of nodes provided as input.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">If more then one answer is possible, return the first node according to the order of the input sequence.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Example usage : <pre class=\"ace-static\" ace-mode=\"xquery\">most-elements( ( &lt;a&gt;&lt;b/&gt;&lt;/a&gt;, &lt;a/&gt;, &lt;b/&gt;) )</pre></p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">The function invocation in the example above returns : <pre class=\"ace-static\" ace-mode=\"xquery\">(&lt;a&gt;&lt;b/&gt;&lt;/a&gt;)</pre></p>\n", "summary" : "<p>  Returns the single node having the largest number of descending elements (sub-elements at any given depth)\n in a sequence of nodes provided as input.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "s", "type" : null, "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> A sequence of nodes.</div>" } ], "returns" : { "type" : "element(*)", "description" : "The node having the largest number of descending elements in the input sequence." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "most-frequent", "qname" : "con:most-frequent", "signature" : "($s) as item()", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Returns the single most frequent node in a sequence of nodes provided as input.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">If more then one answer is possible, returns the first node according to the order of the input sequence.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Example usage : <pre class=\"ace-static\" ace-mode=\"xquery\">most-frequent( ( \"a\", \"a\", \"b\") )</pre></p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">The function invocation in the example above returns : <pre class=\"ace-static\" ace-mode=\"xquery\">(\"a\")</pre></p>\n", "summary" : "<p>  Returns the single most frequent node in a sequence of nodes provided as input.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "s", "type" : null, "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> A sequence of nodes.</div>" } ], "returns" : { "type" : "item()", "description" : "The most frequent node in the input sequence." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "most-nodes", "qname" : "con:most-nodes", "signature" : "($s) as element(*)", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Returns the single node having the largest number of descending nodes (sub-nodes at any given depth) in a\n sequence of nodes provided as input.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">If more then one answer is possible, return the first node according to the order of the input sequence.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Example usage : <pre class=\"ace-static\" ace-mode=\"xquery\">most-nodes( ( &lt;a&gt;&lt;b/&gt;&lt;/a&gt;, &lt;a/&gt;, &lt;b/&gt;) )</pre></p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">The function invocation in the example above returns : <pre class=\"ace-static\" ace-mode=\"xquery\">(&lt;a&gt;&lt;b/&gt;&lt;/a&gt;)</pre></p>\n", "summary" : "<p>  Returns the single node having the largest number of descending nodes (sub-nodes at any given depth) in a\n sequence of nodes provided as input.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "s", "type" : null, "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> A sequence of nodes.</div>" } ], "returns" : { "type" : "element(*)", "description" : "The node having the largest number of descending nodes in the input sequence." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 2, "name" : "most-similar-edit-distance", "qname" : "con:most-similar-edit-distance", "signature" : "($s as xs:string*, $m as xs:string) as xs:string?", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Returns the single most similar string, in terms of the edit distance metric towards an input string,\n in a sequence of strings provided as input. If more than one string has a maximum similarity (a minimum\n value for the edit distance metric), the function return the first string according to the order of the\n input sequence.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Example usage : <pre class=\"ace-static\" ace-mode=\"xquery\">most-similar-edit-distance( ( \"aaabbbccc\", \"aaabbb\", \"eeefff\" ), \"aaab\" )</pre></p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">The function invocation in the example above returns : <pre class=\"ace-static\" ace-mode=\"xquery\">( \"aaabbb\" )</pre></p>\n", "summary" : "<p>  Returns the single most similar string, in terms of the edit distance metric towards an input string,\n in a sequence of strings provided as input.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "s", "type" : "xs:string", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> A sequence of strings.</div>" }, { "name" : "m", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The string towards which we want to measure the edit distance.</div>" } ], "returns" : { "type" : "xs:string?", "description" : "The most similar string in the input sequence." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 2, "name" : "most-tokens", "qname" : "con:most-tokens", "signature" : "($s as xs:string*, $r as xs:string) as xs:string?", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Returns the single longest string, in terms of the number of tokens, in a sequence of strings provided as input.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">If more then one answer is possible, return the first string according to the order of the input sequence.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Example usage : <pre class=\"ace-static\" ace-mode=\"xquery\">most-tokens( ( \"a b c\", \"a b\", \"a\"), \" +\" )</pre></p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">The function invocation in the example above returns : <pre class=\"ace-static\" ace-mode=\"xquery\">(\"a b c\")</pre></p>\n", "summary" : "<p>  Returns the single longest string, in terms of the number of tokens, in a sequence of strings provided as input.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "s", "type" : "xs:string", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> A sequence of strings.</div>" }, { "name" : "r", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> A regular expression forming the delimiter character(s) which mark the boundaries between adjacent tokens.</div>" } ], "returns" : { "type" : "xs:string?", "description" : "The longest string in the input sequence, in terms of the number of tokens." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 2, "name" : "most-xpaths", "qname" : "con:most-xpaths", "signature" : "($s as element(*)*, $paths as xs:string*) as element(*)*", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Returns the single element from an input sequence of elements that matches the largest number of\n XPath expressions from a given set, producing a non-empty set of nodes.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">If more then one answer is possible, return the first element according to the order of the input sequence.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Example usage : <pre class=\"ace-static\" ace-mode=\"xquery\"> most-xpaths( ( &lt;a&gt;&lt;b/&gt;&lt;/a&gt;, &lt;d&gt;&lt;c/&gt;&lt;b/&gt;&lt;/d&gt;, &lt;d/&gt;) , (\".//b\", \".//c\") ) </pre></p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">The function invocation in the example above returns : <pre class=\"ace-static\" ace-mode=\"xquery\"> ( &lt;d&gt;&lt;c/&gt;&lt;b/&gt;&lt;/d&gt; ) </pre></p>\n", "summary" : "<p>  Returns the single element from an input sequence of elements that matches the largest number of\n XPath expressions from a given set, producing a non-empty set of nodes.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "s", "type" : "element(*)", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> A sequence of elements.</div>" }, { "name" : "paths", "type" : "xs:string", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> A sequence of strings denoting XPath expressions.</div>" } ], "returns" : { "type" : "element(*)*", "description" : "The element that matches the largest number of XPath expressions producing a non-empty set of nodes." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "shortest", "qname" : "con:shortest", "signature" : "($s as xs:string*) as xs:string?", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Returns the single shortest string, in terms of the number of characters, in a sequence of strings provided as input.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">If more then one answer is possible, return the first string according to the order of the input sequence.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Example usage : <pre class=\"ace-static\" ace-mode=\"xquery\">shortest( ( \"a\", \"aa\", \"aaa\") )</pre></p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">The function invocation in the example above returns : <pre class=\"ace-static\" ace-mode=\"xquery\">(\"a\")</pre></p>\n", "summary" : "<p>  Returns the single shortest string, in terms of the number of characters, in a sequence of strings provided as input.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "s", "type" : "xs:string", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> A sequence of strings.</div>" } ], "returns" : { "type" : "xs:string?", "description" : "The shortest string in the input sequence." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 2, "name" : "some-xpaths", "qname" : "con:some-xpaths", "signature" : "($s as element(*)*, $paths as xs:string*) as element(*)*", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Returns the elements from a sequence of elements that, when matched to a given set of XPath expressions,\n produce a non-empty set of nodes for some of the cases.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Example usage : <pre class=\"ace-static\" ace-mode=\"xquery\"> some-xpaths( ( &lt;a&gt;&lt;b/&gt;&lt;/a&gt;, &lt;d&gt;&lt;c/&gt;&lt;/d&gt;, &lt;d/&gt;), (\".//b\", \".//c\") ) </pre></p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">The function invocation in the example above returns : <pre class=\"ace-static\" ace-mode=\"xquery\"> ( &lt;a&gt;&lt;b/&gt;&lt;/a&gt; , &lt;d&gt;&lt;c/&gt;&lt;/d&gt; ) </pre></p>\n", "summary" : "<p>  Returns the elements from a sequence of elements that, when matched to a given set of XPath expressions,\n produce a non-empty set of nodes for some of the cases.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "s", "type" : "element(*)", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> A sequence of elements.</div>" }, { "name" : "paths", "type" : "xs:string", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> A sequence of strings denoting XPath expressions.</div>" } ], "returns" : { "type" : "element(*)*", "description" : "The elements that, when matched to the given set of XPath expressions, return a non-empty set of nodes for at least one of the cases." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "superstring", "qname" : "con:superstring", "signature" : "($s as xs:string*) as xs:string?", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Returns the single string, from an input sequence of strings, that appears more frequently as part\n of the other strings in the sequence. If no such string exists, the function returns an empty sequence.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">If more then one answer is possible, the function returns the first string according to the order of the input sequence.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Example usage : <pre class=\"ace-static\" ace-mode=\"xquery\">super-string( ( \"aaa bbb ccc\", \"aaa bbb\", \"aaa ddd\", \"eee fff\" ) )</pre></p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">The function invocation in the example above returns : <pre class=\"ace-static\" ace-mode=\"xquery\">( \"aaa bbb\" )</pre></p>\n", "summary" : "<p>  Returns the single string, from an input sequence of strings, that appears more frequently as part\n of the other strings in the sequence.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "s", "type" : "xs:string", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> A sequence of strings.</div>" } ], "returns" : { "type" : "xs:string?", "description" : "The string that appears more frequently as part of the other strings in the sequence." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 2, "name" : "validating-schema", "qname" : "con:validating-schema", "signature" : "($s as element(*)*, $schema as element(*)) as element(*)*", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Returns the nodes from an input sequence of nodes that validate against a given XML Schema.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Example usage : <pre class=\"ace-static\" ace-mode=\"xquery\"> validating-schema ( ( &lt;a/&gt; , &lt;b/&gt; ), &lt;xs:schema xmlns:xs=\"http://www.w3.org/2001/XMLSchema\"&gt;&lt;xs:element name=\"a\" /&gt;&lt;/xs:schema&gt; ) </pre></p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">The function invocation in the example above returns : <pre class=\"ace-static\" ace-mode=\"xquery\"> ( &lt;a/&gt; ) </pre></p>\n", "summary" : "<p>  Returns the nodes from an input sequence of nodes that validate against a given XML Schema.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "s", "type" : "element(*)", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> A sequence of elements.</div>" }, { "name" : "schema", "type" : "element(*)", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> An element encoding an XML Schema.</div>" } ], "returns" : { "type" : "element(*)*", "description" : "The nodes that validate against the XML Schema. <b xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> Attention : This function is still not implemented. </b>" }, "errors" : [  ] } ], "variables" : [  ] }, "http://zorba.io/modules/node" : { "ns" : "http://zorba.io/modules/node", "description" : " This module defines a set of function which can be used\n to determine (1) the relationship between two nodes (e.g. if one\n is the ancestor if another) and (2) properties of given\n nodes (e.g. their level in the tree).\n The same functionality can be achieved with simple XPath expressions.\n However, please note that using the functions in this modules instead\n of path expressions guarantees better performance.\n", "sees" : [  ], "authors" : [ "<xqdoc:author xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Matthias Brantner</xqdoc:author>" ], "version" : null, "encoding" : "utf-8", "namespaces" : [ { "uri" : "http://zorba.io/modules/node", "prefix" : "node" }, { "uri" : "http://zorba.io/options/versioning", "prefix" : "ver" } ], "functions" : [ { "isDocumented" : true, "arity" : 2, "name" : "ancestor-of", "qname" : "node:ancestor-of", "signature" : "($node1 as node(), $node2 as node()) as xs:boolean external", "description" : " Determines whether the node given as second argument is an\n ancestor of the node given as first argument.\n", "summary" : "<p> Determines whether the node given as second argument is an\n ancestor of the node given as first argument.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "node1", "type" : "node()", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the potential descendant</div>" }, { "name" : "node2", "type" : "node()", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the potential ancestor</div>" } ], "returns" : { "type" : "xs:boolean", "description" : "true if $node2 is an ancestor of $node1; false otherwise." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 2, "name" : "child-of", "qname" : "node:child-of", "signature" : "($node1 as node(), $node2 as node()) as xs:boolean external", "description" : " Determines whether the node given as second argument is a\n child of the node given as first argument.\n", "summary" : "<p> Determines whether the node given as second argument is a\n child of the node given as first argument.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "node1", "type" : "node()", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the potential parent</div>" }, { "name" : "node2", "type" : "node()", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the potential child</div>" } ], "returns" : { "type" : "xs:boolean", "description" : "true if $node2 is a child of $node1; false otherwise." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "copy", "qname" : "node:copy", "signature" : "($input as node()*) as node()* external", "description" : " Return a deep copy of every given node according to the properties\n specified in the static context of the invoking module.<p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n Please note that a copy of a node is parentless.\n", "summary" : "<p> Return a deep copy of every given node according to the properties\n specified in the static context of the invoking module.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "input", "type" : "node()", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the node to copy</div>" } ], "returns" : { "type" : "node()*", "description" : "a deep copy of every node in the input sequence or the empty sequence if $input is the empty sequence." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 2, "name" : "descendant-of", "qname" : "node:descendant-of", "signature" : "($node1 as node(), $node2 as node()) as xs:boolean external", "description" : " Determines whether the node given as second argument is a\n descendant of the node given as first argument.\n", "summary" : "<p> Determines whether the node given as second argument is a\n descendant of the node given as first argument.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "node1", "type" : "node()", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the potential ancestor</div>" }, { "name" : "node2", "type" : "node()", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the potential descendant</div>" } ], "returns" : { "type" : "xs:boolean", "description" : "true if $node2 is a descendant of $node1; false otherwise." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 2, "name" : "following-of", "qname" : "node:following-of", "signature" : "($node1 as node(), $node2 as node()) as xs:boolean external", "description" : " Determines whether the node given as second argument is\n following the node given as first argument.\n", "summary" : "<p> Determines whether the node given as second argument is\n following the node given as first argument.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "node1", "type" : "node()", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the potential preceding node</div>" }, { "name" : "node2", "type" : "node()", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the potential following node</div>" } ], "returns" : { "type" : "xs:boolean", "description" : "true if $node2 is following the node $node1; false otherwise." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 2, "name" : "following-sibling-of", "qname" : "node:following-sibling-of", "signature" : "($node1 as node(), $node2 as node()) as xs:boolean external", "description" : " Determines whether the node given as second argument is a\n following-sibling of the node given as first argument.\n", "summary" : "<p> Determines whether the node given as second argument is a\n following-sibling of the node given as first argument.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "node1", "type" : "node()", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the potential preceding-sibling</div>" }, { "name" : "node2", "type" : "node()", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the potential following-sibling</div>" } ], "returns" : { "type" : "xs:boolean", "description" : "true if $node2 is a following-sibling of $node1; false otherwise." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 2, "name" : "least-common-ancestor", "qname" : "node:least-common-ancestor", "signature" : "($node1 as node(), $node2 as node()) as node()? external", "description" : " Computes the least common ancestor of two given nodes in\n the tree.\n", "summary" : "<p> Computes the least common ancestor of two given nodes in\n the tree.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "node1", "type" : "node()", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the first node</div>" }, { "name" : "node2", "type" : "node()", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the second node</div>" } ], "returns" : { "type" : "node()?", "description" : "the least common ancestor of the two given nodes or the empty sequence if the two nodes are not part of the same tree." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "level", "qname" : "node:level", "signature" : "($node as node()) as xs:integer external", "description" : " Computes the level of a given node in the tree.\n Note: The first level has the number one.\n", "summary" : "<p> Computes the level of a given node in the tree.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "node", "type" : "node()", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the node for which the level should be computed</div>" } ], "returns" : { "type" : "xs:integer", "description" : "The level as xs:integer of the given node in the tree." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 2, "name" : "parent-of", "qname" : "node:parent-of", "signature" : "($node1 as node(), $node2 as node()) as xs:boolean external", "description" : " Determines whether the node given as second argument is a\n parent of the node given as first argument.\n", "summary" : "<p> Determines whether the node given as second argument is a\n parent of the node given as first argument.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "node1", "type" : "node()", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the potential child</div>" }, { "name" : "node2", "type" : "node()", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the potential parent</div>" } ], "returns" : { "type" : "xs:boolean", "description" : "true if $node2 is a parent of $node1; false otherwise." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 2, "name" : "preceding-of", "qname" : "node:preceding-of", "signature" : "($node1 as node(), $node2 as node()) as xs:boolean external", "description" : " Determines whether the node given as second argument is\n preceding the node given as first argument.\n", "summary" : "<p> Determines whether the node given as second argument is\n preceding the node given as first argument.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "node1", "type" : "node()", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the potential following node</div>" }, { "name" : "node2", "type" : "node()", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the potential preceding node</div>" } ], "returns" : { "type" : "xs:boolean", "description" : "true if $node2 is preceding the node $node1; false otherwise." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 2, "name" : "preceding-sibling-of", "qname" : "node:preceding-sibling-of", "signature" : "($node1 as node(), $node2 as node()) as xs:boolean external", "description" : " Determines whether the node given as second argument is a\n preceding-sibling of the node given as first argument.\n", "summary" : "<p> Determines whether the node given as second argument is a\n preceding-sibling of the node given as first argument.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "node1", "type" : "node()", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the potential following-sibling</div>" }, { "name" : "node2", "type" : "node()", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the potential preceding-sibling</div>" } ], "returns" : { "type" : "xs:boolean", "description" : "true if $node2 is a preceding-sibling of $node1; false otherwise." }, "errors" : [  ] } ], "variables" : [  ] }, "http://zorba.io/modules/archive" : { "ns" : "http://zorba.io/modules/archive", "description" : " This module provides functionality to work with (possibly compressed)\n archives. For example, it provides functions to retrieve the names or\n extract the values of several entries in a ZIP archive. Moreover,\n there exist functions that allow to create or update archives.<p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n The following archive formats and compression algorithms are supported:\n <ul xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">\n   <li>ZIP (with compression DEFLATE or STORE)</li>\n   <li>TAR (with compression GZIP)</li>\n </ul>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n", "sees" : [  ], "authors" : [ "<xqdoc:author xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Luis Rodgriguez, Juan Zacarias, and Matthias Brantner</xqdoc:author>" ], "version" : null, "encoding" : "utf-8", "namespaces" : [ { "uri" : "http://zorba.io/modules/archive", "prefix" : "a" }, { "uri" : "http://zorba.io/options/versioning", "prefix" : "ver" } ], "functions" : [ { "isDocumented" : true, "arity" : 2, "name" : "create", "qname" : "a:create", "signature" : "($entries as item()*, $contents as item()*) as xs:base64Binary external", "description" : " Creates a new ZIP archive out of the given entries and contents. <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n All entries are compressed with the DEFLATE compression algorithm.<p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n The parameters $entries and $contents have the same meaning as for\n the function a:create with three arguments.<p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n Entry entries can include a type element, this element can have one\n of two possible values: \"regular\" or \"directory\". If \"regular\" is\n specified then the entry will be created as a regular file; if \"directory\"\n is specified then the entry will be created as a directory, no contents\n will be read from $contents in this case. If no value is specified for type\n then it will be set to \"regular\". <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n Example:\n <pre xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">\n $zip-file := a:create(\n    ({ \"encoding\" : \"ISO-8859-1\", \"type\" : \"directory\", \"name\" : \"dir1\" }, \"dir1/file1\"),\n    (\"file contents\"))\n </pre>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n", "summary" : "<p> Creates a new ZIP archive out of the given entries and contents.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "entries", "type" : "item()", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the meta data for the entries in the archive. Each entry can be of type xs:string or a JSON oibject describing the entry.</div>" }, { "name" : "contents", "type" : "item()", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the content for the archive. Each item in the sequence can be of type xs:string or xs:base64Binary.</div>" } ], "returns" : { "type" : "xs:base64Binary", "description" : "the generated archive as xs:base64Binary" }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">a:ENTRY-COUNT-MISMATCH if the number of entries that don't describe directories differs from the number of items in the $contents sequence: count($non-directory-entries) ne count($contents)</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">a:INVALID-ENTRY-VALS if a values in an entry object are invalid</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">a:INVALID-ENCODING if a given encoding is invalid or not supported</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">err:FORG0006 if an item in the contents sequence is not of type xs:string or xs:base64Binary</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 3, "name" : "create", "qname" : "a:create", "signature" : "($entries as item()*, $contents as item()*, $options as object()) as xs:base64Binary external", "description" : " Creates a new archive out of the given entries and contents. <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n The $entries arguments provides meta data for each entry in the archive.\n For example, the name of the entry (mandatory) or the last-modified date\n (optional). An entry can either be of type xs:string to describe the entry\n name or of type xs:base64Binary to provide additional meta data.<p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n The $contents sequence provides the data (xs:string or xs:base64Binary) for\n the entries that should be included in the archive. Its length needs to\n match the length of the entries in the $entries sequence that don't describe\n directory entries (a:ARCH0001). All items of type xs:base64Binary are decoded\n before being added to the archive.<p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n For each entry, the name, last-modified date and time, and compression\n can be specified. In addition, an encoding can be specified which is used to\n store entries of type xs:string. If no last-modified attribute is given, the\n default is the current date and time. The compression is useful if various\n entries in a ZIP archive are compressed using different compression\n algorithms (i.e. store or deflate).<p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n For example, the following sequence may be used to describe an archive\n containing two elements: <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n <pre xmlns:xqdoc=\"http://www.xqdoc.org/1.0\" class=\"ace-static\" ace-mode=\"xquery\">{\n   \"last-modified\" : \"{fn:current-dateTime()}\"\n   \"name\" : \"myfile.txt\"\n },\n {\n   \"encoding\" : \"ISO-8859-1\",\n   \"compression\" : \"store\",\n   \"name\" : \"dir/myfile.xml\"\n }\n </pre>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n The $options argument may be used to describe general options for the\n archive.  For example, the following options can be used to create a ZIP\n archive in which all entries are compressed with the DEFLATE compression\n algorithm: <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n <pre xmlns:xqdoc=\"http://www.xqdoc.org/1.0\" class=\"ace-static\" ace-mode=\"xquery\">{\n   \"format\" : \"ZIP\",\n   \"compression\" : \"DEFLATE\"\n }\n </pre>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n The result of the function is the generated archive as a item of type\n xs:base64Binary.<p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n", "summary" : "<p> Creates a new archive out of the given entries and contents.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "entries", "type" : "item()", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the meta data for the entries in the archive. Each entry can be of type xs:string or an JSON object describing the entry.</div>" }, { "name" : "contents", "type" : "item()", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the content for the archive. Each item in the sequence can be of type xs:string or xs:base64Binary.</div>" }, { "name" : "options", "type" : "object()", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the options used to generate the archive.</div>" } ], "returns" : { "type" : "xs:base64Binary", "description" : "the generated archive as xs:base64Binary" }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">a:ENTRY-COUNT-MISMATCH if the number of entries describing non-directories differs from the number of items in the $contents sequence: count($non-directoy-entries) ne count($contents)</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">a:INVALID-OPTIONS if the options argument contains invalid values</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">a:INVALID-ENTRY-VALS if any values in an entry are invalid</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">a:INVALID-ENCODING if a given encoding is invalid or not supported</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">a:DIFFERENT-COMPRESSIONS-NOT-SUPPORTED if different compression algorithms were selected but the actual version of libarchive doesn't support it.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">err:FORG0006 if an item in the contents sequence is not of type xs:string or xs:base64Binary</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 2, "name" : "delete", "qname" : "a:delete", "signature" : "($archive as xs:base64Binary, $entry-names as xs:string*) as xs:base64Binary external", "description" : " Deletes entries from an archive. <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n", "summary" : "<p> Deletes entries from an archive.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "archive", "type" : "xs:base64Binary", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the archive to extract the entries from as xs:base64Binary</div>" }, { "name" : "entry-names", "type" : "xs:string", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> a sequence of names for entries which should be deleted</div>" } ], "returns" : { "type" : "xs:base64Binary", "description" : "the updated base64Binary" }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">a:CORRUPTED-ARCHIVE if $archive is not an archive or corrupted</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 1, "name" : "entries", "qname" : "a:entries", "signature" : "($archive as xs:base64Binary) as object()* external", "description" : " Returns the header information of all entries in the given archive as a JSON\n objects sequence. <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n Such information includes the name of the entry, the uncompressed size,\n as well as the last-modified timestamp. Note that not all values are\n available in every archive.<p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n", "summary" : "<p> Returns the header information of all entries in the given archive as a JSON\n objects sequence.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "archive", "type" : "xs:base64Binary", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the archive to list the entries from as xs:base64Binary</div>" } ], "returns" : { "type" : "object()*", "description" : "a sequence of strings, one for each entry in the archive" }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">a:CORRUPTED-ARCHIVE if $archive is not an archive or corrupted</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 1, "name" : "extract-binary", "qname" : "a:extract-binary", "signature" : "($archive as xs:base64Binary) as xs:base64Binary* external", "description" : " Returns the entries identified by the given paths from the archive\n as base64Binary. <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n", "summary" : "<p> Returns the entries identified by the given paths from the archive\n as base64Binary.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "archive", "type" : "xs:base64Binary", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the archive to extract the entries from as xs:base64Binary</div>" } ], "returns" : { "type" : "xs:base64Binary*", "description" : "one xs:base64Binary item for the contents of each entry in the archive" }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">a:CORRUPTED-ARCHIVE if $archive is not an archive or corrupted</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 2, "name" : "extract-binary", "qname" : "a:extract-binary", "signature" : "($archive as xs:base64Binary, $entry-names as xs:string*) as xs:base64Binary* external", "description" : " Returns the entries identified by the given paths from the archive\n as base64Binary. <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n", "summary" : "<p> Returns the entries identified by the given paths from the archive\n as base64Binary.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "archive", "type" : "xs:base64Binary", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the archive to extract the entries from as xs:base64Binary</div>" }, { "name" : "entry-names", "type" : "xs:string", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> a sequence of names for entries which should be extracted</div>" } ], "returns" : { "type" : "xs:base64Binary*", "description" : "a sequence of xs:base64Binary itmes for the given sequence of names or the empty sequence if no entries match the given names." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">a:CORRUPTED-ARCHIVE if $archive is not an archive or corrupted</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 1, "name" : "extract-text", "qname" : "a:extract-text", "signature" : "($archive as xs:base64Binary) as xs:string* external", "description" : " Extracts the contents of all entries in the given archive as text\n using UTF-8 as default encoding. <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n", "summary" : "<p> Extracts the contents of all entries in the given archive as text\n using UTF-8 as default encoding.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "archive", "type" : "xs:base64Binary", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the archive to extract the entries from as xs:base64Binary</div>" } ], "returns" : { "type" : "xs:string*", "description" : "one string for the contents of each entry in the archive" }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">a:CORRUPTED-ARCHIVE if $archive is not an archive or corrupted</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">err:FOCH0001 if any of the entries contains invalid utf-8 characters</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 2, "name" : "extract-text", "qname" : "a:extract-text", "signature" : "($archive as xs:base64Binary, $entry-names as xs:string*) as xs:string* external", "description" : " Extracts the contets of the entries identified by a given sequence of\n names as text.\n The default encoding used to read the string is UTF-8. <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n", "summary" : "<p> Extracts the contets of the entries identified by a given sequence of\n names as text.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "archive", "type" : "xs:base64Binary", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the archive to extract the entries from as xs:base64Binary</div>" }, { "name" : "entry-names", "type" : "xs:string", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> a sequence of names for entries which should be extracted</div>" } ], "returns" : { "type" : "xs:string*", "description" : "a sequence of strings for the given sequence of names or the empty sequence if no entries match the given names." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">a:CORRUPTED-ARCHIVE if $archive is not an archive or corrupted</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">err:FOCH0001 if any of the entries requested contains invalid utf-8 characters</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 3, "name" : "extract-text", "qname" : "a:extract-text", "signature" : "($archive as xs:base64Binary, $entry-names as xs:string*, $encoding as xs:string) as xs:string* external", "description" : " Extracts the contets of the entries identified by a given sequence of\n names as text. Each entry is treated with the given encoding. <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n", "summary" : "<p> Extracts the contets of the entries identified by a given sequence of\n names as text.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "archive", "type" : "xs:base64Binary", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the archive to extract the entries from as xs:base64Binary</div>" }, { "name" : "entry-names", "type" : "xs:string", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> a sequence of entry names that should be extracted</div>" }, { "name" : "encoding", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> a encoding for transcoding each of the extracted entries</div>" } ], "returns" : { "type" : "xs:string*", "description" : "a sequence of strings for the given sequence of names or the empty sequence if no entries match the given names." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">a:CORRUPTED-ARCHIVE if $archive is not an archive or corrupted</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">a:INVALID-ENCODING if the given $encoding is invalid or not supported</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">err:FOCH0001 if a transcoding error happens</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 1, "name" : "options", "qname" : "a:options", "signature" : "($archive as xs:base64Binary) as object() external", "description" : " Returns the algorithm and format options as a JSON object for a given archive.\n For example, for a ZIP archive, the following options element\n would be returned: <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n <pre xmlns:xqdoc=\"http://www.xqdoc.org/1.0\" class=\"ace-static\" ace-mode=\"xquery\">{\n   \"format\" : \"ZIP\",\n   \"compression\" : \"DEFLATE\"\n }\n </pre>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n", "summary" : "<p> Returns the algorithm and format options as a JSON object for a given archive.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "archive", "type" : "xs:base64Binary", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the archive as xs:base64Binary</div>" } ], "returns" : { "type" : "object()", "description" : "the algorithm and format options as a JSON object" }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">a:CORRUPTED-ARCHIVE if $archive is not an archive or corrupted</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 3, "name" : "update", "qname" : "a:update", "signature" : "($archive as xs:base64Binary, $entries as item()*, $contents as item()*) as xs:base64Binary external", "description" : " Adds and replaces entries in an archive according to\n the given spec. The contents can be string and base64Binary items. <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n The parameters $entries and $contents have the same meaning as for\n the function a:create with three arguments.<p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n", "summary" : "<p> Adds and replaces entries in an archive according to\n the given spec.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "archive", "type" : "xs:base64Binary", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the archive to add or replace content</div>" }, { "name" : "entries", "type" : "item()", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the meta data for the entries in the archive. Each entry can be of type xs:string or a JSON object. For mandatory fields in the JSON object see create function.</div>" }, { "name" : "contents", "type" : "item()", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the content for the archive. Each item in the sequence can be of type xs:string or xs:base64Binary.</div>" } ], "returns" : { "type" : "xs:base64Binary", "description" : "the updated xs:base64Binary" }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">a:ENTRY-COUNT-MISMATCH if the number of entry elements differs from the number of items in the $contents sequence: count($non-directory-entries) ne count($contents)</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">a:INVALID-ENTRY-VALS if a value for an entry element is invalid</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">a:INVALID-ENCODING if a given encoding is invalid or not supported</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">a:DIFFERENT-COMPRESSIONS-NOT-SUPPORTED if different compression algorithms were selected but the actual version of libarchive doesn't support it.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">err:FORG0006 if an item in the contents sequence is not of type xs:string or xs:base64Binary</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">a:CORRUPTED-ARCHIVE if $archive is not an archive or corrupted</xqdoc:error>" ] } ], "variables" : [  ] }, "http://www.zorba-xquery.com/modules/converters/html" : { "ns" : "http://www.zorba-xquery.com/modules/converters/html", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">\n This module provides functions to <a href=\"http://www.w3.org/People/Raggett/tidy/\" target=\"_blank\">tidy</a> a HTML document. <br/>\n The functions in this module take an HTML document (a string) as parameter,\n tidy it in order to result in valid XHTML, and return this XHTML document as a document-node.\n </p>\n", "sees" : [  ], "authors" : [ "<xqdoc:author xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Sorin Nasoi</xqdoc:author>" ], "version" : null, "encoding" : "utf-8", "namespaces" : [ { "uri" : "http://ww.w3.org/2005/xqt-errors", "prefix" : "err" }, { "uri" : "http://www.zorba-xquery.com/modules/converters/html", "prefix" : "html" }, { "uri" : "http://www.zorba-xquery.com/modules/converters/html-options", "prefix" : "html-options" }, { "uri" : "http://zorba.io/modules/schema", "prefix" : "schema" }, { "uri" : "http://zorba.io/options/versioning", "prefix" : "ver" } ], "functions" : [ { "isDocumented" : true, "arity" : 1, "name" : "parse", "qname" : "html:parse", "signature" : "($html as xs:string) as document()", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">This function tidies the given HTML string and returns\n a valid XHTML document node.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">This functions automatically sets the following tidying parameters:\n   <ul>\n    <li>output-xml=yes</li>\n    <li>doctype=omit</li>\n    <li>quote-nbsp=no</li>\n    <li>char-encoding=utf8</li>\n    <li>newline=LF</li>\n    <li>tidy-mark=no</li>\n   </ul>\n </p>\n", "summary" : "<p>  This function tidies the given HTML string and returns\n a valid XHTML document node.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "html", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the HTML string to tidy</div>" } ], "returns" : { "type" : "document()", "description" : "the tidied XML document" }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">html:InternalError if an internal error occurred while tidying the string.</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 2, "name" : "parse", "qname" : "html:parse", "signature" : "($html as xs:string, $options as element(html-options:options)) as document()", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">This function tidies the given HTML string and returns\n a valid XHTML document node.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">The second parameter allows to specify options that\n configure the tidy process. This parameter is a sequence\n of name=value pairs. Allowed parameter names and values\n are documented at <a href=\"http://tidy.sourceforge.net/docs/quickref.html\">\n http://tidy.sourceforge.net/docs/quickref.html</a>.</p>\n", "summary" : "<p>  This function tidies the given HTML string and returns\n a valid XHTML document node.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "html", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the HTML string to tidy</div>" }, { "name" : "options", "type" : "element(html-options:options)", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> a set of name and value pairs that provide options to configure the tidy process that have to be validated against the \"http://www.zorba-xquery.com/modules/converters/html-options\" schema.</div>" } ], "returns" : { "type" : "document()", "description" : "the tidied XHTML document node" }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">err:XQDY0027 if $options can not be validated against the html-options schema</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">html:TidyOption if there was an error with one of the options in the $options parameter that couldn't have been caught by validating against the schema</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">html:InternalError if an internal error occurred while tidying the string.</xqdoc:error>" ] } ], "variables" : [  ] }, "http://zorba.io/modules/data-cleaning/set-similarity" : { "ns" : "http://zorba.io/modules/data-cleaning/set-similarity", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">This library module provides similarity functions for comparing sets of XML\n nodes (e.g., sets of XML elements, attributes or atomic values).</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">These functions are particularly useful for matching near duplicate sets of XML nodes.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">The logic contained in this module is not specific to any particular XQuery implementation.</p>\n", "sees" : [  ], "authors" : [ "<xqdoc:author xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Bruno Martins</xqdoc:author>" ], "version" : null, "encoding" : "utf-8", "namespaces" : [ { "uri" : "http://zorba.io/modules/data-cleaning/set-similarity", "prefix" : "set" }, { "uri" : "http://zorba.io/options/versioning", "prefix" : "ver" } ], "functions" : [ { "isDocumented" : true, "arity" : 2, "name" : "deep-intersect", "qname" : "set:deep-intersect", "signature" : "($s1, $s2) as item()*", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Returns the intersection between two sets, using the deep-equal() function to compare the XML nodes from the sets.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Example usage : <pre class=\"ace-static\" ace-mode=\"xquery\"> deep-intersect ( ( \"a\", \"b\", \"c\") , ( \"a\", \"a\", <d/> ) ) </pre></p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">The function invocation in the example above returns : <pre class=\"ace-static\" ace-mode=\"xquery\"> (\"a\") </pre></p>\n", "summary" : "<p>  Returns the intersection between two sets, using the deep-equal() function to compare the XML nodes from the sets.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "s1", "type" : null, "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The first set.</div>" }, { "name" : "s2", "type" : null, "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The second set.</div>" } ], "returns" : { "type" : "item()*", "description" : "The intersection of both sets." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 2, "name" : "deep-union", "qname" : "set:deep-union", "signature" : "($s1, $s2) as item()*", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Returns the union between two sets, using the deep-equal() function to compare the XML nodes from the sets.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Example usage : <pre class=\"ace-static\" ace-mode=\"xquery\"> deep-union ( ( \"a\", \"b\", \"c\") , ( \"a\", \"a\", <d/> ) ) </pre></p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">The function invocation in the example above returns : <pre class=\"ace-static\" ace-mode=\"xquery\"> (\"a\", \"b\", \"c\", <d/> ) </pre></p>\n", "summary" : "<p>  Returns the union between two sets, using the deep-equal() function to compare the XML nodes from the sets.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "s1", "type" : null, "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The first set.</div>" }, { "name" : "s2", "type" : null, "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The second set.</div>" } ], "returns" : { "type" : "item()*", "description" : "The union of both sets." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 2, "name" : "dice", "qname" : "set:dice", "signature" : "($s1, $s2) as xs:double", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Returns the Dice similarity coefficient between two sets of XML nodes.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">The Dice coefficient is defined as defined as twice the shared information between the input sets\n (i.e., the size of the intersection) over the sum of the cardinalities for the input sets.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Example usage : <pre class=\"ace-static\" ace-mode=\"xquery\"> dice ( ( \"a\", \"b\", <c/> ) , ( \"a\", \"a\", \"d\") ) </pre></p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">The function invocation in the example above returns : <pre class=\"ace-static\" ace-mode=\"xquery\"> 0.4 </pre></p>\n", "summary" : "<p>  Returns the Dice similarity coefficient between two sets of XML nodes.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "s1", "type" : null, "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The first set.</div>" }, { "name" : "s2", "type" : null, "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The second set.</div>" } ], "returns" : { "type" : "xs:double", "description" : "The Dice similarity coefficient between the two sets." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "distinct", "qname" : "set:distinct", "signature" : "($s) as item()*", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Removes exact duplicates from a set, using the deep-equal() function to compare the XML nodes from the sets.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Example usage : <pre class=\"ace-static\" ace-mode=\"xquery\"> distinct ( ( \"a\", \"a\", <b/> ) ) </pre></p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">The function invocation in the example above returns : <pre class=\"ace-static\" ace-mode=\"xquery\"> (\"a\", <b/> ) </pre></p>\n", "summary" : "<p>  Removes exact duplicates from a set, using the deep-equal() function to compare the XML nodes from the sets.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "s", "type" : null, "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> A set.</div>" } ], "returns" : { "type" : "item()*", "description" : "The set provided as input without the exact duplicates (i.e., returns the distinct nodes from the set provided as input)." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 2, "name" : "jaccard", "qname" : "set:jaccard", "signature" : "($s1, $s2) as xs:double", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Returns the Jaccard similarity coefficient between two sets of XML nodes.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">The Jaccard coefficient is defined as the size of the intersection divided by the size of the\n union of the input sets.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Example usage : <pre class=\"ace-static\" ace-mode=\"xquery\"> jaccard ( ( \"a\", \"b\", <c/> ) , ( \"a\", \"a\", \"d\") ) </pre></p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">The function invocation in the example above returns : <pre class=\"ace-static\" ace-mode=\"xquery\"> 0.25 </pre></p>\n", "summary" : "<p>  Returns the Jaccard similarity coefficient between two sets of XML nodes.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "s1", "type" : null, "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The first set.</div>" }, { "name" : "s2", "type" : null, "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The second set.</div>" } ], "returns" : { "type" : "xs:double", "description" : "The Jaccard similarity coefficient between the two sets." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 2, "name" : "overlap", "qname" : "set:overlap", "signature" : "($s1, $s2) as xs:double", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Returns the overlap coefficient between two sets of XML nodes.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">The overlap coefficient is defined as the shared information between the input sets\n (i.e., the size of the intersection) over the size of the smallest input set.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Example usage : <pre class=\"ace-static\" ace-mode=\"xquery\"> overlap ( ( \"a\", \"b\", <c/> ) , ( \"a\", \"a\", \"b\" ) ) </pre></p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">The function invocation in the example above returns : <pre class=\"ace-static\" ace-mode=\"xquery\"> 1.0 </pre></p>\n", "summary" : "<p>  Returns the overlap coefficient between two sets of XML nodes.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "s1", "type" : null, "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The first set.</div>" }, { "name" : "s2", "type" : null, "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The second set.</div>" } ], "returns" : { "type" : "xs:double", "description" : "The overlap coefficient between the two sets." }, "errors" : [  ] } ], "variables" : [  ] }, "http://www.28msec.com/modules/http/util/cache" : { "ns" : "http://www.28msec.com/modules/http/util/cache", "description" : " This module contains a collection of convenience functions to help\n developers work with HTTP caching options.\n", "sees" : [  ], "authors" : [ "<xqdoc:author xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">28msec</xqdoc:author>" ], "version" : null, "encoding" : "utf-8", "namespaces" : [ { "uri" : "http://zorba.io/annotations", "prefix" : "an" }, { "uri" : "http://www.28msec.com/modules/http/util/cache", "prefix" : "cache" }, { "uri" : "http://www.28msec.com/modules/http/response", "prefix" : "resp" }, { "uri" : "http://zorba.io/options/versioning", "prefix" : "ver" } ], "functions" : [ { "isDocumented" : true, "arity" : 0, "name" : "no-cache", "qname" : "cache:no-cache", "signature" : "() as empty-sequence()", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Forces a client not to cache the response by setting the appropriate\n HTTP headers.</p>\n In detail, the following HTTP headers will be set in the response:\n <ul xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">\n   <li><tt>Cache-Control: no-cache</tt></li>\n   <li><tt>Pragma: no-cache</tt></li>\n   <li><tt>Expires: 0</tt></li>\n </ul>\n", "summary" : "<p>  Forces a client not to cache the response by setting the appropriate\n HTTP headers.</p>", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "http://zorba.io/annotations", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [  ], "returns" : { "type" : "empty-sequence()", "description" : "On success, the empty-sequence is returned" }, "errors" : [  ] } ], "variables" : [  ] }, "http://www.zorba-xquery.com/modules/schema-tools" : { "ns" : "http://www.zorba-xquery.com/modules/schema-tools", "description" : " This module provides funtionality to get sample XMLSchema from XML instances\n and sample XML instances from XMLSchema.\n <a xmlns:xqdoc=\"http://www.xqdoc.org/1.0\" href=\"http://xmlbeans.apache.org/\">Apache XMLBeans</a> library is used to implement\n inst2xsd and xsd2inst functions.\n <br xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n <br xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n <b xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Note:</b> Since this module has a Java library dependency a JVM required\n to be installed on the system. For Windows: jvm.dll is required on the system\n path ( usually located in \"C:\\Program Files\\Java\\jre6\\bin\\client\".\n", "sees" : [ "<xqdoc:see xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">http://xmlbeans.apache.org/</xqdoc:see>", "<xqdoc:see xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">http://xmlbeans.apache.org</xqdoc:see>" ], "authors" : [ "<xqdoc:author xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Cezar Andrei</xqdoc:author>" ], "version" : null, "encoding" : "utf-8", "namespaces" : [ { "uri" : "http://www.w3.org/2005/xqt-errors", "prefix" : "err" }, { "uri" : "http://zorba.io/modules/schema", "prefix" : "schema-options" }, { "uri" : "http://www.zorba-xquery.com/modules/schema-tools", "prefix" : "schema-tools" }, { "uri" : "http://www.zorba-xquery.com/modules/schema-tools/schema-tools-options", "prefix" : "st-options" }, { "uri" : "http://zorba.io/options/versioning", "prefix" : "ver" } ], "functions" : [ { "isDocumented" : true, "arity" : 2, "name" : "inst2xsd", "qname" : "schema-tools:inst2xsd", "signature" : "($instances as element(*)+, $options as element(st-options:inst2xsd-options)?) as document()*", "description" : " The inst2xsd function takes a set of sample XML instance elements as input and\n generates a set of sample XMLSchema documents that define\n the content of the given input.\n <br xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n Please consult the\n <a xmlns:xqdoc=\"http://www.xqdoc.org/1.0\" href=\"http://xmlbeans.apache.org/\">official documentation for further\n information</a>.\n <br xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n Example:<pre xmlns:xqdoc=\"http://www.xqdoc.org/1.0\" class=\"ace-static\" ace-mode=\"xquery\">\n  import module namespace st = \"http://www.zorba-xquery.com/modules/schema-tools\";\n  declare namespace sto =\n      \"http://www.zorba-xquery.com/modules/schema-tools/schema-tools-options\";\n  let $instances := (&lt;a&gt;&lt;b/&gt;&lt;c/&gt;&lt;/a&gt;, &lt;b/&gt;, &lt;c&gt;ccc&lt;/c&gt;)\n  let $options  :=\n     &lt;sto:inst2xsd-options xmlns:sto=\n       \"http://www.zorba-xquery.com/modules/schema-tools/schema-tools-options\"&gt;\n       &lt;sto:design&gt;vbd&lt;/sto:design&gt;\n       &lt;sto:simple-content-types&gt;smart&lt;/sto:simple-content-types&gt;\n       &lt;sto:use-enumeration&gt;10&lt;/sto:use-enumeration&gt;\n     &lt;/sto:inst2xsd-options&gt;\n  return\n      st:inst2xsd($instances, $options)\n </pre>\n <br xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n", "summary" : "<p> The inst2xsd function takes a set of sample XML instance elements as input and\n generates a set of sample XMLSchema documents that define\n the content of the given input.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "instances", "type" : "element(*)", "occurrence" : "+", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The input XML instance elements</div>" }, { "name" : "options", "type" : "element(st-options:inst2xsd-options)", "occurrence" : "?", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> Options:<br/> <ul> <li>design: Choose the generated schema design<br/> - rdd: Russian Doll Design - local elements and local types<br/> - ssd: Salami Slice Design - global elements and local types<br/> - vbd (default): Venetian Blind Design - local elements and global complex types</li> <li>simple-content-types: type of leaf nodes<br/> - smart (default): try to find the right simple XMLSchema type<br/> - always-string: use xsd:string for all simple types</li> <li>use-enumeration: - when there are multiple valid values in a list<br/> - 1: never use enumeration<br/> - 2 or more (default 10): use enumeration if less than this number of occurrences - number option</li> <li>verbose: - stdout verbose info<br/> - true: - output type holder information<br/> - false (default): no output</li></ul></div>" } ], "returns" : { "type" : "document()*", "description" : "The generated XMLSchema documents." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">schema-tools:VM001 If Zorba was unable to start the JVM.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">schema-tools:JAVA-EXCEPTION If Apache XMLBeans throws an exception.</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 3, "name" : "xsd2inst", "qname" : "schema-tools:xsd2inst", "signature" : "($schemas as element(*)+, $rootElementName as xs:string, $options as element(st-options:xsd2inst-options)?) as document()", "description" : " The xsd2inst function takes a set of XML Schema elements as input and the\n local name of the root element and\n generates a document that represents one sample XML instance of the given\n input schemas. The local name is searched in schema global element definitions\n in the order of schemas parameter.\n <br xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n Please consult the\n <a xmlns:xqdoc=\"http://www.xqdoc.org/1.0\" href=\"http://xmlbeans.apache.org/\">official documentation for further\n   information</a>.\n <br xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n Example: <pre xmlns:xqdoc=\"http://www.xqdoc.org/1.0\" class=\"ace-static\" ace-static=\"xquery\">\n  import module namespace st = \"http://www.zorba-xquery.com/modules/schema-tools\";\n  declare namespace sto =\n      \"http://www.zorba-xquery.com/modules/schema-tools/schema-tools-options\";\n  let $xsds  :=\n     ( &lt;xs:schema xmlns:xs=\"http://www.w3.org/2001/XMLSchema\"\n           attributeFormDefault=\"unqualified\"\n           elementFormDefault=\"qualified\"&gt;\n         &lt;xs:element name=\"a\" type=\"aType\"/&gt;\n         &lt;xs:complexType name=\"aType\"&gt;\n           &lt;xs:sequence&gt;\n             &lt;xs:element type=\"xs:string\" name=\"b\"/&gt;\n             &lt;xs:element type=\"xs:string\" name=\"c\"/&gt;\n           &lt;/xs:sequence&gt;\n         &lt;/xs:complexType&gt;\n       &lt;/xs:schema&gt; )\n  let $options :=\n    &lt;sto:xsd2inst-options xmlns:sto=\n      \"http://www.zorba-xquery.com/modules/schema-tools/schema-tools-options\"&gt;\n      &lt;sto:network-downloads&gt;false&lt;/sto:network-downloads&gt;\n      &lt;sto:no-pvr&gt;false&lt;/sto:no-pvr&gt;\n      &lt;sto:no-upa&gt;false&lt;/sto:no-upa&gt;\n    &lt;/sto:xsd2inst-options&gt;\n  return\n      st:xsd2inst($xsds, \"a\", $options)\n </pre><br xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"/>\n", "summary" : "<p> The xsd2inst function takes a set of XML Schema elements as input and the\n local name of the root element and\n generates a document that represents one sample XML instance of the given\n input schemas.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "schemas", "type" : "element(*)", "occurrence" : "+", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> elements representing XMLSchema definitions</div>" }, { "name" : "rootElementName", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> The local name of the instance root element. If multiple target namespaces are used, first one found - using the sequence order - will be used.</div>" }, { "name" : "options", "type" : "element(st-options:xsd2inst-options)", "occurrence" : "?", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> Options:<br/><ul> <li>network-downloads: boolean (default false)<br/> - true allows XMLBeans to use network when resolving schema imports and includes</li> <li>no-pvr: boolean (default false)<br/> - true to disable particle valid (restriction) rule, false otherwise</li> <li>no-upa: boolean (default false)<br/> - true to disable unique particle attribution rule, false otherwise</li></ul></div>" } ], "returns" : { "type" : "document()", "description" : "The generated output document, representing a sample XML instance." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">schema-tools:VM001 If Zorba was unable to start the JVM.</xqdoc:error>", "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">schema-tools:JAVA-EXCEPTION If Apache XMLBeans throws an exception.</xqdoc:error>" ] } ], "variables" : [  ] }, "http://xbrl.io/modules/bizql/entities" : { "ns" : "http://xbrl.io/modules/bizql/entities", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">This module provides functions for retrieving information about entities.\n Entities submit archives (see archives module),\n for example, to a reporting authority. Entities\n are identified with an EID (Entity ID).</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">With this module, you can retrieve all entities, or a certain number of entities\n with their EIDs, or obtain the EID of entities you already have.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Entities are stored in a MongoDB datasource called <b>xbrl</b>.</p>\n <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Each entity can be associated with a set of tags that makes it easier\n to manage them. The e:entities#1 function\n allows to retrieve all entities with a given tag.</p>\n", "sees" : [  ], "authors" : [ "<xqdoc:author xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Charles Hoffman</xqdoc:author>", "<xqdoc:author xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Matthias Brantner</xqdoc:author>", "<xqdoc:author xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Dennis Knochenwefel</xqdoc:author>", "<xqdoc:author xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Ghislain Fourny</xqdoc:author>" ], "version" : null, "encoding" : "utf-8", "namespaces" : [ { "uri" : "http://www.28msec.com/modules/credentials", "prefix" : "credentials" }, { "uri" : "http://xbrl.io/modules/bizql/entities", "prefix" : "entities" }, { "uri" : "http://www.28msec.com/modules/mongodb", "prefix" : "mongo" }, { "uri" : "http://zorba.io/options/versioning", "prefix" : "ver" } ], "functions" : [ { "isDocumented" : true, "arity" : 1, "name" : "eid", "qname" : "entities:eid", "signature" : "($entities-or-ids as item()*) as atomic*", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Converts the input to a normalized entity identifier (EID). The input\n can be either an EID, or an entity object which contains an _id.</p>\n", "summary" : "<p>  Converts the input to a normalized entity identifier (EID).</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "entities-or-ids", "type" : "item()", "occurrence" : "*", "description" : "" } ], "returns" : { "type" : "atomic*", "description" : "the normalized EIDs." }, "errors" : [ "<xqdoc:error xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">entities:INVALID_PARAMETER if the EID or entity is not valid.</xqdoc:error>" ] }, { "isDocumented" : true, "arity" : 0, "name" : "entities", "qname" : "entities:entities", "signature" : "() as object()*", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Return all entities.</p>\n", "summary" : "<p>  Return all entities.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [  ], "returns" : { "type" : "object()*", "description" : "all entities." }, "errors" : [  ] }, { "isDocumented" : true, "arity" : 1, "name" : "entities", "qname" : "entities:entities", "signature" : "($entities-or-ids as item()*) as object()*", "description" : " <p xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">Return the entities with the given EIDs.</p>\n", "summary" : "<p>  Return the entities with the given EIDs.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "entities-or-ids", "type" : "item()", "occurrence" : "*", "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the ids of the entities (EIDs) or the entities themselves.</div>" } ], "returns" : { "type" : "object()*", "description" : "the entities with the given EIDs the empty sequence if no entity was found or if the input is an empty sequence." }, "errors" : [  ] } ], "variables" : [ { "name" : "entities:col", "type" : "string", "description" : " Name of the collection the entities are stored in.\n" } ] }, "http://www.zorba-xquery.com/modules/cryptography/hmac" : { "ns" : "http://www.zorba-xquery.com/modules/cryptography/hmac", "description" : " This module provides a function to generate hash-based\n message authentication codes (HMAC) involving a cryptographic\n hash function (e.g. SHA1).\n", "sees" : [  ], "authors" : [ "<xqdoc:author xmlns:xqdoc=\"http://www.xqdoc.org/1.0\">William Candillon</xqdoc:author>" ], "version" : null, "encoding" : "utf-8", "namespaces" : [ { "uri" : "http://www.zorba-xquery.com/modules/cryptography/hmac", "prefix" : "hmac" }, { "uri" : "http://zorba.io/options/versioning", "prefix" : "ver" } ], "functions" : [ { "isDocumented" : true, "arity" : 2, "name" : "sha1", "qname" : "hmac:sha1", "signature" : "($message as xs:string, $secret-key as xs:string) as xs:string external", "description" : " This function provides hash-based message authentication code using\n the SHA1 algorithm.\n", "summary" : "<p> This function provides hash-based message authentication code using\n the SHA1 algorithm.</p>", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "message", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the message to be authenticated</div>" }, { "name" : "secret-key", "type" : "xs:string", "occurrence" : null, "description" : "<div xmlns:xqdoc=\"http://www.xqdoc.org/1.0\"> the secret key used for calculating the authentication</div>" } ], "returns" : { "type" : "xs:string", "description" : "hash-based base64 encoded message authentication code" }, "errors" : [  ] } ], "variables" : [  ] }, "http://api.28.io/csvview" : { "ns" : "http://api.28.io/csvview", "description" : "", "sees" : [  ], "authors" : [  ], "version" : null, "encoding" : null, "namespaces" : [ { "uri" : "", "prefix" : "an" }, { "uri" : "http://api.28.io/csvview", "prefix" : "csvview" }, { "uri" : "http://api.28.io/model", "prefix" : "model" } ], "functions" : [ { "isDocumented" : false, "arity" : 1, "name" : "column", "qname" : "csvview:column", "signature" : "($data) as xs:string", "description" : "", "summary" : "", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "data", "type" : null, "occurrence" : null, "description" : "" } ], "returns" : { "type" : "xs:string", "description" : "" }, "errors" : [  ] }, { "isDocumented" : false, "arity" : 1, "name" : "line", "qname" : "csvview:line", "signature" : "($entries as xs:string*)", "description" : "", "summary" : "", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "entries", "type" : "xs:string", "occurrence" : "*", "description" : "" } ], "returns" : { "type" : null, "description" : "" }, "errors" : [  ] }, { "isDocumented" : false, "arity" : 2, "name" : "show-nodes", "qname" : "csvview:show-nodes", "signature" : "($nodes, $show-headers as xs:boolean)", "description" : "", "summary" : "", "annotation_str" : " %an:sequential", "annotations" : [ { "prefix" : "an", "ns" : "", "name" : "sequential", "value" : "" } ], "updating" : false, "parameters" : [ { "name" : "nodes", "type" : null, "occurrence" : null, "description" : "" }, { "name" : "show-headers", "type" : "xs:boolean", "occurrence" : null, "description" : "" } ], "returns" : { "type" : null, "description" : "" }, "errors" : [  ] } ], "variables" : [  ] }, "http://api.28.io/util" : { "ns" : "http://api.28.io/util", "description" : "", "sees" : [  ], "authors" : [  ], "version" : null, "encoding" : null, "namespaces" : [ { "uri" : "http://www.28msec.com/modules/http/request", "prefix" : "req" }, { "uri" : "http://api.28.io/util", "prefix" : "util" } ], "functions" : [ { "isDocumented" : false, "arity" : 1, "name" : "param-or-body", "qname" : "util:param-or-body", "signature" : "($param-name as xs:string) as xs:string?", "description" : "", "summary" : "", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "param-name", "type" : "xs:string", "occurrence" : null, "description" : "" } ], "returns" : { "type" : "xs:string?", "description" : "" }, "errors" : [  ] }, { "isDocumented" : false, "arity" : 1, "name" : "path", "qname" : "util:path", "signature" : "($from as xs:integer) as xs:string*", "description" : "", "summary" : "", "annotation_str" : "", "annotations" : [  ], "updating" : false, "parameters" : [ { "name" : "from", "type" : "xs:integer", "occurrence" : null, "description" : "" } ], "returns" : { "type" : "xs:string*", "description" : "" }, "errors" : [  ] } ], "variables" : [ { "name" : "util:collection-namespace", "type" : "item()*", "description" : "" } ] } };
});

define("ace/mode/xquery_worker",["require","exports","module","ace/lib/oop","ace/worker/mirror","ace/mode/xquery/xqlint","ace/mode/xquery/modules"], function(require, exports, module) {
"use strict";
    
var oop = require("../lib/oop");
var Mirror = require("../worker/mirror").Mirror;
var XQLintLib = require("./xquery/xqlint");
var XQLint =  XQLintLib.XQLint;
var Modules = require("./xquery/modules").Modules;

var getModuleResolverFromModules = function(modules){
    return function(uri){
            var index = modules;
            var mod = index[uri];
            var variables = {};
            var functions = {};
            mod.functions.forEach(function(fn){
                functions[uri + '#' + fn.name + '#' + fn.arity] = {
                    params: []
                };
                fn.parameters.forEach(function(param){
                    functions[uri + '#' + fn.name + '#' + fn.arity].params.push('$' + param.name);
                });
            });
            mod.variables.forEach(function(variable){
                var name = variable.name.substring(variable.name.indexOf(':') + 1);
                variables[uri + '#' + name] = { type: 'VarDecl', annotations: [] };
            });
            return {
                variables: variables,
                functions: functions
            };
    };
};

var XQueryWorker = exports.XQueryWorker = function(sender) {
    Mirror.call(this, sender);
    this.setTimeout(200);
    this.opts = {
        styleCheck: false
    };
    this.availableModuleNamespaces = Object.keys(Modules);
    this.moduleResolver = getModuleResolverFromModules(Modules);
    var that = this;

    this.sender.on("complete", function(e){
        if(that.xqlint) {
            var pos = { line: e.data.pos.row, col: e.data.pos.column };
            var proposals = that.xqlint.getCompletions(pos);
            that.sender.emit("complete", proposals);
        }
    });

    this.sender.on("setAvailableModuleNamespaces", function(e){
        that.availableModuleNamespaces = e.data;
    });

    this.sender.on("setModuleResolver", function(e){
        that.moduleResolver = getModuleResolverFromModules(e.data);
    });
};

oop.inherits(XQueryWorker, Mirror);

(function() {
    
    this.onUpdate = function() {
        this.sender.emit("start");
        var value = this.doc.getValue();
        var sctx = XQLintLib.createStaticContext();
        if(this.moduleResolver) {
            sctx.setModuleResolver(this.moduleResolver);
        }
        if(this.availableModuleNamespaces) {
            sctx.availableModuleNamespaces = this.availableModuleNamespaces;
        }
        var opts = {
            styleCheck: this.styleCheck,
            staticContext: sctx,
        };
        this.xqlint = new XQLint(value, opts);
        this.sender.emit("markers", this.xqlint.getMarkers());
    };
}).call(XQueryWorker.prototype);

});

define("ace/lib/es5-shim",["require","exports","module"], function(require, exports, module) {

function Empty() {}

if (!Function.prototype.bind) {
    Function.prototype.bind = function bind(that) { // .length is 1
        var target = this;
        if (typeof target != "function") {
            throw new TypeError("Function.prototype.bind called on incompatible " + target);
        }
        var args = slice.call(arguments, 1); // for normal call
        var bound = function () {

            if (this instanceof bound) {

                var result = target.apply(
                    this,
                    args.concat(slice.call(arguments))
                );
                if (Object(result) === result) {
                    return result;
                }
                return this;

            } else {
                return target.apply(
                    that,
                    args.concat(slice.call(arguments))
                );

            }

        };
        if(target.prototype) {
            Empty.prototype = target.prototype;
            bound.prototype = new Empty();
            Empty.prototype = null;
        }
        return bound;
    };
}
var call = Function.prototype.call;
var prototypeOfArray = Array.prototype;
var prototypeOfObject = Object.prototype;
var slice = prototypeOfArray.slice;
var _toString = call.bind(prototypeOfObject.toString);
var owns = call.bind(prototypeOfObject.hasOwnProperty);
var defineGetter;
var defineSetter;
var lookupGetter;
var lookupSetter;
var supportsAccessors;
if ((supportsAccessors = owns(prototypeOfObject, "__defineGetter__"))) {
    defineGetter = call.bind(prototypeOfObject.__defineGetter__);
    defineSetter = call.bind(prototypeOfObject.__defineSetter__);
    lookupGetter = call.bind(prototypeOfObject.__lookupGetter__);
    lookupSetter = call.bind(prototypeOfObject.__lookupSetter__);
}
if ([1,2].splice(0).length != 2) {
    if(function() { // test IE < 9 to splice bug - see issue #138
        function makeArray(l) {
            var a = new Array(l+2);
            a[0] = a[1] = 0;
            return a;
        }
        var array = [], lengthBefore;
        
        array.splice.apply(array, makeArray(20));
        array.splice.apply(array, makeArray(26));

        lengthBefore = array.length; //46
        array.splice(5, 0, "XXX"); // add one element

        lengthBefore + 1 == array.length

        if (lengthBefore + 1 == array.length) {
            return true;// has right splice implementation without bugs
        }
    }()) {//IE 6/7
        var array_splice = Array.prototype.splice;
        Array.prototype.splice = function(start, deleteCount) {
            if (!arguments.length) {
                return [];
            } else {
                return array_splice.apply(this, [
                    start === void 0 ? 0 : start,
                    deleteCount === void 0 ? (this.length - start) : deleteCount
                ].concat(slice.call(arguments, 2)))
            }
        };
    } else {//IE8
        Array.prototype.splice = function(pos, removeCount){
            var length = this.length;
            if (pos > 0) {
                if (pos > length)
                    pos = length;
            } else if (pos == void 0) {
                pos = 0;
            } else if (pos < 0) {
                pos = Math.max(length + pos, 0);
            }

            if (!(pos+removeCount < length))
                removeCount = length - pos;

            var removed = this.slice(pos, pos+removeCount);
            var insert = slice.call(arguments, 2);
            var add = insert.length;            
            if (pos === length) {
                if (add) {
                    this.push.apply(this, insert);
                }
            } else {
                var remove = Math.min(removeCount, length - pos);
                var tailOldPos = pos + remove;
                var tailNewPos = tailOldPos + add - remove;
                var tailCount = length - tailOldPos;
                var lengthAfterRemove = length - remove;

                if (tailNewPos < tailOldPos) { // case A
                    for (var i = 0; i < tailCount; ++i) {
                        this[tailNewPos+i] = this[tailOldPos+i];
                    }
                } else if (tailNewPos > tailOldPos) { // case B
                    for (i = tailCount; i--; ) {
                        this[tailNewPos+i] = this[tailOldPos+i];
                    }
                } // else, add == remove (nothing to do)

                if (add && pos === lengthAfterRemove) {
                    this.length = lengthAfterRemove; // truncate array
                    this.push.apply(this, insert);
                } else {
                    this.length = lengthAfterRemove + add; // reserves space
                    for (i = 0; i < add; ++i) {
                        this[pos+i] = insert[i];
                    }
                }
            }
            return removed;
        };
    }
}
if (!Array.isArray) {
    Array.isArray = function isArray(obj) {
        return _toString(obj) == "[object Array]";
    };
}
var boxedString = Object("a"),
    splitString = boxedString[0] != "a" || !(0 in boxedString);

if (!Array.prototype.forEach) {
    Array.prototype.forEach = function forEach(fun /*, thisp*/) {
        var object = toObject(this),
            self = splitString && _toString(this) == "[object String]" ?
                this.split("") :
                object,
            thisp = arguments[1],
            i = -1,
            length = self.length >>> 0;
        if (_toString(fun) != "[object Function]") {
            throw new TypeError(); // TODO message
        }

        while (++i < length) {
            if (i in self) {
                fun.call(thisp, self[i], i, object);
            }
        }
    };
}
if (!Array.prototype.map) {
    Array.prototype.map = function map(fun /*, thisp*/) {
        var object = toObject(this),
            self = splitString && _toString(this) == "[object String]" ?
                this.split("") :
                object,
            length = self.length >>> 0,
            result = Array(length),
            thisp = arguments[1];
        if (_toString(fun) != "[object Function]") {
            throw new TypeError(fun + " is not a function");
        }

        for (var i = 0; i < length; i++) {
            if (i in self)
                result[i] = fun.call(thisp, self[i], i, object);
        }
        return result;
    };
}
if (!Array.prototype.filter) {
    Array.prototype.filter = function filter(fun /*, thisp */) {
        var object = toObject(this),
            self = splitString && _toString(this) == "[object String]" ?
                this.split("") :
                    object,
            length = self.length >>> 0,
            result = [],
            value,
            thisp = arguments[1];
        if (_toString(fun) != "[object Function]") {
            throw new TypeError(fun + " is not a function");
        }

        for (var i = 0; i < length; i++) {
            if (i in self) {
                value = self[i];
                if (fun.call(thisp, value, i, object)) {
                    result.push(value);
                }
            }
        }
        return result;
    };
}
if (!Array.prototype.every) {
    Array.prototype.every = function every(fun /*, thisp */) {
        var object = toObject(this),
            self = splitString && _toString(this) == "[object String]" ?
                this.split("") :
                object,
            length = self.length >>> 0,
            thisp = arguments[1];
        if (_toString(fun) != "[object Function]") {
            throw new TypeError(fun + " is not a function");
        }

        for (var i = 0; i < length; i++) {
            if (i in self && !fun.call(thisp, self[i], i, object)) {
                return false;
            }
        }
        return true;
    };
}
if (!Array.prototype.some) {
    Array.prototype.some = function some(fun /*, thisp */) {
        var object = toObject(this),
            self = splitString && _toString(this) == "[object String]" ?
                this.split("") :
                object,
            length = self.length >>> 0,
            thisp = arguments[1];
        if (_toString(fun) != "[object Function]") {
            throw new TypeError(fun + " is not a function");
        }

        for (var i = 0; i < length; i++) {
            if (i in self && fun.call(thisp, self[i], i, object)) {
                return true;
            }
        }
        return false;
    };
}
if (!Array.prototype.reduce) {
    Array.prototype.reduce = function reduce(fun /*, initial*/) {
        var object = toObject(this),
            self = splitString && _toString(this) == "[object String]" ?
                this.split("") :
                object,
            length = self.length >>> 0;
        if (_toString(fun) != "[object Function]") {
            throw new TypeError(fun + " is not a function");
        }
        if (!length && arguments.length == 1) {
            throw new TypeError("reduce of empty array with no initial value");
        }

        var i = 0;
        var result;
        if (arguments.length >= 2) {
            result = arguments[1];
        } else {
            do {
                if (i in self) {
                    result = self[i++];
                    break;
                }
                if (++i >= length) {
                    throw new TypeError("reduce of empty array with no initial value");
                }
            } while (true);
        }

        for (; i < length; i++) {
            if (i in self) {
                result = fun.call(void 0, result, self[i], i, object);
            }
        }

        return result;
    };
}
if (!Array.prototype.reduceRight) {
    Array.prototype.reduceRight = function reduceRight(fun /*, initial*/) {
        var object = toObject(this),
            self = splitString && _toString(this) == "[object String]" ?
                this.split("") :
                object,
            length = self.length >>> 0;
        if (_toString(fun) != "[object Function]") {
            throw new TypeError(fun + " is not a function");
        }
        if (!length && arguments.length == 1) {
            throw new TypeError("reduceRight of empty array with no initial value");
        }

        var result, i = length - 1;
        if (arguments.length >= 2) {
            result = arguments[1];
        } else {
            do {
                if (i in self) {
                    result = self[i--];
                    break;
                }
                if (--i < 0) {
                    throw new TypeError("reduceRight of empty array with no initial value");
                }
            } while (true);
        }

        do {
            if (i in this) {
                result = fun.call(void 0, result, self[i], i, object);
            }
        } while (i--);

        return result;
    };
}
if (!Array.prototype.indexOf || ([0, 1].indexOf(1, 2) != -1)) {
    Array.prototype.indexOf = function indexOf(sought /*, fromIndex */ ) {
        var self = splitString && _toString(this) == "[object String]" ?
                this.split("") :
                toObject(this),
            length = self.length >>> 0;

        if (!length) {
            return -1;
        }

        var i = 0;
        if (arguments.length > 1) {
            i = toInteger(arguments[1]);
        }
        i = i >= 0 ? i : Math.max(0, length + i);
        for (; i < length; i++) {
            if (i in self && self[i] === sought) {
                return i;
            }
        }
        return -1;
    };
}
if (!Array.prototype.lastIndexOf || ([0, 1].lastIndexOf(0, -3) != -1)) {
    Array.prototype.lastIndexOf = function lastIndexOf(sought /*, fromIndex */) {
        var self = splitString && _toString(this) == "[object String]" ?
                this.split("") :
                toObject(this),
            length = self.length >>> 0;

        if (!length) {
            return -1;
        }
        var i = length - 1;
        if (arguments.length > 1) {
            i = Math.min(i, toInteger(arguments[1]));
        }
        i = i >= 0 ? i : length - Math.abs(i);
        for (; i >= 0; i--) {
            if (i in self && sought === self[i]) {
                return i;
            }
        }
        return -1;
    };
}
if (!Object.getPrototypeOf) {
    Object.getPrototypeOf = function getPrototypeOf(object) {
        return object.__proto__ || (
            object.constructor ?
            object.constructor.prototype :
            prototypeOfObject
        );
    };
}
if (!Object.getOwnPropertyDescriptor) {
    var ERR_NON_OBJECT = "Object.getOwnPropertyDescriptor called on a " +
                         "non-object: ";
    Object.getOwnPropertyDescriptor = function getOwnPropertyDescriptor(object, property) {
        if ((typeof object != "object" && typeof object != "function") || object === null)
            throw new TypeError(ERR_NON_OBJECT + object);
        if (!owns(object, property))
            return;

        var descriptor, getter, setter;
        descriptor =  { enumerable: true, configurable: true };
        if (supportsAccessors) {
            var prototype = object.__proto__;
            object.__proto__ = prototypeOfObject;

            var getter = lookupGetter(object, property);
            var setter = lookupSetter(object, property);
            object.__proto__ = prototype;

            if (getter || setter) {
                if (getter) descriptor.get = getter;
                if (setter) descriptor.set = setter;
                return descriptor;
            }
        }
        descriptor.value = object[property];
        return descriptor;
    };
}
if (!Object.getOwnPropertyNames) {
    Object.getOwnPropertyNames = function getOwnPropertyNames(object) {
        return Object.keys(object);
    };
}
if (!Object.create) {
    var createEmpty;
    if (Object.prototype.__proto__ === null) {
        createEmpty = function () {
            return { "__proto__": null };
        };
    } else {
        createEmpty = function () {
            var empty = {};
            for (var i in empty)
                empty[i] = null;
            empty.constructor =
            empty.hasOwnProperty =
            empty.propertyIsEnumerable =
            empty.isPrototypeOf =
            empty.toLocaleString =
            empty.toString =
            empty.valueOf =
            empty.__proto__ = null;
            return empty;
        }
    }

    Object.create = function create(prototype, properties) {
        var object;
        if (prototype === null) {
            object = createEmpty();
        } else {
            if (typeof prototype != "object")
                throw new TypeError("typeof prototype["+(typeof prototype)+"] != 'object'");
            var Type = function () {};
            Type.prototype = prototype;
            object = new Type();
            object.__proto__ = prototype;
        }
        if (properties !== void 0)
            Object.defineProperties(object, properties);
        return object;
    };
}

function doesDefinePropertyWork(object) {
    try {
        Object.defineProperty(object, "sentinel", {});
        return "sentinel" in object;
    } catch (exception) {
    }
}
if (Object.defineProperty) {
    var definePropertyWorksOnObject = doesDefinePropertyWork({});
    var definePropertyWorksOnDom = typeof document == "undefined" ||
        doesDefinePropertyWork(document.createElement("div"));
    if (!definePropertyWorksOnObject || !definePropertyWorksOnDom) {
        var definePropertyFallback = Object.defineProperty;
    }
}

if (!Object.defineProperty || definePropertyFallback) {
    var ERR_NON_OBJECT_DESCRIPTOR = "Property description must be an object: ";
    var ERR_NON_OBJECT_TARGET = "Object.defineProperty called on non-object: "
    var ERR_ACCESSORS_NOT_SUPPORTED = "getters & setters can not be defined " +
                                      "on this javascript engine";

    Object.defineProperty = function defineProperty(object, property, descriptor) {
        if ((typeof object != "object" && typeof object != "function") || object === null)
            throw new TypeError(ERR_NON_OBJECT_TARGET + object);
        if ((typeof descriptor != "object" && typeof descriptor != "function") || descriptor === null)
            throw new TypeError(ERR_NON_OBJECT_DESCRIPTOR + descriptor);
        if (definePropertyFallback) {
            try {
                return definePropertyFallback.call(Object, object, property, descriptor);
            } catch (exception) {
            }
        }
        if (owns(descriptor, "value")) {

            if (supportsAccessors && (lookupGetter(object, property) ||
                                      lookupSetter(object, property)))
            {
                var prototype = object.__proto__;
                object.__proto__ = prototypeOfObject;
                delete object[property];
                object[property] = descriptor.value;
                object.__proto__ = prototype;
            } else {
                object[property] = descriptor.value;
            }
        } else {
            if (!supportsAccessors)
                throw new TypeError(ERR_ACCESSORS_NOT_SUPPORTED);
            if (owns(descriptor, "get"))
                defineGetter(object, property, descriptor.get);
            if (owns(descriptor, "set"))
                defineSetter(object, property, descriptor.set);
        }

        return object;
    };
}
if (!Object.defineProperties) {
    Object.defineProperties = function defineProperties(object, properties) {
        for (var property in properties) {
            if (owns(properties, property))
                Object.defineProperty(object, property, properties[property]);
        }
        return object;
    };
}
if (!Object.seal) {
    Object.seal = function seal(object) {
        return object;
    };
}
if (!Object.freeze) {
    Object.freeze = function freeze(object) {
        return object;
    };
}
try {
    Object.freeze(function () {});
} catch (exception) {
    Object.freeze = (function freeze(freezeObject) {
        return function freeze(object) {
            if (typeof object == "function") {
                return object;
            } else {
                return freezeObject(object);
            }
        };
    })(Object.freeze);
}
if (!Object.preventExtensions) {
    Object.preventExtensions = function preventExtensions(object) {
        return object;
    };
}
if (!Object.isSealed) {
    Object.isSealed = function isSealed(object) {
        return false;
    };
}
if (!Object.isFrozen) {
    Object.isFrozen = function isFrozen(object) {
        return false;
    };
}
if (!Object.isExtensible) {
    Object.isExtensible = function isExtensible(object) {
        if (Object(object) === object) {
            throw new TypeError(); // TODO message
        }
        var name = '';
        while (owns(object, name)) {
            name += '?';
        }
        object[name] = true;
        var returnValue = owns(object, name);
        delete object[name];
        return returnValue;
    };
}
if (!Object.keys) {
    var hasDontEnumBug = true,
        dontEnums = [
            "toString",
            "toLocaleString",
            "valueOf",
            "hasOwnProperty",
            "isPrototypeOf",
            "propertyIsEnumerable",
            "constructor"
        ],
        dontEnumsLength = dontEnums.length;

    for (var key in {"toString": null}) {
        hasDontEnumBug = false;
    }

    Object.keys = function keys(object) {

        if (
            (typeof object != "object" && typeof object != "function") ||
            object === null
        ) {
            throw new TypeError("Object.keys called on a non-object");
        }

        var keys = [];
        for (var name in object) {
            if (owns(object, name)) {
                keys.push(name);
            }
        }

        if (hasDontEnumBug) {
            for (var i = 0, ii = dontEnumsLength; i < ii; i++) {
                var dontEnum = dontEnums[i];
                if (owns(object, dontEnum)) {
                    keys.push(dontEnum);
                }
            }
        }
        return keys;
    };

}
if (!Date.now) {
    Date.now = function now() {
        return new Date().getTime();
    };
}
var ws = "\x09\x0A\x0B\x0C\x0D\x20\xA0\u1680\u180E\u2000\u2001\u2002\u2003" +
    "\u2004\u2005\u2006\u2007\u2008\u2009\u200A\u202F\u205F\u3000\u2028" +
    "\u2029\uFEFF";
if (!String.prototype.trim || ws.trim()) {
    ws = "[" + ws + "]";
    var trimBeginRegexp = new RegExp("^" + ws + ws + "*"),
        trimEndRegexp = new RegExp(ws + ws + "*$");
    String.prototype.trim = function trim() {
        return String(this).replace(trimBeginRegexp, "").replace(trimEndRegexp, "");
    };
}

function toInteger(n) {
    n = +n;
    if (n !== n) { // isNaN
        n = 0;
    } else if (n !== 0 && n !== (1/0) && n !== -(1/0)) {
        n = (n > 0 || -1) * Math.floor(Math.abs(n));
    }
    return n;
}

function isPrimitive(input) {
    var type = typeof input;
    return (
        input === null ||
        type === "undefined" ||
        type === "boolean" ||
        type === "number" ||
        type === "string"
    );
}

function toPrimitive(input) {
    var val, valueOf, toString;
    if (isPrimitive(input)) {
        return input;
    }
    valueOf = input.valueOf;
    if (typeof valueOf === "function") {
        val = valueOf.call(input);
        if (isPrimitive(val)) {
            return val;
        }
    }
    toString = input.toString;
    if (typeof toString === "function") {
        val = toString.call(input);
        if (isPrimitive(val)) {
            return val;
        }
    }
    throw new TypeError();
}
var toObject = function (o) {
    if (o == null) { // this matches both null and undefined
        throw new TypeError("can't convert "+o+" to object");
    }
    return Object(o);
};

});
